#ifndef CONSTANTS_H
#define CONSTANTS_H


/* Astronomical constants
------------------------------------------------------------------------------*/
#define AMU       3.985893280e14 /* Earth mass times gravitational constant   */
#define HERG      8.068135900e2  /* Seconds per herg                          */
#define LITESPEED 2.997930000e8  /* Speed of light                            */
#define REM       6.378165000e6  /* Mean radius of the earth at equator       */
#define WE        7.292115642e-5 /* Solar inertial rotation rate of earth     */


/* Trigonometry constants.
------------------------------------------------------------------------------*/
#define PIE            3.141592653589793238 /* PI                             */
#define PIE2           (PIE * 2.0)          /* Two PI                         */
#define HALFPIE        (PIE / 2.0)          /* One half of PI                 */
#define RAD_PER_DEG    (PIE / 180.0)        /* Radians per degree             */
#define DEG_PER_RAD    (180.0 / PIE)        /* Degrees per radian             */


/* Atmospheric correction constants
------------------------------------------------------------------------------*/
#define ATMO_ALINES       720   /* Lines per A coefficient Look Up Table      */
#define ATMO_BLINES     12960   /* Lines per B coefficient Look Up Table      */
#define ATMO_NSATK          7   /* # of NOAA sats (6-12) with known K values  */
#define ATMO_NSATS          6   /* # of NOAA sats (6-11) with known S values  */
#define ATMO_OZCNT         40   /* # of ozone increments in LUT               */
#define ATMO_PRCNT         18   /* # of relative azimuth increments in LUT    */
#define ATMO_SECNT         12   /* # of satellite zenith increments in LUT    */
#define ATMO_SUCNT         12   /* # of solar zenith increments in LUT        */
#define ATMO_TGCNT         16   /* # of Tg increments in LUT                  */
#define ATMO_TLINES        16   /* Lines per Tg Look Up Table                 */
#define ATMO_ZICNT          5   /* # of elevation increments in LUT           */
#define STD_OZONE       0.319   /* Average standard ozone over the earth      */


/* Scaling constants
------------------------------------------------------------------------------*/
#define NO_SCALING	0
#define US_BYTE		1
#define US_I2		2
#define US_I4		3
#define US_REAL		4
#define AVHRR_BYTE	5
#define AVHRR_10BIT	6
#define AVHRR_I2	7
#define AVHRR_I4	8
#define AVHRR_REAL	9


/* Mask value constants
------------------------------------------------------------------------------*/
#define MASK_NO_DATA	0
#define MASK_WATER	1
#define MASK_INTERRUPT	2
#define MASK_SOLZEN	3


/* Values to be used with the AVHRR Quality Control (QC) band
------------------------------------------------------------------------------*/
#define SZANGLE   70		      /* Solar zenith angle at which to start */
				      /* flagging as extreme in qc band	      */
#define SZSCALE  100		      /* Scale factor to apply to qc band for */
				      /* the extreme solar zenith angles      */


/* Macros for scaling and unscaling a pixel value.  Note that UNSCALE uses
   multiplication instead of division.  For speed considerations, the
   application should use the inverse of the scaling factor and send that to
   this macro as the "unscaling factor".  Multiplications are faster than
   divisions.
------------------------------------------------------------------------------*/
#define SCALE(value,min,max,scale,offset) \
    (((value) < (min)) ? ((min) * (scale) + (offset)) : \
    (((value) > (max)) ? ((max) * (scale) + (offset)) : \
    ((value) * (scale) + (offset))))

#define UNSCALE(value,unscale,offset) (((value) - (offset)) * (unscale))

#endif
