#ifndef PRODUCTS_H
#define PRODUCTS_H

#include "model.h"


/* Defines used by the product generation routines
==============================================================================*/
#define DAY_NIGHT   0     /* Order is for both day and night data             */

#define LOCK        1     /* Define for c_lockfil                             */
#define LOCKED      1     /* Define for c_lockfil                             */
#define UNLOCK      0     /* Define for c_lockfil                             */

#define NOPROCESS   0     /* Scene does not meet the order criteria           */
#define PROCESS     1     /* Scene matches the order criteria, produce product*/

#define BOTHPOLES   0     /* Standing request crosses both poles              */
#define NOPOLE      1     /* Standing request crosses neither pole            */
#define NORTHPOLE   2     /* Standing request crosses just the north pole     */
#define SOUTHPOLE   3     /* Standing request crosses just the south pole     */

#define MXPRODS    50     /* Max number of product orders to return to TAE    */
#define NPIXELS    25     /* # of pixels to skip around the edge of an image  */

#define PROTECTION "(system:rwd,owner:rwd,group:rwd,world:rwd)"
#define TABLE "prodgen.table" /* Standing order table in ADAPSTABLES          */


/* Structure for containing all of the description lines from the prodgen table
==============================================================================*/
struct DESCRIPT
   {
   long nlines;           /* Number of lines in the description of the table  */
   char *buffer;          /* Pointer to all of the description information    */
   };


/* Structure to contain one standing order
==============================================================================*/
struct ORDER
   {
   long   account;        /* Customer's account number                        */
   long   ordernum;       /* N'th order for the specified account number      */
   long   start_date;     /* Starting date of order.  Integer in YYYYMMDD     */
   long   end_date;       /* Ending date of order.  Integer in YYYYMMDD       */
   long   age;            /* Number of days past acquisition to produce prods */
   char   station[4];     /* 3 character ID of station that acquired the data */
   short  satnum;         /* Number of satellite the order is for (10, 11,..) */
   short  day_night;      /* DAY, NIGHT, or DAY_NIGHT                         */
   short  dtype;          /* Flag indicating if this is GAC or HRPT/LAC data  */
   double northlat;       /* Northern latitude of the area of interest        */
   double southlat;       /* Southern latitude of the area of interest        */
   double westlong;       /* Westernmost longitude of the area of interest    */
   double eastlong;       /* Easternmost longitude of the area of interest    */
   short  polecross;      /* Flag indicating a pole crossing                  */
   long   minlines;       /* Minimum number of lines in the output product    */
   char   window;	  /* Flag for windowing.  If set only lines in area
				of interest are used			      */
   long   product_code;   /* Code for the product that we should generate     */
   short  bands[5];       /* Flag (true or false) to include each band        */
   char   description[18];/* Short 17 character description of the order      */
   char   output[CMLEN];  /* directory/media to put output for products       */
   };


/* Structure to contain one pixels latitude and longitude
==============================================================================*/
struct POINT
   {
   long   line;           /* Line number of the pixel in the ADAPS image      */
   long   sample;         /* Sample number of the pixel in the ADAPS image    */
   double lat;            /* Latitude in degrees for this pixel               */
   double lon;            /* Longitude in degrees for this pixel              */
   };


/* Prototype functions to be called by any ADAPS support routine or module
==============================================================================*/
int c_pg2struct
(
    char *str,             /* IN  - String read from the prodgen table        */
    struct ORDER *order   /* OUT - Ptr to a standing order structure          */
);

int c_pgpoints
(
    struct MODEL *model,  /* IN  - Ptr to the model and orbital info structure*/
    long npixels,	  /* IN  - Number of pixels to skip between points    */
    long *q               /* OUT - Ptr to the queue that is created           */
);

int c_pgread
(
    struct DESCRIPT *descript, /* OUT - Ptr to struct containing description  */
    long *orders_q             /* OUT - Q holding all standing order strings  */
);

int c_pgtest
(
   struct MODEL  *model,  /* IN - Ptr to a archive model structure            */
   struct ORDER  *order,  /* IN - Ptr to a standing order structure           */
   long points_q,         /* IN - Queue of points along the edge of the image */
   long *start,           /* OUT - First line of the image in the product     */
   long *nlines           /* OUT - Number of lines in the product             */
);

int c_pgwrite
(
    struct DESCRIPT *descript,/* IN - Ptr to struct containing the description*/
    long orders_q             /* IN - Q holding all standing order strings    */
);

#endif
