#ifndef _RADPAK_H_
#define _RADPAK_H_

/* The radiometric mapping grid structure contains a radiometric mapping grid
   mapping input pixel values (line,sample) to output pixel values,
   information needed to fill the output space image's DDR, 
   and miscellaneous mapping grid parameters and statistics.
 ------------------------------------------------------------------------*/
struct RADGRID
  {
  long	fiterr_valid;	  /* Valid field for fit error fields */
  long	griderr_valid;	  /* Valid field for griding error fields */
  long	 ncoeff;	  /* Number of polynomial coefficients in X & Y.
			     This field is zero if other than a polynomial
			     model was used to generate the mapping grid. */
  long	 nrows;	    	  /* Number of geometric mapping grid rows */
  long	 ncols;	    	  /* Number of geometric mapping grid columns */
  long	 lines;	    	  /* Number of lines in the output image */
  long	 samples;    	  /* Number of samples in the output image */
  long   *out_lines;      /* Pointer to the locations of the horizontal 
			     coordinate (line number) on each grid row, going
			     from top to bottom.  The output grid cells are of
			     consistent size and are rectangular, perpendicular
			     to the line, sample axis of the output image. 
			     Therefore, only intersections need be given.  */
  long   *out_samps; 	  /* Pointer to the locations of the vertical 
			     coordinate (pixel number) on each grid column,
			     going from left to right.  Again, only inter-
			     sections need be given.  */
  double *coeffs;	  /* Pointer to transformation coefficients.  The
			     first ncoeff values are the Y transformation
			     coefficients, followed by ncoeff X tranformation
			     coefficients.  */
  double max_grid_err[2]; /* Maximum horizontal and vertical grid interpolation
			     errors at an of 16 reference points.  Gridding 
			     routines give an estimate of the overall quality 
			     of the gridding process using a 4 x 4 matrix of 
			     reference points located at the intersections of
			     four vertical and four horizontal lines in the 
			     image region covered by the mapping grid.  The 
			     lines are located 3/126, 43/126, 83/126, and 
			     123/126 of the distance from one edge of the area 
			     to the other. */
  double ave_grid_err[2]; /* Average of the absolute values of the horizontal
			     and vertical grid interpolation errors of the
		    	     sixteen reference points. */
  double rms_grid_err;    /* RMS of the residual errors of the sixteen 
			     reference points.  */
  double act_grid_err[32];/* Actual fitting errors for each of the 16
			     reference points; each pair of values is given
			     in (line, sample) order, going left to right
			     in each row starting with the top row */
  double grid_tol;	  /* Tolerance used in reducing grid */
  double max_fit_err[2];  /* Maximum horizontal and vertical residual errors
			     of the tie points from the modeling process, if
			     tie points were used and model = "POLYNOMIAL". */
  double ave_fit_err[2];  /* Average of the absolute values of the horizontal
			     and vertical residual errors of the tie points
			     from the modeling process, if tie points were used
			     and model = "POLYNOMIAL."  */
  double rms_fit_err;     /* RMS of the residual errors from the modeling 
			     process, if tie points were used and model =
   			     "POLYNIOMIAL."  */
  double *z_values;       /* Pointer to a buffer of delta pixel values 
			     one for each grid intersection.  
			     This buffer has as size of nrows * ncols * 
			     sizeof(double) bytes. */
  char	 model[15];       /* Type of model used to generate the geometric
			     mapping grid.  Standard values are "POLYNOMIAL",
			     "SPACECRAFT", and "FINITE ELEMENT".  Other
			     values are allowed, if need be. */
  };

#endif /* _RADPAK_H_ */
