#!/bin/csh -f
#
# FUNCTION: aobrw_tran.csh
#
# PURPOSE:  Perform transfer of stitched AVHRR browse images
#	    Assume we are already in the directory where the browse
#	    images reside.
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# T.Baltzer	   11/96  Development, stole from BWTRANS
#


#------------------------
# Set up global variables
#------------------------
set noglob
set brwsys  = edcsgs16
set brwdir  = /sgs16/browse/graphics/orb_seg/tmp
set logfile = `echo "[$ADAPSLOG]archbrw" | tr / . | sed  's/\[\./\[/'`
set ntries  = 3


#-----------------------------------------------------------------
# Set up csh.  If there is a Berkley version of rcp use it instead
#-----------------------------------------------------------------
unalias cd
unalias rcp
if (-e /usr/bsd/rcp) alias rcp /usr/bsd/rcp


#---------------------------------------------------------
# Ensure the correct number of arguments were supplied.
#---------------------------------------------------------
if ($#argv != 2) then
   echo ""
   echo "Usage: aobrw_tran.csh <input_file> <wrtlog_path>"
   echo ""
   echo "       <input_file> is the list of files to be transferred."
   echo "       <wrtlog_path> is the path of the write_log executable."
   echo "       located."
   echo ""
   echo "[aobrw_tran-fatal] Fatal error encountered"
   exit 1
endif


#------------------------------------------------------------------------------
# Set up an alias for the write_log command.  The path could be
# the current directory in which case we must set up an alias before we change
# directories.
#------------------------------------------------------------------------------
if ("$2" == ".") then
   alias wrtlog `pwd`/write_log
else
   alias wrtlog $2/write_log
   endif


#---------------------------------------------------
# Make sure the list (file) already exists
#---------------------------------------------------
set input_file = $1
if (! -e $input_file) then
    set text = "Input list ${input_file} does not exist"
    wrtlog $logfile archbrw aobrw_tran $text
    echo "[informational]" $text
    echo "[archbrw-fatal] Fatal error encountered"
    exit 1
endif


#--------------------------------------------
# Transfer all files in the current directory
# that are listed in the input listing.
#--------------------------------------------
unset noglob
set files = `cat $input_file`
set noglob
@ nfiles = 0
foreach file ($files)
   if (-z $file) continue

   #----------------------------------------------
   # Retrieve just the scene ID from the file name
   #----------------------------------------------
   set sceneid = `echo $file | awk -F\_ '{print $1}'`


   #--------------------------------
   # Issue a message to the log file
   #--------------------------------
   set text = "Starting the browse transfer"
   wrtlog $logfile $sceneid archbrw-start $text

   #-------------------
   # Transfer the image
   #-------------------
   @ i = 0
   while ($i < $ntries)
      @ i += 1
      rcp $file $user"@"$brwsys":"$brwdir/$file
      if (! $status) then
	 set text = "Completed the browse transfer successfully"
	 wrtlog $logfile $sceneid archbrw-success $text
	 break
      else if ($i < $ntries) then
         sleep 60
      else
	 set text = "ERROR encountered in the browse transfer"
	 wrtlog $logfile $sceneid archbrw-error $text
	 echo "[archbrw-error]" $text
	 exit 1
      endif
   end
   @ nfiles += 1

end

set text = "Successfully transferred ${nfiles} browse images"
wrtlog $logfile "" archbrw-tran $text
echo "[archbrw-tran]" $text
exit 0
