#!/bin/csh
#
# FUNCTION: Uptmcor
#
# PURPOSE: Disseminate the NBS time correction data files to all
#          systems.
#          This function is performed in a shell script because
#          in a pdf the ADAPSTABLES variable gets expanded on
#          the host system instead of the target system (the
#          rightmost occurrence of ADAPSTABLES in the remote
#          copy commands below must get expanded on the target
#          system).
#
# CALLING SYNTAX:
#
#          uptmcor.csh "host name"
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# T. Baltzer       01/98  Development (copied from upnav.csh)
# T. Baltzer	   07/98  Add edcsgs18, put into a foreach loop
# D.Lloyd          11/98  Added edcsgs20 to the ALLSYSTEMS variable
# K.Vanumamalai    04/09  Removed edcsgs20 from the ALLSYSTEMS since it has
# gone away and added edclxs100.


# We currently have LAS/ADAPS installed on the systems defined by ALLSYSTEMS:
# -------------------------------------------------------------
set ALLSYSTEMS = "edcsns1 edclxs100 edcsgs20"


# Initialize the return status to "success".  If an error does occur during a
# remote copy, this return status will be changed to a "failure" status.
# -------------------------------------------------------------
set returnstat = 0


# Propagate the TIME CORRECTION files to each of the systems
# (except the host system).
# -------------------------------------------------------------
foreach onesystem ($ALLSYSTEMS)
   if ($1 != $onesystem) then
      scp $ADAPSTABLES/*.timcor $onesystem\:'$ADAPSTABLES' || set returnstat = 1
   endif
end

# New systems that do not allow rcp
#----------------------------------
set NEWSYSTEMS = "edclxs64 edclxw100"
foreach onesystem ( $NEWSYSTEMS )
    scp $ADAPSTABLES/*.timcor adaps@$onesystem\:'$ADAPSTABLES'
    if ( $status ) then
        set returnstat = 1
    endif
end

echo "End of UPTMCOR shell script"

exit $returnstat

