#!/usr/local/bin/perl
#
#  ####                                                 #####
# #    #  #    #    ##    #    #   ####   ######        #    #   ####   #    #
# #       #    #   #  #   ##   #  #    #  #             #    #  #    #  #   #
# #       ######  #    #  # #  #  #       #####         #####   #       ####
# #       #    #  ######  #  # #  #  ###  #             #    #  #       #  #
# #    #  #    #  #    #  #   ##  #    #  #             #    #  #    #  #   #
#  ####   #    #  #    #  #    #   ####   ###### ###### #####    ####   #    #
#
#
# USAGE:        change_bck <filename> $cwd $home
#
# DESCRIPTION:  This program is used to convert absolute path names in the
#               backup files that are 'diff'ed.  If the path names are all
#               different, the user will have to wade through thousands of
#               lines to look for actual problems.  Its scheme is as follows:
#
#    bbbbb:     Any occurance of bbbbb in <filename> will be replaced with
#               $cwd.
#
#    aaaaa:     Any occurance of aaaaa in <filename> will be replaced with 
#               $cwd with $cwd's first '/' removed and subsequent '/'s
#               replaced with '.'s.  (This is due to the VAXish filenaming
#               conventions)
#
#*****************************************************************************

# Check if improper usage of program.
if ($#ARGV != 2)
{
    print "Usage: change_bck filename $cwd $home\n";
    exit 1;
}

my $filename = $ARGV[0];
my $cwd = $ARGV[1];
my $home = $ARGV[2];

# Building unix_cwd.  Must add backslashes before slashes.
my @dir_array = split m%/%, $cwd;
my $unix_cwd = join "\\\\/", @dir_array;

# Changing all bbbbb to unix_cwd.
system "sed s/bbbbb/$unix_cwd/g $filename > temp; mv temp $filename";

# Building vax_cwd.  First checking if below home directory.
my $INHOME = 0;
if ($cwd =~ /^$home/)
{
    $INHOME = 1;
}

my $NUM_SLASHES = 1;
if ($INHOME)
{
    $NUM_SLASHES = split(m%/%, $home) - 1;
}

my $vax_cwd = join ".", @dir_array[$NUM_SLASHES..$#dir_array];

# Changing all aaaaa to vax_cwd.
system "sed s/aaaaa/$vax_cwd/g $filename > temp; mv temp $filename";
