#!/bin/csh -f
#
# FUNCTION:  chowner.csh
#
# PURPOSE:   Change the owner for the specified file(s) to release or to ddps.
#            If a directory is specified, change the ownership of the directory
#	     and recursively change the ownership of all of the contents of
#	     that directory.
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# J.Fenno          05/94  Development
# D.Etrheim,       11/96  Selected owner/group based on location in repository
# D.Zokaites  


#-----------------
# Set up the shell
#-----------------
onintr ABORT


#-------------------------------
# Remember the current directory
#-------------------------------
set origdir = `pwd`


# -------------
# Set variables
# -------------
set comfile=$0   #command file being executed
set owner=$1     #owner to assign files to 
set group=$2     #group to assign files to 


#-------------------------------------------
# Change the ownership of all file arguments
#-------------------------------------------
shift ; shift 
foreach arg ($*)
    #------------------------------------------
    # Ignore the CVS lock files and directories
    #------------------------------------------
    if ("$arg" =~ \#*) continue


    #-----------------------------------------------------------------
    # If the specified argument does not exist, issue an error message
    #-----------------------------------------------------------------
    if (! -e $arg) then
	echo "chowner.csh: $arg does not exist"
	continue
    endif


    #-------------------------------
    # If the argument is a directory
    #-------------------------------
    if ((-d $arg) && (-r $arg) && (-x $arg)) then
	echo "Changing ownership for all files in $arg"
	cd $arg
	$comfile $owner $group `/bin/ls`
	cd $origdir
    else if (-d $arg) then
	echo "chowner.csh: access denied to $arg"
    endif


    #-------------------------------------
    # Change the ownership of the argument
    #-------------------------------------
    if (-o $arg) then
       chgrp $group $arg
       chown $owner $arg
    endif
end


#------------------
# Exit successfully
#------------------
exit 0






ABORT:
    echo "chowner.csh: user aborted"
    exit 1
