#!/bin/csh
#
# FUNCTION:     clean_install.csh
#
# PURPOSE:      Clean up the installation directory and make sure the las/adaps
#               directory structures are setup.
#
# Usage:
#       -  Option to only checkout -tae or -las or -all
#   $1  -  Directory to checkout the software to
#   $2  -  Tag to be used to check out the source from the CVS repository
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

#------------------------------------------------------
# Verify the correct number of arguments were specified
#------------------------------------------------------
if ( $#argv < 1 ) then
    goto USAGE
endif

#------------------------------------------------------
# Save the input arguments to meaningful variable names
#------------------------------------------------------
set install_dir = $1

#------------------------------------------
# Make sure the checkout directories exists
#------------------------------------------
if (! (-d $install_dir)) then
   echo "Installation directory $install_dir does not exist"
   echo ""
   goto USAGE
endif

echo "Cleaning the installation directory"
#-------------------------------------------------------------------------
# Clean up the easy way ... Remove everything and re-create the directory
# structure required for the install.
#-------------------------------------------------------------------------
cd $install_dir
set nfiles = `/bin/ls | /bin/wc -l`
if ($nfiles != 0) then
    /bin/rm -rf *
    if ($status) then
        echo "ERROR: Cleaning the installation directory $install_dir"
        exit 1
    endif
endif

echo "Re-create the installation directory structure"
lasdir_setup.csh

echo "Installation directory clean up completed successfully"
exit 0

USAGE:
cat << EOM
usage: clean_install.csh <install_dir>

       Clean up the <install_dir> directory by removing the contents of the
       directory and then re-creating the necessary directory structure for
       the TAE, LAS, and ADAPS installation.

       <install_dir> - Directory to clean up.  The contents of the directory
                       will be removed and the "lasdir_setup.csh" script will
                       be run to recreate the proper directories for checkout
                       and installation.
EOM
exit 1
