#!/bin/csh
#
# FUNCTION:     dist_software.csh
#
# PURPOSE:      Distribute the release software to the associated systems.
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

set sys_err    = 0
set usr        = release
set assoc_sun  = ()
set assoc_sgs20  = ()

#--------------------------------------------------------
# Check for associated systems and validity of the system
#--------------------------------------------------------
set system = `uname -n`
if ($system == "edcsgs20") then
    set assoc_systems = ($assoc_sgs20)
else if ($system == "edcsns1") then
    set assoc_systems = ($assoc_sun)
else
    echo "ERROR: Invalid system ($system) to distribute from"
    exit 1
endif

#-------------------------------------
# Distribute to each associated system
#-------------------------------------
foreach sys ($assoc_systems)
    dist_one_system.csh $sys
    if ($status) then
        echo "ERROR: Distributing to $sys"
        set sys_err = 1
    endif
end

#--------------------------------------
# Log an error or successful completion
#--------------------------------------
if ($sys_err == 1) then
    echo "Distribution to all systems associated with $system did not occur"
    exit 1
else
    echo "Distribution from $system completed successfully"
endif

exit 0
