#!/bin/csh
#
# FUNCTION:     distribute_doc.csh
#
# PURPOSE:      Distribute the ADAPS and LAS documentation on the WWW server.
# 		It is assumed the user running this function has scp keys set up
#		for the specified host and source directory.  All output should be
#		echo'd to standard output so the invoking command can capture the
#		messages.
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# J.Fenno          10/95  Development


#-----------------
# Set up the shell
#-----------------
if ($?noclobber) then
    unset noclobber
endif
unalias cd


#------------------
# Get the arguments
#------------------
if ($#argv != 3) then
    echo 'ERROR: invalid number of arguments'
    echo 'usage: distribute_doc.csh <srcsys> <srcdir> <dstdir>'
    echo ''
    echo '       Distribute documentation on the current system.'
    echo ''
    echo '       <srcsys> is the name of the computer where the documentation'
    echo '       resides.'
    echo ''
    echo '       <srcdir> is the directory that contains the documentation.'
    echo '       The directory may be an absolute or relative path.'
    echo ''
    echo '       <dstdir> is a directory, relative to $EDCSOFT, where the'
    echo '       make las command should place the appropriate files.'
    exit 1
endif
set srcsys = $1
set srcdir = $2
set dstdir = $3
shift
shift
shift


#------------------------------------------
# Ensure the destination directory is valid
#------------------------------------------
if (("$dstdir" != "run/adaps/doc") && ("$dstdir" != "run/las/doc") && \
    ("$dstdir" != "../release/doc")) then
    echo "ERROR: invalid destination directory"
    exit 1
endif


#-----------------------------
# Create a temporary directory
#-----------------------------
cd
set stamp = `date '+%d_%m_%y.%H%M%S' | tr "[A-Z]" "[a-z]"`
set tmpdir = "docdir.$stamp"
onintr -
mkdir $tmpdir
if ($status) then
    echo "ERROR: creating temporary directory"
    exit 1
endif


#-----------------------------
# Ensure the group is software
#-----------------------------
chgrp software $tmpdir
chmod 775 $tmpdir
chmod g+s $tmpdir
cd $tmpdir
if ($status) then
    echo "ERROR: accessing temporary directory"
    /bin/rm -rf $tmpdir
    exit 1
endif
onintr FATAL


#----------------------------------------------------------------------------
# Copy all of the documentation to the current system.  Be careful here.  You
# shouldn't use $srcdir/* because there may be hidden files in $srcdir.  An
# asterisk will not match these.
#----------------------------------------------------------------------------
scp -qr ${srcsys}:"$srcdir" .


#----------------------------------------------------------------
# Moved into the directory that was copied from the source system
#----------------------------------------------------------------
set tmp = `echo $srcdir | awk -F/ '{print $NF}'`
cd $tmp


#-----------
# Set DSTDIR
#-----------
setenv DSTDIR $EDCSOFT/$dstdir


#-----------------------
# Make the documentation
#-----------------------
make las
if ($status) goto BUILDERR


#----------------------------------------------------------------------------
# Compile was successful.  Remove the temporary directory and all source code
#----------------------------------------------------------------------------
cd
/bin/rm -rf $tmpdir


#--------------
# End of script
#--------------
exit 0


#-----------------
# All build errors
#-----------------
BUILDERR:
    onintr -
    echo "ERROR: documentation installation failed on `uname -n`"
    goto FATAL


#-------------------------------
# All fatal errors or user abort
#-------------------------------
FATAL:
    onintr -
    cd
    /bin/rm -rf $tmpdir
    exit 1
