#!/bin/csh
#
# FUNCTION:     relcron.csh
#
# PURPOSE:      Main function to release and distribute LAS/ADAPS code.
#               This function must be placed in the cron jobs for the release
#               account on the "main source code" system.  It is currently in
#               release@edcsgs20.cr.usgs.gov:cron/cron.jobs.
#               The output from running this script is put in a log file in
#               "$HOME/bldlog/relcron_$stamp.log" and then sent to the LAS
#               administrator.
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

#-----------------------------------------------
# Define the systems this function should run on
#-----------------------------------------------
set admin    = (tbeckman@usgs.gov)
set usr      = release
set bldsys   = (edcsgs20 edcsns1)
set halfline = "====================================="
set stamp    = `date '+%d_%b_%Y.%H%M%S' | tr "[A-Z]" "[a-z]"`
set output   = "$HOME/bldlog/relcron_$stamp.log"

#-----------------------
# Build on each system
#-----------------------
foreach system ($bldsys)
    #------------------------------
    # Ensure the build system is up
    #------------------------------
    ssh -q $system -l $usr -n date > /dev/null
    if ($status) then
	    echo "ERROR: $system is down at `date`"        >>& $output
        continue
    endif

    if ($system == "edcsgs20") then
        set install_dir = "/sgs20/csb/release/las/release_build"
    else if ($system == "edcsns1") then
        set install_dir = "/edcsns1/csb2/release/release_build"
    else
        echo "Invalid system $system to distribute from" >>& $output
        continue
    endif
    
    #------------------------------------------------------
    # Copy new files to the release directory after copying
    # the release files to the old directory
    # * The release directory is relative to $EDCSOFT
    #------------------------------------------------------
    echo "Releasing on $system at `date`"              >>& $output
    echo "$halfline$halfline"                          >>& $output
    ssh -q $system -l $user -n release_software.csh $install_dir >>& $output
    if ($status) then
        echo "ERROR: Releasing software from $system"  >>& $output
        continue
    endif

    #------------------------
    # Spacing in the log file
    #------------------------
    echo ""                                             >>& $output
        
    #-------------------------------------------------------
    # Distribute the software to the systems associated with
    # the build system
    #-------------------------------------------------------
    echo "Distributing from $system at `date`"           >>& $output
    echo "$halfline$halfline"                            >>& $output
    ssh -q $system -l $usr -n dist_software.csh          >>& $output
    if ($status) then
	    echo "ERROR: Distributing software from $system" >>& $output
        continue
    endif

    #------------------------
    # Spacing in the log file
    #------------------------
    echo ""                                              >>& $output
end

#----------------------------------
# Mail the results to $adimistrator
#----------------------------------
elm -s"$system release `date '+%D %T'`" $admin < $output >& /dev/null
/bin/rm -f $output
