#!/bin/csh
#
# FUNCTION:	ckdrive.csh
#
# PURPOSE:	To check a drive, for a BOT(Beginning of Tape) status,
#               some number of times at 10 second intervals.
#
# PARAMETERS:
#		$1 - The device name (example:/dev/rmt/tps7d6nrnsv)
#		$2 - The number of times to check
#
# USAGE:	ckdrive.csh /dev/rmt/tps7d6nrnsv 12
#

#  Set up the timestamp
#----------------------
set timestamp = `date '+%m%d%y%H%M%S'`

#  Initialize the counter to zero and
#  the number of seconds to sleep
#------------------------------------
set counter = 0
set seconds = 10

#  Create the initially empty timestamp file here
#------------------------------------------------
cat << EOF > $timestamp.tmp
EOF

#  Loop X(# of times) until ready or end of count
#------------------------------------------------
while -z $timestamp.tmp
   mt -f $1 status |& grep BOT >& $timestamp.tmp
   @ counter += 1

   #  Check counter and if at the max number of tries
   #  remove the timestamp file and exit bad
   #-------------------------------------------------
   if (($counter == $2) && (-z $timestamp.tmp)) then
      rm $timestamp.tmp
      exit 1
   endif

   #  Check if BOT is found and exit loop before sleeping
   #-----------------------------------------------------
   if (!(-z $timestamp.tmp)) then
      break
   endif

   sleep $seconds
end

#  Remove the timestamp file and exit with success status
#--------------------------------------------------------
rm $timestamp.tmp
exit 0
