#ifndef ADAPSUTIL_H
#define ADAPSUTIL_H

#include "tae.h"
#include "model.h"

void c_cvtime
(
    long *tsec,		/* Total seconds		*/
    long *hour,		/* Hours			*/
    long *min,		/* Minutes			*/
    long *sec		/* Seconds			*/
);

void c_cvtsec
(
    long *tsec,		/* Total seconds		*/
    long *hour,		/* Hours			*/
    long *min,		/* Minutes			*/
    long *sec,		/* Seconds			*/
    long *fsec		/* Fractional seconds		*/
);

void c_cvtmil
(
    long *milsec,	/* Total milliseconds		*/
    long *hour,		/* Hours			*/
    long *min,		/* Minutes			*/
    long *sec,		/* Seconds			*/
    long *fsec		/* Fractional seconds		*/
);

void c_cvtoffset
(
    long inyear,		/* Input year				*/
    long injday,		/* Input julian day			*/
    long intime,		/* Input time (in milliseconds)		*/
    long inoffset,		/* Input time offset (in milliseconds)	*/
    long *outyear,		/* Output year				*/
    long *outjday,		/* Output julian day			*/
    long *outtime		/* Output time (in milliseconds)	*/
);

void c_cvtoffsete
(
    long in_year,		/* Input two digit year		*/
    long in_month,		/* Input month			*/
    long in_day,		/* Input day of month		*/
    long in_hour,		/* Input hours			*/
    long in_min,		/* Input minutes		*/
    long in_sec,		/* Input seconds		*/
    long in_frac,		/* Input milliseconds		*/
    long in_offset,		/* Offset to apply (millisec)	*/
    long *out_year,		/* Output two digit year	*/
    long *out_month,		/* Output month			*/
    long *out_day,		/* Output day of month		*/
    long *out_hour,		/* Output hours			*/
    long *out_min,		/* Output minutes		*/
    long *out_sec,		/* Output seconds		*/
    long *out_frac		/* Output milliseconds		*/
);

int c_daysdiff
(
    long *month_1,		/* Month of first date	(input)		*/
    long *day_1,		/* Day of first date	(input)		*/
    long *year_1,		/* Year of first date	(input)		*/
    long *month_2,		/* Month of second date	(input)		*/
    long *day_2,		/* Day of second date	(input)		*/
    long *year_2,		/* Year of second date	(input)		*/
    long *ndays			/* Number of days between
				   first and second date (output)	*/
);

int c_getsat
(
   char *key,			/* Input key to find the other codes	*/
   long *satnum,		/* Satellite number (11)		*/
   char *satnam,		/* Satellite name   (NOAA-11)		*/
   char *satno,			/* Satellite number (N11)		*/
   char *satid,			/* Satellite id     (19531)		*/
   char *satcode 		/* Satellite code   (I)			*/
);

int c_lockfil
(
   char *hname,		/* Host file name of the file to be locked           */

   long *lockflg 	/* Locking operation (1 - LOCK, 0 - UNLOCK)	     */
);

void c_pck32
(
    short *inbuf,		/* Series of 10-bit words stored in 16 bits */
    unsigned char *outbuf,	/* Pointer to the 32-bit output buffer	    */
    long *nwords		/* Number of input 16-bit shorts to pack    */
);

void c_pixflip
(
    short pixels[],   /* IN/OUT - Array of I*2 pixel values                 */
    long nvalues      /* IN     - Total number of short values in the line  */
);

int c_pixscale
(
    long *idtype,	/* IN  - Data type of the input buffer                */
    long *odtype,	/* IN  - Data type of the output buffer               */
    unsigned char *in,	/* IN  - Input buffer                                 */
    unsigned char *out,	/* OUT - Output buffer                                */
    long *ns,		/* IN  - Number of samples in the buffers             */
    double iscale,	/* IN  - Scaling factor used in the input buffer      */
    double ioffset,	/* IN  - Offset used in the input buffer              */
    double oscale,	/* IN  - Scaling factor to be used in output buffer   */
    double ooffset,	/* IN  - Offset to be used in the output buffer       */
    long *rndopt	/* IN  - Rounding option (TRUE, FALSE)                */
);

void c_unpck32
(
    unsigned char *inbuf,	/* Pointer to the 32-bit input buffer	    */
    short *outbuf,	/* Series of 10 bit words stored in 16 bits         */
    long *nwords,	/* Number of 16-bit words to unpack	            */
    long *swap		/* Byte swap flag				    */
);

int c_windpar
(
    struct PARBLK *vblock,	/* INPUT: Variable block		*/
    struct MODEL *model,	/* INPUT: Model and orbital information	*/
    long window[4]		/* OUTPUT: Window specification		*/
);

int c_wrtlog
(
   char	*logfile,		/* Log file to write to			*/
   char *key,			/* Key in the logfile			*/
   char *code,			/* Operation code			*/
   char *text 			/* Description text			*/
);

void c_makesecs
    (
    long    date[],             /* date (month, day, year)              */
    long    time[],             /* time (hour, min, sec)                */
    long    *totsecs            /* total seconds                        */
    );

#endif
