/****************************************************************************
acu_command

PURPOSE:	This is a simple interface to send commands to the ACU and
		get the ACU response.  Both command and response are printed
		on stdout.

USAGE:		acu_command [port command]

		If no command is issued the following is used:
		port 6100 command 'SYST:TIME,?'


****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>

static int execute_command( int, const char *, char * );

#define BUFLEN 1024
#define TRUE 1
#define FALSE !TRUE


int main( int argc, char *argv[] )
{
    char IPaddress[25];		/* ACU IP address			*/
    char command[BUFLEN];	/* Command string			*/
    char portnum[10];		/* ACU port number			*/
    char acu_msg[BUFLEN];	/* Message returned by ACU		*/
    char stl[20];

    int  sk;			/* Socket descriptor			*/

    struct hostent     *hp;	/* host name of address			*/
    struct sockaddr_in skaddr; 	/* socket information structure		*/
    int i;
    int done, tries;


    /* Set up default command, IP address and port strings
    ------------------------------------------------------*/
    strcpy( IPaddress, "192.168.10.3" );
    strcpy( portnum, "6100" );

    /* Check for command line arguments.  If there are
    the first one is a port number.and the second an
    alternative command.
    --------------------------------------------------*/
    if ( argc == 1 )
    {
        printf("\nUsage: %s [satellite]\n\n", argv[0]);
	return 1;
    }

    /* Create a socket
    ------------------*/
    if (( sk = socket( PF_INET, SOCK_STREAM, 0 )) < 0 )
    {
        printf("Problem creating socket\n");
	return 1;
    }

    /* Set the client name and port
    -------------------------------*/
    skaddr.sin_family = AF_INET;
    if (( hp = gethostbyname( IPaddress )) != 0 )
        memcpy( &skaddr.sin_addr.s_addr, hp->h_addr, hp->h_length );
    else
    {
        if ( inet_aton( IPaddress, &skaddr.sin_addr ) == 0 )
	{
	    printf("Invalid IP address: %s\n",IPaddress);
	    return 1;
	}
    }
    skaddr.sin_port = htons(atoi(portnum));


    /* establish the connection
    ---------------------------*/
    if (( connect( sk, (struct sockaddr *) &skaddr, sizeof( skaddr ))) < 0  )
    {
        printf("Problem connecting socket\n");
	return 1;
    }

    /* Build the command
    --------------------*/
    switch ( atoi( argv[1] ))
    {
        case 17 :
	    strcpy( stl, "NOAA 17" );
	    break;
        case 18 :
	    strcpy( stl, "NOAA 18" );
	    break;
    }
    sprintf( command, "MSSN:POIN,%s,1,@", stl );

printf( "Entering operator level\n");
execute_command( sk, "CCFG:ACCS,1,operator,@", acu_msg );
printf( "ACU said: %s\n", acu_msg );
if ( strcmp( acu_msg, "ccfg:accs,0,0" ) != 0 )
{
    printf( "Couldn't enter observer access\nTrying administrator\n" );
    execute_command( sk, "CCFG:ACCS,0,Admin3880,@", acu_msg );
    printf( "ACU said: %s\n", acu_msg );
    if ( strcmp( acu_msg, "ccfg:accs,0,0" ) != 0 )
    {
        printf( "Unable to enter admin mode either!\n" );
	/* shutdown( sk, SHUT_RDWR ); */
	close( sk );
	return 1;
    }
}


/*
printf( "Trying put in manual on first go: OPNS:AMOD,1,1,0,0,@\n" );
execute_command( sk, "OPNS:AMOD,1,1,0,0,@", acu_msg );
printf( "ACU said: %s\n", acu_msg );
*/

    tries = 1;
    do
    {
	/* Execute the command
	----------------------*/
	printf( "\n   Command: %s\n", command );
	if ( execute_command( sk, command, acu_msg ))
	{
	    printf( "Error executing: %s\n", command );
	    /* shutdown( sk, SHUT_RDWR ); */
	    close( sk );
	    return 1;
	}

	/* Tell us what the acu said
	----------------------------*/
	printf( "  Response: %s\n\n", acu_msg );

	if ( strcmp( acu_msg, "mssn:poin,0,0" ) == 0 )
	{
	    printf( "Command successful on try %d\n", tries );
	    done = 1;
	}
	else
	{
	    printf( "Try %d failed\n", tries );
	    sleep( 2 );
	    done = 0;
	    tries++;
	}

#if 0
	if ( strcmp( acu_msg, "mssn:poin,0,205" ) == 0 && tries == 1 )
	{
	    printf( "Trying put in manual on first go: OPNS:AMOD,1,1,0,0,@\n" );
	    execute_command( sk, "OPNS:AMOD,1,1,0,0,@", acu_msg );
	    printf( "ACU said: %s\n", acu_msg );
	    printf( "Sleeping for 30\n" );
	    sleep( 30 );
	}
#endif

	if ( tries == 10 )
	{
	    printf( "Giving up after 10 tries\n" );
	    done = 1;
	}
    }
    while ( !done );

printf( "Entering observer level\n");
execute_command( sk, "CCFG:ACCS,2,@", acu_msg );
printf( "ACU said: %s\n", acu_msg );
    /* shutdown( sk, SHUT_RDWR ); */
    close( sk );
    return 0;
}  /* END main */



int execute_command( int socket, const char *command_str, char *response )
{
    char message[BUFLEN+2];
    int  mlen;
    int  done;
    int  n, i;

    /* Copy the command into message upper casing it as we go
    for ( n = 0; n < strlen( command_str ); n++ )
        message[n] = command_str[n];
    ---------------------------------------------------------*/
    strcpy( message, command_str );
    n = strlen( message );
    /* Add the CR LF and null terminator */
    message[n++] = 0x0d;
    message[n++] = 0x0a;
    message[n]   = '\0';
    mlen = n;

    /* Write message to the socket
    ------------------------------*/
    n = write( socket, message, mlen );
    if ( n != mlen )
    {
        printf("Error sending message: %s\n", message);
	return 1;
    }

    /* Get the response
    -------------------*/
    done = i = 0;
    do
    {
        n = read( socket, &response[i], 1 );
	if ( n != 1 )
	    done = 1;
	else if ( response[i] == 0x0a && response[i-1] == 0x0d )
	    done = 1;
	i++;
    } while ( !done );

    /* If we got here properly there will be CR LF at the end of response
    ---------------------------------------------------------------------*/
    if ( response[i-2] == 0x0d && response[i-1] == 0x0a )
        response[i-2] = '\0';
    else
        response[i] = '\0';

    return 0;

}  /* END execute_command */
