
/*!
    \file ColoredTableItem.cpp
    \brief Provides the implementation for ColoredTableItem.

    See ColoredTableItem.
*/

/* System Includes */

/* Qt Includes */

/* Local Includes */
#include "ColoredTableItem.h"

/*!
    This constructor allows the user to create a table item directly from the
    table using the default colors.

    \param parent Input: The parent widget from which to create the table item.

    \param editType Input: The edit type for the table item.
*/
ColoredTableItem::ColoredTableItem(QTable *parent, EditType editType)
: QTableItem(parent, editType),
  myForegroundColor(Qt::black),
  myBackgroundColor(Qt::white)
{
    // Relevant initializations are already done.
}

/*!
    This constructor allows the user to create a table item directly from the
    table using the default colors.

    \param parent Input: The parent widget from which to create the table item.

    \param editType Input: The edit type for the table item.

    \param text Input: The text for the table item.
*/
ColoredTableItem::ColoredTableItem(QTable *parent,
                                   EditType editType,
                                   const QString &text)
: QTableItem(parent, editType, text),
  myForegroundColor(Qt::black),
  myBackgroundColor(Qt::white)
{
    // Relevant initializations are already done.
}

/*!
    This constructor allows the user to create a table item directly from the
    table using the default colors.

    \param parent Input: The parent widget from which to create the table item.

    \param editType Input: The edit type for the table item.

    \param text Input: The text for the table item.

    \param pixmap Input: The pixmap for the table item.
*/
ColoredTableItem::ColoredTableItem(QTable *parent,
                                   EditType editType,
                                   const QString &text,
                                   const QPixmap &pixmap)
: QTableItem(parent, editType, text, pixmap),
  myForegroundColor(Qt::black),
  myBackgroundColor(Qt::white)
{
    // Relevant initializations are already done.
}

/*!
    This method sets the foreground color to use.

    \param foregroundColor Input: The foreground color to use.
*/
void ColoredTableItem::setForegroundColor(QColor &foregroundColor)
{
    myForegroundColor = foregroundColor;
}

/*!
    This method sets the background color to use.

    \param backgroundColor Input: The background color to use.
*/
void ColoredTableItem::setBackgroundColor(QColor &backgroundColor)
{
    myBackgroundColor = backgroundColor;
}

/*!
    This method provides the painting for the table item.

    \param painter Input: The QPainter device used to paint this cell.

    \param colorGroup Input: The color group used to color this cell.

    \param rect Input: The rectangular area to paint.

    \param selected Input: TRUE if the cell should be highlighted, FALSE if not.
*/
void ColoredTableItem::paint(QPainter *painter,
                             const QColorGroup &colorGroup,
                             const QRect &rect,
                             bool selected)
{
    QColorGroup cgNewGroup(colorGroup);

    cgNewGroup.setColor(QColorGroup::Base, myBackgroundColor);
    cgNewGroup.setColor(QColorGroup::Text, myForegroundColor);

    QTableItem::paint(painter, cgNewGroup, rect, selected);
}

