
#ifndef COLOREDTABLEITEM_H
#define COLOREDTABLEITEM_H

/*!
    \file ColoredTableItem.h
    \brief Provides an extension of QTableItem.

    See ColoredTableItem.
*/

/* System Includes */

/* Qt Includes */
#include "qtable.h"
#include "qcolor.h"

/* Local Includes */

/*!
    \brief This class provides an extension of QTableItem.

    The sole purpose of the ColoredTableItem class is to provide an
    extension to the QTableItem class that allows for specifying colors
    for the foreground and background of the table item.
*/
class ColoredTableItem : public QTableItem
{
    public:

        ColoredTableItem(QTable *parent, EditType editType);

        ColoredTableItem(QTable *parent, EditType editType,
                         const QString &text);

        ColoredTableItem(QTable *parent, EditType editType,
                         const QString &text, const QPixmap &pixmap);

        void setForegroundColor(QColor &foregroundColor);
        void setBackgroundColor(QColor &backgroundColor);

    protected:

        void paint(QPainter *painter, const QColorGroup &colorGroup,
                   const QRect &rect, bool selected);

    private:


        /*!
            \brief The foreground(text) color.  The default color
            is black.
        */
        QColor myForegroundColor;

        /*!
            \brief The background(cell) color.  The default color
            is white.
        */
        QColor myBackgroundColor;
};

#endif /* COLOREDTABLEITEM_H */

