#ifndef ADAPSMONITOR_H
#define ADAPSMONITOR_H

#include <sys/types.h>
#include <Xm/Xm.h>
#include "widget_name.h"

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE  (!FALSE)
#endif
#ifndef CMLEN
#define CMLEN 256
#endif
#ifndef SUCCESS
#define SUCCESS 1
#endif
#ifndef FAIL
#define FAIL (!SUCCESS)
#endif
#ifndef INVALID
#define INVALID  0
#endif
#ifndef VALID
#define VALID 1
#endif
#ifndef REDISPLAY
#define REDISPLAY 2
#endif
#ifndef YES
#define YES 2
#endif
#ifndef NO
#define NO  1
#endif

#define DEBUG FALSE

/* Any changes to ItemType must be reflected in get_item_type().
The enumerated value NTYPES must be the last one defined.
---------------------------------------------------------------*/
typedef enum ItemType { UNDEFINED = 0, BATCH, PROCESS, VAXPROCESS, SYSTEM,
                        DATAFILE, NTYPES } ItemType;

/* Any Changes to MonState must be reflected in get_monitor_state().
The enumerated value NSTATES must be the last one defined.
--------------------------------------------------------------------*/
typedef enum MonState { UNINITIALIZED = 0, ENABLED, DISABLED, NSTATES }MonState;

typedef enum States { UNCHECKED = 0, RUNNING, UPTODATE, NOTINQUEUE, RWAST,
		      SUSPENDED, HUNG, RESPONDING, NOTRESPONDING, OUTOFDATE,
		      NOTRUNNING, INQUEUE } States;

#define DEFAULTINTERVAL  60000	/* one minutes worth of milliseconds	*/
#define MAXTIMERINTERVAL 1800000/* thirty minutes worth of milliseconds	*/
#define MAXINTERVAL      20160	/* two weeks worth of minutes		*/
#define MAXBATCHJOB      3	/* maximum number of batch jobs		*/

/* These are the lengths of the main display fields
---------------------------------------------------*/
#define MONITORLEN      8
#define NAMELEN         50
#define STATUSLEN       16
#define ASOFLEN         18

/* Define the labels for the modify and alert window toggle buttons
-------------------------------------------------------------------*/
#define ENABLESTRING	"Enabled"
#define DISABLESTRING	"Disabled"

/* Define the colors for the enabled/disabled push buttons
----------------------------------------------------------*/
#define NORMALBGCOLOR		"wheat"
#define CHANGEBGCOLOR		"yellow2"
#define ALERTCHANGEBGCOLOR	"yellow2"
#define ALERTNORMALBGCOLOR	"firebrick2"

/* Define the default initialization and monitor state files
------------------------------------------------------------*/
#define ADAPSMONITORDEFAULT     "adapsmonitor.default"
#define ADAPSMONITORSTATE       "adapsmonitor.state"
#define ADAPSMONITORLOG         "adapsmonitor.log"

/* Item data structure
----------------------*/
struct ITEM {
    struct ITEM *next;		/* pointer to the next data item	*/
    struct ITEM *prev;		/* pointer to the previous data item	*/
    char	itemname[80];	/* Item name				*/
    char	machine[80];	/* machine where item resides or runs	*/
    ItemType	itemtype;	/* type of item				*/
    int		mon_interval;	/* monitor interval in seconds		*/
    int		rwast_interval;	/* monitor interval when in RWAST state	*/
    MonState	initial_state;	/* initial monitoring state		*/
    int		num_batch;	/* number of jobs running in batch	*/
    char	userid[20];	/* user ID of job running in batch	*/
    char	log_name[120];	/* log file name for process		*/
    MonState	monitor_state;	/* current monitoring state		*/
    States	current_state;	/* current item state			*/
    time_t	last_check;	/* time of last check of the item	*/
    short	alarm;		/* flag: T/F alarm set for item		*/
    short	rwast_flg;	/* flag indicating vax process is in	*/
    				/* RWAST state				*/
    time_t	rwast_time;	/* time vax process entered RWAST state	*/
    int		job_number;	/* batch job number			*/
    char	display[CMLEN];	/* display string			*/
    short	fieldflg[5];	/* flags: fields VALID/INVALID		*/
    				/* fieldflg[0] indicates main display	*/
				/* is in need of redisplay, fieldflg[1]	*/
				/* is for monitor_field, fieldflg[2]	*/
				/* for name_field etc			*/
    /* monitor-status-field display string, its flag is fieldflg[1]	*/
    char   monitor_field[MONITORLEN+1];
    /* name-field display string, its flag is fieldflg[2]		*/
    char   name_field[NAMELEN+1];
    /* status-field display string, its flag is fieldflg[3]		*/
    char   status_field[STATUSLEN+1];
    /* os-of-time-field display string, its flag is fieldflg[4]		*/
    char   asoftime_field[ASOFLEN+1];
    int	   item_index;		/* index of the item into the scrolled	*/
    				/* list of the main display (1 based)	*/
    short  mon_state_mod;	/* flag indicating a change in mon state*/
    char   normalbgcolor[CMLEN];/* toggle normal background color	*/
    char   changebgcolor[CMLEN];/* toggle changed background color	*/
    };

/* function prototypes
----------------------*/
struct ITEM *initstruct
    (
    char    *infile,            /* IN - input file name			  */
    short   defaultflg          /* IN - TRUE/FALSE flag: use default file */
				/*    if infile is not present or corrupt */
    );

struct ITEM *allocate_item( void );

void free_itemlist
    (
    struct ITEM **itemlist	/* IN/OUT - list structure to free	*/
    );

void append_item
    (
    struct ITEM **itemlist,	/* IN/OUT - Item list to append to	*/
    struct ITEM *item		/* IN - Item to append to list		*/
    );

int check_line
    (
    char inbuf[]                /* IN - input buffer to check		*/
    );

ItemType get_item_type
    (
    char *str                   /* IN - string representation of type   */
    );

MonState get_monitor_state
    (
    char  *str                  /* IN - string representation of state	*/
    );

int fill_item
    (
    char   *buf,                /* IN - buffer of input values		*/
    struct ITEM *item           /* IN/OUT - pointer to structure to fill*/
    );

void write_log
    (
    int    flag,		/* IN - TRUE/FALSE fatality flag	*/
    				/*      TRUE = issue fatal error message*/
    char   *key,                /* IN - error/message key		*/
    char   *code,               /* IN - error/message code		*/
    char   *msg			/* IN - message string			*/
    );

struct ITEM *initialize
    (
    char *argv[]                /* IN - argument list			*/
    );

Widget create_main_display
    (
    Widget toplevel,            /* IN - top level shell widget		*/
    struct ITEM *list           /* IN - item list structure             */
    );

void exit_callback
    (
    Widget    w,                /* IN - widget associated with the CB   */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

int get_user_response
    (
    Widget   parent,            /* IN - widget of the caller		*/
    char     *question,         /* IN - message to be displayed		*/
    char     *ans1,             /* IN - the no answer button label	*/
    char     *ans2,             /* IN - the yes answer button label	*/
    int      default_answer     /* IN - default answer to use		*/
    );

void response
    (
    Widget    widget,           /* IN - widget associated with callback */
    XtPointer client_data,      /* IN/OUT - pointer to int, the answer  */
    XtPointer call_data         /* IN - call back struct for response   */
    );

void post_dialog
    (
    Widget   parent,            /* IN - widget of the caller            */
    char     *question,         /* IN - message to be displayed         */
    char     *ans               /* IN - the yes button label            */
    );

void create_displist
    (
    struct ITEM *item           /* IN - current item			*/
    );

struct ITEM *get_list_address
    (
    void
    );

void store_list_address
    (
    struct ITEM *list           /* IN - address to store or NULL        */
    );

struct ITEM *get_alarm_address
    (
    void
    );

void store_alarm_address
    (
    struct ITEM *list           /* IN - address to store or NULL        */
    );

void update_displist
    (
    Widget    list_w,           /* IN - the list widget			*/
    struct ITEM *list           /* IN - structure with items to list	*/
    );

Widget create_modify
    (
    Widget parent,		/* IN - the parent screen widget        */
    struct ITEM *item           /* IN - the list of items               */
    );

void modify_callback
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

void cancel_modify_callback
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

void apply_modify_callback
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

void load_newfile_callback
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data: ITEM struct pointer*/
    XtPointer call_data         /* IN - call data                       */
    );

void fileok_callback
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN/OUT - pointer to ITEM structure   */
    XtPointer call_data         /* IN - file selection callback struct  */
    );

void filecancel_callback
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

struct ITEM *copy_itemlist
    (
    struct ITEM *inlist         /* IN - list to be copied               */
    );

void print_list
    (
    struct ITEM *item,          /* IN - item list to print              */
    char *filename              /* IN - file name to print to           */
    );

void cleanup_displist
    (
    Widget w                    /* IN - the scrolled list widget	*/
    );

Widget create_modify_rowcol
    (
    Widget parent,              /* IN - parent widget                   */
    struct ITEM *item           /* IN - list of items                   */
    );

void enable_callback
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

void store_rowcol_widget
    (
    Widget rowcol		/* IN - widget value to store or NULL	*/
    );

Widget get_rowcol_widget
    (
    void
    );

Widget scroll_window_child
    (
    Widget w,			/* IN - calling widget value		*/
    char   widget_name[]	/* IN - name of widget to get		*/
    );


Widget create_toggle_array
    (
    Widget parent,              /* IN - parent widget                   */
    struct ITEM *list,          /* IN - list of items                   */
    char   resource_root[]      /* IN - root widget name                */
    );

Widget get_top_shell
    (
    Widget w                    /* IN - calling widget value            */
    );

void destroy_alert
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

void apply_alert_changes
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

void alert_user
    (
    struct ITEM *alarmlist      /* IN - list of alarms to display       */
    );

void test_something
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

void save_state 
    (
    struct ITEM *item           /* IN - item list to be saved           */
    );

unsigned long get_timer_interval
    (
    char *argv[]                /* IN - command line argument array     */
    );

Widget get_fileselectwindow
    (
    Widget w			/* IN - calling widget value		*/
    );

Widget get_application_shell
    (
    Widget w			/* IN - calling widget value		*/
    );

Widget get_toplevel_shell
    (
    Widget w			/* IN - calling widget value		*/
    );

States check_item
    (
    struct ITEM *item,          /* IN - pointer to item structure       */
    time_t      now		/* IN - the current time                */
    );

void checkstatus_callback
    ( 
    XtPointer    client_data,	/* IN - client data, timer interval	*/
    XtIntervalId *id		/* IN - ID of the timer callback        */
    );

States check_queue
    (
    struct ITEM *item,          /* IN - the item structure to check     */
    int    occurrence           /* IN - which occurrence to check       */
    );

States check_vax
    (
    struct ITEM *item           /* IN - the item structure to check     */
    );

States check_system
    (
    struct ITEM *item           /* IN - the item structure to check     */
    );

States check_file
    (
    struct ITEM *item,          /* IN - the item structure to check     */
    time_t      now,            /* IN - the current time                */
    int    check_name           /* IN - flag T/F, T = check itemname    */
                                /*                F = check log_name    */
    );
#endif
