/************************************************************************
FUNCTION:	destroy_alert
		apply_alert_changes

PURPOSE:	Callbacks for the alert window
************************************************************************/
#include <stdio.h>
#include "adapsmonitor.h"



/**************************************************************************
FUNCTION:	destroy_alert

PURPOSE:	Remove the alarm popup dialog box.

INPUT:          widget information
		client data: pointer to the alarm list

OUTPUT:		none

RETURNS:	none

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development

PROJECT:        ADAPS
************************************************************************/
void destroy_alert
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data, alarm list         */
    XtPointer call_data         /* IN - call data                       */
    )
    {
    struct ITEM *alarmlist = (struct ITEM *)client_data; /* the alarms	*/
    Widget topshell;		/* top shell of calling widget		*/

    /* free the alarmlist
    ---------------------*/
    free_itemlist( &alarmlist );

    /* get the top level alarm shell and destroy it
    -----------------------------------------------*/
    topshell = get_top_shell( w );
    XtDestroyWidget( topshell );

    /* Write a message that the user acknowledged the alert
    -------------------------------------------------------*/
    write_log( FALSE, "destroy_alert", "acknowledge",
        "Alert message acknowledged" );
    }  /* END destroy_alert */



/**************************************************************************
FUNCTION:	apply_alert_changes

PURPOSE:	Make any changes to monitoring status that the user may
		have made to the main item list and remove the popup box.

INPUT:          widget information
		client data: pointer to the alarm list

OUTPUT:		changes to main item list

RETURNS:	none

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development

PROJECT:        ADAPS
**************************************************************************/
void apply_alert_changes
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data, alarm list         */
    XtPointer call_data         /* IN - call data                       */
    )
    {
    char   wstring[CMLEN];	/* widget string			*/

    short  changes = FALSE;	/* flag: T/F changes made to alarm list	*/

    struct ITEM *alarmlist = (struct ITEM *)client_data; /* the alarms	*/
    struct ITEM *curr_alarm;	/* pointer to current alarm		*/
    struct ITEM *curr_item;	/* pointer to current item		*/
    struct ITEM *itemlist;	/* the main item list			*/

    Widget scrolledlist;	/* the main display scrolled list 	*/
    Widget topshell;		/* top shell of calling widget		*/


    /* Go through the two lists and updating the item list
    with any changes.  For each alarm search the item list
    for the corresponding item.  NOTE: both lists are
    ordered the same so multiple searches are unecessary.
    ------------------------------------------------------*/
    curr_item = itemlist = get_list_address();
    curr_alarm = alarmlist;
    /* for each alarm */
    while ( curr_alarm != NULL )
        {
	/* look for corresponding item */
	while ( strcmp( curr_alarm->itemname, curr_item->itemname ) != 0 ||
	        strcmp( curr_alarm->machine, curr_item->machine ) != 0 )
	    curr_item = curr_item->next;
	
	/* see if the monitoring state changed */
	if ( curr_item->monitor_state != curr_alarm->monitor_state )
	    {
	    /* update main item list */
	    curr_item->monitor_state = curr_alarm->monitor_state;
	    curr_item->fieldflg[1] = curr_alarm->fieldflg[1];
	    curr_item->alarm = curr_alarm->alarm;
	    create_displist( curr_item );
	    changes = TRUE;
	    }
	
	/* go to next alarm */
	curr_alarm = curr_alarm->next;
	}  /* END while */

    
    /* If any changes were made update the main display
    ---------------------------------------------------*/
    if ( changes )
        {
	/* get the scrolled window that contains the display */
	sprintf( wstring, "%s.%s.scrolledlistSW.%s", WnMAINW, WnMWFORM2,
	    WnSCROLLEDLIST );
	scrolledlist = XtNameToWidget( get_toplevel_shell( w ), wstring );

	/* update main display */
	update_displist( scrolledlist, itemlist );

	/* save the current state */
	save_state( itemlist );
	}

    /* free the alarmlist
    ---------------------*/
    free_itemlist( &alarmlist );

    /* get the top level alarm shell and destroy it
    -----------------------------------------------*/
    topshell = get_top_shell( w );
    XtDestroyWidget( topshell );
    }  /* END apply_alert_changes */
