/*****************************************************************************
FUNCTION:       check_file

PURPOSE:        Check a file to make sure it is up to date

INPUT:          ITEM structure

OUTPUT:         none

RETURNS:        States - UPTODATE, OUTOFDATE and UNCHECKED

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   10/00   D.Lloyd         Initial development

PROJECT:        ADAPS
******************************************************************************/
#include <string.h>
#include <stdio.h>
#include "adapsmonitor.h"


States check_file
    (
    struct ITEM *item,		/* IN - the item structure to check	*/
    time_t      now,            /* IN - the current time                */
    int    check_name		/* IN - flag T/F, T = check itemname	*/
    				/*                F = check log_name	*/
    )
    {
    char   buf[CMLEN];          /* the pipe output buffer               */
    char   syscommand[CMLEN];   /* system command to query the queue    */
    FILE   *pipefd;		/* the pipe file descriptor		*/
    long   cur_time;		/* current time of the file		*/
    States status = UNCHECKED;	/* function return status		*/
    int    pipestatus;		/* return status of the pipe command	*/


    /* Build a system command, because of the use of rsh there
    is no need to check for and expand environment variables.
    ----------------------------------------------------------*/
    if ( check_name )
	sprintf( syscommand, "rsh %s -n /sbin/stat -mq '%s'",
	    item->machine, item->itemname );
    else
        sprintf( syscommand, "rsh %s -n /sbin/stat -mq '%s'",
	    item->machine, item->log_name );

    /* Open the pipe and exec the command
    -------------------------------------*/
    pipefd = popen( syscommand, "r" );
    if ( !pipefd )
        {
	write_log( FALSE, "check_file", "nopipe", "Unable to open pipe" );
	return status;
	}
    
    /* Read the pipe input
    ----------------------*/
    while ( fgets( buf, CMLEN, pipefd ) != NULL );
    pipestatus = pclose( pipefd );
    if ( pipestatus != 0 )
        return status;
    

    /* Convert the time to an integer and
    check it against the last time.
    -------------------------------------*/
    if ( sscanf( buf, "%ld", &cur_time ) != 1 )
        return status;
    if ( now - (time_t)cur_time < item->mon_interval )
	{
	item->last_check = (time_t)cur_time;
        status = UPTODATE;
	}
    else
	{
	item->last_check = now;
        status = OUTOFDATE;
	}
    
    return status;
    }  /* END check_file */
