/*****************************************************************************
FUNCTION:	check_item

PURPOSE:	Check the status of the item passed.

INPUT:		pointer to the item to be checked
		the current time

OUTPUT:		none

RETURNS:	States - the current state of the item

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development

PROJECT:        ADAPS
******************************************************************************/
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include "adapsmonitor.h"



States check_item
    (
    struct ITEM *item,		/* IN - pointer to item structure	*/
    time_t	now		/* IN - the current time		*/
    )
    {
    int    occurrence;		/* which batch occurrence to check for	*/
    States status = UNCHECKED;	/* function return status		*/


    switch ( item->itemtype )
        {
	case  DATAFILE :
	    status = check_file( item, now, TRUE );
	    break;
	    
	case  BATCH :
	    /* Check the status of all items that
            match user ID If one occurrence is not
            in queue quit and return its status.
	    --------------------------------------*/
	    for ( occurrence = 1; occurrence <= item->num_batch; occurrence++ )
		{
		status = check_queue( item, occurrence );
		if ( status == NOTINQUEUE )
		    break;
		}
	    if ( status == NOTINQUEUE )
	        break;
	    
	    /* NO break, fall through to next case.
	    Even though the process is in the queue,
	    it may not be doing anything.
	    ---------------------------------------*/

	case  PROCESS :
	    if ( check_file( item, now, FALSE ) == UPTODATE )
	        status = RUNNING;
	    else
	        status = NOTRUNNING;
	    
	    break;

	case  VAXPROCESS :
	    /* get the status of the vax process
	    ------------------------------------*/
	    status = check_vax( item );
	    item->last_check = now;

	    if ( status == RWAST )
	        {
		/* If the rwast_flg is TRUE, the last time we checked
                this vax process its state was RWAST.  We need to see
                if the RWAST time has been exceeded.
		-----------------------------------------------------*/
		if ( item->rwast_flg == TRUE )
		    {
		    /* If we haven't exceeded the time, send back
		    the last state we have.  Otherwise we leave
		    the status as RWAST which will be returned.
		    ---------------------------------------------*/
		    if ( now - item->rwast_time < item->rwast_interval )
		        status = item->current_state;
		    }

		/* Otherwise this is the first time the RWAST
                state has been encountered.  Set the flag and
                time to now and set the status to the current
                state.since we don't want to trigger a RWAST
                state until the RWAST time interval has elapsed.
		------------------------------------------------*/
		else
		    {
		    item->rwast_flg = TRUE;
		    item->rwast_time = now;
		    status = item->current_state;
		    }
		}  /* END if status */

	    /* The process is not in RWAST but may have
	    been in the past.  So reset the flag and time.
	    ---------------------------------------------*/
	    else
	        {
		item->rwast_flg = FALSE;
		item->rwast_time = 0;
		}

            break;

	case  SYSTEM :
	    /* Check the system
	    -------------------*/
	    status = check_system( item );
	    item->last_check = now;

	    break;

	default :
	    status = UNCHECKED;
	}


    return status;
    }  /* END check_item */

	    

