/*****************************************************************************
FUNCTION:	check_queue

PURPOSE:	Check the batch queue for the presence of the items process.

INPUT:		ITEM structure

OUTPUT:		none

RETURNS:	States - INQUEUE, NOTINQUEUE, UNCHECKED

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development
  1.1   10/01	D.Lloyd		CR5595: Changed to call queuestat.csh instead
  				of qstat and only compare the first letter of
				the queue status word.

PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "adapsmonitor.h"


States check_queue
    (
    struct ITEM *item,		/* IN - the item structure to check	*/
    int    occurrence		/* IN - which occurrence to check	*/ 
    )
    {
    States status = UNCHECKED;	/* function return/process status	*/
    FILE   *pipefd;		/* the pipe file descriptor		*/
    char   syscommand[CMLEN];   /* system command to query the queue	*/
    char   buf[CMLEN];		/* the pipe output buffer		*/
    char   questatus[CMLEN];	/* status of the queue process		*/
    char   user[CMLEN];
    int    count = 0;		/* count of processes in queue		*/


    /* Set up the command string
    ----------------------------*/
    sprintf( syscommand, "rsh %s -n queuestat.csh %s",
        item->machine, item->itemname );


    /* Open the pipe and exec the command
    -------------------------------------*/
    pipefd = popen( syscommand, "r" );
    if ( !pipefd )
        {
	write_log( FALSE, "check_queue", "nopipe", "Unable to open pipe" );
	return status;
	}

    
    /* Read the pipe input
    ----------------------*/
    while ( fgets( buf, CMLEN, pipefd ) != NULL )
        {
	count++;
	if ( count == occurrence )
	    {
	    /* If nothing is returned from the
	    pipe the job is not in the queue.
	    ----------------------------------*/
	    if ( sscanf( buf, "%*s %*s %s %*s %s", user, questatus ) == 0 )
	        status = NOTINQUEUE;
	    else
		{
		if ( strcasecmp( user, item->userid ) != 0 )
		    status = NOTINQUEUE;
		else if ( toupper( questatus[0] ) != 'R' )
		    status =  NOTRUNNING;
		else
		    status = RUNNING;
		}
	    }
	}  /* END while */

    
    /* Close the pipe
    -----------------*/
    if ( pclose( pipefd ) != 0 )
        return UNCHECKED;


    /* We only care if the process was found in the queue
    or not, so if it was found return that it was in the queue
    ----------------------------------------------------------*/
    if ( status == RUNNING || status == NOTRUNNING )
        return INQUEUE;
    else
	return NOTINQUEUE;
    } /* END check_queue */
