/*****************************************************************************
FUNCTION:	check_system

PURPOSE:	Check that a system is responding

INPUT:		ITEM structure

OUTPUT:		none

RETURNS:	States - RESPONDING, NOTRESPONDING, UNCHECKED

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development

PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "adapsmonitor.h"


States check_system
    (
    struct ITEM *item		/* IN - the item structure to check	*/
    )
    {
    char   buf[CMLEN];		/* the pipe output buffer		*/
    char   syscommand[CMLEN];   /* system command to query the queue	*/

    FILE   *pipefd;		/* the pipe file descriptor		*/

    int    count = 0;		/* count of processes in queue		*/

    States status = UNCHECKED;	/* function return/process status	*/


    /* Set up the command string.  Note the -n in the rsh command. 
    This is to prevent it from blocking when no imput is available.
    ---------------------------------------------------------------*/
    sprintf( syscommand, "rsh %s -n date", item->machine );


    /* Open the pipe and exec the command
    -------------------------------------*/
    pipefd = popen( syscommand, "r" );
    if ( !pipefd )
        {
	write_log( FALSE, "check_system", "nopipe", "Unable to open pipe" );
	return status;
	}

    
    /* Read the pipe input.  Count the number of
    lines returned from the stdout of the rsh call.
    Any errors in connection will be returned to 
    stderr and won't be counted here.
    ----------------------------------------------*/
    while ( fgets( buf, CMLEN, pipefd ) != NULL )
	count++;

    
    /* Close the pipe
    -----------------*/
    if ( pclose( pipefd ) != 0 )
        return status;


    if ( count == 0 )
        return NOTRESPONDING;
    else
        return RESPONDING;
    } /* END check_system */
