/*****************************************************************************
FUNCTION:	check_vax

PURPOSE:	Check process on the vax and return its status

INPUT:		ITEM structure

OUTPUT:		none

RETURNS:	States - RUNNING, RWAST, SUSPENDED, NOTRUNNING, UNCHECKED

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development

PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "adapsmonitor.h"


States check_vax
    (
    struct ITEM *item		/* IN - the item structure to check	*/
    )
    {
    char   buf[CMLEN];		/* the pipe output buffer		*/
    char   state[CMLEN];	/* status of the queue process		*/
    char   syscommand[CMLEN];   /* system command to query the queue	*/

    FILE   *pipefd;		/* the pipe file descriptor		*/

    int    count = 0;		/* count of processes in queue		*/

    States status = UNCHECKED;	/* function return/process status	*/


    /* Set up the command string.  Note the -n in the rsh command.  This 
    is to prevent the command from blocking when no input is desired.
    -------------------------------------------------------------------*/
    sprintf( syscommand, "rsh vaxh -n sh sys | grep -i %s", item->itemname );


    /* Open the pipe and exec the command
    -------------------------------------*/
    pipefd = popen( syscommand, "r" );
    if ( !pipefd )
        {
	write_log( FALSE, "check_vax", "nopipe", "Unable to open pipe" );
	return status;
	}

    
    /* Read the pipe input
    ----------------------*/
    while ( fgets( buf, CMLEN, pipefd ) != NULL )
        {
	count++;
	if ( sscanf( buf, "%*s %*s %s", state ) == 0 )
	    status = UNCHECKED;
	else
	    {
	    if ( strcasecmp( state, "RWAST" ) == 0 )
		status =  RWAST;
	    else if ( strcasecmp( state, "SUS" ) == 0 )
		status = SUSPENDED;
	    else
		status = RUNNING;
	    }
	}  /* END while */

    
    /* Close the pipe
    -----------------*/
    if ( pclose( pipefd ) != 0 )
        return status;


    if ( count == 0 )
        return NOTRUNNING;
    else
        return status;
    } /* END check_vax */
