/**************************************************************************
FUNCTION:	create_main_display

PURPOSE:	Create the main display window.

INPUTS:		top level shell widget
		list of items

OUTPUTS:	none

RETURNS:	the main display widget

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
***************************************************************************/
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include "adapsmonitor.h"



Widget create_main_display
    (
    Widget toplevel,		/* IN - top level shell widget		*/
    struct ITEM *list		/* IN - item list structure		*/
    )
    {
    Widget      return_window;	/* main window widget                   */
    Widget      mwform1;        /* main window form widgets             */
    Widget      mwform2;        /* main window form widgets             */
    Widget      mwform3;        /* main window form widgets             */
    Widget      header;         /* static header for form widget        */
    Widget      modifybutton;   /* modify pushbutton widget             */
    Widget      exitbutton;     /* exit pushbutton widget               */
    Widget      scrolledlist;	/* the scrolled list widget		*/


    /* Create the main window, this is the one that gets returned.
    --------------------------------------------------------------*/
    return_window = XtVaCreateManagedWidget( WnMAINW,
        xmFormWidgetClass, toplevel, NULL );
    
    /* Attach a form widget to the main window
    to hold the static header information.
    ------------------------------------------*/
    mwform1 = XtVaCreateWidget( WnMWFORM1, xmFormWidgetClass, return_window,
        XmNleftAttachment, XmATTACH_FORM,
	XmNleftOffset, 2,
	XmNrightAttachment, XmATTACH_FORM,
	XmNrightOffset, 2,
	XmNtopAttachment, XmATTACH_FORM,
	XmNtopOffset, 2,
	XmNheight, 32,
	XmNresizable, True,
	XmNx, 2,
	XmNy, 2,
	NULL );
    
    /* Create the static header and attach it to the first form.
    ------------------------------------------------------------*/
    header = XtVaCreateWidget( WnHEADER, xmLabelWidgetClass, mwform1,
        XtVaTypedArg, XmNlabelString, XmRString,
        "Monitor         Process/System/File                            Status            As of", 87,
        XmNalignment, XmALIGNMENT_BEGINNING,
        XmNx, 7,
        XmNy, 8,
        XmNheight, 23,
        NULL );
    XtManageChild( header );

    /* Attach another form to the main window to
    hold the monitor listing and status.
    --------------------------------------------*/
    mwform2 = XtVaCreateWidget( WnMWFORM2, xmFormWidgetClass, return_window,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 2,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 2,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopOffset, 36,
        XmNbottomAttachment, XmATTACH_WIDGET,
        XmNbottomOffset, 2,
        XmNx, 2,
        XmNy, 36,
        XmNheight, 365,
        NULL );

    /* Create a scrolled list and attach it to the second form
    ----------------------------------------------------------*/
    scrolledlist = XmCreateScrolledList(mwform2, WnSCROLLEDLIST, NULL, 0 );
    XtVaSetValues(XtParent(scrolledlist), XmNx, 0, XmNy, 0,
        XmNleftAttachment, XmATTACH_FORM,
        XmNtopAttachment, XmATTACH_FORM,
        XmNrightAttachment, XmATTACH_FORM,
        XmNbottomAttachment, XmATTACH_FORM,
        NULL );
    XtVaSetValues(scrolledlist,
        XmNx, 0,
        XmNy, 0,
        NULL );
    XtManageChild( scrolledlist );
    update_displist( scrolledlist, list );
    XtManageChild( mwform2 );


    /* Attach a third form to the mainwindow to contain
    the button bar Labels for the buttons are contained
    in Adapsmonitor.ad and fallbacks.h
    ----------------------------------------------------*/
    mwform3 = XtVaCreateWidget( WnMWFORM3, xmFormWidgetClass, return_window,
        XmNheight, 40,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 2,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 2,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 2,
        XmNresizable, True,
	XmNfractionBase, 5,
        XmNy, 403,
        NULL );


    /* Add an exit button to the button bar
    ---------------------------------------*/
    exitbutton = XtVaCreateWidget( WnEXITBUTTON, xmPushButtonWidgetClass,
        mwform3,
        XmNheight, 30,
        XmNwidth, 100,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
	XmNrightAttachment, XmATTACH_POSITION,
	XmNrightPosition, 5,
	XmNrightOffset, 20,
        XmNresizable, False,
        NULL );
    XtAddCallback( exitbutton , XmNactivateCallback, exit_callback, NULL );
    XtManageChild( exitbutton );


    /* Add a modify pushbutton to the button bar
    --------------------------------------------*/
    modifybutton = XtVaCreateWidget( WnMODIFYBUTTON, xmPushButtonWidgetClass,
        mwform3,
        XmNheight, 30,
        XmNwidth, 100,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
	XmNleftAttachment, XmATTACH_POSITION,
	XmNleftPosition, 0,
	XmNleftOffset, 20,
        XmNresizable, False,
        NULL );
    XtManageChild( modifybutton );
    XtAddCallback( modifybutton , XmNactivateCallback, modify_callback,
        NULL );


    XtManageChild( mwform3 );
    XtManageChild( mwform1 );


    /* Attach mwform2 to mwform3.  This is done here because
    mwform3's value was not set when setting up mwform2.
    ----------------------------------------------------*/
    XtVaSetValues( mwform2, XmNbottomWidget, mwform3, NULL );

    return return_window;
    }  /* END create_main_display */
