/*****************************************************************************
FUNCTION:	create_modify

PURPOSE:	To create the modify window.  This function creates all of
		the modify window except for the row column child of the
		scrolled window.  Since the row column child is a dynamic
		entity, ie the number of its children can vary during
		runtime, it is created when the main window Modify button
		is pressed.

INPUT:		parent widget
		pointer to ITEM structure

OUTPUT:		none

RETURNS:	The monitor widget

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <Xm/Xm.h>
#include <Xm/Label.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>
#include <Xm/ScrolledW.h>
#include <Xm/RowColumn.h>
#include "adapsmonitor.h"


Widget create_modify
    (
    Widget parent,		/* IN - the parent screen widget	*/
    struct ITEM *item		/* IN - the list of items		*/
    )
    {
    Widget modifyform;		/* the main form widget			*/
    Widget headerform;		/* form that contains the header widget	*/
    Widget header;		/* the header or title widget		*/
    Widget scrollform;		/* form that contains the scroll widget	*/
    Widget scrollwindow;	/* window that contains working widgets	*/
    Widget buttonform;		/* form that contains the buttons	*/
    Widget applybutton;		/* button to apply changes		*/
    Widget cancelbutton;	/* button to cancel changes		*/
    Widget loadbutton;		/* button to load new file		*/


    /* Create the main form widget.  This is the widget that
    gets passed back to main.
    --------------------------------------------------------*/
    modifyform = XtVaCreateWidget( WnMODIFY, xmFormWidgetClass,
        parent,
	XmNx, 20,
	XmNy, 20,
	NULL );


    /* Add the first form to the main form.  This form contains
    a header or title or the window.
    -----------------------------------------------------------*/
    headerform = XtVaCreateWidget( WnHEADERFORM, xmFormWidgetClass, modifyform,
        XmNleftAttachment, XmATTACH_FORM,
	XmNleftOffset, 2,
	XmNrightAttachment, XmATTACH_FORM,
	XmNrightOffset, 2,
	XmNtopAttachment, XmATTACH_FORM,
	XmNtopOffset, 2,
	XmNheight, 32,
	XmNresizable, True,
	XmNx, 2,
	XmNy, 2,
	NULL );
    XtManageChild( headerform );
    
    /* Create the static header and attach it to the first form.
    ------------------------------------------------------------*/
    header = XtVaCreateWidget( WnMODIFYHEADER, xmLabelWidgetClass, headerform,
	XmNx, 7,
	XmNy, 8,
	XmNheight, 23,
	NULL );
    XtManageChild( header );

    /* Attach another form to the window to hold the 
    working widgets which are the buttons to enable and
    disable monitoring.
    ---------------------------------------------------*/
    scrollform = XtVaCreateWidget( WnSCROLLFORM, xmFormWidgetClass, modifyform,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 2,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 2,
        XmNtopAttachment, XmATTACH_WIDGET,
        XmNtopOffset, 36,
        XmNbottomAttachment, XmATTACH_WIDGET,
        XmNbottomOffset, 2,
        XmNx, 2,
        XmNy, 36,
        NULL );


    /* Attach a scrollable window to the second form
    ------------------------------------------------*/
    scrollwindow = XtVaCreateManagedWidget( WnSCROLLEDWINDOW1,
        xmScrolledWindowWidgetClass,
	scrollform,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 2,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 2,
        XmNtopAttachment, XmATTACH_FORM,
        XmNtopOffset, 2,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 2,
	XmNscrollingPolicy, XmAUTOMATIC,
	XmNscrollBarDisplayPolicy, XmAS_NEEDED,
	XmNvisualPolicy, XmCONSTANT,
	NULL );
    XtManageChild( scrollwindow );
    XtManageChild( scrollform );


    /* Attach a third form to contain the button bar
    ------------------------------------------------*/
    buttonform = XtVaCreateWidget( WnBUTTONFORM, xmFormWidgetClass, modifyform,
        XmNheight, 40,
        XmNleftAttachment, XmATTACH_FORM,
        XmNleftOffset, 2,
        XmNrightAttachment, XmATTACH_FORM,
        XmNrightOffset, 2,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 2,
        XmNresizable, True,
	XmNfractionBase, 5,
        XmNy, 403,
        NULL );
    XtManageChild( buttonform );

    /* Add a cancel button to the button bar
    ----------------------------------------*/
    cancelbutton = XtVaCreateWidget( WnCANCELBUTTON, xmPushButtonWidgetClass,
        buttonform,
        XmNheight, 30,
        XmNwidth, 100,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNrightAttachment, XmATTACH_POSITION,
	XmNrightPosition, 5,
        XmNrightOffset, 20,
        XmNresizable, False,
        NULL );
    XtManageChild( cancelbutton );

    /* Add a Load button to the button bar
    ---------------------------------------*/
    loadbutton = XtVaCreateWidget( WnLOADBUTTON, xmPushButtonWidgetClass,
        buttonform,
        XmNheight, 30,
        XmNwidth, 100,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNleftAttachment, XmATTACH_POSITION,
	XmNleftPosition, 1,
	XmNleftOffset, 20,
        XmNresizable, False,
        NULL );
    XtManageChild( loadbutton );

    /* Add an Apply button to the button bar
    ----------------------------------------*/
    applybutton = XtVaCreateWidget( WnAPPLYBUTTON, xmPushButtonWidgetClass,
        buttonform,
        XmNheight, 30,
        XmNwidth, 100,
        XmNbottomAttachment, XmATTACH_FORM,
        XmNbottomOffset, 5,
        XmNleftAttachment, XmATTACH_POSITION,
	XmNleftPosition, 0,
	XmNleftOffset, 20,
        XmNresizable, False,
        NULL );
    XtManageChild( applybutton );

    /* Attach scrollform to buttonform.  This is done here because
    buttonform's value was not available when setting up scrollform.
    ---------------------------------------------------------------*/
    XtVaSetValues( scrollform, XmNbottomWidget, buttonform, NULL );

    return modifyform;
    }  /* END create_modify */
