/**************************************************************************
FUNCTION:	create_toggle_array

PURPOSE:	Create the row column form for the parent window with
		buttons for each item in the item list.  This can be used
		to create the window for both the modify window and the
		alert window by specifying the appropriate root widget
		name.

INPUTS:		parent widget
		pointer to the item list
		root widget name WnALERTWINDOW or WnMODIFY

OUTPUTS:	none

RETURNS:	Widget the row column widget created

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include "adapsmonitor.h"


Widget create_toggle_array
    (
    Widget parent,		/* IN - parent widget			*/
    struct ITEM *list,		/* IN - list of items			*/
    char   widget_root[]	/* IN - root widget name		*/
    )
    {
    Widget formw;		/* form to hold the buttons and labels	*/
    Widget pushbutton;		/* the enabled/disabled push button	*/
    Widget rowcol;		/* the row column widget		*/

    XmString text;		/* text for button and label names	*/

    char   alertstring[CMLEN];	/* the alert label 			*/
    char   button_name[CMLEN];	/* resource name of the buttons		*/
    char   changebgcolor[CMLEN];/* color of background for a change	*/
    char   form_name[CMLEN];	/* resource name of the button labels	*/
    char   label_name[CMLEN];	/* resource name of the button labels	*/
    char   normalbgcolor[CMLEN];/* color of background for no change	*/
    char   rowcol_name[CMLEN];	/* resource name of the buttons		*/

    short  numcol;		/* number of columns desired		*/

    struct ITEM *curr;		/* current item in item list		*/


    /* Set up the resource names and number of columns
    --------------------------------------------------*/
    if ( strcmp( widget_root, WnMODIFY ) == 0 )
        {
	strcpy( rowcol_name, WnMODROWCOL );
	strcpy( button_name, WnMODBUTTONS );
	strcpy( label_name,  WnMODLABELS );
	strcpy( form_name,   WnMODFORM );
	strcpy( changebgcolor, CHANGEBGCOLOR );
	strcpy( normalbgcolor, NORMALBGCOLOR );
	numcol = 2;
	}
    else if ( strcmp( widget_root, WnALERTWINDOW ) == 0 )
        {
	strcpy( rowcol_name, WnALERTROWCOL );
	strcpy( button_name, WnALERTBUTTONS );
	strcpy( label_name,  WnALERTLABELS );
	strcpy( form_name,   WnALERTFORM );
	strcpy( changebgcolor, ALERTCHANGEBGCOLOR );
	strcpy( normalbgcolor, ALERTNORMALBGCOLOR );
	numcol = 1;
	}
    else
        return NULL;

    /* Create the row column widget
    -------------------------------*/
    rowcol = XtVaCreateManagedWidget( rowcol_name, xmRowColumnWidgetClass,
        parent,
	XmNnumColumns, numcol,
	XmNpacking, XmPACK_COLUMN,
	XmNorientation, XmVERTICAL,
	NULL );


    /* Create each of the buttons and labels
    ----------------------------------------*/
    curr = list;
    /* for each item in the list
    ----------------------------*/
    while ( curr != NULL ) 
        {
	/* Set the background color for this item
	-----------------------------------------*/
	strcpy( curr->changebgcolor, changebgcolor );
	strcpy( curr->normalbgcolor, normalbgcolor );
	curr->mon_state_mod = FALSE;

	/* Create a form to hold the button and
	label, attaching it to the row column
	---------------------------------------*/
	formw = XtVaCreateManagedWidget( form_name, xmFormWidgetClass,
	    rowcol, NULL );

	/* Create the button for this item.  We need the Widget
	structure to register the call back for the push button.
	-------------------------------------------------------*/
	if ( curr->monitor_state == ENABLED )
	    text = XmStringCreateLocalized( ENABLESTRING );
	else
	    text = XmStringCreateLocalized( DISABLESTRING );
	pushbutton = XtVaCreateManagedWidget( button_name,
	    xmPushButtonWidgetClass, formw,
	    XmNlabelString, text,
	    XmNleftAttachment, XmATTACH_FORM,
	    XmNwidth, 80,
	    XmNresizable, False,
	    NULL );
	XmStringFree( text );
	XtAddCallback( pushbutton, XmNactivateCallback, enable_callback,
	    curr );

	/* Create the label for this item.  We don't need the
	return value for the widget.
	-----------------------------------------------------*/
	if ( strcmp( widget_root, WnALERTWINDOW ) == 0 )
	    {
	    sprintf( alertstring, "%s %s", curr->name_field,
	        curr->status_field );
	    text = XmStringCreateLocalized( alertstring );
	    }
	else
	    text = XmStringCreateLocalized( curr->name_field );
	XtVaCreateManagedWidget( label_name, xmLabelWidgetClass,
	    formw,
	    XmNleftAttachment, XmATTACH_WIDGET,
	    XmNleftWidget, pushbutton,
	    XmNlabelString, text,
	    XmNmarginTop, 3,
	    NULL );
	XmStringFree( text );

	/* Go to the next item
	----------------------*/
	curr = curr->next;
        }  /* END while curr != NULL */

    return rowcol;
    }  /* END create_toggle_array */
