/*************************************************************************
FUNCTION:	enable_callback

PURPOSE:	To switch the push button label from enabled to disabled
		or vice versa and to modify the structure to reflect
		the new state.

INPUTS:		widget data
		pointer to an item list

OUTPUTS:	new widget label
		item structure monitor state

RETURNS:	none
*************************************************************************/
#include <stdio.h>
#include <string.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include "adapsmonitor.h"


void enable_callback
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    )
    {
    struct ITEM *list = (struct ITEM *)client_data; /* the list		*/
    XmString text;		/* text for button and label names	*/
    char bgcolor[CMLEN];	/* color string for a pushed button	*/


    /* Change the monitor state and create the new label string
    -----------------------------------------------------------*/
    list->fieldflg[1] = INVALID;
    if ( list->monitor_state == ENABLED )
	{
        list->monitor_state = DISABLED;
	list->alarm = FALSE;
	text = XmStringCreateLocalized( DISABLESTRING );
	}
    else
	{
        list->monitor_state = ENABLED;
	if (( list->current_state != RUNNING ) &&
	    ( list->current_state != UPTODATE ))
	    list->alarm = TRUE;
	text = XmStringCreateLocalized( ENABLESTRING );
	}

    /* Change background color of the button to indicate the change
    ---------------------------------------------------------------*/
    if ( list->mon_state_mod == FALSE )
        {
        strcpy( bgcolor, list->changebgcolor );
	list->mon_state_mod = TRUE;
	}
    else
        {
	strcpy( bgcolor, list->normalbgcolor );
	list->mon_state_mod = FALSE;
	}
    
    /* Change the widget label
    --------------------------*/
    XtVaSetValues( w,
        XmNlabelString, text,
	XtVaTypedArg, XmNbackground, XmRString, bgcolor, strlen(bgcolor),
	NULL );
    XmStringFree( text );
    }  /* END enable_callback */
