/****************************************************************************
FUNCTION:	exit_callback

PURPOSE:	To provide an exit confirmation which allows the user
  		to cancel the exit or to exit.  On exit memory is freed.

INPUT:	        The exit button widget
		client data -- pointer to the item list
		call data -- not used

OUTPUT:         None

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
#include <stdlib.h>
#include <Xm/Xm.h>
#include "adapsmonitor.h"


void exit_callback
    (
    Widget    w,		/* IN - widget associated with the CB	*/
    XtPointer client_data,	/* IN - client data, item list		*/
    XtPointer call_data		/* IN - call data			*/
    )
    {
    struct ITEM *itemlist = get_list_address(); /* the item list	*/

    /* Display a popup message and get response.
    Return immediately if the user responds no.
    --------------------------------------------*/
    if ( get_user_response( w, "Do you really want to quit?", "Yes",
                               "No", YES ) == NO )
	return;

    /* save the state before releasing the list
    -------------------------------------------*/
    save_state( itemlist );

    /* Free the list
    ----------------*/
    free_itemlist( &itemlist );

    /* Destroy any windows
    ----------------------*/
    XtDestroyWidget( get_toplevel_shell( w ));

    /* write a log file and exit
    ----------------------------*/
    write_log( FALSE, "exit_callback", "exit",
        "ADAPSMONITOR terminated by user" );
    exit(0);
    }  /* END exit_callback */
