/****************************************************************************
FUNCTION:	get_timer_interval

PURPOSE:	Parse the command line for a -interval switch and return
		the value specified.  If none is specified return a zero.

INPUTS:		argv, array of command line arguments

OUTPUTS:	none

RETURNS:	unsigned long, timer interval in milliseconds or zero

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development

PROJECT:        ADAPS
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "adapsmonitor.h"


unsigned long get_timer_interval
    (
    char *argv[]		/* IN - command line argument array	*/
    )
    {
    unsigned long timer_interval = 0; /* interval from command line	*/
    char msg[CMLEN];		/* log message				*/


    /* Search for the -interval switch in argv
    ------------------------------------------*/
    while ( *++argv != NULL )
        {
	if ( strcmp( *argv, "-interval" ) == 0 )
	    {
	    argv++;
	    /* NOTE: don't combine the if and the else if with an OR.  If
	    *argv is NULL the combined statement may cause a segmentation
	    fault depending on the expression evaluation sequence.
	    -------------------------------------------------------------*/
	    if ( *argv == NULL )
	        {
		write_log( FALSE, "get_timer_interval", "invalid-arg",
		    "Usage: -interval <timer interval in seconds>" );
		}
	    else if ( *argv[0] == '-' )
	        {
		write_log( FALSE, "get_timer_interval", "invalid-arg",
		    "Usage: -interval <timer interval in seconds>" );
		}
	    else
	        {
		timer_interval = strtoul( *argv, NULL, 10 );
		timer_interval *= 1000;
		/* Make sure the timer interval is reasonable.
		----------------------------------------------*/
		if ( timer_interval > MAXTIMERINTERVAL )
		    {
		    sprintf( msg, "Timer interval %d reset to %d",
		        timer_interval / 1000, DEFAULTINTERVAL / 1000 );
		    write_log( FALSE, "get_timer_interval", "toobig", msg );
		    return DEFAULTINTERVAL;
		    }
		}
	    break;
	    }  /* END if */
	}  /* END while */
    
    return timer_interval;
    }  /* END get_timer_interval */
