/*****************************************************************************
FUNCTION:	initialize

PURPOSE:	To scan the input argument list for a initialization file
  		specification and to initialize the item list.  If the
		initialization file is not specified the $ADAPSTABLES/
		adapsmonitor.default file will be used.

INPUT:		The command line argv[] array

OUTPUT:		None

RETURNS:	struct ITEM *, the filled list

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "adapsmonitor.h"

struct ITEM *initialize
    (
    char *argv[]		/* IN - argument list			*/
    )
    {
    char   filename[CMLEN];	/* initialization file name		*/
    char   *adapstables;	/* pointer to environment variable	*/

    short  fileflg;		/* flag T/F: found a -f switch in argv	*/


    /* Search for a -f argument in argv.  This argument
    indicates the initialization file name.
    ---------------------------------------------------*/
    fileflg = FALSE;
    while ( *++argv != NULL )
        {
	if ( strcmp( *argv, "-f" ) == 0 )
	    {
	    argv++;
	    /* There must be another argument
	    ---------------------------------*/
	    if ( *argv == NULL )
	        write_log( FALSE, "initialize", "invalid-arg",
		    "Usage: -f <filename>, switch ignored" );
	    /* The argument following -f should not be another switch
	    ---------------------------------------------------------*/
	    else if ( *argv[0] == '-' )
		/* write an error message */
		write_log( FALSE, "initialize", "invalid-arg",
		    "Usage: -f <filename>, switch ignored" );
	    else
		fileflg = TRUE;
	    break;
	    }
	}  /* END while */
    
    /* If a file was specified copy the file name otherwise
    build the default file name.
    -------------------------------------------------------*/
    if ( fileflg )
	strcpy( filename, *argv );
    else
        {
        adapstables = getenv( "ADAPSTABLES" );
        if ( adapstables  == NULL )
	    {
	    write_log( TRUE, "initialize", "environment",
	        "ADAPSTABLES not defined" );
	    return NULL;
	    }
	sprintf( filename, "%s/%s", adapstables, ADAPSMONITORDEFAULT );
	}
    
    /* return an initialized structure
    ----------------------------------*/
    return initstruct( filename, TRUE );
    }  /* END initialize */
