#ifndef MONITORCONTROL_H
#define MONITORCONTROL_H

#include <qwidget.h>
#include <qtimer.h>
#include "monitorcontrolbase.h"

typedef struct MONITOR_STRUCT
{
    QString type;
    QString name;
    int interval;
    int default_interval;
    QString file;
    int pid;
    int ppid;
    int row;
    int itype;
    int dsp_flg;
    bool stop_pending;
    bool kill_pending;
} MONITOR_STRUCT;

enum TAGS { RUNNING, IGNORE, START, STOP, STOPPEND, STOPPED, RESTART, KILL,
    UPTODATE, STOWED, REMOTE, NOTRUNNING, KILLED, NOTUPTODATE,
    NOTSTOWED, LOCAL, HANG };
enum OBJ_TYPE { UNK, A_DAEMON, A_FILE, AN_ACU };

class MonitorControl : public MonitorControlBase
{
    Q_OBJECT

public:
    MonitorControl( QWidget* parent = 0, const char* name = 0,
        WFlags fl = 0 );  
    ~MonitorControl();

public slots:
    void proc_change( int, int );
    void file_change( int, int );
    void acu_change( int, int );
    void response( int, int, OBJ_TYPE );
    void check( void );
    void testem( void );
    void all_start( void );
    void all_stop( void );

private:
    bool init( void );
    QStringList exec_cmd( QString );
    bool read_resource( void );
    bool process_running( MONITOR_STRUCT *, bool );
    bool file_up2date( MONITOR_STRUCT );
    void set_tag( MONITOR_STRUCT, TAGS );
    bool acu_query( MONITOR_STRUCT );
    bool stop_daemon( MONITOR_STRUCT * );
    int kill_daemon( MONITOR_STRUCT * );
    bool start_daemon( MONITOR_STRUCT * );
    bool remove_stop( MONITOR_STRUCT * );
    bool restart_daemon( MONITOR_STRUCT * );
    int  spawn_daemon( MONITOR_STRUCT * );
    bool create_stop( MONITOR_STRUCT * );

    QStringList process_list;
    QStringList file_list;
    QStringList monitor_defaults;
    MONITOR_STRUCT monitor_struct[30];
    int the_count;
    QMap <TAGS, QString> tags;
    QMap <TAGS, QColor> colors;
    QString ADAPSTABLES;
    QString ADAPSBASE;
    QString ADAPSLOG;
    QString file_cmd;
    QString proc_cmd;
    QString spawn_cmd;
    QTimer *timex;

};

#endif
#if 0
    tags[RUNNING]    = "Running";     colors[RUNNING]    = grn_color;
    tags[IGNORE]     = "Ignore";      colors[IGNORE]     = ltgrn_color;
    tags[START]      = "Start";       colors[START]      = blue_color;
    tags[STOP]       = "Stop";        colors[STOP]       = stop_color;
    tags[STOPPED]    = "Stopped";     colors[STOPPED]    = stop_color;
    tags[RESTART]    = "Restart";     colors[RESTART]    = stop_color;
    tags[KILL]       = "Kill";        colors[KILL]       = black_color;
    tags[UPTODATE]   = "Up to date";  colors[UPTODATE]   = grn_color;
    tags[STOWED]     = "Stowed";      colors[STOWED]     = grn_color;
    tags[REMOTE]     = "Remote";      colors[REMOTE]     = grn_color;
    tags[NOTRUNNING] = "Not Running"; colors[NOTRUNNING] = stop_color;
    tags[KILLED]     = "Killed";      colors[KILLED]     = black_color;
    tags[NOTUPTODATE]= "Out of date"; colors[NOTUPTODATE]= stop_color;
    tags[NOTSTOWED]  = "Not stowed";  colors[NOTSTOWED]  = stop_color;
    tags[LOCAL]      = "Local";       colors[LOCAL]      = stop_color;
#endif
