/****************************************************************************
FUNCTION:	post_dialog

PURPOSE:	A general purpose routine to popup a message dialog and get
  		the user response.  This is a full application modal function
		so that the system continues to respond to other events.
		But it doesn't allow the user to do anything else until he
		or she responds to the dialog.

INPUT:          widget of the caller		
		message to be displayed		
		the yes answer button label	

OUTPUT:         None

RETURNS:	none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include "adapsmonitor.h"


extern XtAppContext app;	/* main window parent declared external	*/
				/* so that the event loop can continue	*/
extern Widget toplevel;

void post_dialog
    (
    Widget   parent,		/* IN - widget of the caller		*/
    char     *question,		/* IN - message to be displayed		*/
    char     *ans		/* IN - the yes button label		*/
    )
    {
    static   Widget dialog;	/* dialog widget static to avoid dupes	*/

    XmString text;		/* text of the message			*/
    XmString yes;		/* text of the yes label		*/

    int      answer;		/* answer from response			*/
    

    /* Create the dialog box if it isn't already created
    ----------------------------------------------------*/
    if ( !dialog )
	{
	dialog = XmCreateMessageDialog( toplevel, WnDIALOG2, NULL, 0 );
	XtVaSetValues( dialog,
	    XmNdialogStyle,	XmDIALOG_FULL_APPLICATION_MODAL,
	    NULL );
        /* unmanage all but one button */
	XtUnmanageChild( XmMessageBoxGetChild( dialog, XmDIALOG_HELP_BUTTON ));
	XtUnmanageChild( XmMessageBoxGetChild( dialog, XmDIALOG_CANCEL_BUTTON));
	XtAddCallback( dialog, XmNokCallback, response, &answer );
	}

    /* Set up the text strings
    --------------------------*/
    text = XmStringCreateLocalized( question );
    yes = XmStringCreateLocalized( ans );
    XtVaSetValues( dialog,
        XmNmessageString, text,
	XmNokLabelString, yes,
	NULL );
    XmStringFree( text );
    XmStringFree( yes );

    /* Realize the dialog
    ---------------------*/
    XtManageChild( dialog );
    XtPopup( XtParent( dialog), XtGrabNone );


    /* duplicate the main processing loop while
    waiting for a response from the user.
    -------------------------------------------*/
    answer = FALSE;
    while ( answer == FALSE )
        XtAppProcessEvent( app, XtIMAll );

    /* Remove the dialog from the display
    -------------------------------------*/
    XtPopdown( XtParent( dialog ));
    XmUpdateDisplay( parent );

    }  /* END post_dialog */
