/******************************************************************************
FUNCTION:       save_state

PURPOSE:	To save the current setup in the default initialization file

INPUT:		Item list

OUTPUT:		initialization file

RETURNS:	none

Version Date    Programmer      Modifications
------- ------- --------------- --------------------------------------------
  1.0   08/00   D.Lloyd         Initial development

PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "adapsmonitor.h"

extern Widget toplevel;		/* top level shell widget		*/

void save_state
    (
    struct ITEM *item		/* IN - item list to be saved		*/
    )
    {
    FILE   *ouf;		/* output file descriptor		*/
    struct ITEM *curr;		/* current item being saved		*/
    char   *adapstables;	/* pointer to environment variable      */
    char   ofile[CMLEN];	/* output file name			*/
    char   obuf[CMLEN];		/* output buffer			*/
    char   tmp[CMLEN];		/* temporary string			*/


    /* Get the file name and open the file,
    don't fail on open error just return.
    --------------------------------------*/
    adapstables = getenv( "ADAPSTABLES" );
    if ( adapstables == NULL )
        {
	write_log( FALSE, "save_state", "environment",
	    "ADAPSTABLES not defined" );
	return;
	}
    sprintf( ofile, "%s/%s", adapstables, ADAPSMONITORSTATE );
    ouf = fopen( ofile, "w" );
    if ( !ouf )
        {
	sprintf( tmp, "Unable to open $ADAPSTABLES/%s", ADAPSMONITORSTATE );
	write_log( FALSE, "save_state", "fopen", tmp );
	return;
	}

    /* Save the state by writing the appropriate
    values for each item in the list to the file
    --------------------------------------------*/
    curr = item;
    while ( curr != NULL )
        {
	/* start the string with the name and machine */
	sprintf( obuf, "%s %s ", curr->itemname, curr->machine );

	/* convert the enumerated type to a string and cat to buffer */
	switch ( curr->itemtype )
	    {
	    case BATCH :
	        strcat( obuf, "batch " );
		break;
	    case PROCESS :
	        strcat( obuf, "process " );
		break;
	    case VAXPROCESS :
	        strcat( obuf, "vaxprocess " );
		break;
	    case SYSTEM :
	        strcat( obuf, "system " );
		break;
	    case DATAFILE :
	        strcat( obuf, "datafile " );
		break;
	    }
	
	/* convert the intervals to minute strings and cat to buffer */
	sprintf( tmp, "%d %d ", curr->mon_interval / 60,
	    curr->rwast_interval / 60 );
	strcat( obuf, tmp );

	/* convert current state to string and cat to buffer */
	switch ( curr->monitor_state )
	    {
	    case ENABLED :
	        strcat( obuf, "enabled " );
		break;
	    case DISABLED :
	        strcat( obuf, "disabled " );
		break;
	    default :
	        strcat( obuf, "enabled " );
	    }

        /* finally convert number of batch jobs to string and
	cat it and the user id and log file to buffer.  Terminate
	the buffer with a new line since fputs() doesn't do it.
	---------------------------------------------------------*/
        sprintf( tmp, "%d %s %s\n", curr->num_batch, curr->userid,
	    curr->log_name );
	strcat( obuf, tmp );

	/* write the buffer */
	if ( fputs( obuf, ouf ) == EOF )
	    {
	    write_log( FALSE, "save_state", "fputs", "Error saving state" );
	    break;
	    }

	/* go to the next item */
	curr = curr->next;
	}  /* END while */


    if ( fclose( ouf ) != 0 )
	{
        sprintf( tmp, "Error closing file: %s", ofile );
        post_dialog( toplevel, tmp, "Acknowledge" );
	}

    write_log( FALSE, "save_state", "save", "Change of state saved" );
    }  /* END save_state */
