/***************************************************************************
FUNCTION:	get_item_type
  		get_monitor_state
		scroll_window_child
		get_rowcol_widget
		store_rowcol_widget
                get_application_shell
                get_toplevel_shell
		get_top_shell
		get_fileselectwindow


PURPOSE:	Miscellaneous support routines

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
***************************************************************************/
#include <stdio.h>
#include "adapsmonitor.h"


/**************************************************************************
FUNCTION:	get_item_type

PURPOSE:	Determine the enumerated item type of a string value.

INPUT:          string value of item

OUTPUT:         None

RETURNS:	ItemType: enumerated item type

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
**************************************************************************/
ItemType get_item_type
    (
    char *str			/* IN - string representation of type	*/
    )
    {
    int     i;			/* loop control variable		*/


    /* Valid string values for the item type field.  The string values
    defined here must be in the same sequence as the enumerated values
    defined in ItemType in adapsmonitor.h.  The 'undefined' value is
    here only as a place holder and is not valid.
    ------------------------------------------------------------------*/
    static char valid_types[NTYPES][12] = {{ "undefined" },{ "batch" },
                                       { "process" },{ "vaxprocess" },
                                       { "system" },{ "datafile" }};

    
    /* Match the string to one of the defined valid types
    Note that 'undefined' is not a valid type that's why
    the loop index begins at 1.
    -----------------------------------------------------*/
    for ( i = 1; i < NTYPES; i++ )
        if ( strcmp( str, valid_types[i] ) == 0 )
	    return (ItemType)i;
    
    /* nothing matched
    ------------------*/
    return UNDEFINED;
    }  /* END get_item_type */


/**************************************************************************
FUNCTION:	get_monitor_state

PURPOSE:	Determine the enumerated monitor state of a string value.

INPUT:          string value of monitor state

OUTPUT:         None

RETURNS:	MonState: enumerated value of string

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
**************************************************************************/
MonState get_monitor_state
    (
    char  *str			/* IN - string representation of state	*/
    )
    {
    int     i;			/* loop control variable		*/

    /* Valid string values for the monitor state field.  The string values
    defined here must be in the same sequence as the enumerated values
    defined in MonState in adapsmonitor.h.  The 'undefined' value is
    here only as a place holder and is not valid.
    ------------------------------------------------------------------*/
    static char valid_types[NSTATES][14] = {{ "uninitialized" },{ "enabled" },
                                       { "disabled" }};
    
    /* Match the string to one of the defined valid types
    Note that 'uninitialized' is not a valid type which
    is why the loop starts at 1.
    -----------------------------------------------------*/
    for ( i = 1; i < NSTATES; i++ )
        if ( strcmp( str, valid_types[i] ) == 0 )
	    return (MonState)i;
    
    /* Nothing matched
    ------------------*/
    return UNINITIALIZED;
    }  /* END get_monitor_state */


/************************************************************************
FUNCTION:	scroll_window_child

PURPOSE:	Return the value of a widget who is a child of the
		scroll window who is a child of the modify window.

INPUTS:		the name of the child widget

OUTPUTS:	none

RETURNS:	Widget, the grandchild widget

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*************************************************************************/
Widget scroll_window_child
    (
    Widget w,			/* IN - calling widget value		*/
    char widget_name[]          /* IN - name of widget to get		*/
    )
    {
    char   wstring[CMLEN];	/* widget string for search		*/

    sprintf( wstring, "%s.%s.%s.%s", WnMODIFYWIND, WnMODIFY, WnSCROLLFORM,
        widget_name );

    return XtNameToWidget( get_toplevel_shell( w ), wstring );
    }  /* END scroll_window_child */


static Widget current_rowcol = NULL;/* the parent of the scrolled window   */
/************************************************************************
FUNCTION:	store_rowcol_widget

PURPOSE:	Store the current value of the rowcol widget.

INPUTS:		widget value to store

OUTPUTS:	none

RETURNS:	none

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
************************************************************************/
void store_rowcol_widget
    (
    Widget rowcol		/* IN - widget value to store or NULL	*/
    )
    {
    current_rowcol = rowcol;
    }  /* END store_rowcol_widget */


/************************************************************************
FUNCTION:	get_rowcol_widget

PURPOSE:	Get the current value of the rowcol widget.

INPUTS:		none

OUTPUTS:	none

RETURNS:	Widget, stored value of rowcol widget

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
************************************************************************/
Widget get_rowcol_widget
    (
    void
    )
    {
    return current_rowcol;
    }  /* END get_rowcol_widget */


/***************************************************************************
FUNCTION:	get_application_shell

PURPOSE:	Climb to the top of the widget tree and return the top
		level application shell

INPUTS:		calling widget data

OUTPUTS:	none

RETURNS:	Widget - top level application shell

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	08/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
Widget get_application_shell
    (
    Widget w			/* IN - calling widget value            */
    )
    {
    while( w && !XtIsApplicationShell( w ))
        w = XtParent( w );
    
    return w;
    }  /* END get_application_shell */



/***************************************************************************
FUNCTION:	get_toplevel_shell

PURPOSE:	Climb to the top of the widget tree and return the top
		level shell

INPUTS:		calling widget data

OUTPUTS:	none

RETURNS:	Widget - top level shell widget

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	08/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
Widget get_toplevel_shell
    (
    Widget w			/* IN - calling widget value            */
    )
    {
    while( w && !XtIsTopLevelShell( w ))
        w = XtParent( w );
    
    return w;
    }  /* END get_toplevel_shell */



/***************************************************************************
FUNCTION:	get_top_shell

PURPOSE:	Climb to the top of the widget tree and return the shell

INPUTS:		calling widget data

OUTPUTS:	none

RETURNS:	Widget, shell widget

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	08/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
Widget get_top_shell
    (
    Widget w			/* IN - calling widget value		*/
    )
    {
    while( w && !XtIsWMShell( w ))
        w = XtParent( w );
    
    return w;
    }  /* END get_top_shell */


/***************************************************************************
FUNCTION:	get_fileselectwindow

PURPOSE:	Return the widget value for the file select dialog window

INPUTS:		none

OUTPUTS:	none

RETURNS:	Widget - the file selection dialog widget

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	08/00	D.Lloyd		Initial development

PROJECT:	ADAPS
****************************************************************************/
Widget get_fileselectwindow
    (
    Widget w                    /* IN - calling widget value            */
    )
    {
    char   wstring[CMLEN];

    sprintf( wstring, "%s.%s", WnFILESELECTPOPUP, WnFILESELECT );

    return XtNameToWidget( get_toplevel_shell( w ), wstring );
    }  /* END get_fileselectwindow */
