/*****************************************************************************
FUNCTION:	update_displist

PURPOSE:	To update the display list on the main window

INPUT:          the list widget
		pointer to the ITEM structure list

OUTPUT:         None

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	D.Lloyd		Initial development

PROJECT:	ADAPS
******************************************************************************/
#include <Xm/List.h>
#include <Xm/Text.h>
#include "adapsmonitor.h"

void update_displist
    (
    Widget    list_w,		/* IN - the list widget			*/
    struct ITEM *list		/* IN - structure with items to list	*/
    )
    {
    XmString    str;		/* string to update the list widget	*/
    int         count;		/* number of entries in display list	*/
    struct ITEM *curr;		/* pointer to the current item		*/


    /* Go through the list checking for display strings that need to
    be redisplayed or added to the display list.  The structure member
    fieldflg[0] indicates an item that needs to be redisplayed.  A zero
    in the structure member item_index indicates the item is not in the
    display list yet.
    -------------------------------------------------------------------*/
    curr = list;
    do
        {
	if ( curr->item_index == 0 )
	/* add to display list */
	    {
	    /* get the number of entries in the display list */
	    XtVaGetValues( list_w, XmNitemCount, &count, NULL );

	    /* convert the new display string to an X string */
	    str = XmStringCreateLocalized( curr->display );

	    /* Add the string to the end of the list and store the index */
	    XmListAddItemUnselected( list_w, str, 0 );
	    curr->item_index = count + 1;
	    XmStringFree( str );
	    }
	else if ( curr->fieldflg[0] == REDISPLAY )
	    {
	    curr->fieldflg[0] = VALID;
	    /* convert the display string to an X string and replace it */
	    str = XmStringCreateLocalized( curr->display );
	    XmListReplaceItemsPosUnselected( list_w, &str, 1, curr->item_index);
	    XmStringFree( str );
	    }

	/* point to the next item in the list */
	curr = curr->next;
	}
    while ( curr != NULL );

    }  /* END update_displist */
