/****************************************************************************
NAME:		CKQUE

PURPOSE:	Search the output from the UNIX function "qstat" to
		determine is a function is running in batch.  If it
		isn't remove the stop file.  The script queuestat.csh
		is used as a brigde to the qstat function due to
		changes in the qstat function.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  2.0	 D. Hollaren   04/94	Original development
  2.1    T. Baltzer    01/95    Limit the search string to 10 characters
				to accomodate the variance in qstat output.
  2.2    D. Lloyd      10/01    Modifed to call the queuestat.csh script
                                as a bridge to qstat.  See CR 5595.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"

int main()
{
char	instop[CMLEN],		/* Input stop file name (parameter file)   */
	srchstr[CMLEN],		/* string to search the queue for (function) */
	command[CMLEN];		/* Command to execute			   */

long	std_out,		/* Standard output file                    */
	element = 0,		/* Input name element			   */
	access = IREAD,		/* Read access on the stop file		   */
	fatal = -1;		/* Fatal error code			   */

struct PARBLK vblock;		/* TAE parameter block			   */
struct VARIABLE *var;		/* Pointer to the variable within vblock   */
/*
#ifndef sg
    {
    long nonfatal = 1;
    c_errmsg("CKQUE only supported on SGI systems","ckque-notsgi",&nonfatal);
    c_errmsg("Fatal error encountered","ckque-fatal",&fatal);
    }
#endif
*/

/*
  Initialize LAS
*/
c_inlas(&vblock, &std_out);

/*
  Retrieve the input stop file name.  The stop file is a TAE parameter file
  that contains the stop file name and the function to be stopped.
*/
if (c_getnam(&vblock,"INSTOP",&element,instop,"stop",&access) != E_SUCC)
    c_errmsg("Fatal error encountered","ckque-fatal",&fatal);

/*
  Retrieve the search string.  The search string is the function name
  in the queue that is to be stopped.  When the search string is not found
  the function was stopped or never running.
*/
var = p_find(&vblock, "SRCHSTR");
strcpy(srchstr, SVAL(*var, 0));

/*
  Make sure that the search string is not more than 10 characters long,
  since some qstat outputs are limited to 10 characters in this field.
  10 characters should be enough to uniquely identify the process.
*/
srchstr[10] = '\0';

/*
  Build the system command to search for the specified search string (function)
*/
sprintf(command, "queuestat.csh %s > /dev/null", srchstr);

/*
  Search the batch queue status function output for the specified function
*/
if (system(command))
    {
    /*
      The specified function name wasn't found in the qstat.  The
      function was stopped or was never running.  Remove the stop
      file.
    */
    c_delfil(instop);
    }

/*
  Complete the function
*/
z_exit(1, "CKQUE completed successfully");

/*
  Put a return here to make the compiler happy.
*/
return(E_SUCC);
}

