#ifndef ANGINTERP_H
#define ANGINTERP_H

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "grid.h"

#define MAXLINES 100

#define NO_SCALE     0	/* Angle values are not scaled                        */
#define US_SCALE     1	/* Angle values are scaled for US composites          */
#define GLOBAL_SCALE 2	/* Angle values are scaled for Global composites      */


/* Structure used for storing scaling, offset quantities */
/*-------------------------------------------------------*/
struct SCOFF
   {
   double minval;               /* Minimum value of appropriate quantity */
   double maxval;               /* Maximum value of appropriate quantity */
   double scale;                /* Scale value to apply                  */
   double offset;               /* Offset value to apply                 */
   double unscale;              /* Value to apply to unscale             */
   };

/* Prototypes for all functions */
/*------------------------------*/

int c_interp
(
    double tl,     	/* top left post of grid			*/
    double tr,     	/* top right post of grid			*/
    double bl,     	/* bottom left post of grid			*/
    double br,     	/* bottom right post of grid			*/
    long dx,     	/* number of columns to interpolate across	*/
    long dy,     	/* number of rows to interpolate across		*/
    float *outbuf 	/* buffer of interpolated values		*/
);

int getpar
(
    struct PARBLK *vblock,	/* IN  - TAE variable block                   */
    char hosin[][CMLEN],	/* OUT - Input host file name                 */
    long window[][4],		/* OUT - Specified window of the image        */
    long bands[][MAXBND+1],	/* OUT - Specified bands of the image         */
    long *totbnd,		/* OUT - Total number of bands in image       */
    struct DDR *ddr,		/* OUT - Input image ddr                      */
    char hosout[][CMLEN],	/* OUT - Output host file name                */
    char taeout[][CMLEN],	/* OUT - Output tae file name                 */
    char *angle_gridfile,	/* OUT - Angles grid file name                */
    struct GRID *angle_grid,	/* OUT - Angles grid                          */
    long *odtype,		/* OUT - Input image data type                */
    long *scaling,		/* OUT - Scaling method requested by the user */
    long *msgtim,		/* OUT - Processing message time              */
    char *minmax		/* OUT - Minmax calculating flag              */
);

int nearest_neighbor
(
    double top_left,	/* IN  - Top left corner of a grid cell               */
    double top_right,	/* IN  - Top right corner of a grid cell              */
    double bottom_left,	/* IN  - Bottom left corner of a grid cell            */
    double bottom_right,/* IN  - Bottom right corner of a grid cell           */
    long ns,		/* IN  - Number of samples in the grid cell           */
    long nl,		/* IN  - Number of lines in the grid cell             */
    float *outbuf	/* OUT - Output buffer of nearest neighbor values     */
);

int process
(
    struct GRID *grid,	/* IN - Angles grid generated by RECTIFY              */
    long window[],	/* IN - Window for the input image                    */
    long bands[],	/* IN - Bands to be created for the output image      */
    long nbands,	/* IN - Number of the bands to be placed in output    */
    long dtype,		/* IN - Data type of the output image                 */
    long scaling,	/* IN - Scaling option specified by the user          */
    struct SCOFF scoff[N_ANGLES], /* Scale factors, offsets for scaling       */
    long msgtim		/* IN - Number of seconds between processing messages */
);

int set_scale_offset
(
    long dtype,		/* IN  - Data type of the output angles               */
    long scaling,	/* IN  - Scaling option specified by the user         */
    struct SCOFF scoff[] /* Structure containing angle scaling, offsets       */
);

int set_up_images
(
    char *hosout,	/* IN - output host file name                         */
    long window[],	/* IN - window of input image                         */
    long totbnd,	/* IN - number of bands in OUT                        */
    long odtype		/* IN - data type of OUT                              */
);

#endif
