/*******************************************************************************
NAME:    c_interp 

PURPOSE: Interpolate a buffer of data given a set of grid postings.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    T.Baltzer        01/93  Development
 

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"

int c_interp
(
    double tl,     	/* top left post of grid			*/
    double tr,     	/* top right post of grid			*/
    double bl,     	/* bottom left post of grid			*/
    double br,     	/* bottom right post of grid			*/
    long dx,     	/* number of columns to interpolate across	*/
    long dy,     	/* number of rows to interpolate across		*/
    float *outbuf 	/* buffer of interpolated values		*/
)
{
float 	coef[4];     	/* coefficients used in interpolation		*/
float 	*outptr;     	/* pointer to output buffer			*/
float	temp1,
	temp2;

long	row,
	col;

coef[0] = tl;
coef[1] = (tr - tl) / dx;
coef[2] = (bl - tl) / dy;
coef[3] = (br - tr - bl + tl) / (dx * dy);

/*
  Work with a temporary pointer so we don't destroy the one passed in
*/
outptr = outbuf;

/* basis of algorithm:
      output pixel = coef[0] + coef[1]x + coef[2]y + coef[3]xy
   or rearranging for speed:
      output pixel = coef[0] + coef[2]y + (x * (coef[1] + coef[3]y))
   where
      x = column number
      y = row number
*/
for (row = 0; row < dy; row++)
   {
   temp1 = coef[0] + (coef[2] * (float)row);
   temp2 = coef[1] + (coef[3] * (float)row);
   for (col = 0; col < dx; col++)
      {
      *outptr = temp1 + ((float)col * temp2);
      outptr++;
      }
   }

return(E_SUCC);
}
