/*******************************************************************************
NAME:		nearest_neighbor 

PURPOSE:	Using four corner points determine the nearest neighbor value
		for each pixel in a rectangular buffer.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    03/94  J.Fenno          Development
 

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"


int nearest_neighbor
(
    double top_left,	/* IN  - Top left corner of a grid cell               */
    double top_right,	/* IN  - Top right corner of a grid cell              */
    double bottom_left,	/* IN  - Bottom left corner of a grid cell            */
    double bottom_right,/* IN  - Bottom right corner of a grid cell           */
    long ns,		/* IN  - Number of samples in the grid cell           */
    long nl,		/* IN  - Number of lines in the grid cell             */
    float *outbuf	/* OUT - Output buffer of nearest neighbor values     */
)
{
long line;		/* Loop control                                       */
long midline;		/* Number of lines in the top half of the grid cell   */
long midsample;		/* Number of samples in the left half of the grid cell*/
long sample;		/* Loop control                                       */


/* Compute the number of lines and samples that split the cell into four areas
------------------------------------------------------------------------------*/
midline = nl / 2;
midsample = ns / 2;


/* Determine the nearest neighbor for the top half of the grid cell
-------------------------------------------------------------------*/
for (line = 1; line <= midline; line++)
    {
    /* Those values nearest the top left
    ------------------------------------*/
    for (sample = 1; sample <= midsample; sample++)
	*(outbuf++) = top_left;

    /* Those values nearest the top right
    -------------------------------------*/
    for (; sample <= ns; sample++)
	*(outbuf++) = top_right;
    }


/* Determine the nearest neighbor for the top half of the grid cell
-------------------------------------------------------------------*/
for (; line <= nl; line++)
    {
    /* Those values nearest the bottom left
    ---------------------------------------*/
    for (sample = 1; sample <= midsample; sample++)
	*(outbuf++) = bottom_left;

    /* Those values nearest the bottom right
    ----------------------------------------*/
    for (; sample <= ns; sample++)
	*(outbuf++) = bottom_right;
    }


return(E_SUCC);
}
