/*******************************************************************************
FUNCTION:	set_scale_offset

PURPOSE:	Depending upon the scaling option and the output data type,
		set the scaling and offset values.


*******************************************************************************/
#include "las.h"
#include "adapssup.h"
#include "anginterp.h"


int set_scale_offset
(
    long dtype,		/* IN  - Data type of the output angles               */
    long scaling,	/* IN  - Scaling option specified by the user         */
    struct SCOFF scoff[] /* Structure containing angle scaling, offsets       */
)
{
long nonfatal = 1;	/* Nonfatal error flag                                */
long combined_type;	/* Scaling option, data type combination	      */
long status;		/* Return error status				      */


/* Set the scaling and offset values according to the desired scaling option
----------------------------------------------------------------------------*/
switch (scaling)
    {
    case US_SCALE:
	switch (dtype)
	    {
	    case EBYTE:
		combined_type = US_BYTE;
		break;
	    case EWORD:
		combined_type = US_I2;
		break;
	    case ELONG:
		combined_type = US_I4;
		break;
	    case EREAL:
		combined_type = US_REAL;
		break;
	    
	    default:
		c_errmsg("Invalid data type for US scaling",
		    "anginterp-dtype", &nonfatal);
		return(E_FAIL);
	    }
	break;

    case GLOBAL_SCALE:
	switch (dtype)
	    {
	    case EBYTE:
		combined_type = AVHRR_BYTE;
		break;

	    /* For global scaling and odtype=i*2 always use the 10bit scaling
	       factor and offset.  It was decided that the user should not have
	       the choice of using the 16bit values.
	    ------------------------------------------------------------------*/
	    case EWORD:
		combined_type = AVHRR_10BIT;
		break;
	    case ELONG:
		combined_type = AVHRR_I4;
		break;
	    case EREAL:
		combined_type = AVHRR_REAL;
		break;
	    
	    default:
		c_errmsg("Invalid data type for GLOBAL scaling",
		    "anginterp-dtype", &nonfatal);
		return(E_FAIL);
	    }
	break;

    default:
	combined_type = NO_SCALING;
	break;

    }

status = c_getscale(combined_type,"satzenith",&scoff[SATZEN].minval,
		    &scoff[SATZEN].maxval,&scoff[SATZEN].scale,
		    &scoff[SATZEN].unscale,&scoff[SATZEN].offset);
if (status != E_SUCC)
    return(E_FAIL);

status = c_getscale(combined_type,"solzenith",&scoff[SOLZEN].minval,
		    &scoff[SOLZEN].maxval,&scoff[SOLZEN].scale,
		    &scoff[SOLZEN].unscale,&scoff[SOLZEN].offset);
if (status != E_SUCC)
    return(E_FAIL);

status = c_getscale(combined_type,"relazimuth",&scoff[RELAZ].minval,
		    &scoff[RELAZ].maxval,&scoff[RELAZ].scale,
		    &scoff[RELAZ].unscale,&scoff[RELAZ].offset);
if (status != E_SUCC)
    return(E_FAIL);


return(E_SUCC);
}
