#ifndef ANSISTACK_H
#define ANSISTACK_H


#define SYS_SUCC      0         /* UNIX successful exit status                */


#ifndef SEEK_SET
#define SEEK_SET 0
#endif
#ifndef SEEK_CUR
#define SEEK_CUR 1
#endif
#ifndef SEEK_END
#define SEEK_END 2
#endif


#define NASSOC                 9	/* Max number of associated files     */
#define GROUPSIZE     (NASSOC+1)	/* Max files in a file group          */
#define SMALLSTR              30	/* Length of a small string           */
#define TPFIELDS              11	/* Number of fields in a tape label   */
#define TPNAMELEN             17	/* Max length of a file name on tape  */

#define DISKNAME               0	/* Files are named with disknames[]   */
#define TAPENAME               1	/* Files are named with tapenames[]   */

#define TMARK_9TRACK       0.025	/* Mbytes used for 9-track tape marks */
#define TMARK_3480         0.038	/* Mbytes used for 3480 tape marks    */
#define TMARK_3490         0.068	/* Mbytes used for 3490 tape marks    */
#define TMARK_4MM          0.400	/* Mbytes used for 4mm tape marks     */
#define TMARK_8LO          0.256	/* Mbytes used for 8LO tape marks     */
#define TMARK_8HI          0.256	/* Mbytes used for 8HI tape marks     */

/* The TMARK values below are the same as 8LO and 8HI because
   of insuffiecient data to accurately calculate the values
-------------------------------------------------------------*/
#define TMARK_DLT4000      0.256        /* Mbytes used for DLT4000 tape marks */
#define TMARK_DLT7000      0.256        /* Mbytes used for DLT4000 tape marks */
#define TMARK_D3           0.256        /* Mbytes used for DLT4000 tape marks */


struct FILEGROUP
    {
    char dir[CMLEN];			/* Directory where files are located  */
    char disknames[GROUPSIZE][CMLEN];	/* File names as found on disk        */
    char tapenames[GROUPSIZE][CMLEN];	/* File names as found on tape        */

    long nfiles;			/* Number of files in the group       */
    long mbytes;			/* Total size of all files in group   */

    short nameflag;			/* Indicates which name files are in  */
    short rename;			/* Flag to rename before stacking     */
    };

/* Prototypes */

long compare_tape_names
(
    void *ptr1,         /* IN - Ptr to a FILEGROUP structure                  */
    void *ptr2          /* IN - Ptr to a FILEGROUP structure                  */
);    

int create_group
(
    char *filename,             /* IN  - Host file name of input file         */
    char aftype[][CMLEN],       /* IN  - List of file extensions              */
    long nassoc,                /* IN  - Number of file extensions            */
    char *command,              /* IN  - Command being run                    */
    char *ttype,                /* IN  - Type of tape being stacked to        */
    struct FILEGROUP **ptr      /* OUT - Ptr to a new malloc'd structure      */
);

int move_queue
(
    long q,             /* IN - Queue containing the file groups              */
    char *dstdir,       /* IN - Destination directory                         */
    char *errdir        /* IN - Error directory                               */
);  

int print_group
(
    void *ptr           /* IN - Ptr to a FILEGROUP structure                  */
);

int tape_names
(
    struct FILEGROUP *ptr,      /* IN/OUT - Ptr to a FILEGROUP structure      */
    char *command               /* IN     - Command being run                 */
);

#endif

