/*******************************************************************************
FUNCTION:	ansistack_drv

PURPOSE:	Allocate the specified tape drive and ensure it is an ANSI
		labeled tape.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Stolen from AUTOSTACK


PROJECT: ADAPS
*******************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "las.h"
#include "tapeio.h"
#include "util.h"
#include "ansistack_drv.h"

#define BUFSIZE    1000	/* Length of buffer used for reading from tape        */
#define NTRIES        3	/* Number of times to attempt to open a tape drive    */
#define RECSIZE      80	/* Number of bytes in an ANSI record                  */
#define SLEEP_TIME   30	/* Number of seconds to sleep between mount attempts  */
#define VOLLEN        6	/* Length of the volume ID (ie. tape ID)              */



main()
{
char density[SMALLSTR];		/* Density of the assigned tape drive         */
char device[CMLEN];		/* Device name of the assigned tape drive     */
char drive[SMALLSTR];		/* Tape drive where the tape is mounted       */
char *ptr[1];			/* Ptr to a string for calling q_string()     */
char str[CMLEN];		/* Temporary string                           */
char tapeid[SMALLSTR];		/* ID of the tape that is mounted             */
char ttype[SMALLSTR];		/* Desired tape type (8hi,8lo,4mm,3480,3490..)*/
char tmp_ttype[SMALLSTR];	/* Mounted tape type (8hi,8lo,4mm,3480,3490..)*/

long access   = TREAD;		/* Type of access to the tape drive           */
long attempts;			/* Number of attempts at mounting a tape      */
long eofs     = 0;		/* Write zero EOF marks when closing the tape */
long errmode  = HANDLE;		/* Error handling mode for tapeio functions   */
long i;				/* For call to complt                         */
long nbytes;			/* Number of bytes read using c_tpread        */
long nonfatal =  1;		/* Nonfatal error flag                        */
long rewind   = REWIND;		/* Rewind option for tapeio functions         */
long rewoffl  = REWOFFL;	/* Rewind and take tape offline               */
long size;			/* Size of the ANSI buffer                    */
long skip     = 0;		/* # of files to skip when a tape is opened   */
long status;			/* Function return status                     */
long std_out;			/* Standard output file                       */
long tape_unit;			/* Tape unit as returned by tapeio            */

short appendflg;		/* Flag indicating if drive allows appending  */
short drive_supplied;		/* Flag indicating if user supplied tape drive*/
short success;			/* Flag indicating if correct tape was mounted*/

unsigned char buffer[BUFSIZE];	/* Buffer to receive ANSI records             */

struct PARBLK vblock;		/* TAE parameter block                        */
struct VARIABLE *parm;		/* Ptr to a TAE variable                      */



/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Retrieve the tape type
-------------------------*/
parm = p_find(&vblock, "TTYPE");
strcpy(ttype, SVAL(*parm, 0));
i = (long) strlen(ttype);
c_up2low(ttype, &i);


/* Determine the desired density
--------------------------------*/
if (!strcmp(ttype,"8hi"))
    strcpy(density,"HI");

else if (!strcmp(ttype,"8lo"))
    strcpy(density,"LO");

else
    strcpy(density,"0");


/* Retrieve the tape drive
--------------------------*/
parm = p_find(&vblock, "DRIVE");
if (parm->v_count == 0)
    {
    drive_supplied = FALSE;
    strcpy(drive, "");
    }
else
    {
    drive_supplied = TRUE;
    strcpy(drive, SVAL(*parm, 0));
    }


/* Mount the correct type of tape
---------------------------------*/
if (drive_supplied)
    {
    /* The user specified the desired tape drive so it is assumed to be
       already mounted and the tape drive just needs to be reopened.
    -------------------------------------------------------------------*/
    status = c_tpreop(&tape_unit,&skip,&errmode,&rewind,density,&access,drive);
    attempts = 1;
    while ((status != TAPE_SUCC) && (attempts++ < NTRIES))
	{
	c_errmsg("Error opening tape.  Retrying...", " ", &nonfatal);
	sleep(SLEEP_TIME);
	status = c_tpreop(&tape_unit,&skip,&errmode,&rewind,density,&access,drive);
	}


    /* If a tape was not successfully mounted, return
    -------------------------------------------------*/
    if (status != TAPE_SUCC)
	{
	c_errmsg("Error mounting tape", "ansistack-mount", &nonfatal);
	z_exit(-1, "ANSISTACK_DRV failed");
	}


    /* Determine the type of tapes used in the drive
    ------------------------------------------------*/
    status = drive_info(drive, density, tmp_ttype, device, &appendflg);
    if (status != E_SUCC)
	{
	sprintf("Error determining tape type for drive %s", drive);
	c_errmsg(str, "ansistack-drive", &nonfatal);
	c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
	z_exit(-1, "ANSISTACK_DRV failed");
	}
    

    /* Ensure the tape types match
    ------------------------------*/
    if (strcmp(ttype, tmp_ttype) != 0)
	{
	sprintf(str, "Drive %s uses %s tapes not %s", drive, tmp_ttype, ttype);
	c_errmsg(str, "ansistack-ttype", &nonfatal);
	c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
	z_exit(-1, "ANSISTACK_DRV failed");
	}


    /* Ensure the tape drive allows files to be appended
    ----------------------------------------------------*/
    if (appendflg == FALSE)
	{
	sprintf(str, "Drive %s does not allow file appending", drive);
	c_errmsg(str, "ansistack-append", &nonfatal);
	c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
	z_exit(-1, "ANSISTACK_DRV failed");
	}
    }
else
    {
    /* Ask the operators for a tape until the correct type is mounted
    -----------------------------------------------------------------*/
    attempts = 0;
    success = FALSE;
    while (!success)
	{
	attempts++;


	/* Have the operator mount a tape.  Note that the tape
	   drive is returned in the tape mounting comment.
	------------------------------------------------------*/
	sprintf(drive, "Please mount an ANSI initialized %s tape", ttype);
	status = c_tpopen(&tape_unit, &skip, &errmode,
	    &rewind, density, &access, drive, tapeid);
	if (status != TAPE_SUCC)
	    {
	    c_errmsg("Error mounting tape", "ansistack-mount", &nonfatal);
	    z_exit(-1, "ANSISTACK_DRV failed");
	    }


	/* Determine the type of tapes used in the assigned drive
	---------------------------------------------------------*/
	status = drive_info(drive, density, tmp_ttype, device, &appendflg);
	if (status != E_SUCC)
	    {
	    sprintf("Error determining tape type for drive %s", drive);
	    c_errmsg(str, "ansistack-drive", &nonfatal);
	    c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
	    z_exit(-1, "ANSISTACK_DRV failed");
	    }
	

	/* Ensure the tape types match
	------------------------------*/
	if (strcmp(ttype, tmp_ttype) == 0)
	    success = TRUE;
	else
	    {
	    sprintf(str, "Operator mounted a %s tape", tmp_ttype);
	    c_errmsg(str, " ", &nonfatal);
	    c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
	    if (attempts > NTRIES)
		{
		sprintf(str, "Operator did not mount a %s tape in %d attempts",
		    ttype, attempts);
		c_errmsg(str, " ", &nonfatal);
		c_errmsg("Error mounting tape", "ansistack-mount", &nonfatal);
		z_exit(-1, "ANSISTACK_DRV failed");
		}
	    }
	}


    /* Ensure the tape drive allows files to be appended
    ----------------------------------------------------*/
    if (appendflg == FALSE)
	{
	sprintf(str, "Drive %s does not allow file appending", drive);
	c_errmsg(str, "ansistack-append", &nonfatal);
	c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
	z_exit(-1, "ANSISTACK_DRV failed");
	}
    }


/* When working on a VMS system we do not want a colon in the drive name
   (ie. msa0:) because we will be using the drive as a file name.
------------------------------------------------------------------------*/
ptr[0] = strchr(drive, ':');
if (ptr[0] != NULL)
    *ptr[0] = '\0';


/* Place the tape drive into the TAE parameter block
----------------------------------------------------*/
ptr[0] = drive;
if (q_string(&vblock, "DRIVE", 1, (const char **)ptr, P_UPDATE) != SUCCESS)
    {
    c_errmsg("Error sending parameter to TAE", "ansistack-tae", &nonfatal);
    c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
    z_exit(-1, "ANSISTACK_DRV failed");
    }


/* The tape has been successfully mounted.  Ensure the first record on the
   tape is RECSIZE bytes long.  This is will indicate that the tape is most
   likely an ANSI labeled tape.  Note that we must ask for more bytes than
   we are expecting.  If we only ask for as many bytes as we are expecting
   and the first record is larger, then c_tpread() will only return the
   number of bytes that we asked for.
---------------------------------------------------------------------------*/
size = BUFSIZE;
nbytes = c_tpread(&tape_unit, buffer, &size, &errmode);
if (nbytes != RECSIZE)
    {
    c_errmsg("Mounted tape is not ANSI labeled", "ansistack-ansi", &nonfatal);
    c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
    z_exit(-1, "ANSISTACK_DRV failed");
    }


/* Close the tape and rewind it
-------------------------------*/
status = c_tpclos(&tape_unit, &errmode, &rewind, &eofs);
if (status != TAPE_SUCC)
    {
    c_errmsg("Error rewinding tape", "ansistack-rewind", &nonfatal);
    c_tpclos(&tape_unit, &errmode, &rewoffl, &eofs);
    z_exit(-1, "ANSISTACK_DRV failed");
    }


/* Copy the tape ID from the buffer.  It starts in position
   four of the buffer and consists of nonblank characters.
-----------------------------------------------------------*/
strncpy(tapeid, (char *)(buffer + 4), VOLLEN);
i = 0;
while ((i < VOLLEN) && !isspace(tapeid[i]))
    i++;
tapeid[i] = '\0';


/* Place the tape ID in the TAE parameter block
-----------------------------------------------*/
ptr[0] = tapeid;
if (q_string(&vblock, "TAPEID", 1, (const char **)ptr, P_UPDATE) != SUCCESS)
    {
    c_errmsg("Error sending parameter to TAE", "ansistack-tae", &nonfatal);
    z_exit(-1, "ANSISTACK_DRV failed");
    }


/* Return the parameter block to TAE
------------------------------------*/
if (q_out(&vblock) != SUCCESS)
    {
    c_errmsg("Error sending parameter to TAE", "ansistack-tae", &nonfatal);
    z_exit(-1, "ANSISTACK_DRV failed");
    }


/* Issue a successful completion message
----------------------------------------*/
z_exit(1, "ANSISTACK_DRV completed successfully");


/* Put this return here to make the compiler happy
--------------------------------------------------*/
return E_SUCC;
}
