/*******************************************************************************
FUNCTION:	check_tape_names

PURPOSE:	Ensure the tape name for each file in a group does not appear
		in a queue of tape names for files that have already been
		stacked to the tape.  The comparison must only use the first
		TPNAMELEN characters since this is all that is allowed for
		file names on ANSI labeled tapes.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "queue.h"
#include "ansistack.h"



int check_tape_names
    (
    struct FILEGROUP **group_ptr,	/* IN - Ptr to a file group           */
    long q 				/* IN - Queue of tape names           */
    )

{
char *tpname;		/* Ptr to a string containing a tape name             */

long i;			/* Loop control                                       */


c_qfirst(q, (void **)&tpname);
while (tpname != NULL)
    {
    /* Compare this queue element with each tape name in the group
    --------------------------------------------------------------*/
    for (i = 0; i < (*group_ptr)->nfiles; i++)
	{
	if (strncmp((*group_ptr)->tapenames[i], tpname, TPNAMELEN) == 0)
	    {
	    free(*group_ptr);
	    *group_ptr = NULL;
	    return(E_SUCC);
	    }
	}


    /* Get a pointer to the next tape name in the queue
    ---------------------------------------------------*/
    c_qnext(q, (void **)&tpname);
    }


return(E_SUCC);
}
