/*******************************************************************************
FUNCTION:	drive_info

PURPOSE:	Return the type of tape, the density, and the device name for a
		given tape drive.  8MM tape types should be changed to 8HI and
		8LO depending upon the density.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    07/92  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "util.h"
#include "ansistack_drv.h"

/* tprdconf prototype */
int tprdconf
(
    struct TAPEDESC *drive,
    char *tapenam,
    char *density,
    long inout
);


int drive_info
(
    char drive[],	/* IN  - Tape drive name                              */
    char density[],	/* IN/OUT - Density of the tape drive                 */
    char type[],	/* OUT - Type of tape (4mm,8hi,8lo,3480,3490,9-track) */
    char device[],	/* OUT - Physical device name of the drive            */
    short *appendflg 	/* OUT - Does the drive allow appending?              */
)

{
long length;			/* Length of a string                         */

struct TAPEDESC tape_drive;	/* Tape descriptor record for tprdconf()      */


/* Call the LAS support tprdconf() to read the configuration file.
------------------------------------------------------------------*/
if(tprdconf(&tape_drive, drive, density, TREAD) != TAPE_SUCC)
    return(E_FAIL);

strcpy(density, tape_drive.density);
length = (long) strlen(density);
c_up2low(density, &length);

strcpy(type, tape_drive.dtype);
length = (long) strlen(type);
c_up2low(type, &length);

strcpy(device, tape_drive.name);


/* Change the 8mm type to 8hi or 8lo depending upon the density
---------------------------------------------------------------*/
if (strcmp(type, "8mm") == 0)
    {
    if (strcmp(density, "hi") == 0)
	strcpy(type, "8hi");
    else
	strcpy(type, "8lo");
    }


/* Determine if the tape drive allows file to be appended to the end of tape
----------------------------------------------------------------------------*/
if (tape_drive.flags & NOAPP)
    *appendflg = FALSE;
else
    *appendflg = TRUE;

return(E_SUCC);
}
