/*******************************************************************************
FUNCTION:	move_queue

PURPOSE:	Move all of the file groups stored in a queue from their
		current directory to the specified directory.  If an error is
		encountered when moving one of the file groups, move the file
		group with the problem to the error directory and leave all of
		the other file groups in the directory where they where before
		the call to this function.

		Return E_SUCC if successful, E_FAIL if not.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "queue.h"
#include "ansistack_wrt.h"


int move_queue
(
    long q,		/* IN - Queue containing the file groups              */
    char *dstdir,	/* IN - Destination directory                         */
    char *errdir 	/* IN - Error directory                               */
)

{
char srcdir[CMLEN];	/* Directory where the file groups where located      */

long i;			/* Loop control                                       */
long j;			/* Loop control                                       */

struct FILEGROUP *ptr;	/* Ptr to a file group                                */


/* If the queue is empty then just return
-----------------------------------------*/
if (c_qsize(q) < 1)
    return(E_SUCC);


/* Get the first file group in the queue
----------------------------------------*/
i = 0;
c_qfirst(q, (void **)&ptr);


/* Remember the directory where the first file group was located.  If an error
   is encountered then all of the file groups that were moved before the error
   will be moved back to this directory.
------------------------------------------------------------------------------*/
strcpy(srcdir, ptr->dir);


/* Move all of the file groups
------------------------------*/
do  {
    i++;
    if (move_group(ptr, dstdir) == E_SUCC)
	c_qnext(q, (void **)&ptr);
    else
	{
	/* Move the offending file group to the error
	   directory and remove it from the queue
	---------------------------------------------*/
	move_group(ptr, errdir);
	c_qdelete(q, i);


	/* Move all of the file groups prior to the error
	-------------------------------------------------*/
	c_qfirst(q, (void **)&ptr);
	for (j = 1; j < i; j++)
	    {
	    move_group(ptr, srcdir);
	    c_qnext(q, (void **)&ptr);
	    }

	/* Return an error flag
	-----------------------*/
	return(E_FAIL);
	}
    } while (ptr != NULL);

return(E_SUCC);
}
