/*******************************************************************************
FUNCTION:	read_tape_names

PURPOSE:	Read the tape names of all the files that have already been
		stacked to tape and place them in a queue.  This will allow
		ansistack_get() to ensure that files with the same tape name
		will not be stacked to high capacity tapes (4mm and 8mm).


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "queue.h"
#include "ansistack.h"


int read_tape_names
(
    char *infile,	/* IN  - File containing file names copied to tape    */
    long q 		/* OUT - Queue containing file names copied to tape   */
)

{
char *ptr;		/* Ptr to a string                                    */
char str[CMLEN];	/* Temporary working string                           */

unsigned int len;	/* Length of a string                                 */

long access;		/* File access flag                                   */
long nonfatal = 1;	/* Nonfatal error flag                                */

FILE *fp;		/* File pointer to infile                             */


/* Determine if the file exists
-------------------------------*/
access = 0;
if (c_lsstat(infile, &access) == E_FAIL)
    return(E_SUCC);


/* Open the file for reading
----------------------------*/
fp = fopen(infile, "r");
if (fp == NULL)
    {
    c_errmsg("Error opening temporary file", "ansistack-open", &nonfatal);
    z_exit(-1, "ANSISTACK_GET failed");
    }


/* Read each of the files that have been stacked to tape.  These names will
   be the names of each file as it appears on the ANSI labeled tape.
---------------------------------------------------------------------------*/
while (fscanf(fp, "%s", str) == 1)
    {
    /* Allocate memory to store the file name read
    ----------------------------------------------*/
    len = strlen(str) + 1;
    ptr = (char *)malloc(len * sizeof(char));
    if (ptr == NULL)
	{
	c_errmsg("Error allocating memory", "ansistack-alloc", &nonfatal);
	c_qdestroy(q);
	fclose(fp);
	z_exit(-1, "ANSISTACK_GET failed");
	}
    

    /* Store the file name into the queue
    -------------------------------------*/
    strcpy(ptr, str);
    c_qinsert(q, ptr, (long (*)())strcmp);
    }


/* Close the input file
-----------------------*/
fclose(fp);
return(E_SUCC);
}
