/*******************************************************************************
FUNCTION:	rename_group

PURPOSE:	Rename all of the files in a group from their disk name to
		their tape name or vise versa.  If an error is encountered
		when renaming a file, the entire file group should be left
		as it was before the call to this function.

		Return E_SUCC if successful, E_FAIL if not.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "ansistack.h"


int rename_group
(
    struct FILEGROUP *ptr 	/* IN - Ptr to a file group structure         */
)

{
char newname[CMLEN];	/* Complete new name of a file                        */
char oldname[CMLEN];	/* Complete old name of a file                        */

long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long nonfatal = 1;	/* Nonfatal error flag                                */


/* If the disk names and tape names are the same
   for this file group, just return successfully.
------------------------------------------------*/
if (ptr->rename == FALSE)
    return(E_SUCC);


/* Rename each file
-------------------*/
for (i = 0; i < ptr->nfiles; i++)
    {
    /* Create the old and new file names
    ------------------------------------*/
    if (ptr->nameflag == DISKNAME)
	{
	sprintf(oldname, "%s%s", ptr->dir, ptr->disknames[i]);
	sprintf(newname, "%s%s", ptr->dir, ptr->tapenames[i]);
	}
    else
	{
	sprintf(oldname, "%s%s", ptr->dir, ptr->tapenames[i]);
	sprintf(newname, "%s%s", ptr->dir, ptr->disknames[i]);
	}


    /* Rename one file
    ------------------*/
    if (rename(oldname, newname))
	{
	c_errmsg("Error renaming file group", "ansistack-rename", &nonfatal);
	for (j = 0; j < i; j++)
	    {
	    if (ptr->nameflag == DISKNAME)
		{
		sprintf(oldname, "%s%s", ptr->dir, ptr->disknames[i]);
		sprintf(newname, "%s%s", ptr->dir, ptr->tapenames[i]);
		}
	    else
		{
		sprintf(oldname, "%s%s", ptr->dir, ptr->tapenames[i]);
		sprintf(newname, "%s%s", ptr->dir, ptr->disknames[i]);
		}
	    rename(newname, oldname);
	    }
	return(E_FAIL);
	}
    }


/* Reset the name flag
----------------------*/
if (ptr->nameflag == DISKNAME)
    ptr->nameflag = TAPENAME;
else
    ptr->nameflag = DISKNAME;


return(E_SUCC);
}
