/*******************************************************************************
FUNCTION:	rename_queue

PURPOSE:	Rename all of the file groups stored in a queue from their
		disk name to their tape name or vise versa.  If an error is
		encountered when renaming one of the file groups, move the
		file group with the problem to the error directory and leave
		all of the other file groups as they where before the call to
		this function.

		Return E_SUCC if successful, E_FAIL if not.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "queue.h"
#include "ansistack_wrt.h"


int rename_queue
(
    long q,		/* IN - Queue containing the file groups              */
    char *errdir,	/* IN - Error directory                               */
    long outform 	/* Output format				      */
)

{
long i;			/* Loop control                                       */
long j;			/* Loop control                                       */

struct FILEGROUP *ptr;	/* Ptr to a file group                                */


/* If the queue is empty then just return
-----------------------------------------*/
if (c_qsize(q) < 1)
    return(E_SUCC);


/* Rename all of the file groups
--------------------------------*/
i = 0;
c_qfirst(q, (void **)&ptr);
do  {
    i++;
    if (outform == 0)
	{
	if (rename_group(ptr) != E_SUCC)
	    {
	    /* Move the offending file group to the error
	       directory and remove it from the queue
	    ---------------------------------------------*/
	    move_group(ptr, errdir);
	    c_qdelete(q, i);


	    /* Re-rename all of the file groups prior to the error
	    ------------------------------------------------------*/
	    c_qfirst(q, (void **)&ptr);
	    for (j = 1; j < i; j++)
		{
		rename_group(ptr);
		c_qnext(q, (void **)&ptr);
		}

	    /* Return an error flag
	    -----------------------*/
	    return(E_FAIL);
	    }
	}
    c_qnext(q, (void **)&ptr);
    } while (ptr != NULL);

return(E_SUCC);
}
