/*******************************************************************************
FUNCTION:	tape_names

PURPOSE:	Given a pointer to a FILEGROUP structure and the type of
		command that is being run, determine what the name of each file
		in the group will be when written to an ANSI labeled tape.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "ansistack.h"


int tape_names
(
    struct FILEGROUP *ptr,	/* IN/OUT - Ptr to a FILEGROUP structure      */
    char *command 		/* IN     - Command being run                 */
)

{
char dir[CMLEN];	/* Directory portion of a file name                   */
char ext[CMLEN];	/* Extension portion of a file name                   */
char root[CMLEN];	/* Root portion of a file name                        */
char tp_extension;	/* Tape extension character                           */

long i;			/* Loop control                                       */
int len;		/* Length of a string                                 */
int maxlen;		/* Maximum length of a set of strings                 */



/* Determine the maximum length of the disk file names
------------------------------------------------------*/
maxlen = strlen(ptr->disknames[0]);
for (i = 1; i < ptr->nfiles; i++)
    {
    len = strlen(ptr->disknames[i]);
    if (len > maxlen)
	maxlen = len;
    }



/* If the maximum length of the disk file names is less than the maximum
   length of the allowable tape file names then there is no need to rename
   the files before copying them to tape.
--------------------------------------------------------------------------*/
if (maxlen <= TPNAMELEN)
    {
    ptr->rename = FALSE;
    for (i = 0; i < ptr->nfiles; i++)
	strcpy(ptr->tapenames[i], ptr->disknames[i]);
    return(E_SUCC);
    }


/* Create the tape names for each command
-----------------------------------------*/
if (strcmp(command, "avhrr") == 0)
    {
    ptr->rename = TRUE;
    for (i = 0; i < ptr->nfiles; i++)
	{
	c_splitname(ptr->disknames[i], dir, root, ext);
	if (strlen(root) < TPNAMELEN)
	    sprintf(ptr->tapenames[i], "%s.%s", root, ext);
	else
	    {
	    strcpy(ptr->tapenames[i], root);
	    ptr->tapenames[i][16] = '.';
	    }
	if (strcmp(ext, "ahdr") == 0)
	    ptr->tapenames[i][0] = 'h';
	else
	    ptr->tapenames[i][0] = 'i';
	}
    }
else if (strcmp(command, "mssx") == 0)
    {
    /* The standard renaming scheme for MSSX file groups depends upon the root
       file names being exactly TPNAMELEN characters long.  If the root names
       are less than or equal to this then there is no need to rename them
       before stacking to tape.  If the root names are longer than TPNAMELEN
       characters then use the standard renaming scheme.
    -------------------------------------------------------------------------*/
    c_splitname(ptr->disknames[0], dir, root, ext);


    /* If the maximum length of the root file names is less than the maximum
       length of the allowable tape file names then there is no need to rename
       the files before copying them to tape.
    --------------------------------------------------------------------------*/
    if (strlen(root) <= TPNAMELEN)
	{
	ptr->rename = FALSE;
	for (i = 0; i < ptr->nfiles; i++)
	    strcpy(ptr->tapenames[i], ptr->disknames[i]);
	}
    else
	{
	ptr->rename = TRUE;
	root[TPNAMELEN - 2] = '\0';
	tp_extension = 'a';
	for (i = 0; i < ptr->nfiles; i++)
	    sprintf(ptr->tapenames[i], "%s.%c", root, tp_extension++);
	}
    }
else
    {
    ptr->rename = TRUE;
    c_splitname(ptr->disknames[0], dir, root, ext);
    if (strlen(root) > (TPNAMELEN - 2))
	root[TPNAMELEN - 2] = '\0';
    tp_extension = 'a';
    for (i = 0; i < ptr->nfiles; i++)
	sprintf(ptr->tapenames[i], "%s.%c", root, tp_extension++);
    }

return(E_SUCC);
}
