/*******************************************************************************
FUNCTION:	tape_summary

PURPOSE:	Create and add one file group to a tape summary file.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "ansistack.h"


/* Define the beginning location for the number of groups and the
   total number of files stacked to the tape within the header.
-----------------------------------------------------------------*/
#define NGROUP_POS 171
#define NFILE_POS  196


int tape_summary
(
    struct FILEGROUP *ptr,	/* IN - Ptr to a FILEGROUP structure          */
    char *tapeid,		/* IN - Tape ID                               */
    char *ttype,		/* IN - Type of tape                          */
    char *drive,		/* IN - Tape drive                            */
    char *summary,		/* IN - File name of file to create           */
    long ngroups,		/* IN - Total number of file groups on tape   */
    long outform 		/* IN - Output format 0=Level 0, 1=Level 1B   */
)

{
char date_time[CMLEN];	/* String version of current date and time            */
char host[CMLEN];	/* Name of the current computer system                */
char user[CMLEN];	/* Name of the current user                           */
char tmptapename[CMLEN];/* Tape name of file				      */

long i;			/* Loop control                                       */
long nonfatal = 1;	/* Nonfatal error flag                                */
long now;		/* Current date and time encoded as a long            */

FILE *fp;		/* File pointer                                       */

struct tm *today;	/* Structure containing current date and time         */




/* If no file name is supplied then do not create the file
----------------------------------------------------------*/
if (strlen(summary) == 0)
    return(E_SUCC);


/* Open the tape summary file.  If this is the first file group stacked to the
   tape then create the file header, otherwise,  update the NGroups and NFiles
   fields within the header.
------------------------------------------------------------------------------*/
if (ngroups == 1)
    {
    /* Open the summary file and erase the prior contents
    -----------------------------------------------------*/
    fp = fopen(summary, "w");
    if (fp == NULL)
	{
	c_errmsg("Error opening summary file", "ansistack-open", &nonfatal);
	return(E_FAIL);
	}


    /* Determine the date and time
    ------------------------------*/
    now = time(NULL);
    today = localtime(&now);
    sprintf(date_time, "%02d/%02d/%04d %02d:%02d:%02d", today->tm_mon + 1,
	today->tm_mday, today->tm_year + 1900, today->tm_hour, today->tm_min,
	today->tm_sec % 60);
    

    /* Determine the host and user names
    ------------------------------------*/
    strcpy(user, cuserid(NULL));
    gethostname(host, CMLEN);


    /* Create the file header.  Note that each of the header lines must
       contain 80 characters and a newline character.  This will allow
       us to seek to the correct position for the number of file groups
       and the total number of files when these values are updated on
       subsequent function calls.
    -------------------------------------------------------------------*/
    fprintf(fp, "TapeID:  %-15.15sTapeType: %-15.15sCreated: %-22.22s\n",
	tapeid, ttype, date_time);

    fprintf(fp, "System:  %-15.15sDrive:    %-15.15sOwner:   %-22.22s\n",
	host, drive, user);

    fprintf(fp, "NGroups: %-15dNFiles:   %-46d\n", ngroups, ptr->nfiles);

    fprintf(fp, "%-80.80s\n", " ");

    fprintf(fp, "%-27.27s %-52.52s\n", "Tape filename", "Disk filename");

    fprintf(fp, "%-27.27s %-52.52s\n", "---------------------------",
	"----------------------------------------------------");
    }
else
    {
    /* Open the summary file for updating
    -------------------------------------*/
    fp = fopen(summary, "r+");
    if (fp == NULL)
	{
	c_errmsg("Error opening summary file", "ansistack-open", &nonfatal);
	return(E_FAIL);
	}


    /* Update the number of file groups and total number of files on the tape
    -------------------------------------------------------------------------*/
    fseek(fp, NGROUP_POS, SEEK_SET);
    fprintf(fp, "%-15d", ngroups);

    fseek(fp, NFILE_POS, SEEK_SET);
    fprintf(fp, "%-15d", ngroups * ptr->nfiles);


    /* Position the file pointer at EOF
    -----------------------------------*/
    fseek(fp, 0, SEEK_END);
    }


/* Print the tape name and disk name for every file in the group
----------------------------------------------------------------*/
for (i = 0; i < ptr->nfiles; i++)
    {
    if (outform == 0)
	{
	fprintf(fp, "%-27.27s %-1s\n", ptr->tapenames[i], ptr->disknames[i]);
	}
    else
	{
	strcpy(tmptapename,ptr->disknames[i]);
	strncpy(&tmptapename[strlen(tmptapename)-4],"l1b\0",4);
	fprintf(fp, "%-27.27s %-1s\n", tmptapename, ptr->disknames[i]);
	}
    }


/* Close the file and return
----------------------------*/
fclose(fp);
return(E_SUCC);
}
