#!/bin/csh -f
#
# FUNCTION: aotrans.csh
#
# PURPOSE:  Perform transfer of stitched AVHRR browse images
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# T.Baltzer	   11/96  Development, stole from BWTRANS
#


#------------------------
# Set up global variables
#------------------------
set noglob
set brwsys  = edcsgs10
set brwdir  = /prod/ims/graphics/orb_seg/tmp
set logfile = `echo "[$ADAPSLOG]aotrans" | tr / . | sed  's/\[\./\[/'`
set ntries  = 3


#-----------------------------------------------------------------
# Set up csh.  If there is a Berkley version of rcp use it instead
#-----------------------------------------------------------------
unalias cd
unalias rcp
if (-e /usr/bsd/rcp) alias rcp /usr/bsd/rcp


#---------------------------------------------------------
# Ensure the path to write_log was supplied as an argument
#---------------------------------------------------------
if ($#argv != 3) then
   echo ""
   echo "Usage: aotrans <lpath> <wpath> <bpath>"
   echo ""
   echo "       <lpath> is the directory where the lsfiles command is located."
   echo "       <wpath> is the directory where the write_log command is"
   echo "       located."
   echo "       <bpath> is the directory where the browse images are"
   echo "       located."
   echo ""
   exit 1
endif


#------------------------------------------------------------------------------
# Set up an alias for the lsfiles and write_log commands.  The paths could be
# the current directory in which case we must set up an alias before we change
# directories.
#------------------------------------------------------------------------------
if ("$1" == ".") then
   alias lsfiles `pwd`/lsfiles
else
   alias lsfiles $1/lsfiles
   endif
if ("$2" == ".") then
   alias wrtlog `pwd`/write_log
else
   alias wrtlog $2/write_log
   endif


#----------------------------
# cd to the correct directory
#----------------------------
cd $3


#--------------------------------------------
# Transfer all files in the current directory
# NOTE:
# The permanent file matching scheme:
#    set files = `lsfiles '\.brw$'`
# has been altered to be:
#    set files = `lsfiles '^ao.*\.brw$'`
# This is to discriminate against non-stitched
# orbits (not ao....).  We only want to
# transfer stitched orbits at this time.
#--------------------------------------------
unset noglob
set files = `lsfiles '^ao.*\.brw$'`
set noglob
foreach file ($files)
   if (-z $file) continue

   #----------------------------------------------
   # Retrieve just the scene ID from the file name
   #----------------------------------------------
   set sceneid = `echo $file | awk -F\_ '{print $1}'`

   #-----------------------------------------------------
   # Strip off the extension for the file name on $brwsys
   #-----------------------------------------------------
   set newfile = `echo $file | awk -F\. '{print $1}' | tr "[A-Z]" "[a-z]"`

   #--------------------------------
   # Issue a message to the log file
   #--------------------------------
   set text = "Starting the browse transfer"
   wrtlog $logfile $sceneid aotrans-start $text

   #-------------------
   # Transfer the image
   #-------------------
   @ i = 0
   while ($i < $ntries)
      @ i += 1
      rcp $file $user"@"$brwsys":"$brwdir/$newfile
      if (! $status) then
	 set text = "Completed the browse transfer successfully"
	 wrtlog $logfile $sceneid aotrans-success $text
	 break
      else if ($i < $ntries) then
         sleep 60
      else
         mv $file $ADAPSERR/.
	 set text = "ERROR encountered in the browse transfer"
	 wrtlog $logfile $sceneid aotrans-error $text
      endif
   end

   #----------------------------------------------------------
   # Check to see if the user is trying to stop this process
   #----------------------------------------------------------
   set stopfile = $ADAPSTABLES/aotrans.stop
   if (-e $stopfile) then
      set text = "AOTRANS terminated by user"
      wrtlog $logfile aotrans aotrans-ustop $text
      exit 0
   endif
end

exit 0
