/*******************************************************************************
FUNCTION:	ARCHIVE

PURPOSE:	Manage all archive activities related to ADAPS processing.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    06/00  T. Baltzer       Original development

PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "tae.h"
#include "util.h"
#include "adaps.h"
#include "archive.h"
#include "archio.h"

main()
{
struct DROPPED  drop;		/* Dropped line data structure 		     */
struct MODEL 	model;		/* Archive header model info		     */	
struct PARBLK	vblock;		/* TAE parameter block			     */

int	foundflg;		/* Ceosid found or not			     */

long	access = IREAD,		/* Access flag				     */
	len,			/* Length of string			     */
	status,			/* Return status			     */
	std_out,		/* LUN to standard output		     */
	zero = 0;		/* Constant of zero			     */

char	hosin[CMLEN],		/* Input host file name			     */
	stackid[4];		/* CEOS ID which we are stacking a tape for  */

struct VARIABLE *parm;          /* Ptr to a TAE parameter                    */


/*
  Initialization
*/
c_inlas(&vblock,&std_out);


/*
  Determine the CEOSID list we should be looking in
*/
parm = p_find(&vblock, "CEOSID");
strcpy(stackid, SVAL(*parm, 0));
len = (long)strlen(stackid);
c_up2low(stackid,&len);


/*
  Retrieve input AVHRR image file name
*/
status = c_getnam(&vblock, "IN", &zero, hosin, "arch", &access);
if (status != E_SUCC)
    c_lasmsg(TRUE,"archive-fatal","Fatal error encountered");


/*
  Get the AVHRR header information
*/
status = c_getahdr(hosin,&model,&drop);
if (status != E_SUCC)
    c_lasmsg(TRUE,"archive-fatal","Fatal error encountered");


/*
  Call the routine to determine if this ceos id is in the list
*/
status = read_select(model.station,stackid,&foundflg);
if (status != E_SUCC)
    c_lasmsg(TRUE,"archive-fatal","Fatal error encountered");


/*
  Return the value of the 'found flag' to TAE land.
*/
status = q_intg(&vblock,"FOUNDFLG",1,&foundflg,P_UPDATE);
if (status != SUCCESS)
    {
    c_lasmsg(FALSE,"archive-send","Error sending FOUNDFLG to TAE");
    c_lasmsg(TRUE,"archive-fatal","Fatal error encountered");
    }


/*
  Write the parameter block to TAE
*/
status = q_out(&vblock);
if (status != SUCCESS)
    {
    c_lasmsg(FALSE,"archive-qout","Error sending variable block to TAE");
    c_lasmsg(TRUE,"archive-fatal","Fatal error encountered");
    }


/*
Exit with a success status, with no diagnostic message
*/
z_exit(1,"");


/*
  Put a return here to make the compiler happy
*/
return(E_SUCC);

}

