/*******************************************************************************
FUNCTION:	read_select

PURPOSE:	Read the appropriate select file to determine if the specified
		ceos id resides within the select file.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    06/00  T.Baltzer        Original development


PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"


int read_select
    (
    char ceosid[],	/* CEOS identifier of interest			*/
    char stackid[],	/* CEOS identifier which we are stacking for	*/
    int *foundflg 	/* Found CEOS id in file or not			*/
    )

{
FILE *fp;                    /* File pointer to the data base file            */

int  i,			     /* Looping variable			      */
     nlines,                 /* Number of lines to be read from a file        */
     status;                 /* Return status                                 */

char description[80],        /* Receiving station description                 */
     disk[CMLEN],            /* Disk locations                                */
     filenam[CMLEN],         /* File name for select			      */
     station[4],             /* Receiving station                             */
     str[CMLEN];             /* Temporary string                              */


/* Retrieve the ADAPSTABLES environment variable
------------------------------------------------*/
if (getenv("ADAPSTABLES") != NULL)
   strcpy(disk, getenv("ADAPSTABLES"));
else
   {
   c_lasmsg(FALSE, "archive-env", "Error retrieving ADAPSTABLES");
   return(E_FAIL);
   }
strcat(disk, "/");


/* Determine the selective copy file name
----------------------------------------------*/
if ((strncmp(ceosid,"KM",2) == 0) || (strncmp(ceosid,"km",2) == 0))
   sprintf(filenam, "%sselectorb.%s", disk, stackid);
else
   sprintf(filenam, "%sselect.%s", disk, stackid);


/* Open the selective copy file
-----------------------------------*/
fp = fopen(filenam, "r");
if (fp == NULL)
   {
   c_lasmsg(FALSE,"archive-open","Error opening selective copy file %s",
	    filenam);
   return(E_FAIL);
   }


/* Read the first line to determine the number of lines in the description
--------------------------------------------------------------------------*/
if (fgets(str, CMLEN, fp) == NULL)
   {
   fclose(fp);
   c_lasmsg(FALSE,"archive-read","Error reading selective copy file %s",
	    filenam);
   return(E_FAIL);
   }


/* Determine the number of lines in the description of the file (Must be > 0)
-----------------------------------------------------------------------------*/
if ((sscanf(str, "%d", &nlines) != 1) || (nlines < 1))
   {
   fclose(fp);
   c_lasmsg(FALSE,"archive-read","Error reading selective copy file %s",
	    filenam);
   return(E_FAIL);
   }


/* Read the remaining lines of the description
----------------------------------------------*/
for (i = 1; i < nlines; i++)
   {
   if (fgets(str, CMLEN, fp) == NULL)
      {
      fclose(fp);
      c_lasmsg(FALSE,"archive-read","Error reading selective copy file %s",
	       filenam);
      return(E_FAIL);
      }
   }


/* Read each line until a match is found
----------------------------------------*/
status = E_SUCC;
while (status != EOF)
   {
   status = fscanf(fp, "%3s %s", station, description);
   if (status == EOF)
      {
      *foundflg = FALSE;
      continue;
      }
   else if (status < 1)
      {
      fclose(fp);
      c_lasmsg(FALSE,"archive-read","Error reading selective copy file %s",
	       filenam);
      return(E_FAIL);
      }

   /* Check if a station code is in the selective copy file
   --------------------------------------------------------*/
   if (strcmp(ceosid,station) == 0)
      {
      *foundflg = TRUE;
      break;
      }
   }


/* Close the selective copy file
--------------------------------*/
fclose(fp);
return(E_SUCC);
}
