/* This is a dippy little program that takes a spectral response function,
such as one published by NOAA and converts it to one that is usable by 6s.

INPUT:	spectral response function in the form of:
		wavelength (in microns) percent response at the wavelength
	    eg:
	        0.5 0.57
		0.502 0.5206
		0.504 0.5348
		...

		0.598 80.02
		0.6 82.31
		0.602 82.36
		...

		0.796 0.25
		0.798 0.2935
		0.8 0.2765

	    Note the file is in in increasing wavelength order.

OUTPUT:	standard output is either an adjusted curve in columnar form
	or a fortran data statement.

RETURNS: 0 on success
         1 on failure

USAGE:   convert_to_6s <input file> <starting wave number | -1> <y/n>

	use -1 the first time to determine the actual starting wave number
	to use for generation of the function.

---------------------------------------------------------------------------*/
#include <stdio.h>


int main( int argc, char *argv[] )
{
FILE *inf;
float wavelength, response, x1, x2;
float wv[2000],res[2000];
int   num;
int firstindex;
int i,j;
float first,second,wv1,wv2;
float firstwave;
int fortran,count;


if ( argc != 4 )
    {
    printf("\n\nUsage: %s: <infile> <starting wavelength> <fortran flag y/n>\n",
        argv[0]);
    printf("Use -1 for wavelength the first time\n\n");
    return 1;
    }

inf = fopen( argv[1], "r" );
if ( !inf )
    {
    printf("\nUnable to open file: %s\n\n", argv[1] );
    return 1;
    }

num = 0;
while ( fscanf( inf, "%f %f", &wavelength, &response ) != EOF )
    {
    wv[num] = wavelength;
    res[num] = response / 100.0;
    num++;
    }

fclose(inf);

if ( argv[3][0] == 'y' || argv[3][0] == 'Y' )
    fortran = 1;
else
    fortran = 0;

sscanf(argv[2], "%f", &firstwave);
if ( firstwave == -1 )
    firstindex = (int)(400.0 * ( wv[0] - 0.245 ));
else
    firstindex = (int)(400.0 * ( firstwave - 0.245 ));

if ( fortran )
    printf("c first index is: %d\n", firstindex );
else
    printf("# first index is: %d\n", firstindex );
if ( firstwave == -1.0 )
    {
    int x;
    printf("the first wavelength of the function is %f\n", wv[0]);
    printf("    %f / 0.0025 = %f\n", wv[0], wv[0] / 0.0025 );
    x = (int)(wv[0] / 0.0025);
    printf("if it is evenly divisible by 0.0025 use %f\n", wv[0]);
    printf("for the second run, otherwise use %f\n", (float)x * 0.0025);
    return 0;
    }

if ( fortran )
    {
    printf("c starting wave: %f\n", firstwave);
    printf("      DATA (SR(??,L),L=1,1501)/ %d*0.,", firstindex-2);
    }

count = j = 0;
for ( wavelength = firstwave; wavelength <= wv[num-1]; wavelength += 0.0025 )
    {
    i = 0;
    while ( wv[i++] <= wavelength );
    i--;

    if ( fortran )
        {
	if ( j == 0 )
	    {
	    j++;
	    printf("\n     a 0.000000,");
	    }
	else if ( j % 5 == 0 )
	    printf("\n     a");
	}

    if ( wavelength == wv[i] )
	if ( fortran )
	    {
	    count++;
	    printf(" %f,", res[i] );
	    }
	else
	    printf("%6.4f %f\n", wavelength, res[i] );
    
    else
        {
	if ( i == 0 )
	    {
	    wv1 = wv[0] - 0.0025;
	    first = 0.0;
	    }
	else
	    {
	    wv1 = wv[i-1];
	    first = res[i-1];
	    }
	
	if ( i > num )
	    {
	    wv2 = wv[num-1] + 0.0025;
	    second = 0.0;
	    }
	else
	    {
	    wv2 = wv[i];
	    second = res[i];
	    }
	
	x2 = wv2 - wv1;
	x1 = wavelength - wv1;
	if ( fortran )
	    {
	    count++;
	    printf(" %f,", (second-first)*(x1/x2)+first);
	    }
	else
	    printf("%6.4f %f\n", wavelength, (second-first)*(x1/x2)+first);
	}
    j++;
    }
if (fortran )
    {
    printf("\n     a %d*0.0/\n", 1501-firstindex-count+1);
    printf("c  ending wave: %f\n", wavelength-0.0025);
    }
else
    printf("\n");
}
