      program ssssss
c     program ssssss(input,output,tape5,tape6)
 
 
 
c**********************************************************************c
c                                                                      c
c                                                                      c
c                                                                      c
c                                                                      c
c                                                                      c
c       ********************************************************       c
c       *          second  simulation of satellite signal      *       c
c       *                in the solar spectrum                 *       c
c       *       ... (6s.. ....... (6s.1)....... (6s.1)..       *       c
c       *             .... (6s.1)........(6s.1)...             *       c
c       *                  ...... (6s.1)......                 *       c
c       *              version 3.1                             *       c
c       *                                                      *       c
c       *  this code predicts the satellite signal from 0.25   *       c
c       *  to 4.0 microns assuming cloudless atmosphere.       *       c
c       *  the main atmospheric effects (gaseous absorption    *       c
c       *  by water vapor,carbon dioxyde,oxygen and ozone;     *       c
c       *  scattering by molecules and aerosols) are taken     *       c
c       *  into account. non-uniform surfaces may be           *       c
c       *  considered,as well as bidirectional reflectances    *       c
c       *            as boundary conditions                    *       c
c       *                                                      *       c
c       *   the following input parameters are needed          *       c
c       *         geometrical conditions                       *       c
c       *         atmospheric model for gaseous components     *       c
c       *         aerosol model (type and concentration)       *       c
c       *         spectral condition                           *       c
c       *         ground reflectance (type and spectral var.)  *       c
c       *   at each step, you can either select some proposed  *       c
c       *  standard conditions (for example,spectral bands of  *       c
c       *  satellite for spectral conditions) or define your   *       c
c       *  own conditions(in the example,you have to define    *       c
c       *  the assumed spectral response).                     *       c
c       *                                                      *       c
c       *   more details are given at each data input step     *       c
c       *                                                      *       c
c       ********************************************************       c
c                                                                      c
c                                                                      c
c                                                                      c
c                                                                      c
c                                                                      c
c**********************************************************************c
 
 
 
 
 
 
c**********************************************************************c
c                                                                      c
c                                                                      c
c       ********************************************************       c
c       *             authors of this code are                 *       c
c       *                                                      *       c
c       *  (1) Deuze j.l.;(1) Herman m.;(1),(2) Morcrette j.j. *       c
c       *               (1) Tanre d.; (1),(3) Vermote e.       *       c
c       *                                                      *       c
c       *                       from                           *       c
c       *                                                      *       c
c       *     (1) laboratoire d' optique atmospherique         *       c
c       *         universite des sciences et techniques        *       c
c       *         de lille                                     *       c
c       *         u.e.r. de physique fondamentale              *       c
c       *         59655 villeneuve d' ascq cedex               *       c
c       *         france                                       *       c
c       *                                                      *       c
c       *     (2) e.c.m.w.f.                                   *       c
c       *                                                      *       c
c       *     (3) code 923 / gimms group                       *       c
c       *         gsfc/nasa                                    *       c
c       *         greenbelt, md 20771                          *       c
c       *         usa                                          *       c
c       *                                                      *       c
c       *                                                      *       c
c       *                                                      *       c
c       ********************************************************       c
c                                                                      c
c                                                                      c
c**********************************************************************c
 
 
c**********************************************************************c
c       ********************************************************       c
c       *                limits of validity                    *       c
c       *                                                      *       c
c       *   geometrical parameters    solar zenith angle and   *       c
c       *                             satellite zenith angle   *       c
c       *                             must be less than 60 and *       c
c       *                             50 degrees respectively. *       c
c       *                                                      *       c
c       *   atmospheric model         no limitations           *       c
c       *                                                      *       c
c       *   aerosol model             the visibility must be   *       c
c       *                             better than 5.0km        *       c
c       *                             for smaller values       *       c
c       *                             calculations might be    *       c
c       *                             no more valid.           *       c
c       *                                                      *       c
c       *   spectral conditions       the gaseous transmittance*       c
c       *                             and the scattering func  *       c
c       *                             tions are valid from 0.25*       c
c       *                             to 4.0 micron. but the   *       c
c       *                             treatment of interaction *       c
c       *                             between absorption and   *       c
c       *                             scattering is correct for*       c
c       *                             not too large absorption *       c
c       *                             if you want to compute   *       c
c       *                             signal within absorption *       c
c       *                             bands,this interaction   *       c
c       *                             ought to be reconsidered *       c
c       *                                                      *       c
c       *   ground reflectance (type) you can consider a patchy*       c
c       *                             structure:that is a circu*       c
c       *                             lar target of radius rad *       c
c       *                             and of reflectance roc,  *       c
c       *                             within an environnement  *       c
c       *                             of reflectance roe.      *       c
c       *                                                      *       c
c       *   ground reflectance (type continued): for uniform   *       c
c       *                             surface conditions only, *       c
c       *                             you may consider directio*       c
c       *                             nal reflectance as bounda*       c
c       *                             ry conditions.           *       c
c       *                             some analytical model are*       c
c       *                             proposed, the user can   *       c
c       *                             specify his own values.  *       c
c       *                             the code assumes that the*       c
c       *                             brdf is spectrally inde- *       c
c       *                             pendent                  *       c
c       *                                                      *       c
c       *   ground reflectance (spectral variation) four typi  *       c
c       *                             cal reflectances are pro *       c
c       *                             posed, defined within    *       c
c       *                             given spectral range.    *       c
c       *                             this range differs accor *       c
c       *                             ding to the selected case*       c
c       *                             the reflectance is set to*       c
c       *                             0 outside this range,due *       c
c       *                             to the deficiency of data*       c
c       *                             user must verify these   *       c
c       *                             limits. that is obviously*       c
c       *                             irrelevant for brdf      *       c
c       *                                                      *       c
c       ********************************************************       c
c**********************************************************************c
 
c****************************************************************************c
c  for considering brdf< we have to compute the downward radiance in the     c
c  whole hemisphere. to perform such computions, we selected the successive  c
c  orders of scattering method. that method requires numerical integration   c
c  over angles and optical depth. the integration method is the gauss method,c
c  mu is the number of angles nmu+1, nmu is settled to 24. the accuracy of   c
c  the computations is obviously depending on the nmu value. this value      c
c  can be easily changed as a parameter as well as the nt value which        c
c  is the number of layers for performing the vertical integration. the      c
c  downward radiance is computed for nmu values of the zenith angle and np   c
c  values of the azimuth angle. the integration of the product of the        c
c  radiance by the brdf is so performed over the nmu*np values. np is settledc
c  to 13, that value can be also changed. mu2 is equal to 2 times nmu.       c
c  xlmus is the downward radiance, xf the downward irradiance, rm and gb     c
c  the angles and the weights for the gauss integration over the zenith, rp  c
c  and bp respectively for the azimuth integration.                          c
c****************************************************************************c
 
      parameter(nt_p=13,mu_p=13,mu2_p=24,np_p=13)
      dimension anglem(mu2_p),weightm(mu2_p),
     s   rm(-mu_p:mu_p),gb(-mu_p:mu_p),rp(np_p),gp(np_p)
      dimension  xlmus(-mu_p:mu_p,np_p),xlmuv(-mu_p:mu_p,np_p)
      dimension angmu(10),angphi(13),brdfints(-mu_p:mu_p,np_p)
     s    ,brdfdats(10,13),
     s    brdfintv(-mu_p:mu_p,np_p),brdfdatv(10,13),robar(1501),
     s    robarp(1501),robard(1501),xlm1(-mu_p:mu_p,np_p),
     s    xlm2(-mu_p:mu_p,np_p)
 
        real anglem,weightm,rm,gb
        real rp,gp,xlmus,xlmuv,angmu,angphi,brdfints,brdfdats
        real brdfintv,brdfdatv,robar,robarp,robard,xlm1,xlm2
        real c,wldisc,ani,anr,aini,ainr,rocl,roel,zpl,ppl,tpl,whpl
        real wopl,xacc,phasel,cgaus,pha,betal,s,wlinf,wlsup,delta
        real sigma,z,p,t,wh,wo,ext,ome,gasym,phase,roatm,dtdir
        real dtdif,utdir,utdif,sphal,wldis,trayl,traypl,pi,pi2,step
        real asol,phi0,avis,phiv,tu,xlon,xlat,xlonan,hna,dsol
        real phi,phirad,xmus,xmuv,xmup,xmud,adif,uw,uo3,taer55
        real taer,v,xps,uwus,uo3us,xpp,taer55p,puw,puo3,puwus
        real puo3us,wl,wlmoy,tamoy,tamoyp,pizmoy,pizmoyp,trmoy
        real trmoyp,fr,rad,spalt
        real albbrdf,par1,par2,par3,par4,robar1,xnorm1,rob,xnor
        real rdown,rdir,robar2,xnorm2,ro,roc,roe,rapp,rocave,roeave
        real seb,sbor,swl,sb,refet,refet1,refet2,refet3,alumet
        real tgasm,rog,dgasm,ugasm,sdwava,sdozon,sddica,sdoxyg
        real sdniox,sdmoca,sdmeth,suwava,suozon,sudica,suoxyg
        real suniox,sumoca,sumeth,stwava,stozon,stdica,stoxyg,stniox
        real stmoca,stmeth,sodray,sodaer,sodtot,fophsr,fophsa,sroray
        real sroaer,srotot,ssdaer,sdtotr,sdtota,sdtott,sutotr,sutota
        real sutott,sasr,sasa,sast,adtwava,dtozon,dtdica,dtoxyg
        real dtniox,dtmeth,dtmoca,autwava,utozon,utdica,utoxyg,utniox
        real utmeth,utmoca,attwava,ttozon,ttdica,ttoxyg,ttniox
        real ttmeth,ttmoca,dtwava,utwava,ttwava,coef,romix,rorayl
        real roaero,phaa,phar,tsca,tray,trayp,taerp,dtott,utott
        real astot,asray,asaer,utotr,utota,dtotr,dtota,dgtot,tgtot
        real ugtot,edifr,edifa,tdird,tdiru,tdifd,tdifu,roprim,fra
        real fae,avr,romeas1,romeas2,romeas3,alumeas,sodrayp
        real sodaerp,sodtotp,tdir,tdif,etn,esn,es,ea0n,ea0,ee0n
        real ee0,tmdir,tmdif,xla0n,xla0,xltn,xlt,xlen,xle,pizera
        real fophst,pizerr,pizert,xrad,xa,xb,xc
        real sha,sham
        integer nt,mu,mu2,np,k,iwr,mum1,idatmp
        integer j,iread,l,igeom,month,jday,nc,nl,idatm,iaer,n
        integer iwave,iinf,isup,ik,i,inhomo,idirec,ibrdf,igroun
        integer igrou1,igrou2,isort
c***********************************************************************
c                             return to 6s
c***********************************************************************
      dimension c(4),wldisc(10),ani(2,3),anr(2,3),aini(2,3),ainr(2,3)
      dimension rocl(1501),roel(1501)
      character etiq1(8)*60,nsat(35)*17,atmid(7)*46,reflec(7)*71
 
      logical ier
      common/sixs_ier/iwr,ier
c***********************************************************************
c     for considering pixel and sensor  altitude
c***********************************************************************
      real pps,palt,ftray
      common /sixs_planesim/zpl(34),ppl(34),tpl(34),whpl(34),wopl(34)
      common /sixs_test/xacc
c***********************************************************************
c     for considering brdf
c***********************************************************************
      common /sixs_sos/phasel(10,83),cgaus(83)
      common /sixs_trunc/pha(83),betal(0:80)
      real optics(3),struct(4)
      integer options(5)
      integer pild,pihs
      real pxLt,pc,pRl,pTl,pRs
      real pws,pcl
c***********************************************************************
c                             return to 6s
c***********************************************************************
      common /sixs_ffu/s(1501),wlinf,wlsup
      common /sixs_del/ delta,sigma
      common /sixs_atm/z(34),p(34),t(34),wh(34),wo(34)
      common /sixs_aer/ext(10),ome(10),gasym(10),phase(10)
      common /sixs_disc/ roatm(3,10),dtdir(3,10),dtdif(3,10),
     s utdir(3,10),utdif(3,10),sphal(3,10),wldis(10),trayl(10),
     s traypl(10)
 
 
c****************************************************************************c
c   angmu and angphi are the angles were the brdf is measured. these values  c
c   can be changed as soon as they are well distributed over the whole space c
c   before the gauss integration, these values are interpolated to the gauss c
c   angles                                                                   c
c****************************************************************************c
      data angmu /85.0,80.0,70.0,60.0,50.0,40.0,30.0,20.0,10.0,0.00/
      data angphi/0.00,30.0,60.0,90.0,120.0,150.0,180.0,
     s          210.0,240.0,270.0,300.0,330.0,360.0/
 
c***********************************************************************
c                             return to 6s
c***********************************************************************
      data wldisc /0.400,0.488,0.515,0.550,0.633,
     s             0.694,0.860,1.536,2.250,3.750/
 
 
      data etiq1/
     s '(1h*,22x,34h user defined conditions          ,t79,1h*)',
     s '(1h*,22x,24h meteosat observation   ,t79,1h*)          ',
     s '(1h*,22x,25h goes east observation   ,t79,1h*)         ',
     s '(1h*,22x,25h goes west observation   ,t79,1h*)         ',
     s '(1h*,22x,30h avhrr (noaa 8) observation   ,t79,1h*)    ',
     s '(1h*,22x,30h avhrr (noaa 9) observation   ,t79,1h*)    ',
     s '(1h*,22x,24h h.r.v.   observation   ,t79,1h*)          ',
     s '(1h*,22x,24h t.m.     observation   ,t79,1h*)          '/
 
       data nsat/
     s ' constant        ',' user s          ',
     s ' meteosat        ',' goes east       ',' goes west       ',
     s ' avhrr 1 (noaa6) ',' avhrr 2 (noaa6) ',
     s ' avhrr 1 (noaa7) ',' avhrr 2 (noaa7) ',
     s ' avhrr 1 (noaa8) ',' avhrr 2 (noaa8) ',
     s ' avhrr 1 (noaa9) ',' avhrr 2 (noaa9) ',
     s ' avhrr 1 (noaa10)',' avhrr 2 (noaa10)',
     s ' avhrr 1 (noaa11)',' avhrr 2 (noaa11)',
     s ' hrv1 1          ',' hrv1 2          ',' hrv1 3          ',
     s ' hrv1 pan        ',
     s ' hrv2 1          ',' hrv2 2          ',' hrv2 3          ',
     s ' hrv2 pan        ',
     s '  tm  1          ','  tm  2          ','  tm  3          ',
     s '  tm  4          ','  tm  5          ','  tm  7          ',
     s '  mss 4          ','  mss 5          ',
     s '  mss 6          ','  mss 7          '/
 
      data atmid /
     s 'no absorption computed                        ',
     s 'tropical          (uh2o=4.12cm,uo3=.247cm-atm)',
     s 'midlatitude summer(uh2o=2.93cm,uo3=.319cm-atm)',
     s 'midlatitude winter(uh2o=.853cm,uo3=.395cm-atm)',
     s 'subarctic  summer (uh2o=2.10cm,uo3=.480cm-atm)',
     s 'subarctic  winter (uh2o=.419cm,uo3=.480cm-atm)',
     s 'us  standard 1962 (uh2o=1.42cm,uo3=.344cm-atm)'/
 
 
      data  reflec /
     & '(1h*,22x,39h user defined spectral reflectance     ,f6.3,t79
     & ,1h*) ',
     & '(1h*,22x,27h monochromatic reflectance ,f6.3,t79,1h*)',
     & '(1h*,22x,39h constant reflectance over the spectra ,f6.3,t79
     & ,1h*) ',
     & '(1h*,22x,39h spectral vegetation ground reflectance,f6.3,t79
     & ,1h*) ',
     & '(1h*,22x,39h spectral clear water reflectance      ,f6.3,t79
     & ,1h*) ',
     & '(1h*,22x,39h spectral dry sand ground reflectance  ,f6.3,t79
     & ,1h*) ',
     & '(1h*,22x,39h spectral lake water reflectance       ,f6.3,t79
     & ,1h*) '/
c***********************************************************************
c   Parameters  initialization
c***********************************************************************
      nt=nt_p
      mu=mu_p
      mu2=mu2_p
      np=np_p
      iwr=6
      ier=.FALSE.
c***********************************************************************
c  preliminary computations for gauss integration
c***********************************************************************
      pi=acos(-1.)
      pi2=2*pi
      do k=1,13
      angphi(k)=angphi(k)*pi/180.
      enddo
      do k=1,10
      angmu(k)=cos(angmu(k)*pi/180.)
      enddo
      call gauss(-1.,1.,anglem,weightm,mu2)
      call gauss(0.,pi2,rp,gp,np)
      mum1=mu-1
      do 581 j=-mum1,-1
      k=mu+j
      rm(-j-mu)=anglem(k)
      gb(-j-mu)=weightm(k)
  581 continue
      do 582 j=1,mum1
      k=mum1+j
      rm(mu-j)=anglem(k)
      gb(mu-j)=weightm(k)
  582 continue
      gb(-mu)=0.
      gb(0)=0.
      gb(mu)=0.
 
c***********************************************************************
c                             return to 6s
c***********************************************************************
c constantes values
      sigma=0.056032
      delta=0.0279
      xacc=1.e-06
      iread=5
      step=0.0025
      do 1111 l=1,10
      wldis(l)=wldisc(l)
 1111 continue
 
c**********************************************************************c
c                                                                      c
c                                                *     sun             c
c                                              \ * /                   c
c                                            * * * * *                 c
c                                   z          / * \                   c
c                                   +           /*                     c
c            satellite    /         +          /                       c
c                       o/          +         /                        c
c                      /.\          +        /.                        c
c                     / . \  _avis-_+_-asol_/ .                        c
c                       .  \-      -+      /  .    north               c
c                       .   \       +     /   .  +                     c
c                       .    \      +    /    .+                       c
c                       .     \     +   /    +.                        c
c                       .      \    +  /   +  .                        c
c                       .       \   + /  +    .                        c
c                       .        \  +/ +      .                        c
c    west + + + + + + + . + + + + +\+ + + + + . + + + + + + + + east   c
c                       .          +..        .                        c
c                       .        + .   .      .                        c
c                       .      +  .      .    .                        c
c                       .    +   .       .'.  .                        c
c                       .  +    .. . , '     ..                        c
c                       .+     .       \       .                       c
c                      +.     .         \        .                     c
c                    +  .    .           \         .                   c
c             south     .   .       (phiv-phi0)                        c
c                                                                      c
c                                                                      c
c                                                                      c
c**********************************************************************c
 
c**********************************************************************c
c       igeom               geometrical conditions                     c
c               --------------------------------------                 c
c                                                                      c
c                                                                      c
c   you choose your own conditions; igeom=0                            c
c         0     enter solar zenith angle   (in degrees )               c
c                     solar azimuth angle        "                     c
c                     satellite zenith angle     "                     c
c                     satellite azimuth angle    "                     c
c                                                                      c
c   or you select one of the following satellite conditions:igeom=1to7 c
c         1       meteosat observation                                 c
c                 enter month,day,decimal hour (universal time-hh.ddd) c
c                       n. of column,n. of line.(full scale 5000*2500) c
c                                                                      c
c         2       goes east observation                                c
c                 enter month,day,decimal hour (universal time-hh.ddd) c
c                      n. of column,n. of line.(full scale 17000*12000)c
c                                                                      c
c         3       goes west observation                                c
c                 enter month,day,decimal hour (universal time-hh.ddd) c
c                      n. of column,n. of line.(full scale 17000*12000)c
c                                                                      c
c         4       avhrr ( noaa8 )                                      c
c                 enter month,day,decimal hour (universal time-hh.ddd) c
c                       n. of column(1-2048),xlonan,hna                c
c                       give long.(xlonan) and overpass hour (hna) at  c
c                       the ascendant node at equator                  c
c                                                                      c
c         5       avhrr ( noaa9 )                                      c
c                 enter month,day,decimal hour (universal time-hh.ddd) c
c                       n. of column(1-2048),xlonan,hna                c
c                       give long.(xlonan) and overpass hour (hna) at  c
c                       the ascendant node at equator                  c
c                                                                      c
c         6       hrv   ( spot )    * enter month,day,hh.ddd,long.,lat.c
c                                                                      c
c         7       tm    ( landsat ) * enter month,day,hh.ddd,long.,lat.c
c                                                                      c
c                                                                      c
c     note:       for hrv and tm experiments long. and lat. are the    c
c                 coordinates of the scene center.                     c
c                 lat. must be > 0 for north lat., < 0 for south lat.  c
c                 long. must be > 0 for east long., <0 for west long.  c
c                                                                      c
c                 solar and viewing positions are computed             c
c                                                                      c
c**********************************************************************c
	write (6,*) 'enter igeom'
      read(iread,*) igeom
      goto(1001,1002,1003,1004,1005,1006,1007),igeom
c   igeom=0.....
	write (6,*) 'enter asol,phi0,avis,phiv,month,jday'
      read(iread,*) asol,phi0,avis,phiv,month,jday
      goto 22
c
 1001 read(iread,*) month,jday,tu,nc,nl
      call posmto(month,jday,tu,nc,nl,
     1            asol,phi0,avis,phiv,xlon,xlat)
      goto 22
 1002 read(iread,*) month,jday,tu,nc,nl
      call posge(month,jday,tu,nc,nl,
     1           asol,phi0,avis,phiv,xlon,xlat)
      goto 22
 1003 read(iread,*) month,jday,tu,nc,nl
      call posgw(month,jday,tu,nc,nl,
     1           asol,phi0,avis,phiv,xlon,xlat)
      goto 22
 1004 read(iread,*) month,jday,tu,nc,xlonan,hna
      call posnoa(month,jday,tu,nc,xlonan,hna,
     1            asol,phi0,avis,phiv,xlon,xlat)
      goto 22
 1005 read(iread,*) month,jday,tu,nc,xlonan,hna
      call posnoa(month,jday,tu,nc,xlonan,hna,
     1            asol,phi0,avis,phiv,xlon,xlat)
      goto 22
 1006 read(iread,*) month,jday,tu,xlon,xlat
      call posspo(month,jday,tu,xlon,xlat,
     a            asol,phi0,avis,phiv)
      goto 22
 1007 read(iread,*) month,jday,tu,xlon,xlat
      call poslan(month,jday,tu,xlon,xlat,
     s            asol,phi0,avis,phiv)
   22 continue
      if(ier) stop
      dsol=1.
      call varsol(jday,month,
     1                           dsol)
 
c**********************************************************************c
c                                                                      c
c                                 / scattered direction                c
c                               /                                      c
c                             /                                        c
c                           / adif                                     c
c    incident   + + + + + + + + + + + + + + +                          c
c    direction                                                         c
c                                                                      c
c**********************************************************************c
      phi=abs(phiv-phi0)
      phirad=(phi0-phiv)*pi/180.
      if (phirad.lt.0.) phirad=phirad+2.*pi
      if (phirad.gt.(2.*pi)) phirad=phirad-2.*pi
      xmus=cos(asol*pi/180.)
      xmuv=cos(avis*pi/180.)
      xmup=cos(phirad)
      xmud=-xmus*xmuv-sqrt(1.-xmus*xmus)*sqrt(1.-xmuv*xmuv)*xmup
c test vermote bug
      if (xmud.gt.1.) xmud=1.
      if (xmud.lt.-1.) xmud=-1.
      adif=acos(xmud)*180./pi
 
c**********************************************************************c
c       idatm      atmospheric model                                   c
c                 --------------------                                 c
c                                                                      c
c                                                                      c
c  you select one of the following standard atmosphere: idatm=0 to 6   c
c         0    no gaseous absorption                                   c
c         1    tropical                )                               c
c         2    midlatitude summer      )                               c
c         3    midlatitude winter      )                               c
c         4    subarctic summer        )      from lowtran             c
c         5    subarctic winter        )                               c
c         6    us standard 62          )                               c
c                                                                      c
c  or you define your own atmospheric model idatm=7 or 8               c
c         7    user profile  (radiosonde data on 34 levels)            c
c              enter altitude       (  in km )                         c
c                    pressure       (  in mb )                         c
c                    temperature    (  in k  )                         c
c                    h2o density    (in  g/m3)                         c
c                    o3  density    (in  g/m3)                         c
c                                                                      c
c           for example, altitudes are  from  0 to 25km step of 1km    c
c                        from 25 to 50km step of 5km                   c
c                        and two values at 70km and 100km              c
c                        so you have 34*5 values to input.             c
c         8    enter water vapor and ozone contents                    c
c                 uw  (in  g/cm2 )                                     c
c                 uo3 (in  cm-atm)                                     c
c                 profil is taken from us62                            c
c                                                                      c
c**********************************************************************c
      uw=0.
      uo3=0.
	write (6,*) 'idatm (enter 8)'
      read(iread,*) idatm
      if(idatm.eq.0) go to 5
      if(idatm.eq.8) then
	write (6,*) 'enter uw,uo3'
	read(iread,*) uw,uo3
	endif
      if(idatm.ne.7) go to 6
      do 7 k=1,34
      read(iread,*) z(k),p(k),t(k),wh(k),wo(k)
    7 continue
      go to 5
    6 if(idatm.eq.1)  call tropic
      if(idatm.eq.2)  call midsum
      if(idatm.eq.3)  call midwin
      if(idatm.eq.4)  call subsum
      if(idatm.eq.5)  call subwin
      if(idatm.eq.6)  call us62
c     we have to define an atmosphere to compute rayleigh optical depth
    5 if(idatm.eq.0.or.idatm.eq.8)  call us62
 
 
c**********************************************************************c
c                                                                      c
c       iaer       aerosol model(type)                                 c
c                  --------------                                      c
c                                                                      c
c                                                                      c
c  you select one of the following standard aerosol models: iaer=0to3  c
c         0  no aerosols                                               c
c         1  continental model  )                                      c
c         2  maritime model     )  according to sra models             c
c         3  urban model        )                                      c
c         5  shettle model for background desert aerosol               c
c                                                                      c
c  or you define your own model: iaer=4                                c
c                                                                      c
c         4 enter the volumic percentage of each component             c
c                 c(1) = volumic % of dust-like                        c
c                 c(2) = volumic % of water-soluble                    c
c                 c(3) = volumic % of oceanic                          c
c                 c(4) = volumic % of soot                             c
c                   between 0 to 1                                     c
c                                                                      c
c**********************************************************************c

	write (6,*) 'enter iaer' 
      read(iread,*) iaer
      if(iaer.ne.4) go to 8
      read(iread,*) (c(n),n=1,4)
    8 continue
 
      goto(43,40,41,42,43),iaer+1
 
   40 c(1)=0.70
      c(2)=0.29
      c(3)=0.00
      c(4)=0.01
      go to 43
   41 c(1)=0.00
      c(2)=0.05
      c(3)=0.95
      c(4)=0.00
      go to 43
   42 c(1)=0.17
      c(2)=0.61
      c(3)=0.00
      c(4)=0.22
   43 call aeroso(iaer,c,xmud)
 
 
 
c**********************************************************************c
c              aerosol model (concentration)                           c
c              ----------------------------                            c
c                                                                      c
c                                                                      c
c   you have an estimate of the meteorological parameter: the visibi   c
c   lity v,                                                            c
c            enter directly the value of v in km(the aerosol optical   c
c            depth will be computed from a standard aerosol profile)   c
c                                                                      c
c   or you have an estimate of aerosol optical depth ,enter v=0 for    c
c            the visibility and enter the aerosol optical depth at 550 c
c                                                                      c
c   warning:  if iaer=0, enter v=-1                                    c
c**********************************************************************c
      taer55=0.
      taer=0.
	write (6,*) 'enter v'
      read(iread,*) v
      if(v) 71,10,11
   10 read(iread,*) taer55
      v=exp(-log(taer55/2.7628)/0.79902)
      goto 71
   11 call oda550(iaer,v,taer55)
   71 continue
 
c**********************************************************************c
c xps is the parameter to express the  altitude of target              c
c                                                                      c
c                                                                      c
c                  xps >0. means the target is at the sea level        c
c                                                                      c
c                  xps <0. means you know the altitude of the target   c
c                        expressed in km and you put that value as xps c
c                                                                      c
c                                                                      c
c**********************************************************************c
	write (6,*) 'enter target elevation (in neg. km)'
       read(iread,*) xps
       if (xps.ge.0.) then
	  xps=0.
	  uwus=1.424
	  uo3us=0.344
	  else
	  if (idatm.ne.8) then
             call pressure(uw,uo3,xps)
	     else
             call pressure(uwus,uo3us,xps)
	  endif
       endif
 
c**********************************************************************c
c                                                                      c
c  xpp is the parameter to express the sensor altitude                 c
c                                                                      c
c                                                                      c
c           xpp=-1000 means that the sensor is a board a satellite     c
c           xpp=    0 means that the sensor is at the ground level     c
c                                                                      c
c                                                                      c
c     for aircraft simulations                                         c
c           xpp <0 means you know the altitude of the sensor expressed c
c                  in kilometers units                                 c
c           xpp >0 means you know the altitude of the sensor expressed c
c                  in pressure units (mb)                              c
c                                                                      c
c     for aircraft simulations only, you have to give                  c
c	puw,po3   (water vapor content,ozone content between the       c
c                  aircraft and the surface)                           c
c	taerp     (the aerosol optical thickness at 550nm between the  c
c                  aircraft and the surface)                           c
c    if these data are not available, enter negative values for all    c
c    of them, puw,po3 will then be interpolated from the us62 standard c
C    profile according to the values at ground level. Taerp will be    c
c    computed according to a 2km exponential profile for aerosol.      c
c**********************************************************************c
	write (6,*) 'enter sensor elevation (-1000 for satellite)'
        read(iread,*) xpp
        if ((xpp.gt.-0.001).and.(xpp.le.0)) then
         palt=0.
         pps=p(1)
	 idatmp=0
         else
          xpp=-xpp
	 if (xpp.gt.100.) then
	    palt=1000.
	    pps=0.
	    taer55p=taer55
	    ftray=1.
	    idatmp=4
	  else
            read(iread,*) puw,puo3
	    if (puw.lt.0.) then
               call presplane(puw,puo3,xpp,ftray)
               puw=puw*uw/uwus
               puo3=puo3*uo3/uo3us
	       idatmp=2
	       else
	       call presplane(puwus,puo3us,xpp,ftray)
	       idatmp=8
            endif
	    palt=zpl(34)
	    pps=ppl(34)
            read(iread,*) taer55p
	    if (taer55p.lt.0.) then
c a scale heigh of 2km is assumed in case no value is given for taer55p
               taer55p=taer55*(1.-exp(-palt/2.))
               else
C compute effective scale heigh
               sham=exp(-palt/4.)
               sha=1.-(taer55p/taer55)
               if (sha.ge.sham) then
                  taer55p=taer55*(1.-exp(-palt/4.))
                  else
                  sha=-palt/log(sha)
                  taer55p=taer55*(1.-exp(-palt/sha))
                  endif
            endif
         endif
      endif
 
c**********************************************************************c
c      iwave input of the spectral conditions                          c
c            --------------------------------                          c
c                                                                      c
c  you choose to define your own spectral conditions: iwave=-1,0 or 1  c
c                   (three user s conditions )                         c
c        -1  enter wl (monochr. cond,  gaseous absorption is included) c
c                                                                      c
c         0  enter wlinf, wlsup. the filter function will be equal to 1c
c            over the whole band.                                      c
c                                                                      c
c         1  enter wlinf, wlsup and user's filter function s(lambda)   c
c                          ( by step of 0.0025 micrometer).            c
c                                                                      c
c                                                                      c
c   or you select one of the following satellite spectral bands:       c
c                                                iwave=2 to 21         c
c         2  vis band of meteosat     ( 0.350-1.110 )                  c
c         3  vis band of goes east    ( 0.490-0.900 )                  c
c         4  vis band of goes west    ( 0.490-0.900 )                  c
c         5  1st band of avhrr(noaa6) ( 0.500-0.740 )                  c
c         6  2nd      "               ( 0.690-1.080 )                  c
c         7  1st band of avhrr(noaa7) ( 0.500-0.740 )                  c
c         8  2nd      "               ( 0.690-1.080 )                  c
c         9  1st band of avhrr(noaa8) ( 0.500-0.740 )                  c
c        10  2nd      "               ( 0.690-1.080 )                  c
c        11  1st band of avhrr(noaa9) ( 0.500-0.740 )                  c
c        12  2nd      "               ( 0.690-1.080 )                  c
c        13  1st band of avhrr(noaa10 ( 0.500-0.740 )                  c
c        14  2nd      "               ( 0.690-1.080 )                  c
c        15  1st band of avhrr(noaa11 ( 0.500-0.740 )                  c
c        16  2nd      "               ( 0.690-1.080 )                  c
c        17  1st band of hrv1(spot1)  ( 0.460-0.710 )                  c
c        18  2nd      "               ( 0.590-0.760 )                  c
c        19  3rd      "               ( 0.740-0.950 )                  c
c        20  pan      "               ( 0.460-0.950 )                  c
c        21  1st band of hrv2(spot1)  ( 0.460-0.710 )                  c
c        22  2nd      "               ( 0.590-0.760 )                  c
c        23  3rd      "               ( 0.740-0.950 )                  c
c        24  pan      "               ( 0.460-0.950 )                  c
c        25  1st band of tm(landsat5) ( 0.430-0.550 )                  c
c        26  2nd      "               ( 0.500-0.650 )                  c
c        27  3rd      "               ( 0.590-0.750 )                  c
c        28  4th      "               ( 0.730-0.945 )                  c
c        29  5th      "               ( 1.515-1.870 )                  c
c        30  7th      "               ( 1.975-2.405 )                  c
c        31  1st band of mss(landsat5)( 0.475-0.640 )                  c
c        32  2nd      "               ( 0.580-0.750 )                  c
c        33  3rd      "               ( 0.655-0.855 )                  c
c        34  4th      "               ( 0.785-1.100 )                  c
c  note: wl has to be in micrometer                                    c
c**********************************************************************c
      do 38 l=1,1501
      s(l)=1.
   38 continue
	write (6,*) 'enter iwave'
      read(iread,*) iwave
      if (iwave) 16,17,18
   16 read(iread,*) wl
      wlinf=wl
      wlsup=wl
      go to 19
   17 read(iread,*) wlinf,wlsup
      go to 19
   18 goto (110,111,112,112,114,114,114,114,114,114,114,114
     s      ,114,114,114,114,118,118,118,118,118,118,118,118
     s     ,121,121,121,121,121,121,127,127,127,127),iwave
  110 read(iread,*) wlinf,wlsup
      iinf=(wlinf-.245)/0.0025+0.5
      isup=(wlsup-.245)/0.0025+0.5
      do 1113 ik=1,1501
      s(ik)=0.
 1113 continue
      read(iread,*) (s(i),i=iinf,isup)
      goto 20
  111 call meteo
      go to 19
  112 call goes(iwave-2)
      go to 19
  114 call avhrr(iwave-4)
      go to 19
  118 call hrv(iwave-16)
      go to 19
  121 call tm(iwave-24)
      go to 19
  127 call mss(iwave-30)
   19 iinf=(wlinf-.245)/0.0025+0.5
      isup=(wlsup-.245)/0.0025+0.5
   20 continue
 
 
c**********************************************************************c
c here, we first compute an equivalent wavelenght which is the input   c
c value for monochromatic conditions or the integrated value for a     c
c filter functionr (vall equivwl) then, the atmospheric properties are c
c computed for that wavelength (call discom then call specinterp)      c
c molecular optical thickness is computed too (call odrayl). lastly    c
c the successive order of scattering code is called three times.       c
c first for a sun at thetas with the scattering properties of aerosols c
c and molecules, second with a pure molecular atmosphere, then with thec
c actual atmosphere for a sun at thetav. the iso code allows us to     c
c compute the scattering transmissions and the spherical albedo. all   c
c these computations are performed for checking the accuracy of the    c
c analytical expressions and in addition for computing the averaged    c
c directional reflectances                                             c
c**********************************************************************c
      if(iwave.ne.-1) then
      call equivwl(iinf,isup,step,
     s               wlmoy)
      else
      wlmoy=wl
      endif
      call discom (idatmp,iaer,xmus,xmuv,phi
     a            ,taer55,taer55p,palt,
     a           phirad,nt,mu,np,rm,gb,rp
     s         ,ftray,xlm1,xlm2)
      if(iaer.ne.0) then
      call specinterp(wlmoy,taer55,taer55p,
     s     tamoy,tamoyp,pizmoy,pizmoyp)
      endif
      call odrayl(wlmoy,
     s                   trmoy)
      trmoyp=trmoy*ftray
      if (idatmp.eq.4) then
          trmoyp=trmoy
          tamoyp=tamoy
          endif
      if (idatmp.eq.0) then
         trmoyp=0.
         tamoyp=0.
         endif
 
c*********************************************************************c
c     inhomo        ground reflectance (type)                          c
c                   ------------------                                 c
c                                                                      c
c  you consider an homogeneous surface:                                c
c     enter - inhomo=0                                                 c
c                you may consider directional surface  effects         c
c                  idirec=0 (no directional effect)                    c
c                          you have to specify the surface reflectance:c
c                          igroun  (see note1) which is uniform and    c
c                          lambertian                                  c
c                  idirec=1 ( directional effect)                      c
c                          you have to specify the brdf of the surface c
c                           for the actual solar illumination you  are c
c                           considering as well as the brdf for a sun  c
c                           which would be at an angle thetav, in      c
c                           addition you have to give the surface      c
c                           albedo (spherical albedo). you can also    c
c                           select one of the selected model from the  c
c                           ibrdf value (see note2). 3 reflectances    c
c                           are computed, robar,robarp and robard      c
c                                                                      c
c  you consider a non uniform surface, the surface is considered as a  c
c            circular target with a reflectance roc and of radius r    c
c            (expressed in km) within an environment of reflectance    c
c            roe                                                       c
c     enter - inhomo=1, then                                           c
c             igrou1,igrou2,rad                                        c
c                  - the target reflectance :igrou1  (see note)        c
c                  - the envir. reflectance :igrou2  (see note)        c
c                  - the target radius in km                           c
c                                                                      c
c                                                                      c
c                            ****tree****                              c
c                                                                      c
c                               inhomo                                 c
c                             /          \                             c
c                            /            \                            c
c                           /              \                           c
c                          /                \                          c
c                 ------- 0 -------       -----1 -----                 c
c                        /               /   \       \                 c
c                    idirect            /     \       \                c
c                    /  \              /       \       \               c
c                   /    \            /         \       \              c
c                  /      \       igrou1       igrou2    rad           c
c                 0        1        roc          roe     f(r)          c
c                /          \                                          c
c               /            \                                         c
c           igroun          ibrdf                                      c
c        (roc = roe)        (roc)                                      c
c                           (robar)                                    c
c                           (robarp)                                   c
c                           (robard)                                   c
c                                                                      c
c                   ground reflectance (spectral variation)            c
c                   ---------------------------------------            c
c note1: values of the reflectance selected by igroun,igrou1 or igrou2 c
c        may correspond to the following cases,                        c
c         0  constant value of ro (or roc,or roe) whatever the wavelen c
c            gth. you enter this constant value of ro (or roc or roe). c
c        -1  you have to enter the value of ro (or roc,or roe) by step c
c            of 0.0025 micron from wlinf to wlsup (if you have used thec
c            satellite bands,see implicit values for these limits).    c
c         1  mean spectral value of green vegetation                   c
c         2  mean spectral value of clear water                        c
c         3  mean spectral value of sand                               c
c         4  mean spectral value of lake water                         c
c                                                                      c
c                       ground reflectance (brdf)                      c
c                       -------------------------                      c
c note2: values of the directional reflectance is assumed spectrally   c
c        independent, so you have to specify, the brdf at the          c
c        wavelength for monochromatic condition of the mean value      c
c        over the spectral band                                        c
c         0  you have to enter the value of ro for sun at thetas by    c
c            step of 10 degrees for zenith view  angles (from 0 to 80  c
c            and the value for 85) and by step of 30 degrees for       c
c            azimuth view angles from 0 to 360 degrees, you have to do c
c            same for a sun which would be at thetav. in addition, the c
c            spherical albedo of the surface has to be specified       c
c        you also may select one of the following models               c
c         1  hapke model                                               c
c             the parameters are:                                      c
c              par1=om  (albedo)
c              par2=af assymetry parameter for the phase function
c              par3=s0 amplitude of hot spot
c              par4=h width of the hot spot
c
c         2  verstraete et al. model                                   c
c  references:
c
c  1. verstraete, m. m., b. pinty and r. e. dickinson (1990) a physical
c     model of the bidirectional reflectance of vegetation canopies,
c     part 1: theory, journal of geophysical research, 95, 11755-11765.
c  2. pinty, b., m. m. verstraete and r. e. dickinson (1990) a physical
c     model of the bidirectional reflectance of vegetation canopies,
c     part 2: inversion and validation, journal of geophysical
c     research, 95, 11767-11775.
c  3. dickinson, r. e., b. pinty and m. m. verstraete (1990) relating
c     surface albedos in gcm to remotely sensed data, agricultural and
c     forest meteorology, 52, 109-131.
c  4. pinty, b. and m. m. verstraete (1991) extracting information on
c     surface properties from bidirectional reflectance measurements,
c     journal of geophysical research, 96, 2865-2874.
c  5. press, w. h., et al. (1986) numerical recipes: the art of
c     scientific computing, cambridge university press, 818 pp.
c
c  copies of the first four papers can be obtained from the authors.
c
c             the parameters are:                                      c
c   there is three lines of parameters: line 1 (choice of options)
c                                       line 2 (structural parameters)
c                                       line 3 (optical parameters)
c line 1:  opt3 opt4 opt5
c   opt1=1   parametrized     model (see ref2)
c
c   opt2=1   reflectance factor (see ref1)
c
c   opt3=    0 for given values of kappa (see struct below)
c            1 for goudriaan's parameterization of kappa
c            2 for dickinson et al's correction to goudriaan's
c              parameterization of kappa (see ref. 3)
 
c   opt4=    0 for isotropic phase function
c            1 for heyney and greensteins' phase function
c            2 for legendre polynomial phase function
c
c   opt5=    0 for single scattering only
c            1 for dickinson et al. parameterization of multiple
c              scattering
c
c line 2:  str1 str2 str3 str4
c
c   str1=    'leaf area density', in m2 m-3
c   str2=    radius of the sun flecks on the scatterer, in m
c   str3=    leaf orientation parameter:
c            if opt3=0 then str3=kappa1
c            if opt3=1 or 2  then str3=chil
c   str4=    leaf orientation parameter (continued):
c            if opt3=0 then str4=kappa2
c            if opt3=1 or 2 then str4 is not used
c
c line 3:  optics1 optics2 optics3
c   optics1= single scattering albedo, n/d value between 0.0
c            and 1.0
c   optics2= phase function parameter:
c            if opt4=0 then this input is not used
c            if opt4=1 then asymmetry factor, n/d value between -1.0
c                           and 1.0
c            if opt4=2 then first coefficient of legendre polynomial
c   optics3= second coefficient of legendre polynomial (if opt4=2)
c
c         3  Roujean et al. model                                      c
c             the parameters are:                                      c
c                 par1: k0 (albedo..)
c                 par2: k1 (geometric parameter for hot spot effect
c                 par3: k2 (geometric parameter for hot spot effect
c
c         4  walthall et al. model                                     c
c             the parameters are:                                      c
c             par1: a (term in square ts*tv
c             par2: ap term in square ts*ts+tv*tv
c             par3: b term in ts*tv*cos(phi) (limacon de pascal)
c             par4: albedo
 
c         5  minnaert model                                            c
c             the parameters are:                                      c
c              par1: ??
c              par2: ??
 
c         6   Ocean
c             the parameter is:
c              pws: wind speed in meter per second
c              pcl: chlorophyl content in mg/m3
c
c         7   Iaquinta and Pinty model
c             Reference:
c             Iaquinta J. and B. Pinty (1994) Adaptation of a          c
c bidirectional reflectance model including the hot-spot to an         c
c optically thin canopy. Proc. Sixth International Symposium on        c
c Physical Measurements and Signatures in Remote Sensing, in press.    c
c             the parameters are:                                      c
c             line 1: choice of option (pild,pihs)                     c
c             line 2: structural parameters (pxLt,pc)                  c
c             line 3: optical    parameters (pRl,pTl,pRs)              c
c                                                                      c 
c                                                                      c 
c  Line 1: pild,pihs                                                   c
c                                                                      c 
c          pild=1     planophile   leaf distribution                   c 
c          pild=2     erectophile  leaf distribution                   c 
c          pild=3     plagiophile  leaf distribution                   c 
c          pild=4     extremophile leaf distribution                   c 
c          pild=5     uniform      leaf distribution                   c 
c                                                                      c 
c          pihs=0     no hot spot                                      c 
c          pihs=1        hot spot                                      c 
c                                                                      c 
c                                                                      c 
c                                                                      c 
c  Line 2: pxLt,pc                                                     c
c                                                                      c 
c          pxLt      Leaf area index [1.,15.]                          c 
c          pc        Hot spot parameter: 2*r*Lambda [0.,2.]            c
c                                                                      c 
c                                                                      c 
c                                                                      c 
c  Line 3: pRl,pTl,pRs                                                 c
c                                                                      c 
c          pRl       Leaf reflectance  [0.,0.99]                       c 
c          pTl       Leaf transmitance [0.,0.99]                       c 
c          pRs       Soil albedo       [0.,0.99]                       c 
c                                                                      c 
c          NB: pRl+PTl <0.99                                           c 
c**********************************************************************c
									
      fr=0.
      rad=0.
      do 1116 ik=1,1501
      rocl(ik)=0.
      roel(ik)=0.
 1116 continue
 
c**********************************************************************c
c     uniform or non-uniform surface conditions                        c
c**********************************************************************c
	write (6,*) 'enter inhomo (0 for uniform)'
      read(iread,*) inhomo
      if(inhomo) 30,30,31
  30  	write (6,*) 'enter idirec (0 for lambertian)'
	read(iread,*) idirec
      if(idirec)21,21,25
 
c**********************************************************************c
c     uniform conditions with brdf conditions                          c
c**********************************************************************c
c call os to compute downward radiation field for robar
 25   rm(-mu)=-xmuv
      rm(mu)=xmuv
      rm(0)=-xmus
      spalt=1000.
      call os(tamoy,trmoy,pizmoy,tamoyp,trmoyp,spalt,
     s               phirad,nt,mu,np,rm,gb,rp,
     s                     xlmus)
c call os to compute downward radiation field for robarp
      if (idatmp.ne.0) then
      rm(-mu)=-xmus
      rm(mu)=xmus
      rm(0)=-xmuv
      call os(tamoyp,trmoyp,pizmoy,tamoyp,trmoyp,spalt,
     s               phirad,nt,mu,np,rm,gb,rp,
     s                     xlmuv)
c
      endif
c
	write (6,*) 'enter ibrdf'
      read(iread,*) ibrdf
c*********************************************************************c
      if(ibrdf)23,23,24
c**********************************************************************c
c     brdf from in-situ measurements                                   c
c**********************************************************************c
  23  do 900 k=1,13
      do 900 j=1,10
      read(iread,*) brdfdats(j,k)
  900 continue
      do 901 k=1,13
      do 901 j=1,10
      read(iread,*) brdfdatv(j,k)
  901 continue
      read(iread,*) albbrdf
      rm(-mu)=phirad
      rm(mu)=xmuv
      rm(0)=xmus
      call brdfgrid(mu,np,rm,rp,brdfdats,angmu,angphi,
     s                 brdfints)
      rm(-mu)=2.*pi-phirad
      rm(mu)=xmus
      rm(0)=xmuv
      call brdfgrid(mu,np,rm,rp,brdfdatv,angmu,angphi,
     s                 brdfintv)
      go to 69
c**********************************************************************c
c     brdf from hapke's model                                          c
c**********************************************************************c
  24  if(ibrdf.eq.1) then
      read(iread,*) par1,par2,par3,par4
      rm(-mu)=phirad
      rm(mu)=xmuv
      rm(0)=xmus
      call hapkbrdf(par1,par2,par3,par4,mu,np,rm,rp,
     s           brdfints)
      rm(-mu)=2.*pi-phirad
      rm(mu)=xmus
      rm(0)=xmuv
      call hapkbrdf(par1,par2,par3,par4,mu,np,rm,rp,
     s           brdfintv)
      call hapkalbe(par1,par2,par3,par4,
     s       albbrdf)
      go to 69
      endif
c**********************************************************************c
c     brdf from verstraete et al's model                               c
c**********************************************************************c
      if(ibrdf.eq.2) then
	write (6,*) 'enter michels options 3 and 5'
      read(iread,*) (options(i),i=3,5)
      options(1)=1
      options(2)=1
	
	write (6,*) 'enter 4 structural parameters'
      read(iread,*) (struct(i),i=1,4)
	write (6,*) 'enter 3 optical parameters'
      read(iread,*) (optics(i),i=1,3)
      rm(-mu)=phirad
      rm(mu)=xmuv
      rm(0)=xmus
      call versbrdf(options,optics,struct,mu,np,rm,rp,
     s           brdfints)
      rm(-mu)=2.*pi-phirad
      rm(mu)=xmus
      rm(0)=xmuv
      call versbrdf(options,optics,struct,mu,np,rm,rp,
     s           brdfintv)
      call versalbe(options,optics,struct,
     s       albbrdf)
      go to 69
      endif
c**********************************************************************c
c     brdf from Roujean et al's model                                  c
c**********************************************************************c
      if(ibrdf.eq.3) then
      read(iread,*) par1,par2,par3
      rm(-mu)=phirad
      rm(mu)=xmuv
      rm(0)=xmus
      call roujbrdf(par1,par2,par3,mu,np,rm,rp,
     s           brdfints)
      rm(-mu)=2.*pi-phirad
      rm(mu)=xmus
      rm(0)=xmuv
      call roujbrdf(par1,par2,par3,mu,np,rm,rp,
     s           brdfintv)
      call roujalbe(par1,par2,par3,
     s       albbrdf)
      go to 69
      endif
c**********************************************************************c
c     brdf from walthall et al's model
c**********************************************************************c
      if(ibrdf.eq.4) then
      read(iread,*) par1,par2,par3,par4
      rm(-mu)=phirad
      rm(mu)=xmuv
      rm(0)=xmus
      call waltbrdf(par1,par2,par3,par4,mu,np,rm,rp,
     s           brdfints)
      rm(-mu)=2.*pi-phirad
      rm(mu)=xmus
      rm(0)=xmuv
      call waltbrdf(par1,par2,par3,par4,mu,np,rm,rp,
     s           brdfintv)
      call waltalbe(par1,par2,par3,par4,
     s       albbrdf)
      go to 69
      endif
c**********************************************************************c
c     brdf from minnaert's model                                       c
c**********************************************************************c
      if(ibrdf.eq.5) then
      read(iread,*) par1,par2
      rm(-mu)=phirad
      rm(mu)=xmuv
      rm(0)=xmus
      call minnbrdf(par1,par2,mu,np,rm,
     s           brdfints)
      rm(-mu)=2.*pi-phirad
      rm(mu)=xmus
      rm(0)=xmuv
      call minnbrdf(par1,par2,mu,np,rm,
     s           brdfintv)
      call minnalbe(par1,par2,
     s       albbrdf)
      go to 69
      endif
 
c**********************************************************************c
c     brdf from ocean condition
c**********************************************************************c
      if(ibrdf.eq.6) then
      read(iread,*) pws,pcl
      rm(-mu)=phirad
      rm(mu)=xmuv
      rm(0)=xmus
      call oceabrdf(pws,pcl,wlmoy,mu,np,rm,rp,
     s           brdfints)
      rm(-mu)=2.*pi-phirad
      rm(mu)=xmus
      rm(0)=xmuv
      call oceabrdf(pws,pcl,wlmoy,mu,np,rm,rp,
     s           brdfintv)
      call oceaalbe(pws,pcl,wlmoy,
     s       albbrdf)
      go to 69
      endif
c
c**********************************************************************c
c     brdf from Iaquinta and Pinty model
c**********************************************************************c
      if(ibrdf.eq.7) then
	write (6,*) 'enter pild, pihs'
      read(iread,*) pild,pihs
	write (6,*) 'enter pxLt,pc'
      read(iread,*) pxLt,pc
	write (6,*) 'enter pRl,pTl,pRs'
      read(iread,*) pRl,pTl,pRs
      rm(-mu)=phirad
      rm(mu)=xmuv
      rm(0)=xmus
      call iapibrdf(pild,pxlt,prl,ptl,prs,pihs,pc,mu,np,rm,rp,
     s           brdfints)
      rm(-mu)=2.*pi-phirad
      rm(mu)=xmus
      rm(0)=xmuv
      call iapibrdf(pild,pxlt,prl,ptl,prs,pihs,pc,mu,np,rm,rp,
     s           brdfintv)
      call iapialbe(pild,pxlt,prl,ptl,prs,pihs,pc,
     s       albbrdf)
      go to 69
      endif
c
   69 continue
c**********************************************************************c
c the downward irradiance was computed for a sun at thetas and         c
c several viewing directions (mu zenith times np azimuth). then, the   c
c code computes the product of ldown*brdf integrated over the total    c
c hemisphere and gives the averaged directional reflectance after the  c
c normalization. the resulting reflectance is named robar              c
c**********************************************************************c
      robar1=0.
      xnorm1=0.
      do 83 j=1,np
      rob=0.
      xnor=0.
      do 84 k=1,mu-1
      rdown=xlmus(-k,j)
      rdir=brdfintv(k,j)
      rob=rob+rdown*rdir*rm(k)*gb(k)
      xnor=xnor+rdown*rm(k)*gb(k)
   84 continue
      robar1=robar1+rob*gp(j)
      xnorm1=xnorm1+xnor*gp(j)
   83 continue
 
c**********************************************************************c
c the downward irradiance was computed for a sun at thetav and         c
c several viewing directions (mu zenith times np azimuth). then, the   c
c code computes the product of ldown*brdf integrated over the total    c
c hemisphere and gives the averaged directional reflectance after the  c
c normalization. the resulting reflectance is named robarp             c
c**********************************************************************c
      robar2=0.
      xnorm2=0.
      do 85 j=1,np
      rob=0.
      xnor=0.
      do 86 k=1,mu-1
      rdown=xlmuv(-k,j)
      rdir=brdfints(k,j)
      rob=rob+rdown*rdir*rm(k)*gb(k)
      xnor=xnor+rdown*rm(k)*gb(k)
   86 continue
      robar2=robar2+rob*gp(j)
      xnorm2=xnorm2+xnor*gp(j)
   85 continue
 
c  robard is assumed equal to albbrdf
c       print 301,brdfints(mu,1),robar1,xnorm1,
c    s       robar2,xnorm2,albbrdf
c       print 301,robar1/xnorm1,robar2/xnorm2
c       print 301,betal(0)/3,pizmoy
c301  format(6(f10.4,2x))
c501  format(5(i10,2x))
      do 335 l=1,1501
      rocl(l)=brdfints(mu,1)
      roel(l)=brdfints(mu,1)
      robar(l)=robar1/xnorm1
      if (idatmp.ne.0) then
          robarp(l)=robar2/xnorm2
          else
          robarp(l)=0.
          xnorm2=1.
          robar2=0.
          endif
      robard(l)=albbrdf
  335 continue
      go to 34
 
c**********************************************************************c
c     uniform surface with lambertian conditions                       c
c**********************************************************************c
  21  write (6,*) 'enter iground (enter 0)'
	read(iread,*) igroun
      if(igroun) 29,32,33
  29  read(iread,*) (rocl(i),i=iinf,isup)
      goto 36
  32  write (6,*) 'enter ro'
	read(iread,*) ro
      do 35 l=1,1501
      rocl(l)=ro
   35 continue
      goto 36
  33  if(igroun.eq.1) call vegeta(rocl)
      if(igroun.eq.2) call clearw(rocl)
      if(igroun.eq.3) call sand  (rocl)
      if(igroun.eq.4) call lakew (rocl)
   36 do 39 l=1,1501
      roel(l)=rocl(l)
   39 continue
      go to 34
 
c**********************************************************************c
c     non-uniform conditions with lambertian conditions                c
c**********************************************************************c
 31   read(iread,*) igrou1,igrou2,rad
      if(igrou1) 59,60,63
  59  read(iread,*) (rocl(i),i=iinf,isup)
      goto 61
  60  read(iread,*) roc
      do 64 l=1,1501
      rocl(l)=roc
   64 continue
      go to 61
  63  if(igrou1.eq.1) call vegeta(rocl)
      if(igrou1.eq.2) call clearw(rocl)
      if(igrou1.eq.3) call sand  (rocl)
      if(igrou1.eq.4) call lakew (rocl)
   61 if(igrou2) 66,62,65
  66  read(iread,*) (roel(i),i=iinf,isup)
      goto 34
  62  read(iread,*) roe
      do 45 l=1,1501
      roel(l)=roe
   45 continue
      go to 34
  65  if(igrou2.eq.1) call vegeta(roel)
      if(igrou2.eq.2) call clearw(roel)
      if(igrou2.eq.3) call sand  (roel)
      if(igrou2.eq.4) call lakew (roel)
   34 continue
 
 
c**********************************************************************c
c                                                                      c
c       rapp    that input parameter allows to activate atmospheric    c
c               correction mode                                        c
c                                                                      c
c               if rapp <-1. no atmospheric correction is performed    c
c                                                                      c
c               if rapp >0. :  the code retrieve the rog value that    c
c               will produce a radiance equal to rapp [w/m2/str/mic]   c
c               in the atmospheric conditions described by user before c
c                                                                      c
c                                                                      c
c               if -1.<rapp<0. : the code retrieve the rog value that  c
c               will produce a 'reflectance' (radiance*pi/(mus*es))    c
c               equal to -rapp where mus is the cosine of solar zenith c
c               angle, es is the solar constant integrated upon the    c
c               filter response and taking account for earth-solar     c
c               distance es is in [w/m2/sr/mic].                       c
c                                                                      c
c**********************************************************************c

	write (6,*) 'enter rapp (-2)' 
      read(iread,*) rapp
 
c**********************************************************************c
c**********************************************************************c
c                                                                      c
c                     example of input cards                           c
c                                                                      c
c 4                            (avhrr observation)                     c
c 7 6   10.1  600  0.0  10.0   (month,day, htu, cn, longan, han)       c
c                                                                      c
c 8                            (user's   model)                        c
c 3.0   0.35                   ( uh2o(g/cm2) ,uo3(cm-atm) )            c
c 4                            (aerosols model)                        c
c 0.25  0.25  0.25  0.25       ( % of:dust-like,water-sol,oceanic,soot)c
c 23.0                         (visibility (km) )                      c
c 6                            (avhrr 2 (noaa 8) band)                 c
c 1                            (ground type,i.e. inho)                 c
c 2    1    0.50               (target,env.,radius(km) )               c
c 1                            (test assumed)                          c
c 0.10                       (atmospheric correction of rapp=0.10      c
c                                                                      c
c**********************************************************************c
 
 
c**********************************************************************c
c                     print of initial conditions                      c
c                                                                      c
c**********************************************************************c
 
c     geometrical conditions
      write(iwr, 98)
      write(iwr, etiq1(igeom+1))
      if(igeom.eq.0) then
	 write(iwr, 1401)
	 write(iwr, 103)month,jday
	 endif
      if(igeom.ne.0) write(iwr, 101)month,jday,tu,xlat,xlon
      write(iwr, 102)asol,phi0
      write(iwr, 1110)avis,phiv,adif,phi
 
c     atmospheric model
      write(iwr, 1119)
      if(idatm-7)226,227,228
  228 write(iwr, 1281)uw,uo3
      goto 219
  227 write(iwr, 1272)
      do 229 i=1,34
      write(iwr, 1271)z(i),p(i),t(i),wh(i),wo(i)
  229 continue
      goto 219
  226 write(iwr, 1261)atmid(idatm+1)
 
c     aerosols model (type)
 219  if (iaer.lt.4) then
          goto(230,231,232,233),iaer+1
          else
          if (iaer.ge.5) goto(234),iaer-4
          write(iwr, 134)(c(i),i=1,4)
          goto 249
          endif
  234 write(iwr, 131)' desertic'
      goto 249
  233 write(iwr, 131)' urban'
      go to 249
  232 write(iwr, 131)'maritime'
      goto 249
  231 write(iwr, 131)'continental'
      goto 249
  230 write(iwr, 1301)
  249 continue
 
c     aerosol model (concentration)
      if(iaer.eq.0) write(iwr, 1401)
      if(iaer.eq.0) goto 1112
      if(abs(v).le.xacc) write(iwr, 140)taer55
      if(abs(v).gt.xacc) write(iwr, 141)v,taer55
 
c     spectral condition
 1112 write(iwr, 148)
 
      if(iwave.eq.-1) write(iwr, 149) wl
      if(iwave.ne.-1) write(iwr, 1510) nsat(iwave+1), wlinf,wlsup
 
c     ground reflectance (type and spectral variation)
      if(idirec.eq.0) then
      rocave=0.
      roeave=0.
      seb=0.
 
      do 264 i=iinf,isup
      sbor=s(i)
      if(i.eq.iinf.or.i.eq.isup) sbor=sbor*0.5
      wl=.245+i*step
      call solirr(wl,
     1            swl)
      swl=swl*dsol
      rocave=rocave+rocl(i)*sbor*swl*step
      roeave=roeave+roel(i)*sbor*swl*step
      seb=seb+sbor*swl*step
  264 continue
      rocave=rocave/seb
      roeave=roeave/seb
      isort=0
      ro=rocave
 
      if(inhomo.eq.0) goto 260
      write(iwr, 169)rad
      igroun=igrou1
      ro=rocave
      write(iwr, 170)
      goto 261
 
  262 igroun=igrou2
      ro=roeave
      write(iwr, 171)
      goto 261
 
  260 write(iwr, 168)
  261 if (igroun.gt.0)write(iwr, reflec(igroun+3))ro
      if (igroun.gt.0)goto 158
      if(igroun.eq.-1) write(iwr, reflec(1))ro
      if(igroun.eq.-1) goto 158
      if(iwave.eq.-1)  write(iwr, reflec(2))ro
      if(iwave.ne.-1)  write(iwr, reflec(3))ro
 158  isort=isort+1
      if(inhomo.eq.0) goto 999
      if(isort.eq.2) goto 999
      goto 262
      else
      if(ibrdf.ge.1) then
         goto(2001,2002,2003,2004,2005,2006,2007),ibrdf
 2001    write(iwr, 191)par1,par2,par3,par4
         write(iwr, 187)
     *brdfints(mu,1),robar1/xnorm1,robar2/xnorm2,albbrdf
         goto 2008
 2002    write(iwr, 192)optics(1),struct(1),struct(2)
         if (options(5).eq.0) write(iwr, 200)
         if (options(5).eq.1) write(iwr, 201)
         if (options(3).eq.0) write(iwr, 197)struct(3),struct(4)
         if (options(3).eq.1) write(iwr, 198)struct(3)
         if (options(3).eq.2) write(iwr, 199)struct(3)
         if (options(4).eq.0) write(iwr, 202)
         if (options(4).eq.1) write(iwr, 203)optics(2)
         if (options(4).eq.2) write(iwr, 204)optics(2),optics(3)
         write(iwr, 187)
     *brdfints(mu,1),robar1/xnorm1,robar2/xnorm2,albbrdf
         goto 2008
 2003    write(iwr, 193)par1,par2,par3
         write(iwr, 187)
     *brdfints(mu,1),robar1/xnorm1,robar2/xnorm2,albbrdf
         goto 2008
 2004    write(iwr, 194)par1,par2,par3,par4
         write(iwr, 187)
     *brdfints(mu,1),robar1/xnorm1,robar2/xnorm2,albbrdf
         goto 2008
 2005    write(iwr, 195)par1,par2,par3,par4
         write(iwr, 187)
     *brdfints(mu,1),robar1/xnorm1,robar2/xnorm2,albbrdf
         goto 2008
 2006    write(iwr, 196)pws,pcl
         write(iwr, 187)
     *brdfints(mu,1),robar1/xnorm1,robar2/xnorm2,albbrdf
         goto 2008
 2007    write(iwr, 205) pRl,pTl,pRs,PxLt
         if (pihs.eq.0) then
            write(iwr,*) '*  no hot spot'
            else
            write(iwr,*) '*  hot spot parameter',pc
            endif
         if (pild.eq.1) write(iwr,*) '*  planophile   leaf distribution'
         if (pild.eq.2) write(iwr,*) '*  erectophile  leaf distribution'
         if (pild.eq.3) write(iwr,*) '*  plagiophile  leaf distribution'
         if (pild.eq.4) write(iwr,*) '*  extremophile leaf distribution'
         if (pild.eq.5) write(iwr,*) '*  uniform      leaf distribution'
         write(iwr, 187)
     *brdfints(mu,1),robar1/xnorm1,robar2/xnorm2,albbrdf
         goto 2008
 2008    endif
      endif
  191 format(1h*,2x,41h hapke model selected for ground boundary,t79,1h*
     s       /,1h*,12x,3hom:,f5.3,1x,3haf:,f5.3,1x,3hs0:,f5.3,1x,
     s       2hh:,f5.3)
  192 format(1h*,2x,41h pinty model selected for ground boundary,t79,1h*
     s       /,1h*,12x,3hom:,f5.3,1x,5hrad :,f5.3,1x,6hlad  :,f5.3,1x,
     s        t79,1h*)
  193 format(1h*,2x,41h Roujean mod. select. for ground boundary,t79,1h*
     s       /,1h*,12x,3hk0:,f5.3,1x,3hk1:,f5.3,1x,3hk2:,f5.3,
     s       t79,1h*)
  194 format(1h*,2x,41h walthall model       for ground boundary,t79,1h*
     s       /,1h*,12x,2ha:,f5.3,1x,3hap:,f5.3,1x,2hb:,f5.3,1x,
     s       3hom:,f5.3,t79,1h*)
  195 format(1h*,2x,41h minnaert model       for ground boundary,t79,1h*
     s       /,1h*,12x,5hpar1:,f5.3,1x,5hpar2:,f5.3,t79,1h*)
  196 format(1h*,2x,41h ocean    model       for ground boundary,t79,1h*
     s       /,1h*,12x,16hwind speed [m/s]:,f5.2,
     s             10x,26hchlorophyl content [mg/m3]:,f5.2,t79,1h*)
  197 format(1h*,2x,41h given kappa1 and kappa2:                ,t79,1h*
     s       /,1h*,12x,5hkpa1:,f5.3,1x,5hkpa2:,f5.3,t79,1h*)
  198 format(1h*,2x,41h goudrian's parametrization of kappa :   ,t79,1h*
     s       /,1h*,12x,6h ksil:,f5.3,1x,t79,1h*)
  199 format(1h*,2x,41h modified goudrian's parametrization :   ,t79,1h*
     s       /,1h*,12x,6h ksil:,f5.3,1x,t79,1h*)
  200 format(1h*,2x,40h single scattering only                 ,t79,1h*)
  201 format(1h*,2x,40h multiple scattering (dickinson et al)  ,t79,1h*)
  202 format(1h*,2x,40h isotropic phase function               ,t79,1h*)
  203 format(1h*,2x,40h heyney-greenstein's phase function  :  ,t79,1h*,
     s       /,1h*,12x,6hassym:,f5.3,1x,t79,1h*)
  204 format(1h*,2x,40h legendre polynomial phase function  :  ,t79,1h*,
     s       /,1h*,12x,6hbeta1:,f5.3,1x,6hbeta2:,f5.3,t79,1h*)
  205 format(1h*,2x,41h Iaquinta and Pinty BRDF model          ,t79,1h*,
     s       /,1h*,12x,3hRl:,f5.3,1x,3hTl:,f5.3,1x,3hRs:,f5.3,1x
     s       ,1x,4hLAl:,f5.3,t79,1h*)
 
c pressure at ground level (174) and altitude (175)
  999 write(iwr, 173)
      write(iwr, 174)p(1)
      write(iwr, 175)xps
      if (xps.gt.0.) then
      write(iwr, 176)uw,uo3
      endif
      write(iwr, 172)
 
c plane simulation output if selected
      if (palt.lt.1000.) then
      write(iwr, 178)
      write(iwr, 179)pps
      write(iwr, 180)palt
      write(iwr, 181)
      write(iwr, 182)puo3
      write(iwr, 183)puw
      write(iwr, 184)taer55p
      endif
 
 
c atmospheric correction [yes rapp >-1.] [no rapp <-1.]
      if (rapp.gt.-1.) then
	 write(iwr, 177)
         if (rapp.lt.0.) then
            write(iwr, 185)-rapp
	    else
            write(iwr, 186)rapp
	 endif
      write(iwr, 172)
      endif
c**********************************************************************c
c                                                                      c
c                                                                      c
c                     start of computations                            c
c                                                                      c
c                                                                      c
c                                                                      c
c**********************************************************************c
 
      sb=0.
      seb=0.
      refet=0.
      refet1=0.
      refet2=0.
      refet3=0.
      alumet=0.
      tgasm=0.
      rog=0.
      dgasm=0.
      ugasm=0.
      sdwava=0.
      sdozon=0.
      sddica=0.
      sdoxyg=0.
      sdniox=0.
      sdmoca=0.
      sdmeth=0.
      suwava=0.
      suozon=0.
      sudica=0.
      suoxyg=0.
      suniox=0.
      sumoca=0.
      sumeth=0.
      stwava=0.
      stozon=0.
      stdica=0.
      stoxyg=0.
      stniox=0.
      stmoca=0.
      stmeth=0.
      sodray=0.
      sodrayp=0.
      sodaer=0.
      sodaerp=0.
      sodtot=0.
      sodtotp=0.
      fophsr=0.
      fophsa=0.
      sroray=0.
      sroaer=0.
      srotot=0.
      ssdaer=0.
      sdtotr=0.
      sdtota=0.
      sdtott=0.
      sutotr=0.
      sutota=0.
      sutott=0.
      sasr=0.
      sasa=0.
      sast=0.
      do 52 i=1,2
      do 53 j=1,3
      ani(i,j)=0.
      aini(i,j)=0.
      anr(i,j)=0.
      ainr(i,j)=0.
   53 continue
   52 continue
      do 51 l=iinf,isup
      sbor=s(l)
      if(l.eq.iinf.or.l.eq.isup) sbor=sbor*0.5
      if(iwave.eq.-1) sbor=1.0
      roc=rocl(l)
      roe=roel(l)
      wl=.25+(l-1)*step
c
      call abstra(idatm,wl,xmus,xmuv,uw/2.,uo3,uwus,uo3us,
     i             idatmp,puw/2.,puo3,puwus,puo3us,
     a      adtwava,dtozon,dtdica,dtoxyg,dtniox,dtmeth,dtmoca,
     a      autwava,utozon,utdica,utoxyg,utniox,utmeth,utmoca,
     a      attwava,ttozon,ttdica,ttoxyg,ttniox,ttmeth,ttmoca )
c
      call abstra(idatm,wl,xmus,xmuv,uw,uo3,uwus,uo3us,
     i             idatmp,puw,puo3,puwus,puo3us,
     a      dtwava,dtozon,dtdica,dtoxyg,dtniox,dtmeth,dtmoca,
     a      utwava,utozon,utdica,utoxyg,utniox,utmeth,utmoca,
     a      ttwava,ttozon,ttdica,ttoxyg,ttniox,ttmeth,ttmoca )
 
c
      call solirr(wl,
     s            swl)
      swl=swl*dsol
      if(iwave.eq.-1) swl=1.0
c
      coef=sbor*step*swl
      call interp (iaer,idatmp,wl,taer55,taer55p,xmud,
     s             romix,rorayl,roaero,phaa,phar,tsca,
     s             tray,trayp,taer,taerp,dtott,utott,
     s             astot,asray,asaer,
     s             utotr,utota,dtotr,dtota)
      dgtot=dtwava*dtozon*dtdica*dtoxyg*dtniox*dtmeth*dtmoca
      tgtot=ttwava*ttozon*ttdica*ttoxyg*ttniox*ttmeth*ttmoca
      ugtot=utwava*utozon*utdica*utoxyg*utniox*utmeth*utmoca
      edifr=utotr-exp(-trayp/xmuv)
      edifa=utota-exp(-taerp/xmuv)
      if (ibrdf.ge.1) then
         tdird=exp(-(trayp+taerp)/xmus)
         tdiru=exp(-(trayp+taerp)/xmuv)
         tdifd=dtott-tdird
         tdifu=utott-tdiru
	 roprim=romix+roc*tdird*tdiru+
     s          robar(l)*tdifd*tdiru+robarp(l)*tdifu*tdird+
     s          robard(l)*tdifd*tdifu/(1.-astot*robard(l))
         else
         call enviro(edifr,edifa,rad,palt,
     s               fra,fae,fr)
         avr=roc*fr+(1.-fr)*roe
         roprim=romix
     s       +roc*dtott*exp(-(trayp+taerp)/xmuv)/(1.-avr*astot)
     s       +avr*dtott*(utott-exp(-(trayp+taerp)/xmuv))/(1.-avr*astot)
c       write(iwr, 301)roprim,rorayl,roaero,dtott,avr
c       write(iwr, 301)tgtot,swl,pi,roc,utott,astot
      endif
      romeas1=(roprim-rorayl)*tgtot+rorayl
      romeas2=(romix-rorayl)*tgtot*attwava/ttwava
     s         +(roprim-romix)*tgtot+rorayl
      romeas3=(roprim-romix)*tgtot+romix
      alumeas=xmus*swl*romeas2/pi
      fophsa=fophsa+phaa*coef
      fophsr=fophsr+phar*coef
      sasr=sasr+asray*coef
      sasa=sasa+asaer*coef
      sast=sast+astot*coef
      sroray=sroray+rorayl*coef
      sroaer=sroaer+roaero*coef
      sodray=sodray+tray*coef
      sodaer=sodaer+taer*coef
      sodrayp=sodrayp+trayp*coef
      sodaerp=sodaerp+taerp*coef
      ssdaer=ssdaer+tsca*coef
      sodtot=sodtot+(taer+tray)*coef
      sodtotp=sodtotp+(taerp+trayp)*coef
      srotot=srotot+(romix)*coef
      rog=rog+roc*coef
      refet=refet+romeas2*coef
      refet1=refet1+romeas1*coef
      refet2=refet2+romeas2*coef
      refet3=refet3+romeas3*coef
      alumet=alumet+alumeas*sbor*step
      tgasm=tgasm+tgtot*coef
      dgasm=dgasm+dgtot*coef
      ugasm=ugasm+ugtot*coef
      sdwava=sdwava+dtwava*coef
      sdozon=sdozon+dtozon*coef
      sddica=sddica+dtdica*coef
      sdoxyg=sdoxyg+dtoxyg*coef
      sdniox=sdniox+dtniox*coef
      sdmeth=sdmeth+dtmeth*coef
      sdmoca=sdmoca+dtmoca*coef
      suwava=suwava+utwava*coef
      suozon=suozon+utozon*coef
      sudica=sudica+utdica*coef
      suoxyg=suoxyg+utoxyg*coef
      suniox=suniox+utniox*coef
      sumeth=sumeth+utmeth*coef
      sumoca=sumoca+utmoca*coef
      stwava=stwava+ttwava*coef
      stozon=stozon+ttozon*coef
      stdica=stdica+ttdica*coef
      stoxyg=stoxyg+ttoxyg*coef
      stniox=stniox+ttniox*coef
      stmeth=stmeth+ttmeth*coef
      stmoca=stmoca+ttmoca*coef
      sdtotr=sdtotr+dtotr*coef
      sdtota=sdtota+dtota*coef
      sdtott=sdtott+dtott*coef
      sutotr=sutotr+utotr*coef
      sutota=sutota+utota*coef
      sutott=sutott+utott*coef
      sb=sb+sbor*step
      seb=seb+coef
c
c     output at the ground level.
c
      tdir=exp(-(tray+taer)/xmus)
      tdif=dtott-tdir
      etn=dtott*dgtot/(1.-avr*astot)
      esn=tdir*dgtot
      es=tdir*dgtot*xmus*swl
      ea0n=tdif*dgtot
      ea0=tdif*dgtot*xmus*swl
      ee0n=dgtot*avr*astot*dtott/(1.-avr*astot)
      ee0=xmus*swl*dgtot*avr*astot*dtott/(1.-avr*astot)
      ani(1,1)=esn/etn
      ani(1,2)=ea0n/etn
      ani(1,3)=ee0n/etn
      ani(2,1)=es
      ani(2,2)=ea0
      ani(2,3)=ee0
      do 955 j=1,3
      aini(1,j)=aini(1,j)+ani(1,j)*coef
      aini(2,j)=aini(2,j)+ani(2,j)*sbor*step
  955 continue
 
c     output at satellite level
      tmdir=exp(-(tray+taer)/xmuv)
      tmdif=utott-tmdir
      xla0n=(rorayl+roaero)*tgtot
      xla0=xla0n*xmus*swl/pi
      xltn=roc*dtott*tmdir*tgtot/(1.-avr*astot)
      xlt=xltn*xmus*swl/pi
      xlen=avr*dtott*tmdif*tgtot/(1.-avr*astot)
      xle=xlen*xmus*swl/pi
      anr(1,1)=xla0n
      anr(1,2)=xlen
      anr(1,3)=xltn
      anr(2,1)=xla0
      anr(2,2)=xle
      anr(2,3)=xlt
      do 56 j=1,3
      ainr(1,j)=ainr(1,j)+anr(1,j)*coef
      ainr(2,j)=ainr(2,j)+anr(2,j)*sbor*step
   56 continue
   51 continue
 
c     integrated values of apparent reflectance, radiance
c     and gaseous transmittances (total,downward,separately gases)
      refet=refet/seb
      refet1=refet1/seb
      refet2=refet2/seb
      refet3=refet3/seb
      tgasm=tgasm/seb
      dgasm=dgasm/seb
      ugasm=ugasm/seb
      sasa=sasa/seb
      sasr=sasr/seb
      sast=sast/seb
      sdniox=sdniox/seb
      sdmoca=sdmoca/seb
      sdmeth=sdmeth/seb
      sdwava=sdwava/seb
      sdozon=sdozon/seb
      sddica=sddica/seb
      suniox=suniox/seb
      sumoca=sumoca/seb
      sumeth=sumeth/seb
      suwava=suwava/seb
      suozon=suozon/seb
      sudica=sudica/seb
      suoxyg=suoxyg/seb
      sdoxyg=sdoxyg/seb
      stniox=stniox/seb
      stmoca=stmoca/seb
      stmeth=stmeth/seb
      stwava=stwava/seb
      stozon=stozon/seb
      stdica=stdica/seb
      stoxyg=stoxyg/seb
      sdtotr=sdtotr/seb
      sdtota=sdtota/seb
      sdtott=sdtott/seb
      sutotr=sutotr/seb
      sutota=sutota/seb
      sutott=sutott/seb
      rog=rog/seb
      sroray=sroray/seb
      sroaer=sroaer/seb
      srotot=srotot/seb
      alumet=alumet/sb
      pizera=0.0
      if(iaer.ne.0) pizera=ssdaer/sodaer
      sodray=sodray/seb
      sodaer=sodaer/seb
      sodtot=sodtot/seb
      sodrayp=sodrayp/seb
      sodaerp=sodaerp/seb
      sodtotp=sodtotp/seb
      fophsa=fophsa/seb
      fophsr=fophsr/seb
 
      do 57 j=1,3
      aini(1,j)=aini(1,j)/seb
      ainr(1,j)=ainr(1,j)/seb
      if(iwave.eq.-1) then
      aini(2,j)=0.
      ainr(2,j)=0.
      else
      aini(2,j)=aini(2,j)/sb
      ainr(2,j)=ainr(2,j)/sb
      endif
   57 continue
 
      if(iwave.ne.-1)goto 160
      alumet=0.
      seb=0.
      sb=0.
 
c**********************************************************************c
c                                                                      c
c                       print of final results                         c
c                                                                      c
c                                                                      c
c                                                                      c
c**********************************************************************c
 160  write(iwr, 430 )refet,alumet,tgasm
      write(iwr, 431 )refet1,refet2,refet3
 
      if(inhomo.ne.0) then
      write(iwr, 432)(aini(1,j),j=1,3),'environment','target',
     s          (ainr(1,j),j=1,3)
      write(iwr, 434)(aini(2,j),j=1,3),'environment','target',
     s         (ainr(2,j),j=1,3)
      endif
      if(inhomo.eq.0) then
      write(iwr, 432)(aini(1,j),j=1,3),'background ','pixel ',
     s		(ainr(1,j),j=1,3)
      write(iwr, 434)(aini(2,j),j=1,3),'background ','pixel ',
     s         (ainr(2,j),j=1,3)
      endif
      write(iwr, 437)sb,seb
c**********************************************************************c
c                                                                      c
c                                                                      c
c                    print of complementary results                    c
c                                                                      c
c                                                                      c
c**********************************************************************c
 
      write(iwr, 929)
      write(iwr, 930)
      write(iwr, 931)'global gas. trans. :',dgasm,ugasm,tgasm
      write(iwr, 931)'water   "     "    :',sdwava,suwava,stwava
      write(iwr, 931)'ozone   "     "    :',sdozon,suozon,stozon
      write(iwr, 931)'co2     "     "    :',sddica,sudica,stdica
      write(iwr, 931)'oxyg    "     "    :',sdoxyg,suoxyg,stoxyg
      write(iwr, 931)'no2     "     "    :',sdniox,suniox,stniox
      write(iwr, 931)'ch4     "     "    :',sdmeth,sumeth,stmeth
      write(iwr, 931)'co      "     "    :',sdmoca,sumoca,stmoca
      write(iwr, 1401)
      write(iwr, 1401)
 
      write(iwr, 931)'rayl.  sca. trans. :',sdtotr,sutotr,sutotr*sdtotr
      write(iwr, 931)'aeros. sca.   "    :',sdtota,sutota,sutota*sdtota
      write(iwr, 931)'total  sca.   "    :',sdtott,sutott,sutott*sdtott
      write(iwr, 1401)
      write(iwr, 1401)
 
      write(iwr, 939)
      write(iwr, 931)'spherical albedo   :',sasr,sasa,sast
      write(iwr, 931)'optical depth total:',sodray,sodaer,sodtot
      write(iwr, 931)'optical depth plane:',sodrayp,sodaerp,sodtotp
      write(iwr, 931)'reflectance        :',sroray,sroaer,srotot
 
      fophst=(sodray*fophsr+sodaer*fophsa)/(sodray+sodaer)
      write(iwr, 931)'phase function     :',fophsr,fophsa,fophst
 
      pizerr=1.
      pizert=(pizerr*sodray+pizera*sodaer)/(sodray+sodaer)
      write(iwr, 931)'sing. scat. albedo :',pizerr,pizera,pizert
      write(iwr, 1401)
      write(iwr, 1401)
 
c**********************************************************************c
c                                                                      c
c                                                                      c
c                    atmospheric correction                            c
c                                                                      c
c                                                                      c
c**********************************************************************c
      if (rapp.ge.-1.) then
	 if (rapp.ge.0.) then
	    xrad=rapp
	    rapp=pi*xrad*sb/xmus/seb
	    else
	    rapp=-rapp
	    xrad=xmus*seb*(rapp)/pi/sb
	 endif
         rog=rapp/tgasm
         rog=(rog-srotot)/sutott/sdtott
         rog=rog/(1.+rog*sast)
	 xa=pi*sb/xmus/seb/tgasm/sutott/sdtott
	 xb=srotot/sutott/sdtott
	 xc=sast
         write(iwr, 940)
         write(iwr, 941)rapp
         write(iwr, 942)xrad
         write(iwr, 943)rog
         write(iwr, 944)xa,xb,xc
      endif
      stop
 
c**********************************************************************c
c                                                                      c
c                   output editing formats                             c
c                                                                      c
c                                                                      c
c**********************************************************************c
   98 format(/////,1h*,30(1h*),16h 6s version 3.1 ,30(1h*),t79
     s       ,1h*,/,1h*,t79,1h*,/,
     s       1h*,22x,34h geometrical conditions identity  ,t79,1h*,/,
     s       1h*,22x,34h -------------------------------  ,t79,1h*)
  101 format(1h*,1x, 7h month:,i3,7h day : ,i3,
     s                 16h universal time:,f6.2,
     s                 10h (hh.dd)  ,11h*latitude: ,f7.2,5h deg ,
     s                 12h longitude: ,f7.2,5h deg ,t79,1h*)
  102 format(1h*,22h solar zenith angle:  ,f6.2,5h deg ,
     s     29h solar azimuthal angle:      ,f6.2,5h deg ,t79,1h*)
  103 format(1h*,1x, 7h month:,i3,7h day : ,i3)
 1110 format(1h*,22h view zenith angle:   ,f6.2,5h deg ,
     s       29h view azimuthal angle:       ,f6.2,5h deg ,
     s      t79,1h*,/,
     s       1h*,22h scattering angle:    ,f6.2,5h deg ,
     s           29h azimuthal angle difference: ,f6.2,5h deg ,
     s      t79,1h*)
 1119 format(1h*,t79,1h*,/,
     s       1h*,22x,31h atmospheric model description ,t79,1h*,/,
     s       1h*,22x,31h ----------------------------- ,t79,1h*)
 1261 format(1h*,30h atmospheric model identity : ,a46,t79,1h*)
 1272 format(1h*,30h atmospheric model identity : ,
     s                 33h user defined atmospheric model  ,t79,1h*,/,
     s       1h*,12x,11h*altitude  ,11h*pressure  ,
     s           11h*temp.     ,11h*h2o dens. ,11h*o3 dens.  ,t79,1h*)
 1271 format(1h*,12x,5e11.4,t79,1h*)
 1281 format(1h*,12x,31h atmospheric model identity :  ,t79,1h*,
     s     /,1h*,12x,35h user defined water content : uh2o=,f6.3,
     s                  7h g/cm2 ,t79,1h*,
     s     /,1h*,12x,35h user defined ozone content : uo3 =,f6.3,
     s                  7h cm-atm,t79,1h*)
 1301 format(1h*,12x,25h aerosols type identity :,
     s                 24h no aerosols computed   ,t79,1h*)
  131 format(1h*,12x,25h aerosols type identity :,1x,
     s                 a11,15h aerosols model,t79,1h*)
  134 format(1h*,12x,25h aerosols type identity :,
     s               29h user defined aerosols model ,t79,1h*,/,
     s  1h*,36x,f6.3,15h % of dust-like,t79,1h*,/,
     s  1h*,36x,f6.3,19h % of water-soluble,t79,1h*,/,
     s  1h*,36x,f6.3,13h % of oceanic,t79,1h*,/,
     s  1h*,36x,f6.3,10h % of soot,t79,1h*)
  140 format(1h*,5x,29h optical condition identity :,t79,1h*,/,
     s       1h*,5x,31h user def. opt. thick. at 550nm :,f7.4,
     s       t79,1h*,/,1h*,t79,1h*)
  141 format(1h*,24x,29h optical condition identity :,t79,1h*,/,
     s       1h*,14x,12h visibility ,f6.2,4h km ,
     s                 19h opt. thick. 550nm ,f7.4,t79,1h*,/,
     s                  1h*,t79,1h*)
  148 format(1h*,22x,21h spectral condition  ,t79,1h*,/,1h*,
     s             22x,21h ------------------  ,t79,1h*)
  149 format(1h*,12x,34h monochromatic calculation at wl :,
     s                              f6.3,8h micron ,t79,1h*)
 1510 format(1h*,22x,a17,26hvalue of filter function  ,t79,1h*,/,
     s 1h*,22x,8h wl inf=,f6.3,4h mic,8h wl sup=,f6.3,4h mic,t79,1h*)
  168 format(1h*,t79,1h*,
     s                 /,1h*,44x,14h target type  ,t79,1h*,/,1h*,
     s                         44x,14h -----------  ,t79,1h*,/,1h*,
     s                         32x,20h homogeneous ground ,t79,1h*)
  169 format(1h*,t79,1h*,
     s                 /,1h*,22x,14h target type  ,t79,1h*,/,1h*,
     s                         22x,14h -----------  ,t79,1h*,/,1h*,
     s    22x,41h inhomogeneous ground , radius of target ,f6.3,
     s         5h km  ,t79,1h*)
  170 format(1h*,22x,22h target reflectance : ,t79,1h*)
  171 format(1h*,22x,29h environmental reflectance : ,t79,1h*)
  172 format(1h*,t79,1h*,/,79(1h*),///)
  173 format(1h*,t79,1h*,/,
     s       1h*,22x,30h target elevation description ,t79,1h*,/,
     s       1h*,22x,30h ---------------------------- ,t79,1h*)
  174 format(1h*,22x,22h ground pressure  [mb]    ,f7.2,1x,t79,1h*)
  175 format(1h*,22x,22h ground altitude  [km]    ,f6.3,1x,t79,1h*)
  176 format(1h*,12x,34h gaseous content at target level: ,t79,1h*,
     s     /,1h*,12x,6h uh2o=,f6.3,7h g/cm2 ,
     s           12x,6h  uo3=,f6.3,7h cm-atm,t79,1h*)
  177 format(1h*,77(1h*),t79,1h*,/,
     s       1h*,23x,34h atmospheric correction activated ,t79,1h*,/,
     s       1h*,23x,34h -------------------------------- ,t79,1h*,/)
  187 format(1h*,77(1h*),t79,1h*,/,
     s       1h*,23x,34h brdf selected                    ,t79,1h*,/,
     s 1h*,12x,40h     rodir    robar    ropbar    albedo ,t79,1h*,/,
     s       1h*,12x,4(f9.4,1x),t79,1h*)
  185 format(1h*,22x,30h input apparent reflectance : , f6.3,t79,1h*)
  186 format(1h*,22x,39h input measured radiance [w/m2/sr/mic] ,
     s       f6.3,t79,1h*)
 
  178 format(1h*,t79,1h*,/,
     s       1h*,22x,30h plane simulation description ,t79,1h*,/,
     s       1h*,22x,30h ---------------------------- ,t79,1h*)
  179 format(1h*,22x,22h plane  pressure  [mb]    ,f7.2,1x,t79,1h*)
  180 format(1h*,22x,22h plane  altitude  [km]    ,f6.3,1x,t79,1h*)
  181 format(1h*,2x,37h atmosphere under plane description: ,t79,1h*)
  182 format(1h*,11x,26h ozone content            ,f6.3,1x,t79,1h*)
  183 format(1h*,11x,26h h2o   content            ,f6.3,1x,t79,1h*)
  184 format(1h*,11x,26haerosol opt. thick. 550nm ,f6.3,1x,t79,1h*)
 
  430 format(79(1h*),/,1h*,t79,1h*,/,
     s       1h*,24x,27h integrated values of  :   ,t79,1h*,/,
     s       1h*,24x,27h --------------------      ,t79,1h*,/,
     s       1h*,16x,21h apparent reflectance  ,f5.3,1x,
     s                 26h appar. rad.(w/m2/sr/mic) ,f8.3,1x,t79,1h*,/,
     s       1h*,22x,30h total gaseous transmittance  ,f5.3,
     s  t79,1h*,/,1h*,t79,1h*,/,1h*,t79,1h*)
  431 format(79(1h*),/,1h*,t79,1h*,/,
     s       1h*,24x,27h coupling aerosol -wv  :   ,t79,1h*,/,
     s       1h*,24x,27h --------------------      ,t79,1h*,/,
     s       1h*,16x,21h wv above aerosol      ,f5.3,1x,
     s                 26h wv mixed with aerosol    ,f5.3,1x,t79,1h*,/,
     s       1h*,22x,30h wv under aerosol             ,f5.3,
     s  t79,1h*,/,1h*,t79,1h*,/,1h*,t79,1h*)
  432 format(1h*,t79,1h*,/,1h*,
     s        24x,32h int. normalized  values  of  : ,t79,1h*,/,1h*,
     s        24x,32h ---------------------------    ,t79,1h*,/,1h*,
     s             22x,31h% of irradiance at ground level,
     s  t79,1h*,/,1h*,5x,17h% of direct  irr.,
     s                    4x,17h% of diffuse irr.,
     s                    4x,17h% of enviro. irr ,t79,1h*,/,
     s             1h*,3(10x,f10.3),t79,1h*,/,
     s 1h*,22x,31h reflectance at satellite level  ,t79,1h*,/,
     s                1h*,5x,17hatm. intrin. ref.,
     s                    3x,a11,5h ref.,
     s                    2x,a6,12h reflectance,t79,1h*,/,
     s             1h*,3(10x,f10.3),t79,1h*,/)
  437 format(1h*,t79,1h*,/,1h*,10x,29hint. funct filter (in mic)
     s               ,10x,26h int. sol. spect (in w/m2),t79,1h*,/,
     s1h*,10x,f10.3,30x,f10.3,t79,1h*,/,1h*,t79,1h*,/,79(1h*))
  434 format(1h*,24x,24h int. absolute values of,t79,
     s 1h*,/,1h*,24x,24h -----------------------               ,
     s  t79,1h*,/,1h*,22x,33hirr. at ground level (w/m2/mic)  ,
     s  t79,1h*,/,1h*, 5x,17hdirect solar irr.,
     s             4x,17hatm. diffuse irr.,
     s             4x,17henvironment  irr ,t79,1h*,/,
     s             1h*,3(10x,f10.3),t79,1h*,/,
     s        1h*,22x,33hrad at satel. level (w/m2/sr/mic),t79,1h*,/,
     s                1h*,5x,17hatm. intrin. rad.,
     s                    4x,a11,5h rad.,
     s                    4x,a6,9h radiance,t79,1h*,/,
     s             1h*,3(10x,f10.3),t79,1h*,/)
  929 format(1h ,////)
  930 format(79(1h*),/,1h*,t79,1h*,/,
     s       1h*,t27,27h integrated values of  :   ,t79,1h*,/,
     s       1h*,t27,27h --------------------      ,t79,1h*,/,
     s       1h*,t79,1h*,/,
     s       1h*,t30,10h downward ,t45,10h  upward  ,
     s            t60,10h   total  ,t79,1h*)
  931 format(1h*,6x,a20,t32,f5.3,t47,f5.3,t62,f5.3,t79,1h*)
  939 format(1h*,t79,1h*,/,1h*,
     s             t30,10h rayleigh ,t45,10h aerosols ,
     s            t60,10h   total  ,t79,1h*,/,1h*,t79,1h*)
  940 format(79(1h*),/,/,/,/,79(1h*),/
     s       1h*,23x,31h atmospheric correction result ,t79,1h*,/,
     s       1h*,23x,31h ----------------------------- ,t79,1h*)
  941 format(1h*,15x,33h input apparent reflectance      :, f6.3,
     s           t79,1h*)
  942 format(1h*,15x,33h measured radiance [w/m2/sr/mic] :, f6.3,
     s           t79,1h*)
  943 format(1h*,15x,39h atmospherically corrected reflectance :,
     s             f6.3,t79,1h*)
  944 format(1h*,15x,39h coefficients xa xb xc                 :,
     s             3(f7.5),t79,1h*,/,1h*,10x,
     s       ' y=xa*(measured radiance)-xb ; acr=y/(1.+xc*y)',
     s               t79,1h*,/,79(1h*))
 1401 format(1h*,t79,1h*)
      end
