/*******************************************************************************
NAME:           GETPAR

PURPOSE:        Get the user specified parameters
*******************************************************************************/
#include <stdio.h>
#include "tae.h"
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "atmocor.h"

extern void sunearthdis( long, float * );

int getpar
    (
    struct PARBLK *vblock,	/* IN pointer to parameter block	*/
    char   hosin[][CMLEN],	/* OUT host input image file names	*/
    char   taeout[][CMLEN],	/* OUT tae output file name		*/
    char   hosout[][CMLEN],	/* OUT host output file name		*/
    long   window[][WINDOW],	/* OUT input window specification	*/
    long   bands[][MAXVISBAND+1],/* OUT image band specifications	*/
    long   numbands[],		/* OUT number of bands in each image	*/
    long   *satnum,		/* OUT satelite number			*/
    long   *radopt,		/* OUT flag (T/F) input data radiance	*/
    double *dsquared,		/* OUT sun-earth distance squared	*/
    long   *msgtim,		/* OUT time between display messages	*/
    long   *mmflag,		/* OUT flag (T/F) calculate min/max	*/
    char   *input_scaling,	/* OUT input scaling			*/
    char   *angle_scaling,	/* OUT angles scaling			*/
    char   *output_scaling,	/* OUT output scaling			*/
    long   *odtype,		/* OUT output data type			*/
    int    *band_sequence	/* OUT band sequence of input image(s)	*/
    )
{
struct	VARIABLE *val;		/* pointer to TAE variable in vblock	*/

char taein[NUM_IMGS][CMLEN];	/* input tae image file names		*/

float aus;			/* sun/earth distance squared		*/

int  i, j;			/* loop counters			*/

long count;			/* number of bands listed in BANDSEQ	*/
long dtypes[MAXVISBAND+1];	/* data types of input images		*/
long element = 0;		/* parameter element number		*/
long input_bands[MAXVISBAND][MAXBND+1]; /* input band specifications	*/
long input_imglim = MAXVISBAND;	/* maximum number of input images	*/
long input_nmrimg;		/* actual number of input images specified */
long input_qflag[MAXVISBAND][2];/* subwindow flags			*/
long jday;			/* julian day of input image		*/
long nbands[MAXVISBAND];	/* number of input bands in specification */
long one = 1;			/* constant of one			*/
long same;			/* same window flag			*/
long totbnd;			/* total bands specified		*/


/* Retrieve and parse the input image specification
---------------------------------------------------*/
if ( c_getin( vblock, "IN", &element, &taein[IMG_INPUT], &hosin[IMG_INPUT],
        &input_nmrimg, &window[IMG_INPUT], input_bands, nbands,
	&totbnd, &input_imglim, &same, input_qflag ) != E_SUCC )
    c_lasmsg( TRUE, "atmocor-fatal", "Fatal error encountered" );


/* Return the bands specified.  The call to c_getin needs an array
dimensioned as MAXBANDS but for our purposes we only need one with
a second dimension of MAXVISBAND+1 so copy the values returned from
the c_getin call to the smaller bands array.
------------------------------------------------------------------*/
for ( i = 0; i < input_nmrimg; i++ )
    {
    for ( j = 0; j < nbands[i]; j++ )
	bands[IMG_INPUT+i][j] = input_bands[i][j];
    bands[IMG_INPUT+i][nbands[i]] = 0;
    numbands[IMG_INPUT+i] = nbands[i];
    }

/* Set up the output image bands based on input
-----------------------------------------------*/
#ifdef DEBUG_IMG
for ( i = 0; i < MAXBANDS; i++ )
    bands[IMG_OUTPUT][i] = i + 1;
numbands[IMG_OUTPUT] = MAXBANDS;
#else
for ( i = 0; i < totbnd; i++ )
    bands[IMG_OUTPUT][i] = i + 1;
numbands[IMG_OUTPUT] = totbnd;
#endif

/* Check for the valid number of input bands.  This is hard-wired for
now since the code is not entirely versatile enough to process only
AVHRR band 2, for example.  Someday ATMOCOR should be fixed so that
a variable number of bands may be specified.  The routine process()
for instancealways assumes two bands are specified.  For now always
input AVHRR band 1 and AVHRR band 2, in any order.
--------------------------------------------------------------------*/
if ( totbnd != MAXVISBAND )
    {
    c_lasmsg(FALSE,"atmocor-badbnd","Invalid number of input bands specified");
    return E_FAIL;
    }

/* Check for the valid number of input images
---------------------------------------------*/
if ( input_nmrimg > MAXVISBAND )
    {
    c_lasmsg(FALSE,"atmocor-badimg","Invalid number of input images specified");
    return E_FAIL;
    }

/* Make sure the band sequence specifies the
same number of bands as the input bands
--------------------------------------------*/
c_getcnt( vblock, "BANDSEQ", (int *)&count );
if ( count != totbnd )
    {
    c_lasmsg(FALSE,"atmocor-badbnd",
        "Input band specification and band sequence don't match");
    return E_FAIL;
    }

/* Get the band sequence
------------------------*/
val = p_find( vblock, "BANDSEQ" );
for ( i = 0; i < count; i++ )
    band_sequence[i] = IVAL( *val, i );

/* Get the input image scaling
------------------------------*/
val = p_find( vblock, "INSCALE" );
strcpy( input_scaling, SVAL( *val, 0 ));

/* Determine if the input image contains radiance or percent reflectance
------------------------------------------------------------------------*/
val = p_find(vblock,"RADOPT");
if ( strcmp( SVAL( *val, 0 ), "RADIANCE" ) == 0 )
    *radopt = TRUE;
else
    *radopt = FALSE;

/* Get the angles image
-----------------------*/
if ( c_getin( vblock, "ANGLES", &element, &taein[IMG_ANGLES],
        &hosin[IMG_ANGLES], &input_nmrimg, &window[IMG_ANGLES], input_bands,
	nbands, &totbnd, &input_imglim, &same, input_qflag ) != E_SUCC )
    c_lasmsg( TRUE, "atmocor-fatal", "Fatal error encountered" );

/* Return the bands specified
-----------------------------*/
for ( i = 0; i < input_nmrimg; i++ )
    {
    for ( j = 0; j < nbands[i]; j++ )
	bands[IMG_ANGLES+i][j] = input_bands[i][j];
    bands[IMG_ANGLES+i][nbands[i]] = 0;
    numbands[IMG_ANGLES+i] = nbands[i];
    }

/* Check for the valid number of angle bands
--------------------------------------------*/
if ( totbnd != 3 )
    {
    c_lasmsg(FALSE,"atmocor-bndcnt","Three band angle image not specified");
    return E_FAIL;
    }

/* Get the data type of the input image(s)
------------------------------------------*/
if ( c_gettyp( (const char (*)[CMLEN])&hosin[IMG_ANGLES], &input_nmrimg,
        dtypes ) != E_SUCC )
    return E_FAIL;

/* Get the angles scaling option
--------------------------------*/
val = p_find( vblock, "ANGSCALE" );
strcpy( angle_scaling, SVAL( *val, 0 ));

/* Get the elevation image
--------------------------*/
if ( c_getin( vblock, "ELEV", &element, &taein[IMG_DEM],
        &hosin[IMG_DEM], &input_nmrimg, &window[IMG_DEM], input_bands,
	nbands, &totbnd, &input_imglim, &same, input_qflag ) != E_SUCC )
    c_lasmsg( TRUE, "atmocor-fatal", "Fatal error encountered" );

if ( nbands[0] != 1 )
    {
    c_lasmsg(FALSE,"atmocor-bndcnt","One band elevation image not specified");
    return E_FAIL;
    }

/* Return the bands specified
-----------------------------*/
for ( i = 0; i < input_nmrimg; i++ )
    {
    for ( j = 0; j < nbands[i]; j++ )
	bands[IMG_DEM+i][j] = input_bands[i][j];
    bands[IMG_DEM+i][nbands[i]] = 0;
    numbands[IMG_DEM+i] = nbands[i];
    }

/* Get the ozone image
----------------------*/
if ( c_getin( vblock, "OZONE", &element, &taein[IMG_OZONE],
        &hosin[IMG_OZONE], &input_nmrimg, &window[IMG_OZONE], input_bands,
	nbands, &totbnd, &input_imglim, &same, input_qflag ) != E_SUCC )
    c_lasmsg( TRUE, "atmocor-fatal", "Fatal error encountered" );

if ( nbands[0] != 1 )
    {
    c_lasmsg(FALSE,"atmocor-bndcnt","One band ozone image not specified");
    return E_FAIL;
    }

/* Return the bands specified
-----------------------------*/
for ( i = 0; i < input_nmrimg; i++ )
    {
    for ( j = 0; j < nbands[i]; j++ )
	bands[IMG_OZONE+i][j] = input_bands[i][j];
    bands[IMG_OZONE+i][nbands[i]] = 0;
    numbands[IMG_OZONE+i] = nbands[i];
    }

/* Get the water vapor image
----------------------------*/
if ( c_getin( vblock, "WATER", &element, &taein[IMG_WATER],
        &hosin[IMG_WATER], &input_nmrimg, &window[IMG_WATER], input_bands,
	nbands, &totbnd, &input_imglim, &same, input_qflag ) != E_SUCC )
    c_lasmsg( TRUE, "atmocor-fatal", "Fatal error encountered" );

if ( nbands[0] != 1 )
    {
    c_lasmsg(FALSE,"atmocor-bndcnt","One band water vapor image not specified");
    return E_FAIL;
    }

/* Return the bands specified
-----------------------------*/
for ( i = 0; i < input_nmrimg; i++ )
    {
    for ( j = 0; j < nbands[i]; j++ )
	bands[IMG_WATER+i][j] = input_bands[i][j];
    bands[IMG_WATER+i][nbands[i]] = 0;
    numbands[IMG_WATER+i] = nbands[i];
    }

/* Retrieve output data type
----------------------------*/
i = 1;
val = p_find( vblock, "ODTYPE" );
if ( strcmp( SVAL( *val, 0 ), "BYTE" ) == 0 )
    *odtype = EBYTE;
else if ( strcmp( SVAL( *val, 0 ), "I*2" ) == 0 )
    *odtype = EWORD;
else if ( strcmp( SVAL( *val, 0 ), "I*4" ) == 0 )
    *odtype = ELONG;
else if ( strcmp( SVAL( *val, 0 ), "R*4" ) == 0 )
    *odtype = EREAL;
else if ( strcmp( SVAL( *val, 0 ), "SAME" ) == 0 )
    c_gettyp( (const char (*)[CMLEN])&hosin[IMG_INPUT], &one, odtype );
#ifdef DEBUG_IMG
if ( *odtype != EREAL )
    {
    *odtype = EREAL;
    c_lasmsg(FALSE,"atmocor-odtype","Output data type changed to R*4 in debug");
    }
#endif

/* Get the output image scaling
-------------------------------*/
val = p_find( vblock, "OUTSCALE" );
strcpy( output_scaling, SVAL( *val, 0 ));

/* Get the output image name
----------------------------*/
if ( c_getout( vblock, "OUT", taeout, hosout ) == E_FAIL )
    return E_FAIL;

/* Retrieve the satellite number
--------------------------------*/
val = p_find( vblock, "SATNUM" );
*satnum = IVAL( *val, 0 );

/* Retrieve the julian date
---------------------------*/
val = p_find( vblock, "JDATE" );
jday = IVAL( *val, 0 );

/* Correct for variations in earth-sun distance, which introduce a +/- 3.5%
error in the computations. These errors are small at equinoxes, but are
significant at solstices.
---------------------------------------------------------------------------*/
sunearthdis( jday, &aus );
*dsquared = aus * aus;

/* Retrieve LAS globals
-----------------------*/
val = p_find( vblock, "$msgtim" );
*msgtim = IVAL( *val, 0 );

val = p_find( vblock, "$minmax" );
if ( strcmp( SVAL( *val, 0 ), "YES" ) == 0 )
    *mmflag = TRUE;
else
    *mmflag = FALSE;

return E_SUCC;
}
