/*******************************************************************************
NAME:		OPEN_LAS 

PURPOSE:	Open all the LAS files
*******************************************************************************/
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "atmocor.h"


int open_las
(
    struct GDESC **gdesc,	/* OUT pointer to TAE parameter block  	*/
    struct FDESC *fdesc[],	/* OUT File descriptors to the images	*/
    char   hosin[][CMLEN],	/* IN list of input file names		*/
    char   hosout[][CMLEN],	/* IN host file name of output image	*/
    long   odtype,		/* IN output data type			*/
    long   bands[][MAXVISBAND+1],/* IN list of image bands		*/
    long   nbands[],		/* IN number of bands in each image	*/
    long   offset[],		/* OUT offset array into thebuf		*/
    unsigned char **thebuf,	/* OUT the image I/O data buffer	*/
    long   window[]		/* IN Window to open input images with	*/
)
{
int     band;		/* index into the offset array		*/
int     fd;		/* index into the fdesc array		*/

long	idtype = EREAL;	/* input date type			*/
long	image;		/* loop index counting the images	*/
long	one = 1;	/* value of one				*/
long	opt[3];		/* type conversion option upon writing image */
long	status;		/* function return status code          */
long    access = IREAD;	/* access type variable			*/
long    bufsiz;		/* size of the data buffer		*/
long    zero = 0;	/* value of zero for eopens call	*/


/* Set the output conversion values (only used for write access)
---------------------------------------------------------------*/
opt[0] = opt[1] = opt[2] = 0;

/* Open the input image(s)
--------------------------*/
for ( fd = 0, image = IMG_DEM, band = 0; image < NUM_IMGS; image++)
    {
    if ( hosin[image][0] != '\0' )
	{
	status = c_eopens( &fdesc[fd++], hosin[image], bands[image],
	    &window[SL], &window[SS], &window[NL], &window[NS],
	    &offset[band], &access, &idtype, &zero, opt );
	if (status != E_SUCC)
	    {
	    c_lasmsg(FALSE,"atmocor-open","Error opening the input image");
	    return E_FAIL;
	    }
	band += nbands[image];
	}
    }

/* Open output image
--------------------*/
access = IWRITE;
image = IMG_OUTPUT;
#ifdef DEBUG_IMG
nbands[image] = MAXBANDS;
if ( odtype != EREAL )
    {
    odtype = EREAL;
    c_lasmsg(FALSE,"atmocor-odtype","Output data type is being changed to R*4");
    }
#endif
status = c_eopens( &fdesc[fd], hosout[0], &nbands[image], &one, &one,
    &window[NL], &window[NS], &offset[band], &access, &odtype, &zero, opt );
if (status != E_SUCC)
    {
    close_las( fdesc );
    c_lasmsg(FALSE,"atmocor-open","Error opening the output image");
    return E_FAIL;
    }

/* Allocate the data buffer space
---------------------------------*/
if (( *thebuf = (unsigned char *)c_ealloc( fdesc, &bufsiz )) == NULL )
    {
    close_las( fdesc );
    c_lasmsg(FALSE,"atmocor-alloc","Error allocating memory");
    return E_FAIL;
    }

/* Group the file descriptors
-----------------------------*/
if ( c_egroup( fdesc, gdesc, *thebuf, &bufsiz ) != E_SUCC )
    {
    close_las( fdesc );
    c_lasmsg(FALSE,"atmocor-group","Error grouping images");
    return E_FAIL;
    }

return E_SUCC;
}
