#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "las.h"
#include "adaps.h"
#include "atmocor.h"

static short first_call = TRUE;
static char filename[CMLEN] = "";


int quickcor
(
    double chans[],	/* IN  - AVHRR channels 1 and 2 in radiance           */
    double angs[],	/* IN  - View angles (must be absolute values)        */
    double elev,	/* IN  - Elevation in kilometers                      */
    double ozone,	/* IN  - Ozone value                                  */
    double sun_earth,	/* IN  - Sun-Earth dist. squared (astronomical units) */
    double qcor[]	/* OUT - Atmospherically corrected in reflectance     */
)
{
char str[CMLEN];	/* Temporary string                                   */
char arguments[CMLEN];	/* Arguments to quickcor                              */

long i;			/* Loop control                                       */
long j;			/* Loop control                                       */
long ntries;		/* Number of times to try quickcor                    */
long nonfatal = 1;	/* Nonfatal error status                              */

int status;		/* Needed to wait until the pipe is closed            */

short success;		/* Flag indicating successful call to quickcor        */

FILE *fp;		/* File ptr for opening pipe to the quickcor command  */


/*
  Create a unique file name for calling quickcor
*/
if (first_call)
    {
    sprintf(filename, "_,#quickcor.%d#,_", getpid());
    first_call = FALSE;
    }


/*
  Correct both channels
*/
for (i = 0; i < 2; i++)
    {
    /*
      Open the temporary file and write the necessary arguments to quickcor
    */
    fp = fopen(filename, "w");
    if (fp == NULL)
	{
	c_errmsg("Error opening quickcor file", "atmocor-debug", &nonfatal);
	return(E_FAIL);
	}
    sprintf(arguments, "%d %7.5lf %7.5lf %7.5lf %7.5lf %7.5lf %7.5lf %7.5lf",
	i+1, chans[i], angs[SOLZEN], angs[SATZEN], angs[RELAZ], elev, ozone,
	sun_earth);
    fprintf(fp, "%s\n", arguments);
    fclose(fp);


    /*
      Run quickcor.  Open a pipe to the quickcor command so we can read the
      information it prints to stdout.  Skip the first 23 lines of output and
      read the atmospherically corrected pixel (8th value on line 24).
    */
    for (ntries = 0, success = FALSE; (ntries < 5) && !success; ntries++)
	{
	sprintf(str, "quickcor < %s", filename);
	fp = popen(str, "r");
	if (fp != NULL)
	    {
	    for (j = 0; j < 24; j++)
		fgets(str, CMLEN, fp);
	    if (sscanf(str, "%*s %*s %*s %*s %*s %*s %*s %lf", &qcor[i]) == 1)
		success = TRUE;
	    }
	pclose(fp);
	wait(&status);
	}
    

    /*
      If quickcor did not work
    */
    if (!success)
	{
	qcor[i] = 0.0;
	c_errmsg(arguments, "quickcor-err", &nonfatal);
	}

    /*
      Remove the temporary file
    */
    remove(filename);
    }

return(E_SUCC);
}
