#include <stdio.h>
#include <math.h>

#define MAX 1000
#define BANDS   18
#define LINES    5
#define SAMPLES  5

static char filename[MAX];


int usage()
{
fprintf(stderr, "usage: compare <file>\n\n");
fprintf(stderr, "       %s\n",
"Compare the results from ATMOCOR to quickcor.  To use this function,");
fprintf(stderr, "       %s\n",
"compile ATMOCOR with DEBUG_IMG defined in atmocor.h.  This will cause");
fprintf(stderr, "       %s\n",
"ATMOCOR to produce 16 band images.  You must then run ATMOCOR using a");
fprintf(stderr, "       %s\n",
"5x5 window over the desired area in your image.  Run LIST on this output");
fprintf(stderr, "       %s\n",
"image, saving the output in a file.  The resulting file should be");
fprintf(stderr, "       %s\n",
"supplied as the argument to this function.");
fprintf(stderr, "\n");
fprintf(stderr, "       %s\n",
"Good luck, and may the force be with you 8^)!!!!!!\n");
exit(1);
}




int main (int argc, char *argv[])
{
char str[MAX];
char tmp[MAX];

double pixels[BANDS][LINES][SAMPLES];
double diff;

int i, j, k;
int line;
int sample;
int band;
int status;


FILE *fp;


/* Check the arguments */
if ((argc != 2) && (argc != 3))
    usage();


/* Create an unique file name */
sprintf(filename, "_,#quickcor.%d#,_", getpid());


/* Open the file */
fp = fopen(argv[1], "r");
if (fp == NULL)
    {
    fprintf(stderr, "Error opening %s\n", argv[1]);
    exit(1);
    }


/* Read until EOF */
band = 0;
while ((fgets(str, MAX, fp) != NULL) && (band < 16))
    {
    /* Look for the string "Image:" */
    sscanf(str, "%s", tmp);
    if (strcmp(tmp, "Image:") != 0)
	{
	fprintf(stderr, "File does not conform to expected format.\n");
	fclose(fp);
	exit(1);
	}


    /* Expecting a line of sample numbers */
    fgets(str, MAX, fp);
    if (sscanf(str, "%d %d %d %d %d %d", &i, &i, &i, &i, &i, &i) != 5)
	{
	fprintf(stderr, "File does not contain five samples.\n");
	fclose(fp);
	exit(1);
	}
    

    /* Read the pixel values for this band */
    for (line = 0; line < LINES; line++)
	{
	fgets(str, MAX, fp);
	if (sscanf(str, "%d %le %le %le %le %le", &i, &pixels[band][line][0],
	    &pixels[band][line][1], &pixels[band][line][2],
	    &pixels[band][line][3], &pixels[band][line][4]) != 6)
	    {
	    fprintf(stderr, "File does not contain five samples.\n");
	    fclose(fp);
	    exit(1);
	    }
	}

    band++;
    }


/* Close the input file */
fclose(fp);


/* Convert all angle values to absolute values */
for (i = 0; i < LINES; i++)
    for (j = 0; j < SAMPLES; j++)
	{
	pixels[9][i][j]  = fabs(pixels[9][i][j]);
	pixels[10][i][j] = fabs(pixels[10][i][j]);
	pixels[11][i][j] = fabs(pixels[11][i][j]);
	}


/* Print the input file for testing */
#ifdef DEBUG
for (i = 0; i < 16; i++)
    {
    for (j = 0; j < LINES; j++)
	{
	for (k = 0; k < SAMPLES; k++)
	    printf("%6.3lf  ", pixels[i][j][k]);
	printf("\n");
	}
    printf("\n");
    }
#endif


/* Call quickcor for every sample on each line */
for (line = 0; line < LINES; line++)
    {
    printf("Line %d: ", line + 1);
    printf("chan radiance  solzen   satzen    relaz     elev   ozone sun_earth\n");

    for (sample = 0; sample < SAMPLES; sample++)
	{
	printf("Sample %d: ", sample + 1);
	fflush(stdout);

	/* Channel 1 */
	/* Open a temporary file */
	fp = fopen(filename, "w");

	printf("1 %5.5lf %5.5lf %5.5lf %5.5lf %5.5lf %5.5lf 1\n",
	    pixels[0][line][sample],
	    pixels[10][line][sample],
	    pixels[9][line][sample],
	    pixels[11][line][sample],
	    pixels[8][line][sample],
	    pixels[12][line][sample]);
	
	fprintf(fp, "1 %5.5lf %5.5lf %5.5lf %5.5lf %5.5lf %5.5lf 1\n",
	    pixels[0][line][sample],
	    pixels[10][line][sample],
	    pixels[9][line][sample],
	    pixels[11][line][sample],
	    pixels[8][line][sample],
	    pixels[12][line][sample]);
	
	/* Close the file */
	fclose(fp);

	/* Run quickcor */
	sprintf(str, "quickcor < %s", filename);
	fp = popen(str, "r");

	/* Read 24 lines */
	for (i = 0; i < 24; i++)
	    fgets(str, MAX, fp);
	
	/* Get the quickcor result */
	sscanf(str, "%*s %*s %*s %*s %*s %*s %*s %lf",
	    &pixels[16][line][sample]);
	
	/* Close the pipe */
	pclose(fp);
	wait(&status);

	/* Remove the temporary file */
	remove(filename);



	/* Channel 2 */
	/* Open a temporary file */
	fp = fopen(filename, "w");

	printf("          2 %5.5lf %5.5lf %5.5lf %5.5lf %5.5lf %5.5lf 1\n",
	    pixels[1][line][sample],
	    pixels[10][line][sample],
	    pixels[9][line][sample],
	    pixels[11][line][sample],
	    pixels[8][line][sample],
	    pixels[12][line][sample]);
	
	fprintf(fp, "2 %5.5lf %5.5lf %5.5lf %5.5lf %5.5lf %5.5lf 1\n",
	    pixels[1][line][sample],
	    pixels[10][line][sample],
	    pixels[9][line][sample],
	    pixels[11][line][sample],
	    pixels[8][line][sample],
	    pixels[12][line][sample]);
	
	/* Close the file */
	fclose(fp);

	/* Run quickcor */
	sprintf(str, "quickcor < %s", filename);
	fp = popen(str, "r");

	/* Read 24 lines */
	for (i = 0; i < 24; i++)
	    fgets(str, MAX, fp);
	
	/* Get the quickcor result */
	sscanf(str, "%*s %*s %*s %*s %*s %*s %*s %lf",
	    &pixels[17][line][sample]);
	
	/* Close the pipe */
	pclose(fp);
	wait(&status);

	/* Remove the temporary file */
	remove(filename);
	}

    printf("\n");
    }


/* Print the results */
for (i = 0; i < LINES; i++)
    {
    for (j = 0; j < SAMPLES; j++)
	{
	diff = pixels[14][i][j] - pixels[16][i][j];
	printf("%06.3lf=%06.3lf (%+5.3lf)  ",
	    pixels[14][i][j], pixels[16][i][j], diff);

	diff = pixels[15][i][j] - pixels[17][i][j];
	printf("%06.3lf=%06.3lf (%+5.3lf)\n",
	    pixels[15][i][j], pixels[17][i][j], diff);
	}
    }
exit(0);
}
