
	program quickcor

********
******** uses phil teillet's A and B coef tables for quick atmospheric
******** correction ("single point").
********

	implicit none
	character*80	afile(2), bfile(2), ofile
	real*4	ths(12), thv(12), phi(18), z(5), rsurf, rtoa, o3
	real*4	a(5,12,12), b(5,18,12,12), s(5,2), ss, y, pi, d2r
	real*4	rad, solz, satz, raz, h, d1, d2, d3, d4, e0(2)
	real*4	b000, b001, b010, b011, b100, b101, b110, b111
	real*4	b00, b01, b10, b11, b0, b1, bb
	real*4  a00, a01, a10, a11, a0, a1, aa
	real*4	to3(40,16), tg, dd
	integer i, j, k, l, ich, ll, path, conc
	integer	l1, l2, l3, l4, u1, u2, u3, u4
******* E0 coeffiecients for NOAA-14
	data	e0
     +		/1605.0, 1029.0/
	data	ths
     +		/0.,15.,30.,40.,50.,58.,64.,69.,73.,76.,78.,80./
	data	thv
     +		/0.,10.,20.,30.,40.,46.,51.,56.,60.,64.,67.,70./
	data	phi
     +		/0.,11.,23.,37.,53.,67.,79.,90.,100.,110.,120.,130.,
     +		140.,150.,160.,168.,174.,180./
	data	z
     +		/0.0, 0.5, 1.1, 1.8, 2.6/
*****   A, B, S, Coefficients and Ozone for NOAA-14
	data	afile	/'acoef_n14_ch1.lut','acoef_n14_ch2.lut'/
	data	bfile	/'bcoef_n14_ch1.lut','bcoef_n14_ch2.lut'/
	data	ofile	/'ozone_n14_ch1.lut'/
	data	s
     +          /0.048642, 0.046084, 0.043148, 0.039896, 0.036404,
     +           0.017996, 0.017004, 0.015870, 0.014622, 0.013290/

	pi = dacos(-1.d0)
	d2r = pi/180.

*	write (6,*) 'enter chan, rtoa, solz, satz, raz, h, o3, dd: '
*	read (5,*) ich, rtoa, solz, satz, raz, h, o3, dd
	write (6,*)
     +	'enter chan, radiance, solzen, satzen, relaz, elev, ozone, dd: '
	read (5,*) ich, rad, solz, satz, raz, h, o3, dd

********
******** load correction coefficient tables
********

	write (6,*) 'loading A file...'
	open (unit = 10, file = afile(ich), type ='old')
        do i = 1, 12
          do j = 1, 12
            do l = 1, 5
              read (10,*) a(l,j,i)
	    end do
	  end do
	end do
	close (unit = 10)

	write (6,*) 'loading B file...'
	open (unit = 11, file = bfile(ich), type ='old')
	do i = 1, 12
	  do j = 1, 12
	    do k = 1, 18
	      do l = 1, 5
	        read (11,*) b(l,k,j,i)
              end do
            end do
          end do
        end do
	close (unit = 11)

	write (6,*) 'loading ozone file...'
	open (unit = 12, file = ofile, type = 'old')
	do i = 1, 16
	  l = 1
	  do j = 1, 4
	    read (12,*) (to3(ll,i), ll = l, l+9)
	    l = l + 10
	  end do
	end do
	close (unit = 12) 

********
******** set variables for interpolation of h on z (elevation)
********

	if (h .le. z(1)) then
	  l1 = 1
	  u1 = 1
	  d1 = 0.0
	else if (h .ge. z(5)) then
	  l1 = 5
	  u1 = 5
	  d1 = 1.0
	else
	  l1 = 0
	  do while (h .gt. z(l1+1))
	    l1 = l1 + 1
	  end do
	  u1 = l1 + 1
	  d1 = (h - z(l1))/(z(u1) - z(l1))
	end if

	write (6,*) 'elev   ', l1, u1, z(l1), z(u1), d1

********
******** set variables for interpolation of raz on phi (relative azimuth)
********

        if (raz .le. phi(1)) then
          l2 = 1
          u2 = 1
          d2 = 0.0
        else if (raz .ge. phi(18)) then
          l2 = 18
          u2 = 18
          d2 = 1.0
        else
          l2 = 0
          do while (raz .gt. phi(l2+1))
            l2 = l2 + 1
          end do
          u2 = l2 + 1
	  d2 = (raz - phi(l2))/(phi(u2) - phi(l2))
        end if

	write (6,*) 'phirel ', l2, u2, phi(l2), phi(u2), d2

********
******** set variables for interpolation of satz on thv (sat zenith)
********

        if (satz .le. thv(1)) then
          l3 = 1
          u3 = 1
          d3 = 0.0
        else if (satz .ge. thv(12)) then
          l3 = 12
          u3 = 12
          d3 = 1.0
        else
          l3 = 0
          do while (satz .gt. thv(l3+1))
            l3 = l3 + 1
          end do
          u3 = l3 + 1
	  d3 = (satz - thv(l3))/(thv(u3) - thv(l3))
        end if

	write (6,*) 'satzen ', l3, u3, thv(l3), thv(u3), d3

********
******** set variables for interpolation of solz on ths (sun zenith)
********

        if (solz .le. ths(1)) then
          l4 = 1
          u4 = 1
          d4 = 0.0
        else if (solz .ge. ths(12)) then
          l4 = 12
          u4 = 12
          d4 = 1.0
        else
          l4 = 0
          do while (solz .gt. ths(l4+1))
            l4 = l4 + 1
          end do
          u4 = l4 + 1
	  d4 = (solz - ths(l4))/(ths(u4) - ths(l4))
        end if

	write (6,*) 'sunzen ', l4, u4, ths(l4), ths(u4), d4

********
******** interpolate a coefficient
********

	write (6,*) 'a interpolates...'
	write (6,*) a(u1,l3,l4), a(l1,l3,l4)
	write (6,*) a(u1,l3,u4), a(l1,l3,u4)
	write (6,*) a(u1,u3,l4), a(l1,u3,l4)
	write (6,*) a(u1,u3,u4), a(l1,u3,u4)
	a00 = a(l1,l3,l4) + (a(u1,l3,l4) - a(l1,l3,l4))*d1
	a01 = a(l1,l3,u4) + (a(u1,l3,u4) - a(l1,l3,u4))*d1
	a10 = a(l1,u3,l4) + (a(u1,u3,l4) - a(l1,u3,l4))*d1
	a11 = a(l1,u3,u4) + (a(u1,u3,u4) - a(l1,u3,u4))*d1
	a0 = a00 + (a10 - a00)*d3
	a1 = a01 + (a11 - a01)*d3
	aa = a0 + (a1 - a0)*d4

********
******** interpolate b coefficient
********

	write (6,*) 'b interpolates...'
	write (6,*) b(u1,l2,l3,l4), b(l1,l2,l3,l4)
	write (6,*) b(u1,l2,l3,u4), b(l1,l2,l3,u4)
	write (6,*) b(u1,l2,u3,l4), b(l1,l2,u3,l4)
	write (6,*) b(u1,l2,u3,u4), b(l1,l2,u3,u4)
	write (6,*) b(u1,u2,l3,l4), b(l1,u2,l3,l4)
	write (6,*) b(u1,u2,l3,u4), b(l1,u2,l3,u4)
	write (6,*) b(u1,u2,u3,l4), b(l1,u2,u3,l4)
	write (6,*) b(u1,u2,u3,u4), b(l1,u2,u3,u4)
	b000 = b(l1,l2,l3,l4) + (b(u1,l2,l3,l4) - b(l1,l2,l3,l4))*d1
	b001 = b(l1,l2,l3,u4) + (b(u1,l2,l3,u4) - b(l1,l2,l3,u4))*d1
	b010 = b(l1,l2,u3,l4) + (b(u1,l2,u3,l4) - b(l1,l2,u3,l4))*d1
	b011 = b(l1,l2,u3,u4) + (b(u1,l2,u3,u4) - b(l1,l2,u3,u4))*d1
	b100 = b(l1,u2,l3,l4) + (b(u1,u2,l3,l4) - b(l1,u2,l3,l4))*d1
	b101 = b(l1,u2,l3,u4) + (b(u1,u2,l3,u4) - b(l1,u2,l3,u4))*d1
	b110 = b(l1,u2,u3,l4) + (b(u1,u2,u3,l4) - b(l1,u2,u3,l4))*d1
	b111 = b(l1,u2,u3,u4) + (b(u1,u2,u3,u4) - b(l1,u2,u3,u4))*d1
        b00 = b000 + (b100 - b000)*d2
        b01 = b001 + (b101 - b001)*d2
        b10 = b010 + (b110 - b010)*d2
        b11 = b011 + (b111 - b011)*d2
        b0 = b00 + (b10 - b00)*d3
        b1 = b01 + (b11 - b01)*d3
        bb = b0 + (b1 - b0)*d4

********
******** interpolate s coefficient
********

	ss = s(l1,ich) + (s(u1,ich) - s(l1,ich))*d1

********
******** lookup ozone gaseous transmittance and rescale
********

	conc = nint (o3*100-10)
	path = nint ((solz + satz)/10) + 1
	tg = to3(conc,path)
	write (6,*) 'conc=', conc, ' path=', path

********
******** convert rtoa to radiance and correct (assume Earth is 1 AU from Sun)
********

	if (rad .lt. 0) then
	  rtoa = -rad
	  rad = rtoa*e0(ich)*cos(solz*d2r)/pi
	  write (6,*) 'rad < 0!', rtoa, rad
*	  rad = rtoa*e0(ich)/pi
	end if
	if (ich .eq. 1) then
	  y = dd * aa/tg * rad + bb
	else
	  y = dd * aa * rad + bb
	end if
	rsurf = 100.*y/(100. + y*ss)

********
******** results
********

	write (6,'(5(a,f10.6))')
     +	  'a=', aa, ' b=', bb, ' ss=', ss, ' rsurf=', rsurf,
     +	  ' tg=', tg

	stop
	end

