/*******************************************************************************
FUNCTION:	SCALE 

PURPOSE:	Scale the given pixel values and store them in the output
		buffer.
*******************************************************************************/
#include "las.h"
#include "typlim.h"
#include "adaps.h"
#include "atmocor.h"


int scale
(
    long totalbands,
    double atmo[],		/* IN - Atmospherically corrected pixel values*/
    long validflg[],		/* IN - Valid pixel flag (NOT mask values)    */
    struct SCOFF scoff[],	/* IN - Scale and offset values for scaling   */
    long mmflag,		/* IN - Flag to compute min/max values        */
    double minval[],		/* IN/OUT - Min pixel values                  */
    double maxval[]		/* IN/OUT - Max pixel values                  */
)
{
long band;			/* Loop control                               */

/*
  Scale the atmospherically corrected pixel and store it in the output buffer
*/
for (band = 0; band < totalbands; band++)
    {
    /* For global scaling, certain values are used for mask values and are
       not actual valid data.  These values should not be scaled but simply
       copied into the output buffer.  Note the variable 'scaled' must be a
       double.
    -------------------------------------------------------------------------*/
    if (validflg[band])
        atmo[band] = SCALE(atmo[band], scoff[OUT_REFLECT].minval,
		       scoff[OUT_REFLECT].maxval, scoff[OUT_REFLECT].scale,
		       scoff[OUT_REFLECT].offset);


    /* Determine if the new value is a min/max for the current band
    ---------------------------------------------------------------*/
    if (mmflag)
	{
	if (atmo[band] > maxval[band])
	    maxval[band] = atmo[band];
	else if (atmo[band] < minval[band])
	    minval[band] = atmo[band];
	}
    }

return(E_SUCC);
}
