/*******************************************************************************
FUNCTION:	update_ddr

PURPOSE:	Update the ddr of the output image.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ddr.h"
#include "atmocor.h"

int update_ddr
(
    struct PARBLK *vblock,	/* IN - Pointer to TAE parameter block        */
    char hosin[][CMLEN],	/* IN - Host file names of input images       */
    char hosout[],		/* IN - Host file name of the output image    */
    long windows[][4],		/* IN - Windows for the input images          */
    long *total_bands		/* IN - Total bands in the output image       */
)
{
long band;			/* Current band                               */
long bands[1][MAXBND+1];	/* Band numbers from the input image          */
long cflag	= COMB;		/* Combine flag for updating the DDR          */
long key;			/* Key for the desired field of the DDR       */
long nbands[2];			/* Number of bands from the input image       */
long one	= 1;		/* Constant value of one                      */
long status;			/* Return status from a support routine       */

double dblone	= 1;		/* Double constant value of one		      */

struct BDDR bddr[20];		/* Band information from the DDR              */
struct DDR ddr;


/* Set up the constants
-----------------------*/
bands[0][0] = 1;
bands[0][1] = 2;
bands[0][2] = 0;
nbands[0] = 2;
nbands[1] = 0;


/* Use input ddr to update ddr for output image
-----------------------------------------------*/
if ( c_upddr(vblock, (const char (*)[CMLEN])&hosin[IMG_INPUT], hosout, &one,
        (const long (*)[4])windows, (const long (*)[MAXBND+1])bands, nbands,
	&dblone, &dblone, total_bands, &cflag) != E_SUCC )
    {
    c_lasmsg(FALSE,"atmocor-upddr","Error updating output DDR");
    return E_FAIL;
    }

if ( c_getddr( hosout, &ddr ) != E_SUCC )
    {
    c_lasmsg(FALSE,"atmocor-getddr","Error retrieving output DDR");
    return E_FAIL;
    }
for (band = 1; band <= *total_bands; band++)
    if ( c_getbdr( hosout, &bddr[band], &band ) != E_SUCC )
        {
	c_lasmsg(FALSE,"atmocor-getbdr","Error retrieving output band DDR");
	return E_FAIL;
	}

/* Update the ddr and band records 
----------------------------------*/
for (band = 1; band <= *total_bands; band++)
    {
    /* Fill in appropriate data fields
    ----------------------------------*/
    key = DDBAND;
    c_putbdf(&bddr[band], &key, (unsigned char *)&band);
    key = DDSRC;
#ifdef DEBUG_IMG
    switch (band)
	{
	case  1:
	    status = c_putbdf(&bddr[band], &key, "AVHRR channel 1");
	    break;
	case  2:
	    status = c_putbdf(&bddr[band], &key, "AVHRR channel 2");
	    break;
	case  3:
	    status = c_putbdf(&bddr[band], &key, "Acoeff channel 1");
	    break;
	case  4:
	    status = c_putbdf(&bddr[band], &key, "Acoeff channel 2");
	    break;
	case  5:
	    status = c_putbdf(&bddr[band], &key, "Bcoeff channel 1");
	    break;
	case  6:
	    status = c_putbdf(&bddr[band], &key, "Bcoeff channel 2");
	    break;
	case  7:
	    status = c_putbdf(&bddr[band], &key, "Scoeff channel 1");
	    break;
	case  8:
	    status = c_putbdf(&bddr[band], &key, "Scoeff channel 2");
	    break;
	case  9:
	    status = c_putbdf(&bddr[band], &key, "Elevation (meters)");
	    break;
	case 10:
	    status = c_putbdf(&bddr[band], &key, "Satellite zenith");
	    break;
	case 11:
	    status = c_putbdf(&bddr[band], &key, "Solar zenith");
	    break;
	case 12:
	    status = c_putbdf(&bddr[band], &key, "Relative azimuth");
	    break;
	case 13:
	    status = c_putbdf(&bddr[band], &key, "Ozone");
	    break;
	case 14:
	    status = c_putbdf(&bddr[band], &key, "Water vapor");
	    break;
	case 15:
	    status = c_putbdf(&bddr[band], &key, "ATMOCOR AVHHR channel 1");
	    break;
	case 16:
	    status = c_putbdf(&bddr[band], &key, "ATMOCOR AVHHR channel 2");
	    break;
	case 17:
	    status = c_putbdf(&bddr[band], &key, "Tg all gases");
	    break;
	case 18:
	    status = c_putbdf(&bddr[band], &key, "Tw water vapor chan 1");
	    break;
	case 19:
	    status = c_putbdf(&bddr[band], &key, "Tw water vapor chan 2");
	    break;
	default:
	    status = c_putbdf(&bddr[band], &key, "Unknown");
	    break;
	}
#else
    switch (band)
	{
	case  1:
	    status = c_putbdf(&bddr[band], &key, "ATMOCOR AVHHR channel 1");
	    break;
	case  2:
	    status = c_putbdf(&bddr[band], &key, "ATMOCOR AVHHR channel 2");
	    break;
	default:
	    status = c_putbdf(&bddr[band], &key, "Unknown");
	    break;
	}
#endif
    if ( status != E_SUCC )
        c_lasmsg(FALSE,"atmocor-bddr","Error updating BDDR");
    }

if ( c_putddr( hosout, &ddr ) != E_SUCC )
    {
    c_lasmsg(FALSE,"atmocor-ddr","Error writing output DDR");
    return E_FAIL;
    }
for (band = 1; band <= *total_bands; band++)
    if ( c_putbdr( hosout, &bddr[band] ) != E_SUCC )
        {
	c_lasmsg(FALSE,"atmocor-bddr","Error writing output band DDR");
	return E_FAIL;
	}

return E_SUCC;
}
