/*******************************************************************************
NAME:		DEGLL

PURPOSE:	Convert latitude/longitude pairs in total degrees to
		line/sample pairs

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "model.h"
#include "util.h"
#include "protos.h"

void degll
(
struct MODEL  *model,  	/* I: Model and orbital information		   */
double  unit,		/* I: DEG or RAD unit scale factor                 */
long    totpairs,	/* I: Total number of latitude/longitude pairs     */
double  lats[],		/* I: Input latitudes in DEG or RAD                */
double  longs[],        /* I: Input longitudes in DEG or RAD               */
long    lines[],	/* O: Converted line numbers                       */
long    samps[],	/* O: Converted sample numbers                     */
FILE    *prt_fd         /* I: Print file descriptor                        */
)

{
long	i;		/* Looping variable                             */

double	lat,		/* Latitude of the input                        */
	lng,		/* Longitude of the input                       */
	elev = 0.0;	/* Elevation constant				*/

float	line,		/* Calculated line                              */
	sample;		/* Calculated sample                            */


/*
  Convert the latitude/longitude pairs
*/
for (i = 0; i < totpairs; i++)
    {
    /*
      Convert the latitude and longitude to radians
      (unit was set to appropriatly in getpar)
    */
    lat = lats[i] * unit;
    lng = longs[i] * unit;

    /*
      Get the line and sample
    */
    c_ll2ls(&lat,&lng,&elev,model,&line,&sample,NULL);

    /*
      Rounding!
    */
    lines[i] = (long)(line + 0.05);
    samps[i] = (long)(sample + 0.05);

    /*
      Write the results to the print file if requested
    */
    if (prt_fd != NULL)
	{
	fprintf(prt_fd,
	    "Latitude/Longitude:  %13.8f %13.8f   Line/Sample:  %5d %5d\n",
	    (float)lats[i],(float)longs[i],lines[i],samps[i]);
	}
    }

return;
}
