/*******************************************************************************
NAME:		DEGLS

PURPOSE:	Convert line/sample pairs to latitude/longitude paris in
		total degrees

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "model.h"
#include "protos.h"

void degls
(
struct MODEL *model,  	/* I: Model and orbital information		*/
double  unit,           /* I: DEG or RAD unit scale factor              */
long    totpairs,       /* I: Total number of line/sample pairs		*/
long    lines[],        /* I: Lines to convert				*/
long    samps[],        /* I: Samples to convert			*/
double  lats[],		/* O: Output latitude values			*/
double  longs[],  	/* O: Output longitude values			*/
FILE    *prt_fd         /* I: Print file descriptor                     */
)

{
long	i;		/* Looping variable				*/

double	lat,		/* Calculated latitude value			*/
	lng,		/* Calculated longitude value			*/
	elev = 0.0;	/* Elevation constant				*/

float	line,		/* Line value to convert			*/
	sample;		/* Sample value to convert			*/

/*
  Convert the line/sample pairs
*/
for (i = 0; i < totpairs; i++)
    {
    /*
      Get the latitude/longitude values
    */
    line   = (float)lines[i];
    sample = (float)samps[i];
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);

    /*
      Convert to degrees.  Unit is set appropratly in getpar.
    */
    lats[i]  = lat * unit;
    longs[i] = lng * unit;

    /*
      Write the results to the print file if requested
    */
    if (prt_fd != NULL)
	{
	fprintf(prt_fd,
	    "Line/Sample:  %5d %5d    Latitude/Longitude:  %13.8f %13.8f\n",
	    lines[i],samps[i],(float)lats[i],(float)longs[i]);
	}
    }

return;
}
