/*******************************************************************************
NAME:		DMSLL

PURPOSE:	Convert latitude/longitude pairs in Degrees, Minutes, and
		Seconds (DMS) to line/sample pairs

ALGORITHM REFERENCES:

   Expected format of the input latitude/longitude coordinates is:
	"DDDD MM SS"
******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "model.h"
#include "constants.h"
#include "protos.h"

void dmsll
(
struct MODEL  *model,  	/* I: Model and orbital information		*/
long	totpairs,	/* I: Total number of latitude/longitude pairs	*/
char    slats[][11],    /* I: Latitudes in DMS format                   */
char    slongs[][11],   /* I: Longitudes in DMS format                  */
long	lines[],	/* O: Converted line numbers			*/
long	samps[],	/* O: Converted sample numbers	         	*/
FILE	*prt_fd 	/* I: Print file descriptor			*/
)

{
long	i,		/* Looping variable				*/
	deg,		/* Temp variable for Degrees			*/
	mns,		/* Temp variable for Minutes			*/
	sec,		/* Temp variable for Seconds			*/
	sign;		/* Sign of the latitude/longitude		*/

double	lat,		/* Latitude of the input			*/
	lng,		/* Longitude of the input			*/
	elev = 0.0;	/* Elevation					*/

float	line,		/* Calculated line				*/
	sample;		/* Calculated sample				*/

/*
  Convert the latitude/longitude pairs
*/
for (i = 0; i < totpairs; i++)
    {
    /*
      Extract the latitude degree, minutes, and seconds.
    */
    sscanf(slats[i],"%4d %2d %2d",&deg,&mns,&sec);

    /*
      Calculate the latitude in radians
    */
    if (deg < 0)
	sign = -1;
    else
	sign = 1;
    lat = sign * (abs(deg) + mns / 60.0 + sec / 3600.0) * RAD_PER_DEG;

    /*
      Extract the longtitude degree, minutes, and seconds.
    */
    sscanf(slongs[i],"%4d %2d %2d",&deg,&mns,&sec);

    /*
      Calculate the longtitude in radians
    */
    if (deg < 0)
	sign = -1;
    else
	sign = 1;
    lng = sign * (abs(deg) + mns / 60.0 + sec / 3600.0) * RAD_PER_DEG;

    /*
      Get the line and sample
    */
    c_ll2ls(&lat,&lng,&elev,model,&line,&sample,NULL);

    /*
      Rounding!
    */
    lines[i] = (long)(line + 0.05);
    samps[i] = (long)(sample + 0.05);

    /*
      Write the results to the print file if requested
    */
    if (prt_fd != NULL)
        {
        fprintf(prt_fd,
            "Latitude/Longitude:  %s  %s   Line/Sample:  %5d %5d\n",
            slats[i],slongs[i],lines[i],samps[i]);
        }
    }

return;
}
