/*******************************************************************************
NAME:		DMSLS

PURPOSE:	Convert line/sample pairs to latitude/longitude paris in
		Degrees, Minutes and Seconds

ALGORITHM REFERENCES:
   The output latitude/longitudes will be in DMS format:
	(DDDD MM SS)
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"	
#include "model.h"
#include "constants.h"
#include "protos.h"

void dmsls
(
struct MODEL  *model,  	/* I: Model and orbital information		*/
long	totpairs,	/* I: Total number of latitude/longitude pairs  */
long	lines[],	/* I: Line numbers to convert			*/
long	samps[],	/* I: Sample numbers to convert			*/
char	slats[][11],	/* O: Converted DMS latitudes			*/
char	slongs[][11],	/* O: Converted DMS longtitudes	        	*/
FILE	*prt_fd 	/* I: Print file descriptor                     */
)

{
long	i,		/* Looping variable                             */
	deg,		/* Temp variable for Degrees                    */
	mns,		/* Temp variable for Minutes                    */
	sec;		/* Temp variable for Seconds                    */

double	lat,		/* Latitude of the output			*/
	lng,		/* Longitude of the output			*/
	elev = 0.0,	/* Elevation constant				*/
	temp;		/* Temp value for latitude/longitude conversion	*/

float	line,		/* Line to calculate from			*/
	sample;		/* Sample to calculate from			*/

/*
  Convert the line/sample pairs
*/
for (i = 0; i < totpairs; i++)
    {
    /*
      Convert the line/sample
    */
    line   = (float)lines[i];
    sample = (float)samps[i];
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);

    /*
      Find the degree, minute, and seconds of the latitude
    */
    temp = fabs((lat * DEG_PER_RAD));
    deg = (long)temp;
    mns = (long)((temp - deg) * 60);
    sec = (long)((((temp - deg) * 60) - mns) * 60);
    if (lat < 0.0)
	deg *= -1;

    /*
      Convert to the DMS format
    */
    sprintf(slats[i],"%4d %02d %02d",deg,mns,sec);

    /*
      Find the degree, minute, and seconds of the latitude
    */
    temp = fabs((lng * DEG_PER_RAD));
    deg = (long)temp;
    mns = (long)((temp - deg) * 60);
    sec = (long)((((temp - deg) * 60) - mns) * 60);
    if (lng < 0.0)
	deg *= -1;

    /*
      Convert to the DMS format
    */
    sprintf(slongs[i],"%4d %02d %02d",deg,mns,sec);

    /*
      Write the results to the print file if requested
    */
    if (prt_fd != NULL)
	{
	fprintf(prt_fd,
	    "Line/Sample:  %5d %5d    Latitude/Longitude:  %s  %s\n",
	    lines[i],samps[i],slats[i],slongs[i]);
	}
    }

return;
}
