/****************************************************************************
NAME:		GETPAR

PURPOSE:	Get user entered parameters from the TAE parameter block

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "constants.h"
#include "util.h"
#include "protos.h"

/*
  Defines for print options
*/
#define TERM   0
#define LP     1
#define FIL    2

/*
  Conversion options
*/
#define	DMS	0.0
#define	RAD	1.0

int getpar
(
struct PARBLK   *vblock,   /* I: TAE variable block                          */
char    *hosin,            /* O: Host file names of input images             */
char	*subcmd,	   /* O: Subcommand used to run application          */
long    *totpairs,         /* O: Total number of pairs to convert            */
char    slats[][11],       /* O: Latitudes in the DMS format                 */
char    slongs[][11],      /* O: Longitudes in the DMS format                */
double  lats[],            /* O: Latitudes to convert/return (DEG or RAD)    */
double  longs[],           /* O: Longtitudes to convert/return (DEG or RAD)  */
long    lines[],           /* O: Lines to convert/return                     */
long    samps[],           /* O: Lines to convert/return                     */
double  *unit,             /* O: Unit the coordinates were entered/returned  */
FILE    **prt_fd,          /* O: Print file descriptor                       */
char    *prtfile,	   /* O: Print file				     */
char    *ofiles,	   /* O: List of output files created		     */
long 	*ocount 	   /* O: Number of output files created		     */
)

{
struct VARIABLE *val;	   /* Pointer to a TAE variable in vblock	     */

long	access = IREAD,	   /* File access indicatori		             */
	print[3],	   /* Print options				     */
	status,		   /* Status variable				     */
	i,		   /* Counter variable				     */
	zero = 0,          /* Constant zero				     */
	fatal = -1,	   /* Fatal error code				     */
	nonfatal = 1;	   /* Nonfatal error code			     */

/*
  Get the subcommand the application was run with
*/
val = p_find(vblock,"SUBCMD");
strcpy(subcmd,SVAL(*val,0));

/*
  Get the input ADDR
*/
status = c_getnam(vblock,"INFILE",&zero,hosin,"addr",&access);
if (status != E_SUCC)
    return(E_FAIL); 

/*
  Get the coordinate unit
*/
val = p_find(vblock,"COORUNIT");
if (strcmp(SVAL(*val,0),"DEG") == 0)
    {
    if (strcmp(subcmd,"LINESAMP") == 0)
	*unit = RAD_PER_DEG;
    else
	*unit = DEG_PER_RAD;
    }
else if (strcmp(SVAL(*val,0),"RAD") == 0)
    *unit = RAD;
else 
    *unit = DMS;

/*
  Get the latitude/longitude pairs or the line/sample pairs
*/
if (strcmp(subcmd,"LINESAMP") == 0)
    {
    if (*unit == DMS)
	{
	/*
	  Get the latitudes in DMS format
	*/
	val = p_find(vblock,"SLAT");
	*totpairs = val->v_count;
	if (*totpairs > 0)
	    {
	    for (i = 0; i < *totpairs; i++)
		strcpy(slats[i],SVAL(*val,i));
	    }
	else
	    {
	    c_errmsg("No latitudes specified","avcoord-nolat",&nonfatal);
	    c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);
	    }

	/*
	  Get the longitudes in DMS format
	*/
	val = p_find(vblock,"SLONG");
	if (val->v_count != *totpairs)
	   {
	   c_errmsg("The number of latitude and longitude values must be equal",
		    "avcoord-nvals",&nonfatal);
	   c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);
	   }

	for (i = 0; i < *totpairs; i++)
	    strcpy(slongs[i],SVAL(*val,i));
	}
    else
	{
	/*
	  Get the latitudes
	*/
	val = p_find(vblock,"LAT");
	*totpairs = val->v_count;
	if (*totpairs > 0)
	    {
	    for (i = 0; i < *totpairs; i++)
		lats[i] = RVAL(*val,i);
	    }
	else
	    {
	    c_errmsg("No latitudes specified","avcoord-nolat",&nonfatal);
	    c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);
	    }

	/*
	  Get the longitudes
	*/
	val = p_find(vblock,"LONG");
	if (val->v_count != *totpairs)
	   {
	   c_errmsg("The number of latitude and longitude values must be equal",
		    "avcoord-nvals",&nonfatal);
	   c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);
	   }

	for (i = 0; i < *totpairs; i++)
	    longs[i] = RVAL(*val,i);
	}
    }
else
    {
    /*
      Get the lines
    */
    val = p_find(vblock,"LINE");
    *totpairs = val->v_count;
    if (*totpairs > 0)
	{
	for (i = 0; i < *totpairs; i++)
            {
	    lines[i] = IVAL(*val,i);
            if (lines[i] <= 0)
               {
               c_errmsg("Line must be greater than zero","avcoord-line",
                        &nonfatal);
               c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);
               }
            }
	}
    else
	{
	c_errmsg("No lines specified","avcoord-noline",&nonfatal);
	c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);
	}
   
    /*
      Get the samples
    */
    val = p_find(vblock,"SAMPLE");
    if (val->v_count != *totpairs)
	{
	c_errmsg("The number of line and sample values must be equal",
		 "avcoord-nvals",&nonfatal);
        c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);
	}

    for (i=0; i<val->v_count; i++)
        {
	samps[i] = IVAL(*val,i);
            if (samps[i] < 0)
               {
               c_errmsg("Sample must be a greater than zero","avcoord-sample",
                        &nonfatal);
               c_errmsg("Fatal error encountered","avcoord-fatal",&fatal);
               }
	}
    }

/*
  Get the print option
*/
strcpy(prtfile,"avcoord");
status = c_setprt(vblock,(int *)&print[TERM],(int *)&print[FIL],
    (int *)&print[LP],prtfile,ofiles);
if (status != E_SUCC)
    return(E_FAIL);

/*
  When a report is generated count it as an output file created
*/
if (print[FIL])
    *ocount += 1;

/*
  Open a print file if specified by the user
*/
if (print[TERM] || print[FIL] || print[LP])
    {
    *prt_fd = fopen(prtfile,"w");
    if (*prt_fd == NULL)
        {
        c_errmsg("Error opening output print file","avcoord-open",&nonfatal);
	return(E_FAIL);
        }
    }
else
    *prt_fd = NULL;

return(E_SUCC);
}
