#ifndef PROTOT_H
#define PROTOS_H

#include "model.h"

/* degll.c */
void degll
(
struct MODEL  *model,   /* I: Model and orbital information                */
double  unit,           /* I: DEG or RAD unit scale factor                 */
long    totpairs,       /* I: Total number of latitude/longitude pairs     */
double  lats[],         /* I: Input latitudes in DEG or RAD                */
double  longs[],        /* I: Input longitudes in DEG or RAD               */
long    lines[],        /* O: Converted line numbers                       */
long    samps[],        /* O: Converted sample numbers                     */
FILE    *prt_fd         /* I: Print file descriptor                        */
);

/* degls.c */
void degls
(
struct MODEL *model,    /* I: Model and orbital information                */
double  unit,           /* I: DEG or RAD unit scale factor                 */
long    totpairs,       /* I: Total number of line/sample pairs            */
long    lines[],        /* I: Lines to convert                             */
long    samps[],        /* I: Samples to convert                           */
double  lats[],         /* O: Output latitude values                       */
double  longs[],        /* O: Output longitude values                      */
FILE    *prt_fd         /* I: Print file descriptor                        */
);

/* dmsll.c */
void dmsll
(
struct MODEL  *model,   /* I: Model and orbital information                */
long    totpairs,       /* I: Total number of latitude/longitude pairs     */
char    slats[][11],    /* I: Latitudes in DMS format                      */
char    slongs[][11],   /* I: Longitudes in DMS format                     */
long    lines[],        /* O: Converted line numbers                       */
long    samps[],        /* O: Converted sample numbers                     */
FILE    *prt_fd         /* I: Print file descriptor                        */
);

/* dmsls.c */
void dmsls
(
struct MODEL  *model,   /* I: Model and orbital information                */
long    totpairs,       /* I: Total number of latitude/longitude pairs     */
long    lines[],        /* I: Line numbers to convert                      */
long    samps[],        /* I: Sample numbers to convert                    */
char    slats[][11],    /* O: Converted DMS latitudes                      */
char    slongs[][11],   /* O: Converted DMS longtitudes                    */
FILE    *prt_fd         /* I: Print file descriptor                        */
);

/* getpar.c */
int getpar
(
struct PARBLK   *vblock,   /* I: TAE variable block                         */
char    *hosin,            /* O: Host file names of input images            */
char    *subcmd,           /* O: Subcommand used to run application         */
long    *totpairs,         /* O: Total number of pairs to convert           */
char    slats[][11],       /* O: Latitudes in the DMS format                */
char    slongs[][11],      /* O: Longitudes in the DMS format               */
double  lats[],            /* O: Latitudes to convert/return (DEG or RAD)   */
double  longs[],           /* O: Longtitudes to convert/return (DEG or RAD) */
long    lines[],           /* O: Lines to convert/return                    */
long    samps[],           /* O: Lines to convert/return                    */
double  *unit,             /* O: Unit the coordinates were entered/returned */
FILE    **prt_fd,          /* O: Print file descriptor                      */
char    *prtfile,          /* O: Print file                                 */
char    *ofiles,           /* O: List of output files created               */
long    *ocount            /* O: Number of output files created             */
);
#endif
