#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "vll.h"
#include "calcoef.h"

#define NODEBUG TRUE 		/* set to FALSE when debugging with dbx	*/

#define NCHANS	5		/* Number of AVHRR channels		*/
#define REPORT	1		/* output report file			*/

/* Prototypes */

void clip_loop
(
struct CALSTR *calstr,          /* Calibration parameter structure      */
long    window[],               /* Image window specification           */
long    channel[],              /* Channels to process                  */
long    maxchan,                /* Maximum number of channels to process*/
float   cliplim[NCHANS][2],     /* Clipping limits (scaled)             */
long    clipcnt[NCHANS][2],     /* Clipping statistics                  */
long offset2[NCHANS],           /* Offsets into "buffer" for output bands*/   
unsigned char *buffer           /* image I/O buffer                     */
);

void clip_updt
(
struct CALSTR *calstr,          /* Calibration parameter structure      */
float   ovalue,                 /* Temp variable for the output pixel   */
long    channel[],              /* Channels to process                  */
long    chan,                   /* Looping variables                    */
float   cliplim[NCHANS][2],     /* Clipping limits (scaled)             */
long clipcnt[NCHANS][2]         /* Clipping statistics                  */
);

void clip_writ
(
FILE    *fptr,                  /* file descriptor for report file      */
struct  CALSTR *calstr,         /* Calibration parameter structure      */
float   clipact[NCHANS][2],     /* Clipping limits (actual)             */
float   cliplim[NCHANS][2],     /* Clipping limits (scaled)             */
long    clipcnt[NCHANS][2],     /* Clipping occurrences (counts)        */
long    channel[],              /* AVHRR channels to process            */
long    maxchan,                /* Number of channels in "channel"      */
long    totpix                  /* Total number of pixels in a band     */
);

int clip_init
(
struct  CALSTR *calstr,         /* Calibration parameter structure      */
float   clipact[NCHANS][2],     /* Clipping limits (actual)             */
float   cliplim[NCHANS][2],     /* Clipping limits (scaled)             */
long	clipcnt[NCHANS][2]      /* Clipping statistics                  */
);

void getpar
(
struct  PARBLK *vblock,         /* TAE parameter block                  */
char    taein[][CMLEN],         /* TAE input name                       */
char    hosin[][CMLEN],         /* Host input name                      */
char    hosmnr[CMLEN],          /* host minor frame label services name */
long    window[][4],            /* Window specification                 */
long    bands[][MAXBND+1],      /* band specification                   */
long    nbands[],               /* Number of bands                      */
long    *nimgs,                 /* Number of images in input            */
long    *totbnd,                /* Total number of bands                */
char    taeout[][CMLEN],        /* TAE output name                      */
char    hosout[][CMLEN],        /* Host output name                     */
long    channel[],              /* Channels needed                      */
long    *maxchan,               /* Number of channels needed            */
long    *msgtim,                /* Time between processing messages     */
char    *minmax,                /* Min/Max flag                         */
struct  MODEL  *model,          /* Model and orbital information        */
long    *nout,                  /* Number of output files               */
struct  CALSTR *calstr,         /* Calibration structure                */
long    *mline                  /* Image master line                    */
);

void open_las
(
struct  PARBLK  *vblock,        /* TAE parameter block                  */
char    hosin[][CMLEN],         /* Input host file names                */
long    *nimgs,                 /* Number of input images               */
char    *hosout,                /* Output host file names               */
long    window[][4],            /* Window specification of input images */
long    bands[][MAXBND+1],      /* Bands to be processed for all images */
long    nbands[],               /* Number of bands per input image      */
long    *totbnds,               /* Total bands to be processed          */
long    *odtype,                /* Data type of output image            */
struct  MODEL   *model,         /* Model and orbital information        */
int     fdesc[MAXIMG+1],        /* I/O image file descriptors           */
int     *gdesc,                 /* group descriptor used for image I/O  */
long    *nout,                  /* Number of output files               */
long    offset1[MAXIMG*NCHANS], /* offsets into "buffer" for input bands*/
long    offset2[NCHANS],        /* offsets into "buffer" for output bands*/
unsigned char **buffer          /* image I/O buffer                     */
);

void process
(
int     *gdesc,                 /* Group descriptor used for image I/O  */
struct  PARBLK *vblock,         /* TAE paramter block                   */
long    window[],               /* Window to process                    */
long    mline,                  /* Image master line number             */
long    channel[],              /* Channels to process                  */
long    maxchan,                /* Maximum number of channels to process*/
char    *minmax,                /* Min/max flag                         */
double  omin[NCHANS],           /* min pixel value array                */
double  omax[NCHANS],           /* max pixel value array                */
long    *msgtim,                /* Time between processing messages     */
char    *hosmnr,                /* Input minor frame label services name*/
char    hosout[][CMLEN],        /* Output host file names               */
long    *nout,                  /* Number of output files               */
float   clipact[NCHANS][2],     /* Clipping limits (actual)             */
float   cliplim[NCHANS][2],     /* Clipping limits (scaled)             */
long    clipcnt[NCHANS][2],     /* Clipping statistics                  */
long    offset1[MAXIMG*NCHANS], /* Offsets into "buffer" for input bands*/
long    offset2[NCHANS],        /* offsets into "buffer" output bands   */
unsigned char   *buffer,        /* image I/O buffer                     */
struct  MODEL   *model,         /* Model                                */
struct  CALSTR  *calstr,        /* Calibration parameters structure     */
FILE    *fptr,                  /* Pointer to report file               */
struct COEFFSTR *sumsqr,        /* Sum of squared values for each coeff */
struct COEFFSTR *wtmean,        /* Weighted mean for each coeff         */
struct COEFFSTR *stdev          /* Std dev for each coeff               */
);

void report_open
(
char ohfiles[][CMLEN],  /* Calibration host file name   */
FILE **fptr,            /* File pointer to report file  */
struct COEFFSTR *sumsqr,/* Sum of squared values for each coeff */
struct COEFFSTR *wtmean,/* Weighted mean for each coeff         */
struct COEFFSTR *stdev  /* Std dev for each coeff               */
);

void report_init
(
struct COEFFSTR *sumsqr,        /* Sum of squared values for each coeff */
struct COEFFSTR *wtmean,        /* Weighted mean for each coeff         */
struct COEFFSTR *stdev          /* Std dev for each coeff               */
);

void report_updt
(
long    line,                   /* Line in image being processed        */
long    *lastline,              /* Last line when coeff were updated    */
struct CALSTR   *calstr,        /* Calibration parameters structure     */
struct COEFFSTR *coeff,         /* All coefficients (optical, thermal)  */
struct COEFFSTR *sumsqr,        /* Sum of squared values for each coeff */
struct COEFFSTR *wtmean         /* Weighted mean for each coeff         */
);

void report_fini
(
long    line,                   /* Line in image being processed        */
long    lastline,               /* Last line when coeff were updated    */
long    nl,                     /* Number of lines in image to process  */
struct CALSTR   *calstr,        /* Calibration parameters structure     */
struct COEFFSTR *coeff,         /* All coefficients (optical, thermal)  */
struct COEFFSTR *sumsqr,        /* Sum of squared values for each coeff */
struct COEFFSTR *wtmean,        /* Weighted mean for each coeff         */
struct COEFFSTR *stdev          /* Std dev for each coeff               */
);

void report_writ
(
FILE    *fptr,                  /* file descriptor for report file      */
struct  COEFFSTR *wtmean,       /* Weighted mean for each coeff         */
struct  COEFFSTR *stdev,        /* Std dev for each coeff               */
struct CALSTR    *calstr,       /* Calibration parameters structure     */
long    start_line,             /* Starting line of the cal period      */
long    end_line,               /* Ending line of the cal period        */
long    satnum                  /* Satellite number (14 = noaa14)       */
);

void report_clos
(
FILE    *fptr,                  /* file descriptor for report file      */
struct CALSTR *calstr           /* Calibration parameters structure     */
);

