/*******************************************************************************
FUNCTION:	CLIP_LOOP

PURPOSE:	Loop thru a buffer of output data to determine the
		clipping statistics.

ALGORITHM:

ALGORITHM REFERENCES
*******************************************************************************/

#include "las.h"
#include "adapssup.h"
#include "avhrrcal.h"

void clip_loop
(
struct CALSTR *calstr,		/* Calibration parameter structure	*/
long	window[],		/* Image window specification		*/
long	channel[],		/* Channels to process			*/
long	maxchan,		/* Maximum number of channels to process*/
float	cliplim[NCHANS][2],	/* Clipping limits (scaled)		*/
long	clipcnt[NCHANS][2],	/* Clipping statistics			*/
long offset2[NCHANS],		/* Offsets into "buffer" for output bands*/
unsigned char *buffer		/* image I/O buffer			*/
)

{

static long datasize[NDTYPES] = {0,1,2,4,4,8};


float	ovalue;			/* Temp variable for the output pixel	*/

unsigned char 	*outpix1;	/* Pointer into the output buffer	*/
short 	*outpix2;		/* Pointer into the output buffer	*/
long 	*outpix3;		/* Pointer into the output buffer	*/
float 	*outpix4;		/* Pointer into the output buffer	*/

long	chan, samp;       /* Looping variables				*/


/*
  Lookup the output value for each sample of each band that is needed
*/
for (chan = 0; chan < maxchan; chan++)
    {
    if (calstr->odtype == EBYTE)
	outpix1 = (unsigned char *)&buffer[offset2[chan]*datasize[EBYTE]];
    else if (calstr->odtype == EWORD)
	outpix2 = (short *)&buffer[offset2[chan]*datasize[EWORD]];
    else if (calstr->odtype == ELONG)
	outpix3 = (long *)&buffer[offset2[chan]*datasize[ELONG]];
    else if (calstr->odtype == EREAL)
	outpix4 = (float *)&buffer[offset2[chan]*datasize[EREAL]];

    /*
      Process the samples from a line
    */
    for (samp = 0; samp < window[NS]; samp++)
	{
	/*
	  Set the output pixel value.
	*/
	if (calstr->odtype == EBYTE)
	    {
	    ovalue = (float)*outpix1;
	    outpix1++;
	    }
	else if (calstr->odtype == EWORD)
	    {
	    ovalue = (short)*outpix2;
	    outpix2++;
	    }
	else if (calstr->odtype == ELONG)
	    {
	    ovalue = (float)(long)*outpix3;
	    outpix3++;
	    }
	else if (calstr->odtype == EREAL)
	    {
	    ovalue = *outpix4;
	    outpix4++;
	    }

	/*
	  Update clipping statistics for this output pixel
	*/
	clip_updt(calstr,ovalue,channel,chan,cliplim,clipcnt);
	}

    }

return;
}

/*******************************************************************************
FUNCTION:	CLIP_UPDT

PURPOSE:	Update the clipping statistics for the report.

ALGORITHM:

ALGORITHM REFERENCES
*******************************************************************************/

void clip_updt
(
struct CALSTR *calstr,		/* Calibration parameter structure	*/
float	ovalue,			/* Temp variable for the output pixel	*/
long	channel[],		/* Channels to process			*/
long	chan,			/* Looping variables			*/
float	cliplim[NCHANS][2],	/* Clipping limits (scaled)		*/
long clipcnt[NCHANS][2]		/* Clipping statistics			*/
)

{

/*
  Update clipping statistics
*/
if (calstr->odtype == EBYTE)
    {
    if ((unsigned char)ovalue == (unsigned char)cliplim[channel[chan]-1][0])
	clipcnt[channel[chan]-1][0]++;
    else if ((unsigned char)ovalue == (unsigned char)cliplim[channel[chan]-1][1])
	clipcnt[channel[chan]-1][1]++;
    }
else if (calstr->odtype == EWORD)
    {
    if ((short)ovalue == (short)cliplim[channel[chan]-1][0])
	clipcnt[channel[chan]-1][0]++;
    else if ((short)ovalue == (short)cliplim[channel[chan]-1][1])
	clipcnt[channel[chan]-1][1]++;
    }
else if (calstr->odtype == ELONG)
    {
    if ((long)ovalue == (long)cliplim[channel[chan]-1][0])
	clipcnt[channel[chan]-1][0]++;
    else if ((long)ovalue == (long)cliplim[channel[chan]-1][1])
	clipcnt[channel[chan]-1][1]++;
    }
else if (calstr->odtype == EREAL)
    {
    if (ovalue == cliplim[channel[chan]-1][0])
	clipcnt[channel[chan]-1][0]++;
    else if (ovalue >= cliplim[channel[chan]-1][1])
	clipcnt[channel[chan]-1][1]++;
    }
}

/******************************************************************************
FUNCTION:	CLIP_WRIT

PURPOSE:	Write the calibration clipping (saturation) summary

*******************************************************************************/

void clip_writ
(
FILE 	*fptr,			/* file descriptor for report file	*/
struct	CALSTR *calstr,		/* Calibration parameter structure	*/
float	clipact[NCHANS][2],	/* Clipping limits (actual)		*/
float	cliplim[NCHANS][2],	/* Clipping limits (scaled)		*/
long	clipcnt[NCHANS][2],	/* Clipping occurrences (counts)	*/
long	channel[],		/* AVHRR channels to process		*/
long	maxchan,		/* Number of channels in "channel"	*/
long	totpix 			/* Total number of pixels in a band	*/
)

{
long 	i, j;			/* loop counter				*/

float	uclipper;		/* Upper clipping percent		*/
float	lclipper;		/* Lower clipping percent		*/


/* Write the statistics to the report
   -----------------------------------*/

fprintf(fptr,"\n                           AVHRR calibration saturation statistics     ");
fprintf(fptr,"\n                          -----------------------------------------\n\n");
fprintf(fptr," AVHRR     lower      upper       lower      upper       lower       lower      upper      upper   \n");
fprintf(fptr,"channel  saturation  saturation  saturation saturation  saturation saturation saturation saturation\n");
fprintf(fptr,"          (actual)   (actual)    (scaled)   (scaled)   occurrences  (percent) occurrences (percent)\n");
fprintf(fptr,"---------------------------------------------------------------------------------------------------\n");
for (j = 0; j < maxchan; j++)
    {
    i = channel[j] - 1;
    lclipper = (float)clipcnt[i][0] / (float)totpix * 100.0;
    uclipper = (float)clipcnt[i][1] / (float)totpix * 100.0;
    if (calstr->odtype == EBYTE)
        fprintf(fptr,"  %1d    %7d     %7d     %7d    %7d      %8d     %6.2f   %8d     %6.2f \n",
		i+1,(long)clipact[i][0],(long)clipact[i][1],
		(unsigned char)cliplim[i][0],(unsigned char)cliplim[i][1],
		clipcnt[i][0],lclipper,clipcnt[i][1],uclipper);    
    else if (calstr->odtype == EWORD)
        fprintf(fptr,"  %1d    %7d     %7d     %7d    %7d      %8d     %6.2f   %8d     %6.2f \n",
		i+1,(long)clipact[i][0],(long)clipact[i][1],
		(short)cliplim[i][0],(short)cliplim[i][1],
		clipcnt[i][0],lclipper,clipcnt[i][1],uclipper);    
    else if (calstr->odtype == ELONG)
        fprintf(fptr,"  %1d    %7d     %7d     %7d    %7d      %8d     %6.2f   %8d     %6.2f \n",
		i+1,(long)clipact[i][0],(long)clipact[i][1],
		(long)cliplim[i][0],(long)cliplim[i][1],
		clipcnt[i][0],lclipper,clipcnt[i][1],uclipper);    
    else if (calstr->odtype == EREAL)
        fprintf(fptr,"  %1d     %6.1f      %6.1f      %6.1f     %6.1f      %8d     %6.2f   %8d     %6.2f \n",
		i+1,clipact[i][0],clipact[i][1],
		cliplim[i][0],cliplim[i][1],
		clipcnt[i][0],lclipper,clipcnt[i][1],uclipper);    
    }

if (calstr->odtype == EBYTE)
    fprintf(fptr,"\n\n Output data type: BYTE   ");
else if (calstr->odtype == EWORD)
    fprintf(fptr,"\n\n Output data type: INTEGER*2   ");
else if (calstr->odtype == ELONG)
    fprintf(fptr,"\n\n Output data type: INTEGER*4   ");
else if (calstr->odtype == EREAL)
    fprintf(fptr,"\n\n Output data type: REAL*4   ");

fprintf(fptr," Scaling option: %s ",calstr->scale_opt);

if (calstr->rad_opt)
    fprintf(fptr,"\n\n Units of 'actual' values, visible bands: Radiance ");
else
    fprintf(fptr,"\n\n Units of 'actual' values, visible bands: Percent reflectance ");

fprintf(fptr,"\n Units of 'actual' values, thermal bands: Temperature (K) ");

/* Close files & return
   --------------------*/
fclose(fptr);

return;
}

/*******************************************************************************
FUNCTION:	CLIP_INIT

PURPOSE:	Initialize the clipping report statistics.

ALGORITHM:

ALGORITHM REFERENCES
*******************************************************************************/

int clip_init
(
struct 	CALSTR *calstr,		/* Calibration parameter structure	*/
float	clipact[NCHANS][2],	/* Clipping limits (actual)		*/
float	cliplim[NCHANS][2],	/* Clipping limits (scaled)		*/
long	clipcnt[NCHANS][2]	/* Clipping statistics			*/
)

{
long    soption;        /* Scaling option as expected by c_getscale()	*/
long	status;		/* Return status variable			*/
long	i;		  /* Looping variables				*/

struct  SCALESTR a_scale;	/* Reflectance scaling structure	*/
struct  SCALESTR r_scale;	/* Radiance scaling structure		*/
struct  SCALESTR t_scale;	/* Temperature scaling structure	*/

double  a_unscale;              /* Reflectance unscale factor           */
double	r_unscale;              /* Radiance unscale factor              */
double	t_unscale;              /* Temperature unscale factor           */


/*
  Set the scaling based on the scaling option and the quantity.
  Get the scaling for reflectance, radiance, temperature.
*/
c_getscaleopt(calstr->scale_opt,calstr->odtype,&soption);
status = c_getscale(soption,"reflectance",&a_scale.minval,&a_scale.maxval,
                  &a_scale.scale,&a_unscale,&a_scale.offset);
if (status != E_SUCC)
    return(status);

status = c_getscale(soption,"radiance",&r_scale.minval,&r_scale.maxval,
                  &r_scale.scale,&r_unscale,&r_scale.offset);
if (status != E_SUCC)
    return(status);

status = c_getscale(soption,"thermal",&t_scale.minval,&t_scale.maxval,
                  &t_scale.scale,&t_unscale,&t_scale.offset);
if (status != E_SUCC)
    return(status);

/*
Initialize the clipping statistics arrays;
clipact[NCHANS][0] - lower clipping limit (actual) per band
clipact[NCHANS][1] - upper clipping limit (actual) per band
cliplim[NCHANS][0] - lower clipping limit (scaled) per band
cliplim[NCHANS][1] - upper clipping limit (scaled) per band
clipcnt[NCHANS][0] - occurrences of lower clipping limit per band
clipcnt[NCHANS][1] - occurrences of upper clipping limit per band
*/
if (calstr->rad_opt)
    {
    cliplim[0][0] = cliplim[1][0] = (r_scale.minval * r_scale.scale)
   				    + r_scale.offset;
    cliplim[0][1] = cliplim[1][1] = (r_scale.maxval * r_scale.scale)
				    + r_scale.offset;
    clipact[0][0] = clipact[1][0] = r_scale.minval;
    clipact[0][1] = clipact[1][1] = r_scale.maxval;
    }
else
    {
    cliplim[0][0] = cliplim[1][0] = (a_scale.minval * a_scale.scale)
				    + a_scale.offset;
    cliplim[0][1] = cliplim[1][1] = (a_scale.maxval * a_scale.scale)
				    + a_scale.offset;
    clipact[0][0] = clipact[1][0] = a_scale.minval;
    clipact[0][1] = clipact[1][1] = a_scale.maxval;
    }
cliplim[2][0] = cliplim[3][0] = cliplim[4][0] = (t_scale.minval * t_scale.scale)
						+ t_scale.offset;
cliplim[2][1] = cliplim[3][1] = cliplim[4][1] = (t_scale.maxval * t_scale.scale)
						+ t_scale.offset;
clipact[2][0] = clipact[3][0] = clipact[4][0] = t_scale.minval;
clipact[2][1] = clipact[3][1] = clipact[4][1] = t_scale.maxval;

/* Round non-floating point values */
for (i = 0; i < NCHANS; i++)
    {
    if (calstr->odtype == EBYTE)
	{
	cliplim[i][0] = (unsigned char)(cliplim[i][0] + 0.5);
	cliplim[i][1] = (unsigned char)(cliplim[i][1] + 0.5);
	}
    else if (calstr->odtype == EWORD)
	{
	cliplim[i][0] = (short)(cliplim[i][0] + 0.5);
	cliplim[i][1] = (short)(cliplim[i][1] + 0.5);
	}
    else if (calstr->odtype == ELONG)
	{
	cliplim[i][0] = (float)(long)(cliplim[i][0] + 0.5);
	cliplim[i][1] = (float)(long)(cliplim[i][1] + 0.5);
	}
    }

/* Initialize to zero occurrences */
for (i = 0; i < NCHANS; i++)
    clipcnt[i][0] = clipcnt[i][1] = 0;

return(E_SUCC);
}
