/*******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Retrieve the input parameters and check their validity

ALGORITHM:

ALGORITHM REFERENCES:
*******************************************************************************/
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "avhrrcal.h"

void getpar
(
struct	PARBLK *vblock,		/* TAE parameter block			*/
char	taein[][CMLEN],		/* TAE input name			*/
char	hosin[][CMLEN],		/* Host input name			*/
char	hosmnr[CMLEN],		/* host minor frame label services name	*/
long	window[][4],		/* Window specification			*/
long	bands[][MAXBND+1],	/* band specification			*/
long	nbands[],		/* Number of bands			*/
long	*nimgs,			/* Number of images in input		*/
long	*totbnd,		/* Total number of bands		*/
char	taeout[][CMLEN],	/* TAE output name			*/
char	hosout[][CMLEN],	/* Host output name			*/
long	channel[],		/* Channels needed			*/
long	*maxchan,		/* Number of channels needed		*/
long	*msgtim,		/* Time between processing messages	*/
char	*minmax,		/* Min/Max flag				*/
struct	MODEL  *model,		/* Model and orbital information	*/
long	*nout,			/* Number of output files		*/
struct	CALSTR *calstr,		/* Calibration structure		*/
long	*mline 			/* Image master line			*/
)


{
long	fopt;			/* PRINT flag				*/
long	i;			/* Looping variable			*/
long	imglim = MAXIMG;	/* maximum number of input images	*/
long	len;			/* Length of the input tae name		*/
long	lopt;			/* PRINT flag				*/
long	same;			/* flag to check input image windows	*/
long	status;			/* Return status variable		*/
long	topt;			/* PRINT flag				*/
long	zero = 0;		/* Value of zero			*/

static	long qflag[MAXIMG][2];	/* flag to check for subwindow		*/

struct	VARIABLE *var;		/* pointer to a TAE variable		*/
struct	DDR	 ddr;		/* DDR information structure		*/


/* Initialize the channels to be all zeros
------------------------------------------*/
for ( i = 0; i < NCHANS; i++ )
     channel[i] = 0;

/* Get the input images
-----------------------*/
status = c_getin( vblock, "IN", &zero, taein, hosin, nimgs, window, bands,
         nbands, totbnd, &imglim, &same, qflag );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
if ( same == 0 )
    {
    c_lasmsg( FALSE, "avhrrcal-wsiz", "Image windows differ in size");
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
    }
if (*nbands > NCHANS)
    {
    c_lasmsg( FALSE, "avhrrcal-bands", "Too many bands specified");
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
    }
*nout = 1;

/* Check for more than one image name specification.  If the user
specifies an input file name such as IN="file1(:1)+file2(:3)" we will
error out because we need a minor frame associated with an image to
generate the thermal calibration coefficients.  The way c_calcoeffs()
is written we cannot specify more than one minor frame label services
file so error out.
---------------------------------------------------------------------*/
if ( *nimgs > 1 )
    c_lasmsg( TRUE, "avhrrcal-mult", "Multiple input files not allowed" );

/* Make the minor frame label services file name
------------------------------------------------*/
c_lsmknm( hosin[0], "mnr", hosmnr );

/* If a window was specified inform the user that it will not be used
---------------------------------------------------------------------*/
for ( i = 0; i < *nimgs; i++ )
    {
    if ( qflag[i][0] )
	{
	c_lasmsg( FALSE, "avhrrcal-wind",
		"Windowing is not allowed -- full image size will be used" );
	break;
	}
    }

/* Get the output image name
----------------------------*/
status = c_getout( vblock, "OUT", (char (*)[CMLEN])taeout[0],
		   (char (*)[CMLEN])hosout[0] );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );

/* Get the satellite channel numbers
------------------------------------*/
var = p_find( vblock, "SATCHAN" );
*maxchan = (*var).v_count;
if ( *maxchan != *totbnd )
    {
    c_lasmsg( FALSE, "avhrrcal-chans",
		"Number of channels and bands are not equal" );
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
    }

for ( i = 0; i < *maxchan; i++ )
    channel[i] = IVAL( *var, i );

/* Get the output data type
---------------------------*/
var = p_find( vblock, "ODTYPE" );
if ( strcmp( SVAL( *var, 0 ), "BYTE" ) == 0 )
    calstr->odtype = EBYTE;
else if ( strcmp( SVAL( *var, 0 ), "I*2" ) == 0 )
    calstr->odtype = EWORD;
else if ( strcmp( SVAL( *var, 0 ), "I*4" ) == 0 )
    calstr->odtype = ELONG;
else if ( strcmp( SVAL( *var, 0 ), "R*4" ) == 0 )
    calstr->odtype = EREAL;
else
    calstr->odtype = 0;	/* Signal to be same as input data type */

/* Get the temperature range
----------------------------*/
var = p_find( vblock, "TEMPRNG" );
calstr->temp_range = IVAL( *var, 0 );

/* Get the information from the image DDR.
------------------------------------------------------------*/
status = c_getddr( (char *)hosin, &ddr );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
*mline = ddr.master_line;

/* Get the model and orbital information from the AVHRR DDR.
  Needed here to get the satellite number.
------------------------------------------------------------*/
status = c_getaddr( (char *)hosin, model );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );

/* Get the Error option flag
----------------------------*/
var = p_find( vblock, "ERRFLG" );
if ( strcmp( SVAL( *var, 0 ), "YES" ) == 0 )
    calstr->lin_opt = TRUE;
else
    calstr->lin_opt = FALSE;

/* Get the scaling option
-------------------------*/
var = p_find( vblock, "SCALOPT" );
strcpy( calstr->scale_opt, SVAL( *var, 0 ));

/* Get the Reflectance/Radiance option
--------------------------------------*/
var = p_find( vblock, "RADOPT" );
if ( strcmp( SVAL( *var, 0 ), "RADIANCE" ) == 0 )
    calstr->rad_opt = TRUE;
else
    calstr->rad_opt = FALSE;

/* Get the Radiance of space option
-----------------------------------*/
var = p_find( vblock, "ROSOPT" );
if ( strcmp( SVAL( *var, 0 ), "ZERO" ) == 0 )
    calstr->ros_opt = TRUE;
else
    calstr->ros_opt = FALSE;

/* Get the print option and open the temporary file
---------------------------------------------------*/
strcpy( hosout[REPORT], "avhrrcal" );
if (c_setprt( vblock, (int *)&topt, (int *)&fopt, (int *)&lopt, hosout[REPORT],
        taeout[REPORT] ) != E_SUCC )
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
(*nout) += 1;

/* Get the time between processing messages
-------------------------------------------*/
var = p_find( vblock, "$MSGTIM" );
*msgtim = IVAL( *var, 0 );

/* Get the min/max flag
-----------------------*/
var = p_find( vblock, "$MINMAX" );
strcpy( minmax, SVAL( *var, 0 ));

/* Get the input calibration method 
-----------------------------------*/
var = p_find( vblock, "CALTYPE" );
strcpy( calstr->calmethod, SVAL( *var, 0 ));
len = (long)strlen( calstr->calmethod );
c_up2low( calstr->calmethod, &len );

/* Get the illumination correction flag
---------------------------------------*/
var = p_find( vblock, "ILLUMFLG" );
if ( strcmp( SVAL( *var, 0 ), "YES" ) == 0 )
    calstr->illum = TRUE;
else
    calstr->illum = FALSE;

/* Get the calibration period
-----------------------------*/
var = p_find( vblock, "CALPER" );
calstr->calper = IVAL( *var, 0 );

/* Get the PRT temperature period
---------------------------------*/
var = p_find( vblock, "NCALS" );
calstr->prtper = IVAL( *var, 0 );

/*
  Check to make sure we are using the minimum # of lines
  allowable to initialize calibration coefficients and
  not more lines than are in the image
--------------------------------------------------------*/
if ((calstr->prtper * calstr->calper) < MAXLNS)
    {
    c_lasmsg(FALSE, "avhrrcal-few",
        "Too few lines specified for coefficient initialization");
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
    }
if ((calstr->prtper * calstr->calper) >= ddr.nl)
    {
    c_lasmsg(FALSE, "avhrrcal-many",
        "Too many lines specified for coefficient initialization");
    c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
    }

/* The application always reads LAS images
-------------------------------------------*/
calstr->file_type = LASIMAGE;

return;
}
