/*******************************************************************************
FUNCTION:	OPEN_LAS

PURPOSE:	Open the input and output images

ALGORITHM 
  Set variables for the image open
     Windowing is not allowed in this application so set the window to
     the size of the input image
  Determine the output data type if not specified
  Open the output image
  Allocate I/O buffer
  Group images

ALGORITHM REFERENCES
*******************************************************************************/
#include "worgen.h"
#include "util.h"
#include "imageio.h"
#include "avhrrcal.h"

void open_las
(
struct	PARBLK	*vblock,	/* TAE parameter block			*/
char	hosin[][CMLEN],		/* Input host file names		*/
long	*nimgs,			/* Number of input images		*/
char	*hosout,		/* Output host file names		*/
long	window[][4],		/* Window specification of input images	*/
long	bands[][MAXBND+1],	/* Bands to be processed for all images	*/
long	nbands[],		/* Number of bands per input image	*/
long	*totbnds,		/* Total bands to be processed		*/
long	*odtype,		/* Data type of output image		*/
struct	MODEL	*model,		/* Model and orbital information	*/
int	fdesc[MAXIMG+1],	/* I/O image file descriptors 		*/
int	*gdesc,			/* group descriptor used for image I/O	*/
long	*nout,			/* Number of output files		*/
long	offset1[MAXIMG*NCHANS],	/* offsets into "buffer" for input bands*/
long	offset2[NCHANS],	/* offsets into "buffer" for output bands*/
unsigned char **buffer 		/* image I/O buffer			*/
)

{
long	access;			/* image I/O access type		*/
long	band;			/* Loop index counting the bands	*/
long	bufsiz;			/* Size of image I/O buffer		*/
long	idtype = EREAL;		/* Convert the input to floating	*/
long	image;			/* Loop index counting input images	*/
long	nl, ns;			/* Number of lines and samples		*/
long	one = 1;		/* Constant one				*/
long	opt[3];			/* Data conversion option		*/
long	sl, ss;			/* Starting line and sample		*/
long	status;			/* Return status code               	*/
long	zero = 0;		/* Constant zero			*/


/* Set variables for the image open
-----------------------------------*/
access = IREAD;
sl = 1;
ss = 1;
nl = 0;
ns = 0;
opt[0] = opt[1] = opt[2] = 0;

/* Open the input image
-----------------------*/
for ( band = 0, image = 0; image < *nimgs; image++ )
   {
   status = c_eopens( (struct FDESC **)&fdesc[image], hosin[image],
			bands[image], &sl, &ss, &nl, &ns, &offset1[band],
			&access, &idtype, &zero, opt );
   if ( status != E_SUCC )
      c_lasmsg( TRUE, "avhrrcal-fatal", "Fatal error encountered" );
   band += nbands[image];

   /* Windowing is not allowed in this application so set the window to
     the size of the input image
   --------------------------------------------------------------------*/
   window[image][0] = sl;
   window[image][1] = ss;
   window[image][2] = nl;
   window[image][3] = ns;
   }

/* Determine the output data type if not specified
--------------------------------------------------*/
if ( *odtype == 0 )
    c_gettyp( (const char (*)[CMLEN])hosin, nimgs, odtype );

/* Open the output image
------------------------*/
access = IWRITE;	
status = c_eopens( (struct FDESC **)&fdesc[*nimgs], hosout, totbnds, &one,
		   &one, &nl, &ns, offset2, &access, odtype, &zero, opt );
if ( status != E_SUCC )
   c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );
fdesc[*nimgs+1] = 0;

/* Generate an addr file for the output image
---------------------------------------------*/
status = c_putaddr( hosout, model );
if ( status != E_SUCC )
   c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );

/* Allocate I/O buffer
----------------------*/
if (( *buffer = (unsigned char *)c_ealloc( (struct FDESC **)fdesc,
	&bufsiz )) == NULL )
   {
   c_lasmsg( FALSE, "avhrrcal-alloc", "Error allocating dynamic memory" );
   c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );
   }

/* Group images
---------------*/
if ( c_egroup( (struct FDESC **)fdesc, (struct GDESC **)gdesc, *buffer,
		&bufsiz ) != E_SUCC )
   c_clean( vblock, "avhrrcal", (const char (*)[CMLEN])hosout, nout );

return;
}
