/******************************************************************************
NAME:			GETPAR

PURPOSE:  retrieve the TAE parameters from the parameter block 

ALGORITHM DESCRIPTION:  see in line documentation
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "util.h"
#include "archio.h"

void getpar
(
    struct PARBLK *vblock,	/* Pointer to TAE parameter block */
    char *hosin,                /* Host name of file specified in IN */
    struct MODEL *model,	/* Model and orbital information  */
    char *tstamp,		/* Time stamp */
    char *tapeid,		/* Tape id */
    long *dbflag		/* Data base temp or daily file flag */
)
{   
struct VARIABLE  *nam;          /* Pointer to a TAE variable in vblock */

long   nonfatal = 1,            /* Nonfatal error code */
       fatal = -1,              /* Fatal error code */
       element = 0,             /* Constant zero */
       access = IREAD,          /* Access for which hosin is opened */ 
       bhour, bmin, bsec;	/* Browse time	*/

struct DROPPED drop;

/* Get the host file name
--------------------------*/
if (c_getnam(vblock, "IN", &element, hosin, "arch", &access) != E_SUCC)
    c_errmsg("Fatal error encountered", "avhrrdb-fatal", &fatal);
 
/* Get the header information
-----------------------------*/
if (c_getahdr(hosin,model,&drop) != E_SUCC)
   c_errmsg("Fatal error encountered","avhrrdb-fatal",&fatal);

/* Retrieve browse time stamp
-----------------------------*/
nam = p_find(vblock,"TSTAMP");
strcpy(tstamp,SVAL(*nam,0));
if (strlen(tstamp) < 6)
    {
    c_errmsg("Invalid time stamp","avhrrdb-time",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrdb-fatal",&fatal);
    }

sscanf(tstamp,"%2d%2d%2d",&bhour,&bmin,&bsec);
if ((bhour > 24) || (bmin > 60) || (bsec > 60) ||
    (bhour < 0)  || (bmin < 0)  || (bsec < 0))
    {
    c_errmsg("Invalid time stamp","avhrrdb-time",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrdb-fatal",&fatal);
    }

/* Get the contributor of the input data
----------------------------------------*/
nam = p_find(vblock,"TAPEID");
if ((*nam).v_count <= 0)
    {
    strcpy(tapeid, "      ");
    *dbflag = FALSE;
    }
else
    {
    strcpy(tapeid,SVAL(*nam,0));
    *dbflag = TRUE;
    }
 
return;
}
