/*******************************************************************************
NAME:		UPASSOC	

PURPOSE:	Updates the LAS associated AVHRR DDR and the dropped line files.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  5.0    04/91  D.VanderZee      Development
  5.1    04/93  J.Fenno          Modified to read archive images directly from
				 tape.  Also made changes to incorporate the
				 new archio support routines.
  6.0    09/97  T.Baltzer        Rewrote to take advantage of new supports
				 c_windmod(), c_winddrp(), c_putdrpl()
				
PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h> 
#include <sys/file.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "labserv.h"



int upassoc
(
    char *image,		/* Host file name of output image	      */
    char outfiles[][CMLEN],	/* List of output files                       */
    long *n_outfiles,		/* Number of output files                     */
    long window[],		/* Window for the AVHRR archive image         */
    struct DROPPED *drop,	/* Dropped line data structure                */
    struct MODEL   *model,	/* AVHRR model information                    */
    long bands[] 		/* Band specifications for each input image   */
)


{

long status;			/* Return status variable                     */

char addr_fname[CMLEN];		/* AVHRR ddr file name                        */
char drpl_fname[CMLEN];		/* Dropped line file name                     */

struct MODEL   window_model;	/* AVHRR model information for user window    */
struct DROPPED window_drop;	/* Dropped line data structure for user window*/



/* Generate a model structure and a drop line structure for the specified
   window.  It won't hurt to do this even if the image is not to be windowed.
-----------------------------------------------------------------------------*/
status = c_windmod(model,drop,&window_model,&window_drop,window,bands,TRUE);
if (status != E_SUCC)
   return(E_FAIL);


/* Save the AVHRR model information to the AVHRR DDR file and save the file
   name for the completion message or for clean up.  Because the ADDR is
   associated with a LAS image, which are always "North-up", the sat_view flag
   in the model structure must be FALSE.
   Note that the AVHRR model is only for the lines in the user specified window
   (may be the entire image or a window thereof).
-----------------------------------------------------------------------------*/
window_model.sat_view = FALSE;            /* set sat_view to false */
status = c_putaddr(image, &window_model);
if (status != E_SUCC)
    return(E_FAIL);
window_model.sat_view = model->sat_view;  /* restore sat_view flag */
c_lsmknm(image, "addr", addr_fname);
strcpy(outfiles[*n_outfiles], addr_fname);
(*n_outfiles)++;


/* Save the AVHRR drop line information to the AVHRR DRPL file and save the
   file name for the completion message or for clean up.
-----------------------------------------------------------------------------*/
status = c_putdrpl(image, &window_drop);
if (status != E_SUCC)
    return(E_FAIL);
c_lsmknm(image, "drpl", drpl_fname);
strcpy(outfiles[*n_outfiles], drpl_fname);
(*n_outfiles)++;


return(E_SUCC);
}
