/*******************************************************************************
FUNCTION:	avhrrstack_

PURPOSE:	Rename the tape summary file that was created by ANSISTACK and
		return the tape ID to TAE.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    01/94  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "queue.h"
#include "avhrrstack_.h"



main()
{
char dir[CMLEN];		/* Directory portion of a file name           */
char drive[CMLEN];		/* Tape drive on which the tape was created   */
char ext[CMLEN];		/* Extension portion of a file name           */
char groups[CMLEN];		/* Number of file groups                      */
char *ptr[1];			/* Ptr to a string for calling q_string()     */
char root[CMLEN];		/* Root portion of a file name                */
char str[CMLEN];		/* Temporary string                           */
char sumfile[CMLEN];		/* Tape summary file                          */
char system[CMLEN];		/* System on which the tape was created       */
char tapeid[CMLEN];		/* ID of the tape that is mounted             */

long access;			/* Type of access for the summary file        */
long nonfatal =  1;		/* Nonfatal error flag                        */
long queue;			/* Queue of file groups stored on the tape    */
long std_out;			/* Standard output file                       */
long zero     = 0;		/* Constant of zero                           */

struct PARBLK vblock;		/* TAE parameter block                        */



/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Retrieve the name of the tape summary file
---------------------------------------------*/
access = IREAD;
if (c_getnam(&vblock, "SUMFILE", &zero, sumfile, "list", &access) != E_SUCC)
    z_exit(-1, "AVHRRSTACK_ failed");

/* Read the tape summary file
-----------------------------*/
if (read_summary("avhrrstack",sumfile,tapeid,system,drive,groups,&queue) != E_SUCC)
    z_exit(-1, "AVHRRSTACK_ failed");


/* Free all of the memory used by the queue of file group names
---------------------------------------------------------------*/
c_qdestroy(queue);


/* Split the summary file name into its pieces and rename it using the tape ID
------------------------------------------------------------------------------*/
c_splitname(sumfile, dir, root, ext);
sprintf(str, "%s%s.%s", dir, tapeid, ext);
rename(sumfile, str);


/* Place the tape ID in the TAE parameter block
-----------------------------------------------*/
ptr[0] = tapeid;
if ((q_string(&vblock, "TAPEID", 1, (const char **)ptr, P_UPDATE) != SUCCESS) ||
    (q_out(&vblock) != SUCCESS))
    {
    c_errmsg("Error sending parameter to TAE", "avhrrstack-tae", &nonfatal);
    z_exit(-1, "AVHRRSTACK_ failed");
    }


/* Issue a successful completion message
----------------------------------------*/
z_exit(1, "AVHRRSTACK_ completed successfully");


/* Put a return here to make the compiler happy
------------------------------------------------*/
return(1);

}
