/****************************************************************************
FUNCTION:	get_scenes

PURPOSE:	Get the scene filenames, from a tape summary file,
                for a given tape ID.

Version  Date   Programmer       Reason
-------  -----  ---------------  --------------------------------------------
  1.0    04/97  R.Dilley         Original Development

PROJECT: ADAPS
*****************************************************************************/
#include <stdio.h>
#include <time.h>
#include "tae.h"
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "queue.h"
#include "avhrrstack_.h"

#define MAX_SCENES 25

main()
{
   char drive[CMLEN];	  /* Tape drive on which the tape was created       */
   char ngroups[CMLEN];	  /* Number of file groups stored on the tape       */
   char sumfile[CMLEN];	  /* Full path specification of file containing IDs */
   char system[CMLEN];	  /* System on which the tape was created           */
   char tapeid[CMLEN];	  /* ID of the tape to printa shipping form for     */

   long fatal    = -1;    /* Fatal error flag                               */
   long queue;		  /* Queue of AVHRR images stacked to the tape      */
   long status;		  /* Return status of a function                    */
   long std_out;	  /* Standard output file                           */
   long first;		  /* The first scene to start with                  */
   long maxscenes;	  /* The maximum number of scenes to return         */

   struct PARBLK vblock;  /* TAE parameter block                            */

/* Initialize LAS
-----------------*/
   c_inlas(&vblock, &std_out);

/* Get all of the parameters from the variable block
----------------------------------------------------*/
   getpar_sc(&vblock, sumfile, &first, &maxscenes);

/* Read the tape summary file to extract the scene names in queue
-----------------------------------------------------------------*/
   status = read_summary("shipform", sumfile, tapeid, system,
                         drive, ngroups, &queue);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered", "shipform-fatal", &fatal);

/* Send the results back to TAE
-------------------------------*/
   taeland(&vblock, queue, first, maxscenes);

/* Free all the memory used by the queue of image names
-------------------------------------------------------*/
   c_qdestroy(queue);

/* Exit the C code
------------------*/
   z_exit(1,"GETSCENES completed successfully");

/* Put a return here to make the compiler happy
------------------------------------------------*/
return(1);
}

void getpar_sc
(
   struct PARBLK *vblock,	/* IN - TAE parameter block                 */
   char *sumfile,		/* OUT - Host name of the tape summary file */
   long *first,                 /* OUT - The first scene to read            */
   long *maxscenes              /* OUT - The maximum number of scenes       */
)

{
   char tmp[CMLEN];		/* temporary string                         */

   struct VARIABLE *parm;	/* Ptr to a TAE variable                    */

   parm = p_find(vblock, "SUMFILE");
   strcpy(tmp,SVAL(*parm,0));
   c_cmhost(tmp,sumfile);

   parm = p_find(vblock, "FIRST");
   *first = IVAL(*parm,0);

   parm = p_find(vblock, "MAXSCENES");
   *maxscenes = IVAL(*parm,0);

   return;
}

void taeland
(
   struct PARBLK *vblock, /* IN - TAE parameter block                       */
   long queue,            /* IN - Queue of AVHRR images stacked to the tape */
   long first,            /* IN - The first scene to read                   */
   long maxscenes         /* IN - The maximum number of scenes to return    */
)

{
   TEXT *sptr[MAX_SCENES];/* pointer array for updating vblock              */
   void *ptr;             /* pointer to an image in the queue               */

   long i;                /* counter                                        */
   long fatal    = -1;    /* Fatal error status                             */
   long nonfatal = 1;     /* Nonfatal error status                          */
   int nscenes;		  /* Number of scenes returned                      */

/* Locate the first image name in the queue
-------------------------------------------*/
   c_qfirst(queue, &ptr);

/* Let sptr point to the file names
-----------------------------------*/
   i = 0;
   while ((i < first-1) && (ptr != NULL))
   {
      c_qnext(queue, &ptr);
      i++;
   }
   nscenes = 0;
   for (i = 0;i < maxscenes; i++)
   {
      if(ptr == NULL) break;
      sptr[i] = ptr;
      c_qnext(queue, &ptr);
      nscenes++;
   }

/* Output the string values to the variable block
-------------------------------------------------*/
   if (q_string(vblock,"SNAMES",i,(const char **)sptr,P_UPDATE) != SUCCESS)
   {
      c_errmsg("Error returning parameter to TAE","shipform-rparm",&nonfatal);
      c_errmsg("Fatal error encountered","shipform-fatal",&fatal);
   }

/* Output the integer value to the variable block
-------------------------------------------------*/
   if (q_intg(vblock,"NSCENES",1,&nscenes,P_UPDATE) != SUCCESS)
   {
      c_errmsg("Error returning parameter to TAE","shipform-rparm",&nonfatal);
      c_errmsg("Fatal error encountered","shipform-fatal",&fatal);
   }

/* Output the variable block back to TAE
----------------------------------------*/
   if (q_out(vblock) != SUCCESS)
   {
      c_errmsg("Error returning parameter to TAE","shipform-rparm",&nonfatal);
      c_errmsg("Fatal error encountered","shipform-fatal",&fatal);
   }

   return;
}
