/*******************************************************************************
FUNCTION:	updatedb

PURPOSE:	Update the database records for every AVHRR archive image
		contained on an AVHRR archive tape.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    03/94  J.Fenno          Development


PROJECT: ADAPS
*******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adapsutil.h"
#include "queue.h"
#include "avhrrstack_.h"


main()
{
char drive[CMLEN];		/* Tape drive on which the tape was created   */
char groups[CMLEN];		/* Number of file groups                      */
char logfile[CMLEN];		/* Logfile used for processing messages       */
char *ptr;			/* Ptr to a string                            */
char str[CMLEN];		/* Temporary string                           */
char sumfile[CMLEN];		/* Tape summary file                          */
char system[CMLEN];		/* System on which the tape was created       */
char tapeid[CMLEN];		/* ID of the tape that is mounted             */

long access;			/* Type of access for the summary file        */
long fatal    = -1;		/* Fatal error flag                           */
long len;			/* Length of a string                         */
long nonfatal =  1;		/* Nonfatal error flag                        */
long queue;			/* Queue of file groups stored on the tape    */
long std_out;			/* Standard output file                       */
long zero     = 0;		/* Constant of zero                           */

struct PARBLK vblock;		/* TAE parameter block                        */
struct VARIABLE *parm;		/* Ptr to a TAE variable                      */



/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Get TAPEID from the variable block (must be all lowercase)
-------------------------------------------------------------*/
parm = p_find(&vblock, "TAPEID");
strcpy(tapeid, SVAL(*parm,0));
len = (long)strlen(tapeid);
c_up2low(tapeid, &len);


/* Retrieve the name of the log file
------------------------------------*/
parm = p_find(&vblock, "LOGFILE");
if (parm->v_count != 1)
    {
    ptr = getenv("ADAPSLOG");
    if (ptr == NULL)
	{
	c_errmsg("Error retrieving ADAPSLOG", "updatedb-env", &nonfatal);
	c_errmsg("Fatal error encountered", "updatedb-fatal", &fatal);
	}
    sprintf(logfile, "%s/updatedb.log", ptr);
    }
else
    {
    access = IUPDATE;
    if (c_getnam(&vblock, "LOGFILE", &zero, logfile, "log", &access) != E_SUCC)
	c_errmsg("Fatal error encountered", "updatedb-fatal", &fatal);
    }


/* Create the name of the tape summary file
-------------------------------------------*/
ptr = getenv("ADAPSARC");
if (ptr == NULL)
    {
    c_errmsg("Error retrieving ADAPSARC", "updatedb-env", &nonfatal);
    c_wrtlog(logfile, "Error", "updatedb-error", "Error retrieving ADAPSARC");
    c_errmsg("Fatal error encountered", "updatedb-fatal", &fatal);
    }
sprintf(sumfile, "%s/forms/%s.list", ptr, tapeid);


/* Read the tape summary file
-----------------------------*/
if (read_summary("updatedb",sumfile,tapeid,system,drive,groups,&queue) != E_SUCC)
    {
    c_wrtlog(logfile, "Error", "updatedb-error", "Error reading summary file");
    c_errmsg("Fatal error encountered", "updatedb-fatal", &fatal);
    }


/* Uppercase the tape ID for all futher uses
--------------------------------------------*/
len = (long)strlen(tapeid);
c_low2up(tapeid, &len);


/* Update the database records for each image
---------------------------------------------*/
if (update_dbase(&queue, tapeid, logfile) != E_SUCC)
    {
    char *chr;
    char scene_id[CMLEN];

    /* The update command failed so issue an error message for each image
    ---------------------------------------------------------------------*/
    c_qfirst(queue, (void **)&chr);
    while (chr != NULL)
	{
	strncpy(scene_id, chr, 16);
	scene_id[16] = '\0';
	sprintf(str, "Error updating database record for %s", chr);
	c_errmsg(str, "updatedb-update", &nonfatal);
	sprintf(str, "Error updating database record");
	c_wrtlog(logfile, scene_id, "updatedb-error", str);
	c_qnext(queue, (void **)&chr);
	}
    }


/* Free all of the memory used by the queue of file group names
---------------------------------------------------------------*/
c_qdestroy(queue);


/* Issue a successful completion message
----------------------------------------*/
z_exit(1, "UPDATEDB completed successfully");

/* Put a return here to make the compiler happy
------------------------------------------------*/
return(1);

}
