/*******************************************************************************
FUNCTION:	browse

PURPOSE:	Conversion of an ADAPS LAS image to a browse image.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    08/90  D.Hollaren       Modified version of VMSOUT
  5.0    03/92  E.Cihak/         LAS-5.0 version for ADAPS
                T.Rockvam

ALGORITHM:	An AVHRR LAS image is written to a browse format file.  The
		browse is made up of 408-byte records.  The first record is a
		header, of which the first 114 bytes contain information about
		the image, and the rest is filled with blanks.

		A GAC image is written in full resolution, while a LAC image is
		subsampled every 4th line and 5th sample (to produce the same
		resolution as a GAC image).  The one input image may be
		windowed and subbanded.

PROJECT:	ADAPS		                
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "model.h"
#include "browse.h"

/*
  The following variables must be declared global for image I/O routines
*/
struct FDESC *fdesc[1];        	/* image file descriptors used for image I/O */

int main (void)
{
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

struct	PARBLK	vblock;		/* parameter block		      	*/

static char oname[MAXBND][CMLEN]; /* output file host name(s)	      	*/

char 	hosin[1][CMLEN],	/* input file name                    	*/
	sceneid[17];		/* input image scene id			*/

long	bands[1][MAXBND+1],  	/* band numbers specified	      	*/
	idtype,			/* input data type                    	*/
	lskip,			/* number of lines to skip to subsample	*/
	msamp,			/* master image sample			*/
	mline,			/* master image line			*/
	msgtim,			/* seconds between processing messages	*/
	nbands,  		/* number of bands in input image     	*/
	nmrout,			/* number of output files created     	*/
	outlines,		/* number of lines after subsampling	*/
	outsamps,		/* number of samples after subsampling	*/
	sskip,			/* number of samples to skip to subsample*/
	std_out, 		/* standard output	              	*/
	totbnd,   		/* total number of bands              	*/
	window[1][4];		/* window specification	              	*/
 
struct  MODEL 	model;		/* Model and orbital information	*/
struct  LINE	brw_line;       /* header record for output image     	*/

/*
  LAS initialization
*/
c_inlas(&vblock,&std_out);

/*
  Get user specified parameters and TAE globals
*/
getpar (&vblock,&model,hosin,sceneid,oname,bands,&nbands,&idtype,&totbnd,
	window,&lskip,&sskip,&mline,&msamp,&outlines,&outsamps,&nmrout,
       &msgtim);

/*
  Set up image header
*/
setup_head(&model,&brw_line,bands[0],nbands,window[0],mline,msamp,
	   outlines,sceneid);

/*
  Convert from LAS image to browse file 
*/
outbr(&vblock,hosin[0],oname,&nmrout,window[0],outlines,outsamps,
      bands[0],nbands,&idtype,&msgtim,&lskip,&sskip,&brw_line);

/*
  Output completion message 
*/
c_complt("BROWSE", oname, &nmrout);

/*
  Return a value here to make the compiler happy
*/
return(SUCCESS);

}
