/*******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	To retrieve and edit the parameters.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    08/90  D.Hollaren       Modified version of VMSOUT
  5.0    03/92  E.Cihak/         LAS-5.0 version for ADAPS
                T.Rockvam
  5.1    07/93  J.Fenno          Modified for the archive conversion process.

PROJECT:	ADAPS		                
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"	
#include "tae.h"	
#include "util.h"	
#include "adaps.h"
#include "adapssup.h"

void getpar
(
    struct PARBLK *vblock,	/* parameter block	             	*/
    struct MODEL *model,	/* Model and orbital information	*/
    char hosin[][CMLEN], 	/* image file HOST name    	     	*/
    char *sceneid,		/* image file scene id			*/
    char oname[][CMLEN],	/* output file name(s)         		*/
    long (*bands)[MAXBND+1],	/* band numbers specified	     	*/
    long *nbands,		/* number of bands in input image    	*/
    long *idtype,		/* input data type		     	*/
    long *totbnd,		/* total number of bands specified   	*/
    long (*window)[4],		/* window specification		     	*/
    long *lskip, 		/* number lines to skip to subsample 	*/
    long *sskip,		/* number samples to skip to subsample	*/
    long *mline,		/* image master line			*/
    long *msamp,		/* image master sample			*/
    long *outlines,		/* number of lines after subsampling    */
    long *outsamps,		/* number of samples after subsampling  */
    long *nmrout,		/* number of output files            	*/
    long *msgtim		/* seconds between processing messages 	*/
)
{
struct VARIABLE	*in;		/* pointer to TAE variable		*/
struct DDR 	brw_ddr;	/* DDR structure		        */

char    dir[CMLEN];		/* Directory portion of a file name     */
char    ext[CMLEN];		/* Extension portion of a file name     */
char    root[CMLEN];		/* Root portion of a file name          */
char	taein[1][CMLEN]; 	/* image file TAE name		     	*/

long	access,			/* file access				*/
	element = 0,		/* element for getin call	     	*/
	fatal = -1,		/* fatal error flag		     	*/
	i,			/* loop index			     	*/
	imglimit = 1,   	/* number of images to expect input 	*/
	nmrimg,			/* number of input files             	*/
	nonfatal = 1,   	/* nonfatal error flag               	*/
	qflag[MAXIMG][2],	/* default window and band flag 	*/
	same,	        	/* same window flag		     	*/
	sixteen = 16,		/* length of sceneid			*/
	status;			/* status indicator		     	*/
/* 
  Get the TAE globals needed
*/
in = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*in,0);


/* 
  Get the name of the input file
*/
status = c_getin(vblock,"IN",&element,taein,hosin,&nmrimg,window,
		 bands,nbands,totbnd,&imglimit,&same,qflag);
if (status != E_SUCC)
    c_errmsg ("Fatal error encountered", "browse-fatal", &fatal);


/* 
  Get the data type & master line/sample of 
  the input image from the ddr
*/
status = c_getddr(hosin[0],&brw_ddr);
if (status == E_FAIL)
    c_errmsg ("Fatal error encountered", "browse-fatal", &fatal);
if (brw_ddr.dtype > EWORD) /* ELONG, EREAL, EDOUBLE */
    {
    c_errmsg ("Invalid input image data type", "browse-dtype", &nonfatal);
    c_errmsg ("Fatal error encountered", "browse-fatal", &fatal);
    }
else /* EBYTE or EWORD */
    *idtype = brw_ddr.dtype;
*mline = brw_ddr.master_line;
*msamp = brw_ddr.master_sample;


/* 
  Get the AVHRR header record
*/
if (c_getaddr(hosin[0],model) != E_SUCC)
   c_errmsg ("Fatal error encountered", "browse-fatal", &fatal);


/*
  Get the scene id of the LAS image
*/
c_getscene(model,sceneid);
c_low2up(sceneid,&sixteen);


/*
  During the archive conversion process the starting time of each scene can
  change by two or three seconds.  We want to keep the old scene ID so that
  the scene ID in the browse header matches the production data base.
*/
c_splitname(hosin[0], dir, root, ext);
if ((strncmp(root, "map_t", 5) == 0) && (strlen(root) == 18))
    strncpy(&sceneid[10], &root[5], 6);


/* 
  Set values for lskip and sskip to use subsampling, if necessary
*/
if (model->dtype == GAC)
   {
   *lskip = 1;
   *sskip = 1;
   }
else
   {
   *lskip = 4;
   *sskip = 5;
   }


/*
  Figure number of lines/samples in output browse image
*/
*outlines = ((window[0][NL] - 1) / *lskip) + 1;
*outsamps = ((window[0][NS] - 1) / *sskip) + 1;
if (*outsamps > 408) *outsamps = 408;

/* 
  Get the name(s) of the output file(s) and make sure the correct number
  of output names was specified
*/
in = p_find(vblock, "OUT");
*nmrout = in->v_count;

if ((*nmrout != 1) && (*nmrout != *nbands))
    c_errmsg("Invalid number of output files specified","browse-onames",&fatal);

access = IWRITE;
for (i = 0; i < *nmrout; i++)
    {
    status = c_getnam(vblock,"OUT",&i,oname[i],"brw",&access);
    if (status != E_SUCC)
        c_errmsg ("Fatal error encountered","browse-fatal",&fatal);
    }

return;
}
