/*******************************************************************************
FUNCTION:	setup_head

PURPOSE:	Set up the browse header record 

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    08/90  D.Hollaren       Modified version of VMSOUT
  5.0    03/92  E.Cihak/         LAS-5.0 version for ADAPS
                T.Rockvam

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "model.h"
#include "browse.h"
#include "typlim.h"


void setup_head
(
    struct MODEL *model,	/* Model and orbital information	*/
    struct LINE *brw_line,	/* header for the output browse file 	*/
    long bands[],		/* band numbers specified 		*/
    long nbnd,			/* number of bands 			*/
    long window[],		/* window of the input image		*/
    long mline,			/* master image line			*/
    long msamp,			/* master image sample			*/
    long outlines,		/* number of output lines		*/
    char *sceneid		/* sceneid of the LAS image 		*/
) 
{
long	i,		 	/* loop counter 			*/
	nwlat, nwlon,		/* lat/long of upper left corner 	*/
	nelat, nelon,		/* lat/long of upper right corner 	*/
	swlat, swlon,		/* lat/long of lower left corner 	*/
	selat, selon,		/* lat/long of lower right corner 	*/
	cnlat, cnlon;		/* lat/long of center of pass 		*/

char	bnd[10],		/* string of band numbers specified 	*/
	temp[2];		/* temporary string for band numbers 	*/

float	nline,			/* northern line of the window		*/
	sline,			/* southern line of the window		*/
	cline,			/* central line of the window		*/
	csamp,			/* central sample of the window		*/
	esamp,			/* eastern sample of the window		*/
	wsamp;			/* western sample of the window		*/

double  lat, lng,		/* latitude/longitude values of window	*/
	elev = 0.0;		/* Elevation				*/
 
/*
  Convert the latitudes and longitudes in the header to total minutes
  An AVHRR LAS image has always been flipped if necessary so north line 
  is always SL and west sample is always SS
*/
nline = (float)window[SL] + (float)mline;
sline = (float)(window[SL] + window[NL] - 1 + mline);
cline = (float)(window[SL] + window[NL]/2 - 1 + mline);
wsamp = (float)window[SS] + (float)msamp;
esamp = (float)(window[SS] + window[NS] - 1 + msamp);
csamp = (float)(window[SS] + window[NS]/2 - 1 + msamp);

c_ls2ll(&nline,&wsamp,&elev,model,&lat,&lng,NULL);
rad2min(lat,&nwlat);
rad2min(lng,&nwlon);

c_ls2ll(&nline,&esamp,&elev,model,&lat,&lng,NULL);
rad2min(lat,&nelat);
rad2min(lng,&nelon);

c_ls2ll(&sline,&wsamp,&elev,model,&lat,&lng,NULL);
rad2min(lat,&swlat);
rad2min(lng,&swlon);

c_ls2ll(&sline,&esamp,&elev,model,&lat,&lng,NULL);
rad2min(lat,&selat);
rad2min(lng,&selon);

c_ls2ll(&cline,&csamp,&elev,model,&lat,&lng,NULL);
rad2min(lat,&cnlat);
rad2min(lng,&cnlon);

/*
  Write out bands specified
*/
bnd[0] = '\0';
for (i = 0; i < nbnd; i++)
   {
   sprintf (temp, "%d", bands[i]); 
   strcat (bnd, temp);
   }

/*
  Fill in header version
*/
brw_line->version = 0X02;

/*
  Write remaining header fields
  always say 408 samples because glis browse display is expecting that
*/
sprintf(brw_line->header,
        "%30s%3d%3d%4d%4d%-9s%6d%6d%6d%6d%6d%6d%6d%6d%6d%6d",
        sceneid,MINUTINY,MAXUTINY,outlines,408,bnd,
        nwlat,nwlon,nelat,nelon,selat,selon,swlat,swlon,cnlat,cnlon);
/*
for (i = 113; i < 407; i++) 
   brw_line->header[i] = ' ';
*/

return;
}
