/******************************************************************************
NAME:           BRWRENM

PURPOSE:        Rename an AVHRR browse image to sceneid;brw

PROGRAM HISTORY:
VERSION  AUTHOR        DATE     REASON
-------  ------        -----    -----------------------------
  1.0    T. Baltzer    05/96    Original development
  1.1    T. Baltzer    06/96    Add time stamp to make unique
				file names.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "util.h"


int main()
{
FILE *fp;               /* File ptr */

struct PARBLK vblock;	/* TAE parameter block  */

char ext[CMLEN];	/* Extension portion of a file name */
char inhostname[CMLEN]; /* Input file host name */
char inlasname[CMLEN];  /* Input file LAS name */
char outhostname[CMLEN];/* Output file host name specification */
char outlasname[CMLEN]; /* Output file LAS name specification */
char root[CMLEN];	/* Root portion of a file name */
char sceneid[17];	/* Scene id */
char srcdir[CMLEN];     /* Source directory */
char str[CMLEN];        /* Temporary working string */

long strlth;		/* String length */
long status;		/* Error status */
long std_out;		/* Standard output */

struct VARIABLE *val;           /* pointer to TAE variable with vblock        */

long now;		/* time now		*/
struct tm *curtime;	/* current time		*/
char tstamp[8];		/* time stamp		*/


/* Initialize LAS */
c_inlas(&vblock,&std_out);


/* Get the name of the file to rename.  Don't use c_getnam() since it cannot
   handle upper-case names and the AVHRR browse images as archived by GLIS
   are of the syntax "sourcelink.1KM".  Since c_cmhost() and c_cmtae() force
   a file name to lower-case, hardwire the "1km" suffix back to "1KM".
   This idiosycracy makes the BRWRENM functionality very specific.
-----------------------------------------------------------------------------*/
val = p_find(&vblock,"INFILE");
strcpy(inlasname,SVAL(*val,0));
strlth = (long)strlen(inlasname);
strcpy(ext,&inlasname[strlth-4]);
if (strcmp(ext,";1KM") != 0)
    {
    c_lasmsg(0,"brwrenm-invl","Invalid browse image name specified");
    return(E_FAIL);
    }
c_cmhost(inlasname,inhostname);
strcpy(inlasname, inhostname);
strlth = (long)strlen(inhostname);
inhostname[strlth-2] = 'K';
inhostname[strlth-1] = 'M';



/* Open the file and get the first CMLEN bytes
---------------------------------------------- */
fp = fopen(inhostname, "r");
if (fp == NULL)
    {
    c_lasmsg(0,"brwrenm-open","Error opening browse image");
    return(E_FAIL);
    }

fgets(str, CMLEN, fp);

fclose(fp);


/* Extract the scene id from the string 
   and lower case it.
----------------------------------------*/
strncpy(sceneid,&str[15],16);
sceneid[16] = '\0';
strlth = (long)strlen(sceneid);
c_up2low(sceneid,&strlth);


/* Get the current time to use as a time stamp, do this to ensure
   a unique new file name when there are duplicate scene id's.
------------------------------------------------------------------*/
now = time(NULL);
curtime = localtime(&now);
sprintf(tstamp,"_%02d%02d%02d",curtime->tm_hour,curtime->tm_min,
			       curtime->tm_sec);


/* Determine the new name of the file
--------------------------------------*/
c_splitname(inhostname, srcdir, root, ext);
strcpy(outhostname,srcdir);
strcat(outhostname,sceneid);
strcat(outhostname,tstamp);
strcat(outhostname,".brw");


/* Rename the file
--------------------------------------------*/
status = renamefile(inhostname,outhostname,0,0,0);
if (status != E_SUCC)
    {
    c_lasmsg(0,"brwrenm-renm","Error renaming browse image");
    return(E_FAIL);
    }


/* Print success message.  Hard-wire the suffix of the input file
   name from "1km" back to "1KM".
---------------------------------------------------------------------*/
strlth = (long)strlen(inlasname);
inlasname[strlth-2] = 'K';
inlasname[strlth-1] = 'M';
strcpy(outlasname, outhostname);
c_lasmsg(0,"brwrenm-succ","%s renamed to %s",inlasname,outlasname);


return E_SUCC;
}
