/*******************************************************************************
FUNCTION:	CHECKSPACE

PURPOSE:	flag whether or not there is enough available disk space for 
		the specified file

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       05/92    T. Rockvam          Original development
 1.1       11/92    J. Fenno            Moved the checkspace executable from
                                        ADAPSBIN to wherever checkspace is
					released.

EXIT STATUS:
	 0 	the requested amount of space is available
	-1 	the requested amount of space is not available
	 2	an invalid number of parameters was supplied

PROJECT:	ADAPS

NOTE:		This is done in C instead of cshell because using cshell 
		would require the use of df which acts differently on 
		each system
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "adapssup.h"

/* function prototype */
int dirspace
(
char 	*dstdir,		/* destination dir to search for space  */
double	*space 			/* space in destination dir		*/
);


int main
(
int argc,                       /* number of arguments                  */
char *argv[]                    /* argument: directory to check         */
)

{
char	dstdir[CMLEN];	/* destination disk			*/
char	infile[CMLEN];	/* input file				*/
char	insize[CMLEN];	/* number of bytes to search for	*/

double 	avail,		/* number of bytes available		*/
	nbytes;		/* number of bytes to search for	*/


long	len;		/* length of file name			*/

/*
  Get parameters 
*/
if (argc < 4)
    {
    printf("Usage: checkspace infile size dstdir\n");
    printf(" where: infile = input file\n");
    printf("        size   = number of bytes to search for\n");
    printf("        dstdir = destination directory\n");
    exit(2);
    }

/*
  Get the input file name
*/
strcpy(infile,argv[1]);
if (infile[0] == '\0')
    {
    /*
      If infile is not specified get size from next arg ("size")
    */
    strcpy(insize,argv[2]);
    if (insize[0] == '\0')
       {
       printf("[checkspace-spec]  Either INFILE or NBYTES must be specified\n");
       exit(-1);
       }
    nbytes = atof(insize);
    }
else
    {
    /*
      Get the size of infile
    */
    len = (long)strlen(infile);
    if (infile[len-1] == '/')
        infile[len-1] = '\0';
    c_getsizes((char (*)[256])infile,1,&nbytes);
    /*
      If file doen't exist, exit with error
    */
    if (nbytes < 0)
        exit(-1);
    }

/*
  Get the directory to search
*/
strcpy(dstdir,argv[3]);

/* 
  Find amount of space in the directory 
*/
if (dirspace(dstdir,&avail) != E_SUCC)
    exit(-1);

/*
  If not enough space, return error status
*/
if (avail < nbytes)
    exit(-1);

return 0;
}
