/******************************************************************************
FUNCTION:	dirspace

PURPOSE: 	check to see how much space is available in the directory

PROGRAM HISTORY:
 Version   	 Date    	 Author       	 Change Request
 -------   	 ----    	 ------       	 --------------
   1.0           06/92           T. Rockvam      Original development
							
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "diskio.h"

#if defined(sgi) || defined(DGUX)
#include        <sys/statfs.h>
#else
#ifdef sun
#include        <sys/statvfs.h>
#else
#include        <sys/vfs.h>
#endif
#endif

int dirspace
(
char 	*dstdir,		/* destination dir to search for space  */
double	*space 			/* space in destination dir		*/
)
{
#ifdef sun
struct statvfs   statfsp;        /* file system statistics       	*/
#else
struct statfs   statfsp;        /* file system statistics       	*/
#endif
struct stat     statp;        	/* file statistics       		*/

double 	num_free;		/* number of free bytes			*/

long long	reserved;	/* number of reserved bytes		*/
long	len;			/* length of directory name		*/

/*
  Get host directory name
*/
len = (long)strlen(dstdir);
if (dstdir[len-1] == '/')
    dstdir[len-1] = '\0';

/*
  Try to access the directory   
*/
if (stat(dstdir,&statp) == -1)
    return(E_FAIL);

/*
  Check if it really is a directory
*/
if (!(statp.st_mode & S_IFDIR))
    return(E_FAIL);

/*
  Get the file status information to determine the space avaliable	
*/
#if defined(sgi) || defined(DGUX)
    if (statfs(dstdir,&statfsp,sizeof(struct statfs),0) == -1)
#else
#ifdef sun
    if (statvfs(dstdir,&statfsp) == -1)
#else
    if (statfs(dstdir,&statfsp) == -1)
#endif
#endif
    return(E_FAIL);

/*
  Calculate the amount of available space				
*/
#ifdef sgi
       num_free = (double) statfsp.f_bfree * statfsp.f_bsize;
#else
       num_free = (double) statfsp.f_bavail * statfsp.f_bsize;
#endif

/*
  Subtract the amount of space that is software reserved		
*/
if ((reserved = c_dkrspac(dstdir)) == E_FAIL)
    return(E_FAIL);

*space = num_free - reserved;

return(E_SUCC);
}
