/******************************************************************************
NAME:		CORNERS

PURPOSE:    	The latitude and longitude are retreived from the DDR
		or the ADDR and return to the user through TAE name
		parameters.  The number of lines and samples in the
		image are also return along with the direction of the
		pass for images in satellite projection.

PROGRAM HISTORY
    PROGRAMMER	  DATE		REASON
    ----------	  ----		------
    D. Hollaren	  12/91		Conversion to LAS5.0

PROJECT:	ADAPS

ALGORITHM:

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util2.h"
#include "adaps.h"

/* Function prototype */
int getpar
(
struct PARBLK	*vblock,   /* TAE variable block			    */
char		*subcmd,   /* Subcommand used to run application            */
struct MODEL	*model,    /* Model and orbital information		    */
struct DDR	*ddr 	   /* AVHRR LAS Data Descriptor record		    */
);

int main()
{
struct PARBLK	vblock;		/* TAE variable block			     */
struct MODEL	model;		/* Model and orbital information	     */
struct DDR	ddr;		/* LAS image Data Descriptor Record	     */

long	status,			/* Return status variable		     */
	std_out,		/* Standard output			     */
	zero = 0,		/* Constant zero			     */
	fatal = -1;		/* Fatal error code			     */

float	line,			/* Line number relative to original pass     */
	samp;			/* Sample number relative to original pass   */

double	lat,			/* Latitude of line/sample corner	     */
	lng,			/* Longitude of line/sample corner	     */
	elev = 0.0;		/* Elevation				     */

char	subcmd[5],		/* Subcommand used to run the application    */
	outfil[1][CMLEN];	/* Output files created			     */

TAEINT	 array[2];		/* Array for output to variable block	     */
TAEFLOAT points[2];		/* Points in total degrees		     */

/*
  Initalize las
*/
c_inlas(&vblock,&std_out);

/*
  Get the AVHRR LAS image and retreive the model information
*/
status = getpar(&vblock,subcmd,&model,&ddr);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","corners-fatal",&fatal);

/*
  Write the coorner coordinates to the variable block
*/
if (strcmp(subcmd,"FULL") == 0)
    {
    points[0] = (float)(model.nw[0] * DEG_PER_RAD);
    q_real(&vblock,"ULLAT",1,points,P_UPDATE);

    points[0] = (float)(model.nw[1] * DEG_PER_RAD);
    q_real(&vblock,"ULLON",1,points,P_UPDATE);

    points[0] = (float)(model.nn[0] * DEG_PER_RAD);
    q_real(&vblock,"UNLAT",1,points,P_UPDATE);

    points[0] = (float)(model.nn[1] * DEG_PER_RAD);
    q_real(&vblock,"UNLON",1,points,P_UPDATE);

    points[0] = (float)(model.ne[0] * DEG_PER_RAD);
    q_real(&vblock,"URLAT",1,points,P_UPDATE);

    points[0] = (float)(model.ne[1] * DEG_PER_RAD);
    q_real(&vblock,"URLON",1,points,P_UPDATE);

    points[0] = (float)(model.cw[0] * DEG_PER_RAD);
    q_real(&vblock,"CLLAT",1,points,P_UPDATE);

    points[0] = (float)(model.cw[1] * DEG_PER_RAD);
    q_real(&vblock,"CLLON",1,points,P_UPDATE);

    points[0] = (float)(model.cn[0] * DEG_PER_RAD);
    q_real(&vblock,"CNLAT",1,points,P_UPDATE);

    points[0] = (float)(model.cn[1] * DEG_PER_RAD);
    q_real(&vblock,"CNLON",1,points,P_UPDATE);

    points[0] = (float)(model.ce[0] * DEG_PER_RAD);
    q_real(&vblock,"CRLAT",1,points,P_UPDATE);

    points[0] = (float)(model.ce[1] * DEG_PER_RAD);
    q_real(&vblock,"CRLON",1,points,P_UPDATE);

    points[0] = (float)(model.sw[0] * DEG_PER_RAD);
    q_real(&vblock,"LLLAT",1,points,P_UPDATE);

    points[0] = (float)(model.sw[1] * DEG_PER_RAD);
    q_real(&vblock,"LLLON",1,points,P_UPDATE);

    points[0] = (float)(model.sn[0] * DEG_PER_RAD);
    q_real(&vblock,"LNLAT",1,points,P_UPDATE);

    points[0] = (float)(model.sn[1] * DEG_PER_RAD);
    q_real(&vblock,"LNLON",1,points,P_UPDATE);

    points[0] = (float)(model.se[0] * DEG_PER_RAD);
    q_real(&vblock,"LRLAT",1,points,P_UPDATE);

    points[0] = (float)(model.se[1] * DEG_PER_RAD);
    q_real(&vblock,"LRLON",1,points,P_UPDATE);

    array[0] = model.nlines;
    q_intg(&vblock,"NLINES",1,array,P_UPDATE);

    array[0] = model.nsamples;
    q_intg(&vblock,"NSAMPS",1,array,P_UPDATE);

    array[0] = model.cpass;
    q_intg(&vblock,"ADFLAG",1,array,P_UPDATE);
    }
else
    {
    line = (float)(ddr.master_line + 1);
    samp = (float)(ddr.master_sample + 1);
    c_ls2ll(&line,&samp,&elev,&model,&lat,&lng,NULL);
    points[0] = (float)(lat * DEG_PER_RAD);
    q_real(&vblock,"ULLAT",1,points,P_UPDATE);

    points[0] = (float)(lng * DEG_PER_RAD);
    q_real(&vblock,"ULLON",1,points,P_UPDATE);

    line = (float)(ddr.master_line + 1);
    samp = (float)(ddr.master_sample + ddr.ns);
    c_ls2ll(&line,&samp,&elev,&model,&lat,&lng,NULL);
    points[0] = (float)(lat * DEG_PER_RAD);
    q_real(&vblock,"URLAT",1,points,P_UPDATE);

    points[0] = (float)(lng * DEG_PER_RAD);
    q_real(&vblock,"URLON",1,points,P_UPDATE);

    line = (float)(ddr.master_line + ddr.nl);
    samp = (float)(ddr.master_sample + 1);
    c_ls2ll(&line,&samp,&elev,&model,&lat,&lng,NULL);
    points[0] = (float)(lat * DEG_PER_RAD);
    q_real(&vblock,"LLLAT",1,points,P_UPDATE);

    points[0] = (float)(lng * DEG_PER_RAD);
    q_real(&vblock,"LLLON",1,points,P_UPDATE);

    line = (float)(ddr.master_line + ddr.nl);
    samp = (float)(ddr.master_sample + ddr.ns);
    c_ls2ll(&line,&samp,&elev,&model,&lat,&lng,NULL);
    points[0] = (float)(lat * DEG_PER_RAD);
    q_real(&vblock,"LRLAT",1,points,P_UPDATE);

    points[0] = (float)(lng * DEG_PER_RAD);
    q_real(&vblock,"LRLON",1,points,P_UPDATE);
    }

/*
  Send the varible block to TAE
*/
q_out(&vblock);

/*
  Display a completion message
*/
c_complt("CORNERS",outfil,&zero);
return E_SUCC;
}
