/*****************************************************************************
FUNCTION:	UPDDR

PURPOSE:	To update the ddr of the output image.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    06/99	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "creatmf.h"


int upddr
(
char	*outfile		/* host file name of output image	*/
)

{
long	band;			/* band number for getbdr() call	*/
long	status;			/* function return status		*/

struct	DDR	ddr;		/* ddr structure			*/
struct	BDDR	bddr[2];	/* band ddr structures			*/


/* get the current ddr
----------------------*/
status = c_getddr( outfile, &ddr );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"creatmf-getddr","Error retrieving output image ddr");
    return E_FAIL;
    }
band = 1;
status = c_getbdr ( outfile, &bddr[0], &band );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"creatmf-getbdr","Error retrieving output band ddr");
    return E_FAIL;
    }
band = 2;
status = c_getbdr ( outfile, &bddr[1], &band );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"creatmf-getbdr","Error retrieving output band ddr");
    return E_FAIL;
    }


/* update the projection parameters, coordinates and distances
and validate each field.
--------------------------------------------------------------*/
ddr.proj_code = 0;		/* geographic projection	*/
ddr.valid[0] = 1;		/* validate proj_code		*/
ddr.zone_code = 62;		/* zone is 62			*/
ddr.valid[1] = 1;		/* validate zone_code		*/
ddr.datum_code = 19;		/* datum is sphere		*/
ddr.valid[2] = 1;		/* validate datum_code		*/
/* Note: all projection parameters left as zero			*/
ddr.valid[3] = 1;		/* validate proj_parm		*/
ddr.upleft[0] = 89.5;		/* set up corner coordinates	*/
ddr.upleft[1] = -179.5;
ddr.loleft[0] = -89.5;
ddr.loleft[1] = -179.5;
ddr.upright[0] = 89.5;
ddr.upright[1] = 179.5;
ddr.loright[0] = -89.5;
ddr.loright[1] = 179.5;
ddr.valid[6] = 1;		/* validate corner coordinates	*/
ddr.pdist_y = ddr.pdist_x = 1.0;/* projection distance is one	*/
ddr.valid[5] = 1;		/* validate pdist		*/
strcpy( ddr.proj_units, "degrees" );
ddr.valid[4] = 1;		/* validate proj_units		*/
ddr.line_inc = ddr.sample_inc = 1.0;
ddr.valid[7] = 1;		/* validate increments		*/


/* write the updated ddr
------------------------*/
status = c_putddr ( outfile, &ddr );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"creatmf-putddr","Error writing the output image ddr");
    return E_FAIL;
    }
status = c_putbdr ( outfile, &bddr[0] );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"creatmf-putbdr","Error writing band 1 ddr");
    return E_FAIL;
    }
status = c_putbdr ( outfile, &bddr[1] );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"creatmf-putbdr","Error writing band 2 ddr");
    return E_FAIL;
    }


return E_SUCC;
}
