#ifndef DCW2RAS_H
#define DCW2RAS_H

#define SCANFIL	0	/* Index to temporary file array for scan file  */
#define SORTFIL 1	/* Index to temporary file array for sort file  */

#define NULZONE 62      /* GCTP constant signifying null zone   	*/
#define DEGREES 4       /* Degrees                              	*/

#define NORTH 1         /* North Pole                                   */
#define SOUTH -1        /* South Pole                                   */
#define NONE 0          /* No Pole                                      */

#define MINLAT 0
#define MAXLAT 1
#define MINLON 2
#define MAXLON 3

#ifndef MIN
#define MIN(a,b)  ((a) < (b) ? (a) : (b))
#endif
#ifndef MIN4
#define MIN4(a,b,c,d) (MIN(a,b) < MIN(c,d) ? MIN(a,b) : MIN(c,d))
#endif
#ifndef MAX4
#define MAX4(a,b,c,d) (LASMAX(a,b) > LASMAX(c,d) ? LASMAX(a,b) : LASMAX(c,d))
#endif

/* Function prototypes */
#include "las.h"
#include "dcw.h"
#include "model.h"

/* cal_lin_smp.c */
int cal_lin_smp
(
    struct DDR *ddr, 
    long    *ddr_units, 
    long	*geo_code, 
    long    *geo_units, 
    double	*lat, 
    double	*lon, 
    long   	*line, 
    long   	*samp
);

/* check_proj_pole.c */
int check_proj_pole
(
    struct DDR *ddr, 
    long 	*ddr_units, 
    long    *geo_code, 
    long    *geo_zone, 
    long    *geo_units, 
    double  geo_parms[]
);

/* find_path.c */
long find_path
(
    char	directory[]
);

/* finish.c */
void finish
(
    char    hosin[][CMLEN], 
    char    hosout[][CMLEN], 
    char    taein[][CMLEN], 
    char    taeout[][CMLEN], 
    struct PARBLK	*vblock, 
    struct MODEL	*model, 
    long 	window[1][4], 
    long    bands[1][MAXBND+1], 
    long    nbands[1], 
    char    proj[6], 
    long    overlay, 
    char    mnmx[]
);

/* genaddr.c */
void genaddr
(
    char hosout[CMLEN], 
    struct  MODEL   *model
);

/* genrst.c */
void genrst
(
    struct	PARBLK	*vblock, 
    long	window[], 
    long    *odtype, 
    long    overlay, 
    long    *backgrnd, 
    long    imgback, 
    long    *msgtim, 
    FILE	*fp_sort
);

/* getpar.c */
void getpar
(
    struct	PARBLK	*vblock, 
    struct  DDR	*ddr, 
    char	*cdpath, 
    long	*acount, 
    char    library[][10], 
    struct	DCW_COVERAGE	dcw_coverage[], 
    char    taein[][CMLEN], 
    char    hosin[][CMLEN], 
    char    taeout[][CMLEN], 
    char    hosout[][CMLEN], 
    long    window[][4], 
    long    bands[][MAXBND+1], 
    long    nbands[], 
    long    *odtype, 
    long    *pixval, 
    long    *backgrnd, 
    long    *imgback, 
    long    *linwt, 
    long    *overlay, 
    char    *proj, 
    long    *pntint, 
    long    *msgtim, 
    char    *mnmx, 
    double  *minlat, 
    double  *minlon, 
    double  *maxlat, 
    double  *maxlon, 
    long    *crossflg, 
    struct	MODEL	*model
);

/* inirst.c */
void inirst
(
    struct	PARBLK	*vblock, 
    char	*hosin, 
    char    *hosout, 
    long	window[], 
    long    bands[], 
    long    *odtype, 
    long    overlay, 
    long    imgback, 
    FILE	**fp_sort
);

/* llminmax.c */
void llminmax
(
    struct MODEL	*model, 
    long	ss, 
    long    es, 
    long    *crossflg, 
    double	*minlat, 
    double  *minlon, 
    double  *maxlat, 
    double  *maxlon
);

/* open_scan.c */
void open_scan
(
    FILE	**fp_scn
);

/* pminmax.c */
int pminmax
(
    struct  DDR     *ddr, 
    long    *ddr_units, 
    long    *geo_code, 
    long    *geo_zone, 
    long    *geo_units, 
    double  geo_parms[], 
    long    window[], 
    double  *minlat, 
    double  *minlon, 
    double  *maxlat, 
    double  *maxlon, 
    long    *crossflg
);

/* procproj.c */
int procproj
(
    struct DCW_LINE *dcw_line, 
    struct DDR *ddr, 
    long	window[], 
    long    pixval, 
    long    linwt, 
    long    pntint, 
    FILE 	*fp_scn, 
    long    *geo_code, 
    long    *geo_units, 
    long    *ddr_units 
);

/* procsat.c */
int procsat
(
    struct DCW_LINE	*dcw_line, 
    struct DDR     *ddr, 
    struct MODEL   *model, 
    long	window[], 
    double	lineinc, 
    double  sampinc, 
    long    pixval, 
    long    linwt, 
    long    pntint, 
    FILE 	*fp_scn
);

/* profindlat.c */
int profindlat
(
    long 	flag, 
    struct DDR *ddr, 
    long    *ddr_units, 
    long    *geo_code, 
    long    *geo_zone, 
    long    *geo_units, 
    double  geo_parms[], 
    double	*mylat
);

/* projsel.c */
int projsel
(
    struct  PARBLK  *vblock, 
    struct  DDR	*ddr, 
    char    *cdpath, 
    char    library[][10], 
    long    acount, 
    struct  DCW_COVERAGE *dcw_coverage, 
    long    pixval, 
    long    linwt, 
    long    window[], 
    long    pntint, 
    long    *msgtim, 
    FILE    *fp_scn, 
    double  minlat, 
    double  minlon, 
    double  maxlat, 
    double  maxlon
);

/* prominmax.c */
int prominmax
(
    struct DDR *ddr, 
    long 	*ddr_units, 
    long    *geo_code, 
    long    *geo_zone, 
    long    *geo_units, 
    double  geo_parms[], 
    long    window[4], 
    double	*minlat, 
    double  *minlon, 
    double  *maxlat, 
    double  *maxlon
);

/* satsel.c */
int satsel
(
    struct  PARBLK	*vblock, 
    struct  DDR     *ddr, 
    struct	MODEL	*model, 
    char	*cdpath, 
    char    library[][10], 
    long    acount, 
    struct	DCW_COVERAGE *dcw_coverage, 
    long	pixval, 
    long    linwt, 
    long    window[], 
    long    pntint, 
    long    *msgtim, 
    FILE	*fp_scn, 
    double	minlat, 
    double  minlon, 
    double  maxlat, 
    double  maxlon, 
    long    crossflg
);

/* scani.c */
void scani
(
    FILE	*fp_scn, 
    long	l1, 
    long    s1, 
    long    l2, 
    long    s2, 
    long	window[], 
    long    pixval, 
    long    linwt
);

/* sortscan.c */
void sortscan
(
    struct PARBLK *vblock
);

#endif
