/*******************************************************************************
NAME:   	FINISH

PURPOSE:        Update the image's ddr and history; create an addr file for OUT

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "util.h"
#include "util2.h"
#include "dcw2ras.h"

void finish
(
char    hosin[][CMLEN],  	/* input host file			*/
char    hosout[][CMLEN], 	/* output hos file image name         	*/
char    taein[][CMLEN],  	/* input TAE image name                 */
char    taeout[][CMLEN], 	/* output TAE image name                */
struct PARBLK	*vblock,	/* TAE parameter block			*/
struct MODEL	*model,		/* AVHRR model information		*/
long 	window[1][4],   	/* window specification for input/output image*/
long    bands[1][MAXBND+1],     /* band number to be processed          */
long    nbands[1],      	/* number of bands per input image      */
char    proj[6],        	/* projection type (satellite or reprojected)*/
long    overlay,        	/* overlay line work on input image flag*/
char    mnmx[]         		/* min/max calculation flag             */
)
{
double  dblone = 1.0,   	/* double value of 1                    */
        minval[MAXBND],   	/* minimum value of image               */
        maxval[MAXBND];   	/* maximum value of image               */

long	zero = 0,       	/* constant zero                        */
        one = 1;        	/* constant one                         */

if (overlay)
    {
    /* update the min & max 			*/
    if (strcmp(mnmx,"YES") == 0)
	{
    	/* invalidate the min & max 		*/
    	c_svmnmx(hosin[0],&one,minval,maxval,"NO");
        c_minmax((const char (*)[256])hosin,&one,window,bands,nbands,minval,maxval);
	}

    /* update the image's history		*/
    c_hstout(vblock,hosin[0],(const char (*)[256])" ",&zero," ");

    c_complt("DCW2RAS",taein,&one);
    }
else
    {
    /* generate an AVHRR ddr for the new file 	*/
    if (strcmp(proj,"SAT") == 0)
    	genaddr(hosout[0],model);

    /* update the new image's ddr     	*/
    c_upddr(vblock,(const char (*)[256])hosin,hosout[0],&one,
        (const long (*)[4])window,(const long (*)[257])bands,nbands,
		     &dblone,&dblone,&one,&one);

    /* calculate the min & max 			*/
    if (strcmp(mnmx,"YES") == 0)
        c_minmax((const char (*)[256])hosout[0],&one,window,bands,nbands,minval,maxval);

    /* update the image's history		*/
    c_hstout(vblock,hosout[0],(const char (*)[256])hosin,&one," ");

    c_complt("DCW2RAS",taeout,&one);
    }
}
