/*******************************************************************************
NAME:		GETPAR

PURPOSE:	Retreive the user specified input parameters

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "util2.h"
#include "adaps.h"
#include "archio.h"
#include "dcw.h"
#include "dcw2ras.h"
#include "proj.h"
#include "typlim.h"

void getpar
(
struct	PARBLK	*vblock,	/* TAE parameter block			     */
struct  DDR	*ddr,		/* DDR information			     */
char	*cdpath,		/* location of dcw			     */
long	*acount,		/* number of areas to rasterize		     */
char    library[][10],		/* libraries (areas) to use		     */
struct	DCW_COVERAGE	dcw_coverage[],	/* DCW coverage decriptions	     */
char    taein[][CMLEN],		/* input TAE image name			     */
char    hosin[][CMLEN],		/* input host image name		     */
char    taeout[][CMLEN],	/* output TAE image name		     */
char    hosout[][CMLEN],	/* output host image name		     */
long    window[][4],		/* window specification			     */
long    bands[][MAXBND+1],	/* bands to be processed		     */
long    nbands[],		/* band numbers selected		     */
long    *odtype,		/* output data type			     */
long    *pixval,		/* pixel intensity value for rasterized pixel*/
long    *backgrnd,		/* background value for rasterized image     */
long    *imgback,		/* background filled with image data flag    */
long    *linwt,			/* width of the rasterized line		     */
long    *overlay,		/* overlay line work option		     */
char    *proj,			/* projection type (satellite or rectified)  */
long    *pntint,		/* point interval option		     */
long    *msgtim,		/* time between processing messages	     */
char    *mnmx,			/* min/max calculation flag		     */
double  *minlat,            	/* minumum latitude in the image             */
double  *minlon,            	/* minumum longitude in the image            */
double  *maxlat,            	/* maximum latitude in the image             */
double  *maxlon,            	/* maximum longitude in the image            */
long    *crossflg,		/* i.d.l. crossing flag			     */
struct	MODEL	*model 		/* AVHRR model info			     */
)
{
struct VARIABLE	*var;

struct MODEL	winmodel;	/* AVHRR model info for user's window	     */

long	nmrimg,			/* number of input image specified	     */
	totbnd,			/* Total number of bands		     */
	same_window,		/* flag indicating if images hav same window */
	qflag[1][2],		/* flag indicating if subwindows specified   */
	nonfatal = 1,		/* non-fatal error code			     */
	fatal = -1,		/* fatal error code			     */
	zero = 0,		/* constant of zero			     */
	one = 1,		/* constant of one			     */
	status,			/* status return code			     */
	len,			/* string length			     */
	i,j,			/* looping variables			     */
	attribnum,		/* attribute number for each feature	     */
	startsec;		/* start time of user's window		     */

long    geo_code = GEO,		/* projection code                      */
        geo_zone = NULZONE,	/* projection zone                      */
        geo_units = DEGREES,	/* projection units                     */
        ddr_units;              /* projection units from image ddr      */

double  geo_parms[15];          /* projection parameters                */
double	tmp1,tmp2;		/* temporary				     */

char 	buf[12];		/* used to check compression of image	     */
FILE	*fd;

/*
  Get the input image name defining the area in which the vector
  data is to be rasterized
*/
status = c_getin(vblock,"IN",&zero,taein,hosin,&nmrimg,window,bands,nbands,
		 &totbnd,&one,&same_window,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);

/*
  Get the DDR information
*/
status = c_getddr(hosin[0],ddr);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
*odtype = ddr->dtype;

/*
  Get the output file name.  When an output name is not specified update the
  input image by overlaying line work on the input image.
*/
status = c_getout(vblock,"OUT",(char (*)[256])taeout[0],hosout);
if (strlen(taeout[0]) == 0)
    {
    *overlay = TRUE;
    *backgrnd = 0;
    /*
      Check to make sure that there is only one band to overlay linework on
    */
    if ((nmrimg > 1) || (totbnd > 1))
	{
	c_errmsg("Only a single band image allowed on input","dcw2ras-onebnd",
		  &nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}
    /*
      Make sure the image is not compressed so we can update it
    */
    if ((fd = fopen(hosin[0],"r")) != NULL)
        {
        if (fread(buf,sizeof(char),11,fd) != E_FAIL)
    	    {
	    buf[10] = '\0';
	    if (strcmp(buf,"Compressed") == 0)
	        {
	        c_errmsg("Compressed image cannot be opened for update",
		    "dcw2ras-update",&nonfatal);
   	        c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	        }
    	    }
        }
    }
else
    if (status == E_SUCC)
        {
        *overlay = FALSE;
        var = p_find(vblock,"BACKGRND");
	if (var->v_count == 0)
	    {
	    *imgback = TRUE;
            *backgrnd = 0;
    	    /*
      	      Check to ensure there is only one band to overlay linework on
    	    */
    	    if ((nmrimg > 1) || (totbnd > 1))
	        {
		c_errmsg("Only a single band image allowed on input",
			 "dcw2ras-onebnd",&nonfatal);
		c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
		}
	    }
	else
            *backgrnd = IVAL(*var,0);
        }
    else
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);

/*
  Get the pixel value intensity for the line work
*/
var = p_find(vblock,"PIXVAL");
*pixval = IVAL(*var,0);

/*
  Check that the background and pixel value are in a valid range for the data
  type of the input image.
*/
if (*odtype == EBYTE)
    {
    if ((*backgrnd > MAXUTINY) || (*backgrnd < MINUTINY))
	{
	c_errmsg("BACKGRND is not within the valid range for byte data",
	    "dcw2ras-range",&nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}

    if ((*pixval > MAXUTINY) || (*pixval < MINUTINY))
	{
	c_errmsg("PIXVAL is not within the valid range for byte data",
	    "dcw2ras-range",&nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}
    }
else if (*odtype == EWORD)
    {
    if ((*backgrnd > MAXWORD) || (*backgrnd < MINWORD))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*2 data",
	    "dcw2ras-range",&nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}

    if ((*pixval > MAXWORD) || (*pixval < MINWORD))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*2 data",
	    "dcw2ras-range",&nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}
    }
else if (*odtype == ELONG)
    {
    if ((*backgrnd > MAXLONG) || (*backgrnd < MINLONG))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*2 data",
	    "dcw2ras-range",&nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}

    if ((*pixval > MAXLONG) || (*pixval < MINLONG))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*2 data",
	    "dcw2ras-range",&nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}
    }
else /* (*odtype == EREAL) */
    {
    if ((*backgrnd > MAXREAL) || (*backgrnd < MINREAL))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*2 data",
	    "dcw2ras-range",&nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}

    if ((*pixval > MAXREAL) || (*pixval < MINREAL))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*2 data",
	    "dcw2ras-range",&nonfatal);
	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}
    }

/*
  Get the location of the DCW
*/
var = p_find(vblock, "CDPATH");
strcpy(cdpath, SVAL(*var, 0));

/*
  Get the projection type
*/
var = p_find(vblock,"PROJ");
strcpy(proj,SVAL(*var,0));

/*
  Retrieve the AVHRR DDR when the input image is in satellite prespective
*/
if (strcmp(proj,"SAT") == 0)
    {
    status = c_getaddr((char *)hosin,model);
    if (status != E_SUCC)
    	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
    model->sat_view = FALSE;

    /* set up a new model for just the user's window */
    /* initmod uses sat_view TRUE so we have to worry about ASC/DESC */
    winmodel.ehour = winmodel.emin = winmodel.esec = 0;
    if (model->dtype == GAC)
	if (model->cpass == ASCENDING)
            startsec = (long)(model->ssec + (model->nlines -
		(ddr->master_line+window[0][SL]+window[0][NL]-1))/GAC_PER_SEC);
	else
            startsec=(long)(model->ssec+(ddr->master_line+window[0][SL]-1)
	        /GAC_PER_SEC);
    else
	if (model->cpass == ASCENDING)
            startsec = (long)(model->ssec + (model->nlines - 
		(ddr->master_line+window[0][SL]+window[0][NL]-1))/LAC_PER_SEC);
	else
            startsec=(long)(model->ssec+(ddr->master_line+window[0][SL]-1)
	        /LAC_PER_SEC);

    c_initmod(&model->satnum,&model->year,&model->month,&model->day,
	      &model->shour,&model->smin,&startsec,&model->sfrac,
	      &model->dtype,&window[0][NL],&winmodel);
    winmodel.sat_view = TRUE;
    }
else
    {
    /*
      Check validity of the projection code, 
      corner projection coordinates, and projection distance
    */
    status = c_ckunit(ddr->proj_units,&ddr_units);
    if ((status != E_SUCC) ||
        (ddr->valid[0] == INVAL) ||
        (ddr->valid[DDCCV] == INVAL) ||
        (ddr->valid[DDPDV] == INVAL))
        {
        c_errmsg("Invalid projection information in DDR","dcw2ras-proj",
                 &nonfatal);
    	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
        }
    for (i = 0; i < 15; i++)
        geo_parms[i] = 0.0;
    }

/*
  Get the specified libraries (areas of interest)
*/
var = p_find(vblock, "LIBRARY");
*acount = (*var).v_count;
for (j = 0; j < *acount; j++)
    {
    strcpy(library[j], SVAL(*var,j));
    len = (long)strlen(library[j]);
    c_up2low(library[j],&len);
    }

/*
  Get the corner coordinates
*/
var = p_find(vblock,"ULCOOR");
if (var->v_count == 2)
    {
    *maxlat = RVAL(*var,0);
    *minlon = RVAL(*var,1);
    var = p_find(vblock,"LRCOOR");
    if (var->v_count == 2)
    	{
    	*minlat = RVAL(*var,0);
    	*maxlon = RVAL(*var,1);
	if (*minlon > *maxlon) 
	    {
	    tmp1 = *minlon;
	    *minlon = *maxlon;
	    *maxlon = tmp1;
	    *crossflg = TRUE;
	    }
    	}
    else if (var->v_count == 0)
	if (strcmp(proj,"SAT") == 0)
    	    llminmax(&winmodel,window[0][SS],window[0][SS]+window[0][NS]-1,
		crossflg,minlat,&tmp1,&tmp2,maxlon);
	else
	    if (pminmax(ddr,&ddr_units,&geo_code,&geo_zone,&geo_units,
		geo_parms, window[0],minlat,&tmp1,&tmp2,maxlon,crossflg) 
		!= E_SUCC)
    		c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
    else
    	{
    	c_errmsg("LRCOOR must have 0 or 2 values", "dcw2ras-values",&nonfatal);
    	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
    	}
    }
else if (var->v_count == 0)
    {
    /* calculate the image's corners */
    if (strcmp(proj,"SAT") == 0)
    	llminmax(&winmodel,window[0][SS],window[0][SS]+window[0][NS]-1,
		 crossflg,minlat,minlon,maxlat,maxlon);
    else
	if (pminmax(ddr,&ddr_units,&geo_code,&geo_zone,&geo_units,geo_parms,
		    window[0],minlat,minlon,maxlat,maxlon,crossflg) != E_SUCC)
    	    c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
    var = p_find(vblock,"LRCOOR");
    if (var->v_count == 2)
    	{
	/* replace LRCOOR if specified */
    	*minlat = RVAL(*var,0);
    	*maxlon = RVAL(*var,1);
    	}
    else if (var->v_count != 0)
    	{
    	c_errmsg("LRCOOR must have 0 or 2 values", "dcw2ras-values",&nonfatal);
    	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
    	}
    }
else
    {
    c_errmsg("ULCOOR must have 0 or 2 values", "dcw2ras-values",&nonfatal);
    c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
    }

/*
  Get selected features
*/
i = 0;
while (dcw_coverage[i].desc != NULL)
    {
    var = p_find(vblock, dcw_coverage[i].name);
    attribnum = 1;
    /* dcw_coverage[i].getstatus = TRUE; */
    if (dcw_definecover(cdpath, library[0], &dcw_coverage[i]) == E_SUCC)
	{
	/*
	  Get the selected attributes in each feature
	*/
	if ((*var).v_count == 0)
	    dcw_coverage[i].numvdt = 0;
	for (j = 0; j < (*var).v_count; j++)
	    {
	    dcw_coverage[i].vdt[IVAL(*var,j)-1].selected = TRUE;
	    dcw_coverage[i].vdt[IVAL(*var,j)-1].attindex = attribnum++;
	    }
	}
    else
	{
    	c_errmsg("Could not find coverage information","dcw2ras-cover",
		&nonfatal);
    	c_errmsg("Fatal error encountered","dcw2ras-fatal",&fatal);
	}
    i++;
    }

/*
  Get the width of the line
*/
var = p_find(vblock,"LINWT");
*linwt = IVAL(*var,0);

/*
  Get the point interval option
*/
var = p_find(vblock,"PNTINT");
*pntint = IVAL(*var,0);

/*
  Get the time between processing messages
*/
var = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*var,0);

/*
  Get the min/max processing flag
*/
var = p_find(vblock,"$MINMAX");
strcpy(mnmx,SVAL(*var,0));

return;
}
