/*****************************************************************************
NAME:		INIRST 

PURPOSE:	Initailize the image I/O

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "imageio.h"
#include "util.h"
#include "dcw2ras.h"

void inirst
(
struct	PARBLK	*vblock,/* TAE parameter block				*/
char	*hosin,		/* input host file name				*/
char    *hosout,	/* output host file name			*/
long	window[],	/* window to process				*/
long    bands[],	/* bands to process				*/
long    *odtype,	/* output data type				*/
long    overlay,	/* overlay flag					*/
long    imgback,	/* image background flag			*/
FILE	**fp_sort 	/* file pointer to the sorted file		*/
)
{

extern int fdesc[3];	/* image file descriptors used for image I/O	*/
extern int gdesc;	/* group file descriptor used for image I/O	*/

extern char *thebuf;	/* image buffer					*/
extern long ooffset[2];	/* offsets into "thebuf" for output bands	*/
extern long ioffset[2];	/* offsets into "thebuf" for input bands	*/

extern char tfiles[3][CMLEN];	/* output temporary host file names	*/
extern long numout;		/* number of temp host files		*/

static	long	opt[3]={0,0,0};	/* optional parameter array for EOPENS	*/
long	zero = 0,	/* constant zero				*/
	one = 1,	/* constant one					*/
	nonfatal = 1,	/* non-Fatal error code				*/
	access,		/* file access					*/
	bufsiz,		/* image I/O buffer size			*/
	status;		/* return status variable			*/

/*
  Open the sorted scan file
*/
*fp_sort = fopen(tfiles[numout-1],"r");
if (*fp_sort == NULL)
    {
    c_errmsg("Error opening scan file","dcw2ras-opnscn",&nonfatal);
    c_clean(vblock,"dcw2ras",(const char (*)[256])tfiles,&numout);
    }

if (overlay)
    {
    access = IUPDATE;
    status = c_eopens((struct FDESC **)&fdesc[0],hosin,bands,&window[SL],&window[SS],&window[NL],
		      &window[NS],ooffset,&access,odtype,&zero,opt);
    fdesc[1] = 0;
    }
else
    {
    if (imgback)
	{
    	/* open the input image since we need its data for the background */
        access = IREAD;
	status = c_eopens((struct FDESC **)&fdesc[0],hosin,bands,&window[SL],&window[SS],
		      	  &window[NL],&window[NS],ioffset,&access,odtype,
		      	  &zero,opt);
	if (status == E_FAIL)
    	    {
    	    fclose(*fp_sort);
    	    c_clean(vblock,"dcw2ras",(const char (*)[256])tfiles,&numout);
    	    }

        access = IWRITE;
    	status = c_eopens((struct FDESC **)&fdesc[1],hosout,&one,&one,&one,
		      	  &window[NL],&window[NS],ooffset,&access,odtype,
		      	  &zero,opt);
	fdesc[2] = 0;
	}
    else
	{
        access = IWRITE;
    	/* open the output image only */
    	status = c_eopens((struct FDESC **)&fdesc[0],hosout,&one,&one,&one,
		      	  &window[NL],&window[NS],ooffset,&access,odtype,
		      	  &zero,opt);
	fdesc[1] = 0;
	}
    strcpy(tfiles[numout],hosout);
    numout++;
    }

if (status == E_FAIL)
    {
    fclose(*fp_sort);
    c_clean(vblock,"dcw2ras",(const char (*)[256])tfiles,&numout);
    }

/*
  Allocate the I/O buffer
*/
thebuf = c_ealloc((struct FDESC **)fdesc,&bufsiz);
if (thebuf == NULL)
    {
    c_errmsg("Error allocating dynamic memory","dcw2ras-alloc",&nonfatal);
    fclose(*fp_sort);
    c_egclse((struct GDESC **)&gdesc);
    c_clean(vblock,"dcw2ras",(const char (*)[256])tfiles,&numout);
    }

/*
  Group the image
*/
status = c_egroup((struct FDESC **)fdesc,(struct GDESC **)&gdesc,
	(unsigned char *)thebuf,&bufsiz);
if (status != E_SUCC)
    {
    fclose(*fp_sort);
    c_egclse((struct GDESC **)&gdesc);
    c_clean(vblock,"dcw2ras",(const char (*)[256])tfiles,&numout);
    }

return;
}
