/******************************************************************************
FUNCTION:	PMINMAX

PURPOSE: 	Determine the min and max latitude and longitude of the image
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dcw2ras.h"

int pminmax
(
struct  DDR     *ddr,           /* DDR information                      */
long    *ddr_units,             /* projection units from image ddr      */
long    *geo_code,         	/* projection code                      */
long    *geo_zone,     		/* projection zone                      */
long    *geo_units,    		/* projection units                     */
double  geo_parms[],          	/* projection parameters                */
long    window[],		/* input image window			*/
double  *minlat,                /* minumum latitude in the image             */ 
double  *minlon,                /* minumum longitude in the image            */ 
double  *maxlat,                /* maximum latitude in the image             */ 
double  *maxlon,                /* maximum longitude in the image            */
long    *crossflg 		/* idl crossing flag			*/
)
{
long 	status;		/* function return status 	*/

double 	lat1,lon1,	/* lat/long coord		*/
	lat2,lon2;	/* lat/long coord		*/

/* First check to see if it crosses a pole 
   ---------------------------------------*/
status = check_proj_pole(ddr,ddr_units,geo_code,geo_zone,
			 geo_units,geo_parms);
if (status == NORTH)
    {
    /* if so (e.g. for North Pole) find minlat, use (minlat, 90, -180, 180) */
    if (profindlat(NORTH,ddr,ddr_units,geo_code,geo_zone,geo_units,geo_parms,
                   &lat1) != E_SUCC)
	return(E_FAIL);
    *minlat = (float)lat1;
    *maxlat =   90.0;
    *minlon = -180.0;
    *maxlon =  180.0;
    }
else if (status == SOUTH)
    {
    /* if so (e.g. for South Pole) find maxlat, use (-90, maxlat, -180, 180) */
    if (profindlat(SOUTH,ddr,ddr_units,geo_code,geo_zone,geo_units,geo_parms,
                   &lat1) != E_SUCC)
	return(E_FAIL);
    *minlat =  -90.0;
    *maxlat = (float)lat1;
    *minlon = -180.0;
    *maxlon =  180.0;
    }
else
    {
    /* if no pole use the usual min/max routine */
    status = prominmax(ddr,ddr_units,geo_code,geo_zone,geo_units,geo_parms,
                   window,&lat1,&lon1,&lat2,&lon2);
    if (status != E_SUCC)
        return(E_FAIL);

    *minlat = (float)lat1;
    *maxlat = (float)lat2;
    *minlon = (float)lon1;
    *maxlon = (float)lon2;

    if ((minlon - maxlon) > 180.0)
        *crossflg = TRUE;
    }

return(E_SUCC);
}
