/*******************************************************************************
NAME:		PROCPROJ

PURPOSE:	Process a line segment from the World Data Bank
		We quickly process the segment & don't bother to 
		check for duplicate segments

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dcw.h"
#include "adaps.h"
#include "dcw2ras.h"

int procproj
(
struct DCW_LINE *dcw_line,      /* current line                              */
struct DDR *ddr,	/* image DDR information			     */
long	window[],	/* window to process				     */
long    pixval,		/* pixel intensity value for rasterized pixel        */
long    linwt,		/* line weight of the rasterized line		     */
long    pntint,		/* point interval				     */
FILE 	*fp_scn,	/* file pointer to the scan file	             */
long    *geo_code,	/* geographic projection code			     */
long    *geo_units,	/* geographic projection units			     */
long    *ddr_units     	/* projection units from image ddr      	     */
)
{
double	lat,		/* temporary latitude in a double		     */
	lon;		/* temporary longitude in a double		     */

long 	l1, s1,		/* line/sample of first endpoint in line segment     */
	l2, s2;		/* line/sample of second endpoint in line segment    */

long	i;		/* looping counters				     */

/*
  Set floats into doubles
*/
lon = (double)dcw_line->pts[0];
lat = (double)dcw_line->pts[1];

/*
  Calculate the line/sample for the starting point of the line segments
*/
cal_lin_smp(ddr,ddr_units,geo_code,geo_units, &lat,&lon,&l2,&s2);

/*
  Process all the remaining points of the line segment
*/
for (i = 1; i < dcw_line->npts; i+=pntint)
    {
    /*
      Move the end of the line segment to be the beginning of the next
      line segment
    */
    l1 = l2;
    s1 = s2;

    /*
      Convert the latitude/longitude of the next point to a line/sample
    */
    lon = dcw_line->pts[2*i];
    lat = dcw_line->pts[2*i+1];
    cal_lin_smp(ddr,ddr_units,geo_code,geo_units, &lat,&lon,&l2,&s2);

    /*
      Validate the line segment is at least partially in the image
    */
    if (((l1 < window[SL]) && (l2 < window[SL])) ||
	((l1 > (window[SL] + window[NL]-1)) &&
	(l2 > (window[SL] + window[NL]-1))))
	continue;

    if (((s1 < window[SS]) && (s2 < window[SS])) ||
	((s1 > (window[SS] + window[NS] - 1)) &&
	(s2 > (window[SS] + window[NS]-1))))
	continue;

    /*
      Validate the line segment doesn't cross the whole image
    */
    if (((s1 < window[SS]) && (s2 > (window[SS] + window[NS] - 1))) ||
	((s1 > (window[SS] + window[NS] - 1)) && (s2 < window[SS])))
	continue;
    if (((l1 < window[SL]) && (l2 > (window[SL] + window[NL] - 1))) ||
	((l1 > (window[SL] + window[NL] - 1)) && (l2 < window[SL])))
	continue;

    /*
      Determine the samples to be filled in for this line segment
    */
    scani(fp_scn,l1,s1,l2,s2,window,pixval,linwt);
    }

/*
  Find out if we still need to process the last point
*/
if (((i - dcw_line->npts + 1) % pntint) != 0)
    {
    /*
      Move the end of the line segment to be the beginning of the next
      line segment
    */
    l1 = l2;
    s1 = s2;

    /*
      Convert the latitude/longitude of the last point to a line/sample
    */
    lat = dcw_line->pts[(dcw_line->npts-1)*2+1];
    lon = dcw_line->pts[(dcw_line->npts-1)*2];
    cal_lin_smp(ddr,ddr_units,geo_code,geo_units, &lat,&lon,&l2,&s2);

    /*
      If the line segment is at least partially in the image,
      determine the samples to be filled in for this line segment
    */
    if (((l1 >= window[SL]) || (l2 >= window[SL])) &&
       ((l1 <= (window[SL] + window[NL] - 1)) ||
        (l2 <= (window[SL] + window[NL] - 1))))
    if (((s1 >= window[SS]) || (s2 >= window[SS])) &&
       ((s1 <= (window[SS] + window[NS] - 1)) ||
        (s2 <= (window[SS] + window[NS] - 1))))
            scani(fp_scn,l1,s1,l2,s2,window,pixval,linwt);
    }

return(E_SUCC);
}
