/*******************************************************************************
NAME:   SATSEL

PURPOSE:        Select lines from DCW for placement in the image

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "util2.h"
#include "dcw.h"
#include "adaps.h"
#include "dcw2ras.h"

int satsel
(
struct  PARBLK	*vblock,	/* TAE parameter block			     */
struct  DDR     *ddr,           /* DDR information                           */
struct	MODEL	*model,	   	/* AVHRR model information		     */
char	*cdpath,		/* dcw location				     */
char    library[][10],		/* list of libraries (areas)		     */
long    acount,			/* number of areas			     */
struct	DCW_COVERAGE *dcw_coverage,	/* Coverage areas		     */
long	pixval,			/* pixel value of rasterized line	     */
long    linwt,			/* weight of the line to be processed	     */
long    window[],		/* window to process			     */
long    pntint,			/* point intervals 			     */
long    *msgtim,		/* time between processing messages	     */
FILE	*fp_scn,		/* file pointer to scan file		     */
double	minlat,			/* minimum latitude			     */
double  minlon,			/* minimum longitude			     */
double  maxlat,			/* maximum latitude			     */
double  maxlon,			/* maximum longitude			     */
long    crossflg           	/* indicates crossing Internat'l Date Line   */
)
{
struct	DCW_VECTOR dcw_vector;	/* set of lines for the area		     */

extern char tfiles[3][CMLEN];	/* output temporary host file names	     */
extern long numout;		/* number of output temp files		     */
extern long curent;		/* current "line" for processimg messages    */

long    i, 		   	/* looping variables			     */
	len,		   	/* string length			     */
	msgfmt = GEN1,	   	/* general message format one		     */
	max_entity = 9999, 	/* maximum entity for processing message     */
	nonfatal = 1,	   	/* non-fatal error code			     */
	segment,	   	/* current segment			     */
	status,stat,   	   	/* return status			     */
	zero = 0;	   	/* constant of zero			     */

char	ptext[ERRLEN],     	/* processing message text		     */
	curlib[10];		/* current library			     */

double	lineinc,sampinc;	/* line and sample increment		     */
double	lat1,lat2,
	lon1,lon2;

/*
  Find out if this image was subsampled
  other than the normal
*/
if (model->dtype == GAC)
    {
    if (ddr->line_inc != 4.0)
        /* divide by the original line increment */
        lineinc = ddr->line_inc / 4.0;
    else
        lineinc = 1.0;
    if (ddr->sample_inc != 5.0)
        /* divide by the original sample increment */
        sampinc = ddr->sample_inc / 5.0;
    else
        sampinc = 1.0;
    }
else
    {
    lineinc = ddr->line_inc;
    sampinc = ddr->sample_inc;
    }

/*
  Rasterize each area
*/
for (i = 0; i < acount; i++)
    {
    /*
      Break the areas up so we don't read so many lines from dcw at once
    */
    if (crossflg)
        {
        lon1 = maxlon;
        if ((maxlon - minlon) > 15.0)
	    {
            lon2 = ((int)(maxlon/15) + 1.0 - (maxlon<0.0)) * 15.0;
            if (lon2 >= 180.0) lon2 = 180.0;
	    }
        else
	    {
            lon2 = minlon;
            if (lon2 < lon1)   lon2 = 180.0;
	    }
        }
    else
        {
        lon1 = minlon;
        if ((maxlon - minlon) > 15.0)
            lon2 = ((int)(minlon/15) + 1.0 - (minlon<0.0) ) * 15.0;
        else
            lon2 = maxlon;
        }

    /*
      Initialize and start the processing message
    */
    strcpy(curlib,library[i]);
    len = (long)strlen(curlib);
    c_low2up(curlib,&len);
    sprintf(ptext,"for %s ",curlib);
    curent = 1;
    c_inittm("DCW2RAS","Rasterized","line"," ",ptext,&max_entity,&zero,msgtim,
	 &msgfmt);
    c_startm();

    for(;;)
	{
	lat1 = minlat;
	if ((maxlat - minlat) > 15.0)
    	    lat2 = ((int)(minlat/15) + 1.0 - (minlat<0.0) ) * 15.0;
	else
    	    lat2 = maxlat;

	for(;;)
	    {
	    /*
  	      Allocate first line memory block
  	      (if this block is exceeded, it will be reallocated within the
  	      dcw support routines).
	    */
    	    dcw_vector.numlines = 0;
    	    dcw_vector.maxline = LINEBLOCK;
	    dcw_vector.line = (struct DCW_LINE *)malloc(LINEBLOCK*sizeof
							(struct DCW_LINE));
	    if (dcw_vector.line == NULL)
	    	{
	        c_errmsg("Error allocating dynamic memory",
		    "dcw2ras-alloc", &nonfatal);
		return(E_FAIL);
		}

    	    /*
      	      Perform the dcw query, loading all lines that fit our area
    	    */
    	    status = dcw_query(cdpath,library[i],dcw_coverage,&dcw_vector,
		       	    lat1,lon1,lat2,lon2);
    	    if (status == E_FAIL)
	        {
 	        c_errmsg("Error occurred querying DCW","dcw2ras-query",
			&nonfatal);
	        return(E_FAIL);
	        }

    	    /*
      	      If lines were found, generate corresponding 
	      line/samples (rasterize)
    	    */
    	    for (segment = 0; segment < dcw_vector.numlines; segment++)
      	    	{
	    	/*
	      	  Process each segment
	    	*/
	    	stat = procsat(&dcw_vector.line[segment],ddr,model,window,
			    	lineinc,sampinc,pixval,linwt,pntint,fp_scn);
	    	if (stat != E_SUCC)
	            {
	            fclose(fp_scn);
	            c_clean(vblock,"dcw2ras",(const char (*)[256])tfiles,&numout);
	            }
   	 
	    	/*
	      	  Increment the segment counter
	    	*/
	    	curent++;
	    	}

	    if (lat2 == maxlat) break;
	    lat1 = lat2;
	    lat2 += 15.0;
	    if (lat2 > maxlat) lat2 = maxlat;

	    /*
	      Free lines
	    */
	    dcw_freelines(&dcw_vector);
	    }

	if (crossflg)
	    {
	    if (lon2 == minlon) break;
	    lon1 = lon2;
	    if (lon1 >= 180.0) lon1 = -180.0;
	    lon2 = lon1 + 15.0;
	    if ((lon2 < 0.0) && (lon2 > minlon)) lon2 = minlon;
	    }
	else
	    {
	    if (lon2 == maxlon) break;
	    lon1 = lon2;
	    lon2 += 15.0;
	    if (lon2 > maxlon) lon2 = maxlon;
	    }
	}

    /*
      Stop the processing message
    */
    curent--;
    c_stoptm();
    }

/*
  Close the scan file
*/
fclose(fp_scn);

return(E_SUCC);
}
