/******************************************************************************
NAME:		DSPADDR

PURPOSE:	Display the AVHRR Data Descriptor Record in a formatted
		output.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 D. Hollaren   03/92	Origianal Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "model.h"
#include "archio.h"
#include "dspaddr.h"

int main()
{
FILE	*fp;			/* File pointer to the print file	     */

long	fopt,			/* Output file option			     */
	status,			/* Return status			     */
	std_out,			/* LUN to standard output */
	count = 1,		/* Number of output files created	     */
	fatal = -1,		/* Fatal error code			     */
	nonfatal = 1;		/* Nonfatal error code			     */

char	hosin[CMLEN],		/* Input host file name			     */ 
	prtfil[CMLEN],		/* Temporary print file name for fileutil    */
	outfil[1][CMLEN];	/* Output files created			     */

struct PARBLK	vblock;		/* TAE variable block			     */
struct MODEL	model;		/* Model and orbital information	     */
struct DROPPED	drop;		/* Avhrr dropped line information	     */

/*
  Initialization LAS
*/
c_inlas(&vblock,&std_out);

/*
  Get the input parameters
*/
get_addr_par(&vblock,hosin,&fopt,prtfil,outfil[0]);

/*
  Get the model and orbital information from the AVHRR DDR
*/
status = c_getaddr(hosin,&model);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","dspaddr-fatal",&fatal);

/*
  Get the dropped line information
*/
status = getdrop(hosin,&drop);

/*
  Open the temporary print file
*/
fp = fopen(prtfil,"w");
if (fp == NULL)
    {
    c_errmsg("Error opening temporary file for fileutil","dspaddr-open",
	     &nonfatal);
    c_errmsg("Fatal error encountered","dspaddr-fatal",&fatal);
    }

/*
  Write out a formatted report
*/
write_report(hosin,&model,&drop,fp);

/*
  Set the number of output files created
*/
if (!fopt)
    count = 0;

/*
  Display a standard completion message
*/
c_complt("DSPADDR",outfil,&count);
return E_SUCC;
}
