/******************************************************************************
NAME:		GETDROP

PURPOSE:	Read the associated dropped line file and put the
		information into the dropped line structure.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 D. Hollaren   03/92	Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "labserv.h"
#include "vll.h"
#include "archio.h"
#include "dspaddr.h"

int getdrop
(
char		*hfile,	/* Host image file name				     */
struct DROPPED	*drop 	/* Dropped line structure			     */
)
{
char	dfile[CMLEN];	/* Dropped line file name			     */

long	i,		/* Looping variable				     */
	totmiss,	/* Total number of missed lines			     */
	nonfatal = 1;	/* Nonfatal error code				     */

FILE	*fp;		/* File pointer to the dropped line file	     */

/*
  Get the name of the associated dropped line file
*/
c_lsmknm(hfile,"drpl",dfile);

/*
  Open the associated dropped line file
*/
fp = fopen(dfile,"r");
if (fp == NULL)
    {
    c_errmsg("Error opening the dropped line file","dspaddr-open",&nonfatal);
    return(E_FAIL);
    }

/*
  Get the number of gaps and total number of missed lines
*/
fscanf(fp,"%d %d",&drop->ngaps,&totmiss);

/*
  Check the maximum number of gaps allowed
*/
if (drop->ngaps > MAX_GAPS)
    {
    c_errmsg("Too many line gaps found;  Reset to maximum","dspaddr-gaps",
	     &nonfatal);
    drop->ngaps = MAX_GAPS;
    }

/*
  Get the line number were the gaps start and the number of lines in each gap
*/
for (i = 0; i < drop->ngaps; i++)
    {
    if (fscanf(fp,"%d %d",&drop->lmissed[i],&drop->nmissed[i]) != 2)
	{
	c_errmsg("No corresponding line number or number of missed lines",
		 "dspaddr-read",&nonfatal);
	drop->ngaps = i - 1;
	return(E_FAIL);
	}

    }

/*
  Close the associated dropped line file
*/
fclose(fp);

return(E_SUCC);
}
