/******************************************************************************
NAME:		DSPMNR

PURPOSE:	Display the AVHRR Minor Frame Format output.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 T. Baltzer    01/93	Origonal Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
    Kidwell, Katherine B. "NOAA Polar Orbiter Data Users Guide"
    3-4 through 3-6 pp.
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util2.h"
#include "labserv.h"
#include "adaps.h"
#include "dspmnr.h"

int main()
{
FILE	*fp;			/* File pointer to the print file	     */

long	minfd;                  /* Label services minor frame file descriptor*/

long	fopt,			/* Output file option			     */
	status,			/* Return status			     */
	std_out,			/* LUN to standard output */
	count = 1,		/* Number of output files created	     */
	fatal = -1,		/* Fatal error code			     */
	nonfatal = 1,		/* Nonfatal error code			     */
	sl,			/* Starting  lines in image to display	     */
	nl,			/* Number of lines in image to display	     */
	zero = 0;		/* Constant of zero			     */

char	hosmnr[CMLEN],		/* Input minor frame file name		     */ 
	listopt[7][4],		/* List of options to report		     */
	prtfil[CMLEN],		/* Temporary print file name for fileutil    */
	outfil[1][CMLEN];	/* Output files created			     */

struct PARBLK	vblock;		/* TAE variable block			     */

/*
  Initialization LAS
*/
c_inlas(&vblock,&std_out);

/*
  Get the input parameters
*/
getpar(&vblock,hosmnr,&fopt,prtfil,outfil[0],&sl,&nl,listopt);

/*
  Open the minor frame file
*/
status = c_lsopen(&minfd,hosmnr,&zero);
if (status == E_FAIL)
    c_errmsg("Fatal error encountered","dspmnr-fatal",&fatal);

/*
  Open the temporary print file
*/
fp = fopen(prtfil,"w");
if (fp == NULL)
    {
    c_errmsg("Error opening temporary file","dspmnr-open",
	     &nonfatal);
    c_errmsg("Fatal error encountered","dspmnr-fatal",&fatal);
    }

/*
  Write out a formatted report
*/
status = write_report(hosmnr,&minfd,fp,sl,nl,listopt);
if (status == E_FAIL)
    c_errmsg("Fatal error encountered","dspmnr-fatal",&fatal);

/*
  Set the number of output files created
*/
if (!fopt)
    count = 0;

/*
  Close the minor frame file
*/
c_lsclos(&minfd,hosmnr,&zero);

/*
  Display a standard completion message
*/
c_complt("DSPMNR",outfil,&count);
return E_SUCC;
}
