/******************************************************************************
NAME:		GETPAR

PURPOSE:	Retrieves parameters used to display Minor Frame Format.

PROGRAM HISTORY:
VERSION  AUTHOR        DATE     REASON
-------  ------        -----    -----------------------------
  5.0    T. Baltzer    01/93    Origonal Development

ALGORITHM

ALGORITHM REFERENCES	none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "labserv.h"
#include "util.h"
#include "util2.h"
#include "adaps.h"
#include "adapsutil.h"
#include "dspmnr.h"

void getpar
(
struct PARBLK	*vblock,	/* TAE parameter block			*/
char		*hosmnr,	/* Minor Frame host file name		*/
long		*fopt,		/* Output to file option		*/
char            *prtfil,	/* Temporary print file name		*/
char            *outfil,	/* List of output files created		*/
long		*sl,		/* starting line to start displaying	*/
long		*nl,		/* number of lines to display		*/
char            listopt[][4] 	/* List of options to report		*/
)
{ 
struct DDR      ddr;            /* DDR information structure            */

struct VARIABLE *val;           /* pointer to a TAE variable            */

long	access = IREAD,		/* Access type to read the ADDR		*/
	fatal = -1,             /* Fatal error code                     */
	index,			/* index				*/
	len,			/* length of string			*/
	lopt,			/* Output to line printer option	*/
	nonfatal = 1,		/* Non-fatal error code			*/
	status,                 /* Return value                         */
	topt,			/* Output to terminal option		*/
	zero = 0;		/* Constant of zero			*/

/*
  Get the input name
*/
status = c_getnam(vblock,"IN",&zero,hosmnr,"mnr",&access);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","dspmnr-fatal",&fatal);

/*
  Retrieve the starting line number
*/
val = p_find(vblock,"SL");
*sl = IVAL(*val,0);

/*
  Retrieve the number of lines to display.  If the user did
  not specify number of lines ('NL'), the remainder of the
  Minor Frame data will be displayed starting at line number
  'SL'.
*/
val = p_find(vblock,"NL");
if (val->v_count != 0)
   {
   *nl = IVAL(*val,0);
   }
else
   {
   /*
     Get the number of lines in the image from the ddr
   */
   status = c_getddr(hosmnr,&ddr);
   if (status != E_SUCC)
       {
       c_errmsg("DDR not available / NL not specified","dspmnr-ddrnl",&nonfatal);
       c_errmsg("Fatal error encountered","dspmnr-fatal",&fatal);
       }
   *nl = ddr.nl - *sl + 1;
   }

/*
  Make certain a positive number of lines are to be displayed
*/
if (*nl < 1)
   {
   c_errmsg("Invalid number of lines specified","dspmnr-lines",&nonfatal);
   c_errmsg("Fatal error encountered","dspmnr-fatal",&fatal);
   }

/*
   Get the options to list
*/
val = p_find(vblock,"LISTOPT");
for (index = 0; index < val->v_count; index++)
   {
   strcpy(listopt[index],SVAL(*val,index));
   len = (long)strlen(listopt[index]);
   c_low2up(listopt[index],&len);
   }

/*
  Get print option
*/
strcpy(prtfil,"dspmnr");
status = c_setprt(vblock,(int *)&topt,(int *)fopt,(int *)&lopt,prtfil,outfil);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","dspmnr-fatal",&fatal);

return;
}
