/******************************************************************************
NAME:		WRITE_LINE

PURPOSE:	Write formatted output for one line of the AVHRR
		Minor Frame Format.

PROGRAM HISTORY:

VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 T. Baltzer    01/93	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "adapsutil.h"
#include "dspmnr.h"

void write_line
(
long	line,		/* Line counter					*/
short   min_frm[],	/* Buffer of minor frame data 			*/
char	lname[],	/* LAS name of minor frame file			*/
FILE 	*fp,		/* File pointer to print file			*/
char	listopt[][4] 	/* List of options to report			*/
)
{

long	count,			/* Counter				*/
	frame,			/* Frame sync variable			*/
	jday,			/* Julian day				*/
	milsec,			/* Julian time in milliseconds		*/
	nperiods,		/* Number of periods			*/
	telvar,			/* Telemetry variable			*/
	satidx,                 /* Acquisition satellite id		*/
	satnum,			/* NOAA satellite number		*/
	tvar1,			/* Temporary variable for channel 1	*/
	tvar2,			/* Temporary variable for channel 2	*/
	tvar3,			/* Temporary variable for channel 3	*/
	tvar4,			/* Temporary variable for channel 4	*/
	tvar5,			/* Temporary variable for channel 5	*/
	arryptr,		/* Array pointer			*/
	thour,			/* Julian time (hours)			*/
	tmin,			/* Julian time (minutes)		*/
	tsec,			/* Julian time (seconds)		*/
	tfrac,			/* Julian time (fraction of seconds)	*/
	wcount;			/* Word counter				*/



fprintf(fp,"\n\n\nFILE: %s\n\n",lname);
fprintf(fp,"Line Number %d :\n",line);
fprintf(fp,"----------------------------------------------------------------------\n");

/***********************************************************/
if (strcmp(listopt[0],"ON") == 0)
   {
   fprintf(fp,"FRAME SYNC\n");
   fprintf(fp,"                           Bit Number\n");
   fprintf(fp,"                     1 2 3 4 5 6 7 8 9 10\n");
   fprintf(fp,"                     --------------------\n");
   for (wcount = 1; wcount <= 6; wcount++)
      {
      fprintf(fp,"    Word Number %2d: ", wcount);
      for (count = 9; count >= 0; count--)
         {
         frame = ((min_frm[0] >> count) & 0x1);
         fprintf(fp,"%2d", frame);
         }
      fprintf(fp,"\n");
      }
   fprintf(fp,"\n");
   }

/***********************************************************/
if (strcmp(listopt[1],"ON") == 0)
   {
   fprintf(fp,"ID (AVHRR)\n");
   /*
     Determine sync
   */
   if (((min_frm[6] >> 9) & 0x1) == 0)
      fprintf(fp,"    Sync:  Internal Sync");
   else
      fprintf(fp,"    Sync:  AVHRR Sync");
   /*
     Determine minor frame number
   */
   switch ((min_frm[6] >> 7) & 0x3)
      {
      case 0: fprintf(fp,"    Minor Frame Number: (unused)\n");break;
      case 1: fprintf(fp,"    Minor Frame Number:  1\n");	break;
      case 2: fprintf(fp,"    Minor Frame Number:  2\n");	break;
      case 3: fprintf(fp,"    Minor Frame Number:  3\n");	break;
      }
   /*
     Get the acquisition satellite id
     and satellite number
   */
   satidx = (min_frm[6] >> 3) & 0xf;
   switch (satidx)
       {
       case 1: satnum = 11; break;
       case 9: satnum = 12; break;
       case 11: satnum = 9; break;
       case 15: satnum = 10; break;
       default: satnum = 11;
       }
   fprintf(fp,"    Acquisition Satellite ID:  %d",satidx);
   fprintf(fp,"    NOAA Satellite Number:  %d\n",satnum);
   /*
     Determine frame status
   */
   if (((min_frm[6] >> 2) & 0x1) == 0)
      fprintf(fp,"    Frame Status:  Stable\n");
   else
      fprintf(fp,"    Frame Status:  Resync Occurred\n");
   }

/***********************************************************/
if (strcmp(listopt[2],"ON") == 0)
   {
   fprintf(fp,"\nTIME CODE\n");
   /*
     Get the julian date and time of the current line
   */
   jday = (min_frm[8] >> 1) & 0x3ff;
   fprintf(fp,"    Julian Date:  %d\n",jday);
   milsec = min_frm[9] & 0x7f;
   milsec = (milsec << 10) | (min_frm[10] & 0x3ff);
   milsec = (milsec << 10) | (min_frm[11] & 0x3ff);
   fprintf(fp,"    Time (msec):  %d",milsec);
   /*
     Convert the time in milliseconds to a readable format
   */
   c_cvtmil(&milsec,&thour,&tmin,&tsec,&tfrac);
   fprintf(fp,"    Time (HH:MM:SS.SSS):  %d:%d:%d.%3d\n",
           thour,tmin,tsec,tfrac);
   }

/***********************************************************/
if (strcmp(listopt[3],"ON") == 0)
   {
   fprintf(fp,"\nTELEMETRY (AVHRR)\n");
   /*
     Get the ramp calibration values for each channel
   */
   telvar = min_frm[12] & 0x3ff;
   fprintf(fp,"    Ramp Calibration AVHRR Channel 1:  %d\n",telvar);
   telvar = min_frm[13] & 0x3ff;
   fprintf(fp,"    Ramp Calibration AVHRR Channel 2:  %d\n",telvar);
   telvar = min_frm[14] & 0x3ff;
   fprintf(fp,"    Ramp Calibration AVHRR Channel 3:  %d\n",telvar);
   telvar = min_frm[15] & 0x3ff;
   fprintf(fp,"    Ramp Calibration AVHRR Channel 4:  %d\n",telvar);
   telvar = min_frm[16] & 0x3ff;
   fprintf(fp,"    Ramp Calibration AVHRR Channel 5:  %d\n",telvar);
   telvar = min_frm[17] & 0x3ff;
   fprintf(fp,"    AVHRR Internal Target Temperature Data 1:  %d\n",telvar);
   telvar = min_frm[18] & 0x3ff;
   fprintf(fp,"    AVHRR Internal Target Temperature Data 2:  %d\n",telvar);
   telvar = min_frm[19] & 0x3ff;
   fprintf(fp,"    AVHRR Internal Target Temperature Data 3:  %d\n",telvar);
   telvar = min_frm[20] & 0x3ff;
   fprintf(fp,"    AVHRR Patch Temperature:  %d\n",telvar);
   }

/***********************************************************/
if (strcmp(listopt[4],"ON") == 0)
   {
   fprintf(fp,"\nINTERNAL TARGET DATA (AVHRR)\n");
   fprintf(fp,"                    Channel  Channel  Channel\n");
   fprintf(fp,"                       3        4        5\n");
   fprintf(fp,"                    -------  -------  -------\n");
   for (count = 1, arryptr = 22; count <= 10; count++)
      {
      tvar3 = min_frm[arryptr++] & 0x3ff;
      tvar4 = min_frm[arryptr++] & 0x3ff;
      tvar5 = min_frm[arryptr++] & 0x3ff;
      fprintf(fp,"    Word Number %2d: %6d   %6d   %6d\n",
              count,tvar3,tvar4,tvar5);
      }
   }

/***********************************************************/
if (strcmp(listopt[5],"ON") == 0)
   {
   fprintf(fp,"\nSPACE-SCAN DATA (AVHRR)\n");
   fprintf(fp,"                    Channel  Channel  Channel  Channel  Channel\n");
   fprintf(fp,"                       1        2        3        4        5\n");
   fprintf(fp,"                    -------  -------  -------  -------  -------\n");
   for (count = 1, arryptr = 52; count <= 10; count++)
      {
      tvar1 = min_frm[arryptr++] & 0x3ff;
      tvar2 = min_frm[arryptr++] & 0x3ff;
      tvar3 = min_frm[arryptr++] & 0x3ff;
      tvar4 = min_frm[arryptr++] & 0x3ff;
      tvar5 = min_frm[arryptr++] & 0x3ff;
      fprintf(fp,"    Word Number %2d: %6d   %6d   %6d   %6d   %6d\n",
              count,tvar1,tvar2,tvar3,tvar4,tvar5);
      }
   }

/***********************************************************/
if (strcmp(listopt[6],"ON") == 0)
   {
   fprintf(fp,"\nSYNC (AVHRR)\n");
   /*
     Determine sync status
   */
   if (((min_frm[102] >> 9) & 0x1) == 0)
      fprintf(fp,"    AVHRR Sync:  Early");
   else
      fprintf(fp,"    AVHRR Sync:  Late");
   /*
     Get the number of periods
   */
   nperiods = min_frm[102] & 0x1ff;
   fprintf(fp,"    Number of 0.9984-MHz periods:  %d\n",nperiods);
   }

return;
}
