/***************************************************************************
FUNCTION:	FILLTOVS

PURPOSE:	To fill the holes in a TOVS water vapor image and create
		a QC image indicating the origin of the values in the
		filled image.

Version	Date	Programmer	Modification
-------	-----	------------	--------------------------------
  1.0	10/99	D.Lloyd		Initial Development

PROJECT:	ADAPS


***************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "filltovs.h"



int main( void )
{
char	infile[3][CMLEN];	/* input file names			*/
char	out[2][CMLEN];		/* list of output files			*/
char	outfile[CMLEN];		/* output file name			*/
char	qcimg[CMLEN];		/* QC image file name			*/

float	qcbuf[NLINES][NSAMPS] = {0.0};	/* quality control buffer	*/
float	wvbuf[NLINES][NSAMPS];	/* daily observation H2O buffer		*/

long	outcnt = 2;		/* count of output files		*/
long	status;			/* function return status		*/
long	stdlun;			/* standard output			*/
long	zero = 0;		/* zero constant			*/

struct	PARBLK	vblock;		/* parameter block			*/


/* Initialize LAS
-----------------*/
c_inlas( &vblock, &stdlun );


/* Get the parameters
---------------------*/
status = getpar( &vblock, infile, outfile, qcimg );
if ( status != E_SUCC )
    c_clean(&vblock,"FILLTOVS","",&zero);


/* Create the output image buffers
----------------------------------*/
status = process( infile, wvbuf, qcbuf );
if ( status != E_SUCC )
    c_clean(&vblock,"FILLTOVS","",&zero);


/* Create and write the output images
-------------------------------------*/
status = write_output( outfile, qcimg, wvbuf, qcbuf );
if ( status != E_SUCC )
    c_clean(&vblock,"FILLTOVS","",&zero);


/* Display a standard completion message
----------------------------------------*/
strcpy(out[0], outfile);
strcpy(out[1], qcimg);
c_complt( "FILLTOVS", out, &outcnt );
return E_SUCC;
} 
