/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the input parameters

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      10/99    D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS

ALGORITHM REFERENCES:   NONE

*******************************************************************************/
#include "las.h"
#include "util.h"
#include "filltovs.h"


int getpar
(
struct	PARBLK *vblock,		/* Input parameter block		*/
char	infile[3][CMLEN],	/* input file names			*/
char	*outfile,		/* Output file name			*/
char	*qcimg			/* QC image file name			*/
)
{
long	access;			/* file	access type (read write)	*/
long	element = 0;		/* input parameter element number	*/
long	status;			/* function return status		*/


/* Get the input water vapor image
name and check for its existance.
----------------------------------*/
access = IREAD;
status = c_getnam( vblock, "IN", &element, infile[0], "img", &access );
if ( status != E_SUCC )
    return E_FAIL;


/* Get the output file name and make sure it doesn't exist
----------------------------------------------------------*/
access = IWRITE;
status = c_getnam( vblock, "OUT", &element, outfile, "img", &access );
if (status != E_SUCC)
    return E_FAIL;


/* Get the QC file name and make sure it doesn't exist
------------------------------------------------------*/
status = c_getnam( vblock, "QC", &element, qcimg, "img", &access );
if (status != E_SUCC)
    return E_FAIL;


/* Get the first fill image name
and check for its existance.
--------------------------------*/
access = IREAD;
status = c_getnam( vblock, "FILL", &element, infile[1], "img", &access );
if ( status != E_SUCC )
    return E_FAIL;


/* Get the second fill image name
and check for its existance.
---------------------------------*/
element = 1;
status = c_getnam( vblock, "FILL", &element, infile[2], "img", &access );
if ( status != E_SUCC )
    return E_FAIL;

return E_SUCC;
}
