/*****************************************************************************
FUNCTION:	UPDDR

PURPOSE:	To update the ddr of an output image with the correct
		geographic projection parameters and datum code.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/99	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "filltovs.h"


int upddr
(
char	*outfile		/* host file name of output image	*/
)
{
long	band;			/* band number for getbdr() call	*/
long	status;			/* function return status		*/

struct	DDR	ddr;		/* ddr structure			*/
struct	BDDR	bddr;		/* band ddr structure			*/


/* get the current ddr
----------------------*/
status = c_getddr( outfile, &ddr );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"filltovs-getddr","Error retrieving %s image ddr",outfile);
    return E_FAIL;
    }
band = 1;
status = c_getbdr ( outfile, &bddr, &band );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"filltovs-getbdr","Error retrieving %s band ddr",outfile);
    return E_FAIL;
    }


/* update the projection parameters, coordinates and distances
and validate each field.
--------------------------------------------------------------*/
ddr.proj_code = 0;		/* geographic projection	*/
ddr.valid[0] = 1;		/* validate proj_code		*/
ddr.zone_code = 62;		/* zone is 62			*/
ddr.valid[1] = 1;		/* validate zone_code		*/
ddr.datum_code = 19;		/* datum is sphere		*/
ddr.valid[2] = 1;		/* validate datum_code		*/
/* Note: all projection parameters left as zero			*/
ddr.valid[3] = 1;		/* validate proj_parm		*/
ddr.upleft[0] = 89.5;		/* set up corner coordinates	*/
ddr.upleft[1] = -179.5;		/* these corner coordinates	*/
ddr.loleft[0] = -89.5;		/* set up the entire world	*/
ddr.loleft[1] = -179.5;
ddr.upright[0] = 89.5;
ddr.upright[1] = 179.5;
ddr.loright[0] = -89.5;
ddr.loright[1] = 179.5;
ddr.valid[6] = 1;		/* validate corner coordinates	*/
ddr.pdist_y = ddr.pdist_x = 1.0;/* projection distance is one	*/
ddr.valid[5] = 1;		/* validate pdist		*/
strcpy( ddr.proj_units, "degrees" );
ddr.valid[4] = 1;		/* validate proj_units		*/
ddr.line_inc = ddr.sample_inc = 1.0;
ddr.valid[7] = 1;		/* validate increments		*/


/* write the updated ddr
------------------------*/
status = c_putddr ( outfile, &ddr );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"filltovs-putddr","Error writing the %s image ddr",outfile);
    return E_FAIL;
    }
status = c_putbdr ( outfile, &bddr );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"filltovs-putbdr","Error writing %s band ddr",outfile);
    return E_FAIL;
    }

return E_SUCC;
}
