/***************************************************************************
FUNCTION:	WRITE_OUTPUT

PURPOSE:	To write the output buffers to new image files

Version	Date	Programmer	Modification
-------	-----	------------	--------------------------------
  1.0	10/99	D.Lloyd		Initial Development

PROJECT:	ADAPS


***************************************************************************/
#include "las.h"
#include "util.h"
#include "filltovs.h"



int write_output
(
char	outfile[CMLEN],		/* Output file name			*/
char	qcimg[CMLEN],		/* QC image file name			*/
float	wvbuf[NLINES][NSAMPS],	/* daily observation H2O buffer		*/
float	qcbuf[NLINES][NSAMPS]	/* quality control buffer		*/
)
{
struct FDESC *fdesc;		/* image I/O file descriptor		*/

long	band = 1;		/* image band number			*/
long	dtype = EREAL;		/* image data type			*/
long	nextflg = CREAT;	/* call type: we need to create files	*/
long	nlbuf = NLINES;		/* total number of lines in buffer	*/
long	nsbuf = NSAMPS;		/* total number of samples in buffer	*/
long	sli = 1;		/* starting line of image window	*/
long	ssi = 1;		/* starting sample of image window	*/
long	status;			/* Function return status		*/
long	wtype = SAME;		/* no output image conversion		*/

unsigned char	*buffer;	/* pointer to output buffer		*/



/* Create and write the water vapor values image.
------------------------------------------------*/
buffer = (unsigned char *)wvbuf; /* point to the buffer we want write	*/
status = put_buf( buffer, &nlbuf, &nsbuf, &nlbuf, &nsbuf, &sli, &ssi,
		  &nlbuf, &nsbuf, &nextflg, &band, &band,
		  &fdesc, outfile, &dtype, &wtype );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"filltovs-ptbuf","Error writing to file %s",outfile);
    c_eclose( &fdesc );
    return E_FAIL;
    }

/* close the output image file
------------------------------*/
status = c_eclose( &fdesc );
if ( status != E_SUCC )
    c_lasmsg(FALSE,"filltovs-clos","Error closing file %s",outfile);


/* Create and write the QC image
--------------------------------*/
buffer = (unsigned char *)qcbuf; /* point to the buffer we want write	*/
status = put_buf( buffer, &nlbuf, &nsbuf, &nlbuf, &nsbuf, &sli, &ssi,
		  &nlbuf, &nsbuf, &nextflg, &band, &band,
		  &fdesc, qcimg, &dtype, &wtype );
if ( status != E_SUCC )
    {
    c_lasmsg(FALSE,"filltovs-ptbuf","Error writing to file %s",qcimg);
    c_eclose( &fdesc );
    return E_FAIL;
    }

/* close the output image file
------------------------------*/
status = c_eclose( &fdesc );
if ( status != E_SUCC )
    c_lasmsg(FALSE,"filltovs-clos","Error closing file %s",qcimg);


/* We need to modify the ddr of each output file
------------------------------------------------*/
status = upddr( outfile );
if ( status != E_SUCC )
    return E_FAIL;
status = upddr( qcimg );
if ( status != E_SUCC )
    return E_FAIL;


return E_SUCC;
} 
