/*******************************************************************************
NAME:    FINDFILES

PURPOSE: Find the file names and number of files

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  5.0    D.Hollaren       04/92  Original Development
  5.1    T.Rockvam        05/92  Converted into a general utility to be called
                                 by other procedures
  5.2    T.Baltzer        07/92  Add LOCKFLG logic
  5.3    J.Fenno          11/92  Replaced call to c_complt with z_exit
  5.4    T.Baltzer        06/96  Add OUTFLG parameter
  5.5    C.Engebretson    07/96  Add FIRST parameter
  5.6	 T.Baltzer	  11/96  Make a procedure pdf (findfiles.pdf) as well
				 as a process pdf (findfiles_.pdf) so that the
				 variable OUTFILES can be cleared coming in.
  5.7    D.Etrheim        03/97  Replaced call to c_complt with z_exit as
				 it was taken out from previous release.
				 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under LAS.
   Should be called from a procedure PDF.

PROJECT:  ADAPS
*******************************************************************************/
#include <errno.h>
#include <stdio.h>
#include "las.h"
#include "util.h"


main()
{
struct PARBLK vblock;		/* parameter block			     */

struct VARIABLE *var;		/* structure for file names	      	     */
				
long fatal = -1;		/* fatal error code			     */
long one = 1;			/* value of one			             */
long i;				/* looping variable			     */
long first_call = 1;		/* only one call to c_getfiles		     */
long firstfile = 0;		/* first filename number		     */
long maxfiles;			/* maximum number of files to retrieve	     */
long status;			/* return status of support calls	     */
long stdlun;			/* standard output			     */
long nfiles;			/* number of files found		     */
long ck_lock = 0;		/* 1 = return list of unlocked files only,   */
                                /* 0 = return list of all files found	     */
long ck_out = 1;		/* 1 = return list of files in OUTFILES	     */
				/* 0 = don't return list of files	     */


char hnames[MAXFILES][CMLEN];   /* array for all files in the input list     */
char lockstr[4];		/* lock file specification (yes/no)	     */
char hostspec[CMLEN];		/* host file specification 		     */
char filespec[CMLEN];		/* LAS-format file specification	     */
char *vec[MAXFILES];		/* array of pointers for q_string	     */

/*
  Initialize LAS
*/
c_inlas(&vblock, &stdlun);

/*
  Get the maximum number of files to retrieve
*/
var = p_find(&vblock,"MAXFILES");
maxfiles = IVAL(*var,0);

/*
  Get the first filename number
*/
var = p_find(&vblock, "FIRST");
firstfile = IVAL(*var,0) - 1;

/*
  Get the lock file flag
*/
var = p_find(&vblock,"LOCKFLG");
strcpy(lockstr, SVAL (*var,0));
if ((lockstr[0] == 'Y') || (lockstr[0] == 'y'))
ck_lock = 1;

/*
  Get the output file flag
*/
var = p_find(&vblock,"OUTFLG");
strcpy(lockstr, SVAL (*var,0));
if ((lockstr[0] == 'Y') || (lockstr[0] == 'y'))
    ck_out = 1;
else
    ck_out = 0;

/*
  Get the name of the file to find
*/
var = p_find(&vblock,"INFILE");
strcpy(filespec, SVAL (*var,0));
status = c_cmhost(filespec, hostspec);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","findfiles-fatal",&fatal);

/*
  Get the file names and number of files
  returned value of nfiles will be no more 
  than user-specified maxfiles
*/

if (firstfile != 0)
    { 

    /* 
      If we are skipping over some files (FIRST param was not nulled) call
      c_getfiles an initial time to read past the unwanted entries
    */
    status = c_getfiles(hostspec, hnames, &nfiles, firstfile, first_call,
			ck_lock);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","findfiles-fatal",&fatal);
    first_call = 0;
    }

status = c_getfiles(hostspec, hnames, &nfiles, maxfiles,
                    first_call, ck_lock);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","findfiles-fatal",&fatal);

/*
  Write the found files to the vblock if the user has specified to
*/
if ((nfiles > 0) && (ck_out == 1))
    {
    for (i = 0; i < nfiles; i++)
	vec[i] = hnames[i];

    if (q_string(&vblock,"OUTFILES",nfiles,vec,P_UPDATE) != SUCCESS)
	c_errmsg("Unable to pass values to TAE parameter block",
	    "findfiles-pass",
	     &fatal);
    }

/*
  Write the number of output files to the variable block
*/
if (q_intg(&vblock,"NFILES",1,&nfiles,P_UPDATE) != SUCCESS)
    c_errmsg("Unable to pass values to TAE parameter block", "findfiles-pass",
	 &fatal);


/*
  Update the variable block
*/
status = q_out(&vblock);
if (status != SUCCESS)
    {
    switch(status)
	{
	case EMSGSIZE:
	    c_lasmsg(TRUE, "findfiles-size",
		     "Parameter block too large to send to TAE monitor");
	    break;
	default:
	    c_lasmsg(TRUE, "findfiles-send",
		 "Error sending TAE vblock, error code = %d", status);
	}
    }

/*
  Display a completion message.  Don't use c_complt.
*/
c_errmsg("Program FINDFILES completed successfully","informational",&one);
z_exit(1, "FINDFILES completed successfully");
}
