/*****************************************************************************
FUNCTION:	GETGRID

PURPOSE:	Load up a buffer of sub-sampled values at the specified grid
		points from a LUI

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    04/99	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "getatm.h"
#include "adapsgeo.h"



int getgrid
(

long	luiptr,			/* IN - pointer to LUI			*/
struct	LUIIMAGE *luiimage,	/* IN - LUI image structure		*/
long	numgrids,		/* IN - number of sample grids		*/
float	*latlon[],		/* IN - lat lon buffer			*/
long	dtype,			/* IN - data type, EREAL,EWORD		*/
float	gridbuf[]		/* OUT - grid buffer			*/

)
{
double	lat;			/* latitude for c_geo2ls call		*/
double	lon;			/* longitude for c_geo2ls call		*/

float	floatval;		/* R*4 var for retrieving LUI values	*/

long	i;			/* loop control variable		*/
long	longval;		/* I*4 var for retrieving LUI values	*/
long	luiline;		/* LUI image line number		*/
long	luisamp;		/* LUI image sample			*/
long	status;			/* function return status		*/

short	shortval;		/* I*2 var for retrieving LUI values	*/


/* For each grid point along the scan line
------------------------------------------*/
for ( i = 0; i < (numgrids + 1); i++ )
    {
    /* Get the lat/lon of the grid point
    ------------------------------------*/
    lat = latlon[0][i];
    lon = latlon[1][i];

    if ( lat == -999.0 || lon == -999.0 )
        gridbuf[i] = -1.0;
    else
	{
	/* Convert the lat/lon to a line sample in the LUI
	--------------------------------------------------*/
	c_geo2ls( &luiimage->ullat, &luiimage->ullon, &luiimage->pdistx,
		    &luiimage->pdisty, &lat, &lon, &luiline, &luisamp );
	luiline++;	/* c_geo2ls() returns zero based lines		*/
	luisamp++;	/* c_geo2ls() returns zero based samples	*/
	
	/* Get the value for the corresponding
	line/sample and convert it to a float
	--------------------------------------*/
	switch ( dtype )
	    {
	    case  EWORD :
		status = c_getlui( luiptr, luiline, luisamp,
		    (unsigned char *)&shortval );
		if ( status != E_SUCC )
		    return E_FAIL;
		gridbuf[i] = (float)shortval;
		break;
	    case  ELONG :
		status = c_getlui( luiptr, luiline, luisamp,
		    (unsigned char *)&longval );
		if ( status != E_SUCC )
		    return E_FAIL;
		gridbuf[i] = (float)longval;
		break;
	    case  EREAL :
		status = c_getlui( luiptr, luiline, luisamp,
		    (unsigned char *)&floatval );
		if ( status != E_SUCC )
		    return E_FAIL;
		gridbuf[i] = floatval;
		break;
	    }
	}
    }

/* duplicate the last value for interpolation purposes
------------------------------------------------------*/
i = numgrids;
gridbuf[i+1] = gridbuf[i];

return E_SUCC;
}
