/*****************************************************************************
FUNCTION:	OPENLAS

PURPOSE:	Open the LAS output image.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    03/99	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "getatm.h"



int openlas
(
char	*hosout,		/* IN - host ouptut file name		*/
struct	FRAME	*frame,		/* IN - framing parameters		*/
long	window[],		/* IN - input image window values	*/
long	linegrid,		/* IN - line grid size			*/
long	sampgrid,		/* IN - sample grid size		*/
long	angflg,			/* IN - generate angles bands flag	*/
long	lsbflg,			/* IN - generate the line samples bands	*/
int	fdesc[],		/* OUT - file descriptor		*/
int	*gdesc,			/* OUT - group descriptor		*/
char	**thebuf,		/* OUT - image output buffer		*/
long	offset[],		/* OUT - offset into output buffer	*/
int	compflg			/* IN - image compression flag		*/
)

{
long	access = IWRITE;	/* image I/O access type		*/
long	bufsiz;			/* number of bytes in image I/O buffer	*/
long	ns;			/* number of image samples		*/
long	nl;			/* number of image lines		*/
long	odtype = EREAL;		/* output image data type		*/
long	one = 1;		/* value of one				*/
long	opt[3];			/* optional parameters -- unused	*/
long	status;			/* function return status		*/
long	totbnd;			/* total number of bands in output	*/
long	zero = 0;		/* value of zero			*/


/* Determine the number of bands in the output image
----------------------------------------------------*/
if ( angflg )
    totbnd = 3;
else
    totbnd = 0;
if ( lsbflg )
    totbnd += 2;
if ( strlen( frame->water.hostname ) > 0 )
    totbnd++;
if ( strlen( frame->ozone.hostname ) > 0 )
    totbnd++;
if ( strlen( frame->elev.hostname ) > 0 )
    totbnd++;
if ( strlen( frame->qc.hostname ) > 0 )
    totbnd++;
if ( totbnd == 0 )
    {
    c_lasmsg( FALSE, "getatm-bands", "No output bands specified" );
    return E_FAIL;
    }

/* Assign output lines and samples.  We want an extra grid in
each direction so that there will be values for interpolation
on the right edge and last line of image.
-------------------------------------------------------------*/
frame->ngline = window[NL] / linegrid + 1;
frame->ngsamp = window[NS] / sampgrid + 1;
nl = window[NL];
ns = window[NS];


/* Set the image compression options
------------------------------------*/
if ( compflg == TRUE )
    {
    opt[0] = EREAL;
    opt[1] = COMPRES;
    opt[2] = 0;
    }
else
    opt[0] = opt[1] = opt[2] = 0;


/* Open the output image
------------------------*/
status = c_eopens((struct FDESC **)&fdesc[0], hosout, &totbnd, &one, &one,
                   &nl, &ns, offset, &access, &odtype, &zero, opt );
if ( status != E_SUCC )
    return E_FAIL;
fdesc[1] = 0;


/* Allocate the buffer
----------------------*/
*thebuf = c_ealloc((struct FDESC **)fdesc, &bufsiz );
if ( *thebuf == NULL )
    return E_FAIL;


/* Group the image
------------------*/
status = c_egroup((struct FDESC **)fdesc, (struct GDESC **)gdesc, *(unsigned char **)thebuf,
    &bufsiz );
if ( status != E_SUCC )
    return E_FAIL;

return E_SUCC;
}
