/*****************************************************************************
FUNCTION:	PROCESS

PURPOSE:	Process the image.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    03/99	D.Lloyd		Initial development
  1.1    01/00  D.Lloyd		Corrected an error where qc band was written
  				to when not specified. SRF 3126
  1.2    12/00  D.Lloyd		Modified to use the window samples values so
				that the output image is the same as the
				input image.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "vll.h"
#include "labserv.h"
#include "dltf.h"
#include "getatm.h"
#include "adapsgeo.h"
#include "adapssup.h"

#define	NODEBUG	TRUE		/* set to FALSE for debugging with dbx	*/
extern	long	curent;		/* counter for processing message	*/

void swap ( float *array[] );

int process
(
int	*gdesc,			/* IN - group descriptor		*/
long	offset[],		/* IN - offset into the output buffer	*/
char	*thebuf,		/* IN - output buffer			*/
struct	FRAME	*frame,		/* IN - framing parameters		*/
struct	MODEL	*model,		/* IN - AVHRR model structure		*/
long	angflg,			/* IN - generate angles bands flag	*/
long	linegrid,		/* IN - line gridsize			*/
long	sampgrid,		/* IN - sample gridsize			*/
long	*msgtim,		/* IN - message time interval		*/
long	interpflg,		/* IN - interpolation flag, NN or BL	*/
long	lsbflg,			/* IN - line/sample band flag		*/
struct	DDR	*ddr,		/* IN - image ddr structure		*/
long	window[],		/* IN - input image window		*/
long	scaling,		/* IN - scale flag			*/
char	*hosin			/* IN - input file host name		*/
)

{
double	all_zero[15] = {0.0};	/* zero projection parameters		*/
double	rot_coeffs[6];		/* rotation coefficients		*/
double	ullat = 90.0;		/* upper left latitude for geo image	*/
double	ullon = -180.0;		/* upper left longitude for geo image	*/
double	pdistx = 1.0;		/* geo image x pixel distance		*/
double	pdisty = 1.0;		/* geo image y pixel distance		*/
double	proj_coord[8];		/* corner coords in projection space	*/
double	lat,lon;

long	ang0;			/* index into ptr for solar zenith	*/
long	ang1;			/* index into ptr for satellite zenith	*/
long	ang2;			/* index into ptr for relative azimuth	*/
long	degrees = 4;		/* units code (degrees) for geo proj	*/
long	elev;			/* index into ptr for elevation		*/
long	elevflag = FALSE;	/* flag: create elevation band		*/
long	elevlui = -1;		/* pointer to elevation look up image	*/
long	flag = ALLCAT;		/* all categories			*/
long	geo = 0;		/* geographic projection flag		*/
long	h2o;			/* index into ptr for water vapor	*/
long	i,j,k;			/* loop counters			*/
long	image_coord[8];		/* corner coords in image space		*/
long	l,s;			/* temporary line/sample for c_geo2ls	*/
long	line;			/* index into ptr for line value	*/
long	linenum;		/* image line number			*/
long	msgflag = TRUE;		/* flag for printing error messages	*/
long	nbands;			/* number of bands in output image	*/
long	nextl;			/* line number of next grid		*/
long	o3;			/* index into ptr for ozone		*/
long	ozoneflag = FALSE;	/* flag: create ozone band		*/
long	ozonelui = -1;		/* pointer to ozone look up image	*/
long	prevl;			/* line number of previous/current grid	*/
long	proj_units;		/* projection units value		*/
long	prtprm[2];		/* print flags for c_transinit()	*/
long	qc;			/* index into ptr for qc		*/
long	qcflag = FALSE;		/* flag: create qc band			*/
long	qclui = -1;		/* pointer to the qc look up image	*/
long	samp;			/* index into ptr for sample value	*/
long	*sampindx;		/* sample indices of line grid points	*/
long	sample;			/* image sample				*/
long	scale_opt;		/* scaling option			*/
long	status;			/* function return status		*/
long	numgrids;		/* number of sample grid points		*/
long	utm = 62;		/* UTM projection code			*/
long	waterflag = FALSE;	/* flag: create water vapor band	*/
long	waterlui = -1;		/* pointer to water vapor look up image	*/

float	*elev_buf[2];		/* elevation grid buffer		*/
float	floatval;		/* temporary floating point value	*/
float	*latlon[2];		/* lat long grid buffer			*/
float	*linebuf[2];		/* line grid buffer			*/
float	*ozonebuf[2];		/* ozone grid buffer			*/
float	*ptr[MAXBNDS];		/* pointer to image buffer		*/
float	*qcbuf[2];		/* qc grid buffer			*/
float	*relazbuf[2];		/* relative azimuth grid buffer		*/
float	*sampbuf[2];		/* sample grid buffer			*/
float	*satzebuf[2];		/* sat zenith angle grid buffer		*/
float	*solzebuf[2];		/* solar zenith angle grid buffer	*/
float	*waterbuf[2];		/* water vapor grid buffer		*/

struct	SCOFF	scoff[N_ANGLES];/* scaling coefficient structure	*/

/* The following variables were added for CR 4761 they are for reading from
 * the minor frame label services file.
------------------------------------------------------------------------*/
long    minorframe;		/* minor frame file descriptor		*/
char    linekey[KEYL];		/* line number key for minor frame file */
long    one = 1;		/* Constant                             */
long    minor_nbytes = MNR_WORDS*sizeof(short); /* num bytes in min frm */
char    char_buf[1];		/* buffer for label services calls      */
short   min_frm[MNR_WORDS];	/* minor frame buffer                   */
char    dtype[DTYPL];		/* Data type for label services calls   */
int     band3is_thermal = FALSE;/* Flag TRUE/FALSE: KLM band 3 thermal	*/
int     minorframe_open = FALSE;/* Flag TRUE/FALSE: file is open	*/

/* For KLM series satellites, those that have two sensors for channel three,
 * open the label services minor frame file.
 ---------------------------------------------------------------------------*/
if ( model != NULL )
    {
    if ( model->satnum >= 15 )
	{
	char hosmnr[CMLEN];	/* label services file name		*/
	long access = 0;	/* access mode to open minor frame file	*/

	c_lsmknm( hosin, "mnr", hosmnr );
	status = c_lsopen( &minorframe, hosmnr, &access );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"lsopen","Unable to open minor frame");
	    c_lasmsg(FALSE,"qcband",
	        "Channel 3 information will not be updated");
	    minorframe_open = FALSE;
	    }
	else
	    minorframe_open = TRUE;
	}
    }


/* Load the scaling structure if needed
---------------------------------------*/
if ( scaling == US_SCALE || scaling == GLOBAL_SCALE )
    {
    if ( scaling == US_SCALE )
        scale_opt = US_REAL;
    else
        scale_opt = AVHRR_REAL;
    
    status = c_getscale( scale_opt, "satzenith", &scoff[SATZEN].minval,
        &scoff[SATZEN].maxval, &scoff[SATZEN].scale, &scoff[SATZEN].unscale,
	&scoff[SATZEN].offset );
    status |= c_getscale( scale_opt, "solzenith", &scoff[SOLZEN].minval,
        &scoff[SOLZEN].maxval, &scoff[SOLZEN].scale, &scoff[SOLZEN].unscale,
	&scoff[SOLZEN].offset );
    status |= c_getscale( scale_opt, "relazimuth", &scoff[RELAZ].minval,
        &scoff[RELAZ].maxval, &scoff[RELAZ].scale, &scoff[RELAZ].unscale,
	&scoff[RELAZ].offset );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"getatm-scale","Error getting scale factors");
	return E_FAIL;
	}
    }

/* Allocate the grid buffers.  These buffers are allocated only
once so we will depend on the system to free the memory on exit.
----------------------------------------------------------------*/
numgrids = window[NS] / sampgrid;
j = numgrids + 2;
for ( i = 0; i < 2; i++ )
    {
    if (( waterbuf[i] = (float *)malloc( j * sizeof(float))) == NULL )
	{
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
        return E_FAIL;
	}
    if (( ozonebuf[i] = (float *)malloc( j * sizeof(float))) == NULL )
	{
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
        return E_FAIL;
	}
    if (( elev_buf[i] = (float *)malloc( j * sizeof(float))) == NULL )
	{
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
        return E_FAIL;
	}
    if (( solzebuf[i] = (float *)malloc( j * sizeof(float))) == NULL )
	{
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
        return E_FAIL;
	}
    if (( satzebuf[i] = (float *)malloc( j * sizeof(float))) == NULL )
	{
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
        return E_FAIL;
	}
    if (( relazbuf[i] = (float *)malloc( j * sizeof(float))) == NULL )
	{
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
        return E_FAIL;
	}
    if (( latlon[i] = (float *)malloc( j * sizeof(float))) == NULL )
	{
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
        return E_FAIL;
	}
    if (( qcbuf[i] = (float *)malloc( j * sizeof(float))) == NULL )
	{
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
        return E_FAIL;
	}
    if (( linebuf[i] = (float *)malloc( j * sizeof(float))) == NULL )
        {
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
	return E_FAIL;
	}
    if (( sampbuf[i] = (float *)malloc( j * sizeof(float))) == NULL )
        {
	c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
	return E_FAIL;
	}
    }

/* Allocate and initialize the grid sampling index buffer.
This buffer contains the indices of the grid points along
the scan line. This buffer is allocated once so we will
depend on the system to free the memory on exit.
----------------------------------------------------------*/
if (( sampindx = (long *)malloc( (j+1) * sizeof(long))) == NULL )
    {
    c_lasmsg( FALSE, "getatm-alloc", "Error allocating memory" );
    return E_FAIL;
    }

/* Calculate the indices of the grid points here and store them
in the buffer.  They are calculated here so that they don't have
to be constantly recalculated later.
----------------------------------------------------------------*/
for ( i = 0; i < j-1; i++ )
    sampindx[i] = i * sampgrid + window[SS];
sampindx[j-1] = sampindx[j] = window[NS] + window[SS] - 1;

/* If we have a DDR initialize the projection transformation
------------------------------------------------------------*/
if ( ddr )
    {
    if ( strcasecmp( ddr->proj_units, "radians" ) == 0 )
        proj_units = 0;
    else if ( strcasecmp( ddr->proj_units, "feet" ) == 0 )
        proj_units = 1;
    else if ( strcasecmp( ddr->proj_units, "meters" ) == 0 )
        proj_units = 2;
    else if ( strcasecmp( ddr->proj_units, "seconds of arc" ) == 0 )
        proj_units = 3;
    else if ( strcasecmp( ddr->proj_units, "degrees of arc" ) == 0 )
        proj_units = 4;
    else if ( strcasecmp( ddr->proj_units, "packed DMS" ) == 0 )
        proj_units = 5;
    else
        {
	c_lasmsg(FALSE,"getatm-projunit","Incorrect projection units in DDR");
	return E_FAIL;
	}

    prtprm[0] = prtprm[1] = FALSE;
    if ( c_transinit( &ddr->proj_code, &proj_units, &ddr->zone_code,
             &ddr->datum_code, ddr->proj_coef, &geo, &degrees, &utm,
	     &ddr->datum_code, all_zero, prtprm, "\0" ) != E_SUCC )
	{
	c_lasmsg(FALSE,"getatm-transinit",
	    "Error initializing projection transformation");
	return E_FAIL;
	}

    /* Set up rotation coefficients in case the image
    has been rotated.  In most cases we are using square
    unrotated images but to be safe set this up anyway.
    ----------------------------------------------------*/
    proj_coord[0] = ddr->upleft[0];
    proj_coord[1] = ddr->upleft[1];
    proj_coord[2] = ddr->loleft[0];
    proj_coord[3] = ddr->loleft[1];
    proj_coord[4] = ddr->upright[0];
    proj_coord[5] = ddr->upright[1];
    proj_coord[6] = ddr->loright[0];
    proj_coord[7] = ddr->loright[1];
    image_coord[0] = 1;            /* ul y */
    image_coord[1] = 1;            /* ul x */
    image_coord[2] = ddr->nl;      /* ll y */
    image_coord[3] = 1;            /* ll x */
    image_coord[4] = 1;            /* ur y */
    image_coord[5] = ddr->ns;      /* ur x */
    image_coord[6] = ddr->nl;      /* lr y */
    image_coord[7] = ddr->ns;      /* lr x */
    status = c_rtcoef( proj_coord, image_coord, rot_coeffs, &msgflag );
    if ( status != E_SUCC )
        {
	c_lasmsg(FALSE,"getatm-rtcoef","Error initializing rotation coord");
	return E_FAIL;
	}
    }

/* Initialize the lat/lon buffers and angles grid (if specified)
with line 1 so that when the main processing loop starts we can
get the next grid line -- thereafter we will always have 2 lines
of grid data to work with.  Note: the angles bands won't be
calculated if the angflg = FALSE.
----------------------------------------------------------------*/
loadll( 1, sampgrid, angflg, model, ddr, proj_units, window, scaling,
	scoff, latlon[0], latlon[1], satzebuf[1], solzebuf[1], relazbuf[1],
	rot_coeffs );

/* Count the bands, open the LUI's and initialize the grids.
The first time we want to initialize the grid with line 1
so that ... ( same as above comment ).
If a hostname was specified, set the process flag and an
index into the ptr array.  Next load the image as a Look Up
Image (LUI) and initialize the line 1 grid.
------------------------------------------------------------*/
nbands = 0;
/* Check for a water vapor image
--------------------------------*/
if ( frame->water.hostname[0] != 0 )
    {
    waterflag = TRUE;
    h2o = nbands++;
    i = 1;
    status = c_loadlui( frame->water.hostname, frame->water.window,
    			EREAL, &waterlui, &i );
    if ( status != E_SUCC )
        return E_FAIL;
    status = getgrid( waterlui, &frame->water, numgrids, latlon,
    			EREAL, waterbuf[1] );
    if ( status != E_SUCC )
	{
	closelui( elevlui, ozonelui, qclui, waterlui );
        return E_FAIL;
	}
    }
/* Check for an elevation image
-------------------------------*/
if ( frame->elev.hostname[0] != 0 )
    {
    elevflag = TRUE;
    elev = nbands++;
    i = 1;
    status = c_loadlui( frame->elev.hostname, frame->elev.window,
    			EREAL, &elevlui, &i );
    if ( status != E_SUCC )
	{
	closelui( elevlui, ozonelui, qclui, waterlui );
        return E_FAIL;
	}
    status = getgrid( elevlui, &frame->elev, numgrids, latlon,
    			EREAL, elev_buf[1] );
    if ( status != E_SUCC )
	{
	closelui( elevlui, ozonelui, qclui, waterlui );
        return E_FAIL;
	}
    }
/* Check for an ozone image
---------------------------*/
if ( frame->ozone.hostname[0] != 0 )
    {
    ozoneflag = TRUE;
    o3 = nbands++;
    i = 1;
    status = c_loadlui( frame->ozone.hostname, frame->ozone.window,
    			EREAL, &ozonelui, &i );
    if ( status != E_SUCC )
	{
	closelui( elevlui, ozonelui, qclui, waterlui );
        return E_FAIL;
	}
    status = getgrid( ozonelui, &frame->ozone, numgrids, latlon,
    			EREAL, ozonebuf[1] );
    if ( status != E_SUCC )
	{
	closelui( elevlui, ozonelui, qclui, waterlui );
        return E_FAIL;
	}
    }
/* Check for a qc image
-----------------------*/
if ( frame->qc.hostname[0] != 0 )
    {
    qcflag = TRUE;
    qc = nbands++;
    i = 1;
    status = c_loadlui( frame->qc.hostname, frame->qc.window,
			EREAL, &qclui, &i );
    if ( status != E_SUCC )
	{
	closelui( elevlui, ozonelui, qclui, waterlui );
	return E_FAIL;
	}
    status = getgrid( qclui, &frame->qc, numgrids, latlon,
		      EREAL, qcbuf[1] );
    if ( status != E_SUCC )
	{
	closelui( elevlui, ozonelui, qclui, waterlui );
	return E_FAIL;
	}
    }

/* The angles grids have already been initialized by the loadll() call
so all we have to do here is set up the indices into the ptr array.
----------------------------------------------------------------------*/
if ( angflg )
    {
    ang0 = nbands++;
    ang1 = nbands++;
    ang2 = nbands++;
    }
 

/* Initialize the line sample grids and set up indices into the ptr array
-------------------------------------------------------------------------*/
if ( lsbflg )
    {
    line = nbands++;
    samp = nbands++;
    for ( k = 0; k <= window[NS] / sampgrid; k++ )
        {
	lat = latlon[0][k];
	lon = latlon[1][k];
	if ( lat < -360.0 || lon < -360.0 )
	    {
	    linebuf[1][k] = -1.0;
	    sampbuf[1][k] = -1.0;
	    }
	else
	    {
	    c_geo2ls( &ullat, &ullon, &pdistx, &pdisty, &lat, &lon, &l, &s );
	    linebuf[1][k] = (float)( l + 1 );
	    sampbuf[1][k] = (float)( s + 1 );
	    }
	}
    linebuf[1][k] = linebuf[1][k-1];
    sampbuf[1][k] = sampbuf[1][k-1];
    }

/* Do the first estep and initialize the ptr array into thebuf
--------------------------------------------------------------*/
status = c_estep((struct GDESC **)gdesc );
if ( status != E_SUCC )
    {
    closelui( elevlui, ozonelui, qclui, waterlui );
    return E_FAIL;
    }
for ( i = 0; i < nbands; i++ )
    ptr[i] = (float *)(thebuf + offset[i] * sizeof(float));


/* Start the processing message
-------------------------------*/
curent = 1;
#if NODEBUG
c_inittm("GETATM","wrote","line","bands"," ", &window[NL], &nbands, 
	msgtim, &flag );
c_startm();
#endif


/* Main processing loop
-----------------------*/
nextl = 1;
for ( linenum = 1; linenum <= window[NL]; linenum++ )
    {
    /* Get this lines minor frame
    -----------------------------*/
    if ( minorframe_open )
        {
	/* build the line number for the lsread call.  Note that the minor
	 * frame stores the original line number so we don't have to bother
	 * with modifying the line number for ascending vs. descending.
	 * ----------------------------------------------------------------*/
	sprintf( linekey, "%d", linenum + (int)window[SL] - 1 );
	status = c_lsread( &minorframe, linekey, &one,
	   &minor_nbytes, char_buf, (unsigned char *)min_frm, dtype );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"lsread","Error reading minor frame");
	    return E_FAIL;
	    }

	/* Check minor frame word six bit 1.  The bit is
	cleared (0) for band 3B and set (1) for band 3A.
	------------------------------------------------*/
	if (( min_frm[6] &1 ) == 0 )
	    band3is_thermal = TRUE;
	else
	    band3is_thermal = FALSE;
	}

    /* Decide if we need to get another line of grid values,
    We need another line when the current line no longer falls
    between the two line grids we already have.
    ----------------------------------------------------------*/
    if (( linenum - 1 ) % linegrid == 0 )
        {
	prevl = nextl;
	nextl = linenum + linegrid;
	if ( nextl > window[NL] )
	    nextl = window[NL];

	/* swap the angles grid buffers if specified */
	if ( angflg )
	    {
	    swap( satzebuf );
	    swap( solzebuf );
	    swap( relazbuf );
	    }

	/* load up the latlon and angles buffers for the next line */
	loadll( nextl, sampgrid, angflg, model, ddr, proj_units, window,
        	scaling, scoff, latlon[0], latlon[1], satzebuf[1],
		solzebuf[1], relazbuf[1], rot_coeffs );

	/* now load each of the other grids we need */
	/* load up the line sample buffers */
	if ( lsbflg )
	{
	swap( linebuf );
	swap( sampbuf );
	for ( k = 0; k <= window[NS] / sampgrid; k++ )
	    {
	    lat = latlon[0][k];
	    lon = latlon[1][k];
	    if ( lat == -999.0 || lon == -999.0 )
	        {
		linebuf[1][k] = -1.0;
		sampbuf[1][k] = -1.0;
		}
	    else
		{
		c_geo2ls(&ullat, &ullon, &pdistx, &pdisty, &lat, &lon, &l, &s);
		linebuf[1][k] = (float)( l + 1 );
		sampbuf[1][k] = (float)( s + 1 );
		}
	    }
	linebuf[1][k] = linebuf[1][k-1];
	sampbuf[1][k] = sampbuf[1][k-1];
	}

	/* Load the water grid */
	if ( waterflag )
	    {
	    swap( waterbuf );
	    status = getgrid( waterlui, &frame->water, numgrids,
	    			latlon, EREAL, waterbuf[1] );
	    if ( status != E_SUCC )
		{
		closelui( elevlui, ozonelui, qclui, waterlui );
		return E_FAIL;
		}
	    }

	/* Load the ozone grid */
	if ( ozoneflag )
	    {
	    swap( ozonebuf );
	    status = getgrid( ozonelui, &frame->ozone, numgrids,
	    			latlon, EREAL, ozonebuf[1] );
	    if ( status != E_SUCC )
		{
		closelui( elevlui, ozonelui, qclui, waterlui );
		return E_FAIL;
		}
	    }

	/* Load the elevation grid */
	if ( elevflag )
	    {
	    swap( elev_buf );
	    status = getgrid( elevlui, &frame->elev, numgrids,
	    			latlon, EREAL, elev_buf[1] );
	    if ( status != E_SUCC )
		{
		closelui( elevlui, ozonelui, qclui, waterlui );
		return E_FAIL;
		}
	    }

	/* Load the qc grid */
	if ( qcflag )
	    {
	    swap( qcbuf );
	    status = getgrid( qclui, &frame->qc, numgrids,
	    			latlon, EREAL, qcbuf[1] );
	    if ( status != E_SUCC )
		{
		closelui( elevlui, ozonelui, qclui, waterlui );
		return E_FAIL;
		}
	    }
	}

    /* Fill in the output buffer with interpolated
    values from each of the grids.
    ----------------------------------------------*/
    i = -1;
    for ( sample = 1; sample <= window[NS]; sample++ )
        {
	if (( sample - 1 ) % sampgrid == 0 )
	    i++;
	/* Interpolate the water grid buffer */
	if ( waterflag )
	    {
	    /* nearest valid neighbor
	    -------------------------*/
	    ptr[h2o][sample-1] = nvn( linenum, prevl, nextl,
	            sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    waterbuf[0][i], waterbuf[0][i+1],
		    waterbuf[1][i], waterbuf[1][i+1] );
	    }
	
	/* Interpolate the ozone grid buffer */
	if ( ozoneflag )
	    {
	    /* nearest valid neighbor
	    -------------------------*/
	    ptr[o3][sample-1] = nvn( linenum, prevl, nextl,
	            sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    ozonebuf[0][i], ozonebuf[0][i+1],
		    ozonebuf[1][i], ozonebuf[1][i+1] );
	    }

	/* Interpolate the elevation grid buffer */
	if ( elevflag )
	    {
	    /* nearest valid neighbor
	    -------------------------*/
	    ptr[elev][sample-1] = nvn( linenum, prevl, nextl,
	            sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    elev_buf[0][i], elev_buf[0][i+1],
		    elev_buf[1][i], elev_buf[1][i+1] );
	    }

	/* Interpolate the angles grid buffers */
	if ( angflg )
	    {
	    /* Interpolate the satellite zenith angle grid buffer */
	    /* nearest neighbor
	    -------------------*/
	    if ( interpflg == NN )
		floatval = nn( linenum, prevl, nextl, sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    satzebuf[0][i], satzebuf[0][i+1],
		    satzebuf[1][i], satzebuf[1][i+1], &floatval );
	    /* bi-linear
	    ------------*/
	    else
		interp( linenum, prevl, nextl, sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    satzebuf[0][i], satzebuf[0][i+1],
		    satzebuf[1][i], satzebuf[1][i+1], &floatval );
	    ptr[ang0][sample-1] = floatval;
	    /* Interpolate the solar zenith angle grid buffer */
	    /* nearest neighbor
	    -------------------*/
	    if ( interpflg == NN )
		floatval = nn( linenum, prevl, nextl, sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    solzebuf[0][i], solzebuf[0][i+1],
		    solzebuf[1][i], solzebuf[1][i+1], &floatval );
	    /* bi-linear
	    ------------*/
	    else
		interp( linenum, prevl, nextl, sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    solzebuf[0][i], solzebuf[0][i+1],
		    solzebuf[1][i], solzebuf[1][i+1], &floatval );
	    ptr[ang1][sample-1] = floatval;
	    /* Interpolate the relative azimuth grid buffer.  It has been
	    determined, through experimentation, that when grid points lie
	    on either side of zero a nearest neighbor interpolation should
	    be performed.  Check for this phenomena and force NN when found.
	    ---------------------------------------------------------------*/
	    /* nearest neighbor
	    -------------------*/
	    if ( interpflg == NN || 
	       (( relazbuf[0][i] > 0.0 || relazbuf[0][i+1] > 0.0 ||
	          relazbuf[1][i] > 0.0 || relazbuf[1][i+1] > 0.0 ) &&
	        ( relazbuf[0][i] < 0.0 || relazbuf[0][i+1] < 0.0 ||
	          relazbuf[1][i] < 0.0 || relazbuf[1][i+1] < 0.0 )))
		{
		floatval = nn( linenum, prevl, nextl, sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    relazbuf[0][i], relazbuf[0][i+1],
		    relazbuf[1][i], relazbuf[1][i+1], &floatval );
		}
	    /* bi-linear
	    ------------*/
	    else
		interp( linenum, prevl, nextl, sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    relazbuf[0][i], relazbuf[0][i+1],
		    relazbuf[1][i], relazbuf[1][i+1], &floatval );
	    ptr[ang2][sample-1] = floatval;
	    }
	
	/* Interpolate the qc grid buffer */
	if ( qcflag )
	    {
	    ptr[qc][sample-1] = nvn( linenum, prevl, nextl,
	            sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    qcbuf[0][i], qcbuf[0][i+1],
		    qcbuf[1][i], qcbuf[1][i+1] );

	    /* Flag solar zenith angles > SZANGLE degrees
	    in the QC band by adding the scale offset SZSCALE.
	    -----------------------------------------------------*/
	    if ( angflg )
		if ( ptr[ang1][sample-1] > SZANGLE ||
		     ptr[ang1][sample-1] < -SZANGLE )
		ptr[qc][sample-1] += SZSCALE;

	    /* If we're dealing with a KLM series satellite,
	    modify the qc band to indicate which channel is
	    present by adding the values to the qc value.
	    -----------------------------------------------*/
	    if ( minorframe_open )
		{
	        if ( band3is_thermal )
		    ptr[qc][sample-1] += QCBAND3B;
		else
		    ptr[qc][sample-1] += QCBAND3A;
		}
	    }

	/* Interpolate the line grid buffer */
	if ( lsbflg )
	    {
	    ptr[line][sample-1] = nvn( linenum, prevl, nextl,
	            sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    linebuf[0][i], linebuf[0][i+1],
		    linebuf[1][i], linebuf[1][i+1] );
	    }

	/* Interpolate the samp grid buffer */
	if ( lsbflg )
	    {
	    ptr[samp][sample-1] = nvn( linenum, prevl, nextl,
	            sample + window[SS] - 1,
		    sampindx[i], sampindx[i+1],
		    sampbuf[0][i], sampbuf[0][i+1],
		    sampbuf[1][i], sampbuf[1][i+1] );
	    }
	}

    /* Write the line and readjust the pointers into thebuf
    -------------------------------------------------------*/
    status = c_estep((struct GDESC **)gdesc );
    if ( status != E_SUCC )
	{
	closelui( elevlui, ozonelui, qclui, waterlui );
        return E_FAIL;
	}
    for ( i = 0; i < nbands; i++ )
        ptr[i] = (float *)(thebuf + offset[i] * sizeof(float));
    curent++;
    }

/* Stop the processing message
------------------------------*/
c_stoptm();

/* Close the LUI's
------------------*/
closelui( elevlui, ozonelui, qclui, waterlui );

return E_SUCC;
}

/* Local function to swap the rows
of one of the a 2D grid arrays.
----------------------------------*/
void swap(  float *array[] )
{
float	*tmp;

tmp = array[0];
array[0] = array[1];
array[1] = tmp;

}
