/*****************************************************************************
FUNCTION:	UPDATE_DDR

PURPOSE:	Update the output image DDR with source information for
		each band.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    04/99	D.Lloyd		Initial development
  1.1    12/00  D.Lloyd		Set the band ddr direction member to NULL

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "getatm.h"



int update_ddr
(
char	*hosname,		/* host name of output file		*/
struct	DDR	*input_ddr,	/* input image DDR or NULL		*/
struct	FRAME	*frame,		/* output framing parameters		*/
long	angflg,			/* generate angles bands flag		*/
long	lsflg,			/* generate line/sample flag		*/
long	scaling			/* angles scaling flag			*/
)

{
struct	DDR	ddr;		/* band independent data descriptor rec	*/
struct	BDDR	bddr;		/* band dependent ddr's			*/

long	band;			/* DDR band number			*/
long	i;			/* loop control variable		*/
long	nbands;			/* number of bands in output image	*/
long	status;			/* function return status		*/

char	ext[CMLEN];		/* extention on image name		*/
char	image[CMLEN];		/* image name				*/
char	root[CMLEN];		/* root of image name			*/
char	source[MAXBNDS][CMLEN];	/* source for each band			*/


/* Get the root name of the output image
----------------------------------------*/
c_splitname( hosname, image, root, ext );
strcat( image, root );

/* Get the image DDR
--------------------*/
status = c_getddr( image, &ddr );
if ( status != E_SUCC )
    return E_FAIL;
nbands = ddr.nbands;

/* If the input DDR is not NULL, copy its projection
information to the output DDR.
----------------------------------------------------*/
if ( input_ddr )
    {
    ddr.master_line = input_ddr->master_line;
    ddr.master_sample = input_ddr->master_sample;
    /* copy valid flags */
    for ( i = 0; i < 8; i++ )
        ddr.valid[i] = input_ddr->valid[i];
    /* copy projection and datum */
    ddr.proj_code = input_ddr->proj_code;
    ddr.zone_code = input_ddr->zone_code;
    ddr.datum_code = input_ddr->datum_code;
    strcpy( ddr.proj_units, input_ddr->proj_units );
    /* copy the projection coefficients */
    for ( i = 0; i < 15; i++ )
        ddr.proj_coef[i] = input_ddr->proj_coef[i];
    /* copy corner coords */
    ddr.upleft[0] = input_ddr->upleft[0];
    ddr.upleft[1] = input_ddr->upleft[1];
    ddr.loleft[0] = input_ddr->loleft[0];
    ddr.loleft[1] = input_ddr->loleft[1];
    ddr.upright[0] = input_ddr->upright[0];
    ddr.upright[1] = input_ddr->upright[1];
    ddr.loright[0] = input_ddr->loright[0];
    ddr.loright[1] = input_ddr->loright[1];
    /* copy the projection distance and increment */
    ddr.pdist_y = input_ddr->pdist_y;
    ddr.pdist_x = input_ddr->pdist_x;
    ddr.line_inc = input_ddr->line_inc;
    ddr.sample_inc = input_ddr->sample_inc;
    }

/* Build an array of source names
---------------------------------*/
i = 0;
if ( frame->water.hostname[0] != 0 )
    strcpy( source[i++], "Water Vapor" );
if ( frame->elev.hostname[0] != 0 )
    strcpy( source[i++], "Elevation" );
if ( frame->ozone.hostname[0] != 0 )
    strcpy( source[i++], "Ozone" );
if ( frame->qc.hostname[0] != 0 )
    strcpy( source[i++], "Quality Control" );
if ( angflg )
    {
    strcpy( source[i++], "Satellite Zenith" );
    strcpy( source[i++], "Solar Zenith" );
    strcpy( source[i++], "Relative Azimuth" );
    if ( scaling == US_SCALE )
        {
	strcat( source[i-3], " US_SCALE" );
	strcat( source[i-2], " US_SCALE" );
	strcat( source[i-1], " US_SCALE" );
	}
    else if ( scaling == GLOBAL_SCALE )
        {
	strcat( source[i-3], " GLOBAL_SCALE" );
	strcat( source[i-2], " GLOBAL_SCALE" );
	strcat( source[i-1], " GLOBAL_SCALE" );
	}
    }
if ( lsflg )
    {
    strcpy( source[i++], "Geo Line" );
    strcpy( source[i++], "Geo Sample" );
    }

/* Create a clean DDR
---------------------*/
status = c_putddr( image, &ddr );
if ( status != E_SUCC )
    return E_FAIL;

/* Update the source for each band
----------------------------------*/
bddr.minval = bddr.maxval = 0.0;
bddr.valid = 0;
for ( i = 0; i < nbands; i++ )
    {
    band = i+1;
    bddr.bandno = band;
    strcpy( bddr.source, source[i] );
    bddr.instrument[0] = '\0';
    bddr.date[0] = '\0';
    bddr.time[0] = '\0';
    bddr.direction[0] = '\0';
    status = c_putbdr( image, &bddr );
    if ( status != E_SUCC )
	c_lasmsg(FALSE,"getatm-bddr",
		    "Error updating DDR for band %d", band);
    }


return E_SUCC;
}
