/******************************************************************************
FUNCTION:	GETCALTYPE


PURPOSE:	Determine the calibration technique historically used for a
		given scene.

PROGRAM HISTORY:
  Version	Date       Author      Request
  -------	----	   ------      -------
    1.0		06/01	   T. Baltzer  Original development

ALGORITHM:
******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "getcaltype.h"


int main(void)
{
long	 std_out;		/* Standard output			*/

struct PARBLK vblock;		/* TAE parameter block                	*/

char	acqdate[7];		/* Acquisition date MMDDYY		*/
char	*array[1];		/* Return parameter value		*/
char	caltype[20];		/* Calibration technique (vermote,etc)	*/
char	satnum[3];		/* Satellite number (16 for NOAA16)	*/
char	sceneid[17];		/* Scene identifier			*/

long	zero = 0;		/* Constant zero			*/

struct VARIABLE	*val;		/* pointer to a TAE variable 	      	*/

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);

/*
  Retrieve the scene identifier
*/
val = p_find(&vblock,"SCENEID");
strcpy(sceneid,SVAL(*val,0));


/*
  Determine the satellite number and acquisition date
*/
strncpy(satnum,&sceneid[2],2);
satnum[2] = '\0';

strncpy(acqdate,&sceneid[4],6);
acqdate[6] = '\0';


/*
  Make the call to determine the calibration technique
*/
if ( gettechnique(satnum, acqdate, caltype) != E_SUCC )
    c_lasmsg(TRUE,"getcaltype-fatal","Fatal error encountered");


/*
  Return the calibration technique, if known.  Otherwise, return an error.
*/
if (caltype)
    {
    /*
      Return the calibration technique to the parameter block
    */
    array[0] = caltype;
    if (q_string(&vblock,"CALTYPE",1,array,P_UPDATE) != SUCCESS)
	c_lasmsg(TRUE,"getcaltype-tae","Error returning parameter to TAE");

    /*
      Send the variable block to TAE
    */
    if (q_out(&vblock) != SUCCESS)
	c_lasmsg(TRUE,"getcaltype-tae","Error returning parameter to TAE");
    }
else
    {
    c_lasmsg(TRUE,"getcaltype-nova","Calibration type unavailable");
    }

/*
  Display a completion message
*/
c_complt("GETCALTYPE","",&zero);


/*
  Put return here to make the compiler happy
*/
return(E_SUCC);
}

