/******************************************************************************
FUNCTION:	GETTECHNIQUE


PURPOSE:	Determine the calibration technique historically used.

PROGRAM HISTORY:
  Version	Date       Author      Request
  -------	----	   ------      -------
    1.0		06/01	   T. Baltzer  Original development

ALGORITHM:
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "getcaltype.h"

#define LINE_SIZ	80	/* Maximum size of line in the table */


int gettechnique
    (
    char satnum[],		/* Satellite number */
    char acqdate[],		/* Acquisition date */
    char caltype[]		/* Calibration type */
    )

{

char	disk[CMLEN];		/* Calibration technique file name	*/
char	enddate[9];		/* Ending date in table			*/
char	oneline[LINE_SIZ];	/* One line of data in the table	*/
char	startdate[9];		/* Starting date in table		*/

FILE	*fp;			/* File pointer to the table		*/


/*
  Assemble the name of the table to read
*/
strcpy(disk,getenv("ADAPSTABLES"));
if (disk == NULL)
    {
    c_lasmsg(FALSE,"getcaltype-tab","Error retrieving value of ADAPSTABLES");
    return(E_FAIL);
    }
sprintf(disk,"%s/noaa%s.caltype", disk, satnum);


/*
  Open the table
*/
fp = fopen(disk,"r");
if (fp == NULL)
    {
    c_lasmsg(FALSE,"getcaltype-open","Error opening the calibration table");
    return(E_FAIL);
    }


/*
  Read past the header records
*/
if (fgets(oneline,LINE_SIZ,fp) == NULL)
    {
    c_lasmsg(FALSE,"getcaltype-read","Error reading the header information");
    return(E_FAIL);
    }
if (fgets(oneline,LINE_SIZ,fp) == NULL)
    {
    c_lasmsg(FALSE,"getcaltype-read","Error reading the header information");
    return(E_FAIL);
    }

/*
  Get data records until the applicable one is found or until EOF
*/
for ( ; ; )
    {
    if (fgets(oneline,LINE_SIZ,fp) == NULL)
	{
	c_lasmsg(FALSE,"getcaltype-calt",
		 "Error determining calibration technique");
	return(E_FAIL);
	}
    sscanf(oneline,"%s %s %s",startdate, enddate, caltype);

    if (lies_between(acqdate, startdate, enddate) == E_SUCC) break;
    }


return(E_SUCC);
}

