/****************************************************************************** FUNCTION:	LIES_BETWEEN


PURPOSE:	Determine if a date lies within a certain range.

PROGRAM HISTORY:
  Version	Date       Author      Request
  -------	----	   ------      -------
    1.0		06/01	   T. Baltzer  Original development

ALGORITHM:
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"



int lies_between
    (
    char acqdate[], 		/* acquisition date MMDDYY	*/
    char startdate[],		/* start date	    MMDDYYYY	*/
    char enddate[]		/* ending date	    MMDDYYYY	*/
    )

{
int acq_DD;	/* Acquisition day	*/
int acq_MM;	/* Acquisition month	*/
long acq_YYYY;	/* Acquisition year	*/
int end_DD;	/* Ending day		*/
int end_MM;	/* Ending month		*/
long end_YYYY;	/* Ending year		*/
int str_DD;	/* Starting day		*/
int str_MM;	/* Starting month	*/
long str_YYYY;	/* Starting year	*/


/* Check to see if the acquisition date is greater than or equal to the
   start date */

sscanf(startdate,"%02d%02d%04ld", &str_MM, &str_DD, &str_YYYY);	/* check year */
sscanf(acqdate,"%02d%02d%02ld", &acq_MM, &acq_DD, &acq_YYYY);
acq_YYYY = c_4digityr(acq_YYYY);
if (acq_YYYY < str_YYYY)
    return(E_FAIL);		/* fail: acquistion year prior to start year */
else if (acq_YYYY == str_YYYY)	/* On same year, check month */
    {
    if (acq_MM < str_MM)
	return(E_FAIL);		  /* fail: acq month prior to start month */
    else if (acq_MM == str_MM)	  /* On same month, check day  */
	{
	if (acq_DD < str_DD)
	    return(E_FAIL);	  /* fail: acq day prior to start day */
	}
    }

/* To get to here, the acquisition date must be greater than or equal to
   the start date!  Now determine if the acquisition date is less than or
   equal to the end date...
*/

sscanf(enddate,"%02d%02d%04ld", &end_MM, &end_DD, &end_YYYY);	/* check year */
if (acq_YYYY > end_YYYY)
    return(E_FAIL);		/* fail: acq year greater than end year */
else if (acq_YYYY == end_YYYY)	/* On same year, check month */
    {
    if (acq_MM > end_MM)
	return(E_FAIL);		    /* fail: acq month greater than end month */
    else if (acq_MM == end_MM)   /* On same month, check day */
	{
	if (acq_DD > end_DD)
	    return(E_FAIL);	/* fail: acq day greater than end day */
	}
    }

/* If all the above checks were survived, the acquisition date falls within
   the start date and end date.
*/

return(E_SUCC);
}

