/******************************************************************************
FUNCTION:	GETDRIVEINFO

PURPOSE:	Get the tape device name, data type, and density given the
		drive name and return the values back to the TAE
		parameter block.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       12/93    T. Baltzer		initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include "las.h"		/* LAS include			       */
#include "tapeio.h"		/* Tape library			       */

main()
{
struct PARBLK   vblock;		/* TAE parameter block			*/

char drive_density[CMLEN];	/* Tape drive density			*/
char drive_name[CMLEN];		/* Tape drive name			*/
char *sptr[1];                  /* pointer array for updating vblock    */

long std_out;			/* standard output device 		*/
long fatal = -1;		/* Fatal error status			*/
long nonfatal = 1;		/* Non fatal error status		*/

struct TAPEDESC tape_drive;	/* Tape description structure		*/

struct VARIABLE *p_find();      /* function to find a TAE variable      */
struct VARIABLE *var;           /* pointer to a TAE variable            */

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);


/* Get the drive name
------------------------*/
var = p_find(&vblock,"DRIVE");
strcpy(drive_name,SVAL(*var,0));


/* Call the LAS support tprdconf to read the
   configuration file.  Send 0 as the density.
---------------------------------------------*/
strcpy(drive_density, "0");
if(tprdconf(&tape_drive, drive_name, drive_density, TREAD) != TAPE_SUCC)
   {
   c_errmsg("Error reading tape configuration file","getdriveinfo-frd",&nonfatal);
   c_errmsg("Fatal error encountered","getdriveinfo-fatal",&fatal);
   }


/*
  Output the values to the variable block
*/
sptr[0] = tape_drive.name;
q_string(&vblock,"DEVICE",1,sptr,P_UPDATE);
sptr[0] = tape_drive.dtype;
q_string(&vblock,"DTYPE",1,sptr,P_UPDATE);
sptr[0] = tape_drive.density;
q_string(&vblock,"DENSITY",1,sptr,P_UPDATE);
sptr[0] = tape_drive.thost;
q_string(&vblock,"THOST",1,sptr,P_UPDATE);
q_out(&vblock);


/*
  Exit with a success status without displaying a completion message
*/
z_exit(1,"");
}
