/*******************************************************************************
FUNCTION:	getpar

PURPOSE:	Retrieve the parameters from tae parameter block	
			 
Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    01/94  T. Baltzer	 Initial development
				

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "archio.h"


int getpar
(
    struct PARBLK *vblock,	/* Pointer to TAE parameter block             */
    char hosin[][CMLEN],	/* Host file name of input image              */
    struct MODEL  *model,       /* AVHRR model structure                      */
    long *nlines		/* Number of lines to skip between evaluations*/
)
{

long access 	= IREAD;	/* Access flag				      */
long element	=  0;		/* Parameter element number                   */
long nonfatal	= 1;		/* Nonfatal error status		      */
long status;			/* Function return status                     */

struct DROPPED  drop;           /* dropped line data structure                */
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block   */


/* Retrieve archive image name
-----------------------------------*/
status = c_getnam(vblock, "IN", &element, (char *)hosin, "ahdr", &access);
if (status != E_SUCC)
    {
    c_errmsg("Fatal error encountered","getregions-fatal",&nonfatal);
    return(E_FAIL);
    }


/* Get the AVHRR header information
-----------------------------------*/
status = c_getahdr((char *)hosin, model, &drop);
if (status != E_SUCC)
    {
    c_errmsg("Fatal error encountered","getregions-fatal",&nonfatal);
    return(E_FAIL);
    }


/* Retrieve number of lines to process between
---------------------------------------------------*/
val = p_find(vblock,"NLINES");
*nlines = IVAL(*val,0);


return(E_SUCC);
}
