/*******************************************************************************
FUNCTION:	check_date

PURPOSE:	Check the validity of the julian date of the current
		line compared to the previous julian date.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Conversion to LAS5.0
  2.1    04/93  J.Fenno          Modified calls to the new archio supports

ALGORITHM REFERENCE:
  See "NOAA Technical Memorandum NESS 107" for a detailed description
  of the HRPT Minor Frame Format.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "ingest.h"
#include "worgen.h"
#include "avhrr.h"

int check_date
(
long	*jday,		/* Julian date of current line		     */
long	*jold,		/* Julian date of previous line		     */
long	*milsec,	/* Time in milliseconds of current line	     */
double	*milold		/* Time in milliseconds of previous line     */
)
{
/*
  Check to see if the julian date crossed over to a new day or is just bad
*/
if (*jday != *jold) 
    {
    if ((*jday == *jold + 1) || ((*jold > (MAX_JDAY - 2)) && (*jday == 1)))
	{
	/*
	  Check to determine if the time stamp was over midnight
	*/
	if ((*milsec > 0) && (*milsec < MIL_PER_SEC) &&
	    (*milold > MIL_PER_DAY - MIL_PER_SEC) && (*milold < MIL_PER_DAY))
	    {
	    /*
	      Subtract a days worth of milliseconds
	    */
	    *milold -= MIL_PER_DAY;
	    }
	else
	    *jday = *jold;
	}
    else
	{
	*jday = *jold;
	/*
	return(BAD_DAY);
	*/
	}
    }

return(E_SUCC);
}
