/*******************************************************************************
FUNCTION:	fill_missing

PURPOSE:	Fill any missing lines with zero filled lines, and save
		the lines number of the missing lines in the dropped line
		structure.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    05/91  D.Hollaren       Conversion to LAS5.0
  2.1    04/93  J.Fenno          Modified calls to the new archio supports

ALGORITHM REFERENCE:
  See "NOAA Technical Memorandum NESS 107" for a detailed description
  of the HRPT Minor Frame Format.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "ingest.h"
#include "archio.h"

int fill_missing
(
long	*arc_fd,		/* File descriptor to the archive file       */
struct	DROPPED	*drop,		/* Dropped line index			     */
long	*missing,		/* Number of missing lines in a gap	     */
short	*zerobuf		/* Fill for missing lines		     */
)
{
extern long	curent;		/* Current line being processed		     */

long	nonfatal = 1,		/* Non-fatal error code			     */
	status,			/* Return status variable		     */
	i;			/* Looping variable			     */

char	msg[ERRLEN];		/* Error messages			     */

/*
  Return if no missing lines were found
*/
if (*missing <= 0)
    return(E_SUCC);

/*
  ****************** Temporary for evaluation *******************
  Check the missing line limit
if (*missing > MISSING_LIMIT)
    {
    sprintf(msg,"%d lines missed at line %d exceeds the limit %d",
			*missing,curent,MAX_MISSING);
    c_errmsg(msg,"ingest-missed",&nonfatal);
    return(E_FAIL);
    }
  ****************** Temporary for evaluation *******************
*/

/*
  Save the location and number of dropped lines for the
  dropped line index file
*/
drop->lmissed[drop->ngaps] = curent;
drop->nmissed[drop->ngaps] = *missing;

/*
  Count the number of missing line gaps.  If the number of gaps
  is over the limit the image may not be worth processing.
*/
drop->ngaps++;
if (drop->ngaps > MAX_GAPS)
    {
    sprintf(msg,"%s %d %s %d",
	    "The number of dropped line gaps exceeds the limit of",
            MAX_GAPS,"at line",curent);
    c_errmsg(msg,"ingest-gaps",&nonfatal);
    return(E_FAIL);
    }

/*
  Fill the missing gap with zero fill records
*/
for (i = 0; i < *missing; i++)
    {
    status = c_arcwrite(arc_fd,zerobuf,zerobuf);
    if (status != E_SUCC)
        return(E_FAIL);

    curent++;
    }

return(E_SUCC);
}
