/*******************************************************************************
FUNCTION:	find_satid

PURPOSE:	Find the most commonly occuring value, given an array
		filled with values.  In the case of a "tie", the first
		occurring element will be returned.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/96  T.Baltzer        Inital development of THIS routine


ALGORITHM REFERENCE:

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "ingest.h"
#include "worgen.h"
#include "avhrr.h"

int find_satid
(
long satidx_buf[MAX_BAD],	/* Buffer containing the last MAX_BAD sat ids*/
long *satidx			/* Acquisition satellite id		     */
)
{
long already_in_buf;		/* Flag indicating if value is already in buf*/
long i, j;			/* Counter				     */
long most_occurances;		/* Value of occurances in the most occuring  */
long num_elem = 0;		/* Number of elements entered into sat_value */
long sat_value[MAX_BAD];	/* Buffer containing the actual sat id value */
long sat_occur[MAX_BAD];	/* Buffer containing the number of occurances*/


/*
  Process thru the entire input buffer, populating the local buffers
*/
for (i = 0; i < MAX_BAD; i++)
    {

    /* Try to find the value in the buffers.  If it is already there, increment
       the occurrance.
    */
    already_in_buf = FALSE;
    for (j = 0; j < num_elem; j++)
	{
	if (sat_value[j] == satidx_buf[i])
	    {
	    sat_occur[j]++;
	    already_in_buf = TRUE;
	    break;
	    }
	}

    /*
       If the value wasn't already in the buffers, enter it into the buffers.
    */
    if (already_in_buf == FALSE)
	{
	sat_value[num_elem] = satidx_buf[i];
	sat_occur[num_elem] = 1;
	num_elem++;
	}

    }

/*
   Now the local buffers have been populated with the actual values and the
   number of occurrances thereof.  Now read thru the local buffers and
   determine the value of the most frequently occurring element.
*/
most_occurances = sat_occur[0];
*satidx = sat_value[0];

for (i = 0; i < num_elem; i++)
    {
    if (sat_occur[i] > most_occurances)
	{
	*satidx = sat_value[i];
	most_occurances = sat_occur[i];
	}
    }

return(E_SUCC);
}
