/*******************************************************************************
FUNCTION:	getpar

PURPOSE:	Get the user specified parameters.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Conversion to LAS5.0
  2.1    04/93  J.Fenno          Modified calls to the new archio supports
  2.2    09/94  J.Fenno          Added code to check the return status from the
				 c_getnam() calls.


PROJECT:	ADAPS
*******************************************************************************/
#include <string.h>
#include "ingest.h"
#include "labserv.h"
#include "util.h"
#include "avhrr.h"

int getpar
(
struct PARBLK	*vblock,	/* TAE parameter block			    */
char		*hosin,		/* Input host file name			    */
char		*hosout,	/* Output host file name		    */
char		*tmpfil,	/* Temporary created host file name	    */
char		*tmptip,	/* Optional tips file name		    */
long		*genfil,	/* Flag to generate an output file name	    */
long		*year,		/* Year the data was acquired		    */
char		*station,	/* Station ID the data was collected at	    */
long		*dtype,		/* Type of data to process (GAC,LAC,HRPT)   */
long		*msgtim		/* Time between processing messages	    */
)
{
struct VARIABLE *val;		/* Pointer to TAE variable with vblock	    */

int	pid;			/* Process ID for a unique temporary name   */

long	element = 0,		/* Element number for getnam call	    */
	access,			/* Type of access wanted for getname call   */
	fatal   = -1,		/* Fatal error flag                         */
	len;			/* String length of station		    */

char	taeout[CMLEN],		/* Output tae file name			    */
	taetmp[CMLEN];		/* Temporary TAE file name		    */

char	indir[CMLEN],
	inroot[CMLEN],
	inext[CMLEN];
char	outdir[CMLEN],
	outroot[CMLEN],
	outext[CMLEN];

/*
  Retrieve the input acquired image
*/
access = IREAD;
if (c_getnam(vblock,"INFILE",&element,hosin,"acq",&access) != E_SUCC)
    c_errmsg("Fatal error encountered", "ingest-fatal", &fatal);

/*
  Retrieve the output ingest name
*/
val = p_find(vblock,"OUTFILE");
strcpy(hosout,SVAL(*val,0));
c_splitname( hosout, outdir, outroot, outext );
if ( strlen( outext ) == 0 )
    strcpy( outext, ".arch" );

/* If the user specified a filename use c_getnam to make sure it doesn't
exist.  Can't call this if the user only specified a directory
------------------------------------------------------------------------*/
if ( strlen( outroot ) != 0 )
    {
    access = IWRITE;
    if (c_getnam(vblock,"OUTFILE",&element,hosout,"arch",&access) != E_SUCC)
	c_errmsg("Fatal error encountered", "ingest-fatal", &fatal);
    }

/*
  Assume the user put in a directory and file name
*/
*genfil = FALSE;

if (strlen(hosout) > 0)
    {
    /*
      Build a unique temporary output file name for processing
    */
    strcpy(taeout, hosout);


    /*
      Check to see if the user specified a file name, otherwise set a
      flag so that a output name will be generated for the user.
      And strip off the extension.
    */
    if ( strlen( outroot ) == 0 )
	{
	strcpy(taeout,outdir);
	*genfil = TRUE;
	}

    /*
      Create a temporary tae file name
    */
    pid = getpid();
    sprintf(taetmp,"%sA%d.tmp",outdir,pid);
    }
else
    {
    /*
      Create a temporary tae file name
    */
    pid = getpid();
    sprintf(taetmp,"A%d.tmp",pid);

    /*
      Set a flag to generate an output name for the user
    */
    *genfil = TRUE;
    }

/*
  Get a host file name for the temp file
*/
c_cmhost(taetmp,tmpfil);

/*
  Get the year the data was acquired
*/
val = p_find(vblock,"YEAR");
sscanf(SVAL(*val,0),"%2d",year);

/*
  Get the station that the acquire file is from
*/
val = p_find(vblock,"CEOSID");
strcpy(station,SVAL(*val,0));
len = (long)strlen(station);
c_low2up(station,&len);

/*
  Switch the station code for china because they stitch all
  there passes together
*/
if ((strcmp(station,"UQC") == 0) || (strcmp(station,"GZC") == 0))
    strcpy(station,"BJG");

/*
  Get the type of data to process
*/
val = p_find(vblock,"ACQTYPE");
if (val->v_count == 0)
    {
    c_splitname(hosin,indir,inroot,inext);
    if ((inroot[1] == 'G') || (inroot[1] == 'g'))
	*dtype = GAC;
    else if ((inroot[1] == 'L') || (inroot[1] == 'l'))
	*dtype = LAC;
    else
	*dtype = HRPT;
    }
else if (strcmp(SVAL(*val,0),"GAC") == 0)
    *dtype = GAC;
else if (strcmp(SVAL(*val,0),"LAC") == 0)
    *dtype = LAC;
else
    *dtype = HRPT;

/*
  Determine if the user wants to save the TIP data
*/
val = p_find(vblock,"TIPOPT");
if (strcmp(SVAL(*val,0),"YES") == 0)
    c_lsmknm(tmpfil,"tip",tmptip);
else
    strcpy(tmptip,"");

/*
  Get the time between processing messages
*/
val = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*val,0);

return E_SUCC;
}
