/******************************************************************************
FUNCTION:	CK_CHANS

PURPOSE:	Check the channels to be read.  User input as well as data
		acquired from the input file header records are considered.

Version	Date	Programmer	Modifications
-------	-----	---------------	----------------------------------------------
  1.0  	01/92	T.Baltzer	Development
  1.1  	05/93	J.Fenno		Modified to use the new AVHRR model and the
				new archio supports.
  2.0	11/97	D.Lloyd		Eliminated section that builds channels when
				TBM not present (taken care of in c_l1bopn())
				and modified to return status.  Part of major
				INGEST1B rewrite.

PROJECT:	ADAPS
******************************************************************************/
#include "ingest1b.h"


int ck_chans
(
struct	MODEL *model,	/* IN: AVHRR model				*/
long	inchan[],	/* IN: User input satellite channels to process	*/
long	*nchans,	/* IN: number of channels in inchan		*/
long	seqchan[]	/* OUT: array of sequential band numbers	*/
)
{
char	msg[CMLEN];	/* message string				*/
long	i;		/* counter					*/
long	j;		/* counter					*/
long	seq_chan_num = 0;	/* sequential channel number in input	*/


/*
  If the band numbers to process were not specified by the user, we now
  know how many bands are in the input file, and will default to process all
  of them.
---------------------------------------------------------------------------*/
if (*nchans == 0)
   {
   for (i = 0; i < MOSTBND; i++)
      {
      inchan[i] = model->chans[i];
      if ( inchan[i] )
         (*nchans)++;
      }
   }


/*
  Check to see if the user specified channels exist in the input file
  according to the TBM header.
---------------------------------------------------------------------*/
for (i = 0; i < MOSTBND; i++)
   {
   if ( inchan[i] )		/* user wants this channel		*/
      {
      if ( ! model->chans[i] )	/* wanted channel not in input file	*/
	 {
	 sprintf( msg, "Specified channel %ld not in input file", inchan[i] );
         c_lasmsg(TRUE,"ingest1b-cnot",msg);
	 }
      }
   }


/*
  Determine which sequential bands in input file are to be processed.  The
  assumption is that all bands in input file are in ascending order (If
  the TBM header says channels 1, 3, 4 exist in the input file, then the
  first band is channel 1, the second band contains channel 3, the
  third band contains channel 4).
--------------------------------------------------------------------------*/
for (i = 0; i < MOSTBND; i++)
   seqchan[i] = 0;

for (i = 0, j = 0; i < MOSTBND; i++)
   {
   if (model->chans[i])	/* this channel is in the input file */
      {
      seq_chan_num++;
      if (inchan[i])
         seqchan[j++] = seq_chan_num;	/* specify sequential position	*/
      }
   }

return E_SUCC;

}
