/*****************************************************************************
FUNCTION:	CLOSE_OUTPUT

PURPOSE:	Close the output archive and associated file and rename.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    11/97	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <string.h>
#include "labserv.h"
#include "ingest1b.h"
#include "adapssup.h"



int close_output
(
long    *out_fd,		/* IN: File descriptor to output image	*/
char    *hosout,		/* IN: Output host image names		*/
char    tmpfil[],		/* IN: Temporary output file name	*/
char    tmpass[],		/* IN: Temp associated output file name	*/
long    genfil,                 /* IN: generate output file name flag	*/
char	out[][CMLEN]		/* OUT: output file names created	*/
)
{
char    assout[CMLEN];          /* Output associated file		*/
char	msg[CMLEN];		/* LAS message string			*/
char    sceneid[CMLEN];         /* Scene id				*/

long	mode = 0;		/* test for file presence		*/
long	status;			/* function return status		*/

struct	DROPPED	drop;		/* dropped line structure		*/
struct	MODEL	model;		/* AVHRR model structure		*/


/*
  Close the output image files
------------------------------*/
c_arclose( tmpfil, out_fd );


/*
  Get the model from the newly created archive
----------------------------------------------*/
c_getahdr( tmpfil, &model, &drop );


/*
  Generate a file name using the model information when no output file
  name was specified by the user.
----------------------------------------------------------------------*/
if ( genfil )
    {
    c_getscene( &model, sceneid );
    strcat( sceneid, ".arch" );
    strcat( hosout, sceneid );
    }


/*
  Check for presence of existing files with the file name prefix
  just created.  If it did the archive image will retain the
  temporary file name.
----------------------------------------------------------------*/
status = c_lsstat( hosout, &mode );
if ( status == E_SUCC )
    {
    sprintf( msg, "File %s exists, keeping temporary name", hosout );
    c_lasmsg(FALSE," ",msg);
    strcpy(out[0], tmpfil);
    strcpy(out[1], tmpass);
    return E_SUCC;
    }


/*
  Rename the output file to the sceneid or the user specified name
------------------------------------------------------------------*/
if ( rename( tmpfil, hosout ) != E_SUCC )
    {
    sprintf( msg, "Error renaming output file %s", tmpfil );
    c_lasmsg(FALSE,"ingest1b-renm",msg);
    }
strcpy(out[0], hosout);
    


/*
  Rename the associated file
----------------------------*/
c_lsmknm( hosout, "ahdr", assout );
if ( rename( tmpass, assout ) != E_SUCC )
    {
    sprintf( msg, "Error renaming output file %s", tmpass );
    c_lasmsg(FALSE,"ingest1b-renm",msg);
    }
strcpy(out[1], assout);


return E_SUCC;

}
