/*****************************************************************************
FUNCTION:	DET_SIZE

PURPOSE:	Determine an approximate size in lines of an archive image
		based on a Level 1b AVHRR model and user specified window.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    11/97	D.Lloyd		Initial development -- stolen from old
				INGEST1B main.c

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "ingest1b.h"



void det_size
(
struct	MODEL	*model,		/* IN: AVHRR model structure		*/
long		window[4],	/* IN: input window to process		*/
long		*numlines	/* OUT: approximate number of lines	*/
)
{
float           endsec;         /* ending time in seconds               */
float           strsec;         /* starting time in seconds             */


/*
  Start by determining the total possible number of lines in the output
  image.  Reduce this number depending on the values of SL and NL.
  Since SL and NL do not consider drop lines, numlines can only be considered
  an approximation.
-----------------------------------------------------------------------------*/
strsec = (((( model->shour * 60 ) + model->smin ) * 60 ) + model->ssec ) +
         ( model->sfrac / 1000.0 );
endsec = (((( model->ehour * 60 ) + model->emin ) * 60 ) + model->esec ) +
         ( model->efrac / 1000.0 );

if ( endsec < strsec ) endsec += 86400;  /* Spans a day */

if ( model->dtype == GAC )
    *numlines = (long)(( endsec - strsec ) * GAC_PER_SEC);
else
    *numlines = (long)(( endsec - strsec ) * LAC_PER_SEC);

*numlines = *numlines - window[SL] + 1;

if ( window[NL] != -1 )
    if ( *numlines > window[NL] )
	*numlines = window[NL];

}
