/******************************************************************************
FUNCTION:	GET_OUT

PURPOSE:	Get the user specified output file name parameters.


Version	Date	Programmer	Modifications
-------	-----	---------------	----------------------------------------------
  1.0	02/91	D.Hollaren	Conversion to LAS 5.0
  1.1	01/92	T.Baltzer	Stolen from MIARCH & modified
  1.2	05/93	J.Fenno		Modified to use the new AVHRR model and the
				new archio supports.
  2.0	11/97	D.Lloyd		Modified as part of major INGEST1B rewrite
				Eliminated vms swap code.  Added return values.

PROJECT:	ADAPS
******************************************************************************/
#include "util.h"
#include "labserv.h"
#include "ingest1b.h"



int get_out
(
struct PARBLK	*vblock,	/* IN: TAE parameter block		*/
char		*hosout,	/* OUT: Output host file name		*/
char		*tmpass,	/* OUT: Temporary associated file name	*/
long		*genfil,	/* OUT: Flag for output file generation	*/
char		*tmpfil		/* OUT: temporary output file name	*/
)
{
char	hosdir[CMLEN];		/* Temporary TAE file name		*/
char	hosext[CMLEN];		/* Temporary TAE file name		*/
char	hosroot[CMLEN];		/* Temporary TAE file name		*/

long	element = 0;		/* Element number for getnam call	*/
long	pid;			/* Process ID for a unique temp name	*/
long	status;			/* Return status			*/
long	waccess = IWRITE;	/* File access flag			*/


/*
  Retrieve the output ingest name
---------------------------------*/
status = c_getnam( vblock, "OUT", &element, hosout, "arch", &waccess );
if ( status != E_SUCC )
    return E_FAIL;

/*
  Assume the user put in a directory and file name
--------------------------------------------------*/
*genfil = FALSE;

if ( strlen( hosout ) > 0 )
    {
    /*
      Get the directory & root name from the user's host name
    ---------------------------------------------------------*/
    c_splitname( hosout, hosdir, hosroot, hosext );

    /*
      Check to see if the user specified a file name.  If not, set a
      flag so that a output name will be generated for the user.
    ----------------------------------------------------------------*/
    if ( strlen( hosroot ) == 0 )
	{
	*genfil = TRUE;
	strcpy( hosout, hosdir );
	}

    /*
      Create a temporary file name
    ------------------------------*/
    pid = getpid();
    sprintf( tmpfil, "%sa%ld.arch", hosdir, pid );
    }
else
    {
    /*
      Create a temporary file name
    ------------------------------*/
    pid = getpid();
    sprintf( tmpfil, "a%ld.arch", pid );

    /*
      Set a flag to generate an output name for the user
    ----------------------------------------------------*/
    *genfil = TRUE;
    }

/*
  Generate a temporary associated file name from the temp file and
  later rename if needed
------------------------------------------------------------------*/
c_lsmknm( tmpfil, "ahdr", tmpass );

return E_SUCC;

}
