/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the input parameters


Version	Date	Programmer	Modifications
-------	-----	---------------	----------------------------------------------
  1.0	01/92	T.Baltzer	Development
  1.1	05/93	J.Fenno		Modified to use the new AVHRR model and the 
				new
				archio supports.
  2.0	11/97	D.Lloyd		Modified to allow disk input and eliminated
				globals.  Part of major INGEST1B rewrite.

PROJECT:	ADAPS
******************************************************************************/
#include "util.h"
#include "ingest1b.h"



int getpar
(
struct	PARBLK *vblock,	/* IN: Variable block				*/
long	*skip,		/* OUT: File number on tape to process		*/
char	*hosout,	/* OUT: Host output name			*/
char	*tmpass,	/* OUT: Temporary Output associated file name	*/
long	*genfil,	/* OUT: Flag to generate an output file name	*/
long	*nchans,	/* OUT: Number of bands specified in inchan	*/
long	inchan[],	/* OUT: User input satellite channels to process*/
long	*msgtim,	/* OUT: Time between processing messages	*/
char	*tmpfil,	/* OUT: temporary output file name		*/
char	*infile,	/* OUT: input file name				*/
struct	FDESC	*in_fd	/* OUT: input file descriptor			*/
)
{
long	access = IREAD;	/* disk access flag				*/
long	bands[MOSTBND];	/* array of band numbers to process		*/
long	element = 0;	/* element number for getnam call		*/
long	i; 		/* loop index variable				*/
int     status;

struct	VARIABLE *val;		/* pointer to a TAE variable 	        */


/*
  Get the tape mount comment and drive number if specified
----------------------------------------------------------*/
val = p_find( vblock, "COMMENT" );
if ( val->v_count != 0 )
    {
    for( i = 0; i < val->v_count; i++ )
	strcpy( in_fd->comment[i], SVAL( *val, i ));
    in_fd->ftype = L1BTAPE;

    val = p_find( vblock, "SKIP" );
    if ( val->v_count != 0 )
	*skip = IVAL( *val, 0 );

    val = p_find( vblock, "DRIVE" );
    if ( val->v_count != 0 )
	strcpy( in_fd->drive, SVAL( *val, 0 ));
    else
	c_lasmsg(TRUE,"ingest1b-parm","Must enter DRIVE and COMMENT for tape");
    }


/*
  Or get the input file name
----------------------------*/
else
    {
    val = p_find( vblock, "INFILE" );
    if ( val->v_count != 0 )
	{
	strcpy( infile, SVAL( *val, 0 ));
	in_fd->ftype = L1BDISK;
	access = IREAD;
	if (c_getnam(vblock,"INFILE",&element,infile,"l1b",&access) != E_SUCC)
	    c_lasmsg(TRUE,"ingest1b-inf","Input file doesn't exist");
	}
    else
	c_lasmsg(TRUE,"ingest1b-parm","Must enter INFILE or COMMENT/DRIVE");
    }


/*
  Get output file name or temporary anme
----------------------------------------*/
status = get_out( vblock, hosout, tmpass, genfil, tmpfil );
if ( status != E_SUCC )
    return E_FAIL;



/*
  Retrieve channels to be processed.  If no channels are specified, all
  channels will be the default.  It is not known at this time which
  channels are in the input file, so assignments will not be made here.
-----------------------------------------------------------------------*/
val = p_find( vblock, "CHANS" );
*nchans = val->v_count;

for ( i = 0; i < MOSTBND; i++ )		/* initialize to zero		*/
   {
   bands[i] = 0;
   inchan[i] = FALSE;
   }
if ( *nchans > 0 )			/* channels were user specified	*/
   {
   for ( i = 0; i < *nchans; i++ )	/* Retrieve the bands specified	*/
      {
      bands[i] = IVAL( *val, i );
      if ( inchan[bands[i]-1] )
         {
	 c_lasmsg(FALSE,"ingest1b-dupl","Duplicate channel specified");
         ( *nchans )--;
	 }
      else
         inchan[bands[i]-1] = bands[i];
      }
   }



/*
  Retrieve number of seconds between processing messages
--------------------------------------------------------*/
val = p_find( vblock, "$msgtim" );
*msgtim = IVAL( *val, 0 );

return E_SUCC;

}
