#ifndef INGEST1B_H
#define INGEST1B_H

/* Include all of the required system include files
---------------------------------------------------*/
#include <stdio.h>
#include <string.h>


/* Include all of the required LAS and ADAPS include files
----------------------------------------------------------*/
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "tapeio.h"
#include "level1bio.h"


/* Define all application specific values
-----------------------------------------*/
#define MAX_VOL       3    /* Max number of tapes to read		*/
#define TOOLONG     100    /* Wait too long before processing, give msg	*/
#define L1BDISK     600    /* File type is Level 1b disk		*/
#define L1BTAPE     601    /* File type is Level 1b tape		*/
#define L1BREAD     610    /* File open for read			*/
#define L1BWRT      620    /* File open for write			*/


/*
  File descriptor definition
----------------------------*/
struct FDESC {
    long	ftype;		/* file type: L1BDISK or L1BTAPE	*/
    FILE	*fd;		/* disk file descriptor			*/
    char	drive[CMLEN];	/* tape drive name			*/
    long	tun;		/* tape logical unit number		*/
    long	numtapes;	/* number of tapes listed in comment	*/
    char	comment[MAX_VOL+1][CMLEN];	/* tape mount comments	*/
    long	volnum;		/* current volume number		*/
    long	tape;		/* flag indicating tape open/closed	*/
    long	openfor;	/* flag: L1BREAD or L1BWRT		*/
} fdesc;

/* Function prototypes
----------------------*/
int ck_chans
(
struct  MODEL *model,   /* IN: AVHRR model                              */
long    inchan[],       /* IN: User input satellite channels to process */
long    *nchans,        /* IN: number of channels in inchan             */
long    seqchan[]       /* OUT: array of sequential band numbers        */
);

int close_output
(
long    *out_fd,                /* IN: File descriptor to output image  */
char    *hosout,                /* IN: Output host image names          */
char    tmpfil[],               /* IN: Temporary output file name       */
char    tmpass[],               /* IN: Temp associated output file name */
long    genfil,                 /* IN: generate output file name flag   */
char    out[][CMLEN]            /* OUT: output file names created       */
);

void det_size
(
struct  MODEL   *model,         /* IN: AVHRR model structure            */
long            window[4],      /* IN: input window to process          */
long            *numlines       /* OUT: approximate number of lines     */
);

int get_out
(
struct PARBLK   *vblock,        /* IN: TAE parameter block              */
char            *hosout,        /* OUT: Output host file name           */
char            *tmpass,        /* OUT: Temporary associated file name  */
long            *genfil,        /* OUT: Flag for output file generation */
char            *tmpfil         /* OUT: temporary output file name      */
);

int getpar
(
struct  PARBLK *vblock, /* IN: Variable block                           */
long    *skip,          /* OUT: File number on tape to process          */
char    *hosout,        /* OUT: Host output name                        */
char    *tmpass,        /* OUT: Temporary Output associated file name   */
long    *genfil,        /* OUT: Flag to generate an output file name    */
long    *nchans,        /* OUT: Number of bands specified in inchan     */
long    inchan[],       /* OUT: User input satellite channels to process*/
long    *msgtim,        /* OUT: Time between processing messages        */
char    *tmpfil,        /* OUT: temporary output file name              */
char    *infile,        /* OUT: input file name                         */
struct  FDESC   *in_fd  /* OUT: input file descriptor                   */
);

int open_input
(
char            *infile,        /* IN: input file name                  */
long            window[],       /* IN: input window to process          */
struct FDESC    *in_fd,         /* IN/OUT: input file descriptor        */
struct MODEL    *model,         /* OUT: AVHRR model structure           */
long            *recsiz,        /* OUT: Level 1b record size            */
long            *goodlines      /* OUT: number lines recorded in DSH    */
);

int process
(
struct FDESC    *in_fd,         /* IN: input file descriptor            */
long            *out_fd,        /* IN: output file descriptor           */
char            tmpfil[],       /* IN: temp file name for archive file  */
long            window[],       /* IN: window to process                */
long            inchan[],       /* IN: input channels to process        */
long            seqchan[],      /* IN: sequential band numbers to proc. */
struct MODEL    *model,         /* IN: AVHRR model structure            */
long            *recsiz,        /* IN: input record size                */
long            *goodlines,     /* IN: number of lines in input file    */
long            *msgtim         /* IN: time between processing messages */
);

int put_drop    
(
long    *out_fd,                /* IN: File descriptor to output image  */
struct  MODEL *model,           /* IN: AVHRR model information          */
short   *outbuf                 /* OUT: Output buffer                   */
);

int put_line
(
long            *out_fd,        /* IN: File descriptor to output image  */
unsigned char   *bscan,         /* IN: 2 byte input words (from input)  */
short           inbuf[],        /* IN: Input buffer                     */
long            totchan,        /* IN: total number of bands in input   */
long            inchan[],       /* IN: boolean array of input bands     */
long            seqchan[],      /* IN: sequential band numbers to get   */
struct  MODEL   *model,         /* IN: AVHRR model information          */
short           *outbuf         /* OUT: Output buffer                   */
);

int read_file
(
struct FDESC    *in_fd,         /* IN: input file descriptor            */
long            *out_fd,        /* IN: output file descriptor           */
long            window[],       /* IN: Input window                     */
long            inchan[],       /* IN: Flags for input bands            */
long            seqchan[],      /* IN: sequential band numbers to proc. */
long            totchan,        /* IN: total number bands in the input  */
long            goodlines,      /* IN: total number good lines in input */
long            *rec_size,      /* IN: Record size of data read         */
unsigned char   *bscan,         /* IN: Pointer to Level 1b scan line    */
short           *outbuf,        /* IN: Output buffer                    */
struct MODEL    *model,         /* IN: AVHRR model structure            */
struct DROPPED  *drop           /* OUT: dropped line structure          */
);

int ret_val
(
struct  PARBLK  *vblock,        /* IN: pointer to variable block        */
long            f_status        /* IN: function status value to return  */
);

int unpk_break
(
unsigned char *bscan,   /* IN: 2 byte input words                       */
short ibuf[],           /* OUT: Series of 16 bit unpacked words         */
long *nwords            /* IN: Number of 16-bit input words to unpack   */
);

int zero_fill
(
long    *out_fd,        /* IN: output file descriptor                   */
struct  MODEL *model,   /* IN: AVHRR model information                  */
struct  DROPPED *drop,  /* IN: Dropped line information                 */
long    num_0_lines,    /* IN: Number of dropped lines to zero fill     */
short   *outbuf         /* IN/OUT: Output buffer                        */
);
#endif
