/*****************************************************************************
FUNCTION:	OPEN_INPUT

PURPOSE:	Open a Level 1b tape or disk and calculate the approximate
		number of lines in the image.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    11/97	D.Lloyd		Initial development
  1.1    12/98	D.Lloyd		Modifications for NOAA KLM series sats.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "ingest1b.h"
#include "level1bio.h"



int open_input
(
char		*infile,	/* IN: input file name			*/
long		window[],	/* IN: input window to process		*/
struct FDESC	*in_fd,		/* IN/OUT: input file descriptor	*/
struct MODEL	*model,		/* OUT: AVHRR model structure		*/
long		*recsiz,	/* OUT: Level 1b record size		*/
long		*goodlines	/* OUT: number lines recorded in DSH	*/
)
{
long		status;		/* function return status		*/

unsigned char	buf[ARS_HDR_SIZ];	/* TBM/ARS header record buffer	*/
unsigned char	dsh[MAXRECSIZ];	/* DSH record buffer			*/


/* 
  Open the file either tape or disk
-----------------------------------*/
if ( in_fd->ftype == L1BDISK )
    {
    status = c_l1bopndr(infile,&in_fd->fd,recsiz,buf,dsh,model);
    if ( status != E_SUCC )
	return E_FAIL;
    in_fd->openfor = L1BREAD;
    }
else
    {
    status = c_l1bopntr(in_fd->drive,&in_fd->tun,buf,dsh,recsiz,
			model,&in_fd->tape);
    if ( status != E_SUCC )
	return E_FAIL;
    in_fd->openfor = L1BREAD;
    in_fd->volnum = 1;
    }


/*
  Extract the line count from the DSH record.
---------------------------------------------*/
if ( model->satnum <= 14 )
    *goodlines = (long)(*(short *)(&dsh[8]));
else
    *goodlines = (long)(*(short *)(&dsh[128]));

/*
  Assign the number of samples
------------------------------*/
if (model->dtype == GAC)
   window[NS] = GAC_SAMPS;
else
   window[NS] = LAC_SAMPS;

return E_SUCC;

}
