/******************************************************************************
FUNCTION:	PUT_LINE

PURPOSE:	Write a line of LAC/GAC data to the AVHRR archive file format.


Version	Date	Programmer	Modifications
-------	-----	---------------	-----------------------------------------------
  1.0	01/92	T.Baltzer	Development
  1.1	05/93	J.Fenno		Modified to use the new AVHRR model and the
				new archio supports.
  2.0	11/97	D.Lloyd		Eliminated most globals and call to clean_up().
				Added return values.
				Part of major INGEST1B rewrite.
  2.1	12/98	D.Lloyd		Modifications for NOAA KLM series sats.

PROJECT:	ADAPS
******************************************************************************/
#include "ingest1b.h"
#include "adapsutil.h"

extern long curent;             /* current processing line              */


int put_line
(
long		*out_fd,	/* IN: File descriptor to output image	*/
unsigned char	*bscan,		/* IN: 2 byte input words (from input)	*/
short		inbuf[],	/* IN: Input buffer			*/
long		totchan,	/* IN: total number of bands in input	*/
long		inchan[],	/* IN: boolean array of input bands	*/
long		seqchan[],	/* IN: sequential band numbers to get	*/
struct	MODEL	*model,		/* IN: AVHRR model information		*/
short		*outbuf		/* OUT: Output buffer			*/
)
{
long	i, j, k, m, n;		/* Looping variables			*/
long	nwords;			/* number of words 			*/
long	outsamps;		/* number of samples in one output line	*/
long	swap = FALSE;		/* swap bytes flag			*/

short	*sptr;			/* pointer to a short			*/

/*
  Determine the size of the output buffer
-----------------------------------------*/
if ( model->dtype == GAC )
   outsamps = GAC_SAMPS;
else
   outsamps = LAC_SAMPS;

/*
  Unpack the telemetry data into the output buffer.
  35 x 4 bytes per word = 140 bytes 
  140 bytes x 3/4 elements per word = 105 unpacked shorts returned
  103 shorts is what is really wanted, ignore the trailing 2 shorts

  NOTE:  NOAA KLM telemetry is not packed -- copy to output buffer
-------------------------------------------------------------------*/
nwords = ARCH_MNR / 4 * 3;
if ( model->satnum <= 14 )
    c_unpck32( bscan + TELOFF, outbuf, &nwords, &swap );
else
    {
    sptr = (short *)&bscan[1056];
    for ( i = 0; i < nwords; i++, sptr++ )
        outbuf[i] = *sptr;
    }

/*
  Copy the image data to the output buffer.  If not all of the bands are
  being copied, zeros will be put in the BIP output buffer.  The input
  buffer contains all of the channels on the tape.
    i - index for output buffer 'outbuf'
    j - counts through all samples in a scanline
    m - points to every 'first' channel sample in BIP input buffer
    k - counts through all channels in output buffer (MOSTBND always)
    n - steps through sequential band positions in input buffer (tape)
------------------------------------------------------------------------*/
for ( i = MNR_WORDS, j = 0, m = 0; j < outsamps; j++, m += totchan )
   {
   for ( k = 0, n = 0; k < MOSTBND; k++ )
      {
      if ( inchan[k] )
         outbuf[i++] = inbuf[m+seqchan[n++]-1];
      else
         outbuf[i++] = 0;
      }
   }

/*
  Write a line to the output file
---------------------------------*/
if ( c_arcwrite( out_fd, &outbuf[0], &outbuf[MNR_WORDS] ) != E_SUCC )
   return E_FAIL;
curent++;

return E_SUCC;

}
