/******************************************************************************
FUNCTION:	UNPK_BREAK

PURPOSE:	Unpack a 16-bit word (10-bits used) into an I*2 word


Version	Date	Programmer	Modifications
-------	-----	--------------- -----------------------------------------------
  1.0	01/92	T.Baltzer	Development
  1.1	05/93	J.Fenno		Modified to use the new AVHRR model and the new
				archio supports.


ALGORITHM 
    Top six bits are unused.  One 10-bit word is packed right
    justified into one 16-bit word.

    Format of input buffer:

    +-----------------------+
    ! 000000   :   Word 1   !   1st 16-bit word
    +-----------------------+
    ! 000000   :   Word 2   !   2nd 16-bit word
    +-----------------------+
    etc.

    Format of output buffer:

    +--------------------+
    !  0  :    Word 1    !  1st 16-bit word
    +--------------------+
    !  0  :    Word 2    !  2nd 16-bit word
    +--------------------+
    etc.

PROJECT:	ADAPS
******************************************************************************/
#include "ingest1b.h"


int unpk_break
(
unsigned char *bscan,	/* IN: 2 byte input words 			*/
short ibuf[],		/* OUT: Series of 16 bit unpacked words		*/
long *nwords		/* IN: Number of 16-bit input words to unpack	*/
)
{
static short masknum = 0X03FF;	/* Mask to retrieve the 10 bit pixel	*/

long	i;		/* Looping variable				*/
short	*iword;		/* Pointer to 16 bit word from the input buffer	*/


/*
  Unpack a 16-bit word into one 10-bit word
-------------------------------------------*/
for ( i = 0; i < *nwords; i++ )
    {
    /*
      Retrieve the 16 bit word from the input buffer
    ------------------------------------------------*/
    iword = (short *)bscan;

    /*
      Use the mask to get the last 10 bits
    --------------------------------------*/
    ibuf[i] = *iword & masknum;

    /*
      Increment the input buffer address by the length of a 16 bit word
    -------------------------------------------------------------------*/
    bscan += EWORD;
    }

return E_SUCC;

}
