/******************************************************************************
FUNCTION:	ZERO_FILL

PURPOSE:	Copies zero-filled lines into the AVHRR archive image.


Version	Date	Programmer	Modifications
-------	-----	---------------	----------------------------------------------
  1.0	01/92	T.Baltzer	Development
  1.1	05/93	J.Fenno		Modified to use the new AVHRR model and the
				new archio supports.
  2.0	11/97	D.Lloyd		Eliminated most globals, removed call to
				clean_up(), and added return values.  Part
				of major rewrite to INGEST1B.

PROJECT:	ADAPS
******************************************************************************/
#include "ingest1b.h"

extern long curent;	/* current processing line			*/


int zero_fill
(
long	*out_fd,	/* IN: output file descriptor			*/
struct	MODEL *model,	/* IN: AVHRR model information			*/
struct	DROPPED *drop,	/* IN: Dropped line information			*/
long	num_0_lines,	/* IN: Number of dropped lines to zero fill	*/
short   *outbuf		/* IN/OUT: Output buffer			*/
)
{
char	msg[ERRLEN];	/* Error messages				*/

long	i;		/* counter					*/


/*
  Save the location and number of dropped lines.  If the number of gaps
  is over the limit, the image may not be worth processing.
------------------------------------------------------------------------*/
model->totmiss += num_0_lines;
drop->lmissed[drop->ngaps] = curent + 1;
drop->nmissed[drop->ngaps] = num_0_lines;
drop->ngaps++;
if (( model->dtype == GAC ) && ( drop->ngaps > MAX_GAPS ))
   {
   sprintf(msg,"%s %d %s %ld",
           "The number of dropped line gaps exceeds the limit of",
           MAX_GAPS,"at line",curent + 1);
   c_lasmsg(FALSE,"ingest1b-gaps",msg);
   return E_FAIL;
   }

if ((( model->dtype == LAC ) || ( model->dtype == HRPT )) && 
	( drop->ngaps > MAX_GAPS ))
   {
   sprintf(msg,"%s %d %s %ld",
           "The number of dropped line gaps exceeds the limit of",
           MAX_GAPS,"at line",curent + 1);
   c_lasmsg(FALSE,"ingest1b-gaps",msg);
   return E_FAIL;
   }

sprintf(msg,"%ld %s %ld",num_0_lines,
        "dropped lines written to output image starting at line",curent + 1);
c_lasmsg(FALSE," ",msg);


/*
  Now write the specified number of zero-filled lines to the image.
-------------------------------------------------------------------*/
for ( i = 0; i < num_0_lines; i++ )
   put_drop( out_fd, model, outbuf );

return E_SUCC;

}
