/*************************************************************************
FUNCTION:	L1BLABEL

PURPOSE:	Generates a tape label file for a Level 1b tape

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       08/90    J. LaVergne		Reformatted for 3 in labels
 5.0	   12/91    T. Rockvam		Conversion to LAS 5.0
					Made separate to allow
					being called by stack1b and mklevel1b
					taken from ADAPS LABEL program
 5.1	   06/92    T. Rockvam		Uses new generic label printing 
					facilities: l1blabel is called once for
					each label
					PDF checks facility name: if not EDC, 
					don't print the label

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"

main()

{
struct PARBLK vblock;  		/* input parameter block                */

char	out[CMLEN],		/* output files (tape directory)	*/
	dirfil[40],		/* tape directory file name		*/
	sceneid[CMLEN],		/* scene identifier			*/
	spec[CMLEN],		/* filename specification		*/
	hostspec[CMLEN],	/* host filename specification		*/
	density[5],		/* tape density				*/
	ordernum[14],		/* order number				*/
	unitnum[4],		/* unit number				*/
	drive[CMLEN],		/* drive tape was mounted on		*/
	subcmd[10],      	/* buffer to read into                  */
	*ptr,			/* index into string			*/
	lbltxt[11][27],		/* text to print to free-formatted label*/
	filename[100][CMLEN],   /* space for filenames from c_getfiles  */
	*vec[11];		/* pointers to text strings		*/

char	uplat[12],		/* upper left  latitude			*/
	uplong[12],		/* upper left  longitude		*/
	lolat[12],		/* lower right latitude			*/
	lolong[12];		/* lower right longitude		*/

long	acqdate,		/* acquisition date			*/
	cdate;			/* creation date			*/

long	nvols,			/* number of volumes used		*/
	nfiles,			/* number of files on this tape		*/
	now,			/* time now				*/
	nl,			/* number of lines in output file	*/
	ns,			/* number of samples in output file	*/
	vol;			/* current volume number		*/

long	i,			/* loop counter				*/
	status,			/* return status of support calls	*/
	std_out,                /* standard output                      */
	one = 1,		/* value of one				*/
	zero = 0,		/* value of zero			*/
	nonfatal = 1,		/* nonfatal error code			*/
	fatal = -1;		/* fatal error code			*/

struct tm *curtime;		/* current time				*/

struct VARIABLE *var;           /* pointer to a TAE variable            */

FILE	*fpi,			/* pointer to input label data file	*/
	*outfile;               /* pointer to output tape directory file*/

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);

/*
  Determine current date
*/
now = time(NULL);
curtime = localtime(&now);
cdate = ((curtime->tm_mon + 1) * 10000) + 
	 (curtime->tm_mday * 100) + 
	  curtime->tm_year;

/*
  Get the subcommand 
*/
var = p_find(&vblock, "_SUBCMD");
strcpy(subcmd,SVAL(*var,0));
if (strcmp(subcmd,"MULTI") == 0)
    {
/*
    Get the number of files on tape
*/
    var = p_find(&vblock,"SCENEID");
    nfiles = var->v_count;
    }

if (strcmp(subcmd,"SPEC") == 0)
    {

/* If we're dealing with the MULTISPEC subcommand, first convert the filename
   specification from a LAS-type format to an actual hostname, then call
   c_getfiles() to build a list of filenames, and then finally convert them
   all back to LAS filenames. */

    var = p_find(&vblock, "INFILE"); 
    strcpy(spec, SVAL(*var,0));
    status = c_cmhost(spec, hostspec);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","l1blabel-fatal",&fatal);

    c_getfiles(hostspec, filename, &nfiles, 100, 1, 1);
    }

/* 
  Get the number of volumes used 
*/
var = p_find(&vblock,"NVOLS");
nvols = IVAL(*var,0);

/* 
  Get the current volume
*/
var = p_find(&vblock,"VOL");
vol = IVAL(*var,0);

/*
  Open the label data file
*/
fpi = fopen("lab.dat","r");
if (fpi == NULL)
    {
    c_errmsg("Label data file not found","l1blabel-nodata", &nonfatal);
    c_errmsg("Fatal error encountered","l1blabel-fatal", &fatal);
    }

/*
  Get information from label data file
*/
/* scene id */ 
fscanf(fpi,"%s",sceneid);

/* acquisition date */
fscanf(fpi,"%d",&acqdate);

/* upper latitude */
fscanf(fpi,"%s",uplat);

/* upper longitude */
fscanf(fpi,"%s",uplong);

/* lower latitude */
fscanf(fpi,"%s",lolat);

/* lower longitude */
fscanf(fpi,"%s",lolong);

/* density */
fscanf(fpi,"%s",density);

/* number of lines */
fscanf(fpi,"%d",&nl);

/* number of samples */
fscanf(fpi,"%d",&ns);

/* order number */
fscanf(fpi,"%s",ordernum);

/* unit number */
fscanf(fpi,"%s",unitnum);

/* drive name */
fscanf(fpi,"%s",drive);

/* 
  Close the label data file
*/
fclose(fpi);

/*
  Create one label for this volume
*/
if (strcmp(subcmd,"SINGLE") == 0)
    {
/*
    Label based on scene id for one scene
*/
    /* Correct spacing to next label 
    sprintf(lbltxt[0],"\n");
    */

    /* Line 1 */
    sprintf(lbltxt[0],"      EROS Data Center");

    /* Lines 2 & 3 */
    sprintf(lbltxt[1],"           AVHRR");
    sprintf(lbltxt[2],"--------------------------");

    /* Line 4 */
    sprintf(lbltxt[3],"Scene ID: %16.16s",sceneid);

    /* Line 5 */
    sprintf(lbltxt[4],"AqDt ");
    if ((acqdate) == 0)
	sprintf(&lbltxt[4][5],"        ");
    else
	sprintf(&lbltxt[4][5],"%02d/%02d/%02d",acqdate/10000,
		(acqdate % 10000) / 100, acqdate % 100);
    sprintf(&lbltxt[4][13]," CrDt");
    if ((cdate) == 0)
	sprintf(&lbltxt[4][18],"        ");
    else
	sprintf(&lbltxt[4][18],"%02d/%02d/%02d",cdate/10000,
		(cdate % 10000) / 100, cdate % 100);

    /* Line 6 */
    sprintf(lbltxt[5],"UpL:%-10.10s %-10.10s",uplat,uplong);

    /* Line 7 */
    sprintf(lbltxt[6],"LoR:%-10.10s %-10.10s",lolat,lolong);

    /* Line 8 */
    sprintf(lbltxt[7],"NL:");
    if (nl == 0)
	sprintf(&lbltxt[7][3],"     ");
    else
	sprintf(&lbltxt[7][3],"%5d",nl);

    sprintf(&lbltxt[7][8]," NS:");
    if (ns == 0)
	sprintf(&lbltxt[7][12],"    ");
    else
	sprintf(&lbltxt[7][12],"%4d",ns);

    sprintf(&lbltxt[7][16]," #Files:01");

    /* Line 9 */
    sprintf(lbltxt[8],"Seq:%2d/%-2d",vol,nvols);

    sprintf(&lbltxt[8][9]," Bpi:%4.4s",density);

    sprintf(&lbltxt[8][18]," Dr:%4.4s",drive);

    /* Line 10 */
    sprintf(lbltxt[9],"Format: L1B   UNLABELED");

    /* Line 11 */
    sprintf(lbltxt[10],"Order#: %13.13s/%4.4s",
	    ordernum,unitnum);
    }
else
    {
/*
    Label based on order number for multiple scenes
*/
    /* Correct spacing to next label
    sprintf(outfile,"\n"); 
    */

    /* Line 1 */
    sprintf(lbltxt[0],"      EROS Data Center");

    /* Lines 2 & 3 */
    sprintf(lbltxt[1],"           AVHRR");
    sprintf(lbltxt[2],"--------------------------");

    /* Line 4 */
    sprintf(lbltxt[3],"Order #: %13.13s", ordernum);

    /* Line 5 */
    sprintf(lbltxt[4],"#Files:");
    if (nfiles == 0)
	sprintf(&lbltxt[4][7],"  ");
    else
	sprintf(&lbltxt[4][7]," %02d",nfiles);

    /* Line 6 */
    sprintf(lbltxt[5],"Seq: %2d/%-2d",vol,nvols);

    sprintf(&lbltxt[5][9],"        Bpi:%4.4s",density);

    /* Line 7 */
    sprintf(lbltxt[6],"Format: L1B   UNLABELED");

    /* Line 8 */
    sprintf(lbltxt[7],"CrDt: ");
    if ((cdate) == 0)
	sprintf(&lbltxt[7][6],"        ");
    else
	sprintf(&lbltxt[7][6],"%02d/%02d/%02d",cdate/10000,
		(cdate % 10000) / 100, cdate % 100);
    sprintf(&lbltxt[7][14],"     Dr:%4.4s",drive);

    /* Lines 9 - 11 (blank) */
    sprintf(lbltxt[8]," ");
    sprintf(lbltxt[9]," ");
    sprintf(lbltxt[10]," ");
    }

/*
  Send back the text strings for the label
*/
for (i = 0; i < 11; i++)
    vec[i] = lbltxt[i];
if (q_string(&vblock,"TEXT",11,vec,P_UPDATE) != SUCCESS)
    {
    c_errmsg("Error returning parameter to TAE","l1blabel-rparm",&nonfatal);
    c_errmsg("Fatal error encountered","l1blabel-fatal",&fatal);
    }

/*
  Update the TAE parameter block 
*/
if (q_out(&vblock) != SUCCESS)
   {
   c_errmsg("Error returning parameter to TAE","l1blabel-rparm",&nonfatal);
   c_errmsg("Fatal error encountered","l1blabel-fatal",&fatal);
   }

/*
  Create a tape directory file for tapes containing multiple images
  create the directory only with the first label
*/
if (((subcmd[0] == 'M') || (!strcmp(subcmd,"SPEC"))) && (vol == 1))
    {
    sprintf(dirfil,"%s.tdir",ordernum);
    strcpy(out,dirfil);
    outfile = fopen(dirfil,"w");
    if (outfile == NULL)
	{
    	c_errmsg("Could not open file -- tape directory not created",
		"l1blabel-noopen",&nonfatal);
    	c_errmsg("Fatal error encountered","l1blabel-fatal", &fatal);
	}
    fprintf(outfile,"              AVHRR LEVEL-1B TAPE DIRECTORY\n");
    fprintf(outfile,"\n");
    fprintf(outfile,"Note: This directory indicates which scenes are contained\n");
    fprintf(outfile,"      on the tape for order #%s.  It is important\n",ordernum);
    fprintf(outfile,"      to keep it, because the scene ids are NOT printed\n");
    fprintf(outfile,"      on the tape label.\n");
    fprintf(outfile,"\n");
    fprintf(outfile,"File No. 	Scene Id\n");
    fprintf(outfile,"\n");

    if (!strcmp(subcmd,"MULTI"))
    {
	var = p_find(&vblock,"SCENEID");
	for (i = 0; i < var->v_count; i++)
	    {
	    strcpy(sceneid,SVAL(*var,i));
	    if ((ptr = strchr(sceneid,';')) != NULL)
		*ptr = (char)NULL;
	    if ((ptr = strchr(sceneid,']')) == NULL)
		fprintf(outfile,"%2d              %s\n",i+1,sceneid);
	    else
		{
		ptr++;
		fprintf(outfile,"%2d              %s\n",i+1,ptr);
		}
	    }
    }

    else
    {
	for (i = 0; i < nfiles; i++)
	{
            if ((ptr = strrchr(filename[i],'.')) != NULL)
                *ptr = (char)NULL;
            if ((ptr = strrchr(filename[i],'/')) == NULL)
                fprintf(outfile,"%2d              %s\n",i+1,filename[i]);
            else
                {
                ptr++;
                fprintf(outfile,"%2d              %s\n",i+1,ptr);
                }
	}
     }

    fclose(outfile);
    c_complt("L1BLABEL",out,&one);
    }
else
    {
    c_complt("L1BLABEL",out,&zero);
    }
}
