/***************************************************************************
NAME      		  BLK_EM_UP

PURPOSE: Create a list of bad data blocks from bad data segments.
         The data segments are one line only, the data blocks may be multiple
         lines long.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Baltzer	Oct. 1991	Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS

PROJECT   	         ADAPS		                

LANGUAGE		  C	

ALGORITHM REFERENCE
  A special thanks to Jim Fenno for his massive input into the development of
this algorithm.

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "linedrop.h"


FUNCTION blk_em_up(cur_line)

long cur_line;                  /* current line in image being processed  */

{

struct BAD_BLK *blk_ptr;	/* pointer to a bad block of data      */


blk_ptr = alph_blk;		/* initialize pointer to start of list */

while (alph_seg)			/* bad data segment list is not empty */
   {
   /* find appropriate data block in the list */
   if ( find_blk(&blk_ptr,alph_seg->start_samp,cur_line) == E_FAIL )
      return(E_FAIL);
   /* see if block and segment do not lap at all:             ______________	*/
   /*							      |            |	*/
   /*							      --------------	*/
   /*						 xxxxxxxxxxx			*/
   if ( !blk_ptr || blk_ptr->start_samp > alph_seg->end_samp )
      {
      if ( insert_blk(&blk_ptr,cur_line,alph_seg->start_samp,cur_line,
                      alph_seg->end_samp) == E_FAIL )
         return(E_FAIL);
      if ( delete_seg(alph_seg) == E_FAIL )
         return(E_FAIL);
      }
   else
      {
      /* see if block overhangs to the left of the             ______________	*/
      /* beginning of the segment:			       |            |   */
      /*	 					       --------------	*/
      /*						           xxxxxxxxxxxx	*/
      if (alph_seg->start_samp > blk_ptr->start_samp)
         {
         split_rpt(blk_ptr,alph_seg->start_samp,cur_line);
         }
      /* see if segment overhangs to the left of the           ______________	*/
      /* beginning of the block:			       |            |   */
      /*	 					       --------------	*/
      /*						   xxxxxxxxxxxxx	*/
      else if (blk_ptr->start_samp > alph_seg->start_samp)
         {
         if ( insert_blk(&blk_ptr,cur_line,alph_seg->start_samp,cur_line,
                         blk_ptr->start_samp - 1) == E_FAIL )
            return(E_FAIL);
         alph_seg->start_samp = blk_ptr->start_samp;
         }

      /***** ( At this point the starting sample of the segment and the     *****/
      /***** ( starting sample of the block are equal.                      *****/

      /* see if block overhangs to the right of the           ______________	*/
      /* end of the segment:			      	      |            |    */
      /*	 					      --------------	*/
      /*						      xxxxxxxxxx	*/
      if (blk_ptr->end_samp > alph_seg->end_samp)
         {
         if ( split_blk(&blk_ptr,cur_line,alph_seg->end_samp) == E_FAIL )
            return(E_FAIL);
         if ( delete_seg(alph_seg) == E_FAIL )
            return(E_FAIL);
         }
      /* see if segment overhangs to the right of the        ______________	*/
      /* end of the block:			      	     |            |     */
      /*	 					     --------------	*/
      /*						     xxxxxxxxxxxxxxxxx	*/
      else if (alph_seg->end_samp > blk_ptr->end_samp)
         {
         blk_ptr->end_line = cur_line;
         alph_seg->start_samp = blk_ptr->end_samp + 1;
         }
      /* the block and the segment must overlap precisely:   ______________	*/
      /*                  			      	     |            |     */
      /*	 					     --------------	*/
      /*						     xxxxxxxxxxxxxx   	*/
      else
         {
         blk_ptr->end_line = cur_line;
         if ( delete_seg(alph_seg) == E_FAIL )
            return(E_FAIL);
         }
      }
   }	/* end of while construct */

return(E_SUCC);

}

