/***************************************************************************
NAME      		  FIND_BLK

PURPOSE: Find the bad data block structure to be located at in order to 
         correctly process the current bad data segment.  As the doubly
         linked list of bad data blocks is traversed from left to right,
         any blocks that have ended will be reported and deleted.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Baltzer	Oct. 1991	Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS

PROJECT   	         ADAPS		                

LANGUAGE		  C	

ALGORITHM
  Traverse through the bad data block list from left to right.  If the block
is obsolete ( end of block has been defined ) delete it from the list.  Continue
until a block is found that is ongoing (not obsolete).  Check to see if this
block is the block that lies closest to the segment to the right.  If not,
continue traversing the list.  If such a block is not found, the pointer to
the end of the block list is returned (NULL). 
  Picture Example:

---------------      -----------------    -------------------   ---------------
|      A      |      |       B       |    |        C        |   |     D       |
---------------      |               |    -------------------   |             |
                     |               |                          |             |
                     |               |      xxxxxx segment xxxxxxxxxxxxxx     |

Block A and C are obsolete blocks but still in the linked list.  Block B lies
to the left of the bad data segment.  Block D is the block which find_blk is
searching for.

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "linedrop.h"


FUNCTION find_blk(blk_ptr,ss_seg,cur_line)

struct BAD_BLK **blk_ptr;	/* pointer to the data block structure    */

long ss_seg;			/* starting sample of bad segment	  */
long cur_line;			/* current line in image being processed  */

{

for (;;)
   {
   while ( *blk_ptr && ((*blk_ptr)->end_line < cur_line - 1))	/* obsolete block */
      {
      fix_blk((*blk_ptr)->start_line,(*blk_ptr)->start_samp,
              (*blk_ptr)->end_line,(*blk_ptr)->end_samp);
      if ( delete_blk(blk_ptr) == E_FAIL )
         return(E_FAIL);
      }


   if ( *blk_ptr )		/* not pointing to NULL, this block has not ended */
      {
      if ( ss_seg <= (*blk_ptr)->end_samp )
         return(E_SUCC);		/* correct block for the current segment  */
      else
         (*blk_ptr) = (*blk_ptr)->next;	/* not correct block, continue	  */
      }
   else
      return(E_SUCC);	/* pointing to NULL, so we are done */

   }	/* end infinite while construct */

}
