/***************************************************************************
NAME      		  FINDBAD

PURPOSE: Finds bad data segments in a line of an image.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Baltzer	Oct. 1991	Original development
S. Nelson	Jun. 1992	Added option to fix dropped lines

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS

PROJECT   	         ADAPS		                

LANGUAGE		  C	

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "linedrop.h"

extern long curent;		/* Current processing line 		    */

FUNCTION findbad(kerndim, npixels, ns, dtypei, size, segment_count, thrval,
		 thebuf, lines, nblks, nbrln, fixlast)


   long		fixlast;	/* Flag to fix the line after dropline	      */
   long         kerndim;        /* Length of a moving one dimensional window
                                   through the input image                    */
   long		*lines;		/* List of line numbers that start each
				   block of drop lines			      */
   long		nblks;		/* Number of blocks with drop lines	      */
   long		*nbrln;		/* List of number of lines in each block
				   of drop lines			      */
   long         npixels;        /* Number of pixels that are
                                    made bad when a bad pixel is found        */
   long		ns;		 /* Number of samples per line to be processed*/
   long         dtypei;         /* data type of this image                    */
   long         size;           /* Number of bytes per pixel                  */
   long 	*segment_count;  /* Number of bad segments in a line          */
   long         thrval;         /* Threshold value used to set a threshold
                                    for the sum of the squared differences    */


   unsigned char *thebuf;        /* Pointer to buffer containing the line to
                                   search for bad segments                    */
{
float   avg_diff;               /* Average of the square of the differences   */
float   difference;             /* Difference of adjacent pixels.	      */
float   pixel1;                 /* Temporary placeholder for a pixel value    */
float   pixel2;                 /* Temporary placeholder for a pixel value    */
float   sum;                    /* Cumulative sum of the differences squared  */

long    bad_pixels;		/* Bad pixel counter within a bad segment     */
long	blck;			/* Loop control variable for drop line test   */
long	dr_flag;		/* Flag identifying drop line		      */
long	cntr_pix;		/* Pixel at the center of the kernel 	      */
long   	in_previous_bad_seg;	/* Boolean indicating whether kernel center   */
				/* pixel lies within the range of an existing */
				/* bad segment.  True = 1, False = 0	      */
long	start_smpl;		/* starting sample of a bad segment	      */
long    sample;			/* Index traversing kernel to find the average*/
				/* difference squared			      */
long   	half_kern;		/* half of the kernel			      */
long    half_npixels; 	        /* half of the pixels in npixels	      */
long	one = 1;		/* the value of one			      */

struct BAD_SEG *newptr;         /* pointer to new element 		      */


/* Initialize the variables
   ------------------------*/

dr_flag = FALSE;
alph_seg = beta_seg = NULL;
half_kern = (kerndim - 1) / 2;
half_npixels = (npixels - 1) / 2;
bad_pixels = 0;
in_previous_bad_seg = FALSE;
*segment_count = 0;

/* Check if line is identified in the drop line file 
   -------------------------------------------------*/
if (nblks != 0)
   {
   for (blck = 0; ((blck < nblks) && (dr_flag == FALSE)); blck++)
      {
      if ((curent >= lines[blck]) && (curent < (lines[blck] + nbrln[blck]
								 + fixlast)))
	 {
	 if (insert_seg(one,&newptr) == E_FAIL)
	    return(E_FAIL);
	 newptr->start_samp = 1;
	 newptr->end_samp = ns;
	 newptr->length = ns;
	 dr_flag = TRUE;
	 (*segment_count)++;
         }
      }
   }

/* If line is not drop line, begin processing for bad segment
   ---------------------------------------------------------- */
if (dr_flag == FALSE)
{

/* Initialize the summation for the first kernel
   ---------------------------------------------*/
sum = 0.0;
for (sample = 1; sample < kerndim; sample++)
   {
   switch (dtypei)
      {
      case EBYTE:
              pixel1 = *(thebuf + ((sample - 1) * size));
              pixel2 = *(thebuf + (sample * size));
	      break;
      case EWORD:
              pixel1 = *(short *) (thebuf + ((sample - 1) * size));
              pixel2 = *(short *) (thebuf + (sample * size));
	      break;
      case ELONG:
              pixel1 = *(long *) (thebuf + ((sample - 1) * size));
              pixel2 = *(long *) (thebuf + (sample * size));
              break;
      case EREAL:
              pixel1 = *(float *) (thebuf + ((sample - 1) * size));
              pixel2 = *(float *) (thebuf + (sample * size));
  	      break;
      }
   difference = pixel1 - pixel2;
   sum += (difference * difference); 
   }

/* Slide the kernel across the current line */

for (cntr_pix = half_kern + 1;   ; cntr_pix++)
   {
   /* Start the loop by calculating the average difference squared,
      and determining if the pixel is bad or not.
      --------------------------------------------------------------*/
   avg_diff = sum / (kerndim - 1);

   if ((long)avg_diff >= thrval )	/* bad pixel has been located */
      {
      if (!in_previous_bad_seg)		/* not in previous bad segment */
         {
         start_smpl = cntr_pix - half_npixels;
         if (start_smpl < 1)   start_smpl = 1;
         if ( insert_seg(start_smpl,&newptr) == E_FAIL )
            return(E_FAIL);
         in_previous_bad_seg = TRUE;
         (*segment_count)++;
         }
      bad_pixels = npixels;
      }
   else					/* pixel is good, not bad */
      {
      if (in_previous_bad_seg)
         {
         bad_pixels--;
         if (bad_pixels == 0)
            {
            in_previous_bad_seg = FALSE;
            newptr->end_samp = cntr_pix - half_npixels - 1;
            newptr->length = newptr->end_samp - newptr->start_samp + 1;
            }
         }
      }

   if ( cntr_pix == ns - half_kern ) break;  /* exit infinite 'for' loop */
					     /* (at the end of the line) */

   /* There are more kernels on this line to be processed.   Calculate   */
   /* the next kernel summation by subtracting, adding one squared	 */
   /* difference to the running summation.				 */

   sample = cntr_pix - half_kern;
   switch (dtypei)
      {
      case EBYTE:
              pixel1 = *(thebuf + ((sample - 1) * size));
              pixel2 = *(thebuf + (sample * size));
	      break;
      case EWORD:
              pixel1 = *(short *) (thebuf + ((sample - 1) * size));
              pixel2 = *(short *) (thebuf + (sample * size));
	      break;
      case ELONG:
              pixel1 = *(long *) (thebuf + ((sample - 1) * size));
              pixel2 = *(long *) (thebuf + (sample * size));
              break;
      case EREAL:
              pixel1 = *(float *) (thebuf + ((sample - 1) * size));
              pixel2 = *(float *) (thebuf + (sample * size));
  	      break;
      }
   difference = pixel1 - pixel2;
   sum -= (difference * difference); 	/* subtract left pixel */

   sample = cntr_pix + half_kern;
   switch (dtypei)
      {
      case EBYTE:
              pixel1 = *(thebuf + ((sample - 1) * size));
              pixel2 = *(thebuf + (sample * size));
	      break;
      case EWORD:
              pixel1 = *(short *) (thebuf + ((sample - 1) * size));
              pixel2 = *(short *) (thebuf + (sample * size));
	      break;
      case ELONG:
              pixel1 = *(long *) (thebuf + ((sample - 1) * size));
              pixel2 = *(long *) (thebuf + (sample * size));
              break;
      case EREAL:
              pixel1 = *(float *) (thebuf + ((sample - 1) * size));
              pixel2 = *(float *) (thebuf + (sample * size));
  	      break;
      }
   difference = pixel1 - pixel2;
   sum += (difference * difference); 	/* add right pixel */

   }	/* end of 'for' construct */

/* at end of the line, terminate outstanding bad data segment */
if (in_previous_bad_seg)
   {
   newptr->end_samp = (cntr_pix) - (npixels - bad_pixels) + half_npixels;
   if (newptr->end_samp > ns)   newptr->end_samp = ns;
   newptr->length = newptr->end_samp - newptr->start_samp + 1;
   }
}       /* End of processing if not drop line */
return(E_SUCC);
}
