/***************************************************************************
NAME      		  FIX_BLK

PURPOSE: Fix a block of bad data by replacing the pixels within the bad
         block using an interpolation algorithm.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Baltzer	Oct. 1991	Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS

PROJECT   	         ADAPS		                

LANGUAGE		  C	

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "linedrop.h"

FUNCTION fix_blk(sl,ss,endl,ends)

long sl;			/* starting line of the bad data block    */
long ss;			/* starting sample of the bad data block  */
long endl;			/* ending line of the bad data block      */
long ends;			/* ending sample of the bad data block    */

{

long nl;			/* number of lines in repair window  */
long ns;			/* number ofsamples in repair window */
long non_fatal = 1;		/* non-fatal error status	     */
long inwind[1][4];		/* input window areas to fix         */
long nwinds = 1;		/* number of repair windows	     */
long nimgs = 1;			/* number of images to fix	     */
long update = 1;		/* assume update mode always	     */

char text[60];			/* error message text */

nl = endl - sl + 1;
ns = ends - ss + 1;

inwind[0][SL] = sl;
inwind[0][SS] = ss;
inwind[0][NL] = nl;
inwind[0][NS] = ns;

if ( inwind[0][NL] == fix.window[0][NL])
   {
   sprintf(text,"Repair window (%d, %d, %d, %d: %d) spans entire image vertically",
           sl, ss, nl, ns, fix.bands[0][0]);
   c_errmsg(text,"linedrop-vspan",&non_fatal);
   c_errmsg("Repair window will remain unchanged","linedrop-vspan",&non_fatal);
   }
else
   {
   vertical(fix.vblock, fix.hostname, fix.bands, inwind, nwinds, nimgs, fix.dtype,
            fix.size, update, fix.window);

   /* load up informational message, and output to a print file */
   sprintf(text,"Window (%d, %d, %d, %d: %d) repaired",
           sl, ss, nl, ns, fix.bands[0][0]);
   fprintf(fix.out_fd,"%s\n",text);
   }

   return(E_SUCC);
}
