/***************************************************************************
NAME      		  LINEDROP

PURPOSE:  Find and correct bad line segments.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
M. Choate       Aug. 1991       Original development
K. Kimball      Aug. 1991       Updating to 5.0 standards
T. Baltzer      Oct. 1991	REWRITTEN to use existing COPY from pdf
				          to use dynamic linked lists
					  to reuse code from SEGMREPR:
						- vertical.c
						- terminate.c
						- closefile.c
S. Nelson	Jun. 1992	Added option to fix dropped lines.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   LINEDROP must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)

ALGORITHM REFERENCES     

   Much of the control structure for this routine was copied from
   LNFILL
******************************************************************************/

#include <stdio.h>
#include "las.h"

#define MAX_BLOCKS 1000  /* Maximum number of blocks in drop line file */

struct FIX_BLK
    {
    struct PARBLK *vblock;       /*  TAE parameter block                     */
    char hostname[1][CMLEN];     /* host name of image to be repaired        */
    char minmax[4];              /* min/max flag                             */
    long bands[1][1];  		 /* band to be processed                     */
    long msgtim;                 /* elapsed time between display messages    */
    long dtype;                  /* data type of images to be repaired       */
    long size;                   /* number of bytes required for this dtype  */
    long window[1][4];		 /* window specification of input images     */
    FILE *out_fd;		 /* ascii output file descriptor	     */
    };
					
/* The following variables must be declared global for image I/O routines
   ----------------------------------------------------------------------*/

int  fdesc[MAXIMG+1];           /* Image file descriptors for input images   */
int  gdesc;		        /* Group descriptor used for image I/O       */

struct BAD_SEG *alph_seg;          /* ptr to first bad segment */
struct BAD_SEG *beta_seg;          /* ptr to last bad segment */

struct BAD_BLK *alph_blk;        /* ptr to first bad block */
struct BAD_BLK *beta_blk;        /* ptr to last bad block  */

struct FIX_BLK fix;              /* data required to fix the image */

main()
{
extern long curcat;              /* Current processing band                   */
extern long curent;		 /* Current processing line		     */

struct PARBLK	vblock;         /* TAE parameter block    		     */
char    fname[CMLEN];           /* Output file name                          */
char	hosin[MAXIMG][CMLEN];	/* Input image host name		     */
char    minmax[4];	        /* Minmax flag buffer 			     */
char	taein[MAXIMG][CMLEN];	/* Input image LAS name			     */
char	drfile[CMLEN];		/* Dropline input file name		     */

double  maxval[MAXBND + 1];  	/* Max pixel value array                     */
double  minval[MAXBND + 1];  	/* Min pixel value array                     */

FILE *out_fd;                   /* Output file descriptor for recording blocks
                                   that are fixed                            */

float   thrperc;                /* Threshold percent indicating the perenct of
                                   pixels in a line that must be good for the
                                   line to be valid.                         */

static  long bands[MAXIMG][MAXBND + 1];
                                /* Band specifications		             */
long	dpblk;			/* Loop control variable		     */
long	drpflg;			/* Flag for large data gaps		     */
long	dtype[MAXIMG];		/* Data type of each input image	     */
long	fixlast;		/* Flag to fix the line after dropline	     */
long    kerndim;                /* Length of a moving one dimensional window
                                   through the input image                   */
long	lines[MAX_BLOCKS];	/* Drop lines in the image	   	     */
long	maxlfix;		/* Maximum number of droplines to fix	     */
long 	msgtim;			/* Elpased time between display messages     */
long 	nbands[MAXIMG];		/* Number of bands per input image           */
long	nblks = 0;		/* Number of blocks of drop lines	     */
long	nbrln[MAX_BLOCKS];	/* Number of lines in a block of drop lines  */
long 	nmrimg;			/* Number of input images for image          */
long	nonfatal = 1;		/* Do not stop processing 		     */
long    npixels;                /* Number of pixels around a pixel that are
                                   made bad when a bad pixel is found        */
long  	std_out;		/* Logical unit # for standard output        */
long    thrval;                 /* Threshold value used to set a threshold
                                   for the sum of the squared differences    */
long    totbnd;			/* Total number of bands in input images     */
long 	window[MAXIMG][4];	/* Window specification for input images     */
long	update = 1;		/* update mode set to 'yes'		     */	
long	print[3];		/* print option				     */
long	zero = 0;		/* value of zero			     */
char    prtfile[CMLEN];         /* print file name                           */


/* Initialize LAS and TAE
   ---------------------- */
c_inlas(&vblock, &std_out);

/* Get user-specified parameters and TAE globals
   --------------------------------------------- */
getpar(&vblock, fname, hosin, minmax, taein, &thrperc, bands,
       &kerndim, &msgtim, &nmrimg, nbands, &npixels, &thrval,
       &totbnd, window, print, prtfile, drfile, &fixlast, &maxlfix);

/* Open i/o images and initalize processing
   ---------------------------------------- */
open_files(prtfile, hosin, minmax, minval, maxval, &out_fd, dtype, &msgtim,
	   nbands, &nmrimg, &totbnd, window, drfile, lines, &nblks, nbrln);

/* Identify and repair the bad data segments
   ----------------------------------------- */
proces(thrperc, bands, &kerndim, nbands, &nmrimg, &npixels, dtype, &thrval,
				 window, hosin, lines, nblks, nbrln, fixlast);

/* If linedrop file was used, check if any data gaps are too large, if yes,
   zero fill that line
   -----------------------------------------------------------------------*/
if (drfile[0] != NULL)
   {
   drpflg = FALSE;
   for (dpblk=0; dpblk < nblks; dpblk++)
      {
      if (nbrln[dpblk] > maxlfix)
	 {
	 if (drpflg == FALSE)
	    c_errmsg("Input values set to zero","linedrop-maxline", &nonfatal);
	 if ((lines[dpblk] + nbrln[dpblk] - 1) <= window[0][NL])
	 zerofil(&vblock, hosin, bands, nbands, dtype[zero], fix.size,
		            lines, nbrln, dpblk, update, window);
	 drpflg = TRUE;
	 }
      }
   }
/* Update the ddr and close the function
   ------------------------------------- */
terminate(&vblock, fix.hostname, minmax, maxval, minval, bands, nbands,
		    nmrimg, update, window, out_fd, taein, print, fname);
}
