/*****************************************************************************
NAME:		OPEN_FILES

PURPOSE:  Perform initial bookkeeping required to begin processing.
     

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
1.0	   Tim Baltzer     10/91	EDC		Origonal development	
1.0	   S. Nelson	    6/92	EDC		Added option to fix
							dropped lines


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS

PROJECT			LAS

ALGORITHM:

ALGORITHM REFERENCES		none

NOTE: More than one input image is checked for here although this senario
      should never occur.  This coding was in existence, and is left as is
      in the event that more than one input image would be allowed some day.

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "worgen.h"
#include "linedrop.h"

#define MAX_BLOCKS 1000 /* Maximum number of blocks in drop line file */
/*
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc[];	        /* image file descriptors used for image I/O */

extern long curcat;		/* current processing band		     */
extern long curent;		/* current processing line		     */

FUNCTION open_files(prtfile, hosin, minmax, minval, maxval, out_fd, dtype,
		    msgtim, nbands, nmrimg, totbnd, window, drfile, lines,
		    nblks, nbrln)
		

   char		*drfile;		/* input dropline file name	      */
   char         *prtfile;               /* Output file name                   */
   char 	hosin[][CMLEN];		/* Host names of input images	      */
   char		*minmax;		/* Min max flag                       */

   double  	*minval;		/* Min pixel value array              */
   double  	*maxval;		/* Max pixel value array              */

   FILE         **out_fd;               /* Output file descriptor used to record
                                           the blocks being fixed             */

   long 	dtype[];		/* Data type of each input image      */
   long		*lines;			/* List of line numbers that start each
				 	   block of drop lines		      */
   long		*msgtim;		/* Elapsed time between display 
					   messages                           */
   long		nbands[];		/* Number of bands per input image    */
   long		*nblks;			/* Number of blocks with drop lines   */
   long		*nbrln;			/* List of number of lines in each block
					   of drop lines		      */
   long		*nmrimg;		/* Number of input images             */
   long 	*totbnd;		/* Total number of bands to be 
                                           processed                          */
   long		(*window)[4];		/* Window specification of input      */


{

long	access = IUPDATE;	/* Image I/O access type                   */
long    fatal = -1;		/* Fatal error flag			   */
long	i;			/* Loop control variable		   */
long	image;			/* Loop index counting input images        */
long	msg_flag=ONECAT;	/* Flag for processing message             */
long	nonfatal = 1;		/* Nonfatal error flag 			   */
long    one = 1;		/* Address with a value of one	           */
long	status;			/* Function return status code             */
long	*tlines;		/* Total number of drop lines		   */

static long opt[3]={0,0,0};	/* optional parameter array for EOPENR	   */

FILE	*fd;			/* File containing drop lines              */

/* Open output ascii file 
   -----------------------*/
*out_fd = fopen(prtfile, "w");
if (*out_fd == NULL)
   {
   c_errmsg("Cannot open file to record blocks being fixed",
            "linedrop-open",&nonfatal);
   c_errmsg("Fatal error encountered","linedrop-fatal", &fatal);
   }
fix.out_fd = *out_fd;


/* Open all the input images returning the data type of each image
   --------------------------------------------------------------- */

for (image = 0; image < *nmrimg; image++)
   {
   dtype[image] = 0;
   status = c_eopenr(&fdesc[image],hosin[image],&nbands[image],
                     &window[image][0],&window[image][1],&window[image][2],
                     &window[image][3],&access,&dtype[image],opt,&one);
   if (status != E_SUCC)
      {
      closefile(image - 1);
      c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
      }

   if (strcmp(minmax,"YES") == 0) 		 /* Initialize min/max arrays */
      {
      status = c_inmnmx(&dtype[image],&nbands[image],minval,maxval);
      if (status != E_SUCC)
         {
         closefile(image - 1);
         c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
         }
      }
   }

/* Open the drop line file
 -----------------------*/
if (drfile[0] != NULL)
   {
   if ((fd = fopen(drfile,"r")) == NULL)
      {
      c_errmsg("Error opening line drop file","linedrop-open",&nonfatal);
      closefile(image - 1);
      c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
      }

   /* Get first line of the drop line file--number of blocks and total lines
      ---------------------------------------------------------------------*/
   fscanf(fd,"%d %d", nblks, &tlines);

   /* Check for too many blocks
      -------------------------*/
   if (*nblks >= MAX_BLOCKS)
      {
      c_errmsg("Too many dropped line correction values","linedrop-line",
								&nonfatal);
      closefile(image - 1);
      c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
      }

   /* Read in the values from the drop line file
      -----------------------------------------*/
   for (i = 0; i < *nblks; i++)
      {
      if (fscanf(fd,"%d %d",&lines[i],&nbrln[i]) != 2)
	 {
	 c_errmsg("Mismatched values in dropped line file",
     					"linedrop-nbrln",&nonfatal);
	 closefile(image - 1);
	 c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
	 } 
      }
   }		/* End of processing of drop line file  */

c_inittm("LINEDROP","Processed","line","bands"," ",&window[0][2],
         totbnd,msgtim,&msg_flag);

curent = 1;
curcat = 1;
c_startm();

return;
}
