/*******************************************************************************
NAME:   		     ZEROFIL 

PURPOSE    To replace drop lines that have been fixed back to zero when
	   the number of lines in the dropline exceeds MAXLFIX.

PROGRAM HISTORY:
VERSION  PROGRAMMER   DATE   CODE/CONT  REASON
-------  -----------  -----  ---------  ------
  1.0    S. Nelson    01/82  EDC        Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
    ZEROFIL must be run under TAE.

PROJECT			LAS

NOTE:  Although this function currently works with only one image, the image
       loop is used.  This will help if multiple images are allowed in the
       future.

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "segmrepr.h"           /* to retrieve segmrepr constants             */

#define MAX_BLOCKS 1000		/* Maximum number of blocks in drop line file */

/* The following variables must be declared global for image I/O routines
------------------------------------------------------------------------------*/
extern int  fdesc[];	        /* image file descriptors used for image I/O  */

FUNCTION zerofil(vblock, hostname, bands, nbands, odtype, size, lines, nbrln,
					dpblk, update, window)

   struct PARBLK *vblock;	/*  TAE parameter block			      */

   char hostname[][CMLEN];	/* host names of images	to be repaired        */

   long	bands[][MAXBND+1];	/* bands to be processed for all images       */
   long dpblk;			/* Dropline section which is to be replaced   */
   long	lines[MAX_BLOCKS];	/* Drop lines in the image		      */
   long nbands[MAXIMG];		/* Number of bands per input image	      */
   long	nbrln[MAX_BLOCKS];	/* Number of lines in a block of drop lines   */
   long odtype;                 /* data type of images to be repaired         */
   long size;                   /* number of bytes required for this odtype   */
   long update;                 /* flag to modify input images                */
   long	window[][4];		/* window specification of input images       */

{

long    actual_band;          /* actual band # of current band within image   */
long    current_band;         /* current band of image that's being processed */
long	drpbl;		      /* bottom line of data gap	  	      */
long	drpsl;		      /* starting line of data gap		      */
long	fix_line;	      /* current line to be fixed		      */
long    fatal = -1;           /* fatal error flag                             */
long	image;                /* loop control                                 */
long	nmrimg;		      /* number of images changed		      */
long	nonfatal = 1;         /* value of one                                 */
long 	ns;                   /* number of image samples                      */
long    one = 1;              /* one                                          */
long    sample;               /* sample indicator                             */
long	status;               /* function return status code                  */
long	zero = 0;	      /* The value of zero			      */

unsigned char *thebuf;        /* image input buffer                           */

/*------------------------------------------------------------------------*/
ns = window[0][NS];
drpsl = lines[dpblk];
drpbl = lines[dpblk] + nbrln[dpblk] -1;
nmrimg = 1;

/* Allocate memory
------------------*/
if ((thebuf = (unsigned char *) malloc(ns*size)) == NULL)
   {
   closefile(nmrimg);
   c_errmsg("Error allocating dynamic memory","linedrop-alloc",&nonfatal);
   if (update)
      c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
   else
      c_clean(vblock,"linedrop",hostname,&one);
   }

/* For each of the images that needs to be updated
--------------------------------------------------*/
for (image = 0; image < nmrimg; image++)
   {

   /* Repair each of the input bands for the current image
   -------------------------------------------------------*/
   for (current_band = 0; current_band < nbands[image]; current_band++)
      {
      actual_band = bands[image][current_band];

      /* For each line of data to be zero filled, change the current band
      ------------------------------------------------------------------*/
      for (fix_line = drpsl; fix_line <= drpbl; fix_line++)
	 {
	 switch (odtype)
	    {
	    case EBYTE:
		for (sample = 0; sample < ns; sample++)
		   *(thebuf + sample) = zero; 
		break;
	    case EWORD:
		for (sample = 0; sample < ns; sample++)
		   *(short *)(thebuf + (sample * size)) = zero;
		break;
	    case ELONG:
		for (sample = 0; sample < ns; sample++)
		   *(long *)(thebuf + (sample * size)) = zero;
		break;
	    case EREAL:
		for (sample = 0; sample < ns; sample++)
		   *(float *)(thebuf + (sample * size)) = zero;
		break;
	    } /* switch (odtype) */

	 /* Write out the lines with fixed windows
	 -----------------------------------------*/
	 status=c_ewrite(&fdesc[image],&actual_band,&fix_line,thebuf,&one);
	 if (status == E_FAIL)
	    {
	    free(thebuf);
	    closefile(nmrimg);
	    if (update)
		c_errmsg("Fatal error encountered","linedrop-fatal",&fatal);
	    else
		c_clean(vblock,"linedrop",hostname,&one);
	    }
	 } /* for each line of the drop line section */
      } /* for each band of the current image */
   } /* for each image */

/* Free buffers
---------------*/
free(thebuf);
return;
}
