/*******************************************************************************
NAME:		LLRANGE

PURPOSE:	Calculate the minimum and maximum latitude and longitude
		of an image

PROGRAM HISTORY:
Version      Date    Programmer		Request
-------      ----    ----------         -------
  5.0        02/93   T. Rockvam         Original development; taken mostly from
					WDB2RAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	Must be run under TAE

ALGORITHM REFERENCES:  see WDB2RAS

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "llrange.h"

main()
{
struct	PARBLK	vblock;	/* TAE parameter block				*/
struct  DDR     ddr;    /* DDR information                              */

long	std_out,	/* standard output				*/
	status,		/* return status variable			*/
	nonfatal = 1,	/* nonfatal error code				*/
	fatal = -1,	/* fatal error code				*/
	zero = 0;	/* zero output files				*/

char	hosin[1][CMLEN],  /* input host file				*/
	taeout[1][CMLEN], /* output tae file name                	*/
	msg[CMLEN],	/* informational message			*/
	proj[6];	/* projection type (satellite or reprojected)	*/

float	minlat = 0.0,	/* minimum latitude				*/
	maxlat = 0.0,	/* maximum latitude				*/
	minlon = 0.0,	/* minimum longitude				*/
	maxlon = 0.0;	/* maximum longitude				*/

TAEFLOAT point[1];	/* coordinate in degrees			*/

/*
  Initialize the TAE parameter block
*/
c_inlas(&vblock,&std_out);

/*
  Get the input parameters
*/
getpar(&vblock,&ddr,hosin,proj);

/*
  Get the min and max lat and long
*/
if (strcmp(proj,"SAT") == 0)
    status = satminmax(hosin[0],&minlat,&minlon,&maxlat,&maxlon);
else
    status = projminmax(&ddr,&minlat,&minlon,&maxlat,&maxlon);

if (status != E_SUCC)
    c_errmsg("Fatal error encountered","llrange-fatal",&fatal);

/*
  Return the minimum latitude to the parameter block
*/
point[0] = minlat;
q_real(&vblock,"MINLAT",1,point,P_UPDATE);

/*
  Return the maximum latitude to the parameter block
*/
point[0] = maxlat;
q_real(&vblock,"MAXLAT",1,point,P_UPDATE);

/*
  Return the minimum longitude to the parameter block
*/
point[0] = minlon;
q_real(&vblock,"MINLONG",1,point,P_UPDATE);

/*
  Return the maximum longitude to the parameter block
*/
point[0] = maxlon;
q_real(&vblock,"MAXLONG",1,point,P_UPDATE);

/*
  Send the variable block back to TAE
*/
q_out(&vblock);

/*
  Issue completion message
*/
sprintf(msg,"Latitude range %4.2f to %4.2f",minlat,maxlat);
c_errmsg(msg,"informational",&nonfatal);
sprintf(msg,"Longitude range %4.2f to %4.2f",minlon,maxlon);
c_errmsg(msg,"informational",&nonfatal);
c_complt("LLRANGE",taeout,&zero);
return E_SUCC;
}
