/*******************************************************************************
FUNCTION:	advance_buf

PURPOSE:   	Write the current line in the line detection buffer to the
		output image and read the next line from the input image.

		This function will return E_SUCC upon successful completion,
		E_EOF if there is no more data in the input image, or it will
		call errout() for any other errors.

*******************************************************************************/
#include <stdio.h>
#include "acqio.h"
#include "lndetect.h"


extern long bands_all[]; /* All bands in an AVHRR archive image               */


FUNCTION advance_buf (line, buffer)

    long line;			/* IN     - Current line in the buffer        */
    struct BUFFER *buffer;	/* IN/OUT - Buffer for line detection         */
{
char str[CMLEN];	/* Temporary working string                           */

double mean;		/* Mean pixel value for a line                        */
double variance;	/* Variance in pixel values for a line                */

long dtype    = EWORD;	/* Data type of AVHRR image data                      */
long fill     = TRUE;	/* Fill flag for c_bip2bil()                          */
long line2read;		/* Line number of the next line to read               */
long maxsam = MAX_ACQ;	/* maximum number of samples in acquire		      */
long nonfatal =  1;	/* Nonfatal error flag                                */
long nvalues;		/* Number of values for the Minor Frame data          */
long status;		/* Function return status                             */
long zero = 0;		/* Zero constant value				      */

short *img;		/* Ptr to buffer of image data                        */
short *mnr;		/* Ptr to buffer of Minor Frame data                  */
short *tmp_img;		/* Temporary ptr to buffer of image data              */
short *tmp_mnr;		/* Temporary ptr to buffer of Minor Frame data        */
static short mnrzero[MAX_ACQ];  /* minor frame for missing lines              */



/* Get a pointer to the current line in the line detection buffer
-----------------------------------------------------------------*/
get_line(line, 1, buffer, &mnr, &img, &mean, &variance);


/* If the line is a dropped line then write the zero'd BIP buffer.  Otherwise,
   convert the BIL buffer to BIP and write that to the output image.
-----------------------------------------------------------------------------*/
if (buffer->drop_line[line] == BAD)
    {
    /* Set up the minor frame for a dropped line without a time stamp code */
    c_pxfill(mnrzero,&dtype,&maxsam,&zero);

    status = c_arcwrite(&buffer->outdesc, mnrzero, buffer->zero);
    }
else
    {
    c_bil2bip(img, buffer->bip, &buffer->ns, bands_all, &fill);
    status = c_arcwrite(&buffer->outdesc, mnr, buffer->bip);
    }
if (status != E_SUCC)
    errout();


/* Retrieve the buffer locations where the Minor Frame and image
   data are to be stored for the next line of the input image.
----------------------------------------------------------------*/
line2read = line + 1 + HALFLINES;
get_line(line2read, 1, buffer, &mnr, &img, &mean, &variance);


/* Read the next line from the input image
------------------------------------------*/
status = c_arcread(&buffer->indesc, &tmp_mnr, &tmp_img);
if (status == E_EOF)
    return(E_EOF);
if (status != E_SUCC)
    errout();


/* Copy the minor frame data to the line detection buffer
---------------------------------------------------------*/
nvalues = MNR_WORDS;
c_pxcopy(tmp_mnr, mnr, &dtype, &nvalues);


/* Convert the new line from BIP to BIL
---------------------------------------*/
status = c_bip2bil(tmp_img, img, &buffer->ns, bands_all, &fill);
if (status != E_SUCC)
    errout();

return(E_SUCC);
}
