/*******************************************************************************
FUNCTION:	check_bad

PURPOSE:   	Check the specified current line with a previous bad line.  If
		the current line looks like the bad line then return BAD,
		otherwise return GOOD.

		If the line looks different in any one of the bands being used
		for line detection then the line will be considered good.

*******************************************************************************/
#include "lndetect.h"

extern double THRESHOLD_MIN[];
extern long bands_det[];
extern long drop_bad[NCHANS];

FUNCTION check_bad (current, previous, buffer)

    long current;		/* IN - Current line in the buffer            */
    long previous;		/* IN - Previous bad line in the buffer       */
    struct BUFFER *buffer;	/* IN - Buffer for line detection             */
{
char str[CMLEN];	/* Temporary string                                   */

double delta_mean;	/* Absolute difference of line mean values            */
double mean_cur;	/* Mean pixel value for the current line              */
double mean_prev;	/* Mean pixel value for the previous line             */
double threshold;	/* Threshold value                                    */
double var_cur;		/* Variance for the current line                      */
double var_prev;	/* Variance for the previous line                     */

long i;			/* Loop control                                       */
long nonfatal = 1;	/* Nonfatal error flag                                */

short *img;		/* Ptr to line of image data                          */
short *mnr;		/* Ptr to Minor Frame data                            */



/* Check each of the bands that are being used for line detection
-----------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    if (bands_det[i])
	{
	/* Get the mean and variance values for the current
	   and previous lines within the specified band.
	---------------------------------------------------*/
	get_line(current,bands_det[i],buffer,&mnr,&img,&mean_cur,&var_cur);
	get_line(previous,bands_det[i],buffer,&mnr,&img,&mean_prev,&var_prev);


	/* Compute the threshold value for the line means
	   and the difference in the line means.
	-------------------------------------------------*/
	threshold = ZVALUE * sqrt((var_cur + var_prev) / (double)buffer->ns);
	delta_mean = fabs(mean_cur - mean_prev);


	/* If the threshold value is larger than the minimum value then
	   compare the difference of the means to the threshold value.
	   If the threshold value is too low then do not use this band
	   in the comparison.
	----------------------------------------------------------------*/
	if ((threshold > THRESHOLD_MIN[i]) && (delta_mean > threshold))
	    return(GOOD);
	}


/* The difference in line mean values was not large enough in any band to
   consider the current line good.  Increment the bad line counter under
   the first band used for line detection
--------------------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    if (bands_det[i])
	{
	drop_bad[i] += 1;
	break;
	}
return(BAD);
}
