/*******************************************************************************
FUNCTION:	check_good

PURPOSE:   	Check the specified current line with a previous good line.  If
		the current line looks like the good line then return GOOD,
		otherwise return BAD.

		If the line looks different in any one of the bands being used
		for line detection then the line will be considered bad.

*******************************************************************************/
#include "lndetect.h"

extern double THRESHOLD_MIN[];
extern long bands_det[];
extern long drop_good[NCHANS];


FUNCTION check_good (current, previous, buffer)

    long current;		/* IN - Current line in the buffer            */
    long previous;		/* IN - Previous bad line in the buffer       */
    struct BUFFER *buffer;	/* IN - Buffer for line detection             */
{
char str[CMLEN];	/* Temporary string                                   */

double delta_mean;	/* Absolute difference of line mean values            */
double mean_cur;	/* Mean pixel value for the current line              */
double mean_prev;	/* Mean pixel value for the previous line             */
double threshold;	/* Threshold value                                    */
double var_cur;		/* Variance for the current line                      */
double var_prev;	/* Variance for the previous line                     */

long i;			/* Loop control                                       */
long nonfatal = 1;	/* Nonfatal error flag                                */

short *img;		/* Ptr to line of image data                          */
short *mnr;		/* Ptr to Minor Frame data                            */


/* Check each of the bands that are being used for line detection
-----------------------------------------------------------------*/
for (i = 0; i < NCHANS; i++)
    if (bands_det[i])
	{
	/* Get the mean and variance values for the current
	   and previous lines within the specified band.
	---------------------------------------------------*/
	get_line(current,bands_det[i],buffer,&mnr,&img,&mean_cur,&var_cur);
	get_line(previous,bands_det[i],buffer,&mnr,&img,&mean_prev,&var_prev);


	/* Compute the threshold value for the line means
	   and the difference in the line means.
	-------------------------------------------------*/
	threshold = ZVALUE * sqrt((var_cur + var_prev) / (double)buffer->ns);
	delta_mean = fabs(mean_cur - mean_prev);


	/* If the threshold value is larger than the minimum value and the
	   difference in the mean values is greater than the threshold value
	   then assume the line is bad.  If the threshold value is too small
	   then do not use this band in the comparison.
	--------------------------------------------------------------------*/
	if ((threshold > THRESHOLD_MIN[i]) && (delta_mean > threshold))
	    {
	    drop_good[i] += 1;
	    return(BAD);
	    }
	}


/* The difference in line mean values was not large
   enough to consider the current line bad.
---------------------------------------------------*/
return(GOOD);
}
