/*******************************************************************************
FUNCTION:	check_outfile

PURPOSE:   	If the output file name is empty then create a name using the
		scene ID and a time stamp.

*******************************************************************************/
#include "lndetect.h"

extern char outfile[];		/* IN/OUT - Host name of the output image     */


FUNCTION check_outfile (model)

    struct MODEL *model;	/* IN - AVHRR model structure                 */
{
char dir[CMLEN];	/* Directory portion of a file name                   */
char ext[CMLEN];	/* Extension portion of a file name                   */
char root[CMLEN];	/* Root portion of a file name                        */
char sceneid[CMLEN];	/* Scene ID of the input image                        */
char tstamp[7];		/* Current time stamp                                 */

long exists    = 0;	/* Test for existence when using c_lsstat()           */
long time_val;		/* Time variable                                      */

short done;		/* Loop control                                       */

struct tm *lctm;	/* Local time structure                               */


/* Split the file name apart to see whats there.  It is assumed that getpar()
   will ensure there is either just a directory path or a full file name.
-----------------------------------------------------------------------------*/
c_splitname(outfile, dir, root, ext);


/* If there is an extension then return
---------------------------------------*/
if (strlen(ext))
    return(E_SUCC);


/* Get the scene ID from the model
----------------------------------*/
c_getscene(model, sceneid);


/* Loop until we find a time stamp that will result in a unique file name
-------------------------------------------------------------------------*/
done = FALSE;
while (!done)
    {
    time(&time_val);
    lctm = localtime(&time_val);
    sprintf(tstamp,"%02d%02d%02d", lctm->tm_hour, lctm->tm_min, lctm->tm_sec);

    sprintf(outfile, "%s%s_%s.arch", dir, sceneid, tstamp);
    if (c_lsstat(outfile, &exists) == E_SUCC)
	sleep(2);
    else
	done = TRUE;
    }
return(E_SUCC);
}
