/*******************************************************************************
FUNCTION:	examine_line

PURPOSE:   	Examine the specified line in the line detection buffer to
		determine if it is a good or bad line.

		Return the number of line that should be skipped in the line
		detection buffer.

*******************************************************************************/
#include "lndetect.h"



FUNCTION examine_line (line, buffer)

    long line;			/* IN - Current line in the buffer            */
    struct BUFFER *buffer;	/* IN - Buffer for line detection             */
{
long i;			/* Loop control                                       */
long nonfatal = 1;	/* Nonfatal error flag                                */

short good;		/* Loop control                                       */


/* If the current line is already bad then return
-------------------------------------------------*/
if (buffer->drop_line[line] == BAD)
    return(0);


/* Check each of the previous HALFLINES lines in the line detection buffer
--------------------------------------------------------------------------*/
for (i = 1; i <= HALFLINES; i++)
    {
    /* If a good line is found then compare it to the current line
    --------------------------------------------------------------*/
    if (buffer->drop_line[line - i] == GOOD)
	{
	/* If the line is determined to be bad then mark it as such
	-----------------------------------------------------------*/
	if (check_good(line, line - i, buffer) == BAD)
	    buffer->drop_line[line] = BAD;
	else if (check_pixels(line, i, buffer) == BAD)
	    buffer->drop_line[line] = BAD;


	/* The line has been checked, return zero so no lines are skipped
	-----------------------------------------------------------------*/
	return(0);
	}
    }


/* All of the previous HALFLINES lines are considered bad.  Compare the
   current line with each of these lines.  If the current line is different
   from ALL of these bad lines, consider it a good line.  If the current
   line is the same as any of the bad lines then consider the line bad and
   reset the algorithm by assuming the next line that is not marked as bad,
   is good.
----------------------------------------------------------------------------*/
for (i = 1, good = TRUE; (i <= HALFLINES) && good; i++)
    if (check_bad(line, line - i, buffer) == BAD)
	good = FALSE;
if (good)
    return(0);


/* The current line looks the same as one of the previous bad lines.  This
   line must be marked as bad and the next line must be forced to be good.
   NOTE: Watch out for lines that were dropped by the ingest procedure.  These
         lines must remain bad.  Therefore, the next non-dropped line must be
	 forced to be good.
------------------------------------------------------------------------------*/
buffer->drop_line[line] = BAD;
i = line + 1;
while ((buffer->drop_line[i] == BAD) && (i <= buffer->nl - HALFLINES))
    i++;
return(i - line);
}
