/*******************************************************************************
FUNCTION:	get_line

PURPOSE:   	Given a line number and a band number from the input image,
		return a pointer to the Minor Frame and image data for that
		line and band.

		Since the data is stored in the buffer in BIL format, when band
		one is requested the image data pointer that is returned
		actually points to all of the image data.

*******************************************************************************/
#include "lndetect.h"

extern long bands_det[]; /* Bands used for bad line detection                 */


FUNCTION get_line (line, band, buffer, mnr, img, mean, variance)

    long line;			/* IN  - Current line in the buffer           */
    short band;			/* IN  - Band number desired                  */
    struct BUFFER *buffer;	/* IN  - Buffer for line detection            */
    short **mnr;		/* OUT - Ptr to the Minor Frame data          */
    short **img;		/* OUT - Ptr to the image data                */
    double *mean;		/* OUT - Mean pixel value for the line        */
    double *variance;		/* OUT - Variance in pixel values for the line*/
{
short lindex;			/* Line index                                 */
short bindex;			/* Band index                                 */


/* Determine the indices for the requested line and band
--------------------------------------------------------*/
lindex = (line - 1) % NBUFLINES;
bindex = band - 1;


/* Return the appropriate pointers
----------------------------------*/
*mnr = buffer->mnr[lindex];
*img = buffer->img[lindex] + (buffer->ns * bindex);


/* Return the statistics if this is a band used in the line detection
---------------------------------------------------------------------*/
if (bands_det[bindex])
    {
    *mean = buffer->mean[bindex][line];
    *variance = buffer->variance[bindex][line];
    }
else
    {
    *mean = 0.0;
    *variance = 0.0;
    }

return(E_SUCC);
}
