#include <stdio.h>
#include <time.h>
#include <math.h>
#include "las.h"
#include "tapeio.h"
#include "adaps.h"
#include "archio.h"


/* Constants to indicate good and bad lines and pixel values
------------------------------------------------------------------------------*/
#define BAD              1
#define GOOD             0


/* Constants for the number of image lines that are stored in the line
   detection buffer (must be odd) and for one half of the lines held in the
   buffer.
------------------------------------------------------------------------------*/
#define HALFLINES        2
#define NBUFLINES        5


/* Statistical value taken from a standard normal distribution table ensuring a
   confidence of 99.9%.  This value indicates that in AVHRR data the mean of
   the pixel values on one line will be within a certain margin of the mean of
   the previous line, 99.9% of the time.
------------------------------------------------------------------------------*/
#define ZVALUE        3.27



/* Within the following "line detection buffer" the fields drop_line, mean, and
   variance are pointers to dynamically allocated arrays.  These arrays will be
   indexed by the line numbers 1, 2, 3, ...  The zero'th elements are NOT used.
------------------------------------------------------------------------------*/
struct BUFFER
    {
    char *drop_line;		/* Flag for each image line  0=good 1=dropped */
    char *drop_pixel;		/* Flag for each pixel 0=good 1=dropped       */

    double *mean[NCHANS];	/* Mean value of each line within each band   */
    double *variance[NCHANS];	/* Variance of each line within each band     */

    long indesc;		/* AVHRR descriptor for input image           */
    long nl;			/* Number of lines in the image               */
    long ns;			/* Number of samples in the image             */
    long outdesc;		/* AVHRR descriptor for output image          */

    short *bip;			/* Buffer for converting from BIL to BIP      */
    short *img[NBUFLINES];	/* Array pointing to BIL buffers for each line*/
    short *mnr[NBUFLINES];	/* Array pointing Minor Frame for each line   */
    short *zero;		/* Buffer of zero'd BIP image data            */
    };
