/*******************************************************************************
FUNCTION:	open_images

PURPOSE:   	Open the input and output images.

*******************************************************************************/
#include <string.h>
#include "lndetect.h"

extern char infile[];		/* IN  - Host name of the input image         */
extern char outfile[];		/* IN  - Host name of the output image        */


FUNCTION open_images (tapeid, outhost, outlas, nfiles, buffer, drop, model)

    char *tapeid;		/* IN  - Tape ID if the image is on tape      */
    char outhost[][CMLEN];	/* OUT - Host names of output files created   */
    char outlas[][CMLEN];	/* OUT - LAS names of output files created    */
    long *nfiles;		/* OUT - Number of output files created       */
    struct BUFFER *buffer;	/* OUT - Line detection buffer structure      */
    struct DROPPED *drop;	/* OUT - AVHRR dropped line structure         */
    struct MODEL *model;	/* OUT - AVHRR model structure                */
{
char comment[CMLEN];	/* Tape mounting comment                              */

long access   = IREAD;	/* Access for opening the input image                 */
long status;		/* Function return status                             */
long tpunit   = NULL;	/* Tape drive descript if image is on tape            */
long window[4];		/* Window for the input image                         */



/* Initialize the archive descriptor records
   before any calls are made to errout().
--------------------------------------------*/
buffer->indesc = NULL;
buffer->outdesc = NULL;


/* Get the header for the input image
-------------------------------------*/
if (strlen(tapeid))
    {
    sprintf(comment, "Please mount AVHRR archive tape %s", tapeid);
    status = c_tpgetahdr(infile, comment, &tpunit, model, drop);
    }
else
    status = c_getahdr(infile, model, drop);
if (status != E_SUCC)
    errout();


/* Due to the fact that c_arcwrite() will write each image line in the order
   that we send them to it, we must process the input image in satellite view.
   If we process the input image in "North-up" order then the output image will
   be flipped incorrectly for all ascending images.
------------------------------------------------------------------------------*/
model->sat_view = TRUE;


/* Open the input image
-----------------------*/
window[SL] = 0;
status = c_arcopnrd(infile, &access, window, model, &tpunit, &buffer->indesc);
if (status != E_SUCC)
    errout();


/* Open the output image
------------------------*/
check_outfile(model);
status = c_arcopnwrt(outfile, &model->dtype, &model->nlines, &buffer->outdesc);
if (status != E_SUCC)
    errout();


/* Add the output image file names to the global variables
----------------------------------------------------------*/
strcpy(outhost[0], outfile);
c_lsmknm(outfile, "ahdr", outhost[1]);
strcpy(outlas[0], outhost[0]);
strcpy(outlas[1], outhost[1]);
*nfiles = 2;


return(E_SUCC);
}
