/*******************************************************************************
FUNCTION:	print_stats

PURPOSE:   	Print the statistics about which pixels have been zeroed and
		which lines have been dropped.

*******************************************************************************/
#include "lndetect.h"


extern long drop_good[];
extern long drop_bad[];
extern long drop_adj[];
extern long drop_badkernel[];
extern long drop_zerokernel[];
extern long drop_total[];
extern long drop_reset[];
extern long drop_pixels[];
extern long bands_det[];
extern long bands_spt[];
extern char infile[];
extern struct BUFFER bbuffer;


FUNCTION print_stats ()

{
char bands[20];		/* Bands the line detection algorithm was run on      */
char hostname[CMLEN];	/* Host name of the output file                       */
char root[CMLEN];	/* Root portion of a file name                        */
char str[CMLEN];	/* Temporary string                                   */

double percent;		/* Percentage of pixels zeroed                        */

long i;			/* Loop control                                       */
long nonfatal = 1;	/* Nonfatal error flag                                */
long total;		/* Variable to total the number of pixels and lines   */



/* Write the image name and the bands used for detection
--------------------------------------------------------*/
strcpy(bands, "");
for (i = 0; i < NCHANS; i++)
    {
    if ((bands_det[i]) && (strlen(bands) == 0))
	sprintf(bands, "%d", i+1);
    else if (bands_det[i])
	{
	sprintf(str, ", %d", i+1);
	strcat(bands, str);
	}
    }
c_errmsg(" ", " ", &nonfatal);
c_splitname(infile, str, root, str);
sprintf(str, "Image: %s", root);
c_errmsg(str, " ", &nonfatal);
sprintf(str, "Detection Bands: %s", bands);
c_errmsg(str, " ", &nonfatal);
c_errmsg(" ", " ", &nonfatal);


/* Write the information about the pixels that were zeroed
----------------------------------------------------------*/
total = 0;
for (i = 0; i < NCHANS; i++)
    total += drop_pixels[i];
percent = (double)total / (double)(bbuffer.nl * bbuffer.ns * NCHANS) * 100.0;
sprintf(str,"%d pixels zeroed for spot noise (%4.2lf%s)", total, percent, "%%");
c_errmsg(str, " ", &nonfatal);

for (i = 0; i < NCHANS; i++)
    {
    if (bands_spt[i])
	{
	sprintf(str, "   Band %d: %7d", i+1, drop_pixels[i]);
	c_errmsg(str, " ", &nonfatal);
	}
    }
c_errmsg(" ", " ", &nonfatal);


/* Write the information about the lines that were dropped
----------------------------------------------------------*/
total = 0;
for (i = 0; i < NCHANS; i++)
    {
    total += drop_good[i];
    total += drop_bad[i];
    total += drop_adj[i];
    total += drop_badkernel[i];
    total += drop_zerokernel[i];
    total += drop_total[i];
    total += drop_reset[i];
    }
if (total == 0)
    {
    sprintf(str, "No bad lines found", total);
    c_errmsg(str, " ", &nonfatal);
    return(E_SUCC);
    }
else
    {
    sprintf(str, "%d bad lines dropped", total);
    c_errmsg(str, " ", &nonfatal);
    }

sprintf(str, "           Good   Bad    Adj    ZKern  BKern  Total  Reset");
c_errmsg(str, " ", &nonfatal);

sprintf(str, "           -----  -----  -----  -----  -----  -----  -----");
c_errmsg(str, " ", &nonfatal);

for (i = 0; i < NCHANS; i++)
    {
    if (bands_det[i])
	{
	sprintf(str, "   Band %d: %5d  %5d  %5d  %5d  %5d  %5d  %5d", i+1,
	    drop_good[i], drop_bad[i], drop_adj[i], drop_zerokernel[i],
	    drop_badkernel[i], drop_total[i], drop_reset[i]);
	c_errmsg(str, " ", &nonfatal);
	}
    }
c_errmsg(" ", " ", &nonfatal);
return(E_SUCC);
}
