/*******************************************************************************
FUNCTION:	save_ahdr

PURPOSE:   	Update the dropped line information for the output image and
		than save the model and dropped line structures in the AVHRR
		archive header file.

*******************************************************************************/
#include "lndetect.h"

extern char outfile[];		/* IN - Host name of the output image         */

FUNCTION save_ahdr (buffer, model, drop)

    struct BUFFER *buffer;	/* IN - Line detection buffer                 */
    struct MODEL *model;	/* IN - AVHRR model structure                 */
    struct DROPPED *drop;	/* IN - AVHRR dropped line structure          */
{
char str[CMLEN];	/* Temporary working string                           */

long i;			/* Loop control                                       */
long nonfatal =  1;	/* Nonfatal error flag                                */
long status;		/* Function return status                             */


/* Store all of the dropped lines in the dropped line structure.  Note that
   buffer->drop_line is not a zero relative array.  Thus, if
   buffer->drop_line[37] is set then line 37 is a dropped line.
------------------------------------------------------------------------------*/
i = 1;
drop->ngaps = 0;
while (i <= model->nlines)
    {
    /* Is this the start of a gap?
    ------------------------------*/
    if (buffer->drop_line[i] == GOOD)
	i++;
    else
	{
	/* Do not exceed the maximum number of dropped line gaps
	--------------------------------------------------------*/
	if (drop->ngaps == MAX_GAPS)
	    {
	    sprintf(str, "Exceeded the maximum of %d dropped line gaps",
		MAX_GAPS);
	    c_errmsg(str, "lndetect-ngaps", &nonfatal);
	    errout();
	    }


	/* Process the entire gap
	-------------------------*/
	drop->lmissed[drop->ngaps] = i;
	drop->nmissed[drop->ngaps] = 0;
	while ((i <= model->nlines) && (buffer->drop_line[i] == BAD))
	    {
	    drop->nmissed[drop->ngaps] += 1;
	    i++;
	    }
	drop->ngaps++;
	}
    }


/* Create the header file for the output image
----------------------------------------------*/
status = c_putahdr(outfile, model, drop);
if (status != E_SUCC)
    errout();
return(E_SUCC);
}
