/*******************************************************************************
NAME:           DROP_ADD

PURPOSE:        Add the dropped line info from the input image to the 
		dropped line info of the output image

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    ------          ------
  2.0           8/92    T. Rockvam     	Original Development

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"

FUNCTION drop_add(indrop,outdrop,in_startline,line_offset)

struct DROPPED *indrop;			/* dropped line structure	*/
struct DROPPED *outdrop;		/* dropped line structure	*/

long	in_startline,			/* start line of input image 	*/
	line_offset;			/* diff between output,input lines*/
{
long 	ingap,				/* input gap counter		*/
	outgap,				/* output gap counter		*/
	nonfatal = 1;			/* nonfatal error flag		*/

/*
  If input dropped info is empty, just return
*/
if (indrop->ngaps == 0)
    return;

/*
  Find the first gap in the input dropped structure that 
  falls after in_startline
*/
ingap = 0;
while (indrop->lmissed[ingap] < in_startline)
    {
    ingap++;
    if (ingap > indrop->ngaps)
	return;
    }

/*
  Set the gap back to the one just before in_startline
  if it surrounds in_startline and copy a partial gap
   e.g., ---start of gap
          --in_startline
         ---end of gap
*/
outgap = outdrop->ngaps;
if ((ingap > 0) && 
    (indrop->lmissed[ingap-1] + indrop->nmissed[ingap-1] - 1 >= in_startline))
    {
    ingap--;
    outdrop->ngaps += 1;

    /*
      If drop structure is full, do not add any more
    */
    if (outdrop->ngaps > MAX_GAPS)
	{
    	outdrop->ngaps -= 1;
	c_errmsg("Cannot add any more dropped lines to output image",
		 "makorb-addrop",&nonfatal);
	return;
	}

    outdrop->lmissed[outgap] = in_startline + line_offset;
    outdrop->nmissed[outgap] = indrop->lmissed[ingap-1] + 
			       indrop->nmissed[ingap-1] - in_startline;
    outgap++;
    ingap++;
    }

/*
  Add the remaining input dropped line structure to the output
*/
for (;ingap < indrop->ngaps; ingap++,outgap++)
    {
    /*
      Found a gap in input starting beyond in_startline 
    */
    outdrop->ngaps += 1;

    /*
      If drop structure is full, do not add any more
    */
    if (outdrop->ngaps > MAX_GAPS)
	{
    	outdrop->ngaps -= 1;
	c_errmsg("Cannot add any more dropped lines to output image",
		 "makorb-addrop",&nonfatal);
	return;
	}

    outdrop->lmissed[outgap] = indrop->lmissed[ingap] + line_offset;
    outdrop->nmissed[outgap] = indrop->nmissed[ingap];
    }

return;
}
