/****************************************************************************
NAME:		FILL_MISSING

PURPOSE:	Fill any missing lines with zero filled lines, and save
		the lines number of the missing lines in the dropped line
		structure.  The dropped line will NOT have a valid time
		stamp code.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		REASON
-------		----	------		------
  2.0		5/91	D. Hollaren	Conversion to LAS5.0
  2.1		9/91    T. Rockvam	Taken from INGEST for use in MAKORB
  2.2		4/93    T. Baltzer	Write drop line WITHOUT time stamp

ALGORITHM REFERENCE:
  See "NOAA Technical Memorandum NESS 107" for a detailed description
  of the HRPT Minor Frame Format.
  See fill_missing.c in the ADAPS application INGEST
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "acqio.h"
#include "archio.h"

FUNCTION fill_missing(arc_fd,ahdr_drop,curline,missing,zerobuf)

long		*arc_fd;	/* file descriptor to the archive file       */

struct DROPPED	*ahdr_drop;	/* dropped line index			     */

long	missing,		/* number of missing lines in a gap	     */
	curline;		/* current (first missing) line number	     */

short	*zerobuf;		/* fill for missing lines		     */
{
long	dtype = EWORD,          /* type of data being processed              */
	i,			/* looping variable			     */
	maxsam = MAX_ACQ,       /* maximum number of samples in acquire      */
	nonfatal = 1,		/* non-fatal error code			     */
	status,			/* return status variable		     */
	zero = 0;               /* constant zero                             */

static short mnrbuf[MAX_ACQ];  	/* minor frame for missing lines	     */


/*
  Save the location and number of dropped lines for the
  dropped line index file if it is not already in there
*/
if (!(is_dropped(curline,ahdr_drop)))
    {
    if (ahdr_drop->ngaps <= MAX_GAPS)
	{
	/* still room in dropped line structure */
    	ahdr_drop->lmissed[ahdr_drop->ngaps] = curline;
    	ahdr_drop->nmissed[ahdr_drop->ngaps] = missing;
    	ahdr_drop->ngaps++;
	}
    else
	{
	c_errmsg("Cannot add any more dropped lines to output image",
                 "makorb-addrop",&nonfatal);
	}
    }

/*
  Set up the minor frame for a dropped line
  without a time stamp code
*/
c_pxfill(mnrbuf,&dtype,&maxsam,&zero);

/*
  Fill the missing gap with zero fill records
*/
for (i = 0; i < missing; i++)
    {
    status = c_arcwrite(arc_fd,mnrbuf,zerobuf);
    if (status != E_SUCC)
        return(E_FAIL);
    }

return(E_SUCC);
}
