/*******************************************************************************
NAME:           GETPRI

PURPOSE:        Get the priority associated with a station (CEOS id).
		The priority.station table is searched for the CEOS id.
		The priority is read from the table, unless the CEOS id
		does not appear, in which case the priority is set to zero.
		If the file does not exist the priority is set to zero. 

*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "model.h"
#include "makorb.h"

#define BSIZ  80		/* buffer size				*/

FUNCTION getpri(pass,priority)

struct ORBIT    *pass;       	/* image information              	*/

long	*priority;		/* station priority			*/
{
FILE    *fp;

char    disk[CMLEN],            /* disk location of station id table    */
        pri_file[CMLEN],        /* station priority file name           */
	buffer[BSIZ],		/* buffer to read station id file with  */
	*gstatus,               /* status from fgets routine                 */
	sta_key[4];		/* station id key			*/

long    skip,			/* # of header records to skip 		*/ 
	i,			/* looping variable			*/
        nonfatal = 1,           /* non-fatal error code                 */
        status,             	/* function return status		*/
	tmp_pri;		/* priority read from the table		*/

/*
  Set up a 0 priority
*/
*priority = 0;

/*
  Get the station priority file name
*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(disk,(char *)getenv("ADAPSTABLES"));
else
    {
    c_errmsg("Error retrieving value of ADAPSTABLES","makorb-find",&nonfatal);
    return;
    }

#ifdef vms
    sprintf(pri_file,"%s%s",disk,"priority.stations");
#else
    sprintf(pri_file,"%s/%s",disk,"priority.stations");
#endif

/*
  Open the station priority file
*/
fp = fopen(pri_file,"r");
if (fp == NULL)
    return;

/*
  Get the number of header records in the table
*/
status = fscanf(fp,"%3d",&skip);
if (status < 1)
    {
    fclose(fp);
    c_errmsg("Error reading the station priority header","acqinfo-read",
	     &nonfatal);
    return;
    }

/*
  Skip over the header records
*/
for (i = 0; i < skip; i++)
    {
    gstatus = fgets(buffer,BSIZ,fp);
    if (gstatus == NULL)
	{
        fclose(fp);
        c_errmsg("Error reading the station priority header","acqinfo-read",
		 &nonfatal);
        return;
        }
    }

/*
  Read each line until a match to the station key is found
*/
while (status != EOF)
    {
    gstatus = fgets(buffer,BSIZ,fp);
    if (gstatus == NULL)
        {
        if (feof(fp) != 0)
            {
            status = EOF;
            continue;
            }
        else
	    {
	    fclose(fp);
	    c_errmsg("Error reading the station priority file","makorb-read",
		     &nonfatal);
	    return;
	    }
        }

    status = sscanf(buffer,"%3s %2d",sta_key,&tmp_pri);

    /*
      Check to see if the station number matches the specified key
      Break out if they match
    */
    if (strcmp(pass->model.station,sta_key) == 0)
	{
	*priority = tmp_pri;
	break;
	}
    }

/*
  Close the acquire station id file
*/
fclose(fp);

return;
}
