/*******************************************************************************
NAME:           TIME_LINE

PURPOSE:        Get the line number based on reference start time and time of 
		interest, assuming the reference time is line 1

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    ------          ------
  2.0           9/92    T. Rockvam     	Original Development

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"

FUNCTION time_line(ref_time,interest_time)

long	*ref_time,		/* reference time in milliseconds	*/
	interest_time;		/* time of interest in milliseconds	*/
{
long	line;			/* line number at interest_time		*/

double	mil_per_line,		/* # milliseconds per line		*/
	dline;			/* exact (double) line number		*/

/*
  Find number of milliseconds per line
*/
mil_per_line = (double)(MIL_PER_SEC / LAC_PER_SEC);

if (*ref_time == -1)
    {
    *ref_time = interest_time;
    line = 1;
    }
else
    {
    /*
      Calculate line number based on time difference
      add 1.0 for line relative to line 1; add 0.5 to round
    */
    dline = (interest_time - *ref_time) / mil_per_line + 1.0 + 0.5;
    line = (long)dline;
    }

return (line);
}
