#!/bin/csh -f
#
# This is a simple script to execute ADAPS daemons in the background.and save
# the name of the spawned process and its process ID in $ADAPSLOG/daemons.
# This is intended to be called from the ADAPS menus but can also be called
# from the command line.  Argument is one of the standard ADAPS daemons.
# Examples:
#
# start_daemon.csh ingest_daemon
# start_daemon.csh refaid_daemon "(SFL NGC NOA)"
# 
# Note: This script only works correctly when executed from a C-shell.  That
# means it can't be run from cron.  Correctly means that the daemon is
# executed *and* the PID is saved.  When executed from cron the $$ variable
# is not set and the my_child variable is not set and, obviously, neither is
# the tm_child variable.  Testing showed that executing from at, at least out
# of a C-shell, does work.  I've left some of the debug info at the bottom
# of the file for future work.
#---------------------------------------------------------------------------

# If no argument is specified exit with error
if ( ${#argv} < 1 ) then
    exit -1
endif

# If two arguments are specified set the 'a' variable
if ( ${#argv} == 2 ) then
    set a = $2
# Otherwise set 'a' to be empty
else
    set a = ""
endif
set Proc = $1

# If the user isn't set exit with error
if ( ! $?user ) then
    exit -1
endif

# Set up the LAS environment if not previously done.
# Note: this will have to change based on the system install
# For now just bail out if the environment isn't set up
if ( ! $?EDCSOFT ) then
    exit -1
    setenv EDCSOFT /edclxs64/data/adaps/las/build
    source $EDCSOFT/env/las/assign/lasenv.csh $EDCSOFT
endif

# By default execute all procedures in the log directory so that all messages
# are written to that directory.  Save the current directory and change to
# the log directory
set curdir = `pwd`
cd $ADAPSLOG

# Get a time stamp for the logs
set tstamp = `date '+%Y%m%d%H%M%S'`

# Execute the daemon in the background and give it a rest
echo "$Proc |stdout=${Proc}_$tstamp.log| $a" |  tae >& ${Proc}_$tstamp.stdout &
sleep 1

# We need to know the PID of the spawned taetm session.  It appears that a
# tae session is spawned first then a taetm session which is the actual
# daemon.  First get the tae child of this script
set my_child = `ps -fu $user | grep $$ | grep tae$ | awk '{print $2}'`
# Next get the taetm session spawned by the tae session
set tm_child = `ps -fu $user | grep taetm$ | grep $my_child | awk '{print $2}'`

# The tm_child variable is the most important thing to know but save both for
# future reference.
echo -n `date '+%F %T'` >> $ADAPSLOG/daemons
echo "|$Proc|$tm_child|$my_child" >> $ADAPSLOG/daemons

# Change back to original directory and exit
cd $curdir
exit 0

