/* Read a KLM level 1b data header and print out useful information
-------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>

#define BUFLEN 15872

void main( int argc, char *argv[] )
{
unsigned char buf[BUFLEN];
float *fptr;
int   *iptr;
short *sptr;
FILE  *inf;
char  str[256];
int   i, j;
int   bytes2skip;


if ( argc < 2 )
    {
    printf("\nUsage %s <input file> [bytes2skip]\n\n", argv[0]);
    exit(1);
    }
if ( argc == 3 )
    bytes2skip = atoi( argv[2] );
else
    bytes2skip = 0;

inf = fopen( argv[1], "r" );
if ( !inf )
    {
    printf("\nUnable to open file: %s", argv[1]);
    exit(1);
    }

fread( buf, sizeof( unsigned char ), bytes2skip, inf );
fread( buf, sizeof( unsigned char ), BUFLEN, inf );
fclose( inf );

sptr = (short *)&buf[5-1];
printf("Version: %d\n", *sptr);
sptr++;
printf("Version Year: %d\n", *sptr);
sptr++;
printf("Version Day: %d\n", *sptr);

for ( i = 0, j = 23-1; j < 65-1; j++, i++ )
    str[i] = buf[j];
str[i] = '\0';
printf("Data Set Name: %s\n", str);

for ( i = 0, j = 65-1; j < 73-1; j++, i++ )
    str[i] = buf[j];
str[i] = '\0';
printf("Processing Block: %s\n", str);

sptr = (short *)&buf[73-1];
printf("Version: %d", *sptr);
switch ( *sptr )
    {
    case 4 : printf(" = NOAA-15\n"); break;
    case 2 : printf(" = NOAA-16\n"); break;
    default : printf(" = unknown\n"); break;
    }
sptr++;
printf("Instrument ID: %d\n", *sptr);
sptr++;
printf("Data type: %d", *sptr);
switch ( *sptr )
    {
    case 1 : printf(" = LAC\n"); break;
    case 2 : printf(" = GAC\n"); break;
    case 3 : printf(" = HRPT\n"); break;
    case 4 : printf(" = TIP\n"); break;
    case 5 : printf(" = HIRS\n"); break;
    case 6 : printf(" = MSU\n"); break;
    case 7 : printf(" = SSU\n"); break;
    case 8 : printf(" = DCS\n"); break;
    case 9 : printf(" = SEM\n"); break;
    case 10 : printf(" = AMSU-A\n"); break;
    case 11 : printf(" = AMSU-B\n"); break;
    default : printf(" = unknown\n"); break;
    }

sptr++;
printf("TIP cource: %d", *sptr);
switch ( *sptr )
    {
    case 0 : printf(" = unused, GAC/LAC/HRPT data\n"); break;
    case 1 : printf(" = GAC embedded AMSU and TIP\n"); break;
    case 2 : printf(" = stored TIP\n"); break;
    case 3 : printf(" = HRPT/LAC embedded AMSU and TIP\n"); break;
    case 4 : printf(" = stored AIP\n"); break;
    default : printf(" = unknown\n"); break;
    }

iptr = (int *)&buf[81-1];
printf("Start of Data Set Day Count: %d\n", *iptr);
sptr = (short *)&buf[85-1];
printf("Start of Data Set Year: %d\n", *sptr);
iptr++;
printf("Start of Data Set Day of Year: %d\n", *sptr);
iptr = (int *)&buf[89-1];
printf("Start of Data Set UTC time ms: %d\n", *iptr);
iptr++;
printf("End of Data Set Day Count: %d\n", *iptr);
sptr = (short *)&buf[97-1];
printf("End of Data Set Year: %d\n", *sptr);
iptr++;
printf("End of Data Set Day of Year: %d\n", *sptr);
iptr = (int *)&buf[101-1];
printf("End of Data Set UTC time ms: %d\n", *iptr);

/* Calibration
--------------*/
sptr = (short *)&buf[187-1];
printf("Ramp/Auto Calibration bit field: %d\n", *sptr);
sptr++;
printf("Year of most recent Solar Calibration: %d\n", *sptr);
sptr++;
printf("Day of Year of most recent Solar Calibration: %d\n", *sptr);
sptr = (short *)&buf[201-1];
printf("IR TT1 CC1: %d\n", *sptr);
sptr++; printf("IR TT1 CC2: %d\n", *sptr);
sptr++; printf("IR TT1 CC3: %d\n", *sptr);
sptr++; printf("IR TT1 CC4: %d\n", *sptr);
sptr++; printf("IR TT1 CC5: %d\n", *sptr);
sptr++; printf("IR TT1 CC6: %d\n", *sptr);
sptr++; printf("IR TT2 CC1: %d\n", *sptr);
sptr++; printf("IR TT2 CC2: %d\n", *sptr);
sptr++; printf("IR TT2 CC3: %d\n", *sptr);
sptr++; printf("IR TT2 CC4: %d\n", *sptr);
sptr++; printf("IR TT2 CC5: %d\n", *sptr);
sptr++; printf("IR TT2 CC6: %d\n", *sptr);
sptr++; printf("IR TT3 CC1: %d\n", *sptr);
sptr++; printf("IR TT3 CC2: %d\n", *sptr);
sptr++; printf("IR TT3 CC3: %d\n", *sptr);
sptr++; printf("IR TT3 CC4: %d\n", *sptr);
sptr++; printf("IR TT3 CC5: %d\n", *sptr);
sptr++; printf("IR TT3 CC6: %d\n", *sptr);
sptr++; printf("IR TT4 CC1: %d\n", *sptr);
sptr++; printf("IR TT4 CC2: %d\n", *sptr);
sptr++; printf("IR TT4 CC3: %d\n", *sptr);
sptr++; printf("IR TT4 CC4: %d\n", *sptr);
sptr++; printf("IR TT4 CC5: %d\n", *sptr);
sptr++; printf("IR TT4 CC6: %d\n", *sptr);

iptr = (int *)&buf[257-1];
printf("Ch 1 Solar filtered irradiance: %d\n", *iptr++);
printf("Ch 1 equivalent width: %d\n", *iptr++);
printf("Ch 2 Solar filtered irradiance: %d\n", *iptr++);
printf("Ch 2 equivalent width: %d\n", *iptr++);
printf("Ch 3a Solar filtered irradiance: %d\n", *iptr++);
printf("Ch 3a equivalent width: %d\n", *iptr++);
printf("Ch 3b Central wavenumber: %d\n", *iptr++);
printf("Ch 3b Constant 1: %d\n", *iptr++);
printf("Ch 3b Constant 2: %d\n", *iptr++);
printf("Ch 4 Central wavenumber: %d\n", *iptr++);
printf("Ch 4 Constant 1: %d\n", *iptr++);
printf("Ch 4 Constant 2: %d\n", *iptr++);
printf("Ch 5 Central wavenumber: %d\n", *iptr++);
printf("Ch 5 Constant 1: %d\n", *iptr++);
printf("Ch 5 Constant 2: %d\n", *iptr++);

sptr = (short *)&buf[425-1];
printf("Patch Temp CC1: %d\n", *sptr++);
printf("Patch Temp CC2: %d\n", *sptr++);
printf("Patch Temp CC3: %d\n", *sptr++);
printf("Patch Temp CC4: %d\n", *sptr++);
printf("Patch Temp CC5: %d\n", *sptr++);
sptr++;
printf("Patch Temp extended CC1: %d\n", *sptr++);
printf("Patch Temp extended CC2: %d\n", *sptr++);
printf("Patch Temp extended CC3: %d\n", *sptr++);
printf("Patch Temp extended CC4: %d\n", *sptr++);
printf("Patch Temp extended CC5: %d\n", *sptr++);
sptr++;
printf("Patch Power CC1: %d\n", *sptr++);
printf("Patch Power CC2: %d\n", *sptr++);
printf("Patch Power CC3: %d\n", *sptr++);
printf("Patch Power CC4: %d\n", *sptr++);
printf("Patch Power CC5: %d\n", *sptr++);
sptr++;
printf("Radiator Temp CC1: %d\n", *sptr++);
printf("Radiator Temp CC2: %d\n", *sptr++);
printf("Radiator Temp CC3: %d\n", *sptr++);
printf("Radiator Temp CC4: %d\n", *sptr++);
printf("Radiator Temp CC5: %d\n", *sptr++);
sptr++;
printf("Blackbody Temp 1 CC1: %d\n", *sptr++);
printf("Blackbody Temp 1 CC2: %d\n", *sptr++);
printf("Blackbody Temp 1 CC3: %d\n", *sptr++);
printf("Blackbody Temp 1 CC4: %d\n", *sptr++);
printf("Blackbody Temp 1 CC5: %d\n", *sptr++);
sptr++;
printf("Blackbody Temp 2 CC1: %d\n", *sptr++);
printf("Blackbody Temp 2 CC2: %d\n", *sptr++);
printf("Blackbody Temp 2 CC3: %d\n", *sptr++);
printf("Blackbody Temp 2 CC4: %d\n", *sptr++);
printf("Blackbody Temp 2 CC5: %d\n", *sptr++);
sptr++;
printf("Blackbody Temp 3 CC1: %d\n", *sptr++);
printf("Blackbody Temp 3 CC2: %d\n", *sptr++);
printf("Blackbody Temp 3 CC3: %d\n", *sptr++);
printf("Blackbody Temp 3 CC4: %d\n", *sptr++);
printf("Blackbody Temp 3 CC5: %d\n", *sptr++);
sptr++;
printf("Blackbody Temp 4 CC1: %d\n", *sptr++);
printf("Blackbody Temp 4 CC2: %d\n", *sptr++);
printf("Blackbody Temp 4 CC3: %d\n", *sptr++);
printf("Blackbody Temp 4 CC4: %d\n", *sptr++);
printf("Blackbody Temp 4 CC5: %d\n", *sptr++);
sptr++;
printf("Electronics Current CC1: %d\n", *sptr++);
printf("Electronics Current CC2: %d\n", *sptr++);
printf("Electronics Current CC3: %d\n", *sptr++);
printf("Electronics Current CC4: %d\n", *sptr++);
printf("Electronics Current CC5: %d\n", *sptr++);
sptr++;
printf("Motor Current CC1: %d\n", *sptr++);
printf("Motor Current CC2: %d\n", *sptr++);
printf("Motor Current CC3: %d\n", *sptr++);
printf("Motor Current CC4: %d\n", *sptr++);
printf("Motor Current CC5: %d\n", *sptr++);
sptr++;
printf("Earth Shield Position CC1: %d\n", *sptr++);
printf("Earth Shield Position CC2: %d\n", *sptr++);
printf("Earth Shield Position CC3: %d\n", *sptr++);
printf("Earth Shield Position CC4: %d\n", *sptr++);
printf("Earth Shield Position CC5: %d\n", *sptr++);
sptr++;
printf("Electronics Temp CC1: %d\n", *sptr++);
printf("Electronics Temp CC2: %d\n", *sptr++);
printf("Electronics Temp CC3: %d\n", *sptr++);
printf("Electronics Temp CC4: %d\n", *sptr++);
printf("Electronics Temp CC5: %d\n", *sptr++);
sptr++;
printf("Cooler Housing Temp CC1: %d\n", *sptr++);
printf("Cooler Housing Temp CC2: %d\n", *sptr++);
printf("Cooler Housing Temp CC3: %d\n", *sptr++);
printf("Cooler Housing Temp CC4: %d\n", *sptr++);
printf("Cooler Housing Temp CC5: %d\n", *sptr++);
sptr++;
printf("Baseplate Temp CC1: %d\n", *sptr++);
printf("Baseplate Temp CC2: %d\n", *sptr++);
printf("Baseplate Temp CC3: %d\n", *sptr++);
printf("Baseplate Temp CC4: %d\n", *sptr++);
printf("Baseplate Temp CC5: %d\n", *sptr++);
sptr++;
printf("Motor Housing Temp CC1: %d\n", *sptr++);
printf("Motor Housing Temp CC2: %d\n", *sptr++);
printf("Motor Housing Temp CC3: %d\n", *sptr++);
printf("Motor Housing Temp CC4: %d\n", *sptr++);
printf("Motor Housing Temp CC5: %d\n", *sptr++);
sptr++;
printf("A/D Conv Temp CC1: %d\n", *sptr++);
printf("A/D Conv Temp CC2: %d\n", *sptr++);
printf("A/D Conv Temp CC3: %d\n", *sptr++);
printf("A/D Conv Temp CC4: %d\n", *sptr++);
printf("A/D Conv Temp CC5: %d\n", *sptr++);
sptr++;
printf("Det. 4 Bias CC1: %d\n", *sptr++);
printf("Det. 4 Bias CC2: %d\n", *sptr++);
printf("Det. 4 Bias CC3: %d\n", *sptr++);
printf("Det. 4 Bias CC4: %d\n", *sptr++);
printf("Det. 4 Bias CC5: %d\n", *sptr++);
sptr++;
printf("Det. 5 Bias CC1: %d\n", *sptr++);
printf("Det. 5 Bias CC2: %d\n", *sptr++);
printf("Det. 5 Bias CC3: %d\n", *sptr++);
printf("Det. 5 Bias CC4: %d\n", *sptr++);
printf("Det. 5 Bias CC5: %d\n", *sptr++);
sptr++;
printf("Ch 3b Blackbody View CC1: %d\n", *sptr++);
printf("Ch 3b Blackbody View CC2: %d\n", *sptr++);
printf("Ch 3b Blackbody View CC3: %d\n", *sptr++);
printf("Ch 3b Blackbody View CC4: %d\n", *sptr++);
printf("Ch 3b Blackbody View CC5: %d\n", *sptr++);
sptr++;
printf("Ch 4 Blackbody View CC1: %d\n", *sptr++);
printf("Ch 4 Blackbody View CC2: %d\n", *sptr++);
printf("Ch 4 Blackbody View CC3: %d\n", *sptr++);
printf("Ch 4 Blackbody View CC4: %d\n", *sptr++);
printf("Ch 4 Blackbody View CC5: %d\n", *sptr++);
sptr++;
printf("Ch 5 Blackbody View CC1: %d\n", *sptr++);
printf("Ch 5 Blackbody View CC2: %d\n", *sptr++);
printf("Ch 5 Blackbody View CC3: %d\n", *sptr++);
printf("Ch 5 Blackbody View CC4: %d\n", *sptr++);
printf("Ch 5 Blackbody View CC5: %d\n", *sptr++);

/* Now repeat the coefficients so they are in the
same format as required by the noaaXX.tel file.
-------------------------------------------------*/
printf("\n\nAnalog Telemetry Conversion Coefficients\n\n");
sptr = (short *)&buf[425-1];
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("patch temperature\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("patch temperature extended\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("patch power\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("radiator temperature\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("blackbody temp 1\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("blackbody temp 2\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("blackbody temp 3\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("blackbody temp 4\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("electronics current\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("motor current\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("earth shield position\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("electronics temp\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("cooler housing temp\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("baseplate temp\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("motor housing temp\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("A/D converter temp\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("detector #4 bias voltage\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("detector #5 bias voltage\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("chan 3b blackbody view\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("chan 4 blackbody view\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("chan 5 blackbody view\n");
sptr++;
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("%-8d", *sptr++);
printf("reference voltage\n\n");
sptr++;

}
