/******************************************************************************

FUNCTION:	FINISH

PURPOSE:	To close the tape and image files

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
 1.0				Original development
 5.0	12/91	T. Rockvam	LAS 5.0 Conversion
 6.0	02/98	D.Lloyd		Modified to use Level 1b supports and to return
				the number of bytes written to tape.  Only
				modifies variable block if called from tae.
				Generates file name if necessary.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	May be run under TAE or unix

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/

#include <stdio.h>
#include <time.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "diskio.h"
#include "labserv.h"
#include "adapssup.h"
#include "level1bio.h"

void finish
(
    struct PARBLK *vblock,	/* parameter block			*/
    char *drive,		/* drive name				*/
    long *nvols,		/* number of volumes used		*/
    long tplun,			/* tape unit number			*/
    FILE *fdout,		/* output file descriptor		*/
    char archnam[],		/* input image host file name		*/
    long *archfd,		/* archive image file descriptor	*/
    char hosout[],		/* Output host file name		*/
    struct MODEL *model,	/* avhrr model structure		*/
    char *tmpfil,		/* temporary file name			*/
    long genfil,		/* flag: generate file name from sceneid*/
    char *out,			/* output file name			*/
    long *kbyteswritten,	/* number of bytes written to tape	*/
    long tae 			/* flag: called from taeland		*/
)
{
char    hosdir[CMLEN];          /* host directory name                  */
char    hosext[CMLEN];          /* host extension                       */
char    hosroot[CMLEN];         /* host root name                       */
char	sceneid[CMLEN];		/* scene id				*/
char	tmp[CMLEN];		/* temporary string			*/
char	*tmpstring[1];		/* Drive name for updating vblock	*/

long	mode = 0;		/* test for file presense		*/
long	now;			/* time now				*/
long	status;			/* function return status		*/
long	tape = 1;		/* tape open flag			*/

struct	tm *curtime;		/* current time structure		*/


/*
  Close image file
-------------------*/
if ( *archfd != 0 )
    {
    status = c_arclose( archnam, archfd );
    if ( status != E_SUCC )
        c_lasmsg(FALSE,"mklevel1b-imgcler","Error closing archive image");
    }


/*
  Close output, if disk output free reserved space on a given pack
-------------------------------------------------------------------*/
if ( tplun )
    status = c_l1bclost( &tplun, &tape );
else
    {
    c_l1bclosd( fdout );
    c_dkfree( hosout );
    }


/*
  Generate a file name using the model when output file
  wasn't specified.
--------------------------------------------------------*/
tmp[0] = 0;
sceneid[0] = 0;
if ( genfil )
    {
    c_splitname( tmpfil, hosdir, hosroot, hosext );
    c_getscene( model, sceneid );
    sprintf( hosout, "%s%s.l1b", hosdir, sceneid );
    if ( tplun )
        strcpy(out, hosout);
    else if ( c_lsstat( hosout, &mode ) == E_SUCC )
	{
	/* name is in use so tack on time stamp */
	now = time( NULL );
	curtime = localtime( &now );
	sprintf( hosroot, "%s_%02d%02d%02d", sceneid,
		curtime->tm_hour, curtime->tm_min, curtime->tm_sec );
	sprintf( hosout, "%s%s.l1b", hosdir, hosroot );
	rename( tmpfil, hosout );
        strcpy(out, hosout);
	}
    else
	{
	if ( rename( tmpfil, hosout ) != E_SUCC )
	    {
	    c_lasmsg(FALSE,"mklevel1b-renm","Error renaming output file");
	    c_splitname( tmpfil, hosdir, hosroot, hosext );
            strcpy(out, tmpfil);
	    }
	else
	    {
            strcpy(out, hosout);
	    c_splitname( hosout, hosdir, hosroot, hosext );
	    }
	}
    }
else
    {
    strcpy(out, hosout);
    c_splitname( hosout, hosdir, hosroot, hosext );
    }


/*
  Update the variable block only if called from tae
-----------------------------------------------------*/
if ( tae )
    {
    /*
      Update the variable block drive name in case it has changed
    --------------------------------------------------------------*/
    tmpstring[0] = drive;
    status = q_string( vblock, "DRIVE", 1, (const char **)tmpstring, P_UPDATE );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"mklevel1b-vbupder","Error updating variable block");

    status = q_intg( vblock, "NVOLS", 1, (int *)nvols, P_UPDATE );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"mklevel1b-vbupder","Error updating variable block");

    /*
      Update the output file name in case it wasn't specified or
      it changed.
    -------------------------------------------------------------*/
    if ( tplun )
        strcpy(tmp, sceneid);
    else
        strcpy(tmp, hosroot);
    tmpstring[0] = tmp;
    status = q_string(vblock,"OUTNAME", 1, (const char **)tmpstring, P_UPDATE );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"mklevel1b-vbupder","Error updating variable block");

    /*
      Update number of bytes written to tape
    -----------------------------------------*/
    status = q_intg( vblock, "NBYTES", 1, (int *)kbyteswritten, P_UPDATE );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"mklevel1b-vbupder","Error updating variable block");

    status = q_out( vblock );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"mklevel1b-vbupder","Error updating variable block");
    }

}
