/*****************************************************************************
FUNCTION:	GETARGV

PURPOSE:	Retrieve parameters from the unix command line via argv[].

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    03/98	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "las.h"
#include "notae.h"
#include "labserv.h"
#include "util.h"
#include "getargv.h"



void getargv
(
    long argc,			/* count of arguments in argv[]		*/
    char *argv[],		/* array of arguments			*/
    char *comment,		/* tape mount comment(s)		*/
    char *infile,		/* input file name			*/
    char *outfile,		/* output file name			*/
    char *acctnum,		/* account number			*/
    char *ordernum,		/* order number				*/
    char *unitnum,		/* unit number				*/
    char *prodesc,		/* product description			*/
    char *drive,		/* drive tape is mounted on		*/
    char *ttype,		/* tape type				*/
    long *nvols,		/* volume number in this set		*/
    long *mvolflg,		/* flag: TRUE allows multiple volumes	*/
    char *density,		/* tape density				*/
    long window[],		/* window to extract from input image	*/
    long chans[],		/* channels to extract from input image	*/
    long *nchans,		/* count of channels			*/
    long *packflg,		/* flag: TRUE means pack output		*/
    char *sumfile,		/* summary file name			*/
    long *maxbytes,		/* max allowable kbytes for tape write	*/
    char tmpfil[],		/* temporary output file name		*/
    long *genfil		/* flag: generate filename from sceneid	*/
)

{
char	hosdir[CMLEN];		/* host directory name			*/
char	hosext[CMLEN];		/* host extension name			*/
char	hosroot[CMLEN];		/* host root name			*/
char	list[20][CMLEN];	/* list of input parsed parameters	*/
char	msg[CMLEN];		/* LAS message string			*/
char	*str;			/* pointer into tempstring		*/
char	subcmd[CMLEN];		/* subcommand				*/
char	tempstring[CMLEN];	/* temporary parameter value string	*/

long	i;			/* loop index variable			*/
long	mode = 0;		/* test for file presence		*/
long	numlist;		/* number of parameters in list		*/
long	temp;			/* temporary integer value		*/

/*
  Set up the parameter structure
----------------------------------*/
char *scmdlist[] = { "line", "latlong", "latrange", NULL };
char *params[] = { "comment", "infile", "acctnum", "ordernum", "unitnum",
	"drive", "ttype", "outfile", "nvols", "mvolflg", "tapedens", "sl",
	"nl", "chans", "prodesc", "packflg", "sumfile", "maxbytes", NULL };
enum pnames { COMMENT, INFILE, ACCTNUM, ORDERNUM, UNITNUM, DRIVE, TTYPE,
	OUTFILE, NVOLS, MVOLFLG, TAPEDENS, _SL, _NL, CHANS, PRODESC, PACKFLG,
	SUMFILE, MAXBYTES };
long count[] = { 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 5, 1, 1, 1, 1 };
enum PTYPE ptype[] = { STRING, STRING, STRING, STRING, STRING, STRING,
	 STRING, STRING, INTEGER, STRING, STRING, INTEGER, INTEGER,
	INTEGER, STRING, STRING, STRING, INTEGER };
struct PARAMETER parms;
parms.plist = params;
parms.pcount = count;
parms.ptype = ptype;

/*
  Parse argv and count the returned parameters
-----------------------------------------------*/
c_parsepar( argc, argv, &parms, scmdlist, list );
numlist = 0;
while ( list[numlist++][0] != '\0' );
numlist--;

/*
  list[0] is the subcommand
----------------------------*/
strcpy( subcmd, list[0] );

/*
  Retrieve the input file.
----------------------------*/
if ( c_getsval( "infile", list, tempstring ) != E_SUCC )
    c_lasmsg(TRUE,"mklevel1b-parm",
	"Required parameter not specified: INFILE");
else
    {
    c_cmhost( tempstring, infile );
    c_splitname( infile, hosdir, hosroot, hosext );
    if ( hosext[0] == 0 )
	strcat( infile, ".arch" );
    }

/*
  Retrieve the output file name or tape mount comment
------------------------------------------------------*/
tempstring[0] = comment[0] = outfile[0] = 0;
hosdir[0] = hosroot[0] = hosext[0] = 0;
c_getsval( "comment", list, comment );
c_getsval( "outfile", list, tempstring );
if ( tempstring[0] != 0 )
    {
    c_cmhost( tempstring, outfile );
    c_splitname( outfile, hosdir, hosroot, hosext );
    }
if ( comment[0] == 0 && hosroot[0] == 0 )
    {
    /* no output specified signal to generate filename */
    *genfil = TRUE;
    sprintf( tmpfil, "A%d.l1b", getpid() );
    strcpy( outfile, hosdir );
    strcat( outfile, tmpfil );
    }
else if ( comment[0] != 0 && tempstring[0] != 0 )
    /* only one output can be specified */
    c_lasmsg(TRUE,"mklevel1b-parm","Can only specify one output");
else if ( tempstring[0] != 0 )
    {
    c_cmhost( tempstring, outfile );
    c_splitname( outfile, hosdir, hosroot, hosext );
    if ( hosext[0] == 0 )
	strcat( outfile, ".l1b" );
    }

/*
  Make sure the output file doesn't already exist
--------------------------------------------------*/
if ( c_lsstat( outfile, &mode ) == E_SUCC )
    {
    sprintf( msg, "%s already exists", outfile );
    c_lasmsg(TRUE,"mklevel1b-exists",msg);
    }


/*
  get tape information if mount comment is specified
-----------------------------------------------------*/
if ( comment[0] != 0 )
    {
    /* get the drive name */
    if ( c_getsval( "drive", list, drive ) != E_SUCC )
	c_lasmsg(TRUE,"mklevel1b-parm",
	    "Required parameter not specified: DRIVE");

    /* get the tape type, if not specified default is 3490 */
    if ( c_getsval( "ttype", list, ttype ) != E_SUCC )
	strcpy( ttype, "3490" );
    else if ( strncasecmp( ttype, "8hi", 3 ) != 0 &&
    	      strncasecmp( ttype, "8lo", 3 ) != 0 &&
	      strncasecmp( ttype, "3480", 4 ) != 0 &&
	      strncasecmp( ttype, "3490", 4 ) != 0 &&
	      strncasecmp( ttype, "4mm", 3 ) != 0  &&
              strncasecmp( ttype, "dlt4000", 7 ) != 0 &&
              strncasecmp( ttype, "dlt7000", 7 ) != 0 &&
              strncasecmp( ttype, "d3", 2 ) != 0 )
        c_lasmsg(TRUE,"mklevel1b-ttype",
	    "Invalid tape type specified");
	

    /* get the density, if not specified default is 0 */
    if ( c_getsval( "tapedens", list, density ) != E_SUCC )
	{
	if ( strncasecmp ( ttype, "8hi", 3 ) == 0 )
	    strcpy( density, "HI" );
	else if ( strncasecmp ( ttype, "8lo", 3 ) == 0 )
	    strcpy( density, "LO" );
	else
	    strcpy( density, "0" );
	}

    /* get the multiple volume flag, if not specified default is TRUE */
    if ( c_getsval( "mvolflg", list, tempstring ) != E_SUCC )
	*mvolflg = TRUE;
    else
	{
	if ( strncasecmp( tempstring, "NO", 2 ) == 0 )
	    *mvolflg = FALSE;
	else
	    *mvolflg = TRUE;
	}

    /* get volume number, if not specified default is 1 */
    if ( c_getsval( "nvols", list, tempstring ) != E_SUCC )
	*nvols = 1;
    else
	c_getival( tempstring, parms.pcount[NVOLS], &temp, nvols );

    /* get max allowable bytes for tape */
    if ( c_getsval( "maxbytes", list, tempstring ) == E_SUCC )
        {
	c_getival( tempstring, 1, &temp, maxbytes );
	*maxbytes *= 900;
	}
    else
	{
	/* not specified, set based on tape type */
	if ( strncasecmp( ttype, "3480", 4 ) == 0 )
	    *maxbytes = 270000;
	if ( strncasecmp( ttype, "3490", 4 ) == 0 )
	    *maxbytes = 700000;
	if ( strncasecmp( ttype, "4mm", 4 ) == 0 )
	    *maxbytes = 1200000;
	if ( strncasecmp( ttype, "8lo", 4 ) == 0 )
	    *maxbytes = 2250000;
	if ( strncasecmp( ttype, "8hi", 4 ) == 0 )
	    *maxbytes = 4500000;
        if ( strncasecmp( ttype, "dlt4000", 7 ) == 0 )
            *maxbytes = 18000000;
        if ( strncasecmp( ttype, "dlt7000", 7 ) == 0 )
            *maxbytes = 31500000;
        if ( strncasecmp( ttype, "d3", 2 ) == 0 )
            *maxbytes = 45000000;
	}
    }


/*
  Get account, order and unit numbers
--------------------------------------*/
if ( c_getsval( "acctnum", list, acctnum ) != E_SUCC )
    acctnum[0] = 0;
if ( c_getsval( "ordernum", list, ordernum ) != E_SUCC )
    ordernum[0] = 0;
if ( c_getsval( "unitnum", list, unitnum ) != E_SUCC )
    unitnum[0] = 0;


/*
  Get the window specification
-------------------------------*/
if ( c_getsval( "sl", list, tempstring ) != E_SUCC )
    window[SL] = 1;
else
    c_getival( tempstring, 1, &temp, &window[SL] );
if ( c_getsval( "nl", list, tempstring ) != E_SUCC )
    window[NL] = -1;
else
    c_getival( tempstring, 1, &temp, &window[NL] );

window[SS] = 1;
window[NS] = -1;

if ( c_getsval( "packflg", list, tempstring ) != E_SUCC )
    *packflg = TRUE;
else
    {
    if ( strncasecmp( tempstring, "NO", 2 ) == 0 )
	*packflg = FALSE;
    else
	*packflg = TRUE;
    }

/*
  Get summary information
--------------------------*/
if ( c_getsval( "sumfile", list, sumfile ) != E_SUCC )
    {
    /* get $ADAPSARC then build it as:
	   $ADAPSARC/forms/scenes/sceneid.prs */
    str = getenv( "ADAPSARC" );
    strcpy( sumfile, str );
    strcat( sumfile, "/forms/scenes/" );
    }
if ( c_getsval( "prodesc", list, prodesc ) != E_SUCC )
    strcpy ( prodesc, "LEVEL1B" );


/*
  Get the satellite channels and number of channels
----------------------------------------------------*/
if ( c_getsval( "nchans", list, tempstring ) != E_SUCC )
    *nchans = 5;
else
    c_getival( tempstring, 1, &temp, nchans );

if ( c_getsval( "chans", list, tempstring ) != E_SUCC )
    {
    /* assign the default settings since they weren't specified */
    for ( i = 0; i < 5; i++ )
	chans[i] = i + 1;
    }
else
    {
    /* initialize to zero */
    for ( i = 0; i < 5; i++ )
	chans[i] = 0;
    c_getival( tempstring, 5, &temp, chans );
    *nchans = temp;
    for ( i = temp-1; i >= 0; i-- )
	{
	chans[chans[i]-1] = chans[i];
	if ( chans[i] != 1 )
	    chans[i] = 0;
	}
    }
}
