/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the parameters specified by the user.

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
 1.0				Original development
 5.0	12/91	T. Rockvam	LAS 5.0 Conversion
 6.0	02/98	D.Lloyd		Removed code to retrieve window parameters,
				this is now done with c_windpar(), and summary
				flag.  Added parameters max_bytes and
				kbyteswritten.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "tae.h"
#include "util.h"
#include "tapeio.h"
#include "adaps.h"


void getpar
(
    struct PARBLK *vblock,	/* input parameter block		*/
    char *archnam,		/* input image host file name		*/
    long bands[NCHANS],		/* bands to process			*/
    long *nbands,		/* number of bands to process		*/
    char comment[CMLEN],	/* tape mount comment			*/
    char hosout[CMLEN],		/* output host file name		*/
    char acctnum[10],		/* account number			*/
    char ordernum[14],		/* order number				*/
    char unitnum[5],		/* unit number				*/
    char prodesc[18],		/* product description			*/
    char *drive,		/* drive name				*/
    long *pack,			/* packing flag				*/
    long *mvol,			/* multiple tape volume flag		*/
    long *nvols,		/* number of volumes used		*/
    char *density,		/* tape density				*/
    char *ttype,		/* tape type				*/
    long *msgtim,		/* time between processing messages	*/
    char out[CMLEN],		/* Product Summary Report name		*/
    char *tmpfil,		/* temporary file name			*/
    long *genfil,		/* flag: generate file name from sceneid*/
    long *max_bytes,		/* maximum bytes allowed for tape	*/
    long *kbyteswritten		/* number of bytes written to tape	*/
)

{
char	hosdir[CMLEN];		/* host directory name			*/
char	hosext[CMLEN];		/* host extension			*/
char	hosroot[CMLEN];		/* host root name			*/
char	tmp[CMLEN];		/* temporary filename			*/

long	access;			/* file access				*/
long	hold;			/* used to sort band numbers		*/
long	i,j;			/* looping variable			*/
long	procid;			/* process id number			*/
long	status;			/* status returned from support calls	*/
long	zero = 0;		/* value of zero			*/

struct	VARIABLE *var;		/* pointer to a TAE variable		*/


/*
  Get the tape mount comment or output file name
*/
var = p_find( vblock, "COMMENT" );
if ( var->v_count != 0 )
    {
    strcpy( comment, SVAL( *var, 0 ));
    hosout[0] = '\0';
    *genfil = TRUE;
    }
else
    {
    access = IWRITE;
    status = c_getnam( vblock, "OUTFILE", &zero, hosout, "l1b", &access );
    if ( status != E_SUCC )
        c_lasmsg(TRUE,"mklevel1b-fatal","Fatal error encountered");
    c_splitname( hosout, hosdir, hosroot, hosext );
    if ( strlen( hosroot ) == 0 )
	{
	strcpy( hosout, hosdir );
	procid = getpid();
	sprintf( tmpfil, "A%ld.l1b", procid );
	strcat( hosout, tmpfil );
	strcpy( tmpfil, hosout );
	*genfil = TRUE;
	}
    else
	*genfil = FALSE;
    comment[0] = '\0';
    }


/*
  Get input image
*/
access = IREAD;
status = c_getnam( vblock, "INFILE", &zero, archnam, "arch", &access );
if ( status != E_SUCC )
    c_lasmsg(TRUE,"mklevel1b-fatal","Fatal error encountered");


/*
  Get the specified bands
*/
var = p_find( vblock, "CHANS" );
*nbands = var->v_count;
if ( *nbands == 0 )
    {
    for ( i = 0; i < NCHANS; i++ )
	bands[i] = i + 1;
    *nbands = NCHANS;
    }
else
    {
    for ( i = 0; i < *nbands; i++ )
	bands[i] = IVAL( *var, i );
    /* sort so bands are in order */
    for ( i = 1; i < *nbands; i++ )
	{
        for ( j = 1; j <= *nbands - i; j++ )
	    {
	    if ( bands[j-1] > bands[j] )
	        {
	        hold = bands[j-1];
	        bands[j-1] = bands[j];
	        bands[j] = hold;
	        }
	    }
	}
    }


/*
  Get the account number
*/
acctnum[0] = 0;
var = p_find( vblock, "ACCTNUM" );
if ( var->v_count != 0 )
    strcpy( acctnum, SVAL( *var, 0 ));

/*
  Get the order number
*/
ordernum[0] = 0;
var = p_find( vblock, "ORDERNUM" );
if ( var->v_count != 0 )
    strcpy( ordernum, SVAL( *var, 0 ));

/*
  Get the unit number
*/
unitnum[0] = 0;
var = p_find( vblock, "UNITNUM" );
if ( var->v_count != 0 )
    strcpy( unitnum, SVAL( *var, 0 ));


/*
  Get the product description
*/
var = p_find( vblock, "PRODESC" );
strcpy( prodesc, SVAL( *var, 0 ));

/*
  Get the Product Summary File name
*/
var = p_find( vblock, "SUMFILE" );
strcpy( tmp, SVAL( *var, 0 ));
c_cmhost( tmp, out );


/*
  Get the packing flag
*/
var = p_find( vblock, "PACKFLG" );
if ( strcmp( SVAL( *var, 0 ), "YES" ) == 0 )
    *pack = TRUE;
else
    *pack = FALSE;

/*
  Get the multi-volume flag
*/
var = p_find( vblock, "MVOLFLG" );
if ( strcmp( SVAL( *var, 0 ), "YES" ) == 0 )
    *mvol = TRUE;
else
    *mvol = FALSE;


/*
  Get the number of volumes used
*/
var = p_find( vblock, "NVOLS" );
*nvols = IVAL( *var, 0 );

/*
  Get the tape drive name
*/
var = p_find( vblock, "DRIVE" );
strcpy( drive, SVAL( *var, 0 ));

/*
  Get the tape density
*/
var = p_find( vblock, "TAPEDENS" );
strcpy( density, SVAL( *var, 0 ));

/*
  Get the tape type
*/
var = p_find( vblock, "TTYPE" );
strcpy( ttype, SVAL( *var, 0 ));

/*
  Determine the desired density
*/
if ( !strcasecmp( ttype, "8hi" ))
    strcpy( density, "HI" );
else if ( !strcasecmp( ttype, "8lo" ))
    strcpy( density, "LO" );
else
    strcpy( density, "0" );

/*
  Get the maximum number of bytes allowed
*/
var = p_find( vblock, "MAXBYTES" );
*max_bytes = IVAL( *var, 0 );

/*
  Get the number of bytes already written to tape
*/
var = p_find( vblock, "NBYTES" );
*kbyteswritten = IVAL( *var, 0 );


/*
  Get the time between processing messages
*/
var = p_find( vblock,"$MSGTIM" );
*msgtim = IVAL( *var, 0 );
}
