#include "level1bio.h"


#ifndef RAD
#define RAD(p)    p * PIE / 180
#endif
#ifndef MIN
#define MIN(a,b)  ((a) < (b) ? (a) : (b))
#endif
#ifndef MIN4
#define MIN4(a,b,c,d) (MIN(a,b) < MIN(c,d) ? MIN(a,b) : MIN(c,d))
#endif
#ifndef MAX4
#define MAX4(a,b,c,d) (LASMAX(a,b) > LASMAX(c,d) ? LASMAX(a,b) : LASMAX(c,d))
#endif
#define RND(x)    (( x > 0.0 ) ? (x) + 0.5 : (x) - 0.5 )


#define CALPER   5   /* Calibration period	 */
#define PRTPER  10   /* PRT temperature period   */

struct LINE
       {
       short    *min_frm;
       short    (*data)[NCHANS];
       } ;

#define TWO22   4194304.0
#define TWO30   1073741824.0

/* Prototypes for MKLEVEL1B coding */


void finish
(
    struct PARBLK *vblock,	/* parameter block			*/
    char *drive,		/* drive name				*/
    long *nvols,		/* number of volumes used		*/
    long tplun,			/* tape unit number			*/
    FILE *fdout,		/* output file descriptor		*/
    char archnam[],		/* input image host file name		*/
    long *archfd,		/* archive image file descriptor	*/
    char hosout[],		/* Output host file name		*/
    struct MODEL *model,	/* avhrr model structure		*/
    char *tmpfil,		/* temporary file name			*/
    long genfil,		/* flag: generate file name from sceneid*/
    char *out,			/* output file name			*/
    long *kbyteswritten,	/* number of bytes written to tape	*/
    long tae 			/* flag: called from taeland		*/
);

void getargv
(
    long argc,			/* count of arguments in argv[]		*/
    char *argv[],		/* array of arguments			*/
    char *comment,		/* tape mount comment(s)		*/
    char *infile,		/* input file name			*/
    char *outfile,		/* output file name			*/
    char *acctnum,		/* account number			*/
    char *ordernum,		/* order number				*/
    char *unitnum,		/* unit number				*/
    char *prodesc,		/* product description			*/
    char *drive,		/* drive tape is mounted on		*/
    char *ttype,		/* tape type				*/
    long *nvols,		/* volume number in this set		*/
    long *mvolflg,		/* flag: TRUE allows multiple volumes	*/
    char *density,		/* tape density				*/
    long window[],		/* window to extract from input image	*/
    long chans[],		/* channels to extract from input image	*/
    long *nchans,		/* count of channels			*/
    long *packflg,		/* flag: TRUE means pack output		*/
    char *sumfile,		/* summary file name			*/
    long *maxbytes,		/* max allowable kbytes for tape write	*/
    char tmpfil[],		/* temporary output file name		*/
    long *genfil		/* flag: generate filename from sceneid	*/
);

void getpar
(
    struct PARBLK *vblock,	/* input parameter block		*/
    char *archnam,		/* input image host file name		*/
    long bands[NCHANS],		/* bands to process			*/
    long *nbands,		/* number of bands to process		*/
    char comment[CMLEN],	/* tape mount comment			*/
    char hosout[CMLEN],		/* output host file name		*/
    char acctnum[10],		/* account number			*/
    char ordernum[14],		/* order number				*/
    char unitnum[5],		/* unit number				*/
    char prodesc[18],		/* product description			*/
    char *drive,		/* drive name				*/
    long *pack,			/* packing flag				*/
    long *mvol,			/* multiple tape volume flag		*/
    long *nvols,		/* number of volumes used		*/
    char *density,		/* tape density				*/
    char *ttype,		/* tape type				*/
    long *msgtim,		/* time between processing messages	*/
    char out[CMLEN],		/* Product Summary Report name		*/
    char *tmpfil,		/* temporary file name			*/
    long *genfil,		/* flag: generate file name from sceneid*/
    long *max_bytes,		/* maximum bytes allowed for tape	*/
    long *kbyteswritten		/* number of bytes written to tape	*/
);

void mk1btm
(
    short time[4],		/* four 16-bit words containing 4 10-bit words
			   	   of raw time				      */
    unsigned char *year,	/* byte of last two digits of year in binary  */
    unsigned char *code		/* resulting 6-byte timecode		      */
);

void pck_zen_dec
(
    unsigned char dec[51], /* Bytes representing decimal place for each s.z.a */
    unsigned char outbuf[20]    /* 20-byte output buffer		      */
);

int prep1b
(
    long *tun,			/* tape logical unit number		*/
    FILE **fdout,		/* file descriptor of output file	*/
    char *hosout,		/* output host file name		*/
    char *density,		/* density of tape specification	*/
    char *drive,		/* tape drive name			*/
    long *recsiz,		/* Level 1b record size			*/
    struct MODEL *model,	/* Model and orbital information	*/
    unsigned char *tbm,		/* pointer to TBM header record		*/
    unsigned char *dsh,		/* pointer to Data Set header record	*/
    long *tape 			/* tape open flag			*/
);

int prepimg
(
    char archnam[CMLEN], 	/* input image host file name           */
    long *archfd,		/* archive image file descriptor	*/
    struct MODEL *model		/* Model and orbital information	*/
);

int process
(
    long *tplun,		/* IN: output tape lun			*/
    FILE *fdout,		/* IN: output file descriptor		*/
    long *archfd,		/* IN: archive image file descriptor	*/
    long window[4],		/* IN: input window			*/
    long *bands,		/* IN: band numbers to write to tape	*/
    long nbands,		/* IN: number of bands to write		*/
    long pack,			/* IN: packing flag			*/
    long recsiz,		/* IN: Level 1b record size		*/
    long msgtim,		/* IN: time for processing message	*/
    struct MODEL *model,	/* IN: Model and orbital information	*/
    long max_bytes,		/* IN: max bytes to write to tape	*/
    long *kbyteswritten,	/* IN/OUT: bytes written to tape	*/
    long *tape 			/* IN/OUT: tape open flag		*/
);

int process_klm
(
    long *tplun,		/* IN: output tape lun			*/
    FILE *fdout,		/* IN: output file descriptor		*/
    long *archfd,		/* IN: archive image file descriptor	*/
    long window[4],		/* IN: input window			*/
    long *bands,		/* IN: band numbers to write to tape	*/
    long nbands,		/* IN: number of bands to write		*/
    long pack,			/* IN: packing flag			*/
    long recsiz,		/* IN: Level 1b record size		*/
    long msgtim,		/* IN: time for processing message	*/
    struct MODEL *model,	/* IN: Model and orbital information	*/
    long max_bytes,		/* IN: max bytes to write to tape	*/
    long *kbyteswritten,	/* IN/OUT: bytes written to tape	*/
    long *tape 			/* IN/OUT: tape open flag		*/
);

