/******************************************************************************
FUNCTION:	PREP1B

PURPOSE:	Prepare output tape to receive image

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
 1.0				Original development
 5.0	12/91	T. Rockvam	Conversion to LAS 5.0
 6.0	02/98	D.Lloyd		Modified to use Level 1b I/O supports
				and changed name from preptp().
 6.1	01/99	D.Lloyd		Modified for NOAA-K

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS

ALGORITHM REFERENCES:   NONE

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tae.h"
#include "diskio.h"
#include "adaps.h"
#include "model.h"
#include "level1bio.h"
#include "tapeio.h"

int prep1b
(
    long *tun,			/* tape logical unit number		*/
    FILE **fdout,		/* file descriptor of output file	*/
    char *hosout,		/* output host file name		*/
    char *density,		/* density of tape specification	*/
    char *drive,		/* tape drive name			*/
    long *recsiz,		/* Level 1b record size			*/
    struct MODEL *model,	/* Model and orbital information	*/
    unsigned char *tbm,		/* pointer to TBM header record		*/
    unsigned char *dsh,		/* pointer to Data Set header record	*/
    long *tape 			/* tape open flag			*/
)
{
long long imagesize;		/* Size of output image (bytes)		*/
long	headersize;		/* Size of the TBM/ARS header		*/
long	status;			/* return status			*/

/* Determine TBM/ARS header size
--------------------------------*/
if ( model->satnum >= 15 )
    headersize = ARS_HDR_SIZ;
else
    headersize = TBM_HDR_SIZ;

/* Before opening the level1b image for write access, call c_dkcre() so that
   it follows DATAPATH.
*/
if ( hosout[0] )
    {
    if ( model->dtype == GAC )
	imagesize = (long long) *recsiz * ( model->nlines / 2 + 1 );
    else
	imagesize = (long long) *recsiz * ( model->nlines * 2 + 2 );
    imagesize += headersize;
    if ( c_dkcre((char (*)[])NULL, hosout, imagesize, 600 ) != E_SUCC )
	{
	c_lasmsg(FALSE,"mklevel1b-open","Error opening output file");
	return E_FAIL;
	}

    /*
      Open the output file
    */
    status = c_l1bopndw( hosout, recsiz, tbm, dsh, &headersize, fdout );
    if ( status != E_SUCC )
	{
	c_lasmsg(FALSE,"mklevel1b-open","Error opening output file");
	return E_FAIL;
	}
    *tun = 0;
    }
/*
  Else open the tape drive
*/
else
    {
    /*
      Reopen the tape after opened by GETDRIVE
    */
    status = c_l1bopntw( drive, density, tun, tbm, dsh, recsiz, tape );
    if ( status != TAPE_SUCC )
        return E_FAIL;
    *fdout = NULL;
    }

return E_SUCC;
}
