/*****************************************************************************
FUNCTION:	TAPELIST

PURPOSE:	Create and update the tape listing file maintained in
		ADAPSARC/forms directory for creating tape labels and
		shipping memos.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    04/98	D.Lloyd		Initial development stollen from tape_summary
  				in ANSISTACK.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "tae.h"
#include "adaps.h"

#define NGROUP_POS 171
#define NFILE_POS  196


int main( void )
{
char	subcommand[CMLEN];	/* sub command string			*/
char	drive[CMLEN];		/* Tape drive where tape was created	*/
char	tapeid[6];		/* tape id of the mounted tape		*/
char	ttype[CMLEN];		/* tape type				*/
char	sceneid[CMLEN];		/* scene id				*/
char	outfile[CMLEN];		/* outpuf file name			*/
char	date_time[CMLEN];	/* String version of date and time	*/
char	host[CMLEN];		/* Name of the current computer system	*/
char	user[CMLEN];		/* Name of the current user		*/
char	level1b[CMLEN];		/* level 1b file name			*/
char	archive[CMLEN];		/* archive file name			*/

FILE	*ouf;			/* output file handle			*/

long	std_out;		/* standard output file			*/
long	ngroups = 0;		/* number of groups on tape		*/
long	nfiles = 0;		/* number of files on tape		*/
long	now;			/* current date and time as a long	*/

struct	PARBLK	vblock;		/* TAE parameter block			*/
struct	VARIABLE *var;		/* pointer to TAE variable		*/
struct	tm	*today;		/* current date and time structure	*/



/*
  Initialize LAS
-----------------*/
c_inlas( &vblock, &std_out );


/*
  Get the subcommand
---------------------*/
var = p_find( &vblock, "_SUBCMD" );
strcpy( subcommand, SVAL( *var, 0 ));


/*
  Get the tapeid
-----------------*/
var = p_find( &vblock, "TAPEID" );
strcpy( tapeid, SVAL( *var, 0 ));

/*
  Build the file name
----------------------*/
sprintf( outfile, "%s/forms/%s.list", getenv( "ADAPSARC" ), tapeid );

/*
  Create the file with the header information
----------------------------------------------*/
if ( strcasecmp( subcommand, "CREATE" ) == 0 )
    {
    /* Get the tape type */
    var = p_find( &vblock, "TTYPE" );
    strcpy( ttype, SVAL( *var, 0 ));

    /* Get the drive */
    var = p_find( &vblock, "DRIVE" );
    strcpy( drive, SVAL( *var, 0 ));

    /* get current date and time */
    now = time( NULL );
    today = localtime( &now );
    sprintf( date_time, "%02d/%02d/%04d %02d:%02d:%02d", today->tm_mon + 1,
    	today->tm_mday, today->tm_year + 1900, today->tm_hour, today->tm_min,
	today->tm_sec % 60 );

    /* determine the host and user names */
    strcpy( user, cuserid( NULL ));
    gethostname( host, CMLEN );

    /* open the output file erasing previous contents */
    ouf = fopen( outfile, "w" );

    /* Create the file header.  Note that each of the header lines must
       contain 80 characters and a newline character.  This will allow
       us to seek to the correct position for the number of file groups
       and the total number of files when these values are updated on
       subsequent function calls.
    -------------------------------------------------------------------*/
    fprintf( ouf, "TapeID:  %-15.15sTapeType: %-15.15sCreated: %-22.22s\n",
        tapeid, ttype, date_time );

    fprintf( ouf, "System:  %-15.15sDrive:    %-15.15sOwner:   %-22.22s\n",
        host, drive, user );

    fprintf( ouf, "NGroups: %-15ldNFiles:   %-46ld\n", ngroups, nfiles );

    fprintf( ouf, "%-80.80s\n", " " );

    fprintf( ouf, "%-27.27s %-52.52s\n", "Tape filename", "Disk filename" );

    fprintf( ouf, "%-27.27s %-52.52s\n", "---------------------------",
        "----------------------------------------------------" );

    }

/*
  Update the number of groups and files, add the disk and tape
  file names.
--------------------------------------------------------------*/
else
    {
    /* Get the sceneid */
    var = p_find( &vblock, "SCENEID" );
    strcpy( sceneid, SVAL( *var, 0 ));

    /* open the output file for updating */
    ouf = fopen( outfile, "r+" );
    if ( ouf == NULL )
        {
	c_lasmsg(FALSE,"mklevel1b-open","Error opening file listing");
	return E_FAIL;
	}

    fseek( ouf, NGROUP_POS, SEEK_SET );
    fscanf( ouf, "%ld", &ngroups );
    ngroups++;
    nfiles = ngroups * 2;
    fseek( ouf, NGROUP_POS, SEEK_SET );
    fprintf( ouf, "%-15ld", ngroups );
    fseek( ouf, NFILE_POS, SEEK_SET );
    fprintf( ouf, "%-15ld", nfiles );

    /* Position the file pointer at EOF */
    fseek( ouf, 0, SEEK_END );

    /* print the tape and disk name for the scene */
    sprintf( level1b, "%s.l1b", sceneid );
    sprintf( archive, "%s.ahdr", sceneid );
    fprintf( ouf, "%-27.27s %-1s\n", level1b, archive );
    sprintf( archive, "%s.arch", sceneid );
    fprintf( ouf, "%-27.27s %-1s\n", level1b, archive );

    }

/*
  Close the file and return
----------------------------*/
fclose( ouf );
return E_SUCC;

}
