/****************************************************************************
NAME:		GETPAR

PURPOSE:	Get the input parameters
*****************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "tae.h"

FUNCTION getpar(vblock,calgrey,caldens,calcount,congrey,condens,concount,
		from,to,encount,lutname,subcmd,last,disk,fopt,prtfil,outfil)

struct PARBLK *vblock;	/* Parameter block 				     */

long	calgrey[],	/* Calibration grey level values		     */
	congrey[],	/* Control grey level values			     */
	from[],		/* Enhancement from grey level values   	     */
	*calcount,	/* Number of calibration pairs specified	     */
	*concount,	/* Number of control pairs specified		     */
	*encount,	/* Number of enhancement pairs specified	     */
	*last,		/* Flag indicating last LUT table		     */
	*fopt;          /* Output to file option			     */


float	caldens[],	/* Calibration density values			     */
	condens[],	/* Control density values			     */
	to[];		/* Enhancement to grey level values		     */

char 	*lutname,	/* Name of LUT being generated			     */
	*subcmd,	/* Subcommand the function was run with		     */
	*disk,		/* Disk location of the saved LUTs		     */
	*prtfil,        /* Temporary print file name			     */
	*outfil;        /* List of output files created			     */
{
struct VARIABLE *p_find(); /* TAE function 			             */
struct VARIABLE *nam;	   /* Pointer to the parameter in the variable block */

long	i,		/* Looping index				     */
        topt,           /* Output to terminal option			     */
        lopt,           /* Output to line printer option		     */
	fatal = -1,     /* Fatal error flag                     	     */
	nonfatal = 1;   /* Nonfatal error flag                  	     */

char	*getenv();	/* Function to get environment variable              */

/*
  Retrieve the ADAPS tables disk location
*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(disk,getenv("ADAPSTABLES"));
else
    {
    c_errmsg("Error retreiving value of ADAPSTABLES environment variable",
	     "modlut-find",&nonfatal);
    c_errmsg("Fatal error encountered","modlut-fatal",&fatal);
    }

/*
  Get name of LUT being defined
*/
nam = p_find (vblock,"INLUT");
strcpy(lutname,SVAL(*nam,0));

/* 
  Get calibration grey level and density values
*/
nam = p_find(vblock, "CALGREY");
*calcount = (*nam).v_count;
for (i = 0; i < *calcount; i++)
    calgrey[i] = IVAL(*nam,i);

nam = p_find(vblock, "CALDENS");
if ((*nam).v_count != *calcount)
    {
    c_errmsg(
        "The number of calibration grey level and density values must be equal",
        "modlut-calnum", &nonfatal);
    c_errmsg("Fatal error encountered","modlut-fatal",&fatal);
    }

for (i = 0; i < *calcount; i++)
   caldens[i] = RVAL(*nam,i);

/*
  Sort the calibration grey level and density values
*/
sortc(calgrey,caldens,*calcount);

/* 
  Get control grey level and density values
*/
nam = p_find(vblock, "CONGREY");
*concount = (*nam).v_count;
for (i = 0; i < *concount; i++)
    congrey[i] = IVAL(*nam,i);

nam = p_find(vblock, "CONDENS");
if ((*nam).v_count != *concount)
    {
    c_errmsg(
            "The number of control grey level and density values must be equal",
	    "modlut-connum", &nonfatal);
    c_errmsg("Fatal error encountered","modlut-fatal",&fatal);
    }

for (i = 0; i < *concount; i++)
   condens[i] = RVAL(*nam,i);

/* 
  Sort the control grey levels and density values
*/
sortc(congrey,condens,*concount);

/* 
  Get enhancement from and to grey level values
*/
nam = p_find(vblock, "FROM");
*encount = (*nam).v_count;
for (i = 0; i < *encount; i++)
    from[i] = IVAL(*nam,i);

nam = p_find(vblock, "TO");
if ((*nam).v_count != *encount)
    {
    c_errmsg("The number of enhancement from and to values must be equal",
	     "modlut-enhnum", &nonfatal);
    c_errmsg("Fatal error encountered","modlut-fatal",&fatal);
    }

for (i = 0; i < *encount; i++)
   to[i] = IVAL(*nam,i);

/*
  Sort the arrays based on the FROM array
*/
sortc(from,to,*encount);

/*
  Get specified subcommand
*/
nam = p_find(vblock,"SUBCMD");
strcpy(subcmd,SVAL(*nam,0));

/*
  Get flag indicating last LUT table
*/
nam = p_find(vblock, "LAST");
*last = IVAL(*nam,0);

if (strcmp(subcmd,"LIST") == 0)
    {
    /*
      Get the print file name
    */
    nam = p_find(vblock,"PRTFILE");
    strcpy(prtfil,SVAL(*nam,0));
 
    /*
      Get print option and open output file, if necessary
    */
    if (strlen(prtfil) == 0)
        {
        strcpy(prtfil,"modlut");
        if (c_setprt(vblock,&topt,fopt,&lopt,prtfil,outfil) != E_SUCC)
            c_errmsg("Fatal error encountered","modlut-fatal",&fatal);
        }
    }
 
return;
}
