/****************************************************************************
NAME:		PIECE

PURPOSE:	Generate a piece-wise linear LUT from a set of FROM and TO
		values.
*****************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "modlut.h"

FUNCTION piece(from,to,nelements,enlut)

long	from[];		/* Enhancement FROM grey level values             */

float	to[],		/* Enhancement TO grey level values               */
	enlut[];	/* Enhancement LUT                                */

long	nelements;	/* Number of FROM/TO elements			    */
{
long	i,		/* Looping variable				    */
	icnt;		/* Place counter				    */

float	lowx,		/* Low value in the FROM array			    */
	highx,		/* High value in the FROM array			    */
	lowy,		/* Low value in the TO array			    */
	highy;		/* High value in the TO array			    */

/*
  Set the initial high/low values for the generation of a piece-wise linear LUT
*/
icnt  = 0;
lowx  = from[icnt];
highx = from[icnt + 1];
lowy  = to[icnt];
highy = to[icnt + 1];

/*
  Generate the piece-wise linear LUT
*/
for (i = 0; i < LUTSIZ; i++)
    {
    /*
      When the next element in the FROM array is reached reset the
      low/high values
    */
    if (i > highx)
	{
	icnt++;
	lowx  = from[icnt];
	highx = from[icnt + 1];
	lowy  = to[icnt];
	highy = to[icnt + 1];
	}

    /*
      Calculate the output LUT value
    */
    if (i >= lowx)
	enlut[i] = lowy + (highy - lowy) * (i - lowx) / (highx - lowx);
    else
	enlut[i] = lowy;
    }

return;
}
