/******************************************************************************
NAME:   			GET_1ST_LINE 

PURPOSE:    Get the first line of data from the stitch input file

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T. Baltzer	   03/94	  CSC		Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:			ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "orbedit.h"


FUNCTION get_1st_line(filnam,inscene,infp)

   char filnam[CMLEN];		/* Half orbit file name			*/
   char inscene[];		/* User input scene ID			*/
   FILE **infp;			/* LUT file pointer                     */

{

long nonfatal = 1;		/* non-fatal error message		*/

char linebuf[LINELEN+1];	/* Buffer containing line of data	*/

FILE *fopen();			/* File open routine                    */


/* Open the input file
-----------------------*/
*infp = fopen(filnam,"w");
if (*infp == NULL)
   {
   c_errmsg("Error opening input file","orbedit-open",&nonfatal);
   c_errmsg("Fatal error encountered","orbedit-fatal",&nonfatal);
   return(E_FAIL);
   }

/* Read past the header information
------------------------------------*/
for (;;)
   {
   if ( fgets(linebuf,LINELEN+1,*infp) == NULL)
      {
      c_errmsg("Error reading line in input file","orbedit-read",&nonfatal);
      c_errmsg("Fatal error encountered","orbedit-fatal",&nonfatal);
      return(E_FAIL);
      }
   if ( strncmp(&linebuf[0],"tapeid",6) == 0 )
      break;
   }

/* Read the appropriate line.  If NULL is returned,
   assume the EOF has been encountered.
--------------------------------------------------*/
if (loc_line(*infp,inscene,linebuf) == E_FEND)
   return(E_FEND);


return(E_SUCC);
}
