/******************************************************************************
NAME:   			LOC_LINE 

PURPOSE:    Locate the line of interest within the file.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T. Baltzer	   03/94	  CSB		Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:			ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "stitch.h"             /* Orbital stitch include file          */


FUNCTION loc_line(infp,sceneid,linebuf)

FILE *infp;			/* LUT file pointer			*/
char sceneid[];			/* User input scene ID			*/
char linebuf[LINELEN+1];	/* Buffer containing line of data	*/

{

long i;			/* Counter				*/
long nonfatal = 1;	/* Non-fatal error status		*/
long size = 16;		/* Size of scene ID			*/

char lnscene[17];	/* Scene from line of data		*/

/* Read a line, if NULL is returned, assume the EOF
   has been encountered. Don't print an error message,
   but return with an eof status.
   If the line read is a blank line, get the next
   line;  Otherwise, extract the scene ID and determine
   if it is the one specified by the user.
   Parse the scene id AH11062392072142
                      AH                    HL
                        11                  NOAA satellite number
                          06   		    month of pass
                            23              day of pass
                              92            year of pass
                                07          start time of pass (hours)
                                  21        start time of pass (minutes)
                                    42      start time of pass (seconds)
------------------------------------------------------------------------*/
for (;;)
   {
   if ( fgets(linebuf,LINELEN+1,infp) == NULL)
      {
      c_errmsg("Scene ID not located","orbedit-sloc",&nonfatal);
      return(E_FEND);
      }
   if (strcmp(linebuf,"\n") != 0)
      {
      strncpy(&lnscene[0],&linebuf[7],2);
      strncpy(&lnscene[2],&linebuf[37],2);
      strncpy(&lnscene[4],&linebuf[10],2);
      strncpy(&lnscene[6],&linebuf[13],2);
      strncpy(&lnscene[8],&linebuf[16],2);
      strncpy(&lnscene[10],&linebuf[19],2);
      strncpy(&lnscene[12],&linebuf[21],2);
      strncpy(&lnscene[14],&linebuf[23],2);
      lnscene[16] = '\0';
      for (i = 2; i < 16; i++)
         if (lnscene[i] == ' ') 
            lnscene[i] = '0';  	/* left justified implicit 0's */
      c_up2low(lnscene,&size);	/* lower case the scene ID     */
      if (strcmp(lnscene,sceneid) == 0)
         break;
      }
   }


return(E_SUCC);
}
