/******************************************************************************
FUNCTION:	ORBEDIT

PURPOSE:	Edit the status flags in an orbital stitch list.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       03/94    T. Baltzer		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			ADAPS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"		/* LAS include			       */
#include "stitch.h"             /* Orbital stitch include file	       */


main()
{
struct PARBLK   vblock;		/* TAE parameter block			*/

char filnam[CMLEN];		/* Name of input file			*/
char lndflg[2];			/* Line detect flag L=Line detected	*/
char locflg[2];			/* Location flag S=Silo D=Disk		*/
char orbflg[2];			/* Stitched flag O=Stitched orbit	*/
char sceneid[17];		/* Scene identifier			*/

long count = 0;			/* Count of files generated		*/
long fatal = -1;		/* Fatal error status			*/
long nonfatal = 1;		/* Non-fatal error status		*/
long status;			/* Return status			*/
long std_out;			/* Standard output device 		*/

FILE *infp;			/* Input file pointer			*/


/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);


/*
  Get the input parameters
*/
status = getpar(&vblock,filnam,sceneid,locflg,lndflg,orbflg);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);


/*
  Open the input file, read past the header,
  and locate the appropriate line within the file
*/
status = get_line(filnam,sceneid,&infp);
if (status != E_SUCC)
   {
   fclose(infp);
   c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);
   }


/*
  Write the new flag status to the line of the file if the user
  has specified a value.  Otherwise, just skip over that flag.
*/
wrt_flg(infp,locflg,lndflg,orbflg);


/*
  Close the input file
*/
if ( fclose(infp) != E_SUCC )
   {
   c_errmsg("Error closing input file","orbedit-clos",&nonfatal);
   c_errmsg("Nonfatal error encountered","orbedit-warn",&nonfatal);
   }


/*
  Exit with a success status
*/
c_complt("ORBEDIT","",&count);
}
