/******************************************************************************
FUNCTION:	WRT_FLG

PURPOSE:	Write the user specified flags to the file.  This function
		assumes the file pointer is at the end of the desired line.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       03/94    T. Baltzer		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			ADAPS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"		/* LAS include			       */
#include "stitch.h"             /* Orbital stitch include file	       */


FUNCTION wrt_flg(infp,locflg,lndflg,orbflg)

   FILE *infp;			/* Input file pointer			*/
   char locflg[];		/* Location flag S=Silo D=Disk		*/
   char lndflg[];		/* Line detect flag L=Line detected	*/
   char orbflg[];		/* Stitched flag O=Stitched orbit	*/

{

long fatal = -1;		/* Fatal error status			*/
long nonfatal = 1;		/* Non-fatal error status		*/
long status;			/* Return status			*/


/*
  Position the file pointer to the "flags" position on the line
*/
status = fseek(infp,-3,SEEK_CUR);
if (status != E_SUCC)
   {
   fclose(infp);
   c_errmsg("Error seeking in the file","orbedit-seek",&nonfatal);
   c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);
   }


/*
  Write the new flag status to the line of the file if the user
  has specified a value.  Otherwise, just skip over that flag.
*/
if (strcmp(locflg,"") != 0)
   {
   status = fputs(locflg,infp);
   if (status == EOF)
      {
      fclose(infp);
      c_errmsg("Error writing to the file","orbedit-writ",&nonfatal);
      c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);
      }
   }
else
   {
   status = fseek(infp,1,SEEK_CUR);
   if (status != E_SUCC)
      {
      fclose(infp);
      c_errmsg("Error seeking in the file","orbedit-seek",&nonfatal);
      c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);
      }
   }
if (strcmp(lndflg,"") != 0)
   {
   status = fputs(lndflg,infp);
   if (status == EOF)
      {
      fclose(infp);
      c_errmsg("Error writing to the file","orbedit-writ",&nonfatal);
      c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);
      }
   }
else
   {
   status = fseek(infp,1,SEEK_CUR);
   if (status != E_SUCC)
      {
      fclose(infp);
      c_errmsg("Error seeking in the file","orbedit-seek",&nonfatal);
      c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);
      }
   }
if (strcmp(orbflg,"") != 0)
   {
   status = fputs(orbflg,infp);
   if (status == EOF)
      {
      fclose(infp);
      c_errmsg("Error writing to the file","orbedit-writ",&nonfatal);
      c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);
      }
   }
else
   {
   status = fseek(infp,1,SEEK_CUR);
   if (status != E_SUCC)
      {
      fclose(infp);
      c_errmsg("Error seeking in the file","orbedit-seek",&nonfatal);
      c_errmsg("Fatal error encountered","orbedit-fatal",&fatal);
      }
   }


}
