/*******************************************************************************
FUNCTION:	readarch_jul

PURPOSE:	Read AVHRR archive images from a JULIAN archive tape.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.2    08/93  J.Fenno          Development


PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "adaps.h"


/* System return status
=======================*/
#ifdef vms
#define SYS_SUCC      1    /* VMS  successful exit status                     */
#else
#define SYS_SUCC      0    /* UNIX successful exit status                     */
#endif


main()
{
struct PARBLK   vblock;    /* TAE parameter block                             */
struct VARIABLE *p_find(); /* Function to retrieve a TAE variable             */
struct VARIABLE *parm;     /* Pointer to TAE variable within vblock           */

long i;			   /* Counter					      */
long len;		   /* Length of the scene id			      */
long nfiles;               /* Number of files returned by c_getfiles          */
long nimages  =  0;        /* Number of archive images read to disk           */
long fatal    = -1;        /* Fatal error code                                */
long stdlun;               /* Standard output                                 */

char *images[20];          /* Scenes actually copied to disk by ansiin        */
char drive[CMLEN];         /* Tape drive the tape is mounted on               */
char outdir[CMLEN];        /* Disk location where the images should be placed */
char scene_id[20];	   /* Scene ID requested by the user                  */
char str[CMLEN];           /* Temporary string                                */
char sys_command[4*CMLEN]; /* Command sent to the operating system            */
char tapeid[CMLEN];        /* Tape ID                                         */
char tmpnames[2][CMLEN];   /* Temporary file names                            */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &stdlun);


/* Get the tape drive that the tape is mounted on
-------------------------------------------------*/
parm = p_find(&vblock, "DRIVE");
strcpy(drive, SVAL(*parm, 0));


/* Get the tape ID
------------------*/
parm = p_find(&vblock, "TAPEID");
strcpy(tapeid, SVAL(*parm, 0));


/* Get the scene ID of the image to be read.  Lower case the scene ID and
   add a '.' since all files written to an archive tape end in a '.'
--------------------------------------------------------------------------*/
parm = p_find(&vblock, "SCENEID");
strcpy(scene_id, SVAL(*parm, 0));
len = strlen(scene_id);
c_up2low(scene_id, &len);
strcat(scene_id, ".");



/* Get the directory where the image should be placed and then chdir to that
   directory.  After calling c_cmhost the last character for UNIX directories
   should be a '/' and for VMS it should be ']'.
-----------------------------------------------------------------------------*/
parm = p_find(&vblock, "OUTDIR");
if (parm->v_count != 0)
    {
    strcpy(str, SVAL(*parm, 0));
    c_cmhost(str, outdir);
    strcat( outdir, "/" );
    if (outdir[strlen(outdir) - 1] != HOST_DIR_END)
	c_errmsg("Invalid directory specification","readarch-outdir",&fatal);
    if (chdir(outdir))
	c_errmsg("Error moving to specified directory","readarch-chdir",&fatal);
    }


#ifdef vms

    /* Set up the ansiin command that will read the scene from tape.  VMS
       systems must have double quotes around the argument string so that all
       lowercase letters are not converted to uppercase.
    -------------------------------------------------------------------------*/
    sprintf(sys_command, "ansiin \"-Tvl\" %s %s", drive, tapeid);

#else

    /* Set up the ansiin command that will read the scene from tape.  UNIX
       systems must have single quotes around the scene IDs to protect any
       asterisks from shell expansion.
    -------------------------------------------------------------------------*/
    sprintf(sys_command, "ansiin -Tvl %s %s", drive, tapeid);

#endif


/* Make the system call to ansiin
---------------------------------*/
if (system(sys_command) != SYS_SUCC)
    c_errmsg("Error reading image(s) from tape", "readarch-read", &fatal);


/* Find all of the files read from tape.  There should
   only be one 15 character scene ID found.
------------------------------------------------------*/
if (c_getfiles("a*.", tmpnames, &nfiles, 2, TRUE, FALSE))
    c_errmsg("Error returned by c_getfiles", "readarch-support", &fatal);
if (nfiles != 1)
    c_errmsg("Multiple images found from JULIAN tape", "readarch-njul", &fatal);


/* Rename the image so it appears that it was read
   from tape with a 16 character scene ID.
--------------------------------------------------*/
sprintf(tmpnames[1], "%s%s", outdir, scene_id);
if (c_dkmove(tmpnames[0], tmpnames[1], NULL))
    c_errmsg("Error returned by c_dkmove", "readarch-support", &fatal);


/* Rename all of the files read from tape.  May have GAC,
   HRPT, or LAC images intermixed on a single tape.
---------------------------------------------------------*/
rename_files("ag*.", "eaf", images, &nimages);
rename_files("ah*.", "eaf", images, &nimages);
rename_files("al*.", "eaf", images, &nimages);


/* Return the LAS names of each archive image read to disk
----------------------------------------------------------*/
q_string(&vblock, "SCENES", nimages, images, P_UPDATE);
if (q_out(&vblock) != SUCCESS)
    {
    strcpy(str, "Error sending parameters to TAE");
    c_errmsg(str, "readarch-parm", &fatal);
    }
for (i = 0; i < nimages; i++)
    free(images[i]);

z_exit(1, "READARCH_JUL completed successfully");
}
