/*******************************************************************************
FUNCTION:	rename_files

PURPOSE:	Rename a set of AVHRR archive image or header files that have
		been read from an archive tape.  Header files will be of the
		form hg*. hh*. hl*. or ho*. and should be renamed to ag*.ahdr
		ah*.ahdr al*.ahdr or ao*.ahdr.  Image files will be of the form
		ig*. ih*. il*. or io*. and should be renamed to ag*.arch
		ah*.arch al*.arch or ao*.arch.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/92  J.Fenno          Development
  1.1    07/93  J.Fenno          Stolen from READARCH


PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"


FUNCTION rename_files(filename, extension, images, nimages)

   char *filename;         /* IN - Specification for the files to be renamed  */
   char *extension;        /* IN - Extension for the files that are renamed   */
   char *images[];         /* IN/OUT - Archive images that have been renamed  */
   long *nimages;          /* IN/OUT - Number of archive image renamed        */
{
char dir[CMLEN];           /* Directory portion of a file name                */
char ext[CMLEN];           /* Extension of a file name                        */
char files[1][CMLEN];      /* Names of the files returned by c_getfiles       */
char newname[CMLEN];       /* New name for a file                             */
char root[CMLEN];          /* Root portion of a file name                     */

long fatal    = -1;        /* Fatal error flag                                */
long nfiles;               /* Number of files returned by c_getfiles          */


/* Get the first file that matches the given file name
------------------------------------------------------*/
if (c_getfiles(filename, files, &nfiles, 1, TRUE, FALSE))
    c_errmsg("Error returned by c_getfiles", "readarch-support", &fatal);


/* While there is a file matching the given specification rename it
-------------------------------------------------------------------*/
while (nfiles > 0)
    {
    /* Take the file name returned from c_getfiles and create the new file
       name.  Replace the first character with an 'a' and add either the ahdr
       or arch extension.
    -------------------------------------------------------------------------*/
    c_splitname(files, dir, root, ext);
    sprintf(newname, "%sa%s.%s", dir, &root[1], extension);


    /* Rename the file
    ------------------*/
    if (c_dkmove(files, newname, NULL))
	c_errmsg("Error returned by c_dkmove", "readarch-support", &fatal);
    

    /* If the extension isn't ahdr then save the
       root name of the image copied to disk
    --------------------------------------------*/
    if (strcmp(extension, "ahdr") != 0)
	{
	c_splitname(newname, dir, root, ext);
	images[*nimages] = malloc(strlen(root) + 1);
	if (images[*nimages] == NULL)
	    c_errmsg("Error allocating memory", "readarch-alloc", &fatal);
	strcpy(images[*nimages], root);
	(*nimages)++;
	}


    /* Get the next file that matches the given file name
    -----------------------------------------------------*/
    if (c_getfiles(filename, files, &nfiles, 1, TRUE, FALSE))
	c_errmsg("Error returned by c_getfiles", "readarch-support", &fatal);
    }
}
