/*******************************************************************************
FUNCTION:	c_closelui

PURPOSE:	Given a pointer to an opened Look Up Image (LUI), close the
		file and deallocate its buffer.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    03/93  J.Fenno          Development


PROJECT:  ADAPS
*******************************************************************************/
#include "rectify.h"


/* Reference the array of LUI structures
----------------------------------------*/
extern long firstlui;     /* Flag indicating no LUIs have been loaded yet     */
extern long lui_access[]; /* Number of pixels accessed outside of each LUI    */
extern struct LUI lui[];  /* Array of MAXLUIS Look Up Images                  */


int c_closelui
(
    long  i              /* IN - Ptr to an opend LUI image                    */
)
{
char str[CMLEN];         /* Temporary string                                  */
long nonfatal = 1;       /* Nonfatal error flag                               */


/* Ensure i points to a valid LUI
---------------------------------*/
if ((i < 0) || (i >= MAXLUIS))
    {
    c_errmsg("Invalid look up image specified", "c_closelui-lui", &nonfatal);
    return(E_FAIL);
    }


/* If no LUIs have been opened then return
------------------------------------------*/
if (firstlui)
    return(E_SUCC);


/* If the file descriptor has a value then close the file
---------------------------------------------------------*/
if (lui[i].fdesc)
    c_eclose(&lui[i].fdesc);


/* If the buffer is non-null
----------------------------*/
if (lui[i].buffer != NULL)
    free(lui[i].buffer);


/* Issue a message if the user accessed pixels outside of the LUI
-----------------------------------------------------------------*/
if (lui_access[i])
    {
    sprintf(str, "Accessed %1d pixels outside look up image", lui_access[i]);
    c_errmsg(str, "c_closelui-out", &nonfatal);
    c_errmsg("Nonfatal error encountered", "c_closelui-warn", &nonfatal);
    }


lui[i].fdesc  = 0;
lui[i].buffer = NULL;
return(E_SUCC);
}
