/*******************************************************************************
FUNCTION:	convertgrid

PURPOSE:	Convert a new geometric mapping grid from a GRID structure to
		an old GEOGRID structure.  There are five dynamically allocated
		memory locations in both grid structures.  The fields within
		the old structure will just point to the already allocated
		memory in the new structure.  Thus, there is no need to free
		the memory within the old structure before the program exits.

		Once the GEOM package and c_grdred are updated to use the new
		structure this function can be deleted.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    02/93  J.Fenno          Development

PROJECT:  ADAPS
*******************************************************************************/
#include "rectify.h"


int convertgrid
(
    struct GRID    *new,  /* IN  - A new GRID structure                       */
    struct GEOGRID *old   /* OUT - An old GEOGRID structure                   */
)
{
long  i;                  /* Loop control                                     */


/* Initialize every field of old
--------------------------------*/
strcpy(old->model, new->model);
strcpy(old->units, new->units);
for (i = 0; i < 32; i++)
    old->act_grid_err[i] = new->act_grid_err[i];
for (i = 0; i < 2; i++)
    old->ave_fit_err[i]  = new->ave_fit_err[i];
for (i = 0; i < 2; i++)
    old->ave_grid_err[i] = new->ave_grid_err[i];
old->coeffs = new->coeffs;                      /* shared memory */
for (i = 0; i < 8; i++)
    old->corners[i] = new->corners[i];
old->grid_tol = new->grid_tol;
for (i = 0; i < 2; i++)
    old->max_fit_err[i] = new->max_fit_err[i];
for (i = 0; i < 2; i++)
    old->max_grid_err[i] = new->max_grid_err[i];
for (i = 0; i < 2; i++)
    old->pdist[i] = new->pdist[i];
for (i = 0; i < 15; i++)
    old->projprms[i] = new->projprms[i];
old->rms_fit_err   = new->rms_fit_err;
old->rms_grid_err  = new->rms_grid_err;
old->code          = new->code;
old->datum         = new->datum;
old->fiterr_valid  = new->fiterr_valid;
old->griderr_valid = new->griderr_valid;
old->lines         = new->lines;
old->ncoeff        = new->ncoeff;
old->ncols         = new->ncols;
old->nrows         = new->nrows;
old->out_lines     = new->out_lines;            /* shared memory */
old->out_samps     = new->out_samps;            /* shared memory */
old->proj_valid    = new->proj_valid;
old->samples       = new->samples;
old->zone          = new->zone;
old->in_lines      = (double *)new->values[0];  /* shared memory */
old->in_samps      = (double *)new->values[1];  /* shared memory */
return(E_SUCC);
}
