/*******************************************************************************
FUNCTION:	initgrid

PURPOSE:	Initialize a grid structure


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  ???    ?????  ???????          Devlepment
  1.0    03/93  J.Fenno          Restructured to initialize a new grid structure

PROJECT:  ADAPS
*******************************************************************************/
#include <math.h>
#include "rectify.h"

extern double ceil();

int initgrid
(
    struct GRID  *grid,   /* OUT - An initialized grid structure              */
    struct FRAME *frame,  /* IN  - Output framing parameters                  */
    struct PROJ  *outp,   /* IN  - Output projection parameters               */
    long   satnum         /* IN  - Number of satellite that acquired the data */
)
{
double temp;              /* Temporary variable for remainder test            */

long  i;                  /* Loop control                                     */


/* Determine the number of grid rows and grid columns for this image
--------------------------------------------------------------------*/
temp = ((double)frame->window[NL] / (double)frame->ngline) + 1.0;
grid->nrows = (long)ceil(temp);
temp = ((double)frame->window[NS] / (double)frame->ngsamp) + 1.0;
grid->ncols = (long)ceil(temp);


/* Allocate enough memory to store the grid rows and columns
------------------------------------------------------------*/
grid->out_lines = (long *)malloc(grid->nrows * sizeof(long));
grid->out_samps = (long *)malloc(grid->ncols * sizeof(long));
if ((grid->out_lines == NULL) || (grid->out_samps == NULL))
    return(NO_SPACE);


/* Initialize every field of the grid
-------------------------------------*/
strcpy(grid->gridname, "");
for (i = 0; i < MAXGRIDVALS; i++)
    strcpy(grid->labels[i], "");
strcpy(grid->model, "SPACECRAFT");
switch (outp->units)
    {
    case 0:
	strcpy(grid->units,"radians");
	break;
    case 1:
	strcpy(grid->units,"feet");
	break;
    case 2:
	strcpy(grid->units, "meters");
	break;
    case 3:
	strcpy(grid->units, "seconds");
	break;
    case 4:
	strcpy(grid->units, "degrees");
	break;
    case 5:
	strcpy(grid->units, "dms");
	break;
    default:
	strcpy(grid->units, "");
    }
for (i = 0; i < 32; i++)
    grid->act_grid_err[i] = 0;
for (i = 0; i < 2; i++)
    grid->ave_fit_err[i]  = 0;
for (i = 0; i < 2; i++)
    grid->ave_grid_err[i] = 0;
grid->coeffs = NULL;
for (i = 0; i < 8; i++)
    grid->corners[i] = frame->corners[i];
grid->grid_tol = frame->tol;
for (i = 0; i < 2; i++)
    grid->max_fit_err[i] = 0;
for (i = 0; i < 2; i++)
    grid->max_grid_err[i] = 0;
for (i = 0; i < 2; i++)
    grid->pdist[i] = frame->o_dist[i];
for (i = 0; i < 15; i++)
    grid->projprms[i] = outp->proj_coef[i];
grid->rms_fit_err   = 0;
grid->rms_grid_err  = 0;
grid->code          = outp->proj_code;
grid->datum         = outp->datum_code;
grid->dtype         = ELONG;            /* temp type for c_copygrid */
grid->fiterr_valid  = GEO_NOTVALID;
grid->griderr_valid = GEO_NOTVALID;
grid->lines         = frame->window[NL];
grid->ncoeff        = 0;
grid->nvalues       = 0;
for (i = 0; i < grid->nrows; i++)
    grid->out_lines[i] = frame->window[SL] + (i * frame->ngline);
for (i = 0; i < grid->ncols; i++)
    grid->out_samps[i] = frame->window[SS] + (i * frame->ngsamp);
grid->proj_valid    = GEO_VALID;
grid->samples       = frame->window[NS];
grid->satnum        = satnum;
grid->zone          = outp->zone_code;
for (i = 0; i < MAXGRIDVALS; i++)
    grid->values[i] = NULL;

return(E_SUCC);
}
