/*******************************************************************************
NAME		pxscale

PURPOSE		Unscale an input array given an input scale and offset
		and then rescale the array with a given output scale
		and offset.  The data type of the input array can
		also be converted to any specified output data type.

*******************************************************************************/
#include "worgen.h"
#include "pixman.h"


/*
  Globals to keep track of the number of pixels too large or small
*/
static unsigned long ntoobig   = 0.0;
static unsigned long ntoosmall = 0.0;



FUNCTION c_pxscale_report()
{
char str[CMLEN];
long nonfatal = 1;

sprintf(str, "%lu pixels were too small", ntoosmall);
c_errmsg(str, "informational", &nonfatal);

sprintf(str, "%lu pixels were too large", ntoobig);
c_errmsg(str, "informational", &nonfatal);

return(E_SUCC);
}



/*
  Function prototypes
*/
static byt_byt ();
static byt_wrd ();
static byt_uwrd ();
static byt_lng ();
static byt_ulng ();
static byt_flt ();
static byt_dbl ();
static wrd_byt ();
static wrd_wrd ();
static wrd_uwrd ();
static wrd_lng ();
static wrd_ulng ();
static wrd_flt ();
static wrd_dbl ();
static uwrd_byt ();
static uwrd_wrd ();
static uwrd_uwrd ();
static uwrd_lng ();
static uwrd_ulng ();
static uwrd_flt ();
static uwrd_dbl ();
static lng_byt ();
static lng_wrd ();
static lng_uwrd ();
static lng_lng ();
static lng_ulng ();
static lng_flt ();
static lng_dbl ();
static ulng_byt ();
static ulng_wrd ();
static ulng_uwrd ();
static ulng_lng ();
static ulng_ulng ();
static ulng_flt ();
static ulng_dbl ();
static flt_byt ();
static flt_wrd ();
static flt_uwrd ();
static flt_lng ();
static flt_ulng ();
static flt_flt ();
static flt_dbl ();
static dbl_byt ();
static dbl_wrd ();
static dbl_uwrd ();
static dbl_lng ();
static dbl_ulng ();
static dbl_flt ();
static dbl_dbl ();


FUNCTION c_pxscale (in, idtype, ns, iscale, ioffset, imask, nmasks, out,
                    odtype, oscale, ooffset, omask, omax, omin, rndopt)

    void *in;		/* IN  - Input buffer containing pixels to rescale    */
    long idtype;	/* IN  - Input data type                              */
    long ns;		/* IN  - Number of samples in the input buffer        */
    double iscale;	/* IN  - Scaling factor for the input buffer          */
    double ioffset;	/* IN  - Offset value for the input buffer            */
    long imask[];	/* IN  - Mask values contained in the input buffer    */
    long nmasks;	/* IN  - Number of mask values in imask and omask     */
    void *out;		/* OUT - Output buffer of rescaled pixels             */
    long odtype;	/* IN  - Output data type                             */
    double oscale;	/* IN  - Scaling factor for the output buffer         */
    double ooffset;	/* IN  - Offset value for the output buffer           */
    long omask[];	/* IN  - Matching output mask values for imask        */
    double omax;	/* IN  - Maximum output pixel value                   */
    double omin;	/* IN  - Minimum output pixel value                   */
    long rndopt;	/* IN  - Rounding option flag                         */
{
long	status = E_SUCC;	/* Return status			*/

double rnd;			/* Rounding value			*/


/*
  Set the rounding value
*/
if (rndopt)
    rnd = 0.5;
else
    rnd = 0.0;

if (idtype == EBYTE)
    {
    if (odtype == EBYTE)
	byt_byt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EWORD)
	byt_wrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EUWORD)
	byt_uwrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == ELONG)
	byt_lng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EULONG)
	byt_ulng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EREAL)
	byt_flt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EDOUBLE)
	byt_dbl(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    }
else if (idtype == EWORD)
    {
    if (odtype == EBYTE)
	wrd_byt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EWORD)
	wrd_wrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EUWORD)
	wrd_uwrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == ELONG)
	wrd_lng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EULONG)
	wrd_ulng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EREAL)
	wrd_flt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EDOUBLE)
	wrd_dbl(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    }
else if (idtype == EUWORD)
    {
    if (odtype == EBYTE)
	uwrd_byt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EWORD)
	uwrd_wrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EUWORD)
	uwrd_uwrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == ELONG)
	uwrd_lng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EULONG)
	uwrd_ulng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EREAL)
	uwrd_flt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EDOUBLE)
	uwrd_dbl(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    }
else if (idtype == ELONG)
    {
    if (odtype == EBYTE)
	lng_byt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EWORD)
	lng_wrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EUWORD)
	lng_uwrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == ELONG)
	lng_lng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EULONG)
	lng_ulng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EREAL)
	lng_flt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EDOUBLE)
	lng_dbl(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    }
else if (idtype == EULONG)
    {
    if (odtype == EBYTE)
	ulng_byt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EWORD)
	ulng_wrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EUWORD)
	ulng_uwrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == ELONG)
	ulng_lng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EULONG)
	ulng_ulng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EREAL)
	ulng_flt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EDOUBLE)
	ulng_dbl(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    }
else if (idtype == EREAL)
    {
    if (odtype == EBYTE)
	flt_byt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EWORD)
	flt_wrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EUWORD)
	flt_uwrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == ELONG)
	flt_lng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EULONG)
	flt_ulng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EREAL)
	flt_flt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EDOUBLE)
	flt_dbl(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    }
else if (idtype == EDOUBLE)
    {
    if (odtype == EBYTE)
	dbl_byt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EWORD)
	dbl_wrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EUWORD)
	dbl_uwrd(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == ELONG)
	dbl_lng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EULONG)
	dbl_ulng(in,out,ns,&status,iscale,ioffset,oscale,ooffset,rnd,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EREAL)
	dbl_flt(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    else if (odtype == EDOUBLE)
	dbl_dbl(in,out,ns,&status,iscale,ioffset,oscale,ooffset,imask,
	    omask,nmasks,omax,omin);
    }

return(status);
}



static byt_byt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
                rnd,imask,omask,nmasks,omax,omin)

unsigned char *in;		/* Input buffer				*/
unsigned char *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double iscale,ioffset;		/* Input scale and offset values	*/
double oscale,ooffset;		/* Output scale and offset values	*/
double rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double	tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}



static byt_wrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
                rnd,imask,omask,nmasks,omax,omin)

unsigned char *in;		/* Input buffer				*/
short *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double iscale,ioffset;		/* Input scale and offset values	*/
double oscale,ooffset;		/* Output scale and offset values	*/
double rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}



static byt_uwrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
                 rnd,imask,omask,nmasks,omax,omin)

unsigned char *in;		/* Input buffer				*/
unsigned short *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}



static byt_lng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
                rnd,imask,omask,nmasks,omax,omin)

unsigned char *in;		/* Input buffer				*/
long *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}



static byt_ulng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
                 rnd,imask,omask,nmasks,omax,omin)

unsigned char *in;		/* Input buffer				*/
unsigned long *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}



static byt_flt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                omask,nmasks,omax,omin)

unsigned char *in;		/* Input buffer				*/
float *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	*out = (float)tmp;
    }
}



static byt_dbl (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                omask,nmasks,omax,omin)

unsigned char *in;		/* Input buffer				*/
double *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    *out = omask[i];
	    break;
	    }
    if (i == nmasks)
	*out = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (*out > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (*out < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    }
}



static wrd_byt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
                rnd,imask,omask,nmasks,omax,omin)

short *in;			/* Input buffer				*/
unsigned char *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}



static wrd_wrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

short *in;			/* Input buffer				*/
short *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}



static wrd_uwrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

short *in;			/* Input buffer				*/
unsigned short *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}



static wrd_lng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

short *in;			/* Input buffer				*/
long *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}



static wrd_ulng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

short *in;			/* Input buffer				*/
unsigned long *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}



static wrd_flt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                omask,nmasks,omax,omin)

short *in;			/* Input buffer				*/
float *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	*out = (float)tmp;
    }
}



static wrd_dbl (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                omask,nmasks,omax,omin)

short *in;			/* Input buffer				*/
double *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    *out = omask[i];
	    break;
	    }
    if (i == nmasks)
	*out = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (*out > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (*out < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    }
}



static uwrd_byt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

unsigned short *in;		/* Input buffer				*/
unsigned char *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}



static uwrd_wrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

unsigned short *in;		/* Input buffer				*/
short *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}



static uwrd_uwrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		  rnd,imask,omask,nmasks,omax,omin)

unsigned short *in;		/* Input buffer				*/
unsigned short *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}



static uwrd_lng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

unsigned short *in;		/* Input buffer				*/
long *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}



static uwrd_ulng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		  rnd,imask,omask,nmasks,omax,omin)

unsigned short *in;		/* Input buffer				*/
unsigned long *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}



static uwrd_flt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                 omask,nmasks,omax,omin)

unsigned short *in;		/* Input buffer				*/
float *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	*out = (float)tmp;
    }
}



static uwrd_dbl (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                 omask,nmasks,omax,omin)

unsigned short *in;		/* Input buffer				*/
double *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    *out = omask[i];
	    break;
	    }
    if (i == nmasks)
	*out = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (*out > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (*out < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    }
}



static lng_byt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

long *in;			/* Input buffer				*/
unsigned char *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}



static lng_wrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

long *in;			/* Input buffer				*/
short *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
 	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}



static lng_uwrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

long *in;			/* Input buffer				*/
unsigned short *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
 	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}



static lng_lng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

long *in;			/* Input buffer				*/
long *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}



static lng_ulng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

long *in;			/* Input buffer				*/
unsigned long *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}



static lng_flt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                omask,nmasks,omax,omin)

long *in;			/* Input buffer				*/
float *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	*out = (float)tmp;
    }
}



static lng_dbl (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                omask,nmasks,omax,omin)

long *in;			/* Input buffer				*/
double *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    *out = omask[i];
	    break;
	    }
    if (i == nmasks)
	*out = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (*out > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (*out < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    }
}



static ulng_byt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

unsigned long *in;		/* Input buffer				*/
unsigned char *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}



static ulng_wrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

unsigned long *in;		/* Input buffer				*/
short *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}



static ulng_uwrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		  rnd,imask,omask,nmasks,omax,omin)

unsigned long *in;		/* Input buffer				*/
unsigned short *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}



static ulng_lng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

unsigned long *in;		/* Input buffer				*/
long *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}



static ulng_ulng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		  rnd,imask,omask,nmasks,omax,omin)

unsigned long *in;		/* Input buffer				*/
unsigned long *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}



static ulng_flt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                 omask,nmasks,omax,omin)

unsigned long *in;		/* Input buffer				*/
float *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	*out = (float)tmp;
    }
}



static ulng_dbl (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                 omask,nmasks,omax,omin)

unsigned long *in;		/* Input buffer				*/
double *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    *out = omask[i];
	    break;
	    }
    if (i == nmasks)
	*out = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (*out > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (*out < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    }
}



static flt_byt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

float *in;			/* Input buffer				*/
unsigned char *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}



static flt_wrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

float *in;			/* Input buffer				*/
short *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}



static flt_uwrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

float *in;			/* Input buffer				*/
unsigned short *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}



static flt_lng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

float *in;			/* Input buffer				*/
long *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}



static flt_ulng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

float *in;			/* Input buffer				*/
unsigned long *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}



static flt_flt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
		omask,nmasks,omax,omin)

float *in;			/* Input buffer				*/
float *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	*out = (float)tmp;
    }
}



static flt_dbl (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                omask,nmasks,omax,omin)

float *in;			/* Input buffer				*/
double *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    *out = omask[i];
	    break;
	    }
    if (i == nmasks)
	*out = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (*out > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (*out < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    }
}



static dbl_byt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

double *in;			/* Input buffer			*/
unsigned char *out;		/* Output buffer		*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned char)(tmp - rnd);
	else
	    *out = (unsigned char)(tmp + rnd);
	}
    }
}



static dbl_wrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

double *in;			/* Input buffer				*/
short *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (short)(tmp - rnd);
	else
	    *out = (short)(tmp + rnd);
	}
    }
}



static dbl_uwrd (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

double *in;			/* Input buffer				*/
unsigned short *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned short)(tmp - rnd);
	else
	    *out = (unsigned short)(tmp + rnd);
	}
    }
}



static dbl_lng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		rnd,imask,omask,nmasks,omax,omin)

double *in;			/* Input buffer				*/
long *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (long)(tmp - rnd);
	else
	    *out = (long)(tmp + rnd);
	}
    }
}



static dbl_ulng (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		 rnd,imask,omask,nmasks,omax,omin)

double *in;			/* Input buffer				*/
unsigned long *out;		/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
double	rnd;			/* Rounding value			*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	{
	if (tmp < 0.0)
	    *out = (unsigned long)(tmp - rnd);
	else
	    *out = (unsigned long)(tmp + rnd);
	}
    }
}



static dbl_flt (in,out,ns,flag,iscale,ioffset,oscale,ooffset,
		imask,omask,nmasks,omax,omin)

double *in;			/* Input buffer				*/
float *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
double tmp;			/* Temporary calculation buffer		*/
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    tmp = omask[i];
	    break;
	    }
    if (i == nmasks)
	tmp = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (tmp > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (tmp < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    else
	*out = (float)tmp;
    }
}



static dbl_dbl (in,out,ns,flag,iscale,ioffset,oscale,ooffset,imask,
                omask,nmasks,omax,omin)

double *in;			/* Input buffer				*/
double *out;			/* Output buffer			*/
long ns;			/* Number of sample in the buffers      */
long *flag;			/* Bounds status flag			*/
double	iscale,ioffset;		/* Input scale and offset values	*/
double	oscale,ooffset;		/* Output scale and offset values	*/
long imask[];			/* Input mask values                    */
long omask[];			/* Output mask values                   */
long nmasks;			/* Number of mask values                */
double omax,omin;		/* Output max/min values                */
{
long i;				/* Loop control				*/

for(; ns--; in++, out++)
    {
    for (i=0; i<nmasks; i++)
	if (*in == imask[i])
	    {
	    *out = omask[i];
	    break;
	    }
    if (i == nmasks)
	*out = (((*in - ioffset) * iscale) * oscale) + ooffset;

    if (*out > omax)
	{
	*out = 0;
	*flag |= TOOBIG;
	ntoobig++;
	}
    else if (*out < omin)
	{
	*out = 0;
	*flag |= TOOSMALL;
	ntoosmall++;
	}
    }
}
