/*******************************************************************************
FUNCTION:	RESCALE

PURPOSE:	Unscale an image and then re-scale it as specified by the user.


VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  2.0	 D. Hollaren   01/94	Original development
  2.1    J. Fenno      01/95    Modifications for a new scaling method.  Fixed
				so the first band of the specified image isn't
				always the one that is rescaled.  Will now
				rescale the specified band.


PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "rescale.h"



/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern long curent;		/* current processing line                    */
extern long curcat;		/* current processing band                    */



main()
{
long
	bands[MAXIMG][MAXBND+1],	/* bands to be processed - all images*/
	idtype,				/* data type of input data           */
	imask[MAXMASKS],		/* Input image mask values           */
	msgtim,				/* elapsed time between display msg  */
	nbands[MAXIMG],			/* number of bands per input image   */
	nmasks,				/* Number of input image masks       */
	nmrimg,				/* number of input images            */
	odtype,				/* data type of output data          */
	omask[MAXMASKS],		/* Output image mask values          */
	one		= 1,		/* value of one			     */
	rndopt,				/* rounding option (TRUE, FALSE)     */
	std_out,			/* LUN to standard output    	     */
	totbnd,				/* total number of bands             */
	window[MAXIMG][4];		/* window for each input image       */

char
	hosin[MAXIMG][CMLEN],		/* host names of input images        */
	hosout[CMLEN],			/* host name of output image 	     */
	minmax[4],			/* min max flag                      */
	taein[MAXIMG][CMLEN],		/* TAE names of input images         */
	taeout[CMLEN];			/* TAE name of output image	     */

double	iscale[MAXBND+1],		/* Input scale factor		     */
	ioffset[MAXBND+1],		/* Input offset factor		     */
	oscale[MAXBND+1],		/* Output scale factor		     */
	ooffset[MAXBND+1],		/* Output offset factor		     */
	omax[MAXBND+1],			/* Maximum output pixel values       */
	omin[MAXBND+1];			/* Minimum output pixel values       */

struct PARBLK vblock;			/* TAE parameter block               */



/*
  Initialization
*/
c_inlas(&vblock,&std_out);


/*
  Get parameters and globals
*/
getpar(&vblock, hosin, hosout, minmax, taein, taeout, bands, &idtype, imask,
    &msgtim, nbands, &nmasks, &nmrimg, &odtype, omask, &rndopt, &totbnd,
    window, iscale, ioffset, oscale, ooffset, omax, omin);


/*
  Process image
*/
process(&vblock, hosin, hosout, minmax, bands, idtype, imask, msgtim,
    nbands, nmasks, nmrimg, odtype, omask, rndopt, totbnd, window, iscale,
    ioffset, oscale, ooffset, omax, omin);


/*
  Update history file 
*/
if (c_hstout(&vblock,hosout,hosin[0],&nmrimg," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","rescale-warn",&one);


/*
  Output completion message 
*/
c_complt("RESCALE",taeout,&one);
}
