/******************************************************************************
NAME:		GET_NRD_PAR

PURPOSE:	Retrieves NORAD parameter information

ALGORITHM
	retrieve required parameters from the variable block

ALGORITHM REFERENCES	none
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "util.h"
#include "norad.h"

#define TERM   0
#define LP     1
#define FIL    2


void get_nrd_par
(
struct PARBLK	*vblock,	/* TAE parameter block			*/
char		*satid,		/* Satellite id number			*/
long		*epyr,		/* Epoch Year				*/
long            *epdy,		/* Epoch Day				*/
double		*epfd,		/* Epoch Fractional Day			*/
double          *inclin,	/* Inclination				*/
double          *rtasc,		/* Right Ascension of the Ascending Node*/
double          *ecc,		/* Eccentricity				*/
double          *argper,	/* Argument of Perigee			*/
double          *meanan,	/* Mean Anomaly				*/
double          *meanm,		/* Mean Motion				*/
FILE            **print_file,   /* File descriptor of output file       */
char            out[1][CMLEN],  /* List of output files                 */
long            *out_cnt        /* Number of output files               */
)
 
{ 
struct VARIABLE *var;		/* Pointer to a TAE variable in vblock	*/

char	temp[21],		/* Temporary string			*/
        print_filename[CMLEN],  /* output file name              */
	fileutil_file[CMLEN];	/* Temporary file for fileutil         */ 

long	nonfatal = 1,           /* Nonfatal error code                  */
	status,                 /* Return value                         */
	print[3],    	        /* Print options 	 		*/
	zero = 0;		/* Constant value			*/


/*
  Get the satellite name (N09, N10, N11, N12, N14, N15, N16, etc)
*/

var = p_find(vblock, "SATID");
strcpy(satid, SVAL(*var, 0));

/*
  Get the epoch year
*/
var = p_find(vblock, "EPOCHYR");
*epyr = IVAL(*var,0);

/*
  Get the epoch day
*/
var = p_find(vblock, "EPOCHDY");
*epdy = IVAL(*var,0);

/*
  Get the epoch fractional day
*/
var = p_find(vblock, "EPOCHFD");
strcpy(temp,"0.");
strcat(temp, SVAL(*var,0));
sscanf(temp,"%lf",epfd);

/*
  Get the Inclination
*/
var = p_find(vblock, "INCLINE");
sscanf(SVAL(*var,0),"%lf",inclin);

/*
  Get the Right Ascension of the Ascending Node
*/
var = p_find(vblock, "RTASCEN");
sscanf(SVAL(*var,0),"%lf",rtasc);

/*
  Get the Eccentricity
*/
var = p_find(vblock, "ECCENT");
strcpy(temp,"0.");
strcat(temp, SVAL(*var, 0));
sscanf(temp,"%lf",ecc);

/*
  Get the Argument of Perigee
*/
var = p_find(vblock, "ARGPER");
sscanf(SVAL(*var,0),"%lf",argper);

/*
  Get the Mean Anomaly
*/
var = p_find(vblock, "MEANAN");
sscanf(SVAL(*var,0),"%lf",meanan);

/*
  Get the mean motion
*/
var = p_find(vblock, "MEANMO");
sscanf(SVAL(*var,0),"%lf",meanm);

/*
  Get print option and open output file, if necessary 
--------------------------------------------------------
*/
*out_cnt = 0;
strcpy (fileutil_file, "NORAD");
status = c_setprt (vblock, (int *)&print[TERM], (int *)&print[FIL],
         (int *)&print[LP], fileutil_file, print_filename);
if (status != E_SUCC)
	c_clean(vblock,"NORAD",(const char (*)[CMLEN])fileutil_file,&zero);
if (print[FIL] == 1)
	{
        strcpy (out[*out_cnt], print_filename);
        *out_cnt += 1;
	}
*print_file = fopen (fileutil_file, "w");
if (*print_file == NULL)
	{
	c_errmsg("Error opening output file","norad-outopn",&nonfatal);
	c_clean(vblock,"NORAD",(const char (*)[CMLEN])fileutil_file,&zero);
	}
}
