/******************************************************************************
NAME:		GET_NSP_PAR

PURPOSE:	Retrieves NAVSPASUR parameter information

PROGRAM HISTORY:
Version     Date     Author             Description
-------     ----     ------             --------------
 1.0        02/92    T. Baltzer         Original Development

ALGORITHM
	Retrieve required parameters from the variable block
	and load them into a character string.

ALGORITHM REFERENCES	none
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "util.h"
#include "navspasur.h"

#define TERM   0
#define LP     1
#define FIL    2

void get_nsp_par
(
struct PARBLK	*vblock,	/* TAE parameter block			*/
char		line1[],	/* navspasur line of information	*/
FILE            **print_file,   /* File descriptor of output file       */
char		out[1][CMLEN],  /* List of output files                 */
long		*out_cnt,	/* Number of output files		*/
char		fileutil_file[] /* Temporary file for fileutil */
)

{ 
char	argper[9],	/* argument of perigee (revs)		*/
	eccent[9],	/* eccentricity				*/
	epdate[7],	/* date of epoch YYMMDD			*/
	incline[9],	/* inclination (revs)			*/
	longasc[9],	/* longitud of ascending node (revs)	*/
	meanan[9],	/* mean anomaly (revs)			*/
	meanmo[9],	/* mean motion (rads/herg)		*/
	odecay[6],	/* orbital decay (1.E-5 rads/herg**2)	*/
	orbit[6],	/* orbit number				*/
	print_filename[CMLEN], /* output file name              */
	satid[6];	/* Satellite id number			*/

long	nonfatal = 1,		/* Nonfatal error code                  */
	print[3],		/* print options			*/
	status,                 /* Return value                         */
	decay,			/* Orbital decay value			*/
	zero = 0;		/* Constant value			*/

struct VARIABLE *var;		/* Pointer to a TAE variable in vblock	*/

/*
  Get the input parameters
*/

var = p_find(vblock, "SATID");
strcpy(satid, SVAL(*var, 0));

var = p_find(vblock, "ORBIT");
strcpy(orbit, SVAL(*var, 0));

var = p_find(vblock, "MEANAN");
strcpy(meanan, SVAL(*var, 0));

var = p_find(vblock, "MEANMO");
strcpy(meanmo, SVAL(*var, 0));

var = p_find(vblock, "ODECAY");
strcpy(odecay, SVAL(*var, 0));
/*
  Some times the orbital decay is way off.  It needs to be reset
  to zero when it is.
*/
sscanf(odecay,"%5d",&decay);
if (decay > 90000)
    strcpy(odecay,"00000");

var = p_find(vblock, "ECCENT");
strcpy(eccent, SVAL(*var, 0));

var = p_find(vblock, "ARGPER");
strcpy(argper, SVAL(*var, 0));

var = p_find(vblock, "LONGASC");
strcpy(longasc, SVAL(*var, 0));

var = p_find(vblock, "INCLINE");
strcpy(incline, SVAL(*var, 0));

var = p_find(vblock, "EPDATE");
strcpy(epdate, SVAL(*var, 0));


/*
  Load the parameters into a string
*/

strncpy(&line1[0],satid,5);
strncpy(&line1[5],orbit,5);
strncpy(&line1[10],meanan,8);
strncpy(&line1[18],meanmo,8);
strncpy(&line1[26],odecay,5);
strncpy(&line1[31],eccent,8);
strncpy(&line1[39],argper,8);
strncpy(&line1[47],longasc,8);
strncpy(&line1[55],incline,8);
strncpy(&line1[63],epdate,6);


/*
  Get print option and open output file, if necessary 
--------------------------------------------------------
*/
*out_cnt = 0;
strcpy (fileutil_file, "NAVSPASUR");
status = c_setprt (vblock, (int *)&print[TERM], (int *)&print[FIL],
         (int *)&print[LP], fileutil_file, print_filename);
if (status != E_SUCC)
	c_clean(vblock,"NAVSPASUR",(const char (*)[CMLEN])fileutil_file,&zero);

if (print[FIL] == 1)
	{
        strcpy (out[*out_cnt], print_filename);
        *out_cnt += 1;
	}

if (print[TERM] || print[FIL] || print[LP])
    {
    *print_file = fopen (fileutil_file, "w");
    if (*print_file == NULL)
	{
	c_errmsg("Error opening output file","navspasur-outopn",&nonfatal);
	c_clean(vblock,"NAVSPASUR",(const char (*)[CMLEN])fileutil_file,&zero);
	}
    }
else
    *print_file = NULL;

}
