/*****************************************************************************
NAME: 			GET_TRK_PAR

PURPOSE:		Retrieve SATRACK parameter information

ALGORITHM: 		Retrieve necessary parameters from variable block

ALGORITHM REFERENCES:	none
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "util.h"
#include "satrack.h"

#define   TERM    0
#define   FIL     2
#define   LP      1


void  get_trk_par
(
struct PARBLK   *vblock,         /* TAE parameter block */
char   *filnam,                  /* Name of input file */
FILE   **print_file,             /* File descriptor to output file */
char   out[][CMLEN],             /* List of output files */
long   *n_out,                   /* Number of output files */
long   numsat[],		 /* Satellite numbers to print out */
char   fileutil_file[]           /* Temporary file for fileutil */ 
)

{
struct VARIABLE *nam;            /* Pointer to a TAE variable in vblock */

long   access = IREAD,		 /* Type of access for getnam call */
       element = 0,		 /* Element number for getnam call */
       i,			 /* counter */
       status,                   /* Function return value */
       print[3],                 /* Print options */
       nonfatal = 1,             /* Nonfatal error code */
       zero = 0;		 /* Constant value */

char   print_filename[CMLEN];    /* Output filename specified by user */ 


/* Get the name of the file to read
-----------------------------------*/
     status = c_getnam(vblock,"INFILE",&element,filnam,"",&access);
     if (status != E_SUCC)
	c_clean(vblock,"SATRACK",(const char (*)[CMLEN])fileutil_file,&zero);

/* Set print options and open output file, if necessary 
-------------------------------------------------------*/
     *n_out = 0;
     strcpy (fileutil_file, "SATRACK");
     status = c_setprt (vblock, (int *)&print[TERM], (int *)&print[FIL],
                 (int *)&print[LP], fileutil_file, print_filename);
     if (status != E_SUCC)
	c_clean(vblock,"SATRACK",(const char (*)[CMLEN])fileutil_file,&zero);

     if (print[FIL] == 1)
        {
        strcpy (out[*n_out], print_filename);
        *n_out += 1;
        }

     *print_file = fopen (fileutil_file, "a");
     if (*print_file == NULL)
        {
        c_errmsg ("Error opening output file", "satrack-outopn", &nonfatal);
	c_clean(vblock,"SATRACK",(const char (*)[CMLEN])fileutil_file,&zero);
        }

/* Get the satellite number(s)
------------------------------*/
     nam = p_find(vblock,"SATNUM");
     for (i = 0; i < nam->v_count; i++)
        numsat[i] = IVAL(*nam,i);
}
