/*******************************************************************************
FUNCTION:	NAVSPASUR

PURPOSE:	Tutor the user for input navspasur information and
		write it to the appropriate update file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0	   02/92    T. Baltzer		Original development
  1.1      01/95    J. Fenno            Added code for NOAA-14.
  1.2      06/98    D. Lloyd		Added code for NOAA-15
  1.3      09/00    D. Lloyd		Added code for NOAA-16
  1.4      04/02    D. Lloyd		Added code for NOAA-17
  1.5      11/04    D. Lloyd		Added code for NOAA-18

PROJECT		ADAPS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "tae.h"
#include "util.h"
#include "util2.h"
#include "model.h"
#include "navspasur.h"

int main( void )
{
/* Satellite numbers to print out*/
static long numsat[TOTSAT] = {12,14,15,16,17,18,19,2};

long	out_cnt = 0,    /* Number of output files created (update only)     */
	one = 1,	/* Constant value				    */
	stdlun;		/* Logical Unit number				    */

char	out[1][CMLEN],  /* List of output files created                     */
	fileutil_file[CMLEN], /* Temporary file for fileutil	  	    */
	line1[EPHEM_SIZ+1];/* Navspasur line of data to process		    */

struct PARBLK	vblock;	/* TAE parameter block				    */

FILE    *print_file;    /* File descriptor to the output print file         */

/*
  Initialize LAS
*/
c_inlas(&vblock,&stdlun);

/*
  Get the parameters from the variable block
*/
get_nsp_par(&vblock,line1,&print_file,out,&out_cnt,fileutil_file);

/*
  Process the data just acquired
*/
if (put_navsp(line1,&print_file,numsat) != E_SUCC)
   {
   fclose(print_file);
   c_clean(&vblock,"NAVSPASUR",(const char (*)[CMLEN])fileutil_file,&one);
   }

/*
  Close the print file
*/
fclose(print_file);

/*
  Write a completion message
*/
c_complt("NAVSPASUR",out,&out_cnt);
return E_SUCC;
}
