/*******************************************************************************
FUNCTION:	NORAD

PURPOSE:	Tutor the user for input norad information, calculate
		information to be entered into the Antenna Control
		Unit (ACU), print the information to a file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0	   04/87    D. Hollaren		Original Development
  2.0      05/91    D. Hollaren		Conversion to LAS-5.0
  3.0	   02/92    T. Baltzer		Rewrite to reuse code

PROJECT		ADAPS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "tae.h"
#include "util2.h"
#include "constants.h"
#include "norad.h"


int main( void )
{
double	epfd,		/* Epoch Fractional Day				    */
	inclin,		/* Inclination					    */
	rtasc,		/* Right Ascension of the Ascending Node	    */
	ecc,		/* Eccentricity					    */
	argper,		/* Argument of perigee				    */
	meanan,		/* Mean anomaly					    */
	meanm;		/* Mean Motion					    */

long	epyr,		/* Epoch Year					    */
	epdy,		/* Epoch Day					    */
	stdlun,		/* Logical Unit number				    */
	out_cnt;        /* Number of output files                           */

FILE	*print_file;	/* File descriptor to the output print file	    */

char	satid[6],	/* Satellite ID					    */
	out[1][CMLEN];  /* List of output files created                     */

struct PARBLK	vblock;	/* TAE parameter block				    */


/*
  Initialize LAS
*/
c_inlas(&vblock,&stdlun);

/*
  Get the parameters from the variable block
*/
get_nrd_par(&vblock,satid,&epyr,&epdy,&epfd,&inclin,&rtasc,&ecc,&argper,
   	    &meanan,&meanm,&print_file,out,&out_cnt); 

/*
  Process the data just acquired
*/
put_norad(satid,epyr,epdy,epfd,inclin,rtasc,ecc,argper,meanan,
          meanm,&print_file);

/*
  Close the print file
*/
fclose(print_file);

/*
  Write a completion message
*/
c_complt("NORAD",out,&out_cnt);
return E_SUCC;
}
