/*****************************************************************************
NAME: 			PUT_NAVSP

PURPOSE:  Open appropriate update file for the navspasur data and
          write data to the update file in chronological order

ALGORITHM: 
 See in-line documentation

ALGORITHM REFERENCES:  none
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <sys/file.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "util2.h"
#include "model.h"
#include "navspasur.h"

int put_navsp
(
char   *line1, /* Line read from input file */ 
FILE   **print_file,       /* File descriptor of the output print file */
long   numsat[]            /* Satellite numbers to print out */
)
{
long   access = 2,	    /* File access(0=exist 1=execute 2=write 4=read)*/
       byear,		    /* Year found in the buffer	*/
       iinline,             /* Lines remaining to read in ephemeris file */
       i,                   /* Loop counter */
       lyear,		    /* Year found in the line of data */
       nonfatal = 1,        /* Nonfatal error code */
       nrec,                /* Number of initial records in ephem file */
       nhead,               /* Number of header records in updated ephem file */
       record,              /* Number of records in updated ephem file */
       offset,              /* Number of bytes moved in fseek */
       move,                /* Flag showing when records need to be moved in
                                    order to insert line */
       head,                /* Flag showing when header needs to be updated */
       n_seek,              /* Current file pointer value */
       n_write,             /* Number of bytes written in fwrite */
       navid;               /* NAVSPASUR Satellite id */

char   bdate[9],	    /* Date as found in the buffer */
       buf[EPHEM_SIZ+1],    /* Line read from ephem file */
       ldate[9],	    /* Date as found in the line of data */
       mesg[ERRLEN],        /* Informational processing message */ 
       *snow, 		    /* return status from a file read	*/
       setfile[CMLEN],      /* Path of update file */
       sat[10],             /* Satellite type in ephem file header */
       header[EPHEM_SIZ+1], /* Updated header for ephem file */
       disk [CMLEN];        /* Pointer to path returned from getenv */ 	

FILE   *desc;               /* File descriptor for ephem file */ 

/* Set path and name of update file
-----------------------------------*/
if (getenv("ADAPSTABLES") == NULL)
    return(E_FAIL);
strcpy(disk, getenv("ADAPSTABLES"));
sscanf(line1, "%5d", &navid);

sprintf(setfile, "%s/%d.dat", disk, navid);

/* Make sure the file exists with write permission,
   open the update file
-------------- -------------------------------------*/
     if (c_lsstat(setfile,&access) != E_SUCC)
	{
	c_errmsg ("Error locating file","put_navsp-locf",&nonfatal);
	return(E_FAIL);
        }

     desc = fopen (setfile, "r+");  
     if (desc == NULL)
        {
        c_errmsg ("Error opening output file", "put_navsp-outopn", &nonfatal);
	return(E_FAIL);
        }

/* Read the first line of the ephemeris fileto get the number of ephem
   and header records and the satellite
-------------------------------------------------------------------------*/
     snow = fgets (buf, EPHEM_SIZ, desc);
     if (snow == NULL)
        {
        c_errmsg ("Error reading input file", "put_navsp-inread",&nonfatal);
	return(E_FAIL);
	}
     sscanf (buf, "%d%d%s", &nrec, &nhead, sat);

/* Read the ephemeris file line by line from the end and compare each 
   line with the line from the input file to determine its position
---------------------------------------------------------------------*/
     move = FALSE;
     head = FALSE;
     record = nrec;
     iinline = nrec;

     while (iinline > 0)
        {
        offset = ((iinline + nhead) - 1) * EPHEM_SIZ;
        n_seek = fseek (desc, offset, 0);
	if (n_seek != 0)
           {
           c_errmsg ("Error seeking input file", "put_navsp-inseek",&nonfatal);
           return(E_FAIL);
	   }

        snow = fgets (buf, EPHEM_SIZ, desc); 
        if (snow == NULL)
           {
           c_errmsg ("Error reading input file", "put_navsp-inread",&nonfatal);
	   return(E_FAIL);
           }

	/* Create date strings of the form YYYYMMDD */

	sscanf(&line1[63],"%2d",&lyear);
	lyear = c_4digityr(lyear);
	sprintf(ldate,"%d",lyear);
	strncat(ldate,&line1[65],4);
	ldate[8] = '\0';
	sscanf(&buf[63],"%2d",&byear);
	byear = c_4digityr(byear);
	sprintf(bdate,"%d",byear);
	strncat(bdate,&buf[65],4);
	bdate[8] = '\0';

	if (strcmp (ldate, bdate) == 0)
           {    
           sprintf(mesg,"Duplicate record found on date %s in file %s", 
                   &line1[63],setfile);
           c_errmsg(mesg,"put_navsp-dup",&nonfatal);
           break;
	   }
	else if (strcmp (ldate, bdate) > 0)
           {

	   /* If record date is less than line in ephem file move all
	      records after it down one and insert the new record if
	      move = TRUE 
	      --------------------------------------------------------*/
           if (move)
	      {
              for (i = nrec; i > iinline; i--)
                 {
            
	         offset = ((i + nhead) - 1) * EPHEM_SIZ;
                 n_seek = fseek (desc, offset, 0);
		 if (n_seek != E_SUCC)
		    {
		    c_errmsg ("Error seeking input file", "put_navsp-inseek",
		    	      &nonfatal);
		    return(E_FAIL);
		    }

                 snow = fgets (buf, EPHEM_SIZ, desc);
		 if (snow == NULL)
		    {
		    c_errmsg ("Error reading input file", "put_navsp-inread",
			 &nonfatal);
		    return(E_FAIL);
	 	    }

                 offset = (i + nhead) * EPHEM_SIZ;
                 n_seek = fseek (desc, offset, 0);
		 if (n_seek != E_SUCC)
		    {
		    c_errmsg ("Error seeking input file", "put_navsp-inseek",
		    	      &nonfatal);
		    return(E_FAIL);
		    }

                 strcpy (&buf[69], "\n");
                 n_write = fputs (buf, desc);
	         if (n_write == EOF)
	            {
                    c_errmsg ("Error writing data to file", "put_navsp-write",
	   	              &nonfatal);
	            return(E_FAIL);
	            }
                 }

              offset = (i + nhead) * EPHEM_SIZ;
              n_seek = fseek (desc, offset, 0);
	      if (n_seek != E_SUCC)
		 {
		 c_errmsg ("Error seeking input file", "put_navsp-inseek",
		 	   &nonfatal);
		 return(E_FAIL);
		 }

              strcpy (&line1[69],"\n");
              n_write = fputs (line1, desc);
	      if (n_write == EOF)
		 {
                 c_errmsg ("Error writing data to file", "put_navsp-write",
	                   &nonfatal);
	         return(E_FAIL);
	         }

	      record += 1;
              head = TRUE;
              break;
	      }
/* Otherwise, just append record to the end of the ephem file
-------------------------------------------------------------*/ 
           else
	      {
              offset = (nrec + nhead) * EPHEM_SIZ;
              n_seek = fseek (desc, offset, 0);
	      if (n_seek != E_SUCC)
		 {
		 c_errmsg ("Error seeking input file", "put_navsp-inseek",
		 	   &nonfatal);
		 return(E_FAIL);
		 }

              strcpy (&line1[69], "\n");
              n_write = fputs (line1, desc);
	      if (n_write == EOF)
	         {
                 c_errmsg ("Error writing data to file", "put_navsp-write",
	                   &nonfatal);
		 return(E_FAIL);
	         }

              record += 1;
              head = TRUE;
              break;
              }
           }

/* If line is neither equal nor greater than the current line in
   the ephem file, iinline is decremented to move pointer back a line 
   and move is set to TRUE
-------------------------------------------------------------------*/
        else
           {
           iinline--;
           move = TRUE; 
           }
        }

/* If record was added, write new header to ephem file
------------------------------------------------------*/
        if (head)
           {
           sprintf (header, "%6d%4d %-58s\n", record, nhead, sat);
           offset = 0;
           n_seek = fseek (desc, offset, 0);
	   if (n_seek != E_SUCC)
	      {
	      c_errmsg ("Error seeking input file", "put_navsp-inseek",
	                &nonfatal);
	      return(E_FAIL);
	      }

           n_write = fputs (header, desc);
	   if (n_write == EOF)
	      {
              c_errmsg ("Error writing data to file","put_navsp-write",
	                &nonfatal);
	      return(E_FAIL);
	      }
           }

/* Close the input file
-----------------------*/
fclose(desc);


/* Write the information to a file for output to the device specified
---------------------------------------------------------------------*/
     if (*print_file != NULL)
	{
	if ( gen_rpt(navid,numsat) == E_SUCC)
	    {
	    if (calc_navsp(line1,print_file) != E_SUCC)
		return(E_FAIL);
	    }
	}

return(E_SUCC);
}

