/****************************************************************************
NAME:	SEP_NORAD

PURPOSE:  Seperate the meaningful norad information from the two input
	  lines of norad information and then call put_norad to process
	  it.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      02/92    T. Baltzer	        Original Development

ALGORITHM REFERENCES:   none
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "worgen.h"
#include "norad.h"
#include "satrack.h"

void sep_norad
(
char   *line1,              /* First line of norad information */
char   *line2,              /* Second line of norad information */
FILE   **print_file         /* File descriptor of the output print file */
)

{

char   satid[6],           /* Satellite id number */
       junk2[7],	   /* Unused portion of data */
       junk4[3],	   /* Unused portion of data */
       sepfd[12],	   /* string rep. of epoch fractional day */
       smeanm[13],	   /* string rep. of mean motion */
       temp2[15],          /* Temporary string */
       temp1[15];          /* Temporary string */

long   epyr,               /* Epoch year */
       epdy,               /* Epoch day */
       junk1,	   	   /* Unused portion of data */
       junk3,	   	   /* Unused portion of data */
       junk5;	   	   /* Unused portion of data */

double epfd,               /* Epoch fractional day */
       inclin,             /* Inclination angle */
       rtasc,              /* Right ascension of the ascending node */
       ecc,                /* Eccentricity */
       argper,             /* Argument of perigee */
       meanan,             /* Mean anomaly */ 
       meanm;              /* Mean motion */

/* Get the epoch year, epoch day, and epoch fractional day 
----------------------------------------------------------*/
sscanf (line1, "%d%s%d%s%2d%3d%9s", &junk1, junk2,
        &junk3, junk4, &epyr, &epdy, sepfd);
epfd = atof (sepfd);

/* Get the satellite id, inclination, right ascension of the 
   ascending node, eccentricity, argument of perigee, mean 
   anomaly, and mean motion
------------------------------------------------------------*/
strcpy (temp1, "0.");
sscanf (line2, "%d%5s%8lf%9lf%7s%8lf%8lf%11s", &junk5, satid,
        &inclin, &rtasc, temp2, &argper, &meanan, smeanm);
meanm = atof (smeanm);
strcat (temp1, temp2);
sscanf (temp1, "%lf", &ecc);
 
/* Take the norad data and print it to the specified file, also
   calculate information to be entered into the Antenna Control
   Unit(ACU).
*/
put_norad(satid,epyr,epdy,epfd,inclin,rtasc,ecc,argper,meanan,
	  meanm,print_file);
}
