/******************************************************************************
NAME:		TIMEOFLIGHT

PURPOSE:	Calculate the time of flight

ALGORITHM

ALGORITHM REFERENCES	none
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "adaps.h"
#include "navspasur.h"

void timeoflight
(
double	*truea1,	/* true anomally 				*/
double	*truea2,	/* true anomally				*/
struct MODEL    *model, /* Model and orbital information		*/
double 	*tof 		/* time of flight				*/
)

{ 
double cose1;
double cose2;
double e1;
double e2;
double tof1;
double tof2;

if (*truea1 < 0.0)
    *truea1 = PIE2 + *truea1;

if (*truea1 > PIE2)
    *truea1 -= PIE2;

if (*truea2 < 0.0)
    *truea2 = PIE2 + *truea2;

if (*truea2 > PIE2)
    *truea2 -= PIE2;

cose1 = (model->ecc + cos(*truea1)) / (1.0 + (model->ecc * cos(*truea1)));

if (fabs(cose1) >= 1.0)
    cose1 = fabs(cose1) / cose1;

e1 = acos(cose1);
if (*truea1 > PIE)
    e1 = PIE2 - e1;

cose2 = (model->ecc + cos(*truea2)) / (1.0 + (model->ecc * cos(*truea2)));
if (fabs(cose2) >= 1.0)
    cose2 = fabs(cose2) / cose2;

e2 = acos(cose2);
if (*truea2 > PIE)
    e2 = PIE2 - e2;

tof1 = sqrt(model->smajor_axis * model->smajor_axis * model->smajor_axis) *
       (e1 - (model->ecc * sin(e1))) * HERG;

tof2 = sqrt(model->smajor_axis * model->smajor_axis * model->smajor_axis) *
       (e2 - (model->ecc * sin(e2))) * HERG;

if ((*truea2 < 0.0) && (tof2 > 0.0))
   tof2 = -tof2;

*tof = tof2 - tof1;
}
