/************************************************************************
FUNCTION:	FIND_CONFLICTS

PURPOSE:	Search the linked list of passes for conflicts and tag
		them.


PROGRAM HISTORY:
Version Date    Author          Modification
_______ _____   _____________   ___________________
  1.0   10/98   D.Lloyd         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under Unix

PROJECT         ADAPS

$Log: find_conflicts.c,v $
Revision 1.3  2002/08/02 14:17:41  dlloyd
Modified the header file to only include headers for its prototypes.
Added necessary include in each .c file.
Moved external declaration from .h to .c file.
Added the Log directive.

*************************************************************************/
#include "worgen.h"
#include "schedcnflt.h"

int find_conflicts(

    struct PASS **passlist	/* IN: pointer to the pass list		*/
)
{
struct	PASS	*curr;		/* pointer to current pass		*/
struct	PASS	*prev;		/* pointer to previous pass		*/


if ( (*passlist)->next != NULL )	/* no conflict only one in list	*/
    {
    curr = *passlist;
    do
	{
	prev = curr;
	curr = prev->next;

	/* The current and previous passes are on the same day
	------------------------------------------------------*/
	if ( prev->sdate == curr->sdate )
	    {
	    if (( prev->etime > curr->stime ) && curr->sched )
	        prev->conflict = TRUE;
	    }
	
	/* The previous pass starts on the previous day
	-----------------------------------------------*/
	else
	    {
	    if (( prev->etime > (curr->stime+SEC_PER_DAY)) && curr->sched )
		prev->conflict = TRUE;
	    }
	}
    while ( curr->next != NULL );
    }

return E_SUCC;
}
