/************************************************************************
FUNCTION:	PRINT_CONFLICTS

PURPOSE:	Search the linked list of passes for conflicts and
		print the line to the output file along with the entry
		that is scheduled.


PROGRAM HISTORY:
Version Date    Author          Modification
_______ _____   _____________   ___________________
  1.0   10/98   D.Lloyd         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under Unix

PROJECT         ADAPS

$Log: print_conflicts.c,v $
Revision 1.7  2009/07/14 15:26:57  adaps
*** empty log message ***

Revision 1.6  2002/08/08 18:23:53  dlloyd
Changed confusing message when two scheduled scenes conflict.  If the first
scene is the primary satellite the second scene is listed as 'No conflict'.

Revision 1.5  2002/08/08 15:59:36  dlloyd
Modified the report so that the 16-digit sceneid is used in place of the
satellite and orbit number and shortened the line width by 4.

Revision 1.4  2002/08/02 14:17:41  dlloyd
Modified the header file to only include headers for its prototypes.
Added necessary include in each .c file.
Moved external declaration from .h to .c file.
Added the Log directive.

*************************************************************************/
#include <stdlib.h>
#include "worgen.h"
#include "schedcnflt.h"


int print_conflicts(

    FILE	*ouf,		/* IN: output file descriptor		*/
    struct PASS **passlist	/* IN: pointer to pass list		*/
)
{
long	conflict = FALSE;	/* flag: conflict in schedule		*/

struct	PASS	*curr;		/* pointer to current pass		*/
struct	PASS	*prev;		/* pointer to previous pass		*/

prev = *passlist;
curr = prev->next;
do
    {
    if ( curr->sched && prev->sched && prev->conflict )
        {
	conflict = TRUE;
	if ( atoi(prev->satnum) == 19 )
	    {
	    fprintf( ouf, "%s Scheduled\n", prev->line );
	    fprintf( ouf, "%s No conflict\n\n", curr->line );
	    }
	else
	    {
	    fprintf( ouf, "%s Conflict\n", prev->line );
	    fprintf( ouf, "%s Scheduled\n\n", curr->line );
	    }
	}
    else if ( curr->sched && prev->conflict )
        {
	conflict = TRUE;
	fprintf( ouf, "%s Conflict\n", prev->line );
        fprintf( ouf, "%s Scheduled\n\n", curr->line );
	}
    prev = curr;
    curr = curr->next;
    }
while( curr != NULL );

if ( !conflict )
    fprintf( ouf, "\nNo conflicts detected during this time period\n" );


return E_SUCC;
}
