/************************************************************************
FUNCTION:       PROCESS

PURPOSE:	Process each line of the input file.  Building a sorted
		linked list of pass information.

PROGRAM HISTORY:
Version Date    Author          Modification
_______ _____   _____________   ___________________
  1.0   10/98   D.Lloyd         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under Unix

PROJECT         ADAPS

$Log: process.c,v $
Revision 1.6  2002/08/08 15:59:37  dlloyd
Modified the report so that the 16-digit sceneid is used in place of the
satellite and orbit number and shortened the line width by 4.

Revision 1.5  2002/08/02 18:13:59  dlloyd
Corrected an error in reading the header lines from the autogen report, it
was reading one less line in one place and one too many in another.  CR 6560

Revision 1.4  2002/08/02 14:17:41  dlloyd
Modified the header file to only include headers for its prototypes.
Added necessary include in each .c file.
Moved external declaration from .h to .c file.
Added the Log directive.

*************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "worgen.h"
#include "schedcnflt.h"


int process(

    FILE	*inf,		/* IN: input file descriptor		*/
    struct PASS	**passlist	/* OUT: pointer to pass list		*/
)
{
char	buffer[132];		/* input file line buffer		*/
char	satnum[3];		/* current satellite number		*/
char	temp[9];		/* date time string			*/
char    sceneid[18];

long	i;			/* loop control variable		*/

struct	PASS	*ptr;		/* pointer to a new pass structure	*/


/* Process the entire file
--------------------------*/
while ( TRUE )
    {
    /* get a new line of data */
    /* break out when end of file reached */
    if ( fgets( buffer, 132, inf ) == NULL )
        break;

    /* Check for beginning of new satellite.  This is indicated
    by a ^L (decimal 12) in buffer[0].
    -----------------------------------------------------------*/
    if ( buffer[0] == 12 )
        {
	/* skip to the line containing the satellite name */
	for ( i = 0; i < PREHEADERLINES; i++ )
	    fgets( buffer, 132, inf );
	
	/* get the satellite number */
	strncpy( satnum, &buffer[44], 2 );
	satnum[2] = 0;

	/* skip to beginning of pass information */
	for ( i = 0; i < POSTHEADERLINES+1; i++ )
	    fgets( buffer, 132, inf );
	}
    
    /* Allocate and fill a new pass structure with the pass information
    -------------------------------------------------------------------*/
    ptr = (pass *)malloc( sizeof( pass ));
    if ( ptr == NULL )
        {
	printf( "Unable to allocate memory\n" );
	return E_FAIL;
	}
    strcpy( ptr->satnum, satnum );

    /* Get start time/date and change from UTC to local
    ---------------------------------------------------*/
    strncpy( temp, &buffer[14], 8 );		/* get start date	*/
    temp[8] = 0;

    /* Build the fisrt part of the sceneid */
    sprintf( sceneid, "ah%s", ptr->satnum );
    sceneid[4] = temp[0];
    sceneid[5] = temp[1];
    sceneid[6] = temp[3];
    sceneid[7] = temp[4];
    sceneid[8] = temp[6];
    sceneid[9] = temp[7];

    ptr->sdate = get_date( temp );
    strncpy( temp, &buffer[26], 8 );		/* get start time	*/
    temp[8] = 0;

    /* Build the rest of the sceneid */
    sceneid[10] = temp[0];
    sceneid[11] = temp[1];
    sceneid[12] = temp[3];
    sceneid[13] = temp[4];
    sceneid[14] = temp[6];
    sceneid[15] = temp[7];
    sceneid[16] = ' ';
    sceneid[17] = 0;

    ptr->stime = get_time( temp );
    set_local( &ptr->sdate, &ptr->stime );	/* convert to local	*/
    datestring( ptr->sdate, temp );
    strncpy( &buffer[14], temp, 8 );		/* store local in buffer*/
    timestring( ptr->stime, temp );
    strncpy( &buffer[26], temp, 8 );

    
    /* Get end date/time and change from UTC to local
    -------------------------------------------------*/
    strncpy( temp, &buffer[38], 8 );		/* get end date		*/
    temp[8] = 0;
    ptr->edate = get_date( temp );
    strncpy( temp, &buffer[50], 8 );		/* get end time		*/
    temp[8] = 0;
    ptr->etime = get_time( temp );
    set_local( &ptr->edate, &ptr->etime );	/* convert to local	*/
    datestring( ptr->edate, temp );
    strncpy( &buffer[38], temp, 8 );		/* store local in buffer*/
    timestring( ptr->etime, temp );
    strncpy( &buffer[50], temp, 8 );

    ptr->conflict = FALSE;
    /* set the scheduled flag */
    if (( buffer[109] ) == '*' && ( buffer[110] == '*' ))
        ptr->sched = TRUE;
    else
        ptr->sched = FALSE;
    strcpy( ptr->line, sceneid );	/* copy the sceneid to the line	*/
    buffer[69] = 0;			/* shorten the buffer length	*/
    strcat( ptr->line, &buffer[14] );	/* copy modified buffer to line	*/

    /* Move the length of pass up 4
    places again to shorten the line
    --------------------------------*/
    for ( i = 63; i < 70; i++ )
        ptr->line[i] = ptr->line[i+4];
    ptr->line[69] = ' ';
    ptr->line[70] = 0;

    ptr->next = NULL;


    /* Insert the pass into the linked list in its proper
    location.  The list is ordered by date/time of start.
    -----------------------------------------------------*/
    insert( &ptr, passlist );

    }

return E_SUCC;
}
