/************************************************************************
FUNCTION:	SCHEDCNFLT

PURPOSE:	Read the report generated by xplot-autogen and look for
		conflicts with scheduled passes and non-scheduled passes.
		A conflict, in this context, is defined as: the start time
		of the scheduled pass lies between the start and end time
		of another pass.

PROGRAM HISTORY:
Version	Date	Author		Modification
_______	_____	_____________	___________________
  1.0	10/98	D.Lloyd		Initial Development
  1.1   02/00   D.Lloyd		Minor Y2K printf format change

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under Unix

PROJECT		ADAPS

$Log: schedcnflt.c,v $
Revision 1.6  2002/08/02 18:25:06  dlloyd
Made main an int function

Revision 1.5  2002/08/02 14:17:42  dlloyd
Modified the header file to only include headers for its prototypes.
Added necessary include in each .c file.
Moved external declaration from .h to .c file.
Added the Log directive.

*************************************************************************/
#include <stdlib.h>
#include "worgen.h"
#include "schedcnflt.h"


int main( argc, argv )

    int		argc;		/* IN: argument count			*/
    char	*argv[];	/* IN: pointer to argument list		*/

{
FILE	*inf;			/* Input file descriptor		*/
FILE	*ouf;			/* output file descriptor		*/

long	status;			/* function return status		*/

struct	PASS	*curr;		/* pointer to current pass		*/
struct	PASS	*passlist;	/* list of pass structures		*/
struct	PASS	*prev;		/* pointer to previous pass		*/

struct	tm	*lctm;		/* local time structure			*/

time_t	t;			/* seconds since Jan 1, 1970		*/


/* Get the local time
---------------------*/
t = time( NULL );
lctm = localtime( &t );


/* Initialize the pass structures
---------------------------------*/
passlist = curr = NULL;


/* Check for input parameter and print usage if none found
----------------------------------------------------------*/
if ( argc < 3 )
    {
    printf( "\nUsage: %s <autogen_listing_file_name> ", argv[0] );
    printf( "<output_file_name> [master_schedule]\n\n" );
    exit(1);
    }

/* Update the schedule with a master schedule if the
argument is listed.
----------------------------------------------------*/
if ( argc == 4 )
    schedupdate( argv[1], argv[3] );

/* Open input and output files
------------------------------*/
inf = fopen( argv[1], "r" );
if ( !inf )
    {
    printf( "\nUnable to open file: %s for read\n\n", argv[1] );
    exit(1);
    }

ouf = fopen( argv[2], "w" );
if ( !ouf )
    {
    printf( "\nUnable to open file: %s for write\n\n", argv[2] );
    exit(1);
    }


/* Print some header information to the output file
---------------------------------------------------*/
print_header( ouf, lctm );


/* Process the input file and build the linked list
---------------------------------------------------*/
status = process( inf, &passlist );
if ( status != E_SUCC )
    exit(1);


/* Look for conflicts in the schedule
-------------------------------------*/
find_conflicts( &passlist );


/* Print conflicts in the output file
-------------------------------------*/
print_conflicts( ouf, &passlist );


/* Print a footer to the output file
------------------------------------*/
print_footer( ouf, &passlist );


/* Free memory and close files
------------------------------*/
curr = passlist;
while ( curr != NULL )
    {
    prev = curr;
    curr = prev->next;
    free( prev );
    }
fclose( inf );
fclose( ouf );

return 0;
}
