/************************************************************************
FUNCTION:	SCHEDCNFLT

PURPOSE:	This is the include file for SCHEDCNFLT.  It defines the
		structure for the pass linked list and contains function
		prototypes.


PROGRAM HISTORY:
Version Date    Author          Modification
_______ _____   _____________   ___________________
  1.0   10/98   D.Lloyd         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under Unix

PROJECT         ADAPS

$Log: schedcnflt.h,v $
Revision 1.5  2002/08/02 18:13:59  dlloyd
Corrected an error in reading the header lines from the autogen report, it
was reading one less line in one place and one too many in another.  CR 6560

Revision 1.4  2002/08/02 14:17:42  dlloyd
Modified the header file to only include headers for its prototypes.
Added necessary include in each .c file.
Moved external declaration from .h to .c file.
Added the Log directive.

*************************************************************************/
#ifndef SCHEDCNFLT_H
#define SCHEDCNFLT_H


#define SEC_PER_DAY	86400
#define SEC_PER_HR	3600
#define SEC_PER_MIN	60

#define PREHEADERLINES   6      /* number lines before the satnum line  */
#define POSTHEADERLINES 12      /* number lines after the satnum line   */


/* Linked list structure.  Holds information for one pass.
----------------------------------------------------------*/
typedef struct	PASS {
    char	line[140];	/* buffer to hold information		*/
    char	satnum[3];	/* sattelite number			*/
    long	sdate;		/* starting date YYDDD of pass		*/
    long	stime;		/* starting time of pass in seconds	*/
    long	edate;		/* ending date YYDDD of pass		*/
    long	etime;		/* ending time in seconds		*/
    long	conflict;	/* flag indicating a conflict		*/
    long	sched;		/* flag indicating pass is scheduled	*/
    struct PASS	*next;		/* pointer to next structure in list	*/
    } pass;

/* Included for FILE and struct tm
----------------------------------*/
#include <stdio.h>
#include <time.h>

/* Function prototypes
----------------------*/
int datestring( long, char * );
int find_conflicts( struct PASS ** );
long get_date( char * );
long get_time( char * );
int insert( struct PASS **, struct PASS ** );
int print_conflicts( FILE *, struct PASS ** );
int print_footer( FILE *, struct PASS ** );
int print_header( FILE *, struct tm * );
int process( FILE *, struct PASS ** );
int set_local( long *, long * );
int timestring( long, char * );
int schedupdate( char *, char * );

#endif
