/*****************************************************************************
FUNCTION:	SCHEDUPDATE

PURPOSE:	Update the current autogen print file with a master
		schedule file.  Note: only scenes scheduled in the
		master will be marked in the autogen file.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    11/98	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS

$Log: schedupdate.c,v $
Revision 1.6  2002/08/02 18:22:42  dlloyd
Initialized satnum to zero

Revision 1.5  2002/08/02 18:20:23  dlloyd
Added long modifier to printf

Revision 1.4  2002/08/02 18:13:59  dlloyd
Corrected an error in reading the header lines from the autogen report, it
was reading one less line in one place and one too many in another.  CR 6560

Revision 1.3  2002/08/02 14:17:42  dlloyd
Modified the header file to only include headers for its prototypes.
Added necessary include in each .c file.
Moved external declaration from .h to .c file.
Added the Log directive.

*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "schedcnflt.h"



int schedupdate(

char	*autogen,		/* autogen print file name		*/
char	*master			/* master schedule file name		*/
)
{
char	buf[151];		/* file input/output buffer		*/
char	*status;		/* function return status		*/
char	temp[6];		/* temporary conversion buffer		*/

FILE	*afd;			/* autogen file descriptor		*/
FILE	*mfd;			/* master file descriptor		*/
FILE	*tfd;			/* temp file descriptor			*/

long	i;			/* loop control				*/
long	nlines;			/* number of lines in master schedule	*/
long	sat[200];		/* array of satellite number		*/
long	orb[200];		/* array of orbit number		*/
long	satnum = 0;		/* sat num of current autogen line	*/
long	orbit;			/* orbit number of current autogen line	*/
long	schflg;			/* scheduled flag			*/
long	count;			/* count of EROS scheduled scenes	*/


/* Open files
-------------*/
afd = fopen( autogen, "r" );
if ( !afd )
    {
    printf( "\nUnable to open file: %s\n\n", autogen );
    return E_FAIL;
    }

mfd = fopen( master, "r" );
if ( !mfd )
    {
    printf( "\nUnable to open file: %s\n\n", master );
    return E_FAIL;
    }

tfd = fopen( "Temp_xx.file", "w" );
if ( !tfd )
    {
    printf( "\nUnable to open temporary file\n\n" );
    return E_FAIL;
    }

/* Read the master schedule and build a list of orbits
and satellite id's for scheduled passes.
------------------------------------------------------*/
fscanf( mfd, "%ld\n", &nlines );
/* temporary
sat = (long *)malloc( sizeof(long) * nlines );
orb = (long *)malloc( sizeof(long) * nlines );
if ( !sat || !orb )
    {
    printf( "\nUnable to allocate memory\n\n" );
    return E_FAIL;
    }
------------*/

count = 0;
for ( i = 0; i < nlines; i++ )
    {
    fgets( buf, 150, mfd );
    if ( buf[72] == 'E' )
	{
	strncpy( temp, &buf[18], 2 );
	temp[2] = 0;
	sat[count] = atoi( temp );
	strncpy( temp, &buf[79], 5 );
	temp[5] = 0;
	orb[count] = atoi( temp );
	count++;
	}
    }

/* Read the autogen report file, mark those that are
scheduled in the master and clear the rest writing
the file to a temp file.
----------------------------------------------------*/
while ( 1 )
    {
    status = fgets( buf, 150, afd );
    if ( status == NULL )
        break;
    
    /* look of newline character which indicates a header block */
    if ( buf[0] == 12 )
        {
	/* save the preheader lines */
	fputs( buf, tfd );
	for ( i = 0; i < PREHEADERLINES; i++ )
	    {
	    fgets( buf, 150, afd );
	    fputs( buf, tfd );
	    }

	/* get the satellite number */
	strncpy( temp, &buf[44], 2 );
	temp[2] = 0;
	satnum = atoi( temp );
	
	/* save the rest of the header */
	for ( i = 0; i < POSTHEADERLINES; i++ )
	    {
	    fgets( buf, 150, afd );
	    fputs( buf, tfd );
	    }
	
	/* get the first line after the header */
	fgets( buf, 150, afd );
	}
    
    /* get the orbit number */
    sscanf( buf, "%ld", &orbit );

    /* see if this satellite and orbit is scheduled */
    schflg = FALSE;
    for ( i = 0; i < count; i++ )
        {
	if ( satnum == sat[i] && orbit == orb[i] )
	    {
	    schflg = TRUE;
	    break;
	    }
	}
    
    /* if scheduled mark it as such, otherwise clear the field */
    if ( schflg )
        buf[109] = buf[110] = '*';
    else
        buf[109] = buf[110] = ' ';
    
    /* save the line */
    fputs( buf, tfd );
    }


/* Close the files.
-------------------*/
fclose( afd );
fclose( mfd );
fclose( tfd );


/* Rename the temp file as the autogen file.
--------------------------------------------*/
rename( "Temp_xx.file", autogen );

return E_SUCC;
}
