/******************************************************************************
FUNCTION:	DELETE_REC

PURPOSE:	Deletes a record from the master schedule.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0
  5.1	   07/92     T. Rockvam		Separated each delete subcommand into
					a separate file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "schedit.h"

void delete_rec
(
struct	SCHED	*schrec			/* Record information for delete*/
)
{
extern long	nrecs;			/* Number of master records	*/
extern struct SCHED *master[MAXRECS];	/* Master schedule addresses	*/

struct SCHED	*recptr;		/* Pointer to master schedule	*/

long	rec,				/* Record counter		*/
	dflag,				/* Delete flag			*/
	firstrec,			/* First master record to delete*/
	lastrec;			/* Last master record to delete */

/*
  If the schedule is empty there is nothing to delete
*/
if (nrecs == 0)
    {
    printf("[schedit-nodel] No records to be deleted\n");
    return;
    }

/*
  Find the range of records that will be deleted
*/
firstrec = find_loc(schrec->lstart,START);
lastrec  = find_loc(schrec->lstop,STOP);

/*
  If the first and last record are equal make sure that the start and stop
  acquisition times of the pass are completely within the specified range.
  Also if the last record is before the first record there is nothing to
  delete.
*/
if (firstrec == lastrec)
    {
    recptr = *(master + firstrec);
    if ((schrec->lstart <= recptr->lstart) && (schrec->lstop >= recptr->lstop))
	dflag = TRUE;
    else
	dflag = FALSE;
    }
else if (firstrec > lastrec)
    dflag = FALSE;
else
    dflag = TRUE;

/*
  Remove the records to be deleted or inform the user that there weren't any
  to remove
*/
if (dflag)
    {
    /*
      Adjust the number of master records
    */
    nrecs = nrecs - ((lastrec - firstrec) + 1);

    /*
      Move the record up from the bottom, in essence, removing the deleted
      records.
    */
    for (rec = firstrec; rec < nrecs; rec++)
	*(master + rec) = *(master + rec + (lastrec - firstrec) + 1);

    printf("[informational] %d records were deleted\n",lastrec - firstrec + 1);
    }
else
    {
    printf("[schedit-nodel] No records to be deleted\n");
    }

return;
}
