/******************************************************************************
FUNCTION:	FIND_LOC

PURPOSE:	Finds the location of the first record occurring later
		than the record key passed in.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "schedit.h"

int find_loc
(
long	rectime,		/* Time to find				*/
long	dirflag			/* Direction to search schedule in	*/
)
{
extern long	nrecs;			/* Number of master records	*/
extern struct SCHED *master[MAXRECS];	/* Master schedule addresses	*/

struct SCHED	*recptr;	/* Pointer to master schedule		*/

long	rec,			/* Record counter			*/
	mastime;		/* Time from the master schedule	*/

if (dirflag)
    {
    /*
      Search the list of records in ascending order
    */
    for (rec = 0; rec < nrecs; rec++)
        {
        /*
          Set the pointer to the master record location.  Search from top down
        */
        recptr = *(master + rec);
    
        /*
          Get the start time from the master schedule
        */
	mastime = recptr->lstart;
    
        /*
          When the master time is less than or equal to the search time,
          we have found where the record belongs.
        */
        if (mastime >= rectime)
	    break;
        }
    }
else
    {
    /*
      Search the list of records in descending order
    */
    for (rec = nrecs - 1; rec >= 0; rec--)
        {
        /*
          Set the pointer to the master record location.  Search from bottom up
        */
        recptr = *(master + rec);
    
        /*
          Get the stop time from the master schedule
        */
	mastime = recptr->lstop;
    
        /*
          When the master time is greater than or equal to the search time,
          we have found where the record belongs.
        */
        if (mastime <= rectime)
	    break;
        }
    }

if (rec < 0)
    rec = 0;
	
/*
  Return the index were the record should be placed
*/
return(rec);
}
