/******************************************************************************
FUNCTION:	GET_SCHED

PURPOSE:	Loads the master schedule file into memory.  If the master
		schedule file does not exist, it prints an informational
		message and returns with a successful return code.  Doing
		this allows function PUT_SCHED to create a new master
		schedule file without treating that as a special case.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  5.0      01/89    D. Hollaren		Conversion to LAS-5.0

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "worgen.h"
#include "schedit.h"

int get_sched
(
FILE	**mfp,	 		/* File pointer for master schedule file*/
char    *operation		/* operation to perform			*/
)
{
extern long	nrecs;		      /* Number of records in the master sched*/
extern struct SCHED *master[MAXRECS]; /* Pointers to master schedule structure*/

struct SCHED	*schrec;	/* Pointer to master schedule record	*/

long	rec,			/* Record counter			*/
	i;			/* Looping counter			*/

char	schedfile[CMLEN];	/* name of schedule file		*/

/*
  Set up the schedule file name
*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(schedfile,getenv("ADAPSTABLES"));
else
    {
    printf("[schedit-find] Error retrieving value of ADAPSTABLES\n");
    return(E_FAIL);
    }
strcat(schedfile,"/master.sch");

/*
  Try to open the schedule file
  If it is for a listing, open for read only.  CR 4911
*/
for (i = 0; i < 10; i++)
    {
    if ((strcmp( operation, "-rl" ) == 0) || (strcmp( operation, "-rb" ) == 0))
        *mfp = fopen(schedfile,"r");
    else
	*mfp = fopen(schedfile,"r+");
    if (*mfp == NULL)
	{
	printf("[schedit-mopen] Cannot open the master schedule file\n");
	sleep(5);
	}
    else
	break;
    }

if (*mfp == NULL)
    {
    printf("[schedit-mopen] Cannot open the master schedule file\n");
    return(E_FAIL);
    }

/*
  Get the number of entries
*/
fscanf(*mfp,"%d\n",&nrecs);

/*
  Get each record from the master schedule
*/
for (rec = 0; rec < nrecs; rec++)
    {
    /*
      Allocate space for this record and save the address
    */
    schrec = (struct SCHED *)(calloc(1, sizeof(struct SCHED)));
    *(master + rec) = schrec;
    fscanf(*mfp,SCHEDULE_FORMAT,schrec->scene_id, schrec->satid,
	   &schrec->lstart, &schrec->lstop, &schrec->rstart, &schrec->rstop,
	   &schrec->opmode, schrec->stat,&schrec->tfile, schrec->pflag,
	   schrec->dnflag, schrec->station,schrec->atype,schrec->orbit);
    }

return(E_SUCC);
}

