/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	get SCHEDIT parameters for SAVPARMS to write them to a file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "savparms.h"

int getpar
(
struct  PARBLK  *vblock,	/* TAE variable block           */
char    *schedfile,		/* schedule update file         */
char    *subcmd,		/* subcommand                   */
char    *satid,			/* satellite id                 */
char    *acqstat,		/* acquisition status           */
char    *prodflag,		/* product order flag           */
char    *station,		/* station code                 */
char    *upstation,		/* stations to update code      */
char    *dtype,			/* data type                    */
char    *sceneid,		/* scene id                     */
long    *orbit,			/* orbit number                 */
long    *a_start_tsec,		/* acq total seconds start time */
long    *a_stop_tsec,		/* acq total seconds stop time  */
long    *p_start_tsec,		/* pass total seconds start time*/
long    *p_stop_tsec,		/* pass total seconds stop time */
long    *opcode,		/* operation code               */
long    *hdtrfile		/* HDTR tape file number        */
)
{
struct VARIABLE *val;           	/* pointer to TAE variable in V-block*/

char	msg[ERRLEN];			/* error message		*/

long	one = 1,			/* value of one			*/
        sdim = 10,              	/* TAE block char parm length 	*/
	fatal = -1,			/* fatal error code		*/
	nonfatal = 1,			/* nonfatal error code		*/
	len,				/* string length		*/
	shortyr,			/* 2-digit year			*/
	acqsdate[3],			/* acquisition start date	*/
	acqedate[3],			/* acquisition end date		*/
	acqstime[3],			/* acquisition start time	*/
	acqetime[3],			/* acquisition end time		*/
	passdate[3],			/* pass start date		*/
	pasedate[3],			/* pass end date		*/
	passtime[3],			/* pass start time		*/
	pasetime[3];			/* pass end time		*/

/*
  Get the subcommand
*/
c_getstr(vblock,"_SUBCMD",&one,&sdim,subcmd);

/*
  Get the name of the schedule update file
  tmpsched is a local variable in the SCHEDIT procedure
*/
val = p_find(vblock,"tmpsched");
strcpy(schedfile,SVAL(*val,0));

/*
  Get parameters for each subcommand
ADD DELETE DELBYSID MODIFY MODBYSID MODBYSTA
*/
if ((strcmp(subcmd,"ADD") == 0) || (strcmp(subcmd,"MODIFY") == 0) || 
    (strcmp(subcmd,"MODBYSID") == 0) || (strcmp(subcmd,"MODBYSTA") == 0))
    {
    val = p_find(vblock,"SATID");
    if (val->v_count == 1)
        strcpy(satid,SVAL(*val,0));
    else
        strcpy(satid,"-");
    len = (long)strlen(satid);
    c_low2up(satid,&len);

    val = p_find(vblock,"STATUS");
    if (val->v_count == 1)
        strcpy(acqstat,SVAL(*val,0));
    else
        strcpy(acqstat,"-");

    val = p_find(vblock,"PRODFLAG");
    if (val->v_count == 1)
        strcpy(prodflag,SVAL(*val,0));
    else
        strcpy(prodflag,"-");

    val = p_find(vblock,"STATION");
    if (val->v_count == 1)
	if (strcmp(SVAL(*val,0),"SFL") == 0)
            strcpy(station,"E");
	else if (strcmp(SVAL(*val,0),"NGC") == 0)
            strcpy(station,"F");
	else
            strcpy(station,"W");
    else
        strcpy(station,"-");

    val = p_find(vblock,"ATYPE");
    if (val->v_count == 1)
        strcpy(dtype,SVAL(*val,0));
    else
        strcpy(dtype,"-");
    if ((strcmp(dtype,"LAC") == 0) || 
 	(strcmp(dtype,"GAC") == 0))
	strcat(dtype,"_");

    val = p_find(vblock,"ORBIT");
    if (val->v_count == 1)
        *orbit = IVAL(*val,0);
    else
        *orbit = 0;

    val = p_find(vblock,"OPCODE");
    if (val->v_count == 1)
        *opcode = IVAL(*val,0);
    else
        *opcode = 15;

    val = p_find(vblock,"HDTRFILE");
    if (val->v_count == 1)
        {
        *hdtrfile = IVAL(*val,0);
        if (*hdtrfile == 0) *hdtrfile = 99;
        }
    else
	if (strcmp(subcmd,"ADD") == 0)
            *hdtrfile = 99;
	else
            *hdtrfile = 00;
    }

if ((strcmp(subcmd,"ADD") == 0) || (strcmp(subcmd,"MODIFY") == 0) || 
    (strcmp(subcmd,"DELETE") == 0) || (strcmp(subcmd,"MODBYSTA") == 0))
    {
    val = p_find(vblock,"ASTADATE");
    acqsdate[0] = IVAL(*val,0);
    acqsdate[1] = IVAL(*val,1);
    acqsdate[2] = IVAL(*val,2);
    if (checkday(acqsdate) != E_SUCC)
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);

    val = p_find(vblock,"ASTATIME");
    acqstime[0] = IVAL(*val,0);
    acqstime[1] = IVAL(*val,1);
    acqstime[2] = IVAL(*val,2);
    if (checktim(acqstime) != E_SUCC)
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);

    makesecs(acqsdate,acqstime,a_start_tsec);

    val = p_find(vblock,"ASTODATE");
    if (val->v_count == 0)
        {
        /*
          If no ending date, set to same as beginning date
        */
        acqedate[0] = acqsdate[0];
        acqedate[1] = acqsdate[1];
        acqedate[2] = acqsdate[2];
        }
    else if (val->v_count == 3)
        {
        acqedate[0] = IVAL(*val,0);
        acqedate[1] = IVAL(*val,1);
        acqedate[2] = IVAL(*val,2);
        if (checkday(acqedate) != E_SUCC)
	    c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
        }
    else
	{
        c_errmsg("Invalid number of parameters for ASTODATE","savparms-invnum",
		 &nonfatal);
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}

    val = p_find(vblock,"ASTOTIME");
    acqetime[0] = IVAL(*val,0);
    acqetime[1] = IVAL(*val,1);
    acqetime[2] = IVAL(*val,2);
    if (checktim(acqetime) != E_SUCC)
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);

    makesecs(acqedate,acqetime,a_stop_tsec);

    if (*a_stop_tsec < *a_start_tsec)
        {
	sprintf(msg,
     "Acquisition stop time %02d:%02d:%02d is before start time %02d:%02d:%02d",
	acqetime[0],acqetime[1],acqetime[2],
	acqstime[0],acqstime[1],acqstime[2]);
        c_errmsg(msg,"savparms-stoptime",&nonfatal);
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
        }
    }

/*
  Special time error checking for adding new acquisitions
*/
if (strcmp(subcmd,"ADD") == 0)
    {
    if ((*a_start_tsec + 1200) < *a_stop_tsec)
        {
	sprintf(msg,"Acquisition stop time %02d:%02d:%02d is more than 20 minutes from start time %02d:%02d:%02d",
	acqetime[0],acqetime[1],acqetime[2],
	acqstime[0],acqstime[1],acqstime[2]);
        c_errmsg(msg,"savparms-timespan",&nonfatal);
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
        }

    val = p_find(vblock,"PSTADATE");
    if (val->v_count == 3)
        {
        passdate[0] = IVAL(*val,0);
        passdate[1] = IVAL(*val,1);
        passdate[2] = IVAL(*val,2);
        if (checkday(passdate) != E_SUCC)
	    c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
        }
    else if (val->v_count == 0)
        {
        passdate[0] = acqsdate[0];
        passdate[1] = acqsdate[1];
        passdate[2] = acqsdate[2];
        }
    else
	{
        c_errmsg("Invalid number of parameters for PSTADATE","savparms-invnum",
		 &nonfatal);
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}
    
    val = p_find(vblock,"PSTATIME");
    if (val->v_count == 3)
        {
        passtime[0] = IVAL(*val,0);
        passtime[1] = IVAL(*val,1);
        passtime[2] = IVAL(*val,2);
        if (checktim(passtime) != E_SUCC)
	    c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
        }
    else if (val->v_count == 0)
        {
        passtime[0] = acqstime[0];
        passtime[1] = acqstime[1];
        passtime[2] = acqstime[2];
        }
    else
	{
        c_errmsg("Invalid number of parameters for PSTATIME","savparms-invnum",
		 &nonfatal);
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}
    
    makesecs(passdate,passtime,p_start_tsec);

    val = p_find(vblock,"PSTODATE");
    if (val->v_count == 3)
        {
        pasedate[0] = IVAL(*val,0);
        pasedate[1] = IVAL(*val,1);
        pasedate[2] = IVAL(*val,2);
        if (checkday(pasedate) != E_SUCC)
	    c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
        }
    else if (val->v_count == 0)
        {
        pasedate[0] = acqedate[0];
        pasedate[1] = acqedate[1];
        pasedate[2] = acqedate[2];
        }
    else
	{
        c_errmsg("Invalid number of parameters for PSTODATE","savparms-invnum",
		 &nonfatal);
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}

    val = p_find(vblock,"PSTOTIME");
    if (val->v_count == 3)
        {
        pasetime[0] = IVAL(*val,0);
        pasetime[1] = IVAL(*val,1);
        pasetime[2] = IVAL(*val,2);
        if (checktim(pasetime) != E_SUCC)
	    c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
        }
    else if (val->v_count == 0)
        {
        pasetime[0] = acqetime[0];
        pasetime[1] = acqetime[1];
        pasetime[2] = acqetime[2];
        }
    else
	{
        c_errmsg("Invalid number of parameters for PSTOTIME","savparms-invnum",
		 &nonfatal);
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}

    makesecs(pasedate,pasetime,p_stop_tsec);

    if (*p_stop_tsec < *p_start_tsec)
        {
	sprintf(msg,
     "Pass stop time %02d:%02d:%02d is before start time %02d:%02d:%02d",
	pasetime[0],pasetime[1],pasetime[2],
	passtime[0],passtime[1],passtime[2]);
        c_errmsg(msg,"savparms-stoptime",&nonfatal);
	c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
        }

    /*
      Build the sceneid
    */
    shortyr = passdate[2] % 100;
    sprintf(sceneid,"a%c%c%c%02d%02d%02d%02d%02d%02d",dtype[0],satid[1],
	    satid[2],passdate[0],passdate[1],shortyr,
	    passtime[0],passtime[1],passtime[2]);    
    len = (long)strlen(sceneid);
    c_low2up(sceneid,&len);
    }

if ((strcmp(subcmd,"DELBYSID") == 0) || (strcmp(subcmd,"MODBYSID") == 0))
    {
    val = p_find(vblock,"SCENEID");
    strcpy(sceneid,SVAL(*val,0));
    len = (long)strlen(sceneid);
    c_low2up(sceneid,&len);
    }

if (strcmp(subcmd,"MODBYSTA") == 0)
    {
    val = p_find(vblock,"UPSTA");
    if (val->v_count == 1)
	if (strcmp(SVAL(*val,0),"SFL") == 0)
            strcpy(upstation,"E");
	else if (strcmp(SVAL(*val,0),"NGC") == 0)
            strcpy(upstation,"F");
	else
            strcpy(upstation,"W");
    else
	/*
	  "-" indicates all stations will be updated
	*/
        strcpy(upstation,"-");
    }

return(E_SUCC);
}
