/******************************************************************************
FUNCTION:	LIST_SCHED

PURPOSE:	List_schedule reformats the master schedule to make it
		more readable and writes it to a disk file.  The reformatting
		includes breaking the schedule into pages (with a header on
		each page) and converting the time stored in total seconds
		into a gregorian day and time.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "worgen.h"
#include "schedit.h"

int list_sched
(
char	*filename,		      /* Report file name		      */
long	sform			      /* Report format (long or short)        */
)
{
extern long	nrecs;		      /* Number of records in the master sched*/
extern struct SCHED *master[MAXRECS]; /* Pointers to master schedule structure*/

struct SCHED	*nontape[MAXRECS];    /* Pointer to non-tape records	      */
struct SCHED	*schrec;	      /* Pointer to master schedule record    */

struct tm	*ltime;		/* Local time				      */
struct tm	*gtime;		/* GMT time				      */

FILE	*report;		/* File pointer to the report file	      */

long	rec,			/* Record counter			      */
	line,			/* Line count for # lines written to a page   */
	clock,			/* Total seconds from 00:00:00, Jan 1, 1971   */
	gsecs,			/* Total seconds at GMT time		      */
	lsecs,			/* Total sec local time (5/6 hours before GMT)*/
	otype,			/* Type of header to write in report	      */
	loffset,		/* Difference in seconds between GMT & local  */
	pfile = 0,		/* Flag to indicate if print file is open     */
	nonrec = 0;		/* Total non-tape records in schedule	      */
	
/* 
  Open the report file
*/
if ((strlen(filename) > 0) && (strcmp(filename,"--") != 0))
    {
    report = fopen(filename,"w");
    if (report == NULL)
	{
	printf("[schedit-ropen] Error opening report file %s\n",filename);
	return(E_FAIL);
	}
    pfile = 1;
    }
else
    report = stdout;

#ifdef vms
    /* there is NO WAY on VMS to automatically catch daylight savings 
       time or standard time, must reset DAYSAV in sched.h and recompile 
       twice a year */
    if (DAYSAV)
	loffset = 18000;
    else
	loffset = 21600;
#else
    /*
      Compute difference between GMT & local time (either 5 or 6 hours)
    */
    clock = time(0);
    ltime = localtime(&clock);
    lsecs = (ltime->tm_hour * 3600) + (ltime->tm_min*60) + ltime->tm_sec;

    gtime = gmtime(&clock);
    gsecs = (gtime->tm_hour * 3600) + (gtime->tm_min*60) + gtime->tm_sec;

    if (gsecs < lsecs)
	/* add one day on to gsecs */
	gsecs += 24 * 3600;

    if ((gsecs - lsecs) > 19799)
	loffset = 21600;
    else
	loffset = 18000;
#endif

/*
  Write the header for passes received to tape
*/
line = 0;
otype = 0;
write_head(report,&line,&otype);

/*
  Search the schedule for the records on tape
*/
for (rec = 0; rec < nrecs; rec++)
    {
    schrec = *(master + rec);
    if ((*schrec->stat == TAPE) || (*schrec->stat == BOTH))
	{
	if ((!sform) || (schrec->opmode > 0))
	    write_line(report,&line,schrec,&loffset,&otype);
	}
    else
	{
	/*
	  Save the records that are not on tape
	*/
	*(nontape + nonrec) = schrec;
	nonrec++;
	}
    }

/*
  Write out the non-tape records
*/
otype = 1;
write_head(report,&line,&otype);

for (rec = 0; rec < nonrec; rec++)
    {
    schrec = *(nontape + rec);
    if ((!sform) || (schrec->opmode > 0))
	write_line(report,&line,schrec,&loffset,&otype);
    }

write_foot(report,&line);
	    
if (pfile)
    fclose(report);

return(E_SUCC);
}
