/******************************************************************************
FUNCTION:	MODIFY_REC

PURPOSE:	Modifies the fields of a record from the master schedule.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "schedit.h"

void modify_rec
(
struct	SCHED	*schrec			/* Record information for modify*/
)
{
extern long	nrecs;			/* Number of master records	*/
extern struct SCHED *master[MAXRECS];	/* Master schedule addresses	*/

struct SCHED	*recptr;		/* Pointer to master schedule	*/

long	rec,				/* Record counter		*/
	mflag,				/* Modify flag			*/
	firstrec,			/* First master record to modify*/
	lastrec;			/* Last master record to modify	*/

/*
  If the schedule is empty there is nothing to delete
*/
if (nrecs == 0)
    {
    printf("[schedit-nomod] No records to be modified\n");
    return;
    }

/*
  Find the range of records that will be modified
*/
firstrec = find_loc(schrec->lstart,START);
lastrec  = find_loc(schrec->lstop,STOP);

/*
  If the first and last record are equal make sure that the start and stop
  acquisition times of the pass are completely within the specified range.
  Also if the last record is before the first record there is nothing to
  modify.
*/
if (firstrec == lastrec)
    {
    recptr = *(master + firstrec);
    if ((schrec->lstart <= recptr->lstart) && (schrec->lstop >= recptr->lstop))
	mflag = TRUE;
    else
	mflag = FALSE;
    }
else if (firstrec > lastrec)
    mflag = FALSE;
else
    mflag = TRUE;

/*
  Modify the records to be changed or inform the user that there weren't any
  to modify
*/
if (mflag)
    {
    for (rec = firstrec; rec <= lastrec; rec++)
	{
	recptr = *(master + rec);

	/*
	  Modify the satellite id if specified
	  Changing the satellite requires recalculating the Day/Night flag
	  and changing the sceneid
	*/
	if (schrec->satid[0] != '-')
	    {
	    strcpy(recptr->satid,schrec->satid);
    	    if (setdn(recptr->satid,recptr->rstart,recptr->rstop,recptr->dnflag,
		recptr->atype) == E_FAIL)
        	{
        	printf("[schedit-dayn] Error occurred setting day/night indicator for %d acquisition\n",recptr->rstart);
        	printf("[schedit-dayn] Setting indicator to DAY\n");
        	strcpy(recptr->dnflag,"D");
        	}
    	    recptr->scene_id[2] = schrec->satid[1];
    	    recptr->scene_id[3] = schrec->satid[2];
	    }

	/*
	  Operation code can only be changed to TAPEtoDISK when it is
	  on tape
	*/
	if (schrec->opmode == TAPEtoDISK)
	    {
	    if ((recptr->tfile != 99) || (schrec->tfile != 99))
		{
		recptr->opmode = schrec->opmode;
		strcpy(schrec->stat,"P");
		}
	    }
	else if (schrec->opmode != NOOPFIVE)
	    recptr->opmode = schrec->opmode;

	if (schrec->stat[0] != '-')
	    strcpy(recptr->stat,schrec->stat);

	if (schrec->tfile != 0)
	    recptr->tfile  = schrec->tfile;

	if (schrec->pflag[0] != '-')
	    strcpy(recptr->pflag,schrec->pflag);

	if (schrec->station[0] != '-')
	    strcpy(recptr->station,schrec->station);

	if (schrec->atype[0] != '-')
	    strcpy(recptr->atype,schrec->atype);

	if (schrec->orbit[0] != '-')
	    strcpy(recptr->orbit,schrec->orbit);
	}
    printf("[informational] %d records were modified\n",lastrec - firstrec + 1);
    }
else
    {
    printf("[schedit-nomod] No records to be modified\n");
    }

return;
}
