/******************************************************************************
FUNCTION:	MODIFY_SID

PURPOSE:	Modify a record from the master schedule by scene-id.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "schedit.h"

void modify_sid
(
struct	SCHED	*schrec			/* Record information for modify*/
)
{
extern long	nrecs;			/* Number of master records	*/
extern struct SCHED *master[MAXRECS];	/* Master schedule addresses	*/

struct SCHED	*recptr;		/* Pointer to master schedule	*/

long	rec,				/* Record counter		*/
	mflag;				/* Modify flag			*/

mflag = FALSE;

/*
  Search the list of records in ascending order for the specified scene-id
*/
for (rec = 0; rec < nrecs; rec++)
    {
    recptr = *(master + rec);
    if (strcasecmp(recptr->scene_id,schrec->scene_id) == 0)
	{
	mflag = TRUE;
	break;
	}
    }

if (mflag)
    {
    /*
      Modify the satellite id if specified
      Changing the satellite requires recalculating the Day/Night flag        
    */
    if (schrec->satid[0] != '-')
	{
	strcpy(recptr->satid,schrec->satid);
        if (setdn(recptr->satid,recptr->rstart,recptr->rstop,recptr->dnflag,
            recptr->atype) == E_FAIL)
            {
            printf("[schedit-dayn] Error occurred setting day/night indicator for %d acquisition\n",recptr->rstart);
            printf("[schedit-dayn] Setting indicator to DAY\n");
            strcpy(recptr->dnflag,"D");
            }
        recptr->scene_id[2] = schrec->satid[1];
        recptr->scene_id[3] = schrec->satid[2];
        }

    /*
      Operation code can only be changed to TAPEtoDISK when it is on tape
    */
    if (schrec->opmode == TAPEtoDISK)
	{
	if ((recptr->tfile != 99) || (schrec->tfile != 99))
	    {
	    recptr->opmode = schrec->opmode;
	    strcpy(recptr->stat,"P");
	    }
	}
    else if (schrec->opmode != NOOPFIVE)
	recptr->opmode = schrec->opmode;

    if (schrec->stat[0] != '-')
	strcpy(recptr->stat,schrec->stat);

    if (schrec->tfile != 0)
	recptr->tfile = schrec->tfile;

    if (schrec->pflag[0] != '-')
	strcpy(recptr->pflag,schrec->pflag);

    if (schrec->station[0] != '-')
	strcpy(recptr->station,schrec->station);

    if (schrec->atype[0] != '-')
	strcpy(recptr->atype,schrec->atype);

    if (schrec->orbit[0] != '-')
	strcpy(recptr->orbit,schrec->orbit);

    printf("[informational] Record with scene ID %s was modified\n",
	schrec->scene_id);
    }
else
    {
    printf("[schedit-noscene] Record with scene ID %s not found\n",
	schrec->scene_id);
    }

return;
}
