/******************************************************************************
FUNCTION:	PUT_SCHED

PURPOSE:	Writes the schedule to a new master schedule file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      01/89     D. Hollaren        Conversion to LAS-5.0

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "worgen.h"
#include "schedit.h"

int put_sched
(
FILE    *mfp                  /* File pointer for master schedule file*/
)
{
extern long nrecs;		      /* Number of records in master schedule */
extern struct SCHED *master[MAXRECS]; /* Pointer to the schedule structure    */

struct SCHED	*schrec;	/* Pointer to master schedule record	*/

FILE	*nfp;			/* File pointer to the new master schedule*/

long	rec,			/* Record counter			*/
	i;			/* Looping counter			*/

char	schedfile[CMLEN];	/* Schedule file name			*/


/*
  Set up the schedule file name
*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(schedfile,getenv("ADAPSTABLES"));
else
    {
    printf("[schedit-find] Error retrieving value of ADAPSTABLES\n");
    return(E_FAIL);
    }
#ifdef vms
strcat(schedfile,"master.sch");
#else
strcat(schedfile,"/master.sch");
#endif

/*
  Close the old and open a new schedule file
*/
fclose(mfp);
for (i = 0; i < 10; i++)
    {
    nfp = fopen(schedfile,"w");
    if (nfp == NULL)
	printf("[schedit-mopen] Cannot open the master schedule file\n");
    else
	break;
    }

if (nfp == NULL)
    {
    printf("[schedit-mopen] Cannot open the master schedule file\n");
    return(E_FAIL);
    }

/*
  Write the number of records to the master schedule file
*/
fprintf(nfp,"%d\n",nrecs);

/*
  Write the records to the file
*/
for (rec = 0; rec < nrecs; rec++)
    {
    schrec = *(master + rec);
    fprintf(nfp,SCHEDULE_FORMAT,schrec->scene_id, schrec->satid,
	    schrec->lstart, schrec->lstop, schrec->rstart, schrec->rstop,
	    schrec->opmode, schrec->stat, schrec->tfile, schrec->pflag,
	    schrec->dnflag, schrec->station, schrec->atype, schrec->orbit);
    free(schrec);
    }

/*
  Close the schedule file
*/
fclose(nfp);

return(E_SUCC);
}
