/******************************************************************************
FUNCTION:	SAVPARMS

PURPOSE:	Saves parameters in the temporary file used to modify the 
		schedule file

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development in PDF using DCL
  5.0      07/92     T. Rockvam         Conversion to LAS-5.0;
					uses a C program instead of DCL to be
					unix portable

ALGORITHM REFERENCES	SCHEDIT.PDF
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "savparms.h"

void main( void )
{
struct	PARBLK vblock;		/* TAE variable block			*/

char	satid[4],		/* satellite id (N10, N11, N12)		*/
	schedfile[CMLEN],	/* schedule update file			*/
	acqstat[2],		/* acquisition status (D,T,B,P,M,N)	*/
	prodflag[2],		/* flag 				*/
	station[2],		/* station (E, F, W,)			*/
	upstation[2],		/* station to update (E, F, W,)		*/
	dtype[5],		/* data type (HRPT, LAC_, GAC_)		*/
	sceneid[17],		/* scene id				*/
	subcmd[10];		/* subcommand				*/

long	orbit,			/* orbit number				*/
        acq_start_sec,		/* acq total second start time  	*/
        acq_stop_sec,		/* acq total second stop time   	*/
        pass_start_sec,		/* pass total second start time 	*/
        pass_stop_sec,		/* pass total second stop time  	*/
	opcode,			/* operation code			*/
	hdtrfile,		/* HDTR tape file number 		*/
	stdlun;			/* standard output			*/
	
/*
  Initialize LAS
*/
c_inlas(&vblock,&stdlun);

/*
  Get parameters
*/
getpar(&vblock,schedfile,subcmd,satid,acqstat,prodflag,station,upstation,
	dtype,sceneid,&orbit,&acq_start_sec,&acq_stop_sec,&pass_start_sec,
	&pass_stop_sec,&opcode,&hdtrfile);


/*
  Write information to the file
*/
wrt_tmpfile(subcmd,schedfile,sceneid,satid,acq_start_sec,acq_stop_sec,
	    pass_start_sec,pass_stop_sec,opcode,acqstat,hdtrfile,prodflag,
	    station,upstation,dtype,orbit);

}
