/******************************************************************************
FUNCTION:	WRT_TMPFILE

PURPOSE:	Write the temporary file used to update the schedule

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "savparms.h"

void wrt_tmpfile
(
char    *subcmd,		/* subcommand selected                  */
char    *schedfile,		/* name of schedule update file         */
char    *sceneid,		/* scene id                             */
char    *satid,			/* satellite id (N10, N11, N12)         */
long    a_start_sec,		/* acq total second start time          */
long    a_stop_sec,		/* acq total second stop time           */
long    p_start_sec,		/* pass total second start time         */
long    p_stop_sec,		/* pass total second stop time          */
long    opcode,			/* operation code                       */
char    *acqstat,		/* acquisition status (D,T,B,P,M,N)     */
long    hdtrfile,		/* HDTR tape file number                */
char    *prodflag,		/* product flag                         */
char    *station,		/* station (E, F, W,)                   */
char    *upstation,		/* station to update (E, F, W, all)     */
char    *dtype,			/* data type (HRPT, LAC_, GAC_)         */
long    orbit			/* orbit number                         */
)
{
FILE 	*fd;			/* file pointer to temp file		*/

long	nonfatal = 1,		/* nonfatal error code			*/
	fatal = -1,		/* fatal error code			*/
	status;			/* function return status		*/

char	orbnum[2];		/* orbit number				*/

/*
  Open the schedule update file
*/
fd = fopen(schedfile,"a");
if (fd == NULL)
    {
    c_errmsg("Error opening temporary file","savparms-open",&nonfatal);
    c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
    }

/*
  Write a record for each type of subcommand
*/
if (strcmp(subcmd,"ADD") == 0)
    {
    status = fprintf(fd,
	     "%16.16s %s %09d %09d %09d %09d %1d %c %02d %c %c %4.4s %05d\n",
	     sceneid,satid,a_start_sec,a_stop_sec,p_start_sec,
	     p_stop_sec,opcode,acqstat[0],hdtrfile,prodflag[0],station[0],
	     dtype,orbit);
    if (status == EOF)
	{
	fclose(fd);
        c_errmsg("Error writing to temporary file","savparms-write",
		 &nonfatal);
        c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}
    }
else if (strcmp(subcmd,"DELETE") == 0)
    {
    status = fprintf(fd,"%09d %09d\n",a_start_sec,a_stop_sec);
    if (status == EOF)
	{
	fclose(fd);
        c_errmsg("Error writing to temporary file","savparms-write",
		 &nonfatal);
        c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}
    }
else if (strcmp(subcmd,"DELBYSID") == 0)
    {
    status = fprintf(fd,"%16.16s\n",sceneid);
    if (status == EOF)
	{
	fclose(fd);
        c_errmsg("Error writing to temporary file","savparms-write",
		 &nonfatal);
        c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}
    }
else if (strcmp(subcmd,"MODIFY") == 0)
    {
    if (orbit == 0)
	strcpy(orbnum,"-");
    else
	sprintf(orbnum,"%d",orbit);

    status = fprintf(fd,
	     "%09d %09d %s %02d %c %02d %c %c %4.4s %s\n",
	     a_start_sec,a_stop_sec,satid,opcode,acqstat[0],hdtrfile,
	     prodflag[0],station[0],dtype,orbnum);
    if (status == EOF)
	{
	fclose(fd);
        c_errmsg("Error writing to temporary file","savparms-write",
		 &nonfatal);
        c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}
    }
else if (strcmp(subcmd,"MODBYSTA") == 0)
    {
    if (orbit == 0)
	strcpy(orbnum,"-");
    else
	sprintf(orbnum,"%d",orbit);

    status = fprintf(fd,
	     "%09d %09d %c %s %02d %c %02d %c %c %4.4s %s\n",
	     a_start_sec,a_stop_sec,upstation[0],satid,opcode,acqstat[0],
	     hdtrfile,prodflag[0],station[0],dtype,orbnum);
    if (status == EOF)
	{
	fclose(fd);
        c_errmsg("Error writing to temporary file","savparms-write",
		 &nonfatal);
        c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}
    }
else if (strcmp(subcmd,"MODBYSID") == 0)
    {
    if (orbit == 0)
	strcpy(orbnum,"-");
    else
	sprintf(orbnum,"%d",orbit);

    status = fprintf(fd,
	     "%16.16s %s %02d %c %02d %c %c %4.4s %s\n",
	     sceneid,satid,opcode,acqstat[0],hdtrfile,prodflag[0],station[0],
	     dtype,orbnum);
    if (status == EOF)
	{
	fclose(fd);
        c_errmsg("Error writing to temporary file","savparms-write",
		 &nonfatal);
        c_errmsg("Fatal error encountered","savparms-fatal",&fatal);
	}
    }

/*
  Close the temporary file
*/
status = fclose(fd);
if (status != E_SUCC)
    c_errmsg("Error closing temporary file","savparms-close",&nonfatal);

return;
}
