/******************************************************************************
NAME: GETPAR 

PURPOSE: To obtain the necessary parameters for the SCRUNCH function.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Baltzer	May  1992	initial development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT: LAS		                

******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "scrunch.h"

/*
   The following variables must be declared
   global for image I/O routines
-------------------------------------------*/
extern int  fdesc1[MAXIMG+1];	/* image file descriptors used for image I/O */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands  */


int getpar
(
  struct PARBLK *vblock,      /* Pointer to TAE block */
  long window[][4],           /* Window specification */
  long  *maxnl,               /* Maximum number of lines in output */
  long  *maxns                /* Maximum number of samples in output */
)
{
  long access = IREAD;              /* Type of access for file */
  long band;                        /* Band index */
  long bands[MAXIMG][MAXBND+1];     /* Band specification */
  long dtype;                       /* Data type of previous input image */
  long element = 0;                 /* Parameter element number */
  char hosin[MAXIMG][CMLEN];        /* Host file names of input images */
  long i;                           /* Counter-no. of input images */
  long imglim = MAXIMG;             /* Max number of images to be processed */
  long nbands[MAXIMG];              /* Number of bands per input image */
  long nl;                          /* Temporary no. of lines in input image */
  long nmrimg;                      /* Number of input images */
  long ns;                          /* Temp no. of samples in input image */
  long odtype;                      /* Data type of input image */
  long qflag[MAXIMG][2];            /* Flag telling if subwindows specified */
  long same;                        /* Flag if images have same window */
  long sl;                          /* Starting line of input image */
  long ss;                          /* Starting sample of input image */
  long status;                      /* Function return status code */
  long totbnd;                      /* Total number of bands */
  long zero = 0;                    /* Constant 0 */
 
  char taein[MAXIMG][CMLEN];        /* TAE file names of input images */

  struct VARIABLE *val;             /* Pointer to a TAE variable */


  /* Call getin to parse the input image specification.
     Getin returns the TAE image name(s), host image name(s),
     bands to be processed, window specification, and number
     of images parsed.
  -----------------------------------------------------------*/
  status = c_getin(vblock,"IN",&element,taein,hosin,&nmrimg,window,bands,
                   nbands,&totbnd,&imglim,&same,qflag);
  if (status != E_SUCC)
    return(E_FAIL);
  if (same == 0)
  {
    c_lasmsg(FALSE,"scrunch-window",
             "Input imags do not have same window sizes");
    return(E_FAIL);
  }

  /* Check data type of input images, etc.
  ----------------------------------------*/
  nl = 0;
  ns = 0;
  sl = 1;
  ss = 1;
	
  for (band = 0,i = 0; i < nmrimg; i++)
  {
    dtype = 0;
    status = c_eopens(&fdesc1[i],hosin[i],bands[i],&sl,&ss,&nl,&ns,
		      &offset1[band],&access,&dtype,&zero,&zero);
    if (status != E_SUCC)
      return(E_FAIL);
    band += nbands[i];
    status = c_eclose(&fdesc1[i]);
    if (status != E_SUCC)
      c_lasmsg(FALSE,"scrunch-warn","Nonfatal error encountered");
    if (i == 0)
      odtype = dtype;
    else if (dtype != odtype)
    {
      c_lasmsg(FALSE,"scrunch-dtype",
               "Input images are not the same data type");
      return(E_FAIL);
    }	
  }

  /* Retrieve the maximum number of lines
  ---------------------------------------*/
  val = p_find(vblock,"MAXNL");
  *maxnl = IVAL(*val,0);

  /* Retrieve the maximum number of samples
  -----------------------------------------*/
  val = p_find(vblock,"MAXNS");
  *maxns = IVAL(*val,0);

  if ((*maxnl < 2) || (*maxns < 2))
  {
    if (*maxnl < 2)
      c_lasmsg(FALSE,"scrunch-invalid","MAXNL must be greater than 1");
    else
      c_lasmsg(FALSE,"scrunch-invalid","MAXNS must be greater than 1");
    return(E_FAIL);
  }

  return(E_SUCC);
}
