/******************************************************************************
NAME:   			BREAK_ORB 

PURPOSE:    Determine where to break the half orbit.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		REASON
-------	   ----------      ----		------
  1.0	   T. Baltzer	   02/94	Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:			ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "stitch.h"             /* Orbital stitch include file         */


FUNCTION break_orb(prevbuf,currbuf,prevline,currline,break_flg)

   char prevbuf[];		/* Text buffer of previous line		*/
   char currbuf[];		/* Text buffer of current line		*/
   struct ONELN *prevline;	/* Structure of previous line data	*/
   struct ONELN *currline;	/* Structure of current line data	*/
   long *break_flg;		/* Break half orbit (TRUE of FALSE)	*/

{

long status;			/* return status value			*/
long stoptime;			/* Acquisition stop time		*/
long startime;			/* Acquisition start time		*/
long ndays;			/* days between prevdate, currdate	*/

char prevdate[7];		/* Date of previous line MMDDYY		*/
char currdate[7];		/* Date of current  line MMDDYY		*/


/*
  Get the start time in seconds of the current line
*/
startime = currline->start_sec;


/*
  Get the stop time in seconds of the previous line
*/
stoptime = prevline->end_sec;


/*
  Get the current line's date
*/
strncpy(&currdate[0],&currbuf[10],2);
strncpy(&currdate[2],&currbuf[13],2);
strncpy(&currdate[4],&currbuf[16],2);
currdate[6] = '\0';


/*
  Get the previous line's date
*/
strncpy(&prevdate[0],&prevbuf[10],2);
strncpy(&prevdate[2],&prevbuf[13],2);
strncpy(&prevdate[4],&prevbuf[16],2);
prevdate[6] = '\0';


/*
  If day has been spanned, add time of day(s) between
*/
status = days_ago(currdate,prevdate,&ndays);
if (status != E_SUCC)
   return(E_FAIL);
startime += (ndays * 86400);


/*
  If start time - stop time is > 31 minutes
  then this next line is part of the next orbit
*/
if ((startime - stoptime) > 1860)
   *break_flg = TRUE;
else
   *break_flg = FALSE;


return(E_SUCC);
}
