/*******************************************************************************
FUNCTION:	evalreg

PURPOSE:	Evaluate a lat/long to determine which of the 12 regions
		of the earth it lies within.
			 
Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    01/94  T. Baltzer	 Initial development
				

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"


FUNCTION evalreg (lat,lng,regflg)

double lat;			/* Latitude in question			      */
double lng;			/* Longitude in question		      */
long regflg[];			/* Region flag, 0 = not found, 1 = found      */

{

/* The 12 regions are defined as follows (degrees are easier to read):

region number   min latitude   max latitude   min longitude   max longitude
1               40.73333        90.             -180.           -40.
2               40.73333        90.             -40.            180.
3               0.              40.73333        -180.           -40.
4               0.              40.73333        -40.            180.
5               -40.73333       0.              -180.           -100.
6               -40.73333       0.              -100.           -20.
7               -40.73333       0.              -20.            80.
8               -40.73333       0.              80.             180.
9               -90.            -40.73333       -180.           -100.
10              -90.            -40.73333       -100.           -20.
11              -90.            -40.73333       -20.            80.
12              -90.            -40.73333       80.             180.
*/
static double reg[12][4] =
        {
                0.7109880,      1.570796,       -3.141593,      -0.7109880,
                0.7109880,      1.570796,       -0.7109880,     3.141593,
                0.0,            0.7109880,      -3.141593,      -0.7109880,
                0.0,            0.7109880,      -0.7109880,     3.141593,
                -0.7109880,     0.0,            -3.141593,      -1.745329,
                -0.7109880,     0.0,            -1.745329,      -0.349066,
                -0.7109880,     0.0,            -0.349066,      1.396263,
                -0.7109880,     0.0,            1.396263,       3.141593,
                -1.570796,      -0.7109880,     -3.141593,      -1.745329,
                -1.570796,      -0.7109880,     -1.745329,      -0.349066,
                -1.570796,      -0.7109880,     -0.349066,      1.396263,
                -1.570796,      -0.7109880,     1.396263,       3.141593
        };

long i;				/* Counter				      */


for (i = 0; i < 12; i++)
   {
   if ((lat >= reg[i][0]) && (lat <= reg[i][1]) &&
       (lng >= reg[i][2]) && (lng <= reg[i][3]))
      {
      regflg[i] = TRUE;
      break;
      }
   }

return;
}
