/******************************************************************************
FUNCTION:	GETREGIONS

PURPOSE:	Find the region numbers which lie within a given image
		based on model information.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       02/94    T. Baltzer		initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include "las.h"		/* LAS include			       */
#include "adaps.h"		/* ADAPS include		       */
#include "stitch.h"             /* Orbital stitch include file         */

FUNCTION getregions(oneline,regflg)

   struct ONELN *oneline;	/* Structure with info from one line	*/
   long regflg[12];		/* Region flag: 0=Not found, 1=Found	*/

{

long fatal = -1;		/* Fatal error status			*/
long linenum;			/* Line number under consideration	*/
long i;				/* Counter				*/
long skpln = 100;		/* Number of lines between evaluations	*/
long sampnum;			/* Sample number under consideration	*/
long status;			/* Return status			*/

float flt_line;			/* Floating point version of line 	*/
float flt_samp;			/* Floating point version of sample	*/

double lat,			/* Latitude of line/sample		*/
       lng,			/* Longitude of line/sample		*/
       elev = 0.0;		/* Elevation				*/

struct MODEL model;		/* AVHRR model information		*/


/* Initialize the model
-----------------------*/
status = c_initmod(&oneline->satnum,&oneline->year,&oneline->month,
		   &oneline->day,&oneline->shour,&oneline->smin,
		   &oneline->ssec,&oneline->sfrac,&oneline->dtype,
                   &oneline->nlines,&model);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","subregions-fatal",&fatal);


/* To determine the regions which lie within the image,
   select pixels at sample 1, sample n/2, and sample n
   where n is the number of samples in the image.  These
   columns represent leftmost, nadir, and rightmost values.
   The pixel line numbers will start at 1 and increment
   by every skpln lines.  The final line number
   will be used even if it does not fall on the increment.
     Given these pixel values, evaluate their location to
   determine which of the 12 regions on earth that they
   fall into.
*/
for (i = 0; i < 3; i++)
   {
   switch (i)
      {
      case 0: sampnum = 1;
	      break;
      case 1: sampnum = model.nsamples / 2;
	      break;
      case 2: sampnum = model.nsamples;
	      break;
      }
   for (linenum = 1; linenum <= model.nlines; linenum += skpln)
      {
      flt_line = (float)linenum;
      flt_samp = (float)sampnum;
      c_ls2ll(&flt_line,&flt_samp,&elev,&model,&lat,&lng,NULL);
      evalreg(lat,lng,regflg);
      }
   if (linenum - skpln != model.nlines)
      {
      linenum = model.nlines;
      flt_line = (float)linenum;
      flt_samp = (float)sampnum;
      c_ls2ll(&flt_line,&flt_samp,&elev,&model,&lat,&lng,NULL);
      evalreg(lat,lng,regflg);
      }
   }

}
