/******************************************************************************
NAME:   			INITVALS 

PURPOSE:    Get information from one line of data to initialize the model.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		REASON
-------	   ----------      ----		------
  1.0	   T. Baltzer	   02/94	Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:			ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "stitch.h"             /* Orbital stitch include file         */



FUNCTION initvals(linebuf,oneline)

char linebuf[CMLEN+1];		/* Buffer containing line of data	*/
struct ONELN *oneline;		/* Structure with info from one line 	*/

{

char tmpstr[3];		/* Temporary string variable		*/

long i;			/* Counter				*/
long tmpint;		/* Temporary integer variable		*/


/* Get the satellite number
---------------------------*/
strncpy(tmpstr,&linebuf[37],2);
tmpstr[2] = '\0';
sscanf(tmpstr,"%d",&oneline->satnum);


/* Get the acquisition month, day, year, 
   starting hour, minute, second, frac second
---------------------------------------------*/
strncpy(tmpstr,&linebuf[10],2);
tmpstr[2] = '\0';
sscanf(tmpstr,"%d",&oneline->month);

strncpy(tmpstr,&linebuf[13],2);
tmpstr[2] = '\0';
sscanf(tmpstr,"%d",&oneline->day);

strncpy(tmpstr,&linebuf[16],2);
tmpstr[2] = '\0';
sscanf(tmpstr,"%d",&oneline->year);

strncpy(tmpstr,&linebuf[19],2);
tmpstr[2] = '\0';
for (i = 0; i < 2; i++)
   if (tmpstr[i] == ' ')  tmpstr[i] = '0';
sscanf(tmpstr,"%d",&oneline->shour);

strncpy(tmpstr,&linebuf[21],2);
tmpstr[2] = '\0';
for (i = 0; i < 2; i++)
   if (tmpstr[i] == ' ')  tmpstr[i] = '0';
sscanf(tmpstr,"%d",&oneline->smin);

strncpy(tmpstr,&linebuf[23],2);
tmpstr[2] = '\0';
for (i = 0; i < 2; i++)
   if (tmpstr[i] == ' ')  tmpstr[i] = '0';
sscanf(tmpstr,"%d",&oneline->ssec);

oneline->sfrac = 0;


/* Determine the data type
   HRPT or LAC
--------------------------*/
if ((linebuf[8] == 'l') || (linebuf[8] == 'L'))
   oneline->dtype = 1;	/* LAC */
else
    oneline->dtype = 3;  /* HRPT */


/* Calculate the number of lines in the image.  Do this by determining
the start and end time in seconds, subtract to determine the elapsed
time, and multiply by the number of lines per second.
-----------------------------------------------------------------------*/
oneline->start_sec = oneline->ssec + (oneline->smin * 60) +
		     (oneline->shour * 3600);

strncpy(tmpstr,&linebuf[30],2);
tmpstr[2] = '\0';
for (i = 0; i < 2; i++)
   if (tmpstr[i] == ' ')  tmpstr[i] = '0';
sscanf(tmpstr,"%d",&tmpint);
oneline->end_sec = tmpint;

strncpy(tmpstr,&linebuf[28],2);
tmpstr[2] = '\0';
for (i = 0; i < 2; i++)
   if (tmpstr[i] == ' ')  tmpstr[i] = '0';
sscanf(tmpstr,"%d",&tmpint);
oneline->end_sec += (tmpint * 60);

strncpy(tmpstr,&linebuf[26],2);
tmpstr[2] = '\0';
for (i = 0; i < 2; i++)
   if (tmpstr[i] == ' ')  tmpstr[i] = '0';
sscanf(tmpstr,"%d",&tmpint);
oneline->end_sec += (tmpint * 3600);

/* overlap a day? */
if (oneline->end_sec < oneline->start_sec)
   oneline->end_sec += SEC_PER_DAY;

oneline->nlines = (oneline->end_sec - oneline->start_sec) * LAC_PER_SEC;

}
