/******************************************************************************
FUNCTION:	WRTHALF

PURPOSE:	Write a half orbit of info to the output file if at least
		one of the user specified regions lies at least partially
		within the half orbit.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       02/94    T. Baltzer		initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include "las.h"		/* LAS include			       */
#include "adaps.h"		/* ADAPS include		       */
#include "stitch.h"             /* Orbital stitch include file         */


FUNCTION wrthalf(outfile,orbbuf,nscenes,usrreg,orbflg)

   char outfile[CMLEN];		/* Output file name			*/
   char orbbuf[MAX_PASS][CMLEN];/* Half orbit line buffer		*/
   long nscenes;		/* Number of scenes in orbbuf		*/
   long usrreg[12];		/* Regions as specified by the user	*/
   long orbflg;			/* Flag to print single orbits or not	*/
{

char onebuf[CMLEN];		/* Buffer containing line of data	*/

struct ONELN currline;		/* Structure with info from current line*/

long fatal = -1;		/* Fatal error status			*/
long j;				/* Counter				*/
long status;			/* Return status			*/
long print_flg = FALSE;		/* Print the half orbit?		*/
long regflg[12];		/* Region flag: 0=Not found, 1=Found	*/


/*
  Initialize the region numbers to all false
*/
for (j = 0; j < 12; j++)
   regflg[j] = FALSE;


/*
  Determine the regions within the half orbit's input scenes.
  Don't reset the regions flag array for each input image,
  but rather set the region to TRUE each time it is found,
  for all input scenes within the half orbit.
*/
for (j = 0; j < nscenes; j++)
   {
   initvals(&orbbuf[j][0],&currline);
   getregions(&currline,regflg);
   }


/*
  Determine if any of the user specified regions lie within this half
  orbit of data
*/
for (j = 0; j < 12; j++)
   {
   if (usrreg[j] && regflg[j])
      {
      print_flg = TRUE;
      break;
      }
   }


/*
  This paragraph is an "after the fact" addition.  If the entire list
  of input scenes into one half orbit consists of only one scene, and
  that one input scene is itself a half orbit, we may not want to
  restitch this single half orbit.  Consequently, we may not want to
  write it to the output file.
*/
if ((nscenes == 1) &&
    (strncmp(&orbbuf[0][7],"AO",2) == 0) &&
    (orbflg == FALSE))
    print_flg = FALSE;


/*
  Print the half orbit only if at least one of the user specified regions
  lies at least partially within at least one of the input scenes.  Note
  that the entire half orbit will or will not be written.
*/
if (print_flg)
   {
   /*
     Print a blank line that will seperate the half orbits
   */
   status = wrtbuf(outfile,"\n");
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","subregions-fatal",&fatal);

   /*
     Print the scenes
   */
   for (j = 0; j < nscenes; j++)
      {
      strcpy(onebuf,orbbuf[j]);
      status = wrtbuf(outfile,onebuf);
      if (status != E_SUCC)
         c_errmsg("Fatal error encountered","subregions-fatal",&fatal);
      }

   }

}
