/*******************************************************************************
FUNCTION:	getpar

PURPOSE:	Retrieve the parameters from tae parameter block	
			 
Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  5.0    09/90  D.VanderZee      Converted to C
  5.1    09/93  J.Fenno          Stolen from old SUNSAT.  Complete rewrite.
				

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"



FUNCTION getpar (vblock, hosin, taeout, hosout, nmrimg, window, bands,
		 nbands, totbnd, odtype, npixels, msgtim, minmax)

    struct PARBLK *vblock;	/* pointer to TAE parameter block             */
    char hosin[][CMLEN];	/* host file names of input images            */
    char *taeout;		/* TAE file name of output image              */
    char *hosout;		/* host file name of output image             */
    long *nmrimg;		/* number of input images                     */
    long window[][4];		/* window specifications for input image      */
    long bands[][N_ANGLES];	/* band specifications for output image       */
    long *nbands;		/* number of bands for output image           */
    long *totbnd;		/* total number of bands to be processed      */
    long *odtype;		/* output data type                           */
    long *npixels;		/* interpolated sample interval               */
    long *msgtim;		/* elapsed time between display messages      */
    char *minmax;		/* min max flag buffer                        */
{
char	dtype[5];		/* data type string                           */
char 	taein[CMLEN];           /* TAE file names of input images             */
char    str[CMLEN];		/* Temporary string variable                  */

long element     =  0;		/* parameter element number                   */
long fatal       = -1;		/* fatal error flag                           */
long i;				/* counter                                    */
long imglim      =  1;		/* maximum number of images to be processed   */
long nonfatal    =  1;		/* non-fatal error flag                       */
long qflag[1][2];		/* flag indicating if subwindows or bands     */
long same;			/* flag indicating if images have same window */
long status;			/* Function return status                     */
long tmpbands[MAXBND+1];	/* Temporary bands variable for opening input */

struct VARIABLE	*p_find();	/* function to find a TAE variable            */
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block   */


/* Retrieve and parse input image specification 
-----------------------------------------------*/
status = c_getin(vblock, "IN", &element, taein, hosin[0], nmrimg,
    window, tmpbands, nbands, totbnd, &imglim, &same, qflag);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","sunsat-fatal",&fatal);
if (*nmrimg != 1)
    {
    c_errmsg("Only one input image is allowed","sunsat-nmrimg",&nonfatal);
    c_errmsg("Fatal error encountered","sunsat-fatal",&fatal);
    }


/* Retrieve output image name
-----------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) != E_SUCC)
    c_errmsg("Fatal error encountered","sunsat-fatal",&fatal);


/* Retrieve number of pixels to interpolate between
---------------------------------------------------*/
val = p_find(vblock,"NPIXELS");
*npixels = IVAL(*val,0);


/* Retrieve output data type.  Note that the user must enter a data type.
-------------------------------------------------------------------------*/
val = p_find(vblock,"odtype");
strcpy(dtype,SVAL(*val,0));
if (strcmp(dtype,"BYTE") == 0)
    *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
    *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
    *odtype = ELONG;
else
    *odtype = EREAL;


/* Retrieve band numbers for output image
-----------------------------------------*/
val = p_find(vblock,"BANDS");
for (i = 0; i < val->v_count; i++)
    {
    bands[0][i] = IVAL(*val,i);
    }
*totbnd = *nbands = i;


/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

return(E_SUCC);
}
