/*******************************************************************************
FUNCTION:	init_avhrr

PURPOSE:	Read the addr and the ddr files for the input image.
			 
Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  5.0    05/91  D.VanderZee      Development
  5.1    08/91  T.Baltzer        Rewrite to call c_getaddr()
  5.2    09/93  J.Fenno          Complete rewrite of SUNSAT.
				

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"



FUNCTION init_avhrr (image_name, model, ml, ms)

    char *image_name;	/* IN  - host file name of input image file           */
    struct MODEL *model;/* OUT - model and orbital information                */
    long *ml;		/* OUT - master line                                  */
    long *ms;		/* OUT - master sample                                */
{
long    fatal = -1,		/* fatal error flag */
	status;

struct DDR      ddr;            /* las image ddr */


/* Get the avhrr model information
----------------------------------*/
status = c_getaddr(image_name,model);
if (status != E_SUCC )
    c_errmsg("Fatal error encountered","sunsat-fatal",&fatal);


/* We must ensure that the AVHRR model will convert all pixels into a
   "North-up" attitude so that the correct angles are computed for
   ascending passes.
---------------------------------------------------------------------*/
model->sat_view = FALSE;


/* Open image DDR file and read master line/sample
--------------------------------------------------*/
if (c_getddr(image_name,&ddr) != E_SUCC)
    c_errmsg("Fatal error encountered","sunsat-fatal",&fatal);
*ml = ddr.master_line;
*ms = ddr.master_sample;


return(E_SUCC);
}
