/*******************************************************************************
FUNCTION:	interpolate	

PURPOSE:	Interpolate the calculated angles from two pixels to all of the
		pixels lying between them.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/93  J.Fenno          Rewrite of the original SUNSAT.


PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"



FUNCTION interpolate (angles, ss, es)

    double angles[][N_ANGLES];/* IN/OUT - Angles for every pixel in a line    */
    long  ss;		/* IN - Starting sample to compute angles for         */
    long  es;		/* IN - Ending sample to compute angles for           */
{
double diff[N_ANGLES];	/* Difference between the angle values of ss and es   */
double percent;		/* Percentage of the distance between ss and es       */

long i;			/* Loop control                                       */
long j;			/* Loop control                                       */


/* Compute the difference between each of the angle values at ss and es
-----------------------------------------------------------------------*/
for (i = 0; i < N_ANGLES; i++)
    diff[i] = angles[es][i] - angles[ss][i];


/* Interpolate the angle values for each pixel between ss and es
----------------------------------------------------------------*/
for (i = ss + 1; i < es; i++)
    {
    percent = (double)(i - ss) / (double)(es - ss);
    for (j = 0; j < N_ANGLES; j++)
	angles[i][j] = angles[ss][j] + percent * diff[j];
    }

return(E_SUCC);
}
