/*******************************************************************************
FUNCTION:	scale_angles

PURPOSE:	Scale the output angle values according to the output data type.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.1    ??/??  C.Wivell         Development
  5.0    08/91  T.Baltzer        Converted to LAS5.0
  5.1    09/93  J.Fenno          Complete rewrite of SUNSAT
  5.2    03/94  J.Fenno          Modified so the conversion from radians to
				 degrees happens before the angles are
				 interpolated.  Due to a change in the model the
				 relative azimuth angles are now in the range
				 0-360.  Changed so they remain 0-180.


PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"



FUNCTION scale_angles (angles, ss, es, odtype)

    double angles[][N_ANGLES];	/* IN/OUT - Angle values to be scaled         */
    long ss;			/* IN - Starting sample to scale angles for   */
    long es;			/* IN - Ending sample to scale angles for     */
    long odtype;		/* IN - Output data type                      */
{
register long i;		/* Loop control                               */
register long j;		/* Loop control                               */

switch (odtype)
    {
    case EBYTE:
	for (i = ss; i <= es; i++)
	    angles[i][SATZEN] +=  90.0;
	break;

    case EWORD:
    case ELONG:
	for (i = ss; i <= es; i++)
	    for (j = 0; j < N_ANGLES; j++)
		angles[i][j] *=  10.0;
	break;
    }

return(E_SUCC);
}
