/******************************************************************************

FUNCTION:	FINISH

PURPOSE:	To close the tape and image files

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
 1.0    04/98   T. Baltzer	Original development (borrowed from MKLEVEL1B)


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	May be run under TAE or unix

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "level1bio.h"
#include "tapel1b.h"

void finish
(
struct	PARBLK	*vblock,	/* parameter block			*/
char	*drive,			/* drive name				*/
long	*nvols,			/* number of volumes used		*/
long	tplun,			/* tape unit number			*/
FILE	*in_fd,			/* input image file descriptor		*/
long	*kbyteswritten,		/* number of bytes written to tape	*/
char	sceneid[],		/* Scene id				*/
char	lasout[],		/* LAS output name of summary file	*/
long	tae			/* flag: called from tae land		*/
)
{
char	*tmpstring[1];		/* Drive name for updating vblock	*/

long	status;			/* function return status		*/
long	tape = 1;		/* tape open flag			*/


/*
  Close image file
-------------------*/
if ( in_fd != NULL )
    {
    status = c_l1bclosd( in_fd );
    if ( status != E_SUCC )
        c_lasmsg(FALSE,"tapel1b-imgcler","Error closing Level-1b image");
    }


/*
  Close output tape
-------------------------------------------------------------------*/
if ( tplun )
    status = c_l1bclost( &tplun, &tape );


/*
  Update the variable block only if called from tae
-----------------------------------------------------*/
if ( tae )
    {
    /*
      Update the variable block drive name in case it has changed
    --------------------------------------------------------------*/
    tmpstring[0] = drive;
    status = q_string( vblock, "DRIVE", 1, (const char **)tmpstring, P_UPDATE );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"tapel1b-vbupder","Error updating variable block");

    status = q_intg( vblock, "NVOLS", 1, (int *)nvols, P_UPDATE );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"tapel1b-vbupder","Error updating variable block");


    /* Send the SCENEID back to tae land
    --------------------------------------*/
    tmpstring[0] = sceneid;
    status = q_string(vblock,"SCENEID",1,(const char **)tmpstring,P_UPDATE);
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"tapel1b-vbupder","Error updating variable block");


    /* Send the summary file name back to tae land
    ------------------------------------------------*/
    tmpstring[0] = lasout;
    status = q_string(vblock,"SUMFILE",1,(const char **)tmpstring,P_UPDATE);
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"tapel1b-vbupder","Error updating variable block");


    /*
      Update number of bytes written to tape
    -----------------------------------------*/
    status = q_intg( vblock, "NBYTES", 1, (int *)kbyteswritten, P_UPDATE );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"tapel1b-vbupder","Error updating variable block");

    status = q_out( vblock );
    if ( status != SUCCESS )
	c_lasmsg(FALSE,"tapel1b-vbupder","Error updating variable block");
    }

}
