/*****************************************************************************
FUNCTION:	GETARGV

PURPOSE:	Retrieve parameters from the unix command line via argv[].

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    03/98	T. Baltzer	Initial development (stolen from MKLEVEL1B)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "notae.h"
#include "getargv.h"
#include "tapel1b.h"



void getargv
(
long	argc,			/* count of arguments in argv[]		*/
char	*argv[],		/* array of arguments			*/
char	*comment,		/* tape mount comment(s)		*/
char	*infile,		/* input file name			*/
char	*acctnum,		/* account number			*/
char	*ordernum,		/* order number				*/
char	*unitnum,		/* unit number				*/
char	*prodesc,		/* product description			*/
char	*drive,			/* drive tape is mounted on		*/
char	*ttype,			/* tape type				*/
long	*nvols,			/* volume number in this set		*/
long	*mvolflg,		/* flag: TRUE allows multiple volumes	*/
char	*density,		/* tape density				*/
long	*maxbytes		/* max allowable kbytes for tape write	*/
)
{
char	list[20][CMLEN];	/* list of input parsed parameters	*/
char	tempstring[CMLEN];	/* temporary parameter value string	*/
char	hosdir[CMLEN];		/* host directory name			*/
char	hosext[CMLEN];		/* host extension name			*/
char	hosroot[CMLEN];		/* host root name			*/

long	numlist;		/* number of parameters in list		*/
long	temp;			/* temporary integer value		*/
long	temparray[1];		/* Temporary array of longs		*/

/*
  Set up the parameter structure
----------------------------------*/
char *scmdlist[] = { NULL };
char *params[] = { "comment", "infile", "acctnum", "ordernum", "unitnum",
	"drive", "ttype", "nvols", "mvolflg", "tapedens",
	"prodesc", "maxbytes", NULL };
enum pnames { COMMENT, INFILE, ACCTNUM, ORDERNUM, UNITNUM, DRIVE, TTYPE,
	NVOLS, MVOLFLG, TAPEDENS, PRODESC, MAXBYTES };
long count[] = { 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
enum PTYPE ptype[] = { STRING, STRING, STRING, STRING, STRING, STRING,
	 STRING, INTEGER, STRING, STRING, STRING, INTEGER };
struct PARAMETER parms;
parms.plist = params;
parms.pcount = count;
parms.ptype = ptype;

/*
  Parse argv and count the returned parameters
-----------------------------------------------*/
c_parsepar( argc, argv, &parms, scmdlist, list );
numlist = 0;
while ( list[numlist++][0] != '\0' );
numlist--;

/*
  Retrieve the input file.
----------------------------*/
if ( c_getsval( "infile", list, tempstring ) != E_SUCC )
    c_lasmsg(TRUE,"tapel1b-parm",
	"Required parameter not specified: INFILE");
else
    {
    c_cmhost( tempstring, infile );
    c_splitname( infile, hosdir, hosroot, hosext );
    if ( hosext[0] == 0 )
	strcat( infile, ".l1b" );
    }

/*
  Retrieve the tape mount comment
------------------------------------------------------*/
tempstring[0] = comment[0] = 0;
hosdir[0] = hosroot[0] = hosext[0] = 0;
c_getsval( "comment", list, comment );

/*
  get tape information if mount comment is specified
-----------------------------------------------------*/
if ( comment[0] != 0 )
    {
    /* get the drive name */
    if ( c_getsval( "drive", list, drive ) != E_SUCC )
	c_lasmsg(TRUE,"tapel1b-parm",
	    "Required parameter not specified: DRIVE");

    /* get the tape type, if not specified default is 3490 */
    if ( c_getsval( "ttype", list, ttype ) != E_SUCC )
	strcpy( ttype, "3490" );
    else if ( strncasecmp( ttype, "8hi", 3 ) != 0 &&
    	      strncasecmp( ttype, "8lo", 3 ) != 0 &&
	      strncasecmp( ttype, "3480", 4 ) != 0 &&
	      strncasecmp( ttype, "3490", 4 ) != 0 &&
	      strncasecmp( ttype, "4mm", 3 ) != 0 &&
	      strncasecmp( ttype, "dlt4000", 7 ) != 0 &&
	      strncasecmp( ttype, "dlt7000", 7 ) != 0 &&
	      strncasecmp( ttype, "d3", 2 ) != 0)
        c_lasmsg(TRUE,"tapel1b-ttype",
	    "Invalid tape type specified");
	

    /* get the density, if not specified default is 0 */
    if ( c_getsval( "tapedens", list, density ) != E_SUCC )
	{
	if ( strncasecmp ( ttype, "8hi", 3 ) == 0 )
	    strcpy( density, "HI" );
	else if ( strncasecmp ( ttype, "8lo", 3 ) == 0 )
	    strcpy( density, "LO" );
	else
	    strcpy( density, "0" );
	}

    /* get the multiple volume flag, if not specified default is TRUE */
    if ( c_getsval( "mvolflg", list, tempstring ) != E_SUCC )
	*mvolflg = TRUE;
    else
	{
	if ( strncasecmp( tempstring, "NO", 2 ) == 0 )
	    *mvolflg = FALSE;
	else
	    *mvolflg = TRUE;
	}

    /* get volume number, if not specified default is 1 */
    if ( c_getsval( "nvols", list, tempstring ) != E_SUCC )
	*nvols = 1;
    else
	{
	c_getival( tempstring, parms.pcount[NVOLS], &temp, temparray );
	*nvols = temparray[0];
	}

    /* get max allowable bytes for tape */
    if ( c_getsval( "maxbytes", list, tempstring ) == E_SUCC )
	{
	c_getival( tempstring, 1, &temp, temparray );
	*maxbytes = temparray[0] * 900;
	}
    else
	{
	/* not specified, set based on tape type */
	if ( strncasecmp( ttype, "3480", 4 ) == 0 )
	    *maxbytes = 270000;
	if ( strncasecmp( ttype, "3490", 4 ) == 0 )
	    *maxbytes = 700000;
	if ( strncasecmp( ttype, "4mm", 4 ) == 0 )
	    *maxbytes = 1200000;
	if ( strncasecmp( ttype, "8lo", 4 ) == 0 )
	    *maxbytes = 2250000;
	if ( strncasecmp( ttype, "8hi", 4 ) == 0 )
	    *maxbytes = 4500000;
	if ( strncasecmp( ttype, "dlt4000", 7 ) == 0 )
	    *maxbytes = 18000000;
	if ( strncasecmp( ttype, "dlt7000", 7 ) == 0 )
	    *maxbytes = 31500000;
	if ( strncasecmp( ttype, "d3", 2 ) == 0 )
	    *maxbytes = 45000000;
	}

    }


/*
  Get account, order and unit numbers
--------------------------------------*/
if ( c_getsval( "acctnum", list, acctnum ) != E_SUCC )
    c_lasmsg(TRUE,"tapel1b-parm",
	"Required parameter not specified: ACCTNUM");
if ( c_getsval( "ordernum", list, ordernum ) != E_SUCC )
    c_lasmsg(TRUE,"tapel1b-parm",
	"Required parameter not specified: ORDERNUM");
if ( c_getsval( "unitnum", list, unitnum ) != E_SUCC )
    c_lasmsg(TRUE,"tapel1b-parm",
	"Required parameter not specified: UNITNUM");


/*
  Get product description
--------------------------*/
if ( c_getsval( "prodesc", list, prodesc ) != E_SUCC )
    strcpy ( prodesc, "LEVEL1B" );

}
