/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the parameters specified by the user.

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
 1.0	04/98	T. Baltzer	Original development (borrowed from MKLEVEL1B)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "adaps.h"
#include "tapel1b.h"


void getpar
(
struct	PARBLK	*vblock,	/* input parameter block		*/
char    *infile,		/* input image host file name		*/
char	comment[CMLEN],		/* tape mount comment			*/
char	acctnum[10],		/* account number			*/
char	ordernum[14],		/* order number				*/
char	unitnum[5],		/* unit number				*/
char	prodesc[18],		/* product description			*/
char	*drive,			/* drive name				*/
long	*mvol,			/* multiple tape volume flag		*/
long	*nvols,			/* number of volumes used		*/
char	*density,		/* tape density				*/
char	*ttype,			/* tape type				*/
long	*msgtim,		/* time between processing messages	*/
long	*max_bytes,		/* maximum bytes allowed for tape	*/
long	*kbyteswritten		/* number of bytes written to tape	*/
)
{
long	access;			/* file access				*/
long	status;			/* status returned from support calls	*/
long	zero = 0;		/* value of zero			*/

struct	VARIABLE *var;		/* pointer to a TAE variable		*/


/*
  Get the tape mount comment
*/
var = p_find( vblock, "COMMENT" );
if ( var->v_count != 0 )
    {
    strcpy( comment, SVAL( *var, 0 ));
    }


/*
  Get input image
*/
access = IREAD;
status = c_getnam( vblock, "INFILE", &zero, infile, "l1b", &access );
if ( status != E_SUCC )
    c_lasmsg(TRUE,"tapel1b-fatal","Fatal error encountered");


/*
  Get the account number
*/
acctnum[0] = 0;
var = p_find( vblock, "ACCTNUM" );
if ( var->v_count != 0 )
    strcpy( acctnum, SVAL( *var, 0 ));

/*
  Get the order number
*/
ordernum[0] = 0;
var = p_find( vblock, "ORDERNUM" );
if ( var->v_count != 0 )
    strcpy( ordernum, SVAL( *var, 0 ));

/*
  Get the unit number
*/
unitnum[0] = 0;
var = p_find( vblock, "UNITNUM" );
if ( var->v_count != 0 )
    strcpy( unitnum, SVAL( *var, 0 ));


/*
  Get the product description
*/
var = p_find( vblock, "PRODESC" );
strcpy( prodesc, SVAL( *var, 0 ));


/*
  Get the multi-volume flag
*/
var = p_find( vblock, "MVOLFLG" );
if ( strcmp( SVAL( *var, 0 ), "YES" ) == 0 )
    *mvol = TRUE;
else
    *mvol = FALSE;


/*
  Get the number of volumes used
*/
var = p_find( vblock, "NVOLS" );
*nvols = IVAL( *var, 0 );

/*
  Get the tape drive name
*/
var = p_find( vblock, "DRIVE" );
strcpy( drive, SVAL( *var, 0 ));

/*
  Get the tape density
*/
var = p_find( vblock, "TAPEDENS" );
strcpy( density, SVAL( *var, 0 ));

/*
  Get the tape type
*/
var = p_find( vblock, "TTYPE" );
strcpy( ttype, SVAL( *var, 0 ));

/*
  Determine the desired density
*/
if ( !strcasecmp( ttype, "8hi" ))
    strcpy( density, "HI" );
else if ( !strcasecmp( ttype, "8lo" ))
    strcpy( density, "LO" );
else
    strcpy( density, "0" );

/*
  Get the maximum number of bytes allowed
*/
var = p_find( vblock, "MAXBYTES" );
*max_bytes = IVAL( *var, 0 );

/*
  Get the number of bytes already written to tape
*/
var = p_find( vblock, "NBYTES" );
*kbyteswritten = IVAL( *var, 0 );


/*
  Get the time between processing messages
*/
var = p_find( vblock,"$MSGTIM" );
*msgtim = IVAL( *var, 0 );
}
