/***************************************************************************
FUNCTION:	TOV_INGEST

PURPOSE:	To create a .img file from an ASCII TOVS data file.
		The header lines are removed, if present,  and all negative,
		non-observed, values are replaced by -1.0.  The image created
		is a single band BSQ transfer image.  Each image contains
		180 lines by 360 samples of values for a single days
		observation, either AM or PM depending on input file.

Version	Date	Programmer	Modification
-------	-----	------------	--------------------------------
  1.0	02/99	D.Lloyd		Initial Development

PROJECT:	ADAPS


***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tovsin.h"


#define NLINES          180
#define NSAMPS          360

void main( void )
{
char	infile[CMLEN];		/* Input file name			*/
char	outfile[CMLEN];		/* Output file name			*/
char	str[301];		/* temporary string for removing header	*/

FILE	*inf;			/* input file descriptor		*/
FILE	*ouf;			/* output file descriptor		*/

float	buf[NSAMPS];		/* I/O buffer				*/
float	tempfloat;		/* temp float value for scanf()		*/

int	c;			/* return value from fgetc()		*/

long	i;			/* loop counter				*/
long	j;			/* loop counter				*/
long	outcnt = 1;		/* count of output files		*/
long	status;			/* Function return status		*/
long	stdlun;			/* Standard output			*/

struct	PARBLK	vblock;		/* Parameter block			*/


/* Initialize LAS
-----------------*/
c_inlas( &vblock, &stdlun );


/* Get the parameters
---------------------*/
status = getpar( &vblock, infile, outfile );
if ( status != E_SUCC )
    c_lasmsg( TRUE, "tovsin-fatal", "Fatal error occured" );


/* open the files
-----------------*/
inf = fopen( infile, "r" );
if ( !inf )
    c_lasmsg( TRUE, "tovsin-open", "Unable to open: %s", infile );
ouf = fopen( outfile, "w" );
if ( !ouf )
    c_lasmsg( TRUE, "tovsin-open", "Unable to open: %s", outfile );

/* Skip 5 header lines if present.  Header lines are
indicated by a ';' at the beginning of the line
----------------------------------------------------*/
for ( i = 0; i < 5; i++ )
    {
    /* Get the first character of the line
    --------------------------------------*/
    c = fgetc( inf );

    /* If its a header line eat the line
    otherwise break out of the loop.
    ------------------------------------*/
    if ( c == ';' )
	fgets( str, 300, inf );
    else
        break;
    }
/* Note: the floating point values in the input file are stored
as %f14.4 and no value is greater than 10.0 so don't bother
putting the first charater of the first data line back.
---------------------------------------------------------------*/

/* Read the input file, convert all negatives to -1
and write to output file in binary.
---------------------------------------------------*/
for ( j = 0; j < NLINES; j++ )
    {
    /* read a line of data
    ----------------------*/
    for ( i = 0; i < NSAMPS; i++ )
	{
	fscanf( inf, "%f", &tempfloat );
	if ( tempfloat < 0 )
	    buf[i] = -1.0;
	else
	    buf[i] = tempfloat;
	}

    /* write the new line
    ---------------------*/
    fwrite( &buf[1], sizeof(float), NSAMPS, ouf );
    }

/* Close files
--------------*/
fclose( inf );
fclose( ouf );

/* Display a standard completion message
----------------------------------------*/
c_complt( "TOVSIN", outfile, &outcnt );
} 
