/****************************************************************************

FUNCTION:	GETPAR

PURPOSE:	Retrieve input parameters.

ALGORITHM REFERENCES

	None.

****************************************************************************/

#include <ctype.h>
#include "las.h"

void getpar
(
    struct PARBLK *vblock,	/* TAE variable block			*/
    char *tapeid,		/* Tape ID				*/
    int *image,			/* Images to extract			*/
    int *numimg,		/* Number of images to extract		*/
    char *dstdir		/* Destination directory 		*/
)
{
    struct VARIABLE *val;	/* Pointer to a TAE variable		*/
    char subcmd[7];		/* Subcommand (RANGE or SELECT)		*/
    long start, stop;		/* Starting/stopping sets for RANGE	*/
    int i;			/* Loop counter				*/
    int j;			/* Temporary holder variable		*/

    /* Retrieve tape ID provided by user, and convert to all-lowercase
       to provide conformity with TPACQUIRE standards.
       ---------------------------------------------------------------*/
    val = p_find(vblock, "TAPEID");
    strcpy(tapeid, SVAL(*val, 0));
    for (i = 0; i < strlen(tapeid); ++i)
	if (isupper(tapeid[i])) tapeid[i] = tolower(tapeid[i]);

    /* Retrieve the application subcommand.
       ------------------------------------*/
    val = p_find(vblock, "_SUBCMD");
    strcpy(subcmd, SVAL(*val, 0));

    /* Determine which images we need to process from the tape, based on
       the subcommand given to the application.
       -----------------------------------------------------------------*/
    if (!strcmp(subcmd, "RANGE"))
	{

	/* Retrieve the starting and stopping savesets, and fill in 
	   the image array with all of the savesets in between.  Note
	   the number of images to read from tape (giving a value of
	   zero if the entire tape is to be read.)
	   ----------------------------------------------------------*/
	val = p_find(vblock, "START");
	start = IVAL(*val, 0);
	val = p_find(vblock, "STOP");
	stop = (val->v_count == 0) ? 0 : IVAL(*val, 0);
	if (stop == 0)
	    *numimg = 0;
	else
	    *numimg = (stop - start) + 1;
	j = start - 1;
	for (i = 0; i < stop - start + 1; ++i)
	    image[i] = j++;
	}
    else
	{

	/* Extract all of the user-specified image numbers.  Subtract one
	   from the actual given location to account for zero-based arrays.
	   ---------------------------------------------------------------*/
	val = p_find(vblock, "IMAGENUM");
	*numimg = val->v_count;
	for (i = 0; i < val->v_count; ++i)
		image[i] = IVAL(*val, i) - 1;
	}

    /* Retrieve the destination directory (ADAPSPROD or ADAPSACQ).
       -----------------------------------------------------------*/
    val = p_find(vblock, "DSTDIR");
    strcpy(dstdir, SVAL(*val, 0));

    return;
}
