/*****************************************************************************

FUNCTION:       PROCESS_FILE

PURPOSE:        This function processes the data found in a VMS BACKUP 
		record of type REC_FILE.  This record contains information
		about the forthcoming file, including the filename, the 
		record type/length, etc.

ALGORITHM REFERENCES

        Source code for "vmsbackup" by John D. Carey and Sven-Ove Westberg

*****************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "readcpa.h"

int process_file
(
    unsigned char *ptr		/* Pointer into block buffer		*/
)
{
    int i, j = 0;		/* Loop counters			*/
    int flag = 0;		/* Used to signal start of filename	*/
    int16 check;		/* Verifies validity of this record	*/
    int16 namelen;		/* Length of the filename		*/
    char filename[255];		/* Name of the file within saveset	*/
    char *tmp;			/* Temporary pointer for processing	*/

    /* The first two bytes of a REC_FILE record must be "01 01", in other
       words, a 16-bit value of 257.  If not, the record is corrupted.
       -----------------------------------------------------------------*/ 
    check = to_int16(ptr, 0);
    if (check != 257)
	{
	close_tape();
	c_lasmsg(TRUE, "readcpa-filerec", "Invalid file record");
	}

    /* Extract the name of the forthcoming file.  Ignore the VMS directory
       structure (everything up to the right bracket), strip off the
       semicolon extension, and convert to lowercase if needed.
       -------------------------------------------------------------------*/
    namelen = to_int16(ptr, 2);
    for (i = 0; i < namelen; ++i)
	{
	if (ptr[6 + i] == ']') 
	    flag = 1;
	else if (flag)
	    {
	    filename[j] = (ptr[6 + i] == ';' ? '\0' : ptr[6 + i]);
	    if (isupper(filename[j])) 
		filename[j] = tolower(filename[j]); 
	    ++j;
	    }
	}

    /* Return an integer describing the nature of this file.  These 
       correspond to:

          -1: File is of no interest to us
           0: File is the image auxiliary data file
       1 - 5: File is image data for band <x>.

       This information is determined by looking at the extension.
       ------------------------------------------------------------*/
    tmp = strchr(filename, '.');
    if (!tmp)
	return -1;
    tmp++;

    if (!strcmp(tmp, "aux"))
	return (0);
    else if (!strncmp(tmp, "ch", 2))
	return (tmp[2] - '0');
    else return (-1);
}
