/******************************************************************************

FUNCTION:	READ_HEADER

PURPOSE:	Reads the ANSI header record for a file on a VMS backup
	 	tape.  From this record, obtain the saveset number and the
		block size.

ALGORITHM REFERENCES:	"vmsbackup" source, by Carey and Westberg

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "readcpa.h"
#include "tapeio.h"

int read_header
(
		long unit,	/* Tape device to read from		*/
		long *blocksize, /* Size of blocks within saveset	*/
		int *setnum	/* Saveset number on tape		*/
)
{
    char label[LABEL_SIZE];	/* Buffer for reading label records	*/
    long status;		/* Status condition for system calls	*/
    long mode = HANDLE;		/* Allow tape I/O to handle errors	*/
    long to_read = LABEL_SIZE;	/* Bytes to read from tape		*/

    *blocksize = -1;

    /* Read 80-byte header records one at a time until there are none
       remaining.  Extract the saveset number from the HDR1 line, and
       the block size from the HDR2 line.
       --------------------------------------------------------------*/
    while ((status = c_tpread(&unit, (unsigned char *)label, &to_read, &mode)) != TAPE_EOF)
	{
	if ((status == TAPE_EOT) || (status == TAPE_EOV))
	    return -1;

	if (status != LABEL_SIZE)
	    {
	    close_tape();
	    c_lasmsg(TRUE, "readcpa-label", "Malformed label record");
	    }

	else if (!strncmp(label, "HDR1", 4)) 
	    sscanf(label + 31, "%4d", setnum);

	else if (!strncmp(label, "HDR2", 4))
	    sscanf(label + 5, "%5d", blocksize);
        }

    return 0;
}
