/******************************************************************************

FUNCTION:	READ_HEADER

PURPOSE:	Reads the ANSI trailer record, checks for bad label records,
		and finds the "EOF1" label.

ALGORITHM REFERENCES:	"vmsbackup" source, by Carey and Westberg

******************************************************************************/

#include <string.h>
#include "las.h"
#include "readcpa.h"
#include "tapeio.h"

int read_trailer
(
		long unit,	/* File descriptor of input tape device	*/
		int setnum	/* Number of current saveset		*/
)
{
    char label[LABEL_SIZE];	/* Buffer for reading label records	*/
    long status;		/* Status condition for system calls	*/
    long mode = HANDLE;		/* Allow tape I/O to handle errors	*/
    long to_read = LABEL_SIZE;	/* Number of bytes to read from tape	*/

    /* Read 80-byte header records one at a time until there are none
       remaining.  We should eventually encounter EOF1, which will
       "officially" signify the end of the saveset.
       --------------------------------------------------------------*/
    while ((status = c_tpread(&unit, (unsigned char *)label, &to_read, &mode)) != TAPE_EOF)
	{
	if (status != LABEL_SIZE)
	    {
	    close_tape();
            c_lasmsg(TRUE, "readcpa-label", "Malformed label record");
	    }

	else if (!strncmp(label, "EOF1", 4))
	    printf("[informational] End of saveset %d\n\n", setnum);
	}

    return 0;
}
