/****************************************************************************

FUNCTIONS:	TAPE

PURPOSE:	The functions handle the mounting and unmounting of 
		magnetic tapes from which the acquisition images are
		to be read.


ALGORITHM REFERENCES
	None.

****************************************************************************/
#include "las.h"
#include "tapeio.h"


/* Keep a static copy of the unit number returned by c_tpopen().  This
   allows us to call close_tape() from anywhere in the application
   without having to pass the unit number from function to function.
   -------------------------------------------------------------------*/
static long tapeunit = 0;

int mount_tape
(
    char *tapeid,	/* ID of tape to be mounted			*/
    long *unit		/* Tape device number returned by c_tpopen	*/
)
{
    long skip = 0;	/* Number of files on tape to skip		*/
    long mode = HANDLE;	/* Flag for tape I/O functions to handle errors	*/
    long rew = REWIND;	/* Flag for c_tpopen() to rewind the tape	*/
    char dens[5];	/* Density of the tape being read		*/
    long inout = TREAD;	/* Flag to open the tape in read mode		*/
    char tape[CMLEN];	/* Identification of the mounted tape		*/
    long status;	/* Status code returned by library functions	*/
    char message[CMLEN];/* Mount request sent to operators		*/

    sprintf(dens, "0");
    sprintf(message, "Please mount %s", tapeid);

    /* Mount the current tape volume.
       ------------------------------ */
    status = c_tpopen(unit, &skip, &mode, &rew, dens, &inout,
		      message, tape);
    if (status != TAPE_SUCC)
	return(E_FAIL);
    else 
	{
	tapeunit = *unit;
	return(E_SUCC);
	}
}

int close_tape(void)
{
    long mode = NO_HANDLE; /* Display errors, but take no other action  */
    long rew = REWOFFL; /* Flag for tape to be rewound/taken offline	*/
    long eofopt = 0;	/* Write zero EOFs to end of tape 		*/
    long status;	/* Status code returned by library functions	*/

    status = c_tpclos(&tapeunit, &mode, &rew, &eofopt);
    if (status != TAPE_SUCC)
	c_lasmsg(FALSE, "readcpa-nonfatal", "Nonfatal error encountered");

    return (0);
}
