/*****************************************************************************
FUNCTION:       check_sum

PURPOSE:        Determines if the TAR record contains a valid checksum.
		If it does then the record if a TAR header record.

Version  Date   Programmer      Modifications
-------  -----  --------------- ---------------------------------------------
  1.0    06/97  R.Dilley        Initial development
  2.0    05/98  D.Lloyd         Modifications for LAS coding standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

ALGORITHM: This algorithm was derived from the GNU tar source.

NOTE:
The `chksum' field is the ASCII representation of the octal
value of the simple sum of all bytes in the header record.  Each
8-bit byte in the header is added to an unsigned integer,
initialized to zero, the precision of which shall be no less than
seventeen bits.  When calculating the checksum, the `chksum'
field is treated as if it were all blanks.

PROJECT:        ADAPS
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tarex.h"


enum stats check_sum
(
union record *rcd
)
{
char	*ptr;		/* pointer into header record		*/
long	i;		/* loop counter				*/
long	sum;		/* simple sum of header records		*/
long	signed_sum;	/* a signed version of sum		*/
long	rec_sum;	/* checksum value stored in header	*/


/*
  Get the checksum value recorded in the header
  and point to the first character in the header.
-------------------------------------------------*/
sscanf(rcd->header.check_sum, "%o", &rec_sum);
ptr = rcd->charptr;

/*
  Initialize the sums
----------------------*/
sum = signed_sum = 0;

/*
  Calculate a simple signed and unsigned sum of
  all the bytes in the header.
------------------------------------------------*/
for(i = sizeof(*rcd); --i >= 0;)
    {
    sum += 0xFF & *ptr;
    signed_sum += *ptr++;
    }

/*
  Subtract out the checksum value since it wasn't
  used in the initial summation.
-------------------------------------------------*/
for(i = sizeof(rcd->header.check_sum); --i >= 0;)
    {
    sum -= 0xFF & rcd->header.check_sum[i];
    signed_sum -= (char) rcd->header.check_sum[i];
    }

/*
  Now add in the spaces that _were_ used in the
  initial calculation of the checksum.
------------------------------------------------*/
i = ' ' * sizeof(rcd->header.check_sum);
sum += i;
signed_sum += i;

/*
  If these sums match the sum stored in the header
  we have a valid header.
--------------------------------------------------*/
if(sum == rec_sum && signed_sum == rec_sum)
    return(HEADER_SUCCESS);

/*
  Otherwise it's BAD
---------------------*/
return(HEADER_FAILURE);
}
