/*************************************************************************
FUNCTION:	convert_cpa

PURPOSE:	Convert a 'raw' CPA file to a format compatible with
		the INGEST routines ADAPSTABLES/station.ids file CP3
		format.

INPUT:		input file name

OUTPUT:		converted file (same file name as original)

RETURNS:	0 sucess
		-1 failure

PROJECT:	ADAPS

$Log: convert_cp3.c,v $
Revision 1.2  2002/06/06 16:22:13  dlloyd
Merging development branch.  CR 6374

Revision 1.1.2.1  2002/06/06 15:40:37  dlloyd
Modfied to use tarex for CPA tapes and to convert the raw format to one that
ingest will recognize using CP3 station format.  CR 6374

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adapsutil.h"
#include "tarex.h"

#define  CPARECSIZE	22016
#define  CP3RECSIZE	22180
#define  CP3TRAILER	154
#define  FS_SIZE	10

int convert_cp3( char *infilename )
    {
    unsigned char input_buffer[CPARECSIZE];
    unsigned char output_buffer[CP3RECSIZE];
    unsigned char allzero[CP3TRAILER] = {'\0'};
    unsigned char frame_sync[FS_SIZE] = { 0x84, 0x02, 0x6f, 0x01, 0x5c,
    					  0x03, 0x9d, 0x01, 0x0f, 0x02, };
    
    char outputfilename[CMLEN];	/* output file name			*/
    char log[CMLEN];		/* log file name			*/
    char key[16];		/* log file key				*/
    char dir[CMLEN];		/* file directory path			*/
    char root[CMLEN];		/* file root name			*/
    char ext[CMLEN];		/* file extension			*/
    char *adapslog;		/* ADAPSLOG environment variable	*/

    FILE *infile;	/* input file descriptor			*/
    FILE *outfile;	/* output file descriptor			*/

    int num_read;	/* Number of bytes read from input file		*/
    int num_write;	/* number of bytes written to output file	*/


    /* Get the log file name
    ------------------------*/
    adapslog = getenv( "ADAPSLOG" );
    if ( !adapslog )
        c_lasmsg(TRUE,"convcpa-env","Unable to retrieve ADAPSLOG");
    sprintf( log, "%s/tpacquire.log", adapslog );
    c_splitname( infilename, dir, root, ext );
    if ( strlen( root ) > 15 )
        root[15] = '\0';
    strcpy( key, root );
    

    /* Open the files
    -----------------*/
    sprintf( outputfilename, "%sZTemp", infilename );
    if (( infile = fopen( infilename, "r" )) == NULL )
        {
	c_lasmsg(TRUE,"convcpa-open","Unable to open input file: %s",
	    infilename);
	return -1;
	}

    if (( outfile = fopen( outputfilename, "w" )) == NULL )
        {
	c_lasmsg(TRUE,"convcpa-open","Unable to open output file: %s",
	    outputfilename);
	fclose( infile );
	return -1;
	}
    

    /* Process the file
    -------------------*/
    num_read = fread( input_buffer, sizeof(unsigned char), CPARECSIZE, infile );
    while( num_read == CPARECSIZE )
        {
	memcpy( &output_buffer[0], frame_sync, FS_SIZE );
	memcpy( &output_buffer[FS_SIZE], input_buffer, CPARECSIZE );
	memcpy( &output_buffer[FS_SIZE+CPARECSIZE], allzero, CP3TRAILER );
	num_write = fwrite( output_buffer, sizeof(unsigned char), CP3RECSIZE,
	    outfile );
	if ( num_write != CP3RECSIZE )
	    {
	    c_lasmsg(TRUE,"convcpa-write",
	        "Error writing to file, wrote: %d or %d bytes",
	        num_write, CP3RECSIZE );
	    fclose( infile );
	    fclose( outfile );
	    return -1;
	    }
	num_read = fread( input_buffer, sizeof(unsigned char), CPARECSIZE,
	    infile );
	}  /* END while */
    

    /* Remove the input file
    ------------------------*/
    if ( remove( infilename ))
        c_lasmsg(TRUE,"convcpa-rm","Unable to remove file: %s", infilename );
    if ( rename( outputfilename, infilename ))
        c_lasmsg(TRUE,"convcpa-ren","Unable to rename output file: %s",
	    outputfilename );
    
    fclose( infile );
    fclose( outfile );

    c_lasmsg(FALSE,"convcpa-cmplt","convert_cpa completed successfully");
    c_wrtlog( log, key, "Complete","File successfully converted");
    return 0;
    }  /* END convert_cpa */

