/*****************************************************************************
FUNCTION:	FILL_TAR_BLK

PURPOSE:	Read from tape and fill a buffer with enough data for a 
		complete TAR block.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    06/97	R.Dilley	Initial development
  2.0	 05/98	D.Lloyd		Modifications for LAS coding standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "tarex.h"



int fill_tar_blk
(
long	*unit,			/* IN: tape logical unit number		*/
char	*tape_buffer,		/* IN: pointer to tape buffer		*/
char	*tar_buffer,		/* OUT: pointer to tar buffer		*/
long	tape_blksize		/* IN: tape block size			*/
)

{
static	long cur_pos = 0;	/* current position in tape buffer	*/
long	i;			/* loop control variable		*/
long	status;			/* functio return status		*/
long	mode = HANDLE;		/* handle tape errors			*/


/* Read tape_blksize blocks from the tape and place
   in tape_buffer.  Exit (while) if we have enough
   for a tar_block or some other status.  c_tpread
   returns the size read from the tape or a status.
---------------------------------------------------*/
status = cur_pos;  /* To catch tape_blks larger than tar_blks */
while( cur_pos < DEFAULT_TAR_BLK )	/* was <=	*/
    {
    status = c_tpread( unit, (unsigned char *)&tape_buffer[cur_pos], &tape_blksize, &mode );

    /*
      Let the user know where tape file marks are.  This will
      help with extracting files during a subsequent session
      when errors occurred.
    ---------------------------------------------------------*/
    if ( status == TAPE_EOF )
        {
	c_lasmsg(FALSE," ","TAPE_EOF");
	cur_pos = 0;
	}
    else if ( status == TAPE_EOV )
        {
	c_lasmsg(FALSE," ","TAPE_EOV");
        cur_pos = 0;
	}
    else if ( status == TAPE_EOT )
        {
	c_lasmsg(FALSE," ","TAPE_EOT");
	break;
	}
    else
	cur_pos += status; /* status contains the amount read(tape_blksize) */
    }

/* If status is > 0 then there were no errors 
   and we have a complete tar_block.  Else we
   have to check the status for how to process
----------------------------------------------*/
if( status > 0 )
    {
    memcpy( tar_buffer, tape_buffer, DEFAULT_TAR_BLK );
    cur_pos -= DEFAULT_TAR_BLK;
    if( cur_pos > 0 )
	memcpy( tape_buffer, &tape_buffer[DEFAULT_TAR_BLK], cur_pos );
    }
else
    {
    /* clear out the tar buffer so that a valid header can't be found */
    for ( i = 0; i < DEFAULT_TAR_BLK; i++ )
        tar_buffer[i] = 0;
    switch( status )
	{
	case TAPE_EOF:
	case TAPE_EOV:
	case TAPE_EOT:
	    break;
	default:      /* all other conditions */
	    status = TPREAD_READ;
	    break;
	}
    }

return status;
}

