/*****************************************************************************
FUNCTION:       GETPAR

PURPOSE:        Retrieve parameters from TAE variable block.

Version  Date   Programmer      Modifications
-------  -----  --------------- ---------------------------------------------
  1.0    06/97  R.Dilley        Initial development
  2.0    05/98  D.Lloyd         Modifications.  Added drive name parameter.
  				Allow NULL end file number to denote read
				all files.  LAS coding standards.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util2.h"
#include "tarex.h"

void getpar
(
   struct PARBLK *vblock,  /* TAE variable block                */
   char *tapeid,           /* Tape ID                           */
   char *drive,		   /* tape drive name			*/
   char *ceosid,           /* CEOS ID of recieving station      */
   char *dstdir,           /* Destination directory             */
   long *images,           /* The images to extract             */
   long *nimages,          /* The number of images to extract   */
   long *msgtim		   /* time for processing message	*/
)
{
   struct VARIABLE *var;      /* Pointer to a TAE variable       */
   char subcmd[7];            /* Subcommand RANGE or SELECT      */
   long start;                /* Start image for RANGE           */
   long stop;                 /* Stop image for RANGE            */
   int i;                     /* Extra variables                 */

/* Retrieve the tape ID and convert to all-lowercase
   to provide conformity with TPACQUIRE standards.
----------------------------------------------------*/
   var = p_find(vblock, "TAPEID");
   strcpy(tapeid, SVAL(*var,0));
   i = strlen(tapeid);
   c_up2low(tapeid,(long *)&i);

/* retrieve the tape drive name
--------------------------------*/
   var = p_find(vblock, "DRIVE");
   strcpy(drive, SVAL(*var,0));

/* Retrieve the CEOS ID 
-----------------------*/
   var = p_find(vblock, "CEOSID");
   strcpy(ceosid, SVAL(*var,0));
   if ( strcasecmp( ceosid, "cpa" ) == 0 )
       strcpy( ceosid, "cp3" );

/* Retrieve the destination directory
-------------------------------------*/
   var = p_find(vblock, "DSTDIR");
   strcpy(dstdir, SVAL(*var, 0));

/* Retrieve the application subcommand
--------------------------------------*/
   var = p_find(vblock, "_SUBCMD");
   strcpy(subcmd, SVAL(*var,0));

/* Determine the images needed based on the subcommand
------------------------------------------------------*/
   if(strcmp(subcmd,"RANGE") == 0)
   {
/* Retrieve the starting and stopping scenes in the RANGE
   subcommand and construct the image array
---------------------------------------------------------*/
      var = p_find(vblock, "START");
      start = IVAL(*var,0);
      var = p_find(vblock, "STOP");
      if ( var->v_count != 0 )
	  stop = IVAL(*var,0);
      else
          stop = MAX_IMAGES;
      for(i = 0; start < stop+1; i++,start++)
         images[i] = start;
      *nimages = i;
   }
   else
   {
/* Retrieve the images from the SELECT subcommand
   and construct the image array.
-------------------------------------------------*/
      var = p_find(vblock, "IMAGENUM");
      *nimages = var->v_count;
      for(i = 0; i < *nimages; i++)
         images[i] = IVAL(*var, i);
   }

/* Get the time between processing messages
-------------------------------------------*/
var = p_find( vblock, "$MSGTIM" );
*msgtim = IVAL( *var, 0 );

}
