/*****************************************************************************
FUNCTION:	PROCESS

PURPOSE:	Extract the tar'd files from tape.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    05/98	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "adapsutil.h"
#include "tapeio.h"
#include "tarex.h"


extern  long    curent;	/* current processing block/line		*/

#define Z  1
#define GZ 2

void process
(
char	tar_buf[RECORDS][RECORD_SIZE],	/* current tar block            */
long    msgtim,         /* time between processing messages             */
char    log[CMLEN],     /* log file path and name                       */
char    key[16],        /* key field for c_wrtlog()                     */
long    image[MAX_IMAGES],      /* array of image number to extract     */
long    tplun,          /* tape logical unit number                     */
char    *tape_buf,      /* tape input buffer                            */
long    tape_blksize,   /* block size of the tape blocks                */
enum    stats   *bufstat,       /* tar buffer status                    */
char    dstdir[CMLEN],  /* destination directory of the images          */
char    tapeid[CMLEN],  /* ID of tape mounted                           */
char    ceosid[4],      /* the CEOS ID of the receiving station         */
char    errdir[CMLEN],  /* error directory name                         */
char    out[MAX_IMAGES][CMLEN], /* list of output images created        */
long    nimages,        /* the number of files to extract               */
long	*count		/* count of images extracted			*/
)

{
FILE    *ouf;           /* output file descriptor                       */

char    aname[CMLEN];   /* output file name                             */
char    fname[CMLEN];   /* temporary output file name                   */
char    msg[CMLEN];     /* message string for wrtlog and lasmsg         */

enum    stats   valid;  /* header status                                */

long    bad_hdr;        /* flag: print bad header message               */
long    blocks;         /* count of blocks processed looking for header */
long    cur_file;       /* the current tape file number                 */
long    cur_rec;        /* current record in tar block                  */
long    dmy = 1;        /* dummy variable for processing initialization */
long    done;           /* flag: for terminating while loop             */
long    first;          /* flag: first bad header                       */
long    flag = GEN2;    /* processing message type flag                 */
long    imagenum;       /* index into image array                       */
long    incmplt;        /* flag: indicates read error before complete   */
long    last_size;      /* number of bytes in the last 512 record       */
long    lcount;         /* local count of output images created         */
long    msgflg;         /* flag: processing message initialized         */
long    nblks;          /* number blocks processed looking for header   */
long    nrecords;       /* number of records used by the file           */
long    prcsflg;        /* flag: process current file to disk           */
long    size;           /* file size obtained from tar header           */
long    status;         /* function return status                       */
long    year;           /* year extracted from tape file header         */

struct  tm      *ltime; /* time structure for extracting year           */

time_t  mtime;          /* creation time of the tape file               */

union   record  *rcd;   /* pointer to the tar record header block       */

int file_compressed;	/* flag indicating the input file is compressed	*/


/*
  Initialize loop parameters.
------------------------------*/
done = FALSE;
lcount = imagenum = cur_rec = 0;
cur_file = 1;


/*
  Main while loop.
  There are three conditions for exiting this loop: TAPE_EOT is
  reached, a tape read error occurs before completely extracting
  a file, or all selected images have been extracted.
-----------------------------------------------------------------*/
while ( !done )
    {
    blocks = nblks = 1;
    valid = HEADER_FAILURE;
    first = bad_hdr = TRUE;
    incmplt = msgflg = FALSE;

    /* find a valid header, continue getting tar blocks until
       one is found or TAPE_EOT is reached.
    ----------------------------------------------------------*/
    while ( valid == HEADER_FAILURE )
        {
        valid = get_hdr( tar_buf, &cur_rec, &size );
        if ( valid == HEADER_FAILURE )
            {
            /* give the user a message the first time throught this loop */
            if ( first )
                {
                first = FALSE;
                c_lasmsg(FALSE," ","Searching for TAR header");

                /*
                  start a processing message to indicate the number
                  of blocks searched while looking for the header
                ----------------------------------------------------*/
                curent = 0;
                flag = GEN1;
                c_inittm("TAREX","Searched","block"," "," ",
                            &dmy,&dmy,&msgtim,&flag);
                curent = 1;
                c_startm();
                msgflg = TRUE;
                }

            /* 
              Give an error message if more than MIN_BAD blocks have been
              searched.  This provides an error in the log file so that
              missed files may be highlighted.  I say 'may' since I picked
              an arbitrary value for MIN_BAD.  Small files, though unlikely
              to occur, could be missed but not indicated by an error msg.
            ---------------------------------------------------------------*/
            if ( bad_hdr && ( nblks > MIN_BAD ))
                {
                bad_hdr = FALSE;
                sprintf( msg, "Error: searching for TAR header" );
                c_lasmsg(FALSE,"tarex-header",msg);
                c_wrtlog( log, key, "tarex-header", msg );
                sprintf( msg, "Unable to process file %d", image[imagenum] );
                c_lasmsg(FALSE," ",msg);
                c_wrtlog( log, key, "informational", msg );
                /*
                  Also increment the image num so we know at least one file
                  was missed.  Without examining each record more closely
                  we cannot be sure how many files were missed so just
                  default to one file.
                ------------------------------------------------------------*/
                if ( cur_file == image[imagenum] )
                    imagenum++;
                cur_file++;
                }
            status = fill_tar_blk( &tplun, tape_buf, (char *)tar_buf, tape_blksize );
            if ( status == TAPE_EOT )
                break;
            nblks++;
            curent = nblks;
            cur_rec = 0;
            *bufstat = BUF_FULL;
            }
        }  /* END while: header search loop */
    if ( msgflg )
        c_stoptm();

    /*
      If tape end was reached while looking for a valid header
      we are done.
    ----------------------------------------------------------*/
    if ( status == TAPE_EOT )
        {
        done = TRUE;
        continue;
        }

    /*
      We have a valid TAR header at this point.  Extract file
      information from the header to build the output file
      name and to determine file size.
    ----------------------------------------------------------*/
    rcd = (union record *) &tar_buf[cur_rec][0];
    nrecords = ( size + RECORD_SIZE - 1 ) / RECORD_SIZE;
    last_size = size % RECORD_SIZE;
    cur_rec++;  /* now pointing at first record of file */
    /* determine if this file should be written to disk */
    if ( cur_file == image[imagenum] )
        {
        prcsflg = TRUE;
        /* build the file name use: tapeid, year image number and ceosid */
        sscanf( rcd->header.mtime, "%o", &mtime );
        ltime = localtime( &mtime );
        year = ltime->tm_year;
        if ( year >= 100 )
            year -= 100;
	if ( strcasecmp( ceosid, "cp3" ) == 0 )
	    /* Can't depend on the year from the cpa header and
	    can't depend on the year being 2-digit so check the
	    files date string.  The file names look like:
	        N16_20020317_1720.raw
	    or 
		N16_020317_1720.raw
	    for an ingested sceneid of: ah16031702172116
	    We must assume that the date string length indicates
	    either a 2- or 4-digit year.
	    ---------------------------------------------------*/
	    {
	    char *first, *last;
	    first = strchr( rcd->header.name, '_' );
	    last = strrchr( rcd->header.name, '_' );
	    if (( last - first ) == 7 ) /* assume 2-digit year */
		sscanf( rcd->header.name, "%*4c%2d", &year );
	    else
		sscanf( rcd->header.name, "%*6c%2d", &year );

	    /* Another thing to consider is whether the
	    file is compressed or not.  Compressed files
	    end with a .Z so check for it and set the 
	    file_compressed flag if the Z is in file name.
	    --------------------------------------------*/
	    if ( strrchr( rcd->header.name, 'Z' ) != NULL )
	        file_compressed = Z;
	    else if ( strrchr( rcd->header.name, 'z' ) != NULL &&
	              strrchr( rcd->header.name, 'g' ) != NULL )
	        file_compressed = GZ;
	    else
	        file_compressed = FALSE;
	    }

	/* this is a temporary file name so INGEST can't see it */
	sprintf( fname, "%s/z%s_%02d_fn%03d.%s", dstdir, tapeid, year,
		image[imagenum], ceosid );
        /* this is the final file name, one that INGEST recognizes */
        sprintf( aname, "%s/a%s_%02d_fn%03d.%s", dstdir, tapeid, year,
                image[imagenum], ceosid );
        /* let the user know we are extracting a file */
        sprintf( msg, "Extracting tape file %d", image[imagenum] );
        c_lasmsg(FALSE," ",msg);
        c_wrtlog( log, key, "informational", msg );
        /* open the output file */
        ouf = fopen( fname, "w" );
        if ( !ouf )
            {
            c_lasmsg(FALSE,"tarex-open","Unable to open %s",fname);
            prcsflg = FALSE;
            done = TRUE;
            continue;
            }
        }
    else
        {
        /* let the user know we are skipping the file */
        sprintf(msg,"Skipping tape file %d",cur_file);
        c_lasmsg(FALSE," ",msg);
        c_wrtlog( log, key, "informational", msg );
        prcsflg = FALSE;
        }

    /*
      Start a processing message indicating how many records
      have been read from tape (for tape file skip) or written
      to disk (for an extraction).
    -----------------------------------------------------------*/
    curent = 0;
    flag = GEN2;
    if ( prcsflg )
        c_inittm("TAREX","Wrote","record"," "," ",&nrecords,&dmy,&msgtim,&flag);
    else
        c_inittm("TAREX","Read","record"," "," ",&nrecords,&dmy,&msgtim,&flag);
    curent = 1;
    c_startm();

    /*
      Process the file by writing the records to disk or skip
      the file by doing nothing with the records.  Nrecords must
      be read from the tape even if we are skipping the file
      since another tar'd file may exist in the current tape
      file.
    -------------------------------------------------------------*/
    while ( nrecords > 0 )
        {
        /* flush full buffer    */
        if ( cur_rec + nrecords > RECORDS )
            {
            blocks += RECORDS-cur_rec;
            if ( prcsflg )
                wrt_buf( ouf, (char *)tar_buf, cur_rec, RECORDS-cur_rec, RECORD_SIZE );
            *bufstat = BUF_EMPTY;
            nrecords -= RECORDS - cur_rec;
            }

        /* flush the full buffer except for the garbage in the last record */
        else if ( cur_rec + nrecords == RECORDS )
            {
            blocks += nrecords-1;
            if ( prcsflg )
                wrt_buf( ouf, (char *)tar_buf, cur_rec, nrecords, last_size );
            *bufstat = BUF_EMPTY;
            nrecords -= RECORDS - cur_rec;
            }

        /* write nrecords from the current record except for garbage at end */
        else if ( cur_rec + nrecords < RECORDS )
            {
            blocks += nrecords-1;
            if ( prcsflg )
                wrt_buf( ouf, (char *)tar_buf, cur_rec, nrecords, last_size );
            *bufstat = BUF_PRTL;
            cur_rec += nrecords;
            nrecords = 0;
            }

        /* fill up the tar buffer if it's empty */
        if ( *bufstat == BUF_EMPTY )
            {
            cur_rec = 0;
            status = fill_tar_blk( &tplun, tape_buf, (char *)tar_buf, tape_blksize );
            if ( status < 0 )
                {
                incmplt = TRUE;
                nrecords = 0;
		/*
		  If the tar file wasn't created with a blocking factor
		  of 20 (default tar block size of 10240), then tape end
		  may be reached before extracting the entire file even
		  if the entire file _could_ be extracted.  If, in future,
		  tapes are encountered with this problem this logic will 
		  have to be revised.
		---------------------------------------------------------*/
                if ( status == TAPE_EOT )
                    done = TRUE;
                continue;
                }
            *bufstat = BUF_FULL;
            }
        curent = blocks;
        }  /* END while: tape record processing loop */
    c_stoptm();

    /*
      If we are processing the file, close and rename it so INGEST
      can pick it up.  Add the name to the list of output names.
    ---------------------------------------------------------------*/
    if ( prcsflg )
        {
        fclose( ouf );

        /*
          If the file extraction was incomplete based on the
          number of records stored in the header, move the partial
          file to ADAPSERR and print a message.
        -----------------------------------------------------------*/
        if ( incmplt )
            {
            sprintf(msg,"Error: file extraction not complete");
            c_lasmsg(FALSE,"tarex-incmplt",msg);
            c_wrtlog( log, key, "tarex-incmplt", msg );
            sprintf( aname, "%s/z%s_%d_fn%03d.%s", errdir, tapeid, year,
                image[imagenum], ceosid );
            rename( fname, aname );
            }

        /*
          Otherwise rename it and print a message
        ------------------------------------------*/
        else
            {
	    if ( strcasecmp( ceosid, "cp3" ) == 0 )
		{
		/* Uncompress the file if needed.  Rename the
		file to put the .Z extension back on the file
		---------------------------------------------*/
		if ( file_compressed )
		    {
		    char syscmd[CMLEN];	/* system command string*/
		    char cname[CMLEN];	/* compressed file name	*/

		    /* rename the file */
		    if ( file_compressed == Z )
			{
			sprintf(cname, "%s.Z", fname);
			rename(fname, cname);
			sprintf(syscmd,"uncompress %s.Z", fname);
			}
		    else
			{
			sprintf(cname, "%s.gz", fname);
			rename(fname, cname);
			sprintf(syscmd,"gunzip %s.gz", fname);
			}

		    /* uncompress it */
		    if ( system( syscmd ))
		        {
			sprintf(msg,"Error: uncompressing file");
			c_lasmsg(FALSE,"tarex-incmplt",msg);
			c_wrtlog( log, key, "tarex-incmplt", msg );
			if ( file_compressed == Z )
			    sprintf( aname, "%s/z%s_%d_fn%03d.%s.Z", errdir,
				tapeid, year,image[imagenum], ceosid );
			else
			    sprintf( aname, "%s/z%s_%d_fn%03d.%s.gz", errdir,
				tapeid, year,image[imagenum], ceosid );
			rename( cname, aname );
			}
		    }

		/* Convert the raw image
		to an ingestable format.
		------------------------*/
		convert_cp3( fname );
		}

            if ( rename( fname, aname ) != 0 )
                {
                c_lasmsg(FALSE,"tarex-renam","Error renaming %s", fname);
                strcpy( out[lcount++], fname );
                }
            else
                strcpy( out[lcount++], aname );
            sprintf( msg, "File %d complete", image[imagenum] );
            c_lasmsg(FALSE," ",msg);
            c_wrtlog( log, key, "informational", msg );
            }
        imagenum++;
        }


    /*
      Decide if we can quit before reaching tape end: all
      files the user specified have been extracted.
    --------------------------------------------------*/
    cur_file++;
    if ( cur_file > image[nimages-1] )
        done = TRUE;
    }  /* END Main while loop */


/*
  Return the count of images processed
---------------------------------------*/
*count = lcount;

}
