/*******************************************************************************
NAME:   	FINISH

PURPOSE:        Update the image's ddr and history; create an addr file for OUT

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "model.h"

FUNCTION finish(hosin,hosout,taein,taeout,vblock,model,window,
		bands,nbands,proj,overlay,mnmx)

struct PARBLK	*vblock;	/* TAE parameter block			*/
struct MODEL	*model;		/* Model and orbital information	*/

char    hosin[][CMLEN],  	/* input host file			*/
        taein[][CMLEN],  	/* input TAE image name                 */
        hosout[][CMLEN], 	/* output hos file image name         	*/
        taeout[][CMLEN], 	/* output TAE image name                */
        mnmx[];        		/* min/max calculation flag             */
 
long 	window[1][4],   	/* window specification for input/output image*/
        bands[1][MAXBND+1],     /* band number to be processed          */
        nbands[1],      	/* number of bands per input image      */
        overlay;        	/* overlay line work on input image flag*/

char    proj[6];        	/* projection type (satellite or reprojected)*/
{
double  dblone = 1.0,   	/* double value of 1                    */
        minval[MAXBND],   	/* minimum value of image               */
        maxval[MAXBND];   	/* maximum value of image               */

long	zero = 0,       	/* constant zero                        */
        one = 1;        	/* constant one                         */

if (overlay)
    {
    if (strcmp(mnmx,"YES") == 0)
	{
        /* invalidate the min & max 		*/
        c_svmnmx(hosin[0],&one,minval,maxval,"NO");

        /* update the min & max 			*/
        c_minmax(hosin,&one,window,bands,nbands,minval,maxval);
	}

    /* update the image's history		*/
    c_hstout(vblock,hosin[0]," ",&zero," ");

    c_complt("WDB2RAS",taein,&one);
    }
else
    {
    /* generate an AVHRR ddr for the new file 	*/
    if (strcmp(proj,"SAT") == 0)
    	genaddr(hosout[0],model);

    /* update the new image's ddr     	*/
    c_upddr(vblock,hosin,hosout[0],&one,window,bands,nbands,
		     &dblone,&dblone,&one,&one);

    if (strcmp(mnmx,"YES") == 0)
        /* update the min & max 			*/
        c_minmax(hosout[0],&one,window,bands,nbands,minval,maxval);

    /* update the image's history		*/
    c_hstout(vblock,hosout[0],hosin,&one," ");

    c_complt("WDB2RAS",taeout,&one);
    }
}
