/*****************************************************************************
NAME:		GENRST 

PURPOSE:	Generate the rasterized image

******************************************************************************/
#include <stdio.h>
#include "las.h"

FUNCTION genrst(vblock,window,odtype,overlay,backgrnd,imgback,msgtim,fp_sort)

struct	PARBLK	*vblock; /* TAE parameter block				     */

long	window[],	/* window to process				     */
	*odtype,	/* output data type				     */
	overlay,	/* overlay linework flag			     */
	*backgrnd,	/* background pixel value			     */
	imgback,	/* image background flag			     */
	*msgtim;	/* time between processing messages		     */

FILE	*fp_sort;	/* file pointer to the sorted file		     */
{
extern int fdesc[3];	/* image file descriptors used for image I/O	     */
extern int gdesc;	/* group file descriptor used for image I/O	     */

extern char *thebuf;	/* image buffer					     */
extern long ooffset[2];	/* offsets into "thebuf" for output bands	     */
extern long ioffset[2];	/* offsets into "thebuf" for input bands	     */

extern char tfiles[3][CMLEN];	/* output temporary host file names	     */
extern long numout;	/* number of output temp files			     */
extern long curent;	/* current "line" for processimg messages	     */

long	curlin,		/* line being processed from the window		     */
	endline,	/* end line of window				     */
	nbytes,		/* number of bytes read from sort file		     */
	line,		/* line number containing vector data		     */
	prior,		/* priority value of the pixel to be rasterized	     */
	start,		/* start sample containing vector data for the line  */
	stop,		/* stop sample containing vector data for the line   */
	pixval,		/* pixel value for the line			     */ 
	status,		/* return status variable			     */
	i,		/* looping variable				     */
	zero = 0,	/* constant zero				     */
	nonfatal = 1,	/* non-fatal error code				     */
	msgfmt = GEN2;	/* flag for processing message format		     */

char 	fillc;
short	fills;
long	filll;
float	fillf;
double	filld;
unsigned char *fillval;

/*
  Defines the number of bytes for each of the four data types
*/
static long size[NDTYPES] = {0,1,2,4,4,8};

/*
  Set up the pixel fill value
*/
if (*odtype == EBYTE)
    {
    fillc = (unsigned char)(*backgrnd);
    fillval = (unsigned char *)(&fillc);
    }
else if (*odtype == EWORD)
    {
    fills = (short)(*backgrnd);
    fillval = (unsigned char *)(&fills);
    }
else if (*odtype == ELONG)
    {
    filll = (long)(*backgrnd);
    fillval = (unsigned char *)(&filll);
    }
else if (*odtype == EREAL)
    {
    fillf = (float)(*backgrnd);
    fillval = (unsigned char *)(&fillf);
    }
else
    {
    filld = (double)(*backgrnd);
    fillval = (unsigned char *)(&filld);
    }

/*
  Get the first record from the sorted scan file.  When there are
  no records, all data must be outside the image.
*/
nbytes = fscanf(fp_sort,"%10d %10d %10d %10d %11d\n",
		&line,&prior,&start,&stop,&pixval);
if (nbytes == EOF)
    {
    c_errmsg("No vector data occurs in the output image","wdb2ras-empty",
	     &nonfatal);
    fclose(fp_sort);
    c_egclse(&gdesc);
    c_clean(vblock,"wdb2ras",tfiles,&numout);
    }

/*
  Initialize the processing message
*/
curent = 1;
c_inittm("WDB2RAS","Processed","line"," "," ",&window[NL],&zero,msgtim,
	 &msgfmt);
c_startm();

/*
  Generate any required preliminary background lines that occur
  before the first line containing vector data
*/
for (curlin = window[SL]; curlin <= line; curlin++)
    {
    status = c_estep(&gdesc);
    if (status != E_SUCC)
	{
    	fclose(fp_sort);
    	c_egclse(&gdesc);
        c_clean(vblock,"wdb2ras",tfiles,&numout);
	}

    /*
      Fill the next line with the background value
    */
    if (imgback)
        c_pxcopy(thebuf+(ioffset[0]*size[*odtype]), 
	         thebuf+(ooffset[0]*size[*odtype]),
	         odtype,&window[NS]);
    else if (!overlay)
        c_pxfill(thebuf+(ooffset[0]*size[*odtype]),odtype,&window[NS],fillval);

    curent++;
    }

curlin = line;	

/*
  Do for each record of the sorted scan file
*/
endline = window[SL] + window[NL] - 1;
while (curlin <= endline)
    {
    if (curlin < line)
	{
	status = c_estep(&gdesc);
	if (status != E_SUCC)
	    {
    	    fclose(fp_sort);
    	    c_egclse(&gdesc);
	    c_clean(vblock,"wdb2ras",tfiles,&numout);
	    }

	/*
	  Fill the next line with the background value
	*/
	if (imgback)
            c_pxcopy(thebuf+(ioffset[0]*size[*odtype]), 
	             thebuf+(ooffset[0]*size[*odtype]),
	             odtype,&window[NS]);
	else if (!overlay)
    	   c_pxfill(thebuf+(ooffset[0]*size[*odtype]),odtype,&window[NS],
		fillval);

	curent++;
	curlin++;
	}
    else
	{
	/*
	  Load the next segment of the current image line.
	  If the current pixel value does not equal the background
	  a line with a higher priority has already been drawn.
	*/
	for (i = (start - window[SS]); i <= (stop - window[SS]); i++)
	    {
	    switch (*odtype)
		{
		case EBYTE:
		  *(char *)(thebuf + (ooffset[0] + i) * size[*odtype]) = pixval;
		  break;
		case EWORD:
		  *(short *)(thebuf + (ooffset[0] + i) * size[*odtype])= pixval;
		  break;
		case ELONG:
		  *(long *)(thebuf + (ooffset[0] + i) * size[*odtype]) = pixval;
		  break;
		case EREAL:
		  *(float *)(thebuf+(ooffset[0]+i)*size[*odtype])=(float)pixval;
		  break;
		}
	    }
    
	    nbytes = fscanf(fp_sort,"%10d %10d %10d %10d %11d\n",
			    &line,&prior,&start,&stop,&pixval);
	    if (nbytes == EOF)
		    break;
	}
    }

/*
  Generate any required background lines that occur after the
  last line containing vector data
*/
while (curlin < endline)
    {
    /*
      Fill the next line with the background value
    */
    status = c_estep(&gdesc);
    if (status != E_SUCC)
	{
    	fclose(fp_sort);
    	c_egclse(&gdesc);
	c_clean(vblock,"wdb2ras",tfiles,&numout);
	}

    if (imgback)
        c_pxcopy(thebuf+(ioffset[0]*size[*odtype]), 
	         thebuf+(ooffset[0]*size[*odtype]),
	         odtype,&window[NS]);
    else if (!overlay)
        c_pxfill(thebuf+(ooffset[0]*size[*odtype]),odtype,&window[NS],fillval);

    curlin++;
    curent++;
    }

/*
  Final write
*/
status = c_estep(&gdesc);
if (status != E_SUCC)
    {
    fclose(fp_sort);
    c_egclse(&gdesc);
    c_clean(vblock,"wdb2ras",tfiles,&numout);
    }

/*
  Close the image
*/
c_egclse(&gdesc);

/*
  If an output image was created remove it from the list of files to get
  rid of when an error occurs
*/
if (!overlay)
    numout--;

/*
  Stop the processing message
*/
c_stoptm();

/*
  Close and delete the sorted scan file
*/
fclose(fp_sort);
numout--;
c_delfil(tfiles[numout]);

/*
  Free the image buffer
*/
free(thebuf);

return;
}
