/*******************************************************************************
NAME:		GETPAR

PURPOSE:	Retreive the user specified input parameters

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "typlim.h"
#include "wdb2ras.h"

FUNCTION getpar(vblock,ddr,area,acount,feature,fcount,ftypes,taein,hosin,
		taeout,hosout,window,bands,nbands,odtype,pixval,backgrnd,
		imgback,linwt,overlay,proj,pntint,msgtim,mnmx,hisflag)

struct	PARBLK	*vblock;	/* TAE parameter block			     */
struct  DDR	*ddr;		/* DDR information			     */

long	*acount,		/* number of areas to rasterize		     */
	window[][4],		/* window specification			     */
	bands[][MAXBND+1],	/* bands to be processed		     */
	nbands[],		/* band numbers selected		     */
	*odtype,		/* output data type			     */
	*pixval,		/* pixel intensity value for rasterized pixel*/
	*backgrnd,		/* background value for rasterized image     */
	*imgback,		/* background filled with image data flag    */
	*linwt,			/* width of the rasterized line		     */
	*overlay,		/* overlay line work option		     */
	*pntint,		/* point interval option		     */
	*msgtim,		/* time between processing messages	     */
	fcount[],		/* number of option for each feature	     */
	ftypes[],		/* feature types selected		     */
	feature[][15];		/* feature classifications to be rasterized  */

char	area[][8],		/* area to be rasterized (NA, SA, EU, AF, AS)*/
	taein[][CMLEN],		/* input TAE image name			     */
	hosin[][CMLEN],		/* input host image name		     */
	taeout[][CMLEN],	/* output TAE image name		     */
	hosout[][CMLEN],	/* output host image name		     */
	*proj,			/* projection type (satellite or rectified)  */
	*hisflag,		/* history update flag			     */
	*mnmx;			/* min/max calculation flag		     */
{
struct VARIABLE	*var;
struct VARIABLE *p_find();

long	nmrimg,			/* number of input image specified	     */
	totbnd,			/* Total number of bands		     */
	same_window,		/* flag indicating if images hav same window */
	qflag[1][2],		/* flag indicating if subwindows specified   */
	nonfatal = 1,		/* non-fatal error code			     */
	fatal = -1,		/* fatal error code			     */
	zero = 0,		/* constant of zero			     */
	one = 1,		/* constant of one			     */
	eight = 8,		/* constant of eight			     */
	status,			/* status return code			     */
	lcount = 0,		/* total number of features		     */
	i;			/* looping variable			     */

/*
  Get the input image name defining the area in which the vector
  data is to be rasterized
*/
status = c_getin(vblock,"IN",&zero,taein,hosin,&nmrimg,window,bands,nbands,
		 &totbnd,&one,&same_window,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);

/*
  Get the DDR information
*/
status = c_getddr(hosin,ddr);
*odtype = ddr->dtype;

/*
  Get the output file name.  When an output name is not specified update the
  input image by overlaying line work on the input image.
*/
status = c_getout(vblock,"OUT",taeout,hosout);
if (strlen(taeout[0]) == 0)
    {
    *overlay = TRUE;
    *backgrnd = 0;
    /*
      Check to make sure that there is only one band to overlay linework on
    */
    if ((nmrimg > 1) || (totbnd > 1))
	{
	c_errmsg("Only a single band image allowed on input","wdb2ras-onebnd",
		  &nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}
    }
else
    if (status == E_SUCC)
        {
        *overlay = FALSE;
        var = p_find(vblock,"BACKGRND");
	if (var->v_count == 0)
	    {
	    *imgback = TRUE;
            *backgrnd = 0;
    	    /*
      	      Check to ensure there is only one band to overlay linework on
    	    */
    	    if ((nmrimg > 1) || (totbnd > 1))
	        {
		c_errmsg("Only a single band image allowed on input",
			 "wdb2ras-onebnd",&nonfatal);
		c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
		}
	    }
	else
            *backgrnd = IVAL(*var,0);
        }
    else
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);

/*
  Get the pixel value intensity for the line work
*/
var = p_find(vblock,"PIXVAL");
*pixval = IVAL(*var,0);

/*
  Check that the background and pixel value are in a valid range for the data
  type of the input image.
*/
if (*odtype == EBYTE)
    {
    if ((*backgrnd > MAXUTINY) || (*backgrnd < MINUTINY))
	{
	c_errmsg("BACKGRND is not within the valid range for byte data",
	    "wdb2ras-brange",&nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}

    if ((*pixval > MAXUTINY) || (*pixval < MINUTINY))
	{
	c_errmsg("PIXVAL is not within the valid range for byte data",
	    "wdb2ras-prange",&nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}
    }
else if (*odtype == EWORD)
    {
    if ((*backgrnd > MAXWORD) || (*backgrnd < MINWORD))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*2 data",
	    "wdb2ras-brange",&nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}

    if ((*pixval > MAXWORD) || (*pixval < MINWORD))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*2 data",
	    "wdb2ras-prange",&nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}
    }
else if (*odtype == ELONG)
    {
    if ((*backgrnd > MAXLONG) || (*backgrnd < MINLONG))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*2 data",
	    "wdb2ras-brange",&nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}

    if ((*pixval > MAXLONG) || (*pixval < MINLONG))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*2 data",
	    "wdb2ras-prange",&nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}
    }
else /* (*odtype == EREAL) */
    {
    if ((*backgrnd > MAXREAL) || (*backgrnd < MINREAL))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*2 data",
	    "wdb2ras-brange",&nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}

    if ((*pixval > MAXREAL) || (*pixval < MINREAL))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*2 data",
	    "wdb2ras-prange",&nonfatal);
	c_errmsg("Fatal error encountered","wdb2ras-fatal",&fatal);
	}
    }

/*
  Get the area of interest
*/
var = p_find(vblock,"AREA");
*acount = var->v_count;
for (i = 0; i < *acount; i++)
    {
    strcpy(area[i],SVAL(*var,i));
    c_low2up(area[i],&eight);
    }

/*
  Get the features to be rasterized.
  Coastlines, Islands, and Lakes
*/
var = p_find(vblock,"CIL");
fcount[0] = var->v_count;
ftypes[0] = CIL;
lcount += fcount[0];
for (i = 0; i < fcount[0]; i++)
    feature[0][i] = IVAL(*var,i);

/*
  Rivers
*/
var = p_find(vblock,"RIV");
fcount[1] = var->v_count;
ftypes[1] = RIV;
lcount += fcount[1];
for (i = 0; i < fcount[1]; i++)
    feature[1][i] = IVAL(*var,i);

/*
  Internal boundaries
*/
var = p_find(vblock,"BDY");
fcount[2] = var->v_count;
ftypes[2] = BDY;
lcount += fcount[2];
for (i = 0; i < fcount[2]; i++)
    feature[2][i] = IVAL(*var,i);

/*
  Political boundaries
*/
var = p_find(vblock,"PDY");
fcount[3] = var->v_count;
ftypes[3] = PDY;
lcount += fcount[3];
for (i = 0; i < fcount[3]; i++)
    feature[3][i] = IVAL(*var,i);

/*
  Railroads
*/
var = p_find(vblock,"RRR");
fcount[4] = var->v_count;
ftypes[4] = RRR;
lcount += fcount[4];
for (i = 0; i < fcount[4]; i++)
    feature[4][i] = IVAL(*var,i);

/*
  Roads
*/
var = p_find(vblock,"RDS");
fcount[5] = var->v_count;
ftypes[5] = RDS;
lcount += fcount[5];
for (i = 0; i < fcount[5]; i++)
    feature[5][i] = IVAL(*var,i);

/*
  When no features are specified, rasterize the area (file!) specified
  by the user in the area parameter.
*/ 
ftypes[6] = 0;
if (lcount == 0)
    fcount[6] = 1;
else
    fcount[6] = 0;

/*
  Get the width of the line
*/
var = p_find(vblock,"LINWT");
*linwt = IVAL(*var,0);

/*
  Get the projection type
*/
var = p_find(vblock,"PROJ");
strcpy(proj,SVAL(*var,0));

/*
  Get the point interval option
*/
var = p_find(vblock,"PNTINT");
*pntint = IVAL(*var,0);

/*
  Get the time between processing messages
*/
var = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*var,0);

/*
  Get the min/max processing flag
*/
var = p_find(vblock,"$MINMAX");
strcpy(mnmx,SVAL(*var,0));

/*
  Get the history update flag
*/
var = p_find(vblock,"$HISTRY");
strcpy(hisflag,SVAL(*var,0));

return;
}
