/*******************************************************************************
FUNCTION:	GET_OUT

PURPOSE:	Get the user specified output file name parameters.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    02/91  D.Hollaren       Conversion to LAS 5.0
  1.1    01/92  T.Baltzer        Stolen from MIARCH & modified
  1.2    05/93  J.Fenno          Modified to use the new AVHRR model and the new
				 archio supports.
  1.3    11/96  A.Vander Vorste	 Stolen from INGEST1B and modified
  1.4    11/97  T.Baltzer	 Stolen from Vander Vorste WINDOW1B

PROJECT:	ADAPS
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "localprotos.h"


int get_out
(
struct PARBLK	*vblock,	/* TAE parameter block			    */
char		*hosout,	/* Output host file name		    */
long		*genfil,	/* Flag to generate an output file name	    */
char	outnam[2][CMLEN],	/* Output files                             */
long		*outcnt		/* Number of output files		    */
)
{
long	pid;			/* Process ID for a unique temporary name   */

long	element = 0,		/* Element number for getnam call	    */
	status,			/* Return status			    */
        waccess = IWRITE;       /* File access flag                         */

char	hosdir[CMLEN],		/* Temporary TAE file name		    */
	hosroot[CMLEN],		/* Temporary TAE file name		    */
	hosext[CMLEN],		/* Temporary TAE file name		    */
	tmpfil[CMLEN];		/* Temporary output file name		    */

/*
  Retrieve the output window1b name
*/
status = c_getnam(vblock,"OUTFILE",&element,hosout,"l1b",&waccess);
if (status != E_SUCC)
    return(E_FAIL);

/*
  Assume the user put in a directory and file name
*/
*genfil = FALSE;

if ((long)(strlen(hosout)) > 0)
    {
    /*
      Get the directory & root name from the user's host name
    */
    c_splitname(hosout,hosdir,hosroot,hosext);
    *outcnt = 1;

    /*
      Check to see if the user specified a file name.  If not, set a
      flag so that a output name will be generated for the user.
    */
    if (strlen(hosroot) == 0)
	{
	*genfil = TRUE;
	strcpy(hosout,hosdir);
	strcpy(outnam[1],hosdir);
	}
    else
	{
        strcpy(outnam[1],hosdir);
        strcat(outnam[1],hosroot);
        strcat(outnam[1],".");
        strcat(outnam[1],hosext);
	}

    /*
      Create a temporary file name
    */
    pid = getpid();
    sprintf(tmpfil,"%sA%ld.tmp",hosdir,pid);
    strcpy(outnam[0],tmpfil);
    *outcnt = 2;
    }
else
    {
    /* the user did not supply an output file name */

    /*
      Create a temporary file name
    */
    pid = getpid();
    sprintf(tmpfil,"A%ld.tmp",pid);
    strcpy(outnam[0],tmpfil);

    /*
      Set a flag to generate an output name for the user
    */
    *genfil = TRUE;
    }

return(E_SUCC);
}
