/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the input parameters


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0	 11/97  T. Baltzer     	 Development (stole from Vander Vorste version)

PROJECT:	ADAPS
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "util.h"
#include "localprotos.h"



int getpar
(
struct PARBLK *vblock,	/* Variable block				*/
char comment[][CMLEN],	/* Description of the tape(s)			*/
char infile[],		/* Input filename				*/
long *skip,		/* Number of files to skip before processing	*/
char *hosout,		/* Host output name				*/
long *genfil,		/* Flag to generate an output file name		*/
long *unchans,		/* Number of bands specified by user 	        */
long uchan[NCHANS],	/* User input satellite channels to process	*/
long *msgtim,		/* Time between processing messages		*/
long *pack_flg,		/* Flag to tell if user asked for packed data   */
char outnam[2][CMLEN],	/* Output files					*/
long *outcnt,		/* Number of output files			*/
char drive[]		/* Input drive name				*/
)


{
long    access=0,	/* file access					*/	
	i=0, 		/* Looping variable				*/
	status=0,	/* status returned from support calls   	*/
	zero = 0;	/* value of zero				*/


struct VARIABLE	*val;		/* pointer to a TAE variable 	        */


/*
  Retrieve input filename. If *infile equals '\0' then 
  we know that we need to read from tape instead.
*/
access = IREAD;
status = c_getnam(vblock,"INFILE",&zero,infile,"l1b",&access);
if (status != E_SUCC)
    return(E_FAIL);


/*
  Since the user did not specify a file name, we must read from tape.
  Get the drive name of the mounted tape.
*/
if (strcmp(infile,"") == 0)
    {
    val = p_find( vblock, "DRIVE" );
    if ( val->v_count != 0 )
	strcpy(drive, SVAL( *val, 0 ));
    else
	c_lasmsg(TRUE,"window1b-parm","Must enter INFILE or DRIVE parameter");
    }


/*
  Retrieve tape mount messages, make sure unspecified
  elements are nulled.
*/
val = p_find(vblock,"COMMENT");
for (i = 0; i < val->v_count; i++)
    strcpy(comment[i],SVAL(*val,i));
for (i = val->v_count; i < 3; i++)
    comment[i][0] = '\0';


/*
  Retrieve number of the file to skip on tape
*/
val = p_find(vblock,"SKIP");
*skip = IVAL(*val,0);


/*
  Retrieve channels to be processed.  If no channels are specified, all
  channels will be the default.  It is not known at this time which
  channels are in the file, so assignments will not be made here.
*/
val = p_find(vblock,"CHANS");
*unchans = val->v_count;


/* getting what the user requested for channels */
for (i = 0; i < NCHANS; i++)	/* initialize to zero */
   uchan[i] = FALSE;

/* if the user didn't specifiy any channels - it will be assumed that
   the user asked for all the channels */
if (*unchans == 0)
   {
   for (i = 0; i < NCHANS; i++)
	uchan[i] = i + 1;
   *unchans = NCHANS;
   }
else	/* channels were user specified */
   {
   for (i = 0; i < *unchans; i++)	/* Retrieve the bands specified */
      {
      uchan[IVAL(*val,i) - 1] = IVAL(*val,i);
      }
   }


/*
  Get the flag to pack or not pack the data
*/
val=p_find(vblock,"PACKFLG");
if(strcmp(SVAL(*val,0),"YES") == 0)
    *pack_flg=TRUE;
else
    *pack_flg=FALSE;


/*
  Get the output file name
*/
status = get_out(vblock,hosout,genfil,outnam,outcnt);
if (status != E_SUCC)
    return(E_FAIL);


/*
  Retrieve number of seconds between processing messages
*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);

return(E_SUCC);
}
