/****************************************************************************
NAME:		WRTBUF

PURPOSE:	Open the specified file and write a line of buffered data.


PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  1.0	 T. Baltzer    12/93	Original Development, stole from c_wrtlog

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#ifdef vms
#include <file.h>
#else
#include <sys/types.h>
#include <sys/file.h>
#endif
#include "las.h"
#ifndef SEEK_END    /* Not defined on PowerNode */
#define SEEK_END 2
#endif
#ifdef sun
#include <fcntl.h>
#endif

FUNCTION wrtbuf(filename,buffer)

char	filename[],		/* File to write to			*/
	buffer[];		/* Output buffer			*/
{
int	fd;			/* File descriptor to the file		*/

long	nonfatal = 1,		/* Non-fatal error code			*/
	maxtime = 60,		/* Maximum to wait on open/lock		*/
	flags,			/* Open flags				*/
	status,			/* Return status			*/
	len;			/* Length of the output buffer		*/


/*
  Open and lock the file for a moment of exclusive use
*/
flags = O_WRONLY | O_CREAT | O_APPEND;
status = cmlock(filename,&fd,flags,maxtime);
if (status != E_SUCC)
    {
    c_errmsg("Error locking the output file","wrtascii-lock",&nonfatal);
    return(E_FAIL);
    }

/*
  Seek to the end of the file because cmlock
  resets flag and opens the file for read/write, not append
*/
status = lseek(fd, 0, SEEK_END);
if (status == E_FAIL)
    {
    c_errmsg("Error skipping to end of the file","wrtascii-skip",&nonfatal);
    return(E_FAIL);
    }

/*
  Write a line of buffer data to the file
*/
len = strlen(buffer);
write(fd,buffer,len);

/*
  Unlock and close the file
*/
status = unlock(&fd,maxtime);
if (status != E_SUCC)
    {
    c_errmsg("Error unlocking the file","wrtascii-unlock",&nonfatal);
    close(fd);
    }
    
return(E_SUCC);
}
