#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <Xm/Xm.h>
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <X11/cursorfont.h>
#include "adaps.h"
#include "scenes.h"

#ifndef MAX
#define     MAX(a,b) ((a) > (b) ? (a) : (b)) 
#endif
#ifndef MIN
#define     MIN(a,b) ((a) < (b) ? (a) : (b)) 
#endif
#ifndef MIN4
#define MIN4(a,b,c,d) (MIN(a,b) < MIN(c,d) ? MIN(a,b) : MIN(c,d))
#endif
#ifndef MAX4
#define MAX4(a,b,c,d) (MAX(a,b) > MAX(c,d) ? MAX(a,b) : MAX(c,d))
#endif

#define GEO 0
#define NORTH 1
#define SOUTH -1
#define WIDTH 792
#define HEIGHT 612

extern long qtemp;                      /* queue id of temp list of scenes */
extern long qkeep;                      /* queue id of list of kept scenes */
extern Widget      sceneBox;
extern long comp();

void dump_window(w, client, call)
Widget w;		/* dumpButton */
XtPointer client;
XtPointer call;
{	
FILE *fd;
XmString xstring;
struct SCENEINFO scene;
struct SCENEINFO *scene_found;
struct  MODEL model;
char    place,dchar,*sceneid,outfile[100];
long 	satno, sfsec = 0,
	smonth,sday,syear,sjday,shour,smin,ssec, 
	emonth,eday,eyear,ejday,ehour,emin,esec,
	status, pos, orbit, passdir1, passdir2, passdir3, dayn;
long	projcode,mid_x,mid_y,northlat,southlat,eastlong,westlong,dlat,dlong;
long	crossflg = FALSE,
	nlines;
double	poslng[4], neglng[4],	/* positive and negative longitudes */
	elev = 0.0;
float 	line,sample,scale;
double 	eqxing,dtime,lat1,lng1,lat2,lng2,lat3,lng3,lat4,lng4,sunzen,zero = 0.0;
double	clat,clng;

/*
  Get the scene id from the text field 
*/
XtVaGetValues(sceneBox,XmNtextString,&xstring,NULL);
XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&sceneid);
XmStringFree(xstring);
if ((strcmp(sceneid," ") == 0) || (strcmp(sceneid,"") == 0))
    {
    plot_err("No scene is displayed");
    return;
    }

/*
  Find information about the scene
*/
strcpy(scene.sceneid,sceneid);
XtFree(sceneid);
status = c_qfind(qtemp,&scene,&scene_found,comp,&pos);
if (pos == 0)
    {
    /* try a keep scene */
    status = c_qfind(qkeep,&scene,&scene_found,comp,&pos);
    }
if (pos <= 0)
    {
    plot_err("Could not find information about scene");
    return;
    }

/* 
  scene info was found; generate ps file 
*/
sscanf(scene_found->sceneid,"%c%c%02d%02d%02d%02d%02d%02d%02d",&place,
	   &dchar,&satno,&smonth,&sday,&syear,&shour,&smin,&ssec);
consec(&scene_found->passtart,&syear,&sjday,&smonth,&sday,&shour,&smin,&ssec);

if (scene_found->dtype == GAC)
    nlines = (scene_found->passtop - scene_found->passtart) * GAC_PER_SEC;
else
    nlines = (scene_found->passtop - scene_found->passtart) * LAC_PER_SEC;
model.ehour = model.emin = model.esec = model.efrac = 0;
status = c_initmod(&satno,&syear,&smonth,&sday,&shour,&smin,&ssec,
	&sfsec,&scene_found->dtype,&nlines,&model);
if (status != 0)
    {
    plot_err("error setting model information");
    return;
    }

/* start */
dtime = model.shour*3600 + model.smin*60 + model.ssec;
c_getorb(&dtime,&model,&orbit,&passdir1,&eqxing);

/* end */
dtime = model.shour*3600 + model.smin*60 + model.ssec
	    + (scene_found->passtop - scene_found->passtart);
c_getorb(&dtime,&model,&orbit,&passdir3,&eqxing);

/* middle */
dtime = model.shour*3600 + model.smin*60 + model.ssec
	    + ((scene_found->passtop - scene_found->passtart) / 2.0);
c_getorb(&dtime,&model,&orbit,&passdir2,&eqxing);

line = ((scene_found->passtop - scene_found->passtart)/2.0) * LAC_PER_SEC;
sample = 1024.0;
c_ls2ll(&line,&sample,&elev,&model,&lat1,&lng1,NULL);
c_daynite(&sjday,&syear,&dtime,&lat1,&lng1,&sunzen,&dayn);
consec(&scene_found->passtop,&eyear,&ejday,&emonth,&eday,&ehour,&emin,&esec);

/*
  Figure out the dimensions of the output space
*/
if (((passdir1 != passdir2) && (passdir2 != passdir3)) ||  /* cross both poles*/
     (scene_found->dtype == GAC))
    {
    projcode = GEO;
    mid_y = HEIGHT/2;		/* for space on top & bottom */
    mid_x = WIDTH/2;
    scale = (HEIGHT-120)/180;
    northlat = 90;
    southlat = -90;
    eastlong = 180;
    westlong = -180;
    dlat  = 180;
    dlong = 360;
    }
else if ((passdir1 == ASCENDING) && (passdir3 == DESCENDING))	/* north pole */
    {
    projcode = NORTH;
    mid_y = HEIGHT/2;
    mid_x = WIDTH/2;
    if (mid_x < mid_y)
	scale = mid_x - 120;
    else
	scale = mid_y - 120;
    northlat = 90;
    southlat = 0;
    eastlong = 180;
    westlong = -180;
    dlat  = 90;
    dlong = 360;
    }
else if ((passdir1 == DESCENDING) && (passdir3 == ASCENDING))	/* south pole */
    {
    projcode = SOUTH;
    mid_y = HEIGHT/2;
    mid_x = WIDTH/2;
    if (mid_x < mid_y)
	scale = mid_x - 120;
    else
	scale = mid_y - 120;
    northlat = 0;
    southlat = -90;
    eastlong = 180;
    westlong = -180;
    dlat  = 90;
    dlong = 360;
    }
else 							/* straight pass */
    {
    projcode = GEO;
    /*mid_y = (HEIGHT-30)/2;	/* for space at top only */
    mid_y = HEIGHT/2;		/* for space on top & bottom */
    mid_x = WIDTH/2;
    line = 1.0;
    sample = 1.0; 	c_ls2ll(&line,&sample,&elev,&model,&lat1,&lng1,NULL);
    sample = 2048.0; 	c_ls2ll(&line,&sample,&elev,&model,&lat2,&lng2,NULL);
    line = (scene_found->passtop - scene_found->passtart) * LAC_PER_SEC;
    sample = 1.0; 	c_ls2ll(&line,&sample,&elev,&model,&lat3,&lng3,NULL);
    sample = 2048.0; 	c_ls2ll(&line,&sample,&elev,&model,&lat4,&lng4,NULL);
    /* convert all to degrees */
    lat1 *= DEG_PER_RAD; lat2 *= DEG_PER_RAD;
    lat3 *= DEG_PER_RAD; lat4 *= DEG_PER_RAD;
    lng1 *= DEG_PER_RAD; lng2 *= DEG_PER_RAD;
    lng3 *= DEG_PER_RAD; lng4 *= DEG_PER_RAD;
    if (passdir1 == ASCENDING)
	{
	northlat = ceil ((lat3 > lat4 ? lat3 : lat4)/10) * 10;
	eastlong = ceil ((lng1 > lng3 ? lng1 : lng3)/10) * 10;
	southlat = floor((lat1 < lat2 ? lat1 : lat2)/10) * 10;
	westlong = floor((lng2 < lng4 ? lng2 : lng4)/10) * 10;
	}
    else
	{
	southlat = floor((lat3 < lat4 ? lat3 : lat4)/10) * 10;
	westlong = floor((lng1 < lng3 ? lng1 : lng3)/10) * 10;
	northlat = ceil ((lat1 > lat2 ? lat1 : lat2)/10) * 10;
	eastlong = ceil ((lng2 > lng4 ? lng2 : lng4)/10) * 10;
	}
    northlat = ceil (MAX4(lat1,lat2,lat3,lat4)/10) * 10;
    southlat = floor(MIN4(lat1,lat2,lat3,lat4)/10) * 10;
    eastlong = ceil (MAX4(lng1,lng2,lng3,lng4)/10) * 10;
    westlong = floor(MIN4(lng1,lng2,lng3,lng4)/10) * 10;

    dlat  = northlat - southlat;
    dlong = eastlong - westlong;

    /* find out if we cross the I.D.L */
    line = line / 2.0;
    if (scene_found->dtype == GAC) sample = 205.0;
    else sample = 1024.0;
    c_ls2ll(&line,&sample,&elev,&model,&clat,&clng,NULL);
    clng *= DEG_PER_RAD;
    if ((clng > eastlong) || (clng < westlong) || (dlong > 180))
	{
	crossflg = TRUE;
	/* take the smallest positive long and largest negative long */
	if (lng1 < 0.0)	{neglng[0] = lng1; poslng[0] =  180.0;}
	else 		{poslng[0] = lng1; neglng[0] = -180.0;}
	if (lng2 < 0.0)	{neglng[1] = lng2; poslng[1] =  180.0;}
	else 		{poslng[1] = lng2; neglng[1] = -180.0;}
	if (lng3 < 0.0)	{neglng[2] = lng3; poslng[2] =  180.0;}
	else 		{poslng[2] = lng3; neglng[2] = -180.0;}
	if (lng4 < 0.0)	{neglng[3] = lng4; poslng[3] =  180.0;}
	else 		{poslng[3] = lng4; neglng[3] = -180.0;}
     	eastlong=floor(MIN4(poslng[0],poslng[1],poslng[2],poslng[3])/10) * 10;
     	westlong=ceil (MAX4(neglng[0],neglng[1],neglng[2],neglng[3])/10) * 10;
	westlong += 360; /* new */
	/* dlong = 360 - eastlong + westlong; */
	dlong = westlong - eastlong;
	}

    if ((HEIGHT-120)/abs(dlat) < (WIDTH-120)/abs(dlong))
	/*scale = (HEIGHT-30) / abs(dlat);	/* space at top only */
	scale = (HEIGHT-120) / abs(dlat);	/* space on top & bottom */
    else
	/*scale = WIDTH / abs(dlong);*/
	scale = (WIDTH-120) / abs(dlong);
    }

/*
  Open the PostScript program file
*/
sprintf(outfile,"%s.ps",scene.sceneid);
fd = fopen(outfile,"w");

/*
  Write commands to draw header
*/
fprintf(fd,"%%!PS-Adobe-1.0\n");
fprintf(fd,"%% HEADER\n");
ps_header(fd,projcode,scale,mid_x,mid_y,dlat,dlong,satno,orbit,dayn,
	  sjday,smonth,sday,syear,shour,smin,ssec,
	  ejday,emonth,eday,eyear,ehour,emin,esec,eqxing);

/*
  Write commdands to draw lat/long with labels
  labels occur outside of border
*/
fprintf(fd,"%% LATLONG\n");
ps_latlong(fd,projcode,scale,mid_x,mid_y,dlat,dlong,northlat,southlat,
	   westlong,eastlong,crossflg);

/*
  Write commands to draw border
  and clip drawing area (after lat/long so labels show up)
*/
fprintf(fd,"%% BORDER\n");
ps_border(fd,projcode,dlat,dlong,scale,mid_x,mid_y);

/*
  Write commands to draw circle
  only if not on Int'l Date line
*/
fprintf(fd,"%% CIRCLE\n");
ps_circle(fd,projcode,scale,mid_x,mid_y,northlat,southlat,westlong,eastlong,
	  crossflg);

/*
  Write commands to draw linework
*/
fprintf(fd,"%% LINEWORK\n");
ps_lines(fd,projcode,scale,mid_x,mid_y,northlat,southlat,westlong,eastlong,
	 crossflg);

/*
  Write commands to draw pass
*/
fprintf(fd,"%% PASS\n");
ps_pass(fd,&model,passdir2,projcode,mid_x,mid_y,scale,
	northlat,southlat,westlong,eastlong,crossflg);

/*
  Close the PostScript file
*/
fprintf(fd,"showpage\n");
fclose(fd);

}
