#include <stdio.h>
#include <math.h>
#include "adaps.h"
#define WIDTH 792
#define HEIGHT 612
#define GEO 0
#define NORTH 1
#define SOUTH -1

void ps_pass(fd,model,passdir,projcode,mid_x,mid_y,scale,
		northlat,southlat,westlong,eastlong,crossflg)

FILE	*fd;			/* PostScript file	*/

struct MODEL *model;		/* model for this scene		*/

long	passdir,		/* pass direction		*/
	projcode,		/* projection (polar,geographic)*/
	mid_x,			/* middle x of drawing area	*/
	mid_y,			/* middle y of drawing area	*/
	northlat,		/* northernmost lat in scene	*/
	southlat,		/* southernmost lat in scene	*/
	westlong,		/* westernmost long in scene	*/
	eastlong,		/* easternmost long in scene	*/
	crossflg;			/* range of longitude		*/

float	scale;			/* scaling factor		*/
{	
char	str[10];		/* time label string		*/

long	x1,y1,			/* x,y coords			*/
	x2,			/* x coord of last point	*/
	ns,nl,			/* # lines & samples in scene	*/
	hr,min,sec,		/* time at a 1-minute line	*/
	line_inc,		/* line increment for drawing	*/
	samp_inc,		/* sample increment for drawing	*/
	midlat,midlong;		/* middle lat & long of scene	*/
	
float	midline,midsample,	/* middle line & sample of scene*/
	line,sample;		/* line & sample of scene	*/

double	lat,lng,		/* lat,long at a certain point	*/
	elev = 0.0;		/* elevation			*/

/*
  Set up gray lines and unclip the area in case labels hang over
*/
fprintf(fd,"0.5 setgray\n");
fprintf(fd,"grestore\n");

/*
  Set up nl and ns for the pass
*/
if (model->dtype == GAC)
    {
    ns = GAC_SAMPS;
    /* nl = length * GAC_PER_SEC; */
    nl = model->nlines;
    line_inc = 60 * GAC_PER_SEC;
    samp_inc = 51;
    }
else
    {
    ns = LAC_SAMPS + 1;		/* add 1 to make the loop come out even */
    /* nl = length * LAC_PER_SEC; */
    nl = model->nlines;
    line_inc = 60 * LAC_PER_SEC;
    samp_inc = 256;
    }
midsample = (float)((long)(ns / 2.0 + 0.5));
midline = nl / 2.0;
midlat = (northlat + southlat) / 2;
/* if (((eastlong - westlong) > 180) && ((eastlong - westlong) != 360)) */
/* if (crossflg)
    westlong = 360 + westlong;	/* indicates crossing Int'l Date Line */
midlong = (eastlong + westlong) / 2;

if (projcode != GEO)	/* polar views */
    {
    /*
      Set up the first point, start line nadir
    */
    line = 1.0;
    sample = midsample;
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    /* if ((projcode * lat) > 0.0) */
    x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
    fprintf(fd,"%d %d moveto\n",x1,y1);
    for (sample = midsample; sample <= ns; sample+=samp_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    	y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
    	fprintf(fd,"%d %d lineto\n",x1,y1);
        }
    sample = ns;
    for (line = 1.0; line <= nl+1; line+=line_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    	y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
    	fprintf(fd,"%d %d lineto\n",x1,y1);
        }
    line = nl;
    for (sample = ns; sample >= 1.0; sample-=samp_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    	y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
    	fprintf(fd,"%d %d lineto\n",x1,y1);
        }
    sample = 1.0;
    for (line = nl+1; line >= 1.0; line-=line_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    	y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
    	fprintf(fd,"%d %d lineto\n",x1,y1);
        }
    line = 1.0;
    for (sample = 1.0; sample <= midsample; sample+=samp_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    	y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
    	fprintf(fd,"%d %d lineto\n",x1,y1);
        }
    sample = midsample;
    for (line = 1.0; line <= nl+1; line+=line_inc)	/* nadir track */
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    	y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
    	fprintf(fd,"%d %d lineto\n",x1,y1);
        }
    /* make sure we finish up the line */
    /*
    line = nl;
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
    fprintf(fd,"%d %d lineto\n",x1,y1);
    */

    fprintf(fd,"stroke\n");

    /*
      Print the lines every minute across the pass
    fprintf(fd,"[2 2] 0 setdash\n");
    */
    fprintf(fd,"[1 2] 0 setdash\n");
    for (line = line_inc + 1; line <= nl+1; line+=line_inc)
        {
        sample = 1.0;
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    	y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng) + mid_y;
        fprintf(fd,"%d %d moveto\n",x1,y1);
        for (sample = samp_inc + 1; sample <= ns; sample+=samp_inc)
    	    {
    	    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	    x1=projcode*scale*cos(lat)*sin(lng) + mid_x;
    	    y1= -scale*sin(HALFPIE)*cos(lat)*cos(lng)+mid_y;
    	    fprintf(fd,"%d %d lineto\n",x1,y1);
    	    }
        fprintf(fd,"stroke\n");
        }
    }

else	/* plate carree passes */
    {
    /*
      Set up the first point, start line nadir
    */
    line = 1.0;
    sample = midsample;
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    fprintf(fd,"%d %d moveto\n",x1,y1);

    /*
      Go across top line from nadir to right side 
    */
    for (sample = midsample; sample <= ns; sample+=samp_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    	y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    	x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    	if (abs(x2 - x1) < mid_x)
    	    fprintf(fd,"%d %d lineto\n",x1,y1);
	else
	    {
    	    fprintf(fd,"stroke\n");
    	    fprintf(fd,"%d %d moveto\n",x1,y1);
	    }
	x2 = x1;
        }

    /*
      Go down side
    */
    sample = ns;
    for (line = 1.0; line < nl; line+=line_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    	y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    	x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    	if (abs(x2 - x1) < mid_x)
    	    fprintf(fd,"%d %d lineto\n",x1,y1);
	else
	    {
    	    fprintf(fd,"stroke\n");
    	    fprintf(fd,"%d %d moveto\n",x1,y1);
	    }
	x2 = x1;
        }
    /*
      Make sure we get all of the line
    */
    line = nl;
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    if (abs(x2 - x1) < mid_x)
    	fprintf(fd,"%d %d lineto\n",x1,y1);
    else
    	{
   	fprintf(fd,"stroke\n");
    	fprintf(fd,"%d %d moveto\n",x1,y1);
	}
    x2 = x1;

    /*
      Go across bottom line from right to left side 
    */
    for (sample = ns; sample >= 1.0; sample-=samp_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    	y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    	x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    	if (abs(x2 - x1) < mid_x)
    	    fprintf(fd,"%d %d lineto\n",x1,y1);
	else
	    {
    	    fprintf(fd,"stroke\n");
    	    fprintf(fd,"%d %d moveto\n",x1,y1);
	    }
	x2 = x1;
        }

    /*
      Go up side
    */
    sample = 1.0;
    for (line = nl; line > 1.0; line-=line_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    	y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    	x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    	if (abs(x2 - x1) < mid_x)
    	    fprintf(fd,"%d %d lineto\n",x1,y1);
	else
	    {
    	    fprintf(fd,"stroke\n");
    	    fprintf(fd,"%d %d moveto\n",x1,y1);
	    }
	x2 = x1;
        }
    /*
      Make sure we get all the way back up
    */
    line = 1.0;
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    if (abs(x2 - x1) < mid_x)
    	fprintf(fd,"%d %d lineto\n",x1,y1);
    else
	{
    	fprintf(fd,"stroke\n");
    	fprintf(fd,"%d %d moveto\n",x1,y1);
	}
    x2 = x1;

    /*
      Go across top from left to nadir
    */
    for (sample = 1.0; sample <= midsample; sample+=samp_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    	y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    	x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    	if (abs(x2 - x1) < mid_x)
    	    fprintf(fd,"%d %d lineto\n",x1,y1);
	else
	    {
    	    fprintf(fd,"stroke\n");
    	    fprintf(fd,"%d %d moveto\n",x1,y1);
	    }
	x2 = x1;
        }

    /*
      Go down nadir track
    */
    sample = midsample;
    for (line = 1.0; line < nl; line+=line_inc)
        {
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    	y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    	x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    	if (abs(x2 - x1) < mid_x)
    	    fprintf(fd,"%d %d lineto\n",x1,y1);
	else
	    {
    	    fprintf(fd,"stroke\n");
    	    fprintf(fd,"%d %d moveto\n",x1,y1);
	    }
	x2 = x1;
        }
    /*
      Make sure we finish up the line 
    */
    line = nl;
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    if (abs(x2 - x1) < mid_x)
    	fprintf(fd,"%d %d lineto\n",x1,y1);
    else
	{
    	fprintf(fd,"stroke\n");
    	fprintf(fd,"%d %d moveto\n",x1,y1);
	}
    x2 = x1;

    fprintf(fd,"stroke\n");

    /*
      Add the labels for the starting and ending times
      Don't do this for GAC
    */
    if (model->dtype == GAC) return;

    fprintf(fd,"[] 0 setdash\n");
    fprintf(fd,"0.0 setgray\n");
    if (passdir == DESCENDING)
        sample = ns;
    else
	sample = 1.0;

    line = 1.0;	/* start line */
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    fprintf(fd,"%d %d moveto\n",x1+5,y1);
    sprintf(str,"%02d:%02d:%02d", model->shour,model->smin,model->ssec);
    fprintf(fd,"(%s) show\n",str);
    fprintf(fd,"stroke\n");

    line = nl+1.0;	/* end line */
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    fprintf(fd,"%d %d moveto\n",x1+5,y1);
    sprintf(str,"%02d:%02d:%02d",model->ehour,model->emin,model->esec);
    fprintf(fd,"(%s) show\n",str);
    fprintf(fd,"stroke\n");

    /*
      Print the lines every minute across the pass
    fprintf(fd,"0.5 setgray\n");
    fprintf(fd,"[2 2] 0 setdash\n");
    */
    fprintf(fd,"[1 2] 0 setdash\n");

    for (line = line_inc + 1; line <= nl+1; line+=line_inc)
        {
        sample = 1.0;
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
        if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
        y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
        x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
        fprintf(fd,"%d %d moveto\n",x1,y1);

        for (sample = samp_inc + 1; sample <= ns; sample+=samp_inc)
    	    {
    	    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    	    if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
    	    y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
    	    x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;
    	    fprintf(fd,"%d %d lineto\n",x1,y1);
    	    }
        fprintf(fd,"stroke\n");
        }

    /*
      Print the labels for every minute across the pass
    */
    fprintf(fd,"[] 0 setdash\n");
    fprintf(fd,"0.0 setgray\n");

    hr = model->shour;
    min = model->smin;
    sec = model->ssec;

    if (passdir == ASCENDING)
    	sample = 1.0;
    else
    	sample = ns;
    for (line = line_inc + 1; line <= nl+1; line+=line_inc)
        {
	/* 
	  Set up the label
	*/
        min += 1;
        if (min >= 60) 
            {
            min -= 60; 
            hr +=1;
            }
        if (hr >= 24) 
            hr -= 24;

	/*
 	  Find where to put the label
	*/
    	c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
        if ((westlong > 180) && (lng < 0.0)) lng += PIE2;
        y1 = mid_y + (lat*DEG_PER_RAD - midlat) * scale;
        x1 = mid_x + (lng*DEG_PER_RAD - midlong) * scale;

	/*
 	  Label the line on the right side
	*/
    	fprintf(fd,"%d %d moveto\n",x1+5,y1);
    	sprintf(str,"%02d:%02d:%02d",hr,min,sec);
    	fprintf(fd,"(%s) show\n",str);
    	fprintf(fd,"stroke\n");
	}
    }

fprintf(fd,"[] 0 setdash\n");

}
