/******************************************************************************
FUNCTION:	findclosest

PURPOSE:	Find the time, and lat/long of the closest approach in 
		this orbit

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
1.0        4/87     Hollaren
2.0        6/89     LaVergne	
5.0	   10/92    T.Rockvam	Conversion to LAS 5.0
				Uses new model
*******************************************************************************/
#include <stdio.h>
#include "adaps.h"

int findclosest (model,stalat,stalon,offtime,applat,applon)

struct MODEL *model;		/* model information			*/

long	*offtime;		/* offset in time for closest approach	*/

double	stalat,			/* station latitude			*/
	stalon,			/* station longitude			*/
	*applat,		/* approach latitude			*/
	*applon;		/* approach longitude			*/
{
float	pasline,		/* line in pass at station lat/long	*/
	passamp,		/* sample in pass at station lat/long	*/
	nadirsamp;		/* nadir sample in pass			*/

double	time,			/* time of closest approach		*/
	truea,			/* true anomaly				*/
        rsat,          		/* dist of satellite from earth center  */
        thetas,        		/* true anomaly past equator            */
	anlng,			/* longitude of ascending node		*/
	inclin;			/* inclination angle			*/
double  elev = 0.0;             /* elevation                            */

long	zdot,			/* pass direction			*/
	eorbit,			/* orbit number				*/
	porbit;			/* orbit number				*/

/*
  Find the line/sample at the station lat/long
*/
c_ll2ls(&stalat,&stalon,&elev,model,&pasline,&passamp,NULL);

/*
  Find the lat/long at the nadir line/samp
*/
nadirsamp = LAC_NADIR;
c_ls2ll(&pasline,&nadirsamp,&elev,model,applat,applon,NULL);

/*
  Calculate the time offset at this point using the line #
*/
*offtime = (long)(pasline - 1.0)/LAC_PER_SEC;

return(0);
}
