/******************************************************************************
FUNCTION:	findpasses

PURPOSE:	Find passes that match our criteria for date/time and length

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
1.0        4/87     Hollaren
2.0        6/89     LaVergne
5.0        10/92    T.Rockvam   Rewritten with LAS5.0 and new model.
*******************************************************************************/
#include <stdio.h>
#include <Xm/Xm.h>
#include "adaps.h"
#include "scenes.h"

extern void makesecs();

int findpasses (report,qtemp,qkeep,satid,satnam,satno,
		targlat,targlon,sdate,stime,edate,etime,minsecs,varsecs,mxdist,
		opmode,plotflag,procMsgText)

long	qtemp,
	qkeep;

FILE	*report;		/* report of all passes			*/

char	*satnam,		/* satellite name			*/
	*satno;			/* satellite code			*/

long	satid,			/* satellite id				*/
	sdate[],		/* start date (mm dd yy)		*/
	stime[],		/* start time (hh mm ss)		*/
	edate[],		/* end date (mm dd yy)			*/
	etime[],		/* end time (hh mm ss)			*/
	minsecs,		/* minimum total pass time		*/
	varsecs,		/* time variance each side of center	*/
	opmode,			/* operation mode			*/
	plotflag;		/* which passes to keep track of	*/

float	targlat,		/* station latitude			*/
	targlon,		/* station longitude			*/
	mxdist;			/* maximum dist (meters) from station to sat*/

Widget  procMsgText;
{
struct SCENEINFO this_scene;

struct MODEL model;    		/* orbital parameters			*/

/************************************************************************
  TIME variables:
   Some variables are in total seconds from 1986 for a single point of 
   reference, and for ease in looping thru days.
   The rest of the variables are in total seconds on the day of the pass.  
   The model routines expect total seconds from the day of the pass, even 
   if the day of the pass is not the same as the day of the 
   ephemeris information.  						*/

double	dtotsecs,		/* double value of total seconds-used to*/
				/* increment time by exactly one orbit	*/
	approachsecs;		/* total seconds from start of scene 	*/
				/* jday to time of closest approach 	*/

long	totsecs,		/* total seconds at time of current orbit*/
	ototsecs,		/* original total seconds at user's start time*/
	stopsecs,		/* total seconds at user's end time	*/
	daystart,
	daystop,
	zerotime[3],		/* start of day	(00:00:00)		*/
	midnight[3],		/* end of day	(23:59:59)		*/
	offtime,		/* offset in time from current start 	*/
				/* time to closest approach 		*/
	closestsecs,		/* total seconds since 1986 to time of 	*/
				/* closest approach			*/
	startoff,		/* offset from closest approach to start*/
				/* of pass 				*/
	startsecs,		/* seconds from 1986 to start of pass	*/
	startdate[3],		/* start date	(month day year)	*/
	starttime[3],		/* start time	(hour min sec)		*/
	endoff,			/* time offset from closest approach to */
				/* end of pass 				*/
	endsecs,		/* seconds from 1986 to end of pass	*/
	enddate[3],		/* end date (month day year)		*/
	endtime[3],		/* end time (hour min sec)		*/
	lasttime = 0,		/* seconds from 1986 to last pass	*/
	time_in_view;		/* total seconds satellite is in cov circle*/
/************************************************************************/

long	passdir,		/* pass direction			*/
	jday,			/* julian day				*/
	year,			/* two-digit year			*/
	status,			/* function return status		*/
	nselected = 0,		/* number of passes selected		*/
	max_entity = 100,	/* maximum # of orbits to process at a  */
				/* time (7 days worth) 			*/
	lincnt = 25,		/* line count of current report page	*/
	flag = 2,		/* type of processing message 		*/
	lzero = 0,		/* long value of zero			*/
        nlines = 5400;          /* default nlines for 15-minute pass    */

double	tlat,			/* target latitude			*/
	tlon,			/* target longitude			*/
	passlat,		/* pass latitude of closest approach	*/
	passlon,		/* pass longitude of closest approach	*/
	sfsec = 0.0,		/* fractional seconds			*/
	eqxing,			/* equatorial crossing			*/
	angdist,		/* angular distance bet us & satellite	*/
	dist,			/* distance in meters bet us & satellite*/
	zero = 0.0;		/* double value of zero			*/

char	select[3];
char	nselect[20];		/* xx Scenes Selected			*/
XmString xstring;

long	comp();

/*
  All scenes are HRPT live acquisition
*/
this_scene.where = ' ';
this_scene.dtype = HRPT;
this_scene.station = EDC;
this_scene.opcode = opmode;
this_scene.hdtrfile = 99;
this_scene.prodflag = '.';
this_scene.status = 'P';

/*
  Set up limits for looping thru all passes of the date range regardless of 
  user's time range
*/
zerotime[0] = zerotime[1] = zerotime[2] = 0;
midnight[0] = 23;
midnight[1] = midnight[2] = 59;
makesecs(sdate,zerotime,&totsecs);
makesecs(edate,midnight,&stopsecs);
makesecs(sdate,stime,&ototsecs); /* save the original */

/*
  Set start time to the first orbit in this time range
  if it returns -1, totsecs was not reset, which is ok
*/
status = setfirsttime(&totsecs,satid,&model);
dtotsecs = (double)totsecs;

/*
  Loop until we hit all orbits
*/
while (totsecs <= stopsecs)
    {
    /*
      Convert total seconds to hour,min,sec
    */
    consec(&totsecs,&startdate[2],&jday,&startdate[0],&startdate[1],
	   &starttime[0],&starttime[1],&starttime[2]);

    /*
      Find the time limits for this day
    */
    makesecs(startdate,stime,&daystart);
    makesecs(startdate,etime,&daystop);
    /* if end time is after midnight, add one day to the time */
    if (daystop < daystart)
        daystop += SEC_PER_DAY;

    /*
      Set up model information for this time
    */
    year = startdate[2] % 100;
    model.ehour = model.emin = model.esec = model.efrac = 0;
    status = c_initmod(&satid,&year,&startdate[0],&startdate[1],
	 &starttime[0],&starttime[1],&starttime[2],&sfsec,&this_scene.dtype,
	 &nlines,&model);
    if (status != 0)
        return(-1);

    /*
      Find the time of closest approach for this orbit
    */
    tlat = (double)targlat;
    tlon = (double)targlon;
    status = findclosest(&model,tlat,tlon,&offtime,&passlat,&passlon);
    if (status == -1)
	{
        /*
          Add one orbit on to time
        */
        dtotsecs += PIE2 * HERG / model.mean_motion;
        totsecs = (long)(dtotsecs + 0.5);
	continue;
	}

    /* 
      getorb expects seconds from 0:00:00 of the scene's jday, 
      not ephem jday, if they are different 
    */
    approachsecs = model.shour*3600 + model.smin*60 + model.ssec + offtime;
    closestsecs = totsecs + offtime;

    /*
      Get orbit number for this time
    */
    c_getorb(&approachsecs,&model,&this_scene.orbit,&passdir,&eqxing);

    /*
      Find the distance from the station (at the point of closest approach)
    */
    calcdist(tlat,tlon,passlat,passlon,model.mean_motion,
	     &angdist,&dist,&time_in_view);

    /*
      If the pass is close enough and was not already found
      continue checking it out
    */
    if ((abs(lasttime - closestsecs) > 10) && (time_in_view > 0))
	{
	/*
	  Hang on the time of last pass in case the next orbit
	  returns the same one
	*/
	lasttime = closestsecs;

        /*
          Find the starting and ending time of the pass
        */
	findends(&model,closestsecs-varsecs,closestsecs+varsecs,
		 closestsecs,passlat,passlon,&startoff,&endoff);
        if (startoff == endoff)
            {
            /* Outside the U.S.-closest point is not in coverage circle */
            /* Start and end times would have been +/- half the  */
            /* time in view */
            startsecs = closestsecs - time_in_view/2;
            endsecs   = closestsecs + time_in_view/2;
            }
        else
            {
	    startsecs = closestsecs + startoff;
	    endsecs = closestsecs + endoff;
	    }
        consec(&startsecs,&startdate[2],&jday,&startdate[0],&startdate[1],
	   &starttime[0],&starttime[1],&starttime[2]);
        consec(&endsecs,&enddate[2],&jday,&enddate[0],&enddate[1],
	   &endtime[0],&endtime[1],&endtime[2]);
	this_scene.acqstart = this_scene.passtart = startsecs;
	this_scene.acqstop  = this_scene.passtop  = endsecs;

        /*
          Find the length of the pass; see if meets our length criteria
          If long enough, close enough, and in the user's time period,
          save record for schedule update
	  Make sure the scene at least partially falls in the user's
	  original time window
        */
        if (((endsecs - startsecs) >= minsecs) &&
             (dist <= mxdist) &&
             (startsecs <= daystop) && (endsecs >= daystart) &&
	     (endsecs > ototsecs))	/* make sure in original time window */
	    {	/* a SELECTED pass */
	    nselected += 1;
	    sprintf(nselect,"%2d",nselected);
    	    XtVaSetValues(procMsgText,XmNvalue,nselect,NULL);
	    strcpy(select,"**");
	    sprintf(this_scene.sceneid,"AH%02d%02d%02d%02d%02d%02d%02d",satid,
		    startdate[0],startdate[1],startdate[2],
		    starttime[0],starttime[1],starttime[2]);
	    this_scene.status = 'P';
	    qinsert(qkeep,&this_scene,comp);
	    if ((plotflag == 0) || (plotflag == 2))
		{
	     	qinsert(qtemp,&this_scene,comp);
		}
	    }
	else
	    {	/* a NON-SELECTED pass */
	    strcpy(select,"  ");
	    if ((plotflag == 1) || (plotflag == 2))
		{
	    	sprintf(this_scene.sceneid,"AH%02d%02d%02d%02d%02d%02d%02d",
		  satid,
		  startdate[0],startdate[1],startdate[2],
		  starttime[0],starttime[1],starttime[2]);
		this_scene.status = 'N';
	     	qinsert(qtemp,&this_scene,comp);
		}
	    }
	/*
	  Write the record to the list of all passes
	*/
        write_report(report,&lincnt,this_scene.orbit,satnam,satno,sdate,stime,
	    edate,etime,startdate,starttime,enddate,endtime,endsecs-startsecs,
	    closestsecs,time_in_view,targlat,targlon,dist,select,opmode);
	}

    /*
      Add one orbit on to time
    */
    dtotsecs += PIE2 * HERG / model.mean_motion;
    totsecs = (long)(dtotsecs + 0.5);
    }


return(0);
}
