/******************************************************************************
FUNCTION:	header

PURPOSE:	Print header information to the output report file

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
  5.0      10/92    T.Rockvam	Original development

*******************************************************************************/
#include <stdio.h>
#include "adaps.h"

void header(report,lincnt,satnam,satno,sdate,stime,edate,etime,targlat,targlon)

FILE	*report;		/* file descriptor for autosched report */
long	*lincnt;		/* current line on report page		*/

char 	*satnam,		/* satellite name ("NOAA-11")		*/
	*satno;			/* satellite id ("19531")		*/

long	sdate[3],		/* start date of interest		*/
	stime[3],		/* start time of interest		*/
	edate[3],		/* end date of interest			*/
	etime[3];		/* end time of interest			*/

float	targlat,targlon;	/* target lat,long			*/
{
static long	apgnum = 0;	/* page number 				*/

static char month[12][4] = {"JAN","FEB","MAR","APR","MAY","JUN",
		      "JUL","AUG","SEP","OCT","NOV","DEC"};

/*
  Check the line count to determine if the report should continue on a new page
  (only on subsequent pages, not the first one, where lincnt = 1)
*/
if ((*lincnt == 0) || (*lincnt >= 58))
    fprintf(report,"\f");

/*
  Add the standard EDC header
*/	
apgnum += 1;
sheadc(report,"EROS Data Center","Sioux Falls, South Dakota  57198",apgnum, 
	"AUTOGEN","AVHRR Data Acquisition and Processing System",
	"Automatic Pass Acquisition Schedule");

/*
  Tell the user the satellite name and ID
*/
fprintf(report,"\n Schedule of passes to be acquired for %s  ID = %s\n\n",
    satnam,satno);

/*
  Show the times that were entered
*/
fprintf(report," Period of interest:\n");
fprintf(report,"    Start %02d-%s-%d %02d:%02d:%02d GMT\n",sdate[1],month[sdate[0]-1],sdate[2],stime[0],stime[1],stime[2]);
fprintf(report,"    Stop  %02d-%s-%d %02d:%02d:%02d GMT\n\n",edate[1],month[edate[0]-1],edate[2],etime[0],etime[1],etime[2]);

/*
  Show where the antenna is
*/
if (targlat < 0.0)
    fprintf(report," Receiving station coordinates:   LAT. %4.2lf S\n",-targlat*DEG_PER_RAD);
else
    fprintf(report," Receiving station coordinates:   LAT. %4.2lf N\n", targlat*DEG_PER_RAD);

if (targlon < 0.0)
    fprintf(report,"                                  LON. %4.2lf W\n\n",-targlon*DEG_PER_RAD);
else
    fprintf(report,"                                  LON. %4.2lf E\n\n", targlon*DEG_PER_RAD);

/*
  Write out the headings
*/
fprintf(report,"   Orbit        Start       Start       End         End      Length     Closest      Time in   Target       Pass    Operation\n");
fprintf(report,"   Number       Date        Time        Date        Time     of Pass    Approach      View    Distance    Selected    Mode   \n");
fprintf(report,"   ______       _____       _____       ____        ____     _______    ________     _______  ________    ________  _________\n\n");

*lincnt = 19;

return;
}
