/******************************************************************************
FUNCTION:	write_report

PURPOSE:	Print pass information to the output report file

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
 1.0				Original development
 5.0	   10/92    T.Rockvam	Conversion to LAS 5.0
*******************************************************************************/
#include <stdio.h>
#include "autogen.h"

void write_report(report,lincnt,orbit,satnam,satno,intsdate,intstime,
	 intedate,intetime,sdate,stime,edate,etime,passlength,closestsecs,
	 viewtime,targlat,targlon,dist,select,opmode)

FILE	*report;		/* file descriptor for autosched report */
long	*lincnt;		/* current line in report page		*/

char 	*satnam,		/* satellite name ("NOAA-11")		*/
	*satno,			/* satellite id ("19531")		*/
	*select;		/* select indicator "  " or "**"	*/

long	orbit,			/* orbit number				*/
        intsdate[3],            /* interval start date (mm dd yy)       */
        intstime[3],            /* interval start time (hh mm ss)       */
        intedate[3],            /* interval end date (mm dd yy)         */
        intetime[3],            /* interval end time (hh mm ss)         */
	sdate[3],		/* start date of interest (mm dd yy)	*/
	stime[3],		/* start time of interest (hh mm ss)	*/
	edate[3],		/* end date of interest (mm dd yy)	*/
	etime[3],		/* end time of interest (hh mm ss)	*/
	passlength,		/* length of pass (seconds)		*/
	closestsecs,		/* time of closest approach (secs from 1986) */
	viewtime,		/* time in view (seconds)		*/
	opmode;			/* operations code			*/

float	targlat,targlon; 	/* target lat,long			*/

double	dist;			/* distance from target to closest point*/
{
long	length[3],		/* (hh mm ss) of pass length		*/
	closest[3],		/* (hh mm ss) of closest approach	*/
	view[3],		/* (hh mm ss) of view length		*/
	temp[4],		/* temp var for mm/dd/yy,jday		*/
	fsec;			/* fractional secs			*/

/*
  Check the line count to determine if the report should continue on a new page
*/
if ((*lincnt == 0) || (*lincnt >= 58))
    header(report,lincnt,satnam,satno,intsdate,intstime,intedate,intetime,
	   targlat,targlon);

/*
  Convert time of closest approach, length of pass, time in view to hour,min,sec
*/
consec(&closestsecs,&temp[0],&temp[1],&temp[2],&temp[3],
           &closest[0],&closest[1],&closest[2]);
c_cvtsec(&passlength,&length[0],&length[1],&length[2],&fsec);
c_cvtsec(&viewtime,&view[0],&view[1],&view[2],&fsec);

/*
  Write the record to the list of all passes
*/
fprintf(report,
"   %05d      %02d/%02d/%02d    %02d:%02d:%02d    %02d/%02d/%02d    %02d:%02d:%02d   %02d:%02d:%02d   %02d:%02d:%02d     %02d:%02d:%02d  %8.2lf      %s        %d\n", 
	orbit,
	sdate[0],sdate[1],sdate[2],stime[0],stime[1],stime[2],
	edate[0],edate[1],edate[2],etime[0],etime[1],etime[2],
	length[0],length[1],length[2],
	closest[0],closest[1],closest[2],
	view[0],view[1],view[2],dist/M_PER_MILE,select,opmode);
(*lincnt)++;

return;
}
