/******************************************************************************
FUNCTION:	callb_auto

PURPOSE:	callbacks for autogen
		get_autogen_info: popup the autogen dialog
		call_autogen:	get parameters from the dialog & call autogen
		cancel_auto:	cancel autogen
		update_times:	update default times based on satellite

PROGRAM HISTORY:
Version Date    Author		Change Request
------- -----   ---------	--------------
1.0	01/93	T.Rockvam	Original Development
1.5     09/00   D.Lloyd		Added defaults for NOAA 16
1.6     04/02   D.Lloyd		Added defaults for NOAA 17
1.7     11/04   D.Lloyd		Added defaults for NOAA 18
1.8     07/09   K.Vanumamalai   Added defaults for NOAA 19

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT		ADAPS
*******************************************************************************/
#include "xplot.h"

extern void plot_err();
extern void plot_pos();
extern void draw_circle();
extern void clear_info_area();
extern void normal_buttons();

void get_autogen_info(w, client, call)
Widget w;			/* autoButton */
XtPointer client;
XtPointer call;
{
XtManageChild(autogenFrame);
}

void call_autogen(w, client, call)
Widget w;			/* autoAcceptButton */
XtPointer client;
XtPointer call;
{
Widget  autoSatBox;
Widget  autoSatMenu;
Widget 	sceneList;
Widget  sat_widget;
XmString xstring;
Colormap the_cmap;
int 	the_screen;
XColor  color_actual,
        color_wanted;
WidgetList 	satButtons;
Cardinal 	numButtons;

struct SCENEINFO *scene_found;		/* pointer to scene info returned    */

long	i,
	plotflag = 3,	/* 0->selected 1->non-selected 2->both */
	satno = 0,
	status;

char	error[100],
	*latstring,
	*longstring, 
	*startdate,
	*starttime,
	*stopdate,
	*stoptime,
	*mintime,
	*vartime,
	*opmode,
	*mxdist,
	*repname;
float	targlat,
	targlon;

autoSatBox = (Widget)client;

/*
  Set the buttons back to normal mode if needed
*/
normal_buttons();

/*
  Get satellite from the option menu
  find the index in the list of menu buttons of the current option (menuHistory)
  this will be the index into all_satellites of the satellite number
*/
XtVaGetValues(autoSatBox,XmNsubMenuId,&autoSatMenu,NULL);
XtVaGetValues(autoSatMenu,XmNnumChildren,&numButtons,XmNchildren,&satButtons,
	NULL);
XtVaGetValues(autoSatMenu,XmNmenuHistory,&sat_widget,NULL);
for (i = 0; i < numButtons; i++)
    {
    if (satButtons[i] == sat_widget)
	{
	satno = all_satellite[i];
	break;
	}
    }

/*
  Get remaining autogen parameters from text fields in the autoFrame 
*/
latstring = XmTextGetString(targLatText);
longstring = XmTextGetString(targLongText);
startdate = XmTextGetString(staDateText);
starttime = XmTextGetString(staTimeText);
stopdate = XmTextGetString(stoDateText);
stoptime = XmTextGetString(stoTimeText);
mintime = XmTextGetString(minTimeText);
vartime = XmTextGetString(varTimeText);
mxdist = XmTextGetString(maxDistText);
opmode = XmTextGetString(opModeText);
repname = XmTextGetString(autoFnameText);

if (XmToggleButtonGetState(autoRSButton))
    plotflag = 0;
else if (XmToggleButtonGetState(autoRNButton))
    plotflag = 1;
else if (XmToggleButtonGetState(autoRBButton))
    plotflag = 2;

/*
  Remove all previous scenes in the list
*/
qclear(&qtemp);

/*
  autogen must convert string parameters to longs/floats
  Note: autogen loads up the scenes structure but doesn't add to the sceneBox 
	list; must do that upon return here
  pass down the name of the text field to update as processing goes on
*/
status = autogen(repname,qtemp,qkeep,satno,latstring,longstring,
	startdate,starttime,stopdate,stoptime,mintime,vartime,mxdist,opmode,
	plotflag,error,procMsgText);
if (status)	/* invalid status; get rid of processing message & issue error*/
    {
    XFlush(display);
    XtPopdown(procMsgShell);
    plot_err(error);
    return;
    }

XtUnmanageChild(autogenFrame);

/*
  Set the GC to blue for drawing the coverage circle
*/
the_screen  = DefaultScreen(display);
the_cmap = DefaultColormap(display,the_screen);
status = XAllocNamedColor(display,the_cmap,"Blue",
         &color_actual,&color_wanted);
if (status == 0)
    plot_err("Error allocating color blue");
XSetForeground(display,the_GC,color_actual.pixel);

/*
  Draw in a new coverage circle
*/
sscanf(latstring, "%f",&targlat);
sscanf(longstring,"%f",&targlon);
draw_circle(display,the_GC,world_pix,targlat,targlon,mid_x,mid_y);
XSetWindowBackgroundPixmap(display,XtWindow(drawingArea),world_pix);
XClearWindow(display,XtWindow(drawingArea));

/*
  Set the GC back to red for drawing pass plots
*/
status=XAllocNamedColor(display,the_cmap,"Red",&color_actual,&color_wanted);
if (status == 0)
    plot_err("Error allocating color red");
XSetForeground(display,the_GC,color_actual.pixel);

/*
  Get the name of the widget id in the list where we will add the scene id's
*/
sceneList = XmSelectionBoxGetChild(sceneBox,XmDIALOG_LIST);

/*
  Remove all previous scenes in the sceneList
  and add new ones
*/
XmListDeleteAllItems(sceneList);
xstring = XmStringCreateLtoR(" ",XmSTRING_DEFAULT_CHARSET);
XtVaSetValues(sceneBox,XmNtextString,xstring,NULL);
XmStringFree(xstring);

status = c_qfirst(qtemp,&scene_found);
while (scene_found)
    {
    xstring = XmStringCreateLtoR(scene_found->sceneid,XmSTRING_DEFAULT_CHARSET);
    XmListAddItem(sceneList,xstring,0);
    XmStringFree(xstring);
    status = c_qnext(qtemp,&scene_found);
    if (status != 0)
	{
    	plot_err("Error getting next scene");
	return;
	}
    }

/*
  Highlight & plot the first item in the list
*/
plot_pos(1);

/*
  Free all the strings we used
*/
XtFree(latstring); 	XtFree(longstring);
XtFree(startdate); 	XtFree(starttime);
XtFree(stopdate); 	XtFree(stoptime);
XtFree(mintime); 	XtFree(vartime);
XtFree(mxdist); 	XtFree(opmode);
XtFree(repname);
}

void cancel_auto(w, client, call)
Widget w;			/* autoCancelButton */
XtPointer client;
XtPointer call;
{
Widget w1;

w1 = (Widget)client;

XtUnmanageChild(w1);
}

void update_times(w, client, call)
Widget w;			/* autoN10Button,autoN11Button,autoN12Button*/
XtPointer client;
XtPointer call;
{
long	flag;

flag = (long)client;

if (flag == 10)
    {
    XmTextSetString(staTimeText,"12:00:00");
    XmTextSetString(stoTimeText,"16:30:00");
    }
else if (flag == 11)
    {
    XmTextSetString(staTimeText,"16:00:00");
    XmTextSetString(stoTimeText,"23:59:59");
    }
else if (flag == 12)
    {
    XmTextSetString(staTimeText,"12:00:00");
    XmTextSetString(stoTimeText,"16:30:00");
    }
else if (flag == 14)
    {
    XmTextSetString(staTimeText,"16:00:00");
    XmTextSetString(stoTimeText,"23:59:59");
    }
else if (flag == 15)
    {
    XmTextSetString(staTimeText,"12:00:00");
    XmTextSetString(stoTimeText,"17:00:00");
    }
else if (flag == 16)
    {
    XmTextSetString(staTimeText,"16:00:00");
    XmTextSetString(stoTimeText,"23:59:59");
    }
else if (flag == 17)
    {
    XmTextSetString(staTimeText,"13:00:00");
    XmTextSetString(stoTimeText,"20:59:59");
    }
else if (flag == 18)
    {
    XmTextSetString(staTimeText,"16:00:00");
    XmTextSetString(stoTimeText,"23:59:59");
    }
else if (flag == 19)
    {
    XmTextSetString(staTimeText,"13:00:00");
    XmTextSetString(stoTimeText,"20:59:59");
    }
else
    {
    XmTextSetString(staTimeText,"");
    XmTextSetString(stoTimeText,"");
    }
}
