/******************************************************************************
FUNCTION:	callb_misc

PURPOSE:	callbacks for miscellaneous functions
		quit:		free resources and exit
		load_help:	initiate a help process
		accept_count:	acknowledge the count of selected scenes from 
				autogen/recorder schedule
		help_reset:	notify me that there is no help process running

PROGRAM HISTORY:
Version Date    Author		Change Request
------- -----   ---------	--------------
1.0	01/93	T.Rockvam	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT		ADAPS
*******************************************************************************/
#include "xplot.h"

extern long comp();
extern void dom_header();
extern void domsat_report();
extern void plot_err();

void quit(w, client, call)
Widget w;			/* exitButton */
XtPointer client;
XtPointer call;
{
struct SCENEINFO *scene_found;
FILE	*schedfd;
FILE	*domfd;
long	status;
long	lincnt = 1;

/*
  Free the pixmaps
*/
XFreePixmap(display,northPole);
XFreePixmap(display,southPole);
XFreePixmap(display,world_pix);

/*
  Free the GC
*/
XFreeGC(display,the_GC);

/*
  Print out all the qkeep Domsat scenes to the domsat report
*/
status = 0;
c_qfirst(qkeep,&scene_found);
while (scene_found)
    {
    if (scene_found->where == 'D')
	{
	status = 1;
	break;
	}
    status = c_qnext(qkeep,&scene_found);
    if (status != 0)
    	{
    	plot_err("Error getting next scene");
    	return;
    	}
    }
if (status)
    {
    domfd = fopen("dsched.prt","w");
    dom_header(domfd,&lincnt);
    c_qfirst(qkeep,&scene_found);
    while (scene_found)
        {
        if (scene_found->where == 'D')
    	    domsat_report(domfd,&lincnt,scene_found);
        status = c_qnext(qkeep,&scene_found);
        if (status != 0)
	    {
    	    plot_err("Error getting next scene");
	    return;
	    }
        }
    fclose(domfd);
    }

/*
  Print out all the qkeep scenes to the schedit-add file
*/
if (c_qsize(qkeep) > 0)
    {
    schedfd = fopen("add.sch","w");
    status = c_qfirst(qkeep,&scene_found);
    while (scene_found)
        {
        savsched(schedfd,scene_found);
        status = c_qnext(qkeep,&scene_found);
        if (status != 0)
	    {
    	    plot_err("Error getting next scene");
	    return;
	    }
        }
    fclose(schedfd);
    }

/*
  Free the queues
*/
c_qdestroy(qtemp);
c_qdestroy(qkeep);

/*
  Close the display
*/
XCloseDisplay(display);

exit(0);
}

/* 
  Following code stolen from TIEPTS:
*/
void load_help(w, client, call)
Widget w;		/*fileSelectionBox,or loadHelpButton*/
XtPointer client;
XtPointer call;
{
FILE	*fp;		/* File pointer to temporary file.		      */

char    fname[20],      /* Temporary file name.                               */
        *getenv(),      /* Returns the value of an environment variable.      */
        *help_dir,      /* Directory containing help files.                   */
        help_file[100], /* Help file directory and name.                      */
        home_file[100], /* Home document directory and name.                  */
        *mosaic_exe;    /* Mosaic path and executable.                        */

/*
 * Get the value of the environment variable LASROOT in order to build
 * the directory location of the help files.
 */
help_dir = getenv("ADAPSDOC");
if (help_dir == '\0')
    {
    plot_err("Error retrieving help file directory");
    return;
    }

mosaic_exe = malloc(strlen("Mosaic")+2);
if (mosaic_exe == NULL)
    {
    plot_err("Insufficient memory available");
    return;
    }
sprintf(mosaic_exe,"Mosaic");

strcpy(help_file,help_dir);
strcpy(home_file,help_dir);
strcat(help_file,"/xsched/");
strcat(home_file,"/xsched/");
if (client == NULL)
    strcat(help_file,"xplot.html");
else
    strcat(help_file,client);
strcat(home_file,"xsched.html");

/*
 * Fork and exec a Mosaic process if one is not currently active.
 */
if ((!help_fork) && (help_pid != 0))
    {
    help_pid = fork();
    help_fork = TRUE;

    if (help_pid == 0)
        {
        execlp(mosaic_exe, "Mosaic", help_file, "-home", home_file, NULL);
        perror("Mosaic");
        exit(1);
        }
    }

/*
 * Create the file telling Mosaic which HTML file to display and send
 * Mosaic a signal indicating the file is ready for Mosaic to read.
 */
else if (help_pid != 0)
    {
    sprintf(fname,"/tmp/Mosaic.%d",help_pid);
    fp = fopen(fname,"w");
    if (fp == NULL)
        {
        plot_err("Error communicating with help process");
        return;
        }
    fprintf(fp,"goto\n");
    fprintf(fp,"file://localhost%s\n",help_file);
    fclose(fp);
    if (kill(help_pid, SIGUSR1) != 0)
        plot_err("Error communicating with help process");
    }
}

/* 
  Following code stolen from TIEPTS:
*/
void help_reset()
{
int     child_pid = 0,          /* Child process ID.                          */
        status;                 /* Status indicator.                          */

char    fname[25];              /* Temporary file name.                       */

signal (SIGCLD,SIG_IGN);

/*
 * Remove temporary file
 */
sprintf(fname,"/tmp/Mosaic.%d",help_pid);
unlink(fname);

/*
 * Wait for the child process to die so that we don't have defunct processes
 * hanging around.
 */
if (help_pid != -1)
    {
    child_pid = waitpid(help_pid,&status,WNOHANG);
    if (child_pid != 0)
        {
        help_pid = -1;
        help_fork = FALSE;
        }
    }

/*
 * Reset the signal handler.
 */
signal(SIGCLD,help_reset);
}


void accept_count(w, client, call)
Widget w;			/* okButton */
XtPointer client;
XtPointer call;
{
Widget w1;			/* shell widget */
w1 = (Widget)client;

XFlush(display);
XtPopdown(w1);
}
