/******************************************************************************
FUNCTION:	callb_new

PURPOSE:	callbacks for new passes
		get_new_pass_info: popup the new pass dialog, initializing info
				   from the sceneid in sceneBox if available
		no_new_pass:	   popdown new pass dialog
		create_new_pass:   get info from new pass dialog, add scene to 
				   list,queues,and plot

PROGRAM HISTORY:
Version Date    Author		Change Request
------- -----   ---------	--------------
1.0	01/93	T.Rockvam	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT		ADAPS
*******************************************************************************/
#include "xplot.h"

#define  NEXT_BUTTON XmDIALOG_CANCEL_BUTTON

extern long comp();
extern void makesecs();
extern void plot_err();
extern void consec();
extern void normal_buttons();

void get_new_pass_info(w, client, call)
Widget w;			/* sceneBox/newButton */
XtPointer client;
XtPointer call;
{
struct SCENEINFO scene;			/* information about one scene	     */
struct SCENEINFO *scene_found;		/* pointer to scene info returned    */

Widget	satMenu;
WidgetList 	satButtons;
Cardinal 	numButtons;
XmString xstring;
char	info[9];
char	*str;
char	place,
	dtype[5],
	dchar;
long 	i,
	hour,min,sec,
	month,day,year,
	pos,
	status = 0,
	satno;

/*
  First get any available information from the sceneid
*/
if (get_sceneid(scene.sceneid) == 0)
    status = sscanf(scene.sceneid,"%c%c%02d%02d%02d%02d%02d%02d%02d",&place,
	&dchar,&satno,&month,&day,&year,&hour,&min,&sec);

if (status == 9)	/* a complete scene id was entered */
    {
    /*
      Output and save preliminary information about the scene
    */
    if ((dchar == 'l') || (dchar == 'L'))
        strcpy(dtype,"LAC");
    else if ((dchar == 'g') || (dchar == 'G'))
        strcpy(dtype,"GAC");
    else
        strcpy(dtype,"HRPT");

    /* find the index into all_satellites, this is the button to set in
       the option menu */
    XtVaGetValues(nSatBox,XmNsubMenuId,&satMenu,NULL);
    XtVaGetValues(satMenu,XmNnumChildren,&numButtons,XmNchildren,&satButtons,
	NULL);
    for (i = 0; i < all_numsats; i++)
        {
        if (satno == all_satellite[i])
	    break;
        }
    XtVaSetValues(nSatBox,XmNmenuHistory,satButtons[i],NULL);

    XtVaGetValues(nDtypeBox,XmNsubMenuId,&satMenu,NULL);
    XtVaGetValues(satMenu,XmNnumChildren,&numButtons,XmNchildren,&satButtons,
	NULL);
    for (i = 0; i < numButtons; i++)
        {
	XtVaGetValues(satButtons[i],XmNlabelString,&xstring,NULL);
	XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&str);
	XmStringFree(xstring);
        if (strcmp(str,dtype) == 0)
	    {
	    XtFree(str);
	    break;
	    }
	else
	    XtFree(str);
	}
    XtVaSetValues(nDtypeBox,XmNmenuHistory,satButtons[i],NULL);

    sprintf(info,"%02d/%02d/%02d",month,day,year);
    XmTextSetString(nDateText,info);

    sprintf(info,"%02d:%02d:%02d",hour,min,sec);
    XmTextSetString(nTimeText,info);

    status = c_qfind(qtemp,&scene,&scene_found,comp,&pos);
    if (pos > 0)
        {
        sprintf(info,"%d",(scene_found->passtop-scene_found->passtart)/60);
        /* Add the length to the new pass length */
        XmTextSetString(nLengthText,info);
        }
    }

XtManageChild(newPassFrame);
}

void no_new_pass(w, client, call)
Widget w;                               /* newCancelButton */
XtPointer client;
XtPointer call;
{
XtUnmanageChild(newPassFrame);
}

void create_new_pass(w, client, call)
Widget w;			/* newOkButton */
XtPointer client;
XtPointer call;
{
struct SCENEINFO scene;                 /* information about one scene       */
struct SCENEINFO *scene_found;          /* pointer to scene info returned    */

XmString xstring;
Widget  satBox;
Widget  satMenu;
Widget  sat_widget;
Widget	sceneList;
Widget	sceneText;
WidgetList satButtons;
Cardinal numButtons;
struct 	MODEL model;
char	info[20];
char	*datestr;
char	*timestr;
char	*lengthstr;
char	*str;
long	dtype = HRPT,
	i,
	ldate,
	len,			/* string length		*/
	length,			/* length in seconds of new pass*/
	nlines,
	pos,
	passdir,
	pdate[3],
	ptime[3],
	jday,
	ehour,emin,esec,
	satno = 0,
	status,
	sfsec = 0;
double	eqxing;
double	zero = 0.0;
char	sep1,sep2;

sceneList = XmSelectionBoxGetChild(sceneBox,XmDIALOG_LIST);
sceneText = XmSelectionBoxGetChild(sceneBox,XmDIALOG_TEXT);

/*
  Set the buttons back to normal mode if needed
  and clear out the list
*/
if (XtIsManaged(XmSelectionBoxGetChild(sceneBox,NEXT_BUTTON)))
    {
    /* Clear out sceneList and scenes linked list */
    XmListDeleteAllItems(sceneList);
    qclear(&qtemp);
    }
normal_buttons();

/*
  Get the satellite:   find the index in the list of menu buttons of the 
  current option (menuHistory) this will be the index into all_satellites of 
  the satellite number
*/
satBox = (Widget)client;
XtVaGetValues(satBox,XmNsubMenuId,&satMenu,NULL);
XtVaGetValues(satBox,XmNmenuHistory,&sat_widget,NULL);
XtVaGetValues(satMenu,XmNnumChildren,&numButtons,XmNchildren,&satButtons,
    NULL);
for (i = 0; i < numButtons; i++)
    {
    if (satButtons[i] == sat_widget)
	{
        satno = all_satellite[i];
	break;
	}
    }

/*
  Get the data type
*/
XtVaGetValues(nDtypeBox,XmNsubMenuId,&satMenu,NULL);
XtVaGetValues(nDtypeBox,XmNmenuHistory,&sat_widget,NULL);
/* get the label, this will tell us the dtype */
XtVaGetValues(sat_widget,XmNlabelString,&xstring,NULL);
XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&str);
if (strcmp(str,"LAC") == 0) 	 dtype = LAC;
else if (strcmp(str,"GAC") == 0) dtype = GAC;
else 				 dtype = HRPT;
XtFree(str);

datestr = XmTextGetString(nDateText);
if (datestr != NULL)
    {
    status = sscanf(datestr,"%d%c%d%c%d",&pdate[0],&sep1,&pdate[1],&sep2,
		    &pdate[2]);
    if (status != 5)
	{
        status = sscanf(datestr,"%d",&ldate);
	if (status != 1) return;
        pdate[0] = ldate / 10000;
        pdate[1] = (ldate / 100) % 100;
        pdate[2] = ldate % 100;
	}
    pdate[2] = pdate[2] % 100;
    XtFree(datestr);
    }

timestr = XmTextGetString(nTimeText);
if (timestr != NULL)
    {
    status = sscanf(timestr,"%d%c%d%c%d",&ptime[0],&sep1,&ptime[1],&sep2,
		    &ptime[2]);
    if (status != 5)
	{
        status = sscanf(timestr,"%d",&ldate);
	if (status != 1) return;
        ptime[0] = ldate / 10000;
        ptime[1] = (ldate / 100) % 100;
        ptime[2] = ldate % 100;
	}
    XtFree(timestr);
    }

/* use default 15-minute pass */
length = 900;	
lengthstr = XmTextGetString(nLengthText);
if (lengthstr != NULL)
    {
    status = sscanf(lengthstr,"%d",&length);
    if (status == 1)		/* user entered minutes */
        length *= 60;		/* we want seconds      */
    XtFree(lengthstr);
    }

/*
  Set up model
*/
if (dtype == LAC)
    {
    sprintf(scene.sceneid,"AL%02d%02d%02d%02d%02d%02d%02d",satno,pdate[0],
	pdate[1],pdate[2],ptime[0],ptime[1],ptime[2]);
    nlines = length * LAC_PER_SEC;
    }
else if (dtype == GAC)
    {
    sprintf(scene.sceneid,"AG%02d%02d%02d%02d%02d%02d%02d",satno,pdate[0],
	pdate[1],pdate[2],ptime[0],ptime[1],ptime[2]);
    nlines = length * GAC_PER_SEC;
    }
else
    {
    sprintf(scene.sceneid,"AH%02d%02d%02d%02d%02d%02d%02d",satno,pdate[0],
	pdate[1],pdate[2],ptime[0],ptime[1],ptime[2]);
    nlines = length * LAC_PER_SEC;
    }
model.ehour = model.emin = model.esec = model.efrac = 0;
status = c_initmod(&satno,&pdate[2],&pdate[0],&pdate[1],&ptime[0],
	&ptime[1],&ptime[2],&sfsec,&dtype,&nlines,&model);
if (status != 0)
    {
    plot_err("Error setting model information");
    return;
    }
    
/*
  Find out the orbit number & direction
  getorb expects seconds from 0:00:00 of the scene jday, not ephem jday
*/
zero = model.shour*3600 + model.smin*60 + model.ssec + (length/2);
c_getorb(&zero,&model,&scene.orbit,&passdir,&eqxing);
XtUnmanageChild(newPassFrame);

/*
  Find the scene in the memory structure 
  if it exists update it; otherwise, add new
*/
status = c_qfind(qtemp,&scene,&scene_found,comp,&pos);
if (pos == 0)
    {
    len = strlen(scene.sceneid);
    c_low2up(scene.sceneid,&len);
    makesecs(pdate,ptime,&scene.passtart);
    scene.passtop = scene.passtart + length;
    makesecs(pdate,ptime,&scene.acqstart);

    if (dtype == LAC)
        scene.acqstop = scene.acqstart + (length/2);
    else if (dtype == GAC)
        scene.acqstop = scene.acqstart + (length/19);
    else
        scene.acqstop = scene.acqstart + length;
    scene.where = ' '; 		scene.dtype = dtype;
    scene.station = EDC; 	scene.opcode = 1;
    scene.hdtrfile = 99; 	scene.status = 'N';
    scene.prodflag = '.';
    qinsert(qtemp,&scene,comp);

    /* Add the scene to the list */
    xstring = XmStringCreateLtoR(scene.sceneid,XmSTRING_DEFAULT_CHARSET);
    XmListAddItem(sceneList,xstring,0);
    }
else
    {
    /* Update the scene info in the memory structure */
    makesecs(pdate,ptime,&scene_found->passtart);
    scene_found->passtop = scene_found->passtart + length;

    /* for LAC and GAC, transfer time is less than actual pass time */
    if (scene_found->dtype == LAC)
        scene_found->acqstop = scene_found->acqstart + (length/2);
    else if (scene_found->dtype == GAC)
        scene_found->acqstop = scene_found->acqstart + (length/19);
    else
        scene_found->acqstop = scene_found->acqstart + length;

    /* hang on to the end time, dtype, and station for output to info area */
    scene.passtop = scene_found->passtop;
    scene.dtype = scene_found->dtype;
    scene.station = scene_found->station;
    scene_found->orbit = scene.orbit;
    len = strlen(scene.sceneid);   /* use scene or scene_found (same sceneid) */
    c_low2up(scene.sceneid,&len);
    xstring = XmStringCreateLtoR(scene.sceneid,XmSTRING_DEFAULT_CHARSET);
    }

/* see if it is in qkeep too */
status = c_qfind(qkeep,&scene,&scene_found,comp,&pos);
if (pos > 0)
    {
    /* update only the fields that may have changed: stop times */
    makesecs(pdate,ptime,&scene_found->passtart);
    scene_found->passtop = scene_found->passtart + length;

    if (scene_found->dtype == LAC)
        scene_found->acqstop = scene_found->acqstart + (length/2);
    else if (scene_found->dtype == GAC)
        scene_found->acqstop = scene_found->acqstart + (length/19);
    else
        scene_found->acqstop = scene_found->acqstart + length;

    /* if it is scheduled already, set the scheduled flag in the qtemp version*/
    if (scene_found->status == 'P')
	{
	status = c_qfind(qtemp,&scene,&scene_found,comp,&pos);
	if (pos >= 0)
	    scene_found->status = 'P';
	}
    }

/* Add the scene to the sceneBox textfield */
/* use the TextField directly so we don't have to wait for an expose event */
XtVaSetValues(sceneText,XmNvalue,scene.sceneid,NULL);

/* Highlight & plot the new item in the list */
XClearWindow(display,XtWindow(drawingArea));
if (XmListItemExists(sceneList,xstring))
    XmListSelectItem(sceneList,xstring,True);
else
    plot_err("Error finding scene in list");
XmStringFree(xstring);

/* Output info about the scene */
sprintf(info,"%02d",satno);
XmTextSetString(satText,info);

if (scene.dtype == GAC)
    XmTextSetString(dtypeText,"GAC");
else if (scene.dtype == LAC)
    XmTextSetString(dtypeText,"LAC");
else
    XmTextSetString(dtypeText,"HRPT");

sprintf(info,"%02d/%02d/%02d",pdate[0],pdate[1],pdate[2]);
XmTextSetString(dateText,info);

consec(&scene.passtop,&pdate[0],&jday,&pdate[1],&pdate[2],&ehour,&emin,&esec);
sprintf(info,"%02d:%02d:%02d-%02d:%02d:%02d",ptime[0],ptime[1],ptime[2],
    ehour,emin,esec);
XmTextSetString(timeText,info);

sprintf(info,"%05d",scene.orbit);
XmTextSetString(orbitText,info);

if (scene.station == WALLOPS)
  XmTextSetString(stationText,"Wallops");
else if (scene.station == FAIRBANKS)
  XmTextSetString(stationText,"Gilmore");
else if (scene.station == EDC)
  XmTextSetString(stationText,"EDC");
else
  XmTextSetString(stationText,"");

}
