/******************************************************************************
FUNCTION:	callb_plot

PURPOSE:	callbacks for plotting scenes
		plot_pass: 	do the actual plotting
		plot_next: 	get the new first item in the sceneList & plot
		review_all: 	load scheduled passes into sceneList & plot the 
				first one

PROGRAM HISTORY:
Version Date    Author		Change Request
------- -----   ---------	--------------
1.0	01/93	T.Rockvam	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT		ADAPS
*******************************************************************************/
#include "xplot.h"

#define  SCHEDULE_BUTTON XmDIALOG_OK_BUTTON
#define  NEXT_BUTTON XmDIALOG_CANCEL_BUTTON

extern void plotit();
extern void plotpole();
extern long comp();
extern void consec();
extern void plot_err();
extern void plot_pos();
extern void clear_info_area();
extern void delete_scene();

void plot_pass(w, client, call)
Widget w;			/* sceneBox or sceneList or drawingArea */
XtPointer client;
XtPointer call;
{
struct SCENEINFO scene;			/* information about one scene	     */
struct SCENEINFO *scene_found;		/* pointer to scene info returned    */

XmString xstring;
struct 	MODEL model;
char	place,
	dchar;
char	info[20];
long	month,day,year,jday,
	hour,min,sec,
	ehour,emin,esec,
	length,
	satno,
	status,
	pos,
	sfsec = 0;
long	orbit,
	whichpole,
	nlines,
	passdir,
	passdir1,
	passdir2;
double	eqxing;
double	dtime;

/*
Code for redrawing on expose events
XmDrawingAreaCallbackStruct *dacs;
if (w == drawingArea)
    {
    dacs=(XmDrawingAreaCallbackStruct *)call;
    if (dacs->event->xexpose.count > 0)
	return;
    }
*/

/*
  Get the sceneid from the text in the sceneBox
*/
if (get_sceneid(scene.sceneid) != 0)
    return;

/*
  Find the correct SCENES record
*/
status = c_qfind(qtemp,&scene,&scene_found,comp,&pos);
if (pos <= 0)
    return;

/*
  See if it is scheduled or not
*/
if (scene_found->status == 'P')
    {
    XtVaSetValues(scheduledLabel, XmNlabelString, xstring=XmStringCreateLtoR(
        "    S C H E D U L E D", XmSTRING_DEFAULT_CHARSET),NULL);
    XmStringFree(xstring);
    }
else
    {
    XtVaSetValues(scheduledLabel, XmNlabelString, xstring=XmStringCreateLtoR(
        "N O T   S C H E D U L E D", XmSTRING_DEFAULT_CHARSET),NULL);
    XmStringFree(xstring);
    }
XtManageChild(scheduledLabel);

/*
  If the length is too long (more than 150 min) issue a warning
*/
length = scene_found->passtop-scene_found->passtart;
if (length > 9000)
    {
    plot_err("Scene is too long");
    return;
    }
if (length < 0)
    {
    plot_err("Scene has negative length");
    return;
    }

/*
  Set up model for this scene
*/
sscanf(scene_found->sceneid,"%c%c%02d%02d%02d%02d%02d%02d%02d",
	&place,&dchar,&satno,&month,&day,&year,&hour,&min,&sec);
consec(&scene_found->passtart,&year,&jday,&month,&day,&hour,&min,&sec);

if (scene_found->dtype == GAC)
    nlines = (scene_found->passtop - scene_found->passtart) * GAC_PER_SEC;
else
    nlines = (scene_found->passtop - scene_found->passtart) * LAC_PER_SEC;

model.ehour = model.emin = model.esec = model.efrac = 0;
status = c_initmod(&satno,&year,&month,&day,&hour,&min,&sec,
	 &sfsec,&scene_found->dtype,&nlines,&model);
if (status != 0)
    {
    plot_err("Error setting model information");
    return;
    }

/*
  Find out the direction at the center of the pass
  getorb expects seconds from 0:00:00 of the scene jday, not ephem jday
*/
dtime = model.shour*3600 + model.smin*60 + model.ssec + (length / 2.0);
c_getorb(&dtime,&model,&orbit,&passdir,&eqxing);

sprintf(info,"%02d",satno);
XmTextSetString(satText,info);

if (scene_found->dtype == GAC)
    XmTextSetString(dtypeText,"GAC");
else if (scene_found->dtype == LAC)
    XmTextSetString(dtypeText,"LAC");
else
    XmTextSetString(dtypeText,"HRPT");

sprintf(info,"%02d/%02d/%02d",month,day,year);
XmTextSetString(dateText,info);

consec(&scene_found->passtop,&year,&jday,&month,&day,&ehour,&emin,&esec);
sprintf(info,"%02d:%02d:%02d-%02d:%02d:%02d",hour,min,sec,ehour,emin,esec);
XmTextSetString(timeText,info);

sprintf(info,"%05d",scene_found->orbit);
XmTextSetString(orbitText,info);

if (scene_found->station == WALLOPS)
  XmTextSetString(stationText,"Wallops");
else if (scene_found->station == FAIRBANKS)
  XmTextSetString(stationText,"Gilmore");
else if (scene_found->station == EDC)
  XmTextSetString(stationText,"EDC");
else
  XmTextSetString(stationText,"");

/* 
  Plot the pass 
*/
XClearWindow(display,XtWindow(drawingArea));

/*
  Find the direction at start & end of pass
*/
dtime = model.shour*3600 + model.smin*60 + model.ssec;
c_getorb(&dtime,&model,&orbit,&passdir1,&eqxing);
dtime += length;
c_getorb(&dtime,&model,&orbit,&passdir2,&eqxing);
if ((passdir1 != passdir) || (passdir2 != passdir))
    {
    XtManageChild(poleFrame);
    if (passdir1 == ASCENDING)
	{
        XSetWindowBackgroundPixmap(display,XtWindow(poleDrawingArea),northPole);
	whichpole = 1;
	}
    else
	{
        XSetWindowBackgroundPixmap(display,XtWindow(poleDrawingArea),southPole);
	whichpole = -1;
	}
    /* clear the window to show the pixmap and get rid of previous passes */
    XClearWindow(display,XtWindow(poleDrawingArea));
    plotpole(poleDrawingArea,&model,whichpole);
    }
else
    {
    /*
      Hide the pole view if previously showing
    */
    XtUnmanageChild(poleFrame);
    }

plotit(&model);
}

void plot_next(w, client, call)
Widget w;			/* sceneBox */
XtPointer client;
XtPointer call;
{
struct SCENEINFO scene;			/* information about one scene	     */
struct SCENEINFO *scene_found;		/* pointer to scene info returned    */
long	status,
	pos;

/* get the scene id from the text field */
if (get_sceneid(scene.sceneid) != 0)
    return;

/* remove from list in memory */
status = c_qfind(qtemp,&scene,&scene_found,comp,&pos);
if (status != -1)
    c_qdelete(qtemp,pos);

/* clear the pass off the drawing area */
XClearWindow(display,XtWindow(drawingArea));

/* remove from sceneBox list and text */
/* also plot the scene which takes its place */
delete_scene(scene.sceneid);

}

void review_all(w, client, call)
Widget w;				/* reviewButton */
XtPointer client;
XtPointer call;
{
XmString xstring;
Widget	sceneList;
struct SCENEINFO *scene_found;		/* pointer to scene info returned    */
long	status;

/*
  Look through qkeep; if any are status 'P' we have some to be scheduled
*/
status = 0;
c_qfirst(qkeep,&scene_found);
while (scene_found)
    {
    if (scene_found->status == 'P')
        {
        status = 1;
        break;
        }
    status = c_qnext(qkeep,&scene_found);
    if (status != 0)
        {
        plot_err("Error getting next scene");
        return;
	}
    }
if (!status)
    {
    plot_err("No scenes currently scheduled");
    return;
    }

/*
  Get the widget id of the scenes list in sceneBox
*/
sceneList = XmSelectionBoxGetChild(sceneBox,XmDIALOG_LIST);

/*
  Set up for review mode
*/
XtUnmanageChild(XmSelectionBoxGetChild(sceneBox,SCHEDULE_BUTTON));
XtManageChild(XmSelectionBoxGetChild(sceneBox,NEXT_BUTTON));
XmProcessTraversal(XmSelectionBoxGetChild(sceneBox,NEXT_BUTTON),
        XmTRAVERSE_CURRENT);

/*
  Clear out sceneList and scenes linked list
*/
XmListDeleteAllItems(sceneList);
qclear(&qtemp);

/*
  Load sceneList and linked list with scenes from qkeep
*/
status = c_qfirst(qkeep,&scene_found);
while (scene_found)
    {
    if (scene_found->status == 'P')	/* only review chosen passes */
	{
	xstring=XmStringCreateLtoR(scene_found->sceneid,
	    XmSTRING_DEFAULT_CHARSET);
     	XmListAddItem(sceneList,xstring,0);
     	XmStringFree(xstring);
     	qinsert(qtemp,scene_found,comp);
	}
    status = c_qnext(qkeep,&scene_found);
    if (status != 0)
	{
    	plot_err("Error getting next scene");
	return;
	}
    }

/*
  Highlight & plot the first one
*/
plot_pos(1);

}
