/******************************************************************************
FUNCTION:	callb_rec

PURPOSE:	callbacks for recorder schedule
		load_rec_sch:	read recorder schedule and schedule based on 
				options is recorder dialog
		cancel_load:	cancel loading any recorder schedule
		get_recorder_file:	popup the recorder file dialog
		get_recorder_satellites:popup recorder satellites/dtypes dialog
		get_recorder_criteria:  popup the recorder criteria dialog
		add_rec_criteria:	add criteria to the list
		modify_rec_criteria:	modify an automatic scheduling criterion
		delete_rec_criteria:	delete a criterion


PROGRAM HISTORY:
Version Date    Author		Change Request
------- -----   ---------	--------------
1.0	01/93	T.Rockvam	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT		ADAPS
*******************************************************************************/
#include "xplot.h"

extern Widget   create_criteria_dialog();
extern Widget   create_sat_dialog();
extern long comp();
extern void consec();
extern void plot_err();
extern void plot_pos();
extern void clear_info_area();
extern void normal_buttons();

void load_rec_sch(w, client, call)
Widget w;			/* recDoneButton */
XtPointer client;		/* optionSelectionBox */
XtPointer call;
{
Widget	sceneList;
Widget	optionBox;
Widget	dtypeBox;
WidgetList dtypeButtons;
XtPointer temp_widget;
XmString xstring;
XmStringTable criteria_list;
Cardinal numButtons;

struct 	MODEL model;
struct 	SCENEINFO *keep_scene;		/* pointer to scene to keep	      */
struct 	SCENEINFO *temp_scene;		/* pointer to temp scene info returned*/

char	*filename;
char	*str;
char	*pattern;
char	*criteria;
char	c_dtype[6],
	c_station[8],
	c_direction[6];
char	nselect[20];
char	msg[200];
char	place,
	dchar;
long	pos;
long	count = 0;
long	status = 0;
long	dtypes;
long	i;
long	bad = 0;		/* number of scenes with invalid lengths */
long	sfsec = 0;
long	schedule_it;		/* flag to schedule a pass */
long	selected_sat;		/* satellite for this pass was scheduled */
long	cri_count = 0;		/* number of selection criteria	     */
long	*cri_satno,		/* satellite criteria	*/
	*cri_dtype,		/* data type criteria	*/
	*cri_station,		/* station criteria	*/
	*cri_direction;		/* direction criteria	*/
long    satno,passdir,
	year,jday,month,day,
	hour,min,sec,
	nlines;
double	eqxing,
	zero = 0.0;

/*
  Set the buttons back to normal mode if needed
*/
normal_buttons();

/*
  Get the criteria from the optionSelectionBox
*/
optionBox = (Widget)client;
XtVaGetValues(optionBox,XmNlistItemCount,&cri_count,XmNlistItems,&criteria_list,
    NULL);
cri_satno     = (long *)calloc(cri_count,sizeof(long));
cri_dtype     = (long *)calloc(cri_count,sizeof(long));
cri_station   = (long *)calloc(cri_count,sizeof(long));
cri_direction = (long *)calloc(cri_count,sizeof(long));
for (i = 0; i < cri_count; i++)
    {
    XmStringGetLtoR(criteria_list[i],XmSTRING_DEFAULT_CHARSET,&criteria);
    sscanf(criteria,"N%d %s %s %s",&cri_satno[i],c_dtype,c_station,c_direction);
    XtFree(criteria);
    if (strcmp(c_dtype,"HRPT") == 0)
	cri_dtype[i] = HRPT;
    else if (strcmp(c_dtype,"LAC") == 0)
	cri_dtype[i] = LAC;
    else if (strcmp(c_dtype,"GAC") == 0)
	cri_dtype[i] = GAC;
    else
	cri_dtype[i] = 0;

    if (strcmp(c_station,"Gilmore") == 0)
	cri_station[i] = FAIRBANKS;
    else if (strcmp(c_station,"Wallops") == 0)
	cri_station[i] = WALLOPS;
    else
	cri_station[i] = 0;

    if (strcmp(c_direction,"DESC") == 0)
	cri_direction[i] = DESCENDING;
    else if (strcmp(c_direction,"ASC") == 0)
	cri_direction[i] = ASCENDING;
    else
	cri_direction[i] = 0;
    }

/*
  Get the desired data types-these are all the dtype buttons
*/
XtVaGetValues(optionSelectionBox,XmNuserData,&temp_widget,NULL);
dtypeBox = (Widget)temp_widget;
XtVaGetValues(dtypeBox,XmNnumChildren,&numButtons,XmNchildren,&dtypeButtons,
    NULL);
for (dtypes = 0, i = 0; i < numButtons; i++)
    {
    XtVaGetValues(dtypeButtons[i],XmNlabelString,&xstring,NULL);
    XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&str);
    if (strcmp(str,"HRPT") == 0)
	dtypes += 100;
    else if (strcmp(str,"LAC") == 0)
	dtypes += 10;
    else if (strcmp(str,"GAC") == 0)
	dtypes += 1;
    XmStringFree(xstring);
    XtFree(str);
    }

/*
  Get the name of the widget id in the list where we will add the scene id's
*/
sceneList = XmSelectionBoxGetChild(sceneBox,XmDIALOG_LIST);

/*
  Remove all previous scenes in the list
*/
XmListDeleteAllItems(sceneList);
xstring = XmStringCreateLtoR(" ",XmSTRING_DEFAULT_CHARSET);
XtVaSetValues(sceneBox,XmNtextString,xstring,NULL);
XmStringFree(xstring);

/*
  Inform user of # of scenes selected
*/
sprintf(nselect,"%2d",0);
XmTextSetString(procMsgText,nselect);
XtPopup(procMsgShell,XtGrabNone);
forcedialog(procMsgShell);

/*
  Get the name of the file in the text string
*/
XtVaGetValues(fileSelectionBox,XmNtextString,&xstring,NULL);
XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&filename);
XmStringFree(xstring);

/*
  Get the file pattern text string
*/
XtVaGetValues(fileSelectionBox,XmNpattern,&xstring,NULL);
XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&pattern);
XmStringFree(xstring);
if (strcmp(pattern,"*.sch") == 0)
    {
    /* Remove all previous scenes in the list */
    qclear(&qtemp);
    /* Load all items from the schedule into the temp list */
    status=readrecorder(filename,qtemp,chosen_numsats,chosen_satellite,dtypes);
    if (status == -1)
	return;
    XtFree(filename);
    }
else
    {
    plot_err("Unknown file type");
    return;
    }
XtFree(pattern);

/* tjr */sprintf(msg,"Scenes with invalid lengths:\n");

status = c_qfirst(qtemp,&temp_scene);
while (temp_scene)
    {
    if (((temp_scene->passtop-temp_scene->passtart) < 0) ||
    	((temp_scene->passtop-temp_scene->passtart) > 9000))
	{
    	bad += 1;
	if (bad < 10)
	    {
	    strcat(msg,temp_scene->sceneid);
	    strcat(msg,"\n");
	    }
/* tjr */temp_scene->passtop = temp_scene->passtart + 900;
/* tjr */if (temp_scene->dtype == GAC)
/* tjr */    temp_scene->acqstop = temp_scene->acqstart + 48; /* 900/19 */
/* tjr */else if (temp_scene->dtype == LAC)
/* tjr */    temp_scene->acqstop = temp_scene->acqstart + 450;
/* tjr */else
/* tjr */    temp_scene->acqstop = temp_scene->acqstart + 900;
	}
    sscanf(temp_scene->sceneid,"%c%c%02d%02d%02d%02d%02d%02d%02d",&place,
	   &dchar,&satno,&month,&day,&year,&hour,&min,&sec);
    consec(&temp_scene->passtart,&year,&jday,&month,&day,&hour,&min,&sec);

    if (temp_scene->dtype == GAC)
	nlines = (temp_scene->passtop-temp_scene->passtart) * GAC_PER_SEC;
    else
	nlines = (temp_scene->passtop-temp_scene->passtart) * LAC_PER_SEC;
    model.ehour = model.emin = model.esec = model.efrac = 0;
    status = c_initmod(&satno,&year,&month,&day,&hour,&min,&sec,
	     &sfsec,&temp_scene->dtype,&nlines,&model);
    if (status != 0)
        {
        plot_err("Error setting model information");
        return;
        }
    /* getorb expects seconds from 0:00:00 of scene jday, not ephem jday */
    zero = model.shour*3600 + model.smin*60 + model.ssec 
	   + (temp_scene->passtop-temp_scene->passtart)/2.0;
    c_getorb(&zero,&model,&temp_scene->orbit,&passdir,&eqxing);
    /* First make sure we are working with this satellite */
    selected_sat = False;
    for (i = 0; i < chosen_numsats; i++)
	{
	if (satno == chosen_satellite[i])
	    {
	    selected_sat = True;
	    break;
	    }
	}
    /* Now check to schedule it */
    schedule_it = False;
    for (i = 0; i < cri_count; i++)
	{
	if ((satno == cri_satno[i]) &&
	   ((temp_scene->dtype == cri_dtype[i])||(cri_dtype[i] == 0)) && 
	   ((temp_scene->station==cri_station[i])||(cri_station[i]==0)) &&
	   ((passdir == cri_direction[i]) || (cri_direction[i] == 0)))
	    {
	    schedule_it = True;
	    break;
	    }
	}
    if (selected_sat)
	{
	if (schedule_it)
	    {
	    /* flag to automatically schedule it */
	    /* if it is already in qkeep just update it */
	    status = c_qfind(qkeep,temp_scene,&keep_scene,comp,&pos);
	    if (pos > 0)
		{
		keep_scene->status = 'P';
		keep_scene->opcode = 1;
		}
	    else
		{
	        temp_scene->status = 'P';
	        temp_scene->opcode = 1;
	        qinsert(qkeep,temp_scene,comp);
		}
	    count += 1;
	    sprintf(nselect,"%2d",count);
	    XmTextSetString(procMsgText,nselect);
	    }
    	else
    	    {
	    /* leave status as 'N' and opcode as 0 */
    	    /* place the scene id into the scene selection box */
      	    xstring=XmStringCreateLtoR(temp_scene->sceneid,
		    XmSTRING_DEFAULT_CHARSET);
    	    XmListAddItem(sceneList,xstring,0);
	    XmStringFree(xstring);
    
    	    /* place the scene info into the temp memory structure */
	    /* add to qkeep as a 'N'on-reception */
	    status = c_qfind(qkeep,temp_scene,&keep_scene,comp,&pos);
	    if (pos == 0)	/* not there; add it */
		{
	        temp_scene->status = 'N';
	        temp_scene->opcode = 0;
	        qinsert(qkeep,temp_scene,comp);
		}
    	    }
	}
    status = c_qnext(qtemp,&temp_scene);
    if (status != 0)
	{
    	plot_err("Error getting next scene");
	return;
	}
    }

XtUnmanageChild(recorderFrame);
XtDestroyWidget(XtParent(criteriaFrame));
criteriaFrame = NULL;

/*
  Highlight & plot the first item in the list if there still is one
*/
plot_pos(1);

free(cri_satno);
free(cri_dtype);
free(cri_station);
free(cri_direction);

/*
  If there were any errors, say so
*/
/*tjr*/if (bad > 0)
/*tjr*/    {
/*tjr*/    if (bad >= 10) strcat (msg,"+");
/*tjr*/    plot_err(msg);
/*tjr*/    return;
/*tjr*/    }
}

void cancel_load(w, client, call)
Widget w;			/* fileSelectionBox */
XtPointer client;
XtPointer call;
{
/* get rid of criteria form too if it is up */
if (criteriaFrame != NULL)
    {
    XtDestroyWidget(XtParent(criteriaFrame));
    criteriaFrame = NULL;
    }

if (satFrame != NULL)
    {
    XtDestroyWidget(XtParent(satFrame));
    satFrame = NULL;
    }

XtUnmanageChild(recorderFrame);
}

void get_recorder_file(w, client, call)
Widget w;			/* recButton */
XtPointer client;
XtPointer call;
{
XtManageChild(recorderFrame);
}

void get_recorder_criteria(w, client, call)
Widget w;			/* done button in satBox */
XtPointer client;		/* satBox 		 */
XtPointer call;
{
Widget 		satBox;
Widget 		dtypeBox;
WidgetList      satButtons;
XmString	xstring;
XtPointer	temp_widget;
Cardinal        numButtons;
long            i,j;
long		dtypes;
char		*str;

/*
  If a criteriaFrame panel is already up, destroy this one so we can make 
  another one
*/
if (criteriaFrame != NULL)
    {
    XtDestroyWidget(XtParent(criteriaFrame));
    criteriaFrame = NULL;
    }

satBox = (Widget)client;

/*
  Find the satellites that were chosen
  The list of satButtons is based on the original list of chosen_satellite.  
  Reset chosen_satellite to be only those buttons that are "on".
*/
XtVaGetValues(satBox,XmNnumChildren,&numButtons,XmNchildren,&satButtons,NULL);
for (i = 0, j = 0; i < numButtons; i++)
    {
    if (XmToggleButtonGetState(satButtons[i]))
        {
        chosen_satellite[j] = chosen_satellite[i];
        j++;
        }
    }
chosen_numsats = j;

/*
  Find the dtypes that were toggled
*/
dtypes = 0;

/*
  Get id's for the children (dtype buttons)
*/
XtVaGetValues(satBox,XmNuserData,&temp_widget,NULL);
dtypeBox = (Widget)temp_widget;
XtVaGetValues(dtypeBox,XmNnumChildren,&numButtons,XmNchildren,&satButtons,NULL);

/*
  Find the dtypes that were toggled
*/
dtypes = 0;
for (i = 0; i < numButtons; i++)
    {
    if (XmToggleButtonGetState(satButtons[i]))
        {
	/* get the label, this will tell us the dtype */
	XtVaGetValues(satButtons[i],XmNlabelString,&xstring,NULL);
	XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&str);
	if (strcmp(str,"HRPT") == 0)
	    dtypes += 100;
	else if (strcmp(str,"LAC") == 0)
	    dtypes += 10;
	else if (strcmp(str,"GAC") == 0)
	    dtypes += 1;
	XmStringFree(xstring);
	XtFree(str);
        }
    }

/*
  Generate the criteria dialog
*/
criteriaFrame = 
  create_criteria_dialog(MainWindow,chosen_satellite,chosen_numsats,dtypes);

XtDestroyWidget(XtParent(satFrame));
satFrame = NULL;
XtManageChild(criteriaFrame);
}

void get_recorder_satellites(w, client, call)
Widget w;			/* ok button in fileSelectionBox */
XtPointer client;
XtPointer call;
{
XmString	xstring;
long		dtypes;		/* LAC GAC HRPT */
char		*filename;

/*
  If a criteriaFrame panel is already up, destroy it
*/
if (criteriaFrame != NULL)
    {
    XtDestroyWidget(XtParent(criteriaFrame));
    criteriaFrame = NULL;
    }

/*
  If a satFrame panel is already up, destroy this one so we can make another one
*/
if (satFrame != NULL)
    {
    XtDestroyWidget(XtParent(satFrame));
    satFrame = NULL;
    }

/*
  Get the name of the file in the text string
*/
XtVaGetValues(fileSelectionBox,XmNtextString,&xstring,NULL);
XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&filename);
XmStringFree(xstring);

/*
  Scan the recorder schedule for satellites and data types
*/
scanrecorder(filename,&chosen_numsats,chosen_satellite,&dtypes);
XtFree(filename);

/*
  Generate the satellites dialog
*/
satFrame=create_sat_dialog(MainWindow,chosen_satellite,chosen_numsats,dtypes);

XtManageChild(satFrame);
}

void modify_rec_criteria(w, client, call)
Widget w;			/* optionSelectionBox */
XtPointer client;		/* satBox	      */
XtPointer call;
{
Widget 	satBox;			/* box of satellite buttons */
Widget 	optionList;		/* list of selection criteria */
WidgetList      satButtons;
Cardinal        numButtons;
XmString 	xstring;
long	i,
	satno;
char	dtype[6],
	station[8],
	direction[6];
char	*criteria;

satBox = (Widget)client;

/* get the parameters from the selectionbox */
XtVaGetValues(w,XmNtextString,&xstring,NULL);
XmStringGetLtoR(xstring,XmSTRING_DEFAULT_CHARSET,&criteria);
if (sscanf(criteria,"N%d %s %s %s",&satno,dtype,station,direction) != 4)
    return;
XtFree(criteria);

/* remove from selection box */
optionList = XmSelectionBoxGetChild(w,XmDIALOG_LIST);
XmListDeleteItem(optionList,xstring);
XmStringFree(xstring);

/* update the boxes */
/* satellite buttons are based on chosen_satellite */
XtVaGetValues(satBox,XmNnumChildren,&numButtons,XmNchildren,&satButtons,
        NULL);
for (i = 0; i < chosen_numsats; i++)
    {
    if (chosen_satellite[i] == satno)
	{
	XmToggleButtonSetState(satButtons[i],True,True);
	break;
	}
    }
if (strcmp(dtype,"HRPT") == 0)
    if (recHRPTButton != NULL)
    	XmToggleButtonSetState(recHRPTButton,True,True);
else if (strcmp(dtype,"LAC") == 0)
    if (recLACButton != NULL)
    	XmToggleButtonSetState(recLACButton,True,True);
else if (strcmp(dtype,"GAC") == 0)
    if (recGACButton != NULL)
    	XmToggleButtonSetState(recGACButton,True,True);
else
    {
    if (recHRPTButton != NULL)
    	XmToggleButtonSetState(recHRPTButton,False,False);
    if (recLACButton != NULL)
    	XmToggleButtonSetState(recLACButton,False,False);
    if (recGACButton != NULL)
    	XmToggleButtonSetState(recGACButton,False,False);
    }

if (strcmp(station,"Gilmore") == 0)
    XmToggleButtonSetState(recGCButton,True,True);
else if (strcmp(station,"Wallops") == 0)
    XmToggleButtonSetState(recWIButton,True,True);
else
    {
    XmToggleButtonSetState(recGCButton,False,False);
    XmToggleButtonSetState(recWIButton,False,False);
    }

if (strcmp(direction,"ASC") == 0)
    XmToggleButtonSetState(recASCButton,True,True);
else if (strcmp(direction,"DESC") == 0)
    XmToggleButtonSetState(recDESCButton,True,True);
else
    {
    XmToggleButtonSetState(recASCButton,False,False);
    XmToggleButtonSetState(recDESCButton,False,False);
    }
}

void delete_rec_criteria(w, client, call)
Widget w;			/* optionSelectionBox */
XtPointer client;
XtPointer call;
{
XmString xstring;
Widget optionList;

/* get the parameters from the selectionbox */
XtVaGetValues(w,XmNtextString,&xstring,NULL);

/* remove from selection box */
optionList = XmSelectionBoxGetChild(w,XmDIALOG_LIST);
if (XmListItemExists(optionList,xstring))
    XmListDeleteItem(optionList,xstring);
XmStringFree(xstring);
xstring = XmStringCreateLtoR(" ",XmSTRING_DEFAULT_CHARSET);
XtVaSetValues(w,XmNtextString,xstring,NULL);
XmStringFree(xstring);
}

void add_rec_criteria(w, client, call)
Widget w;			/* recAcceptButton */
XtPointer client;		/* satBox 	   */
XtPointer call;
{
Widget		optionList;
Widget		satBox;
WidgetList 	satButtons;
Cardinal 	numButtons;
XmString 	xstring;
char		criteria[24];
long		i;
long		satno=0;

satBox = (Widget)client;

/*
  Get the parameters from the toggle buttons 
  the list of satButtons is created parallel to chosen_satellite
*/
XtVaGetValues(satBox,XmNnumChildren,&numButtons,XmNchildren,&satButtons,NULL);
for (i = 0; i < numButtons; i++)
    {
    if (XmToggleButtonGetState(satButtons[i]))
        {
        satno = chosen_satellite[i];
        break;
        }
    }
if (satno == 0)
    return;
criteria[0] = '\0';
sprintf(criteria,"N%02d",satno);

if ((recHRPTButton != NULL) && (XmToggleButtonGetState(recHRPTButton)))
    strcat(criteria," HRPT");
else if ((recLACButton != NULL) && (XmToggleButtonGetState(recLACButton)))
    strcat(criteria," LAC");
else if ((recGACButton != NULL) && (XmToggleButtonGetState(recGACButton)))
    strcat(criteria," GAC");
else
    strcat(criteria," ???");

if (XmToggleButtonGetState(recGCButton))
    strcat(criteria," Gilmore");
else if (XmToggleButtonGetState(recWIButton))
    strcat(criteria," Wallops");
else
    strcat(criteria," ???");

if (XmToggleButtonGetState(recASCButton))
    strcat(criteria," ASC");
else if (XmToggleButtonGetState(recDESCButton))
    strcat(criteria," DESC");
else
    strcat(criteria," ???");

/* create a string for the optionBox list */
xstring = XmStringCreateLtoR(criteria,XmSTRING_DEFAULT_CHARSET);

/* add to the list */
optionList = XmSelectionBoxGetChild(optionSelectionBox,XmDIALOG_LIST);
XmListAddItem(optionList,xstring);
XmStringFree(xstring);
xstring = XmStringCreateLtoR(" ",XmSTRING_DEFAULT_CHARSET);
XtVaSetValues(optionSelectionBox,XmNtextString,xstring,NULL);
XmStringFree(xstring);
}
