/******************************************************************************
FUNCTION:	callb_sch

PURPOSE:	callbacks for scheduling
		add_to_sched: move pass from qtemp to qkeep
		drop_pass:    remove pass from qtemp, qkeep, sceneList

PROGRAM HISTORY:
Version Date    Author		Change Request
------- -----   ---------	--------------
1.0	01/93	T.Rockvam	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT		ADAPS
*******************************************************************************/
#include "xplot.h"

extern long comp();
extern void plot_err();
extern void clear_info_area();
extern void delete_scene();

void add_to_sched(w, client, call)
Widget w;			/* sceneBox */
XtPointer client;
XtPointer call;
{
struct SCENEINFO scene;			/* information about one scene	     */
struct SCENEINFO *scene_found;		/* pointer to scene info returned    */
struct SCENEINFO *scene_found2;		/* pointer to scene info returned    */

long	status,
	pos,
	pos2;

/* get the scene id from the text field */
if (get_sceneid(scene.sceneid) != 0)
    return;

/* remove from temp list in memory; add to keep list */
status = c_qfind(qtemp,&scene,&scene_found,comp,&pos);
if (pos > 0)
    {
    /* if it is already in qkeep update it; else add it */
    status = c_qfind(qkeep,scene_found,&scene_found2,comp,&pos2);
    if (pos2 > 0)
	{
	/* update only the fields that may have changed: start/stop times */
	/* if same sceneid, it is same start time */
	scene_found2->passtart = scene_found->passtart;	

	/* if length changed, so will the stop time */
	scene_found2->passtop = scene_found->passtop;	

	/* if same sceneid, dtype will be same */
	scene_found2->dtype = scene_found->dtype;	

	/* acqstop depends on length which may have changed */
	if (scene_found->dtype == LAC)
	    scene_found2->acqstop = scene_found->acqstart + 
		(scene_found2->passtop - scene_found2->passtart)/2;
	else if (scene_found->dtype == GAC)
	    scene_found2->acqstop = scene_found->acqstart + 
		(scene_found2->passtop - scene_found2->passtart)/19;
	else
	    scene_found2->acqstop = scene_found2->acqstart + 
		(scene_found2->passtop - scene_found2->passtart);

	/* orbit goes along with time */
	scene_found2->orbit = scene_found->orbit;	

	/* always P when scheduling */
        scene_found2->status = 'P';

	/* always 1 when scheduling */
        scene_found2->opcode = 1;
	}
    else
	{
	scene_found->status = 'P';
        scene_found->opcode = 1;
        qinsert(qkeep,scene_found,comp);
	}
    c_qdelete(qtemp,pos);
    }

/* clear the pass off the drawing area */
XClearWindow(display,XtWindow(drawingArea));

/* remove from sceneBox list and text */
/* also plot the scene that takes its place */
delete_scene(scene.sceneid);

}

void drop_pass(w, client, call)
Widget w;			/* sceneBox */
XtPointer client;
XtPointer call;
{
struct SCENEINFO scene;			/* information about one scene	     */
struct SCENEINFO *scene_found;		/* pointer to scene info returned    */
long	status;
long	pos;

/* get the scene id from the text field */
if (get_sceneid(scene.sceneid) != 0)
    return;

/* remove from list in memory */
status = c_qfind(qtemp,&scene,&scene_found,comp,&pos);
if ((status != -1) && (pos != 0))
    c_qdelete(qtemp,pos);

/* remove from list of saved passes */
status = c_qfind(qkeep,&scene,&scene_found,comp,&pos);
if ((status != -1) && (pos != 0))
    {
    if (scene_found->where == 'D')	/* this is domsat; leave in schedule */
	{
	scene_found->status = 'N';
	scene_found->opcode = 0;
	}
    else
	{
    	c_qdelete(qkeep,pos);
	}
    }

/* clear the pass off the drawing area */
XClearWindow(display,XtWindow(drawingArea));

/* remove from sceneBox list */
/* also plot the scene that takes its place */
delete_scene(scene.sceneid);

}
