/******************************************************************************
FUNCTION:	CONSEC

PURPOSE:	Converts the total seconds from 1986 to year, month, day, hour,
		minute, and seconds.
	  	if totsecs is negative, it means seconds BEFORE 00:00:00 on 
		1/1/86.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      12/86     P. Mumford         Initial development
  5.0      10/92     T. Rockvam        	Conversion to LAS-5.0
  5.1	   07/98     D. Lloyd		Modified for Y2K compliance

ALGORITHM REFERENCES	none
*******************************************************************************/
/*
  DLLOYD Y2K -- changed to 4-digit year
----------------------------------------*/
#define START_OF_TIME	1986
#define YEAR_OFFSET	1

void consec(totsecs,year,jday,month,day,hours,minutes,seconds)

long	*totsecs,		/* Total seconds from 1986		*/
	*year,			/* Year of totsecs			*/
	*jday,			/* Julian day of totsecs		*/
	*month,			/* Month of totsecs			*/
	*day,			/* Day of totsecs			*/
	*hours,			/* Hour of totsecs			*/
	*minutes,		/* Minutes of totsecs			*/
	*seconds;		/* Seconds of totsecs			*/
{
long	yr_size;		/* Number of days in the year		*/

/*
  Get number of minutes & seconds left over in totsecs
*/
*seconds = *totsecs;
*minutes = *seconds / 60;
*seconds %= 60;

/*
  Get number of hours & minutes left over
*/
*hours = *minutes / 60;
*minutes %= 60;

/*
  Get number of days & hours left over
*/
*jday = *hours / 24;
*hours %= 24;

/*
  Add 1 to jday to count the target date
*/
if (*totsecs >= 0)
    {
    *year = START_OF_TIME;
    *jday += 1;
    while (1)
        {
        if (c_isleap(year))
	    yr_size = 366;
        else
	    yr_size = 365;
        if (*jday <= yr_size)
	    break;
        *jday -= yr_size;
        *year += 1;
        }
    }
else
    {
    *year = START_OF_TIME-1;
    *jday -= 1;
    while (1)
        {
        if (c_isleap(year))
	    yr_size = -366;
        else
	    yr_size = -365;
        if (*jday >= yr_size)
	    break;
        *jday -= yr_size;
        *year -= 1;
        }

    if (c_isleap(year))
	yr_size = 366;
    else
	yr_size = 365;
    *jday = yr_size + *jday + 1;
    *seconds = 60 + *seconds;
    *minutes = 59 + *minutes;
    *hours = 23 + *hours;
    if (*seconds >= 60)
	{
	*seconds = *seconds - 60;
	*minutes = *minutes + 1;
	}
    if (*minutes >= 60)
	{
	*minutes = *minutes - 60;
	*hours = *hours + 1;
	}
    if (*hours >= 24)
	{
	*hours = *hours - 24;
	*jday = *jday + 1;
	}
    if (*jday > yr_size)
	{
	*jday = *jday - yr_size;
	*year = *year + 1;
	}
    }

/*
  Find the month and day
*/
c_julgre(year,jday,month,day);

/*
  DLLOYD Y2K -- convert to 2-digit year
----------------------------------------*/
*year %= 100;

return;
}
