/******************************************************************************
FUNCTION:	domsat_report

PURPOSE:	Print domsat pass information to a report file

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
 1.0				Original development
 5.0	   10/92    T.Rockvam	Conversion to LAS 5.0
*******************************************************************************/
#include "xplot.h"

void domsat_report(report,lincnt,this_scene)

FILE	*report;		/* file descriptor for autosched report */

long	*lincnt;		/* line count of current report page	*/

struct  SCENEINFO *this_scene;	/* this scene's information		*/
{
char	station[10],
	opcode[3],
	dtype[5];

long	asdate[3],
	astime[3],
	aetime[3],
	psdate[3],
	pstime[3],
	petime[3],
	jday;

void dom_header();
void consec();

/*
  Check the line count to determine if the report should continue on a new page*/
if ((*lincnt == 0) || (*lincnt >= 58))
    dom_header(report,lincnt);

/*
  Convert acquisition & pass times to hh:mm:ss
*/
consec(&this_scene->acqstop,&asdate[2],&jday,&asdate[0],&asdate[1],
	&aetime[0],&aetime[1],&aetime[2]);
consec(&this_scene->acqstart,&asdate[2],&jday,&asdate[0],&asdate[1],
	&astime[0],&astime[1],&astime[2]);
consec(&this_scene->passtop,&psdate[2],&jday,&psdate[0],&psdate[1],
	&petime[0],&petime[1],&petime[2]);
consec(&this_scene->passtart,&psdate[2],&jday,&psdate[0],&psdate[1],
	&pstime[0],&pstime[1],&pstime[2]);
if (this_scene->station == WALLOPS)
    strcpy(station,"Wallops  ");
else if (this_scene->station == FAIRBANKS)
    strcpy(station,"Fairbanks");
else
    strcpy(station,"EROS     ");

if (this_scene->status == 'P')
    strcpy(opcode,"**");
else
    strcpy(opcode,"  ");

if (this_scene->dtype == GAC)
    strcpy(dtype,"GAC ");
else if (this_scene->dtype == LAC)
    strcpy(dtype,"LAC ");
else
    strcpy(dtype,"HRPT");

/*
  Write the record to the report
*/
fprintf(report,"%s   N%c%c   %05d   %02d/%02d/%02d  %02d:%02d:%02d  %02d:%02d:%02d   %02d/%02d/%02d  %02d:%02d:%02d  %02d:%02d:%02d   %s   %s   %s\n",
	this_scene->sceneid,
	this_scene->sceneid[2],this_scene->sceneid[3],
	this_scene->orbit,
	asdate[0],asdate[1],asdate[2],
	astime[0],astime[1],astime[2], 
	aetime[0],aetime[1],aetime[2],
	psdate[0],psdate[1],psdate[2],
	pstime[0],pstime[1],pstime[2], 
	petime[0],petime[1],petime[2],
	station,dtype,opcode);

(*lincnt)++;

return;
}
