/******************************************************************************
FUNCTION:	plotit

PURPOSE:	plot the pass 

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
 1.0	   11/92    T.Rockvam   Conversion to LAS5.0/original development with X

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT         ADAPS

ALGORITHM REFERENCES    Xlib Referenc Manual, O'Reilly & Assoc.
*******************************************************************************/
#include "xplot.h"

void plotit(model)

struct MODEL *model;
{
Window	the_window;

float	nl,ns,
	midsample,
	midline;
float	line,
	sample;
double	lat,lng,
	elev = 0.0;
int	x1,y1,
	x2,y2;
long	line_inc,               /* line increment for drawing   */
        samp_inc;               /* sample increment for drawing */
void conv_pts();

the_window = XtWindow(drawingArea);

/*
  Set up nl and ns for the pass
*/
if (model->dtype == GAC)
    {
    ns = GAC_SAMPS;
    nl = model->nlines;
    line_inc = 60 * GAC_PER_SEC;
    samp_inc = 51;
    }
else
    {
    ns = LAC_SAMPS + 1;
    nl = model->nlines;
    line_inc = 60 * LAC_PER_SEC;
    samp_inc = 256;
    }
midsample = (float)((long)(ns / 2.0 + 0.5));
midline = nl / 2.0;

/*
  Set up the first point, start line nadir
*/
line = 1.0;
sample = midsample;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
conv_pts(scale,mid_x,mid_y,lat,lng,&x1,&y1);

/*
  Trace the start line from nadir to right side
*/
line = 1.0;
for (sample = midsample+samp_inc; sample <= ns; sample+=samp_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
    if (abs(x2 - x1) < mid_x)
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    x1 = x2;
    y1 = y2;
    }

/*
  Trace the right side from start line to end line
*/
sample = ns;
for (line = 1.0; line < nl; line+=line_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
    if (abs(x2 - x1) < mid_x)
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    x1 = x2;
    y1 = y2;
    }
/*
  Make sure we get all of the line
*/
line = nl;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
if (abs(x2 - x1) < mid_x)
    XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
x1 = x2;
y1 = y2;

/*
  Trace the end line from right side to left side
*/
line = nl;
for (sample = ns; sample > 1.0; sample-=samp_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
    if (abs(x2 - x1) < mid_x)
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    x1 = x2;
    y1 = y2;
    }

/*
  Trace the left side from end line to start line
*/
sample = 1.0;
for (line = nl; line > 1.0; line-=line_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
    if (abs(x2 - x1) < mid_x)
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    x1 = x2;
    y1 = y2;
    }
/*
  Make sure we get all of the line
*/
line = 1.0;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
if (abs(x2 - x1) < mid_x)
    XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
x1 = x2;
y1 = y2;

/*
  Trace the start line from left side to nadir
*/
line = 1.0;
for (sample = 1.0; sample < midsample; sample+=samp_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
    if (abs(x2 - x1) < mid_x)
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    x1 = x2;
    y1 = y2;
    }

/*
  Trace the nadir line from start line to end line
*/
sample = midsample;
for (line = 1.0; line < nl; line+=line_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
    if (abs(x2 - x1) < mid_x)
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    x1 = x2;
    y1 = y2;
    }
/*
  Make sure we get all of the line
*/
line = nl;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
if (abs(x2 - x1) < mid_x)
    XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
x1 = x2;
y1 = y2;


/*
  Draw an arrow indicating direction
*/
c_ls2ll(&midline,&midsample,&elev,model,&lat,&lng,NULL);
conv_pts(scale,mid_x,mid_y,lat,lng,&x1,&y1);

line = midline - (midsample/2.0);
sample = midsample - (midsample/2.0);
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
if (abs(x2 - x1) < mid_x)
    XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);

sample = midsample + (midsample/2.0);
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
conv_pts(scale,mid_x,mid_y,lat,lng,&x2,&y2);
if (abs(x2 - x1) < mid_x)
    XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
}
