/******************************************************************************
FUNCTION:	plotpole

PURPOSE:	plot the pass over the pole

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
 1.0	   11/92    T.Rockvam   Conversion to LAS5.0/original development with X

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT         ADAPS

ALGORITHM REFERENCES    Xlib Referenc Manual, O'Reilly & Assoc.
*******************************************************************************/
#include "xplot.h"

void plotpole(poleDrawingArea,model,whichpole)

Widget	poleDrawingArea;
struct 	MODEL *model;
long	whichpole;		/* 1 => North  -1 => South	*/
{
Window	the_window;
float	nl,ns,
	midsample;
float	line,
	sample;
float 	pole_scale;             /* scaling for drawing in the drawingArea */
double	lat,lng,
	elev = 0.0;
int	x1,y1,
	x2,y2;
long	line_inc,
	samp_inc;
Dimension window_width;

/*
  Set up scaling for drawing
*/
XtVaGetValues(poleDrawingArea,XmNwidth,&window_width,NULL);
pole_scale = window_width / 2.0;
the_window = XtWindow(poleDrawingArea);

/*
  Set up nl and ns for the pass
*/
if (model->dtype == GAC)
    {
    ns = GAC_SAMPS;
    nl = model->nlines;
    line_inc = 60 * GAC_PER_SEC;
    samp_inc = 51;
    }
else
    {
    ns = LAC_SAMPS+1;
    nl = model->nlines;
    line_inc = 60 * LAC_PER_SEC;
    samp_inc = 256;
    }
midsample = (float)((long)(ns / 2.0 + 0.5));

/*
  Set up the first point, start line nadir
*/
line = 1.0;
sample = midsample;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);

/*
  Convert to Orthographic coordinates
  centered at 90 lat, 0 lng
*/
x1 = (int)(whichpole * pole_scale * cos(lat) * sin(lng) + pole_scale);
y1 = (int)(pole_scale * sin(HALFPIE) * cos(lat) * cos(lng) + pole_scale);

/*
  Trace the start line from nadir to right side
*/
line = 1.0;
for (sample = midsample+1.0; sample <= ns; sample+=samp_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    x2 = (int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
    y2 = (int)(pole_scale*(sin(HALFPIE)* cos(lat) * cos(lng)) + pole_scale);
    if ((whichpole * lat) > 0.0)
	{
    	if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
            XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
	}
    x1 = x2;
    y1 = y2;
    }
/*
  Trace the right side from start line to end line
*/
sample = ns;
for (line = 1.0; line < nl; line+=line_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    x2=(int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
    y2=(int)(pole_scale* (sin(HALFPIE) * cos(lat) * cos(lng)) + pole_scale);
    if ((whichpole * lat) > 0.0)
	{
    	if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
            XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
	}
    x1 = x2;
    y1 = y2;
    }
/*
  Make sure we finish the line
*/
line = nl;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
x2 = (int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
y2 = (int)(pole_scale*(sin(HALFPIE)* cos(lat) * cos(lng)) + pole_scale);
if ((whichpole * lat) > 0.0)
    {
    if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    }
x1 = x2;
y1 = y2;

/*
  Trace the end line from right side to left side
*/
line = nl;
for (sample = ns; sample >= 1.0; sample-=samp_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    x2=(int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
    y2=(int)(pole_scale* (sin(HALFPIE) * cos(lat) * cos(lng)) + pole_scale);
    if ((whichpole * lat) > 0.0)
	{
    	if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
            XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
	}
    x1 = x2;
    y1 = y2;
    }

/*
  Trace the left side from end line to start line
*/
sample = 1.0;
for (line = nl; line > 1.0; line-=line_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    x2=(int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
    y2=(int)(pole_scale* (sin(HALFPIE) * cos(lat) * cos(lng)) + pole_scale);
    if ((whichpole * lat) > 0.0)
	{
    	if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
            XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
	}
    x1 = x2;
    y1 = y2;
    }
/*
  Make sure we finish the line
*/
line = 1.0;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
x2 = (int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
y2 = (int)(pole_scale*(sin(HALFPIE)* cos(lat) * cos(lng)) + pole_scale);
if ((whichpole * lat) > 0.0)
    {
    if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    }
x1 = x2;
y1 = y2;

/*
  Trace the start line from left side to nadir
*/
line = 1.0;
for (sample = 1.0; sample <= midsample; sample+=samp_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    x2=(int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
    y2=(int)(pole_scale* (sin(HALFPIE) * cos(lat) * cos(lng)) + pole_scale);
    if ((whichpole * lat) > 0.0)
	{
    	if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
            XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
	}
    x1 = x2;
    y1 = y2;
    }

/*
  Trace the nadir line from start line to end line
  Set up the first point, start line nadir
*/
line = 1.0;
sample = midsample;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
x1 = (int)(whichpole * pole_scale * cos(lat) * sin(lng) + pole_scale);
y1 = (int)(pole_scale * sin(HALFPIE) * cos(lat) * cos(lng) + pole_scale);

for (line = 1.0; line < nl; line+=line_inc) 
    {
    c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
    x2=(int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
    y2=(int)(pole_scale* (sin(HALFPIE) * cos(lat) * cos(lng)) + pole_scale);
    if ((whichpole * lat) > 0.0)
	{
    	if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
            XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
	}
    x1 = x2;
    y1 = y2;
    }
/*
  Make sure we finish the line
*/
line = nl;
c_ls2ll(&line,&sample,&elev,model,&lat,&lng,NULL);
x2 = (int)(whichpole * pole_scale * (cos(lat) * sin(lng)) + pole_scale);
y2 = (int)(pole_scale*(sin(HALFPIE)* cos(lat) * cos(lng)) + pole_scale);
if ((whichpole * lat) > 0.0)
    {
    if ((abs(x2 - x1) < pole_scale) && (abs(y2 - y1) < pole_scale))
        XDrawLine(display,the_window,the_GC,x1,y1,x2,y2);
    }

}
