/******************************************************************************
FUNCTION:       scanrecorder

PURPOSE:        Scan the recorder schedule to find out which satellites were 
		there

PROGRAM HISTORY:
Version    Date     Author      Change Request
-------    -----    ---------   --------------
1.1	   1/93	    T. Rockvam  Original Development

*******************************************************************************/
#include "xplot.h"

#define HRPT 0
#define LAC 1
#define GAC 2

FUNCTION scanrecorder(filename,chosen_numsats,chosen_satellite,dtypes)

char    *filename;			/* name of recorder file	*/

long	*chosen_numsats;		/* number of satellites in file	*/
long	chosen_satellite[];		/* satellites in file		*/
long	*dtypes;			/* data types in file		*/
{
FILE	*rfile;				/* recorder schedule file	*/

char	record[89],			/* a record from the rec. sched */
	action[4],			/* action of that record	*/
	datatype[5];			/* string data type		*/

long	i,				/* looping variable		*/
	satno,				/* satellite number		*/
	sat_there[MAXSATS],		/* flag for satellite present	*/
	dtype[3];			/* flag for HRPT, LAC, and GAC	*/

void plot_err();

/*
  Initialize each satellite and data type to False
*/
for (i = 0; i < MAXSATS; i++)
    sat_there[i] = False;
for (i = 0; i < 3; i++)
    dtype[i] = False;

/*
  Check out the file
*/
rfile = fopen(filename,"r");
if (rfile == NULL)
    {
    plot_err("Error opening recorder schedule");
    return(-1);
    }

/*
  Read all records
*/
while (TRUE)
    {
    /* break when EOF is hit */
    if (fgets(record,89,rfile) == NULL)
        break;

    if (strlen(record) < 45) continue;
    sscanf(&record[34],"%3s",action);
    if ( record[38] == 'S' )
	sscanf(&record[44],"%3s",datatype);
    else if ( record[38] == 'E' )
	sscanf(&record[42],"%3s",dtype);
    else
        sscanf(&record[38],"%3s",dtype);

    if (strcmp(datatype,"HRP") == 0)
	dtype[HRPT] = True;
    else if (strcmp(datatype,"LAC") == 0)
	dtype[LAC] = True;
    else if (strcmp(datatype,"GAC") == 0)
	dtype[GAC] = True;

    if ((strcmp(action,"REC") == 0) ||
        (strcmp(action,"STP") == 0) ||
        (strcmp(action,"PBK") == 0) ||
        (strcmp(action,"XFR") == 0) ||
        (strcmp(action,"AOS") == 0))		/* acquisition of signal */
	{
	sscanf(&record[23],"%2d",&satno);
	sat_there[satno-1] = True;
	}
    }

fclose(rfile);

/*
  Go thru the arrays
*/
*chosen_numsats = 0;
for (i = 0; i < MAXSATS; i++)
    {
    if (sat_there[i])
	{
	chosen_satellite[*chosen_numsats] = i + 1;
	(*chosen_numsats) += 1;
	}
    }

*dtypes = 0;
if (dtype[HRPT]) *dtypes += 100;
if (dtype[LAC])  *dtypes += 10;
if (dtype[GAC])  *dtypes += 1;

return(0);
}
