/******************************************************************************
FUNCTION:	set_up_background

PURPOSE:	set up the WDBII outline of the world in the drawing area

PROGRAM HISTORY:
Version		Date	Author		Change Request
-------    	-----	------		--------------
 1.0	   	11/92	T.Rockvam	Conversion to LAS-5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
 Must be run on a system that supports the X Windows environment

PROJECT		ADAPS

ALGORITHM REFERENCES	Xlib Referenc Manual, O'Reilly & Assoc.
*******************************************************************************/
#include "xplot.h"
#include "constants.h"
#include "WholeWorld.h"

void set_up_background(the_display,drawingArea)

Display *the_display;		/* the display		*/
Widget  drawingArea;		/* the drawing area	*/
{
Window the_window;
int the_screen;

XGCValues the_GC_values;           /* for handling the GC */

Colormap the_cmap;
XColor  color_actual,
        color_wanted;

long	nonfatal = 1,	/* nonfatal error status			     */
	status;		/* return status variable			     */

Dimension window_width,window_height;

void draw_circle();
void conv_pts();

/*
  Find out the size of the drawing area we are working with 
*/
the_screen  = DefaultScreen(the_display);
the_window = XtWindow(drawingArea);
the_cmap = DefaultColormap(the_display,the_screen);

/* we hope at this point width = 2*height !! */
XtVaGetValues(drawingArea,XmNwidth,&window_width,XmNheight,&window_height,NULL);

mid_x = window_width / 2.0;
mid_y = window_height / 2.0;
/*mid_y = (window_height + 40.0) / 2.0;	/* with annotation */

/* scale is same for height & width */
/*scale = window_height / PIE2;		/* mercator */
/*scale = window_width / 360.0;		/* plate carree */
scale = window_height / 180.0;		/* plate carree */
/* scale = (window_height - 40) / 180.0; /* plate carree with annotation */
/* scale = mid_y / 2.3034;		/* miller	*/

/*
  Set up an all-white GC
*/
the_GC = XCreateGC(the_display,the_window,None,&the_GC_values);
XSetBackground(the_display,the_GC,WhitePixel(the_display,the_screen));
XSetForeground(the_display,the_GC,WhitePixel(the_display,the_screen));

/*
  Get the pixmap from the include file; this will work ok as long as the
  size of drawingArea and the size of the saved bitmap agree
*/
world_pix = XCreatePixmapFromBitmapData(the_display,the_window,WholeWorld_bits,
    WholeWorld_width,WholeWorld_height,BlackPixel(the_display,the_screen),
    WhitePixel(the_display,the_screen),DefaultDepth(the_display,the_screen));

/*
  Set the GC to blue for drawing the coverage circle into the pixmap
*/
status = XAllocNamedColor(the_display,the_cmap,"Blue",
	 &color_actual,&color_wanted);
if (status == 0)
    c_errmsg("error allocating color blue","XPlot-color",&nonfatal);
XSetForeground(the_display,the_GC,color_actual.pixel);

/*
  Draw the EDC coverage circle into the pixmap
*/
draw_circle(the_display,the_GC,world_pix,43.74,-96.63,mid_x,mid_y);

/*
  Make the pixmap the window background
*/
XSetWindowBackgroundPixmap(the_display,the_window,world_pix);
XClearWindow(the_display,the_window);

/*
  Set the GC to red for drawing pass plots
*/
status=XAllocNamedColor(the_display,the_cmap,"Red",&color_actual,&color_wanted);
if (status == 0)
    c_errmsg("error allocating color red","XPlot-color",&nonfatal);
XSetForeground(the_display,the_GC,color_actual.pixel);

}
