/******************************************************************************
FUNCTION:	SHEADC

PURPOSE:	SHEADC prints a standard five-line heading in a file which
		will be printed.

  		NOTE:   The print file must be opened prior to calling SHEADC.
         		If the print file was opened with "open" or "creat"
			(which return a file descriptor instead of a file
			pointer), then the program must first call "fdopen"
			to get the file pointer to be used by SHEADC.
   
PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      01/86    K. Zanter		initail development
	   01/86    K. Zanter           PRF #3381 	 
  5.0      06/88    D. Hollaren		
  5.1      07/98    D.Lloyd		Modified for Y2K compliance

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
The page width of the printer must be between 80 and 132 characters.

ALGORITHM REFERENCES	none
  NOTE:  There should be five blank spaces at the beginnng of each line.
	 This margin was left in the event that the report would be bound.

  The following is a sample output heading (width = 80):

     Tue. July 7 1988          EROS Data Center            Report No. EXAMPLE
     Time 12:24        Sioux Falls, South Dakota  57198    Page 0001
                              Software Inventory
                                  LAS Modules

*******************************************************************************/
#include "xplot.h"
#include <time.h>

FUNCTION sheadc(fp,name,address,pgnum,repnum,title1,title2)
   
FILE  *fp;		 /* pointer to print file   	       */

long pgnum;	         /* page number		               */

char  *address;          /* facility's address                 */
char  *name;             /* facility's name		       */
char  *repnum;           /* report number/program name 	       */
char  *title1;           /* first user-supplied title line     */
char  *title2;           /* second user-supplied title line    */
{
char wday[6];	     	/* current weekday, Sun. - Sat.	       */
char mon[5];	     	/* current mont, Jan. - Dec.	       */

long addr_len;          /* length of "address"		       */
long name_len;          /* length of "name"		       */
long title1_len;        /* length of "title1"		       */
long title2_len;	/* length of "title2"		       */

long day;               /* current day of month, 1 - 31        */
long yr;		/* current year 	               */
long hour;              /* current hour of 24 hour clock       */
long minute;  	     	/* current minute of 24 hour clock     */

long width = 132;	/* page width of printer   	       */
long last_tab_stop;     /* position for last tab stop	       */
long mid;               /* position for middle of the page     */
long mid_start_pos;     /* starting position for middle words  */
long skip;		/* number of positions to skip over    */
long start_pos;         /* starting position                   */
long word_end;	     	/* position for end of middle word     */

long i;		     	/* loop counter		               */
long n;                 /* number of characters to be written  */

char fmt[10];           /* format specification		       */
  
struct tm *tm_ptr;	
long time();
long my_time;

/*
  Check validity of page numer
*/
if (pgnum > 9999)
    pgnum -= 9999;

/*
  Determine string lengths of input arguments
*/
name_len = strlen(name);
addr_len = strlen(address);
title1_len = strlen(title1);
title2_len = strlen(title2);

/*
  Get the local time
*/
my_time = time(0);
tm_ptr = localtime(&my_time);

/*
  Get the hours and minutes from the local time
*/
hour = tm_ptr->tm_hour;
minute = tm_ptr->tm_min;

/*
  Determine the day of the week
*/
switch (tm_ptr->tm_wday)
    {
    case 0 : strcpy(wday,"Sun. "); break;
    case 1 : strcpy(wday,"Mon. "); break;
    case 2 : strcpy(wday,"Tue. "); break;
    case 3 : strcpy(wday,"Wed. "); break;
    case 4 : strcpy(wday,"Thu. "); break;
    case 5 : strcpy(wday,"Fri. "); break;
    case 6 : strcpy(wday,"Sat. "); break;
    }

/*
  Determine the month
*/
switch (tm_ptr->tm_mon)
    {
    case 0 : strcpy(mon,"Jan "); break;
    case 1 : strcpy(mon,"Feb "); break;
    case 2 : strcpy(mon,"Mar "); break;
    case 3 : strcpy(mon,"Apr "); break;
    case 4 : strcpy(mon,"May "); break;
    case 5 : strcpy(mon,"Jun "); break;
    case 6 : strcpy(mon,"Jul "); break;
    case 7 : strcpy(mon,"Aug "); break;
    case 8 : strcpy(mon,"Sep "); break;
    case 9 : strcpy(mon,"Oct "); break;
    case 10: strcpy(mon,"Nov "); break;
    case 11: strcpy(mon,"Dec "); break;
    }

/*
  Get the day and year from the local time
*/
day = tm_ptr->tm_mday;
/*
  DLLOYD Y2K
-------------*/
yr = c_4digityr( tm_ptr->tm_year );

/*
  Compute middle of page
*/
mid = width / 2;

/*
  Compute position of last tab
*/
last_tab_stop = width - 20;

/*
  NOTE:  There should be five blank spaces at the beginnng of each line.
	 This margin was left in the event that the report would be bound.
  Print the first line
*/
fprintf (fp,"     %s%s",wday,mon);
if (day < 10)
   fprintf (fp," ");
fprintf (fp,"%d %d",day,yr);

mid_start_pos = mid - (name_len / 2);
if (mid_start_pos < 23) 
    mid_start_pos = 23;

skip = mid_start_pos - 21;
for (i = 1; i < skip; i++)
    fprintf(fp," ");

n = (last_tab_stop - 1) - mid_start_pos;
sprintf (fmt,"%%-.%ds",n);
fprintf (fp,fmt,name);

word_end = mid_start_pos + (name_len - 1);
if (word_end >= (last_tab_stop - 1))
    word_end = last_tab_stop - 2;

skip = last_tab_stop - word_end;
for (i = 1; i < skip; i++)
    fprintf (fp," ");

fprintf (fp,"Report No. %-.10s\n",repnum);

/*
  Print the second line
*/
fprintf (fp,"     Time ");
if (hour < 10)
   fprintf (fp," ");

fprintf (fp,"%d:%02d",hour,minute);

mid_start_pos = mid - (addr_len / 2);
if (mid_start_pos < 17) 
    mid_start_pos = 17;

skip = mid_start_pos - 15;
for (i = 1; i < skip; i++)
    fprintf(fp," ");

n = (last_tab_stop - 1) - mid_start_pos;
sprintf (fmt,"%%-.%ds",n);
fprintf (fp,fmt,address);

word_end = mid_start_pos + (addr_len - 1);
if (word_end >= (last_tab_stop - 1))
    word_end = last_tab_stop - 2;

skip = last_tab_stop - word_end;
for (i = 1; i < skip; i++)
    fprintf (fp," ");

fprintf (fp,"Page %04d\n",pgnum);

/*
  Print the third line
*/
start_pos = mid - (title1_len / 2);
if (start_pos < 6)
   start_pos = 6;

for (i = 1; i < start_pos; i++)
   fprintf (fp," ");

n = (width - start_pos) + 1;
sprintf (fmt,"%%-.%ds\n",n);
fprintf (fp,fmt,title1);
 
/*
  Print the fourth line
*/
start_pos = mid - (title2_len / 2);
if (start_pos < 6)
   start_pos = 6;

for (i = 1; i < start_pos; i++)
   fprintf(fp," ");

n = (width - start_pos) + 1;
sprintf(fmt,"%%-.%ds\n",n);
fprintf(fp,fmt,title2);
 
/*
  Print a blank line
*/
fprintf(fp ,"\n");

return;
}
