#ifdef LAS_LITTLE_ENDIAN
/* system has an i386 processor */

/*=============================================================================
Define the operation arrays.  These definitions are actually the array
initializers.  The arrays aren't declared until the swapping macros are
expanded at compile time.

An operation array is an integer array comprised of 3-element operations sets
and a count of the operations sets.  The first element in the array is the
count and the operation sets follow.  Each operation set contains three
elements in the following order:

    a. starting index (0-based), within the data buffer, of a block of words
    b. size, in bytes, of the data words
    c. number of words in the block

The definition of a block of words is: one or more words that have the same
size and are located in continuous memory locations.  Words may be of
differing data types as long as the word size is equal.
-----------------------------------------------------------------------------*/
#define TESTDUMMY \
    { \
    3,		 \
    9, 4, 2, \
    21, 8, 2, \
    0, 2, 3 \
    }


/* The following is the definition of the sharp minor frame format.  In the
absence of documentation of this format, it was determined by looking at the
values used from the input buffer in the file: acqio/sharp_mnr.c
-----------------------------------------------------------------------------*/
#define SHARPMINOROPARRAY \
    {             \
        8,        \
       24, 4,  1, \
    20520, 2,  1, \
    20524, 2,  1, \
    20552, 2, 50, \
    20676, 2,  1, \
    20722, 2, 30, \
    20652, 2,  5, \
    20666, 2,  3  \
    }


/* Start of NOAA KLM data record definitions.
==============================================================================*/

/* The following define is for endian swapping of NOAA KLM Data Set Header
(DSH) records.

The following commented table is from the NOAA KLM User's Guide and is
included here for reference.  The reference table is 1-based.  The data types
are 'c' character, 'u' unsigned integer and 'i' signed integer.  The first
column indicates which triplet of the define (1-based) takes care of the byte
swapping for the corresponding line.  The data size is in number of bytes.
Since these defines are to be used only for byte swapping and not
interpretation, the signed and unsigned data types, of the same size, are
combined into the same triplet where feasible.
------------------------------------------------------------------------------*/

/* Definition of NOAA KLM DSH record
  Start    End   DT  Word   Number
  Octet   Octet      Size  of Words
     1       3   c    3       1
     4       4   c    1       1
1    5      16   u    2       6
    17      22   i    2       3        zero fill
    23      64   c    42      1
    65      72   c    8       1
2   73      80   u    2       4
3   81      84   u    4       1
4   85      88   u    2       2
5   89      96   u    4       2
6   97     100   u    2       2
7  101     104   u    4       1
8  105     108   u    2       2
   109     116   i    2       4        zero fill
9  117     120   u    4       1
   121     122   i    2       1        zero fill
10 123     124   u    2       1
11 125     128   u    4       1
12 129     156   u    2       14
   157     160   i    4       1        zero fill
   161     168   c    8       1
   169     176   c    8       1
   177     186   i    2       5        zero fill
13 187     200   u    2       7
13 201     248   i    2       24
   249     256   i    4       2        zero fill
14 257     316   i    4       15
   317     328   i    4       3        zero fill
   329     336   c    8       1
15 337     340   u    2       2
15 341     342   i    2       1        zero fill
15 343     348   i    2       3
15 349     352   u    2       2
16 353     356   u    4       1
16 357     404   i    4       12
16 405     408   u    4       1
   409     424   i    4       4        zero fill
17 425     688   i    2       132
   689     EOR   i    4       ??       zero fill
                              EOR = 15872 for LAC 5-band packed datasets
                              EOR = 4608  for GAC 5-band packed datasets
                              EOR = 22528 for LAC 5-band unpacked datasets
                              EOR = 18432 for LAC 4-band unpacked datasets
                              EOR = 5632  for GAC 5-band unpacked datasets
                              and so on ...
--------------------------------------------------------------------------*/
#define KLMDATASETHEADER  \
    {                     \
     17,                  \
      4, 2,   6,          \
     72, 2,   4,          \
     80, 4,   1,          \
     84, 2,   2,          \
     88, 4,   2,          \
     96, 2,   2,          \
    100, 4,   1,          \
    104, 2,   2,          \
    116, 4,   1,          \
    122, 2,   1,          \
    124, 4,   1,          \
    128, 2,  14,          \
    186, 2,  31,          \
    256, 4,  15,          \
    336, 2,   8,          \
    352, 4,  14,          \
    424, 2, 132           \
    }

/*==============================================================================
  The following defines are for endian swapping of NOAA KLM data records.

The defines are all named 'KLMDATAREC_' followed by 3 characters.  The
first character is either an 'L' for LAC/HRPT data or a 'G' for GAC data.  The
second character is either 'P' for packed or 'U' for unpacked data.  The third
character denotes the number of bands 1 through 5.

The commented tables prior to each set of defines are from the NOAA KLM User's
Guide and are included here for reference.  These reference tables are 1-based.
The data types are 'c' character, 'u' unsigned integer and 'i' signed integer.
The first column indicates which triplet of the define (1-based) takes care of
the byte swapping for the corresponding line.  The data size is in number of
bytes.  Since these defines are to be used only for byte swapping and not
interpretation, the signed and unsigned data types, of the same size, are
combined into the same triplet where feasible.
------------------------------------------------------------------------------*/

/* Definition of invariant part of NOAA KLM data record.  The first section
of the data record is independent of the data record type and is defined here
to be included in each of the separate data types.

X   from -   to        type    size    num
1      1 -     2        u       2       1
1      3 -     4        u       2       1
1      5 -     6        u       2       1
1      7 -     8        i       2       1
2      9 -    12        u       4       1
3     13 -    14        u       2       1
      15 -    24        i       2       5       zero fill
4     25 -    28        u       4       1
4     29 -    32        u       4       1
5     33 -    38        u       2       3
5     39 -    40        u       2       1
      41 -    48        i       4       2       zero fill
6     49 -   300        i       4       63
6    301 -   312        i       4       3       zero fill
6    313 -   320        u       4       2
7    321 -   326        i       2       3
7    327 -   328        u       2       1
7    329 -   634        i       2       153
     635 -   640        i       2       3       zero fill
8    641 -  1048        i       4       102
    1049 -  1056        i       4       2       zero fill
9   1057 -  1262        u       2       103
    1263 -  1264        i       2       1       zero fill
----------------------------------------------------------*/
#define KLMDATARECINVAR \
        0, 2,     4,    \
        8, 4,     1,    \
       12, 2,     1,    \
       24, 4,     2,    \
       32, 2,     4,    \
       48, 4,    68,    \
      320, 2,   157,    \
      640, 4,   102,    \
     1056, 2,   103

/* Definition of NOAA KLM date record LAC packed 5-band
X   from -   to        type    size    num
10  1265 - 14920        u       4       3414
   14921 - 14928        i       4       2       zero fill
11 14929 - 14932        u       2       2
   14933 - 14944        i       4       3       zero fill
12 14945 - 14948        u       4       1
   14949 - 14970        u       1       22
   14971 - 14976        i       2       3       zero fill
13 14977 - 14984        u       4       2
14 14985 - 15496        u       2       256
   15497 - 15872        i       4       94      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_LP5 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
     1264, 4,  3414,   \
    14928, 2,     2,   \
    14944, 4,     1,   \
    14976, 4,     2,   \
    14984, 2,   256    \
    }

/* Definition of NOAA KLM date record LAC unpacked 5-band
X   from -   to        type    size    num
10  1265 - 21744        u       2       10240
   21745 - 21752        i       4       2       zero fill
11 21753 - 21756        u       2       2
   21757 - 21768        i       4       3       zero fill
12 21769 - 21722        u       4       1
   21773 - 21794        u       1       22
   21795 - 21800        i       2       3       zero fill
13 21801 - 21808        u       4       2
14 21809 - 22320        u       2       256
   22321 - 22528        i       4       52      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_LU5 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
     1264, 2, 10240,   \
    21752, 2,     2,   \
    21768, 4,     1,   \
    21800, 4,     2,   \
    21808, 2,   256    \
    }

/* Definition of NOAA KLM date record LAC unpacked 4-band
X   from -   to        type    size    num
10  1265 - 17648        u       2       8192
   17649 - 17656        i       4       2       zero fill
11 17657 - 17660        u       2       2
   17661 - 17672        i       4       3       zero fill
12 17673 - 17676        u       4       1
   17677 - 17698        u       1       22
   17699 - 17704        i       2       3       zero fill
13 17705 - 17712        u       4       2
14 17713 - 18224        u       2       256
   18225 - 18432        i       4       52      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_LU4 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
     1264, 2,  8192,   \
    17656, 2,     2,   \
    17672, 4,     1,   \
    17704, 4,     2,   \
    17712, 2,   256    \
    }

/* Definition of NOAA KLM date record LAC unpacked 3-band
X   from -   to        type    size    num
10  1265 - 13552        u       2       6144
   13553 - 13560        i       4       2       zero fill
11 13561 - 13564        u       2       2
   13565 - 13576        i       4       3       zero fill
12 13577 - 13580        u       4       1
   13581 - 13602        u       1       22
   13603 - 13608        i       2       3       zero fill
13 13609 - 13616        u       4       2
14 13617 - 14128        u       2       256
   14129 - 14336        i       4       52      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_LU3 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
     1264, 2,  6144,   \
    13560, 2,     2,   \
    13576, 4,     1,   \
    13608, 4,     2,   \
    13616, 2,   256    \
    }

/* Definition of NOAA KLM date record LAC unpacked 2-band
X   from -   to        type    size    num
10  1265 -  9456        u       2       4096
    9457 -  9464        i       4       2       zero fill
11  9465 -  9468        u       2       2
    9468 -  9480        i       4       3       zero fill
12  9481 -  9484        u       4       1
    9485 -  9506        u       1       22
    9507 -  9512        i       2       3       zero fill
13  9513 -  9520        u       4       2
14  9521 - 10032        u       2       256
   10033 - 10240        i       4       52      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_LU2 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
    1264, 2,   4096,   \
    9464, 2,      2,   \
    9480, 4,      1,   \
    9512, 4,      2,   \
    9520, 2,    256    \
    }

/* Definition of NOAA KLM date record LAC unpacked 1-band
X   from -   to        type    size    num
10  1265 -  5360        u       2       2048
    5361 -  5368        i       4       2       zero fill
11  5369 -  5372        u       2       2
    5373 -  5384        i       4       3       zero fill
12  5385 -  5388        u       4       1
    5389 -  5410        u       1       22
    5411 -  5416        i       2       3       zero fill
13  5417 -  5424        u       4       2
14  5425 -  5936        u       2       256
    5937 -  6144        i       4       52      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_LU1 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
    1264, 2,   2048,   \
    5368, 2,      2,   \
    5384, 4,      1,   \
    5416, 4,      2,   \
    5424, 2,    256    \
    }

/* Definition of NOAA KLM date record GAC packed 5-band
X   from -   to        type    size    num
10  1265 -  3992        u       4       682
    3993 -  4000        i       4       2       zero fill
11  4001 -  4004        u       2       2
    4005 -  4016        i       4       3       zero fill
12  4017 -  4020        u       4       1
    4021 -  4042        u       1       22
    4043 -  4048        i       2       3       zero fill
13  4049 -  4056        u       4       2
14  4057 -  4160        u       2       52
    4161 -  4608        i       4       112     zero fill
----------------------------------------------------------*/
#define KLMDATAREC_GP5 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
    1264, 4,    682,   \
    4000, 2,      2,   \
    4016, 4,      1,   \
    4048, 4,      2,   \
    4056, 2,     52    \
    }

/* Definition of NOAA KLM date record GAC unpacked 5-band
X   from -   to        type    size    num
10  1265 -  5354        u       2       2045
    5355 -  5360        i       2       3       zero fill
11  5361 -  5364        u       2       2
    5365 -  5368        i       4       1       zero fill
12  5369 -  5372        u       4       1
    5373 -  5394        u       1       22
    5395 -  5400        i       2       3       zero fill
13  5401 -  5408        u       4       2
14  5409 -  5512        u       2       52
    5513 -  5632        i       4       30      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_GU5 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
    1264, 2,   2045,   \
    5360, 2,      2,   \
    5368, 4,      1,   \
    5400, 4,      2,   \
    5408, 2,     52    \
    }

/* Definition of NOAA KLM date record GAC unpacked 4-band
X   from -   to        type    size    num
10  1265 -  4536        u       2       1636
    4537 -  4544        i       2       4       zero fill
11  4545 -  4548        u       2       2
    4549 -  4552        i       4       1       zero fill
12  4553 -  4556        u       4       1
    4557 -  4578        u       1       22
    4579 -  4584        i       2       3       zero fill
13  4585 -  4592        u       4       2
14  4593 -  4696        u       2       52
    4697 -  4816        i       4       30      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_GU4 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
    1264, 2,   1636,   \
    4544, 2,      2,   \
    4552, 4,      1,   \
    4584, 4,      2,   \
    4592, 2,     52    \
    }

/* Definition of NOAA KLM date record GAC unpacked 3-band
X   from -   to        type    size    num
10  1265 -  3718        u       2       1227
    3718 -  3720        i       2       1       zero fill
11  3721 -  3724        u       2       2
    3725 -  3728        i       4       1       zero fill
12  3729 -  3732        u       4       1
    3733 -  3754        u       1       22
    3755 -  3760        i       2       3       zero fill
13  3761 -  3768        u       4       2
14  3769 -  3872        u       2       52
    3873 -  3992        i       4       30      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_GU3 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
    1264, 2,   1227,   \
    3720, 2,      2,   \
    3728, 4,      1,   \
    3760, 4,      2,   \
    3768, 2,     52    \
    }

/* Definition of NOAA KLM date record GAC unpacked 2-band
X   from -   to        type    size    num
10  1265 -  2900        u       2       818
    2901 -  2904        i       2       2       zero fill
11  2905 -  2908        u       2       2
    2909 -  2912        i       4       1       zero fill
12  2913 -  2916        u       4       1
    2917 -  2938        u       1       22
    2939 -  2944        i       2       3       zero fill
13  2945 -  2952        u       4       2
14  2953 -  3056        u       2       52
    3057 -  3176        i       4       30      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_GU2 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
    1264, 2,    818,   \
    2904, 2,      2,   \
    2912, 4,      1,   \
    2944, 4,      2,   \
    2952, 2,     52    \
    }

/* Definition of NOAA KLM date record GAC unpacked 1-band
X   from -   to        type    size    num
10  1265 -  2082        u       2       409
    2083 -  2088        i       2       3       zero fill
11  2089 -  2092        u       2       2
    2093 -  2096        i       4       1       zero fill
12  2097 -  2100        u       4       1
    2101 -  2122        u       1       22
    2123 -  2128        i       2       3       zero fill
13  2129 -  2136        u       4       2
14  2137 -  2240        u       2       52
    2241 -  2360        i       4       30      zero fill
----------------------------------------------------------*/
#define KLMDATAREC_GU1 \
    {                  \
    14,                \
    KLMDATARECINVAR,   \
    1264, 2,    409,   \
    2088, 2,      2,   \
    2096, 4,      1,   \
    2128, 4,      2,   \
    2136, 2,     52    \
    }

/* END of NOAA KLM data record definitions.
==============================================================================*/

/* Start of NOAA data record definitions. (non-KLM data)
==============================================================================*/

/* The following define is for endian swapping of NOAA Data Set Header (DSH)

The following table is from the NOAA Polar Orbiter Data (POD)

    Table K-4. Format of Dataset Header Record before September 8, 1992.

      Byte #       # of Bytes                    Contents
        1              1       Spacecraft ID
        2              1       Data type
       3-8             6       Start time - time code from first frame of data
       9-10            2       Number of scans
      11-16            6       End time - time code from last frame of data
      17-23            7       Processing Block ID (ASCII)
        24             1       Ramp/Auto Calibration
      25-26            2       Number of data gaps
      27-32            6       DACS Quality
      33-34            2       Calibration Parameter ID
        35             1       DACS Status
      36-37            5       Spares, zero-filled
      39-40            2       4-digit year
      41-84            44      44 character data set name (EBCDIC)
      85-end        variable   Spares, zero-filled to size of data record

------------------------------------------------------------------------------*/
#define NOAADATASETHEADER \
    {           \
     5,         \
     2, 2, 1,   \
     4, 4, 1,   \
     8, 2, 2,   \
    12, 4, 1,   \
    38, 2, 1    \
    }

/*==============================================================================
  The following defines are for endian swapping of NOAA data records (non-KLM)

The defines are all named 'NOAADATAREC_' followed by 3 characters.  The
first character is either an 'L' for LAC/HRPT data or a 'G' for GAC sata.  The
second charater is either 'P' for packed or 'U' for unpacked data.  The third
character denotes the number of bands 1 through 5.

The commented tables prior to each set of defines have been extracted from the
NOAA Polar Orbiter User's Guide tables 3.2.2.1-1 and 3.1.2.1-1 and are 
included here for reference.  These reference tables are 1-based.
------------------------------------------------------------------------------*/

/* Definition of invariant part of the NOAA data record.  This first section
of the data record is independenat of the data record type and is defined here
to be included in each of the separate data types.

X   from -    to       size        num
1      1 -      2       2           1
2      3 -      8       2           3
       9 -     12       2           2
3     12 -     52       4          10
      53 -              1           1
      54 -    104       1          51
4    105 -    308       2         102
5    309 -    448       4          35 this is 103 words of minor frame packed
------------------------------------------------------------------------------*/
#define NOAADATARECINVAR \
       0,  2,   2,       \
       4,  4,   1,       \
      12,  4,  10,       \
     104,  2, 102,       \
     308,  4,  35

/* Definition of NOAA data record LAC packed 5-band
X   from -    to       size        num
6    449 -  7400        4         1738
6      1 -  6704        4         1676 start of second record same one for us
    6705 -  6724        ?            ? 51 decimal portion of zenith in 20 bytes
    6725 -  6726        2            1
    6727 -  7400        1          674 spare bytes
------------------------------------------------------------------------------*/
#define DATAREC_LP5   \
    {                 \
     6,               \
    NOAADATARECINVAR, \
    448,  4,  3414    \
    }

/* Definition of NOAA data record LAC unpacked 5-band
X   from -    to       size        num
6    449 - 20928        2         10240
-----------------------------------------------------*/
#define DATAREC_LU5   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  2, 10240    \
    }

/* Definition of NOAA data record LAC unpacked 4-band
X   from -    to       size        num
6    449 - 16928        2         8192
-----------------------------------------------------*/
#define DATAREC_LU4   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  2, 8192     \
    }

/* Definition of NOAA data record LAC unpacked 3-band
X   from -    to       size        num
6    449 - 12736        2         6144
-----------------------------------------------------*/
#define DATAREC_LU3   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  2, 6144     \
    }

/* Definition of NOAA data record LAC unpacked 2-band
X   from -    to       size        num
6    449 -  8640        2         4096
-----------------------------------------------------*/
#define DATAREC_LU2   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  2, 4096     \
    }

/* Definition of NOAA data record LAC unpacked 1-band
X   from -    to       size        num
6    449 -  4544        2         2048
-----------------------------------------------------*/
#define DATAREC_LU1   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  2, 2048     \
    }

/* Definition of NOAA data record GAC packed 5-band
X   from -    to       size        num
6    449 -  3176        4          682
-----------------------------------------------------*/
#define DATAREC_GP5   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  4,  682     \
    }

/* Definition of NOAA data record GAC unpacked 5-band
X   from -    to       size        num
6    449 -  4538        2         2045
-----------------------------------------------------*/
#define DATAREC_GU5   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  2, 2045     \
    }

/* Definition of NOAA data record GAC unpacked 4-band
X   from -    to       size        num
6    449 -  3720        2         1636
-----------------------------------------------------*/
#define DATAREC_GU4   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  2, 1636     \
    }

/* Definition of NOAA data record GAC unpacked 3-band
X   from -    to       size        num
6    449 -  2902        2         1227
-----------------------------------------------------*/
#define DATAREC_GU3   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,  2, 1227     \
    }

/* Definition of NOAA data record GAC unpacked 2-band
X   from -    to       size        num
6    449 -  2084        2          818
-----------------------------------------------------*/
#define DATAREC_GU2   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,   2, 818     \
    }

/* Definition of NOAA data record GAC unpacked 1-band
X   from -    to       size        num
6    449 -  1266        2          409
-----------------------------------------------------*/
#define DATAREC_GU1   \
    {                 \
    6,                \
    NOAADATARECINVAR, \
    448,   2, 409     \
    }

/* END of NOAA data record definitions. (non-KLM data)
==============================================================================*/


/*=============================================================================
Define the swapping macros. 

Note: The variable names were chosen to help prevent name collisions with
variables used in the macro call.  There have been cases where "i" was used as
part of the parameter to the macro which collided with an "i" declared within
the macro.  This resulted in a very difficult bug to track.  All variables
have a short prefix from the beginning of the macro name and have _MV (for
Macro Variable) appended to the name.
------------------------------------------------------------------------------*/

/* Note: The following three macros have been written using char variable
swaps so that when run on hardware which cannot access word variables that
are not on word boundaries the process will not suffer catastrophic failures.
------------------------------------------------------------------------------*/

/* Macro to swap bytes in a buffer segment of 16-bit values.
------------------------------------------------------------*/
#define swap2_byte_buffer_segment( swap2_bufseg_MV, swap2_numbytes_MV )        \
    {                                                                          \
    int  swap2_i_MV;                                                           \
    unsigned char swap2_c_MV;                                                  \
    for ( swap2_i_MV = 0; swap2_i_MV < (swap2_numbytes_MV)*2; swap2_i_MV += 2 )\
        {                                                                      \
	swap2_c_MV                      = (swap2_bufseg_MV)[swap2_i_MV];       \
	(swap2_bufseg_MV)[swap2_i_MV]   = (swap2_bufseg_MV)[swap2_i_MV+1];     \
	(swap2_bufseg_MV)[swap2_i_MV+1] = swap2_c_MV;                          \
	}                                                                      \
    }

/* Macro to swap bytes in a buffer segment of 32-bit values.
------------------------------------------------------------*/
#define swap4_byte_buffer_segment( swap4_bufseg_MV, swap4_numbytes_MV )        \
    {                                                                          \
    int  swap4_i_MV;                                                           \
    unsigned char swap4_c_MV;                                                  \
    for ( swap4_i_MV = 0; swap4_i_MV < (swap4_numbytes_MV)*4; swap4_i_MV += 4 )\
        {                                                                      \
	swap4_c_MV                      = (swap4_bufseg_MV)[swap4_i_MV];       \
	(swap4_bufseg_MV)[swap4_i_MV]   = (swap4_bufseg_MV)[swap4_i_MV+3];     \
	(swap4_bufseg_MV)[swap4_i_MV+3] = swap4_c_MV;                          \
	swap4_c_MV                      = (swap4_bufseg_MV)[swap4_i_MV+1];     \
	(swap4_bufseg_MV)[swap4_i_MV+1] = (swap4_bufseg_MV)[swap4_i_MV+2];     \
	(swap4_bufseg_MV)[swap4_i_MV+2] = swap4_c_MV;                          \
	}                                                                      \
    }

/* Macro to swap bytes in a buffer segment of 64-bit values.
------------------------------------------------------------*/
#define swap8_byte_buffer_segment( swap8_bufseg_MV, swap8_numbytes_MV )        \
    {                                                                          \
    int  swap8_i_MV;                                                           \
    unsigned char swap8_c_MV;                                                  \
    for ( swap8_i_MV = 0; swap8_i_MV < (swap8_numbytes_MV)*8; swap8_i_MV += 8 )\
        {                                                                      \
	swap8_c_MV                      = (swap8_bufseg_MV)[swap8_i_MV];       \
	(swap8_bufseg_MV)[swap8_i_MV]   = (swap8_bufseg_MV)[swap8_i_MV+7];     \
	(swap8_bufseg_MV)[swap8_i_MV+7] = swap8_c_MV;                          \
	swap8_c_MV                      = (swap8_bufseg_MV)[swap8_i_MV+1];     \
	(swap8_bufseg_MV)[swap8_i_MV+1] = (swap8_bufseg_MV)[swap8_i_MV+6];     \
	(swap8_bufseg_MV)[swap8_i_MV+6] = swap8_c_MV;                          \
	swap8_c_MV                      = (swap8_bufseg_MV)[swap8_i_MV+2];     \
	(swap8_bufseg_MV)[swap8_i_MV+2] = (swap8_bufseg_MV)[swap8_i_MV+5];     \
	(swap8_bufseg_MV)[swap8_i_MV+5] = swap8_c_MV;                          \
	swap8_c_MV                      = (swap8_bufseg_MV)[swap8_i_MV+3];     \
	(swap8_bufseg_MV)[swap8_i_MV+3] = (swap8_bufseg_MV)[swap8_i_MV+4];     \
	(swap8_bufseg_MV)[swap8_i_MV+4] = swap8_c_MV;                          \
	}                                                                      \
    }



/* Macro to swap bytes in a mixed buffer.  A mixed buffer is defined as one
 * with multiple data types mixed throughout the buffer.  Input to the macro
 * is a pointer to the mixed buffer and an array defining the operations to be
 * performed on the buffer.  The operation arrays are defined in the top
 * portion of this file.
 -----------------------------------------------------------------------------*/
#define correct_endian_in_mixed_buffer( the_buff_MV, mixed_arr_MV )            \
    {                                                                          \
    unsigned char *correct_buf_MV = (unsigned char *)(the_buff_MV);            \
    int correct_j_MV;                                                          \
    for ( correct_j_MV = 1; correct_j_MV < (mixed_arr_MV)[0]*3;                \
          correct_j_MV += 3 )                                                  \
        {                                                                      \
	switch ((mixed_arr_MV)[correct_j_MV+1])                                \
	    {                                                                  \
	    case 2 : swap2_byte_buffer_segment                                 \
	                (&(correct_buf_MV[(mixed_arr_MV)[correct_j_MV]]),      \
		           (mixed_arr_MV)[correct_j_MV+2]);                    \
		break;                                                         \
	    case 4 : swap4_byte_buffer_segment                                 \
	                (&(correct_buf_MV[(mixed_arr_MV)[correct_j_MV]]),      \
		           (mixed_arr_MV)[correct_j_MV+2]);                    \
		break;                                                         \
	    case 8 : swap8_byte_buffer_segment                                 \
	                (&(correct_buf_MV[(mixed_arr_MV)[correct_j_MV]]),      \
		           (mixed_arr_MV)[correct_j_MV+2]);                    \
		break;                                                         \
	    }                                                                  \
	}                                                                      \
    }


/* These two macros are the high level macros that are to be referenced within
the code for swapping of mixed buffer types.  Although they expand to the same
code use the import macro after a read and the export macro just prior to a
write.  It is important that these macros are used since the operation array
is expanded and declared here.

Usage note: due to the way these macros get expanded, there will be cases
when they must be surrounded by braces.  An example is:

    if ( test )
        import_endian_mixed_buffer( buf, arr );
    else
        ...

since the code will expand as:

    if ( test )
        {
	expansion ...
	};
    else

the compiler issues a parse error before the else.  The preceding code segment
should be written as:

    if ( test )
        {
	import_endian_mixed_buffer( buf, arr );
	}
    else

------------------------------------------------------------------------------*/
/* Macro to import a buffer to the local endian format
------------------------------------------------------*/
#define import_endian_mixed_buffer( buf, arr )                                 \
    {                                                                          \
    static int import_array_MV[] = arr;                                        \
    correct_endian_in_mixed_buffer((buf), (import_array_MV));                  \
    }

/* Macro to export a buffer to big endian format
------------------------------------------------*/
#define export_endian_mixed_buffer( buf, arr )                                 \
    {                                                                          \
    static int export_array_MV[] = arr;                                        \
    correct_endian_in_mixed_buffer((buf), (export_array_MV));                  \
    }

#else
/* This is not an i386 processor
==============================================================================*/

/* Define the operation arrays as nothing.  Insure there is a dummy definition
here for each operation array defined in the if portion of this file.
------------------------------------------------------------------------------*/
#define TESTDUMMY
#define SHARPMINOROPARRAY
#define KLMDATASETHEADER
#define KLMDATAREC_LP5
#define KLMDATAREC_LU5
#define KLMDATAREC_LU4
#define KLMDATAREC_LU3
#define KLMDATAREC_LU2
#define KLMDATAREC_LU1
#define KLMDATAREC_GP5
#define KLMDATAREC_GU5
#define KLMDATAREC_GU4
#define KLMDATAREC_GU3
#define KLMDATAREC_GU2
#define KLMDATAREC_GU1
#define NOAADATASETHEADER
#define DATAREC_LP5
#define DATAREC_LU5
#define DATAREC_LU4
#define DATAREC_LU3
#define DATAREC_LU2
#define DATAREC_LU1
#define DATAREC_GP5
#define DATAREC_GU5
#define DATAREC_GU4
#define DATAREC_GU3
#define DATAREC_GU2
#define DATAREC_GU1





/* Define the following two macros as NULL operations.  These are supposed to
be the only macros referenced in code so they are the only ones needed here.
------------------------------------------------------------------------------*/
#define import_endian_mixed_buffer( buf, arr )
#define export_endian_mixed_buffer( buf, arr )

#endif
