/******************************************************************************
NAME:           ACQSKIP

PURPOSE:        Skip the specified number of records.  A negative number
                skips backwards.

PROGRAM HISTORY:
VERSION  AUTHOR        DATE     REASON
-------  ------        -----    -----------------------------
  5.0    D. Hollaren   02/92    Original Development
  5.1    D.Lloyd       02/00    Changed name and moved to local function only

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include "worgen.h"
#include "tae.h"
#include "acqio.h"

int acqskip
(
    long *fdesc,                /* File descriptor to the archive file       */
    long nrecs                  /* Number of records to skip                 */
)
{
struct ACQDESC *acq;            /* Pointer to the archive input structure    */

off_t pos;			/* Position seeked to                        */

long    offset,                 /* Byte offset to set the file pointer to    */
        nonfatal = 1;           /* Non-fatal error code                      */

/*
  Set the archive structure to the file descriptor address
*/
acq = (struct ACQDESC *) *fdesc;

/*
  If we are flipping the data, skipping back means going forward in the file
  and vice versa
*/
if (acq->flip_flg)
    nrecs = -nrecs;

if (acq->rec_line < 0)
    offset = -1 * acq->rec_line * acq->byte_line * nrecs;
else
    offset = acq->rec_line * acq->byte_line * nrecs;

/*
  Seek to the specified location
*/
pos = lseek(acq->fd,offset,1);
if (pos < 0)
    {
    c_errmsg("Error skiping records in the acquire file","acqseek-skip",
             &nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
