#ifndef LOCAL_PROTOS_H
#define LOCAL_PROTOS_H


/* Prototype routines that are not to be called outside of this directory     */

int acq_frame
(
    short *inbuf,       /* Buffer to check for fame sync errors         */
    long *bufsiz,       /* Size of the buffer to check                  */
    long *line,         /* Current line being processed                 */
    long *nbad          /* Number of bad frame sync error lines found   */
);

int passdrctn
(
struct ACQDESC *acq     /* struct containing pass information           */
);

int read_bands
(
    struct ACQDESC *acq,        /* Pointer to the acqusition input structure */
    unsigned char inbuf[]       /* Input buffer for acquire                  */
);

int get_line
(
    struct ACQDESC *acq,        /* Acquisition structure                     */
    unsigned char *acqbuf       /* Output buffer for acquire image data      */
);

int sharp_mnr
(
    unsigned char *inbuf,       /* Input buffer from the sharp format        */
    short *mnrbuf               /* Output buffer for HRPT minor frame        */
);

void tmcor_interp
(
    long before_year,           /* Entry BEFORE input time/date: year        */
    long before_jday,           /* Entry BEFORE input time/date: julian day  */
    long before_time,           /* Entry BEFORE input time/date: time        */
    long before_timcor,         /* Entry BEFORE input time/date: t correction*/
    long after_year,            /* Entry AFTER input time/date: year         */
    long after_jday,            /* Entry AFTER input time/date: julian day   */
    long after_time,            /* Entry AFTER input time/date: time         */
    long after_timcor,          /* Entry AFTER input time/date: t correction */
    long f_year,                /* Year to find                              */
    long f_jday,                /* Julian date to seach for                  */
    long f_time,                /* Time to search for                        */
    long *timcor                /* Time correction value                     */
);

int tmcor_read
(
    long satnum,                /* Satellite number                          */
    long f_year,                /* Year to find                              */
    long f_jday,                /* Julian date to seach for                  */
    long f_time,                /* Time to search for                        */
    long *before_year,          /* Entry BEFORE input time/date: year        */
    long *before_month,         /* Entry BEFORE input time/date: month       */
    long *before_day,           /* Entry BEFORE input time/date: day         */
    long *before_time,          /* Entry BEFORE input time/date: time        */
    long *before_timcor,        /* Entry BEFORE input time/date: t correction*/
    long *before_flag,          /* Entry BEFORE input time/date: value found?*/
    long *after_year,           /* Entry AFTER input time/date: year         */
    long *after_month,          /* Entry AFTER input time/date: month        */
    long *after_day,            /* Entry AFTER input time/date: day          */
    long *after_time,           /* Entry AFTER input time/date: time         */
    long *after_timcor,         /* Entry AFTER input time/date: t correction */
    long *after_flag            /* Entry AFTER input time/date: value found? */
);

/* The following prototypes were moved from acqio.h and renamed as local
functions.
-----------------------------------------------------------------------------*/
int acqhead
(
    int *fdesc,                /* File descriptor to the acquire file       */
    long *year,                 /* Year of the pass                          */
    char *station,              /* Station ID                                */
    long *dtype,                /* Type of data (HRPT, LAC, or GAC)          */
    struct SATPASS *pass,       /* Satellite pass information                */
    long headtype
);

int acqinfo
(
    char *station,               /* Station ID                                */
    long *img_typ,               /* Type of image data (HRPT, LAC, GAC)       */
    long *skip_head,            /* Number of records to skip go get to image */
    long *rec_line,             /* Number of records per line                */
    long *byte_line,            /* Total number of bytes per line            */
    long *mnr_ptr,              /* Pointer to the start of HRPT minor frame  */
    long *mnr_siz,              /* Size of the HRPT minor frame (bytes)      */
    long *tip_ptr,              /* Pointer to the start of TIP data          */
    long *tip_siz,              /* Size of the TIP data (bytes)              */
    long *img_ptr,              /* Pointer to the image data                 */
    long *img_siz,              /* Size of the image data (bytes)            */
    long *img_fmt,              /* Image format (BIP, BIL, BIP)              */
    long *swap_flg,             /* Byte swap indicator                       */
    long *flip_flg,             /* Flip data indicator                       */
    long *headtype,             /* information header type                   */
    char *sta_name              /* Station ID name                           */
);

int acqskip
(
    long *fdesc,                /* File descriptor to the archive file       */
    long nrecs                  /* Number of records to skip                 */
);

#endif

