/******************************************************************************
FUNCTION:       timcor_interp

PURPOSE:        Interpolate between the two time correction values found in
                the time correction tables to determine the time correction
                at the time of interest.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    02/98  T.Baltzer        Development


PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "avhrr.h"
#include "acqio.h"


void tmcor_interp
(
    long before_year,           /* Entry BEFORE input time/date: year        */
    long before_jday,           /* Entry BEFORE input time/date: julian day  */
    long before_time,           /* Entry BEFORE input time/date: time        */
    long before_timcor,         /* Entry BEFORE input time/date: t correction*/
    long after_year,            /* Entry AFTER input time/date: year         */
    long after_jday,            /* Entry AFTER input time/date: julian day   */
    long after_time,            /* Entry AFTER input time/date: time         */
    long after_timcor,          /* Entry AFTER input time/date: t correction */
    long f_year,                /* Year to find                              */
    long f_jday,                /* Julian date to seach for                  */
    long f_time,                /* Time to search for                        */
    long *timcor                /* Time correction value                     */
)

{

float   ftimcor;                /* Floating point value of time correction   */

/*
  If a year is spanned (assume 1 year at the most is spanned), then
  increment the julian day count.
---------------------------------------------------------------------*/
if (after_year > before_year)
    {
    if (c_isleap(&before_year))
        after_jday += 366;
    else
        after_jday += 365;
    }

/*
  If a julian day(s) is spanned, increment the time accordingly
---------------------------------------------------------------------*/
if (after_jday > before_jday)
    after_time += ((after_jday - before_jday) * MIL_PER_DAY);


/*
  If a year is spanned (assume 1 year at the most is spanned), then
  increment the julian day count.
---------------------------------------------------------------------*/
if (f_year > before_year)
    {
    if (c_isleap(&before_year))
        f_jday += 366;
    else
        f_jday += 365;
    }

/*
  If a julian day(s) is spanned, increment the time accordingly
---------------------------------------------------------------------*/
if (f_jday > before_jday)
    f_time += ((f_jday - before_jday) * MIL_PER_DAY);


/*
  Interpolate the time correction value from the two closest entries
  in the time correction table.  Round the value instead of truncating.
------------------------------------------------------------------------*/
ftimcor = ((float)(f_time - before_time) / (float)(after_time - before_time) *
          (float)(after_timcor - before_timcor)) + (float)before_timcor;
if (ftimcor < 0.0)
    *timcor = (long)(ftimcor - 0.5);
else
    *timcor = (long)(ftimcor + 0.5);

return;
}

