/*****************************************************************************
  FUNCTION:	list_coef

  PURPOSE:	This is just a little ditty written to pull the calibration
  		coefficients from a NOAA level 1b.  It only works for
		a packed 5-band image but could be altered to work with
		others by changing the REC define for the proper record
		size.

  USAGE:        list_coef <level 1b file> [channel list]
		   Specify: 1 2 3 4 or 5 for chan option
		   Example: list_klm_coef ah14051699123456 1 2

  		omitting the channel list will print all the channel
		coefficient.  
*****************************************************************************/
#include <stdio.h>

#define  REC     14800
#define  TWO22   4194304.0
#define  TWO30   1073741824.0

void main( int argc, char *argv[] )
{
FILE	*inf;
char	cbuf[REC];
char	infile[120];
float	coef[6][5];
int	chan;
int	chan1 = 0;
int	chan2 = 0;
int	chan3 = 0;
int	chan4 = 0;
int	chan5 = 0;
int	i;
int	j;
int	line;
int	nbytes;
int	selectall;
long	*lptr[6];

/* Check for command line arguments
and issue a usage message if none
-----------------------------------*/
if ( argc < 2 )
    {
    printf( "\nUsage: %s <level1b_file> [chan option]\n", argv[0] );
    printf( "    Specify: 1 2 3 4 or 5 for chan option\n" );
    printf( "    Example: %s ah14051699123456 1 2\n\n", argv[0] );
    exit(1);
    }

/* Get the channel option, if specified
---------------------------------------*/
if ( argc > 2 )
    {
    for ( i = 2; i <= argc; i++ )
	{
        chan = atoi( argv[i] );
	switch ( chan )
	    {
	    case 1 : chan1 = 1; break;
	    case 2 : chan2 = 1; break;
	    case 3 : chan3 = 1; break;
	    case 4 : chan4 = 1; break;
	    case 5 : chan5 = 1; break;
	    }
	}
    selectall = 0;
    }
else
    selectall = 1;

/* Open the input file
----------------------*/
strcpy( infile, argv[1] );
inf = fopen( infile, "r" );
if ( !inf )
    {
    strcat( infile, ".l1b" );
    inf = fopen( infile, "r" );
    }
if ( !inf )
    {
    printf( "\nUnable to open: %s or %s\n\n", argv[1], infile );
    exit(1);
    }

/* Read past the TBM and Data header records
--------------------------------------------*/
fread( cbuf, sizeof(char), 122, inf );
fread( cbuf, sizeof(char), REC, inf );

/* Set up pointers to the calibration coefficients
--------------------------------------------------*/
lptr[0] = (long *)&cbuf[12];	/* channel 1	*/
lptr[1] = (long *)&cbuf[20];	/* channel 2	*/
lptr[2] = (long *)&cbuf[28];	/* channel 3	*/
lptr[3] = (long *)&cbuf[36];	/* channel 4	*/
lptr[4] = (long *)&cbuf[44];	/* channel 5	*/

line = 0;
while ( 1 )
    {
    /* Read a line of data and exit at EOF
    --------------------------------------*/
    nbytes = fread( cbuf, sizeof(char), REC, inf );
    if ( nbytes != REC )
        break;
    line++;

    /* Get the coefficients from the line
    -------------------------------------*/
    for ( i = 0; i < 5; i ++ )
	{
	coef[i][0] = (float)lptr[i][0];
	coef[i][1] = (float)lptr[i][1];
	}

    /* Unscale the coefficients
    ---------------------------*/
    /* Channel 1 */
    coef[0][0] /= TWO30;	/* slope	*/
    coef[0][1] /= TWO22;	/* intercept	*/
    /* Channel 2 */
    coef[1][0] /= TWO30;	/* slope	*/
    coef[1][1] /= TWO22;	/* intercept	*/
    /* Channel 3 */
    coef[2][0] /= TWO30;	/* slope	*/
    coef[2][1] /= TWO22;	/* intercept	*/
    /* Channel 4 */
    coef[3][0] /= TWO30;	/* slope	*/
    coef[3][1] /= TWO22;	/* intercept	*/
    /* Channel 5 */
    coef[4][0] /= TWO30;	/* slope	*/
    coef[4][1] /= TWO22;	/* intercept	*/


    /* Print the results
    --------------------*/
    printf( "Line: %4d ", line );
    if ( selectall || chan1 )
	printf( " C1  %f  %f  ", coef[0][0], coef[0][1] );
    if ( selectall || chan2 )
	printf( " C2  %f  %f  ", coef[1][0], coef[1][1] );
    if ( selectall || chan3 )
	printf( " C3  %f  %f  ", coef[2][0], coef[2][1] );
    if ( selectall || chan4 )
	printf( " C4  %f  %f  ", coef[3][0], coef[3][1] );
    if ( selectall || chan5 )
	printf( " C5  %f  %f  ", coef[4][0], coef[4][1] );
    printf( "\n" );
    }

fclose( inf );
printf( "At end\n" );
}
