/*****************************************************************************
  FUNCTION:	list_klm_coef

  PURPOSE:	This is just a little ditty written to pull the calibration
  		coefficients from a NOAA KLM level 1b.  It only works for
		a packed 5-band image but could be altered to work with
		others by changing the REC define for the proper record
		size.

  USAGE:        list_klm_coef <KLM level 1b file> [channel list]
		   Specify: 1 2 3 4 or 5 for chan option
		   Example: list_klm_coef ah14051699123456 1 2

  		omitting the channel list will print all the channel
		coefficient.  Specifying 3 will list both 3A and 3B.
*****************************************************************************/
#include <stdio.h>

#define  REC  15872

void main( int argc, char *argv[] )
{
FILE	*inf;
char	cbuf[REC];
char	infile[120];
float	coef[6][5];
int	chan;
int	chan1 = 0;
int	chan2 = 0;
int	chan3 = 0;
int	chan4 = 0;
int	chan5 = 0;
int	i;
int	j;
int	line;
int	nbytes;
int	selectall;
long	*lptr[6];

/* Check for command line arguments
and issue a usage message if none
-----------------------------------*/
if ( argc < 2 )
    {
    printf( "\nUsage: %s <klm_level1b_file> [chan option]\n", argv[0] );
    printf( "    Specify: 1 2 3 4 or 5 for chan option\n" );
    printf( "    Example: %s ah14051699123456 1 2\n", argv[0] );
    exit(1);
    }

/* Get the channel option, if specified
---------------------------------------*/
if ( argc > 2 )
    {
    for ( i = 2; i <= argc; i++ )
	{
        chan = atoi( argv[i] );
	switch ( chan )
	    {
	    case 1 : chan1 = 1; break;
	    case 2 : chan2 = 1; break;
	    case 3 : chan3 = 1; break;
	    case 4 : chan4 = 1; break;
	    case 5 : chan5 = 1; break;
	    }
	}
    selectall = 0;
    }
else
    selectall = 1;

/* Open the input file
----------------------*/
strcpy( infile, argv[1] );
inf = fopen( infile, "r" );
if ( !inf )
    {
    strcat( infile, ".l1b" );
    inf = fopen( infile, "r" );
    }
if ( !inf )
    {
    printf( "\nUnable to open: %s or %s\n\n", argv[1], infile );
    exit(1);
    }

/* Read past the ARS and Data header records
--------------------------------------------*/
fread( cbuf, sizeof(char), 512, inf );
fread( cbuf, sizeof(char), REC, inf );

/* Set up pointers to the calibration coefficients
--------------------------------------------------*/
lptr[0] = (long *)&cbuf[48];	/* channel 1	*/
lptr[1] = (long *)&cbuf[108];	/* channel 2	*/
lptr[2] = (long *)&cbuf[168];	/* channel 3A	*/
lptr[3] = (long *)&cbuf[228];	/* channel 3B	*/
lptr[4] = (long *)&cbuf[252];	/* channel 4	*/
lptr[5] = (long *)&cbuf[276];	/* channel 5	*/

line = 0;
while ( 1 )
    {
    /* Read a line of data and exit at EOF
    --------------------------------------*/
    nbytes = fread( cbuf, sizeof(char), REC, inf );
    if ( nbytes != REC )
        break;
    line++;

    /* Get the coefficients from the line
    -------------------------------------*/
    for ( i = 0; i < 3; i ++ )
	for ( j = 0; j < 5; j++ )
	    coef[i][j] = (float)lptr[i][j];

    for ( i = 3; i < 6; i++ )
        for ( j = 0; j < 3; j++ )
	    coef[i][j] = (float)lptr[i][j];

    /* Unscale the coefficients
    ---------------------------*/
    /* Channel 1 */
    coef[0][0] /= 10000000000;	/* slope 1	*/
    coef[0][1] /= 10000000;	/* intercept 1	*/
    coef[0][2] /= 10000000000;	/* slope 2	*/
    coef[0][3] /= 10000000;	/* intercept 2	*/
    /* coef[0][4] /= 1;		   intersection */
    /* Channel 2 */
    coef[1][0] /= 10000000000;	/* slope 1	*/
    coef[1][1] /= 10000000;	/* intercept 1	*/
    coef[1][2] /= 10000000000;	/* slope 2	*/
    coef[1][3] /= 10000000;	/* intercept 2	*/
    /* coef[1][4] /= 1;		   intersection */
    /* Channel 3A */
    coef[2][0] /= 10000000000;	/* slope 1	*/
    coef[2][1] /= 10000000;	/* intercept 1	*/
    coef[2][2] /= 10000000000;	/* slope 2	*/
    coef[2][3] /= 10000000;	/* intercept 2	*/
    /* coef[2][4] /= 1;		   intersection */

    /* Channel 3B */
    coef[3][0] /= 1000000;
    coef[3][1] /= 1000000;
    coef[3][2] /= 1000000;
    /* Channel 4 */
    coef[4][0] /= 1000000;
    coef[4][1] /= 1000000;
    coef[4][2] /= 1000000;
    /* Channel 5 */
    coef[5][0] /= 1000000;
    coef[5][1] /= 1000000;
    coef[5][2] /= 1000000;


    /* Print the results
    --------------------*/
    printf( "Line: %4d ", line );
    if ( selectall || chan1 )
	printf( " C1  %f  %f  %f  %f  %f  ",
	    coef[0][0], coef[0][1], coef[0][2], coef[0][3], coef[0][4] );
    if ( selectall || chan2 )
	printf( " C2  %f  %f  %f  %f  %f  ", 
	    coef[1][0], coef[1][1], coef[1][2], coef[1][3], coef[1][4] );
    if ( selectall || chan3 )
	printf( " C3A  %f  %f  %f  %f  %f  ",
	    coef[2][0], coef[2][1], coef[2][2], coef[2][3], coef[2][4] );
    if ( selectall || chan3 )
	printf( " C3B  %f  %f  %f  ", coef[3][0], coef[3][1], coef[3][2] );
    if ( selectall || chan4 )
	printf( " C4  %f  %f  %f  ", coef[4][0], coef[4][1], coef[4][2] );
    if ( selectall || chan5 )
	printf( " C5  %f  %f  %f  ", coef[5][0], coef[5][1], coef[5][2] );
    printf( "\n" );
    }

fclose( inf );
printf( "At end\n" );
}
