/*****************************************************************************
  NAME:		list_space

  PURPOSE:	To display the CALIB values of a level 0 or NOAA KLM level 1b
  		file.
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void unpck32 ( unsigned char *, short *, long * );

#define MASK1 0X3ff00000
#define MASK2 0X000ffc00
#define MASK3 0X000003ff
#define  L0REC  13796
#define  L1REC  15872
#define  NUM  1000

void main( int argc, char *argv[] )
{
FILE	*inf;
unsigned char	cbuf[L1REC];
char	infile[120];
int	i, j, k;
int	l1bflag = 0;
int	nbytes;
long	nwords = 120;
long	calib[5];
short	shortbuf[150];
short	*sptr;

if ( argc < 3 )
    {
    printf("\nUsage: %s <infile> <l1b/l0>\n\n", argv[0]);
    exit(1);
    }

if ( strcmp( argv[2], "l1b") == 0 )
    {
    printf("Can't do level 1b right now\n");
    exit(1);
    l1bflag = 1;
    strcpy( infile, argv[1] );
    strcat( infile, ".l1b" );
    }
else
    {
    strcpy( infile, argv[1] );
    strcat( infile, ".arch" );
    }
inf = fopen( infile, "r" );
if ( !inf )
    {
    printf("\nUnable to open file: %s\n\n",infile);
    exit(1);
    }

if ( l1bflag )
    {
    fread( cbuf, sizeof(char), 512, inf );
    fread( cbuf, sizeof(char), L1REC, inf );
    sptr = (short *)&cbuf[1090];
    }
else
    sptr = (short *)&cbuf[46];


i = 0;
for ( j = 54; j < 103; j += 5 )
    for ( k = 0; k < 3; k++ )
	printf( "%4d ", 0 );
printf("\n");
while ( 1 )
    {
    if ( l1bflag )
	nbytes = fread( cbuf, sizeof(char), L1REC, inf );
    else
        {
	nbytes = fread( cbuf, sizeof(char), L0REC, inf );
	unpck32( cbuf, shortbuf, &nwords);
	}
    if ( nbytes != L1REC && nbytes != L0REC )
        break;
    i++;
    calib[0] = sptr[0];
    calib[1] = sptr[1];
    calib[2] = sptr[2];
    if ( l1bflag )
	printf("line: %4d  CALIB  %3d %3d %3d\n", i,calib[0],calib[1],calib[2]);
    else
	{
	printf("line: %4d SPACE     ", i );
	for ( j = 54; j < 103; j += 5 )
	    for ( k = 0; k < 3; k++ )
		printf( "%4d ", shortbuf[j+k] );
	printf("\n");
	    /*
	printf("line: %4d  CALIB  %3d %3d %3d\n",
	    i,shortbuf[17],shortbuf[18],shortbuf[19]);
	    */
	}
    }

fclose( inf );
printf("At end\n");
}




void unpck32
(
    unsigned char *inbuf,	/* Pointer to the 32-bit input buffer	    */
    short *outbuf,	/* Series of 10 bit words stored in 16 bits         */
    long *nwords	/* Number of 16-bit words to unpack	            */
)
{
short	*optr;		/* Pointer to the output buffer			    */

long	*iptr,		/* Pointer to the 32-bit input buffer	            */
	num,		/* Number of full 32-bit long words		    */
	extra,		/* Extra 10 bit words				    */
	i;		/* Looping variable				    */

/*
  Calculate the number of 32-bit long words that contain
  three 10 bit words
*/
num = *nwords / 3;

extra = (*nwords % 3);

/*
  Set pointers to the input and output buffers
*/
iptr = (long *)inbuf;
optr = outbuf;

/*
  Unpack a 16-bit word into one 10-bit word
*/
for (i = 0; i < num; i++)
    {
    /*
      Use the masks to get each 10 bits
    */
    *optr = (short)((*iptr & MASK1) >> 20);
    optr += 1;

    *optr = (short)((*iptr & MASK2) >> 10);
    optr += 1;

    *optr = (short)(*iptr & MASK3);
    optr += 1;

    iptr += 1;
    }

/*
  Handle any 10-bit words which are left over
*/
if (extra >= 1)
    {
    *optr = (short)((*iptr & MASK1) >> 20);

    if (extra == 2)
	{
	optr += 1;
	*optr = (short)((*iptr & MASK2) >> 10);
	}
    }

return;
}
