/*******************************************************************************
FUNCTION:	C_GETSCALEOPT

PURPOSE:	Set the scaling option as expected by the support routine
		c_getscale().

ALGORITHM:

Note:  For GLOBAL scaling, two byte integer data types could either be
       convensional "I*2" integer or "10-BIT" integer (unsigned values
       from 0-1023).  However, for the sake of simplicity, AVHRR data
       that is two byte integer with GLOBAL scaling will be
       ASSUMED TO BE "10-BIT" data.  The "10-BIT" data type is only
       possible when GLOBAL scaling is being used.  This assumption 
       essentially disables convensional 16-bit integer data with GLOBAL
       scaling.
  
*******************************************************************************/
#include <stdio.h>		/* Standard I/O include			*/
#include "las.h"		/* LAS include file			*/
#include "adaps.h"		/* ADAPS include file			*/

void c_getscaleopt
(
    char *scale_opt,	/* Scaling option (US,GLOBAL,NONE)	*/
    long odtype,	/* Output data type			*/
    long *soption	/* Scaling option (US_I2,AVHRR_BYTE,etc)*/
)

{

/*
  Initialize to an invalid option
*/
*soption = -1;

if (strcmp(scale_opt,"GLOBAL") == 0)
    {
    if (odtype == EBYTE)
	*soption = AVHRR_BYTE;
    else if (odtype == EWORD)
	*soption = AVHRR_10BIT;
    else if (odtype == ELONG)
	*soption = AVHRR_I4;
    else if (odtype == EREAL)
	*soption = AVHRR_REAL;
    }
else if (strcmp(scale_opt,"US") == 0)
    {
    if (odtype == EBYTE)
	*soption = US_BYTE;
    else if (odtype == EWORD)
	*soption = US_I2;
    else if (odtype == ELONG)
	*soption = US_I4;
    else if (odtype == EREAL)
	*soption = US_REAL;
    }
else if (strcmp(scale_opt,"NONE") == 0)
    {
    *soption = NO_SCALING;
    }

return;
}
