/*******************************************************************************
NAME:		WDBREL

PURPOSE:	Read the line segments from the WDBII relative file

PROGRAM HISTORY:
Version      Date    Programmer         Request
-------      ----    ----------         -------
  1.0        10/91   D. Hollaren        Initial development

*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "tae.h"

#define MAX_PTS	50	/* maximum number of points in a record		     */
#define NBYTES	4	/* number of bytes for each point		     */

int c_wdbrel
(
    FILE *fp_rel,	/* file pointer to the WDBII relative file	     */
    long *total_pts,	/* total number of points in the line segment	     */
    long *num_pts,	/* number of points to process at this time          */
    float pnts[]	/* latitude/Longitude point defining the segments    */
)
{
long	nitems,		/* number of items to read from the file	     */
	status,		/* return status				     */
	nonfatal = 1;	/* non-fatal error code				     */

/*
  Check to see if done
*/
if (*total_pts == 0)
    return(E_EOF);

/*
  Read the points for the segment from the WDBII relative file
*/
nitems = 2 * MAX_PTS;
status = (long)fread(pnts,sizeof(float),nitems,fp_rel);
if (status == 0)
    {
    c_errmsg("Error reading the WDB2 file","wdbrel-read",&nonfatal);
    return(E_FAIL);
    }

/*
  Determine the number of points read
*/
if (*total_pts <= MAX_PTS)
    {
    *num_pts = *total_pts;
    *total_pts = 0;
    }
else
    {
    *num_pts = MAX_PTS;
    *total_pts -= MAX_PTS;
    }

return(E_SUCC);
}
 
