/*******************************************************************************
NAME:		WDBSEQ

PURPOSE:	Find the segments to be processed from the WDB2 file based
		on the specified rank and geographic location specified.

PROGRAM HISTORY:
Version      Date    Programmer         Request
-------      ----    ----------         -------
  1.0        10/91   D. Hollaren        Initial development
  1.1	     06/92   T. Rockvam		Added flag for areas over the 
					International Date Line (for checking 
					range of longitude)
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"

#define RELSIZ	400	/* bytes per record in the WDBII relative file	     */

int c_wdbseq
(
    FILE *fp_seq,	/* file pointer to the WDBII sequential file	     */
    long ranks[],	/* ranking of interest				     */
    long crossflg,	/* indicates crossing Internat'l Date Line (180 long)*/
    float *minlat,	/* minimum latitude of area of interest		     */
    float *minlon,	/* minimum longitude of area of interest	     */
    float *maxlat,	/* maximum latitude of area of interest		     */
    float *maxlon,	/* maximum longitude of area of interest	     */
    long *total_pts,	/* total number of points in the line segment	     */
    FILE *fp_rel 	/* file pointer to the WDBII relative file	     */
)
{
long	offset,		/* offset to seek into the WDBII relative file	     */
	status = 1;	/* end Of File flag				     */

/*
  Record structure for the WDBII sequential file
*/
struct SEQ
    {
    long hdrid;		/* WDBII header ID 				     */
    long rank;		/* WDBII feature rank				     */
    long npts;		/* number of lat/long pairs in WDBII relative record */
    long ptr;		/* pointer to the WDBII relative record of lat/long's*/
    float minlat;	/* minimum latitude  the WDBII segment covers	     */
    float minlon;	/* minimum longitide the WDBII segment covers	     */
    float maxlat;	/* maximum latitude  the WDBII segment covers	     */
    float maxlon;	/* maximum longitide the WDBII segment covers	     */
    } seq;

/*
  Find all the records with the specfied ranking and in the
  defined geographic area
*/
while (status != 0)
    {
    /*
      Read a record from the WDBII sequential file
    */
    status = (long)fread(&seq,sizeof(seq),1,fp_seq);
    if ((status == 0) || (seq.rank == 0)) /* eof or last (zero) record */
	continue;

    /*
      Determine if this line segment is of the desired rank
    */
    if (ranks[seq.rank-1])
	{
	/*
	  Determine if this line segment falls within the output area
	*/
	if (crossflg)
	    {
	    if (((seq.maxlat >= *minlat) && (seq.minlat <= *maxlat)) &&
	        ((seq.maxlon >= *maxlon) || (seq.minlon <= *minlon)))
	        {
	        /*
	          Set the WDBII relative file pointer to the starting record
	          containing the desired line segment points.
	        */
	        offset = (seq.ptr - 1) * RELSIZ;
	        fseek(fp_rel,offset,0);
	        *total_pts = seq.npts;
	        break;
	        }
	    }
	else
	    {
	    if (((seq.maxlat >= *minlat) && (seq.minlat <= *maxlat)) &&
	        ((seq.maxlon >= *minlon) && (seq.minlon <= *maxlon)))
	        {
	        /*
	          Set the WDBII relative file pointer to the starting record
	          containing the desired line segment points.
	        */
	        offset = (seq.ptr - 1) * RELSIZ;
	        fseek(fp_rel,offset,0);
	        *total_pts = seq.npts;
	        break;
	        }
	    }
	}
    }

return(status);
}
 
