/*******************************************************************************
NAME:		C_WVSFILE

PURPOSE:	Find out which wvs area and file the point is in
		Files are named wvs[0-9][00-16].data, where 0-9 is area
		and 00-08 are the 10-latitude-degree subarea.
		(e.g.,wvs103.data).  The calling routine is responsible
		for putting the file name together.
		
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#define LAT 0
#define LON 1

int c_wvsfile
(
    double coord[],
    long *area,
    long *subarea
)
{
/*
  Set up the boundaries of the areas
*/
static long wvs_minlat[10]={-90, 25, -60, 0, 0, -60, 0, -60, -60, 50};

/*
  Figure out which area by process of elimination
*/
if (coord[LAT] >= 80.0)
    {
    *area = 9;
    }
else if (coord[LAT] >= 65.0)
    {
    if ((coord[LON] >= -30.0) && (coord[LON] <= 100.0))
    	*area = 1;
    else if ((coord[LON] >=  100.0) || (coord[LON] <= -170.0))
    	*area = 3;
    else if ((coord[LON] >= -170.0) && (coord[LON] <= -140.0))
    	*area = 4;
    else if ((coord[LON] >= -140.0) && (coord[LON] <= -75.0))
    	*area = 9;
    else if ((coord[LON] >=  -75.0) && (coord[LON] <= -30.0))
    	*area = 6;
    }
else if (coord[LAT] >= 50.0)
    {
    if ((coord[LON] >= -30.0) && (coord[LON] <= 100.0))
    	*area = 1;
    else if ((coord[LON] >=  100.0) || (coord[LON] <= -170.0))
    	*area = 3;
    else if ((coord[LON] >= -170.0) && (coord[LON] <= -110.0))
    	*area = 4;
    else if ((coord[LON] >= -110.0) && (coord[LON] <= -75.0))
    	*area = 9;
    else if ((coord[LON] >=  -75.0) && (coord[LON] <= -30.0))
    	*area = 6;
    }
else if (coord[LAT] >= 35.0)
    {
    if ((coord[LON] >= -30.0) && (coord[LON] <= 100.0))
    	*area = 1;
    else if ((coord[LON] >=  100.0) || (coord[LON] <= -170.0))
    	*area = 3;
    else if ((coord[LON] >= -170.0) && (coord[LON] <= -110.0))
    	*area = 4;
    else if ((coord[LON] >= -110.0) && (coord[LON] <= -30.0))
    	*area = 6;
    }
else if (coord[LAT] >= 25.0)
    {
    if ((coord[LON] >= -30.0) && (coord[LON] <= 40.0))
    	*area = 1;
    else if ((coord[LON] >= 40.0) && (coord[LON] <= 100.0))
    	*area = 2;
    else if ((coord[LON] >= 100.0) || (coord[LON] <= -170.0))
    	*area = 3;
    else if ((coord[LON] >= -170.0) && (coord[LON] <= -110.0))
    	*area = 4;
    else if ((coord[LON] >= -110.0) && (coord[LON] <= -30.0))
    	*area = 6;
    }
else if (coord[LAT] >=  0.0)
    {
    if ((coord[LON] >= 20.0) && (coord[LON] <= 100.0))
    	*area = 2;
    else if ((coord[LON] >= 100.0) || (coord[LON] <= -170.0))
    	*area = 3;
    else if ((coord[LON] >= -170.0) && (coord[LON] <= -110.0))
    	*area = 4;
    else if ((coord[LON] >= -110.0) && (coord[LON] <= -30.0))
    	*area = 6;
    else if ((coord[LON] >= -30.0) && (coord[LON] <= 20.0))
    	*area = 7;
    }
else if (coord[LAT] >=-60.0)
    {
    if ((coord[LON] >= 20.0)  && (coord[LON] <= 130.0))
    	*area = 2;
    else if ((coord[LON] >=  130.0) || (coord[LON] <= -70.0))
    	*area = 5;
    else if ((coord[LON] >=  -70.0) && (coord[LON] <= -30.0))
    	*area = 8;
    else if ((coord[LON] >= -30.0) && (coord[LON] <= 20.0))
    	*area = 7;
    }
else
    {
    *area = 0;
    }

*subarea = ((long)(coord[LAT] - wvs_minlat[*area])) / 10;
if (*subarea < 0) *subarea = 0;

return(E_SUCC);
}
