/******************************************************************************
FUNCTION:	CCLOAD_STRUCT

PURPOSE:	Read all of the necessary calibration parameters from the
		appropriate tables (files).  Calculate all of the possible
		calibration parameters from these values.  Load these values
		into the calibration structure.

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
  1.0	09/98	T. Baltzer	Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "calcoef.h"
#include "adapssup.h"
#include "local_protos.h"


int ccload_struct
(
    struct MODEL *model,    /* IN  - Model and orbital information	*/
    struct CALSTR *calstr,  /* OUT - Calibration parameter structure	*/
    struct COEFFSTR *coeff  /* OUT - Calibration coeff struct		*/
)
{

long status;		/* Return status code */


/*
  Load the information relative to the NOAA satellite (file "noaaXX.cal")
*/
status = ccread_sat(model->satnum, calstr );
if (status != E_SUCC)
    return(status);


/*
  Calculate the counts of space values
*/
cccalc_C0( calstr );


/*
  Read the K values from the appropriate table (rao.cal, vermote.cal, etc.)
  These values are used for converting from radiance to reflectance.
  If the specified method doesn't have values available, try to get PRELAUNCH
  values.
*/
if (c_getkvals(model->satnum, calstr->calmethod, calstr->Kvals) != E_SUCC)
    {
    if (strcmp(calstr->calmethod,"prelaunch") != 0)
        {
        if (c_getkvals(model->satnum, "prelaunch", calstr->Kvals) != E_SUCC)
            return(status);
        else
            c_lasmsg(FALSE,"calcoeff-kval","PRELAUNCH K values being used");
        }
    else
        return(status);
    }


/*
  Read the slope/intercept degradation tables ("rao.cal", "vermote.cal", etc.).
  For PRELAUNCH, compute these values from data read from the "noaaXX.cal" file.
*/
status = ccload_dgrd (model, calstr, coeff);
if (status != E_SUCC)
    return(status);

return(E_SUCC);
}
