/****************************************************************************
NAME:		GETLINE

PURPOSE:	

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 D. Hollaren   03/92	

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"

#define SIMAX (12.7847*RAD_PER_DEG) /* Maximum look angle                     */

void getline
(
    long quadnum,	/* Quadrant the latitude is wanted in		     */
    long mul,	 	/* Multiplier for min/max lat for start of scene     */
    double inlat,	/* Desired lat					     */
    long lines[3],	/* Line numbers of possible latitudes		     */
    long quads[3],	/* Quad numbers of line numbers found		     */
    struct MODEL *model	/* Model and orbital information		     */
)
{
double  anlng,          /* longitude of ascending node          	     */
        inclin,         /* inclination angle                    	     */
	k,		/* angle of satellite		 		     */
	lats0,		/* latitude of satellite at start line 		     */
	mm,		/* mean motion of satellite in rads/sec 	     */
	psi,		/* viewing angle				     */
        rsat,           /* distance of satellite from earth center 	     */
	sinlats1,	/* sine of satellite's lat at inlat 		     */
	sinpsi,		/* sine of viewing angle			     */
        slong,          /* geocentric longitude of satellite   	     	     */
	theta0,		/* true anomaly past equator at line 1  	     */
	theta[3],	/* true anomaly past equator at inlat 		     */
	cosa,		/* cosine of longitude				     */
	delta_long,	/* delta longitude from ascending node		     */
	sat_long,	/* satellite longitude from ascending node	     */
	time,           /* total seconds past epoch             	     */
	timelapsed,	/* seconds from beginning of scene		     */
        truea;          /* true anomaly past perigee            	     */

long    eorbit,         /* orbit counter from equator           	     */
	i,		/* looping variable				     */
	line,		/* line within the image			     */
        porbit,         /* orbit counter from perigee           	     */
	zdot;           /* ascending(1) / descending(-1) flag   	     */

/*
  Find satellite's position at start line
*/
time = model->sec_epoch + model->delta_time;
brouwer(&time,model,&truea,&rsat,&theta0,&anlng,&inclin,&zdot,
        &eorbit,&porbit,&lats0,&slong);

/*
  Calculate the mean motion of satellite in rads/sec
*/
mm = model->mean_motion / HERG;

/*
  Find angle of satellite
*/
k = PIE - inclin;

/*
  Find psi for desired latitude
*/
if (inlat != 0.0)
    sinpsi = fabs(cos(k)/sin(inlat));
else
    sinpsi = cos(k) / fabs(cos(k));

if (sinpsi > 1.0) 
    sinpsi = 1.0;
else if (sinpsi < -1.0) 
    sinpsi = -1.0;

psi = asin(sinpsi);
if (psi > SIMAX)
    {
    psi = SIMAX;
    sinpsi = sin(psi);
    }

/*
  Find satellite's position at desired lat
*/
sinlats1 = (sin(inlat) + mul * cos(k) * sinpsi) / cos(psi);

if (sinlats1/sin(k) > 1.0)
    theta[0] = asin(1.0);
else if (sinlats1/sin(k) < -1.0)
    theta[0] = asin(-1.0);
else
    theta[0] = asin(sinlats1/sin(k));

/*
  Set the other two possible locations within the pass that could
  cross the specified latitude
*/
theta[1] = PIE - theta[0];
theta[2] = theta[0] + PIE2;

for (i = 0; i < 3; i++)
    {
    if ((inlat != HALFPIE) && (inlat != -HALFPIE))
        cosa = cos(theta[i]) / cos(inlat);
    else
        cosa = cos(theta[i]) / fabs(cos(theta[i]));
    if (cosa > 1.0) 
	cosa = 1.0;
    else if (cosa < -1.0) 
	cosa = -1.0;
    delta_long = acos(cosa) - WE * theta[i] / mm;
    sat_long = anlng - delta_long;
    if (sat_long < -PIE)
	sat_long += PIE2;
    else if (sat_long > PIE)
	sat_long -= PIE2;

    if (inlat > 0.0)
        if (sat_long > 0.0)
	    quads[i] = 1;
        else
	    quads[i] = 2;
    else
        if (sat_long > 0.0)
	    quads[i] = 4;
        else
	    quads[i] = 3;

    /*
      We don't care if it is north/south of equator, just whether
      it matches in longitude
    */
    if ((quadnum == 1) && (quads[i] == 4)) quads[i] = 1;
    if ((quadnum == 4) && (quads[i] == 1)) quads[i] = 4;

    if ((quadnum == 2) && (quads[i] == 3)) quads[i] = 2;
    if ((quadnum == 3) && (quads[i] == 2)) quads[i] = 3;

    /*
      Determine the line location within the image
    */
    timelapsed = (theta[i] - theta0) / mm;

    if (model->dtype == GAC)
	line = (long)(timelapsed * GAC_PER_SEC);
    else
	line = (long)(timelapsed * LAC_PER_SEC);

    lines[i] = line;
    }

return;
}
 
