#ifndef LOCAL_PROTOS_H
#define LOCAL_PROTOS_H


/* Prototype routines that are not to be called outside of this directory     */

void cccalc_C0
(
    struct CALSTR *calstr  /* Calibration parameter structure		*/
);

void cccalc_dgrd
(
    struct CALSTR *calstr,  /* Calibration parameter structure		*/
    struct COEFFSTR *coeff  /* OUT - Calibration coefficient structure	*/
);

void ccgen_luts
(
    struct CALSTR *calstr,	/* Calibration structure		*/
    struct MODEL *model,  	/* AVHRR model structure		*/
    long channel[],	  	/* Channels to process			*/
    long nchans,	  	/* Number of channels to process	*/
    struct COEFFSTR *coeff,	/* Calibration coefficients		*/
    float cwncon2[],	  	/* CWN times constant two		*/
    float cwn3con1[],	  	/* CWN times constant one		*/
    struct SCALESTR *a_scale, 	/* Scaling values for reflectance	*/
    struct SCALESTR *r_scale, 	/* Scaling values for radiance		*/
    struct SCALESTR *t_scale, 	/* Scaling values for temperature	*/
    float lut[][MAXDAT]  /* Temperature look-up table for the channel	*/
);

int ccload_dgrd
(
    struct MODEL *model,	/* IN - Model and orbital information	*/
    struct CALSTR *calstr,	/* IN/OUT - Calibration parameter struct*/
    struct COEFFSTR *coeff	/* Calibration coefficients structure	*/
);

int ccload_dgrd1
(
    char *hostname,		/* IN  - Calibration table host file name  */
    long satnum,		/* IN  - Satellite number		   */
    long ayear,			/* IN  - Acquisition year		   */
    long aday,			/* IN  - Acquisition day		   */
    struct CALSTR *calstr, 	/* IN/OUT - Calibration parameter structure*/
    struct COEFFSTR *coeff	/* OUT - Calibration coefficients	   */
);

int ccload_dgrd2
(
    char *hostname,	   /* IN  - Calibration table host file name          */
    long satnum,	   /* IN  - Satellite number                          */
    long ayear,		   /* IN  - Acquisition year                          */
    long aday,		   /* IN  - Julian acquisition day                    */
    struct CALSTR *calstr, /* IN/OUT - Calibration parameter structure	      */
    struct COEFFSTR *coeff /* OUT - Calibration coefficients structure	      */
);

int ccload_struct
(
    struct MODEL *model,    /* IN  - Model and orbital information	*/
    struct CALSTR *calstr,  /* OUT - Calibration parameter structure	*/
    struct COEFFSTR *coeff  /* OUT - Calibration coeff struct		*/
);

void ccnonlin_energy
(
    long channel,		/* Channel number			*/
    float *energy		/* Energy in, corrected energy out	*/
);

void ccnonlin_temp
(
    struct MODEL *model,	/* AVHRR model structure		*/
    struct CALSTR *calstr,	/* Calibration parameter structure	*/
    float lut[][MAXDAT],	/* Calculated temperature LUT		*/
    long channel,		/* Channel number			*/
    long temp_limit		/* Maximum valid temperature limit	*/
);

int ccprim
(
    struct CALSTR *calstr,	/* Calibration parameter structure	      */
    long slprim,		/* Line at which to start priming	      */
    long mline,			/* Image master line number		      */
    struct COEFFSTR *coeff,	/* All coefficients (optical, thermal)	      */
    struct COEFFSTR *lastcoeff,	/* Last set of calibration coeff saved	      */
    struct OCOEFFSTR *origocoeff,/* Original optical coefficient struct	      */
    long *prtcnt,		/* PRT counts				      */
    long prtndx[],		/* Offsets to find the four sample prt values */
    long *prtpop,		/* PRT population (CALPER*PRTPER)	      */
    long *filedes,		/* file descriptor structure		      */
    double bnmrp[][60],		/* Numerator of beta function phihat*deltanu  */
    double enmr[][60],		/* Numerator of exp function		      */
    struct MODEL *model,	/* Model and orbital information	      */
    long intsum[],		/* Sum of internal counts		      */
    long spcsum[]		/* Sum of space counts			      */
);

int ccread_sat
(
    long satnum,		/* Satellite number			*/
    struct CALSTR *calstr	/* Calibration parameter structure	*/
);

int ccread_dgrd1
(
    char *hostname,		/* IN  - Calibration table host file name     */
    long satnum,		/* IN  - Satellite number		      */
    struct COEFFSTR *coeff	/* OUT - Calibration coefficient structure    */
);

int ccread_dgrd2
(
    char *hostname,	/* IN  - Calibration table host file name             */
    long adate,		/* IN  - Scene acquisition date                       */
    long satnum,	/* IN  - Satellite number                             */
    long *launch,	/* OUT - launch date as yyddd                         */
    double a[2],	/* OUT - ((counts/rad)/day)/day                       */
    double b[2],	/* OUT - (counts/rad)/day                             */
    double c[2],	/* OUT - counts/radiance                              */
    double d[2],	/* OUT - (counts/day)/day                             */
    double e[2],	/* OUT - counts/day                                   */
    double f[2]		/* OUT - counts                                       */
);

int ccfindstr
(
    long fd,			/* file descriptor			*/
    char buffer[MAXBUF],	/* buffer to read data into		*/
    char srchstr[]		/* String to search file in file	*/
);

int ccreadline
(
    long fd,			/* file descriptor			*/
    long nlines,		/* Number of lines to skip		*/
    char buffer[MAXBUF]		/* buffer to read data into		*/
);

int ccsconfig
(
    short min_frm[],		/* Buffer to read minor frame data into */
    struct MODEL *model,	/* Model and orbital information	*/
    struct CALSTR *calstr	/* Calibration parameters structure 	*/
);

int ccupdt
(
    struct CALSTR *calstr,	/* Calibration parameter structure	      */
    long curlin,		/* Current line                               */
    long mline,			/* Master line				      */
    short min_frm[],        	/* Buffer containing minor frame data         */
    struct COEFFSTR *coeff,	/* Calibration coefficients		      */
    struct COEFFSTR *lastcoeff,	/* Last set of Calibration coefficients	      */
    struct OCOEFFSTR *origocoeff,/* Original optical coefficients	      */
    long prtcnt[],		/* PRT counts                                 */
    long cntsum[],		/* Sum of prt counts                          */
    long prtsmp[],              /* Number of samples of prt counts            */
    long prtndx[],		/* Offsets to find the four sample prt values */
    long *nxtprt,		/* Index in PRTCNT of the next prt	      */
    long prtpop,		/* PRT population (CALPER*PRTPER)             */
    long *sampls,		/* Number of samps of target & space counts   */
    long intsum[],		/* Sum of internal counts                     */
    long spcsum[],		/* Sum of space counts                        */
    long slprim,                /* Starting line at which priming began	      */
    double bnmrp[][60],		/* Numerator of beta function phihat*deltanu  */
    double enmr[][60],		/* Numerator of exp function                  */
    struct MODEL *model,	/* Model and orbital information	      */
    long flag			/* Flag to force coefficient recalculation    */
);

void cctmp2rad
(
    struct CALSTR *calstr,	/* Calibration parameter structure	      */
    struct COEFFSTR *coeff,	/* Calibration coefficients		      */
    long *sampls,		/* Number of samps of target & space counts   */
    long intsum[],		/* Sum of internal counts                     */
    long spcsum[],		/* Sum of space counts                        */
    double bnmrp[][60],		/* Numerator of beta function phihat*deltanu  */
    double enmr[][60],		/* Numerator of exp function                  */
    double *tmpave		/* average temperature                        */
);

void ccillum_corr
(
    long curlin,		/* Current line                               */
    long mline,			/* Master line				      */
    struct CALSTR *calstr,	/* Calibration parameter structure	      */
    struct COEFFSTR *coeff,	/* Calibration coefficients		      */
    struct OCOEFFSTR *origocoeff, /* Original optical calibration coefficients*/
    struct MODEL *model		/* Model and orbital information	      */
);

int cctmp2rad3
(
    struct CALSTR *calstr,	/* Calibration parameter structure	      */
    struct COEFFSTR *coeff,	/* Calibration coefficients		      */
    long *sampls,		/* Number of samps of target & space counts   */
    long intsum[],		/* Sum of internal counts                     */
    long spcsum[],		/* Sum of space counts                        */
    double *tmpave		/* average temperature                        */
);

void findpolecross
(
    long *maxline,		/* line number of maximum lat		*/
    long *minline,		/* line number of minimum lat		*/
    struct MODEL *model		/* Model and orbital information	*/
);

void getline
(
    long quadnum,	/* Quadrant the latitude is wanted in		     */
    long mul,	 	/* Multiplier for min/max lat for start of scene     */
    double inlat,	/* Desired lat					     */
    long lines[3],	/* Line numbers of possible latitudes		     */
    long quads[3],	/* Quad numbers of line numbers found		     */
    struct MODEL *model	/* Model and orbital information		     */
);

void refine
(
    double lat1,
    double lat2,
    long *line1,	/* northern-most line			*/
    long *line2,	/* southern-most line			*/
    long maxln,		/* line number of maximum latitude	*/
    long minln,		/* line number of minimum latitude      */
    long passdir,	/* direction of the user-specified pass	*/
    struct MODEL *model /* Model and orbital information	*/
);

void refinepole
(
    double lat1,	/* lat on first side of pole		*/
    double lat2,	/* lat on other side of pole		*/
    long *line1,	/* line corr to lat1			*/
    long *line2,	/* line	corr to lat2			*/
    long pole,		/* North or South Pole			*/
    long poleln,	/* line number of the pole		*/
    long side,		/* side of image to search		*/
    struct MODEL *model /* Model and orbital information	*/
);

void sunearthdis
(
    long jday,		/* Julian day				*/
    float *aus		/* Number of astronomical units		*/
);

int ccread_noaa
(
long satnum,		/* IN - satellite number		*/
long acquisition_year,	/* IN - acquisition year		*/
long acquisition_jday,	/* IN - acquisition julian date		*/
struct CALSTR *calstr	/* OUT - calibration structure		*/
);

#endif
