/******************************************************************************
FUNCTION:	REFINEPOLE

PURPOSE:	Refine the line number of the lat of interest for lats
		crossing a pole; this is done because findline gives 
		only an approximation

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 5.0 	   12/91    T. Rockvam		Original development for LAS 5.0 
					based on fndsln from old makencal

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "model.h"

#define NORTH 1
#define SOUTH -1

void refinepole
(
    double lat1,	/* lat on first side of pole		*/
    double lat2,	/* lat on other side of pole		*/
    long *line1,	/* line corr to lat1			*/
    long *line2,	/* line	corr to lat2			*/
    long pole,		/* North or South Pole			*/
    long poleln,	/* line number of the pole		*/
    long side,		/* side of image to search		*/
    struct MODEL *model /* Model and orbital information	*/
)
{
double	poleline,	/* line number of pole			*/
	thislat,	/* lat at current line number		*/
	thislong,	/* long at current line number		*/
	elev = 0.0;	/* Elevation				*/

float	firstline,	/* first line (before pole)		*/
	secndline,	/* second line (after pole)		*/
	sample,		/* sample number			*/
	firststep,	/* how to move 1st line toward pole,2nd line away from*/
	secndstep;	/* how to move 1st line away from,2nd line toward pole*/

/*
  Set up lines of max & min latitude
*/
if (pole == NORTH)
    {
    firststep = 1,
    secndstep = -1;
    }
else
    {
    firststep = -1,
    secndstep = 1;
    }
poleline = (float)poleln;
firstline = (float)*line1;
secndline = (float)*line2;
sample = (float)side;

/*
  Find the latitude at the first line
*/
c_ls2ll(&firstline,&sample,&elev,model,&thislat,&thislong,NULL);

/*
  Refine number to exactly match first lat of interest
  If we get to the maximum image line go no further
*/
while ((thislat < lat1) && (firstline != poleline))
    {
    /* move toward pole */
    firstline += firststep;
    c_ls2ll(&firstline,&sample,&elev,model,&thislat,&thislong,NULL);
    }
while ((thislat > lat1) && (firstline != poleline))
    {
    /* move away from pole */
    firstline += secndstep;
    c_ls2ll(&firstline,&sample,&elev,model,&thislat,&thislong,NULL);
    }


/*
  Find the latitude at the second line
*/
c_ls2ll(&secndline,&sample,&elev,model,&thislat,&thislong,NULL);

/*
  Refine line number to exactly match second lat of interest
  if we get to the minimum image line go no further
*/
while ((thislat < lat2) && (secndline != poleline))
    {
    /* move toward pole */
    secndline += secndstep;
    c_ls2ll(&secndline,&sample,&elev,model,&thislat,&thislong,NULL);
    }
while ((thislat > lat2) && (secndline != poleline))
    {
    /* move away from pole */
    secndline += firststep;
    c_ls2ll(&secndline,&sample,&elev,model,&thislat,&thislong,NULL);
    }

*line1 = (long)firstline;
*line2 = (long)secndline;

return;
}
 
