/******************************************************************************
FUNCTION:	SUNEARTHDIS

PURPOSE:	This subroutine finds the distance between the sun and the
		earth depending on the Julian day

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      04/87    P. Mumford          Initial development
  1.1      01/90    C. Wivell           Optical sensor degradation and
                                        illumination correction
  5.0      02/90    D. Hollaren         Conversion to LAS-5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        ADAPS

ALGORITHM REFERENCES:  NONE

*******************************************************************************/
#include <math.h>
#include "worgen.h"
#include "constants.h"

void sunearthdis
(
    long jday,		/* Julian day				*/
    float *aus		/* Number of astronomical units		*/
)
{
float   m,		/* mean earth motion			*/
	nu,		/* mean anomaly				*/
	e,		/* eccentricity of earth's orbit	*/
	r0;		/* average earth-sun distance in au's	*/

r0 = 1.0;
e = 0.0167;
m = (float)(jday)/365.25 * PIE2;
nu = m + 2.0 * e * sin(m);
*aus = (r0 * (1.0 - e*e))/(1.0 + e * cos(nu));

return;
}
 
