/*******************************************************************************
FUNCTION:	bufwrite

PURPOSE:	Given an archive descriptor record, write the buffer to the
		archive image.

		Return E_SUCC if the write operation is successful and E_FAIL
		for all errors.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#ifdef sun
#include <sys/types.h>
#include <unistd.h>
#endif

#include "las.h"
#include "tapeio.h"
#include "adaps.h"
#include "archio.h"


int bufwrite
(
    struct ARCDESC *arc     /* IN - Archive descriptor record                 */
)
{
char str[CMLEN];            /* Temporary string                               */

long nbytes;                /* Number of bytes to seek in the archive image   */
long nonfatal = 1;          /* Nonfatal error flag                            */
long status;                /* Return status from a function call             */


/* Check the access of the archive image
----------------------------------------*/
if ((arc->access != IWRITE) && (arc->access != IUPDATE))
    {
    strcpy(str, "Archive image not opened for write or update access");
    c_errmsg(str, "bufwrite-access", &nonfatal);
    return(E_FAIL);
    }


/* Ensure the image is on disk and not tape
-------------------------------------------*/
if (!arc->disk)
    {
    strcpy(str, "Archive image not on disk");
    c_errmsg(str, "bufwrite-disk", &nonfatal);
    return(E_FAIL);
    }


/* Ensure the first and last values are valid
---------------------------------------------*/
if ((arc->first < 1) || (arc->last < 1) || (arc->last < arc->first))
    {
    strcpy(str, "Invalid buffer bounds");
    c_errmsg(str, "bufwrite-bounds", &nonfatal);
    return(E_FAIL);
    }


/* Seek to the correct position in the file
-------------------------------------------*/
if (arc->ascending && !arc->satview)
    {
    nbytes = arc->last * arc->arcbytes * -1;
    status = (long)lseek(arc->fdesc, nbytes, SEEK_END);
    if (status < 0)
	{
	strcpy(str, "Error seeking in the archive image");
	c_errmsg(str, "bufwrite-seek", &nonfatal);
	return(E_FAIL);
	}
    }
else
    {
    nbytes = (arc->first - 1) * arc->arcbytes;
    status = (long)lseek(arc->fdesc, nbytes, SEEK_SET);
    if (status != nbytes)
	{
	strcpy(str, "Error seeking in the archive image");
	c_errmsg(str, "bufwrite-seek", &nonfatal);
	return(E_FAIL);
	}
    }


/* Write the entire buffer to the image file
--------------------------------------------*/
nbytes = (arc->last - arc->first + 1) * arc->arcbytes;
status = write(arc->fdesc, arc->packed, nbytes);
if (status != nbytes)
    {
    strcpy(str, "Error writing the archive image");
    c_errmsg(str, "bufwrite-write", &nonfatal);
    return(E_FAIL);
    }


/* Set the savebuf flag to indicate the buffer does not need to be saved
------------------------------------------------------------------------*/
arc->savebuf = FALSE;
return(E_SUCC);
}
 
