/****************************************************************************
NAME:		C_GETADDR

PURPOSE:	Get the AVHRR DDR information from an associated file

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		7/91	D. Hollaren	   CSB      Original development

ALGORITHM DESCRIPTION:
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "vll.h"
#include "model.h"
#include "archio.h"

/*
  Number of records in an archive header
*/
#define MAXHDRRECS (HDR_RECS + ATT_RECS + 1)

int c_getaddr
(
    char *host_img,		  /* Host input image name		     */
    struct MODEL *model		  /* Model and orbital information	     */
)
{
long	addr_fd,		  /* Avhrr DDR file descriptor		     */
	status,			  /* Return status variable		     */
	size,			  /* Size of header information		     */
	i,			  /* Looping counter			     */
	version,		  /* Header version number		     */
	zero = 0,		  /* Constant of zero			     */
	access = IREAD,		  /* File access indicator		     */
	nonfatal = 1;		  /* Non-fatal error code		     */

char	host_addr[CMLEN],	  /* Host input avhrr ddr name		     */
	dbuf[1],		  /* Data buffer for label services routines */
	lskey[1],		  /* Key field for label services routines   */
        buffer[MAXHDRRECS][HDR_SIZ+1];/* Buffer for the header        	     */
char dtype[4];

/*
  Create the avhrr ddr host file name
*/
c_lsmknm(host_img,"addr",host_addr);

/*
  Open the AVHRR DDR file
*/
status = c_lsopen(&addr_fd,host_addr,&access);
if (status != E_SUCC)
    {
    c_errmsg("Error opening the avhrr ddr label services file","getaddr-open",
	     &nonfatal);
    return(E_FAIL);
    }

/*
  Read the image information records from the AVHRR DDR and determine the
  version of the header
*/
lskey[0] = '\0';
for (i = 0; i < MAXHDRRECS; i++)
    {
    size = HDR_SIZ + 1;
    strcpy(dtype,"B");
    status = c_lsread(&addr_fd,lskey,&size,&zero,buffer[i],
		      (unsigned char *)dbuf,dtype);
    if (status == E_EOF)
	break;
    else if (status != E_SUCC)
	{
	c_errmsg("Error reading the header from the avhrr ddr file",
		 "getaddr-read",&nonfatal);
	return(E_FAIL);
	}
    }

/*
  Close the avhrr ddr file
*/
status = c_lsclos(&addr_fd,host_addr,&zero);
if (status != E_SUCC)
    {
    c_errmsg("Error closing the avhrr ddr label services file","getaddr-close",
	     &nonfatal);
    return(E_FAIL);
    }

/* Determine the version of the ADDR.  Not a really good thing to
   look at but it is the only thing in the old ADDR that is unique.
------------------------------------------------------------------*/
if (strncmp(&buffer[2][3],"ORoll",5) == 0)
    version = 1;
else
    version = 2;

/*
  Set the image information into the model structure
*/
c_sethead(buffer,model,version);

/*
  Send the buffer to set the satellite attributes
*/
if (version == 1)
    {
    i = HDR_RECS;
    model->sat_view = FALSE;
    }
else
    {
    i = HDR_RECS_V2;
    c_setatt((char (*)[])&buffer[i][0],model);
    i += ATT_RECS;
    }

/*
  Set the ephemris information into the model structure
*/
c_setephem(&buffer[i][9],model);

/*
  Initialize the satellite attitude correction coefficients
model->delta_alt = 0;
for (i = 0; i < 5; i++)
    {
    model->yaw_coefs[i] = 0.0;
    model->roll_coefs[i] = 0.0;
    model->pitch_coefs[i] = 0.0;
    }
*/

/*
  Set the satellite information into the model structure.
*/
c_setsat(model);

return(E_SUCC);
}
 
