/*******************************************************************************
NAME:		C_GETAHDR

PURPOSE:	Get the associated archive header information and place the
		information in the AVHRR model and dropped line structures.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    02/92  D.Hollaren       Development
  2.1    04/93  J.Fenno          Modified so the entire archive header is read
				 into a buffer and sent to c_hdrread().

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "worgen.h"
#include "labserv.h"
#include "adaps.h"
#include "archio.h"

#define MAXHDRRECS (DRP_RECS + CEOS_RECS) /* # of records in an archive header*/


int c_getahdr
(
    char *arch_name,          /* IN  - Name of the archive image            */
    struct MODEL *model,        /* OUT - Model and orbital information        */
    struct DROPPED *drop	/* OUT - Archive dropped line information     */
)
{
long	access = IREAD,		/* File access indicator                      */
	bytes_read,             /* The number of bytes read from the header   */
	fd,			/* Archive header file descriptor             */
	i;                      /* Loop control                               */

char	ahdr_name[CMLEN],	/* Name of the associated archive header      */
	buffer[MAXHDRRECS][HDR_SIZ+1];/* Buffer for the archive header        */


/* Create the archive header name
---------------------------------*/
c_lsmknm(arch_name, "ahdr", ahdr_name);


/* Open archive header file
---------------------------*/
if (c_hdropen((int *)&fd, ahdr_name, &access) != E_SUCC)
    return(E_FAIL);


/* Read all of the records from the archive header
--------------------------------------------------*/
bytes_read = HDR_SIZ;
i = 0;
while ((bytes_read == HDR_SIZ) && (i < MAXHDRRECS))
    {
    bytes_read = read(fd, buffer[i], HDR_SIZ);
    buffer[i++][bytes_read] = '\0';
    }


/* Close the archive header file
--------------------------------*/
c_hdrclose((int *)&fd);


/* Pass the buffer to c_hdrread() to extract the data
-----------------------------------------------------*/
if (c_hdrread(buffer, model, drop) != E_SUCC)
    return(E_FAIL);

return(E_SUCC);
}
 
