/*******************************************************************************
NAME:		C_HDRREAD

PURPOSE:	Read the AVHRR archive header information and place it into the
		model and dropped line structures.
		
Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Conversion to LAS5.0
  2.1    04/92  J.Fenno          Rewrote to use the CEOS_IEF header format.
 				 Placed the EROS Archive Header in the free
				 text portion of the CEOS_IEF header.
  2.2    04/93  J.Fenno          The entire archive header is now passed in a
				 buffer.

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "tae.h"
#include "adaps.h"
#include "archio.h"


int c_hdrread
(
    char buffer[][HDR_SIZ+1], /* IN  - Buffer containing archive header     */
    struct MODEL *model,      /* OUT - Model and orbital information        */
    struct DROPPED *drop      /* OUT - Dropped line index information       */
)
{
char    str[CMLEN];             /* Temporary working string                   */

long    i,		        /* Looping counter                            */
        nonfatal = 1,	        /* Non-fatal error code                       */
        skip,                   /* Loop control                               */
	version;                /* The version of the header being read       */



/* Ensure this is a CEOS_IEF header
-----------------------------------*/
if (strncmp(buffer[0], CEOS_START, strlen(CEOS_START)) != 0)
    {
    c_errmsg("Archive header not CEOS_IEF format", "hdrread-ceos", &nonfatal);
    return(E_FAIL);
    }


/* As long as the first two characters are "/" and "*", skip
   lines looking for the starting string of the EDC archive header
------------------------------------------------------------------*/
skip  = TRUE;
version = 0;
i = 1;
while (skip)
    {
    if (strncmp(buffer[i], "/*", 2) != 0)
	skip = FALSE;
    else
	{
	sscanf(&buffer[i][3], "%s", str);
	if (strcmp(str, SFL_START) == 0)
	    {
	    skip = FALSE;
	    version = 1;
	    }
	else if (strcmp(str, SFL_START_V2) == 0)
	    {
	    skip = FALSE;
	    version = 2;
	    }
	}
    i++;
    }


/* If the version is zero then we must have found the first line of meta data
-----------------------------------------------------------------------------*/
if (version == 0)
    {
    c_errmsg("EDC archive header not present", "hdrread-header", &nonfatal);
    return(E_FAIL);
    }


/* Send the buffer to c_sethead()
---------------------------------*/
c_sethead((char (*)[])&buffer[i][0], model, version);
if (version == 1)
    {
    i += HDR_RECS;
    model->sat_view = TRUE;
    }
else if (version == 2)
    {
    i += HDR_RECS_V2;
    c_setatt((char (*)[])&buffer[i][0], model);
    i += ATT_RECS;
    }

/* Send the buffer to c_setephem()
----------------------------------*/
c_setephem(&buffer[i][9], model);
i += 1;


/* Set the satellite information in the model structure
-------------------------------------------------------*/
c_setsat(model);


/* Send the buffer to c_setdrop()
---------------------------------*/
if (c_setdrop((char (*)[])&buffer[i][0], drop) != E_SUCC)
    return(E_FAIL);

return(E_SUCC);
}
 
