/*******************************************************************************
NAME:		C_HDRWRITE

PURPOSE:	Write the archive header information 
		

Version  Date	Programmer       Modification
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Conversion to LAS5.0
  2.1    04/92  J.Fenno          Rewrote to use the CEOS_IEF header format.
                                 Placed the EROS Archive Header in the free
				 text portion of the CEOS_IEF header.
  2.2    09/92  J.Fenno          Corrected error messages.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "worgen.h"
#include "labserv.h"
#include "tae.h"
#include "model.h"
#include "archio.h"

int c_hdrwrite
(
int    *out_fd,         /* IN - File descriptor to archive header           */
struct MODEL   *model,	/* IN - Model and orbital information               */
struct DROPPED *drop 	/* IN - Dropped line index information              */
)
{
char	buffer[DRP_RECS][HDR_SIZ+1]; /* Buffer to write archive header with  */

long	nbuf,		/* Number of dropped line ASCII buffers to write     */
	now,            /* Current date and time encoded as a long           */
	status,		/* Return status from writes			     */
	nonfatal = 1,	/* Non-fatal error code				     */
	i;		/* Looping control			             */

struct tm *today;       /* Structure containing the current date and time    */


/* Determine the current date and time
--------------------------------------*/
now   = time(NULL);
today = localtime(&now);


/* Create and write the CEOS_IEF header
---------------------------------------*/
strcpy(buffer[0], CEOS_START);
for (i = (long)strlen(buffer[0]); i < HDR_SIZ - 2; i++)
   buffer[0][i] = ' ';
buffer[0][i] = '\0';
strcat(buffer[0], "*/");

strcpy(buffer[1], "/*SFL");
for (i = (long)strlen(buffer[1]); i < HDR_SIZ - 2; i++)
   buffer[1][i] = ' ';
buffer[1][i] = '\0';
strcat(buffer[1], "*/");

/************************************************************
DLLOYD Y2K
  Okay, the tm_year is years from 1900 so in year 2000 it 
  would be 100 and MODed would be 00 which is what we want.
*************************************************************/
sprintf(buffer[2], "/*%02d%02d%02d%02d%02d%02d", today->tm_year % 100,
   today->tm_mon + 1, today->tm_mday, today->tm_hour, today->tm_min,
   today->tm_sec % 60);
for (i = (long)strlen(buffer[2]); i < HDR_SIZ - 2; i++)
   buffer[2][i] = ' ';
buffer[2][i] = '\0';
strcat(buffer[2], "*/");

strcpy(buffer[3], "/*NEWACQ");
for (i = (long)strlen(buffer[3]); i < HDR_SIZ - 2; i++)
   buffer[3][i] = ' ';
buffer[3][i] = '\0';
strcat(buffer[3], "*/");

strcpy(buffer[4], "/*00001");
for (i = (long)strlen(buffer[4]); i < HDR_SIZ - 2; i++)
   buffer[4][i] = ' ';
buffer[4][i] = '\0';
strcat(buffer[4], "*/");

for (i = 0; i < 5; i++)
   {
   status = write(*out_fd, buffer[i], HDR_SIZ);
   if (status < 0)
      {
      c_errmsg("Error writing the ASCII header information",
	 "hdrwrite-write", &nonfatal);
      return(E_FAIL);
      }
   }


/* Write the SFL_START string
-----------------------------*/
sprintf(buffer[0], "/* %s", SFL_START_V2);
for (i = (long)strlen(buffer[0]); i < HDR_SIZ - 2; i++)
   buffer[0][i] = ' ';
buffer[0][i] = '\0';
strcat(buffer[0], "*/");

status = write(*out_fd, buffer[0], HDR_SIZ);
if (status < 0)
    {
    c_errmsg("Error writing the ASCII header information",
       "hdrwrite-write", &nonfatal);
    return(E_FAIL);
    }


/* Convert the image information from the model structure to ASCII
------------------------------------------------------------------*/
c_tohead(model,buffer);
for (i = 0; i < HDR_RECS_V2; i++)
   {
   status = write(*out_fd, buffer[i], HDR_SIZ);
   if (status < 0)
      {
      c_errmsg("Error writing the ASCII header information",
	 "hdrwrite-write", &nonfatal);
      return(E_FAIL);
      }
   }

/* Convert the attitude correction information from the model
   structure to ASCII
--------------------------------------------------------------*/
c_toatt(model,buffer);
for (i = 0; i < ATT_RECS; i++)
   {
   status = write(*out_fd, buffer[i], HDR_SIZ);
   if (status < 0)
      {
      c_errmsg("Error writing the ASCII header information",
	 "hdrwrite-write", &nonfatal);
      return(E_FAIL);
      }
   }

/* Convert the ephemeris information to ASCII
---------------------------------------------*/
c_toephem(model,buffer[0]);

status = write(*out_fd, buffer[0], HDR_SIZ);
if (status < 0)
    {
    c_errmsg("Error writing the ASCII ephemeris information",
       "hdrwrite-write", &nonfatal);
    return(E_FAIL);
    }


/* Convert the dropped line information to ASCII
------------------------------------------------*/
c_todrop(drop,buffer,&nbuf);
for (i = 0; i < nbuf; i++)
   {
   status = write(*out_fd, buffer[i], HDR_SIZ);
   if (status < 0)
      {
      c_errmsg("Error writing the ASCII header information",
	 "hdrwrite-write", &nonfatal);
      return(E_FAIL);
      }
   }


/* Write the SFL_END string
---------------------------*/
sprintf(buffer[0], "/* %s", SFL_END);
for (i = (long)strlen(buffer[0]); i < HDR_SIZ - 2; i++)
   buffer[0][i] = ' ';
buffer[0][i] = '\0';
strcat(buffer[0], "*/");

status = write(*out_fd, buffer[0], HDR_SIZ);
if (status < 0)
    {
    c_errmsg("Error writing the ASCII header information",
       "hdrwrite-write", &nonfatal);
    return(E_FAIL);
    }


/* Write the CEOS_IEF Meta Data
-------------------------------*/
c_tometa(model,buffer);
for (i = 0; i < 2; i++)
   {
   status = write(*out_fd, buffer[i], HDR_SIZ);
   if (status < 0)
      {
      c_errmsg("Error writing the ASCII header information",
	 "hdrwrite-write", &nonfatal);
      return(E_FAIL);
      }
   }


/* Write the CEOS_END string
----------------------------*/
strcpy(buffer[0], CEOS_END);
for (i = (long)strlen(CEOS_END); i < HDR_SIZ - 2; i++)
   buffer[0][i] = ' ';
buffer[0][i] = '\0';
strcat(buffer[0], "*/");

status = write(*out_fd, buffer[0], HDR_SIZ);
if (status < 0)
    {
    c_errmsg("Error writing the ASCII header information",
       "hdrwrite-write", &nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
 
