/****************************************************************************
NAME:		C_PUTADDR

PURPOSE:	Get the AVHRR DDR information from an associated file

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		7/91	D. Hollaren	   CSB      Original development

ALGORITHM DESCRIPTION:
*****************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "labserv.h"
#include "tae.h"
#include "vll.h"
#include "model.h"
#include "archio.h"

int c_putaddr
(
    char *host_img,		  /* Host input image name		     */
    struct MODEL *model		  /* Model and orbital information	     */
)
{
long	addr_fd,		  /* Avhrr DDR file descriptor		     */
	status,			  /* Return status variable		     */
	size,			  /* Size of header information		     */
	i,			  /* Looping counter			     */
	zero = 0,		  /* Constant of zero			     */
	access = IWRITE,	  /* File access indicator		     */
	nonfatal = 1;		  /* Non-fatal error code		     */

char	host_addr[CMLEN],	  /* Host input avhrr ddr name		     */
	buffer[HDR_RECS_V2][HDR_SIZ+1],/* Header data buffer		     */
	dbuf[1],		  /* Data buffer for label services routines */
	lskey[1];		  /* Key field for label services routines   */


/*
  Create the avhrr DDR host file name
*/
c_lsmknm(host_img,"addr",host_addr);

/*
  Open the AVHRR DDR file
*/
status = c_lsopen(&addr_fd,host_addr,&access);
if (status != E_SUCC)
    {
    c_errmsg("Error opening avhrrddr label services file","putaddr-open",
	     &nonfatal);
    return(E_FAIL);
    }

/*
  Set the ascii image information
*/
c_tohead(model,buffer);

/*
  Write the ascii image information to the AVHRR DDR file
*/
lskey[0] = '\0';
for (i = 0; i < HDR_RECS_V2; i++)
    {
    size = HDR_SIZ;
    status = c_lswrit(&addr_fd,lskey,&size,&zero,buffer[i],dbuf,"B");
    if (status != E_SUCC)
	{
	c_errmsg("Error writing the header to the avhrr ddr file",
		 "putaddr-write",&nonfatal);
	return(E_FAIL);
	}
    }

/*
  Set the ascii attitude corrections information
*/
c_toatt(model,buffer);

/*
  Write the ascii attitude corrections information to the AVHRR DDR file
*/
for (i = 0; i < ATT_RECS; i++)
    {
    size = HDR_SIZ;
    status = c_lswrit(&addr_fd,lskey,&size,&zero,buffer[i],dbuf,"B");
    if (status != E_SUCC)
	{
	c_errmsg("Error writing the header to the avhrr ddr file",
		 "putaddr-write",&nonfatal);
	return(E_FAIL);
	}
    }

/*
  Set the ascii ephemris information
*/
c_toephem(model,buffer[0]);

/*
  Write the ascii ephemeris information to the AVHRR DDR file
*/
size = HDR_SIZ;
status = c_lswrit(&addr_fd,lskey,&size,&zero,buffer[0],dbuf,"B");
if (status != E_SUCC)
    {
    c_errmsg("Error writing the ephemeris to the avhrr ddr file",
	     "putaddr-write",&nonfatal);
    return(E_FAIL);
    }

/*
  Close the AVHRR DDR file
*/
status = c_lsclos(&addr_fd,host_addr,&zero);
if (status != E_SUCC)
    {
    c_errmsg("Error closing the avhrr ddr label services file","putaddr-close",
	     &nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
 
