/****************************************************************************
NAME:		C_PUTAHDR

PURPOSE:	Put the header, ephemeris, and dropped line structures to
		the associated archive header file.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		REASON
-------		----	------		------
  2.0		02/92	D. Hollaren	Original development

ALGORITHM DESCRIPTION:
*****************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "labserv.h"
#include "model.h"
#include "archio.h"

int c_putahdr
(
    char *arch_name,	  	  /* Name of the archive image		     */
    struct MODEL *model,	  /* Model and orbital information	     */
    struct DROPPED *drop	  /* Archive dropped line information        */
)
{
long	ahdr_fd,		  /* Archive header file descriptor	     */
	status,			  /* Return status variable		     */
	access = IWRITE;	  /* File access indicator		     */

char	ahdr_name[CMLEN];	  /* Name of the associated archive header   */

/*
  Create the avhrr ddr host file name
*/
c_lsmknm(arch_name,"ahdr",ahdr_name);

/*
  Open associated archive header file
*/
status = c_hdropen((int *)&ahdr_fd,ahdr_name,&access);
if (status != E_SUCC)
    return(E_FAIL);

/*
  Write the associated archive header file
*/
status = c_hdrwrite((int *)&ahdr_fd,model,drop);
if (status != E_SUCC)
    return(E_FAIL);

/*
  Close the associated archive header file
*/
c_hdrclose((int *)&ahdr_fd);

return(E_SUCC);
}
 
