/*******************************************************************************
NAME:		C_PUTDRPL	

PURPOSE:	Generate the LAS associated AVHRR DRPL (dropped line file).

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/97  T.Baltzer        Development
				
PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h> 
#include "las.h"
#include "labserv.h"
#include "adaps.h"
#include "archio.h"


int c_putdrpl
(
    char *image,		/* Host file name of output image             */
    struct DROPPED *drop	/* Dropped line data structure                */
)

{
FILE *drpl_fp;			/* Dropped line output file pointer           */

char drpl_fname[CMLEN];		/* Dropped line file name                     */

long i;				/* Loop counter                               */
long total_missed;		/* Total number of dropped lines              */



/* Create the name of the associated dropped line file, open it for writing.
---------------------------------------------------------------------------*/
c_lsmknm(image, "drpl", drpl_fname);
drpl_fp = fopen(drpl_fname, "w");
if (drpl_fp == NULL)
    {
    c_lasmsg (FALSE,"putdrpl-open","Error opening dropped line file");
    return(E_FAIL);
    }


/* Compute the total number of drop lines
------------------------------------------*/
total_missed = 0;
for (i = 0; i < drop->ngaps; i++)
    total_missed += drop->nmissed[i];


/* Write the initial ascii header containing the number
   of gaps and the total number of dropped lines
-------------------------------------------------------*/
fprintf(drpl_fp, "%8ld %8ld\n", drop->ngaps, total_missed);


/* Write all dropped line records 
---------------------------------*/
for (i = 0; i < drop->ngaps; i++)
    fprintf(drpl_fp, "%8ld %8ld\n", drop->lmissed[i], drop->nmissed[i]);


/* Close the associated drop line file
--------------------------------------*/
fclose(drpl_fp);


return(E_SUCC);
}
