/*******************************************************************************
NAME:		C_SETATT

PURPOSE:	Convert the ASCII attitude correction information into
		the model structure.
		

Version  Date   Author		 Reason
-------  -----	---------------  -----------------------------------------------
  2.0    03/93  D.Hollaren       Original development

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "model.h"
#include "constants.h"

void c_setatt
(
    char buffer[][HDR_SIZ+1], /* IN  - Buffer containing ASCII header info   */
    struct MODEL *model	      /* OUT - Model and orbital information	     */
)
{
long	i;		    /* Looping variable				     */

/* Read the first line of the archive header information
--------------------------------------------------------*/
sscanf(&buffer[0][9], "%lf", &model->delta_time);
if (strncmp(&buffer[0][23],"Dalt",4) == 0)
    sscanf(&buffer[0][28], "%lf", &model->delta_alt);
else
    model->delta_alt = 0.0;

/* Read the second line of the archive header information
---------------------------------------------------------*/
sscanf(&buffer[1][9], "%lf %lf %lf %lf %lf", &model->roll_coefs[0],
	&model->roll_coefs[1], &model->roll_coefs[2], &model->roll_coefs[3],
	&model->roll_coefs[4]);

/* Read the third line of the archive header information
--------------------------------------------------------*/
sscanf(&buffer[2][9], "%lf %lf %lf %lf %lf", &model->pitch_coefs[0],
	&model->pitch_coefs[1], &model->pitch_coefs[2], &model->pitch_coefs[3],
	&model->pitch_coefs[4]);

/* Read the fourth line of the archive header information
---------------------------------------------------------*/
sscanf(&buffer[3][9], "%lf %lf %lf %lf %lf", &model->yaw_coefs[0],
	&model->yaw_coefs[1], &model->yaw_coefs[2], &model->yaw_coefs[3],
	&model->yaw_coefs[4]);

/*
  Convert the roll, pitch, and yaw coefficients to radians.  These guys
  were saved as degrees for presicion
*/
for (i = 0; i < 5; i++)
    {
    model->roll_coefs[i]  *= RAD_PER_DEG;
    model->pitch_coefs[i] *= RAD_PER_DEG;
    model->yaw_coefs[i]   *= RAD_PER_DEG;
    }

return;
}
 
