/*******************************************************************************
NAME:		C_SETDROP

PURPOSE:	Convert the ASCII dropped line information and place it
		into the dropped line structure.
		
Version  Date   Author           Reason
-------	 -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Development
  2.1    04/92  J.Fenno          Rewrote to use the CEOS_IEF header format.
                                 Placed the EROS Archive Header in the free text
				 portion of the CEOS_IEF header.
  2.2    09/92  J.Fenno          Updated comments
  2.3    04/93  J.Fenno          Modified to check for SFL_HEAD_END

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "tae.h"
#include "adaps.h"
#include "archio.h"


int c_setdrop
(
    char buffer[][HDR_SIZ+1],	/* IN  - Buffer holding dropped line info     */
    struct DROPPED *drop	/* OUT - Dropped line structure               */
)
{
char    str[CMLEN];		/* Temporary string                           */
long    i,			/* Loop control                               */
	line,			/* Current line number in the buffer          */
	nonfatal = 1,		/* Nonfatal error flag                        */
	position;		/* Current position in the buffer             */


/* If the first line of the buffer starts with the string
   SFL_END, then there is no dropped line information.
---------------------------------------------------------*/
sscanf(&buffer[0][3], "%s", str);
if (strncmp(str, SFL_END, strlen(SFL_END)) == 0)
    {
    drop->ngaps = 0;
    return(E_SUCC);
    }


/* Retrieve the number of gaps from the first line of the buffer
----------------------------------------------------------------*/
if (sscanf(&buffer[0][8], "%ld", &drop->ngaps) != 1)
    {
    c_errmsg("Error reading number of gaps", "setdrop-ngaps", &nonfatal);
    drop->ngaps = 0;
    return(E_SUCC);
    }


/* Retrieve each gap.  Each gap is of the form xxxxx-yyyyy where xxxxx is the
   first line in the gap and yyyyy is the number of lines in the gap.  The
   first gap is stored in position 15 of the first line in the buffer.  On all
   of the remaining lines the first gap is stored in position 3.  Because each
   gap is 11 characters long and there is one space separating them, we
   increment our position in the line by 12.
------------------------------------------------------------------------------*/
line     =  0;
position = 15;
for (i = 0; i < drop->ngaps; i++)
    {
    if (position > 63)
	{
	position = 3;
	line++;
	}

    if (sscanf(&buffer[line][position], "%ld-%ld", &drop->lmissed[i],
	&drop->nmissed[i]) != 2)
	{
	c_errmsg("Error reading dropped line information",
	    "setdrop-drop", &nonfatal);
	c_errmsg("Nonfatal error encountered", "setdrop-drop", &nonfatal);
	drop->ngaps = i;
	return(E_SUCC);
	}
    position += 12;
    }

return(E_SUCC);
}
 
