/*******************************************************************************
NAME:    	C_TOATT

PURPOSE:	Convert the attitude corrections informatino in the model
		structure to an ascii buffer.
		
PROGRAM HISTORY:

Version  Date   Author           Reason
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Original development
  2.1    04/92  J.Fenno          Rewrote to use the CEOS_IEF header format.
                                 Placed the EROS Archive Header in the free text
				 portion of the CEOS_IEF header.
  2.2    09/92  J.Fenno          Added ending orbit to the archive header and
				 changed the total number of missed lines from a
				 4 digit number to a 5 digit number.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "model.h"
#include "constants.h"
#include "archio.h"
#include "local_protos.h"


void c_toatt
(
    struct MODEL *model,	/* IN  - Model and orbital information	     */
    char buffer[][HDR_SIZ+1]	/* OUT - Buffer ASCII header information     */
)
{
char tmp_str[HDR_SIZ + 1];	/* Tmp string for building entries of header */

long i;				/* Loop control                              */

double	roll[5],		/* Roll coefficients in degrees		     */
	pitch[5],		/* Pitch coefficients in degrees	     */
	yaw[5];			/* Yaw coefficients in degrees		     */

/* Blank every line of the buffer needed for the header information
-------------------------------------------------------------------*/
for (i = 0; i < ATT_RECS; i++)
   blank_line(buffer[i]);


/* Place the delta time in the first line of the buffer
-------------------------------------------------------*/
strncpy(&buffer[0][3],  "Dtime ", 6);
sprintf(tmp_str,"%+013.5lf",model->delta_time);
strncpy(&buffer[0][9], tmp_str, 13);

strncpy(&buffer[0][23],  "Dalt ", 5);
sprintf(tmp_str,"%+013.5lf",model->delta_alt);
strncpy(&buffer[0][28], tmp_str, 13);

/*
  Convert the roll, pitch, and yaw coefficients to degrees so that more
  presicion can be saved.
*/
for (i = 0; i < 5; i++)
    {
    roll[i]  = model->roll_coefs[i]  * DEG_PER_RAD;
    pitch[i] = model->pitch_coefs[i] * DEG_PER_RAD;
    yaw[i]   = model->yaw_coefs[i]   * DEG_PER_RAD;
    }

/* Place the roll coeffecients in the second line of the buffer
---------------------------------------------------------------*/
strncpy(&buffer[1][3],  "Roll  ", 6);
sprintf(tmp_str,"%+013.10lf %+013.10lf %+013.10lf %+013.10lf %+013.10lf",
	roll[0],roll[1],roll[2],roll[3],roll[4]);
strncpy(&buffer[1][9], tmp_str, 69);

/* Place the pitch coeffecients in the third line of the buffer
---------------------------------------------------------------*/
strncpy(&buffer[2][3],  "Pitch ", 6);
sprintf(tmp_str,"%+013.10lf %+013.10lf %+013.10lf %+013.10lf %+013.10lf",
	pitch[0],pitch[1],pitch[2],pitch[3],pitch[4]);
strncpy(&buffer[2][9], tmp_str, 69);

/* Place the yaw coeffecients in the fourth line of the buffer
---------------------------------------------------------------*/
strncpy(&buffer[3][3],  "Yaw   ", 6);
sprintf(tmp_str,"%+013.10lf %+013.10lf %+013.10lf %+013.10lf %+013.10lf",
	yaw[0],yaw[1],yaw[2],yaw[3],yaw[4]);
strncpy(&buffer[3][9], tmp_str, 69);

return;
}

