/******************************************************************************
NAME:		C_TODROP

PURPOSE:	Convert the dropped line structure to an ascii buffer.
		
PROGRAM HISTORY:

Version  Date   Author           Reason
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Original developement
  2.1    04/92  J.Fenno          Rewrote to use the CEOS_IEF header format.
                                 Placed the EROS Archive Header in the free text
				 portion of the CEOS_IEF header.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "archio.h"
#include "local_protos.h"


void c_todrop
(
    struct DROPPED *drop,       /* IN  - Dropped line index information       */
    char buffer[][DRP_SIZ+1],   /* OUT - Buffer containing dropped line info  */
    long *nbuf			/* OUT - Number of dropped line buffers       */
)
{
char tmp_str[DRP_SIZ+1];       /* Temp string for storing gap information    */

long i;                        /* Loop control                               */
long position;                 /* Index where the next element is placed     */


*nbuf = 0;
i = 0;
while (i < drop->ngaps)
   {
   *nbuf += 1;

   /* Blank out the current buffer
   -------------------------------*/
   blank_line(buffer[*nbuf - 1]);
   position = 3;

   /* If this is the first buffer then write the gap label
   -------------------------------------------------------*/
   if (*nbuf == 1)
      {
      sprintf(tmp_str, "GAPS %05ld: ", drop->ngaps);
      strncpy(&buffer[*nbuf - 1][position], tmp_str, 11);
      position += 12;
      }
   
   /* Place as many gap records as possible in this buffer
   -------------------------------------------------------*/
   while ((i < drop->ngaps) && (position <= 63))
      {
      sprintf(tmp_str, "%05ld-%05ld", drop->lmissed[i],
	      drop->nmissed[i]);
      strncpy(&buffer[*nbuf - 1][position], tmp_str, 11);
      i++;
      position += 12;
      }
   }
return;
}
 
