/*******************************************************************************
NAME:		C_TOEPHEM

PURPOSE:	Convert the ephemeris structure to an ascii buffer.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		REASON
-------		----	------		------
  2.0		02/92	D. Hollaren	Original development
  2.1           04/92   J.Fenno         Rewrote to use the CEOS_IEF header
					format.  Placed the EROS Archive Header
					in the free text portion of the CEOS_IEF
					header.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "model.h"
#include "constants.h"

void c_toephem
(
    struct MODEL *model,	/* Model and orbital information	     */
    char buffer[]		/* Asci buffer for ephemeris information     */
)
{
char    tmp_str[HDR_SIZ + 1];   /* Temp string to store the ephemris data    */

long    i;                      /* Loop control                              */
long	mean_anomaly,		/* Mean anomaly (REVS)			     */
	mean_motion,		/* Mean motion (RAD/HERG)		     */
	decay,			/* Orbital decay (1.0E-5 RADS/HERG^^2)       */
	ecc,			/* Eccentricity				     */
	arg_per,		/* Argument of perigee (REVS)		     */
	lon_anode,		/* Longitude of ascending node		     */
	inclin;			/* Inclination (REVS)			     */


/* Unconvert the elements that have an implied decimal point
------------------------------------------------------------*/
mean_anomaly = (long)((model->mean_anomaly / PIE2) * 100000000.0);
mean_motion  = (long)(model->mean_motion * 100000000.0);
decay        = (long)((model->decay / 1.0e-5) * 100000.0);
ecc          = (long)(model->ecc * 100000000.0);
arg_per      = (long)((model->arg_per / PIE2) * 100000000.0);
lon_anode    = (long)((model->lon_anode / PIE2) * 100000000.0);
inclin       = (long)((model->inclin / PIE2) * 100000000.0);


/* Convert the ephemeris to ascii
---------------------------------*/
sprintf(tmp_str,"%05ld%05ld%08ld%08ld%05ld%08ld%08ld%08ld%08ld%02ld%02ld%02ld",
	model->satid, model->orbit, mean_anomaly, mean_motion,
	decay, ecc, arg_per, lon_anode, inclin, model->epoch_year,
	model->epoch_month, model->epoch_day);


/* Fill the buffer
------------------*/
strcpy(buffer, "/* EPHEM ");
strcat(buffer, tmp_str);
for (i = (long)strlen(buffer); i < HDR_SIZ - 2; i++)
   buffer[i] = ' ';
buffer[i] = '\0';
strcat(buffer, "*/");

return;
}
 
