/*******************************************************************************
NAME:    C_TOHEAD

PURPOSE: Convert the header structure to an ascii buffer.
		
PROGRAM HISTORY:

Version  Date   Author           Reason
-------  -----  ---------------  -----------------------------------------------
  2.0    02/91  D.Hollaren       Original development
  2.1    04/92  J.Fenno          Rewrote to use the CEOS_IEF header format.
                                 Placed the EROS Archive Header in the free text
				 portion of the CEOS_IEF header.
  2.2    09/92  J.Fenno          Added ending orbit to the archive header and
				 changed the total number of missed lines from a
				 4 digit number to a 5 digit number.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "util.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"
#include "archio.h"
#include "local_protos.h"

#define DBL_FLD 13


void c_tohead
(
    struct MODEL *model,	/* IN  - Model and orbital information	     */
    char buffer[][HDR_SIZ+1]	/* OUT - Buffer ASCII header information     */
)
{
char tmp_str[HDR_SIZ + 1];     /* Tmp string for building entries of header  */

long year,		       /* Year of the pass			     */
     i;                        /* Loop control                               */


/* Blank every line of the buffer needed for the header information
-------------------------------------------------------------------*/
for (i = 0; i < HDR_RECS_V2; i++)
   blank_line(buffer[i]);


/* Place the fields from nbs to eorbit in the first line of the buffer
----------------------------------------------------------------------*/
sprintf(tmp_str, "%05ld", model->clock_err);
strncpy(&buffer[0][3], tmp_str, 5);

sprintf(tmp_str, "%02ld", model->satnum);
strncpy(&buffer[0][9], tmp_str, 2);

if (model->dtype == LAC)
    strcpy(tmp_str,"LAC ");
else if (model->dtype == GAC)
    strcpy(tmp_str,"GAC ");
else
    strcpy(tmp_str,"HRPT");
strncpy(&buffer[0][12], tmp_str, 4);

sscanf(model->station, "%s", tmp_str);
for (i = (long)strlen(tmp_str); i < 3; i++)
   strcat(tmp_str, " ");
strncpy(&buffer[0][17], tmp_str, 3);

/**************************************************************
DLLOYD Y2K
   Removed the hard coded year conversion and replaced with
   a las function call.
***************************************************************/
year = c_4digityr( model->year );

sprintf(tmp_str, "%02ld/%02ld/%04ld", model->month % 100,
	model->day % 100, year % 10000);
strncpy(&buffer[0][21], tmp_str, 10);

sprintf(tmp_str, "%03ld", model->jday);
strncpy(&buffer[0][32], tmp_str, 3);

sprintf(tmp_str, "%02ld:%02ld:%02ld.%03ld", model->shour % 100,
	model->smin % 100, model->ssec % 100, model->sfrac % 1000);
for (i = 9; i < 12; i++)	/* replace any trailing spaces with a 0 */
   if (tmp_str[i] == ' ')
      tmp_str[i] = '0';
strncpy(&buffer[0][36], tmp_str, 12);

sprintf(tmp_str, "%02ld:%02ld:%02ld.%03ld", model->ehour % 100,
	model->emin % 100, model->esec % 100, model->efrac % 1000);
for (i = 9; i < 12; i++)	/* replace any trailing spaces with a 0 */
   if (tmp_str[i] == ' ')
      tmp_str[i] = '0';
strncpy(&buffer[0][49], tmp_str, 12);

sprintf(tmp_str, "%05ld", model->sorbit);
strncpy(&buffer[0][62], tmp_str, 5);

sprintf(tmp_str, "%05ld", model->eorbit);
strncpy(&buffer[0][68], tmp_str, 5);


/* Place the fields from spass to sunzen in the second line of the buffer
-------------------------------------------------------------------------*/
switch (model->spass)
   {
   case ASCENDING:
      strcpy(tmp_str, "ASC ");
      break;
   case DESCENDING:
      strcpy(tmp_str, "DESC");
      break;
   default:
      strcpy(tmp_str, "????");
   }
strncpy(&buffer[1][3], tmp_str, 4);

switch (model->cpass)
   {
   case ASCENDING:
      strcpy(tmp_str, "ASC ");
      break;
   case DESCENDING:
      strcpy(tmp_str, "DESC");
      break;
   default:
      strcpy(tmp_str, "????");
   }
strncpy(&buffer[1][8], tmp_str, 4);

switch (model->epass)
   {
   case ASCENDING:
      strcpy(tmp_str, "ASC ");
      break;
   case DESCENDING:
      strcpy(tmp_str, "DESC");
      break;
   default:
      strcpy(tmp_str, "????");
   }
strncpy(&buffer[1][13], tmp_str, 4);

sprintf(tmp_str, "%1ld", model->nchans);
strncpy(&buffer[1][18], tmp_str, 1);

for (i = 0; i < 5; i++)
   if (model->chans[i] > 0)
      tmp_str[i] = (char)('1' + i);
   else
      tmp_str[i] = '0';
tmp_str[i] = '\0';
strncpy(&buffer[1][20], tmp_str, 5);

sprintf(tmp_str, "%05ld", model->nlines);
strncpy(&buffer[1][26], tmp_str, 5);

sprintf(tmp_str, "%04ld", model->nsamples);
strncpy(&buffer[1][32], tmp_str, 4);

sprintf(tmp_str, "%05ld", model->totmiss);
strncpy(&buffer[1][37], tmp_str, 5);

switch (model->day_night)
   {
   case DAY:
      strcpy(tmp_str, "DAY  ");
      break;
   case NIGHT:
      strcpy(tmp_str, "NIGHT");
      break;
   default:
      strcpy(tmp_str, "?????");
   }
strncpy(&buffer[1][43], tmp_str, 5);

strncpy(&buffer[1][49], "SunZenith", 9);
format_dbl_h(model->sunzen * DEG_PER_RAD, 5, tmp_str);
strncpy(&buffer[1][59], tmp_str, DBL_FLD);


/* Place NW and NN in the third line of the buffer
----------------------------------------------------*/
strncpy(&buffer[2][3], "NWest ", 6);
format_dbl_h(model->nw[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[2][10], tmp_str, DBL_FLD);
format_dbl_h(model->nw[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[2][24], tmp_str, DBL_FLD);

strncpy(&buffer[2][40],  "NNadir", 6);
format_dbl_h(model->nn[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[2][47], tmp_str, DBL_FLD);
format_dbl_h(model->nn[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[2][61], tmp_str, DBL_FLD);


/* Place NE and CW in the fourth line of the buffer
----------------------------------------------------*/
strncpy(&buffer[3][3], "NEast ", 6);
format_dbl_h(model->ne[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[3][10], tmp_str, DBL_FLD);
format_dbl_h(model->ne[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[3][24], tmp_str, DBL_FLD);

strncpy(&buffer[3][40],  "CWest ", 6);
format_dbl_h(model->cw[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[3][47], tmp_str, DBL_FLD);
format_dbl_h(model->cw[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[3][61], tmp_str, DBL_FLD);

/* Place CN and CE in the fifth line of the buffer
---------------------------------------------------*/
strncpy(&buffer[4][3], "CNadir", 6);
format_dbl_h(model->cn[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[4][10], tmp_str, DBL_FLD);
format_dbl_h(model->cn[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[4][24], tmp_str, DBL_FLD);

strncpy(&buffer[4][40],  "CEast ", 6);
format_dbl_h(model->ce[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[4][47], tmp_str, DBL_FLD);
format_dbl_h(model->ce[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[4][61], tmp_str, DBL_FLD);

/* Place SW and SN in the sixth line of the buffer
--------------------------------------------------*/
strncpy(&buffer[5][3], "SWest ", 6);
format_dbl_h(model->sw[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[5][10], tmp_str, DBL_FLD);
format_dbl_h(model->sw[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[5][24], tmp_str, DBL_FLD);

strncpy(&buffer[5][40],  "SNadir", 6);
format_dbl_h(model->sn[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[5][47], tmp_str, DBL_FLD);
format_dbl_h(model->sn[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[5][61], tmp_str, DBL_FLD);

/* Place SE and EQCRS in the seventh line of the buffer
-------------------------------------------------------*/
strncpy(&buffer[6][3], "SEast ", 6);
format_dbl_h(model->se[0] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[6][10], tmp_str, DBL_FLD);
format_dbl_h(model->se[1] * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[6][24], tmp_str, DBL_FLD);

strncpy(&buffer[6][40],  "EqCrs ", 6);
format_dbl_h(model->eqcrs * DEG_PER_RAD, 7, tmp_str);
strncpy(&buffer[6][47], tmp_str, DBL_FLD);

strncpy(&buffer[6][63], "SatVw ", 6);
sprintf(tmp_str, "%1ld", model->sat_view);
strncpy(&buffer[6][69], tmp_str, 1);

return;
}



/************************************************
*  Create a blank line for the CEOS_IEF header  *
************************************************/
void blank_line
(
    char str[HDR_SIZ + 1]  /* One line from the CEOS_IEF header */
)
{
long i;

strcpy(str, "/*");
for (i = 2; i < HDR_SIZ - 2; i++)
   str[i] = ' ';
str[i] = '\0';
strcat(str, "*/");
}



/****************************************************************************
*  Format a double value into a 12 character string such that there is a    *
*  leading sign and 'width' digits after the decimal point.  Note that      *
*  widths other than 5 or 7 will result in the error string "????????????". *
****************************************************************************/
void format_dbl_h
(
    double number,  /* the number to be formatted                     */
    long width,	    /* the number of digits after the decimal point   */
    char str[]      /* a string representing number formatted as 0.00 */
)
{

/* Format the number with 'width' digits after the decimal point
----------------------------------------------------------------*/
switch (width)
   {
   case 5:
      sprintf(str, "%+013.5lf", number);
      break;
   case 7:
      sprintf(str, "%+013.7lf", number);
      break;
   case 10:
      sprintf(str, "%+013.10lf", number);
      break;
   default:
      strcpy(str, "?????????????");
      return;
   }

/* If the formatted string is not the correct length return an error string
---------------------------------------------------------------------------*/
if (strlen(str) != 13)
   strcpy(str, "?????????????");
}
 
