/*******************************************************************************
FUNCTION:	packline

PURPOSE:	Pack the current line back into the buffer to store any user
		updates.

		This function will return E_SUCC upon successful completion or
		E_FAIL if an error is encountered.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  2.0    03/02  D.Lloyd          Modified for endian support.  CR 5541

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "adapsutil.h"
#include "archio.h"
#include "lasendian.h"


int packline
(
    struct ARCDESC *arc/* IN - Archive descriptor record                     */
)
{
unsigned char *pck_ptr; /* Ptr into the packed I/O buffer                     */


/* Swap the pixels for ascending passes being processed in a non-satellite
   view and set a pointer to the correct position in the packed buffer.
--------------------------------------------------------------------------*/
if (arc->ascending && !arc->satview)
    {
    /* Insure the buffer is in the proper format for the system using
    the swap buffer so that the samples buffer is not corrupted.
    -----------------------------------------------------------------*/
    memcpy( arc->swpbuf, arc->samples, arc->words_img * sizeof(short));
    exportendian_buf( arc->swpbuf, sizeof(short), arc->words_img );

    c_pixflip(arc->swpbuf, arc->words_img);
    pck_ptr = arc->packed + (arc->last - arc->current) * arc->arcbytes;
    }
else
    pck_ptr = arc->packed + (arc->current - arc->first) * arc->arcbytes;


/* Pack the minor frame and image data back into the I/O buffer
---------------------------------------------------------------*/
c_pck32(arc->swpbuf, &pck_ptr[ARCH_MNR], &arc->words_img);
/* Insure the buffer is in the proper format for the system using
the swap buffer so that the minor buffer is not corrupted.
-----------------------------------------------------------------*/
memcpy( arc->swpbuf, arc->mnr, arc->words_mnr * sizeof(short));
exportendian_buf( arc->swpbuf, sizeof(short), arc->words_mnr );
c_pck32(arc->swpbuf, &pck_ptr[0],        &arc->words_mnr);


/* Set the saveline flag indicating the current line has been saved
-------------------------------------------------------------------*/
arc->saveline = FALSE;

return(E_SUCC);
}
 
