/*******************************************************************************
FUNCTION: c_prtgrid

PURPOSE:  Print a grid to a file.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/92  Development

PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "grid.h"


int c_prtgrid
(
    char *hostname,     /* IN - Host name of the file to contain the grid     */
    struct GRID *grid   /* IN - Grid structure to write to a file             */
)
{
long i;                 /* Loop control                                       */
long j;                 /* Loop control                                       */
long nonfatal = 1;      /* Nonfatal error flag                                */

FILE     *fptr;         /* File ptr for opening the output file               */
unsigned char *p1;      /* Ptr used to walk through grid->values[]            */
short    *p2;           /* Ptr used to walk through grid->values[]            */
long     *p3;           /* Ptr used to walk through grid->values[]            */
float    *p4;           /* Ptr used to walk through grid->values[]            */
double   *p5;           /* Ptr used to walk through grid->values[]            */

fptr = fopen(hostname, "w");
if (fptr == NULL)
   {
   c_errmsg("Error opening file", "c_prtgrid-open", &nonfatal);
   return(E_FAIL);
   }

fprintf(fptr, "grid->gridname   = %s\n", grid->gridname);
for (i = 0; i < grid->nvalues; i++)
   fprintf(fptr, "grid->labels[%02d] = %s\n", i, grid->labels[i]);
fprintf(fptr, "grid->model      = %s\n", grid->model);
fprintf(fptr, "grid->units      = %s\n", grid->units);

for (j = 0; j < 32; j++)
   fprintf(fptr, "grid->act_grid_err[%02d] = %lf\n", j, grid->act_grid_err[j]);
for (j = 0; j < 2; j++)
   fprintf(fptr, "grid->ave_fit_err[%02d]  = %lf\n", j, grid->ave_fit_err[j]);
for (j = 0; j < 2; j++)
   fprintf(fptr, "grid->ave_grid_err[%02d] = %lf\n", j, grid->ave_grid_err[j]);
for (j = 0; j < (2 * grid->ncoeff); j++)
   fprintf(fptr, "grid->coeffs[%02d]       = %lf\n", j, grid->coeffs[j]);
for (j = 0; j < 8; j++)
   fprintf(fptr, "grid->corners[%02d]      = %lf\n", j, grid->corners[j]);
fprintf(fptr, "grid->grid_tol         = %lf\n", grid->grid_tol);
for (j = 0; j < 2; j++)
   fprintf(fptr, "grid->max_fit_err[%02d]  = %lf\n", j, grid->max_fit_err[j]);
for (j = 0; j < 2; j++)
   fprintf(fptr, "grid->max_grid_err[%02d] = %lf\n", j, grid->max_grid_err[j]);
for (j = 0; j < 2; j++)
   fprintf(fptr, "grid->pdist[%02d]        = %lf\n", j, grid->pdist[j]);
for (j = 0; j < 15; j++)
   fprintf(fptr, "grid->projprms[%02d]     = %lf\n", j, grid->projprms[j]);
fprintf(fptr, "grid->rms_fit_err      = %lf\n", grid->rms_fit_err);
fprintf(fptr, "grid->rms_grid_err     = %lf\n", grid->rms_grid_err);


fprintf(fptr, "grid->code             = %d\n", grid->code);
fprintf(fptr, "grid->datum            = %d\n", grid->datum);
fprintf(fptr, "grid->dtype            = %d\n", grid->dtype);
fprintf(fptr, "grid->fiterr_valid     = %d\n", grid->fiterr_valid);
fprintf(fptr, "grid->griderr_valid    = %d\n", grid->griderr_valid);
fprintf(fptr, "grid->lines            = %d\n", grid->lines);
fprintf(fptr, "grid->ncoeff           = %d\n", grid->ncoeff);
fprintf(fptr, "grid->ncols            = %d\n", grid->ncols);
fprintf(fptr, "grid->nrows            = %d\n", grid->nrows);
fprintf(fptr, "grid->nvalues          = %d\n", grid->nvalues);
for (j = 0; j < grid->nrows; j++)
   fprintf(fptr, "grid->out_lines[%03d]   = %d\n", j, grid->out_lines[j]);
for (j = 0; j < grid->ncols; j++)
   fprintf(fptr, "grid->out_samps[%03d]   = %d\n", j, grid->out_samps[j]);
fprintf(fptr, "grid->proj_valid       = %d\n", grid->proj_valid);
fprintf(fptr, "grid->samples          = %d\n", grid->samples);
fprintf(fptr, "grid->satnum           = %d\n", grid->satnum);
fprintf(fptr, "grid->zone             = %d\n", grid->zone);

for (i = 0; i < grid->nvalues; i++)
   {
   switch (grid->dtype)
      {
      case EBYTE:
	 p1 = (unsigned char *)grid->values[i];
	 for (j = 0; j < grid->nrows * grid->ncols; j++)
	    fprintf(fptr, "grid->values[%02d][%05d]     = %d\n",i,j,*(p1++));
	 break;
      case EWORD:
	 p2 = (short *)grid->values[i];
	 for (j = 0; j < grid->nrows * grid->ncols; j++)
	    fprintf(fptr, "grid->values[%02d][%05d] = %hd\n",i,j,*(p2++));
	 break;
      case ELONG:
	 p3 = (long *)grid->values[i];
	 for (j = 0; j < grid->nrows * grid->ncols; j++)
	    fprintf(fptr, "grid->values[%02d][%05d] = %d\n",i,j,*(p3++));
	 break;
      case EREAL:
	 p4 = (float *)grid->values[i];
	 for (j = 0; j < grid->nrows * grid->ncols; j++)
	    fprintf(fptr, "grid->values[%02d][%05d] = %f\n",i,j,*(p4++));
	 break;
      case EDOUBLE:
	 p5 = (double *)grid->values[i];
	 for (j = 0; j < grid->nrows * grid->ncols; j++)
	    fprintf(fptr, "grid->values[%02d][%05d] = %lf\n",i,j,*(p5++));
	 break;
      }
   }
fclose(fptr);
return(E_SUCC);
}
 
