/*****************************************************************************
FUNCTION:	C_L1BCLOSD

PURPOSE:	Close a Level 1b disk file and flush a GAC buffer, if 
		necessary.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    09/97	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include "las.h"
#include "level1bio.h"


int c_l1bclosd
(
FILE		*fd 		/* IN: file descriptor of file to close	*/
)

{
long	accmode;		/* file access mode			*/
long	filedesc;		/* unix file descriptor			*/
long	i;			/* loop control variable		*/
long	recsiz = 0;		/* zero record size			*/
long	status = E_SUCC;	/* function return status		*/
long	val;			/* file descriptor flags value		*/

unsigned char buf[MAXLINSIZ];	/* dummy line buffer			*/


/*
  Determine if the file was opened for reading or writing
*/
filedesc = fileno( fd );
val = fcntl( filedesc, F_GETFL, 0 );
accmode = val & O_ACCMODE;

/*
  If the access mode was for write, call write line with a zero-buffer
  and recsiz = 0 to insure the a GAC data buffer is flushed.  Note:
  c_l1bwritet will be called regardless of data type (since this function 
  is unaware of the data type and we want to keep it that way).  Only a 
  GAC buffer will be flushed -- for LAC the function returns without doing 
  anything.
*/
if ( accmode == O_WRONLY )
    {
    for ( i = 0; i < MAXLINSIZ; i++ )
        buf[i] = 0;
    status = c_l1bwrited( fd, buf, &recsiz );
    if ( status != E_SUCC )
	c_lasmsg(FALSE,"l1bclosd-flsh","Error flushing GAC line buffer");
    }  /* END if */

status = fclose( fd );
if ( status )
    c_lasmsg(FALSE,"l1bclosd-clos","Error closing disk file");

return status;

}  /* END c_l1bclosd */
