/*****************************************************************************
FUNCTION:	C_L1BCLOST

PURPOSE:	Close a Level 1b tape and flush a GAC buffer, if necessary.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "las.h"
#include "tapeio.h"
#include "level1bio.h"


int c_l1bclost
(
long		*tun,		/* IN: tape logical unit number		*/
long		*tape 		/* OUT: tape open flag			*/
)

{
long	eofopt = 1;		/* number of EOF marks to write		*/
long	i;			/* loop control variable		*/
long	mode = NO_HANDLE;	/* this routine will handle errors	*/
long	recsiz = 0;		/* zero record size			*/
long	rewopt = NOREW;		/* rewind option flag			*/
long	status = E_SUCC;	/* function return status		*/

struct TAPEDESC *drive;		/* pointer to a tape descriptor		*/

unsigned char buf[MAXLINSIZ];	/* dummy line buffer			*/


/*
  Point drive at the tape descriptor structure
*/
drive = (struct TAPEDESC *) *tun;

/*
  Decide if the tape was opened for write.  If so make sure the GAC line
  buffer is flushed.  Note: c_l1bwritet will be called regardless of
  data type (since this function is unaware of the data type and we want
  to keep it that way).  Only a GAC buffer will be flushed -- for LAC
  the function returns without doing anything.
*/
if ( drive->inout == TWRITE || drive->inout == TUPDATE )
    {
    for ( i = 0; i < MAXLINSIZ; i++ )
        buf[i] = 0;
    status = c_l1bwritet( tun, buf, &recsiz, tape );
    if ( status != E_SUCC )
        c_lasmsg(FALSE,"l1bclost-flsh","Error flushing GAC line buffer");
    }  /* END if */

/*
  Close the tape and set the tape open flag to FALSE
*/
status = c_tpclos( tun, &mode, &rewopt, &eofopt );
if ( status != TAPE_SUCC )
    c_lasmsg(FALSE,"l1bclost-clos","Error closing tape");
else
    *tape = FALSE;

return status;

}  /* END c_l1bclost */
