/*****************************************************************************
FUNCTION:	C_L1BMKHDR

PURPOSE:	Make TBM or ARS header and DSH records from model and
		dropped line structures.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development
  1.1    12/98  D.Lloyd		Added NOAA KLM make ARS header call

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "avhrr.h"
#include "archio.h"
#include "level1bio.h"
#include "local_protos.h"


int c_l1bmkhdr
(
    struct MODEL *model,	/* IN: AVHRR model structure		*/
    struct DROPPED *drop,	/* IN: dropped line structure		*/
    long window[],		/* IN: window for model SL,SS,EL,and ES	*/
    long chans[],		/* IN: selected channels		*/
    long *recsiz,		/* IN: record size in bytes		*/
    unsigned char *buf,		/* OUT: pointer to TBM/ARS buffer	*/
    unsigned char *dsh		/* OUT: pointer to DSH buffer		*/
)

{
struct MODEL	window_model;	/* windowed model structure		*/
struct DROPPED	window_drop;	/* windowed drop line structure		*/

long	bound_flg = FALSE;	/* drop line bound flag			*/
long	modwind[4];		/* modified window for call to windmod	*/
long	status;			/* function return status		*/


/*
  Make sure the window matches the model as far as satellite view
  is concerned and copy window to modified window.
*/
if ( model->cpass == ASCENDING && model->sat_view == TRUE )
    modwind[0] = model->nlines - window[2] - window[0] + 2;
else
    modwind[0] = window[0];
modwind[1] = window[1];
modwind[2] = window[2];
modwind[3] = window[3];

/*
  Generate a windowed model and dropped line structures
*/
status = c_windmod( model, drop, &window_model, &window_drop, modwind, chans,
                    bound_flg );
if ( status != E_SUCC )
    return status;


/*
  Build a DSH record from the new model information
*/
if ( model->satnum < 15 )
    status = l1b_make_dsh( dsh, &window_model, &window_drop, *recsiz );
else
    status = l1b_make_klmdsh( dsh, &window_model, &window_drop, *recsiz );
if ( status != E_SUCC )
    return status;


/*
  Build a TBM header record or, in case of NOAA KLM, an ARS header
  from the new model and channel information
*/
if ( model->satnum >= 15 )
    status = l1b_make_ars( buf, dsh, &window_model, recsiz );
else
    status = l1b_make_tbm( buf, dsh, &window_model );
if ( status != E_SUCC )
    return status;


return E_SUCC;

}  /* END c_l1bmkhdr */
