/*******************************************************************************
FUNCTION:	C_L1BOPNTR

PURPOSE:	Open and mount the input image tape containing the Level-1B
		image.  Read the TBM header record and the Data Set Header
		record.  Load the AVHRR model from the header records.
		Check the first time stamp in the image.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T.Baltzer        Development
  1.1    12/98  D.Lloyd          Modifications for KLM


PROJECT:	ADAPS
*******************************************************************************/
#include "las.h"
#include "adaps.h"
#include "tapeio.h"
#include "level1bio.h"
#include "local_protos.h"



int c_l1bopntr
(
    char drive[CMLEN],	 /* INPUT : Tape drive name			*/
    long *tun,		 /* OUTPUT: Tape unit number			*/
    unsigned char buf[], /* OUTPUT: TBM/ARS header buffer		*/
    unsigned char dsh[], /* OUTPUT: DSH header buffer			*/
    long *recsiz,	 /* OUTPUT: Record size of data read           	*/
    struct MODEL *model, /* OUTPUT: AVHRR model information		*/
    long *tape		 /* OUTPUT: Flag for tape open			*/
)

{

long	goodlines,		/* Number of good lines in the image	*/
	status,			/* Return status variable		*/
	hdrflg = FALSE;		/* tbm/ars header flag (TRUE or FALSE)	*/

unsigned char linbuf[MAXRECSIZ];        /* First line of data in file   */


/*
  Read the header information from the tape
*/
status = l1b_tape_in(drive,tun,buf,dsh,linbuf,recsiz,
		     &hdrflg,tape);
if (status != E_SUCC)
    return(status);


/*
  Initialize the model structure using the TBM/ARS and DSH headers
*/
status = l1b_load_mod(buf,dsh,*recsiz,hdrflg,&goodlines,model);
if (status != E_SUCC)
    return(status);


/*
  Check the first time stamp in the image
*/
l1b_ck_tstamp(linbuf,model);


return(E_SUCC);
}
