/*******************************************************************************

FUNCTION:	C_L1BRECSIZ

PURPOSE: 	Calculate the record size for a Level-1B image.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T.Baltzer	 Initial development
  2.0    12/98  D.Lloyd          Added a flag to the parameter list for
                                 KLM.  Added code to set KLM record sizes.

PROJECT:	ADAPS
*******************************************************************************/
#include "las.h"
#include "avhrr.h"
#include "level1bio.h"



void c_l1brecsiz
(
    long dtype,			/* Data type GAC/LAC/HRPT	(input)	*/
    long pack_flg,		/* Pack flag TRUE/FALSE		(input)	*/
    long klm_flg,		/* NOAA KLM series flag (T/F)   (input)	*/
    long nbands,		/* Number of channels 1-5	(input)	*/
    long *recsiz		/* Level-1B record size (bytes)	(output)*/
)

{

switch (dtype)
    {
    case GAC:
        if ( !klm_flg )
	    {
	    switch(nbands)
		{
		case 1:
		    *recsiz = UNP_GAC_1;
		    break;
		case 2:
		    *recsiz = UNP_GAC_2;
		    break;
		case 3:
		    *recsiz = UNP_GAC_3;
		    break;
		case 4:
		    *recsiz = UNP_GAC_4;
		    break;
		case 5:
		    /* if the data is packed */
		    if (pack_flg == TRUE)
			*recsiz = PCK_GAC_5;
		    else
			*recsiz = UNP_GAC_5;
		    break;
		}
	    }
	else
	    {
	    switch ( nbands )
	        {
		case 1:
		    *recsiz = KLM_UNP_GAC_1;
		    break;
		case 2:
		    *recsiz = KLM_UNP_GAC_2;
		    break;
		case 3:
		    *recsiz = KLM_UNP_GAC_3;
		    break;
		case 4:
		    *recsiz = KLM_UNP_GAC_4;
		    break;
		case 5:
		    /* if the data is packed */
		    if (pack_flg == TRUE)
		        *recsiz = KLM_PCK_GAC_5;
		    else
		        *recsiz = KLM_UNP_GAC_5;
		    break;
		}
	    }
	    break;
    case LAC:
    case HRPT:
        if ( !klm_flg )
	    {
	    switch(nbands)
		{
		case 1:
		    *recsiz = UNP_LAC_1;
		    break;
		case 2:
		    *recsiz = UNP_LAC_2;
		    break;
		case 3:
		    *recsiz = UNP_LAC_3;
		    break;
		case 4:
		    *recsiz = UNP_LAC_4;
		    break;
		case 5:
		    /* if the data is packed */
		    if (pack_flg == TRUE)
			*recsiz = PCK_LAC_5;
		    else
			*recsiz = UNP_LAC_5;
		    break;
		}
	    }
	else
	    {
	    switch ( nbands )
	        {
		case 1:
		    *recsiz = KLM_UNP_LAC_1;
		    break;
		case 2:
		    *recsiz = KLM_UNP_LAC_2;
		    break;
		case 3:
		    *recsiz = KLM_UNP_LAC_3;
		    break;
		case 4:
		    *recsiz = KLM_UNP_LAC_4;
		    break;
		case 5:
		    /* if the data is packed */
		    if (pack_flg == TRUE)
		        *recsiz = KLM_PCK_LAC_5;
		    else
		        *recsiz = KLM_UNP_LAC_5;
		    break;
		}
	    }
	break;
     }

return;
}
