/*****************************************************************************
FUNCTION:	C_L1BSEEKT

PURPOSE:	Seek to the specified line number in the Level 1b tape file.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development
  1.1    12/98  D.Lloyd         Modifications for NOAA KLM

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "avhrr.h"
#include "tapeio.h"
#include "level1bio.h"
#include "local_protos.h"


#define  TOOLONG  100


int c_l1bseekt
(
    long *tun,			/* IN: tape unit number		*/
    long window[],		/* IN: line number to seek	*/
    long *recsiz,		/* IN: record size in bytes	*/
    struct MODEL *model,	/* IN: AVHRR model structure	*/
    char comment[][CMLEN],	/* IN: tape description(s)	*/
    long *volnum,		/* IN/OUT: tape volume number	*/
    long *tape,			/* IN/OUT: tape open flag	*/
    long *drop_line		/* OUT: actual scan line number	*/
)

{
long		file_line;		/* file relative line number	*/
long		first_line;		/* first scan line number	*/
long		mode = HANDLE;		/* handle tape errors		*/
long		one = 1;		/* value of one			*/
long		scan_line;		/* actual scan line found	*/
long		status;			/* function return status	*/

short		*sptr1;			/* pointer to the first line	*/
short		*sptr2;			/* pointer to second line	*/

unsigned char	buf[MAXLINSIZ];		/* record buffer		*/


/*
  Point the short pointers at the line number locations in the buffer
*/
sptr1 = (short *)buf;
if ( model->dtype == GAC && model->satnum <= 14 )
    sptr2 = (short *)( buf + (*recsiz / 2 ));
else
    sptr2 = (short *)buf;

/*
  Get the first line and find out its actual value.
*/
/* ??? possibly replace this with a get_rec_tape() call so it's already
byteswapped */
status = c_tpread( tun, buf, recsiz, &mode );
if ( status != *recsiz )
    return E_FAIL;
first_line = (short)*sptr1;

/*
  Determine file-relative line number.  If pass is ascending then
  the line number must be calculated relative to the ending line.
*/
if ( model->cpass == ASCENDING && model->sat_view == TRUE )
    file_line = model->nlines - window[0] - window[2] + first_line + 1;
else
    file_line = first_line + window[0] - 1; 

/*
  If we are skipping a significant number of lines, inform
  the user what is going on.
*/
if ( file_line - first_line > TOOLONG )
    c_lasmsg(FALSE," ","Skipping to proper position on tape");

/*
  Look for the desired line number.  If it's a dropped line, it won't
  be found, stop at the next line following.
*/
while ( !( file_line == *sptr1 ) && !( file_line == *sptr2 ))
    {
    /* see if the desired line was a dropped line since the search */
    /* is sequential a number greater than the desired one means   */
    /* it's been passed and therefore dropped                      */
    if ((*sptr1 > file_line) || ((model->dtype == GAC) &&
        (*sptr2 > file_line)))
	break;

    /* skip the next record for LAC -- second half of the line */
    if ( model->dtype != GAC && model->satnum <= 14 )
	{
	status = c_tpfsr( tun, &one, &mode );
	if ( status != TAPE_SUCC )
	    return E_FAIL;
	}  /* END if */

    /* read the next record */
    status = get_rec_tape( tun, buf, recsiz, volnum, comment, tape );
    if ( status != E_SUCC )
	return E_FAIL;
    }  /* END while */

/*
  At this point the tape is 1 record past where it should be regardless
  of how we got here.  So skip 1 record back.
*/
status = c_tpbsr( tun, &one, &mode );
if ( status != TAPE_SUCC )
    return E_FAIL;

/*
  Determine the actual scan line -- since the desired line may have
  been dropped or in second half for GAC.
*/
if ( model->dtype != GAC && model->satnum <= 14 )
    scan_line = *sptr1;
else
    {
    if ( file_line == *sptr1 )
	scan_line = *sptr1;
    else if ( *sptr1 > file_line )
	scan_line = *sptr1;
    else
	scan_line = *sptr2;
    }  /* END else */

/*
  Since there are two lines per record, insure that the first call to
  c_l1breadt by the user returns the proper line.  If the current line
  resides in the second half of the record call c_l1breadt to prime the
  static GAC buffer maintained in c_l1breadt.
*/
if (( model->dtype == GAC && model->satnum <= 14 ) && ( scan_line == *sptr2 ))
    {
    status = c_l1breadt( tun, recsiz, comment, volnum, tape, buf );
    if ( status != E_SUCC )
	return E_FAIL;
    }  /* END if */

/*
  Calculate and return drop lines encountered
*/
*drop_line = scan_line - file_line;
return E_SUCC;

}  /* END c_l1bseekt */
