/*****************************************************************************
FUNCTION:	C_L1BSKIPT

PURPOSE:	Skip a specified number of files on tape.  Must skip in a
		forward direction only.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	T.Baltzer	Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "tapeio.h"


int c_l1bskipt
(
    char drive[CMLEN],	/* INPUT: Tape drive name		  */
    long *skip		/* INPUT: Number of files to skip ( >= 0) */
)

{

long eofopt = 0;	/* Number of EOF marks to write		*/
long inout = TREAD;	/* Read only mode			*/
long mode = HANDLE;	/* Handle tape errors			*/
long rewopt = NOREW;	/* Rewind option flag			*/
long status;		/* Return status			*/
long tun;		/* Tape logical unit number		*/

char density[5];	/* Default tape density			*/


/*
  Error check for a negative skip count
*/
if (*skip < 0)
    {
    c_lasmsg(FALSE,"c_l1bskipt-neg","Invalid skip count: %d",*skip);
    return(E_FAIL);
    }


/*
  Reopen the tape skipping the specified number of files.  Skip may be zero
  if no files are to be skipped.
*/
strcpy(density,"0");
status = c_tpreop( &tun, skip, &mode, &rewopt, density, &inout, drive );
if ( status != TAPE_SUCC )
    return E_FAIL;


/*
  Close the tape
*/
status = c_tpclos( &tun, &mode, &rewopt, &eofopt );
if ( status != TAPE_SUCC )
    c_lasmsg(FALSE,"l1bskipt-clos","Error closing tape");


return(E_SUCC);

}  /* END c_l1bskipt */
