/******************************************************************************
FUNCTION:	C_L1BSUMM

PURPOSE:	Create a level-1b product summary

PROGRAM HISTORY:
Version	Date	Author		Change Request
-------	-----	-------------	--------------
 1.0				Original development
 5.0	12/91	T. Rockvam	Conversion to LAS 5.0 and C language;
				incorporated into mklevel1b
 6.0	02/98	D.Lloyd		Removed tape label generation call and
				modified the lat/long calculations to
				utilize the model.
 7.0	02/98	T. Baltzer	Made into a Level 1b support routine
 7.1	07/98	D.Lloyd		Modified for Y2K compliance

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES


*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapssup.h"
#include "archio.h"
#include "level1bio.h"
#include "local_protos.h"

#ifndef MIN
#define MIN(a,b)  ((a) < (b) ? (a) : (b))
#endif
#ifndef MIN4
#define MIN4(a,b,c,d) (MIN(a,b) < MIN(c,d) ? MIN(a,b) : MIN(c,d))
#endif
#ifndef MAX4
#define MAX4(a,b,c,d) (LASMAX(a,b) > LASMAX(c,d) ? LASMAX(a,b) : LASMAX(c,d))
#endif



int c_l1bsumm
(
char	rootname[CMLEN],/* Root name of the summary file	*/
char	acctnum[10],	/* account number			*/
char	ordernum[14],	/* order number				*/
char	unitnum[5],	/* unit number				*/
char	prodesc[18],	/* product description			*/
char	density[5],	/* tape density				*/
struct	MODEL	*model,	/* Model and orbital information	*/
char	lasout[CMLEN] 	/* Full LAS output name for summary	*/
)

{
char	sceneid[30];	/* input scene id			*/
char    hosout[CMLEN];	/* host summary file name		*/

float	rgwin[4];	/* original user requested area		*/
float	beglat,		/* beginning lat 			*/
	beglong,	/* beginning long 			*/
	endlat,		/* ending lat 				*/
	endlong;	/* ending long 				*/

long	gday,		/* gregorian day			*/
	month,		/* month				*/
	nl,ns,		/* # lines, samples in output file    	*/
	now,		/* time now				*/
	status;		/* return status			*/

char	acqdat[9],	/* formatted acquisition date		*/
	acqtim[9],	/* formateed acquisition time		*/
	curdate[9],	/* current date and time		*/
	satid[8],	/* satellite id				*/
        source[5],      /* Data source                          */
	*strptr;	/* String pointer			*/

struct	tm *curtime;	/* current time				*/


/*
  Create the output file name as:
  "[ADAPSARC.forms.scenes]rootname;prs"
*/
strptr = getenv( "ADAPSARC" );
strcpy( hosout, strptr );
strcat( hosout, "/forms/scenes/" );
strcat( hosout, rootname);
strcat( hosout, ".prs" );
strcpy(lasout, hosout);


/*
  Set up satellite ID name
*/
strcpy(satid,"NOAA-");
sprintf(&satid[5],"%02ld",model->satnum);

/*
  Determine data source from image title
*/
strcpy(source,"HRPT");

/*
  Determine current date and time
*/
now = time(NULL);
curtime = localtime(&now);
/*************************************************************
DLLOYD Y2K
  in year 2000 curtime->tm_year = 100 so must MOD it with 100
  to get appropriate 2-digit year
**************************************************************/
sprintf(curdate,"%02d/%02d/%02d",
    (curtime->tm_mon + 1),curtime->tm_mday,curtime->tm_year%100);

/*
  Determine the gregorian acquisition date ???? use model->month etc.
*/
c_julgre(&(model->year),&(model->jday),&month,&gday);

/*
  Generate a formatted acquisition date
*/
sprintf(acqdat,"%02ld/%02ld/%02ld",month,gday,model->year);

/*
  Generate a formatted acquisition time
*/
sprintf(acqtim,"%02ld:%02ld:%02ld",model->shour,model->smin,model->ssec);

/*
  Save the output area that was selected
*/

beglat  = (float)DEG(MAX4(model->nw[0],model->ne[0],model->sw[0],model->se[0]));
endlat  = (float)DEG(MIN4(model->nw[0],model->ne[0],model->sw[0],model->se[0]));
beglong = (float)DEG(MAX4(model->nw[1],model->ne[1],model->sw[1],model->se[1]));
endlong = (float)DEG(MIN4(model->nw[1],model->ne[1],model->sw[1],model->se[1]));
rgwin[0] = (float)DEG(model->nw[0]);
rgwin[1] = (float)DEG(model->nw[1]);
rgwin[2] = (float)DEG(model->se[0]);
rgwin[3] = (float)DEG(model->se[1]);

/*
  Set number of lines and samples in output image
*/
nl = model->nlines;
ns = model->nsamples;


/*
  Create the product summary file
*/
c_getscene( model, sceneid );
status = l1b_prodsum(hosout,acctnum,ordernum,unitnum,curdate,sceneid,
		     model->sorbit,acqdat,acqtim,prodesc,satid,source,
		     rgwin[0],rgwin[1],rgwin[2],rgwin[3], beglat,endlat,
		     beglong,endlong,nl,ns,density);
if (status != E_SUCC)
    return E_FAIL;


return E_SUCC;
}
