/*****************************************************************************
FUNCTION:	C_L1BUPHDR

PURPOSE:	Creates updated TBM/ARS header and DSH records from model and 
		dropped line structures and writes them to the disk file.
		The file is assumed to be open for write prior to the call.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development
  1.1    12/98  D.Lloyd         Modifications for NOAA KLM
  2.0  03/2002  D.Lloyd         Byte-swap data on little endian systems before
                                writing.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE


PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "avhrr.h"
#include "model.h"
#include "archio.h"
#include "level1bio.h"
#include "lasendian.h"
#include "adapsendian.h"


int c_l1buphdr
(
FILE		*out_fd,	/* IN: output file descriptor		*/
struct MODEL	*model,		/* IN: model structure			*/
struct DROPPED	*drop,		/* IN: dropped line structure		*/
long		window[],	/* IN: window for model			*/
long		chans[],	/* IN: channels for the output		*/
long		*recsiz 	/* IN: record size for output in bytes	*/
)

{
long	file_pos;		/* file position on function entry	*/
long	nbytes;			/* number of bytes written to file	*/
long	status;			/* function return status		*/

unsigned char	dsh[MAXRECSIZ];	/* DSH buffer				*/
unsigned char	buf[ARS_HDR_SIZ];	/* TBM/ARS header buffer	*/


/*
  Make TBM/ARS and DSH from model and drop
*/
status = c_l1bmkhdr( model, drop, window, chans, recsiz, buf, dsh );
if ( status != E_SUCC )
    return E_FAIL;


/*
  Save current file position and go to beginning
*/
file_pos = ftell( out_fd );
fseek( out_fd, 0L, SEEK_SET );

/* Insure the DSH is in the proper endian format prior to the write
-------------------------------------------------------------------*/
if ( model->satnum <= 14 )
    {
    export_endian_mixed_buffer( dsh, NOAADATASETHEADER );
    }
else
    {
    export_endian_mixed_buffer( dsh, KLMDATASETHEADER );
    }

/*
  Write TBM/ARS, DSH, and dummy (if necessary)
*/
if ( model->satnum <= 14 )
    {
    nbytes = (long)fwrite( buf, 1, TBM_HDR_SIZ, out_fd );
    if ( nbytes != TBM_HDR_SIZ )
	return E_FAIL;
    }
else
    {
    nbytes = (long)fwrite( buf, 1, ARS_HDR_SIZ, out_fd );
    if ( nbytes != ARS_HDR_SIZ )
        return E_FAIL;
    }
nbytes = (long)fwrite( dsh, 1, *recsiz, out_fd );
if ( nbytes != *recsiz )
    return E_FAIL;
if ( model->dtype != GAC && model->satnum < 15 )
    {
    nbytes = (long)fwrite( dsh, 1, *recsiz, out_fd );
    if ( nbytes != *recsiz )
	return E_FAIL;
    }  /* END if */


/*
  Reposition file pointer to previous position
*/
fseek( out_fd, file_pos, SEEK_SET );
    

return E_SUCC;

}  /* END c_l1buphdr */	
