/*******************************************************************************
FUNCTION:       COPY_CHAN

PURPOSE:	Copy the requested channels to the output image.
		NOTE: The data must be unpacked before calling this routine.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    05/96  A.Vander Vorste  Development
  1.1    11/97  T.Baltzer	 Modified to make part of support routine.

PROJECT:        ADAPS
*******************************************************************************/
#include "las.h"
#include "adaps.h"



void copy_chan
(
    short *ivid_data, 	/* Pointer to one scan line of input		*/
    short *ovid_data, 	/* Pointer to one scan line of output		*/
    long ichan[NCHANS], /* Input channels present			*/
    long ochan[NCHANS], /* Output channels to be processed		*/
    long nsamples	/* Number of samples per line			*/
)

{

long    i,			/* Loop counter                              */
    	ich,  	                /* Input channel loop counter		     */
	och,			/* Output channel loop counter		     */
	nichan,			/* Number of input channels		     */
	nochan,			/* Number of output channels 		     */
	samp,			/* Sample loop counter			     */
	iseq_chan[NCHANS],	/* holds the sequence of channels of input   */
	oseq_chan[NCHANS];	/* holds the sequence of channels of output  */



/* Making a list of the input/output channels in sequencial order.
   For example: if ichans = ( 0 1 0 1 1 ),
	   then iseq_chan = ( 2 4 5 x x )
	   and  nichan = 3
   ----------------------------------------------------------------*/
for (nichan = nochan = i = 0; i < NCHANS; i++)
    {
    if (ichan[i])
	iseq_chan[nichan++] = i+1;

    if (ochan[i])
	oseq_chan[nochan++] = i+1;
    }


/* Loop for each sample in the line */
for (samp = 0; samp < nsamples; samp++)
    {
    /* Loop for each input channel */
    for (ich = 0; ich < nichan; ich++)
        {
	/* Loop for each output channel */
        for (och = 0; och < nochan; och++)
	    {
	    /* Copy the pixel if the input channel is specified to be written */
	    if (oseq_chan[och] == iseq_chan[ich])
		ovid_data[samp*(nochan)+och] = ivid_data[samp*(nichan)+ich];

	    }
    	}
    }

}
