/******************************************************************************
FUNCTION:       GETPAR

PURPOSE:        Get the input parameters


Version  Date   Programmer       Modifications
-------  -----  ---------------- ----------------------------------------------
         09/97  D. Lloyd         Development

PROJECT:        ADAPS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"


int getpar
(
    struct PARBLK *vblock,	/* Variable block			*/
    char *infile,		/* Input file name			*/
    char *outfile,		/* Output file name			*/
    char *density,		/* Output tape density			*/
    char *ttype,		/* Output tape type			*/
    char *drivename,		/* Drive name of tape			*/
    char comment[][CMLEN],	/* Tape mount comment			*/
    long *io_option,		/* I/O option flag			*/
    long *spare_val		/* spare value use as needed		*/
)
{
long
	access = IUPDATE,	/* File access				*/
	diskin = FALSE,		/* Disk input flag			*/
	diskout = FALSE,	/* Disk output flag			*/
	i,			/* Loop index variable			*/
	tape = FALSE,		/* Tape flag				*/
	fatal = -1,		/* Fatal flag				*/
	status,			/* Return status			*/
	zero = 0;		/* Value of zero			*/

char	tempname[CMLEN];	/* Temporary name variable		*/

struct VARIABLE *p_find();	/* Function to find a TAE variable	*/
struct VARIABLE *var;		/* Pointer to a TAE variable		*/


/*
  Get input filename if specified
*/
var = p_find( vblock, "INFILE" );
if ( var->v_count > 0  )
    {
    diskin = TRUE;
    access = IREAD;
    status = c_getnam( vblock, "INFILE", &zero, infile, "l1b", &access );
    if ( status != E_SUCC )
	c_errmsg("Error getting input file","writel1b-fatal",&fatal);
    }
else
    infile[0] = 0;


/*
  Get output file name if specified
*/
var = p_find( vblock, "OUTFILE" );
if ( var->v_count > 0 )
    {
    diskout = TRUE;
    access = IWRITE;
    status = c_getnam( vblock, "OUTFILE", &zero, outfile, "l1b", &access );
    if ( status != E_SUCC )
        c_errmsg("Error getting input file","writel1b-fatal",&fatal);
    }
else
    outfile[0] = 0;


/*
  Get the tape density
*/
var = p_find( vblock, "TAPEDENS" );
strcpy( density, SVAL( *var,0 ));


/*
  Get the tape type
*/
var = p_find( vblock, "TTYPE" );
strcpy( ttype, SVAL( *var, 0 ));


/*
  Get the tape drive name
*/
var = p_find( vblock, "DRIVENAME" );
strcpy( drivename, SVAL( *var, 0 ));


/*
  Get tape mount comment if specified
*/
var = p_find( vblock, "COMMENT" );
if ( var->v_count > 0 )
    {
    tape = TRUE;
    for ( i = 0; i < var->v_count; i++ )
	strcpy( comment[i], SVAL( *var, i ));
    }
else
    {
    comment[0][0] = 0;
    comment[1][0] = 0;
    }


/*
  Get record size
*/
var = p_find( vblock, "SPAREVAL" );
*spare_val = (long)IVAL( *var, 0 );


/*
  Set I/O option flag and check that illegal option not specified
*/
if ( diskin && diskout && tape )
    return E_FAIL;
else if ( diskin && diskout )
    *io_option = 0;
else if ( diskin && tape )
    *io_option = 1;
else if ( diskout && tape )
    *io_option = 2;
else
    return E_FAIL;
    

return E_SUCC;

}  /* END getpar */

