/*******************************************************************************
FUNCTION:	L1B_DSH_TIME

PURPOSE:	Get the time (year, julian day, GMT time) from the
 	        Data Set Header record format.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T.Baltzer        Development
  1.1	 07/98  D.Lloyd		 Modified for Y2K compliance


ALGORITHM:  Time code format ( 6 bytes)

   1       1       1       1       1       1       1
   YYYYYYYJJJJJJJJJUUUUUTTTTTTTTTTTTTTTTTTTTTTTTTTT

     Year   Day    Unused   GMT Time
    7 bits  9 bits  5 bits   27 bits

ALGORITHM REFERENCES:
	1. NOAA Polor Orbital User's Guide

PROJECT:	ADAPS
*******************************************************************************/
#include "las.h"



void l1b_dsh_time
(
    unsigned char inbuf[],	/* Input buffer 			*/
    long *year,			/* acquisition year			*/
    long *jday,			/* julian day of acquisition		*/
    long *gmtime		/* acquisition GMT time in milliseconds	*/
)

{

short 	*sword;			/* pointer to left-most two bytes 	*/
long	*lword;			/* pointer to right-most four bytes	*/

/*
  Get the year
*/
sword = (short *) inbuf;
*year = (long) ((*sword & 0XFE00) >> 9);
/**********************************************
DLLOYD Y2K
  mod the year with 100 to force it to 2-digit
  just in case it isn't
***********************************************/
*year %= 100;

/*
  Get the Julian day
*/
*jday = (long) (*sword & 0X01FF);

/*
  Get the GMT time in milliseconds
*/
lword = (long *) (inbuf + 2);
*gmtime = *lword & 0X07FFFFFF;


return;
}
