/*******************************************************************************
FUNCTION:	L1B_LOAD_MOD

PURPOSE:	Load the AVHRR model with the TBM/ARS and DSH Header record
		info.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T. Baltzer 	 Development
  1.1    12/98  D.Lloyd          Modifications for NOAA KLM

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "level1bio.h"
#include "local_protos.h"


int l1b_load_mod
(
    unsigned char hdrbuf[],	/* TBM/ARS header buffer		*/
    unsigned char dshbuf[],	/* DSH header buffer			*/
    long recsiz,		/* Record size of data read    		*/
    long hdrflg,		/* TBM/ARS header flag (TRUE or FALSE)	*/
    long *goodlines,	/* Number of good lines in the image		*/
    struct MODEL *model	/* AVHRR model information			*/
)

{

long	status,			/* Return status variable		*/
	totchdsh;		/* Total bands according to DSH 	*/


/*
  Initialize some of the model structure
*/
l1b_init_model(model);


/*
  Extract and load the info from the DSH header into the model
*/
switch ( recsiz )
    {
    case KLM_UNP_LAC_1 :
    case KLM_UNP_LAC_2 :
    case KLM_UNP_LAC_3 :
    case KLM_UNP_LAC_4 :
    case KLM_UNP_LAC_5 :
    case KLM_PCK_LAC_5 :
    case KLM_UNP_GAC_1 :
    case KLM_UNP_GAC_2 :
    case KLM_UNP_GAC_3 :
    case KLM_UNP_GAC_4 :
    case KLM_UNP_GAC_5 :
    case KLM_PCK_GAC_5 :
        status = l1b_load_klmdsh(dshbuf,recsiz,&totchdsh,goodlines,model);
	break;
    default :
	status = l1b_load_dsh(dshbuf,recsiz,&totchdsh,goodlines,model);
    }
if (status != E_SUCC)
    return(status);


/*
  Initialize the model, do it here before we try to make a TBM/ARS header
*/
model->ehour = 0; model->emin = 0; model->esec = 0; model->efrac = 0;
c_initmod(&model->satnum, &model->year, &model->month, &model->day,
          &model->shour, &model->smin, &model->ssec, &model->sfrac,
          &model->dtype, &model->nlines, model);


/*
  If a TBM/ARS header doesn't exist:
     (1) Make a TBM/ARS header from the DSH header.
     (2) Load the model with channel info without the use of a TBM/ARS header
  else (TBM/ARS header does exist)
     Load the model with the TBM/ARS header information.
*/
if (hdrflg == FALSE)
    {
    l1b_load_chan(&totchdsh,model);
    if ( model->satnum <= 14 )
	status = l1b_make_tbm(hdrbuf,dshbuf,model);
    else
	status = l1b_make_ars(hdrbuf,dshbuf,model,&recsiz);
    if (status != E_SUCC)
	return(status);
    }
else
    {
    if ( model->satnum <= 14 )
	status = l1b_load_tbm(hdrbuf,totchdsh,model);
    else
	status = l1b_load_ars(hdrbuf,totchdsh,model);
    if (status != E_SUCC)
	return(status);
    }


return(E_SUCC);
}
