/*****************************************************************************
FUNCTION:	L1B_MAKE_DSH

PURPOSE:	Build a DSH record from AVHRR model and drop line structures.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development
  1.1    01/99	D.Lloyd		Added 4-digit year to bytes 38-39 of record

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "avhrr.h"
#include "archio.h"
#include "level1bio.h"
#include "local_protos.h"


int l1b_make_dsh
(
    unsigned char *dsh,		/* pointer to DSH buffer		*/
    struct MODEL *model,	/* AVHRR model structure		*/
    struct DROPPED *drop,	/* dropped line structure		*/
    long recsiz			/* data record size in bytes		*/
)

{
char	*lines;			/* number of lines in image		*/
char	orbit[5];		/* ASCII representation of orbit number	*/

long	i;			/* lopp index variable			*/
long	msec;			/* milliseconds of acquisition		*/
long	satcode = 0;		/* satellite code			*/
long	totdrop;		/* total drop lines			*/

short	temp;			/* number of data gaps			*/
short	*sptr;			/* pointer to a short			*/


/*
  spacecraft id
*/
switch ( model->satnum )
    {
    case  6 :
    case 13 :
	satcode = 2;
	break;
    case  7 :
	satcode = 4;
	break;
    case  8 :
	satcode = 6;
	break;
    case  9 :
	satcode = 7;
	break;
    case 10 :
	satcode = 8;
	break;
    case 11 :
	satcode = 1;
	break;
    case 12 :
	satcode = 5;
	break;
    case 14 :
	satcode = 3;
	break;
    }  /* END switch */
dsh[0] = (char)satcode;

/*
  Data type
*/
dsh[1] = (char)( model->dtype << 4 );

/*
  Start time, not including the time correction
*/
msec = model->shour * MIL_PER_HOUR;
msec += model->smin * MIL_PER_MIN;
msec += model->ssec * MIL_PER_SEC;
msec += model->sfrac;
msec += model->clock_err;
l1b_pack_time( (char *)(dsh+2), model->year, model->jday, msec );

/*
  End time, not including the time correction
*/
msec = model->ehour * MIL_PER_HOUR;
msec += model->emin * MIL_PER_MIN;
msec += model->esec * MIL_PER_SEC;
msec += model->efrac;
msec += model->clock_err;
l1b_pack_time( (char *)(dsh+10), model->year, model->jday, msec );

/*
  Number of lines
*/
for ( totdrop = 0, i = 0; i < drop->ngaps; i++ )
    totdrop += drop->nmissed[i];
temp = (short)(model->nlines - totdrop);
lines = (char *)&temp;
for ( i = 0; i < 2; i++ )
    dsh[i+8] = lines[i];

/*
  Processing block ID
*/
sprintf( orbit, "%5ld", model->sorbit );
for ( i = 0; i < 5; i++ )
    dsh[i+16] = orbit[i];
sprintf( orbit, "%5ld", model->eorbit );
dsh[21] = orbit[3];
dsh[22] = orbit[4];

/*
  Ramp/auto calibration defaults to zero
*/
dsh[23] = 0;

/*
  Number of data gaps
*/
temp = (short)(drop->ngaps);
lines = (char *)&temp;
for ( i = 0; i < 2; i++ )
    dsh[i+24] = lines[i];

/*
  DACS quality, calibration parameter ID default to zero
*/
for ( i = 26; i < 34; i ++ )
    dsh[i] = 0;

/*
  DACS status a constant defined to be: normal data, data source other,
  tape direction forward, data mode flight data.
*/
dsh[34] = 0x18;

/*
  Zero-filled to make header record the same size as a data record
*/
for ( i = 35; i < recsiz; i++ )
    dsh[i] = 0;

/*
  Store a 4-digit year in bytes 38-39
*/
sptr = (short *)&dsh[38];
*sptr = (short)c_4digityr( model->year );

return E_SUCC;

}  /* END l1b_make_dsh */
