/*****************************************************************************
FUNCTION:	L1B_PACK_TIME

PURPOSE:	Pack the year, julian day, and time (in miliseconds) into
		a 6-byte Level 1b time code format.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    10/97	D.Lloyd		Initial development
  1.1	 07/98	D.Lloyd		Modified for Y2K compliance

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include "las.h"
#include "adaps.h"
#include "avhrr.h"


int l1b_pack_time
(
    char buf[],		/* time code output buffer		*/
    long year,		/* year of acquisition			*/
    long jday,		/* julian day of acquisition		*/
    long msec		/* milliseconds from midnight		*/
)

{
char	arr[6];		/* temporary character array		*/

long	i;		/* loop index				*/
long	*lptr;		/* pointer to a long			*/
long	temp;		/* temporary variable			*/

short	*sptr;		/* pointer to a short			*/


/*
  Point both pointers at the temp array
*/
sptr = (short *)&arr[0];
lptr = (long *)&arr[0];

/*
  Put the year into upper 7 bits of temp and add the julian day
*/
/*********************************************************
DLLOYD Y2K
  force year to 2-digit even though it already should be
**********************************************************/
temp = (( year % 100 ) << 9) + (jday & 0x1ff);

/*
  Put year and jday into first two bytes of buffer
*/
*sptr = (short)temp;
for ( i = 0; i < 2; i++ )
     buf[i] = arr[i];

/*
  Put the milliseconds in the last 4 bytes of buffer
*/
*lptr = msec;
for (i = 0; i < 4; i++ )
    buf[i+2] = arr[i];

return E_SUCC;

}  /* END l1b_pack_time */
