/*************************************************************************
FUNCTION:	L1B_PRODSUM

PURPOSE:	Generate the level-1b product summary report

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       12/87    D. Hollaren         Original development for ADAPS
 5.0 	   12/91    T. Rockvam		Conversion to LAS 5.0; incorporated
					into mklevel1b; changed strncat to
					strcat to cat on the end of blanks

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT		ADAPS

ALGORITHM REFERENCES

*************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#define PWIDE 80		/* width of page			*/

int l1b_prodsum
(
    char *out,		 	 /* output file name 			 */
    char *acctnum,               /* account number                       */
    char *ordernum,              /* order number                         */
    char *unitnum,               /* unit number                          */
    char *cdate,                 /* current date                         */
    char *sceneid,               /* scene identifier                     */
    long orbit,                  /* orbit number                         */
    char *acqdate,               /* acquisition date                     */
    char *acqtime,               /* acquisition time                     */
    char *prodesc,               /* product description                  */
    char *satid,                 /* satellite name                       */
    char *source,                /* sensor data type (HRPT)              */
    double uplat,                /* upper left latitude                  */
    double uplong,               /* upper left longitude                 */
    double lolat,                /* lower right latitude                 */
    double lolong,               /* lower right longitude                */
    double beglat,               /* beginning latitude of output file    */
    double endlat,               /* ending latitude of output file       */
    double beglong,              /* beginning longitude of output file   */
    double endlong,              /* ending longitude of output file      */
    long nl,                     /* number of lines in output file       */
    long ns,                     /* number of samples in output file     */
    char *density              	 /* tape density (6250, 1600)            */
)

{
FILE	*fp;			/* pointer to the output file		*/

long	i;			/* loop counter				*/

char	buffer[PWIDE],		/* output buffer			*/
	blanks[PWIDE],		/* 80 characters of blanks		*/
	fill[PWIDE];		/* string of blanks for filler		*/

/*
  Open the product summary file
*/
fp = fopen(out,"w");
if ( !fp )
    {
    out[0] = 0;
    sprintf(buffer,"Unable to open file: %s",out);
    c_lasmsg(FALSE,"c_l1bsumm-open",buffer);
    return E_FAIL;
    }

/*
  Generate a string of blanks
*/
for (i = 0; i < PWIDE; i++)
    blanks[i] = ' ';
blanks[PWIDE - 1] = '\0';

/*
  Print the header for the level-1b product summary
*/

/* skip two lines from the top of the page */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/* generate the title line */
strcpy(fill,&blanks[PWIDE-22]);
sprintf(buffer,"%sAVHRR LEVEL-1B PRODUCT SUMMARY REPORT",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the underline for the title */
sprintf(buffer,"%s-------------------------------------",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/* write out the account and date */
strcpy(fill,&blanks[PWIDE-17]);
sprintf(buffer,"%sAccount: %s",fill,acctnum);
strcat(buffer,&blanks[PWIDE-57+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Date: %s",cdate);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* write out the order and unit numbers */
strcpy(fill,&blanks[PWIDE-11]);
sprintf(buffer,"%sOrder-units #: %s-%s",fill,ordernum,unitnum);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/*
  Print the identification information for the level-1b product summary
*/

/* generate the string buffer for the scene id and orbit number */
strcpy(fill,&blanks[PWIDE-7]);
sprintf(buffer,"%sScene Identification: %s",fill,sceneid);
strcat(buffer,&blanks[PWIDE-56+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Orbit: %ld",orbit);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the acquistion date and time */
strcpy(fill,&blanks[PWIDE-11]);
sprintf(buffer,"%sAcquisition Date: %s",fill,acqdate);
strcat(buffer,&blanks[PWIDE-57+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Time: %s",acqtime);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the product description and satellite */
strcpy(fill,&blanks[PWIDE-8]);
sprintf(buffer,"%sProduct Description: %s",fill,prodesc);
strcat(buffer,&blanks[PWIDE-52+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Satellite: %s",satid);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the sensor data type */
strcpy(fill,&blanks[PWIDE-11]);
sprintf(buffer,"%sSensor Data Type: %s",fill,source);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);
	
/*
  Print the user specified area for the level-1b product summary
*/

/* generate the title line and write it to the output file */
strcpy(fill,&blanks[PWIDE-31]);
sprintf(buffer,"%sUser Specified Area",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the underline for the title and write it to the output file */
sprintf(buffer,"%s-------------------",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/* generate the string buffer for the upper left lat and beginning lat */
strcpy(fill,&blanks[PWIDE-8]);
sprintf(buffer,"%sUpper Left Latitude: %f",fill,uplat);
strcat(buffer,&blanks[PWIDE-43+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Beginning Latitude: %f",uplat);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the upper left long and ending lat */
strcpy(fill,&blanks[PWIDE-7]);
sprintf(buffer,"%sUpper Left Longitude: %f",fill,uplong);
strcat(buffer,&blanks[PWIDE-46+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Ending Latitude: %f",lolat);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the lower right latitude area */
sprintf(buffer,"%sLower Right Latitude: %f",fill,lolat);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the lower left latitude area */
strcpy(fill,&blanks[PWIDE-6]);
sprintf(buffer,"%sLower Right Longitude: %f",fill,lolong);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);
	
/*
  Print delivered product information for the level-1b product summary 
*/

/* generate the title line and write it to the output file */
strcpy(fill,&blanks[PWIDE-32]);
sprintf(buffer,"%sDelivered Product",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the underline for the title and write it to the output file */
sprintf(buffer,"%s-----------------",fill);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);

/* generate the string buffer for the beginning latitude and density */
strcpy(fill,&blanks[PWIDE-9]);
sprintf(buffer,"%sBeginning Latitude: %f",fill,beglat);
strcat(buffer,&blanks[PWIDE-54+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Density: %s",density);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the ending latitude and number of lines */
strcpy(fill,&blanks[PWIDE-12]);
sprintf(buffer,"%sEnding Latitude: %f",fill,endlat);
strcat(buffer,&blanks[PWIDE-49+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Number Lines: %ld",nl);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the beginning long and number of samples */
strcpy(fill,&blanks[PWIDE-8]);
sprintf(buffer,"%sBeginning Longitude: %f",fill,beglong);
strcat(buffer,&blanks[PWIDE-47+strlen(buffer)]);
sprintf(&buffer[strlen(buffer)],"Number Samples: %ld",ns);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* generate the string buffer for the ending longitude */
strcpy(fill,&blanks[PWIDE-11]);
sprintf(buffer,"%sEnding Longitude: %f",fill,endlong);
strcat(buffer,&blanks[strlen(buffer)]);
fprintf(fp,"%s\n",buffer);

/* skip another two lines */
fprintf(fp,"%s\n%s\n",blanks,blanks);
	
/*
  Close the product summary file
*/
fclose (fp);

return E_SUCC;
}
