/*******************************************************************************
FUNCTION:	L1B_TIM_COR

PURPOSE:	Get the NBS clock offset for a scene and apply it to the model.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T.Baltzer        Original development

PROJECT:	ADAPS
*******************************************************************************/

#include "las.h"
#include "adaps.h"
#include "adapsutil.h"
#include "acqio.h"



int l1b_tim_cor
(
    struct MODEL *model	/* AVHRR model information		*/
)

{

long	etime,		/* ending GMT time in milliseconds	*/
	stime;		/* starting GMT time in milliseconds	*/


/*
  Make sure that the NBS offset has not already been applied to the model.
  If it has been applied, error out.
*/
if (model->clock_err != 0)
    {
    c_lasmsg(FALSE,"l1bopnr-cerr","NBS offset apparently already applied");
    return(E_FAIL);
    }


/*
  Determine the starting/ending time in milliseconds using the model
*/
c_cvtime(&stime,&model->shour,&model->smin,&model->ssec);
stime = stime * 1000 + model->sfrac;

c_cvtime(&etime,&model->ehour,&model->emin,&model->esec);
etime = etime * 1000 + model->efrac;


/* 
  Find the time correction from the $ADAPSTABLES/noaa??.timcor file.
*/
if (c_acqtmcor(&model->satnum,&model->month,&model->day,&model->year,
	       &model->shour,&model->smin,&model->ssec,&model->sfrac,
	       &model->clock_err) != E_SUCC)
    return(E_FAIL);


/*
  Level1b files contain the original satellite time stamps without NBS
  time corrections applied.  Apply the time correction to the original
  satellite time stamps here.  This correction in the model's starting/ending
  times will change the geometry in the model.
  NOTE: Do NOT worry about spanning days or years with this millisecond
  adjustment because at most only one of the times (start or end time)
  will span a day or year.  The model does not store a date for start time
  and a date for stop time, but instead just one start date.  Consequently,
  even if one of the times (start or end time) spans a day, leave it as is.
*/
stime -= model->clock_err;
etime -= model->clock_err;


/*
  Convert the starting/ending time in milliseconds to hours, minutes
  seconds and milliseconds (put in the AVHRR model structure)
*/
c_cvtmil(&stime,&model->shour,&model->smin,&model->ssec,&model->sfrac);
c_cvtmil(&etime,&model->ehour,&model->emin,&model->esec,&model->efrac);


return(E_SUCC);
}
