
#ifndef LOCAL_PROTOS_H
#define LOCAL_PROTOS_H

#include "model.h"


/* Prototype routines that are not to be called outside of this directory     */

void copy_chan
(
    short *ivid_data, 	/* Pointer to one scan line of input		*/
    short *ovid_data, 	/* Pointer to one scan line of output		*/
    long ichan[NCHANS], /* Input channels present			*/
    long ochan[NCHANS], /* Output channels to be processed		*/
    long nsamples	/* Number of samples per line			*/
);

int get_rec_tape
(
    long *tun,			/* Unit number for tape drive		*/
    unsigned char *recptr,	/* Pointer to the Level 1b scan record	*/
    long *nbyte,		/* Record size of data to read		*/
    long *volnum,		/* Volume number			*/
    char comment[][CMLEN],	/* Description of tape			*/
    long *tape			/* Flag for tape open			*/
);

int getpar
(
    struct PARBLK *vblock,	/* Variable block			*/
    char *infile,		/* Input file name			*/
    char *outfile,		/* Output file name			*/
    char *density,		/* Output tape density			*/
    char *ttype,		/* Output tape type			*/
    char *drivename,		/* Drive name of tape			*/
    char comment[][CMLEN],	/* Tape mount comment			*/
    long *io_option,		/* I/O option flag			*/
    long *spare_val		/* spare value use as needed		*/
);

void l1b_ck_tstamp
(
    unsigned char *bscan, /* Pointer to the Level 1b scan line	*/
    struct MODEL *model	  /* AVHRR model information	        */
);

void l1b_dsh_time
(
    unsigned char inbuf[],	/* Input buffer 			*/
    long *year,			/* acquisition year			*/
    long *jday,			/* julian day of acquisition		*/
    long *gmtime		/* acquisition GMT time in milliseconds	*/
);

int l1b_file_in
(
char infile[],		/* Input file                                   */
FILE	**in_fd,	/* File descriptor to the input  file   	*/
unsigned char hdrbuf[],	/* TBM/ARS header buffer			*/
unsigned char dshbuf[],	/* DSH header buffer				*/
unsigned char linbuf[],	/* Buffer containing first line of data		*/
long	*recsiz,	/* Record size of data read             	*/
long *dtype,		/* Data type LAC/GAC				*/
long *linfile,		/* Lines in file according to file size		*/
long *hdrflg,		/* TBM/ARS header flag (yes or no)		*/
long *nchans_tmp 	/* Number of channels (temporarily used)	*/
);

int l1b_format
(
    char infile[CMLEN],		/* Input file name (input)	*/
    long hdr_flg,		/* TBM header present? (input)	*/
    long *pack_flg,		/* Data packed? (output)	*/
    long *nchans,		/* Number of channels (output)	*/
    long *dtype,		/* Data type LAC/GAC (output)	*/
    long *linfile		/* Number of lines in image (output) */
);

void l1b_init_model
(
    struct MODEL *model		/* AVHRR model information              */
);

int l1b_load_ars
(
    unsigned char ars[],	/* ARS header record			*/
    long totchdsh,		/* Total channels from DSH header	*/
    struct MODEL *model		/* AVHRR model structure		*/
);

void l1b_load_chan
(
    long *totchdsh,	/* Total number of bands according to the DSH         */
    struct MODEL *model /* AVHRR model information			      */
);

int l1b_load_dsh
(
    unsigned char dshbuf[],	/* Input buffer from file		*/
    long bytesread,		/* number of bytes read into dshbuf	*/
    long *totchdsh,		/* Total number of bands according to DSH*/
    long *goodlines,		/* total good lines in the input image	*/
    struct MODEL *model		/* AVHRR model information		*/
);

int l1b_load_klmdsh
(
    unsigned char dshbuf[],	/* Input buffer from file		*/
    long bytesread,		/* number of bytes read into dshbuf	*/
    long *totchdsh,		/* Total number of bands according to DSH*/
    long *goodlines,		/* total good lines in the input image	*/
    struct MODEL *model		/* AVHRR model information		*/
);

int l1b_load_mod
(
    unsigned char hdrbuf[],	/* TBM/ARS header buffer		*/
    unsigned char dshbuf[],	/* DSH header buffer			*/
    long recsiz,		/* Record size of data read    		*/
    long hdrflg,		/* TBM/ARS header flag (TRUE or FALSE)	*/
    long *goodlines,	/* Number of good lines in the image		*/
    struct MODEL *model	/* AVHRR model information			*/
);

int l1b_load_tbm
(
    unsigned char tbmbuf[],	/* TBM header record		*/
    long totchdsh,		/* Total channels from DSH header*/
    struct MODEL *model		/* AVHRR model structure	*/
);

int l1b_make_ars
(
unsigned char	*ars,		/* ARS header buffer		*/
unsigned char	*dsh,		/* DSH record			*/
struct MODEL	*model,		/* AVHRR model structure	*/
long		*recsiz 	/* data record size		*/
);

int l1b_make_dsh
(
    unsigned char *dsh,		/* pointer to DSH buffer		*/
    struct MODEL *model,	/* AVHRR model structure		*/
    struct DROPPED *drop,	/* dropped line structure		*/
    long recsiz			/* data record size in bytes		*/
);

int l1b_make_klmdsh
(
    unsigned char *dsh,         /* pointer to DSH buffer                */
    struct MODEL *model,        /* AVHRR model structure                */
    struct DROPPED *drop,	/* dropped line structure		*/
    long recsiz         	/* data record size in bytes            */
);

int l1b_make_tbm
(
    unsigned char *tbm,		/* TBM header buffer		*/
    unsigned char *dsh,		/* DSH record			*/
    struct MODEL *model		/* AVHRR model structure	*/
);

int l1b_pack_time
(
    char buf[],		/* time code output buffer		*/
    long year,		/* year of acquisition			*/
    long jday,		/* julian day of acquisition		*/
    long msec		/* milliseconds from midnight		*/
);

int l1b_prodsum
(
    char *out,		 	 /* output file name 			 */
    char *acctnum,               /* account number                       */
    char *ordernum,              /* order number                         */
    char *unitnum,               /* unit number                          */
    char *cdate,                 /* current date                         */
    char *sceneid,               /* scene identifier                     */
    long orbit,                  /* orbit number                         */
    char *acqdate,               /* acquisition date                     */
    char *acqtime,               /* acquisition time                     */
    char *prodesc,               /* product description                  */
    char *satid,                 /* satellite name                       */
    char *source,                /* sensor data type (HRPT)              */
    double uplat,                /* upper left latitude                  */
    double uplong,               /* upper left longitude                 */
    double lolat,                /* lower right latitude                 */
    double lolong,               /* lower right longitude                */
    double beglat,               /* beginning latitude of output file    */
    double endlat,               /* ending latitude of output file       */
    double beglong,              /* beginning longitude of output file   */
    double endlong,              /* ending longitude of output file      */
    long nl,                     /* number of lines in output file       */
    long ns,                     /* number of samples in output file     */
    char *density              	 /* tape density (6250, 1600)            */
);

int l1b_tape_in
(
    char drive[CMLEN],		/* Tape drive name			*/
    long *tun,			/* Tape unit number			*/
    unsigned char hdrbuf[],	/* TBM/ARS header buffer		*/
    unsigned char dshbuf[],	/* DSH header buffer			*/
    unsigned char linbuf[],	/* Buffer containing first line of data	*/
    long *recsiz,		/* Record size of data read            	*/
    long *hdrflg,		/* TBM/ARS header flag (TRUE or FALSE)	*/
    long *tape			/* Flag for tape open			*/
);

int l1b_tim_cor
(
    struct MODEL *model	/* AVHRR model information		*/
);

void l1bvideo
(
    long irecsiz,		/* Input buffer record size	*/
    long ichan[],		/* Input channels within inbuf	*/
    unsigned char *ibuf,	/* Input buffer			*/
    long orecsiz,		/* Output buffer record size	*/
    long ochan[],		/* Output channels within obuf	*/
    unsigned char *obuf,	/* Output buffer		*/
    long videosize		/* Output video buffer size	*/
);

int put_rec_tape
(
    long *tun,             /* tape logical unit number             */
    unsigned char *recptr, /* pointer to level 1b scan record      */
    long *nbyte,	   /* number of bytes in one record        */
    long *tape             /* tape open flag                       */
);

int span
(
    struct MODEL *model,	/* AVHRR model structure		*/
    long *month,		/* ending month				*/
    long *day,			/* ending day				*/
    long *year,			/* ending year				*/
    long *jday			/* ending julian day			*/
);

#endif

