/*****************************************************************************
FUNCTION:	SPAN

PURPOSE:	Calculate the ending date of a scene based on start and
		end times.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    12/98	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapsutil.h"
#include "avhrr.h"


int span
(
    struct MODEL *model,	/* AVHRR model structure		*/
    long *month,		/* ending month				*/
    long *day,			/* ending day				*/
    long *year,			/* ending year				*/
    long *jday			/* ending julian day			*/
)

{
long	leapflg;		/* leap year flag			*/
long	lday;			/* local copy of day			*/
long	lmon;			/* local copy of month			*/
long	lyear;			/* local copy of year			*/
long	ljday;			/* local copy of jday			*/

static	long	mdays[2][13] = {{365,31,28,31,20,31,30,31,31,30,31,30,31},
				{366,31,29,31,20,31,30,31,31,30,31,30,31}};


if ( model->ehour < model->shour )
    {
    lday = model->day + 1;
    lmon = model->month;
    lyear = model->year;
    ljday = model->jday + 1;
    leapflg = c_isleap( &lyear );
    if ( lday > mdays[leapflg][lmon] )
        {
	lday = 1;
	lmon++;
	if ( lmon > 12 )
	    {
	    lmon = 1;
	    lyear++;
	    ljday = 1;
	    }
	}
    *month = lmon;
    *day = lday;
    *year = lyear;
    *jday = ljday;
    }
else
    {
    *month = model->month;
    *day = model->day;
    *year = c_4digityr( model->year );
    *jday = model->jday;
    }

return E_SUCC;
}
