/******************************************************************************
FUNCTION: TEST

PURPOSE:  Read and write level 1b records.  This may be done in the following
	  combinations:

		disk in, disk out
		disk in, tape out
		tape in, disk out

          Run this from the test.pdf

VERSION  DATE   PROGRAMMER          MODIFICATIONS
-------  -----  ------------------  -------------------------------------------
         09/97  D.Lloyd             Original development

ASSUMPTIONS:  For tape operation it is assumed that a tape has been mounted
	      and a drive name assigned for use by the call to c_tpreop.

PROJECT:  ADAPS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "tapeio.h"
#include "avhrr.h"
#include "level1bio.h"



main()
{
struct
	PARBLK vblock;		/* Variable block			*/

long
	din = FALSE,		/* disk input flag			*/
	dout = FALSE,		/* disk output flag			*/
	end_file = FALSE,	/* End of input file flag		*/
	error = FALSE,		/* I/O error flag			*/
	fatal = -1,		/* Fatal error status			*/
	inrecsiz,		/* Input record size			*/
	io_option = -1,		/* I/O option flag			*/
	one = 1,		/* Count of 1				*/
	orecsiz,		/* output record size			*/
	status,			/* Return status variable		*/
	drop_line,		/* number of drop lines skipped in seek	*/
	spare_val,		/* Spare input parameter value		*/
	std_out = 0,		/* Standard output			*/
	tape = FALSE,		/* Tape open flag			*/
	tun,			/* Tape logical unit number		*/
	volnum = 1,		/* Tape volume number			*/
	window[4];		/* window: SL, SS, EL, ES		*/

long	ichan[5] = {1, 2, 3, 4, 5};	/* Input channel specification	*/
long	ochan[5] = {1, 2, 3, 4, 5};	/* Output channel specification	*/

char
	comment[2][CMLEN],	/* Tape mount comment			*/
	density[CMLEN],		/* Output tape density			*/
	drive[CMLEN],		/* Name of tape drive			*/
	infile[CMLEN],		/* Input file name			*/
	out[2][CMLEN],		/* Output name				*/
	outfile[CMLEN],
	ttype[CMLEN];		/* Output tape type			*/

FILE
	*inf = NULL,		/* Input file descriptor		*/
	*ouf = NULL;		/* Output file descriptor		*/

unsigned char
	buffer[MAXLINSIZ],	/* Input/output buffer			*/
	obuf[MAXLINSIZ],	/* Output buffer			*/
	dsh[MAXLINSIZ],		/* DSH buffer				*/
	tbm[TBM_HDR_SIZ];	/* TBM header buffer			*/

struct MODEL model;	    /* AVHRR model information              */
struct DROPPED dumdrop;     /* Dropped line information                   */


/*
  Initialize LAS
*/
c_inlas( &vblock, &std_out );

/*
  Get input parameters
*/
getpar( &vblock, infile, outfile, density, ttype, drive, comment,
	&io_option, &spare_val );


/*
  Determine the desired density
*/
if ( strcasecmp( ttype, "8hi" ) == 0 )
    strcpy( density, "HI" );
else if ( strcasecmp( ttype, "8lo" ) == 0 )
    strcpy( density, "LO" );
else
    strcpy( density, "0" );

/*
  Open input and output files /tape
*/
switch ( io_option )
    {
    case 0 :
	/*
          Open disk file for input
        */
	status = c_l1bopndr(infile,&inf,&inrecsiz,tbm,dsh,&model);
	if ( status != E_SUCC )
            c_errmsg( "Unable to open input file", "test-fatal", &fatal );
	/*
	  Put the model to a header file here for testing purposes only!
	*/
	if (c_putahdr(outfile,&model,&dumdrop) != E_SUCC)
            c_errmsg( "Unable to write model file", "test-fatal", &fatal );

	/*
	  Set up test window
	*/
	window[0] = spare_val;
	window[1] = 1;
	window[2] = model.nlines;
	window[3] = 2048;

	/*
	  Seek to proper location in input file
	*/
	status = c_l1bseekd( inf, window, &inrecsiz, &model, &drop_line );
	if ( status != E_SUCC )
	    c_lasmsg(FALSE,"Error seeking to line in input","test-seek");
printf("window[0]: %d    drop: %d\n",window[0],drop_line);
	/*
	  Open disk file for output
	*/
	orecsiz = PCK_GAC_5;
	status = c_l1bmkhdr( &model, &dumdrop, window, ochan, &orecsiz,
			     tbm, dsh );
	if ( status != E_SUCC )
            c_errmsg( "Unable to make header str", "test-fatal", &fatal );

	status = c_l1bopndw( outfile, &orecsiz, tbm, dsh, &ouf );
	if ( status != E_SUCC )
            c_errmsg( "Unable to open output file", "test-fatal", &fatal );
	din = dout = TRUE;
	break;
    case 1 : 
	/*
	  Open disk file for input
	*/
	status = c_l1bopndr(infile,&inf,&inrecsiz,tbm,dsh,&model);
	if ( status != E_SUCC )
            c_errmsg( "Unable to open input file", "test-fatal", &fatal );
	/*
	  Put the model to a header file here for testing purposes only!
	*/
	if (c_putahdr(outfile,&model,&dumdrop) != E_SUCC)
            c_errmsg( "Unable to write model file", "test-fatal", &fatal );

	/*
	  Set up test window
	*/
	window[0] = spare_val;
	window[1] = 1;
	window[2] = model.nlines;
	window[3] = 2048;

	/*
	  Seek to proper location in input file
	*/
	status = c_l1bseekd( inf, window, &inrecsiz, &model, &drop_line );
	if ( status != E_SUCC )
	    c_lasmsg(FALSE,"Error seeking to line in input","test-seek");
printf("window[0]: %d    drop: %d\n",window[0],drop_line);

	/*
	  Open tape for output
	*/
	orecsiz = PCK_GAC_5;
	status = c_l1bmkhdr( &model, &dumdrop, window, ochan, &orecsiz,
			     tbm, dsh );
	if ( status != E_SUCC )
            c_errmsg( "Unable to make header str", "test-fatal", &fatal );

	status = c_l1bopntw(drive,density,&tun,tbm,dsh,&orecsiz,&tape);
	if ( status != E_SUCC )
	    c_lasmsg(TRUE,"test-tpopen","Error opening tape");
	din = tape = TRUE;
	break;

    case 2 :
        /*
          Open tape for input
        */
        status = c_l1bopntr(drive,&tun,tbm,dsh,&inrecsiz,&model,&tape);
        if ( status != E_SUCC )
            {
            fclose( ouf );
            c_errmsg( "Error opening tape", "test-fatal", &fatal );
            }
	/*
	  Put the model to a header file here for testing purposes only!
	*/
	if (c_putahdr(outfile,&model,&dumdrop) != E_SUCC)
            c_errmsg( "Unable to write model file", "test-fatal", &fatal );

	/*
	  Set up test window
	*/
	window[0] = spare_val;
	window[1] = 1;
	window[2] = model.nlines;
	window[3] = 2048;

	/*
	  Seek to proper location in input file
	*/
	status = c_l1bseekt( &tun, window, &inrecsiz, &model, comment,
			&volnum, &tape, &drop_line );
	if ( status != E_SUCC )
	    c_lasmsg(FALSE,"Error seeking to line in input","test-seek");
printf("window[0]: %d    drop: %d\n",window[0],drop_line);

        /*
          Open disk file for output
        */
	orecsiz = PCK_GAC_5;
	status = c_l1bmkhdr( &model, &dumdrop, window, ochan, &orecsiz,
			     tbm, dsh );
	if ( status != E_SUCC )
            c_errmsg( "Unable to make header str", "test-fatal", &fatal );

	status = c_l1bopndw( outfile, &orecsiz, tbm, dsh, &ouf );
	if ( status != E_SUCC )
            c_errmsg( "Unable to open output file", "test-fatal", &fatal );
        dout = tape = TRUE;
        break;

    }  /* END switch */


/*
  Write Data records to output device
*/
error = end_file = FALSE;
while ( !error && !end_file )
    {
    switch ( io_option )
	{
	case 0 :
	    status = c_l1breadd(inf,&inrecsiz,buffer);
	    if ( status == E_EOF )
		end_file = TRUE;
	    else if ( status != E_SUCC )
		{
		c_errmsg("Read error","test-read");
		error = TRUE;
		}
	    else
		{
		c_l1b2l1b(inrecsiz, ichan, buffer, orecsiz, ochan, obuf);
		status = c_l1bwrited(ouf,obuf,&orecsiz);
		if ( status != E_SUCC )
		    {
		    c_errmsg("Write error","test-write");
		    error = TRUE;
		    }
		}
	    break;

	case 1 :
            status = c_l1breadd(inf,&inrecsiz,buffer);
            if ( status == E_EOF )
                end_file = TRUE;
            else if ( status != E_SUCC )
                {
                c_errmsg("Read error","test-read");
                error = TRUE;
                }
	    else
		{
		c_l1b2l1b(inrecsiz, ichan, buffer, orecsiz, ochan, obuf);
		status = c_l1bwritet(&tun,obuf,&orecsiz,&volnum,&tape);
		if ( status != E_SUCC )
		    {
		    c_errmsg("Write error","test-write");
		    error = TRUE;
		    }
		}
            break;

	case 2 :
	    status = c_l1breadt(&tun,&inrecsiz,comment,&volnum,&tape,buffer);
	    if ( status == TAPE_EOF || status == TAPE_EOV )
		end_file = TRUE;
            else if ( status != E_SUCC )
                {
                c_errmsg("Read error","test-read");
                error = TRUE;
                }
	    else
		{
		c_l1b2l1b(inrecsiz, ichan, buffer, orecsiz, ochan, obuf);
		status = c_l1bwrited(ouf,obuf,&orecsiz);
		if ( status != E_SUCC )
		    {
		    c_errmsg("Write error","test-write");
		    error = TRUE;
		    }
		}
            break;
	}  /* END switch */

    }  /* END while */

/*
  Update tbm and dsh
*/


/*
  Close tape
*/
if ( tape )
    c_l1bclost( &tun, &tape );

/*
  Close disk file
*/
if ( din )
    c_l1bclosd( inf );
if ( dout )
    c_l1bclosd( ouf );

/*
  Generate a completion message
*/
if ( dout )
    sprintf( out[0], "%s", outfile );
else
    sprintf( out[0], "%s %s", infile, "on tape" );
c_complt( "TEST", out, &one );

}  /* END main */

