/******************************************************************************
NAME:		BLATLONG

PURPOSE:	Calulate the geocentric latitude and longitude of the 
		satellite from the orbital elements.

PROGRAM HISTORY:

VERSION         PROGRAMMER           DATE              COMMENTS
-------         ----------           ----              --------
2.0              WIVELL              2/91              Original development

COMPUTER SOFTWARE OR HARDWARE LIMITATIONS:

PROJECT:             ADAPS

ALGORITHM DESCRIPTION:

Find satellite latitude
Find dlongitude, difference in longitude from ascending node to satellite
Find satellite longitude 

ALGORITHM REFERENCES:

**************************************************************************/
#include <math.h>
#include "worgen.h"
#include "avhrr.h"
#include "constants.h"

void blatlong
(
    double *thetas,		/* True anomaly passed equator		*/
    double *anlng,		/* Lonitude of ascending node		*/
    double *inclin,		/* Inclination angle			*/
    double *slat,		/* Geocentric latitude of satellite	*/
    double *slong		/* Longitude of satellite		*/
)
{
double dslong;		/* Difference in longitude			*/

/*
  Find satellite latitude
*/
*slat = asin(sin(*thetas) * sin(*inclin));

/*
  Find dlongitude, difference in longitude from ascending node to satellite
*/
dslong = acos(cos(*thetas) / cos(*slat));
if (*thetas > PIE)
    dslong = - dslong;

/*
  Find satellite longitude
*/
*slong = *anlng - dslong;
*slong = (*slong / PIE2 - (long)(*slong / PIE2)) * PIE2;
if (*slong < 0.0)
    *slong = PIE2 + *slong;

return;
}
 
