/******************************************************************************
NAME:		C_COORDS

PURPOSE:	Calculate the nine locational latitude/longitude points
		within the specified AVHRR image.

PROGRAM HISTORY:

Version  Date   Author           Reason
-------  -----  ---------------  -----------------------------------------------
  2.0    02/93  D.Hollaren       Original development

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "model.h"

int c_coords
(
    struct MODEL *model	/* IN/OUT - Model and orbital information	     */
)
{
float	line,		/* Line to calculate corners			     */
	sample;		/* Sample to calculate corners			     */

double	elev = 0.0;	/* Elevation					     */

/*
  Calculate the North West point of the image
*/
line   = 1.0;
sample = 1.0;
c_ls2ll(&line,&sample,&elev,model,&model->nw[0],&model->nw[1],NULL);

/*
  Calculate the North Nadir point of the image
*/
line   = 1.0;
sample = (float)(model->nsamples / 2.0);
c_ls2ll(&line,&sample,&elev,model,&model->nn[0],&model->nn[1],NULL);

/*
  Calculate the North East point of the image
*/
line   = 1.0;
sample = (float)model->nsamples;
c_ls2ll(&line,&sample,&elev,model,&model->ne[0],&model->ne[1],NULL);

/*
  Calculate the Center West point of the image
*/
line = model->nlines / 2.0;
sample = 1.0;
c_ls2ll(&line,&sample,&elev,model,&model->cw[0],&model->cw[1],NULL);

/*
  Calculate the Center Nadir point of the image
*/
line = model->nlines / 2.0;
sample = (float)(model->nsamples / 2.0);
c_ls2ll(&line,&sample,&elev,model,&model->cn[0],&model->cn[1],NULL);

/*
  Calculate the Center East point of the image
*/
line = model->nlines / 2.0;
sample = (float)model->nsamples;
c_ls2ll(&line,&sample,&elev,model,&model->ce[0],&model->ce[1],NULL);

/*
  Calculate the South West point of the image
*/
line   = (float)model->nlines;
sample = 1.0;
c_ls2ll(&line,&sample,&elev,model,&model->sw[0],&model->sw[1],NULL);

/*
  Calculate the South Nadir point of the image
*/
line   = (float)model->nlines;
sample = model->nsamples / 2.0;
c_ls2ll(&line,&sample,&elev,model,&model->sn[0],&model->sn[1],NULL);

/*
  Calculate the South East point of the image
*/
line   = (float)model->nlines;
sample = (float)model->nsamples;
c_ls2ll(&line,&sample,&elev,model,&model->se[0],&model->se[1],NULL);

return(E_SUCC);
}
 
