/*******************************************************************************
NAME:		C_DAYNITE

PURPOSE:	Calculate the solar zenith angle at a specified point
		The solar zenith angle is then used to determine if
		the image is a day or night pass.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		2/91	D. Hollaren	   CSB      Conversion to LAS5.0

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <math.h>
#include <stdlib.h>
#include "worgen.h"
#include "adaps.h"

void c_daynite
(
    long *jdate,	/* Julian date					*/
    long *year,		/* Year						*/
    double *tsec,	/* Time at point of interest			*/
    double *plat,	/* Latitude  of point				*/
    double *plong,	/* Longitude of point				*/
    double *sunzen,	/* Solar zenith angle				*/
    long *dayn		/* Day/Night indicator  Day(1) Night(-1)	*/
)
{
double	sunlat,		/* Sun latitude location			*/
	sunlong,	/* Sun longitude location			*/
	costheta;	/* Cosine of solar zenith angle			*/

/*
  Get the location of the sun given the date and time
*/
c_sunpos(jdate,year,tsec,&sunlat,&sunlong);

/*
  Calculate the cosine of the sun zenith angle
*/
costheta = sin(*plat) * sin(sunlat) + cos(*plat) * cos(sunlat) * 
	   cos(*plong - sunlong);

if (fabs(costheta) >= 1.0)
    costheta = abs(costheta) / costheta;

/*
  Calculate the sun zenith angle to determine if it's a day or night
*/
*sunzen = acos(costheta);
if (*sunzen >= PIE/2)
    *dayn = NIGHT;
else
    *dayn = DAY;

return;
}
 
