/******************************************************************************
NAME:		C_LL2LS

PURPOSE:	Calculate the line, sample, and view angles for the specified
		latitude and longitude point.

		NOTE:	View angles will not be calculated when the 
			address of "view_angles" equals NULL.

			The line/sample coordinates will automatically
			be flipped for ascending passes that are
			in the satellite view.

PROGRAM HISTORY:
VERSION         PROGRAMMER           DATE              COMMENTS
-------         ----------           ----              --------
  2.0           D. Hollaren          09/91             Original development

******************************************************************************/
#include "worgen.h"
#include "model.h"
#include "avhrr.h"

void c_ll2ls
(
    double *lat,		/* Latitude of point			*/
    double *lng,		/* Longitude of point			*/
    double *elev,		/* Elevation at the specified point	*/
    struct MODEL *model,	/* Model and orbital information	*/
    float *line,		/* Line in satellite scene		*/
    float *sample,		/* Sample in satellite scene		*/
    double view_angles[]   	/* Zenith and Azimuth angles		*/
)
{
double	pline,		/* Actual line in the image		*/
	psamp;		/* Actual sample in the image		*/

/*
  Calculate the line/sample for the specified latitude/longitude
*/
c_inverse(lat,lng,elev,model,&pline,&psamp,view_angles);

/*
  Flip the line/sample coordinates for ascending passes in the satellite
  view.  The model returns flipped (North up) coordinates.
*/
if ((model->cpass == ASCENDING) && (model->sat_view == TRUE))
    {
    *line   = (float)((model->nlines - pline) + 1);
    *sample = (float)((model->nsamples - psamp) + 1);
    }
else
    {
    *line   = (float)pline;
    *sample = (float)psamp;
    }

return;
}
 
