/******************************************************************************
NAME:		C_LS2LL

PURPOSE:	For a given line and sample in the satellite image
		calculate the latitude, longitude, and view angles
		for the specified point.

		NOTE:	View angles will not be calculated when the 
			address of "view_angles" equals NULL.

			The line/sample coordinates will automatically
			be flipped for ascending passes that are
			in the satellite view.

PROGRAM HISTORY:

VERSION         PROGRAMMER           DATE              COMMENTS
-------         ----------           ----              --------
  2.0           D. Hollaren          09/91             Original development

******************************************************************************/
#include "worgen.h"
#include "model.h"
#include "avhrr.h"

void c_ls2ll
(
    float *line,		/* Line in satellite scene		*/
    float *sample,		/* Sample in satellite scene		*/
    double *elev,		/* Elevation at the specified point     */
    struct MODEL *model,	/* Model and orbital information	*/
    double *lat,		/* Latitude of point		        */
    double *lng,		/* Longitude of point		        */
    double view_angles[]   /* Zenith and relative azimuth angles, when  */
			   /* view_angles == NULL no angles are calculated */
)
{
double	pline,		/* Actual line in the image		             */
	psamp;		/* Actual sample in the image		             */

/*
  Flip the line/sample coordinates for ascending passes in satellite
  view.  The model expects flipped (North up) coordinates.
*/
if ((model->cpass == ASCENDING) && (model->sat_view == TRUE))
    {
    pline = ((float)model->nlines - *line) + 1;
    psamp = ((float)model->nsamples - *sample) + 1;
    }
else
    {
    pline = *line;
    psamp = *sample;
    }

/*
  Calculate the latitude/longitude for the specified line/sample
*/
c_forward(&pline,&psamp,elev,model,lat,lng,view_angles);

return;
}
 
