/*******************************************************************************
NAME:		C_SETEPHEM

PURPOSE:	Convert the ascii ephemris information into a structure and
		fill other information into the ephemris structure needed for
		the model.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		REASON
-------		----	------		------
  2.0		5/91	D. Hollaren	Conversion to LAS5.0
  2.1           04/92   J.Fenno         Rewrote to use the CEOS_IEF header
					format.  Placed the EROS Archive Header
					in the free text portion of the CEOS_IEF
					header.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "worgen.h"
#include "util.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"
#include "local_protos.h"

int c_setephem
(
    char *eline,          /* ASCII ephemris line                */
    struct MODEL *model   /* Model and orbital information	*/
)
{

/*
  Parse the elements into the model structure
*/
sscanf(eline, "%05d%05d%8lf%8lf%5lf%8lf%8lf%8lf%8lf%02d%02d%02d",
       &model->satid, &model->orbit, &model->mean_anomaly, &model->mean_motion,
       &model->decay, &model->ecc, &model->arg_per, &model->lon_anode,
       &model->inclin, &model->epoch_year, &model->epoch_month,
       &model->epoch_day);

/*
  Calculate the julian date of the epoch date
*/
c_grejul(&model->epoch_year,&model->epoch_month,&model->epoch_day,
	 &model->epoch_jday);

/* Convert the elements that have an implied decimal point
----------------------------------------------------------*/
model->mean_anomaly = (model-> mean_anomaly / 100000000.0) * PIE2;
model->mean_motion  = model->mean_motion / 100000000.0;

if (model->decay > 500.0)
   model->decay = 0.0;

model->decay     = (model->decay     / 100000.0) * 1.0e-5;
model->ecc       = model->ecc        / 100000000.0;
model->arg_per   = (model->arg_per   / 100000000.0) * PIE2;
model->lon_anode = (model->lon_anode / 100000000.0) * PIE2;
model->inclin    = (model->inclin    / 100000000.0) * PIE2;

/* Calculate the element rates.  Other junk needed by the model.
----------------------------------------------------------------*/
elrates(model);

return(E_SUCC);
}
 
