/****************************************************************************
NAME:		CALCDMEANM

PURPOSE:	Calculate the delta mean motion.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

*****************************************************************************/
#include <math.h>
#include "worgen.h"
#include "vector.h"
#include "model.h"
#include "local_protos.h"

void calcdmeanm
(
    struct STATE *sat,		/* magnitude of the satellite vector	     */
    struct MVECTOR *los,	/* Line of sight from target to sat vector   */
    struct MVECTOR *dirvec,	/* Attitude correction vector information    */
    double *dma			/* Delta mean anomaly			     */
)
{
double	losmag,		/* Magnitude of the line of sight vector	     */
	angle,		/* Angle between LOS and direction of motion vecs    */
	ds;		/* Distance perpendicular to Z vector		     */

/*
  Calculate the magnitude of the line of sight vector
*/
losmag = norm(los);
      
/*
  Calculate the angle between the line of sight to the target and
  the attitude direction of motion vector
*/
anglebetwn(los,dirvec,&angle);

/*
  Distance of the perpendicular to the Z vector
*/
ds = losmag * cos(angle);

/*
  Calculate the delta mean motion.  This is the amount of correction needed
  for a point to actually fall in the scan plane.
*/
*dma = ds / sat->mag;

return;
}
 
