/****************************************************************************
NAME:		CALCRCOEFS

PURPOSE:	Calculate radius coefficients needed to fit the specified
		radii

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "tae.h"
#include "worgen.h"

void calcrcoefs
(
    double radii[],	/* Radius at start, 1/4, center, 3/4, and end of pass*/
    double *dtime,	/* Delta time					     */
    double alt_coefs[]	/* Altitude coefficients			     */
)
{
double	d1,		/* Delta time					     */
	d2,		/* Delta time squared				     */
	d3,		/* Delta time cubed				     */
	r0,
	rd0,
	a0,
	ad0,
	add0,
	a1,
	a2,
	a3,
	rd1,
	rd2,
	rd3,
	rd4,
	ad1,
	ad2,
	cr1,
	cr2,
	cr3,
	cr4,
	cr5;

long	nonfatal = 1;

d1 = *dtime;
d2 = d1 * d1;
d3 = d2 * d1;

rd1 = (radii[1] - radii[0]) / d1;
rd2 = (radii[2] - radii[1]) / d1;
rd3 = (radii[3] - radii[2]) / d1;
rd4 = (radii[4] - radii[3]) / d1;

a1 = (rd2 - rd1) / d1;
a2 = (rd3 - rd2) / d1;
a3 = (rd4 - rd3) / d1;

ad1 = (a2 - a1) / d1;
ad2 = (a3 - a2) / d1;

add0 = (ad2 - ad1) / d1;

ad0 = ad2 - add0 * d1 / 2.0;
a0  = a3 - add0 * d2 / 2.0 - ad0 * d1;

rd0 = rd4 - (a0 * 1.5 * d1 + ad0 / 2.0 * 1.5 * 1.5 * d2 +
	     add0 / 6.0 * 1.5 * 1.5 * 1.5 * d3);

r0  = radii[4] - (rd0 * 2.0 * d1 + a0 / 2.0 * 4.0 * d2 +
		 ad0 / 6.0 * 8.0 * d3 + add0 / 24.0 * 16.0 * d2 * d2);

alt_coefs[0] = r0;
alt_coefs[1] = rd0;
alt_coefs[2] = a0 / 2.0;
alt_coefs[3] = ad0 / 6.0;
alt_coefs[4] = add0 / 24.0;

cr1 = alt_coefs[0] + 
      alt_coefs[1] * (-2.0 * d1) + 
      alt_coefs[2] * (-2.0 * d1) * (-2.0 * d1) +
      alt_coefs[3] * (-2.0 * d1) * (-2.0 * d1) * (-2.0 * d1) +
      alt_coefs[4] * (-2.0 * d1) * (-2.0 * d1) * (-2.0 * d1) * (-2.0 * d1);

if (fabs(cr1 - radii[0]) >= 1000.0) 
    {
    c_errmsg("The difference between fit and radius 1 is larger than 1 KM",
	     "calcrcoef-warn",&nonfatal);
    }
cr2 = alt_coefs[0] + 
      alt_coefs[1] * (-1.0 * d1) + 
      alt_coefs[2] * (-1.0 * d1) * (-1.0 * d1) +
      alt_coefs[3] * (-1.0 * d1) * (-1.0 * d1) * (-1.0 * d1) +
      alt_coefs[4] * (-1.0 * d1) * (-1.0 * d1) * (-1.0 * d1) * (-1.0 * d1);

if (fabs(cr2 - radii[1]) >= 1000.0) 
    {
    c_errmsg("The difference between fit and radius 2 is larger than 1 KM",
	     "calcrcoef-warn",&nonfatal);
    }

cr3 = alt_coefs[0] + 
      alt_coefs[1] * (0.0 * d1) + 
      alt_coefs[2] * (0.0 * d1) * (0.0 * d1) +
      alt_coefs[3] * (0.0 * d1) * (0.0 * d1) * (0.0 * d1) +
      alt_coefs[4] * (0.0 * d1) * (0.0 * d1) * (0.0 * d1) * (0.0 * d1);

if (fabs(cr3 - radii[2]) >= 1000.0) 
    {
    c_errmsg("The difference between fit and radius 3 is larger than 1 KM",
	     "calcrcoef-warn",&nonfatal);
    }

cr4 = alt_coefs[0] + 
      alt_coefs[1] * (1.0 * d1) + 
      alt_coefs[2] * (1.0 * d1) * (1.0 * d1) +
      alt_coefs[3] * (1.0 * d1) * (1.0 * d1) * (1.0 * d1) +
      alt_coefs[4] * (1.0 * d1) * (1.0 * d1) * (1.0 * d1) * (1.0 * d1);

if (fabs(cr4 - radii[3]) >= 1000.0) 
    {
    c_errmsg("The difference between fit and radius 4 is larger than 1 KM",
	     "calcrcoef-warn",&nonfatal);
    }

cr5 = alt_coefs[0] + 
      alt_coefs[1] * (2.0 * d1) + 
      alt_coefs[2] * (2.0 * d1) * (2.0 * d1) +
      alt_coefs[3] * (2.0 * d1) * (2.0 * d1) * (2.0 * d1) +
      alt_coefs[4] * (2.0 * d1) * (2.0 * d1) * (2.0 * d1) * (2.0 * d1);

if (fabs(cr5 - radii[4]) >= 1000.0)
    {
    c_errmsg("The difference between fit and radius 5 is larger than 1 KM",
	     "rcoef-warn",&nonfatal);
    }

return;
}
 
