/****************************************************************************
NAME:		CALCSAMPLE

PURPOSE:	Calculate the sample location

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "avhrr.h"
#include "constants.h"
#include "model.h"
#include "local_protos.h"

void calcsample
(
    struct MVECTOR *losvec, /* Line of sight vector to target		     */
    struct MVECTOR *angvec, /* Atttude angular momentum vector		     */
    double *offsample,	    /* Window sample offset			     */
    long *dtype,	    /* Type of data (HRPT, LAC, or GAC)		     */
    long *zdot,		    /* Direction of the pass (-1) Asc (1) Des	     */
    double *sample	    /* Sample at the target			     */
)
{
double	angle,		 /* Angle between LOS & angular momentum vecs	     */
	scan_angle;	 /* Satellite scan angle to target		     */

/*
  Calculate the angle between the line of sight to the target and
  the angular momentum vector
*/
anglebetwn(losvec,angvec,&angle);

/*
  Calculate the scan angle
*/
scan_angle = (HALFPIE - angle) * -*zdot;

/*
  Calculate the sample of the target
*/
if (*dtype == GAC)
    *sample = *offsample + GAC_NADIR + GAC_SAMPLE_SLOPE * scan_angle;
else
    *sample = *offsample + LAC_NADIR + LAC_SAMPLE_SLOPE * scan_angle;

return;
}
 
