/****************************************************************************
NAME:		CALCVIEWS

PURPOSE:	Calculate the viewing angles.  Satellite zenith, solor
		zenith, and relative azimuth (the absolute differance
		of the sun and satellite azimuth angles) angles.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "avhrr.h"
#include "model.h"
#include "constants.h"
#include "local_protos.h"

void calcviews
(
    struct MODEL *model,	/* Model and orbital information	     */
    double *tstime,		/* Total seconds to the start time	     */
    struct MVECTOR *losv,	/* Line of sight vector			     */
    double *tarlatd,		/* Target latitude			     */
    double *tarlong,		/* Target longitude			     */
    struct MVECTOR *tarvec,	/* Target location vector		     */
    double view_angles[]	/* View angles (sun & sat zenith, rel az)    */
)
{
double	sunlat,		/* Latitude of the sun at the given date and time    */
	sunlong,	/* Longtitude of the sun at the given date and time  */
	angle,		/* Angle between the LOS and target		     */
	truea,		/* True anomally				     */
	rsat,		/* Radius of the satellite 			     */
	thetas,		/* */
	anlng,		/* */
	inclin,		/* Inclination angle				     */
	satlat,		/* Satellite latitude				     */
	satlong;	/* Satellite longitude				     */

long	zdot,		/* Direction of the pass (-1) Asc (1) Desc	     */
	eorbit,		/* Orbit at the equator				     */
	porbit;		/* Orbit at perigee				     */

/*
  Calculate the angle between the line of sight vector and the target vector
*/
anglebetwn(losv,tarvec,&angle);

/*
  Get the satellite zenith angle
*/
view_angles[0] = PIE - angle;

/*
  Calculate the suns position for the date and time
*/
c_sunpos(&model->epoch_jday,&model->epoch_year,tstime,&sunlat,&sunlong);

/*
  Calculate the sun zenith by creating the great circle from the sun
  latitude/longitude and the target latitude/longitude
*/
greatcircle(&sunlat,&sunlong,tarlatd,tarlong,&view_angles[1]);

/*
  Find the satellites position
*/
brouwer(tstime,model,&truea,&rsat,&thetas,&anlng,&inclin,&zdot,
	&eorbit,&porbit,&satlat,&satlong);

/*
  Calculate the relative sun and satellite azimuth angle
*/
relazimuth(tarlatd,tarlong,&satlat,&satlong,&sunlat,&sunlong,&view_angles[2]);

return;
}
 
