/****************************************************************************
NAME:		CART2SPH

PURPOSE:	Convert catesian coordinates to spherical coordinates


PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <math.h>
#include "worgen.h"
#include "constants.h"
#include "vector.h"

void cart2sph
(
    struct MVECTOR *vec,	/* Vector containing the cartesian coordinates*/
    double *lat,		/* Latitude  in spherical coordinates	     */
    double *longs,		/* Longitude in spherical coordinates	     */
    double *radius		/* Distance from the origin to the point     */
)
{
/*
  Calculate the radius (length of the vector from the origin)
*/
*radius = sqrt(vec->x * vec->x + vec->y * vec->y + vec->z * vec->z);

/*
  Calculate the latitude spherical coordinate
*/
*lat = asin(vec->z / *radius);

/*
  Calculate the longitude spherical coordinate.  The longitude depends on
  the quadrant that the vector lies in.
*/
if (vec->x > 0.0 && vec->y >= 0.0 )
    {
    *longs = atan(vec->y / vec->x);
    }
else if (vec->x < 0.0 && vec->y >= 0.0)
    {
    *longs = PIE + (atan(vec->y / vec->x));
    }
else if (vec->x < 0.0 && vec->y < 0.0)
    {
    *longs = PIE + (atan(vec->y / vec->x));
    }
else if (vec->x > 0.0 && vec->y < 0.0)
    {
    *longs = PIE2 + (atan(vec->y / vec->x));
    }
else if (vec->x == 0.0 && vec->y < 0.0)
    {
    *longs = 3.0 * PIE / 4.0;
    }
else if (vec->x == 0.0 && vec->y > 0.0)
    {
    *longs = PIE / 2.0;
    }

return;
}
 
