/****************************************************************************
NAME:		CHKSAMP

PURPOSE:	Check that the model isn't looking through the earth

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "model.h"
#include "avhrr.h"
#include "constants.h"
#include "local_protos.h"

void chksamp
(
    struct MVECTOR *sat,    /* Satellite location vector		     */
    struct MVECTOR *tar,    /* Target location vector			     */
    struct MVECTOR *losvec, /* Line of sight vector to target		     */
    struct MVECTOR *angvec, /* Angular angular momentum vector		     */
    double *offsample,	    /* Window sample offset			     */
    long *dtype,	    /* Type of data (HRPT, LAC, or GAC)		     */
    long *zdot,		    /* Direction of the pass (-1) Asc (1) Des	     */
    double *sample	    /* Output sample in scene			     */
)
{
double	satlat,		/* Satellite latitude				     */
	satlong,	/* Satellite longitude				     */
	tarlat,		/* Target latitude				     */
	tarlong,	/* Target longitude				     */
	sradius,	/* Radius of the satellite point		     */
	tradius,	/* Radius of the target point			     */
	angle,          /* Angle between LOS & angular momentum vectors      */
	scan_angle,	/* Satellite scan angle to target		     */
	max_gc_angle,	/* Maximum great circle angle (earth central angle)  */
	max_tn_angle,	/* Maximum angle to the earth tanget (sat to target) */
	new_angle,	/* New angle between max and calculated scan angle   */
	gc;		/* Great circle from target to satellite	     */

/*
  Find the latitude/longitude position of the satellite
*/
cart2sph(sat,&satlat,&satlong,&sradius);

/*
  Find the latitude/longitude position of the target
*/
cart2sph(tar,&tarlat,&tarlong,&tradius);

/*
  Calculate the great circle from the satellite to the target
*/
greatcircle(&satlat,&satlong,&tarlat,&tarlong,&gc);

max_gc_angle = acos(tradius / sradius);

/*
  Determine if the great circle is greater than the angle between the satellite
  and the line tangent to the earth
*/
if (gc > max_gc_angle)
    {
    /*
      Calculate the angle between the line of sight to the target and
      the angular momentum vector
    */
    anglebetwn(losvec,angvec,&angle);

    /*
      Calculate the scan angle
    */
    scan_angle = (HALFPIE - angle) * -*zdot;

    /*
      Add a fudge to get past the tangent line to the earth
    */
    if (scan_angle < 0.0)
	scan_angle += .005;
    else
	scan_angle += -.005;

    /*
      Calculate the maximum angle to the tangent
    */
    max_tn_angle = asin(tradius / sradius);

    /*
      The new angle is the maximum tangent angle plus a delta angle
    */
    new_angle = max_tn_angle + fabs(max_tn_angle - fabs(scan_angle));
    if (scan_angle < 0.0)
	new_angle *= -1;

    /*
      Calculate the new sample of the target
    */
    if (*dtype == GAC)
        *sample = *offsample + GAC_NADIR + GAC_SAMPLE_SLOPE * new_angle;
    else
        *sample = *offsample + LAC_NADIR + LAC_SAMPLE_SLOPE * new_angle;
    }

return;
}
 
