/****************************************************************************
NAME:		CIRCVEL

PURPOSE:	Circle velocity


PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "constants.h"
#include "vector.h"
#include "model.h"
#include "local_protos.h"

double circvel
(
    struct STATE *sat	/* Satellite state vector			     */
)
{
struct MVECTOR amovec;	/* Angular momentum vector			     */
struct MVECTOR cirvec;	/* Circular velocity vector			     */

double	incl,		/* Orbital inclination angle			     */
	vcmag;		/* Circular vector magnitude			     */

/*
  Cross product of the satellite target and satellite velocity to get the
  angular momentum vector
*/
cross(&sat->target,&sat->vel,&amovec);

/*
  Calculate the orbital inclination angle
*/
incl = acos(amovec.z / norm(&amovec));

/*
  Cross product of the angular momentum and the satellite location
*/
cross(&amovec,&sat->target,&cirvec);

/*
  Calculate the norm to the circle velocity vector
*/
vcmag = norm(&cirvec);

/*
  Adjust the satellite velocity vector
*/
sat->vel.x = cirvec.x / vcmag * sat->vmag;
sat->vel.y = cirvec.y / vcmag * sat->vmag;
sat->vel.z = cirvec.z / vcmag * sat->vmag;

/*
  Return the orbital inclination angle
*/
return(incl);
}
 
