/****************************************************************************
NAME:		DET2CENTH

PURPOSE:	Calculate the geocentric latitude and the radius of a point
		above the ellipse, given the geodetic latitude and the height.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "ellipsoid.h"

void det2centh
(
    double *latd,	/* Geodetic latitude				     */
    double *height,     /* height in kms (elevation)			     */
    double *latc,	/* Geocentric latitude				     */
    double *radius	/* Radius of the satellite at the point		     */
)
{
double	sinlt,		/* Sine of the geodetic latitude		     */
	w,
	g1,
	g2,
	xc,
	zc;

/*
  Calculate the sine of the latitude
*/
sinlt = sin(*latd);

w = sqrt(1.0 - EE * sinlt * sinlt);
g1 = REQUA / w + *height;
g2 = (REQUA * Q) / w + *height;

xc = g1 * cos(*latd);
zc = g2 * sinlt;

/*
  Calculate the radius of the point
*/
*radius = sqrt(xc * xc + zc * zc);

/*
  Calculate the geocentric latitude of the point
*/
*latc = asin(zc / *radius);

return;
}
 
