/****************************************************************************
NAME:		INTTAR

PURPOSE:	Calculate starting location of target in cartesian coordinates

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "constants.h"
#include "vector.h"
#include "model.h"
#include "adapsutil.h"
#include "local_protos.h"

void inttar
(
    double *tarlatd,		/* Target latitude  (geodetic)  */
    double *tarlong,		/* Target longitude (geodetic)  */
    double *tarelev,		/* Target elevation	     	*/
    struct STATE *tarint	/* Target vector 		*/
)
{
double	latp,		/* Latitude of the point (geocentric)		     */
	radius,		/* Radius of the satellite at the point		     */
	rcoslt;		/* Radius at point times cosine of the latitude	     */

/*
  Calculate the geocentric latitude and radius for the specified latitude
  and elevation.
*/
det2centh(tarlatd,tarelev,&latp,&radius);

/*
  Constant calculation
*/
rcoslt = radius * cos(latp);

/*
  Calculate the initial target information in cartesian coordinates
*/
tarint->target.x = rcoslt * cos(*tarlong);
tarint->target.y = rcoslt * sin(*tarlong);
tarint->target.z = radius * sin(latp);
tarint->mag = radius;
tarint->vel.x = tarint->target.y * -1.0 * WE;
tarint->vel.y = tarint->target.x * WE;
tarint->vel.z = 0.0;
tarint->vmag = rcoslt * WE;

return;
}
 
