#ifndef LOCAL_PROTOS_H
#define LOCAL_PROTOS_H


/* Prototype routines that are not to be called outside of this directory     */

void anglebetwn
(
    struct MVECTOR *vec1,	/* Input vector one			     */
    struct MVECTOR *vec2,	/* Input vector two			     */
    double *angle		/* Angle between the specified vectors	     */
);

void attitude
(
    struct STATE *sat,	 	/* Satellite velocity vector		     */
    struct MVECTOR *nadvec, 	/* Vector from satellite to nadir point	     */
    double *roll,		/* Roll correction			     */
    double *pitch,		/* Pitch correction			     */
    double *yaw,		/* Yaw correction			     */
    struct ATTITUDE *att    	/* Attitude correction vectors information   */
);

void blatlong
(
    double *thetas,		/* True anomaly passed equator		*/
    double *anlng,		/* Lonitude of ascending node		*/
    double *inclin,		/* Inclination angle			*/
    double *slat,		/* Geocentric latitude of satellite	*/
    double *slong		/* Longitude of satellite		*/
);

void brouwcorrect
(
    double dpele[],	/* Mean elements		*/
    double oselem[]	/* Oscillating elements		*/
);

void calcdmeanm
(
    struct STATE *sat,		/* magnitude of the satellite vector	     */
    struct MVECTOR *los,	/* Line of sight from target to sat vector   */
    struct MVECTOR *dirvec,	/* Attitude correction vector information    */
    double *dma			/* Delta mean anomaly			     */
);

void calcdtime
(
    double *dma,		/* Delta mean anomaly		     */
    double *mdot1,		/* Mean motion before center	     */
    double *mdot2,		/* Mean motion after center	     */
    double *dtime		/* Delta time			     */
);

void calcline
(
    double *Time,	/* Time to calculate the line at 	     */
    double *offline,	/* Window offset			     */
    double *midline,	/* Line at the middle the of pass	     */
    long *dtype,	/* Type of data (HRPT, LAC, GAC)	     */
    long *zdot,		/* Direction of the pass (-1) Asc (1) Desc   */
    double *line	/* Calculated line			     */
);

void calcpitch
(
    double *Time,		/* Time to calculate the pitch at	    */
    double pitch_coefs[],	/* Pitch coefficients			    */
    double *pitch		/* Pitch at the specified time		    */
);

void calcrcoefs
(
    double radii[],	/* Radius at start, 1/4, center, 3/4, and end of pass*/
    double *dtime,	/* Delta time					     */
    double alt_coefs[]	/* Altitude coefficients			     */
);

void calcroll
(
    double *Time,		/* Time to calculate the roll for	    */
    double roll_coefs[],	/* Roll coefficients			    */
    double *roll		/* Roll at the specified time		    */
);

void calcsample
(
    struct MVECTOR *losvec, /* Line of sight vector to target		     */
    struct MVECTOR *angvec, /* Atttude angular momentum vector		     */
    double *offsample,	    /* Window sample offset			     */
    long *dtype,	    /* Type of data (HRPT, LAC, or GAC)		     */
    long *zdot,		    /* Direction of the pass (-1) Asc (1) Des	     */
    double *sample	    /* Sample at the target			     */
);

void calcviews
(
    struct MODEL *model,	/* Model and orbital information	     */
    double *tstime,		/* Total seconds to the start time	     */
    struct MVECTOR *losv,	/* Line of sight vector			     */
    double *tarlatd,		/* Target latitude			     */
    double *tarlong,		/* Target longitude			     */
    struct MVECTOR *tarvec,	/* Target location vector		     */
    double view_angles[]	/* View angles (sun & sat zenith, rel az)    */
);

void calcyaw
(
    double *Time,	/* Time to calculate the yaw at			     */
    double yaw_coefs[],	/* Coefficients for yaw calculations		     */
    double *yaw		/* Yaw at the specified time			     */
);

void cart2sph
(
    struct MVECTOR *vec,	/* Vector containing the cartesian coordinates*/
    double *lat,		/* Latitude  in spherical coordinates	     */
    double *longs,		/* Longitude in spherical coordinates	     */
    double *radius		/* Distance from the origin to the point     */
);

void centh2det
(
    double *latc,	/* Geocentric latitude			*/
    double *radius,	/* Radius of the point			*/
    double *latd,	/* Geodetic latitude			*/
    double *height	/* Height of point			*/
);

void chksamp
(
    struct MVECTOR *sat,    /* Satellite location vector		     */
    struct MVECTOR *tar,    /* Target location vector			     */
    struct MVECTOR *losvec, /* Line of sight vector to target		     */
    struct MVECTOR *angvec, /* Angular angular momentum vector		     */
    double *offsample,	    /* Window sample offset			     */
    long *dtype,	    /* Type of data (HRPT, LAC, or GAC)		     */
    long *zdot,		    /* Direction of the pass (-1) Asc (1) Des	     */
    double *sample	    /* Output sample in scene			     */
);

double circvel
(
    struct STATE *sat	/* Satellite state vector			     */
);

void cross
(
    struct MVECTOR *vec1,	/* Input vector one			     */
    struct MVECTOR *vec2,	/* Input vector two			     */
    struct MVECTOR *vec3	/* Output vector cross product		     */
);

void det2centh
(
    double *latd,	/* Geodetic latitude				     */
    double *height,     /* height in kms (elevation)			     */
    double *latc,	/* Geocentric latitude				     */
    double *radius	/* Radius of the satellite at the point		     */
);

void dkeplr
(
    double m,		/* Mean anomaly			*/
    double e,		/* Eccentricity			*/
    double *ea		/* True anomaly			*/
);

double dot
(
    struct MVECTOR *vec1,	/* Input vector one to be multiplied    */
    struct MVECTOR *vec2	/* Input vector two to be multiplied    */
);

void earthradius
(
    double latc,	/* Latitude to find the radius at		     */
    double *radius	/* Radius of the earth at the specified latitude     */
);

void elrates
(
    struct MODEL *model	  /* Model and orbital information	*/
);

void findlos
(
    struct ATTITUDE *att,/* Attitude correction vectors information  */
    double *scan_angle,	 /* Scan angle				     */
    struct MVECTOR *losv /* Line of sight vector		     */
);

void findsample
(
    double scan_angle,	/* Scan angle to the sample			     */
    double offsample,	/* Window sample offset				     */
    long dtype,		/* Type of data (HRPT, LAC, or GAC)		     */
    long zdot,		/* Direction of the pass (-1) Asc (1) Desc	     */
    double *sample	/* Output sample				     */
);

void findscanangle
(
    double *sample,	/* Input sample					     */
    double *offsample,	/* Window sample offset				     */
    long *dtype,	/* Type of data (HRPT, LAC, or GAC)		     */
    long *zdot,		/* Direction of the pass (-1) Asc (1) Desc	     */
    double *scan_angle	/* Output scan angle for the sample		     */
);

void findtarvec
(
    struct MVECTOR *satvec,	/* Satellite location vector		     */
    struct MVECTOR *losv,	/* Line of sight to target vector	     */
    struct MVECTOR *tarvec,	/* Target vector			     */
    double *tarlatc,	/* Target latitude				     */
    double *tarlong,	/* Target longitude				     */
    double *tarrad	/* Radius of the target				     */
);

void findtime
(
    double *line,	/* Line to calculate time for			     */
    double *offline,	/* Offset line for windowing			     */
    double *midline,	/* line at middle of the pass			     */
    long *dtype,	/* Type of data (GAC, LAC, or HRPT)		     */
    long *zdot,		/* Direction of the pass at the center		     */
    double *Time	/* Time of the specified line			     */
);

void greatcircle
(
    double *lat1, 	/* Latitude  of first point  (rads)     */
    double *long1,	/* Longitude of first point  (rads)     */
    double *lat2, 	/* Latitude  of second point (rads)     */
    double *long2,	/* Longitude of second point (rads)     */
    double *eca 	/* Earth Centeral Angle	     (rads)     */
);

void inttar
(
    double *tarlatd,		/* Target latitude  (geodetic)  */
    double *tarlong,		/* Target longitude (geodetic)  */
    double *tarelev,		/* Target elevation	     	*/
    struct STATE *tarint	/* Target vector 		*/
);

void losvec
(
    struct MVECTOR *sat,	/* Satellite location vector		     */
    struct MVECTOR *tar,	/* Target location vector		     */
    struct MVECTOR *los	/* Line of sight from target to satellite vector     */
);

void movesat
(
    struct STATE *satint,	/* Initial satellite location	    */
    double alt_coefs[],		/* Change of altitude for satellite */
    double *mdot1,		/* Mean motion before center	    */
    double *mdot2,		/* Mean motion after center	    */
    double *dtime,		/* Delta time to move the satellite */
    struct STATE *sat		/* New satellite location at dtime  */
);

void movetar
(
    struct STATE *tarint,	/* Initial target location		     */
    double *dtime,		/* Delta time to move the target location    */
    struct STATE *tar	/* New target location dtime later		     */
);

void nadirvec
(
    struct MVECTOR *satvec, /* Satellite vector				     */
    struct MVECTOR *nadvec  /* Vector from the satellite to the nadir point  */
);

void newvec
(
    struct MVECTOR *vec1,	/* Input vector one			     */
    struct MVECTOR *vec2,	/* Input vector two			     */
    long overhalf, 	/* Flag for over 1/2 orbit                           */
    double *new_angle,	/* Angle between the first vector and the new vector */
    double *new_mag,	/* Magnitude of the new vector			     */
    struct MVECTOR *vec3	/* New Output vector			     */
);

double norm
(
    struct MVECTOR *vec	/* Input vector to fine the length of (NORM)	    */
);

void relazimuth
(
    double *tarlatd,	/* Target latitude				     */
    double *tarlong,	/* Target longitude				     */
    double *satlat,	/* Satellite latitude				     */
    double *satlong,	/* Satellite longitude				     */
    double *sunlat,	/* Sun latitude					     */
    double *sunlong,	/* Sun longitude				     */
    double *relaz	/* Relative azimuth value			     */
);

int satatt
(
    long *satnum,	/* Satellite number				*/
    double *roll,	/* Satellite roll correction			*/
    double *pitch,	/* Satellite pitch correction			*/
    double *yaw,	/* Satellite yaw correction			*/
    double *time	/* Satellite time correction			*/
);

void secsincepoch
(
    struct MODEL *model	/* Model and orbital information		*/
);

void sph2cart
(
    double *latp,	/* Latitude					     */
    double *longp,	/* Longitude					     */
    double *radius,	/* Distance from the origin to the point	     */
    struct MVECTOR *vec	/* Vector containing the cartesian coordinates	     */
);

void unit
(
    struct MVECTOR *vec,/* Input vector					     */
    struct MVECTOR *uni	/* Unit vector of the input vector		     */
);

int wind_bands
(
    struct MODEL *model,	/* IN     - AVHRR model information           */
    struct MODEL *window_model,	/* OUT    - AVHRR model of windowed info      */
    long bands[]		/* IN     - Band spec for the output image    */
);

#endif

