/****************************************************************************
NAME:		MOVESAT

PURPOSE:	Move the satellite to a new position DTIME later


PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "constants.h"
#include "vector.h"

void movesat
(
    struct STATE *satint,	/* Initial satellite location	    */
    double alt_coefs[],		/* Change of altitude for satellite */
    double *mdot1,		/* Mean motion before center	    */
    double *mdot2,		/* Mean motion after center	    */
    double *dtime,		/* Delta time to move the satellite */
    struct STATE *sat		/* New satellite location at dtime  */
)
{
double	dm,		/* Delta mean motion				     */
	cosdm,		/* Cosine of the delta mean motion		     */
	sindm,		/* Sine of the delta mean motion		     */
	radius,		/* Radius of the satellite at new delta time	     */
	velocity,	/* Magnitude of the velocity at new delta time       */
	const1,		/* Constant for calculations			     */
	const2,		/* Constant for calculations			     */
	const3,		/* Constant for calculations			     */
	const4;		/* Constant for calculations			     */

/*
  Set the delta mean motion
*/
if (*dtime <= 0.0)
    dm = *mdot1 * *dtime;
else
    dm = *mdot2 * *dtime;

cosdm = cos(dm);
sindm = sin(dm);

/*
  Calculate the radius of the satellite given the change of radius at dtime
*/
radius = alt_coefs[0] + 
	 alt_coefs[1] * *dtime +
	 alt_coefs[2] * *dtime * *dtime +
	 alt_coefs[3] * *dtime * *dtime * *dtime + 
	 alt_coefs[4] * *dtime * *dtime * *dtime * *dtime;

/*
  Calculate the velocity
*/
velocity = sqrt(AMU / radius);

const1 = radius   / satint->mag  * cosdm;
const2 = radius   / satint->vmag * sindm;
const3 = velocity / satint->vmag * cosdm;
const4 = velocity / satint->mag  * sindm;

/*
  Calculate new position and velocity vectors
*/
sat->target.x = satint->target.x * const1 + satint->vel.x * const2;
sat->target.y = satint->target.y * const1 + satint->vel.y * const2;
sat->target.z = satint->target.z * const1 + satint->vel.z * const2;
sat->mag = radius;
sat->vel.x = const3 * satint->vel.x - const4 * satint->target.x;
sat->vel.y = const3 * satint->vel.y - const4 * satint->target.y;
sat->vel.z = const3 * satint->vel.z - const4 * satint->target.z;
sat->vmag = velocity;

return;
}
 
