/****************************************************************************
NAME:		MOVETAR

PURPOSE:	Move the targets position to new position DTIME later

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "constants.h"
#include "vector.h"

void movetar
(
    struct STATE *tarint,	/* Initial target location		     */
    double *dtime,		/* Delta time to move the target location    */
    struct STATE *tar	/* New target location dtime later		     */
)
{
double	dl,		/* Rotation of the earth for delta time		    */
	cosdl,		/* Cosine of dl					    */
	sindl,		/* Sine of dl					    */
	sindlperwe,
	wesindl;

/*
  Calculate the distance the earth rotated for delta time
*/
dl = WE * *dtime;
sindl = sin(dl);
cosdl = cos(dl);

wesindl = WE * sindl;
sindlperwe = sindl / WE;

/*
  Calculate the new target location
*/
tar->target.x = cosdl * tarint->target.x + sindlperwe * tarint->vel.x;
tar->target.y = cosdl * tarint->target.y + sindlperwe * tarint->vel.y;
tar->target.z = tarint->target.z;
tar->mag      = tarint->mag;

tar->vel.x = cosdl * tarint->vel.x - wesindl * tarint->target.x;
tar->vel.y = cosdl * tarint->vel.y - wesindl * tarint->target.y;
tar->vel.z = tarint->vel.z;
tar->vmag  = tarint->vmag;

return;
}
 
