/****************************************************************************
NAME:		NADIRVEC

PURPOSE:	Calculate the vector from the spacecraft to the tangent
		of the ellipsoid.

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "vector.h"
#include "model.h"
#include "local_protos.h"

void nadirvec
(
    struct MVECTOR *satvec, /* Satellite vector				     */
    struct MVECTOR *nadvec  /* Vector from the satellite to the nadir point  */
)
{
double	latc,		/* Latitude  (geocentric)			     */
	latd,		/* Latitude  (geodetic)				     */
	longd,		/* Longitude (geodetic)				     */
	radius,		/* Radius of the satellite			     */
	height,		/* Height of the satellite above the ellipsoid	     */
	hcoslatd;	/* Constant for calculations			     */
          
/*
  Convert the satellite cartesian coordinates into spherical coordinates
*/
cart2sph(satvec,&latc,&longd,&radius);

/*
  Convert the geocentric latitude to a geodetic latitude and get the
  height of the satellite.
*/
centh2det(&latc,&radius,&latd,&height);

/*
  Constant for calculation to save time
*/
hcoslatd = height * cos(latd);

/*
  Calculate the nadir vector
*/
nadvec->x = -hcoslatd * cos(longd);
nadvec->y = -hcoslatd * sin(longd);
nadvec->z = -height * sin(latd);

return;
}
 
