/****************************************************************************
NAME:		NEWVEC

PURPOSE:	Calculate a new vector in the same plane defined by the
		two input vectors given the angle from the first vector
		to the new vector and the magnitude of the new vector

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  3.0    C. Wivell     11/92	Original Development

ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "worgen.h"
#include "vector.h"
#include "constants.h"
#include "model.h"
#include "local_protos.h"

void newvec
(
    struct MVECTOR *vec1,	/* Input vector one			     */
    struct MVECTOR *vec2,	/* Input vector two			     */
    long overhalf, 	/* Flag for over 1/2 orbit                           */
    double *new_angle,	/* Angle between the first vector and the new vector */
    double *new_mag,	/* Magnitude of the new vector			     */
    struct MVECTOR *vec3	/* New Output vector			     */
)
{
double	angle12,	/* Angle between the two input vectors		     */
	v1_mag,		/* Magnitude of input vector one		     */
	v2_mag;		/* Magnitude of input vector two		     */

double	sinang,		/* Sine of the new angle			     */
	cotang12,	/* cotangent of the angle between the two vectors    */
	const1,		/* Length of opposite side one			     */
	const2;		/* Length of opposite side two			     */

/*
  Find the angle between the two input vectors
*/
anglebetwn(vec1,vec2,&angle12);
if (overhalf)
    angle12 = PIE2 - angle12;

/*
  Find the magnitude of the two input vectors
*/
v1_mag = norm(vec1);
v2_mag = norm(vec2);

/*
  Calculate constants for equations
*/
if (angle12 == HALFPIE)
    cotang12 = 0.0;
else
    cotang12 = 1.0 / tan(angle12);
sinang = sin(*new_angle);

/*
  Calculate the length of the opposite sides
*/
const1 = (*new_mag / v1_mag) * (cos(*new_angle) - cotang12 * sinang);
const2 = (*new_mag * sinang) / (v2_mag * sin(angle12));

/*
  Calculate the new vector in the same plane as the two input vectors
*/
vec3->x = const1 * vec1->x + const2 * vec2->x;
vec3->y = const1 * vec1->y + const2 * vec2->y;
vec3->z = const1 * vec1->z + const2 * vec2->z;

return;
}
 
