/*******************************************************************************
NAME:		SATATT

PURPOSE:	Get the satellite attitude corrections

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		8/91	D. Hollaren	   CSB      Original development
  2.1		9/94	T. Baltzer	   CSB	    Rename BUFSIZ to BUFSIZE to
						    avoid system name conflict.

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "worgen.h"
#include "tae.h"

#define BUFSIZE	62

int satatt
(
    long *satnum,	/* Satellite number				*/
    double *roll,	/* Satellite roll correction			*/
    double *pitch,	/* Satellite pitch correction			*/
    double *yaw,	/* Satellite yaw correction			*/
    double *time	/* Satellite time correction			*/
)
{
FILE	*fp;		/* File pointer to attitude corrections file	*/

char	disk[CMLEN],	/* Disk location of ephemeris tables		*/
	satfile[CMLEN],	/* Ephemeris file				*/
	buffer[BUFSIZE],/* Buffer to read satellite attitude file       */
	*gstatus;	/* Status from fgets routine			*/

long	found = FALSE,	/* Flag for found satellite record		*/
	sat_num,	/* Satellite number				*/
	status,		/* Return status				*/
	nonfatal = 1;	/* Non-fatal error code				*/

/*
  Get the satellite attitude corrections file location
*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(disk,getenv("ADAPSTABLES"));
else
    {
    c_errmsg("Error retrieving value of ADAPSTABLES","satatt-find",&nonfatal);
    return(E_FAIL);
    }

#ifdef VMS
    sprintf(satfile,"%ssatellite.att",disk);
#else
    sprintf(satfile,"%s/satellite.att",disk);
#endif

/*
  Open the satellite attitude corrections file 
*/
fp = fopen(satfile,"r");
if (fp == NULL)
    {
    c_errmsg("Error opening the attitude corrections file","satatt-open",
	     &nonfatal);
    return(E_FAIL);
    }

/*
  Skip over the header records
*/
gstatus = fgets(buffer,BUFSIZE,fp);
if (gstatus == NULL)
    {
    fclose(fp);
    c_errmsg("Error skipping the satellite attitude correction header",
	     "satatt-skip",&nonfatal);
    return(E_FAIL);
    }

status = EOF + 1;	/* initialize status to some non-EOF value */
while (status != EOF)
    {
    gstatus = fgets(buffer,BUFSIZE,fp);
    if (gstatus == NULL)
        {
        if (feof(fp) != 0)
            {
            status = EOF;
            continue;
            }
        else
	    {
	    fclose(fp);
	    c_errmsg("Error reading attitude corrections file","satatt-read",
		     &nonfatal);
	    return(E_FAIL);
	    }
        }

    status = sscanf(buffer,"%d%lf%lf%lf%lf",&sat_num,roll,pitch,yaw,time);

    if (*satnum == sat_num)
	{
	found = TRUE;
	break;
	}
    }

fclose(fp);

if (!found)
    {
    c_errmsg("Invalid satellite number","satatt-satnum",&nonfatal);
    *roll = 0.0;
    *pitch = 0.0;
    *yaw = 0.0;
    *time = 0.0;
    return(E_FAIL);
    }

return(E_SUCC);
}
