/*****************************************************************************
NAME:		SECSINCEPOCH

PURPOSE:	Calculate to total seconds since the latest epoch ephemeris
		to the time of interest.

PROGRAM HISTORY:

VERSION         PROGRAMMER           DATE       COMMENTS
-------         ----------           ----       --------
 2.0            C.  WIVELL           2/91	Original develompent

ALGORITHM REFERENCES:
*******************************************************************************/
#include "worgen.h"
#include "avhrr.h"
#include "model.h"
#include "adapsutil.h"

void secsincepoch
(
    struct MODEL *model	/* Model and orbital information		*/
)
{
long	isub;		/* Number days to subtract if crossing years	*/

/*
  If the times cross a year check to see if it was a leap year then 
  subtract off the right number of days
*/
if (model->epoch_year == model->year) 
    isub = 0;
else
    { 
    if (c_isleap(&model->epoch_year))
	isub = 366;
    else
	isub = 365;
    }

/*
  Calculate the total seconds
*/
model->sec_epoch = (model->jday - model->epoch_jday + isub) * SEC_PER_DAY +
		   (model->shour * 3600.0) + (model->smin * 60.0) +
		    model->ssec + (model->sfrac / 1000.0);

return;
}
 
