/*******************************************************************************
NAME:		C_WINDMOD	

PURPOSE:	Given an AVHRR model for an entire image and a bands window,
		update the windowed model to reflect the specified bands window.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    10/97  T. Baltzer       Development
				
PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h> 
#include "las.h"
#include "adaps.h"



int wind_bands
(
    struct MODEL *model,	/* IN     - AVHRR model information           */
    struct MODEL *window_model,	/* OUT    - AVHRR model of windowed info      */
    long bands[]		/* IN     - Band spec for the output image    */
)
{


long i;				/* Loop counter                               */
long nbands;			/* Number of bands per input image	      */


/* Window the bands in the new model as specified by the user.  Make sure that
   the bands requested are present.
   SPECIAL CASE #1:
   If the relationship between the bands and the AVHRR channels is unknown
   (model.chans[0 - 4] == 0 but model.nchans != 0), return all bands regardless
   of what the user asked for.
   SPECIAL CASE #2:
   If the relationship between the bands and the AVHRR channels is unknown
   but all channels are present
   (model.chans[0 - 4] == 0 but model.nchans == 5) then return what the user
   asked for (assume the 5 bands are AVHRR channels 1-5 in sequence).
-----------------------------------------------------------------------------*/
for (i = 0, nbands = 0; i < NCHANS; i++)
    if (model->chans[i] != 0)
	nbands++;
if ((nbands == 0) && (model->nchans != 0))
   {
   if (model->nchans != NCHANS)		/* SPECIAL CASE #1 */
	{
	c_lasmsg(FALSE,"c_windmod-unk",
	         "Relationship between bands and AVHRR channels unknown");
	c_lasmsg(FALSE,"c_windmod-unk","All %d bands will be returned",
		 model->nchans);
	for (i = 0; i < NCHANS; i++)
	    window_model->chans[i] = 0;
	window_model->nchans = model->nchans;
	}
    else				/* SPECIAL CASE #2 */
	{
	for (i = 0; i < NCHANS; i++)
	    window_model->chans[i] = 0;
	for (i = 0; i < NCHANS; i++)
	    if (bands[i] != 0)
		window_model->chans[bands[i] - 1] = bands[i];
	for (i = 0, window_model->nchans = 0; i < NCHANS; i++)
	    if (window_model->chans[i] != 0)
		(window_model->nchans)++;
	}
    }
else					/* NOT A SPECIAL CASE */
    {
    for (i = 0; i < NCHANS; i++)
	window_model->chans[i] = 0;
    for (i = 0; i < NCHANS; i++)
	if (bands[i] != 0)
	    {
	    if (model->chans[bands[i] - 1] == 0)
		{
		c_lasmsg(FALSE,"c_windmod-bnd","Invalid channel specification");
		c_lasmsg(FALSE,"c_windmod-bnd","Channel %d not available",
			 bands[i]);
		return(E_FAIL);
		}
	    else
		{
		window_model->chans[bands[i] - 1] = bands[i];
		}
	    }
    for (i = 0, window_model->nchans = 0; i < NCHANS; i++)
	if (window_model->chans[i] != 0)
	    (window_model->nchans)++;
    }


return(E_SUCC);
}
