/*******************************************************************************
FUNCTION: c_pgwrite

PURPOSE:  Write the standing order table in ADAPSTABLES.  It is assumed that
	  the table has already been locked by the calling function so that
	  another function cannot access or update the table at the same time.
	  Return E_SUCC if successful, E_FAIL otherwise.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development
  1.1    J.Fenno          10/92  Separated prodgen and prodedit into separate
				 functions and created a product generation
				 object library.
  1.2    J.Fenno          12/92  Added age field to the prodgen table
  3.3    A. Vander Vorste 03/96  Added output directory field to prodgen.table 


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Run under TAE

PROJECT: ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adapsutil.h"
#include "products.h"
#include "queue.h"


int c_pgwrite
(
    struct DESCRIPT *descript,/* IN - Ptr to struct containing the description*/
    long orders_q             /* IN - Q holding all standing order strings    */
)
{
char filename[CMLEN];     /* Full path specification of TABLE                 */
char *ptr;                /* Ptr a character string                           */
char string[CMLEN];       /* Temporary working string                         */
char tmpstr[CMLEN];       /* Temporary working string                         */

long account;             /* Account number of the current standing order     */
long i;                   /* Loop control                                     */
long last;                /* Account number of the last standing order        */
long order;               /* Standing order number of the last order written  */
long nonfatal = 1;        /* Nonfatal error flag                              */
long unlock   = UNLOCK;   /* Flag for c_lockfil to unlock a file              */

FILE *table;              /* Ptr to the table file                            */


/* Get the file name for TABLE
------------------------------*/
ptr = (char *)getenv("ADAPSTABLES");
if (ptr == NULL)
   {
   c_errmsg("Error retrieving ADAPSTABLES", "pgwrite-env", &nonfatal);
   return(E_FAIL);
   }
strcpy(filename, ptr);
#ifdef unix
strcat(filename, "/");
#endif
strcat(filename, TABLE);


/* Open the table for writing
-----------------------------*/
table = fopen(filename, "w");
if (table == NULL)
   {
   free(descript->buffer);
   c_qdestroy(orders_q);
   c_lockfil(filename, &unlock);
   sprintf(string, "Error opening %s", filename);
   c_errmsg(string, "pgwrite-open", &nonfatal);
   return(E_FAIL);
   }


/* Update the number of standing orders in line two of the table
----------------------------------------------------------------*/
ptr = descript->buffer;
ptr += CMLEN;
sprintf(string, "%03d", c_qsize(orders_q));
strncpy(ptr, string, 3);


/* Write all of the description lines out to the table.  NOTE: we must add on
   a newline character to the end of each string so that the output file does
   not contain one large string.
-----------------------------------------------------------------------------*/
ptr = descript->buffer;
for (i = 0; i < descript->nlines; i++)
   {
   sprintf(string, "%s\n", ptr);
   if (fputs(string, table) == EOF)
      {
      free(descript->buffer);
      c_qdestroy(orders_q);
      fclose(table);
      c_lockfil(filename, &unlock);
      c_errmsg("Error writing table", "pgwrite-write", &nonfatal);
      return(E_FAIL);
      }
   ptr += CMLEN;
   }


/* Write all of the standing orders to the table
------------------------------------------------*/
last = -1;
while (!c_qempty(orders_q))
   {
   /* Get the first order from the queue
   -------------------------------------*/
   c_qfirst(orders_q, (void **)&ptr);


   /* Retrieve the account number from the string
   ----------------------------------------------*/
   sscanf(ptr, "%d", &account);
   if (account != last)
      {
      last = account;
      order = 1;
      }

   
   /* Store the current order number into string
   ---------------------------------------------*/
   sprintf(tmpstr, "%03d", order++);
   strncpy((ptr + 10), tmpstr, 3);


   /* Write the current order to the table.  NOTE: all strings must have a
      newline character added to them before they are written to the table.
   ------------------------------------------------------------------------*/
   sprintf(string, "%s\n", ptr);
   if (fputs(string, table) == EOF)
      {
      free(descript->buffer);
      c_qdestroy(orders_q);
      fclose(table);
      c_lockfil(filename, &unlock);
      c_errmsg("Error writing table", "pgwrite-write", &nonfatal);
      return(E_FAIL);
      }
   

   /* Delete the first order from the queue
   ----------------------------------------*/
   c_qdelete(orders_q, 1);
   }


/* Close the table
------------------*/
fclose(table);


/* For VMS systems set the file protection and purge the table
--------------------------------------------------------------*/
#ifdef vms
sprintf(string, "set prot=%s %s", PROTECTION, filename);
system(string);
sprintf(string, "purge %s", filename);
system(string);
#endif

return(E_SUCC);
}
