/*******************************************************************************
FUNCTION: c_qcreate()

PURPOSE:  Given a string specifying the order of the queue, create a new queue
	  for the user.  Return the number of the queue if successful, -1
	  otherwise.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development

*******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "queue.h"

struct QNODE *queue[MAXQUEUES];    /* Ptr to dummy header of all Q's    */
struct QNODE *last_pos[MAXQUEUES]; /* Ptr to last accessed position     */
short  qduplicates[MAXQUEUES];     /* Flags indicating if Qs allow dups */
short  qorder[MAXQUEUES];          /* Flags indicating Q orders         */
long   nobjects[MAXQUEUES];        /* Number of objects in the queues   */
long   nqueues = 0;                /* Number of current Q's in use      */


long c_qcreate
(
    char *order,        /* IN - String indicating the queue's order           */
    char *duplicates    /* IN - String indicating if duplicates are allowed   */
)
			/*      If the string "nodups" is sent, then no       */
			/*      duplicates are allowed in the queue, otherwise*/
			/*      duplicates are allowed.                       */
{
long q;                 /* Number of the available queue that is being set up */


/* If the number of queues is 0 then initialize nobjects and last_pos,
   if the number of queues is at the maximum then return an error.
----------------------------------------------------------------------*/
if (nqueues == 0)
   for (q = 0; q < MAXQUEUES; q++)
      {
      last_pos[q] = NULL;
      nobjects[q] = -1;
      }
else if (nqueues == MAXQUEUES)
   return(-1);


/* Determine the number of an open queue
----------------------------------------*/
for (q = 0; q < MAXQUEUES; q++)
   if (nobjects[q] == -1)
      break;


/* Determine the queue order
----------------------------*/
if (strcmp(order, "ascending") == 0)
   qorder[q] = ASCEND;
else if (strcmp(order, "descending") == 0)
   qorder[q] = DESCEND;
else if (strcmp(order, "fifo") == 0)
   qorder[q] = FIFO;
else if (strcmp(order, "lifo") == 0)
   qorder[q] = LIFO;
else
   return(-1);


/* Determine if duplicates are allowed
--------------------------------------*/
if (strcmp(duplicates, "nodups") == 0)
   qduplicates[q] = NODUPS;
else
   qduplicates[q] = DUPS;


/* Allocate a new q node
------------------------*/
queue[q] = (struct QNODE *)malloc(sizeof(struct QNODE));
if (queue[q] == NULL)
   return(-1);


/* Initialize all of the fields
-------------------------------*/
queue[q]->prev = queue[q];
queue[q]->next = queue[q];
queue[q]->obj  = NULL;
last_pos[q]    = NULL;
nobjects[q]    = 0;
nqueues++;


/* Return the number of this queue
----------------------------------*/
return(q);
}
