/*******************************************************************************
FUNCTION: c_qfind()

PURPOSE:  Given the number of a queue, a pointer to an object to search for,
	  and a comparison function, search though the queue looking for the
	  first occurrence of an object that is equal to the given object.
	  If an equal object is found then return a pointer to that object, its
	  position in the queue, and 0.  If no equal object is found then return
	  a NULL pointer, 0 for the position, and 0.  If an error occurs then
	  return a NULL pointer, 0 for the position, and a function value of -1.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development

*******************************************************************************/
#include "queue.h"

extern struct QNODE *queue[MAXQUEUES];   /* Ptr to dummy header of all Q's    */
extern struct QNODE *last_pos[MAXQUEUES];/* Ptr to last accessed position     */
extern short qduplicates[MAXQUEUES];     /* Flags indicating if Qs allow dups */
extern short qorder[MAXQUEUES];          /* Flags indicating Q orders         */
extern long  nobjects[MAXQUEUES];        /* Number of objects in the queues   */
extern long  nqueues;                    /* Number of current Q's in use      */

long c_qfind
(
    long q,             /* IN  - Number of the queue to search through        */
    void *objin,        /* IN  - Ptr to an object to find                     */
    void **objout,      /* OUT - Ptr to a ptr to the object that is found     */
    long (compare)(),	/* IN  - Function to compare two objects	      */
    long *position      /* OUT - Position of the object in the queue          */
)
{
long i;                 /* Counter                                            */
struct QNODE *qptr;     /* Temporary ptr to a queue node                      */


/* Ensure q points to something
-------------------------------*/
if ((nqueues < 1) || (nobjects[q] == -1))
   return(-1);


/* Initialize the output object, and its position
-------------------------------------------------*/
*objout = NULL;
*position = 0;


/* If the queue is empty return 0
---------------------------------*/
if (nobjects[q] == 0)
   return(0);


/* Search through the queue for objin.  For an ascending queue we must keep
   searching through the queue while the object we are looking for is greater
   than the current object in the queue.  For a descending queue we must keep
   searching through the queue while the object we are looking for is less than
   the current object in the queue.  For both FIFO and LIFO queues we must
   search through the entire queue.
------------------------------------------------------------------------------*/
qptr = queue[q]->next;
i = 1;
if (qorder[q] == ASCEND)
   {
   while ((qptr != queue[q]) && (compare(objin, qptr->obj) > 0))
      {
      i++;
      qptr = qptr->next;
      }
   }
else if (qorder[q] == DESCEND)
   {
   while ((qptr != queue[q]) && (compare(objin, qptr->obj) < 0))
      {
      i++;
      qptr = qptr->next;
      }
   }
else
   {
   while ((qptr != queue[q]) && (compare(objin, qptr->obj) != 0))
      {
      i++;
      qptr = qptr->next;
      }
   }


/* If we are back at the head of the queue or if the current
   object isn't equal to objin then the object wasn't found.
------------------------------------------------------------*/
if ((qptr == queue[q]) || (compare(objin, qptr->obj) != 0))
   return(0);


/* We found the first occurrence of this object
-----------------------------------------------*/
*objout     = qptr->obj;
*position   = i;
last_pos[q] = qptr;
return(0);
}
 
