/*******************************************************************************
FUNCTION: c_qinsert()

PURPOSE:  Given the number of a queue, a pointer to an object, and a function to
	  compare two objects, insert the given object in the queue.  Return 0
	  if successful, -1 otherwise.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development

*******************************************************************************/
#include <stdlib.h>
#include "queue.h"

extern struct QNODE *queue[MAXQUEUES];   /* Ptr to dummy header of all Q's    */
extern struct QNODE *last_pos[MAXQUEUES];/* Ptr to last accessed position     */
extern short qduplicates[MAXQUEUES];     /* Flags indicating if Qs allow dups */
extern short qorder[MAXQUEUES];          /* Flags indicating Q orders         */
extern long  nobjects[MAXQUEUES];        /* Number of objects in the queues   */
extern long  nqueues;                    /* Number of current Q's in use      */

long c_qinsert
(
    long q,             /* IN - Number of the queue to insert into            */
    void *object,       /* IN - Ptr to an object to insert                    */
    long (compare)()	/* IN - Function to compare two objects		      */
)
{
long   position;        /* Position of an element in the queue                */
struct QNODE *new_node; /* Ptr to a new queue node                            */
struct QNODE *qptr;     /* Temporary ptr to a queue node                      */
void *tmpobj;           /* Ptr to a temporary queue element                   */


/* Ensure q points to something
-------------------------------*/
if ((nqueues < 1) || (nobjects[q] == -1))
   return(-1);


/* If the Q's order is FIFO or LIFO check for a duplicate value
---------------------------------------------------------------*/
if (((qorder[q] == FIFO) || (qorder[q] == LIFO)) && (qduplicates[q] == NODUPS))
   {
   if (c_qfind(q, object, &tmpobj, compare, &position))
      return(-1);
   if ((tmpobj != NULL) || (position != 0))
      return(0);
   }


/* Set qptr so that the new object is inserted after it
-------------------------------------------------------*/
qptr = queue[q];
switch (qorder[q])
   {
   case ASCEND:
      while ((qptr->next != queue[q]) && (compare(object,qptr->next->obj) > 0))
	 qptr = qptr->next;

      /* Check for duplicate objects if necessary
      -------------------------------------------*/
      if ((qduplicates[q] == NODUPS) &&
	  (qptr->next != queue[q])   &&
	  (compare(object, qptr->next->obj) == 0))
	 return(0);
      break;
   case DESCEND:
      while ((qptr->next != queue[q]) && (compare(object,qptr->next->obj) < 0))
	 qptr = qptr->next;

      /* Check for duplicate objects if necessary
      -------------------------------------------*/
      if ((qduplicates[q] == NODUPS) &&
	  (qptr->next != queue[q])   &&
	  (compare(object, qptr->next->obj) == 0))
	 return(0);
      break;
   case FIFO:
      /* Insert after the end of the queue */
      qptr = qptr->prev;
      break;
   case LIFO:
      /* Insert after the head of the queue */
      break;
   }


/* Create a new qnode
---------------------*/
new_node = (struct QNODE *)malloc(sizeof(struct QNODE));
if (new_node == NULL)
   return(-1);


/* Assign the values to this new qnode
--------------------------------------*/
new_node->obj  = object;
new_node->next = qptr->next;
new_node->prev = qptr;


/* Update the links in the current queue and the global variables
-----------------------------------------------------------------*/
qptr->next->prev = new_node;
qptr->next       = new_node;
last_pos[q]      = new_node;
nobjects[q]++;
return(0);
}
 
