/*******************************************************************************
FUNCTION: c_qprev()

PURPOSE:  Given the number of a queue, return a pointer to the previous object
	  in the queue.  If there is a previous object in the queue then return
	  a pointer to it and 0.  If there isn't a previous object in the queue
	  then return NULL and 0.  Return NULL and -1 to indicate an error.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development

*******************************************************************************/
#include "queue.h"

extern struct QNODE *queue[MAXQUEUES];   /* Ptr to dummy header of all Q's    */
extern struct QNODE *last_pos[MAXQUEUES];/* Ptr to last accessed position     */
extern short qduplicates[MAXQUEUES];     /* Flags indicating if Qs allow dups */
extern short qorder[MAXQUEUES];          /* Flags indicating Q orders         */
extern long  nobjects[MAXQUEUES];        /* Number of objects in the queues   */
extern long  nqueues;                    /* Number of current Q's in use      */

long c_qprev
(
    long q,             /* IN  - Number of the queue                          */
    void **ptr          /* OUT - Ptr to the next object in the queue          */
)
{
/* Initialize ptr
-----------------*/
*ptr = NULL;


/* Ensure q points to something
-------------------------------*/
if ((nqueues < 1) || (nobjects[q] == -1))
   return(-1);


/* Ensure there is a last position accessed and that
   the previous position is not the head of the queue.
------------------------------------------------------*/
if ((last_pos[q] == NULL) || (last_pos[q]->prev == queue[q]))
   return(0);


/* Move the last position accessed and return the previous object
-----------------------------------------------------------------*/
last_pos[q] = last_pos[q]->prev;
*ptr = last_pos[q]->obj;
return(0);
}
 
