/*******************************************************************************
FUNCTION:	c_qremove()

PURPOSE:	Given the number of a queue and a position to remove, remove
		this qnode and return a pointer to the object.  Return 0 if
		successful, -1 otherwise.


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          12/93  Development

*******************************************************************************/
#include <stdlib.h>
#include "queue.h"

extern struct QNODE *queue[MAXQUEUES];   /* Ptr to dummy header of all Q's    */
extern struct QNODE *last_pos[MAXQUEUES];/* Ptr to last accessed position     */
extern short qduplicates[MAXQUEUES];     /* Flags indicating if Qs allow dups */
extern short qorder[MAXQUEUES];          /* Flags indicating Q orders         */
extern long  nobjects[MAXQUEUES];        /* Number of objects in the queues   */
extern long  nqueues;                    /* Number of current Q's in use      */

long c_qremove
(
    long q,             /* IN  - Number of a queue                            */
    long position,      /* IN  - Position in the queue to be deleted          */
    void **ptr          /* OUT - Ptr to the specified object in the queue     */
)
{
long i;                 /* Counter                                            */
struct QNODE *qptr;     /* Temporary ptr to a queue node                      */


/* Initialize ptr
-----------------*/
*ptr = NULL;


/* Ensure q points to something
-------------------------------*/
if ((nqueues < 1) || (nobjects[q] == -1))
   return(-1);


/* If the queue is empty return 0
---------------------------------*/
if (nobjects[q] == 0)
   return(0);


/* Walk through the queue to position
-------------------------------------*/
qptr = queue[q]->next;
i = 1;
while ((qptr != queue[q]) && (i < position))
   {
   i++;
   qptr = qptr->next;
   }


/* If we are back at the head of the queue then return
------------------------------------------------------*/
if (qptr == queue[q])
   return(0);


/* We found the correct position so relink the list
---------------------------------------------------*/
qptr->prev->next = qptr->next;
qptr->next->prev = qptr->prev;


/* Return the object in this qnode and free the qnode
-----------------------------------------------------*/
*ptr = qptr->obj;
free(qptr);


/* Update the number of objects in the queue and the last accessed position
---------------------------------------------------------------------------*/
nobjects[q] -= 1;
last_pos[q] = NULL;
return(0);
}
 
