/******************************************************************************
FUNCTION:	C_CVTOFFSET

PURPOSE:	Apply a millisecond offset to a time, julian day, year and
		return an updated time, julian day, year.  This routine is
		trivial unless a day/year is spanned by applying the offset.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/97  T.Baltzer        Development
  1.1    07/98  D.Lloyd		 Modified for Y2K compliance

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapsutil.h"


void c_cvtoffset
(
    long inyear,		/* Input year				*/
    long injday,		/* Input julian day			*/
    long intime,		/* Input time (in milliseconds)		*/
    long inoffset,		/* Input time offset (in milliseconds)	*/
    long *outyear,		/* Output year				*/
    long *outjday,		/* Output julian day			*/
    long *outtime		/* Output time (in milliseconds)	*/
)

{

long days_per_year;	/* Days in the year	*/
long temp_year;		/* Temporary year	*/
long loutyear;		/* local copy of outyear*/


/*
  Initialize the output year and day to the input year and day.
  If they need to change, they will be reset appropriately
  later in this routine.
*/
/*****************************************************************
DLLOYD Y2K
  make a local copy of output year and convert it to 4-digits
******************************************************************/
loutyear = c_4digityr( inyear );
*outjday = injday;

*outtime = intime + inoffset;		/* Combine the offset with the time */
					/* The offset may be positive or neg */


if (*outtime >= MIL_PER_DAY)		/* Did we span ahead in days?	*/
    {
    while (*outtime >= MIL_PER_DAY)
	{
	*outtime -= MIL_PER_DAY;	/* Subtract one day's worth of time */
	(*outjday)++;			/* Increment the day	*/
	}
    /*****************************
    DLLOYD Y2K
      convert inyear to 4-digits
    ******************************/
    temp_year = c_4digityr( inyear );
    if (c_isleap(&temp_year))		/* Determine days in this year	*/
	days_per_year = 366;
    else
	days_per_year = 365;
    if (*outjday > days_per_year)	/* Did we span ahead one year?	*/
	{
	*outjday -= days_per_year;	/* Subtract one year's worth of days */
	(loutyear)++;			/* Increment the year	*/
	}
    }
else if (*outtime < 0)			/* Did we span back one day?	*/
    {
    while (*outtime < 0)
	{
	*outtime += MIL_PER_DAY;	/* Add one day's worth of time */
	(*outjday)--;			/* Decrement the day	*/
	}
    if (*outjday < 1)			/* Did we span back one year?	*/
	{
	(loutyear)--;			/* Decrement the year	*/
	if (c_isleap(&loutyear))	/* Determine days in this year	*/
	    days_per_year = 366;
	else
	    days_per_year = 365;
	*outjday += days_per_year;	/* Add one year's worth of days */
	}
    }

/*************************************************
DLLOYD Y2K
  if input year was 2-digit return a 2-digit year
**************************************************/
if ( inyear < 100 )
    *outyear = loutyear % 100;
else
    *outyear = loutyear;

return;
}
