/*******************************************************************************
NAME:		C_CVTOFFSETE	

PURPOSE:	Generate the offset year, month, day, and time from an
		input year, month, day, time, and time offset.  This routine
		performs the same function as c_cvtoffset() does, only
		expecting expanded parameters.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    09/97  T. Baltzer       Development
				
PROJECT:	ADAPS

*******************************************************************************/
#include <stdio.h> 
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "adapsutil.h"


void c_cvtoffsete
(
    long in_year,		/* Input two digit year		*/
    long in_month,		/* Input month			*/
    long in_day,		/* Input day of month		*/
    long in_hour,		/* Input hours			*/
    long in_min,		/* Input minutes		*/
    long in_sec,		/* Input seconds		*/
    long in_frac,		/* Input milliseconds		*/
    long in_offset,		/* Offset to apply (millisec)	*/
    long *out_year,		/* Output two digit year	*/
    long *out_month,		/* Output month			*/
    long *out_day,		/* Output day of month		*/
    long *out_hour,		/* Output hours			*/
    long *out_min,		/* Output minutes		*/
    long *out_sec,		/* Output seconds		*/
    long *out_frac		/* Output milliseconds		*/
)

{

long jdate;			/* Julian date			*/
long totalmsec;			/* Total time in milliseconds	*/
long temp_year;			/* Temporary year		*/
long temp_month;		/* Temporary month		*/
long temp_day;			/* Temporary day		*/
long temp_hour;			/* Temporary hour		*/
long temp_min;			/* Temporary minute		*/
long temp_sec;			/* Temporary second		*/


/* Define the temporary variables
   --------------------------------------------------------------*/
temp_hour = in_hour;
temp_min = in_min;
temp_sec = in_sec;
temp_year = in_year;
temp_month = in_month;
temp_day = in_day;


/* Convert the input time to milliseconds 
   --------------------------------------------------------------*/
c_cvtime(&totalmsec,&temp_hour,&temp_min,&temp_sec);
totalmsec = (totalmsec * MIL_PER_SEC) + in_frac;


/* Convert the input date to a julian date
   --------------------------------------------------------------*/
c_grejul(&temp_year,&temp_month,&temp_day,&jdate);


/* Apply the offset to the date/time to generate a modified date/time.
   -----------------------------------------------------------------------*/
c_cvtoffset(in_year,jdate,totalmsec,in_offset,out_year,&jdate,&totalmsec);


/* Convert the date back to a gregorian date
   --------------------------------------------------------------*/
c_julgre(out_year,&jdate,out_month,out_day);


/* Convert the time of day back into hour, min, sec, milliseconds.
   --------------------------------------------------------------------*/
c_cvtmil(&totalmsec,out_hour,out_min,out_sec,out_frac);

return;
}
