/******************************************************************************
FUNCTION:	c_daysdiff

PURPOSE:	Given two month/day/year specifications, find the number of
		days between the two.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       01/98    T. Baltzer		initial development
 1.1	   07/98    D.Lloyd		Modified for Y2K compliance

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include "las.h"
#include "util.h"
#include "adapsutil.h"


int c_daysdiff
(
    long *month_1,		/* Month of first date	(input)		*/
    long *day_1,		/* Day of first date	(input)		*/
    long *year_1,		/* Year of first date	(input)		*/
    long *month_2,		/* Month of second date	(input)		*/
    long *day_2,		/* Day of second date	(input)		*/
    long *year_2,		/* Year of second date	(input)		*/
    long *ndays			/* Number of days between
				   first and second date (output)	*/
)

{

long between_yr;	/* Years between two dates 	*/
long days_per_year;	/* Days in the year		*/
long earliest_year;	/* Earliest year		*/
long earliest_jdate;	/* Earliest julian day		*/
long jdate_1;		/* Julian days in first date	*/
long jdate_2;		/* Julian days in second date	*/
long latest_jdate;	/* Latest julian day		*/
long latest_year;	/* Latest year 			*/
long tyear_1;		/* First temporary year		*/
long tyear_2;		/* Second temporary year	*/


/*
  Convert dates to julian days
  ------------------------------------------------------------------------*/
c_grejul(year_1,month_1,day_1,&jdate_1);
c_grejul(year_2,month_2,day_2,&jdate_2);


/*
  Account for year 2000
  ------------------------------------------------------------------------*/
/**************************
DLLOYD Y2K
  convert years to 4-digit
***************************/
tyear_1 = c_4digityr(*year_1);
tyear_2 = c_4digityr(*year_2);


/*
  If both are in the same year, just subtract the number of days
  ------------------------------------------------------------------------*/
if (tyear_1 == tyear_2)
    {
    if (jdate_1 > jdate_2)
	*ndays = jdate_1 - jdate_2;
    else
	*ndays = jdate_2 - jdate_1;
    return(E_SUCC);
    }

/*
  Years are not the same, need to find:
  (1) For the earliest year, the number of days to the end of that year
  (2) The number of days to add for each year between the two input years
  (3) For the oldest year, the number of days from the beginning of the year
  Begin by finding the earliest and latest year and julian day.
  ---------------------------------------------------------------------------*/
if (tyear_1 > tyear_2)
    {
    earliest_year = tyear_2;
    earliest_jdate = jdate_2;
    latest_year = tyear_1;
    latest_jdate = jdate_1;
    }
else
    {
    earliest_year = tyear_1;
    earliest_jdate = jdate_1;
    latest_year = tyear_2;
    latest_jdate = jdate_2;
    }

/*
  Account for:
  (1) For the earliest year, the number of days to the end of that year
  ---------------------------------------------------------------------------*/
if (c_isleap(&earliest_year))
    days_per_year = 366;
else
    days_per_year = 365;
*ndays = days_per_year - earliest_jdate;


/*
  Account for:
  (2) The number of days to add for each year between the two input years
  ---------------------------------------------------------------------------*/
for (between_yr = earliest_year + 1; between_yr < latest_year; between_yr++)
    {
    if (c_isleap(&between_yr))
	days_per_year = 366;
    else
	days_per_year = 365;
    (*ndays) += days_per_year;
    }


/*
  Account for:
  (3) For the oldest year, the number of days from the beginning of the year
  ---------------------------------------------------------------------------*/
(*ndays) += latest_jdate;

return(E_SUCC);

}
