/*******************************************************************************
NAME:		C_GETSAT

PURPOSE:	Get the satellite identification information.  Given a
		specified key return other satellite information that
		identifies the satellite.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		8/91	D. Hollaren	   CSB      Original development

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/
#include <sys/file.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "util.h"
#include "tae.h"
#include "adapsutil.h"

int c_getsat
(
   char *key,			/* Input key to find the other codes	*/
   long *satnum,		/* Satellite number (11)		*/
   char *satnam,		/* Satellite name   (NOAA-11)		*/
   char *satno,			/* Satellite number (N11)		*/
   char *satid,			/* Satellite id     (19531)		*/
   char *satcode 		/* Satellite code   (I)			*/
)
{
FILE	*fp;

char	disk[CMLEN],		/* Disk location of satellite id table	*/
	satfile[CMLEN],		/* Ephemeris file			*/
	skey[8];		/* String match key			*/

long	nkey,			/* Numeric match key			*/
	nbytes,			/* Number of bytes to upper case	*/
	nonfatal = 1,		/* Non-fatal error code			*/
	found = FALSE,		/* Found a match flag			*/
	status = 5;		/* Read status 				*/

/*
  Get the satellite id file name
*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(disk,getenv("ADAPSTABLES"));
else
    {
    c_errmsg("Error retrieving value of ADAPSTABLES","getsat-find",&nonfatal);
    return(E_FAIL);
    }

#ifdef VMS
    sprintf(satfile,"%s%s",disk,"satellite.ids");
#else
    sprintf(satfile,"%s/%s",disk,"satellite.ids");
#endif

/*
  Open the satellite id file
*/
fp = fopen(satfile,"r");
if (fp == NULL)
    {
    c_errmsg("Error opening the satellite id file","getsat-open",&nonfatal);
    return(E_FAIL);
    }

/*
  Upcase the key for matching
*/
nbytes = (long)strlen(key);
c_low2up(key,&nbytes);

/*
  Check each record in the satellite id file for the specified key.  When a 
  match is found break from the loop and return the other satellite information.
*/
if (strcmp(key,"SATNUM") == 0)
    {
    while (status != EOF)
	{
	/*
 	  Get the satellite information for the satellite number
	*/
	status = fscanf(fp,"%d%s%s%s%s",&nkey,satnam,satno,satid,satcode);
	if (status == EOF)
	    continue;
	else if (status == 0)
	    {
	    fclose(fp);
	    c_errmsg("Error reading the satellite id file","getsat-read",
		     &nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Check to see if the satellite number matches the specified key
	  Break out if they match
	*/
	if (*satnum == nkey)
	    {
	    found = TRUE;
	    break;
	    }
	}
    }
else if (strcmp(key,"SATNAM") == 0)
    {
    while (status != EOF)
	{
	/*
 	  Get the satellite information for the satellite name
	*/
	status = fscanf(fp,"%d%s%s%s%s",satnum,skey,satno,satid,satcode);
	if (status == EOF)
	    continue;
	else if (status == 0)
	    {
	    fclose(fp);
	    c_errmsg("Error reading the satellite id file","getsat-read",
		     &nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Upcase the user's input for checking
	*/
	nbytes = (long)strlen(satnam);
	c_low2up(satnam,&nbytes);

	/*
	  Check to see if the satellite name matches the specified key
	  Break out if they match
	*/
	if (strcmp(satnam,skey) == 0)
	    {
	    found = TRUE;
	    break;
	    }
	}
    }
else if (strcmp(key,"SATNO") == 0)
    {
    while (status != EOF)
	{
	/*
 	  Get the satellite information for the short satellite name
	*/
	status = fscanf(fp,"%d%s%s%s%s",satnum,satnam,skey,satid,satcode);
	if (status == EOF)
	    continue;
	else if (status == 0)
	    {
	    fclose(fp);
	    c_errmsg("Error reading the satellite id file","getsat-read",
	 	    &nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Upcase the user's input for checking
	*/
	nbytes = (long)strlen(satno);
	c_low2up(satno,&nbytes);

	/*
	  Check to see if the short satellite name matches the specified key
	  Break out if they match
	*/
	if (strcmp(satno,skey) == 0)
	    {
	    found = TRUE;
	    break;
	    }
	}
    }
else if ((strcmp(key,"SATID") == 0) || (strcmp(key,"satid") == 0))
    {
    while (status != EOF)
	{
	/*
 	  Get the satellite information for the satellite id
	*/
	status = fscanf(fp,"%d%s%s%s%s",satnum,satnam,satno,skey,satcode);
	if (status == EOF)
	    continue;
	else if (status == 0)
	    {
	    fclose(fp);
	    c_errmsg("Error reading the satellite id file","getsat-read",
		     &nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Upcase the user's input for checking
	*/
	nbytes = (long)strlen(satid);
	c_low2up(satid,&nbytes);

	/*
	  Check to see if the satellite id matches the specified key
	  Break out if they match
	*/
	if (strcmp(satid,skey) == 0)
	    {
	    found = TRUE;
	    break;
	    }
	}
    }
else if ((strcmp(key,"SATCODE") == 0) || (strcmp(key,"satcode") == 0))
    {
    while (status != EOF)
	{
	/*
 	  Get the satellite information for the satellite code
	*/
	status = fscanf(fp,"%d%s%s%s%s",satnum,satnam,satno,satid,skey);
	if (status == EOF)
	    continue;
	else if (status == 0)
	    {
	    fclose(fp);
	    c_errmsg("Error reading the satellite id file","getsat-read",
		     &nonfatal);
	    return(E_FAIL);
	    }

	/*
	  Upcase the user's input for checking
	*/
	nbytes = (long)strlen(satcode);
	c_low2up(satcode,&nbytes);

	/*
	  Check to see if the satellite code matches the specified key
	  Break out if they match.
	*/
	if (strcmp(satcode,skey) == 0)
	    {
	    found = TRUE;
	    break;
	    }
	}
    }
else
    {
    fclose(fp);
    c_errmsg("Invalid satellite search key","getsat-key",&nonfatal);
    return(E_FAIL);
    }

/*
  Close the satellite id file
*/
fclose(fp);

if (!found)
    {
    c_errmsg("Specified satellite key not found","getsat-find",&nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
 
