/******************************************************************************
FUNCTION:	C_MAKESECS

PURPOSE:	Compute the total number of seconds in the date and time
  		The number of seconds is used to write to the temporary 
		schedule file.
		The total seconds is from 00:00:00 on 1/1/86.  Any dates before
		this are counted backwards as seconds BEFORE 00:00:00 1/1/86,
		and are left as a negative number indicating such.

PROGRAM HISTORY:
Version    Date     Author      	Change Request
-------    -----    ---------   	--------------
  1.0      12/86     P. Mumford         Initial development
  5.0      10/92     T. Rockvam         Conversion to LAS-5.0
  5.1	   07/98     D.Lloyd		Modified for Y2K complaince

$Log: c_makesecs.c,v $
Revision 3.1  2002/06/19 12:18:23  dlloyd
Added c_makesecs() as a support function.  CR 4411

*******************************************************************************/
#include <stdio.h>
#include "util2.h"

#define START_OF_TIME   1986

void c_makesecs
(
long	date[],			/* date (month, day, year)		*/
long	time[],			/* time (hour, min, sec)		*/
long	*totsecs		/* total seconds			*/
)
{
long	jday;			/* julian day				*/
long	yr;			/* looping year				*/
long	year4;			/* 4-digit year				*/
long 	totdays;		/* total days from 1/1/1986		*/

/*
  Find number of days at given date
*/
c_grejul(&date[2],&date[0],&date[1],&jday);

/*
  Add up all days from beginning of time (1/1/1986) to now
  input year must be 4 digits
*/
totdays = 0;
/*
  DLLOYD Y2K -- changed conversion to use new routine
------------------------------------------------------*/
year4 = c_4digityr( date[2] );

if (year4 >= START_OF_TIME)
    {
    for (yr = START_OF_TIME; yr < year4; yr++)	/* add from 86 to current year*/
        {
        if (c_isleap(&yr))
            totdays += 366;
        else
	    totdays += 365;
        }
    totdays += jday;				/* add current year */

    /*
      Add up the total seconds
	use (totdays - 1) because target date is included in time[]
    */
    *totsecs = (totdays - 1) * 86400 + time[0] * 3600 + time[1] * 60 + time[2];
    }
else		/* negative time for date before 1986 */
    {
    if (c_isleap(&year4))
	/*totdays += (366 - jday + 1);*/
	totdays += (366 - jday);
    else
	totdays += (365 - jday);
    for (yr = year4+1; yr < START_OF_TIME; yr++)
        {
        if (c_isleap(&yr))
            totdays += 366;
        else
	    totdays += 365;
	}

    /*
      Add up the total seconds
    */
    *totsecs = (-totdays) * 86400 + (time[0] * 3600 + time[1] * 60 + time[2]) 
		- 86400;
    }

return;
}
