/*******************************************************************************
FUNCTION:	c_pixflip

PURPOSE:	Flip the pixels in a line of BIP data.  It is assumed that the
		line contains NCHANS channels.  This function is normally used
		to flip the pixels in a line of ascending AVHRR data.  Nvalues
		is the total number of short values in the line of image data.
		For example, 2048 pixels * 5 channels = 10240.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development

PROJECT:	ADAPS
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "archio.h"
#include "adapsutil.h"


void c_pixflip
(
    short pixels[],     /* IN/OUT - Array of I*2 pixel values                 */
    long nvalues      /* IN     - Total number of short values in the line  */
)
{
register long lopixel;  /* Leftmost pixel that is being swapped               */
register long hipixel;  /* Rightmost pixel that is being swapped              */
register long loindex;  /* Index for the NCHANS values for leftmost pixel     */
register long hiindex;  /* Index for the NCHANS values for rightmost pixel    */

register short i;       /* Loop control                                       */
register short tmp;     /* Temp variable used for swapping short values       */


/* Walk a pointer from the left end of the image line toward the center
   and another pointer from the right end of the line toward the center
-----------------------------------------------------------------------*/
for (lopixel = 0, hipixel = nvalues / NCHANS - 1;
     lopixel < hipixel;
     lopixel++, hipixel--)
    {
    /* Swap each of the NCHANS values for leftmost and rightmost pixels
    -------------------------------------------------------------------*/
    for (loindex = (lopixel * NCHANS), hiindex = (hipixel * NCHANS), i = 0;
	 i < NCHANS;
	 loindex++, hiindex++, i++)
	{
	tmp = pixels[loindex];
	pixels[loindex] = pixels[hiindex];
	pixels[hiindex] = tmp;
	}
    }
}
 
