/******************************************************************************
FUNCTION:	C_UNPCK32

PURPOSE:	Unpack a 16-bit word (10-bits used) into an I*2 word

ALGORITHM 
    Top six bits are unused.  One 10-bit word is packed right
    justified into one 16-bit word.

    Format of input buffer:

    +-----------------------+
    ! 000000   :   Word 1   !   1st 16-bit word
    +-----------------------+
    ! 000000   :   Word 2   !   2nd 16-bit word
    +-----------------------+
    etc.

    Format of output buffer:

    +--------------------+
    !  0  :    Word 1    !  1st 16-bit word
    +--------------------+
    !  0  :    Word 2    !  2nd 16-bit word
    +--------------------+
    etc.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "pixman.h"
#include "adapsutil.h"

#define MASK1 0X3ff00000
#define MASK2 0X000ffc00
#define MASK3 0X000003ff

void c_unpck32
(
    unsigned char *inbuf,	/* Pointer to the 32-bit input buffer	    */
    short *outbuf,	/* Series of 10 bit words stored in 16 bits         */
    long *nwords,	/* Number of 16-bit words to unpack	            */
    long *swap		/* Byte swap flag				    */
)
{
short	*optr;		/* Pointer to the output buffer			    */

long	*iptr,		/* Pointer to the 32-bit input buffer	            */
	num,		/* Number of full 32-bit long words		    */
	extra,		/* Extra 10 bit words				    */
	total,		/* Total number of 32-bit long words		    */
	i,		/* Looping variable				    */
	size = 4;	/* Size of the data to be byte swapped		    */

/*
  Calculate the number of 32-bit long words that contain
  three 10 bit words
*/
num = *nwords / 3;

extra = (*nwords % 3);

if (*swap)
    {
    /*
      Byte swap the data if needed
    */
    total = num + extra;
    c_pxswap(inbuf,&total,&size);
    }

/*
  Set pointers to the input and output buffers
*/
iptr = (long *)inbuf;
optr = outbuf;

/*
  Unpack a 16-bit word into one 10-bit word
*/
for (i = 0; i < num; i++)
    {
    /*
      Use the masks to get each 10 bits
    */
    *optr = (short)((*iptr & MASK1) >> 20);
    optr += 1;

    *optr = (short)((*iptr & MASK2) >> 10);
    optr += 1;

    *optr = (short)(*iptr & MASK3);
    optr += 1;

    iptr += 1;
    }

/*
  Handle any 10-bit words which are left over
*/
if (extra >= 1)
    {
    *optr = (short)((*iptr & MASK1) >> 20);

    if (extra == 2)
	{
	optr += 1;
	*optr = (short)((*iptr & MASK2) >> 10);
	}
    }

/* Put the input buffer back the way it was
-------------------------------------------*/
if (*swap)
    {
    total = num + extra;
    c_pxswap(inbuf,&total,&size);
    }

return;
}
 
