/****************************************************************************
NAME:		C_WRTLOG

PURPOSE:	Open the specified log file and write the key, processing
		code, and informational text to the file.


PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  1.0	 D. Hollaren   06/87	Original Development
  5.0	 D. Hollaren   04/92	Conversion to LAS5.0
  5.1	 D.Lloyd       07/98    Modified for Y2K compliance

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/file.h>
#ifdef sun
#include <fcntl.h>
#endif
#include "adapsutil.h"

#include "las.h"
#ifndef SEEK_END    /* Not defined on PowerNode */
#define SEEK_END 2
#endif

int c_wrtlog
(
   char	*logfile,		/* Log file to write to			*/
   char *key,			/* Key in the logfile			*/
   char *code,			/* Operation code			*/
   char *text 			/* Description text			*/
)
{
int	fd;			/* File descriptor to the log file	*/

long	nonfatal = 1,		/* Non-fatal error code			*/
	maxtime = 60,		/* Maximum to wait on open/lock		*/
	flags,			/* Open flags				*/
	status,			/* Return status			*/
	len;			/* Length of the output buffer		*/

off_t lstatus;			/* Return status			*/

char	cur_date[9],		/* Current date				*/
	cur_time[9],		/* Current time				*/
	buffer[CMLEN];		/* Output buffer			*/

struct tm *lctm;		/* Local time structure			*/
long   time_val;		/* Time variable			*/

/*
  Get the current date.  Have to add one to month because it's zero relative.
*/
time(&time_val);
lctm = localtime(&time_val);
/****************************************************************
DLLOYD Y2K
  in year 2000 lctm->tm_year = 100 so needs to be MODed with 100
*****************************************************************/
sprintf(cur_date,"%02d/%02d/%02d",lctm->tm_mon+1,lctm->tm_mday,
	lctm->tm_year%100);
sprintf(cur_time,"%02d:%02d:%02d",lctm->tm_hour,lctm->tm_min,lctm->tm_sec);

/*
  Open and lock the log file for a moment of exclusive use
*/
flags = O_WRONLY | O_CREAT | O_APPEND;
status = cmlock(logfile,&fd,flags,maxtime);
if (status != E_SUCC)
    {
    c_errmsg("Error locking the log file","wrtlog-lock",&nonfatal);
    return(E_FAIL);
    }

/*
  Seek to the end of the file because cmlock
  resets flag and opens the file for read/write, not append
*/
lstatus = lseek(fd, 0, SEEK_END);
if (lstatus == E_FAIL)
    {
    c_errmsg("Error skipping to end of the log file","wrtlog-skip",&nonfatal);
    return(E_FAIL);
    }

/*
  Write the line to the logfile
*/
sprintf(buffer,"%-16s   %-8s   %-8s   %-16s   %-61s\n",
	key,cur_date,cur_time,code,text);
len = (long)strlen(buffer);
write(fd,buffer,len);

/*
  Unlock and close the log file
*/
status = unlock(&fd,maxtime);
if (status != E_SUCC)
    {
    c_errmsg("Error unlocking the log file","wrtlog-unlock",&nonfatal);
    close(fd);
    }
    
return(E_SUCC);
}
 
