/*******************************************************************************
NAME:		ANGCOR

PURPOSE:	Corrects the satellite zenith and relative azimuth angles.
		The windowing was done incorrectly resulting in calculating
		the angle, 150 samples from were it should have been.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		REASON
-------	   ----------      ----		------
  2.0	   D. Hollaren	   11/83 	Original Development
 
PROJECT:	ADAPS

ALGORITHM:

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include "las.h"	

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
int fdesc[5];		/* Image file descriptors used for image I/O */
int gdesc;		/* Group descriptor used for image I/O       */
long offset1[4];	/* Offsets into thebuf for input bands	     */
long offset2[4];	/* Offsets into "thebuf" for output bands    */
extern long curent;	/* Current processing line		     */
extern long curcat;	/* Current processing band		     */
unsigned char *thebuf;  /* Image I/O buffer                          */

main()
{
struct PARBLK vblock;		/* TAE parameter block                */
struct DDR    ddr;		/* Data Descriptor Record	      */

long	msgtim,			/* Elapsed time between display msg   */
	mmflag,			/* Min/Max flag			      */
	nmrimg,			/* Number of input images             */
	odtype,			/* Data type of output image          */
	status,			/* Function return status code        */
	std_out,		/* Standard output                    */
	totbnd,			/* Total number of bands              */
	nbands[3],		/* Number of bands per input image    */
	bands[3][4],		/* Bands to be processed - all images */
	window[3][4],		/* Window for each input image        */
	one = 1,		/* Value of one			      */
	nonfatal = 1,		/* Nonfatal error code		      */
	upddr_flag = NOCOMB;	/* Flag for DDR update 		      */

char	hosin[3][CMLEN],	/* Host names of input images         */
	hosout[CMLEN],		/* Host name of output image 	      */
	minmax[4],		/* Min max flag                       */
	taein[3][CMLEN],	/* TAE names of input images          */
	taeout[CMLEN];		/* TAE name of output image	      */

double  line_inc,		/* Line increment 		      */
	samp_inc,		/* Sample increment		      */
	minval[3],		/* Minimum pixel value		      */
	maxval[3];		/* Maximum pixel value		      */

/*
  Initialization LAS
*/
c_inlas(&vblock,&std_out);

/*
  Get the input parameters
*/
getpar(&vblock,taein,hosin,taeout,hosout,&nmrimg,window,bands,nbands,&totbnd,
       &msgtim,minmax,&ddr);

/*
  Set the min/max flag
*/
if (strcmp(minmax,"YES") == 0)
    mmflag = TRUE;
else
    mmflag = FALSE;

/*
  Open the input images
*/
odtype = ddr.dtype;
open_img(&vblock,hosin,hosout,nmrimg,window,bands,nbands,totbnd,&odtype);

/*
  Process the image
*/
process(&vblock,&ddr,hosout,window[0],totbnd,odtype,msgtim,mmflag,
	minval,maxval);

/*
  Update the DDR
*/
line_inc = (double)one;
samp_inc = (double)one;
status = c_upddr(&vblock,hosin[0],hosout,&nmrimg,window,bands,nbands,&line_inc,
                 &samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
      c_clean(&vblock,"angcor",hosout,&one);

/*
  Create associated min/max file
*/
if (c_svmnmx(hosout,&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","angcor-warn",&nonfatal);

/*
  Close all image files in the group
*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","angcor-warn",&nonfatal);

/*
  Free buffer
*/
free(thebuf);

/*
  Update history file 
*/
if (c_hstout(&vblock,hosout,hosin[0],&nmrimg," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","angcor-warn",&nonfatal);

/*
  Output completion message 
*/
c_complt("ANGCOR",taeout,&one);
}
