/*************************************************************
NAME		CALC_VIEWS

PURPOSE		Calculate satellite zenith angles

PROGRAM HISTORY
PROGRAMMER	DATE	REASON
----------	----	------
D. Hollaren	11/93	Original development

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "correct.h"
#include "proj.h"

FUNCTION calc_views(in_zen,in_az,out_zen,out_az,ns,dtype,line,proj)

long	ns,		/* Number of samples 			*/
	dtype,		/* Data type (EBYTE,EWORD,ELONG,EREAL)	*/
			/*   EBYTE = unsigned 1 byte  		*/
			/*   EWORD = signed 2 byte  		*/
			/*   ELONG = signed 4 byte  		*/
			/*   EREAL = real data  		*/
	line,		/* Current line number			*/

	*in_zen,	/* Input satellite zenith array 	*/
	*in_az,		/* Input relative azimuth array 	*/
	*out_zen,	/* Output satellite zenith array 	*/
	*out_az;	/* Output relative azimuth array 	*/

struct PROJ *proj;	/* Projection information		*/
{
short	status = E_SUCC; /* Return status			*/

switch(dtype)
    {
    case (EBYTE):
	bang(in_zen,in_az,out_zen,out_az,ns,line,proj);
	break;

    case (EWORD):
	wang(in_zen,in_az,out_zen,out_az,ns,line,proj);
	break;

    case (ELONG):
	lang(in_zen,in_az,out_zen,out_az,ns,line,proj);
	break;

    case (EREAL):
	fang(in_zen,in_az,out_zen,out_az,ns,line,proj);
	break;

    default:
	status = E_FAIL;
	break;
    }

return(status);
}

FUNCTION bang(in_zen,in_az,out_zen,out_az,ns,line,proj)

struct PROJ	*proj;

unsigned char	*in_zen,
		*in_az,
		*out_zen,
		*out_az;

long		ns,
		line;
{
long	samp,
	status,
	fatal = -1;

double	lat,
	lng,
	old_zen,
	new_zen,
	old_relaz;

float	new_relaz;

for (samp = 1; samp <= ns; samp++)
    {
    if (*in_zen == 0)
	{
	*out_zen = *in_zen;
	*out_az  = *in_az;
	}
    else
	{
	/*
	  Determine the latitude/longitude of the point
	*/
	status = ls2ll(proj,line,samp,&lat,&lng);
	if (status == E_FAIL)
	    c_errmsg("Fatal error encountered","angcor-fatal",&fatal);
	else if (status == IN_BREAK)
	    {
	    *out_zen = *in_zen;
	    *out_az  = *in_az;
	    }
	else
	    {
	    /*
	      Unscale the angles and convert to radians
	    */
	    old_zen   = (*in_zen * RAD_PER_DEG) - HALFPIE;
	    old_relaz = (*in_az  * RAD_PER_DEG);

	    /*
	      Calculate the new angles
	    */
	    newang(lat,old_zen,old_relaz,&new_zen,&new_relaz);

	    /*
	      Scale the angles and convert to degrees
	    */
	    *out_zen = (unsigned char)((new_zen + HALFPIE) * DEG_PER_RAD);
	    *out_az  = (unsigned char)(new_relaz * DEG_PER_RAD);
	    }
	}

    in_zen++;
    in_az++;
    out_zen++;
    out_az++;
    }

return;
}

FUNCTION wang(in_zen,in_az,out_zen,out_az,ns,line,proj)

struct PROJ	*proj;

short		*in_zen,
		*in_az,
		*out_zen,
		*out_az;

long		ns,
		line;
{
long	samp,
	status,
	fatal = -1;

double	lat,
	lng,
	old_zen,
	new_zen,
	old_relaz,
	new_relaz;

for (samp = 1; samp <= ns; samp++)
    {
    if (*in_zen == 0)
	{
	*out_zen = *in_zen;
	*out_az  = *in_az;
	}
    else
	{
	/*
	  Determine the latitude/longitude of the point
	*/
	status = ls2ll(proj,line,samp,&lat,&lng);
	if (status == E_FAIL)
	    c_errmsg("Fatal error encountered","angcor-fatal",&fatal);
	else if (status == IN_BREAK)
	    {
	    *out_zen = *in_zen;
	    *out_az  = *in_az;
	    }
 	else
	    {
	    /*
	      Unscale the angles and convert to radians
	    */
	    old_zen   = (*in_zen * RAD_PER_DEG) / 10.0;
	    old_relaz = (*in_az  * RAD_PER_DEG) / 10.0;

	    /*
	      Calculate the new angles
	    */
	    newang(lat,old_zen,old_relaz,&new_zen,&new_relaz);

	    /*
	      Scale the angles and convert to degrees
	    */
	    *out_zen = (short)((new_zen * DEG_PER_RAD) * 10.0);
	    *out_az  = (short)((new_relaz * DEG_PER_RAD) * 10.0);
	    }
	}

    in_zen++;
    in_az++;
    out_zen++;
    out_az++;
    }

return;
}

FUNCTION lang(in_zen,in_az,out_zen,out_az,ns,line,proj)

struct PROJ	*proj;

long 		*in_zen,
		*in_az,
		*out_zen,
		*out_az;

long		ns,
		line;
{
long	samp,
	status,
	fatal = -1;

double	lat,
	lng,
	old_zen,
	new_zen,
	old_relaz,
	new_relaz;

for (samp = 1; samp <= ns; samp++)
    {
    if (*in_zen == 0)
	{
	*out_zen = *in_zen;
	*out_az  = *in_az;
	}
    else
	{
	/*
	  Determine the latitude/longitude of the point
	*/
	status = ls2ll(proj,line,samp,&lat,&lng);
	if (status == E_FAIL)
	    c_errmsg("Fatal error encountered","angcor-fatal",&fatal);
	else if (status == IN_BREAK)
	    {
	    *out_zen = *in_zen;
	    *out_az  = *in_az;
	    }
	else
	    {
	    /*
	      Unscale the angles and convert to radians
	    */
	    old_zen   = (*in_zen * RAD_PER_DEG) / 10.0;
	    old_relaz = (*in_az  * RAD_PER_DEG) / 10.0;

	    /*
	      Calculate the new angles
	    */
	    newang(lat,old_zen,old_relaz,&new_zen,&new_relaz);

	    /*
	      Scale the angles and convert to degrees
	    */
	    *out_zen = (long)((new_zen * DEG_PER_RAD) * 10.0);
	    *out_az  = (long)((new_relaz * DEG_PER_RAD) * 10.0);
	    }
	}

    in_zen++;
    in_az++;
    out_zen++;
    out_az++;
    }

return;
}

FUNCTION fang(in_zen,in_az,out_zen,out_az,ns,line,proj)

struct PROJ	*proj;

float		*in_zen,
		*in_az,
		*out_zen,
		*out_az;

long		ns,
		line;
{
long	samp,
	status,
	fatal = -1;

double	lat,
	lng,
	old_zen,
	new_zen,
	old_relaz,
	new_relaz;

for (samp = 1; samp <= ns; samp++)
    {
    if (*in_zen == 0)
	{
	*out_zen = *in_zen;
	*out_az  = *in_az;
	}
    else
	{
	/*
	  Determine the latitude/longitude of the point
	*/
	status = ls2ll(proj,line,samp,&lat,&lng);
	if (status == E_FAIL)
	    c_errmsg("Fatal error encountered","angcor-fatal",&fatal);
	else if (status == IN_BREAK)
	    {
	    *out_zen = *in_zen;
	    *out_az  = *in_az;
	    }
	else
	    {
	    /*
	      Unscale the angles and convert to radians
	    */
	    old_zen   = (*in_zen * RAD_PER_DEG);
	    old_relaz = (*in_az  * RAD_PER_DEG);

	    /*
	      Calculate the new angles
	    */
	    newang(lat,old_zen,old_relaz,&new_zen,&new_relaz);

	    /*
	      Scale the angles and convert to degrees
	    */
	    *out_zen = (float)(new_zen * DEG_PER_RAD);
	    *out_az  = (float)(new_relaz * DEG_PER_RAD);
	    }
	}

    in_zen++;
    in_az++;
    out_zen++;
    out_az++;
    }

return;
}
