#define REQUA         6378206.4  /* Radius of the earth at equator (Meters)  */
#define SAT_ALTITUDE   850000.0  /* Approx. satellite altitude (Meters)      */

/*
  Define the interior angle from the inclination angle for the satellite)
  kappa = 180 - inclination
*/
#define KAPPA	       1.412538

/*
  Define the delta scan angle needed to correct for a sample shift
  Delta = (samples / pixels per radian)
  Delta = (150 sample error / 1058.818357 pixels per radian) = 0.141667358
*/
# define DELTA_SCAN	 0.141667358

/*
  Define the height of the satellite for ease of use
*/
#define SAT_HEIGHT	REQUA + SAT_ALTITUDE


/*
  Define constants
*/
#define PIE 3.141592653589793238    /* Pi                                    */
#define PIE2            (PIE*2.0)   /* Two PI                                */
#define HALFPIE         (PIE/2.0)   /* One half of PI                        */
#define RAD_PER_DEG     (PIE/180.0) /* Radians per degree                    */
#define DEG_PER_RAD     (180.0/PIE) /* Degrees per radians                   */

/*
  Band pointers
*/
#define BAND1 0
#define BAND2 1
#define BAND3 2

/*
  Structure for passing the projection information
*/
struct PROJ
    {
    long inproj;
    long inunit;
    long inzone;
    long indatum;
    double inparm[15];
    long oproj;
    long ounit;
    long ozone;
    long odatum;
    double oparm[15];
    double coefs[6];
    };
