/****************************************************************************
NAME:		GETPAR

PURPOSE:	Get the input parameters

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include "las.h"

FUNCTION getpar(vblock,taein,hosin,taeout,hosout,nmrimg,window,bands,
                nbands,totbnd,msgtim,minmax,ddr)

struct PARBLK *vblock;		/* pointer to TAE parameter block            */
struct DDR    *ddr;		/* Data Descriptor Record		     */

char	taein[][CMLEN],         /* TAE file names of input images  	     */
	hosin[][CMLEN],         /* host file names of input images 	     */
	*taeout,		/* TAE file name of output image	     */
	*hosout,		/* host file name of output image	     */
	*minmax;		/* min max flag buffer                       */

long	*nmrimg,		/* number of input images                    */
	window[][4],		/* window specifications for each input image*/
	bands[][4],		/* band specifications for each input image  */
	nbands[],		/* number of bands per input image           */
	*totbnd,		/* total number of bands to be processed     */
	*msgtim;		/* elapsed time between display messages     */
{
struct VARIABLE	*p_find();	/* function to find a TAE variable           */
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long	i,			/* counter -- current image		     */
	same,			/* flag indicating if images have same window*/
	count,			/* counter variable			     */
	status,			/* function return status code               */
	qflag[5][2],		/* flag indicating if subwindows specified   */
	imglim = 3,		/* maximum number of images to be processed  */
	fatal = -1,		/* fatal error flag			     */
	nonfatal = 1,		/* nonfatal error flag			     */
	element = 0;		/* parameter element number		     */

/*
  Retrieve and parse input image specification 
*/
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,nbands,
		totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","angcor-fatal",&fatal);

/*
  Retrieve output image name
*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","angcor-fatal",&fatal);

/*
  Retrieve the message time
*/
val = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*val,0);

/*
  Retrieve the min/max flag
*/
val = p_find(vblock,"$MINMAX");
strcpy(minmax,SVAL(*val,0));

status = c_getddr(hosin[0],ddr);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","angcor-fatal",&fatal);

if (ddr->valid[6] == INVAL)
    {
    c_errmsg("Projection information not valid","angcor-noproj",&nonfatal);
    c_errmsg("Fatal error encountered","angcor-fatal",&fatal);
    }

return;
}
