/*****************************************************************************
NAME		INIT_PROJ

PURPOSE		Initialize the projection package

VERSION    PROGRAMMER	   DATE
-------    ----------      ----
  2.0	   D. Hollaren	   11/93		Original development

*****************************************************************************/
#include "las.h"
#include "proj.h"
#include "correct.h"

/*
    Projection Information Package defines.
*/
#define RADIANS         0       /* Output projection units.             */
#define NULZONE         62      /* Output zone code; GCTP constant 	*/

FUNCTION init_proj(ddr,proj)

struct DDR  *ddr;		/* DDR structure			*/
struct PROJ *proj;		/* Projection information		*/
{
double proj_coord[8];		/* Projection coordinates		*/

long	i,			/* Array variable			*/
	prt_flag,		/* Print flag				*/
	prtprm[3],		/* Print parameters flag		*/
	fatal = -1,		/* Fatal error flag			*/
	image_coord[8];		/* Image coordinates line/ sample	*/

/*
  Set the image projection corners
*/
proj_coord[0] = ddr->upleft[0];
proj_coord[1] = ddr->upleft[1];
proj_coord[2] = ddr->loleft[0];
proj_coord[3] = ddr->loleft[1];
proj_coord[4] = ddr->upright[0];
proj_coord[5] = ddr->upright[1];
proj_coord[6] = ddr->loright[0];
proj_coord[7] = ddr->loright[1];

/*
  Find the transformation coefficients mapping image coordinates to
  projection coordinates.
*/
image_coord[0] = image_coord[1] = image_coord[3] = image_coord[4] = 1;
image_coord[2] = image_coord[6] = ddr->nl;
image_coord[5] = image_coord[7] = ddr->ns;
prt_flag = FALSE;

if (c_rtcoef(proj_coord,image_coord,proj->coefs,&prt_flag) != E_SUCC)
    c_errmsg("Fatal error encountered","angcor-fatal",&fatal);

if (c_ckunit(ddr->proj_units,&proj->inunit) != E_SUCC)
    c_errmsg("Fatal error encountered","angcor-fatal",&fatal);

proj->inproj  = ddr->proj_code;
proj->inzone  = ddr->zone_code;
proj->indatum = ddr->datum_code;
proj->oproj   = GEO;
proj->ounit   = RADIANS;
proj->ozone   = NULZONE;
proj->odatum  = ddr->datum_code;

for (i = 0; i < 15; i++)
    {
    proj->inparm[i] = ddr->proj_coef[i];
    proj->oparm[i]  = 0.0;
    }

prtprm[0] = prtprm[1] = FALSE;

if (c_transinit(&proj->inproj, &proj->inunit, &proj->inzone, &proj->indatum,
	     proj->inparm, &proj->oproj, &proj->ounit, &proj->ozone,
	     &proj->odatum, proj->oparm, prtprm, "") != E_SUCC)
    c_errmsg("Fatal error encountered","angcor-fatal",&fatal);

return;
}
