/****************************************************************************
NAME:		NEWANG

PURPOSE:	Calculate the corrected satellite zenith and relative
		azimuth angles

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 D. Hollaren   11/93	Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "correct.h"

double newang(pnt_lat,old_zen,old_relaz,new_zen,new_relaz)

double	pnt_lat,		/* Latitude of the point		    */
	old_zen,		/* Old satellite zenith			    */
	old_relaz,		/* Old relative azimuth 		    */
	*new_zen;		/* New satellite zenith			    */

float	*new_relaz;		/* New relative azimuth			    */
{
double	old_scan,		/* Old scan angle			    */
	old_psi,		/* Old great circle from point to satellite */
	sat_lat,		/* Latitude of the satellite		    */
	old_lat,		/* Latitude of the old point		    */
	new_scan,		/* New scan angle			    */
	new_psi,		/* New great circle from point to satellite */
	old_sataz,		/* Old satellite azimuth		    */
	new_sataz,		/* New satellite azimuth		    */
	sun_az;			/* Old solar azimuth			    */

float	relaz;			/* New relative azimuth			    */

/*
  Calculate the scan angle used to generate the old satellite zenith
*/
old_scan = asin((REQUA * sin(old_zen)) / (REQUA + SAT_ALTITUDE));

/*
  Find the scan angle used to generate the new satellite zenith
*/
new_scan = old_scan - DELTA_SCAN;

/*
  Calculate the great circle from the point to the satellite
*/
old_psi = old_zen - old_scan;

/*
  Calculate the new satellite zenith
*/
*new_zen = asin(((REQUA + SAT_ALTITUDE) * sin(new_scan)) / REQUA);

/*
  Calculate the new great circle from the point to the satellite
*/
new_psi = *new_zen - new_scan;

/*
  Calculate the latitude of the satellite
*/
sat_lat = asin((cos(KAPPA) * sin(new_psi) + sin(pnt_lat)) / cos(new_psi));

/*
  Calculate the latitude of the old point
*/
old_lat = asin(cos(old_psi) * sin(sat_lat) - cos(KAPPA) * sin(old_psi));

/*
  Calculate the old satellite azimuth
*/
if (old_psi != 0.0)
    {
    old_sataz = acos((sin(sat_lat) - cos(old_psi) * sin(old_lat)) /
	   (cos(old_lat) * sin(fabs(old_psi))));
    }
else
    old_sataz = 0.0;

if (old_psi > 0.0)
    old_sataz = PIE2 - old_sataz;

/*
  Calculate the new satellite azimuth
*/
if (new_psi != 0)
    {
    new_sataz = acos((sin(sat_lat) - cos(new_psi) * sin(pnt_lat)) /
		     (cos(pnt_lat) * sin(fabs(new_psi))));
    }
else
    new_sataz = 0.0;

if (new_psi > 0.0)
    new_sataz = PIE2 - new_sataz;

/*
  Extract the solar azimuth from the relative azimuth value.  The solar
  azimuth dosen't change very much so the old value is good enough.
*/
sun_az = old_relaz + old_sataz;

/*
  The new relative azimuth is the old solar azimuth minus the new
  satellite azimuth.
*/
if ((old_psi == 0.0) || ((new_psi < 0.0) && (old_psi > 0.0)))
    relaz = (float)(PIE - old_relaz);
else
    relaz = (float)(sun_az - new_sataz);

if (relaz < 0.0)
    relaz = fabs(relaz);

if (relaz > PIE)
    *new_relaz = PIE2 - relaz;
else
    *new_relaz = relaz;

return;
}
