/****************************************************************************
NAME:		OPEN_IMG

PURPOSE:	Open the input and output images

PROJECT:  ADAPS

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "correct.h"

FUNCTION open_img(vblock,hosin,hosout,nmrimg,window,bands,nbands,totbnd,odtype)

struct PARBLK *vblock;	/*  TAE parameter block			     */

char	hosin[][CMLEN],	/* host names of input images		     */
	*hosout;	/* host name of output image		     */

long	nmrimg,		/* number of input images                    */
	window[][4],	/* window specification of input images      */
	bands[][4],	/* bands to be processed for all images      */
	nbands[],	/* Number of bands per input image	     */
	totbnd,		/* total number of bands to be processed     */
	*odtype;	/* data type of output image                 */
{

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern int fdesc[5];	   /* Image file descriptors used for image I/O    */
extern int gdesc;	   /* Group descriptor used for image I/O          */
extern long offset1[4];	   /* Offsets into thebuf for input bands	   */
extern long offset2[4];    /* Offsets into "thebuf" for output bands       */
extern long curent;	   /* Current processing line		           */
extern long curcat;	   /* Current processing band		           */

extern unsigned char *thebuf;  /* Image I/O buffer			   */

long	access,		   /* Image I/O access type			   */
	bufsiz,		   /* Size of image I/O buffer                     */
	band,		   /* Current band to process			   */
	sl, ss,		   /* Starting line and sample			   */
	nl, ns,		   /* Number of lines and samples		   */
	i,		   /* Looping variable				   */
	status,		   /* Function return status code                  */
	opt[3], 	   /* Open data type conversion options		   */
	one = 1,	   /* Value of one				   */
	zero = 0,	   /* Value of zero                                */
	fatal = -1,	   /* Fatal error code			           */
	nonfatal = 1;	   /* Nonfatal error code			   */

/*
  Open input image(s)
*/
access = IREAD;	
*odtype = 0;
nl = window[0][NL];
ns = window[0][NS];
for (band = 0, i = 0; i < nmrimg; i++)
    {
    sl = window[i][SL];
    ss = window[i][SS];
    status = c_eopens(&fdesc[i],hosin[i],bands[i],&sl,&ss,&nl,&ns,
		      &offset1[band],&access,odtype,&zero,&zero);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","angcor-fatal",&fatal);

    band += nbands[i];
    }

/*
  Open output image
*/
access = IWRITE;	
opt[0] = 0;
opt[1] = NO_COMPRES;
opt[2] = 0;

status = c_eopens(&fdesc[nmrimg],hosout,&totbnd,&one,&one,&nl,&ns,offset2,
		  &access,odtype,&zero,opt);
if (status != E_SUCC)
   c_clean(vblock,"angcor",hosout,&one);

fdesc[nmrimg+1] = 0;

/*
  Allocate I/O buffer
*/
if ((thebuf = c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","angcor-alloc",&nonfatal);
   c_clean(vblock,"angcor",hosout,&one);
   }

/*
  Group images
*/
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   c_clean(vblock,"angcor",hosout,&one);

return;
}
