
/*******************************************************************************
NAME:     areainfo.c 
PURPOSE:  program main()
	  AREAINFO returns study area information 

PROGRAM HISTORY
--------------------------------------------------------------------------------
Date         Programmer       Changes
-----------  ---------------  --------------------------------------------------
Oct 1993?    David Zokaites   Original development for $DCONTRIB (ddps contrib)
Jan 1994?    David Zokaites   Moved to $LASDITTY 
Sep 1995     David Zokaites   Moved to $ADAPSCONTRIB, updated Makefile, added 
			      2nd level help, 
*******************************************************************************/

#include <stdio.h>  /* for file io */ 
#include <string.h> /* for call to strlen() */ 
#include <unistd.h> /* for call to access() */ 
#include <stdlib.h> /* for call to getenv() */ 
#include <ctype.h>  /* for call to tolower() */ 
#include "las.h"    /* for CMLEN, PARBLK, ... macros */ 
#include "typlim.h" /* for MAXLONG macro */ 
#define HEADER 6    /* number of header lines in input file */ 
#define LINELEN 200 /* max number of chars on an input line */ 
/* old style function prototypes */ 
   struct VARIABLE *p_find();  /* function to find a tae parm in pblock */ 
   void main (); 
   void get_info (); 
   void tolow (); 





void main ()
{
/* declare vars */ 
   /* input tae parms */ 
      char  infile[100],       /* input file to read, tae parm INFILE */ 
	    sarea[11];         /* study area to get info for */ 
      double pixsize;          /* pixel size in projected image */ 
   /* output tae parms */   
      char *acoors[4],         /* corner coordinates of projected image, 
				  declared char * for compatability with
				  rectify's parm coors */ 
           *liblist[4],        /* list of dcw libraries */ 
           *projname[1];       /* name of map projection */ 
      double centmer,          /* central meridian of projection */ 
	     slat, elat;       /* start & end latitude to read from tape */ 
      long   anl, ans,         /* size of projected study area image */ 
	     nlibs,            /* number of values in liblist[] */ 
	     quadrant;         /* Earth quadrant to read from tape */ 
   /* processing variables */ 
      char iline[LINELEN],     /* a line in infile */ 
	   message[CMLEN],     /* error message string, parm to c_errmsg() */
	   sa_file[LINELEN];   /* study area as read from infile */ 
      long access=IREAD,       /* input file will be opened for reading */ 
           fatal= -1,          /* fatal error, parm to c_errmsg() */ 
	   headl,              /* loop index, header line to skip */ 
	   index,              /* loop index */ 
	   std_out,            /* standard output */ 
	   zero=0;             /* value of zero */ 
      struct PARBLK pblock;    /* tae parameter block */ 
      struct VARIABLE *tparm;  /* points to input tae parm */ 
      FILE *finput;            /* input file pointer */ 
   /* allocate memory for acoors, liblist, projname.  To return a var to TAE, 
      it must be declared char *var[]. */ 
      for (index=0; index<4; index++)
      {
	 if ((acoors[index] = (char *) malloc (sizeof(char) * LINELEN)) ==NULL)
	    c_errmsg ("Unable to allocate memory", "areainfo-fatal", &fatal); 
	 if ((liblist[index] = (char *) malloc (sizeof(char) * LINELEN)) ==NULL)
	    c_errmsg ("Unable to allocate memory", "areainfo-fatal", &fatal); 
      }
      if ((projname[0] = (char *) malloc (sizeof(char) * LINELEN)) == NULL) 
	 c_errmsg ("Unable to allocate memory", "areainfo-fatal", &fatal); 
/* get input parms */ 
   c_inlas (&pblock, &std_out); 
   tparm = p_find (&pblock, "sarea");   
   strcpy (sarea, SVAL(*tparm, 0)); 
   tolow (sarea);  
   tparm = p_find (&pblock, "pixsize"); 
   pixsize = RVAL (*tparm, 0); 
   if (c_getnam (&pblock, "infile", &zero, infile, "", &access))
      c_errmsg ("Error getting value of parm INFILE.", "areainfo-fatal", 
      &fatal); 
/* open file and skip header */ 
   if ((finput = fopen (infile,"r")) == NULL)
   {
      sprintf (message, 
      "Unable to open INFILE %s for reading, check file protection.", infile); 
      c_errmsg (message, "areainfo-fatal", &fatal); 
   }
   for (headl=0; headl<HEADER; headl++)
      if (fgets (iline, LINELEN-1, finput) == NULL)
      {
	 sprintf (message, "INFILE %s contains too few header lines.", infile);
	 c_errmsg (message, "areainfo-fatal", &fatal); 
      }
/* match study area in file */ 
   for (;;)
   {
   /* read a line on the file */ 
      if (fgets (iline, LINELEN-1, finput) == NULL)
      {
	 sprintf (message, "Did not locate SAREA %s in INFILE %s", 
	 sarea, infile); 
	 c_errmsg (message, "areainfo-fatal", &fatal); 
      }
   /* compare study area on line to input sarea */ 
      if (sscanf (iline, " %s", sa_file) == EOF) 
	 continue;  /* a blank line */ 
      tolow (sa_file); 
      if (!strcmp (sa_file, sarea))
	 break;    /* found correct line of file */ 
   }
/* get study area info */ 
   get_info (iline, infile, pixsize, acoors, liblist, projname, &anl, &ans, 
    &nlibs, &quadrant, &centmer, &slat, &elat);
/* return data to tae */ 
   if (
   q_real   (&pblock, "centmer",     1, &centmer,  P_UPDATE) !=SUCCESS || 
   q_real   (&pblock, "slat",        1, &slat,     P_UPDATE) !=SUCCESS || 
   q_real   (&pblock, "elat",        1, &elat,     P_UPDATE) !=SUCCESS ||
   q_intg   (&pblock, "anl",         1, &anl,      P_UPDATE) !=SUCCESS || 
   q_intg   (&pblock, "ans",         1, &ans,      P_UPDATE) !=SUCCESS || 
   q_intg   (&pblock, "quadrant",    1, &quadrant, P_UPDATE) !=SUCCESS ||
   q_string (&pblock, "acoors",      4, acoors,    P_UPDATE) !=SUCCESS || 
   q_string (&pblock, "projname",    1, projname,  P_UPDATE) !=SUCCESS || 
   q_string (&pblock, "liblist", nlibs, liblist,   P_UPDATE) !=SUCCESS || 
   q_out (&pblock) !=SUCCESS                                            ) 
      c_errmsg ("Error returning parameter values to tae.", "areainfo-fatal", 
      &fatal); 
/* free memory and end program */ 
   free (projname[0]); 
   for (index=0; index<4; index++)
      free (liblist[index] ); 
   c_complt ("AREAINFO", "", &zero);
}





/* FUNCTION:  get study area info from input image line */
void get_info (iline, infile, pixsize, acoors, liblist, projname, anl, ans, 
     nlibs, quadrant, centmer, slat, elat)
   char *iline;       /* input:   info is in this line of data */ 
   char *infile;      /* input:   input file, for error message */ 
   double pixsize;    /* input:   pixel size in projected image */ 
   char *acoors[4];   /* output:  corner coordinates of projected image */ 
   char *liblist[4];  /* output:  list of dcw libraries */ 
   char *projname[1]; /* output:  name of map projection */ 
   long *anl;         /* output:  size of projected study area image */ 
   long *ans;         /* output:  size of projected study area image */ 
   long *nlibs;       /* output:  n values in liblist */ 
   long *quadrant;    /* output:  Earth quadrant to read from tape */ 
   double *centmer;   /* output:  central meridian of projection */ 
   double *slat;      /* output:  start latitude to read from tape */ 
   double *elat;      /* output:  end   latitude to read from tape */ 
{
/* declare variables */ 
   char *linep,          /* points to iline */ 
	message[CMLEN];  /* error message string, parm to c_errmsg() */
   double dcoors[4],     /* a double version of acoors */ 
          anld, ansd;    /* temp double version of anl and ans */ 
   long fatal= -1,       /* fatal error, parm to c_errmsg() */ 
        skips;           /* loop index, string to skip */ 
/* get first 9 vars */ 
   if (sscanf (iline, " %*s %s %lf %s %s %s %s %lf %lf %ld", projname[0], 
   centmer, acoors[0], acoors[1], acoors[2], acoors[3], slat, elat, quadrant) 
   != 9) 
   {
      sprintf (message, "One line of INFILE %s does not have enough info:  %s", 
      infile, iline); 
      c_errmsg (message, "areainfo-fatal", &fatal); 
   }
/* skip over first 10 strings */ 
   for (linep=iline, skips=0; skips<10; skips++)
   {
      for (;  (*linep==' ' || *linep=='\t') && *linep!='\0';  linep++)
	 ;
      for (;   *linep!=' ' && *linep!='\t'  && *linep!='\0';  linep++)
	 ;
   }
/* get liblist with 1-4 components */ 
   if ((*nlibs = sscanf (linep, " %s %s %s %s", liblist[0], liblist[1], 
   liblist[2], liblist[3])) < 1)
   {
      sprintf (message, "One line of INFILE %s does not have enough info:  %s", 
      infile, iline); 
      c_errmsg (message, "areainfo-fatal", &fatal); 
   }
/* check acoors */ 
   if (sscanf (acoors[0], "%lf", dcoors+0) !=1 ||
       sscanf (acoors[1], "%lf", dcoors+1) !=1 ||
       sscanf (acoors[2], "%lf", dcoors+2) !=1 ||
       sscanf (acoors[3], "%lf", dcoors+3) !=1  )
      {
	 sprintf (message, 
	 "Error converting ACOORS (%s %s %s %s) to type real.", 
	 acoors[0], acoors[1], acoors[2], acoors[3]); 
	 c_errmsg (message, "areainfo-fatal", &fatal); 
      }
   if (dcoors[0] < dcoors[2] ||
       dcoors[1] > dcoors[3]  )
      c_errmsg ("In INFILE, maxY < minY or maxX < minX", "areainfo-fatal",
      &fatal); 
/* check length of projname */ 
   if (strlen (projname[0]) > 12)
   {
      sprintf (message, "PROJNAME %s in INFILE %s has more than 12 characters", 
      projname[0], infile); 
      c_errmsg (message, "areainfo-fatal", &fatal); 
   }
/* calculate anl and ans.  In the below lines, 1 corrects for acoors being to
   the center of the pixel instead of the corner, .4999 is for round off err */ 
   anld = (dcoors[0] - dcoors[2])/pixsize + 1.4999; 
   ansd = (dcoors[3] - dcoors[1])/pixsize + 1.4999; 
   *anl = (long) anld; 
   *ans = (long) ansd; 
   if (anld > MAXLONG || ansd > MAXLONG) 
      c_errmsg (
      "Calculated ANL or ANS had a huge value.", "areainfo-fatal", &fatal);
}





/* FUNCTION:  convert input string to lower case */ 
void tolow (string) 
    char *string;  /*  input as mixed case string
		      output as lower case string */ 
{
   for (; *string!='\0'; string++)
      *string = tolower (*string); 
}




