
/******************************************************************************
NAME:     copy2.c 
PURPOSE:  program main() and a few misc functions 

PROGRAM HISTORY
-----------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  --------------------------------------------------
Jun 1994  David Zokaites   Original development for $DCONTRIB (DDPS Contrib).
			   Application was later moved to $LASDITTY. 
Sep 1994  David Zokaites   Revised to overwrite existing output images. 
Dec 1994  David Zokaites   Added parm ZEROLIM to eliminate more marginally-
			   useful images;  eliminated checking c_delfil() 
			   return code in image.c
Feb 1995  David Zokaites   Renamed parm ZEROLIM to MAXMASK;  added parms
			   SCALOPT, MASKVAL, LOGFILE;  expanded the user help;
			   updated Makefile format.
Jun 1995  David Zokaites   Made parm IN multi-valued to allow for longer input 
			   image names;  created files getalias.c and getin.c 
Jun 1995  David Zokaites   Updated getalias.c after a bug was fixed. 
Oct 1995  David Zokaites   Released application to $ADAPSCONTRIB;  reformatted
                           change log;  updated users guide;  changed
			   calculation of year.  
Mar 1996  David Zokaites   Changed end_las() so that the images would only be 
			   closed once.  
******************************************************************************/


#define GLOBALVARS     /* defined only in this file, flags copy2.h */
#include "copy2.h"     /* proc header file */ 
#include <string.h>    /* for call to various string functions */ 
#include <time.h>      /* for call to time(), localtime() */
#include <sys/types.h> /* for call to time() */
#include "util.h"




/* function prototypes, ifdef allows for K&R compilers */ 
#ifdef __DGUX__ 
   void main (void);
   static void get_par (struct UPARMS *uparms);
   static void log_file (char *status, struct UPARMS *uparms); 
#else
   void main ();
   static void get_par ();
   static void log_file ();
#endif 





/* PROGRAM MAIN */ 
void main ()                 
{	                    
/* declare vars */ 
   double totmask;       /* total number of mask pixels in cband image band */ 
   long   preexist,      /* flag, did the output image pre-exist? */ 
          zero=0;        /* value of zero */ 
   struct UPARMS uparms; /* most tae user parms */ 
/* initialize las and tae */ 
   begin_las ("copy2"); 
/* get parameters */ 
   get_par (&uparms); 
/* process images */ 
   /* open images, input must be opened before output */
      if (openrin (uparms.inimage, 0, &inimage)) 
      {
	 /* log error opening input image and complete */ 
	 log_file ("bad-inimage", &uparms); 
	 c_complt(procname, "", &zero);
      }
      if (uparms.cband+1 > inimage.totalbands)
	 c_errmsg (
	 "The value of CBAND is greater than the number of output image bands.",
	 procfatal, &fatal); 
      preexist = openrout (FALSE, &inimage, &outimage); 
   /* create image */ 
      totmask = copy_image (&uparms, &inimage, &outimage); 
   /* close image files and free buffers */ 
      close_inimage  (&inimage); 
      close_outimage (&outimage); 
/* if image does not have too much mask data, 
   write to log file, make ddr and complete */ 
   if (totmask / (outimage.nl * outimage.ns) <= uparms.maxmask / 100.0)
   {
      log_file (preexist ? "pre-created" : "new-created",  &uparms); 
      end_las (&inimage, &outimage); 
   }
/* too much mask data:  write to log, delete image and complete */ 
   log_file (preexist ? "pre-notmade" : "new-notmade",  &uparms); 
   c_delfil (outimage.hostname);
   sprintf (message, "COPY2 did not create output image %s.",
    outimage.taename);
   c_errmsg (message, "informational", &go_on); 
   c_complt(procname, "", &zero);
}





/* FUNCTION:  get and return most TAE user parms */ 
static void get_par (uparms)
   struct UPARMS *uparms;   /* output:  tae parms */ 
{
/* declare vars */ 
   long dataval,            /* for loop index, data value */ 
	dupe,               /* flag, was a duplicate mask value found */ 
	nocheck=IUPDATE,    /* file access mode, arg to c_getnam() */
	pcount,             /* for loop index, parameter count */ 
        topmask,            /* greatest mask value given by parm scalopt */
	umasks,             /* number of user specified mask values */ 
	uval,               /* for loop index, user specified mask value */  
	zero=0;             /* value of zero */ 
   struct VARIABLE *tparm;  /* points to a tae parm in pblock */ 
/* get cband and maxmask parms */ 
   tparm = p_find (&pblock, "cband");     
   uparms->cband = IVAL (*tparm, 0); 
   uparms->cband--;  /* convert from 1 relative to 0 relative */ 
   tparm = p_find (&pblock, "maxmask");   
   uparms->maxmask = IVAL (*tparm, 0); 
/* get logfile (parm is nullable) */ 
   tparm = p_find (&pblock, "logfile");
   uparms->makelog = (*tparm).v_count;
   if (uparms->makelog)
   {
      uparms->taelog = SVAL (*tparm,0); 
      if (c_cmhost (uparms->taelog, uparms->hostlog)) 
      {
	 sprintf (message, "Error converting tae name %s to host name.",
	  uparms->taelog);
	 c_errmsg (message, procfatal, &fatal); 
      }
   }
/* get input image name */ 
   tparm = p_find (&pblock, "in");
   /* null out parm value */ 
      uparms->inimage[0] = '\0'; 
   /* get all values for IN, separate them with " + " */ 
      for (pcount=0;  pcount<tparm->v_count;  pcount++)
      {
	 strcat (uparms->inimage, SVAL (*tparm, pcount) );
	 if (pcount != tparm->v_count -1)
	    strcat (uparms->inimage, " + ");
      }
   /* user message, c_errmsg() core dumps for big strings */
      if (strlen(uparms->inimage) + strlen("Input image is:  ") < CMLEN)
      {
	 sprintf (message, "Input image is:  %s", uparms->inimage);
	 c_errmsg (message, "informational", &go_on);
      }
      else 
	 printf ("[informational] Input image is:  \"%s\"\n", uparms->inimage);
/* get output image name, 
   this logic is duplicated in openrout(),
   file access is checked in openrout(), 
   don't call c_getout() as it gives an error if the image already exists. */
   /* determine host name */ 
      if (c_getnam (&pblock, "out", &zero, uparms->hostouti, "img", &nocheck))
	 c_errmsg ("Error getting output image OUT.", procfatal, &fatal); 
   /* determine tae name */
      strcpy(uparms->taeouti, uparms->hostouti);
/* set mask values determined by parm scalopt */ 
   /* get tae parm */ 
      tparm = p_find (&pblock, "scalopt");   
      uparms->scalopt = SVAL (*tparm, 0); 
      if      (!strcmp (uparms->scalopt, "GLOBAL"))
	 topmask = 9;
      else if (!strcmp (uparms->scalopt, "US"    ))
	 topmask = 0;
      else if (!strcmp (uparms->scalopt, "NONE"  ))
	 topmask = -1;  /* use no mask values */ 
      else
	 c_errmsg ("Program error in get_par().", procfatal, &fatal); 
   /* set mask values, n mask values */ 
      for (dataval=0;  dataval<=topmask;  dataval++)
	 uparms->maskval[dataval] = dataval;
      uparms->nmasks = topmask +1; 
/* get maskval, combine maskval with scalopt masks */ 
   tparm = p_find (&pblock, "maskval");
   umasks = (*tparm).v_count;
   for (uval=0; uval<umasks; uval++)
   {
      /* look for duplicate masks, logic in count_mask() will fail if there are
	 duplicate mask values. */ 
	 for (dupe=FALSE, dataval=0; dataval<uparms->nmasks; dataval++)
	    if (uparms->maskval[dataval] == RVAL (*tparm, uval) )
	    {
	       dupe = TRUE;
	       break;
	    }
      /* optionally add this mask to the combined list */ 
	 if (!dupe)  
	 {
	    uparms->maskval[ uparms->nmasks ] = RVAL (*tparm, uval); 
	    uparms->nmasks++;
	 }
   }
/* echo combined maskval */ 
   if (uparms->nmasks)
   {
      strcpy (message, "Combined mask values are: "); 
      for (dataval=0; dataval<uparms->nmasks; dataval++)
	 sprintf (message+strlen(message), " %.2f", uparms->maskval[dataval]);
      c_errmsg (message, "informational", &go_on); 
   }
   else
      c_errmsg ("No mask values will be used during processing.", procwarn, 
      &go_on); 
}





/* FUNCTION:  optionally write a status line to the log file */ 
static void log_file (status, uparms)
   char *status;           /* input:  status key to write */ 
   struct UPARMS *uparms;  /* input:  structure of user parms */ 
{
/* declare variables */ 
   FILE *filep;                  /* points to output file */ 
   static char time_string[30];  /* local system time as a string */
   long exist;                   /* flag, did the log file pre-exist */ 
   time_t green_time;            /* Greenwich system time */
   struct tm *timestruct;        /* time structure used for local time */
/* return if there is no log to make */ 
   if (!uparms->makelog)
      return; 
/* get system date */
   /* convert greenwich time to local time */ 
      if (time (&green_time) == -1)
	 c_errmsg ("Error returned from time().", procfatal, &fatal); 
      timestruct = localtime (&green_time);
   /* add year to system year.  what happens in year 2000? */ 
   /*
     DLLOYD Y2K -- in year 2000 tm_year = 100 therfore this is okay
    ----------------------------------------------------------------*/
      timestruct->tm_year += 1900;
   /* create time string */ 
      sprintf (time_string, "%4d-%02d-%02d-%02d:%02d", timestruct->tm_year,
       timestruct->tm_mon + 1, timestruct->tm_mday, timestruct->tm_hour, 
       timestruct->tm_min);
/* see if file exists */ 
   exist = ! access (uparms->hostlog, 0);
/* open file */ 
   if ((filep = fopen (uparms->hostlog, "a")) == NULL)
   {
      sprintf (message, "Unable to open LOGFILE %s for appending.",
       uparms->taelog); 
      c_errmsg (message, procfatal, &fatal); 
   }
/* optionally write file header */ 
   if (!exist)
   {
      fprintf (filep, " COPY2 Run Date     Status            Output Image Name \n");
      fprintf (filep, "----------------  -----------  ------------------------------- \n");
   }
/* write to log */ 
   fprintf (filep, "%16s  %-11s  %s \n", time_string, status, uparms->taeouti);
   fclose (filep); 
/* completion message */ 
   sprintf (message, "%s log file %s",   exist ? "Updated" : "Created",  
    uparms->taelog); 
   c_errmsg (message, "informational", &go_on); 
}



