
/* image.h:  data structures for LAS images and function prototypes */
#include "las.h"    /* for las image IO */ 





/* data structure for one las input image, used to facilitate IO.  Because las
   image specs allow for grouping image components, (e.g. in="image1 + image2")
   one input image may correspond to multiple disk image files. */
   struct lasinimage
   {
      char hostname[MAXIMG][CMLEN],/* host name for image files */ 
           taename[MAXIMG][CMLEN]; /* tae name for image components */ 
      int  fdarray[MAXIMG+1];      /* file descriptor array */ 
      long bands[MAXIMG][MAXBND+1],/* band numbers */ 
           cblut[MAXIMG*MAXBND],   /* a lut for current band */ 
 	   dsizeproc,              /* N bytes per pixel as seen by the proc */
 	   dtypeproc,              /* image data type as seen by the proc */ 
 	   dtypein,                /* image data type as read = EBYTE, ... */
           fdlut[MAXIMG*MAXBND],   /* a lut for file descriptor */
           nbands[MAXIMG],         /* number of bands in each component */ 
           ncomponents,            /* number of image components or disk
	                              image files */ 
           nl, ns,                 /* size of input image = window[0][NL], 
				      window[0][NS] */ 
           totalbands,             /* total number of bands */ 
           window[MAXIMG][4];      /* window (sl ss nl ns) for each component*/
      unsigned char *buffer;       /* buffer for image data */ 
   }; 




 
/* data structure for one las output image, used to facilitate IO */ 
   struct lasoutimage
   {
      char   hostname[CMLEN],      /* host name for image file */ 
             taename[CMLEN];       /* tae name for image */ 
      int  fdarray[1+1];           /* file descriptor array */ 
      long dsizeproc,              /* N bytes per pixel as seen by the proc */
 	   dtypeproc,              /* image data type as seen by the proc */ 
 	   dtypeout,               /* image data type to write = EBYTE, ... */
           minmax,                 /* flag, determine min/max for ddr? = T,F */
           nl, ns,                 /* size of output image */ 
           totalbands,             /* total number of bands */ 
	   truncate;               /* flag, was the output data truncated to
	                              fit in the allowed range */ 
      unsigned char *buffer;       /* buffer for image data */ 
      double minval[MAXBND],       /* min pixel value, used to update ddr */ 
             maxval[MAXBND];       /* max pixel value, used to update ddr */ 
   }; 





/* function prototypes.  ifdef allows for old compilers which do not support 
   prototype checking */ 
#ifdef __DGUX__
   /* some las image IO functions */ 
      unsigned char * c_ealloc (int *fdarray, long *buffersz); 
      long c_egclse (int *gdesc); 
      long c_egroup (int *fdarray, int *gdesc, unsigned char *buffer, 
	   long *buffersz); 
      long c_eopens (int *fdesc, char *fname, long bands[], long *sl, long *ss, 
	   long *nl, long *ns, long offset[], long *acc, long *dtype, 
	   long *nbuf, long opt[]); 
      long c_estep (int *gdesc); 
      long c_eclose (int *fdesc); 
      long c_eopenr (int *fdesc, char *fname, long *nbands, long *sl, long *ss,
	   long *nl, long *ns, long *acc, long *dtype, long opt[], long *nline);
      long c_eread (int *fdesc, long *eband, long *eline, unsigned char 
	   *euserbuf, long *nl_blk); 
      long c_ewrite (int *fdesc, long *eband, long *eline, unsigned char 
	   *euserbuf, long *nl_blk); 
   /* some las pixel manipulation functions */ 
      long c_pxconv (long *fromtype, long *totype, unsigned char *frombuf,
	   unsigned char *tobuf, long *ns); 
      long c_pxmnmx (unsigned char *in, long *ns, double *min, double *max, long
	   *dtype);
      void c_pxcopy (unsigned char *frombuf, unsigned char *tobuf, long *dtype,
	   long *ns);
   /* some las ddr functions */ 
      long c_svmnmx (char *hname, long *totbnd, double minval[], double
	   maxval[], char *m_flag); 
      long c_upddr (struct PARBLK *parblk, char host_in[][CMLEN], char
	   *host_out, long *nmrimg, long window[][4], long bands[][MAXBND+1], 
	   long nbands[], double *line_inc, double *samp_inc, long *out_nbands,
	   long *cflag); 
   /* a las utility function */ 
      void c_errmsg (char *message, char *key, long *severity); 
   /* functions in image.c */ 
      void begin_las (char *proc); 
      void close_inimage (struct lasinimage *ini); 
      void close_outimage (struct lasoutimage *outi); 
      void errclean (char *message); 
      long openrin (char *parmname, long dtypeproc, struct lasinimage *ini); 
      long openrout (long dtypeproc, struct lasinimage *ini, 
	   struct lasoutimage *outi); 
      void rread (long curline, long curband, struct lasinimage *ini); 
      void rwrite (long curline, long curband, struct lasoutimage *outi); 
      void convertr (struct lasinimage *ini, struct lasoutimage *outi); 
      void minmaxr (long band, struct lasoutimage *outi); 
      void end_las (struct lasinimage *ini, struct lasoutimage *outi);
#endif  






