
/*******************************************************************************
NAME:     comlogs.c 
PURPOSE:  functions to execute the comlogs subcommand;  
	  combine two-short term logs 
*******************************************************************************/

#include "datelog.h"  // application header file  
#include <stdio.h>    // for file output  
#include <string.h>   // for calling strcpy()  
#include <unistd.h>   // for calling access()  





/*******************************************************************************
NAME:     comlogs()
PURPOSE:  execute the comlogs subcommand.  TAE parms are implicit inputs  
*******************************************************************************/

FUNCTION void comlogs (void) 
{
// declare variables  
   char  curlog[CMLEN],    // host name for current log file  
 	 datefile[CMLEN],  // host name for TAE parm datefile  
	 namebase[CMLEN],  // host name for TAE parm namebase, file name base 
	 oldlog[CMLEN],    // host name for old log file  
	 oldname[PNAME],   // name for previous composite period  
        *pername,          // TAE parm pername, composite period name  
	 twolog[CMLEN],    // host name for combined log file  
	 twonames[PNAME*2+4]; /* name of combined composite period, 2 as 2 
				 names are combined, +4 for null, '_', and good
				 luck */ 
   long nocheck=IUPDATE,   // file access mode, parm to c_getnam() 
        pernum,            // period number as determined by datelog  
	readacc=IREAD,     // file read access, parm to c_getnam()  
	zero=0;            // value of zero  
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

/* get TAE parms.  parm prot is acquired in set_prot() in functions.c 
********************************************************************/
   // get pername  
      tparm = p_find (&pblock, "pername");  
      pername = SVAL(*tparm,0); 

   // get datefile, namebase  
      if (c_getnam(&pblock,"datefile",&zero, datefile, "", &readacc)!=E_SUCC || 
	  c_getnam(&pblock,"namebase",&zero, namebase, "", &nocheck)!=E_SUCC  ) 
	 errclean ("datelog-fatal", "Fatal error encountered");

   // check input parms for imbedded white space and zero-length  
      check_parm ("PERNAME", pername); 

   // lowercase parms to make later string comparisons case-insensitive  
      strlower (pername); 

/* calculate data 
****************/
   get_data (datefile, pername, oldname, &pernum); // get oldname, pernum 
   /* check for being period one.  check must follow get_data() and precede 
      combining period names */ 
      if (pernum == FIRSTPERIOD)
	 {
	 c_lasmsg (FALSE, "datelog-period1", 
		  "A combined short-term log was not created because this %s",
		  "is the first composite period");
	 return; 
	 } 
   get_log_name (namebase, pername, curlog);      // get curlog name  
   get_log_name (namebase, oldname, oldlog);      // get oldlog name 
   sprintf (twonames, "%s_%s", oldname, pername); // combine period names  
   get_log_name (namebase, twonames, twolog);     // get twolog name  

/* output data
*************/
   // enable file locking before writing data 
      make_lock (namebase); 

   // combine 2 files 
      combine (oldlog, curlog, twolog, twonames);
}










/*******************************************************************************
NAME:     combine()
PURPOSE:  combine 2 files to create a 3rd file, all arguments are inputs  
*******************************************************************************/

FUNCTION void combine (
   char *infile1,  // first input file 
   char *infile2,  // second input file 
   char *outfile,  // name of output file 
   char *twonames) // name of combined composit period  
{

// declare variables  
   char line[LINELEN];               // a line in a file  
   FILE *input1, *input2, *output;   // file pointers  
   long header;                      // loop index, header lines to skip 

// check file existence  
   if (access (infile1, 0))
      errclean ("datelog-nolog", "Log file %s does not exist", 
	       host_tae(infile1));
     
   if (access (infile2, 0))
      errclean ("datelog-nolog", "Log file %s does not exist", 
	       host_tae(infile2));

   if (!access(outfile,0) )
      c_lasmsg (FALSE, "datelog-replaced", 
	       "Combined log file %s already existed and was replaced", 
	       host_tae(outfile) ); 
        
// open files  
   if ( (input1=fopen(infile1,"r")) == NULL)
      errclean ("datelog-open", "Unable to open file %s for reading", 
	       host_tae(infile1));

   if ( (input2=fopen(infile2,"r")) == NULL)
      {
      fclose (input1); 
      errclean ("datelog-open", "Unable to open file %s for reading", 
	       host_tae(infile2));
       
      }

   if ( (output=fopen(outfile,"w")) == NULL)
      {
      fclose (input1); 
      fclose (input2); 
      errclean ("datelog-open", "Unable to open file %s for writing", 
	       host_tae(outfile));
      }

// skip header of both input files  
   for (header=0;  header<LOGHEADER;  header++)
      {
      if (fgets (line, LINELEN, input1) == NULL)
	 errclean ("datelog-fileend", 
		  "End of file while reading short-term log %s", 
		  host_tae(infile1)); 

      if (fgets (line, LINELEN, input2) == NULL)
	 errclean ("datelog-fileend", 
		  "End of file while reading short-term log %s", 
	          host_tae(infile2)); 
      }

// write new header  
   sprintf (line, "Processing Log for Combined Period %s", twonames); 
   file_header (output, line); 

// copy data  
   for (;  fgets(line,LINELEN,input1) !=NULL;  )
       if (fputs (line, output) == EOF)
	  errclean ("datelog-badwrite", "Error writing to file %s",
		   host_tae (outfile)); 

   for (;  fgets(line,LINELEN,input2) !=NULL;  )
       if (fputs (line, output) == EOF)
	  errclean ("datelog-badwrite", "Error writing to file %s",
		   host_tae (outfile)); 

   fclose (input1);
   fclose (input2);
   fclose (output); 

// set protection of new file  
   set_prot (outfile); 

// update list of new files  
   strcpy (outfiles.newfiles[outfiles.newcount++], outfile); 
}



