
/*******************************************************************************
NAME:     composit.c
PURPOSE:  functions to execute the composit subcommand; 
	  log information related to a composite of multiple AVHRR scenes.
*******************************************************************************/

#include "datelog.h"  // application header file  
#include <string.h>   // for calling strcpy() 
#include <unistd.h>   // for calling chown()  






/*******************************************************************************
NAME:     composit()
PURPOSE:  execute the composit subcommand, TAE parms are implicit inputs  
*******************************************************************************/

FUNCTION void composit (void) 
{
// declare variables  
   char *pername,        // TAE parm pername, composite period name  
	*dateind,        // TAE parm dateind, place holder for date index  
	*descrpt,        // TAE parm descrpt, description of composite 
         dbname[CMLEN],  // final host name of dbname file  
	 namebase[CMLEN],// host name for TAE parm namebase, file name base  
	*pstatus,        // TAE parm pstatus, processing status  
        *sarea,          // TAE parm sarea, study area  
	*sysdate,        // system date  
	*swindow,        // TAE parm swindow, place holder for scene window  
	*writshor;       // TAE parm writshor, flag: write to short-term log 
   long parmnum,         // loop index, parameter number  
	nocheck=IUPDATE, // file access mode, parm to c_getnam() 
	readacc=IREAD,   // file read access, parm to c_getnam()  
	zero=0;          // value of zero  
   struct VARIABLE *tparm;  // points to a TAE parm in pblock  
   struct fparm curlog,     // host name for current log file  
                dbname0,    /* TAE parm dbname, name for database file
                               as received from c_getnam() */
                permlog;    // host name for TAE parm permlog  
   struct iparm dbuid;      // TAE parm dbuid, uid for dbname file 
   struct sparm cbands,     // TAE parm cbands, composite bands  
                tapeid;     // TAE parm tapeid, composite tape id's  

/* get TAE parms.  parm prot is acquired in set_prot() in functions.c 
********************************************************************/

   // get most TAE parms  
      tparm = p_find (&pblock, "dateind");   dateind  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "descrpt");   descrpt  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "pername");   pername  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "pstatus");   pstatus  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "sarea");     sarea    = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "swindow");   swindow  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "writshor");  writshor = SVAL(*tparm,0); 

   // lowercase parms to make later string comparisons case-insensitive  
      strlower (pername); 

   // get file names  
      name_parm    ("permlog", readacc, &permlog); 
      if (c_getnam (&pblock, "namebase", &zero, namebase, "", &nocheck)!=E_SUCC)
	 errclean ("datelog-fatal", "Fatal error encountered");

   // get tapeid and cbands parms  
      string_parm  ("tapeid", &tapeid); 
      string_parm  ("cbands", &cbands); 
      // error check 
	 if (cbands.count != tapeid.count)
	    errclean ("datelog-count", "CBANDS was given a %s", 
		     "different number of values than TAPEID");

   // get dbname and dbuid  
      integer_parm ("dbuid",  &dbuid); 
      name_parm    ("dbname", nocheck, &dbname0);
      if (dbuid.count != dbname0.count) 
	 errclean ("datelog-badparm", 
		  "DBNAME and DBUID must both have values or be set to NULL"); 

   // check input parms for imbedded white space and zero-length  
      check_parm ("DATEIND", dateind);
      check_parm ("DESCRPT", descrpt);
      check_parm ("PERNAME", pername);
      check_parm ("PSTATUS", pstatus);
      check_parm ("SAREA",   sarea  );
      check_parm ("SWINDOW", swindow);
      for (parmnum=0;  parmnum<tapeid.count;  parmnum++)
	 check_parm ("TAPEID", tapeid.value[parmnum] ); 
      for (parmnum=0;  parmnum<cbands.count;  parmnum++)
	 check_parm ("CBANDS", cbands.value[parmnum] ); 

   // determine curlog   
      if (*writshor=='Y' || *writshor=='y')  // y for yes  
	 curlog.count = 1;
      else
	 curlog.count = 0;

      if (curlog.count)
	 get_log_name (namebase, pername, curlog.value); 

   // make sure user selected a file to write to  
      if (!curlog.count && !permlog.count && !dbname0.count)
	 errclean ("datelog-nofiles", "No files will be updated because "
		   "WRITSHOR=NO, PERMLOG=--, and DBNAME=--"); 

   // get system date  
      sysdate = system_date (FALSE);
      
/* enable file locking before writing data 
*****************************************/
   make_lock (namebase); 

/* optionally write to curlog and permlog 
****************************************/
   // don't call set_prot() as these logs are not new files  
   if (curlog.count)
      write_cdata (curlog.value,  sysdate, pername, sarea, descrpt, &cbands, 
		  pstatus, &tapeid, dateind, swindow);
   if (permlog.count)
      write_cdata (permlog.value, sysdate, pername, sarea, descrpt, &cbands, 
		  pstatus, &tapeid, dateind, swindow); 

/* optionaly write to dbname file 
********************************/
   if (dbname0.count) 
      {
      // get name of database file  
	 dbase_name (dbname0.value, "composite", dbname);
      // write to file  
	 write_cdata (dbname, sysdate, pername, sarea, descrpt, &cbands, 
		     pstatus, &tapeid, dateind, swindow);
      // set protection of new file  
	 set_prot (dbname); 
      // change owner  
	 if (chown (dbname, dbuid.value[0], -1))
	    errclean ("datelog-chown",
		     "Unable to change owner of database file %s", 
	             host_tae(dbname)); 
      }

/* increment count of output files 
*********************************/
   if (curlog.count)
      strcpy (outfiles.upfiles [outfiles.upcount++], curlog.value ); 
   if (permlog.count) 
      strcpy (outfiles.upfiles [outfiles.upcount++], permlog.value); 
   if (dbname0.count) 
      strcpy (outfiles.newfiles[outfiles.newcount++], dbname); 
}









/*******************************************************************************
NAME:     write_cdata()
PURPOSE:  write composite data to a log file, all arguments are inputs  
*******************************************************************************/

FUNCTION void write_cdata (
   char *logname,               // name of file to write to  
   const char *sysdate,         // system date  
   const char *pername,         // composite period name  
   const char *sarea,           // study area  
   const char *descrpt,         // description of the composite  
   const struct sparm *cbands,  // composite bands  
   const char *pstatus,         // processing status  
   const struct sparm *tapeid,  // composite tape id's  
   const char *dateind,         // place holder for date index  
   const char *swindow)         // place holder for scene window  
{

// declare variables  
   long tcount;  // loop index, tape id count  
   FILE *filep;  // points to output file  

// check file existence  
   if (access (logname, 0))
      errclean ("datelog-nolog", "Log file %s does not exist", 
	       host_tae(logname));

// open file  
   if ((filep = fopen (logname, "a")) == NULL)
      errclean ("datelog-open", "Unable to open file %s for appending",
	       host_tae(logname));

// write to file  
   for (tcount=0;  tcount<tapeid->count;  tcount++)
      {
      fprintf (filep, "%-9s",    pername               ); 
      fprintf (filep, "%-8s",    sarea                 ); 
      fprintf (filep, "%-7s",    dateind               );  
      fprintf (filep, "%-8s",    descrpt               ); 
      fprintf (filep, "%8s  ",   cbands->value[tcount] ); 
      fprintf (filep, "%-18s",   sysdate               ); 
      fprintf (filep, "%-16s",   pstatus               ); 
      fprintf (filep, "%-8s",    tapeid->value[tcount] ); 
      fprintf (filep, "%s\n",    swindow               ); 
      }

fclose (filep); 
}


