
/*******************************************************************************
NAME:     datelog.h  
PURPOSE:  application header file 
*******************************************************************************/



extern "C"
{
#include "las.h"       // for las and TAE  
#include "util.h"
}

#define DATEHEADER  4  // datefile has 4 header lines  
#define LOGHEADER   5  /* log files have LOGHEADER header lines, database files 
		 	  have 0 header lines */ 

#define FIRSTPERIOD 1  // first composite period is period number 1  
#define LINELEN   300  // max length of a line in a log file  
#define MAXF       40  // max number of output files  
#define PNAME       8  // length of string for pername parm, incl null  
#define BADSWINDOW -1  /* flag for TAE parm swindow as a struct iparm:
                          ignore swindow;  value should be < 0 */ 
#define NOTINCOMP  -1  /* Value for date index to note that a scene was not
                          entered in the composite.  Value is referenced in 
			  the users guide */ 





// data structure to keep track of output names  
   struct OUTFILES
   {
      char upfiles [MAXF][CMLEN], // names of files updated 
	   newfiles[MAXF][CMLEN]; // names of new files created 
      long upcount,               // number of files updated 
	   newcount;              // number of new files created  
   }; 

// data structures for parameters with multiple values  
   // file name parms  
      struct fparm
      {
	 long count;           // number of values for this parm = 0 or 1  
	 char value[CMLEN];    // parm value  
      };
   // integer parms  
      struct iparm
      {
	 long count;          // number of values for this parm  
	 long value[MAXVAL];  // parm values  
      };
   // string parms  
      struct sparm
      {
	 long count;           // number of values for this parm  
	 char *value[MAXVAL];  // parm values  
      };





// global variables, declared global for convenience  
#ifdef GLOBALVARS
   struct OUTFILES outfiles; // files that were updated or created  
   struct PARBLK pblock;     // TAE parameter block  
#else
   extern struct OUTFILES outfiles; 
   extern struct PARBLK pblock; 
#endif 





// prototype local functions 
   // comlogs.C 
      void comlogs (void); 
      void combine (char *infile1, char *infile2, char 
	   *outfile, char *twonames); 
   // composit.C 
      void composit (void); 
      void write_cdata (char *logname, const char *sysdate, const char 
	   *pername, const char *sarea, const char *descrpt, const struct 
	   sparm *cbands, const char *pstatus, const struct sparm *tapeid, 
	   const char *dateind, const char *swindow); 
   // datelog.C 
      int main (void); 
   // function in errclean.C 
      void errclean (char *mkey, char *format, ...);
   // functions.C  
      void init_count (void); 
      void integer_parm (char *parmname, struct iparm *parmval);
      void name_parm (char *parmname, long access, 
	   struct fparm *parmval);
      void string_parm (char *parmname, struct sparm *parmval);
      void check_parm (const char *parmname, const char *value);
      void get_data (char *datefile, char *pername, char *oldname, 
	   long *pernum);
      void get_dateind (const char *scalopt, const char *sceneid, 
	   const char *groupper, char *curlog, const long pernum, 
	   long *dateind);
      void get_log_name (char *namebase, const char *pername, char *logname);
      void dbase_name (char *basename, const char *extension, char *dbname);
      char * host_tae (char *hostname);
      char * system_date (const long seconds);
      void strlower (char *string);
      void set_prot (char *infile, const long lockmode=FALSE);
      void write_output (const long nfiles, char outnames[MAXF][CMLEN], 
	   const char *action);
      void file_header (FILE *filep, const char *line1);
      void write_log (char *logfile, const char *sysdate, const char
	   *pername, const char *sarea, const long dateind, const char
	   *sceneid, const char *tapeid, const struct iparm *swindow, const
	   char *swstring, const char *pstatus, const long newfile);
      void free_memory (void); 
   // getdate.C 
      void get_date (void); 
   // lock.C  
      void make_lock (const char *lockbase);
      void make_name (const char lockbase[CMLEN], char lockfile[CMLEN]); 
   // makeperm.C 
      void makeperm (void); 
   // makeshort.C 
      void makeshort (void); 
      void makecur (char *curlog, const char *pername);
   // scenein.C 
      void scenein (void); 
   // sceneout.C 
      void sceneout (void); 
      void get_scene (const long dateind, char *curlog, char *sceneid);
