
/*******************************************************************************
NAME:     errclean() 
PURPOSE:  errclean is called to clean up in event of a fatal error.  
          This function accepts a variable number of input arguments much like 
	  c_lasmsg() does.  
*******************************************************************************/
 
#include <string.h>    // for calling strcmp() 
#include <stdarg.h>    // for variable number of arguments  
#include "datelog.h"   // application header file  




FUNCTION void errclean (
   char *mkey,      // message key, parm to c_lasmsg()  
   char *format,    // vsprintf format string for all additional arguments
   ...)
{

// declare variables  
   char message[3*ERRLEN];  // text of message printed by c_lasmsg() 
   va_list argumentp;       // lists a variable number of function arguments  

// get input variables and format message string  
   va_start (argumentp, format);  
   vsprintf (message, format, argumentp); 
   va_end (argumentp);

// user message, should be early in function in case something else has an error
   // prevent writing the same error message twice 
   if ( strcmp (mkey,"datelog-fatal")  ||  
   strcmp (message,"Fatal error encountered"))
      c_lasmsg (FALSE, mkey, message); 

/* if database files are created, they should never be deleted 
   // optionally delete new files  
      long filenum;            // loop index, output files  
      struct VARIABLE *tparm;  // points to $delflg in pblock  
      tparm = p_find (&pblock, "$delflg");
      if ( *SVAL(*tparm,0) == 'Y') // y for yes  
	 for (filenum=0;  filenum<outfiles.newcount;  filenum++)
	    c_delfil (outfiles.newfiles[filenum] );
*/ 

// free dynamically allocated memory  
   free_memory (); 

// issue fatal error and terminate processing  
   c_lasmsg (TRUE, "datelog-fatal", "Fatal error encountered"); 
}


