
/*******************************************************************************
NAME:     getdate()
PURPOSE:  execute the getdate subcommand. Determine and return scene date index.
          Tae parms are implicit inputs. 
*******************************************************************************/

#include "datelog.h"  // application header file  





FUNCTION void get_date ()
{
// declare variables  
   char curlog[CMLEN],   // host name for current log file  
        *pername,        // TAE parm pername, composite period name  
 	 datefile[CMLEN],// host name for TAE parm datefile  
        *groupper,       // TAE parm groupper, group composite period? 
	 namebase[CMLEN],/* host name for TAE parm namebase, 
			    base for file names */ 
	 oldname[PNAME], // name for previous composite period  
        *sceneid,        // TAE parm sceneid, scene id  
	*scalopt;        // TAE parm scalopt, data scaling option  
   int  date;            // integer version of dateind for q_intg()  
   long dateind,         // output TAE parm dateind, date index  
	nocheck=IUPDATE, // file access mode, parm to c_getnam() 
	readacc=IREAD,   // file read access, parm to c_getnam()  
        pernum,          // composite period number  
	zero=0;          // value of zero  
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

/* get data 
**********/

   // get TAE parms  
      tparm = p_find (&pblock, "groupper");  groupper = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "pername");   pername  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "sceneid");   sceneid  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "scalopt");   scalopt  = SVAL(*tparm,0); 
      if (c_getnam(&pblock,"datefile", &zero, datefile, "", &readacc)!=E_SUCC ||
	  c_getnam(&pblock,"namebase", &zero, namebase, "", &nocheck)!=E_SUCC  )
	 errclean ("datelog-fatal", "Fatal error encountered");

   /* check input parms for imbedded white space and zero-length, there is no
      need to check parms that have valid values */ 
      check_parm ("PERNAME", pername); 
      check_parm ("SCENEID", sceneid); 

   // lowercase parms to make later string comparisons case-insensitive  
      strlower (sceneid);
      strlower (pername); 

   // determine pernum, oldname, curlog, dateind  
      get_data (datefile, pername, oldname, &pernum); 
      get_log_name (namebase, pername, curlog); 
      get_dateind (scalopt, sceneid, groupper, curlog, pernum, &dateind);

/* return dateind to TAE 
***********************/
   date = (int) dateind; 
   if (
   q_intg (&pblock, "dateind", 1, &date, P_UPDATE) !=SUCCESS ||
   q_out (&pblock)                                 !=SUCCESS  ) 
      errclean ("datelog-totae", "Error returning data to TAE"); 

/* user message 
**************/
   c_lasmsg (FALSE, "informational", "Returned DATEIND is %d", dateind);
} 





