
/*******************************************************************************
NAME:     makeperm()
PURPOSE:  Function to execute the makeperm subcomand.
          Create permanent log, TAE parms are implicit inputs 
*******************************************************************************/

#include "datelog.h"  // application header file  
#include <string.h>   // for calling strcpy()  
#include <stdio.h>    // for file output  
#include <unistd.h>   // for calling access  




FUNCTION void makeperm (void)   
{
// declare variables  
   char permlog[CMLEN];   // host name for TAE parm permlog  
   long nocheck=IUPDATE,  // file access mode, parm to c_getnam() 
	zero=0;           // value of zero  
   FILE *filep;           // points to permlog  

// get TAE parms.  parm prot is acquired in set_prot() in functions.c  
   if (c_getnam (&pblock, "permlog", &zero, permlog, "", &nocheck)!=E_SUCC) 
      errclean ("datelog-fatal", "Fatal error encountered");

// enable file locking before writing data 
   make_lock (permlog); 

// check file existence  
   if (!access(permlog,0) )
      errclean ("datelog-exist", "Log file %s already exists", 
	       host_tae(permlog)); 
      
// open output file 
   if ((filep = fopen(permlog,"w")) == NULL)
      errclean ("datelog-open", "Unable to open file %s for writing", 
	       host_tae(permlog));

// write file header  
   file_header (filep, "Permanent Processing Log"); 
   fclose (filep); 

// set protection of new file  
   set_prot (permlog); 

// increment count of output files  
   strcpy (outfiles.newfiles[outfiles.newcount++], permlog); 
}


