
/*******************************************************************************
NAME:     makeshort.c
PURPOSE:  functions to execute the makeshort subcommand 
          Initialize a short-term processing log.
*******************************************************************************/

#include "datelog.h"  // application header file  
#include <stdio.h>    // for file output  
#include <string.h>   // for calling strcpy()  
#include <unistd.h>   // for calling access()  






/*******************************************************************************
NAME:     makeshort()
PURPOSE:  execute the makeshort subcommand.  TAE parms are implicit inputs
*******************************************************************************/

FUNCTION void makeshort (void) 
{

// declare variables  
   char  curlog[CMLEN],    // host name for current log file  
        *pername,          // TAE parm pername, composite period name  
	 namebase[CMLEN];  // host name for TAE parm namebase, file name base 
   long nocheck=IUPDATE,   // file access mode, parm to c_getnam() 
	zero=0;            // value of zero  
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

/* get TAE parms.  parm prot is acquired in set_prot() in functions.c 
********************************************************************/

   // pername  
      tparm = p_find (&pblock, "pername");  
      pername = SVAL(*tparm,0); 

   // get namebase  
      if (c_getnam (&pblock, "namebase", &zero, namebase, "", &nocheck)!=E_SUCC)
	 errclean ("datelog-fatal", "Fatal error encountered");

   // check input parms for imbedded white space and zero-length  
      check_parm ("PERNAME", pername); 

   // lowercase parms to make later string comparisons case-insensitive  
      strlower (pername); 

   // determine curlog  
      get_log_name (namebase, pername, curlog); 

/* output data 
*************/

   // enable file locking before writing data 
      make_lock (namebase); 

   // create curlog  
      makecur (curlog, pername); 
}








/*******************************************************************************
NAME:     makecur()
PURPOSE:  create short_term log with a file header 
*******************************************************************************/

FUNCTION void makecur (
         char *curlog,   // input:  host name for file to create  
   const char *pername)  // input:  composite period name  
{

// declare variables  
   char   header[LINELEN];  // a header for the short-term log     
   FILE  *filep;            // file pointer 

// check file existence  
   if (!access(curlog,0) )
      errclean ("datelog-exist", "Log file %s already exists", 
	       host_tae(curlog)); 
      
// open file 
   if ((filep = fopen(curlog,"w")) == NULL)
      errclean ("datelog-open", "Unable to open file %s for writing", 
	       host_tae(curlog));

// write file header 
   sprintf (header, "Processing Log for Compositing Period %s", pername); 
   file_header (filep, header); 
   fclose (filep); 

// set protection of new file  
   set_prot (curlog); 

// update list of output files  
   strcpy (outfiles.newfiles[outfiles.newcount++], curlog); 
}





