
/*******************************************************************************
NAME:     scenein.c
PURPOSE:  function to execute the scenein subcommand 
          Log information related to processing a single AVHRR scene
          that was attempted to be added to the composite.
*******************************************************************************/

#include "datelog.h"  // application header file  
#include <string.h>   // for calling strcpy()  
#include <unistd.h>   // for calling chown()  





/*******************************************************************************
NAME:     scenein()
PURPOSE:  execute the scenein subcommand.  TAE parms are implicit inputs  
*******************************************************************************/

FUNCTION void scenein (void) 
{
// declare variables  
   char  curlog[CMLEN],   // host name for current log file  
        *pername,         // TAE parm pername, composite period name  
 	 datefile[CMLEN], // host name for TAE parm datefile  
	 dbname[CMLEN],   // final host name of dbname file  
	 namebase[CMLEN], // host name for TAE parm namebase, file name base  
	 oldname[PNAME],  // name of previous composite period  
	*pstatus,         // TAE parm pstatus, processing status  
        *sarea,           // TAE parm sarea, study area  
        *sceneid,         // TAE parm sceneid, scene id  
	*sysdate,         // system date  
	*tapeid;          // TAE parm tapeid, tape id for sceneid  
   long dateind,          // TAE parm dateind, date index  
	nocheck=IUPDATE,  // file access mode, parm to c_getnam() 
        pernum,           // period number as determined by datelog  
	readacc=IREAD,    // file read access, parm to c_getnam()  
	zero=0;           // value of zero  
   struct fparm dbname0,  // name of dbname file as received from c_getnam()  
                permlog;  // host name for TAE parm, permlog  
   struct iparm dbuid,    // TAE parm, uid for dbname file  
                swindow;  // TAE parm swindow, scene window  
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

/* get TAE parms.  parm prot is acquired in set_prot() in functions.c 
********************************************************************/
   // get misc TAE parms  
      tparm = p_find (&pblock, "dateind");   dateind  = IVAL(*tparm,0);
      tparm = p_find (&pblock, "pername");   pername  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "pstatus");   pstatus  = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "sarea");     sarea    = SVAL(*tparm,0); 
      tparm = p_find (&pblock, "sceneid");   sceneid  = SVAL(*tparm,0); 

   // get swindow  
      integer_parm ("swindow", &swindow); 

   // get datefile, namebase, permlog  
      name_parm ("permlog", readacc, &permlog); 
      if (c_getnam(&pblock,"datefile", &zero, datefile, "", &readacc)!=E_SUCC ||
	  c_getnam(&pblock,"namebase", &zero, namebase, "", &nocheck)!=E_SUCC  )
	 errclean ("datelog-fatal", "Fatal error encountered");

   // get dbname and dbuid  
      name_parm    ("dbname", nocheck, &dbname0); 
      integer_parm ("dbuid", &dbuid); 
      if (dbname0.count != dbuid.count) 
	 errclean ("datelog-badparm", 
		  "DBNAME and DBUID must both have values or be set to NULL");

   // get tapeid  
      tparm = p_find (&pblock, "tapeid");    
      if (tparm->v_count == 1) 
	 tapeid = SVAL(*tparm,0); 
      else
	 tapeid = "none"; 

   /* check input parms for imbedded white space and zero-length, there is no
      need to check parms that have valid values */ 
      check_parm ("PERNAME", pername); 
      check_parm ("PSTATUS", pstatus); 
      check_parm ("SAREA",   sarea); 
      check_parm ("SCENEID", sceneid); 
      check_parm ("TAPEID",  tapeid); 

   // lowercase parms to make later string comparisons case-insensitive  
      strlower (sceneid);
      strlower (pername); 

   // determine oldname, pernum, curlog  
      get_data (datefile, pername, oldname, &pernum); 
      get_log_name (namebase, pername, curlog); 

   // get system date  
      sysdate = system_date (FALSE); 

/* output data 
*************/
   // enable file locking before writing data 
      make_lock (namebase); 

   // save data to files  
      // curlog  
	 write_log (curlog, sysdate, pername, sarea, dateind, sceneid, 
		   tapeid, &swindow, "", pstatus, FALSE); 

      // permlog  
	 if (permlog.count) 
	    write_log (permlog.value, sysdate, pername, sarea, dateind, sceneid,
		      tapeid, &swindow, "", pstatus, FALSE); 

      // optionaly write to dbname file  
	 if (dbname0.count) 
	    {
	    // get name of database file  
	       dbase_name (dbname0.value, "scenein", dbname);
	    // write to file  
	       write_log (dbname, sysdate, pername, sarea, dateind, sceneid, 
			 tapeid, &swindow, "", pstatus, TRUE); 
	    // set protection of new file  
	       set_prot (dbname); 
	    // change owner  
	       if (chown (dbname, dbuid.value[0], -1))
		  errclean ("datelog-chown", 
			   "Unable to change owner of database file %s", 
			   host_tae(dbname)); 
	    }
}



