
/*******************************************************************************
NAME:     sceneout.c
PURPOSE:  functions to execute the sceneout subcommand.
          Log taking a scene (or some scenes) out of a composite. 
*******************************************************************************/

#include "datelog.h"  // application header file  
#include <stdio.h>    // for file output  
#include <string.h>   // for calling string functions  
#include <unistd.h>   // for calling access()  
#define SCENELEN 16   /* TAE parm sceneid is (string 16), SCENELEN is also max 
			 length (excluding null) of scenes in file */ 
#define MAXDATES 10   // TAE parm dateind has a max count of 10  





/*******************************************************************************
NAME:     sceneout()
PURPOSE:  execute the sceneout subcommand.  TAE parms are implicit inputs 
*******************************************************************************/

FUNCTION void sceneout (void) 
{

// declare variables  
   char  curlog[CMLEN],   // host name for current log file  
        *pername,         // TAE parm pername, composite period name  
	 dbname[CMLEN],   // final host name for dbname file  
	 namebase[CMLEN], // host name for TAE parm namebase, file name base  
	*pstatus,         // TAE parm pstatus, processing status  
        *sarea,           // TAE parm sarea, study area  
         sceneid[MAXDATES][SCENELEN+1], /* scenes in file that correspond to 
					   dateind, 1 for null */ 
	*sysdate,         // system date  
	*swindow,         // TAE parm swindow, place holder for scene window  
	*tapeid;          // TAE parm tapeid, place holder for tape id  
   long nocheck=IUPDATE,  // file access mode, parm to c_getnam() 
	readacc=IREAD,    // file read access, parm to c_getnam()  
	scenenum,         // loop index, scene being processed  
	zero=0;           // value of zero  
   struct fparm dbname0,  // name of dbname file as received from c_getnam()  
                permlog;  // host name for TAE parm permlog  
   struct iparm dateind,  // TAE parm dateind, date index  
                dbuid,    // TAE parm, uid for dbname file  
                swflag;   /* simulates TAE parm swindow as received by 
			     datelog-scenein */
   struct VARIABLE *tparm; // points to a TAE parm in pblock  

/* get TAE parms.  parm prot is acquired in set_prot() in functions.c 
********************************************************************/

// get misc TAE parms  
   tparm = p_find (&pblock, "pername");   pername = SVAL(*tparm,0); 
   tparm = p_find (&pblock, "pstatus");   pstatus = SVAL(*tparm,0); 
   tparm = p_find (&pblock, "sarea");     sarea   = SVAL(*tparm,0); 
   tparm = p_find (&pblock, "swindow");   swindow = SVAL(*tparm,0); 
   tparm = p_find (&pblock, "tapeid");    tapeid  = SVAL(*tparm,0); 
   integer_parm ("dateind", &dateind); 

// flag the fact that swindow is not applicable  
   swflag.count = BADSWINDOW; 

// get namebase, permlog  
   name_parm ("permlog", readacc, &permlog); 
   if (c_getnam (&pblock, "namebase", &zero, namebase, "", &nocheck)!=E_SUCC) 
      errclean ("datelog-fatal", "Fatal error encountered"); 

// get dbname and dbuid  
   name_parm    ("dbname", nocheck, &dbname0); 
   integer_parm ("dbuid", &dbuid); 
   if (dbname0.count != dbuid.count) 
      errclean ("datelog-badparm", 
	       "DBNAME and DBUID must both have values or be set to NULL");

// check input parms for imbedded white space and zero-length  
   check_parm ("PERNAME", pername); 
   check_parm ("PSTATUS", pstatus); 
   check_parm ("SAREA",   sarea); 
   check_parm ("SWINDOW", swindow); 
   check_parm ("TAPEID",  tapeid); 

// lowercase parms to make later string comparisons case-insensitive  
   strlower (pername); 

// determine curlog  
   get_log_name (namebase, pername, curlog); 

// get sceneids for these date indices 
   for (scenenum=0;  scenenum<dateind.count;  scenenum++)
      get_scene (dateind.value[scenenum], curlog, sceneid[scenenum]); 

// get system date  
   sysdate = system_date (FALSE); 

/* output data 
*************/

// enable file locking before writing data 
   make_lock (namebase); 

// save data to files  
   for (scenenum=0;  scenenum<dateind.count;  scenenum++)
      {
      // curlog  
	 write_log (curlog, sysdate, pername, sarea, 
		   dateind.value[scenenum], sceneid[scenenum], tapeid, 
		   &swflag, swindow, pstatus, FALSE); 

      // permlog  
	 if (permlog.count) 
	    write_log (permlog.value, sysdate, pername, sarea, 
		      dateind.value[scenenum], sceneid[scenenum], tapeid, 
		      &swflag, swindow, pstatus, FALSE); 

      // optionaly write to dbname file  
	 if (dbname0.count) 
	    {
	    // get name of database file  
	       dbase_name (dbname0.value, "sceneout", dbname);
	    // write to file  
	       write_log (dbname, sysdate, pername, sarea, 
			 dateind.value[scenenum], sceneid[scenenum], tapeid,
			 &swflag, swindow, pstatus, TRUE);
	    // set protection of new file  
	       set_prot (dbname); 
	    // change owner  
	       if (chown (dbname, dbuid.value[0], -1))
		  errclean ("datelog-chown", 
			   "Unable to change owner of database file %s", 
			   host_tae(dbname)); 
	    }
      }
}







/*******************************************************************************
NAME:     get_scene()
PURPOSE:  return the sceneid that corresponds to the input date index.  This 
	  function typically is called 1 or 2 times per program execution.
          Function is similar to get_dateind() in functions.c  
*******************************************************************************/

FUNCTION void get_scene (
   const long dateind,   // input:   find sceneid that corresponds to this index
         char *curlog,   // input:   current log file, file to process  
         char *sceneid)  // output:  return value  
{

// declare variables  
   char linein[LINELEN],    // input line being processd  
	oldscene[LINELEN];  // old sceneid found in file  
   long header,             // loop index, header line being skipped  
	oldindex;           // a date index found in the file  
   FILE *filep;             // points to curlog  

// open curlog and skip header  
   if (access(curlog,0) )
      errclean ("datelog-nolog", "Log file %s does not exist", 
	       host_tae(curlog));

   if ((filep = fopen(curlog,"r")) == NULL)
      errclean ("datelog-open", "Unable to open file %s for reading", 
	       host_tae(curlog));

   for (header=0; header<LOGHEADER; header++)
      if (fgets (linein, LINELEN, filep) == NULL)
         errclean ("datelog-fileend", 
		  "End of file while reading short-term log %s", 
		  host_tae(curlog)); 

// find this date index in the file  
   for (;;)
      {
      // read a line of the file  
	 if ( fgets(linein, LINELEN, filep) == NULL)
	    {
	    fclose (filep);
	    errclean ("datelog-nodate", 
		     "Date index %d was not found in short-term log %s", 
		     dateind, host_tae(curlog)); 
	    }

      // log files include two types of records.  skip inappropriate records  
	 if (sscanf (linein, " %*s %*s %d %s", &oldindex, oldscene) != 2)
	    continue; 

      // date index was found, return  
	 if (dateind == oldindex) 
	    {
	    fclose (filep); 
	    if (strlen (oldscene) > SCENELEN)
	       errclean ("datelog-toolong", 
			"Sceneid %s in log file %s has more than %d characters",
			oldscene, host_tae(curlog), SCENELEN); 

	    strcpy (sceneid, oldscene); 
	    return; 
	    }
      }
}



