
/******************************************************************************
NAME:     delscene.c 
PURPOSE:  program main()

PROGRAM HISTORY
-------------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  ----------------------------------------------------
May 1994  David Zokaites   Original development for $DCONTRIB (ddps contrib)
Oct 1995  David Zokaites   Released to $ADAPSCONTRIB, updated Makefile and 
                           users guide 
******************************************************************************/


/* Most of the code herein supports COMPIMG="image1+image2".  The problem is 
   sequential IO can only update one image file at a time.  Similary, the code 
   was designed to update minmax.  The problem is that c_svmnmx() wants to 
   change minmax for all bands and that might not be appropriate.  To save a 
   lot of development time, this proc sets minmax to invalid.  Update access
   does not work well with subbanding so that is not allowed although the code
   was developed for that too. */ 



#define GLOBALVARS    /* defined only in this file, flags delscene.h */
#include "delscene.h" /* proc header file */ 





void main ()                 
{	                    
/* declare vars */ 
   char outname[MAXIMG+1][CMLEN]; /* output images name for completion message*/
   struct UPARMS uparms;          /* tae user parms */ 
   long outcount,                 /* number of output files */ 
        zero=0;                   /* value of zero */ 
/* initialize las and tae */ 
   begin_las ("delscene"); 
/* get tae parms */ 
   get_parms (&uparms); 
/* open input images */ 
   opensin ("COMPIMG", &compimg); 
   if (uparms.sepdate)
   {
      openrin ("DATEIMG", uparms.dateup, &dateimg); 
      if (compimg.nl != dateimg.nl || compimg.ns != dateimg.ns)
	 c_errmsg ("COMPIMG and DATEIMG have different NL or NS.", 
	 procfatal, &fatal); 
   }
   else /* set data type and size of date image */ 
   {
      dateimg.dtype = compimg.dtype; 
      dateimg.dsize = compimg.dsize; 
   }
/* error checks */ 
   /* different images should be given */ 
      if (!strcmp (compimg.taename[0], dateimg.taename[0]))
	 c_errmsg (
	 "The same image was given for COMPIMG and DATEIMG.  Set DATEIMG=--", 
	 procfatal, &fatal); 
   /* check for overflow of data type */ 
      check_overflow (uparms.maskval, "COMPIMG", compimg.dtype); 
      if (uparms.sepup)
	 check_overflow (uparms.maskval, "DATEIMG", dateimg.dtype); 
   /* check for dateband being out of range.  >= allows for dateband being 0 
      relative instead of 1 */ 
      if (!uparms.sepdate && uparms.dateband >= compimg.totalbands ||
	   uparms.sepdate && uparms.dateband >= dateimg.totalbands  )
	 c_errmsg (
	 "The value of DATEBAND is greater than the # of bands in the image.",
	 procfatal, &fatal); 
/* make list of output files */ 
   for (outcount=0; outcount<compimg.ncomponents; outcount++)
      strcpy (outname[outcount], compimg.taename[outcount]);
   if (uparms.sepup) 
      strcpy (outname[outcount++], 
      dateimg.taename[ dateimg.fdlut[ uparms.dateband ]]); 
/* process images */ 
   process_image (&compimg, &dateimg, &uparms);  
/* close image files, update history files, update ddr min max info */ 
   finish_images (&compimg, TRUE,  uparms.dateup, uparms.dateband);
   if (uparms.sepdate)
      finish_images (&dateimg, FALSE, uparms.dateup, uparms.dateband); 
/* end program */ 
   c_complt (procname, outname, &outcount); 
}




