
/* getpar.c:  functions to get user parms */ 
#include "typlim.h"   /* for type limits in check_overflow() */ 
#include "delscene.h" /* proc header file */ 





/* FUNCTION:  get tae parms */ 
void get_parms (uparms)
   struct UPARMS *uparms;   /* ouput tae user parms */ 
{
/* declare variables */ 
   long index;              /* for loop index */ 
   struct VARIABLE *tparm;  /* points to a tae parm */ 
/* get most parms */ 
   tparm = p_find (&pblock, "maskval");   uparms->maskval  = IVAL (*tparm, 0);
   tparm = p_find (&pblock, "dateband");  uparms->dateband = IVAL (*tparm, 0);
   tparm = p_find (&pblock, "dateimg");   uparms->sepdate  = (*tparm).v_count; 
   uparms->dateband--;  /* convert from 1 to 0 relative */ 
/* get dateind parm */ 
   tparm = p_find (&pblock, "dateind");   
   uparms->dcount = (*tparm).v_count;
   for (index=0; index<uparms->dcount; index++)
      uparms->dvalue[index] = IVAL (*tparm, index); 
/* get flag dateup */ 
   tparm = p_find (&pblock, "dateup");   
   if ( *SVAL(*tparm,0) =='Y') /* y for yes */ 
      uparms->dateup = TRUE;
   else
      uparms->dateup = FALSE;
/* set sepdateup */ 
   if (uparms->sepdate && uparms->dateup)
      uparms->sepup = TRUE; 
   else
      uparms->sepup = FALSE; 
}





/* FUNCTION:  check for overflow of image data type, all parms are input, 
   a fatal error may be output */ 
void check_overflow (maskval, imagename, dtype)
   long maskval;    /* mask value to check */ 
   char *imagename; /* image name for output error message */ 
   long dtype;      /* data type of image */ 
{
/* check for overflow */ 
   switch (dtype)
   {
      case EBYTE:  
	 if (maskval < MINUTINY || maskval > MAXUTINY)
	 {
	    sprintf (message, "MASKVAL of %d exceeded range of %s data type.",
	    maskval, imagename); 
	    c_errmsg (message, procfatal, &fatal); 
	 }
	 break; 
      case EWORD:  
	 if (maskval < MINWORD || maskval > MAXWORD)
	 {
	    sprintf (message, "MASKVAL of %d exceeded range of %s data type.",
	    maskval, imagename); 
	    c_errmsg (message, procfatal, &fatal); 
	 }
	 break; 
      case ELONG:  
	 if (maskval < MINLONG || maskval > MAXLONG)
	 {
	    sprintf (message, "MASKVAL of %d exceeded range of %s data type.",
	    maskval, imagename); 
	    c_errmsg (message, procfatal, &fatal); 
	 }
	 break; 
      case EREAL:  
	 if (maskval < MINREAL || maskval > MAXREAL)
	 {
	    sprintf (message, "MASKVAL of %d exceeded range of %s data type.",
	    maskval, imagename); 
	    c_errmsg (message, procfatal, &fatal); 
	 }
	 break; 
      default: 
	 c_errmsg ("Program error", procfatal, &fatal); 
   }
}






