
/* image.c by David Zokaites 12-92 to present.  This file consolidates TAE 
   and LAS image processing calls. */ 
#include "delscene.h"    /* proc header file */ 





/* FUNCTION: initialize TAE and LAS processing.  Must be called before all
   other functions in this file */ 
void begin_las (proc) 
   char *proc;             /* input:  name of las proc */ 
{
/* declare vars */ 
   long length,            /* length of a char string */ 
        std_out;           /* standard output */ 
/* initialize las */ 
   c_inlas (&pblock, &std_out); 
/* set global vars for proc name, some are uppercase, some are lowercase */ 
   strcpy  (procname, proc); 
   length = strlen (procname);   
   c_low2up (procname, &length);  
   length = strlen (proc);       
   c_up2low (proc, &length); 
   sprintf (procfatal, "%s-fatal", proc); 
   sprintf (procwarn,  "%s-warn",  proc); 
}





/* FUNCTION close input image files and free buffers */ 
void close_inimage (ini)
   struct lasinimage *ini;  /* input:  close these files */ 
{
/* declare vars */
   long imagenum;  /* loop index, image number */ 
/* close image files */ 
   if (ini->iotype == SIO) /* close group descriptor */ 
   {
      if (c_egclse(&ini->groupd) ) 
	 c_errmsg ("Error closing image files.", procwarn, &go_on); 
   }
   else               /* close individual file descriptors */ 
      for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
	 if (c_eclose (ini->fdarray+imagenum))
	    c_errmsg ("Error closing image files.", procwarn, &go_on); 
/* free buffers */ 
   free (ini->buffer); 
}





/* FUNCTION: open an input image for sequential access and allocate data
   buffer */ 
void opensin (parmname, ini)
   char *parmname;             /* input:  name of tae parm for image */ 
   struct lasinimage *ini;     /* output: opened input image */ 
{
/* declare vars */ 
   long accessm=IUPDATE,       /* image access mode, arg to c_eopens() */ 
        bandnum,               /* loop index, band number */ 
        imagenum,              /* loop index, image number */ 
        buffersz,              /* buffer size returned by c_ealloc() */
        maximg=MAXIMG,         /* max number of images allowed */ 
        qflag[MAXIMG][2],      /* flag, do image components have a band or
				  window specified */ 
	samewindow,            /* flag, do image components have the same 
				  window specified? */ 
        zero=0;                /* value of 0 */ 
   struct VARIABLE *tparm;     /* points to a tae parm in pblock */ 
/* parse input image spec */ 
   /* call c_getin() */ 
      if (c_getin (&pblock, parmname, &zero, ini->taename, ini->hostname, 
      &ini->ncomponents, ini->window, ini->bands, ini->nbands, 
      &ini->totalbands, &maximg, &samewindow, qflag))
      {
	 sprintf (message, "Error getting input image for parm %s.", parmname); 
	 c_errmsg (message, procfatal, &fatal); 
      }
   /* set nl, ns */ 
      ini->nl = ini->window[0][NL]; 
      ini->ns = ini->window[0][NS]; 
/* error checks */ 
   /* image must have one file */ 
      if (ini->ncomponents > 1)
      {
	 sprintf (message, 
	 "Input image must consist of one file.  Do not use %s=image1+image2",
	 parmname); 
	 c_errmsg (message, procfatal, &fatal); 
      }
   /* subbanding does not work */ 
      for (bandnum=1; bandnum<=ini->nbands[0]; bandnum++)
	 if (ini->bands[0][bandnum-1] != bandnum)
	    c_errmsg ("Input images may not be subbanded.", procfatal, &fatal);
   /* different window sizes causes c_egroup() to fail */ 
      if (!samewindow)
      {
	 sprintf (message, 
	 "Input images specified by parm %s do not have the same window size.",
	 parmname);
	 c_errmsg (message, procfatal, &fatal); 
      }
/* update last used fields of input images */ 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      if (c_lused (ini->hostname[imagenum] ))
	 c_errmsg ("Error updating last used fields of input image ddr.", 
	 procwarn, &go_on); 
/* open image files */ 
   ini->fdarray[ini->ncomponents]=0;  /* null terminate array */ 
   ini->dtype=0; 
   for (bandnum=imagenum=0; imagenum<ini->ncomponents;  
   bandnum+=ini->nbands[imagenum], imagenum++)
      if (c_eopens (ini->fdarray+imagenum, ini->hostname[imagenum], 
      ini->bands[imagenum], 
      &ini->window[imagenum][SL], &ini->window[imagenum][SS], 
      &ini->window[imagenum][NL], &ini->window[imagenum][NS], 
      ini->bufoff+bandnum, &accessm, &ini->dtype, &zero, &zero) )
      {
	 sprintf (message, "Error opening image file for image %s",
	 ini->taename[imagenum]); 
	 c_errmsg (message, procfatal, &fatal); 
      }
/* set data type */ 
   switch (ini->dtype)
   {
      case EBYTE:  ini->dsize=1;  break; 
      case EWORD:  ini->dsize=2;  break; 
      case ELONG:  ini->dsize=4;  break; 
      case EREAL:  ini->dsize=4;  break; 
   }
/* allocate memory for image buffer */ 
   if ((ini->buffer = (unsigned char *) 
   c_ealloc(ini->fdarray, &buffersz)) == NULL)
      c_errmsg ("Unable to allocate memory for input image buffer.", 
      procfatal, &fatal); 
/* initialize min max arrays after getting tae global $minmax.  
      see comments in delscene.c 
      tparm = p_find (&pblock, "$minmax");    
      if (*SVAL(*tparm,0)=='Y')   
	 ini->minmax = TRUE;
      else 
	 ini->minmax = FALSE;
      */ 
   ini->minmax = FALSE;
   if (ini->minmax && c_inmnmx (&ini->dtype, &ini->totalbands, 
   ini->minval, ini->maxval))
      c_errmsg ("Error initializing min/max arrays.", procfatal, &fatal); 
/* group image files */ 
   if (c_egroup (ini->fdarray, &ini->groupd, ini->buffer, &buffersz) )
      c_errmsg ("Error grouping input image files.", procfatal, &fatal);
/* initialize first line of image */ 
   if (c_estep (&ini->groupd) )
      c_errmsg ("Error stepping through input images.", procfatal, &fatal);
}





/* FUNCTION: open an input image for random access and allocate data buffers */
void openrin (parmname, oupdate, ini)
   char *parmname;          /* input:  tae name of image to open */ 
   long oupdate;            /* input:  if T, open image for update
			               else, open for read access */ 
   struct lasinimage *ini;  /* output: opened input image */ 
{
/* declare vars */ 
   long accessm,            /* image access mode, arg to c_eopenr() */ 
        bandnum,            /* loop index, band number */ 
        imagenum,           /* loop index, image number */ 
        maximg=MAXIMG,      /* max number of images allowed */ 
	one=1,              /* value of one */ 
        qflag[MAXIMG][2],   /* flag, do image components have a band or
			       window specified */ 
	samewindow,         /* flag, do image components have the same 
			       window specified? */ 
        tband,              /* loop index, total band number */ 
        zero=0;             /* value of zero */ 
   struct VARIABLE *tparm;  /* points to a tae parm in pblock */ 
/* set accessm */ 
   if (oupdate)
      accessm = IUPDATE;
   else
      accessm = IREAD; 
/* parse input image spec */ 
   /* call c_getin() */ 
      if (c_getin (&pblock, parmname, &zero, ini->taename, ini->hostname, 
      &ini->ncomponents, ini->window, ini->bands, ini->nbands, 
      &ini->totalbands, &maximg, &samewindow, qflag))
      {
	 sprintf (message, "Error getting input image for parm %s.", parmname); 
	 c_errmsg (message, procfatal, &fatal); 
      }
   /* set nl, ns */ 
      ini->nl = ini->window[0][NL]; 
      ini->ns = ini->window[0][NS]; 
/* error checks */ 
   /* image must have one file */ 
      if (ini->ncomponents > 1)
      {
	 sprintf (message, 
	 "Input image must consist of one file.  Do not use %s=image1+image2",
	 parmname); 
	 c_errmsg (message, procfatal, &fatal); 
      }
   /* subbanding does not work */ 
      for (bandnum=1; bandnum<=ini->nbands[0]; bandnum++)
	 if (ini->bands[0][bandnum-1] != bandnum)
	    c_errmsg ("Input images may not be subbanded.", procfatal, &fatal);
   /* different window sizes causes logic in process.c to fail */ 
      if (!samewindow)
      {
	 sprintf (message, 
	 "Input images specified by parm %s do not have the same window size.",
	 parmname);
	 c_errmsg (message, procfatal, &fatal); 
      }
/* update last used fields of input images */ 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      if (c_lused (ini->hostname[imagenum] ))
	 c_errmsg ("Error updating last used fields of input image ddr.", 
	 procwarn, &go_on); 
/* open image file and set vars */ 
   ini->iotype = RIO;   /* random io */ 
   ini->dtype=0; 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      if (c_eopenr (ini->fdarray+imagenum, ini->hostname[imagenum], 
      ini->nbands+imagenum, 
      &ini->window[imagenum][SL], &ini->window[imagenum][SS], 
      &ini->window[imagenum][NL], &ini->window[imagenum][NS], 
      &accessm, &ini->dtype, &zero, &one)) 
      {
	 sprintf (message, "Error opening image file for image %s",
	 ini->taename[imagenum]); 
	 c_errmsg (message, procfatal, &fatal); 
      }
/* set dsize */ 
   switch (ini->dtype)
   {
      case EBYTE:  ini->dsize=1;  break; 
      case EWORD:  ini->dsize=2;  break; 
      case ELONG:  ini->dsize=4;  break; 
      case EREAL:  ini->dsize=4;  break; 
   }
/* build luts for later access by rread() */ 
   for (tband=imagenum=0; imagenum<ini->ncomponents; imagenum++)
      for (bandnum=0; bandnum<ini->nbands[imagenum]; bandnum++, tband++)
      {
	 /* Due to subbanding, the parm eband to c_eread() is relative to the 
	 bands in the input -- not output -- image files.  This is accommodated 
	 by cblut.  */ 
	 ini->cblut[tband] = ini->bands[imagenum][bandnum];
	 ini->fdlut[tband] = imagenum; 
      } 
/* allocate memory for image buffer */ 
   if (( ini->buffer = (unsigned char *) malloc (ini->ns * ini->dsize) ) 
   == NULL)
      c_errmsg ("Unable to allocate memory for input image buffer.", 
      procfatal, &fatal); 
/* initialize min max arrays after getting tae global $minmax 
      see comments in delscene.c 
      tparm = p_find (&pblock, "$minmax");    
      if (*SVAL(*tparm,0)=='Y')   
	 ini->minmax = TRUE;
      else 
	 ini->minmax = FALSE;
      */ 
   ini->minmax = FALSE;
   if (ini->minmax && c_inmnmx (&ini->dtype, &ini->totalbands, 
   ini->minval, ini->maxval))
      c_errmsg ("Error initializing min/max arrays.", procfatal, &fatal); 
}





/* FUNCTION:  read data from input image file, using random IO */ 
void rread (curline, curband, ini)
   long curline;           /* input: starting line of block of data to read, 
			      0 relative */ 
   long curband;           /* input: band to read, 0 relative */ 
   struct lasinimage *ini; /* input and output: data structure of input image 
			      to read */
{
/* declare vars */ 
   long imagenum,          /* image number, index to fdarray[] */ 
	one=1;             /* n image lines to read at a time */ 
/* first determine which image file to access then translate from curband as 
   input to curband relative to input image file */ 
   imagenum = ini->fdlut[curband]; 
   curband  = ini->cblut[curband]; 
/* convert from 0 relative to 1 relative */ 
   curline++; 
/* read data from disk image */ 
   if (c_eread (ini->fdarray+imagenum, &curband, &curline, ini->buffer, 
   &one) != ini->ns)
      c_errmsg ("Error reading input image data.", procfatal, &fatal); 
}





/* FUNCTION:  write data to disk image, using random IO */ 
void rwrite (curline, curband, ini)
   long curline;             /* input: starting line of block of data to write, 
			        0 relative */ 
   long curband;            /* input: band to write, 0 relative */ 
   struct lasinimage *ini;  /* input: data structure of image to write */
{
/* declare var */ 
   long one=1;               /* n lines to write at a time */ 
/* convert from 0 relative to 1 relative */ 
   curline++; 
   curband++; 
/* write data to disk */ 
   if (c_ewrite (ini->fdarray, &curband, &curline, ini->buffer, &one) 
   != ini->ns)
      c_errmsg ("Error writing output image data.", procfatal, &fatal);
}





/* FUNCTION:  optionally update minmax for a line of image data for sequential
   processing */ 
void minmaxs (ini)
   struct lasinimage *ini; /* input and output:  structure of image data to 
			      update min/max for */ 
{
/* declare var */ 
   long band;                /* loop index to image bands */ 
/* return if min/max does not need updated */ 
   if (!ini->minmax)
      return; 
/* error check */ 
   if (ini->iotype != SIO)
      c_errmsg ("Logic error:  minmaxs() was called but iotype!=SIO",
      procfatal, &fatal); 
/* update minmax */ 
   for (band=0; band<ini->totalbands; band++)
      if (c_pxmnmx (ini->buffer + ini->bufoff[band] * ini->dsize, 
      &ini->ns, ini->minval+band, ini->maxval+band, &ini->dtype))
	 c_errmsg ("Error updating min max arrays.", procfatal, &fatal); 
}





/* FUNCTION:  optionally update minmax for a line of image data for random
   processing */ 	
void minmaxr (band, ini)
   long band;              /* input:  image buffer holds data for this band */
   struct lasinimage *ini; /* input and output:  structure of image data to 
			      update min/max for */ 
{
/* return if min/max does not need updated */ 
   if (!ini->minmax)
      return; 
/* error check */ 
   if (ini->iotype != RIO)
      c_errmsg ("Logic error:  minmaxr() was called but iotype!=RIO",
      procfatal, &fatal); 
/* update min max */ 
   if (c_pxmnmx (ini->buffer, &ini->ns, ini->minval+band, 
   ini->maxval+band, &ini->dtype) )
      c_errmsg ("Error updating min max arrays.", procfatal, &fatal); 
}






