
/* include file image.h by David Zokaites 12-92 to present.  File helps to 
   consolidates LAS image processing calls.  File includes data structures for 
   LAS images and TAE variables and also function prototypes. */
#include "las.h"    /* for las image IO */ 





/* types of image IO allowed */
   enum iotypes 
   {
      SIO, /* sequential, line by line */ 
      RIO  /* line by line, using LAS random IO */ 
   };





/* data structure for one las input image, used to facilitate IO.  Because las
   image specs allow for grouping image components, (e.g. in="image1 + image2")
   one input image may correspond to multiple disk image files.  The same
   structure is used all types of IO, although all structure members are not 
   always used. */
   struct lasinimage
   {
      char hostname[MAXIMG][CMLEN],/* host name for image files */ 
           taename[MAXIMG][CMLEN]; /* tae name for image components */ 
      struct FDESC* fdarray[MAXIMG+1]; /* file descriptor array */ 
      struct GDESC* groupd;        /* group descriptor, used for SIO */ 
      long bands[MAXIMG][MAXBND+1],/* band numbers */ 
           bufoff[MAXBND],         /* offsets into data buffer, for SIO */ 
           cblut[MAXIMG*MAXBND],   /* a lut for current band, for RIO */ 
 	   dsize,                  /* N bytes per pixel as seen by the proc */
 	   dtype,                  /* image data type as seen by the proc */ 
           fdlut[MAXIMG*MAXBND],   /* a lut for file descriptor, for RIO */
           iotype,                 /* type of io, = SIO, RIO */ 
           minmax,                 /* flag, determine min/max for ddr? = T,F */
           nbands[MAXIMG],         /* number of bands in each component */ 
           ncomponents,            /* number of image components or disk
	                              image files */ 
           nl, ns,                 /* size of input image = window[0][NL], 
				      window[0][NS] */ 
           totalbands,             /* total number of bands */ 
           window[MAXIMG][4];      /* window (sl ss nl ns) for each component*/
      unsigned char *buffer;       /* buffer for image data */ 
      double minval[MAXBND],       /* min pixel value, used to update ddr */ 
             maxval[MAXBND];       /* max pixel value, used to update ddr */ 
   }; 
