
/* process.c:  functions to process the images */ 
#include "delscene.h" /* proc header file */ 





/* FUNCTION:  process images, disk images will be updated, all parms are input*/
void process_image (compimg, dateimg, uparms)
   struct lasinimage *compimg;  /* specs for input composite image */ 
   struct lasinimage *dateimg;  /* specs for input date band image */ 
   struct UPARMS *uparms;       /* user tae parms */ 
{
/* declare vars */ 
   long index,                 /* for loop index:  value of dateind */ 
	line,                  /* for loop index:  image line */ 
        messform=ALLCAT,       /* output message format, arg to c_startm() */ 
        msgtim,                /* tae global $msgtim */ 
	sample;                /* for loop index:  image sample */ 
   float dateval;              /* pixel value in date band */
   struct VARIABLE *tparm;     /* points to a tae parm in pblock */ 
   unsigned char *datebuf;     /* a location in dateimg->buffer */ 
   extern long curent;         /* current entity, for processing message */ 
/* initialize and start processing line message */ 
   tparm = p_find (&pblock, "$msgtim");    
   msgtim = IVAL(*tparm, 0); 
   c_inittm (procname, "Processed", "line", "bands.", "", &compimg->nl,
   &compimg->totalbands, &msgtim, &messform); 
   c_startm ();  
/* process image */ 
   for (line=0; line<compimg->nl; line++, curent++)
   {
   /* get date image buffer */ 
      if (uparms->sepdate)
      {
	 rread (line, uparms->dateband, dateimg);
	 datebuf = dateimg->buffer; 
      }
      else
	 datebuf = compimg->buffer + 
	 compimg->bufoff[ uparms->dateband ] * compimg->dsize;
   /* loop for all samples */ 
      for (sample=0; sample<compimg->ns; sample++,  datebuf+=dateimg->dsize)
      {
	 /* get dateval */ 
	    switch (dateimg->dtype)
	    {
	       case EBYTE: dateval = *          datebuf;  break;
	       case EWORD: dateval = *(short *) datebuf;  break;
	       case ELONG: dateval = *(long  *) datebuf;  break;
	       case EREAL: dateval = *(float *) datebuf;  break;
	    }
	 /* loop for all values of dateind */ 
	    for (index=0; index<uparms->dcount; index++)
	       if ( (long)dateval == uparms->dvalue[ index ])
	       {
		  /* change pixel value in all bands */ 
		  change_pix (sample, dateimg->dtype, compimg, uparms, datebuf);
		  break; 
	       }
      }
      /* update minmax data */ 
	 minmaxs (compimg); 
	 if (uparms->sepup)
	    minmaxr (uparms->dateband, dateimg); 
      /* step through images, reading and writing a line of data */ 
	 if (c_estep (&compimg->groupd) )
	    c_errmsg ("Error stepping through image.", procfatal, &fatal); 
	 if (uparms->sepup)
	    rwrite (line, uparms->dateband, dateimg); 
   }
c_stoptm();  
}





/* FUNCTION:  change pixel value in all image bands, all parms are input, 
   buffers in compimg and dateimg are updated */ 
void change_pix (sample, datetype, compimg, uparms, datebuf)
   long sample;                 /* current image sample being processed */ 
   long datetype;               /* data type of date image */ 
   struct lasinimage *compimg;  /* specs for input composite image */ 
   struct UPARMS *uparms;       /* user tae parms */ 
   unsigned char *datebuf;      /* location of this pixel in dateimg->buffer */
{
/* declare variables */ 
   long band;                   /* for loop index, image band */ 
   unsigned char *compbuf;      /* location of image pixel in compimg->buffer */
/* loop for all bands */ 
   for (band=0; band<compimg->totalbands; band++)
   {
      /* skip over date band if it is not to be updated */ 
	 if (band == uparms->dateband && !uparms->sepdate && !uparms->dateup)
	    continue; 
      /* set bv in buffer */ 
	 compbuf = compimg->buffer + (compimg->bufoff[band]+sample) * 
	 compimg->dsize;
	 switch (compimg->dtype)
	 {
	    case EBYTE: *          compbuf = uparms->maskval;  break;
	    case EWORD: *(short *) compbuf = uparms->maskval;  break; 
	    case ELONG: *(long  *) compbuf = uparms->maskval;  break;
	    case EREAL: *(float *) compbuf = uparms->maskval;  break;
	 }
   }
   /* update date band */ 
      if (uparms->sepup)
	 switch (datetype)
	    {
	       case EBYTE: *          datebuf = uparms->maskval;  break;
	       case EWORD: *(short *) datebuf = uparms->maskval;  break;
	       case ELONG: *(long  *) datebuf = uparms->maskval;  break;
	       case EREAL: *(float *) datebuf = uparms->maskval;  break;
	    }
}





/* FUNCTION:  close image files, update history file, update ddr min max info */
void finish_images (ini, compimg, dateup, dateband) 
   struct lasinimage *ini;  /* input:  specs on image */ 
   long compimg;            /* input:  flag, is this the composite image */ 
   long dateup;             /* input:  flag, was date image updated */ 
   long dateband;           /* input:  band that was updated for date image */
{
/* declare vars */ 
   long bandnum,    /* for loop index, image band */ 
	imagenum,   /* for loop index, image file */ 
	zero=0;     /* value of zero */ 
/* close image files */ 
   close_inimage (ini); 
/* update history */ 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      if (c_hstout (&pblock, ini->hostname[imagenum], "", &zero, ""))
	 c_errmsg ("Error updating image history file.", procwarn, &go_on); 
/* update ddr with min max */ 
   if (compimg)
   {
      for (bandnum=0, imagenum=0; imagenum<ini->ncomponents; 
      bandnum+=ini->nbands[imagenum], imagenum++)
	 if (c_svmnmx (ini->hostname[imagenum], ini->nbands+imagenum, 
	 ini->minval+bandnum, ini->maxval+bandnum, 
	 (ini->minmax ? "YES" : "NO")))
	    c_errmsg("Error updating min/max values in ddr.", procwarn, &go_on);
   }
   else if (dateup)
   {
      bandnum=dateband;
      imagenum= ini->fdlut[bandnum]; 
	 if (c_svmnmx (ini->hostname[imagenum], ini->nbands+imagenum, 
	 ini->minval+bandnum, ini->maxval+bandnum, 
	 (ini->minmax ? "YES" : "NO")))
	    c_errmsg("Error updating min/max values in ddr.", procwarn, 
	    &go_on);
   } 
} 




