
/******************************************************************************
NAME:     check.C
PURPOSE:  perform checks on input images 
******************************************************************************/

// include file 
#include "gacosdates.h"  // application header file  

// prototype local functions  
   static void make_check (const long testval, const long refval, const char 
			   *const descriptor);




/******************************************************************************
NAME:     check_indate()
PURPOSE:  make sure indate is a global dateband image 
******************************************************************************/
FUNCTION void check_indate (
   const struct INIMAGE *indate)  // check this image 
{
// check size  
   make_check (indate->totalbands, datebandi.numbands, "band"   ); 
   make_check (indate->nl        , datebandi.numlines, "lines"  ); 
   make_check (indate->ns        , datebandi.numsamps, "samples"); 

// check data type  
   if (indate->dtypein != datebandi.dtype) 
      errclean ("gdates-indate", 
		"INDATE input image does not have BYTE data type"); 
}




/******************************************************************************
NAME:     make_check()
PURPOSE:  make a check for check_indate();  
          if two input values are not equal, write an error message
******************************************************************************/
FUNCTION static void make_check (
   const long testval,           // compare this value to refval 
   const long refval,            // reference value 
   const char *const descriptor) // a description of images  
{
if (testval != refval) 
   errclean ("gdates-indate", 
	     "INDATE input image does not have %d %s", refval, descriptor); 
}

   

