
/*******************************************************************************
NAME:     dbase.C
PURPOSE:  functions for dealing with data base files.  These functions were
          modified from LAS>DATELOG 
*******************************************************************************/

// include files 
   #include <stdlib.h>     // for calling malloc()
   #include <string.h>     // for calling strlen() 
   #include <sys/types.h>  // for calling time()  
   #include <time.h>       // for calling time(), localtime() 
   #include <unistd.h>     // for calling access(), sleep()  
   #include "gacosdates.h" // application header file 

// prototype local functions 
   #define TIMESTRING 30  // number of characters in a time string 
   static void system_date (const long seconds, char timestring[TIMESTRING] ); 




/*******************************************************************************
NAME:     dbase_name()
PURPOSE:  create a unique name for a host file.  File will be used for 
	  database info.  A sample output name is ./1995-03-16-14:42:07.scenein 
*******************************************************************************/
FUNCTION void dbase_name (
   const struct FILENAME basename,  // base for output file name 
   const char * const extension,    /* file extension for output file name 
			               without a leading '.' */ 
         struct FILENAME &dbname)   // output: name of new file  
{

// declare variables  
   char sdate[TIMESTRING];  // system date for file name  
   long exists,             // flag, does file exist  
        wait;               /* loop index, seconds to wait to change file name, 
		               if system date changes, file name will change */ 

// make a unique name

   for (exists=TRUE, wait=0;  exists && wait<3;  wait++)
      {
      // get file name  
	 system_date (TRUE, sdate); 
	 sprintf (dbname.host, "%s%s.%s", basename.host, sdate, extension);
	 sprintf (dbname.tae,  "%s%s;%s", basename.tae , sdate, extension);
      // check file existence  
	 if (access (dbname.host, 0))
	    exists = FALSE; 
	 else
	    sleep (1); 
      }

   // error check:  file exists  
      if (exists) 
         errclean ("gdates-dbase", "Database file %s already exists", 
	           dbname.tae); 
}








/*******************************************************************************
NAME:     system_date()
PURPOSE:  return system date  
	  if (seconds), return date in yyyy-mm-dd-hh:mm:ss 
	  format;  else use            yyyy-mm-dd-hh:mm 
	  an example output string is 1995-03-16-14:42:07 
*******************************************************************************/
FUNCTION static void system_date (
   const long seconds,                // flag, use seconds in output  
	 char timestring[TIMESTRING]) // returned value 
{
// declare variables  
   time_t green_time;      // Greenwich system time 
   struct tm *timestruct;  // time structure used for local time 

// convert greenwich time to local time  
   if (time (&green_time) == -1)
      errclean ("gdates-time", "Error getting system time");
   timestruct = localtime (&green_time);

/* add year to system year.  
   what happens in year 2000 is not defined by the localtime man page */
   /*
     DLLOYD Y2K -- this is okay in 2000 tm_year = 100
   ---------------------------------------------------*/
   timestruct->tm_year += 1900;

// create time string  
   sprintf (timestring, "%4d-%02d-%02d-%02d:%02d", timestruct->tm_year,
	   timestruct->tm_mon + 1, timestruct->tm_mday, timestruct->tm_hour, 
	   timestruct->tm_min);
   if (seconds)
      sprintf (timestring + strlen(timestring), ":%02d", timestruct->tm_sec); 
}




