
/******************************************************************************
NAME:     main() for GACOSDATES
PURPOSE:  Recalculate date indices for old GACOS global composites.  

Date      Programmer                        Changes
--------  --------------  -----------------------------------------------------
May 1997  David Zokaites  Original development that was a little rushed.  I
			  tested most of the program's primary functions, but
			  memory leaks remain and most fatal errors were not 
			  tested.
******************************************************************************/

// include files 
   #include <new.h>        // for calling set_new_handler() 
   // define a constant to enable a global variable defined in gacosdates.h
   #define GLOBALVAR 
   #include "gacosdates.h" // application header file  






// PROGRAM MAIN  
int main (void)   
{	                    
// declare variables  
   struct PARBLK pblock;             // TAE parameter block  
   struct UPARMS uparms;             // user TAE parameters  

   struct INIMAGE  indate;           // info on input  dateband image  
   struct OUTIMAGE outdate;          // info on output dateband image  

   struct SCEDATES oldinfo[NAREAS],  // date indices and sceneid's in input logs
                   newinfo;          // scenes and new date indices 

   short wasUsed [NAREAS][BYTEVALS]; // histogram of INDATE, only TRUE and
                                     // FALSE are needed for histogram values
   short imageLut[NAREAS][BYTEVALS]; // LUT that maps INDATE to OUTDATE 

// initialize TAE/LAS 
   long std_out;  // standard output, parameter to c_inlas()  
   c_inlas (&pblock, &std_out); 
   c_lasmsg (FALSE, "informational", "Program GACOSDATES beginning execution.");

// get TAE parameters  
   set_delflg (&pblock);     // get $delflg for errclean.C  
			     // done early in main() for error recovery 
   getpar (&pblock, uparms); // get parameters which are used most places
   get_modes (&pblock);      // get TAE parameters dbuid,prot used in mode.C

// set error handler for no memory conditions 
   set_new_handler (&no_memory);

// reduce expected size of global dateband image for testing 
   datebandi.scale_size (uparms.scalfact); 

// open image to make sure indate is a global dateband image.  Close image
   open_bsq_in  (&pblock, "indate", EBYTE, &indate);  // open image 
   check_indate (&indate);                            // verify image 
   close_inimage (&indate);                           // close image 

// get all date indices and sceneid's from the short-term logs 
   read_logs (uparms, oldinfo);

// check for singular mapping between date index and sceneid 
   check_mapping (oldinfo, uparms); 

// histogram indate image
   histogram (&pblock, wasUsed); 

// make sure every dateindex in the histogram appears in the input logs
   check_index (wasUsed, oldinfo); 

// determine new date indices 
   new_indices (oldinfo, newinfo);  

// make a new inlog that can be used by LAS>GACOSADD to determine date index 
   make_inlog (uparms, newinfo); 

// create new short term logs and database files with new date indices 
   make_short (wasUsed, uparms, oldinfo, newinfo); 

// prepare a look-up-table (LUT) that maps INDATE to OUTDATE 
   make_lut (oldinfo, newinfo, imageLut); 

// copy INDATE to OUTDATE
   copy_image (&pblock); 

// map OUTDATE brightness values 
   map_image (&pblock, imageLut); 

// free memory for struct SCEDATES
   // oldinfo.  memory was allocated in read_logs() 
      for (long areanum=0;  areanum<NAREAS;  areanum++)
	 free_memory (oldinfo[areanum]);
   // newinfo.  memory was allocated in new_indices()
      free_memory (newinfo);  

// end program, call c_complt() indirectly
   complete();
}



