
/*******************************************************************************
NAME:     getpar.C 
PURPOSE:  return user TAE parameters;  initialize values in a global structure
	  which describes a global dateband image.
*******************************************************************************/

// include files 
   #include "gacosdates.h" // application header file  
   #include <string.h>     // for calling strlen()  

// prototype local function  
   static void get_file (struct PARBLK *pblock, char *parmname,
			 const short isdir, 
			 struct FILENAME &filename); 





/******************************************************************************
NAME:     getpar()
PURPOSE:  return most user TAE parameters, make a few error checks 
******************************************************************************/
FUNCTION void getpar (
         struct PARBLK *pblock,  // TAE parameter block  
         struct UPARMS &uparms)  // TAE user parameters  
{
// declare variables 
   struct FILENAME indir,    // directory of input  short term logs
		   outdir,   // directory of output short term logs
		   ologdir;  // directory for output inlog

// get parameters  
   struct VARIABLE *tparm; // points to a TAE parameter  
   tparm = p_find (pblock, "pername" );   uparms.pername  = SVAL (*tparm,0);
   tparm = p_find (pblock, "$msgtim" );   uparms.msgtim   = IVAL (*tparm,0);
   tparm = p_find (pblock, "scalfact");   uparms.scalfact = IVAL (*tparm,0);

   get_file (pblock, "dbname",  FALSE, uparms.dbname );
   get_file (pblock, "indir",   TRUE,  indir  );
   get_file (pblock, "outdir",  TRUE,  outdir );
   get_file (pblock, "ologdir", TRUE,  ologdir);

// build name of outinlog 
   sprintf (uparms.outinlog.host, "%s%s.inlog", ologdir.host, uparms.pername); 
   sprintf (uparms.outinlog.tae , "%s%s;inlog", ologdir.tae , uparms.pername); 

// build names of input and output short term logs 
   for (long lognum=0;  lognum<NAREAS;  lognum++)
   {
   // input 
      sprintf (uparms.inlogs[lognum].host, "%ss%02d_%s.log",
               indir.host, datebandi.number[lognum], uparms.pername);
      sprintf (uparms.inlogs[lognum].tae , "%ss%02d_%s;log",
               indir.tae,  datebandi.number[lognum], uparms.pername);

   // ouput 
      sprintf (uparms.outlogs[lognum].host, "%ss%02d_%s.log",
              outdir.host, datebandi.number[lognum], uparms.pername);
      sprintf (uparms.outlogs[lognum].tae ,  "%ss%02d_%s;log",
              outdir.tae,  datebandi.number[lognum], uparms.pername);
   }

// make sure output image does not exist by calling c_getout()
   char hostname[1][CMLEN],  // host name of image file 
        taename [1][CMLEN];  // TAE  name of image file 
   if (c_getout (pblock, "OUTDATE", taename, hostname) !=E_SUCC)
       errclean ("gdates-fatal", "Fatal error encountered");

// make sure indir != outdir
   if (!strcmp (indir.tae, outdir.tae))
       errclean ("gdates-same", "The same value was given for INDIR and "
                "OUTDIR, %s", indir.tae); 

// make a few checks on pername 
   if (strlen (uparms.pername) != 7)
       errclean ("gdates-pername", "PERNAME %s does not have exactly 7 "
		"characters", uparms.pername); 

   char *p = uparms.pername + 4;  // points to a 'p' in pername
   if (*p != 'p')
       errclean ("gdates-pername", "PERNAME %s does not have contain the "
		"letter p as the 5th character.", 
	        uparms.pername); 
}









/******************************************************************************
NAME:     get_file()
PURPOSE:  return host and TAE file/directory names 
******************************************************************************/
FUNCTION static void get_file (
         struct PARBLK *pblock,     // TAE parameter block  
         char *parmname,            // name of TAE parameter for file  
   const short isdir,               // flag, is the name a dir name
         struct FILENAME &filename) // host and TAE file names  
{
// declare variables  
   long readacc = IREAD,  // check directory for read access  
        zero = 0;         // value of zero  

// get host name  
   if (c_getnam (pblock, parmname, &zero, filename.host, "", &readacc)
   != E_SUCC)
      errclean ("gdates-fatal", "Fatal error encountered"); 

// eliminate the trailing '.' on host dir names  that was added by c_getnam()  
   char *lastchar;         // last character in a file name  
   lastchar = filename.host + strlen (filename.host) -1;
   if (*lastchar == '.')
      *lastchar = '\0'; 
   else if (isdir) // write error message 
   {
      struct VARIABLE *tparm;  // points to a TAE parameter 
      tparm = p_find (pblock, parmname); 
      errclean ("gdates-dirname", "Value %s for parameter %s is not a "
                "directory name", SVAL(*tparm,0),  parmname);
   }

// translate host name to TAE name
   strcpy(filename.tae, filename.host);

}







/*******************************************************************************
NAME:     DATEBANDI::DATEBANDI()
PURPOSE:  Constructor, initialize structure describing the global dateband image
*******************************************************************************/
FUNCTION DATEBANDI::DATEBANDI (void) 
{
// set misc members
   this->dtype    = EBYTE;   // byte data type 
   this->numbands =     1;   // number of bands   in a global dateband 
   this->numlines = 17347;   // number of lines   in a global dateband  
   this->numsamps = 40031;   // number of samples in a global dateband  

// describe window sizes.  
   // These values were provided by Kent Lethcoe of Digital Data Production.  
   this->sl[0] =     1; 
   this->sl[1] =     1; 
   this->sl[2] =  4145; 
   this->sl[3] =  4145; 
   this->sl[4] =  8675; 
   this->sl[5] =  8675; 
   this->sl[6] =  8675; 
   this->sl[7] = 13204; 
   this->sl[8] = 13204; 

   this->ss[0] =  2156; 
   this->ss[1] = 17454; 
   this->ss[2] =     1; 
   this->ss[3] = 15569; 
   this->ss[4] =  8898; 
   this->ss[5] = 17793; 
   this->ss[6] = 28913; 
   this->ss[7] =  9974; 
   this->ss[8] = 30528; 

   this->nl[0] =  4144; 
   this->nl[1] =  4144; 
   this->nl[2] =  4530; 
   this->nl[3] =  4530; 
   this->nl[4] =  4529; 
   this->nl[5] =  4529; 
   this->nl[6] =  4529; 
   this->nl[7] =  4144; 
   this->nl[8] =  4144; 

   this->ns[0] = 12572; 
   this->ns[1] = 18815; 
   this->ns[2] = 15568; 
   this->ns[3] = 24463; 
   this->ns[4] =  8895; 
   this->ns[5] = 11120; 
   this->ns[6] = 11119; 
   this->ns[7] =  6741; 
   this->ns[8] =  8426; 

// describe study area names 
   // number[] translates from 9-non fill area numbers to 12 overall study area
   // numbers.  overall image includes areas 5,9,11 which are all fill 

   this->number[0] =  1;  
   this->number[1] =  2; 
   this->number[2] =  3; 
   this->number[3] =  4; 
   this->number[4] =  6; 
   this->number[5] =  7; 
   this->number[6] =  8; 
   this->number[7] = 10; 
   this->number[8] = 12; 
}





/*******************************************************************************
NAME:     DATEBANDI::scale_size()
PURPOSE:  reduce expected image size of global dateband for testing
*******************************************************************************/
FUNCTION void DATEBANDI::scale_size (
   const long scalfact)  // scale factor by which to divide image size for test
{
// return if nothing needs to be done
   if (scalfact == 1)
      return; 

// change overall image size
   this->numlines /= scalfact; 
   this->numsamps /= scalfact; 

// user message
   c_lasmsg (FALSE, "gdates-size", "Warning:  for testing purposes, image "
            "size has been divided by SCALFACT %d and reduced to %d lines by "
	    "%d samples", scalfact, this->numlines, this->numsamps); 

// change window sizes 
   for (long areanum=0;  areanum<NAREAS;  areanum++)
   {
      this->sl[areanum] /= scalfact;  
      this->ss[areanum] /= scalfact;  
      this->nl[areanum] /= scalfact;  
      this->ns[areanum] /= scalfact;  

      // make sure SL and SS >= 1, they are 1 relative 
      if (this->sl[areanum] == 0)   this->sl[areanum] = 1;
      if (this->ss[areanum] == 0)   this->ss[areanum] = 1;
   }
}




