
/*******************************************************************************
NAME:     image2.C
PURPOSE:  a C++ version of some image processing functions 
          I needed to add parameter areanum 
*******************************************************************************/

// include files 
   #include <math.h>       // for call to ceil()  
   #include <stdlib.h>     // for call to malloc()
   #include "gacosdates.h" // application header file 







/*******************************************************************************
NAME:     open_inimage()
PURPOSE:  general function to open an input image and allocate data buffers. 
          function prototype must include parameter defaults 
*******************************************************************************/
FUNCTION void open_inimage (
         struct PARBLK *pblock,   // TAE parameter block  
   const long areanum,            // study area to process 
         struct INIMAGE *inimage, // output: opened input image  
         long accessm,    // = IREAD   image access method, IREAD for input, 
			  //   IUPDATE for updating
          char *parmname,  // ="INDATE" TAE parameter, name of image to open  
   const long dtypeap,    // =EBYTE    convert image data to this type upon
                          //           reading.  pass NOTAPPLY if not converting
   const long iomode,     // =BSQIO    mode of IO. = BILIO, BSQIO, or BLOCKIO  
   const long blocksize)  // =NOTAPPLY size of blocks for BLOCKIO. ignored for
                          //           other io modes 
{

// declare variables  
   long bandnum,          // loop index, band number  
        imagenum,         // loop index, image number  
	linenum,          // loop index, line number  
        maximg=MAXIMG,    // max number of images allowed  
	one=1,            // value of one  
        qflag[MAXIMG][2], /* flag, do image components have a band or
			     window specified */ 
	samewindow,       /* flag, do image components have the same 
			     window specified? */ 
        tband,            // loop index, total band number  
        zero=0;           // value of zero  

// set misc variables  
   inimage->iomode  = iomode;   
   inimage->dtypeap = dtypeap; 

// parse input image spec  
   // call c_getin()  
      if (c_getin (pblock, parmname, &zero, inimage->taename, inimage->hostname,
		  &inimage->nmrimg, inimage->window, inimage->bands, 
		  inimage->nbands, &inimage->totalbands, &maximg, &samewindow, 
		  qflag) !=E_SUCC)
	 errclean ("gdates-fatal", "Fatal error encountered"); 
   // error check:  different window sizes  
      /* In check.C, the image is checked for having one band.  This check is
          not necessary. 
      if (!samewindow)
	 errclean ("gdates-window", 
	          "Images specified by %s do not have the same window size", 
		  parmname);
      */ 

   // set nl, ns  
      inimage->nl = inimage->window[0][NL]; 
      inimage->ns = inimage->window[0][NS]; 

// update last used fields of input images  
   for (imagenum=0;  imagenum<inimage->nmrimg;  imagenum++)
      (void) c_lused (inimage->hostname[imagenum]); 

// open image files  
   for (imagenum=0;  imagenum<inimage->nmrimg;  imagenum++)
      if (c_eopenr (inimage->fdarray+imagenum, inimage->hostname[imagenum], 
		   &inimage->nbands[imagenum], 
		   datebandi.sl+areanum, // revised for GACOSDATES
		   datebandi.ss+areanum, // revised for GACOSDATES
		   datebandi.nl+areanum, // revised for GACOSDATES
		   datebandi.ns+areanum, // revised for GACOSDATES
		   &accessm, &inimage->dtypeap, &zero, &one) !=E_SUCC) 
	 errclean ("gdates-fatal", "Fatal error encountered"); 

// set data size  
   switch (inimage->dtypeap)
      {
      case EBYTE:  inimage->dsizeap=EBYTESIZE;  break; 
      case EWORD:  inimage->dsizeap=EWORDSIZE;  break; 
      case ELONG:  inimage->dsizeap=ELONGSIZE;  break; 
      case EREAL:  inimage->dsizeap=EREALSIZE;  break; 
      default:  errclean ("gdates-fatal", 
                         "Program error:  unknown image data type");
      }

/* get input data type.  
   get datatype of 1st disk image, supporting routines convert other images to 
   data type of first image */ 
   if (c_gettyp (inimage->hostname, &one, &inimage->dtypein) !=E_SUCC)
      errclean ("gdates-fatal", "Fatal error encountered"); 

// optionally set misc variables for block processing  
   if (inimage->iomode == BLOCKIO)
      {
      inimage->blocksize = blocksize; 
      inimage->lblocks = (long) ceil ( (double)inimage->window[0][NL] / 
			 inimage->blocksize ); 
      inimage->sblocks = (long) ceil ( (double)inimage->window[0][NS] / 
			 inimage->blocksize ); 
      inimage->blocknl = inimage->blocksize * inimage->lblocks; 
      inimage->blockns = inimage->blocksize * inimage->sblocks; 
      inimage->fillns  = inimage->blockns - inimage->window[0][NS];
   }

/* build luts for later access by functions to read image data.  Due to 
   subbanding, the parameter eband to c_eread() is relative to the bands in the 
   input -- not output -- image files.  This is accommodated by cblut. */
   for (tband=imagenum=0;  imagenum<inimage->nmrimg;  imagenum++)
      for (bandnum=0;  bandnum<inimage->nbands[imagenum];  bandnum++, tband++)
	 {
	 inimage->cblut[tband] = inimage->bands[imagenum][bandnum];
	 inimage->fdlut[tband] = imagenum; 
	 } 

// allocate memory for image buffers  
   switch (inimage->iomode)
      {
      case BSQIO:  
	 // buffer holds one line for one band  
	 if (( inimage->buffer = (unsigned char *) malloc 
	 (inimage->ns * inimage->dsizeap) ) == NULL)
	    errclean ("gdates-memory", 
	             "Unable to allocate memory for input image buffer"); 
	 break; 

      case BILIO:  
	 // linebuf holds one line for each band  
	 for (bandnum=0;  bandnum<inimage->totalbands;  bandnum++)
	    if (( inimage->linebuf[bandnum] = (unsigned char *) malloc 
	    (inimage->ns * inimage->dsizeap) ) == NULL)
	       errclean ("gdates-memory", 
	                "Unable to allocate memory for input image buffer"); 
	    break; 

      case BLOCKIO:  
	 // buffer holds a block of lines for one band  
	    if ((inimage->buffer = (unsigned char *) malloc 
	    (inimage->blocksize * inimage->blockns * inimage->dsizeap) ) ==NULL)
	       errclean ("gdates-memory", 
	                "Unable to allocate memory for input image buffer"); 
	 // bufloc points to each line in the block  
	    if ((inimage->bufloc = (unsigned char **) 
	    malloc ( sizeof(unsigned char *) * inimage->blocksize)) == NULL)
	       errclean ("gdates-memory", 
	                "Unable to allocate memory for bufloc variable"); 
	    for (linenum=0;  linenum<inimage->blocksize;  linenum++)
	       inimage->bufloc[linenum] = inimage->buffer + 
	       linenum * inimage->dsizeap * inimage->blockns; 
	 break; 

      default:  
	 errclean ("gdates-fatal", 
	          "Program error:  iomode has a bad value in open_in()"); 
      }
	 
// set flag, image was opened  
   inimage->opened = TRUE; 
}









/*******************************************************************************
NAME:     read_bsq2()
PURPOSE:  read data from input image files, using BSQ IO  
*******************************************************************************/
FUNCTION void read_bsq2 (
         long curline,            // input:  line of data to read, 0 relative  
   const long areanum,            // input:  read data for this study area 
         struct INIMAGE *inimage) // output: new image data  
{

// declare variables  
   long imagenum,   // image number, index to fdarray[]  
        curband=0,  // band to read, 0 relative  
	one=1;      // n image lines to read at a time  

/* determine which image file to access then translate from curband as 
   input to curband relative to input image file */ 
   imagenum = inimage->fdlut[curband]; 
   curband  = inimage->cblut[curband]; 

// convert from 0 relative to 1 relative  
   curline++; 

// read data from disk image  
   if (c_eread (inimage->fdarray+imagenum, &curband, &curline, inimage->buffer, 
               &one) != datebandi.ns[areanum])
      errclean ("gdates-fatal", "Fatal error encountered"); 
}








/*******************************************************************************
NAME:     write_bsq2()
PURPOSE:  write data to disk image, using BSQ IO  
*******************************************************************************/
FUNCTION void write_bsq2 (
         long curline,     // input:  starting line of data to write, 0 relative
   const long areanum,     // input:  write data for this study area 
         struct INIMAGE *inimage)  // input:  data structure of image to write 
{

// declare variables  
   long imagenum,   // image number, index to fdarray[]  
        curband=0,  // band to write, 0 relative  
	one=1;      // n image lines to read at a time  

/* determine which image file to access then translate from curband as 
   input to curband relative to input image file */ 
   imagenum = inimage->fdlut[curband]; 
   curband  = inimage->cblut[curband]; 

// convert from 0 relative to 1 relative 
   curline++; 

// write data to disk 
   if (c_ewrite (inimage->fdarray+imagenum, &curband, &curline, inimage->buffer,
		&one) != datebandi.ns[areanum] )
      errclean ("gdates-fatal", "Fatal error encountered"); 
}





