
/*******************************************************************************
NAME:     mode.c 
PURPOSE:  functions to change file owner and protection. 
          File consolidates use of TAE parameters dbuid and prot.
*******************************************************************************/

// include files 
   #include <sys/stat.h>   // for calling chmod() 
   #include <unistd.h>     // for calling chown()  
   #include "gacosdates.h" // application header file 

// declare global variables 
   struct 
   {
      long dbuid, // user id    for database files 
	   prot;  // protection for database files 
   } filemodes;





/*******************************************************************************
NAME:     get_modes()
PURPOSE:  get TAE parameters dbuid and prot 
*******************************************************************************/
FUNCTION void get_modes (
   struct PARBLK *pblock)  // TAE parameter block  
{
// declare variables 
   struct VARIABLE *tparm; // points to a TAE parameter  
   char converter[20];     // format string used to convert type
   long taeprot;           // file protection as input TAE parameter

// get parameters  
   tparm = p_find (pblock, "dbuid");   
   filemodes.dbuid = IVAL (*tparm,0);

// get value of prot parameter as done by LAS>DATELOG 
   // get parameter
      tparm = p_find (pblock, "prot");
      taeprot = IVAL (*tparm,0);

   // convert from octal (entered by user) to integer for chmod()
      sprintf (converter, "%d", taeprot);
      sscanf  (converter, "%od", &filemodes.prot ); 
}






/*******************************************************************************
NAME:     change_prot()
PURPOSE:  change file protection by calling chmod()
*******************************************************************************/
FUNCTION void change_prot (
   const struct FILENAME filename) // change protection on this file 
{
   if (chmod (filename.host, filemodes.prot) ) 
      errclean ("gdates-prot", "Error setting protection for file %s", 
		filename.tae); 
}





/*******************************************************************************
NAME:     change_owner()
PURPOSE:  change owner of a data base file 
*******************************************************************************/
FUNCTION void change_owner (
   const struct FILENAME filename) // change owner of this file 
{
   if (chown (filename.host, filemodes.dbuid, -1))
      errclean ("gdates-chown", "Unable to change owner of database file %s", 
	       filename.tae); 
}





