
/*******************************************************************************
NAME:     new_images.C 
PURPOSE:  functions to make new images 
*******************************************************************************/

// include files 
   #include "gacosdates.h" // application header file  
extern "C"
{
#include "pixman.h"
}




/*******************************************************************************
NAME:     make_lut()
PURPOSE:  prepare a look-up-table (lut) that maps INDATE to OUTDATE 
*******************************************************************************/
FUNCTION void make_lut (
   const struct SCEDATES oldinfo[NAREAS],// date indices & scenes in input logs
   const struct SCEDATES newinfo,        // scenes and their new date indices 
         short imageLut[NAREAS][BYTEVALS])  //output lut, maps INDATE to OUTDATE

{
// declare variables 
   long areanum,    // loop index, area number 
	maskval,    // loop index, mask brightness value
	oldscene;   // loop index, number of old scene id 

// data values < MININDEX are reserved for mask values, provide a 1-1 mapping
// for these data values
   for (areanum=0;  areanum<NAREAS;  areanum++)
      for (maskval=0;  maskval<MININDEX;  maskval++)
	 imageLut[areanum][maskval] = maskval; 

// loop for study areas, scenes in oldinfo
   for (   areanum=0;   areanum<NAREAS;                      areanum++)
      for (oldscene=0;  oldscene<oldinfo[areanum].nrecords;  oldscene++)
      {
	 // set vars with manageable names for old dateindex and sceneid
	    long  olddate = oldinfo[areanum].dateindex[oldscene];
	    char *oldsid  = oldinfo[areanum].sceneid  [oldscene];

	 // skip scenes with date index of FLAGDATE
	    if (olddate == FLAGDATE)
	       continue; 

	 // update LUT 
	    imageLut[areanum][olddate] = new_date (oldsid, newinfo); 
      }

}







/*******************************************************************************
NAME:     copy_image()
PURPOSE:  copy INDATE image to OUTDATE image 
*******************************************************************************/
FUNCTION void copy_image (
   struct PARBLK *pblock)  // input:  TAE parameter block. 
			   // not declared const because old functions in
			   // image.c don't use const 
{
// declare variables  
   const long band =0;      // 0th band is being processed
   long line,               // loop index, image line  
        messform=ALLCAT;    // output message format, parameter to c_inittm()  
   extern long curent;      // current entity, for processing message  
   struct INIMAGE indate;   // input date band image to copy
   struct OUTIMAGE outdate; // output date band image to create 

// open images 
   open_bsq_in  (pblock, "indate",  EBYTE, &indate); 
   open_bsq_out (pblock, "outdate", EBYTE, &indate, &outdate); 

// initialize and start processing line message  
   curent = 1; 
   c_inittm ("GACOSDATES", "Copied", "line", "band", "", &outdate.nl,
            &outdate.totalbands, &outdate.msgtim, &messform); 
   c_startm (); 

// copy image  
   for (curent=1, line=0;  line<outdate.nl;  line++, curent++)   
      {
      // read data  
	 read_bsq (line, band, &indate); 
      // copy data to outdate buffer 
	 c_pxcopy (indate.buffer, outdate.buffer, &indate.dtypeap, &indate.ns);
      // write data to disk image 
	 write_bsq (line, band, &outdate); 
      }
   c_stoptm(); 

// close images 
   close_inimage (&indate);    
   close_outimage (&outdate);  

// make ddr and history file 
   finish_out (pblock, &indate, &outdate);     
}








/******************************************************************************
NAME:     map_image()
PURPOSE:  map brightness values in OUTDATE 
******************************************************************************/
FUNCTION void map_image (
         struct PARBLK *pblock,             // TAE parameter block for $msgtim
					    // and INDATE 
   const short imageLut[NAREAS][BYTEVALS])  // LUT to apply to outdate 
{
// declare variables 
   long areanum,  // loop index:  study area 
	linenum,  // loop index:  image line 
	msgtim,   // $msgtim 
	sampnum;  // loop index:  image sample 

   long 
	messform=ONECAT, // output message format, parameter to c_inittm()  
	nareas=NAREAS;   // number of study areas, parameter to c_inittm() 

   unsigned char *inbuf;  // location of image pixel in indate.buffer  
   extern long curcat,    // current category, for processing message  
	       curent;    // current   entity, for processing message  

   struct INIMAGE outdate; // output date band image to map 

// get $msgtim 
   struct VARIABLE *tparm;  // points to a TAE parameter 
   tparm = p_find (pblock, "$msgtim");
   msgtim = IVAL (*tparm,0); 

// process image, loop for all areas, lines, and samples 
   curent = 1;

// loop for all areas 
   for (areanum=0, curcat=1;  areanum<NAREAS;  areanum++, curcat++)
   {
      // select window to be processed 
	 open_inimage (pblock, areanum, &outdate, IUPDATE, "OUTDATE"); 

      // reset processing message 
	 c_inittm ("GACOSDATES", "Mapped", "line", "study area", "", 
	 datebandi.nl+areanum, &nareas, &msgtim, &messform);
	 c_startm (); 

      // loop for all lines 
	 for (linenum=0, curent=1;  linenum<datebandi.nl[areanum];  
	 linenum++, curent++)
	 {
	 // read a line of data 
	    read_bsq2 (linenum, areanum, &outdate); 

	 // map the image data 
	    for (sampnum=0, inbuf=outdate.buffer;   
	    sampnum<datebandi.ns[areanum];
	    sampnum++, inbuf+= outdate.dsizeap)
	       *inbuf = imageLut[areanum][ *inbuf ]; 

	 // write new data to disk image
	    write_bsq2 (linenum, areanum, &outdate); 
	 } 

      // close image 
	 close_inimage (&outdate);  
	 c_stoptm(); 
   } 

// update list of files created 
   add_outfile_name (outdate.taename[0]); 
}



