
/*******************************************************************************
NAME:     outfiles.C
PURPOSE:  keep track of names of output files.  
	  File consolidates use of OUTFILES. 
*******************************************************************************/

// include files
   #include <stdlib.h>      // for calling qsort()
   #include <string.h>      // for calling strcmp()
   #include "gacosdates.h"  // application header file  

// prototype local function
   static int compare (const void* input1, const void* input2);

// declare list of output files created 
   #define MAXOUTFILES 500  // many data base files could be made 

   struct OUTFILES 
   {
      long count;                        // number of files created 
      char taename [MAXOUTFILES][CMLEN], // TAE  file name 
           hostname[MAXOUTFILES][CMLEN]; // host file name 
      OUTFILES(void) { count=0; };       // constructor for initilization  
   } outfiles; 







/*******************************************************************************
NAME:     add_outfile_name()
PURPOSE:  add a filename to the list of output files 
*******************************************************************************/
FUNCTION void add_outfile_name (const struct FILENAME outname)
{
// check for too many files 
   if (outfiles.count == MAXOUTFILES)
       errclean ("gdates-toomany", 
                 "Too many output files were created, increase MAXOUTFILES"); 

// copy name and increment count of files
   strcpy (outfiles.taename [ outfiles.count ], outname.tae ); 
   strcpy (outfiles.hostname[ outfiles.count ], outname.host); 
   outfiles.count++;
}






/*******************************************************************************
NAME:     add_outfile_name(), an overloaded function 
PURPOSE:  add a TAE name to the list of output files 
*******************************************************************************/
FUNCTION void add_outfile_name (char *taename)
{
// check for too many files 
   if (outfiles.count == MAXOUTFILES)
       errclean ("gdates-toomany", 
                 "Too many output files were created, increase MAXOUTFILES"); 

// convert TAE name to host format
   char hostname[CMLEN];
   if (c_cmhost(taename, hostname))
      errclean ("gdates-fatal", "Fatal error encountered");

// copy name and increment count of files
   strcpy (outfiles.taename [ outfiles.count ], taename ); 
   strcpy (outfiles.hostname[ outfiles.count ], hostname); 
   outfiles.count++;
}





/*******************************************************************************
NAME:     complete()
PURPOSE:  call c_complt()
*******************************************************************************/
FUNCTION void complete (void)
{
// sort list of output files 
   qsort (outfiles.taename, outfiles.count, sizeof(char) *CMLEN, compare);

// complete 
   c_complt ("GACOSDATES", outfiles.taename, &outfiles.count);  
}





/*******************************************************************************
NAME:     delete_outfiles()
PURPOSE:  delete all new files as part of error recovery 
*******************************************************************************/
FUNCTION void delete_outfiles(void)
{
// return if there are no files to delete 
   if (!outfiles.count)
      return; 

// sort list of output files 
   qsort (outfiles.taename, outfiles.count, sizeof(char) *CMLEN, compare);

// user message
   c_lasmsg (FALSE, "gdates-error", 
	    "Removing new files as part of error recovery:"); 

// delete files 
   for (long filenum=0;  filenum<outfiles.count;  filenum++)
   {
      // files were sorted by TAE name, translate TAE name to host 
      char hostname[CMLEN]; 
      if (c_cmhost (outfiles.taename[filenum], hostname))
	 c_lasmsg (FALSE, "gdates-cmtae", "Unable to convert filename from "
		   "host to TAE style"); 

      c_lasmsg (FALSE, "gdates-error", outfiles.taename[filenum]); 
      c_delfil (hostname); 
   }
}





/*******************************************************************************
NAME:     compare()
PURPOSE:  compare two strings to enable sorting via qsort().  Function was 
	  designed to operate on   char varname[MAXOUTFILES][CMLEN]. 
*******************************************************************************/
FUNCTION static int compare (
   const void* input1, const void* input2) // values to compare
{
   return   strcmp ( (char *)input1, (char *)input2); 
} 




