#!/bin/sh
#create a new inlog to test GACOSDATES 
#this script was based on $LASDDP/makeinlog.sh 

#check number of input parameters
   if [ $# != 1 ] 
   then
      echo  'makeinput.sh makes an input file for DATELOG' 
      echo  'makeinput.sh requires one parameters: pername (1993p11)'
      exit 1
   fi 

   pername=$1
   year=`echo $1   |nawk '{ print substr($1,1,4) }'`
   period=`echo $1 |nawk '{ print substr($1,5,3) }'`
   outfile=logs_ol/$pername".inlog_sh"
   tmpfile=makein.tmp

#Check for valid year   
   if [ $year -lt 1990 -o $year -gt 2010 ]
   then
      echo 'You have entered a year that is invalid'
      echo 'Year must be between 1990 and 2010'
      exit 1
   fi

#Check for valid period  
   echo $period | nawk '
   {
      if (length ($1) !=3          ||     #period name must be 3 characters long
	  substr ($1,1,1)   != "p" ||     #period name must start with "p"
	  substr ($1,2,2)+0 < 1    ||          
	  substr ($1,2,2)   > 36    )     #period name has range from 1 to 36 
	    exit 1     
   }'

   if [ $? -eq 1 ]
   then
      echo 'You have entered an invalid period name'
      echo 'A valid period would be p01..p36'
      exit 1
    fi 

#read existing and place scenes in temp file
   #cat gets several input records to process
   #grep skips headers of input files
   #sed changes upper to lower case
   #nawk selectes sceneid
   #sort sceneids uniquely
   #nawk print sceneid and increments date index

   cat logs_in/*.log        |\
   nawk '/ao/ { print $4 }' |\
   sort -u             > $tmpfile             
   
#Check for existing output file
   if [ -f $outfile ] 
   then
	cat $tmpfile | nawk '	
	#read existing file
	BEGIN {
		while (getline < "'$outfile'" == 1)
		  scenearr[$1] = $1
		maxdate=$2
	      }
        #process new sceneids 
	{
	   scene  = $1
	   dindex = $2	
           if (scene in scenearr)
		next
	   printf "%s %7d\n", scene, ++maxdate 
	 }'  >> $outfile

	 #cleanup
	 echo Finished reading NEW scenes and UPDATED $outfile
	 rm $tmpfile
	 exit 0
   fi
		
#make header for output file 
   echo Date indices to be used for $pername       >  $outfile
   echo '-----------------------------------'      >> $outfile 
   echo 'Sceneid           Date Index'             >> $outfile
   echo '----------------  ----------'             >> $outfile

#proces scenes for output file
   nawk '{ printf "%s %7d\n", $1, NR-1+10 }' $tmpfile   >> $outfile

#cleanup 
   rm $tmpfile
   echo Finished reading scenes and made $outfile
   exit 0

