
/******************************************************************************
NAME:     paxsites.c 
PURPOSE:  program main()

PROGRAM HISTORY
-------------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  ----------------------------------------------------
Nov 1994  David Zokaites   Original development for $DCONTRIB (ddps contrib),
			   application was later released to $LASDITTY 
Jan 1995  David Zokaites   Accommodated pax revision that allowed for
			   multi-volume tape sets;  allowed parms PAXDIR and
			   PAXUID to have "null value";  added parms SPROC and
			   TTYPE;  renamed pax log if pax failed;  expanded the
			   user help;  asked ops to scratch the tape if pax
			   failed twice;  several minor changes.
Mar 1995  David Zokaites   Wrote function nodupe_names() to create a new file 
			   name if files in LOGDIR or PAXDIR exist;  wrote
			   function new_name() to prevent overwriting pax error
			   log files;  accommodated a new format in sites.pdf,
			   updated Makefile format. 
Oct 1995  David Zokaites   A few trivial changes, released application to 
			   $ADAPSCONTRIB
******************************************************************************/


 
/* include files */ 
#include <ctype.h>       /* for call to isspace() */ 
#include <errno.h>       /* for use of errno */ 
#include <stdio.h>       /* for file io and call to remove() */ 
#include <stdlib.h>      /* for call to getenv(), system() */ 
#include <string.h>      /* for call to strstr() */ 
#include <unistd.h>      /* for call to access(), chown() */ 
#include <sys/types.h>   /* for call to mkdir() */ 
#include <sys/stat.h>    /* for call to mkdir() */ 
#include "las.h"         /* las include file */
#include "util.h"
 
/* general macros */ 
#define INLINE   300     /* size of an input line of text */
#define MAXNAMES  50     /* max number of image names, this limit is referenced
			    in sites.pdf, also is max count of local dellist in
			    paxsites.pdf */ 
#define PDATES    30     /* max size of string pdates */ 
#define SYSCMD   600     /* size of string with which to call system(), call to
			    pax requires lots of text */ 
/* #define TESTING */    /* to enable testing PAXSITES without bothering
                            computer operators with messages via write */ 
 
/* file names, excluding path */ 
#define CDABORT   "pax_sites.cd_aborted"  /* file used to flag cd errors */ 
#define LABELFILE ".pax.label"            /* file for pax tape label */ 
#define LISTFILE  "pax_sites.file_list"   /* used for file listings */ 
#define PAXABORT  "pax_sites.pax_aborted" /* file used to flag pax errors */ 
 
/* global vars for convenience, args to c_errmsg() */ 
   char  message[CMLEN],                 /* message string */ 
        *procfatal = "pax_sites-fatal",  /* fatal error key */ 
	*procwarn  = "pax_sites-warn";   /* warning message key */ 
   long fatal=-1,                        /* flag, fatal error */ 
        go_on=1;                         /* flag, info message */ 
 
/* structure for file and image names */
   struct FNAMES 
   {
      long count;                     /* number of file and image names */
      /* image/file name info */ 
      char basic   [MAXNAMES][CMLEN], /* basic names as found in sites.pdf eg.
					 sitename */   
	  *dellist [MAXNAMES],        /* tae parm dellist to return, copy of 
                                         taewild[][], must be char*  */
	   taewild [MAXNAMES][CMLEN], /* tae wildcarded image names, eg.
					 [dir]*sitename, does not
					 include ;img as cmdel-f does not like
					 the extension */ 
	   hostwild[MAXNAMES][CMLEN], /* host wildcarded image names as in 
					 dir/*sitename.img, used in 
					 image_exist() to make sure an image 
					 of this name exists. */ 
	   paxfiles[MAXNAMES][CMLEN]; /* host wildcarded file names as in 
					 *sitename*, used in write_pax()
					 as list of files to write to tape */
      /* output pax log file names.  paxtae and paxhost will have "null value" 
	 if uparms.paxcount==0 */ 
      char logtae [MAXNAMES][CMLEN],  /* tae version of logdir files, local
					 pax log files */ 
	   loghost[MAXNAMES][CMLEN],  /* host version of logdir files, local
					 pax log files */ 
           paxtae [MAXNAMES][CMLEN],  /* tae version of paxdir files, 
					 pax log files */ 
	   paxhost[MAXNAMES][CMLEN];  /* host version of paxdir files, 
					 pax log files */ 
   };  
 
/* structure for names of pax log files with an error */ 
   struct ERRFILES
   {
      long count;                   /* number of error files created */ 
      char host[MAXNAMES][CMLEN],   /* host name of error files */ 
           tae [MAXNAMES][CMLEN];   /* tae  name of error files */ 
   };
 
/* structure of input user parameters */ 
   struct UPARMS
   {
      char *baseadd,          /* string to add to logdir and paxdir for file
                                 names */ 
           *dpart2,           /* part2 of description for pax tape label */ 
	   *dpart4,           /* part4 of description for pax tape label */ 
            datefile[CMLEN],  /* host file listing study area dates */
	    imagedirhost[CMLEN], /* host directory for test site images */ 
	   *imagedirtae,         /* tae  directory for test site images */ 
	   *logdirtae,        /* tae  directory for local pax logs */ 
	    logdirhost[CMLEN],/* host directory for local pax logs */ 
	   *paxdirtae,        /* tae  directory for pax logs */
	    paxdirhost[CMLEN],/* host directory for pax logs */
           *perdates,         /* study area dates to translate */
           *sarea,            /* study area */
	   *sitedir,          /* location of sproc */
	   *sproc,            /* name of sproc (often sites.pdf) */ 
           *tapename,         /* user initials to use for the pax tape label */
	   *ttype;            /* type of tape to mount for pax */ 
      long paxuid,            /* uid for pax logs */
	   paxcount;          /* count of paxdir and paxuid parms */ 
   }; 
 
/* function prototypes for local functions */
#ifdef __DGUX__
   void main (void); 
   void get_parms (struct PARBLK *pblock, struct UPARMS *uparms);
   void revise_dates (char *datefile, char *perdates, char *pdates);
   void get_env (char *inpath, char *outpath);
   void read_sites (char *sfile, char *sarea, char namelist[MAXNAMES][CMLEN],
        long *namecount);
   void write_list (char *heading0, char *heading1,
        char namelist[MAXNAMES][CMLEN], long namecount);
   long check_control (char *iinline);
   void complete_names (struct UPARMS *uparms, struct FNAMES *fnames);
   void image_exist (struct FNAMES *fnames, long paxcount);
   void nodupe_names (long paxcount, struct FNAMES *fnames);
   long write_pax (char *dpart3, struct UPARMS *uparms, struct FNAMES *fnames,
        struct ERRFILES *errfiles);
   void send_message (char *comment, long effort);
   void copy_file (char *infile, char *outfile);
   void new_name (char *oldhost, long effort, char *newhost, char *newtae);
   void rm_file (char *filename);
#else /* old style declarations */ 
   void main ();
   void get_parms ();
   void revise_dates ();
   void get_env ();
   void read_sites ();
   void write_list ();
   long check_control ();
   void complete_names ();
   void image_exist ();
   void nodupe_names ();
   long write_pax ();
   void send_message ();
   void copy_file ();
   void new_name ();
   void rm_file ();
#endif





void main ()
{
/* declare vars */
   char *abort[1],          /* tae parm abort to return */ 
	nooutput[1][CMLEN], /* list of 0 output files, arg to c_complt() */ 
        pdates[PDATES],     /* revised version of perdates */ 
        sfile[CMLEN];       /* full path of sproc */ 
   long zero=0;             /* value of 0 */ 
   struct ERRFILES errfiles;/* names of error files */ 
   struct FNAMES fnames;    /* various versions of file and image names */ 
   struct UPARMS uparms;    /* input user parms */
   struct PARBLK pblock;    /* tae parameter block */
   errfiles.count=0;        /* initialize count of error files */ 
/* initialize las and get user parms */
   get_parms (&pblock, &uparms); 
/* get revised dates from datefile */
   revise_dates (uparms.datefile, uparms.perdates, pdates);
/* get full path of sproc */ 
   get_env (uparms.sitedir, sfile);
   sprintf ( sfile+strlen(sfile), "/%s", uparms.sproc); 
/* get image names from sproc */ 
   read_sites (sfile, uparms.sarea, fnames.basic, &fnames.count); 
   write_list ("No test site image names were found in SPROC", 
    "The following test site image names were found in SPROC:", 
    fnames.basic, fnames.count); 
/* complete file and image names */ 
   complete_names (&uparms, &fnames); 
/* make sure an image of each name exists */ 
   image_exist (&fnames, uparms.paxcount);
   write_list ("No test site images exist for this study area",
    "The following test site images exist for this study area:", 
    fnames.taewild, fnames.count); 
/* check for pre-existing log files */  
   nodupe_names (uparms.paxcount, &fnames); 
/* write files to pax tapes */ 
   if (fnames.count && write_pax (pdates, &uparms, &fnames, &errfiles))
      abort[0] = "yes";
   else
      abort[0] = "no";
/* tell user about output files */ 
   if (fnames.count)
   {
      write_list ("", "Created LOGDIR pax log files:", fnames.logtae, 
       fnames.count); 
      if (uparms.paxcount) 
	 write_list ("", "Created PAXDIR pax log files:", fnames.paxtae, 
	 fnames.count); 
   }
   if (errfiles.count)
      write_list ("", "Created LOGDIR pax ERROR log files:", errfiles.tae, 
      errfiles.count); 
/* return data to tae */ 
   if (q_string (&pblock, "abort", 1, abort, P_UPDATE) != SUCCESS ||
       q_string (&pblock, "dellist", fnames.count, fnames.dellist, P_UPDATE) 
       != SUCCESS ||
       q_out (&pblock) != SUCCESS)
      c_errmsg ("Error returning parameters to TAE", procfatal, &fatal); 
/* completion message */
   c_complt ("PAX_SITES", nooutput, &zero); 
}





/* FUNCTION:  initialize las and get parameters from tae */
void get_parms (pblock, uparms)
   struct PARBLK *pblock;   /* input:   tae parameter block */ 
   struct UPARMS *uparms;   /* output:  user parms */
{
/* declare variables*/
   long readacc=IREAD,      /* file access, parm to c_getnam() */
        std_out,            /* standard output*/
	zero=0;             /* value of 0 */
   struct VARIABLE *tparm;  /* points to a tae parm */
/* initialize tae */
   c_inlas (pblock, &std_out); 
/* get most parms from tae */
   tparm = p_find (pblock,"baseadd");    uparms->baseadd    = SVAL (*tparm,0);
   tparm = p_find (pblock,"dpart2");     uparms->dpart2     = SVAL (*tparm,0);
   tparm = p_find (pblock,"dpart4");     uparms->dpart4     = SVAL (*tparm,0);
   tparm = p_find (pblock,"logdir");     uparms->logdirtae  = SVAL (*tparm,0);
   tparm = p_find (pblock,"perdates");   uparms->perdates   = SVAL (*tparm,0);
   tparm = p_find (pblock,"sitedir");    uparms->sitedir    = SVAL (*tparm,0);
   tparm = p_find (pblock,"sproc");      uparms->sproc      = SVAL (*tparm,0);
   tparm = p_find (pblock,"tapename");   uparms->tapename   = SVAL (*tparm,0);
   tparm = p_find (pblock,"imagedir");   uparms->imagedirtae= SVAL (*tparm,0);
   tparm = p_find (pblock,"sarea");      uparms->sarea      = SVAL (*tparm,0);
   tparm = p_find (pblock,"ttype");      uparms->ttype      = SVAL (*tparm,0);
   if (c_getnam (pblock, "datefile", &zero, uparms->datefile, "", &readacc))
      c_errmsg ("Unable to get value of parm DATEFILE", procfatal, &fatal); 
/* get paxcount, paxdir, paxuid  */
   tparm = p_find (pblock,"paxdir");     
   uparms->paxcount = (*tparm).v_count;
   tparm = p_find (pblock,"paxuid");    
   if (uparms->paxcount != (*tparm).v_count )
      c_errmsg ("Parms PAXDIR and PAXUID must both be nulled or given values",
      procfatal, &fatal); 
   if (uparms->paxcount)
   {
      tparm = p_find (pblock,"paxdir");    uparms->paxdirtae = SVAL (*tparm,0);
      tparm = p_find (pblock,"paxuid");    uparms->paxuid    = IVAL (*tparm,0);
   }
/* translate tae names to host names */ 
   if (c_cmhost (uparms->imagedirtae,  uparms->imagedirhost)  || 
       c_cmhost (uparms->logdirtae,    uparms->logdirhost)    || 
       uparms->paxcount && 
       c_cmhost (uparms->paxdirtae,    uparms->paxdirhost)     )
      c_errmsg("Error translating TAE names to host names", procfatal, &fatal);
/* user messages */ 
   sprintf (message, "Input study area is %s", uparms->sarea);
   c_errmsg (message, "informational", &go_on); 
   sprintf (message, "SPROC %s is in directory %s", uparms->sproc, 
   uparms->sitedir);
   c_errmsg (message, "informational", &go_on); 
/* check directory existence */ 
   /* imagedir must exist */ 
      if (access (uparms->imagedirhost, 0))
      {
	 sprintf (message, "IMAGEDIR %s does not exist", uparms->imagedirtae);
	 c_errmsg (message, procfatal, &fatal); 
      }
   /* make logdir if it does not exist
      mkdir() needs an int (not octal) protection mode.  511(int)=777(octal)
      mkdir() combines proctection mode with umask. */ 
      if (access (uparms->logdirhost, 0) )
      {
	 if (mkdir (uparms->logdirhost,511))
	 {
	    sprintf (message, "Unable to create LOGDIR %s", uparms->logdirtae);
	    c_errmsg (message, procfatal, &fatal); 
	 }
	 else 
	 {
	    sprintf (message, "Created LOGDIR %s", uparms->logdirtae);
	    c_errmsg (message, "informational", &go_on); 
	 }
      }
   /* make paxdir if it does not exist */ 
      if (uparms->paxcount && access (uparms->paxdirhost, 0) )
      {
	 if (mkdir (uparms->paxdirhost,511))
	 {
	    sprintf (message, "Unable to create PAXDIR %s", uparms->paxdirtae);
	    c_errmsg (message, procfatal, &fatal); 
	 }
	 else 
	 {
	    sprintf (message, "Created PAXDIR %s", uparms->paxdirtae);
	    c_errmsg (message, "informational", &go_on); 
	 }
      }
}





/* FUNCTION:  get revised dates from datefile */
void revise_dates (datefile, perdates, pdates)
   char *datefile;  /* input:  file to read */
   char *perdates;  /* input:  translate from this version of dates to another*/
   char *pdates;    /* oputput:  better version of dates as found in datefile */
{
/* declare variables */ 
   FILE *filep;         /* points to datefile */ 
   char iinline[INLINE], /* a line in datefile */ 
        *linepos;       /* a position in a line */ 
   long cchars;         /* loop index, number of characters copied */ 
/* open file, c_getnam() checked file protection */ 
   filep = fopen (datefile, "r");
   if (filep == NULL)
   {
      sprintf (message, "Unable to open DATEFILE %s for reading", datefile);
      c_errmsg (message, procfatal, &fatal);
   }
/* read file, looking for perdates */ 
   for (;;)
   {
   /* read a line */ 
      if (fgets (iinline, INLINE, filep) == NULL)
      {
	 fclose (filep); 
	 sprintf (message, "PERDATES value of %s was not found in DATEFILE %s",
	  perdates, datefile); 
	 c_errmsg (message, procfatal, &fatal);
      }
   /* break if perdates is on this line */
      linepos = strstr (iinline, perdates);
      if (linepos != NULL)
	 break; 
   }
   fclose (filep); 
/* get pdates */ 
   /* skip over perdates */ 
      while ( !isspace (*linepos) && *linepos != '\0') 
	 linepos++;
   /* skip white space */ 
      while (  isspace (*linepos) && *linepos != '\0') 
	 linepos++;
   /* error check */ 
      if (*linepos == '\0')
      {
	 sprintf(message, "A line of DATEFILE %s was formatted incorrectly: %s",
	  datefile, iinline);
	 c_errmsg (message, procfatal, &fatal);
      }
   /* copy remaining string, 
      for loop avoids trailing \012 that strcpy() copies */ 
      for (cchars=0; !isspace(*linepos) && *linepos !='\0';  
      linepos++, pdates++, cchars++)
      {
	 if (cchars == PDATES-1)  /* error check:  string too long */ 
	 {
	    sprintf (message, 
	     "In DATEFILE %s, translated PERDATES (%s) string has more than %d characters", 
	      datefile, perdates, PDATES-1);
	    c_errmsg (message, procfatal, &fatal);
	 }
	 *pdates = *linepos;    /* copy a char */ 
      }
      *pdates = '\0';  /* terminate string */ 
}





/* FUNCTION:  get disk dir from env var */ 
void get_env (inpath, outpath)
   char *inpath;   /* input:   path that may have an env var */
   char *outpath;  /* output:  path with no env var */ 
{
/* declare vars */ 
   char *tevar;    /* translated environment var */ 
/* return if inpath is not an env var */ 
   if (*inpath != '$')
   {
      strcpy (outpath, inpath); 
      return;
   }
/* eliminate leading '$' on env var */ 
   inpath++;           
/* translate env var to disk path */ 
   if ((tevar = getenv (inpath)) == NULL)
   {
      sprintf (message, "Unable to get value of environment variable $%s", 
       inpath); 
      c_errmsg (message, procfatal, &fatal); 
   }
   strcpy (outpath, tevar);
}





/* FUNCTION:  read sfile (tae parm SPROC) to get image names */ 
void read_sites (sfile, sarea, namelist, namecount)
   char *sfile;                /* input:  read this file */
   char *sarea;                /* input:  get image names for this study area */
   char namelist[MAXNAMES][CMLEN]; /* output:  list of image names */ 
   long *namecount;                /* output:  number of names in namelist */
{
/* declare vars */ 
   FILE *filep;         /* points to sfile */ 
   char iinline[INLINE], /* a line of sfile */ 
	*tocopy,        /* points to string to copy */ 
	*tofix;         /* a character to fix */ 
   long crit_found,     /* flag, was line with "CRITICAL SECTION" found yet */
	sarea_found;    /* flag, was line with sarea found yet */ 
/* check file existence, 
   c_getnam() was not called but file should certainly exist */ 
   if (access (sfile, 0))
   {
      sprintf (message, "SPROC file %s does not exist", sfile);
      c_errmsg (message, procfatal, &fatal);
   }
/* open file */ 
   filep = fopen (sfile, "r");
   if (filep == NULL)
   {   
      sprintf (message, "Unable to open SPROC file %s for reading", sfile); 
      c_errmsg (message, procfatal, &fatal);
   }
/* process file */ 
   for (*namecount=0, crit_found = sarea_found = FALSE; ; )
   {
   /* read a line, check for end of file errors */ 
      if (fgets (iinline, INLINE, filep) == NULL)
      {
	 if (!crit_found)
	 {
	    sprintf (message, 
	     "String CRITICAL SECTION was not found in SPROC file %s", sfile);
	    c_errmsg (message, procfatal, &fatal); 
	 }
	 if (!sarea_found)
	 {
	    sprintf (message, 
	     "Did not find start of TAE 'if' logic for SAREA %s in SPROC file %s", 
	      sarea, sfile);
	    c_errmsg (message, procfatal, &fatal);
	 }
	 /* this error is unlikely */ 
	 sprintf (message, 
	  "Did not find end of TAE 'if' logic for SAREA %s in SPROC file %s", 
	  sarea, sfile);
	 c_errmsg (message, procfatal, &fatal);
      } 
   /* skip the top of the file up to "CRITICAL SECTION" */ 
      if (!crit_found)
      {
	 if (strstr (iinline, "CRITICAL SECTION") != NULL)
	    crit_found = TRUE; 
	 continue; 
      }
   /* skip the file up to sarea */ 
      if (!sarea_found)
      {
	 if (check_control (iinline) && strstr (iinline, sarea) != NULL)
	    sarea_found = TRUE;
	 continue; 
      }
   /* skip over an info message */ 
      if ( strstr (iinline, "putmsg") != NULL)
	 continue; 
   /* stop processing file at the next control statement */ 
      if (check_control (iinline))
	 break; 
   /* copy image name from this line of text */ 
      /* error check:  too many names */ 
	 if (*namecount == MAXNAMES)
	 {
	    sprintf (message, 
	     "More than %d image names were in SPROC file %s for SAREA %s", 
	     MAXNAMES, sfile, sarea);
	    c_errmsg (message, procfatal, &fatal);
	 }
      /* eliminate trailing white space */
	 for (tofix=iinline+strlen(iinline) -1;  isspace(*tofix);  tofix--)
	    *tofix = '\0'; 
      /* find last white space char in the string */
	 for (tocopy=iinline+strlen(iinline) -1;  !isspace(*tocopy);  tocopy--)
	    ; 
      /* copy image name */ 
	 tocopy++; 
	 strcpy (namelist[ (*namecount)++ ], tocopy); 
   }
}





/* FUNCTION:  write a list of image names to the user */ 
void write_list (heading0, heading1, namelist, namecount)
   char *heading0;  /* input:  heading if the list has   0 entries */ 
   char *heading1;  /* input:  heading if the list has > 0 entries */ 
   char namelist[MAXNAMES][CMLEN]; /* input:  list of image names */ 
   long namecount;                 /* input:  number of names in namelist */
{
/* declare vars */ 
   long namenum;  /* for loop index, image name */ 
/* heading */ 
   if (namecount == 0) 
      c_errmsg (heading0, "informational", &go_on); 
   else 
      c_errmsg (heading1, "informational", &go_on); 
/* list image names */ 
   for (namenum=0; namenum<namecount; namenum++)
   {
      sprintf (message, "   %2d) %s", namenum+1, namelist[namenum] ); 
      c_errmsg (message, "informational", &go_on); 
   }
}





/* FUNCTION:  return TRUE if input line is part of the control of an 
   if-then-else in a pdf */ 
long check_control (iinline)   
   char *iinline;  /* input:  line to check */ 
{
/* the beginning of an if-then-else */ 
   if (strstr (iinline, "if")      != NULL && strstr (iinline, "sarea") != NULL &&
   strstr (iinline, "=") != NULL)
       return (TRUE); 
/* else-if */ 
   if (strstr (iinline, "else-if") != NULL && strstr (iinline, "sarea") != NULL &&
   strstr (iinline, "=") != NULL)
      return (TRUE);
/* end-if */ 
   if (strstr (iinline, "end-if")  != NULL)
      return (TRUE);
/* line is not part of control flow */ 
   return (FALSE); 
}





/* FUNCTION:  complete image and file names */
void complete_names (uparms, fnames)
   struct UPARMS *uparms; /* input:  user parms needed to complete file names */
   struct FNAMES *fnames; /* input:  image and file names to complete
			     output:  completed names */ 
{
/* declare vars */
   long namenum;          /* for loop index, name number */ 
/* process all names */ 
   for (namenum=0;  namenum<fnames->count;  namenum++)
   {
   /* image/file names */ 
      sprintf (fnames->taewild[namenum],  "%s*%s", uparms->imagedirtae, 
       fnames->basic[namenum] ); 
      sprintf (fnames->hostwild[namenum], "%s*%s.img", uparms->imagedirhost, 
       fnames->basic[namenum] ); 
      sprintf (fnames->paxfiles[namenum], "*%s*", 
       fnames->basic[namenum] ); 
      fnames->dellist [namenum] = fnames->taewild[namenum]; 
   /* pax log file names */ 
      sprintf (fnames->logtae[namenum],  "%s%s.%s", uparms->logdirtae,
       uparms->baseadd, fnames->basic[namenum] ); 
      sprintf (fnames->loghost[namenum], "%s%s.%s", uparms->logdirhost,
       uparms->baseadd, fnames->basic[namenum] ); 
      if (uparms->paxcount) 
      {
	 sprintf (fnames->paxtae[namenum],  "%s%s.%s", uparms->paxdirtae,
	  uparms->baseadd, fnames->basic[namenum] ); 
	 sprintf (fnames->paxhost[namenum], "%s%s.%s", uparms->paxdirhost,
	  uparms->baseadd, fnames->basic[namenum] ); 
      } 
   }
}





/* FUNCTION:  make sure an image of each name exists */ 
void image_exist (fnames, paxcount)
   struct FNAMES *fnames;  /* input:   list of images to check
			      output:  list of images which exist */ 
   long paxcount;          /* input:   flag, do fnames->paxtae and
                              fnames->paxhost have valid values */ 
{
/* declare vars */
   FILE *filep;         /* points to a temp file */ 
   char syscmd[SYSCMD]; /* a call to the operating system */ 
   long foundcount,     /* output:  number of image names that were found */
        namenum;        /* for loop index, image name */ 
/* remove LISTFILE if it exists */ 
   rm_file (LISTFILE); 
/* check all names for existence */ 
   for (foundcount=namenum=0;  namenum<fnames->count;  namenum++)
   {
   /* make Bourne shell system command, ignore std err */ 
      sprintf (syscmd, "ls %s > %s 2> /dev/null", fnames->hostwild[namenum], 
       LISTFILE);
   /* call system */ 
      if (system (syscmd) == -1)
      {
	 sprintf (message, "Call to system() failed with errno %d", errno);
	 c_errmsg (message, procfatal, &fatal); 
      }
   /* open file created by system() */ 
      filep = fopen (LISTFILE, "r");
      if (filep == NULL)
      {
	 sprintf (message, "Unable to open file %s for reading", LISTFILE);
	 c_errmsg (message, procfatal, &fatal);
      }
   /* optionally revise the input list 
      if LISTFILE contains any data, a file of the given name exists */ 
      if ( fgetc (filep) != EOF)   
      {
	 if (foundcount != namenum)
	 {
	    strcpy (fnames->basic   [foundcount], fnames->basic   [namenum]);
	    strcpy (fnames->taewild [foundcount], fnames->taewild [namenum]);
	    strcpy (fnames->hostwild[foundcount], fnames->hostwild[namenum]);
	    strcpy (fnames->paxfiles[foundcount], fnames->paxfiles[namenum]);
	    strcpy (fnames->logtae  [foundcount], fnames->logtae  [namenum]);
	    strcpy (fnames->loghost [foundcount], fnames->loghost [namenum]);
	    if (paxcount)
	    {
	       strcpy (fnames->paxtae  [foundcount], fnames->paxtae  [namenum]);
	       strcpy (fnames->paxhost [foundcount], fnames->paxhost [namenum]);
	    }
	    /* copying a pointer must be last */ 
	    fnames->dellist[foundcount] = fnames->taewild[foundcount];
	 }
	 foundcount++;
      }
      fclose (filep);
   }
/* remove temp file */ 
   rm_file (LISTFILE); 
/* revise count of files/images */ 
   fnames->count = foundcount; 
}





/* FUNCTION:  check for pre-existing log files.  append _file2 (or _file3 or 
   ...) to the log file names if the files pre_exist. */ 
void nodupe_names (paxcount, fnames)
   long paxcount;         /* input:  flag, will any PAXDIR files be created */
   struct FNAMES *fnames; /* input:  file names to check for existence 
			     output:  revised file names that dont exist */ 
{
/* declares vars */
   char newend[20],       /* string to optionally append to existing name */ 
	testlog[CMLEN],   /* tentative new LOGDIR file name */ 
	testpax[CMLEN];   /* tentative new PAXDIR file name */ 
   long exist,            /* flag, does either file exist */ 
        nameadd,          /* integer to append to existing name */ 
        namenum;          /* for loop index, name number */ 
/* process all file names */ 
   for (namenum=0;  namenum<fnames->count;  namenum++)
   {
   /* loop until a good name is found */ 
      for (nameadd=1;  ;  nameadd++)   
      {
      /* build file names to check for existence */ 
	 if (nameadd == 1) /* use old name */ 
	    newend[0] = '\0'; 
	 else              /* make new names */ 
	    sprintf (newend, "_file%d", nameadd);
	 sprintf (testlog, "%s%s", fnames->loghost[namenum], newend); 
	 if (paxcount) 
	    sprintf (testpax, "%s%s", fnames->paxhost[namenum], newend); 
      /* if neither file exists ... */ 
	 exist = !access (testlog, 0) || paxcount && !access (testpax, 0);
	 if (!exist) 
	 {
	 /* file(s) do not exist, make tae and host names */ 
	    strcat (fnames->loghost[namenum], newend);
	    strcat (fnames->logtae [namenum], newend);
	    if (paxcount)
	    {
	       strcat (fnames->paxhost[namenum], newend);
	       strcat (fnames->paxtae [namenum], newend);
	    }
	    break; 
	 }
      }
   }
}





/* FUNCTION:  write files to pax tapes, return TRUE if an error was
   encountered. */ 
long write_pax (dpart3, uparms, fnames, errfiles)
   char *dpart3;           /* input:  part 3 of pax tape description */ 
   struct UPARMS *uparms;  /* input:  user tae parms */
   struct FNAMES *fnames;  /* input:  list of image names to write to tape, 
    if an error is encountered fnames will be changed to the list of images 
    successfully written to tape */ 
   struct ERRFILES *errfiles; /* output:  list of output files with errors. */
{
/* declare vars */
   FILE *tlabel;           /* pax tape label file */ 
   char cdabort[CMLEN],    /* file used to flag cd errors */ 
        comment[150],      /* ops comment for output tape */ 
	description[120],  /* description for output tape label */ 
	labelfile[CMLEN],  /* file for pax tape label */ 
	paxabort[CMLEN],   /* file used to flag pax errors */ 
        syscmd[SYSCMD];    /* a call to the operating system */ 
   long effort,            /* for loop index, effort at making a pax tape */ 
        namenum,           /* for loop index, image name */ 
	paxwork;           /* flag, did call to pax work */ 
/* file names must include full path */ 
   sprintf (cdabort,   "%s%s", uparms->imagedirhost, CDABORT);
   sprintf (labelfile, "%s%s", uparms->imagedirhost, LABELFILE);
   sprintf (paxabort,  "%s%s", uparms->imagedirhost, PAXABORT);
/* user message */ 
   sprintf (message, "Status of copying %d file sets to pax tapes:", 
    fnames->count);
   c_errmsg (message, "informational", &go_on);
/* remove files if they exist */ 
   rm_file (cdabort); 
   rm_file (labelfile); 
   rm_file (paxabort); 
/* process all images, allowing two efforts at calling pax */ 
   for (namenum=0;  namenum<fnames->count;  namenum++)
      for (effort=0; ; effort++)
      {
      /* form comment and description for tape */ 
	 sprintf (description, "%s %s %s %s", fnames->basic[namenum], 
	  uparms->dpart2, dpart3, uparms->dpart4); 
	 sprintf (comment, "%s:  %s", uparms->ttype, description); 
      /* optionally prompt ops */ 
	 if (effort)
	    send_message (comment, effort);  
      /* error check:  too many efforts at calling pax */
	 if (effort ==2)
	 {
	    fnames->count = namenum;   /* return n images written ok */ 
	    rm_file (labelfile);       /* remove tape label file if exists */ 
	    return (TRUE);             /* error was encountered */ 
	 }
      /* generate Bourne shell system call that makes files upon error, pax
	 must use a relative path, hence the call to cd */
	 sprintf (syscmd,                "cd %s \n", uparms->imagedirhost); 
	 sprintf (syscmd+strlen(syscmd), "if [ $? != 0 ] \n"); 
	 sprintf (syscmd+strlen(syscmd), "then \n"); 
	 sprintf (syscmd+strlen(syscmd), "   echo > %s \n", cdabort); 
	 sprintf (syscmd+strlen(syscmd), "   exit \n"); 
	 sprintf (syscmd+strlen(syscmd), "fi \n"); 
	 sprintf (syscmd+strlen(syscmd), "\n"); 
	 sprintf (syscmd+strlen(syscmd), 
	  "pax -wqvLb 10240 -M '%s' -P %s > %s 2>&1 \n", comment, 
	  fnames->paxfiles[namenum], fnames->loghost[namenum]); 
	 sprintf (syscmd+strlen(syscmd), "if [ $? != 0 ] \n"); 
	 sprintf (syscmd+strlen(syscmd), "then \n"); 
	 sprintf (syscmd+strlen(syscmd), "   echo > %s \n", paxabort); 
	 sprintf (syscmd+strlen(syscmd), "fi \n"); 
      /* debug 
	 printf ("syscmd=\n%s\n", syscmd); */ 
      /* make pax tape label */ 
	 tlabel = fopen (labelfile, "w");
	 if (tlabel == NULL)
	 {
	    sprintf (message, "Unable to open file %s for writing", labelfile);
	    c_errmsg (message, procfatal, &fatal);
	 }
	 fprintf (tlabel, "CODE: 054\n");
	 fprintf (tlabel, "DESCRIPTION: %s%s\n", description, 
	  effort==0 ? "" : " try 2");
	 fprintf (tlabel, "PROJECT: pathfinder\n"); 
	 fprintf (tlabel, "REQUESTOR: %s\n", uparms->tapename); 
	 fprintf (tlabel, "SCRATCH: **/**/****\n"); 
	 fclose (tlabel); 
      /* run pax by calling system */ 
	 if (system (syscmd) == -1)
	 {
	    sprintf (message, "Call to system() failed with errno %d", errno);
	    c_errmsg (message, procfatal, &fatal); 
	 }
      /* check call to cd for error */ 
	 if ( !access (cdabort, 0))
	 {
	    sprintf (message, "Unable to cd to directory %s", 
	     uparms->imagedirtae); 
	    c_errmsg (message, procfatal, &go_on);
	    rm_file (cdabort);
	    c_errmsg ("Program aborted", procfatal, &fatal); 
	 }
      /* determine success of pax call, write user messge */ 
	 paxwork = access (paxabort, 0);
	 sprintf (message, "   %2d) %s  %s", namenum+1, 
	  fnames->taewild[namenum], paxwork ? "success" : "pax error"); 
	 c_errmsg (message, "informational", &go_on);
      /* optionally copy log file from logdir to paxdir and change owner */ 
	 if (paxwork && uparms->paxcount)
	 {
	    copy_file (fnames->loghost[namenum], fnames->paxhost[namenum] ); 
	    if (chown (fnames->paxhost[namenum], uparms->paxuid, -1))
	    {
	       sprintf (message, "Unable to change owner of PAXDIR file %s", 
		fnames->paxhost[namenum] ); 
	       c_errmsg (message, procfatal, &fatal); 
	    }
	 }
      /* rename log file if pax failed */
	 if (!paxwork)
	 {
	 /* get name for error file */ 
	    new_name (fnames->loghost[namenum], effort+1, 
	     errfiles->host[ errfiles->count ],
	     errfiles->tae [ errfiles->count ] ); 
	 /* rename file and increment error file count */ 
	    if (rename (fnames->loghost[namenum], 
	     errfiles->host[ errfiles->count ] ))
	    {
	       sprintf (message, 
	       "Unable to rename pax error log %s", fnames->loghost[namenum]);
	       c_errmsg (message, procfatal, &fatal);
	    }
	    errfiles->count++; 
	 }
      /* break loop if pax was a success */ 
	 if (paxwork) 
	    break; 
	 rm_file (paxabort);  /* remove file */ 
      }
/* set return code */
   return (FALSE); 
}





/* FUNCTION:  send a message to the operators about a bad tape */ 
void send_message (comment, effort)
   char *comment;       /* input:  comment describing the bad tape */ 
   long effort;         /* input:  n iterations of bad tapes, used to select
			   message to write */ 
{
/* declare vars */
   char syscmd[SYSCMD]; /* call to operating system */ 
/* make system command */ 
#ifdef TESTING 
   sprintf (syscmd,                "mail zokaites << END\n");
#else 
   sprintf (syscmd,                "write ops << END\n");
#endif 
   if (effort == 1) 
   {
      sprintf (syscmd+strlen(syscmd), "\n"); 
      sprintf (syscmd+strlen(syscmd), "Dear Operator, \n"); 
      sprintf (syscmd+strlen(syscmd), "\n"); 
      sprintf (syscmd+strlen(syscmd), "The first effort at writing data to tape failed.  Please \n"); 
      sprintf (syscmd+strlen(syscmd), "scratch the bad tape with the following description: \n"); 
      sprintf (syscmd+strlen(syscmd), "   %s\n", comment); 
      sprintf (syscmd+strlen(syscmd), "About to make a 2nd attempt at writing this data to tape.  For \n");
      sprintf (syscmd+strlen(syscmd), "the 2nd effort, please use a DIFFERENT tape.  A new tape mount \n");
      sprintf (syscmd+strlen(syscmd), "request will appear.  \n"); 
      sprintf (syscmd+strlen(syscmd), "\n"); 
      sprintf (syscmd+strlen(syscmd), "Thanks for your help. \n"); 
      sprintf (syscmd+strlen(syscmd), "END\n"); 
   }
   else if (effort == 2)
   {
      sprintf (syscmd+strlen(syscmd), "\n"); 
      sprintf (syscmd+strlen(syscmd), "Dear Operator, \n"); 
      sprintf (syscmd+strlen(syscmd), "Another effort at writing data to tape failed.  \n"); 
      sprintf (syscmd+strlen(syscmd), "Please scratch the bad tapes with this description: \n"); 
      sprintf (syscmd+strlen(syscmd), "%s\n", comment); 
      sprintf (syscmd+strlen(syscmd), "Thanks again. \n"); 
      sprintf (syscmd+strlen(syscmd), "END\n"); 
   }
   else
      c_errmsg ("Program error in send_message()", procfatal, &fatal); 
/* call system */
   if (system (syscmd) == -1)
   {
      sprintf (message, "Call to system() failed with errno %d", errno);
      c_errmsg (message, procfatal, &fatal); 
   }
}





/* FUNCTION:  copy a file */ 
void copy_file (infile, outfile)
   char *infile;       /* input:  file to copy, corresponds to LOGDIR */ 
   char *outfile;      /* input:  name for new file, corresponds to PAXDIR */ 
{
/* declare variables */
   char aline[INLINE]; /* a line of text to copy */ 
   FILE *input,        /* points to infile */
	*output;       /* points to outfile */ 
/* remove pre-existing output file to get around file protection */ 
   rm_file (outfile); 
/* open files */ 
   if ((input = fopen (infile, "r")) == NULL)
   {
      sprintf (message, "Unable to open file %s for reading", infile);
      c_errmsg (message, procfatal, &fatal);
   }
   if ((output = fopen (outfile, "w")) == NULL)
   {
      sprintf (message, "Unable to open PAXDIR file %s for writing", outfile);
      c_errmsg (message, procfatal, &fatal);
   }
/* copy text */ 
   while (fgets (aline, INLINE, input) != NULL)
      if (fputs (aline, output) == EOF)
      {
	 sprintf (message, "Error writing to PAXDIR file %s", outfile); 
	 c_errmsg (message, procfatal, &fatal);
      }
/* close files */ 
   fclose (input);
   fclose (output); 
}





/* FUNCTION:  return name for an error file, file name will not already exist
   on disk */ 
void new_name (oldhost, effort, newhost, newtae)
   char *oldhost;   /* input:  old host file name */ 
   long effort;     /* input:  append this initial value to oldhost */ 
   char *newhost;   /* output:  revised host file name */ 
   char *newtae;    /* output:  revised tae  file name */ 
{
/* build new host name */ 
   for (;;  effort++)
   {
      /* build new name */ 
	 sprintf (newhost, "%s_try%d", oldhost, effort); 
      /* break loop if file does not exist */ 
	 if (access (newhost, 0))
	    break; 
   }
/* get new tae name */
   strcpy(newtae, newhost);
}





/* FUN
   CTION:  remove a file if it exists */ 
void rm_file (filename)
   char *filename;  /* input:  name of file to remove */ 
{
/* return if file does not exist */ 
   if ( access(filename,0) )
      return; 
/* remove file */ 
   if (remove (filename) )
   {
      sprintf (message, "Unable to delete file %s", filename); 
      c_errmsg (message, procfatal, &fatal);
   }
}


