
/******************************************************************************
NAME:    SATATT, a c++ program
PURPOSE: update satellite attitude in an addr 

  Date      Programmer         Changes
--------  --------------  -----------------------------------
May 1996  David Zokaites  Original development
			  change name from UPADDR to SATATT 
Sep 1996  David Zokaites  Updated makefile to use environment variables, 
			  renamed satatt.c satatt.C, minor tweaks in satatt.C 
******************************************************************************/



// include files
#include <stdlib.h>     // for use of abs
#include <math.h>     // for use of M_PI 
#include <string.h>   // for calling strcmp()  
#include "model.h"    // for use of the ADAPS model  
extern "C"
{
#include "las.h"      // las include file 
#include "util.h"
}


// class for user tae parameters 
   class UPARMS          
   {
   private: 
      char taename[CMLEN],  // tae  name of INOUT file
	   hostname[CMLEN]; // host name of INOUT file
      double roll,          // satellite roll  to assign in degrees
	     pitch,         // satellite pitch to assign in degrees
	     yaw;           // satellite yaw   to assign in degrees
   public: 
	   UPARMS  (void) {}   ;  // a null constructor 
      void getpar  (void)      ;  // the real constructor 
      void update  (void) const;
      void message (void) const;
   }; 







// prototype 2 adaps functions
   extern "C"
   {
      int c_getaddr (const char *host_img,       struct MODEL *model); 
      int c_putaddr (const char *host_img, const struct MODEL *model); 
   };






/******************************************************************************
NAME:     main()
PURPOSE:  program main
******************************************************************************/
int main (void)                 
{	                    
// declare variables 
   class UPARMS uparms;  // user tae parameters 

// initialize las and get tae parameters 
   uparms.getpar (); 

// update file
   uparms.update ();

// end program 
   uparms.message ();
}








/******************************************************************************
NAME:     getpar()
PURPOSE:  get and return user tae parameters
          class UPARMS is updated and returned 
******************************************************************************/
void UPARMS::getpar (void) 
{
// declare variables 
   struct VARIABLE *tparm;   // points to tae parameters
   struct PARBLK pblock;     // tae parameter block 
   long readacc=IREAD,       // file access mode, parm to c_getnam()
        std_out,             // standard output, parm to c_inlas() 
        zero=0;              // value of zero, parm to c_getnam()

// initialize las and tae 
   c_inlas (&pblock, &std_out); 

// get file names, check file for read access
   if (c_getnam (&pblock, "inout", &zero, hostname, "addr", 
   &readacc) != E_SUCC)
      c_lasmsg (TRUE, "satatt-fatal", "Fatal error encountered"); 

   strcpy(taename, hostname);
   
// get attitude
   tparm = p_find (&pblock, "roll");    roll  = RVAL (*tparm,0);
   tparm = p_find (&pblock, "pitch");   pitch = RVAL (*tparm,0);
   tparm = p_find (&pblock, "yaw");     yaw   = RVAL (*tparm,0);

// convert attitude to radians if it was entered in degrees
   tparm = p_find (&pblock, "units");
   if (strcmp (SVAL(*tparm,0), "DEGREES") == 0)
   {
      roll  *= M_PI / 180.0;
      pitch *= M_PI / 180.0;
      yaw   *= M_PI / 180.0;
   } 

// user message
   c_lasmsg (FALSE, "informational", 
   "Roll = %12.10f Pitch = %12.10f Yaw = %12.10f   radians", 
   roll, pitch, yaw); 

// error check
   if (abs(roll ) > 0.16 ||
       abs(pitch) > 0.16 ||
       abs(yaw  ) > 0.16  )
      c_lasmsg (TRUE, "satatt-max", 
      "Absolute value of attitude exceeded max of 0.16 radians"); 

}








/******************************************************************************
NAME:     update()
PURPOSE:  update the addr
******************************************************************************/
void UPARMS::update (void) const 
{
// declare variable 
   struct MODEL model;  // avhrr model described by the addr 

// read the addr 
   if (c_getaddr (hostname, &model) != E_SUCC)
      c_lasmsg (TRUE, "satatt-fatal", "Fatal error encountered");

// change attitude 
   model.roll_coefs [0] = roll ; 
   model.pitch_coefs[0] = pitch; 
   model.yaw_coefs  [0] = yaw  ; 

// make sure other components of attitude are all zero
   for (long acheck=1;  acheck<5;  acheck++)
      if (
      model. roll_coefs [acheck] != 0.0 ||
      model.pitch_coefs [acheck] != 0.0 ||
      model.  yaw_coefs [acheck] != 0.0  )
      {
	 model. roll_coefs [acheck] = 0.0;
	 model.pitch_coefs [acheck] = 0.0;
	 model.  yaw_coefs [acheck] = 0.0;

	 c_lasmsg (FALSE, "satatt-warn", 
	 "Higher order attitude coefficients were reset to zero");
      }

// write a new addr 
   if (c_putaddr (hostname, &model) != E_SUCC)
      c_lasmsg (TRUE, "satatt-fatal", "Fatal error encountered");
}








/******************************************************************************
NAME:     message()
PURPOSE:  write a closing message for the user and exit  
******************************************************************************/
void UPARMS::message (void) const 
{
// declare variables 
   char outfiles[1][CMLEN];  // list of output files, parm to c_complt()
   long zero=0;              // value of zero, parm to c_complt()

// message 
   c_lasmsg (FALSE, "informational", "Program SATATT updated file:");
   c_lasmsg (FALSE, "informational", "   %s", taename);

// finish 
   c_complt ("SATATT", outfiles, &zero); 
}




