
/* functions.c:  functions called by main() */ 
#include "savefiles.h"  /* proc header file */ 
#include "diskio.h"     /* for call to c_dkcopy() */
#include <ctype.h>      /* for call to tolower() */ 
#include <sys/types.h>  /* for call to mkdir() */
#include <string.h>     /* for call to strrchr() */ 
#include <sys/stat.h>   /* for call to mkdir() */ 
#include <unistd.h>     /* for call to access() */
#include "util.h"





/* FUNCTION: initialize TAE and LAS processing. */ 
void begin_las
(
   char *proc             /* input:  name of las proc */ 
)

{
/* declare vars */ 
   long length,            /* length of a char string */ 
        std_out;           /* standard output */ 
/* initialize las */ 
   c_inlas (&pblock, &std_out); 
/* set global vars for proc name, some are uppercase, some are lowercase */ 
   strcpy  (procname, proc); 
   length = (long)strlen (procname);   
   c_low2up (procname, &length);  
   length = (long)strlen (proc);       
   c_up2low (proc, &length); 
   sprintf (procfatal, "%s-fatal", proc); 
   sprintf (procwarn,  "%s-warn",  proc); 
}





/* FUNCTION:  return int (instead of octal) file protection */ 
void get_prot
(
   char *parmname,  /* input:   get prot for this parm */ 
   long *prot       /* output:  dir or file protection */ 
)

{
/* declare variables */ 
   char converter[20];     /* format string used to convert type */ 
   struct VARIABLE *tparm; /* points to a tae parm in pblock */ 
/* get tae parm */ 
   tparm = p_find (&pblock, parmname);
   *prot = IVAL (*tparm, 0);
/* convert from octal to int */ 
   sprintf (converter, "%d", *prot); 
   sscanf  (converter, "%od", prot); 
}





/* FUNCTION:  return a TAE file name parm with multiple values */ 
void name_parm
(
   char *parmname,         /* input:  name of parameter */ 
   long lastpart,          /* input:  flag, return only last part of file name,
                                      not the whole path */ 
   long access,            /* input:  access mode for c_getnam() */
   struct nparm *parmval   /* output: parm values */ 
)

{
/* declare variables */ 
   char *nameonly;          /* points to last part of a file name */ 
   long index;              /* for loop index */ 
   struct VARIABLE *tparm;  /* points to a tae parm in pblock */ 
/* find count of parm */ 
   tparm = p_find (&pblock, parmname);
   parmval->count = (*tparm).v_count;
/* set parm values */ 
   for (index=0; index < parmval->count; index++)
   {
   /* get parm */ 
      if (c_getnam (&pblock, parmname, &index, parmval->value[index], "", 
      &access) ) 
      {
	 sprintf (message, "Error getting parm %s.", parmname); 
	 c_errmsg (message, procfatal, &fatal); 
      }

   /* get to last part of file name, eliminate the path */ 
      if (lastpart)
      {
	 if ( (nameonly = strrchr (parmval->value[index], '/')) == NULL)
	 {
	    sprintf (message, "%s %s is not a file name.", parmname, 
	    parmval->value[index] ); 
	    c_errmsg (message, procfatal, &fatal); 
	 }
	 strcpy (parmval->value[index], ++nameonly);
      }
   }
}





/* FUNCTION:  return a TAE string parm with multiple values, string will be
   converted to lower case */ 
void string_parm
(
   char *parmname,         /* input:  name of parameter */ 
   struct sparm *parmval   /* output: parm values */ 
)

{
/* declare variables */ 
   long index;                 /* for loop index */ 
   struct VARIABLE *tparm;     /* points to a tae parm in pblock */ 
/* find count of parm */ 
   tparm = p_find (&pblock, parmname);
   parmval->count = (*tparm).v_count;
/* set parm values */ 
   for (index=0; index < parmval->count; index++)
   {
      parmval->value[index] = SVAL(*tparm,index);
      strlower ( parmval->value[index] );
   }
}





/* FUNCTION:  convert input string to lower case */ 
void strlower
(
   char *string
)

{
   for (;  *string!='\0';  string++)
      *string = tolower(*string); 
}





/* FUNCTION:  make a directory if it does not exist */ 
void make_dir
(
   char *dirname,  /* input:  name of dir to create */ 
   long dirprot    /* input:  protection for the new dir */ 
)

{
/* check existence */ 
   if (!access(dirname,0) )
      return; 
/* create dir.  mkdir combines umask with the protection specified.  To 
   overcome this, chmod is called */ 
   if (mkdir (dirname, dirprot) || chmod (dirname, dirprot) ) 
   {
      sprintf (message, "Error creating directory %s", dirname); 
      c_errmsg (message, procfatal, &fatal); 
   }
/* update list of output dirs */ 
   strcpy (output.dirnames[output.dircount++], dirname); 
}





/* FUNCTION:  copy a file */ 
void copy_file
(
   char *infile,   /* input:  unix name of input file, incl path */
   char *outfile,  /* input:  unix name of output file, incl path */
   char *outdir,   /* input:  unix name for output dir */ 
   long fileprot   /* input:  protection for output file */ 
)

{
/* declare and set var, directories arg to c_dkcopy() */ 
   char dirlist[2][CMLEN];
   strcpy (dirlist[0], outdir);  /* put files in this dir */ 
   dirlist[1][0] = '\0';         /* a null dir to terminate dirs */ 
/* the calling function checks infile read access by calling c_getnam().
   c_dkcopy appends to an existing file, so if outfile exists I delete it */
/* update list of new files and check write access for outfile */ 
   if (access(outfile,0) ) /* a new file will be created */ 
      strcpy (output.newfiles[output.newcount++], outfile);   /* update list */
   else  /* file will be overwritten */ 
   {
      /* update list of files overwritten */ 
	 strcpy (output.ovfiles [output.ovcount++],  outfile); 
      /* remove file */ 
	 if (remove (outfile)) 
	 {
	    if (access (outfile, 2)) 
	    {
	       sprintf (message, "No write permission for file %s", outfile); 
	       c_errmsg (message, procfatal, &go_on); 
	    }
	    sprintf (message, "Error removing file %s", outfile); 
	    c_errmsg (message, procfatal, &fatal); 
	 }
   }
/* copy file */ 
   if (c_dkcopy (infile, outfile, dirlist) )
   {
      sprintf (message, "Error creating file %s", outfile); 
      c_errmsg (message, procfatal, &fatal); 
   }
/* change file protection */ 
   chmod (outfile, fileprot);
   /* This warning would come up too often, and everything works fine anyway. 
      [savefiles-warn] Error setting protection for file ___:  ___
      An effort is made to set the protection of new files created.  
      This effort succeeeds if the same user creates a subdir and puts a 
      file in that subdir.  If different users do these things, a 
      warning results.  
      if (chmod (outfile, fileprot) ) 
      {
	 sprintf (message, "Error setting protection for file %s", outfile); 
	 c_errmsg (message, procwarn, &go_on); 
      } */ 
}





/* FUNCTION:  write output files message.  This function gives a better
   description of the output created than c_complt() */ 
void write_output
(
   long nfiles,       /* input: number of files created */ 
   char outnames[PVALUES*2][FILEN], 
                      /* input: names of files created */ 
   char *action       /* input: action described as in "created files" */
)

{
/* declare vars */ 
   char tname[CMLEN]; /* las name of output files */ 
   long outf;         /* for loop index, n output files */ 
/* optionally return */ 
   if (!nfiles)
      return;  
/* output header */ 
   sprintf (message, "%s %s:", procname, action); 
   c_errmsg (message, "informational", &go_on);
/* output files, translating host names to tae names */ 
   for (outf=0; outf<nfiles; outf++)
   {
      strcpy(tname, outnames[outf]);
      sprintf (message, "   %s", tname); 
      c_errmsg (message, "informational", &go_on); 
   }
}


