
/******************************************************************************
NAME:     swindow.c 
PURPOSE:  program main()
	  SWINDOW returns scene window in registered image. 

PROGRAM HISTORY
-------------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  ----------------------------------------------------
Jan 1994  David Zokaites   Original developoment for $DCONTRIB (ddps contrib);
			   application later released to $LASDITTY. 
Mar 1994  David Zokaites   Expanded help;  changed comments for clarity;
			   scoors now = acoors +/- npixels * pixsize;  
			   output scoors now has the same origin as
			   the reference image
Oct 1995  David Zokaites   Updated the users guide and Makefile, released to
                           $ADAPSCONTRIB
******************************************************************************/


#include <stdlib.h>   /* for call to atof */ 
#include <math.h>     /* for call to ceil(), floor(), fabs() */ 
#include "swindow.h"  /* proc header file */ 





void main () 
{
/* declare variables */ 
   char *scoors[NCOR];       /* bounded range of scene as a char */ 
   double lng[NCOR],         /* corner coordinates of scene */ 
	  lat[NCOR],         /* corner coordinates of scene */ 
	  xcor[NCOR],        /* reprojected corner coordinates of scene */ 
	  ycor[NCOR],        /* reprojected corner coordinates of scene */ 
          xrange[RVALS],     /* bounded range of scene, min and max */ 
          yrange[RVALS];     /* bounded range of scene, min and max */ 
   long index,               /* for loop index */ 
        swindow[NCOR],       /* output window in reference image */ 
        zero = 0;            /* number of output files */ 
   struct PROJINFO projinfo; /* input & output projection parameters */
   /* allocate memory for scoors; scoors must be declared char *scoors[] to pass
      it back to TAE. */ 
   for (index=0; index<NCOR; index++)
      if ((scoors[index] = (char *) malloc (sizeof(char) * CPROJ)) ==NULL)
	 c_errmsg ("Unable to allocate memory", procfatal, &fatal); 
/* initialize las and tae */ 
   begin_las ("swindow"); 
/* get the lat/lng coordinates of scene corners */ 
   get_corners (lat, lng); 
/* get projection parameters from the projection parameter file */ 
   get_project (&projinfo);
/* adjust projection units.  get_corners() left the corners in radians, but 
   the proj parm file thinks they are in degrees */ 
   projinfo.in_units = 0;
/* reproject corner coordinates */ 
   new_cor (&projinfo, lat, lng, xcor, ycor); 
/* limit corners to size of the reference image */ 
   limit_cor (xcor, ycor, xrange, yrange, scoors); 
/* translate projection coordinates to window in reference image */ 
   lswindow (xrange, yrange, swindow); 
/* return data to tae */ 
   if (
   q_string (&pblock, "scoors",  NCOR, scoors,  P_UPDATE) !=SUCCESS || 
   q_intg   (&pblock, "swindow", NCOR, swindow, P_UPDATE) !=SUCCESS || 
   q_out    (&pblock)                                     !=SUCCESS  ) 
      c_errmsg ("Error returning parameter values to tae.", procfatal, 
      &fatal); 
/* completion message */ 
   sprintf (message, "Returned SCOORS=(%s %s %s %s).", 
   scoors[0], scoors[1], scoors[2], scoors[3]);
   c_errmsg (message, "informational", &go_on); 
   sprintf (message, "Returned SWINDOW=(%d %d %d %d).", 
   swindow[0], swindow[1], swindow[2], swindow[3]);
   c_errmsg (message, "informational", &go_on); 
   c_complt(procname, "", &zero);
}





/* FUNCTION:  initialize las and tae */ 
void begin_las (proc) 
   char *proc;       /* input:  name of las proc */ 
{
/* declare vars */ 
   long length,      /* length of a char string */ 
        std_out;     /* standard output */ 
/* initialize las */ 
   c_inlas (&pblock, &std_out); 
/* set global vars for proc name, some are uppercase, some are lowercase */ 
   strcpy  (procname, proc); 
   length = strlen (procname);   
   c_low2up (procname, &length);  
   length = strlen (proc);       
   c_up2low (proc, &length); 
   sprintf (procfatal, "%s-fatal", proc); 
}





/* FUNCTION:  return the lat/long coordinates of scene corners.  Function was
   based on corners-four adaps proc.  Implicit input:  tae parm inimg, input
   image name */ 
void get_corners (lat, lng)
   double lat[NCOR];   /* output:  lat at the corners */ 
   double lng[NCOR];   /* output:  long at the corners */ 
{
/* declare variables */ 
   char host_img[CMLEN];  /* host name of input image */
   double  elev = 0.0;    /* Elevation */
   float   line,          /* Line number relative to original pass */
	   samp;          /* Sample number relative to original pass */
   long access = IREAD,   /* file access mode */
        zero = 0;         /* value of zero */
   struct MODEL model;    /* Model and orbital information */
   struct DDR   ddr;      /* LAS image Data Descriptor Record */
/* get input image name */
   if (c_getnam(&pblock,"INIMG",&zero,host_img,"img",&access))
      c_errmsg ("Unable to get host name for input image.", procfatal, &fatal); 
/* get model information from associated avhrr ddr */
   if (c_getaddr(host_img, &model))
      c_errmsg ("Unable to get avhrr ddr.", procfatal, &fatal); 
   if (c_getddr(host_img, &ddr))
      c_errmsg ("Unable to get ddr.", procfatal, &fatal); 
/* determine coorner coordinates */
   /* upper left corner */ 
      line = (float)(ddr.master_line   + 1);
      samp = (float)(ddr.master_sample + 1);
      c_ls2ll(&line,&samp,&elev,&model, lat+0, lng+0, NULL);
   /* upper right */ 
      line = (float)(ddr.master_line   + 1);
      samp = (float)(ddr.master_sample + ddr.ns);
      c_ls2ll(&line,&samp,&elev,&model, lat+1, lng+1, NULL);
   /* lower left */ 
      line = (float)(ddr.master_line   + ddr.nl);
      samp = (float)(ddr.master_sample + 1);
      c_ls2ll(&line,&samp,&elev,&model, lat+2, lng+2, NULL);
   /* lower right */ 
      line = (float)(ddr.master_line   + ddr.nl);
      samp = (float)(ddr.master_sample + ddr.ns);
      c_ls2ll(&line,&samp,&elev,&model, lat+3, lng+3, NULL);
}





/* FUNCTION:  return projection parameters from the projection parameter file.  
   Function was based on trancoord/getpar.c/get_proj_prm(). 
   Implicit inputs:  tae parms inproj, iprojkey, oprojkey. */ 
void get_project (projinfo)
   struct PROJINFO *projinfo; /* output:  projection parameters for input and
                                 output projections. */ 
{
/* declare vars for tae parms */ 
   char inproj[CMLEN],  /* host name for tae parm inproj, input proj parm file*/
	iprojkey[10],   /* tae parm iprojkey, key for input projection */
	oprojkey[10];   /* tae parm oprojkey, key for output projection */
   long access = IREAD; /* file access mode, for c_getnam() */ 
   struct VARIABLE *tparm;    /* points to a tae var in pblock */ 
/* declare variables */ 
   struct TAB_DEF tab;        /* Tabular structure of label table */
   struct VECTOR *keyptr,     /* Pointer to key field */
                 *codeptr,    /* Pointer to projection code field */
                 *zoneptr,    /* Pointer to zone code field */
                 *unitptr,    /* Pointer to unit of measure field */
                 *sphptr,     /* Pointer to datum (spheroid) code field */
                 *parmptr,    /* Pointer to projection coefficients field */
                 *get_field_ptr();  /* Function to get field pointer */
   long inproj_found = FALSE,  /* Boolean--input projection found? */
	index,                 /* for loop index */ 
        outproj_found = FALSE, /* Boolean--output projection found? */
	zero = 0;              /* value of zero */ 
/* get tae parms */ 
   tparm = p_find (&pblock, "iprojkey");   strcpy (iprojkey, SVAL (*tparm, 0));
   tparm = p_find (&pblock, "oprojkey");   strcpy (oprojkey, SVAL (*tparm, 0));
   if (c_getnam(&pblock, "INPROJ", &zero, inproj, "proj", &access) )
      c_errmsg ("Unable to get host name for input proj parm file.", procfatal, 
      &fatal); 
/* open the labeled table file & retrieve the label */ 
   open_tab (&tab, inproj, IREAD);
   get_vector (&tab);
/* set pointers to fields in the file */ 
   if ((keyptr  = get_field_ptr(&tab, "PROJKEY"))   == NULL || 
       (codeptr = get_field_ptr(&tab, "PROJTYPE"))  == NULL || 
       (zoneptr = get_field_ptr(&tab, "PROJZONE"))  == NULL || 
       (unitptr = get_field_ptr(&tab, "PROJUNITS")) == NULL || 
       (sphptr  = get_field_ptr(&tab, "PROJSPH"))   == NULL || 
       (parmptr = get_field_ptr(&tab, "PROJPARMS")) == NULL  )
      c_errmsg ("Error reading the project parameter file.", procfatal, &fatal);
/* loop until key is found or end of file */ 
   for (;;)
   {
      if (get_record(&tab) ) 
	 c_errmsg ("Did not find projkey while reading the project parm file.",
	 procfatal, &fatal);
      if (!strcmp(keyptr->ptr.c, iprojkey) ) 
      {
	 projinfo->in_proj = *codeptr->ptr.i4;
	 projinfo->in_zone = *zoneptr->ptr.i4;
	 projinfo->in_units = *unitptr->ptr.i4;
	 projinfo->in_datum = *sphptr->ptr.i4;
	 for (index=0; index < 15; index++)
	    projinfo->in_parms[index] = parmptr->ptr.r8[index];
	 inproj_found = TRUE;
      }
      if (!strcmp(keyptr->ptr.c, oprojkey) ) 
      {
	 projinfo->out_proj = *codeptr->ptr.i4;
	 projinfo->out_zone = *zoneptr->ptr.i4;
	 projinfo->out_units = *unitptr->ptr.i4;
	 projinfo->out_datum = *sphptr->ptr.i4;
	 for (index=0; index < 15; index++)
	    projinfo->out_parms[index] = parmptr->ptr.r8[index];
	 outproj_found = TRUE;
      }
      if (inproj_found && outproj_found) 
	 break;
   }
close_tab(&tab);
}





/* FUNCTION:  reproject corner coordinates.  Function was based on 
   trancoord/map_proj.c/map_proj() */ 
void new_cor (projinfo, lat, lng, xcor, ycor)
   struct PROJINFO *projinfo;  /* input:  from & to projection parameters */
   double lat[NCOR];    /* input:  corner coordinate */ 
   double lng[NCOR];    /* input:  corner coordinate */ 
   double xcor[NCOR];   /* output: reprojected corner coordinate */ 
   double ycor[NCOR];   /* output: reprojected corner coordinate */ 
{
/* declare variable */ 
   long point,          /* for loop index */ 
        prtprm[2];	/* print flag for c_transinit() */
/* initialize the projection transformation package */ 
   prtprm[GEO_TERM] = FALSE;
   prtprm[GEO_FILE] = FALSE;
   if (c_transinit(&projinfo->in_proj, &projinfo->in_units, &projinfo->in_zone, 
   &projinfo->in_datum, projinfo->in_parms, &projinfo->out_proj, 
   &projinfo->out_units, &projinfo->out_zone, &projinfo->out_datum, 
   projinfo->out_parms, prtprm, NULL))
      c_errmsg ("Error initializing projection package.", procfatal, &fatal); 
/* reproject corner coordinates */ 
   for (point=0; point<NCOR; point++)
      if (c_trans (&projinfo->in_proj, &projinfo->in_units,&projinfo->out_proj, 
      &projinfo->out_units, lng+point, lat+point, xcor+point, ycor+point)) 
	 c_errmsg ("Error reprojecting corner point.", procfatal, &fatal); 
} 





/* FUNCTION:  limit scene corners to size of the reference image.  
   Implicit inputs:  tae parm ACOORS, the corners of the reference image, 
		     tae parm pixsize, reference image pixel size */ 
void limit_cor (xcor, ycor, xrange, yrange, scoors)
   double xcor[NCOR];      /* input:  scene x corners */ 
   double ycor[NCOR];      /* input:  scene y corners */ 
   double xrange[RVALS];   /* output: bounded range of scene, min and max */ 
   double yrange[RVALS];   /* output: bounded range of scene, min and max */ 
   char *scoors[NCOR];     /* output: bounded range of scene as a char, 
			      will be output tae parm scoors */ 
{
/* declare variables */ 
   double pixsize,         /* reference image pixel size */ 
          xextent[RVALS],  /* max x extent of the reference image */ 
	  yextent[RVALS];  /* max y extent of the reference image */ 
   long index;             /* for loop index */ 
   struct VARIABLE *tparm; /* points to a tae var in pblock */ 
/* get input tae parm */ 
   tparm=p_find(&pblock,"acoors");   
   xextent[0] = atof (SVAL(*tparm, 1));  /* minx */ 
   xextent[1] = atof (SVAL(*tparm, 3));  /* maxx */ 
   yextent[0] = atof (SVAL(*tparm, 2));  /* miny */ 
   yextent[1] = atof (SVAL(*tparm, 0));  /* maxy */ 
   tparm = p_find (&pblock, "pixsize"); 
   pixsize = RVAL (*tparm, 0); 
/* error check:  make sure min < max */ 
   if (xextent[0] > xextent[1] || 
       yextent[0] > yextent[1]  ) 
      c_errmsg ("Parm ACOORS is incorrect as minimum [xy] > maximum [xy].", 
      procfatal, &fatal);
/* find unbounded scene min and max */ 
   xrange[0] = yrange[0] =  1E+37;   /* set min to large value */ 
   xrange[1] = yrange[1] = -1E+37;   /* set max to small value */ 
   for (index=0; index<NCOR; index++)
   {
      if (xrange[0] > xcor[index]) /* xmin */ 
 	  xrange[0] = xcor[index]; 
      if (xrange[1] < xcor[index]) /* xmax */ 
	  xrange[1] = xcor[index]; 
      if (yrange[0] > ycor[index]) /* ymin */ 
 	  yrange[0] = ycor[index]; 
      if (yrange[1] < ycor[index]) /* ymax */ 
	  yrange[1] = ycor[index]; 
   }
/* bound min and max by input tae parm */ 
   /* loop only works for index=0 and index=1*/
   for (index=0; index<RVALS; index++)  
   {
      if (xrange[index] < xextent[0] || xrange[index] > xextent[1])  
	  xrange[index] = xextent[index]; 
      if (yrange[index] < yextent[0] || yrange[index] > yextent[1])  
	  yrange[index] = yextent[index]; 
   }
/* make sure scene extent is offset an even number of pixels from reference
   range.  This way the rectified image will have the same origin as the 
   reference image. */ 
   for (index=0; index<RVALS; index++)  
   {
      check_offset (pixsize, yextent[1], yrange+index);
      check_offset (pixsize, xextent[0], xrange+index); 
   }
/* convert bounded range to a char */ 
   sprintf (scoors[1], "%.2f", xrange[0]); 
   sprintf (scoors[3], "%.2f", xrange[1]); 
   sprintf (scoors[2], "%.2f", yrange[0]); 
   sprintf (scoors[0], "%.2f", yrange[1]); 
} 





/* FUNCTION:  translate projection coordinates to window in reference image.
   Implicit inputs:  tae parm pixsize, pixel size; 
                     tae parm ACOORS, the corners of the reference image. */
void lswindow (xrange, yrange, swindow)
   double xrange[RVALS]; /* input:  range of scene in projection coordinates, 
				    output as pixels */ 
   double yrange[RVALS]; /* input:  range of scene in projection coordinates, 
				    output as pixels */ 
   long swindow[NCOR];   /* output  window in reference image.  will be output
			    tae parm swindow */ 
{
/* declare variables */ 
   double pixsize,         /* input tae parm pixsize */ 
	  xorigin,         /* acoors[1], xmin for the reference image */ 
	  yorigin;         /* acoors[0], ymax for the reference image */  
   long point;             /* for loop index */ 
   struct VARIABLE *tparm; /* points to a tae var in pblock */ 
/* get tae parms */ 
   tparm = p_find (&pblock, "pixsize"); 
   pixsize = RVAL (*tparm, 0); 
   tparm = p_find(&pblock,"acoors");   
   xorigin = atof (SVAL(*tparm, 1)); 
   yorigin = atof (SVAL(*tparm, 0)); 
/* convert all vars to pixels */ 
   for (point=0; point<RVALS; point++)
   {
      xrange[point] /= pixsize; 
      yrange[point] /= pixsize; 
   }
   xorigin /= pixsize; 
   yorigin /= pixsize; 
/* determine window.  In the below lines, .4999 is for round off err.  For sl 
   and ss, 1 correct for 1 relative instead of 0 relative.  For nl and ns 1 
   corrects for vars being to the center of the pixel instead of the corner. */ 
   swindow[0] = (long) (yorigin   - yrange[1] + 1.4999);  /* sl */ 
   swindow[1] = (long) (xrange[0] - xorigin   + 1.4999);  /* ss */ 
   swindow[2] = (long) (yrange[1] - yrange[0] + 1.4999);  /* nl */ 
   swindow[3] = (long) (xrange[1] - xrange[0] + 1.4999);  /* ns */ 
/* error check:  make sure window is positive */ 
   for (point=0; point<NCOR; point++)
      if (swindow[point] < 1)
      {
	 sprintf (message, 
	 "Calculated SWINDOW (%d %d %d %d) had a value less than 1.", 
	 swindow[0], swindow[1], swindow[2], swindow[3]); 
	 c_errmsg (message, procfatal, &fatal); 
      }
}





/* FUNCTION:  make sure scene extent is offset an even number of pixels from 
   reference range.  This way the rectified image will have the same origin as 
   the reference image. */ 
void check_offset (pixsize, ref, scene)
   double pixsize;  /* input:  pixel size */ 
   double ref;      /* input:  reference image corner */ 
   double *scene;   /* output:  adjusted scene corner */ 
{
/* declare vars */ 
   double diffa, diffb, /* differences in n pixels */ 
	  npixels,      /* n pixels offset from ref to scene */ 
	  pixa, pixb;   /* big and small versions of npixels */ 
/* calculate npixels */ 
   npixels = (ref - *scene)/ pixsize; 
   pixa = ceil(npixels);
   pixb = floor(npixels);
/* determine rounding error */ 
   diffa = fabs (pixa - npixels);
   diffb = fabs (pixb - npixels);
/* minimize round off */ 
   if (diffa < diffb)
      *scene = ref - pixa * pixsize; 
   else
      *scene = ref - pixb * pixsize; 
} 
