
/* swindow.h:  proc header file */ 
#define NCOR   4 /* # of corner points for a scene, combining upleft and lorite 
	 	    in 1 array takes 4 values, lat at 4 corners has 4 values */ 
#define RVALS  2 /* number of values to define a range */ 
#define CPROJ 38 /* number of chars to define a projection coordinate */ 
#include "adaps.h"   /* to read AVHRR satellite model */ 
#include "las.h"     /* for las and tae */ 
#include "ltable.h"  /* to read labeled table of projection info */ 
#include "proj.h"    /* for GEO_TERM macro */ 





/* structure containing input and output projection information */ 
   struct PROJINFO
   {
      long in_proj;         /* Input projection code */
      long in_zone;         /* Input projection zone */
      long in_units;        /* Input projection units code */
      long in_datum;        /* Input projection datum code */
      long out_proj;        /* Output projection code */
      long out_zone;        /* Output projection zone */
      long out_units;       /* Output projection units code */
      long out_datum;       /* Output projection datum code */
      double in_parms[15];  /* Input projection GCTP parameters */
      double out_parms[15]; /* Output projection GCTP parameters */
   };





/* global vars, declared global for convenience */ 
   char message[CMLEN],  /* error message string, parm to c_errmsg()*/
        procname[30],    /* name of las proc */ 
        procfatal[40];   /* name of las proc and "-fatal" */
   long fatal= -1,       /* fatal error, parm to c_errmsg() */ 
        go_on=  1;       /* warning message, informational, parm to c_errmsg()*/
   struct PARBLK pblock; /* tae parameter block */ 





/* function prototypes. ifdef allows for compilers that don't allow prototypes*/
   struct VARIABLE *p_find();  /* function to find a tae parm in pblock */ 
#ifdef __DGUX__ 
   /* local functions */ 
      void main (void);
      void begin_las (char *proc); 
      void get_corners (double latitude[NCOR], double longitude[NCOR]); 
      void get_project (struct PROJINFO *projinfo);
      void new_cor (struct PROJINFO *projinfo, double lat [NCOR], 
  	   double lng[NCOR], double newx[NCOR], double newy[NCOR]); 
      void limit_cor (double xcor[NCOR], double ycor[NCOR], double 
	   xrange[RVALS], double yrange[RVALS], char *scoors[NCOR]); 
      void lswindow (double xrange[RVALS], double yrange[RVALS], 
	   long swindow[NCOR]); 
      void check_offset (double pixsize, double ref, double *scene);
   /* a las utility function */ 
      void c_errmsg (char *message, char *key, long *severity); 
#else /* old style prototypes */ 
   /* local functions */ 
      void main (); 
      void begin_las (); 
      void get_corners (); 
      void get_project (); 
      void new_cor (); 
      void limit_cor (); 
      void lswindow (); 
      void check_offset ();
#endif



