
/* bobmean3.c calculates XXX stuff mean nd and number of sample in mean for
       historical mean nd map (1989-1995) and 1996 nd data. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void usage(char **);

void main(int argc, char *argv[])
{
   FILE *nd_new_file, *nfilein, *nfileout, *meanndfilein, *meanndfileout;
   FILE *parmsfp, *minoutfp, *sosoutfp;
   char parms_fname[128];
   char nd_new[80],n_file_in[80], n_file_out[80], mean_nd_in[80];
   char min_file_in[80], sos_file_in[80];
   char min_file_out[128], sos_file_out[128];
   char mean_nd_out[128];
   int status = -1, n, row, col, imeannd, intnd96;
   int nlines, nsamps;
   double nd96, sum, meannd;

   if ( argc < 2 ) {
       usage( argv );
       exit(1);
   }

   if (( parmsfp = fopen( argv[1], "r")) == NULL ) {
       printf("Unable to open file: %s\n", argv[1] );
       exit(1);
   }
   if ( fscanf( parmsfp, "%d", &nlines ) != 1 ){
       printf("Unable to get number of lines\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%d", &nsamps ) != 1 ){
       printf("Unable to get number of samples\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", nd_new ) != 1 ){
       printf("Unable to get name of ND input file\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", mean_nd_in ) != 1 ){
       printf("Unable to get name of old mean ND input file\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", n_file_in ) != 1 ){
       printf("Unable to get name of old n values input file\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", min_file_in ) != 1 ){
       printf("Unable to get name of old min input file\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", sos_file_in ) != 1 ){
       printf("Unable to get name of old sos input file\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", mean_nd_out ) != 1 ){
       printf("Unable to get name of new ND output file\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", n_file_out ) != 1 ){
       printf("Unable to get name of new n values file\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", min_file_out ) != 1 ){
       printf("Unable to get name of new min file\n" );
       exit(1);
   }
   if ( fscanf( parmsfp, "%s", sos_file_out ) != 1 ){
       printf("Unable to get name of new sos file\n" );
       exit(1);
   }
   //printf( "lines and samps = %d %d\n", nlines, nsamps );
   //printf( "file = %s %s\n", min_file_in, sos_file_in );

   /* Open input files */
   if((nd_new_file = fopen(nd_new,"rb")) == NULL) {
       printf("Can't open input file: '%s'\n", nd_new); 
       exit(status);
   }

   if((meanndfilein = fopen(mean_nd_in,"rb")) == NULL) {
       printf("Can't open input file: '%s'\n", mean_nd_in); 
       exit(status);
   }

   if((nfilein = fopen(n_file_in,"rb")) == NULL) {
       printf("Can't open input file: '%s'\n", n_file_in); 
       exit(status);
   }


   /* Open output files */
   
   if((meanndfileout = fopen(mean_nd_out,"wb")) == NULL) {
       printf("Can't open output file: '%s'\n", mean_nd_out); 
       exit(status);
   }

   if((nfileout = fopen(n_file_out,"wb")) == NULL) {
       printf("Can't open output file: '%s'\n",n_file_out); 
       exit(status);
   }

    /* Calculate meannd for each pixel */
    for( row = 0; row < nlines; row++ ) {
       for(col = 0; col < nsamps; col++ ) {
	  n = getc(nfilein); /* number of hist. obs in mean for this pixel */
	  imeannd = getc(meanndfilein); /* hist. mean value for this pixel */
	  sum = (double)n * (double)imeannd; /* total ND for hist. data */
	  intnd96 = getc(nd_new_file); /* get 1996 ndvi value */
	  nd96 = (double)intnd96;   /* make it a double precision value */
	  if(intnd96 > 0) { /* if not zero, add 96 data */
	      n=n+1; 
	      sum=sum+nd96;
	  }

	   if(n > 0) {
	       meannd = sum/(double)n + 0.5;
	       imeannd = (int)meannd;
	   }

	   /* Write meannd and n values for this pixel */
	   putc(n,nfileout);
	   putc(imeannd,meanndfileout);
       }  /* end col loop */
    }  /* end row loop */

    /* Close files */
    fclose(nd_new_file);
    fclose(nfilein);
    fclose(nfileout);
    fclose(meanndfilein);
    fclose(meanndfileout);

}   /* end main */

void usage( char **argv ) {
    printf("Usage: %s <input file>\n", argv[0] );
    printf("Input file must contain these values:\n" );
    printf("        integer - number of lines\n" );
    printf("        integer - number of samples\n" );
    printf("        string  - input  new NDVI values file name\n" );
    printf("        string  - input  old mean value file name\n" );
    printf("        string  - input  old number of values in mean file name\n");
    printf("        string  - input  old min value file name\n" );
    printf("        string  - input  old sum of squares file name\n" );
    printf("        string  - output new mean value file name\n" );
    printf("        string  - output new number of values in mean file name\n");
    printf("        string  - output new min value file name\n" );
    printf("        string  - output new sum of squares file name\n" );

}
