/*************************************************************************
FUNCTION:	check_status

PURPOSE:	Check the status of the framesync and build a list of
		commands needed to properly configure it.

INPUT:		socket file descriptor

OUTPUT:		commands list

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: check_status.c,v $
Revision 1.2  2002/07/01 18:03:00  dlloyd
Commented variable declarations.

Revision 1.1  2002/05/20 13:23:25  dlloyd
Initial checkin of files

*************************************************************************/
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "acquire.h"


void check_status
    (
    int socket_fd,		/* IN - socket file descriptor		*/
    char *commands[7]		/* OUT - list of commands		*/
    )
    {
    char **command = (char **)commands;/* pointer into the commands array */
    char message[STRLEN];	/* message to be sent to frame sync	*/
    int *message_ptr = (int *)message; /* pointer for retrieving length	*/
    int message_len;		/* message length			*/
    int command_num = 0;	/* index into array of commands		*/
    int num_read;		/* number of bytes read from frame sync	*/
    int num_wrote;		/* number of bytes written to frame sync*/
    int n;			/* temporary count of bytes read/wrote	*/


    /* Ask for the current status
    -----------------------------*/
    strcpy( message, "0000:SSS;" );
    message_len = strlen( message );
    *message_ptr = message_len - 4;
    num_wrote = 0;
    while ( num_wrote < message_len )
	{
	n = write( socket_fd, message, message_len-num_wrote );
	if ( n < 0 )
	    break;
	num_wrote += n;
	}

    /* Get the current status
    -------------------------*/
    num_read = 0;
    while ( num_read < 4 )
	{
	n = recv( socket_fd, message, 4-num_read, MSG_DONTWAIT );
	if ( n < 0 )
	    break;
	num_read += n;
	}
    /* Add 2 to the message_length just to ensure the
    entire message is read.  During testing the message
    was read a second time in another function.
    --------------------------------------------------*/
    message_len = *message_ptr + 2;
    num_read = 0;
    while ( num_read < message_len )
	{
	n = recv( socket_fd, message, message_len-num_read, MSG_DONTWAIT );
	if ( n < 0 )
	    break;
	num_read += n;
	}


    /* If a short message received set the characters not received
    to bad values so that the corresponding commands will be sent.
    --------------------------------------------------------------*/
    if ( num_read < 107 )
        message[107] = 'D';
    if ( num_read < 106 )
        message[106] = 'D';
    if ( num_read < 8 )
        message[10] = '0';
    if ( num_read < 7 )
        message[9] = '1';
    if ( num_read < 5 )
        message[7] = 'B';
    if ( num_read < 2 )
        message[4] = '1';
    
    /* Check for HRPT.  If HRPT is not set,
    the entire command set must be done.
    ---------------------------------------*/
    if ( message[4] != '7' )
	{
        strcpy( &command[command_num++][0], "FTP 7" );
        strcpy( &command[command_num++][0], "COD N" );
        strcpy( &command[command_num++][0], "CRT 0" );
        strcpy( &command[command_num++][0], "DTP 1" );
        strcpy( &command[command_num++][0], "VID E" );
        strcpy( &command[command_num++][0], "TSP E" );
	}  /* END if HRPT */

    /* Otherwise check and add other commands as needed
    ---------------------------------------------------*/
    else
        {
	if ( message[10] != '1' )
	    strcpy( command[command_num++], "DTP 1" );
	if ( message[9] != '0' )
	    strcpy( command[command_num++], "CRT 0" );
	if ( message[7] != 'N' )
	    strcpy( command[command_num++], "COD N" );
	if ( message[107] != 'E' )
	    strcpy( command[command_num++], "VID E" );
	if ( message[106] != 'E' )
	    strcpy( command[command_num++], "TSP E" );
	}  /* END else */
    
    /* NULL terminate the list and return
    -------------------------------------*/
    command[command_num] = NULL;
    }  /* END check_status */
