/*************************************************************************
FUNCTION:	create_socket

PURPOSE:	Open a socket connections with the IP address and port
		passed.

INPUT:		IP address
		port number

OUTPUT:		none

RETURNS:	socket file descriptor

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: create_socket.c,v $
Revision 1.3  2003/04/21 15:28:06  dlloyd
Changed AF_INET to PF_INET.

*************************************************************************/
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/utsname.h>
#include "acquire.h"

#include <stdio.h>


int create_socket
    (
    char *IPaddress,		/* IP address to make connection to	*/
    int   port_number		/* port number to make connection to	*/
    )
    {
    int  socket_fd;		/* socket file descriptor		*/
    struct sockaddr_in skaddr;	/* socket structure                     */
    struct hostent *hostname;	/* host name stucture			*/



    /* Create a socket
    ------------------*/
    if (( socket_fd = socket( PF_INET, SOCK_STREAM, 0 )) < 0 )
        {
	write_log(TRUE,"create_socket","socket","Error creating socket");
	return FAIL;
	}
    
    /* Fill in the address structure
    --------------------------------*/
    skaddr.sin_family = PF_INET;
    if (( hostname = gethostbyname( IPaddress )) != 0 )
        memcpy( &skaddr.sin_addr.s_addr, hostname->h_addr, hostname->h_length );
    else
        {
	if ( inet_aton( IPaddress, &skaddr.sin_addr ) == 0 )
	    {
	    write_log(TRUE,"create_socket","host-lookup",
	        "Error getting host name");
	    return FAIL;
	    }
	}
    
    skaddr.sin_port = htons( port_number );

    /* Establish the connection
    ---------------------------*/
    if ( connect( socket_fd, (struct sockaddr *)&skaddr, sizeof(skaddr)) < 0 )
        {
	write_log(TRUE,"create_socket","connect","Error connecting to socket");
	return FAIL;
	}
    
    return socket_fd;
    }  /* END create_socket */

