/*************************************************************************
FUNCTION:	parse_args

PURPOSE:	Go through the input arguments extracting the values and
		placing them in the appropriate return parameters.  Also
		creates the miscellaneous and time stamp file names from
		the root file name of the output file name.

INPUT:		argv array of arguments 

OUTPUT:		ip address of frame sync
		port number of frame sync
		image file name
		miscellaneous file name
		time stamp file name

RETURNS:	none

PROJECT:	ADAPS, AVHRR, Acquisition System

$Log: parse_args.c,v $
Revision 1.3  2003/04/21 15:26:38  dlloyd
Had to resort back to numeric port address.

Revision 1.2  2002/05/20 14:48:24  dlloyd
Added support for creating and removing a lock file.

Revision 1.1  2002/05/20 13:23:25  dlloyd
Initial checkin of files

*************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "acquire.h"


void parse_args
    (
    char *argv[],               /*  IN - argument array                 */
    char *IPaddress,            /* OUT - frame sync address             */
    int  *port_number,          /* OUT - port number of frame sync      */
    char *image_file_name,      /* OUT - output image file name         */
    char *misc_file_name,       /* OUT - miscellaneous output file name */
    char *tstamp_file_name,	/* OUT - time stamp output file name	*/
    char *lock_file_name	/* OUT - lock file name			*/
    )
    {
    char *strptr;		/* string pointer for file name manip.	*/

    /* Make sure all the output variables are cleared
    -------------------------------------------------*/
    IPaddress[0] = image_file_name[0] = misc_file_name[0] =
	tstamp_file_name[0] = '\0';
    *port_number = 0;


    /* Parse the args
    -----------------*/
    while ( *++argv != NULL )
        {
	if ( strcmp( *argv, "-file" ) == 0 )
	    {
	    argv++;
	    if ( *argv == NULL )
	        {
		write_log( TRUE, "parse_args", "missing-arg",
		    "missing file name" );
		exit(1);
		}
	    else if ( *argv[0] == '-' )
	        {
		write_log( TRUE, "parse_args", "invalid-arg",
		    "missing file name" );
		exit(1);
		}
	    else
	        {
		/* Get the output image file name
		---------------------------------*/
		strcpy( image_file_name, *argv );
		write_log( FALSE, "parse_args", "output-file", image_file_name);

		/* build the other file names
		-----------------------------*/
		strcpy( misc_file_name, *argv );
		strptr = strrchr( misc_file_name, '.' );
		if ( strptr != NULL )
		    *strptr = '\0';
		strcat( misc_file_name, ".misc" );
		strcpy( tstamp_file_name, *argv );
		strptr = strrchr( tstamp_file_name, '.' );
		if ( strptr != NULL )
		    *strptr = '\0';
		strcat( tstamp_file_name, ".tstamp" );
		strcpy( lock_file_name, *argv );
		strptr = strrchr( lock_file_name, '.' );
		if ( strptr != NULL )
		    *strptr = '\0';
		strcat( lock_file_name, ".lock" );
		}  /* END else */
	    }  /* END if -file */
	else if ( strcmp( *argv, "-ipadd" ) == 0 )
	    {
	    argv++;
	    if ( *argv == NULL || *argv[0] == '-' )
	        {
		write_log( FALSE, "parse_args", "arg-error",
		    "Invalid or missing IP, using 'edcavhrr-fs'" );
		strcpy( IPaddress, "edcavhrr-fs" );
		}
	    else
	        {
		strcpy( IPaddress, *argv );
		write_log( FALSE, "parse_args", "-ipadd", IPaddress );
		}
	    }  /*END else if -ipadd */
	else if ( strcmp( *argv, "-port" ) == 0 )
	    {
	    argv++;
	    if ( *argv == NULL || *argv[0] == '-' )
	        {
		write_log( FALSE, "parse_args", "arg-error",
		    "Invalid or missing port number, using '1024'" );
		*port_number = 1024;
		}
	    else
	        {
		write_log( FALSE, "parse_args", "-port", *argv );
		*port_number = atoi( *argv );
		}
	    }  /* END else if -port */
	else
	    {
	    write_log( FALSE, "parse_args", "arg-error", *argv );
	    }  /* END else */
	}  /* END while */
    
    /* if no output image file name was specified quit
    --------------------------------------------------*/
    if ( image_file_name[0] == '\0' )
        {
	write_log(TRUE, "parse_args", "no-file",
	    "ERROR not output file name specified" );
	exit(1);
	}
    
    /* If no IP address of port was specified use the default ones
    --------------------------------------------------------------*/
    if ( IPaddress[0] == '\0' )
        strcpy( IPaddress, "192.168.10.2" );
    if ( *port_number == 0 )
	*port_number = 1024;
    
    }  /* END parse_args */
