#!/bin/csh
#
# cleanup.csh
#
# Purpose: Remove backup files in ADAPSARC older than XX days.  Tidy up the
# ADAPSLOG directory.  The current days logs are in ADAPSLOG; the current
# months logs are in ADAPSLOG/old; and older logs are in ADAPSLOG/old/monthdata.
# This mimics the way the log files are handled on the production system.
# Each time this script is run todays logs are concatenated onto the current
# month logs.  At the beginning of the month last months logs are moved to
# the monthdata directory.
#
#
# Note: to keep the "key" parameter in the wrtlog call within bounds the
# the variable modifiers ':r:t' have been used to only pass the root file
# name:
#
#   wrtlog $logfile $image:r:t cleanup-rm "Removed old file"
#
# This script is designed to be run from cron just prior to midnight.
# The reason for the timing is so the end of month log files are added
# to the ADAPSLOG/old/xxx_MMYY.log for the current month.
#
#$Log: cleanup.csh,v $
#Revision 1.2  2007/10/02 17:55:51  dlloyd
#Changed how LAS is set up
#
#Revision 1.1  2003/04/23 12:12:18  dlloyd
#Initial checking of miscellaneous scripts.
#
#-------------------------------------------------------------------------------

# Get the date
#-------------
set tstamp = `date +_%m%y`
set day = `date +%d`


#-------------------------------------------------------------------------
# Set up the production LAS environment,
# if is hasn't already been setup.
#-------------------------------------------------------------------------
if (! $?EDCSOFT) then
    setenv EDCSOFT /avhrr/software/las/build
    source $EDCSOFT/env/las/assign/lasenv.csh
endif

# Set up the log file
#--------------------
alias wrtlog $ADAPSBASE/write_log
set logfile = $ADAPSLOG/cleanup
wrtlog $logfile Start pushacqu-start "Starting CLEANUP"


# Clean up the $ADAPSARC/sfl directory
#-------------------------------------
set images = `find $ADAPSARC/sfl/ -type f -prune -name 'ah*.sf2' -mtime +12 -print`
foreach image ( $images )
    rm -f $image
    wrtlog $logfile $image:r:t cleanup-rm "Removed old file"
end


# At the beginning of the month clean up the ADAPSLOG/old directory
#------------------------------------------------------------------
if ( $day == "01" ) then
    mv $ADAPSLOG/old/*.log $ADAPSLOG/old/monthdata/.
endif


# Clean up the log directory.  The month log files are named
# as: <root log name>_MMYY.log, eg. acquman_0602.log.
#-----------------------------------------------------------
set logfiles = `ls $ADAPSLOG/*.log`
echo $logfiles
foreach file ( $logfiles )
    wrtlog $logfile $file:r:t cleanup-log "Cat log file to old"

    # Concatenate the contents of the log to the month log
    # File will be created if it doesn't exist.
    cat $file >> ${file:h}/old/${file:r:t}$tstamp.log

    # Empty the contents of the log file.  This little trick
    # maintains the ownership and file permissions of the files.
    tail -0 $file > ${file}TEMP
    cp ${file}TEMP $file
    rm -f ${file}TEMP
end


wrtlog $logfile Complete cleanup-cmplt "CLEANUP complete"
