#!/bin/csh -f
#
# This script stops or kills the current acquman and restarts another
# instance.  If shoacqu is running it will stop when acquman stops
# and another instance will be started.
#
# Usage: start_acquman.csh [now]
#
# now forces an immediate kill of current acquman process.  If now
# isn't used acquman will be stopped with a stop file -- may take up
# to five minutes to happen.
#
# $Log: start_acquman.csh,v $
# Revision 1.3  2007/10/18 13:08:37  dlloyd
# Added setting the display environment variable
#
# Revision 1.2  2003/05/06 13:06:39  dlloyd
# Added LASTOOLS to script and executable paths
#
# Revision 1.1  2003/04/23 12:12:18  dlloyd
# Initial checking of miscellaneous scripts.
#
#########################################################################

# Get the current time
set now = `date +"%D  %T"`

if ( ${#argv} == 1 ) then
    if ( "$1" != "now" ) then
        echo "Usage $0 [now]"
	exit 1
    endif
endif


# Stop acquman
$LASTOOLS/stop_acquman.csh $1
if ( $status == -1 ) then
    echo "Continuing with startup"
    echo "start_acquman.csh        $now  Continuing with startup" >> $ADAPSLOG/acquman.log
endif


# Now start
# Acquman is to be started so that it is immune to hangups
echo "Starting Acquman"
echo "start_acquman.csh       $now   Starting Acquman" >> $ADAPSLOG/acquman.log
nohup $LASTOOLS/acquman &

# We want the moving window display to show up on the big screen
setenv DISPLAY :0.0

# Give Acquman a chance to start and open the server socket
# before starting the moving window display
sleep 2
echo "Starting Shoacqu"
echo "start_acquman.csh        $now  Starting Shoacqu" >> $ADAPSLOG/shoacqu.log
$LASTOOLS/start_shoacqu.csh &

exit 0
