#!/bin/csh -f
#
# This stops the acquman process.  If the 'now' argument is supplied the
# process will be killed immediately otherwise a stop file will be used.  If
# multiple processes are present all processes are killed immediately
# regardless of the 'now' argument.
#
# Usage: stop_acquman.csh [now]
#
# $Log: stop_acquman.csh,v $
# Revision 1.2  2007/09/14 18:27:13  adaps
# Added -f to #!/bin/csh to prevent sourcing the .cshrc file.  Was causing
# some errors that didn't make sense.
#
# Revision 1.1  2003/04/23 12:12:18  dlloyd
# Initial checking of miscellaneous scripts.
#
#===========================================================================

# Get the current time
set now = `date +"%D  %T"`

# Make sure there is no stop file for now
if ( -e $ADAPSTABLES/acquman.stop ) then
    rm -f $ADAPSTABLES/acquman.stop
endif

# find the process id(s) of acquman
set acquman_pid = " "
set acq = `ps -e -o pid -o ucomm | grep acquman`
@ i = ${#acq}
@ j = 0
while ( $i > 1 )
    if ( $acq[$i] == "acquman" ) then
        @ i--
	@ j++
	set acquman_pid = "$acquman_pid $acq[$i]"
	@ i--
    else
        @ i = $i - 2
    endif
end

# If the acquman process was found then stop it
if ( $j > 0 ) then
    # if the now was explicit or more than one
    # process was found force the kill immediately
    if ( $1 == "now" || $j > 1 ) then
        echo "Killing process: $acquman_pid"
	echo "restart.csh        $now  killing process: $acquman_pid" >> $ADAPSLOG/acquman.log
	foreach proc_id ( $acquman_pid )
	    kill -9 $proc_id
	end
    # otherwise do a normal shutdown
    else
        echo "Normal shutdown, please wait"
	echo "restart.csh       $now   Normal shutdown" >> $ADAPSLOG/acquman.log
	echo "Stop" >> $ADAPSTABLES/acquman.stop

	# wait for the process to end
	while ( -e $ADAPSTABLES/acquman.stop )
	    sleep 10
	end
	echo "Acquman shutdown"
    endif
else
    echo "Could not find acquman process"
    exit -1
endif

exit 0
