/*************************************************************************
*************************************************************************/
#ifndef ACQUMAN_H
#define ACQUMAN_H

#define  MINSLEEPINT    10		/* minimum sleep time in seconds*/
/* For a shorter nap time during
testing, uncomment the following.
#define  TESTING
--------------------------------*/
#ifdef   TESTING
#define  NAPTIME        60		/* normal sleep time of 1 min	*/
#else
#define  NAPTIME        300		/* normal sleep time of 5 min	*/
#endif
/* TODO: this value should be shortened somewhat.  It is 8 minutes now and
 * should probably go down to something like 2 minutes depends on how long
 * it takes the antenna to get into position.  With the new ACU, going into
 * program track the antenna immediately moves into position of next rise 
 * so we want to delay that a little now. I'm going with 3 minutes for now */
#define  ACUALARMOFFSET 330		/* offset to ACU alarm of 8 min	*/

#ifdef   FALSE
#undef   FALSE
#endif
#define  FALSE          0		/* don't write a fatal error	*/
#ifdef   TRUE
#undef   TRUE
#endif
#define  TRUE           (!FALSE)	/* write a fatal error message	*/

#define  START          1		/* start acquisition flag value	*/
#define  STOP           2		/* stop acquisition flag value	*/
#define  TIME_ERROR     0    		/* error flag, indicates error	*/
					/* in a time_t variable must	*/
					/* not be negative!		*/

#define  STRINGLEN	512		/* standard string length	*/
#define  KNOWN_PORT	55555		/* ACQUMAN's known port number	*/
#define  FSPORT		"1024"		/* Frame sync known port number	*/
#define  PIDWARN	20		/* number of waiting processes	*/
					/* to warn about		*/

/* The minimum scene size is 2 minutes, or 2*60 seconds.  There are 6   */
/* lines per second and 22016 bytes per line.                           */
#define  MINSCENESIZE	(2 * 60 * 6 * 22016) /* scene size in bytes	*/

/* Communications defines
-------------------------*/
#define RECEIVER_PORT	"/dev/ttyS0"	/* receiver comm port		*/
#define ACU_PORT	"/dev/ttyS1"	/* antenna control unit port	*/
#define NOAA16		3		/* receiver channel for NOAA 16	*/
#define NOAA17		3		/* receiver channel for NOAA 17	*/
#define NOAA18		5		/* receiver channel for NOAA 18	*/
#define TRACK           6161		/* command to track acu		*/
#define STOW		3030		/* command to stow acu		*/
#define ETX		0x03		/* ACU end of transmission	*/
#define ACK		0x06		/* ACU acknowledgement		*/
#define NAK		0x15		/* ACU negative acknowledgement	*/
#define RST		0x05		/* ACU reset			*/
#define CR		0x0d		/* ACU carriage return		*/


/* Included for definition of pid_t
-----------------------------------*/
#include <sys/types.h>

/* Function prototypes
----------------------*/
void get_next_scene
    (
    const char *adapstables,    /* IN - ADAPSTABLES environment variable     */
    char *sceneid,		/* OUT - scene identifier		     */
    char *day_night_flag,	/* OUT - flag indicating a day or night pass */
    char *local_start_time,	/* OUT - local start time as HH:MM:SS A/PM   */
    char *local_stop_time,	/* OUT - local stop time as HH:MM:SS A/PM    */
    time_t *start_time,		/* OUT - UTC start time in seconds since BOT */
    time_t *stop_time, 		/* OUT - UTC stop time in seconds since BOT  */
    time_t *curr_time		/* OUT - current UTC time in secs since BOT  */
    );

int create_socket
    (
    const char *adapstables     /* IN - ADAPSTABLES environment variable*/
    );

int send_message
    (
    int socket_fd,      /* IN - client socket descriptor                */
    const char *message	/* IN - message string to send                  */
    );

int check_for_stop
    (
    const char *adapstables     /* IN - ADAPSTABLES environment variable*/
    );

time_t check_alarm
    (
    time_t input_time           /* IN - input time for calculation      */
    );

int start_acquisition
    (
    const char *adapsacq,       /* IN - Incoming data directory         */
    int shoacqu_socket,         /* IN - socket descriptor for MWD       */
    const char *sceneid,        /* IN - 16-digit scene ID               */
    const char *day_night_flag,	/* IN - day/night flag string           */
    const char *stop_time,	/* IN - local stop time			*/
    pid_t *acquire_id           /* OUT - acquire child process ID       */
    );

int stop_acquisition
    (
    const char *adapsacq,       /* IN - Incoming data directory         */
    const char *adapsing,       /* IN - ADAPSING environment variable   */
    const char *adapserr,       /* IN - ADAPSERR environment variable   */
    pid_t acquire_id,           /* IN - process ID of acquire           */
    const char *sceneid,	/* IN - scene ID of image               */
    pid_t *timecorr_id          /* OUT - process ID of time correction  */
    );

int poll_socket
    (
    int socket_fd               /* IN - socket file descriptor          */
    );

pid_t fork_child
    (
    const char *childname,	/* IN - process name                    */
    const char *arg1,          	/* IN - string containing argument one  */
    const char *arg2,          	/* IN - string containing argument two  */
    const char *arg3,          	/* IN - string containing argument three*/
    const char *arg4           	/* IN - string containing argument four */
    );

void write_log
    (
    int    flag,                /* IN - TRUE/FALSE fatality flag        */
				/*      TRUE = issue fatal error message*/
    const char   *key,		/* IN - error/message key               */
    const char   *code,		/* IN - error/message code              */
    const char   *msg		/* IN - message string                  */
    );

void create_modsid
    (
    const char *adapsing,       /* IN - ADAPSING environment variable   */
    const char *sceneid         /* IN - 16-character scene id           */
    );

void save_port
    (
    const char *adapstables,    /* IN - ADAPSTABLES environment variable*/
    int port_number     	/* IN - port number to save		*/
    );

int get_environment
    (
    char *adapstables,          /* OUT - ADAPSTABLES environment var    */
    char *adapsing,             /* OUT - ADAPSING environment variable  */
    char *adapsacq,             /* OUT - Incoming data directory        */
    char *adapserr              /* OUT - ADAPSERR environment variable  */
    );

pid_t wait_for_children
    (
    pid_t processid             /* IN - process ID to add to list       */
    );

int set_rcvrfreq
    (
    int satellite       /* IN - satellite number: 16, 17 etc.   */
    );

int acu_control
    (
    int command,		/* IN - ACU command: TRACK or STOW	*/
    int sat
    );

#endif
