/*************************************************************************
FUNCTION:	check_alarm

PURPOSE:	Determine and return the difference between current time
		UTC and the time passed in.  Times are in seconds since
		BOT 1/1/86.

INPUT:		time in seconds

OUTPUT:		none

RETURN:		difference time in seconds

PROJECT:	ADAPS, AVHRR Acquisition System

*************************************************************************/
#include <stdio.h>
#include <time.h>
#include "adapsutil.h"
#include "acquman.h"


time_t check_alarm
    (
    time_t input_time		/* IN - input time for calculation	*/
    )
    {
    time_t now;			/* current time in seconds since 1/1/70	*/

    long datearray[3];		/* date (month, day, year) for makesecs	*/
    long timearray[3];		/* time (hour, min, sec) for makesecs	*/
    long utc_time;		/* UTC in seconds since BOT		*/

    struct tm *time_struct;	/* current time structure		*/


    /* Get the local system time and convert to UTC
    -----------------------------------------------*/
    now = time( NULL );
    time_struct = gmtime( &now );

    /* Convert to seconds since BOT
    -------------------------------*/
    timearray[0] = time_struct->tm_hour;
    timearray[1] = time_struct->tm_min;
    timearray[2] = time_struct->tm_sec;
    datearray[0] = time_struct->tm_mon + 1;   /* make the month 1 relative */
    datearray[1] = time_struct->tm_mday;
    datearray[2] = time_struct->tm_year;
    c_makesecs( datearray, timearray, &utc_time );

    /* Return the difference
    ------------------------*/
    if ( utc_time > input_time )
        return 0;
    else
	return (time_t)difftime( input_time, utc_time );
    }  /* END check_alarm */

