/*************************************************************************
FUNCTION:	CREATE_MODSID

PURPOSE:	Generate a 'modsid', which stands for modify by scene ID,
		file for an acquired image.  

IN:		ADAPSING environment variable
		16-digit scene ID

OUT:		creates modsid file

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acquisition System

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "acquman.h"


void create_modsid
    (
    const char *adapsing,	/* IN - ADAPSING environment variable   */
    const char *sceneid         /* IN - 16-character scene id		*/
    )
{
    FILE *modsid;		/* output file descriptor		*/

    char modsid_name[STRINGLEN];/* output file name			*/
    char modsid_buffer[STRINGLEN];/* buffer for output file		*/
    char tstamp[STRINGLEN];	/* time stamp for file name		*/
    char *adapstables;
    char new_modsid[STRINGLEN];

    time_t now;			/* local unix time			*/
    struct tm *curtime;		/* local time structure			*/

    int  length;		/* length of buffer to print to file	*/
    int  num_written;		/* number of bytes written to file	*/


    /* Get the ADAPSTABLES environment variable */
    adapstables = getenv( "ADAPSTABLES" );
    if ( !adapstables )
    {
        write_log(FALSE,"create_modsid","Error","Unable to get ADAPSTABLES");
        strcpy( adapstables, "." );
    }
    strcpy( new_modsid, adapstables );

    /* Create the output string.  The output string is the 16-digit
    scene ID followed by instructions on how to update the master
    schedule.  Determining the meaning of these instructions is
    left as an exercise for the interested reader.  The interested
    parties are referred to ADAPSBASE/schedit.
    ---------------------------------------------------------------*/
    sprintf( modsid_buffer, "%s - 15 D 00 - -    - -", sceneid );


    /* Create the output file name
    Note: the month, day and year are in a different
    sequence to prevent naming conflicts with modsid
    files created on the production system.
    ------------------------------------------------*/
    now = time( NULL );
    curtime = localtime( &now );
    sprintf( tstamp, "%02d%02d%02d%02d%02d%02d", curtime->tm_mon+1,
    	curtime->tm_mday, curtime->tm_year%100, curtime->tm_hour,
	curtime->tm_min, curtime->tm_sec );
    strcpy( modsid_name, adapsing );
    sprintf( modsid_name, "%s/modsid_%s.sch", modsid_name, tstamp );
    sprintf( new_modsid, "%s/modsid_%s.sch", new_modsid, tstamp );


    /* Open the output file
    -----------------------*/
    modsid = fopen( modsid_name, "w" );
    if ( !modsid )
    {
	write_log(FALSE,"create_modsid","Error","Unable to create modsid file");
	write_log(FALSE,"create_modsid","modsid-file",modsid_name);
	return;
    }


    /* Write the file
    -----------------*/
    length = strlen( modsid_buffer );
    num_written = fprintf( modsid, "%s", modsid_buffer );
    if ( num_written != length )
        write_log(FALSE,"create_modsid","Error writing",modsid_name);
    else
	write_log(FALSE,"create_modsid","Created",modsid_name);
    if ( fclose( modsid ) != 0 )
        write_log(FALSE,"create_modsid","Error closing",modsid_name);

    /* File was created in adapsing and now needs to be moved to adapstables
    ------------------------------------------------------------------------*/
    if ( rename( modsid_name, new_modsid ))
        write_log(FALSE,"create_modsid","Error renaming",modsid_name);

}  /* END create_modsid */
