/****************************************************************************
FUNCTION:	exit_callback

PURPOSE:	To provide an exit confirmation which allows the user
  		to cancel the exit or to exit.  Upon exit, memory is freed.

INPUT:	        The exit button widget
		call data -- not used

OUTPUT:         None

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	04/01	T. Baltzer	Initial development

PROJECT:	ADAPS
****************************************************************************/
#include <stdlib.h>
#include <Xm/Xm.h>
#include <signal.h>
#include "acquman.h"

extern	pid_t shoacqu; 		/* IN - shoacqu process id		*/

void exit_callback
    (
    Widget    w,		/* IN - widget associated with the CB	*/
    XtPointer call_data 	/* IN - call data			*/
    )

    {

    /* Display a popup message and get response.
    Return immediately if the user responds no.
    --------------------------------------------*/
    if ( get_user_response( w, "Do you really want to quit?", "Yes",
                               "No", YES ) == NO )
	return;

    /* Destroy any windows
    ----------------------*/
    XtDestroyWidget( get_toplevel_shell( w ));


    /* Terminate the SHOACQU child
    -------------------------------*/
    kill( shoacqu, SIGKILL );


    /* If an ACQUIRE child is running, stop it  ?????????
    -------------------------------------------*/

    /* write a log file and exit
    ----------------------------*/
    write_log( FALSE, "exit_callback", "info",
        "ACQUMAN terminated by user" );
    exit(0);
    }  /* END exit_callback */
