/****************************************************************************
FUNCTION:       fork_child

PURPOSE:        Fork a child process.

INPUT:          Child name
		argument strings

OUTPUT:         none

RETURNS:        child process id or -1 on failure

PROJECT:	ADAPS, AVHRR Acquisition System

*************************************************************************/
#include <stdlib.h>
#include <unistd.h>
#include "acquman.h"

pid_t fork_child
    (
    const char *childname,	/* IN - process name			*/
    const char *arg1,		/* IN - string containing argument one	*/
    const char *arg2,		/* IN - string containing argument two	*/
    const char *arg3,		/* IN - string containing argument three*/
    const char *arg4		/* IN - string containing argument four	*/
    )
    {
    pid_t childid = 0;		/* process ID of the child 		*/


    /* Try to fork the child
    ------------------------*/
    childid = fork();
    if ( childid < 0 )
        {
	write_log(FALSE,"fork_child",childname,"Error forking child process");
	return -1;
	}  /* END if */

    /* If this is the child process execute the process
    ---------------------------------------------------*/
    else if ( childid == 0 )
        {
	if (( execlp( childname, childname, arg1, arg2, arg3, arg4, NULL )) < 0)
	    {
	    write_log(FALSE,"fork_child",childname,"Error executing child");
	    return -1;
	    }  /* END if */
	}  /* END else if */
    
    return childid;
    }  /* END fork_child */
