/******************************************************************************
FUNCTION:	acquman (main)

PURPOSE:	The acquisition manager coordinates ADAPS acquisition
		processes.

		The main function gets things started by initializing the
		data structures and window widgets, then entering an event
		driven loop.

INPUT:          optional command line switches
		    none

OUTPUT:         messages are logged in $ADAPSLOG/acquman.log
		display windows

RETURNS:	None

Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	07/00	T.Baltzer	Initial development

PROJECT:	ADAPS
******************************************************************************/
#include <stdlib.h>
#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Form.h>
#include <Xm/FileSB.h>
#include <X11/Shell.h>
#include "acquman.h"

/* Global variables
-------------------*/
XtAppContext app;		/* main application context		*/
Widget	toplevel;		/* top level application shell		*/

int  sd;			/* Socket descriptor			*/
pid_t shoacqu;			/* SHOACQU child process id		*/

char dnflag[2];			/* Day / Night flag (D/N)		*/
char errmsg[CMLEN];		/* Error message			*/
char sceneid[17];		/* Scene identifier			*/

long gstart_time;		/* GMT start time, secs since 1/1/86 00:00:00 */
long gstop_time;		/* GMT stop time, secs since 1/1/86 00:00:00  */


void main
    (
    int  argc,			/* IN - count of arguments passed in	*/
    char *argv[]		/* IN - argument array			*/
    )
    {
    Widget   fileselectwindow;	/* widget for file selection		*/

    unsigned long timer_interval;/* check status interval in millisec	*/

    #include "fallbacks.h"	/* resource strings to fall back on if	*/
    				/* user doesn't specify them.		*/

    char ltime_disp[9];		/* Local time in display format HH:MM AM/PM   */




    /* Write a startup message to log
    ---------------------------------*/
    write_log( FALSE, "acquman", "info", "ACQUMAN has been started" );
    

    /* Set up the user's locale and get a top level shell
    -----------------------------------------------------*/
    XtSetLanguageProc( NULL, NULL, NULL );
    toplevel = XtVaAppInitialize( &app, WnACQUMAN, NULL, 0,
        &argc, argv, fallbacks, NULL );

    
    /* Create the main display window
    ---------------------------------*/
    create_main_display( toplevel );
    

    /* Get the next scene to acquire (from the master schedule)
    ------------------------------------------------------------*/
    get_next_scene(ltime_disp);


    /* Fork a SHOACQU child to display the live acquisitions
    ---------------------------------------------------------*/
    fork_child("shoacqu", &sd, shoacqu);


    /* Set the start alarm
    -------------------------*/
    set_start_alarm();


    /* Enter the event loop
    -------------------------*/
    XtAppMainLoop( app );

    }  /* END main */

