/*************************************************************************
FUNCTION:	poll_socket

PURPOSE:	To check the status of a socket connection.  This is done
		using a combination of poll() and sending a message to
		the socket.

INPUT:		socket descriptor

OUTPUT:		sends a poll message to the socket

RETURNS:	0 if all is well
		-1 on an error condition

PROJECT:	ADAPS, AVHRR Acquisition System

*************************************************************************/
#include <sys/poll.h>
#include <unistd.h>
#include "acquman.h"



int poll_socket
    (
    int socket_fd		/* IN - socket file descriptor		*/
    )
    {
    struct pollfd poll_struct[1];/* structure for polling socket	*/


    /* Set up the poll
    ------------------*/
    poll_struct[0].fd = socket_fd;
    poll_struct[0].events = POLLOUT;
    poll_struct[0].revents = 0;

    /* And poll the socket.  Poll returns the number of file
    descriptors available.  A value of zero means the poll
    timed out.  And a negative value indicates an error.
    In this situation we are only interested in checking the
    status of available descriptors.
    --------------------------------------------------------*/
    if ( poll( poll_struct, 1, 10 ) > 0 )
        {
	/* the poll was successful so see if it is writable */
	if ( poll_struct[0].revents & POLLOUT )
	    {
	    /* it is so send a poll message */
	    if ( write( socket_fd, "005-poll", 8) < 0 )
	        /* an error occurred on the write */
		return -1;
	    }
	else if ( poll_struct[0].revents & (POLLHUP|POLLNVAL|POLLERR))
	    return -1;
	}  /* END if */
    
    return 0;
    }  /* END poll_socket */
