/*************************************************************************
FUNCTION:	send_message

PURPOSE:	Send a message to the client socket

INPUT:		socket descriptor
		message

OUTPUT:		message sent to socket

RETURNS:	0 on success
		-1 on failure

PROJECT:	ADAPS, AVHRR Acquisition System

*************************************************************************/
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include "acquman.h"


int send_message
    (
    int socked_fd,	/* IN - client socket descriptor		*/
    const char *message	/* IN - message string to send			*/
    )
    {
    char message_buffer[STRINGLEN+3];	/* buffer for output message	*/
    			/* add 3 to accomodate the message length	*/

    int message_length;	/* length of parameter message			*/
    int num_sent;	/* number of bytes written to socket		*/
    

    /* Return immediately if there is no socket connection
    ------------------------------------------------------*/
    if ( socked_fd == 0 )
        return 0;

    /* Add the message length to the beginning of the message
    ---------------------------------------------------------*/
    message_length = strlen( message );
    if ( message_length > STRINGLEN-1 )
        {
	write_log(FALSE,"send_message","length-error",message);
	return 0;
	}
    sprintf( message_buffer, "%03d%s", message_length, message );


    /* Send the message
    -------------------*/
    num_sent = write( socked_fd, message_buffer, strlen( message_buffer ));
    if ( num_sent < strlen( message_buffer ))
	{
	/*  As a test this call has been commented out.  It
	doesn't appear that the call is needed.  If the
	following error message starts appearing, uncomment
	the poll and continue testing while looking for the error.
        return poll_socket( socked_fd );
	*/
	write_log(FALSE,"send_message","write-error",
	    "This is where the poll was");
	}
    
    /* Write the same message to the log file
    -----------------------------------------*/
    write_log(FALSE,"send_message","Information",message_buffer);

    return 0;
    }  /* END send_message */
