/******************************************************************************
FUNCTION:	SET_START_ALARM

PURPOSE:	Set an alarm to initiate the acquisition of the next pass.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      03/14    T. Baltzer		Initial development

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <sys/types.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "acquman.h"

#define BUFF_TIME  60	/* Number of seconds after alarm goes off and
			   acquisition begins */

extern XtAppContext app;      /* The applications context	  	      */
extern long gstart_time;      /* GMT start time, seconds since 1/1/86 00:00:00*/

void set_start_alarm
    (
    )

{

long	alarm_time;		/* Computed alarm time			*/
long	clock;			/* Total seconds from 00:00:00, Jan 1, 1971 */
long	datearry[3];		/* date (month, day, year)		*/
long	gtime_secs;		/* current GMT time total seconds since	*/
				/* 00:00:00 on 1/1/86 			*/
long	timearry[3];		/* time (hour, min, sec)		*/


struct tm	*gtime;		/* GMT time structure			*/


/*
  Get the local / GMT time structures
  Get the total seconds from 00:00:00 on 1/1/86.
------------------------------------------------------*/
clock = time(0);
gtime = gmtime(&clock);

timearry[0] = gtime->tm_hour;
timearry[1] = gtime->tm_min;
timearry[2] = gtime->tm_sec;
datearry[0] = gtime->tm_mon + 1;   /* make the month 1 relative */
datearry[1] = gtime->tm_mday;
datearry[2] = gtime->tm_year;

makesecs(datearry,timearry,&gtime_secs);  /* GMT time, total seconds */

/*
  Compute the difference between the current time and the start time
  of the next pass.  Decrement this time by a predetermined buffer time
  (time between the alarm and the start of pass).
------------------------------------------------------------------------*/
alarm_time =  gstart_time - gtime_secs;
alarm_time -= BUFF_TIME;

alarm_time *= 1000;	/* convert into milliseconds */

/*
  Make sure this is not a negative alarm time!
-------------------------------------------------*/
if (alarm_time < 0)
   alarm_time = 0;

/*
  Set the start alarm using this alarm time
--------------------------------------------------------------*/
printf("[info ???] the start alarm time is %d\n",alarm_time);

alarm_time = 10000;	/* reset the alarm time to ? secs for testing ????? */

XtAppAddTimeOut( app, alarm_time, start_callback, NULL );


return;
}
