/******************************************************************************
FUNCTION:	SET_STOP_ALARM

PURPOSE:	Set an alarm to initiate the termination of a pass.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      03/15    T. Baltzer		Initial development

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <sys/types.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "acquman.h"

#define BUFFE_TIME  20	/* Number of seconds from when the alarm goes off and
			   before the acquisition ends */

extern XtAppContext app;      /* The applications context		      */
extern long gstart_time;      /* GMT start time, seconds since 1/1/86 00:00:00*/
extern long gstop_time;       /* GMT stop  time, seconds since 1/1/86 00:00:00*/

void set_stop_alarm
    (
    )

{

long	alarm_time;		/* Computed alarm time			*/
long	clock;			/* Total seconds from 00:00:00, Jan 1, 1971 */
long	datearry[3];		/* date (month, day, year)		*/
long	gtime_secs;		/* current GMT time total seconds since	*/
				/* 00:00:00 on 1/1/86 			*/
long	timearry[3];		/* time (hour, min, sec)		*/


struct tm	*gtime;		/* GMT time structure			*/


/*
  Get the current GMT time.
  Express this time in total seconds from 00:00:00 on 1/1/86.
----------------------------------------------------------------*/
clock = time(0);
gtime = gmtime(&clock);

timearry[0] = gtime->tm_hour;
timearry[1] = gtime->tm_min;
timearry[2] = gtime->tm_sec;
datearry[0] = gtime->tm_mon + 1;   /* make the month 1 relative */
datearry[1] = gtime->tm_mday;
datearry[2] = gtime->tm_year;

makesecs(datearry,timearry,&gtime_secs);  /* GMT time, total seconds */

/*
  If the current time is greater than the pass start time:
    compute the difference between the current time and the end time
    of the pass.
  else
    compute the difference between the pass start time and the end time.
  Decrement this time by a predetermined buffer time
  (the time between the alarm and the end of the pass).
----------------------------------------------------------------*/
if (gtime_secs > gstart_time)
    alarm_time = gstop_time - gtime_secs;
else
    alarm_time = gstop_time - gstart_time;

alarm_time -= BUFFE_TIME;

/* Make sure we did not compute a negative number!
----------------------------------------------------*/
if (alarm_time < 0)
    alarm_time = 0;

alarm_time *= 1000;	/* convert into milliseconds */

/*
  Set the stop alarm using this alarm time
---------------------------------------------------------*/
printf("[info ???] the stop alarm time is %d\n",alarm_time);

alarm_time = 10000;	/* set to ? secs for testing ????? */

XtAppAddTimeOut( app, alarm_time, stop_callback, NULL);


/* write a log file
----------------------------------------------*/
write_log( FALSE, "set_stop_alarm", "info",
	  "Stop alarm set by ACQUMAN" );


return;
}
