/****************************************************************************
FUNCTION:	stop_callback

PURPOSE:	To initiate the termination of a live acquisition.


Version	Date	Programmer	Modifications
-------	-------	---------------	--------------------------------------------
  1.0	03/01	T. Baltzer	Initial development

PROJECT:	ADAPS
****************************************************************************/
#include <stdlib.h>
#include <Xm/Xm.h>
#include "acquman.h"

#define APPROX_BYTES_PER_LINE	22016
#define LINES_PER_SEC		6
#define SECS_PER_MINUTE		60
#define MIN_NUM_MINUTES		2
#define MIN_FILE_SIZE 		(APPROX_BYTES_PER_LINE * LINES_PER_SEC * SECS_PER_MINUTE * MIN_NUM_MINUTES)

extern char sceneid[17];           /* Scene identifier	*/
extern char dnflag[2];             /* Day / Night flag (D / N)	*/
extern int sd;			   /* Socket descriptor    */


void stop_callback
    (
    XtPointer client_data,	/* IN - client data			      */
    XtIntervalId *time_id	/* IN - call data			      */
    )
    {

    char ltime_disp[9];         /* Local time in display format HH:MM AM/PM   */

    char tmpbuf[100];		/* Temporary Data buffer		      */
    char msgbuf[100];		/* Message buffer			      */
    char file_name[100];	/* Name of file acquired		      */
    char error_name[100];	/* Name of file moved to error directory      */

    long file_size;		/* Size of file acquired		      */


    /* Terminate the ACQUIRE child	?????? to be completed
    ---------------------------------------*/


    /* Check the size of the image file to ensure that we have
       acquired the image.  If not, move it to the error directory.
    ----------------------------------------------------------------*/
    if (getenv("ADAPSACQ") != NULL)
	strcpy(file_name,getenv("ADAPSACQ"));
    else
	{
	write_error( FALSE, "stop_callback", "fsize",
		     "Error retrieving value of ADAPSACQ" );
	strcpy(file_name,"");
	}
    strcat(file_name,"/");
    strcat(file_name,sceneid);
    strcat(file_name,".sfl");
    get_file_size(file_name,&file_size);
    if (file_size < MIN_FILE_SIZE)
	{
	write_error( FALSE, "stop_callback", "fsmall",
		     "Acquired an incomplete image" );
	strcpy(error_name,getenv("ADAPSERR"));
	strcat(error_name,"/");
	strcat(error_name,sceneid);
	strcat(error_name,".sfl");
	rename(file_name, error_name);
	}


    /* Determine the next scene to be acquired (from the master schedule)
    ----------------------------------------------------------------------*/
    get_next_scene(ltime_disp);


    /* Notify the SHOACQU child to terminate the displaying of the
       current acquisition, and display the time of the next scene
       to be acquired.
    ---------------------------------------------------------------*/
    sprintf( tmpbuf, "-next %s", ltime_disp );
    sprintf( msgbuf, "%03d%s", strlen(tmpbuf), tmpbuf);
    printf("Sending ???: %s\n",msgbuf);
    write( sd, msgbuf, strlen(msgbuf));


    /* Set an alarm to trigger the acquisition of the next scene
       in the schedule.
    --------------------------------------------------------------*/
    set_start_alarm ();


    /* Finish everything needed to complete the processing of the
       pass just acquired:
       - create the modsid file 
       - do post processing on the image file ????
       NOTE:  The modsid file and the acquired image will be moved to
	      the production system by PUSHACQU.
    ---------------------------------------------------------------------*/
    create_modsid(sceneid);


    /* write a log file and exit
    ----------------------------*/
    write_log( FALSE, "stop_callback", "info",
        "ACQUMAN terminated live acquisition" );


    }  /* END stop_callback */
