/*******************************************************************************
NAME:		Miscellaneous support routines:
		   C_GREJUL
		   C_4DIGITYR
		   C_ISLEAP

*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "acquman.h"


/*******************************************************************************
NAME:		C_GREJUL

PURPOSE:	Convert a Gregorian date to a Julian date.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  2.0		2/91	D. Hollaren	   CSB      Conversion to LAS5.0

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCE:
*******************************************************************************/

static long days1[13] = {0,31,59,90,120,151,181,212,243,273,304,334,365};
static long days2[13] = {0,31,60,91,121,152,182,213,244,274,305,335,366};

void c_grejul
(
   long *year,          /* I:  Year                                     */
   long *month,         /* I:  Gregorian month                          */
   long *day,           /* I:  Gregorian day                            */
   long *jdate          /* O:  Julian date                              */
)
{

/*
  Calculate the julian date
*/
if (c_isleap(year))
    *jdate = days2[*month-1] + *day;		/* Leap year		*/
else
    *jdate = days1[*month-1] + *day;		/* Non-leap year	*/
}


/*****************************************************************************
FUNCTION:	C_4DIGITYR

PURPOSE:	Convert a 2- or 3- digit year in the range of 0 to 199
		into a 4-digit year returning the 4-digit year and leaving
		the input parameter unchanged.  3-digit year inputs are
		assumed to be in [2000,2099].  4-digit years are returned
		unchanged.  For all other values an error is generated.

Version  Date   Programmer      Modifications
-------  -----  --------------- ---------------------------------------------
  1.0    07/98  D.Lloyd         Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        LAS
*****************************************************************************/

#define  XX  50
#define  YY  5


long c_4digityr
(
   long year       /* I: year value to be converted                    */
)
{
struct	tm	*lctm;		/* local time structure			*/
long	lyear;			/* local derived year value		*/
time_t	t;			/* seconds since Jan 1, 1970		*/

t = time( NULL );		/* get the local time			*/
lctm = localtime( &t );		/* fill the tm structure		*/


/* 2-digit year.  If the value is less than XX (tbd -- say 50)
   then we will assume the year is > 2000 unless adding 2000
   would make the date more than YY (tbd -- say 5) years in the
   future.  Example: year=03, lctm->tm_year=1998 assume year is
   2003, however if year=04 assume 1904.
----------------------------------------------------------------*/
if ( year >= 0 && year < 100 )
    {
    if ( year < XX )
        {
        lyear = year + 2000;
	if ( lyear > ( lctm->tm_year + 1900 + YY ))
	    lyear -= 100;
	}
    else
        lyear = year + 1900;
    }

/* 3-digit year.  Assumed to be a count of the number of
   years since 1900 so simply add 1900
--------------------------------------------------------*/
else if ( year >= 100 && year < 200 )
    lyear = year + 1900;

/* Any 4-digit year is just returned
-------------------------------------*/
else if ( year >= 1000 && year < 2100 )
    lyear = year;

/* Year outside of acceptable range is an error
   those in [200, 999]
------------------------------------------------*/
else
    {
    write_error( FALSE, "c_4digityr", "year", "Year out of range" );
    lyear = -1;
    }

return lyear;
}

/************************************************************************
NAME:		C_ISLEAP

PURPOSE:	Test if year given is a leap year.

PROGRAM HISTORY:

VERSION         PROGRAMMER           DATE              COMMENTS
-------         ----------           ----              --------
 2.0            D. Hollaren          2/91              Conversion to C


ALGORITHM DESCRIPTION:

ALGORITHM REFERENCES:

**************************************************************************/

int c_isleap
(
   long *year             /* I:  Year to test                           */
)

{

long	leap;		/* Leap year flag	*/
long	lyear;

/****************************************************************
DLLOYD Y2K
  make a local copy of year and convert to 4-digit for the check
*****************************************************************/
lyear = c_4digityr(*year);

if (((lyear % 4) != 0) || (((lyear % 100) == 0) && ((lyear % 400) != 0)))
    leap = FALSE;
else
    leap = TRUE;

return(leap);
}
