/*****************************************************************************
NAME:                   CHECK_AUDIT

PURPOSE:                Check a record in the master schedule to determine
			if the scene needs to be updated by the
			schedule auditor.

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "readmsch.h"


int check_audit(

	char inputline[],		/* Buffer of line of data	*/
	char ceosids[3][4],		/* Ceos id's to consider	*/
	long nceosids,			/* # id's specified in ceosids	*/
	char statflg[],			/* Status flag of scene:	*/
					/* D : Delete the acquired scene*/
					/* DM: Delete the missed scene	*/
					/* M : Mark the scene as missed	*/
					/* A : Time to acquire the scene*/
	char sceneid[],			/* Sceneid of scene		*/
	struct TIMESTR *timestr		/* Time structure		*/
)

{

long ceos_match;		/* Ceos id match flag		*/
long i;				/* Counter			*/
long two = 2;			/* Constant value of two	*/


/* Initialize return values 
----------------------------*/
strcpy(sceneid,"");
strcpy(statflg,"");


/* Determine if the scene is past the time to delete it from the
   schedule.  If it is, return the sceneid and the status flag.
-------------------------------------------------------------------*/
   
if (pasttime(inputline,timestr->TIME_DELETE))
    {
    strncpy(sceneid,&inputline[0],16);
    sceneid[16] = '\0';
    c_up2low(sceneid,&two);	/* Only first 2 char are alpha char */

    strcpy(statflg,"D");

    /* If the scene is a missed scene return that information also
       so that it can be noted that a missed scene is being deleted
       from the schedule.  If a scene is still pending, but is past
       the delete time, it has apparently been missed, even if not
       tagged as such.
    ----------------------------------------------------------------*/
    if ((inputline[63] == 'M') || (inputline[63] == 'P'))
	strcat(statflg,"M");

    }


/* If the scene is in a pending status, and if it is past the time to
   be able to acquire the scene from anywhere, we need to mark the
   scene as "missed" in the schedule.  Return the sceneid and the
   status flag.
-------------------------------------------------------------------*/

else if ((inputline[63] == 'P') && (pasttime(inputline,timestr->TIME_MISSED)))
    {
    strncpy(sceneid,&inputline[0],16);
    sceneid[16] = '\0';
    c_up2low(sceneid,&two);	/* Only first 2 char are alpha char */

    strcpy(statflg,"M");
    }

/* If the scene is in a pending status, and if it is now time to
   be able to acquire the scene, we need to start checking
   the logs for select ceos id's to determine if they have been
   acquired.  For those scenes acquired according to the logs,
   the schedule will then be modified to show a "Disk" status.
   This is the mechanism wherein live (SFL) pending scenes get
   marked as acquired in the schedule.  The same goes for DOMSAT
   scenes (NOA, NGC) if the communications dish is being used.
   Return the sceneid and the status flag.
-------------------------------------------------------------------*/

else if ((inputline[63] == 'P') && (pasttime(inputline,timestr->TIME_ACQUIRE)))
    {

    /* See if we have a matching ceos id
    ----------------------------------------*/
    ceos_match = FALSE;
    for (i = 0; i < nceosids; i++)
	if (((inputline[72] == 'E') && (strcmp(ceosids[i],"SFL") == 0)) ||
	    ((inputline[72] == 'F') && (strcmp(ceosids[i],"NGC") == 0)) ||
	    ((inputline[72] == 'W') && (strcmp(ceosids[i],"NOA") == 0)))
	    ceos_match = TRUE;

    if (ceos_match)
	{
	strncpy(sceneid,&inputline[0],16);
	sceneid[16] = '\0';
	c_up2low(sceneid,&two);	/* Only first 2 char are alpha char */

	strcpy(statflg,"A");
	}
    }

/* If no match was made, return failure; otherwise
   return success.
---------------------------------------------------*/
if (strcmp(sceneid,"") == 0)
    return(E_FAIL);
else
    return(E_SUCC);

}

