/*****************************************************************************
NAME:                   CHECK_DOWNL

PURPOSE:                Check a line of the master schedule to determine
			if the scene id should be downloaded from the
			NOAA server.

NOTE:	The scene syntax on the NOAA server could change with the advent of a
	new satellite.  Consequently, in the event of a new NOAA satellite,
	this routine may have to be updated.

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "readmsch.h"


int check_downl(

	char inputline[],		/* Data record from master sch	*/
	char noaaid[CMLEN],		/* NOAA sceneid syntax string	*/
	char sceneid[CMLEN],		/* Sceneid of pending scene	*/
	struct TIMESTR *timestr 	/* Time structure		*/
)

{

char outstr[100];		/* Temporary output string	*/
char tmpstr[5];			/* Temporary string		*/

int  sid;			/* satellite id value		*/

long iday;			/* Integer day of acquisition	*/
long ijdate;			/* Integer julian date		*/
long imonth;			/* Integer month of acquisition	*/
long iyear;			/* Integer year of acquisition	*/

long two = 2;			/* Constant value of two	*/


/* When a "Pend" scene is encountered, determine if a sufficient amount
   of time has passed such that the scene could reasonably be expected to
   reside on the NOAA server.  Extract and return the appropriate info.:
   Examples:  "'NSS.HRPT.NJ.D97172.S0008.E*'"  and  "ah14062197000820"
              "'NSS.LHRR.NJ.D97173.S0441.E*'"  and  "al14062297044110"
-----------------------------------------------------------------------------*/
if ((inputline[63] == 'P') && (pasttime(inputline,timestr->TIME_ACQUIRE)))
    {
    /* Assemble the NOAA sceneid syntax */
    strcpy(outstr,"'NSS.");
    if (inputline[1] == 'H')
	strcat(outstr,"HRPT.N");
    else
	strcat(outstr,"LHRR.N");

    /* Determine and set the satid eg: 14 = 'NJ', 15 = 'NK'
    -------------------------------------------------------*/
    strncpy(tmpstr,&inputline[2],2);
    tmpstr[2] = '\0';
    sscanf(tmpstr,"%d",&sid);
    switch ( sid )
        {
	case  6 : strcat(outstr,"A"); break;
	case  7 : strcat(outstr,"C"); break;
	case  8 : strcat(outstr,"E"); break;
	case  9 : strcat(outstr,"F"); break;
	case 10 : strcat(outstr,"G"); break;
	case 11 : strcat(outstr,"H"); break;
	case 12 : strcat(outstr,"D"); break;
	case 13 : strcat(outstr,"I"); break;
	case 14 : strcat(outstr,"J"); break;
	case 15 : strcat(outstr,"K"); break;
	case 16 : strcat(outstr,"L"); break;
	case 17 : strcat(outstr,"M"); break;
	case 18 : strcat(outstr,"N"); break;
	default : strcat(outstr,"?"); break;
	}

    strcat(outstr,".D");
    strncpy(tmpstr,&inputline[8],2);
    tmpstr[2] = '\0';
    strcat(outstr,tmpstr);
    sscanf(tmpstr,"%ld",&iyear);
    strncpy(tmpstr,&inputline[4],2);
    tmpstr[2] = '\0';
    sscanf(tmpstr,"%ld",&imonth);
    strncpy(tmpstr,&inputline[6],2);
    tmpstr[2] = '\0';
    sscanf(tmpstr,"%ld",&iday);
    c_grejul(&iyear,&imonth,&iday,&ijdate);
    sprintf(tmpstr,"%03ld",ijdate);
    strcat(outstr,tmpstr);
    strcat(outstr,".S");
    strncpy(tmpstr,&inputline[10],4);
    tmpstr[4] = '\0';
    strcat(outstr,tmpstr);
    strcat(outstr,".E*'");
    strcpy(noaaid,outstr);

    /* Assemble the sceneid */
    strncpy(sceneid,&inputline[0],16);
    sceneid[16] = '\0';
    c_up2low(sceneid,&two);	/* Only first 2 char are alpha char */

    return(E_SUCC);
    }

return(E_FAIL);
}

