/*****************************************************************************
NAME:                   GETINFO

PURPOSE:                Get the relevant information from the master schedule
			for a scene which meets predefined constraints.

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "readmsch.h"

#define LINELEN 86	/* the length of a record in the master schedule */

int getinfo(

	char filnam[],			/* Master schedule file name	*/
	long *linenum,			/* Starting line number in sched*/
					/* at which to start processing	*/
	char ceosids[3][4],		/* CEOS id's to consider	*/
	long nceosids,			/* Number of id's in ceosids	*/
	struct TIMESTR *timestr,	/* Time structure		*/
	char noaaid[],			/* NOAA scene id syntax 	*/
	char statflg[],			/* Status flag of scene		*/
	char sceneid[],			/* Sceneid of pending scene	*/
	char subcmd[] 			/* Sub-command			*/
)

{

char inputline[LINELEN];	/* Buffer of line of data	*/

long i;				/* Counter			*/
long line;			/* Line counter			*/
long status;			/* Return status		*/

FILE   *infptr1;		/* File pointer to schedule	*/


/* Open the schedule listing
--------------------------------------*/
infptr1 = fopen(filnam,"r");
if (infptr1 == NULL)
    {
    c_lasmsg(FALSE,"readmsch-sopen","Error opening schedule listing");
    return(E_FAIL);
    }


/* Skip down to the specified starting line.  Always read at least one
   line (first line of the master schedule contains the number of
   entries in the schedule).  So linenum=1 means process the first
   actual record in the master schedule (technically line two).
-----------------------------------------------------------------------------*/
for (line = 0; line < *linenum; line++)
    {
    if (fgets(inputline,LINELEN,infptr1) == NULL)
	{
	c_lasmsg(FALSE,"readmsch-blin","Invalid line specified");
	fclose(infptr1);
	return(E_FAIL);
	}
    }


/* Read the master schedule until the appropriate record has been located or
   until there is no more to read.  Reinitialize the buffer every time to
   ensure no reminant remains from a prior read.
-----------------------------------------------------------------------------*/
for (  ;  ; (*linenum)++ )
    {
    for (i = 0; i < LINELEN; i++)
	inputline[i] = '\0';
    if (fgets(inputline,LINELEN,infptr1) == NULL)
	{
	*linenum = 0;
	fclose(infptr1);
	return(E_SUCC);
	}

    if (strcmp(subcmd,"DOWNL") == 0)
	status = check_downl(inputline,noaaid,sceneid,timestr);
    else
	status = check_audit(inputline,ceosids,nceosids,statflg,
			     sceneid,timestr);

	if (status == E_SUCC)
	    {
	    fclose(infptr1);
	    (*linenum)++;
	    return(E_SUCC);
	    }

    }

/* One should never get to this point in the code! */
}

