/******************************************************************************
NAME:		GETPAR

PURPOSE:        Get the input parameters for READMSCH

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0       06/99    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT                 ADAPS

*******************************************************************************/
#include <stdio.h>              /* C standard I/O include file		*/
#include "las.h"		/* LAS include file			*/
#include "readmsch.h"


int getpar(

	struct	PARBLK *vblock,		/* TAE parameter block	 	 */
	char	subcmd[],		/* Sub-command			 */
	long	*linenum,		/* Line number at which to start */
					/* processing.			 */
	char	filnam[],		/* Name of input file		 */
	char	ceosids[3][4],		/* Array of ceos id's to consider*/
	long 	*nceosids,		/* Number of ceos id's in ceosids*/
	struct TIMESTR *timestr 	/* Time structure		 */
)

{

long access;			/* Type of access for getnam call	*/
long element = 0;		/* Element number for getnam call	*/
long i;				/* Counter				*/
long length;			/* Length of string			*/
long one = 1;			/* Constant value			*/
long status;                    /* Return status			*/

struct  VARIABLE *p_find();     /* function to find a TAE variable      */
struct  VARIABLE *val;          /* pointer to a TAE variable            */



/* Get the subcommand name
----------------------------------------------------*/
c_getstr (vblock,"_SUBCMD",&one,&length,subcmd);


/* Get the CEOS id's
----------------------------------------------------*/
if (strcmp(subcmd,"AUDIT") == 0)
    {
    val = p_find(vblock,"CEOSIDS");
    *nceosids = (val->v_count);
    for (i = 0; i < *nceosids; i++)
	strcpy(ceosids[i],SVAL(*val,i));
    }


/* Get the input master schedule
----------------------------------------------------*/
access = IREAD;
status = c_getnam(vblock,"INFILE",&element,filnam,"",&access);
if (status != E_SUCC)
    return(status);


/* Get the line number (the line number in the master
   schedule at which to start processing).  This
   counter does not include the first line in the master
   schedule which contains the total number of lines in
   the master schedule.  Consequently, linenum=1 actually
   refers to the second line within the master schedule.
----------------------------------------------------*/
val = p_find(vblock, "LINENUM" );
*linenum = IVAL( *val, 0);


/* Get the time related parameters
----------------------------------------------------*/
if (strcmp(subcmd,"AUDIT") == 0)
    {
    val = p_find(vblock, "TIME_DELETE" );
    timestr->TIME_DELETE = IVAL( *val, 0);
    timestr->TIME_DELETE *= 3600;		/* Convert hours to seconds */
    val = p_find(vblock, "TIME_MISSED" );
    timestr->TIME_MISSED = IVAL( *val, 0);
    timestr->TIME_MISSED *= 3600;		/* Convert hours to seconds */
    }
val = p_find(vblock, "TIME_ACQUIRE" );
timestr->TIME_ACQUIRE = IVAL( *val, 0);
timestr->TIME_ACQUIRE *= 60;		/* Convert minutes to seconds */

return(E_SUCC);
}
