/******************************************************************************
NAME:		READMSCH

PURPOSE:        Find the appropriate information from the master schedule

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0       06/99    T. Baltzer          Initial development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT                 ADAPS

*******************************************************************************/
#include <stdio.h>              /* C standard I/O include file		*/
#include "las.h"		/* LAS include file			*/
#include "readmsch.h"		/* This module's include file		*/


main()
{
struct PARBLK   vblock;         /* TAE parameter block			*/

char ceosids[3][4];		/* Array of ceos id's to consider	*/
char filnam[CMLEN];		/* Name of input file			*/
char noaaid[CMLEN];		/* Output NOAA scene id syntax		*/
char sceneid[CMLEN];		/* Sceneid of pending scene		*/
char statflg[3];		/* Status flag of scene			*/
char subcmd[10];		/* Sub-command				*/

long linenum;			/* Line number in the master schedule	*/
				/* at which to start processing.  This	*/
				/* value will also be returned such that*/
				/* it is ready for the next call to	*/
				/* READMSCH without any modification.	*/
long nceosids;			/* Number of ceos id's in ceosids	*/
long status;                    /* Return status			*/
long std_out;                   /* Standard output device		*/

struct TIMESTR timestr;		/* Time structure			*/



/* Initialize LAS
----------------------------*/
c_inlas(&vblock,&std_out);


/* Get the input parameters
-----------------------------------------------------------------*/
status = getpar(&vblock,subcmd,&linenum,filnam,ceosids,&nceosids,&timestr);
if (status != E_SUCC)
    c_lasmsg(TRUE,"readmsch-fatal","Fatal error encountered");


/* Determine the relevant scene in the master schedule and
   return the information needed to process it.
-----------------------------------------------------------------*/
status = getinfo(filnam,&linenum,ceosids,nceosids,&timestr,noaaid,statflg,
	 sceneid,subcmd);
if (status != E_SUCC)
    c_lasmsg(TRUE,"readmsch-fatal","Fatal error encountered");


/* Return the scene information found.
-------------------------------------------------------------*/
taeland(&vblock,subcmd,noaaid,statflg,sceneid,&linenum);


/* Exit with a success status, no message
----------------------------------------------*/
z_exit(1,"");
}
