/******************************************************************************
FUNCTION:	CHECKDAY

PURPOSE:	Validate the date entered by the user

PROGRAM HISTORY:
Version    Date     Author      Change Request
-------    -----    ---------   --------------
1.0        4/87     Hollaren
2.0        6/89     LaVergne    
5.0        10/92    T.Rockvam   Updated to LAS 5.0
                                moved from a PDF proc to a C function
5.1        02/00    D.Lloyd	Modified for Y2K
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util2.h"
#include "worgen.h"
#include "autogen.h"


int checkday
(
    long        date[]		/* date (month, day, year)              */
)
{
long	nonfatal = 1;		/* nonfatal error code			*/
static long days[12] =  {31,28,31,30,31,30,31,31,30,31,30,31};
static long ldays[12] = {31,29,31,30,31,30,31,31,30,31,30,31};

/*
  Check to make sure the month is valid
*/
if ((date[0] < 1) || (date[0] > 12))
    {
    c_errmsg("Invalid month specified","autogen-month",&nonfatal);
    return(E_FAIL);
    }

/*
  Set up the year as a 4-digit year
  call c_4digityr for Y2K
*/
date[2] = c_4digityr( date[2] );

/*
  Check to make sure the year is valid
  check against 2100 for Y2K
*/
if ((date[2] < 1986) || (date[2] > 2100))
    {
    c_errmsg("Invalid year specified","autogen-year",&nonfatal);
    return(E_FAIL);
    }

/*
  Check to make sure the day is valid
*/
if (c_isleap(&date[2]))
    {
    if ((date[1] < 1) || (date[1] > ldays[date[0]-1]))
        {
        c_errmsg("Invalid day of month specified","autogen-day",&nonfatal);
        return(E_FAIL);
        }
    }
else
    {
    if ((date[1] < 1) || (date[1] > days[date[0]-1]))
        {
        c_errmsg("Invalid day of month specified","autogen-day",&nonfatal);
        return(E_FAIL);
        }
    }

return(E_SUCC);
}
