/******************************************************************************
FUNCTION:	cpoly

PURPOSE:	Find out if the satellite location is inside the 
		ascending/descending coverage circle
		We go around the coverage circle, finding the point on each 
		side of the circle that intersects the given latitude (if it 
		does intersect).  The longitudes of these 2 points are saved.  
		If the input longitude falls between the longitudes of these 
		2 points, the input point is deemed "INSIDE".

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
 1.0 	   			Original development
 5.0	   10/92    T.Rockvam	Conversion to LAS 5.0
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "autogen.h"
#define	INSIDE	1
#define	OUTSIDE	0
#define ASCENDING   1
#define DESCENDING -1

int cpoly
(
    float       lon,			/* lat/long of point to check   */
    float       lat			/* lat/long of point to check   */
)
{
extern float circle[NUMPNT][2];		/* coverage circle		*/
long	i, j;				/* looping counters		*/
float	temp[2];			/* longitude on each side of the*/
					/* cov circle at the input latitude */

/*
  Find the point on each side of the circle at that lat 
*/
for(i=0,j=0; i<NUMPNT-1; i++)
    {
    if (((circle[i][1] <= lat) && (circle[i+1][1] > lat)) ||
        ((circle[i][1] >= lat) && (circle[i+1][1] < lat)))
    	{
    	/* found the input lat; save the longitude at this point */
    	temp[j] = (circle[i][0] + circle[i+1][0]) / 2.0;
    	j++;
    	if (j==2)
    	    /* found 2 points on the circle at the given latitude */
    	    if (((temp[0] < lon) && (temp[1] > lon)) ||
    	        ((temp[0] > lon) && (temp[1] < lon)))
    		return(INSIDE);
    	}
    }

return(OUTSIDE);
}
