/******************************************************************************
FUNCTION:	findclosest

PURPOSE:	Find the time, and lat/long of the closest approach in 
		this orbit

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
1.0        4/87     Hollaren
2.0        6/89     LaVergne	
5.0	   10/92    T.Rockvam	Conversion to LAS 5.0
				Uses new model
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "autogen.h"


int findclosest
(
    struct MODEL *model,	/* model information                    */
    double      stalat,		/* station latitude                     */
    double      stalon,		/* station longitude                    */
    long        *offtime,	/* offset in time for closest approach  */
    double      *applat,	/* approach latitude                    */
    double      *applon		/* approach longitude                   */
)
{
float	pasline,		/* line in pass at station lat/long	*/
	passamp,		/* sample in pass at station lat/long	*/
	nadirsamp;		/* nadir sample in pass			*/

double	elev = 0.0;		/* elevation				*/

/*
  Find the line/sample at the station lat/long
*/
c_ll2ls(&stalat,&stalon,&elev,model,&pasline,&passamp,NULL);

/*
  Find the lat/long at the nadir line/samp
*/
nadirsamp = LAC_NADIR;
c_ls2ll(&pasline,&nadirsamp,&elev,model,applat,applon,NULL);

/*
  Calculate the time offset at this point using the line #
*/
*offtime = (long)((pasline - 1.0)/LAC_PER_SEC);

return(E_SUCC);
}
