/******************************************************************************
FUNCTION:	findends

PURPOSE:	Find the start and end time of the pass

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
  1.0      4/87     Hollaren
  2.0      6/89     LaVergne    
  5.0      10/92    T.Rockvam   Conversion to LAS 5.0; uses new model

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "autogen.h"


void findends
(
    struct MODEL        *model,	/* model information            */
    long        mintime,	/* minimum time for the pass    */
    long        maxtime,	/* maximum time for the pass    */
    long        pastime,	/* middle time for the pass     */
    double      paslat,		/* pass lat at closest point    */
    double      paslon,		/* pass long at closest point   */
    long        *startoff,	/* time offset for pass start   */
    long        *endoff		/* time offset for pass end     */
)
{
float	midline,		/* middle line number of pass	*/
	line,			/* working line number		*/
	samp;			/* working sample number	*/

double	elev = 0.0,		/* elevation			*/
	lat,			/* working latitude of scene	*/
	lng,			/* working longitude of scene	*/
	thistime,		/* time of current line number	*/
	startime = 0.0,		/* start time of pass		*/
	endtime = 0.0,		/* end time of pass		*/
	line_inc,		/* increment for checking scene */
	time_inc = 15.0;	/* time offset (15 secs)	*/

/*
  Find the current line/sample of the given lat/long
*/
lat = paslat;
lng = paslon;
c_ll2ls(&lat,&lng,&elev,model,&midline,&samp,NULL);
thistime = (double)pastime;
line_inc = time_inc * LAC_PER_SEC;

/*
  Loop, moving back one interval at a time, until we are 
  out of the coverage circle
*/
lat = paslat;
lng = paslon;
line = midline;
while (cpoly(lng*DEG_PER_RAD,lat*DEG_PER_RAD))
    {
    line -= line_inc;
    thistime -= time_inc;
    c_ls2ll(&line,&samp,&elev,model,&lat,&lng,NULL);
    }

/*
  The loop stops when we are just out of the coverage circle
*/
startime = thistime;
if ((startime < mintime) && (mintime != maxtime))
    startime = mintime;

/*
  Loop, moving forward one interval at a time, until we are 
  out of the coverage circle
*/
lat = paslat;
lng = paslon;
thistime = (double)pastime;
line = midline;
while (cpoly(lng*DEG_PER_RAD,lat*DEG_PER_RAD))
    {
    line += line_inc;
    thistime += time_inc;
    c_ls2ll(&line,&samp,&elev,model,&lat,&lng,NULL);
    }

/*
  The loop stops when we are just out of the coverage circle
*/
endtime = thistime;
if ((endtime > maxtime) && (mintime != maxtime))
    endtime = maxtime;

/*
  Make the time on either side of pastime equal
*/
if ((endtime - pastime) < (pastime - startime))
     startime = pastime - (endtime - pastime);
else
     endtime = pastime + (pastime - startime);

/*
  Find time offset for start & end of pass
*/
*startoff = (long)(startime - pastime);
*endoff   = (long)(endtime - pastime);

return;
}
