/******************************************************************************
FUNCTION:	getpar

PURPOSE:	get parameters from TAE

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
 1.0				Original Development
 5.0       10/92    T.Rockvam	Conversion to LAS 5.0
 5.1       02/00    D.Lloyd     Modified for Y2K
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "adaps.h"
#include "autogen.h"

int getpar
(
    struct  PARBLK      *vblock,/* TAE variable block                   */
    FILE    **report,		/* report file                  */
    FILE    **sched,		/* schedule file                */
    char        *satid,		/* name of the selected satellite       */
    long        sdate[],	/* starting date                        */
    long        edate[],	/* ending date                          */
    long        stime[],	/* starting time                        */
    long        etime[],	/* ending time                          */
    float       target[],	/* lat,long of target area (radians)    */
    long        *opmode,	/* operation mode (data destination)    */
    long        *minsecs,	/* minimum pass time                    */
    long        *varsecs,	/* max variance about closest approach  */
    float       *maxdist,	/* maximum distance from here for passes*/
    long        *schadd,	/* schedule add flag                    */
    char        outnames[][CMLEN],/* output file names                    */
    long        *outcount,	/* number of output files               */
    long        *msgtim		/* message time                         */
)
{
struct VARIABLE	*var;		/* pointer to a parameter		*/

long 	i,			/* loop counter				*/
	nonfatal = 1,		/* nonfatal error code			*/
	ssecs,			/* total seconds of start time		*/
	esecs,			/* total seconds of end time		*/
	mintime[3],		/* minimum pass time			*/
	vartime[3];		/* variance time			*/

char	filnam[100];		/* name of temporary file		*/

struct  tm      *lctm;          /* time structure                       */

time_t  t;                      /* seconds since beginning of unix time */


/*
  Get the satellite
*/
var = p_find(vblock,"SATID");
strcpy(satid,SVAL(*var,0));

/*
  Get and validate the starting and ending dates
*/
var = p_find(vblock,"STADATE");
for (i=0; i<3; i++)
    sdate[i] = IVAL(*var,i);
if (checkday(sdate) != E_SUCC)
    return(E_FAIL);
var = p_find(vblock,"STODATE");
if (var->v_count == 0)
    {
    /* this will give us one weeks worth of data
    --------------------------------------------*/
    t = time( NULL );
    lctm = localtime( &t );
    lctm->tm_mon = sdate[0] - 1;
    lctm->tm_mday = sdate[1];
    /* Y2K -- must subtract 1900 for tm_year */
    lctm->tm_year = sdate[2] - 1900;
    t = mktime( lctm );
    t += 7 * SEC_PER_DAY;
    lctm = localtime( &t );
    edate[0] = lctm->tm_mon + 1;
    edate[1] = lctm->tm_mday;
    edate[2] = lctm->tm_year + 1900;

    /* original was to copy sdate into edate
    for(i=0; i<3; i++)
        edate[i] = sdate[i];
    -----------------------------------------*/
    }
else if (var->v_count == 3)
    for (i=0; i<3; i++)
        edate[i] = IVAL(*var,i);
else
    {
    c_errmsg("Invalid number of parameters for STODATE","autogen-numparm",
	&nonfatal);
    return(E_FAIL);
    }
if (checkday(edate) != E_SUCC)
    return(E_FAIL);

/*
  Get and validate the starting and ending times
*/
var = p_find(vblock,"STATIME");
if (var->v_count == 0)
    /* Set times to pick up day passes */
    {
    if (strcmp(satid,"N09") == 0)
	{
	stime[0] = 18;
	stime[1] = 0;
	stime[2] = 0;
	}
    else if (strcmp(satid,"N10") == 0)
	{
	stime[0] = 12;
	stime[1] = 0;
	stime[2] = 0;
	}
    else if (strcmp(satid,"N11") == 0)
	{
	stime[0] = 16;
	stime[1] = 0;
	stime[2] = 0;
	}
    else /* N12 */
	{
	stime[0] = 12;
	stime[1] = 0;
	stime[2] = 0;
	}
    }
else if (var->v_count == 3)
    for (i=0; i<3; i++)
        stime[i] = IVAL(*var,i);
else
    {
    c_errmsg("Invalid number of parameters for STATIME","autogen-numparm",
	&nonfatal);
    return(E_FAIL);
    }
if (checktim(stime) != E_SUCC)
    return(E_FAIL);

var = p_find(vblock,"STOTIME");
if (var->v_count == 0)
    {
    /* Set times to pick up day passes */
    if (strcmp(satid,"N09") == 0)
	{
	etime[0] = 23;
	etime[1] = 59;
	etime[2] = 59;
	}
    else if (strcmp(satid,"N10") == 0)
	{
	etime[0] = 16;
	etime[1] = 30;
	etime[2] = 0;
	}
    else if (strcmp(satid,"N11") == 0)
	{
	etime[0] = 23;
	etime[1] = 59;
	etime[2] = 59;
	}
    else /* N12 */
	{
	etime[0] = 16;
	etime[1] = 30;
	etime[2] = 0;
	}
    }
else if (var->v_count == 3)
    for (i=0; i<3; i++)
        etime[i] = IVAL(*var,i);
else
    {
    c_errmsg("Invalid number of parameters for STOTIME","autogen-numparm",
	&nonfatal);
    return(E_FAIL);
    }
if (checktim(etime) != E_SUCC)
    return(E_FAIL);

/*
  Make sure time range is valid
*/
makesecs(sdate,stime,&ssecs);
makesecs(edate,etime,&esecs);
if (ssecs >= esecs)
    {
    c_errmsg("Stop time is before start time","autogen-time",&nonfatal);
    return(E_FAIL);
    }

/*
  Get target pass location
*/
var = p_find(vblock,"TARGET");
for (i=0; i<2; i++)
    target[i] = RVAL(*var,i) * RAD_PER_DEG;
if ((target[0] > 90.0) || (target[0] < -90.0))
    {
    c_errmsg("Invalid latitude value","autogen-invlat",&nonfatal);
    return(E_FAIL);
    }
target[2] = 0.0;

/*
  Get the operation destination mode
*/
var = p_find(vblock,"OPMODE");
*opmode = IVAL(*var,0);

/*
  Get the minimum pass time
*/
var = p_find(vblock,"MINTIME");
if (var->v_count == 0)
    for (i=0; i<3; i++)
        mintime[i] = 0;
else if (var->v_count == 3)
    for (i=0; i<3; i++)
        mintime[i] = IVAL(*var,i);
else
    {
    c_errmsg("Invalid number of parameters for MINTIME","autogen-numparm",
	&nonfatal);
    return(E_FAIL);
    }
if (checktim(mintime) != E_SUCC)
    return(E_FAIL);
*minsecs = mintime[0] * 3600 + mintime[1] * 60 + mintime[2];

/*
  Get the variance time each side of a pass
*/
var = p_find(vblock,"VARTIME");
if (var->v_count == 0)
    for (i=0; i<3; i++)
        vartime[i] = 0;
else if (var->v_count == 3)
    for (i=0; i<3; i++)
        vartime[i] = IVAL(*var,i);
else
    {
    c_errmsg("Invalid number of parameters for VARTIME","autogen-numparm",
	&nonfatal);
    return(E_FAIL);
    }
if (checktim(vartime) != E_SUCC)
    return(E_FAIL);
*varsecs = vartime[0] * 3600 + vartime[1] * 60 + vartime[2];

/*
  Get the maximum nautical miles from here to select passes
  Convert to meters
*/
var = p_find(vblock,"MAXMILE");
*maxdist = RVAL(*var,0) * M_PER_MILE;

/*
  Get the flag to add selected passes to the schedule file
  and open temporary file if necessary
*/
*sched = NULL;
var = p_find(vblock,"UPDATFLG");
if (strcmp(SVAL(*var,0),"YES") == 0)
    {
    *schadd = 1;
    var = p_find(vblock,"TMPSCHED");
    strcpy(filnam,SVAL(*var,0));
    *sched = fopen(filnam,"w");
    if (*sched == NULL)
	{
	c_errmsg("Error opening schedule update file","autogen-open",&nonfatal);
	return(E_FAIL);
	}
    strcpy(outnames[*outcount],filnam);
    (*outcount)++;
    }

/*
  Always open a temporary listing file
*/
var = p_find(vblock,"TMPLIST");
strcpy(filnam,SVAL(*var,0));
*report = fopen(filnam,"w");
if (*report == NULL)
    {
    c_errmsg("Error opening autogen report file","autogen-open",&nonfatal);
    return(E_FAIL);
    }
strcpy(outnames[*outcount],filnam);
(*outcount) ++;

/*
  Get the message time
*/
var = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*var,0);

return(E_SUCC);
}
