/******************************************************************************
FUNCTION:	header

PURPOSE:	Print header information to the output report file

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
  5.0      10/92    T.Rockvam	Original development
  5.1      02/00    D.Lloyd     Minor printf format change for Y2K
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "autogen.h"


void header
(
    FILE        *report,	/* file descriptor for autosched report */
    char        *satnam,	/* satellite name ("NOAA-11")           */
    char        *satno,		/* satellite id ("19531")               */
    long        sdate[3],	/* start date of interest               */
    long        stime[3], 	/* start time of interest               */
    long        edate[3], 	/* end date of interest                 */
    long        etime[3], 	/* end time of interest                 */
    float       targlat,	/* target lat,long                      */
    float       targlon		/* target lat,long                      */
)
{
extern long	lincnt;		/* report line count			*/

static char month[12][4] = {"JAN","FEB","MAR","APR","MAY","JUN",
		      "JUL","AUG","SEP","OCT","NOV","DEC"};

/* Add some dummy lines to make this report the same size as the xplot report */
fprintf( report, "\f\n\n\n\n\n\n" );

/*
  Tell the user the satellite name and ID
*/
fprintf(report," Schedule of passes to be acquired for %s  ID = %s\n\n",
    satnam,satno);

/*
  Show the times that were entered
*/
fprintf(report," Period of interest:\n");
/* change year print format to %02d next four lines */
fprintf(report,"    Start %02d-%s-%02d %02d:%02d:%02d GMT\n",sdate[1],
	month[sdate[0]-1],sdate[2]%100,stime[0],stime[1],stime[2]);
fprintf(report,"    Stop  %02d-%s-%02d %02d:%02d:%02d GMT\n\n",edate[1],
	month[edate[0]-1],edate[2]%100,etime[0],etime[1],etime[2]);

/*
  Show where the antenna is
*/
if (targlat < 0.0)
    fprintf(report," Receiving station coordinates:   LAT. %4.2lf S\n",-targlat*DEG_PER_RAD);
else
    fprintf(report," Receiving station coordinates:   LAT. %4.2lf N\n", targlat*DEG_PER_RAD);

if (targlon < 0.0)
    fprintf(report,"                                  LON. %4.2lf W\n\n",-targlon*DEG_PER_RAD);
else
    fprintf(report,"                                  LON. %4.2lf E\n\n", targlon*DEG_PER_RAD);

/*
  Write out the headings
*/
fprintf(report,"   Orbit        Start       Start       End         End      Length     Closest      Time in   Target       Pass    Operation\n");
fprintf(report,"   Number       Date        Time        Date        Time     of Pass    Approach      View    Distance    Selected    Mode   \n");
fprintf(report,"   ______       _____       _____       ____        ____     _______    ________     _______  ________    ________  _________\n\n");

lincnt = 19;

return;
}
