/******************************************************************************
FUNCTION:	write_report

PURPOSE:	Print pass information to the output report file

PROGRAM HISTORY:
Version    Date     Author	Change Request
-------    -----    ---------	--------------
 1.0				Original development
 5.0	   10/92    T.Rockvam	Conversion to LAS 5.0
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adapsutil.h"
#include "autogen.h"

void write_report
(
    FILE        *report,	/* file descriptor for autosched report */
    long        orbit,		/* orbit number                         */
    char        *satnam,	/* satellite name ("NOAA-11")           */
    char        *satno,		/* satellite id ("19531")               */
    long        intsdate[3],	/* interval start date (mm dd yy)       */
    long        intstime[3],	/* interval start time (hh mm ss)       */
    long        intedate[3],	/* interval end date (mm dd yy)         */
    long        intetime[3],	/* interval end time (hh mm ss)         */
    long        sdate[3],	/* start date of interest (mm dd yy)    */
    long        stime[3],	/* start time of interest (hh mm ss)    */
    long        edate[3],	/* end date of interest (mm dd yy)      */
    long        etime[3],	/* end time of interest (hh mm ss)      */
    long        passlength,	/* length of pass (seconds)             */
    long        closestsecs,	/* time of closest approach (secs from 1986) */
    long        viewtime,	/* time in view (seconds)               */
    float       targlat,	/* target lat,long                      */
    float       targlon,	/* target lat,long                      */
    double      dist,		/* distance from target to closest point*/
    long        opmode		/* operations code                      */
)
{
extern long	lincnt;		/* report line count			*/

long	length[3],		/* (hh mm ss) of pass length		*/
	closest[3],		/* (hh mm ss) of closest approach	*/
	view[3],		/* (hh mm ss) of view length		*/
	temp[4],		/* temp var for mm/dd/yy,jday		*/
	fsec;			/* fractional secs			*/

/*
  Check the line count to determine if the report should continue on a new page
*/
if ((lincnt == 0) || (lincnt >= 58))
    header(report,satnam,satno,intsdate,intstime,intedate,intetime,
	   targlat,targlon);

/*
  Convert time of closest approach, length of pass, time in view to hour,min,sec
*/
consec(&closestsecs,&temp[0],&temp[1],&temp[2],&temp[3],
           &closest[0],&closest[1],&closest[2]);
c_cvtsec(&passlength,&length[0],&length[1],&length[2],&fsec);
c_cvtsec(&viewtime,&view[0],&view[1],&view[2],&fsec);

/*
  Write the record to the list of all passes
*/
fprintf(report, "   %05d      %02d/%02d/%02d    %02d:%02d:%02d    ",
	orbit, sdate[0],sdate[1],sdate[2]%100,stime[0],stime[1],stime[2]);
fprintf(report, "%02d/%02d/%02d    %02d:%02d:%02d   %02d:%02d:%02d   ",
	edate[0],edate[1],edate[2]%100,etime[0],etime[1],etime[2],
	length[0],length[1],length[2]);
fprintf(report, "%02d:%02d:%02d     %02d:%02d:%02d  %8.2lf      ",
	closest[0],closest[1],closest[2],
	view[0],view[1],view[2],dist/M_PER_MILE);
/*  I don't want any passes scheduled for this app.
fprintf(report, "%s        %d\n", 
	select,opmode);
*/
fprintf(report, "          %d\n", opmode);
lincnt++;

return;
}
