/******************************************************************************
FUNCTION:	avhrr2proj

PURPOSE:	convert line/sample in the AVHRR image to a line/sample in the 
		projected image

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"

FUNCTION avhrr2proj(avhrrline,avhrrsamp,projline,projsamp,model,elev,ddr,
		    geo_code,geo_zone,geo_units,ddr_units,geo_parms)

struct  MODEL 	*model;	/* AVHRR model info			*/
struct  DDR 	*ddr;	/* ref img ddr				*/

long	*geo_code,	/* projection code                      */
	*geo_zone,     	/* projection zone                      */
	*geo_units,    	/* projection units                     */
	*ddr_units;	/* ddr units				*/

float	*avhrrline,	/* input line in AVHRR image		*/
	*avhrrsamp;	/* input sample in AVHRR image		*/

double	elev,		/* elevation				*/
	*projline,	/* output line in projection image 	*/
	*projsamp,	/* output sample in projection image 	*/
	geo_parms[];	/* geographic parameters 		*/

{
double 	lat,lon,	/* latitude/longitude at the point	*/
	coordx,coordy;	/* x and y projection coordinates	*/

/*
  Find the lat/long location of the line/sample
*/
c_ls2ll(avhrrline,avhrrsamp,&elev,model,&lat,&lon,NULL);
lat *= DEG_PER_RAD;
lon *= DEG_PER_RAD;

/*
  Convert this lat/long location to projection coords
*/
c_trans(geo_code,geo_units, &ddr->proj_code,ddr_units,
       &lon,&lat,&coordx,&coordy);

/*
  Find the line/samp in the projected image
  of this projection coord
*/
*projline = (ddr->upleft[0] - coordy) / ddr->pdist_y + 1.0;
*projsamp = (coordx - ddr->upleft[1]) / ddr->pdist_x + 1.0;

return;
}
