/*****************************************************************************
NAME:				AVHRRCHIPS

PURPOSE:  Creates AVHRR image chips in the raw AVHRR space from a single
	  base image and a set of tie points in a TPS file.

VERSION		DATE	AUTHOR				REASON
-------		----	------				------
  1.0		5/93    D.R. Steinwand/T. Rockvam 	Initial release

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "proj.h"
#include "geompak.h"
#include "avhrrchips.h"

main()
{
struct PARBLK  vblock;         	/* TAE parameter block                  */
struct TPSHEAD ref_hdr;         /* reference tps file header 		*/
struct TPSDATA ref_data;        /* search tps data record 		*/
struct MTPHEAD mtp_hdr;         /* merged tie point header record 	*/
struct MTPDATA mtp_data;        /* merged tie point data record 	*/
struct MODEL model;		/* AVHRR model info			*/
struct DDR refddr;		/* DDR of reference (base) image	*/
struct DDR avhrrddr;		/* DDR of AVHRR image			*/

unsigned char *inbuf;		/* input buffer 			*/
unsigned char *outbuf;		/* output buffer 			*/

char msg[CMLEN],                /* informational message		*/
     refimg[CMLEN],             /* reference image file name		*/
     intps[CMLEN],              /* reference tie point selection file 	*/
     outmtp[CMLEN],             /* output merged tie point file 	*/
     ref_chip_hostname[CMLEN],	/* host name of chip of reference img	*/
     fname = '\0';		/* no output file from projection stuff	*/

int  *ref_desc,                 /* file descriptor for reference tps file */
     *out_desc,                 /* file descriptor for output tie pt file */
     fdchip,			/* file descriptor for output chip image*/
     fdin;			/* file descriptor for input image	*/

long i,				/* looping variables			*/
     nout = 1,			/* number of output files created	*/
     ptyp,                      /* type of tie point data/header record */
     nchips,			/* number of chip images created	*/
     status,			/* function return status		*/
     acc,                       /* file access flag 			*/
     std_out,                   /* file access flag 			*/
     otype,			/* output data type			*/
     one=1,			/* constant of one			*/
     nonfatal=1,		/* nonfatal error code			*/
     fatal= -1,			/* fatal error code			*/
     out_nl, out_ns,		/* size of output chip image		*/
     inwind[4],			/* window of base image to read		*/
     window[1][4],		/* window for updating ddr		*/
     options[1],		/* image open options			*/
     access,			/* file access mode			*/
     bandno[MAXBND+1],		/* band numbers of base image		*/
     nbands[1],			/* number of bands in output chip	*/
     geo_code = GEO,         	/* projection code                      */
     geo_zone = NULZONE,     	/* projection zone                      */
     geo_units = DEGREES,    	/* projection units                     */
     ddr_units,              	/* projection units from image ddr      */
     prtprm[2];              	/* print parameters                     */

float 	*resamp_tbl,		/* table of resampling weights		*/
      	ptline, ptsamp,		/* line/sample of tie point		*/
      	aline, asamp,		/* AVHRR line/sample of tie point	*/
      	ulline, ulsamp,		/* upper left line/sample		*/
	leftsamp,		/* left sample limit for selecting chips*/
	rightsamp;		/* right sample limit for selecting chips*/

double 	a[4],b[4],		/* mapping coefficients			*/
	dblone = 1.0,		/* constant 1.0 			*/
	elev,			/* elevation at the tie point		*/
	lat, lon,		/* lat/long coordinate			*/
	tll, tls, trl, trs, 	/* top left & right corner of window	*/
	brl, brs, bll, bls,	/* bottom left & right corner of window */
	geo_parms[15];          /* projection parameters                */

/*
  Initialize processing
*/
c_inlas(&vblock,&std_out);

/*
  Initialize images and buffer space
*/
out_nl = out_ns = CHIPSIZE;
getpar(&vblock,refimg,&refddr,&ddr_units,&otype,bandno,intps,outmtp,&model,
	&avhrrddr);
outbuf = (unsigned char *)malloc(out_nl * out_ns);
if (outbuf == NULL)
    {
    c_errmsg("Error allocating dynamic memory","avhrrchips-alloc",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

/*
  Set up the left & right limits for selecting chips
*/
leftsamp  = EXCLUDE * model.nsamples;
rightsamp = model.nsamples - (EXCLUDE * model.nsamples);

/*
  Initialize the GCTP
*/
for (i = 0; i < 15; i++)
    geo_parms[i] = 0.0;
prtprm[GEO_TERM] = prtprm[GEO_FILE] = FALSE;
status = c_transinit(&geo_code,&geo_units,&geo_zone,&refddr.datum_code,
    geo_parms, &(refddr.proj_code),&ddr_units,&(refddr.zone_code),
    &(refddr.datum_code),refddr.proj_coef,prtprm,&fname);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);

/*
  Open TPS file and read header records
*/
ptyp = TPS; acc = IREAD;
if (c_ptopen(&ref_desc,intps,&ptyp,&acc,&ref_hdr) != E_SUCC) 
    {
    c_errmsg("Error opening tie point selection file","avhrrchips-open",
	&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

/*
  Initialize MTP header records 
*/
for (i = 0; i < 8; i++)
    mtp_hdr.corners[i] = ref_hdr.corners[i];
mtp_hdr.pdist[0] = ref_hdr.pdist[0];
mtp_hdr.pdist[1] = ref_hdr.pdist[1];
for (i = 0; i < 15; i++) 
    mtp_hdr.projprms[i] = ref_hdr.projprms[i];
mtp_hdr.proj_valid = ref_hdr.proj_valid;
mtp_hdr.code = ref_hdr.code;
mtp_hdr.zone = ref_hdr.zone;
mtp_hdr.datum = ref_hdr.datum;
strcpy(mtp_hdr.chip_mode, "CHIP-TEM");
strcpy(mtp_hdr.units, ref_hdr.units);
strcpy(mtp_hdr.sea_name, "");
strcpy(mtp_hdr.ref_name, ref_hdr.name);

/*
  Open output MTP file
*/
ptyp = MTP; acc = IWRITE; 
if (c_ptopen(&out_desc,outmtp,&ptyp,&acc,&mtp_hdr) != E_SUCC)
    {
    c_ptclse(&ref_desc);
    c_errmsg("Error opening output merged tie point file","avhrrchips-open",
	&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

/*
  Process each control point
*/
if (warp_calc_kernel(&resamp_tbl,-1.0) != E_SUCC)
    {
    free(outbuf);
    c_ptclse(&ref_desc);
    c_ptclse(&out_desc);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

nchips = 1;
for (;;)
    {
    status = c_ptio(&ref_desc, &ref_data);
    if (status == E_EOF) break;
    if (status == E_FAIL) 
	{
	free(outbuf);
    	c_ptclse(&ref_desc);
	c_ptclse(&out_desc);
	c_errmsg("Error reading tie point file","avhrrchips-read",&nonfatal);
	c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
	}

    /*
      Calculate nominal coordinates in raw AVHRR image
    */
    lat = ref_data.coord[0] * RAD_PER_DEG;
    lon = ref_data.coord[1] * RAD_PER_DEG;
    elev = ref_data.elevation;
    c_ll2ls(&lat,&lon,&elev,&model,&aline,&asamp,NULL);

    /* adjust for difference in model & image window */
    ptline = aline - avhrrddr.master_line;
    ptsamp = asamp - avhrrddr.master_sample;

    /*
      Transfer contents of TPS file to MTP file & 
      set up parameters for correlation
    if ((ptline > BORDERWIDTH) && (ptline < model.nlines-BORDERWIDTH) &&
        (ptsamp > BORDERWIDTH) && (ptsamp < model.nsamples-BORDERWIDTH))
    */
    if ((ptline > BORDERWIDTH) && (ptline < avhrrddr.nl-BORDERWIDTH) &&
        (ptsamp > BORDERWIDTH) && (ptsamp < avhrrddr.ns-BORDERWIDTH) &&
        (asamp > leftsamp) && (asamp < rightsamp))
	{
	/*
	  Write results to mtp file
	*/
	mtp_data.geo_coord[0] = ref_data.coord[0];
	mtp_data.geo_coord[1] = ref_data.coord[1];
	mtp_data.elevation    = ref_data.elevation;
	mtp_data.zoom_factor  = ref_data.zoom_factor;
	mtp_data.nominal[0]   = ptline;
	mtp_data.nominal[1]   = ptsamp;
	mtp_data.ref_coord[0] = CHIPSIZE/2.0 + (ptline - (long)ptline);
	mtp_data.ref_coord[1] = CHIPSIZE/2.0 + (ptsamp - (long)ptsamp);
	mtp_data.chip_size[0] = CHIPSIZE;
	mtp_data.chip_size[1] = CHIPSIZE;
	mtp_data.offsets[0]   = CHIPSIZE/4;
	mtp_data.offsets[1]   = CHIPSIZE/4;
	strcpy(mtp_data.pt_id, ref_data.pt_id);
	sprintf(mtp_data.ref_chip,"chip%d;img", nchips);
	sprintf(ref_chip_hostname,"chip%d.img", nchips);
	strcpy(mtp_data.sea_chip, "");

	/*
	  Calculate chip extents in raw AVHRR image
	  and convert corners to projection coordinates
	  Use the AVHRR coords to convert to projection
	*/
	ulline = (long)aline - CHIP_OFF;
	ulsamp = (long)asamp - CHIP_OFF;

	avhrr2proj(&ulline,&ulsamp,&tll,&tls,&model,elev,&refddr,
		   &geo_code,&geo_zone,&geo_units,&ddr_units,geo_parms);
	if ((tll < 1) || (tll > refddr.nl) ||
	    (tls < 1) || (tls > refddr.ns))
	    continue;

	asamp = ulsamp + CHIPSIZE - 1;
	avhrr2proj(&ulline,&asamp,&trl,&trs,&model,elev,&refddr,
		   &geo_code,&geo_zone,&geo_units,&ddr_units,geo_parms);
	if ((trl < 1) || (trl > refddr.nl) ||
	    (trs < 1) || (trs > refddr.ns))
	    continue;

	aline = ulline + CHIPSIZE - 1;
	avhrr2proj(&aline,&asamp,&brl,&brs,&model,elev,
		   &refddr,&geo_code,&geo_zone,&geo_units,&ddr_units,geo_parms);
	if ((brl < 1) || (brl > refddr.nl) ||
	    (brs < 1) || (brs > refddr.ns))
	    continue;

	avhrr2proj(&aline,&ulsamp,&bll,&bls,&model,elev,&refddr,
		   &geo_code,&geo_zone,&geo_units,&ddr_units,geo_parms);
	if ((bll < 1) || (bll > refddr.nl) ||
	    (bls < 1) || (bls > refddr.ns))
	    continue;

	/*
	  Now that we have a chip in both ref & srch image, write to mtp
	*/
	status = c_ptio(&out_desc, &mtp_data);
	if (status == E_FAIL)
            {
	    free(outbuf);
    	    c_ptclse(&ref_desc);
	    c_ptclse(&out_desc);
            c_errmsg("Error writing merged tie point file","avhrrchips-write",
		&nonfatal);
            c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
            }

	/*
	  Calculate window of ref image
	*/
	inwind[SL] = (long)(MIN4(tll,trl,bll,brl)+0.5);
	inwind[SS] = (long)(MIN4(tls,trs,bls,brs+0.5));
	inwind[NL] = (long)(MAX4(tll,trl,bll,brl)+0.5) - inwind[SL] + 1;
	inwind[NS] = (long)(MAX4(tls,trs,bls,brs)+0.5) - inwind[SS] + 1;

	/*
	  Read this window into a buffer
	*/
	options[0] = 0;
	access = IREAD;
	otype = EBYTE;
	status = c_eopenr(&fdin,refimg,&one,&inwind[SL],&inwind[SS],
		&inwind[NL],&inwind[NS],&access,&otype,options,&inwind[NL]);
	if (status != E_SUCC) 
	    {
	    free(outbuf);
    	    c_ptclse(&ref_desc);
	    c_ptclse(&out_desc);
	    c_errmsg("Error reading reference image","avhrrchips-read",
		&nonfatal);
	    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
	    }
	inbuf = (unsigned char *)malloc(inwind[NL] * inwind[NS]);
	if (inbuf == NULL)
	    {
	    free(outbuf);
    	    c_ptclse(&ref_desc);
	    c_ptclse(&out_desc);
	    c_errmsg("Error allocating dynamic memory","avhrrchips-alloc",
		&nonfatal);
	    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
	    }
	c_eread(&fdin,&bandno[0],&one,inbuf,&inwind[NL]);

	/*
	  Calculate mapping coefficients
	*/
	calc_map_coefs(tls,trs,bls,brs,CHIPSIZE,CHIPSIZE,a);
	calc_map_coefs(tll,trl,bll,brl,CHIPSIZE,CHIPSIZE,b);

	/*
	  Warp ref image into AVHRR satellite view
	*/
	warp_tb_byte(inbuf,outbuf,CHIPSIZE,CHIPSIZE,inwind,a,b,resamp_tbl,0); 

	/*
	  Write output buffer to chip image
	*/
	otype = EBYTE;
	options[0] = options[1] = 0;
	acc = IWRITE;
	status = c_eopenr(&fdchip,ref_chip_hostname,&one,&one,&one,&out_nl,
            &out_ns,&acc,&otype,options,&out_nl);
	if (status != E_SUCC)
	    {
	    free(outbuf);
    	    c_ptclse(&ref_desc);
	    c_ptclse(&out_desc);
	    sprintf(msg,"Error creating chip %s for %s",mtp_data.ref_chip,
		mtp_data.pt_id);
	    c_errmsg(msg,"avhrrchips-create",&nonfatal);
	    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
	    }
        c_ewrite(&fdchip,&one,&one,outbuf,&out_nl);
	c_eclose(&fdin);
	c_eclose(&fdchip);
	sprintf(msg,"Successfully created chip %d",nchips);
	c_errmsg(msg," ",&nonfatal);

  	window[0][SL] = inwind[SL];
  	window[0][SS] = inwind[SS];
  	window[0][NL] = inwind[NL];
  	window[0][NS] = inwind[NS];
	nbands[0] = 1;
  	c_upddr(&vblock,refimg,ref_chip_hostname,&one,window,bandno,
		 nbands,&dblone,&dblone,&one,&one);
	free(inbuf);
	nchips++;
	}
    }

/*
  Close down processing and exit
*/
free(outbuf);
c_ptclse(&ref_desc);
c_ptclse(&out_desc);

nchips--;
sprintf(msg,"Successfully created %d chip images",nchips);
c_errmsg(msg," ",&nonfatal);

/*
  If no chips were found, don't keep an empty mtp file
*/
if (nchips == 0)
    {
    c_delfil(outmtp);
    nout = 0;
    }

c_complt("AVHRRCHIPS",outmtp,&nout);
}
