/*******************************************************************************
NAME:		GETPAR

PURPOSE:	Retreive the user specified input parameters
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "typlim.h"

FUNCTION getpar(vblock,refimg,refddr,ddr_units,odtype,bands,intps,outmtp,model,
		avhrrddr)

struct	PARBLK	*vblock;	/* TAE parameter block			     */
struct  DDR	*refddr;	/* DDR information			     */
struct  MODEL  	*model;   	/* AVHRR model                               */
struct  DDR	*avhrrddr;	/* DDR information			     */

char	refimg[CMLEN],		/* name of projected reference image	     */
	*intps,			/* input tps file name			     */
	*outmtp;		/* outpout mtp file name		     */

long	*odtype,		/* data type of ref image		     */
	*ddr_units,		/* projection units of ref image	     */
	bands[];		/* bands of ref image		       	     */
{
long	access,			/* file access mode			     */
	bandnos[MAXBND+1],	/* band number of AVHRR image		     */
	fatal = -1,		/* fatal error code			     */
	nbands,			/* number of bands in IN		     */
	nmrimg,			/* number of input image specified	     */
	nonfatal = 1,		/* non-fatal error code			     */
	one = 1,		/* constant of one			     */
	qflag[1][2],		/* flag indicating if subwindows specified   */
	same_window,		/* flag indicating if images hav same window */
	status,			/* status return code			     */
	totbnd,			/* total number of bands		     */
	window[1][4],		/* input image window specification	     */
	zero = 0;		/* constant of zero			     */

char	srchimg[CMLEN];		/* name of AVHRR search image (for model)    */
char	taein[CMLEN];		/* TAE name of ref image		     */

/*
  Get the reference image name
*/
status = c_getin(vblock,"IN",&zero,taein,refimg,&nmrimg,window,bands,&nbands,
		 &totbnd,&one,&same_window,qflag);
if (status != E_SUCC)
    {
    c_errmsg("Error retrieving base image of IN","avhrrchips-in",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

/*
  Make sure only one band was specified
*/
if (totbnd > 1)
    {
    c_errmsg("Base image must be a single-band image",
	 "avhrrchips-onebnd",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

/*
  Get the DDR information
  and validate projection parameters
*/
status = c_getddr(refimg,refddr);
*odtype = refddr->dtype;
status = c_ckunit(refddr->proj_units,ddr_units);
if ((status != E_SUCC) ||
    (refddr->valid[0] == INVAL) ||
    (refddr->valid[DDCCV] == INVAL) ||
    (refddr->valid[DDPDV] == INVAL))
    {
    c_errmsg("Invalid DDR of base image","avhrrchips-invddr",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

/*
  Get the AVHRR search image name
*/
if (c_getin(vblock,"IN",&one,taein,srchimg,&nmrimg,window,bandnos,&nbands,
		 &totbnd,&one,&same_window,qflag) != E_SUCC)
    {
    c_errmsg("Error retrieving AVHRR image of IN","avhrrchips-in",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

/*
  Retrieve the AVHRR DDR of the search image in satellite perspective
*/
status = c_getaddr(srchimg,model);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
model->sat_view = FALSE;
status = c_getddr(srchimg,avhrrddr);

/*
  Get the input tps file name
*/
access = IREAD;
if (c_getnam(vblock,"INTS",&zero,intps,"tps",&access) != E_SUCC) 
    {
    c_errmsg("Error retrieving value of INTS", "avhrrchips-ret",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

/*
  Get the output mtp file name
*/
access = IWRITE;
if (c_getnam(vblock,"OUTMT",&zero,outmtp,"mtp",&access) != E_SUCC) 
    {
    c_errmsg("Error retrieving value of OUTMT", "avhrrchips-ret",&nonfatal);
    c_errmsg("Fatal error encountered","avhrrchips-fatal",&fatal);
    }

return(E_SUCC);
}
