#!/bin/csh
#
# check_avhrr.csh
#
# This script checks the disk usage of the /avhrr disk and issues a warning
# dialog as an TCL/TK pop-up.  Warning will remain on the screen until
# acknowledged by operators.  If a previous warning is still running and the
# usage has changed, a new warning is issued (previous one destroyed),
# otherwise the old warning is left as is.  If the disk usage becomes
# critical, files more than 2 days old will be purged from $ADAPSARC/sfl.
# If disk usage drops below the warning level and a previous message is 
# running, it will be destroyed.
#
# There are no input parameters.
# The script may be run from cron if the DISPLAY variable is hard-coded
# Messages are also logged to $ADAPSLOG/check_space.log
#
# $Log: check_avhrr.csh,v $
# Revision 1.1  2003/05/05 15:16:54  dlloyd
# Initial check in of module
#
#---------------------------------------------------------------------------

if ( ! $?DISPLAY ) then
    setenv DISPLAY :0
endif

set now = `date +"%D %T"`
set ofile = "$ADAPSTABLES/space.txt"
set logfile = "$ADAPSLOG/check_space.log"

# Get the current disk usage
#---------------------------
set temp = `df /avhrr | grep avhrr | awk -F% '{print $1}'`
set current_usage = `echo $temp | awk '{print $5}'`

# If the disk usage is at or above 99% drastic measures
# are needed. The cleanup.csh script uses an mtime of +7
# to clean try using a +3 to see if we get any results.
#-------------------------------------------------------
if ( $current_usage >= 99 ) then
    echo "$now Emergency measures being used" >> $logfile
    set images = `find $ADAPSARC/sfl/ -type f -prune -name 'ah*.sf2' -mtime +3 -print`
    foreach image ( $images )
        rm -f $image
    end
    # See if the emergency procedure did anything
    #--------------------------------------------
    set temp = `df /avhrr | grep avhrr | awk -F% '{print $1}'`
    set current_usage = `echo $temp | awk '{print $5}'`
endif

# If the disk usage is at or above 98% issue the warning
#-------------------------------------------------------
if ( $current_usage >= 98 ) then
    # Check to see if a warning message is already present
    #-----------------------------------------------------
    set process = `ps -ej | grep space.tcl`
    set procid = `echo $process | awk '{print $1}'`
    if ( $procid > 0 ) then
        # Since a warning is present see if the
	# usage has changed since its issuance
	#--------------------------------------
	if ( -e $ofile ) then
	   set temp = `cat $ofile | awk -F% '{print $1}'`
	   set old_usage = `echo $temp | awk '{print $7}'`
	   # if the usage is the same exit
	   #--------------------------------
	   if ( $current_usage == $old_usage ) then
	       echo "$now Same message, quitting" >> $logfile
	       exit 0
	   endif
	endif
	# Otherwise kill the old process and continue
	#--------------------------------------------
	echo "killing $procid"
        kill -9 $procid
    endif

    # Write to the log file
    #----------------------
    echo "$now /avhrr disk is at $current_usage% usage" >> $logfile
    # Create the error mesage file and pop it up
    #-------------------------------------------
    echo "\n\n\n\n\t$now /avhrr disk is at $current_usage% usage" >> $ofile
    echo "\n\n\tDisk will be automatically cleaned up" >> $ofile
    echo "\tif it reaches 99% capacity" >> $ofile
    $LASTOOLS/space.tcl -geom 60x10+260+290

    # Finally remove the file once the pop-up is done
    #------------------------------------------------
    rm -f $ofile
else
    # See if there is a previous warning and, if so,
    # remove it since we are below the warning threshold
    #---------------------------------------------------
    set process = `ps -ej | grep space.tcl`
    set procid = `echo $process | awk '{print $1}'`
    if ( $procid > 0 ) then
	echo "$now Removing old warning. Usage $current_usage%" >> $logfile
        kill -9 $procid
    endif
    # And touch the logfile, just in case we want the
    # Adapsmonitor to ensure this thing runs regularly
    #-------------------------------------------------
    touch $logfile
endif

exit 0
