/*

$Log: check_conf.c,v $
Revision 1.3  2003/09/29 12:25:59  dlloyd
Corrected error in second range of times.

Revision 1.2  2003/04/25 18:45:41  dlloyd
Included string.h

Revision 1.1  2003/04/25 18:15:45  dlloyd
Initial checkin of module

*************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <string.h>

#define HEADERLINES  10


int main( int argc, char *argv[] )
{
FILE *inf;		/* input file descriptor: conflict report	*/
FILE *ouf;		/* output file descriptor: temp TCL/TK interface*/
char buffer[512];	/* input buffer for reading past header lines	*/
char conf_sceneid[17];	/* conflicting scene identifier			*/
char conf_startdate[10];/* conflicting scene start date			*/
char conf_starttime[10];/* conflicting scene start time			*/
char conf_conflict[30];	/* conflicting scene 'Conflict' flag		*/
char sceneid[17];	/* scheduled scene identifier			*/
char startdate[10];	/* scheduled scene start date			*/
char starttime[10];	/* scheduled scene start time			*/
char scheduled[30];	/* scheduled scene 'Scheduled' flag		*/
char satid;		/* ACU satellite ID pulled frome sceneid	*/

int  line;		/* loop counter					*/
int  daylight_savings;	/* daylight savings time flag pulled from tm	*/
int  time_diff;
int  status;

struct tm tm_conflict;
struct tm *tm_local;
time_t now;
time_t conflict_time;

/* get the current time
-----------------------*/
now = time(0);
tm_local = localtime( &now );
daylight_savings = tm_local->tm_isdst;

/* check for command line arguments
-----------------------------------*/
if ( argc != 3 )
    {
    printf("Usage: %s <input file> <output file>\n", argv[0]);
    return -1;
    }

/* open the input file
----------------------*/
inf = fopen( argv[1], "r" );
if ( !inf )
    {
    printf("Unable to open %s\n", argv[1]);
    return -1;
    }

/* read past the header stuff in the input
------------------------------------------*/
for ( line = 0; line < HEADERLINES; line++ )
    if ( fgets( buffer, 120, inf ) == NULL )
	{
        printf("Error reading from input\n");
	fclose( inf );
	return -1;
	}

while ( fscanf( inf, "%s %s %s %*s %*s %*s %s\n", conf_sceneid, conf_startdate,
                conf_starttime, conf_conflict ) == 4 )
    {
    if ( strcmp( conf_conflict, "Conflict" ) == 0 )
        /* we have a conflict with the next scene */
	{
	/* Get the next line */
	fscanf( inf, "%s %s %s %*s %*s %*s %s\n", sceneid, startdate,
			starttime, scheduled );
	/* get the start time */
	sscanf( startdate, "%d/%d/%d", &tm_conflict.tm_mon,
	                               &tm_conflict.tm_mday,
				       &tm_conflict.tm_year );
	sscanf( starttime, "%d:%d:%d", &tm_conflict.tm_hour,
	                               &tm_conflict.tm_min,
				       &tm_conflict.tm_sec );
	/* make the month 0-relative */
	tm_conflict.tm_mon--;
	/* convert the year to years since 1900 */
	tm_conflict.tm_year %= 100;
	tm_conflict.tm_year += 100;
	/* zero out the remaining elements */
	tm_conflict.tm_wday = tm_conflict.tm_yday = 0;
	tm_conflict.tm_isdst = daylight_savings;
	/* convert the start time to seconds */
	conflict_time = mktime( &tm_conflict );

	/* Checke the conflict time against the current time.
	If we are within MININTERVAL seconds, give the alert
	by writing to the  output file.
	----------------------------------------------------*/
	time_diff = conflict_time - now;
	if (( time_diff <= 1180 ) && ( time_diff >= -120 ))
	    {
	    printf("time difference before: %d\n",time_diff);
	    if ( time_diff < 1180 && time_diff >= 480 )
	        {
		sleep ( time_diff - 480 );
	        status = 5;
		}
	    else if ( time_diff < 480 && time_diff >= 180 )
	        {
		sleep ( time_diff - 180 );
	        status = 3;
		}
	    else if ( time_diff < 180 && time_diff >= 60 )
	        {
		sleep ( time_diff - 60 );
	        status = 1;
		}
	    else
		{
		sleep( 60 );
		status = 1;
		}

	    now = time(0);
	    time_diff = conflict_time - now;
	    printf("time difference after: %d\n",time_diff);
	    ouf = fopen( argv[2], "w" );
	    if ( !ouf )
	        {
		printf("Unable to open %s", argv[2]);
		}
	    else
		{
		int min;
		min = (int)(time_diff / 60.0 + 0.5);
		if ( min >= 2 )
		    fprintf(ouf,"\n    Conflict within %d minutes\n",
			(int)( time_diff / 60.0 + 0.5));
		else if ( min == 1 )
		    fprintf(ouf,"\n    Conflict within %d minute\n",
			(int)( time_diff / 60.0 + 0.5));
		else
		    fprintf(ouf,"\n    Conflict now!\n");
		satid = conf_sceneid[3];
		fprintf(ouf,"\n    Conflicting satellite: N%c\n", satid);
	        fprintf(ouf,"\t%s %s %s %s\n", conf_sceneid, conf_startdate,
		    conf_starttime, conf_conflict);
		satid = sceneid[3];
		fprintf(ouf,"\n    Make sure ACU is tracking: N%c\n", satid);
	        fprintf(ouf,"\t%s %s %s %s\n", sceneid, startdate, starttime,
		    scheduled);
		fclose( ouf );
		}
	    fclose( inf );
	    return status;
	    }
	}

    /* grab the next 1 line
    for ( line = 0; line < 1; line++ )
	{
	if ( fgets( buffer, 120, inf ) == NULL )
	    {
	    fclose( inf );
	    return 0;
	    }
	}
    */
    }

fclose( inf );
return 0;
}
