/*****************************************************************************
FUNCTION:	ckarch

PURPOSE:	To compare the number of lines in an archive file with the
		number of lines computed by time stamps of first and last
		lines.

Version  Date	Programmer		Modifications
-------  -----  ----------------------  -------------------------------------
  1.0    09/97  D.Lloyd                 Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "avhrr.h"
#include "ckarch.h"


void main( void )
{
FILE *inf;			    /* input file descriptor             */
FILE *ouf;			    /* output file descriptor            */

char  infile[CMLEN];		    /* input file name                   */
char  logfile[CMLEN];               /* log file name                     */
char  sceneid[CMLEN];		    /* scene identification		 */
char  text[CMLEN];		    /* text string for wrtlog            */

unsigned char  buffer[REC_SIZE];    /* input buffer                      */
unsigned char *buf = buffer;	    /* pointer to input buffer           */

long  btime;			    /* beginning line time stamp         */
long  calc_lines;		    /* lines calculated from file size   */
long  elapsed;			    /* elapsed time in ms                */
long  etime;			    /* ending line time stamp            */
long  fatal = -1;		    /* fatal error flag                  */
long  file_size;		    /* file size in bytes                */
long *lword;			    /* pointer to a long                 */
long  std_out;			    /* standard output                   */
long  time_line;		    /* lines calculated from time stamps */

struct PARBLK vblock;		    /* Variable block                    */


/*
  Initialize LAS
*/
c_inlas( &vblock, &std_out );


/*
  Get the input parameters
*/
getpar( &vblock, infile, logfile, sceneid );


/*
  Open the input file
*/
if ((inf = fopen( infile, "r" )) == NULL )
    c_errmsg("Error opening input file","ckarch-file",&fatal);


/*
  Read the first record from the file and extract the timestamp
*/
fread( buffer, 1L, REC_SIZE, inf );
lword = (long *)(buf + 12);
btime = *lword & 0x7ffffff;


/*
  Seek to the last record, read it and extract its timestamp
*/
fseek( inf, -REC_SIZE, SEEK_END );
fread( buffer, 1L, REC_SIZE, inf );
etime = *lword & 0x7ffffff;


/*
  Adjust time if we go over a day
*/
if ( etime < btime )
    etime += 86400000;
elapsed = etime - btime;


/*
  Calculate the number of lines based on timestamps -- add one
  for the last line and .5 to round
*/
time_line = (float)elapsed / (MIL_PER_SEC / LAC_PER_SEC) + 1.5;


/*
  Determine the file size and calculate the actual number of
  lines in the file.
*/
fseek( inf, 0L, SEEK_END );
file_size = ftell( inf );
calc_lines = file_size / REC_SIZE;


/*
  If the line counts don't jive, print results to log file
*/
sprintf( text, "Lines expected: %d  Actual: %d", time_line, calc_lines );
if ( calc_lines != time_line )
    c_wrtlog( logfile, sceneid, "FAILURE", text );
else
    c_wrtlog( logfile, sceneid, "SUCCESS", text );

/*
  Close input file
*/
fclose( inf );

}  /* END main */  
