      SUBROUTINE ALBTMP (IREC,IDATA,ISCAN,ATVALU,DD,COUNT1,TBB,GOTTBB)
C*******************************************************************
C
C     NAME: ALBTMP
C
C     FUNCTION: COMPUTES ALBEDOS (AVHRR CH 1 AND 2) AND TEMPERATURES
C      (CH 3, 4, 5) FOR EACH PIXEL ALONG THE SCANLINE.
C
C
C     INPUT PARAMETERS:
C        IDATA   -   3220 BYTE RECORD OF GAC DATA AS READ FROM
C                            A GAC FULL DATASET COPY TAPE.
C
C     OUTPUT PARAMETERS:
C        ATVALU  -   ARRAY OF 6,MAXLPX CONTAINING CHANNELS 1,2,3
C                    ALBEDOS AND CHANNELS 3,4,5 TEMPERATURES
C
C     CALLED FROM:
C        AVHFDC
C
C     SUBROUTINES CALLED: TMS3, DUMP1B
C
C
C    COUNTS, INCLUDE JCLCOM, and call to DUMP1B added by D. Love, 9/6/95
C    BBLOOP replaced by ISCAN from AVHFDC for DUMP1B
C
C***********************************************************************
C
C
C...INCLUDE CONSTANT PARAMETERS.
C
      INCLUDE "inc/inclav.f"
C
C
C...INCLUDE COMMON AREA PARAMETERS.
C
      INCLUDE "inc/angle.f"
      INCLUDE "inc/solprm.f"
      INCLUDE "inc/typdat.f"
C
      INCLUDE "inc/spcnst.f"
      INCLUDE "inc/newcon.f"
      INCLUDE "inc/ohed1c.f"
      INCLUDE "inc/baddat.f"
      INCLUDE "inc/jclcom.f"
C
C...DECLARE ARGUMENTS.
C
      REAL*4  ATVALU(MAXLPX,6),
     1        PIXDAT(5), RAD(5), 
     2        NU(3,5), DSCAL(5), DOFFST(5), 
     3        DELTA, ETA,
     5        DRSCAL(5), DROFFST(5)
C
      INTEGER*4  IREC,  I4CAL(10), ICAL(2,5), 
     1           PRT(0:4), 
     2           IDATA(L1BWDS), COUNTS(5),
     3           DD, COUNT1(MAXLPX)
C
      LOGICAL*4  FIRST, 
     1           LOOP, GOTTBB
C                                                                ALBTMP
C...DECLARE HEX CONSTANTS.
C
      INTEGER IZ3FF/ $3FF/
      INTEGER IQMASK/$DDD80000/
C
      EQUIVALENCE ( I4CAL(1), ICAL(1,1) )
C
C
C...VARIABLE INITIALIZATIONS.
C
      DATA LOOP / .FALSE. /, FIRST / .TRUE. /
      DATA ETA / 1.13451578 /, DELTA / 0.271568628 /, PI / 3.1415926 /
      DATA PRT / 5*0.0 /
C
C----------------------------------------------------------------------
C
C...SET UP CALIBRATION ARRAYS
C
      DO 150 I = 1,10
         I4CAL(I) = IDATA((I-1)+4)
  150 CONTINUE
C
C
C...STORE CAL PARMS FOR FIRST EXECUTION OF THIS ROUTINE.
C
      IF (FIRST) THEN
         DO 200 ICHAN = 1,5
            DSCAL(ICHAN) = ICAL(1,ICHAN)/1073741824.0
            DOFFST(ICHAN) = ICAL(2,ICHAN)/4194304.0
            ICHN = ICHAN
C
CCCC        IF (ISC.EQ.2.AND.ICHAN.EQ.5) ICHN = 4
C
            NU(1,ICHAN) = 0.0
            NU(2,ICHAN) = 0.0
            NU(3,ICHAN) = 0.0
            IF (ICHAN .LT. 3) GO TO 200
            NU(1,ICHAN) = WAVENO(1,ICHN-2)
            NU(2,ICHAN) = WAVENO(2,ICHN-2)
            NU(3,ICHAN) = WAVENO(3,ICHN-2)
  200    CONTINUE
         FIRST = .FALSE.
      ELSE
         DO 201 ICHAN = 3,5
            DSCAL(ICHAN) = ICAL(1,ICHAN)/1073741827.
            DOFFST(ICHAN) = ICAL(2,ICHAN)/4194304.
  201    CONTINUE
C
         DO 203 ICHAN = 1,2
            S = X(ICHAN) * EXP(Y(ICHAN)*(DD-Z))
            DSCAL(ICHAN) = S
            DOFFST(ICHAN) = S*O(ICHAN)*(-1.0)
  203    CONTINUE
      ENDIF
C                                                                ALBTMP
C...COMPUTE ALBEDOS/TEMPERATURES FOR EACH PIXEL AND CHANNEL.
C   SKIP CALCULATION OF ALBEDOS FOR CHANNELS 1-2 WHEN THE
C   SOLAR ZENITH ANGLE IS GREATER THAN "ALBLIM" (89.5 DEG).
C
      DO 500 IPIX = 1, NPIXLS
C
C
C... UNPACK RAW COUNTS FOR THE 5 CHANNELS AND CALCULATE ALBEDOS/TEMPS.
C
         DO 400 ICHAN = 1, 5
C
C
C........ IF SOLAR ZENITH ANGLE IS GREATER THAN 89.5 (WELL OVER
C         THE 84.3 LIMIT FOR DAY/NIGHT) SKIP ALBEDO CALCULATIONS
C         FOR CHANNELS 1 AND 2.  (CHANNEL 3 ALBEDO IS HANDLED BY
C         THE TMS3 SUBROUTINE.)
C
C
          IVX = (IPIX - 1)*5 + ICHAN
          IVWORD = ((IVX - 1)/3)+ 1
          IVSAMP = MOD(IVX - 1,3) + 1
C
          IDWORD = ISHFT(ISHFT(IDATA((IVWORD + 113)-1),2),-2)
C
          IVIDEO = 0
          IF (IVSAMP.EQ.1) IVIDEO = IAND(ISHFT(IDWORD,-20), IZ3FF)
          IF (IVSAMP.EQ.2) IVIDEO = IAND(ISHFT(IDWORD,-10), IZ3FF)
          IF (IVSAMP.EQ.3) IVIDEO = IAND(IDWORD, IZ3FF)
C
C --- STORE CH1 COUNT FOR CLOUD ALGORITHM
          IF (ICHAN .EQ. 1) THEN
             COUNT1(IPIX) = IVIDEO
          END IF
          COUNTS(ICHAN) = IVIDEO
C
C ---     IF SOLAR ZENITH ANGLE IS GREATER THAN 84.3, STORE THE
C         CHANNELS 1 AND 2 COUNTS, INSTEAD OF ALBEDO.
C
           IF ((SOLZ(IPIX) .GT. 84.3) .AND. (ICHAN .LT. 3)) THEN
             PIXDAT(ICHAN) = IVIDEO
             GO TO 410
          END IF
C
          PIXDAT(ICHAN) = FLOAT ( IVIDEO )
C                                                                ALBTMP
Cpd
C    Here is the new method of calculating temperatures from P. Davis:
C
          IF (ICHAN .LE. 3) THEN
             PIXDAT(ICHAN) = PIXDAT(ICHAN)*DSCAL(ICHAN) + DOFFST(ICHAN)
C                                        ! /* COMPUTE ALBEDO */
             IF (PIXDAT(ICHAN) .GE. 0.0) THEN
                IF (ICHAN .LT. 3) 
     *             PIXDAT(ICHAN) = (PIXDAT(ICHAN)/COSSZA(IPIX))*SEDSQ
C
                IF (ICHAN .EQ. 3) THEN
                   RAD(ICHAN) = QA(ICHAN) * PIXDAT(ICHAN)
     *                        + QB(ICHAN) * PIXDAT(ICHAN) 
     &				* PIXDAT(ICHAN)
     *                        + QC(ICHAN)
c2345678901234567890123456789012345678901234567890123456789012345678901234
                   PIXDAT(ICHAN) = ((C2*CNU(ICHAN)/TSL(ICHAN)) /
     *                      ALOG(1.0 +  C1 * CNU(ICHAN)**3 / 
     &				RAD(ICHAN)))
     *                    - TIN(ICHAN) / TSL(ICHAN)
                END IF
             ELSE
                PIXDAT(ICHAN) = 0.0
             END IF
          ELSE
            IF (GOTTBB) THEN
C
C   If TBB values available, do Chs 4 and 5
C
             DRSCAL(ICHAN)  = DSCAL(ICHAN)
             DROFFST(ICHAN) = DOFFST(ICHAN)
C            IF (NSAT .NE. 4) THEN
                RICT = C1 * CNU(ICHAN)**3 /
     *                 (EXP((C2*CNU(ICHAN)) /
     *                 (TIN(ICHAN) + TBB * TSL(ICHAN))) - 1)
C
C   Check for non-zero space radiance prime:
C
                SRPUSE = 0.0
                IF (NZSRPS .GT. 0) THEN
                   DO ISRP = 1, NZSRPS
                      IF (OSDATE .GE. IYRDYS(ISRP) .AND.
     *                    OSDATE .LE. IYRDYE(ISRP)) THEN
                         SRPUSE = SRP(ISRP, ICHAN)
                      END IF
                   END DO
                END IF
C
                DRSCAL(ICHAN)  = DSCAL(ICHAN) * ((RICT - RSP(ICHAN)) /
     *                    (RICT - SRPUSE))
                DROFFST(ICHAN) = RSP(ICHAN) + ((RICT - RSP(ICHAN)) /
     *                    (RICT - SRPUSE)) * (DOFFST(ICHAN) - SRPUSE)
C            END IF
C
             PIXDAT(ICHAN) = PIXDAT(ICHAN) * DRSCAL(ICHAN) + 
     &		DROFFST(ICHAN)
c2345678901234567890123456789012345678901234567890123456789012345678901234
C
             IF (PIXDAT(ICHAN) .GT. 0.0) THEN
C                                           ! /* COMPUTE RADIANCE */
                RAD(ICHAN) = QA(ICHAN) * PIXDAT(ICHAN)
     *                     + QB(ICHAN) * PIXDAT(ICHAN) * PIXDAT(ICHAN)
     *                     + QC(ICHAN)
C                                           ! /* COMPUTE TEMPERATURE */
                PIXDAT(ICHAN) = ((C2*CNU(ICHAN)/TSL(ICHAN)) /
     *                   ALOG(1.0 +  C1 * CNU(ICHAN)**3 / RAD(ICHAN)))
     *                 - TIN(ICHAN) / TSL(ICHAN)
             ELSE
                PIXDAT(ICHAN) = 0.0
             END IF
            ELSE
C
C   If no TBB values, skip doing Chs 4 and 5
C
              PIXDAT(ICHAN) = 0.0
            END IF
          END IF
C
  410  CONTINUE
C                                                                ALBTMP
C... STORE ALBEDOS AND CORRECTED TEMPERATURES IN THE ATVALU ARRAY.
C
            IF (ICHAN .LT. 3) THEN
C                                               ! /* CHANNEL 1 OR 2 */
              ATVALU (IPIX,ICHAN) = PIXDAT(ICHAN)
C
            ELSE
C                                               ! /* CHANNELS 3 - 5 */
              ATVALU (IPIX, (ICHAN+1)) = PIXDAT(ICHAN)
C
            ENDIF
C
C
  400    CONTINUE
C                                   !  /* END OF CHANNEL LOOP */
C
C        Call DUMP1B to dump counts, scale and offset values
C
         IF (DUMPRN .AND. ISCAN .GE. IDBEG .AND. ISCAN .LE. IDEND) THEN
            IF (IPIX .EQ. 1) THEN
               WRITE (6, 601) ISCAN, C1, C2, TBB, RICT, 1, 2, 3, 4, 5, 
     *                        ((ICAL(I,J), J=1,5), I=1,2),
     *                        (DSCAL(J), J=1,5), (DOFFST(J), J=1,5),
     *                        (DRSCAL(J), J=1,5), (DROFFST(J), J=1,5),
     *                        (1, 2, 3, 4, 5, J = 1, 3)
 601          FORMAT (20X, '1B dump for scanline ', I6, /,
     *                'C1 = ', E12.4, ', C2 = ', F8.6, 
     *                ', TBB = ', F6.2, ', RICT = ', F8.4, /,  
     *                5('     Channel ', I2, ':'), /,
     *                ' Calibration Slope:  ', 5I16, /,
     *                ' Calibration Offset: ', 5I16, /,
     *                ' Original Slopes:    ', 5F16.4, /,
     *                ' Original Offsets:   ', 5F16.4, /,
     *                ' Recalculated Slopes:  ', 5F16.4, /,
     *                ' Recalculated Offsets: ', 5F16.4, //,
     *                ' Pix: Counts:', 17X, 'RAD:', 35X, 'PIXDAT:', /,
     *                '      ', 5('Ch.', I1, ':'), 
     &			10('  Ch.', I2, ':'), /)  
c2345678901234567890123456789012345678901234567890123456789012345678901234
            END IF
C            CALL DUMP1B(IPIX, COUNTS, RAD, PIXDAT)
            WRITE (6,660) IPIX, COUNTS, RAD, PIXDAT
  660       FORMAT (I6, 5I5, 10F8.2)
         END IF
C
  500 CONTINUE
C                               !  /* END OF PIXEL LOOP */
C
C... COMPUTE CHANNEL 3 ALBEDO FOR EACH PIXEL.
C
      CALL TMS3 (ATVALU, SEDSQ, IREC)
C
      RETURN
      END
