      SUBROUTINE AVHFDC (GACREC, LACREC, ATVALU, ISCAN, IREC,
     1                   PRTST, FLAG, FLAG1, DDAYS, COUNT1)
C*******************************************************************
C
C     NAME: AVHFDC
C
C     FUNCTION: THIS ROUTINE WILL COMPUTE THE VIEWING GEOMETRY AND
C      AND EARTH LOCATION FOR EACH PIXEL. FOR EACH PIXEL, THE
C      CHANNEL 1 AND 2 ALBEDOS AND CHANNELS 3,4,5 TEMPERATURES
C      ARE CALCULATED. ALSO FOR EACH PIXEL, THE CHANNEL 3 ALBEDO
C      IS CALCULATED. DATA VALUES ARE SCALED TO INTEGERS AND
C      RETURNED THROUGH THE CALLING SEQUENCE.
C
C      VERSION 1.1 - LIMIT FOR COS/SOLAR ZEN ANGLE TEST FOR
C                    DAY/NIGHT CHANGED TO 0.10.
C
C      VERSION 1.2 -  02/21/92 - D. CARR (SMSRC)
C                    OUT-OF-BOUNDS CHECK INSTALLED FOR ALBEDOS.
C                    IF VALUE IS < 0 OR > 1023 (MAX FOR 10 BITS)
C                    IT WILL BE RESET TO 0.
C
C      VERSION 2.0 -  9/09/95 - D. LOVE (SMART)
C                    BBLOOP REPLACED BY ISCAN IN ALBTMP CALL
C
C
C     INPUT PARAMETERS:
C        IDATA   -   3220 BYTE RECORD OF GAC DATA AS READ FROM
C                            A GAC FULL DATASET COPY TAPE.
C        SED     -   SUN-EARTH DISTANCE
C
C     OUTPUT PARAMETERS:
C        ISCAN   -   SCAN NUMBER
C        IYYDDD  -   YEAR AND DAY OF YEAR FOR THIS SCAN.
C        STIME   -   TENS OF SECONDS IN DAY OF THIS SCAN.
C        COSSZA  -   ARRAY OF MAXLPX COSINE OF SOLAR ZENITH ANGLES.
C        SPLAT   -   ARRAY OF MAXLPX LATITUDES (DEGREES).
C        SPLON   -   ARRAY OF MAXLPX LONGITUDES ( DEGREES, +E ).
C        ATVALU  -   ARRAY OF 6,MAXLPX CONTAINING CHANNELS 1,2,3
C                    ALBEDOS AND CHANNELS 3,4,5 TEMPERATURES
C
C        DAYNIT  -   DAY/NIGHT FLAG
C                      = 0 DAY
C                      = 1 NIGHT
C
C     CALLED FROM:
C        MAIN
C
C     SUBROUTINES CALLED:
C        TSNTIM   -   SYSTEM ROUTINE TO UNPACK TIME CODE
C        SOLAR    -   COMPUTES SOLAR ZENITH ANGLE (MAXLPX)
C        ALBTMP   -   COMPUTES ALBEDOES AND TEMPERATURES
C                     FOR EACH PIXEL AND CHANNEL
C
C
C***********************************************************************
C
C
C...INCLUDE CONSTANT PARAMETERS.
C
      INCLUDE "inc/inclav.f"
C
C
C...INCLUDE COMMON AREA PARAMETERS.
C
      INCLUDE "inc/angle.f"
      INCLUDE "inc/geog.f"
      INCLUDE "inc/solprm.f"
      INCLUDE "inc/typdat.f"
      INCLUDE "inc/atmisc.f"
      INCLUDE "inc/qcflgs.f"
      INCLUDE "inc/spcnst.f"
      INCLUDE "inc/newcon.f"
C
C
C...DECLARE ARGUMENTS.
C
      REAL*4  ATVALU(MAXLPX,6),
     1        DLAT(MAXANC), DLON(MAXANC),
     4        ETA, 
     5        DELTA, 
     6        T(4), XBAR(4) 
C
      INTEGER*2  I2WORD(4),I2LOC(2,MAXANC)
C
      INTEGER*4  IREC,  I4ZEN(13),
     1           I4WORD(2), WORD2,WORD19, PRT(0:4), ICNT(0:4),
     2           IDATA(L1BWDS), GACREC(G1BWDS), LACREC(L1BWDS),
     3           DDAYS,COUNT1(MAXLPX)
C
      LOGICAL*4  FIRST, FLAG, FLAG1,
     1           LOOP, PRTST, GOTTBB
C
      CHARACTER*1 L1WORD(8), L1ZEN(52)
C
C...DECLARE HEX CONSTANTS.
C
      INTEGER IZ3FF/ $3FF/
      INTEGER IQMASK/$DDD80000/
C
C
      EQUIVALENCE ( I4WORD(1), I2WORD(1), L1WORD(1) )
      EQUIVALENCE ( I4ZEN(1), L1ZEN(1) ), ( I4LOC(1), I2LOC(1,1) )
C
C
C...VARIABLE INITIALIZATIONS.
C
      DATA LOOP / .FALSE. /, FIRST / .TRUE. /, GOTTBB / .FALSE. /
      DATA ETA / 1.13451578 /, DELTA / 0.271568628 /, PI / 3.1415926 /
      DATA PRT / 5 * 0.0 /
C
C----------------------------------------------------------------------
C
C
C...FILL GENERIC DATA BUFFER.
C
      IF (GAC) THEN
C
        DO 2 II = 1, G1BWDS
          IDATA(II) = GACREC(II)
    2   CONTINUE
C
      ELSE
C
        DO 3 II = 1, L1BWDS
          IDATA(II) = LACREC(II)
    3   CONTINUE
C
      END IF
C
C
C...PICK OFF SCAN NUMBER.
C
      I4WORD(1) = IDATA(1)
      ISCAN     = I2WORD(1)
C
C
C...EXTRACT 1B QUALITY INDICATORS, ZEROING OUT THE LAST 3 FLAGS.
C
      ITEMP = IDATA(3)
      ITEMP = ISHFT (ISHFT (ITEMP,-19), 3)
      IQUAL(IREC) = ITEMP
C
C
C...CALL "AVHQC" TO TALLY THE NUMBER OF QUALITY INDICATOR FAILURES
C   AND SET THE "POOR SCAN QUALITY" FLAG IF ANY FAILURES ARE FOUND
C   FOR THIS SCANLINE.
C
      CALL AVHQC (IREC)
C
C
C
C...GET TIME
C
      I4WORD(1) = IDATA(1)
      I4WORD(2) = IDATA(2)
      I4 = I2WORD(2)
      IYY = ISHFT(I4,16)
      IYY = ISHFT(IYY,-25)
      IDD = ISHFT(I4,23)
      IDD = ISHFT(IDD,-23)
      I4 = IDATA(2)
      MILLI = ISHFT(I4, 5)
      MILLI = ISHFT(MILLI,-5)
      STIME = FLOAT(MILLI)/100.
C
C...GET TELEMETRY DATA
C...DECODE 19, 20 & 21ST 10 BIT WORD
C
      WORD2 = IDATA(84)
      WORD19 = ISHFT(WORD2,-20)
C
C
C...ACCUMULATE INTERNAL BB TEMPS
C
      IF (WORD19 .LT. 10) THEN
C
        N = 1
C                                       !  /* NEW 5-VALUE PRT GROUP */
C
        IF ( .NOT. PRTST) THEN
C         WRITE (6,*) '  *** BEGINNING PRT-SYNC SCAN = ', ISCAN
          IPRTSC = ISCAN
          PRTST = .TRUE.
          PRT(0) = WORD19
          KNT = 1
        END IF
C
      ELSE IF (.NOT. PRTST) THEN
C                                       !  /* IF PRT <= 10 AND PRT-SYNC
C
C                                       !     SCAN HAS NOT BEEN REACHED,
        FLAGQ = .TRUE.
C                                       !     DISCARD THIS SCANLINE */
        GO TO 15
C
C
      ELSE
C                                       !  /* ELSE SYNC FOUND, START
C                                       !     AVERAGING TEMPERATURES */
         ISTEMP = ((KNT*50) + IPRTSC + 1)
C
C
CCC   MODIFICATION MADE TO FIX THE AVHFDC DIVIDE BY ZERO ERROR.
C     THE PROBLEM OCCURRED WHEN THERE WAS A LARGE DATA GAP IN THE 1B
C     GAC (DATA GAPS WERE PREVIOUSELY UNACCOUNTED FOR IN THIS PART
C     OF THE CODE).
C
         DO 6 I = 1,4
            IF ( ICNT(I) .EQ. 0 ) GO TO 14
    6    CONTINUE
C
C
         IF (ISCAN .GE. ISTEMP) THEN
            KNT = KNT + 1
C
            DO 11 I = 1, 4
               T(I) = 0.0
               XBAR(I) = PRT(I)/ICNT(I)
               PRT(I) = 0.0
               ICNT(I) = 0
C
               DO 10 J = 0,4
                  IF (ACOEF(J,I).EQ.0.0 .OR. XBAR(I).EQ.0.0) THEN
                     GO TO 10
                  ELSE
                     T(I) = T(I) + ACOEF(J,I)*XBAR(I)**J
                  ENDIF
   10          CONTINUE
C
   11       CONTINUE
C
            TBB = B1*T(1) + B2*T(2) + B3*T(3) + B4*T(4)
            IBBT = NINT(TBB) - 273
C
            IF (IBBT.LT.10) THEN
               IBBT = 10
            ELSE IF (IBBT.GT.20) THEN
               IBBT = 20
            ENDIF
C
C   Set TBB FLAG to show that TBB values are available for ALBTMP
C
            GOTTBB = .TRUE.
C
         END IF
C
   14    CONTINUE
C
         PRT(N) = PRT(N) + WORD19
         ICNT(N) = ICNT(N) + 1
         IF (N .LE. 3) N = N + 1
C
      END IF
C
C
   15 CONTINUE
C
C
C...GET NUMBER OF ZENITH AND EARTH LOCATION ANGLES.
C...IF LESS THAN 51, WRITE MESSAGE;  IF NONE, QUIT.
C
      I4WORD(1) = IDATA( 14 )
      NANGS     = ICHAR(L1WORD(1))
      IF (NANGS .NE. MAXANC) THEN
        CALL EMESG (6, 'AVHFDC', 'OK')
        FLAGQ = .TRUE.
      END IF
C
C
      DO 40 I = 1, MAXANC
C
C...EXTRACT LATITUDE, LONGITUDE, AND SOLAR ZENITH ANGLE INFORMATION
C   FROM DATA BUFFER.
C
         I4LOC(I) = IDATA((I-1) + 27)
C
         DLAT(I) = I2LOC(1,I)/128.0
C                                           !  /* CONVERT TO */
         DLON(I) = I2LOC(2,I)/128.0
C                                           !  /* DEGREES    */
C
   40 CONTINUE
C
C
C...IF THIS IS THE FIRST SCANLINE IN THE PAIR, COMPUTE THE
C   SOLAR ZENITH ANGLE FOR EACH SCAN POINT.
C
C
C WCOLLINS 1/19/99 CHANGE
C
         IF (FLAG1 .AND. NANGS .EQ. MAXANC) THEN
            CALL SOLARR (DLAT, DLON, NANGS, STIME)
         ENDIF
C
C
C...INTERPOLATE BETWEEN THE LATITUDE AND LONGITUDE VALUES AT THE
C   51 ANCHOR POINTS IN ORDER TO DETERMINE EARTH LOCATION FOR
C   EACH POINT ALONG THE SCANLINE.
C
         IF (NANGS .EQ. MAXANC) THEN
            CALL INTERP (DLAT, DLON, NANGS)
         ENDIF
C
C
C...IF THIS IS THE FIRST SCAN IN A PAIR, COMPUTE THE SATELLITE
C   ZENITH AND RELATIVE AZIMUTH ANGLES.  THESE ANGLES USE THE
C   SOLAR ZENITH ANGLE VALUES AND, THEREFORE, MUST BE
C   COMPUTED AFTER THE SOLAR ZENITH ANGLES.
C
C CHANGE BY WCOLLINS
C
      IF (FLAG1 .AND. NANGS .EQ. MAXANC) THEN
        CALL SANGLE(DLAT, DLON, NANGS)
        FLAG1 = .FALSE.
      ENDIF
C
C
C...IF THE SCAN QUALITY FLAG IS TURNED "ON" (INDICATING POOR QUAL.),
C   SKIP THE ASSIGNMENT OF LAND/SEA, DAY/NIGHT, ASC/DESC FLAGS
C   AND THE CALCULATION OF ALBEDOS/TEMPERATURES FOR THIS SCANLINE.
C
      IF (FLAGQ) GO TO 502
C
C
C...DETERMINE MISCELLANEOUS DATA ATTRIBUTES.
C
      CALL ATTRIB (IREC)
C
C
C...COMPUTE ALBEDOS/TEMPERATURES FOR EACH PIXEL AND CHANNEL.
C
      CALL ALBTMP (IREC,IDATA,ISCAN,ATVALU,DDAYS,COUNT1,TBB,GOTTBB)
C
C
  502 CONTINUE
C
C
C.....FORMAT CARDS
C
 1000 FORMAT(1H0,'AVHFDC: NUMBER OF LOCATION ANGLES = ',I3,
     1 '  SCAN NUMBER = ',I6)
C
  999 RETURN
      END

