      SUBROUTINE CHKRNG (CH,ANGLES,LAT,LON,ISCAN,GOTTBB)
C***********************************************************************
C   NAME- CHKRNG
C
C   LANGUAGE- VS FORTRAN  TYPE- SUBROUTINE
C
C   VERSION- 1.1    DATE- 10/07/93   PROGRAMMER- DEBBIE CARR  (SMSRC)
C
C   FUNCTION- THIS SUBROUTINE CHECKS CALCULATED VALUES CONTAINED IN
C             ARRAYS AGAINST PREDETERMINED UPPER AND LOWER BOUND
C             LIMITS. IF OUT OF LIMIT DATA IS FOUND IT
C             WILL EITHER BE FLAGGED AS BAD OR
C             A SUBSTITUTE VALUE WILL BE GIVEN IN ITS PLACE.
C             IF A SUBSTITUTE VALUE IS GIVEN, THE 2X2 ARRAY
C             OF DATA WILL BE PROCESSED NORMALLY THROUGH THE CLAVR
C             CLOUD TESTS.
C             IF THE DATA IS FLAGGED AS RSKPCC, THEN THE 2X2
C             ARRAY WILL NOT BE PROCESSED THROUGH THE CLOUD TESTS.
C             IF CHANNEL 3 TEMPERATURE DATA IS FLAGGED AS RSKPCC,
C             CLOUD TESTS USING CHANNEL 3 DATA WILL BE BYPASSED.
C
C   FILES-
C     NONE
C
C   LOCAL PARAMETERS-
C     VARIABLE            TYPE  FUNCTION
C     --------            ----  ----------------------------------
C     CH                  I*2    ARRAY OF CHANNEL VALUES - ALBS/TEMPS
C     ANGLES              I*2    ARRAY OF ANGLES -SOLAR ZENITH, RELATIVE
C                                AZIMUTH, AND SATELLITE ZEITH
C     LAT                 R*4    ARRAY OF LATITUDES
C     LON                 R*4    ARRAY OF LONGITUDES
C     ISCAN               I*2    SCANLINE NUMBER
C     CSOLZN              I*2    SOLAR ZENITH ANGLE
C     RELAZ               I*2    RELAITIVE AZIMUTH ANGLE
C     SATZEN              I*2    SATELLITE ZENITH ANGLE
C     LATP                R*4    LATITUDE
C     LONP                R*4    LONGITUDE
C     CH1A                I*2    CHANNEL 1 ALBEDO
C     CH2A                I*2    CHANNEL 2 ALBEDO
C     CH3A                I*2    CHANNEL 3 ALBEDO
C     CH3T                I*2    CHANNEL 3 TEMPERATURE
C     CH4T                I*2    CHANNEL 4 TEMPERATURE
C     CH5T                I*2    CHANNEL 5 TEMPERATURE
C     SOLZN               R*4    SOLAR ZENITH ANGLE
C     BDSL                I*4    FLAG USED TO INDICATE SCANLINE WITH
C                                BAD DATA
C
C
C   CALLED BY- CLAVRM
C
C   SUBPROGRAMS CALLED- MINMAX
C
C   RESTRICTIONS-
C
C   COMMON AREAS- INCLAV,TYPDAT,RANGES,BADDAT
C
C   COMMENTS- THE SUBROUTINE MINMAX, WHICH CALCULATES THE MININUM AND
C             MAXIMUM VALUES IN THE ARRAYS PASSED TO THIS SUBROUTINE
C             (OF ALL SCANLINES) IS COMMENTED OUT DURING NORMAL
C             OPERATING MODE OF CLAVRORB.  IT CAN BE UNCOMMENTED FOR
C             POSSIBLE TESTING/DEBUGGING DURING OFF-LINE RUNS OF
C             CLAVRORB.  THE WRITE STATEMENTS IN THIS SUBROUTINE ARE
C             COMMENTED OUT DURING NORMAL OPERATING MODE OF CLAVRORB, BU
C             CAN BE UNCOMMENTED DURING OFF-LINE RUNS OF CLAVRORB.
C
C   CHANGES:
C
C      D. Love, SMART Inc., 9/8/95:  added GOTTBB flag set in AVHFDC
C      to bypass CH 4 and 5 tests if no TBB temperatures avail. to
C      calc. Ch. 4 and 5 temperatures.
C
C
C*******************************************************************
C
      INCLUDE "inc/inclav.f"
      INCLUDE "inc/typdat.f"
      INCLUDE "inc/ranges.f"
      INCLUDE "inc/baddat.f"
      INCLUDE "inc/jclcom.f"

C
      LOGICAL*4            NIGHT, DAYTIM, GOTTBB
C
      INTEGER*2            ANGLES(MAXLPX,2,3),
     1                     CSOLZN,RELAZ,SATZEN
C
      INTEGER*4            BDSL, BADT3, BADT4, BADT5,
     1                     ISCAN(2)
C
      REAL*4               CH(MAXLPX,6,2),
     1                     LAT(MAXLPX,2), LON(MAXLPX,2),
     2                     LATP, LONP,
     6                     CH1A,CH2A,CH3A,
     7                     CH3T,CH4T,CH5T
C
C
C.....INITIALIZE FLAG THAT INDICATES BAD SCAN LINE
C
      BDSL  = 0
C
C
C.....FIND MIN AND MAX OF VALUES IN THE ARRAYS
C
CCCC  CALL MINMAX( CH,ANGLES,LAT,LON )
C                                         !! CHANGE "CH" IN HERE
C
C
C.....LOOP THROUGH EACH SCANLINE AND CHECK FOR BAD DATA.  IF FOUND,
C     PRINT THE BAD VALUE AND INCREMENT COUNTERS THAT KEEP TRACK
C     OF BAD DATA STATISTICS.
C
      DO 30 IREC = 1,2
         BADT3 = 0
         BADT4 = 0
         BADT5 = 0
C                                      ! /* LOOP FOR EACH SCANLINE*/
C
         DO 20 I =1,NPIXLS
C                                      ! /* LOOP FOR EACH PIXEL   */
C
            CSOLZN = ANGLES(I,IREC,1)
C                                      ! /* SOLAR ZENITH ANGLE    */
            SATZEN = ANGLES(I,IREC,2)
C                                      ! /* SATELLITE ZENITH ANGLE*/
            RELAZ  = ANGLES(I,IREC,3)
C                                      ! /* SOLAR ZENITH ANGLE    */
C
            LATP   = LAT(I,IREC)
C                                      ! /* LATITUDE              */
            LONP   = LON(I,IREC)
C                                      ! /* LONGITUDE             */
C
            CH1A   = CH(I,1,IREC)
C                                      ! /* CHANNEL 1 ALBEDO      */
            CH2A   = CH(I,2,IREC)
C                                      ! /* CHANNEL 2 ALBEDO      */
            CH3A   = CH(I,3,IREC)
C                                      ! /* CHANNEL 3 ALBEDO      */
            CH3T   = CH(I,4,IREC)
C                                      ! /* CHANNEL 3 TEMPERATURE */
            CH4T   = CH(I,5,IREC)
C                                      ! /* CHANNEL 4 TEMPERATURE */


            CH5T   = CH(I,6,IREC)
C                                      ! /* CHANNEL 5 TEMPERATURE */
C
C ......................................................................
C     CHECK SOLAR ZENITH ANGLES.
C ......................................................................
C
C     IF COSINES OF SOLAR ZENITH ANGLES ARE LESS THAN -.214 (ARCCOSINE
C     OF -.214 IS EQUAL TO 102.3 DEGREES) AND GREATER THAN -1.0,
C     (ARCCOSINE OF -1.0 IS EQUAL TO 180.0 DEGREES),
C     THEN SET VALUE TO -.214.  IF COSINES ARE GREATER THAN 1.0 OR
C     LESS THAN -1.0 THEN FLAG AS BAD DATA.  2X2 ARRAY TESTS WILL BE
C     SKIPPED IN THIS CASE.
C
C     (LWSZN1 = -1000)
C     (LWSZN = -214;  UPSZN = 1000)
C
C
            IF ( (CSOLZN .LT. LWSZN) .OR. (CSOLZN .GT. UPSZN) ) THEN
C
C
              IF ( (CSOLZN .GT. LWSZN1) .AND.
C                                               ! /* SOLAR ZENITH ANGLE
     1             (CSOLZN .LT. LWSZN) ) THEN
C                                               ! /* BETWEEN 102.3 AND
C
C                                               ! /* 180 DEG, RESET TO
                ANGLES(I,IREC,1) = SZN1C
C                                               ! /* COS(102.3)
C
              ELSE
C                                           ! /* DATA OUT OF LIMITS */
C
                ANGLES(I,IREC,1) = SKP2X2
C                                           ! /* SKIP CLOUD TESTS */
C
                BDSZN = BDSZN + 1
C                                           ! /* BAD DATA COUNTER */
                BDSL  = 1
C
              ENDIF
C
              NIGHT  = .FALSE.
C                                      ! /* SINCE SOLAR ZENITH IS BAD,*/
              DAYTIM = .FALSE.
C                                      ! /* THIS WILL BE TREATED AS A */
C
C                                      ! /* TWILIGHT SCENE.           */
C
            ELSE
C                                       ! /* SOLAR ZENITH IS VALID */
C
              IF (CSOLZN .LT. CALBLM) THEN
                NIGHT  = .TRUE.
                DAYTIM = .FALSE.
              ELSE IF (CSOLZN .GT. CDAYLM) THEN
                DAYTIM = .TRUE.
                NIGHT  = .FALSE.
              ELSE
                DAYTIM = .FALSE.
                NIGHT  = .FALSE.
              END IF
C
            ENDIF
C
C
C ......................................................................
C     CHECK IF SATELLITE ZENITH ANGLES ARE OUT OF LIMITS.
C ......................................................................
C
C     (LWSTZ = 0 ; UPSTZ = 700)
C
            IF ( (SATZEN .LT. LWSTZ) .OR. (SATZEN .GT. UPSTZ) ) THEN
C
              ANGLES(I,IREC,2) = STZ1C
C                                          ! /* RESET TO 102.3*10 DEG */
C
              BDSTZ = BDSTZ + 1
C                                          ! /* BAD DATA COUNTER */
              BDSL  = 1
C
            ENDIF
C
C
C ......................................................................
C     CHECK IF RELATIVE AZIMIUTH ANGLES ARE OUT OF LIMITS.
C ......................................................................
C
C     (LWRLZ = 0;  UPRLZ = 1800)
C
C
            IF (RELAZ .LT. LWRLZ) THEN
C
              IF (RELAZ .LT. (LWRLZ-20)) THEN
C
                ANGLES(I,IREC,3) = RLZ1C
C                                          ! /* RESET TO 204.7*10 */
C
                IF (RELAZ .NE. -999) THEN
C                                            ! /* IF NOT NADIR, INCR */
                  BDRLZ = BDRLZ + 1
C                                            ! /* BAD DATA COUNTER   */
                  BDSL  = 1

cccc  STOWE 3/6/97, reset nadir relaz to  from  -999 to 90 
              else 
                angles(i,irec,3) = 900
cccc STOWE

                END IF
C
              ELSE
C                                          ! /* -2.0 < RELAZ < 0.0... */
                ANGLES(I,IREC,3) = LWRLZ
C                                          ! /* RESET TO 0            */
              ENDIF
C
            ELSE IF (RELAZ .GT. UPRLZ) THEN
C
              IF (RELAZ .GT. (UPRLZ+20)) THEN
C
                ANGLES(I,IREC,3) = RLZ1C
C                                           ! /* RESET TO 204.7*10 */
C
                BDRLZ = BDRLZ + 1
C                                           ! /* BAD DATA COUNTER */
                BDSL  = 1
C
              ELSE
C                                           ! /* > 180.0 AND < 182.0 */
                ANGLES(I,IREC,3) = UPRLZ
C                                           ! /* RESET TO 0 */
              ENDIF
C
            ENDIF
C
C
C ......................................................................
C     CHECK FOR BAD DATA IN ARRAYS LAT AND LON (CONTAINS LATITUDE AND
C     LONGITUDE DATA)
C ......................................................................
C
C     (LWLAT = -90.0; UPLAT = 90.0)
C
            IF ( (LATP .LT. LWLAT) .OR. (LATP .GT. UPLAT) ) THEN
C
C              WRITE (6, 1030) LATP, I, ISCAN(IREC)
C
               IF (LATP .LT. LWLAT) LAT(I,IREC) = LWLATC
               IF (LATP .GT. UPLAT) LAT(I,IREC) = UPLATC
C
               BDLAT = BDLAT + 1
C                                           ! /* BAD DATA COUNTER */
               BDSL  = 1
C
            ENDIF
C
C.....LWLON = -180.0; UPLON = 180.0
C
C
            IF ( (LONP .LT. LWLON) .OR. (LONP .GT. UPLON) ) THEN
C
C             WRITE(6,1040) LONP,I,ISCAN(IREC)
C
              IF (LONP .LT. LWLON) LON(I,IREC) = LWLONC
              IF (LONP .GT. UPLON) LON(I,IREC) = UPLONC
C
              BDLON = BDLON + 1
C                                          ! /* BAD DATA COUNTER */
              BDSL  = 1
C
            ENDIF
C
C
C ......................................................................
C     CHECK FOR BAD DATA IN ARRAY CH (CONTAINS ALBEDO/TEMPERATURE VALUES
C ......................................................................
C
C
C....IF THIS IS NIGHT SCENE DATA (I.E., SOLAR ZENITH > ALBEDO LIMIT
C    OF 89.5) THEN BYPASS CHECKING THE ALBEDOS FOR CH 1, 2, AND 3.
C
            IF (NIGHT) GO TO 10
C
C
C ......................................................................
C    IF CHANNEL 1 IS TOO LOW, THEN SET TO LOWER LIMIT VALUE. IF CHANNEL
C    IS TOO HIGH THEN SET TO UPPER LIMIT VALUES.
C ......................................................................
C
C    (LWCH1A = 0.0; UPCH1A = 102.3)
C
C
            IF (CH1A .LE. LWCH1A) THEN
C
              IF (DAYTIM) THEN
                CH(I,1,IREC) = RSKPCC
C                                            ! /* SKIP CLAVR TESTS */
                BDCH1A = BDCH1A + 1
C                                            ! /* BAD DATA COUNTER */
                BDSL   = 1
              ELSE
C                    ! /* TWILIGHT */
                CH(I,1,IREC) = CH11C1
C                                            ! /* RESET TO 0.0   */
              END IF
C
            ELSE IF (CH1A .GT. UPCH1A) THEN
               CH(I,1,IREC) = CH11C2
C                                           ! /* RESET TO 102.3 */
            END IF
C
C
C ......................................................................
C     IF CHANNEL 2 IS TOO LOW, THEN SET TO LOWER LIMIT VALUE. IF CHANNEL
C     IS TOO HIGH THEN SET TO UPPER LIMIT VALUES.
C ......................................................................
C
C     (LWCH2A = 0.0; UPCH2A = 102.3)
C
            IF (CH2A .LE. LWCH2A) THEN
C
              IF (DAYTIM) THEN
                CH(I,2,IREC) = RSKPCC
C                                            ! /* SKIP CLAVR TESTS */
                BDCH2A = BDCH2A + 1
C                                            ! /* BAD DATA COUNTER */
                BDSL   = 1
              ELSE
C                    ! /* TWILIGHT */
                CH(I,2,IREC) = CH21C1
C                                            ! /* RESET TO 0.0   */
              END IF
C
            ELSE IF (CH2A .GT. UPCH2A) THEN
               CH(I,2,IREC) = CH21C2
C                                           ! /* RESET TO 102.3 */
            END IF
C
C
C ......................................................................
C     CHECK CHANNEL 3 ALBEDOS.  IF VALUE IS TOO LOW, THEN SET TO LOWER
C     LIMIT.  IF VALUE IS TOO HIGH, THEN SET TO UPPER LIMIT.
C ......................................................................
C
C     (LWCH3A = 0.0; UPCH3A = 100.0)
C
              IF (CH3A .LE. LWCH3A) THEN
C
                 CH(I,3,IREC) = CH31C1
C                                             ! /* RESET TO 0.0 */
C
              ELSE IF (CH3A .GT. UPCH3A) THEN
C
                 CH(I,3,IREC) = CH31C2
C                                             ! /* RESET TO 100.0 */
C
              END IF
C
C
   10       CONTINUE
C                          ! /* BRANCH FOR NIGHT SCENES */
C
C
C ......................................................................
C     CHECK CHANNEL 3 TEMPERATURES. IF VALUES ARE OUT OF LIMIT, THEN
C     FLAG VALUE WHICH WILL INDICATE SKIPPING THE CHANNEL 3 TESTS.
C ......................................................................
C
C     (LWCH3T = 170.0; UPCH3T = 370.0)
C
            IF ( (CH3T .LT. LWCH3T) .OR. (CH3T .GT. UPCH3T) ) THEN
C
              CH(I,4,IREC) = RSKPCC
C                                        ! /* SKIP CHANNEL 3 TESTS */
C
              BDCH3T = BDCH3T + 1
              BADT3 = BADT3 + 1
C                                        ! /* BAD DATA COUNTER */
              BDSL   = 1
C
            ENDIF
C
C   Skip testing Ch. 4 and 5 temperatures if no TBB temperatures avail.

ccccccccccccccc luo, 11/21/95
cccc	do not skip, so that cc tests can be correctly skipped
ccc	in clavrm.f
C
c            IF (.NOT. GOTTBB) GO TO 20
cccccccccccccccccc

C
C ......................................................................
C     CHECK CHANNEL 4 TEMPERATURES.  IF VALUES ARE BAD THEN FLAG.
C     (CLOUD CODE TESTS WILL BE SKIPPED.)
C ......................................................................
C
C     (LWCH4T = 170.0; UPCH4T = 370.0)
C
            IF ( (CH4T .LT. LWCH4T) .OR. (CH4T .GT. UPCH4T) ) THEN
C
              CH(I,5,IREC) = RSKPCC
C                                         ! /* SKIP CLOUD TESTS */
C
              BDCH4T = BDCH4T + 1
              BADT4 = BADT4 + 1
C                                         ! /* BAD DATA COUNTER */
c              write(6,*) 'new',ch4t,bdch4t
              BDSL   = 1
C
            ENDIF
C
C
C ......................................................................
C     CHECK CHANNEL 5 TEMPERATURES.  IF VALUES ARE BAD THEN FLAG.
C     (CLOUD CODE TESTS WILL BE SKIPPED.)
C ......................................................................
C
C     (LWCH5T = 170.0; UPCH5T = 370.0)
C
            IF ( (CH5T .LT. LWCH5T) .OR. (CH5T .GT. UPCH5T) ) THEN
C
              CH(I,6,IREC) = RSKPCC
C                                          ! /* SKIP CLOUD TESTS */
C
              BDCH5T = BDCH5T + 1
              BADT5 = BADT5 + 1
C                                          ! /* BAD DATA COUNTER */
              BDSL   = 1
C
            ENDIF
C
C
   20    CONTINUE
C                    ! /* END OF PIXEL LOOP */
C
C
C.....INCREMENT TOTAL NUMBER OF BAD SCANLINES IF NEEDED
C
         IF ( BDSL.EQ.1 )  TOTBSL = TOTBSL + 1
C
         ITOBAD = BADT3 + BADT4 + BADT5
C        IF (ITOBAD .GT. 0 .AND. DUMPRN .AND. GOTTBB)
C    *      WRITE (6,36) ISCAN(IREC), 3, BADT3, 4, BADT4, 5, BADT5
C  36       FORMAT(' out-of-limits values found in scanline', I7, ':',
C    *             3('  Ch. ', I1, ':', I4))
C
C                    ! /* END OF SCANLINE LOOP */
   30 CONTINUE
C
      RETURN
C
 1000 FORMAT(1X,'***DATA OUT OF LIMITS ---- SOLAR ZENITH ANGLE= ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1010 FORMAT(1X,'***DATA OUT OF LIMITS ---- SATELLITE ZENITH ANGLE= ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1020 FORMAT(1X,'***DATA OUT OF LIMITS ---- RELATIVE AZIMUTH ANGLE= ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1030 FORMAT(1X,'***DATA OUT OF LIMITS ---- LATITUDE= ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1040 FORMAT(1X,'***DATA OUT OF LIMITS ---- LONGITUDE= ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1050 FORMAT(1X,'***DATA OUT OF LIMITS ---- CHANNEL 1 ALBEDO = ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1060 FORMAT(1X,'***DATA OUT OF LIMITS ---- CHANNEL 2 ALBEDO = ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1070 FORMAT(1X,'***DATA OUT OF LIMITS ---- CHANNEL 3 ALBEDO = ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1080 FORMAT(1X,'***DATA OUT OF LIMITS ---- CHANNEL 3 TEMPERATURE = ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1090 FORMAT(1X,'***DATA OUT OF LIMITS ---- CHANNEL 4 TEMPERATURE = ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
 1100 FORMAT(1X,'***DATA OUT OF LIMITS ---- CHANNEL 5 TEMPERATURE = ',
     1           F9.2,'   PIXEL= ',I5,'   SCANLINE NUMBER= ',I8)
C
C
      END

cccc  Stowe, 3/3/97, need to pass the value of STZ1C thru
      include "inc/bclav.f"
cccc

