      SUBROUTINE CLAVIN                                                 
C                                                                       
C********************************************************************** 
C                                                                       
C   NAME- CLAVIN                                                        
C                                                                       
C   LANGUAGE- VS FORTRAN  TYPE- SUBROUTINE                              
C                                                                       
C   VERSION- 1.0    DATE- 02/01/93   PROGRAMMER- DEBBIE CARR (SMSRC)    
C                                                                       
C                                                                       
C   FUNCTION-                                                           
C                                                                       
C      READS AND QUALITY CHECKS INPUT PARAMETERS (JCL & MISC.)          
C      FOR THE CLAVR CLOUD CLASSIFICATION PROGRAM ("CLAVRORB").         
C      ALSO ASSIGNS VALUES TO CERTAIN VARIABLES BASED ON THE            
C      USER INPUTS.                                                     
C                                                                       
C                                                                       
C   LOCAL PARAMETERS-                                                   
C                                                                       
C      VARIABLE                TYPE  FUNCTION                           
C      --------                ----  --------                           
C      FINLIN (35)             C*1   A FINAL MESSAGE OUTPUT LINE        
C                                                                       
C                                                                       
C                                                                       
C   COMMON AREAS- TYPDAT, PARM1C (CONSTANTS), JCLCOM                    
C                                                                       
C                                                                       
C   CALLED BY-  CLAVRORB                                                
C                                                                       
C                                                                       
C   SUBPROGRAMS CALLED-                                                 
C                                                                       
C                                                                       
C   RESTRICTIONS-                                                       
C                                                                       
C                                                                       
C   COMMENTS-                                                           
C                                                                       
C                                                                       
C***********************************************************************
C***********************************************************************
C                                                                       
C                                                                       
C...LINK IN CONSTANT PARAMETERS AND COMMON VARIABLES.                   
C                                                                       
      INCLUDE "inc/inclav.f"                                                  
      INCLUDE "inc/parm1c.f"                                                 
      INCLUDE "inc/typdat.f"                                                  
      INCLUDE "inc/jclcom.f"                                                  
C                                                                       
C                                                                       
C***********************************************************************
C***********************************************************************
C                                                                       
C                                                                       
C...VARIABLE DECLARATIONS                                               
C                                                                       
      INTEGER*4  IDAT, IDUMP, IOUT1C, IBPRME,                           
     1           IMGPC, IMGSTA, NCSCAN, ON                              
c
C                                                                       
      DATA  ON /1/                                                      
C                                                                       
C***********************************************************************
C***********************************************************************
C                                                                       
C                                                                       
C...READ AND ECHO INPUT PARAMETERS--                                    
C                                                                       
C     1.  ONOFF  -- OFFLINE (0) OR ONLINE (1) RUN FLAG                  
C     2.  IDJCL  -- JCL-SPECIFIED SATELLITE ID                          
C     3.  IDAT   -- 1B DATA TYPE, (0) GAC OR (1) LAC                    
C     4.  TAPDSK -- READ 1B INPUT DATA FROM TAPE (0) OR DISK (1)        
C     5.  IDUMP  -- DUMP (1) COMPUTED PARAMETERS FOR EACH SCANLINE      
C                   PROCESSED                                           
C     6.  IOUT1C -- WRITE 1C INFORMATION TO DISK FILE (1),              
C                   TAPE/CARTRIDGE FILE (2), OR NO FILE (0)             
C     7.  IBPRME -- OPTION TO PRODUCE 1B' FILE (1 = MAKE 1B')           
C     8.  IMGPC  -- FLAG TO PRODUCE TSO/TEMP. FILES FOR PC IMAGING      
C     9.  ORBIT  -- PROCESS ENTIRE (1) OR PARTIAL (0) ORBIT             
C     A.  IMGSTA -- OPTION TO CREATE IMAGE WORKSTATION FILES (1=YES)    
C     B.  IBSTRT -- STARTING SCANLINE TO PROCESS (FOR PARTIAL ORBITS)   
C     C.  IBEND  -- ENDING SCANLINE TO PROCESS (FOR PARTIAL ORBITS)     
C     D.  IPBEG  -- STARTING LAC PIXEL TO PROCESS (FOR PARTIAL ORBITS)  
C     E.  IPEND  -- ENDING LAC PIXEL TO PROCESS (FOR PARTIAL ORBITS)    
C     F.  IDBEG  -- STARTING SCANLINE TO PRINT (FOR DATA DUMPS)         
C     G.  IDEND  -- ENDING SCANLINE TO PRINT (FOR DATA DUMPS)  
       open(unit=5,file='init.dat')         
C                                                                       
      READ(5,1000)  ONOFF, IDJCL, IDAT, TAPDSK, IDUMP, IOUT1C, IBPRME,  
     1              IMGPC, ORBIT, IMGSTA, IBSTRT, IBEND, IPBEG, IPEND,  
     2              IDBEG, IDEND  
C                                                                       
C     WRITE(6,1010) ONOFF, IDJCL, IDAT, TAPDSK, IDUMP, IOUT1C, IBPRME,  
C    1              IMGPC, ORBIT, IMGSTA, IBSTRT, IBEND, IPBEG, IPEND,  
C    2              IDBEG, IDEND                                        
C                                                                       
C                                                                       
C   SET LOGICAL VARIABLE INDICATING DATA TYPE TO BE PROCESSED, AND      
C   MISCELLANEOUS PARAMETERS WHICH ARE TYPE-DEPENDENT.                  
C                                                                       
      IF (IDAT .EQ. 0) THEN                                             
C                                                                       
        GAC = .TRUE.                                                    
        IPBEG = BGACPX                                                  
        IPEND = EGACPX                                                  
        NPIXLS = MAXGPX                                                 
        IMGPIX = NGIMPX                                                 
        NRECS  = 1                                                      
        ISKIP = INT(IBSTRT/2)                                           
        FACTOR = 8.0                                                    
        PXFIX1 = 5                                                      
        PXFIX2 = 189                                                    
        PXFIX3 = 213                                                    
        LAC = .FALSE.                                                   
C                                                                       
      ELSE IF (IDAT .EQ. 1) THEN                                        
C                                                                       
        LAC = .TRUE.                                                    
        NPIXLS = MAXLPX                                                 
        IMGPIX = NLIMPX                                                 
        NRECS  = 1                                                      
        ISKIP = IBSTRT-1                                                
        FACTOR = 40.0                                                   
        PXFIX1 = 21                                                     
        PXFIX2 = 944                                                    
        PXFIX3 = 1064                                                   
        GAC = .FALSE.                                                   
C                                                                       
      ELSE                                                              
C                                                                       
        CALL EMESG (7, 'CLAVIN', 'FATAL')                               
C                                                                       
      END IF                                                            
C                                                                       
C                                                                       
C...IF USER DESIRES A 'DUMP RUN' (I.E., IDUMP=1), SET LOGICAL VARIABLE  
C   DUMPRN TO 'TRUE'.                                                   
C                                                                       
      IF (IDUMP.EQ.1) DUMPRN = .TRUE.                                   
C                                                                       
C                                                                       
C...IF USER DESIRES CREATION OF A 1B' FILE, SET LOGICAL VARIABLE        
C   BPRIME TO 'TRUE'.                                                   
C                                                                       
      IF (IBPRME.EQ.1) BPRIME = .TRUE.                                  
C                                                                       
C                                                                       
C...IF USER DESIRES CREATION OF TSO/TEMP. FILES FOR IMAGING             
C   SET THE APPROPRIATE LOGICAL VARIABLES TO 'TRUE',                    
C   AND TEST THE PARTIAL ORBIT FLAG AND BEGINNING/ENDING SCANLINES      
C   TO ENSURE USER IS NOT ATTEMPTING TO PROCESS MORE THAN NLIMPX SCANS. 
C                                                                       
      IF ((IMGPC .EQ. 1) .OR. (IMGSTA .EQ. 1)) THEN                     
C                                                                       
        IF (ORBIT .EQ. 0) THEN                                          
C                                                                       
          NCSCAN = (IBEND - IBSTRT + 1)                                 
C                                                                       
          IF (NCSCAN .LE. NLIMPX) THEN                                  
            IF (IMGPC  .EQ. 1) MAKPCI = .TRUE.                          
            IF (IMGSTA .EQ. 1) MAKSTI = .TRUE.                          
          ELSE                                                          
            CALL EMESG (8, 'CLAVIN', 'OK')                              
          END IF                                                        
C                                                                       
        ELSE                                                            
          CALL EMESG (8, 'CLAVIN', 'OK')                                
        END IF                                                          
C                                                                       
      END IF                                                            
C                                                                       
C                                                                       
C...DEPENDING ON USER'S CHOICE FOR OUTPUT OF 1C DATA                    
C     (1C DISK FILE, 1C TAPE FILE), POSITION THE APPROPRIATE TAPE       
C     TO THE BEGINNING FILE NUMBER OR OPEN THE 1C DISK FILE.            
C     IF USER DOES NOT SPECIFY A VALID 1C OUTPUT DESTINATION,           
C     TERMINATE PROCESSING.  IF 1C DISK FILE, ALSO CALL SUBROUTINE      
C     TO READ/UNPACK 1C HOUSEKEEPING RECORD.                            
C                                                                       
         IF (IOUT1C.EQ.0) THEN                                          
C                                                                       
            IF (ONOFF .EQ. ON) CALL EMESG (9, 'CLAVIN', 'FATAL')        
C                                                                       
            DISK1C = .FALSE.                                            
            TAPE1C = .FALSE.                                            
C                                                                       
         ELSE IF (IOUT1C.EQ.1) THEN                                     
C                                                                       
            DISK1C = .TRUE.                                             
            TAPE1C = .FALSE.                                            
C                                                                       
         ELSE IF (IOUT1C.EQ.2) THEN                                     
C                                                                       
            IF (ONOFF .EQ. ON) CALL EMESG (9, 'CLAVIN', 'FATAL')        
C                                                                       
            TAPE1C = .TRUE.                                             
            DISK1C = .FALSE.                                            
C                                                                       
         ELSE                                                           
C                                                                       
            CALL EMESG (9, 'CLAVIN', 'FATAL')                           
C                                                                       
         END IF                                                         
C                                                                       
C                                                                       
      RETURN                                                            
C                                                                       
 1000 FORMAT(I5, /, I5, /, 8I5, 3( /, I5, 1X, I5))  
                    
C                                                                       
 1010 FORMAT(5X,'CLAVRORB JCL CARD PARAMETERS.....',/,                  
     1   8X,'OFFLINE (0) OR ONLINE (1) RUN                 =  ', I5,/,  
     2   8X,'JCL-SPECIFIED SATELITTE ID                    =  ', I5,/,  
     3   8X,'1B DATA TYPE-- GAC (0) / LAC (1)              =  ', I5,/,  
     4   8X,'1B AVHRR INPUT-- TAPE(0) / DISK(1)            =  ', I5,/,  
     5   8X,'DIAGNOSTIC DUMP FLAG (1 = DUMP, 0 = NO DUMP)  =  ', I5,/,  
     6   8X,'NO 1C FILE(0) / 1C DISK(1) / 1C TAPE(2)       =  ', I5,/,  
     7   8X,'CREATE 1B PRIME TAPE (1=YES, 0=NO)            =  ', I5,/,  
     8   8X,'CREATE FILES FOR PC IMAGING (1=YES, 0=NO)     =  ', I5,/,  
     9   8X,'PROCESS THE ENTIRE ORBIT (1=YES, 0=NO)        =  ', I5,/,  
     A   8X,'CREATE IMAGE WORKSTATION FILES (1=YES, 0=NO)  =  ', I5,/,  
     B   8X,'BEGINNING SCAN LINE (FOR PARTIAL ORBITS)      =  ', I5,/,  
     C   8X,'ENDING SCAN LINE (FOR PARTIAL ORBITS)         =  ', I5,/,  
     D   8X,'BEGINNING PIXEL (FOR LAC/PARTIAL ORBITS)      =  ', I5,/,  
     E   8X,'ENDING PIXEL (FOR LAC/PARTIAL ORBITS)         =  ', I5,/,  
     F   8X,'BEGINNING SCANLINE TO PRINT (FOR DATA DUMPS)  =  ', I5,/,  
     G   8X,'ENDING SCANLINE TO PRINT (FOR DATA DUMPS)     =  ', I5,/)  
C                                                                       
C                                                                       
      END                                                               
