      SUBROUTINE CLAVRM (CH, ANGLES, LAT, LON, CLDCOD, CLRCLD, ISCAN,
     *                   CH1COUNT, GOTTBB)
C**********************************************************************
C
C    FUNCTION - SUBROUTINE CLAVRM WILL OPERATE ON EACH SET OF 2X2
C     PIXELS.  DEPENDING WHETHER THEY ARE IN DAYLIGHT OR NOT WILL
C     DETERMINE IF THE DAYTIME CLAVR OR THE NIGHTTIME CLAVR ALGORITHM
C     IS CALLED.
C
C     CHANGES FOR CLAVR ALGORITHM VERSION A.3H:
C     -----------------------------------------------------------------
C     (1)  MEAN LONGITUDE (ALON) FOR 2X2 PIXEL ARRAY NOW PASSED
C          TO SUBROUTINE CLAVRD FOR USE IN DETERMINING IF ARRAY
C          IS LOCATED IN A DESERT REGION.
C
C    INPUT PARAMETERS
C     CH      ARRAY CONTAINING CHANNELS 1,2,3 ALBEDOS AND CHANNELS
C             3,4,5 TEMPERATURES FOR TWO SCAN LINES. THE NUMBER
C             OF ELEMENTS DEPENDS ON THE DATA SOURCE
C     ANGLES  ARRAY OF SOLAR, SATELLITE AND RELATIVE AZIMUTH ZENITH
C             ANGLES FOR EACH PIXEL
C     LAT     LATITUDES FOR 2X409 PIXEL ARRAY
C
C
C     OUTPUT PARAMETERS
C      CLDCOD  -   COLOR CODE OF PASS 1 TEST:
C      CLRCLD  -   CLEAR/CLOUD (0/1) FLAG FOR EACH PIXEL
C
C
C     COMMON BLOCKS
C      DTHRES  -   CONTAINS DAYTIME THRESHOLDS FOR LAND/SEA
C      NTHRES  -   CONTAINS NIGHTTIME THRESHOLDS FOR LAND/SEA
C
C
C*********************************************************************
C
C
C...INCLUDE CONSTANT PARAMETERS.
C
      INCLUDE "inc/inclav.f"
C
C
C...INCLUDE COMMON AREAS.
C
      INCLUDE "inc/dthres.f"
      INCLUDE "inc/nthres.f"
      INCLUDE "inc/typdat.f"
      INCLUDE "inc/kounts.f"
      INCLUDE "inc/atmisc.f"
      INCLUDE "inc/baddat.f"
      INCLUDE "inc/apath.f"

ccccc	Larry's common block to check bad sat angle with
ccccc		would-be good CC, 2/28/97
	include "inc/larry.f"
	include "inc/ranges.f"
ccccc

C
C
      INTEGER*2 CLDCOD(MAXLPX,2),ANGLES(MAXLPX,2,3),
     1          CLRCLD(MAXLPX,2), CO(4), CC,
     2          DNC, DNFLAG, ANGL(4,3), DNF(4)
C
      INTEGER*4  LST, LSTSUM,
     1          ISCAN(2),
     2          TCOUNT(4), CH1COUNT(MAXLPX,2)
C
      REAL*4    P(4,6),  LAT(MAXLPX,2), RLAT(4),
     1          ALAT, LON(MAXLPX,2), RLON(4), ALON, CH(MAXLPX,6,2)
C
      LOGICAL   GOTTBB
C
C---------------------------------------------------------------------
C
C
C.....CHECK FOR DATA OUT OF LIMITS
C
      CALL CHKRNG (CH, ANGLES, LAT, LON, ISCAN, GOTTBB)
C
C
C.....LOOP OVER SCAN SPOTS AND PICK UP ALBEDOS/TEMPERATURES
C
      ITEMP = INT((IPEND - IPBEG + 1) / 2)
C
      DO 500 JJ = 1, ITEMP
         J = JJ*2-1
         LSTSUM = LSBUF(J,1)+LSBUF(J+1,1)+LSBUF(J,2)+LSBUF(J+1,2)
C
C...IF 0 TO 2 OF THE BOXES HAVE LAND THEN THE LAND/SEA TAG IS SET TO 0
C...IF 3 OR 4 OF THE BOXES HAVE LAND THEN THE LAND/SEA TAG IS SET TO 1
C
         IF (LSTSUM.LE.2) LST = 0
         IF (LSTSUM.GE.3) LST = 1
C
         RLAT(1) = LAT(J,1)
         RLAT(2) = LAT(J,2)
         RLAT(3) = LAT(J+1,1)
         RLAT(4) = LAT(J+1,2)
         ALAT = (RLAT(1)+RLAT(2)+RLAT(3)+RLAT(4))/4.0
C
         RLON(1) = LON(J,1)
         RLON(2) = LON(J,2)
         RLON(3) = LON(J+1,1)
         RLON(4) = LON(J+1,2)
         ALON = (RLON(1)+RLON(2)+RLON(3)+RLON(4))/4.0
C
C...ALL FOUR PIXELS MUST BE IN DAYLIGHT TO USE DAYTIME ALGROITHM
C
         DNFLAG = 0
         DNF(1) = DAYNIT(J,1)
         DNF(2) = DAYNIT(J,2)
         DNF(3) = DAYNIT(J+1,1)
         DNF(4) = DAYNIT(J+1,2)
         DNC = DNF(1)+DNF(2)+DNF(3)+DNF(4)
         IF (DNC.GT.0) DNFLAG = 1
C
         DO 5 IANG = 1, 3
            ANGL(1,IANG) = ANGLES(J,1,IANG)
            ANGL(2,IANG) = ANGLES(J,2,IANG)
            ANGL(3,IANG) = ANGLES(J+1,1,IANG)
            ANGL(4,IANG) = ANGLES(J+1,2,IANG)
    5    CONTINUE
C
C
         CO(1) = 0
         CO(2) = 0
         CO(3) = 0
         CO(4) = 0
C
C....INITIALIZE THE CLOUD CODE VARIABLE TO ZERO.
C
         CC = 0
C
            TCOUNT(1) = CH1COUNT(J,1)
            TCOUNT(2) = CH1COUNT(J,2)
            TCOUNT(3) = CH1COUNT(J+1,1)
            TCOUNT(4) = CH1COUNT(J+1,2)
C
         DO 10 IC = 1,6
            P(1,IC) = CH(J,IC,1)
            P(2,IC) = CH(J,IC,2)
            P(3,IC) = CH(J+1,IC,1)
            P(4,IC) = CH(J+1,IC,2)
   10    CONTINUE
C
C
C.....CHECK TO SEE IF BAD SOLAR ZENITH ANGLE WILL CAUSE THE CLOUD
C     CODE TESTS TO BE SKIPPED.  (SKP2X2 = 9990)
C
      DO 6 I=1,4
C
        IF ( ANGL(I,1).EQ.SKP2X2 ) GO TO 490
	
ccccc	2/26/97, STOWE, add checks on other angles
ccc	BUT, switched to after the tests, 2/28/97
ccc	if(angl(i,2).eq.stz1c)then
ccc	write(6,*)'Larry: bad satz =',angl(i,2)
ccc	goto 490
ccc	endif
ccc	if(angl(i,3).eq.rlz1c)then
ccc	write(6,*)'Larry: bad rela =',angl(i,3)
ccc	goto 490
ccc	endif
ccccc

C
    6 CONTINUE
C
C
C.....CHECK TO SEE IF BAD CHANNEL 1, 2 OR 3 ALBEDOS OR THE CHANNEL
C     4 OR 5 TEMPERATURES SHOULD CAUSE THE CLAVR TESTS TO BE SKIPPED.
C     (THE CHANNEL 3 TEMPERATURE WILL ONLY BE CHECKED IN THE
C     CLAVR TESTS WHICH RELY ON THAT VALUE.)
C
      DO 7 I = 1, 4
C
        IF ( P(I,1).EQ.RSKPCC ) THEN
C                                         ! /* BAD CHANNEL 1 ALBEDO */
          TSTSKP(10,1) = TSTSKP(10,1) + 1
          GO TO 490
        END IF
C
        IF ( P(I,2).EQ.RSKPCC ) THEN
C                                         ! /* BAD CHANNEL 2 ALBEDO */
          TSTSKP(10,2) = TSTSKP(10,2) + 1
          GO TO 490
        END IF
C
C   CH 3 ALBEDOS NOT TESTED FOR SKIPPING TESTS.
CC
CC      IF ( P(I,3).EQ.RSKPCC ) THEN
C                                         ! /* BAD CHANNEL 3 ALBEDO */
CC        TSTSKP(10,3) = TSTSKP(10,3) + 1
CC        GO TO 490
CC      END IF
C
        IF ( P(I,5).EQ.RSKPCC ) THEN
C                                         ! /* BAD CHANNEL 4 TEMP */
          TSTSKP(10,5) = TSTSKP(10,5) + 1
          GO TO 490
        END IF
C
        IF ( P(I,6).EQ.RSKPCC ) THEN
C                                         ! /* BAD CHANNEL 5 TEMP */
          TSTSKP(10,6) = TSTSKP(10,6) + 1
          GO TO 490
        END IF
C
    7 CONTINUE
C
C
C
C...CHOOSE THE APPROPRIATE THRESHOLDS
C...AND CALL APPROPRIATE CLAVR ALGORITHM
C
         IF (DNFLAG.EQ.0) THEN
C
           IF (LST .EQ. 0) THEN
             CALL DAYOCN (P, ANGL, ALAT, THRO, CC, CO)
           ELSE IF (LST .EQ. 1) THEN
             CALL DAYLND (P, ANGL, ALAT, ALON, THRL, CC, CO)
           ELSE
             CALL EMESG (10, 'CLAVRM', 'OK')
           ENDIF
C
         ELSE IF (DNFLAG .EQ. 1) THEN
C
           IF (LST .EQ. 0) THEN
             CALL NITOCN (P, ANGL, ALAT, THRON, CC, CO, TCOUNT)
           ELSE IF (LST .EQ. 1) THEN
             CALL NITLND(P, ANGL, ALAT, ALON, THRLN, CC, CO, TCOUNT)
           ELSE
             CALL EMESG (10, 'CLAVRM', 'OK')
           ENDIF
C
         ELSE
             CALL EMESG (11, 'CLAVRM', 'OK')
         ENDIF
C
C
c         IF ( CC .EQ. 0 ) WRITE (6,1000) OTEST, RTEST, LTEST
C

ccccc	2/28/97, STOWE, add checks on sat angles
cc		if good cc but bad angle, reset cc=0 and co=0
cc		also accumulate counters
ccc	see also the changes in clavrorb.f and the addition
cc		of inc/larry.f

	if(cc.ne.0)then
ccc	STOWE 3/10/97 count nadir pix with good cc
	  do i=1,4
	    if(angl(i,2).eq.0)then
	      thirdc=thirdc+1
	      endif
	    enddo
ccc
	  do i=1,4
	    if(angl(i,2).eq.stz1c)then
c	write(6,*)'Larry: bad satz =',angl(i,2),'stz1c =',stz1c
	      lrrysat=lrrysat+1
	      goto 6666
	    endif
	    if(angl(i,3).eq.rlz1c)then
c	write(6,*)'Larry: bad rela =',angl(i,3), 'rlz1c =',rlz1c
	      lrryrel=lrryrel+1
	      goto 6666
	    endif
	  enddo
	  goto 7777

6666	  continue
	  cc=0
	  co(1)=0
	  co(2)=0
	  co(3)=0
	  co(4)=0

7777	  continue
	endif
ccccc		end of 2/28/97 STOWE

C
  490    CONTINUE
C
         IF ( CC .EQ. 0 ) TOTCCM(0) = TOTCCM(0) + 1
C
C
         CLDCOD(J,1)   = CC
         CLDCOD(J,2)   = CC
         CLDCOD(J+1,1) = CC
         CLDCOD(J+1,2) = CC
C
         CLRCLD(J,1)   = CO(1)
         CLRCLD(J,2)   = CO(2)
         CLRCLD(J+1,1) = CO(3)
         CLRCLD(J+1,2) = CO(4)
C
  500 CONTINUE
C
C
      RETURN
C
 1000 FORMAT (//,'  **  NO COLOR CODE DETERMINED FOR CLAVR PATH:  ',
     1        A6, '-', A6, '-', A6, '  ***', /)
C
      END

cccc	Larry 3/3/97
	include "inc/bclav.f"
