      PROGRAM CLAVRORB
C
C**********************************************************************
C
C
C   NAME- CLAVRORB
C
C   LANGUAGE- VS FORTRAN  TYPE- MAIN
C
C   VERSION- 1.0    DATE- ??/??/89   PROGRAMMER- CRAIG LONG  (SSAI)
C   VERSION- 2.0    DATE- 02/01/93   PROGRAMMER- DEBBIE CARR (SMSRC)
C   VERSION- 3.0  (P1.X4) 09/01/95   PROGRAMMER- DOUG LOVE   (SMART)
C
C   FUNCTION-
C
C      - READ AVHRR 1B DATASET (TAPE CONTAINS TBM HEADER;DISK DOES NOT),
C      - PROCESS THE DATA, I.E. COMPUTE ALBEDOS FOR CHANNELS 1,2 &3,
C        COMPUTE TEMPERATURES FOR CHANNELS 3,4 & 5, COMPUTE VIEWING
C        GEOMETRY (SOLAR ZENITH, SATELLITE ZENITH, RELATIVE AZIMUTH).
C      - ACCESS CLAVR CLOUD PROCESSING ALGORITHMS AND DETERMINE IF
C        2X2 PIXEL ARRAYS ARE CLEAR, CLOUDY OR MIXED. THE PROGRAM WILL
C        PROCESS GAC PIXELS ON A 2X409 BASIS (2 RECORDS, 409 SAMPLES)
C        *OR* LAC PIXELS ON A 2X2048 BASIS (2 RECORDS, 2048 SAMPLES).
C      - WRITE THE ABOVE INFORMATION ONTO A DISK OR TAPE IN SUCH A
C        FORMAT THAT THE USER CAN USE THIS 1C DATA WITH THE 1B DATA
C        SCAN LINE BY SCAN LINE.
C
C
C
C
C
C
C
C   CALLING SEQUENCE PARAMETERS-
C
C      VARIABLE         I/O   TYPE   FUNCTION
C      --------         ---   ----   --------
C      ALDONE            I    L*4    BOOLEAN TEST FOR COMPLETENESS OF
C                                    *ALL* CATEGORIES
C
C
C   LOCAL PARAMETERS-         {{ LOCAL & PASSED MIXED RIGHT NOW }}
C
C      VARIABLE               TYPE  FUNCTION
C      --------               ----  --------
C      PASFAL (1:3)           I*4   TOTAL # PIXELS WHICH ARE:
C                                     (1) CLEAR  (2) CLOUDY  (3) MIXED
C      IGACHD   -   GAC HEADER RECORD (6440 BYTES TOTAL, INTEGER FORMAT)
C      SED      -   SUN-EARTH DISTANCE (IN A.U.)
C      GACREC   -   GAC DATA RECORD (3220 BYTES PER L.R.)
C      GACA     -   GAC ALBEDOS AND TEMPERATURES FOR EACH PIXEL
C                   IN A 2 X 409 ARRAY (ELEMENTS 1,2,3 ARE CH
C                   1,2,3 ALBEDOS, ELEMENTS 4,5,6 ARE CH 3,4,5
C                   TEMPERATURES. GACA(6,409,20)
C      EQTR     -   EQUATION OF TIME IN HOURS
C      SOLDEC   -   SOLAR DECLINATION
C      CGACHD   -   GAC HEADER RECORD FROM TAPE/DISK FILE (CHAR FORMAT)
C      NSAT     -   SATELLITE ID NUMBER
C      OSSEC    -   BEGINNING COVERAGE TIME IN MILLE SECONDS
C      ISC      -   INDEX TO SATELLITE NUMBER (NOAA-9 (1), NOAA-10 (2),
C               -   NOAA-11 (3)
C      OEYEAR   -   YEAR MINUS 1900
C      OEDAY    -   DAY OF YEAR
C      CORCT4   -   NONLINEAR TEMPERATURE CORRECTION VALUES FOR CH 4
C      CORCT5   -   NONLINEAR TEMPERATURE CORRECTION VALUES FOR CH 5
C      SED      -   SUN EARTH DISTANCE RATIO
C      SEDSQ    -   SUN EARTH DISTANCE RATIO SQUARED
C      GMT      -   GREENWICH TIME IN HOURS FOR THE BEGGINING OF COVERAG
C      NUMSCN   -   TOTAL NUMBER OF SCANS ON TAPE/DISK
C      IBLOCK   -   INDEX TO NUMBER OF RECORDS TO BE PROCESSED
C      IQUAL    -   QUALITY INDICATORS AS DISCRIBED IN NOAA USER GUIDE
C      CH       -   ARRAY OF CHANNELS 1 THRU 5'S DATA
C      NSLINE   -   COUNTER OF SCANS PROCESSED
C      IRET     -   RETURN FLAG (GOOD/ERROR)
C      DAYNIT   -   ARRAY OF DAY/NIGHT FLAGS
C      FLAG     -   FLAG FOR PRINTING DATA
C      FLAG1    -   FLAG FOR COMPUTING VIEWING GEOMETRY, INDICATES
C                   THAT FIRST SCAN IN THE PAIR IS BEING PROCESSED
C      FLAGQ    -   FLAG INCASE QUALITY INDICATORS SHOW ERROR
C      SPLAT    -   SATELLITE VIEWED LATITUDE OF PIXEL
C      SPLON    -   SATELLITE VIEWED LONGITUDE OF PIXEL
C      LSTAG    -   LAND/SEA TAG (1=LAND, 0=OCEAN)
C      LSBUF    -   DATA BUFFER OF LAND/SEA TAGS
C      COSZ     -   ARRAY OF COSINE OF SOLAR ZENITH ANGLE FOR EACH PIXL
C      SATZ     -   ARRAY OF SATELLITE ZENITH ANGLES FOR EACH PIXEL
C      RELZ     -   ARRAY OF RELATIVE AZIMUTHS ANGLES FOR EACH PIXEL
C      ANGLES    -   3-D ARRAY CONTAINING THE ABOVE ANGLES
C      RLAT     -   ARRAY OF LATITUDE FOR BOTH SCANS
C      RLON     -   ARRAY OF LONGITUDES FOR BOTH SCANS
C      CLDCOD   -   COLOR CODES FOR WHICH CLAVR TEST PRODUCED FINAL
C                   CLEAR/CLOUDY DETERMINATION
C      CLRCLD   -   CLEAR/CLOUDY DETERMINATION FOR EACH PIXEL
C      OUTPUT   -   OUTPUT BUFFER USED FOR IMAGE FILE CREATION
C      OUTI2    -   HALF WORD OF 1C FORMAT OUTPUT
C      ICLRS    -   TOTAL NUMBER OF CLEAR PIXELS AS DETERMINED BY CLAVR
C      ICLDS    -   TOTAL NUMBER OF CLOUDY PIXLS AS DETERMINED BY CLAVR
C      IMIXS    -   TOTAL NUMBER OF MIXED PIXELS AS DETERMINED BY CLAVR
C
C
C   FILES-
C
C    DSN OR DESCRIPTIVE TITLE    I/O  UNIT  FUNCTIONAL DESCRIPTION
C    --------------------------  ---  ----  ----------------------
C    TST.RA11.DKC.NH.NONLIN       I    ??   NON-LINEAR CORR. FACTORS FOR
C                                           AVHRR CH4 & 5, NOAA-11
C                      FT05F001   I    05   JCL INPUT PARAMETERS
C                      FT06F001   O    06   PRINTED OUTPUT
C
C
C   SUBPROGRAMS CALLED-
C
C      FFPOSN   -   SYSTEM ROUTINE TO POSITION A TAPE TO A CERTAIN FILE
C      FFPUT    -   SYSTEM ROUTINE TO WRITE A LOGICAL RECORD OF DATA
C                   TO THE OUTPUT DEVICE
C      ASEBCD   -   SYSTEM ROUTINE TO CONVERT ASCII TO EBCDIC
C      AVHEDR   -   UNPACKS GAC AND LAC HEADER RECORDS
C      ESD      -   COMPUTES EARTH-SUN DISTANCE
C      AVHFDC   -   UNPACKS GAC RECORD, COMPUTES ALBEDOS AND
C                   AND TEMPERATURES AND VIEWING GEOMETRY
C      GETLST   -   OBTAIN LAND/SEA TAGS FROM OPERATIONS
C      CLAVRM   -   CLOUD SCREENING SOFTWARE FOR PASS 1
C      GRID05   -   DETERMINES  0.5 X 0.5 GLOBAL GRID LOCATION
C      PREVEW   -   CREATES A PREVIEW STRIP USING CHAN 4 TEMP DATA
C                   USING 1 IN 4 SCANS
C      PRINTCON -   PRINTS THE CONSTANT VALUES READ IN BY INSPCN
C
C
C   RESTRICTIONS-
C
C
C
C
C   COMMENTS-
C
C          NEW PROCESSING OPTION ADDED TO PROVIDE DIAGNOSTIC DUMPS
C          FOR EACH SCANLINE PROCESSED.  DATA PROVIDED INCLUDES...
C              -  SCANLINE
C              -  PIXEL NUMBER
C              -  LATITUDE
C              -  LONGITUDE
C              -  ALBEDO VALUES FOR CHANNELS 1, 2, AND 3
C              -  TEMPERATURES FOR CHANNELS 3, 4, AND 5
C              -  SOLAR ZENITH, SATELLITE ZENITH, AND
C                 RELATIVE AZIMUTH ANGLES
C              -  LAND/SEA TAG ASSIGNMENT
C              -  CLAVR COLOR CODE
C              -  CH 4 - CH 5 DIFFERENCE
C          THE USER SHOULD EXERCISE CAUTION WHEN INVOKING THIS
C          OPTION DUE TO THE LARGE AMOUNT OF PRINTOUT PRODUCED
C          FOR EACH SCANLINE PROCESSED.  THE OPTION MAY BE
C          INVOKED BY SETTING THE DUMP/NO DUMP FLAG IN THE
C          EXECUTION JCL.
C
C
C
C   COMMON AREAS-
C
C      KOUNTS   -   COUNTERS FOR EACH CLAVR TEST AND
C                   SUMMATION OF PASS/FAILS FOR EACH CLAVR TEST
C      ANGLE    -   VIEWING ANGLES
C      GEOG     -   LAT/LON ARRAYS
C      SOL      -   SUN/EARTH DISTANCE RATIO
C      NONLIN   -   NONLINEAR CORRECTION VALUES FOR TEMPERATURES
C      SOLPRM   -   SOLAR PARAMETERS
C
C
C   BLOCK DATAS-
C
C
C***********************************************************************
C***********************************************************************
C
C
C  LINK IN (GENERAL) AND 1C FILE CONSTANT PARAMETERS.
C
      INCLUDE "inc/inclav.f"
      INCLUDE "inc/parm1c.f"
C
C
C***********************************************************************
C***********************************************************************
C
C
C  LINK IN COMMON AREAS.
C
      INCLUDE "inc/clvunt.f"
      INCLUDE "inc/jclcom.f"
      INCLUDE "inc/hk1c.f"
      INCLUDE "inc/ohed1c.f"
      INCLUDE "inc/kounts.f"
      INCLUDE "inc/angle.f"
      INCLUDE "inc/geog.f"
      INCLUDE "inc/solprm.f"
      INCLUDE "inc/typdat.f"
      INCLUDE "inc/atmisc.f"
      INCLUDE "inc/qcflgs.f"
      INCLUDE "inc/spcnst.f"
      INCLUDE "inc/newcon.f"
      INCLUDE "inc/baddat.f"
      INCLUDE "inc/ranges.f"

cccc    2/28/97 STOWE, tracking pix w/ good cc and bab sat angle
        include "inc/larry.f"


C
C
C***********************************************************************
C***********************************************************************
C
C
C  LOCAL VARIABLE DECLARATIONS
C
      CHARACTER*1 TBMHED(122),  CGACHD(G1BRCL) 
C
C     CHARACTER*4  CTIME(7)
C
      CHARACTER*6  VERSON
C
      CHARACTER*8  PROGRM
C
      INTEGER*4    MON,DAY,YR
      CHARACTER*8  CTIME

      INTEGER*2  CLDCOD(MAXLPX,2),
     1           ANGLES(MAXLPX,2,3),
     3           CLRCLD(MAXLPX,2) 
C
      INTEGER*4  GACREC(G1BWDS,2),IGACHD(1610),
     1           IQTEMP,
     3           LREC4,
     4           LLSTAG,  KCLAVR(2), ISCAN(2),
     5           I4LOC2(MAXANC,2), ADLAST, N1CREX, IBLOCK,
     6           AVHLAC(L1BWDS),
     7           BEG1C, END1C, C1REC,
     8           HOUSE(L1CWDS), IXSCAN,
     9           DDAYS, DDAY0,CH1COUNT(MAXLPX,2),COUNT1(MAXLPX)
C
      LOGICAL*4  FIRST, FLAG, FLAG1,
     1           PRTST, GOTTBB
C
      REAL*4     CH (MAXLPX,6,2),
     1           RLAT(MAXLPX,2), RLON(MAXLPX,2), ATVALU (MAXLPX,6) 
C
      REAL*8     SED      
C
C
      EQUIVALENCE (CGACHD(1), IGACHD(1))
C
C
C
      DATA  GACREC/G1BWDS*0, G1BWDS*0/
      DATA  AVHLAC/L1BWDS*0/
C
C
      DATA  FIRST/.TRUE./,LREC4/12288/, LLSTAG/0/, KCLAVR/0,0/,
     1 ADLAST/-1/, N1CREX/0/,
     2 IVREX /0/
C
      DATA CLDCOD/MAXLPX*1, MAXLPX*1/, FLAG/.FALSE./, FLAG1/.FALSE./,
     1 ISW/1/, NPCFIL/60/,
     2 BEG1C/0/, END1C/0/, C1REC/0/, IXSCAN/0/
C
C
      DATA VERSON /'A.4H  '/, PROGRM /'CLAVRORB'/

ccccc   STOWE 3/3/97, initiate lrrysat & lrryrel, a correction/make-up
        lrrysat=0
        lrryrel=0
cc	third counter 3/10/97
	thirdc=0
ccccc


C
C----------------------------------------------------------------------
C
C
C...CALL W3LIB SUBROUTINE DATIME TO FIND THE CURRENT SYSTEM
C   DATE AND TIME (CHARACTER FORMAT).
C
CMZ     CALL DATIM2(CTIME)
      CALL  IDATE(MON,DAY,YR)
      CALL  TIME(CTIME)
      WRITE (6, 1000) MON,DAY,YR,CTIME 
 1000 FORMAT (/1X, 5('*.'),
     2       'CLAVRORB (P1.X5) PROCESSING DATE/TIME ==>  ',
     3        2(I2'-'),I2,1X,A8,3X,15('*.')) 
C
C
C... CALL W3LIB SUBROUTINE W3FQ02 TO FIND THE CURRENT GREENWICH TIME.
C    (START TIME OF CLAVR PROCESSING.)
C    FOR THE GMTIME ARRAY, THE IMPORTANT ELEMENTS ARE:
C
C          (1)  GMT YEAR OF CENTURY           (0-99)
C          (2)  GMT DAY OF YEAR               (1-366)
C          (5)  (GMT HOUR*100) + MINUTES      (0-2359)
C          (6)  (GMT SECS*100) + HUNDREDTHS   (0-5999)
C          (8)  RETURN CODE                   (0-2)
C                 0,1 = SUCCESSFUL COMPLETION
C                   2 = POSS. 1 HOUR ERROR
C
C...PRINT OUTPUT REPORT HEADING MESSAGE.
C
CMZ     WRITE(6,900) VERSON, CTIME
C
C
C...CALL SUBROUTINE TO READ AND QC INPUT PARAMETERS.
C
      CALL CLAVIN
C
C     OPEN(UNIT=UNITMF,FILE='data/monitor.dat',STATUS='UNKNOWN')
      IF(MAKSTI) THEN
C       OPEN(UNIT=IUIMAG,FILE='IMGDAT1C.SGI.DATA',FORM='UNFORMATTED')
C       OPEN(UNIT=IUFOOT,FILE='IMGDAT1C.SGI.FOOTER',FORM='UNFORMATTED')
      END IF

C
C...CALL SUBROUTINES TO OPEN 1B AND 1C FILES.  IF A 1B PRIME FILE IS
C   TO BE CREATED, CALL SUBROUTINE TO WRITE HEADER INFO TO THE FILE.
C
      CALL OPEN1B (TBMHED, CGACHD, TAPDSK)
C
C     CALL DPARAM(DDAYS)
CMZ      IF (BPRIME) CALL WBPHED (TBMHED, CGACHD, TAPDSK)
C
      IF (DISK1C .OR. TAPE1C) THEN
         CALL OPEN1C (PROGRM, DISK1C, HOUSE, BEG1C)
C
        IF (BEG1C .EQ. -1) THEN
C                                ! /* -1 = NEW SYSTEM STARTUP */
          BEG1C = 2
          C1REC = 2
        ELSE
          C1REC = BEG1C
        END IF
C
      END IF
C
C
C     INITIALIZE SPACECRAFT CONSTANT DEPENDENT INFORMATION
C
      CALL INSPCN
C     CALL PRINTCON
C
C...CALL SOLCMP TO COMPUTE SOLAR PARAMETERS.
C
      CALL SOLCMP (SED,DDAY0)
C
      CALL DPARAM(DDAYS,DDAY0)
C     WRITE(*,*)'# DAYS SINCE LAUCH', DDAYS, DDAY0
C
C
C... IF CREATING A 1C FILE ON TAPE, CALL SUBROUTINES "HKPREP" AND
C    "W1CHED" TO PREPARE AND WRITE THE 1C HOUSEKEEPING AND ORBIT
C    HEADER RECORDS TO THE TAPE FILE.
C
      IF (TAPE1C) THEN
        CALL HKPREP
        CALL W1CHED (-1, SED)
      END IF
C
C
C.. CALL SUBROUTINE WRLIMS TO PRINT OUT QC LIMITS.
C
      CALL WRLIMS
C
C..................................................
C
C
C   BYPASS SCANLINES NOT WANTED
C
      IF (ORBIT.EQ.0) THEN
C
         IF (IBSTRT.GT.NUMSCN) THEN
           CALL EMESG (4, 'CLAVRORB', 'FATAL')
           GO TO 999
         END IF
C
C
         IF (IBSTRT.GT.1) THEN
C
            DO 550 ITIMES = 1, ISKIP
              IF (GAC) THEN
               READ (UNIT=UNIT1B,END=701,ERR=540) GACREC
C
              ELSE
C
               LACIDX = INT(L1BWDS / 2)
               READ (UNIT=UNIT1B,END=701,ERR=540)
     1               (AVHLAC(JWORD), JWORD = 1, LACIDX)
C
               LACIDX = LACIDX + 1
               READ (UNIT=UNIT1B,END=701,ERR=540)
     1               (AVHLAC(JWORD), JWORD = LACIDX, L1BWDS)
C
              END IF
C
              GO TO 550
C
  540         CONTINUE
C                                   !  /* DATA RECORD READ ERROR */
              CALL EMESG (15, 'CLAVRORB', 'OK')
C
  550       CONTINUE
C
         END IF
C
C        WRITE(6,955) ISKIP
C
      ELSE
         IBEND = NUMSCN
         IBSTRT = 1
      END IF
C
C
C...LOOP OVER AN ORBITS WORTH OF AVHRR RECORDS
C
C     WRITE (6,*)
C     WRITE (6,*) ' MAIN:  STARTING AND ENDING SCANLINES TO ',
C    1            'PROCESS ==> ', IBSTRT, ' TO ', IBEND
C     WRITE (6,*)
C
      NSLINE = 0
C
      DO 700 IBLOCK = 1, (IBEND-IBSTRT+1)/2
C
         IF ( ( GAC .AND. (MOD(IBLOCK,20).EQ.1)) .OR.
     1        ( LAC .AND. (MOD(IBLOCK, 2).EQ.1)) )    THEN
            FLAG = .TRUE.
         END IF
C
         IF (MOD(IBLOCK,2).EQ.1) THEN
            FLAG1 = .TRUE.
         ELSE
            FLAG1 = .FALSE.
         END IF
C
C
C...READ A RECORDS WORTH OF GAC DATA (2 SCANLINES)
C
         IF (GAC) THEN
C
           READ (UNIT=UNIT1B,END=701,ERR=690) GACREC
C
         END IF
C
C
C... RECORD PROCESSING LOOP .........................................
C....................................................................
C
C
C... INITIALIZE NECESSARY ARRAYS
C
         CALL INARRY (CH, FLAG1, CLDCOD, CLRCLD)
C
C
         DO 600 IREC = 1, 2
C
C
C... INCREMENT COUNTER FOR THE NUMBER OF SCANLINES PROCESSED.
C
          IXSCAN = IXSCAN + 1
C
C
C...READ IN TWO RECORDS WORTH OF LAC DATA (1 SCANLINE)
C
          IF (LAC) THEN
C
           LACIDX = INT(L1BWDS / 2)
           READ (UNIT=UNIT1B,END=701,ERR=690)
     1           (AVHLAC(JWORD), JWORD = 1, LACIDX)
C
           LACIDX = LACIDX + 1
           READ (UNIT=UNIT1B,END=701,ERR=690)
     1           (AVHLAC(JWORD), JWORD = LACIDX, L1BWDS)
C
          END IF
C
          NSLINE = NSLINE+1
C
C...LOOP THRU DATA PIXELS. CALCULATE EARTH LOCATION
C...VIEWING GEOMETRY, ALBEDOS AND TEMPERATURES FOR
C...EACH PIXEL.
C
          CALL AVHFDC (GACREC(1,IREC), AVHLAC, ATVALU, ISCAN(IREC),
     1                 IREC, PRTST,FLAG,FLAG1,DDAYS,COUNT1,GOTTBB)
C
C
C...STORE LAT/LON AT 51 ANCHOR POINTS FOR THIS SCAN IN THE
C   2-SCAN EARTH LOCATION ARRAY (I4LOC2).
C
          DO 500 IL = 1,MAXANC
            I4LOC2(IL,IREC) = I4LOC(IL)
  500     CONTINUE
C
C
C... PIXEL PROCESSING LOOP ..........................................
C....................................................................
C
C...LOAD ARRAYS FOR CLAVR PROCESSING.
C
            DO 560 IPP = IPBEG, IPEND
C
               IP = IPP - IPBEG + 1
C
C --- STORE CHANNEL COUNT FOR ROUTINE CLAVRM
               CH1COUNT(IP, IREC) = COUNT1(IPP)
C
               DO 559 IC = 1, 6
                 CH(IP, IC, IREC) = ATVALU(IPP, IC)
  559          CONTINUE
C
               ANGLES(IP,IREC,1) = COSSZA(IP) * 1000.0
               ANGLES(IP,IREC,2) = SATZ(IPP)
               ANGLES(IP,IREC,3) = RELZ(IPP)
C
               RLAT(IP,IREC) = SPLAT(IPP)
               RLON(IP,IREC) = SPLON(IPP)
C
  560       CONTINUE
C
  600    CONTINUE
C                                 !  /* GO ON TO 2ND SCAN IN PAIR */
C
C
C...IF THE CONDITION CODES RETURNED FROM AVHFDC INDICATE AN ERROR,
C   ZERO OUT THE CLAVR FLAG ARRAYS AND SKIP INVOCATION OF THE
C   CLAVR ALGORITHM.  ALSO, INCREMENT THE COUNTER FOR THE NUMBER OF
C   2X2 ARRAYS FOR WHICH "NO DECISION" COULD BE MADE.
C   ALSO, SET THE CLAVR "EDIT" FLAG INCORPORATED WITH THE GAC 1B
C   QUALITY INDICATORS "ON" (I.E., EQUAL TO 1).
C   IF NO ERRORS ARE PRESENT, INVOKE CLAVR ROUTINES.
C

ccc	luoad
c	write(6,*)ixscan, (adflag(luo), luo=1,2)
ccc


         IF (FLAGQ) THEN
C
           TOTCCM (0) = TOTCCM (0) + INT(2*IMGPIX/4)
C
           DO 611 IR = 1, 2
C
C             WRITE (6,*) '  ** BAD SCAN FLAGGED AT #', ISCAN(IR)
C
              IQTEMP = IQUAL(IR)
              IQTEMP = ISHFT (ISHFT (IQTEMP,-1), 1) + 1
              IQUAL(IR) = IQTEMP
C
              QFLAG(16) = QFLAG(16) + 1
C
              DO 610 IP = 1, IMGPIX
                CLDCOD(IP,IR) = 0
                CLRCLD(IP,IR) = 0
  610         CONTINUE
C
  611      CONTINUE
C
C NOTE: Minze V. Chien 11/23/93
C       . commented the following line to use FLAGQ to test
C         and avoid writing BAD SCAN LINE to 1C
C         (see note on the later CALL to W1CDAT)
C       . FLAGQ would be reset to .FALSE. after the test before
C         calling W1CDAT
C
C           FLAGQ = .FALSE.
C
C
         ELSE
C
           CALL CLAVRM (CH, ANGLES, RLAT, RLON, CLDCOD, CLRCLD, ISCAN,
     *                  CH1COUNT, GOTTBB)
C
         END IF
C
C
C...IF USER CHOOSES TO HAVE TSO/TEMPORARY FILES CREATED FOR
C   PC DOWNLOAD/IMAGING, CALL SUBROUTINE 'DOWNLD' TO WRITE THE
C   ALBEDOS, TEMPERATURES, COLOR CODES, CLEAR/CLOUD FLAGS, AND
C   AND COSINE OF THE SOLAR ZENITH ANGLE TO THE APPROPRIATE
C   TEMPORARY DATA SETS.
C
C        IF (MAKPCI)
C    1      CALL DOWNLD (NPCFIL, CH, CLDCOD, CLRCLD, ANGLES)
C
C
C...IF USER CHOOSES TO HAVE A DIAGNOSTIC DUMP OF SCAN INFO PRODUCED
C   AND THE APPROPRIATE RANGE OF SCANLINES IS BEING PROCESSED,
C   CALL SUBROUTINE TO PRINT OUT THE NECESSARY INFORMATION.
C
      IF (DUMPRN) THEN
        IF ((IXSCAN .GE. IDBEG) .AND. (IXSCAN .LE. IDEND)) THEN
          CALL DUMPIT (CH, ANGLES, ISCAN, RLAT, RLON, CLDCOD,
     1                 CLRCLD)
        END IF
      END IF
C
C
C...ENCODE CLOUD CODE INTO GAC VIDEO DATA IF 1B' FILE DESIRED.
C
         IF (GAC .AND. (BPRIME))
     1      CALL WBPDAT (GACREC, CLDCOD, NBPREX)
C
C
C...WRITE 3 ALBEDOS, 3 TEMPS, 3 ANGLES, L/S TAG, AND PASS1 CODE TO TAPE
C...WRITE ONE SCAN AT A TIME
C...SHIFT ALL BUT THE ANGLES BY 4 BITS (16); SHIFT ANGLES BY 7 BITS(128)
C
C
C-------------------------------------
C...PROCESS FOR WORKSTATION IMAGE FILE
C-------------------------------------
         IF (MAKSTI) THEN
           CALL IMGDAT(ISCAN,IBLOCK, CH, ANGLES,RLAT,RLON,
     *        CLRCLD, CLDCOD)
           IVREX = IVREX + 1
C
         END IF
C
C---------------------------
C...PROCESS FOR 1C DISK/TAPE
C---------------------------
C
C
C  NOTE: Minze V. Chien 11/23/93
C   1. Use the quality control flag FLAGQ to check if the SCAN LINE
C      is BAD or not
C   2. For BAD SCAN LINE do NOT call W1CDAT to write the output
C      because some bad data will cause integer overflow when
C      W1CDAT tries to encode them using ISHFT function
C
C
         IF (FLAGQ) THEN
            FLAGQ = .FALSE.
         ELSE
C
            IF ((DISK1C) .OR. (TAPE1C)) THEN
              CALL W1CDAT (DISK1C, C1REC, ISCAN, CLRCLD,
     1                    CLDCOD, CH, ANGLES, I4LOC2, N1CREX)
            END IF
         END IF
C
C
         FLAG = .FALSE.
C
         GO TO 700
C
C
  690   CONTINUE
C                             !  /* DATA RECORD READ ERROR */
        CALL EMESG (15, 'CLAVRORB', 'OK')
C
C
C.....GO ON TO NEXT DATA BLOCK.
C
  700 CONTINUE
C
C
C... END OF ORBIT PROCESSING...........................................
C
C
  701 CONTINUE
C
C
C... UPDATE 1C HOUSEKEEPING AND ORBIT HEADER RECORDS.
C
      IF (DISK1C) THEN
         END1C = C1REC
         CALL W1CHK  (PROGRM, BEG1C, END1C, HOUSE)
         CALL W1CHED (DISK1C, BEG1C, SED)
      END IF
C
C
C...CREATE IMAGE/PREVIEW FILE.
C
C     IF ((MAKSTI) .AND. (GAC)) THEN
C
C        CALL PREVEW
C
C        WRITE(6,926)
C        WRITE (6,*) '   ++++',IVREX,' RECORDS SUCCESSFULLY WRITTEN',
C    1               ' TO IMAGE FILE   ++++'
C
C     END IF
C
C
C...IF CREATING FILES FOR DOWNLOAD/IMAGING, CALL SUBROUTINE
C   FOOTER TO CREATE A 'FOOTER' BUFFER WITH HOUSEKEEPING INFORMATION.
C
C     IF ( (MAKSTI) .OR. (MAKPCI) ) THEN
C       CALL FOOTER (CTIME, VERSON)
C     END IF
C
C
C...CALL SUBROUTINE TO GENERATE OUTPUT REPORTS.
C
      CALL REPCLV(N1CREX)
C
C
C...CALL SUBROUTINE CLOSFIL TO CLOSE ALL OUTPUT FILES.
C
      CALL CLOSFL (NBPREX, N1CREX)

ccccc   2/28/97 STOWE, # pix w/ bad sat angle & good cc
	write(6,*)'Implemented Mar 10, 1997:'
        write(6,*)'Fix to remove bad sza with cc ne 0:',lrrysat, 
     &' Arrays.'
        write(6,*)'Fix to remove bad rel with cc ne 0:',lrryrel, 
     &' Arrays.'
        write(6,*)
     &'# nadir pix w/ relz reset from 204.7 to 90 & cc ne 0:', 
     &thirdc,' Pixels.'


C
C
C...GENERATE NORMAL COMPLETION MESSAGE.
C
C     WRITE(6,928)
C
C
C
  999 CONTINUE
C
      STOP
C
C
C.....FORMAT CARDS
C
  900 FORMAT(///,1X,130(1H*),//,20X,'C L A V R O R B  -- CLAVR ',
     1 'METHOD OF CLOUD DETECTION:  VERSION - ', A10, /, 20X,
     2 '     CURRENT PROCESSING DATE/TIME...  ',
     3 7A4, //, 1X, 130(1H*), /)
  928 FORMAT(//,5X,'***   F I N I   ***',//)
  955 FORMAT(/,5X,'###   SKIPPING',I6,' PAIRS OF SCAN LINES   ###',/)
  958 FORMAT(1H0,': UNEXPECTED END-OF-FILE.  THIS DATA',
     > ' REQUEST SKIPPED.')
  974 FORMAT(2F6.2)
  975 FORMAT(11F6.2)
  989 FORMAT(/,5X,'XXX E R R O R   READING FROM UNIT 20',/)
C
      END
      INCLUDE "inc/bclav.f"
