      SUBROUTINE DAYLND (P, ANGL, ALAT, ALON, THR, CC, CO)              
C********************************************************************** 
C                                                                       
C    FUNCTION - SUBROUTINE CLAVR WILL EXECUTE THE PASS 1 CLOUD          
C     DETECTION ALGORITHMS THAT WERE DEVELOPED ON THE VICOM FOR         
C     AVHRR LAC DATA. THE ALGORITHMS CAN BE APPLIED TO AVHRR GAC        
C     DATA AS WELL.                                                     
C                                                                       
C               ***** DAYTIME LAND *****                                
C                                                                       
C    INPUTS:                                                            
C     P       2X2 ARRAY OF CHANNELS 1,2,3 ALBEDOS AND CHANNELS          
C             3,4,5 TEMPERATURES.                                       
C     ANGL    ARRAY OF SOLAR, SATELLITE AND RELATIVE AZIMUTH ANGLES     
C             FOR THE SAME 2X2 ARRAY AS ABOVE                           
C     ALAT    MEAN LATITUDE FOR 2X2 PIXEL ARRAY                         
C     ALON    MEAN LONGITUDE FOR 2X2 PIXEL ARRAY                        
C     THR     ARRAY OF 10 THRESHOLDS WHICH WERE PREDETERMINED TO BE     
C             EITHER FOR DAY/NIGHT AND LAND/SEA                         
C                                                                       
C                                                                       
C    OUTPUT PARAMETERS                                                  
C     CC      COLOR CODE OF PASS 1 TEST:                                
C                                                                       
C                  1 : RED       :O/L: CLOUDY RGCT > C3AR               
C                  2 : RED       :O  : CLOUDY RGCT > TUR (T4)           
C                  4 : GREEN     :O/L: CLOUDY RRCT > TUR                
C                  5 : YELLOW    :O/L: CLOUDY C3AT > TUR                
C                  6 : PINK      :O/L: CLOUDY FMFT                      
C                  7 : LT PURPLE :O/L: CLOUDY TUT                       
C                  8 : LT BLUE   :O/L: CLOUDY TGCT                      
C                  9 : PINK      :O/L: CLOUDY ICE FMFT                  
C                 11 : LT BLUE   :  L: CLOUDY ICE TGCT                  
C                                                                       
C                 12 : BLACK     :O/L: CLEAR                            
C                 13 : WHITE     :O/L: CLEAR RGCT                       
C                 14 : GREY      :O/L: CLEAR RUT                        
C                 15 : OLIVE     :O/L: CLEAR RRCT                       
C                 16 : DK PURPLE :O/L: CLEAR C3AT                       
C                                                                       
C                 17 : RED       :O/L: MIXED RGCT > C3AR                
C                 18 : RED       :O  : MIXED RGCT > TUR (T4)            
C                 19 : BLUE      :O/L: MIXED RUT  > TUR                 
C                 20 : GREEN     :O/L: MIXED RRCT > TUR                 
C                 21 : YELLOW    :O/L: MIXED C3AT > TUR                 
C                 22 : PINK      :O/L: MIXED FMFT                       
C                 23 : LT PURPLE :O/L: MIXED TUT                        
C                 24 : LT BLUE   :O/L: MIXED TGCT                       
C                 25 : PINK      :O/L: MIXED ICE FMFT                   
C                 26 : LT PURPLE :O/L: MIXED ICE TUT                    
C                 27 : LT BLUE   :  L: MIXED ICE TGCT                   
C                                                                       
C                 28 : WHITE     :O/L: CLEAR ICE RGCT                   
C                 29 : GREY      :O/L: CLEAR ICE RUT                    
C                 30 : DK PURPLE :  L: CLEAR C3AT > TGCR                
C                 31 : OLIVE     :  L: CLEAR RRCT > TGCR                
C                                                                       
C      CO      CLEAR/OVERCAST FLAG FOR EACH OF 2X2 PIXEL ARRAY          
C      PF      PASS/FAIL FLAG FOR CLEAR/CLOUDY/MIXED (0/1/2)            
C                                                                       
C                                                                       
C     SUBROUTINES CALLED:                                               
C       RGCT    -   REFLECTANCE GROSS CLOUD TEST                        
C       RUT     -   REFLECTANCE UNIFORMITY TEST                         
C       C3AR    -   CHANNEL 3 ALBEDO RESTORAL TEST                      
C       RRCT    -   CHANNEL 2/CHANNEL 1 RATIO TEST                      
C       C3AT    -   CHANNEL 3 ALBEDO TEST                               
C       TGCR    -   THERMAL GROSS CLOUD RESTORAL TEST (CH 5)            
C       TUR     -   THERMAL UNIFORMITY RESTORAL TEST (CHANNEL 5)        
C       FMF     -   FOUR-MINUS-FIVE TEST (CHANNEL 4-CHANNEL 5)          
C       TUT     -   THERMAL UNIFORMITY TEST (CHANNEL 5)                 
C       TGCT    -   THERMAL GROSS CLOUD TEST (CHANNEL 5)                
C                                                                       
C*********************************************************************  
C                                                                       
      INCLUDE "inc/apath.f"                                                   
C                                                                       
C                                                                       
      CHARACTER*6  CCLASS(0:2)                                          
C                                                                       
      INTEGER*2 ANGL(4,3), CO(4), CC, SETDCC                            
C                                                                       
      INTEGER*4 PF, RS, LNDFLG                                          
C                                                                       
      LOGICAL*4  DESERT, DESCHK                                         
C                                                                       
      REAL*4  P(4,6), THR(10), ALAT, ALON, LNDLIM                       
C                                                                       
C                                                                       
      DATA  CCLASS /'CLEAR ', 'CLOUDY', 'MIXED '/                       
C                                                                       
      DATA  LNDFLG /1/, LNDLIM /60/                                     
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C                                                                       
      OTEST = '        '                                                
      RTEST = '        '                                                
      LTEST = '        '                                                
C                                                                       
C                                                                       
C...CHECK TO SEE IF PIXEL ARRAY LIES IN A DESERT REGION.                
C                                                                       
      DESERT = DESCHK (ALAT, ALON)                                      
C                                                                       
C                                                                       
C...CALL REFLECTANCE GROSS CLOUD TEST; USE CHANNEL 1 ALBEDO SINCE       
C   OVER LAND.                                                          
C                                                                       
      CALL RGCT(P(1,1),PF,THR(1),CO)                                    
      OTEST = 'RGCT  '                                                  
C                                                                       
C                                                                       
C...RGCT: CLEAR                                                         
C                                                                       
      IF (PF.EQ.0) THEN                                                 
C                                                                       
         CALL RUT(P(1,1),PF,THR(2),CO)   
C                                        ! /* USE CH 1 ALBEDO */        
         OTEST = 'RUT   '                                               
C                                                                       
C                                                                       
C...RGCT: RUT: CLEAR                                                    
C                                                                       
C...IF OVER LAND AND THE PIXEL ARRAY IS LOCATED IN A DESERT REGION,     
C   SKIP DOWN TO THE TUT TEST.                                          
C                                                                       
         IF (PF.EQ.0) THEN                                              
C                                                                       
            IF (DESERT) GO TO 10   
C                                  ! /* SKIP RRCT AND C3AT TESTS */     
C                                                                       
            CALL RRCT(P(1,1),P(1,2),PF,CO)                              
            OTEST = 'RRCT  '                                            
C                                                                       
C...RGCT: RUT: RRCT: CLEAR                                              
C                                                                       
            IF (PF.EQ.0) THEN                                           
C                                                                       
C***********CHANGES TO MODULE*******************************************
C                                                                       
CCCCC          CALL C3AT(P(1,3), P(1,6), PF, THR(5), CO)                
C                                                                       
               CALL C3AT(P, PF, THR(5), CO)                             
C                                                                       
C***********************************************************************
C                                                                       
               OTEST = 'C3AT  '                                         
C                                                                       
C...RGCT: RUT: RRCT: C3AT: CLEAR                                        
C                                                                       
   10             CONTINUE                                              
C                                                                       
               IF (PF.EQ.0) THEN                                        
C                                                                       
                  CALL TUT(P(1,5),PF,THR(9),CO)                         
                  OTEST = 'TUT   '                                      
C                                                                       
C...RGCT: RUT: RRCT: C3AT: TUT: CLEAR                                   
C                                                                       
                  IF (PF.EQ.0) THEN                                     
                     CALL FMFT (P(1,5), P(1,6), LNDFLG, PF, CO)         
                     OTEST = 'FMFT  '                                   
C                                                                       
C...RGCT: RUT: RRCT: C3AT: FMF: TUT: CLEAR                              
C                                                                       
                     IF (PF.EQ.0) THEN                                  
                           IF (ABS(ALAT) .LT. LNDLIM) THEN              
                              CALL TGCT(P(1,5),PF,THR(10),CO)           
                              OTEST = 'TGCT  '                          
                           ENDIF                                        
                      ENDIF                                             
                  ENDIF                                                 
C                                                                       
C...RGCT: RUT: RRCT: C3AT: FMF: TUT: TGCT:CLR/BLACK                     
C...RGCT: RUT: RRCT: C3AT: FMF: TUT: TGCT:CLD/LT BLUE                   
C                                                                       
                  RESULT = CCLASS(PF)                                   
C                                                                       
               ELSE                   
C                                     ! /* FAILED C3AT */               
C                                                                       
                  CALL TUR (LNDFLG,P(1,5),ANGL,RS,PF,THR(6),CO)         
C                                                                       
                  IF (RS.EQ.0) THEN                                     
                    RTEST = 'TUR   '                                    
                    CALL FT(P(1,5),P(1,6),THR(10),LNDFLG,ALAT,CO)       
C                                                                       
                  ELSE                                                  
C                                                                       
                    CALL TGCR(P(1,5),RS,PF,THR(7),CO)                   
C                                                                       
                    IF (RS.EQ.0) THEN                                   
                      RTEST = 'TGCR  '                                  
                      CALL TFT(P(1,5),P(1,6),THR(10),LNDFLG,ALAT,CO)    
                    ELSE                                                
                      RESULT = CCLASS(RS)                               
                    ENDIF                                               
C                                                                       
                  ENDIF                                                 
C                                                                       
                ENDIF                                                   
C                                                                       
C...RGCT: RUT: RRCT: CLOUDY                                             
C                                                                       
            ELSE                    
C                                   ! /* FAILED RRCT */                 
              CALL DLRES1 (P, ALAT, ALON, THR, CO, PF, LNDFLG, ANGL)          
            ENDIF                                                       
C                                                                       
C                                                                       
C...RGCT: RUT: LAND :CLOUDY                                             
C                                                                       
         ELSE                       
C                                   ! /* FAILED RUT */                  
              CALL DLRES1 (P, ALAT, ALON, THR, CO, PF, LNDFLG, ANGL)          
         ENDIF                                                          
C                                                                       
C                                                                       
C...RGCT: LAND: CLOUDY                                                  
C                                                                       
      ELSE                          
C                                   ! /* FAILED RGCT */                 
        CALL DLRES1 (P, ALAT, ALON, THR, CO, PF, LNDFLG, ANGL)                
      ENDIF                                                             
C                                                                       
C                                                                       
      CC = SETDCC(LNDFLG)                                               
C                                                                       
C                                                                       
      RETURN                                                            
      END                                                               
