      SUBROUTINE DAYOCN (P, ANGL, ALAT, THR, CC, CO)
C**********************************************************************
C
C    FUNCTION - SUBROUTINE CLAVR WILL EXECUTE THE PASS 1 CLOUD
C     DETECTION ALGORITHMS THAT WERE DEVELOPED ON THE VICOM FOR
C     AVHRR LAC DATA. THE ALGORITHMS CAN BE APPLIED TO AVHRR GAC
C     DATA AS WELL.
C
C
C
C    INPUTS:
C     P       2X2 ARRAY OF CHANNELS 1,2,3 ALBEDOS AND CHANNELS
C             3,4,5 TEMPERATURES.
C     ANGL    ARRAY OF SOLAR, SATELLITE AND RELATIVE AZIMUTH ANGLES
C             FOR THE SAME 2X2 ARRAY AS ABOVE
C     ALAT    MEAN LATITUDE FOR 2X2 PIXEL ARRAY
C     THR     ARRAY OF 10 THRESHOLDS WHICH WERE PREDETERMINED TO BE
C             EITHER FOR DAY/NIGHT AND LAND/SEA
C
C
C    OUTPUT PARAMETERS
C     CC      COLOR CODE OF PASS 1 TEST:
C
C                  1 : RED       :O/L: CLOUDY RGCT > C3AR
C                  2 : RED       :O  : CLOUDY RGCT > TUR (T4)
C                  4 : GREEN     :O/L: CLOUDY RRCT > TUR
C                  5 : YELLOW    :O/L: CLOUDY C3AT > TUR
C                  6 : PINK      :O/L: CLOUDY FMFT
C                  7 : LT PURPLE :O/L: CLOUDY TUT
C                  8 : LT BLUE   :O/L: CLOUDY TGCT
C                  9 : PINK      :O/L: CLOUDY ICE FMFT
C                 11 : LT BLUE   :  L: CLOUDY ICE TGCT
C
C                 12 : BLACK     :O/L: CLEAR
C                 13 : WHITE     :O/L: CLEAR RGCT
C                 14 : GREY      :O/L: CLEAR RUT
C                 15 : OLIVE     :O/L: CLEAR RRCT
C                 16 : DK PURPLE :O/L: CLEAR C3AT
C
C                 17 : RED       :O/L: MIXED RGCT > C3AR
C                 18 : RED       :O  : MIXED RGCT > TUR (T4)
C                 19 : BLUE      :O/L: MIXED RUT  > TUR
C                 20 : GREEN     :O/L: MIXED RRCT > TUR
C                 21 : YELLOW    :O/L: MIXED C3AT > TUR
C                 22 : PINK      :O/L: MIXED FMFT
C                 23 : LT PURPLE :O/L: MIXED TUT
C                 24 : LT BLUE   :O/L: MIXED TGCT
C                 25 : PINK      :O/L: MIXED ICE FMFT
C                 26 : LT PURPLE :O/L: MIXED ICE TUT
C                 27 : LT BLUE   :  L: MIXED ICE TGCT
C
C                 28 : WHITE     :O/L: CLEAR ICE RGCT
C                 29 : GREY      :O/L: CLEAR ICE RUT
C                 30 : DK PURPLE :  L: CLEAR C3AT > TGCR
C                 31 : OLIVE     :  L: CLEAR RRCT > TGCR
C
C      CO      CLEAR/OVERCAST FLAG FOR EACH OF 2X2 PIXEL ARRAY
C      PF      PASS/FAIL FLAG FOR CLEAR/CLOUDY/MIXED (0/1/2)
C
C
C     SUBROUTINES CALLED:
C       RGCT    -   REFLECTANCE GROSS CLOUD TEST
C       RUT     -   REFLECTANCE UNIFORMITY TEST
C       C3AR    -   CHANNEL 3 ALBEDO RESTORAL TEST
C       RRCT    -   CHANNEL 2/CHANNEL 1 RATIO TEST
C       C3AT    -   CHANNEL 3 ALBEDO TEST
C       TGCR    -   THERMAL GROSS CLOUD RESTORAL TEST (CH 5)
C       TUR     -   THERMAL UNIFORMITY RESTORAL TEST (CHANNEL 5)
C       FMF     -   FOUR-MINUS-FIVE TEST (CHANNEL 4-CHANNEL 5)
C       TUT     -   THERMAL UNIFORMITY TEST (CHANNEL 5)
C       TGCT    -   THERMAL GROSS CLOUD TEST (CHANNEL 5)
C
C
C     RESTRICTIONS-
C
C       ORIGINAL (PRIMARY) TEST NAMES ARE SET IMMEDIATELY AFTER THE
C       TEST IS CALLED REGARDLESS OF THE RESULT OF THE TEST.
C
C       RESTORAL TEST NAMES ARE SET AFTER THE TEST IS CALLED, BUT
C       * ONLY * IF THE 2X2 ARRAY * PASSES * THE RESTORAL.
C
C       "LAST" TEST NAMES INDICATE THE LAST TEST CALLED AFTER
C       SUCCESSFULLY PASSING A RESTORAL.
C
C*********************************************************************
C
      INCLUDE "inc/inclav.f"
      INCLUDE "inc/apath.f"
      INCLUDE "inc/spcnst.f"
C
C
      CHARACTER*6  CCLASS(0:2)
C
      INTEGER*2 ANGL(4,3), CO(4), CC, SETDCC
C
      INTEGER*4 PF, RS, OCNFLG
C
      REAL*4  P(4,6), THR(10), ALAT, GAMMA(4)
C
C

ccccc   luo
        DATA  DTR/0.017453293/


      DATA  OCNFLG /0/
C
      DATA  CCLASS /'CLEAR ', 'CLOUDY', 'MIXED '/
C
C---------------------------------------------------------------------
C
C
C...INITIALIZE THE ORIGINAL, RESTORAL, AND LAST TEST NAMES.
C
      OTEST = '        '
      RTEST = '        '
      LTEST = '        '
C  rao 10/24/96 set CC=0 for excessive specular reflection.
      DO 10 IP = 1, 4
            COSSZA = ANGL(IP, 1) / 1000.
            SAT = DTR*ANGL(IP, 2) / 10.
            REL = DTR*ANGL(IP, 3) / 10.
            SINSZA = SQRT(1.0 - COSSZA*COSSZA)
            IF (REL .LT. 0.0) GO TO 10
                GAMMP = ACOS(COSSZA*COS(SAT)
     *                + SINSZA*SIN(SAT)*COS(REL))
                GAMMA(IP) = GAMMP/DTR
                ANGL1 = (ACOS(COSSZA)*RD2DEG)
                IF (GAMMA(IP) .LT. 20.0 .AND. ANGL1 .GT. 45.0) THEN      
                    CC = 0
                    RETURN
                ENDIF  
   10  CONTINUE
C
C
C...CALL REFLECTANCE GROSS CLOUD TEST; USE CHANNEL 2 ALBEDO SINCE
C   OVER OCEAN.
C
      CALL RGCT(P(1,2),PF,THR(1),CO)
      OTEST = 'RGCT  '
C
C
C...RGCT: CLEAR
C
      IF (PF.EQ.0) THEN
C
         CALL RUT(P(1,2),PF,THR(2),CO)
C                                        ! /* USE CH 2 ALBEDO */
         OTEST = 'RUT   '
C
C
C...RGCT: RUT: CLEAR
C
C...IF OVER LAND AND THE PIXEL ARRAY IS LOCATED IN A DESERT REGION,
C   SKIP DOWN TO THE TUT TEST.
C
         IF (PF.EQ.0) THEN
C
            CALL RRCT(P(1,1),P(1,2),PF,CO)
            OTEST = 'RRCT  '
C
C...RGCT: RUT: RRCT: CLEAR
C
            IF (PF.EQ.0) THEN
C
C********CHANGES TO MODULE**********************************************
C
CCCCCC         CALL C3AT(P(1,3), P(1,6), PF, THR(5), CO)
C
CCCCCC  luo 
               DO 20 IP = 1, 4
                     REL = DTR*ANGL(IP, 3) / 10.
                     IF (REL .LT. 0.0) GO TO 20
                     IF (GAMMA(IP) .LT. 40.) GO TO 7777
   20          CONTINUE
C
               CALL C3AT(P, PF, THR(5), CO)
C
C***********************************************************************
C
               OTEST = 'C3AT  '
C
C...RGCT: RUT: RRCT: C3AT: CLEAR
C
 7777          IF (PF.EQ.0) THEN
C
                  CALL TUT(P(1,5),PF,THR(9),CO)
                  OTEST = 'TUT   '
C
C...RGCT: RUT: RRCT: C3AT: TUT: CLEAR
C
                  IF (PF.EQ.0) THEN
                     CALL FMFT (P(1,5), P(1,6), OCNFLG, PF, CO)
                     OTEST = 'FMFT  '
C
C...RGCT: RUT: RRCT: C3AT: FMF: TUT: CLEAR
C
                     IF (PF.EQ.0) THEN
                       IF (ABS(ALAT).LT.50) THEN
                         CALL TGCT(P(1,5),PF,THR(10),CO)
                         OTEST = 'TGCT  '
C
C...RGCT: RUT: RRCT: C3AT: FMF: TUT: TGCT:CLR/BLACK
C...RGCT: RUT: RRCT: C3AT: FMF: TUT: TGCT:CLD/LT BLUE
C
                       END IF
C
                     END IF
C
                   END IF
C
                   RESULT = CCLASS(PF)
C
               ELSE
C
                  CALL TUR(OCNFLG,P(1,5),ANGL,RS,PF,THR(6),CO)
C
                  IF (RS.EQ.0) THEN
                    RTEST = 'TUR   '
                    CALL FT(P(1,5),P(1,6),THR(10),OCNFLG,ALAT,CO)
                  ELSE
                    RESULT = CCLASS(PF)
                  ENDIF
C
               ENDIF
C
C
            ELSE
C
C...RGCT: RUT: RRCT: CLOUDY
C
C...ATTEMPT TO RESTORE CLOUDY ARRAY TO CLEAR.
C
              CALL DORES1 (P,ANGL,ALAT,THR,CO,PF,OCNFLG)
C
            ENDIF
C
C
         ELSE
C
C...RGCT: RUT: CLOUDY
C
C...ATTEMPT TO RESTORE CLOUDY ARRAY TO CLEAR.
C
           CALL DORES1 (P,ANGL,ALAT,THR,CO,PF,OCNFLG)
C
         ENDIF
C
C
      ELSE
C
C...RGCT: CLOUDY
C
C...ATTEMPT TO RESTORE CLOUDY ARRAY TO CLEAR.
C
           CALL DORES1 (P,ANGL,ALAT,THR,CO,PF,OCNFLG)
C
      ENDIF
C
C
      CC = SETDCC(OCNFLG)
C
C
      RETURN
      END
