      SUBROUTINE DLRES1 (P, ALAT, ALON, THR, CO, PF, LNDFLG, ANGL)            
C********************************************************************** 
C                                                                       
C    FUNCTION - SUBROUTINE CLAVR WILL EXECUTE THE PASS 1 CLOUD          
C     DETECTION ALGORITHMS THAT WERE DEVELOPED ON THE VICOM FOR         
C     AVHRR LAC DATA. THE ALGORITHMS CAN BE APPLIED TO AVHRR GAC        
C     DATA AS WELL.                                                     
C                                                                       
C               ***** DAYTIME LAND RESTORAL PATH *****                  
C                                                                       
C    INPUTS:                                                            
C     P       2X2 ARRAY OF CHANNELS 1,2,3 ALBEDOS AND CHANNELS          
C             3,4,5 TEMPERATURES.                                       
C     ALAT    MEAN LATITUDE FOR 2X2 PIXEL ARRAY                         
C     ALON    MEAN LONGITUDE FOR 2X2 PIXEL ARRAY                        
C     THR     ARRAY OF 10 THRESHOLDS WHICH WERE PREDETERMINED TO BE     
C             EITHER FOR DAY/NIGHT AND LAND/SEA                         
C                                                                       
C                                                                       
C    OUTPUT PARAMETERS                                                  
C     CC      COLOR CODE OF PASS 1 TEST:                                
C                                                                       
C                  1 : RED       :O/L: CLOUDY RGCT > C3AR               
C                  2 : RED       :O  : CLOUDY RGCT > TUR (T4)           
C                  4 : GREEN     :O/L: CLOUDY RRCT > TUR                
C                  5 : YELLOW    :O/L: CLOUDY C3AT > TUR                
C                  6 : PINK      :O/L: CLOUDY FMFT                      
C                  7 : LT PURPLE :O/L: CLOUDY TUT                       
C                  8 : LT BLUE   :O/L: CLOUDY TGCT                      
C                  9 : PINK      :O/L: CLOUDY ICE FMFT                  
C                 11 : LT BLUE   :  L: CLOUDY ICE TGCT                  
C                                                                       
C                 12 : BLACK     :O/L: CLEAR                            
C                 13 : WHITE     :O/L: CLEAR RGCT                       
C                 14 : GREY      :O/L: CLEAR RUT                        
C                 15 : OLIVE     :O/L: CLEAR RRCT                       
C                 16 : DK PURPLE :O/L: CLEAR C3AT                       
C                                                                       
C                 17 : RED       :O/L: MIXED RGCT > C3AR                
C                 18 : RED       :O  : MIXED RGCT > TUR (T4)            
C                 19 : BLUE      :O/L: MIXED RUT  > TUR                 
C                 20 : GREEN     :O/L: MIXED RRCT > TUR                 
C                 21 : YELLOW    :O/L: MIXED C3AT > TUR                 
C                 22 : PINK      :O/L: MIXED FMFT                       
C                 23 : LT PURPLE :O/L: MIXED TUT                        
C                 24 : LT BLUE   :O/L: MIXED TGCT                       
C                 25 : PINK      :O/L: MIXED ICE FMFT                   
C                 26 : LT PURPLE :O/L: MIXED ICE TUT                    
C                 27 : LT BLUE   :  L: MIXED ICE TGCT                   
C                                                                       
C                 28 : WHITE     :O/L: CLEAR ICE RGCT                   
C                 29 : GREY      :O/L: CLEAR ICE RUT                    
C                 30 : DK PURPLE :  L: CLEAR C3AT > TGCR                
C                 31 : OLIVE     :  L: CLEAR RRCT > TGCR                
C                                                                       
C      CO      CLEAR/OVERCAST FLAG FOR EACH OF 2X2 PIXEL ARRAY          
C      PF      PASS/FAIL FLAG FOR CLEAR/CLOUDY/MIXED (0/1/2)            
C                                                                       
C                                                                       
C     SUBROUTINES CALLED:                                               
C       RGCT    -   REFLECTANCE GROSS CLOUD TEST                        
C       RUT     -   REFLECTANCE UNIFORMITY TEST                         
C       C3AR    -   CHANNEL 3 ALBEDO RESTORAL TEST                      
C       RRCT    -   CHANNEL 2/CHANNEL 1 RATIO TEST                      
C       C3AT    -   CHANNEL 3 ALBEDO TEST                               
C       TGCR    -   THERMAL GROSS CLOUD RESTORAL TEST (CH 5)            
C       TUR     -   THERMAL UNIFORMITY RESTORAL TEST (CHANNEL 5)        
C       FMF     -   FOUR-MINUS-FIVE TEST (CHANNEL 4-CHANNEL 5)          
C       TUT     -   THERMAL UNIFORMITY TEST (CHANNEL 5)                 
C       TGCT    -   THERMAL GROSS CLOUD TEST (CHANNEL 5)                
C                                                                       
C*********************************************************************  
C                                                                       
      INCLUDE "inc/apath.f" 
      INCLUDE "inc/inclav.f"                                                          
C                                                                       
C                                                                       
      CHARACTER*6  CCLASS(0:2)                                          
C                                                                       
      INTEGER*2 CO(4), ANGL(4,3)                                                  
C                                                                       
      INTEGER*4 PF, RS, LNDFLG                                          
C                                                                       
      REAL*4  P(4,6), THR(10), ALAT, ALON, LNDLIM                       
C                                                                       
C                                                                       
      DATA  CCLASS /'CLEAR ', 'CLOUDY', 'MIXED '/                       
C                                                                       
      DATA  LNDLIM /60/                                                 
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C                                                                       
      CALL TGCR(P(1,5),RS,PF,THR(7),CO)                                 
C                                                                       
C                                                                       
C...RGCT: RUT: RRCT: TGCR: C3AR: LAND: UNRESTORED CLD/GREEN             
C                                                                       
      IF (RS.EQ.0) THEN                                                 
C                                                                       
        RTEST = 'TGCR  '                                                
        CALL TFT(P(1,5),P(1,6),THR(10),LNDFLG,ALAT,CO)                  
C                                                                       
      ELSE                                                              
C                                                                       
C*********CHANGES TO MODULE*********************************************
C                                                                       
CCCCC   CALL C3AR(P(1,3),RS,PF,THR(3),CO)                               
C                                                                       
        CALL C3AR(P,RS,PF,THR(3),CO)                                    
C                                                                       
C***********************************************************************
C                                                                       
C                                                                       
        IF (RS.EQ.0) THEN                                               
          RTEST = 'C3AR  '                                              
          CALL TFT(P(1,5),P(1,6),THR(10),LNDFLG,ALAT,CO)                
        ELSE                                                            
C                                                                       
C --- CHECK GAMMA ANGLE: FOR LATITUDES 60 SOUTH POLEWARD, IF THE        
C     GAMMA ANGLE < 50 DEGREES, RESTORE TO CLEAR                        
C                                                                       
          IF (ALAT .LT. -60.0) THEN                                     
             CCOUNT = 0                                                 
             DO I = 1, 4                                                
                ANGL1 = (ACOS(ANGL(I,1)/1000.0)*RD2DEG)                 
                ANGL2 = ANGL(I,2)/10.0                                  
                ANGL3 = ANGL(I,3)/10.0                                  
                GAMMA = ACOS(COS(ANGL1*DEG2RD)*COS(ANGL2*DEG2RD)        
     1                + SIN(ANGL1*DEG2RD)*SIN(ANGL2*DEG2RD)             
     1                * COS(ANGL3*DEG2RD))*RD2DEG                       
                IF (GAMMA .LT. 50.0) CCOUNT = CCOUNT + 1                
             END DO                                                     
             IF (CCOUNT .GT.0) THEN                                     
                RTEST = 'C3AR  '                                        
                CALL TFT(P(1,5),P(1,6),THR(10),LNDFLG,ALAT,CO)          
             ELSE                                                       
                RESULT = CCLASS(RS)                                     
             END IF                                                     
          ELSE                                                          
             RESULT = CCLASS(RS)                                        
          END IF                                                        
        ENDIF                                                           
C                                                                       
      ENDIF                                                             
C                                                                       
C                                                                       
      RETURN                                                            
      END                                                               
