      SUBROUTINE DUMPIT (CH, ANGLES, ISCAN, RLAT, RLON, CLDCOD, CLRCLD)  
C                                                                       
C********************************************************************** 
C                                                                       
C   NAME- DUMPIT                                                        
C                                                                       
C   LANGUAGE- VS FORTRAN  TYPE- SUBROUTINE                              
C                                                                       
C   VERSION- 1.0    DATE- 02/04/93   PROGRAMMER- DEBBIE CARR (SMSRC)    
C                                                                       
C                                                                       
C   FUNCTION-                                                           
C                                                                       
C      PRINTS DIAGNOSTIC DUMP OF SCANLINE INFORMATION.                  
C                                                                       
C                                                                       
C   LOCAL PARAMETERS-                                                   
C                                                                       
C      VARIABLE                TYPE  FUNCTION                           
C      --------                ----  --------                           
C      FINLIN (35)             C*1   A FINAL MESSAGE OUTPUT LINE        
C                                                                       
C                                                                       
C                                                                       
C   COMMON AREAS- INCLAV (CONSTANTS), TYPDAT                            
C                                                                       
C                                                                       
C   CALLED BY-  CLAVRORB                                                
C                                                                       
C                                                                       
C   SUBPROGRAMS CALLED- ACOS                                            
C                                                                       
C                                                                       
C   RESTRICTIONS-                                                       
C                                                                       
C                                                                       
C   COMMENTS-                                                           
C                                                                       
C                                                                       
C***********************************************************************
C***********************************************************************
C                                                                       
C                                                                       
C...LINK IN CONSTANT PARAMETERS AND COMMON VARIABLES.                   
C                                                                       
      INCLUDE "inc/inclav.f"                                                  
      INCLUDE "inc/typdat.f"                                                  
      INCLUDE "inc/atmisc.f"                                                  
C                                                                       
C***********************************************************************
C***********************************************************************
C                                                                       
C                                                                       
C...DECLARE VARIABLES.                                                  
C                                                                       
      INTEGER*2  ANGLES(MAXLPX,2,3), CLDCOD(MAXLPX,2),                   
     1           CLRCLD(MAXLPX,2)                                       
C                                                                       
      INTEGER*4  ISCAN(2)                                               
C                                                                       
      REAL*4     CH(MAXLPX,6,2), RLAT(MAXLPX,2), RLON(MAXLPX,2)         
C                                                                       
C***********************************************************************
C***********************************************************************
C                                                                       
C                                                                       
C...PRINT THE FOLLOWING INFORMATION FOR EACH                            
C   PIXEL OF THE 2 SCANLINES CURRENTLY BEING PROCESSED...               
C                                                                       
C         *  SCANLINE                                                   
C         *  PIXEL NUMBER                                               
C         *  LATITUDE                                                   
C         *  LONGITUDE                                                  
C         *  ALBEDO VALUES FOR CHANNELS 1, 2, AND 3                     
C         *  TEMPERATURES FOR CHANNELS 3, 4, AND 5                      
C         *  SOLAR ZENITH, SATELLITE ZENITH, AND RELATIVE AZIMUTH ANGLES
C         *  LAND/SEA TAG ASSIGNMENT                                    
C         *  COAST TAGS                                                 
C         *  CLAVR COLOR CODE                                           
C         *  CH 4 - CH 5                                                
C                                                                       
        DO 20 IR = 1, 2                                                 
C                                                                       
          WRITE (6,1000)                                                
C                                                                       
          DO 10 IP = 1, IPEND-IPBEG+1                                   
C                                                                       
            ALBED1 = CH(IP,1,IR)                                        
            ALBED2 = CH(IP,2,IR)                                        
            ALBED3 = CH(IP,3,IR)                                        
C                                                                       
            TEMP3  = CH(IP,4,IR)                                        
            TEMP4  = CH(IP,5,IR)                                        
            TEMP5  = CH(IP,6,IR)                                        
            DIFF45 = TEMP4 - TEMP5                                      
C                                                                       
            ANGL1  = (ACOS(ANGLES(IP,IR,1)/1000.0) * RD2DEG)             
            ANGL2  = ANGLES(IP,IR,2)/10.0                                
            ANGL3  = ANGLES(IP,IR,3)/10.0                                
C                                                                       
            WRITE (6,1010) ISCAN(IR), IP,                               
     1                    RLAT(IP,IR), RLON(IP,IR),                     
     2                    ALBED1, ALBED2, ALBED3,                       
     3                    TEMP3,  TEMP4,  TEMP5,                        
     4                    ANGL1, ANGL2, ANGL3,                          
     5                    DIFF45,                                       
     6                    CLDCOD(IP,IR), CLRCLD(IP,IR),                 
     7                    LSBUF(IP,IR), COASTG(IP,IR),                  
     8                    DAYNIT(IP,IR), ADFLAG(IR)                     
C                                                                       
   10    CONTINUE                                                       
   20  CONTINUE                                                         
C                                                                       
C                                                                       
      RETURN                                                            
C                                                                       
 1000 FORMAT(1H1,                                                       
     1   2X, 62('*-'), '*', /,                                          
     2   2X, '                          CH1   CH2   CH3   CH3   ',      
     3 'CH4   CH5   SOLR  SAT  REL    CH4-  COLR  CLR/ ', /,            
     4   2X, ' SCAN PIX  LAT     LON    ALB   ALB   ALB   TMP   TMP  ', 
     5 ' TMP   ZEN   ZEN  AZMTH  CH5   CODE  CLD   L/S  CST  D/N  A/D', 
     6 /,3X, 62('*-'), '*', /)                                          
C                                                                       
 1010 FORMAT(1X, I5, 1X, I3, 2F8.2, 9(1X,F5.1), 1X, F5.1, I5, I5,       
     1  3X, I2, 3X, I2, 3X, I2, 3X, I2)                                 
C                                                                       
      END                                                               
