      SUBROUTINE FMFR (PIX4, PIX5, LSTAG, RS, PF, CO)                   
C                                                                       
C********************************************************************** 
C                                                                       
C   NAME- FMFR                           EXPIRES- 12/31/99              
C                                                                       
C   LANGUAGE- VS FORTRAN  TYPE- SUBROUTINE                              
C                                                                       
C   VERSION- 1.0    DATE- 02/20/92   PROGRAMMER- DEBBIE CARR (SMSRC)    
C                                                                       
C   FUNCTION-*$ED23DC*-  AVHRR CHANNEL 4 MINUS CHANNEL 5 RESTORAL TEST. 
C                                                                       
C                                                                       
C   CALLING SEQUENCE PARAMETERS-                                        
C                                                                       
C      VARIABLE         I/O   TYPE   FUNCTION                           
C      --------         ---   ----   --------                           
C      PIX4 (4)          I    R*4    CHANNEL 4 TEMPS - ALL FOUR PIXELS  
C      PIX5 (4)          I    R*4    CHANNEL 5 TEMPS - ALL FOUR PIXELS  
C      LSTAG             I    I*4    LAND/SEA TAG                       
C      RS                O    I*4    RESTORAL FLAG (0 IF RESTORED)      
C      PF                O    I*4    PASS/FAIL FLAG FOR PIXEL ARRAY     
C                                    (0=CLEAR,1=CLOUDY,2=MIXED)         
C      CO (4)           I/O   I*2    CLEAR/CLOUD FLAG FOR               
C                                    INDIVIDUAL PIXELS                  
C                                    (0=CLEAR,1=CLOUDY)                 
C                                                                       
C                                                                       
C   LOCAL PARAMETERS-                                                   
C                                                                       
C      VARIABLE               TYPE   FUNCTION                           
C      --------               ----   --------                           
C      DLIMIT                 R*4    CH4-CH5 DIFFERENCE LIMIT (THRSHLD) 
C      T4HIGH                 I*4    UPPER LIMIT FOR CHANNEL 4 TEMPS    
C      T4LOW                  I*4    LOWER LIMIT FOR CHANNEL 4 TEMPS    
C      DIF                    R*4    CH4-CH5 TEMPERATURE DIFFERENCE     
C      ICT                    I*4    COUNTER FOR THE NUMBER OF          
C                                    PIXELS WHICH FAIL THE CH4-CH5      
C                                    RESTORAL TEST                      
C      ICH4T                  I*4    CHANNEL 4 TEMPERATURE ROUNDED      
C                                    TO THE NEAREST INTEGER             
C                                                                       
C                                                                       
C   COMMON AREAS- DAYLU                                                 
C                                                                       
C                                                                       
C   INCLUDE/PARAMETER AREAS-                                            
C                                                                       
C                                                                       
C   CALLED BY- CLAVRN                                                   
C                                                                       
C                                                                       
C   SUBPROGRAMS CALLED- (NINT-INTRINSIC FUNCTION)                       
C                                                                       
C                                                                       
C   RESTRICTIONS-                                                       
C                                                                       
C                                                                       
C   COMMENTS-                                                           
C                                                                       
C                                                                       
C********************************************************************** 
C                                                                       
C                                                                       
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
      INCLUDE "inc/daylu.f"                                                   
C                                                                       
C                                                                       
C... CALLING SEQUENCE VARIABLE DECLARATIONS                             
C                                                                       
      INTEGER*2  CO(4)                                                  
      INTEGER*4  RS, PF, LSTAG                                          
C                                                                       
      REAL*4     PIX4(4), PIX5(4)                                       
C                                                                       
C                                                                       
C... LOCAL VARIABLE DECLARATIONS                                        
C                                                                       
      INTEGER*4  ICH4T, ICT, T4HIGH, T4LOW                              
C                                                                       
      REAL*4     DIF                                                    
C                                                                       
C                                                                       
C... LOCAL VARIABLE INITIALIZATIONS                                     
C                                                                       
      DATA  T4HIGH /320/, T4LOW /200/                                   
C                                                                       
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
C                                                                       
C                                                                       
C... INITIALIZE COUNTER FOR # OF TIMES DIFFERENCE LIMIT IS EXCEEDED.    
C                                                                       
      ICT=0                                                             
C                                                                       
C                                                                       
C... LOOP THROUGH THE FOUR AVHRR/GAC PIXELS.                            
C                                                                       
      DO 10 I = 1, 4                                                    
C                                                                       
C                                                                       
C... IF CHANNEL 5 IS SATURATED (I.E., TEMPERATURE EXCEEDS THRESHOLD),   
C    SET THE RESTORAL FLAG (RS) EQUAL TO THE VALUE OF THE PASS/FAIL     
C    FLAG (PF) WHICH WAS INPUT TO THIS SUBROUTINE, THEN EXIT SINCE      
C    A RESTORAL OF A PIXEL ARRAY TO 'CLEAR' CANNOT BE PERFORMED         
C    IF CHANNEL 5 IS SATURATED.                                         
C                                                                       
        IF ( PIX5(I) .GT. 315.0 ) GO TO 10                              
C       IF ( PIX5(I) .GT. 315.0 ) THEN                                  
C         RS = PF                                                       
C         GO TO 30                                                      
C       END IF                                                          
C                                                                       
C                                                                       
C... ROUND THE CHANNEL 4 TEMPERATURE TO THE NEXT INTEGER VALUE, THEN    
C    TEST THE VALUE AGAINST UPPER AND LOWER LIMITS.  IF THE CH4 TEMP    
C    EXCEEDS THE UPPER LIMIT, RESET THE TEMP TO THE UPPER LIMIT.  IF    
C    THE LOWER LIMIT IS EXCEEDED, RESET TO THAT LIMIT.  ROUNDING OF     
C    THE CHANNEL 4 TEMPERATURE TO THE NEAREST INTEGER VALUE IS          
C    NECESSARY FOR DETERMINING WHICH THRESHOLD TO USE FOR THE           
C    CH4-CH5 DIFFERENCE TEST.                                           
C                                                                       
         ICH4T = NINT (PIX4(I))                                         
C                                                                       
         IF (ICH4T .GT. T4HIGH) THEN                                    
           ICH4T = T4HIGH                                               
         ELSE IF (ICH4T .LT. T4LOW) THEN                                
           ICH4T = T4LOW                                                
         ENDIF                                                          
C                                                                       
C                                                                       
C... DETERMINE THE APPROPRIATE DIFFERENCE THRESHOLD TO USE BASED        
C    ON THE ROUNDED CHANNEL 4 TEMPERATURE VALUE AND WHETHER THE         
C    PIXEL IS OVER LAND (LSTAG=1) OR OCEAN (LSTAG=0).                   
C                                                                       
         IF (LSTAG .EQ. 1) THEN                                         
           DLIMIT = FMFLND(ICH4T)                                       
         ELSE                                                           
           DLIMIT = FMFOCN(ICH4T)                                       
         END IF                                                         
C                                                                       
C                                                                       
C... COMPUTE THE DIFFERENCE BETWEEN THE CHANNEL 4 AND 5 TEMPERATURES    
C    AND TEST TO SEE IF THE DIFFERENCE EXCEEDS THE THRESHOLD.           
C    INCREMENT THE 'ICT' COUNTER FOR EACH THRESHOLD FAILURE.            
C                                                                       
         DIF = PIX4(I) - PIX5(I)                                        
C                                                                       
         IF (DIF .GT. DLIMIT) ICT = ICT + 1                             
C                                                                       
C                                                                       
C... GO ON TO NEXT ARRAY PIXEL.                                         
C                                                                       
   10 CONTINUE                                                          
C                                                                       
C                                                                       
C... IF  ** ALL FOUR **  PIXELS PASSED THE CH4-CH5 TEST, RESTORE ALL    
C    OF THE CLEAR/OVERCAST (CO) FLAGS FOR THE ARRAY TO '0' (CLEAR),     
C    AND RESET THE PASS/FAIL (PF) AND RESTORAL (RS) FLAGS TO '0'.       
C                                                                       
      IF (ICT .EQ. 0) THEN                                              
C                                                                       
        DO 20 I = 1, 4                                                  
          CO(I) = 0                                                     
   20   CONTINUE                                                        
C                                                                       
        PF = 0                                                          
        RS = 0                                                          
C                                                                       
      ELSE                                                              
C                                                                       
C...... ELSE, IF ALL FOUR PIXELS DID NOT PASS, SET THE RESTORAL         
C       FLAG EQUAL TO THE PASS/FAIL FLAG WHICH WAS INPUT TO             
C       THIS ROUTINE.  THE CLEAR/OVERCAST FLAGS WILL RETAIN             
C       THE SAME VALUES ASSIGNED PRIOR TO THIS ROUTINE.                 
C                                                                       
        RS = PF                                                         
C                                                                       
      END IF                                                            
C                                                                       
C                                                                       
   30 CONTINUE                                                          
C                                                                       
C                                                                       
      RETURN                                                            
      END                                                               
