      SUBROUTINE FT(T4, T5, THR10, LST, ALAT, CO)                       
C********************************************************************** 
C                                                                       
C     PROGRAM: FT (FMFT, TGCT)                                          
C                                                                       
C     PURPOSE: IN THE DAYTIME PASS1 OCEAN ALGORITHM, A PIXEL            
C              FAILING EITHER THE RGCT, RUT, RRCT OR C3AT AND BEING     
C              RESTORED BY THE TUR TEST                                 
C           OR IN THE DAYTIME PASS1 LAND ALGORITHM AFTER THE TUT TEST   
C              HAS BEEN PASSED THE FMFT AND TGCT TESTS ARE NEXT IN LINE 
C                                                                       
C              ** ONLY CALLED FOR DAYTIME SCENES **                     
C                                                                       
C     CHANGES FOR CLAVR ALGORITHM VERSION A.3H                          
C     ----------------------------------------------------------------- 
C     (1)  LAND/SEA TAG NOW PASSED TO SUBROUTINE FMFT SINCE TERRAIN-    
C          DEPENDENT (LAND VS. OCEAN) THRESHOLDS ARE NOW BEING USED     
C          IN THE CH4 - CH5 TEST.                                       
C                                                                       
C     (2)  FOR DAYTIME/OCEAN AREAS, CALL TO TGCT IS BYPASSED IF         
C          ABS(LAT) IS GREATER THAN OR EQUAL TO 50.  IF ABS(LAT)        
C          IS LESS THAN 50 (I.E., EQUATORWARD OF 50), THE PIXEL         
C          ARRAY WILL BE LABELED CLEAR.                                 
C                                                                       
C                                                                       
C     PROGRAMMER CRAIG S. LONG (SSAI)                                   
C                                                                       
C     DATE: FEBRUARY, 1990                                              
C                                                                       
C     INPUTS:                                                           
C             TEST     I*4    FLAG TELLING WHICH CLOUD TEST THIS        
C                             SUBROUTINE IS ENTERED FROM                
C                             1 = RGCT                                  
C                             2 = RUT                                   
C                             3 = RRCT                                  
C                             4 = C3AT                                  
C                                                                       
C             T4       R*4    CHANNEL 4 TEMPERATURES OF 2X2 ARRAY       
C             T5       R*4    CHANNEL 5 TEMPERATURES OF 2X2 ARRAY       
C             THR10    R*4    THRESHOLD FLAG FOR TGCT TEST              
C             LST      I*4    LAND/SEA TAG                              
C             ALAT     R*4    MEAN LATITUDE OF 2X2 PIXELS               
C                                                                       
C     OUTPUTS:                                                          
C             CC       I*2    CLOUD CODE                                
C             CO       I*2    CLEAR/CLOUD FLAG                          
C                                                                       
C********************************************************************** 
C                                                                       
C                                                                       
      INCLUDE "inc/apath.f"                                                 
C                                                                       
C                                                                       
      CHARACTER*6  CCLASS(0:2)                                          
C                                                                       
      REAL T4(4), T5(4), THR10, ALAT                                    
C                                                                       
      INTEGER*2  CO(4)                                                  
      INTEGER*4  PF, LST                                                
C                                                                       
C                                                                       
      DATA  CCLASS/'CLEAR ', 'CLOUDY', 'MIXED '/                        
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
      CALL FMFT(T4, T5, LST, PF, CO)                                    
      LTEST = 'FMFT  '                                                  
      IF (PF.EQ.0) THEN                                                 
C                                                                       
C...TUT: FMF: TGCT: CLR/GREY                                            
C...TUT: FMF: TGCT: CLD/LT BLUE                                         
C                                                                       
         IF (LST.EQ.1) THEN                     
C                                               ! /* LAND PATH */       
            IF (ABS(ALAT).LT.60) THEN                                   
               CALL TGCT(T4,PF,THR10, CO)                               
               LTEST = 'TGCT  '                                         
C                                                                       
C...TUT: FMF: LAND: SUBARTIC: TGCT:CLR/CLEAR                            
C...TUT: FMF: LAND: SUBARTIC: TGCT:CLD/LT BLUE                          
C                                                                       
            ENDIF                                                       
C                                                                       
            RESULT = CCLASS(PF)                                         
C                                                                       
C                                                                       
         ELSE                                    
C                                                ! /* OCEAN PATH */     
C                                                                       
            IF (ABS(ALAT).LT.50) THEN                                   
               CALL TGCT(T4,PF,THR10, CO)                               
               LTEST = 'TGCT  '                                         
C                                                                       
C...TUT: FMF: OCEAN: TGCT:CLR/CLEAR                                     
C...TUT: FMF: OCEAN: TGCT:CLD/LT BLUE                                   
C                                                                       
            ENDIF                                                       
C                                                                       
            RESULT = CCLASS(PF)                                         
C                                                                       
         ENDIF                                                          
C                                                                       
C                                                                       
C...TUT: FMF: CLD/PINK                                                  
C                                                                       
      ELSE                                  
C                                           ! /* FAILED FMFT */         
         RESULT = CCLASS(PF)                                            
      ENDIF                                                             
C                                                                       
C                                                                       
      RETURN                                                            
      END                                                               
