      SUBROUTINE GHRLST (ISPREC)                                        
C********************************************************************** 
C                                                                       
C   NAME- GHRLST                       EXPIRES- 12/31/99                
C                                                                       
C   LANGUAGE- FORTRAN     TYPE- SUBROUTINE                              
C                                                                       
C   VERSION- 1.0    DATE- 08/20/81   PROGRAMMER- ROB FRISTROM           
C   VERSION- 1.1    DATE- 03/25/82   PROGRAMMER- R.M. YAMAMOTO S/DS13   
C   VERSION- 2.0    DATE- 02/02/83   PROGRAMMER- SUTTON, J. L. (SASC)   
C   VERSION- 3.0    DATE- 02/22/93   PROGRAMMER- D. CARR  (SMSRC)       
C                                                                       
C   FUNCTION-*$SF13RF* GIVEN A LATITUDE AND LONGITUDE, THIS ROUTINE     
C        RETRIVES A HIGH RESOLUTION LAND SEA TAG FOR THE LATITUDE AND   
C        LONGITUDE WHICH ARE ROUNDED TO THE NEAREST SIXTENTH OF A       
C        DEGREE.                                                        
C                                                                       
C        VERSION 3.0 - SUBROUTINE NOW READS ENTIRE LAND/SEA DATABASE    
C                      INTO AN ARRAY FOR PROCESSING; ALSO MODIFIED      
C                      TO PROCESS ENTIRE SCANLINE ON ONE CALL.          
C                                                                       
C                                                                       
C   PARAMETERS-                                                         
C      VARIABLES     TYPE   DESCRIPTION                                 
C      ---------     ----   -----------                                 
C      SLAT          R*4    ENTERED LATITUDE                            
C      SLON          R*4    ENTERED LONGITUDE                           
C      JRCBUF        I*2    BUFFER FOR DATA RECORD OF 10X10 DEG BLOCK.  
C      LSTAG         I*4    HIGH RESOLUTION LAND/SEA TAG.               
C                                                                       
C   FILES-                                                              
C      DSN OR DESCRIPTIVE TITLE  I/O UNIT FUNCTIONAL DESCRIPTION        
C      ------------------------  --- ---- ----------------------        
C      NSS.PSATAVST.LSTAGS.      I    77  HIGH RESOLUTION LAND/SEA TAG  
C      TEMPHIGH                           FILE                          
C                                                                       
C                                                                       
C   KEY LOCAL PARAMETERS-                                               
C      VARIABLES     TYPE   DESCRIPTION                                 
C      ---------     ----   ----------                                  
C      JDIR (1600)   I*2    ARRAY OF RECORD NUMBERS FOR                 
C                           10X10 DEGREE BLOCKS IN LAND/                
C                           SEA TAG FILE                                
C      JRCBUF (1600) I*2    BUFFER FOR 10X10 DEGREE BLOCK               
C                           DATA RECORD                                 
C      JDIRFG        I*4    DATABASE DIRECTORY READ FLAG--              
C                             0 = DIRECTORY RECORD NOT READ             
C                             1 = DIRECTORY RECORD READ                 
C      RECLAT        R*4    LATITUDE USED FOR COMPUTING RECORD NUMBER   
C      RECLNG        R*4    LONGITUDE USED FOR COMPUTING RECORD NUMBER  
C      ISIXTN(1)     I*2    THE NUMBER OF 16'S NORTH OF RECLAT          
C      ISIXTN(2)     I*2    THE NUMBER OF 16'S EAST OF RECLNG           
C      JDIRFG        I*2    THE DIRECTORY READ FLAG: 0 = NOT READ       
C                                                    1 = WAS READ.      
C      IWORD         I*2    THE INDEX USED TO LOCATE THE RECORD         
C                           NUMBER OF THE REQUESTED 10X10 DEG BLOCK     
C      JWORD         I*2    THE INDEX USED TO LOCATE THE LAND/SEA       
C                           TAG WITHIN THE 10X10 DEG BLOCK.             
C      JDIR(IWORD)   I*2    RECORD NUMBER OF A 10X10 DEG BLOCK.         
C      SWLAT         R*4    LATITUDE OF SOUTHWEST CORNER OF A           
C                           10X10 DEG BLOCK.                            
C      SWLNG         R*4    LONGITUDE OF SOUTHWEST CORNER OF A          
C                           10X10 DEG BLOCK.                            
C      ITAG          I*4    RESULT OF LOGICAL AND OPERATION. 0  = > SEA 
C                                                 NONZERO VALUE = > LAND
C   SUBPROGRAMS CALLED- DREAD.                                          
C                                                                       
C   EXIT STATES- ABNORMAL TERMINATION OF DREAD ERROR OCCURS.            
C                                                                       
C   COMMON AREAS- CLVUNT, GEOG, TYPDAT, ATMISC                          
C                                                                       
C   COMMENTS- MODIFIED TO ENABLE ROUTINE TO ACCESS SOUTHERN AND         
C             WESTERN HEMISPHERE. RMY. 3/25/82                          
C             MODIFIED TO INTERPOLATE LATITUDE AND LONGITUDE TO         
C             THE NEAREST 1/16TH DEGREE INTERSECTION AND CONVERT        
C             ALL LATITUDES AND LONGITUDES TO POSITIVE VALUES FOR       
C             COMPUTING RECORD NUMBERS.  JLS.  2/2/83                   
C                                                                       
C***********************************************************************
C                                                                       
      INCLUDE "inc/inclav.f"                                                  
      INCLUDE "inc/typdat.f"                                                  
      INCLUDE "inc/atmisc.f"                                                  
      INCLUDE "inc/clvunt.f"                                                  
      INCLUDE "inc/geog.f"                                                    
C                                                                       
C***********************************************************************
C                                                                       
      INTEGER*2  JDIR(1600), JRCBUF(1600), HIGHLS(2:465,1:1600)         
      INTEGER*2  ISIXTN(2),LAT1,LNG1,LAT10,LNG10                        
      INTEGER*2  IWORD,JWORD,KWORD,LATROW,LNGCOL                        
C                                                                       
      INTEGER*4  JDIRFG                                         
      INTEGER*4  ITAG, I4WORD                                           
      INTEGER*4  HEX(16)                                                
C                                                                       
      REAL*4 RECLAT,RECLNG,FRCLAT,FRCLNG,SIXTN(2),SWLAT,SWLNG           
C                                                                       
C                                                                       
      DATA HEX/$8000,$4000,$2000,$1000,$0800,$0400,$0200,$0100          
     >,$0080,$0040,$0020,$0010,$0008,$0004,$0002,$0001/                 
      DATA JDIRFG/0/
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C                                                                       
C ON FIRST CALL TO THIS SUBROUTINE, READ IN THE DIRECTORY AND           
C   ALL DATA RECORDS.                                                   
C                                                                       
      IF (JDIRFG.EQ.0) THEN                                             
C                                                                       
CMZ         CALL DREAD (LFILE,1,JDIR,*1000)                                
C
         OPEN(23,FILE='data/lstags.dat',
     1           STATUS='OLD', RECORDTYPE='FIXED',
     1           ACCESS='DIRECT',
     1           FORM='UNFORMATTED', RECL=896)
C 
         READ(23,REC=1,ERR=1000) (JDIR(K), K=1,1600)
         CALL SWAP16(JDIR, 1600)
C                                                                       
C        WRITE(6,900) (JDIR(IW+6),IW=1,648)                             
  900    FORMAT(/,5X,'LAND/SEA DIRECTORY:',18(/,5X,36I3))               
C                                                                       
         DO 20 IREC = 2, 465                                            
C                                                                       
CMZ           CALL DREAD (LFILE, IREC, JRCBUF, *1000)                      
           READ(23,REC=IREC,ERR=1000) (JRCBUF(K), K=1,1600)
           CALL SWAP16(JRCBUF,1600)
C                                                                       
           DO 10 IW = 1, 1600                                           
             HIGHLS(IREC,IW) = JRCBUF(IW)                               
   10      CONTINUE                                                     
C                                                                       
   20    CONTINUE                                                       
C                                                                       
         JDIRFG = 1                                                     
C                                                                       
      ENDIF                                                             
C                                                                       
C                                                                       
C PIXEL PROCESSING LOOP.                                                
C                                                                       
      DO 40 IPP = IPBEG, IPEND                                          
C                                                                       
        IPIX = IPP - IPBEG + 1                                          
C                                                                       
        LSTAG = LLSTAG          
C                               !  /* DEFAULT SETTING OF CURRENT        
C                               
C                               !     PIXEL'S TAG = LAST PIXEL'S TAG */ 
        SLAT = SPLAT(IPP)                                               
        SLON = SPLON(IPP)                                               
C                                                                       
        IF (SLAT .LE. -90.0 .OR. SLAT .GE.  90.0 .OR.                   
     1      SLON .LE.-180.0 .OR. SLON .GE. 180.0) GO TO 30              
C                                                                       
C FIND THE RECORD NUMBER OF THE DESIRED 10 X 10 DEGREE BLOCK            
C                                                                       
      RECLAT = SLAT+90.03125                                            
      RECLNG = SLON+180.03125                                           
      IF (RECLAT.GE.180.00) RECLAT = 179.9375                           
      IF (RECLNG.GE.360.00) RECLNG = RECLNG-360.00                      
      LAT10 = IFIX(RECLAT/10.0)                                         
      LNG10 = IFIX(RECLNG/10.0)                                         
      IWORD = LAT10*36+LNG10+7                                          
      IRECRD = JDIR(IWORD)                                              
C                                                                       
C IF JDIR(IWORD) = 0 THEN ALL TAGS ARE SEA TAGS. RETURN A SEA TAG.      
      IF (IRECRD.EQ.0) THEN                                             
        LSTAG = 0                                                       
        GO TO 30                                                        
      END IF                                                            
C                                                                       
C IF JDIR(IWORD) = -1 THEN ALL TAGS ARE LAND TAGS. RETURN A LAND TAG.   
      IF (IRECRD.EQ.-1) THEN                                            
        LSTAG = 1                                                       
        GO TO 30                                                        
      END IF                                                            
C                                                                       
C  CONVERT DECIMAL PORTION OF LATITUDE AND LONGITUDE TO                 
C  NEAREST SIXTEENTH OF A DEGREE                                        
      LAT1 = IFIX(RECLAT)                                               
      LNG1 = IFIX(RECLNG)                                               
      FRCLAT = RECLAT-LAT1*1.0                                          
      FRCLNG = RECLNG-LNG1*1.0                                          
      SIXTN(1) = FRCLAT*16.0                                            
      SIXTN(2) = FRCLNG*16.0                                            
      ISIXTN(1) = IFIX(SIXTN(1))                                        
      ISIXTN(2) = IFIX(SIXTN(2))                                        
C                                                                       
C FIND LAT AND LONG OF SOUTH WEST CORNER OF 10 DEGREE BLOCK             
      SWLAT = (IFIX(RECLAT*0.1))*10.0                                   
      SWLNG = (IFIX(RECLNG*0.1))*10.0                                   
      LATROW = IFIX(RECLAT-SWLAT)                                       
      LNGCOL = IFIX(RECLNG-SWLNG)                                       
C                                                                       
C WITHIN THE 10 X 10 DEG BLOCK, FIND THE 2 BYTE WORD POSITION.          
      JWORD = LATROW*160+ISIXTN(1)*10+LNGCOL+1                          
      KWORD = ISIXTN(2)+1                                               
C                                                                       
C LOCATE THE LAND/SEA TAG WITHIN THE 16 BIT WORD.                       
      I4WORD = HIGHLS(IRECRD,JWORD)                                     
      ITAG = IAND(I4WORD,HEX(KWORD))                                    
      IF (ITAG.EQ.0) LSTAG = 0                                          
      IF (ITAG.NE.0) LSTAG = 1                                          
C                                                                       
   30 CONTINUE                                                          
C                                                                       
      LSBUF(IPIX,ISPREC) = LSTAG                                        
      LLSTAG = LSTAG                                                    
C                                                                       
   40 CONTINUE                  
C                               !  /* GO ON TO NEXT PIXEL */            
C                                                                       
      GO TO 1020                
C                               !  /* SKIP OVER ERROR MESSAGE */        
C                                                                       
C                                                                       
 1000 CONTINUE                                                          
      CALL EMESG (12, 'GHRLST', 'FATAL')                                
C                                                                       
C                                                                       
 1020 CONTINUE                                                          
C                                                                       
C                                                                       
      RETURN                                                            
C                                                                       
      END                                                               
