      SUBROUTINE INARRY (CH, FLAG1, CLDCOD, CLRCLD)                     
C***********************************************************************
C   NAME- INARRY                                                        
C                                                                       
C   LANGUAGE- VS FORTRAN  TYPE- SUBROUTINE                              
C                                                                       
C   VERSION- 1.1    DATE- 10/08/93   PROGRAMMER- DEBBIE CARR (SMSRC)    
C                                                                       
C   FUNCTION- THIS SUBROUTINE INITIALIZES KEY ARRAYS BEFORE A PAIR OF   
C             SCANLINES IS PROCESSED.  THE FOLLOWING ARRAYS ARE INITIALI
C                                                                       
C                                                                       
C             CH     -   CONTAINS ALB/TEMP VALUES                       
C                                                                       
C             (FOUND IN COMMON GEOG)                                    
C             COSSZA -   CONTAINS SOLAR ZENITH ANGLE INFORMATION        
C             SATZ   -   CONTAINS SATELLITE ZENITH ANGLE INFORMATION    
C             RELZ   -   CONTAINS RELATIVE AZIMUTH ANGLE INFORMATION    
C                                                                       
C             CLDCOD -   CONTAINS THE CLOUD CODES                       
C             CLRCLD -   CONTAINS THE CLEAR/OVERCAST FLAGS              
C                                                                       
C             (FOUND IN COMMON ATMISC)                                  
C             LSBUF  -   CONTAINS THE LAND/SEA TAGS                     
C             COASTG -   CONTAINS THE COAST TAGS                        
C             ADFLAG -   CONTAINS THE COAST TAGS                        
C             DAYNIT -   CONTAINS THE DAY/NIGHT FLAGS                   
C                                                                       
C                                                                       
C   FILES-                                                              
C     NONE                                                              
C                                                                       
C   LOCAL PARAMETERS-                                                   
C     VARIABLE            TYPE  FUNCTION                                
C     --------            ----  ----------------------------------      
C     CH                  I*2    ARRAY USED TO CONTAIN ALB/TEMP VALUES  
C     FLAG1               L*4    INDICATES WHETHER OR NOT TO PROCESS    
C                                THE ANGLE VALUES.  FLAG1 IS DETERMINED 
C                                IN CLAVRORB.                           
C     CLDCOD              I*2    ARRAY USED TO CONTAIN THE CLOUD CODES  
C     CLRCLD              I*2    ARRAY USED TO CONTAIN THE CLEAR/OVERCAS
C                                FLAGS.                                 
C                                                                       
C                                                                       
C                                                                       
C                                                                       
C   CALLED BY- CLAVRORB                                                 
C                                                                       
C   SUBPROGRAMS CALLED- NONE                                            
C                                                                       
C   COMMON AREAS- INCLAV,ANGLE,GEOG,RANGES                              
C                                                                       
C*******************************************************************    
C                                                                       
C.....LINK IN COMMON AREAS                                              
C                                                                       
      INCLUDE "inc/inclav.f"                                                 
      INCLUDE "inc/typdat.f"                                                
      INCLUDE "inc/angle.f"                                                   
      INCLUDE "inc/geog.f"                                                    
      INCLUDE "inc/atmisc.f"                                                  
      INCLUDE "inc/ranges.f"                                                  
C                                                                       
      INTEGER*2            CLDCOD(MAXLPX,2),                            
     1                     CLRCLD(MAXLPX,2)                             
C                                                                       
      REAL*4               CH(MAXLPX,6,2)                               
C                                                                       
      LOGICAL*4            FLAG1                                        
C                                                                       
C                                                                       
C --------------------------------------------------------------------  
C                                                                       
C                                                                       
C....INITIALIZE THE SOLAR ZENITH, RELATIVE AZIMUTH, AND SATELLITE       
C     ZENITH ANGLES IF FLAG1 = .TRUE. (INDICATES THE FIRST SCAN OF      
C     EVERY OTHER SCAN PAIR, WHICH IS THE FREQUENCY AT WHICH THE        
C     ANGLES ARE CALCULATED AT.)                                        
C                                                                       
      IF ( FLAG1 ) THEN                                                 
C                                                                       
        DO 10 I = 1,MAXLPX                                              
C                                                                       
          SOLZ(I)   = 0.0                                               
          COSSZA(I) = INSZN                                             
          SATZ(I)   = INSTZ                                             
          RELZ(I)   = INRLZ                                             
C                                                                       
   10   CONTINUE                                                        
C                                                                       
      ENDIF                                                             
C                                                                       
C                                                                       
C....INITIALIZE THE ALBEDO/TEMPERATURE, LATITUDE, & LONGITUDE ARRAYS.   
C                                                                       
      DO 30 I = 1, MAXLPX                                               
C                                                                       
         SPLAT(I) = INLAT                                               
         SPLON(I) = INLON                                               
C                                                                       
        DO 20 IREC = 1, 2                                               
C                                                                       
          CH(I,1,IREC) = INCH1A                                         
          CH(I,2,IREC) = INCH2A                                         
          CH(I,3,IREC) = INCH3A                                         
          CH(I,4,IREC) = INCH3T                                         
          CH(I,5,IREC) = INCH4T                                         
          CH(I,6,IREC) = INCH5T                                         
C                                                                       
   20   CONTINUE                                                        
C                                                                       
   30 CONTINUE                                                          
C                                                                       
C                                                                       
C....INITIALIZE THE CLOUD CODE, CLEAR/CLOUD, DAY/NIGHT,                 
C    LAND/SEA, AND COAST TAG ARRAYS.                                    
C                                                                       
      DO 50 I = 1, NPIXLS                                               
C                                                                       
        DO 40 IREC = 1, 2                                               
C                                                                       
          CLDCOD(I,IREC) = 0                                            
          CLRCLD(I,IREC) = 0                                            
          DAYNIT(I,IREC) = 0                                            
          LSBUF(I,IREC)  = 0                                            
          COASTG(I,IREC) = 0                                            
C                                                                       
   40   CONTINUE                                                        
C                                                                       
   50 CONTINUE                                                          
C                                                                       
C                                                                       
C                                                                       
      RETURN                                                            
      END                                                               
