C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
C                                                                       
C   VERSION-1.0  -  10/15/91  -  DEBBIE CARR (SMSRC)  -  COMMON /OHED1C/
C                                                                       
C                                                                       
C   PURPOSE - STORAGE OF ORBIT HEADER INFORMATION FOR THE 1C FILE.      
C                                                                       
C                                                                       
C     1C ORBIT HEADER RECORD DESCRIPTION  (ALL WORDS, I*4)              
C                                                                       
C     WORD     DESCRIPTION                               VARIABLE NAME  
C     ----     -----------                               -------------  
C     1        ORBIT INDEX # (FOR 1C FILE HOUSEKEEPING)  NEWIDX         
C     2        SATELLITE ID                              NSAT           
C     3        ORBIT # (PROCESSING BLOCK ID)             ORBNUM         
C     4        AVHRR DATA TYPE                           DATYPE         
C     5        GROUND STATION                            GRDSTA         
C     6        RAMP/AUTO CALIBRATION                     RMPCAL         
C     7        # INPUT DATA FRAMES WITHOUT SYNC ERRORS   NOSYNC         
C     8        # TIP PARITY ERRORS                       PARERR         
C     9        # AUXILIARY SYNC ERRORS                   AUXSYC         
C     10       CALIBRATION PARAMETER ID                  IDCALP         
C     11       DACS STATUS                               DACSTA         
C     12       START DATE OF ORBIT (YEAR, D.O.Y.)        OSDATE         
C     13       START TIME OF ORBIT (GMT) (HOUR, MIN)     OSTIME         
C     14       END DATE OF ORBIT (YEAR, D.O.Y.)          OEDATE         
C     15       END TIME OF ORBIT (GMT) (HOUR, MIN)       OETIME         
C     16       DATE OF CLAVR PROCESSING (YEAR, D.O.Y.)   PDATE          
C     17       START TIME OF CLAVR (GMT) (HOUR, MIN)     PSTIME         
C     18       END TIME OF CLAVR (GMT) (HOUR, MIN)       PETIME         
C     19       NUMBER OF SCANLINES IN ORBIT              NUMSCN         
C     20       NUMBER OF DATA GAPS IN ORBIT              NUMGAP         
C     21       EARTH-SUN DISTANCE RATIO FOR THIS ORBIT   C1ESD          
C     22-EOB   SPARE (FILL, -1)                          (N/A)          
C                                                                       
C                                                                       
C   USED BY-  MAIN CLAVRORB, BLOCK DATA B$CLAV, SUBROUTINES AVHEDR,     
C             FOOTER, W1CHED                                            
C                                                                       
C   COMMENTS- ALL VARIABLES EXCEPT THE CLAVR PROCESSING DATE/TIME       
C             AND EARTH-SUN DISTANCE RATIO ARE SET IN SUBROUTINE        
C             "AVHEDR".  THE CLAVR DATE/TIME AND ESD ARE SET IN         
C             THE MAIN PROGRAM ("CLAVRORB").  ALL PARAMETERS ARE        
C             STORED IN THE 1C OUTPUT BUFFER, THEN WRITTEN              
C             TO THE 1C FILE IN SUBROUTINE "W1CHED".                    
C                                                                       
C                                                                       
      CHARACTER*4  CDATYP (3)                                           
C                                                                       
      CHARACTER*8  SPC (0:8), CRAFT                                     
C                                                                       
      INTEGER*4  NEWIDX, NSAT,   DATYPE, ORBNUM,                        
     1           OSDATE, OSTIME, OETIME, PDATE,  PSTIME, PETIME,        
     2           GRDSTA, NUMSCN, NUMGAP, C1ESD,  RMPCAL,                
     3           IDCALP, DACSTA, OSYEAR, OSDAY,  OSSEC,  OSHOUR,        
     4           OSMINS, OEYEAR, OEDAY,  OESEC,  OEHOUR, OEMINS,        
     5           OEDATE, NOSYNC, PARERR, AUXSYC                         
C                                                                       
      COMMON /OHED1C/ NEWIDX, NSAT,   DATYPE, ORBNUM,                   
     1                OSDATE, OSTIME, OETIME, PDATE,  PSTIME, PETIME,   
     2                GRDSTA, NUMSCN, NUMGAP, C1ESD,  RMPCAL,           
     3                IDCALP, DACSTA, OSYEAR, OSDAY,  OSSEC,  OSHOUR,   
     4                OSMINS, OEYEAR, OEDAY,  OESEC,  OEHOUR, OEMINS,   
     5                OEDATE, NOSYNC, PARERR, AUXSYC,                   
     6                CDATYP, SPC,    CRAFT                             
C                                                                       
C                                                                       
C      VARIABLE               TYPE  FUNCTION                            
C      --------               ----  --------                            
C      NEWIDX                  I*4  ORBIT INDEX # (FOR 1C FILE          
C                                   HOUSEKEEPING)                       
C      NSAT                    I*4  SATELLITE ID                        
C                                   0 = SPARE                           
C                                   1 = NOAA-11                         
C                                   2 = NOAA-12                         
C                                   4 = NOAA-7                          
C                                   6 = NOAA-8                          
C                                   7 = NOAA-9                          
C                                   8 = NOAA-10                         
C      ORBNUM                  I*4  ORBIT # (PROCESSING BLOCK ID)       
C      DATYPE                  I*4  AVHRR DATA TYPE                     
C                                   1 = LAC                             
C                                   2 = GAC                             
C                                   3 = HRPT                            
C                                   (ANYTHING ELSE ==> NOT AVHRR)       
C      GRDSTA                  I*4  GROUND STATION                      
C                                   0 = (UNUSED)                        
C                                   1 = GILMORE CREEK, AK               
C                                   2 = WALLOPS ISLAND, VA              
C                                   3 = SOCC                            
C      RMPCAL                  I*4  RAMP/AUTO CALIBRATION               
C                                   (INDICATES RAMP NON-LINEARITY       
C                                   FOR THE 5 INSTRUMENT CHANNELS)      
C      NOSYNC                  I*4  # INPUT DATA FRAMES WITHOUT         
C                                   SYNC ERRORS                         
C      PARERR                  I*4  # TIP PARITY ERRORS                 
C      AUXSYC                  I*4  # AUXILIARY SYNC ERRORS             
C      IDCALP                  I*4  CALIBRATION PARAMETER ID            
C                                   (IDENTIFIES THE CALIBRATION         
C                                   PARAMETER INPUT DATA SET WHICH      
C                                   WAS USED TO CALIBRATE THE DATA      
C                                   IN THE 1B FILE)                     
C      DACSTA                  I*4  DACS STATUS                         
C                                   (PSEUDO NOISE, DATA SOURCE,         
C                                   TAPE DIRECTION, AND                 
C                                   DATA MODE FLAGS)                    
C      OSDATE                  I*4  START DATE OF ORBIT                 
C                                   (YEAR, D.O.Y.)                      
C      OSTIME                  I*4  START TIME OF ORBIT (GMT)           
C                                   (HOUR, MIN)                         
C      OEDATE                  I*4  END DATE OF ORBIT                   
C                                   (YEAR, D.O.Y.)                      
C      OETIME                  I*4  END TIME OF ORBIT (GMT)             
C                                   (HOUR, MIN)                         
C      PDATE                   I*4  DATE OF CLAVR PROCESSING            
C                                   (YEAR, D.O.Y.)                      
C      PSTIME                  I*4  START TIME OF CLAVR (GMT)           
C                                   (HOUR, MIN)                         
C      PETIME                  I*4  END TIME OF CLAVR (GMT)             
C                                   (HOUR, MIN)                         
C      NUMSCN                  I*4  NUMBER OF SCANLINES IN ORBIT        
C      NUMGAP                  I*4  NUMBER OF DATA GAPS IN ORBIT        
C      C1ESD                   I*4  EARTH-SUN DISTANCE RATIO            
C                                   FOR THIS ORBIT (SCALED BY 1000)     
C      CDATYP (3)              C*4  LIST OF AVHRR DATA TYPES            
C                                   (CHARACTER FORMAT)                  
C      SPC (11)                C*8  LIST OF NOAA SPACECRAFT NAMES       
C                                   (CHARACTER FORMAT)                  
C      CRAFT                   C*8  SPACECRAFT NAME (CHARACTER FORMAT)  
C                                                                       
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
C                                                                       
