C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
C                                                                       
C   VERSION-1.0  -  05/18/93  -  CID PRADERAS(SMSRC)  -  COMMON/RANGES/ 
C                                                                       
C                                                                       
C   PURPOSE - STORES LOWER AND UPPER LIMITS, INITIALIZING VALUES        
C             OF THE PROCESSING ARRAYS, AND REPLACEMENT VALUES IN       
C             THE 1C DATA SET WHEN CALCULATED VALUES ARE OUT OF         
C             LIMITS FOR THE FOLLOWING PARAMETERS:                      
C                                                                       
C             1) LATITUDES                                              
C             2) LONGITUDES                                             
C             3) SOLAR ZENITH ANGLES                                    
C             4) SATELLITE ZENITH ANGLES                                
C             5) RELATIVE AZIMUTH ANGLES                                
C                                                                       
C             6) CHANNEL 1 - ALBEDO                                     
C             7) CHANNEL 2 - ALBEDO                                     
C             8) CHANNEL 3 - ALBEDO                                     
C             9) CHANNEL 3 - TEMPERATURE                                
C            10) CHANNEL 4 - TEMPERATURE                                
C            11) CHANNEL 5 - TEMPERATURE                                
C                                                                       
C                                                                       
C                                                                       
C   USED BY - CLAVRORB, CHKRNG, INARRY, RFLAGS                          
C                                                                       
C                                                                       
      INTEGER*4  LWSZN,  LWSZN1, UPSZN                                  
      INTEGER*4  LWRLZ,  UPRLZ                                          
      INTEGER*4  LWSTZ,  UPSTZ                                          
C                                                                       
      INTEGER*4  INRLZ,  INSTZ                                          
C                                                                       
      REAL*4     LWCH1A, UPCH1A,                                        
     1           LWCH2A, UPCH2A,                                        
     2           LWCH3A, UPCH3A,                                        
     3           LWCH3T, UPCH3T,                                        
     4           LWCH4T, UPCH4T,                                        
     5           LWCH5T, UPCH5T                                         
C                                                                       
      REAL*4     INLAT,  INLON, INSZN                                   
C                                                                       
      REAL*4     INCH1A, INCH2A, INCH3A,                                
     1           INCH3T, INCH4T, INCH5T                                 
C                                                                       
      REAL*4     LWLATC, UPLATC                                         
      REAL*4     LWLONC, UPLONC                                         
C                                                                       
      REAL*4     LWLAT, UPLAT                                           
      REAL*4     LWLON,  UPLON                                          
C                                                                       
      INTEGER*4  SZN1C, STZ1C, RLZ1C                                    
C                                                                       
      REAL*4     CH11C1, CH11C2,                                        
     1           CH21C1, CH21C2,                                        
     2           CH31C1, CH31C2,                                        
     3           CH3T1C, CH4T1C, CH5T1C                                 
C                                                                       
      COMMON /RANGES/ LWLAT,   UPLAT,   LWLON,   UPLON,                 
     1                LWSZN,   LWSZN1,  UPSZN,   LWRLZ,  UPRLZ,         
     2                LWSTZ,   UPSTZ,                                   
     3                INLAT,   INLON,                                   
     4                INSZN,   INRLZ,   INSTZ,                          
     5                SZN1C,   STZ1C,   RLZ1C,                          
     6                LWLATC,  UPLATC,  LWLONC, UPLONC,                 
     7                INCH1A,  INCH2A,  INCH3A,                         
     8                INCH3T,  INCH4T,  INCH5T,                         
     9                LWCH1A,  UPCH1A,                                  
     A                LWCH2A,  UPCH2A,  LWCH3A,  UPCH3A,                
     B                LWCH3T,  UPCH3T,  LWCH4T,  UPCH4T,                
     C                LWCH5T,  UPCH5T,                                  
     D                CH11C1,  CH11C2,                                  
     E                CH21C1,  CH21C2,                                  
     F                CH31C1,  CH31C2,                                  
     G                CH3T1C,  CH4T1C,  CH5T1C                          
C                                                                       
C                                                                       
C                                                                       
C      VARIABLE               TYPE  FUNCTION                            
C      --------               ----  --------                            
C      LWLAT                   R*4  LATITUDE LOWER LIMIT                
C      UPLAT                   R*4  LATITUDE UPPER LIMIT                
C      LWLON                   R*4  LONGITUDE LOWER LIMIT               
C      UPLON                   R*4  LONGITUDE UPPER LIMIT               
C                                                                       
C      LWSZN                   I*4  SOLAR ZENITH ANGLE LOWER LIMIT      
C      UPSZN                   I*4  SOLAR ZENITH ANGLE UPPER LIMIT      
C      LWRLZ                   I*4  RELATIVE AZIMUTH ANGLE LOWER LIMIT  
C      UPRLZ                   I*4  RELATIVE AZIMUTH ANGLE UPPER LIMIT  
C                                                                       
C      LWSTZ                   I*4  SATELLITE ZENITH ANGLE LOWER LIMIT  
C      UPSTZ                   I*4  SATELLITE ZENITH ANGLE UPPER LIMIT  
C                                                                       
C      LWCH1A                  R*4  CHANNEL 1 ALBEDO LOWER LIMIT        
C      UPCH1A                  R*4  CHANNEL 1 ALBEDO UPPER LIMIT        
C      LWCH2A                  R*4  CHANNEL 2 ALBEDO LOWER LIMIT        
C      UPCH2A                  R*4  CHANNEL 2 ALBEDO UPPER LIMIT        
C      LWCH3A                  R*4  CHANNEL 3 ALBEDO LOWER LIMIT        
C      UPCH3A                  R*4  CHANNEL 3 ALBEDO UPPER LIMIT        
C      LWCH3T                  R*4  CHANNEL 3 TEMPERATURE LOWER LIMIT   
C      UPCH3T                  R*4  CHANNEL 3 TEMPERATURE UPPER LIMIT   
C      LWCH4T                  R*4  CHANNEL 4 TEMPERATURE LOWER LIMIT   
C      UPCH4T                  R*4  CHANNEL 4 TEMPERATURE UPPER LIMIT   
C      LWCH5T                  R*4  CHANNEL 5 TEMPERATURE LOWER LIMIT   
C      UPCH5T                  R*4  CHANNEL 5 TEMPERATURE UPPER LIMIT   
C                                                                       
C      INLAT                   R*4  INITIAL VALUE FOR LATITUDE          
C      INLON                   R*4  INITIAL VALUE FOR LONGITUDE         
C      INCH1A                  R*4  INITIAL VALUE FOR CHANNEL 1 ALBEDO  
C      INCH2A                  R*4  INITIAL VALUE FOR CHANNEL 2 ALBEDO  
C      INCH3A                  R*4  INITIAL VALUE FOR CHANNEL 3 ALBEDO  
C      INCH3T                  R*4  INITIAL VALUE FOR CHANNEL 3 TEMP    
C      INCH4T                  R*4  INITIAL VALUE FOR CHANNEL 4 TEMP    
C      INCH5T                  R*4  INITIAL VALUE FOR CHANNEL 5 TEMP    
C      INSZN                   R*4  INITIAL VALUE FOR SOLAR ZENITH      
C      INRLZ                   I*4  INITIAL VALUE FOR RELATIVE AZIMUTH  
C      INSTZ                   I*4  INITIAL VALUE FOR SATELLITE ZENITH  
C                                                                       
C      LWLATC                  R*4  LATITUDE VALUE FOR 1C WHEN          
C                                   CALCULATED VALUE IS BELOW LOWER     
C                                   LIMIT                               
C      UPLATC                  R*4  LATITUDE VALUE FOR 1C WHEN          
C                                   CALCULATED VALUE IS ABOVE UPPER     
C                                   LIMIT                               
C      LWLONC                  R*4  LONGITUDE VALUE FOR 1C WHEN         
C                                   CALCULATED VALUE IS BELOW LOWER     
C                                   LIMIT                               
C      UPLONC                  R*4  LONGITUDE VALUE FOR 1C WHEN         
C                                   CALCULATED VALUE IS ABOVE UPPER     
C                                   LIMIT                               
C      SZN1C                   I*4  SOLAR ZENITH VALUE FOR 1C WHEN      
C                                   LIMITS ARE EXCEDED                  
C      STZ1C                   I*4  SATELLITE ZENITH VALUE FOR 1C WHEN  
C                                   LIMITS ARE EXCEDED                  
C      RLZ1C                   I*4  RELATIVE AZIMUTH VALUE FOR 1C WHEN  
C                                   LIMITS ARE EXCEDED                  
C      CH11C1                  R*4  CHANNEL 1 ALBEDO VALUE FOR 1C WHEN  
C                                   LOWER LIMIT IS EXCEDED              
C      CH11C2                  R*4  CHANNEL 1 ALBEDO VALUE FOR 1C WHEN  
C                                   UPPER LIMIT IS EXCEDED              
C      CH21C1                  R*4  CHANNEL 2 ALBEDO VALUE FOR 1C WHEN  
C                                   LOWER LIMIT IS EXCEDED              
C      CH21C2                  R*4  CHANNEL 2 ALBEDO VALUE FOR 1C WHEN  
C                                   UPPER LIMIT IS EXCEDED              
C      CH31C1                  R*4  CHANNEL 3 ALBEDO VALUE FOR 1C WHEN  
C                                   LOWER LIMIT IS EXCEDED              
C      CH31C2                  R*4  CHANNEL 3 ALBEDO VALUE FOR 1C WHEN  
C                                   UPPER LIMIT IS EXCEDED              
C      CH3T1C                  R*4  CHANNEL 3 TEMP VALUE FOR 1C WHEN    
C                                   LIMIT IS EXCEDED                    
C      CH4T1C                  R*4  CHANNEL 4 TEMP VALUE FOR 1C WHEN    
C                                   LIMIT IS EXCEDED                    
C      CH5T1C                  R*4  CHANNEL 5 TEMP VALUE FOR 1C WHEN    
C                                   LIMIT IS EXCEDED                    
C                                                                       
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
C                                                                       
