      SUBROUTINE INSPCN
C***********************************************************************
C   NAME- INSPCN
C
C   LANGUAGE- VS FORTRAN  TYPE- SUBROUTINE
C
C   VERSION- 1.0    DATE- 08/31/93   PROGRAMMER- CID PRADERAS  (SMSRC)
C   VERSION- 2.0    DATE- 08/09/95   PROGRAMMER- Doug Love     (SMART)
C
C   FUNCTION- THIS SUBROUTINE INITIALIZES SPACECRAFT-DEPENDANT CONSTANTS
C             THROUGHOUT THE CLAVRORB PROGRAMS.
C             Version 2 recognizes the satellite ID and opens the
C             correct constant files.  The Ch. 4 and 5 nonlinear
C             correction files are eliminated.
C
C   FILES-
C         UNIT=UNSPCN, FILE='data/const.noaa07', constants file
C         UNIT=UNSPCN, FILE='data/const.noaa09', constants file
C         UNIT=UNSPCN, FILE='data/const.noaa11', constants file
C         UNIT=UNSPCN, FILE='data/const.noaa12', constants file
C         UNIT=UNSPCN, FILE='data/const.noaa14', constants file
C
C
C   LOCAL PARAMETERS-
C
C     VARIABLE            TYPE  FUNCTION
C     --------            ----  ----------------------------------
C     FATAL                C*5  'FATAL'
C     PROGRM               C*6  'INSPCN'
C
C   CALLED BY- CLAVRORB
C
C   SUBPROGRAMS CALLED- EMESG(64,PROGRM,FATAL)
C
C   COMMON AREAS- SPCNST, OHED1C, CLVUNT, QCPARMS
C
C*******************************************************************
C
C.....LINK IN COMMON AREAS
C
      INCLUDE "inc/ohed1c.f"
      INCLUDE "inc/clvunt.f"
      INCLUDE "inc/spcnst.f"
      INCLUDE "inc/newcon.f"
C
      CHARACTER*2  SATIDS(7)
      CHARACTER*5  FATAL
      CHARACTER*6  PROGRM
      CHARACTER*17 CONSTF
C
      DATA         PROGRM / 'INSPCN' /, FATAL / 'FATAL' /
      DATA         SATIDS / '11', '11', '14', '07', '12', '11', '09'/
C
C.....WRITE OPENING MESSAGE
C
C     WRITE(6,1000)
C.....OPEN SPACECRAFT CONSTANTS FILE
C
      IF (NSAT.EQ.2 .OR. NSAT.EQ.6 .OR. NSAT.GT.7 .OR. NSAT.LT.1) THEN
          WRITE (6,666) NSAT
 666      FORMAT (///, ' ****  ILLEGAL SATELLITE NO. ', I12, '**** ')
          STOP
      ELSE
          CONSTF = 'data/const.noaa'//SATIDS(NSAT)
          OPEN (UNIT=UNSPCN, FILE=CONSTF, ACCESS='SEQUENTIAL',
     1          ERR= 130, STATUS= 'OLD', FORM= 'FORMATTED')
      END IF
C
C.....READ SAT ID  IN SPACECRAFT CONSTANTS FILE
C
      READ (UNSPCN, 1010, ERR=100) SPCNID
C
C.....READ IN ARRAY OF LIMITS OF ENERGY ( INTO BOUND(2,3) )
C
      DO 10 I=1,3
         READ (UNSPCN, 1020, ERR=100) BOUND(1,I), BOUND(2,I)
   10 CONTINUE
C
C.....READ IN ( WAVENO(3,3) ) --
C
      READ (UNSPCN, 1020, ERR=100) CH1W, CH2W
      READ (UNSPCN, 1020, ERR=100) CH1F, CH2F
      READ (UNSPCN, 1020, ERR=100) CH3WNO, CH3WLN
      READ (UNSPCN, 1060, ERR=100) CH3SIN
      READ (UNSPCN, 1020, ERR=100) CH3RWN, CH3RWL
C
C.....READ IN NO. PERIODS, NONZERO SRP START/END DATES, SRP PARAMETERS
C
      READ (UNSPCN, 1210, ERR=100) NZSRPS, IYRDYS(1), IYRDYE(1),
     *                                     IYRDYS(2), IYRDYE(2)
      READ (UNSPCN, 1220, ERR=100) ((SRP(I,J), J=4,5), I=1,2)
C
      DO 20 I=1,3
C
         READ (UNSPCN, 1030, ERR=100) WAVENO(1,I), WAVENO(2,I),
     1                                WAVENO(3,I)
C
   20 CONTINUE
C
C.....READ IN 5TH ORDER COEFFICIENTS FOR EACH PRT COUNT TO TEMPERATURE
C     CONVERSION
C
      DO 30 I=1,4
C
        READ (UNSPCN, 1040, ERR=100) ACOEF(0,I),ACOEF(1,I),
     1                ACOEF(2,I),ACOEF(3,I),ACOEF(4,I)
C
   30 CONTINUE
C
C.....READ IN COEFFICIENTS FOR 4TH DEGREE POLYNOMIAL ( B1, B2, B3, B4 )
C
        READ (UNSPCN, 1050, ERR=100) B1, B2, B3, B4
C
C.....READ IN VARIABLES A, B, C, D, S3PI, CNUO
C
        READ (UNSPCN, 1060, ERR=100) A
        READ (UNSPCN, 1060, ERR=100) B
        READ (UNSPCN, 1060, ERR=100) C
        READ (UNSPCN, 1060, ERR=100) D
C
        READ (UNSPCN, 1070, ERR=100) S3PI
        READ (UNSPCN, 1080, ERR=100) CNUO
C
      DO 40 I=1,2
         READ (UNSPCN, 1090, ERR=100) X(I), Y(I), O(I)
  40  CONTINUE
C
        READ (UNSPCN, 1100, ERR=100) D0, Z
C
C.....READ IN GAMMA ANGLE LIMITS.
C
      READ (UNSPCN, 1160, ERR=100) (GAMLIM(I), I = 1, 5)
C
C  READ THE NEW COEFFICIENTS IN THE CONSTANTS FILE THAT
C  REPLACE THE ABOVE CH4 AND CH5 TABLES:
C
      READ (UNSPCN, 1010, ERR=100) IDEGF
      READ (UNSPCN, 1170, ERR=100) CNU(3), TIN(3), TSL(3)
      READ (UNSPCN, 1170, ERR=100) CNU(4), TIN(4), TSL(4)
      READ (UNSPCN, 1170, ERR=100) CNU(5), TIN(5), TSL(5)
      READ (UNSPCN, 1010, ERR=100) IRLINF
      READ (UNSPCN, 1180, ERR=100) QA(3), QB(3), QC(3)
      READ (UNSPCN, 1180, ERR=100) QA(4), QB(4), QC(4)
      READ (UNSPCN, 1180, ERR=100) QA(5), QB(5), QC(5)
      READ (UNSPCN, 1190, ERR=100) RSP
C
      GO TO 200
C
C
  100 CONTINUE
C.....FATAL ERROR READING THE SPACECRAFT CONSTANTS FILE
C
      CALL EMESG(61,PROGRM,FATAL)
C
  110 CONTINUE
C.....FATAL ERROR READING THE CHANNEL 4 NONLINEAR CORRECTION FILE
C
      CALL EMESG(62,PROGRM,FATAL)
C
  120 CONTINUE
C.....FATAL ERROR READING THE CHANNEL 5 NONLINEAR CORRECTION FILE
C
      CALL EMESG(63,PROGRM,FATAL)
C
  130 CONTINUE
C.....FATAL ERROR OPENING THE SPACECRAFT CONSTANTS FILE
C
      CALL EMESG(64,PROGRM,FATAL)
C
  200 CONTINUE
C
C.....PRINT OUT MESSAGE: SUCCESSFULLY LOADED SPACECRAFT CONSTANTS INTO A
C
C     WRITE(6,1140)
C
 1000 FORMAT (1X,/,'****STARTING SUBROUTINE INSPCN: READS IN ',
     1          'SPACECRAFT CONSTANTS.',/)
 1010 FORMAT (I5)
 1020 FORMAT (2F9.5)
 1030 FORMAT (3F8.2)
 1040 FORMAT (2F11.6, E15.6, 2F11.6)
 1050 FORMAT (4F6.2)
 1060 FORMAT (F9.5)
 1070 FORMAT (F6.3)
 1080 FORMAT (F8.2)
 1090 FORMAT (F9.4, F9.6, F9.2)
 1100 FORMAT (2I6)
 1120 FORMAT (1X, F6.2, F6.2)
 1130 FORMAT (1X, 11F6.2)
 1140 FORMAT (1X,/,'***END OF SUBROUTINE INSPCN: SUCCESSFULLY LOADED ',
     1         'SPACECRAFT CONSTANTS INTO ARRAYS.',/)
 1150 FORMAT (I2)
 1160 FORMAT ( 5(F4.1, 1X))
 1170 FORMAT (F8.3, F9.5, F9.6)
 1180 FORMAT (F8.5, F10.7, F8.4)
 1190 FORMAT (F7.4, F8.3, F6.3)
 1210 FORMAT (I3, 4I7)
 1220 FORMAT (4F8.3)
C
      RETURN
      END
