      SUBROUTINE NITLND(P, ANGL, ALAT, ALON, THR, CC, CO, TCOUNT)                     
C********************************************************************** 
C                                                                       
C     FUNCTION - SUBROUTINE NITLND EXECUTES THE NIGHT PASS 1 CLOUD      
C      DETECTION ALGORITHM FOR A 2X2 ARRAY                              
C                                                                       
C      ***** NIGHTTIME LAND *****                                       
C                                                                       
C                                                                       
C     INPUTS:                                                           
C      P      ARRAY CONTAINING CHANNELS 1,2,3 ALBEDOS AND CHANNELS      
C             3,4,5 TEMPERATURES FOR 2X2 ARRAY                          
C      LNDFLG LAND/SEA FLAG. LNDFLG=0 IS OCEAN, LNDFLG=1 IS LAND        
C      ALAT   MEAN LATITUDE FOR 2X2 ARRAY                               
C      THR    NIGHTTIME THRESHOLDS FOR EITHE LAND OR OCEAN              
C                                                                       
C                                                                       
C     OUTPUT PARAMETERS                                                 
C      CC      -   COLOR CODE OF PASS 1 TEST:                           
C                  1 : RED        L/O  : CLOUDY TGCT                    
C                  8 : YELLOW       O  : CLOUDY ULST                    
C                  9 : PINK       L/O  : CLOUDY FMFT                    
C                  10: LT PURPLE  L/O  : CLOUDY CIRT                    
C                  12: BLACK      L/O  : CLEAR                          
C                  17: RED        L/O  : MIXED  TGCT                    
C                  18: BLUE       L/O  : MIXED  TUT                     
C                  24: YELLOW       O  : MIXED  ULST                    
C                  25: PINK       L/O  : CLOUDY FMFT                    
C                  26: LT PURPLE  L/O  : MIXED  CIRT                    
C      CO      -   CLEAR/CLOUD FLAG                                     
C      PF      -   PASS/FAIL FLAG                                       
C                                                                       
C                                                                       
C     SUBROUTINES CALLED:                                               
C      TGCT    -   THERMAL GROSS CLOUD TEST (CHANNEL 4)                 
C      TUT     -   THERMAL UNIFORMITY TEST (CHANNEL 4)                  
C      ULST -      UNIFORM LOW STRATUS TEST                             
C      FMFT    -   CHANNEL 4 MINUS CHANNEL 5 TEST                       
C      CIRT    -   CIRRUS TEST (CHANNELS 3-5/5)                         
C                                                                       
C*********************************************************************  
C                                                                       
      INCLUDE "inc/apath.f"                                                   
C                                                                       
C                                                                       
      CHARACTER*6  CCLASS(0:2)                                          
C                                                                       
      INTEGER*2  ANGL(4,3), CO(4), CC, SETNCC                                      
C                                                                       
      INTEGER*4 PF, RS, TCOUNT(4)                                          
C                                                                       
      LOGICAL*4  DESCHK, DESERT                                         
C                                                                       
      REAL*4  P(4,6),THR(10), ALAT, LATLIM                             
C                                                                       
C                                                                       
      DATA  CCLASS /'CLEAR ', 'CLOUDY', 'MIXED '/                       
      DATA  LNDFLG /1/, LATLIM /30.0/                                   
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C                                                                       
      OTEST = '        '                                                
      RTEST = '        '                                                
      LTEST = '        '                                                
C                                                                       
C                                                                       
      DESERT = DESCHK (ALAT, ALON)                                      
C                                                                       
C                                                                       
C...CALL THERMAL GROSS CLOUD TEST                                       
C                                                                       
      CALL TGCT(P(1,5),PF,THR(1),CO)                                    
      OTEST = 'TGCT  '                                                  
C                                                                       
C                                                                       
      IF (PF.NE.0) THEN              
C                                 ! /* NIGHT/LAND RESTORAL PATH */   
C                                                                       
C...FAIL TGCT CLEAR TEST                                                
C                                                                       
         IF (ABS(ALAT) .GT. LATLIM) THEN                                
           CALL FMFR (P(1,5), P(1,6), LNDFLG, RS, PF, CO)               
C                                                                       
           IF (RS .EQ. 0) THEN                                          
C                                                                       
C...PASS FMFR RESTORAL TEST                                             
C                                                                       
             RTEST = 'FMFR  '                                           
             GO TO 10                                                   
C                                                                       
           END IF                                                       
C                                                                       
         ENDIF                                                          
C                                                                       
C...ABS(MEAN LATITUDE) IS LESS THAN OR EQUAL TO 50 DEGREES.             
C                                                                       
         RESULT = CCLASS(PF)                                            
C                                                                       
      ELSE                                    
C                                             ! /* PASSED TGCT */       
C                                                                       
C...PASS TGCT CLEAR TEST                                                
C                                                                       
   10    CONTINUE                                                       
C                                                                       
         CALL TUT(P(1,5),PF,THR(2),CO)                                  
         IF (RTEST .EQ. 'FMFR  ') THEN                                  
           LTEST = 'TUT   '                                             
         ELSE                                                           
           OTEST = 'TUT   '                                             
         END IF                                                         
C                                                                       
         IF (PF.EQ.0) THEN                                              
C                                                                       
C...PASS TUT CLEAR TEST                                                 
C                                                                       
C                                                                       
           IF (DESERT) GO TO 30                                         
C                                                                       
C...TEST FOR T4 GREATER THAN 271 AND LESS THAN 289                      
C                                                                       
           ICT = 0                                                      
C                                                                       
           DO 21 IP = 1, 4                                              
             IF (P(IP,5).GT.271 .AND. P(IP,5).LT.289) THEN              
               ICT = ICT + 1                                            
             ENDIF                                                      
   21      CONTINUE                                                     
C                                                                       
C...T4 GREATER THAN 271 AND LESS THAN 289                               
C                                                                       
           IF (ICT.NE.4) GO TO 30                                       
C                                                                       
C --- CHECK IF CHANNEL 1 COUNT GREATER OR EQUAL TO 45                   
C                                                                       
C --- TAKE OUT CHANNEL 1 COUNT CHECKING FOR ULST !!! 7/22/94            
C                                                                       
           CALL ULST(P,PF,THR(3),CO)                                    
C                                                                       
           IF (RTEST .EQ. 'FMFR  ') THEN                                
             LTEST = 'ULST  '                                           
           ELSE                                                         
             OTEST = 'ULST  '                                           
           END IF                                                       
C                                                                       
           IF (PF.EQ.0) THEN                                            
C                                                                       
C...PASS ULST CLEAR TEST                                                
C                                                                       
   30        CONTINUE                                                   
C                                                                       
C...SKIP FMFT TEST IF WE'VE ALREADY GONE THROUGH FMFR RESTORAL.         
C                                                                       
             IF (RTEST .NE. 'FMFR  ') THEN                              
               CALL FMFT(P(1,5), P(1,6), LNDFLG, PF, CO)                  
               OTEST = 'FMFT  '                                         
             END IF
C                                                     
             IF (PF.EQ.0) THEN 
C                                                                       
C...PASS OR BYPASS FMFT CLEAR TEST                                      
C                                                                       
C   CHECK SOLAR ZENITH ANGLE (ONLY FOR THE 1ST PIXEL).  IF LESS THAN    
C   90 DEGREES, SKIP THE CIRT TEST SINCE IT MAY BE CONTAMINATED BY      
C   SUNLIGHT.  (SPECIAL NOTE-- SOLAR ZENITH ANGLES ARE STORED AT THIS   
C   POINT AS COS(SZA)*1000.  THEREFORE, "ANGL" VALUES LESS THAN 1       
C   WILL CORRESPOND TO SOLAR ZENITH ANGLE VALUES GREATER THAN OR        
C   EQUAL TO 90 DEGREES-- THE CRITERIA FOR PERFORMING THE CIRT TEST.)   
C                                                                       
               IF (ANGL(1,1) .LT. 1) THEN   ! /* SZA >= 90 */           
C                                                                       
                  ICT1 = 0                                              
                  DO 27 IP = 1, 4                                       
                    IF (TCOUNT(IP) .GE. 45) THEN                        
                      ICT1 = ICT1 + 1                                   
                    ENDIF                                               
   27             CONTINUE                                                         
C                                                                       
                  IF (ICT1 .EQ. 0) THEN                                 
                    CALL CIRT(P,PF,CO)                                       
C                                                                       
                    IF (RTEST .EQ. 'FMFR  ') THEN                            
                      LTEST = 'CIRT  '                                       
                    ELSE                                                     
                      OTEST = 'CIRT  '                                       
                    END IF                                                   
                  ENDIF                                                      
C                                                                       
               ENDIF    
C
C                                                    
             ENDIF                                                          
C
            ENDIF
C
         ENDIF
C                                                                       
         RESULT = CCLASS(PF)                                            
C                                                                       
      ENDIF                                                             
C                                                                       
C                                                                       
      CC = SETNCC(LNDFLG)                                               
C                                                                       
C                                                                       
      RETURN                                                            
      END                                                               
