      SUBROUTINE SANGLE (DLAT, DLON, NANGS)                             
C********************************************************************** 
C                                                                       
C  COMMENTS - THE EXTRAPOLATION ROUTINE FOR THE RELATIVE AZIMUTH ANGLE  
C             SOMETIMES PRODUCES VALUES SLIGHTLY OUT OF RANGE.  THE     
C             EXPECTED RANGE IS FROM 0 TO 180.0 DEGREES.  IF THE        
C             RESULT IS SLIGHTLY OFF (I.E., BETWEEN -2.0 AND 0 OR       
C             BETWEEN 180.0 AND 182.0) IT WILL BE RESET TO THE          
C             PROPER LOWER OR UPPER BOUNDARY IN THE QC CHECK            
C             SUBROUTINE, "CHKRNG".                                     
C                                                                       
C********************************************************************** 
C                                                                       
      INCLUDE "inc/inclav.f"                                                  
      INCLUDE "inc/typdat.f"                                                  
      INCLUDE "inc/angle.f"                                                  
C                                                                       
C---------------------------------------------------------------------- 
C                                                                       
C                                                                       
      REAL*4 DLAT(MAXANC), DLON(MAXANC), PSIX                                   
      REAL*8 GEOX,COSGEO,DSPLAT,DSPLON,DSPSOL                      
      REAL*8 DENOM, NUMOR, DDLAT, DDLON                                 
C                                                                       
C                                                                       
C---------------------------------------------------------------------- 
C                                                                       
      DSPLAT = DLAT(26)*DEG2RD                                          
      DSPLON = DLON(26)*DEG2RD                                          
      DSPSOL = COSZX(26)                                                
C                                                                       
C                                                                       
C...COMPUTE SATELLITE ZENITH AND RELATIVE AZIMUTH ANGLES AT             
C   THE 51 ANCHOR POINTS OF THE SCANLINE.                               
C                                                                       
      DO 10 IP = 1, MAXANC                                                  
C                                                                       
         IF (IP.EQ.26) THEN                                             
C                                                                       
            COSGEO = 1.0                                                
            GEOX   = 0.0                                                
            RELZX(26)  = -999.9                                         
C                                                                       
         ELSE                                                           
C                                                                       
            DDLAT = DLAT(IP)*DEG2RD                                     
            DDLON = DLON(IP)*DEG2RD                                     
C                                                                       
            COSGEO = DCOS(DDLAT)*DCOS(DSPLAT)*DCOS(DDLON-DSPLON) +      
     >       DSIN(DDLAT)*DSIN(DSPLAT)                                   
C                                                                       
            IF (COSGEO.GT.1.0D0) THEN                                   
               COSGEO = 1.0D0                                           
            ELSE IF (COSGEO.LT.-1.0D0) THEN                             
               COSGEO = -1.0D0                                          
            ENDIF                                                       
C                                                                       
            GEOX = DACOS(COSGEO)                                        
C                                                                       
            NUMOR = COSGEO*COSZX(IP)-DSPSOL                             
            DENOM = DSIN(GEOX)*SQRT(1.0 - COSZX(IP)*COSZX(IP))          
C                                                                       
            IF (DENOM.EQ.0.0D0) THEN                                    
               PSIX = 1.0                                               
            ELSE                                                        
               PSIX = NUMOR/DENOM                                       
            ENDIF                                                       
C                                                                       
            IF (PSIX.GT.1.0) THEN                                       
               PSIX = 1.00                                              
            ELSE IF (PSIX.LT.-1.0) THEN                                 
               PSIX = -1.00                                             
            ENDIF                                                       
C                                                                       
            RELZX(IP) = ACOS(PSIX)*RD2DEG                               
         ENDIF                                                          
C                                                                       
         SATZX(IP) = GEOX*RD2DEG + ABS(26-IP)*2.16512                   
   10 CONTINUE                                                          
C                                                                       
C                                                                       
C...INTERPOLATE BETWEEN THE 51 ANCHOR POINTS TO EXTEND THE              
C   ANGLE VALUES ACROSS THE ENTIRE SCANLINE.                            
C                                                                       
      IFACTR = INT(FACTOR)                                              
C                                                                       
      DO 20 IPIX = 1, NPIXLS                                            
C                                                                       
        IXP = (IPIX - PXFIX1)/IFACTR + 1                                
        IF (IXP .GE. NANGS) IXP = NANGS - 1                             
        DELX = IPIX - ((IXP - 1)*IFACTR + PXFIX1)                       
C                                                                       
C                                                                       
C                                              
C                                              ! /* SATELLITE ZENITH */ 
        SATZ(IPIX) = 10.0 *                                             
     1             ((SATZX(IXP+1)-SATZX(IXP))/FACTOR*DELX+SATZX(IXP))   
C                                                                       
C                                                                       
        IF (IXP.EQ.25) THEN                    
C                                              ! /* RELATIVE AZIMUTH */ 
C                                                                       
          DELXX = IPIX - PXFIX2                                         
          RELZ(IPIX)=10.*((RELZX(25)-RELZX(24))/FACTOR*DELXX+RELZX(24)) 
C                                                                       
        ELSE IF (IXP.EQ.26) THEN                                        
C                                                                       
          DELXX = IPIX - PXFIX3                                         
          RELZ(IPIX)=10.*((RELZX(28)-RELZX(27))/FACTOR*DELXX+RELZX(27)) 
C                                                                       
        ELSE                                                            
C                                                                       
          RELZ(IPIX) = 10.0 *                                           
     1               ((RELZX(IXP+1)-RELZX(IXP))/FACTOR*DELX+RELZX(IXP)) 
C                                                                       
C                                                                       
        ENDIF                                                           
C                                                                       
C                                                                       
C...AT NADIR, STORE A FILLER VALUE FOR THE RELATIVE AZIMUTH ANGLE       
C   SINCE THE ANGLE IS UNDEFINED AT THIS POINT.                         
C                                                                       
        IF (IPIX.EQ.205) RELZ(IPIX) = -999.9                            
C                                                                       
C                                                                       
C                                                                       
   20 CONTINUE                                 
C                                              ! /* NEXT PIXEL */       
C                                                                       
C                                                                       
C                                                                       
      RETURN                                                            
C                                                                       
      END                                                               
