      SUBROUTINE SOLLOC(DYJ,ISW,RV,DDAY0)                                     
C********************************************************************** 
C                                                                       
C       SOLLOC - CALCULATES SOLAR ORBIT AND POSITIONS                   
C                                                                       
C                                                                       
C     DYJ                 R*8        JULIAN DAY FROM JD FUNCTION        
C     ISW                 I*4        SWITCH USED TO BYPASS SETTING      
C                                    OF INITIAL VALUES                  
C     RV                  R*8        SUN-EARTH DISTANCE RATIO           
C                                                                       
C                                                                       
C                                                                       
      IMPLICIT REAL*8(A-H,O-Z)                                          
C                                                                       
      INCLUDE "inc/solprm.f"                                                  
C                                                                       
      INTEGER*4      DDAY0
      REAL*8         GLNGSN         
C                                   ! /* SOLAR LONGITUDE*/              
C                                                                       
C                                                                       
      IF(ISW.EQ.0) GO TO 1                                              
      PI=3.141592653589793D0                                            
      DTR=PI/180.D0                                                     
      RTD=180.D0/PI                                                     
      CIRCL=360.D0                                                      
      AMA=358.47583D0                                                   
      BMA=0.9856002670D0                                                
      CMA=0.000150D0                                                    
      DMA=0.000003D0                                                    
      AEC=0.01675104D0                                                  
      BEC=0.00004180D0                                                  
      CEC=0.000000126D0                                                 
      ALP=281.22083D0                                                   
      BLP=0.0000470684D0                                                
      CLP=0.000453D0                                                    
      DLP=0.000003D0                                                    
      AOB=23.452294D0                                                   
      BOB=0.0130125D0                                                   
      COB=0.00000164D0                                                  
      DOB=0.000000503D0                                                 
      ARG=99.69098D0                                                    
      BRG=36000.7689250D0                                               
      CRG=0.0003870833D0                                                
      ALND=259.183275D0                                                 
      BLND=0.0529539222D0                                               
      CLND=0.002078D0                                                   
      DLND=0.000002D0                                                   
      ASLG=279.69668D0                                                  
      BSLG=0.9856473354D0                                               
      CSLG=0.000303D0                                                   
      ALNGM=270.434164D0                                                
      BLNGM=13.1763965268D0                                             
      CLNGM=0.001133D0                                                  
      DLNGM=0.0000019D0                                                 
      ISW=0                                                             
C...DETERMINE DAYS SINCE 1900                                           
  1   D=DYJ-2415020.D0                                                  
C     WRITE(6,900) D                                                    
  900 FORMAT(5X,'DAYS SINCE JAN 1, 1900 =', F8.1)                       
      DDAY0 = D       
C...DETERMINE YEARS*100 SINCE 1900                                      
      T=D/36525.D0                                                      
      TS=T*T                                                            
      TC=TS*T                                                           
C                                                                       
C...COMPUTE MEAN ANOMOLY - AVANOM                                       
C                                                                       
      AVANOM=AMA+BMA*D-CMA*TS-DMA*TC                                    
      AVANOM=DMOD(AVANOM,CIRCL)                                         
      AVANR=AVANOM*DTR                                                  
C                                                                       
C...COMPUTE ECCENTRICITY - ECCENT                                       
C                                                                       
      ECCENT=AEC-BEC*T-CEC*TS                                           
C                                                                       
C...COMPUTE ECCENTRIC ANOMOLY - ECANOM                                  
C                                                                       
      ECANOM=AVANR+ECCENT*DSIN(AVANR)+0.5D0*ECCENT*ECCENT*DSIN(2.D0     
     1 *AVANR)                                                          
C                                                                       
C...COMPUTE TRUE ANOMOLY - TRANOM                                       
C                                                                       
      IF(ECANOM.NE.PI) GO TO 3                                          
      TRANOM=180.D0                                                     
      GO TO 2                                                           
  3   TRANOM=2.D0*DATAN(DSQRT((1.D0+ECCENT)/(1.D0-ECCENT))*DTAN(ECANOM  
     1   /2.D0))*RTD                                                    
      IF(TRANOM.LT.0.D0) TRANOM=TRANOM+180.D0                           
      IF(ECANOM.GT.PI) TRANOM=TRANOM+180.D0                             
C                                                                       
C...RV = RADIUS VECTOR OF APPARENT SUN                                  
C                                                                       
  2   RV=(1.D0-ECCENT*ECCENT)/(1.D0+ECCENT*DCOS(TRANOM*DTR))            
C                                                                       
C...PRGLNG = MEAN LONGITUDE OF PERIGEE                                  
C                                                                       
      PRGLNG=ALP+BLP*D+CLP*TS+DLP*TC                                    
      PRGLNG=DMOD(PRGLNG,CIRCL)                                         
C                                                                       
C...SOLNG = LONGITUDE OF APPARENT SUN                                   
C                                                                       
      SOLNG=PRGLNG+TRANOM                                               
C                                                                       
C...ASCNDM = LONGITUDE OF MEAN LUNAR ASENDING NODE                      
C                                                                       
      ASCNDM=ALND-BLND*D+CLND*TS+DLND*TC                                
      ABN=DABS(ASCNDM)                                                  
      SGN=ASCNDM/ABN                                                    
      ASCNDM=360.D0+SGN*DMOD(ABN,CIRCL)                                 
      SNANDM=DSIN(ASCNDM*DTR)                                           
      CSANDM=DCOS(ASCNDM*DTR)                                           
C                                                                       
C...AVSLLG = LONGITUDE OF MEAN SUN                                      
C                                                                       
      AVSLLG=ASLG+BSLG*D+CSLG*TS                                        
      AVSLLG=DMOD(AVSLLG,CIRCL)                                         
      SNTWSG=DSIN(2.D0*AVSLLG*DTR)                                      
      CSTWSG=DCOS(2.D0*AVSLLG*DTR)                                      
C                                                                       
C...AVLNGM = LONGITUDE OF MEAN MOON                                     
C                                                                       
      AVLNGM=ALNGM+BLNGM*D-CLNGM*TS+DLNGM*TC                            
      AVLNGM=DMOD(AVLNGM,CIRCL)                                         
      SNTWMG=DSIN(2.D0*AVLNGM*DTR)                                      
      CSTWMG=DCOS(2.D0*AVLNGM*DTR)                                      
C                                                                       
C...GNUTAT = NUTATION IN LONGITUDE                                      
C                                                                       
      GNUTAT=-0.00475D0*SNANDM-0.000348D0*SNTWSG-0.0000694D0*SNTWMG     
C                                                                       
C...RDAPLG = REDUCTION TO APPARENT LONGITUDE. 2ND TERM = ABBERATIO      
C                                                                       
      RDAPLG=GNUTAT-0.005686D0/RV                                       
      SOLNG=SOLNG+RDAPLG                                                
      IF(SOLNG.GE.360.D0) SOLNG=SOLNG-360.D0                            
C                                                                       
C...OBLIQ = MEAN OBLIQUITY OF ECLIPTIC                                  
C                                                                       
      OBLIQ=AOB-BOB*T-COB*TS+DOB*TC                                     
C                                                                       
C...BLQNUT = NUTATION IN OBLIQUITY                                      
C                                                                       
      BLQNUT=0.002283D0*CSANDM+0.000768D0*CSTWSG+0.000234D0*CSTWMG      
      OBLIQ=OBLIQ+BLQNUT                                                
      SNDEC=DSIN(SOLNG*DTR)*DSIN(OBLIQ*DTR)                             
C                                                                       
C...DEC = DECLINATION OF APPARENT SUN                                   
C                                                                       
      SOLDEC=DASIN(SNDEC)                                              
C                                                                       
C...RASUN = RIGHT ASCENSION OF APPARENT SUN                             
C                                                                       
      RASUN=DACOS(DCOS(SOLNG*DTR)/DSQRT(1.D0-SNDEC*SNDEC))*RTD         
      IF(SOLDEC.LT.0.D0) RASUN=360.D0-RASUN                             
C                                                                       
C...RAGRN = RIGHT ASCENSION OF GREENWICH=GHA OF ARIES                   
C                                                                       
      DDINT=IDINT(D+0.5D0)                                              
      RAGRN=ARG+BRG*T+CRG*TS+360.D0*(D-DDINT+0.5D0)                     
C                                                                       
C...EQEQNX = EQUATION OF EQUINOXES = NUTATION IN RT. ASCENSION          
C                                                                       
      EQEQNX=-0.004370D0*SNANDM-0.000353D0*SNTWSG-0.000041D0*SNTWMG     
      RAGRN=RAGRN+EQEQNX                                                
      RAGRN=DMOD(RAGRN,CIRCL)                                           
C                                                                       
C...GLNGSN = GEOGRAPHIC WEST LONGITUDE OF APPARENT SUN                  
C                                                                       
      GLNGSN=RAGRN-RASUN                                                
C                                                                       
C...GLGMSN = GEOGRAPHIC WEST LONGTIUDE OF MEAN SUN                      
C                                                                       
      GLGMSN=360.D0*(D-IDINT(D))                                        
      DLNG=GLNGSN-GLGMSN                                                
      IF(DLNG.LT.-180.D0) DLNG=DLNG+360.D0                              
      IF(DLNG.GT.180.D0) DLNG=DLNG-360.D0                               
C                                                                       
C...EQTIME = EQUATION OF TIME IN HOURS                                  
C                                                                       
      EQTR=DLNG/15.0D0                                                  
C                                                                       
      RETURN                                                            
      END                                                               
