      SUBROUTINE TMS3 (ATVALU, SEDSQ, IREC)                             
C*******************************************************************    
C                                                                       
C     FUNCTION - THIS ROUTINE WILL COMPUTE A CHANNEL 3 ALBEDO           
C      FROM THE TRIPLE-MINUS-SPLIT (TMS) ALGORITHM USING                
C      CHANNELS 3,4,5 TEMPERATURES.                                     
C                                                                       
C     INPUTS:                                                           
C      C3    -   CHANNEL 3 TEMPERATURE                                  
C      C4    -   CHANNEL 4 TEMPERATURE                                  
C      C5    -   CHANNEL 5 TEMPERATURE                                  
C      COSSZA -   COSINE OF SOLAR ZENITH ANGLE                          
C      SED    -   SUN-EARTH DISTANCE                                    
C                                                                       
C     OUTPUTS:                                                          
C      IALB3  -   CHANNEL 3 ALBEDO (SCALED) IN PER CENT(0-100%)         
C                                                                       
C     CALLED FROM:                                                      
C      GACFDC                                                           
C                                                                       
C     SUBROUTINES CALLED:                                               
C      NONE                                                             
C                                                                       
C*************************************************************          
C                                                                       
C                                                                       
C...INCLUDE CONSTANT PARAMETERS.                                        
C                                                                       
      INCLUDE "inc/inclav.f"                                                  
C                                                                       
C                                                                       
C...INCLUDE COMMON AREAS.                                               
C                                                                       
      INCLUDE "inc/typdat.f"                                                 
      INCLUDE "inc/angle.f"                                                   
      INCLUDE "inc/atmisc.f"                                                 
      INCLUDE "inc/baddat.f"                                                
      INCLUDE "inc/spcnst.f"                                                  
C                                                                       
C                                                                       
      REAL*4  T3, T4, T5, ATVALU (MAXLPX,6)                             
C                                                                       
      REAL*8 TMS, DELT3, T0, C2T3, C2T0, EXPT0, EXPT3, EXDIFF, EXT31,   
     1       EXT01, EXT3T0, DELR3                                       
C                                                                       
                                                                        
      DATA PI/3.14159265/                                               
C                                                                       
C                                                                       
C                                                                       
C                                                                       
C...DECLARE CONSTANTS                                                   
C                                                                       
C     CN3 = CNU0*CNU0*CNU0                                              
      CN3 = CNUO*CNUO*CNUO                                              
      C1N3 = C1*CN3                                                     
C     C2NU = C2*CNU0                                                    
      C2NU = C2*CNUO                                                    
C                                                                       
C                                                                       
      K = 0                
C                          !!! REPLACE K WITH LOOP VARIABLE !!!         
C                                                                       
C                                                                       
      DO 20 I = 1, NPIXLS                                               
C                                                                       
         K = K+1                                                        
C                                                                       
C                                                                       
C... SKIP ALBEDO CALCULATION IF WELL INTO NIGHTTIME (SOLAR ZEN > 89.5). 
C                                                                       
         IF (SOLZ(I) .GT. ALBLIM) THEN                                  
           ATVALU (K, 3) = 0.0                                          
           GO TO 20                                                     
         END IF                                                         
C                                                                       
C                                                                       
C...UNPACK CHANNEL 3,4,5 TEMPERATURES AND COMPUTE TMS.                  
C                                                                       
         T3 = ATVALU (K, 4)                                             
         T4 = ATVALU (K, 5)                                             
         T5 = ATVALU (K, 6)                                             
C                                                                       
C                                                                       
         IF (T3.EQ.0.0 .OR. T4.EQ.0.0 .OR. T5.EQ.0.0) THEN              
C                                                                       
           ATVALU (K, 3) = 0.0                                          
C                                                                       
         ELSE IF (ABS(T4-T5) .GT. 20.0) THEN                            
C                                                                       
           ATVALU (K, 3) = 0.0                                          
C                                                                       
C                                                                       
         ELSE                                                           
C                                                                       
           TMS = A*T3 + B*T4 + C*T5 +D                                  
C                                                                       
           DELT3 = TMS/A                                                
           T0 = T3-DELT3                                                
           C2T3 = C2NU/T3                                               
           C2T0 = C2NU/T0                                               
           EXPT0 = DEXP(C2T0)                                           
           EXPT3 = DEXP(C2T3)                                           
           EXDIFF = EXPT0-EXPT3                                         
           EXT31 = EXPT3-1.0                                            
           EXT01 = EXPT0-1.0                                            
           EXT3T0 = EXT31*EXT01                                         
           DELR3 = C1N3*(EXDIFF/EXT3T0)                                 
C                                                                       
           ATVALU (K, 3) = ((DELR3*SEDSQ)/(COSSZA(I)*S3PI))*100.        
C                                                                       
         ENDIF                                                          
C                                                                       
C                                                                       
   20 CONTINUE               
C                            !  /* GO ON TO NEXT PIXEL */               
C                                                                       
C                                                                       
C                                                                       
      RETURN                                                            
      END                                                               
