      SUBROUTINE TUR (LST,PIX,ANGL,RS,PF,TH,CO)                       
C********************************************************************** 
C                                                                       
C      FUNCTION - THERMAL UNIFORMITY RESTORAL TEST. THIS TEST           
C      USES THERMAL CHANNEL 5 TO DETERMINE IF CLOUD/MIXED PIXEL         
C      ARRAY FROM THE C3AT TEST IS TO BE RESTORED TO CLEAR. MOSTLY      
C      USED OVER OCEAN SURFACES TO RESTORE CLEAR SPECULAR AREAS.        
C                                                                       
C      INPUT:                                                           
C       TEST  -   TEST BY WHICH THE TUR HAS BEEN PASSED TO              
C       LST   -   LAND/SEA TAG                                          
C       PIX   -   CHANNEL 5 PIXEL ARRAY                                 
C       ANGL-   SOLAR/SATELLITE/RELATIVE AZIMUTH ANGLES FOR 2X2       
C       PF    -   PREVIOUS FLAG FROM C3AT TEST                          
C       TH    -   LAND/OCEAN THRESHOLDS                                 
C                                                                       
C      OUTPUT:                                                          
C       RS    -   RESTORE FLAG. 0=CLEAR,1 OR 2 =CLOUD/MIXED             
C       CO    -   CLEAR/CLOUD FLAG                                      
C                                                                       
C********************************************************************** 
C                                                                       
      INCLUDE "inc/apath.f"                                                  
      INCLUDE "inc/spcnst.f"                                                  
C                                                                       
C                                                                       
      CHARACTER*6  TNAME(4)                                             
C                                                                       
      REAL*4  PIX(4), GAMMA,  SAT, REL, DTR, MAX, MIN 
C                                                                       
      INTEGER*4 RS,PF,LST                                               
      INTEGER*2 CO(4),ANGL(4,3)                                       
C                                                                       
      DATA  DTR/0.017453293/                                            
C                                                                       
C     DATA  ANG   / 10.0,     30.0,     10.0,     30.0/                 
      DATA  TNAME /'RGCT  ', 'RUT   ', 'RRCT  ', 'C3AT  '/              
C                                                                       
C---------------------------------------------------------------------  
C                                                                       
C...IF 2X2 ARRAY IS OVER OCEAN THAN THE GAMMA ANGLE (SUN GLINT CONE)    
C...IS DETERMINED TO SEE IF THE ARRAY LAYS WITHIN IT.                   
C                                                                       
C...FIRST, DETERMINE PROPER ANGLE LIMIT TO USE BASED ON THE ORIGINAL    
C   CLAVR CLOUD TEST WHICH WAS PASSED.                                  
C                                                                       
      IXANG = 0                                                         
C                                                                       
      DO 10 ITEST = 1, 4                                                
        IF (OTEST .EQ. TNAME(ITEST)) IXANG = ITEST                      
   10 CONTINUE                                                          
C                                                                       
      IF (IXANG .EQ. 0) THEN                                            
        CALL EMESG (22, 'TUR', 'OK')                                    
        GO TO 99                                                        
      END IF                                                            
C                                                                       
C                                                                       
      IF (LST.EQ.0) THEN                     
C                                            ! /* OCEAN ONLY */         
C                                                                       
         DO 20 IP = 1, 4                                                
            COSSZA = ANGL(IP,1)/1000.                                 
            SAT = DTR*ANGL(IP,2)/10.                                  
            REL = DTR*ANGL(IP,3)/10.                                  
            SINSZA = SQRT(1.0-COSSZA*COSSZA)                            
C                                                                       
            IF (REL.LT.0.0) GO TO 20                                    
            GAMMA= ACOS(COSSZA*COS(SAT) + SINSZA*SIN(SAT)*COS(REL))     
            GAMMA = GAMMA/DTR                                           
            IF (GAMMA .GT. GAMLIM(IXANG)) GO TO 99                         
   20    CONTINUE                                                       
C                                                                       
      ENDIF                                                             
C                                                                       
C...USE THE FIRST PIXEL TO SET THE MIN AND MAX VALUES                   
C                                                                       
      MIN=PIX(1)                                                        
      MAX=MIN                                                           
C                                                                       
C...DO THE OTHER 3 TO GET THE MAX/MIN VALUE FOR ALL FOUR                
C                                                                       
      DO 30 I=2,4                                                       
        IF(PIX(I).GT.MAX) MAX=PIX(I)                                    
        IF(PIX(I).LT.MIN) MIN=PIX(I)                                    
   30 CONTINUE                                                          
C                                                                       
C...IF THE DIFFERENCE IS LESS THAN THE THRESHOLD THEN THE ARRAY IS NOT  
C...RESTORED TO CLEAR                                                   
C                                                                       
      IF((MAX-MIN).LT.TH) GO TO 100                                     
   99 RS=PF                                                             
      GO TO 999                                                         
C                                                                       
  100 RS=0                                                              
      PF=0                                                              
      CO(1) = 0                                                         
      CO(2) = 0                                                         
      CO(3) = 0                                                         
      CO(4) = 0                                                         
  999 CONTINUE                                                          
C                                                                       
      RETURN                                                            
      END                                                               
