! $Id: clavr_mod.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!-------------------------------------------------------------------------
! CLAVR ROUTINES
!-------------------------------------------------------------------------
module CLAVR_ROUTINES
 use CONSTANTS
 use AVHRR_ROUTINES
 use EQUAL_AREA_GRID
 implicit none
 private
 public:: COMPILE_GRIDCELL_ARRAYS, COMPUTE_WRITE_GRIDCELL_ARRAYS, &
          CREATE_GRIDCELL_ARRAYS, &
          DESTROY_GRIDCELL_ARRAYS,INITIALIZE_GRIDCELL_ARRAYS,&
          READ_GRIDCELL_SFCTYPE


 !------- grid-cell array declarations
real(kind=real4), dimension(:,:), allocatable, public:: data
integer (kind=int1), dimension(:), allocatable, public:: cell_sfc_type, cell_type,cell_asc_des,cell_ch3a_on
real(kind=real4), dimension(:,:,:), allocatable, public:: cell_mean,cell_std
real(kind=real4), dimension(:,:), allocatable, public:: cell_wat_cld_mean,cell_ice_cld_mean,cell_cld_mean, &
                                                cell_wat_cld_std,cell_ice_cld_std,cell_cld_std
integer (kind=int4), dimension(:), allocatable, public:: cell_orbit
real(kind=real4), dimension(:),allocatable, public:: cell_mean_sst,cell_mean_ndvi,&
                            cell_std_sst,cell_std_ndvi,cell_time, &
                            cell_mean_ist,cell_std_ist,cell_mean_lst,cell_std_lst,&
                            cell_mean_nddi,cell_std_nddi

real(kind=real4), dimension(:), allocatable, public:: satzen_mean,solzen_mean,&
                                               relaz_mean,glint_mean,zsfc_mean
integer(kind=int2), dimension(:,:,:), allocatable, public:: cell_mask_count
integer(kind=int2), dimension(:,:), allocatable, public:: cell_type_count
integer(kind=int2), dimension(:), allocatable, public:: cell_pixel_count, &
                                   cell_snow_count,cell_term_count,&
                                   cell_fire_count,cell_dust_count, &
                                   cell_ocean_count,cell_land_count, &
                                   cell_day,cell_year
integer(kind=int2), dimension(:,:), allocatable, public:: cell_water_count_cm2,cell_water_count_cm3
integer(kind=int1), dimension(:), allocatable, public::blank_clavr


 contains
!-----------------------------------------------------------------------
! GRIDCELL ROUTINES
!----------------------------------------------------------------------- 
subroutine CREATE_GRIDCELL_ARRAYS(nchan_clavr,nmask_clavr,ntype,num_pix, &
                                  num_blank_clavr_bytes,ncell)

  integer(kind=int4),intent(in):: nchan_clavr,nmask_clavr,ntype, &
                                  num_blank_clavr_bytes,ncell
  integer(kind=int2),intent(in):: num_pix

!----- temp array used for mapping pixel data
  print *,"in create gridcell arrays ", nchan_clavr, num_pix
  allocate(data(nchan_clavr,num_pix))
  data = 0.0

!-- grid-cell arrays
  allocate(cell_mean(nchan_clavr,0:nmask_clavr-1,ncell), &
      cell_cld_mean(nchan_clavr,ncell),&
      cell_wat_cld_mean(nchan_clavr,ncell),&
      cell_ice_cld_mean(nchan_clavr,ncell),&
      cell_std(nchan_clavr,0:nmask_clavr-1,ncell), &
      cell_cld_std(nchan_clavr,ncell), &
      cell_wat_cld_std(nchan_clavr,ncell), &
      cell_ice_cld_std(nchan_clavr,ncell), &
      cell_mask_count(nchan_clavr,0:nmask_clavr-1,ncell), &
      cell_asc_des(ncell), cell_ch3a_on(ncell),&
      cell_type(ncell), &
      cell_type_count(0:ntype-1,ncell), &
      cell_pixel_count(ncell),&
      cell_snow_count(ncell), &
      cell_ocean_count(ncell),cell_land_count(ncell), &
      cell_fire_count(ncell),cell_dust_count(ncell), &
      cell_water_count_cm2(nchan_clavr,ncell),cell_water_count_cm3(nchan_clavr,ncell),&
      satzen_mean(ncell),solzen_mean(ncell),&
      relaz_mean(ncell), glint_mean(ncell), &
      zsfc_mean(ncell), &
      cell_orbit(ncell),cell_time(ncell), &
      cell_term_count(ncell), &
      cell_mean_sst(ncell),cell_mean_ndvi(ncell),cell_mean_nddi(ncell),&
      cell_std_sst(ncell),cell_std_ndvi(ncell),cell_std_nddi(ncell),&
      cell_mean_lst(ncell),cell_std_lst(ncell),&
      cell_mean_ist(ncell),cell_std_ist(ncell),&
      cell_day(ncell),cell_year(ncell))

allocate(blank_clavr(num_blank_clavr_bytes))
blank_clavr = 0

allocate(cell_sfc_type(ncell))
cell_sfc_type = 0

end subroutine CREATE_GRIDCELL_ARRAYS

!-------------------------------------------------
subroutine INITIALIZE_GRIDCELL_ARRAYS()

  cell_mean = 0.0
  cell_cld_mean = 0.0
  cell_wat_cld_mean = 0.0
  cell_ice_cld_mean = 0.0
  cell_std = 0.0
  cell_cld_std = 0.0
  cell_wat_cld_std = 0.0
  cell_ice_cld_std = 0.0
  satzen_mean =  0.0
  solzen_mean = 0.0
  relaz_mean =  0.0
  glint_mean = 0.0
  zsfc_mean = 0.0
  cell_orbit = missing_value_int4
  cell_time =  0.0
  cell_pixel_count = 0
  cell_snow_count = 0
  cell_term_count = 0
  cell_fire_count = 0
  cell_dust_count = 0
  cell_mask_count = 0
  cell_asc_des = 0
  cell_ch3a_on =  missing_value_int1
  cell_type_count = 0
  cell_mean_sst = 0.0
  cell_mean_ndvi = 0.0
  cell_mean_nddi = 0.0
  cell_std_sst = 0.0
  cell_std_ndvi = 0.0
  cell_std_nddi = 0.0
  cell_mean_ist = 0.0
  cell_std_ist = 0.0
  cell_mean_lst = 0.0
  cell_std_lst = 0.0
  cell_ocean_count = 0
  cell_land_count = 0
  cell_water_count_cm2 = 0
  cell_water_count_cm3 = 0
  cell_type = missing_value_int1
  cell_day = missing_value_int2
  cell_year = missing_value_int2

end subroutine INITIALIZE_GRIDCELL_ARRAYS


!-------------------------------------------------------------------------------
!
!-------------------------------------------------------------------------------
 subroutine DESTROY_GRIDCELL_ARRAYS()
     deallocate(cell_asc_des,cell_ch3a_on,&
      cell_mean,cell_std,cell_wat_cld_mean,cell_ice_cld_mean,cell_cld_mean,&
      cell_wat_cld_std, cell_ice_cld_std,cell_cld_std,cell_sfc_type,cell_type, &
      cell_type_count,cell_mask_count,cell_pixel_count, &
      cell_snow_count,cell_fire_count,cell_dust_count,cell_term_count, &
      satzen_mean,solzen_mean,relaz_mean,glint_mean,zsfc_mean, &
      cell_orbit,cell_time,cell_mean_sst,cell_mean_ndvi,cell_mean_nddi,&
      cell_std_sst,cell_std_ndvi,cell_std_nddi,cell_day,cell_year, &
      cell_mean_lst,cell_std_lst,cell_mean_ist,cell_std_ist,data)
 end subroutine DESTROY_GRIDCELL_ARRAYS


 subroutine COMPILE_GRIDCELL_ARRAYS(nmask_clavr,num_pix)
   integer(kind=int4), intent(in):: nmask_clavr
   integer(kind=int2), intent(in):: num_pix
   integer(kind=int4):: i,j,ipix,imask,iterm,icoast,ifire,isnow,idust,itype,icell

 
 scan_loop_map:  do j = 1,2

!----- populate the data array
       data(1,:) = alb1(:,j)
       data(2,:) = alb2(:,j)
       if (ch3a_on(j) == 1) then
         data(3,:) = alb3a(:,j)
       else
        data(3,:) = missing_value_real4
       endif
       if (ch3a_on(j) == 0) then
         data(4,:) = rad3b(:,j)
       else
        data(4,:) = missing_value_real4
       endif
       data(5,:) = rad4(:,j)
       data(6,:) = rad5(:,j)

!-- unpack mask bits to get at icoast and iterm
      do i = 1,8
       data_mask(i,:,:) = modulo(ishft(data_bit,i-8),2_int1)
       mask(i,:,:) = modulo(ishft(mask_byte_1,i-8),2_int1)
       mask(i+8,:,:) = modulo(ishft(mask_byte_2,i-8),2_int1)
       mask(i+16,:,:) = modulo(ishft(mask_byte_3,i-8),2_int1)
      enddo
!-------------------------------------------------
! loop through each pixel on scanline and map it
!-------------------------------------------------

 pix_loop_map:    do ipix = 1,num_pix

           imask = cld_mask(ipix,j)
           itype = cld_type(ipix,j)
           iterm = data_mask(2,ipix,j)
           icoast = data_mask(4,ipix,j)
           isnow = mask(23,ipix,j)
           ifire = mask(24,ipix,j)
           idust = mask(22,ipix,j)

!----- cloud type each pixel

          call FIND_EQUAL_AREA_CELL(lat(ipix,j),lon(ipix,j),icell)


!--------------------------------------------------------------------------
! map data for each pixel only if the land mask of that pixel is consistent
! with the predominant sfc type and excluding coast - this is the PATMOS logic
!---------------------------------------------------------------------------
 if ( (land_mask(ipix,j) == min(1_int1,cell_sfc_type(icell))) .and.  &   !sfc_type_check
                     (icoast == 0) ) then

           cell_asc_des(icell) = ascend(j)                     !the last scan line determines asc/des
           if (cell_ch3a_on(icell) < 0) then                   !initially cell_ch3a_on is set to a negative number
             cell_ch3a_on(icell) = ch3a_on(j)
           endif
           if (cell_ch3a_on(icell) /= ch3a_on(j)) then         !if any nonuniformity of this value, set to 2
             cell_ch3a_on(icell) = 2
           endif

           cell_pixel_count(icell) = cell_pixel_count(icell) + 1

           if (iterm == 1) then
             cell_term_count(icell) = cell_term_count(icell) + 1
           endif

           if (ifire == 1) then
             cell_fire_count(icell) = cell_fire_count(icell) + 1
           endif
           if (idust == 1) then
             cell_dust_count(icell) = cell_dust_count(icell) + 1
           endif

!------ compile mean sst or ndvi for clear pixels
!note added, ndvi check on sst to mitigate navigation errors
           if (imask == 0) then
            if (isnow == 0) then
               if ((land_mask(ipix,j)==0).and.(ndvi(ipix,j) < 0.0)) then
                 cell_mean_sst(icell) = cell_mean_sst(icell) + sst(ipix,j)
                 cell_std_sst(icell) = cell_std_sst(icell) + sst(ipix,j)**2
                 cell_ocean_count(icell) = cell_ocean_count(icell) + 1
               else
                 cell_mean_ndvi(icell) = cell_mean_ndvi(icell) + ndvi(ipix,j)
                 cell_std_ndvi(icell) = cell_std_ndvi(icell) + ndvi(ipix,j)**2
                 cell_mean_nddi(icell) = cell_mean_nddi(icell) + nddi(ipix,j)
                 cell_std_nddi(icell) = cell_std_nddi(icell) + nddi(ipix,j)**2
                 cell_mean_lst(icell) = cell_mean_lst(icell) + lst(ipix,j)
                 cell_std_lst(icell) = cell_std_lst(icell) + lst(ipix,j)**2
                 cell_land_count(icell) = cell_land_count(icell) + 1
               endif
            else
                 cell_mean_ist(icell) = cell_mean_ist(icell) + ist(ipix,j)
                 cell_std_ist(icell) = cell_std_ist(icell) + ist(ipix,j)**2
                 cell_snow_count(icell) = cell_snow_count(icell) + 1
            endif
           endif

!-- compile mean and standard deviation of observations for each cloud mask value
              cell_mask_count(:,imask,icell) = cell_mask_count(:,imask,icell) + 1
              cell_type_count(itype,icell) = cell_type_count(itype,icell) + 1
              cell_mean(:,imask,icell) = cell_mean(:,imask,icell) + data(:,ipix)
              cell_std(:,imask,icell) = cell_std(:,imask,icell) + data(:,ipix)**2

!---- compile water and ice cloud radiances for cloudy pixels only
          if (imask == 3) then
            if (itype <=3) then
              cell_wat_cld_mean(:,icell) = cell_wat_cld_mean(:,icell) + data(:,ipix)
              cell_wat_cld_std(:,icell) = cell_wat_cld_std(:,icell) + data(:,ipix)**2
            else
              cell_ice_cld_mean(:,icell) = cell_ice_cld_mean(:,icell) + data(:,ipix)
              cell_ice_cld_std(:,icell) = cell_ice_cld_std(:,icell) + data(:,ipix)**2
            endif
          endif


! do for imask - nmask-1 which is reserved for all pixels
          cell_mask_count(:,nmask_clavr-1,icell) = cell_mask_count(:,nmask_clavr-1,icell) + 1
          cell_mean(:,nmask_clavr-1,icell) = cell_mean(:,nmask_clavr-1,icell)+ data(:,ipix)
          cell_std(:,nmask_clavr-1,icell) =  cell_std(:,nmask_clavr-1,icell) + data(:,ipix)**2

!--------- compute fraction of cloudy pixels that are water
           if (imask == 2 .and. itype <= 3) then
               cell_water_count_cm2(:,icell) = cell_water_count_cm2(:,icell) + 1
           endif
           if (imask == 3 .and. itype <= 3) then
               cell_water_count_cm3(:,icell) = cell_water_count_cm3(:,icell) + 1
           endif

!----------- map time and angles for each pixel
           cell_time(icell) = cell_time(icell) + time(j)/60.0/60.0/1000.0
           satzen_mean(icell) = satzen_mean(icell) + satzen(ipix,j)
           solzen_mean(icell) = solzen_mean(icell) + solzen(ipix,j)
           relaz_mean(icell) = relaz_mean(icell) + relaz(ipix,j)
           glint_mean(icell) = glint_mean(icell) + glintzen(ipix,j)
           zsfc_mean(icell) = zsfc_mean(icell) + zsfc(ipix,j)

        end if  !sfc_type_check

       end do  pix_loop_map
     end do scan_loop_map

 end subroutine COMPILE_GRIDCELL_ARRAYS 


!----------------------------------------------------------------------------------
! write out the records the clavr file
!----------------------------------------------------------------------------------
 subroutine COMPUTE_WRITE_GRIDCELL_ARRAYS(orbit_number,start_day,end_day, &
                                      start_year,end_year,count_thresh, &
                                      nmask_clavr,sc_id_new,nrec_clavr)
  integer(kind=int4),intent(in):: orbit_number,count_thresh,nmask_clavr
  integer(kind=int2),intent(in):: start_day,start_year,end_day,end_year
  integer(kind=int1),intent(in):: sc_id_new
  integer(kind=int4), intent(out):: nrec_clavr



  integer(kind=int4):: icell
  

!--- determine predominant cloud type in each cell

    do icell = 1, nequal
      if (cell_pixel_count(icell) > 0) then
        cell_type(icell) = minval(maxloc(cell_type_count(:,icell))) - 1
        cell_time(icell) = cell_time(icell) / cell_pixel_count(icell)
        cell_orbit(icell) = orbit_number
        if (cell_time(icell) < 24.00) then
            cell_day(icell) = start_day
        else
            cell_day(icell) = end_day
       endif
       if (cell_day(icell) < 366) then
            cell_year(icell) = start_year
       else
            cell_year(icell) = end_year
       endif
      else
       cell_type(icell) = missing_value_int1
       cell_time(icell) = missing_value_real4
       cell_day(icell) = missing_value_int2
       cell_year(icell) = missing_value_int2
       cell_orbit(icell) = missing_value_int4
      endif
    enddo

!--- based on number of ice/water pixels, select water or ice as the cloudy radiance

!--- if water most common, set cloudy radiance to ice cloud values
    where ((cell_water_count_cm3 >= cell_mask_count(:,3,:) - cell_water_count_cm3) .and.  &
           (cell_water_count_cm3 > count_thresh))
          cell_cld_mean = cell_wat_cld_mean/cell_water_count_cm3
          cell_cld_std = sqrt( max(0.0,(cell_wat_cld_std/cell_water_count_cm3 - &
             cell_cld_mean**2)))
    elsewhere
          cell_cld_mean = missing_value_real4
          cell_cld_std = missing_value_real4
    endwhere

!--- if ice most common, set cloudy radiance to ice cloud values
    where ((cell_water_count_cm3 < cell_mask_count(:,3,:) - cell_water_count_cm3) .and.  &
           (cell_mask_count(:,3,:) - cell_water_count_cm3 > count_thresh))
          cell_cld_mean = cell_ice_cld_mean/(cell_mask_count(:,3,:)-cell_water_count_cm3)
          cell_cld_std = sqrt( max(0.0,(cell_ice_cld_std / &
             (cell_mask_count(:,3,:)-cell_water_count_cm3) - cell_cld_mean**2)))
    elsewhere
          cell_cld_mean = missing_value_real4
          cell_cld_std = missing_value_real4
    endwhere

!--- compute means and standard deviations in each grid-cell


!----- cldmask = 0
  where (cell_mask_count(:,0,:) >= count_thresh)
          cell_mean(:,0,:) = cell_mean(:,0,:)/cell_mask_count(:,0,:)
          cell_std(:,0,:) = sqrt( max(0.0,(cell_std(:,0,:)/cell_mask_count(:,0,:) - &
             cell_mean(:,0,:)**2)))
  elsewhere
          cell_mean(:,0,:) = missing_value_real4
          cell_std(:,0,:) = missing_value_real4
  endwhere

!----- cldmask = 1
  where (cell_mask_count(:,1,:) >= count_thresh)
          cell_mean(:,1,:) = cell_mean(:,1,:)/cell_mask_count(:,1,:)
          cell_std(:,1,:) = sqrt( max(0.0,(cell_std(:,1,:)/cell_mask_count(:,1,:) - &
             cell_mean(:,1,:)**2)))
  elsewhere
          cell_mean(:,1,:) = missing_value_real4
          cell_std(:,1,:) = missing_value_real4
  endwhere

!----- cldmask = 2
  where (cell_mask_count(:,2,:) >= count_thresh)
          cell_mean(:,2,:) = cell_mean(:,2,:)/cell_mask_count(:,2,:)
          cell_std(:,2,:) = sqrt( max(0.0,(cell_std(:,2,:)/cell_mask_count(:,2,:) - &
             cell_mean(:,2,:)**2)))
  elsewhere
          cell_mean(:,2,:) = missing_value_real4
          cell_std(:,2,:) = missing_value_real4
  endwhere


!----- cldmask = 3
  where (cell_mask_count(:,3,:) >= count_thresh)
          cell_mean(:,3,:) = cell_mean(:,3,:)/cell_mask_count(:,3,:)
          cell_std(:,3,:) = sqrt( max(0.0,(cell_std(:,3,:)/cell_mask_count(:,3,:) - &
             cell_mean(:,3,:)**2)))
  elsewhere
          cell_mean(:,3,:) = missing_value_real4
          cell_std(:,3,:) = missing_value_real4
  endwhere


!----- cldmask = 4
  where (cell_mask_count(:,4,:) >= count_thresh)
          cell_mean(:,4,:) = cell_mean(:,4,:)/cell_mask_count(:,4,:)
          cell_std(:,4,:) = sqrt( max(0.0,(cell_std(:,4,:)/cell_mask_count(:,4,:) - &
             cell_mean(:,4,:)**2)))
  elsewhere
          cell_mean(:,4,:) = missing_value_real4
          cell_std(:,4,:) = missing_value_real4
  endwhere

!-------
!------- compute means of angles
  where (cell_pixel_count >= count_thresh)
   satzen_mean = satzen_mean / cell_pixel_count
   solzen_mean = solzen_mean / cell_pixel_count
   relaz_mean = relaz_mean / cell_pixel_count
   glint_mean = glint_mean / cell_pixel_count
   zsfc_mean = zsfc_mean / cell_pixel_count
  elsewhere
   satzen_mean = missing_value_real4
   solzen_mean = missing_value_real4
   relaz_mean = missing_value_real4
   glint_mean = missing_value_real4
   zsfc_mean = missing_value_real4
  end where

!------ compute mean and std sst and ndvi
   where((cell_ocean_count >= count_thresh).and.(cell_sfc_type == 0))
     cell_mean_sst = cell_mean_sst / cell_ocean_count
     cell_std_sst = sqrt( max(0.0,(cell_std_sst / cell_ocean_count)-cell_mean_sst**2))
   elsewhere
     cell_mean_sst = missing_value_real4
     cell_std_sst = missing_value_real4
   endwhere
   where((cell_land_count >= count_thresh).and.(cell_sfc_type > 0))
     cell_mean_ndvi = cell_mean_ndvi / cell_land_count
     cell_std_ndvi = sqrt( max(0.0,(cell_std_ndvi / cell_land_count)-cell_mean_ndvi**2))
     cell_mean_nddi = cell_mean_nddi / cell_land_count
     cell_std_nddi = sqrt( max(0.0,(cell_std_nddi / cell_land_count)-cell_mean_nddi**2))
     cell_mean_lst = cell_mean_lst / cell_land_count
     cell_std_lst = sqrt( max(0.0,(cell_std_lst / cell_land_count)-cell_mean_lst**2))
   elsewhere
     cell_mean_ndvi = missing_value_real4
     cell_std_ndvi = missing_value_real4
     cell_mean_nddi = missing_value_real4
     cell_std_nddi = missing_value_real4
     cell_mean_lst = missing_value_real4
     cell_std_lst = missing_value_real4
   endwhere
   where(cell_snow_count >= count_thresh)
     cell_mean_ist = cell_mean_ist / cell_snow_count
     cell_std_ist = sqrt( max(0.0,(cell_std_ist / cell_snow_count)-cell_mean_ist**2))
   elsewhere
     cell_mean_ist = missing_value_real4
     cell_std_ist = missing_value_real4
   endwhere

!---- write out gridded data

nrec_clavr = 1
do icell = 1, nequal

!--- test for ch3a transition and reset values
     if (cell_ch3a_on(icell) == 2) then
         cell_mean(3,:,icell) = missing_value_real4
         cell_std(3,:,icell) = missing_value_real4
         cell_mean(4,:,icell) = missing_value_real4
         cell_std(4,:,icell) = missing_value_real4
     endif


     if (cell_mask_count(1,nmask_clavr-1,icell) >= count_thresh) then
      write (unit=13,rec=nrec_clavr+1)  icell, sc_id_new, cell_asc_des(icell), &
             cell_ch3a_on(icell),cell_orbit(icell), &
             cell_time(icell),cell_day(icell),cell_year(icell),&
             cell_sfc_type(icell), &
             satzen_mean(icell), &
             solzen_mean(icell),relaz_mean(icell), &
             glint_mean(icell),zsfc_mean(icell), cell_type(icell), &
             cell_mask_count(1,:,icell), &        !only write one out
             cell_water_count_cm2(1,icell), &     !ditto
             cell_water_count_cm3(1,icell), &     !ditto
             cell_term_count(icell), cell_snow_count(icell),&
             cell_fire_count(icell), cell_dust_count(icell), &
             cell_mean(:,0,icell), cell_std(:,0,icell), &
             cell_mean(:,1,icell), cell_std(:,1,icell), &
             cell_mean(:,2,icell), cell_std(:,2,icell), &
             cell_mean(:,3,icell), cell_std(:,3,icell), &
             cell_mean(:,4,icell), cell_std(:,4,icell), &
             cell_cld_mean(:,icell), cell_cld_std(:,icell), &
             cell_mean_sst(icell), cell_std_sst(icell), &
             cell_mean_ndvi(icell), cell_std_ndvi(icell),&
             cell_mean_nddi(icell), cell_std_nddi(icell),&
             cell_mean_lst(icell), cell_std_lst(icell),&
             cell_mean_ist(icell), cell_std_ist(icell),&
             cell_type_count(:,icell), &
             blank_clavr
             nrec_clavr = nrec_clavr + 1
           endif
  enddo
  nrec_clavr = nrec_clavr -1

  print *,"number of records written to clavr file = ", nrec_clavr

  end subroutine COMPUTE_WRITE_GRIDCELL_ARRAYS


!------------------------------------------------------------------
! READ_GRIDCELL_SFCTYPE - reads in a precomputed file containing 
!    the dominant surface type for each grid-cell
!
! input
!   dlat - the resolution of the grid-cells
!   data_dir - the directory containing the sfc_data subdirectory 
!
! output - cell_sfc_type, passed only through shared memory
!
!-------------------------------------------------------------------
  subroutine READ_GRIDCELL_SFCTYPE(dlat,data_dir)
   real(kind=real4), intent(in):: dlat
   character(len=*), intent(in):: data_dir
   character(len=72):: cell_sfc_file

!--- determine file to use

    if (dlat == 0.25) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_025.dat"
    elseif (dlat == 0.5) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_05.dat"
    elseif (dlat == 1.0) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_1.dat"
    elseif (dlat == 2.5) then
         cell_sfc_file = trim(data_dir)//"sfc_data/cell_sfc_type_25.dat"
    else
         print *,"no sfc file available for this resolution, stopping"
         stop
     endif



!------- read in grid-cell surface
    print *,"***PROCESS_AVHRR - unit 7 read ==>",data_dir,cell_sfc_file
    print *,"Static Input Data"
    open(unit=7,file=trim(cell_sfc_file),status="old",action="read", &
         access="direct",recl=nequal)
    read(unit=7,rec=1) cell_sfc_type
    close(unit=7)


     print *, "read in cell_sfc_type file ", dlat, trim(cell_sfc_file)

 end subroutine READ_GRIDCELL_SFCTYPE

end module CLAVR_ROUTINES
