! $Id: clavr_thresh.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!---------------------------------------------------------------------
!
! Cloud mask Thresholds for CLAVR-x
!
!  Note, that most thresholds are stored in 2x2 arrays such that
!  0,0 - night, ocean
!  1,0 - day, ocean
!  0,1 - night, land
!  1,1 - day, land
!
! modified Nov 02 - added TIGR derived FMFT and TMFT thresholds
!
!----------------------------------------------------------------------
module CLOUD_MASK_THRESHOLDS
 use CONSTANTS
 implicit none
 private
 public:: READ_THRESHOLDS,FMFT_THRESHOLD,ULST_THRESHOLD,CIRT_THRESHOLD

!------ these variables pertain to new fmft and tmft lookup tables
 integer, public, parameter:: nt4_lut = 14, nzen_lut = 4
 real(kind=real4), dimension(nt4_lut), save, public:: t4_lut
 real(kind=real4), dimension(nzen_lut), save, public:: zen_lut
 real(kind=real4), dimension(nzen_lut, nt4_lut), save, public:: fmft_hi_land_lut, &
                   fmft_lo_land_lut,fmft_hi_ocean_lut,fmft_lo_ocean_lut, &
                   tmft_hi_land_lut,tmft_lo_land_lut,tmft_hi_ocean_lut,tmft_lo_ocean_lut

!--- key's fmft parameters
integer,parameter, public:: nfmft_key = 13
real(kind=real4), dimension(nfmft_key), save, public:: ct_thresh,wt_thresh,zc


!--------- these variable pertain to old CLAVR-1 lookup tables
 integer, private, parameter:: nfmft = 121,nulst=121,ncirt=121
 real(kind=real4), dimension(nfmft), save, private:: fmflnd, fmfocn
 real(kind=real4), dimension(nulst), save, private:: ulst
 real(kind=real4), dimension(ncirt), save, private:: cirt

!-------- these variables pertain to ocean reflectance tables
 integer, public, parameter:: nsolzen_ocn_ref=9,nsatzen_ocn_ref=9,nrelaz_ocn_ref=19
 real(kind=real4), public, save:: del_solzen_ocn_ref, del_satzen_ocn_ref, &
                                  del_relaz_ocn_ref
 real(kind=real4), dimension(nsolzen_ocn_ref), save, public:: solzen_ocn_ref
 real(kind=real4), dimension(nsatzen_ocn_ref), save, public:: satzen_ocn_ref
 real(kind=real4), dimension(nrelaz_ocn_ref), save, public:: relaz_ocn_ref
 real(kind=real4), dimension(nsolzen_ocn_ref,nsatzen_ocn_ref,nrelaz_ocn_ref), &
               save, public:: vis2_ocn_ref,vis21_ocn_ref,nir1_ocn_ref, &
                              nir2_ocn_ref

!--------------- these are the fixed land/ocean thresholds
real(kind=real4), dimension(0:1,0:1), save, public::  &
               vis_ref_thresh,win_tmp_thresh, &
               win_tmp_uni_clr,win_tmp_uni_cld,win_tmp_uni_ucld,&
               vis_ref_uni_clr,vis_ref_uni_cld,vis_ref_uni_ucld, &
               polar_lat,win_tmp_uni_res,nir_ref_uni_clr

real(kind=real4), dimension(0:1,0:1,0:2), save, public:: nir_ref_thresh

real(kind=real4),save, public:: glint_angle,glint_angle_lo_sun, &
                 vis_ref_ratio_snow_min,vis_ref_ratio_snow_max

!real(kind=real4), dimension(0:1,0:1,0:2), parameter, public :: nir_ref_thresh = &
!                  reshape((/1000.0, 1000.0, 1000.0, 1000.0, 1000.0, 1000.0, 10000.0, 10000.0, &
!                  1000.0, 1000.0, 1000.0, 1000.0/),(/2,2,3/))

!------umd stuff
integer, parameter, public:: ntype_umd=15,nscat_umd=7
real(kind=real4), dimension(0:ntype_umd-1,nscat_umd), public,save:: ch1_mean_umd,ch21_mean_umd,&
                                                                    ch1_std_umd
real(kind=real4), dimension(0:ntype_umd-1), public,save:: std_factor
real(kind=real4), dimension(0:nscat_umd-1), public,save:: scat_angle_umd
real(kind=real4), public, parameter:: min_scat_umd = 75.0,delta_scat_umd = 15.0

 contains

!---------------------------------------------------------------------
function FMFT_THRESHOLD(t4,land) result(fmft_thresh)
  real(kind=real4), intent(in):: t4
  logical, intent(in):: land
  real (kind=real4):: fmft_thresh
  integer:: it4

      it4 = min(max(1,nint(t4 - 200.0)),nfmft)
      if (land .eqv. .true.) then
       fmft_thresh = fmflnd(it4)
      else
       fmft_thresh = fmfocn(it4)
      endif
 end function FMFT_THRESHOLD

!---------------------------------------------------------------------
function ULST_THRESHOLD(t4) result(ulst_thresh)
  real(kind=real4), intent(in):: t4
  real (kind=real4):: ulst_thresh
  integer:: it4
      it4 = min(max(1,nint(t4 - 200.0)),nulst)
      ulst_thresh = ulst(it4)
 end function ULST_THRESHOLD
!--------------------------------------------------------------------
function CIRT_THRESHOLD(t4) result(cirt_thresh)
  real(kind=real4), intent(in):: t4
  real (kind=real4):: cirt_thresh
  integer:: it4
      it4 = min(max(1,nint(t4 - 200.0)),ncirt)
      cirt_thresh = ulst(it4)
 end function CIRT_THRESHOLD
!----------------------------------------------------------------------
 subroutine READ_THRESHOLDS(data_dir)
   character(len=*), intent(in)::data_dir
   character(len=72):: header
   integer:: isolzen,isatzen

!----------------------------------------------
! key's fmft
!-----------------------------------------------
  ct_thresh = reshape((/0.65,  0.57, 0.54, 0.50, 0.54, 0.57, 0.65, 0.75,  1.00, &
                        1.50,3.06,5.77,9.41/),(/nfmft_key/))
  wt_thresh = reshape((/-0.8, -0.91,-1.01,-1.07, -1.1, -1.02, -0.95,  &
                        -0.85, -0.75, -0.6, -0.5, -0.3, -0.15/),(/nfmft_key/))
  zc =        reshape((/23.4,  23.5, 23.7, 23.9, 24.0,  24.1, 24.0,  &
                        23.7, 23.2, 20.5, 19.7, 19.0, 18.0/),(/nfmft_key/))
!----------------------------------------
! visible channel reflectance thresholds
!----------------------------------------
  vis_ref_thresh(0,0) = 1000.0    !ocean - night
  vis_ref_thresh(1,0) = 30.0      !ocean - day
  vis_ref_thresh(0,1) = 1000.0    !land - night
  vis_ref_thresh(1,1) = 44.0      !land - day
!----------------------------------------
! window channel brightness temperature thresholds
!----------------------------------------
  win_tmp_thresh(0,0) = 271.0    !ocean - night
  win_tmp_thresh(1,0) = 271.0    !ocean - day
  win_tmp_thresh(0,1) = 249.0    !land - night
  win_tmp_thresh(1,1) = 249.0    !land - day

!------------------------------------------------------
!  channel 3a - 1.6 micron reflectance thresholds
!------------------------------------------------------
!   nir_ref_ocean_c3a = 3.0 * (1.0+2.0*1.5) / (1.0+2.0*1.5*muo)

!ch3aon = 0
    nir_ref_thresh(0,0,0)  = 1000.0   !ocean-night
    nir_ref_thresh(1,0,0)  = 3.0      !ocean-day
    nir_ref_thresh(0,1,0)  = 1000.0   !land-night
    nir_ref_thresh(1,1,0)  = 100.0    !land-day
!ch3aon = 1
    nir_ref_thresh(0,0,1)  = 1000.0   !ocean-night
    nir_ref_thresh(1,0,1)  = 10.0      !ocean-day
    nir_ref_thresh(0,1,1)  = 1000.0   !land-night
    nir_ref_thresh(1,1,1)  = 100.0    !land-day
!ch3aon = 2
    nir_ref_thresh(0,0,2)  = 1000.0   !ocean-night
    nir_ref_thresh(1,0,2)  = 1000.0      !ocean-day
    nir_ref_thresh(0,1,2)  = 1000.0   !land-night
    nir_ref_thresh(1,1,2)  = 1000.0    !land-day

!--------------------------
! glint angle definition
!--------------------------
  glint_angle = 40.0          
  glint_angle_lo_sun = 70.0          

!-----------------------------
! maximum vis_ref_ratio for snow/ice
!-----------------------------
    vis_ref_ratio_snow_min = 0.80
    vis_ref_ratio_snow_max = 1.40

!------------------------------------------
! uniformity thresholds
!------------------------------------------
    win_tmp_uni_clr(0,0)  = 0.5   !ocean-night
    win_tmp_uni_clr(1,0)  = 0.5   !ocean-day
    win_tmp_uni_clr(0,1)  = 3.0   !land-night
    win_tmp_uni_clr(1,1)  = 3.0   !land-day

    vis_ref_uni_clr(0,0)  = 999.0 !ocean-night
    vis_ref_uni_clr(1,0)  =  5.0   !ocean-day
    vis_ref_uni_clr(0,1)  = 999.0 !land-night
    vis_ref_uni_clr(1,1)  = 50.0   !land-day

    nir_ref_uni_clr(0,0)  = 100.0 !ocean-night
    nir_ref_uni_clr(1,0)  = 100.0   !ocean-day
    nir_ref_uni_clr(0,1)  = 100.0 !land-night
    nir_ref_uni_clr(1,1)  = 100.0   !land-day
  
!--- cloudy class
    win_tmp_uni_cld(0,0)  = 50.0   !ocean-night
    win_tmp_uni_cld(1,0)  = 50.0   !ocean-day
    win_tmp_uni_cld(0,1)  = 50.0   !land-night
    win_tmp_uni_cld(1,1)  = 50.0   !land-day

    vis_ref_uni_cld(0,0)  = 999.0   !ocean-night
    vis_ref_uni_cld(1,0)  = 50.0   !ocean-day
    vis_ref_uni_cld(0,1)  = 999.0   !land-night
    vis_ref_uni_cld(1,1)  = 50.0   !land-day

!-- uniform cloud class
    win_tmp_uni_ucld(0,0)  = 10.0   !ocean-night
    win_tmp_uni_ucld(1,0)  = 10.0   !ocean-day
    win_tmp_uni_ucld(0,1)  = 10.0   !land-night
    win_tmp_uni_ucld(1,1)  = 10.0   !land-day

    vis_ref_uni_ucld(0,0)  = 999.0   !ocean-night
    vis_ref_uni_ucld(1,0)  = 10.0   !ocean-day
    vis_ref_uni_ucld(0,1)  = 999.0   !land-night
    vis_ref_uni_ucld(1,1)  = 10.0   !land-day

!--- clear restorals
    win_tmp_uni_res(0,0)  = 0.5   !ocean-night
    win_tmp_uni_res(1,0)  = 0.5   !ocean-day
    win_tmp_uni_res(0,1)  = 1.0   !land-night
    win_tmp_uni_res(1,1)  = 1.0   !land-day

!--------------------------------------------------
! definition of polar regions
!--------------------------------------------------
   polar_lat(0,0)  =  60.0  !ocean-night
   polar_lat(1,0)  =  60.0  !ocean-day
   polar_lat(0,1)  =  30.0  !land-night
   polar_lat(1,1)  =  30.0  !land-day

!----------------------------------------------------------------------
! thresholds that vary with window brightness temp FMFT & CIRT
!-----------------------------------------------------------------------
!fmft threshold look up tables (200-320k) for land and ocean
      fmflnd = reshape((/0.00,                                         &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.01, 0.03, 0.05, 0.08, 0.11, 0.14, 0.18, 0.23, 0.28, 0.34, &
          0.41, 0.48, 0.57, 0.66, 0.76, 0.87, 1.00, 1.13, 1.27, 1.42, &
          1.59, 1.76, 1.94, 2.14, 2.34, 2.55, 2.77, 3.00, 3.24, 3.48, &
          3.73, 3.99, 4.26, 4.52, 4.80, 5.08, 5.35, 5.64, 5.92, 6.20, &
          6.48, 6.76, 7.03, 7.30, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80, &
          7.80, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80, 7.80/),(/nfmft/))

      fmfocn= reshape((/0.00,                                         &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, &
          0.01, 0.03, 0.05, 0.07, 0.09, 0.11, 0.13, 0.14, 0.16, 0.17, &
          0.18, 0.19, 0.19, 0.20, 0.20, 0.20, 0.21, 0.21, 0.22, 0.23, &
          0.24, 0.25, 0.27, 0.30, 0.33, 0.36, 0.41, 0.45, 0.51, 0.58, &
          0.65, 0.73, 0.82, 0.92, 1.01, 1.15, 1.27, 1.40, 1.54, 1.69, &
          1.84, 1.99, 2.15, 2.31, 2.46, 2.62, 2.77, 2.93, 3.08, 3.24, &
          3.39, 3.55, 3.70, 3.86, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, &
          4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, &
          4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00, 4.00/),(/nfmft/))

!...cirt threshold look up table (200-320k)
      cirt = reshape ((/0.00,                                     &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,  &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,  &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,  &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,  &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,  &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,  &
          0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,  &
          0.00, 0.00, 0.00, 0.0009,0.0027,0.0044,0.0062,0.0080,0.0098,  &
          0.0115,0.0133,0.0151,0.0169,0.0186,0.0204,0.0222,0.0240,  &
          0.0257,0.0275,0.0293,0.0311,0.0328,0.033,0.033,0.033,0.033,  &
          0.033,0.033,0.033,0.033,0.033,0.033,0.033,0.033,0.033,0.033,  &
          0.033,0.033,0.033,0.033,0.033,0.033,0.033,0.033,0.033,0.033,  &
          0.033,0.033,0.033,0.033/),(/ncirt/))

!...ulst threshold look up table (200-320k)
        ulst =  reshape((/0.079,                                        &
          0.081,0.084,0.087,0.090,0.093,0.097,0.100,0.104,0.107,0.111,  &
          0.115,0.119,0.123,0.127,0.132,0.136,0.141,0.146,0.151,0.156,  &
          0.162,0.168,0.173,0.179,0.186,0.192,0.199,0.206,0.213,0.220,  &
          0.228,0.236,0.244,0.253,0.262,0.271,0.280,0.290,0.300,0.310,  &
          0.321,0.332,0.344,0.356,0.368,0.381,0.395,0.408,0.422,0.437,  &
          0.452,0.468,0.484,0.501,0.519,0.537,0.556,0.575,0.595,0.616,  &
          0.637,0.659,0.682,0.706,0.731,0.756,0.782,0.810,0.838,0.867,  &
          0.897,0.928,0.961,0.994,1.029,1.064,1.101,1.140,1.179,1.220,  &
          1.263,1.307,1.352,1.399,1.448,1.499,1.551,1.605,1.661,1.718,  &
          1.778,1.840,1.904,1.970,2.039,2.110,2.183,2.259,2.338,2.419,  &
          2.503,2.590,2.680,2.774,2.870,2.970,3.073,3.180,3.291,3.406,  &
          3.524,3.647,3.773,3.905,4.041,4.181,4.327,4.477,4.633,4.794/), &
          (/nulst/))



!--------------------------------------------------------------------------
! read in clear ocean reflectance data
!--------------------------------------------------------------------------
         open(unit=7,file=trim(data_dir)//"luts/clr_ocn/ch2_clr_ocn_ref_t1.dat",&
                status="old",action="read",position="rewind",form="formatted")
         open(unit=8,file=trim(data_dir)//"luts/clr_ocn/ch21_clr_ocn_ref_t1.dat",&
                status="old",action="read",position="rewind",form="formatted")
         open(unit=9,file=trim(data_dir)//"luts/clr_ocn/ch3a_clr_ocn_ref_t1.dat",&
                status="old",action="read",position="rewind",form="formatted")
         open(unit=10,file=trim(data_dir)//"luts/clr_ocn/ch3b_clr_ocn_ref.dat",&
                status="old",action="read",position="rewind",form="formatted")
         read(unit=7,fmt="(a)") header
         print *, header
         read(unit=8,fmt="(a)") header
         print *, header
         read(unit=9,fmt="(a)") header
         print *, header
         read(unit=10,fmt="(a)") header
         print *, header
         read(unit=7,fmt=*)
         read(unit=7,fmt=*) relaz_ocn_ref
         read(unit=8,fmt=*)
         read(unit=8,fmt=*) 
         read(unit=9,fmt=*)
         read(unit=9,fmt=*) 
         read(unit=10,fmt=*)
         read(unit=10,fmt=*)
         do isolzen = 1,nsolzen_ocn_ref
           read(unit=7,fmt=*) solzen_ocn_ref(isolzen)
           read(unit=8,fmt=*) 
           read(unit=9,fmt=*) 
           read(unit=10,fmt=*) 
           do isatzen = 1, nsatzen_ocn_ref
            read(unit=7,fmt=*) satzen_ocn_ref(isatzen), &
                      vis2_ocn_ref(isolzen,isatzen,:)
            read(unit=8,fmt=*) satzen_ocn_ref(isatzen), &
                      vis21_ocn_ref(isolzen,isatzen,:)
            read(unit=9,fmt=*) satzen_ocn_ref(isatzen), &
                      nir1_ocn_ref(isolzen,isatzen,:)
            read(unit=10,fmt=*) satzen_ocn_ref(isatzen), &
                      nir2_ocn_ref(isolzen,isatzen,:)
           enddo
         enddo
         close(unit=7)
         close(unit=8)
         close(unit=9)
         close(unit=10)
         del_solzen_ocn_ref = solzen_ocn_ref(2) - solzen_ocn_ref(1)
         del_satzen_ocn_ref = satzen_ocn_ref(2) - satzen_ocn_ref(1)
         del_relaz_ocn_ref = relaz_ocn_ref(2) - relaz_ocn_ref(1)
!-----------------------------------------------------------------------
! set fmft thresholds using tables
!-----------------------------------------------------------------------

 zen_lut = reshape((/7.5, 22.5, 37.5, 52.5/),(/nzen_lut/))
!zen_lut = reshape((/0.0,15.0,30.0,45.0,60.0/),(/nzen_lut/))

t4_lut = reshape((/190.0, 200.0, 210.0, 220.0, 230.0, 240.0, 250.0,&
                   260.0, 270.0, 280.0, 290.0, 300.0, 310.0, 320.0/),(/nt4_lut/))

fmft_hi_land_lut = reshape((/&
  0.595,     0.725,     0.685,   -99.000,  &
  0.595,     0.725,     0.685,   -99.000,  &
  0.595,     0.725,     0.685,   -99.000,  &
  0.595,     0.725,     0.685,   -99.000,  &
  0.775,     0.745,     0.775,     0.715,  &
  0.745,     0.745,     0.755,     0.795,  &
  0.855,     0.815,     0.765,     0.775,  &
  0.875,     0.875,     0.905,     0.885,  &
  1.185,     1.155,     1.275,     1.135,  &
  2.835,     3.255,     3.295,     3.175,  &
  5.475,     5.395,     5.245,     5.325,  &
  6.195,     6.035,     6.055,     6.105,  &
  6.185,     6.225,     6.835,     5.925,  &
  6.185,     6.225,     6.835,     5.925/),  &
    (/nzen_lut,nt4_lut/))

fmft_lo_land_lut = reshape((/&
 -0.465,    -1.485,    -1.025,   -99.000,  &
 -0.465,    -1.485,    -1.025,   -99.000,  &
 -0.465,    -1.485,    -1.025,   -99.000,  &
 -0.465,    -1.485,    -1.025,   -99.000,  &
 -1.205,    -1.615,    -1.145,    -1.345,  &
 -1.305,    -1.385,    -1.265,    -1.275,  &
 -1.375,    -1.455,    -1.385,    -1.485,  &
 -1.525,    -1.455,    -1.665,    -1.525,  &
 -1.215,    -1.255,    -1.205,    -1.295,  &
 -1.245,    -1.095,    -1.035,    -0.975,  &
 -0.445,    -0.395,    -0.315,    -0.465,  &
 -0.445,    -0.395,    -0.315,    -0.465,  &
 -0.445,    -0.395,    -0.315,    -0.465,  &
 -0.445,    -0.395,    -0.315,    -0.465/),  &
    (/nzen_lut,nt4_lut/))

fmft_hi_ocean_lut = reshape((/&
  0.665,     0.685,     0.655,     0.645,  &
  0.665,     0.685,     0.655,     0.645,  &
  0.665,     0.685,     0.655,     0.645,  &
  0.665,     0.685,     0.655,     0.645,  &
  0.665,     0.685,     0.655,     0.645,  &
  0.695,     0.705,     0.695,     0.675,  &
  0.775,     0.775,     0.765,     0.765,  &
  0.905,     0.925,     0.935,     0.965,  &
  1.165,     1.165,     1.245,     1.375,  &
  1.985,     2.215,     2.475,     3.195,  &
  4.085,     4.135,     4.195,     4.545,  &
  4.245,     4.115,     4.315,     5.095,  &
  4.245,     4.115,     4.315,     5.095,  &
  4.245,     4.115,     4.315,     5.095/),  &
    (/nzen_lut,nt4_lut/))


fmft_lo_ocean_lut = reshape((/&
 -0.205,    -0.225,    -0.335,    -0.415,  &
 -0.205,    -0.225,    -0.335,    -0.415,  &
 -0.205,    -0.225,    -0.335,    -0.415,  &
 -0.205,    -0.225,    -0.335,    -0.415,  &
 -0.205,    -0.225,    -0.335,    -0.415,  &
 -0.205,    -0.435,    -0.455,    -0.385,  &
 -0.325,    -0.285,    -0.225,    -0.585,  &
  0.025,    -0.035,    -0.085,    -0.135,  &
 -0.115,    -0.085,    -0.025,    -0.065,  &
  0.035,    -0.055,     0.045,     0.095,  &
  0.335,     0.385,     0.515,     0.575,  &
  0.565,     0.735,     0.825,     0.925,  &
  0.565,     0.735,     0.825,     0.925,  &
  0.565,     0.735,     0.825,     0.925/),  &
    (/nzen_lut,nt4_lut/))


tmft_hi_land_lut = reshape((/&
  2.635,     2.505,     3.395,     3.500,  &
  2.635,     2.505,     3.395,     3.500,  &
  2.635,     2.505,     3.395,     3.500,  &
  2.635,     2.505,     3.395,     3.500,  &
  2.615,     2.655,     2.685,     2.505,  &
  1.865,     1.835,     1.845,     1.915,  &
  1.815,     1.785,     1.815,     1.795,  &
  1.885,     1.885,     1.875,     1.875,  &
  2.135,     2.115,     2.095,     2.105,  &
  6.825,     7.445,     8.305,     7.125,  &
 19.055,    18.485,    17.795,    17.025,  &
 20.625,    19.775,    19.355,    19.895,  &
 18.115,    15.935,    20.395,    16.025,  &
 18.115,    15.935,    20.395,    16.025/),  &
    (/nzen_lut,nt4_lut/))

tmft_lo_land_lut = reshape((/&
  0.145,    -0.165,    -0.075,    -0.075,  &
  0.145,    -0.165,    -0.075,    -0.075,  &
  0.145,    -0.165,    -0.075,    -0.075,  &
  0.145,    -0.165,    -0.075,    -0.075,  &
 -0.805,    -0.975,    -0.795,    -1.045,  &
 -1.195,    -1.065,    -1.125,    -1.175,  &
 -1.225,    -1.285,    -1.285,    -1.285,  &
 -2.425,    -1.325,    -2.105,    -1.975,  &
 -1.685,    -1.595,    -1.535,    -2.045,  &
 -4.205,    -4.145,    -3.645,    -3.585,  &
 -2.425,    -1.715,    -2.275,    -2.015,  &
  0.585,    -0.585,     0.825,     0.345,  &
  0.655,     1.905,     0.475,     1.385,  &
  0.655,     1.905,     0.475,     1.385/),  &
    (/nzen_lut,nt4_lut/))

tmft_hi_ocean_lut = reshape((/&
  1.285,     1.685,     1.735,     2.115,  &
  1.285,     1.685,     1.735,     2.115,  &
  1.285,     1.685,     1.735,     2.115,  &
  1.285,     1.685,     1.735,     2.115,  &
  1.285,     1.685,     1.735,     2.115,  &
  1.005,     0.965,     1.055,     1.195,  &
  0.815,     0.815,     0.805,     0.825,  &
  0.955,     0.955,     0.965,     0.975,  &
  1.415,     1.415,     1.535,     1.705,  &
  3.815,     4.365,     5.015,     7.195,  &
 10.905,    11.445,    11.855,    13.505,  &
 10.225,     9.845,     9.985,    11.645,  &
 10.225,     9.845,     9.985,    11.645,  &
 10.225,     9.845,     9.985,    11.645/),  &
    (/nzen_lut,nt4_lut/))

tmft_lo_ocean_lut = reshape((/&
  0.245,     0.335,     0.255,     0.285,  &
  0.245,     0.335,     0.255,     0.285,  &
  0.245,     0.335,     0.255,     0.285,  &
  0.245,     0.335,     0.255,     0.285,  &
  0.245,     0.335,     0.255,     0.285,  &
  0.095,     0.115,     0.135,     0.065,  &
 -0.335,    -0.115,    -0.255,    -0.275,  &
 -0.145,    -0.235,    -0.085,    -0.215,  &
 -0.125,    -0.105,     0.035,    -0.075,  &
 -0.035,    -0.375,    -0.355,    -0.325,  &
  0.715,     0.655,     0.835,     0.775,  &
  0.715,     0.655,     0.835,     0.775,  &
  0.715,     0.655,     0.835,     0.775,  &
  0.715,     0.655,     0.835,     0.775/),  &
    (/nzen_lut,nt4_lut/))

!--------------------------------------------------------------------------
! set ch1 thresholds for umd types
!-----------------------------------------------------------------------

scat_angle_umd = reshape((/82.5,97.5,112.5,127.5,142.5,157.5,172.5/),(/nscat_umd/))

std_factor = reshape((/3.0,3.0,3.0,4.0,4.0,3.0,3.0,3.0,3.0,3.0, &
                       3.0,3.0,3.0,3.0,3.0,3.0,3.0,3.0,3.0,3.0/),(/ntype_umd/))

   ch1_mean_umd = reshape((/&
 44.0,    44.0, 44.000,44.0000, 44.000, 44.000, 44.000,  &          !0
 11.6575, 6.43535, 5.18526, 5.53282,7.50403,7.05260, 9.45309, &     !1
 44.0,     8.24039, 6.41333, 5.46957, 7.12071, 7.06713, 8.98686, &  !2
 12.3086,  6.49614, 5.12911, 5.70573, 7.25429, 6.65823, 44.00, &    !3
 13.7671 , 8.97270, 6.54116, 6.90868, 9.40795, 12.0147, 7.1943, &   !4
 14.2167,  7.10081, 6.12671, 6.05182, 7.11398, 6.81985, 7.6329, &   !5
 14.2167,  7.10081, 6.12671, 6.05182, 7.11398, 6.81985, 7.6329, &   !6
 11.6913,  7.88678, 7.45223, 7.18168,9.00203,10.6333, 11.205, &     !7
 10.7681,  9.74169, 8.76297, 9.24842,11.6366, 15.3418, 15.091, &    !8
 13.2548, 16.9562, 17.8674, 17.4102,21.2538, 25.7314, 26.080, &     !9
 10.6815, 15.0607, 13.7965, 13.3560,15.5227, 19.7536, 22.817,  &    !10
 44.000, 44.0000, 44.0000, 44.0000,44.0000, 44.0000, 44.000,  &     !11
 64.000, 64.0000, 64.0000, 64.0000,64.0000, 64.0000, 64.000,  &     !12
 44.000, 44.0000, 44.0000, 44.0000,44.0000, 44.0000, 44.000,  &     !13
 44.0, 44.0, 44.0, 44.0, 44.0, 44.00, 44.000/), &     !14
     (/ntype_umd,nscat_umd/),order=(/2,1/))

ch1_mean_umd = 44.0
ch1_mean_umd(12,:) = 100.0
ch1_mean_umd(14,:) = 100.0


   ch1_std_umd = reshape((/&
 0.000  , 0.00   , 0.0000, 0.0000, 0.0000, 0.000 , 0.0,&        !0
 2.21137, 1.59893, 1.2268, 1.2381, 1.6888, 1.4415, 3.09615,&  !1
 0.00000, 3.93557, 3.40052, 2.2041, 2.7311, 2.5486, 2.8977,&  !2
 1.55743, 1.53872, 0.9920, 0.8824, 1.7397, 1.0062, 0.0000,&   !3
 4.07884, 4.19125, 3.0735, 1.7562, 4.4647, 4.2762, 1.52437,&   !4
 2.91151, 2.90520, 2.5425, 2.1977, 2.7840, 2.1876, 2.41444,&   !5
 2.91151, 2.90520, 2.5425, 2.1977, 2.7840, 2.1876, 2.41444,&   !5
 3.72378, 3.79165, 3.8215, 3.8424, 3.6245, 3.8713, 5.96642,&    !7
 3.99424, 5.80722, 4.3881, 5.2897, 5.9694, 5.8615, 3.81213,&   !8
 4.63007, 6.33436, 6.6392, 6.2348, 7.7386, 5.1827, 4.73086,&    !9
 3.44592, 5.34324, 5.6573, 5.0163, 6.6280, 6.1605, 5.57383,&    !10
 0.000, 0.0, 0.0, 0.0, 0.0, 0.0, 0.000, &      !11
 0.000, 0.0, 0.0, 0.0, 0.0, 0.0, 0.000, &      !12
 0.000, 0.0, 0.0, 0.0, 0.0, 0.0, 0.000, &      !13
 0.000, 0.0, 0.0, 0.0, 0.0, 0.0, 0.000/),&      !14
     (/ntype_umd,nscat_umd/),order=(/2,1/))

ch1_std_umd = 0.0

   ch21_mean_umd = reshape((/&
 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, &     !0
 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, &     !1
 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, &     !2
 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, &     !3
 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, &     !4
 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, &     !5
 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, &     !6
 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, 1.3, &     !7
 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, &     !8
 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, &     !9
 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, &     !10
 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, &     !11
 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, &     !12
 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, &     !13
 1.1, 1.1, 1.1, 1.1, 1.1, 1.1, 1.1/), &   !14
     (/ntype_umd,nscat_umd/),order=(/2,1/))
 ch21_mean_umd = 1.1

  end subroutine READ_THRESHOLDS

end module CLOUD_MASK_THRESHOLDS
