! $Id: cloud_mask.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!---------------------------------------------------------------------
! This module performs a cloud mask on 2 by 2 pixel arrays
! the resulting cloud mask codes are
!
! cloud mask codes
! 0 - clear
! 1 - mixed clear
! 2 - mixed cloudy
! 3 - cloud
!
! it applies each test to each pixel and each test has its
! own byte which is also an output of this routine
!
! cloud bytes
! bit  1 (k= 8) - TMFT - three minus four test (4 - 11 um) (not done)
! bit  2 (k= 7) - FMFT - four minus five test ( 11 - 12 um)
! bit  3 (k= 6) - CIRT - cirrus test (4 - 11 um)
! bit  4 (k= 5) - ULST  - uniform low stratus test (4 - 12 um)
! bit  5 (k= 4) - TGCT - thermal gross contrast test
! bit  6 (k= 3) - C3AT - nir reflectance test
! bit  7 (k= 2) - RRCT - visible reflectance ratio test
! bit  8 (k= 1) - RGCT - visible reflectance gross contrast test
! bit  9 (k=16) - DYNAMIC thermal restoral test
! bit 10 (k=15) - DYNAMIC thermal gross contrast test
! bit 11 (k=14) - 
! bit 12 (k=13) - 
! bit 13 (k=12) - 
! bit 14 (k=11) - 
! bit 15 (k=10) - 
! bit 16 (k= 9) - CSST - climatology of SST test 
! bit 17 (k=24) - fire(1) / nofire(0)
! bit 18 (k=23) - snow(1) / nosnow(0)
! bit 19 (k=22) - dust(1) / nodust(0)
! bit 20 (k=21) - glint(1) / noglint(0)  ! this is for the glint test
! bit 21 (k=20) - TUT_cloud - temperature uniformity test for cloud
! bit 22 (k=19) - RUT_cloud - reflectance uniformity test for cloud
! bit 23 (k=18) - TUT_clear - temperature uniformity test for clear
! bit 24 (k=17) - RUT_clear - reflectance uniformity test for clear
!
!
! data bits these are inputs)
! bit 1 (k=8) ch3a on (1) / ch3a off (0)
! bit 2 (k=7) snow (1) / nosnow (0)
! bit 3 (k=6) desert (1) / nodesert (0)
! bit 4 (k=5) glint (1) / noglint (0)
! bit 5 (k=4) coast (1) / nocoast (0)
! bit 6 (k=3) land (1) / sea (0)
! bit 7 (k=2) in term (1) / out of terminator (0)
! bit 8 (k=1) day (1) / night (0) 
!
!
! mod
!  Nov-02 - note using t3b-t4 for tmft and ulst, turned of cirt
!----------------------------------------------------------------------
module CLOUD_MASK
 use CONSTANTS
 use CLOUD_MASK_THRESHOLDS
 use AVHRR_ROUTINES
 implicit none
 private
 public:: CLOUD_MASK_22,DYN_WIN1_CLOUD_MASK,MAX_SFC_TEMP

 contains
!------------------------------------------------------------------------
!
!------------------------------------------------------------------------
 subroutine MAX_SFC_TEMP(max_sst,std_sst,max_lst,std_lst)
   real(kind=real4), intent(inout):: max_sst,std_sst,max_lst,std_lst

   integer(kind=int2), dimension(size(cld_mask,1),size(cld_mask,2))::  &
                            clear_land_mask,clear_ocean_mask
   integer:: skip_mask,nclear_land,nclear_ocean,min_count
   real(kind=real4):: min_land_temp,min_ocean_temp,max_lst_tmp, &
                      std_lst_tmp, max_sst_tmp, &
                      std_sst_tmp,min_ocean_std,min_land_std, &
                      mean_clear_win1_tmp_land, mean_sst_tmp,&
                      mean2_clear_win1_tmp_land, mean2_sst_tmp

   min_land_temp = 244.0
   min_land_std = 5.0
   min_ocean_temp = 270.0
   min_ocean_std = 1.0
   skip_mask = 0
   min_count = 5
   
    
!--- maximum clear temp over land
  clear_land_mask = (1-skip_mask)*(land_mask)*(max(0,1-cld_mask))
  nclear_land = sum(clear_land_mask)
  max_lst_tmp = max(min_land_temp,maxval(clear_land_mask*sst_mask))

  if (nclear_land < min_count) then
      max_lst_tmp = min_land_temp
      std_lst_tmp = min_land_std
  else
    mean_clear_win1_tmp_land = sum(clear_land_mask*sst_mask)/nclear_land
    mean2_clear_win1_tmp_land = sum(clear_land_mask*sst_mask*sst_mask)/nclear_land
    std_lst_tmp = sqrt(max(0.2,(mean2_clear_win1_tmp_land -  &
                                   mean_clear_win1_tmp_land**2)))
  endif


!------ maximum clear temp over ocean
  clear_ocean_mask = max(0,(1-skip_mask)*(1-land_mask))*(max(0,1-cld_mask))
  max_sst_tmp = max(270.0,maxval(clear_ocean_mask*sst_mask))
  nclear_ocean = sum(clear_ocean_mask)
  if (nclear_ocean < min_count) then
   max_sst_tmp = min_ocean_temp
   std_sst_tmp = min_ocean_std
  else
   mean_sst_tmp = sum(clear_ocean_mask*sst_mask)/nclear_ocean
   mean2_sst_tmp =sum(clear_ocean_mask*sst_mask*sst_mask)/nclear_ocean
   std_sst_tmp = sqrt(max(0.0,(mean2_sst_tmp -  &
                                   mean_sst_tmp**2)))
  endif


!--- determine running mean
    if (max_sst < 0.0) then
        max_sst = max(min_ocean_temp,max_sst_tmp)
        std_sst = max(min_ocean_std,std_sst_tmp)
    else
       if (nclear_ocean > min_count) then
         max_sst = 0.90*max_sst + 0.10*max_sst_tmp
         std_sst = 0.90*std_sst + 0.10*std_sst_tmp
       else
         max_sst = 0.98*max_sst + 0.02*min_ocean_temp
         std_sst = 0.98*std_sst + 0.02*min_ocean_std
       endif
    endif

    if (max_lst < 0.0) then
        max_lst = max(min_land_temp,max_lst_tmp)
        std_lst = max(5.0,std_lst_tmp)
    else
       if (nclear_land > min_count) then
           max_lst = 0.90*max_lst + 0.10*max_lst_tmp
           std_lst = 0.90*std_lst + 0.10*std_lst_tmp
       else
           max_lst = 0.98*max_lst + 0.02*min_land_temp
           std_lst = 0.98*std_lst + 0.02*min_land_std
       endif
    endif


!   write(unit=77,fmt="(i5,10f10.3)") nclear_land,max_lst_tmp, max_lst
!   write(unit=77,fmt="(i5,10f10.3)") nclear_ocean,max_sst_tmp, max_sst

 end subroutine MAX_SFC_TEMP


!------------------------------------------------------------------------
! This routine is to be called after the cloud mask.
! It's purpose to use the results from the cloud mask to redefine
! land and water thresholds on the bt4 and bt45 to detect
! cloud that might have been missed (especially in glint regions)
! 
! this code stores running means of values computed from each pair
! of scanlines
!
! this is a work in progress
!------------------------------------------------------------------------
 subroutine DYN_WIN1_CLOUD_MASK(lat_nadir,&
                                max_sst,max_lst, &
                                std_sst,std_lst)
    real(kind=real4), intent(inout):: max_sst,max_lst,std_sst,std_lst
    real(kind=real4), intent(in):: lat_nadir

    real(kind=real4):: lst_thresh, &
                       sst_thresh, &
                       lst_thresh_mtn,lst_thresh_des,&
                       sst_res_thresh,lst_res_thresh,&
                       delta_land,max_delta_land,min_delta_land, &
                       min_land_temp_apply,min_ocean_temp_apply, &
                       max_lat_apply,delta_ocean,min_delta_ocean,max_delta_ocean
    integer:: i, j, k,mask_weight

   min_land_temp_apply = 277.0    !(these must be greater than the values
   min_ocean_temp_apply = 273.0   !( for min_land_temp and min_ocean_temp)

!--------------- unpack bits into bytes
  do i = 1,8
     mask(i+8,:,:) = modulo(ishft(mask_byte_2,i-8),2_int1)
     mask(i+16,:,:) = modulo(ishft(mask_byte_3,i-8),2_int1)
  enddo

!-----------------------------------------------------------------
! determine new cloud code, note skip this if this pixel
! has been classified as snow,fire or dust
!-----------------------------------------------------------------

!--- ocean
    min_delta_ocean = 3.0
    max_delta_ocean = 10.0
    delta_ocean = max(min_delta_ocean,min(max_delta_ocean,6.0*std_sst))
!   sst_thresh = min(289.0,max(270.0,max_sst - delta_ocean)) 
    sst_thresh = min(289.0,max(270.0,max_sst - delta_ocean))

!--- land
    min_delta_land = 10.0
    max_delta_land = 30.0
    delta_land = max(min_delta_land,min(max_delta_land,6.0*std_lst))
    lst_thresh = min(280.0,max(244.0,max_lst - delta_land))

!-- desert or mtn
    lst_thresh_mtn = lst_thresh - 10.0
    lst_thresh_des = lst_thresh - 5.0

    sst_res_thresh = max_sst - 0.5*std_sst
    lst_res_thresh = max_lst - 0.5*std_lst

!   write(unit=77,fmt="(i5,10f10.3)") nclear_land,max_lst,std_lst,delta_land,lst_thresh
    write(unit=77,fmt="(10f10.3)") max_sst,std_sst,delta_ocean,sst_thresh

    do i = 1, size(cld_mask,1)

       do j = 1, size(cld_mask,2)
 

!----- apply dynamic cloud mask to land
         if ((land_mask(i,j)  == 1) .and. (max_lst > min_land_temp_apply)) then

          if (zsfc(i,j) > 1000.0) then
            if (sst_mask(i,j) < lst_thresh_mtn) then
             mask(15,i,j) = 1
            endif
          else
             if (desert(i,j) == 1) then
              if (sst_mask(i,j) < lst_thresh_des) then
               mask(15,i,j) = 1
              endif
             else
              if (sst_mask(i,j) < lst_thresh) then
               mask(15,i,j) = 1
              endif
             endif
          endif

!-------- clear restoral for land
          if (sst_mask(i,j) > lst_res_thresh) then
            if (mask(18,i,j) == 0) then
               mask(16,i,j) = 1
            endif
          endif

!------ apply dynamic thermal test to ocean (ice-free)
         elseif ((land_mask(i,j) == 0) .and. (max_sst > min_ocean_temp_apply)) then  !ocean

          if (mask(23,i,j) == 0) then
!           if ((sst_mask(i,j) < sst_thresh) .and. (mask(18,i,j) == 1)) then
            if (sst_mask(i,j) < sst_thresh) then
             mask(15,i,j) = 1
            endif

!-------- clear restoral for ocean
            if (sst_mask(i,j) > sst_res_thresh) then
             if (mask(18,i,j) == 0) then
                mask(16,i,j) = 1
             endif
            endif

           endif

          endif
          mask_weight = 1

     
!--- apply threshold, set appropriate mask bit  and recompute mask
          if (mask(15,i,j)*mask_weight > 0) then
             if (sum(mask(19:20,i,j)) == 0) then
               cld_mask(i,j) = 3
             else
               cld_mask(i,j) = 2
             endif
          endif

!------- apply restoral
           if ((cld_mask(i,j) == 1) .and. (mask(16,i,j) == 1)) then
              cld_mask(i,j) = 0
           endif


!---------------------------------------
! repack mask_byte_2
!----------------------------------------
         mask_byte_2(i,j) = 0
         do k = 1, 8
           mask_byte_2(i,j) = mask_byte_2(i,j) + 2**(8-k) * mask(k+8,i,j)
         enddo

      enddo
    enddo

 end subroutine DYN_WIN1_CLOUD_MASK

!------------------------------------------------------------------------
 subroutine CLOUD_MASK_22(max_lst,max_sst)

 real(kind=real4), intent(in):: max_sst,max_lst
 integer:: i,j,k,ii,n_cld_test,nx,ny,iday,iland,iscat_umd
 
 integer(kind=int1), dimension(24,size(alb1,1),size(alb1,2)):: mask_weight
 real(kind=real4):: bt4_uni,alb1_uni,bt3b_uni,alb2_uni,alb3a_uni,alb3b_uni, &
                    min_uni,zsfc_uni
                    
 real(kind=real4):: fmft_thresh_hi,fmft_thresh_lo,tmft_thresh_hi, tmft_thresh_lo
 real(kind=real4):: rgct_thresh,bt3b4_ulst,cirt_min, &
                   mean_alb1,mean_bt4,mean_alb3b,mean_zsfc,uni_factor, &
                   cirt_ratio,cirt_thresh, &
               glintzen22,solzen22,satzen22,relaz22,lat22,glint_angle_thresh, &
               xsol,xzen,xaz,xt4,vis21_ocn_ref_thresh,vis2_ocn_ref_thresh, &
               nir1_ocn_ref_thresh,nir2_ocn_ref_thresh,scat_angle, &
               rrct_thresh_min,rrct_thresh_max,rgct_thresh_land,rrct_thresh_land
 logical:: day22,term22,coast22,desert22,desert3b22,snow_input22,land22,polar22,mtn22, &
           shadow,snow,fire,dust,snow_potential_therm,snow_potential_ref, &
           ocean22

 integer:: ch3a_on22,isol,izen,iaz,icomp_vis_thresh,icold,it4


!------- variables for jeff key's tests
!integer:: ikey
!real(kind=real4):: bt45_adj,zc_adj

 nx = size(alb1,1)
 ny = size(alb1,2)

 cld_mask = 0
 mask_weight = 1
 mask = 0
 data_mask = 0
 n_cld_test = 9            !number of cloud tests on mask array
 data_bit = 0
 mask_byte_1 = 0
 mask_byte_2 = 0
 mask_byte_3 = 0

 min_uni = 0.01

!-----------------------------------------------------------------
! check to see if there is a channel switch in this scanline pair
!-----------------------------------------------------------------
  if ((ch3a_on(1) /= ch3a_on(2)) .or. (maxval(ch3a_on) > 1)) then
     cld_mask = 1
     return
  endif

!----- determine if ch3a is on or off
   if ((ch3a_on(1) == 1) .and. (ch3a_on(2) == 1)) then
           ch3a_on22 = 1
   elseif ((ch3a_on(1) == 0) .and. (ch3a_on(2) == 0)) then
          ch3a_on22 = 0
   else
          ch3a_on22 = 2
   endif   

!-------------------------------------------------------
! loop over all 2x2 pixel arrays in scanlines
!-------------------------------------------------------
 ii = 1
 ii_loop: do 

  if (ii == nx) then
      ii = ii - 1
  endif

  if (ii > nx) then
      exit
  endif

!--- compute characteristics of this pixel array
   glintzen22 = 0.25*(sum(glintzen(ii:ii+1,:)))
   solzen22 = 0.25*(sum(solzen(ii:ii+1,:)))
   satzen22 = 0.25*(sum(satzen(ii:ii+1,:)))
   relaz22 = 0.25*(sum(relaz(ii:ii+1,:)))
   lat22 = 0.25*(sum(lat(ii:ii+1,:)))

   day22 = .false.
   if (solzen22 < 85.0) then
       day22 = .true.
   endif                         

   if (solzen22 < 60.0) then
        glint_angle_thresh = glint_angle
   else
        glint_angle_thresh = glint_angle_lo_sun
   endif
   glint_angle_thresh = min(70.0,max(40.0,solzen22))

   term22 = .false.
   if ((day22 .eqv. .false.) .and. &
       (0.25*sum(chan_counts(1,ii:ii+1,:)) > ch1_dark_count+1)) then
            term22 = .true.
   endif 

    if ( sum(land_mask(ii:ii+1,:)) == 0) then
             land22 = .false.
             coast22 = .false.
             ocean22 = .true.
   elseif ( sum(land_mask(ii:ii+1,:)) == 4) then
             land22 = .true.
             coast22 = .false.
             ocean22 = .false.
   else
             land22 = .true.
             coast22 = .true.
             ocean22 = .false.
   endif       

   desert22 = .false.
   if (maxval(desert(ii:ii+1,:)) == 1) then
    desert22 = .true.
   endif

   desert3b22 = .false.
   if (maxval(desert3b(ii:ii+1,:)) == 1) then
    desert3b22 = .true.
   endif
 
   snow_input22 = .false.
   if (maxval(snow_input(ii:ii+1,:)) == 1) then
    snow_input22 = .true.
   endif    
 
   mtn22 = .false.
   if (sum(zsfc(ii:ii+1,:))/4.0 > 2000.0) then
     mtn22 = .true.
   endif
 

!---- set indices for thresholds
 iday = 0
 iland = 0
 if (day22 .eqv. .true.) then
    iday = 1
 endif
 if (land22 .eqv. .true.) then
    iland = 1
 endif

  polar22 = .false.
  if (abs(lat22) > polar_lat(iday,iland)) then
     polar22 = .true.
  endif

!------------------------------------------------------------
! compute ocean thresholds (for every fourth pixel over water)
!------------------------------------------------------------
  icomp_vis_thresh = 0
  if ( ((ii == 1) .or. (modulo(ii-1,4) == 0)).and.(land22 .eqv. .false.))  then
    icomp_vis_thresh = 1
  endif
  if (coast22 .eqv. .true.) then
    icomp_vis_thresh = 1
  endif
  if (icomp_vis_thresh == 1) then
   isol = max(1,min(nsolzen_ocn_ref-1,int(solzen22 / del_solzen_ocn_ref) + 1))
   izen = max(1,min(nsatzen_ocn_ref-1,int(satzen22 / del_satzen_ocn_ref) + 1))
   iaz =  max(1,min(nrelaz_ocn_ref-1,int(relaz22 / del_relaz_ocn_ref) + 1))
   xsol = (solzen22-solzen_ocn_ref(isol))/del_solzen_ocn_ref
   xzen = (satzen22-satzen_ocn_ref(izen))/del_satzen_ocn_ref
   xaz = (relaz22-relaz_ocn_ref(iaz))/del_relaz_ocn_ref

   vis2_ocn_ref_thresh =  &
             (1.0-xsol)*(1.0-xzen)*(1.0-xaz)*vis2_ocn_ref(isol,izen,iaz) + &
             (1.0-xsol)*(1.0-xzen)*(xaz)*vis2_ocn_ref(isol,izen,iaz+1) + &
             (1.0-xsol)*(xzen)*(1.0-xaz)*vis2_ocn_ref(isol,izen+1,iaz) + &
             (1.0-xsol)*(xzen)*(xaz)*vis2_ocn_ref(isol,izen+1,iaz+1) + &
             (xsol)*(1.0-xzen)*(1.0-xaz)*vis2_ocn_ref(isol+1,izen,iaz) + &
             (xsol)*(1.0-xzen)*(xaz)*vis2_ocn_ref(isol+1,izen,iaz+1) + &
             (xsol)*(xzen)*(1.0-xaz)*vis2_ocn_ref(isol+1,izen+1,iaz) + &
             (xsol)*(xzen)*(xaz)*vis2_ocn_ref(isol+1,izen+1,iaz+1) 
 
   vis21_ocn_ref_thresh =  &
             (1.0-xsol)*(1.0-xzen)*(1.0-xaz)*vis21_ocn_ref(isol,izen,iaz) + &
             (1.0-xsol)*(1.0-xzen)*(xaz)*vis21_ocn_ref(isol,izen,iaz+1) + &
             (1.0-xsol)*(xzen)*(1.0-xaz)*vis21_ocn_ref(isol,izen+1,iaz) + &
             (1.0-xsol)*(xzen)*(xaz)*vis21_ocn_ref(isol,izen+1,iaz+1) + &
             (xsol)*(1.0-xzen)*(1.0-xaz)*vis21_ocn_ref(isol+1,izen,iaz) + &
             (xsol)*(1.0-xzen)*(xaz)*vis21_ocn_ref(isol+1,izen,iaz+1) + &
             (xsol)*(xzen)*(1.0-xaz)*vis21_ocn_ref(isol+1,izen+1,iaz) + &
             (xsol)*(xzen)*(xaz)*vis21_ocn_ref(isol+1,izen+1,iaz+1) 

   nir1_ocn_ref_thresh =  &
             (1.0-xsol)*(1.0-xzen)*(1.0-xaz)*nir1_ocn_ref(isol,izen,iaz) + &
             (1.0-xsol)*(1.0-xzen)*(xaz)*nir1_ocn_ref(isol,izen,iaz+1) + &
             (1.0-xsol)*(xzen)*(1.0-xaz)*nir1_ocn_ref(isol,izen+1,iaz) + &
             (1.0-xsol)*(xzen)*(xaz)*nir1_ocn_ref(isol,izen+1,iaz+1) + &
             (xsol)*(1.0-xzen)*(1.0-xaz)*nir1_ocn_ref(isol+1,izen,iaz) + &
             (xsol)*(1.0-xzen)*(xaz)*nir1_ocn_ref(isol+1,izen,iaz+1) + &
             (xsol)*(xzen)*(1.0-xaz)*nir1_ocn_ref(isol+1,izen+1,iaz) + &
             (xsol)*(xzen)*(xaz)*nir1_ocn_ref(isol+1,izen+1,iaz+1) 

   nir2_ocn_ref_thresh =  &
             (1.0-xsol)*(1.0-xzen)*(1.0-xaz)*nir2_ocn_ref(isol,izen,iaz) + &
             (1.0-xsol)*(1.0-xzen)*(xaz)*nir2_ocn_ref(isol,izen,iaz+1) + &
             (1.0-xsol)*(xzen)*(1.0-xaz)*nir2_ocn_ref(isol,izen+1,iaz) + &
             (1.0-xsol)*(xzen)*(xaz)*nir2_ocn_ref(isol,izen+1,iaz+1) + &
             (xsol)*(1.0-xzen)*(1.0-xaz)*nir2_ocn_ref(isol+1,izen,iaz) + &
             (xsol)*(1.0-xzen)*(xaz)*nir2_ocn_ref(isol+1,izen,iaz+1) + &
             (xsol)*(xzen)*(1.0-xaz)*nir2_ocn_ref(isol+1,izen+1,iaz) + &
             (xsol)*(xzen)*(xaz)*nir2_ocn_ref(isol+1,izen+1,iaz+1) 

!----- 
!  vis2_ocn_ref_thresh = min(30.0,vis2_ocn_ref_thresh)
!  vis2_ocn_ref_thresh = vis2_ocn_ref_thresh * 1.10
   vis21_ocn_ref_thresh = vis21_ocn_ref_thresh * 1.10
   nir1_ocn_ref_thresh = nir1_ocn_ref_thresh * 1.10
   nir2_ocn_ref_thresh = nir2_ocn_ref_thresh * 1.10

   endif 

!------------------------------------------------------
! compute uniformity values for this pixel array
!------------------------------------------------------

 bt4_uni = max(min_uni,maxval(bt4(ii:ii+1,:)) - minval(bt4(ii:ii+1,:)))
 alb1_uni  = max(min_uni,maxval(alb1(ii:ii+1,:)) - minval(alb1(ii:ii+1,:)))
 alb2_uni  = max(min_uni,maxval(alb2(ii:ii+1,:)) - minval(alb2(ii:ii+1,:)))
 zsfc_uni = max(min_uni,maxval(zsfc(ii:ii+1,:)) - minval(zsfc(ii:ii+1,:)))
 
 mean_alb1 = 0.25 * sum(alb1(ii:ii+1,:))
 mean_bt4 = 0.25 * sum(bt4(ii:ii+1,:))                             
 mean_zsfc = 0.25 * sum(zsfc(ii:ii+1,:))                             
 
!--------- test for channel 3b or 3a
 bt3b_uni = max(min_uni,maxval(bt3b(ii:ii+1,:)) - minval(bt3b(ii:ii+1,:)))
 alb3b_uni  = max(min_uni,maxval(alb3b_mask(ii:ii+1,:)) - minval(alb3b_mask(ii:ii+1,:)))
 alb3a_uni  = max(min_uni,maxval(alb3a(ii:ii+1,:)) - minval(alb3a(ii:ii+1,:)))
 mean_alb3b = 0.25 * sum(alb3b_mask(ii:ii+1,:))


!----------------------------------------------------
! for cold pixels,  use mean ch3b reflectance
!---------------------------------------------------
 if (ch3a_on22 == 0) then
  where (bt4(ii:ii+1,:) < 250.0)
    alb3b_mask(ii:ii+1,:) = mean_alb3b
  end where
 endif
!-------------------------------------------------------
! test to see if this data is not invalid
!--------------------------------------------------------
! if ((bt4_uni == 0.0) .and. (maxval(bt4(ii:ii+1,:))>=325.0)) then
!     return
! endif
!-------------------------------------
! set bits pertaining to surface type
!-------------------------------------
  if (day22 .eqv. .true.) then
      data_mask(1,ii:ii+1,:) = 1
  end if 
  if (term22 .eqv. .true.) then
     data_mask(2,ii:ii+1,:) = 1
  endif
  if (land22 .eqv. .true.) then
      data_mask(3,ii:ii+1,:) = 1
  end if 
  if (coast22 .eqv. .true.) then
      data_mask(4,ii:ii+1,:) = 1
  end if 
  if (glintzen22 < 40.0) then 
    data_mask(5,ii:ii+1,:) = 1
  endif
  if (desert22 .eqv. .true.) then
      data_mask(6,ii:ii+1,:) = 1
  end if 
! if (snow_input22 .eqv. .true.) then
!     data_mask(7,ii:ii+1,:) = 1
! end if 
  if (ch3a_on22 == 1) then
      data_mask(8,ii:ii+1,:) = 1
  end if 


!---------------------------------------
! test for uniformity cloud and clear
!-----------------------------------------

 uni_factor = 1.0
 if (mean_zsfc > 1000.0) then
   uni_factor = 2.0
 endif

!--- tests for defining spatially uniform clear pixels
   if (day22 .eqv. .true.) then
    if (alb1_uni > uni_factor*max(0.5,mean_alb1*vis_ref_uni_clr(iday,iland)/100.0)) then
      mask(17,ii:ii+1,:) = 1
    endif
   else                         !(implicit assumes ch3b on during night)
    if (mean_bt4 > 250.0) then
     if ((land22 .eqv. .false.) .and. (alb3b_uni > uni_factor*max(1.0,mean_alb3b))) then
       mask(17,ii:ii+1,:) = 1
     endif
     if ((land22 .eqv. .true.) .and. (alb3b_uni > uni_factor*max(5.0,mean_alb3b))) then
       mask(17,ii:ii+1,:) = 1
     endif
    endif
   endif

!  if (bt4_uni > win_tmp_uni_clr(iday,iland)) then
!     mask(18,ii:ii+1,:) = 1
!  endif  
   if (bt4_uni > uni_factor*win_tmp_uni_clr(iday,iland) + zsfc_uni*0.0100) then
      mask(18,ii:ii+1,:) = 1
   endif  

!--- uniformity tests for defining totally cloudy pixels
   if (alb1_uni > mean_alb1*vis_ref_uni_cld(iday,iland)/100.0) then
        mask(19,ii:ii+1,:) = 1
   endif
   mask(19,ii:ii+1,:) = 0      !this is turned off
   if (bt4_uni > max(2.0,(280.0-mean_bt4)*win_tmp_uni_cld(iday,iland)/100.0)) then
     mask(20,ii:ii+1,:) = 1
   endif  


!-------------------------------------------
! loop through each pixel in 2x2 array and
! apply cloud tests to each pixel
!------------------------------------------
 do i = ii,ii+1
   do j = 1,ny


!-----------------------------------------------------------------
! test for potential of snow falsely triggering ref or therm tests
!------------------------------------------------------------------
    snow_potential_ref = snow_input22
    snow_potential_therm = snow_input22

!---- check for reflectance conditions that indicate snow potential
    if (day22 .eqv. .true.) then
      if (ch3a_on22 == 1) then
        if ((alb3a(i,j) < 15.0) .and. (alb1(i,j) > 10.0) .and.  &
            (bt4(i,j) < 277.0) .and. (ndsi(i,j) > 0.4)) then
!           (alb2(i,j) < 50.0)) then
            snow_potential_ref = .true.

            if ((bt4(i,j) < 260.0) .and. (ndsi(i,j) < 0.7)) then
             snow_potential_ref = .false.
            endif

        endif
      elseif (ch3a_on22 == 0) then
        if ((alb3b_mask(i,j) < 5.0) .and. (alb1(i,j) > 10.0).and.(alb2(i,j) < 50.0)) then
            snow_potential_ref = .true.
        endif
      endif

!--- not as universal as i thought
!     if ((vis_ref_ratio > 0.80) .and. (land22 .eqv. .false.)) then
!         snow_potential_ref = .false.
!     endif

!---- there are no frozen bodies of water in the tropics
      if ((abs(lat22) < 30.0) .and. (land22 .eqv. .false.)) then
          snow_potential_ref = .false.
      endif

    endif

!---- check for temperature conditions that indicate snow potential
   
    if (abs(lat22) > 50.0)  then

       snow_potential_therm = .true.

      if ((land22 .eqv. .false.) .and. (max_sst > 280.0) .and. (max_sst - bt4(i,j) > 30.0)) then
       snow_potential_therm = .false.
      endif

      if ((land22 .eqv. .true.) .and. (max_lst > 290.0) .and. (max_lst - bt4(i,j) > 30.0)) then
       snow_potential_therm = .false.
      endif

    elseif (abs(lat22) > 30.0) then

      if ((land22 .eqv. .true.) .and. (bt4(i,j) < 277.0) .and. &
          (max_lst - bt4(i,j) < 20.0)) then
         snow_potential_therm = .true.
      endif

      if (land22 .eqv. .false.) then
        snow_potential_therm = .false.
      endif

    endif    

    if (mtn22 .eqv. .true.) then
      snow_potential_therm = .true.
    endif

!-- snow should have low t4-t5
    if (bt45(i,j) > 1.0) then
      snow_potential_therm = .false.
    endif

!--- if thermal conditions show no snow potential, turn off ref
    if (snow_potential_therm .eqv. .false.) then
        snow_potential_ref = .false.
    endif

    if (snow_potential_ref .eqv. .true.) then
      data_mask(7,ii:ii+1,:) = 1
    endif


!------------------------------------------------------------
! determine fmft, tmft, ulst thresholds for this pixel
!-------------------------------------------------------------
! izen = max(1,min(nzen_lut-1,int(satzen(i,j)/10.0)+1))

  izen = max(1,min(nzen_lut-1,int((satzen(i,j)-zen_lut(1))/15.0)+1))
  xzen = max(0.0,min(1.0,(satzen(i,j)-zen_lut(izen))/15.0))

  it4 = max(1,min(nt4_lut-1,int((bt4(i,j) - 190.0)/10.0)+1))
  xt4 = (bt4(i,j)-t4_lut(it4))/10.0

  if (land22 .eqv. .true.) then
    fmft_thresh_hi = (1.0-xzen)*(1.0-xt4)*fmft_hi_land_lut(izen, it4) + &
                     (1.0-xzen)*(xt4)*fmft_hi_land_lut(izen, it4+1) + &
                     (xzen)*(1.0-xt4)*fmft_hi_land_lut(izen+1, it4) + &
                     (xzen)*(xt4)*fmft_hi_land_lut(izen+1, it4+1)

    fmft_thresh_lo = (1.0-xzen)*(1.0-xt4)*fmft_lo_land_lut(izen, it4) + &
                     (1.0-xzen)*(xt4)*fmft_lo_land_lut(izen, it4+1) + &
                     (xzen)*(1.0-xt4)*fmft_lo_land_lut(izen+1, it4) + &
                     (xzen)*(xt4)*fmft_lo_land_lut(izen+1, it4+1)

    tmft_thresh_hi = (1.0-xzen)*(1.0-xt4)*tmft_hi_land_lut(izen, it4) + &
                     (1.0-xzen)*(xt4)*tmft_hi_land_lut(izen, it4+1) + &
                     (xzen)*(1.0-xt4)*tmft_hi_land_lut(izen+1, it4) + &
                     (xzen)*(xt4)*tmft_hi_land_lut(izen+1, it4+1)

    tmft_thresh_lo = (1.0-xzen)*(1.0-xt4)*tmft_lo_land_lut(izen, it4) + &
                     (1.0-xzen)*(xt4)*tmft_lo_land_lut(izen, it4+1) + &
                     (xzen)*(1.0-xt4)*tmft_lo_land_lut(izen+1, it4) + &
                     (xzen)*(xt4)*tmft_lo_land_lut(izen+1, it4+1)
   else
    fmft_thresh_hi = (1.0-xzen)*(1.0-xt4)*fmft_hi_ocean_lut(izen, it4) + &
                     (1.0-xzen)*(xt4)*fmft_hi_ocean_lut(izen, it4+1) + &
                     (xzen)*(1.0-xt4)*fmft_hi_ocean_lut(izen+1, it4) + &
                     (xzen)*(xt4)*fmft_hi_ocean_lut(izen+1, it4+1)

    fmft_thresh_lo = (1.0-xzen)*(1.0-xt4)*fmft_lo_ocean_lut(izen, it4) + &
                     (1.0-xzen)*(xt4)*fmft_lo_ocean_lut(izen, it4+1) + &
                     (xzen)*(1.0-xt4)*fmft_lo_ocean_lut(izen+1, it4) + &
                     (xzen)*(xt4)*fmft_lo_ocean_lut(izen+1, it4+1)

    tmft_thresh_hi = (1.0-xzen)*(1.0-xt4)*tmft_hi_ocean_lut(izen, it4) + &
                     (1.0-xzen)*(xt4)*tmft_hi_ocean_lut(izen, it4+1) + &
                     (xzen)*(1.0-xt4)*tmft_hi_ocean_lut(izen+1, it4) + &
                     (xzen)*(xt4)*tmft_hi_ocean_lut(izen+1, it4+1)

    tmft_thresh_lo = (1.0-xzen)*(1.0-xt4)*tmft_lo_ocean_lut(izen, it4) + &
                     (1.0-xzen)*(xt4)*tmft_lo_ocean_lut(izen, it4+1) + &
                     (xzen)*(1.0-xt4)*tmft_lo_ocean_lut(izen+1, it4) + &
                     (xzen)*(xt4)*tmft_lo_ocean_lut(izen+1, it4+1)
   endif

!------ determine land reflectance thresholds
   scat_angle = -1.0*(cos(satzen(i,j)*dtor)*cos(solzen(i,j)*dtor)- &
                          sin(satzen(i,j)*dtor)*sin(solzen(i,j)*dtor)*cos(relaz(i,j)*dtor))
   scat_angle = acos(max(-1.0,min(1.0,scat_angle)))/dtor
   iscat_umd = min(nscat_umd,max(1,int((scat_angle-min_scat_umd)/delta_scat_umd) + 1))
   rgct_thresh_land =  ch1_mean_umd(sfc_type(i,j),iscat_umd) +                          &
                      std_factor(sfc_type(i,j))*ch1_std_umd(sfc_type(i,j),iscat_umd)
   rrct_thresh_land =  ch21_mean_umd(sfc_type(i,j),iscat_umd)

!print *,"before rgct"
!---- RGCT
     if ((day22 .eqv. .true.) .and. (snow_potential_ref .eqv. .false.)) then

      if ((land22 .eqv. .true.) .and. (coast22 .eqv. .false.)) then
       rgct_thresh = rgct_thresh_land
      elseif (land22 .eqv. .false.) then
       rgct_thresh = vis2_ocn_ref_thresh
      else
       rgct_thresh = 44.0
      endif

!--- apply
       if ((land22 .eqv. .true.) .and. (alb1(i,j) > rgct_thresh))  then
        mask(1,i,j) = 1
       endif
       if ((land22 .eqv. .false.) .and. (alb2(i,j) > rgct_thresh)) then
         mask(1,i,j) = 1
       endif

     endif

!print *, "past rgct"


!-- RRCT
if ((day22 .eqv. .true.) .and. (snow_potential_ref .eqv. .false.)) then
   if (land22 .eqv. .true.) then
!    if ((solzen(i,j) > 75.0) .or. (sfc_type(i,j) == 12) .or.  &
     if ((sfc_type(i,j) == 12) .or.  &
         (coast22 .eqv. .true.)) then
        rrct_thresh_min = 0.00
        rrct_thresh_max = 0.00
     else
        rrct_thresh_min = 0.00
        rrct_thresh_max = rrct_thresh_land
     endif
   else
        rrct_thresh_min = vis21_ocn_ref_thresh
        rrct_thresh_max = 99.0
   endif

!--apply test
   if ((alb21(i,j) > rrct_thresh_min).and. &
       (alb21(i,j) < rrct_thresh_max) .and. &
       (bt4(i,j) < 300.0)) then
       mask(2,i,j) = 1
   endif
endif

!apply test
!       if (alb21*(iland-0.5) <  &
!                     alb21_thresh(iday,iland)*(iland-0.5)) then
!----------------------------------------------------------------------
!-- C3AT  - channel 3a/3b albedo test
!--  during day over ocean use 62 derived thresholds
!--  ignore deserts during day due to high sfc reflectance
!--  at night, compute a ch3b albedo, look for large
!--  negative (water cloud) and positive (cirrus) values
!--  desert at night will produce positive alb3b, so use tighter threshold
!----------------------------------------------------------------------------

!--- ch3b c3at
   if ((ch3a_on22 == 0) .and. (bt4(i,j) > 240.0)) then
     if (day22 .eqv. .true.) then
        if ((land22 .eqv. .false.).and.(coast22 .eqv. .false.)) then
           if (alb3b_mask(i,j) > nir2_ocn_ref_thresh) then
             mask(3,i,j) = 1
           endif
        else
          if (desert3b22 .eqv. .false.) then
           if (alb3b_mask(i,j) > nir_ref_thresh(iday,iland,ch3a_on22)) then
             mask(3,i,j) = 1
           endif       
          endif
        endif

     elseif (term22 .eqv. .false.) then

!      icold = 0
!      if (bt4(i,j) < 250.0) then
!        icold = 1
!      endif
!-- test for positive values
!      if ((land22 .eqv. .false.) .and.  &
!          (alb3b_mask(i,j) > 8.0 + icold*2.0*alb3b_uni)) then
!            mask(3,i,j) = 1
!      endif
!      if ((land22 .eqv. .true.) .and. (desert3b22 .eqv. .false.) .and.  &
!          (alb3b_mask(i,j) > 12.0 + icold*2.0*alb3b_uni)) then
!            mask(3,i,j) = 1
!      endif 
!-- test for negative values
!      if (desert3b22 .eqv. .false.) then
!       if ((alb3b_mask(i,j)< -1.0*(4.0+icold*alb3b_uni))) then
!            mask(3,i,j) = 1
!        endif 
!      else
!        if ((alb3b_mask(i,j) < -1.0*(2.0+icold*alb3b_uni))) then
!            mask(3,i,j) = 1
!        endif 
!      endif


     endif
   endif


!--ch3a c3at 
   if (ch3a_on22 == 1) then
      if (solzen22 <  88.0) then

         if ((land22 .eqv. .false.) .and. (data_mask(5,i,j) == 0)) then !water clouds unfrozen ocean
           if ( (alb1(i,j)>10.0).and.(alb3a1(i,j) > 0.5)) then
            mask(3,i,j) = 1
           endif
         endif

         if (land22 .eqv. .true.) then

!------ ice and mixed phase cloud
            if ((alb21(i,j) < 1.2).and.(alb3a1(i,j) < 1.1).and. &
                (alb3a(i,j) > 15.0).and.(bt4(i,j) < 260.0)) then
              mask(3,i,j) = 1
            endif

!--- water cloud
!-- no clear land pixel should have this condition???
            if ((alb3a1(i,j) < 1.1) .and. (alb3a(i,j) > 20.0)) then
              mask(3,i,j) = 1
            endif

!           if ((alb21 < 1.2).and.(alb3a1(i,j) < 1.1).and. &
!               (alb3a(i,j) > 25.0)) then
!             mask(3,i,j) = 1
!           endif

!---- thin cirrus 
!         if ((snow_potential_ref .eqv. .false.) .and. (alb3a1(i,j) < 1.5) .and.  &
!             (desert3b22 .eqv. .false.) .and. (alb1(i,j) > 10.0)) then
!             mask(3,i,j) = 1
!         endif


!--- where snow is impossible, go after all ice cloud over land

         endif
     endif
   endif


!---- TGCT
!  if (((snow_potential_therm .eqv. .false.) .or. (ocean22 .eqv. .true.)) .and.&
   if ((snow_potential_therm .eqv. .false.) .and. &
       (bt4(i,j) < win_tmp_thresh(iday,iland)) ) then
     mask(4,i,j) = 1
   endif

!---- ULST
   if ((ch3a_on22 == 0).and. (desert3b22 .eqv. .false.) .and. &
       (bt4(i,j) > 250.0)) then
    if (day22 .eqv. .false.) then

     if (land22 .eqv. .false.) then
      tmft_thresh_lo = exp(-9.375 + 0.0342 * bt4(i,j))-1.0
      if (bt3b5(i,j) < tmft_thresh_lo) then
       mask(5,i,j) = 1
      endif
     else
      if ((bt4(i,j) > 271.0) .and. (bt4(i,j) < 289.0)) then
       tmft_thresh_lo = exp(-9.375 + 0.0342 * bt4(i,j))-3.0
       if (bt3b5(i,j) < tmft_thresh_lo) then
        mask(5,i,j) = 1
       endif
      endif
     endif 
    endif
   endif

!--- CIRT 
  cirt_min = 0.0100
  if (ch3a_on22 == 0) then
   if ((day22 .eqv. .false.) .and. (term22 .eqv. .false.).and.(bt4(i,j) > 240.00)) then

    cirt_ratio = (bt3b5(i,j))/(bt5(i,j))
    if (bt4(i,j) < 273.0) then
     cirt_thresh = cirt_min
    elseif (bt4(i,j) > 292.0) then
     cirt_thresh = 0.033
    else
     cirt_thresh = max(cirt_min,1.77467e-3 * bt4(i,j) - 0.485328)
    endif

    if (cirt_ratio  > cirt_thresh) then
      mask(6,i,j) = 1
    end if
   end if
  endif
  mask_weight(6,i,j) = 0

!---- FMFT
!----------------
!  fmft_thresh = FMFT_THRESHOLD(bt4(i,j),land22)  !old CLAVR-1 threshold

!------- Key's FMFT
! ikey = min(nfmft_key-1,max(1,int((bt4(i,j)-190.0)/10.0)+1))
! zc_adj = zc(ikey) + (bt4(i,j)-(190.0+(ikey-1)*10.0))*(zc(ikey+1)-zc(ikey))/(10.0)
! fmft_thresh_hi = ct_thresh(ikey) +  (bt4(i,j)-(190.0+(ikey-1)*10.0))*(ct_thresh(ikey+1)-ct_thresh(ikey))/(10.0)
! fmft_thresh_lo = wt_thresh(ikey) + (bt4(i,j)-(190.0+(ikey-1)*10.0))*(wt_thresh(ikey+1)-wt_thresh(ikey))/(10.0)
! bt45_adj = bt45(i,j) - (23.6 - zc_adj)*(1-coszen(i,j))/(1.0-0.1589*(1.0-coszen(i,j)))

!--------- FMFT
   if ((bt4(i,j) > 230.0).and.(bt4(i,j) < 315.0)) then
    if (bt45(i,j) > fmft_thresh_hi) then
      mask(7,i,j) = 1
    endif
    if ((bt45(i,j) < fmft_thresh_lo).and.(desert22 .eqv. .false.).and.(bt4(i,j) < 289.0)) then
     mask(7,i,j) = 1
    endif
   endif

!-------- TMFT
 if ((ch3a_on22 == 0) .and. (bt3b(i,j) > 240.0)) then

       if (day22 .eqv. .true.) then

         if ((land22 .eqv. .false.) .and. (data_mask(5,i,j) == 0) .and. (snow_input22 .eqv. .false.)) then
               tmft_thresh_hi = tmft_thresh_hi
         elseif (land22 .eqv. .true.) then
               tmft_thresh_hi = max(5.0,20.0 * cossolzen(i,j))
         else
               tmft_thresh_hi = 99.0
         endif 

       endif

       if ((day22 .eqv. .false.) .and. (term22 .eqv. .false.)) then
         if ( bt3b5(i,j) > max(4.5,tmft_thresh_hi)) then 
!        if ( bt3b5(i,j) > tmft_thresh_hi) then
           mask(8,i,j) = 1
         endif
       endif

 endif
!----- old nightimeTMFT
!        tmft_thresh= min(10.0,max(2.0,10.0 - (0.4)*(280.0-bt4(i,j))))
!         if (bt3b4(i,j) > tmft_thresh) then
!           mask(8,i,j) = 1
!         endif
!         if ((land22.eqv. .false.).and.(bt3b4(i,j) < -0.2)) then
!           mask(8,i,j) = 1
!         endif
!         if ((land22.eqv. .true.).and.(bt3b4(i,j) < -4.0)) then
!           mask(8,i,j) = 1
!         endif


!--------------------------------------------------------
! CSST - SST CLIMATOLOGY TEST
!--------------------------------------------------------
  if ((snow_potential_therm .eqv. .false.) .and. &
      (land22 .eqv. .false.) .and. (sst_clim(i,j) > 270.0)) then
       if (sst_mask(i,j) < sst_clim(i,j) - 5.0) then
           mask(9,i,j) = 1
       endif
  endif
! mask_weight(9,i,j) = 0





end do
end do

!----------------------------------------------------------
! RESTORAL TESTS
!-----------------------------------------------------------
 do i = ii,ii+1
   do j = 1,ny

 shadow = .false.
 snow = .false.
 dust = .false.
 fire = .false.
 bt4_uni = max(min_uni,maxval(bt4(ii:ii+1,:)) - minval(bt4(ii:ii+1,:)))

!---------------------------------------------------------
! shadow restoral
!---------------------------------------------------------
if ((day22 .eqv. .true.).and.(land22 .eqv. .true.) .and. &
    (coast22 .eqv. .false.) .and. (alb1(i,j) < 10.0).and. &
    (ndvi(i,j) < 0.0)) then
   shadow = .true.
endif

!--------------------------------------------------
! dust over ocean restoral test
! 
! this is based on VIIRS algo, but does not seem
! to work - turning off
!---------------------------------------------------
  if ((day22 .eqv. .true.) .and. (data_mask(5,i,j) == 0) .and.  &
      (land22 .eqv. .false.) .and. (polar22 .eqv. .false.) .and. &
      (ch3a_on22 == 0) ) then
      if ( (bt4(i,j) > 270.0) .and. &
           (bt45(i,j) < 1.0) .and. &
           (sum(mask(17:18,i,j)) == 0) .and. &
           (mask(1,i,j) == 1) .and. &
           (alb3b_mask(i,j) > 3.0) .and. &
           (alb3b_mask(i,j) < 10.0) .and. &
           (sum(mask(4:7,i,j)) == 0) ) then
           dust = .true.
      endif
  endif
  dust = .false.
!-----------------------------------------------------------
! fire
!-----------------------------------------------------------
if ((ch3a_on22 == 0).and. (day22 .eqv. .true.) .and.  &
    (land22 .eqv. .true.) .and.  &
    (desert22 .eqv. .false.) .and. (coast22 .eqv. .false.))  then

 if ( (bt3b(i,j) > 310.0) .and.  &
      (bt3b(i,j) - bt4(i,j) > 6.0) .and. &
      (bt4(i,j) > 260.0) .and. &
      (bt45(i,j) < 4.0) .and. &
      (alb2(i,j) < 25.0) .and. &
      (bt3b_uni> 8.0) .and. &
      (alb2_uni < 10.0) .and. &
      (bt3b_uni / bt4_uni > 2.0) ) then

      fire = .true.

 endif

endif

!print *, "snow day res"

!----------------------------------------------------------
! snow restoral tests
!----------------------------------------------------------

 if (snow_input22 .eqv. .true.) then
   snow = .true.
 else
   snow = .false.
 endif

!----- Add MODIS ndsi threshold if ch3a is available 
! this test will flag ice cloud as snow, we restrict snow to following
!----------------------------------------------------------------------
if (snow_potential_ref .eqv. .true.) then
  if ((day22 .eqv. .true.) .and. (ch3a_on22 == 1) .and. &
      (alb3a1(i,j) < 0.5) .and. (alb1(i,j) > 10.0) .and. (alb3a(i,j) < 15.0)) then

     if (land22 .eqv. .true.) then
       snow = .true.
     endif
     if ((land22 .eqv. .false.) .and. (alb21(i,j) < vis_ref_ratio_snow_min)) then
       snow = .true.
     endif

!--- condition 1
!     if (alb21(i,j) < vis_ref_ratio_snow_min) then  ! artic snow pack
!      snow = .true.
!     endif

!------ ice clouds except cirrus should be colder
!    if (bt4(i,j) > 260.0)  then
!      snow = .true.
!     endif

!------- snow in forests
!    if ((alb21(i,j) > 1.3).and.(bt4(i,j) > 250.0))  then
!      snow = .true.
!     endif

 endif
 
!----------- channel 3b snow day snow restoral
  if ((day22 .eqv. .true.) .and. (ch3a_on22 == 0) .and. &
      (bt3b4(i,j) < 8.0) .and. (alb1(i,j) > 10.0) .and. (alb3b_mask(i,j) < 6.0)) then

     if (land22 .eqv. .true.) then
       snow = .true.
     endif
     if ((land22 .eqv. .false.).and.(alb21(i,j) < vis_ref_ratio_snow_min)) then
       snow = .true.
     endif
  endif

 endif


!----------------------------------------------------------
! apply restoral tests  - by turning off appropriate tests
!---------------------------------------------------------
!print *, "apply res"

if (shadow .eqv. .true.) then
    mask(21,i,j) = 1
end if 
if (dust .eqv. .true.) then
    mask(22,i,j) = 1
endif
if (snow .eqv. .true.) then
     mask(23,i,j) = 1
endif
if (fire .eqv. .true.) then
  mask(24,i,j) = 1
endif

enddo
enddo

!----------------------------------------------------------
! based on restoral tests turn off appropriate mask weights
!----------------------------------------------------------
do i = ii,ii+1
 do j = 1,ny
 
!-- turn off no tests for shadow
 
   if (mask(22,i,j) == 1) then     !dust
       mask_weight(1:4,i,j) = 0
   endif
   if (mask(23,i,j) == 1) then    !snow
       mask_weight(1:2,i,j) = 0
       if (ch3a_on22 == 1) then   
         mask_weight(3,i,j) = 0
       endif
   endif
   if (mask(24,i,j) == 1) then    !fire
       mask_weight(:,i,j) = 0
   endif
 
!------------------------------------------------------
! desert day - check if this is really necessary?
!------------------------------------------------------
!  if ( (day22 .eqv. .true.) .and. (desert22 .eqv. .true.) ) then
!     if ((bt4(i,j) > 293.0).and.(bt4(i+1,j) > 293.0) .and. & 
!         (bt4(i,j+1) > 293.0).and.(bt4(i+1,j+1) > 293.0))  then
!       mask_weight(1:3,:,:) = 0
!       stop
!     endif
!  endif

enddo
enddo
!-----------------------------------------------------------
! based on test results, make decision
!-----------------------------------------------------------
do i = ii,ii+1
   do j = 1,ny
   if (sum(mask(1:n_cld_test,i,j)*mask_weight(1:n_cld_test,i,j)) == 0) then
       if (sum(mask(17:18,i,j)) == 0) then
         cld_mask(i,j) = 0 
       else
         cld_mask(i,j) = 1
       endif
    else
       if (sum(mask(19:20,i,j)) == 0) then
         cld_mask(i,j) = 3 
       else
         cld_mask(i,j) = 2
       endif
    endif

!-----------------------------------------------------------------
! recheck that cloudy pixels are not classified as snowy
!-----------------------------------------------------------------
    if (cld_mask(i,j) > 1) then
       mask(22,i,j) = 0
       mask(23,i,j) = 0
       mask(24,i,j) = 0
    endif

!-----------------------------------------------------------------
! call all ocean pixels < 270 snow - this is for cloud mask check
!----------------------------------------------------------------
    if ((land22 .eqv. .false.) .and. (cld_mask(i,j) == 0) .and. &
        (bt4(i,j) < 270.0)) then
         mask(23,i,j) = 1
    endif

!----------------------------------------------------------------
! call all shadow pixels as partly clear
!----------------------------------------------------------------
    if ((cld_mask(i,j) == 0).and.(mask(21,i,j) == 1)) then
       cld_mask(i,j) = 1
    endif
  

!-----------------------------------------------------------------
! pack masks into data_bit, mask_byte_1,2,3
!------------------------------------------------------------------
   data_bit(i,j) = 0
   mask_byte_1(i,j) = 0
   mask_byte_2(i,j) = 0
   mask_byte_3(i,j) = 0
   do k = 1, 8
      data_bit(i,j) = data_bit(i,j) + 2**(8-k) * data_mask(k,i,j)
      mask_byte_1(i,j) = mask_byte_1(i,j) + 2**(8-k) * mask(k,i,j)
      mask_byte_2(i,j) = mask_byte_2(i,j) + 2**(8-k) * mask(k+8,i,j)
      mask_byte_3(i,j) = mask_byte_3(i,j) + 2**(8-k) * mask(k+16,i,j)
   enddo

  end do
 end do

!--------------------------------------------------------------
! if any pixel is 2x2 is cloudy, surrounding can't be snow
!--------------------------------------------------------------
    if ((sum(mask(23,ii:ii+1,:)) >= 1) .and. (maxval(cld_mask(ii:ii+1,:)) == 3)) then
        mask(23,ii:ii+1,:) = 0
        cld_mask(ii:ii+1,:) = 2
    endif

 ii = ii + 2

 end do ii_loop

 end subroutine CLOUD_MASK_22

end module CLOUD_MASK
