! $Id: elevation_map.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!------------------------------------------------------------------------
! a elevrain mask taken NASA Langley CERES/SARB Surface Properties
! http://www-surf.larc.nasa.gov/surf/pages/digelev.html
!
! global - 10'  (2160 x 1080) starts at NP and GM
!
!-------------------------------------------------------------------------
module ELEVATION_MAP
  use CONSTANTS
  implicit none
  private
  public:: READ_ELEVATION_MAP
  integer, parameter, public:: num_lon_elev = 2160, num_lat_elev = 1080 
  real, parameter, public:: first_lon_elev = 0.0, first_lat_elev = 90.0, & 
                            last_lon_elev = 360.0, last_lat_elev = -90.0
  real, parameter, public:: del_lon_elev = (last_lon_elev - first_lon_elev)/num_lon_elev
  real, parameter, public:: del_lat_elev = (last_lat_elev - first_lat_elev)/num_lat_elev
  integer(kind=int2),dimension(num_lon_elev,num_lat_elev), public, save::  &
                               elev_map

  contains

  subroutine READ_ELEVATION_MAP(elev_name)
   character(len=*), intent(in):: elev_name
   integer:: ilat

   print *,"opening ", trim(elev_name)
   open(unit=7,file=trim(elev_name),access="direct",status="old", &
       action="read",recl=int(2*num_lon_elev))
   do ilat = 1, num_lat_elev
      read(unit=7,rec=ilat) elev_map(:,ilat)
   enddo
   
   close(unit=7)
   print *, "digital elev map read in,  range (m) = ", minval(elev_map), maxval(elev_map) 

  end subroutine READ_ELEVATION_MAP

end module ELEVATION_MAP
