! $Id: sfc_prop_umd.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!------------------------------------------------------------------
! this is module of surface properties (reflectance, emisivity) for
! the UMD surface type classification
!
!
!     0: Water
!     1: Evergreen Needleleaf Forests
!     2: Evergreen Broadleaf Forests
!     3: Deciduous Needleleaf Forests
!     4: Deciduous Broadleaf Forests
!     5: Mixed Forests
!     6: Woodlands
!     7: Wooded Grasslands/Shrubs
!     8: Closed Bushlands or Shrublands
!     9: Open Shrublands
!     10: Grasses
!     11: Croplands
!     12: Bare
!     13: Mosses and Lichens
!     14: Snow  ( i added this)
!
!  this properties are bases on the ones given by the CERES SARB
!  team for the IGBP classes. The closest IGBP class was used.
!
! spectral intervals for albedoes:
!         0.2-0.7/0.7-1.3/1.3-1.9/1.9-2.5/2.5-3.5/3.5-4.0 (microns)
! 
! spectral range for window emissivities = ??
!
! for both desert and igbp files, longitudes goes from 0 to 360
! but the input is assumed to go from -180 to 180
!------------------------------------------------------------------
module SURFACE_PROPERTIES
use CONSTANTS
implicit none
private
public:: GET_SFC_PROPS,SETUP_UMD_PROPS, READ_LAND_COVER_UMD
integer, parameter, public:: ntype_sfc=15,nspec_sfc=6,nzen_emiss=10
real(kind=real4), dimension(0:ntype_sfc-1,nspec_sfc), public,save:: spec_albedo
real(kind=real4), dimension(0:ntype_sfc-1,nzen_emiss), public,save:: &
                                                  win1_emiss,win2_emiss
real(kind=real4), dimension(0:ntype_sfc-1), public, save:: alb_solzen_factor
real(kind=real4), dimension(nzen_emiss), public, save:: zen_emiss
real(kind=real4), public, save:: delta_zen_emiss

!------
 integer, parameter, public:: num_lat_land_mask = 2880, num_lon_land_mask = 5760
  real, parameter, public:: first_lon_land_mask = -180.0, first_lat_land_mask = 90.0, &
                            last_lon_land_mask = 180.0, last_lat_land_mask = -90.0
 real, parameter, public:: del_lon_land_mask = (last_lon_land_mask - first_lon_land_mask)/num_lon_land_mask
 real, parameter, public:: del_lat_land_mask = (last_lat_land_mask - first_lat_land_mask)/num_lat_land_mask
 integer(kind=int1),dimension(num_lon_land_mask,num_lat_land_mask), public, save::  &
                               land_cover_umd
contains
!-------------------------------------------------------------------------
  subroutine READ_LAND_COVER_UMD(land_cover_umd_name)
   character(len=*), intent(in):: land_cover_umd_name
   integer:: ilat
   print *,"opening ", trim(land_cover_umd_name)
   open(unit=7,file=trim(land_cover_umd_name),access="direct",status="old", &
       action="read",recl=int(num_lon_land_mask))
   read(unit=7,rec=1)
   do ilat = 1, num_lat_land_mask
      read(unit=7,rec=ilat) land_cover_umd(:,ilat)
   enddo
   close(unit=7)
 
  end subroutine READ_LAND_COVER_UMD
!-----------------------------------------------------------------
! read into memory the radiative properties for each UMD type
!-----------------------------------------------------------------
subroutine SETUP_UMD_PROPS()

   spec_albedo = reshape((/&
     0.0163,0.0163,0.0163,0.0163,0.0163,0.0163,& ! (17) OCEAN WATER
     0.0406,0.1991,0.2020,0.2020,0.2020,0.2020,& !( 1) EVERGREEN NEEDLE FOR
     0.0444,0.2179,0.2210,0.2210,0.2210,0.2210,& !( 2) EVERGREEN BROAD FOR 
     0.0406,0.1991,0.2020,0.2020,0.2020,0.2020,& ! ( 3) DECIDUOUS NEEDLE FOR 
     0.0542,0.2943,0.2980,0.2980,0.2980,0.2980,& ! ( 4) DECIDUOUS BROAD FOR
     0.0423,0.2500,0.2500,0.2500,0.2500,0.2500,& ! ( 5) MIXED FOREST
     0.0423,0.2500,0.2500,0.2500,0.2500,0.2500,& ! ( 5) MIXED FOREST
     0.0576,0.2700,0.2700,0.2700,0.2700,0.2700,& ! ( 8) WOODY SAVANNA
     0.1242,0.3072,0.3400,0.3400,0.3400,0.3400,& ! ( 6) CLOSED SHRUBS
     0.1781,0.2942,0.2580,0.2580,0.2580,0.2580,& ! ( 7) OPEN/SHRUBS
     0.0788,0.2919,0.2980,0.2980,0.2980,0.2980,& ! (10) GRASSLAND
     0.0677,0.2520,0.2520,0.2520,0.2520,0.2520,& ! (14) CROP MOSAIC
     0.3124,0.4019,0.3900,0.3900,0.3900,0.3900,& ! (16) BARREN/DESERT
     0.0918,0.2466,0.2650,0.2650,0.2650,0.2650,& ! (18) TUNDRA
     0.9763,0.8194,0.1743,0.1310,0.1310,0.1310/),& ! (15) ANTARCTIC SNOW
     (/ntype_sfc,nspec_sfc/),order=(/2,1/))

   alb_solzen_factor = reshape((/&
     0.41,&  ! (17) OCEAN WATER
     0.40,&  ! ( 1) EVERGREEN NEEDLE FOR
     0.44,&  ! ( 2) EVERGREEN BROAD FOR
     0.32,&  !  ( 3) DECIDUOUS NEEDLE FOR
     0.39,&  ! ( 4) DECIDUOUS BROAD FOR
     0.22,&  ! ( 5) MIXED FOREST
     0.22,&  ! ( 5) MIXED FOREST
     0.15,&  ! ( 8) WOODY SAVANNA
     0.28,&  ! ( 6) CLOSED SHRUBS
     0.40, &  ! ( 7) OPEN/SHRUBS
     0.22,&  ! (10) GRASSLAND
     0.12,&  ! (14) CROP MOSAIC
     0.40,&   ! (16) BARREN/DESERT
     0.58,&  ! (18) TUNDRA
     0.10/),&  ! (15) Antartic Snow
     (/ntype_sfc/))

!------ zenith angles for emissivity tables

   zen_emiss = reshape((/ &
   0.000,10.00,20.00,30.00,40.00,50.00,60.00,70.00,80.00,90.0/),(/nzen_emiss/))
   delta_zen_emiss = zen_emiss(2) - zen_emiss(1)
   
!--- first  window channel (avhrr ch4)
   win1_emiss = reshape((/ &
   0.993,0.992,0.992,0.991,0.989,0.982,0.965,0.935,0.935,0.935, & !17
   0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990, & !1
   0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990, & !2
   0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978, & !3
   0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978, & !4
   0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984, & !5
   0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984, & !5
   0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982, & !8
   0.955,0.955,0.955,0.955,0.955,0.955,0.955,0.955,0.955,0.955, & !6
   0.897,0.897,0.897,0.897,0.897,0.897,0.897,0.897,0.897,0.897, & !7
   0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982, & !10
   0.983,0.983,0.983,0.983,0.983,0.983,0.983,0.983,0.983,0.983, & !14
   0.869,0.869,0.869,0.869,0.869,0.869,0.869,0.869,0.869,0.869, & !16
   0.981,0.981,0.981,0.981,0.981,0.981,0.981,0.981,0.981,0.981,& !18
   1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000 & !15
   /),(/ntype_sfc,nzen_emiss/),order=(/2,1/))


!--- second  window channel (avhrr ch5)
   win2_emiss = reshape((/ &
   0.989,0.989,0.988,0.987,0.983,0.973,0.949,0.905,0.905,0.905, & !17
   0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990, & !1
   0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990,0.990, & !2
   0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978, & !3
   0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978,0.978, & !4
   0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984, & !5
   0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984, & !5
   0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982, & !8
   0.955,0.955,0.955,0.955,0.955,0.955,0.955,0.955,0.955,0.955, & !6
   0.897,0.897,0.897,0.897,0.897,0.897,0.897,0.897,0.897,0.897, & !7
   0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982,0.982, & !10
   0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984,0.984, & !11
   0.983,0.983,0.983,0.983,0.983,0.983,0.983,0.983,0.983,0.983, & !14
   0.869,0.869,0.869,0.869,0.869,0.869,0.869,0.869,0.869,0.869, & !16
   0.981,0.981,0.981,0.981,0.981,0.981,0.981,0.981,0.981,0.981, & !18
   1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000,1.000 & !15
   /),(/ntype_sfc,nzen_emiss/),order=(/2,1/))


end subroutine SETUP_UMD_PROPS
!---------------------------------------------------------------------------
! routine to compute surface radiative properties for AVHRR channels
! input:
!          lat, lon - latitude and longitude in degrees
!          solzen - solar zenith angle in degrees
! output:  alb1,alb2,alb3 - surface albedos as a fraction of 1
!          ems3,ems4,ems5 - surface emissivities as a fraction of 1
!
! note - ems3 is computed as 1.0 - alb3 even for nighttime
!
!----------------------------------------------------------------------------
 subroutine GET_SFC_PROPS(scene,solzen,satzen,alb1,alb2,alb3a,alb3b, &
                          ems3b,ems4,ems5,ems6)  
  integer(kind=int1),intent(in):: scene
  real (kind=ipre), intent(in):: solzen,satzen
  real (kind=ipre), intent(out):: alb1,alb2,alb3a,alb3b,ems3b,ems4,ems5,ems6
  real (kind=real4):: zenadj,xzen
  integer:: izen

  if (solzen < 90.0) then
     zenadj =  ( 1.0+alb_solzen_factor(scene) ) /  &
               ( 1.0+2.0*alb_solzen_factor(scene)* &
                 cos(solzen*3.14159265/180.0) )
   else
     zenadj = 1.0
   endif

   alb1 = zenadj * spec_albedo(scene,1)
   alb2 = zenadj * spec_albedo(scene,2)
   alb3a = zenadj * spec_albedo(scene,3)
   alb3b = zenadj * spec_albedo(scene,6)
   
   ems3b = 1.0 - alb3b

   izen =  max(1,min(nzen_emiss-1, &
                int((satzen - zen_emiss(1))/delta_zen_emiss) + 1))
   xzen = (satzen - zen_emiss(izen))/(zen_emiss(izen+1) - zen_emiss(izen))
   ems4 = (1.0-xzen)*win1_emiss(scene,izen) + xzen*win1_emiss(scene,izen+1)
   ems5 = (1.0-xzen)*win2_emiss(scene,izen) + xzen*win2_emiss(scene,izen+1)

!----------------- no idea what to do for ch6
   ems6 = ems5

 end subroutine GET_SFC_PROPS

end module SURFACE_PROPERTIES
