! $Id: sst_climatology.f90,v 1.1 2003/05/20 16:26:42 dlloyd Exp $
!------------------------------------------------------------------------
! a climatology of SST for each month based on 50 km data
! from OSDPD's 50 km data taken from SAA
!
! global (80S - 85N)  - 0.5'  (721 x 331) starts at 80S and GM
!
!-------------------------------------------------------------------------
module SST_CLIMATOLOGY
  use CONSTANTS
  implicit none
  private
  public:: READ_SST_CLIMATOLOGY_MAP
  integer, parameter, public:: num_lon_sst_clim = 721, num_lat_sst_clim = 331 
  real, parameter, public:: first_lon_sst_clim = 0.0, first_lat_sst_clim = -80.0, & 
                            last_lon_sst_clim = 360.0, last_lat_sst_clim = 85.0
  real, parameter, public:: del_lon_sst_clim = (last_lon_sst_clim - first_lon_sst_clim)/num_lon_sst_clim
  real, parameter, public:: del_lat_sst_clim = (last_lat_sst_clim - first_lat_sst_clim)/num_lat_sst_clim
  real, parameter, private:: min_sst_clim = -4.0, max_sst_clim=36.0, min_sst_clim_std = -1.0, &
                             max_sst_clim_std = 10.0
  integer(kind=int1),dimension(num_lon_sst_clim,num_lat_sst_clim), private::  temp
  real(kind=real4),dimension(12,num_lon_sst_clim,num_lat_sst_clim), public, save::  &
                               sst_clim_map
  real(kind=real4),dimension(num_lon_sst_clim,num_lat_sst_clim), public, save::  &
                               sst_clim_map_min,sst_clim_map_std

  contains

  subroutine READ_SST_CLIMATOLOGY_MAP(sst_clim_name)
   character(len=*), intent(in):: sst_clim_name
   integer:: m

   print *,"opening ", trim(sst_clim_name)

   open(unit=7,file=trim(sst_clim_name),access="direct",status="old", &
       action="read",recl=num_lon_sst_clim*num_lat_sst_clim)

   do m = 1,12
      read(unit=7,rec=m)  temp
      sst_clim_map(m,:,:) =  min_sst_clim + (max_sst_clim - min_sst_clim)*(temp + 128)/255.0
   enddo
   read(unit=7,rec=13)  temp
   sst_clim_map_min =  min_sst_clim + (max_sst_clim - min_sst_clim)*(temp + 128)/255.0
   read(unit=7,rec=14)  temp
   sst_clim_map_std =  min_sst_clim_std + (max_sst_clim_std - min_sst_clim_std)*(temp + 128)/255.0
 
   close(unit=7)

!-------- convert from Celsius to Kelvin

!    sst_clim_map = sst_clim_map + 273.15
   where(sst_clim_map == min_sst_clim)
     sst_clim_map = missing_value_real4
   endwhere
   where(sst_clim_map > min_sst_clim)
     sst_clim_map = sst_clim_map + 273.15
   endwhere

   where(sst_clim_map_min == min_sst_clim)
     sst_clim_map_min = missing_value_real4
   endwhere
   where(sst_clim_map_min > min_sst_clim)
     sst_clim_map_min = sst_clim_map_min + 273.15
   endwhere

   where(sst_clim_map_std == min_sst_clim_std)
     sst_clim_map_std = missing_value_real4
   endwhere
   print *, "sst_clim data read in,  range (m) = ", minval(sst_clim_map), maxval(sst_clim_map) 

  end subroutine READ_SST_CLIMATOLOGY_MAP

end module SST_CLIMATOLOGY
