/*****************************************************************************
FUNCTION:	CLOSEIMAGES

PURPOSE:	Close image and minor frame files.

INPUT:		Array of file descriptors - NULL terminated
		minor frame file descriptor
		intput image file name
		image type flag

OUTPUT:		none

RETURNS:	E_SUCC on success
		E_FAIL on failure

*****************************************************************************/
#include "las.h"
#include "labserv.h"
#include "clavr.h"



int closeimages
    (
    struct FDESC **fdesc,	/* IN - Null terminated array of descriptors*/
    long *minorframe_fdesc,	/* IN - minor frame file descriptor         */
    char *input_file,		/* IN - input file name			    */
    int input_image_type	/* IN - input image type: SINGLE_SCENE or   */
    				/*      COMPOSITE			    */
    )
    {
    char filename[CMLEN];	/* minor frame file name		    */

    int d_index;		/* descriptor index			    */
    int return_status = E_SUCC;	/* status to return to calling function	    */
    int status;			/* function return status		    */

    long action = 0;		/* normal close for label services file     */



    /* Close images.  The array of file descriptors
    are terminated with a NULL so loop until the
    NULL is found closing each image.
    -----------------------------------------------*/
    d_index = 0;
    while ( fdesc[d_index] != 0 )
        {
	status = c_eclose( &fdesc[d_index] );
	if ( status != E_SUCC )
	    return_status = E_FAIL;
	d_index++;
	}
    
    if ( return_status == E_FAIL )
        c_lasmsg(FALSE,"clavr-close","Error closing an image file");
    

    /* Close the minor frame if specified
    -------------------------------------*/
    if ( input_image_type == SINGLE_SCENE )
        {
        /* create a minor frame file name
	---------------------------------*/
	c_lsmknm( input_file, "mnr", filename );
	status = c_lsclos( minorframe_fdesc, filename, &action );
	if ( status != E_SUCC )
	    {
	    c_lasmsg(FALSE,"clavr-close","Error closing minor frame");
	    return_status = E_FAIL;
	    }
	}
    
    return return_status;
    }  /* END closeimages */
