#!/bin/csh
#
# FUNCTION:	cleannsidc.csh
#
# PURPOSE:	Clean-up NSIDC image transfer directory
#		(must be executed from that directory).
#               When NSIDC downloads an image from EDC, they are
#               responsible for deleting the associated header
#               file ('.ahdr').  EDC is responsible for eventually
#               unlinking the image file ('.arch') file - thus
#               making this script necessary.
#
# ALGORITM:	For each file ".arch" file found in the pwd,
#		if a matching ".ahdr" file does not exist in
#		the pwd, unlink the ".arch" file from the silo
#               and delete the ".sum" file (if one exists).
#
# NOTES:        To get around the 'too many files in the directory'
#               problem, the selective num0/num1/num2 matching
#               was implemented.
#
# DIRECTIONS:	1) go into directory to clean up (/sgs7/disk2/nsidc)
#		2) execute script: "csh cleannsidc.csh"
#
#
unalias ls
echo "======= Running cleannsidc.csh on `date` ========" >> cleannsidc.log
#
foreach num0 (0 1 2 3 4 5 6 7 8 9)
   foreach num1 (0 1 2 3 4 5)
      foreach num2 (0 1 2 3 4 5 6 7 8 9)
         foreach archfile (`ls *$num2$num1$num0.arch`)
            set ahdrfile = `echo $archfile | awk 'FS=".arch" {print $1}'`
	    set ahdrfile = "$ahdrfile.ahdr"
            if (! -e $ahdrfile) then
               unlink $archfile
               echo "Unlink $archfile, unmatched $ahdrfile"  >> cleannsidc.log
               set sumfile = `echo $archfile | awk 'FS=".arch" {print $1}'`
	       set sumfile = "$sumfile.sum"
               if ( -e $sumfile) then
    	          rm -f $sumfile
                  echo "Delete $sumfile, unmatched $ahdrfile"  >> cleannsidc.log
	       endif
            endif
         end
      end
   end
end

