#!/bin/csh
#
# FUNCTION:	cleannsidcsum.csh
#
# PURPOSE:	Clean-up NSIDC image transfer directory
#		(must be executed from that directory).
#               When NSIDC downloads an image from EDC, they are
#               responsible for deleting the associated header
#               file ('.ahdr').  EDC is responsible for eventually
#               unlinking the image file ('.arch') file.  This
#               script will clean up the '.sum' files that don't
#               have an associated '.ahdr' or '.arch' file.
#
# ALGORITM:	For each file ".sum" file found in the pwd,
#		if a matching ".ahdr" file does not exist in
#		the pwd, and a matching ".arch" file does not
#               exist in the pwd, delete the ".sum" file.
#
# NOTES:        Add 'badflag' logic because the first time thru it returns a
#               string of "=" (don't know why).
#               To get around the 'too many files in the directory'
#               problem, the selective num0/num1/num2 matching
#               was implemented.
#
# DIRECTIONS:	1) go into directory to clean up (/sgs7/disk2/nsidc)
#		2) execute script: "csh cleannsidcsum.csh"
#
#
unalias ls
echo "======= Running cleannsidcsum.csh on `date` ========" >> cleannsidcsum.log
#
foreach num0 (0 1 2 3 4 5 6 7 8 9)
   foreach num1 (0 1 2 3 4 5)
      foreach num2 (0 1 2 3 4 5 6 7 8 9)
	 set badflag = 1
         foreach sumfile (`ls *$num2$num1$num0.sum`)
            if ($badflag != 1) then
               set ahdrfile = `echo $sumfile | awk 'FS=".sum" {print $1}'`
	       set ahdrfile = "$ahdrfile.ahdr"
               if (! -e $ahdrfile) then
                  set archfile = `echo $sumfile | awk 'FS=".sum" {print $1}'`
	          set archfile = "$archfile.arch"
                  if (! -e $archfile) then
                     rm -f $sumfile
                     echo "Removed $sumfile, unmatched $ahdrfile and $archfile"  >> cleannsidcsum.log
		  endif
               endif
	    else
	       set badflag = 0
	    endif
         end
      end
   end
end

