/*******************************************************************************
NAME:				COMPATM


PURPOSE:    Generate a water vapor image, ozone image, quality control image
	    all in the same space and same size as the input image.

PROGRAM HISTORY:

VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T.Baltzer	  11/99 	  CSC		Original Development
  1.1      D.Lloyd        10/00           CSC           Added a new file that
  							updates the output
							DDR's to be the same
							as the input DDR.
 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:			ADAPS

******************************************************************************/


#include <stdio.h>
#include <string.h>
#include "las.h"	
#include "util.h"	
#include "compatm.h"	


main()
{

int i;				/* Counter				*/
long msgtim;			/* Elapsed time between display msg	*/
long num_in_img = NUM_IN_IMG;	/* Number of input images		*/
int nwater;			/* Number of water bands		*/
int scenelist[MAXSCENE];	/* Scene listing			*/
long std_out;			/* LUN to standard output		*/
long two = 2;			/* Constant value of two		*/

char datelist[MAXDCOMP][7];	/* Date listing				*/
char hosin[MAXIMG][CMLEN];	/* Host names of input images		*/
char hosout[2][CMLEN];		/* Host name of output image	        */
char minmax[4];			/* Min max flag				*/
char scale[7];			/* Scaling technique, US or GLOBAL	*/
char taeout[2][CMLEN];		/* TAE name of output image		*/

float ozonebuf[2][NLINES][NSAMPS];		/* Ozone image buffers	*/
float qcbuf[MAXDCOMP][NLINES][NSAMPS];		/* QC band buffers	*/
float waterbuf[MAXDCOMP][NLINES][NSAMPS];	/* Water vapor buffers	*/

struct MYFD  myfd[MAXIMG+1];	/* File descriptors to the images	*/
struct PARBLK vblock;		/* TAE parameter block			*/


/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);


/* Get parameters and globals
-----------------------------*/
getpar(&vblock,myfd,hosin,hosout,waterbuf,qcbuf,&nwater,ozonebuf,
       scenelist,datelist,&msgtim,minmax,scale);


/* Open the images and then call the main processing routine
--------------------------------------------------------------*/
pre_process(&vblock,myfd,hosin,hosout,waterbuf,qcbuf,nwater,ozonebuf,
	 scenelist,datelist,msgtim,minmax,scale);


/*  Update history files 
-------------------------*/
for (i = 0; i < 2; i++)
    if (c_hstout(&vblock,hosout[i],hosin,&num_in_img," ") != E_SUCC)
	c_lasmsg(FALSE,"compatm-warn","Nonfatal error encountered");


/* Output completion message 
------------------------------*/
for (i = 0; i < 2; i++)
    strcpy(taeout[i], hosout[i]);
c_complt("COMPATM",taeout,&two);


/* Put this return here to make the compiler happy
--------------------------------------------------*/
return(E_SUCC);

}
