
/* Include file for the COMPATM module	*/

#ifndef COMPATM_H
#define COMPATM_H

#define NUM_IN_IMG  4   /* Number of input images		 */
#define NUM_OUT_BND 3   /* Number of output bands		 */
#define MAXDCOMP   20   /* Maximum number of days in a composite */
#define MAXSCENE  300	/* Maximum number of scene reference in date log    */

#define NLINES   180    /* Number of lines   in an image in geometric space */
#define NSAMPS   360    /* Number of samples in an image in geometric space */

#define DATE_BAND  1    /* Band number for composite image file descriptor  */
#define SOLZ_BAND  2    /* Band number for composite image file descriptor  */
#define LINE_BAND  3    /* Band number for composite image file descriptor  */
#define SAMP_BAND  4    /* Band number for composite image file descriptor  */
#define OUT_BAND   5    /* Band number for composite image file descriptor  */
#define OUTQC_BAND 6    /* Band number for composite image file descriptor  */


/* Structure containing the necessary information for the input/ouput images,
   a subset of the structure required for the imageio supports 
------------------------------------------------------------------------------*/
struct MYFD
    {
    int nlines;
    int nsamples;
    int nbands;
    long bands[MAXBND+1];
    };


/* Prototypes */

void get_composite_in
(
    struct PARBLK *vblock,      /* IN:  Pointer to TAE parameter block       */
    int which_band,             /* IN:  Which band has been specified        */
    char parmname[],            /* IN:  Parameter name of image              */
    int numbnd,                 /* IN:  Number of bands in parmname          */
    struct MYFD myfd[],         /* OUT: File information                     */
    char hostname[]             /* OUT: Host file name                       */
);

void get_composite_out
(
    struct PARBLK *vblock,      /* IN:  Pointer to TAE parameter block       */
    int which_band,             /* IN:  Which band has been specified        */
    char parmname[],            /* IN:  Parameter name of image              */
    int numbnd,                 /* IN:  Number of bands in parmname          */
    int nlines,                 /* IN:  Number of lines in image             */
    int nsamples,               /* IN:  Number of samples in image           */
    struct MYFD myfd[],         /* OUT: File information                     */
    char hostname[]             /* OUT: Host file name                       */
);

void get_composite_info
(
    struct PARBLK *vblock,      /* IN:  pointer to TAE parameter block        */
    struct MYFD myfd[],         /* OUT: File information                      */
    char hosin[MAXIMG][CMLEN],  /* OUT: Host names of input images            */
    char hosout[2][CMLEN]       /* OUT: Host name of output image             */
);

void get_geographic_images
(
    struct PARBLK *vblock,                      /* IN:  TAE parameter block   */
    float waterbuf[MAXDCOMP][NLINES][NSAMPS],   /* OUT: Water vapor buffers   */
    float ozonebuf[MAXDCOMP][NLINES][NSAMPS],   /* OUT: Ozone image buffers   */
    float qcbuf[MAXDCOMP][NLINES][NSAMPS],      /* OUT: QC band buffers       */
    int *nwater                                 /* OUT: Number of water bands */
);

int get_geographic_buffers
(
    char imgnames[MAXDCOMP][CMLEN],     /* IN:  Input geographic image name   */
    int nimages,                        /* IN:  Number of images              */
    long *band,                         /* IN:  image band number             */
    float imgbufs[MAXDCOMP][NLINES][NSAMPS] /* OUT: Output image buffers      */
);

void getpar
(
    struct PARBLK *vblock,              /* IN: pointer to TAE parameter block */
    struct MYFD myfd[],                 /* OUT: File information              */
    char hosin[MAXIMG][CMLEN],          /* OUT: Host names of input images    */
    char hosout[2][CMLEN],              /* OUT: Host name of output image     */
    float waterbuf[MAXDCOMP][NLINES][NSAMPS],   /* OUT: Water vapor buffers   */
    float qcbuf[MAXDCOMP][NLINES][NSAMPS],      /* OUT: QC band buffers       */
    int *nwater,                                /* OUT: Number of water bands */
    float ozonebuf[MAXDCOMP][NLINES][NSAMPS],   /* OUT: Ozone image buffers   */
    int scenelist[MAXSCENE],                    /* OUT: Scene listing         */
    char datelist[MAXDCOMP][7],                 /* OUT: Date listing          */
    long *msgtim,                       /* OUT: time between display messages */
    char mmflag[],                      /* OUT: min max flag buffer           */
    char scale[]                        /* OUT: Scaling technique, US GLOBAL  */
);

void pre_process
(
    struct PARBLK *vblock,      /* IN:  TAE parameter block                  */
    struct MYFD  myfd[],        /* IN: File descriptors to the images        */
    char hosin[][CMLEN],        /* IN: host names of input images            */
    char hosout[][CMLEN],       /* IN: host name of output image             */
    float waterbuf[MAXDCOMP][NLINES][NSAMPS],   /* IN: Water vapor buffers   */
    float qcbuf[MAXDCOMP][NLINES][NSAMPS],      /* IN: QC band buffers       */
    int nwater,                                 /* IN: Number of water bands */
    float ozonebuf[MAXDCOMP][NLINES][NSAMPS],   /* IN: Ozone image buffers   */
    int scenelist[MAXSCENE],                    /* IN: Scene listing         */
    char datelist[MAXDCOMP][7],                 /* IN: Date listing          */
    long msgtim,                                /* IN: Time between messages */
    char minmax[4],                             /* IN: Min max flags         */
    char *scale                                 /* IN: Scaling technique     */
);

void process
(
    struct PARBLK *vblock,              /* IN: TAE parameter block           */
    char hosin[][CMLEN],                /* IN: host names of input images    */
    char hosout[][CMLEN],               /* IN: host name of output image     */
    struct MYFD myfd[],                 /* IN: File descriptors to the images */
    struct GDESC **gdesc,               /* IN: group descriptor for image I/O */
    long offset1[MAXIMG*MAXBND],        /* IN: offsets for input bands       */
    long offset2[MAXBND],               /* IN: offsets for output bands      */
    char *thebuf,                       /* IN: image I/O buffer              */
    float waterbuf[MAXDCOMP][NLINES][NSAMPS],   /* IN: Water vapor buffers   */
    float qcbuf[MAXDCOMP][NLINES][NSAMPS],      /* IN: QC band buffers       */
    float ozonebuf[MAXDCOMP][NLINES][NSAMPS],   /* IN: Ozone image buffers   */
    int scenelist[MAXSCENE],                    /* IN: Scene listing         */
    char datelist[MAXDCOMP][7],                 /* IN: Date listing          */
    int nwater,                         /* IN: Number of water bands         */
    long msgtim,                        /* IN: time between display messages */
    char *minmax,                       /* IN: min max flag                  */
    char *scale                         /* IN: Scaling technique US, GLOBAL  */
);

void update_bdr
(
    struct PARBLK *vblock,      /* IN: TAE parameter block                   */
    char hosout[][CMLEN]        /* IN: host name of output image             */
);

void update_ddr
(
    struct PARBLK *vblock,      /* IN: TAE parameter block                   */
    char hosin[][CMLEN],        /* IN: host name of input image              */
    char hosout[][CMLEN]        /* IN: host name of output image             */
);

#endif

