/*******************************************************************************
NAME:		GET_COMPOSITE_IN

PURPOSE:	Get the user specified composite image names

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T.Baltzer      11/99           CSC           Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:                        ADAPS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"	
#include "util.h"	
#include "adaps.h"	
#include "compatm.h"


void get_composite_in
(
    struct PARBLK *vblock,	/* IN:  Pointer to TAE parameter block	     */
    int which_band,		/* IN:  Which band has been specified	     */
    char parmname[],		/* IN:  Parameter name of image		     */
    int numbnd,			/* IN:  Number of bands in parmname	     */
    struct MYFD myfd[],		/* OUT: File information		     */
    char hostname[]		/* OUT: Host file name			     */
)

{

char hosin[NUM_IN_IMG][CMLEN],	/* Host image file name			     */
     taein[NUM_IN_IMG][CMLEN];	/* TAE image file names			     */

long	bands[1][MAXBND+1],	/* Bands specifications			     */
	element		=  0,	/* parameter element number		     */
	imglim		=  1,	/* maximum number of images to be processed  */
	nbands[1],		/* Number of bands in angle image            */
	nmrimg,			/* Number of input images                    */
	qflag[1][2],		/* flag indicating if subwindows specified   */
	same,			/* same window flag			     */
	totbnd,			/* Total number of input image bands	     */
	window[NUM_IN_IMG][4];	/* Input image window specification          */

int	band_index;		/* Band index (zero relative)		     */


/* Compute the band index (zero relative) for the band being processed
-----------------------------------------------------------------------*/
band_index = which_band - 1;


/* Retrieve and parse image specification 
--------------------------------------------*/
if (c_getin(vblock, parmname, &element, taein[band_index], hosin[band_index],
    &nmrimg, window, bands, nbands, &totbnd, &imglim, &same, qflag) != E_SUCC)
    c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");


/* Check for the valid number of input bands.
--------------------------------------------------*/
if (totbnd != numbnd)
    {
    c_lasmsg(FALSE,"compatm-badbnd","Invalid number of input bands specified");
    c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");
    }

/* Check for the valid number of input images
-------------------------------------------------*/
if (nmrimg != 1)
    {
    c_lasmsg(FALSE,"compatm-badimg","Invalid number of input images specified");
    c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");
    }

/* Load fields within the file descriptor structure
-----------------------------------------------------------*/
myfd[band_index].nlines = window[0][NL];
myfd[band_index].nsamples = window[0][NS];
myfd[band_index].nbands = nbands[0];
myfd[band_index].bands[0] = bands[0][0];

strcpy(hostname,hosin[band_index]);

}
