/*******************************************************************************
NAME:		GET_COMPOSITE_INFO

PURPOSE:	Get the information for the user specified images in
		composite space.

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T.Baltzer      11/99           CSC           Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:                        ADAPS


*******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "adaps.h"	
#include "compatm.h"


void get_composite_info
(
    struct PARBLK *vblock,	/* IN:  pointer to TAE parameter block        */
    struct MYFD myfd[],		/* OUT: File information		      */
    char hosin[MAXIMG][CMLEN],	/* OUT: Host names of input images	      */
    char hosout[2][CMLEN]	/* OUT: Host name of output image	      */
)

{

/* Retrieve and parse date band image specification 
---------------------------------------------------*/
get_composite_in(vblock,DATE_BAND,"DATEBAND",1,myfd,hosin[0]);

/* Retrieve and parse solar zenith band image specification 
-----------------------------------------------------------*/
get_composite_in(vblock,SOLZ_BAND,"SOLZEN",1,myfd,hosin[1]);

/* Retrieve and parse the 'line' image specification 
-----------------------------------------------------*/
get_composite_in(vblock,LINE_BAND,"LINES",1,myfd,hosin[2]);

/* Retrieve and parse the 'sample' image specification 
-------------------------------------------------------*/
get_composite_in(vblock,SAMP_BAND,"SAMPLES",1,myfd,hosin[3]);

/* Retrieve and parse the output image specification 
-------------------------------------------------------*/
get_composite_out(vblock,OUT_BAND,"OUT",2,myfd[0].nlines,myfd[0].nsamples,
	 myfd,hosout[0]);

/* Retrieve and parse the output QC image specification 
-------------------------------------------------------*/
get_composite_out(vblock,OUTQC_BAND,"QCOUT",1,myfd[0].nlines,myfd[0].nsamples,
	 myfd,hosout[1]);

}
