/**************************************************************************

FUNCTION:	GET_GEOGRAPHIC_BUFFERS

PURPOSE:	Open and read the geographic input images into buffers.

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T.Baltzer      11/99           CSC           Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:                        ADAPS

***************************************************************************/
#include "las.h"
#include "util.h"
#include "compatm.h"


int get_geographic_buffers
( 
    char imgnames[MAXDCOMP][CMLEN],	/* IN:  Input geographic image name   */
    int nimages,			/* IN:  Number of images	      */
    long *band,				/* IN:  image band number	      */
    float imgbufs[MAXDCOMP][NLINES][NSAMPS] /* OUT: Output image buffers      */
)

{

struct	FDESC	*fdesc;		/* image I/O file descriptor		*/

long	dtype = EREAL,		/* image data type			*/
	fill = NOFIL,		/* fill technique specification flag	*/
	nextflg,		/* call type: CREAT INIT CURNT NEXT	*/
	nlav,			/* number of valid lines in buffer	*/
	nlb = 0,		/* number of image boundary lines	*/
	nlbuf = NLINES,		/* total number of lines in buffer	*/
	nli = NLINES,		/* number of lines in image window	*/
	nsav,			/* number of valid samples in buffer	*/
	nsb = 0,		/* number of image boundary samples	*/
	nsbuf = NSAMPS,		/* total number of samples in buffer	*/
	nsi = NSAMPS,		/* number of samples in image window	*/
	slcb = 1,		/* starting line of data within image	*/
	sli = 1,		/* starting line of image window	*/
	sscb = 1,		/* starting sample of data within image	*/
	ssi = 1,		/* starting sample of image window	*/
	status;			/* Function return status		*/

int	i;			/* loop control variable		*/

unsigned char	*buffer[MAXDCOMP];/* pointer to input buffer		*/


/* Read the geographic image into buffer space
   (the call to get_buf opens the image).
   Close the geographic image.
------------------------------------------*/
for (i = 0; i < nimages; i++)
    {
    buffer[i] = (unsigned char *)imgbufs[i];
    nextflg = INIT;
    status = get_buf( buffer[i], &nlbuf, &nsbuf, &nlav, &nsav, &sli, &ssi, &nli,
		  &nsi, &nlb, &nsb, &slcb, &sscb, &nextflg, band, &fill,
		  &fdesc, imgnames[i], &dtype);
    if ( status != E_SUCC )
	{
	c_lasmsg(FALSE,"compatm-gtbuf","Error reading image %s", imgnames[i]);
	c_eclose( &fdesc );
	return E_FAIL;
	}

    /* Close the geographic image
    ------------------------------*/
    status = c_eclose( &fdesc );
    if ( status != E_SUCC )
	c_lasmsg(FALSE,"compatm-clos","Error closing file %s", imgnames[i]);

    }

return(E_SUCC);
} 
