/*******************************************************************************
NAME:		GETPAR

PURPOSE:	Get the user specified parameters

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T.Baltzer      11/99           CSC           Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:                        ADAPS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"	
#include "adaps.h"	
#include "compatm.h"


void getpar
(
    struct PARBLK *vblock,		/* IN: pointer to TAE parameter block */
    struct MYFD myfd[],			/* OUT: File information 	      */
    char hosin[MAXIMG][CMLEN],		/* OUT: Host names of input images    */
    char hosout[2][CMLEN],		/* OUT: Host name of output image     */
    float waterbuf[MAXDCOMP][NLINES][NSAMPS],	/* OUT: Water vapor buffers   */
    float qcbuf[MAXDCOMP][NLINES][NSAMPS],	/* OUT: QC band buffers       */
    int *nwater,				/* OUT: Number of water bands */
    float ozonebuf[MAXDCOMP][NLINES][NSAMPS],	/* OUT: Ozone image buffers   */
    int	scenelist[MAXSCENE],			/* OUT: Scene listing	      */
    char datelist[MAXDCOMP][7],			/* OUT: Date listing	      */
    long *msgtim,			/* OUT: time between display messages */
    char mmflag[],			/* OUT: min max flag buffer           */
    char scale[]			/* OUT: Scaling technique, US GLOBAL  */
)

{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

int i;				/* Counter */


/* Get the composite images
-----------------------------*/
get_composite_info(vblock, myfd, hosin, hosout);


/* Get the images in geographic space
-----------------------------------------------------------*/
get_geographic_images(vblock, waterbuf, ozonebuf, qcbuf, nwater );


/* Get the 'scene' list and 'date' list
---------------------------------------*/
val = p_find(vblock,"SCENELIST");
for (i = 0; i < val->v_count; i++)
   scenelist[i] = IVAL(*val,i);
val = p_find(vblock,"DATELIST");
for (i = 0; i < val->v_count; i++)
   strcpy(datelist[i],SVAL(*val,i));


/* Retrieve the scaling technique used
---------------------------------------*/
val = p_find(vblock,"SCALE");
strcpy(scale,SVAL(*val,0));


/* Retrieve LAS globals
-----------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);

val = p_find(vblock,"$minmax");
strcpy(mmflag,SVAL(*val,0));

}
