/******************************************************************************

NAME:   			PRE_PROCESS

PURPOSE:    Open, group the images; get ready to process.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T.Baltzer	   11/99	EDC/CSB		original development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:			ADAPS

******************************************************************************/

#include "las.h"
#include "util.h"
#include "compatm.h"

void pre_process
(
    struct PARBLK *vblock,	/* IN:  TAE parameter block		     */
    struct MYFD  myfd[],	/* IN: File descriptors to the images	     */
    char hosin[][CMLEN],	/* IN: host names of input images	     */
    char hosout[][CMLEN],	/* IN: host name of output image	     */
    float waterbuf[MAXDCOMP][NLINES][NSAMPS],	/* IN: Water vapor buffers   */
    float qcbuf[MAXDCOMP][NLINES][NSAMPS],	/* IN: QC band buffers	     */
    int nwater,					/* IN: Number of water bands */
    float ozonebuf[MAXDCOMP][NLINES][NSAMPS],	/* IN: Ozone image buffers   */
    int scenelist[MAXSCENE],			/* IN: Scene listing	     */
    char datelist[MAXDCOMP][7],			/* IN: Date listing	     */
    long msgtim,				/* IN: Time between messages */
    char minmax[4],				/* IN: Min max flags	     */
    char *scale					/* IN: Scaling technique     */
)

{

long	access,			/* image I/O access type                     */
	bufsiz,			/* size of image I/O buffer                  */
	dtype = EREAL,		/* input data type			     */
	nl,			/* number of image lines                     */
	ns,			/* number of image samples                   */
	offset1[MAXIMG*MAXBND],	/* offsets into thebuf for input bands	     */
	offset2[MAXBND],	/* offsets into "thebuf" for output bands    */
	opt[3],			/* option array				     */
	outbands[1],		/* Output number of bands		     */
	sl,			/* starting image line                       */
	ss,			/* starting image sample                     */
	status,			/* function return status code               */
	two = 2,		/* two					     */
	zero = 0;		/* value of zero                             */

int	band,			/* loop index counting the bands             */
	image,			/* loop index counting input images          */
	noutimg = 2,		/* Number of output images		     */
	save_nl,		/* Retain previous number of lines in image  */
	save_ns;		/* REtain previous number of samples in img  */

char    *thebuf;		/* image I/O buffer                          */

struct	GDESC *gdesc;		/* group descriptor used for image I/O       */
struct	FDESC *fdesc[MAXIMG+1];	/* image file descriptors used for image I/O */


/* Open input images
----------------------*/
access = IREAD;
sl = 1;  /* no windowing allowed */
ss = 1;  /* no windowing allowed */
for (band=0,image=0;image<NUM_IN_IMG;image++)
   {
   nl = 0;  /* get number of lines from the ddr */
   ns = 0;  /* get number of samples from the ddr */
   status = c_eopens(&fdesc[image],hosin[image],myfd[image].bands,
		     &sl,&ss,&nl,&ns,&offset1[band],&access,&dtype,&zero,&zero);
   if (status != E_SUCC)
      c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");
   if (image == 0)
      {
      save_nl = nl;
      save_ns = ns;
      }
   else
      {
      if ((nl != save_nl) || (ns != save_ns))
	 {
	 c_lasmsg(FALSE,"compatm-imgs","Invalid sized input image specified");
         c_lasmsg(TRUE,"compatm-fatal","Fatal error encountered");
	 }
      }
   band += myfd[image].nbands;
   }


/* Open output images
-----------------------*/

access = IWRITE;	/* Write access for output images */
opt[1] = COMPRES;	/* Always compress the output images */
opt[2] = 0;		/* Standard option terminator	*/

for (band=0, image=NUM_IN_IMG; image < (noutimg+NUM_IN_IMG); image++)
   {
   if (band == 0)
      {
      opt[0] = EREAL;	/* water/ozone image data type */
      outbands[0] = 2;
      }
   else
      {
      opt[0] = EBYTE;	/* qc image data type */
      outbands[0] = 1;
      }

   status = c_eopens(&fdesc[image],hosout[image-NUM_IN_IMG],outbands,
		     &sl,&ss,&nl,&ns,&offset2[band],&access,&dtype,&zero,opt);
   if (status != E_SUCC)
      c_clean(vblock,"compatm",hosout,&two);
   band += myfd[image].nbands;
   }

fdesc[image] = 0;


/* Allocate I/O buffer
----------------------*/
if ((thebuf = c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_lasmsg(FALSE,"compatm-alloc","Error allocating dynamic memory");
   c_clean(vblock,"compatm",hosout,&two);
   }


/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,(unsigned char *)thebuf,&bufsiz) != E_SUCC)
   c_clean(vblock,"compatm",hosout,&two);


/* Finally do the processing here!
----------------------------------*/
process(vblock,hosin,hosout,myfd,&gdesc,offset1,offset2,thebuf,waterbuf,qcbuf,
	ozonebuf,scenelist,datelist,nwater,msgtim,minmax,scale);


/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_lasmsg(FALSE,"compatm-warn","Nonfatal error encountered");


/* Free buffer
--------------*/
free(thebuf);


}
