/******************************************************************************
NAME:   			UPDATE_BDR 

PURPOSE:    Update the band DDR's

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T.Baltzer      11/99           CSC           Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:                        ADAPS

******************************************************************************/

#include <string.h>
#include "las.h"
#include "util.h"

void update_bdr
(
    struct PARBLK *vblock,	/* IN: TAE parameter block		     */
    char hosout[][CMLEN] 	/* IN: host name of output image	     */
)

{

struct BDDR bddr;		/* Band data descriptor record		     */
struct BDDR bddr2;		/* Band data descriptor record		     */
struct DDR ddr;			/* Data descriptor record		     */

long    one = 1,		/* constant 				     */
	two = 2;		/* constant				     */


/* Get the DDR info for the water/ozone image
----------------------------------------------*/
if ( c_getddr( hosout[0], &ddr ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);

/* Get the BDDR info for the water/ozone image,
   and add the data source field.
----------------------------------------------*/
if ( c_getbdr( hosout[0], &bddr, &one ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);
strcpy(bddr.source,"Full water vapor image");

if ( c_getbdr( hosout[0], &bddr2, &two ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);
strcpy(bddr2.source,"Ozone image");

/* Save the DDR info for the water/ozone image
----------------------------------------------*/
if ( c_putddr( hosout[0], &ddr ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);

/* Save the BDDR info for the water/ozone image
-----------------------------------------------*/
if ( c_putbdr( hosout[0], &bddr ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);

if ( c_putbdr( hosout[0], &bddr2 ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);


/* Get the DDR info for the QC image
-------------------------------------------*/
if ( c_getddr( hosout[1], &ddr ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);

/* Get the BDDR info for the QC image
   and add the data source field.
-------------------------------------------*/
if ( c_getbdr( hosout[1], &bddr, &one ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);
strcpy(bddr.source,"Quality control band");

/* Save the DDR info for the QC image
-------------------------------------------*/
if ( c_putddr( hosout[1], &ddr ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);

/* Save the BDDR info for the QC image
-------------------------------------------*/
if ( c_putbdr( hosout[1], &bddr ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);

}
