/******************************************************************************
NAME:   			UPDATE_DDR 

PURPOSE:    Update the output DDR's

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         REASON
-------    ----------      ----         ------
  1.0      D.Lloyd        10/00         Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   COMPATM must be run under TAE.

PROJECT:                        ADAPS

******************************************************************************/

#include <string.h>
#include "las.h"
#include "util.h"

void update_ddr
(
    struct PARBLK *vblock,	/* IN: TAE parameter block		     */
    char hosin[][CMLEN],	/* IN: host name of input image	             */
    char hosout[][CMLEN] 	/* IN: host name of output image	     */
)

{
long	i;			/* loop control				     */
int	j;			/* loop control				     */

struct DDR inddr;		/* Data descriptor record		     */
struct DDR outddr;		/* Data descriptor record		     */
struct BDDR outbandddr[2];	/* Data descriptor record		     */

long    one = 1;		/* constant 				     */
long    two = 2;		/* constant 				     */


/* Get the DDR of the first input image
---------------------------------------*/
if ( c_getddr( hosin[0], &inddr ) != E_SUCC )
   c_clean(vblock,"compatm",hosout,&two);

/* Update both output DDR's with the input DDR information
----------------------------------------------------------*/
for ( j = 0; j < 2; j++ )
    {
    /* Get the DDR info for the output image
    ----------------------------------------*/
    if ( c_getddr( hosout[j], &outddr ) != E_SUCC )
       c_clean(vblock,"compatm",hosout,&two);

    /* Copy the information
    -----------------------*/
    outddr.master_line = inddr.master_line;
    outddr.master_sample = inddr.master_sample;
    /* copy valid flags */
    for ( i = 0; i < 8; i++ )
        outddr.valid[i] = inddr.valid[i];
    /* copy projection and datum */
    outddr.proj_code = inddr.proj_code;
    outddr.zone_code = inddr.zone_code;
    outddr.datum_code = inddr.datum_code;
    strcpy( outddr.proj_units, inddr.proj_units );
    /* copy the projection coefficients */
    for ( i = 0; i < 15; i++ )
        outddr.proj_coef[i] = inddr.proj_coef[i];
    /* copy corner coords */
    outddr.upleft[0] = inddr.upleft[0];
    outddr.upleft[1] = inddr.upleft[1];
    outddr.loleft[0] = inddr.loleft[0];
    outddr.loleft[1] = inddr.loleft[1];
    outddr.upright[0] = inddr.upright[0];
    outddr.upright[1] = inddr.upright[1];
    outddr.loright[0] = inddr.loright[0];
    outddr.loright[1] = inddr.loright[1];
    /* copy the projection distance and increment */
    outddr.pdist_y = inddr.pdist_y;
    outddr.pdist_x = inddr.pdist_x;
    outddr.line_inc = inddr.line_inc;
    outddr.sample_inc = inddr.sample_inc;

    /* Get the band DDR's of the output image
    -----------------------------------------*/
    for ( i = 1; i <= outddr.nbands; i++ )
	if ( c_getbdr( hosout[j], &outbandddr[i-1], &i ) != E_SUCC )
	    c_clean(vblock,"compatm",hosout,&two);

    /* Save the DDR info for the output image
    -----------------------------------------*/
    if ( c_putddr( hosout[j], &outddr ) != E_SUCC )
       c_clean(vblock,"compatm",hosout,&two);

    /* Rewrite the band DDR's
    -------------------------*/
    for ( i = 0; i < outddr.nbands; i++ )
        if ( c_putbdr( hosout[j], &outbandddr[i] ) != E_SUCC )
	    c_clean(vblock,"compatm",hosout,&two);
    }

}
