/*******************************************************************************
NAME:		FPROCPROJ

PURPOSE:	(Fast PROCess Projected) Process a line segment from the DCW
		Quickly process the segment (don't bother to 
		check for duplicate segments)
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dcw.h"
#include "adaps.h"

FUNCTION fprocproj(dcw_line,ddr,window,pixval,chipbuf,
		   geo_code,geo_zone,geo_units,ddr_units,geo_parms)

struct DCW_LINE *dcw_line;      /* current line                              */
struct DDR *ddr;	/* image DDR information			     */

float	*chipbuf;	/* buffer of image data				     */

long	window[],	/* window to process				     */
	pixval,		/* pixel intensity value for rasterized pixel        */
	*geo_code,	/* geographic projection code			     */
	*geo_zone,	/* geographic projection zone			     */
	*geo_units,	/* geographic projection units			     */
	*ddr_units;	/* ddr projection units			     	     */

double  geo_parms[];	/* geographic projection parameters		     */
{
double	lat,		/* temporary latitude in a double		     */
	lon;		/* temporary longitude in a double		     */

long	nonfatal = 1;	/* nonfatal error code				     */
long	i;		/* looping counter				     */
long	l1, s1,		/* line/sample of first endpoint in line segment     */
	l2, s2;		/* line/sample of second endpoint in line segment    */

/*
  Set floats into doubles
*/
lon = (double)dcw_line->pts[0];
lat = (double)dcw_line->pts[1];

/*
  Calculate the line/sample for the starting point of the line segments
*/
if (cal_lin_smp(ddr,geo_code,geo_units,ddr_units,geo_zone,geo_parms,
    &lat,&lon,&l2,&s2) != E_SUCC)
    {
    c_errmsg("Error calculating line/samp","dcwchips-calc",&nonfatal);
    return(E_FAIL);
    }

/*
  Process all the remaining points of the line segment
*/
for (i = 1; i < dcw_line->npts; i++)
    {
    /*
      Move the end of the line segment to be the beginning of the next
      line segment
    */
    l1 = l2;
    s1 = s2;

    /*
      Convert the latitude/longitude of the next point to a line/sample
    */
    lon = dcw_line->pts[2*i];
    lat = dcw_line->pts[2*i+1];
    if (cal_lin_smp(ddr,geo_code,geo_units,ddr_units,geo_zone,geo_parms,
	&lat,&lon,&l2,&s2) != E_SUCC)
	{
	c_errmsg("Error calculating line/samp","dcwchips-calc",&nonfatal);
	return(E_FAIL);
	}

    /*
      Validate the line segment is at least partially in the image
    */
    if (((l1 < window[SL]) && (l2 < window[SL])) ||
	((l1 > (window[SL] + window[NL]-1)) &&
	(l2 > (window[SL] + window[NL]-1))))
	continue;

    /*
      Validate the line segment is at least partially in the image
    */
    if (((s1 < window[SS]) && (s2 < window[SS])) ||
	((s1 > (window[SS] + window[NS] - 1)) &&
	(s2 > (window[SS] + window[NS]-1))))
	continue;

    /*
      Determine the samples to be filled in for this line segment
    */
    scani(chipbuf,l1,s1,l2,s2,window,pixval);
    }

return(E_SUCC);
}
