/*******************************************************************************
NAME:		FPROCSAT

PURPOSE:	(Fast PROCess Satellite) Process a line segment from the DCW
                Quickly process the segment (don't bother to
                check for duplicate segments)

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dcw.h"
#include "adaps.h"

FUNCTION fprocsat(dcw_line,ddr,model,elev,window,lineinc,sampinc,
		  pixval,chipbuf)

struct DCW_LINE	*dcw_line;	/* current line				     */

long	window[],	/* window to process				     */
	pixval;		/* pixel intensity value for rasterized pixel        */

float	*chipbuf;	/* chip pixel values				     */

double	lineinc,	/* subsampling factor of lines			     */
	sampinc,	/* subsampling factor of samples		     */
	*elev;		/* elevation of chip			  	     */

struct DDR     *ddr;	/* DDR information for the image		     */
struct MODEL   *model;	/* model information for AVHRR satellite data        */
{
long	i; 		/* looping counters				     */
long	l1, s1,		/* line/Sample of first endpoint in line segment     */
	l2, s2;		/* line/Sample of second endpoint in line segment    */

float	fline,		/* float value of line at given lat/long	     */
	fsamp;		/* float value of sample at given lat/long	     */

double	lat,		/* temporary latitude in a double		     */
	lon;		/* temporary longitude in a double		     */

/*
  Calculate the line/sample for the starting point of the line segment
  c_ll2ls returns line/samp relative to the original image
*/
lon = dcw_line->pts[0] * RAD_PER_DEG;
lat = dcw_line->pts[1] * RAD_PER_DEG;
c_ll2ls(&lat,&lon,elev,model,&fline,&fsamp,NULL);

/*
  Correct for windowed and/or subsampled image 
*/
l2 = (long)((fline - ddr->master_line) / lineinc + 0.5);
s2 = (long)((fsamp - ddr->master_sample) / sampinc + 0.5);

/*
  Process all the remaining points of the line segment
*/
for (i = 1; i < dcw_line->npts; i++)
    {
    /*
      Move the end of the line segment to be the beginning of the next
      line segment
    */
    l1 = l2;
    s1 = s2;

    /*
      Convert the latitude/longitude of the next point to a line/sample
    */
    lon = dcw_line->pts[2*i] * RAD_PER_DEG;
    lat = dcw_line->pts[2*i+1] * RAD_PER_DEG;
    c_ll2ls(&lat,&lon,elev,model,&fline,&fsamp,NULL);
    l2 = (long)((fline - ddr->master_line) / lineinc + 0.5);
    s2 = (long)((fsamp - ddr->master_sample) / sampinc + 0.5);

    /*
      Validate the line segment is at least partially in the image
    */
    if (((l1 < window[SL]) && (l2 < window[SL])) ||
       ((l1 > (window[SL] + window[NL] - 1)) &&
        (l2 > (window[SL] + window[NL] - 1))))
        continue;

    if (((s1 < window[SS]) && (s2 < window[SS])) ||
       ((s1 > (window[SS] + window[NS] - 1)) &&
        (s2 > (window[SS] + window[NS] - 1))))
        continue;

    /*
      Determine the samples to be filled in for this line segment
    */
    scani(chipbuf,l1,s1,l2,s2,window,pixval);
    }

return(E_SUCC);
}
