/*******************************************************************************
NAME:		GETPAR

PURPOSE:	Retreive the user specified input parameters

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "dcw.h"
#include "typlim.h"

FUNCTION getpar(vblock,ddr,model,cdpath,acount,library,dcw_coverage,taein,
		hosin,tpsfile,mtpfile,bands,nbands,odtype,pixval,
		backgrnd,imgback,proj,msgtim)
		

struct	PARBLK	*vblock;	/* TAE parameter block			     */
struct  DDR	*ddr;		/* DDR information			     */
struct  MODEL  	*model;   	/* AVHRR model                              */
struct	DCW_COVERAGE *dcw_coverage;	/* DCW coverage decriptions	     */

long	*acount,		/* number of areas to rasterize		     */
	bands[][MAXBND+1],	/* bands to be processed		     */
	nbands[],		/* band numbers selected		     */
	*odtype,		/* output data type			     */
	*pixval,		/* pixel intensity value for rasterized pixel*/
	*backgrnd,		/* background value for rasterized image     */
	*imgback,		/* background filled with image data flag    */
	*msgtim;		/* time between processing messages	     */

char	*cdpath,		/* location of dcw			     */
	library[][CMLEN],	/* libraries (areas) to use		     */
	taein[][CMLEN],		/* input TAE image name			     */
	hosin[][CMLEN],		/* input host image name		     */
	*tpsfile,		/* input tps file name			     */
	*mtpfile,		/* outpout mtp file name		     */
	*proj;			/* projection type (satellite or rectified)  */
{
struct VARIABLE	*var;
struct VARIABLE *p_find();

long	access,			/* file access mode			     */
	attribnum,		/* attribute number for each feature	     */
	i,j,			/* looping variables			     */
	len,			/* string length			     */
	nmrimg,			/* number of input image specified	     */
	nonfatal = 1,		/* non-fatal error code			     */
	one = 1,		/* constant of one			     */
	qflag[1][2],		/* flag indicating if subwindows specified   */
	same_window,		/* flag indicating if images hav same window */
	status,			/* status return code			     */
	totattribs = 0,		/* total number of attribute selected  	     */
	totbnd,			/* Total number of bands		     */
	window[1][4],		/* input image window specification	     */
	zero = 0;		/* constant of zero			     */

/*
  Get the input image name defining the area in which the vector
  data is to be rasterized
*/
status = c_getin(vblock,"IN",&zero,taein,hosin,&nmrimg,window,bands,nbands,
		 &totbnd,&one,&same_window,qflag);
if (status != E_SUCC)
   return(E_FAIL);

/*
  Get the DDR information
  and validate projection parameters
*/
status = c_getddr(hosin,ddr);
*odtype = ddr->dtype;

/*
  Get the projection type
*/
var = p_find(vblock,"PROJ");
strcpy(proj,SVAL(*var,0));

if (strcmp(proj,"OTH") == 0)
    {
    if ((ddr->valid[0] == INVAL) || 
	(ddr->valid[DDCCV] == INVAL) ||
        (ddr->valid[DDPDV] == INVAL))
    	{
    	c_errmsg("Invalid projection information in DDR","dcwchips-proj",
                 &nonfatal);
    	return(E_FAIL);
    	}
    }

/*
  Get the input tps file name
*/
access = IREAD;
if (c_getnam(vblock,"INTS",&zero,tpsfile,"tps",&access) != E_SUCC) 
    return(E_FAIL);

/*
  Get the output mtp file name
*/
access = IWRITE;
if (c_getnam(vblock,"OUTMT",&zero,mtpfile,"mtp",&access) != E_SUCC) 
    return(E_FAIL);

/*
  Get the pixel value intensity for the line work
*/
var = p_find(vblock,"PIXVAL");
*pixval = IVAL(*var,0);

/*
  Get the background pixel value
*/
var = p_find(vblock,"BACKGRND");
if (var->v_count == 0)
    {
    *imgback = TRUE;
    *backgrnd = 0;
    if (totbnd > 1)
        {
        c_errmsg("Only a single band image allowed on input",
	     "dcwchips-onebnd",&nonfatal);
    	return(E_FAIL);
        }
    }
else
    {
    *imgback = FALSE;
    *backgrnd = IVAL(*var,0);
    }

/*
  Check that the background and pixel value are in a valid range for the data
  type of the input image.
*/
if (*odtype == EBYTE)
    {
    if ((*backgrnd > MAXUTINY) || (*backgrnd < MINUTINY))
	{
	c_errmsg("BACKGRND is not within the valid range for byte data",
	    "dcwchips-brange",&nonfatal);
    	return(E_FAIL);
	}

    if ((*pixval > MAXUTINY) || (*pixval < MINUTINY))
	{
	c_errmsg("PIXVAL is not within the valid range for byte data",
	    "dcwchips-prange",&nonfatal);
    	return(E_FAIL);
	}
    }
else if (*odtype == EWORD)
    {
    if ((*backgrnd > MAXWORD) || (*backgrnd < MINWORD))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*2 data",
	    "dcwchips-brange",&nonfatal);
    	return(E_FAIL);
	}

    if ((*pixval > MAXWORD) || (*pixval < MINWORD))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*2 data",
	    "dcwchips-prange",&nonfatal);
    	return(E_FAIL);
	}
    }
else if (*odtype == ELONG)
    {
    if ((*backgrnd > MAXLONG) || (*backgrnd < MINLONG))
	{
	c_errmsg("BACKGRND is not within the valid range for integer*4 data",
	    "dcwchips-brange",&nonfatal);
    	return(E_FAIL);
	}

    if ((*pixval > MAXLONG) || (*pixval < MINLONG))
	{
	c_errmsg("PIXVAL is not within the valid range for integer*4 data",
	    "dcwchips-prange",&nonfatal);
    	return(E_FAIL);
	}
    }
else /* (*odtype == EREAL) */
    {
    if ((*backgrnd > MAXREAL) || (*backgrnd < MINREAL))
	{
	c_errmsg("BACKGRND is not within the valid range for real data",
	    "dcwchips-brange",&nonfatal);
    	return(E_FAIL);
	}

    if ((*pixval > MAXREAL) || (*pixval < MINREAL))
	{
	c_errmsg("PIXVAL is not within the valid range for real data",
	    "dcwchips-prange",&nonfatal);
    	return(E_FAIL);
	}
    }

/*
  Get the location of the DCW
*/
var = p_find(vblock, "CDPATH");
strcpy(cdpath, SVAL(*var, 0));

/*
  Get the specified libraries (areas of interest)
*/
var = p_find(vblock, "LIBRARY");
*acount = (*var).v_count;
for (j = 0; j < *acount; j++)
    {
    strcpy(library[j], SVAL(*var,j));
    len = strlen(library[j]);
    c_up2low(library[j],&len);
    }

/*
  Get selected features
*/
i = 0;
while (dcw_coverage[i].desc != NULL)
    {
    var = p_find(vblock, dcw_coverage[i].name);
    attribnum = 1;

    /* Only use library[0], all use same feature sets */
    if (dcw_definecover(cdpath, library[0], &dcw_coverage[i]) == E_SUCC)
	{
	/*
	  Get the selected attributes in each feature
	*/
	if ((*var).v_count == 0)
	      dcw_coverage[i].numvdt = 0;
	for (j = 0; j < (*var).v_count; j++)
	    {
	    dcw_coverage[i].vdt[IVAL(*var,j)-1].selected = TRUE;
	    dcw_coverage[i].vdt[IVAL(*var,j)-1].attindex = attribnum++;
	    totattribs++;
	    }
	}
    i++;
    }

if (totattribs == 0)
    {
    c_errmsg("No DCW features were selected", "dcwchips-feature",&nonfatal);
    return(E_FAIL);
    }


/*
  Retrieve the AVHRR DDR when the input image is in satellite prespective
*/
if (strcmp(proj,"SAT") == 0)
    {
    status = c_getaddr(hosin,model);
    if (status != E_SUCC)
    	return(E_FAIL);
    }

/*
  Get the time between processing messages
*/
var = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*var,0);

return(E_SUCC);
}
