/*******************************************************************************
NAME:   SATSEL

PURPOSE:        Select lines from DCW for placement in the image

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "dcw.h"
#include "adaps.h"

FUNCTION satsel(ddr,model,elev,hosin,bandno,totbnd,hosout,cdpath,library,
		acount,dcw_coverage,pixval,window,backgrnd,imgback,
		msgtim,odtype,minlat,minlon,maxlat,maxlon)


struct  DDR     *ddr;           /* DDR information                           */
struct	MODEL	*model;	   	/* AVHRR model information		     */
struct	DCW_COVERAGE *dcw_coverage;	/* Coverage areas		     */

char	*cdpath,		/* dcw location				     */
	library[][CMLEN],	/* list of libraries (areas)		     */
	hosin[],		/* name of input image			     */
	hosout[];		/* name of output image			     */

long	pixval,			/* pixel value of rasterized line	     */
	bandno[],		/* band number to read			     */
	totbnd,			/* total # bands in input image		     */
	backgrnd,		/* background pixel value		     */
	imgback,		/* flag to put image data in backgrnd	     */
	acount,			/* number of areas			     */
	window[],		/* window to process			     */
	*msgtim,		/* time between processing messages	     */
	odtype;			/* output data type			     */

double	minlat,			/* minimum latitude			     */
	minlon,			/* minimum longitude			     */
	maxlat,			/* maximum latitude			     */
	maxlon,			/* maximum longitude			     */
	*elev;			/* elevation of chip			     */
{
extern long curent;		/* current "line" for processimg messages    */

struct	DCW_VECTOR dcw_vector;	/* set of lines for the area		     */

int fd;				/* image file descriptor used for image I/O  */
float	*chipbuf;		/* buffer of pixels			     */
unsigned char	*outbuf;	/* pixels output to chip		     */

char	ptext[ERRLEN],     	/* processing message text		     */
	curlib[10];		/* current library			     */

long    i, 		   	/* looping variables			     */
	idtype = EREAL,		/* want to deal with float on input	     */
	len,		   	/* string length			     */
	msgfmt = GEN1,	   	/* general message format one		     */
	max_entity = 9999, 	/* maximum entity for processing message     */
	nonfatal = 1,	   	/* non-fatal error code			     */
	segment,	   	/* current segment			     */
	status,   	   	/* return status			     */
	totlines = 0,		/* total # of lines read from DCW            */
	options[2],		/* image i/o options			     */
	zero = 0,	   	/* constant of zero			     */
	one = 1,		/* constant of one			     */
	access;			/* file access				     */

double	lineinc,sampinc;	/* line and sample increment		     */

options[0] = options[1] = 0;
/*
  Find out if this image was subsampled
  other than the normal
*/
if (model->dtype == GAC)
    {
    if (ddr->line_inc != 4.0)
        /* divide by the original line increment */
        lineinc = ddr->line_inc / 4.0;
    else
        lineinc = 1.0;
    if (ddr->sample_inc != 5.0)
        /* divide by the original sample increment */
        sampinc = ddr->sample_inc / 5.0;
    else
        sampinc = 1.0;
    }
else
    {
    lineinc = ddr->line_inc;
    sampinc = ddr->sample_inc;
    }

/*
  Allocate space for chip image pixels (2-d array)
*/
chipbuf = (float *)malloc(window[NL] * window[NS] * sizeof(float));
if (odtype == EBYTE)
    outbuf = (unsigned char *)calloc(window[NL] * window[NS],sizeof(char));
else if (odtype == EWORD)
    outbuf = (unsigned char *)calloc(window[NL] * window[NS],sizeof(short));
else if (odtype == ELONG)
    outbuf = (unsigned char *)calloc(window[NL] * window[NS],sizeof(long));
else if (odtype == EREAL)
    outbuf = (unsigned char *)calloc(window[NL] * window[NS],sizeof(float));
if ((chipbuf == NULL) || (odtype == NULL))
    {
    c_errmsg("Error occurred allocating dynamic memory","dcwchips-alloc",
	&nonfatal);
    return(E_FAIL);
    }

if (imgback)
    {
    /*
      Open the input image as real
    */
    access = IREAD;
    status = c_eopenr(&fd,hosin,&totbnd,&window[SL],&window[SS],
                      &window[NL],&window[NS],&access,&idtype,
                      options,&window[NL]);
    if (status == E_SUCC)
	{
        /*
          Get the image data for background
        */
        if (c_eread(&fd,&bandno[0],&one,chipbuf,&window[NL]) == E_FAIL)
	    {
	    c_errmsg("Error occurred reading image background","dcwchips-read",
	        &nonfatal);
	    free(chipbuf);
	    free(outbuf);
	    return(E_FAIL);
	    }
        c_eclose(&fd);
	}
    }
else
    {
    /* 
      Initialize the buffer to the background value
    */
    for (i = 0; i < window[NL] * window[NS]; i++)
	chipbuf[i] = backgrnd;
    }

/*
  Rasterize each area
*/
for (i = 0; i < acount; i++)
    {
    /*
      Allocate first line memory block
      (if this block is exceeded, it will be reallocated within the
      dcw support routines).
    */
    dcw_vector.numlines = 0;
    dcw_vector.maxline = LINEBLOCK;
    dcw_vector.line = (struct DCW_LINE *)malloc(LINEBLOCK * 
						sizeof(struct DCW_LINE));
    if (dcw_vector.line == NULL)
	{
	c_errmsg("Error occurred allocating dynamic memory","dcwchips-alloc",
	    &nonfatal);
	free(chipbuf);
	free(outbuf);
	return(E_FAIL);
	}

    /*
      Perform the dcw query, loading all lines that fit our area
    c_errmsg("Please wait while querying dcw"," ",&nonfatal);
    */
    dcw_query(cdpath,library[i],dcw_coverage,&dcw_vector,
	      minlat,minlon,maxlat,maxlon);

    /*
      If no lines were found go to next area
    */
    totlines += dcw_vector.numlines;
    if (dcw_vector.numlines == 0)
	{
        dcw_freelines(&dcw_vector);
	continue;
	}

    /*
      Initialize and start the processing message
    */
    strcpy(curlib,library[i]);
    len = strlen(curlib);
    c_low2up(curlib,&len);
    sprintf(ptext,"of %d for %s",dcw_vector.numlines,curlib);
    curent = 1;
    max_entity = dcw_vector.numlines;
    c_inittm("DCWCHIPS","Rasterized","line"," ",ptext,&max_entity,&zero,
		 msgtim,&msgfmt);
    c_startm();

    /*
      If lines were found, generate corresponding line/samples (rasterize)
    */
    for (segment = 0; segment < dcw_vector.numlines; segment++)
      	{
	/*
	  Process each segment
	*/
	status = fprocsat(&dcw_vector.line[segment],ddr,model,elev,window,
			lineinc,sampinc,pixval,chipbuf);
	if (status != E_SUCC)
	    {
	    free(chipbuf);
	    free(outbuf);
            return(E_FAIL);
	    }

	/*
	  Increment the segment counter
	*/
	curent++;
	}

    /*
      Stop the processing message
    */
    curent--;
    c_stoptm();

    /*
      Free lines
    */
    dcw_freelines(&dcw_vector);
    }

/* 
  If anything was rasterized save the chip
*/
if (totlines > 0)
    {
    /*
      Open the output chip as odtype
    */
    access = IWRITE;
    if (c_eopenr(&fd,hosout,&one,&one,&one,&window[NL],&window[NS],&access,
	&odtype,options,&window[NL]) != E_SUCC)
        {
        c_errmsg("Error occurred opening chip image","dcwchips-open",&nonfatal);
	free(chipbuf);
	free(outbuf);
        return(E_FAIL);
        }

    /*
      Convert the data and write it out
    */
    i = window[NL] * window[NS];
    c_pxconv(&idtype,&odtype,chipbuf,outbuf,&i);

    if (c_ewrite(&fd,&one,&one,outbuf,&window[NL]) == E_FAIL)
        {
       c_errmsg("Error occurred writing chip image","dcwchips-write",&nonfatal);
	free(chipbuf);
	free(outbuf);
        return(E_FAIL);
        }

    c_eclose(&fd);
    }

free(chipbuf);
free(outbuf);

if (totlines == 0)
    {
    c_errmsg("No vector data occurs in the output image","dcwchips-empty",
	    &nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
