/******************************************************************************
NAME:   			CONVERT 

PURPOSE:    Convert incorrectly calibrated temperature into a correctly
	    calibrated temperature.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   FIXCAL must be run under TAE.

PROJECT:			LAS

ALGORITHM:

   ALGORITHM REFERENCES:		none
******************************************************************************/

#include "las.h"
#include "adaps.h"
#include "fixcal.h"

FUNCTION convert(inbuf,odtype,ns,lut,inclip,outclip)

   char *inbuf;			/* Pointer to input buffer		*/
   long	odtype;			/* data type of output image		*/
   long ns;			/* number of image samples		*/
   short lut[];			/* Look-up Table			*/
   long *inclip;		/* Input pixels clipped			*/
   long *outclip;		/* Output pixels clipped		*/

{

unsigned char byte_buf;
char *inptr;			/* Input buffer pointer			*/

short short_buf;

long i;				/* Counter				*/
long one = 1;			/* Value of one				*/



/* Extract one pixel from the buffer.  If the pixel is a zero (0), leave it as
   is in the buffer.  Otherwise, map the input pixel value to the output pixel
   value and place it in the buffer.  Increment the buffer pointer. 
-----------------------------------------------------------------------------*/
inptr = inbuf;
for (i = 0; i < ns; i++)
    {

    if (odtype == EBYTE)
	{
        c_pxcopy(inptr,&byte_buf,&odtype,&one);
	if (byte_buf != 0)
	    {
	    if ((byte_buf == MIN_PIX_VALUE_US)||(byte_buf == MAX_PIX_VALUE_US))
		{
		(*inclip)++;
		(*outclip)++;
		}
	    else if ((lut[byte_buf] == MIN_PIX_VALUE_US) ||
		     (lut[byte_buf] == MAX_PIX_VALUE_US))
		(*outclip)++;
	    byte_buf = lut[byte_buf];
            c_pxcopy(&byte_buf,inptr,&odtype,&one);
	    }
	inptr += sizeof(char);
	}
    else if (odtype == EWORD)
	{
        c_pxcopy(inptr,&short_buf,&odtype,&one);
	if (short_buf != 0)
	    {
	    if ((short_buf==MIN_PIX_VALUE_1KM)||(short_buf==MAX_PIX_VALUE_1KM))
		{
		(*inclip)++;
		(*outclip)++;
		}
	    else if ((lut[short_buf] == MIN_PIX_VALUE_1KM) ||
		     (lut[short_buf] == MAX_PIX_VALUE_1KM))
		(*outclip)++;
	    short_buf = lut[short_buf];
            c_pxcopy(&short_buf,inptr,&odtype,&one);
	    }
	inptr += sizeof(short);
	}

    }

return;
}
