/*******************************************************************************
NAME:				FIXCAL

PURPOSE:    Fix calibration by updating a LAS image.  Used for NOAA-14
	    images in which calibration was improperly applied.


PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T.Baltzer	   5/97 	  CSC		Original Development
 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   FIXCAL must be run under TAE.

PROJECT:			LAS

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCES:		none
******************************************************************************/

#include <stdio.h>
#include "las.h"	

long   curent;				/* current processing line	     */

main()
{
struct PARBLK vblock;			/* TAE parameter block               */

long 	bandnum;			/* Band number to update	     */
long	clipper;			/* Percent of clipped pixels	     */
long	channel;			/* AVHRR channel specification	     */
long	inclip;				/* Input pixels that were clipped    */
long	msgtim;				/* elapsed time between display msg  */
long	odtype;				/* data type of output image         */
long	one = 1;			/* value of one			     */
long	outclip;			/* Output pixels that were clipped   */
long	status;				/* function return status code       */
long	std_out;			/* LUN to standard output    	     */
long	totbnd;				/* total number of bands             */
long	totpixels;			/* Total number of pixels in band    */
long	window[MAXIMG][4];		/* window for each input image       */
long	zero = 0;			/* Value of zero		     */

char	hosin[MAXIMG][CMLEN];		/* host names of input images        */
char	minmax[4];			/* min max flag                      */

/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);
inclip = 0;
outclip = 0;

/* Get parameters and globals
-----------------------------*/
getpar(&vblock,hosin[0],window,&totbnd,&odtype,&msgtim,minmax,&channel,&bandnum);

/* Process image
----------------------------------------------*/
proces(&vblock,hosin[0],window,totbnd,odtype,msgtim,minmax,channel,&bandnum,
       &inclip,&outclip);

/*  Update history file 
-----------------------*/
if (c_hstout(&vblock,hosin[0],"",&zero," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","fixcal-warn",&one);

/* Output clipping statistics
------------------------------*/
totpixels = window[0][NL] * window[0][NS];
clipper = (100.0 * inclip / totpixels) + 0.5;
c_lasmsg(FALSE,"fixcal-clip","Input pixels clipped:  %d of %d (%d percent)",
	 inclip,totpixels,clipper);
clipper = (100.0 * outclip / totpixels) + 0.5;
c_lasmsg(FALSE,"fixcal-clip","Output pixels clipped: %d of %d (%d percent)",
	 outclip,totpixels,clipper);

/* Output completion message 
----------------------------*/
c_complt("FIXCAL"," ",&zero);
}


/******************************************************************************
NAME:   			GETPAR 

ALGORITHM:
******************************************************************************/

FUNCTION getpar(vblock,hosin,window,totbnd,odtype,msgtim,minmax,channel,bandnum)

   struct PARBLK *vblock;	/* pointer to TAE parameter block            */
   char hosin[][CMLEN];         /* host file names of input images 	     */
   long	(*window)[4];		/* window specifications for each input image*/
   long	*totbnd;		/* total number of bands to be processed     */
   long	*odtype;		/* output data type                          */
   long	*msgtim;		/* elapsed time between display messages     */
   char	*minmax;		/* min max flag buffer                       */
   long *channel;		/* AVHRR channel specification		     */
   long *bandnum;		/* Band number to update		     */
{
struct VARIABLE	*p_find();	/* function to find a TAE variable           */
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long	bands[MAXIMG][MAXBND+1];/* bands to be processed - all images	     */
long	element;		/* parameter element number		     */
long    fatal = -1;		/* fatal error flag			     */
long	imglim;			/* maximum number of images to be processed  */
long	nbands[MAXIMG];		/* number of bands per input image	     */
long	nmrimg;			/* number of input images                    */
long    nonfatal = 1;		/* nonfatal error flag			     */
long	qflag[MAXIMG][2];	/* flag indicating if subwindows specified   */
long	same;			/* flag indicating if images have same window*/
long	status;			/* function return status code               */

char	taein[MAXIMG][CMLEN];	/* TAE names of input images		     */

/* Retrieve and parse input image specification 
-----------------------------------------------*/
element = 0;
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,&nmrimg,window,bands,nbands,
		totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","fixcal-fatal",&fatal);
if (*totbnd > 1)
   {
   c_errmsg("More than one band specified","fixcal-band",
	    &nonfatal);
   c_errmsg("Fatal error encountered","fixcal-fatal",&fatal);
   }
if (qflag[0][0] == 1)
   {
   c_errmsg("Windowing except for band spec not allowed","fixcal-win",
	    &nonfatal);
   c_errmsg("Fatal error encountered","fixcal-fatal",&fatal);
   }
*bandnum = bands[0][0];


/* Retrieve the channel spec
-----------------------------*/
val = p_find(vblock,"CHANNEL");
*channel = IVAL(*val,0);

/* Hardwire output data type to "SAME"
---------------------------------------*/
*odtype = 0;

/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

return;
}
