#ifndef CALCOEF_H
#define CALCOEF_H


#define MAXLNS  50	/* Maximum number of lines                            */
#define MAXBUF 512	/* Maximum number of bytes in prefix record           */
#define NSATS   15	/* Number of satellites tiros-n thru noaa14           */

#define MIN_PIX_VALUE_US        1       /* Min pixel val for US thermal band */
#define MAX_PIX_VALUE_US        255     /* Max pixel val for US thermal band */
#define MIN_PIX_VALUE_1KM       10      /* Min pixel val for 1KM thermal band */
#define MAX_PIX_VALUE_1KM       1018    /* Max pixel val for 1KM thermal band */
#define MAX_LUT_DIM		MAX_PIX_VALUE_1KM + 1	/* Max LUT dimension  */

/*
  Define the different methods of calibration
*/
#define KAUFMAN   0
#define PRELAUNCH 1
#define RAO       2
#define TEILLET   3
#define VERMOTE   4

/*
  Gain scaling factor (2^30) and intercept scaling factor (2^22)
  (NOAA Poloar Orbiter Data Users Guide - Calibration of AVHRR Data)
*/
#define TWO30	1073741824.0
#define	TWO22	4194304.0

/*
  Constant used to calculate the calibration coeffcients
  (NOAA Poloar Orbiter Data Users Guide - Appendix A)
*/
#define C1	1.1910659E-5
#define C2	1.438833

/*
  Prefix data contains the full telemetry information.
  (NESS 107 - For full telemetry information)
*/
#define PREFIX_FULL_TELEMETRY		1

/*
  Prefix data contains telemetry is from a level-1b tape
  (NOAA Poloar Orbiter Data Users Guide - For telemetry information)
*/
#define PREFIX_LEVEL1B_TELEMETRY	2

/*
  Prefix data contains calibration coefficients from a level-1b tape
  (NOAA Poloar Orbiter Data Users Guide - For coefficient information)
*/
#define PREFIX_LEVEL1B_CALCOEFS		3

#endif

/* Structure used for storing scaling quantities */
/*-----------------------------------------------*/
struct SCALESTR
   {
   double minval;               /* Minimum value of appropriate quantity */
   double maxval;               /* Maximum value of appropriate quantity */
   double scale;                /* Scale value to apply                  */
   double offset;               /* Offset value to apply                 */
   };


