/******************************************************************************
NAME:   			PROCES 

PURPOSE:    Update the input image.

PROGRAM HISTORY:

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   FIXCAL must be run under TAE.

PROJECT:			LAS

ALGORITHM:

ALGORITHM REFERENCES:		none
******************************************************************************/

#include "las.h"
#include "fixcal.h"


extern long curent;		/* current processing line		     */

FUNCTION proces(vblock,hosin,window,totbnd,odtype,msgtim,minmax,channel,
		bandnum,inclip,outclip)

   struct PARBLK *vblock;	/*  TAE parameter block			     */
   char hosin[][CMLEN];		/* host names of input images		     */
   long	(*window)[4];		/* window specification of input images      */
   long totbnd;			/* total number of bands to be processed     */
   long	odtype;			/* data type of output image                 */
   long	msgtim;			/* elapsed time between display messages     */
   char	*minmax;		/* min max flag                              */
   long channel;		/* AVHRR channel spec			     */
   long *bandnum;		/* Band number to process		     */
   long *inclip;		/* Input pixels clipped			     */
   long *outclip;		/* Output pixels clipped		     */

{

static double  minval[MAXBND];	/* min pixel value array                     */
static double  maxval[MAXBND];	/* max pixel value array                     */

int	fdesc[MAXIMG+1];	/* image file descriptors used for image I/O */

long	access;			/* image I/O access type                     */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long    mmflag;
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long	nbandsimg;		/* Number of bands in one image		     */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long	size;			/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long	zero = 0;		/* value of zero                             */

char    *thebuf;		/* image I/O buffer                          */

short lut[MAX_LUT_DIM];		/* Look-up Table			     */


/* Open input image
----------------------*/
access = IUPDATE;
sl = window[0][SL];
ss = window[0][SS];
nl = window[0][NL];
ns = window[0][NS];
nbandsimg = 0;
status = c_eopenr(&fdesc[0],hosin[0],&nbandsimg,&sl,&ss,&nl,&ns,
                  &access,&odtype,&zero,&zero);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","fixcal-fatal",&fatal);

/* Setup for data type
----------------------*/
switch(odtype)
   {
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:
   case EREAL:
		c_errmsg("Invalid data type","fixcal-dtyp",&nonfatal);
		c_clean(vblock,"fixcal","",&zero);
		break;
   }

/* Allocate I/O buffer
----------------------*/
if ((thebuf = malloc(ns * size)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","fixcal-alloc",&nonfatal);
   c_clean(vblock,"fixcal","",&zero);
   }

/* Load the Look-up Table
------------------------------*/
gen_luts(vblock,odtype,channel,lut);

/* Setup for processing message utility
---------------------------------------*/
c_inittm("FIXCAL","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
   mmflag is set to be used within the processing loop -- quicker than strcmp
-----------------------------------------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   c_inmnmx(&odtype,&totbnd,minval,maxval);
   mmflag = TRUE;
   }
else
   mmflag = FALSE;

/* Main processing loop
-----------------------*/
for (curlin = 1; curlin <= nl; curlin++)
    {
    status = c_eread(&fdesc[0],bandnum,&curlin,thebuf,&one);
    if (status != ns)
	c_clean(vblock,"fixcal","",&zero);
    convert(thebuf,odtype,ns,lut,inclip,outclip);
    status = c_ewrite(&fdesc[0],bandnum,&curlin,thebuf,&one);
    if (status != ns)
	c_clean(vblock,"fixcal","",&zero);

    if (mmflag)
	status = c_pxmnmx(thebuf,&ns, &minval[0],&maxval[0],&odtype); 

    curent++;
    }

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosin[0],&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","fixcal-warn",&one);

/* Free buffer
--------------*/
free(thebuf);

return;
}
