/*******************************************************************************
FUNCTION:	gapexp

PURPOSE:   	For line gaps exceeding MINLINES in size, expand the gap in
		both directions by 2 lines.

Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  0.1    04/96  T. Baltzer       Original development
  1.1    07/96  T. Baltzer       Mods to support calls to implement DATAPATH
				 (c_arcopnwrt() and c_arclose()).

PROJECT:	ADAPS 
*******************************************************************************/
#include "gapexp.h"



/* Global variables to allow easy calling of the error handling routine
------------------------------------------------------------------------------*/
char outhost[2][CMLEN];	/* Host name of the files created by this function    */
char outlas[2][CMLEN];	/* Host name of the files created by this function    */

char infile[CMLEN];	/* Host name of the input image                       */
char outfile[CMLEN];	/* Host name of the output image                      */

long curent;		/* Current processing line                            */
long indesc;		/* AVHRR descriptor for input image		      */
long nfiles = 0;	/* Number of files created by this function           */
long outdesc;		/* AVHRR descriptor for output image		      */

struct PARBLK vblock;	/* TAE parameter block                                */

main()
{
long minlines;		/* Minimum lines in gap before expanding	      */
long msgfmt   =  3;	/* Format of the processing message                   */
long msgtim;		/* Elapsed time between processing messages           */
long status;		/* Function return status                             */
long std_out;		/* LUN to standard output                             */
long zero     =  0;	/* Value of zero for passing to functions             */

struct DROPPED drop;	/* AVHRR dropped line structure                       */
struct MODEL model;	/* AVHRR model structure                              */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Get the parameters
---------------------*/
getpar(&minlines, &msgtim);


/* Open the input and output images
-----------------------------------*/
open_images(outhost, outlas, &nfiles, &drop, &model);


/* Initialize processing message utility
----------------------------------------*/
curent = 1;
c_inittm("GAPEXP", "Processed", "line", "", "", &model.nlines,
    &zero, &msgtim, &msgfmt);
c_startm();


/* Process the image
------------------------------------*/
process (&drop, &model, minlines);


/* Close both the input and output images
-----------------------------------------*/
status = c_arclose(infile, &indesc);
if (status != E_SUCC)
    errout();
indesc = NULL;
status = c_arclose(outfile, &outdesc);
if (status != E_SUCC)
    errout();
outdesc = NULL;


/* Stop the processing message
------------------------------*/
c_stoptm();


/* Save the header file for the output image
--------------------------------------------*/
status = c_putahdr(outfile, &model, &drop);
if (status != E_SUCC)
    errout();


/* Issue messages about the number of pixels and lines dropped
--------------------------------------------------------------*/
print_stats (&drop, &model);


/* Output completion message 
----------------------------*/
c_complt("GAPEXP", outlas, &nfiles);
}
