/*******************************************************************************
FUNCTION:	getpar

PURPOSE:   	Retrieve the parameters from the variable block.

*******************************************************************************/
#include "gapexp.h"

extern struct PARBLK vblock;	/* TAE parameter block			      */

extern char infile[CMLEN];	/* Host name of the input image		      */
extern char outfile[CMLEN];	/* Host name of the output image	      */

FUNCTION getpar (minlines, msgtim)

    long *minlines;		/* OUT - Minimum lines in gap before expanding*/
    long *msgtim;		/* OUT - Seconds between processing messages  */
{

long access;		/* File access mode 				      */
long fatal    = -1;	/* Fatal error flag                                   */
long zero     =  0;	/* Constant of zero                                   */

struct VARIABLE *p_find(); /* Function to retrieve a TAE variable             */
struct VARIABLE *parm;     /* Pointer to TAE parameter within vblock          */


/* Get the input image name.
------------------------------------------------------------------------*/
access = IREAD;
if (c_getnam(&vblock, "IN", &zero, infile, "arch", &access) != E_SUCC)
    c_errmsg("Fatal error encountered", "gapexp-fatal", &fatal);


/* Get the output image name.
------------------------------------------------------------------------*/
access = IWRITE;
if (c_getnam(&vblock, "OUT", &zero, outfile, "arch", &access) != E_SUCC)
    c_errmsg("Fatal error encountered", "gapexp-fatal", &fatal);


/* Get the Minimum lines in a gap which needs to be expanded.
------------------------------------------------------------------------*/
parm = p_find(&vblock, "MINLINES");
*minlines = IVAL(*parm,0);


/* Get the number of seconds between processing messages
--------------------------------------------------------*/
parm = p_find(&vblock, "$MSGTIM");
*msgtim = IVAL(*parm,0);

return(E_SUCC);
}
