/*******************************************************************************
FUNCTION:	open_images

PURPOSE:   	Open the input and output images.

*******************************************************************************/
#include "gapexp.h"

extern long indesc;		/* OUT - AVHRR descriptor for input image     */
extern long outdesc;		/* OUT - AVHRR descriptor for output image    */

extern char infile[CMLEN];	/* Host name of the input image		      */
extern char outfile[CMLEN];	/* Host name of the output image	      */

FUNCTION open_images (outhost, outlas, nfiles, drop, model)

    char outhost[][CMLEN];	/* OUT - Host names of output files created   */
    char outlas[][CMLEN];	/* OUT - LAS names of output files created    */
    long *nfiles;		/* OUT - Number of output files created       */
    struct DROPPED *drop;	/* OUT - AVHRR dropped line structure         */
    struct MODEL *model;	/* OUT - AVHRR model structure                */
{

long access   = IREAD;	/* Access for opening the input image                 */
long status;		/* Function return status                             */
long tpunit   = NULL;	/* Tape drive descript if image is on tape            */
long window[4];		/* Window for the input image                         */



/* Get the header for the input image
-------------------------------------*/
status = c_getahdr(infile, model, drop);
if (status != E_SUCC)
    errout();


/* Due to the fact that c_arcwrite() will write each image line in the order
   that we send them to it, we must process the input image in satellite view.
   If we process the input image in "North-up" order then the output image will
   be flipped incorrectly for all ascending images.
------------------------------------------------------------------------------*/
model->sat_view = TRUE;


/* Open the input image
-----------------------*/
window[SL] = 0;
status = c_arcopnrd(infile, &access, window, model, &tpunit, &indesc);
if (status != E_SUCC)
    errout();


/* Open the output image
------------------------*/
status = c_arcopnwrt(outfile, &model->dtype, &model->nlines, &outdesc);
if (status != E_SUCC)
    errout();


/* Add the output image file names to the global variables
----------------------------------------------------------*/
strcpy(outhost[0], outfile);
c_lsmknm(outfile, "ahdr", outhost[1]);
strcpy(outlas[0], outhost[0]);
strcpy(outlas[1], outhost[1]);
*nfiles = 2;


return(E_SUCC);
}
