/******************************************************************************
FUNCTION:	LISTEDIT

PURPOSE:	Edit browse list required to regenerate AVHRR browse images
		from archive tape.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       02/97    T. Baltzer		Initial development
 1.1       03/99    D.Lloyd		Added code for new subcommand to return
 					list of tapeids, removed column specific
					dependencies in input file, and forced
					leading zeroes on tapeids and time
					stamps.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			ADAPS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"		/* LAS include			       */
#include "util.h"
#include "listedit.h"


void main()
{
struct PARBLK   vblock;		/* TAE parameter block			*/

char filnam[CMLEN];		/* Name of input file			*/
char pflag[2];			/* Process flag Y=browse generated N=not*/
char sceneid[17];		/* Scene identifier			*/
char subcmd[10];		/* Sub command				*/
char tapeid[7];			/* Tape id				*/
char tstamp[7];			/* Time stamp				*/
char dummy[1][CMLEN];		/* dummy string for c_complt call	*/

long count = 0;			/* Count of files generated		*/
long fatal = -1;		/* Fatal error status			*/
long nonfatal = 1;		/* Non-fatal error status		*/
long status;			/* Return status			*/
long std_out;			/* Standard output device 		*/

FILE *infp;			/* Input file pointer			*/


/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);


/*
  Get the input parameters
*/
status = getpar(&vblock,subcmd,filnam,sceneid,tstamp,tapeid,pflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","listedit-fatal",&fatal);


/*
Open the input file, read past the header, process appropriately.
*/
status = processit(&vblock,subcmd,filnam,sceneid,tstamp,tapeid,pflag,&infp);
if (status != E_SUCC)
    {
    fclose(infp);
    c_errmsg("Fatal error encountered","listedit-fatal",&fatal);
    }


/*
  Close the input file
*/
if ( fclose(infp) != E_SUCC )
   {
   c_errmsg("Error closing input file","listedit-clos",&nonfatal);
   c_errmsg("Nonfatal error encountered","listedit-warn",&nonfatal);
   }


/*
  Exit with a success status
*/
dummy[0][0] = '\0';
c_complt("LISTEDIT",dummy,&count);
}
