/******************************************************************************
NAME:   			LOC_LINE 

PURPOSE:    Locate the line to update and update the status flag.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T. Baltzer	   02/97	  CSB		Original Development
  1.1	   D.Lloyd	   03/99	Removed dependancies for column
 					specific locations in infile and force
					leading zeroes for time stamp
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:			ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "listedit.h"


int loc_line
(
FILE *infp,			/* LUT file pointer			*/
char sceneid[],			/* User input scene ID			*/
char tstamp[],			/* User input time stamp 		*/
char pflag[],			/* Processing flag			*/
char linebuf[BLINELEN+2]	/* Buffer containing line of data	*/
)

{

long size = 16;		/* Size of scene ID			*/
long value;

char lnscene[17];	/* Scene from line of data		*/
char lntstamp[7];	/* Time stamp from line of data		*/
char stamp[7];		/* local copy of time stamp		*/

int  n;			/* num conversions that sscanf did	*/


/* Insure the time stamp we are comparing against has leading zeros
-------------------------------------------------------------------*/
value = atoi( tstamp );
sprintf( stamp, "%06d", value );
stamp[6] = '\0';

/* Read a line, if NULL is returned, assume the EOF
   has been encountered.  Print an error message,
   return with an eof status.
   If the line read is a blank line, get the next
   line.
   Extract the scene ID, time stamp and determine
   if they are the ones specified by the user.
------------------------------------------------------------------------*/
for (;;)
   {
   if ( fgets(linebuf,BLINELEN+2,infp) == NULL)
      {
      c_lasmsg(FALSE,"listedit-not","Scene ID: %s, Time stamp: %s not located",
	       sceneid,stamp);
      return(E_FEND);
      }
   if (strcmp(linebuf,"\n") != 0)
      {
      n = sscanf( linebuf, "%*d %s %d", lnscene, &value );
      /* Make sure sscanf actually retrieved values before checking.
      If the line returned is blank but not an empty string, sscanf
      will not assign values to the variables.  If these already
      contain values from a previous read, those values will be used
      and duplicate entries will be returned.  Added for CR 2802
      --------------------------------------------------------------*/
      if ( n == 2 )
	  {
	  sprintf( lntstamp, "%06d", value ); /* force leading 0's */
	  lnscene[16] = '\0';
	  lntstamp[6] = '\0';
	  c_up2low(lnscene,&size);	/* lower case the scene ID     */
	  if (strcmp(lnscene,sceneid) == 0)
	     {
	     if (strcmp(lntstamp,stamp) == 0)
		break;
	     }
	  }
      }
   }

wrt_flg(infp,pflag);

return(E_SUCC);
}
