/******************************************************************************
NAME:	   LOC_TAPEID

PURPOSE:   Build a list of tapeids from the input file and return it

PROGRAM HISTORY:
VERSION	   PROGRAMMER	DATE	REASON
-------	   ----------	----	----------------------
  1.0	   D.Lloyd	03/99	Initial development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:			ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "listedit.h"


int loc_tapeid
(
struct	PARBLK	*vblock,	/* TAE parameter block			*/
FILE	*infp,			/* LUT file pointer			*/
char	linebuf[BLINELEN+2]	/* Buffer containing line of data	*/
)
{
char	lscene[17];		/* scene id from line buffer		*/
char	ltapeid[7];		/* tapeid from line buffer		*/
char	*sptr[200];		/* pointer array for updating vblock	*/
char	xtapeid[200][7];	/* Return time stamp	 		*/

long	flag;			/* control flag 			*/
long	i;			/* loop control variable		*/
long	nscenes;		/* Number of scenes returned		*/
int	nscenes_arr[1];		/* Number of scenes array		*/
long	tpval;			/* tapeid numerical value		*/
long	value;			/* time stamp numerical value		*/

int  n;				/* num conversions that sscanf did	*/


/* Read a line, if NULL is returned, assume the EOF has been encountered.
   If the line read is a blank line, get the next line.
   If the line matches the tape id of interest,
   and the processing flag does not equal "Y",
   extract the scene ID and time stamp.
------------------------------------------------------------------------*/
nscenes = 0;
while ( nscenes < 200 )
    {
    if ( nscenes == 200 ) break;
    if ( fgets( linebuf, BLINELEN+2, infp ) == NULL )
      break;

    if ( strcmp( linebuf, "\n" ) != 0 )
	{
	n = sscanf( linebuf, "%d %s %d", &tpval, lscene, &value );
        /* Make sure sscanf actually retrieved values before checking.
        If the line returned is blank but not an empty string, sscanf
        will not assign values to the variables.  If these already
        contain values from a previous read, those values will be used
        and duplicate entries will be returned.  Added for CR 2802
        --------------------------------------------------------------*/
	if ( n == 3 )
	    {
	    sprintf( ltapeid, "%06d", tpval );
	    flag = TRUE;
	    for ( i = 0; i < nscenes; i++ )
		if ( strncmp( xtapeid[i], ltapeid, 6 ) == 0 )
		    flag = FALSE;
	    if ( flag )
		{
		strncpy( xtapeid[nscenes], ltapeid, 6 );
		xtapeid[nscenes][6] = '\0';
		nscenes++;
		}
	    }
	}
    }


/* Output the integer values to the variable block
--------------------------------------------------*/
nscenes_arr[0] = nscenes;
if ( q_intg( vblock, "NTAPES", 1, nscenes_arr, P_UPDATE ) != SUCCESS )
   {
   c_lasmsg( FALSE, "listedit-rparm", "Error returning parameter to TAE" );
   c_lasmsg( TRUE, "listedit-fatal", "Fatal error encountered" );
   }


/* Output the string values to the variable block if there are any.
-------------------------------------------------------------------*/
if ( nscenes > 0 )
   {
   for ( i = 0; i < nscenes; i++ )
      sptr[i] = xtapeid[i];
   if ( q_string( vblock, "XTAPEID", nscenes, sptr, P_UPDATE ) != SUCCESS )
      {
      c_lasmsg( FALSE, "listedit-rparm", "Error returning parameter to TAE" );
      c_lasmsg( TRUE, "listedit-fatal", "Fatal error encountered" );
      }
   }


if ( q_out( vblock ) != SUCCESS )
   {
   c_lasmsg( FALSE, "listedit-rparm", "Error returning parameter to TAE" );
   c_lasmsg( TRUE, "listedit-fatal", "Fatal error encountered" );
   }

return E_SUCC;
}
