/******************************************************************************
NAME:   			LOC_TSTAMP 

PURPOSE:    Locate the scenes residing on a specified tape and return them to
	    TAE.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T. Baltzer	   02/97	  CSB		Original Development
  1.1      D.Lloyd         03/99        Removed dependancies for column
                                        specific locations in infile and force
					leading zeroes for time stamp

 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:			ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "listedit.h"


int loc_tstamp
(
struct PARBLK   *vblock,     /* TAE parameter block			*/
FILE *infp,			/* LUT file pointer			*/
char sceneid[],			/* Tape id				*/
char linebuf[BLINELEN+2]	/* Buffer containing line of data	*/
)
{

long size = 16;			/* Size of scene ID			*/
long nscenes;			/* Number of scenes returned		*/
long value;			/* numeric value of time stamp		*/

char xsceneid[20][17];		/* Return scene ID			*/
char xtstamp[20][7];		/* Return time stamp	 		*/
char lscene[17];		/* scene id from line buffer		*/
char flag;			/* copy flag from line buffer		*/

int  n;				/* num conversions that sscanf did	*/


/* Read a line, if NULL is returned, assume the EOF has been encountered.
   If the line read is a blank line, get the next line.
   If the line matches the tape id of interest,
   and the processing flag does not equal "Y",
   extract the scene ID and time stamp.
------------------------------------------------------------------------*/
nscenes = 0;
for (;;)
   {
   if ( fgets(linebuf,BLINELEN+2,infp) == NULL)
      break;

   if (strcmp(linebuf,"\n") != 0)
      {
      n = sscanf( linebuf, "%*d %s %d %c", lscene, &value, &flag );
      /* Make sure sscanf actually retrieved values before checking.
      If the line returned is blank but not an empty string, sscanf
      will not assign values to the variables.  If these already
      contain values from a previous read, those values will be used
      and duplicate entries will be returned.  Added for CR 2802
      --------------------------------------------------------------*/
      if ( n == 3 )
	  {
	  if (strncasecmp(lscene,sceneid,16) == 0)
	      {
	      if (flag != 'Y')
		 {
		 strncpy(xsceneid[nscenes],lscene,16);	/* SCENE ID */
		 xsceneid[nscenes][16] = '\0';
		 c_up2low(xsceneid[nscenes],&size);/* lower case the scene ID */
		 sprintf( xtstamp[nscenes], "%06d", value );/* force leading 0*/
		 xtstamp[nscenes][6] = '\0';
		 nscenes++;
		 }
	      }
	  }
      }
   }

taeland(vblock,nscenes,xsceneid,xtstamp);

return(E_SUCC);
}
