/******************************************************************************
NAME:   			PROCESSIT 

PURPOSE:    Process the input list

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T. Baltzer	   02/97	  CSB		Original Development
  1.1      D.Lloyd         03/99	Added call for new subcommand for
  					returning list of tapeids
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:			ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "listedit.h"


int processit
(
   struct PARBLK   *vblock,     /* TAE parameter block			*/
   char subcmd [],		/* Sub command				*/
   char filnam[],		/* Half orbit file name			*/
   char inscene[],		/* User input scene ID			*/
   char instamp[],		/* User input time stamp		*/
   char tapeid[],		/* Tape id				*/
   char pflag[],		/* Processing flag			*/
   FILE **infp			/* Input file pointer			*/
)
{

long nonfatal = 1;		/* non-fatal error message		*/

char linebuf[BLINELEN+2];	/* Buffer containing line of data	*/


/* Open the input file
-----------------------*/
*infp = fopen(filnam,"r+");
if (*infp == NULL)
   {
   c_errmsg("Error opening input file","listedit-open",&nonfatal);
   c_errmsg("Fatal error encountered","listedit-fatal",&nonfatal);
   return(E_FAIL);
   }

/* Read past the header information
------------------------------------*/
if ( fgets(linebuf,BLINELEN+2,*infp) == NULL)
   {
   c_errmsg("Error reading line in input file","listedit-read",&nonfatal);
   c_errmsg("Fatal error encountered","listedit-fatal",&nonfatal);
   return(E_FAIL);
   }
rewind( *infp );
for (;;)
   {
   if ( fgets(linebuf,BLINELEN+2,*infp) == NULL)
      {
      rewind( *infp );
      break;
      }
   if ( strncmp(&linebuf[0],"tape_id",7) == 0 )
      break;
   }

/* Process appropriately.
--------------------------------------------------*/
if (strcmp(subcmd,"UPDATE") == 0)
    {
    if (loc_line(*infp,inscene,instamp,pflag,linebuf) == E_FEND)
	return(E_FEND);
    }
else if (strcmp(subcmd,"FETCH") == 0)
    {
    if ( inscene[0] == 0 )
        {
	if (loc_scene(vblock,*infp,tapeid,linebuf) == E_FEND)
	    return(E_FEND);
	}
    else
        {
        if (loc_tstamp(vblock,*infp,inscene,linebuf) == E_FEND)
	    return(E_FEND);
	}
    }
else if (strcmp(subcmd,"TAPIDS") == 0)
    {
    if (loc_tapeid(vblock,*infp,linebuf) == E_FEND)
        return(E_FEND);
    }

return(E_SUCC);
}
