/******************************************************************************
FUNCTION:	TAELAND

PURPOSE:	Send a list of scenes back to TAE land.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       02/97    T. Baltzer		initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			ADAPS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"		/* LAS include			       */
#include "listedit.h"		/* Local include file		       */


int taeland
(
   struct PARBLK *vblock,	/* TAE parameter block			*/
   long nscenes,		/* number of passes on the tape		*/
   char xsceneid[][17],		/* scene identifier			*/
   char xtstamp[][7]		/* time stamp      			*/
)
{

char *sptr[100];		/* pointer array for updating vblock    */

long i;				/* counter				*/
long fatal = -1;		/* Fatal error status			*/
long nonfatal = 1;		/* Non-fatal error status		*/
int  nscenes_arr[1];		/* Number of scenes array		*/


/*
  Output the integer values to the variable block
*/
nscenes_arr[0] = nscenes;
if (q_intg(vblock,"NSCENES",1,nscenes_arr,P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error returning parameter to TAE","listedit-rparm",&nonfatal);
   c_errmsg("Fatal error encountered","listedit-fatal",&fatal);
   }


/*
  Output the string values to the variable block if there are any.
*/
if (nscenes > 0)
   {
   for (i = 0; i < nscenes; i++)
      sptr[i] = xsceneid[i];
   if (q_string(vblock,"XSCENEID",nscenes,sptr,P_UPDATE) != SUCCESS)
      {
      c_errmsg("Error returning parameter to TAE","listedit-rparm",&nonfatal);
      c_errmsg("Fatal error encountered","listedit-fatal",&fatal);
      }
   for (i = 0; i < nscenes; i++)
      sptr[i] = xtstamp[i];
   if (q_string(vblock,"XTSTAMP",nscenes,sptr,P_UPDATE) != SUCCESS)
      {
      c_errmsg("Error returning parameter to TAE","listedit-rparm",&nonfatal);
      c_errmsg("Fatal error encountered","listedit-fatal",&fatal);
      }
   }


if (q_out(vblock) != SUCCESS)
   {
   c_errmsg("Error returning parameter to TAE","listedit-rparm",&nonfatal);
   c_errmsg("Fatal error encountered","listedit-fatal",&fatal);
   }

return E_SUCC;
}
