/*******************************************************************************
NAME:				MULLI


PURPOSE:    Reverse the effects of illumination correction applied to an image
	    during calibration.  This capability is necessary because if a
	    product was calibrated with illumination correction, it should
	    not be later atmospherically corrected without first reversing
	    the effects of illumination correction.

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         REASON
-------    ----------      ----         ------
  1.0      T.Baltzer      04/00         Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   MULLI must be run under TAE.

PROJECT:			ADAPS

******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"	
#include "mulli.h"	


main()
    {

    char hosin[MAXIMG][CMLEN];	/* Host names of input images		*/
    char hosout[1][CMLEN];	/* Host name of output image	        */
    char minmax[4];		/* Min max flag				*/
    char sceneidlist[MAXSCENE][17];/* Scene list			*/

    int nsceneidlist;		/* Number of scenes in 'sceneidlist'	*/
    int sceneptlist[MAXSCENE];	/* Scene list pointer			*/

    long msgtim;		/* Elapsed time between display msg	*/
    long num_in_img = NUM_IN_IMG;/* Number of input images		*/
    long std_out;		/* LUN to standard output		*/
    long one = 1;		/* Constant value of one		*/

    struct FILEINFO  fileinfo[MAXIMG+1];/* File information structure	*/
    struct PARBLK vblock;		/* TAE parameter block		*/


    /* Initialization
    ---------------------------*/
    c_inlas(&vblock,&std_out);


    /* Get parameters and globals
    -----------------------------*/
    getpar(&vblock,fileinfo,hosin,hosout,sceneptlist,sceneidlist,
	   &nsceneidlist,&msgtim,minmax);


    /* Open the images and then call the main processing routine
    --------------------------------------------------------------*/
    pre_process(&vblock,fileinfo,hosin,hosout,sceneptlist,sceneidlist,
		nsceneidlist,msgtim,minmax);


    /*  Update history files 
    -------------------------*/
    if (c_hstout(&vblock,hosout[0],hosin,&num_in_img," ") != E_SUCC)
	c_lasmsg(FALSE,"mulli-warn","Nonfatal error encountered");


    /* Output completion message 
    ------------------------------*/
    c_complt("MULLI",hosout,&one);


    /* Put this return here to make the compiler happy
    --------------------------------------------------*/
    return(SUCCESS);

    }
