
/* Include file for the MULLI module	*/

#ifndef MULLI_H
#define MULLI_H

#include "model.h"

#define NUM_IN_IMG  2   /* Number of input images		 */
#define NUM_OUT_BND 2   /* Number of output bands		 */
#define MAXSCENE  300	/* Maximum number of scene reference in date log    */

#define IN_BAND    1    /* Band number for composite image file descriptor  */
#define DATE_BAND  2    /* Band number for composite image file descriptor  */
#define OUT_BAND   3    /* Band number for composite image file descriptor  */


/* Structure containing the necessary information for the input/ouput images.
------------------------------------------------------------------------------*/
struct FILEINFO
    {
    int nlines;			/* Number of lines	*/
    int nsamples;		/* Number of samples	*/
    int nbands;			/* Number of bands	*/
    long bands[MAXBND+1];	/* The actual bands	*/
    };

/* Structure used for storing scaling, offset quantities
--------------------------------------------------------*/
struct SCOFF
    {
    double minval;      /* Minimum value of appropriate quantity        */
    double maxval;      /* Maximum value of appropriate quantity        */
    double scale;       /* Scale value to apply                         */
    double offset;      /* Offset value to apply                        */
    double unscale;     /* Value to apply to unscale                    */
    };


/* Prototypes */

void get_composite_in
(
    struct PARBLK *vblock,      /* IN:  Pointer to TAE parameter block       */
    int which_band,             /* IN:  Which band has been specified        */
    char parmname[],            /* IN:  Parameter name of image              */
    int numbnd,                 /* IN:  Number of bands in parmname          */
    struct FILEINFO fileinfo[], /* OUT: File information                     */
    char hostname[]             /* OUT: Host file name                       */
);

void get_composite_out
(
    struct PARBLK *vblock,      /* IN:  Pointer to TAE parameter block       */
    int which_band,             /* IN:  Which band has been specified        */
    char parmname[],            /* IN:  Parameter name of image              */
    int numbnd,                 /* IN:  Number of bands in parmname          */
    int nlines,                 /* IN:  Number of lines in image             */
    int nsamples,               /* IN:  Number of samples in image           */
    struct FILEINFO fileinfo[], /* OUT: File information                     */
    char hostname[]             /* OUT: Host file name                       */
);

void get_composite_info
(
    struct PARBLK *vblock,      /* IN:  pointer to TAE parameter block        */
    struct FILEINFO fileinfo[], /* OUT: File information                      */
    char hosin[MAXIMG][CMLEN],  /* OUT: Host names of input images            */
    char hosout[][CMLEN]        /* OUT: Host name of output image             */
);

void getpar
(
    struct PARBLK *vblock,              /* IN: pointer to TAE parameter block */
    struct FILEINFO fileinfo[],         /* OUT: File information              */
    char hosin[MAXIMG][CMLEN],          /* OUT: Host names of input images    */
    char hosout[][CMLEN],               /* OUT: Host name of output image     */
    int sceneptlist[MAXSCENE],          /* OUT: Scene list pointers           */
    char sceneidlist[MAXSCENE][17],     /* OUT: Scene listing                 */
    int *nsceneidlist,                  /* OUT: Number in sceneidlist         */
    long *msgtim,                       /* OUT: time between display messages */
    char mmflag[]                       /* OUT: min max flag buffer           */
);

void pre_process
(
    struct PARBLK *vblock,      /* IN: TAE parameter block                   */
    struct FILEINFO  fileinfo[],/* IN: File information structure            */
    char hosin[][CMLEN],        /* IN: host names of input images            */
    char hosout[][CMLEN],       /* IN: host name of output image             */
    int sceneptlist[MAXSCENE],          /* IN: Scene pointer listing         */
    char sceneidlist[MAXSCENE][17],     /* IN: Scene listing                 */
    int nsceneidlist,                   /* IN: Number in sceneidlist         */
    long msgtim,                        /* IN: Time between messages         */
    char minmax[4]                      /* IN: Min max flags                 */
);

int process
(
    char hosout[][CMLEN],               /* IN: host name of output image     */
    struct FILEINFO fileinfo[],         /* IN: File information structure    */
    struct GDESC **gdesc,               /* IN: group descriptor for image I/O */
    long offset1[MAXIMG*MAXBND],        /* IN: offsets for input bands       */
    long offset2[MAXBND],               /* IN: offsets for output bands      */
    char *thebuf,                       /* IN: image I/O buffer              */
    int sceneptlist[MAXSCENE],          /* IN: Scene pointer listing         */
    struct MODEL model[MAXSCENE],       /* IN: Array of AVHRR models         */
    long msgtim,                        /* IN: time between display messages */
    char *minmax,                       /* IN: min max flag                  */
    struct SCOFF *scoff,                /* IN: Scaling coefficient structure */
    struct DDR *ddr,                    /* IN: Composite image DDR           */
    float aus2[MAXSCENE]                /* IN: sun-earth distance squared    */
);

int update_bdr
(
    char hosout[][CMLEN]        /* IN: host name of output image             */
);

int my_trans_init
(
    struct  DDR     *ddr,           /* IN:  image ddr structure             */
    long    *proj_units,            /* OUT: projection units value          */
    double  rot_coeffs[6]           /* OUT: rotation coefficients           */
);

int load_model
(
    char sceneid[],              /* IN:  Acquisition scene id                 */
    struct MODEL *model          /* OUT: Model and orbital information        */
);


#endif

