/****************************************************************************** NAME:                           PROCESS_LONG

PURPOSE:    Get scene acquisition information from one line of data
	    (long-format log file).

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      04/00          CSB           Stole from REATMO
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "mulliinfo.h"


int process_long
(
    char linebuf[CMLEN+1],	/* IN:  Buffer containing line of data     */
    int sceneptlist[],		/* IN/OUT:  List of scene pointers	   */
    char sceneidlist[][17],	/* IN/OUT:  List of scene id's in composite*/
    int *ndays,			/* IN/OUT: Number of days in date list     */
    char periods[][CMLEN],  	/* IN:  Periods used in the composite      */
    int nperiods		/* IN:  Number of periods in 'periods'     */
)

{

char onescene[17];	/* One 16 char scene id			*/
char period[CMLEN];	/* Period in line buffer		*/
char strindex[4];	/* String version of index		*/

int index;		/* Index representing composite number	*/
int i;			/* Counter				*/
int nscanned;		/* Number of items scanned from string	*/


/* Parse out the fields from the line buffer.  Ignore bad lines.
----------------------------------------------------------------*/
nscanned = sscanf(linebuf,"%s %*s %s %s", period, strindex, onescene);
if ( nscanned != 3 )
    return E_SUCC;


/* If the Date Index is "-1" then this scene was not included
   in the composite.  We are done considering this scene.
----------------------------------------------------------------*/
if (strcmp(strindex,"-1") == 0)
    return(E_SUCC);


/* If the Period Name in the log was not specified in PERIODS,
   then this scene was not included in the composite.  We are
   done considering this scene.
----------------------------------------------------------------*/
for (i = 0; i < nperiods; i++)
    {
    if (strncmp(period,periods[i],strlen(periods[i])) == 0)
	break;
    }
if (i == nperiods)
    return(E_SUCC);


/* If "not-a" appears as Date Index, then don't consider this line
   (often appears as last line in the log).
----------------------------------------------------------------------*/
if (strcmp(strindex,"not-a") == 0)
    return(E_SUCC);


/* Determine the date log index for this scene
--------------------------------------------------*/
index = atoi(strindex);
if (index >= MAXSCENE)
    {
    c_lasmsg(FALSE,"mullii-indx","Invalid scene index specified");
    c_lasmsg(TRUE,"mullii-fatal","Fatal error encountered");
    }


/* Update the scene pointer list.
   Copy the scene to the list of scene id's.
----------------------------------------------*/
sceneptlist[index] = *ndays;
strcpy(sceneidlist[*ndays],onescene);
(*ndays)++;


return(E_SUCC);
}
