/****************************************************************************** NAME:                           PROCESS_SHORT

PURPOSE:    Get scene acquisition information from one line of data
	    (short-format log file).

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      04/00          CSB           Stole from REATMO
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "mulliinfo.h"


int process_short
(
    char linebuf[CMLEN+1],	/* IN:  Buffer containing line of data     */
    int sceneptlist[],		/* IN/OUT:  List of scene pointers	   */
    char sceneidlist[][17],	/* IN/OUT:  List of scene id's in composite*/
    int *ndays 			/* IN/OUT: Number of days in date list     */
)

{

char onescene[17];	/* One 16 character scene id		*/
char strindex[4];	/* String version of index		*/

int index;		/* Index representing composite number	*/
int nscanned;		/* Number of items scanned from string	*/


/* This scene was used in the composite, parse out the fields
from the line buffer.  Ignore bad lines in input file.
-------------------------------------------------------------*/
nscanned = sscanf(linebuf,"%s %s",onescene,strindex);
if ( nscanned != 2 )
    return E_SUCC;


/* Determine the date band index number
-------------------------------------------------*/
index = atoi(strindex);
if (index >= MAXSCENE)
    {
    c_lasmsg(FALSE,"mullii-indx","Invalid scene index specified");
    c_lasmsg(TRUE,"mullii-fatal","Fatal error encountered");
    }


/* Update the scene pointer list.
   Copy the scene id into the scene id list.
-------------------------------------------------*/
sceneptlist[index] = *ndays;
strcpy(sceneidlist[*ndays],onescene);
(*ndays)++;


return(E_SUCC);
}
