/******************************************************************************
FUNCTION:	COMB

PURPOSE:	calculate a statistical combination

ALGORITHM REFERENCES
     x       x!
    C  =  --------
     y    y!(x-y)!

   In a combination the larger factorial on the bottom of 
   the fraction will completely factor into the x! on top, 
   leaving (x)(x-1)(x-2)...(x-y+1) on top  if y < (x-y)
   or      (x)(x-1)(x-2)...(y+1)   on top  if y > (x-y).  
   The we just need to divide out the remaining factor 
   on the bottom.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "navigate.h"

int comb(
long    x,		/* total number of elements 	*/
long	y 		/* number of elements to group 	*/
)

{
long	diff;		/* x - y			*/
long	i;		/* looping variable		*/
long	temp = 1;	/* temporary			*/
long	larger,		/* larger of y and (x-y)	*/
	smaller;

  
/* Find out which bottom factor is bigger
   --------------------------------------*/
diff = x - y;
if (y > diff)
    {
    larger = y;
    smaller = diff;
    }
else
    {
    larger = diff;
    smaller = y;
    }

/* Multiply the remaining factors on top
   -------------------------------------*/
for (i = x; i > larger; i--)
    temp *= i;

/* Divide out the remaining factors on bottom
   ------------------------------------------*/
for (i = 1; i <= smaller; i++)
    temp /= i;

return(temp);
}
