/****************************************************************************
NAME:		GETPAR

PURPOSE:   	Get parameters & open files for navigation
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "adaps.h"
#include "geompak.h"
#include "navigate.h"

void getpar(
	struct MODEL	*model,		/* AVHRR model information	*/
	struct DDR	*ddr,		/* AVHRR image ddr		*/
	long		*msgtim,	/* Message timer		*/
	long		*num_out,	/* Number of output files	*/
	int		**in_tpl,	/* Input TPL file descriptor	*/
	char		filename[][CMLEN], /* Name for output PRINT file*/
	char		*print_file,	/* Host print file name		*/
	char		*plot_file,	/* Host plot file name		*/
	char		*db_file,	/* Data base file name		*/
	FILE		**fptr,		/* Output report file descriptor*/
	FILE		**pptr,		/* Output plot file descriptor	*/
	FILE		**dbptr, 	/* Output dbase file descriptor	*/
	char		*host_name,	/* Host file name 		*/
	char		*tae_name,	/* TAE file name		*/
	long		*sarea 		/* Study area specification	*/
)

{
struct PARBLK vblock;		/* TAE parameter block 			*/
struct TPLHEAD header;		/* tie point location file header data 	*/

long 	topt, fopt, lopt,	/* PRINT (term,file,lp) flags 		*/
	ptyp = TPL,		/* type of tie point file 		*/
	acc = IREAD,		/* file access				*/
	std_out,		/* standard output LUN 			*/
	status,			/* function return status 		*/
	nonfatal = 1,		/* nonfatal flag for errmsg 		*/
	fatal = -1,		/* fatal flag for errmsg 		*/
	zero = 0;		/* constant of zero 			*/

char 	in[CMLEN];		/* the input Tie Point Location file name*/

struct VARIABLE *var;		/* pointer to a TAE variable in vblock 	*/

/* Initialize LAS processing
   -------------------------*/
c_inlas(&vblock,&std_out);

/* Get the message timer 
   ---------------------*/
var = p_find(&vblock, "$msgtim");
*msgtim = IVAL(*var, 0);


/* Get the tie point location file name
   ------------------------------------*/
if (c_getnam(&vblock,"INTL",&zero,in,"tpl",&acc) != E_SUCC)
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);

/* Open the verification (TPL) file
   ---------------------------------*/
if (c_ptopen(in_tpl,in,&ptyp,&acc,&header) != E_SUCC)
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);


/* Get the model info based on the
   image specified in header.sea_name
   ----------------------------------*/
strcpy(tae_name,header.sea_name);
c_cmhost(header.sea_name,host_name);
if (c_getaddr(host_name,model) != E_SUCC)
    {
    c_ptclse(in_tpl);
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
    }
if (c_getddr(host_name,ddr) != E_SUCC)
    {
    c_ptclse(in_tpl);
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
    }


/* Get the print option and open the temporary file
   ------------------------------------------------*/
strcpy(print_file, "navigate");
status = c_setprt(&vblock,&topt,&fopt,&lopt,print_file,filename);
if (status != E_SUCC)
    {
    c_ptclse(in_tpl);
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
    }

if (fopt) 
    *num_out = 1;
else
    *num_out = 0;

*fptr = fopen(print_file, "w");
if (*fptr == NULL)
    {
    c_ptclse(in_tpl);
    c_errmsg("Error opening report file","navigate-open",&nonfatal);
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
    }

/* Get the name of the plot file
--------------------------------*/
acc = IWRITE;
strcpy(plot_file,"");
if (c_getnam(&vblock,"OUTPLOT",&zero,plot_file,"plt",&acc) != E_SUCC)
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
if (strlen(plot_file) != 0)
    {
    *pptr = fopen(plot_file, "w");
    if (*pptr == NULL)
	{
	c_ptclse(in_tpl);
	c_errmsg("Error opening plot file","navigate-open",&nonfatal);
	c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
	}
    (*num_out)++;
    strcpy(filename[*num_out - 1], plot_file);
    }


/* Get the name of the data base file
--------------------------------------*/
acc = IWRITE;
strcpy(db_file,"");
if (c_getnam(&vblock,"DBFILE",&zero,db_file,"dat",&acc) != E_SUCC)
    c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
if (strlen(db_file) != 0)
    {
    *dbptr = fopen(db_file, "w");
    if (*dbptr == NULL)
	{
	c_ptclse(in_tpl);
	c_errmsg("Error opening database file","navigate-open",&nonfatal);
	c_errmsg("Fatal error encountered","navigate-fatal",&fatal);
	}
    (*num_out)++;
    strcpy(filename[*num_out - 1], db_file);
    }


/* Get the study area, if specified
   ---------------------------------*/
var = p_find(&vblock,"SAREA");
if (var->v_count == 0)
    *sarea = 0;
else
    *sarea = IVAL(*var, 0);


return;
}
