#ifndef NAVIGATE
#define NAVIGATE


#define MAXPTS 2000		/* maximum # points			      */
#define PTS_PER_GROUP 25	/* maximum # points per group		      */
#define MAXGRPS (MAXPTS/PTS_PER_GROUP)	
				/* maximum # of groups			      */

#define MAXPITCHRATE  0.002	/* maximum pitch (was 0.001) pixels/line      */
#define MAXROLLRATE  0.002	/* maximum roll  (was 0.001) pixels/line      */
#define MAXYAW  0.008		/* maximum yaw   (was 0.0008) dlines/samp     */
#define MAXALT  0.008		/* maximum alt   (was 0.0082) 		      */
#define MAXMEDIAN  0.8		/* maximum median residual for agreement w/fit*/
#define MAXRESID  40.0        	/* max resid for all points' agreement w/ fit */


/* Let the prototypes begin */

int calc_attitudes(
        FILE    *fptr,                  /* Output report                      */
        struct MODEL *model,            /* AVHRR model info from search image */
        struct DDR *ddr,                /* AVHRR image ddr                    */
        double dline[][PTS_PER_GROUP],  /* delta lines                        */
        double dsamp[][PTS_PER_GROUP],  /* delta samples                      */
        double refline[][PTS_PER_GROUP],/* reference lines                    */
        double refsamp[][PTS_PER_GROUP],/* reference samples                  */
        char id[][PTS_PER_GROUP][20],   /* buffer of tie point identifiers    */
        long used[][PTS_PER_GROUP],     /* flag if point was used             */
        long npts,                      /* # of points read from tpl file     */
        double linecoefs[],             /* Least Squares coeff's mapping lines*/
        double samplecoefs[],           /* Least Squares coeff's mapping samps*/
        long *msgtim,                   /* processing message time            */
        long ngroups,                   /* number of point groups             */
        long pts_in_group[]             /* number of points in each group     */
);


int comb(
long    x,              /* total number of elements     */
long    y               /* number of elements to group  */
);


void getpar(
        struct MODEL    *model,         /* AVHRR model information      */
        struct DDR      *ddr,           /* AVHRR image ddr              */
        long            *msgtim,        /* Message timer                */
        long            *num_out,       /* Number of output files       */
        int             **in_tpl,       /* Input TPL file descriptor    */
        char            filename[][CMLEN], /* Name for output PRINT file*/
        char            *print_file,    /* Host print file name         */     
        char            *plot_file,     /* Host plot file name          */
        char            *db_file,       /* Data base file name          */
        FILE            **fptr,         /* Output report file descriptor*/
        FILE            **pptr,         /* Output plot file descriptor  */   
        FILE            **dbptr,        /* Output dbase file descriptor */
        char            *host_name,     /* Host file name               */
        char            *tae_name,      /* TAE file name                */
        long            *sarea          /* Study area specification     */
);


void heapsort(
        long n,
        double ra[],
        double key[]
);


int lmsplane(
        FILE *fptr,                             /* output file pointer        */
        double refline[][PTS_PER_GROUP],        /* reference line             */
        double refsample[][PTS_PER_GROUP],      /* reference sample           */
        double dline[][PTS_PER_GROUP],          /* delta line (search - ref)  */
        double dsample[][PTS_PER_GROUP],        /* delta sample (search - ref)*/
        char id[][PTS_PER_GROUP][20],           /* buffer to hold tie point id*/
        long    used[][PTS_PER_GROUP],          /* flag if point was used     */
        long    npoints,                        /* # input points             */
        double tlines,                          /* total # lines in image     */
        double tsamples,                        /* total # samples in image   */
        double linecoefs[],                     /* final calc'd line coeffs   */
        double samplecoefs[],                   /* final calc'd sample coeffs */
        long    *msgtim,                        /* processing message time    */
        long    ngroups,                        /* # of point groups          */
        long    pts_in_group[]                  /* # of points in each group  */
);


void lsplane(
        long    npoints,        /* Number of points                     */
        double  x1vec[],        /* first independent values (lines)     */
        double  x2vec[],        /* second independent values (samples)  */
        double  yvec[],         /* dependant values (errors)            */
        double  coefs[]         /* Return coefficients                  */
);


void plane2attit(
        double linecoefs[],             /* Plane line coefficients      */
        double samplecoefs[],           /* Plane sample coefficients    */
        double zdot,                    /* Pass direction ASC/DESC      */
        long   dtype,                   /* Pass data type LAC/HRPT/GAC  */
        double satalt,                  /* Satellite altitude           */
        double totlines,                /* Total # lines in scene       */
        double offline,                 /* Line offset (master line)    */
        double offsample,               /* Sample offset (master sample)*/
        double rollcoefs[],             /* New roll coefficients        */
        double pitchcoefs[],            /* New pitch coefficients       */
        double yawcoefs[],              /* New yaw coefficients         */
        double *altcoef                 /* Delta altitude correction    */
);


int readpnts(
        int     **in_tpl,                   /* input TPL file descriptor      */
        long    *npts,                      /* # of points read from tpl file */
        char    id[][PTS_PER_GROUP][20],    /* buffer of tie point identifiers*/
        double  dline[][PTS_PER_GROUP],     /* err in ver pt in line dir      */
        double  dsamp[][PTS_PER_GROUP],     /* err in ver pt in sample dir    */
        double  refline[][PTS_PER_GROUP],   /* tie pt ref lines (orig AVHRR)  */
        double  refsamp[][PTS_PER_GROUP],   /* tie pt ref samps (orig AVHRR)  */
        double  srcline[][PTS_PER_GROUP],   /* tie pt search lines ("true")   */
        double  srcsamp[][PTS_PER_GROUP],   /* tie pt search samps ("true")   */
        double  latitude[][PTS_PER_GROUP],  /* tie pt latitudes               */
        double  longitude[][PTS_PER_GROUP], /* tie pt longitudes              */
        double  elevation[][PTS_PER_GROUP], /* tie pt elevations              */
        long    active[][PTS_PER_GROUP],    /* tie pt active flag             */
        long    *ngroups,                   /* number of groups               */
        long    pts_in_group[]              /* number of points in each group */
);


long qcompare(
        void    *ptr1,
        void    *ptr2
);


void write_db(
        struct MODEL *model,                    /* AVHRR model information    */
        FILE    *dbptr,                         /* output database file       */
        char id[MAXGRPS][PTS_PER_GROUP][20],    /* buffer to hold tie point id*/
        long used[MAXGRPS][PTS_PER_GROUP],      /* flag if point was used     */
        long sarea                              /* study area                 */
);


void write_report(
        struct MODEL *model,            /* AVHRR model information            */
        struct DDR *ddr,                /* AVHRR image ddr                    */
        FILE    *fptr,                  /* output report                      */
        FILE    *pptr,                  /* output plot file                   */
        long    npts,                   /* total number of verification points*/
        double  refline[][PTS_PER_GROUP],/* err in verification pt in line dir*/
        double  refsamp[][PTS_PER_GROUP],/* err in verification pt in samp dir*/
        double  srcline[][PTS_PER_GROUP],/* tie point search lines            */
        double  srcsamp[][PTS_PER_GROUP],/* tie point search samples          */
        double  latitude[][PTS_PER_GROUP],/* tie point latitudes              */
        double  longitude[][PTS_PER_GROUP],/* tie point longitudes            */
        double  elevation[][PTS_PER_GROUP],/* tie point elevations            */
        long    active[][PTS_PER_GROUP],/* tie point active flag              */
        double  linecoefs[],            /* Least Squares coeffs mapping lines */
        double  sampcoefs[],            /* Least Squares coeffs mapping samps */
        char    id[][PTS_PER_GROUP][20],/* buffer to hold tie point identifier*/
        long    used[][PTS_PER_GROUP],  /* flag if point was used             */
        long    ngroups,                /* number of point groups             */
        long    pts_in_group[],         /* number of points in each group     */
        char    plot_file[]             /* plot file name                     */
);


#endif

