/****************************************************************************
NAME:		WRITE_DB

PURPOSE:   	Generate a data base file.
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "adaps.h"
#include "navigate.h"

void write_db(
	struct MODEL *model,			/* AVHRR model information    */
	FILE 	*dbptr,				/* output database file	      */
	char id[MAXGRPS][PTS_PER_GROUP][20],	/* buffer to hold tie point id*/
	long used[MAXGRPS][PTS_PER_GROUP],	/* flag if point was used     */
	long sarea 				/* study area		      */
)

{

double  temp;			/* Input location in total degrees	*/

char	sceneid[17];		/* Scene id				*/

long    deg,			/* Degrees				*/
        mins,			/* Minutes				*/
        sec,			/* Seconds				*/
	i,j,			/* Counters				*/
	num_of_wvs,		/* Number of wvs points used in fit	*/
	num_of_dcw;		/* Number of dcw points used in fit	*/


/* Print the scene id
   ---------------------*/
c_getscene(model,sceneid);
fprintf(dbptr,"Scene   ID:  %s\n",sceneid);


/* Print the study area
   ----------------------*/
if (sarea == 0)
    fprintf(dbptr,"Study area:\n");
else
    fprintf(dbptr,"Study area:  %d\n",sarea);


/* Print the start, end time
   --------------------------*/
fprintf(dbptr,"Start Time:  %02d:%02d:%02d.%03d\n",
        model->shour, model->smin, model->ssec, model->sfrac);

fprintf(dbptr,"End   Time:  %02d:%02d:%02d.%03d\n",
        model->ehour, model->emin, model->esec, model->efrac);


/*
  Convert and print the North Nadir point
  ---------------------------------------*/
temp = model->nn[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(dbptr,"        NN: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->nn[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(dbptr,"%4dD %02dM %02dS\n",deg,mins,sec);


/*
  Convert and print the South Nadir point
  ---------------------------------------*/
temp = model->sn[0] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(dbptr,"        SN: %3dD %02dM %02dS  ",deg,mins,sec);

temp = model->sn[1] * DEG_PER_RAD;
c_cvtdms(&temp,&deg,&mins,&sec);
fprintf(dbptr,"%4dD %02dM %02dS\n",deg,mins,sec);


/*
  Print the orbit number (start of pass)
  ---------------------------------------*/
fprintf(dbptr,"     Orbit:  %05d\n",model->sorbit);


/*
  Print the roll (roll_coefs[0]), Droll (roll_coefs[1]),
            pitch (pitch_coefs[0]), Dpitch (pitch_coefs[1]),
            yaw (yaw_coefs[0]), Daltitude (delta_alt)
  -----------------------------------------------------------*/
fprintf(dbptr,"      Roll:  %lf\n",model->roll_coefs[0]);
fprintf(dbptr,"     Droll:  %lf\n",model->roll_coefs[1]);
fprintf(dbptr,"     Pitch:  %lf\n",model->pitch_coefs[0]);
fprintf(dbptr,"    Dpitch:  %lf\n",model->pitch_coefs[1]);
fprintf(dbptr,"       Yaw:  %lf\n",model->yaw_coefs[0]);
fprintf(dbptr," Daltitude:  %lf\n",model->delta_alt);


/* Print the number of points used in the fit for each type
   (WVS or DCW).
  -----------------------------------------------------------*/
num_of_wvs = num_of_dcw = 0;
for (i = 0; i < MAXGRPS; i++)
   {
   for (j = 0; j < PTS_PER_GROUP; j++)
      {
      if (used[i][j] == TRUE)
	 {
	 if (strncmp(id[i][j],"pt",2) == 0)
	    num_of_wvs++;
	 else
	    num_of_dcw++;
	 }
      }
   }
fprintf(dbptr,"# WVS pts :  %d\n",num_of_wvs);
fprintf(dbptr,"# DCW pts :  %d\n",num_of_dcw);


fclose(dbptr);

return;
}
