/******************************************************************************
FUNCTION:	NEXTDAY

PURPOSE:	Get the day, month, year of some date which is n days away
		from the specified reference date.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       09/93    T. Baltzer		initial development
 1.1       07/98    T. Baltzer		make y2k compliant

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"		/* LAS include			       */


main()
{
struct PARBLK   vblock;		/* TAE parameter block			*/

char prog[11];			/* program exit string			*/
char outdate[3][5];		/* output month, day, year		*/
char *sptr[3];                  /* pointer array for updating vblock    */

long day;			/* day					*/
long days_per_year;		/* days in a given year			*/
long ndays;			/* increment/decrement value		*/
long fatal = -1;		/* fatal error status			*/
long nonfatal = 1;		/* non-fatal error status		*/
long jdate;			/* julian date				*/
long month;			/* month				*/
long one = 1;			/* value of one				*/
long std_out;			/* standard output device 		*/
long status;			/* return status			*/
long year;			/* year					*/
long year4d;			/* 4 digit version of year		*/

struct VARIABLE *p_find();      /* function to find a TAE variable      */
struct VARIABLE *var;           /* pointer to a TAE variable            */


/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);


/*
  Get the 'current' or reference month, day, year
*/
var = p_find(&vblock,"INDATE");
month = IVAL(*var,0);
day = IVAL(*var,1);
year = IVAL(*var,2);

/*
  Determine a 4 digit version of whatever year was entered (2 digit or 4 digit).
*/
year4d = c_4digityr(year);


/*
  Get the increment / decrement value
*/
var = p_find(&vblock,"NDAYS");
ndays = IVAL(*var,0);


/*
  Convert to julian date
*/
c_grejul(&year4d,&month,&day,&jdate);


/*
  Increment / decrement the julian date
*/
jdate += ndays;


/*
  Adjust the year and julian day until the new year has been selected.
*/
if (ndays >= 0)
   {
   if (c_isleap(&year4d))
      days_per_year = 366;
   else
      days_per_year = 365;
   while (jdate > days_per_year)
      {
      jdate -= days_per_year;
      year4d++;
      if (c_isleap(&year4d))
         days_per_year = 366;
      else
         days_per_year = 365;
      }
   }
else
   {
   while (jdate < 1)
      {
      year4d--;
      if (c_isleap(&year4d))
         days_per_year = 366;
      else
         days_per_year = 365;
      jdate += days_per_year;
      }
   }

/*
  Convert back to month, day, year
*/
c_julgre(&year4d,&jdate,&month,&day);


/*
  Output the values to the variable block
*/
sprintf(&outdate[0][0],"%02d",month);
sprintf(&outdate[1][0],"%02d",day);
if (year > 1000)
    sprintf(&outdate[2][0],"%04d",year4d);
else
    sprintf(&outdate[2][0],"%02d",year4d % 100);

sptr[0] = outdate[0];
sptr[1] = outdate[1];
sptr[2] = outdate[2];
status = q_string(&vblock,"OUTDATE",3,sptr,P_UPDATE);
if (status != SUCCESS)
   {
   c_errmsg("Error sending variables to TAE", "nextday-dtae",&nonfatal);
   c_errmsg("Fatal error encountered", "nextday-fatal",&fatal);
   }
status = q_out(&vblock);
if (status != SUCCESS)
   {
   c_errmsg("Error sending variables to TAE", "nextday-dtae",&nonfatal);
   c_errmsg("Fatal error encountered", "nextday-fatal",&fatal);
   }


/*
  Terminate the program without a completion message
*/
strcpy(prog," ");
z_exit(one,prog);
}
