/******************************************************************************
FUNCTION:	SCHEDSTAT

PURPOSE:	To return the schedule entry status of the input SCENEID from
		the file ADAPSTABLES/master.sch.  The status is returned to
		the TAE parameter block.
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"

#define BUFLEN 86	/* length of schedule entry + 1, for line feed	*/


int main( void )
{
FILE   *masterfp;	/* file pointer to master.sch			*/

char   *adapstables;	/* pointer to environment variable		*/
char   curscene[17];	/* current scene ID from the input buffer	*/
char   inbuf[CMLEN];	/* input buffer, one entry from the schedule	*/
char   mastersch[CMLEN];/* master schedule file name			*/
char   scene_status[10];/* scene status string to return to TAE land	*/
char   sceneid[17];	/* scene ID to find in schedule			*/
char   schedule_status;	/* the status from the schedule			*/
char   *sptr[1];	/* pointer to string array for q_string call	*/

int    linesread;	/* number of lines read from master.sch		*/
int    numlines;	/* number of schedule entries in the master.sch	*/

long   std_out;		/* standard output				*/

struct VARIABLE *var;	/* pointer to TAE variable			*/
struct PARBLK vblock;	/* parameter block				*/


/* Initialize LAS and retrive the scene ID from the vblock
----------------------------------------------------------*/
c_inlas( &vblock, &std_out );
var = p_find( &vblock, "SCENEID" );
strcpy( sceneid, SVAL( *var, 0 ));

/* Get the master schedule name and open it
-------------------------------------------*/
adapstables = getenv( "ADAPSTABLES" );
sprintf( mastersch, "%s/master.sch", adapstables );
masterfp = fopen( mastersch, "r" );
if ( !masterfp )
    c_lasmsg(TRUE,"schedstat-open","Unable to open %s",mastersch);

/* Read the first line of the master schedule
which contains a count of the schedule entries
----------------------------------------------*/
fgets( inbuf, BUFLEN, masterfp );
sscanf( inbuf, "%d", &numlines );

/* Read the schedule entries until the sceneid
is found or until the entire schedule is read.
----------------------------------------------*/
linesread = 0;
sprintf( scene_status, "NOT FOUND" );
while ( linesread < numlines )
    {
    /* read a line and extract the scene ID */
    fgets( inbuf, BUFLEN, masterfp );
    linesread++;
    sscanf( inbuf, "%s", curscene );

    if ( strcasecmp( sceneid, curscene ) == 0 )
        { /* found the scene */
	/* extract the schedule status from the buffer */
	sscanf( inbuf, "%*s %*s %*s %*s %*s %*s %*d %c",
	    &schedule_status );

	/* set the scene status for return to TAE */
	if ( schedule_status == 'D' )
	    sprintf( scene_status, "DISK" );
	else if ( schedule_status == 'M' )
	    sprintf( scene_status, "MISS" );
	else if ( schedule_status == 'P' )
	    sprintf( scene_status, "PEND" );
	else
	    sprintf( scene_status, "UNKNOWN" );

	/* set linesread to numlines so that the loop exits */
	linesread = numlines;
	}  /* END if strcasecmp */
    }  /* END while */

/* Return the scene status to TAE land
--------------------------------------*/
sptr[0] = scene_status;
if ( q_string( &vblock, "STATFLAG", 1, (const char **)sptr, P_UPDATE )
     != SUCCESS)
    {
    c_lasmsg(FALSE,"schedstat-rparm","Error returning parameter to TAE");
    c_lasmsg(TRUE,"schedstat-fatal","Fatal error encountered");
    }

if ( q_out( &vblock ) != SUCCESS)
    {
    c_lasmsg(FALSE,"schedstat-qout","Error writing to parameter block");
    c_lasmsg(TRUE,"schedstat-fatal","Fatal error encountered");
    }

/* Make a quiet exit
--------------------*/
z_exit( 1, "" );

/* Make the compiler happy
--------------------------*/
return 0;
}  /* END schedstat */
