/******************************************************************************
FUNCTION:	FINDSC

PURPOSE:	Find the scenes which need to be transfered
		from a tape to the silo or disk.  This routine
		is the heart of the GET_SEEN application.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       04/94    T. Baltzer		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			ADAPS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"		/* LAS include			       */
#include "stitch.h"		/* Orbital stitch include file	       */


FUNCTION findsc(infp,strscene,endscene,tapeid,dest,sceneid,nscenes)

   FILE *infp;			/* Input file pointer			*/
   char strscene[];		/* Scene id at which to start processing*/
   char endscene[];		/* Scene id at which to end processing	*/
   char tapeid[];		/* Tape id of interest			*/
   char dest[];			/* Destination (DISK or SILO)		*/
   char sceneid[][17];		/* Scene id's on the tape of interest	*/
   long nscenes[];		/* Number of scenes on tape		*/
{


struct TAPESTR oneline;		/* One line of data			*/

long listflg = FALSE;		/* List flag, output file or not	*/
long status;			/* Return status			*/
long location;			/* Location of scene			*/
				/*   0 = undetermined			*/
				/*   1 = on disk in ADAPSORB		*/
				/*   2 = on the silo			*/
				/*   3 = apparently on tape		*/


nscenes[0] = 0;


/*
  If no starting scene was specified, then
  automatically try to list starting with first scene
*/
if (strcmp(strscene,"") == 0)
   listflg = TRUE;

/*
  Processing loop
*/
for (;;)
   {
   /*
     Get info from a line of data from the input file
   */
   status = line_info(infp,&oneline);
   if (status == E_FEND)
      break;

   /*
     If not listing and should start, set flag
   */
   if ((!listflg) && (strcmp(oneline.sceneid,strscene) == 0))
      listflg = TRUE;

   /*
     If the scene is within the range of starting scene and
     ending scene, and originates from the tape of interest,
     then find out if the scene is on disk, silo, or just tape.
     If the scene resides on tape only and the destination is
     the silo OR if the scene doesn't reside on the disk and the
     destination is the disk, then add this scene to 
     the list of scenes.
   */
   if ((listflg) && (strcmp(oneline.tapeid,tapeid) == 0))
      {
      scene_loc(&oneline,&location);
      if (((location == 3) && (strcmp(dest,"silo") == 0)) ||
          ((location != 1) && (strcmp(dest,"disk") == 0)))
         {
	 strcpy(sceneid[nscenes[0]],oneline.sceneid);
         nscenes[0]++;
         }
      }

   /*
     If this is the ending scene specified by the
     user, then quit.
   */
   if (strcmp(oneline.sceneid,endscene) == 0)
      break;
   }


return;
}
