/******************************************************************************
FUNCTION:	GET_SEEN

PURPOSE:	Generate a list of scenes which need to be transfered
		from a tape to the silo or disk.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       11/93    T. Baltzer		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			ADAPS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"		/* LAS include			       */
#include "stitch.h"		/* Orbital stitch include file	       */


main()
{
struct PARBLK   vblock;		/* TAE parameter block			*/

char dest[5];			/* Destination (DISK or SILO)		*/
char endscene[17];		/* Scene id at which to end processing	*/
char filnam[CMLEN];		/* Name of input file			*/
char strscene[17];		/* Scene id at which to start processing*/
char sceneid[20][17];	        /* Scene id's on the tape of interest	*/
char tapeid[7];			/* Tape id of interest			*/

long nscenes[1];		/* Number of scenes on tape		*/
long nonfatal = 1;		/* Non-fatal error status		*/
long std_out;			/* Standard output device 		*/

FILE *infp;			/* Input file pointer			*/


/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);


/*
  Get the input parameters
*/
getpar(&vblock,filnam,strscene,endscene,tapeid,dest);


/*
  Initialize the input file
*/
init_input(filnam,&infp);


/*
  Get all of the scenes for this tape
*/
findsc(infp,strscene,endscene,tapeid,dest,sceneid,nscenes);


/*
  Return scene id's back to tae.  If none were found,
  warn the user.
*/
if (nscenes[0] == 0)
   {
   c_errmsg("No scenes found","get_seen-noscn",&nonfatal);
   c_errmsg("Nonfatal error encountered","get_seen-warn",&nonfatal);
   }

taeland(&vblock,nscenes,sceneid);


/*
  Close the input file
*/
if ( fclose(infp) != E_SUCC )
   {
   c_errmsg("Error closing input file","get_seen-clos",&nonfatal);
   c_errmsg("Nonfatal error encountered","get_seen-warn",&nonfatal);
   }


/*
  Exit with a success status, no message
*/
z_exit(1,"");

}
