/*******************************************************************************
FUNCTION:       DAYS_AGO

PURPOSE:        Given a reference date and some previous date, determine
                how many days ago the previous date is from the reference date.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0       10/93    T. Baltzer          initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT                 LAS

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>              /* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"                /* LAS include                         */

FUNCTION days_ago(ref_date,prev_date,ndays)

char ref_date[];                /* Reference date MMDDYY                */
char prev_date[];               /* Date prior to reference date MMDDYY  */
long *ndays;                    /* Previous date is this number of days ago */
{

long day;                       /* day                                  */
long days_per_year;             /* days in a given year                 */
long month;                     /* month                                */
long year;                      /* year                                 */
long ref_year;                  /* Reference year                       */
long ref_jul;                   /* Reference julian day                 */
long prev_year;                 /* Previous year                        */
long prev_jul;                  /* Previous julian day                  */

char smonth[3];                 /* string version of month              */
char sday[3];                   /* string version of day                */
char syear[3];                  /* string version of year               */


/* Convert the input reference date to a Julian date
------------------------------------------------------*/
strncpy(smonth,&ref_date[0],2);
smonth[2] = '\0';
strncpy(sday,&ref_date[2],2);
sday[2] = '\0';
strncpy(syear,&ref_date[4],2);
syear[2] = '\0';

if (( sscanf(smonth,"%d",&month) == E_FAIL ) ||
    ( sscanf(sday  ,"%d",&day  ) == E_FAIL ) ||
    ( sscanf(syear ,"%d",&year ) == E_FAIL ))
        {
        c_lasmsg(FALSE,"halforb-dcon","Error converting date");
        c_lasmsg(FALSE,"halforb-fatal","Fatal error encountered");
        return(E_FAIL);
        }

c_grejul(&year,&month,&day,&ref_jul);
ref_year = year;


/* Convert the input previous date to a Julian date
-----------------------------------------------------*/
strncpy(smonth,&prev_date[0],2);
smonth[2] = '\0';
strncpy(sday,&prev_date[2],2);
sday[2] = '\0';
strncpy(syear,&prev_date[4],2);
syear[2] = '\0';

if (( sscanf(smonth,"%d",&month) == E_FAIL ) ||
    ( sscanf(sday  ,"%d",&day  ) == E_FAIL ) ||
    ( sscanf(syear ,"%d",&year ) == E_FAIL ))
        {
        c_lasmsg(FALSE,"halforb-dcon","Error converting date");
        c_lasmsg(FALSE,"halforb-fatal","Fatal error encountered");
        return(E_FAIL);
        }

c_grejul(&year,&month,&day,&prev_jul);
prev_year = year;


/* Determine the number of days between the two dates.  This
   is a three step process.
------------------------------------------------------*/
*ndays = 0;
day = prev_jul;
year = prev_year;


/* (1) Get the number of days in the previous year if the
   year is not the same as the reference year.
----------------------------------------------------------*/
if (c_4digityr(year) < c_4digityr(ref_year))
    {
    if (c_isleap(&year))
        days_per_year = 366;
    else
        days_per_year = 365;
    *ndays += days_per_year -  prev_jul;
    year = (year + 1) % 100;
    day = 0;
    }


/* (2) Get the number of days in the whole years which lie
   between the reference and the previous year.
------------------------------------------------------*/
for ( ;c_4digityr(year) < c_4digityr(ref_year); year = (year + 1) % 100)
    {
    if (c_isleap(&year))
        days_per_year = 366;
    else
        days_per_year = 365;
    *ndays += days_per_year;
    day = 0;
    }


/* (3) Get the number of days between the reference and
   previous dates which lie in the reference year.
------------------------------------------------------*/
*ndays += ref_jul - day;

return(E_SUCC);
}
