/******************************************************************************
NAME:                           GET_1ST_LINE 

PURPOSE:    Get the first line of data from the stitch input file
            (FOR THIS HALF ORBIT, it may be line 25 or whatever).

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      08/93          CSB           Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "stitch.h"             /* Orbital stitch include file         */


FUNCTION get_1st_line(filnam,linenum,oneline,infp)

   char filnam[CMLEN];          /* Half orbit file name                 */
   long linenum[];              /* Line number of read (1 relative)     */
   struct TAPESTR *oneline;     /* Structure with one line of info      */
   FILE **infp;                 /* LUT file pointer                     */

{

long i;                         /* counter                              */

char linebuf[CMLEN+1];          /* Buffer containing line of data       */


/* Open the input file
-----------------------*/
*infp = fopen(filnam,"r");
if (*infp == NULL)
    {
    c_lasmsg(FALSE,"halforb-open","Error opening input file");
    c_lasmsg(FALSE,"halforb-fatal","Fatal error encountered");
    return(E_FAIL);
    }

/* Read past the header information
------------------------------------*/
for (;;)
    {
    if ( fgets(linebuf,CMLEN+1,*infp) == NULL)
        {
        c_lasmsg(FALSE,"halforb-read","Error reading line in input file");
        c_lasmsg(FALSE,"halforb-fatal","Fatal error encountered");
        return(E_FAIL);
        }
    if ( strncmp(&linebuf[0],"tapeid",6) == 0 )
        break;
    }

/* Read the appropriate line, if NULL is returned,
   assume the EOF has been encountered. Don't
   print an error message.
   If the line of interest is a blank line,
   the next line will be returned.
--------------------------------------------------*/
for (i = 1; i <= linenum[0]; i++)
    {
    if (line_info(*infp,oneline) == E_FEND)
        return(E_FEND);
    }

return(E_SUCC);
}
