/*****************************************************************************
NAME:                   GETPAR

PURPOSE:                Retrieve HALFORB parameter information

ALGORITHM:

ALGORITHM REFERENCES:   none
*****************************************************************************/
#include <stdio.h>
#include "tae.h"
#include "worgen.h"
#include "adaps.h"


FUNCTION getpar(vblock,filnam,linenum,strscene,endscene,logfile,
                procflg,procopt) 

   struct PARBLK   *vblock;     /* TAE parameter block  */
   char   filnam[];             /* Name of input file   */
   long   linenum[];            /* Starting line number */
   char   strscene[];           /* Starting scene id    */
   char   endscene[];           /* Ending scene id      */
   char   logfile[];            /* Log file name        */
   long   procflg[];            /* Processing flag      */
   char   procopt[];            /* Processing option    */

{

long   access,                  /* Type of access for getnam call       */
element = 0,             /* Element number for getnam call       */
size,                    /* length of string                     */
status,                  /* Function return value                */
zero = 0;                /* Value of zero                        */

struct VARIABLE *p_find();      /* function to find a TAE variable      */
struct VARIABLE *var;           /* pointer to a TAE variable            */


/* Get the name of the log file
-------------------------------*/
access = IUPDATE;
status = c_getnam(vblock,"LOGFILE",&zero,logfile,"log",&access);
if (status != E_SUCC)
    return(status);


/* Get the name of the file to read and make
sure the file exists with read permission
--------------------------------------------*/
access = IREAD;
status = c_getnam(vblock,"INFILE",&element,filnam,"",&access);
if (status != E_SUCC)
    return(status);


/* Get the starting line number at which processing should begin
-----------------------------------------------------------------*/
var = p_find(vblock,"LINENUM");
linenum[0] = IVAL(*var,0);


/* Get the starting scene id, if specified
-------------------------------------------*/
var = p_find(vblock,"STRSCENE");
if (var->v_count == 0)
    {
    strcpy(strscene,"");
    }
else
    {
    strcpy(strscene,SVAL(*var,0));
    size = strlen(strscene);
    c_up2low(strscene,&size);
    }


/* Get the ending scene id, if specified
-------------------------------------------*/
var = p_find(vblock,"ENDSCENE");
if (var->v_count == 0)
    {
    strcpy(endscene,"");
    }
else
    {
    strcpy(endscene,SVAL(*var,0));
    size = strlen(endscene);
    c_up2low(endscene,&size);
    }


/* Get the processing flag
---------------------------*/
var = p_find(vblock,"PROCFLG");
procflg[0] = IVAL(*var,0);


/* Get the processing option
---------------------------*/
var = p_find(vblock,"PROCOPT");
strcpy(procopt,SVAL(*var,0));


return(E_SUCC);
}
