/******************************************************************************
FUNCTION:       HALFORB

PURPOSE:        Automate stitching of appropriate scenes into a half orbit.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0       01/93    T. Baltzer          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT                 ADAPS

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>              /* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"                /* LAS include                         */
#include "stitch.h"             /* Orbital stitch include file         */


main()
{
struct PARBLK   vblock;         /* TAE parameter block                  */

char procopt[7];                /* Processing option:                   */
/*      READ:    called by orbread      */
/*      STITCH:  called by orbstitch    */

long doneflg[1];                /* Done flag:  All half orbis processed?
							  	     Initialize to 1 (true) and return to  
                                     TAE only if value is 0 (false).    */

char endscene[17];              /* Scene id at which to end processing  */

char locate[MAX_PASS][5];       /* Location of scene:                   
								     UNKW - unknown     DISK - disk     
								     SILO - silo        TAPE - tape     */

char filnam[CMLEN];             /* Name of input file                   */
char lndflgs[MAX_PASS];         /* Line detection flag per input scene  */
char logfile[CMLEN];            /* Name of log file                     */
char orbflgs[MAX_PASS];         /* Orbital stitch flag per input scene  */
char orbitnm[17];               /* Orbit name of stitched output image  */
char sceneid[MAX_PASS][17];     /* Scene identifier                     */
char srclnk[MAX_PASS][7];       /* Data base source link                */
char strscene[17];              /* Scene id at which to start processing*/
char tapeid[MAX_PASS][7];       /* Tape identifier                      */

long elimflg;                   /* Elimination flag TRUE or FALSE       */

long linenum[1];                /* Line number to begin processing at   
								     This refers to valid data lines and 
								     blank lines are ignored, for instance*/

long nscenes[1];                /* Number of passes in the orbit        */

long procflg[1];                /* Processing flag:                     
								    0 - Starting scene id not yet found 
								        Processing has not begun yet    
								    1 - Starting scene id found, process
								    2 - Ending scene found, last orbit  */

long status;                    /* Return status                        */
long std_out;                   /* Standard output device               */


/* Initialize LAS
---------------------------*/
c_inlas(&vblock,&std_out);
doneflg[0] = 1;


/* Get the input parameters
----------------------------*/
status = getpar(&vblock,filnam,linenum,strscene,endscene,logfile,
procflg,procopt);
if (status != E_SUCC)
    c_lasmsg(TRUE,"halforb-fatal","Fatal error encountered");


/*------------------------------------------------------------------------------
Get the appropriate information to stitch the next specified orbit:
If an error occurs print a fatal error and quit.
If EOF, return an EOF flag = TRUE back to TAE and end.
If successful, and the number of passes is at least one:
return orbit values back to TAE and end.
If successful, but the number of passes is zero:
get the next half orbit of information.

This last possibility exists because a half orbit of scenes is
gotten and then reduced if:
- there are duplicates
- the start or end scene eliminates part of the half orbit
- the scene does not exist on the disk, silo, or tape
Thru this process of reduction, the scenes in the half orbit may all
be eliminated!
Finally, eliminat() is called to determine if the entire half
orbit should be eliminated, that is, not returned to TAE.
------------------------------------------------------------------------------*/
for ( ; ; )
    {
    status = proces(tapeid,sceneid,locate,srclnk,lndflgs,orbflgs,
    orbitnm,nscenes,linenum,procflg,strscene,endscene,
    filnam,logfile);
    if (status != E_SUCC)
        {
        if (status == E_FEND)
            {
            eof2tae(&vblock,doneflg);
            break;
            }
        else
            c_lasmsg(TRUE,"halforb-fatal","Fatal error encountered");
        }
    else
        {
        if (nscenes[0] > 0)
            {
            eliminat(procopt,sceneid,locate,nscenes,lndflgs,orbflgs,doneflg,
            &elimflg);
            if (elimflg)
                {
                if (procflg[0] == 2)     /* if eliminated and end scene found */
                    {                    /* then treat this like an eof       */
                    eof2tae(&vblock,doneflg);
                    break;
                    }
                else
                    {
                    continue;
                    }
                }
            taeland(&vblock,tapeid,sceneid,locate,srclnk,orbitnm,
            nscenes,linenum,doneflg,procflg);
            break;
            }
        }
    }

/* Exit with a success status, no message
------------------------------------------*/
z_exit(1,"");
}
