/******************************************************************************
NAME:                           LINE_INFO 

PURPOSE:    Get information from one line of data.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      07/93          CSB           Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "adaps.h"
#include "stitch.h"             /* Orbital stitch include file         */


FUNCTION line_info(infp,oneline)

FILE *infp;                     /* LUT file pointer                     */
struct TAPESTR *oneline;        /* Structure with one line of info      */

{

char linebuf[CMLEN+1];          /* Buffer containing line of data       */

long i;                 /* Counter                              */
long size = 16;         /* Size of scene ID                     */


/* Read a line, if NULL is returned, assume the EOF has been encountered.
   Don't print an error message.  If the line of interest is a blank line, 
   get the next line.
--------------------------------------------------*/
for (;;)
    {
    if ( fgets(linebuf,CMLEN+1,infp) == NULL)
        return(E_FEND);
    if (strcmp(linebuf,"\n") != 0)
        break;
    }


/* Parse out the tape id
---------------------------------------*/
strncpy(oneline->tapeid,&linebuf[0],6);
oneline->tapeid[6] = '\0';


/* Parse the scene id  AH11062392072142
AH                    HL
  11                  NOAA satellite number
    06                        month of pass
      23                      day of pass
        92            year of pass
          07          start time of pass (hours)
            21                start time of pass (minutes)
              42              start time of pass (seconds)
-----------------------------------------------------------*/
strncpy(&(oneline->sceneid[0]),&linebuf[7],2);
strncpy(&(oneline->sceneid[2]),&linebuf[37],2);
strncpy(&(oneline->sceneid[4]),&linebuf[10],2);
strncpy(&(oneline->sceneid[6]),&linebuf[13],2);
strncpy(&(oneline->sceneid[8]),&linebuf[16],2);
strncpy(&(oneline->sceneid[10]),&linebuf[19],2);
strncpy(&(oneline->sceneid[12]),&linebuf[21],2);
strncpy(&(oneline->sceneid[14]),&linebuf[23],2);
oneline->sceneid[16] = '\0';

for (i = 2; i < 16; i++)
    if (oneline->sceneid[i] == ' ')
        oneline->sceneid[i] = '0';        /* left justified implicit 0's */

c_up2low(oneline->sceneid,&size); 		  /* lower case the scene ID     */
        
        
/* Get the data base source link
-------------------------------------------*/
strncpy(oneline->srclnk,&linebuf[44],6);
oneline->srclnk[6] = '\0';


/* Get the acquisition stop time
-------------------------------------------*/
strncpy(oneline->stoptm,&linebuf[26],6);
oneline->stoptm[6] = '\0';
for (i = 0; i < 5; i++)
    if (oneline->stoptm[i] == ' ')
        oneline->stoptm[i] = '0';        /* left justified implicit 0's */
        
        
/* Get the status flags
-------------------------------*/
oneline->locflg = linebuf[51];
oneline->lndflg = linebuf[52];
oneline->orbflg = linebuf[53];


return(E_SUCC);
}
