/******************************************************************************
FUNCTION:       TAELAND
 
PURPOSE:        Send a list of scenes and associated data back to TAE land.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0       01/93    T. Baltzer          initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT                 ADAPS

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>              /* C standard I/O include file         */
#include <time.h>               /* C standard TIME include file        */
#include "las.h"                /* LAS include                         */
#include "stitch.h"             /* Orbital stitch include file         */


FUNCTION taeland(vblock,tapeid,sceneid,locate,srclnk,orbitnm,
                 nscenes,linenum,doneflg,procflg)

   struct PARBLK *vblock;       /* TAE parameter block                  */
   char tapeid[][7];            /* tape identifier                      */
   char sceneid[][17];          /* scene identifier                     */
   char locate[][5];            /* Location of scene                    */
   char srclnk[][7];            /* Data base source link                */
   char orbitnm[];              /* Orbit name of stitched output image  */
   long nscenes[];              /* number of passes in the orbit        */
   long linenum[];              /* Line number to begin processing at   */
   long doneflg[];              /* Done processing flag                 */
   long procflg[];              /* Processing flag                      */

{

char *sptr[MAX_PASS];           /* pointer array for updating vblock    */

long eofflg = 0;                /* EOF flag = FALSE                     */
long i;                         /* counter                              */


/* Output the string values to the variable block
--------------------------------------------------*/
for (i = 0; i < nscenes[0]; i++)
    sptr[i] = tapeid[i];

if (q_string(vblock,"TAPEID",nscenes[0],sptr,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

for (i = 0; i < nscenes[0]; i++)
    sptr[i] = sceneid[i];

if (q_string(vblock,"SCENEID",nscenes[0],sptr,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

for (i = 0; i < nscenes[0]; i++)
    sptr[i] = locate[i];

if (q_string(vblock,"LOCATE",nscenes[0],sptr,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

for (i = 0; i < nscenes[0]; i++)
    sptr[i] = srclnk[i];

if (q_string(vblock,"SRCLNK",nscenes[0],sptr,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

sptr[0] = orbitnm;
if (q_string(vblock,"ORBITNM",1,sptr,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");


/* Output the integer values to the variable block.  Output the
done flag only if it is 0 (false).
---------------------------------------------------*/
if (q_intg(vblock,"NSCENES",1,nscenes,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

if (q_intg(vblock,"LINENUM",1,linenum,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

if ((doneflg[0] == 0)&&(q_intg(vblock,"DONEFLG",1,doneflg,P_UPDATE) != SUCCESS))
        c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

if (q_intg(vblock,"PROCFLG",1,procflg,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

if (q_intg(vblock,"EOFFLG",1,&eofflg,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

if (q_out(vblock) != SUCCESS)
    c_lasmsg(TRUE,"halforb-rparm","Error returning parameter to TAE");

return;
}
