/*******************************************************************************
NAME:		GET_COMPOSITE_IN

PURPOSE:	Get the user specified input composite image names

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         REASON
-------    ----------      ----         ------
  1.0      T.Baltzer      04/00         Original Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   REANGLE must be run under TAE.

PROJECT:                        ADAPS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"	
#include "util.h"	
#include "adaps.h"	
#include "reangle.h"


void get_composite_in
(
    struct PARBLK *vblock,	/* IN:  Pointer to TAE parameter block	     */
    int which_band,		/* IN:  Which band has been specified	     */
    char parmname[],		/* IN:  Parameter name of image		     */
    int numbnd,			/* IN:  Number of bands in parmname	     */
    struct FILEINFO fileinfo[],	/* OUT: File information		     */
    char hostname[]		/* OUT: Host file name			     */
)

{

char hosin[NUM_IN_IMG][CMLEN],	/* Host image file name			     */
     taein[NUM_IN_IMG][CMLEN];	/* TAE image file names			     */

long	bands[1][MAXBND+1],	/* Bands specifications			     */
	element		=  0,	/* parameter element number		     */
	imglim		=  1,	/* maximum number of images to be processed  */
	nbands[1],		/* Number of bands in angle image            */
	nmrimg,			/* Number of input images                    */
	qflag[1][2],		/* flag indicating if subwindows specified   */
	same,			/* same window flag			     */
	totbnd,			/* Total number of input image bands	     */
	window[NUM_IN_IMG][4],	/* Input image window specification          */
	zero = 0;		/* Constant				     */

int	zero_rel_band,		/* Band index (zero relative)		     */
	i;			/* Counter				     */

struct DDR ddr;			/* DDR structure 			     */


/* Determine the band index (zero relative) for the band being processed
--------------------------------------------------------------------------*/
zero_rel_band = which_band - 1;


/* Retrieve and parse image specification 
--------------------------------------------*/
if (c_getin(vblock, parmname, &element, taein[zero_rel_band],
	    hosin[zero_rel_band], &nmrimg, window, bands, nbands,
	    &totbnd, &imglim, &same, qflag) != E_SUCC)
    c_clean(vblock,"reangle","",&zero);


/* Check for the valid number of input bands.
--------------------------------------------------*/
if (totbnd != numbnd)
    {
    c_lasmsg(FALSE,"reangle-badbnd",
    "Invalid number of input bands (%d) specified",totbnd);
    c_clean(vblock,"reangle","",&zero);
    }

/* Check for the valid number of input images
-------------------------------------------------*/
if (nmrimg != 1)
    {
    c_lasmsg(FALSE,"reangle-badimg",
    "Invalid number of input images (%d) specified",nmrimg);
    c_clean(vblock,"reangle","",&zero);
    }

/* Load fields within the file descriptor structure
-----------------------------------------------------------*/
fileinfo[zero_rel_band].nlines = window[0][NL];
fileinfo[zero_rel_band].nsamples = window[0][NS];
fileinfo[zero_rel_band].nbands = nbands[0];
for (i = 0; i < totbnd; i++ )
    fileinfo[zero_rel_band].bands[i] = bands[0][i];

strcpy(hostname,hosin[zero_rel_band]);


/* Make sure the composite is byte data (only type supported
   by this module.
-----------------------------------------------------------*/
if (c_getddr(hosin[zero_rel_band],&ddr) != E_SUCC)
    c_clean(vblock,"reangle","",&zero);

if (ddr.dtype != EBYTE)
    {
    c_lasmsg(FALSE,"reangle-type","Input image must contain BYTE data");
    c_clean(vblock,"reangle","",&zero);
    }

}
