/******************************************************************************
FUNCTION:	LOAD_MODEL

PURPOSE:	Initialize an AVHRR model for an acquisition.


PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0	   04/00    T. Baltzer		Initial development


PROJECT		1KM
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"

#define MAX_ACQ_MIN  15		/* Maximum acquisition minutes for a scene   */

int load_model
(
    char sceneid[],		/* IN:  Acquisition scene id		     */
    struct MODEL *model 	/* OUT: Model and orbital information	     */
)

{
long	status;			/* return status */
char	tmp2char[3];		/* Temporary string */


/* Determine what we can just from the scene identifier
   -----------------------------------------------------*/

if (sceneid[1] == 'G')			/* Data type GAC/LAC/HRPT */
    model->dtype = GAC;
else if (sceneid[1] == 'L')
    model->dtype = LAC;
else
    model->dtype = HRPT;


strncpy(tmp2char,&sceneid[2],2);		/* Satellite number */
tmp2char[2] = '\0';
model->satnum = atoi(tmp2char);

strncpy(tmp2char,&sceneid[4],2);		/* Acquisition month */
tmp2char[2] = '\0';
model->month = atoi(tmp2char);

strncpy(tmp2char,&sceneid[6],2);		/* Acquisition day */
tmp2char[2] = '\0';
model->day = atoi(tmp2char);

strncpy(tmp2char,&sceneid[8],2);		/* Acquisition year */
tmp2char[2] = '\0';
model->year = atoi(tmp2char);

strncpy(tmp2char,&sceneid[10],2);		/* Acquisition starting hour */
tmp2char[2] = '\0';
model->shour = atoi(tmp2char);

strncpy(tmp2char,&sceneid[12],2);		/* Acquisition starting minute*/
tmp2char[2] = '\0';
model->smin = atoi(tmp2char);

strncpy(tmp2char,&sceneid[14],2);		/* Acquisition starting second*/
tmp2char[2] = '\0';
model->ssec = atoi(tmp2char);

model->sfrac = 0;				/* Acquisition frac second    */


/*
  We will not worry about the NBS time correction because even if the
  lat/long is slightly out of the model's range, a zero or negative
  line/sample will be returned, which is fine for our purposes.

  Set the acquisition number of minutes to MAX_ACQ_MIN to make
  sure that this model spans the necessary timeframe.
*/
if (model->dtype == GAC)
    model->nlines = GAC_PER_SEC * 60 * MAX_ACQ_MIN;
else
    model->nlines = LAC_PER_SEC * 60 * MAX_ACQ_MIN;

/* Initialize the model
*/
status = c_initmod(&model->satnum,&model->year,&model->month,&model->day,
		   &model->shour,&model->smin,&model->ssec,&model->sfrac,
		   &model->dtype,&model->nlines,model);
if (status != E_SUCC)
    return(status);

return(E_SUCC);
}
