/*****************************************************************************
FUNCTION:	MY_TRANS_INIT

PURPOSE:	Initialize the transformation information.

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    04/00	T.Baltzer	Initial development, stole from GETATM

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:	ADAPS
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "reangle.h"
#include "adapssup.h"


int my_trans_init
(
    struct	DDR	*ddr,	/* IN:  image ddr structure		*/
    long	*proj_units,	/* OUT: projection units value		*/
    double	rot_coeffs[6] 	/* OUT: rotation coefficients		*/
)

{
double	all_zero[15] = {0.0};	/* zero projection parameters		*/
double	proj_coord[8];		/* corner coords in projection space	*/

long	degrees = DEGREE;	/* units code (degrees) for geo proj	*/
long	geo = GEO;		/* geographic projection flag		*/
long	image_coord[8];		/* corner coords in image space		*/
long	msgflag = TRUE;		/* flag for printing error messages	*/
long	prtprm[2];		/* print flags for c_transinit()	*/
long	zonenum = 62;		/* UTM projection zone number		*/



/* Check the unit code for coordinates and return an integer code for use
   in the Projection Transformation Package.
-----------------------------------------------*/
if (c_ckunit( ddr->proj_units, proj_units ) != E_SUCC)
    return E_FAIL;


/* Initialize the projection transformation
-----------------------------------------------*/
prtprm[0] = prtprm[1] = FALSE;
if ( c_transinit( &ddr->proj_code, proj_units, &ddr->zone_code,
             &ddr->datum_code, ddr->proj_coef, &geo, &degrees, &zonenum,
	     &ddr->datum_code, all_zero, prtprm, "\0" ) != E_SUCC )
    return E_FAIL;

/* Set up rotation coefficients in case the image
has been rotated.  In most cases we are using square
unrotated images but to be safe set this up anyway.
----------------------------------------------------*/
proj_coord[0] = ddr->upleft[0];
proj_coord[1] = ddr->upleft[1];
proj_coord[2] = ddr->loleft[0];
proj_coord[3] = ddr->loleft[1];
proj_coord[4] = ddr->upright[0];
proj_coord[5] = ddr->upright[1];
proj_coord[6] = ddr->loright[0];
proj_coord[7] = ddr->loright[1];
image_coord[0] = 1;            /* ul y */
image_coord[1] = 1;            /* ul x */
image_coord[2] = ddr->nl;      /* ll y */
image_coord[3] = 1;            /* ll x */
image_coord[4] = 1;            /* ur y */
image_coord[5] = ddr->ns;      /* ur x */
image_coord[6] = ddr->nl;      /* lr y */
image_coord[7] = ddr->ns;      /* lr x */
if ( c_rtcoef( proj_coord, image_coord, rot_coeffs, &msgflag ) != E_SUCC )
    return E_FAIL;


return E_SUCC;

}

