/****************************************************************************** NAME:                           PROCESS_LONG

PURPOSE:    Get scene acquisition information from one line of data
	    (long-format log file).

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  1.0      T. Baltzer      10/99          CSB           Original Development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:                        ADAPS

ALGORITHM:
   See in-line documentation.

   ALGORITHM REFERENCES:                none
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "adaps.h"
#include "reatmoinfo.h"


int process_long
(
    char linebuf[CMLEN+1],	/* INPUT:  Buffer containing line of data     */
    char datelist[][7],		/* INPUT:  List of dates in composite	      */
    int *ndays,			/* IN/OUTPUT: Number of days in date list     */
    int scenelist[], 		/* IN/OUTPUT:List of scenes used in composite */
    char periods[][CMLEN],  	/* INPUT:  Periods used in the composite      */
    int nperiods		/* INPUT:  Number of periods in 'periods'     */
)

{

char onedate[7];	/* One date in form MMDDYY		*/
char strindex[4];	/* String version of index		*/
char period[CMLEN];	/* Period in line buffer		*/
char sceneid[17];	/* 16 char scene id			*/

int i;                 /* Counter                              */
int index;		/* Index representing composite number	*/
int nscanned;		/* Number of items scanned from string	*/


/* Parse out the fields from the line buffer.  If and error is encountered
simply return, i.e. ignore bad lines in the input file.
-------------------------------------------------------------------------*/
nscanned = sscanf(linebuf,"%s %*s %s %s", period, strindex, sceneid);
if ( nscanned != 3 )
    return E_SUCC;


/* If the Date Index is "-1" then this scene was not included
   in the composite.  We are done considering this scene.
----------------------------------------------------------------*/
if (strcmp(strindex,"-1") == 0)
    return(E_SUCC);


/* If the Period Name in the log was not specified in PERIODS,
   then this scene was not included in the composite.  We are
   done considering this scene.
----------------------------------------------------------------*/
for (i = 0; i < nperiods; i++)
    {
    if (strncmp(period,periods[i],strlen(periods[i])) == 0)
	break;
    }
if (i == nperiods)
    return(E_SUCC);


/* If "not-a" appears as Date Index, then don't consider this line
   (often appears as last line in the log).
----------------------------------------------------------------------*/
if (strcmp(strindex,"not-a") == 0)
    return(E_SUCC);


/* This scene was used in the composite, extract the acquisition date
----------------------------------------------------------------------*/
strncpy(onedate,&sceneid[4],6);
onedate[6] = '\0';


/* Determine if this date has already been stored in the date list.  If it
   hasn't, store it in the date list.
--------------------------------------------------------------------------*/
for (i = 0; i < *ndays; i++)
    if (strcmp(onedate,datelist[i]) == 0)  break;
if (i == (*ndays))
    {
    strcpy(datelist[*ndays],onedate);
    (*ndays)++;
    }


/* Update the scene list to point to the appropriate entry in the date list
--------------------------------------------------------------------------*/
index = atoi(strindex);
if (index >= MAXSCOMP)
    {
    c_lasmsg(FALSE,"reatmoi-indx","Invalid scene index specified");
    c_lasmsg(TRUE,"reatmoi-fatal","Fatal error encountered");
    }
scenelist[index] = i;


return(E_SUCC);
}
