/******************************************************************************
FUNCTION:       TAELAND
 
PURPOSE:        Send information back to TAE land.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
 1.0       10/99    T. Baltzer          initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT                 ADAPS

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>              /* C standard I/O include file		*/
#include <time.h>               /* C standard TIME include file		*/
#include "las.h"                /* LAS include				*/
#include "reatmoinfo.h"         /* Local include			*/


void taeland
(
    struct PARBLK *vblock,	/* INPUT: TAE parameter block		*/
    char datelist[][7],		/* INPUT: date listing			*/
    int scenelist[],		/* INPUT: scene list			*/
    char ozonelist[][3],	/* INPUT: ozone date listing		*/
    int *ndays,			/* INPUT: number of days in datelist	*/
    int *nmonths,		/* INPUT: number of months in ozonelist	*/
    int *satnum			/* INPUT: Satellite number (14 = noaa-14)*/
)

{

char *sptr[MAXDAYS];           /* pointer array for updating vblock    */

int i;                         /* counter                              */


/* Output the string values to the variable block
--------------------------------------------------*/
for (i = 0; i < *ndays; i++)
    sptr[i] = datelist[i];

if (q_string(vblock,"DATELIST",*ndays,sptr,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"reatmoi-rparm","Error returning parameter to TAE");

for (i = 0; i < *nmonths; i++)
    sptr[i] = ozonelist[i];

if (q_string(vblock,"OZONELIST",*nmonths,sptr,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"reatmoi-rparm","Error returning parameter to TAE");


/* Output the integer values to the variable block.
---------------------------------------------------*/
if (q_intg(vblock,"NDAYS",1,ndays,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"reatmoi-rparm","Error returning parameter to TAE");

if (q_intg(vblock,"NMONTHS",1,nmonths,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"reatmoi-rparm","Error returning parameter to TAE");

if (q_intg(vblock,"SCENELIST",MAXSCOMP,scenelist,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"reatmoi-rparm","Error returning parameter to TAE");

if (q_intg(vblock,"SATNUM",1,satnum,P_UPDATE) != SUCCESS)
    c_lasmsg(TRUE,"reatmoi-rparm","Error returning parameter to TAE");


if (q_out(vblock) != SUCCESS)
    c_lasmsg(TRUE,"reatmoi-rparm","Error returning parameter to TAE");

}
