/*******************************************************************************
FUNCTIONS:	create_main_display
		getdrawingarea
		getlabelw

PURPOSE:	main display and widget routines

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: create_main_display.c,v $
Revision 1.4  2002/07/18 17:39:30  dlloyd
Periodic check in during development

Revision 1.2  2002/05/02 18:51:17  dlloyd
Modified the way the window widths are set.

Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <stdio.h>
#include <Xm/MainW.h>
#include <Xm/Label.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/ScrolledW.h>
#include "shoacqu.h"


/* Variables local to this file
-------------------------------*/
static Widget	drawing_area;	/* holds the image display		*/
static Widget	next_text;	/* the next scene text widget		*/


/*******************************************************************************
FUNCTION:	create_main_display

PURPOSE:	Create the main display window.

INPUT:		top level shell widget
		next scheduled acquisition time

OUTPUT:		none

RETURNS:	the main display widget
*******************************************************************************/
Widget create_main_display
    (
    Widget toplevel,		/* IN - top level widget		*/
    char *next_time		/* IN - next scheduled acquisition time	*/
    )
    {
    char	bgcolor[] = "black";/* window background color		*/
    char	fgcolor[] = "white";/* window foreground color		*/
    char	msg[STRLEN];	/* initial message text to display	*/

    Dimension	border_width;	/* width of the scroll bar border	*/
    Dimension	scrollbar_width;/* width of the scroll bar		*/
    Dimension	scroll_width;	/* width of the scroll bar		*/

    Display	*dpy;		/* pointer to the display		*/

    Widget	draw_area_form;	/* form to hold drawing area		*/
    Widget	label_form;	/* form to hold the next scene text	*/
    Widget	return_window;	/* main window widget to return		*/
    Widget	scroll_bar;	/* scroll bar widget			*/
    Widget	scroll_window;	/* scrolled window			*/

    XFontStruct	*font;		/* display text font structure		*/
    XmFontListEntry entry;	/* one entry for the font list		*/
    XmFontList	fontlist;	/* list of available fonts for display	*/


    /* Create the main window, this is the one that gets returned
    -------------------------------------------------------------*/
    return_window = XtVaCreateManagedWidget( WnMAINFORM,
    	xmFormWidgetClass, toplevel,
	NULL );
    

    /* Attach a form to the main window
    to hold the drawing area.
    -----------------------------------*/
    draw_area_form = XtVaCreateManagedWidget( WnDRAWFORM,
        xmFormWidgetClass, return_window,
	XmNleftAttachment, XmATTACH_FORM,
	XmNleftOffset, 2,
	XmNrightAttachment, XmATTACH_FORM,
	XmNrightOffset, 2,
	XmNtopAttachment, XmATTACH_FORM,
	XmNtopOffset, 2,
	XmNheight, DISPHEIGHT,
	XmNx, 2,
	XmNy, 2,
	NULL );
    
    /* Attach a form to the main window
    to contain the next scene text.
    -----------------------------------*/
    label_form = XtVaCreateManagedWidget( WnLABELFORM,
        xmFormWidgetClass, return_window,
	XmNheight, MSGHEIGHT-2,
	XmNleftAttachment, XmATTACH_FORM,
	XmNleftOffset, 2,
	XmNrightAttachment, XmATTACH_FORM,
	XmNrightOffset, 2,
	XmNbottomAttachment, XmATTACH_FORM,
	XmNbottomOffset, 2,
	XtVaTypedArg, XmNbackground, XmRString, bgcolor, strlen(bgcolor),
	XmNresizable, False,
	NULL );
    
    /* Create a scrolled window within the drawing form
    ---------------------------------------------------*/
    scroll_window = XtVaCreateManagedWidget( "scroll_window",
        xmScrolledWindowWidgetClass, draw_area_form,
	XmNheight, DISPHEIGHT,
	XmNmaximum, IMGHEIGHT,
	XmNincrement, 1,
	XmNscrollingPolicy, XmAUTOMATIC,
	NULL );

    /* Now that the scrolled window is available, get the width
    of the scroll bar and borders and set the window widths.
    -----------------------------------------------------------*/
    XtVaGetValues( scroll_window, XmNverticalScrollBar, &scroll_bar, NULL );
    XtVaGetValues( scroll_bar, XmNwidth, &scrollbar_width, NULL );
    XtVaGetValues( scroll_window, XmNspacing, &border_width, NULL );
    scroll_width = scrollbar_width + border_width * 3;
    XtVaSetValues( scroll_window, XmNwidth, IMGWIDTH + scroll_width, NULL );
    XtVaSetValues( draw_area_form, XmNwidth, IMGWIDTH + scroll_width, NULL );
    scroll_width = scrollbar_width + border_width * 4;
    XtVaSetValues( toplevel, XmNmaxWidth, IMGWIDTH + scroll_width, NULL );
    XtVaSetValues( toplevel, XmNminWidth, IMGWIDTH + scroll_width, NULL );

    /* Attach the drawing area to the scrolled window
    -------------------------------------------------*/
    drawing_area = XtVaCreateManagedWidget( WnDRAWAREA,
        xmDrawingAreaWidgetClass, scroll_window,
	XtVaTypedArg, XmNbackground, XmRString, bgcolor, strlen(bgcolor),
	XmNheight, IMGHEIGHT,
	XmNwidth, IMGWIDTH,
	NULL );
    
    /* Get a font for the display and set the initial text
    ------------------------------------------------------*/
    dpy = XtDisplay( toplevel );
    font = XLoadQueryFont( dpy,
        "-hrnet&hcr-grtex-bold-r-normal--42-300-100-100-p-271-iso8859-7" );
    if ( font == NULL )
	{
	font = XLoadQueryFont( dpy,
	    "-adobe-times-bold-r-normal--34-240-100-100-p-177-iso8859-1");
	}
    entry = XmFontListEntryCreate( "tag1", XmFONT_IS_FONT, font );
    fontlist = XmFontListAppendEntry( NULL, entry );
    XmFontListEntryFree( &entry );
    sprintf( msg, "Next acquisition at %s", next_time );

    /* Attach the next scene text widget to its form
    ------------------------------------------------*/
    next_text = XtVaCreateManagedWidget( WnNEXTTEXT, xmLabelWidgetClass,
        label_form,
	XtVaTypedArg, XmNlabelString, XmRString, msg, strlen( msg ),
	XtVaTypedArg, XmNbackground, XmRString, bgcolor, strlen(bgcolor),
	XtVaTypedArg, XmNforeground, XmRString, fgcolor, strlen(fgcolor),
        XmNalignment, XmALIGNMENT_CENTER,
	XmNfontList, fontlist,
	XmNx, 7,
	XmNy, 8,
	XmNheight, MSGHEIGHT,
	NULL );

    /* Manage the widgets
    ---------------------*/
    XtManageChild( draw_area_form );
    XtManageChild( label_form );
    XtManageChild( next_text );
    XtManageChild( drawing_area );

    /* Attach the label form to the drawing area form
    -------------------------------------------------*/
    XtVaSetValues( draw_area_form, XmNbottomWidget, label_form, NULL );

    XtManageChild( return_window );
    return return_window;
    }  /* END create_main_display */


/*******************************************************************************
FUNCTION:	getdrawingarea

PURPOSE:	Return the drawing area widget value

INPUT:		none

OUTPUT:		none

RETURNS:	Widget - drawing area widget value
*******************************************************************************/
Widget getdrawingarea( void )
    {
    return drawing_area;
    }  /* END getdrawingarea */


/*******************************************************************************
FUNCTION:	getlabelw

PURPOSE:	Return the label widget value

INPUT:		none

OUTPUT:		none

RETURNS:	Widget - label widget value
*******************************************************************************/
Widget getlabelw( void )
    {
    return next_text;
    }  /* END getlabelw */
