/*******************************************************************************
FUNCTION:	get_next_scene

PURPOSE:	To read from the socket and get the next scene

INPUT:		socket file descriptor

OUTPUT:		next scene 
		day night flag
		array of bands to display as rgb

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: get_next_scene.c,v $
Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include "shoacqu.h"


void get_next_scene
    (
    int  sock_fd,		/* IN - socket descriptor		*/
    int  *rgb_bands             /* OUT - array of bands to display rgb  */
    )
    {
    char buf[STRLEN];		/* buffer used to read from the socket	*/
    char dnflag;		/* day/night flag 'D', 'd', 'N' or 'n'	*/
    char filename[STRLEN];	/* input file name			*/
    char msg[STRLEN];		/* log message				*/

    int  msglength;		/* message length expected from parent	*/
    int  n;			/* number of bytes read from socket	*/


    /* If there is no socket connection, clear the file name and return
    -------------------------------------------------------------------*/
    if ( sock_fd == 0 )
        {
	putinfile( "" );
	return;
	}

    /* Read the first 3 bytes of the message.  This contains
    the ASCII representation of the message length to expect
    and convert the string to an integer.
    --------------------------------------------------------*/
    n = 0;
    while ( n < 3 )
        n += read( sock_fd, &buf[n], 3 );
    buf[n] = '\0';
    sscanf( buf, "%d", &msglength );
	
    /* read the message from the socket.  We are expecting
    a file name and a day/night flag.
    ------------------------------------------------------*/
    n = 0;
    while ( n < msglength )
	n += read( sock_fd, &buf[n], msglength );
    buf[n] = '\0';

    /* Get the values from the string and store them
    ------------------------------------------------*/
    sscanf( buf, "%s %c", filename, &dnflag );
    putinfile( filename );
    if ( dnflag == 'N' || dnflag == 'n' )
	putdaynightflag( NIGHT );
    else
	putdaynightflag( DAY );

    {
    int num;
    
    /* If RGB bands are sent retrieve them, otherwise
    set them to the default values.
    -------------------------------------------------*/
    num = sscanf( buf, "%*s %*c %d %d %d", &rgb_bands[0], &rgb_bands[1],
        &rgb_bands[2] );
    if ( num < 3 )
	{
	if ( dnflag == 'N' || dnflag == 'n' )
	    {
	    rgb_bands[0] = 3;
	    rgb_bands[1] = rgb_bands[2] = 3;
	    }
	else
	    {
	    rgb_bands[0] = rgb_bands[1] = 2;
	    rgb_bands[2] = 1;
	    }
	}
    }

    sprintf( msg, "Acquman sent: %s %c\n", filename, dnflag );
    write_log( FALSE, "get_next_scene", "msg", msg );
    }  /* END get_next_scene */
