/*******************************************************************************
FUNCTIONS:	get_visual
		getshift

PURPOSE:	visual information functions

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: get_visual.c,v $
Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include "shoacqu.h"


/* variables local to this file
-------------------------------*/
static struct SHIFT shift;


/*******************************************************************************
FUNCTION:	get_visual

PURPOSE:	Obtain the visual information for the display and create
		a graphics context for it.

INPUT:		pointer to the main display

OUTPUT:		RGB shift values

RETURNS:	visual information structure
*******************************************************************************/
XVisualInfo *get_visual
    (
    Display *display		/* pointer to the display		*/
    )
    {
    int		num_visuals;	/* number of visuals returned		*/

    XVisualInfo	*vinfo;		/* pointer to visual			*/
    XVisualInfo	vtemplate;	/* template for visual settings		*/


    /* Get the visual
    -----------------*/
    vtemplate.screen = DefaultScreen( display );
    vtemplate.class = TrueColor;
    vtemplate.depth = 24;
    num_visuals = 0;
    vinfo = XGetVisualInfo( display, VisualScreenMask |
        VisualDepthMask | VisualClassMask, &vtemplate, &num_visuals );
    if ( num_visuals == 0 )
        return NULL;
    
    /* Set the shift values
    -----------------------*/
    if ( vinfo->red_mask == 0xff0000 )
	shift.red = 16;
    else if ( vinfo->red_mask == 0xff00 )
	shift.red = 8;
    else if ( vinfo->red_mask == 0xff )
	shift.red = 0;

    if ( vinfo->green_mask == 0xff0000 )
	shift.green = 16;
    else if ( vinfo->green_mask == 0xff00 )
	shift.green = 8;
    else if ( vinfo->green_mask == 0xff )
	shift.green = 0;

    if ( vinfo->blue_mask == 0xff0000 )
	shift.blue = 16;
    else if ( vinfo->blue_mask == 0xff00 )
	shift.blue = 8;
    else if ( vinfo->blue_mask == 0xff )
	shift.blue = 0;

    return vinfo;
    }  /* END get_visual */


/*******************************************************************************
FUNCTION:	getshift

PURPOSE:	return the RGB shift values structure

INPUT:		none

OUTPUT:		none

RETURNS:	struct SHIFT - the RGB shift structure
*******************************************************************************/
const struct SHIFT *getshift( void )
    {
    return &shift;
    }  /* END getshift */
