/*******************************************************************************
FUNCTION:	open_socket

PURPOSE:	Open a socket connection with the parent process.  The socket
		is on the local machine.

INPUT:		parent port number

OUTPUT:		none

RETURNS:	int - socket descriptor

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: open_socket.c,v $
Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/utsname.h>
#include "shoacqu.h"


int open_socket
    (
    int port			/* IN - parent port number		*/
    )
    {
    int    sk;			/* socket descriptor			*/

    struct hostent *hp;		/* host name address structure		*/
    struct sockaddr_in skaddr;	/* socket address structure		*/
    struct utsname name;	/* host name structure			*/


    /* Create the socket
    --------------------*/
    if (( sk = socket( AF_INET, SOCK_STREAM, 0 )) < 0 )
        {
	write_log( TRUE, "open_socket", "sockerr", "Problem creating socket" );
	return FAIL;
	}
    
    /* Fill in the address structure
    --------------------------------*/
    if (( uname( &name )) < 0 )
        {
	write_log( TRUE, "open_socket", "host-name",
	    "Problem getting host name" );
	return FAIL;
	}
    skaddr.sin_family = AF_INET;
    if (( hp = gethostbyname( name.nodename )) != 0 )
	memcpy( &skaddr.sin_addr.s_addr, hp->h_addr, hp->h_length );
    else
        {
	write_log( TRUE, "open_socket", "host-lookup",
	    "Problem getting IP address" );
	return FAIL;
	}
    skaddr.sin_port = htons( port );

    /* Attempt to establish connection with the parent
    --------------------------------------------------*/
    if ( connect( sk, (struct sockaddr *)&skaddr, sizeof(skaddr)) < 0 )
        {
	write_log( TRUE, "open_socket", "connect",
	    "Problem connecting to parent" );
	return FAIL;
	}
    
    /* if all was well return the descriptor
    ----------------------------------------*/
    return sk;
    }  /* END open_socket */
