/*******************************************************************************
FUNCTION:	parse_args

PURPOSE:	To extract the command line arguments.

INPUTS:		argc, count of values in argv
		argv, input arguments

OUTPUT:		infile, input file name.  Switch -f
		next_text, next scheduled start time HH:MM [A|P]m.  Switch -next
		daynightflag, DAY or NIGHT.  Switch -dn
		port, the parent's port number.  Switch -port

RETURNS:	int, SUCCESS or FAIL

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: parse_args.c,v $
Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "shoacqu.h"


int parse_args
    (
    int  argc,		/* IN - count of arguments in argv		*/
    char *argv[],	/* IN - input argument array			*/
    char *infile,	/* OUT - input file name			*/
    char *next_text,	/* OUT - next scheduled start time		*/
    int  *daynightflag,	/* OUT - flag DAY or NIGHT			*/
    int  *port		/* OUT - parent's socket port number		*/
    )
    {
    int  dn_flag = FALSE;	/* flag: T/F did we get a daynightflag	*/
    int  infile_flag = FALSE;	/* flag: T/F did we get an infile	*/
    int  nexttext_flag = FALSE;	/* flag: T/F did we get next text	*/
    int  port_flag = FALSE;	/* flag: T/F did we get a port number	*/


    /* Extract the arguments
    ------------------------*/
    while ( *++argv != NULL )
        {
	if ( strcmp( *argv, "-f" ) == 0 )
	    {
	    argv++;
	    if ( *argv == NULL )
	        {
		write_log( FALSE, "parse_args", "missing-arg",
		    "Usage: -f <file name>" );
		}
	    else if ( *argv[0] == '-' )
	        {
		write_log( FALSE, "parse_args", "invalid-arg",
		    "Usage: -f <file name>" );
		}
	    else
	        {
		strcpy( infile, *argv );
		putinfile( infile );
		infile_flag = TRUE;
		write_log( FALSE, "parse_args", "-f", infile );
		}
	    }  /* END if -f */
	else if ( strcmp( *argv, "-next" ) == 0 )
	    {
	    argv++;
	    if ( *argv == NULL )
	        {
		write_log( FALSE, "parse_args", "missing-arg",
		    "Usage: -next <next start time HH:MM Xm>" );
		}
	    else if ( *argv[0] == '-' )
	        {
		write_log( FALSE, "parse_args", "invalid-arg",
		    "Usage: -next <next start time HH:MM Xm>" );
		}
	    else
	        {
		if ( strlen( *argv ) > STRLEN-1 )
		    {
		    strncpy( next_text, *argv, STRLEN-1 );
		    next_text[STRLEN] = '\0';\
		    }
		else
		    strcpy( next_text, *argv );
		nexttext_flag = TRUE;
		write_log( FALSE, "parse_args", "-next", next_text );
		}
	    }  /* END else if -next */
	else if ( strcmp( *argv, "-dn" ) == 0 )
	    {
	    argv++;
	    if ( *argv == NULL )
	        {
		write_log( FALSE, "parse_args", "missing-arg",
		    "Usage: -dn < [D|d|N|n] >" );
		}
	    else if ( *argv[0] == '-' )
	        {
		write_log( FALSE, "parse_args", "invalid-arg",
		    "Usage: -dn < [D|d|N|n] >" );
		}
	    else
	        {
		if ( *argv[0] == 'D' || *argv[0] == 'd' )
		    *daynightflag = DAY;
		else if ( *argv[0] == 'N' || *argv[0] == 'n' )
		    *daynightflag = NIGHT;
		else
		    {
		    write_log( FALSE, "parse_args", "invalid-arg",
		        "Usage: -dn < [D|d|N|n] > Using D as default" );
		    *daynightflag = DAY;
		    }
		putdaynightflag( *daynightflag );
		dn_flag = TRUE;
		write_log( FALSE, "parse_args", "-dn", *argv );
		}
	    }  /* END else if -dn */
	else if ( strcmp( *argv, "-port" ) == 0 )
	    {
	    argv++;
	    if ( *argv == NULL )
	        {
		write_log( FALSE, "parse_args", "missing-arg",
		    "Usage: -port <port number>" );
		}
	    else if ( *argv[0] == '-' )
	        {
		write_log( FALSE, "parse_args", "invalid-arg",
		    "Usage: -port <port number>" );
		}
	    else
	        {
		*port = atoi( *argv );
		port_flag = TRUE;
		write_log( FALSE, "parse_args", "-port", *argv );
		}
	    }  /* END else if -port */
	else
	    write_log( FALSE, "parse_args", "invalid-arg", *argv );
	}  /* END while *++argv */
    
    /* Do a little error checking.  If there were no
    arguments specified there is no reason to continue.
    ---------------------------------------------------*/
    if ( !infile_flag && !nexttext_flag && !dn_flag && !port_flag )
        {
	write_log( TRUE, "parse_args", "No-args",
	    "No arguments specified so I have nothing to do" );
	return FAIL;
	}
    
    if ( !port_flag )
        *port = -1;
    
    return SUCCESS;
    }  /* END parse_args */
