/****************************************************************************
FUNCTION:	post_dialog

PURPOSE:	A general purpose routine to popup a message dialog.  This
		is a MODELESS dialog and only waits for the user response
		no other action is taken.

INPUT:          widget of the caller		
		message to be displayed		
		the yes answer button label	

OUTPUT:         None

RETURNS:	None

PROJECT:	ADAPS, AVHRR Acquisition system

$Log: post_dialog.c,v $
Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

****************************************************************************/
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include "shoacqu.h"


void post_dialog
    (
    Widget   parent,		/* IN - widget of the caller		*/
    char     *question,		/* IN - message to be displayed		*/
    char     *ans		/* IN - the yes button label		*/
    )
    {
    static   Widget dialog;	/* dialog widget static to avoid dupes	*/

    XmString text;		/* text of the message			*/
    XmString yes;		/* text of the yes label		*/


    /* Create the dialog box if it isn't already created
    ----------------------------------------------------*/
    if ( !dialog )
	{
	dialog = XmCreateMessageDialog( parent, WnDIALOG2, NULL, 0 );
	XtVaSetValues( dialog,
	    XmNdialogStyle,	XmDIALOG_MODELESS,
	    NULL );
        /* unmanage all but one button */
	XtUnmanageChild( XmMessageBoxGetChild( dialog, XmDIALOG_HELP_BUTTON ));
	XtUnmanageChild( XmMessageBoxGetChild( dialog, XmDIALOG_CANCEL_BUTTON));
	XtAddCallback( dialog, XmNokCallback, response, NULL );
	}

    /* Set up the text strings
    --------------------------*/
    text = XmStringCreateLocalized( question );
    yes = XmStringCreateLocalized( ans );
    XtVaSetValues( dialog,
        XmNmessageString, text,
	XmNokLabelString, yes,
	NULL );
    XmStringFree( text );
    XmStringFree( yes );

    /* Realize the dialog
    ---------------------*/
    XtManageChild( dialog );
    XtPopup( XtParent( dialog), XtGrabNone );
    }  /* END post_dialog */



/*******************************************************************************
FUNCTION:	response

PURPOSE:	Respond to the user by removing the dialog box

INPUT:          widget of the caller		
		message to be displayed		
		the yes answer button label	

OUTPUT:         None

RETURNS:	None
*******************************************************************************/
void response
    (
    Widget   dialog,		/* IN - widget of the caller		*/
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    )
    {
    /* Remove the dialog from the display
    -------------------------------------*/
    XtPopdown( XtParent( dialog ));
    }  /* END response */
