#ifndef SHOACQUA_H
#define SHOACQUA_H


/* definitions
--------------*/
/* Note: for the linux server use 630.
If your're on a higher resolution display 1000 will work.
Need to figure out how to set this later
---------------------------------------------------------*/
#define DISPHEIGHT		1500

#define SUCCESS			0
#define FAIL			-1
#define SUBSAMPLE_VALUE         1
#define SAMPLES_PER_LINE        2048
#define NUMBANDS		5
#define BYTESPERSAMPLE          10  /* there are 5 bands per sample and 16 */
				    /* bits per band hence 10 bytes per    */
				    /* sample (or pixel if you prefer)     */
#define SUBSAMPLE_INTERVAL      (SUBSAMPLE_VALUE * BYTESPERSAMPLE)
#define LINESS_INT              SUBSAMPLE_VALUE
#define LINESIZE                22016
#define IMGHEIGHT               ( 5500 / SUBSAMPLE_VALUE )
#define IMGWIDTH                (SAMPLES_PER_LINE / SUBSAMPLE_VALUE)
#define IMGLINESIZE             (IMGWIDTH * 4)
#define IMGWINDOW               (IMGLINESIZE * DISPHEIGHT - 1)
#define MSGHEIGHT		65
#define MSGWIDTH		IMGWIDTH
#define WINHEIGHT		(DISPHEIGHT + MSGHEIGHT + 10)
#define WINWIDTH		IMGWIDTH
#define VIDEOPTR                1500
#define DAY                     1
#define NIGHT                   2
#define STRLEN			512
#define HEADER			512

#define FALSE			0
#ifndef TRUE
#define TRUE			(!FALSE)
#endif


/* Widget resource name definitions
-----------------------------------*/
#define WnSHOACQU		"Shoacqu"
#define WnMAINFORM		"mwform"
#define WnDRAWFORM		"drawingareaform"
#define WnLABELFORM		"nextlabelform"
#define WnDRAWAREA		"drawing_area"
#define WnNEXTTEXT		"next_text"
#define WnDIALOG2		"msgdialog"


/* structure definitions
------------------------*/
struct SHIFT
    {
    int	red;
    int green;
    int blue;
    };


/* Function prototypes
----------------------*/
#include <Xm/MainW.h>

Widget create_main_display
    (
    Widget toplevel,            /* IN - top level widget                */
    char *next_time             /* IN - next scheduled acqisition time  */
    );

Widget getdrawingarea( void );

Widget getlabelw( void );

void display_image
    (
    GC     gc,                  /* IN - the graphics context            */
    XImage *theimage,           /* IN - pointer to the display image    */
    Pixmap pixmap,              /* IN - background pixmap for display   */
    int    infd,                /* IN - input file descriptor           */
    int    daynightflag,        /* IN - flag DAY or NIGHT               */
    int    *rgb_bands,          /* IN - array of bands to display rgb   */
    const struct SHIFT *shift   /* IN - RGB shift values                */
    );

void get_next_scene
    (
    int  sock_fd,               /* IN - socket descriptor               */
    int  *rgb_bands		/* OUT - array of bands to display rgb	*/
    );

void get_next_time
    ( 
    int sock_fd                 /* IN - socket descriptor               */
    );

XVisualInfo *get_visual
    (
    Display *display            /* pointer to the display               */
    );

const struct SHIFT *getshift( void );

int openinput
    (
    char *filename              /* IN - input file name                 */
    );

int get_next_line
    (
    int         infd,           /* IN - input file descriptor           */
    char        *buf            /* OUT - buffer of image data to display*/
    );

int readline
    (
    int         infd,           /* IN - input file descriptor           */
    char        *buf            /* OUT - buffer of image data to display*/
    );

int open_socket
    (           
    int port                    /* IN - parent port number              */
    );

int parse_args 
    ( 
    int  argc,          /* IN - count of arguments in argv              */
    char *argv[],       /* IN - input argument array                    */
    char *infile,       /* OUT - input file name                        */
    char *next_text,    /* OUT - next scheduled start time              */
    int  *daynightflag, /* OUT - flag DAY or NIGHT                      */
    int  *port          /* OUT - parent's socket port number            */
    );

void putdaynightflag
    (
    int flag                    /* IN - flag DAY or NIGHT               */
    );

void putinfile
    (
    char *filename              /* IN - input file name                 */
    );

int getdaynightflag ( void);

void getinfile
    (
    char *filename              /* OUT - input file name                */
    );

void redraw_image
    (
    Widget    w,                /* IN - widget associated with callback */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

void start_display
    (
    GC     gc,                  /* IN - the graphics context            */
    XImage *theimage,           /* IN - pointer to the display image    */
    Pixmap pixmap,              /* IN - background pixmap for display   */
    int    *rgb_bands,          /* IN - arrays of bands to display rgb  */
    int    sock_fd              /* IN - socket file descriptor          */
    );

void socket_select( void );

void write_log
    (
    int  flag,                  /* IN - TRUE/FALSE fatality flag        */
                                /*      TRUE = issue fatal error message*/
    char *key,                  /* IN - error/message key               */
    char *code,                 /* IN - error/message code              */
    char *msg                   /* IN - message string                  */
    );

void post_dialog
    (
    Widget   parent,            /* IN - widget of the caller            */
    char     *question,         /* IN - message to be displayed         */
    char     *ans               /* IN - the yes button label            */
    );

void response
    (
    Widget   dialog,            /* IN - widget of the caller            */
    XtPointer client_data,      /* IN - client data                     */
    XtPointer call_data         /* IN - call data                       */
    );

int read_socket
    (
    int sock_fd,                /* IN - socket descriptor               */
    int rgb_bands[]             /* OUT - band combinations for display  */
    );

void getstoptime
    (
    char *stop_time             /* OUT - stop time                      */
    );

void putstoptime
    (
    const char *stop_time,      /* IN - stop time HH:MM:SS              */
    const char *am_pm_flag      /* IN - AM/PM flag                      */
    );

#endif
