/*******************************************************************************
FUNCTIONS:	getdaynightflag
		getinfile
		putdaynightflag
		putinfile
		redraw_image
		putstoptime
		getstoptime

PURPOSE:	Miscellaneous subroutines

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: shoacqusubs.c,v $
Revision 1.2  2002/08/26 13:02:21  dlloyd
Added support for displaying a stop time.

Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <string.h>
#include "shoacqu.h"


/* variables local to this file
-------------------------------*/
static char infile[STRLEN];	/* input file name			*/
static int  daynightflag;	/* day/night flag associated with infile*/
static char stoptime[12] = {0};	/* stop time to display HH:MM:SS        */


/*******************************************************************************
FUNCTION:	putdaynightflag

PURPOSE:	store the value of the day/night flag in a local variable

INPUT:		flag value, DAY or NIGHT

OUTPUT:		none

RETURNS:	none
*******************************************************************************/
void putdaynightflag
    (
    int	flag			/* IN - flag DAY or NIGHT		*/
    )
    {
    daynightflag = flag;
    }  /* END putdaynightflag */


/*******************************************************************************
FUNCTION:	putinfile

PURPOSE:	store the input file name in a local variable

INPUT:		file name

OUTPUT:		none

RETURNS:	none
*******************************************************************************/
void putinfile
    (
    char *filename		/* IN - input file name			*/
    )
    {
    strcpy( infile, filename );
    }  /* END putinfile */


/*******************************************************************************
FUNCTION:	getdaynightflag

PURPOSE:	return the value of the day/night flag that is stored in a
		local variable

INPUT:		none

OUTPUT:		none

RETURNS:	int - flag value
*******************************************************************************/
int getdaynightflag
    (
    void
    )
    {
    return daynightflag;
    }  /* END getdaynightflag */


/*******************************************************************************
FUNCTION:	getinfile

PURPOSE:	return the input file name that is stored in a local variable

INPUT:		none

OUTPUT:		file name

PROGRAM:	ADAPS, AVHRR Acquisition System
*******************************************************************************/
void getinfile
    (
    char *filename		/* OUT - input file name		*/
    )
    {
    strcpy( filename, infile );
    }  /* END getinfile */


/*******************************************************************************
FUNCTION:       putstoptime

PURPOSE:        store the stop time in a local variable

INPUT:          stop time HH:MM:SS
		AM/PM flag

OUTPUT:         none

PROGRAM:        ADAPS, AVHRR Acquisition System
*******************************************************************************/
void putstoptime
    (
    const char *stop_time,	/* IN - stop time HH:MM:SS		*/
    const char *am_pm_flag	/* IN - AM/PM flag			*/
    )
    {
    strcpy( stoptime, stop_time );
    strcat( stoptime, am_pm_flag );
    }  /* END putstoptime */


/*******************************************************************************
FUNCTION:	getstoptime

PURPOSE:	return the stop time that is stored in a local variable

INPUT:		none

OUTPUT:		stop time

PROGRAM:	ADAPS, AVHRR Acquisition System
*******************************************************************************/
void getstoptime
    (
    char *stop_time		/* OUT - stop time			*/
    )
    {
    strcpy( stop_time, stoptime );
    stoptime[0] = '\0';
    }  /* END getstoptime */
