/*******************************************************************************
FUNCTION:	start_display

PURPOSE:	Set up for a new display: open input file, clear old display
		and set 'Acquiring scene' message.  Call display.  Close
		input file on exit.

INPUT:		the graphics context
		pointer to the display image
		pixmap for the background display
		array of bands to display in rgb
		socket descriptor

OUTPUT:		none

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: start_display.c,v $
Revision 1.5  2002/08/26 13:02:21  dlloyd
Added support for displaying a stop time.

Revision 1.4  2002/08/26 12:16:22  dlloyd
Moved code to clear the display to the beginning of the function.  Added
a "Waiting for input file" message.

Revision 1.3  2002/07/18 17:39:31  dlloyd
Periodic check in during development

Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

*******************************************************************************/
#include <unistd.h>
#include "shoacqu.h"

extern Display     *display;

void start_display
    (
    GC     gc,			/* IN - the graphics context		*/
    XImage *theimage,		/* IN - pointer to the display image	*/
    Pixmap pixmap,		/* IN - background pixmap for display	*/
    int    *rgb_bands,          /* IN - arrays of bands to display rgb  */
    int    sock_fd		/* IN - socket file descriptor		*/
    )
    {
    char infile[STRLEN];	/* input file name			*/
    char *thebuf = theimage->data;/* pointer to the image data		*/
    char message[STRLEN];	/* message to display			*/
    char stop_time[12];		/* scene stop time			*/

    int  daynightflag;		/* flag DAY or NIGHT			*/
    int  i;
    int  infd;			/* input file descriptor		*/

    const struct SHIFT *shift;	/* RGB shift values			*/

    XmString text;		/* X string for widget text		*/


    /* Clear the display
    --------------------*/
    memset( thebuf, 0, IMGLINESIZE );
    for ( i = 0; i < IMGHEIGHT; i++ )
	XPutImage( XtDisplay( getdrawingarea()), pixmap,
	    gc, theimage, 0, 0, 0, i, IMGWIDTH, 1 );
    XCopyArea( XtDisplay( getdrawingarea()), pixmap,
        XtWindow( getdrawingarea()), gc, 0, 0, IMGWIDTH, IMGHEIGHT, 0, 0 );

    /* Change the message to 'acquiring scene'
    ------------------------------------------*/
    text = XmStringCreateLocalized( "Waiting for input file" );
    XtVaSetValues( getlabelw(), XmNlabelString, text, NULL );
    XmStringFree( text );

    /* Get the next scene information and open input file
    -----------------------------------------------------*/
    shift = getshift();
    daynightflag = getdaynightflag();
    getinfile( infile );
    if ( infile[0] == '\0' )
	{
	write_log( FALSE, "start_display", "nofile", "No file name specified" );
        return;
	}
    infd = openinput( infile );

    /* Change the message to 'acquiring scene'
    ------------------------------------------*/
    getstoptime( stop_time );
    if ( stop_time[0] == '\0' )
	strcpy( message, "Acquiring scene" );
    else
	strcpy( message, "Acquiring scene, stop time " );
    strcat( message, stop_time );
    text = XmStringCreateLocalized( message );
    XtVaSetValues( getlabelw(), XmNlabelString, text, NULL );
    XmStringFree( text );

    /* Do the display
    -----------------*/
    display_image( gc, theimage, pixmap, infd, daynightflag, rgb_bands, shift );

    /* Close and return
    if ( sock_fd != 0 )
	get_next_time( sock_fd );
    -------------------*/
    close( infd );
    }  /* END start_display */
