/*******************************************************************************
NAME:    GETPAR

PURPOSE: Get the user input required by the SILO application.

PROGRAM HISTORY:

VERSION  AUTHOR           DATE   REASON
-------  ---------------  -----  -----------------------------------------------
  1.0    T.Baltzer        04/93  Development.
  1.1    J.Fenno          04/94  Added the LINK and WAIT subcommands.


PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "silo.h"


int getpar
(
    struct PARBLK *vblock,	/* TAE parameter block                        */
    char subcmd[],		/* SILO subcommand                            */
    char infile[][CMLEN],	/* Input file specification                   */
    char outdir[],		/* Output directory specification             */
    char cache[],		/* Cache flag value                           */
    char link[],		/* Symbol link to create                      */
    long *lockflg,		/* Flag for ignoring locked files             */
    long *maxfiles,		/* Maximum number of files to return          */
    long *nfiles                /* Number of input files                      */
)
{
long fatal    = -1;		/* Fatal error flag                           */
long len;			/* Length of a string                         */
long nonfatal =  1;		/* Nonfatal error flag                        */

struct VARIABLE *val;		/* Pointer to TAE variable within vblock      */
long ii;                        /* loop counter                               */


/* Get the subcommand
----------------------*/
val = p_find(vblock, "_SUBCMD");
strcpy(subcmd, SVAL(*val,0));
len = (long)strlen(subcmd);
c_up2low(subcmd, &len);


/* Retrieve the parameters depending upon the subcommand
--------------------------------------------------------*/
if (strcmp(subcmd, "delete") == 0)
    {
    val = p_find(vblock, "INFILE");
    *nfiles = val->v_count;
    for (ii = 0; ii < *nfiles; ii++)
      {
      strcpy(infile[ii], SVAL(*val,ii));
      }
    }
else if (strcmp(subcmd, "dir") == 0)
    {
    val = p_find(vblock, "INFILE");
    strcpy(infile[0], SVAL(*val,0));
    *nfiles = 1;

    val = p_find(vblock, "LOCKFLG");
    if (strcmp(SVAL(*val,0), "YES") == 0)
	*lockflg = TRUE;
    else
	*lockflg = FALSE;

    val = p_find(vblock, "MAXFILES");
    *maxfiles = IVAL(*val,0);
    }
else if (strcmp(subcmd, "link") == 0)
    {
    val = p_find(vblock, "INFILE");
    strcpy(infile[0], SVAL(*val,0));
    *nfiles = 1;

    if (wildcard(infile[0]))
	{
	c_errmsg("Wildcard characters not permitted in INFILE",
	    "silo-wild", &nonfatal);
	if ( strchr( infile[0], '[' ))
	    c_errmsg("Use host filename syntax to eliminate square brackets",
		"silo-wild", &nonfatal);
	c_errmsg("Fatal error encountered", "silo-fatal", &fatal);
	}

    val = p_find(vblock, "LINK");
    strcpy(link, SVAL(*val,0));

    if (wildcard(link))
	{
	c_errmsg("Wildcard characters not permitted in LINK",
	    "silo-wild", &nonfatal);
	if ( strchr( infile[0], '[' ))
	    c_errmsg("Use host filename syntax to eliminate square brackets",
		"silo-wild", &nonfatal);
	c_errmsg("Fatal error encountered", "silo-fatal", &fatal);
	}
    }
else if (strcmp(subcmd, "read") == 0)
    {
    val = p_find(vblock, "INFILE");
    *nfiles = val->v_count;
    for (ii = 0; ii < *nfiles; ii++)
      {
      strcpy(infile[ii], SVAL(*val,ii));
      }

    val = p_find(vblock, "OUTDIR");
    if (val->v_count)
	strcpy(outdir, SVAL(*val,0));
    else
	c_cmhost("", outdir);  /* get cwd */

    val = p_find(vblock, "CACHFLG");
    if (strcmp("YES", SVAL(*val,0)) == 0)
	strcpy(cache, "-sv");
    else
	strcpy(cache, "-v");
    }
else if (strcmp(subcmd, "wait") == 0)
    {
    val = p_find(vblock, "INFILE");
    strcpy(infile[0], SVAL(*val,0));
    *nfiles = 1;

    if (wildcard(infile[0]))
	{
	c_errmsg("Wildcard characters not permitted in INFILE",
	    "silo-wild", &nonfatal);
	if ( strchr( infile[0], '[' ))
	    c_errmsg("Use host filename syntax to eliminate square brackets",
		"silo-wild", &nonfatal);
	c_errmsg("Fatal error encountered", "silo-fatal", &fatal);
	}
    }
else if (strcmp(subcmd, "write") == 0)
    {
    val = p_find(vblock, "INFILE");
    *nfiles = val->v_count;
    for (ii = 0; ii < *nfiles; ii++)
      {
      strcpy(infile[ii], SVAL(*val,ii));
      }

    val = p_find(vblock, "OUTDIR");
    if (val->v_count)
	strcpy(outdir, SVAL(*val,0));
    else
	strcpy(outdir, "");
    }
else
    {
    c_errmsg("Unknown subcommand", "programmer-error", &nonfatal);
    c_errmsg("Fatal error encountered", "silo-fatal", &fatal);
    }

/* Convert to host name 
-----------------------*/
if (strcmp(subcmd, "write") != 0)
    {
    for ( ii = 0; ii < *nfiles; ii++ )
	if ( siloname( infile[ii] ) != E_SUCC )
	    return E_FAIL;
    }


return(E_SUCC);
}
