/*****************************************************************************
FUNCTION:	SILONAME

PURPOSE:	To convert the input parameter to an acceptable file
		specification for hsm.

Version	Date	Programmer	Comments
-------	-----	-------------	---------------------------------------
  1.0	11/00	D.Lloyd		Initial development

PROJECT:	ADAPS
*****************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"


int siloname
(
char *infile		/* IN/OUT file name to convert			*/
)
{
char *lasdir_start;	/* pointer to beginning of las directory spec	*/
char *lasdir_end;	/* pointer to end of las directory spec		*/
char *charptr;		/* pointer to a character			*/
char hostfile[CMLEN];	/* host file name				*/
int  i;			/* counter					*/


/* If one of the las file delimeters is in the file name, convert
the las file name to a host file name.  The umXXX class of hsm
functions are used to communicate with the hsm system.
These functions require a host name.  Replace all las specifications
with their host equivalents.  Relative path names and user names
will not be expanded since there is no way to retrieve that
information.  In those cases, return an error.
-------------------------------------------------------------------*/
if (( strchr( infile, '[' ) || strchr( infile, ']' ) || strchr( infile, ';' ))
    && ( !strchr( infile, '/' )))
    {
    /* Point to the las directory token, if specified
    -------------------------------------------------*/
    lasdir_start = strchr( infile, '[' );
    lasdir_end = strchr( infile, ']' );

    /* Make sure both directory delimiters are specified
    ----------------------------------------------------*/
    if (( lasdir_start == NULL && lasdir_end != NULL ) ||
	( lasdir_start != NULL && lasdir_end == NULL ))
	{
	c_lasmsg(FALSE,"silo-filespec","Invalid LAS file specification");
	return E_FAIL;
	}

    /* Parse the directory specification
    ------------------------------------*/
    for ( charptr = lasdir_start, i = 0; charptr <= lasdir_end; charptr++, i++ )
        {
	switch ( *charptr )
	    {
	    case '[' :
	    case ']' :
	    case '.' :
	        hostfile[i] = '/';
		break;
	    case '-' :
	    case '~' :
	        c_lasmsg(FALSE,"silo-spec","Invalid specification");
		return E_FAIL;
	    default :
	        hostfile[i] = *charptr;
	    }
	}
    
    /* Now do the rest
    ------------------*/
    for ( charptr = lasdir_end + 1; *charptr != '\0'; charptr++, i++ )
        {
	switch ( *charptr )
	    {
	    case '.' :
	        hostfile[i] = '_';
		break;
	    case ';' :
	        hostfile[i] = '.';
		break;
	    default :
	        hostfile[i] = *charptr;
	    }
	}
    hostfile[i] = '\0';
    }

/* If there are no las delimeters use the file name as is
---------------------------------------------------------*/
else
    strcpy( hostfile, infile );


/* If /hsm is not at the beginning of the file specification, add it.
---------------------------------------------------------------------*/
if ( strncmp( hostfile, "/hsm", 4 ) != 0 )
    {
    sprintf( infile, "/hsm%s", hostfile );
    }

return E_SUCC;
}
