/*************************************************************************
FUNCTION:	movefiles

PURPOSE:	Move all the files associated with the input file
		to the destination directory.  The associated files
		have the same root and extensions of: tstamp, misc
		and lock.

INPUT:		input file name, default extension is sf2
		destination directory environment variable (no $)

OUTPUT:		Log messages for files moved

RETURNS:	SUCCESS
		FAIL

PROJECT:	ADAPS, AVHRR Aquisition System

$Log: movefiles.c,v $
Revision 1.3  2003/01/08 12:52:59  dlloyd
Clean up prior to production release.

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "timecorr.h"



int movefiles
    (
    char *input_file,		/* IN - name of file to be moved	*/
    char *destination_env	/* IN - environment var name of dest.	*/
    )
    {
    char *destination_dir;	/* destination directory name		*/
    char  new_filename[STRLEN];	/* new file name to be renamed		*/
    char  old_filename[STRLEN];	/* old file name			*/
    char  root_filename[STRLEN];/* input file without the extension	*/
    				/* or directory path			*/
    char  filename_noext[STRLEN];/* input file without the extension	*/
    char *strchr_ptr;		/* pointer into string			*/

    /* These are the current supported extensions always put the sf2	*/
    /* extension last because PUSHACQU won't start until it shows up	*/
    char *file_extensions[] = {"lock","tstamp","misc","sf2",NULL};
    char **next_file = (char **)file_extensions;

    struct stat filestat;	/* file status structure to determine	*/
    				/* if file exists			*/
    

    /* Get the destination directory
    --------------------------------*/
    destination_dir = getenv( destination_env );
    if ( !destination_dir )
        {
	write_log(TRUE,"movefiles","getenv","Unable to determine destination");
	return FAIL;
	}
    

    /* Get the full path name without an
    extension and the root file name.
    ------------------------------------*/
    strcpy( filename_noext, input_file );
    strchr_ptr = strrchr( filename_noext, '.' );
    if ( !strchr_ptr )
        {
	write_log(TRUE,"movefiles","strrchr","Couldn't find '.' in file name");
	return FAIL;
	}
    *strchr_ptr = '\0';

    strchr_ptr = strrchr( filename_noext, '/' );
    if ( !strchr_ptr )
        {
	write_log(TRUE,"movefiles","strrchr","Couldn't find '/' in file name");
	return FAIL;
	}
    strcpy( root_filename, strchr_ptr+1 );


    /* Process each file
    --------------------*/
    while ( *next_file != NULL )
        {
	/* build the old name */
	sprintf( old_filename, "%s.%s", filename_noext, *next_file );

	/* build the new file name */
	sprintf( new_filename, "%s/%s.%s", destination_dir, root_filename,
	         *next_file );

        /* if the file exists, rename it */
	if (( stat( old_filename, &filestat )) == 0 )
	    {
	    if (( rename( old_filename, new_filename )) != 0 )
	        write_log(FALSE,"movefiles","rename-err",old_filename);
	    else
	        write_log(FALSE,"movefiles","rename-succ",new_filename);
	    }
	
	/* go to the next extension */
	next_file++;
	}  /* END while */
    
    return SUCCESS;
    }  /* END movefiles */
