/*************************************************************************
FUNCTION:	process

PURPOSE:	Process the time stamp file and determine a mean time
		correction.

INPUT:		input file descriptor (open for read)

OUTPUT:		mean time correction

RETURNS:	mean time correction

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: process.c,v $
Revision 1.5  2003/01/08 13:02:34  dlloyd
Corrected undeclared variable.

Revision 1.4  2003/01/08 12:52:59  dlloyd
Clean up prior to production release.

*************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "timecorr.h"


int process
    (
    FILE *tstamp_file,		/* IN - open file descriptor		*/
    long *satime		/* OUT - first good satellite time	*/
    )
    {
    unsigned char buf[BUFSIZE]; /* input buffer                         */

    long  framesync;            /* frame sync time in milliseconds      */
    long  satellite;            /* satellite time in milliseconds       */

    int   num;                  /* number of bytes read from file       */
    int   numsamples;		/* count of valid samples collected	*/
    int   records;		/* loop control				*/
    int   satmil[SAMPLES];	/* array of satellite millisecond vals	*/

    long  milliseconds;		/* millisec from midnight_time to acquire_time*/
    long  microseconds;		/* microsecond that acquire got the record */
    time_t acquire_time;	/* Time that acquire got the record	*/
    struct tm *acquire_timestruct; /* time structure for acquire_time	*/
    struct tm midnight_timestruct; /* time structure for midnight	*/
    long seconds_since_midnight;/* seconds from midnight_time to acquire_time*/
    time_t midnight_time;	/* Unix midnight time (seconds)		*/
 


    numsamples = records = 0;
    /* Look for SAMPLES records but give up after 1000 tries
    --------------------------------------------------------*/
    while ( numsamples < SAMPLES && records++ < 1000 )
        {
	/* Get the time of the clock measurement taken at 
	the time this record was received by acquire.
	-------------------------------------------------*/
        fread( &acquire_time, sizeof(time_t), 1, tstamp_file );
        fread( &microseconds, sizeof(long), 1, tstamp_file );

	/* Get the rest of the record and, if
	it is a Video record, process the time.
	--------------------------------------*/
	num = fread( buf, sizeof(unsigned char), BUFSIZE, tstamp_file );
        if (( buf[0] == 5 ) && ( strcmp( &buf[1], "Video" ) == 0 ))
            {
            /* Compute milliseconds since midnight to the clock measurement
	    of this record.  This is done by filling a structure with the
	    acquire time and setting the time to 00:00:00; converting that
	    structure to seconds; computing the difference in seconds;
	    converting the difference to microseconds; and finally adjusting
	    for GM time.
            ---------------------------------------------------------------*/
            acquire_timestruct = localtime( &acquire_time );
            midnight_timestruct = *acquire_timestruct;
            midnight_timestruct.tm_sec = 0;
	    midnight_timestruct.tm_min = midnight_timestruct.tm_hour = 0;
            midnight_time = mktime( &midnight_timestruct );
            seconds_since_midnight = acquire_time - midnight_time;
            milliseconds = (seconds_since_midnight * 1000) +
                           (microseconds / 1000);
            milliseconds -= midnight_timestruct.tm_gmtoff * 1000;

	    /* Get the satellite and framesync time from the buffer
	    Framesync time is not used currently -- it will possibly
	    be used when GDP makes software modifications.
	    -------------------------------------------------------*/
            sscanf( &buf[11], "%ld %*c%*c %*d:%ld", &framesync, &satellite );
	    sscanf((const char *)&buf[27], "%ld", &satellite );

	    /* Calculate the difference between the satellite time
	    and the measurement taken when the frame was received.
	    Store this in the array of time differences and count
	    the sample.  Note: midnight traversal is not taken into
	    account.  There will be at most two occurances in any
	    given scene and they will be ignored when the weighted
	    mean is computed.
	    ------------------------------------------------------*/
            satmil[numsamples] = (int)(satellite - milliseconds);
	    numsamples++;

	    /* Diagnostic print ????
	    printf("satellite: %ld, framesync: %ld, diff: %d\n",
	        satellite, framesync, satmil[numsamples-1] );
	    ------------------------*/
	    }  /* END if */
	}  /* END while */

    
    /* Bail out if there aren't enough samples
    ------------------------------------------*/
    if ( numsamples < SAMPLES )
        return MEAN_ERROR;


    /* Sort the samples and return the weighted
    mean, weighted_mean() requires a sorted list.
    ---------------------------------------------*/
    insertion_sort( satmil, SAMPLES );
    return weighted_mean( satmil );
    }  /* END process */
