/*************************************************************************
FUNCTION:	savedelta

PURPOSE:	Save the delta time correction to a file.  This file is
		used during following timecorr executions to smooth the
		time correction output.

		Note: This function was taken from old vax code in
		acquire.c hence the cryptic naming conventions and
		the scarcity of comments.

IN:		satellite time
		measured delta
		calculated delta
		satellite ID

OUT:		delta file is updated
		or created if it doesn't exist

RETURNS:	none

PROJECT:	ADAPS, AVHRR Acqusition System

$Log: savedelta.c,v $
Revision 1.5  2003/04/17 19:46:20  dlloyd
Modified the way target is calculated slightly

Revision 1.4  2003/01/08 12:52:59  dlloyd
Clean up prior to production release.

*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "timecorr.h"


void savedelta
    (
    int satid,		/* IN - satellite ID, 15, 16 17 etc.		*/
    long startime,	/* IN - start time in milliseconds		*/
    int measured_delta,	/* IN - measured delta				*/
    int calc_delta	/* IN - calculated delta			*/
    )
    {
    FILE *old_deltas;		/* file descriptor of old deltas file	*/

    char *adapstables;		/* pointer to ADAPSTABLES		*/
    char old_deltas_name[STRLEN];/* file name of old deltas		*/

    int entries;		/* number of entries in file		*/
    int oldest;			/* index of oldest entry in file	*/
    long second;		/* index of second oldest entry in file	*/
    int millierr;
    int target;
    int i;

    adapstables = getenv( "ADAPSTABLES" );
    if ( adapstables == NULL )
        {
	write_log(FALSE,"savedelta","getenv","Error retrieving ADAPSTABLES");
	return;
	}
    sprintf( old_deltas_name, "%s/noaa%02d.delta", adapstables, satid );
    old_deltas = fopen( old_deltas_name, "r+" );
    if ( !old_deltas )
	{
        old_deltas = fopen( old_deltas_name, "w" );
	if ( !old_deltas )
	    {
	    write_log(FALSE,"savedelta","fopen","Error opening delta file");
	    return;
	    }
	entries = oldest = 0;
	}
    

    fscanf( old_deltas, "%d %d\n", &entries, &oldest );

    if ( entries >= MAXELE )
	{
        oldest = ( oldest + 1 ) % MAXELE;
	target = oldest -1;
	if ( target < 0 )
	    target += MAXELE;
	}
    else
        target = entries++;
    
    /* If the difference in calculated and measured delta is greater
    than SATIME1K_THRESH, restart the time series.  This helps to
    prevent mixing deltas after a clock correction.
    ----------------------------------------------------------------*/
    if ( abs( calc_delta - measured_delta ) > SATIME1K_THRESH )
        {
	entries = oldest = 0;
	target = entries++;
	write_log(FALSE,"savedelta","threashold","Resetting the time series");
	}
    
    rewind( old_deltas );
    fprintf( old_deltas, "%02d %02d\n", entries, oldest );

    for ( i = 0; i < target; i++ )
        {
	fscanf( old_deltas, "%ld %d\n", &second, &millierr );
	}
    
    fprintf( old_deltas, "%9ld %9d\n", startime, measured_delta );
    fclose( old_deltas );
    }  /* END savedelta */
