/*************************************************************************

	timecorr.h

$Log: timecorr.h,v $
Revision 1.8  2006/07/31 19:26:44  dlloyd
Had to increase the reasonable value to 5000.  NOAA is not limiting the
clock drift to 1000 as originally planned.

Revision 1.7  2003/12/24 19:07:54  dlloyd
Modified to move one file at a time in case errors occur.  On errors the
image is no longer moved to error.  Fixed a bug in th savedelta routine.

Revision 1.6  2003/04/18 12:29:22  dlloyd
Put in a sanity check for reasonable value (1000ms)

Revision 1.5  2003/03/04 13:14:29  dlloyd
Changed propagation delay from 191 to 205 because of changes to the
framesync hardware which increased the delay.

Revision 1.4  2003/01/08 12:52:59  dlloyd
Clean up prior to production release.

*************************************************************************/
#ifndef TIMECORR_H
#define TIMECORR_H


#define TRUE       1
#define FALSE      (!TRUE)
#define SUCCESS    0
#define FAIL       -1


#define BUFSIZE		36	/* size of the time stamp buffer	*/
#define STRLEN		256	/* standard string length		*/

/* The following flags must be large values so they can't be mistaken for
actual time differences.
------------------------------------------------------------------------*/
#define MEAN_ERROR      -9999   /* flag: unable to compute a mean value	*/
#define TOOFEWENTRIES	-8888	/* flag: too few entries in delta file	*/
#define REASONABLE_VALUE 5000	/* max reasonable value for the delta	*/

#define SAMPLES		50	/* number of time stamp records that	*/
				/* we must have for processing		*/ 
#define MIN_MEAN_COUNT  5       /* must have at least this number of    */
				/* values in the weighted mean		*/
#define STDDEV_THRESH   1.0     /* stdev must be <= to this value       */
				/* Also for weighted mean		*/

#define ORDER		1	/* order of the regression to perform	*/
#define MAXELE		20      /* MAXIMUM no. of data points in the	*/
				/* delta error databases		*/
#define SATIME1K_THRESH	50	/* maximum difference between measured	*/
				/* and calculated time corrections	*/

#define TWENTYMINUTES   1200	/* number of seconds in twenty minutes	*/

/* The following value was based on empirical data.  The output of the
function, process(), was compared to the time correction values output
by the predecessor to the current acquisition system, vaxh acquire().
The propagation delay on vaxh was one line length or 166ms.  No 
documentation was found for this choice.  The best we can do here is
take the mean difference between the two systems.
Before the software modifications to the framesyc the propdelay was 191.
Those modifications changed the delay to 205 (mean of 300 measurments)
------------------------------------------------------------------------*/
#define PROPDELAY       205	/* propagation delay to be added to	*/
				/* measured time correction		*/


/* Function prototypes
----------------------*/
void write_log
    (
    int  flag,                  /* IN - TRUE/FALSE fatality flag        */
                                /*      TRUE = issue fatal error message*/
    char *key,                  /* IN - error/message key               */
    char *code,                 /* IN - error/message code              */
    char *msg                   /* IN - message string                  */
    );


int movefiles
    (
    char *input_file,           /* IN - name of file to be moved        */
    char *destination_env       /* IN - environment var name of dest.   */
    );


#include <stdio.h>
int process
    (
    FILE *tstamp_file,          /* IN - open file descriptor            */
    long *satime
    );

float calc_stdev( int satmil[], int first, int last, float *mean );
int weighted_mean( int satmil[] );
void insertion_sort( int array[], int n );

int calcdelta
    (
    int measured_delta,         /* IN - measured delta value            */
    long startime,              /* IN - start time in seconds           */
    int satid                   /* IN - satellite ID, 15, 16 17 etc.    */
    );

void savedelta
    (
    int satid,          /* IN - satellite ID, 15, 16 17 etc.            */
    long startime,      /* IN - start time in milliseconds              */
    int measured_delta, /* IN - measured delta                          */
    int calc_delta      /* IN - calculated delta                        */
    );

long makeseconds( char *root_filename );

int move1file
    (
    char *input_file,		/* IN - name of file to be moved	*/
    char *destination_env	/* IN - environment var name of dest.	*/
    );
#endif
