#!/bin/csh
#
# FUNCTION: Upnav
#
# PURPOSE: Disseminate the ephemeris data files to all systems.
#          This function is performed in a shell script because
#          in a pdf the ADAPSTABLES variable gets expanded on
#          the host system instead of the target system (the
#          rightmost occurrence of ADAPSTABLES in the remote
#          copy commands below must get expanded on the target
#          system).
#
# CALLING SYNTAX:
#
# 	   upnav.csh "host name"
#
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# T. Baltzer       09/94  Development
# J. Fenno         01/95  Updated to include all systems.
# J. Fenno         04/95  Updated to include all systems.
# T. Baltzer       08/96  Remove references to sg2 since it has gone away.
# T. Baltzer       04/97  Remove references to dg5 since it has gone away.
#			  Return error or success status instead of always
#			  success.
# D.Etrheim        04/97  Change sun1 to edcsns1
# T.Baltzer        07/98  Add edcsgs18, put into a foreach loop
# D.Lloyd          11/98  Added edcsgs20 to the ALLSYSTEMS variable
# K.Vanumamalai    07/09  Added edclxs100 to the ALLSYSTEMS variable


# We currently have LAS/ADAPS installed on the systems defined by ALLSYSTEMS:
# -------------------------------------------------------------
set ALLSYSTEMS = "edcsgs20 edclxs100 edcsns1"


# Initialize the return status to "success".  If an error does occur during a
# remote copy, this return status will be changed to a "failure" status.
# -------------------------------------------------------------
set returnstat = 0


# Propagate the files to each of the systems (except the host system).
# -------------------------------------------------------------
foreach onesystem ($ALLSYSTEMS)
    if ($1 != $onesystem) then
 	scp $ADAPSTABLES/[012]*.dat $onesystem\:'$ADAPSTABLES'  || set returnstat = 1
    endif
end

# New systems that do not allow rcp
#----------------------------------
set NEWSYSTEMS = "edclxs64 edclxw100"
foreach onesystem ( $NEWSYSTEMS )
    scp $ADAPSTABLES/[0123]*.dat adaps@$onesystem\:'$ADAPSTABLES'
    if ( $status ) then
        set returnstat = 1
    endif
end


echo "End of UPNAV shell script"

exit $returnstat

