/****************************** ACQUIRE.C ***********************************/
/* Modification history for entire package resides here due to most changes
   being made in this module.  This modification history does not reflect
   changes prior to 7/24/89.

07/24/89 TAB	Modified ACQUMAN.H and ACQUSUBS.C to change the size of an HDT
		file from 1500 to 2500 feet so that long (15 minute) passes
		don't cause ACQUIRE to 'get lost' due to exceeding 1500 foot
		file limit.

07/25/89 TAB	Added logicals to point to HDT/ACU/RCV/RIPS terminal ports,
		making ACQUIRE portable.

07/27/89 TAB	Fixed logic problems in stoptape() caused by 7/24 change.  Also
		fixed spaghetti code, added logic to skip a file if we have
		written past EOF on previous pass and to wrap to file 10 at EOT.

07/28/89 TAB	Removed extraneous code in error detection after lowering
		priority.  Now, anytime we return to ACQUMAN with a NOTOK status
		we know that it is because we couldn't open the disk file.
		Also modified ACQUMAN to change the pass to go to tape whenever
		there are problems opening the disk file (probably full disk).
		Also added code to bufopen() to check for diskspace before
		trying to open, as a disk open can take a long time to fail if
		the disk is fragmented or has erase-on-delete enabled.

07/28/89 TAB	Finally fixed elusive ACCVIO problem!  If ACQUIRE returns to
		ACQUMAN without cancelling cyclic input from BCA0:, the iosb
		is deallocated from stack but BCDRIVER keeps writing it!  Added
		code in ACQUIRE() to cancel cyclic input and close the channel
		and made isb global.

07/31/89 TAB	Reduced sleeptime in outpipe from 2 sec. to 1 sec. as engineers
		have corrected "ringing" problem.

09/08/89 TAB	Fixed bug in savedelta which caused extra spaces in SATIMERR.Nnn
		files.

09/11/89 TAB	Modified Acquire to always use computed SATIMERR.Nnn averages
		for delta time corrections, even for dish to disk, per J. Boyd.

09/11/89 TAB	Fixed bug in finddelta() which would corrupt memory and cause
		bogus errors.  A -1 subscript was set, but missing continue
		statement caused it to be used before being incremented.

09/15/89 TAB	Modified finddelta() to prevent 0 delta times from being saved
		to the SATIMERR.Nnn files.  (not 0, actually 167 adjustment).

01/04/90 TAB	Modified outpipe() in machdpnd so that we now close BCA0:!
		Multiple open channels had been accumulating, eventually
		hanging acquire (and possibly the system?).

04/11/90 TAB	Removed code which did bytewise move of data from input buffers
		to output buffers.  Changed bufopen, bufrit and buflsh to que
		disk writes directly from input buffers.  Added code to time
		disk writes and report excessive times and lost data.

04/12/90 TAB	Replaced RIPS quicklook display with Peritek.  Modified
		vidsetup, videout and deleted vidclose.

05/03/90 TAB	Added code to suspend process "Ingest1" on EROSH at beginning
		of pass and to resume it again at the end.

05/04/90 TAB	Modified most of the package to accomodate new schedule.  Note
		that mode corresponds to wo.opcode

05/11/90 TAB	Changed elevated priority to 15.  Also, resume auto ingest
		after we close this image.

05/18/90 TAB	Changed to allow two additional schedule fields and name changes

06/01/90 TAB	Modified videout() to mirror ascending passes.  Also fixed bug
		in suspend of auto_ingest (had lower case name).  Implemented
		acquire on new Domsat system.

06/04/90 TAB	Added sanity check to delta time calculation to prevent
		excessive adjustments due to error.

06/11/90 TAB	Modified acquman mainline so that setupipe() is called just
		before pass.  Also added code to send messages to Wallops
		inhale process so that it will give up the 760 when acquire
		needs it.  Also modified outpipe() to eliminate sleep.

06/19/90 TAB	Fixed bug in ingest suspend/resume.

06/26/90 TAB	Added detection of frame shift syndrome.  Optimized videout()
		to reduce cpu overhead.  Fixed bug which was introduced
		04/11/90 which caused records not to be overwriten (due to
		timestamp).  Added code to save delta times to logicals so
		that INHALE processes can use them.

07/26/90 TAB	Modified acquire() to create output file in [ACQUIRE]
		directory, and then rename it to [IMAGES] when done.  This
		is to prevent problems with ingest reading the image too soon.

08/13/90 TAB	Finally fixed colors with new color map!  Also cleaned up code
		in videstup() and videout().

09/04/90 TAB	Modified acquire(), acquman() and droplog() to increase sceneid
		to 16 characters and to change proclog name.  Modified to change
		protection of output file to (O:RWED,G:RWED,W:RE).  Also removed
		last vestiges of suspnd() and resume() which suspended all
		processes on the system.

10/09/90 TAB	Modified nbstime() to limit retries to read NBS clock and to
		close channel upon error.

10/17/90 TAB	Added code for text display "Next acquisition at hh:mm ?M"

10/23/90 TAB	Modified acquman and quicklook_text() to remember last message
		and skip it if it already up on the display.  Also modified to
		display "Next acquisition pending..." for HDT->DISK and to
		display "Next acquisition unavailable" for DISH->HDT.

05/09/91 TAB	Modified acquman and all necessary subroutines to provide
		support for NOAA-12.  Also fixed bug which could cause overwrite
		of header record.

05/26/92 TAB	Modified header() in acquire to create new header information
		needed for the 1k system.  Info is at offset 256, and old
		header info is unchanged but may be removed later.  Also
                changed output filename to reflect new station naming.

09/14/92 TAB	Added logs for calculated delta times for use in 1k system.

01/08/93 TAB	Fixed problem caused by 09/14/92 mod which caused the address
		of delta to be logged instead of delta whenever calculated
		delta was 0.	

01/12/93 TAB	Added code to suspend and resume netcopy.

06/28/93 TAB	Modified to suspend both netcopy procs and to allow suspends
		to queue.  Also modified so that priority is raised prior to
		suspending and lowered after resuming.

07/02/93 TAB	Fixed problem in acquman() where passes which aborted using
		the ABORT macro were causing WALLOPS to remain dormant until
		the next successful acquire pass.

09/20/93 TAB	Changed setuppipe so that if errors occur, they are displayed
		and it retries up to 5 times with a 2 second sleep each time.

11/14/94 TAB	Modified vidsetup() to reverse NOAA 9

01/09/95 TAB	Modified acquman and all necessary subroutines to provide
		support for NOAA-14.

08/05/98 TTB	Modified acquman and all necessary subroutines to provide
		support for NOAA-15.

09/04/98 DEL	Added an event timer to prevent acquire() from waiting for
		event flag EFDAT beyond the duration of the scheduled pass.
		Also, when duration is exceeded, the image file is removed
		and the mode is changed to 0 indicating the scene was missed.
		Mode was changed to pass by reference.

08/02/99 DEL	Modified acquman and all necessary functions for y2k
		compliance.  Defined a global year offset to be applied to
		the year in case the system clock needs to be set to some
		year other than the current year.  Removed the calls to
		localtime() and replaced with a new function get_year() that
		uses system time to get the year.  Localtime() failed y2k
		testing and should not be used.

09/26/00 DEL	Modified acquman and all necessary subroutines to provide
		support for NOAA-16.

10/03/01 DEL	Corrected the way that time corrections were determined.  A
		technique of finding the mean of the largest number of
		measured time differences with the lowest standard deviation
		was implemented.  This give a greater confidence that the
		measured difference is the true difference.  Another check
		was added that compares the calculated difference with the
		measured difference and, if the calculated difference is not
		within reason, the measured difference is used instead.
		Replaced the old bubble sort with an insertion sort.  See
		CR 5258

04/08/02 DEL	Modified acquman and all necessary subroutines to provide
		support for NOAA-17.
*******************************************************************************/
#include	<math.h>
#include	"acquman.h"
#include	<rms.h>

#define	SATLOSS	10
#define	SCH$C_SUSP	9
#define	SCH$C_SUSPO	10

int 	start_hour, start_minute, start_second, start_mil, start_julian;
int	start_month, start_dom, start_year;
int	alarm, timex[2] = {-100000000,-1};
short	isb[4];			/* Must be global, because if we exit this
				   routine, BCDRIVER keeps writing it!!! */

/* This stuff is for event timing */
unsigned int shift = 10000, avg_1, avg_2;
unsigned short	elapsed[7500][2];
unsigned int	start_time[7500][2],stop_time[2],elapsed_time[2];
unsigned int	elapsed_millisec,cnt;
unsigned int	lcl_elapsed_time[2];
int	overwrite;

int	measured_delta;
char date_time[30];

acquire(mode,satid,startime,stoptime,sceneid,station)
int	*mode,satid, startime, stoptime;
char	sceneid[],station[];
/*
==========================================================================
==									==
==	This is the routine that will generate the High Resolution 	==
==	Picture Transmission (HRPT) disk file.  This routine will 	==
==	open the disk file and begin polling the frame buffer until 	==
==	data reception begins.  Then, minor frames will be read and 	==
==	the timecodes will be monitored until the start time occurs. 	==
==	Then the data will be copied from the frame buffer to the 	==
==	disk with time stamps added.  This will continue until the 	==
==	stop time is reached, or the signal is no longer available.	==
==	Then a header record is written at the beginning of the file,	==
==	the file is closed, and this routine terminates, returning	==
==	control to the Acquisition Manager.				==
==									==
==========================================================================
*/
{
unsigned char acmode[1] = '\2';

$DESCRIPTOR(n09_logical, "N09");
$DESCRIPTOR(n10_logical, "N10");
$DESCRIPTOR(n11_logical, "N11");
$DESCRIPTOR(n12_logical, "N12");
$DESCRIPTOR(n14_logical, "N14");
$DESCRIPTOR(n15_logical, "N15");
$DESCRIPTOR(n16_logical, "N16");
$DESCRIPTOR(n17_logical, "N17");
$DESCRIPTOR(table, "LNM$SYSTEM_TABLE");
char	equiv[25];
int	crelnm_attributes = 512;

struct {
	unsigned short	a_buffer_length;
	unsigned short	a_item_code;
	int		*a_buffer_address;
	short		*a_return_length_address;

	unsigned short	s_buffer_length;
	unsigned short	s_item_code;
	char		*s_buffer_address;
	short		*s_return_length_address;

	int	end;
	} crelnm_itmlst = {4, 3, &crelnm_attributes, 0,
		   0, 2, &equiv, 0, 0};
short	stamp[17];
int	sequence, bad, now;

int	i, j, dumy, pid, status, state;
int	records, orbit, mactime[2], defpri;

int	filedes;
char	filename[256], buffer[80+1];

int	millisec, delta, secofyr, stpofyr;
int	year, month, date;
int	day, hour, minute, second, mil, tick;
int	consecerr, astrap(), durtrap();
int	milb4lst = 0, milllast = 0;
int	mf = 1;

int	bsl = 0, ff = 0, tpe = 0, aspe = 0, bs = 0, mp = 0, spe = 0;

short	bcchan;
int	stat, wfordata;

extern short quicklook_flag;

#ifdef	DEBUG
LOGENTRY"ACQUIRE>mode=%d satid=%d startime=%d stoptime=%d\n",
		*mode,	satid,	startime,	stoptime);
#endif

if((*mode&0x0f)!=0x0b)/* not software synthesis of data */
	{
#ifdef	DEBUG
	LOGENTRY"ACQUIRE>INITIALIZING CYCLIC I/O\n");
#endif	DEBUG
	startio(&bcchan,isb);
	}


sprintf(filename,"HRPT1:[ACQUIRE]%s.SFL",sceneid);
records = 6 * (stoptime - startime);

make_julian(stoptime,&year,&day,&hour,&minute,&second);
stpofyr = second + 60 * (minute + 60 * (hour + 24 * (day-1)));
make_julian(startime,&year,&day,&hour,&minute,&second);
secofyr = second + 60 * (minute + 60 * (hour + 24 * (day-1)));
 
LOGENTRY"!	FILE SHOULD CONTAIN:	 %d FRAMES\n",records);

sprintf(buffer,"%s WILL CONTAIN %d LINES",filename,records);
droplog(sceneid,"ACQUIRE         ",buffer,proclog_name);

if (OK != bufopen(filename,records) )
	{
	stat = sys$cancel(bcchan);	/* Else BCDRIVER will keep going !*/
	stat = sys$dassgn(bcchan);
	return(NOTOK);
	}

make_julian(startime,&year,&day,&hour,&minute,&second);
JULGRE(year, day, month, date);
start_month = month;
start_dom = date;
start_year = year % 100;
start_julian = day;
/* the following line is to build text startime for 1k delta time log */
sprintf(date_time,"%02d/%02d/%02d %02d:%02d:%02d.000", month, date, year % 100,
	hour, minute, second);

sprintf(stamp,"TIMECODE                SEQUENCE");
stamp[4] = year % 100;
stamp[5] = month;
stamp[6] = date;
stamp[11] = 1000;

if (quicklook_flag == 1)
	vidsetup(satid,hour);

measured_delta = MEAN_ERROR;
if((*mode&0x08)==0x08)					/*synthetic data */
	{
#ifdef	DEBUG
	LOGENTRY"ACQUIRE>Using SYNTHETIC data:\n");
#endif	DEBUG
	delta = MEAN_ERROR;
	}
else if((*mode&0x04)==0x04)				/*playback from HDT */
	{
	calcdelta(satid,startime,&delta);
#ifdef	DEBUG
	LOGENTRY
	     "!	CALCULATED SATELLITE CLOCK ERROR:	%d MILLISECONDS\n"
		,delta);
#endif	DEBUG
	}
else	{
	finddelta(satid,startime,&delta,&bcchan,isb);
	measured_delta = delta;
#ifdef	DEBUG
	LOGENTRY
	"!	MEASURED SATELLITE CLOCK ERROR:  	%d MILLISECONDS\n"
		,delta);
#endif	DEBUG

	calcdelta(satid,startime,&delta);
#ifdef	DEBUG
	LOGENTRY
	     "!	CALCULATED SATELLITE CLOCK ERROR:	%d MILLISECONDS\n"
		,delta);
#endif	DEBUG

	/* Added for CR 5258.  If the difference in measured and
	calculated deltas is large and the measured delta is not
	MEAN_ERROR, use the measured delta.  Save the delta.
	--------------------------------------------------------*/
	if ( measured_delta != MEAN_ERROR && measured_delta != PROPDLAY )
	    savedelta(satid,startime,measured_delta,delta);
	if (( abs ( measured_delta - delta ) > MAXDELTADIFF ) &&
	    ( measured_delta != MEAN_ERROR ))
	    delta = measured_delta;

	sprintf(equiv,"%d",delta);
	crelnm_itmlst.s_buffer_length = strlen(equiv);
	if (satid == 1)
		status = SYS$CRELNM(0, &table, &n09_logical,
			acmode,	&crelnm_itmlst);
	    else if (satid == 2)
		status = SYS$CRELNM(0, &table, &n10_logical,
			acmode,	&crelnm_itmlst);
	    else if (satid == 3)
		status = SYS$CRELNM(0, &table, &n11_logical,
			acmode,	&crelnm_itmlst);
	    else if (satid == 4)
		status = SYS$CRELNM(0, &table, &n12_logical,
			acmode,	&crelnm_itmlst);
	    else if (satid == 5)
		status = SYS$CRELNM(0, &table, &n14_logical,
			acmode,	&crelnm_itmlst);
	    else if (satid == 6)
		status = SYS$CRELNM(0, &table, &n15_logical,
			acmode,	&crelnm_itmlst);
	    else if (satid == 7)
		status = SYS$CRELNM(0, &table, &n16_logical,
			acmode,	&crelnm_itmlst);
	    else if (satid == 8)
		status = SYS$CRELNM(0, &table, &n17_logical,
			acmode,	&crelnm_itmlst);
	}

#ifdef	DEBUG
LOGENTRY"ACQUIRE>RAISING PRIORITY TO 31\n");
#endif	DEBUG
stat = sys$setpri(0,0,0xf,&defpri);
if((stat & 0x01)==0)
	{
	LOGENTRY"ACQUIRE> ERROR RAISING PRIORITY\n");
	goto GIVEUP;
	}
pid = scan("INGEST1",&state);
if (pid != 0)
	{
	status = sys$suspnd(&pid,0,0);
	if (status == SS$_NORMAL)
		LOGENTRY"ACQUIRE> Suspended INGEST1 (%08x)\n",pid);
	}
pid = scan("INGEST2",&state);
if (pid != 0)
	{
	status = sys$suspnd(&pid,0,0);
	if (status == SS$_NORMAL)
		LOGENTRY"ACQUIRE> Suspended INGEST2 (%08x)\n",pid);
	}
pid = scan("NETCOPY",&state);
if (pid != 0)
	{
	status = sys$suspnd(&pid,0,0);
	if (status == SS$_NORMAL)
		LOGENTRY"ACQUIRE> Suspended NETCOPY (%08x)\n",pid);
	}


i = 0;
#ifdef	DEBUG
LOGENTRY"ACQUIRE>LOOKING FOR TIMES OF %d thru %d\n",1000*(secofyr%(24*3600))
						   ,1000*(stpofyr%(24*3600)));
#endif	DEBUG

bad = wfordata = second = 0;

/* Set event timer to prevent waiting beyond length of pass
-----------------------------------------------------------*/
timex[0] = ( stpofyr % ( 24 * 3600 )) - ( secofyr % ( 24 * 3600 ));
LOGENTRY"ACQUIRE> EXPECTING DURATION OF %d seconds\n",timex[0]);
timex[1] = 10000000;
quadmul( timex );
timex[1] = ~timex[1];
timex[0] = ~( timex[0] - 1 );
alarm = 0;
sys$setimr( EFDAT, timex, durtrap, 99 );
fflush(LOGFILE);

do	{
	milb4lst = milllast;
	milllast = millisec;

KILLTIME:
	if((*mode&0x0f)==0x0b)
		{
		i = (i+1) % NUMBUF;
		fill(i);
		}
	else	{
		stat = sys$waitfr(EFDAT);

		/* See if event flag was set because duration was exceeded
		----------------------------------------------------------*/
		if ( alarm == 2 )
			{
			LOGENTRY"ACQUIRE>DURATION OF PASS EXCEEDED\n");
			sequence = 0;
			startio(&bcchan, isb);
			goto GIVEUP;
			}

		if(stat != SS$_NORMAL)
			{
			LOGENTRY"ACQUIRE>ERROR WAITING FOR EVENT FLAG\n");
			if(OK != startio(&bcchan, isb))goto GIVEUP;
			goto KILLTIME;
			}
		stat = sys$clref(EFDAT);
		if((stat&0x01)!=SS$_NORMAL)
			{
			LOGENTRY"ACQUIRE>ERROR CLEARING EVENT FLAG\n");
			if(OK != startio(&bcchan, isb))goto GIVEUP;
			goto KILLTIME;
			}
		if(isb[0] != SS$_NORMAL)
			{
			LOGENTRY"ACQUIRE>cyclic read error\n");
			LOGENTRY"ACQUIRE> I/O status = %d\n",isb[0]);
			if(OK != startio(&bcchan, isb))goto GIVEUP;
			goto KILLTIME;
			}
		i = (isb[2] - 1) % NUMBUF;
		}

	if((*mode&0x08)==0x08)break;

	millisec = hrptbuff[i][11];
	millisec += hrptbuff[i][10] << 10;
	millisec += (hrptbuff[i][9] & 0x007f) << 20;
	millisec -= delta;

	if((hrptbuff[i][11089]&0x0fffe)!=0)
		{
		LOGENTRY
	"ACQUIRE>>-TIME=%dms: bsl=%x ff=%x tpe=%x aspe=%x bs=%x mp=%x spe=%x\n",
		millisec,
		(hrptbuff[i][11089]&0x08000)>>15,
		(hrptbuff[i][11089]&0x04000)>>14,
		(hrptbuff[i][11089]&0x03f00)>> 8,
		(hrptbuff[i][11089]&0x000f0)>> 4,
		(hrptbuff[i][11089]&0x00008)>> 3,
		(hrptbuff[i][11089]&0x00004)>> 2,
		(hrptbuff[i][11089]&0x00002)>> 1);
		if(++consecerr>=SATLOSS)if(OK!=startio(&bcchan,isb))goto GIVEUP;
		}

	if(	 (hrptbuff[i][0]!=0x0284) || (hrptbuff[i][1]!=0x016f)
	||	 (hrptbuff[i][2]!=0x035c) || (hrptbuff[i][3]!=0x019d)
	||	 (hrptbuff[i][4]!=0x020f) || (hrptbuff[i][5]!=0x0095)
	  )	{
		LOGENTRY"ACQUIRE>>--	iosb[2] = $%x	time=%d\n",
						isb[2],millisec);
		LOGENTRY"ACQUIRE>>--	$%x $%x $%x $%x $%x $%x ...\n",
			hrptbuff[i][0], hrptbuff[i][1], hrptbuff[i][2],
			hrptbuff[i][3], hrptbuff[i][4], hrptbuff[i][5]);
		if(++consecerr >= SATLOSS)
			if(OK != startio(&bcchan, isb))goto GIVEUP;
		goto KILLTIME;
		}
	consecerr = 0;
	second = millisec / 1000;
	second += ((hrptbuff[i][8] >> 1)-1) * 3600 * 24;
#ifdef DEBUG
if(!(isb[2]%60))LOGENTRY"ACQUIRE>found %d\n",millisec);
#endif	DEBUG
	}while	( (second<secofyr)
		||(millisec<=milllast)
		||(milllast<=milb4lst)
		||(millisec>(milb4lst+340)) );

/* Clear event timer since we have exited the above do while
   normally.
------------------------------------------------------------*/
sys$cantim( 99 );

LOGENTRY"ACQUIRE>Internal satellite ID # %d\n",
			(hrptbuff[i][6]>>3 & 0x0f));
if((*mode&0x08)==0x08)
	{
	nbstime(&day,&hour,&minute,&second,&tick);
	tick += 1000*(second+60*(minute+60*hour));
	}

for(sequence=1;sequence<=records;sequence++)
	{
	overwrite = 0;
	if((*mode&0x0f)==0x0b)
		{
		i = (i+1) % NUMBUF;
		fill(i);
		}
	else
		{
		timex[1] = -1;	alarm = 0;
		if((sequence-records)<=-1024)
			timex[0]=0;			/*wait 429 sec.s*/
		    else
			timex[0]=(sequence-records)<<21;/*wait 0.21 sec/rec*/
		sys$setimr(EFTIM,timex,astrap,0);

		if(SS$_WASSET != sys$readef(EFDAT,&dumy))
			wfordata++;
		stat = sys$waitfr(EFDAT);

		/* This code is for event timing */
		sys$gettim(&start_time[sequence-1]);
		elapsed[sequence-1][0] = elapsed[sequence-1][1] = 0;

		sys$cantim(0,0);	sys$clref(EFTIM);
		if(alarm == 1)
			goto GIVEUP;
		sys$clref(EFDAT);
		if(isb[0] != SS$_NORMAL)
			{
			LOGENTRY"ACQUIRE>cyclic read error after %d records\n",
								isb[2]);
			LOGENTRY"ACQUIRE> I/O status = %d\n",isb[0]);
			goto GIVEUP;
			}
		i = (isb[2] - 1) % NUMBUF;
		}

	if((*mode&0x08)==0x08)
		{
		while((tick+=(1000/6))>86400000){tick-=(1000*60*60*24);day++;}
		hrptbuff[i][11] = 0x03ff & tick;
		hrptbuff[i][10] = 0x03ff & (tick >> 10);
		hrptbuff[i][9] = 0x0280 + 0x007f & (tick >> 20);
		hrptbuff[i][8] = day << 1;
		hrptbuff[i][7] = 0x0000;
		hrptbuff[i][6] = 0x025b|(mf<<7);mf++;if(mf>3)mf=1;
		}

	millisec = hrptbuff[i][11];
	millisec += hrptbuff[i][10] << 10;
	millisec += (hrptbuff[i][9] & 0x007f) << 20;
	millisec -= delta;

	mil = millisec % 1000;
	second = millisec / 1000;
	minute = second   / 60;		second	%= 60;
	hour = minute	  / 60;		minute	%= 60;
	day = hour	  / 24;		hour	%= 24;

#ifdef	DEBUG
	if(sequence == 1)
		{
		LOGENTRY"!	ACTUAL START TIME	%02d:%02d:%02d.%03d\n",
					hour, minute, second, mil);
		LOGENTRY"!	ERRORS DURING PROCESSING:\n");
		start_hour = hour;
		start_minute = minute;
		start_second = second;
		start_mil = mil;
		}
#endif	DEBUG

	if((hrptbuff[i][0]!=0x0284)||(hrptbuff[i][1]!=0x016f)
	||	(hrptbuff[i][2]!=0x035c)||(hrptbuff[i][3]!=0x019d)
	||	(hrptbuff[i][4]!=0x020f)||(hrptbuff[i][5]!=0x0095)
	  )	{
		bad++;	consecerr++;
		LOGENTRY"!		%02d:%02d:%02d.%03d",
					hour, minute, second, mil);
		LOGENTRY"	ACQUIRE DETECTED BAD SYNC. PATTERN\n");

		LOGENTRY
		"ACQUIRE>>* %d	%d	%d	%d	%d	BADPATT\n",
					millisec,i,sequence,bad,consecerr);
		LOGENTRY"ACQUIRE>>-	isb=$%x,$%x,$%x,$%x\n",
				isb[0], isb[1], isb[2], isb[3]);
		LOGENTRY"ACQUIRE>>-	$%x $%x $%x $%x $%x $%x ...\n",
				hrptbuff[i][0], hrptbuff[i][1],hrptbuff[i][2],
				hrptbuff[i][3], hrptbuff[i][4],hrptbuff[i][5]);
		LOGENTRY
		 "ACQUIRE>>-\tbsl=%x ff=%x tpe=%x aspe=%x bs=%x mp=%x spe=%x\n",
		  (hrptbuff[i][11089]&0x08000)>>15,
		  (hrptbuff[i][11089]&0x04000)>>14,
		  (hrptbuff[i][11089]&0x03f00)>> 8,
		  (hrptbuff[i][11089]&0x000f0)>> 4,
		  (hrptbuff[i][11089]&0x00008)>> 3,
		  (hrptbuff[i][11089]&0x00004)>> 2,
		  (hrptbuff[i][11089]&0x00002)>> 1);
		LOGENTRY"!			FRAME DISCARDED\n");
		if(--sequence < 0)sequence = 0;

		/* If frame synch pattern has shifted one or two words, then
	           we have experienced the frame shift syndrome and will not
		   recover without resetting the 760 */
		if(((hrptbuff[i][1]==0x0284)&&(hrptbuff[i][2]==0x016f)
		    &&(hrptbuff[i][3]==0x035c)&&(hrptbuff[i][4]==0x019d)
		    &&(hrptbuff[i][5]==0x020f))
				||
		   ((hrptbuff[i][2]==0x0284)&&(hrptbuff[i][3]==0x016f)
		    &&(hrptbuff[i][4]==0x035c)&&(hrptbuff[i][5]==0x019d)))
			{
			LOGENTRY"!\t***Detected frame shift syndrome, ");
			LOGENTRY"760 interface reset! (Seq %d)\n",
				sequence+1);
			if(OK != startio(&bcchan, isb))
				goto GIVEUP;
			consecerr = 1;
			}

		if(consecerr >= SATLOSS)
			{
			LOGENTRY"!\t\t***%d CONSECUTIVE BAD FRAMES\n",
				consecerr);
			LOGENTRY"!\t\t***BUFFERED INTERFACE CHANNEL RESET\n");
			if(OK != startio(&bcchan, isb))goto GIVEUP;
			consecerr = 1;
		}	}
		
	else	{	/* if FRAME SYNC PATTERN is purfect, then */
		consecerr = 0;

		if((hrptbuff[i][11089]&0x0ffff)!=0)
			{
			LOGENTRY"!\t\t%02d:%02d:%02d.%03d",
				hour, minute, second, mil);
			LOGENTRY"\tQUALITY ERROR DETECTED BY FRAME SYNC.\n");
			LOGENTRY
			  "ACQUIRE>>* %d\t%d\t%d\t%d\t%d\tBADQUAL\n",
			   millisec,i,sequence,bad,consecerr);
			LOGENTRY"ACQUIRE>>-\tisb=$%x,$%x,$%x,$%x\n",
				isb[0], isb[1], isb[2], isb[3]);
			LOGENTRY
		"ACQUIRE>>-\tbsl=%x ff=%x tpe=%x aspe=%x bs=%x mp=%x spe=%x\n",
			  (hrptbuff[i][11089]&0x08000)>>15,
			  (hrptbuff[i][11089]&0x04000)>>14,
			  (hrptbuff[i][11089]&0x03f00)>> 8,
			  (hrptbuff[i][11089]&0x000f0)>> 4,
			  (hrptbuff[i][11089]&0x00008)>> 3,
			  (hrptbuff[i][11089]&0x00004)>> 2,
			  (hrptbuff[i][11089]&0x00002)>> 1);
			if((hrptbuff[i][11089]&0x08000)!=0)
				{
				bsl++;
	     			LOGENTRY"!\t\t\t\t\tBIT SYNC. LOSS\n");
				}
			if((hrptbuff[i][11089]&0x04000)!=0)
				{
				ff++;
     				LOGENTRY"!\t\t\t\t\tFIRST FRAME INDICATOR\n");
				}
			if((hrptbuff[i][11089]&0x03f00)!=0)
				{
				for(dumy=0x02000;dumy>=0x00100;dumy>>=1)
				if((dumy&hrptbuff[i][11089])!=0)tpe++;
	     			LOGENTRY"!\t\t\t\t\tTIP PARITY ERROR(S)\n");
				}
			if((hrptbuff[i][11089]&0x000F0)!=0)
				{
				aspe += (hrptbuff[i][11089]&0x000F0)>>4;

				LOGENTRY
				    "!\t\t\t\t\t%d AUX SYNC PATTERN ERRORS\n",
				    (hrptbuff[i][11089]&0x000F0)>>4);
				}
			if((hrptbuff[i][11089]&0x080)!=0)
				{
				bs++;
				LOGENTRY"!\t\t\t\t\tBIT SLIP\n");
				}
			if((hrptbuff[i][11089]&0x040)!=0)
				{
				mp++;
				LOGENTRY"!\t\t\t\t\tMISSED PATTERN\n");
				}
			if((hrptbuff[i][11089]&0x020)!=0)
				{
				spe++;
				LOGENTRY"!\t\t\t\t\tSYNC. PATTERN ERROR\n");
				}
			LOGENTRY"!\t\t\tFRAME RETAINED\n");
			}

		if((millisec<milb4lst)
		&& ((milb4lst/1000)>(stpofyr%(24*60*60))))
			{
			bad += 2;	consecerr += 2;
			if((sequence-=2) < 1)sequence = 1;
			LOGENTRY"!\t\t%02d:%02d:%02d.%03d",
				hour, minute, second, mil);
			LOGENTRY"\tTIMESTAMPS OF TWO PREVIOUS FRAMES BAD\n");
	     		LOGENTRY"!\t\t\tPREVIOUS TWO FRAMES DISCARDED\n");
			LOGENTRY"ACQUIRE>>* %d\t%d\t%d\t%d\t%d\tBADTIME\n",
				millisec,i,sequence,bad,consecerr);
			LOGENTRY"ACQUIRE>>-	isb=$%x,$%x,$%x,$%x\n",
				isb[0], isb[1], isb[2], isb[3]);
			LOGENTRY"ACQUIRE>>-\t\t\t%d < %d < %d\n",
				milb4lst, milllast, millisec);
			milb4lst = milllast = millisec;
			overwrite = 2;
			}
		else if(millisec<milb4lst)
			{ 
			bad++; consecerr++; if(--sequence<0)sequence=0;
			LOGENTRY"!\t\t%02d:%02d:%02d.%03d",
				hour, minute, second, mil);
			LOGENTRY"\tTIMESTAMP < TWO PREVIOUS FRAMES\n");
			LOGENTRY"!\t\t\tFRAME DISCARDED\n");
			LOGENTRY"ACQUIRE>>* %d\t%d\t%d\t%d\t%d\tBADTIME\n",
				millisec,i,sequence,bad,consecerr);
			LOGENTRY"ACQUIRE>>-\tisb=$%x,$%x,$%x,$%x\n",
				isb[0], isb[1], isb[2], isb[3]);
			LOGENTRY"ACQUIRE>>-\t\t\t%d < %d < %d\n",
				milb4lst, milllast, millisec);
			continue;
			}
		else if (millisec<milllast) 
			{ 
			bad++; consecerr++;
			if(--sequence < 0)sequence = 0;
			LOGENTRY"!\t\t%02d:%02d:%02d.%03d",
				hour, minute, second, mil);
			LOGENTRY"\tTIMESTAMP < PREVIOUS FRAME\n");
		     	LOGENTRY"!\t\t\tPREVIOUS FRAME DISCARDED\n");
			LOGENTRY"ACQUIRE>>* %d\t%d\t%d\t%d\t%d\tBADTIME\n",
				millisec,i,sequence,bad,consecerr);
			LOGENTRY"ACQUIRE>>-\tisb=$%x,$%x,$%x,$%x\n",
				isb[0], isb[1], isb[2], isb[3]);
			LOGENTRY"ACQUIRE>>-\t\t\t%d < %d < %d\n",
				milb4lst, milllast, millisec);
			milllast=milb4lst;
			overwrite = 1;
			}

/*THIS WAS */	if((milb4lst > (millisec - 500))
/*ADDED    */	&& (consecerr == 0)
/*AND THEN */	&& (hrptbuff[0][8]==hrptbuff[1][8])
/*REMOVED  */	&& (hrptbuff[2][8]==hrptbuff[3][8])
/*AND THEN */	&& (hrptbuff[0][8]==hrptbuff[2][8])
/*RE-ADDED */	&& (stpofyr < ((milb4lst/1000)+((hrptbuff[i][8]>>1)-1)*3600*24))
		  )	{
			LOGENTRY"!\t\t%02d:%02d:%02d.%03d",
				hour, minute, second, mil);
			LOGENTRY"\tFOUND FRAME AFTER STOP TIME\n");
			LOGENTRY"!\t\t\tFRAME DISCARDED\n");
			LOGENTRY
			  "ACQUIRE>>* %d found record from after stoptime\n",
				millisec);
			LOGENTRY"ACQUIRE>>-\tisb=$%x,$%x,$%x,$%x\n",
				isb[0], isb[1], isb[2], isb[3]);
			LOGENTRY"ACQUIRE>>\t%d < %d < %d\n",milb4lst,
				milllast, millisec);
			goto GIVEUP;
			}

		milb4lst = milllast;	milllast = millisec;

		for(j=0;j<17;j++)hrptbuff[i][WORDCNT -100 +j] = stamp[j];
		if(day != 0) 	hrptbuff[i][WORDCNT -100 +6] = date + day;

		hrptbuff[i][WORDCNT -100 +  7] = hour;
		hrptbuff[i][WORDCNT -100 +  8] = minute;
		hrptbuff[i][WORDCNT -100 +  9] = second;
		hrptbuff[i][WORDCNT -100 + 10] = millisec%1000;
		hrptbuff[i][WORDCNT -100 + 16] = sequence;

		if(OK != bufrit(i,sequence))goto GIVEUP;
		}
	if (quicklook_flag == 1)
		videout(i,consecerr);
	}

GIVEUP:

stat = sys$cantim(0,0);
stat = sys$cancel(bcchan);
stat = sys$dassgn(bcchan);

LOGENTRY"ACQUIRE TEST> ALARM=%d seq=%d\n",alarm,sequence);
fflush(LOGFILE);

/* If we got here because the pass duration was exceeded set
the mode to 0 indicating the scene is missed.
------------------------------------------------------------*/
if ( alarm == 2 )
	{
	*mode = 0;
	sequence = 0;
	}
orbit = 0; if(--sequence < 0)sequence = 0;
buflsh(sequence,orbit,year,satid,delta);
chmod(filename,509);	/* set prot=(O:RWED,G:RWED,W:RE) */

/* If duration of pass was exceeded, remove the image file
otherwise rename it.
-----------------------------------------------------------*/
if ( alarm == 2 )
	remove(filename);
else
	rename(filename,"HRPT1:[IMAGES]");

pid = scan("INGEST1",&state);
if ((pid != 0) && ((state == SCH$C_SUSP) || (state == SCH$C_SUSPO)))
	{
	status = sys$resume(&pid,0);
	if (status == SS$_NORMAL)
		LOGENTRY"ACQUIRE> Resumed INGEST1 (%08x)\n",pid);
	}
pid = scan("INGEST2",&state);
if ((pid != 0) && ((state == SCH$C_SUSP) || (state == SCH$C_SUSPO)))
	{
	status = sys$resume(&pid,0);
	if (status == SS$_NORMAL)
		LOGENTRY"ACQUIRE> Resumed INGEST2 (%08x)\n",pid);
	}
pid = scan("NETCOPY",&state);
if ((pid != 0) && ((state == SCH$C_SUSP) || (state == SCH$C_SUSPO)))
	{
	status = sys$resume(&pid,0);
	if (status == SS$_NORMAL)
		LOGENTRY"ACQUIRE> Resumed NETCOPY (%08x)\n",pid);
	}

stat = sys$setpri(0,0,defpri,&defpri);
if(stat & 0x01==0)
	LOGENTRY"ACQUIRE> ERROR LOWERING PRIORITY:%d\n",stat);


LOGENTRY"!		SUMMARY OF FRAME SYNC. ERRORS:\n");
LOGENTRY"!			BIT. SYNC.LOSSES:	%d\n",bsl);
LOGENTRY"!			FIRST FRAMES:		%d\n",ff);
LOGENTRY"!			TIP PARITY ERRORS:	%d\n",tpe);
LOGENTRY"!			AUX. PATTERN ERRORS:	%d\n",aspe);
LOGENTRY"!			BIT SLIPS:		%d\n",bs); 
LOGENTRY"!			MISSED PATTERNS:	%d\n",mp);
LOGENTRY"!			SYNC. PATTERN ERRORS:	%d\n",spe);
LOGENTRY"!	ACTUAL STOP TIME	%02d:%02d:%02d.%03d\n",
					hour, minute, second, mil);
LOGENTRY"!	FILE ACTUALLY CONTAINS:	%d FRAMES\n",sequence);
LOGENTRY"!	FRAMES MISSED:		%d\n",bad);
LOGENTRY"ACQUIRE> WAIT FOR DATA ON %d LINES\n",wfordata);
#ifdef	DEBUG
LOGENTRY"ACQUIRE> WRITING HEADER RECORD.\n");
#endif	DEBUG
/* This code is for event timing */
LOGENTRY"ACQUIRE TEST> ALARM=%d seq=%d\n",alarm,sequence);

/* Skip the for loop completely if the duration was exceeded.
For some reason this loop executed even though sequence = 0
and runs forever.  The if statement is a kludge just to get
things working in production.
-------------------------------------------------------------*/
if ( alarm != 2 )
    {
    for(cnt=avg_1=avg_2=0;cnt<sequence-1;cnt++)
	{
	subquad (&start_time[cnt+1],&start_time[cnt],&lcl_elapsed_time);
	ediv (&lcl_elapsed_time,&shift,&elapsed_millisec);
	elapsed[cnt][1] = elapsed_millisec;

	if (elapsed[cnt][1] > 200)
		{
		LOGENTRY"** Line %04d  Time = %4d ms\n",
			cnt+1,elapsed[cnt][1]);
		}
	avg_2 += elapsed[cnt][1];
	}
    LOGENTRY"\n**  Average record interval = %4d\n", avg_2 / sequence);
    }
sprintf(buffer,"%d lines written, %d bad lines detected",sequence,bad);
droplog(sceneid,"ACQUIRE         ",buffer,proclog_name);

return(OK);
}

/*******************************************************
  Duration trap
  Service routime for setting the event flag EFDAT
  to prevent waiting beyond end of pass.
*******************************************************/
durtrap()
{
sys$setef( EFDAT );
alarm = 2;
}

astrap()
/********************************************************
*							*
********************************************************/
{
if((timex[1] != -1)||(timex[0] == 0))
	LOGENTRY"ASTRAP>> TIMED OUT AFTER %d seconds\n",
				(-429*timex[1])-(timex[0]/10000000) );
else	LOGENTRY"ASTRAP>> TIMED OUT AFTER %d seconds\n", -timex[0]/10000000);

alarm = 1;
sys$setef(EFDAT);
}

startio(bcchan,isb)
short	*bcchan, isb[];
/********************************************************
*							*
********************************************************/
{
int	sys$assign(), sys$qiow(), sys$qio(), sys$cancel(), sys$dassgn();
int	stat, i, now;
$DESCRIPTOR(desc,"_BCB0");
int	modtab[2*7] = {1, 1, 2, 4, 3, 0, 4, 0, 5, 1, 6, 0, 8, 1};
int	buftab[2*NUMBUF];

RESET:;
LOGENTRY"STARTIO> RESETTING 760 AND RESTARTING CYCLIC I/O\n");
stat = sys$cancel(*bcchan);
stat = sys$dassgn(*bcchan);

for(i=0;i<NUMBUF;i++)
	{
	buftab[(2*i)+0] = &hrptbuff[i][0];
	buftab[(2*i)+1] = BYTECNT;
	}

stat = sys$assign (&desc,bcchan,0,0);
if(stat & 0x01 ==0)
	{
	LOGENTRY"STARTIO> ERROR ATTACHING TO THE DEVICE\n");
	return(NOTOK);
	}
#ifdef	DEBUG
LOGENTRY"STARTIO> bcchan = ($%x) = $%x\n",bcchan,*bcchan);
#endif	DEBUG
stat = sys$qiow(EFDAT,*bcchan,IO$_RESET,isb,0,0,0,0,0,0,0,0);
if	( stat & 0x01 == 0 )
	{
	LOGENTRY"STARTIO> ERROR INITIALIZING THE DEVICE: stat = %d\n",stat);
	return(NOTOK);
	}
else if	(isb[0] != SS$_NORMAL)
	{
	LOGENTRY"STARTIO> ERROR INITIALIZING THE DEVICE:isb[0] = %d\n",isb[0]);
	return(NOTOK);
	}

i = 0;
STARTCYC:
stat = sys$qio (EFDAT,*bcchan,IO$_RCYCLIC,isb,0,0,
		buftab,NUMBUF*2*4,modtab,7*2*4,0,0);
if((stat & 0x01) == 0)
	{
	LOGENTRY"STARTIO> ERROR INITIALIZING CYCYLIC I/O: stat = %d\n",stat);
	if(stat == 0x02c4)
		{
		if(++i >= 10)goto RESET;
		sleep(1);
		goto STARTCYC;
		}
	return(NOTOK);
	}
return(OK);
}

header(buffer,records,orbit,year,satid,delta)
char	buffer[];
int	records, orbit, year, satid, delta;
/*
==========================================================================
==									==
==	This routine creates the header record that is appended to	==
==	the beginning of the HRPT image file.  The output of this	==
==	routine is left in 'buffer' to be written by the caller.	==
==									==
==========================================================================
*/
{
int	i, tmp_sat=0;
/* old header info from 0 to 255 */
buffer[0] = buffer[1] = buffer[2] = buffer[3] = 0xaa;	/* magic sync. number*/
buffer[4] = (43*512) & 0x00ff;		/* bytes per record	(low part)   */
buffer[5] = ((43*512)>>8) & 0x00ff;	/* bytes per record	(high part)  */
buffer[6] = (BYTECNT-200)& 0x00ff;	/* offset to time code	(low part)   */
buffer[7] = ((BYTECNT-200)>>8)&0x00ff;	/* offset to time code	(high part)  */
buffer[8] = 1;buffer[9] = 0;		/* record offset to data (next rec.) */
buffer[10] = 1;buffer[11] = 0;		/* type type number (unused )	     */
buffer[12] = records & 0x00ff;		/* number of records	(low part)   */
buffer[13] = (records >> 8) & 0x0ff;	/* number of records	(high part)  */
buffer[14] = orbit & 0x00ff;		/* orbit number		(low part)   */
buffer[15] = (orbit >> 8) & 0x0ff;	/* orbit number 	(high part)  */
buffer[16] = year & 0x00ff;		/* year of data acq.	(low part)   */
buffer[17] = (year >> 8) & 0x0ff;	/* year of data acq.	(high part)  */
buffer[18] = 1;buffer[19] = 0;		/* tape rate. ??		     */
buffer[20] = 0;buffer[21] = 0;		/* data compression indicat.(0=no)   */
buffer[22] = 1;buffer[23] = 0;		/* byte swapping indicator. (1=no)   */
for(i=24;i<256;i++)buffer[i] = 0;	/* 0 fill old header		     */
for(i=256;i<512;i++)buffer[i] = 32;	/* space fill new header	     */
/* new header info from 256 to 511 */
if (satid == 1)
	tmp_sat = 9;
    else if (satid == 2)
	tmp_sat = 10;
    else if (satid == 3)
	tmp_sat = 11;
    else if (satid == 4)
	tmp_sat = 12;
    else if (satid == 5)
	tmp_sat = 14;
    else if (satid == 6)
	tmp_sat = 15;
    else if (satid == 7)
	tmp_sat = 16;
    else if (satid == 8)
	tmp_sat = 17;

sprintf(&buffer[256],
"%5d %2d HRPT SFL %4d 2048 %02d/%02d/%02d %3d %02d:%02d:%02d.%03d",
delta, tmp_sat, records, start_month, start_dom, start_year,
start_julian, start_hour, start_minute, start_second, start_mil);
}

finddelta(satid, startime, delta, bcchan, isb)
int	satid, startime, *delta;
short	*bcchan, isb[];
/*
==========================================================================
==									==
==									==
==									==
==========================================================================
*/
{
int	i, j, stat;
int	now;

int	millerr, totmilerr;

int	day[SAMPLES], hour[SAMPLES], minute[SAMPLES];
int	nbssec[SAMPLES+1], nbsmil[SAMPLES], satmil[SAMPLES];
int	consecerr;

*delta = MEAN_ERROR;
nbsecond(&now);
if(now>=startime)
	{
	LOGENTRY"FINDDELTA> no time to calculate delta\n");
	return(NOTOK);
	}

timex[1] = 10*1000*1000;	timex[0] = startime - now;
quadmul(timex);	timex[1] =  ~timex[1];	timex[0] = ~(timex[0]-1);
sys$setimr(EFTIM,timex,astrap,0);
alarm = 0;

#ifdef	DEBUG
LOGENTRY"FINDDELTA> summing errors over %d records\n",SAMPLES);
#endif	DEBUG

stat = sys$clref(EFDAT);
if((stat&0x01)!=SS$_NORMAL)
	{
	sys$cantim(0,0);
	sys$clref(EFTIM);
	LOGENTRY"FINDDELTA> ERROR CLEARING EVENT FLAG\n");
	return(NOTOK);
	}

for(j=0; j<SAMPLES; j++)
	{
	stat = sys$waitfr(EFDAT);
	if (stat != SS$_NORMAL)
		{
		sys$cantim(0,0);	sys$clref(EFTIM);
		LOGENTRY"FINDDELTA> ERROR WAITING FOR EVENT FLAG\n");
		return(NOTOK);
		}
	if(alarm == 1)
		{
		LOGENTRY"FINDDELTA> no time left to calculate delta\n");
		sys$cantim(0,0);	sys$clref(EFTIM);
		return(NOTOK);
		}
	stat = sys$clref(EFDAT);
	if((stat&0x01)!=SS$_NORMAL)
		{
		LOGENTRY"FINDDELTA> ERROR CLEARING EVENT FLAG\n");
		sys$cantim(0,0);	sys$clref(EFTIM);
		return(NOTOK);
		}
	if(isb[0] != SS$_NORMAL)
		{
		LOGENTRY"FINDDELTA> cyclic read error\n");
		LOGENTRY"FINDDELTA> I/O status = %d\n",isb[0]);
		if(OK != startio(bcchan, isb))return(NOTOK);
		j = -1;/*START OVER SO 15 CONSECETIVE VALID RECORDS ARE USED*/
		continue;			/* avoid using -1 subscript */
		}

	stat = nbstime(&day[j],&hour[j],&minute[j],&nbssec[j],&nbsmil[j]);
	if (stat != OK)
		{
		sys$cantim(0,0);	sys$clref(EFTIM);
		LOGENTRY"FINDDELTA> NBS CLOCK READ ERROR\n");
		return(NOTOK);
		}

	i = (isb[2] - 1) % NUMBUF;

	if( ((hrptbuff[i][11089]&0x4000)!=0)
	||  ((hrptbuff[i][11089]&0x00f0)>=0x0080)
	||   (hrptbuff[i][0]!=0x0284)||(hrptbuff[i][1]!=0x016f)
	||   (hrptbuff[i][2]!=0x035c)||(hrptbuff[i][3]!=0x019d)
	||   (hrptbuff[i][4]!=0x020f)||(hrptbuff[i][5]!=0x0095)
	  )	{
		LOGENTRY"FINDDELTA> ERROR IN DATA STREAM CONTENTS:\n");
		LOGENTRY"FINDDELTA>	iosb[2] = $%d\n",isb[2]);
		LOGENTRY"FINDDELTA>	$%x $%x $%x $%x $%x $%x ... $%x\n",
			hrptbuff[i][0], hrptbuff[i][1], hrptbuff[i][2],
			hrptbuff[i][3], hrptbuff[i][4], hrptbuff[i][5],
			hrptbuff[i][11089]);
		if(++consecerr >= SATLOSS)
			if(OK != startio(bcchan,isb))return(NOTOK);
		j = -1;/*START OVER SO 15 CONSECUTIVE VALID RECORDS ARE USED*/
		continue;			/* avoid using -1 subscript */
		}
	else	{
		consecerr = 0;

		satmil[j] = hrptbuff[i][11] + (hrptbuff[i][10]<<10) 
			+ ((hrptbuff[i][9]&0x007f)<<20);

/*#ifdef DEBUG
		LOGENTRY"FINDDELTA> rec #%02d: sat=%d ms.  NBS=%d ms.\n",
			j,satmil[j],(nbsmil[j]+1000*(nbssec[j]+60*(minute[j]
			+60*hour[j]))));
#endif DEBUG /**/
		}
	}

sys$cantim(0,0);	sys$clref(EFTIM);

/******************	FIND MEDIAN ERROR 	******************/
for(totmilerr=j=0;j<SAMPLES;j++)
	{
	millerr = satmil[j] - 	(nbsmil[j] + 1000*(nbssec[j] + 60*
				(minute[j] + 60*hour[j])));
	millerr %= 60*60*1000;
	if(millerr > 30*60*1000)millerr -= 60*60*1000;
	satmil[j] = millerr;

/*#ifdef	DEBUG
	LOGENTRY"FINDDELTA> sample = %d: error = %d\n",j,millerr);
endif	DEBUG	/**/
	}
/* Added for CR 5258.  Use an insertion
sort instead of a bubble sort.
---------------------------------------*/
insertion_sort(satmil,SAMPLES);

/* Added for CR 5258.  Use the weighted mean
of the data instead of the median value.
---------------------------------------------*/
*delta = weighted_mean( satmil );
if ( *delta == MEAN_ERROR )
    return NOTOK;
if ((*delta > 2000) || (*delta < -2000))
	{
	LOGENTRY"FINDDELTA>> Sanity check on delta failed.  Was %d\n",*delta);
	*delta = MEAN_ERROR;
	return NOTOK;
	}
/*****************************************************************/

*delta += PROPDLAY;

#ifdef	DEBUG
LOGENTRY"FINDDELTA> average error is %d milliseconds\n",*delta);
LOGENTRY"FINDDELTA> (including a %d millisec. delay constant)\n",PROPDLAY);
#endif	DEBUG
return(OK);
}

quadmul(quad)
int	quad[2];
{
union	{
	unsigned int	i;
	unsigned short	s[2];
	} i, j, prod0, prod1, prod2, prod3, temp, temp2;

i.i = quad[0];
j.i = quad[1];

prod0.i = i.s[0] * j.s[0];
prod1.i = i.s[0] * j.s[1];
prod2.i = i.s[1] * j.s[0];
prod3.i = i.s[1] * j.s[1];

temp2.s[0] = prod0.s[0];
temp.i =  prod0.s[1] + prod1.s[0] + prod2.s[0];
temp2.s[1] = temp.s[0];
quad[0] = temp2.i;

quad[1] = temp.s[1] + prod1.s[1] + prod2.s[1] + prod3.i; 
}
/*---------------------------------------------------
The stupid little bubble sort has been replaced with
a more efficient insertion sort in acqusubs.c.  Even
though they are the same Big O, the insertion sort
is more efficient.
----------------------------------------------------*/

savedelta(satid,startime,delta,calcdelta)
int	satid, startime, delta,calcdelta;
/*
==========================================================================
==									==
==	This routine adds one entry to the file that logs the		==
==	satelites' timer errors for each live acquisition.  These	==
==	files are used by the 'calcdelta' routine to predict the	==
==	satelite timer error for prerecorded acquisitions.		==
==									==
==========================================================================
*/
{
FILE	*stream;
int	entries, oldest, second, millierr, target, i;
#ifdef	DEBUG
LOGENTRY"SAVEDELTA> updating error log file: %s\n",SATIMERR[satid]);
#endif	DEBUG

stream = fopen(SATIMERR[satid],"r+");
if(stream == NULL)	/* file does not exist yet */
	{
#ifdef	DEBUG
LOGENTRY"SAVEDELTA> COULDN'T OPEN FILE: CREATING A NEW ONE\n");
#endif	DEBUG
 	stream = fdopen(creat(SATIMERR[satid],0666,"rat=cr","rfm=stmcr"),"r+");
	entries = 0; oldest = 0;
	}
else	fscanf(stream,"%02d	%02d\n",&entries,&oldest);

if(entries >= MAXELE)	target = oldest = (oldest + 1) % MAXELE;
else			target = entries++;
rewind(stream);

/* Added CR 5258.  If the difference in calculated and measured
delta is greater than SATIME1K_THRESH, restart the time series.
This helps to prevent mixing deltas after a clock correction.
---------------------------------------------------------------*/
if ( abs( calcdelta - delta ) > SATIME1K_THRESH )
    {
    entries = oldest = 0;
    target = entries++;
    LOGENTRY"SAVEDELTA> SATIME1K_THRESH reached, resetting time series\n");
    }

#ifdef	DEBUG
LOGENTRY"SAVEDELTA> no. %d is the oldest of %d entries.\n",oldest,entries);
#endif	DEBUG

#ifdef	DEBUG
LOGENTRY"SAVEDELTA>	%02d	%02d\n",entries,oldest);
#endif	DEBUG
fprintf(stream,"%02d	%02d\n",entries,oldest);
for(i=0;i<target;i++)
	{
	fscanf(stream,"%d	%d\n",&second,&millierr);
#ifdef	DEBUG
LOGENTRY"SAVEDELTA>	%9d	%9d\n",second,millierr);
#endif	DEBUG
	}
#ifdef	DEBUG
LOGENTRY"SAVEDELTA>	%9d	%9d\n",startime,delta);
#endif	DEBUG
fprintf(stream,"%9d	%9d\n",startime,delta);
fclose(stream);
}

calcdelta(satid,startime,delta)
int	satid, startime, *delta;
/*
==========================================================================
==									==
==	This reads the satelites' timer error log file and calculates	==
==	the delta for 'startime'.  This uses a polynomial regression.	==
==									==
==========================================================================
*/
{
FILE	*stream;
FILE	*onek;
int	entries, oldest;
int	satime, millierror;
int	i, j, row, col, order;

double	matrix[ORDER+1][ORDER+1], solut[ORDER+1], sumx2n[2*ORDER+1];
double	power, pxy, realdelta;

for(j=0;j <= ORDER;j++)	solut[j] = 0;
for(j=0;j<=ORDER*2;j++)sumx2n[j] = 0;

order = ORDER;

#ifdef	DEBUG
LOGENTRY"CALCDELTA> using error file: %s\n",SATIMERR[satid]);
#endif	DEBUG

stream = fopen(SATIMERR[satid],"r");
if(stream == NULL)
	{
	LOGENTRY"CALCDELTA>unable to open error log file\n");
	*delta = 0;
	return(NOTOK);
	}

fscanf(stream,"%02d	%02d\n",&entries,&oldest);

#ifdef	DEBUG
LOGENTRY"CALCDELTA> %02d	%02d\n",entries,oldest);
#endif	DEBUG

if(entries <= ORDER)
	{
	order = entries - 1;
	LOGENTRY"CALCDELTA>>TOO FEW POINTS IN ERROR LOG FILE\n");
	}

#ifdef	DEBUG
	LOGENTRY"CALCDELTA> using a regression of order %d\n",order);
#endif	DEBUG

for(i=0;i<entries;i++)
	{
	fscanf(stream,"%d	%d\n",&satime,&millierror);
#ifdef	DEBUG
LOGENTRY"CALCDELTA> %9d(%9d)	%9d\n", satime, satime-startime, millierror);
#endif	DEBUG
	power = 1;
	for(j=0;j<=order;j++)
		{
		sumx2n[j] += power;
		solut[j] += millierror * power;
		power *= (satime-startime);
		}
	for(;j<=order*2;j++)
		{
		sumx2n[j] += power;
		power *= (satime-startime);
		}
	for(row=0;row<=order;row++)for(col=0;col<=order;col++)
		{
		matrix[row][col] = sumx2n[row+col];
	}	}
fclose(stream);
gauss(matrix,solut,order);

*delta = (int)(solut[0] + 0.5);

#ifdef	DEBUG
LOGENTRY"CALCDELTA>estimated satellite clock error = %d millisec.\n",*delta);
#endif	DEBUG

onek = fopen(SATIME1K[satid],"a+");
if(onek == NULL)
        {
#ifdef  DEBUG
LOGENTRY"CALCDELTA> COULDN'T OPEN 1K FILE, UNKNOWN ERROR\n");
#endif  DEBUG
        }
    else
        {
	/* Added for CR 5258.  If the difference in measured and
	calculated deltas is large and the measured delta is not
	MEAN_ERROR, use the measured delta.
	--------------------------------------------------------*/
	if (( abs ( measured_delta - *delta ) > MAXDELTADIFF ) &&
	    ( measured_delta != MEAN_ERROR ))
		fprintf(onek,"%s %05d\n",date_time,measured_delta);
	    else
		fprintf(onek,"%s %05d\n",date_time,*delta);
        fclose(onek);
        }

}

gauss(matrix,solut,order)
double	matrix[ORDER+1][ORDER+1], solut[ORDER+1];
int	order;
{
int	i, j, k;

#ifdef	DEBUG
	LOGENTRY"GAUSS>DIAGONALIZING MATRIX OF DIMENSION %d\n",order+1);
	printit(matrix,solut,order);
#endif	DEBUG
for(i=0;i<=order;i++)
	{
	solut[i] /= matrix[i][i];
	for(j=order;j>=i;j--)matrix[i][j] /= matrix[i][i];
	for(j=0;j<=order;j++)if(j!=i)
		{
		solut[j] -= solut[i] * matrix[j][i];
		for(k=order;k>=i;k--)matrix[j][k]-=matrix[i][k]*matrix[j][i];
	}	}
#ifdef	DEBUG
	printit(matrix,solut,order);
#endif	DEBUG
}

#ifdef	DEBUG
printit(matrix,solut,order)
double	matrix[ORDER+1][ORDER+1], solut[ORDER+1];
int	order;
{
int	i, j;

for(i=0;i<=order;i++)
	{
	LOGENTRY"PRINTIT>  ");
	for(j=0;j<=order;j++)
		LOGENTRY"%e	",matrix[i][j]);
	LOGENTRY"%e\n",solut[i]);
	}
LOGENTRY"\n");
}
#endif	DEBUG

fill(num)
int	num;
{
char	mess[9][8] = {	/*t    e    s    t    p    a    t    t	*/
			{0xfe,0xfe,0x7c,0xfe,0xfc,0x10,0xfe,0xfe},
			{0x10,0x80,0x82,0x10,0x82,0x28,0x10,0x10},
			{0x10,0x80,0x80,0x10,0x82,0x44,0x10,0x10},
			{0x10,0xf8,0x7c,0x10,0xfc,0x82,0x10,0x10},
			{0x10,0x80,0x02,0x10,0x80,0xfe,0x10,0x10},
			{0x10,0x80,0x82,0x10,0x80,0x82,0x10,0x10},
			{0x10,0xfe,0x7c,0x10,0x80,0x82,0x10,0x10},
			{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
			{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}};
int	i, j;
static int	k = 0;

hrptbuff[num][0]=0x0284; hrptbuff[num][1]=0x016f; hrptbuff[num][2]=0x035c;
hrptbuff[num][3]=0x019d; hrptbuff[num][4]=0x020f; hrptbuff[num][5]=0x0095;

hrptbuff[num][12] = 0x0254;	hrptbuff[num][13] = 0x0255; 
hrptbuff[num][14] = 0x01b2;	hrptbuff[num][15] = 0x01b2; 
hrptbuff[num][16] = 0x01b7;	hrptbuff[num][17] = 0x0001;
hrptbuff[num][18] = 0x0001;	hrptbuff[num][19] = 0x0001; 
hrptbuff[num][20] = 0x01d3;	hrptbuff[num][21] = 0x0001;
hrptbuff[num][11089] = 0x0000;

for(i=0;i<2048;i++)for(j=0;j<5;j++)hrptbuff[num][(5*i)+j+750] = i/2;

if((++k&0x0f) == 0x00)
	{
	if(((k>>4)&0x03)==0)hrptbuff[num][11089] = 0x0bffe;
	if(((k>>4)&0x03)==1)
		{
		for(i=0;i<6;i++)hrptbuff[num][i] |= 1<<i;
		hrptbuff[num][11089] = 0x0002;
		}
	if(((k>>4)&0x03)==2)hrptbuff[num][11] |= 0x0200;
	if(((k>>4)&0x03)==3)hrptbuff[num][10] |= 0x0111;
	}
else for(i=0;i<256;i++)
    if((mess[k/10][(i/8)%8]&(0x80>>(i%8)))!=0)for(j=0;j<40;j++) 
	hrptbuff[num][(40*i)+j+750]=(hrptbuff[num][(40*i)+j+750]+0x200)&0x3ff;
return(OK);
}

/*****************************************************/
#define	VDAYBND	1
#define	VNITBND	3
#define ASCEND  0
#define DESCEND 1
int	VIDBND;
int	UP_DOWN;
#define	VIDSUB	2		/* SUBSAMPLE RATE FOR VIDEO DISPLAY OUTPUT */
#define VIDCHUNK 8		/* Write 8 bytes to Peritek in one chunk */

unsigned char vidbuf[1024], nilbuf[1024];
int peritek_line;

vidsetup(satid,hour)
int	satid, hour;
{
static int	opened=0;
int	i, stat;

if( ((satid == 1)&&(hour >= 3)&&(hour < 15))
 || ((satid == 2)&&(hour >= 9)&&(hour < 21))
 || ((satid == 3)&&(hour < 12))
 || ((satid == 4)&&(hour >= 9)&&(hour < 21))
 || ((satid == 5)&&(hour < 12))
 || ((satid == 6)&&(hour >= 9)&&(hour < 21))
 || ((satid == 7)&&(hour < 12))
 || ((satid == 8)&&(hour >= 13)&&(hour < 22))
  )	UP_DOWN = DESCEND;
else	UP_DOWN = ASCEND;

if( (satid == 1)&&(UP_DOWN == ASCEND)
 || (satid == 2)&&(UP_DOWN == DESCEND)
 || (satid == 3)&&(UP_DOWN == ASCEND)
 || (satid == 4)&&(UP_DOWN == DESCEND)
 || (satid == 5)&&(UP_DOWN == ASCEND)
 || (satid == 6)&&(UP_DOWN == DESCEND)
 || (satid == 7)&&(UP_DOWN == ASCEND)
 || (satid == 8)&&(UP_DOWN == DESCEND)
  )	VIDBND = VDAYBND;
else	VIDBND = VNITBND;

/* Kludge for NOAA 9 - it drifted and is getting flipped on the monitor */
if (satid == 1)
	{
	if (UP_DOWN == DESCEND)
		UP_DOWN = ASCEND;
	    else
		UP_DOWN = DESCEND;
	}

if (UP_DOWN == DESCEND)
	peritek_line = 1023;
    else
	peritek_line = 0;

for(i=0;i<1024;i++)
	nilbuf[i] = 0;

for (i = 0; i < 256; i++)		/* Define color map bbggrrrr */
	cmap(0, i,
		 (i & 0xf0)+(i>>4),
	 	((i & 0x0f) * 17),
	 	((i & 0x0f) * 17));

setpl(0);				/* Select primary plane */
clearall();				/* Clear all planes */
/* NOTE: The following line is required to be between Peritek's routines
   and our routines, otherwise the system will crash! */
setpl(0);				/* req to prevent crash */
scrl(0);				/* reset scroll */

return(OK);
}

videout(num,qual)
int	num, qual;
{
int	i, j, k, stat;
static	int	line, totqual;

totqual += qual;
if(VIDSUB == ++line)
	{
	line = 0;
/*******************************************************************************
Pixels used for daytime display;

		Band 1			Band 2
	---------------------	---------------------
	| | |G|G|B|B| | | | |	| | |R|R|R|R| | | | |
	---------------------	---------------------

			   Peritek
		      -----------------
		      |R|R|R|R|G|G|B|B|
		      -----------------
Note:  For daytime only, if the band 1 or band 2 values are over 255, we use
       255 as the pixel value.
================================================================================

Pixels used for nightime display;

		Band 3			Band 4
	---------------------	---------------------
	|G|G|B|B| | | | | | |	|R|R|R|R| | | | | | |
	---------------------	---------------------

			   Peritek
		      -----------------
		      |R|R|R|R|G|G|B|B|
		      -----------------
*******************************************************************************/

	if(VIDBND==VDAYBND)
	    for(i=0;i<1024;i++)
		{
		j = hrptbuff[num][ 749+(5*2*i)+VIDBND];
		if(j >= 256)
			j = 255;
		k = hrptbuff[num][ 749+(5*2*i)+VIDBND+1 ];
		if(k >= 256)
			k = 255;

		if (UP_DOWN == ASCEND)
			vidbuf[1023-i] = 
			  (0x0f & (j>>4)) | (0xf0 & k);
/*			  (0x03 & (j>>4)) | (0x0c & (j>>2)) | (0xf0 & k);*/
		    else
			vidbuf[i] = 
			  (0x0f & (j>>4)) | (0xf0 & k);
/*			  (0x03 & (j>>4)) | (0x0c & (j>>2)) | (0xf0 & k);*/
		}
	    else
	      for(i=0;i<1024;i++)
		{
		j = hrptbuff[num][ 749+(5*2*i)+VIDBND]>>2;
		k = hrptbuff[num][ 749+(5*2*i)+VIDBND+1 ]>>2;
		if (UP_DOWN == ASCEND)
			vidbuf[1023-i] = 
			  (0x0f & (j>>4)) | (0xf0 & k);
/*			  (0x03 & (j>>4)) | (0x0c & (j>>2)) | (0xf0 & k);*/
		    else
			vidbuf[i] = 
			  (0x0f & (j>>4)) | (0xf0 & k);
/*			  (0x03 & (j>>4)) | (0x0c & (j>>2)) | (0xf0 & k);*/
		}
	if(totqual == 0)		/* good line */
		if (UP_DOWN == ASCEND)
			{
			/* write line, scroll down */
			for (i=0;i<1024;i+=VIDCHUNK)
				wrmm(0, peritek_line, i, &vidbuf[i], VIDCHUNK);

			scrlrl(-1);
			peritek_line++;
			if (peritek_line == 1024)
				peritek_line = 0;
			}
		    else
			{
			/* write line, scroll up */
			for (i=0;i<1024;i+=VIDCHUNK)
				wrmm(0, peritek_line, i, &vidbuf[i], VIDCHUNK);

			scrlrl(1);
			peritek_line--;
			if (peritek_line == -1)
				peritek_line = 1023;
			}
	    else			/* dropped (null) line */
		if (UP_DOWN == ASCEND)
			{
			/* write line, scroll down */
			for (i=0;i<1024;i+=VIDCHUNK)
				wrmm(0, peritek_line, i, &nilbuf[i], VIDCHUNK);

			scrlrl(-1);
			peritek_line++;
			if (peritek_line == 1024)
				peritek_line = 0;
			}
		    else
			{
			/* write line, scroll up */
			for (i=0;i<1024;i+=VIDCHUNK)
				wrmm(0, peritek_line, i, &nilbuf[i], VIDCHUNK);

			scrlrl(1);
			peritek_line--;
			if (peritek_line == -1)
				peritek_line = 1023;
			}
	totqual = 0;
	}
return(OK);
}


int	filptr, writcount;
struct	FAB	thefab;
int	sys$qio(), sys$qiow(), stat;
unsigned short	isbfil[4][4];

bufopen(filename,records)
char	filename[];
int	records;
{
int free;
struct
{
	short	len;		/* buffer length			*/
	short	item_code;	/* information item requested		*/
	char	*buf;		/* return buffer			*/
	int	*rlen;		/* return length			*/
	int 	endoflist;
}itemlist = {4,DVI$_FREEBLOCKS,&free,0,0};

$DESCRIPTOR (dua1_desc, "hrpt1:");

thefab = cc$rms_fab;

thefab.fab$l_fna = filename;
thefab.fab$b_fns = strlen(filename);
thefab.fab$b_fac = FAB$M_BIO | FAB$M_PUT;
thefab.fab$l_fop = FAB$M_UFO | FAB$M_CBT;
thefab.fab$l_alq = 1 + 43*records;
thefab.fab$b_shr = FAB$M_SHRPUT | FAB$M_UPI;
thefab.fab$w_ifi = 0;

/* First lets see if there is enough room on the target disk... */
stat = sys$getdviw(0,0,&dua1_desc,&itemlist,0,0,0,0);
if (stat != 1)
	{
	LOGENTRY"BUFOPEN>>Error %d on getdviw\n",stat);
	return(NOTOK);
	}

/* Note: look for 17 extra for file header and possible cluster fragment */
if (free < thefab.fab$l_alq + 17)
	{
	LOGENTRY"BUFOPEN>>Need %d blocks, only %d free\n",
		thefab.fab$l_alq+17,free);
	return(NOTOK);
	}

stat = sys$create(&thefab);
if(stat != RMS$_NORMAL)
	{
	LOGENTRY"BUFOPEN>>image file create error:\n");
	if(stat == RMS$_FUL)
		LOGENTRY"BUFOPEN>>		THE DISK IS FULL!\n");
	return(NOTOK);
	}

writcount = 0;
sys$setef(32);
sys$setef(33);
sys$setef(34);
sys$setef(35);
filptr = 2;
isbfil[0][0] = isbfil[1][0] = isbfil[2][0] = isbfil[3][0] = 1;
return(OK);
}

bufrit(bufnum,sequence)
int	bufnum, sequence;
{
int	i, dumy;
unsigned int stat;
int crashit(), disk_ast();

if(SS$_WASSET != sys$readef(bufnum+32,&dumy))
    {
    LOGENTRY"BUFRIT>>RECORD %d DROPPED DUE TO DISK WRITE LAG ON RECORD %d\n",
	sequence,sequence-4);
    return(OK);
    }

if(!(isbfil[bufnum][0] & 1))
	{
	LOGENTRY"BUFRIT>>BAD IOSB FROM PRIOR DISK WRITE\n");
	LOGENTRY"BUFRIT>>IOSB = %d %d %d %d\n",isbfil[bufnum][0],
		isbfil[bufnum][1],isbfil[bufnum][2],isbfil[bufnum][3]);
	LOGENTRY"BUFRIT>>SEQUENCE NUMBER:%d\n",sequence);
	return(NOTOK);
	}

writcount++;

/* Get time, calculate elapsed millisec since 760 I/O completed.  Stash
result in millisec array indexed by sequence-1 */
sys$gettim(&stop_time);
subquad (&stop_time,&start_time[sequence-1],&elapsed_time);
ediv (&elapsed_time,&shift,&elapsed_millisec);
elapsed[sequence-1][0] = elapsed_millisec;

/* Adjust file pointer for overwriting previous record(s) */
if (overwrite > 0)
	{
	filptr -= 43 * overwrite;
	if (filptr < 2)
		filptr = 2;
	writcount -= overwrite;
	overwrite = 0;
	}

stat = sys$qio (bufnum+32,thefab.fab$l_stv,IO$_WRITEVBLK,isbfil[bufnum],
	disk_ast,sequence,&hrptbuff[bufnum][0],22016,filptr,0,0,0);

filptr += 43;
if(!(stat & 1))
	{
	LOGENTRY"BUFRIT>>BAD STATUS FROM DISK WRITE\n");
	LOGENTRY"BUFRIT>>SEQUENCE NUMBER:%d\n",sequence);
	return(NOTOK);
	}
return(OK);
}

buflsh(records,orbit,year,satid,delta)
int	records, orbit, year, satid, delta;
{
int	ritesize, dumy, counter;

for(counter=0;counter<4;counter++)
	{
	stat = sys$waitfr(counter+32);
	if(!(stat & 1))
		LOGENTRY"BUFLSH>>BAD STATUS FROM DISK WAIT: stat = %d\n",stat);
	if(!(isbfil[counter][0] & 1))
	LOGENTRY"BUFLSH>>BAD IOSB FROM DISK WRITE (#%d):iosb[0] = %d\n",counter,
		isbfil[counter][0]);
	}

LOGENTRY"BUFLSH>>THE DISK WAS ACCESSED %d TIMES\n",writcount);

header(&hrptbuff[0][0],records,orbit,year,satid,delta);
stat = sys$qiow(32,thefab.fab$l_stv,IO$_WRITEVBLK,
	isbfil[0],0,0,&hrptbuff[0][0],512,1,0,0,0);
if(!(stat & 1))
	LOGENTRY"BUFLSH>>HEADER  record write failed!:	STATUS = %d\n",stat);
if(!(isbfil[0][0] & 1))
	LOGENTRY"BUFLSH>>BAD IOSB FROM header WRITE:iosb[0] = %d\n"
			,isbfil[0][0]);

stat = sys$dassgn(thefab.fab$l_stv);
if(stat != SS$_NORMAL)
	LOGENTRY"BUFLSH>>ERROR CLOSING IMAGE FILE: %d\n",stat);
return(OK);
}
disk_ast(sequence)
int sequence;
{
unsigned int	lcl_stop_time[2],lcl_elapsed_time[2];
unsigned int	lcl_elapsed_millisec;

sys$gettim(&lcl_stop_time);
subquad (&lcl_stop_time,&start_time[sequence-1],&lcl_elapsed_time);
ediv (&lcl_elapsed_time,&shift,&lcl_elapsed_millisec);
elapsed[sequence-1][1] = lcl_elapsed_millisec;
}
