#include	<errno.h>
#include	<math.h>
#include	"acquman.h"
static	int	memorylocation = 1;
#define	HDTRADDSPC	5
#define MIN_SECONDS     60
#define HOUR_SECONDS    (MIN_SECONDS * MIN_SECONDS)
#define DAY_SECONDS     (24 * HOUR_SECONDS)
#define YR_SECONDS      (365 * DAY_SECONDS)
#define LEAP_YR_SECONDS (366 * DAY_SECONDS)

/*
==========================================================================
==									==
==		ACQUSUBS: acquisition manager subroutines		==
==									==
==	This file contains all of the subroutines called by the 	==
==	acquisition manager except for the ACQUIRE routine.  Their 	==
==	calling	sequences are as follows:				==
==									==
==	#include	"[reed]acquman.h"				==
==	struct	SCHED	wo;	/* WORK ORDER: one schedule record	==
==	int	mode;						==
==	char	newflag;						==
==	int	file, freqnum, start, tapeloc;				==
==									==
==	getinstr(&wo);							==
==	markstat(wo);							==
==	scanacu(start);							==
==	setupipe(wo.opcode,numsat);					==
==	searchdt(start,file);						==
==	startape(&file);						==
==	stoptape(file,wo.opcode);					==
==	scan(&username,&state);						==
==========================================================================
*/

getinstr(woptr)
struct	SCHED	*woptr;			/* WORK ORDER: one schedule record*/
/*
==========================================================================
==									==
==	This scans the schedule file for the first record whose		==
==	status is pending.  If no pending requests are found,		==
==	this returns NOTOK.						==
==									==
==========================================================================
*/
{
FILE	*stream;
int	i, records;
int	year, day, hour, minute, second;
extern	int noshare	errno;
extern	int noshare	vaxc$errno;

/********************************************************************/
#ifdef	SINGLESTEP

year = get_year();

printf("\nSINGLESTEP>enter satellite id (0 to exit):");
scanf("%3x",&woptr->satid);
if(woptr->satid == 0)exit(1);
if (strcmp(woptr->satid,"N09") == 0)
	numsat = 1;
  else if (strcmp(woptr->satid,"N10") == 0)
	numsat = 2;
  else if (strcmp(woptr->satid,"N11") == 0)
	numsat = 3;
  else if (strcmp(woptr->satid,"N12") == 0)
	numsat = 4;

printf("SINGLESTEP>enter start time:(ddd:hh:mm:ss)");
scanf("%3d:%2d:%02d:%02d",&day, &hour, &minute, &second);
JULTIM_TO_TOTSECS(second,minute,hour,day,year,woptr->lstart);

sprintf(&woptr->scene_id,"AH%02d%02d%03d%02d%02d%01d0",
		numsat +8, year, day, hour, minute, second/10);

printf("SINGLESTEP> enter stop time:(ddd:hh:mm:ss)");
scanf("%3d:%2d:%02d:%02d",&day, &hour, &minute, &second);
JULTIM_TO_TOTSECS(second,minute,hour,day,year,woptr->lstop);

printf("SINGLESTEP>enter op. code:");
scanf("%1x",&woptr->opcode);
woptr->stat[0] = 'P';

printf("SINGLESTEP>enter tape file no.:");
scanf("%2d",&woptr->tfile);

return(OK);
#endif	SINGLESTEP
/********************************************************************/

#ifdef	DEBUG
LOGENTRY"GETINSTR> scan schedule file for workorder\n");
#endif	DEBUG

for( ;NULL==(stream=fopen(SCHEDULE,"r+")); sleep(60) )
	{
	LOGENTRY"GETINSTR> SCHEDULE FILE OPEN FAILED:errno = %x\n",errno);
	if(errno == EVMSERR)
	    {
	    if (vaxc$errno == 98954)
		LOGENTRY"GETINSTR>>Schedule file locked by another user\n");
	      else
		LOGENTRY"GETINSTR>>vax c error number = %x\n",vaxc$errno);
/*	    if(vaxc$errno == 65536)zero();
*/	    }
	LOGENTRY"GETINSTR>>sleep one minute and try again\n");
	}

if(0 == fscanf(stream,"%d\n",&records))
	{
	LOGENTRY"GETINSTR>>FIRST SCHEDULE FILE READ FAILED\n");
	if(0!=fclose(stream))
	   LOGENTRY"GETINSTR>>SCHEDULE FILE CLOSE FAILED:errno = %d\n",errno);
	return(NOTOK);
	}

for(i=0;i<records;i++)
	{
	GETSCHREC(stream,woptr);
	if((woptr->stat[0] == PENDING) && (woptr->station[0] == 'E'))
		{
if(0!=fclose(stream))
	   LOGENTRY"GETINSTR> SCHEDULE FILE CLOSE FAILED:errno = %d\n",errno);
#ifdef	FOROPS
		LOGENTRY"GETINSTR> ");
		PUTSCHREC(LOGFILE,woptr);
		make_julian(woptr->lstart,&year,&day,&hour,&minute,&second);
		LOGENTRY"GETINSTR> startime = %d, %03d:%02d:%02d:%02d\n",
			year, day, hour, minute, second);
#endif	FOROPS
		return(OK);
	}	}
if(0!=fclose(stream))
	   LOGENTRY"GETINSTR> SCHEDULE FILE CLOSE FAILED:errno = %d\n",errno);
return(NOTOK);
}

markstat(newo)
struct	SCHED	newo;
/*
==========================================================================
==									==
==	This reads the schedule file until an record whose start-	==
==	time is equal to 'newo.lstart'.  Then, after that record's	==
==	acquired flag is set, it's written back into the schedule	==
==	file.								==
==									==
==========================================================================
*/
{
FILE	*stream;
int	i, records;
struct	SCHED	wo;

#ifdef	SINGLESTEP
return(OK);
#endif

#ifdef	DEBUG
LOGENTRY"MARKSTAT>update workorder in schedule file\n");
#endif	DEBUG
if(NULL == (stream = fopen(SCHEDULE,"r+")) )
	{
	LOGENTRY"MARKSTAT>>ERROR:");
	LOGENTRY"MARKSTAT>>SCHEDULE FILE OPEN FAILED\n");
	return(NOTOK);
	}

if(0 == fscanf(stream,"%d\n",&records))
	{
	LOGENTRY"MARKSTAT>>FIRST SCHEDULE FILE READ FAILED\n");
	return(NOTOK);
	}

for(i=0;i<records;i++)
	{
	GETSCHREC(stream,&wo);
	if(newo.lstart == wo.lstart)goto	MARKIT;
	}
fclose(stream);
if(newo.opcode == SOFTPATGEN)return(OK);	/*may not be in schedule */
LOGENTRY"MARKSTAT>>schedule entry not found!\n");
LOGENTRY"MARKSTAT>>PROBABLE SOFTWARE ERROR (bug)\n");
return(NOTOK);

MARKIT:
#ifdef	FOROPS
		LOGENTRY"MARKSTAT> Old: ");
		PUTSCHREC(LOGFILE,&wo);
		LOGENTRY"MARKSTAT> New: ");
		PUTSCHREC(LOGFILE,&newo);
#endif	FOROPS
fseek(stream,-RECSIZ,1);

/* Prevent schedule file corruption due too invalid tapefile value.  Pass will
   be marked as "missed" upon next schedule scan... but not corrupted! */
if (newo.tfile < 100)
	PUTSCHREC(stream,&newo);
    else
	LOGENTRY"MARKSTAT>> Schedule update supressed to prevent corruption\n");
fclose(stream);
return(OK);
}

scanacu(satid)
int	satid;
/*
==========================================================================
==									==
==	This querys the ACU for the start-time of its next predicted	==
==	acquisition.  If the ACU's response is for a satelite pass	==
==	that will occur before the well before the beginning of the	==
==	desired pass, the ACU is instructed not to receive that pass.	==
==	This process is repeated until the ACU responds with a start-	==
==	time that matches the requested pass.  If the ACU responds	==
==	with a pass that will occur after the requested one, then 	==
==	this routine returns an error condition.			==
==									==
==========================================================================
*/
{
int	nextsat, nextime;

for(;;)	{
	askacu(&nextsat,&nextime);
/************************************************/
/*						*/
/*	until acu control code is written:	*/
/*						*/
		acumode(ON);return(OK);
/*						*/
/************************************************/

	if(nextsat == satid)
		{
		acumode(ON);
		return(OK);
		}
	telacu();
	}
}

setupipe(mode,freqnum)
int	mode,freqnum;
/*========================================================================
==									==
==	This subroutine generates the 18 sixteen-bit instructions that	==
==	are required to control the Model 726-01 PCM Bit Synchronizer,	==
==	the Model 720 Bit synchronizer, the 2 sixteen-bit instructions  ==
==	for the frame sync., and the 1 four-bit instruction for the	==
==	receiver.  Only one bit synch may be in the pipeline at a time. ==
==									==
========================================================================*/

{
int	chan_rcvr;	/* Channel for new receiver */
int	status;
short	outbuf[13];	/* (0-10=bs726 or 0-4=bs720) 11-12=fs */
short	framcom[4] = 	{
			0x10f0,	/* SLOW	OPERATIONAL	command */
			0x1efa,	/* SLOW	TEST		command */
			0x10f0,	/* FAST	OPERATIONAL	command */
			0x1ef9	/* FAST	TEST		command */
			};
int	i;

#define	LIVE	0x00f0;
#define	FAST	0x0070;

#ifdef	DEBUG
	LOGENTRY"SETUPIPE>configuring HRPT Data Pipeline\n");
#endif	DEBUG

/* NOTE: If we are using the 720 bit synch, elements 5 thru 10 will be 0, and
	 outpipe will ignore them. */

for(i=0;i<13;i++)outbuf[i] = 0;

if((mode&0x04) == 4)	/* data from the HDT */
	{
#ifdef BIT_SYNCH_726
	outbuf[0] = (BITSYNC<<12) + (0<<8) + 2;	/* data from the HDT */
	outbuf[1] = (BITSYNC<<12) + (1<<8) + 0; /* normal polarity */
	outbuf[2] = (BITSYNC<<12) + (2<<8) + 03;/* RNRZL from HDT */
	outbuf[3] = (BITSYNC<<12) + (3<<8) + 0;	/* 9-bit fwd random no. gen. */
	outbuf[4] = (BITSYNC<<12) + (4<<8) + 3;	/* RNRZL to HDT always */
	outbuf[5] = (BITSYNC<<12) + (5<<8) + 1;	/* 0.1% loop bandwidth	*/
	outbuf[6] = (BITSYNC<<12) + (6<<8) + (PBCKBTRAT[6] & 0X0F);
	outbuf[7] = (BITSYNC<<12) + (7<<8) + (PBCKBTRAT[0] & 0X0F);
	outbuf[8] = (BITSYNC<<12) + (8<<8) + (PBCKBTRAT[2] & 0X0F);
	outbuf[9] = (BITSYNC<<12) + (9<<8) + (PBCKBTRAT[3] & 0X0F);
	outbuf[10]= (BITSYNC<<12) +(10<<8) + (PBCKBTRAT[4] & 0X0F);
#else
	outbuf[0] = (AXBSYNC<<12) + (1<<8) 
			+ ((PBCKBTAUX[0] & 0X0F)<<4) + (PBCKBTAUX[2] & 0X0F);
	outbuf[1] = (AXBSYNC<<12) + (2<<8)
			+ ((PBCKBTAUX[3] & 0X0F)<<4) + (PBCKBTAUX[4] & 0X0F);
	outbuf[2] = (AXBSYNC<<12) + (3<<8) + (PBCKBTAUX[6] & 0X0F);
	outbuf[4] = (AXBSYNC<<12) + (4<<8) + (2<<3) + 5;
	outbuf[3] = (AXBSYNC<<12) + (5<<8) + (1<<7);
#endif
	}
    else
	{
#ifdef BIT_SYNCH_726
	outbuf[0] = (BITSYNC<<12) + (0<<8) + 1;	/* data from the Receiver */
	outbuf[1] = (BITSYNC<<12) + (1<<8) + 0; /* normal polarity */
	outbuf[2] = (BITSYNC<<12) + (2<<8) + 12;/* BiPhase-L from Receiver */
	outbuf[3] = (BITSYNC<<12) + (3<<8) + 0;	/* 9-bit fwd random no. gen. */
	outbuf[4] = (BITSYNC<<12) + (4<<8) + 3;	/* RNRZL to HDT always */
	outbuf[5] = (BITSYNC<<12) + (5<<8) + 1;	/* 0.1% loop bandwidth	*/
	outbuf[6] = (BITSYNC<<12) + (6<<8) + (LIVBITRAT[6] & 0X0F);
	outbuf[7] = (BITSYNC<<12) + (7<<8) + (LIVBITRAT[0] & 0X0F);
	outbuf[8] = (BITSYNC<<12) + (8<<8) + (LIVBITRAT[2] & 0X0F);
	outbuf[9] = (BITSYNC<<12) + (9<<8) + (LIVBITRAT[3] & 0X0F);
	outbuf[10]= (BITSYNC<<12) +(10<<8) + (LIVBITRAT[4] & 0X0F);
#else
	outbuf[0] = (AXBSYNC<<12) + (1<<8) 
			+ ((LIVBITAUX[0] & 0X0F)<<4) + (LIVBITAUX[2] & 0X0F);
	outbuf[1] = (AXBSYNC<<12) + (2<<8)
			+ ((LIVBITAUX[3] & 0X0F)<<4) + (LIVBITAUX[4] & 0X0F);
	outbuf[2] = (AXBSYNC<<12) + (3<<8) + (LIVBITAUX[6] & 0X0F);
	outbuf[4] = (AXBSYNC<<12) + (4<<8) + (1<<3) + 5;
	outbuf[3] = (AXBSYNC<<12) + (5<<8) + (1<<2);
#endif
	}

for(i=0;i<2;i++)outbuf[11+i] = framcom[i+((mode&0x04)>>1)];

#ifdef BIT_SYNCH_726
if((mode&0x80)==0x80)	/* get data from test pattern generater */
	outbuf[0] = (BITSYNC<<12) + (0<<8) + 4;
#endif

/****** Send commands to "new" receiver to switch channel ******/

if((mode&0x04) == 0)			/* data from the DISH */
	{
	chan_rcvr = ttopen(RCVRS232);
/* freqnum is actually the numeric satellite number */
	if (freqnum == 1)
		{
		commrcvr(chan_rcvr," ");
		status = commrcvr(chan_rcvr,"X1");
		LOGENTRY"SETUPIPE> Setting rcvr to chan 1\n");
		}
	    else if (freqnum == 2)
		{
		commrcvr(chan_rcvr," ");
		status = commrcvr(chan_rcvr,"X2");
		LOGENTRY"SETUPIPE> Setting rcvr to chan 2\n");
		}
	    else if (freqnum == 3)
		{
		commrcvr(chan_rcvr," ");
		status = commrcvr(chan_rcvr,"X3");
		LOGENTRY"SETUPIPE> Setting rcvr to chan 3\n");
		}
	    else if (freqnum == 4)
		{
		commrcvr(chan_rcvr," ");
		status = commrcvr(chan_rcvr,"X2");
		LOGENTRY"SETUPIPE> Setting rcvr to chan 2\n");
		}
	    else if (freqnum == 5)
                {
                commrcvr(chan_rcvr," ");
                status = commrcvr(chan_rcvr,"X5");
                LOGENTRY"SETUPIPE> Setting rcvr to chan 5\n");
                }
	    else if (freqnum == 6) /* Added for NOAA 15 */
		{
		commrcvr(chan_rcvr," ");
		status = commrcvr(chan_rcvr,"X2");
		LOGENTRY"SETUPIPE> Setting rcvr to chan 2\n");
		}
	    else if (freqnum == 7) /* Added for NOAA 16 */
		{
		commrcvr(chan_rcvr," ");
		status = commrcvr(chan_rcvr,"X5");
		LOGENTRY"SETUPIPE> Setting rcvr to chan 5\n");
		}
	    else if (freqnum == 8) /* Added for NOAA 17 */
		{
		commrcvr(chan_rcvr," ");
		status = commrcvr(chan_rcvr,"X1");
		LOGENTRY"SETUPIPE> Setting rcvr to chan 1\n");
		}
	    else
		{
#ifdef	DEBUG
		LOGENTRY"SETUPIPE>>Invalid freqnum %d\n",freqnum);
#endif	DEBUG
		status == OK;
		}
#ifdef	DEBUG
	if (status != OK)
		LOGENTRY"SETUPIPE>>Problem setting receiver channel\n");
#endif	DEBUG
	ttclos(chan_rcvr);
	}
return(outpipe(outbuf,13,freqnum));
}

searchdt(start,file)
int	start, file;
{
/*
==========================================================================
==									==
==	This scans the HDT for the first record of the desired pass	==
==									==
==========================================================================
*/
int	destfoot, currfoot, timestamp, backstep;
int	chan, lastop;
int	yr, day, hour, min, sec, secofyear;
char	buffer[16];
int	loopcount, lastfind;

make_julian(start,&yr,&day,&hour,&min,&sec);
secofyear = sec + 60 * (min + 60 * (hour + 24*(day-1)));

#ifdef	DEBUG
	LOGENTRY"SEARCHDT>looking for file number %d.\n",file);
#endif	DEBUG

if(OK !=mountape(start))return(NOTOK);
if(OK != spinto(file))	return(NOTOK);

chan = ttopen(HDTRS232);
commtape(chan,"CMSTP");		/* Load tape */
sleep(3);			/* Allow time for tape to load */
commtape(chan,"TROFAL");

#ifdef	DEBUG
	LOGENTRY"SEARCHDT>DESTINATION TIME IS %d SECONDS INTO YEAR\n",
			secofyear);
#endif	DEBUG

loopcount = lastfind = 0;
CHECKTIME:
commtape(chan,SSPDREAD);
if(((file / 10)&1)==1)	commtape(chan,"CMFWD");/* start HDT moving */
else			commtape(chan,"CMREV");/* start HDT moving */

if(OK != findtime(&timestamp))	/*readtime stamp from 1 HRPT minor frame*/
	{
	ttclos(chan);
	return(NOTOK);
	}
commtape(chan,"CMSTP");	/* stop the HDT */
#ifdef	DEBUG
	LOGENTRY"SEARCHDT>FOUND RECORD FROM %d SECONDS INTO YEAR\n",
			timestamp );
#endif	DEBUG
backstep = secofyear - timestamp;
backstep *= 2.5; /*feet per second record speed*/

if(((loopcount<2)||(lastfind != timestamp))&&(backstep < 0))
	{
	loopcount++;	lastfind = timestamp;
	backstep -= EXTRASPACE;
	commtape(chan,"FTUSE");
	resptape(chan,buffer);
	sscanf(buffer,"%d",&destfoot);
	if(((file/10)&1)==1)	destfoot += backstep;
	else			destfoot -= backstep;

#ifdef	DEBUG
	LOGENTRY"SEARCHDT>DESTINATION FOOT COUNTER IS %d\n",destfoot);
#endif	DEBUG
	if((destfoot < -500)||(destfoot > 8000))
		{
		ttclos(chan);
		return(NOTOK);
		}
	lastop = 0;
	commtape(chan,SSPDWRIT);
LOOP:	commtape(chan,"FTUSE");
	resptape(chan,buffer);
	sscanf(buffer,"%d",&currfoot);
	if (((destfoot-currfoot)<3)&&((currfoot-destfoot)<3))
		commtape(chan,"CMSTP");
	else if (destfoot<currfoot)
		{
		if(lastop == 3)goto LOOP;
		lastop = 3;
		commtape(chan,"CMREV");
		goto LOOP;
		}
	else if (destfoot>currfoot)
		{
		if(lastop == 4)goto LOOP;
		lastop = 4;
		commtape(chan,"CMFWD");
		goto LOOP;
		}
	goto	CHECKTIME;
	}
if(((file / 10)&1)!=1)	commtape(chan,"CMFWD");/* start HDT moving */
else			commtape(chan,"CMREV");/* start HDT moving */
sleep(1);
commtape(chan,"CMSTP");
ttclos(chan);
return(OK);
}

mountape(totsec)
int	totsec;
{
int	dayofyear;
int	year, month, date, hour, minute, second;
int	stat;

char	answer[512], buffer[16];
short	chan, iosb[4];

char	opmess[] = "\000\377\377\377\000\000\000\000\
PLEASE MOUNT HDT WITH DATA FROM 01/01/1987,22:30:00";
$DESCRIPTOR(opdsc,opmess);

char	virmes[] = "\000\377\377\377\000\000\000\000\
PLEASE MOUNT A VIRGIN HIGH DENSITY TAPE";
$DESCRIPTOR(virdsc,virmes);

char	replyto[]= "\000\377\377\377\000\000\000\000\
REPLY/TO=nn [	   A(bort: don't do acquisition)\r\n\
		or C(ontinue using current tape)\r\n\
		or N(ew tape has been mounted)]\r\n";
$DESCRIPTOR(rply2,replyto);

char	badans[]= "\000\377\377\377\000\000\000\000\
INVALID OPERATOR RESPONSE";
$DESCRIPTOR(badsc,badans);

virmes[0] = opmess[0] = replyto[0] = badans[0] = OPC$_RQ_RQST;

stat = sys$crembx(0,&chan,512,0,0,0,0);
if((stat & 0x01)==0)
	{
	LOGENTRY"MOUNTAPE>>ERROR CREATING MAILBOX FOR OPERATOR RESPONSE\n");
	return(NOTOK);
	}

if(totsec == 0)
	{
	stat = sys$sndopr(&virdsc,0);
	if((stat & 0x01)==0)
		{
		LOGENTRY"MOUNTAPE>>ERROR SENDING MESSAGE TO OPERATOR\n");
		return(NOTOK);
		}
	}
else	{
	make_julian(totsec, &year, &dayofyear, &hour, &minute, &second);
	JULGRE(year,dayofyear, month,date);

	sprintf(&opmess[40],	"%02d/%02d/%04d,%02d:%02d:%02d",
		month,date,year,hour,minute,second);

	stat = sys$sndopr(&opdsc,0);
	if((stat & 0x01)==0)
		{
		LOGENTRY"MOUNTAPE>>ERROR SENDING MESSAGE TO OPERATOR\n");
		return(NOTOK);
		}
	}

GETANSWER:;
stat = sys$sndopr(&rply2,chan);
if((stat & 0x01)==0)
	{
	LOGENTRY"MOUNTAPE>>ERROR SENDING MESSAGE TO OPERATOR\n");
	return(NOTOK);
	}

stat = sys$qiow(0,chan,IO$_READVBLK,&iosb,0,0,answer,512,0,0,0,0);
if((stat & 0x01)==0)
	{
	LOGENTRY"MOUNTAPE>>ERROR RECEIVING ANSWER FROM OPERATOR\n");
	return(NOTOK);
	}

answer[iosb[1]] = 0x00;
#ifdef	DEBUG
	LOGENTRY"MOUNTAPE>>The operator replied:%s\n",&answer[8]);
#endif	DEBUG

switch(answer[8])
	{
case 'a':
case 'A':LOGENTRY"MOUNTAPE>>operator requested abort.\n");
	 return(NOTOK);

case 'c':
case 'C':return(OK);

case 'n':
case 'N':chan = ttopen(HDTRS232);
	commtape(chan,"CMSTP");		/* Load tape */
	sleep(3);			/* Allow time for tape to load */
	/* The following lines position the tape to 500 feet and zero counter */
	commtape(chan,"FTUSE 2000");	/* set foot counter to 02000	*/
	spinto(11);/* select channels 1 & 8  & go to footcount 02500	*/
	commtape(chan,"FTUSE 00000");	/* set foot counter to 00000	*/
	ttclos(chan);
	return(OK);

default :stat = sys$sndopr(&badsc,0);
	if((stat & 0x01)==0)
		{
		LOGENTRY"MOUNTAPE>>ERROR SENDING MESSAGE TO OPERATOR\n");
		return(NOTOK);
		}
	goto	GETANSWER;
	}
}

startape(file,whatodo)
int	*file;
enum	READWRITE	whatodo;
{
int	footcount, filler1, filler2, selectrack;  /* filler prevents clobber */
int	chan, i, counter;
char	buffer[10];

chan = ttopen(HDTRS232);
commtape(chan,"CMSTP");		/* Load tape */
sleep(3);			/* Allow time for tape to load */

for(footcount=12000,counter=0;(footcount > 10050) && (counter < 10);counter++)
	{
	LOGENTRY"STARTAPE> poll tapedrive for footcount\n");
	commtape(chan,"FTUSE");
	resptape(chan,buffer);
	sscanf(buffer,"%d",&footcount);
	}
if (counter == 10)
	LOGENTRY"STARTAPE>> Failed to get good tape counter\n");

#ifdef	DEBUG
LOGENTRY"STARTAPE> THERE ARE %d FEET USED\n",footcount);
LOGENTRY"STARTAPE> poll tapedrive for selected channel\n");
#endif	DEBUG

commtape(chan,"CHA");
resptape(chan,buffer);
sscanf(buffer,"%d",&selectrack);

if((selectrack&0x01)==0x01)
	*file = (footcount+10)/FILESIZE + 10*selectrack;
    else
	*file = 1 + (footcount-10)/FILESIZE + 10*selectrack;

#ifdef	DEBUG
LOGENTRY"STARTAPE> HDTR FILE NUMBER IS %d\n",*file);
#endif	DEBUG

if(whatodo == RECORD)
	{
#ifdef	DEBUG
	LOGENTRY"STARTAPE> setup HDTR to go at 30 ips and RECORD\n");
#endif	DEBUG
	commtape(chan,"TROFAL");
	sprintf(buffer,"TRON%02d",selectrack);
 	commtape(chan,buffer);
	sprintf(buffer,"TRON%02d",selectrack+7);
 	commtape(chan,buffer);
	commtape(chan,SSPDWRIT);
	if((selectrack&1)==1)	commtape(chan,"CMRFD");
	else			commtape(chan,"CMRRV");
	}
else	{
#ifdef	DEBUG
	LOGENTRY"STARTAPE>setup HDTR to go at 60 ips and PLAYBACK\n\n");
#endif	DEBUG
	commtape(chan,"TROFAL");
	sprintf(buffer,"CHA%d",selectrack);
	commtape(chan,buffer);
	sprintf(buffer,"CHB%d",selectrack+7);
	commtape(chan,buffer);
	commtape(chan,SSPDREAD);
	if((selectrack&1)==1)	commtape(chan,"CMFWD");
	else			commtape(chan,"CMREV");
	}
ttclos(chan);
return(OK);
}

stoptape(file,mode)
int	file, mode;
{
int	footcount, newfoot, i, nextfile;
int	chan;				/* logical unit number of I/O port */
int	counter;
char	buffer[16];


chan = ttopen(HDTRS232);
commtape(chan,"CMSTP");

for(newfoot=12000,counter=0;(newfoot > 10050) && (counter < 10);counter++)
	{
	LOGENTRY"STOPTAPE> poll tapedrive for footcount\n");
	commtape(chan,"FTUSE");
	resptape(chan,buffer);
	sscanf(buffer,"%d",&newfoot);
	}
if (counter == 10)
	LOGENTRY"STOPTAPE>> Failed to get good tape counter\n");

ttclos(chan);

#ifdef	DEBUG
LOGENTRY"STOPTAPE>THERE ARE %d FEET USED\n",newfoot);
#endif	DEBUG

if((mode&0x04)==0x04)return(OK);

if(((file/10)&0x01)==0x01)
	{
	nextfile = file + 1;
	if ((nextfile % 10) == FILONTRK)
		nextfile += 10;
	}
    else
	{
	nextfile = file - 1;
	if ((nextfile % 10) == 0)
		nextfile += 10;
	}
if (nextfile >= 80)
	{
	LOGENTRY"STOPTAPE>END OF TAPE - NEXT FILE IS 10\n");
	nextfile = 10;
	}
footcount = (nextfile%10) * FILESIZE;

/* If nextfile is on this track but we are already positioned past
   the foot marker for BOF, then we must have written a long file!  If
   so, then skip to next file. */
if (((file /10) != (nextfile / 10)) ||
    ((footcount > newfoot)&&(((file/10)&0x01)==0x01)) ||
  	    ((footcount < newfoot)&&(((file/10)&0x01)==0x00)))
		spinto(nextfile);
    else
	{
	if(((nextfile/10)&0x01)==0x01)
		{
		nextfile = nextfile + 1;
		if ((nextfile % 10) == FILONTRK)
			nextfile += 10;
		}
	    else
		{
		nextfile = nextfile - 1;
		if ((nextfile % 10) == 0)
			nextfile += 10;
		}
	if (nextfile >= 80)
		{
		LOGENTRY"STOPTAPE>END OF TAPE - NEXT FILE IS 10\n");
		nextfile = 10;
		}
	    else
		LOGENTRY"STOPTAPE>Last pass was long - skipping to file %d\n"
			,nextfile);

	spinto(nextfile);
	}

return(OK);
}

#define	AWHILE	2000

nbsecond(totsec)
int	*totsec;
{
static	int	lastime;
int	day, hour, min, sec, millisec, stat;
int	year, deltime;

stat = nbstime(&day, &hour, &min, &sec, &millisec);
if(stat != OK)return(stat);

year = get_year();

#ifdef	DEBUG
LOGENTRY"NBSECOND> its %d, %d:%02d:%02d:%02d and all is well\n",
	year, day, hour, min, sec);
#endif	DEBUG

JULTIM_TO_TOTSECS(sec,min,hour,day,year,*totsec);

if((*totsec < lastime)||(*totsec > lastime+AWHILE))
	{
	LOGENTRY"NBSECOND>>sanity check failed:%d<%d<%d\n",
					lastime,*totsec,lastime+AWHILE);
	lastime = *totsec;
	if(OK != nbsecond(totsec))return(NOTOK);
	}
lastime = *totsec;
return(OK);
}

droplog(scid,mess,text,logname)
char	scid[], mess[], text[], logname[];
{
struct	DESC	siddesc, mesdesc, txtdesc, logdesc;
char	sceneid[17], message[17], explain[61], log[61], pad[61];
sprintf(pad,"                                                            ");

strncpy(sceneid,scid,16);
strncpy(message,mess,16);
strncpy(explain,text,60);
strncpy(log,logname,60);

strncat(sceneid,pad,16-strlen(sceneid));
strncat(message,pad,16-strlen(message));
strncat(explain,pad,60-strlen(explain));
strncat(log,pad,60-strlen(log));

siddesc.addr = sceneid;		siddesc.length = 16;
mesdesc.addr = message;		mesdesc.length = 16;
txtdesc.addr = explain;		txtdesc.length = 60;
logdesc.addr = log;		logdesc.length = 60;

return( logwrt(&siddesc,&mesdesc,&txtdesc,&logdesc) );
}
# define PSCAN$_NODENAME 13
# define PSCAN$_GETJPI_BUFFER_SIZE 24

scan(usr,state)
char *usr;
int *state;
{
int 	context;		/* scan context */
int	pid;			/* process identification */
char	username[13];		/* process name */
int	owner;			/* owner if subprocess */
char	nodename[6] = "EROSH";	/* nodename of remote system */
int	scan_iosb[2], proc_state;

struct	JPI_ITEM
{
	short	len;			/* buffer length */
	short	item_code;		/* information item requested */
	char	*buf;			/* return buffer */
	int	*rlen;			/* return length */
};

struct	SCAN_ITEM
{
	short	len;			/* buffer length */
	short	item_code;		/* information item requested */
	char	*buf;			/* return buffer */
	int	code;			/* item code */
};

struct
{
	struct	JPI_ITEM items[4];
	int	endoflist;
} itemlist				=
					{
		/* pid		*/	 4 ,JPI$_PID	,&pid		,0
		/* process name */	,12,JPI$_USERNAME,username	,0
		/* owner	*/	,4 ,JPI$_OWNER	,&owner		,0
		/* state	*/	,4 ,JPI$_STATE	,&proc_state	,0
					};
struct
{
	struct	SCAN_ITEM items[2];
	int	endoflist;
} scanlist				=
					{
		/* node		*/	 5 ,PSCAN$_NODENAME,nodename,0
		/* buffer	*/	,0 ,PSCAN$_GETJPI_BUFFER_SIZE,1000,0
					};

short	status;				/* status returned by calls */
int	counter;

	itemlist.endoflist = scanlist.endoflist = 0;
	status = sys$process_scan(&context,&scanlist);
	if (status != SS$_NORMAL)
		{
		printf("SCAN>> scan status = %d\n",status);
		return(0);
		}

	for(status=0;status != SS$_NOMOREPROC;)
		{
		status = sys$getjpiw(1,&context,0,&itemlist,&scan_iosb,0,0);
		/* Check next process if this one returns a bad status */
		if (status == SS$_NORMAL)
			status = scan_iosb[0];
		if (status != SS$_NORMAL)
			continue;

		/* Un-pad the user name */
		for (counter=12;counter >= 0;counter--)
			{
			if (username[counter] == 32)
				username[counter] = 0;
			}
		if ((strcmp(usr,username) == 0) && (owner == 0))
			{
			*state = proc_state;
			return(pid);
			}
		}
	return(0);
}


/*-------------------------------------------------------
Get and return a 2-digit system year.  An offset is 
applied to adjust the year in case the system clock is
set to a year other than the current year.
---------------------------------------------------------*/
int get_year()
{
unsigned int    bintim[2];	/* system binary time		*/
char    year_str[5];		/* 4-digit year string		*/
int     int_year;		/* integer value of year	*/
int     date_len;		/* length of date string	*/
struct  DESC time_desc;		/* vax time descriptor		*/
char    date_str[30];		/* vax system date string	*/

/* get the system time
----------------------*/
sys$gettim( &bintim );
time_desc.addr = &date_str;
time_desc.length = 25;
sys$asctim( &date_len, &time_desc, &bintim, 0 );

/* extract the year from the string
-----------------------------------*/
strncpy( year_str, &date_str[7], 4 );
year_str[4] = 0;
sscanf( year_str, "%d", &int_year );

/* apply the offset and scrunch to 2-digits
-------------------------------------------*/
int_year = YEAR_OFFSET + int_year;
int_year %= 100;

return int_year;
}


/***********************************************************************
FUNCTION:	GET_OFFSET

PURPOSE:	Calculate an offset in seconds between the local system
		clock and the NBS clock.  This function was written to
		replace the localtime() call since it is unreliable in
		year 2000.
************************************************************************/
int get_offset( int now )
{
char    date_str[30];           /* the date string from asctim()        */

int	hour;			/* local system hour			*/
int	min;			/* local system minute			*/
int	nbsday;			/* dummy for make_julian call		*/
int	nbsec = now;		/* local copy of NBS time		*/
int	nbshr;			/* NBS hours				*/
int	nbsmin;			/* NBS minutes				*/
int	nbssec;			/* dummy for make_julian call		*/
int	nbsyr;			/* dummy for make_julian call		*/
int     date_len;               /* length of date string                */
int     i;                      /* loop counter                         */

static	int offset = 0;		/* offset in seconds to be returned	*/

struct  DESC time_desc;         /* time descriptor for asctim() call    */

unsigned int    bintim[2];      /* vax binary time                      */


/* Compute an offset in seconds.  We need to insure that an hour is
not spanned thereby computing the wrong offset.  So if the offset has
been previously set and the NBS and local minute are on opposite
sides of an hour return the old offset.  Otherwise loop until the
minutes are both on the same side of the hour and compute the offset.
---------------------------------------------------------------------*/
do
    {
    /* Get the system binary time and convert it to an ASCII string
    ---------------------------------------------------------------*/
    sys$gettim( &bintim );
    time_desc.addr = &date_str;
    time_desc.length = 25;
    sys$asctim( &date_len, &time_desc, &bintim, 0 );

    /* Remove the delimiters from the date string
    ---------------------------------------------*/
    date_len = strlen( date_str );
    for ( i = 0; i < date_len; i++ )
	if ( date_str[i] == '-' || date_str[i] == ':' || date_str[i] == '.' )
	    date_str[i] = ' ';

    /* Extract the hour and minute from the date string
    ---------------------------------------------------*/
    sscanf( date_str, "%*d %*s %*d %d %d", &hour, &min ); 

    /* Convert NBS seconds to a julian time
    ---------------------------------------*/
    make_julian( nbsec, &nbsyr, &nbsday, &nbshr, &nbsmin, &nbssec);

    /* If the local or NBS minute are within 5 minutes of an hour
    rollover and on opposite sides get another NBS second and do
    the loop again.  This will prevent computing the wrong offset
    when hours are spanned.  Note that the loop will not be
    repeated if an offset has been previously computed.
    -------------------------------------------------------------*/
    if ((( min <= 59 && min >= 55 ) && ( nbsmin >=  0 && nbsmin <=  5 )) ||
        (( min <=  5 && min >=  0 ) && ( nbsmin >= 55 && nbsmin <= 59 )))
        {
	if ( offset == 0 )
	    {
	    sleep( 10 );
	    nbsecond( &nbsec );
	    }
	continue;
	}

    /* Determine offset
    -------------------*/
    if ( nbshr < hour )		/* if NBS has rolled over midnight	*/
	nbshr += 24;		/* make it bigger by a day		*/
    offset = hour - nbshr;	/* determine the offset in hours	*/
    offset *= 3600;		/* convert hours to seconds		*/
    }
while ( offset == 0 );

return offset;
}

/*************************************************************************
This function is a replacement for the macro MAKE_JULIAN which had a bug.
It is the reverse of the makesecs() function of xplot which creates the
integer date that is being extracted here.
*************************************************************************/
make_julian(int start,int *year,int *day,int *hour,int *minute,int *second)
{
int yr;			/* local year count				*/
int seconds;		/* local copy of start time			*/
int cur_yrseconds;	/* number of seconds in the current year	*/
int nxt_yrseconds;	/* number of seconds in the next year		*/


/* Copy the start time and set the starting year
------------------------------------------------*/
seconds = start;
yr = START_OF_TIME;

/* Loop as long as there is still a years worth
of seconds left in seconds.  Break out of the
loop when there is less than a years worth.
-----------------------------------------------*/
while ( seconds > 0 )
    {
    /* Set the correct number of seconds for the current
    year and the following year.  If current year is a
    leap year the next year will not be.
    ----------------------------------------------------*/
    if ( yr % 4 == 0 )
        {
        cur_yrseconds = LEAP_YR_SECONDS;
        nxt_yrseconds = YR_SECONDS;
        }
    else
        {
        cur_yrseconds = YR_SECONDS;
	/* Set next years seconds, it may be a leap year
	------------------------------------------------*/
        if ( (yr + 1) % 4 == 0 )
            nxt_yrseconds = LEAP_YR_SECONDS;
        else
            nxt_yrseconds = YR_SECONDS;
        }

    /* If there are is a full years worth of seconds left,
    go to the next year and subtract the year from seconds
    ------------------------------------------------------*/
    if ( seconds >= cur_yrseconds )
        {
        yr++;
        seconds -= cur_yrseconds;
        }

    /* Check that there is another full year of seconds
    left in seconds, if not break out of the loop.
    ---------------------------------------------------*/
    if ( seconds < nxt_yrseconds )
        break;
    }

/* Return the year
------------------*/
*year = yr;

/* Return the julian day and get the left over seconds.
Days are counted from one so add one to the result.
-------------------------------------------------------*/
*day = seconds / DAY_SECONDS + 1;
seconds %= DAY_SECONDS;

/* Return the hour and get the left over seconds.
-------------------------------------------------*/
*hour = seconds / HOUR_SECONDS;
seconds %= HOUR_SECONDS;

/* Return the minute and get the left over seconds.
---------------------------------------------------*/
*minute = seconds / MIN_SECONDS;
seconds %= MIN_SECONDS;

/* Finally return the seconds.
------------------------------*/
*second = seconds;
}  /* END make_julian */


/*************************************************************************
FUNCTION:	calc_stdev

PURPOSE:	Calculate the mean and standard deviation of a set of
		values.  The first and last index of the array determine
		the set of values from the input array to use in the
		calculation.  An error is returned if last - first is
		less than one.

INPUT:		integer array 
		integer first: index of first element to use
		integer last: index of last element to use

OUTPUT:		float mean: the mean of the elements

RETURN:		float, standard deviation of elements
		float MEAN_ERROR, if only one element is specified
*************************************************************************/
float calc_stdev( int satmil[], int first, int last, float *mean )
{
float	variance;	/* variance of the elements			*/
float	sum;		/* sum of the elements				*/

int	i;		/* loop index					*/
int	num_elements;	/* number of elements in the set		*/

/* Determine the number of elements in the set
and return an error if num_elements <= 1
----------------------------------------------*/
num_elements = last - first + 1;
if ( num_elements <= 1 )
    return MEAN_ERROR;

/* Compute the mean of the element set
--------------------------------------*/
sum = 0.0;
for ( i = first; i <= last; i++ )
    sum += (float)satmil[i];
*mean = sum / (float)num_elements;

/* Compute the variance of the element set
-----------------------------------------*/
variance = 0.0;
for ( i = first; i <= last; i++ )
    variance = (satmil[i] - *mean) * (satmil[i] - *mean);
variance /= (float)num_elements - 1.0;

/* Return the standard deviation
--------------------------------*/
return sqrt( variance );
}  /* END calc_stdev */


/*************************************************************************
FUNCTION:       weighted_mean

PURPOSE:        Determine the mean of a sorted array but only using the
                most common values.  The reason for this is to eliminate
                the effects of noise or loss of data on the mean.  The
                weighted mean returned will be from the largest set of
                values with a standard deviation less than or equal to
                STDDEV_THRESH.  The final set of values will have at
                least MIN_MEAN_COUNT members.

INPUTS:         int satmil[SAMPLES] -- sorted in ascending order

OUTPUTS:        none

RETURNS:        weighted mean -- if one was found
                MEAN_ERROR    -- if mean couldn't be determined

ALGORITHM:
    Determine stddev of all values in the set
    Set number of values in each set to SAMPLES-1
    While the stddev is greater than STDDEV_THRESH and number of values in
      set is greater than or equal to MIN_MEAN_COUNT
        Determine the number of sets with numval values
        Loop through the numval sets: from first = 0 to numsets
            determine index of last value in set
            calculate stdev and mean of current set
            if the current stdev is less than stdev
                let stdev = current stdev
                let mean = current mean
            end-if
        end-loop
        subtract one from numval
    End while

    if the number of values is less than MIN_MEAN_COUNT
        let mean = MEAN_ERROR
    return mean

*************************************************************************/
int weighted_mean( int satmil[] )
{
float   current_mean;   /* mean of current set of values                */
float   current_stdev;  /* standard deviation of current set of values  */
float   mean;           /* mean of set of values with lowest stdev      */
float   stdev;          /* lowest standard deviation                    */

int     first;          /* index of first value in set                  */
int     numsets;        /* number of sets in satmil with numval values  */
int     numval;         /* number of values in the set                  */
int     last;           /* index of last value in set                   */


/* Get the standard deviation and mean of the entire data set
-------------------------------------------------------------*/
stdev = calc_stdev( satmil, 0, SAMPLES-1, &mean );

/* Calculate standard deviation and mean of succesively smaller sets
of values until the number of values in the data set are less than the
minimum (MIN_MEAN_COUNT) or a stndard deviation is calculated that is
less than or equal to the maximum (STDDEV_THRESH).
----------------------------------------------------------------------*/
numval = SAMPLES-1;
while ( numval >= MIN_MEAN_COUNT && stdev > STDDEV_THRESH )
    {
    /* Determine the number of data sets of numval size.
    ---------------------------------------------------*/
    numsets = SAMPLES - numval + 1;

    /* Loop through each data set calculating its stdev and mean
    -----------------------------------------------------------*/
    for ( first = 0; first < numsets; first++ )
        {
        last = first + numval - 1;
        current_stdev = calc_stdev( satmil, first, last, &current_mean );

        /* If the current stdev is less than any previous
        one, save it and the mean
        -------------------------------------------------*/
        if ( current_stdev < stdev && current_stdev != MEAN_ERROR )
            {
            stdev = current_stdev;
            mean = current_mean;
            }
        }  /* END for */

    /* Decrement the number of values in a data set
    -----------------------------------------------*/
    numval--;
    }  /* END while */

/* If we got out of the while loop with less than the
minimum number of values return the error value.
-----------------------------------------------------*/
if ( numval < MIN_MEAN_COUNT )
    return MEAN_ERROR;

return (int)( mean + 0.5 );
}  /* END weighted_mean */



/***********************************************************************
FUNCTION:	insertion_sort

PURPOSE:	Sort a small ( n < 50 ) integer array.  The input array
		is overwritten with sorted values.

INPUT:		integer array: array to be sorted
		integer n: number of elements in array

OUTPUT:		input array is overwritten with sorted values

RETURNS:	none
***********************************************************************/
void insertion_sort( int array[], int n )
{
int	i, j;	/* loop index variables		*/
int	temp;	/* temporary swapping variable	*/

for ( j = 1; j < n; j++ )
    {
    temp = array[j];
    i = j - 1;
    while ( i >= 0 && array[i] > temp )
        {
	array[i+1] = array[i];
	i--;
	}
    array[i+1] = temp;
    }
}  /* END insertion_sort */
