$!
$!  UPDATE  -   Checks for update files and updates the master schedule
$!  		update files are created by schedit, autogen, or xplot
$!		they are named:
$!	add.sch    - scenes to be added to the schedule
$!	delete.sch - scenes listed by date to be deleted
$!	delsid.sch - scenes listed by scene id to be deleted
$!	modify.sch - scenes to be modified based on date range
$!	modsid.sch - scenes to be modified based on scene id
$!	modsta.sch - scenes to be modified based on date range and station
$!	sort.sch   - indicates the schedule is to be sorted
$!
$!	each file is used with the proper option of ADAPSTEST:schedit
$!
$! 	The sched.log;2 file is assigned and deassigned as needed to the file
$!	isn't continually locked by this process.  New messages are written
$!	to ;2.  Then ;1 and ;2 are copied to ;3.  Then ;1 and ;2 are purged
$!	and ;3 is renamed as ;1.  This keeps the log file at sched.log;1 and
$!	allows adding sys$output to the logfile.  One thing to remember:
$!	files are copied together backwards, so the log file will have the
$!	most recent messages at the top.
$!
$!	This runs as an infinite loop, one parameter is required:
$! P1 - Directory where update files are found (either directory or logical)
$!
$! Set up error status
$ status = -1
$
$
$! Set up the search file specification
$ search_dir = f$trnlnm("ADAPSTABLES")
$ if (search_dir .eqs. "")
$ then
$   write sys$output "Do not know ADAPSTABLES"
$   exit status
$ endif
$ search_file = "''search_dir'" + "*.sch"
$
$
$! Set up the subdirectory for other *.sch files
$ len = f$length("''search_dir'")
$ hold_dir = f$extract(0,len-1,"''search_dir'") + ".hold]"
$
$
$! Set up the command for schedit
$ schedit :== $ADAPSTEST:schedit
$
$
$floop:
$    	update_file = f$search("''search_file'")
$	sub_name = f$parse("''update_file'",,,"name",)
$
$    	if (sub_name .eqs. "ADD")
$	then
$ 	    assign sched.log;2 sys$output
$	    show time
$	    schedit -af 'update_file'
$	    purge/keep=7 ADAPSTABLES:master.sch
$ 	    deassign sys$output
$	    copy sched.log;* sched.log;3
$	    purge sched.log
$	    rename sched.log;3 sched.log;1
$
$
$	else if (sub_name .eqs. "DELETE")
$	then
$ 	    assign sched.log;2 sys$output
$	    show time
$	    schedit -df 'update_file'
$	    purge/keep=7 ADAPSTABLES:master.sch
$ 	    deassign sys$output
$	    copy sched.log;* sched.log;3
$	    purge sched.log
$	    rename sched.log;3 sched.log;1
$
$
$	else if (sub_name .eqs. "DELSID")
$	then
$ 	    assign sched.log;2 sys$output
$	    show time
$	    schedit -ds 'update_file'
$	    purge/keep=7 ADAPSTABLES:master.sch
$ 	    deassign sys$output
$	    copy sched.log;* sched.log;3
$	    purge sched.log
$	    rename sched.log;3 sched.log;1
$
$
$	else if (sub_name .eqs. "MODIFY")
$	then
$ 	    assign sched.log;2 sys$output
$	    show time
$	    schedit -mf 'update_file'
$	    purge/keep=7 ADAPSTABLES:master.sch
$ 	    deassign sys$output
$	    copy sched.log;* sched.log;3
$	    purge sched.log
$	    rename sched.log;3 sched.log;1
$
$
$	else if (sub_name .eqs. "MODSID")
$	then
$ 	    assign sched.log;2 sys$output
$	    show time
$	    schedit -ms 'update_file'
$	    purge/keep=7 ADAPSTABLES:master.sch
$ 	    deassign sys$output
$	    copy sched.log;* sched.log;3
$	    purge sched.log
$	    rename sched.log;3 sched.log;1
$
$
$	else if (sub_name .eqs. "MODSTA")
$	then
$ 	    assign sched.log;2 sys$output
$	    show time
$	    schedit -ma 'update_file'
$	    purge/keep=7 ADAPSTABLES:master.sch
$ 	    deassign sys$output
$	    copy sched.log;* sched.log;3
$	    purge sched.log
$	    rename sched.log;3 sched.log;1
$
$
$	else if (sub_name .eqs. "SORT")
$	then
$	    sort/key=(position:21,size=9) adapstables:master.sch adapstables:master.sch
$	    delete 'update_file'
$	    purge/keep=7 adapstables:master.sch
$
$
$! 	no files found, wait 5 min
$	else if (update_file .eqs. "")
$	then
$	    wait 00:05:00.00
$
$
$! 	for all other file names other than master.sch, move it out of the way
$ 	else if (sub_name .nes. "MASTER")
$	then
$ 	    assign sched.log;2 sys$output
$	    show time
$	    write sys$output "moving ''update_file' to ''hold_dir'"
$	    rename 'update_file' 'hold_dir'
$ 	    deassign sys$output
$	    copy sched.log;* sched.log;3
$	    purge sched.log
$	    rename sched.log;3 sched.log;1
$
$	endif
$	endif
$	endif
$	endif
$	endif
$	endif
$	endif
$	endif
$	endif
$
$
$	goto floop
$
$fdone:
$ status = 1
$ exit status
