/*******************************************************************************
NAME:   	PROJSEL

PURPOSE:	Select lines from WVS for placement in the image which is 
		not in satellite projection (use DDR for projection information)
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "adaps.h"
#include "proj.h"
#include "wvschips.h"
 
FUNCTION projsel(vblock,ddr,hosin,bandno,totbnd,hosout,wvspath,
		 wvsfile,pixval,window,backgrnd,imgback,
		 msgtim,odtype,minlat,minlon,maxlat,maxlon)

struct PARBLK   *vblock;        /* TAE parameter block                  */
struct  DDR	*ddr;		/* DDR information			*/

char    *wvspath,               /* wvs location                         */ 
	*wvsfile,     		/* WVS file name               		*/
	hosin[][CMLEN],		/* name of input image			*/
	hosout[];		/* name of output image			*/

long    pixval,                 /* pixel value of rasterized line       */
        window[],               /* window to process                    */
	backgrnd,		/* background pixel value		*/
	imgback,		/* flag to put image data in backgrnd	*/
	bandno[][MAXBND+1],	/* band number to read			*/
	totbnd[],		/* total # bands in input image		*/
        *msgtim,                /* time between processing messages     */
	odtype;			/* output data type			*/

double  minlat,                 /* minimum latitude			*/
	minlon,                 /* minimum longitude                    */
	maxlat,                 /* maximum latitude                     */
	maxlon;                 /* maximum longitude                    */
{
extern long curent;		/* current "line" for processimg messages*/

float *chipbuf;			/* buffer of chip pixel values		*/
int 	fd;			/* image file descriptor		*/

struct WVS_VECTOR wvs_vector;	/* wvs vector (list of line segments)	*/

long	i, 		   	/* looping variables		     	*/
	inwind[1][4],           /* window for updating ddr		*/
	idtype = EREAL,		/* work with floats			*/
    	ddr_units,              /* projection units from image ddr      */
	geo_code,		/* projection code			*/
	geo_zone,		/* projection zone			*/
	geo_units,		/* projection units			*/
	max_entity = 9999, 	/* maximum entity for processing message*/
	msgfmt = GEN2,	   	/* general message format one	     	*/
	nonfatal = 1,	   	/* non-fatal error code		     	*/
	segment,		/* current segment in vector		*/
	status,   	   	/* return status			*/
	options[3],		/* image i/o options			*/
	zero = 0,	   	/* constant of zero		     	*/
	one = 1,		/* constant of one			*/
	access;			/* file access				*/

double 	geo_parms[15],		/* projection parameters		*/
	dblone = 1.0;

/*
  Set up projection information
*/
geo_code = GEO;
geo_zone = NULZONE;
geo_units = DEGREES;
for (i = 0; i < 15; i++)
    geo_parms[i] = 0.0;
c_ckunit(ddr->proj_units,&ddr_units);

/*
  Allocate space for chip image pixels (2-d array)
*/
chipbuf = (float *)malloc(window[NL] * window[NS] * sizeof(float));
if (chipbuf == NULL)
    {
    c_errmsg("Error occurred allocating dynamic memory","wvschips-alloc", 
	&nonfatal);
    return(E_FAIL);
    }

if (imgback)
    {
    /*
      Open the input image as float
    status = c_eopenr(&fd,hosin,&totbnd,&window[SL],&window[SS],
    */
    access = IREAD;
    options[0] = options[1] = options[2] = 0;
    status = c_eopenr(&fd,hosin[0],&totbnd[0],&window[SL],&window[SS],
                      &window[NL],&window[NS],&access,&idtype,
                      options,&window[NL]);
    if (status == E_SUCC)
	{
        /*
          Get the image data for background
        */
        if (c_eread(&fd,&bandno[0][0],&one,chipbuf,&window[NL]) == E_FAIL)
	    {
	    c_errmsg("Error occurred reading image background",
		"wvschips-imgread",&nonfatal);
    	    free(chipbuf);
	    return(E_FAIL);
	    }
        c_eclose(&fd);
	}
    }
else
    {
    /* 
      Initialize the buffer to the background value
    */
    for (i = 0; i < window[NL] * window[NS]; i++)
	chipbuf[i] = backgrnd;
    }

/*
  Allocate first line memory block
  (if this block is exceeded, it will be reallocated within the
  wvs support routines).
*/
wvs_vector.numlines = 0;
wvs_vector.maxline = LINEBLOCK;
wvs_vector.line = (struct WVS_LINE *)malloc(LINEBLOCK *
						sizeof(struct WVS_LINE));
if (wvs_vector.line == NULL)
    {
    c_errmsg("Error occurred allocating dynamic memory","wvschips-alloc", 
	&nonfatal);
    free(chipbuf);
    return(E_FAIL);
    }

/*
  Perform the wvs query, loading all lines that fit our area
*/
c_wvsquery(wvspath,wvsfile,&wvs_vector,minlat,minlon,maxlat,maxlon);

/*
  If no lines were found go to next area
*/
if (wvs_vector.numlines == 0)
    {
    c_errmsg("No vector data occurs in the output image","wvschips-empty",
	&nonfatal);
    c_wvsfree(&wvs_vector);
    free(chipbuf);
    return(E_FAIL);
    }

/*
  Initailize and start the processing message
*/
max_entity = wvs_vector.numlines;
curent = 1;
c_inittm("WVSCHIPS","Rasterized","line"," "," ",&max_entity,&zero,msgtim,
    &msgfmt);
c_startm();

/*
  Process all lines found
*/
for (segment = 0; segment < wvs_vector.numlines; segment++)
    {
    /*
      Process each segment
    */
    status = procproj(&wvs_vector.line[segment],ddr,window,pixval,
        chipbuf,&geo_code,&geo_zone,&geo_units,&ddr_units,
        geo_parms);
    if (status != E_SUCC)
        {
	c_wvsfree(&wvs_vector);
        free(chipbuf);
        return(E_FAIL);
        }

    /*
      Increment the segment counter
    */
    curent++;
    }

/*
  Stop the processing message
*/
curent--;
c_stoptm();

/*
  Free lines
*/
c_wvsfree(&wvs_vector);

/*
  Open the output chip as converting from idtype to odtype
*/
access = IWRITE;
options[0] = odtype;
options[1] = options[2] = 0;
if (c_eopenr(&fd,hosout,&one,&one,&one,&window[NL],&window[NS],&access,
   	&idtype,options,&window[NL]) != E_SUCC)
    {
    c_errmsg("Error occurred opening output chip image",
	"wvschips-open",&nonfatal);
    free(chipbuf);
    return(E_FAIL);
    }

/*
  Convert the data and write it out
*/
if (c_ewrite(&fd,&one,&one,chipbuf,&window[NL]) == E_FAIL)
    {
    c_errmsg("Error occurred writing chip image","wvschips-write",&nonfatal);
    free(chipbuf);
    return(E_FAIL);
    }
c_eclose(&fd);
free(chipbuf);

/*
  Update the DDR of the chip so it has all the correct
  projection info
*/
inwind[0][0] = window[0];
inwind[0][1] = window[1];
inwind[0][2] = window[2];
inwind[0][3] = window[3];
c_upddr(vblock,hosin,hosout,&one,inwind,bandno,totbnd,
		 &dblone,&dblone,&one,&one);

return(E_SUCC);
}
