/*******************************************************************************
NAME:   SATSEL

PURPOSE:        Select lines from WVS for placement in the image

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "adaps.h"
#include "wvschips.h"

FUNCTION satsel(vblock,ddr,model,elev,hosin,bandno,totbnd,hosout,wvspath,
		wvsfile,pixval,window,backgrnd,imgback,
		msgtim,odtype,minlat,minlon,maxlat,maxlon)

struct PARBLK	*vblock;	/* TAE parameter block			*/
struct  DDR     *ddr;           /* DDR information                           */
struct	MODEL	*model;	   	/* AVHRR model information		     */

char	*wvspath,		/* wvs location				     */
	*wvsfile,		/* WVS file name			     */
	hosin[][CMLEN],		/* name of input image			     */
	hosout[];		/* name of output image			     */

long	pixval,			/* pixel value of rasterized line	     */
	bandno[][MAXBND+1],	/* band number to read			     */
	totbnd[],		/* total # bands in input image		     */
	backgrnd,		/* background pixel value		     */
	imgback,		/* flag to put image data in backgrnd	     */
	window[],		/* window to process			     */
	*msgtim,		/* time between processing messages	     */
	odtype;			/* output data type			     */

double	minlat,			/* minimum latitude			     */
	minlon,			/* minimum longitude			     */
	maxlat,			/* maximum latitude			     */
	maxlon,			/* maximum longitude			     */
	*elev;			/* elevation of chip			     */
{
extern long curent;		/* current "line" for processimg messages    */

struct	WVS_VECTOR wvs_vector;	/* set of lines for the area		     */

int fd;				/* image file descriptor used for image I/O  */
float	*chipbuf;		/* buffer of pixels			     */

long    i, 		   	/* looping variable			     */
	idtype = EREAL,		/* want to deal with float on input	     */
	inwind[1][4],		/* window for updating ddr		     */
	msgfmt = GEN2,	   	/* general message format one		     */
	max_entity = 9999, 	/* maximum entity for processing message     */
	nonfatal = 1,	   	/* non-fatal error code			     */
	segment,	   	/* current segment			     */
	status,   	   	/* return status			     */
	options[3],		/* image i/o options			     */
	zero = 0,	   	/* constant of zero			     */
	one = 1,		/* constant of one			     */
	access;			/* file access				     */

double	lineinc,sampinc,	/* line and sample increment		     */
	dblone = 1.0;

/*
  Find out if this image was subsampled
  other than the normal
*/
if (model->dtype == GAC)
    {
    if (ddr->line_inc != 4.0)
        /* divide by the original line increment */
        lineinc = ddr->line_inc / 4.0;
    else
        lineinc = 1.0;
    if (ddr->sample_inc != 5.0)
        /* divide by the original sample increment */
        sampinc = ddr->sample_inc / 5.0;
    else
        sampinc = 1.0;
    }
else
    {
    lineinc = ddr->line_inc;
    sampinc = ddr->sample_inc;
    }

/*
  Allocate space for chip image pixels (2-d array)
*/
chipbuf = (float *)malloc(window[NL] * window[NS] * sizeof(float));
if (chipbuf == NULL)
    {
    c_errmsg("Error occurred allocating dynamic memory","wvschips-alloc",
	&nonfatal);
    return(E_FAIL);
    }

if (imgback)
    {
    /*
      Open the input image as real
    */
    access = IREAD;
    options[0] = options[1] = options[2] = 0;
    status = c_eopenr(&fd,hosin[0],&totbnd[0],&window[SL],&window[SS],
                      &window[NL],&window[NS],&access,&idtype,
                      options,&window[NL]);
    if (status == E_SUCC)
	{
        /*
          Get the image data for background
        */
        if (c_eread(&fd,&bandno[0][0],&one,chipbuf,&window[NL]) == E_FAIL)
	    {
	    c_errmsg("Error occurred reading image background",
		"wvschips-imgread",&nonfatal);
	    free(chipbuf);
	    return(E_FAIL);
	    }
        c_eclose(&fd);
	}
    }
else
    {
    /* 
      Initialize the buffer to the background value
    */
    for (i = 0; i < window[NL] * window[NS]; i++)
	chipbuf[i] = backgrnd;
    }

/*
  Allocate first line memory block
  (if this block is exceeded, it will be reallocated within the
  WVS support routine).
*/
wvs_vector.numlines = 0;
wvs_vector.maxline = LINEBLOCK;
wvs_vector.line = 
	(struct WVS_LINE *)malloc(LINEBLOCK * sizeof(struct WVS_LINE));
if (wvs_vector.line == NULL)
    {
    c_errmsg("Error occurred allocating dynamic memory","wvschips-alloc",
	&nonfatal);
    free(chipbuf);
    return(E_FAIL);
    }

/*
  Perform the wvs query, loading all lines that fit our area
*/
c_wvsquery(wvspath,wvsfile,&wvs_vector,minlat,minlon,maxlat,maxlon);

/*
  If no lines were found return
*/
if (wvs_vector.numlines == 0)
    {
    c_errmsg("No vector data occurs in the output image","wvschips-empty",
	    &nonfatal);
    c_wvsfree(&wvs_vector);
    free(chipbuf);
    return(E_FAIL);
    }

/*
  Initialize and start the processing message
*/
curent = 1;
max_entity = wvs_vector.numlines;
c_inittm("WVSCHIPS","Rasterized","line"," "," ",&max_entity,&zero,
    msgtim,&msgfmt);
c_startm();

/*
  If lines were found, generate corresponding line/samples (rasterize)
*/
for (segment = 0; segment < wvs_vector.numlines; segment++)
    {
    /*
      Process each segment
    */
    status = procsat(&wvs_vector.line[segment],ddr,model,elev,window,
    	lineinc,sampinc,pixval,chipbuf);
    if (status != E_SUCC)
	{
    	c_wvsfree(&wvs_vector);
	free(chipbuf);
	return(E_FAIL);
	}

    /*
      Increment the segment counter
    */
    curent++;
    }

/*
  Stop the processing message
*/
curent--;
c_stoptm();

/*
  Free lines
*/
c_wvsfree(&wvs_vector);

/*
  Open the output chip as odtype
*/
access = IWRITE;
options[0] = odtype;
options[1] = options[2] = 0;
if (c_eopenr(&fd,hosout,&one,&one,&one,&window[NL],&window[NS],&access,
	&idtype,options,&window[NL]) != E_SUCC)
    {
    c_errmsg("Error occurred opening output chip image",
	"wvschips-open",&nonfatal);
    free(chipbuf);
    return(E_FAIL);
    }

/*
  Convert the data and write it out
*/
if (c_ewrite(&fd,&one,&one,chipbuf,&window[NL]) == E_FAIL)
    {
    c_errmsg("Error occurred writing chip image","wvschips-write",&nonfatal);
    free(chipbuf);
    return(E_FAIL);
    }
c_eclose(&fd);
free(chipbuf);

/*
  Update the chip image's ddr     	
*/
inwind[0][0] = window[0];
inwind[0][1] = window[1];
inwind[0][2] = window[2];
inwind[0][3] = window[3];
c_upddr(vblock,hosin,hosout,&one,inwind,bandno,totbnd,
		 &dblone,&dblone,&one,&one);

return(E_SUCC);
}
