/*
	Program History
		Ken Gacke	6/29/92		Added the -f option to allow
						user to specify the file seq
						number for the header label.
						Required for the -A option for
						the 1KM project.
		Ken Gacke	6/23/92		Added the -A and -R option to
						append data to a tape without
						rewinding it first.  Used for
						1KM project.
		Ken Gacke	???		Modified to use tape i/o library
		KJ,CG		???		Original Development and mods
		?????		08/31/92	Convert to VMS.  Note that EOT
						logic has been totally removed
						(no longer tries to back space
						files and write EOF marks).  If
						an EOT mark is encountered, a fatal
						error is announced and execution
						terminates.  This change is motivated
						by the varied behavior of tape devices
						when an EOT is detected.

*/
#include "worgen.h"
#include "tapeio.h"
#define	GLOBALS
#include	"ansi.h"
#undef	GLOBALS


#define	USAGE1	"usage: ANSIOUT -[TvbfdrtDFaAlmnsR] [drive] [volid] [blksz]"
#define USAGE2	"        [filenum] [density] [recsz] [length] file [file] ...\n"

#define MAXFIL  500	/* maximum number of files to get       */

main (argc,argv)
int	argc;
char	**argv;
{
int	Aflag = ANSI_FALSE;	/* append data from current tape position */
int	aflag = ANSI_FALSE;	/* append to end of tape		*/
int	args;			/* arguments used in parsing		*/
int	blkcnt;			/* number of blocks			*/
int	blksz = 8000;		/* block size for current file(s)	*/
int	disk;			/* disk file descriptor			*/
int	files = 0;		/* number of files output		*/
int	file_sect_no = 1;	/* file section number			*/
int	file_seq_no = 1;	/* file sequence number			*/
int	newtape = ANSI_TRUE;	/* new tape?				*/
int	reccnt;			/* number of records			*/
int	rewflg = 1;		/* rewind tape at completion */
int	rewind = 1;		/* rewind flag				*/
char	rec_format = 'D';	/* record format for current file(s)	*/
int	recsz = 100;		/* record size for current file(s)	*/
int	status = OK;		/* program exit status			*/
int	tape;			/* file desc (struct TAPEDESC)		*/
long	subfiles = 0;		/* preliminary number of files to process*/
char	*volume_ident;		/* desired volume identifier		*/
static	char file_ary[MAXFIL][CMLEN];/* array of file names		*/
char	dirspec[CMLEN];		/* directory part of file name		*/
char	rootspec[CMLEN];	/* root part of the file name		*/
char	extspec[CMLEN];		/* extension part of the file name	*/
long	startf = 1;		/* starting file number to dirlist()	*/
long	ii;			/* counter				*/
char	tmpfil[20];		/* temporary file name 			*/
FILE 	*fp;                    /* file pointer for temp file sent to fileutil*/
char    *malloc(), *getusername(), *strcat(), *strcpy();
char buffer[CMLEN];


#ifdef unix
#ifdef sun
   /* I have no idea if this is correct, but it builds */
   setvbuf(stdout, buffer , _IOLBF,  CMLEN);
#else
   setlinebuf(stdout);
#endif
#endif

#define	HANDLE_EOT							  \
  {									  \
  if (ndcdb)								  \
    {									  \
    status = EOT;							  \
    goto EOT_EXIT;							  \
    }									  \
  else									  \
    {									  \
    fprintf (stderr,"\n");						  \
    fprintf (stderr,"ANSIOUT: EOT encountered writing file %s\n",file_ary[files]);  \
    fprintf (stderr,"ANSIOUT: file will be restarted on next tape\n");	  \
    newpg = 1;						 		  \
    reelswitch ("ANSIOUT",&tape,0,EOFF);				  \
    file_seq_no = 1;							  \
    close (disk);							  \
    status = OK;							  \
    goto START_COPY;							  \
    }									  \
  }
density = "";
drive_name = NULL;
ndcdb = 0; splitfile = 0;
irgsize = 0.8;
tfiles = 0;
tlen = 0.0; twrit = 0.0;
volume_ident = malloc(8);
*volume_ident = 0;
checkvol = 1;
labelled = ANSI_TRUE;
lflag = 0; newpg = 1;

for (argc--, argv++; argc && **argv == '-'; argc -= args, argv += args)
    {
    args = 1;
    for ((*argv)++; **argv; (*argv)++)
	switch (**argv)
	    {
	    case 'T':
		    drive_name = *(argv + args++);
		    break;
	    case 'R':
		    rewflg = 0;
		    break;
	    case 'A':
		    Aflag = ANSI_TRUE;
		    newtape = ANSI_FALSE;
		    rewind = 0;
		    break;
	    case 'a':
		    aflag = ANSI_TRUE;
		    newtape = ANSI_FALSE;
		    rewind = 0;
		    break;
	    case 'b':
		    blksz = atoi (*(argv + args++));
		    break;
	    case 'd':
		    density = *(argv + args++);
		    break;
	    case 'f':
    		    file_seq_no = atoi(*(argv + args++));
		    break;
	    case 'l':
		    lflag = 2;
		    break;
	    case 'm':
		    lflag = 1;
		    break;
	    case 'n':
		    ndcdb = 1;
		    break;
	    case 'r':
		    recsz = atoi (*(argv + args++));
		    break;
	    case 's':
		    splitfile = 1;
		    break;
	    case 't':
		    tlen = atoi (*(argv + args++));
		    /* assume we can use 95% of tape */
		    tlen *= (12.0 * .95);
		    break;
	    case 'v':
		    volume_ident = *(argv + args++);
		    strupr (volume_ident,volume_ident);
		    break;
	    case 'D':					/* variable	*/
	    case 'F':					/* fixed	*/
		    rec_format = **argv;
		    break;
	    default:
		    fprintf (stderr,USAGE1);
		    fprintf (stderr,USAGE2);
		    exit (NOT_OK);
	    }
    }
if (ndcdb)
    lflag = 0;
if (rec_format == 'F')
    {
    if (blksz % recsz != 0)
	{
	fprintf(stderr,"ANSIOUT: Block size must be multiple of record size\n");
	exit (NOT_OK);
	}
    }
if (Aflag)	/* mount tape without rewind, and do not check ANSI label */
    {
    a_mountvol ("ANSIOUT",
	volume_ident,&tape,"enter tape id for first volume: ",rewind);
    bsf_tape (tape,1);  /* backspace 1 tape mark */
    }
else
    {
    mountvol ("ANSIOUT",volume_ident,&tape,"enter tape id for first volume: ",
	  rewind,0);
    }
if (aflag)
    {
    file_seq_no = directory (tape,ANSI_FALSE,ANSI_FALSE);
    if (file_seq_no != 0)
	{
	file_seq_no =
	    antoi(hdr1.info.file_seq_no,sizeof(hdr1.info.file_seq_no))+1;
	bsf_tape (tape,0);
	}
    else
	{
	newtape = ANSI_TRUE;
	rewind_tape (tape,ANSI_TRUE);
	file_seq_no = 1;
	}
    }

init_labels (volume_ident,getusername());

if (newtape)
    {
    put_volume (tape);
    print_volume ();
    }

/* open a file solely for the purpose of calling dirlist()
-------------------------------------------------------------*/
strcpy(tmpfil,"ansiotmp");
fp = fopen(tmpfil,"w");
if (fp == NULL)
   {
   fprintf (stderr,"ANSIOUT: Error opening temporary print file");
   exit(NOT_OK);
   }

nfiles = 0;
while (argc)
   {
   /* expand the wild cards if there are any */
   status = dirlist(tmpfil, *argv, &file_ary[nfiles][0], &subfiles, MAXFIL, startf);
   if (status == NOT_OK)
       {
       fclose(fp);
       exit(status);
       }
   nfiles += (int)subfiles;
   argv++;
   argc--;
   }

/* strip the directory specification off of the file name */
for (ii=0; ii < nfiles; ii++)
   {
   c_splitname(file_ary[ii],dirspec,rootspec,extspec);
   strcpy(file_ary[ii],rootspec);
   if (strcmp(extspec,"") != 0)
      {
      strcat(file_ary[ii],".");
      strcat(file_ary[ii],extspec);
      }
   }

/***  Main processing loop  ***/

for (  ; nfiles;  nfiles--, files++, file_seq_no++, tfiles++)
    {
    START_COPY:
    disk = open_disk (file_ary[files],O_RDONLY,0);
    file_sect_no = 1;

    load_labels (file_ary[files],file_sect_no++,file_seq_no,rec_format,recsz,blksz);
    status = put_headers (&tape);
    if (status == EOT)
	HANDLE_EOT;
    write_eof (tape);

    if (rec_format == 'D')				/* variable */
	status = write_D_tape ("ANSIOUT",&tape,disk,&reccnt,&blkcnt,
				file_seq_no,recsz,blksz);
    else /* rec_format == 'F' */			/* fixed    */
	status = write_F_tape ("ANSIOUT",&tape,disk,&reccnt,&blkcnt,
				file_seq_no,recsz,blksz);
    if (status == EOT)
	HANDLE_EOT;
    write_eof (tape);

    status = put_eofs (tape,blkcnt);
    if (status == EOT)
	HANDLE_EOT;
/*
** should not write eof here, the tape_close will write 2 eof's TAR 7/7/97
**  write_eof (tape);
**********************/

    close (disk);
    print_file (reccnt,blkcnt);
    check_eof (blkcnt);
    }

EOT_EXIT:
if (!ndcdb)
    {
    if (lflag != 0)
	printf  ("\n%d files output\n",tfiles);
    printf ("\n%d total files output\n",files);
    }
fprintf (stderr,"\nANSIOUT: drive %s: %d files output\n",drive_name,tfiles);
fprintf (stderr,"ANSIOUT: %d total files written\n",files);
#ifdef vms
write_eof (tape);
#endif
close_tape (tape,NOREW);
if (rewflg)
    rewind_tape (tape,ANSI_FALSE);
fclose(fp);
exit (status);
}
