/******************************************************************************
FUNCTION:   DIRLIST

PURPOSE:    Dirlist handles wildcarded file names and returns a list of the
	    file specified by the filename.

PROGRAM HISTORY:
Version  Author        Date   Change Request
-------  ------------  -----  --------------
  ???    T.Butzer      03/86  Original Implementation
  ???    B.Ailts       12/87  Changed the include file specifications
  ???    K.Gacke       04/88
  5.0    D.Hollaren    04/88  changed include spec to las changed cerrmsg to
			      c_errmsg and pass status code by address return
			      status of E_SUCC/E_FAIL
  5.0    B.Davis       09/88  conversion to 5.0, added an argument to allow 
			      temp files to remain hidden, fixed bugs
  5.0    B.Davis       01/89  Fixed unix bug of returning nfiles of 0 and 
			      status of E_FAIL when file(s) not found.  Now
			      returns nfiles of 0 and status of E_SUCC.
  5.0    B.Davis &
	 D.Hollaren    01/89  Upcased vms exentsion before comparision.
  5.0    J.McGuthry    02/89  added a condition such that if compiled on IBM
			      AIX getcwd is executed else getwd is executed
			      ( get working directory )
  5.1    D.Etrheim     01/91  Standardized error message handling
  5.1    J.Fenno       09/91  Fixed index problem when reading the files names
			      in the UNIX version.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
Under VMS;
    Open a tempory file,
    get the file names of the specified directory one at a time,
	remove the version number
	put files of the directory into the temporary file.
    After all file names are in the temporary file
	read the temp file placing nulls at the end of each string
	place the file names into the return array of file names.

Under UNIX;
    Open a tempory file,
	get the file names of the specified directory
	put files of the directory into the temporary file.
    After all file names are in the temporary file
	read the temp file placing nulls at the end of each string
	place the file names into the return array of file names.

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>
#include "worgen.h"

/* The dirlist routine builds a list of file names from a "wildcard" file
   specification.  The genf routine (see rmsio.c) is called to actually
   generate the filenames.(VMS)
-------------------------------------------------------------------------*/
#ifdef vms
#include "desc.h"

#define FILELIST "SYS$SCRATCH:filelist_"

int dirlist(nameoffile,filespec, file_ary, nfiles, maxfiles, startf)

   long *nfiles;           /* number of files generated in list   */
   long maxfiles;          /* max number to list at a time        */
   long startf;            /* flag for first call                 */

   char *filespec;         /* input spec                          */
   char file_ary[][CMLEN]; /* array of file names to be output    */
   char *nameoffile;       /* pointer to temp file of file names  */
{
   struct DESC wild;       /* wildcard file spec passed to genf   */
   struct DESC file;       /* filename returned by genf           */

   long count;             /* counter for nfiles                  */
   long flag;              /* flag for genf, file found or not    */
   long i;                 /* char array index                    */
   long name_size;         /* size of filename returned by genf   */
   long status;            /* status variable                     */

   char filerslt[CMLEN];   /* resultant file name from genf       */
   char *ptr;              /* temporary pointer                   */
   char temp[CMLEN];       /* temp name storage                   */
   char start;             /* parameter to genf                   */
   char tmpnam[CMLEN];     /*temp fil nam for filutil to be chked */
   char wildstr[CMLEN];    /* holds wildcard file specification   */

   char *strcpy(), *strchr();    /* string functions		  */

   static FILE *tmpfp;     /* file pointer to file of file names  */
   FILE *tmpfile();        /* file pointer to file of file names  */
/******************************************************************************/

*nfiles = 0;
if (startf == 1)
   {
   tmpfp = tmpfile();
   if (tmpfp == NULL)
      {
      fprintf (stderr,"ansiout: Error opening temporary file");
      return(E_FAIL);
      }

   /* get the full name of the temporary file that will be passed to fileutil,
      this needs to be checked because it will not be around after processing,
      but will show up in a directory listing during processing.
   ---------------------------------------------------------------------------*/
   if (strcmp(nameoffile," ") != 0)
      {
      wild.addr = nameoffile;
      wild.length = strlen(nameoffile);
      file.addr = tmpnam;
      file.length = CMLEN;
      start = 1;	
      status = genf(&wild,&file,&name_size,&start,&flag);
      if (status != E_SUCC)
	 {
	 fprintf (stderr,"ansiout: Error getting file names");
	 return(E_FAIL);
	 }
      ptr = (char *)strchr(tmpnam,';');
      *ptr = '\0';
      }
   else
      strcpy(tmpnam,nameoffile);

   /* Concatenate the directory and file specification
      and build a descriptor pointing to it.
   ---------------------------------------------------*/
   wild.addr = filespec;
   wild.length = strlen(wild.addr);

   /* Build a descriptor pointing to the first entry in the
      filelist array to receive the first file name.
   --------------------------------------------------------*/
   file.addr = filerslt;
   file.length = CMLEN;

   /* Call genf (see worrmsio.c) to get filenames,
      one at a time, from the wildcard string.
   -----------------------------------------------*/
   start = 1;	
   flag = 0;	
   *nfiles = 0;
   count = 0;
   status = genf(&wild,&file,&name_size,&start,&flag);
   if (status != E_SUCC)
      {
      fprintf (stderr,"ansiout: Error getting file names");
      return(E_FAIL);
      }
   while (flag == 1)
      {
      start = 0;
      filerslt[name_size] = '\0';
      /* lose the version number */
      strcpy(temp,filerslt);
      ptr = (char *)strchr(temp,';');
      *ptr = '\0';
      if (strcmp(tmpnam,temp) != 0)
	 {
	 fprintf(tmpfp, "%s\n",temp);
	 (*nfiles)++;
	 count++;
	 }
      status = genf(&wild,&file,&name_size,&start,&flag);
      if (status != E_SUCC)
	 {
	 fprintf (stderr,"ansiout: Error getting file names");
	 return(E_FAIL);
	 }
      }
   fseek(tmpfp,0,0);
   }

*nfiles = 0;
count = 0;
for (count = 0; count < maxfiles; count++)
   {
   status = fgets(temp,CMLEN,tmpfp);
   if (temp[0] == '\0')
      break;
   i = strlen(temp);
   temp[i-1] = '\0';
   c_up2low(temp,&i);
   strcpy(file_ary[*nfiles], temp);
   (*nfiles)++;
   }
if (*nfiles < maxfiles) 
   status = fclose(tmpfp);
return (E_SUCC);
}
#endif
/******************************************************************************/
#ifdef unix
#include <sys/types.h>
#include <sys/stat.h>
#define FILELIST "/tmp/filelist"

int dirlist (nameoffile,filespec, file_ary, nfiles, maxfiles, startf)

   long *nfiles;            /* number of files generated in list  */
   long maxfiles;           /* max number to list at a time       */
   long startf;             /* flag for first call                */
   char *filespec;          /* input spec                         */
   char file_ary[][CMLEN];  /* array of file names to be output   */
   char *nameoffile;        /* pointer to temp file of file names */
{
long count;                 /* counter for nfiles                 */
long exists;                /* flag for file existence            */
long i;                     /* char array index                   */
long n_files;               /* file_array index                   */
long rstat;                 /* return status                      */

#ifdef _IBMR2
   short c;   /* char return of fget */
#else
#ifdef sgi
   short c;   /* char return of fget */
#else
#ifdef DGUX
   short c;   /* char return of fget */
#else
   char c;    /* char return of fget */
#endif
#endif
#endif

char *ptr;           /* temporary pointer                    */
char temp[CMLEN];    /* temp name storage                    */
char sysstr[100];    /* conatins echo filespec for popen     */
char tmpnam[CMLEN];  /* temp fil nam for filutil to be chked */
char wd[CMLEN];      /* current working directory            */

char *strcpy(), *strcat(), *strchr();  /* string functions   */

static FILE *tmpfp;  /* file pointer to file of file names   */
/******************************************************************************/

if (startf == 1)
   {	
   sprintf(sysstr, "/bin/echo %s",filespec);
   tmpfp = popen(sysstr, "r");
   if (tmpfp == NULL)
      {
      fprintf (stderr,"ansiout: Error opening temporary file");
      rstat = E_FAIL;
      goto theend;
      }
   }

i = 0;

if (startf > 1)
   {
   count = 1;
   c = fgetc(tmpfp);
   while ((c != EOF) && (count < startf))
      {
      if(c == ' ')
	 {
	 n_files++;
	 count++;
	 }
      if (count < startf)
      c = fgetc(tmpfp);
      }
   }

#ifdef aiws
   getcwd(wd);
#else
#ifdef sun
   getcwd(wd,CMLEN);
#else
   getwd(wd);
#endif
#endif

/* get the full name of the temporary file that will be passed to fileutil,
   this needs to be checked because it will not be around after processing,
   but will show up in a directory listing during processing
---------------------------------------------------------------------------*/
strcpy(tmpnam,wd);
strcat(tmpnam,"/");
strcat(tmpnam,nameoffile);
n_files = 0;
count = 0;

while ((c=fgetc(tmpfp)) != EOF && (count < maxfiles))
   {
   if(c == ' ' || c == '\n')
      {
      c = '\0';
      file_ary[n_files][i] = c;
      if (strcmp(tmpnam,file_ary[n_files]) != 0)
	 {
	 ptr = (char *)strchr(file_ary[n_files], '/');
	 if (ptr == 0)
	    {
	    sprintf(temp,"%s/%s",wd, file_ary[n_files]);
	    strcpy(file_ary[n_files],temp);
	    }
	 n_files++;
	 count++;
	 }
      i = 0;
      }
   else
      {
      if (c == '*')
	 {	
	 *nfiles = 0;
	 rstat = E_SUCC;
	 goto theend;
	 }
      file_ary[n_files][i] = c;
      i++;
      }
   } /* while c != EOF */

i--;
file_ary[n_files][i] = '\0';
*nfiles = n_files;

if (*nfiles == 1)
   {
   exists = 0;
   if (c_lsstat(file_ary[0], &exists) != E_SUCC)
      {
      *nfiles = 0;
      rstat = E_SUCC;
      goto theend;
      }
   }
rstat = E_SUCC;

theend:
if (*nfiles < maxfiles) 
   {
   pclose(tmpfp);
   }
return (rstat);
}

#endif
