#include "worgen.h"
#include "tapeio.h"
#include "ansicp.h"
#include "string.h"

static unsigned char buf2[BUFSZ];
static unsigned char *bptr;
static unsigned long buf_len = 0;

extern long outcnt;
extern long outtotal;
extern long outbytes;
extern long outfcnt;

/******************************************************************************

NAME: reformat_block

PURPOSE:
  To reformat a block of data read in from an ANSI labeled tape to a new block
  size.  The data can be either from a block with fixed or variable length
  record sizes.  It will also reformat spanned blocks.


  This routine is called once for every block read, and once after EOF has been
  reached for a file.  It assumes that the new block size is big enough to 
  handle at least one record from the old block.

  It makes use of a static global data buffer, a static data pointer and a
  static counter.

******************************************************************************/


reformat_block(out,buf,blk_size,rec_size,bytes_read,rec_format,blkcount,
	       reccount, mapper)
long out;                             /* tape descriptor for the output tape */
unsigned char buf[];                  /* input data buffer                   */
long blk_size;                        /* output block size                   */
long rec_size;                        /* record size or max record size      */
long bytes_read;                      /* the number of bytes read            */
char rec_format;                      /* F=fixed, D=variable, S=spanned      */
long *blkcount;
long *reccount;
long mapper;
{
   long recsz;                          /* size of the current record        */
   unsigned char *ptr;                  /* record pointer in the input block */
   char recszbuf[10];                   /* record size string                */
   long bytes_written;
   
   
   /* EOF, EOV, or EOT has been reached and there is still data in the 
      output buffer , write it out and increment the block counter 
      ----------------------------------------------------------------- */
   
   if(bytes_read < 0 && buf_len > 0)
   {
      bytes_written = write_tape(out,buf2,buf_len);
      if(mapper)
	domap (outmapfile, bytes_written,&outbytes,&outfcnt,&outcnt,
	       &outtotal);
      buf_len = 0;
      (*blkcount)++;
      return;
   }
       
   bptr = &buf2[buf_len];

   switch (rec_format)
   {
    case 'D':
      for (ptr=&buf[0]; ptr != &buf[bytes_read]; ptr+=recsz)
      {
	 if (strncmp((char *)ptr,"^^^^",4))	/* if not pad chars,	*/
	 {				        /* write record		*/
	    strncpy (recszbuf,(char *)ptr,4);
	    recsz = atoi (recszbuf);
	    (*reccount)++;

	    if(buf_len + recsz > blk_size)
	    {
	       bytes_written = write_tape(out,buf2,buf_len);
	       if(mapper)
		 domap (outmapfile, bytes_written,&outbytes,&outfcnt,&outcnt,
			&outtotal);
	       buf_len = 0;
	       bptr = buf2;
	       (*blkcount)++;
	    
	    }
	    memcopy(bptr,ptr,recsz);
	    bptr += recsz;
	    buf_len += recsz;
	 }
	 else		/* pad chars found; go on to next block	*/
	   break;
      }
      break;
    case 'F':
      for (ptr = &buf[0];ptr != &buf[bytes_read];ptr += rec_size )
      {
	 if(buf_len + rec_size > blk_size)
	 {
	    bytes_written = write_tape(out,buf2,buf_len);
	    if(mapper)
	      domap (outmapfile, bytes_written,&outbytes,&outfcnt,&outcnt,
		     &outtotal);
	    buf_len = 0;
	    bptr = buf2;
	    (*blkcount)++;
	 }
	 memcopy(bptr,ptr,rec_size);
	 bptr += rec_size;
	 buf_len += rec_size;
	 (*reccount)++;
      }
      break;
      
    case 'S':
      for (ptr = &buf[0]; ptr != &buf[bytes_read]; ptr += recsz )
      {
	 if (strncmp((char *)ptr,"^^^^^",5))		/* if not pad chars,	*/
	 {				/* write record		*/
	    strncpy (recszbuf,(char *)ptr,1);
	    strncpy (recszbuf,(char *)ptr+1,4);
	    recsz = atoi (recszbuf);
	    (*reccount)++;

	    if(buf_len + recsz > blk_size)
	    {
	       bytes_written = write_tape(out,buf2,buf_len);
	       if(mapper)
		 domap (outmapfile, bytes_written,&outbytes,&outfcnt,&outcnt,
			&outtotal);
	       buf_len = 0;
	       bptr = buf2;
	       (*blkcount)++;
	    
	    }
	    memcopy(bptr,ptr,recsz);
	    bptr += recsz;
	    buf_len += recsz;
	 }
	 else		/* pad chars found; go on to next block	*/
	   break;
      }
      break;
      
    default:
      break;
   }
}

scan_block(buf,blk_size,rec_size,bytes_read,rec_format,blkcount,reccount)
unsigned char buf[];                  /* input data buffer                   */
long blk_size;                        /* output block size                   */
long rec_size;                        /* record size or max record size      */
long bytes_read;                      /* the number of bytes read            */
char rec_format;                      /* F=fixed, D=variable, S=spanned      */
long *blkcount;
long *reccount;
{
   long recsz;                          /* size of the current record        */
   unsigned char *ptr;                  /* record pointer in the input block */
   char recszbuf[10];                   /* record size string                */
   long bytes_written;
   
   
   /* EOF, EOV, or EOT has been reached and there is still data in the 
      output buffer , write it out and increment the block counter 
      ----------------------------------------------------------------- */
   
   if(bytes_read < 0 && buf_len > 0)
   {
      buf_len = 0;
      return;
   }
       
   bptr = &buf2[buf_len];

   switch (rec_format)
   {
    case 'D':
      for (ptr=&buf[0]; ptr != &buf[bytes_read]; ptr+=recsz)
      {
	 if (strncmp((char *)ptr,"^^^^",4))	/* if not pad chars,	*/
	 {				        /* write record		*/
	    (*reccount)++;
	    
	    strncpy (recszbuf,(char *)ptr,4);
	    recsz = atoi (recszbuf);

	    if(buf_len + recsz > blk_size)
	    {
	       buf_len = 0;
	       bptr = buf2;
	    
	    }
	    bptr += recsz;
	    buf_len += recsz;
	 }
	 else		/* pad chars found; go on to next block	*/
	   break;
      }
      break;
    case 'F':
      for (ptr = &buf[0];ptr != &buf[bytes_read];ptr += rec_size )
      {
	 (*reccount)++;
	 
	 if(buf_len + rec_size > blk_size)
	 {
	    buf_len = 0;
	    bptr = buf2;
	 }
	 bptr += rec_size;
	 buf_len += rec_size;
      }
      break;
      
    case 'S':
      for (ptr = &buf[0]; ptr != &buf[bytes_read]; ptr += recsz )
      {
	 if (strncmp((char *)ptr,"^^^^^",5))	/* if not pad chars,	*/
	 {				/* write record		*/
	    strncpy (recszbuf,(char *)ptr,1);
	    strncpy (recszbuf,(char *)ptr+1,4);
	    recsz = atoi (recszbuf);
	    (*reccount)++;

	    if(buf_len + recsz > blk_size)
	    {
	       buf_len = 0;
	       bptr = buf2;
	    
	    }
	    bptr += recsz;
	    buf_len += recsz;
	 }
	 else		/* pad chars found; go on to next block	*/
	   break;
      }
      break;
      
    default:
      break;
   }
}
