/* these lines from stdedc.h needed for tapeio.h only	*/
typedef long LONG;		/* 32 bit integer			*/
typedef char TEXT;		/* 8 bit for characters			*/
typedef int BOOL;		/* boolean variable			*/
typedef int INT;		/* used for lib routines returning ints	*/


#include	<stdio.h>
#ifdef vms
#include	<file.h>
#else
#include	<sys/file.h>
#endif
#include	"worgen.h"
#include	"tapeio.h"
#ifdef unix
#include	<sys/types.h>
#include	<sys/ioctl.h>
#include	<sys/mtio.h>
#endif
#ifdef sun
#include        <fcntl.h>
#endif


extern float	irgsize;		/* inter record gap		*/
extern float	twrit;			/* tape length written (inches)	*/
extern int	tape_bpi;		/* tape bpi			*/
extern char	*density;		/* tape density 		*/
extern char	*drive_name;		/* tape drive			*/
extern char	comment[];		/* user request, tape drive	*/

static long mode=HANDLE;

FUNCTION int open_the_tape (rewind,fmode)
int	rewind;				/* rewind tape			*/
int	fmode;				/* file access mode		*/
{
struct TAPEDESC *tp;
long		inout;
long		opt1 = rewind;
long		skip = 0;
long		unit = 0;
int		status;
char		tapeid[CMLEN];
char		*strcpy();

switch (fmode)
    {
    case O_RDONLY:
		inout = TREAD;
		break;
    case O_WRONLY:
		inout = TWRITE;
		break;
    case O_RDWR:
		inout = TUPDATE;
		break;
    default:
		fprintf(stderr,"(open_the_tape) Mode %d is not allowed",fmode);
		exit(-1);
    }

strcpy(comment,"please mount ANSI tape");
status = c_tpopen(&unit,&skip,&mode,&opt1,density,&inout,comment,tapeid);
if (status != TAPE_SUCC)
    exit(-1);
drive_name = comment;
tp = (struct TAPEDESC *) unit;
find_density(tp);
return (unit);
}

FUNCTION int reopen_tape (rewind,fmode)
int	rewind;				/* rewind tape			*/
int	fmode;				/* file access mode		*/
{
struct TAPEDESC *tp;
long		inout;
long		opt1 = rewind;
long		skip = 0;
long		unit = 0;
int		status;

switch (fmode)
    {
    case O_RDONLY:
		inout = TREAD;
		break;
    case O_WRONLY:
		inout = TWRITE;
		break;
    case O_RDWR:
		inout = TUPDATE;
		break;
    default:
		fprintf(stderr,"(reopen_tape) Mode %d is not allowed",fmode);
		exit(-1);
    }

status = c_tpreop(&unit,&skip,&mode,&opt1,density,&inout,drive_name);
if (status != TAPE_SUCC)
    exit(-1);
tp = (struct TAPEDESC *) unit;
find_density(tp);
return (unit);
}

FUNCTION int find_density(tp)

struct TAPEDESC *tp;

{
if (strcmp(tp->dtype,"9-track") == 0)
    {
    if ((strcmp(tp->density,"800")==0)||(strcmp(tp->density,"LO")==0))
	{
        tape_bpi = 800;
        irgsize = .8;
	}
    else if ((strcmp(tp->density,"1600")==0)||(strcmp(tp->density,"MED")==0))
	{
        tape_bpi = 1600;
        irgsize = .8;
	}
    else if ((strcmp(tp->density,"6250")==0)||
	     (strcmp(tp->density,"HI")==0)  ||
	     (strcmp(tp->density,"0")==0))
	{
        tape_bpi = 6250;
        irgsize = .4;
	}
    else
	{
	fprintf(stderr,"Unknown density <%s>, assuming 6250\n",tp->density);
	tape_bpi = 6250;
	irgsize = .4;
	}
    }
else if (strcmp(tp->dtype,"3480") == 0)
    {
    if ((strcmp(tp->density,"LO") == 0) || (strcmp(tp->density,"0")==0))
	{
        tape_bpi = 38000;
        irgsize = .4;
	}
    else
	{
	fprintf(stderr,"Unknown density <%s>, assuming denlo\n",tp->density);
	tape_bpi = 38000;
	irgsize = .4;
	}
    }
else if (strcmp(tp->dtype,"8mm") == 0)
    {
    if ((strcmp(tp->density,"LO") == 0) || (strcmp(tp->density,"0")==0))
	{
        tape_bpi = 512000;
        irgsize = .4;
	}
    else if (strcmp(tp->density,"HI") == 0)
	{
        tape_bpi = 1024000;
        irgsize = .4;
	}
    else
	{
	fprintf(stderr,"Unknown density <%s>, assuming denlo\n",tp->density);
	tape_bpi = 512000;
	irgsize = .4;
	}
    }
return;
}

FUNCTION close_tape(fd,rewndflg)
int	fd;				/* file desc (struct TAPEDESC)	*/
long	rewndflg;			/* rewind option flag		*/
	{
	long		eofopt;
	long		rewopt = rewndflg;
	long		unit = fd;
	int		status;

	eofopt = 2;
	status = c_tpclos(&unit,&mode,&rewopt,&eofopt);
	if (status != TAPE_SUCC)
		exit(-1);
	return;
	}

FUNCTION int read_tape (fd,buffer,count)
int		fd;			/* file desc (struct TAPEDESC)	*/
unsigned char	*buffer;		/* buffer address		*/
int		count;			/* transfer count		*/
	{
	long		bufsz = count;
	long		status;
	long		unit = fd;

	status = c_tpread(&unit,buffer,&bufsz,&mode);
	if (status < 0)
	    if (status != TAPE_EOT && status != TAPE_EOV && status != TAPE_EOF)
                {
                fprintf (stderr,"[ansi-read] Error reading tape\n");
                fprintf (stderr,"[ansi-read] Fatal error encountered\n");
                }
	return(status);
	}

FUNCTION int write_tape(fd,buffer,count)
int		fd;			/* file desc (struct TAPEDESC)	*/
unsigned char	*buffer;		/* buffer address		*/
int		count;			/* transfer count		*/
{
long		bufsz = count;
long		status;
long		unit = fd;

/* This code has been altered such that an EOT status will not be returned      */
/* but will cause this routine to exit.  This is because inconsistant behavior  */
/* occurs when the various tape drives encounter an EOT, so don't recover, quit.*/

status = c_tpwrit(&unit,buffer,&bufsz,&mode);
if (status < 0)
   {
   if (status == TAPE_EOT || status == TAPE_EOV)
      fprintf (stderr,"[ansi-write]EOT or EOV encountered while writing to tape\n");
   fprintf (stderr,"[ansi-fatal]Fatal error encountered\n");
   fflush(stderr);
   exit(-1);
   }
if (status >= 0 && status != count)
    {
    fprintf(stderr,"[ansi-write] Short transfer I/O error");
    fprintf (stderr,"[ansi-fatal]Fatal error encountered\n");
   fflush(stderr);
    exit(-1);
    }
return (status);
}

FUNCTION write_eof (fd)
int	fd;				/* file desc (struct TAPEDESC)	*/
{
long	count = 1;
long	unit = fd;
int	status;

status = c_tpweof(&unit,&count,&mode);
if (status != TAPE_SUCC)
   {
   if (status == TAPE_EOT || status == TAPE_EOV)
      fprintf (stderr,"[ansi-weof]EOT or EOV encountered while writing to tape\n");
   fprintf(stderr,"[ansi-weof]Error writing eof mark\n");
   fprintf (stderr,"[ansi-fatal]Fatal error encountered\n");
   exit(-1);
   }
twrit += 4.0;				/* tapemark size ~4.0 inches	*/
return;
}

FUNCTION fsf_tape (fd,count)
int	fd;				/* file desc (struct TAPEDESC)	*/
int	count;				/* # files to space		*/
{
/*  this is a stripped-down, slightly modified version of ditto do_fsf  */

int		status;

status = c_tpfsf(&fd,&count,&mode);
if (status == TPFSR_FSR)
    {
    fprintf (stderr,"[ansi-fsf] Error encountered on fsf of tape\n");
    perror ("fsf_tape");
    exit (-1);
    }
return;
}

FUNCTION bsf_tape(fd,files)
int	fd;				/* file desc (struct TAPEDESC)	*/
int	files;				/* number of files to backspace */
{
long	count = files;
long	unit = fd;
int	status;

status = c_tpbsf(&unit,&count,&mode);
if (status != TAPE_SUCC)
	exit(-1);
return;
}

FUNCTION rewind_tape(fd,wait)
int	fd;				/* file desc (struct TAPEDESC)	*/
int	wait;				/* wait for rewind?		*/
	{
	long		opt1 = REWIND;
	long		unit = fd;
	int		status;

	if (wait)
		{
		status = c_tprew(&unit,&mode,&opt1);
		if (status != TAPE_SUCC)
			exit(-1);
		else
			;
		}
	else
		{
		long tape;

		tape = reopen_tape (REWIND,O_RDONLY);
		close_tape (tape,NOREW);
		}
	return;
	}
