#include "worgen.h"
#include "tapeio.h"
#define	GLOBALS
#include	"ansi.h"
#undef	GLOBALS

#define	USAGE1	"usage: ulout -[Tbdrtalmns] "
#define USAGE2	"[drive] [blksz] [density] [recsz] [length] file [file] ...\n"

static skip_eov();

main (argc,argv)
int	argc;
char	**argv;
{
int	aflag = 0;		/* append to end of tape		*/
int	args;			/* arguments used in parsing		*/
int	blkcnt;			/* number of blocks			*/
int	blksz = 8000;		/* block size for current file(s)	*/
int	disk;			/* disk file descriptor			*/
int	files = 0;		/* number of files output		*/
int	reccnt;			/* number of records			*/
int	recsz = 100;		/* record size for current file(s)	*/
int	rewind = 1;		/* rewind flag				*/
int	status = OK;		/* program exit status			*/
int	tape = 2;		/* tape file descriptor			*/
char	*volume_ident;		/* desired volume identifier		*/
char buffer[CMLEN];

/***   Initialize globals for unlabelled tape.   ***/

#ifdef unix
#ifdef sun
   /* I have no idea if this is correct, but it builds */
   setvbuf(stdout, buffer , _IOLBF,  CMLEN);
#else
   setlinebuf(stdout);
#endif
#endif


drive_name = NULL;
density = "";
irgsize = 0.8;
tlen = 0.0; twrit = 0.0; nfiles = 0;
volume_ident = (char *) 0;
checkvol = 0; labelled = ANSI_FALSE;
lflag = 0; newpg = 1;

/***   Process the options.   ***/

for (argc--, argv++; argc && **argv == '-'; argc -= args, argv += args)
    {
    args = 1;
    for ((*argv)++; **argv; (*argv)++)
	switch (**argv)
	    {
            case 'T':
                    drive_name = *(argv + args++);
                    break;
	    case 'a':
		    aflag++;
		    rewind = 0;
		    break;
	    case 'b':
		    blksz = atoi(*(argv + args++));
		    break;
	    case 'd':
		    density = *(argv + args++);
		    break;
	    case 'l':
		    lflag = 2;
		    break;
	    case 'm':
		    lflag = 1;
		    break;
	    case 'n':
		    ndcdb = 1;
		    break;
	    case 'r':
		    recsz = atoi(*(argv + args++));
		    break;
	    case 's':
		    splitfile = 1;
		    break;
	    case 't':
		    tlen = atoi (*(argv + args++));
		    /* assume we can use 95% of tape */
		    tlen *= (12.0 * .95);
		    break;
	    default:
		    fprintf (stderr,USAGE1);
		    fprintf (stderr,USAGE2);
		    exit (NOT_OK);
	    }
    }

if (ndcdb)
    lflag = 0;
if (blksz % recsz != 0)
    {
    fprintf (stderr,"ulout: Block size must be multiple of record size\n");
    exit (NOT_OK);
    }

mountvol ("ULOUT",volume_ident,&tape,"first tape: ",rewind,0);

if (aflag)				/* appending, skip to end of volume */
    {
    rewind_tape (tape,ANSI_TRUE);
    if (skip_eov (tape) == 0)
	rewind_tape (tape,ANSI_TRUE);
    else
	bsf_tape (tape,0);
    }

/* Copy each of the named disk files to tape. */

nfiles = argc;
for (; argc; argc--, argv++, files++, tfiles++)
    {
    START_COPY:
    disk = open_disk (*argv,O_RDONLY,0);
    status = write_F_tape("ULOUT",&tape,disk,&reccnt,&blkcnt,files,recsz,blksz);
    if (status == EOT)
	{
	if (ndcdb)
	    {
	    status = EOT;
	    goto EOT_EXIT;
	    }
	else
	    {
	    fprintf (stderr,"ulout: EOT encountered writing file %s\n",*argv);
	    fprintf (stderr,"ulout: file will be restarted on next tape\n");
	    twrit = 0; tlen = 0; newpg = 1;
	    reelswitch ("ULOUT",&tape,0,EOFF);
	    close (disk);
	    status = OK;
	    goto START_COPY;
	    }
	}
    write_eof (tape);
    close (disk);
    print_ul (*argv,recsz,blksz,reccnt,blkcnt);
    }

EOT_EXIT:
if (!ndcdb)
    {
    if (lflag != 0)
	printf ("\n%d files output\n",tfiles);
    printf ("\n%d total files output\n",files);
    }
fprintf (stderr,"ulout: drive %s: %d files output\n",drive_name,tfiles);
fprintf (stderr,"ulout: %d total files written\n",files);
write_eof (tape);
close_tape (tape,NOREW);
rewind_tape (tape,ANSI_FALSE);
exit (status);
}

static skip_eov (tape)
register int	tape;			/* tape file descriptor */

/*  Skip over all data on the output tape so that the files can be	*/
/*  appended.  This is done by reading records until an end-of-volume	*/
/*  is returned.							*/
{
register unsigned char	*buffer;	/* I/O buffer address		*/
register int		count;		/* number of bytes to read	*/
register int		status;		/* return status		*/

buffer = buf;
count = BUFSZ;

if ((status = read_tape (tape,buffer,count)) == TAPE_EOV)
    return (0);
if (status == TAPE_EOT)
    {
    bsf_tape (tape,1);
    return (0);
    }
for (;;)
    {
    status = read_tape (tape,buffer,count);
    if (status == TAPE_EOT)
        {
        bsf_tape (tape,1);
	break;
        }
    if (status == TAPE_EOV)
	break;
    }
return(1);
}
