#!/bin/csh
#
# FUNCTION:     force_release.csh
#
# PURPOSE:      Release software to the associated systems.
# $1 - Module to be force released.  This is actually a directory that
#      contains the binaries for the module to be released.
#      e.g.) $LASBASE/release/<module>
#
# $2 - Option to only release on a specified system and associated systems.
#      Options are -edcsgs20, -edcsns1, and -all.
#          "NOTE THIS IS NOT CURRENTLY IMPLEMENTED!"
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# D. Hollaren      02/00  Development

#-----------------------------------------------
# Define the systems this function should run on
#-----------------------------------------------
set admin    = (tbeckman@usgs.gov)
set usr      = release
set distsys   = (edcsgs20 edcsns1)
set halfline = "====================================="
set stamp    = `date '+%d_%b_%Y.%H%M%S' | tr "[A-Z]" "[a-z]"`
set output   = "force_$stamp.log"

#------------------------------------------------------
# Verify the correct number of arguments were specified
#------------------------------------------------------
if ( $#argv < 1 ) then
    goto USAGE
endif

#----------------------------------------------
# Get the name of the file to be force released
#----------------------------------------------
set module = $1

echo $module
#---------------------------
# Distrubte from each system
#---------------------------
foreach system ($distsys)
    #-------------------------------------
    # Ensure the distribution system is up
    #-------------------------------------
    ssh -q $system -l $usr -n date > /dev/null
    if ($status) then
        echo "ERROR: $system is down at `date`"        >>& $output
        exit 1
    endif

    #---------------------------------
    # Distribute to associated systems
    #---------------------------------
    ssh -q $system -l $usr -n force_distribute.csh $module  >>& $output
    if ($status) then
        echo "ERROR: Force releasing $module"        >>& $output
        exit 1
    endif

    #----------------------------------
    # Mail the results to $adimistrator
    #----------------------------------
    elm -s"$system release `date '+%D %T'`" $admin < $output >& /dev/null
    /bin/rm -f $output
end
exit 0

USAGE:
cat << EOM
usage: force_release.csh <module>

       Force release the specified module

       <module> - Name of the module to force release.  The <module> name
                  is actually a directory name under a "release" directory.
                  e.g.) $LASBASE/release/<module>
                  This directory contains the actual binaries to release.
EOM
exit 1
