#!/usr/local/bin/perl
#
# fileage is a script to determine whether a given file is older than
# a prescribed number of minutes.
#
# Usage: fileage.pl <filename> <max_minutes>
#
# where <max_minutes> is the maximum number of minutes old the file may
# be.
#
# Return values:   0    file < max_minutes old
#                  1    file > max_minutes old

# Verify the number of arguments.
if ($#ARGV != 1)
{
    print "Usage:  fileage.pl <filename> <max_minutes>\n";
    exit 1;
}

# Get the arguments.
my $filename = $ARGV[0];
my $max_minutes = $ARGV[1];

# Make sure the file exists.
if (! -e $filename)
{
    print "File $filename does not exist.\n";
    exit 1;
}

# Check if the file is less than max_minutes old.
# Note that the "-M" test returns the (fractional) number of days since
# the file was last modified, so we need to convert max_minutes to days.
if (-M $filename < $max_minutes/(24*60))
{
    exit 0;
}
else
{
    exit 1;
}
