#include "ditbrowse.h"

extern class Log log;
extern class List List;



/*******************************************************************************
                            Tape::Tape
*******************************************************************************/
Tape::Tape (void)
{
thebufsize = 0;
buffer1 = NULL;
buffer2 = NULL;
this->prepbuf(BR_RECSIZE);
}



/*******************************************************************************
                            Tape::mount
*******************************************************************************/
void Tape::mount (
    char *drive		// IN - Drive to open.  If "", will prompt operators
    )
{
char dens[] = "0";
char msg[CMLEN];

long inout = TREAD;
long mode = HANDLE;
long opt1 = REWIND;
long skip = 0;
long status;

short opened = FALSE;


/*
 * If the user did not specify a tape drive, request one from the operators
 */
if (strlen(drive) == 0)
    {
    strcpy(msg, "Please mount a DISP tape locally");
    status = c_tpopen(&unit, &skip, &mode, &opt1, dens, &inout, msg, tapeid);
    if (status != TAPE_SUCC)      
	log.Report(TRUE, "drive", "Error mounting tape drive");
    strcpy(drive, msg);
    opened = TRUE;
    }

/*
 * Make sure drive specified is local
 */
if (strchr(drive, '@'))
    log.Report(TRUE, "local", "Drive (%s) must be local", drive);


/*
 * If the user specified a tape drive to use, open it
 */
if (!opened)
    {
    status = c_tpreop(&unit, &skip, &mode, &opt1, dens, &inout, drive);
    if (status != TAPE_SUCC)
	log.Report(TRUE, "drive", "Error mounting tape drive %s", drive);
    }

thistape = (struct TAPEDESC *)unit;
}



/*******************************************************************************
                            Tape::fetch
*******************************************************************************/
void Tape::fetch (
    long bytes,		// IN - Number of bytes to read from the tape
    long expected	// IN - Number of bytes expected to be successfully read
    )
{ 
if ((read(thistape->fd, buffer1, (unsigned)bytes)) != expected)      
    log.Report(TRUE, "read",  "Error reading from tape");
}



/*******************************************************************************
                            Tape::Tread
*******************************************************************************/
long Tape::Tread (
    long bytes,		// IN - Number of bytes to read from the tape
    long offset		// IN - Offset into buffer1 where the data should go
    )
{ 
return ((read(thistape->fd, &buffer1[offset], (unsigned)bytes))); 
}



/*******************************************************************************
                            Tape::prepbuf
*******************************************************************************/
void Tape::prepbuf (
    long bytes		// IN - Required size of the buffers (bytes)
    )
{
/*
 * If the buffers are large enough then return
 */
if (bytes <= thebufsize)
    return;


/*
 * The buffers are not large enough so re-allocate them
 */
if ((buffer1 = (unsigned char *)realloc(buffer1, (unsigned)bytes)) == NULL)
    log.Report(TRUE, "realloc", "Error allocating buffer memory");
if ((buffer2 = (unsigned char *)realloc(buffer2, (unsigned)bytes)) == NULL)
    log.Report(TRUE, "realloc", "Error allocating buffer memory");
}



/*******************************************************************************
                            Tape::close
*******************************************************************************/
void Tape::close(void)
{
long opt1=REWOFFL;
long mode=HANDLE;

List.finish();
free (buffer1);
free (buffer2);
c_tpclos(&unit, &mode, &opt1, &opt1);
}



/*******************************************************************************
                            Tape::proces_head
*******************************************************************************/
void Tape::proces_head (
    char *header,	// OUT - DIT browse buffer to be filled
    char *sceneid,	// I/O - ID of scene to be processed
    long *nlines,	// OUT - Number of lines in the browse image
    long *nsamps	// OUT - Number of samples in the browse image
    )
{
long i, j;		// Loop control
long num_bytes;
long num_records;
char temp[CMLEN];

memcpy(header, buffer1, HEADER_DATA_SIZE);
header[HEADER_DATA_SIZE] = '\0';


/*
 * Extract the number of lines from the header
 */
memcpy(temp, &header[31], 4);
temp[4] = '\0';
*nlines = atol(temp);


/*
 * Extract the number of samples from the header
 */
memcpy(temp, &header[35], 4);
temp[4] = '\0';
*nsamps = atol(temp);


/*
 * Extract the scene ID from the header
 */
for (i=0, j=15; i<16; i++, j++)
    {
    if (header[j] == ' ')
	break;
    temp[i] = header[j];
    }
temp[i] = '\0';


/*
 * The scene ID extracted from the ANSI header file (sceneid) should match
 * the ID found in the browse header record.  If not, issue an error msg.
 */
if (strcmp(sceneid, temp) != 0)
    {
    /*
     * The ID found in the ANSI header file should be unique because the files
     * should have been stacked to tape from a disk.  Therefore, use the ID
     * from the ANSI header.  This logic comes from tape 095893 where images
     * 66-90 have shorter names in the browse header than in the ANSI header.
     * Using the shorter ID means there are duplicate file names and all but
     * the last jpeg image are over-written.
     *
     * Create a record for the ID found in the browse header and continue.
     */
    List.update(temp, 0, 0, FALSE, FALSE, FALSE, FALSE, TRUE, TRUE,
	"This ID from browse hdr does not match ANSI hdr");

    /*
     * Issue an error message to the log file.
     */
    log.Report(FALSE, "error", "ANSI hdr = %s  Browse hdr = %s", sceneid, temp);
    log.Report(FALSE, "warn", "JPEG file uses: %s", sceneid);
    }


/*
 * Determine the number of bytes needed to hold the entire image in memory.
 * Call prepbuf() to resize the buffers if necessary.
 */
num_bytes = (*nlines) * (*nsamps) + HEADER_DATA_SIZE;
num_records = num_bytes % BR_RECSIZE + 1;
prepbuf(num_records * BR_RECSIZE);
}



/*******************************************************************************
                            Tape::GetTapeid
*******************************************************************************/
void Tape::GetTapeid (
    char *logdir,	// IN  - Directory where log file should be created
    char *jpegdir	// I/O - Directory where jpeg images should be stored
    )
{
/*
 * Extract the tape ID from the first record of the first ANSI header file.
 * It is assumed that this record is already stored in buffer1.
 */
memcpy(tapeid, &buffer1[4], 6);
tapeid[6] = '\0';


/*
 * Kodak mis-labelled tape 095956.  The physical label is still 095956 but the
 * ANSI header file indicates 095056.  Change the label read from the ANSI
 * header and pretend it is correct.
 */
if (strcmp(tapeid, "095056") == 0)
    strcpy(tapeid, "095956");


/*
 * Initialize the log file
 */
log.InitLog(logdir, tapeid, "ditbrw");
log.Report(FALSE, "tapeid", "Mounted tape %s", tapeid);


/*
 * Now that the log has been initialized, record the change in tape ID
 */
if (strcmp(tapeid, "095956") == 0)
    {
    log.Report(FALSE, "tplbl",
	"Mis-labelled tape ID hard coded from 095056 to 095956");
    }


/*
 * Ensure that jpegdir exists
 */
if (access(jpegdir,0) != 0)
    log.Report(TRUE, "exist", "Directory does not exist %s", jpegdir);

log.Report(FALSE, "jpegdir", "Output jpeg directory = %s", jpegdir);
List.start(logdir, tapeid);
}



/*******************************************************************************
                            Tape::flip
*******************************************************************************/
void Tape::flip (
    const long nlines,	// IN - Number of lines in the browse image
    const long nsamples	// IN - Number of samples in the browse image
    )
{
long buf1;		// Index into buffer1
long buf2;		// Index into buffer2
long i;			// Loop control
long line;		// Current line being processed


/*
 * NOTE:  Don't forget about the header information at the begining of the
 *        image buffers.  This is ASCII information that should not be flipped.
 *        The header is HEADER_DATA_SIZE bytes in size.
 */


/*
 * Flip the image in buffer1 vertically and store it in buffer2.  The copying
 * will proceed backwards through buffer1 and forward through buffer2.  The
 * flipping in the sample direction could occur during this loop but the data
 * has to be copied back to buffer1 anyway so, to keep the code a little more
 * clear, these steps are performed separately.
 */
for (line=0; line<nlines; line++)
    {
    /*
     * Calculate the starting location of this line in each buffer
     */
    buf1 = HEADER_DATA_SIZE + (nlines - line - 1) * nsamples;
    buf2 = HEADER_DATA_SIZE + line * nsamples;


    /*
     * Copy every sample for this line
     */
    for (i=0; i<nsamples; i++)
	this->buffer2[buf2++] = this->buffer1[buf1++];
    }


/*
 * Flip the image in buffer2 horizontally and store it back in buffer1
 */
for (line=0; line<nlines; line++)
    {
    /*
     * Calculate the starting location of this line in both buffers.  The
     * starting location for buffer2 will be the last sample in the current
     * line.  The index for buffer1 will increase in the sample direction and
     * for buffer2 it will decrease (ie go backwards).
     */
    buf1 = HEADER_DATA_SIZE + line * nsamples;
    buf2 = HEADER_DATA_SIZE + (line + 1) * nsamples - 1;


    /*
     * Copy every sample for this line
     */
    for (i=0; i<nsamples; i++)
	this->buffer1[buf1++] = this->buffer2[buf2--];
    }
}
