extern "C" 
{
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "lasp.h"
#include "tapeio.h"
}

void getparm(int argc, char **argv, char *path, long *quality, long *start,
    long *end, char *drive, char *logpath);
void rotate (char *hdr);
void timer_start (void);
void timer_stop (long count);

extern "C"
{
void write_JPEG_file (unsigned char *buffer, char *jpeg_file_name, int quality,
    char *header, long nlines, long nsamps, long nbands);
}


#define HT_RECSIZE              80 /* Record size for ANSI hdr & trl files    */
#define BR_RECSIZE           32768 /* Record size for browse image files      */
#define HEADER_DATA_SIZE       122 /* Browse header data size                 */
#define DEFAULT_JPEG_QUALITY    75 /* jpeg compression quality factor         */
#define DEFAULT_START            1 /* Default first image to process from tape*/
#define NO_END                  -1 /* Flag indicating to process all images   */

#define JPEG_ROOT_DIR        "/edcsns17/browse/DIT/disp.in"
#define DEFAULT_LOG_PATH     "/edcsns17/browse/DIT/disp.log"
#define LOG_FILE_NAME        "ditbrowse.log"
#define LIST_FILE_NAME       "ditbrowse.list"


/******************************************************************************/
class Tape
    {
    private:
        long unit;
        long thebufsize;
        struct TAPEDESC *thistape;

    public:
        char tapeid[7];
        unsigned char *buffer1;
        unsigned char *buffer2;

        Tape();
        void mount(char *drive);
        void fetch(long bytes, long expected);
        long Tread(long bytes, long offset);
        void prepbuf(long bytes);
        void proces_head(char *header, char *sceneid, long *nlines,
	    long *nsamps);
        void close();
        void GetTapeid(char *logdir, char *jpegpath);
	void flip(const long nlines, const long nsamples);
    };

class List
    {
    private:
        char list_file_name[CMLEN];

    public:
        void start(char *logpath, char *tapeid);
        void update(char *scene, long tnum, long quality, long tflag,
	    long cflag, long fflag, long jflag, long eflag, long nflag,
	    char *path);
        void finish(void);
	short expect(void);
    };

class Log
    {
    private:
        char logfile[CMLEN];
        char tapeid[CMLEN];
        char keyroot[CMLEN];

    public:
	Log();
        void InitLog(char *dir, char *id, char *root);
        void Report(long flag, char *key, char *fmt, ...);
    };
