#include "ditbrowse.h"


static void usage (char *program)
{
fprintf(stderr, "usage: %s %s\n", program,
    "[-help] [-p <jpeg-dir>] [-q <quality>] [-d <drive>]");
fprintf(stderr, "       [-l <log-dir>] [-s <start>] [-e <end>]\n");
exit(1);
}


void getparm (
    int argc,		// IN  - Number of command line arguments
    char **argv,	// IN  - Array of ptrs to the arguments
    char *jpegdir,	// OUT - Directory for jpeg images that are created
    long *quality,	// OUT - JPEG quality factor
    long *start,	// OUT - First image to process
    long *end,		// OUT - Last image to process
    char *drive,	// OUT - Tape drive to use
    char *logdir	// OUT - Directory for log and list files
    )
{
long offset;


/*
 * Too few arguments?
 */
if (argc < 2)
    usage(argv[0]);


/*
 * Is the user requesting help?
 */
if ((strcmp(argv[1], "-h") == 0) || (strcmp(argv[1], "-help") == 0))
    {
    system("ditbrowse.doc");
    exit(1);
    }


/*
 * Check for an odd number of arguments
 */
if (argc % 2 != 1)
    usage(argv[0]);


/*
 * Set all of the default values
 */
strcpy(jpegdir, JPEG_ROOT_DIR);
strcpy(logdir, DEFAULT_LOG_PATH);
*quality = DEFAULT_JPEG_QUALITY;
*start   = DEFAULT_START;
*end     = NO_END;


/*
 * Process the arguments
 */
for (offset = 1; offset < argc; offset += 2)
    {
    if ((strlen(argv[offset]) != 2) || (argv[offset][0] != '-'))
	usage(argv[0]);

    switch (argv[offset][1])
	{
	case 'p':
	    /*
	     * Path for the images
	     */
	    strcpy(jpegdir, argv[offset + 1]);
	    break;

	case 'q':
	    /*
	     * jpeg compression quality
	     */
	    sscanf(argv[offset + 1], "%d", quality);
	    break;

	case 's':
	    /*
	     * Starting image to process
	     */
	    *start = atol(argv[offset + 1]);
	    break;

	case 'e':
	    /*
	     * Ending image to process
	     */
	    *end = atol(argv[offset + 1]);
	    break;

	case 'd':
	    /*
	     * Tape drive to be used
	     */
	    strcpy(drive, argv[offset + 1]);
	    break;

	case 'l':
	    /*
	     * Directory for the log file
	     */
	    strcpy(logdir, argv[offset + 1]);
	    break;

	default:
	    usage(argv[0]);
	}
    }


/*
 * Check the quality setting
 */
if ((*quality < 1) || (*quality > 100))
    c_lasmsg(TRUE,"ditbrowse-quality","Quality must be between 1 and 100");


/*
 * Check the starting and ending image numbers
 */
if (*start < 1)
    c_lasmsg(TRUE,"ditbrowse-start","Start must be greater than zero");
else if ((*end != NO_END) && (*start > *end))
    c_lasmsg(TRUE,"ditbrowse-end","End must be greater than %d",*start);
}
