
/******************************************************************************
NAME:     las2host.c 
PURPOSE:  a unix utility to convert file name(s) from LAS to host format, 
          developed for LAS>LASSHELL 

PROGRAM HISTORY
-------------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  ----------------------------------------------------
Apr 1995  David Zokaites   Original development for $LASDITTY
Oct 1995  David Zokaites   Changed indentation, released to $LASTOOLS
******************************************************************************/




#include <stdio.h>  /* for calls to fprintf() */ 
#include <stdlib.h> /* for calls to exit() */
#include <string.h> /* for calls to strcmp() */ 
#include "las.h"    /* las header file */ 





/* function prototypes, ifdef allows for K&R compilers */ 
#ifdef __DGUX__
   int main (int argc, char *argv[]); 
   void print_usage (void);
#else 
   int main (); 
   void print_usage ();
#endif






/* PROGRAM MAIN */ 
FUNCTION int main (argc, argv)
   int argc;      /* number of input arguments */
   char *argv[];  /* array of input arguments */ 
{
/* declare variables */ 
   char outname[CMLEN];  /* output:  file name in host format */ 
   int filenum,          /* for loop index, filenames */ 
       oneline;          /* flag, does user want one file printed per line */ 

/* check number of input arguments */ 
   if (argc < 2)
      print_usage (); 

/* get oneline parm, set curfile */ 
   if (strcmp (argv[1], "-1"))
      {
      oneline = FALSE;  
      filenum = 1; 
      }
   else
      {
      oneline = TRUE;
      filenum = 2; 
      if (argc < 3)
	 print_usage (); 
      }

/* convert filenames */ 
   for (;  filenum<argc;  filenum++)
      {
      /* convert name from las to host */ 
	 if (c_cmhost (argv[filenum], outname) )
	    {
	    fprintf (stderr, 
	    "las2host:  error converting file name '%s' from %s", 
	    argv[filenum], "LAS to host format\n"); 
	    return (1); 
	    }

      /* print filename with spaces or newlines between filenames */ 
	 fprintf (stdout, "%s", outname); 
	 if (oneline)
	    fprintf (stdout, "\n"); 
	 else if (argc > 1 && filenum != argc -1)
	    fprintf (stdout, " "); 
      }

   if (!oneline) 
      fprintf (stdout, "\n"); 

/* successful completion */
   return (0); 
} 





/******************************************************************************
NAME:     print_usage() 
PURPOSE:  print out a usage error message 
******************************************************************************/

FUNCTION void print_usage ()
{
   fprintf (stderr, "Las2host converts input file and directory names "); 
   fprintf (stderr, "from LAS to host format.\n");
   fprintf (stderr, "Usage:  las2host [-1] filename(s)\n"); 
   fprintf (stderr, "The -1 flag results in printing one filename per line.\n");
   fprintf (stderr, "Note that LAS filenames include metacharacters ([];)\n");
   fprintf (stderr, "that require quotes as in the following example:\n");
   fprintf (stderr, "%% las2host '[ebeth.child]toy;box' \n");
   exit (1);
}
