#include "lfind.h"
#define MAXDIRS 13



/*******************************************************************************
FUNCITON: pdf_usage

PURPOSE:  Print the usage message and error out.

Programmer       Date   Modifications
---------------  -----  --------------------------------------------------------
J.Fenno          03/93  Development

PROJECT:  ADAPS
*******************************************************************************/
static int pdf_usage ()
{
printf("usage: lfind pdf <file>\n");
printf("\n");
printf("       Search through all of the application directories for any\n");
printf("       PDF matching *<file>*.pdf.  All LAS and ADAPS directories\n");
printf("       will be searched.\n");
exit(1);
}




/*******************************************************************************
FUNCITON: pdf

PURPOSE:  Search the user's path looking for the specified command.

Programmer       Date   Modifications
---------------  -----  --------------------------------------------------------
J.Fenno          03/93  Development
J.Fenno		 08/95	Added subdirectories with LASLOCAL and LASCONTRIB.

PROJECT:  ADAPS
*******************************************************************************/
int pdf (argc, argv)
    int argc;
    char *argv[];
{
char cmd[256];			/* Command to send to operating system        */
char dirs[MAXDIRS][256];	/* Directories to search through              */
char pdf[256];			/* Name of a pdf file                         */
char *ptr;			/* Ptr to a string variable                   */

int i;				/* Loop control                               */
int len;			/* Length of the pdf argument                 */
int status;			/* Status sent to pclose()                    */

FILE *pdf_pipe;			/* Pipe to a list of pdf files                */


/* Get each of the possible directories
---------------------------------------*/
ptr = getenv("CVSROOT");
if (ptr == NULL)
    {
    printf("Error retrieving CVSROOT\n");
    exit(1);
    }
sprintf(dirs[0],  "%s/las/base/app", ptr);
sprintf(dirs[1],  "%s/las/base/ditto", ptr);
sprintf(dirs[2],  "%s/las/contrib/app", ptr);
sprintf(dirs[3],  "%s/las/contrib/app/lams", ptr);
sprintf(dirs[4],  "%s/las/local/app", ptr);
sprintf(dirs[5],  "%s/las/local/app/lams", ptr);
sprintf(dirs[6],  "%s/las/local/app/lasdrain", ptr);
sprintf(dirs[7],  "%s/las/local/app/lb", ptr);
sprintf(dirs[8],  "%s/las/local/app/mssx", ptr);
sprintf(dirs[9],  "%s/las/local/app/pds", ptr);
sprintf(dirs[10], "%s/adaps/base/app", ptr);
sprintf(dirs[11], "%s/adaps/contrib/app", ptr);
sprintf(dirs[12], "%s/adaps/local/app", ptr);


/* Check the arguments
----------------------*/
if (argc != 1)
    pdf_usage();


/* Determine the length of the pdf string
-----------------------------------------*/
len = strlen(argv[0]);


/* For each of the application directories look for all pdfs
------------------------------------------------------------*/
for (i = 0; i < MAXDIRS; i++)
    {
    /* Change to the specified directory
    ------------------------------------*/
    chdir(dirs[i]);


    /* Open a pipe to a list all of the pdfs in the subdirectories
    --------------------------------------------------------------*/
    sprintf(cmd, "echo */*%s*.pdf*", argv[0]);
    pdf_pipe = popen(cmd, "r");
    if (pdf_pipe == NULL)
	{
	printf("error opening pipe\n");
	exit(1);
	}


    /* For each pdf
    ---------------*/
    while (fscanf(pdf_pipe, "%s", pdf) == 1)
	{
	ptr = strchr(pdf, '*');
	if (ptr == NULL)
	    printf("%s/%s\n", dirs[i], pdf);
	}


    /* Close the pdf pipe
    ---------------------*/
    pclose(pdf_pipe);
    wait(status);
    }

return(0);
}
