#include "lfind.h"
#define MAXDIRS 6



/*******************************************************************************
FUNCITON: sup_usage

PURPOSE:  Print the usage message and error out.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/93  Development

PROJECT:  ADAPS
*******************************************************************************/
static int sup_usage ()
{
printf("usage: lfind sup <file>\n");
printf("\n");
printf("       Search through all of the support directories for the\n");
printf("       specified file.\n");
exit(1);
}




/*******************************************************************************
FUNCITON: sup

PURPOSE:  Search the user's path looking for the specified command.

Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          03/93  Development

PROJECT:  ADAPS
*******************************************************************************/
int sup (argc, argv)
    int argc;
    char *argv[];
{
char cmd[256];			/* Command for the pipe                       */
char dirs[MAXDIRS][256];	/* Directories to search through              */
char sup[256];			/* Name of a support file                     */
char *ptr;			/* Ptr to a string variable                   */

int i;				/* Loop control                               */
int len;			/* Length of the pdf argument                 */
int num;			/* Number of supports found                   */
int status;			/* Status sent to pclose()                    */

FILE *sup_pipe;			/* Pipe to a list of support files            */


/* Get each of the possible directories
---------------------------------------*/
ptr = getenv("CVSROOT");
if (ptr == NULL)
    {
    printf("Error retrieving CVSROOT\n");
    exit(1);
    }
sprintf(dirs[0], "%s/las/base/support", ptr);
sprintf(dirs[1], "%s/las/local/support", ptr);
sprintf(dirs[2], "%s/las/contrib/support", ptr);
sprintf(dirs[3], "%s/adaps/base/support", ptr);
sprintf(dirs[4], "%s/adaps/local/support", ptr);
sprintf(dirs[5], "%s/adaps/contrib/support", ptr);


/* Check the arguments
----------------------*/
if (argc != 1)
    sup_usage();


/* Determine the length of the support file string
--------------------------------------------------*/
len = strlen(argv[0]);


/* For each of the support directories look for all support files
-----------------------------------------------------------------*/
num = 0;
for (i = 0; i < MAXDIRS; i++)
    {
    /* Change to the specified directory
    ------------------------------------*/
    chdir(dirs[i]);


    /* Open a pipe to a list all of the support files in the subdirectories
    -----------------------------------------------------------------------*/
    sprintf(cmd, "echo */*%s*", argv[0]);
    sup_pipe = popen(cmd, "r");
    if (sup_pipe == NULL)
	{
	printf("error opening pipe\n");
	exit(1);
	}


    /* For each support
    -------------------*/
    while (fscanf(sup_pipe, "%s", sup) == 1)
	{
	ptr = strchr(sup, '*');
	if (ptr == NULL)
	    {
	    printf("%s/%s\n", dirs[i], sup);
	    num++;
	    }
	}


    /* Close the pdf pipe
    ---------------------*/
    pclose(sup_pipe);
    wait(status);
    }

if (num)
    return(0);
else
    return(1);
}
