#!/bin/csh
#
# FUNCTION: fmtlug.csh
#
# PURPOSE:  Format mail messages containing new software requests for the
#           weekly LUG notes.
#
# $Author: tbeckman $
# $Date: 2000/06/29 21:44:51 $
# $Source: /edclxs64/www/adaps/software/src/bin/lug/fmtlug.csh,v $
# $Revision: 1.1 $
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# J.Fenno          09/95  Development
#
#
set noglob

#-------------------------
# Check for an output file
#-------------------------
if ($#argv != 1) then
    echo 'usage: fmtlug.csh <file>'
    echo ''
    echo '       Format a new change request mail message for printing in the'
    echo '       weekly LUG notes.  The mail message is read from standard'
    echo '       input and the reformatted message is appended to <file>.'
    exit 1
endif
set lugmsg = $1


#--------------------------------------------------------------
# Throw away everything until the subject line is located.  For
# a simplistic check, don't read more than 20 lines of input.
#--------------------------------------------------------------
set tmp = 0
set line = "dummy"
set word = "dummy"
while ("$word" != "Subject:")
    set line = "$<"
    if ($tmp < 20) then
	@ tmp += 1
	set word = `echo $line | awk '{print $1}'`
    else
	echo "fmtlug: No subject found in $tmp lines"
	exit 1
    endif
end


#---------------------
# Locate the CR number
#---------------------
set tmp = 0
while ("$word" != ">Number:")
    set line = "$<"
    if ($tmp < 30) then
	@ tmp += 1
	set word = `echo $line | awk '{print $1}'`
    else
	echo "fmtlug: No number found in $tmp lines"
	exit 1
    endif
end
set number = `echo $line | awk '{print $2}'`

#--------------------
# Locate the category
#--------------------
set line = "$<"
set word = `echo $line | awk '{print $1}'`
if ("$word" != ">Category:") then
    echo "fmtlug: Category not found"
    exit 1
endif
set category = `echo $line | awk '{print $2}'`


#--------------------
# Locate the synopsis
#--------------------
set line = "$<"
set word = `echo $line | awk '{print $1}'`
if ("$word" != ">Synopsis:") then
    echo "fmtlug: Synopsis not found"
    exit 1
endif
set synopsis = `echo $line | awk '{ for (i = 2; i <= NF; i++) print $i }'`


#---------------
# Locate the EWR
#---------------
set line = "$<"
set word = `echo $line | awk '{print $1}'`
if ("$word" != ">EWR:") then
    echo "fmtlug: EWR not found"
    exit 1
endif
set ewr = `echo $line | awk '{print $2}'`


#--------------------
# Locate the severity
#--------------------
set line = "$<"
set line = "$<"
set word = `echo $line | awk '{print $1}'`
if ("$word" != ">Severity:") then
    echo "fmtlug: Severity not found"
    exit 1
endif
set severity = `echo $line | awk '{print $2}'`


#--------------------
# Locate the priority
#--------------------
set line = "$<"
set word = `echo $line | awk '{print $1}'`
if ("$word" != ">Priority:") then
    echo "fmtlug: Priority not found"
    exit 1
endif
set priority = `echo $line | awk '{print $2}'`


#----------------------
# Locate the originator
#----------------------
set line = "$<"
set line = "$<"
set line = "$<"
set line = "$<"
set line = "$<"
set line = "$<"
set line = "$<"
set line = "$<"
set word = `echo $line | awk '{print $1}'`
if ("$word" != ">Originator:") then
    echo "fmtlug: Originator not found"
    exit 1
endif
set originator = `echo $line | awk '{ for (i = 2; i <= NF; i++) print $i }'`


#------------------------------------------------------------------------------
# Skip to the description of the problem.  Messages that are forwarded from the
# release account's mail filter will have a leading space on every line.  This
# is a simplistic check...
#------------------------------------------------------------------------------
set tmp = 0
while (("$line" != ">Description:") && ("$line" != " >Description:"))
    set line = "$<"
    @ tmp += 1
    if ($tmp > 30) then
	echo "fmtlug: No description found in $tmp lines"
	exit 1
    endif
end

#------------------------------------------
# Write the information to a formatted file
#------------------------------------------
set tmp = "------------------------------------------------------------------"
echo "" >> $lugmsg
echo "" >> $lugmsg
echo "$synopsis" >> $lugmsg
echo "$tmp" >> $lugmsg
echo "$originator" >> $lugmsg
if (`echo $category | wc -c` < 5) then
    echo "CR $category/$number		EWR $ewr" >> $lugmsg
else
    echo "CR $category/$number	EWR $ewr" >> $lugmsg
endif
echo "Priority: $priority	Severity: $severity" >> $lugmsg
echo "" >> $lugmsg


#---------------------
# Copy the description
#---------------------
set tmp = 0
set line = "$<"
while (("$line" != ">How-To-Repeat:") && ("$line" != " >How-To-Repeat:"))
    if ($tmp < 100) then
	echo "$line" >> $lugmsg
	@ tmp += 1
	set line = "$<"
    else
	echo "fmtlug: Description longer than $tmp lines"
	exit 1
    endif
end
exit 0
