#!/bin/csh
#
# FUNCTION: lug_notes.csh
#
# PURPOSE:  Check all of the release directories for applications that are
#           being released.  When a "force_tstamp.log" file exists in a
#           directory the application was force released.
#
# $Author: dlabahn $
# $Date: 2002/09/26 21:30:18 $
# $Source: /edclxs64/www/adaps/software/src/bin/lug/lug_notes.csh,v $
# $Revision: 1.4 $
#
# Programmer       Date   Modifications
# ---------------  -----  ------------------------------------------------------
# J.Fenno          12/94  Development
# J.Fenno          08/95  No need to base directories off of $EDCSOFT.
# D.Etrheim        06/96  Reorganize the listing of directories 
#

unalias cd
unalias ls
set reldir=/sg1/csb/release/notices
set curdir=`pwd`
set lug = $curdir/weekly.lug
set new_crs = $curdir/newcrs.numb
set date=`date +"%b %d %Y"`
set cnt=1
set num=1

#-------------------------------------------
# Make sure the file that has new CRs exists
#-------------------------------------------
if (! -e $new_crs) then
    echo "$new_crs does not exist."
    exit 1
endif

mv $lug $lug.bck
echo "                           LAS/ADAPS Weekly Notes" > $lug
echo "                                 $date " >> $lug
echo " " >> $lug
echo " " >> $lug

#----------------------------------------
# Get the list of release note file names
#----------------------------------------
echo "`/bin/ls -1t $reldir/rel.* | /bin/cut -d/ -f 6`" > $curdir/temp

#----------------------------------------------
# Get the latest file(s) with the release notes
#----------------------------------------------
foreach file (`/bin/cat $curdir/temp`)
    #------------------------------------------
    # Save at least the three latest file names
    #------------------------------------------
    if ($cnt == 1 ) then
        set relfile=$reldir/${file}
    else if ( $cnt == 2 ) then
        set relfile2=$reldir/${file}
    else if ( $cnt == 3 ) then
        set relfile3=$reldir/${file}
    endif

    #-------------------------
    # Increment the file count
    #-------------------------
    set cnt=`/bin/expr $cnt + 1`

    #----------------------------------
    # Only check the three latest files
    #----------------------------------
    if ($cnt > 3) then
        break
    endif
end

#---------------------
# Remove the temp file
#---------------------
/bin/rm -f $curdir/temp

echo "Checking for files to be RELEASED"
echo ""

foreach dir_path ( \
    $LASBASE/release \
    $LASCONTRIB/release \
    $LASLOCAL/release \
    $LASDITTO/release \
    $ADAPSBASE/release \
    $ADAPSLOCAL/release \
    $ADAPSCONTRIB/release \
    $AIRBASE/release )

    #--------------------
    # Check for any files
    #--------------------
    cd $dir_path
    set nfiles=`/bin/ls | /bin/wc -l`
    if ($nfiles == 0) then
        continue
    endif

    #---------------------------------------
    # Create a name the user will understand
    #---------------------------------------
    if ( $dir_path == $LASBASE/release ) then
        set rel = "LASBASE"
    else if ( $dir_path == $LASCONTRIB/release ) then
        set rel = "LASCONTRIB"
    else if ( $dir_path == $LASLOCAL/release ) then
        set rel = "LASLOCAL"
    else if ( $dir_path == $LASDITTO/release ) then
        set rel = "LASDITTO"
    else if ( $dir_path == $ADAPSBASE/release ) then
        set rel = "ADAPSBASE"
    else if ( $dir_path == $ADAPSLOCAL/release ) then
        set rel = "ADAPSLOCAL"
    else if ( $dir_path == $ADAPSCONTRIB/release ) then
        set rel = "ADAPSCONTRIB"
    else if ( $dir_path == $AIRBASE/release ) then
        set rel = "AIRBASE"
    endif

    #-------------------------------------
    # Check the directories to be released
    #-------------------------------------
    foreach func (*)
        #------------------------------------------
        # The file was not a directory so remove it
        #------------------------------------------
        if (! -d $func) then
            echo "Unknown file $func in release directory ${rel}/release"
            /bin/rm -f $func
            continue
        endif

        #-----------------------------------------------------
        # Check to see if there are any files in the directory
        # When there are no files remove the directory
        #-----------------------------------------------------
        cd $func
        set nfiles=`/bin/ls | /bin/wc -l`
        if ($nfiles == 0) then
            cd ..
            echo "Empty release directory ${rel}/release/$func"
            /bin/rm -rf $func
            continue
        endif

        #----------------------------------------------------
        # Upcase the name and log the function to be released
        #----------------------------------------------------
        set name = `echo $func | /bin/tr '[a-z]' '[A-Z]'`
        set nfiles=`/bin/ls FORCE* | /bin/wc -l`
        if ($nfiles == 0) then
            set verb = "will be"
        else
            set verb = "was"
        endif
        echo "$num. $name $verb released to ${rel}."
        echo ""
        echo "$num. $name $verb released to ${rel}."  >> $lug
        echo "" >> $lug

        #---------------------------
        # Increment the number count
        #---------------------------
        set num=`expr $num + 1` 

        #--------------------------------
        # Get the info from release notes 
        #--------------------------------
        foreach release_file ($relfile $relfile2 $relfile3)
            #--------------------------------------------------------------
            # Check to see if there is any info on the released application
            #--------------------------------------------------------------
            if ( { /bin/grep -i $func $release_file >/dev/null } ) then
                #-------------------------------------------------------
                # Get the actual name from the release file so the "awk"
                # will not fail on case.
                #-------------------------------------------------------
                set name = "`/bin/grep -i $func $release_file | /bin/cut -d' ' -f1`"
                set name = "`echo $name | /bin/cut -d' ' -f1`"

                #-------------------------------------------------
                # Retreive the release info from the release notes
                #-------------------------------------------------
                /bin/cat $release_file | /bin/awk '/'$name'/, /\*\*\*/' | /bin/grep -v "\*\*\*" >> $curdir/temp

#                doedit $curdir/temp

                #--------------------------------------------
                # Add the release information to the lug file
                #--------------------------------------------
                /bin/cat $curdir/temp >> $lug
                echo "" >> $lug

                #---------------------
                # Remove the temp file
                #---------------------
                /bin/rm -f $curdir/temp
            endif
        end             # end of the release files

        #--------------------------------------
        # Remove the release directory reported
        #--------------------------------------
        cd ..
#        /bin/rm -rf $func
    end                 # end of the functions in the directory loop
end                     # end of the directory search loop

#-------------------------------------
# Change back to the current directory
#-------------------------------------
cd $curdir

#------------------------------
# Log new Change Requests (CRs)
#------------------------------
echo "" >> $lug
echo "" >> $lug
echo "${num}.  New Requests" >> $lug
newcr.csh $new_crs $curdir/cr.new
if (! $status) then
    #---------------------------------
    # Add new requests to the lug file
    #---------------------------------
    cat $curdir/cr.new >> $lug
    rm -f $curdir/cr.new
endif

echo "" >>$lug
echo "--------------------------------------------------------------------">>$lug
# Change the following line since $user is not the same as my email address
echo "Questions or concerns can be emailed to $user@usgs.gov">>$lug
#mail -s <LUG>Wkly Notes edcbb@edcsgs9.cr.usgs.gov < $lug
