#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "pdfhtml.h"
#include "local_protos.h"

long pass_one
(
    FILE    *ifp,               /* Input file pointer  (to PDF)              */
    FILE    *ofp                /* Output file pointer (to HTML)             */
)
{
struct SUBCMDS subcmd[35];      /* Parameters and defaults per subcmd        */

long    put_txt = FALSE,        /* Write text to output flag                 */
        level2 = FALSE,         /* Process and save level 2 help             */
        do_sub = TRUE,          /* Save subcommand parameters                */
        fparm = TRUE,           /* First parameter flag                      */
        status = E_SUCC,        /* Status of read                            */
        nsubs = 0,              /* Number of parameters in the function      */
        cur_pos = 0,            /* Current position in the input file        */
        titl_pos = 0,           /* Position marker used to combine multi-
                                   line titles                               */
        len,                    /* Length of a string                        */
        i;                      /* Looping variable                          */

char    string[4*STR_SIZ],      /* String read from the input file           */
        key[KEY_SIZ],           /* First word in each line to key off of     */
        func[PARM_SIZ],         /* Name of the function                      */
        parm[PARM_SIZ],         /* Name of the function                      */
        sub_name[PARM_SIZ],     /* Name of the function                      */
        *def,                   /* Pointer to the parameter default value    */
        tmp_str[STR_SIZ];       /* String read from the input file           */

memset(subcmd,0,sizeof(subcmd));

/*
  Process the input file
*/
while (status == E_SUCC)
    {
    /*
      Read a record from the input file (PDF)
    */
    status = nxt_rec(ifp,string,key);
    if (status != E_SUCC)
        break;

    /*
      Check the key
    */
    if ((strncmp(key,"SUBCMD",6) == 0) && (do_sub))
        {
        /*
          Save the subcommand name
        */
        sscanf(string,"%*s %s",subcmd[nsubs].name);
        }
    else if ((strncmp(key,"END-SUB",7) == 0) && (do_sub))
        {
        /*
          Increment the number of subcommands found
        */
        nsubs++;
        }
    else if ((strncmp(key,"BODY",4) == 0) || (strncmp(key,"END-PROC",8) == 0))
        {
        /*
          All subcommand parameters have been saved, Ignore any other
          subcommand parameter keys found
        */
        do_sub = FALSE;
        }
    else if (strncmp(key,"PARM",4) == 0)
        {
        /*
          Upper case the string so the parameters and defaults
          are upper case in the user's guide
        */
        len = (long)strlen(string);
        c_low2up(string,&len);

        /*
          Extract the parameter name
        */
        sscanf(string,"%*s %s",subcmd[nsubs].parm[subcmd[nsubs].nparms].name);

        /*
          Extract the default for the parameter
        */
        if ((def = strstr(string,"DEFAULT")))
            {
            def = strchr(def,'=');
            while ((*def == ' ') || (*def == '(') || (*def == '=') ||
                   (*def == '+'))
                def++;

            *(def + strlen(def) - 1) = '\0';
            strcpy(subcmd[nsubs].parm[subcmd[nsubs].nparms].deflt,def);
            def = strchr(subcmd[nsubs].parm[subcmd[nsubs].nparms].deflt,')');
            if (def != NULL)
                *def = '\0';
            }
        else
            strcpy(subcmd[nsubs].parm[subcmd[nsubs].nparms].deflt," ");

        /*
          Increment the parameter count
        */
        subcmd[nsubs].nparms++;

        /*
          Set the processing flag
        */
        put_txt = FALSE;
        }
    else if (strncmp(key,"!.HEADER",8) == 0)
        {
        /*
          Read the next line to get the name of the function
        */
        status = nxt_rec(ifp,string,key);
        if (status != E_SUCC)
            break;

        /*
          Extract the function name
        */
        if ((def = strstr(string,"NAME")))
            {
            def = strchr(def,'=');
            while ((*def == ' ') || (*def == '(') || (*def == '=') ||
                   (*def == '+'))
                def++;

            sscanf(def,"%s %*s",func);
            }
        else
            sscanf(string,"%*s %s %*s",func);

        /*
          Set the processing flags
        */
        put_txt = FALSE;
        }
    else if (strncmp(key,".TITLE",5) == 0)
        {
        /*
          Read the next line to get the title
        */
        status = nxt_rec(ifp,string,key);
        if (status != E_SUCC)
            break;
        string[strlen(string)-1] = '\0';

        /*
          Save the current position in the input file.
        */
        titl_pos = ftell(ifp);

        /*
          Continue adding additional lines onto the title until
          we find a blank line or a line starting with a '.' or a '!'
        */
        status = nxt_rec(ifp,tmp_str,key);
        if(status != E_SUCC)
           break;
        tmp_str[strlen(tmp_str)-1] = '\0';

        while((strcmp(tmp_str, "") != 0) && (tmp_str[0] != '.') &&
           (tmp_str[0] != '!'))
        {
           /* Add a space between the two strings. Don't need to worry about
              if there is already a space there since HTML will only
              print one space. */
           strcat(string, " ");

           /* Add the new line to the title string */
           strcat(string, tmp_str);
           titl_pos = ftell(ifp);
           status = nxt_rec(ifp, tmp_str, key);
           if(status != E_SUCC)
              break;
           tmp_str[strlen(tmp_str)-1] = '\0';
        }
        if(status != E_SUCC)
           break;

        /*
          If the saved postion is valid back the input file
          pointer up to allow processing of the line just
          read in.
        */
        if (titl_pos < 0)
           break;
        else
           fseek(ifp,titl_pos,0);
        titl_pos = 0;

        /*
          Start HTML by writing the title to the output file
        */
        fprintf(ofp,"<TITLE>%s</TITLE>\n\n",string);

        /*
          Write the header
        */
        fputs("<H1>User's Guide</H1>\n\n",ofp);

        /*
          Upper case the key
        */
        len = (long)strlen(func);
        c_low2up(func,&len);


        /*
          Write the function name
        */
        fprintf(ofp,"<H3>%s</H3>\n\n",func);

        /* Write the title again.  This puts it within the document. */
        fputs("<BLOCKQUOTE>\n",ofp);
        fprintf(ofp,"<TITLE2>%s</TITLE2>\n\n",string);
        fputs("</BLOCKQUOTE>\n\n",ofp);


        /*
          Set the processing flags
        */
        put_txt = FALSE;
        }
    else if (strncmp(key,".HELP",5) == 0)
        {
        /*
          Write the function header
        */
        fputs("<H3>Function:</H3>\n\n",ofp);
        fputs("<BLOCKQUOTE>\n",ofp);

        /*
          Set the processing flags
        */
        put_txt = TRUE;
        }
    else if ((strncmp(key,"!.EXAMPLE",9) == 0) ||
             (strncmp(key,"!.NONFATAL",10) == 0) ||
             (strncmp(key,"!.FATAL",7) == 0) ||
             (strncmp(key,"!.DESC",6) == 0) ||
             (strncmp(key,"!.USERNOTE",10) == 0) ||
             (strncmp(key,".LEVEL1",7) == 0))
        {
        /*
          Skip these the first pass and turn off the text output
        */
        put_txt = FALSE;
        }
    else if (strncmp(key,"!.DYNLOCAL",10) == 0)
        {
        /*
          Turn off the indent block from the previous block
        */
        if ((nsubs == 0) && (level2))
            {
            fputs("</DL>\n",ofp);
            fputs("</BLOCKQUOTE>\n\n",ofp);
            }

        /*
          Write the paramter heading
        */
        fputs("<H3>Dynamic Parameters:</H3>\n\n",ofp);

        /*
          Set the block indent when there are no subcommands
        */
        if ((nsubs == 0) && (level2))
            fputs("<BLOCKQUOTE>\n",ofp);

        /*
          Set a definition list
        */
        fputs("<DL>\n",ofp);

        /*
          Set the processing flags
        */
        fparm = TRUE;
        put_txt = TRUE;
        }
    else if (strncmp(key,".LEVEL2",7) == 0)
        {
        /*
          Turn off the indent block from the previous block
        */
        fputs("</BLOCKQUOTE>\n\n",ofp);

        /*
          Write the paramter heading
        */
        fputs("<H3>Parameters:</H3>\n\n",ofp);

        /*
          Set the block indent when there are no subcommands
        */
        if (nsubs == 0)
            fputs("<BLOCKQUOTE>\n",ofp);

        /*
          Set a definition list
        */
        fputs("<DL>\n",ofp);

        /*
          Set the processing flags
        */
        fparm = TRUE;
        level2  = TRUE;
        put_txt = TRUE;
        }
    else if (((strncmp(key,"!.LIST",6) == 0) ||
              (strncmp(key,"!.KEEPLIST",10) == 0) ||
              (strncmp(key,"!.KEEPTEXT",10) == 0)) && (put_txt))
        {
        status = wrt_list(ifp,ofp,string,key);
        if (status != E_SUCC)
            break;
        }
    else if ((strncmp(key,"!.FIG",5) == 0) && (put_txt))
        {
        status = wrt_fig(ifp,ofp,string,key);
        if (status != E_SUCC)
            break;
        }
    else if ((strncmp(key,".SUBCMD",7) == 0) && (level2))
        {
        /*
          Set the processing flags
        */
        put_txt = TRUE;

        /*
          Write the subcommand name to the HTML file
        */
        sscanf(string,"%*s %s",sub_name);
        fprintf(ofp,"<DT><B>Subcommand -%s:</B>\n<DD>",sub_name);

        /*
          Loop through the subcommands to verify the current subcommand
          is in the subcommand list
        */
        for (i = 0; i < nsubs; i++)
            {
            /*
              Check that the subcommand is in the valid subcommand list
            */
            if (strcmp(sub_name,subcmd[i].name) == 0)
                {
                /*
                  Found a valid subcommand, write the subcommand info
                  to the HTML file
                */
                while (status == E_SUCC)
                    {
                    /*
                      Save the current position in the input file.
                      The file is read from the current position to the
                      end of file to resolve the subcommand parameters.
                    */
                    cur_pos = ftell(ifp);

                    /*
                      Read a line from the input file
                    */
                    status = nxt_rec(ifp,string,key);
                    if (status != E_SUCC)
                        break;

                    if ((strncmp(key,"!.FIG",5) == 0) && (put_txt))
                        {
                        status = wrt_fig(ifp,ofp,string,key);
                        if (status != E_SUCC)
                            break;
                        else
                            continue;
                        }
                    else if (((strncmp(key,"!.LIST",6) == 0) ||
                              (strncmp(key,"!.KEEPLIST",10) == 0) ||
                              (strncmp(key,"!.KEEPTEXT",10) == 0)) && (put_txt))
                        {
                        status = wrt_list(ifp,ofp,string,key);
                        if (status != E_SUCC)
                            break;
                        else
                            continue;
                        }

                    /*
                      When the next subcommand or a parameter variable is
                      found end this loop
                    */
                    if ((strncmp(key,".SUBCMD",7) == 0) ||
                        (strncmp(key,".VAR",4) == 0) ||
                        (strncmp(key,".END",4) == 0))
                        break;

                    /*
                      Write the subcommand information to the HTML file
                    */
                    wrt_rec(ofp,string,key);
                    }

                /*
                  Write the subcommand parameters to the HTML file
                */
                if (subcmd[i].nparms != 0)
                    {
                    /*
                      Initial setup of the subcommand block
                    */
                    fputs("<P>\n",ofp);
                    fputs("<BLOCKQUOTE>\n",ofp);
                    fputs("<DL>\n",ofp);

                    /*
                      If the saved postion is valid back the input file
                      pointer up to make sure that all the paramters for
                      a subcommand are picked up.
                    */
                    if (cur_pos < 0)
                        break;
                    else
                        fseek(ifp,cur_pos,0);

                    /*
                      Get the subcommand parameters and write them to
                      the HTML file.  The input file pointer is moved
                      to pick up the subcommand parameters.
                    */
                    pass_sub(ifp,ofp,&subcmd[i]);

                    /*
                      The subcommand parameters are resolved, Reset the
                      input file pointer back for processing of the next
                      subcommand
                    */
                    fseek(ifp,cur_pos,0);

                    /*
                      Processed the subcommand so break out of the loop
                    */
                    break;
                    }
                else
                    {
                    /*
                      The subcommand parameters are resolved, Reset the
                      input file pointer back for processing of the next
                      subcommand
                    */
                    fseek(ifp,cur_pos,0);

                    /*
                      Processed the subcommand so break out of the loop
                    */
                    break;
                    }
                }
            }

        /*
          Reset the put text flag 
        */
        put_txt = FALSE;
        }
    else if ((strncmp(key,".END",4) == 0) ||
             (strncmp(key,"!.FINAL",7) == 0))
        {
        /*
          End of the first pass, turn the blocking off
        */
        if (nsubs == 0)
            {
            fputs("</DL>\n",ofp);
            fputs("</BLOCKQUOTE>\n\n",ofp);
            }
        else
            fputs("</DL>\n\n",ofp);

        /*
          Break out of the while loop because the end of the PDF was found
        */
        break;
        }
    else if ((level2) && (nsubs == 0))
        {
        /*
          Extract the parameter name from the PDF
        */
        if (strncmp(string,".VAR",4) == 0)
            {
            /*
              Get the parameter
            */
            sscanf(string,"%*s %s",parm);

            /*
              Search the saved parameter list for a match to current
              parmeter to determine if it is valid and if there is a default
            */
            for (i = 0; i < subcmd[0].nparms; i++)
                {
                if (strcmp(parm,subcmd[0].parm[i].name) == 0)
                    {
                    /*
                      No new line needed before the first parameter
                    */
                    if (!fparm)
                        fputs("<P>\n",ofp);

                    /*
                      Force the new line before the next parameter
                    */
                    fparm = FALSE;

                    /*
                      Get the default.  Put the default in parens
                    */
                    if (strcmp(subcmd[0].parm[i].deflt," ") == 0)
                        {
                        fprintf(ofp,"<DT><B>%s</B>\n<DD>",
                                subcmd[0].parm[i].name);
                        }
                    else
                        {
                        fprintf(ofp,"<DT><B>%s(%s)</B>\n<DD>",
                               subcmd[0].parm[i].name,subcmd[0].parm[i].deflt);
                        }

                    break;
                    }
                }
            }
        else
            {
            /*
              Write the level 2 text to the HTML file
            */
            wrt_rec(ofp,string,key);
            }
        }
    else if (put_txt)
        {
        /*
          Write the text to the HTML file
        */
        wrt_rec(ofp,string,key);
        }
    }
return status;
}
