#include <stdio.h>
#include <string.h>
#include "pdfhtml.h"
#include "local_protos.h"

void pass_sub
(
FILE    *ifp,                   /* Input file pointer  (to PDF)              */
FILE    *ofp,                   /* Output file pointer (to HTML)             */
struct SUBCMDS  *subcmd         /* Subcommand parameters                     */
)
{
long    put_txt = FALSE,        /* Write text to ouput flag                  */
        status = E_SUCC,        /* Status of the read                        */
        i;                      /* Looping variable                          */

char    *ptr,
        key[KEY_SIZ],           /* First word in each line to key off of     */
        parm[PARM_SIZ],         /* Name of the function                      */
        string[STR_SIZ],        /* String read from the input file           */
        sub_name[PARM_SIZ];     /* Name of the subcommand                    */

/*
  Check the number of parameters in the subcommand
*/
if (subcmd->nparms == 0)
    {
    /*
      No parameters for this subcommand, complete the block and return
    */
    fputs("</DL>\n\n",ofp);
    return;
    }

/*
  Process the input file for this subcommand
*/
while (status == E_SUCC)
    {
    /*
      Read a line from the input file
    */
    status = nxt_rec(ifp,string,key);
    if (status != E_SUCC)
        break;

    /*
      Check the key
    */
    if (strncmp(key,".SUBCMD",7) == 0)
        {
        /*
          Ignore this key
        */
        continue;
        }
    else if (strncmp(key,".VAR",4) == 0)
        {
        /*
          Set the write text flag
        */
        put_txt = FALSE;

        /*
          Get the parameter
        */
        if ((ptr = strchr(string,'-')))
            {
            /*
              This parameter is specific to a subcommand
              Extract the parameter
            */
            sscanf(string,"%*s %[^-]s",parm);
            ptr++;

            /*
              Extract the subcommand
            */
            sscanf(ptr,"%s",sub_name);

            /*
              Ignore the parameter if the subcommand is not
              the same as the current subcommand
            */
            if (strcmp(sub_name,subcmd->name) != 0)
                continue;
            }
        else
            sscanf(string,"%*s %s",parm);


        /*
          Search the saved subcommand parameter list for a match to the
          current parmeter to determine if it is valid and if there it
          has a default
        */
        for (i = 0; i < subcmd->nparms; i++)
            {
            /*
              Search for the current parameter in the subcommand parameter list
            */
            if (strcmp(parm,subcmd->parm[i].name) == 0)
                {
                /*
                  Put new lines after the last parameter
                */
                fputs("<P>\n",ofp);

                /*
                  Get the default.  Put the default in parens
                */
                if (strcmp(subcmd->parm[i].deflt," ") == 0)
                    fprintf(ofp,"<DT><B>%s</B>\n<DD>",subcmd->parm[i].name);
                else
                    fprintf(ofp,"<DT><B>%s(%s)</B>\n<DD>",
                            subcmd->parm[i].name,subcmd->parm[i].deflt);

                /*
                  Set the flag to write text
                */
                put_txt = TRUE;

                /*
                  Found the valid parameter, break out of the loop
                */
                break;
                }
            }
        }
    else if (((strncmp(key,"!.LIST",6) == 0) ||
             (strncmp(key,"!.KEEPLIST",10) == 0) ||
             (strncmp(key,"!.KEEPTEXT",10) == 0)) && (put_txt))
        {
        status = wrt_list(ifp,ofp,string,key);
        if (status != E_SUCC)
            break;
        }
    else if ((strncmp(key,"!.FIG",5) == 0) && (put_txt))
        {
        status = wrt_fig(ifp,ofp,string,key);
        if (status != E_SUCC)
            break;
        }
    else if ((strncmp(key,".END",4) == 0) ||
             (strncmp(key,"!.FINAL",7) == 0))
        {
        /*
          End the block
        */
        fputs("</DL>\n",ofp);
        fputs("</BLOCKQUOTE>\n\n",ofp);

        /*
          End of the subcommand pass, break out of the while loop
        */
        break;
        }
    else if (put_txt)
        {
        /*
          Write the text to the HTML file
        */
        wrt_rec(ofp,string,key);
        }
    }

return;
}
