#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "las.h"
#include "util.h"
#include "pdfhtml.h"
#include "local_protos.h"

void pass_two
(
    FILE    *ifp,               /* Input file pointer  (to PDF)              */
    FILE    *ofp                /* Output file pointer (to HTML)             */
)
{
long    put_txt = FALSE,        /* Flag to write text to output              */
        listnumflg,             /* if false, this line that is just a
                                   number and no spaces is not part of a
                                   list, so just write to HTML file          */
        num_item = NONE,        /* Number item type format                   */
        end_line = FALSE,       /* End of number item string flag            */
        examp = FALSE,          /* Example flag                              */
        status = E_SUCC,        /* Status of read                            */
        len,                    /* Length of a string                        */
        i;                      /* Looping variable                          */

char    string[4*STR_SIZ],      /* String read from the input file           */
        temp[STR_SIZ],          /* Temp string read from the input file      */
        key[KEY_SIZ],           /* First word in each line to key off of     */
        *ptr;                   /* Search pointer into the string            */

/*
  Process the input file
*/
while (status == E_SUCC)
    {
    /*
      Read a line from the input file (PDF)
    */
    status = nxt_rec(ifp,string,key);
    if (status != E_SUCC)
        break;

    /* (Re)Set the list number flag (the number at the beginning of the line is
       part of the numbering of a list) */
    listnumflg = TRUE;

    /*
      Check the keys for processing
    */
    if (strncmp(key,"!.EXAMPLE",9) == 0)
        {
        /*
          Read the next line for the section title
        */
        status = nxt_rec(ifp,string,key);
        if (status != E_SUCC)
            break;

        /*
          Write the section title to the HTML
        */
        string[strlen(string)-1] = '\0';
        fprintf(ofp,"<H3>%s</H3>\n\n",string);
        fputs("<OL>\n",ofp);

        /*
          Set the processing flags
        */
        examp = TRUE;
        num_item = BOLD;
        put_txt  = TRUE;
        }
    else if (strncmp(key,"!.DESC",6) == 0)
        {
        /*
          End the previous ordered list
        */
        fputs("</OL>\n\n",ofp);

        /*
          Read the next line for the section title
        */
        status = nxt_rec(ifp,string,key);
        if (status != E_SUCC)
            break;

        /*
          Write the section title to the HTML
        */
        string[strlen(string)-1] = '\0';
        fprintf(ofp,"<H3>%s</H3>\n\n",string);
        fputs("<BLOCKQUOTE>\n",ofp);

        /*
          Set the processing flags
        */
        examp = FALSE;
        num_item = NONE;
        put_txt = TRUE;
        }
    else if (strncmp(key,"!.NONFATAL",10) == 0)
        {
        /*
          End the previous block
        */
        fputs("</BLOCKQUOTE>\n\n",ofp);

        /*
          Read the next line for the section title
        */
        status = nxt_rec(ifp,string,key);
        if (status != E_SUCC)
            break;

        /*
          Write the section title to the HTML
        */
        string[strlen(string)-1] = '\0';
        fprintf(ofp,"<H3>%s</H3>\n\n",string);
        fputs("<OL>\n",ofp);

        /*
          Set the processing flags
        */
        examp = FALSE;
        num_item = BOLD;
        put_txt  = TRUE;
        }
    else if (strncmp(key,"!.FATAL",7) == 0)
        {
        /*
          End the previous ordered list
        */
        fputs("</OL>\n\n",ofp);

        /*
          Read the next line for the section title
        */
        status = nxt_rec(ifp,string,key);
        if (status != E_SUCC)
            break;

        /*
          Write the section title to the HTML
        */
        string[strlen(string)-1] = '\0';
        fprintf(ofp,"<H3>%s</H3>\n\n",string);
        fputs("<OL>\n",ofp);

        /*
          Set the processing flags
        */
        examp = FALSE;
        num_item = BOLD;
        put_txt  = TRUE;
        }
    else if (strncmp(key,"!.USERNOTE",10) == 0)
        {
        /*
          End the previous ordered list
        */
        fputs("</OL>\n\n",ofp);

        /*
          Read the next line for the section title
        */
        status = nxt_rec(ifp,string,key);
        if (status != E_SUCC)
            break;

        /*
          Write the section title to the HTML
        */
        string[strlen(string)-1] = '\0';
        fprintf(ofp,"<H3>%s</H3>\n\n",string);
        fputs("<OL>\n",ofp);

        /*
          Set the processing flags
        */
        examp = FALSE;
        num_item = REGULAR;
        put_txt  = TRUE;
        }
    else if (((strncmp(key,"!.LIST",6) == 0) ||
              (strncmp(key,"!.KEEPLIST",10) == 0) ||
              (strncmp(key,"!.KEEPTEXT",10) == 0)) && (put_txt))
        {
        status = wrt_list(ifp,ofp,string,key);
        if (status != E_SUCC)
            break;
        }
    else if ((strncmp(key,"!.FIG",5) == 0) && (put_txt))
        {
        status = wrt_fig(ifp,ofp,string,key);
        if (status != E_SUCC)
            break;
        }
    else if (strncmp(key,".LEVEL1",7) == 0)
        {
        /*
          End the previous ordered list
        */
        fputs("</OL>\n",ofp);

        /*
          Set the processing flags
        */
        examp = FALSE;
        num_item = NONE;
        put_txt  = FALSE;

        /*
          Done with the first pass break out of the while loop
        */
        break;
        }
    else if (num_item)
        {
        /*
          Search for the numbered line
        */
        if (strchr("1234567890",string[0]) != 0)
            {
            /*
              Found a numbered line, put a space after the previous line
            fputs("<P>\n",ofp);
            */

            /*
              Make sure the full number line is there
            */
            end_line = FALSE;
            while ((!end_line) && (num_item == BOLD))
                {
                status = nxt_rec(ifp,temp,key);
                if (status != E_SUCC)
                    break;

                /*
                  Add the new string to the current string
                */
                strcat(string,temp);

                /*
                  Check to see if we are at the end of the line.  Indicated
                  by white space before the next line
                */
                end_line = TRUE;
                for (i = 0; i < strlen(temp); i++)
                    {
                    if (isspace((int)temp[i]) == 0)
                        {
                        end_line = FALSE;
                        break;
                        }
                    }
                }

            /* If the string contains no spaces, it was just a line that had
               a number only on it.  Just bypass the num_item check and send
               it to the HTML file */
            if(strchr(string,' ') == NULL)
                listnumflg = FALSE;

            /*
              Determine the way to process the number item
            */
            if ((num_item == BOLD) && listnumflg)
                {
                /*
                  Lower case the string after the "LAS>" for examples
                */
                if (examp)
                    {
                    /*
                      Find the first greater than sign
                    */
                    if ((ptr = strchr(string,'>')) == NULL)
                        {
                        printf("No \">\" in the example string\n");
                        exit(-1);
                        }
                    ptr++;

                    /*
                      Lower case everything after the ">"
                    */
                    len = (long)strlen(ptr);
                    c_up2low(ptr,&len);
                    }

                /*
                  Write the bold list item to the HTML
                */
                string[strlen(string)-1] = '\0';
                special(string,temp);
                fprintf(ofp,"<B><LI>%s",strchr(temp,' '));
                }
            else if ((num_item == REGULAR) && listnumflg)
                {
                /*
                  Write the normal list item to the HTML
                */
                special(string,temp);
                fprintf(ofp,"<LI>%s",strchr(temp,' '));
                }
            else if (!listnumflg)
                {
                /*
                  Found a number in the string, but it shouldn't be
                  treated as a list item
                */
                wrt_rec(ofp,string,key);
                }

            if ((num_item == BOLD) && listnumflg)
                fputs("</B><P>\n",ofp);

            put_txt = TRUE;
            }
        else if (put_txt)
            {
            /*
              Write the text to the HTML file
            */
            wrt_rec(ofp,string,key);
            }
        }
    else if (put_txt)
        {
        /*
          Write the text to the HTML file
        */
        wrt_rec(ofp,string,key);
        }
    }
}
