#!/bin/sh
#check.sh by David Zokaites September 1996 
#check supports for compatability with my prototypes based on html documentation





#a shell function to include a prototype for this function in the source file 
   #prototoype must be included just after the last "#include" statement 
add_ptype ()
{

#find line number of last "#include" 
   lastinc=`
   nawk '
      BEGIN       { lastinc = 0   }
      /^#include/ { lastinc = NR  } 
      END         { print lastinc }
   ' $file.c `

#copy the source file up to the last "#include"
   head -$lastinc $file.c > $file.new.c 

#find prototype for this source file
   nawk -v refdir=$refdir -v ssystem=$ssystem -v sdir=$sdir -v file=$file '
      BEGIN {
	    #read prototype file to find prototype
	       pfile = "../../"  ssystem  "p.h"
	       while (getline < pfile)
	       {
		  if ( index($0,file) && index($0,"(") && check_match() ) 
		  {
		     begin = 1       #set flag, beginning of prototype was found
		     print "/* FUNCTION PROTOTYPE */"  #print a C comment 
		  }

		  if (begin)
		  {
		     print                   #print prototype text 
		     if ( index($0,");") )   #marks the end of the prototype
		     {
			print ""             #a blank line 
			break
		     }
		  }
	       }

	    #error check:  no prototype found
	       if (!begin)
		 system ("echo " sdir "/" file ".c >> " refdir "/check.noptype")
	    }

      #function to see if function name was found 
	 function check_match() {
            #eliminate "(" character
               checkstr = $0 
               sub ("[(]", "", checkstr)
	    #check for match
	       for (field=1;  field<NF;  field++)
		  if ($field == file)
		     return(1)
	    #no match
	       return (0)
	 }

   ' $file.c > $file.ptype

   #save prototype to check.done, and copy it to source file 
      cat $file.ptype                |\
	grep -v 'FUNCTION PROTOTYPE' |\
	grep -v '^$'  >> $refdir/check.done 
      cat $file.ptype >> $file.new.c 
      
   #add the rest of the source file
      lastinc=`expr $lastinc + 1`
      tail +$lastinc $file.c >> $file.new.c 
}







#effective start of script
   refdir=`pwd`              #save current directory location for output files
   #initialize output files 
   echo Prototyped these functions             > $refdir/check.done   
   echo Did not find prototype for these files > $refdir/check.noptype 
   echo                                        > $refdir/check.log 
   echo                                        > $refdir/check.nocheck

#loop for all systems, libraries, and files 
   for ssystem in adaps las
   do
      cd $ssystem 

      for sdir in *      #sdir = source directory 
      do
	 #user message
	    echo Checking $ssystem/$sdir 
	    (echo ; echo) >> $refdir/check.log 

	 cd $sdir
	 for file in `ls -1 *.c | grep -v new.c` 
	 do
	    #remove .c extension from file variable
	       file=`echo $file | sed -e 's/.c$//' `
	    #user message
	       echo Checking $ssystem/$sdir/$file.new.c >> $refdir/check.log 
	    #add a prototype to the file 
	       add_ptype 
	    #compile file with prototype checking 
	       cc -prototypes -c $file.new.c -o $file.o \
		-I$ADAPSINCLUDE -I$LASINCLUDE -I$TINC   \
		2>> $refdir/check.log 
	    #delete object file
	       rm $file.o 2> /dev/null 
	    #retain intermediate files for manual prototype checking 
	       #rm $file.new.c     #source with a prototype
	       #rm $file.ptype     #prototype
	 done 
	 cd ..
      done
      cd ..
   done 

#determine which prototypes were not checked for accuracy
   #comm requires files to be sorted.  grep '(' finds function calls 
   cd $refdir 
   grep -h '(' adapsp.h lasp.h | sort > allp.h 
   grep    '(' check.done      | sort > check.done2
   comm -23 allp.h check.done2 > check.nocheck 
   rm allp.h check.done2

#completion message
   echo 
   echo check.sh completed and created output:
   echo '  check.done:     intermediate file of prototypes which were checked'
   echo '  check.nocheck:  prototypes that were not checked'
   echo '  check.noptype:  source file that were not prototyped'
   echo '  check.log:      compilation/processing log'
   echo '  Be sure to check this log for errors!' 
   echo execute the following commands to delete intermediates
   echo '  find . \( -name \*.ptype -o -name \*.new.c \) -print -exec rm {} \;'
   echo '  rm -r adaps las'

   exit 0 
