
/*******************************************************************************
NAME:    makep.C, a c++ program  
PURPOSE: convert a LAS document html file to a C++ function prototype
 
   A note about errors:  
   The input html files have generally the same format.  However, there are
   subtle differences that cause errors in this program.  The best thing to do
   is to correct the input html files.  This program has several error traps
   that were never executed.  The error traps that were executed do not cause
   the program to exit.  A printout of the erronenous prototype is helpful for
   correcting errors in the input file. 

   A note about the input html files:
   If I was rewriting the documentation for the ADAPS and LAS supports, the
   function declaration part of the documentation would be a function
   prototype.  A much simpler version of this program would then be necessary.

  Date      Programmer                        Changes
--------  --------------  -----------------------------------------------------
Aug 1994  David Zokaites  Original development
Jan 1995  David Zokaites  No longer necessary to print "c_" in front of the 
                          function name in the output prototypes, rewrote 
			  makep.sed, put (char *) in front of malloc. 
Jun 1996  David Zokaites  Rewrote application, converted it to c++, added blank
			  lines and function headers in the LAS style, used
			  c++ keyword, "const", in output function prototypes,
			  deleted makep.sed and corrected input html files, 
			  allowed for ADAPS prototypes in addition to LAS 
Dec 1996  David Zokaites  Renamed file from makep.c to makep.C, placed under
			  CVS 
*******************************************************************************/




// include files 
   #include <ctype.h>    // for call to isspace()  
   #include <iostream.h> // for output to the terminal 
   #include <fstream.h>  // for file io
   #include <stdio.h>    // for call to sprintf() 
   #include <stdlib.h>   // for call to exit() 
   #include <string.h>   // for call to strstr(), strcpy(), ...  




// structure to describe a function prototype
   struct PTYPE 
   {
      enum { ARGLEN  = 50 };       // max n chars in each function argument 
				   // type or name 
      enum { MAXARGS = 30 };       // max n arguments in a function  
      char fargs[MAXARGS][ARGLEN], // name and type of function arguments
	   fname[ARGLEN],          // name of function being prototyped  
	   ftype[ARGLEN];          // type of function being prototyped  
      int  argcount;               // number of function arguments 
   };




// function prototypes
   int  main (const int argc, const char *argv[]); 
   void get_name (const char *linein, char *fname, char *ftype, int *voidarg, 
	int *type_found); 
   void fix_codes (char *ioline); 
   void get_parm (const char *infile, const char *linein, int *argcount, 
	char fargs[PTYPE::MAXARGS][PTYPE::ARGLEN]); 
   int  check_const (const char *linein, char *farg);
   char *get_arg (char * bigarg); 
   void delimiters (const int argcount, 
	char fargs[PTYPE::MAXARGS][PTYPE::ARGLEN]);
   void write_string (char *instring, const int type_and_name, 
	const char *endstring, const int lastline); 




// symbolic constants  
   #define EXITERR    1           // exit value for errors  
   #define LINEIN  1500           // max n chars on an input line, html 
                                  // documents have big lines   
 
   #define FTEXT   "FUNCTION"     // marks beginning of a function declaration 
				  // in a html file 
   #define RETTEXT "RETURN VALUE" // marks section in a html file which
                                  // describes function return value  
 
   #define TRUE       1           // a condition is true  
   #define FALSE      0           // a condition is false  
 
   #define LEADINGC   3           // n leading characters to indent output 
                                  // prototype
   #define LINEOUT   80           // max n chars in an output line excluding 
                                  // the newline 










/******************************************************************************
NAME:     main()
PURPOSE:  program main 
******************************************************************************/
int main (
   const int   argc,   // input:  number of input arguments  
   const char *argv[]) // input:  value of input args  
{

// declare variables  
   struct PTYPE ptype;       // data for the prototype 
    ptype.argcount = 0;      // initialize count of arguments 
   char linein[LINEIN],      // a line read from the input file  
	lineout[LINEOUT];    // a line of text to output  
   int argnum,               // loop index, argument number  
       lastline,             // flag, was last line of parm declaration found 
       lspace,               // loop index, leading spaces  
       ranout = FALSE,       // flag, did main() "run out of data" 
       type_found,           // flag, was function type found  
       voidarg;              // flag, does this function have (void) argument 

// check input parameters 
   if (argc !=2)
   {
      cerr<< "makep usage:   makep   input file name  (output is stdout) \n";
      cerr<< "makep converts LAS document html file to a function prototype\n";
      exit (EXITERR); 
   }
   const char *infile = argv[1];   // input file name 

// open input file  
   ifstream filep (infile);
   if (!filep)
   {
      cerr << "makep:  unable to open input file " << infile << endl; 
      exit (EXITERR); 
   }

// read file up to where the the function name appers.  function name appears 
// on the same line as does FTEXT 
   while  ( filep.getline(linein,LINEIN) && strstr(linein,FTEXT) == NULL)
      ; 
   // error check:  FTEXT not found 
   if (filep.eof() )
   {
      cerr << "makep:  string " << FTEXT << " was not found in file ";
      cerr << infile << endl;
      exit (EXITERR);
   } 

// get function name and type
   get_name (linein, ptype.fname, ptype.ftype, &voidarg, &type_found); 

// read file to the end of the block that includes 
// FUNCTION funct_name (arg1, arg2, arg3, arg4, argn)  
   while (strchr(linein,')')==NULL && filep.getline(linein,LINEIN) )
      ; 
   // error check:  ')' was not found 
   if (filep.eof() )
   {
      cerr << "makep:  ')' was not found in file " << infile << endl;
      exit (EXITERR);
   } 

/* get function parameters and their type from the section that looks like:
   <PRE>
   double   *matrix;
   long      flag;</PRE> */ 
// process file up to and including the line with "</PRE>" 
   if (!voidarg)
      for (lastline=FALSE;  !lastline && filep.getline(linein,LINEIN) ; )
      {
	 // "</PRE>" is on the line of the last parm declaration  
	    if (strstr (linein, "</PRE>") != NULL)
	       lastline=TRUE;
	 // eliminate control codes  
	    fix_codes (linein); 
	 // get function argument  
	    get_parm (infile, linein, &ptype.argcount, ptype.fargs);
      }

/* Determine which function parms are "const".  
   Determination of "const" is found from the PARAMETER description section.  

   For LAS supports, part of the html file looks like 
      <H2>insys (input, long)</H2>

   For ADAPS, it would be 
      <H2>insys (long, input)</H2>
*/ 
   for (argnum=0;  
   !ranout && argnum<ptype.argcount && filep.getline(linein,LINEIN) ; )
   {
      if (check_const (linein, ptype.fargs[argnum]))
	 argnum++;
      if (strstr (linein, RETTEXT) != NULL)
	 ranout = TRUE; 
   }
   if (ranout || argnum != ptype.argcount)
   {
      cerr << "makep:  ran out of data in main() for file " << infile << endl; 
      cerr << "makep:  make sure function parameters are listed ";
      cerr <<          "and described in the same order" << endl; 
   }

// read file up to the section with RETTEXT
   while (filep.getline(linein,LINEIN) )
      if (strstr (linein, RETTEXT) != NULL)
	 break;  
   // error check:  RETTEXT was not found
   if (!ranout && filep.eof() ) 
      cerr << "makep:  did not find string " << RETTEXT << " in file " 
      << infile << endl; 

// get function type 
   while  (!type_found  &&  filep.getline(linein,LINEIN) )
   {
      // look for "has no return value"  
	 if (strstr (linein, "has no return value") != NULL)
	 {
	    strcpy (ptype.ftype, "void");
	    type_found = TRUE; 
	 }
      // look for "returns"
	 if (strstr (linein, "returns") != NULL)
	 {
	    strcpy (ptype.ftype, "int");
	    type_found = TRUE; 
	 }
   }
   // error check:  type not found
   if (!type_found)
   {
      cerr << "makep:  did not identify function type in file " << infile;
      cerr << endl; 
   }

// file has been parsed, print out function prototype  
   // error check  
      if (voidarg && ptype.argcount > 0)
      {
	 cerr << "makep:  error in main, voidarg && ptype.argcount>0 \n";
	 exit (EXITERR); 
      }
   // add delimters to function arguments  
      delimiters (ptype.argcount, ptype.fargs);
   // print function type and name  
      for (lspace=0;  lspace<LEADINGC;  lspace++)  // write leading spaces  
	 lineout[lspace] = ' ';
      sprintf (lineout+LEADINGC, 
      "%s %s (%s", ptype.ftype, ptype.fname, voidarg ? "void" : ""  ); 
      write_string (lineout, 0, "", ptype.argcount==0 ? TRUE : FALSE); 
   // print function arguments  
      for (argnum=0; argnum<ptype.argcount; argnum++)
	 write_string (lineout, 
	 strlen (ptype.fname) + strlen(ptype.ftype), 
	 ptype.fargs[argnum],  
	 argnum+1 < ptype.argcount ? FALSE : TRUE); 

return (0); 
}








/******************************************************************************
NAME:     get_name()
PURPOSE:  get name (and type) of function being prototyped  
******************************************************************************/
void get_name (
   const char *linein0,// input:   function name is in this line of text 
   char *fname,        // output:  function name  
   char *ftype,        // output:  function type (may be a garbage string)  
   int *voidarg,       // output:  flag, does this function have void argument 
   int *type_found)    // output:  flag, was function type found  
{
// declare variables  
   char *copyend,        // copy linein up to here  
	*end1,           // a potential value for copyend  
	*end2,           // a potential value for copyend  
        *floc,           // location of FTEXT in linein  
	linein[LINEIN],  // working copy of linein0
	*linep;          // points to linein

// make working copy of linein0
   strcpy (linein, linein0);

// determine if this function has void argument by finding "()"  
   if ( strstr(linein,"()") != NULL)
      *voidarg = TRUE;   
   else 
      *voidarg = FALSE;

// eliminate last '(' character  
   if ((copyend = strrchr (linein, '(')) == NULL)
   {
      cerr << "makep:  error in get_name(), did not find '(' in linein \n";
      exit (EXITERR); 
   }
   *copyend = '\0'; 

// replace FTEXT with spaces
   if ((floc = strstr (linein, FTEXT)) == NULL)
   {
      cerr << "makep:  logic error in get_name() \n";
      exit (EXITERR); 
   }
   copyend = floc + strlen(FTEXT); 
   for (;  floc != copyend;  floc++)
      *floc = ' ';
   fix_codes (linein);   // eliminate extra spaces 

// copy function type if it is given  
   // copy up to '*' or ' ', whichever is later  
   end1 = strrchr (linein, '*'); 
   end2 = strrchr (linein, ' ');
   copyend = end1 > end2 ? end1 : end2; 
   if (copyend != NULL)
   {
      for (linep=linein;  linep!=copyend;  linep++, ftype++)
	 *ftype = *linep;
      if (*linep=='*')  // copy a '*'  
      {
	 *ftype = *linep;
	 ftype++;
      }
      *ftype = '\0'; 
      *type_found = TRUE; 
   }
   else 
      *type_found = FALSE; 

// copy function name  
   if (*type_found && (isspace (*linep) || *linep == '*'))
      strcpy (fname, ++linep); 
   else 
      strcpy (fname, linein); 
}








/******************************************************************************
NAME:     fix_codes()
PURPOSE:  eliminate control codes, extra white space, ';' and ',' characters
	  from input string 
******************************************************************************/
void fix_codes (
   char *ioline)   // input as line to correct, output as corrected line  
{
// declare variables  
   char *fromc,           // a character to copy from here 
        *ioline_ref,      // a reference copy of ioline 
	*looper,          // user to loop through an array 
	*toc,             // a character to copy to here 
        toreturn[LINEIN]; // text to be returned via ioline 
   int ccopied;           // count of characters copied 

// initialize variables  
   ioline_ref = ioline; 

// replace  ; , \t  chars with white space  
   for (looper=ioline;  *looper!='\0';  looper++)
      if (*looper == ';' || *looper == ',' || *looper == '\t')
	 *looper = ' ';  

// replace control codes with spaces 
   for (looper=ioline;  *looper!='\0';  looper++)
   {
      if (*looper=='<')       // start of a control code  
      {
	 for ( ;  *looper!='>'  &&  *looper!='\0'  ;  looper++)
	    *looper = ' ';
	 if (*looper == '>')
	    *looper = ' ';       // replace '>' with a space 
      }
   }

// copy string from ioline to toreturn while eliminating extra white space
   // eliminate leading white space  
      for (ccopied=0, fromc=ioline;  isspace(*fromc);  fromc++)
	 ; 
   // copy
      for (toc = toreturn;  *fromc!='\0';  ccopied++)
      {
	 // copy a good char
	 *toc++ = *fromc++;         
	 // skip multiple spaces
	 while ( isspace (*fromc)  &&  isspace(*(fromc+1) ) ) 
	    fromc++;
      }

// add trailing null and make sure there is no trailing space 
   *toc = '\0';
   if (ccopied)
      toc--;
   if (isspace(*toc) )
      *toc = '\0';

// copy final fixed string to the original pointer ioline and finish  
   strcpy (ioline_ref, toreturn);    
}








/******************************************************************************
NAME:     get_parm()
PURPOSE:  get function parameter.  
          If *linein has only one word, it will not include the type of the 
	  function parm so this routine will get arg type from previous line of 
	  fargs[].  Argument type may include more than one word of text  
******************************************************************************/
void get_parm (
   const char *infile, // in:   html file being read, for error messages 
   const char *linein, // in:   a line of text with a function arg
   int *argcount,      // I/O:  n arguments to function, gets incremented by 1 
   char fargs[PTYPE::MAXARGS][PTYPE::ARGLEN]) 
		       // I/O:  all the arguments to the function, this array
		       //	is returned with one more entry.  
{
// declare variables  
   char *linein2,   // a copy of linein  
	*lastword,  // start of last word in linein  
	*lineout;   // points to the output line of text  

// if line is null, there is no data to process 
   if (!strlen (linein))
      return;

// if there is a ')' present without a '(', this line is part of the 
// function description 
   if (strchr (linein, ')') != NULL &&
       strchr (linein, '(') == NULL  ) 
      return;

// error check:  too many arguments 
   if (*argcount +1 >= PTYPE::MAXARGS)
   {
      cerr << "makep:  get_parm() has more than " << PTYPE::MAXARGS;
      cerr << "arguments.\n";
      exit (EXITERR); 
   }

// error check:  string is too long  
   if (strlen (linein) > PTYPE::ARGLEN)
   {
      cerr << "makep:  function arg length was too long for arg " << linein;
      cerr << endl; 
      cerr << "makep:  problem occurred for file " << infile << endl;
      exit (EXITERR); 
   }

// optionally copy text and return  
   if (strchr (linein, ' ') != NULL)   // parm type is included in linein  
   {
      strcpy (fargs[(*argcount)++], linein);
      return;
   }

// copy arg type from previous arg parameter  
   // get location of last space char  
      if ((lastword = strrchr (fargs[*argcount -1], ' ')) == NULL)
      {
	 cerr << "makep:  parm type was not found in get_parm() \n";
	 return;
      }
   // error check:  string too long  
      if (lastword - fargs[*argcount -1] + strlen (linein) +1 > PTYPE::ARGLEN)
      {
	 cerr << "makep:  error in get_parm:  string too long.\n";
	 exit (EXITERR); 
      }
   // copy string up to last space char  
      for (linein2= fargs[(*argcount)-1],  
	   lineout= fargs[(*argcount)++];    
	   linein2!=lastword;  linein2++, lineout++)
	 *lineout = *linein2; 
   // copy the space  
      *lineout = ' '; 
      lineout++; 
   // terminate string  
      *lineout = '\0'; 
// copy arg name  
   strcat (fargs[*argcount -1], linein); 
}








/******************************************************************************
NAME:     check_const()
PURPOSE:  modify farg which checking if a function argument is "const";
          return TRUE if linein includes farg  
******************************************************************************/
int check_const (
   const char *linein,   // input:  line of text to check  
	 char *farg)     // input:  function argument to check for const 
                         // output:  argument with "const" prepended  
{
// declare variables  
   char keepstring[PTYPE::ARGLEN];  // a reference string to keep  

// look for function argument  
   if (strncmp (linein, "<H2>", strlen("<H2>") ) != 0)
      return (FALSE);
   if (strstr (linein, get_arg(farg)) == NULL)
      return (FALSE); 

// check for argument being const 
   if (strstr (linein, "(input,") == NULL && 
       strstr (linein,  "input)") == NULL  )  // arg is not const  
      return (TRUE); 

// argument is const, prepend const to function argument  
   strcpy (keepstring, farg);
   sprintf (farg, "const %s", keepstring);
   return (TRUE);
}








/******************************************************************************
NAME:     get_arg()
PURPOSE:  return an argument name without any type. 
	  return value must be discarded before function is called again 
******************************************************************************/
char * get_arg (
   char * bigarg)  // const input:  argument name with type  
{
// declare variables  
   static char rstring[PTYPE::ARGLEN]; // return string  
   char *copyto,                       // character being copied  
	*cspot,                        // character spot being looked at  
	*lastchar,                     // last useful character  
	 workstring[PTYPE::ARGLEN];    // working copy of bigarg  

// eliminate any trailing square brackets  
   strcpy (workstring, bigarg);
   for (cspot=workstring;   *cspot!='\0';   cspot++)
      if (*cspot=='['  ||  *cspot=='[')
      {
	 *cspot = '\0';
	 break;
      }

// replace special characters with spaces  
   for (cspot=workstring;   *cspot!='\0';   cspot++)
      if (*cspot=='*'  ||  *cspot=='(' || *cspot==')')
	 *cspot = ' ';

// delete trailing spaces  
   for (cspot=workstring+strlen(workstring)-1;   *cspot==' ';   cspot--)
      *cspot='\0';

// find the last space character  
   lastchar=workstring; 
   for (cspot=workstring;  *cspot!='\0';  cspot++)
      if (*cspot==' ')
	 lastchar=cspot +1;  // +1 skips the space  

// copy selected string  
   for (cspot=lastchar, copyto=rstring;  *cspot!='\0';  cspot++, copyto++)
      *copyto = *cspot;
   *copyto = '\0';

// return  
   return (rstring);
}








/******************************************************************************
NAME:     delimiters()
PURPOSE:  add commas and spaces to function arguments to delimit them in 
          the function prototype  
******************************************************************************/
void delimiters (
   const int argcount,  // input:  n arguments to function  
   char fargs[PTYPE::MAXARGS][PTYPE::ARGLEN]) 
			// input:  the arguments to the function, 
			// output:  arguments with appended delimiters
{
// add delimiters  
   for (int argnum=0;  argnum<argcount-1;  argnum++)
      strcat (fargs[argnum], ", ");
}








/******************************************************************************
NAME:     write_string()
PURPOSE:  write input string:  
          append input strings if they will fit in LINEOUT
          else write 1st string and copy 2nd string to 1st string  
******************************************************************************/
void write_string (
   char *instring,         // input:  base string to write, 
		           // output: expanded string to write later  
   const int type_and_name,// input:  strlen(funtion type) + strlen(function
                           //         name).  used for indentation 
   const char *endstring,  // input:  add this to instring if it fits  
   const int lastline)     // input:  flag, is endstring the last line to output
{
// set addsize, size of string to add after endstring  
   // account for ");" after endstring  
   int addsize = lastline  ? 2 : 0;  
   // account for trailing ' ' that might be deleted  
   if ( *(endstring+strlen(endstring)-1 ) == ' ') 
      addsize = -1;

// error check:  is instring too big?  
   if (strlen (instring) > LINEOUT +1)  // +1 for space that gets deleted  
   {
      cerr << "makep:  error in write_string:  instring is too big.\n";
      exit (EXITERR); 
   }

// see if endstring can be added to instring  
   if (strlen (instring) + strlen (endstring) + addsize <= LINEOUT) 
      strcat (instring, endstring); 
   else   // strings could not be combined  
   {
      // eliminate trailing ' ' on instring  
	 if ( *(instring+strlen(instring)-1 ) == ' ')
	      *(instring+strlen(instring)-1 ) = '\0'; 
      // write instring  
	 cout << instring << endl; 
      // print leading spaces to indent function parms from function type  
	 for (int lspace=0;  lspace<LEADINGC + type_and_name +2;  lspace++)  
	    instring[lspace] = ' ';
	 sprintf (instring+lspace, "%s", endstring);   
   }

// if lastline, write instring and end the function  
   if (lastline)
      cout << instring << ");" << endl; 
}




