#!/bin/csh -f

#
# NAME:    makep.csh 
# PURPOSE: script to generate function prototypes from ADAPS and LAS document 
#          html files
#
#   Date      Programmer                        Changes
# --------  --------------  ----------------------------------------------------
# Aug 1994  David Zokaites  original development 
# Jan 1995  David Zokaites  accommodated new las directory tree  
# May 1996  David Zokaites  rewrote script, added error checks, 
#                           enabled adaps documentation
#

#check input arguments
   if ( $#argv != 1 || $1 != "adaps" && $1 != "las") then 
      echo makep.csh:  usage:  adaps or las
      echo makep.csh creates adapsp.h or lasp.h to prototype 
      echo the supporting functions 
      exit 1 
   endif 
   set tocheck = $1

#set variables 
   set curdir = $cwd             #save working dir
   set outfile = "$tocheck"p.h   #name of output file 
   set errfile = "$tocheck"p.err #name of error  file 
   #set name of reference document directory 
   if ($tocheck == "las"  ) set docdir = $LASDOC/programmer/supports
   if ($tocheck == "adaps") set docdir = $ADAPSDOC/programmer/supports 

#intialize output files
   echo > $curdir/$outfile
   echo > $curdir/$errfile

#make prototypes 
   cd $docdir
   foreach dir ( `ls -1F | egrep '[/]$' | sed -e "s/[/]//" ` )
      cd $docdir/$dir 
      echo Processing directory $cwd 
      /bin/echo    >> $curdir/$outfile 
      echo // $dir >> $curdir/$outfile
      foreach file ( `ls -1 *.html | grep -v HEADER.html` ) 
	 echo '  ' $file 
	 ($curdir/makep $docdir/$dir/$file >> $curdir/$outfile) >>& \
	  $curdir/$errfile
      end
   end
   cd $curdir 

#check for errors 
   echo Listing errors 
   cat $curdir/$errfile 

#completion message
   echo makep.csh completed and created $outfile and $errfile 
