/*******************************************************************************
FUNCTION: bsplit

	  Split a file on 1 byte boundaries.  The resulting files are named
	  <filenameXXXXX> where <filename> is the original name of the file
	  and XXXXX is a number from 00001 to 99999 indicating which portion of
	  the original file it contains.


Programmer       Date     Modification
---------------  -------  ------------------------------------------------------
T.Butzer         10/1992  Original development
J.Fenno          04/1992  Modification for University of Miami 1k image
			  transfers

*******************************************************************************/
#include <stdio.h>

#define DEFAULT  50000
#define FALSE        0
#define MAXFILES 99999
#define TRUE         1


main(argc, argv)

   int  argc;
   char *argv[];
{
FILE *infile;
FILE *outfile;

char infile_name[256];
char outfile_name[256];

long num_bytes;
long num_files;

register ch;
register count;
register filesize;


/* Retrieve the arguments to the function
-----------------------------------------*/
if (argc == 2)
   {
   if (argv[1][0] == '-')
      {
      fprintf(stderr, "useage: bsplit [-bytes] <filename>\n");
      exit(1);
      }
   
   num_bytes = DEFAULT;
   strcpy(infile_name, argv[1]);
   fprintf(stderr, "default blocking = %1d bytes\n", num_bytes);
   }
else if (argc == 3)
   {
   if (argv[1][0] == '-')
      {
      sscanf(&argv[1][1], "%d", &num_bytes);
      strcpy(infile_name, argv[2]);
      }
   else if (argv[2][0] == '-')
      {
      sscanf(&argv[2][1], "%d", &num_bytes);
      strcpy(infile_name, argv[1]);
      }
   else
      {
      fprintf(stderr, "useage: bsplit [-bytes] <filename>\n");
      exit(1);
      }

   if (num_bytes < 1)
      {
      fprintf(stderr, "illegal blocking factor: %1d\n", num_bytes);
      exit(1);
      }
   }
else
   {
   fprintf(stderr, "\n");
   fprintf(stderr, "Useage: bsplit [-bytes] <filename>\n\n");
   fprintf(stderr, "        Split the input file into separate files\n");
   fprintf(stderr, "        containing the specified number of bytes.\n");
   fprintf(stderr, "        The default is %1d bytes.\n", DEFAULT);
   fprintf(stderr, "\n");
   exit(1);
   }


/* Open the input file for reading
----------------------------------*/
infile = fopen(infile_name, "r");
if (infile == NULL)
   {
   fprintf(stderr, "error opening file %s\n", infile_name);
   exit(1);
   }


/* Split up the input file into num_bytes blocks
------------------------------------------------*/
filesize = num_bytes;
num_files = 0;
while (TRUE)
   {

   /* Create a the appropriate output file name and open it for writing
   --------------------------------------------------------------------*/
   num_files++;
   sprintf(outfile_name, "%s%05d", infile_name, num_files);
   outfile = fopen(outfile_name, "w");
   if (outfile == NULL)
      {
      fprintf(stderr, "error opening file %s\n", outfile_name);
      exit(1);
      }

   /* Write the specified number of bytes to the output file
   ---------------------------------------------------------*/
   for (count = 0; count < filesize; count++)
      {
      ch = getc(infile);
      if (ch == EOF)
	 {
	 fclose(infile);
	 fclose(outfile);
	 exit(0);
	 }
      putc(ch, outfile);
      }
   
   /* If we have created MAXFILES then place the rest in this file
   ---------------------------------------------------------------*/
   if (num_files == MAXFILES)
      {
      fprintf(stderr, "Creating maximum number of files (%d)\n", num_files);
      for (;;)
	 {
	 ch = getc(infile);
	 if (ch == EOF)
	    {
	    fclose(infile);
	    fclose(outfile);
	    exit(0);
	    }
	 putc(ch, outfile);
	 }
      }

   /* Close the output file
   ------------------------*/
   fclose(outfile);

   /* Check to see if we are at EOF
   --------------------------------*/
   ch = getc(infile);
   if (ch == EOF)
      {
      fclose(infile);
      exit(0);
      }
   ungetc(ch, infile);
   }
}
