/*****************************************************************************
*
*   NAME: ULAS 
*
*   FUNCTION: Allows LAS functions to be called from unix command line 
*
*   PROGRAM HISTORY:
*
*   Version     DATE    Programmer      CODE            Request
*   -------     ----    ----------      ----            -------
*   1.0         10/94   T. Mittan       CSB             inital development
*   1.1         11/94   K. Lowell       CSB             Mods for command line
*							parameter entry
*
*   COMMENTS:  When entering in parameters from the UNIX command line,
*              just enter in the parameters enclosed in quotes, and
*              do not list them of the form PARM=Value.  This will not
*              work for pdfs run from the UNIX command line.  Due to this
*              limitation, list either all parameters in order, or
*              just required parameters if all following parameters have
*              default values.
*
*  EXAMPLE: ulas copy "bob" "bob2"
*           ulas cluster "[x.y.z.test]land(1,1,200,200)" "out" "241" "12" 
*****************************************************************************/
#include "las.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

main(argc,argv)

int argc;                       /* number of arguments                  */
char *argv[];                   /* argument: directory to check         */

{
char taeprocstr[256];		/* tae process pointer			*/
char quote[2];			/* quotation mark			*/
FILE *lasproc;			/* file poiter to las pipe		*/
long fatal = -1;		/* fatal error message			*/
long nonfatal = 1;		/* non fatal error message		*/
long status;			/* return status			*/
char func[100];			/* function to run			*/
char parmfile[100];		/* parameter file to use		*/
char *ptr;			/* pointer to string			*/
char comstring[256];		/* output command string  		*/
long ii;			/* loop counter				*/
long yy;			/* dummy var				*/

/* Check for sufficient arguments
-------------------------------*/
if (argc < 2)
    {
    c_errmsg("tape <function> <parm>","ulas-usage", &nonfatal);
    c_errmsg("Fatal error encountered","ulas-fatal",&fatal);
    }

/* Fill function and parameter variables
--------------------------------------*/
strcpy(func,argv[1]);
if (argc < 3)
   strcpy(parmfile,"");
else
   strcpy(parmfile,argv[2]);

          
strcpy(quote,"\"");

/* Get TAE executable
-------------------*/
ptr = (char *)getenv("TAEBIN");

if (ptr == NULL)
    {
    c_errmsg("Environment variable TAEBIN undefined","ulas-tae", &nonfatal);
    c_errmsg("Fatal error encountered","ulas-fatal",&fatal);
    }

/* Build the output command call
---------------------------------*/
strcpy(comstring,func);
for (ii=2;ii<argc;ii++)
  {
  strcat(comstring," ");
  strcat(comstring,"\"");
  strcat(comstring,argv[ii]);
  strcat(comstring,"\"");
  }

sprintf(taeprocstr,"%s/%s",ptr,"taetm");

/* invoke las and run the function with the parameter file 
--------------------------------------------------------*/
lasproc = popen(taeprocstr,"w");
fprintf(lasproc,"let $message =%ssilent%s \n",quote,quote);
fprintf(lasproc,"%s\n",comstring);
fprintf(lasproc,"exit\n");

status = pclose(lasproc);
wait(&yy);
}
