/*
 *  This is the main function source file for the main panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <math.h>
#include "xid.h"
#include "xid_widgets/Context.h"
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include "xid_colortool/ct_interface.h"

void
main_config(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "configuration" panel which displays the USGS
    logo and identifies the visual and size of the current display. 

By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
*******************************************************************************/
{
Arg 	args[1];		/* X resource arguments.		*/

XmString    
	xmstr[1];		/* Motif format string variable.	*/

char	size_string[15],	/* Character string containing the screen
				   size.				*/
	visual_string[25];	/* Character string containing the visual
				   depth and type.			*/

/*
 *  Assign the visual depth and type to a string.
 */
if (xid.visual_info[0].visual->class == PseudoColor)
    strcpy(visual_string,"8-bit PseudoColor");
else if (xid.visual_info[0].visual->class == DirectColor)
    strcpy(visual_string,"24-bit DirectColor");

/*
    Display the appropriate information.
*/
XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
    visual_string,XmSTRING_DEFAULT_CHARSET))); 
XtSetValues(config_info_visual,args,1);
XmStringFree( xmstr[0] );

sprintf(size_string,"%d x %d",DisplayWidth(xid.display,0),
    DisplayHeight(xid.display,0));
XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
    size_string,XmSTRING_DEFAULT_CHARSET))); 
XtSetValues(config_info_size,args,1);
XmStringFree( xmstr[0] );

/*
    Pop up the configuration shell.
*/
XtManageChild(Config_form);
XtPopup(Shell_Config, XtGrabNone);

/*
    Make the configuration button on the main panel insensitive.
*/
XtSetSensitive(main_config_button,False);
}

void
main_context(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "mouse context" panel which gives the user
    information about the functions associated with the mouse buttons.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
char	active_context[ELEMENT_MAX_LENGTH];
					/* Active context name.		      */

/*
    Pop up the mouse context shell.
*/
XtManageChild(Context_form);
XtPopup(Shell_Mouse, XtGrabNone);

/*
    Make sure the appropriate label is visible on the option menu.  This needs
    to be done because when XeContextSetActive is called before the context
    widget is managed, the option menu label is not updated properly (I think
    because it is a gadget and not a widget).
*/
XeContextGetActive((XeContextWidget)context_mouse,active_context);
XeContextSetActive((XeContextWidget)context_mouse,active_context);

/*
    Make the mouse context button on the main panel insensitive.
*/
XtSetSensitive(main_context_button,False);
}

void
main_cursor(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "cursor" panel. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Create the cursor panel.
*/
if (cursor_bulletinboard == NULL)
    Cursor_bulletinboard = Createcursor_bulletinboard(Shell_Cursor);
XtManageChild(Cursor_bulletinboard);
XtPopup(Shell_Cursor,XtGrabNone);

/*
    Set the cursor button on the main panel and the scale on the cursor panel
    to insensitive.
*/
XtSetSensitive(main_cursor_button,False);
XtSetSensitive(cursor_scale,False);
}

void
main_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine frees the appropriate resources, closes the main panel and
    main image window and exits the XID application.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	i;			/* Looping index.			      */

/*
    Free the appropriate resources before exiting the application.
*/
if (xid_psd.color_psd != NULL)
    free(xid_psd.color_psd);
if (xid_mem.imgpix)
    {
    XFreePixmap(xid.display,xid_mem.image_pixmap);
    xid_mem.image_pixmap = (Pixmap)NULL;
    }
if (xid_zoom.image_pixmap != NULL)
    XFreePixmap(xid.display,xid_zoom.image_pixmap);
if (xid_zoom.image_data != NULL)
    free(xid_zoom.image_data);
if (xid_cursor.pointer_pixmap != NULL)
    {
    XFreePixmap(xid.display,xid_cursor.pointer_pixmap);
    XFreeGC(xid.display,xid_cursor.pointer_gc);
    }
if ((xid.num_vert > 0) || (xid.num_horiz > 0))
    {
    XFreeGC(xid.display,xid.line_gc);
    xid.line_gc = NULL;
    }

/*
    Free any graphics mask pixmaps that have been allocated.
*/
for (i=0; i < MASK_MAX; i++)
    if (xid_mask.mask_pixmap[i] != NULL)
	{
	XFreePixmap(xid.display,xid_mask.mask_pixmap[i]);
	xid_mask.mask_pixmap[i] = (Pixmap)NULL;
	xid_mask.color[i] = -1;
	}

/*
    Free the graphics GC and fonts.
*/ 
if (xid_graphics.rb_gc != NULL)
    XFreeGC(xid.display,xid_graphics.rb_gc); 
for (i=0; i < USER_GRFONTS; i++)
    if (xid_graphics.font_info[i] != NULL)
	XFreeFont(xid.display,xid_graphics.font_info[i]);
/*
    Free the graphics pixmaps.
*/
if (xid_graphics.pixmap[0] != NULL)
    for (i=0; i < USER_GRCOLORS; i++)
        XFreePixmap(xid.display,xid_graphics.pixmap[i]);

if (xid_graphics.full_pix[0] != NULL)
    for (i=0; i < USER_GRCOLORS; i++)
        XFreePixmap(xid.display,xid_graphics.full_pix[i]);

if (xid_graphics.zoom_pix[0] != NULL)
    for (i=0; i < USER_GRCOLORS; i++)
        XFreePixmap(xid.display,xid_graphics.zoom_pix[i]);

/*
    Free the icon Pixmap.
*/
if (xid.icon_pixmap != NULL)
    XFreePixmap(xid.display,xid.icon_pixmap);

/*
    Close the main panel and main image window.
*/
XtUnmanageChild(main_form);
XtPopdown(Shell_MainMenu);
XtUnmanageChild(main_image_scrollwin);
XtPopdown(Shell_MainWin);

/*
    Kill the Mosaic process if one is running.
*/
if (xid.help_pid != -1)
    {
    kill(xid.help_pid, SIGTERM);
    }

/*
    Exit the application.
*/
exit(0);
}

void
main_fill(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "fill" panel. 

By L. Huewe, USGS/EROS Data Center, 7/94

Algorithm Description:
*******************************************************************************/
{
/*
    If no image is currently being viewed, print a warning message and return.
*/
if (xid.view == -1)
    {
    xid_message("No image data currently being viewed");
    return;
    }

/*
    Create the fill panel.
*/
if (fill_form == NULL)
    fill_form = Createfill_form(Shell_Fill);

XtManageChild(fill_form);
XtPopup(Shell_Fill,XtGrabNone);

/*
    Set the fill button on the main panel to insensitive.
*/
XtSetSensitive(main_fill_button,False);

/*
    Add the "Fill" context and make it active.
*/
if (xid_fill.flags & FILL_USER)
    {
    xid_contextadd(CON_FILL);
    XeContextSetActive((XeContextWidget)context_mouse,context_names[CON_FILL]);
    }
}

void
main_fullres(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "full resolution" panel.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    A full resolution window cannot be loaded unless an image is currently
    displayed in the main image window.
*/
if (xid.view == -1)
    xid_message("No image currently displayed to window");
else
    {
/*
    Pop-up the full resolution panel and image window.
*/
    XtManageChild(Fullres_panel_bulletinboard);
    XtManageChild(Fullres_scrollwin);

    XtPopup(Shell_FullRes,XtGrabNone);
    XtPopup(Shell_FullResWin,XtGrabNone);

/*
    Set the full resolution button on the main panel to insensitive.
*/
    XtSetSensitive(main_fullres_button,False);

/*
    Initialize and allocate space for the full resolution structure.
*/
    fullres_create();
    }
}

void
main_histo(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "histogram" panel. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	create = FALSE;		/* Indicates whether or not the histogram 
				   panel was created.			      */

/*
    Create the histogram panel.
*/
if (histo_form == NULL)
    {
    Histo_form = Createhisto_form(Shell_Histo);
    create = TRUE;
    }
XtManageChild(Histo_form);
XtPopup(Shell_Histo,XtGrabNone);

/*
    Set the histogram button on the main panel to insensitive.
*/
XtSetSensitive(main_histo_button,False);

/*
    If the current mode is XZ or YZ, call the routine which adds the
    appropriate event handlers and draws the cursor tracking line.
*/
if ((!create) && (xid_histo.flags & HISTO_XZ))
    {
/*
    Increase the number of horizontal tracking lines by 1.  This needs to be
    done so that histo_xyz works correctly.
*/
    xid.num_horiz++;
    histo_xyz(histo_xz_button,"XZ",call);
    }
else if ((!create) && (xid_histo.flags & HISTO_YZ))
    {
/*
    Increase the number of vertical tracking lines by 1.  This needs to be
    done so that histo_xyz works correctly.
*/
    xid.num_vert++;
    histo_xyz(histo_yz_button,"YZ",call);
    }
}

void
main_image_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine destroys the main panel and main image window. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Destroy the main panel and main image window.
*/
main_destroy(w, client, call);
}

void
main_init(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "initialize" panel. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Create the initialize panel.
*/
Init_bulletinboard = Createinit_bulletinboard(Shell_Init);
XtManageChild(Init_bulletinboard);
XtPopup(Shell_Init,XtGrabNone);

/*
    Set initial values in the "init" structure.
*/
init_create();

/*
    Set the initialize button on the main panel to insensitive.
*/
XtSetSensitive(main_init_button,False);
}

void
main_layout(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "layout" panel. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Create the layout panel.
*/
if (layout_bulletinBoard == NULL)
    Layout_bulletinboard = Createlayout_bulletinBoard(Shell_Layout);
XtManageChild(Layout_bulletinboard);
XtPopup(Shell_Layout, XtGrabNone);

/*
    Set the layout button on the main panel to insensitive.
*/
XtSetSensitive(main_layout_button,False);
}

void
main_load(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "load" panel (for LAS and nonLAS images). 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	i;		/* Looping index.				      */

if (w == main_load_las)
    {
    xid_load.type = LAS;
/*
    Create the file selection box. 
*/
    Load_form = Createload_fileselector(Shell_Load);
    XtManageChild(Load_form);
    XtPopup(Shell_Load,XtGrabNone);
    }
else if (w == main_load_nonlas)
    {
    xid_load.type = NONLAS;
/*
    Initialize projection information structure.
*/
    xid_projinfo.proj_code = xid_projinfo.zone_code = xid_projinfo.datum_code = 
	-999;
    strcpy(xid_projinfo.proj_units,"");
    xid_projinfo.pdist_x = xid_projinfo.pdist_y = xid_projinfo.line_inc = 
	xid_projinfo.sample_inc = 0.0;
    xid_projinfo.upleft[0] = xid_projinfo.upleft[1] = 0.0; 
    xid_projinfo.loleft[0] = xid_projinfo.loleft[1] = 0.0; 
    xid_projinfo.upright[0] = xid_projinfo.upright[1] = 0.0; 
    xid_projinfo.loright[0] = xid_projinfo.loright[1] = 0.0; 
    for (i=0; i < 15; i++)
	xid_projinfo.proj_coef[i] = 0.0;

/*
    Create the file selection box. 
*/
    Load_form = Createload_fileselector(Shell_Load);

    XtManageChild(Load_form);
    XtPopup(Shell_Load,XtGrabNone);
    }
/*
    Set the load image buttons to insensitive.
*/
XtSetSensitive(main_load_las,False);
XtSetSensitive(main_load_nonlas,False);
}

void
main_mask(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "mask" panel. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Create the mask panel.
*/
if (mask_bulletinboard == NULL)
    Mask_bulletinboard = Createmask_bulletinboard(Shell_Mask);

mask_create();
XmTextSetString(mask_value_text,"");
XtManageChild(Mask_bulletinboard);
XtPopup(Shell_Mask,XtGrabNone);

/*
    Set the mask button on the main panel to insensitive.
*/
XtSetSensitive(main_mask_button,False);
}

void
main_show(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "show" panel. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Create the show panel.
*/
if (Show_form == NULL)
    {
    Show_form = Createshow_form(Shell_Show);
    Createshow_savedialog(Shell_Show);
    Createshow_infodialog(Shell_Show);
    }

/*
    Set up the lists of valid plane numbers.
*/
xid_image_makelist(show_redlist,show_greenlist,show_bluelist,show_savelist,
    TRUE);

XtManageChild(Show_form);
XtPopup(Shell_Show,XtGrabNone);

/*
    Set the show button on the main panel to insensitive.
*/
XtSetSensitive(main_show_button,False);
}

void
main_profile(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "profile" panel, adds the callback routines
    needed to scroll the profile window along with the main image window
    and calls the appropriate profile initialization routines. 

By L. Huewe, USGS/EROS Data Center, 11/91

Modified by L. Huewe, USGS/EROS Data Center, 3/94
    - Modified to add the appropriate callbacks to scroll the profile when the
      image scrollbars are moved with the arrow keys.

Algorithm Description:
*******************************************************************************/
{
long	create = FALSE;		/* Indicates whether or not the panel was
				   created.				      */

/*
    If no image is currently being viewed, print a warning message and return.
*/
if (xid.view == -1)
    {
    xid_message("No image data currently being viewed");
    return;
    }

/*
    Create the profile panel.
*/
if (profile_form == NULL)
    {
    Profile_form = Createprofile_form(Shell_Profile);
    create = TRUE;
    }
XtManageChild(Profile_form);
XtPopup(Shell_Profile,XtGrabNone);

/*
    Set the profile button on the main panel to insensitive.
*/
XtSetSensitive(main_profile_button,False);

/*
    Add the appropriate callbacks to move the profile scrollbars with the
    main image window scrollbars.
*/
XtAddCallback(main_hor_scrollbar,XmNdragCallback,profile_move,(caddr_t)0);
XtAddCallback(main_hor_scrollbar,XmNvalueChangedCallback,profile_move,
    (caddr_t)0);
XtAddCallback(main_hor_scrollbar,XmNdecrementCallback,profile_move,(caddr_t)0);
XtAddCallback(main_hor_scrollbar,XmNincrementCallback,profile_move,(caddr_t)0);
XtAddCallback(main_hor_scrollbar,XmNpageDecrementCallback,profile_move,
    (caddr_t)0);
XtAddCallback(main_hor_scrollbar,XmNpageIncrementCallback,profile_move,
    (caddr_t)0);

XtAddCallback(main_vert_scrollbar,XmNdragCallback,profile_move,(caddr_t)0);
XtAddCallback(main_vert_scrollbar,XmNvalueChangedCallback,profile_move,
    (caddr_t)0);
XtAddCallback(main_vert_scrollbar,XmNdecrementCallback,profile_move,(caddr_t)0);
XtAddCallback(main_vert_scrollbar,XmNincrementCallback,profile_move,(caddr_t)0);
XtAddCallback(main_vert_scrollbar,XmNpageDecrementCallback,profile_move,
    (caddr_t)0);
XtAddCallback(main_vert_scrollbar,XmNpageIncrementCallback,profile_move,
    (caddr_t)0);

/*
    Perform initial profile setups.
*/
profile_create();

/*
    Set the appropriate variables and add appropriate event handlers to 
    reflect the default profile direction.
*/
if ((create) || (xid_profile.flags & PROF_X))
    {
/*
    Increase the number of horizontal tracking lines by 1.  This needs to be
    done so that profile_dir works correctly.
*/
    xid.num_horiz++;
    profile_dir(profile_form,"X",0);
    }
else if (xid_profile.flags & PROF_Y)
    {
/*
    Increase the number of vertical tracking lines by 1.  This needs to be
    done the first time so that profile_dir works correctly.
*/
    xid.num_vert++;
    profile_dir(profile_form,"Y",0);
    }
else if (xid_profile.flags & PROF_Z)
    profile_dir(profile_form,"Z",0);
}

void
main_zoom(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "zoom" panel. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Can't zoom if no image is being displayed.
*/
if (xid.view == -1)
    xid_message("No image currently displayed to zoom");
else
    {
/*
    Create the zoom panel and zoom image window.
*/
    XtManageChild(Zoom_bulletinboard);
    XtManageChild(Zoom_scrollwin);

    XtPopup(Shell_Zoom,XtGrabNone);
    XtPopup(Shell_ZoomWin,XtGrabNone);

/*
    Set the zoom button on the main panel to insensitive.
*/
    XtSetSensitive(main_zoom_button,False);

/* 
    Initialize the zoom structure.
*/
    zoom_create();
    }
}

void
main_graphics(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "graphics" panel. 

By L. Huewe, USGS/EROS Data Center, 5/92

Algorithm Description:
*******************************************************************************/
{
/*
    Create the graphics panel.
*/
Graphics_form = Creategraphics_form(Shell_Graphics);
Createedit_form(Shell_Graphics);
XtManageChild(Graphics_form);
XtPopup(Shell_Graphics,XtGrabNone);

/*
    Set the graphics button on the main panel to insensitive.
*/
XtSetSensitive(main_graphics_button,False);
}

void
help_create(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine intializes variables for the help panel.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
XEvent  *event;         /* X Event structure.                                 */

event = (XEvent *)call;
if (event->type == MapNotify)
    {
/*
   Initialize the variable indicating the widget that help is currently 
   displayed for and set the active flag.
*/
    xid_help.widget = w;
    xid_help.flags = HELP_ACTIVE;
    }
}

void
help_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine closes the help panel and resets the appropriate variables. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Close the help panel.
*/
XtUnmanageChild(Help_form);
XtPopdown(Shell_Help);

/*
    If help was activated from the main panel, set the help button on the
    main panel to sensitive.
*/
if (xid_help.widget == main_help_button)
    XtSetSensitive(main_help_button,True);

/*
    Reset the active flag and the variable indicating which widget that help
    is currently displayed for.
*/
xid_help.flags = xid_help.flags ^ HELP_ACTIVE;
xid_help.widget = (Widget)NULL;
}

void
help_update(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine fires off a Mosaic process if one is not already active and
    displays the appropriate HTML help file.

By L. Huewe, USGS/EROS Data Center, 11/91
Modified by L. Huewe, USGS/EROS Data Center, 12/93
  - Modified to use Mosaic as the help viewer.

Algorithm Description:
*******************************************************************************/
{
FILE	*fp;		/* File pointer to appropriate help file.	      */

char	fname[20],	/* Temporary file name.				      */
	*getenv(),	/* Function to return the value of an environment
			   variable.					      */
	*help_dir,	/* Directory containing help files.		      */
	help_file[100],	/* Help file directory and name.                      */
	home_file[100],	/* Home document directory and name.		      */
	*mosaic_dir,	/* Directory containing Mosaic executable.	      */
	*mosaic_exe;	/* Mosaic path and executable.			      */

/*
    Get the value of the environment variable LASDOC in order to build
    the directory location of the help files.
*/
help_dir = getenv("LASDOC");
if (help_dir == '\0')
    {
    xid_message("Error retrieving help file directory");
    return;
    }

/*
    Get the value of the environment variable LASTOOLS.  This is the
    directory containing the Mosaic executable.
*/
mosaic_dir = getenv("LASTOOLS");
if (mosaic_dir == '\0')
    {
    xid_message("Error retrieving Mosaic directory");
    return;
    }
mosaic_exe = malloc(strlen(mosaic_dir)+strlen("Mosaic")+2);
if (mosaic_exe == NULL)
    {
    xid_message("Insufficient memory available");
    return;
    }
sprintf(mosaic_exe,"%s/Mosaic",mosaic_dir);

strcpy(help_file,help_dir);
strcpy(home_file,help_dir);
strcat(help_file,"/xid/");
strcat(home_file,"/xid/");
strcat(help_file,client);
strcat(home_file,"introduction.html");

/*
    Fork and exec a Mosaic process if one is not currently active.
*/
if (xid.help_pid == -1)
    {
    xid.help_pid = fork();

    if (xid.help_pid == 0)
	{
        execlp(mosaic_exe, "Mosaic", help_file, "-home", home_file, NULL);
	perror("Mosaic");
	exit(1);
	}
    }
/*
    Create the file telling Mosaic which HTML file to display and send
    Mosaic a signal indicating the file is ready for Mosaic to read.
*/
else
    {
    sprintf(fname,"/tmp/Mosaic.%d",xid.help_pid);
    fp = fopen(fname,"w");
    if (fp == NULL)
	{
	xid_message("Error communicating with help process");
	return;
	}
    fprintf(fp,"goto\n");
    fprintf(fp,"file://localhost%s\n",help_file);
    fclose(fp);
    if (kill(xid.help_pid, SIGUSR1) != 0)
	xid_message("Error communicating with help process");
    }
}

void
help_reset()

/*******************************************************************************
Purpose:
    This routine is triggered each time a child process is killed.  This allows
    XID to keep track of whether or not a new Mosaic process needs to be
    fired up when the user requests help or whether the current process can
    be updated.
 								
By L. Huewe, USGS/EROS Data Center, 7/92

Algorithm Description:
*******************************************************************************/
{
int	child_pid = 0,		/* Child process ID.			      */
	status;			/* Status indicator.			      */

char	fname[25];		/* Temporary file name.			      */

signal (SIGCLD,SIG_IGN);

/*
    Remove temporary file.
*/
sprintf(fname,"/tmp/Mosaic.%d",xid.help_pid);
unlink(fname);

/*
    Wait for the child process to die so that we don't have defunct processes
    hanging around.
*/
if(xid.help_pid != -1)
    {
    child_pid = waitpid(xid.help_pid, &status, WNOHANG);
    if(child_pid != 0)
        xid.help_pid = -1;
    }

/*
    Reset the signal handler.
*/
signal(SIGCLD,help_reset);

}

/* ARGSUSED */
void
context_dismiss(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
    This routine closes the "Mouse Context" panel. 
 								
By L. Huewe, USGS/EROS Data Center, 7/92

Algorithm Description:
*******************************************************************************/
{
/*
    Set the "Mouse Context" button to sensitive.
*/
XtSetSensitive(main_context_button,True);

/*
    Pop down the "Mouse Context" panel.
*/
XtUnmanageChild(Context_form);
XtPopdown(Shell_Mouse);
}

void
main_pref(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "preferences" panel. 

By L. Huewe, USGS/EROS Data Center, 02/93

Modified by L. Huewe, USGS/EROS Data Center, 3/94
    - Modified to make sure the toggle button indicating the number of image
      colors is set appropriately.

Algorithm Description:
*******************************************************************************/
{
/*
    Create the preferences panel.
*/
if (pref_form == NULL)
    Pref_form = Createpref_form(Shell_Pref);

/*
    Make sure the toggle button is set to the current state.
*/
if (*xid.colormap_ptr == xid.colormap)
    XmToggleButtonSetState(pref_200, True, True);
else
    XmToggleButtonSetState(pref_256, True, True);

XtManageChild(Pref_form);
XtPopup(Shell_Pref, XtGrabNone);

/*
    Set the preferences button on the main panel to insensitive.
*/
XtSetSensitive(main_preferences_button,False);
}

void
main_flicker(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "flicker" panel. 

By L. Huewe, USGS/EROS Data Center, 4/93

Algorithm Description:
*******************************************************************************/
{
int	i;
/*
    Create the flicker panel.
*/
if (flicker_form == NULL)
    {
    flicker_form = Createflicker_form(Shell_Flicker);
    xid_flicker.rate = 1;
    xid_flicker.flags = 0;
    }
XtManageChild(flicker_form);
XtPopup(Shell_Flicker,XtGrabNone);

/*
    Initialize variables.
*/
xid_flicker.current_image = xid_flicker.nplanes = 0;
xid_flicker.flags |= FLICKER_ACTIVE;
for (i=0; i < 20; i++)
    xid_flicker.cflags[i] = FALSE;

/*
    Set up the lists of valid plane numbers.
*/
xid_image_makelist(flicker_redlist,flicker_greenlist,flicker_bluelist,
    flicker_savelist,FALSE);

/*
    Create the flicker GC.
*/
xid_flicker.gc = XCreateGC(xid.display,XtWindow(main_image_bulletinboard),0,0);

/*
    Set the flicker button on the main panel to insensitive.
*/
XtSetSensitive(main_flicker_button,False);
}

void
main_power(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "power spectrum" panel and calls the appropriate 
    initialization routines. 

By L. Huewe, USGS/EROS Data Center, 5/93

Algorithm Description:
*******************************************************************************/
{
double	temp;			/* Temporary variable.			      */

long	count,			/* Number of active planes.		      */
	create = FALSE,		/* Indicates whether or not the panel was
				   created.				      */
	i,			/* Looping index.		 	      */
	no_z = FALSE,		/* Indiciates whether or not the Z option
				   is valid.				      */
	plane_idx;		/* Memory plane index of displayed image.     */

char    points[10];             /* Number of points string.	   	      */

/*
    If no image is currently being viewed, print a warning message and return.
*/
if (xid.view == -1)
    {
    xid_message("No image data currently being viewed");
    return;
    }

/*
    Create the power spectrum panel.
*/
if (power_form == NULL)
    {
    Power_form = Createpower_form(Shell_Power);
    create = TRUE;
    }
XtManageChild(Power_form);
XtPopup(Shell_Power,XtGrabNone);

/*
    Determine whether or not Z calculations are valid.
*/
for (i=0, count=0; i < xid.max_plane; i++)
    if (xid_planes[i].flags & USED)
   	count++;
/*
    Perform initial setups.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;
xid_power.mindb = 0.0;
if (xid_planes[plane_idx].dspwin[NS] > xid_planes[plane_idx].dspwin[NL])
    temp = (double)xid_planes[plane_idx].dspwin[NS];
else
    temp = (double)xid_planes[plane_idx].dspwin[NL];
if (count > temp)
    temp =  (double)count;
xid_power.maxdb = (20.0 * log10(temp * 255.0 * 255.0)) + 10;

/*
    More than 2 planes of image data must be available for Z calculations.
*/
if (count < 2)
    {
    XtSetSensitive(power_z_button,False);
    no_z = TRUE;
    }

/*
    Set the number of points for both the maximum entropy and the periodogram
    methods.
*/
xid_power.per_npts = (long)(nearest_power2(xid_planes[plane_idx].dspwin[NS]) 
    / 2.0);
sprintf(points,"%d",xid_power.per_npts);
XmTextFieldSetString(power_ppoints_text,points);

if (xid_power.flags & POWER_PER)
    {
    xid_power.ent_npts = xid_power.per_npts; 
    sprintf(points,"%d",xid_power.ent_npts);
    XmTextFieldSetString(power_epoints_text,points);
    }
else
    {
    xid_power.ent_npts = (long)(xid_planes[plane_idx].dspwin[NS] / 2.0);
    sprintf(points,"%d",xid_power.ent_npts);
    XmTextFieldSetString(power_epoints_text,points);
    }

/*
    Set the power spectrum button on the main panel to insensitive.
*/
XtSetSensitive(main_power_button,False);

/*
    Allocate space for maximum entropy coefficients.
*/
if ((xid_power.flags & POWER_ENT) && (xid_power.coef == NULL))
    {
    xid_power.coef = (float *)malloc(xid_power.order * sizeof(float));
    if (xid_power.coef == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
    }
/*
    Reset the direction if the Z dimension is currently selected and it is
    no longer valid.
*/
if ((xid_power.flags & POWER_Z) && no_z)
    {
    xid_power.flags = xid_power.flags & ~(POWER_X | POWER_Y | POWER_Z | 
	POWER_XY);
    xid_power.flags = xid_power.flags | POWER_X;
    XtVaSetValues(power_x_button,XmNset,True,NULL);
    XtVaSetValues(power_z_button,XmNset,False,NULL);
    }

/*
    Set the appropriate variables and add appropriate event handlers to 
    reflect the current plot direction.
*/
if ((create) || (xid_power.flags & POWER_X))
    {
/*
    Increase the number of horizontal tracking lines by 1.  This needs to be 
    done so that power_dir works correctly.
*/
    xid.num_horiz++;
    power_dir(power_form,"X",0);
    }
else if (xid_power.flags & POWER_Y)
    {
/*
    Increase the number of vertical tracking lines by 1.  This needs to be 
    done so that power_dir works correctly.
*/
    xid.num_vert++;
    power_dir(power_form,"Y",0);
    }
else if (xid_power.flags & POWER_Z)
    power_dir(power_form,"Z",0);

/*
    Add the appropriate event handler based on the mode (automatic or manual).
*/
if (xid_power.flags & POWER_MAN)
    {
    XtRemoveEventHandler(main_image_bulletinboard,
        PointerMotionMask|ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,
	NULL);
    XtAddEventHandler(main_image_bulletinboard,
        ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,NULL);
    }
else
    {
    XtRemoveEventHandler(main_image_bulletinboard,
        ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,NULL);
    XtAddEventHandler(main_image_bulletinboard,
        PointerMotionMask|ButtonReleaseMask|KeyReleaseMask,FALSE,power_draw,
	NULL);
    }
}

void
main_save(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine creates the "save" panel. 

By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
*******************************************************************************/
{
/*
    Create the save panel.
*/
if (save_form == NULL)
    {
    save_form = Createsave_form(Shell_Save);
    xid_save.flags = SAVE_MAIN | SAVE_ALL;
    }
XtManageChild(save_form);
XtPopup(Shell_Save,XtGrabNone);

/*
    Set the save button on the main panel to insensitive.
*/
XtSetSensitive(main_save_button,False);
xid_save.flags |= SAVE_ACTIVE;
}

#define COLORTOOL
#ifdef COLORTOOL

extern Widget ct_help;

void
main_colortool(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
{

/*******************************************************************************
Purpose:
    This routine creates the "interface" panel. 
*******************************************************************************/

 int	i,
	nonlinear;
 
 if (Colortool_Interface == NULL)
      {
       Colortool_Interface= Createcolortool_Interface(Colortool_Shell);
      }
 else
      {
       red.adjust.index = xid.red_count - 1;
       allrgb.adjust.index = xid.red_count - 1;
       for (i=0; i < xid.red_count; i++)
	   {
	   red.adjust.from[i] = xid.red_from[i];
	   red.adjust.to[i] = xid.red_to[i];
	   allrgb.adjust.from[i] = xid.red_from[i];
	   allrgb.adjust.to[i] = xid.red_to[i];
	   }
       piece(xid.red_from, xid.red_to, &xid.red_count, red.map);
       piece(xid.red_from, xid.red_to, &xid.red_count, allrgb.map);

       if (xid_map.flags & MAP_ALL)
	   {
	   CT_Color = &allrgb;
	   XmToggleButtonSetState (ct_stretch_color_red, False, False);
	   XmToggleButtonSetState (ct_stretch_color_green, False, False);
	   XmToggleButtonSetState (ct_stretch_color_blue, False, False);
	   XmToggleButtonSetState (ct_stretch_color_all, True, False);
	   XmToggleButtonSetState (ct_stretch_color_sat, False, False);
	   }
       else if (xid_map.flags & MAP_RED)
	   {
	   CT_Color = &red;
	   XmToggleButtonSetState (ct_stretch_color_red, True, False);
	   XmToggleButtonSetState (ct_stretch_color_green, False, False);
	   XmToggleButtonSetState (ct_stretch_color_blue, False, False);
	   XmToggleButtonSetState (ct_stretch_color_all, False, False);
	   XmToggleButtonSetState (ct_stretch_color_sat, False, False);
	   }

       green.adjust.index = xid.green_count - 1;
       for (i=0; i < xid.green_count; i++)
	   {
	   green.adjust.from[i] = xid.green_from[i];
	   green.adjust.to[i] = xid.green_to[i];
	   }
       piece(xid.green_from, xid.green_to, &xid.green_count, green.map);

       if (xid_map.flags & MAP_GREEN)
	   {
	   CT_Color = &green;
	   XmToggleButtonSetState (ct_stretch_color_red, False, False);
	   XmToggleButtonSetState (ct_stretch_color_green, True, False);
	   XmToggleButtonSetState (ct_stretch_color_blue, False, False);
	   XmToggleButtonSetState (ct_stretch_color_all, False, False);
	   XmToggleButtonSetState (ct_stretch_color_sat, False, False);
	   }

       blue.adjust.index = xid.blue_count - 1;
       for (i=0; i < xid.blue_count; i++)
	   {
	   blue.adjust.from[i] = xid.blue_from[i];
	   blue.adjust.to[i] = xid.blue_to[i];
	   }
       piece(xid.blue_from, xid.blue_to, &xid.blue_count, blue.map);

       if (xid_map.flags & MAP_BLUE)
	   {
	   CT_Color = &blue;
	   XmToggleButtonSetState (ct_stretch_color_red, False, False);
	   XmToggleButtonSetState (ct_stretch_color_green, False, False);
	   XmToggleButtonSetState (ct_stretch_color_blue, True, False);
	   XmToggleButtonSetState (ct_stretch_color_all, False, False);
	   XmToggleButtonSetState (ct_stretch_color_sat, False, False);
	   }

      nonlinear = FALSE;
      if (xid.red_count != 2) 
	  nonlinear = TRUE;
      else if ((xid.red_from[0] != 0) || (xid.red_from[1] != 255) || 
	  (xid.red_to[0] != 0) || (xid.red_to[1] != 255))
	  nonlinear = TRUE;
      if (xid.green_count != 2) 
	  nonlinear = TRUE;
      else if ((xid.green_from[0] != 0) || (xid.green_from[1] != 255) || 
	  (xid.green_to[0] != 0) || (xid.green_to[1] != 255))
	  nonlinear = TRUE;
      if (xid.blue_count != 2) 
	  nonlinear = TRUE;
      else if ((xid.blue_from[0] != 0) || (xid.blue_from[1] != 255) || 
	  (xid.blue_to[0] != 0) || (xid.blue_to[1] != 255))
	  nonlinear = TRUE;
      if (nonlinear)
          ct_new_curr_stretch();
      }

  XtManageChild(Colortool_Interface);
  XtPopup(Colortool_Shell,XtGrabNone);

  /*  The following code is similar to the handling for PWL (Piece-Wise Linear
      option in map_func function of xid_fun/map.c
  */
  xid_map.subpanel = Colortool_Shell;
  xid_pwl.color_pwl = (XColor *)NULL;

  /*  
      The following should be uncommented when done with debugging
  */
  XtSetSensitive(main_map_shell,False);
}

#endif COLORTOOL
