/****************************************************************************/
/*   PROGRAM NAME  : ct_cmapcb.c           	                            */
/*   AUTHOR        : g e nostrand                                           */
/*   DATE          : july 14, 1993                                          */
/*   LOCATION      : DSRC PRINCETON, NJ                                     */
/*                                                                          */
/*   PROGRAM DESCRIPTION: 						    */
/*

	This module contains the callbacks for the X Colormap Trak
	display.  This module can be compiled as a separate application
	or as a module within Xid.  The define INXID is true when
	linked with Xid.
  
*/
/*   PROGRAM MODIFICATIONS:                                                 */
/****************************************************************************/


/*
 * Generated by the ICS builderXcessory (BX).
 *
 *
 * Builder Xcessory 2.5.
 *
 */
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>

#include <Xm/Xm.h>
#include <math.h>

#define INXID
#ifdef INXID
#include "xid.h"
#include "ct_interface.h"
#endif

#define EXTERNAL extern
/* Start Global Widget Declarations */
EXTERNAL Widget	topLevelShell;
EXTERNAL Widget	xcmap_hls_ls_label;
EXTERNAL Widget	xcmap_hls_hs_label;
EXTERNAL Widget	xcmap_cmap_default;
EXTERNAL Widget	xcmap_cmap_private;
EXTERNAL Widget	xcmap_green_slider;
EXTERNAL Widget	xcmap_blue_slider;
EXTERNAL Widget	xcmap_color_sample;
EXTERNAL Widget	xcmap_red_slider;
EXTERNAL Widget	xcmap_colors;
EXTERNAL Widget	xcmap_ctype_rgb;
EXTERNAL Widget	xcmap_ctype_hls;
EXTERNAL Widget	xcmap_pixel_slider;
EXTERNAL Widget	xcmap_color_cone;
EXTERNAL Widget	xcmap_rgb_label;
EXTERNAL Widget	xcmap_sort_option;
EXTERNAL Widget	xcmap_rgblist_shell;
EXTERNAL Widget	xcmap_rgblist_main;
EXTERNAL Widget	xcmap_rgblist_bb;
EXTERNAL Widget	xcmap_color_list_sw;
EXTERNAL Widget	xcmap_color_list;
EXTERNAL Widget xcmap_huesat_radio;
EXTERNAL Widget xcmap_lightsat_radio;
EXTERNAL Widget xcmap_hue_tb;
EXTERNAL Widget xcmap_sat_tb;
EXTERNAL Widget xcmap_light_tb;
EXTERNAL Widget xcmap_lightsat_tb;


#ifndef INXID
EXTERNAL Widget	Shell000;
#endif  


/****************************************************************************/
/*
	Global fields for current pixel information
*/
/****************************************************************************/

int	CurrPixel = 0;		/* Current pixel number		*/
int     CurrRed = 0, CurrGreen=0, CurrBlue=0;	/* Current RGB values	*/
float   CurrHue = 0.0, CurrLight=0, CurrSat=0.0;/* Current HLS values   */
int	CurrHueSector = -1;	/* Sector of Hue-Saturation projection	*/
int	CurrLightHalf = -1;	/* Top/Bottom half of Light-Sat projection */

/*  Current and Previous X Color Map		*/
XColor	XCmapCurrColors[256], XCmapOldColors[256];

Display *XCmapDisplay;
#ifdef INXID
int	XCmapXIDMode = True;
#else
int	XCmapXIDMode = False;
#endif

/*****************************************************************************/
/*
    XCmapInfo structure tracks information for the current color data
    including colormap pixel, RGB and HLS color values, and location 
    information in the HLS projections.
*/
/*****************************************************************************/
typedef struct _XCmapInfo {
    int			pixel;
    unsigned short	red, green, blue;
    float		hue, light, sat;
    int			hsx, hsy, lsx, lsy;	/* Position in projection     */
}	XCmapInfo, *XCmapInfoPtr;

/*****************************************************************************/
/*
    XCmapOrigInfo maintains information on the original colormap
    XCmapInfoList maintains information on the current colormap
    XCmapInfoListPtr is a sorted list of pointers to XCmapInfoList 
	Sorting can be done on pixel values or RGB or HLS elements
    XCmapInfoSort is a flag to specify sorting order and can be
	PRGBHLS for sorting on pixel, red, green, blue, hue, light or sat.
*/
/*****************************************************************************/
XCmapInfo	XCmapOrigInfo[256];
XCmapInfo	XCmapInfoList[256];
XCmapInfoPtr	XCmapInfoListPtr[256];
char		XCmapInfoSort;

/*****************************************************************************/
/*
    The XCmapHLSInfo structure stores information relating to the HLS
    projections.  The XCmapHS array tracks information for the primary
    color sectors of the Hue-Saturation projection.  The XCmapLS array
    tracks information for the top and bottom half of the HLS cone with 
    respect to lightness.  Data was previously tracked for saturation 
    levels of the two halves of the hexacone in terms of hue but this 
    information is no longer used.  
*/
/*****************************************************************************/
typedef struct _XCmapHLSInfo {
    int			pixels;
    unsigned long	red, green, blue;
    float		hue, light, sat;
    int			hsx, hsy, lsx, lsy;	/* Position in projection     */
    float		hlsfactor;

}   XCmapHLSInfo, *XCmapHLSInfoPtr;

/*  HLS Hue-Saturation Hexagon divided into 6 sectors for primary colors      */
/*  Red, Yellow, Green, Cyan, Blue, and Magenta			              */
#define RSECTOR		0
#define YSECTOR		1
#define GSECTOR		2
#define CSECTOR		3
#define BSECTOR		4
#define MSECTOR		5
#define ALLSECTOR	6

/*****************************************************************************/
/*
	HLS Light-Saturation diamond projection has top/bottom half
	specified by HIGHLIGHT/LOWLIGHT.  Colors with saturation
	greater than .5 are in the top half or HIGHLIGHT half.
	Right/Left saturation half specified by MRYSAT/GCBSAT.
	Magenta, Red and Yellow colors are in the right half or MRYSAT
	and green, cyan and blue colors are in the left half or GCBSAT.

*/
/*****************************************************************************/

#define LOWLIGHT	0
#define MRYSAT		1
#define HIGHLIGHT	2
#define GCBSAT		3
#define ALLLIGHT	4

XCmapHLSInfo	XCmapHS [ALLSECTOR+1];

XCmapHLSInfo	XCmapLS [ALLLIGHT+1];

/*****************************************************************************/
/*
	Local X variables for separate drawing areas and colormap
*/
/*****************************************************************************/
GC	XCmapGC = NULL;
int	WriteCMap [256];
Colormap XCmap;

int	XCmapIMGPixel = 0;
int	XCmapColors = 256;

/*****************************************************************************/
/*
	Dimensions of drawing areas and fonts determined at run time
*/
/*****************************************************************************/
Dimension	XCmapWidth=0;
Dimension	XCmapHeight=0;
Dimension	XCmapCenter=0;
Dimension	XCmapRadius=0;
Dimension	XCmapFontw=0;
Dimension	XCmapFonth=0;
/*****************************************************************************/
/*
	Global variables specifying current modes
*/
/*****************************************************************************/
Visual		*XCmapVisual=NULL;

Pixel		 XCmapBackground, XCmapForeground, XCmapWhite;

/* Calculate size of each color square in RGB color map drawing area 	*/
#define HALFWIDTH	(XCmapWidth / 2)
#define HALFHEIGHT	(XCmapHeight/2)
#define COLORPERLINE	16
#define SQUARE		(XCmapWidth / COLORPERLINE)

/* Mode of colormap display, RGB color square or HLS projections	*/
#define RGBTYPE	1
#define HLSTYPE 2
int	XCmapType = RGBTYPE;

/* Hue-Saturation modification mode					*/
#define HUEMODE 1
#define SATMODE 2
int	XCmapHSmode = 0;

/* Light-Saturation modification mode, currently only LIGHTMODE available */
#define LIGHTMODE 1
#define SATMODE 2
int	XCmapLSmode = 0;

/* Use DEFAULT X ColorMap or create private colormap flag		*/
#define DEFAULT	1
#define PRIVATE 2
#ifdef INXID
int	XCmapStatus = PRIVATE;
#else
int	XCmapStatus = DEFAULT;
#endif

/*****************************************************************************/
/*
	Defines for drawing color in Hue-Saturation projection
*/
/*****************************************************************************/
#define PI 3.1415926535
#define HANDLESIZE	2	/* Size of adjustment squares		*/
#define HANDLEDIST	36	/* Distance from center of handle squared = 6*/
#define PLOTPOINTSIZE	3	/* Radious of color circle 	*/
#define PLOTPOINTDIST	18	/* Square of Hypotenuse	*/
#define XFULLCIRCLE	23040
#define LEN30SQUARED	.75 	/* Distance at 30 Degrees squared	*/

/*****************************************************************************/
/*
	HLS-RGB conversion routines adapted from the book Computer Graphics
	Principles and Practices, Second Edition, Foley, van Dam, Feiner, and
	Hughes in section 13.3.  

	rgbhls - input is rgb color in range 0-255, output hls
	hlsrgb - input is hls color, out is rgb in range 0-255

	Hue = angle from 0-360, red at 0.0
	Lightness = 0.0 to 1.0, white at 1.0
	Saturation = 0.0 to 1.0, saturated at 1.0, grey at 0.0	
*/
/*****************************************************************************/
#define HLSMAX(max,r,g,b) max=r;if (g>max) max=g; if (b>max) max=b;
#define HLSMIN(max,r,g,b) max=r;if (g<max) max=g; if (b<max) max=b;
#ifdef _NO_PROTO
rgbhls (red, green, blue, hue, light, sat)
int red;
int green;
int blue;
float *hue;
float *light;
float *sat;
#else
rgbhls (int red, int green, int blue, float *hue, float *light, float *sat)
#endif
{
    float	fred, fgreen, fblue;
    float	fhue, flight, fsat;
    float	fmax, fmin, fdelta;

    /*  Scale RGB to range [0,1]	*/
    fred = red / 255.0;
    fgreen = green / 255.0;
    fblue = blue / 255.0;

    /*  Find maximum and minimum RGB value	*/
    HLSMAX(fmax, fred, fgreen, fblue);
    HLSMIN(fmin, fred, fgreen, fblue);

    flight = (fmax + fmin) / 2.0;

    if (fmax == fmin)
    {
	fsat = 0;
	fhue = 0;
    }
    else
    {
	if (flight <= .5)
	    fsat = (fmax - fmin)/ (fmax + fmin);
	else
	    fsat =  (fmax - fmin) / (2 -fmax - fmin);
	
	fdelta = fmax - fmin;
	if (fred == fmax)
	    fhue = (fgreen - fblue) / fdelta;
	else
	if (fgreen == fmax) 
	    fhue = 2 + (fblue - fred) / fdelta;
	else
	if (fblue == fmax)
	    fhue = 4 + (fred - fgreen) / fdelta;

	fhue = fhue * 60.0;
	if (fhue < 0.0)
	   fhue = fhue + 360.0;

    }
#ifdef TESTRGBHLS
    printf ("%f %f %f \n", fhue, flight, fsat);
#endif

    *hue = fhue;
    *light = flight;
    *sat = fsat;

}
/*****************************************************************************/
/*

	Adjust value based on hue sector.

*/
/*****************************************************************************/
#define FROUND(fx)	((int) ((fx)+.5))
float
#ifdef _NO_PROTO
rgbval (n1, n2, hueval)
float n1;
float n2;
float hueval;
#else
rgbval (float	n1, float n2, float hueval)
#endif
{

    if (hueval > 360.0) hueval -= 360.0;
    if (hueval < 0.0)   hueval += 360.0;

    if (hueval < 60.0) 
	return (n1 + (n2 - n1) * (hueval/60.0));
    if (hueval < 180.0) 
	return (n2);
    if (hueval < 240.0) 
	return (n1 + (n2 - n1) * ((240.0 - hueval)/60.0));

    return (n1);

}
/*****************************************************************************/
/*
	Convert HLS color to RGB
*/
/*****************************************************************************/
#ifdef _NO_PROTO
hlsrgb (hue, light, sat, red, green, blue) 
float hue;
float light;
float sat;
int *red;
int *green;
int *blue;
#else
hlsrgb (float hue, float light, float sat, int *red, int *green, int *blue) 
#endif
{

    float m1=0.0, m2=0.0;

    if (light <= 0.5)
	m2 = light*(1 + sat);
    else
	m2 = light + sat - light * sat;

    m1 = 2.0 * light - m2;

    if ( sat == 0)
    {
	*red = *green = *blue = light * 255;
    }
    else
    {
	*red = FROUND(rgbval (m1, m2, hue + 120.0) * 255.0);
	*green =  FROUND(rgbval (m1, m2, hue) * 255.0);
	*blue =  FROUND(rgbval (m1, m2, hue - 120.0) * 255.0);
    }
#ifdef TESTRGBHLS
    printf ("%d %d %d \n", *red, *green, *blue);
#endif

}
/*****************************************************************************/
/*
	Routine to select window to copy colormap for stand-alone tool

	Similar to mechanisms for xwininfo
*/
/*****************************************************************************/
#ifndef INXID
/*
 * Routine to let user select a window using the mouse
 */

Window Select_Window(dpy)
     Display *dpy;
{
  int status;
  Cursor cursor;
  XEvent event;
  int   screen = DefaultScreen(dpy);
  Window target_win = None, root = RootWindow(dpy,screen);
  int buttons = 0;

  /* Make the target cursor */
  cursor = XCreateFontCursor(dpy, XC_crosshair);

  /* Grab the pointer using target cursor, letting it room all over */
  status = XGrabPointer(dpy, root, False,
			ButtonPressMask|ButtonReleaseMask, GrabModeSync,
			GrabModeAsync, root, cursor, CurrentTime);
  if (status != GrabSuccess) XtError("Can't grab the mouse.");

  /* Let the user select a window... */
  while ((target_win == None) || (buttons != 0)) {
    /* allow one more event */
    XAllowEvents(dpy, SyncPointer, CurrentTime);
    XWindowEvent(dpy, root, ButtonPressMask|ButtonReleaseMask, &event);
    switch (event.type) {
    case ButtonPress:
      if (target_win == None) {
	target_win = event.xbutton.subwindow; /* window selected */
	if (target_win == None) target_win = root;
      }
      buttons++;
      break;
    case ButtonRelease:
      if (buttons > 0) /* there may have been some down before we started */
	buttons--;
       break;
    }
  } 

  XUngrabPointer(dpy, CurrentTime);      /* Done with pointer */

  {
     Window root;
     int	dummyi;
     unsigned int dummy;
     if (XGetGeometry (dpy, target_win, &root, &dummyi, &dummyi, 
		&dummy, &dummy, &dummy, &dummy) && target_win != root)
     {
	 target_win = XmuClientWindow (dpy, target_win);
     }
  }
  return(target_win);
}
#endif
/*****************************************************************************/
/*
	Dummy functions for map callbacks, not used.
*/
/*****************************************************************************/
#ifndef INXID
/* ARGSUSED */
void
#ifdef _NO_PROTO
map_component(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
map_component(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
}

/* ARGSUSED */
void
#ifdef _NO_PROTO
map_destroy(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
map_destroy(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
}

/* ARGSUSED */
void
#ifdef _NO_PROTO
map_restore(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
map_restore(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
}


/* ARGSUSED */
void
#ifdef _NO_PROTO
xcmap_apply(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_apply(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
}

/* ARGSUSED */
void
#ifdef _NO_PROTO
xcmap_cancel(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_cancel(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
}
#endif

/*****************************************************************************/
/*

	Find writable entries in default colormap

*/
/*****************************************************************************/
void
find_writable_cmap ()
{
    XWindowAttributes win_attr;
    int	plane_masks[1];
    int	colors[256];
    int	status;
    int	i;

    for (i=0; i<255; i++) WriteCMap [i] = False;
    if (XCmapVisual->class == TrueColor) return;
#ifdef INXID
    for (i=0; i<255; i++) WriteCMap [i] = True;
    return;
#else

    status = XAllocColorCells (XCmapDisplay, XCmap, False, 
    plane_masks, 0, colors, 1);
    while (status == 1)
    {
	if ((XCmapVisual->class == DirectColor)
	|| (XCmapVisual->class == TrueColor))
	    WriteCMap [colors[0] & 0xff] = True;
	else
	    WriteCMap [colors[0]] = True;
	status = XAllocColorCells (XCmapDisplay, XCmap, False, 
	plane_masks, 0, colors, 1);
    }
#endif
}
/*****************************************************************************/
/*
	Set pixel value appropriate for X Visual
*/
/*****************************************************************************/
#ifdef _NO_PROTO
xcmap_set_pixel (pixel)
int pixel;
#else
xcmap_set_pixel (int	pixel)
#endif
{

    int	drawpixel;

    if ((XCmapVisual->class == DirectColor)
    || (XCmapVisual->class == TrueColor))
	drawpixel = pixel | pixel << 8 | pixel << 16;
    else
	drawpixel = pixel;

    return (drawpixel);

}
/*****************************************************************************/
/*
	Format entry for X Colormap List showing pixel, RGB and HLS values
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_format_list_entry (pixel, liststr)
int pixel;
char *liststr;
#else
xcmap_format_list_entry (int	pixel, char	*liststr)
#endif
{

    sprintf (liststr, "%.3d %.3d %.3d %.3d %4.1f %3.3f %3.3f", pixel,
				     XCmapInfoList[pixel].red,
				     XCmapInfoList[pixel].green,
				     XCmapInfoList[pixel].blue,
				     XCmapInfoList[pixel].hue,
				     XCmapInfoList[pixel].light,
				     XCmapInfoList[pixel].sat);
}


/*****************************************************************************/
/*
	Set the elements of the X Colormap List widget
*/
/*****************************************************************************/
xcmap_set_color_list ()
{
    int	pixel;
    char	liststr[240];

    if (! xcmap_color_list ) return;

    XmListDeleteAllItems (xcmap_color_list);
    for (pixel = 0; pixel < 256; pixel ++)
    {

        xcmap_format_list_entry (pixel, liststr);

	XmListAddItem (xcmap_color_list, 
		XmStringCreateLtoR(liststr, XmSTRING_DEFAULT_CHARSET),pixel+1);
    }
    XmListSetPos (xcmap_color_list, CurrPixel+1);
    XmListSelectPos (xcmap_color_list, CurrPixel+1, False);

}
/*****************************************************************************/
/*
    Draw square with selected pixel color.  Set slider to either RGB or 
    HLS representation of pixel color.  
*/
/*****************************************************************************/
xcmap_show_curr_pixel_color ()
{
    void	xcmap_refresh_color_sample ();
    int	drawpixel;

    XCmapCurrColors[CurrPixel].pixel = xcmap_set_pixel (CurrPixel);
    XQueryColor (XtDisplay(xcmap_red_slider), XCmap, &XCmapCurrColors[CurrPixel]);

    CurrRed = XCmapCurrColors[CurrPixel].red / 256;
    CurrGreen = XCmapCurrColors[CurrPixel].green / 256;
    CurrBlue = XCmapCurrColors[CurrPixel].blue / 256;

    rgbhls (CurrRed, CurrGreen, CurrBlue, &CurrHue, &CurrLight, &CurrSat);
    if (XCmapType == RGBTYPE) {
	XmScaleSetValue (xcmap_red_slider, CurrRed);
	XmScaleSetValue (xcmap_green_slider, CurrGreen);
	XmScaleSetValue (xcmap_blue_slider, CurrBlue);
	xcmap_refresh_color_sample ();
    }
    else
    {

	XmScaleSetValue (xcmap_red_slider, (int) (CurrHue * 10));
	XmScaleSetValue (xcmap_green_slider, (int) (CurrLight * 1000));
	XmScaleSetValue (xcmap_blue_slider, (int) (CurrSat * 1000));

	xcmap_refresh_color_sample ();
    }
    if (WriteCMap [ CurrPixel])
    {
	Arg	args[1];

	XtSetArg (args[0], XmNsensitive, True);
	XtSetValues (xcmap_red_slider, args, 1);
	XtSetValues (xcmap_green_slider, args, 1);
	XtSetValues (xcmap_blue_slider, args, 1);
    }
    else
    {
	Arg	args[1];

	XtSetArg (args[0], XmNsensitive, False);
	XtSetValues (xcmap_red_slider, args, 1);
	XtSetValues (xcmap_green_slider, args, 1);
	XtSetValues (xcmap_blue_slider, args, 1);

    }

    if (xcmap_color_list)
    {
	XmListSetPos (xcmap_color_list, CurrPixel+1);
	XmListSelectPos (xcmap_color_list, CurrPixel+1, False);
    }
    XmScaleSetValue (xcmap_pixel_slider, CurrPixel);
}


/*****************************************************************************/
/*
	Get current X colormap information and store RGB and HLS info
*/
/*****************************************************************************/
void
xcmap_upd_cmap_info ()
{
    int pixel;
    int		colors=256;

    for (pixel=0; pixel < 256; pixel++)
	XCmapCurrColors[pixel].pixel = xcmap_set_pixel(pixel);
    XQueryColors (XCmapDisplay,  XCmap, XCmapCurrColors, colors);
    for (pixel=0; pixel < 256; pixel++)
    {
	XCmapInfoList[pixel].pixel = pixel;
	XCmapInfoList[pixel].red = XCmapCurrColors[pixel].red / 256;
	XCmapInfoList[pixel].green = XCmapCurrColors[pixel].green / 256;
	XCmapInfoList[pixel].blue = XCmapCurrColors[pixel].blue / 256;
	rgbhls (XCmapInfoList[pixel].red, 
		XCmapInfoList[pixel].green, XCmapInfoList[pixel].blue, 
		&XCmapInfoList[pixel].hue,&XCmapInfoList[pixel].light,
		&XCmapInfoList[pixel].sat);
    }

}

/*****************************************************************************/
/*
	Draw RGB color square display
*/
/*****************************************************************************/
void
xcmap_draw_rgb_colormap ()
{
    int 	pixel, pixelx, pixely;
    int		colors=256;
    float	hue, light, sat;


    xcmap_upd_cmap_info();

    for (pixel = 0; pixel < 256; pixel++)
    {
	int     drawpixel;

	pixelx = pixel % COLORPERLINE * SQUARE;
	pixely = pixel / COLORPERLINE * SQUARE;
	drawpixel = xcmap_set_pixel (XCmapInfoListPtr[pixel]->pixel);
        XSetForeground(XtDisplay(xcmap_colors), XCmapGC, drawpixel);
	XFillRectangle (XtDisplay(xcmap_colors), XtWindow(xcmap_colors),
		XCmapGC, pixelx, pixely, SQUARE, SQUARE);
    }
}


/*****************************************************************************/
/*
	Lightness factor specifies distance from center of projection
	for plotting saturation.  The further the lightness from .5, the 
	smaller the radius of the hexagon becomes.  Huefactor also adjust
	for saturation plot by calculating distance from center of hue sector
	and adjusting to hexagonal coordinates instead of circular.
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_calc_lightfactor (hue, light, lightfactor)
float hue;
float light;
float *lightfactor;
#else
xcmap_calc_lightfactor (float hue, float light, float *lightfactor)
#endif
{

    static float lasthue = -1.0, lastlight = -1.0, lastlightfactor = -1.0;
    double 	huefactor, sqroot=.5;
    
    if (hue != lasthue || lastlight != light)
    {
	if (light <= 0.5)
	    *lightfactor = light * 2.0 * XCmapRadius;
	else
	    *lightfactor = (1-light) * 2.0 * XCmapRadius;
	huefactor = hue;
	huefactor = fmod (huefactor, (double) 60.0);
	huefactor = ((huefactor - 30.0 )/30.0 * .5);
	huefactor = huefactor * huefactor;
	huefactor += LEN30SQUARED;
	huefactor = pow (huefactor, sqroot);
	*lightfactor *= huefactor;

	lasthue = hue;
	lastlight = light;
	lastlightfactor = *lightfactor;
    }
    else
    {
	*lightfactor = lastlightfactor;
    }

}

/*****************************************************************************/
/*
	Calculate sin and cos for specified hue angle in radians
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_calc_sincos (huerad, calcsin, calccos)
float huerad;
double *calcsin;
double *calccos;
#else
xcmap_calc_sincos (float huerad, double *calcsin, double *calccos)
#endif
{
    static float lasthuerad = -1.0;
    static double lastsin= -1.0, lastcos = -1.0;

    if (huerad != lasthuerad)
    {
	*calcsin = sin(huerad);
	*calccos = cos(huerad);

	lasthuerad = huerad;
	lastsin = *calcsin;
	lastcos = *calccos;
    }
    else
    {
	*calcsin = lastsin;
	*calccos = lastcos;
    }
}

/*****************************************************************************/
/*
	Calculate color posiion in Hue-Saturation hexagonal projection
*/
/*****************************************************************************/
void 
#ifdef _NO_PROTO
xcmap_calc_hue_sat_pixel (hue, light, sat, pixelx, pixely)
float hue;
float light;
float sat;
int *pixelx;
int *pixely;
#else
xcmap_calc_hue_sat_pixel (
    float hue, float light, float sat, int *pixelx, int *pixely)
#endif
{
    static float lasthue = -1.0, lastlight = -1.0, lastsat = -1.0;
    static int	lastpixelx=0, lastpixely=0;
    double	huerad, calcsin, calccos;
    int		satfactor;
    float	lightfactor;

    if (hue != lasthue || light != lastlight || sat != lastsat)
    {

	huerad = (hue/180.0) * PI;
	xcmap_calc_lightfactor (hue, light, &lightfactor);
	xcmap_calc_sincos(huerad, &calcsin, &calccos);
	*pixelx = (XCmapCenter + (calccos * lightfactor * sat));
	*pixely = (XCmapCenter - (calcsin * lightfactor * sat));
	
	lasthue = hue;
	lastlight = light;
	lastsat = sat;
	lastpixelx = *pixelx;
	lastpixely = *pixely;
    }
    else
    {
	*pixelx = lastpixelx;
	*pixely = lastpixely;
    }

}
/*****************************************************************************/
/*
	Calculate color position in Light-Saturation diamond projection
*/
/*****************************************************************************/
void 
#ifdef _NO_PROTO
xcmap_calc_light_sat_pixel (hue, light, sat, pixelx, pixely)
float hue;
float light;
float sat;
int *pixelx;
int *pixely;
#else
xcmap_calc_light_sat_pixel (
    float hue, float light, float sat, int *pixelx, int *pixely)
#endif
{
    static float lasthue = -1.0, lastlight = -1.0, lastsat = -1.0;
    static int	lastpixelx=0, lastpixely=0;
    double	huerad, calcsin, calccos;
    int		satfactor;
    float	lightfactor;

    if (hue != lasthue || light != lastlight || sat != lastsat)
    {
	huerad = (hue/180.0) * PI;
	xcmap_calc_lightfactor (hue, light, &lightfactor);
	xcmap_calc_sincos(huerad, &calcsin, &calccos);

	satfactor = 1;
	if (calccos < 0.0)
	    satfactor = -1;
	*pixelx = (XCmapCenter + (satfactor * sat * lightfactor));
	*pixely = ((XCmapCenter+XCmapRadius) - (light * XCmapRadius * 2.0 ));

	lasthue = hue;
	lastlight = light;
	lastsat = sat;
	lastpixelx = *pixelx;
	lastpixely = *pixely;
    }
    else
    {
	*pixelx = lastpixelx;
	*pixely = lastpixely;
    }

}
/*****************************************************************************/
/*
	Draw color in both Hue-Saturation projection and Light-Saturation
	Color shown as circles at appropriate projection location
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_draw_hls_color(hue, light, sat, pixel)
float hue;
float light;
float sat;
int pixel;
#else
xcmap_draw_hls_color(float hue, float light, float sat, int pixel)
#endif
{
    int 	pixelx, pixely;
    float 	huerad;

    int		satfactor;
    float	lightfactor;
    double 	huefactor, sqroot=.5;

    /*  Draw the Hue/Saturation projection		*/
    if (XCmapInfoList[pixel].hue != hue
    ||  XCmapInfoList[pixel].light != light
    ||  XCmapInfoList[pixel].sat != sat
    || (XCmapInfoList[pixel].hsx == 0 && XCmapInfoList[pixel].hsy == 0))
    {
	xcmap_calc_hue_sat_pixel ( hue, light, sat, &pixelx, &pixely);
	XCmapInfoList[pixel].hsx = pixelx;
	XCmapInfoList[pixel].hsy = pixely;
    }
    else
    {
	pixelx = XCmapInfoList[pixel].hsx;
	pixely = XCmapInfoList[pixel].hsy;
    }

    pixelx -= PLOTPOINTSIZE;
    pixely -= PLOTPOINTSIZE;
    XFillArc  (XtDisplay(xcmap_colors), XtWindow(xcmap_colors),
	    XCmapGC, pixelx, pixely, 2*PLOTPOINTSIZE, 2*PLOTPOINTSIZE,
	    0, XFULLCIRCLE);

    /*  Draw the Saturation/Lightness projection	*/
    if (XCmapInfoList[pixel].hue != hue
    ||  XCmapInfoList[pixel].light != light
    ||  XCmapInfoList[pixel].sat != sat
    || (XCmapInfoList[pixel].lsx == 0 && XCmapInfoList[pixel].lsy == 0))
    {
	xcmap_calc_light_sat_pixel ( hue, light, sat, &pixelx, &pixely);
	XCmapInfoList[pixel].lsx = pixelx;
	XCmapInfoList[pixel].lsy = pixely;
    }
    else
    {
	pixelx = XCmapInfoList[pixel].lsx;
	pixely = XCmapInfoList[pixel].lsy;
    }
    pixelx -= PLOTPOINTSIZE;
    pixely -= PLOTPOINTSIZE;
    XFillArc  (XtDisplay(xcmap_color_cone), XtWindow(xcmap_color_cone),
	    XCmapGC, pixelx, pixely, 2*PLOTPOINTSIZE, 2*PLOTPOINTSIZE,
	    0, XFULLCIRCLE);
}
/*****************************************************************************/
/*
	Erase previous colors so new colors can be shown.  Erase by 
	redrawing colors with backgound color.
*/
/*****************************************************************************/

void
xcmap_erase_hls_colors()
{
    int pixel;

    for (pixel = XCmapIMGPixel; pixel < 256; pixel++)
    {
	XSetForeground(XtDisplay(xcmap_colors), XCmapGC, xcmap_set_pixel(XCmapBackground));
	xcmap_draw_hls_color (XCmapInfoList[pixel].hue, 
			      XCmapInfoList[pixel].light, 
			      XCmapInfoList[pixel].sat, 
			      pixel);
    }

}
/*****************************************************************************/
/*
	Show new colormap in HLS projections
*/
/*****************************************************************************/

void 
xcmap_show_hls_colors ()
{
    int pixel;

    for (pixel = XCmapIMGPixel; pixel < 256; pixel++)
    {
	XSetForeground(XtDisplay(xcmap_colors), XCmapGC, xcmap_set_pixel(pixel));
	xcmap_draw_hls_color (XCmapInfoList[pixel].hue, 
			      XCmapInfoList[pixel].light, 
			      XCmapInfoList[pixel].sat, 
			      pixel);
    }

}
/*****************************************************************************/
/*
	Draw HLS Hue-Saturation projection and Light-Saturation projection
	labels and outlines.  
*/
/*****************************************************************************/
void
xcmap_draw_hls_colormap()
{
    int		colors=256;
    int 	pixel, pixelx, pixely;
    int		blackdrawn = False;
    float 	huerad;
    static char	hexlabel[] = "RYGCBM";
    static char *lightlabel[] = { "0 ", ".5", "1 ", ".5", "0 " };

    int		satfactor;
    float	lightfactor;
    int	red, green, blue;
    float hue, light, sat;
    Display	*display;

    display = XtDisplay(xcmap_colors);
    XClearWindow(display, XtWindow(xcmap_colors));
    XClearWindow(display, XtWindow(xcmap_color_cone));
    for (pixel=0; pixel < 256; pixel++)
	XCmapCurrColors[pixel].pixel = xcmap_set_pixel(pixel);
    XQueryColors (display, XCmap, XCmapCurrColors, colors);


    for (pixel = XCmapIMGPixel; pixel < 256; pixel++)
    {
	red = XCmapCurrColors[pixel].red / 256;
	green = XCmapCurrColors[pixel].green / 256;
	blue = XCmapCurrColors[pixel].blue / 256;

	if (red == 0 && green == 0 && blue == 0)
	    if (blackdrawn)
		continue;
	    else
		blackdrawn = True;

	rgbhls (red, green, blue, &hue, &light, &sat);
	XSetForeground(XtDisplay(xcmap_color_cone), XCmapGC, xcmap_set_pixel(pixel));
	xcmap_draw_hls_color (hue, light, sat, pixel);

	XCmapInfoList[pixel].red = red;
	XCmapInfoList[pixel].green = green;
	XCmapInfoList[pixel].blue = blue;
	XCmapInfoList[pixel].hue = hue;
	XCmapInfoList[pixel].light = light;
	XCmapInfoList[pixel].sat = sat;

    }


    /*  Draw the Hexagon ouline of the HLS model	*/
    XSetForeground(XtDisplay(xcmap_colors), XCmapGC, 
		   xcmap_set_pixel(XCmapWhite));
    hue = 0.0;	/* Red	*/
    sat = 1.0;  /* Fully saturated */
    huerad = (hue/180.0) * PI;
    pixelx = XCmapCenter + (cos(huerad) * XCmapRadius * sat);
    pixely = XCmapCenter - (sin(huerad) * XCmapRadius * sat);
    XDrawString(display, XtWindow(xcmap_colors),XCmapGC,
	pixelx, pixely, &hexlabel[0], 1);
    for (hue = 60.0; hue <= 360.0; hue += 60.0)
    {
	int npixelx, npixely;
	int lpixelx, lpixely;	/* label coordinates */

	huerad = (hue/180.0) * PI;
	npixelx = XCmapCenter + (cos(huerad) * XCmapRadius * sat);
	npixely = XCmapCenter - (sin(huerad) * XCmapRadius * sat);

	XDrawLine (display, XtWindow(xcmap_colors),XCmapGC, 
			pixelx, pixely, npixelx, npixely);

	lpixelx = npixelx;
	if (hue == 180.0) lpixelx -= XCmapFontw;
	lpixely = npixely;
	if (npixely > XCmapCenter) lpixely += XCmapFonth;
	XDrawString(display, XtWindow(xcmap_colors),XCmapGC,
	    lpixelx , lpixely, &hexlabel[(int) (hue/60) ], 1);

	pixelx = npixelx;
	pixely = npixely;
    }
    /*  Draw the cone outline of the HLS model		*/
    XSetForeground(XtDisplay(xcmap_color_cone), XCmapGC, 
			xcmap_set_pixel(XCmapWhite));
    hue = 0.0;	/* Red	*/
    sat = 1.0;  /* Fully saturated */
    light = 0.5;  /* half lightness */
    huerad = (hue/180.0) * PI;
    satfactor = 1;
    pixelx = (XCmapCenter + (satfactor * XCmapRadius * sat));
    pixely = ((XCmapCenter+XCmapRadius) - (light * XCmapRadius * 2.0 ));
    for (hue = 90.0; hue <= 360.0; hue += 90.0)
    {
	int npixelx, npixely;
	int lpixelx, lpixely;	/* label coordinates */

	lpixelx = 1;
	lpixely = pixely;
	if ((hue-90.0) == 180.0) 
	    { lpixelx -= XCmapFonth; }
	if ((hue-90.0) == 270.0) lpixely += XCmapFonth;
	XDrawString(display, XtWindow(xcmap_color_cone),XCmapGC,
	    lpixelx , lpixely, lightlabel[(int) (hue/90) ], 2);

	huerad = (hue/180.0) * PI;
	sat = cos(huerad);
	light = sin(huerad) * 0.5 + 0.5;
	npixelx = (XCmapCenter + (satfactor * XCmapRadius * sat));
	npixely = ((XCmapCenter+XCmapRadius) - (light * XCmapRadius * 2.0 ));

	XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC, 
			pixelx, pixely, npixelx, npixely);

	pixelx = npixelx;
	pixely = npixely;
    }

    /*  Draw legend for saturation */
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	XCmapCenter, XCmapHeight-1, XCmapCenter + XCmapRadius, XCmapHeight-1);
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	XCmapCenter, XCmapHeight-1, XCmapCenter, XCmapHeight-1);
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	XCmapCenter+XCmapRadius/2, XCmapHeight-1, 
	XCmapCenter+XCmapRadius/2, XCmapHeight-4);
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	XCmapCenter+XCmapRadius, XCmapHeight-1, 
	XCmapCenter+XCmapRadius, XCmapHeight-6);

    XDrawString(display, XtWindow(xcmap_color_cone),XCmapGC,
	XCmapCenter-XCmapFontw, XCmapHeight-1 , "0", 1);
    XDrawString(display, XtWindow(xcmap_color_cone),XCmapGC,
	XCmapCenter+XCmapRadius, XCmapHeight-1 , "1", 1);

    /*  Draw legend for lightness */
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	0, XCmapCenter-XCmapRadius, 0, XCmapCenter + XCmapRadius);
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	0, XCmapCenter-XCmapRadius, 5, XCmapCenter - XCmapRadius);
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	0, XCmapCenter-(XCmapRadius/2), 2, XCmapCenter - (XCmapRadius/2));
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	0, XCmapCenter , 5, XCmapCenter);
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	0, XCmapCenter+(XCmapRadius/2), 2, XCmapCenter + (XCmapRadius/2));
    XDrawLine (display, XtWindow(xcmap_color_cone),XCmapGC,
	0, XCmapCenter + XCmapRadius, 5, XCmapCenter + XCmapRadius);
}
/*****************************************************************************/
/*
	Calculate absolute value for floating point variable
*/
/*****************************************************************************/
#ifdef _NO_PROTO
float xcmap_absf (val)
float val;
#else
float xcmap_absf (float val)
#endif
{
    if (val > 0.0)
	return (val);
    else
	return (-val);
}
/*****************************************************************************/
/*
	Store current colormap information 
*/
/*****************************************************************************/
xcmap_save_colors ()
{
    int pixel;

    for (pixel=0; pixel < 256; pixel++)
	XCmapOldColors[pixel] = XCmapCurrColors[pixel];
}
/*****************************************************************************/
/*
	Adjust the saturation of all colors in current hue sector
	by the saturation factor specified.
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_adjust_sat (satfactor)
float satfactor;
#else
xcmap_adjust_sat (float satfactor)
#endif
{
    int i, pixel, pixelx, pixely;
    int red, green, blue;
    float hue, light, sat;
    int hshue;
    int		colors=256;
#ifdef _NO_PROTO
    void xcmap_draw_hls_color();
    void xcmap_calc_hue_sat_pixel();
#else
    void xcmap_draw_hls_color(float hue, float light, float sat, int pixel);
    void xcmap_calc_hue_sat_pixel(
        float hue, float light, float sat, int *pixelx, int *pixely);
#endif

    for (pixel=0; pixel < 256; pixel++)
    {
	XCmapCurrColors[pixel].pixel = xcmap_set_pixel(pixel);
	XCmapCurrColors[pixel].flags = DoRed | DoGreen | DoBlue;
	XCmapCurrColors[pixel].red = XCmapInfoList [pixel].red << 8;
	XCmapCurrColors[pixel].green = XCmapInfoList [pixel].green << 8;
	XCmapCurrColors[pixel].blue = XCmapInfoList [pixel].blue << 8;
    }
    /*
    XQueryColors (XCmapDisplay, XCmap, XCmapCurrColors, colors);
    */
    for (i=XCmapIMGPixel; i < 256; i++)
    {
	red = XCmapInfoList [i].red;
	green = XCmapInfoList [i].green;
	blue = XCmapInfoList [i].blue;
	hue = XCmapInfoList [i].hue;
	light = XCmapInfoList [i].light;
	sat = XCmapInfoList [i].sat;

	hshue = (hue + 30.0) / 60;
	if (hshue == 6) hshue = 0;
	
	if ((hshue == CurrHueSector && sat > 0.0)
	|| (CurrHueSector == ALLSECTOR && sat > 0.0))
	{
	    /*
	    printf ("Adjusting color %d from %d, %d, %d, %f, %f, %f\n",
		i, red, green, blue, hue, light, sat);
	    */
	    sat = sat * satfactor;
	    if (sat > 1.0) sat = 1.0;
	    hlsrgb(hue, light, sat, &red, &green, &blue);
	    /*
	    printf ("Adjusting color %d to %d, %d, %d, %f, %f, %f\n",
		i, red, green, blue, hue, light, sat);
	    XSetForeground(XtDisplay(xcmap_color_cone), XCmapGC, 
		xcmap_set_pixel(i));
	    xcmap_draw_hls_color(hue, light, sat, i);
	    */
	    XCmapInfoList [i].red = red;
	    XCmapInfoList [i].green = green;
	    XCmapInfoList [i].blue = blue;
	    XCmapInfoList [i].sat = sat;
	    xcmap_calc_hue_sat_pixel ( hue, light, sat, &pixelx, &pixely);
	    XCmapInfoList [i].hsx = pixelx;
	    XCmapInfoList [i].hsy = pixely;
	    XCmapCurrColors[i].red = red << 8;
	    XCmapCurrColors[i].green = green << 8;
	    XCmapCurrColors[i].blue = blue << 8;
	
	}
    }
    XStoreColors (XCmapDisplay, XCmap, XCmapCurrColors, colors);
}

/*****************************************************************************/
/*
	When doing saturation adjustments, calculate the saturation factor
	by the new distance from the center of hexagon and compare to 
	old distance.  
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_calc_new_sat (pixelx, pixely)
int pixelx;
int pixely;
#else
xcmap_calc_new_sat (int	pixelx, int pixely)
#endif
{
    float satfactor;
    double currdist, newdist;
    int	dist;

    switch (CurrHueSector) {
    case RSECTOR:
    case CSECTOR:
	dist = (XCmapHS[CurrHueSector].hsx - XCmapCenter);
	currdist = (double) dist / XCmapRadius;
	dist = (pixelx - XCmapCenter);
	newdist = (double) dist / XCmapRadius;
	satfactor = newdist / currdist;
	if (satfactor < 0.0) satfactor = -satfactor;
	/*
	printf("Hue/Saturation %d, pick %d from %d\n", CurrHueSector, 
			pixelx, XCmapHS[CurrHueSector].hsx);
	printf("New saturation factor = %f\n", satfactor);
	*/
	xcmap_adjust_sat (satfactor);
	break;
    case YSECTOR:
    case GSECTOR:
    case BSECTOR:
    case MSECTOR:
    case ALLSECTOR:

	dist = (XCmapHS[CurrHueSector].hsx - XCmapCenter);
	currdist = (double) dist;
	currdist *= currdist;
	currdist += (double) ((XCmapHS[CurrHueSector].hsy - XCmapCenter) * 
		    (XCmapHS[CurrHueSector].hsy - XCmapCenter));
	currdist = sqrt (currdist);

	newdist = (pixelx - XCmapCenter) * (pixelx - XCmapCenter);
	newdist += (pixely - XCmapCenter) * (pixely - XCmapCenter);
	newdist = sqrt (newdist);

	satfactor =   newdist / currdist;

	/*
	printf("Hue/Saturation %d, pick %d from %d\n", CurrHueSector, 
			pixelx, XCmapHS[CurrHueSector].hsx);
	printf("New saturation factor = %f\n", satfactor);
	*/

	xcmap_adjust_sat (satfactor);
        break;

    }

}


/*****************************************************************************/
/*
	Adjust hue for colors in hue sector by adding huefactor
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_adjust_hue (huefactor)
float huefactor;
#else
xcmap_adjust_hue (float huefactor)
#endif
{
    int i, pixel, pixelx, pixely;
    int red, green, blue;
    float hue, light, sat;
    int hshue;
    int		colors=256;
#ifdef _NO_PROTO
    void xcmap_draw_hls_color();
    void xcmap_calc_hue_sat_pixel();
#else
    void xcmap_draw_hls_color(float hue, float light, float sat, int pixel);
    void xcmap_calc_hue_sat_pixel(
        float hue, float light, float sat, int *pixelx, int *pixely);
#endif

    for (pixel=0; pixel < 256; pixel++)
    {
	XCmapCurrColors[pixel].pixel = xcmap_set_pixel(pixel);
	XCmapCurrColors[pixel].flags = DoRed | DoGreen | DoBlue;
	XCmapCurrColors[pixel].red = XCmapInfoList [pixel].red << 8;
	XCmapCurrColors[pixel].green = XCmapInfoList [pixel].green << 8;
	XCmapCurrColors[pixel].blue = XCmapInfoList [pixel].blue << 8;
    }
    /*
    XQueryColors (XCmapDisplay, XCmap, XCmapCurrColors, colors);
    */
    for (i=XCmapIMGPixel; i < 256; i++)
    {
	red = XCmapInfoList [i].red;
	green = XCmapInfoList [i].green;
	blue = XCmapInfoList [i].blue;
	hue = XCmapInfoList [i].hue;
	light = XCmapInfoList [i].light;
	sat = XCmapInfoList [i].sat;

	hshue = (hue + 30.0) / 60;
	if (hshue == 6) hshue = 0;
	
	if (hshue == CurrHueSector || CurrHueSector == ALLSECTOR)
	{
	    /*
	    printf ("Adjusting color %d from %d, %d, %d, %f, %f, %f\n",
		i, red, green, blue, hue, light, sat);
	    */
	    hue += huefactor;
	    if (hue > 360.0) hue -= 360.0;
	    if (hue < 0.0) hue += 360.0;
	    hlsrgb(hue, light, sat, &red, &green, &blue);
	    /*
	    printf ("Adjusting color %d to %d, %d, %d, %f, %f, %f\n",
		i, red, green, blue, hue, light, sat);
	    XSetForeground(XtDisplay(xcmap_color_cone), XCmapGC, 
		xcmap_set_pixel(i));
	    xcmap_draw_hls_color(hue, light, sat, i);
	    */
	    XCmapInfoList [i].red = red;
	    XCmapInfoList [i].green = green;
	    XCmapInfoList [i].blue = blue;
	    XCmapInfoList [i].hue = hue;
	    xcmap_calc_hue_sat_pixel ( hue, light, sat, &pixelx, &pixely);
	    XCmapInfoList [i].hsx = pixelx;
	    XCmapInfoList [i].hsy = pixely;
	    XCmapCurrColors[i].red = red << 8;
	    XCmapCurrColors[i].green = green << 8;
	    XCmapCurrColors[i].blue = blue << 8;
	
	}
    }
    XStoreColors (XCmapDisplay, XCmap, XCmapCurrColors, colors);
}
/*****************************************************************************/
/*
	Calculate angle from hexagon center given x and y coordinates
*/
/*****************************************************************************/

double
xcmap_calc_xy_angle (pixelx, pixely)
{
    double	relx, rely;	/* Relative to center	*/
    double	newhue;
    int 	dist;

    dist = pixelx - XCmapCenter;
    relx = (double) dist;
    dist = XCmapCenter - pixely;        /* Make y positive in top half */
    rely = (double) dist;
    if (relx == 0) 
	if (rely > 0)
	    newhue = 90.0;
	else
	    newhue = 270.0;
    else
    {
	if (relx > 0) 
	    newhue = atan (rely / relx) * 180.0 / PI;
	else
	    newhue = atan (-rely / -relx) * 180.0 / PI + 180.0;
	if (newhue < 0.0)
	    newhue += 360.0;
    }
    return (newhue);
}
/*****************************************************************************/
/*
	Calculate difference between previous hue angle and new cursor 
	location.  Adjust colors appropriately.  Note that the user can
	select to either adjust hue for a particular hue sector or 
	rotate all hues.
*/
/*****************************************************************************/

void
#ifdef _NO_PROTO
xcmap_calc_new_hue (lpixelx, lpixely, pixelx, pixely)
int lpixelx;
int lpixely;
int pixelx;
int pixely;
#else
xcmap_calc_new_hue (int lpixelx, int lpixely, int pixelx, int pixely)
#endif
{
    float huefactor;
    double	relx, rely;	/* Relative to center	*/
    double	newhue;
    double	oldhue;

    switch (CurrHueSector) {
    default:
	oldhue = xcmap_calc_xy_angle (lpixelx, lpixely);
	newhue = xcmap_calc_xy_angle (pixelx, pixely);
	huefactor = newhue - oldhue;
	XCmapHS[CurrHueSector].hue += huefactor;
	/*
	printf("Angle at location %d, %d = %f\n", pixelx, pixely, newhue);
	*/
	xcmap_adjust_hue (huefactor);
	break;

    }

}

/*****************************************************************************/
/*
	Adjust the colors in the Light-Saturation projection based on
	lightfactor multiplier.
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_adjust_light (lightfactor)
float lightfactor;
#else
xcmap_adjust_light (float lightfactor)
#endif
{
    int i, pixel;
    int red, green, blue;
    float hue, light, sat;
    float difflight;
    int		colors=256;
#ifdef _NO_PROTO
    void xcmap_draw_hls_color();
#else
    void xcmap_draw_hls_color(float hue, float light, float sat, int pixel);
#endif

    for (pixel=0; pixel < 256; pixel++)
    {
	XCmapCurrColors[pixel].pixel = xcmap_set_pixel(pixel);
	XCmapCurrColors[pixel].flags = DoRed | DoGreen | DoBlue;
	XCmapCurrColors[pixel].red = XCmapInfoList [pixel].red << 8;
	XCmapCurrColors[pixel].green = XCmapInfoList [pixel].green << 8;
	XCmapCurrColors[pixel].blue = XCmapInfoList [pixel].blue << 8;
    }
    /*
    XQueryColors (XCmapDisplay, XCmap, XCmapCurrColors, colors);
    */
    for (i=XCmapIMGPixel; i < 256; i++)
    {
	red = XCmapInfoList [i].red;
	green = XCmapInfoList [i].green;
	blue = XCmapInfoList [i].blue;
	hue = XCmapInfoList [i].hue;
	light = XCmapInfoList [i].light;
	sat = XCmapInfoList [i].sat;

	if (CurrLightHalf == ALLLIGHT
	|| (CurrLightHalf == HIGHLIGHT && light >= .5)
	|| (CurrLightHalf == LOWLIGHT && light < .5 && light > 0.0))
	{

	    /*
	    printf ("Adjusting color %d from %d, %d, %d, %f, %f, %f\n",
		i, red, green, blue, hue, light, sat);
	    */
	    if (CurrLightHalf == ALLLIGHT)
		light *= lightfactor;
	    else
	    {
		difflight = light - .5;
		light = .5 + difflight * lightfactor;
	    }
	    if (light > 1.0) light = 1.0;
	    if (light < 0.0) light = 0.0;
	    hlsrgb(hue, light, sat, &red, &green, &blue);
	    /*
	    printf ("Adjusting color %d to %d, %d, %d, %f, %f, %f\n",
		i, red, green, blue, hue, light, sat);

	    XSetForeground(XtDisplay(xcmap_color_cone), XCmapGC, 
		xcmap_set_pixel(i));
	    xcmap_draw_hls_color(hue, light, sat, i);
	    */
	    XCmapInfoList [i].red = red;
	    XCmapInfoList [i].green = green;
	    XCmapInfoList [i].blue = blue;
	    XCmapInfoList [i].light = light;
	    XCmapCurrColors[i].red = red << 8;
	    XCmapCurrColors[i].green = green << 8;
	    XCmapCurrColors[i].blue = blue << 8;
	
	}
    }
    XStoreColors (XCmapDisplay, XCmap, XCmapCurrColors, colors);
}
/*****************************************************************************/
/*
	Calculate lightness factor from new cursor position.  Factor is
	relative distance from center of diamond.  
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_calc_new_light (lpixelx, lpixely, pixelx, pixely)
int lpixelx;
int lpixely;
int pixelx;
int pixely;
#else
xcmap_calc_new_light (int lpixelx,int lpixely, int pixelx, int pixely)
#endif
{
    float lightfactor;
    float currdist, newdist;

    switch (CurrLightHalf) {
    case ALLLIGHT:
	lightfactor = ((float) lpixely / pixely);
	break;
    case LOWLIGHT:
	lightfactor = (pixely - XCmapCenter) /
		((float)lpixely - XCmapCenter) ;
	if (lightfactor < 0.0) lightfactor = -lightfactor;
	break;
    case HIGHLIGHT:
	lightfactor = (XCmapCenter - pixely ) /
		((float)XCmapCenter - lpixely ) ;
	if (lightfactor < 0.0) lightfactor = -lightfactor;

	break;
    }
    /*
    printf("Lightness pick at %d from %d \n", pixely,
		lpixely);
    printf("New lightness factor = %f\n", lightfactor);
    */
    xcmap_adjust_light (lightfactor);

}


/*****************************************************************************/
/*
	The handles in the HLS projections represent statistical averages
	for either the huesector or the lightness half.  These are shown as
	filled rectangle.  The overall average is shown as an open rectangle.
*/
/*****************************************************************************/
void
xcmap_draw_hls_stats ()
{
    int i;
    int pixelx, pixely, lastpixelx, lastpixely;
    float hue;

    if (XCmapHSmode != 0)
    {
	for (i=0; i <= ALLSECTOR; i++)
	{
	    if (i == ALLSECTOR)
	    {
		pixelx = XCmapHS[i].hsx;
		pixely = XCmapHS[i].hsy;
	    }
	    else
	    {

		/*
		hue = XCmapHS[i].hue;
		xcmap_calc_hue_sat_pixel ( hue, .5, XCmapHS[i].sat, 
					&pixelx, &pixely);
		printf ("Hue=%d, sat = %f, x = %d, y = %d\n", 
			i, XCmapHS[i].sat, pixelx, pixely);
		XCmapHS[i].hsx = pixelx;
		XCmapHS[i].hsy = pixely;
		*/
		pixelx = XCmapHS[i].hsx;
		pixely = XCmapHS[i].hsy;
		if (pixelx == 0 && pixely == 0) 
		{
		    pixelx = XCmapCenter;
		    pixely = XCmapCenter;
		}
	    }
	    if (i > 0 && i != ALLSECTOR)
		XDrawLine  (XtDisplay(xcmap_colors), 
			XtWindow(xcmap_colors),
			XCmapGC, pixelx, pixely, 
			 XCmapHS[i-1].hsx,  XCmapHS[i-1].hsy);

	    pixelx -= HANDLESIZE+1;
	    pixely -= HANDLESIZE+1;
	    if (i == ALLSECTOR)
		XDrawRectangle  (XtDisplay(xcmap_colors), 
			XtWindow(xcmap_colors),
			XCmapGC, pixelx, pixely, 
			2*(HANDLESIZE+1), 2*(HANDLESIZE+1));
	    else
		XFillRectangle  (XtDisplay(xcmap_colors), 
			XtWindow(xcmap_colors),
			XCmapGC, pixelx, pixely, 
			2*(HANDLESIZE+1), 2*(HANDLESIZE+1));

	}

	XDrawLine  (XtDisplay(xcmap_colors), 
		XtWindow(xcmap_colors),
		XCmapGC,XCmapHS[MSECTOR].hsx, XCmapHS[MSECTOR].hsy,
		XCmapHS[0].hsx,  XCmapHS[0].hsy);
    }

    if (XCmapLSmode != 0)
    {
	for (i=0; i <= ALLLIGHT; i += 2)
	{
	    xcmap_calc_light_sat_pixel ( XCmapLS[i].hue, 
			XCmapLS[i].light, XCmapLS[i].sat,
					&pixelx, &pixely);
	    /*
	    printf ("LS#=%d, light = %f, x = %d, y = %d\n", 
			i, XCmapLS[i].light, pixelx, pixely);
	    */
	    XCmapLS[i].lsx = pixelx;
	    XCmapLS[i].lsy = pixely;
	    if (i > 0 && i != ALLLIGHT)
		XDrawLine  (XtDisplay(xcmap_color_cone), 
			XtWindow(xcmap_color_cone),
			XCmapGC, pixelx, pixely, 
			 XCmapLS[i-2].lsx,  XCmapLS[i-2].lsy);

	    pixelx -= HANDLESIZE+1;
	    pixely -= HANDLESIZE+1;
	    if (i == ALLLIGHT)
		XDrawRectangle  (XtDisplay(xcmap_color_cone), 
			XtWindow(xcmap_color_cone),
			XCmapGC, pixelx, pixely, 
			2*(HANDLESIZE+1), 2*(HANDLESIZE+1));
	    else
		XFillRectangle  (XtDisplay(xcmap_color_cone), 
			XtWindow(xcmap_color_cone),
			XCmapGC, pixelx, pixely, 
			2*(HANDLESIZE+1), 2*(HANDLESIZE+1));

	}

/*********************
	XDrawLine  (XtDisplay(xcmap_color_cone), 
		XtWindow(xcmap_color_cone),
		XCmapGC,XCmapLS[GCBSAT].lsx, XCmapLS[GCBSAT].lsy,
		XCmapLS[0].lsx,  XCmapLS[0].lsy);
*********************/
    }
}

/*****************************************************************************/
/*
*/
/*****************************************************************************/
void
xcmap_erase_hls_stats ()
{
    XSetForeground(XtDisplay(xcmap_colors), XCmapGC, 
	(XCmapBackground));
    xcmap_draw_hls_stats ();
}

/*****************************************************************************/
/*
*/
/*****************************************************************************/
void
xcmap_show_hls_stats ()
{

    XSetForeground(XtDisplay(xcmap_colors), XCmapGC, 
	xcmap_set_pixel(XCmapWhite));
    xcmap_draw_hls_stats ();
}
/*****************************************************************************/
/*
	Initialize HLS statistics information for both projections.
*/
/*****************************************************************************/
void 
xcmap_init_hls_stats ()
{
    int i;

    for (i=0; i <= MSECTOR; i++)
    {
	XCmapHS[i].hue = 0.0;
	XCmapHS[i].sat = 0.0;
	XCmapHS[i].pixels = 0;
	XCmapHS[i].hsx = 0;
	XCmapHS[i].hsy = 0;
    }

    XCmapHS[ALLSECTOR].hue = 0.0;
    XCmapHS[ALLSECTOR].sat = 0.0;
    XCmapHS[ALLSECTOR].pixels = 0;
    XCmapHS[ALLSECTOR].hsx = 0;
    XCmapHS[ALLSECTOR].hsy = 0;

    XCmapLS[ALLLIGHT].hue = 0.0;
    XCmapLS[ALLLIGHT].light = 0.0;
    XCmapLS[ALLLIGHT].sat = 0.0;
    XCmapLS[ALLLIGHT].pixels = 0;

    XCmapLS[HIGHLIGHT].hue = 0.0;
    XCmapLS[HIGHLIGHT].light = 0.0;
    XCmapLS[HIGHLIGHT].sat = 0.0;
    XCmapLS[HIGHLIGHT].pixels = 0;

    XCmapLS[LOWLIGHT].hue = 0.0;
    XCmapLS[LOWLIGHT].light = 0.0;
    XCmapLS[LOWLIGHT].sat = 0.0;
    XCmapLS[LOWLIGHT].pixels = 0;

    XCmapLS[GCBSAT].hue = 180.0;
    XCmapLS[GCBSAT].light = 0.5;
    XCmapLS[GCBSAT].sat = 0.0;
    XCmapLS[GCBSAT].pixels = 0;

    XCmapLS[MRYSAT].hue = 0.0;
    XCmapLS[MRYSAT].light = 0.5;
    XCmapLS[MRYSAT].sat = 0.0;
    XCmapLS[MRYSAT].pixels = 0;
}

/*****************************************************************************/
/*
	Find the mean for each sector by dividing by number of pixels
*/
/*****************************************************************************/
void
xcmap_avg_hls_stats ()
{
    int i;

    for (i=0; i <= ALLSECTOR; i++)
	if (XCmapHS[i].pixels > 0)
	{
	    XCmapHS[i].sat /= (XCmapHS[i].pixels * .5);
	    XCmapHS[i].hsx /= XCmapHS[i].pixels;
	    XCmapHS[i].hsy /= XCmapHS[i].pixels;
	}
	else
	{
	    XCmapHS[i].sat = 0.0;
	    XCmapHS[i].hsx = XCmapCenter;
	    XCmapHS[i].hsy = XCmapCenter;
	}

    if (XCmapLS[ALLLIGHT].pixels > 0)
	XCmapLS[ALLLIGHT].light /= XCmapLS[ALLLIGHT].pixels;

    if (XCmapLS[HIGHLIGHT].pixels > 0)
	XCmapLS[HIGHLIGHT].light /= XCmapLS[HIGHLIGHT].pixels;
    else
	XCmapLS[HIGHLIGHT].light = .5;

    if (XCmapLS[LOWLIGHT].pixels > 0)
	XCmapLS[LOWLIGHT].light /= XCmapLS[LOWLIGHT].pixels;
    else
	XCmapLS[LOWLIGHT].light = .5;

    if (XCmapLS[GCBSAT].pixels > 0)
    {
	XCmapLS[GCBSAT].sat /= XCmapLS[GCBSAT].pixels;
	XCmapLS[GCBSAT].sat /= .5;
    }
    else
	XCmapLS[GCBSAT].sat = 0.0;
    
    if (XCmapLS[MRYSAT].pixels > 0)
    {
	XCmapLS[MRYSAT].sat /= XCmapLS[MRYSAT].pixels;
	XCmapLS[MRYSAT].sat /= .5;
    }
    else
	XCmapLS[MRYSAT].sat = 0.0;

}

/*****************************************************************************/
/*
	Find differences between current colors and originals
*/
/*****************************************************************************/
void
xcmap_calc_hls_adj ()
{
    int pixel;

    float huediff;
    float hueadj = 0.0, lightadj = 0.0, satadj = 0.0;

    for (pixel = XCmapIMGPixel; pixel < 256; pixel++)
    {
      huediff = (XCmapInfoList[pixel].hue - XCmapOrigInfo[pixel].hue);
      if (xcmap_absf (huediff) > 180.0)
	  huediff = (360.0 - xcmap_absf(huediff))  
	    * (xcmap_absf(huediff) / huediff);
      hueadj += huediff;
      if (XCmapOrigInfo[pixel].light > 0.0)
	lightadj+=(XCmapInfoList[pixel].light / XCmapOrigInfo[pixel].light);
      else
	lightadj+=(XCmapInfoList[pixel].light / .000001);
      if (XCmapOrigInfo[pixel].sat > 0.0)
	satadj += (XCmapInfoList[pixel].sat / XCmapOrigInfo[pixel].sat);
      else
	 satadj += (XCmapInfoList[pixel].sat / .000001);
    }
#ifdef DEBUGPRINT
    {
	printf ("Avg hue adj = %f, light adj = %f, sat adj = %f \n",
	    hueadj / (256 - XCmapIMGPixel), lightadj / (256 - XCmapIMGPixel),
	    satadj / (256 - XCmapIMGPixel));
    }
#endif
}


/*****************************************************************************/
/*
	Calculate HLS statistics by binning pixel into correct sector and
	summing X and Y positions.
*/
/*****************************************************************************/
void
xcmap_calc_hls_stats ()
{
    int i;
    int	red, green, blue;
    float hue, light, sat;
    float lightfactor;
    int hshue, lshue, lsidx;
    int pixelx, pixely, lastpixelx, lastpixely;

    xcmap_init_hls_stats ();

    for (i=XCmapIMGPixel; i < 256; i++)
    {
	red = XCmapInfoList [i].red;
	green = XCmapInfoList [i].green;
	blue = XCmapInfoList [i].blue;
	hue = XCmapInfoList [i].hue;
	light = XCmapInfoList [i].light;
	sat = XCmapInfoList [i].sat;

	hshue = (hue + 30.0) / 60;
	if (hshue == 6) hshue = 0;
	if (hue >=90.0 && hue < 270.0)
	    lshue = GCBSAT;
	else
	    lshue = MRYSAT;

	if (light > .5)
	{
	    lsidx = HIGHLIGHT;
	    lightfactor = 1 - light;	
	}
	else
	{
	    lsidx = LOWLIGHT;
	    lightfactor = light;
	}

	if (sat != 0.0 && light != 0.0)
	{
	    XCmapHS[ALLSECTOR].hue += hue;
	    XCmapHS[ALLSECTOR].sat += (sat * lightfactor);;
	    XCmapHS[ALLSECTOR].hsx += XCmapInfoList [i].hsx;
	    XCmapHS[ALLSECTOR].hsy += XCmapInfoList [i].hsy;
	    XCmapHS[ALLSECTOR].pixels++;

	    XCmapHS[hshue].sat += (sat * lightfactor);;
	    XCmapHS[hshue].hue += hue;
	    XCmapHS[hshue].hsx += XCmapInfoList [i].hsx;
	    XCmapHS[hshue].hsy += XCmapInfoList [i].hsy;
	    XCmapHS[hshue].pixels++;

	    XCmapLS[lshue].sat += (sat * lightfactor);;
	    XCmapLS[lshue].pixels++;

	    XCmapLS[lsidx].light += light;
	    XCmapLS[lsidx].pixels++;

	    XCmapLS[ALLLIGHT].light += light;
	    XCmapLS[ALLLIGHT].pixels++;
	}
    
    }

    xcmap_avg_hls_stats ();
    xcmap_calc_hls_adj ();
}

#define DISTSQUARED(x1, y1, x2, y2) \
	((x1-x2) * (x1-x2) + (y1-y2) * (y1-y2))
#define ONHSSQUARE(hsidx) \
	(DISTSQUARED(XCmapHS[hsidx].hsx, XCmapHS[hsidx].hsy, pixelx, pixely) < HANDLEDIST)
/*****************************************************************************/
/*
	Handle button press in RGB square.  Press could either indicate 
	picking color in RGB square or picking handle in HS projection.

	XtAddEventHandler (xcmap_colors, 
		ButtonPressMask|ButtonReleaseMask|Button1MotionMask, 
		False, (XtEventHandler)xcmap_pick_color, (XtPointer) NULL);

*/
/*****************************************************************************/
#ifdef _NO_PROTO
xcmap_pick_color (w, client, report)
Widget w;
XtPointer client;
XEvent *report;
#else
xcmap_pick_color (Widget w, XtPointer client, XEvent	*report)
#endif
{

    int		pixelx, pixely, pixel;
    static int  lpixelx, lpixely;
    int		red, green, blue;
    void	xcmap_refresh_color_sample ();

    pixelx = report->xbutton.x;
    pixely = report->xbutton.y;

    if (pixelx < 0) pixelx = 0;
    if (pixely < 0) pixely = 0;
    if (pixelx >= XCmapWidth) pixelx = XCmapWidth - 1;
    if (pixely >= XCmapHeight) pixely = XCmapHeight - 1;


    if (XCmapType == RGBTYPE)
    {

	pixel = (pixelx / SQUARE) + (pixely / SQUARE) * COLORPERLINE;
	CurrPixel = XCmapInfoListPtr[pixel]->pixel;
	XmScaleSetValue (xcmap_pixel_slider, CurrPixel);
	xcmap_show_curr_pixel_color ();
    }
    else
    {
	int	i, dist, lastdist=999;
	switch (report->type) {
	case MotionNotify:
	    /*  Calculate motion vecter to determine proper effect	*/
	    /*  Saturation increase is outward from center		*/
	    /*  Saturation decrease is toward center			*/
	    /*  Drawing becomes very sloppy since a general redraw      */
	    /*  is necessary.  Will try doing an Xor draw first to see  */
	    /*  if this mitigates the redraw problem.  			*/


	    /*  Compare new saturation to previous and calculate 	*/
	    /*  transformation to apply to colors in this hue sector	*/
	    /*  Since saturation points represent an average saturation */
	    /*  apply transformation such that average moves to new     */
	    /*  position directed by mouse.  				*/

		if (CurrHueSector != -1)
		{
		    xcmap_erase_hls_stats();
		    if (XCmapHSmode == HUEMODE)
			xcmap_calc_new_hue (lpixelx, lpixely, pixelx, pixely);
		    else
		        xcmap_calc_new_sat (pixelx, pixely);
		    xcmap_calc_hls_stats ();
		    xcmap_show_hls_stats ();
		    lpixelx = pixelx;
		    lpixely = pixely;
		}

	    break;

	case ButtonPress:
	    switch (report->xbutton.button) {
	    case Button1:
		/*  Find closest saturation box 		*/
		/*  Set current hue to closest box		*/
		/*  Set state to hue selected			*/

		CurrHueSector = -1;
		if (ONHSSQUARE(ALLSECTOR))
		    CurrHueSector=ALLSECTOR;
		else
		    for (i=0; i<= MSECTOR; i++)
		    {
			if (ONHSSQUARE(i))
			    CurrHueSector = i;
		    }
		xcmap_erase_hls_colors ();
		xcmap_calc_hls_stats ();
		xcmap_show_hls_stats ();
		xcmap_save_colors();
		lpixelx = pixelx;
		lpixely = pixely;
		break;
	    case Button2:
	    case Button3:
		break;
	    }
	    break;
	case ButtonRelease:
	    switch (report->xbutton.button) {
	    case Button1:
		xcmap_draw_hls_colormap();
		xcmap_calc_hls_stats ();
		xcmap_show_hls_stats ();
		xcmap_set_color_list ();
		xcmap_show_curr_pixel_color ();
		break;
	    case Button2:
	    case Button3:
		break;
	     }
	}
    }
}
/*****************************************************************************/
/*
	Handle button press event in Light-Saturation diamond projection

	XtAddEventHandler (xcmap_color_cone, 
		ButtonPressMask|ButtonReleaseMask|Button1MotionMask, 
		False, (XtEventHandler)xcmap_pick_color_cone, (XtPointer) NULL);

*/
/*****************************************************************************/

#define ONLSSQUARE(lsidx) \
	(DISTSQUARED(XCmapLS[lsidx].lsx, XCmapLS[lsidx].lsy, pixelx, pixely) < HANDLEDIST)
#ifdef _NO_PROTO
xcmap_pick_color_cone (w, client, report)
Widget w;
XtPointer client;
XEvent *report;
#else
xcmap_pick_color_cone (Widget w, XtPointer client, XEvent	*report)
#endif
{

    int		pixelx, pixely, pixel;
    static int	lpixelx, lpixely;
    int		red, green, blue;
    void	xcmap_refresh_color_sample ();

    pixelx = report->xbutton.x;
    pixely = report->xbutton.y;


    if (pixelx < 0) pixelx = 0;
    if (pixely < 0) pixely = 0;
    if (pixelx >= XCmapWidth) pixelx = XCmapWidth - 1;
    if (pixely >= XCmapHeight) pixely = XCmapHeight - 1;

    switch (report->type) {
    case MotionNotify:
	    if (CurrLightHalf != -1)
	    {
		xcmap_erase_hls_stats();
		xcmap_calc_new_light (lpixelx, lpixely, pixelx, pixely);
		xcmap_calc_hls_stats ();
		xcmap_show_hls_stats ();
		lpixelx = pixelx;
		lpixely = pixely;
	    }

	break;

    case ButtonPress:
	switch (report->xbutton.button) {
	case Button1:
	    /*  Find closest saturation box 		*/
	    /*  Set current hue to closest box		*/
	    /*  Set state to hue selected			*/

	    CurrLightHalf = -1;
	    if (ONLSSQUARE(ALLLIGHT))
		CurrLightHalf = ALLLIGHT;
	    else
	    if (pixely < XCmapCenter)
	    {
		if (ONLSSQUARE(HIGHLIGHT))
		    CurrLightHalf = HIGHLIGHT;
	    }
	    else
		if (ONLSSQUARE(LOWLIGHT))
		    CurrLightHalf = LOWLIGHT;
	    lpixelx = pixelx;
	    lpixely = pixely;
	    xcmap_erase_hls_colors ();
	    xcmap_calc_hls_stats ();
	    xcmap_show_hls_stats ();
	    xcmap_save_colors();
	    break;
	case Button2:
	case Button3:
	    break;
	}
	break;
    case ButtonRelease:
	switch (report->xbutton.button) {
	case Button1:
	    xcmap_draw_hls_colormap();
	    xcmap_calc_hls_stats ();
	    xcmap_show_hls_stats ();
	    xcmap_set_color_list ();
	    xcmap_show_curr_pixel_color();
	    break;
	case Button2:
	case Button3:
	    break;
	 }
    }
}

/*****************************************************************************/
/*
	Save original colomap for comparison with adjusted colors
*/
/*****************************************************************************/
void
xcmap_save_orig_cmap ()
{
    int pixel;
    int		colors=256;

    for (pixel=0; pixel < 256; pixel++)
	XCmapOrigInfo [pixel] = XCmapInfoList[pixel];
}



/* ARGSUSED */
/*****************************************************************************/
/*
	Main function for drawing the colormap displays, redraws on expose
	Handles expose on both drawing areas

   	XtAddCallback(xcmap_colors, XmNexposeCallback, xcmap_draw_colors, (XtPointer)0);

 	XtAddCallback(xcmap_color_cone, XmNexposeCallback, xcmap_draw_colors, (XtPointer)0);

*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_draw_colors(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_draw_colors(Widget w, XtPointer client, XtPointer call)
#endif
{
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    XGCValues		values;
    unsigned long	valuemask = 0;
    int			pixel;
    int			pixelx, pixely;
    int			items = 0;
    Display		*display;
    XFontStruct          *font_info;

    if (XCmapGC == NULL)
    {

	XtAddEventHandler (xcmap_colors, 
		ButtonPressMask|ButtonReleaseMask|Button1MotionMask, 
		False, (XtEventHandler)xcmap_pick_color, (XtPointer) NULL);
	XtAddEventHandler (xcmap_color_cone, 
		ButtonPressMask|ButtonReleaseMask|Button1MotionMask, 
		False, (XtEventHandler)xcmap_pick_color_cone, (XtPointer) NULL);
	XCmapGC = XCreateGC (XtDisplay(w), XtWindow(w), valuemask, &values);
	XCmapDisplay = XtDisplay (w);
	font_info=XLoadQueryFont(XCmapDisplay, 
		"-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1");
	XSetFont(XCmapDisplay, XCmapGC, font_info->fid);
	XCmapFontw = font_info->max_bounds.rbearing - 
			font_info->min_bounds.lbearing;
	XCmapFonth = font_info->max_bounds.ascent + 
			font_info->max_bounds.descent;
	XtVaGetValues (w, XmNwidth, &XCmapWidth, 
			  XmNheight , &XCmapHeight, NULL);
	XCmapCenter = HALFWIDTH;
	XCmapRadius = XCmapCenter - XCmapFonth;
#ifndef INXID
	XtVaGetValues (Shell000, XtNvisual, &XCmapVisual, NULL);
#endif
	if (XCmapStatus == DEFAULT)
	{
	    XCmap = DefaultColormap(XCmapDisplay, 0);
	    find_writable_cmap();
	}
	for (pixel = 0; pixel < 256; pixel++)
	{
	    XCmapInfoListPtr [pixel] = &XCmapInfoList [pixel];
	}
	if (XCmapXIDMode)
	{
	    XCmapIMGPixel = 56;
	    XCmapColors = 200;
	}
	else
	{
	    XCmapIMGPixel = 0;
	    XCmapColors = 256;
	}
	CurrPixel = XCmapIMGPixel;

    }

    if (XCmapType == RGBTYPE)
	xcmap_draw_rgb_colormap();
    else
    {
	xcmap_draw_hls_colormap();
	xcmap_calc_hls_stats ();
	xcmap_show_hls_stats ();
    }

    if (xcmap_color_list)
	XtVaGetValues (xcmap_color_list, XmNitems, &items, NULL);

    if (items == 0)
    {
	xcmap_set_color_list ();
	xcmap_show_curr_pixel_color ();
    }
	
    
}

/* ARGSUSED */
/*****************************************************************************/
/*
	Expose now handled in xcmap_draw_colors
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_draw_color_cone(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_draw_color_cone(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    XGCValues		values;
    unsigned long	valuemask = 0;
    int			pixel;
    int			pixelx, pixely;
    Display		*display;


    /*  Depend on expose event of xcmap_colors drawing area to draw both
    if (XCmapType == HLSTYPE)
	xcmap_draw_hls_colorcone();
    */
    
}

/*****************************************************************************/
/*
	Configure display for RGB space manipulation.
		Set sliders to red, green and blue values.
		Manage RGB color square
		Unmanage HLS projections
*/
/*****************************************************************************/
void
xcmap_config_rgb ()
{
    Arg    	args[512];
    Cardinal   	argcnt;
    Boolean   	argok;
    Widget 	retval;
    XmString	xmstr[32];

    int	red, green, blue;
    float hue, light, sat;
    XColor	qcolor;

    qcolor.pixel = CurrPixel;
    XQueryColor (XCmapDisplay, XCmap, &qcolor);

    CurrRed = qcolor.red / 256;
    CurrGreen = qcolor.green / 256;
    CurrBlue = qcolor.blue / 256;

    argcnt=0;
    XtSetArg(args[argcnt], XmNmaximum, 255); argcnt++;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNdecimalPoints, 0); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, CurrRed); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"Red", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 160); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 57); argcnt++;
    XtSetValues (xcmap_red_slider, args, argcnt);
    XmStringFree( xmstr[0] );

    argcnt=0;
    XtSetArg(args[argcnt], XmNmaximum, 255); argcnt++;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNdecimalPoints, 0); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, CurrGreen); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"Green", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 160); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 57); argcnt++;
    XtSetValues (xcmap_green_slider, args, argcnt);
    XmStringFree( xmstr[0] );

    argcnt=0;
    XtSetArg(args[argcnt], XmNmaximum, 255); argcnt++;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNdecimalPoints, 0); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, CurrBlue); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"Blue", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 160); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 57); argcnt++;
    XtSetValues (xcmap_blue_slider, args, argcnt);
    XmStringFree( xmstr[0] );


    if (XtIsManaged (xcmap_color_cone))
	XtUnmanageChild (xcmap_color_cone);
    if (XtIsManaged (xcmap_hls_hs_label))
	XtUnmanageChild (xcmap_hls_hs_label);
    if (XtIsManaged (xcmap_hls_ls_label))
	XtUnmanageChild (xcmap_hls_ls_label);
    if (XtIsManaged (xcmap_huesat_radio))
	XtUnmanageChild (xcmap_huesat_radio);
    if (XtIsManaged (xcmap_lightsat_radio))
	XtUnmanageChild (xcmap_lightsat_radio);
    if (!XtIsManaged (xcmap_rgb_label))
	XtManageChild (xcmap_rgb_label);
    if (!XtIsManaged (xcmap_sort_option))
	XtManageChild (xcmap_sort_option);

}

/*****************************************************************************/
/*
	Configure display for HLS color projections
		Set sliders to hue, lightness and saturation
		Manage HLS projections and labels
		Unmanage RGB color square
*/
/*****************************************************************************/
void
xcmap_config_hls ()
{
    Arg    	args[512];
    Cardinal   	argcnt;
    Boolean   	argok;
    Widget 	retval;
    XmString	xmstr[32];

    int	red, green, blue;
    float hue, light, sat;
    XColor	qcolor;

    if (!XtIsManaged (xcmap_color_cone))
	XtManageChild (xcmap_color_cone);
    if (!XtIsManaged (xcmap_hls_hs_label))
	XtManageChild (xcmap_hls_hs_label);
    if (!XtIsManaged (xcmap_hls_ls_label))
	XtManageChild (xcmap_hls_ls_label);
    if (XtIsManaged (xcmap_rgb_label))
	XtUnmanageChild (xcmap_rgb_label);
    if (XtIsManaged (xcmap_sort_option))
	XtUnmanageChild (xcmap_sort_option);
    if (!XtIsManaged (xcmap_huesat_radio))
	XtManageChild (xcmap_huesat_radio);
    if (!XtIsManaged (xcmap_lightsat_radio))
	XtManageChild (xcmap_lightsat_radio);

    XmToggleButtonSetState (xcmap_hue_tb, False, False);
    XmToggleButtonSetState (xcmap_sat_tb, False, False);
    XmToggleButtonSetState (xcmap_light_tb, False, False);
    qcolor.pixel = CurrPixel;
    XQueryColor (XCmapDisplay, XCmap, &qcolor);

    CurrRed = qcolor.red / 256;
    CurrGreen = qcolor.green / 256;
    CurrBlue = qcolor.blue / 256;

    rgbhls (CurrRed, CurrGreen, CurrBlue, &CurrHue, &CurrLight, &CurrSat);

    argcnt=0;
    XtSetArg(args[argcnt], XmNmaximum, 3600); argcnt++;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNdecimalPoints, 1); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, (int) (CurrHue * 10)); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"Hue", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 160); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 57); argcnt++;
    XtSetValues (xcmap_red_slider, args, argcnt);
    XmStringFree( xmstr[0] );

    argcnt=0;
    XtSetArg(args[argcnt], XmNmaximum, 1000); argcnt++;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNdecimalPoints, 3); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, (int) (CurrLight * 1000)); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"Light", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 160); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 57); argcnt++;
    XtSetValues (xcmap_green_slider, args, argcnt);
    XmStringFree( xmstr[0] );

    argcnt=0;
    XtSetArg(args[argcnt], XmNmaximum, 1000); argcnt++;
    XtSetArg(args[argcnt], XmNminimum, 0); argcnt++;
    XtSetArg(args[argcnt], XmNdecimalPoints, 3); argcnt++;
    XtSetArg(args[argcnt], XmNvalue, (int) (CurrSat * 1000)); argcnt++;
    XtSetArg(args[argcnt], XmNtitleString, (xmstr[0]=XmStringCreateLtoR(
		"Sat", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 160); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 57); argcnt++;
    XtSetValues (xcmap_blue_slider, args, argcnt);
    XmStringFree( xmstr[0] );
}



/*****************************************************************************/
/*
	Find a nice gray in the set of colors to use for the background of
	the HLS projections.
*/
/*****************************************************************************/
int
#ifdef _NO_PROTO
find_gray_pixel (graylight)
float graylight;
#else
find_gray_pixel (float graylight)
#endif
{
    int i, graypixel = 0;
    float hue, light, sat;

    float dist, graydist = 9999.0;

    for (i = 0; i<256; i++)
    {
	light = XCmapInfoListPtr[i]->light;
	sat = XCmapInfoListPtr[i]->sat;

	dist = light - graylight;
	dist = dist * dist;
	dist = dist + (sat * sat);

	if (dist < graydist)
	{
	    graypixel = i;
	    graydist = dist;
	}
    }
    return (XCmapInfoListPtr[graypixel]->pixel);

}



/*****************************************************************************/
/*
	Sort by pixel number
*/
/*****************************************************************************/
int 
#ifdef _NO_PROTO
xcmap_compare_pixel (first, last)
XCmapInfoPtr *first;
XCmapInfoPtr *last;
#else
xcmap_compare_pixel (XCmapInfoPtr *first, XCmapInfoPtr *last)
#endif
{
    return (((*first)->pixel > (*last)->pixel) ? 1 :
		((*first)->pixel == (*last)->pixel) ? 0 : -1);
}

/*****************************************************************************/
/*
	Sort by red intensity
*/
/*****************************************************************************/
int 
#ifdef _NO_PROTO
xcmap_compare_red (first, last)
XCmapInfoPtr *first;
XCmapInfoPtr *last;
#else
xcmap_compare_red (XCmapInfoPtr *first, XCmapInfoPtr *last)
#endif
{
    return (((*first)->red > (*last)->red) ? 1 :
		((*first)->red == (*last)->red) ? 0 : -1);
}

/*****************************************************************************/
/*
	Sort by green intensity
*/
/*****************************************************************************/
int 
#ifdef _NO_PROTO
xcmap_compare_green (first, last)
XCmapInfoPtr *first;
XCmapInfoPtr *last;
#else
xcmap_compare_green (XCmapInfoPtr *first, XCmapInfoPtr *last)
#endif
{
    return (((*first)->green > (*last)->green) ? 1 :
		((*first)->green == (*last)->green) ? 0 : -1);
}

/*****************************************************************************/
/*
	Sort by blue intensity
*/
/*****************************************************************************/
int 
#ifdef _NO_PROTO
xcmap_compare_blue (first, last)
XCmapInfoPtr *first;
XCmapInfoPtr *last;
#else
xcmap_compare_blue (XCmapInfoPtr *first, XCmapInfoPtr *last)
#endif
{
    return (((*first)->blue > (*last)->blue) ? 1 :
		((*first)->blue == (*last)->blue) ? 0 : -1);
}

/*****************************************************************************/
/*
	Sort by hue angle
*/
/*****************************************************************************/
int 
#ifdef _NO_PROTO
xcmap_compare_hue (first, last)
XCmapInfoPtr *first;
XCmapInfoPtr *last;
#else
xcmap_compare_hue (XCmapInfoPtr *first, XCmapInfoPtr *last)
#endif
{
    return (((*first)->hue > (*last)->hue) ? 1 :
		((*first)->hue == (*last)->hue) ? 0 : -1);
}

/*****************************************************************************/
/*
	Sort by lightness
*/
/*****************************************************************************/
int 
#ifdef _NO_PROTO
xcmap_compare_light (first, last)
XCmapInfoPtr *first;
XCmapInfoPtr *last;
#else
xcmap_compare_light (XCmapInfoPtr *first, XCmapInfoPtr *last)
#endif
{
    return (((*first)->light > (*last)->light) ? 1 :
		((*first)->light == (*last)->light) ? 0 : -1);
}

/*****************************************************************************/
/*
	Sort by saturation
*/
/*****************************************************************************/
int 
#ifdef _NO_PROTO
xcmap_compare_sat (first, last)
XCmapInfoPtr *first;
XCmapInfoPtr *last;
#else
xcmap_compare_sat (XCmapInfoPtr *first, XCmapInfoPtr *last)
#endif
{
    return (((*first)->sat > (*last)->sat) ? 1 :
		((*first)->sat == (*last)->sat) ? 0 : -1);
}

/*****************************************************************************/
/*
	Configure qsort comparison functions and initialize list of 
	colors.
*/
/*****************************************************************************/
void
xcmap_sort_colors ()
{
    int		(*compare)();
    int		pixel;

    for (pixel = 0; pixel < 256; pixel++)
    {
	XCmapInfoListPtr [pixel] = &XCmapInfoList [pixel];
    }

    switch (XCmapInfoSort) {
    case 'P':
	compare = xcmap_compare_pixel;
	break;
    case 'R':
	compare = xcmap_compare_red;
	break;
    case 'G':
	compare = xcmap_compare_green;
	break;
    case 'B':
	compare = xcmap_compare_blue;
	break;
    case 'H':
	compare = xcmap_compare_hue;
	break;
    case 'L':
	compare = xcmap_compare_light;
	break;
    case 'S':
	compare = xcmap_compare_sat;
	break;
    }

    /*  Set up for Xid mode	*/
    qsort (&(XCmapInfoListPtr[XCmapIMGPixel]), XCmapColors, sizeof (XCmapInfoPtr), compare);

}
/* ARGSUSED */
/*****************************************************************************/
/*
	Sort list of colors by sort order selected

	XtAddCallback(xcmap_sort_pixel, XmNactivateCallback, 	
		xcmap_sort_rgb_colors, (XtPointer)"P|R|G|B|H|L|S");
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_sort_rgb_colors(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_sort_rgb_colors(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

	XCmapInfoSort = *((char *)client);

	xcmap_sort_colors ();

	xcmap_draw_rgb_colormap ();
}


/*****************************************************************************/
/*
	Initialize variables for X Colormap Trak window
*/
/*****************************************************************************/
void
xcmap_setup_window()
{

	XCmapGC = NULL;
	XCmapStatus = PRIVATE;
	XCmapInfoSort = 'L';
	XCmapIMGPixel = 0;
	XCmapColors = 256;
	xcmap_upd_cmap_info();
	xcmap_save_orig_cmap();
	xcmap_sort_colors ();
	find_writable_cmap();
	XCmapBackground = xcmap_set_pixel (find_gray_pixel(.75));
	XCmapWhite = xcmap_set_pixel (find_gray_pixel(1.0));
	XCmapForeground = xcmap_set_pixel (XCmapInfoListPtr[0]->pixel);

}
/* ARGSUSED */
/*****************************************************************************/
/*
	Main callback function for push button and radio buttons

	Paramater is a string specifying option selected
		Default - use default colormap (stand-alone only)
		Private - use private colormap (stand-alone only)
		RGB - Show RGB color square and sliders
		HLS - Show HLS projections and sliders
		Hue - Adjust HUE rotation
		Sat - Adjust Saturation factor
		Light - Adjust lightness
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_func(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_func(Widget w, XtPointer client, XtPointer call)
#endif
{
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    Arg    	args[512];
    Cardinal   	argcnt;
    Boolean   	argok;
    Widget 	retval;
    XmString	xmstr[32];

    int	red, green, blue;
    float hue, light, sat;
    XColor	qcolor;

#ifndef INXID
    if (strcmp (client, "Default") == 0) 
    {
	XWindowAttributes win_attr;
	int	plane_masks[1];
	int	colors[256];
	int	status;
	int	i;

	for (i=0; i<255; i++) WriteCMap [i] = False;
	XGetWindowAttributes (XCmapDisplay, XtWindow(w), &win_attr);
	status = XAllocColorCells (XCmapDisplay, win_attr.colormap, False, 
	plane_masks, 0, colors, 1);
	XCmapStatus = DEFAULT;
	while (status == 1)
	{
	    WriteCMap [colors[0]] = True;
	    status = XAllocColorCells (XCmapDisplay, win_attr.colormap, False, 
	    plane_masks, 0, colors, 1);
	}
    }
    else
    if (strcmp (client, "Private") == 0) 
    {
	Window	window;
	static Window lastwindow = NULL;
	XWindowAttributes win_attr;
	extern	Widget	Shell000;
	XColor	qcolor[256];
	int	pix;
	int	status;

	window = Select_Window (XtDisplay(w));
	printf ("Window = 0x%lx\n", window);
	XGetWindowAttributes (XtDisplay(w), window, &win_attr);
	printf ("Colormap = 0x%lx\n", win_attr.colormap);
	XSelectInput (XtDisplay(w), window, ColormapChangeMask);
	XCmapVisual = win_attr.visual;
	XCmap = XCreateColormap  (XtDisplay(w), XtWindow(w), 
			XCmapVisual,AllocNone);
	for (pix = 0; pix < 256; pix++)
	    XCmapCurrColors[pix].pixel = xcmap_set_pixel(pix);
	status = XQueryColors (XtDisplay(w), win_attr.colormap, XCmapCurrColors, 256);
	find_writable_cmap();
	if (XCmapVisual->class != TrueColor)
	    status = XStoreColors (XtDisplay(w), XCmap, XCmapCurrColors, 256);



#define REDRAW
#ifdef REDRAW
	if (lastwindow != window)
	{
	extern Widget AppShell;
	extern Widget Xcmap_bulletinboard;
	extern Widget  Createxcmap_bulletinboard(Widget);

	lastwindow = window;
	XtPopdown (Shell000);

        XtInitializeWidgetClass(topLevelShellWidgetClass);
        argcnt = 0;
        XtSetArg(args[argcnt], XmNtitle, "X Colormap Trak"); argcnt++;
        XtSetArg(args[argcnt], XmNiconName, "X Colormap Trak"); argcnt++;
        XtSetArg(args[argcnt], XmNx, 636); argcnt++;
        XtSetArg(args[argcnt], XmNy, 57); argcnt++;
	XtSetArg(args[argcnt], XtNvisual, XCmapVisual); argcnt++;
	if ((XCmapVisual->class == DirectColor)
	|| (XCmapVisual->class == TrueColor)) {
	    XtSetArg(args[argcnt], XtNdepth, 24); argcnt++;
	}
	else {
	    XtSetArg(args[argcnt], XtNdepth, 8); argcnt++;
	}

	XtSetArg(args[argcnt],XtNcolormap,XCmap); argcnt++;
        Shell000 = XtCreatePopupShell( "topLevelShell",
		       topLevelShellWidgetClass, AppShell, args, argcnt);

	XCmapGC = NULL;
	XCmapStatus = PRIVATE;
	XCmapInfoSort = 'L';
	XCmapIMGPixel = 0;
	XCmapColors = 256;
	xcmap_upd_cmap_info();
	xcmap_save_orig_cmap();
	xcmap_sort_colors ();
	XCmapBackground = xcmap_set_pixel (find_gray_pixel(.75));
	XCmapForeground = xcmap_set_pixel (XCmapInfoListPtr[0]->pixel);
	Xcmap_bulletinboard = Createxcmap_bulletinboard(Shell000);
	XtManageChild(Xcmap_bulletinboard);
	XtPopup (Shell000, XtGrabNone);
	XmToggleButtonSetState (xcmap_cmap_default, False, False);
	XmToggleButtonSetState (xcmap_cmap_private, True, False);
	if (XCmapType == HLSTYPE)
	{
	    void xcmap_config_hls();

	    xcmap_config_hls();
	    XmToggleButtonSetState (xcmap_ctype_rgb, False, False);
	    XmToggleButtonSetState (xcmap_ctype_hls, True, False);
	}
	else
	{
	    void xcmap_config_rgb();

	    xcmap_config_rgb();
	    XmToggleButtonSetState (xcmap_ctype_rgb, True, False);
	    XmToggleButtonSetState (xcmap_ctype_hls, False, False);
	}
	}
#endif
    }
    else
#endif /* INXID */
    if (strcmp (client, "RGB") == 0) 
    /*   RGB color style selected	*/
    {

	XCmapType = RGBTYPE;
	xcmap_config_rgb ();
	xcmap_set_color_list ();
	xcmap_draw_rgb_colormap ();
    }
    else
    if (strcmp (client, "HLS") == 0) 
    /*   HLS color style selected	*/
    {

	XCmapType = HLSTYPE;
	XCmapHSmode = 0;
	XCmapLSmode = 0;
        
	xcmap_config_hls ();
	xcmap_set_color_list ();
	xcmap_draw_hls_colormap ();

	xcmap_calc_hls_stats ();
	xcmap_show_hls_stats ();
    }
    else
    if (strcmp (client, "Hue") == 0) 
    {
	XCmapHSmode = HUEMODE;
	xcmap_erase_hls_colors ();

	xcmap_calc_hls_stats ();
	xcmap_show_hls_stats ();
    }
    else
    if (strcmp (client, "Sat") == 0) 
    {
	XCmapHSmode = SATMODE;
	xcmap_erase_hls_colors ();

	xcmap_calc_hls_stats ();
	xcmap_show_hls_stats ();
    }
    else
    if (strcmp (client, "Light") == 0) 
    {
	XCmapLSmode = LIGHTMODE;
	xcmap_erase_hls_colors ();

	xcmap_calc_hls_stats ();
	xcmap_show_hls_stats ();
    }

}
#define SLIDERIDLE  0
#define VALUECHANGE 1
#define DRAGSLIDER  2
static int XCmapSlideMode = SLIDERIDLE;

/*****************************************************************************/
/*
	Generic callback for processing slider change
	Client specifies character "PRGBHLS" for pixel, red, green, blue,
	hue, lightness and saturation slider. 

*/
/*****************************************************************************/

/* ARGSUSED */
void
#ifdef _NO_PROTO
xcmap_upd_slider(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_upd_slider(Widget w, XtPointer client, XtPointer call)
#endif
{
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    int	sliderval;
    int	red, green, blue;
    float	hue, light, sat;
    XColor	qcolor;

    if(*((char *)client) == 'R')
    {
	if (XCmapType == RGBTYPE) {
	    XmScaleGetValue(w, &sliderval);
	    CurrRed = sliderval;

	    if (XCmapSlideMode == VALUECHANGE)
		XCmapOldColors[CurrPixel] = XCmapCurrColors[CurrPixel];
	    XCmapCurrColors[CurrPixel].pixel = CurrPixel;
	    XQueryColor (XtDisplay(w), XCmap,&XCmapCurrColors[CurrPixel]);

	    XCmapCurrColors[CurrPixel].red = sliderval << 8;

	    XStoreColor (XtDisplay(w), XCmap, &XCmapCurrColors[CurrPixel]);
	}

    }
    else
    if (*((char *)client) == 'G')
    {
	if (XCmapType == RGBTYPE) {

	    XmScaleGetValue(w, &sliderval);
	    CurrGreen = sliderval;

	    if (XCmapSlideMode == VALUECHANGE)
		XCmapOldColors[CurrPixel] = XCmapCurrColors[CurrPixel];
	    XCmapCurrColors[CurrPixel].pixel = CurrPixel;
	    XQueryColor (XtDisplay(w), XCmap, &XCmapCurrColors[CurrPixel]);

	    XCmapCurrColors[CurrPixel].green = sliderval << 8;

	    XStoreColor (XtDisplay(w), XCmap, &XCmapCurrColors[CurrPixel]);
	}
    }
    else
    if (*((char *)client) == 'B')
    {

	if (XCmapType == RGBTYPE) {
	    XmScaleGetValue(w, &sliderval);
	    CurrBlue = sliderval;

	    if (XCmapSlideMode == VALUECHANGE)
		XCmapOldColors[CurrPixel] = XCmapCurrColors[CurrPixel];
	    XCmapCurrColors[CurrPixel].pixel = CurrPixel;
	    XQueryColor (XtDisplay(w), XCmap, &XCmapCurrColors[CurrPixel]);

	    XCmapCurrColors[CurrPixel].blue = sliderval << 8;

	    XStoreColor (XtDisplay(w), XCmap, &XCmapCurrColors[CurrPixel]);

	}
    }
    else
    if (*((char *)client) == 'P')
    {
        XmScaleGetValue(w, &sliderval);

	CurrPixel = sliderval;



	if (XCmapType == RGBTYPE) {
	    xcmap_show_curr_pixel_color ();
	}
	else
	{
	    int pixelx, pixely;
	    static int lastpixelxhue = -256, lastpixelyhue = -256; 
	    static int lastpixelxlight = -256, lastpixelylight = -256;

	    xcmap_show_curr_pixel_color ();
	    rgbhls (CurrRed, CurrGreen, CurrBlue, &hue, &light, &sat);

	    XSetForeground(XtDisplay(xcmap_color_cone), XCmapGC, xcmap_set_pixel(CurrPixel));
	    xcmap_draw_hls_color (hue, light, sat, CurrPixel);

	    /*  Track color on HLS display			*/
	    /*
	    XSetFunction(XtDisplay(xcmap_colors), XCmapGC, GXxor);
	    */
	    XSetForeground(XtDisplay(xcmap_colors), XCmapGC, 
		XCmapBackground);
	    xcmap_calc_hue_sat_pixel ( hue, light, sat, &pixelx, &pixely);
	    pixelx -= PLOTPOINTSIZE+1;
	    pixely -= PLOTPOINTSIZE+1;
	    if (lastpixelxhue != -256)
		XDrawRectangle  (XtDisplay(xcmap_colors), 
			XtWindow(xcmap_colors),
			XCmapGC, lastpixelxhue, lastpixelyhue, 
			2*(PLOTPOINTSIZE+1), 2*(PLOTPOINTSIZE+1));

	    XSetForeground(XtDisplay(xcmap_colors), XCmapGC, 
		XCmapForeground);
	    XDrawRectangle  (XtDisplay(xcmap_colors), XtWindow(xcmap_colors),
		    XCmapGC, pixelx, pixely, 
		    2*(PLOTPOINTSIZE+1), 2*(PLOTPOINTSIZE+1));

	    lastpixelxhue = pixelx;
	    lastpixelyhue = pixely;
	    /*  Draw the Saturation/Lightness projection	*/
	    xcmap_calc_light_sat_pixel ( hue, light, sat, &pixelx, &pixely);
	    pixelx -= (PLOTPOINTSIZE+1);
	    pixely -= (PLOTPOINTSIZE+1);
	    XSetForeground(XtDisplay(xcmap_colors), XCmapGC, 
		XCmapBackground);
	    if (lastpixelxlight != -256)
		XDrawRectangle  (XtDisplay(xcmap_color_cone), 
			XtWindow(xcmap_color_cone),
			XCmapGC, lastpixelxlight, lastpixelylight, 
			2*(PLOTPOINTSIZE+1), 2*(PLOTPOINTSIZE+1));

	    XSetForeground(XtDisplay(xcmap_colors), XCmapGC, 
		XCmapForeground);
	    XDrawRectangle  (XtDisplay(xcmap_color_cone), 
		    XtWindow(xcmap_color_cone),
		    XCmapGC, pixelx, pixely, 
		    2*(PLOTPOINTSIZE+1), 2*(PLOTPOINTSIZE+1));
	    XSetFunction(XtDisplay(xcmap_colors), XCmapGC, GXcopy);

	    lastpixelxlight = pixelx;
	    lastpixelylight = pixely;

        }

	if (WriteCMap [ CurrPixel])
	{
	    Arg	args[1];

	    XtSetArg (args[0], XmNsensitive, True);
	    XtSetValues (xcmap_red_slider, args, 1);
	    XtSetValues (xcmap_green_slider, args, 1);
	    XtSetValues (xcmap_blue_slider, args, 1);
	}
	else
	{
	    Arg	args[1];

	    XtSetArg (args[0], XmNsensitive, False);
	    XtSetValues (xcmap_red_slider, args, 1);
	    XtSetValues (xcmap_green_slider, args, 1);
	    XtSetValues (xcmap_blue_slider, args, 1);

	}

	if (xcmap_color_list)
	{
	    XmListSetPos (xcmap_color_list, CurrPixel+1);
	    XmListSelectPos (xcmap_color_list, CurrPixel+1, False);
	}
    }

    if (*((char *)client) != 'P')
      if ( XCmapType == HLSTYPE)
      {
	int	hueval, lightval, satval;
	XmString	xmstr[1];
	char		liststr[240];

	XmScaleGetValue (xcmap_red_slider, &hueval);
	XmScaleGetValue (xcmap_green_slider, &lightval);
	XmScaleGetValue (xcmap_blue_slider, &satval);

	CurrHue = hue = hueval / 10.0;
	CurrLight = light = lightval / 1000.0;
	CurrSat = sat = satval / 1000.0;

	hlsrgb (hue, light, sat, &red, &green, &blue);
    
	if (XCmapSlideMode == VALUECHANGE)
	    XCmapOldColors[CurrPixel] = XCmapCurrColors[CurrPixel];
	CurrRed = XCmapCurrColors[CurrPixel].red = red << 8;
	CurrGreen = XCmapCurrColors[CurrPixel].green = green << 8;
	CurrBlue = XCmapCurrColors[CurrPixel].blue = blue << 8;
	XCmapCurrColors[CurrPixel].pixel = CurrPixel;
	XCmapCurrColors[CurrPixel].flags = DoRed | DoGreen | DoBlue;

	XStoreColor (XtDisplay(w), 
	    XCmap,&XCmapCurrColors[CurrPixel]);
	XSetForeground(XtDisplay(xcmap_color_cone), XCmapGC, xcmap_set_pixel(CurrPixel));
	xcmap_draw_hls_color (hue, light, sat, CurrPixel);

	XCmapInfoList[CurrPixel].red = red;
	XCmapInfoList[CurrPixel].green = green;
	XCmapInfoList[CurrPixel].blue = blue;
	XCmapInfoList[CurrPixel].hue = hue;
	XCmapInfoList[CurrPixel].light = light;
	XCmapInfoList[CurrPixel].sat = sat;

        xcmap_format_list_entry (CurrPixel, liststr);

	if (xcmap_color_list)
	{
	    xmstr[0] = XmStringCreateLtoR(liststr, XmSTRING_DEFAULT_CHARSET);
	    XmListReplaceItemsPos(xcmap_color_list,&xmstr[0], 1, CurrPixel + 1);
	    XmStringFree (xmstr[0]);
	}
      }
      else
      {
	XmString	xmstr[1];
	char		liststr[240];

	XCmapInfoList[CurrPixel].red = CurrRed;
	XCmapInfoList[CurrPixel].green = CurrGreen;
	XCmapInfoList[CurrPixel].blue = CurrBlue;
	rgbhls (CurrRed, CurrGreen, CurrBlue, &CurrHue, &CurrLight, &CurrSat);
	XCmapInfoList[CurrPixel].hue = CurrHue;
	XCmapInfoList[CurrPixel].light = CurrLight;
	XCmapInfoList[CurrPixel].sat = CurrSat;

        xcmap_format_list_entry (CurrPixel, liststr);

	if (xcmap_color_list)
	{
	    xmstr[0] = XmStringCreateLtoR(liststr, XmSTRING_DEFAULT_CHARSET);
	    XmListReplaceItemsPos(xcmap_color_list,&xmstr[0], 1, CurrPixel + 1);
	    XmStringFree (xmstr[0]);
	}
      }
}

/* ARGSUSED */
/*****************************************************************************/
/*
	Generic callback for dragging any slider
	Client specifies character "PRGBHLS" for pixel, red, green, blue,
	hue, lightness and saturation slider.  

	XtAddCallback(xcmap_pixel_slider, XmNdragCallback, xcmap_slider, 	
		(XtPointer)"P");
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_slider(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_slider(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

	if (XCmapSlideMode != DRAGSLIDER)
	{
	    if(*((char *)client) != 'P')
		XCmapOldColors[CurrPixel] = XCmapCurrColors[CurrPixel];
	}
      	XCmapSlideMode = DRAGSLIDER;
	xcmap_upd_slider (w, client, call);
}


/* ARGSUSED */
/*****************************************************************************/
/*
	Generic callback for setting any slider
	Client specifies character "PRGBHLS" for pixel, red, green, blue,
	hue, lightness and saturation slider. 

	XtAddCallback(xcmap_pixel_slider, XmNvalueChangedCallback, 
		xcmap_change_slider, (XtPointer)"P");
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_change_slider(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_change_slider(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

	if (XCmapSlideMode != DRAGSLIDER)
	{
	    if(*((char *)client) != 'P')
		XCmapOldColors[CurrPixel] = XCmapCurrColors[CurrPixel];
	    XCmapSlideMode = VALUECHANGE;
	}
	else
	{
	    XCmapSlideMode = SLIDERIDLE;
	}
	xcmap_upd_slider (w, client, call);
}

/* ARGSUSED */
/*****************************************************************************/
/*
	The function xcmap_select_color is the XmNsingleSelectionCallback
	for the xcmap_color_list list widget which displays the color map 
	in the selected sort order.  Selecting an element from the list
	chooses that pixel as the current pixel.  Current pixel is outlined
	by a square on HLS projections and in drawn last to make sure it
	is visible.

	XtAddCallback(xcmap_color_list, XmNsingleSelectionCallback, 		
		xcmap_select_color, (XtPointer)0);
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_select_color(w, client, call)
Widget w;
XtPointer client;
XmListCallbackStruct *call;
#else
xcmap_select_color(Widget w, XtPointer client, XmListCallbackStruct * call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

    CurrPixel = call->item_position-1;

    XmScaleSetValue (xcmap_pixel_slider, CurrPixel);
    xcmap_show_curr_pixel_color ();
}

/*****************************************************************************/
/*
	Redraw with new colors after UNDO
*/
/*****************************************************************************/
void
xcmap_upd_state ()
{
    int items;

    xcmap_save_colors();
    if (XCmapType == RGBTYPE)
	xcmap_draw_rgb_colormap();
    else
    {
	xcmap_draw_hls_colormap();
	xcmap_calc_hls_stats ();
	xcmap_show_hls_stats ();
    }

    if (xcmap_color_list)
	XtVaGetValues (xcmap_color_list, XmNitems, &items, NULL);

    xcmap_set_color_list ();
    xcmap_show_curr_pixel_color ();

}

#define UNDO 1
#define REDO 2
static XCmapUndoState = 1;
/* ARGSUSED */
/*****************************************************************************/
/*
	Restore colors to previous color map. 

	XtAddCallback(xcmap_undo_button, XmNactivateCallback,
		 xcmap_undo, (XtPointer)0);
 
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_undo(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_undo(Widget w, XtPointer client, XtPointer call)
#endif
{
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;
    int colors  = 256;

    if (XCmapUndoState == UNDO)
    {
	XCmapUndoState = REDO;
	XStoreColors (XCmapDisplay, XCmap, XCmapOldColors, colors);
    }
    else
    {
	XCmapUndoState = UNDO;
	XStoreColors (XCmapDisplay, XCmap, XCmapOldColors, colors);
    }
    xcmap_upd_state();
}
/*****************************************************************************/
/*
	Draw color square of current color on expose or when current color
	pixel changes.
*/
/*****************************************************************************/
void
xcmap_refresh_color_sample ()
{

    XSetForeground (XtDisplay(xcmap_color_sample), 
			XCmapGC, xcmap_set_pixel(CurrPixel));
    XFillRectangle (XtDisplay(xcmap_color_sample), 
	XtWindow(xcmap_color_sample), XCmapGC, 0,0,60,60);
    
}
/* ARGSUSED */
/*****************************************************************************/
/*
	Draw color square of current color on expose

	 XtAddCallback(xcmap_color_sample, XmNexposeCallback, 			
		xcmap_draw_color_sample, (XtPointer)0);

*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_draw_color_sample(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_draw_color_sample(Widget w, XtPointer client, XtPointer call)
#endif
{
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

    
    if (XCmapGC != NULL)
    {
	xcmap_refresh_color_sample ();
    }
}

/* ARGSUSED */
/*****************************************************************************/
/*
	Dismiss button callback, exit if standalone, unmanage in Xid

	XtAddCallback(xcmap_dismiss_button, XmNactivateCallback, 
		xcmap_dismiss, (XtPointer)0);
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_dismiss(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_dismiss(Widget w, XtPointer client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

#ifdef INXID
Arg         args[2];
int         argcnt;
Boolean     argok;
XmString    xmstr[256];

    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	"Show Map", XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_show_map_button, args, argcnt);
    XmStringFree (xmstr[0]);
    XtUnmanageChild (ct_cmap_mainwindow);
    XtPopdown (xidCMap);

#else
    exit (0);
#endif
}

#define RESTORED 1
#define ISOLATED 2
/* ARGSUSED */
/*****************************************************************************/
/*
	Isolate current pixel by setting all others to black, change
	label on "Isolate" button to "Restore".  
	On restore, reset colors to original values.  

	XtAddCallback(xcmap_isolate_button, XmNactivateCallback, 
		xcmap_isolate, (XtPointer)0);

*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
xcmap_isolate(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_isolate(Widget w, XtPointer client, XtPointer call)
#endif
{
    XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

    XColor	qcolor[256];
    int		colors=256;
    int		pixel;
    int		status;

    Arg         args[2];
    int         argcnt;
    Boolean     argok;
    XmString    xmstr[256];



    static int isolate_flag = RESTORED;

    if (isolate_flag == RESTORED)
    {
	for (pixel=0; pixel < 256; pixel++)
	    qcolor[pixel].pixel = xcmap_set_pixel(pixel);
	XQueryColors (XtDisplay(xcmap_colors), XCmap, qcolor, colors);
	isolate_flag = ISOLATED;
	for (pixel = XCmapIMGPixel; pixel < 256; pixel++)
	{
	    if (pixel != CurrPixel)
	    {
		qcolor[pixel].red=qcolor[pixel].green=qcolor[pixel].blue = 0;

	    }
	}
	status = XStoreColors (XtDisplay(w), XCmap, qcolor, 256);

      argcnt=0;
      XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
            "Restore", XmSTRING_DEFAULT_CHARSET))); argcnt++;
      XtSetValues (w, args, argcnt);
      XmStringFree (xmstr[0]);

    }
    else
    {
	for (pixel=0; pixel < 256; pixel++)
	    qcolor[pixel].pixel = xcmap_set_pixel(pixel);
	XQueryColors (XtDisplay(xcmap_colors), XCmap, qcolor, colors);
	isolate_flag = RESTORED;
	for (pixel = XCmapIMGPixel; pixel < 256; pixel++)
	{
	    qcolor[pixel].red = XCmapInfoList[pixel].red * 256;
	    qcolor[pixel].green = XCmapInfoList[pixel].green * 256;
	    qcolor[pixel].blue = XCmapInfoList[pixel].blue * 256;

	}
	status = XStoreColors (XtDisplay(w), XCmap, qcolor, 256);

      argcnt=0;
      XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
            "Isolate", XmSTRING_DEFAULT_CHARSET))); argcnt++;
      XtSetValues (w, args, argcnt);
      XmStringFree (xmstr[0]);

    }
}
/*****************************************************************************/
/*
	Create RGB selection list dialog.  Manage and unmanage to show and 
	hide display.

	XtAddCallback(xcmap_show_rgblist_pb, XmNactivateCallback, 		
		xcmap_show_rgblist, (XtPointer)0);

*/
/*****************************************************************************/
void 
#ifdef _NO_PROTO
xcmap_show_rgblist (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
xcmap_show_rgblist (Widget w, XtPointer client, XtPointer call)
#endif
{
    Arg         args[256];
    int         argcnt;
    Boolean     argok;
    XmString    xmstr[256];
    extern	Widget Createxcmap_rgblist_main();

    if (xcmap_rgblist_shell == NULL)
    {

	XtInitializeWidgetClass(topLevelShellWidgetClass);
	argcnt = 0;
	XtSetArg(args[argcnt], XmNtitle, "X Colormap List"); argcnt++;
	XtSetArg(args[argcnt], XmNiconName, "X Colormap List"); argcnt++;
	XtSetArg(args[argcnt], XtNvisual, XCmapVisual); argcnt++;
	XtSetArg(args[argcnt], XtNcolormap, XCmap); argcnt++;
	if ((XCmapVisual->class == DirectColor)
	|| (XCmapVisual->class == TrueColor)) {
	    XtSetArg(args[argcnt], XtNdepth, 24); argcnt++;
	}
	else {
	    XtSetArg(args[argcnt], XtNdepth, 8); argcnt++;
	}
#ifdef INXID
	xcmap_rgblist_shell = XtCreatePopupShell( "xcmap_rgblist_shell", 
	    topLevelShellWidgetClass, AppShell, args, argcnt);
#else
	xcmap_rgblist_shell = XtCreatePopupShell( "xcmap_rgblist_shell", 
	    topLevelShellWidgetClass, Shell000, args, argcnt);
#endif

	xcmap_rgblist_main = Createxcmap_rgblist_main(xcmap_rgblist_shell);

	XtManageChild(xcmap_rgblist_main);
	XtPopup(XtParent(xcmap_rgblist_main), XtGrabNone);

	xcmap_set_color_list ();
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Hide Colormap List", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
    }
    else
    if (!XtIsManaged (xcmap_rgblist_main))
    {
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Hide Colormap List", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);

	XtManageChild(xcmap_rgblist_main);
	XtPopup(XtParent(xcmap_rgblist_main), XtGrabNone);
    }
    else
    {
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Colormap List", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);

	XtUnmanageChild(xcmap_rgblist_main);
	XtPopdown(XtParent(xcmap_rgblist_main));

    }


}
