/*
 *  This is the main include file for the X Image Display application.  It
 *  contains references for all other include files required by the application
 *  and is designed to be called once in each source file.
 *
 *  by D. Akkerman/L. Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

/*
 * Required MOTIF include files.
 */
#include <X11/Xlib.h>
#include <X11/StringDefs.h>

#include <Xm/Xm.h>
#include <X11/Shell.h>
#include <X11/keysym.h>
#include <Xm/ArrowB.h>
#include <Xm/ArrowBG.h>
#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>
#include <Xm/CascadeBG.h>
#include <Xm/Command.h>
#include <Xm/CutPaste.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/DrawnB.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/List.h>
#include <Xm/MainW.h>
#include <Xm/MenuShell.h>
#include <Xm/MessageB.h>
#include <Xm/PanedW.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>
#include <Xm/SelectioB.h>
#include <Xm/SeparatoG.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/TextF.h>
#include <X11/extensions/multibuf.h>

/*
    Declaration of all widget variables.
*/
#include "widget.h"
#include "las.h"
#include "dltf.h"
#include "xid_graphics.h"

/*******************************************************************************
                          Application Defines
*******************************************************************************/
/*
    Macros.
*/
#define DEGTORAD(x)     (x)/57.29577    /* Degrees to radians function.       */

/*
    Working debug flag.
*/

#define XID_DEBUG

/*
    Application name and class constants.
*/
#define BX_APP_NAME "xid"
#define BX_APP_CLASS "XId"

/*
    General constants.
*/
#define NAMLEN          CMLEN   /* Maximum file name length.                  */
#define XID_S           E_SUCC  /* Success!                                   */
#define XID_F           E_FAIL  /* Failure ...                                */
#define NUM_COLORS      8       /* Total number of working colors.            */
#define BLACK_COL       0       /* Indices into the working color array.      */
#define WHITE_COL       1
#define RED_COL         2
#define GREEN_COL       3
#define BLUE_COL        4
#define YELLOW_COL	5
#define MAGENTA_COL	6
#define CYAN_COL	7
#define NUM_WMCOLORS	12	/* # of colorcells used to minimize flashing. */
#define WM_IDX		0	/* Index into color table for window manager
				   colors.				      */
#define NUM_CVALS	8	/* # of colorcells used for drawing colors.   */
#define RGBBW_IDX	12	/* Index into color table for drawing colors. */
#define NUM_WKCOLOR	1	/* # of work colorcells for PSD/MASK.	      */
#define WORK_IDX	20	/* Index into color table for work color cell.*/
#define NUM_CSTORE	35	/* # of colorcells in color store.	      */
#define CSTORE_IDX	21	/* Index into color table for stored colors.  */
#define NUM_IMGCOLORS	200	/* # of image data colorcells.		      */
#define	IMG_IDX		56	/* Index into color table for image colors.   */
#define RGB_DATABASE	"/usr/lib/X11/rgb.txt"
				/* X RGB color database.		      */
#define NUM_COMPOSITE	50	/* Number of composite views to hold
				   information for.			      */

#define MAXCOLOR 65535		/* Maximum color value.			      */
#define MINCOLOR 0		/* Minimum color value.			      */

/* 
    Boolean values.
*/
#ifndef TRUE
#define	FALSE		0
#define	TRUE		1
#endif

/*
    Data types.
*/
#define BYTE		EBYTE
#define I2		EWORD
#define I4		ELONG
#define REAL		EREAL

/*
    File formats.
*/
#define BSQ		0
#define BIL		1

/*
    Array indices.
*/
#define	RED		0
#define GREEN		1
#define BLUE		2
#define BW		3

/*
    Define XID error types.
*/
#define XID_ERROR	0
#define XID_WARNING	1
#define XID_QUESTION	2
#define XID_INFORMATION 3

/* 
    Projection Information Package defines.
*/
#define DEGREES         4       /* Output projection units.             */
#define CLARKE          0       /* Output datum code; Clarke 1866.      */
#define NULZONE         62      /* Output zone code; GCTP constant that
				   signifies a null zone.               */

/*
    Default image plane size and number and viewport size.
*/
#define	DEFC_HEIGHT	"1024"
#define	DEFC_WIDTH	"1024"
#define	DEFC_VHEIGHT	" 512"
#define	DEFC_VWIDTH	" 512"
#define	DEFC_NPLANE	"   3"

#define	DEF_HEIGHT	1024
#define	DEF_WIDTH	1024
#define	DEF_VHEIGHT	 512
#define	DEF_VWIDTH	 512
#define	DEF_NPLANE	   3

/*
    Image types.
*/
#define	LAS	0x1		/* LAS 5.0 images.			      */
#define NONLAS	0x2		/* NonLAS image files.			      */

/*
    Bit flags.
*/
#define	USED	0x1		/* Has data been loaded into plane?	      */
#define LOCK	0x2		/* Is the data in this plane locked?	      */

#define XID_HORIZ	0x1	/* Indicates horizontal tracking line.	      */
#define XID_VERT	0x2	/* Indicates vertical tracking line.	      */
#define XID_COLMSG	0x4	/* Indicates the colors warning message is 
				   to be output when an image is loaded.      */

/*
    Color compression algorithms.
*/
#define	MEDCUT	0x1		/* Median cut algorithm.    		      */
#define	DISMAT	0x2		/* Distribution matrix algorithm.	      */

/*
    Mouse context defines.
*/
#define CON_IMAGE	0	/* Index to image context name.		      */
#define CON_IMAGE2	1	/* Index to image context name (cursor).      */
#define CON_GRSEL	2	/* Index to graphics select context name.     */
#define CON_GREDIT	3	/* Index to graphics edit context name.       */
#define CON_GRMVDEL	4	/* Index to graphics move/delete edit context 
				   name.       				      */
#define CON_GRADD	5	/* Index to graphics add edit context name.   */
#define CON_PROFILE	6	/* Index to profile context name (manual).    */
#define CON_FILL	7	/* Index to fill context name.    	      */

/*
    Histogram defines.
*/
#define HISTO_ACTIVE	0x1	/* Histogram active flag.		      */

#define	HISTO_IMAGE	0x2    	/* Histogram entire image.		      */
#define	HISTO_LINE	0x4    	/* Histogram a single line of an image.       */
#define	HISTO_POLY	0x8    	/* Histogram a polygon within an image.       */

#define	HISTO_DIST	0x10    /* Distributional histogram.		      */
#define HISTO_CUM	0x20    /* Cumulative histogram.		      */

#define	HISTO_XY	0x40	/* Histogram dimensions.		      */
#define HISTO_XZ	0x80
#define HISTO_YZ	0x100

#define	HISTO_MAPPED	0x200	/* Histogram mapped gray level values.	      */
#define HISTO_ORIG	0x400	/* Histogram original gray level values.      */

#define HISTX   	475	/* Offsets for drawing histogram graphs.      */
#define HISTY   	353


/*
    Cursor defines.
*/
#define CURSOR_ACTIVE	0x1	/* Cursor active flag.		              */

#define	CURSOR_XY	0x2	/* Display values in XY dimension.	      */
#define	CURSOR_Z	0x4	/* Display values in Z dimension.	      */

#define	CURSOR_MAPPED	0x8	/* Display mapped gray level values.	      */
#define CURSOR_ORIG	0x10	/* Display original gray level values.        */

#define	CURSOR_AUTO	0x20	/* Update requested information automatically.*/
#define	CURSOR_MAN	0x40	/* Update requested information based on user
				   request.				      */

#define CURSOR_PROJ	0x80	/* Display projection coordinates.	      */
#define CURSOR_LATLONG 	0x100	/* Display latitude/longitude values.	      */
#define CURSOR_UNSCAL	0x200	/* Display unscaled data values.	      */

#define	CURSOR_HEIGHT	 70	/* Height of cursor information drawing area. */
#define CURSOR_WIDTH	434	/* Width of cursor information drawing area.  */

/*
#define CURSOR_FONT "-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1"
*/
#define	INFO_FONT	"8x13bold"
				/* Font for general information.	      */
#define POINTER_FONT	"6x10"	/* Font for pointer information.	      */

/*
    Full resolution window defines.
*/
#define FULLRES_ACTIVE	0x1	/* Full resolution window active flag.	      */
#define FULLRES_USER	0x2	/* User entered window size flag.	      */
#define FULLRES_SHOW	0x4	/* Show outline flag.	      		      */

/*
    General Map defines.
*/
#define	MAP_RED		0x1	/* Map red component of the image.            */
#define MAP_GREEN	0x2	/* Map green component of the image.          */
#define	MAP_BLUE	0x4	/* Map blue component of the image.           */
#define	MAP_ALL		0x8	/* Map all components of the image.           */
#define	MOD_RED		0x10	/* Modified red mapping of the image.         */
#define MOD_GREEN	0x20	/* Modified green mapping of the image.       */
#define	MOD_BLUE	0x40	/* Modified blue mapping of the image.        */

/*
    Pseudocolor defines.
*/
#define	PSD_LIST	0x1	/* Select colors from a predefined color list.*/
#define	PSD_SLIDER	0x2	/* Select colors by combining red, green and 
				   blue components using sliders.	      */
#define	PSD_INIT	0x4	/* Initialize the PSD color table array.      */
#define	PSD_NOINIT	0x8	/* Don't initialize the PSD color table array.*/

/*
    Profile defines.
*/
#define PROF_ACTIVE	0x1	/* Profile active flag.	      		      */

#define	PROF_AUTO	0x2	/* Update profile automatically.	      */
#define	PROF_MAN	0x4	/* Update profile based on user request.      */

#define	PROF_X		0x8	/* Profile dimensions.			      */
#define	PROF_Y		0x10
#define	PROF_Z		0x20
#define	PROF_XY		0x40

#define PROF_CONT	0x80	/* Draw a continuous profile line.	      */
#define PROF_DIS	0x100	/* Draw discrete points.		      */

/*
    Init defines.
*/
#define	INIT_MAIN	0x1	/* Initialize main window.		      */
#define	INIT_PLANE	0x2	/* Initialize a single plane.		      */
#define	INIT_MAP	0x4	/* Initialize the mapping.		      */

#define INIT_ALL	0x8	/* Initialize all planes.		      */

/*
    Zoom defines.
*/
#define ZOOM_ACTIVE	0x1	/* Zoom window active flag.	      	      */

#define	ZOOM_NN		0x2	/* Nearest neighbor.			      */
#define	ZOOM_BI		0x4	/* Bilinear.				      */
#define	ZOOM_CC		0x8	/* Cubic convolution.			      */

#define	ZOOM_POINT	0x10	/* Move to point and click.		      */
#define	ZOOM_CONT	0x20	/* Continuous zoom.			      */

#define ZOOM_MAIN	0x40	/* Main window is zoomed.		      */
#define ZOOM_FULLRES	0x80	/* Full resolution window is zoomed.	      */

#define ZOOM_WINX	100	/* Number of samples in zoom window.	      */
#define ZOOM_WINY	100	/* Number of lines in zoom window.	      */

/*
    Mask defines.
*/
#define	MASK_LIST	0x1	/* Select colors from a predefined color list.*/
#define	MASK_SLIDER	0x2	/* Select colors by combining red, green and 
				   blue components using sliders.	      */

#define MASK_ALL	0x4	/* Clear selected graphics plane.	      */
#define MASK_MAX	4	/* Maximum number of graphics plane masks.    */

/*
    Power Spectrum defines.
*/
#define POWER_ACTIVE	0x1	/* Power spectrum active flag.	      	      */

#define	POWER_AUTO	0x2	/* Update plot automatically.	              */
#define	POWER_MAN	0x4	/* Update plot based on user request.         */

#define	POWER_X		0x8	/* Plot dimensions.			      */
#define	POWER_Y		0x10
#define	POWER_Z		0x20
#define	POWER_XY	0x40

#define POWER_CONT	0x80	/* Draw a continuous line.	      	      */
#define POWER_DIS	0x100	/* Draw discrete points.		      */

#define POWER_ENT	0x200	/* Maximum entropy method.		      */
#define POWER_PER	0x400	/* Periodogram.			 	      */

#define POWER_MINDB	0	/* Minimum decibel value.		      */
#define POWER_MAXDB	50	/* Maximum decibel value.		      */

#define	POWER_SCROLLW	481	/* Original scrolled window width.	      */
#define POWER_SCROLLH	403	/* Original scrolled window height.	      */

#define	POWER_SHELLW	860	/* Original form width.	      		      */
#define POWER_SHELLH	450	/* Original form height.	      	      */

/*
    Flicker defines.
*/
#define FLICKER_MAX	20	/* Maximum number of images that may be
				   flickered.				      */
#define FLICKER_ACTIVE	0x1	/* Flicker window active flag.		      */
#define FLICKER_ING	0x2	/* Currently flickering.		      */
#define FLICKER_MULTI 	0x4	/* Indicates whether or not the multi-buffering
				   extension is available.		      */
#define FLICKER_BUF 	0x8	/* Indicates whether or not the multi-buffering
				   buffers have been created.		      */
#define FLICKER_SEL	0x10	/* Indicates whether or not an item has been
				   selected from the red, green or blue list
				   or from the save list.		      */

/*
    Save defines.
*/
#define SAVE_ACTIVE 	0x1	/* Save window active flag.		      */
#define SAVE_MAIN	0x2	/* Save the main image window.		      */
#define SAVE_FULL	0x4	/* Save the full resolution image window.     */
#define SAVE_ZOOM	0x8	/* Save the zoom image window.		      */
#define SAVE_ALL	0x10	/* Save all loaded image data.		      */
#define SAVE_VIEW	0x20	/* Save viewed image data.		      */
#define SAVE_FRAME	0x40	/* Save the window manager frame.	      */
#define SAVE_NOFRAME	0x80	/* Do not save the window manager frame.      */
#define SAVE_ERROR	0x100	/* Indicates an error has occurred.	      */

/*
    Fill defines.
*/
#define FILL_ACTIVE	0x1	/* Fill active flag.		      	      */
#define	FILL_INTERNAL	0x2    	/* Fill inside polygons.		      */
#define	FILL_EXTERNAL	0x4    	/* Fill outside polygons.       	      */
#define	FILL_USER	0x8    	/* Fill only user-selected polygons.	      */
#define	FILL_ALL	0x10    /* Fill all polygons. 			      */
#define FILL_PWL	0x20    /* Apply a piece-wise linear mapping to the
				   filled area.				      */
#define	FILL_RENUMBER	0x40	/* Renumber specified values of filled area.  */
#define FILL_CONSTANT	0x80	/* Fill with a constant values.		      */
#define FILL_MAIN	0x100	/* Fill in the main image window.	      */
#define FILL_FULLRES	0x200	/* Fill in the full resolution image window.  */
#define FILL_UPDATE	0x400	/* Update the displayed image.		      */
#define FILL_NEW	0x800	/* Generate a new image.		      */

#define FILL_MAXBREAK	100	/* Initial maximum number of breakpoints.     */
#define FILL_MAXPOLY	100	/* Initial maximum number of polygons.        */
#define FILL_MAXISECT   500	/* Initial maximum number of intersection 
				   points.     				      */

/*
    Help defines.
*/
#define HELP_ACTIVE	0x1	/* Help window active flag.	      	      */

/*
    Utility defines.
*/
#define MAX_COLORS	1000	/* Maximum # of colors available in the
				   color list.				      */

/*******************************************************************************
                  	"Generic" Structures
*******************************************************************************/

/*
    Main application structure.
*/
struct XID_STRUCT
 {
 Display	*display;	/* Pointer for X display structure.	      */
 Pixmap		icon_pixmap;	/* Pixmap for the application icon. 	      */
 Colormap	colormap;	/* Working color map.		   	      */
 XVisualInfo	*visual_info;	/* Visual information structure.	      */
 Visual		*visual;	/* Default visual.			      */
 XColor		color_linear[256];
				/* The linear color table (calculated at load
				   time or display time).		      */
 XColor		color_table[256];
				/* The current color table.		      */
 Colormap	imagemap;	/* Working image color map.		      */
 XColor		color_image[256];
				/* The current image color table.	      */
 Colormap	*colormap_ptr;	/* Color map pointer.			      */
 XColor		*colortable_ptr;/* Color table map pointer.		      */
 XColor		color_workclear;/* Used to reset color_workcell.	      */
 XFontStruct 	*font_info;	/* General font information.		      */
 GC		line_gc;	/* Line pixmap GC.			      */
 long		cindex;		/* Index into color store.		      */
 long		max_plane;	/* Maximum plane number.		      */
 long		current_plane;	/* Current plane number.		      */
 long		view;		/* Indicates whether or not an image is
				   currently being viewed.		      */
 long		x;		/* Current pointer X position.		      */
 long	 	y;		/* Current pointer Y position.		      */
 long		line_x;		/* Tracking line X position.		      */
 long		line_y;		/* Tracking line Y position.		      */
 long		num_fullres;	/* The number of XID_FULLRES structures 
				   allocated.				      */
 long 		flags;		/* Bit flags 
				   - Has a horizontal tracking line been 
				     created?
				   - Has a vertical tracking line been
				     created?
				   - Should the colors warning message be
				     output?				      */
 int		screen;		/* X screen value.			      */
 char		color_name[NUM_CSTORE][25];
				/* Color store color strings; used for
				   comparisons.				      */
 int		message;	/* Indicates whether or not the message box
				   is currently displayed.		      */
 long		cursor_color;	/* Index to current cursor color.	      */
 Cursor		pointer_cursor;	/* Cursor ID of pointer cursor.		      */
 long		num_vert;	/* Number of applications requiring a
				   vertical tracking line.		      */
 long		num_horiz;	/* Number of applications requiring a 
				   horizontal tracking line.	 	      */
 long		num_composite;	/* Current composite view index.	      */
 pid_t		help_pid;	/* Process ID of help process.		      */
 long		*red_from;	/* Red band from values for current mapping.  */
 long		*green_from;	/* Green band from values for current mapping.*/
 long		*blue_from;	/* Blue band from values for current mapping. */
 long		*red_to;	/* Red band to values for current mapping.    */
 long		*green_to;	/* Green band to values for current mapping.  */
 long		*blue_to;	/* Blue band to values for current mapping.   */
 long		red_count;	/* Number of red band break points.	      */
 long		green_count;	/* Number of green band break points.	      */
 long		blue_count;	/* Number of blue band break points.	      */
 };

/*
    Image plane structure.
*/
struct XID_MEMPLANE
 {
 double		offset;		/* Offset used to scale non-byte data.        */
 double		slope;		/* Slope used to scale non-byte data.	      */
 double		mean;		/* Mean data value.			      */
 double		stdev;		/* Standard deviation.			      */
 long		plane_number;	/* Memory plane number.			      */
 long		window[4];	/* Image window.		    	      */
 long		dspwin[4];	/* Display window.			      */
 long		band;		/* Band number.				      */
 long		image_type;	/* Image type -- See defines.		      */
 long		histogram[256];	/* Histogram data.			      */
 long		minval;		/* Minimum value.			      */
 long		maxval;		/* Maximum value.			      */
 long		skipl;		/* Line skip factor.			      */
 long		skips;		/* Sample skip factor.			      */
 long		proj_valid;	/* Does image_info contain valid projection
				   information?				      */
 long		corner_valid;	/* Does image_info contain valid corner
				   coordinates?				      */
 struct XID_IMGINFO	
		*image_info;	/* Information about image (projection 
				   information, nl, ns, nbands).	      */
 long 		flags;		/* Bit flags 
				   - Has data been loaded into this plane?
				   - Is the data in this plane locked?        */
 unsigned char	*image_data;	/* Original image data.			      */
 char		filename[NAMLEN];
				/* Image file name.			      */
 };

/*
   Memory layout structure.
*/
struct XID_MEMLAYOUT
 {
 Pixmap		image_pixmap;	/* Pixmap for displayed image data.           */
 Pixmap		graphics_pixmap;/* Pixmap for displayed graphics data.        */
 long		imgpix;		/* Has an image pixmap been created?	      */
 long		mem_width;	/* Width of the memory plane.		      */
 long		mem_height;	/* Height of the memory plane.		      */
 long		view_width;	/* Width of the viewport.		      */
 long		view_height;	/* Height of the viewport.		      */
 long		view_xoffset;	/* Offset to be added to viewport width to 
				   allow the appropriate image size to show 
				   through.			              */
 long		view_yoffset;	/* Offset to be added to viewport height to 
				   allow the appropriate image size to show 
				   through.			      	      */
 };

/*
    Image information structure.
*/
struct XID_IMGINFO
 {
 double proj_coef[15]; /* GCTP projection parameters 
                          Refer to GCTP document for field definitions.	      */
 double upleft[2];     /* Corner coordinates in latitude/longitude or         */
 double loleft[2];     /* northing/easting or projection coordinates (y/x).   */
 double upright[2];    
 double loright[2];    
 double pdist_y;       /* Projection distance/pixel (y-direction) 	      */
 double pdist_x;       /* Projection distance/pixel (x-direction) 	      */
 double line_inc;      /* Line increment for sampling.             	      */
 double sample_inc;    /* Sample increment for sampling.           	      */
                       /* NOTE:  The line/sample increments are 
                          	 the values applied to the original 
                          	 image to obtain this image.  If resampling
                          	 or sub-sampling was not applied, the 
                          	 value contained in these fields will
                          	 be 1.0.				      */
 long		nl;		/* Number of lines.			      */
 long		ns;		/* Number of samples.			      */
 long		nbands;		/* Number of bands.			      */
 long		dtype;		/* Data type of image.
				   1 = unsigned char (byte)
				   2 = short
				   3 = long
				   4 = float				      */
 long		proj_code;	/* GCTP projection code.		      */
 long		zone_code;	/* UTM or State Plane zone code.	      */
 long		datum_code;	/* GCTP datum code.			      */
 long		master_line;	/* Line relative to master image.	      */
 long		master_sample;	/* Sample relative to master image.	      */
 char		proj_units[12];	/* Projection units.			      */
 };

/*
    Current view structure.
*/
struct XID_VIEW
 {
 Pixmap		composite;	/* Composite data.			      */
 Colormap	colormap;	/* Composite color map.			      */
 long		rgbplanes[3];	/* Red, green and blue planes used to form 
				   this composite.			      */
 long		ncolors;	/* Number of colors used.		      */
 long		flags;		/* Bit flags
				   - Color compression algorithm used	      */
 };

/*
    Composite structure.
*/
struct XID_COMPOSITE
 {
 XColor		color_table[256];
				/* Composite image color table.		      */
 long		rgbplanes[3];	/* Red, green and blue planes used to form 
				   this composite.			      */
 long		*red_from;	/* Red band from values for current mapping.  */
 long		*green_from;	/* Green band from values for current mapping.*/
 long		*blue_from;	/* Blue band from values for current mapping. */
 long		*red_to;	/* Red band to values for current mapping.    */
 long		*green_to;	/* Green band to values for current mapping.  */
 long		*blue_to;	/* Blue band to values for current mapping.   */
 long		red_count;	/* Number of red band break points.	      */
 long		green_count;	/* Number of green band break points.	      */
 long		blue_count;	/* Number of blue band break points.	      */
 long		flags;		/* Bit flags
				    - Is the entry currently in use?	      */
 char		*name;		/* Composite image name.		      */
 };

/*******************************************************************************
                      X Image Display Application Structures
*******************************************************************************/

/*
   Memory plane layout structure.
*/
struct XID_LAYOUT
 {
 long		mem_width;	/* Memory plane width.			      */
 long		mem_height;	/* Memory plane height.			      */
 long		mem_depth;	/* Memory plane depth.			      */
 long		view_width;	/* Viewport width.			      */
 long		view_height;	/* Viewport height.			      */
 };

/*
   Load structure.
*/
struct XID_LOAD
 {
 XmString	dirmask;	/* Current directory mask.		      */
 long		windows[MAXIMG][4];
				/* Window specification for each input image. */
 long		bands[MAXIMG][MAXBND+1];
				/* Band numbers for each input image.         */
 long		nbands[MAXIMG]; /* Number of bands for each input image.      */
 long		totbnd;		/* Total number of bands specified.	      */
 long		nimgs;		/* Number of images from input specification. */
 long		skipl;		/* Line skip factor.			      */
 long		skips;		/* Sample skip factor.			      */
 long		type;		/* Type of image file.			      */
 char		image_name[NAMLEN];
				/* Image name.				      */
 char		tnames[MAXIMG][CMLEN];
				/* LAS names from the input specification.    */
 char		hnames[MAXIMG][CMLEN];
				/* Host names from the input specification.   */
 };

struct XID_NONLAS
 {
 long		header_bytes;	/* Number of header bytes in the image file.  */
 long		nbands;		/* Number of bands in the image file.	      */
 long		lines;		/* Number of lines in the image file.	      */
 long		samples;	/* Number of samples in the image file.       */
 long		dtype;		/* Data type of image file.		      */
 long		format;		/* File format (BIL or BSQ).		      */
 };

struct XID_PROJINFO
 {
 double proj_coef[15]; /* GCTP projection parameters 
                          Refer to GCTP document for field definitions.	      */
 double upleft[2];     /* Corner coordinates in latitude/longitude or         */
 double loleft[2];     /* northing/easting or projection coordinates (y/x).   */
 double upright[2];    
 double loright[2];    
 double pdist_y;       /* Projection distance/pixel (y-direction) 	      */
 double pdist_x;       /* Projection distance/pixel (x-direction) 	      */
 double line_inc;      /* Line increment for sampling.             	      */
 double sample_inc;    /* Sample increment for sampling.           	      */
                       /* NOTE:  The line/sample increments are 
                          	 the values applied to the original 
                          	 image to obtain this image.  If resampling
                          	 or sub-sampling was not applied, the 
                          	 value contained in these fields will
                          	 be 1.0.				      */
 long		proj_code;	/* GCTP projection code.		      */
 long		zone_code;	/* UTM or State Plane zone code.	      */
 long		datum_code;	/* GCTP datum code.			      */
 char		proj_units[12];	/* Projection units.			      */
 };

/*
   Histogram structure.
*/
struct XID_HISTO
 {
 Pixmap		red_histo;	/* Pixmap for red histogram.		      */
 Pixmap		green_histo;	/* Pixmap for green histogram.		      */
 Pixmap		blue_histo;	/* Pixmap for blue histogram.		      */
 GC		gc;		/* Histogram window GC.			      */
 double         mean;           /* Mean data value.                           */
 double         stdev;          /* Standard deviation.                        */
 long		z_histo[256];	/* Z dimension histogram.		      */
 long		cum_histo[3][256];
				/* Cumulative histogram data.		      */
 long           minval;         /* Minimum value.                             */
 long           maxval;         /* Maximum value.                             */
 long		*range;		/* Pointer to ranges of mask values.	      */
 long		rcount;		/* Number of range values.		      */
 long		*bin;		/* Pointer to individual bin mask values.     */
 long		bcount;		/* Number of individual bin values.	      */
 long		xaxis_x[2];	/* X Coordinates of the X axis of the 
				   histogram.	      			      */
 long		xoffset;	/* Offset used for drawing X axis.	      */
 long		xlength;	/* Length of X axis.			      */
 long		annot_x[3];	/* X location for annotation string output by
				   percentage calculation.		      */
 long		annot_y1[3];	/* Y location for annotation string output by
				   percentage calculation.		      */
 long		annot_y2[3];	/* Y location for annotation string output by
				   percentage calculation.		      */
 long 		flags;		/* Bit flags
					- Active flag
					- Object to histogram
					- Type of histogram
					- Dimension to histogram
					- Data values to histogram            */
 };

/*
   Cursor structure.
*/
struct XID_CURSOR
 {
 Pixmap		info_pixmap;	/* Pixmap for cursor information.	      */
 Pixmap		clear_pixmap;	/* Pixmap for clearing the cursor window.     */
 Pixmap		pointer_pixmap;	/* Pixmap for cursor pointer.		      */
 GC		gc;		/* Cursor window GC.			      */
 GC		pointer_gc;	/* Pointer GC.			              */
 XFontStruct    *cursor_font;   /* Pointer to font structure for pointer
				   cursor.				      */
 KeyCode       	h_key,          /* 'h' or left key.                           */
		j_key,          /* 'j' or down key.                           */
		k_key,          /* 'k' or up key.                             */
		l_key,          /* 'l' or right key.                          */
		left_key,       /* Left arrow key.                            */
		down_key,       /* Down arrow key.                            */
		up_key,         /* Up arrow key.                              */
		right_key;      /* Right arrow key.                           */
 double		coef[6];        /* Coefficients mapping image coordinates
				   to projection coordinates.                 */
 double		projparm[15];	/* Projection paramters for GCTP.	      */
 long		zplane;		/* Current plane for Z dimension.	      */
 long		max_zplane;	/* Maximum plane for Z dimension.	      */
 long		proj_valid;	/* Is projection information valid?	      */
 long		corner_valid;	/* Are the corner coordinates valid?	      */
 long		geo;		/* Output projection system.		      */
 long		outdatum;	/* Output datum code.			      */
 long		outproj;	/* Output projection units.		      */
 long		outunit;	/* Output unit code for projection 
				   coordinates.				      */
 long		outzone;	/* Output zone code.			      */
 long 		flags;		/* Bit flags
				   - Active flag
				   - Dimension to display values in
				   - Update mode
				   - Display projection coordinates?
				   - Display lat/long values?
				   - Display unscaled data values?	      */
 FILE		*fp;		/* File pointer to ascii file which will 
				   contain selected coordinate information.   */
 };

/*
    Full-resolution window structure.
*/
struct XID_FULLRES
 {
 Pixmap		image_pixmap;	/* Full resolution window pixmap.	      */
 Colormap	colormap;	/* Full resolution window color map.	      */
 GC             gc;             /* Rubber-band rectangle GC.		      */
 double         offset;         /* Offset used to scale non-byte data.        */
 double         slope;          /* Slope used to scale non-byte data.         */
 double         mean;           /* Mean data value.                           */
 double         stdev;          /* Standard deviation.                        */
 unsigned long  rb_fore;        /* Foreground pixel value for rubber-band 
				   rectangle.                         	      */
 int		ulx;		/* Upper left corner of user defined 
				   rectangle (X).			      */
 int		uly;		/* Upper left corner of user defined 
				   rectangle (Y).			      */
 int		prevx;		/* Previous lower right corner of user defined 
				   rectangle (X).			      */
 int		prevy;		/* Previous lower right corner of user defined 
				   rectangle (Y).			      */
 long           dspwin[4];      /* Display window.                            */
 long           histogram[256]; /* Histogram data.                            */
 long           minval;         /* Minimum value.                             */
 long           maxval;         /* Maximum value.                             */
 long           skipl;          /* Line skip factor.                          */
 long           skips;          /* Sample skip factor.                        */
 long		width;		/* Width of full resolution window.           */
 long		height;		/* Height of full resolution window.          */
 long		window[4];	/* Full resolution window loaded.	      */
 long		band;		/* Band loaded.			      	      */
 long 		flags;		/* Bit flags
				   - Active flag			      
				   - User entered window size flag
				   - Show outline flag	      		      */
 unsigned char	*image_data;	/* Image data loaded.			      */
 };

/*
    General map structure.
*/
struct XID_MAP
 {
 Widget		subpanel;	/* Map subpanel currently selected.	      */
 long 		flags;		/* Bit flags
 				   - Color component to map
				   - Mapping component modified               */
 };

/*
   Adjust structure.
*/
struct XID_ADJUST
 {
 XColor		*color_adjust;  /* Current adjusted color table.	      */
 XColor		*color_linear;	/* Linear color table.			      */
 XColor		*adjust_ptr;	/* Pointer to the color table to adjust.      */
 float		lut_slope;	/* Current LUT slope adjustment.	      */
 float		lut_offset;	/* Current LUT offset adjustment.	      */
 long		local_map[3][256];
				/* Local copy of the current adjusted mapping.*/
 };

/*
   Pseudocolor structure.
*/
struct XID_PSD
 {
 XColor		*color_psd; 	/* Current pseudocolor color table.	      */
 XColor		*savecolors;	/* Colors saved for undo function.	      */
 XmTextPosition text_pos;	/* Current position in psd_def_text widget.   */
 long		text_length;	/* Current text length.			      */
 long		*range;		/* Gray level ranges to pseudocolor.          */
 long		rcount;		/* Number of ranges specified.	              */
 long		*bin;		/* Bin values to pseudocolor.		      */
 long		bcount;		/* Number of bin values specified.  	      */
 long 		flags;		/* Bit flags
				   - Color specification type
				   - Does the PSD color table need to be
				     initialized?			      */
 char		*color;		/* Color name.				      */
 char		*text_list;	/* Current text (color and gray level values).*/
 };

/*
   Piece-wise linear structure.
*/
struct XID_PWL
 {
 XColor		*color_pwl; 	/* Current piece-wise linear color table.     */
 long		*red_from;	/* Red band from breakpoint values.	      */
 long		*green_from;	/* Green band from breakpoint values.	      */
 long		*blue_from;	/* Blue band from breakpoint values.	      */
 long		*red_to;	/* Red band to breakpoint values.	      */
 long		*green_to;	/* Green band to breakpoint values.	      */
 long		*blue_to;	/* Blue band to breakpoint values.	      */
 long		red_num;	/* Red band number of from and to values.     */
 long		green_num;	/* Green band number of from and to values.   */
 long		blue_num;	/* Blue band number of from and to values.    */
 long 		flags;		/* Bit flags
 				   - Color component to map		      */
 };

/*
   Save mapping structure.
*/
struct XID_SAVEMAP
 {
 char		*entry_dltf; 	/* DLTF entry name.			      */
 char		hname_dltf[CMLEN];
				/* Host file name of DLTF.		      */
 char		*desc_dltf;	/* DLTF description.			      */
 };

/*
   Load mapping structure.
*/
struct XID_LOADMAP
 {
 struct  ASDLTF asdltf;         /* DLTF structure.                            */
 long		fd;		/* DLTF file descriptor.		      */
 long    	dlen;           /* Length of the data portion of the DLTF
				   entry.                                     */
 char        	dtype[3];       /* Data type of the DLTF entry.               */
 char		*entry_dltf; 	/* DLTF entry name.			      */
 char		hname_dltf[CMLEN];
				/* Host file name of DLTF.		      */
 };

/*
   Profile structure.
*/
struct XID_PROFILE
 {
 Pixmap		info_pixmap;	/* Pixmap for the profile information.        */
 Pixmap		clear_pixmap;	/* Pixmap for clearing the profile window.    */
 GC		gc;		/* Profile window GC.			      */
 XPoint        	*points[3];     /* Pointers to working RGB point arrays.      */
 long		height;		/* Height of profile pixmap.		      */
 long		width;		/* Width of profile pixmap.		      */
 long		view_xoffset;	/* Offset to be added to viewport width to 
				   allow the appropriate profile size to show 
				   through.			              */
 long		view_yoffset;	/* Offset to be added to viewport height to 
				   allow the appropriate profile size to show 
				   through.			      	      */
 long 		flags;		/* Bit flags
				   - Active flag
				   - Update mode
				   - Dimension to profile
				   - Type of profile line (continuous or 
				     discrete)				      */
 };

/*
    Initialization structure.
*/
struct XID_INIT
 {
 long		plane;		/* Plane number to initialize.		      */
 long		rotate_value;	/* Current rotator value.		      */
 long 		flags;		/* Bit flags
				   - Initialization type
				   - Clear all planes			      */
 };

/*
   Structure used to store information needed by the interpolated zoom routines
*/
struct ENLARGE
  {
  long *work;			/* Hybrid space buffer 			      */
  long *horiz_table;		/* Horizontal resampling table 		      */
  long *vert_table;		/* Vertical resampling table 		      */
  float alpha;			/* Parametric cubic convolution alpha value   */
  long in_lines;		/* Number of input lines + resampling pad     */
  long in_samps;		/* Number of input samples + resampling pad   */
  long hybrid_samps;		/* Number of samples in hybrid space          */
  long extract_lines;		/* Number of lines to extract from display    */
  long extract_samps;		/* Number of samples to extract from display  */
  long factor;			/* Zoom factor 				      */
  long horiz_dim;		/* Horizontal resampling table dimension      */
  long vert_dim;		/* Vertical resampling table dimension        */
  unsigned char *input;		/* Input space buffer 			      */
  unsigned char *output;	/* Output space buffer 			      */
  char table_file[CMLEN];	/* Resampling weight table file name 	      */
  char method[6];		/* Resampling method 			      */
  };

/*
    Zoom structure.
*/
struct XID_ZOOM
 {
 Pixmap         image_pixmap;   /* Pixmap for zoomed image data.              */
 struct ENLARGE parms;		/* Zoom enlargement parameters.	   	      */
 long		factor;		/* Zoom factor.				      */
 long		x;		/* Center of zoomed window sample.	      */
 long		y;		/* Center of zoomed window line.	      */
 long		window[4];	/* Window extracted for zoom.		      */
 long 		flags;		/* Bit flags
				   - Active flag
				   - Resampling method
				   - Zoom type
				   - Zoom window			      */
 unsigned char  *image_data;    /* Zoomed image data.                         */
 };

/*
   Mask structure.
*/
struct XID_MASK
 {
 Pixmap		mask_pixmap[MASK_MAX];	
				/* Pixmap for the graphics plane mask.	      */
 GC		gc;		/* Mask window GC.			      */
 long		graphics_mask;	/* Graphics mask number.		      */
 long		*range;		/* Gray level ranges to generate mask for.    */
 long		num_range;	/* Number of ranges specified.	              */
 long		*bin;		/* Bin values to generate mask for.	      */
 long		num_bin;	/* Number of bin values specified.  	      */
 long		red;		/* Red value.				      */
 long		green;		/* Green value.				      */
 long		blue;		/* Blue value.				      */
 long		rotate_value;	/* Current rotator value.		      */
 long		current_color;	/* Index of currently selected color into
				   wk_colors array.			      */
 long		color[MASK_MAX];/* Index of color into wk_colors array.       */
 long 		flags;		/* Bit flags
				   - Color specification type 		      
				   - Clear graphics plane?		      */
 };

/*
    Show structure.
*/
struct XID_SHOW
 {
 long		planes[3]; 	/* Image planes to be displayed.	      */
 char		*composite;	/* Composite name to be displayed.	      */
 };

/*
    Help structure.
*/
struct XID_HELP
 {
 Widget		widget;		/* The widget for which help is currently
				   displayed.				      */
 long 		flags;		/* Bit flags
				   - Active flag.			      */
 };

/*
    Flicker structure.
*/
struct XID_FLICKER
 {
 Pixmap		pixmaps[FLICKER_MAX];
				/* Flicker pixmaps.			      */
 XColor		colortable[FLICKER_MAX][256];
				/* Color tables for images to flicker.	      */
 XtIntervalId	timer;		/* Timeout procedure callback.		      */
 GC		gc;		/* Flicker graphics context.		      */
 Multibuffer 	buffers[FLICKER_MAX];
				/* Display buffers for multi-buffer extension.*/
 long		planes[FLICKER_MAX][3];	
				/* List of plane combinations to flicker.     */
 long		cflags[FLICKER_MAX];
				/* Indicates whether or not the entry is a
				   saved composite view.		      */
 long		nplanes;	/* Number of plane combinations to flicker.   */
 long		rate;		/* Flicker interval in seconds.		      */
 long		current_image;	/* Index indicating current image displayed.  */
 long 		flags;		/* Bit flags
				   - Active flag.
				   - Flickering flag.
				   - Multi-buffer extension flag.
				   - Multi-buffer buffer creation flag.
				   - Item selected flag.		      */
 char		*composite;	/* Composite view to add to flicker list.     */
 };

/*
   Power spectrum structure.
*/
struct XID_POWER
 {
 Pixmap		info_pixmap;	/* Pixmap for the plot information.           */
 Pixmap		clear_pixmap;	/* Pixmap for clearing the window.    	      */
 GC		gc;		/* Power spectrum window GC.		      */
 XPoint        	*ent_points;    /* Pointer to working point array.            */
 XPoint        	*per_points;    /* Pointer to working point array.            */
 float		maxdb;		/* Maximum decibel value.		      */
 float		mindb;		/* Minimum decibel value.		      */
 float		scale;		/* Scale for factor for X or Y axis.	      */
 float		start_freq;	/* Starting frequency.			      */
 float		end_freq;	/* Ending frequency.			      */
 float		b0;		/* Maximum entropy b0 coefficient.	      */
 float		*coef;		/* Maximum entropy coefficients.	      */
 long		height;		/* Height of profile pixmap.		      */
 long		width;		/* Width of profile pixmap.		      */
 long		ent_npts;	/* Maximum entropy number of points.	      */
 long		per_npts;
				/* Periodogram number of points.	      */
 long		order;		/* Order of maximum entropy calculation.      */
 long		view_xoffset;	/* Offset to be added to viewport width to 
				   allow the appropriate profile size to show 
				   through.			              */
 long		view_yoffset;	/* Offset to be added to viewport height to 
				   allow the appropriate profile size to show 
				   through.			      	      */
 long 		flags;		/* Bit flags
				   - Active flag
				   - Update mode
				   - Dimension to plot
				   - Type of plot line (continuous or 
				     discrete)				      */
 };

/*
   Save Image structure.
*/
struct XID_SAVE
 {
 long 		flags;		/* Bit flags
				   - Active flag
				   - Window 
				   - Type 				      */
 char		*name;		/* Saved image name.			      */
 };

/*
   Fill structure.
*/
struct XID_FILL
 {
 long		*from; 		/* From breakpoints.			      */
 long		*to; 		/* To breakpoints.			      */
 long		*polys;		/* Element numbers of selected polygons.      */
 long		num;		/* Number of break points.		      */
 long		num_poly;	/* Number of polygons selected.		      */
 long		max_breakpts;	/* Current number of break points allocated.  */
 long		max_polys;	/* Current number of polygons selected.       */
 long		constant;	/* Constant fill value.			      */
 long 		flags;		/* Bit flags
					- Active flag
					- Fill method (internal or external) 
					- Fill_mode (user-selected or all) 
					- Mapping type (pwl, renumber, constant)
					- Window to fill (main or fullres)
					- Destination (update current image or
					  generate new image)
									      */
 };

/*
    Utility structure.
*/
struct XID_UTIL
 {
 XmString	*color_list[MAX_COLORS];
				/* Color list from the X color database.      */
 long		color_list_num;	/* Total number of list entries.	      */
 };

/*
    Declare the global structures.
*/

#ifndef _xid_h
extern struct XID_STRUCT     xid;
extern struct XID_MEMLAYOUT  xid_mem;
extern struct XID_UTIL	     xidutil;
extern struct XID_LAYOUT     xid_layout;
extern struct XID_LOAD       xid_load;
extern struct XID_HISTO      xid_histo;
extern struct XID_CURSOR     xid_cursor;
extern struct XID_FULLRES    *xid_fullres;
extern struct XID_MAP        xid_map;
extern struct XID_ADJUST     xid_adjust;
extern struct XID_PSD        xid_psd;
extern struct XID_PWL        xid_pwl;
extern struct XID_SAVEMAP    xid_savemap;
extern struct XID_LOADMAP    xid_loadmap;
extern struct XID_PROFILE    xid_profile;
extern struct XID_INIT       xid_init;
extern struct XID_ZOOM       xid_zoom;
extern struct XID_MASK       xid_mask;
extern struct XID_SHOW       xid_show;
extern struct XID_FLICKER    xid_flicker;
extern struct XID_MEMPLANE   *xid_planes;
extern struct XID_COMPOSITE  *xid_composite;
extern struct XID_VIEW       xid_view;
extern struct XID_HELP       xid_help;
extern struct XID_NONLAS     xid_nonlas;
extern struct XID_PROJINFO   xid_projinfo;
extern struct XID_POWER      xid_power;
extern struct XID_SAVE       xid_save;
extern struct XID_FILL       xid_fill;
#else
struct XID_STRUCT     xid;
struct XID_MEMLAYOUT  xid_mem;
struct XID_UTIL       xidutil;
struct XID_LAYOUT     xid_layout;
struct XID_LOAD       xid_load;
struct XID_HISTO      xid_histo;
struct XID_CURSOR     xid_cursor;
struct XID_FULLRES    *xid_fullres;
struct XID_MAP        xid_map;
struct XID_ADJUST     xid_adjust;
struct XID_PSD        xid_psd;
struct XID_PWL        xid_pwl;
struct XID_SAVEMAP    xid_savemap;
struct XID_LOADMAP    xid_loadmap;
struct XID_PROFILE    xid_profile;
struct XID_INIT       xid_init;
struct XID_ZOOM       xid_zoom;
struct XID_MASK       xid_mask;
struct XID_SHOW       xid_show;
struct XID_FLICKER    xid_flicker;
struct XID_MEMPLANE   *xid_planes;
struct XID_COMPOSITE  *xid_composite;
struct XID_VIEW       xid_view;
struct XID_HELP       xid_help;
struct XID_NONLAS     xid_nonlas;
struct XID_PROJINFO   xid_projinfo;
struct XID_POWER      xid_power;
struct XID_SAVE       xid_save;
struct XID_FILL       xid_fill;
#endif

static char *wk_colors[] = {"black","white","red","green","blue",
    "yellow","magenta","cyan"};
				/* Working color strings.		*/

static char *wk_fonts[] = {"-adobe-courier-bold-r-normal--18-180-75-75-m-110-iso8859-1","-adobe-courier-bold-r-normal--14-140-75-75-m-90-iso8859-1", "-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1","-adobe-times-bold-r-normal--18-180-75-75-p-99-iso8859-1","-adobe-times-bold-r-normal--14-140-75-75-p-77-iso8859-1","-adobe-times-bold-r-normal--12-120-75-75-p-67-iso8859-1","-adobe-helvetica-bold-r-normal--18-180-75-75-p-103-iso8859-1","-adobe-helvetica-bold-r-normal--14-140-75-75-p-82-iso8859-1","-adobe-helvetica-bold-r-normal--12-120-75-75-p-70-iso8859-1"};
				/* Working fonts strings.		*/

static char *context_names[] = {"Image","Image_Cursor","Graphics Select",
    "Graphics Edit","Graphics Mv/Del","Graphics Add","Profile","Fill"};
