/****************************************************************************/
/*   PROGRAM NAME  : ct_functions.c                                         */
/*   AUTHOR        : g e nostrand                                           */
/*   DATE          : aug 10, 1993                                           */
/*   LOCATION      : DSRC PRINCETON, NJ                                     */
/*                                                                          */
/*   PROGRAM DESCRIPTION:                                                   */
/*	This module contains functions for 'Dismiss'ing or 'Cancel'ing      */
/*      the X Color Tools display.  An additional function is to start	    */
/*      the gamut mapping preview program GamutTool.  			    */ 
/*   PROGRAM MODIFICATIONS:                                                 */
/****************************************************************************/

#include "xid.h"
/****************************************************************************/
/*
	The function colortool_exit is called when the user chooses Dismiss
	from the X Color Tool.  At this point the user has decided that the
	contrast stretch applied is good and wants to save the contrast 
	stretch color map with the image.  Prior to dismissing, the 
	color map changes are only know to the color tool.  In addition to
	unmanaging the X Color Tool display, other subordinate displays
	are also unmanaged to minimize screen clutter.
*/
/****************************************************************************/
#ifdef _NO_PROTO
void colortool_exit(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
void colortool_exit(Widget w, XtPointer client, XtPointer call)
#endif
{
  XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

  if (XtIsManaged (ct_stretch_mainWindow))
  {
      XtUnmanageChild (ct_stretch_mainWindow);
      XtPopdown (xidStretch);

      argcnt=0;
      XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Stretch", XmSTRING_DEFAULT_CHARSET))); argcnt++;
      XtSetValues (ct_show_stretch_button, args, argcnt);
      XmStringFree (xmstr[0]);

  }
  if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
  {
      XtUnmanageChild (ct_cmap_mainwindow);
      XtPopdown (xidCMap);

      argcnt=0;
      XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Map", XmSTRING_DEFAULT_CHARSET))); argcnt++;
      XtSetValues (ct_show_map_button, args, argcnt);
      XmStringFree (xmstr[0]);

  }

  if (ct_main_image_scrollwin != NULL && XtIsManaged (ct_main_image_scrollwin))
  {
      XtUnmanageChild (ct_main_image_scrollwin);
      XtPopdown (ct_Shell_MainWin);

  }

  if (hist_main_bb != NULL && XtIsManaged (hist_main_bb))
  {
      XtUnmanageChild (hist_main_bb);
      XtPopdown (hist_norm_shell);

      argcnt=0;
      XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Normalize", XmSTRING_DEFAULT_CHARSET))); argcnt++;
      XtSetValues (ct_normalize_button, args, argcnt);
      XmStringFree (xmstr[0]);
  }

  XtUnmanageChild(colortool_interface);
  XtPopdown(Colortool_Shell);

  /*  Integrate the X Color Tool color map adjustments with the rest of Xid */
  /*  This simulates a Dismiss from other mapping displays such as PWL	    */
  xid_map.flags = xid_map.flags | MOD_RED | MOD_GREEN | MOD_BLUE; 
  map_destroy (w, client, call);

}
/****************************************************************************/
/*
	The function colortool_cancel is called when the Cancel button is 
	pressed on the X Color Tools display.  When cancelling the user has
	decided that the contrast stretch adjustments made in the X Color Tool
	should not be applied to the image.  In addition to
	unmanaging the X Color Tool display, other subordinate displays
	are also unmanaged to minimize screen clutter.
*/
/****************************************************************************/

#ifdef _NO_PROTO
void colortool_cancel (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
void colortool_cancel (Widget w, XtPointer client, XtPointer call)
#endif
{

  XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

  if (XtIsManaged (ct_stretch_mainWindow))
  {
      XtUnmanageChild (ct_stretch_mainWindow);
      XtPopdown (xidStretch);

      argcnt=0;
      XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Stretch", XmSTRING_DEFAULT_CHARSET))); argcnt++;
      XtSetValues (ct_show_stretch_button, args, argcnt);
      XmStringFree (xmstr[0]);

  }
  if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
  {
      XtUnmanageChild (ct_cmap_mainwindow);
      XtPopdown (xidCMap);

      argcnt=0;
      XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Map", XmSTRING_DEFAULT_CHARSET))); argcnt++;
      XtSetValues (ct_show_map_button, args, argcnt);
      XmStringFree (xmstr[0]);

  }
  if (ct_main_image_scrollwin != NULL && XtIsManaged (ct_main_image_scrollwin))
  {
      XtUnmanageChild (ct_main_image_scrollwin);
      XtPopdown (ct_Shell_MainWin);

  }
  if (hist_main_bb != NULL && XtIsManaged (hist_main_bb))
  {
      XtUnmanageChild (hist_main_bb);
      XtPopdown (hist_norm_shell);

      argcnt=0;
      XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Normalize", XmSTRING_DEFAULT_CHARSET))); argcnt++;
      XtSetValues (ct_normalize_button, args, argcnt);
      XmStringFree (xmstr[0]);

  }


  XtUnmanageChild(colortool_interface);
  XtPopdown(Colortool_Shell);

  /*  The following code was taken from pwl_cancel in xid_fun/map.c */

  XtSetSensitive(main_map_shell,True);
  xid_map.subpanel = (Widget)NULL;

}

/****************************************************************************/
/*
	The following structures and functions are for the interface with 
	the gamut mapping preview application GamutTool.  
	A special DLTF entry is made for the X Color Tool applied mappings.  

*/
/****************************************************************************/



char ct_previewname[1024];/* Name of preview image saved with View->Save*/
char ct_imagename[1024];  /* Name of image being viewed		  	*/
char ct_dltf_entry[13];	  /* DLTF entry name for Color Tool mapping	*/
int  ct_dltf_try = 0;	  /* Unique DLTF entry for multiple calls to GamutTool*/

/* Format for generated DLTF entry name - ct<Xid Pid><Try>		*/
#define COLORTOOL_DLTF_ENTRY "ct%d%d"
#define COLORTOOL_DLTF_DESC "Entry saved automatically from colortool"

void
ct_save_dltf ()

/*******************************************************************************
Purpose:
	This routine retrieves and verfies the DLTF entry name and
	description specified by the user.  The DLTF name is also 
	determined based on the image name of the displayed image.  
	If necessary, the DLTF name is prompted for using a popup panel.  
	Finally, the appropriate routine is called to write the DLTF
	entry.

By L. Huewe, USGS/EROS Data Center, 3/92

Modified by L. Huewe, USGS/EROS Data Center, 8/93
    - Modified to use XtGrabNone instead of XtGrabExclusive for message pop up. 

Algorithm Description:
*******************************************************************************/
{
Arg	args[1];		/* Variable used to set widget resources.     */

XmString
	xmstr[1];		/* Motif format string.			      */

Position	
	popx,			/* X position of pop-up panel.		      */
	popy;			/* Y position of pop-up panel.		      */

long	exists=0,		/* Test for file existence.		      */
	fd,			/* File descriptor.			      */
	plane_idx,		/* Index into memory plane structure.	      */
	size,			/* Size of character string to allocate.      */
	status,			/* Status indicator.			      */
	write_access = 2;	/* Test for write access to a DLTF.	      */

char	*dltf_desc,		/* User specified DLTF description.	      */
	*dltf_entry,		/* User specified DLTF entry name.	      */
	hname[CMLEN];		/* Host file name of displayed image (first
				   band).				      */

void	savemap_dltfw();	/* Function to write a DLTF entry.	      */

/*
    Print a warning and return if no image data is currently displayed.
*/
if (xid.view == -1)
    {
    xid_message("No image data currently being viewed");
    return;
    }

/*
    Get the entry name and description specified by the user.
*/
dltf_entry = ct_dltf_entry;
if (*dltf_entry == '\0')
    {
    xid_message("User must enter a DLTF entry name");
    return;
    }
if (strlen(dltf_entry) > 12)
    {
    xid_message("DLTF entry name may only contain 12 characters");
    return;
    }
xid_savemap.entry_dltf = (char *) malloc(strlen(dltf_entry)+1);
if (xid_savemap.entry_dltf == NULL)
    {
    xid_message("Insufficient memory available");
    return;
    }
strcpy(xid_savemap.entry_dltf,dltf_entry);
dltf_desc = COLORTOOL_DLTF_DESC;
if (*dltf_desc == '\0')
    size = 2;
else 
    {
    if (strlen(dltf_desc) > 80)
	{
	xid_message("DLTF entry description may only contain 80 characters");
	return;
	}
    size = strlen(dltf_desc);
    }
xid_savemap.desc_dltf = (char *) malloc(size+1);
if (xid_savemap.desc_dltf == NULL)
    {
    xid_message("Insufficient memory available");
    return;
    }
if (size == 2)
    strcpy(xid_savemap.desc_dltf," ");
else
    strcpy(xid_savemap.desc_dltf,dltf_desc);

/*
    Determine the name of the DLTF based on the first band of the displayed
    image.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;
if (xid_planes[plane_idx].image_type == LAS)
    {
    status = c_cmhost(xid_planes[plane_idx].filename,hname);
    if (status != E_SUCC)
	{
	xid_message("Error retrieving host file name");
	return;
	}
    }
else
    strcpy(hname,xid_planes[plane_idx].filename);
c_lsmknm(hname,"dltf",xid_savemap.hname_dltf);

/*
    Does the DLTF exist?
*/
if (c_lsstat(xid_savemap.hname_dltf,&exists) != E_SUCC)
    {
/*
    Try to create the DLTF.  If it cannot be created, prompt the user for
    a DLTF name.
*/
    if ((fd = creat(xid_savemap.hname_dltf,0775)) == -1)
	{
/*
    Set the appropriate error message for the popup panel.
*/
   	XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Unable to create DLTF for displayed image.\n\
	    User must enter DLTF name.", XmSTRING_DEFAULT_CHARSET)));
        XtSetValues(saveloadpopup_label,args,1);
	XmStringFree(xmstr[0]);
/*
    Translate the X/Y position relative to the save mapping panel to an X/Y 
    position relative to the screen.
*/
	XtTranslateCoords(savemap_form,(Position)0,(Position)0,&popx,&popy);
/*
    Position and display the pop-up panel.
*/
	XtMoveWidget(saveload_dltf_popup,popx,popy);
	XtPopup(saveload_dltf_popup,XtGrabNone);
	}
    else
/*
    Write the entry to the DLTF.
*/
        savemap_dltfw();
    }

/*
    Does the user have write access to the DLTF?
*/
else if (c_lsstat(xid_savemap.hname_dltf,&write_access) != E_SUCC)
    {
/*
    Set the appropriate error message for the popup panel.
*/
    XtSetArg(args[0], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Unable to write to DLTF of displayed image.\n\
	    User must enter DLTF name.", XmSTRING_DEFAULT_CHARSET)));
    XtSetValues(saveloadpopup_label,args,1);
    XmStringFree(xmstr[0]);
/*
    Translate the X/Y position relative to the save mapping panel to an X/Y 
    position relative to the screen.
*/
    XtTranslateCoords(savemap_form,(Position)0,(Position)0,&popx,&popy);
/*
    Position and display the pop-up panel.
*/
    XtMoveWidget(saveload_dltf_popup,popx,popy);
    XtPopup(saveload_dltf_popup,XtGrabNone);
    }
else
/*
    Write the entry to the DLTF.
*/
    savemap_dltfw();
XtFree(dltf_entry);
}

/****************************************************************************/
/*
	The function ct_gamut_match is called when the user presses the 
	Gamut Tool button on the X Color Tools display.  At this point the
	user has adjusted the colors in the image to the point where it is 
	desired to preview how the results would look on a particular 
	output device.  

	The parameters to GamutTool specify the image name, a small
	preview image name, and the dltf entry which contains the
	contrast stretch applied to the image.  The preview image must
	have been previously saved prior to initiating the GamutTool.
  
*/
/****************************************************************************/

#ifdef _NO_PROTO
void ct_gamut_match(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
void ct_gamut_match(Widget w, XtPointer client, XtPointer call)
#endif
{
  XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

    char	xgamutmatch[1024];
    extern void ct_displayerror ();

  if (ct_previewname[0] == '\0')
  {
     ct_displayerror("No preview file saved.\nCannot run gamuttool");
     return;

  }
  /*  Make sure current stretch is applied to image */
  map_destroy (w, client, call);

  /*  Write out dltf file with ColorTool entry */

  sprintf (ct_dltf_entry, COLORTOOL_DLTF_ENTRY, getpid, ct_dltf_try++);
  ct_save_dltf ();
  /*  Call X Gamut Match tool to preview mapping mechanisms */
  /* 
  */
#define USEUNIXNAME
#ifdef USEUNIXNAME
  c_cmhost (xid_planes[xid_view.rgbplanes[RED] -1].filename, ct_imagename);
#else
  strcpy(ct_imagename, xid_planes[xid_view.rgbplanes[RED] -1].filename);
#endif
  sprintf (xgamutmatch, "gamuttool %s \"%s\" %s&", ct_previewname,
	ct_imagename, ct_dltf_entry);
	
/*  system(xgamutmatch); */
  system("printenv UIDPATH");
}
