/****************************************************************************/
/*   PROGRAM NAME  : ct_graphicadjust.c                                     */
/*   AUTHOR        : g e nostrand                                           */
/*   DATE          : july 14, 1993                                          */
/*   LOCATION      : DSRC PRINCETON, NJ                                     */
/*                                                                          */
/*   PROGRAM DESCRIPTION:                                                   */
/*   PROGRAM MODIFICATIONS:                                                 */
/****************************************************************************/

#include "xid.h"
#include "ct_interface.h"

/*****************************************************************************/
/*
	 New colortool widgets generated from Builder Xcessory file ct.uil
*/
/*****************************************************************************/


/*  The following define specifies that a saturation stretch is enabled */
#define SATURATION

/*  Conversion functions for RGB and HLS color spaces in ct_cmapcb.c	*/
#ifdef _NO_PROTO
extern void rgbhls();
extern void hlsrgb();
#else
extern void rgbhls(int, int, int, float *, float *, float *);
extern void hlsrgb(float, float, float, int *, int *, int *);
#endif

/*  Display update functions for color map display in ct_cmapcb.c	*/
/*  Update information on current color*/
extern xcmap_show_curr_pixel_color();

/*  Draw either the RGB color square or the HLS hexagon and cone 	*/
extern xcmap_draw_colors(); 		

/*  Draw the HLS hexagon and cone only, required for saturation stretch	*/
extern xcmap_draw_hls_colormap();

extern void	    xcmap_setup_window();
extern Widget       Createxcmap_bulletinboard();

extern Visual       *XCmapVisual;
extern Colormap     XCmap;
extern Display	    *XCmapDisplay;

/*  Functions and variables utilized from ct_stretchoperations.c	*/
extern int	CurrStretch;	/*  Current stretch array index		*/
extern int	NumStretch;	/*  Total number of stretch array elements */

extern void	ct_init_stretch_list ();
extern void 	ct_new_curr_stretch ();
extern void 	ct_displaywarning();
extern void	ct_displayinfo(), ct_undisplayinfo();
extern void	ct_set_stretch(), ct_new_curr_stretch ();

/*  Variables for the Color Tool separate image window	*/
extern XColor  	CT_Img_Color [256];	

extern XVisualInfo	*CT_Img_Visual;

extern Colormap	CT_Img_CMap;
extern Pixmap		CT_Img_Pixmap;

#ifdef _NO_PROTO
void ct_histo_draw ();
#else
void ct_histo_draw (XID_COLORTOOLPTR color);
#endif

/*****************************************************************************/
/*
	ct_framedrawingarea - draw outline around drawing area to show focus
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_framedrawingarea(window, gc, x, y, width, height, color)
Window window;
GC gc;
int x;
int y;
unsigned int width;
unsigned int height;
unsigned long color;
#else
void ct_framedrawingarea(Window window, GC gc, int x, int y, 
                         unsigned int width, unsigned int height, 
                         unsigned long color)
#endif
{
/*  Drawing area no longer requires framing since there is only a single
    drawing area window being utilized					*/

#ifdef FRAMEDRAWING
   XSetLineAttributes(xid.display,gc,4,LineSolid,CapButt,JoinMiter);
   XSetForeground(xid.display,gc,color);
   XDrawRectangle(xid.display,window,gc,x,y,width,height);
#endif
}

/*****************************************************************************/
/*
	ct_highlightwindow - EnterWindowMask event handler, show current 
		focus by outlining drawing area.  
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_highlightwindow(w, data, report)
Widget w;
XtPointer data;
XEvent *report;
#else
void ct_highlightwindow(Widget w, XtPointer data, XEvent *report)
#endif
{
   XID_COLORTOOLPTR     adjust= *((XID_COLORTOOLPTR *) data);

   ct_framedrawingarea(	adjust->window,
                       	adjust->gc,
                       	adjust->x,
		       	adjust->y,
		  	adjust->width,
		      	adjust->height,
			adjust->linecolor);
}

#ifdef _NO_PROTO
void ct_unhighlightwindow(w, data, report)
Widget w;
XtPointer data;
XEvent *report;
#else
void ct_unhighlightwindow(Widget w, XtPointer data, XEvent *report)
#endif
{
   XID_COLORTOOLPTR     adjust= *((XID_COLORTOOLPTR *) data);

   ct_framedrawingarea(	adjust->window,
                       	adjust->gc,
                       	adjust->x,
		       	adjust->y,
		  	adjust->width,
		      	adjust->height,
			adjust->background);
}

/*****************************************************************************/
/*
    ct_drawline - draws the piece-wise linear stretch line and breakpoints
    pixel value is varied to erase previous line by setting pixel to 
    background of drawing area.  
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_drawline(window,  gc, from, to, index, pixel, size)
Window window;
GC gc;
long *from;
long *to;
long index;
unsigned long pixel;
int size;
#else
void ct_drawline(Window window,  GC gc, long *from, long *to, long index, 
                 unsigned long pixel, int size)
#endif
{
   int i;

   XSetLineAttributes(xid.display,gc,2,LineSolid,CapButt,JoinMiter);
   XSetForeground(xid.display,gc,pixel);
   for (i=0;i<index;i++) {
       XFillRectangle(xid.display,window,gc,from[i]-size/2,255-to[i]-size/2,size,size);
       XDrawLine(xid.display,window,gc,from[i],255-to[i],from[i+1],255-to[i+1]); 
   }
   XFillRectangle(xid.display,window,gc,from[index]-size/2,255-to[index]-size/2,size,size);
   XFlush(xid.display);
}
/*****************************************************************************/
/*
	DBGPRINT - debug routine for printing from and to values
*/
/*****************************************************************************/
void
#ifdef _NO_PROTO
DBGPRINT(msg, from, to, i, x, y)
char *msg;
int from;
int to;
int i;
int x;
int y;
#else
DBGPRINT(char *msg, int from, int to, int i, int x, int y)
#endif
{
#ifdef DEBUGPRINT
	printf("%s from[%d]=%d, to[%d]=%d, x=%d, y=%d \n", 
		msg, i, from, i, to, x, y);
#else
#endif
}

/*****************************************************************************/
/*
	ct_setbreakpoint - set new breakpoint at mouse position
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_setbreakpoint(from, to, index, x, y, margin) 
long *from;
long *to;
long *index;
int x;
int y;
int margin;
#else
void ct_setbreakpoint(long *from, long *to, long *index, int x, int y, int margin) 
#endif
{
  long 	sorted=0;
  long  i, fromtmp, totmp;

  if  ( x < 0 || x > 255 ) return;
  if  ( y < 0 || y > 255 ) return;

  for (i=0;i<*index;i++) {
      /* already break point         */
      if      ((x == from[i]) && (y == 255-to[i])) return;    
      else if ((x == from[i]) && (y != 255-to[i])) {          
	 /* change existing break point */
         to[i]=255-y;
	 DBGPRINT("set:change", from[i], to[i], i, x, y);
         return;   
      }
      else if ((x > from[i])   && (x < from[i+1])) {	      
	  /* first check if close enough to current breakpt 	*/
	  DBGPRINT("set:out", from[i], to[i], i, x, y);
	  if (((x >= from[i]-margin/2)   && (x <= from[i]+margin/2)) &&
	      ((y >= 255-to[i]-margin/2) && (y <= 255-to[i]+margin/2))) {
		DBGPRINT("set:close", from[i], to[i], i, x, y);
		if (i > 0 && i < *index)
		    from[i] = x;
		to[i] = 255 - y;
		return;
	  }
	  else
	  /* Check if close to next breakpoint	*/
	  if (((x >= from[i+1]-margin/2)   && (x <= from[i+1]+margin/2)) &&
	      ((y >= 255-to[i+1]-margin/2) && (y <= 255-to[i+1]+margin/2))) {
		DBGPRINT("set:close", from[i+1], to[i+1], i+1, x, y);
		if ((i+1) < *index)
		    from[i+1] = x;
		to[i+1] = 255 - y;
		return;
	  }
	  else
	  /* between breakpts, check if not inside box of adj breakpts */
          if ((y > 255-to[i]) || (y < 255-to[i+1])) return;   
	}
  }

  /*  Special case for x == 255	, last array element */
  if (i == *index && x == from[i]) {
      to[i] = 255 - y;
      DBGPRINT("set:last", from[i], to[i], i, x, y);
      return;
  }

  *index += 1;
  from[*index]=x;
  to[*index]=255-y;
  DBGPRINT("set:new", from[*index], to[*index], *index, x, y);
  while(!sorted) {
      sorted=1;
      for (i=0;i<*index;i++) {
         if (from[i+1] < from[i]) {
               fromtmp=  from[i+1];
               from[i+1]=from[i];
               from[i]=  fromtmp;
               totmp=    to[i+1];
               to[i+1]=  to[i];
               to[i]=    totmp;
               sorted=0;
         }
      }
  }
  return;
}

/*****************************************************************************/
/*
	ct_deletebreakpoint - delete breakpoint at mouse position
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_deletebreakpoint(from, to, index, x, y, margin)
long *from;
long *to;
long *index;
int x;
int y;
int margin;
#else
void ct_deletebreakpoint(long *from, long *to, long *index, int x, int y, int margin)
#endif
{
  int i,found=0;


  if (((x >= from[0]-margin/2)   && (x <= from[0]+margin/2)) &&
      ((y >= 255-to[0]-margin/2) && (y <= 255-to[0]+margin/2))) {
      from[0]=0;
      to[0]=0;
      return;
  }
  if (((x >= from[*index]-margin/2)   && (x <= from[*index]+margin/2)) &&
      ((y >= 255-to[*index]-margin/2) && (y <= 255-to[*index]+margin/2))) {
      from[*index]= 255;
      to[*index]=   255;
      return;
  }
  
  for (i=1;i<*index;i++) {			
      if (((x >= from[i]-margin/2)   && (x <= from[i]+margin/2)) &&
          ((y >= 255-to[i]-margin/2) && (y <= 255-to[i]+margin/2))) {
            found=1;  
      }
      if (found) {
         from[i]=from[i+1];
         to[i]=  to[i+1];
	DBGPRINT("del:found", from[i], to[i], i, x, y);
      }
  }
  if (found) *index -= 1;
}
#define NEWMOVEBREAK
/*****************************************************************************/
/*
	ct_findbreakpoint - find breakpoint at mouse position
*/
/*****************************************************************************/
#ifdef _NO_PROTO
int  ct_findbreakpoint(from, pto, index, x, y)
long *from;
long *pto;
long index;
int x;
int y;
#else
int  ct_findbreakpoint(long *from, long *pto, long index, int x, int y)
#endif
{
    int i;

    for (i=0;i<=index ;i++) {
	if  ((x == from[i]) && (y == pto[i])) break;
    }
    return (i);
}

/*****************************************************************************/
/*
	ct_movebreakpoint - move breakpoint to new mouse position
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_movebreakpoint(from, pto, index, x, y, nextx, nexty)
long *from;
long *pto;
long *index;
int x;
int y;
int *nextx;
int *nexty;
#else
void ct_movebreakpoint(long *from, long *pto, long *index, int x, int y, int *nextx, int *nexty)
#endif
{
    int	i;
    int found = False;
    int	newx, newy;

    newx = *nextx;
    newy = *nexty;
    /*  Constrain values to 0-255 for motion events which extend 
	beyond drawing area widget					*/
    if (newx < 0) newx = 0;
    if (newx > 255) newx = 255;

    if (newy < 0) newy = 0;
    if (newy > 255) newy = 255;

    /* Adjust for difference between X location and to value		*/
    y = 255 - y;
    newy = 255 -newy;

    /*  Find old breakpoint and make sure breakpoint is constrained 
       to a monitonically increasing contrast stretch			*/

    for (i=0;i<=*index ;i++) {
	if  ((x == from[i]) && (y == pto[i])) break;
	DBGPRINT("mov:search", from[i], pto[i], i, x, y);
    }
    if (i > *index) {
	*nextx = newx;
	*nexty = (255 - newy);
	return;
    }
    DBGPRINT("mov:found", from[i], pto[i], i, x, y);

    if (i > 0 && i < (*index))
    {

	if (newx <= from[i-1]) newx = from[i-1]+1;
	if (newx >= from[i+1]) newx = from[i+1]-1;

	if (newy <= pto[i-1]) newy = pto[i-1];
	if (newy >= pto[i+1]) newy = pto[i+1];
    }
    else
    if (i == 0)
    {

	newx = 0;
	if (newy >= pto[i+1]) newy = pto[i+1];
    }
    else
    if (i == (*index))
    {

	newx = 255;
	if (newy <= pto[i-1]) newy = pto[i-1];
    }

    from[i] = newx;
    pto[i] = newy;
    *nextx = newx;
    *nexty = (255 - newy);
    DBGPRINT("mov:set", from[i], pto[i], i, newx, newy);
}

/*****************************************************************************/
/*
	ct_breakpointlocation - display x , y position of current breakpoint
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_breakpointlocation(window,  gc, x, y, foreground)
Window window;
GC gc;
int x;
int y;
unsigned long foreground;
#else
void ct_breakpointlocation(Window window,  GC gc, int x, int y, unsigned long foreground)
#endif
{
   int 		xoffs, yoffs;
   char 	str[20];

   if ( x < 0 || x > 255 || y < 0 || y > 255 ) return;
#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (str, "%3.2f, %3.2f", x / 255.0, (255-y) / 255.0);
   else
#endif /* SATURATION */
   sprintf(str,"(%3d,%3d)",x,255-y);
   if ( x > 165 ) xoffs= -90;
   else           xoffs=  10;
   if ( y > 236 ) yoffs=   0;
   else           yoffs=  20;

   XSetForeground(xid.display,gc,foreground);
   XDrawString(xid.display,window,gc,x+xoffs,y+yoffs,str,strlen(str));
   XFlush(xid.display);
}

/*****************************************************************************/
/*
	ct_apply - apply the contrast stretch to the color specified.
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_apply(color)
XID_COLORTOOLPTR color;
#else
void ct_apply(XID_COLORTOOLPTR color)
#endif
{

   float   scaler; 	/* Used to scale gray level values to the
			   appropriate range (i.e. NUM_IMGCOLORS).    */

    long color_idx,     /* Index into color table.                    */
	 *from,		/* Pointer to "FROM" values.		      */
	 i,		/* Looping index.			      */
	 idx,		/* Index into piece-wise linear table.	      */
	 img_idx,	/* Index into colormap of the first image color.*/
	 init = FALSE,  /* Indicates whether or not the pwl color
			   table needs to be initialized.             */
	 map[256],
	 ncolors,       /* Number of colors to store in colormap.     */
	 nplanes,       /* Number of planes displayed.                */
	 num_break,	/* Number of break points.		      */
	 offset,        /* Offset used in scaling 24-bit values.      */
	 satfrom[256],	/* Saturation FROM break points.	      */
	 satto[256],	/* Saturation TO break points.		      */
	 start,         /* Starting index in the color table array.   */
	 *to;		/* Pointer to "TO" values.		      */


   /* 
    Transfer variables
   */
   xid_map.flags=	color->adjust.flags;

   /*  piece function expect the number of adjust entries not the last index */
   if (xid_map.flags == 0)
       {
       num_break =		color->adjust.index + 1;
       for (i=0; i < num_break; i++)
	   satfrom[i] = color->adjust.from[i];
       for (i=0; i < num_break; i++)
	   satto[i] = color->adjust.to[i];
       from = satfrom; 
       to = satto; 
       }
   if ((xid_map.flags & MAP_ALL) || (xid_map.flags & MAP_RED))
       {
       if (xid_pwl.red_from != NULL)
	   {
	   free(xid_pwl.red_from);
	   free(xid_pwl.red_to);
	   xid_pwl.red_from = xid_pwl.red_to = NULL;
	   xid_pwl.red_num = 0;
	   }
       xid_pwl.red_num=		color->adjust.index + 1;
       xid_pwl.red_from = (long *)(calloc(xid_pwl.red_num, sizeof(long)));	
       for (i=0; i < xid_pwl.red_num; i++)
	   xid_pwl.red_from[i] = color->adjust.from[i];
       xid_pwl.red_to = (long *)(calloc(xid_pwl.red_num, sizeof(long)));	
       for (i=0; i < xid_pwl.red_num; i++)
	   xid_pwl.red_to[i] = color->adjust.to[i];
       from = xid_pwl.red_from;
       to = xid_pwl.red_to;
       num_break = xid_pwl.red_num;
       }
   if ((xid_map.flags & MAP_ALL) || (xid_map.flags & MAP_GREEN))
       {
       if (xid_pwl.green_from != NULL)
	   {
	   free(xid_pwl.green_from);
	   free(xid_pwl.green_to);
	   xid_pwl.green_from = xid_pwl.green_to = NULL;
	   xid_pwl.green_num = 0;
	   }
       xid_pwl.green_num=	color->adjust.index + 1;
       xid_pwl.green_from = (long *)(calloc(xid_pwl.green_num, sizeof(long)));	
       for (i=0; i < xid_pwl.green_num; i++)
	   xid_pwl.green_from[i] = color->adjust.from[i];
       xid_pwl.green_to = (long *)(calloc(xid_pwl.green_num, sizeof(long)));	
       for (i=0; i < xid_pwl.green_num; i++)
	   xid_pwl.green_to[i] = color->adjust.to[i];
       if (xid_map.flags & MAP_GREEN)
	   {
           from = xid_pwl.green_from;
           to = xid_pwl.green_to;
           num_break = xid_pwl.green_num;
	   }
       }
   if ((xid_map.flags & MAP_ALL) || (xid_map.flags & MAP_BLUE))
       {
       if (xid_pwl.blue_from != NULL)
	   {
	   free(xid_pwl.blue_from);
	   free(xid_pwl.blue_to);
	   xid_pwl.blue_from = xid_pwl.blue_to = NULL;
	   xid_pwl.blue_num = 0;
	   }
       xid_pwl.blue_num=	color->adjust.index + 1;
       xid_pwl.blue_from = (long *)(calloc(xid_pwl.blue_num, sizeof(long)));	
       for (i=0; i < xid_pwl.blue_num; i++)
	   xid_pwl.blue_from[i] = color->adjust.from[i];
       xid_pwl.blue_to = (long *)(calloc(xid_pwl.blue_num, sizeof(long)));	
       for (i=0; i < xid_pwl.blue_num; i++)
	   xid_pwl.blue_to[i] = color->adjust.to[i];
       if (xid_map.flags & MAP_BLUE)
	   {
	   from = xid_pwl.blue_from;
	   to = xid_pwl.blue_to;
	   num_break = xid_pwl.blue_num;
	   }
       }

   /*
    Calculate the mapping.
   */

   piece(from,to,&num_break,color->map);

   /*
    Copy to red, green and blue if color is MAP_ALL
   */
   if (xid_map.flags & MAP_ALL)
   {
       for (i=0; i < 3; i++)
       {
	    int	j;
	    CT_Colors[i]->adjust.index = color->adjust.index;
	    for (j=0; j <= color->adjust.index; j++)
	    {
	       CT_Colors[i]->adjust.from[j] = color->adjust.from[j];
	       CT_Colors[i]->adjust.to[j] = color->adjust.to[j];
	    }
	    for (j=0; j < 256; j++)
	       CT_Colors[i]->map[j] = color->map[j];
       }
   }
   /*
    Based on the visual type, determine which portion of the colormap needs
    to be modified and stored.
   */

   if ((xid.visual_info[0].visual->class == PseudoColor) && 
        (*xid.colormap_ptr == xid.colormap)) {
        color_idx = img_idx = IMG_IDX;
        start =   0;
        ncolors = NUM_IMGCOLORS;
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) &&
             (*xid.colormap_ptr == xid.colormap)) {
         color_idx = 0;
         start = img_idx = IMG_IDX;
         ncolors = 256;
   }
   else {
         color_idx = start = img_idx = 0;
         ncolors = 256;
   }

   /*
    Allocate buffer space for the piece-wise linear mapping.
   */

   if (xid_pwl.color_pwl == NULL) {
       xid_pwl.color_pwl = (XColor *) calloc(ncolors,sizeof(XColor));
       if (xid_pwl.color_pwl == NULL) {
	  xid_message("Insufficient memory available");
	  return;
       }
       init = TRUE;
   }

   /*
    Calculate the new color table.
   */

   if ((*xid.colormap_ptr == xid.colormap) && 
       (xid.visual_info[0].visual->class == PseudoColor) && (init)) {
   /*
    Copy the current image colors.
   */
      for (i = 0; i < NUM_IMGCOLORS; i++) xid_pwl.color_pwl[i] = xid.color_table[color_idx + i];
   }
   else if (init) {
   /*
    Copy the reserved colors as well as the current image colors.
   */
       for (i = 0; i < 256; i++) xid_pwl.color_pwl[i] = xid.colortable_ptr[i];
   }

   /*
    Determine how many planes are currently being displayed.
   */

   if ((xid_view.rgbplanes[RED]    ==
        xid_view.rgbplanes[GREEN]) &&
       (xid_view.rgbplanes[GREEN] ==
        xid_view.rgbplanes[BLUE]))
        nplanes = 1;
   else nplanes = 3;

   /*
    Calculate the scaler needed to reduce the image to the appropriate number of colors.
   */

   if ((*xid.colormap_ptr == xid.colormap) && 
       (nplanes == 1)                      && 
       (xid_planes[xid_view.rgbplanes[RED]-1].maxval < NUM_IMGCOLORS))
       scaler = 1.0;
   else if (*xid.colormap_ptr == xid.colormap)
       scaler = (256.0 / (float)NUM_IMGCOLORS);           /* (2^8 / # of colors) */
   else
       scaler = 1.0;

   /*
    Apply the mapping to the color table.
   */
   if (xid.visual_info[0].visual->class == DirectColor) {
      if (*xid.colormap_ptr == xid.colormap)
         offset = (0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE + NUM_WMCOLORS + 1)));
      else
         offset = 0;

      for (i = img_idx; i < 256; i++) {
          idx = (long)(i * scaler + offset); 
	   xid_pwl.color_pwl[start + (i - img_idx)].red = red.map[idx] << 8; 
	   xid_pwl.color_pwl[start + (i - img_idx)].green = green.map[idx] << 8;
	   xid_pwl.color_pwl[start + (i - img_idx)].blue = blue.map[idx] << 8; 
	   xid_pwl.color_pwl[start + (i - img_idx)].flags = DoRed | DoGreen | DoBlue;
      }
       /*
	Store the colormap.
       */
       XStoreColors(xid.display,*xid.colormap_ptr,xid_pwl.color_pwl,ncolors);
   }
   if (xid.visual_info[0].visual->class == PseudoColor 
   || (CT_Img_Visual && CT_Img_Visual->visual->class == PseudoColor))
   {
       XColor	*psd_color, *store_psd_color;
       XColor	local_psd_color[256];

        if (CT_Img_Visual && CT_Img_Visual->visual->class == PseudoColor)
	{
	   psd_color = CT_Img_Color;
	   store_psd_color = local_psd_color;
	   ncolors = 200;
	   for (i=0; i<ncolors; i++) 
	       store_psd_color[i].pixel = psd_color[i].pixel;
	}
	else
	{
	   psd_color = &(xid.color_linear[img_idx]);
	   store_psd_color = xid_pwl.color_pwl;
        }
       for (i=img_idx; i < 256; i++) {
	store_psd_color[i - img_idx].red = 
		red.map[psd_color[i - img_idx].red >> 8] << 8;
	store_psd_color[i - img_idx].green = 
		green.map[psd_color[i - img_idx].green >> 8] << 8;
	store_psd_color[i - img_idx].blue = 
		blue.map[psd_color[i - img_idx].blue >> 8] << 8;
	store_psd_color[i - img_idx].flags = DoRed | DoGreen | DoBlue;

	/* The following conditional code is for saturation adjustments	*/
#ifdef SATURATION
	if (satrgb.adjust.index > 1
	|| satrgb.adjust.to[0] != 0
	|| satrgb.adjust.to[satrgb.adjust.index] != 255)
	{
	    int	satred, satgreen, satblue, satmap;
	    float hue, light, sat;

	    satred = store_psd_color[i-img_idx].red >> 8;
	    satgreen = store_psd_color[i-img_idx].green >> 8;
	    satblue = store_psd_color[i-img_idx].blue >> 8;
	    rgbhls (satred, satgreen, satblue, &hue, &light, &sat);

	    if (sat != 0.0)
	    {
		satmap = (int) (sat * 255.0 + .5);
		sat = ((float) satrgb.map[satmap] / 255.0 );
	    }

	    hlsrgb (hue, light, sat, &satred, &satgreen, &satblue);

	    store_psd_color[i - img_idx].red = satred << 8;
	    store_psd_color[i - img_idx].green = satgreen << 8;
	    store_psd_color[i - img_idx].blue = satblue << 8;
	    store_psd_color[i - img_idx].flags = DoRed | DoGreen | DoBlue;

	}
#endif /* SATURATION */
      }
       /*
	Store the colormap.
       */
        if (CT_Img_Visual && CT_Img_Visual->visual->class == PseudoColor)
	   XStoreColors(xid.display,CT_Img_CMap,store_psd_color,ncolors);
	else
	   XStoreColors(xid.display,*xid.colormap_ptr,store_psd_color,ncolors);
    }
#ifdef DEBUGCOLOR
    for (i=img_idx; i < 256; i++) {
	printf (" %d %d %d %d \n", (start+(i-img_idx)),
	    xid_pwl.color_pwl[start + (i - img_idx)].red >> 8,
	    xid_pwl.color_pwl[start + (i - img_idx)].green >> 8,
	    xid_pwl.color_pwl[start + (i - img_idx)].blue >> 8);
	
    }
#endif

   if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
   {

       xcmap_show_curr_pixel_color();
   }




}

/*****************************************************************************/
/*
	ct_drawingevent - handle events (Expose, Button, Motion) in
		contrast stretch drawing area
*/
/*****************************************************************************/
#ifdef _NO_PROTO
void ct_drawingevent(w, data, report)
Widget w;
XtPointer data;
XEvent *report;
#else
void ct_drawingevent(Widget w, XtPointer data, XEvent *report)
#endif
{
   XID_COLORTOOLPTR   	color= 	*( (XID_COLORTOOLPTR *) data);
   static int      	x,y,size=SIZEBREAKPTBOX;

#define INACTIVE	0
#define MOTION		1
#define SETBREAK	2
#define DELBREAK	3
   static int		MouseMode = INACTIVE; 

   void ct_set_sliders_text ();
#define CURRSTRETCH
#ifdef CURRSTRETCH
#endif

   if ( ! color->window ) 
   {/* only works after window has been mapped */

      /*  Kludge to set window since all colors are mapped to same window */
      red.window=XtWindow(w); 	
      green.window = XtWindow(w);
      blue.window = XtWindow(w);
      allrgb.window = XtWindow(w);
      satrgb.window = XtWindow(w);

      /* Initialize stretch list after window mapped */
      if (NumStretch == 0)
	 ct_init_stretch_list ();
   }

   switch(report->type) {
       case  Expose: 	   if (((XExposeEvent *)report)->count == 0)
				ct_drawline( color->window,
					color->gc,
			 		color->adjust.from,
			  		color->adjust.to,
					color->adjust.index,
					color->linecolor, size);
			   MouseMode = INACTIVE;
                           break;
       case  MotionNotify: ct_drawline( color->window,
					color->gc,
			 		color->adjust.from,
			  		color->adjust.to,
					color->adjust.index,
					color->background, size+size);
                           ct_breakpointlocation(color->window,color->gc,x,y,color->background);
#ifdef NEWMOVEBREAK
                	   ct_movebreakpoint(color->adjust.from,
				color->adjust.to,&color->adjust.index,
				x,y,&report->xbutton.x, &report->xbutton.y);  
			   x=report->xbutton.x;
			   y=report->xbutton.y;
#else
                	   ct_deletebreakpoint(color->adjust.from,color->adjust.to,&color->adjust.index,x,y,size);  
   		           x=report->xbutton.x;
                           y=report->xbutton.y;
                	   ct_setbreakpoint(color->adjust.from,color->adjust.to,&color->adjust.index,x,y,size);  
#endif
                     	   ct_drawline( color->window,
					color->gc,
			 		color->adjust.from,
			  		color->adjust.to,
					color->adjust.index,
					color->linecolor, size);
                           ct_breakpointlocation(color->window,color->gc,x,y,color->foreground);
                           ct_framedrawingarea(	color->window,
						color->gc,
			 			color->x,
			  			color->y,
						color->width,
						color->height,
						color->linecolor);
                           ct_apply(color);
			   ct_histo_draw (color);
			   /*  Too slow to update during pointer motion
			   ct_set_sliders_text (x, y);
			   ****************************/
			   MouseMode = MOTION;
                           break;
       case   ButtonPress: switch(report->xbutton.button) { 
                	       case Button1: 
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->background, size+size);
                           		x=report->xbutton.x;
                           		y=report->xbutton.y;
                	   		ct_setbreakpoint(color->adjust.from,
							 color->adjust.to,
							 &color->adjust.index,x,y,size);  
                           		ct_breakpointlocation(color->window,color->gc,x,y,color->foreground);
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->linecolor, size);
				       ct_set_sliders_text (x, y);
				       MouseMode = SETBREAK;
				       break;
                	       case Button2: 
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->background, size+size);
                           		x=report->xbutton.x;
                           		y=report->xbutton.y;
                	   		ct_deletebreakpoint(color->adjust.from,
							    color->adjust.to,
							    &color->adjust.index,x,y,size);  
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->linecolor, size);
				       ct_set_sliders_text (x, y);
				       MouseMode = DELBREAK;
				       break;
                               case Button3:       
                               default:      
				   MouseMode = INACTIVE;
				   break;
                }
                break;
       case ButtonRelease: switch(report->xbutton.button) { 
                               case Button1: 
#ifdef CURRSTRETCH
					if (MouseMode == MOTION 
					|| MouseMode == SETBREAK)
					{
					   ct_new_curr_stretch();
					   ct_set_sliders_text (x, y);
					   if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
					   {

					       xcmap_draw_colors();

					   }
					}
#endif

                           		ct_breakpointlocation(color->window,color->gc,x,y,color->background);
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->linecolor, size);
                               case Button2: 
                                             ct_apply(color);
					     ct_histo_draw (color);
#ifdef CURRSTRETCH
					if (MouseMode == DELBREAK)
					{
					   ct_new_curr_stretch();
					   if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
					   {
					       xcmap_draw_hls_colormap();
					   }
					}
#endif
   		                             break;
                               default:      break;
                }
		MouseMode = INACTIVE;
                break;
       default: break;
   }


}

/******************************************************************************/
/*
	
	New functions added for new colortool layout with singular
	stretch window.

*/
/******************************************************************************/



/*****************************************************************************/
/*
	ct_hist_type - toggle between cumulative and distibutional histograms
*/
/*****************************************************************************/
#define HIST_DIST	1
#define HIST_CUM	2
static int	CT_Hist_Type = HIST_DIST;
#define HIST_ALLSUM	1
#define HIST_ALLSTACKED 2
#define HIST_ALLLINE	3
static int	CT_Hist_Graph = HIST_ALLSTACKED;
/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_hist_type(w, hist_type, call)
Widget w;
char *hist_type;
XtPointer call;
#else
ct_hist_type(Widget w, char *hist_type, XtPointer call)
#endif
{

    if (*hist_type == 'D')
    {
	CT_Hist_Type = HIST_DIST;
    }
    else
    {
	CT_Hist_Type = HIST_CUM;
    }

    ct_histo_draw (CT_Color);



}


/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_hist_graph(w, hist_graph, call)
Widget w;
char *hist_graph;
XtPointer call;
#else
ct_hist_graph(Widget w, char *hist_graph, XtPointer call)
#endif
{

    if (*hist_graph == 'A')
    {
	CT_Hist_Graph = HIST_ALLSUM;
    }
    else
    if (*hist_graph == 'S')
    {
	CT_Hist_Graph = HIST_ALLSTACKED;
    }
    else
    {
	CT_Hist_Graph = HIST_ALLLINE;
    }

    ct_histo_draw (CT_Color);



}
/******************************************************************************/
/*
	ct_calc_adjusted_hist calculates a new histogram based on the 
	original histogram and the new mapping array.  
*/
/******************************************************************************/
#ifdef _NO_PROTO
void ct_calc_adjusted_hist (histptr, map, adjusted_hist)
long *histptr;
long *map;
long *adjusted_hist;
#else
void ct_calc_adjusted_hist (long *histptr, long *map, long *adjusted_hist)
#endif
{
    long freq;

    for (freq=0; freq < 256; freq++)
	adjusted_hist[freq] = 0;

    for (freq=0; freq < 256; freq++)
	adjusted_hist[map[freq]] += *(histptr+freq);
}
/******************************************************************************/
/*
	ct_calc_cumm_hist calculates a cummulative histogram and maximum
	number within the histogram.
*/
/******************************************************************************/
#ifdef _NO_PROTO
void ct_calc_cumm_hist (histptr, cum_histo, maxp)
long *histptr;
long *cum_histo;
long *maxp;
#else
void ct_calc_cumm_hist (long *histptr, long *cum_histo, long *maxp)
#endif
{
/*
    Calculate the cumulative histogram.  This is done regardless of the option
    chosen because the cumulative histogram is used in percentage calculations.
*/
    long freq;

    for (freq=0; freq < 256; freq++)
	if (*(histptr + freq) > 0)
	{
	    cum_histo[freq] = *(histptr + freq);
	    break;
	}
	else
	    cum_histo[freq] = 0;
    for (freq=freq+1; freq < 256; freq++)
	if (*(histptr + freq) >= 0)
	    cum_histo[freq] = *(histptr + freq) +
					    cum_histo[freq-1];
    else
	cum_histo[freq] = cum_histo[freq-1];
    /*
	Find maximum value.
    */
    *maxp = 0;
    for (freq = 0; freq <= 255; histptr++, freq++)
	if (*maxp < *histptr) *maxp = (float) *histptr;

}

/******************************************************************************/
/*
	ct_draw_tickmarks draws the horizontal and vertical tick marks for
	the histogram axes.  The tickmarks are drawn in separate 
	horizontal and vertical drawing areas adjacent to the histogram.  
*/
/******************************************************************************/
#define TICKSIZE	5
#ifdef _NO_PROTO
ct_draw_tickmarks (maxp, color)
long maxp;
XID_COLORTOOLPTR color;
#else
ct_draw_tickmarks (long maxp, XID_COLORTOOLPTR color)
#endif
{
    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    long	x1, x2, y1, y2;
    long	freq;

    Dimension	hist_horiz_lab_height;	
    Dimension	hist_horiz_lab_width;
    Dimension	hist_vert_lab_height;	
    Dimension	hist_vert_lab_width;

    long    c_width,        /* Calculated font character width.             */
	    c_height;       /* Calculated font character height.            */
    /*
    Calculate the width and height of characters in the working font.
    */
    c_width = xid.font_info->max_bounds.rbearing -
	      xid.font_info->min_bounds.lbearing;

    c_height = xid.font_info->max_bounds.ascent +
	      xid.font_info->max_bounds.descent;

    /*  Draw tick marks on the horizontal axis	*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNheight, &hist_horiz_lab_height); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, &hist_horiz_lab_width); argcnt++;
    XtGetValues(ct_histo_horiz_label, args, argcnt);

    XSetForeground(xid.display,color->gc,color->background);
    XDrawLine (xid.display, XtWindow(ct_histo_horiz_label),
				color->gc, 0,0, hist_horiz_lab_width, 0);
    for (freq = 255; freq > 0; freq=freq-64)
    {
	char	xlabel[10];

	y2 = TICKSIZE;
	x1 = x2 = freq;
	y1 = 0;
	XDrawLine (xid.display, XtWindow(ct_histo_horiz_label),
				color->gc, x1,y1, x2, y2);
	sprintf (xlabel, "%d", freq);
	XDrawString(xid.display,XtWindow(ct_histo_horiz_label),
			color->gc,x1-((strlen(xlabel))*c_width),
			(hist_horiz_lab_height),
			xlabel,strlen(xlabel));

    }
    /*  Draw tick marks on the vertical axis	*/
    XClearWindow (xid.display, XtWindow(ct_histo_vert_label));
    argcnt = 0;
    XtSetArg(args[argcnt], XmNheight, &hist_vert_lab_height); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, &hist_vert_lab_width); argcnt++;
    XtGetValues(ct_histo_vert_label, args, argcnt);
    XDrawLine (xid.display, XtWindow(ct_histo_vert_label),
				color->gc, 0,0,0, hist_vert_lab_height);
    for (y2=0; y2 < hist_vert_lab_height; y2=y2+(hist_vert_lab_height/4))
    {
	char	ylabel[10];

	x1 = 0;
	x2 = TICKSIZE;
	y1 = y2;
	XDrawLine (xid.display, XtWindow(ct_histo_vert_label),
				color->gc, x1,y1, x2, y2);

	sprintf (ylabel, "%d", (hist_vert_lab_height - y2) 
		* (maxp/hist_vert_lab_height));
	XDrawString(xid.display,XtWindow(ct_histo_vert_label),
			color->gc,x2,
			(y2+c_height),
			ylabel,strlen(ylabel));

    }
}


/******************************************************************************/
/*

	ct_histo_stat_pct calculates the percent statistics for the histogram
	and displays the statistics on the histogram info labels.  

*/
/******************************************************************************/

static	  Window	Histo_win;
static	  Widget	Histo_widget;
static    long		Histo_max;
static	  float 	Histo_scale = 1.0;
static    Dimension	Histo_height;	
static    Dimension	Histo_width;

void
#ifdef _NO_PROTO
ct_histo_stat_pct (histptr, cum_histo, horiz_value)
long *histptr;
long *cum_histo;
long horiz_value;
#else
ct_histo_stat_pct ( long *histptr, long *cum_histo, long horiz_value)
#endif
{
    float	percent;
    XmString    xmstr[256];
    char	charlabel [256];
    static	int	histlastx=0, histlasty=0;
    int		x1, y1, x2, y2;

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    if (xid.view == -1)
	return;
#ifdef SATURATION
    {

    if (CT_Color == &satrgb)
	return;
    }
#endif /* SATURATION */

    percent = (cum_histo[horiz_value] / (float) cum_histo[255]) * 100;
    sprintf (charlabel, "%3.2f%% of Data\n   at Pixel %d",
	     percent, horiz_value);
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_pct, args, argcnt);

#ifdef HISTOINDICATOR
    /*  Draw indicator lines to make it easier to read histogram	*/
    XSetForeground (xid.display, CT_Color->gc, 
	CT_Color->foreground);
    XSetFunction (xid.display, CT_Color->gc, GXxor);
    XDrawLine (xid.display, Histo_win, CT_Color->gc, 
	histlastx,Histo_height, histlastx, histlasty);
    XDrawLine (xid.display, Histo_win, CT_Color->gc, 
	histlastx,histlasty, 255, histlasty);

    x1 = x2 = horiz_value;
    y2 = Histo_height - ((histptr[x1] / (float) Histo_max) * Histo_height);
    y1 = Histo_height;

    XDrawLine (xid.display, Histo_win, CT_Color->gc, 
	x1, y1, x2, y2);
    XDrawLine (xid.display, Histo_win, CT_Color->gc, 
	x1, y2, 255, y2);
    XSetFunction (xid.display, CT_Color->gc, GXcopy);
    
    histlastx = horiz_value;
    histlasty = y2;
#endif

}
/******************************************************************************/
/*

	ct_histo_statistics calculates the statistics for the histogram
	and displays the statistics on the histogram info labels.  

*/
/******************************************************************************/
void
#ifdef _NO_PROTO
ct_histo_statistics (histptr, cum_histo)
long *histptr;
long *cum_histo;
#else
ct_histo_statistics ( long *histptr, long *cum_histo)
#endif
{

    double	mean, stdev;
    float	percent;
    long	ct_min, ct_max;

    XmString    xmstr[256];
    char	charlabel [256];

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    long	horiz_value = CT_Color->lastx;

    xid_histcalc (histptr, &ct_min, &ct_max, &mean, &stdev);

    sprintf (charlabel, "Mean    = %f", mean);
    charlabel [strlen(charlabel)-4] = '\0';
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_mean, args, argcnt);
    XmStringFree (xmstr[0]);

    sprintf (charlabel, "St Dev  = %f", stdev);
    charlabel [strlen(charlabel)-4] = '\0';
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_stdev, args, argcnt);
    XmStringFree (xmstr[0]);

    sprintf (charlabel, "Min Pix = %d", ct_min);
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_min, args, argcnt);
    XmStringFree (xmstr[0]);

    sprintf (charlabel, "Max Pix = %d", ct_max);
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_max, args, argcnt);
    XmStringFree (xmstr[0]);
     
    ct_histo_stat_pct (histptr, cum_histo, horiz_value);
}
/******************************************************************************/
/*

	ct_clear_statistics clears the histogram statistics labels.

*/
/******************************************************************************/
void
#ifdef _NO_PROTO
ct_clear_statistics (color)
XID_COLORTOOLPTR color;
#else
ct_clear_statistics ( XID_COLORTOOLPTR	color)
#endif
{

    XmString    xmstr[256];
    char	charlabel [256];

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    charlabel [0] = '\0';
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_mean, args, argcnt);
    XtSetValues (ct_hist_info_stdev, args, argcnt);
    XtSetValues (ct_hist_info_min, args, argcnt);
    XtSetValues (ct_hist_info_max, args, argcnt);
    XtSetValues (ct_hist_info_pct, args, argcnt);
    XmStringFree (xmstr[0]);
}
/******************************************************************************/
/*

	ct_histo_draw draws the histogram for the adjusted color.  If no
	image is displayed, a message is displayed in the histogram window.

*/
/******************************************************************************/
#ifdef _NO_PROTO
void ct_histo_draw (color)
XID_COLORTOOLPTR color;
#else
void ct_histo_draw (XID_COLORTOOLPTR color)
#endif
{

    static char message[] = "No image data for histogram";
    long	bnd;
    long	x1, x2, y1, y2;
    long	freq, lastfreq = -1;

    unsigned long	*histptr;

    unsigned long	*adjusted_hist=color->adjusted_hist;
    unsigned long	*map ;
    int count;
    unsigned long	*cum_histo=color->cum_histo;

    long	all_hist[256];

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    if (xid.view == -1)
    {

	/*  Display message specifying no image displayed	*/

       XSetForeground(xid.display,color->gc,color->background);
       XDrawString(xid.display,Histo_win,color->gc,10,10,
		   message,strlen(message));
       ct_clear_statistics (color);
       XFlush(xid.display);
       return;
    }

    
#ifdef SATURATION
    if (color == &satrgb)
    {
    static char satmsg[] = "No histogram for saturation";

	/*  Display message specifying no image displayed	*/

	XClearWindow (xid.display, Histo_win);
        XSetForeground(xid.display,color->gc,color->background);
        XDrawString(xid.display,Histo_win,color->gc,10,100,
		   satmsg,strlen(satmsg));
        ct_clear_statistics (color);
        XFlush(xid.display);
        return;
    }
#endif /* SATURATION */

    /* Find the color band associated with the color adjustment	*/
    /* The color band is used to index the array of planes to 	*/
    /* Find the histogram data which is generated when the image*/
    /* is initially loaded.					*/

#ifdef OLDXID
    if (color->adjust.flags & MAP_RED) 
	bnd =xid_composite[xid.view].rgbplanes[0];
    if (color->adjust.flags & MAP_GREEN) 
	bnd =xid_composite[xid.view].rgbplanes[1];
    if (color->adjust.flags & MAP_BLUE) 
	bnd =xid_composite[xid.view].rgbplanes[2];

    /*  Kludge - need special handling for ALL	*/
    if (color->adjust.flags & MAP_ALL) 
	bnd =xid_composite[xid.view].rgbplanes[0];
#else
    if (color->adjust.flags & MAP_RED) 
	bnd =xid_view.rgbplanes[0];
    if (color->adjust.flags & MAP_GREEN) 
	bnd =xid_view.rgbplanes[1];
    if (color->adjust.flags & MAP_BLUE) 
	bnd =xid_view.rgbplanes[2];

    /*  Kludge - need special handling for ALL	*/
    if (color->adjust.flags & MAP_ALL) 
	bnd =xid_view.rgbplanes[0];
#endif OLDXID
   


    /*  Get the histogram adjusted for the piecewise linear mapping	*/
    if (color->adjust.flags & MAP_ALL) 
    {
	int	p, i;

	for (i=0;i<256;i++) color->adjusted_hist[i] = 0;
	for (p=0; p<3; p++)
	{
	    /*  Calculate the adjusted histogram for each color */
	    bnd =xid_view.rgbplanes[p];
	    histptr = (unsigned long *)xid_planes[bnd-1].histogram;
	    map = CT_Colors[p]->map;
	    adjusted_hist = CT_Colors[p]->adjusted_hist;
	    cum_histo = CT_Colors[p]->cum_histo;
	    ct_calc_adjusted_hist (histptr, map, adjusted_hist);

	    /*  Sum each colors histogram values in ALL color	*/
	    for (i=0;i<256;i++) color->adjusted_hist[i] += adjusted_hist[i];

	    /*  Calculate cummmulative histogram for each color	*/
	    histptr = adjusted_hist;
	    ct_calc_cumm_hist (histptr, cum_histo, &Histo_max);
	}
	/* Find the maximum value of the histogram for autoscaling the y axis*/
	histptr = color->adjusted_hist;
	adjusted_hist = color->adjusted_hist;
	cum_histo = color->cum_histo;
	ct_calc_cumm_hist (histptr, cum_histo, &Histo_max);

    }
    else
    {
	/*  Get new piecewise linear adjusted values for the color	*/
	map = color->map;
	histptr = (unsigned long *)xid_planes[bnd-1].histogram;
	ct_calc_adjusted_hist (histptr, map, adjusted_hist);
	/* Find the maximum value of the histogram for autoscaling the y axis*/
	histptr = adjusted_hist;
	ct_calc_cumm_hist (histptr, cum_histo, &Histo_max);
    }


    if (CT_Hist_Type == HIST_CUM)
    {
	histptr = cum_histo;

	Histo_max = cum_histo[255];

    }
    Histo_max = Histo_max * Histo_scale;
    /*  Draw the histogram in the color of the color being adjusted	*/
    /*  The size of the histogram is set for a 1 pixel x value or 256	*/
    XClearWindow (xid.display, Histo_win);
    XSetForeground(xid.display,color->gc,color->linecolor);
    XSetLineAttributes(xid.display,color->gc,1,LineSolid,CapButt,JoinMiter);

    /*  Get height, width of drawing area for histogram	*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNheight, &Histo_height); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, &Histo_width); argcnt++;
    XtGetValues(Histo_widget, args, argcnt);

    for (freq = 0; freq < 256; histptr++, freq++)
    {
        if (*histptr > 0)
	{
#define HISTALLSTACKED
#ifdef HISTALLSTACKED
	    if ((color->adjust.flags & MAP_ALL) 
	    && (CT_Hist_Graph != HIST_ALLSUM)) 
	    {
	      unsigned int	histval1, histval2;
	      int	p, newy1=Histo_height;
	      x1 = -1;

	      for (p=0; p<3; p++)
	      {
		XSetForeground(xid.display,CT_Colors[p]->gc,
			CT_Colors[p]->linecolor);
		if (CT_Hist_Type == HIST_CUM)
		{
		    if (lastfreq != -1)
		      histval1 = CT_Colors[p]->cum_histo[lastfreq];
		    histval2 = CT_Colors[p]->cum_histo[freq];
		}
		else
		{
		    if (lastfreq != -1)
		      histval1 = CT_Colors[p]->adjusted_hist[lastfreq];
		    histval2 = CT_Colors[p]->adjusted_hist[freq];
		}
		if (CT_Hist_Graph == HIST_ALLSTACKED)
		{
		  /*  Stacked bar chart for all */
	          y2 = ((histval2 / (float) Histo_max) * Histo_height);
		  y2 = newy1 - y2;
		  if (y2 < 0 ) y2 = 0;
		  x1 = x2 = freq;
		  y1 = newy1;
		  XDrawLine (xid.display, Histo_win, color->gc, x1,y1, x2, y2);
		  newy1 = y2;
		}
		else
		if (CT_Hist_Graph == HIST_ALLLINE)
		{
		  /*  Line graph for each color		*/
		  if (lastfreq != -1 && lastfreq != freq)
		  {
		      y1 = ((histval1 / (float) Histo_max) * Histo_height);
		      x1 = lastfreq;
		      y1 = Histo_height - y1;
		      y2 = ((histval2 / (float) Histo_max) * Histo_height);
		      x2 = freq;
		      y2 = Histo_height - y2;
		      XDrawLine (xid.display, Histo_win, color->gc, x1, y1,
			  x2, y2);
		  }
		}
	      }
	      lastfreq = freq;
	    }
	    else
#endif /*  HISTALLSTACKED  */
	    {
	        y2 = Histo_height - ((*histptr / (float) Histo_max) 
			* Histo_height);
		if (y2 < 0 ) y2 = 0;
		x1 = x2 = freq;
		y1 = Histo_height;
		XDrawLine (xid.display, Histo_win, color->gc, x1,y1, x2, y2);
	    }
	}
    }

    ct_draw_tickmarks (Histo_max, color);

    histptr = adjusted_hist;
    ct_histo_statistics (histptr, cum_histo);

#ifdef HISTISSTRETCH
    ct_drawline( Histo_win,
	    color->gc,
	    color->adjust.from,
	    color->adjust.to,
	    color->adjust.index,
	    color->linecolor, 5);
#endif HISTISSTRETCH
}

#ifdef _NO_PROTO
void ct_histo_notify(w, data, report)
Widget w;
XtPointer data;
XEvent *report;
#else
void ct_histo_notify(Widget w, XtPointer data, XEvent *report)
#endif
{
    /*
    printf("Got notify event\n");
    */
    Histo_win = XtWindow (ct_histo_red);
    Histo_widget = ct_histo_red;
}
/******************************************************************************/
/*
	
	This function is called for expose events on the histogram drawing
	area. 

*/
/******************************************************************************/
#ifdef _NO_PROTO
void ct_histo_expose(w, data, report)
Widget w;
XtPointer data;
XExposeEvent *report;
#else
void ct_histo_expose(Widget w, XtPointer data, XExposeEvent *report)
#endif
{
    XID_COLORTOOLPTR   	color= 	*( (XID_COLORTOOLPTR *) data);

#ifdef DEBUGEXPOSE
    printf("XExposeEvent type = %d, serial = %d, x = %d, y = %d, width = %d, height = %d, count = %d\n", report->type, report->serial, report->x, report->y, report->width, report->height, report->count);
#endif

    if (report->count == 0)
	ct_histo_draw (color);
}
/******************************************************************************/
/*
	
	This function is called for pointer motion events on the 
	histogram drawing area. Keyboard events are also interpreted for
	cursor keys and vi control keys.

*/
/******************************************************************************/
#ifdef _NO_PROTO
void ct_histo_pointer(w, data, report)
Widget w;
XtPointer data;
XEvent *report;
#else
void ct_histo_pointer(Widget w, XtPointer data, XEvent *report)
#endif
{
    XEvent *event;

    long deltax, deltay, move;

    long keycode, state;


    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    /*
	Get the event values from the widget.
    */
    event = (XEvent *) report;

    /*
	Remove all other MotionNotify events from queue -- this makes for
	smoother scrolling by helping to limit the number of events
	generated while we're acting on the present one.
    */
    while (XCheckTypedEvent(xid.display,MotionNotify,event));

    /*
	If a a valid cursor key was pressed on the keyboard then move
	the cursor to the new location.
    */
    if (event->type == KeyRelease)
    {
    /*
	Check the validity of the cursor coordinates.
    */

	if ((event->xkey.x < 0) || (event->xkey.y < 0) ||
	    (event->xkey.x > Histo_width) || (event->xkey.y > Histo_height))
	    return;

    /*
	Capture the key code and the keyboard state.
    */
	keycode = event->xkey.keycode;
	state   = event->xkey.state;

    /*
	Ensure the keys validity.
    */
	if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.j_key) ||
	    (keycode == xid_cursor.h_key) || (keycode == xid_cursor.l_key) ||
	    (keycode == xid_cursor.left_key) || (keycode == xid_cursor.up_key) ||
	    (keycode == xid_cursor.right_key) || (keycode == xid_cursor.down_key))
	    {
    /*
	Reset the delta X and Y amounts.
    */
	    deltax = deltay = 0;

    /*
	Check the keyboard state to see if the Shift key was used.
    */
	    if (state == ShiftMask)
		move = 10;               /* Shift key so use delta of 10.     */
	    else
		move = 1;

    /*
	Set the delta X/Y amount based on the key pressed.
    */
	    if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.up_key))
		deltay -= move;
	    else if ((keycode == xid_cursor.j_key) || (keycode ==
		xid_cursor.down_key))
		deltay += move;
	    else if ((keycode == xid_cursor.h_key) || (keycode ==
		xid_cursor.left_key))
		deltax -= move;
	    else
		deltax += move;

    /*
	If a valid move can be made then move the cursor.
    */
	    if (((event->xkey.x + deltax) < 0) 
	    || ((event->xkey.x + deltax) > Histo_width)
	    || ((event->xkey.y + deltay) < 0) 
	    || ((event->xkey.y + deltay) > Histo_height))
		;
	    else
		XWarpPointer(xid.display,None,None,None,None,None,None,
		    deltax,deltay);
	    }
    }
    /*
    Calculate the gray level value corresponding to the cursor position if the
    cursor is within the bounds of the X axis of the histogram.
    */

    ct_histo_stat_pct (CT_Color->adjusted_hist,
		       CT_Color->cum_histo,
		        (event->xmotion.x ));

}
/******************************************************************************/
/*
	
	This function changes the context of the current contrast stretch
	drawing area to the color specified in the client data. The 
	current color is erased and the new color drawn in the same
	drawing area.  
*/
/******************************************************************************/
static char	*confirm_all_msg = 
	 "ALL resets RGB stretches to ALL stretch\nPlease confirm" ;

void
#ifdef _NO_PROTO
ct_set_sliders_text (newx, newy)
int newx;
int newy;
#else
ct_set_sliders_text (int newx, int newy)
#endif
{

    char	value_string[8];

    newy = 255 - newy;
    if (CT_Color->lasty != newy)
    {
	CT_Color->lasty = newy;
#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", CT_Color->lasty / 255.0);
   else
#endif /* SATURATION */
	sprintf (value_string, "%3d", CT_Color->lasty);
	XmTextFieldSetString (ct_vert_value, value_string);
	XmScaleSetValue (ct_scale_vertical, CT_Color->lasty);
    }
    if (CT_Color->lastx != newx)
    {
	CT_Color->lastx = newx;
#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", CT_Color->lastx / 255.0);
   else
#endif /* SATURATION */
	sprintf (value_string, "%3d", CT_Color->lastx);
	XmTextFieldSetString (ct_horiz_value, value_string);
	XmScaleSetValue (ct_scale_horiz, CT_Color->lastx);
    }

}

void
#ifdef _NO_PROTO
ct_changecolor_func (color)
XID_COLORTOOLPTR color;
#else
ct_changecolor_func (XID_COLORTOOLPTR color)
#endif
{
	char	value_string[8];

	ct_drawline( CT_Color->window,
		CT_Color->gc,
		CT_Color->adjust.from,
		CT_Color->adjust.to,
		CT_Color->adjust.index,
		CT_Color->background, SIZEBREAKPTBOX);


	CT_Color = color;

	
	ct_drawline( CT_Color->window,
		CT_Color->gc,
		CT_Color->adjust.from,
		CT_Color->adjust.to,
		CT_Color->adjust.index,
		CT_Color->linecolor, SIZEBREAKPTBOX);
	
	ct_apply (CT_Color);

	ct_histo_draw (CT_Color);

#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", CT_Color->lasty / 255.0);
   else
#endif /* SATURATION */
	sprintf (value_string, "%3d", CT_Color->lasty);
	XmTextFieldSetString (ct_vert_value, value_string);
	XmScaleSetValue (ct_scale_vertical,  CT_Color->lasty);
#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", CT_Color->lastx / 255.0);
   else
#endif /* SATURATION */
	sprintf (value_string, "%3d", CT_Color->lastx);
	XmTextFieldSetString (ct_horiz_value, value_string);
	XmScaleSetValue (ct_scale_horiz,  CT_Color->lastx);
}

void
ct_change_to_all()
{

    ct_changecolor_func (&allrgb);
    XmToggleButtonSetState (*CT_ColorWidget[STRETCHALL], 
			True, False);

}
void
ct_cancel_all ()
{
    int i;

    for (i=0; i<2; i++)
    {
	if (CT_Colors[i] == CT_Color)
	    XmToggleButtonSetState (*CT_ColorWidget[i], 
			True, False);
    }
    XmToggleButtonSetState (*CT_ColorWidget[STRETCHALL], 
	    False, False);
}


/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_changecolor(w, client, call)
Widget w;
XID_COLORTOOLPTR client;
XtPointer call;
#else
ct_changecolor(Widget w, XID_COLORTOOLPTR client, XtPointer call)
#endif
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

#ifdef SATURATION
	if (client == &allrgb)
#else
	if (client->adjust.flags & MAP_ALL)
#endif
	{
	    int	i,j, diff = False, curr_color;
	    /*  
		Check if any colors have been updated and
		warn user that ALL will override these updates
 	    */
	    for (i=0; i<3;i++)
	    {
		if (CT_Colors[i] == CT_Color)
		    curr_color = i;
		if (CT_Colors[i]->adjust.index != client->adjust.index)
		    diff = True;
		else
		    for (j=0; j<= client->adjust.index;j++)
		    {
			if (CT_Colors[i]->adjust.from[j] 
			!= client->adjust.from[j]) 
			    diff = True;
			if (CT_Colors[i]->adjust.to[j] 
			!= client->adjust.to[j]) 
			    diff = True;
		    }
	    }
	    if (diff)
	    {
		XmToggleButtonSetState (w, False, False);
		/*
		XmToggleButtonSetState (*CT_ColorWidget[curr_color], 
			True, False);
		*/
		ct_get_confirmation (confirm_all_msg, 
			ct_change_to_all, ct_cancel_all);
		return;
	    }
	}

	client->window = CT_Color->window;
	ct_changecolor_func (client);
}


void
#ifdef _NO_PROTO
ct_new_vertical_value (x, y)
int x;
int y;
#else
ct_new_vertical_value (int x, int y)
#endif
{
	int newx, newy;
	/*  Erase previous line by drawing with background color	*/
	ct_drawline( CT_Color->window,
		CT_Color->gc,
		CT_Color->adjust.from,
		CT_Color->adjust.to,
		CT_Color->adjust.index,
		CT_Color->background, SIZEBREAKPTBOX);

	/*  setbreakpoint expects y in X coordinate style	*/
	newx = x;
	newy = 255 - y;
	ct_movebreakpoint (CT_Color->adjust.from,
               CT_Color->adjust.to,
               &CT_Color->adjust.index,x, 255-CT_Color->lasty, 
	       &newx, &newy);

	ct_drawline( CT_Color->window,
		CT_Color->gc,
		CT_Color->adjust.from,
		CT_Color->adjust.to,
		CT_Color->adjust.index,
		CT_Color->linecolor, SIZEBREAKPTBOX);

	ct_apply (CT_Color);
        ct_histo_draw (CT_Color);

	CT_Color->lasty = 255 - newy;
}
/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_change_scale_vertical(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_change_scale_vertical(Widget w, XtPointer client, XtPointer call)
#endif
{
        XmScaleCallbackStruct *scale =(XmScaleCallbackStruct*)call;
	int	x, y;
	char	value_string[8];

	y = scale->value;
	x = CT_Color->lastx;

	ct_new_vertical_value (x, y);
#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", CT_Color->lasty / 255.0);
   else
#endif /* SATURATION */
	sprintf (value_string, "%3d", CT_Color->lasty);
	XmTextFieldSetString (ct_vert_value, value_string);
	if (CT_Color->lasty != y)
	    XmScaleSetValue (ct_scale_vertical, CT_Color->lasty);
#ifdef CURRSTRETCH
	ct_new_curr_stretch();
#endif
       if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
       {
	   xcmap_draw_hls_colormap();
       }
}

/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_drag_scale_vertical(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_drag_scale_vertical(Widget w, XtPointer client, XtPointer call)
#endif
{
        XmScaleCallbackStruct *scale =(XmScaleCallbackStruct*)call;
	int	x, y;
	char	value_string[8];

	y = scale->value;
	x = CT_Color->lastx;

	ct_new_vertical_value (x, y);
#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", CT_Color->lasty / 255.0);
   else
#endif /* SATURATION */
	sprintf (value_string, "%3d", CT_Color->lasty);
	XmTextFieldSetString (ct_vert_value, value_string);
	if (CT_Color->lasty != y)
	    XmScaleSetValue (ct_scale_vertical, CT_Color->lasty);
}


/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_change_vert_text(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_change_vert_text(Widget w, XtPointer client, XtPointer call)
#endif
{

    char	*textstring;
    int	result, y;
    float fy;

    textstring = XmTextFieldGetString (w);

#ifdef SATURATION

    if (CT_Color == &satrgb)
    {
	result = sscanf (textstring, "%f", &fy);
	y = (int) (fy * 255.0 + .5);
    }
    else
#endif /* SATURATION */
    result = sscanf (textstring, "%d", &y);

    if (result == 1 && (y >= 0 && y < 256)) 
    {
	ct_new_vertical_value (CT_Color->lastx, y);
	XmScaleSetValue (ct_scale_vertical, CT_Color->lasty);
	if (CT_Color->lasty != y)
	{
	    char value_string [8];

#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", CT_Color->lasty / 255.0);
   else
#endif /* SATURATION */
	    sprintf (value_string, "%3d", CT_Color->lasty);
	    XmTextFieldSetString (ct_vert_value, value_string);
	}
#ifdef CURRSTRETCH
	ct_new_curr_stretch();
#endif
       if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
       {
	   xcmap_draw_hls_colormap();
       }
		
    }
}



void
#ifdef _NO_PROTO
ct_new_horizontal_value (x, setbreakpoint)
int x;
Boolean setbreakpoint;
#else
ct_new_horizontal_value (int x, Boolean	setbreakpoint)
#endif
{
	int y, map [256];
	int count;
	int movey;
	int	brkpt;
	XID_COLORTOOLPTR color = CT_Color;

	char	value_string[8];


	y = color->map [x];
	XmScaleSetValue (ct_scale_vertical, y);

#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", x / 255.0);
   else
#endif /* SATURATION */
	sprintf (value_string, "%3d", x);
	XmTextFieldSetString (ct_horiz_value, value_string);
#ifdef SATURATION

   if (CT_Color == &satrgb)
       sprintf (value_string, "%3.2f", y / 255.0);
   else
#endif /* SATURATION */
	sprintf (value_string, "%3d", y);
	XmTextFieldSetString (ct_vert_value, value_string);
	
	ct_histo_stat_pct (color->adjusted_hist,
		       color->cum_histo, x);
	if (setbreakpoint)
	{
	    /*  Erase previous line by drawing with background color*/
	    ct_drawline( color->window,
		    color->gc,
		    color->adjust.from,
		    color->adjust.to,
		    color->adjust.index,
		    color->background, SIZEBREAKPTBOX);

	    /*  Erase lines showing scale position		*/
	    XSetForeground(xid.display,color->gc,
		color->background);
	    XDrawLine (xid.display, color->window,
		color->gc, color->lastx, 255, 
		color->lastx, 
		255-(color->lasty-1));
	    XDrawLine (xid.display, color->window,
		color->gc, color->lastx, 
		255-(color->lasty-1), 
		255, 255-(color->lasty-1));
	    brkpt = ct_findbreakpoint (color->adjust.from,
			color->adjust.to, 
			color->adjust.index, 
			color->lastx, 
			color->lasty);
	    if (brkpt <  color->adjust.index)
		ct_breakpointlocation(color->window,color->gc,
		    color->lastx, 255-color->lasty,color->background);


	    /*  setbreakpoint expects y in X coordinate style	*/
	    movey = 255 - y;
	    if (color->lastx == (x+1)
	    ||  color->lastx == (x-1))
		ct_movebreakpoint(color->adjust.from,
		    color->adjust.to,
		    &color->adjust.index,
		    color->lastx, 255-color->lasty, 
		    &x, &movey);
	    else
		/*  This sets the breakpoint to the exact value	*/
		ct_setbreakpoint (color->adjust.from,
		   color->adjust.to,
		   &color->adjust.index,x, movey,
		   0);

	    ct_drawline( color->window,
		    color->gc,
		    color->adjust.from,
		    color->adjust.to,
		    color->adjust.index,
		    color->linecolor, SIZEBREAKPTBOX);
#ifdef CURRSTRETCH
	    ct_new_curr_stretch();
#endif
	   if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
	   {
	       xcmap_draw_hls_colormap();
	   }

	}
	else
	{
	    /*  Draw lines to show scale position	*/

	    XSetForeground(xid.display,color->gc,
		color->background);
	    XDrawLine (xid.display, color->window,
		color->gc, color->lastx, 255, 
		color->lastx, 
		255-(color->lasty-1));
	    XDrawLine (xid.display, color->window,
		color->gc, color->lastx, 
		255-(color->lasty-1), 
		255, 255-(color->lasty-1));

	    brkpt = ct_findbreakpoint (color->adjust.from,
			color->adjust.to, 
			color->adjust.index, 
			color->lastx, 
			color->lasty);
	    if (brkpt <  color->adjust.index)
		ct_breakpointlocation(color->window,color->gc,
		    color->lastx, 255-color->lasty,color->background);

	    XSetForeground(xid.display,color->gc,
		color->foreground);
	    XDrawLine (xid.display, color->window,
		color->gc, x, 255, x, 255-(y-1));
	    XDrawLine (xid.display, color->window,
		color->gc, x, 255-(y-1), 255, 255-(y-1));

	    brkpt = ct_findbreakpoint (color->adjust.from,
			color->adjust.to, 
			color->adjust.index, 
			x, y);
	    if (brkpt <  color->adjust.index)
		ct_breakpointlocation(color->window,color->gc,
		    x, 255-y ,color->foreground);

	    ct_drawline( color->window,
		    color->gc,
		    color->adjust.from,
		    color->adjust.to,
		    color->adjust.index,
		    color->linecolor, SIZEBREAKPTBOX);
	}
	color->lasty = y;

}

/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_change_scale_horiz(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_change_scale_horiz(Widget w, XtPointer client, XtPointer call)
#endif
{
        XmScaleCallbackStruct *scale =(XmScaleCallbackStruct*)call;
	int	x;

#ifdef DEBUG
	printf ("Value of scale = %d\n", scale->value);
#endif DEBUG

	x = scale->value;

	ct_new_horizontal_value (x, True);
	CT_Color->lastx = x;
}

/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_drag_scale_horiz(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_drag_scale_horiz(Widget w, XtPointer client, XtPointer call)
#endif
{
        XmScaleCallbackStruct *scale =(XmScaleCallbackStruct*)call;

	int	x, y;

	x = scale->value;

	ct_new_horizontal_value (x, False);
	CT_Color->lastx = x;
}

/* ARGSUSED */
void
#ifdef _NO_PROTO
ct_change_horiz_text(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_change_horiz_text(Widget w, XtPointer client, XtPointer call)
#endif
{

	char	*textstring;
	int	result, x;
    float fx;

	textstring = XmTextFieldGetString (w);
#ifdef SATURATION

    if (CT_Color == &satrgb)
    {
	result = sscanf (textstring, "%f", &fx);
	x = (int) (fx * 255.0 + .5);
    }
    else
#endif /* SATURATION */

	result = sscanf (textstring, "%d", &x);

	if (result == 1 && (x >= 0 && x < 256)) 
	{
		XmScaleSetValue (ct_scale_horiz, x);
		ct_new_horizontal_value (x, True);
		CT_Color->lastx = x;
		
	}
}


void
#ifdef _NO_PROTO
ct_popup_stretch (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_popup_stretch (Widget w, XtPointer client, XtPointer call)
#endif
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

    if (XtIsManaged (ct_stretch_mainWindow))
    {
	
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Stretch", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtUnmanageChild (ct_stretch_mainWindow);
	XtPopdown (xidStretch);
#define CMAP
    }
    else
    {

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Hide Stretch", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtManageChild (ct_stretch_mainWindow);
	XtPopup (xidStretch, XtGrabNone);

	if (NumStretch == 0)
	    ct_init_stretch_list ();

    }
}

void
#ifdef _NO_PROTO
ct_dismiss_stretch  (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_dismiss_stretch  (Widget w, XtPointer client, XtPointer call)
#endif
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

    if (XtIsManaged(ct_show_stretch_button))
    {
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Stretch", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (ct_show_stretch_button, args, argcnt);
	XmStringFree (xmstr[0]);
    }

    XtUnmanageChild (ct_stretch_mainWindow);
    XtPopdown (xidStretch);
}

void
#ifdef _NO_PROTO
ct_build_map (visualinfo, colormap)
XVisualInfo *visualinfo;
Colormap colormap;
#else
ct_build_map (XVisualInfo *visualinfo, Colormap colormap)
#endif
{
    Arg		args[10];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

    XtInitializeWidgetClass(topLevelShellWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "X Colormap Trak"); argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "X Colormap Trak"); argcnt++;
    XtSetArg(args[argcnt], XmNx, 600); argcnt++;
    XtSetArg(args[argcnt], XmNy, 582); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,visualinfo); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,colormap); argcnt++;
    XtSetArg(args[argcnt],XtNdepth,visualinfo->depth); argcnt++;
    xidCMap = XtCreatePopupShell( "xidCMap", 
        topLevelShellWidgetClass, AppShell, args, argcnt);

    XCmapVisual = visualinfo->visual;
    XCmap = colormap;
    XCmapDisplay = xid.display;
    xcmap_setup_window();
    ct_cmap_mainwindow = Createxcmap_bulletinboard(xidCMap);
}

void
#ifdef _NO_PROTO
ct_popup_map (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_popup_map (Widget w, XtPointer client, XtPointer call)
#endif
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];



    if (xidCMap == NULL)
	if ((xid.visual_info[0].visual->class == PseudoColor)
	|| (!CT_Img_Visual))
	    ct_build_map (xid.visual_info, *(xid.colormap_ptr));
        else
	if (CT_Img_Visual && CT_Img_Visual->visual->class == PseudoColor)
	    ct_build_map (CT_Img_Visual, CT_Img_CMap);

    if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
    {
	
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Map", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtUnmanageChild (ct_cmap_mainwindow);
	XtPopdown (xidCMap);
    }
    else
    {

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Hide Map", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtManageChild (ct_cmap_mainwindow);
	XtPopup(xidCMap,XtGrabNone);

    }
}

#define UNDO	1
#define REDO	2
int Undo_Redo	 = UNDO;

void
#ifdef _NO_PROTO
ct_undo_redo (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_undo_redo (Widget w, XtPointer client, XtPointer call)
#endif
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];
    

    if (Undo_Redo == UNDO)
    {
	CurrStretch = PREVIOUS;
	ct_set_stretch ();

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Redo", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	Undo_Redo = REDO;
	if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
	{
	   xcmap_draw_hls_colormap();
	}

    }
    else
    {
	CurrStretch = PREVIOUS;
	ct_set_stretch ();

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Undo", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	Undo_Redo = UNDO;
	if (ct_cmap_mainwindow && XtIsManaged (ct_cmap_mainwindow))
	{
	   xcmap_draw_hls_colormap();
	}

    }

}

void 
#ifdef _NO_PROTO
ct_histo_up (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_histo_up (Widget w, XtPointer client, XtPointer call)
#endif
{
    char	value_string [5];

    Histo_scale += .1;
    if (Histo_scale >= .91)
	XtSetSensitive (w, False);
    else
	XtSetSensitive (ct_histo_down_button, True);

    sprintf (value_string, "%3.2f", Histo_scale);
    XmTextFieldSetString (ct_histo_scale_text, value_string);
    ct_histo_draw (CT_Color);
}

void 
#ifdef _NO_PROTO
ct_histo_down (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_histo_down (Widget w, XtPointer client, XtPointer call)
#endif
{
    char	value_string [10];

    Histo_scale -= .1;
    if (Histo_scale <= .1)
	XtSetSensitive (w, False);
    else
	XtSetSensitive (ct_histo_up_button, True);

    sprintf (value_string, "%3.2f", Histo_scale);
    XmTextFieldSetString (ct_histo_scale_text, value_string);
    ct_histo_draw (CT_Color);
}
void 
#ifdef _NO_PROTO
ct_change_scale_text (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_change_scale_text (Widget w, XtPointer client, XtPointer call)
#endif
{

    char	*textstring;
    int	result;
    float	scalefactor;

    textstring = XmTextFieldGetString (w);

    result = sscanf (textstring, "%f", &scalefactor);

    if (result == 1 && (scalefactor > 0.0 && scalefactor <= 1.0)) 
    {
	Histo_scale = scalefactor;
	ct_histo_draw (CT_Color);
	    
	if (Histo_scale >= .91)
	    XtSetSensitive (ct_histo_up_button, False);
	else
	    XtSetSensitive (ct_histo_down_button, True);
	if (Histo_scale <= .1)
	    XtSetSensitive (ct_histo_down_button, False);
	else
	    XtSetSensitive (ct_histo_up_button, True);
    }
    else
    {
	char	value_string [10];

	sprintf (value_string, "%3.1f", Histo_scale);
	XmTextFieldSetString (ct_histo_scale_text, value_string);
    }
}



void
ct_build_histnorm ()
{
    Arg		args[10];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];
    extern Widget Createhist_main_bb();

    XtInitializeWidgetClass(topLevelShellWidgetClass);
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "Histogram Normalization Controls"); argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "Histogram Normalization Controls"); argcnt++;
    XtSetArg(args[argcnt], XmNx, 765); argcnt++;
    XtSetArg(args[argcnt], XmNy, 560); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,xid.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,xid.colormap); argcnt++;
    XtSetArg(args[argcnt],XtNdepth,xid.visual_info[0].depth); argcnt++;
    hist_norm_shell = XtCreatePopupShell( "hist_norm_shell", 
        topLevelShellWidgetClass, AppShell, args, argcnt);

    hist_main_bb = Createhist_main_bb(hist_norm_shell);
}



void
#ifdef _NO_PROTO
ct_popup_histnorm (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_popup_histnorm (Widget w, XtPointer client, XtPointer call)
#endif
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];


    if (! hist_norm_shell)
	ct_build_histnorm ();

    if (hist_main_bb && XtIsManaged (hist_main_bb))
    {
	
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Normalize ", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtUnmanageChild (hist_main_bb);
	XtPopdown (hist_norm_shell);
    }
    else
    {

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Hide Norm", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtManageChild (hist_main_bb);
	XtPopup(hist_norm_shell,XtGrabNone);

    }
}
