/****************************************************************************/
/*   PROGRAM NAME  : ct_graphicadjust.c                                     */
/*   AUTHOR        : g e nostrand                                           */
/*   DATE          : july 14, 1993                                          */
/*   LOCATION      : DSRC PRINCETON, NJ                                     */
/*                                                                          */
/*   PROGRAM DESCRIPTION:                                                   */
/*   PROGRAM MODIFICATIONS:                                                 */
/****************************************************************************/

#include "xid.h"
#include "ct_interface.h"

/*****************************************************************************/
/*
 
	 New colortool widgets generated from Builder Xcessory file tct.uil
	 in directory ../xid_colortool.tomk

	 Tom Klinedinst 11/19/93

*/
/*****************************************************************************/

extern XID_COLORTOOLPTR         CT_Colors[];
extern XID_COLORTOOLPTR CT_Color;

void ct_framedrawingarea(Window window, GC gc, int x, int y, 
                         unsigned int width, unsigned int height, 
                         unsigned long color)
{
#ifdef FRAMEDRAWING
   XSetLineAttributes(xid.display,gc,4,LineSolid,CapButt,JoinMiter);
   XSetForeground(xid.display,gc,color);
   XDrawRectangle(xid.display,window,gc,x,y,width,height);
#endif
}

void ct_highlightwindow(Widget w, XtPointer data, XEvent *report)
{
   XID_COLORTOOLPTR     adjust= *((XID_COLORTOOLPTR *) data);

   ct_framedrawingarea(	adjust->window,
                       	adjust->gc,
                       	adjust->x,
		       	adjust->y,
		  	adjust->width,
		      	adjust->height,
			adjust->linecolor);
}

void ct_unhighlightwindow(Widget w, XtPointer data, XEvent *report)
{
   XID_COLORTOOLPTR     adjust= *((XID_COLORTOOLPTR *) data);

   ct_framedrawingarea(	adjust->window,
                       	adjust->gc,
                       	adjust->x,
		       	adjust->y,
		  	adjust->width,
		      	adjust->height,
			adjust->background);
}

void ct_drawline(Window window,  GC gc, long *from, long *to, long index, 
                 unsigned long pixel, int size)
{
   int i;

   XSetLineAttributes(xid.display,gc,2,LineSolid,CapButt,JoinMiter);
   XSetForeground(xid.display,gc,pixel);
   for (i=0;i<index;i++) {
       XFillRectangle(xid.display,window,gc,from[i]-size/2,255-to[i]-size/2,size,size);
       XDrawLine(xid.display,window,gc,from[i],255-to[i],from[i+1],255-to[i+1]); 
   }
   XFillRectangle(xid.display,window,gc,from[index]-size/2,255-to[index]-size/2,size,size);
   XFlush(xid.display);
}
void
DBGPRINT(char *msg, int from, int to, int i, int x, int y)
{
#ifdef DEBUGPRINT
	printf("%s from[%d]=%d, to[%d]=%d, x=%d, y=%d \n", 
		msg, i, from, i, to, x, y);
#else
#endif
}

void ct_setbreakpoint(long *from, long *to, long *index, int x, int y, int margin) 
{
  long 	sorted=0;
  long  i, fromtmp, totmp;

  if  ( x < 0 || x > 255 ) return;
  if  ( y < 0 || y > 255 ) return;

  for (i=0;i<*index;i++) {
      /* already break point         */
      if      ((x == from[i]) && (y == 255-to[i])) return;    
      else if ((x == from[i]) && (y != 255-to[i])) {          
	 /* change existing break point */
         to[i]=255-y;
	 DBGPRINT("set:change", from[i], to[i], i, x, y);
         return;   
      }
      else if ((x > from[i])   && (x < from[i+1])) {	      
	  /* first check if close enough to current breakpt 	*/
	  DBGPRINT("set:out", from[i], to[i], i, x, y);
	  if (((x >= from[i]-margin/2)   && (x <= from[i]+margin/2)) &&
	      ((y >= 255-to[i]-margin/2) && (y <= 255-to[i]+margin/2))) {
		DBGPRINT("set:close", from[i], to[i], i, x, y);
		if (i > 0 && i < *index)
		    from[i] = x;
		to[i] = 255 - y;
		return;
	  }
	  else
	  /* Check if close to next breakpoint	*/
	  if (((x >= from[i+1]-margin/2)   && (x <= from[i+1]+margin/2)) &&
	      ((y >= 255-to[i+1]-margin/2) && (y <= 255-to[i+1]+margin/2))) {
		DBGPRINT("set:close", from[i+1], to[i+1], i+1, x, y);
		if ((i+1) < *index)
		    from[i+1] = x;
		to[i+1] = 255 - y;
		return;
	  }
	  else
	  /* between breakpts, check if not inside box of adj breakpts */
          if ((y > 255-to[i]) || (y < 255-to[i+1])) return;   
	}
  }

  /*  Special case for x == 255	, last array element */
  if (i == *index && x == from[i]) {
      to[i] = 255 - y;
      DBGPRINT("set:last", from[i], to[i], i, x, y);
      return;
  }

  *index += 1;
  from[*index]=x;
  to[*index]=255-y;
  DBGPRINT("set:new", from[*index], to[*index], *index, x, y);
  while(!sorted) {
      sorted=1;
      for (i=0;i<*index;i++) {
         if (from[i+1] < from[i]) {
               fromtmp=  from[i+1];
               from[i+1]=from[i];
               from[i]=  fromtmp;
               totmp=    to[i+1];
               to[i+1]=  to[i];
               to[i]=    totmp;
               sorted=0;
         }
      }
  }
  return;
}

void ct_deletebreakpoint(long *from, long *to, long *index, int x, int y, int margin)
{
  int i,found=0;


  if (((x >= from[0]-margin/2)   && (x <= from[0]+margin/2)) &&
      ((y >= 255-to[0]-margin/2) && (y <= 255-to[0]+margin/2))) {
      from[0]=0;
      to[0]=0;
      return;
  }
  if (((x >= from[*index]-margin/2)   && (x <= from[*index]+margin/2)) &&
      ((y >= 255-to[*index]-margin/2) && (y <= 255-to[*index]+margin/2))) {
      from[*index]= 255;
      to[*index]=   255;
      return;
  }
  
  for (i=1;i<*index;i++) {			
      if (((x >= from[i]-margin/2)   && (x <= from[i]+margin/2)) &&
          ((y >= 255-to[i]-margin/2) && (y <= 255-to[i]+margin/2))) {
            found=1;  
      }
      if (found) {
         from[i]=from[i+1];
         to[i]=  to[i+1];
	DBGPRINT("del:found", from[i], to[i], i, x, y);
      }
  }
  if (found) *index -= 1;
}
#define NEWMOVEBREAK
#ifdef NEWMOVEBREAK
int  ct_findbreakpoint(long *from, long *pto, long index, int x, int y)
{
    int i;

    for (i=0;i<=index ;i++) {
	if  ((x == from[i]) && (y == pto[i])) break;
    }
    return (i);
}

void ct_movebreakpoint(long *from, long *pto, long *index, int x, int y, int *nextx, int *nexty)
{
    int	i;
    int found = False;
    int	newx, newy;

    newx = *nextx;
    newy = *nexty;
    /*  Constrain values to 0-255 for motion events which extend 
	beyond drawing area widget					*/
    if (newx < 0) newx = 0;
    if (newx > 255) newx = 255;

    if (newy < 0) newy = 0;
    if (newy > 255) newy = 255;

    /* Adjust for difference between X location and to value		*/
    y = 255 - y;
    newy = 255 -newy;

    /*  Find old breakpoint and make sure breakpoint is constrained 
       to a monitonically increasing contrast stretch			*/

    for (i=0;i<=*index ;i++) {
	if  ((x == from[i]) && (y == pto[i])) break;
	DBGPRINT("mov:search", from[i], pto[i], i, x, y);
    }
    if (i > *index) {
	*nextx = newx;
	*nexty = (255 - newy);
	return;
    }
    DBGPRINT("mov:found", from[i], pto[i], i, x, y);

    if (i > 0 && i < (*index))
    {

	if (newx <= from[i-1]) newx = from[i-1]+1;
	if (newx >= from[i+1]) newx = from[i+1]-1;

	if (newy <= pto[i-1]) newy = pto[i-1];
	if (newy >= pto[i+1]) newy = pto[i+1];
    }
    else
    if (i == 0)
    {

	newx = 0;
	if (newy >= pto[i+1]) newy = pto[i+1];
    }
    else
    if (i == (*index))
    {

	newx = 255;
	if (newy <= pto[i-1]) newy = pto[i-1];
    }

    from[i] = newx;
    pto[i] = newy;
    *nextx = newx;
    *nexty = (255 - newy);
    DBGPRINT("mov:set", from[i], pto[i], i, newx, newy);
}
#endif

void ct_breakpointlocation(Window window,  GC gc, int x, int y, unsigned long foreground)
{
   int 		xoffs, yoffs;
   char 	str[20];

   if ( x < 0 || x > 255 || y < 0 || y > 255 ) return;
   sprintf(str,"(%3d,%3d)",x,255-y);
   if ( x > 165 ) xoffs= -90;
   else           xoffs=  10;
   if ( y > 236 ) yoffs=   0;
   else           yoffs=  20;

   XSetForeground(xid.display,gc,foreground);
   XDrawString(xid.display,window,gc,x+xoffs,y+yoffs,str,strlen(str));
   XFlush(xid.display);
}

void ct_apply(XID_COLORTOOLPTR color)
{

   float   scaler; 	/* Used to scale gray level values to the
			   appropriate range (i.e. NUM_IMGCOLORS).    */

    long color_idx,     /* Index into color table.                    */
	 i,		/* Looping index.			      */
	 idx,		/* Index into piece-wise linear table.	      */
	 img_idx,	/* Index into colormap of the first image color.*/
	 init = FALSE,  /* Indicates whether or not the pwl color
			   table needs to be initialized.             */
	 map[256],
	 ncolors,       /* Number of colors to store in colormap.     */
	 nplanes,       /* Number of planes displayed.                */
	 offset,        /* Offset used in scaling 24-bit values.      */
	 start;         /* Starting index in the color table array.   */

   /*  Kludge to automatically update histogram - Tom Klinedinst	*/

   void 	ct_histo_draw();

   if (color == CT_Color)
       ct_histo_draw (color);


   /* 
    Transfer variables
   */
   /*  piece function expect the number of adjust entries not the last index */
   xid_pwl.num=		color->adjust.index + 1;
   xid_pwl.from=	color->adjust.from;
   xid_pwl.to=		color->adjust.to;
   xid_map.flags=	color->adjust.flags;

   /*
    Copy to red, green and blue if color is MAP_ALL
   */
   if (xid_map.flags & MAP_ALL)
   {
       for (i=0; i < 3; i++)
       {
	    int	j;
	    CT_Colors[i]->adjust.index = color->adjust.index;
	    for (j=0; j <= color->adjust.index; j++)
	    {
	       CT_Colors[i]->adjust.from[j] = color->adjust.from[j];
	       CT_Colors[i]->adjust.to[j] = color->adjust.to[j];
	    }
       }
   }
   /*
    Calculate the mapping.
   */

   piece(xid_pwl.from,xid_pwl.to,&xid_pwl.num,map);

   /*
    Based on the visual type, determine which portion of the colormap needs
    to be modified and stored.
   */

   if ((xid.visual_info[0].visual->class == PseudoColor) && 
        (*xid.colormap_ptr == xid.colormap)) {
        color_idx = img_idx = IMG_IDX;
        start =   0;
        ncolors = NUM_IMGCOLORS;
   }
   else if ((xid.visual_info[0].visual->class == DirectColor) &&
             (*xid.colormap_ptr == xid.colormap)) {
         color_idx = 0;
         start = img_idx = IMG_IDX;
         ncolors = 256;
   }
   else {
         color_idx = start = img_idx = 0;
         ncolors = 256;
   }

   /*
    Allocate buffer space for the piece-wise linear mapping.
   */

   if (xid_pwl.color_pwl == NULL) {
       xid_pwl.color_pwl = (XColor *) calloc(ncolors,sizeof(XColor));
       if (xid_pwl.color_pwl == NULL) {
	  xid_message("Insufficient memory available");
	  return;
       }
       init = TRUE;
   }

   /*
    Calculate the new color table.
   */

   if ((*xid.colormap_ptr == xid.colormap) && 
       (xid.visual_info[0].visual->class == PseudoColor) && (init)) {
   /*
    Copy the current image colors.
   */
      for (i = 0; i < NUM_IMGCOLORS; i++) xid_pwl.color_pwl[i] = xid.color_table[color_idx + i];
   }
   else if (init) {
   /*
    Copy the reserved colors as well as the current image colors.
   */
       for (i = 0; i < 256; i++) xid_pwl.color_pwl[i] = xid.colortable_ptr[i];
   }

   /*
    Determine how many planes are currently being displayed.
   */

   if ((xid_view.rgbplanes[RED]    ==
        xid_view.rgbplanes[GREEN]) &&
       (xid_view.rgbplanes[GREEN] ==
        xid_view.rgbplanes[BLUE]))
        nplanes = 1;
   else nplanes = 3;

   /*
    Calculate the scaler needed to reduce the image to the appropriate number of colors.
   */

   if ((*xid.colormap_ptr == xid.colormap) && 
       (nplanes == 1)                      && 
       (xid_planes[xid_view.rgbplanes[RED]-1].maxval < NUM_IMGCOLORS))
       scaler = 1.0;
   else if (*xid.colormap_ptr == xid.colormap)
       scaler = (256.0 / (float)NUM_IMGCOLORS);           /* (2^8 / # of colors) */
   else
       scaler = 1.0;

   /*
    Apply the mapping to the color table.
   */
   if (xid.visual_info[0].visual->class == DirectColor) {
      if (*xid.colormap_ptr == xid.colormap)
         offset = (0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE + NUM_WMCOLORS + 1)));
      else
         offset = 0;

      for (i = img_idx; i < 256; i++) {
          idx = (long)(i * scaler + offset); 
	  if ((xid_map.flags & MAP_RED) || (xid_map.flags & MAP_ALL))
	       xid_pwl.color_pwl[start + (i - img_idx)].red = map[idx] << 8; 
	  if ((xid_map.flags & MAP_GREEN) || (xid_map.flags & MAP_ALL))
	       xid_pwl.color_pwl[start + (i - img_idx)].green = map[idx] << 8; 
	  if ((xid_map.flags & MAP_BLUE) || (xid_map.flags & MAP_ALL))
	       xid_pwl.color_pwl[start + (i - img_idx)].blue = map[idx] << 8; 
	   xid_pwl.color_pwl[start + (i - img_idx)].flags = DoRed | DoGreen | DoBlue;
      }
   }
   else {
       for (i=img_idx; i < 256; i++) {
	if ((xid_map.flags & MAP_RED) || (xid_map.flags & MAP_ALL))
	    xid_pwl.color_pwl[i - img_idx].red = map[xid.color_linear[i].red >> 8] << 8;
	if ((xid_map.flags & MAP_GREEN) || (xid_map.flags & MAP_ALL))
	    xid_pwl.color_pwl[i - img_idx].green = map[xid.color_linear[i].green >> 8] << 8;
	if ((xid_map.flags & MAP_BLUE) || (xid_map.flags & MAP_ALL))
	    xid_pwl.color_pwl[i - img_idx].blue = map[xid.color_linear[i].blue >> 8] << 8;
	xid_pwl.color_pwl[i - img_idx].flags = DoRed | DoGreen | DoBlue;
	}
    }
#ifdef DEBUGCOLOR
    for (i=img_idx; i < 256; i++) {
	printf (" %d %d %d %d \n", (start+(i-img_idx)),
	    xid_pwl.color_pwl[start + (i - img_idx)].red >> 8,
	    xid_pwl.color_pwl[start + (i - img_idx)].green >> 8,
	    xid_pwl.color_pwl[start + (i - img_idx)].blue >> 8);
	
    }
#endif
   /*
    Store the colormap.
   */
   XStoreColors(xid.display,*xid.colormap_ptr,xid_pwl.color_pwl,ncolors);

   if (XtIsManaged (ct_cmap_mainwindow))
   {
       extern xcmap_show_curr_pixel_color();

       xcmap_show_curr_pixel_color();
   }

}

void ct_drawingevent(Widget w, XtPointer data, XEvent *report)
{
   XID_COLORTOOLPTR   	color= 	*( (XID_COLORTOOLPTR *) data);
   static int      	x,y,size=SIZEBREAKPTBOX;

#define INACTIVE	0
#define MOTION		1
#define SETBREAK	2
#define DELBREAK	3
   static int		MouseMode = INACTIVE; 

   void ct_set_sliders_text ();
#define CURRSTRETCH
#ifdef CURRSTRETCH
   extern void ct_new_curr_stretch ();
#endif

   if ( ! color->window ) 
   {/* only works after window has been mapped */
      extern XID_COLORTOOL red, green, blue, allrgb;

      /*  Kludge to set window since all colors are mapped to same window */
      red.window=XtWindow(w); 	
      green.window = XtWindow(w);
      blue.window = XtWindow(w);
      allrgb.window = XtWindow(w);
   }

   switch(report->type) {
       case  Expose: 	   ct_drawline( color->window,
					color->gc,
			 		color->adjust.from,
			  		color->adjust.to,
					color->adjust.index,
					color->linecolor, size);
			   MouseMode = INACTIVE;
                           break;
       case  MotionNotify: ct_drawline( color->window,
					color->gc,
			 		color->adjust.from,
			  		color->adjust.to,
					color->adjust.index,
					color->background, size+size);
                           ct_breakpointlocation(color->window,color->gc,x,y,color->background);
#ifdef NEWMOVEBREAK
                	   ct_movebreakpoint(color->adjust.from,
				color->adjust.to,&color->adjust.index,
				x,y,&report->xbutton.x, &report->xbutton.y);  
			   x=report->xbutton.x;
			   y=report->xbutton.y;
#else
                	   ct_deletebreakpoint(color->adjust.from,color->adjust.to,&color->adjust.index,x,y,size);  
   		           x=report->xbutton.x;
                           y=report->xbutton.y;
                	   ct_setbreakpoint(color->adjust.from,color->adjust.to,&color->adjust.index,x,y,size);  
#endif
                     	   ct_drawline( color->window,
					color->gc,
			 		color->adjust.from,
			  		color->adjust.to,
					color->adjust.index,
					color->linecolor, size);
                           ct_breakpointlocation(color->window,color->gc,x,y,color->foreground);
                           ct_framedrawingarea(	color->window,
						color->gc,
			 			color->x,
			  			color->y,
						color->width,
						color->height,
						color->linecolor);
                           ct_apply(color);
			   /*  Too slow to update during pointer motion
			   ct_set_sliders_text (x, y);
			   ****************************/
			   MouseMode = MOTION;
                           break;
       case   ButtonPress: switch(report->xbutton.button) { 
                	       case Button1: 
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->background, size+size);
                           		x=report->xbutton.x;
                           		y=report->xbutton.y;
                	   		ct_setbreakpoint(color->adjust.from,
							 color->adjust.to,
							 &color->adjust.index,x,y,size);  
                           		ct_breakpointlocation(color->window,color->gc,x,y,color->foreground);
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->linecolor, size);
				       ct_set_sliders_text (x, y);
				       MouseMode = SETBREAK;
				       break;
                	       case Button2: 
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->background, size+size);
                           		x=report->xbutton.x;
                           		y=report->xbutton.y;
                	   		ct_deletebreakpoint(color->adjust.from,
							    color->adjust.to,
							    &color->adjust.index,x,y,size);  
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->linecolor, size);
				       ct_set_sliders_text (x, y);
				       MouseMode = DELBREAK;
				       break;
                               case Button3:       
                               default:      
				   MouseMode = INACTIVE;
				   break;
                }
                break;
       case ButtonRelease: switch(report->xbutton.button) { 
                               case Button1: 
#ifdef CURRSTRETCH
					if (MouseMode == MOTION 
					|| MouseMode == SETBREAK)
					{
					   ct_new_curr_stretch();
					   ct_set_sliders_text (x, y);
					   if (XtIsManaged (ct_cmap_mainwindow))
					   {
					       extern xcmap_draw_colors();

					       xcmap_draw_colors();

					   }
					}
#endif

                           		ct_breakpointlocation(color->window,color->gc,x,y,color->background);
					ct_drawline(	color->window,
							color->gc,
			 				color->adjust.from,
			  				color->adjust.to,
							color->adjust.index,
							color->linecolor, size);
                               case Button2: 
                                             ct_apply(color);
#ifdef CURRSTRETCH
					if (MouseMode == DELBREAK)
					{
					   ct_new_curr_stretch();
					   if (XtIsManaged (ct_cmap_mainwindow))
					   {
					       extern xcmap_draw_hls_colormap();
					       xcmap_draw_hls_colormap();
					   }
					}
#endif
   		                             break;
                               default:      break;
                }
		MouseMode = INACTIVE;
                break;
       default: break;
   }


}

/******************************************************************************/
/*
	
	New functions added for new colortool layout with singlar
	stretch window.

	Tom Klinedinst
*/
/******************************************************************************/



#define HIST_DIST	1
#define HIST_CUM	2
static int	CT_Hist_Type = HIST_DIST;
/* ARGSUSED */
void
ct_hist_type(Widget w, char *hist_type, XtPointer call)
{

    if (*hist_type == 'D')
	CT_Hist_Type = HIST_DIST;
    else
	CT_Hist_Type = HIST_CUM;

    ct_histo_draw (CT_Color);



}
/******************************************************************************/
/*
	ct_calc_adjusted_hist calculates a new histogram based on the 
	original histogram and the new mapping array.  
*/
/******************************************************************************/
void ct_calc_adjusted_hist (long *histptr, long *map, long *adjusted_hist)
{
    long freq;

    for (freq=0; freq < 256; freq++)
	adjusted_hist[freq] = 0;

    for (freq=0; freq < 256; freq++)
	adjusted_hist[map[freq]] += *(histptr+freq);
}
/******************************************************************************/
/*
	ct_calc_cumm_hist calculates a cummulative histogram and maximum
	number within the histogram.
*/
/******************************************************************************/
void ct_calc_cumm_hist (long *histptr, long *cum_histo, long *maxp)
{
/*
    Calculate the cumulative histogram.  This is done regardless of the option
    chosen because the cumulative histogram is used in percentage calculations.
*/
    long freq;

    for (freq=0; freq < 256; freq++)
	if (*(histptr + freq) > 0)
	{
	    cum_histo[freq] = *(histptr + freq);
	    break;
	}
	else
	    cum_histo[freq] = 0;
    for (freq=freq+1; freq < 256; freq++)
	if (*(histptr + freq) >= 0)
	    cum_histo[freq] = *(histptr + freq) +
					    cum_histo[freq-1];
    else
	cum_histo[freq] = cum_histo[freq-1];
    /*
	Find maximum value.
    */
    *maxp = 0;
    for (freq = 0; freq <= 255; histptr++, freq++)
	if (*maxp < *histptr) *maxp = (float) *histptr;

#ifdef CAPMAX
    /*  If maximum is significant percentage of total reduce to 5 % */
    if ((*maxp / (float) cum_histo[255]) > .05)
	*maxp = cum_histo[255] * .05;
#endif
}

/******************************************************************************/
/*
	ct_draw_tickmarks draws the horizontal and vertical tick marks for
	the histogram axes.  The tickmarks are drawn in separate 
	horizontal and vertical drawing areas adjacent to the histogram.  
*/
/******************************************************************************/
#define TICKSIZE	5
ct_draw_tickmarks (long maxp, XID_COLORTOOLPTR color)
{
    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    long	x1, x2, y1, y2;
    long	freq;

    Dimension	hist_horiz_lab_height;	
    Dimension	hist_horiz_lab_width;
    Dimension	hist_vert_lab_height;	
    Dimension	hist_vert_lab_width;

    long    c_width,        /* Calculated font character width.             */
	    c_height;       /* Calculated font character height.            */
    /*
    Calculate the width and height of characters in the working font.
    */
    c_width = xid.font_info->max_bounds.rbearing -
	      xid.font_info->min_bounds.lbearing;

    c_height = xid.font_info->max_bounds.ascent +
	      xid.font_info->max_bounds.descent;

    /*  Draw tick marks on the horizontal axis	*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNheight, &hist_horiz_lab_height); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, &hist_horiz_lab_width); argcnt++;
    XtGetValues(ct_histo_horiz_label, args, argcnt);

    XSetForeground(xid.display,color->gc,color->background);
    XDrawLine (xid.display, XtWindow(ct_histo_horiz_label),
				color->gc, 0,0, hist_horiz_lab_width, 0);
    for (freq = 255; freq > 0; freq=freq-64)
    {
	char	xlabel[10];

	y2 = TICKSIZE;
	x1 = x2 = freq;
	y1 = 0;
	XDrawLine (xid.display, XtWindow(ct_histo_horiz_label),
				color->gc, x1,y1, x2, y2);
	sprintf (xlabel, "%d", freq);
	XDrawString(xid.display,XtWindow(ct_histo_horiz_label),
			color->gc,x1-((strlen(xlabel))*c_width),
			(hist_horiz_lab_height),
			xlabel,strlen(xlabel));

    }
    /*  Draw tick marks on the vertical axis	*/
    XClearWindow (xid.display, XtWindow(ct_histo_vert_label));
    argcnt = 0;
    XtSetArg(args[argcnt], XmNheight, &hist_vert_lab_height); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, &hist_vert_lab_width); argcnt++;
    XtGetValues(ct_histo_vert_label, args, argcnt);
    XDrawLine (xid.display, XtWindow(ct_histo_vert_label),
				color->gc, 0,0,0, hist_vert_lab_height);
    for (y2=0; y2 < hist_vert_lab_height; y2=y2+(hist_vert_lab_height/4))
    {
	char	ylabel[10];

	x1 = 0;
	x2 = TICKSIZE;
	y1 = y2;
	XDrawLine (xid.display, XtWindow(ct_histo_vert_label),
				color->gc, x1,y1, x2, y2);

	sprintf (ylabel, "%d", (hist_vert_lab_height - y2) 
		* (maxp/hist_vert_lab_height));
	XDrawString(xid.display,XtWindow(ct_histo_vert_label),
			color->gc,x2,
			(y2+c_height),
			ylabel,strlen(ylabel));

    }
}


/******************************************************************************/
/*

	ct_histo_stat_pct calculates the percent statistics for the histogram
	and displays the statistics on the histogram info labels.  

*/
/******************************************************************************/

static	Window	Histo_win;
static	Widget	Histo_widget;
static    long	Max;
static	  float ScaleFactor = 1.0;
static    Dimension	hist_height;	
static    Dimension	hist_width;

void
ct_histo_stat_pct ( long *histptr, long *cum_histo, long horiz_value)
{
    float	percent;
    XmString    xmstr[256];
    char	charlabel [256];
    static	int	histlastx=0, histlasty=0;
    int		x1, y1, x2, y2;

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    if (xid.view == -1)
	return;

    percent = (cum_histo[horiz_value] / (float) cum_histo[255]) * 100;
    sprintf (charlabel, "%3.2f%% of Data\n   at Pixel %d",
	     percent, horiz_value);
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_pct, args, argcnt);

#ifdef HISTOINDICATOR
    /*  Draw indicator lines to make it easier to read histogram	*/
    XSetForeground (xid.display, CT_Color->gc, 
	CT_Color->foreground);
    XSetFunction (xid.display, CT_Color->gc, GXxor);
    XDrawLine (xid.display, Histo_win, CT_Color->gc, 
	histlastx,hist_height, histlastx, histlasty);
    XDrawLine (xid.display, Histo_win, CT_Color->gc, 
	histlastx,histlasty, 255, histlasty);

    x1 = x2 = horiz_value;
    y2 = hist_height - ((histptr[x1] / (float) Max) * hist_height);
    y1 = hist_height;

    XDrawLine (xid.display, Histo_win, CT_Color->gc, 
	x1, y1, x2, y2);
    XDrawLine (xid.display, Histo_win, CT_Color->gc, 
	x1, y2, 255, y2);
    XSetFunction (xid.display, CT_Color->gc, GXcopy);
    
    histlastx = horiz_value;
    histlasty = y2;
#endif

}
/******************************************************************************/
/*

	ct_histo_statistics calculates the statistics for the histogram
	and displays the statistics on the histogram info labels.  

*/
/******************************************************************************/
void
ct_histo_statistics ( long *histptr, long *cum_histo)
{

    double	mean, stdev;
    float	percent;
    long	ct_min, ct_max;

    XmString    xmstr[256];
    char	charlabel [256];

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    long	horiz_value = CT_Color->lastx;

    xid_histcalc (histptr, &ct_min, &ct_max, &mean, &stdev);

    sprintf (charlabel, "Mean    = %f", mean);
    charlabel [strlen(charlabel)-4] = '\0';
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_mean, args, argcnt);
    XmStringFree (xmstr[0]);

    sprintf (charlabel, "St Dev  = %f", stdev);
    charlabel [strlen(charlabel)-4] = '\0';
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_stdev, args, argcnt);
    XmStringFree (xmstr[0]);


    sprintf (charlabel, "Min Pix = %d", ct_min);
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_min, args, argcnt);
    XmStringFree (xmstr[0]);

    sprintf (charlabel, "Max Pix = %d", ct_max);
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_max, args, argcnt);
    XmStringFree (xmstr[0]);

    ct_histo_stat_pct (histptr, cum_histo, horiz_value);
}
/******************************************************************************/
/*

	ct_clear_statistics clears the histogram statistics labels.

*/
/******************************************************************************/
void
ct_clear_statistics ( XID_COLORTOOLPTR	color)
{

    XmString    xmstr[256];
    char	charlabel [256];

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    charlabel [0] = '\0';
    argcnt=0;
    XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	charlabel, XmSTRING_DEFAULT_CHARSET))); argcnt++;
    XtSetValues (ct_hist_info_mean, args, argcnt);
    XtSetValues (ct_hist_info_stdev, args, argcnt);
    XtSetValues (ct_hist_info_min, args, argcnt);
    XtSetValues (ct_hist_info_max, args, argcnt);
    XtSetValues (ct_hist_info_pct, args, argcnt);
    XmStringFree (xmstr[0]);
}
/******************************************************************************/
/*

	ct_histo_draw draws the histogram for the adjusted color.  If no
	image is displayed, a message is displayed in the histogram window.

*/
/******************************************************************************/
void ct_histo_draw (XID_COLORTOOLPTR color)
{

    static char message[] = "No image data for histogram";
    long	bnd;
    long	x1, x2, y1, y2;
    long	freq;

    long	*histptr;

    long	*adjusted_hist=color->adjusted_hist;
    int map [256];
    int count;
    long	*cum_histo=color->cum_histo;

    long	all_hist[256];

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    if (xid.view == -1)
    {

	/*  Display message specifying no image displayed	*/

       XSetForeground(xid.display,color->gc,color->background);
       XDrawString(xid.display,Histo_win,color->gc,10,10,
		   message,strlen(message));
       ct_clear_statistics (color);
       XFlush(xid.display);
       return;
    }

    
    /* Find the color band associated with the color adjustment	*/
    /* The color band is used to index the array of planes to 	*/
    /* Find the histogram data which is generated when the image*/
    /* is initially loaded.					*/

#ifdef OLDXID
    if (color->adjust.flags & MAP_RED) 
	bnd =xid_composite[xid.view].rgbplanes[0];
    if (color->adjust.flags & MAP_GREEN) 
	bnd =xid_composite[xid.view].rgbplanes[1];
    if (color->adjust.flags & MAP_BLUE) 
	bnd =xid_composite[xid.view].rgbplanes[2];

    /*  Kludge - need special handling for ALL	*/
    if (color->adjust.flags & MAP_ALL) 
	bnd =xid_composite[xid.view].rgbplanes[0];
#else
    if (color->adjust.flags & MAP_RED) 
	bnd =xid_view.rgbplanes[0];
    if (color->adjust.flags & MAP_GREEN) 
	bnd =xid_view.rgbplanes[1];
    if (color->adjust.flags & MAP_BLUE) 
	bnd =xid_view.rgbplanes[2];

    /*  Kludge - need special handling for ALL	*/
    if (color->adjust.flags & MAP_ALL) 
	bnd =xid_view.rgbplanes[0];
#endif OLDXID
   

    /*  Get new piecewise linear adjusted values for the color	*/
    count = color->adjust.index +1;

    piece (color->adjust.from,
	   color->adjust.to,
	   &count,
	   map);

    /*  Get the histogram adjusted for the piecewise linear mapping	*/
    if (color->adjust.flags & MAP_ALL) 
    {
	int	p, i;

	for (i=0;i<256;i++) adjusted_hist[i] = 0;
	for (p=0; p<3; p++)
	{
	    bnd =xid_view.rgbplanes[p];
	    histptr = (long *)xid_planes[bnd-1].histogram;
	    ct_calc_adjusted_hist (histptr, map, all_hist);
	    for (i=0;i<256;i++) adjusted_hist[i] += all_hist[i];
	}

    }
    else
    {
	histptr = (long *)xid_planes[bnd-1].histogram;
	ct_calc_adjusted_hist (histptr, map, adjusted_hist);
    }

    /* Find the maximum value of the histogram for autoscaling the y axis*/
    histptr = adjusted_hist;
    ct_calc_cumm_hist (histptr, cum_histo, &Max);

    if (CT_Hist_Type == HIST_CUM)
    {
	histptr = cum_histo;

	Max = cum_histo[255];

    }
    Max = Max * ScaleFactor;
    /*  Draw the histogram in the color of the color being adjusted	*/
    /*  The size of the histogram is set for a 1 pixel x value or 256	*/
    XClearWindow (xid.display, Histo_win);
    XSetForeground(xid.display,color->gc,color->linecolor);
    XSetLineAttributes(xid.display,color->gc,1,LineSolid,CapButt,JoinMiter);

    /*  Get height, width of drawing area for histogram	*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNheight, &hist_height); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, &hist_width); argcnt++;
    XtGetValues(Histo_widget, args, argcnt);

    for (freq = 0; freq < 256; histptr++, freq++)
    {
        if (*histptr > 0)
	    {
	        y2 = hist_height - ((*histptr / (float) Max) * hist_height);
		if (y2 < 0 ) y2 = 0;
		x1 = x2 = freq;
		y1 = hist_height;
		XDrawLine (xid.display, Histo_win, color->gc, x1,y1, x2, y2);
	    }
    }

    ct_draw_tickmarks (Max, color);

    histptr = adjusted_hist;
    ct_histo_statistics (histptr, cum_histo);

#ifdef HISTISSTRETCH
    ct_drawline( Histo_win,
	    color->gc,
	    color->adjust.from,
	    color->adjust.to,
	    color->adjust.index,
	    color->linecolor, 5);
#endif HISTISSTRETCH
}

/******************************************************************************/
/*
	
	This function is called for expose events on the histogram drawing
	area. 

*/
/******************************************************************************/
void ct_histo_expose(Widget w, XtPointer data, XEvent *report)
{
    XID_COLORTOOLPTR   	color= 	*( (XID_COLORTOOLPTR *) data);

#ifdef HISTISSTRETCH
    Histo_win = XtWindow (ct_stretch_drawing);
    Histo_widget = ct_stretch_drawing;
#else
    Histo_win = XtWindow (w);
    Histo_widget = w;
#endif HISTISSTRETCH

    ct_histo_draw (color);
}
/******************************************************************************/
/*
	
	This function is called for pointer motion events on the 
	histogram drawing area. Keyboard events are also interpreted for
	cursor keys and vi control keys.

*/
/******************************************************************************/
void ct_histo_pointer(Widget w, XtPointer data, XEvent *report)
{
    XEvent *event;

    long deltax, deltay, move;

    long keycode, state;

    Dimension hist_height, hist_width;

    Arg		args[2];
    int		argcnt;
    Boolean	argok;

    /*
	Get the event values from the widget.
    */
    event = (XEvent *) report;

    /*
	Remove all other MotionNotify events from queue -- this makes for
	smoother scrolling by helping to limit the number of events
	generated while we're acting on the present one.
    */
    while (XCheckTypedEvent(xid.display,MotionNotify,event));

    /*
	If a a valid cursor key was pressed on the keyboard then move
	the cursor to the new location.
    */
    if (event->type == KeyRelease)
    {
    /*
	Check the validity of the cursor coordinates.
    */
	/*  Get height, width of drawing area for histogram	*/
	argcnt = 0;
	XtSetArg(args[argcnt], XmNheight, &hist_height); argcnt++;
	XtSetArg(args[argcnt], XmNwidth, &hist_width); argcnt++;
	XtGetValues(Histo_widget, args, argcnt);

	if ((event->xkey.x < 0) || (event->xkey.y < 0) ||
	    (event->xkey.x > hist_width) || (event->xkey.y > hist_height))
	    return;

    /*
	Capture the key code and the keyboard state.
    */
	keycode = event->xkey.keycode;
	state   = event->xkey.state;

    /*
	Ensure the keys validity.
    */
	if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.j_key) ||
	    (keycode == xid_cursor.h_key) || (keycode == xid_cursor.l_key) ||
	    (keycode == xid_cursor.left_key) || (keycode == xid_cursor.up_key) ||
	    (keycode == xid_cursor.right_key) || (keycode == xid_cursor.down_key))
	    {
    /*
	Reset the delta X and Y amounts.
    */
	    deltax = deltay = 0;

    /*
	Check the keyboard state to see if the Shift key was used.
    */
	    if (state == ShiftMask)
		move = 10;               /* Shift key so use delta of 10.     */
	    else
		move = 1;

    /*
	Set the delta X/Y amount based on the key pressed.
    */
	    if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.up_key))
		deltay -= move;
	    else if ((keycode == xid_cursor.j_key) || (keycode ==
		xid_cursor.down_key))
		deltay += move;
	    else if ((keycode == xid_cursor.h_key) || (keycode ==
		xid_cursor.left_key))
		deltax -= move;
	    else
		deltax += move;

    /*
	If a valid move can be made then move the cursor.
    */
	    if (((event->xkey.x + deltax) < 0) 
	    || ((event->xkey.x + deltax) > hist_width)
	    || ((event->xkey.y + deltay) < 0) 
	    || ((event->xkey.y + deltay) > hist_height))
		;
	    else
		XWarpPointer(xid.display,None,None,None,None,None,None,
		    deltax,deltay);
	    }
    }
    /*
    Calculate the gray level value corresponding to the cursor position if the
    cursor is within the bounds of the X axis of the histogram.
    */

    ct_histo_stat_pct (CT_Color->adjusted_hist,
		       CT_Color->cum_histo,
		        (event->xmotion.x ));

}
/******************************************************************************/
/*
	
	This function changes the context of the current contrast stretch
	drawing area to the color specified in the client data. The 
	current color is erased and the new color drawn in the same
	drawing area.  
*/
/******************************************************************************/
static char	*confirm_all_msg = 
	 "ALL resets RGB stretches to ALL stretch, please confirm" ;

void
ct_set_sliders_text (int newx, int newy)
{

    char	value_string[4];

    newy = 255 - newy;
    if (CT_Color->lasty != newy)
    {
	CT_Color->lasty = newy;
	sprintf (value_string, "%3d", CT_Color->lasty);
	XmTextFieldSetString (ct_vert_value, value_string);
	XmScaleSetValue (ct_scale_vertical, CT_Color->lasty);
    }
    if (CT_Color->lastx != newx)
    {
	CT_Color->lastx = newx;
	sprintf (value_string, "%3d", CT_Color->lastx);
	XmTextFieldSetString (ct_horiz_value, value_string);
	XmScaleSetValue (ct_scale_horiz, CT_Color->lastx);
    }

}

void
ct_changecolor_func (XID_COLORTOOLPTR color)
{
	char	value_string[4];

	ct_drawline( CT_Color->window,
		CT_Color->gc,
		CT_Color->adjust.from,
		CT_Color->adjust.to,
		CT_Color->adjust.index,
		CT_Color->background, SIZEBREAKPTBOX);


	CT_Color = color;

	
	ct_drawline( CT_Color->window,
		CT_Color->gc,
		CT_Color->adjust.from,
		CT_Color->adjust.to,
		CT_Color->adjust.index,
		CT_Color->linecolor, SIZEBREAKPTBOX);
	
	ct_apply (CT_Color);

	ct_histo_draw (CT_Color);

	sprintf (value_string, "%3d", CT_Color->lasty);
	XmTextFieldSetString (ct_vert_value, value_string);
	XmScaleSetValue (ct_scale_vertical,  CT_Color->lasty);
	sprintf (value_string, "%3d", CT_Color->lastx);
	XmTextFieldSetString (ct_horiz_value, value_string);
	XmScaleSetValue (ct_scale_horiz,  CT_Color->lastx);
}

extern	Widget	*CT_ColorWidget[];
void
ct_change_to_all()
{
    extern	XID_COLORTOOL	allrgb;

    ct_changecolor_func (&allrgb);
    XmToggleButtonSetState (*CT_ColorWidget[3], 
			True, False);

}
void
ct_cancel_all ()
{
    int i;

    for (i=0; i<2; i++)
    {
	if (CT_Colors[i] == CT_Color)
	    XmToggleButtonSetState (*CT_ColorWidget[i], 
			True, False);
    }
    XmToggleButtonSetState (*CT_ColorWidget[3], 
	    False, False);
}


/* ARGSUSED */
void
ct_changecolor(Widget w, XID_COLORTOOLPTR client, XtPointer call)
{
	XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

	if (client->adjust.flags & MAP_ALL)
	{
	    int	i,j, diff = False, curr_color;
	    /*  
		Check if any colors have been updated and
		warn user that ALL will override these updates
 	    */
	    for (i=0; i<3;i++)
	    {
		if (CT_Colors[i] == CT_Color)
		    curr_color = i;
		if (CT_Colors[i]->adjust.index != client->adjust.index)
		    diff = True;
		else
		    for (j=0; j<= client->adjust.index;j++)
		    {
			if (CT_Colors[i]->adjust.from[j] 
			!= client->adjust.from[j]) 
			    diff = True;
			if (CT_Colors[i]->adjust.to[j] 
			!= client->adjust.to[j]) 
			    diff = True;
		    }
	    }
	    if (diff)
	    {
		XmToggleButtonSetState (w, False, False);
		/*
		XmToggleButtonSetState (*CT_ColorWidget[curr_color], 
			True, False);
		*/
		ct_get_confirmation (confirm_all_msg, 
			ct_change_to_all, ct_cancel_all);
		return;
	    }
	}

	client->window = CT_Color->window;
	ct_changecolor_func (client);
}


void
ct_new_vertical_value (int x, int y)
{
	int newx, newy;
	/*  Erase previous line by drawing with background color	*/
	ct_drawline( CT_Color->window,
		CT_Color->gc,
		CT_Color->adjust.from,
		CT_Color->adjust.to,
		CT_Color->adjust.index,
		CT_Color->background, SIZEBREAKPTBOX);

	/*  setbreakpoint expects y in X coordinate style	*/
	newx = x;
	newy = 255 - y;
	ct_movebreakpoint (CT_Color->adjust.from,
               CT_Color->adjust.to,
               &CT_Color->adjust.index,x, 255-CT_Color->lasty, 
	       &newx, &newy);

	ct_drawline( CT_Color->window,
		CT_Color->gc,
		CT_Color->adjust.from,
		CT_Color->adjust.to,
		CT_Color->adjust.index,
		CT_Color->linecolor, SIZEBREAKPTBOX);

	ct_apply (CT_Color);

	CT_Color->lasty = 255 - newy;
}
/* ARGSUSED */
void
ct_change_scale_vertical(Widget w, XtPointer client, XtPointer call)
{
        XmScaleCallbackStruct *scale =(XmScaleCallbackStruct*)call;
	int	x, y;
	char	value_string[4];

	y = scale->value;
	x = CT_Color->lastx;

	ct_new_vertical_value (x, y);
	sprintf (value_string, "%3d", CT_Color->lasty);
	XmTextFieldSetString (ct_vert_value, value_string);
	if (CT_Color->lasty != y)
	    XmScaleSetValue (ct_scale_vertical, CT_Color->lasty);
#ifdef CURRSTRETCH
	ct_new_curr_stretch();
#endif
       if (XtIsManaged (ct_cmap_mainwindow))
       {
	   extern xcmap_draw_hls_colormap();
	   xcmap_draw_hls_colormap();
       }
}

/* ARGSUSED */
void
ct_drag_scale_vertical(Widget w, XtPointer client, XtPointer call)
{
        XmScaleCallbackStruct *scale =(XmScaleCallbackStruct*)call;
	int	x, y;
	char	value_string[4];

	y = scale->value;
	x = CT_Color->lastx;

	ct_new_vertical_value (x, y);
	sprintf (value_string, "%3d", CT_Color->lasty);
	XmTextFieldSetString (ct_vert_value, value_string);
	if (CT_Color->lasty != y)
	    XmScaleSetValue (ct_scale_vertical, CT_Color->lasty);
}


/* ARGSUSED */
void
ct_change_vert_text(Widget w, XtPointer client, XtPointer call)
{

    char	*textstring;
    int	result, y;

    textstring = XmTextFieldGetString (w);

    result = sscanf (textstring, "%d", &y);

    if (result == 1 && (y >= 0 && y < 256)) 
    {
	ct_new_vertical_value (CT_Color->lastx, y);
	XmScaleSetValue (ct_scale_vertical, CT_Color->lasty);
	if (CT_Color->lasty != y)
	{
	    char value_string [4];

	    sprintf (value_string, "%3d", CT_Color->lasty);
	    XmTextFieldSetString (ct_vert_value, value_string);
	}
#ifdef CURRSTRETCH
	ct_new_curr_stretch();
#endif
       if (XtIsManaged (ct_cmap_mainwindow))
       {
	   extern xcmap_draw_hls_colormap();
	   xcmap_draw_hls_colormap();
       }
		
    }
}



void
ct_new_horizontal_value (int x, Boolean	setbreakpoint)
{
	int y, map [256];
	int count;
	int movey;
	int	brkpt;
	XID_COLORTOOLPTR color = CT_Color;

	char	value_string[4];

	count = color->adjust.index +1;

	piece (color->adjust.from,
	       color->adjust.to,
	       &count,
	       map);

	y = map [x];
	XmScaleSetValue (ct_scale_vertical, y);

	sprintf (value_string, "%3d", x);
	XmTextFieldSetString (ct_horiz_value, value_string);
	sprintf (value_string, "%3d", y);
	XmTextFieldSetString (ct_vert_value, value_string);
	
	ct_histo_stat_pct (color->adjusted_hist,
		       color->cum_histo, x);
	if (setbreakpoint)
	{
	    /*  Erase previous line by drawing with background color*/
	    ct_drawline( color->window,
		    color->gc,
		    color->adjust.from,
		    color->adjust.to,
		    color->adjust.index,
		    color->background, SIZEBREAKPTBOX);

	    /*  Erase lines showing scale position		*/
	    XSetForeground(xid.display,color->gc,
		color->background);
	    XDrawLine (xid.display, color->window,
		color->gc, color->lastx, 255, 
		color->lastx, 
		255-(color->lasty-1));
	    XDrawLine (xid.display, color->window,
		color->gc, color->lastx, 
		255-(color->lasty-1), 
		255, 255-(color->lasty-1));
	    brkpt = ct_findbreakpoint (color->adjust.from,
			color->adjust.to, 
			color->adjust.index, 
			color->lastx, 
			color->lasty);
	    if (brkpt <  color->adjust.index)
		ct_breakpointlocation(color->window,color->gc,
		    color->lastx, 255-color->lasty,color->background);


	    /*  setbreakpoint expects y in X coordinate style	*/
	    movey = 255 - y;
	    if (color->lastx == (x+1)
	    ||  color->lastx == (x-1))
		ct_movebreakpoint(color->adjust.from,
		    color->adjust.to,
		    &color->adjust.index,
		    color->lastx, 255-color->lasty, 
		    &x, &movey);
	    else
		/*  This sets the breakpoint to the exact value	*/
		ct_setbreakpoint (color->adjust.from,
		   color->adjust.to,
		   &color->adjust.index,x, movey,
		   0);

	    ct_drawline( color->window,
		    color->gc,
		    color->adjust.from,
		    color->adjust.to,
		    color->adjust.index,
		    color->linecolor, SIZEBREAKPTBOX);
#ifdef CURRSTRETCH
	    ct_new_curr_stretch();
#endif
	   if (XtIsManaged (ct_cmap_mainwindow))
	   {
	       extern xcmap_draw_hls_colormap();
	       xcmap_draw_hls_colormap();
	   }

	}
	else
	{
	    /*  Draw lines to show scale position	*/

	    XSetForeground(xid.display,color->gc,
		color->background);
	    XDrawLine (xid.display, color->window,
		color->gc, color->lastx, 255, 
		color->lastx, 
		255-(color->lasty-1));
	    XDrawLine (xid.display, color->window,
		color->gc, color->lastx, 
		255-(color->lasty-1), 
		255, 255-(color->lasty-1));

	    brkpt = ct_findbreakpoint (color->adjust.from,
			color->adjust.to, 
			color->adjust.index, 
			color->lastx, 
			color->lasty);
	    if (brkpt <  color->adjust.index)
		ct_breakpointlocation(color->window,color->gc,
		    color->lastx, 255-color->lasty,color->background);

	    XSetForeground(xid.display,color->gc,
		color->foreground);
	    XDrawLine (xid.display, color->window,
		color->gc, x, 255, x, 255-(y-1));
	    XDrawLine (xid.display, color->window,
		color->gc, x, 255-(y-1), 255, 255-(y-1));

	    brkpt = ct_findbreakpoint (color->adjust.from,
			color->adjust.to, 
			color->adjust.index, 
			x, y);
	    if (brkpt <  color->adjust.index)
		ct_breakpointlocation(color->window,color->gc,
		    x, 255-y ,color->foreground);

	    ct_drawline( color->window,
		    color->gc,
		    color->adjust.from,
		    color->adjust.to,
		    color->adjust.index,
		    color->linecolor, SIZEBREAKPTBOX);
	}
	color->lasty = y;

}

/* ARGSUSED */
void
ct_change_scale_horiz(Widget w, XtPointer client, XtPointer call)
{
        XmScaleCallbackStruct *scale =(XmScaleCallbackStruct*)call;
	int	x;

#ifdef DEBUG
	printf ("Value of scale = %d\n", scale->value);
#endif DEBUG

	x = scale->value;

	ct_new_horizontal_value (x, True);
	CT_Color->lastx = x;
}

/* ARGSUSED */
void
ct_drag_scale_horiz(Widget w, XtPointer client, XtPointer call)
{
        XmScaleCallbackStruct *scale =(XmScaleCallbackStruct*)call;

	int	x, y;

	x = scale->value;

	ct_new_horizontal_value (x, False);
	CT_Color->lastx = x;
}

/* ARGSUSED */
void
ct_change_horiz_text(Widget w, XtPointer client, XtPointer call)
{

	char	*textstring;
	int	result, x;

	textstring = XmTextFieldGetString (w);

	result = sscanf (textstring, "%d", &x);

	if (result == 1 && (x >= 0 && x < 256)) 
	{
		XmScaleSetValue (ct_scale_horiz, x);
		ct_new_horizontal_value (x, True);
		CT_Color->lastx = x;
		
	}
}


void
ct_popup_stretch (Widget w, XtPointer client, XtPointer call)
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

    if (XtIsManaged (ct_stretch_mainWindow))
    {
	
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Stretch", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtUnmanageChild (ct_stretch_mainWindow);
	XtPopdown (xidStretch);
#define CMAP
    }
    else
    {
	extern int	NumStretch;
	extern void	ct_init_stretch_list ();

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Hide Stretch", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtManageChild (ct_stretch_mainWindow);
	XtPopup (xidStretch, XtGrabNone);

	if (NumStretch == 0)
	    ct_init_stretch_list ();

    }
}

void
ct_dismiss_stretch  (Widget w, XtPointer client, XtPointer call)
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

    if (XtIsManaged(ct_show_stretch_button))
    {
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Stretch", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (ct_show_stretch_button, args, argcnt);
	XmStringFree (xmstr[0]);
    }

    XtUnmanageChild (ct_stretch_mainWindow);
    XtPopdown (xidStretch);
}

void
ct_popup_map (Widget w, XtPointer client, XtPointer call)
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];

    extern Widget       xidCMap, ct_cmap_mainwindow;

    if (XtIsManaged (ct_cmap_mainwindow))
    {
	
	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Show Map", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtUnmanageChild (ct_cmap_mainwindow);
	XtPopdown (xidCMap);
    }
    else
    {

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Hide Map", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	XtManageChild (ct_cmap_mainwindow);
	XtPopup(xidCMap,XtGrabNone);

    }
}

#define UNDO	1
#define REDO	2
int Undo_Redo	 = UNDO;

void
ct_undo_redo (Widget w, XtPointer client, XtPointer call)
{

    Arg		args[2];
    int		argcnt;
    Boolean	argok;
    XmString    xmstr[256];
    
    extern	int	CurrStretch;
    extern	void	ct_set_stretch(), ct_new_curr_stretch ();

    if (Undo_Redo == UNDO)
    {
	CurrStretch = PREVIOUS;
	ct_set_stretch ();

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Redo", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	Undo_Redo = REDO;
	if (XtIsManaged (ct_cmap_mainwindow))
	{
	   extern xcmap_draw_hls_colormap();
	   xcmap_draw_hls_colormap();
	}

    }
    else
    {
	CurrStretch = PREVIOUS;
	ct_set_stretch ();

	argcnt=0;
	XtSetArg(args[argcnt], XmNlabelString, (xmstr[0]=XmStringCreateLtoR(
	    "Undo", XmSTRING_DEFAULT_CHARSET))); argcnt++;
	XtSetValues (w, args, argcnt);
	XmStringFree (xmstr[0]);
	Undo_Redo = UNDO;
	if (XtIsManaged (ct_cmap_mainwindow))
	{
	   extern xcmap_draw_hls_colormap();
	   xcmap_draw_hls_colormap();
	}

    }

}


#define BITS_USED 4	/* See "Notes" below.				*/

/*
 *  Macro used to calculate a color table index number.
 */

#define RGB_TO_INDEX(r,g,b) \
	(*(shift_red + (r)) + *(shift_green + (g)) + *(shift_blue + (b)))

/*
 *  Macro used to expand a scaled value to an 8-bit value.
 */

#define MAKE_8_BITS(v) \
	(((v) << (8 - bits_used)) | (0xff >> (bits_used)))

/*
 *  Color table entry structure.
 */

struct ENTRY_DEF
 {
  unsigned long index,		/* Pixel value index.			*/
		count;		/* Pixel population count.		*/

  unsigned char	reserved,	/* Boolean for a reserved color entry.	*/
		dn_1,		/* unknown				*/
		dn_2,
		red,		/* Color table entry RGB values.	*/
		green,
		blue;
 };

struct 	ENTRY_DEF *table;	/* Working color table.			*/

long	table_length,		/* # of color table entries.		*/
	bits_used;		/* # of bits for each color.		*/

unsigned long	shift_red[256],	/* Shift values to scale input data.	*/
		shift_green[256],
		shift_blue[256];


FUNCTION ct_rgb2lut(num_images,red,green,blue,npts,lutlen,composite,red_lut,
		  green_lut,blue_lut,num_reserved,reserved_red,reserved_green,
		  reserved_blue)

long	num_images,		/* Number of input images. (I)		*/
	*npts,			/* Total number of input pixels. (I)	*/
	lutlen,			/* Number of LUT entries to use. (I)	*/
	num_reserved;		/* Number of reserved colors in LUT. (I)*/

unsigned char	**red,		/* Red pixel values. (I)		*/
		**green,	/* Green pixel values. (I)		*/
		**blue,		/* Blue pixel values. (I)		*/
		**composite,	/* Output composite image. (O)		*/
		*red_lut,	/* Output red LUT. (O)			*/
		*green_lut,	/* Output green LUT. (O)		*/
		*blue_lut,	/* Output blue LUT. (O)			*/
		*reserved_red,	/* Input reserved red LUT values. (I)	*/
		*reserved_green,/* Input reserved green LUT values. (I)	*/
		*reserved_blue;	/* Input reserved blue LUT values. (I)	*/

/******************************************************************************
 *									      *
 *	Purpose:  This color compression routine is designed to input three   *
 *		  8-bit image bands and output a single image band (composite)*
 *		  and three LUTs.  The routine is designed to work within a   *
 *		  specified color limit.				      *
 *									      *
 *		  This routine is basically a rewrite of the		      *
 *		  NASA JPL rgbs_to_images_and_luts routine developed by       *
 *		  CalTech.  The original copyright appears below:	      *
 *								   	      *
 *	Copyright (c) 1989, California Institiute of Technology.  U.S.	      *
 *	Government sponsorship under NASA Contract NAS7-918 is acknowledged.  *
 *									      *
 *	Modified by D. Akkerman, USGS/EROS Data Center, 8/90.		      *
 *	- adapted for use with the XLASDSP application.			      *
 *	Updated by D. Akkerman, USGS/EROS Data Center, 9/91.		      *
 *	- fixed a problem with clearing table space when the table is full.   *
 *	- rewritten for clarity, debugging, and portability.		      *
 *									      *
 *	Algorithm(flow):  See in-line code.				      *
 *									      *
 *	Notes: The BITS_USED constant is used to define the number of bits    *
 *	per component value (R,G,B) that the data is to be scaled to.  This   *
 *	value is used as a power of 2 to determine the range of the component *
 *	values.  For example, if BITS_USED equals four, then 2^4 or 16 values *
 *	(0 to 15) are allowed for each R, G, or B component value.  Thus, red *
 *	component values are input with a range of 0 to 255 and are scaled    *
 *	(shifted) to a range of 0 to 15.  This value, in turn, also 	      *
 *	determines the length of the color compression table used: 	      *
 *	(1 << (3 * bits_used)).  Again if BITS_USED equals 4, then	      *
 *	table_length = 4096 (or (1 << 12)).  Four is used as the default      *
 *	because it provides the best overall range of color compression,      *
 *	requires a moderate amount of memory, maintains a relatively small    *
 *	table size, and requires a modest amount of processing.  Lower values *
 *	are inadequate because too few colors are maintained, and conversely, *
 *	larger values are expensive in terms of processing resources and      *
 *	don't provide adequate color compression when the total number of     *
 *	colors gets large (simulated colors and total error gets very high).  *
 *									      *
 ******************************************************************************/
{
struct ENTRY_DEF *entry,/* Working color table pointers.		*/
		 *table_ptr;

int	ct_n_compare();	/* Quick sort (qsort) compare function.		*/

register
long	i,j,k;		/* Loop indices.				*/

long	pixel,		/* Pixel loop counter.				*/
	source,		/* Working color table index (source entry).	*/
	next_free_entry,/* Used to index temporary entries for
			   table compression.				*/
	reserved_index,	/* Working reserved pixel index.		*/
	r_temp,		/* Working RGB pixel values; to shorten code.	*/
	g_temp,
	b_temp,
	unclassed_index,/* Unclassified index used in color compression.*/
	best_dist,	/* Closest compressed color (least distance).	*/
	best_index,	/* Closest compressed color table entry.	*/
	dist,		/* Working distance variable.			*/
	square[513],	/* Squares of differences table.		*/
	*square_off,	/* Pointer to differences table midpoint (zero).*/
	red_diff,	/* Working pixel difference variables.		*/
	green_diff,
	blue_diff,
	one = 1;	/* Nonfatal error specifier.			*/


unsigned long	limit,	/* Largest internal R,G,B values.		*/
		index,	/* Color table index values.			*/
		index2;

register
unsigned char *red_ptr,	/* Working reserved color pointers.		*/
	      *green_ptr,
	      *blue_ptr,
			/* Working composite pixel (LUT entry) value.	*/
	      *composite_ptr;

unsigned char pix_red,	/* Working pixel component values.		*/
	      pix_green,
	      pix_blue;

unsigned long adjmap[3][256];	/* PWL adjusted map for RGB		*/


#define DEBUG
#ifdef DEBUG
long	simulated = 0;	/* Number of pixels that were compressed.	*/

float	total_error=0.0;/* Calculated error based on the amount of
			   difference between the target pixel and
			   the closest matching and the number of
			   target pixels so compressed.			*/
#endif


/*
 *  Initialize the color table.
 */

/* Assign the number of bits per component value to use.		*/

bits_used = BITS_USED;

/* Create shift tables to scale component values.			*/

for (i = 0; i < 256; i++)
 {
  shift_red[i]   = (i >> (8 - bits_used)) << (2 * bits_used);
  shift_green[i] = (i >> (8 - bits_used)) << bits_used;
  shift_blue[i]  = (i >> (8 - bits_used));
 }

/* Assign the table length, allocate the table and initialize it.	*/

table_length = (1 << (3 * bits_used));

table = (struct ENTRY_DEF *)malloc(table_length * sizeof(struct ENTRY_DEF));
if (table == NULL)
  {
   c_errmsg("Error allocating dynamic memory","ct_rgb2lut-malloc",&one);
   return(E_FAIL);
  }

(void)memset((char *)table,0,(table_length * sizeof(struct ENTRY_DEF)));

limit = (1 << bits_used);
for (pix_red = 0; pix_red < limit; pix_red++)
  for (pix_green = 0; pix_green < limit; pix_green++)
    for (pix_blue = 0; pix_blue < limit; pix_blue++)
	{
	 index = (pix_red << (2 * bits_used)) |
		 (pix_green << bits_used) |
		 pix_blue;

	 entry = (table + index);
	 entry->red   = pix_red;
	 entry->green = pix_green;
	 entry->blue  = pix_blue;
	 entry->index = index;
	}

/*
 *  Construct the color table from the input parameters.
 */

/* Mark the reserved colors (if any) in the color table.		*/

if (num_reserved != 0)
 {
  red_ptr   = reserved_red;
  green_ptr = reserved_green;
  blue_ptr  = reserved_blue;

  for (i = 0; i < num_reserved; i++)
   {
    index = RGB_TO_INDEX(*red_ptr,*green_ptr,*blue_ptr);
    (table + index)->reserved = 1;
    red_ptr++;
    green_ptr++;
    blue_ptr++;
   }
 }

/*  Calculate the piece-wise linear adjustment maps	*/
for (i=0; i< 3; i++)
{
    int count;

    count = CT_Colors[i]->adjust.index + 1;
    piece (CT_Colors[i]->adjust.from,
	   CT_Colors[i]->adjust.to,
	   &count,
	   adjmap[i]);
}

/* Populate the color table using the input data.			*/

for (i = 0; i < num_images; i++)
 {
  int	mapred, mapgreen, mapblue;

  red_ptr   = *(red + i);
  green_ptr = *(green + i);
  blue_ptr  = *(blue + i);

  for (pixel = 0; pixel < *(npts + i); pixel++)
   {

    mapred = adjmap[0][*red_ptr];
    mapgreen = adjmap[1][*green_ptr];
    mapblue = adjmap[1][*blue_ptr];

    index = RGB_TO_INDEX(mapred, mapgreen, mapblue);
    (table + index)->count++;
    red_ptr++;
    green_ptr++;
    blue_ptr++;
   }
 }

/* Clear space at the front of the table for reserved colors (if any)
   by moving existing entries to empty spaces (if they exist) at the
   end of the table.							*/

if (num_reserved != 0)
 {
  j = (table_length - 1);
  for (i = 0; i < num_reserved; i++)
   {
    if ((table + i)->count != 0)
     {
      for (k = j; k > 0; k--)
       if (((table + k)->count == 0) && ((table + k)->reserved == 0))
	  {
	   (void)memcpy((char *)(table + k),(char *)(table + i),sizeof(*table));
	   break;
	  }
      if (k == 0)
	{
#ifdef DEBUG
	  printf("No free entries -- break used\n");
#endif
	  break;	/* No free entries; no sense continuing. 	*/
	}
     }
   }
 }

/* Compress the color table using reserved colors by preference.	*/

next_free_entry = num_reserved;
for (source = num_reserved; source < table_length; source++)
 {
  if ((table + source)->reserved)
   {
    r_temp = (table + source)->red; /* Use temps to shorten statements.	*/
    g_temp = (table + source)->green;
    b_temp = (table + source)->blue;

    index2 = RGB_TO_INDEX(MAKE_8_BITS(r_temp),MAKE_8_BITS(g_temp),
			 MAKE_8_BITS(b_temp));

    for (reserved_index = 0; reserved_index < num_reserved; reserved_index++)
     {
	index = RGB_TO_INDEX(*(reserved_red + reserved_index),
			     *(reserved_green + reserved_index),
			     *(reserved_blue + reserved_index));

	if (index == index2)
	  (void)memcpy((char *)(table + reserved_index),
		       (char *)(table + source),sizeof(*table));
     }
   }
  else if ((table + source)->count != 0)
	{
	 (void)memcpy((char *)(table + next_free_entry),
		      (char *)(table + source),sizeof(*table));
	 next_free_entry++;
 	}
 }

/*
 *  Sort the non-reserved colors to determine the most popular.
 */

qsort((char *)(table + num_reserved),(next_free_entry - num_reserved),
      sizeof(*table),ct_n_compare);

/*
 *  Now merge infrequent colors with the most popular.
 */

/* Create a table for the squares of differences and assign a pointer
   to zero squared (midpoint).						*/

for (i = -256; i < 256; i++)
  square[i + 256] = (i * i);

square_off = &square[256];

/* Perform the merge.							*/

for (j = 0; j < lutlen; j++)
  (table + j)->dn_1 = j;	/* Number each table entry.		*/

for (unclassed_index = lutlen; unclassed_index < next_free_entry; 
	unclassed_index++)
 {
  /* Calculate the best distance value for the target entry.		*/

  pix_red   = (table + unclassed_index)->red;
  pix_green = (table + unclassed_index)->green;
  pix_blue  = (table + unclassed_index)->blue;

  red_diff    = (pix_red   - (table + lutlen - 1)->red);
  green_diff  = (pix_green - (table + lutlen - 1)->green);
  blue_diff   = (pix_blue  - (table + lutlen - 1)->blue);

  best_dist  = (*(square_off + red_diff) + *(square_off + green_diff) +
	        *(square_off + blue_diff));

  best_index = lutlen - 1;

  table_ptr = (table + lutlen - 2);

  /* Search the most popular colors in the color table for the closest
     match.								*/

  for (j = (lutlen - 2); j >= 0; j--)
   {
    red_diff   = (pix_red   - table_ptr->red);
    green_diff = (pix_green - table_ptr->green);
    blue_diff  = (pix_blue  - table_ptr->blue);

    dist = (*(square_off + red_diff) + *(square_off + green_diff) +
	    *(square_off + blue_diff));

    if (dist < best_dist)
     {
      best_dist = dist;
      best_index = j;
     }

    table_ptr--;
   }

  /* Assign the index of the closest matching color to the compress
     color entry.							*/

  (table + unclassed_index)->dn_1 = best_index;

#ifdef DEBUG
  simulated += (table + unclassed_index)->count;
  total_error += (table + unclassed_index)->count * sqrt((double)best_dist);
#endif
 }

#ifdef DEBUG
printf ("# \tR  \tG  \tB  \tcount \tindex\n");
for (j=0; j< next_free_entry; j++)
    printf ("#%d \t%d \t%d \t%d \t%d\t%d\n", 
    j, table[j].red, table[j].green, table[j].blue, 
    table[j].count, table[j].dn_1);
printf("simulated: %d\n",simulated);
printf("total_error: %g\n",total_error);

#endif

/*
 *  Now create the output LUTs.
 */

for (i = 0; i < lutlen; i++)
 {
  red_lut[i]   = MAKE_8_BITS((table + i)->red);
  green_lut[i] = MAKE_8_BITS((table + i)->green);
  blue_lut[i]  = MAKE_8_BITS((table + i)->blue);
 }

/*
 *  Create the composite image.
 */

/* Associate the index value with an output pixel (LUT entry) value.	*/

for (i = 0; i < next_free_entry; i++)
 {
  index = (table + i)->index;
  pixel = (table + i)->dn_1;
  (table + index)->dn_2 = pixel;
 }

/* Create the output composite values.					*/

for (i = 0; i < num_images; i++)
 { 
  int mapred, mapgreen, mapblue;

  red_ptr       = (*(red + i));
  green_ptr     = (*(green + i));
  blue_ptr      = (*(blue + i));
  composite_ptr = (*(composite + i));

  for (pixel = 0; pixel < *(npts + i); pixel++)
   {
    mapred = adjmap[0][*red_ptr];
    mapgreen = adjmap[1][*green_ptr];
    mapblue = adjmap[1][*blue_ptr];

    index = RGB_TO_INDEX(mapred, mapgreen, mapblue);
    *composite_ptr = (table + index)->dn_2;
    red_ptr++;
    green_ptr++;
    blue_ptr++;
    composite_ptr++;
   }
 }

/*
 *  Free the color table.
 */

free((char *)table);

/*
 *  Finis!
 */

return(E_SUCC);
}

/******************************************************************************
 *									      *
 *									      *
 ******************************************************************************/

FUNCTION ct_n_compare(entry1,entry2)

struct ENTRY_DEF *entry1,	/* Input color table entries. (I)	*/
		 *entry2;

/******************************************************************************
 *									      *
 *	Purpose:  This routine is the compare function used by the qsort      *
 *		  function.  It operates as follows:			      *
 *									      *
 *		  If arg1 is less than arg2 then return 1		      *
 *		  If arg1 is equal to arg2 then return 0		      *
 *		  If arg1 is greater than arg2 then return -1	      	      *
 *									      *
 *		  This arrangement allows the array to be sorted in	      *
 *		  descending order with the lowest address containing the     *
 *		  highest element.					      *
 *									      *
 *	Notes:	  See documentation for the standard qsort function.	      *
 *									      *
 ******************************************************************************/
{
long	return_value;	/* Return compare value.			*/


return_value = (entry1->count < entry2->count) ? 1 :
			(entry1->count == entry2->count) ? 0 : -1;

return(return_value);
} 

FUNCTION ct_load_display(nplanes,imgwin,shell,win,nl,ns,maxval,rptr,gptr,
    bptr,cmflg,cmapsize, colormap, visual_info, pixptr,loadflg,viewflg,flickerflg)

Widget	shell,			/* Widget ID of image window shell (I).	      */
        win;			/* Widget ID of image window (I).	      */

Window  imgwin;                 /* Window ID for main image window (I).       */

XVisualInfo	*visual_info;

Colormap	colormap;

Pixmap	*pixptr;		/* Pointer to pixmap which is to contain the
				   loaded image data (I).		      */

long    cmflg,			/* Indicates whether or not color map should
				   be modified (I).			      */
	cmapsize,		/* Size of colormap			      */
	flickerflg,		/* Indicates whether or not this load is part
				   of the flicker process (I).		      */
	loadflg,		/* Indicates whether this is being called from
				   the loadimg routine (I).		      */
	*maxval,		/* Maximum image data value (I).	      */
	*nl,			/* Number of lines to load (I).	   	      */
	*ns,			/* Number of samples to load (I).	      */
	*nplanes,		/* Number of planes loaded (I).	      	      */
	viewflg;		/* If TRUE, resize viewport to original
				   specification.  If FALSE, base resizing on
				   the current viewport width and height.  (I)*/

unsigned char
	*rptr,			/* Pointer to red image data (I).	      */
	*gptr,			/* Pointer to green image data (I).	      */
	*bptr;			/* Pointer to blue image data (I).	      */

/*******************************************************************************
Purpose:
    This code was modified from the file xid_func/load.c.  Modifications 
    include utilizing the piece-wise linear map to redetermine the appropriate
    colormap entries to be given to each pixel.  The colormap generation is 
    also modified to subdivide the RGB space more appropriately for the data 
    provided by utilizing the modified histogram information.  The RGB space
    is subdivided using the updated mean of red, green and blue as the center
    of the space.  The standard deviation of each color is used to determine
    the sizes of the boxes around the center.  The 5% and 95% values are 
    used to further constrain the initial subdivision.  Once the divisioning
    is complete, the popularity algorithm is then applied to determine the 
    best set of colormap entries to be utilized.  

    This routine outputs the input image data to the current display.  The
    appropriate colormap is defined and installed based on the display type.
    The image data is actually loaded into a pixmap which is used to set
    the background of the main image window.  Load_display also clears any
    currently displayed zoomed or full resolution windows, verifies whether
    or not the profile and cursor Z options are valid and updates the plane
    list displayed by the show function.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
Arg     args[4];                /* Variable used to set widget resources.     */

Widget	widget;			/* Widget ID.				      */

GC      rgb_gc;                 /* Working color GC.                          */

XGCValues
	gc_val;                 /* GC set value array.                        */

XImage  *ximage;                /* X image structure.                         */

Dimension 
	view_height,		/* Viewport height.			      */
	view_width;		/* Viewport width.			      */

static unsigned char
	rr[256],                /* Reserved color values.                     */
	gr[256],
	br[256];

float	scaler;                 /* LUT scaler for actual number of colors.    */

long	buf_size,               /* Number of bytes in a single band.          */
	changegc = FALSE,	/* Were the foreground and background colors
				   of the default GC modified?		      */
	count,			/* Number of active planes.		      */
	current,        	/* Current index into plane number string 
				   array.      				      */
	i,			/* Looping index.			      */
	img_idx,		/* Index into colormap of first image color.  */
	num_imgcolors,		/* Number of image colors.		      */
	offset,                 /* Offset used in scaling the 24-bit LUT.     */
	show_flg,		/* Indicates whether or not the show panel
				   is active.				      */
	state,			/* Toggle button state.			      */
	tform[256];		/* Pixel transform array.		      */

unsigned long
	*data;                	/* Pointer to output values for 24-bit data.  */

unsigned char
        *blut,                  /* Blue LUT for composite image.              */
	*composite,             /* Composite RGB 8-bit image or single band
				   black white 8-bit image.                   */
        *comptr,                /* Pointer to composite buffer.               */
	*dataptr,               /* Pointer to image data stored in memory
				   plane.                                     */
	*glut,                  /* Green LUT for composite image.             */
	*rgbptr[3],             /* Pointers to RGB components.                */
	*rlut;                  /* Red LUT for composite image.               */

char    fullres_opt[4],		/* Current full resolution size option.	      */
	mesg[50],		/* Warning message text.		      */
	points[10],             /* Number of points string.	   	      */
	tmpstr[50];     	/* Temporary string variable.                 */

/*
    Calculate the image data buffer size.
*/
buf_size = *nl * *ns;

/*
    Calculate the scaler needed to reduce the image to the appropriate number
    of colors.
*/
if (cmapsize == NUM_IMGCOLORS)
    {
    scaler = (256.0 / (float)NUM_IMGCOLORS);       /* (2^8 / # of colors) */
    num_imgcolors = NUM_IMGCOLORS;
    img_idx = IMG_IDX;
    }
else 
    {
    scaler = 1.0;
    num_imgcolors = 256;
    img_idx = 0;
    }
/*
    8-bit monitor -- RGB image.
*/ 
if (visual_info[0].visual->class == PseudoColor)
    {
/*
    Allocate buffer space for LUTs.
*/
    rlut = (unsigned char *)malloc(256);
    glut = (unsigned char *)malloc(256);
    blut = (unsigned char *)malloc(256);
/*
    Compress the 3 band RGB image into a single band of image data and
    3 LUTs.
*/
    comptr = composite = (unsigned char *)malloc(buf_size);
    rgbptr[RED] = rptr; 
    rgbptr[GREEN] = gptr; 
    rgbptr[BLUE] = bptr; 
    if (cmflg)
	{
        ct_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
            num_imgcolors,&composite,rlut,glut,blut,0,rr,gr,br);
/*
    Save this LUT to be used as reserved colors for loading of full
    resolution windows.
*/
	for (i=0; i < 256; i++)
	    {
	    rr[i] = rlut[i];
	    gr[i] = glut[i];
	    br[i] = blut[i];
	    }
	}
    else
        ct_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
            num_imgcolors,&composite,rlut,glut,blut,num_imgcolors,rr,gr,br);

/*
    Build the new colormap.
*/
    if (cmflg)
	{
       if (xid_pwl.color_pwl == NULL) 
	   xid_pwl.color_pwl = (XColor *) calloc(cmapsize,sizeof(XColor));
        for (i = 0; i < num_imgcolors; i++)
            {
	    xid_pwl.color_pwl[i].red = (rlut[i] * 257);
	    xid_pwl.color_pwl[i].green = (glut[i] * 257);
	    xid_pwl.color_pwl[i].blue = (blut[i] * 257);
	    xid_pwl.color_pwl[i].pixel = img_idx+i;
            xid_pwl.color_pwl[i].flags = DoRed | DoGreen | DoBlue;
            }
	}

    for (i=0; i < buf_size; i++)
	composite[i] = xid_pwl.color_pwl[composite[i]].pixel;
/*
    Store the new colormap.
*/
    if (cmflg)
        XStoreColors(xid.display,colormap,&xid_pwl.color_pwl[0],
	    num_imgcolors);
    }
else if (visual_info[0].visual->class == TrueColor)
{

	int	adjmap[3][256];
	/*  Calculate the piece-wise linear adjustment maps	*/
	for (i=0; i< 3; i++)
	{
	    int count;

	    count = CT_Colors[i]->adjust.index + 1;
	    piece (CT_Colors[i]->adjust.from,
		   CT_Colors[i]->adjust.to,
		   &count,
		   adjmap[i]);
	}

	data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
	if (data == NULL)
	    {
	    xid_message("Insufficient memory available");
	    return(XID_F);
	    }
/*
    Set pointers to the red, green and blue bands of image data.
*/
	rgbptr[RED] = rptr;
	rgbptr[GREEN] = gptr; 
	rgbptr[BLUE] = bptr; 

/*
    Scale the input data.
*/
	for (i = 0; i < buf_size; i++, rgbptr[RED]++, rgbptr[GREEN]++, 
	    rgbptr[BLUE]++) {
	    data[i] = 0 + ((adjmap[0][*rgbptr[RED]])) + 
	   		  ((adjmap[1][*rgbptr[GREEN]]) << 8) + 
			  ((adjmap[2][*rgbptr[BLUE]]) << 16);
	}
}

/*
    Create a graphics context for the image window.
*/
gc_val.function = GXcopy;
gc_val.plane_mask = AllPlanes;

rgb_gc = XCreateGC(xid.display,imgwin,GCFunction | GCPlaneMask,&gc_val);

/*
    Allocate buffer space.
*/
/*
    Create an X image.
*/
    if (visual_info[0].depth == 24)
	ximage = XCreateImage(xid.display,visual_info[0].visual,
	    visual_info[0].depth,ZPixmap,0,(char *)data,*ns,*nl,8,0);
    else
	ximage = XCreateImage(xid.display,visual_info[0].visual,
	    visual_info[0].depth,ZPixmap,0,(char *)composite,*ns,*nl,8,0);
/*
    Create a pixmap for the image window.
*/
    *pixptr = XCreatePixmap(xid.display,imgwin,*ns,*nl,visual_info[0].
	depth);
/*
    Draw the image into the image pixmap.
*/
    XPutImage(xid.display,*pixptr,rgb_gc,ximage,0,0,0,0,*ns,*nl);

XFlush(xid.display);

/*
    Set the image pixmap to be the background of the image window.
*/
XSetWindowBackgroundPixmap(xid.display,imgwin,*pixptr);

/*
    Clear the window to update the background.
*/
XClearArea(xid.display,imgwin,0,0,0,0,True);

/*
    Set the composite structure fields to reflect the currently displayed
    image.
*/

/*
    Free the appropriate resources.
*/
if ((visual_info[0].depth == 24) && !flickerflg)
    free(data);
else if (visual_info[0].visual->class == PseudoColor)
    {
    if ((!flickerflg) || (flickerflg && *nplanes == 3))
        free(composite);
    if (*nplanes != 1)
	{
	free(rlut);
	free(glut);
	free(blut);
	}
    }
/*
    The data in the image structure has already been freed above.
*/
if (!flickerflg)
    {
    ximage->data = NULL;
    XDestroyImage(ximage);
    }
XFreeGC(xid.display,rgb_gc);
return(XID_S);
}

static XVisualInfo	*big_visual = NULL;
static Widget	ct_Shell_MainWin, 
		    ct_main_image_bb, 
		    ct_main_image_scrollwin;

static Colormap		ct_colormap = 0;
static Pixmap		ct_pixmap;
Widget
ct_Createmain_image_scrollwin(parent)
Widget parent;
{
    Arg    	args[512];
    int    	argcnt;
    Widget 	retval;
    Dimension	shadow,border,hilight,width,height;

    XmRegisterConverters();
 
/*
    Create the main image scrolled window.
*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNscrollingPolicy, XmAUTOMATIC); argcnt++;
    XtSetArg(args[argcnt], XmNscrollBarDisplayPolicy, XmSTATIC); argcnt++;
    XtSetArg(args[argcnt], XmNx, 0); argcnt++;
    XtSetArg(args[argcnt], XmNy, 0); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 512); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 512); argcnt++;
    ct_main_image_scrollwin = XtCreateWidget("main_image_scrollwin",
		xmScrolledWindowWidgetClass,
		parent,
		args,
		argcnt);

    retval = ct_main_image_scrollwin;
 
/*
    Create the bulletin board for the main image window.
*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNmarginWidth, (short)0); argcnt++;
    XtSetArg(args[argcnt], XmNmarginHeight, (short)0); argcnt++;
    XtSetArg(args[argcnt], XmNresizePolicy, XmRESIZE_ANY); argcnt++;
    XtSetArg(args[argcnt], XmNnoResize, False); argcnt++;
    XtSetArg(args[argcnt], XmNx, 0); argcnt++;
    XtSetArg(args[argcnt], XmNy, 0); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 1024); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 1024); argcnt++;
    ct_main_image_bb = XtCreateWidget("main_image_bb",
		xmBulletinBoardWidgetClass,
		ct_main_image_scrollwin,
		args,
		argcnt);

/*
    Add an event handler to detect pointer motion and key release events in
    the main image window.
    XtAddEventHandler(ct_main_image_bb,PointerMotionMask|KeyReleaseMask,
	FALSE,cursor_pointer,NULL);
*/

/*
    Add an event handler to detect exposure events in the main image window.
    XtAddEventHandler(ct_main_image_bb,ExposureMask,FALSE,
        graphics_expose,NULL);
*/
    XtManageChild(ct_main_image_bb);

/*
    Retrieve the widget IDs of the scroll bars on the main image window.
*/
    XtSetArg(args[0],XmNhorizontalScrollBar,&main_hor_scrollbar);
    XtSetArg(args[1],XmNverticalScrollBar,&main_vert_scrollbar);
    XtGetValues(ct_main_image_scrollwin,args,2);

/*
    Turn traversal off on the scrollbars.  If this is not done, the single-
    stepping of the cursor with the arrow keys does not work.
*/
    XtSetArg(args[0],XmNtraversalOn,False);
    XtSetValues(main_hor_scrollbar,args,1);
    XtSetValues(main_vert_scrollbar,args,1);

/*
    Determine the width, shadow thickness, border width and highlight
    thickness of each of the scroll bars and the shadow thickness and 
    border width of the scrolled window.  These values are used to 
    calculate the maximum size of the image window.
*/
    XtSetArg(args[0],XmNwidth,&width);
    XtSetArg(args[1],XmNshadowThickness,&shadow);
    XtSetArg(args[2],XmNborderWidth,&border);
    XtSetArg(args[3],XmNhighlightThickness,&hilight);
    XtGetValues(main_vert_scrollbar,args,4);

    XtSetArg(args[0],XmNheight,&height);
    XtSetArg(args[1],XmNshadowThickness,&shadow);
    XtSetArg(args[2],XmNborderWidth,&border);
    XtSetArg(args[3],XmNhighlightThickness,&hilight);
    XtGetValues(main_hor_scrollbar,args,4);

    XtSetArg(args[0],XmNshadowThickness,&shadow);
    XtSetArg(args[1],XmNborderWidth,&border);
    XtGetValues(ct_main_image_scrollwin,args,2);

    XtSetArg(args[0],XmNshadowThickness,&shadow);
    XtSetArg(args[1],XmNborderWidth,&border);
    XtGetValues(ct_main_image_bb,args,2);

    return( retval );
}


void 
ct_remap_image (Widget w, XtPointer client, XtPointer call)
{
    Arg     args[256];              /* X resource arguments.                */
    long    argcnt;

    int status, i;
    Window imgwin;
    XVisualInfo visual_template, *visual_info;
    unsigned long	visual_mask=0, number_visuals, max_cmap_size=0;

unsigned long   pixelval[4096],  /* Array for pixel values.                    */
                plane_masks[1], /* Working plane mask.                        */
                wk_pixels[1],   /* Return pixel value array.                  */
                red_mask,       /* Mask values to reference 24-bit pixels.    */
                green_mask,
                blue_mask;


    visual_info = xid.visual_info;
#define GETBIGVISUAL
#ifdef GETBIGVISUAL
  if (big_visual == NULL)
  {
    visual_template.screen = xid.screen;
    visual_template.class = TrueColor;
    visual_template.depth = 24;
    visual_mask = VisualScreenMask;     /* Limit to present screen.     */
    visual_mask |= VisualDepthMask;     /* Limit to Depth assigned.     */
    visual_mask |= VisualClassMask;     /* Limit to class assigned.     */

    number_visuals = 0;
    visual_info = XGetVisualInfo(xid.display,visual_mask,&visual_template,
                                        &number_visuals);
    for (i=0; i < number_visuals; i++)
    {
        if (visual_info[i].colormap_size > max_cmap_size)
        {
            max_cmap_size = visual_info[i].colormap_size;
	    big_visual = &(visual_info[i]);
	}
    }

    visual_info = big_visual;

    if (ct_colormap == 0 )
    {
        ct_colormap = XCreateColormap(xid.display,
                            RootWindow(xid.display,xid.screen),
                            big_visual,AllocNone);
	status = XAllocColorCells(xid.display,ct_colormap,True, plane_masks,
                                0,pixelval,max_cmap_size);
    }

/*
 *  Create the main image window shell.
 */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "X Color Tool - Image Window"); argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "Image Window"); argcnt++;
    XtSetArg(args[argcnt], XmNkeyboardFocusPolicy, XmPOINTER); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,big_visual->visual); argcnt++;
    XtSetArg(args[argcnt],XtNdepth,big_visual->depth); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,ct_colormap); argcnt++;
    ct_Shell_MainWin = XtCreatePopupShell("shell_mainwin",
        topLevelShellWidgetClass, AppShell, args, argcnt);
    XtAddCallback(ct_Shell_MainWin, XmNdestroyCallback, main_image_destroy, 0);

    ct_main_image_scrollwin = ct_Createmain_image_scrollwin(ct_Shell_MainWin);
    XtManageChild(ct_main_image_scrollwin);
    XtPopup(ct_Shell_MainWin,XtGrabNone);

  }
  else
      visual_info = big_visual;


#endif

    
    imgwin = XtWindow(ct_main_image_bb);
    status = ct_load_display(&xid_load.totbnd,
		imgwin,
		ct_Shell_MainWin,
		ct_main_image_bb,
		&xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NL],
	        &xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NS],
		&xid_planes[xid_view.rgbplanes[RED] - 1].maxval,
		xid_planes[xid_view.rgbplanes[RED] - 1].image_data,
		xid_planes[xid_view.rgbplanes[GREEN] - 1].image_data,
		xid_planes[xid_view.rgbplanes[BLUE] - 1].image_data,
		TRUE, NUM_IMGCOLORS, ct_colormap ,visual_info, &ct_pixmap,
		TRUE,TRUE,FALSE);

}

void 
ct_histo_up (Widget w, XtPointer client, XtPointer call)
{
    char	value_string [5];

    ScaleFactor += .1;
    if (ScaleFactor >= .91)
	XtSetSensitive (w, False);
    else
	XtSetSensitive (ct_histo_down_button, True);

    sprintf (value_string, "%3.2f", ScaleFactor);
    XmTextFieldSetString (ct_histo_scale_text, value_string);
    ct_histo_draw (CT_Color);
}

void 
ct_histo_down (Widget w, XtPointer client, XtPointer call)
{
    char	value_string [10];

    ScaleFactor -= .1;
    if (ScaleFactor <= .1)
	XtSetSensitive (w, False);
    else
	XtSetSensitive (ct_histo_up_button, True);

    sprintf (value_string, "%3.2f", ScaleFactor);
    XmTextFieldSetString (ct_histo_scale_text, value_string);
    ct_histo_draw (CT_Color);
}
void 
ct_change_scale_text (Widget w, XtPointer client, XtPointer call)
{

    char	*textstring;
    int	result;
    float	scalefactor;

    textstring = XmTextFieldGetString (w);

    result = sscanf (textstring, "%f", &scalefactor);

    if (result == 1 && (scalefactor > 0.0 && scalefactor <= 1.0)) 
    {
	ScaleFactor = scalefactor;
	ct_histo_draw (CT_Color);
	    
	if (ScaleFactor >= .91)
	    XtSetSensitive (ct_histo_up_button, False);
	else
	    XtSetSensitive (ct_histo_down_button, True);
	if (ScaleFactor <= .1)
	    XtSetSensitive (ct_histo_down_button, False);
	else
	    XtSetSensitive (ct_histo_up_button, True);
    }
    else
    {
	char	value_string [10];

	sprintf (value_string, "%3.1f", ScaleFactor);
	XmTextFieldSetString (ct_histo_scale_text, value_string);
    }
}
