/****************************************************************************/
/*   PROGRAM NAME  : ct_interface.h                                         */
/*   AUTHOR        : g e nostrand                                           */
/*   DATE          : july 14, 1993                                          */
/*   LOCATION      : DSRC PRINCETON, NJ                                     */
/*                                                                          */
/*   PROGRAM DESCRIPTION:                                                   */
/*   									    */
/*       Header file for XID ColorTool extensions.  			    */
/*       Major components defined, XID_STRETCHLIST and XID_COLORTOOL.       */
/*  									    */
/*   PROGRAM MODIFICATIONS:                                                 */
/****************************************************************************/

/*  
    Size of breakpoint box on piece-wise linear tool display.  Larger sizes
    make the breakpoint easier to grab with the mouse.  Smaller sizes allow 
    breakpoints to be closer together but harder to grab.  
*/ 
#define SIZEBREAKPTBOX  8

typedef void            (*P_FUNC)();
typedef char            *P_CHAR;


/*
    The XID_STRETCHLIST structure maintains saved piece-wise linear stretches 
    in order to provide a robust mechanism for doing "what if" color analysis.
    At any point, the user can save a stretch up to a maximum of MAXSTRETCHES
    number of saved stretches.  The stretch can then be recalled at a later 
    time and compared with other stretches.  The stucture XID_COLORSTRETCH 
    maintains the stretches for each color (and saturation if defined).       
    The definition of the saved stretches is defined in 
    ct_stretchoperations.c.  The current stretch information is maintained 
    within the XID_COLORTOOL structure for each color.  

    The structure maintains similar data as the structure XID_PWL but for 
    a single color at a time. A variation from the XID_PWL structure is the 
    usage of the index field.  The index field specifies the index of the last 
    entry in the from/to arrays.  To calculate the number of elements in the 
    array, add 1 to index.
*/
#define MAXSTRETCHES	100

/*  Define special stretch list slots to track current and previous values 
    to allow for UNDO/REDO style operations	
*/

#define CURRENT 	0	/*  Most recently applied stretch	*/
#define PREVIOUS 	1	/*  Previously applied stretch		*/
#define LINEAR 		2	/*  Linear ramp stretch used to reset   */
#define FIRSTSTRETCH 	3	/*  First user added stretch index      */

typedef struct _XID_STRETCHLIST {
   long 	 flags;	  /*  Specify color (MAP_{RED,GREEN,BLUE,ALL}         */
   long          index;   /*  Index of last element in from/to arrays         */
   long          *from;   /*  From (original) array breakpoint value	      */
   long          *to;     /*  To (mapped) array breakpoint value              */
} XID_STRETCHLIST, *XID_STRETCHLISTPTR;


/*
    The XID_COLORSTRETCH structure saves the stretches for red, green and 
    blue (and saturation if defined).  The array can be referenced using the 
    defines RED, GREEN and BLUE found in xid.h.  The definition of the
    stretchlist is found in ct_stretchoperations.c
*/
#define STRETCHSAT	BLUE+1
#define STRETCHALL	STRETCHSAT+1
typedef struct _XID_COLORSTRETCH {
  XID_STRETCHLIST	color[4];
} XID_COLORSTRETCH, *XID_COLORSTRETCHPTR;

/*
    The XID_COLORTOOL structure is a convenience structure for maintaining
    all of the information necesary to provide a dynamic interactive 
    piece-wise linear adjustment tool with a mapped dynamic histogram.  
    An XID_COLORTOOL structure is maintained for each color, as well as for
    the ALL and Saturation options of the tool. The XID_COLORTOOL structure 
    can be referenced as individual structures using the variables red,green,
    blue, allrgb and satrgb.  These structure can also be referenced throught
    the array CT_Colors.  Within the ct_graphicoperation.c functions, the 
    current XID_COLORTOOL structure can be referenced through CT_Color.
*/ 
typedef struct _XID_COLORTOOL {
   Window        	window;		/* Window of drawing area           */
   GC 		 	gc;		/* Graphic Context for drawing area */
   XID_STRETCHLIST 	adjust;		/* Piece-wise linear stretch info   */

   int           	x;		/* x location of cursor 	    */
   int           	y;		/* y localtion of cursor	    */
   int			lastx;		/* Previous x location of cursor    */
   int			lasty;		/* Previous y location of cursor    */
   /* Adjusted histogram for mapped image 				    */
   /* Histogram already calculated for image in xid_planes[bnd-1].histogram */
   unsigned long	adjusted_hist[256];
   /* Cumulative histogram for mapped image                                 */
   unsigned long	cum_histo[256];
   /* Mapping from original pixel value to new pixel value                  */
   unsigned long	map[256];
   unsigned int  	width;		/* Width of drawing area	    */
   unsigned int  	height;         /* Height of drawing area           */
   unsigned long 	linecolor;     	/* Color of line for drawing graph  */
   unsigned long 	foreground;     /* Foreground color for drawing area*/
   unsigned long 	background;	/* Background color for drawing area*/
} XID_COLORTOOL, *XID_COLORTOOLPTR;

#ifdef DECLARECT
XID_COLORTOOLPTR       	CT_Color;
XID_COLORTOOL        	red, green, blue, allrgb, satrgb;
XID_COLORTOOLPTR	CT_Colors[5] = { &red, &green, &blue, &satrgb, &allrgb };

Widget			*CT_ColorWidget [5] = {
			&ct_stretch_color_red,
			&ct_stretch_color_green,
			&ct_stretch_color_blue,
			&ct_stretch_color_sat,
			&ct_stretch_color_all
			};
#else
extern XID_COLORTOOLPTR       	CT_Color;
extern XID_COLORTOOL        	red, green, blue, allrgb, satrgb;
extern XID_COLORTOOLPTR	CT_Colors[5] ;

extern Widget			*CT_ColorWidget [5] ;
#endif
