/****************************************************************************/
/*   PROGRAM NAME  : ct_loadimg.c                                           */
/*   AUTHOR        : g e nostrand                                           */
/*   DATE          : july 14, 1993                                          */
/*   LOCATION      : DSRC PRINCETON, NJ                                     */
/*                                                                          */
/*   PROGRAM DESCRIPTION:                                                   */
/*   PROGRAM MODIFICATIONS:                                                 */
/****************************************************************************/

#include "xid.h"
#include "ct_interface.h"

/*****************************************************************************/
/*
	 New colortool widgets generated from Builder Xcessory file ct.uil
*/
/*****************************************************************************/


/*  Variables for the Color Tool separate image window  */
XColor          CT_Img_Color [256];

XVisualInfo     *CT_Img_Visual = NULL;

Colormap        CT_Img_CMap = 0;
Pixmap          CT_Img_Pixmap;


#define BITS_USED 4	/* See "Notes" below.				*/

/*
 *  Macro used to calculate a color table index number.
 */

#define RGB_TO_INDEX(r,g,b) \
	(*(shift_red + (r)) + *(shift_green + (g)) + *(shift_blue + (b)))

/*
 *  Macro used to expand a scaled value to an 8-bit value.
 */

#define MAKE_8_BITS(v) \
	(((v) << (8 - bits_used)) | (0xff >> (bits_used)))

/*
 *  Color table entry structure.
 */

struct ENTRY_DEF
 {
  unsigned long index,		/* Pixel value index.			*/
		count;		/* Pixel population count.		*/

  unsigned char	reserved,	/* Boolean for a reserved color entry.	*/
		dn_1,		/* unknown				*/
		dn_2,
		red,		/* Color table entry RGB values.	*/
		green,
		blue;
 };

struct 	ENTRY_DEF *table;	/* Working color table.			*/

long	table_length,		/* # of color table entries.		*/
	bits_used;		/* # of bits for each color.		*/

unsigned long	shift_red[256],	/* Shift values to scale input data.	*/
		shift_green[256],
		shift_blue[256];


FUNCTION ct_rgb2lut(num_images,red,green,blue,npts,lutlen,composite,red_lut,
		  green_lut,blue_lut,num_reserved,reserved_red,reserved_green,
		  reserved_blue)

long	num_images,		/* Number of input images. (I)		*/
	*npts,			/* Total number of input pixels. (I)	*/
	lutlen,			/* Number of LUT entries to use. (I)	*/
	num_reserved;		/* Number of reserved colors in LUT. (I)*/

unsigned char	**red,		/* Red pixel values. (I)		*/
		**green,	/* Green pixel values. (I)		*/
		**blue,		/* Blue pixel values. (I)		*/
		**composite,	/* Output composite image. (O)		*/
		*red_lut,	/* Output red LUT. (O)			*/
		*green_lut,	/* Output green LUT. (O)		*/
		*blue_lut,	/* Output blue LUT. (O)			*/
		*reserved_red,	/* Input reserved red LUT values. (I)	*/
		*reserved_green,/* Input reserved green LUT values. (I)	*/
		*reserved_blue;	/* Input reserved blue LUT values. (I)	*/

/******************************************************************************
 *									      *
 *	Purpose:  This color compression routine is designed to input three   *
 *		  8-bit image bands and output a single image band (composite)*
 *		  and three LUTs.  The routine is designed to work within a   *
 *		  specified color limit.				      *
 *									      *
 *		  This routine is basically a rewrite of the		      *
 *		  NASA JPL rgbs_to_images_and_luts routine developed by       *
 *		  CalTech.  The original copyright appears below:	      *
 *								   	      *
 *	Copyright (c) 1989, California Institiute of Technology.  U.S.	      *
 *	Government sponsorship under NASA Contract NAS7-918 is acknowledged.  *
 *									      *
 *	Modified by D. Akkerman, USGS/EROS Data Center, 8/90.		      *
 *	- adapted for use with the XLASDSP application.			      *
 *	Updated by D. Akkerman, USGS/EROS Data Center, 9/91.		      *
 *	- fixed a problem with clearing table space when the table is full.   *
 *	- rewritten for clarity, debugging, and portability.		      *
 *									      *
 *	Algorithm(flow):  See in-line code.				      *
 *									      *
 *	Notes: The BITS_USED constant is used to define the number of bits    *
 *	per component value (R,G,B) that the data is to be scaled to.  This   *
 *	value is used as a power of 2 to determine the range of the component *
 *	values.  For example, if BITS_USED equals four, then 2^4 or 16 values *
 *	(0 to 15) are allowed for each R, G, or B component value.  Thus, red *
 *	component values are input with a range of 0 to 255 and are scaled    *
 *	(shifted) to a range of 0 to 15.  This value, in turn, also 	      *
 *	determines the length of the color compression table used: 	      *
 *	(1 << (3 * bits_used)).  Again if BITS_USED equals 4, then	      *
 *	table_length = 4096 (or (1 << 12)).  Four is used as the default      *
 *	because it provides the best overall range of color compression,      *
 *	requires a moderate amount of memory, maintains a relatively small    *
 *	table size, and requires a modest amount of processing.  Lower values *
 *	are inadequate because too few colors are maintained, and conversely, *
 *	larger values are expensive in terms of processing resources and      *
 *	don't provide adequate color compression when the total number of     *
 *	colors gets large (simulated colors and total error gets very high).  *
 *									      *
 ******************************************************************************/
{
struct ENTRY_DEF *entry,/* Working color table pointers.		*/
		 *table_ptr;

int	ct_n_compare();	/* Quick sort (qsort) compare function.		*/

register
long	i,j,k;		/* Loop indices.				*/

long	pixel,		/* Pixel loop counter.				*/
	source,		/* Working color table index (source entry).	*/
	next_free_entry,/* Used to index temporary entries for
			   table compression.				*/
	reserved_index,	/* Working reserved pixel index.		*/
	r_temp,		/* Working RGB pixel values; to shorten code.	*/
	g_temp,
	b_temp,
	unclassed_index,/* Unclassified index used in color compression.*/
	best_dist,	/* Closest compressed color (least distance).	*/
	best_index,	/* Closest compressed color table entry.	*/
	dist,		/* Working distance variable.			*/
	square[513],	/* Squares of differences table.		*/
	*square_off,	/* Pointer to differences table midpoint (zero).*/
	red_diff,	/* Working pixel difference variables.		*/
	green_diff,
	blue_diff,
	one = 1;	/* Nonfatal error specifier.			*/


unsigned long	limit,	/* Largest internal R,G,B values.		*/
		index,	/* Color table index values.			*/
		index2;

register
unsigned char *red_ptr,	/* Working reserved color pointers.		*/
	      *green_ptr,
	      *blue_ptr,
			/* Working composite pixel (LUT entry) value.	*/
	      *composite_ptr;

unsigned char pix_red,	/* Working pixel component values.		*/
	      pix_green,
	      pix_blue;

unsigned long *adjmap[3];	/* PWL adjusted map for RGB		*/


#ifdef DEBUG
long	simulated = 0;	/* Number of pixels that were compressed.	*/

float	total_error=0.0;/* Calculated error based on the amount of
			   difference between the target pixel and
			   the closest matching and the number of
			   target pixels so compressed.			*/
#endif


/*
 *  Initialize the color table.
 */

/* Assign the number of bits per component value to use.		*/

bits_used = BITS_USED;

/* Create shift tables to scale component values.			*/

for (i = 0; i < 256; i++)
 {
  shift_red[i]   = (i >> (8 - bits_used)) << (2 * bits_used);
  shift_green[i] = (i >> (8 - bits_used)) << bits_used;
  shift_blue[i]  = (i >> (8 - bits_used));
 }

/* Assign the table length, allocate the table and initialize it.	*/

table_length = (1 << (3 * bits_used));

table = (struct ENTRY_DEF *)malloc(table_length * sizeof(struct ENTRY_DEF));
if (table == NULL)
  {
   c_errmsg("Error allocating dynamic memory","ct_rgb2lut-malloc",&one);
   return(E_FAIL);
  }

(void)memset((char *)table,0,(table_length * sizeof(struct ENTRY_DEF)));

limit = (1 << bits_used);
for (pix_red = 0; pix_red < limit; pix_red++)
  for (pix_green = 0; pix_green < limit; pix_green++)
    for (pix_blue = 0; pix_blue < limit; pix_blue++)
	{
	 index = (pix_red << (2 * bits_used)) |
		 (pix_green << bits_used) |
		 pix_blue;

	 entry = (table + index);
	 entry->red   = pix_red;
	 entry->green = pix_green;
	 entry->blue  = pix_blue;
	 entry->index = index;
	}

/*
 *  Construct the color table from the input parameters.
 */

/* Mark the reserved colors (if any) in the color table.		*/

if (num_reserved != 0)
 {
  red_ptr   = reserved_red;
  green_ptr = reserved_green;
  blue_ptr  = reserved_blue;

  for (i = 0; i < num_reserved; i++)
   {
    index = RGB_TO_INDEX(*red_ptr,*green_ptr,*blue_ptr);
    (table + index)->reserved = 1;
    red_ptr++;
    green_ptr++;
    blue_ptr++;
   }
 }

/*  Calculate the piece-wise linear adjustment maps	*/
for (i=0; i< 3; i++)
{
    int count;

    count = CT_Colors[i]->adjust.index + 1;
    adjmap[i] = CT_Colors[i]->map;
}

/* Populate the color table using the input data.			*/

for (i = 0; i < num_images; i++)
 {
  int	mapred, mapgreen, mapblue;

  red_ptr   = *(red + i);
  green_ptr = *(green + i);
  blue_ptr  = *(blue + i);

  for (pixel = 0; pixel < *(npts + i); pixel++)
   {

#ifdef ADJUSTEDMAP
    mapred = adjmap[0][*red_ptr];
    mapgreen = adjmap[1][*green_ptr];
    mapblue = adjmap[2][*blue_ptr];
#else
    mapred = *red_ptr;
    mapgreen = *green_ptr;
    mapblue = *blue_ptr;
#endif /* ADJUSTEDMAP */

    index = RGB_TO_INDEX(mapred, mapgreen, mapblue);
    (table + index)->count++;
    red_ptr++;
    green_ptr++;
    blue_ptr++;
   }
 }

/* Clear space at the front of the table for reserved colors (if any)
   by moving existing entries to empty spaces (if they exist) at the
   end of the table.							*/

if (num_reserved != 0)
 {
  j = (table_length - 1);
  for (i = 0; i < num_reserved; i++)
   {
    if ((table + i)->count != 0)
     {
      for (k = j; k > 0; k--)
       if (((table + k)->count == 0) && ((table + k)->reserved == 0))
	  {
	   (void)memcpy((char *)(table + k),(char *)(table + i),sizeof(*table));
	   break;
	  }
      if (k == 0)
	{
#ifdef DEBUG
	  printf("No free entries -- break used\n");
#endif
	  break;	/* No free entries; no sense continuing. 	*/
	}
     }
   }
 }

/* Compress the color table using reserved colors by preference.	*/

next_free_entry = num_reserved;
for (source = num_reserved; source < table_length; source++)
 {
  if ((table + source)->reserved)
   {
    r_temp = (table + source)->red; /* Use temps to shorten statements.	*/
    g_temp = (table + source)->green;
    b_temp = (table + source)->blue;

    index2 = RGB_TO_INDEX(MAKE_8_BITS(r_temp),MAKE_8_BITS(g_temp),
			 MAKE_8_BITS(b_temp));

    for (reserved_index = 0; reserved_index < num_reserved; reserved_index++)
     {
	index = RGB_TO_INDEX(*(reserved_red + reserved_index),
			     *(reserved_green + reserved_index),
			     *(reserved_blue + reserved_index));

	if (index == index2)
	  (void)memcpy((char *)(table + reserved_index),
		       (char *)(table + source),sizeof(*table));
     }
   }
  else if ((table + source)->count != 0)
	{
	 (void)memcpy((char *)(table + next_free_entry),
		      (char *)(table + source),sizeof(*table));
	 next_free_entry++;
 	}
 }

/*
 *  Sort the non-reserved colors to determine the most popular.
 */

qsort((char *)(table + num_reserved),(next_free_entry - num_reserved),
      sizeof(*table),ct_n_compare);

/*
 *  Now merge infrequent colors with the most popular.
 */

/* Create a table for the squares of differences and assign a pointer
   to zero squared (midpoint).						*/

for (i = -256; i < 256; i++)
  square[i + 256] = (i * i);

square_off = &square[256];

/* Perform the merge.							*/

for (j = 0; j < lutlen; j++)
  (table + j)->dn_1 = j;	/* Number each table entry.		*/

for (unclassed_index = lutlen; unclassed_index < next_free_entry; 
	unclassed_index++)
 {
  /* Calculate the best distance value for the target entry.		*/

  pix_red   = (table + unclassed_index)->red;
  pix_green = (table + unclassed_index)->green;
  pix_blue  = (table + unclassed_index)->blue;

  red_diff    = (pix_red   - (table + lutlen - 1)->red);
  green_diff  = (pix_green - (table + lutlen - 1)->green);
  blue_diff   = (pix_blue  - (table + lutlen - 1)->blue);

  best_dist  = (*(square_off + red_diff) + *(square_off + green_diff) +
	        *(square_off + blue_diff));

  best_index = lutlen - 1;

  table_ptr = (table + lutlen - 2);

  /* Search the most popular colors in the color table for the closest
     match.								*/

  for (j = (lutlen - 2); j >= 0; j--)
   {
    red_diff   = (pix_red   - table_ptr->red);
    green_diff = (pix_green - table_ptr->green);
    blue_diff  = (pix_blue  - table_ptr->blue);

    dist = (*(square_off + red_diff) + *(square_off + green_diff) +
	    *(square_off + blue_diff));

    if (dist < best_dist)
     {
      best_dist = dist;
      best_index = j;
     }

    table_ptr--;
   }

  /* Assign the index of the closest matching color to the compress
     color entry.							*/

  (table + unclassed_index)->dn_1 = best_index;

#ifdef DEBUG
  simulated += (table + unclassed_index)->count;
  total_error += (table + unclassed_index)->count * sqrt((double)best_dist);
#endif
 }

#ifdef DEBUG
printf ("# \tR  \tG  \tB  \tcount \tindex\n");
for (j=0; j< next_free_entry; j++)
    printf ("#%d \t%d \t%d \t%d \t%d\t%d\n", 
    j, table[j].red, table[j].green, table[j].blue, 
    table[j].count, table[j].dn_1);
printf("simulated: %d\n",simulated);
printf("total_error: %g\n",total_error);

#endif

/*
 *  Now create the output LUTs.
 */

for (i = 0; i < lutlen; i++)
 {
#ifdef ADJUSTEDMAP
  red_lut[i]   = MAKE_8_BITS((table + i)->red);
  green_lut[i] = MAKE_8_BITS((table + i)->green);
  blue_lut[i]  = MAKE_8_BITS((table + i)->blue);
#else
  red_lut[i]   = adjmap[0][MAKE_8_BITS((table + i)->red)];
  green_lut[i] = adjmap[1][MAKE_8_BITS((table + i)->green)];
  blue_lut[i]  = adjmap[2][MAKE_8_BITS((table + i)->blue)];
#endif /* ADJUSTEDMAP */
 }

/*
 *  Create the composite image.
 */

/* Associate the index value with an output pixel (LUT entry) value.	*/

for (i = 0; i < next_free_entry; i++)
 {
  index = (table + i)->index;
  pixel = (table + i)->dn_1;
  (table + index)->dn_2 = pixel;
 }

/* Create the output composite values.					*/

for (i = 0; i < num_images; i++)
 { 
  int mapred, mapgreen, mapblue;

  red_ptr       = (*(red + i));
  green_ptr     = (*(green + i));
  blue_ptr      = (*(blue + i));
  composite_ptr = (*(composite + i));

  for (pixel = 0; pixel < *(npts + i); pixel++)
   {
#ifdef ADJUSTEDMAP
    mapred = adjmap[0][*red_ptr];
    mapgreen = adjmap[1][*green_ptr];
    mapblue = adjmap[2][*blue_ptr];
#else
    mapred = *red_ptr;
    mapgreen = *green_ptr;
    mapblue = *blue_ptr;
#endif /* ADJUSTEDMAP */

    index = RGB_TO_INDEX(mapred, mapgreen, mapblue);
    *composite_ptr = (table + index)->dn_2;
    red_ptr++;
    green_ptr++;
    blue_ptr++;
    composite_ptr++;
   }
 }

/*
 *  Free the color table.
 */

free((char *)table);

/*
 *  Finis!
 */

return(E_SUCC);
}

/******************************************************************************
 *									      *
 *									      *
 ******************************************************************************/

FUNCTION ct_n_compare(entry1,entry2)

struct ENTRY_DEF *entry1,	/* Input color table entries. (I)	*/
		 *entry2;

/******************************************************************************
 *									      *
 *	Purpose:  This routine is the compare function used by the qsort      *
 *		  function.  It operates as follows:			      *
 *									      *
 *		  If arg1 is less than arg2 then return 1		      *
 *		  If arg1 is equal to arg2 then return 0		      *
 *		  If arg1 is greater than arg2 then return -1	      	      *
 *									      *
 *		  This arrangement allows the array to be sorted in	      *
 *		  descending order with the lowest address containing the     *
 *		  highest element.					      *
 *									      *
 *	Notes:	  See documentation for the standard qsort function.	      *
 *									      *
 ******************************************************************************/
{
long	return_value;	/* Return compare value.			*/


return_value = (entry1->count < entry2->count) ? 1 :
			(entry1->count == entry2->count) ? 0 : -1;

return(return_value);
} 

FUNCTION ct_load_display(nplanes,imgwin,shell,win,nl,ns,maxval,rptr,gptr,
    bptr,cmflg,cmapsize, colormap, visual_info, pixptr,loadflg,viewflg,flickerflg)

Widget	shell,			/* Widget ID of image window shell (I).	      */
        win;			/* Widget ID of image window (I).	      */

Window  imgwin;                 /* Window ID for main image window (I).       */

XVisualInfo	*visual_info;

Colormap	colormap;

Pixmap	*pixptr;		/* Pointer to pixmap which is to contain the
				   loaded image data (I).		      */

long    cmflg,			/* Indicates whether or not color map should
				   be modified (I).			      */
	cmapsize,		/* Size of colormap			      */
	flickerflg,		/* Indicates whether or not this load is part
				   of the flicker process (I).		      */
	loadflg,		/* Indicates whether this is being called from
				   the loadimg routine (I).		      */
	*maxval,		/* Maximum image data value (I).	      */
	*nl,			/* Number of lines to load (I).	   	      */
	*ns,			/* Number of samples to load (I).	      */
	*nplanes,		/* Number of planes loaded (I).	      	      */
	viewflg;		/* If TRUE, resize viewport to original
				   specification.  If FALSE, base resizing on
				   the current viewport width and height.  (I)*/

unsigned char
	*rptr,			/* Pointer to red image data (I).	      */
	*gptr,			/* Pointer to green image data (I).	      */
	*bptr;			/* Pointer to blue image data (I).	      */

/*******************************************************************************
Purpose:
    This code was modified from the file xid_func/load.c.  Modifications 
    include utilizing the piece-wise linear map to redetermine the appropriate
    colormap entries to be given to each pixel.  
*******************************************************************************/
{
Arg     args[4];                /* Variable used to set widget resources.     */

Widget	widget;			/* Widget ID.				      */

GC      rgb_gc;                 /* Working color GC.                          */

XGCValues
	gc_val;                 /* GC set value array.                        */

XImage  *ximage;                /* X image structure.                         */

Dimension 
	view_height,		/* Viewport height.			      */
	view_width;		/* Viewport width.			      */

static unsigned char
	rr[256],                /* Reserved color values.                     */
	gr[256],
	br[256];

float	scaler;                 /* LUT scaler for actual number of colors.    */

long	buf_size,               /* Number of bytes in a single band.          */
	changegc = FALSE,	/* Were the foreground and background colors
				   of the default GC modified?		      */
	count,			/* Number of active planes.		      */
	current,        	/* Current index into plane number string 
				   array.      				      */
	i,			/* Looping index.			      */
	img_idx,		/* Index into colormap of first image color.  */
	num_imgcolors,		/* Number of image colors.		      */
	offset,                 /* Offset used in scaling the 24-bit LUT.     */
	show_flg,		/* Indicates whether or not the show panel
				   is active.				      */
	state,			/* Toggle button state.			      */
	tform[256];		/* Pixel transform array.		      */

unsigned long
	*data;                	/* Pointer to output values for 24-bit data.  */

unsigned char
        *blut,                  /* Blue LUT for composite image.              */
	*composite,             /* Composite RGB 8-bit image or single band
				   black white 8-bit image.                   */
        *comptr,                /* Pointer to composite buffer.               */
	*dataptr,               /* Pointer to image data stored in memory
				   plane.                                     */
	*glut,                  /* Green LUT for composite image.             */
	*rgbptr[3],             /* Pointers to RGB components.                */
	*rlut;                  /* Red LUT for composite image.               */

char    fullres_opt[4],		/* Current full resolution size option.	      */
	mesg[50],		/* Warning message text.		      */
	points[10],             /* Number of points string.	   	      */
	tmpstr[50];     	/* Temporary string variable.                 */

/*
    Calculate the image data buffer size.
*/
buf_size = *nl * *ns;

/*
    Calculate the scaler needed to reduce the image to the appropriate number
    of colors.
*/
if (cmapsize == NUM_IMGCOLORS)
    {
    scaler = (256.0 / (float)NUM_IMGCOLORS);       /* (2^8 / # of colors) */
    num_imgcolors = NUM_IMGCOLORS;
    img_idx = IMG_IDX;
    }
else 
    {
    scaler = 1.0;
    num_imgcolors = 256;
    img_idx = 0;
    }
/*
    8-bit monitor -- RGB image.
*/ 
if (visual_info[0].visual->class == PseudoColor)
    {
    extern void ct_set_stretch();
    extern int	CurrStretch;
/*
    Allocate buffer space for LUTs.
*/
    rlut = (unsigned char *)malloc(256);
    glut = (unsigned char *)malloc(256);
    blut = (unsigned char *)malloc(256);
/*
    Compress the 3 band RGB image into a single band of image data and
    3 LUTs.
*/
    comptr = composite = (unsigned char *)malloc(buf_size);
    if ((rlut == NULL) || (glut == NULL) || (blut == NULL) || (comptr == NULL))
	{
	xid_message("Insufficient memory available");
	return(XID_F);
	}
    rgbptr[RED] = rptr; 
    rgbptr[GREEN] = gptr; 
    rgbptr[BLUE] = bptr; 
    if (cmflg)
	{
        if (ct_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
            num_imgcolors,&composite,rlut,glut,blut,0,rr,gr,br))
	    return (XID_F);
/*
    Save this LUT to be used as reserved colors for loading of full
    resolution windows.
*/
	for (i=0; i < 256; i++)
	    {
	    rr[i] = rlut[i];
	    gr[i] = glut[i];
	    br[i] = blut[i];
	    }
	}
    else
        if(ct_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
            num_imgcolors,&composite,rlut,glut,blut,num_imgcolors,rr,gr,br))
	    return (XID_F);

/*
    Build the new colormap.
*/
    if (cmflg)
	{
        for (i = 0; i < num_imgcolors; i++)
            {
	    CT_Img_Color[i].red = (rlut[i] * 257);
	    CT_Img_Color[i].green = (glut[i] * 257);
	    CT_Img_Color[i].blue = (blut[i] * 257);
	    CT_Img_Color[i].pixel = img_idx+i;
            CT_Img_Color[i].flags = DoRed | DoGreen | DoBlue;
            }
	}

    for (i=0; i < buf_size; i++)
	composite[i] = CT_Img_Color[composite[i]].pixel;
/*
    Store the new colormap.
*/
    if (cmflg)
        XStoreColors(xid.display,colormap,&CT_Img_Color[0],
	    num_imgcolors);
    /*  Apply current stretch to new image window	*/
    CurrStretch = CURRENT;
    ct_set_stretch ();
    }
else if (visual_info[0].visual->class == TrueColor)
{

	unsigned long	*adjmap[3];
	unsigned long	*satmap;
	/*  Calculate the piece-wise linear adjustment maps	*/
	for (i=0; i< 3; i++)
	{
	    int count;

	    count = CT_Colors[i]->adjust.index + 1;
	    adjmap[i] = CT_Colors[i]->map;
	}

	if (satrgb.adjust.index > 1
	|| satrgb.adjust.to[0] != 0
	|| satrgb.adjust.to[satrgb.adjust.index] != 255)
	{
	    int count;

	    count = satrgb.adjust.index + 1;
	    satmap = satrgb.map;
	}
	     

	data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
	if (data == NULL)
	    {
	    xid_message("Insufficient memory available");
	    return(XID_F);
	    }
/*
    Set pointers to the red, green and blue bands of image data.
*/
	rgbptr[RED] = rptr;
	rgbptr[GREEN] = gptr; 
	rgbptr[BLUE] = bptr; 

/*
    Scale the input data.
*/
	for (i = 0; i < buf_size; i++, rgbptr[RED]++, rgbptr[GREEN]++, 
	    rgbptr[BLUE]++) {
	    int red, green, blue;
	    float hue, light, sat;

	    red = adjmap[0][*rgbptr[RED]];
	    green = adjmap[1][*rgbptr[GREEN]];
	    blue = adjmap[2][*rgbptr[BLUE]];
	    if (satrgb.adjust.index > 1
	    || satrgb.adjust.to[0] != 0
	    || satrgb.adjust.to[satrgb.adjust.index] != 255)
	    {
		int satidx;
		rgbhls (red, green, blue, &hue, &light, &sat);

		if (sat != 0.0)
		{
		    satidx = (int) (sat * 255.0 + .5);
		    sat = ((float) satmap[satidx] / 255.0 );
		    hlsrgb (hue, light, sat, &red, &green, &blue);
		}

	    }
	    data[i] = 0 + red + (green << 8) + (blue << 16);
	}
}

/*
    Create a graphics context for the image window.
*/
gc_val.function = GXcopy;
gc_val.plane_mask = AllPlanes;

rgb_gc = XCreateGC(xid.display,imgwin,GCFunction | GCPlaneMask,&gc_val);

/*
    Allocate buffer space.
*/
/*
    Create an X image.
*/
    if (visual_info[0].depth == 24)
	ximage = XCreateImage(xid.display,visual_info[0].visual,
	    visual_info[0].depth,ZPixmap,0,(char *)data,*ns,*nl,8,0);
    else
	ximage = XCreateImage(xid.display,visual_info[0].visual,
	    visual_info[0].depth,ZPixmap,0,(char *)composite,*ns,*nl,8,0);
/*
    Create a pixmap for the image window.
    printf ("Creating Pixmap\n");
*/
    *pixptr = XCreatePixmap(xid.display,imgwin,*ns,*nl,visual_info[0].
	depth);
/*
    Draw the image into the image pixmap.
*/
    XPutImage(xid.display,*pixptr,rgb_gc,ximage,0,0,0,0,*ns,*nl);

XFlush(xid.display);

/*
    Set the image pixmap to be the background of the image window.
*/
XSetWindowBackgroundPixmap(xid.display,imgwin,*pixptr);

/*
    Clear the window to update the background.
printf ("Clearing area \n");
*/
XClearArea(xid.display,imgwin,0,0,0,0,True);

/*
    Set the composite structure fields to reflect the currently displayed
    image.
*/

/*
    Free the appropriate resources.
*/
if ((visual_info[0].depth == 24) && !flickerflg)
    free(data);
else if (visual_info[0].visual->class == PseudoColor)
    {
    if ((!flickerflg) || (flickerflg && *nplanes == 3))
        free(composite);
    if (*nplanes != 1)
	{
	free(rlut);
	free(glut);
	free(blut);
	}
    }
/*
    The data in the image structure has already been freed above.
*/
if (!flickerflg)
    {
    ximage->data = NULL;
    XDestroyImage(ximage);
    }
XFreeGC(xid.display,rgb_gc);
return(XID_S);
}


Widget
ct_Createmain_image_scrollwin(parent, visual_info)
Widget parent;
XVisualInfo *visual_info;
{
    Arg    	args[512];
    int    	argcnt, argok;
    Widget 	retval;
    Dimension	shadow,border,hilight,width,height;

    XmRegisterConverters();
 
/*
    Create the main image scrolled window.
*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNscrollingPolicy, XmAUTOMATIC); argcnt++;
    XtSetArg(args[argcnt], XmNscrollBarDisplayPolicy, XmSTATIC); argcnt++;
    XtSetArg(args[argcnt], XmNx, 0); argcnt++;
    XtSetArg(args[argcnt], XmNy, 0); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 545); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 545); argcnt++;
    if (visual_info->visual->class == TrueColor)
    {
	unsigned long	fgpixel, bgpixel;

	fgpixel = 204 | 204 << 8 | 204 << 16;
	XtSetArg(args[argcnt], XmNforeground, fgpixel); argcnt++;
	bgpixel = 127 | 127 << 8 | 127 << 16;
	XtSetArg(args[argcnt], XmNbackground, bgpixel); argcnt++;

    }

    ct_main_image_scrollwin = XtCreateWidget("ct_main_image_scrollwin",
		xmScrolledWindowWidgetClass,
		parent,
		args,
		argcnt);

    retval = ct_main_image_scrollwin;
 
/*
    Create the bulletin board for the main image window.
*/
    argcnt = 0;
    XtSetArg(args[argcnt], XmNmarginWidth, (short)0); argcnt++;
    XtSetArg(args[argcnt], XmNmarginHeight, (short)0); argcnt++;
    XtSetArg(args[argcnt], XmNresizePolicy, XmRESIZE_ANY); argcnt++;
    XtSetArg(args[argcnt], XmNnoResize, False); argcnt++;
    XtSetArg(args[argcnt], XmNx, 0); argcnt++;
    XtSetArg(args[argcnt], XmNy, 0); argcnt++;
    XtSetArg(args[argcnt], XmNwidth, 1024); argcnt++;
    XtSetArg(args[argcnt], XmNheight, 1024); argcnt++;
    ct_main_image_bb = XtCreateWidget("ct_main_image_bb",
		xmBulletinBoardWidgetClass,
		ct_main_image_scrollwin,
		args,
		argcnt);

/*
    Add an event handler to detect pointer motion and key release events in
    the main image window.
    XtAddEventHandler(ct_main_image_bb,PointerMotionMask|KeyReleaseMask,
	FALSE,cursor_pointer,NULL);
*/

/*
    Add an event handler to detect exposure events in the main image window.
    XtAddEventHandler(ct_main_image_bb,ExposureMask,FALSE,
        graphics_expose,NULL);
*/
    XtManageChild(ct_main_image_bb);

/*
    Retrieve the widget IDs of the scroll bars on the main image window.
*/
    XtSetArg(args[0],XmNhorizontalScrollBar,&ct_main_hor_scrollbar);
    XtSetArg(args[1],XmNverticalScrollBar,&ct_main_vert_scrollbar);
    XtGetValues(ct_main_image_scrollwin,args,2);

    argcnt=0;
    if (visual_info->visual->class == TrueColor)
    {
	unsigned long	pixel;

	pixel = 127 | 127 << 8 | 127 << 16;
	XtSetArg(args[argcnt], XmNforeground, pixel); argcnt++;
	XtSetArg(args[argcnt], XmNbackground, pixel); argcnt++;
	XtSetValues(ct_main_hor_scrollbar, args, argcnt);
	XtSetValues(ct_main_vert_scrollbar, args, argcnt);

    }
/*
    Turn traversal off on the scrollbars.  If this is not done, the single-
    stepping of the cursor with the arrow keys does not work.
*/
    XtSetArg(args[0],XmNtraversalOn,False);
    XtSetValues(ct_main_hor_scrollbar,args,1);
    XtSetValues(ct_main_vert_scrollbar,args,1);

/*
    Determine the width, shadow thickness, border width and highlight
    thickness of each of the scroll bars and the shadow thickness and 
    border width of the scrolled window.  These values are used to 
    calculate the maximum size of the image window.
*/
    XtSetArg(args[0],XmNwidth,&width);
    XtSetArg(args[1],XmNshadowThickness,&shadow);
    XtSetArg(args[2],XmNborderWidth,&border);
    XtSetArg(args[3],XmNhighlightThickness,&hilight);
    XtGetValues(ct_main_vert_scrollbar,args,4);

    XtSetArg(args[0],XmNheight,&height);
    XtSetArg(args[1],XmNshadowThickness,&shadow);
    XtSetArg(args[2],XmNborderWidth,&border);
    XtSetArg(args[3],XmNhighlightThickness,&hilight);
    XtGetValues(ct_main_hor_scrollbar,args,4);

    XtSetArg(args[0],XmNshadowThickness,&shadow);
    XtSetArg(args[1],XmNborderWidth,&border);
    XtGetValues(ct_main_image_scrollwin,args,2);

    XtSetArg(args[0],XmNshadowThickness,&shadow);
    XtSetArg(args[1],XmNborderWidth,&border);
    XtGetValues(ct_main_image_bb,args,2);

    return( retval );
}


void 
#ifdef _NO_PROTO
ct_remap_image (w, client, call)
Widget w;
XtPointer client;
XtPointer call;
#else
ct_remap_image (Widget w, XtPointer client, XtPointer call)
#endif
{
    Arg     args[256];              /* X resource arguments.                */
    long    argcnt;

    int status, i;
    Window imgwin;
    XVisualInfo visual_template, *visual_info;
    unsigned long	visual_mask=0, number_visuals, max_cmap_size=0;

unsigned long   pixelval[4096], /* Array for pixel values.                    */
                plane_masks[1], /* Working plane mask.                        */
                wk_pixels[1],   /* Return pixel value array.                  */
                red_mask,       /* Mask values to reference 24-bit pixels.    */
                green_mask,
                blue_mask;


    visual_info = xid.visual_info;

  if (CT_Img_Visual == NULL)
  {
    visual_template.screen = xid.screen;
    if (xid.visual_info[0].visual->class == PseudoColor)
    {
	visual_template.class = TrueColor;
	visual_template.depth = 24;
    }
    else
    {
	visual_template.class = PseudoColor;
	visual_template.depth = 8;
    }
    visual_mask = VisualScreenMask;     /* Limit to present screen.     */
    visual_mask |= VisualDepthMask;     /* Limit to Depth assigned.     */
    visual_mask |= VisualClassMask;     /* Limit to class assigned.     */

    number_visuals = 0;
    visual_info = XGetVisualInfo(xid.display,visual_mask,&visual_template,
                                        &number_visuals);
    if (number_visuals == 0)
    {

	ct_displaywarning("Visual not available on this display");

	return;

    }
    for (i=0; i < number_visuals; i++)
    {
        if (visual_info[i].colormap_size > max_cmap_size)
        {
            max_cmap_size = visual_info[i].colormap_size;
	    CT_Img_Visual = &(visual_info[i]);
	}
    }

    visual_info = CT_Img_Visual;

    if (CT_Img_CMap == 0 )
    {
	XColor qcolor[256];
        CT_Img_CMap = XCreateColormap(xid.display,
                            RootWindow(xid.display,xid.screen),
                            CT_Img_Visual,AllocNone);
	if (CT_Img_Visual->visual->class == PseudoColor)
	{

	    status = XAllocColorCells(xid.display,CT_Img_CMap,True, plane_masks,
                                0,pixelval,max_cmap_size);
	    for (i=0; i < 56; i++)
	    {
		qcolor[i].pixel = i;
	    }
	    XQueryColors (xid.display, *xid.colormap_ptr, qcolor, 56);
	    for (i=0; i < 56; i++)
	    {
		qcolor[i].pixel = pixelval[i];
	    }
	    XStoreColors (xid.display, CT_Img_CMap, qcolor, 56);
	}
	XtSetSensitive (ct_stretch_color_sat, True);
	XtSetSensitive (ct_show_map_button, True);
    }

/*
 *  Create the main image window shell.
 */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "X Color Tool - Image Window"); argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "Image Window"); argcnt++;
    XtSetArg(args[argcnt], XmNkeyboardFocusPolicy, XmPOINTER); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,CT_Img_Visual->visual); argcnt++;
    XtSetArg(args[argcnt],XtNdepth,CT_Img_Visual->depth); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,CT_Img_CMap); argcnt++;
    ct_Shell_MainWin = XtCreatePopupShell("ct_shell_mainwin",
        topLevelShellWidgetClass, AppShell, args, argcnt);
    XtAddCallback(ct_Shell_MainWin, XmNdestroyCallback, main_image_destroy, 0);

    ct_main_image_scrollwin = 
	ct_Createmain_image_scrollwin(ct_Shell_MainWin, CT_Img_Visual);
    XtManageChild(ct_main_image_scrollwin);
    XtPopup(ct_Shell_MainWin,XtGrabNone);

  }
  else
  {
      visual_info = CT_Img_Visual;
      XtManageChild(ct_main_image_scrollwin);
      XtPopup(ct_Shell_MainWin,XtGrabNone);
  }

  /*  Resize the image viewing area to the size of the image	*/
  XtSetArg(args[0],XmNwidth,
	xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NS]);
  XtSetArg(args[1],XmNheight,
	xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NL]);
  XtSetArg(args[2],XmNmaxWidth,
	xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NS]);
  XtSetArg(args[3],XmNmaxHeight,
	xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NL]);
  XtSetValues(ct_main_image_bb,args,4);
  XFlush(xid.display);
  XtResizeWidget(ct_main_image_bb,
	xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NS],
	xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NL],0);


    
    ct_displayinfo ("Displaying image, please wait...");
    imgwin = XtWindow(ct_main_image_bb);
    status = ct_load_display(&xid_load.totbnd,
		imgwin,
		ct_Shell_MainWin,
		ct_main_image_bb,
		&xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NL],
	        &xid_planes[xid_view.rgbplanes[RED] - 1].dspwin[NS],
		&xid_planes[xid_view.rgbplanes[RED] - 1].maxval,
		xid_planes[xid_view.rgbplanes[RED] - 1].image_data,
		xid_planes[xid_view.rgbplanes[GREEN] - 1].image_data,
		xid_planes[xid_view.rgbplanes[BLUE] - 1].image_data,
		TRUE, NUM_IMGCOLORS, CT_Img_CMap ,visual_info, &CT_Img_Pixmap,
		TRUE,TRUE,FALSE);
    ct_undisplayinfo();

}

