/*
 * histNormal.c 1.2 Date created 6/4/93 Date used    7/20/93
 */
static char     sccsData[] = "@(#)histNormal.c 1.2 6/4/93 DSRC_SCQC";

#define MAIN

#include "scqcdec.h"
#include <math.h>

#ifdef TESTING
usage(prog)
	char           *prog;
{
	static char    *helpmsg[] = {
		"Function: Histogram Normalization of RGB image\n",
		"Options:\n",
		"	[-r mean stdev]	 red plane normal density curve\n",
		"	[-g mean stdev]	 green plane normal density curve\n",
		"	[-b mean stdev]	 blue plane normal density curve\n",
		NULL
	};
	char          **p = helpmsg;

	fprintf(stderr, "Usage: %s {Options} [file]\n", prog);
	while (*p)
		(void) fputs(*p++, stderr);
	exit(-1);
}

float           mean[3] = {128., 128., 128};
float           stdev[3] = {40., 40., 40.};

main(argc, argv)
	char          **argv;
	int             argc;
{
	BIMAGE         *colorimg;
	LNIP_HEADER     lniphdr;
	char           *c, *prog = argv[0];
	char            filename[256];
	char           *imgFilename;
	int             i;


	while (--argc > 0 && (*++argv)[0] == '-')
		for (c = argv[0] + 1; *c != '\0'; c++)
			switch (*c) {
			case 'r':
				--argc;
				mean[0] = atof(*++argv);
				--argc;
				stdev[0] = atof(*++argv);
				break;
			case 'g':
				--argc;
				mean[1] = atof(*++argv);
				--argc;
				stdev[1] = atof(*++argv);
				break;
			case 'b':
				--argc;
				mean[2] = atof(*++argv);
				--argc;
				stdev[2] = atof(*++argv);
				break;
			case 'v':
				verbose++;
				break;
			case 'h':
				usage(prog);
				break;
			}
	if (argc == 0) {
		colorimg = read_lasbimgstr(stdin, &lniphdr);
		if (lniphdr.depth != 3)
			die("read_bseqstr: depth = %d, expect 3\n", lniphdr.depth);
	} else if (argc == 1) {
		imgFilename = argv[--argc];
		colorimg = read_lasbimage(imgFilename, &lniphdr);
		if (lniphdr.depth != 3)
			die("read_bimgseq: depth = %d, expect 3\n", lniphdr.depth);
	} else
		usage(argv[0]);

	for (i = 0; i < 3; i++)
		histNormalizeImg(colorimg[i], mean[i], stdev[i]);

	/* write out files */

	/* as UBYTE */
	write_lasbimgstr(stdout, colorimg, lniphdr);



}
#endif /* TESTING */
float
gaussint(x)
	float           x;
{
	float           result;

	result = ((float) erf((double) x) + 1.) / 2.;
	return result;

}
#ifdef TESTING
histNormalizeImg(img, mean, stdev)
	BIMAGE          img;
	float           mean, stdev;
#else
histNormalizeImg(histptr, mean, stdev, from, to, index)
	long		*histptr;
	float           mean, stdev;
	int		*from, *to, *index;
#endif /* TESTING */
{
	int             i;
	int             norm[256], p[256], lut[256], compose();

	normalDensity(norm, 256, mean, stdev);
	makeInverse(norm, 256);

#ifdef  TESTING
	histogram(p, img);
#else
	/*  Histogram already performed in xid */
	for (i=0; i< 256; i++)
	    p[i] = histptr[i];
#endif
	integrateAndScale(p, 256, 256);

	compose(lut, norm, p, 256);
#ifdef TESTING
	applyLut(lut, img);
#else
	estimatePWL(lut, from, to, index);
#endif


}
integrateAndScale(arr, n, hmax)
	int            *arr, n, hmax;
{
	int             i;

	for (i = 1; i < n; i++)
		arr[i] = arr[i] + arr[i - 1];

	bytscl(arr, hmax);

}
#ifdef TESTING
applyLut(lut, img)
	int            *lut;
	BIMAGE          img;
{
	int             i, j;

	for (j = 0; j < img.h; j++)
		for (i = 0; i < img.w; i++)
			img.ptr[j][i] = lut[img.ptr[j][i]];

}
normInverse(arr, p, img)
	int            *arr, *p;
	BIMAGE          img;
{
	int             i, j;

	for (j = 0; j < img.h; j++)
		for (i = 0; i < img.w; i++)
			img.ptr[j][i] = arr[p[img.ptr[j][i]]];

}
histogram(p, img)
	int            *p;
	BIMAGE          img;
{
	int             i, j;

	for (i = 0; i < 256; i++)
		p[i] = 0;

	for (j = 0; j < img.h; j++)
		for (i = 0; i < img.w; i++)
			p[img.ptr[j][i]]++;

}
#endif /* TESTING */

fbytscl(arr, n)
	float          *arr;
	int             n;
{
	int             i;
	float           hmax = arr[0], hmin = arr[0];
	float           scale;

	for (i = 0; i < n; i++) {
		hmax = MAX(arr[i], hmax);
		hmin = MIN(arr[i], hmin);
	}

	scale = ((hmax - hmin) > 0. ? (255. / (hmax - hmin)) : 1.);

	for (i = 0; i < n; i++) {
		arr[i] = (((float) arr[i] - hmin) * scale);

	}
}
bytscl(arr, n)
	int            *arr;
	int             n;
{
	int             i;
	int             hmax = arr[0], hmin = arr[0];
	float           scale;

	for (i = 0; i < n; i++) {
		hmax = MAX(arr[i], hmax);
		hmin = MIN(arr[i], hmin);
	}

	scale = ((hmax - hmin) > 0. ? (255. / (hmax - hmin)) : 1.);

	for (i = 0; i < n; i++) {
#ifndef sgi
		arr[i] = irint((((float) arr[i] - hmin) * scale));

#else
		double tdouble;

		tdouble = (((float) arr[i] - hmin) * scale);
		tdouble = rint (tdouble);
		arr[i] = (int) tdouble;
#endif
	}
}
normalDensity(norm, n, mean, stdev)
	int            *norm;
	int             n;
	float           mean, stdev;
{
	int             i;
	float          *fnorm, gaussint();

	fnorm = (float *) malloc(n * sizeof(float));

	for (i = 0; i < 256; i++)
		fnorm[i] = gaussint(((float) i - mean) / stdev);

	fbytscl(fnorm, 256);
	for (i = 0; i < 256; i++)
#ifndef sgi
		norm[i] = irint(fnorm[i]);
#else
	{
	    double tdouble;

	    tdouble = (double) fnorm[i];
	    norm[i] = (int) tdouble;
	}
#endif

}
makeInverse(arr, n)
	int            *arr;
	int             n;
{
	int             hmax;
	int             i, *ident, *inverse, *identity();

	hmax = arr[0];
	for (i = 0; i < n; i++)
		hmax = MAX(arr[i], hmax);

	ident = identity(hmax);

	inverse = (int *) malloc(n * sizeof(int));
	for (i = 0; i < n; i++)
		inverse[i] = 0;

	for (i = 0; i < hmax; i++)
		inverse[arr[i]] = i;

	/* fill in the zeros */
	inverse[0] = 0;
	for (i = 1; i < n - 1; i++) {
		if (inverse[i] < inverse[i - 1])
			inverse[i] = inverse[i - 1];
		arr[i] = inverse[i];
	}
}
int            *
identity(n)
	int             n;
{
	int             i, *ident;
	ident = (int *) malloc(n * sizeof(int));

	for (i = 0; i < n; i++)
		ident[i] = i;
	return ident;
}
compose(c, a, b, n)
	int            *c, *a, *b, n;
{
	int             i ;


	for (i = 0; i < n; i++)
		c[i] = a[b[i]];

}

/******************************************************************************/
/*
    estimatePWL - estimate the piece-wise linear breakpoints which would
    produce the lut specified. 
*/
/******************************************************************************/
estimatePWL(lut, from, to, index)
int *lut;
int	*from;
int	*to;
int	*index;
{
    int i, j;
    float slope, lastslope;
#define NEWPOINTMARGIN .5
#define STEP 		10

    *index = 0;
    from[*index] = 0;
    to[*index] = lut[0];
    lastslope = (lut[STEP] - lut[0]) / (float) STEP;
    i = STEP;
    printf("Slope at %d, %d = %f\n", i, lut[i], lastslope);
    for (i=(2*STEP); i< 256; i+=STEP)
    {
	slope = (lut[i] - lut[i-STEP]) / (float) STEP;
	if (ABS(lastslope - slope) > NEWPOINTMARGIN)
	{
	    if (from[*index] != (i-STEP))
	    {
		(*index)++;
		from[*index] = i-STEP;
		to[*index] = lut[i-STEP];
		printf("Transition point at %d, %d \n", i-STEP, lut[i-STEP]);
	    }

	    (*index)++;
	    from[*index] = i;
	    to[*index] = lut[i];
	    printf("New point at %d, %d slope = %f\n", i, lut[i], slope);
	    lastslope = slope;
	}
	else
	    printf("Slope at %d, %d = %f\n", i, lut[i], slope);

    }
    (*index)++;
    from[*index] = 255;
    to [*index] = lut[255];

}

