/*
 *  This is the cursor function source file for the cursor panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <math.h>
#include "xid.h"
#include "proj.h"
#include "../xid_widgets/Context.h"

void
cursor_am(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    selected automatic or manual mode.  It then adds the appropriate event
    handler based on that mode.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
   Set the appropriate flags.
*/
xid_cursor.flags = xid_cursor.flags & ~(CURSOR_MAN | CURSOR_AUTO);
if (*client == 'A')
    xid_cursor.flags = xid_cursor.flags | CURSOR_AUTO;
else
    xid_cursor.flags = xid_cursor.flags | CURSOR_MAN;

/*
   Add the appropriate event handlers for the mode selected.
*/
if (xid_cursor.flags & CURSOR_MAN)
    {
    XtRemoveEventHandler(main_image_bulletinboard,
	ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,
	cursor_track,NULL);
    XtRemoveEventHandler(fullres_win_bulletinboard,
	ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,
	cursor_track,NULL);
    XtRemoveEventHandler(zoom_win_bulletinboard,
	ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,
	cursor_track,NULL);
/*
    Add and make the appropriate context active.
*/
    xid_contextadd(CON_IMAGE2);
    XeContextSetActive((XeContextWidget)context_mouse,
	context_names[CON_IMAGE2]);
    }
else
    {
/*
    Make the "automatic cursor" context active.
*/
    XeContextSetActive((XeContextWidget)context_mouse,context_names[CON_IMAGE]);
/*
    Delete the "manual cursor" context.
*/
    xid_contextdel(CON_IMAGE2);
    XtAddEventHandler(main_image_bulletinboard,
	ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,
	cursor_track,NULL);
    XtAddEventHandler(fullres_win_bulletinboard,
	ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,
	cursor_track,NULL);
    XtAddEventHandler(zoom_win_bulletinboard,
	ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,
	cursor_track,NULL);
    }

}

void
cursor_create(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine adds the appropriate event handler and sets the appropriate
    bit flags based on default values.  It also calls the cursor setup
    routine which defines the appropriate pixmaps for displaying the cursor
    information.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
XEvent	*event;		/* X Event structure.				      */

long	count,		/* Number of active planes.			      */
	i;		/* Looping index.				      */
			   
void	cursor_setup();	/* Function to allocate pixmaps and GC for cursoring. */
			   
event = (XEvent *)call;
if ((event->type == MapNotify) && !(xid_cursor.flags & CURSOR_ACTIVE))
    {
/*
    Add the cursor tracking event handlers.
*/ 
    XtAddEventHandler(main_image_bulletinboard,ButtonReleaseMask|
	PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
    XtAddEventHandler(zoom_win_bulletinboard,ButtonReleaseMask|
	PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
    XtAddEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
	PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
    cursor_setup();
/*
    Determine whether or not Z cursoring is a valid option.
*/
    for (i=0, count=0; i < xid.max_plane; i++)
	if (xid_planes[i].flags & USED)
	    count++;
    if (count < 2)
	XtSetSensitive(cursor_z_button,False);

/*
    Set the appropriate flags.
*/
    xid_cursor.flags |= CURSOR_ACTIVE;
    }
}

void
cursor_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine closes the cursor window, sets the main menu option to
    sensitive, resets bit flags and removes the appropriate event handler.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
   Close the cursor window and set the cursor button on the main panel
   to sensitive.
*/
XtUnmanageChild(Cursor_bulletinboard);
XtPopdown(Shell_Cursor);
XtSetSensitive(main_cursor_button,True);

/*
    Reset the active flag and remove the appropriate event handler.
*/
xid_cursor.flags = xid_cursor.flags ^ CURSOR_ACTIVE;
if (xid_cursor.flags & CURSOR_AUTO)
    {
    XtRemoveEventHandler(main_image_bulletinboard,ButtonReleaseMask|
	PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
    XtRemoveEventHandler(zoom_win_bulletinboard,ButtonReleaseMask|
	PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
    XtRemoveEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
	PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
    }
else
    {
/*
    Delete the "manual cursor" context.
*/
    xid_contextdel(CON_IMAGE2);
    xid_cursor.flags = xid_cursor.flags & ~(CURSOR_MAN | CURSOR_AUTO);
    xid_cursor.flags = xid_cursor.flags | CURSOR_AUTO;
    XmToggleButtonSetState(cursor_auto_button,True,True);
    }

/*
    Free resources no longer needed.
*/
if (xid_cursor.info_pixmap != NULL)
    XFreePixmap(xid.display,xid_cursor.info_pixmap);
if (xid_cursor.clear_pixmap != NULL)
    XFreePixmap(xid.display,xid_cursor.clear_pixmap);
XFreeGC(xid.display,xid_cursor.gc);

/*
    Close the ascii file containing selected coordinates.
*/
if (xid_cursor.fp != NULL)
    {
    fclose(xid_cursor.fp);
    xid_cursor.fp = NULL;
    }
}

void
cursor_latlong(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    chosen to have latitude/longitude values displayed.  If the toggle
    has been unset, the corresponding text value is set to null.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
Arg     args[1];        /* Variable used to set widget resources.             */

XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call;

/*
    Set the appropriate flag.
*/
xid_cursor.flags  = xid_cursor.flags ^ CURSOR_LATLONG;

/*
    If the lat/long toggle has been unset, set its corresponding text value
    to a null string.
*/
if (!(acs->set))
    {
    XtSetArg(args[0],XmNvalue,"");
    XtSetValues(cursor_ll_text,args,1);
    }
}

void
cursor_mo(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    selected original or mapped gray level values to be displayed.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
   Set the appropriate flag.
*/
xid_cursor.flags = xid_cursor.flags & ~(CURSOR_ORIG | CURSOR_MAPPED);
if (*client == 'O')
    xid_cursor.flags = xid_cursor.flags | CURSOR_ORIG;
else
    xid_cursor.flags = xid_cursor.flags | CURSOR_MAPPED;

}

void
cursor_proj(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    chosen to have projection coordinates displayed.  If the toggle has been 
    unset, the corresponding text value is set to null.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
Arg     args[1];        /* Variable used to set widget resources.             */

XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call;

/*
   Set the appropriate flag.
*/
xid_cursor.flags  = xid_cursor.flags ^ CURSOR_PROJ;

/*
    If the projection coordinate toggle has been unset, set its corresponding 
    text value to a null string.
*/
if (!(acs->set))
    {
    XtSetArg(args[0],XmNvalue,"");
    XtSetValues(cursor_pc_text,args,1);
    }
}

void
cursor_unscaled(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    chosen to have unscaled pixel values displayed.  If the toggle has been 
    unset, the corresponding text value is set to null.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
Arg     args[1];        /* Variable used to set widget resources.             */

XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call;

/*
    Set the appropriate flag.
*/
xid_cursor.flags  = xid_cursor.flags ^ CURSOR_UNSCAL;

/*
    If the unscaled  coordinate toggle has been unset, set its corresponding
    text value to a null string.
*/
if (!(acs->set))
    {
    XtSetArg(args[0],XmNvalue,"");
    XtSetValues(cursor_un_text,args,1);
    }
}

void
cursor_xyz(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    selected to cursor in the XY dimension or the Z dimension.  If the
    Z dimension has been selected, the scale widget is made sensitive and
    its maximum value is set to the number of planes that can be cursored
    through.  The cursor mode is also set to manual and the appropriate
    event handler is added.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
Arg     args[1];        /* Variable used to set widget resources.             */

long	i;		/* Looping index.				      */

/*
    If the Z dimension has been chosen, determine the number of valid planes.
*/
if (*client == 'Z')
    {
    for (i=0; i < xid.max_plane; i++)
	{
	if (xid_planes[i].flags & USED)
	    xid_cursor.max_zplane = i + 1;
	}
    }

/*
    Set the appropriate flags.
*/
xid_cursor.flags = xid_cursor.flags & ~(CURSOR_XY | CURSOR_Z);
if (*client == 'Z')
    xid_cursor.flags = xid_cursor.flags | CURSOR_Z;
else
    xid_cursor.flags = xid_cursor.flags | CURSOR_XY;

if (*client == 'Z')
    {
/*
    Allow the scale widget to be sensitive and set its maximum value.
*/
    XtSetSensitive(cursor_scale,True);
    XtSetArg(args[0],XmNmaximum,xid_cursor.max_zplane);
    XtSetValues(cursor_scale,args,1);
    xid_cursor.zplane = 1;

/*
    If the cursor mode is currently automatic, reset it to manual and add
    the appropriate event handler.
*/
    if (xid_cursor.flags & CURSOR_AUTO)
	{
	XtSetArg(args[0],XmNset,False);
	XtSetValues(cursor_auto_button,args,1);
	XtSetArg(args[0],XmNset,True);
	XtSetValues(cursor_man_button,args,1);
	xid_cursor.flags  = xid_cursor.flags ^ (CURSOR_AUTO | CURSOR_MAN); 
	XtRemoveEventHandler(main_image_bulletinboard,ButtonReleaseMask|
	    PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
	XtAddEventHandler(main_image_bulletinboard,
	    ButtonReleaseMask|KeyReleaseMask,FALSE,cursor_track,NULL);
	}
    }
else
/*
    Do not allow the scale to be sensitive if cursoring in the XY dimension.
*/
    XtSetSensitive(cursor_scale,False);
}

void
cursor_zslider(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine detects changes in the value of the scale used in cursoring
    in the Z dimension.  If the value has changed (i.e. the user has selected 
    a new plane), send a synthetic event forcing the values to be updated.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
XButtonReleasedEvent	button_event;

XmScaleCallbackStruct *acs=(XmScaleCallbackStruct*)call;

/*
    If the value of the plane to report values for has changed (i.e. the 
    scale has changed value), retrieve the XY coordinate and send a button 
    release event to the image window.  This forces the cursor information 
    to be updated.
*/
button_event.type = ButtonRelease;
button_event.display = xid.display;
button_event.window = XtWindow(main_image_bulletinboard);
button_event.x = xid.x;
button_event.y = xid.y;
if (acs->value != xid_cursor.zplane)
    {
    xid_cursor.zplane = acs->value;
    XSendEvent(xid.display,XtWindow(main_image_bulletinboard),True,
	ButtonReleaseMask,(XEvent *)&button_event);
    }
}

void
cursor_track(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine is used to calculate the specified information to be output
    to the cursor window.  Once the information is calculated, it is written
    to the pixmap and then copied to the cursor window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Modified by L. Huewe, USGS/EROS Data Cetner, 8/93
    - Fixxed a problem that occurred when viewing the unscaled values of a
      full resolution black and white image or when viewing the unscaled values
      of a zoom image which was created from a full resolution black and white 
      image.

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
{
XEvent	*event;		/* X event structure defining input event.	      */

Arg     args[1];        /* Variable used to set widget resources.     	      */

Window	cursor_win;	/* Cursor window ID.				      */

/**** DROP ****/
KeyCode		a_key,
		s_key;

static	long	start_line,
		end_line,
		num_lines,
		num_blocks,
		nlines;
/**** DROP ****/
static	long	gctp_init = FALSE;
			/* Indicates whether or not GCTP has been initialized.*/

static	char	prev_name[CMLEN] = {""},
			/* Filename of image last processed.		      */
		proj_name[CMLEN];
			/* Projection system name.			      */

double	latitude,	/* Calculated latitude value.			      */
	longitude,	/* Calculated longitude value.			      */
	red_offset,	/* Offset used to scale non-byte data.		      */
	green_offset,	/* Offset used to scale non-byte data.		      */
	blue_offset,	/* Offset used to scale non-byte data.		      */
	projx,          /* X projection coordinate.             	      */
	projy,          /* Y projection coordinate.             	      */
	r_unscale,	/* Unscaled red value.				      */
	g_unscale,	/* Unscaled green value.			      */
	b_unscale,	/* Unscaled blue value.				      */
	red_slope,	/* Slope used to scale non-byte data.		      */
	green_slope,	/* Slope used to scale non-byte data.		      */
	blue_slope,	/* Slope used to scale non-byte data.		      */
	xsub,		/* Current sample (subpixel accuracy).	      	      */
	ysub;		/* Current line (subpixel accuracy).	      	      */

float   scaler;         /* LUT scaler for actual number of colors.    	      */

long	bplane_idx,	/* Index to blue plane number of the currently
			   viewed image.				      */
	curx,		/* Current X file coordinate.			      */
	cury,		/* Current Y file coordinate.			      */
	gplane_idx,	/* Index to green plane number of the currently
			   viewed image.				      */
	i,j,k,		/* Looping indices.				      */
	icoords[8],     /* Image corner line/sample coordinates.              */
	img_idx,	/* Index into colormap of first image color.	      */
	mapval,		/* Index into color table to obtain mapped pixel 
			   value.					      */
	matrix[3][3][3],/* Matrix of current pixel values.		      */
	maxnl,		/* Maximum line value.			      	      */
	maxns,		/* Maximum sample value.		      	      */
	msgflg = FALSE, /* Specifies that error messages are not to be
		           output by the routine.                             */
	nbands,		/* Number of bands displayed.			      */
	offset,		/* Offset into image data buffer for current pixel
			   value.					      */
	pos,		/* Character position within a string.		      */
	prtprm[2],	/* Print option for the Projection Transformation
			   Package.				 	      */
	rplane_idx,	/* Index to red plane number of the currently
			   viewed image.				      */
	status,		/* Status indicator.				      */
/**** DROP ****/
	keycode,
	state;
/**** DROP ****/

char	coords[132],	/* Current X/Y file coordinate in character format.   */
/**** DROP ****/
        dropline_file[CMLEN],
	hname[CMLEN],
	mesg[80],
/**** DROP ****/
        nsstr[15],      /* North/South latitude string.          	      */
	ewstr[15],      /* East/West logitude string.           	      */
	tmpstr[CMLEN];	/* Temporary string variable.			      */

/*
    If no image is currently being viewed, print a warning message and return.
*/
if ((w == main_image_bulletinboard) && (xid.view == -1))
    {
    xid_message("No image data currently being viewed");
    return;
    }
else if ((w == zoom_win_bulletinboard) && (xid_zoom.image_pixmap == NULL)) 
    {
    xid_message("No image data currently being viewed");
    return;
    }
else if ((w == zoom_win_bulletinboard) && (xid_cursor.flags & CURSOR_Z)) 
    {
    xid_message("Unable to cursor in Z dimension on the zoom window");
    return;
    }
else if ((w == fullres_win_bulletinboard) && (xid_fullres == NULL)) 
    {
    xid_message("No image data currently being viewed");
    return;
    }
else if ((w == fullres_win_bulletinboard) && (xid_fullres != NULL)) 
    if (xid_fullres[0].image_data == NULL)
	{
        xid_message("No image data currently being viewed");
	return;
	}
else if ((w == fullres_win_bulletinboard) && (xid_cursor.flags & CURSOR_Z)) 
    {
    xid_message("Unable to cursor in Z dimension on the full resolution window");
    return;
    }

event = (XEvent *)call;

/*
    Determine the appropriate memory indices based on the type of
    cursoring (XY dimension or Z dimension).
*/
if (xid_cursor.flags & CURSOR_XY)
    {
    rplane_idx = xid_view.rgbplanes[RED] - 1;
    gplane_idx = xid_view.rgbplanes[GREEN] - 1;
    bplane_idx = xid_view.rgbplanes[BLUE] - 1;
    }
else
    {
    rplane_idx = xid_cursor.zplane - 1;
    for (i=0; i < xid.max_plane; i++)
        {
    	if (!(xid_planes[rplane_idx].flags & USED)) 
            rplane_idx = ((rplane_idx + 1) > xid_cursor.max_zplane) ? 1 : 
		rplane_idx + 1;
    	else
	    break;
        }
    gplane_idx = bplane_idx = rplane_idx; 
    }

/*
    Determine the number of bands displayed.
*/
if ((rplane_idx == gplane_idx) && (gplane_idx == bplane_idx))
    nbands = 1;
else
    nbands = 3;

/*
    Determine the maximum number of lines and samples in the window.
*/
if (w == main_image_bulletinboard)
    {
    maxnl = xid_planes[rplane_idx].dspwin[NL] - 1;
    maxns = xid_planes[rplane_idx].dspwin[NS] - 1;
    }
else if (w == zoom_win_bulletinboard)
    {
    maxnl = xid_zoom.parms.factor * xid_zoom.window[2];
    maxns = xid_zoom.parms.factor * xid_zoom.window[3];
    }
else if (w == fullres_win_bulletinboard)
    {
    maxnl = xid_fullres[0].dspwin[NL] - 1;
    maxns = xid_fullres[0].dspwin[NS] - 1;
    }

/*
    Calculate the scaler needed to scale values to the appropriate range.
*/
if ((*xid.colormap_ptr == xid.colormap) && (nbands == 1) && (xid_planes[
    xid_view.rgbplanes[RED]-1].maxval < NUM_IMGCOLORS))
    {
    scaler = 1.0;
    img_idx = IMG_IDX;
    }
else if (*xid.colormap_ptr == xid.colormap)
    {
    scaler = (256.0 / (float)NUM_IMGCOLORS);           /* (2^8 / # of colors) */
    img_idx = IMG_IDX;
    }
else
    {
    scaler = 1.0;
    img_idx = 0;
    }

/*
    Can projection information be calculated if requested?
*/
xid_cursor.proj_valid  = xid_planes[rplane_idx].proj_valid;
xid_cursor.corner_valid = xid_planes[rplane_idx].corner_valid;

/*
    Do new coefficients need to be calculated?
*/
if ((strcmp(xid_planes[rplane_idx].filename,prev_name) != 0) && 
    ((xid_cursor.flags & CURSOR_PROJ) || (xid_cursor.flags & CURSOR_LATLONG)) 
    && (xid_planes[rplane_idx].corner_valid))
    {
/*
    Find the transformation coefficients mapping image coordinates to
    projection coordinates.
*/
    icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
    icoords[2] = icoords[6] = xid_planes[rplane_idx].image_info->nl;
    icoords[5] = icoords[7] = xid_planes[rplane_idx].image_info->ns;

    status = c_rtcoef(xid_planes[rplane_idx].image_info->upleft,icoords,
	xid_cursor.coef,&msgflg);
/*
    If the coefficients can't be calculated then set the validity flag to false.
*/
    if (status != E_SUCC)
	xid_cursor.corner_valid = xid_cursor.proj_valid = FALSE;
    else
	xid_cursor.corner_valid = TRUE;
    }

/*
    Does the geographic transformation package need to be reinitialized?
*/
if (((strcmp(xid_planes[rplane_idx].filename,prev_name) != 0) || (!gctp_init)) 
    && (xid_cursor.flags & CURSOR_LATLONG) && (xid_planes[rplane_idx].
    proj_valid))
    {
/*
    Initialize the Projection Tranformation Package and verify the
    unit code.
*/
    if (xid_cursor.proj_valid)
        {
        status = c_ckunit(xid_planes[rplane_idx].image_info->proj_units,
	    &xid_cursor.outunit);
	if (status != E_SUCC)
	    xid_cursor.proj_valid = FALSE;
	}

/*
    Initialize the geographic transformation package.
*/
    if (xid_cursor.proj_valid)
	{
	xid_cursor.geo = 0;
	xid_cursor.outdatum = CLARKE;
	xid_cursor.outproj = DEGREES;
	xid_cursor.outzone = NULZONE;
	prtprm[0] = prtprm[1] = FALSE;
/*
    Force the initialization of the GCTP routines (c_transinit is not enough).
*/
	if (xid_planes[rplane_idx].image_info->proj_code != GEO)
	    status = proj_flush(xid_planes[rplane_idx].image_info->proj_code,
		xid_cursor.outunit,xid_planes[rplane_idx].image_info->
		zone_code,xid_planes[rplane_idx].image_info->datum_code,
		xid_planes[rplane_idx].image_info->proj_coef);
	else
	    status = XID_S;

	if (status != XID_S)
	    xid_cursor.proj_valid = FALSE;
	else
	    {
	    status = c_transinit(&xid_planes[rplane_idx].image_info->proj_code,
		&xid_cursor.outunit,&xid_planes[rplane_idx].image_info->
		zone_code,&xid_planes[rplane_idx].image_info->datum_code,
		xid_planes[rplane_idx].image_info->proj_coef,&xid_cursor.geo,
		&xid_cursor.outproj,&xid_cursor.outzone,&xid_cursor.outdatum,
		xid_cursor.projparm,prtprm,"");
	    if (status != E_SUCC)
		xid_cursor.proj_valid = FALSE;
	    }	
	}
    if (xid_cursor.proj_valid)
	gctp_init = TRUE;
    }

if ((xid_cursor.flags & CURSOR_PROJ) || (xid_cursor.flags & CURSOR_LATLONG))
    {
/*
    Determine the projection name based on the projection code.
*/
    if (xid_cursor.proj_valid)
	{
	c_prostr(&xid_planes[rplane_idx].image_info->proj_code,tmpstr);
	pos = strcspn(tmpstr,")");
	strcpy(proj_name,&tmpstr[pos+1]);
	}
    strcpy(prev_name,xid_planes[rplane_idx].filename);
    }

/*
    Remove all other MotionNotify events from queue -- this makes for
    smoother scrolling by helping to limit the number of events
    generated while we're acting on the present one.
*/
if (!(event->xmotion.send_event))
    while (XCheckTypedEvent(xid.display,MotionNotify,event));

/*
    Check the validity of the cursor coordinates.
*/
if ((event->xmotion.x < 0) || (event->xmotion.y < 0) || (event->xmotion.x > 
  maxns) || (event->xmotion.y > maxnl))
    return;

/*
    Get the new cursor coordinates.
*/
xid.x = event->xmotion.x;
xid.y = event->xmotion.y;

/*
    Set the cursor window ID.
*/
cursor_win = XtWindow(cursor_drawingarea);

/*
    Clear the cursor pixmap for the new information.
*/
XCopyArea(xid.display,xid_cursor.clear_pixmap,xid_cursor.info_pixmap,
    xid_cursor.gc,0,0,CURSOR_WIDTH,CURSOR_HEIGHT,0,0);

/*
    Cursoring in the main image window.
*/
if (w == main_image_bulletinboard)
    {
/*
    Convert the coordinates to (one-relative) file coordinates.
*/
    curx = (((event->xmotion.x + 1) * xid_planes[rplane_idx].skips) +
        xid_planes[rplane_idx].window[SS] - 1) - (xid_planes[rplane_idx].skips
	- 1);
    cury = (((event->xmotion.y + 1) * xid_planes[rplane_idx].skipl) +
        xid_planes[rplane_idx].window[SL] - 1) - (xid_planes[rplane_idx].skipl
	- 1);
/*
    Set the image and file coordinate strings.
*/
    sprintf(coords,"%d   %d",event->xmotion.y + 1,event->xmotion.x + 1);
    XtSetArg(args[0],XmNvalue,coords);
    XtSetValues(cursor_ls_text,args,1);

    sprintf(coords,"%d   %d",cury,curx);
    XtSetArg(args[0],XmNvalue,coords);
    XtSetValues(cursor_fc_text,args,1);

    xsub = (double)curx;
    ysub = (double)cury;
    }
/*
    Cursoring in the zoomed window.
*/
else if (w == zoom_win_bulletinboard)
    {
/*
    Calculate the subpixel file coordinates.
*/
    if (xid_zoom.flags & ZOOM_MAIN)
        cursor_sub(event->xmotion.x,event->xmotion.y,&xsub,&ysub,
	    xid_planes[rplane_idx].skipl,xid_planes[rplane_idx].skips,
	    xid_planes[rplane_idx].window[SL],
	    xid_planes[rplane_idx].window[SS]);
    else if (xid_zoom.flags & ZOOM_FULLRES)
        cursor_sub(event->xmotion.x,event->xmotion.y,&xsub,&ysub,1,1,
	    xid_fullres[0].window[SL],xid_fullres[0].window[SS]);
/*
    Set the image and file coordinate strings.
*/
    sprintf(coords,"%d   %d",((int)(event->xmotion.y / xid_zoom.factor) + 1),
	((int)(event->xmotion.x /xid_zoom.factor) + 1));
    XtSetArg(args[0],XmNvalue,coords);
    XtSetValues(cursor_ls_text,args,1);

    sprintf(coords,"%.4lf   %.4lf",ysub,xsub);
    XtSetArg(args[0],XmNvalue,coords);
    XtSetValues(cursor_fc_text,args,1);
    }
/*
    Cursoring in the full resolution window.
*/
else if (w == fullres_win_bulletinboard)
    {
/*
    Calculate the full resolution file coordinate.
*/
    curx = xid_fullres[0].window[SS] + event->xmotion.x;
    cury = xid_fullres[0].window[SL] + event->xmotion.y;
/*
    Set the image and file coordinate strings.
*/
    sprintf(coords,"%d   %d",(event->xmotion.y  + 1),(event->xmotion.x + 1));
    XtSetArg(args[0],XmNvalue,coords);
    XtSetValues(cursor_ls_text,args,1);

    sprintf(coords,"%d   %d",cury,curx);
    XtSetArg(args[0],XmNvalue,coords);
    XtSetValues(cursor_fc_text,args,1);

    xsub = (double)curx;
    ysub = (double)cury;
    }

/*
    Calculate the projection coordinates.
*/
if ((xid_cursor.flags & CURSOR_PROJ) || (xid_cursor.flags & CURSOR_LATLONG)) 
    {
    if (xid_cursor.corner_valid)
        {
/* 
    Calculate projection coordinates from the line/sample coordinate.
*/
        projy = (double)(xid_cursor.coef[0] * ysub) + (xid_cursor.coef[1] * 
	    xsub) + xid_cursor.coef[2];
        projx = (double)(xid_cursor.coef[3] * ysub) + (xid_cursor.coef[4] * 
	    xsub) + xid_cursor.coef[5];
        sprintf(coords,"Y: %12.3f X: %12.3f %s %s",projy,projx,proj_name,
	    xid_planes[rplane_idx].image_info->proj_units);
        }
    else
        strcpy(coords,"Invalid corner coordinate information");

    if (xid_cursor.flags & CURSOR_PROJ)
	{
/*
    Write out the projection coordinates calculated.
*/
        XtSetArg(args[0],XmNvalue,coords);
        XtSetValues(cursor_pc_text,args,1);
	}
    }

/*
    Calculate the latitude/longitude coordinates.
*/
if (xid_cursor.flags & CURSOR_LATLONG)
    {
    if ((xid_cursor.proj_valid) && (xid_cursor.corner_valid))
	{
/*
    Calculate latitude/longitude coordinates from the projection coordinates.
*/
	status = c_trans(&xid_planes[rplane_idx].image_info->proj_code,
	    &xid_cursor.outunit, &xid_cursor.geo, &xid_cursor.outproj,
	    &projx,&projy,&longitude,&latitude);
	if (status == IN_BREAK)
	    {
	    strcpy(coords,"In break area");
	    if (xid_cursor.flags & CURSOR_PROJ)
		{
		XtSetArg(args[0],XmNvalue,"In break area");
		XtSetValues(cursor_pc_text,args,1);
		}
	    }
	else if (status != E_SUCC)
	    strcpy(coords,"Error calculating latitude/longitude values");
	else
	    {
/* 
    Determine the appropriate latitude/longitude labels.
*/
	    if (latitude > 0)
		strcpy(nsstr,"North");
	    else if (latitude < 0)
		strcpy(nsstr,"South");
	    else
		strcpy(nsstr,"Equator");

	    if (longitude > 0)
		strcpy(ewstr,"East");
	    else if (longitude < 0)
		strcpy(ewstr,"West");
	    else
		strcpy(ewstr,"Prime Meridian");

	    latitude = fabs(latitude);
	    longitude = fabs(longitude);

	    sprintf(coords,"%12.5f %s %12.5f %s",latitude,nsstr,longitude,
		ewstr);
	    }
	}
    else
	strcpy(coords,"Invalid projection information");
/*
    Write out the latitude/longitude coordinates calculated.
*/
    XtSetArg(args[0],XmNvalue,coords);
    XtSetValues(cursor_ll_text,args,1);
    }

/*
    Initialize the matrix of gray level values.
*/
for (i=0; i < 3; i++)
    {
    matrix[RED][0][i] = matrix[RED][1][i] = matrix[RED][2][i] = 0;
    matrix[GREEN][0][i] = matrix[GREEN][1][i] = matrix[GREEN][2][i] = 0;
    matrix[BLUE][0][i] = matrix[BLUE][1][i] = matrix[BLUE][2][i] = 0;
    }

if (event->xmotion.y == 0)
    {
    matrix[RED][0][0] = matrix[RED][0][1] = matrix[RED][0][2] = -1;
    matrix[GREEN][0][0] = matrix[GREEN][0][1] = matrix[GREEN][0][2] = -1;
    matrix[BLUE][0][0] = matrix[BLUE][0][1] = matrix[BLUE][0][2] = -1;
    }
else if (event->xmotion.y == maxnl)
    {
    matrix[RED][2][0] = matrix[RED][2][1] = matrix[RED][2][2] = -1;
    matrix[GREEN][2][0] = matrix[GREEN][2][1] = matrix[GREEN][2][2] = -1;
    matrix[BLUE][2][0] = matrix[BLUE][2][1] = matrix[BLUE][2][2] = -1;
    }

if (event->xmotion.x == 0)
    {
    matrix[RED][0][0] = matrix[RED][1][0] = matrix[RED][2][0] = -1;
    matrix[GREEN][0][0] = matrix[GREEN][1][0] = matrix[GREEN][2][0] = -1;
    matrix[BLUE][0][0] = matrix[BLUE][1][0] = matrix[BLUE][2][0] = -1;
    }
else if (event->xmotion.x == maxns)
    {
    matrix[RED][0][2] = matrix[RED][1][2] = matrix[RED][2][2] = -1;
    matrix[GREEN][0][2] = matrix[GREEN][1][2] = matrix[GREEN][2][2] = -1;
    matrix[BLUE][0][2] = matrix[BLUE][1][2] = matrix[BLUE][2][2] = -1;
    }

/*
    Set gray level values based on the main image window.
*/
if (w == main_image_bulletinboard)
    {
    for (i=0; i < 3; i++)
	for (j=0; j < 3; j++)
	    {
	    offset = ((event->xmotion.y + (i - 1)) * 
		xid_planes[rplane_idx].dspwin[NS]) + (event->xmotion.x) + 
		(j - 1);
	    if (matrix[RED][i][j] != -1)
		{
		matrix[RED][i][j] = (long) (*(xid_planes[rplane_idx].image_data
		    + offset)); 
		matrix[GREEN][i][j] = (long) (*(xid_planes[gplane_idx].
		    image_data + offset));
		matrix[BLUE][i][j] = (long) (*(xid_planes[bplane_idx].
		    image_data + offset));
		}
	    }
    }
/*
    Set gray level values based on the zoomed image window.
*/
else if (w == zoom_win_bulletinboard)
    {
    for (i=0; i < 3; i++)
	for (j=0; j < 3; j++)
	    {
	    offset = ((event->xmotion.y + (i - 1)) * maxns) + (event->xmotion.x)
		+ (j - 1);
	    for (k=0; k < nbands; k++)
		{
		if ((matrix[k][i][j] != -1) && (nbands != 1))
		    matrix[k][i][j] = (long) (*(xid_zoom.image_data + maxns *
			maxnl * k + offset)); 
		else if ((matrix[k][i][j] != -1) && (nbands == 1))
		    {
		    matrix[RED][i][j] = (long) (*(xid_zoom.image_data + 
			offset)); 
		    matrix[GREEN][i][j] = (long) (*(xid_zoom.image_data + 
			offset)); 
		    matrix[BLUE][i][j] = (long) (*(xid_zoom.image_data + 
			offset)); 
		    }
		}
	    }
    }
/*
    Set gray level values based on the full resolution image window.
*/
else if (w == fullres_win_bulletinboard)
    {
    for (i=0; i < 3; i++)
	for (j=0; j < 3; j++)
	    {
	    offset = ((event->xmotion.y + (i - 1)) * 
		xid_fullres[0].dspwin[NS]) + (event->xmotion.x) + 
		(j - 1);
	    if ((matrix[RED][i][j] != -1) && (nbands != 1))
		{
		matrix[RED][i][j] = (long) (*(xid_fullres[RED].image_data
		    + offset)); 
		matrix[GREEN][i][j] = (long) (*(xid_fullres[GREEN].image_data 
		    + offset));
		matrix[BLUE][i][j] = (long) (*(xid_fullres[BLUE].image_data 
		    + offset));
		}
	    else if ((matrix[RED][i][j] != -1) && (nbands == 1))
		{
		matrix[RED][i][j] = (long) (*(xid_fullres[RED].image_data
		    + offset)); 
		matrix[GREEN][i][j] = (long) (*(xid_fullres[RED].image_data 
		    + offset));
		matrix[BLUE][i][j] = (long) (*(xid_fullres[RED].image_data 
		    + offset));
		}
	    }
    }

/*
    Determine the unscaled gray level value.
*/
if (xid_cursor.flags & CURSOR_UNSCAL)
    {
/*
    Set the slope and offset based on the window the user is cursoring in.
*/
    if (w == main_image_bulletinboard)
        {
	red_slope = xid_planes[rplane_idx].slope;
	red_offset = xid_planes[rplane_idx].offset;
	green_slope = xid_planes[gplane_idx].slope;
	green_offset = xid_planes[gplane_idx].offset;
	blue_slope = xid_planes[bplane_idx].slope;
	blue_offset = xid_planes[bplane_idx].offset;
	}
    else if (w == fullres_win_bulletinboard)
	{
	red_slope = xid_fullres[RED].slope;
	red_offset = xid_fullres[RED].offset;
	if (xid.num_fullres > 1)
	    {
	    green_slope = xid_fullres[GREEN].slope;
	    green_offset = xid_fullres[GREEN].offset;
	    }
	else
	    {
	    green_slope = xid_fullres[RED].slope;
	    green_offset = xid_fullres[RED].offset;
	    }
	if (xid.num_fullres > 1)
	    {
	    blue_slope = xid_fullres[BLUE].slope;
	    blue_offset = xid_fullres[BLUE].offset;
	    }
	else
	    {
	    blue_slope = xid_fullres[RED].slope;
	    blue_offset = xid_fullres[RED].offset;
	    }
	}
    else if ((w == zoom_win_bulletinboard) && (xid_zoom.flags & ZOOM_MAIN))
	{
	red_slope = xid_planes[rplane_idx].slope;
	red_offset = xid_planes[rplane_idx].offset;
	green_slope = xid_planes[gplane_idx].slope;
	green_offset = xid_planes[gplane_idx].offset;
	blue_slope = xid_planes[bplane_idx].slope;
	blue_offset = xid_planes[bplane_idx].offset;
	}
    else if ((w == zoom_win_bulletinboard) && (xid_zoom.flags & ZOOM_FULLRES))
	{
	red_slope = xid_fullres[RED].slope;
	red_offset = xid_fullres[RED].offset;
	if (xid.num_fullres > 1)
	    {
	    green_slope = xid_fullres[GREEN].slope;
	    green_offset = xid_fullres[GREEN].offset;
	    }
	else
	    {
	    green_slope = xid_fullres[RED].slope;
	    green_offset = xid_fullres[RED].offset;
	    }
	if (xid.num_fullres > 1)
	    {
	    blue_slope = xid_fullres[BLUE].slope;
	    blue_offset = xid_fullres[BLUE].offset;
	    }
	else
	    {
	    blue_slope = xid_fullres[RED].slope;
	    blue_offset = xid_fullres[RED].offset;
	    }
	}
    r_unscale = ((matrix[RED][1][1] - red_offset) / red_slope);
    g_unscale = ((matrix[GREEN][1][1] - green_offset) / green_slope);
    b_unscale = ((matrix[BLUE][1][1] - blue_offset) / blue_slope);
    sprintf(coords,"%f   %f   %f",r_unscale,g_unscale,b_unscale);

/*
    Write out the unscaled coordinate values.
*/
    XtSetArg(args[0],XmNvalue,coords);
    XtSetValues(cursor_un_text,args,1);
    }

/*
    Set the foreground and background colors and write out the gray level
    values (either original or mapped).
*/
XSetForeground(xid.display,xid_cursor.gc,
    xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
XSetBackground(xid.display,xid_cursor.gc,
    xid.color_table[RGBBW_IDX + WHITE_COL].pixel);

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[RED][0][0],matrix[RED][0][1],
        matrix[RED][0][2]);
else
    {
    mapval = (long)((float)matrix[RED][0][0] / scaler + img_idx);
    matrix[RED][0][0] = xid.colortable_ptr[mapval].red >> 8;
    mapval = (long)((float)matrix[RED][0][1] / scaler + img_idx);
    matrix[RED][0][1] = xid.colortable_ptr[mapval].red >> 8;
    mapval = (long)((float)matrix[RED][0][2] / scaler + img_idx);
    matrix[RED][0][2] = xid.colortable_ptr[mapval].red >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[RED][0][0],matrix[RED][0][1],
        matrix[RED][0][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,6,20,coords,
    strlen(coords));

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[RED][1][0],matrix[RED][1][1],
        matrix[RED][1][2]);
else
    {
    mapval = (long)((float)matrix[RED][1][0] / scaler + img_idx);
    matrix[RED][1][0] = xid.colortable_ptr[mapval].red >> 8;
    mapval = (long)((float)matrix[RED][1][1] / scaler + img_idx);
    matrix[RED][1][1] = xid.colortable_ptr[mapval].red >> 8;
    mapval = (long)((float)matrix[RED][1][2] / scaler + img_idx);
    matrix[RED][1][2] = xid.colortable_ptr[mapval].red >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[RED][1][0],matrix[RED][1][1],
        matrix[RED][1][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,6,40,coords,
    strlen(coords));

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[RED][2][0],matrix[RED][2][1],
        matrix[RED][2][2]);
else
    {
    mapval = (long)((float)matrix[RED][2][0] / scaler + img_idx);
    matrix[RED][2][0] = xid.colortable_ptr[mapval].red >> 8;
    mapval = (long)((float)matrix[RED][2][1] / scaler + img_idx);
    matrix[RED][2][1] = xid.colortable_ptr[mapval].red >> 8;
    mapval = (long)((float)matrix[RED][2][2] / scaler + img_idx);
    matrix[RED][2][2] = xid.colortable_ptr[mapval].red >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[RED][2][0],matrix[RED][2][1],
        matrix[RED][2][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,6,60,coords,
    strlen(coords));

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][0][0],matrix[GREEN][0][1],
        matrix[GREEN][0][2]);
else
    {
    mapval = (long)((float)matrix[GREEN][0][0] / scaler + img_idx);
    matrix[GREEN][0][0] = xid.colortable_ptr[mapval].green >> 8;
    mapval = (long)((float)matrix[GREEN][0][1] / scaler + img_idx);
    matrix[GREEN][0][1] = xid.colortable_ptr[mapval].green >> 8;
    mapval = (long)((float)matrix[GREEN][0][2] / scaler + img_idx);
    matrix[GREEN][0][2] = xid.colortable_ptr[mapval].green >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][0][0],matrix[GREEN][0][1],
        matrix[GREEN][0][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,150,20,coords,
    strlen(coords));

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][1][0],matrix[GREEN][1][1],
        matrix[GREEN][1][2]);
else
    {
    mapval = (long)((float)matrix[GREEN][1][0] / scaler + img_idx);
    matrix[GREEN][1][0] = xid.colortable_ptr[mapval].green >> 8;
    mapval = (long)((float)matrix[GREEN][1][1] / scaler + img_idx);
    matrix[GREEN][1][1] = xid.colortable_ptr[mapval].green >> 8;
    mapval = (long)((float)matrix[GREEN][1][2] / scaler + img_idx);
    matrix[GREEN][1][2] = xid.colortable_ptr[mapval].green >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][1][0],matrix[GREEN][1][1],
        matrix[GREEN][1][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,150,40,coords,
    strlen(coords));

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][2][0],matrix[GREEN][2][1],
        matrix[GREEN][2][2]);
else
    {
    mapval = (long)((float)matrix[GREEN][2][0] / scaler + img_idx);
    matrix[GREEN][2][0] = xid.colortable_ptr[mapval].green >> 8;
    mapval = (long)((float)matrix[GREEN][2][1] / scaler + img_idx);
    matrix[GREEN][2][1] = xid.colortable_ptr[mapval].green >> 8;
    mapval = (long)((float)matrix[GREEN][2][2] / scaler + img_idx);
    matrix[GREEN][2][2] = xid.colortable_ptr[mapval].green >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][2][0],matrix[GREEN][2][1],
        matrix[GREEN][2][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,150,60,coords,
    strlen(coords));

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][0][0],matrix[BLUE][0][1],
        matrix[BLUE][0][2]);
else
    {
    mapval = (long)((float)matrix[BLUE][0][0] / scaler + img_idx);
    matrix[BLUE][0][0] = xid.colortable_ptr[mapval].blue >> 8;
    mapval = (long)((float)matrix[BLUE][0][1] / scaler + img_idx);
    matrix[BLUE][0][1] = xid.colortable_ptr[mapval].blue >> 8;
    mapval = (long)((float)matrix[BLUE][0][2] / scaler + img_idx);
    matrix[BLUE][0][2] = xid.colortable_ptr[mapval].blue >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][0][0],matrix[BLUE][0][1],
        matrix[BLUE][0][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,294,20,coords,
    strlen(coords));

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][1][0],matrix[BLUE][1][1],
        matrix[BLUE][1][2]);
else
    {
    mapval = (long)((float)matrix[BLUE][1][0] / scaler + img_idx);
    matrix[BLUE][1][0] = xid.colortable_ptr[mapval].blue >> 8;
    mapval = (long)((float)matrix[BLUE][1][1] / scaler + img_idx);
    matrix[BLUE][1][1] = xid.colortable_ptr[mapval].blue >> 8;
    mapval = (long)((float)matrix[BLUE][1][2] / scaler + img_idx);
    matrix[BLUE][1][2] = xid.colortable_ptr[mapval].blue >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][1][0],matrix[BLUE][1][1],
        matrix[BLUE][1][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,294,40,coords,
    strlen(coords));

if (xid_cursor.flags & CURSOR_ORIG) 
    sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][2][0],matrix[BLUE][2][1],
        matrix[BLUE][2][2]);
else
    {
    mapval = (long)((float)matrix[BLUE][2][0] / scaler + img_idx);
    matrix[BLUE][2][0] = xid.colortable_ptr[mapval].blue >> 8;
    mapval = (long)((float)matrix[BLUE][2][1] / scaler + img_idx);
    matrix[BLUE][2][1] = xid.colortable_ptr[mapval].blue >> 8;
    mapval = (long)((float)matrix[BLUE][2][2] / scaler + img_idx);
    matrix[BLUE][2][2] = xid.colortable_ptr[mapval].blue >> 8;
    sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][2][0],matrix[BLUE][2][1],
        matrix[BLUE][2][2]);
    }
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,294,60,coords,
    strlen(coords));

sprintf(coords,"%4d",matrix[GREEN][1][1]);
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,198,40,coords,
    strlen(coords));

/*
    Set the foreground to white and write out the target pixel value.  The
    background values of the target pixel have already been set to red,
    green and blue in the routine cursor_setup.
*/
XSetForeground(xid.display,xid_cursor.gc,
    xid.color_table[RGBBW_IDX + WHITE_COL].pixel);

sprintf(coords,"%4d",matrix[RED][1][1]);
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,54,40,coords,
    strlen(coords));

sprintf(coords,"%4d",matrix[BLUE][1][1]);
XDrawString(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,342,40,coords,
    strlen(coords));

/*
    Copy the updated information to the cursor window.
*/
XCopyArea(xid.display,xid_cursor.info_pixmap,cursor_win,xid_cursor.gc,0,0,
    CURSOR_WIDTH,CURSOR_HEIGHT,0,0);

/*
    Is the coordinate information to be written to an ascii file?
if ((event->type == ButtonRelease) && (event->xbutton.button == 2) && 
    ((event->xbutton.state & ControlMask) != 0))
    {
    if (xid_cursor.fp == NULL)
        xid_cursor.fp = fopen("xid_cursor.prt","a");

    if (xid_cursor.fp == NULL)
	xid_message("Error opening text file for coordinate information");
    else
	{
	fprintf(xid_cursor.fp,"%s\n",xid_planes[rplane_idx].filename);
	fprintf(xid_cursor.fp,"File Coordinate: Line = %lf  Sample = %lf\n",
	    ysub,xsub);
	if ((xid_cursor.flags & CURSOR_PROJ) && (xid_cursor.proj_valid))
	    fprintf(xid_cursor.fp,
		"Projection Coordinates: Y = %lf  X = %lf %s %s\n",
		projy,projx,proj_name,xid_planes[rplane_idx].image_info->
		proj_units);
        if ((xid_cursor.flags & CURSOR_LATLONG) && (xid_cursor.proj_valid))
	    fprintf(xid_cursor.fp,"Lat/Long: %lf %s %lf %s\n",latitude,nsstr,
		longitude,ewstr);
	fprintf(xid_cursor.fp,"\n");
	}
    }
*/
/**** DROP ****/
if (event->type == KeyRelease)
    {
/*
    Check the validity of the cursor coordinates.
*/
    if ((event->xkey.x < 0) || (event->xkey.y < 0) ||
        (event->xkey.x > maxns) || (event->xkey.y > maxnl))
        return;
/*
    Capture the key code and the keyboard state.
*/
    keycode = event->xkey.keycode;
    state   = event->xkey.state;

    a_key = XKeysymToKeycode(xid.display,XK_a);
    s_key = XKeysymToKeycode(xid.display,XK_s);

    if ((keycode == a_key) && (state == ControlMask)) 
        {
/*
    Create a dropped line file. 
*/
	if (xid_cursor.fp == NULL) 
	    { 
	    start_line = end_line = 0;
	    if (xid_planes[rplane_idx].image_type == LAS)
		{
		status = c_cmhost(xid_planes[rplane_idx].filename,hname);
		if (status != E_SUCC)
		    {
		    xid_message("Error retrieving host file name");
		    return;
		    }
		}
	    else
		strcpy(hname,xid_planes[rplane_idx].filename);
	    c_lsmknm(hname,"drplx",dropline_file);
    /*
	Check if the file already exists.
    */
	    xid_cursor.fp = fopen(dropline_file,"r+");
	    if (xid_cursor.fp == NULL)
		{
		xid_cursor.fp = fopen(dropline_file,"w");
		fprintf(xid_cursor.fp,"%5d %5d\n",0,0);
		num_lines = num_blocks = 0;
		}
	    else
		{
		fscanf(xid_cursor.fp,"%4d %4d",&num_blocks,&num_lines);
		fseek(xid_cursor.fp,0,SEEK_END);
		}
	    }
	if (xid_cursor.fp == NULL)
	    xid_message("Error opening dropped line file"); 
	else
	    start_line = ysub; 
 	}
    else if ((keycode == s_key) && (state == ControlMask)) 
	{
	if ((xid_cursor.fp != NULL) && (start_line != 0))
	    {
	    end_line = ysub;
	    num_blocks++;
	    nlines = (end_line - start_line + 1);
	    num_lines += nlines;
	    fprintf(xid_cursor.fp,"%d %d\n",start_line,nlines);
	    sprintf(mesg,"starting line = %d number of lines = %d\n",start_line,nlines);
	    xid_message(mesg);
	    status = fseek(xid_cursor.fp,0,SEEK_SET);
	    fprintf(xid_cursor.fp,"%5d %5d\n",num_blocks,num_lines);
	    status = fseek(xid_cursor.fp,0,SEEK_END);
	    start_line = end_line = 0;
	    }
	}
    }
/**** DROP ****/
}

void
cursor_setup()
/*******************************************************************************
Purpose:
    This routine is used to create the appropriate pixmaps and GC's needed
    to display cursor information.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
{
Window	cursor_win;		/* Cursor window ID.			      */

XGCValues 
	values;               	/* Values array for the GC.                   */

XRectangle rectangles[3];       /* Array for rectangle information.           */

Pixel	back;			/* Background color.			      */

unsigned long 
	valuemask = 0;    	/* Specify no values.                         */


/*
    Determine the cursor window ID and create the appropriate pixmaps.
*/
cursor_win = XtWindow(cursor_drawingarea);
xid_cursor.info_pixmap = XCreatePixmap(xid.display,cursor_win,CURSOR_WIDTH,
    CURSOR_HEIGHT,xid.visual_info[0].depth);
xid_cursor.clear_pixmap = XCreatePixmap(xid.display,cursor_win,CURSOR_WIDTH,
    CURSOR_HEIGHT,xid.visual_info[0].depth);

/*
    Create the working GC for the cursor window.
*/
xid_cursor.gc = XCreateGC(xid.display,cursor_win,valuemask,&values);
XSetFont(xid.display,xid_cursor.gc,xid.font_info->fid);
XSetLineAttributes(xid.display,xid_cursor.gc,0,LineSolid,CapRound,JoinRound);

/*
    Assign the rectangle information.
*/
rectangles[0].x = 0; rectangles[1].x = 144; rectangles[2].x = 288;
rectangles[0].y = rectangles[1].y = rectangles[2].y = 0;
rectangles[0].width = rectangles[1].width = rectangles[2].width = 144;
rectangles[0].height = rectangles[1].height = rectangles[2].height = 70;

/*
    Clear the cursor pixmap with the background color of the cursor
    bulletinboard.
*/
XtVaGetValues(cursor_bulletinboard,XmNbackground,&back,NULL);
XSetForeground(xid.display,xid_cursor.gc,back);

XFillRectangle(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,0,0,
    CURSOR_WIDTH,CURSOR_HEIGHT);

/*
    Draw the outlines into the cursor pixmap.
*/
XSetForeground(xid.display,xid_cursor.gc,
    xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
XDrawRectangles(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,rectangles,3);

/*
    Draw the background boxes for the colored "target" (center)
    pixel values.
*/
rectangles[0].x = 59; rectangles[1].x = 203; rectangles[2].x = 347;
rectangles[0].y = rectangles[1].y = rectangles[2].y = 28;
rectangles[0].width = rectangles[1].width = rectangles[2].width = 30;
rectangles[0].height = rectangles[1].height = rectangles[2].height = 16;

XSetForeground(xid.display,xid_cursor.gc,
   xid.color_table[RGBBW_IDX + RED_COL].pixel);
XFillRectangle(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,
    rectangles[0].x,rectangles[0].y,rectangles[0].width,rectangles[0].height);
XSetForeground(xid.display,xid_cursor.gc,
    xid.color_table[RGBBW_IDX + GREEN_COL].pixel);
XFillRectangle(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,
    rectangles[1].x,rectangles[1].y,rectangles[1].width,rectangles[1].height);
XSetForeground(xid.display,xid_cursor.gc,
    xid.color_table[RGBBW_IDX + BLUE_COL].pixel);
XFillRectangle(xid.display,xid_cursor.info_pixmap,xid_cursor.gc,
    rectangles[2].x,rectangles[2].y,rectangles[2].width,rectangles[2].height);

/*
    Fill in the cursor window by copying the information from the
    cursor pixmap to the cursor window.  Copy the same information to
    the clear pixmap.
*/
XCopyArea(xid.display,xid_cursor.info_pixmap,cursor_win,xid_cursor.gc,0,0,
    CURSOR_WIDTH,CURSOR_HEIGHT,0,0);
XCopyArea(xid.display,xid_cursor.info_pixmap,xid_cursor.clear_pixmap,
    xid_cursor.gc,0,0,CURSOR_WIDTH,CURSOR_HEIGHT,0,0);
}

void
cursor_expose(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine is used to redraw the cursor window after an expose event. 
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
Window	cursor_win;		/* Cursor window ID.			      */

XmAnyCallbackStruct *acs=(XmAnyCallbackStruct*)call;

/*
    Copy the current pixmap to the cursor window.
*/
cursor_win = XtWindow(cursor_drawingarea);
XCopyArea(xid.display,xid_cursor.info_pixmap,cursor_win,xid_cursor.gc,
    0,0,CURSOR_WIDTH,CURSOR_HEIGHT,0,0);
}

#define BIG_CURSOR   1
#define SMALL_CURSOR 2
void
cursor_pointer(w, client_data, call_data)
Widget  w;
caddr_t client_data;
caddr_t call_data;
/*******************************************************************************
Purpose:
    This routine is used to draw the crosshair in the appropriate image 
    window at the appropriate location.  The line/sample position is drawn 
    as part of the defined cursor.  This routine handles both mouse movement
    and keyboard movement of the cursor.  It also moves the cursor tracking
    line to the appropriate position if a function such as a profile or
    Z dimension histogram is currently active.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
{
static	int	cursor_height,	/* Cursor height.		       	      */
		cursor_width,	/* Cursor width.		      	      */
		flag;		/* Indicates which cursor to use (large or 
				   small).				      */

XEvent  *event;                 /* The input event.                           */

static char	lstr[20],	/* Line coordinate string.		      */
		sstr[20];	/* Sample coordinate string.		      */

static long 	first = TRUE;	/* Is this the first time this function was
				   called ?				      */

static XColor	fore,		/* Foreground color.			      */
		back;		/* Background color.			      */

double		xsub,		/* Current sample (subpixel accuracy).	      */
		ysub;		/* Current line (subpixel accuracy).	      */

long		curx,		/* Current sample.			      */
		cury,		/* Current line.			      */
		deltax,         /* Amount to warp the cursor.                 */
		deltay,
	    	keycode,        /* Key code from keyboard.                    */
		maxnl,		/* Maximum line value.			      */
		maxns,		/* Maximum sample value.		      */
		move,           /* Amount of cursor single step.              */
		rplane_idx,     /* Index to red plane number of the currently
			   	   viewed image.                              */
		state;          /* Keyboard state (Shift key?)                */

/*
    Make sure an image is currently being viewed.
*/
if ((w == main_image_bulletinboard) && (xid.view == -1))
    return;
else if ((w == zoom_win_bulletinboard) && (xid_zoom.image_pixmap == NULL)) 
    return;
else if ((w == fullres_win_bulletinboard) && (xid_fullres == NULL)) 
    return;
else if ((w == fullres_win_bulletinboard) && (xid_fullres != NULL)) 
    if (xid_fullres[0].image_data == NULL)
	return;

/*
    If this is the first time then assign the fore/background colors,
    clear the working pixmaps and draw the cursor crosshair.
*/
if (first)
    {
    xid.pointer_cursor = (Cursor)NULL;
/*
    Load the appropriate font and create a pixmap and GC for the pointer 
    cursor plane.
*/
    if ((xid_cursor.cursor_font = XLoadQueryFont(xid.display,POINTER_FONT)) 
	== NULL)
        {
	xid_message("Unable to load pointer font");
	return;
        }
    if (strcmp(ServerVendor(xid.display),"Silicon Graphics") == 0)
	{
	cursor_height = 21;
	cursor_width = 21;
	flag = SMALL_CURSOR; 
	}
    else
	{
	cursor_height = 41;
	cursor_width = 63;
	flag = BIG_CURSOR;
	}
    xid_cursor.pointer_pixmap = XCreatePixmap(xid.display, XtWindow(w), 
	cursor_width, cursor_height, 1);
    xid_cursor.pointer_gc= XCreateGC(xid.display,xid_cursor.pointer_pixmap,
	0,0);
    XSetFont(xid.display,xid_cursor.pointer_gc,xid_cursor.cursor_font->fid);

    XSetFunction(xid.display,xid_cursor.pointer_gc,GXcopy);
    XSetForeground(xid.display,xid_cursor.pointer_gc,0);
						       /* 0 for bit operation.*/

    XFillRectangle(xid.display, xid_cursor.pointer_pixmap,
	xid_cursor.pointer_gc, 0, 0, cursor_width, cursor_height);
    XSetForeground(xid.display,xid_cursor.pointer_gc,1); 
						      /* 1 for bit operation. */

/*
    Draw the crosshair.
*/
    if (flag == BIG_CURSOR)
	{
	XDrawLine(xid.display, xid_cursor.pointer_pixmap, xid_cursor.pointer_gc,
	    10, 20, 10, 40);
	XDrawLine(xid.display, xid_cursor.pointer_pixmap, xid_cursor.pointer_gc,
	    0, 30, 20, 30);
	}
    else
	{
	XDrawLine(xid.display, xid_cursor.pointer_pixmap, xid_cursor.pointer_gc,
	    10, 0, 10, 20);
	XDrawLine(xid.display, xid_cursor.pointer_pixmap, xid_cursor.pointer_gc,
	    0, 10, 20, 10);
	}

    XParseColor(xid.display,xid.colormap,"white",&back);
    }

/*
    Get the event values from the widget.
*/
event = (XEvent *)call_data;

/*
    Remove all other MotionNotify events from queue -- this makes for
    smoother scrolling by helping to limit the number of events
    generated while we're acting on the present one.
*/
while (XCheckTypedEvent(xid.display,MotionNotify,event));

/*
    If this is not the first time then free the space used for the old
    cursor.
*/
if (xid.pointer_cursor != NULL)
    {
    XFreeCursor(xid.display,xid.pointer_cursor);
    xid.pointer_cursor = (Cursor)NULL;
    }

/*
    Set the appropriate indices into the memory plane structure and determine
    the maximum number of lines and samples in the window.
*/
rplane_idx = xid_view.rgbplanes[RED] - 1;
if (w == main_image_bulletinboard)
    {
    maxnl = xid_planes[rplane_idx].dspwin[NL] - 1;
    maxns = xid_planes[rplane_idx].dspwin[NS] - 1;
    }
else if (w == zoom_win_bulletinboard)
    {
    maxnl = xid_zoom.parms.factor * xid_zoom.parms.in_lines;
    maxns = xid_zoom.parms.factor * xid_zoom.parms.in_samps;
    }
else if (w == fullres_win_bulletinboard)
    {
    maxnl = xid_fullres[0].dspwin[NL] - 1;
    maxns = xid_fullres[0].dspwin[NS] - 1;
    }

/*
    If a a valid cursor key was pressed on the keyboard then move
    the cursor to the new location.
*/
if (event->type == KeyRelease)
    {
/*
    Check the validity of the cursor coordinates.
*/
    if ((event->xkey.x < 0) || (event->xkey.y < 0) ||
	(event->xkey.x > maxns) || (event->xkey.y > maxnl))
	return;

/*
    Capture the key code and the keyboard state.
*/
    keycode = event->xkey.keycode;
    state   = event->xkey.state;

/*
    Ensure the keys validity.
*/
    if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.j_key) || 
	(keycode == xid_cursor.h_key) || (keycode == xid_cursor.l_key) ||
	(keycode == xid_cursor.left_key) || (keycode == xid_cursor.up_key) ||
	(keycode == xid_cursor.right_key) || (keycode == xid_cursor.down_key))
        {
/*
    Reset the delta X and Y amounts.
*/
	deltax = deltay = 0;

/*
    Check the keyboard state to see if the Shift key was used.
*/
	if (state == ShiftMask)
	    move = 10;		     /* Shift key so use delta of 10.	      */
	else
	    move = 1;

/*
    Set the delta X/Y amount based on the key pressed.
*/
	if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.up_key))
	    deltay -= move;
	else if ((keycode == xid_cursor.j_key) || (keycode == 
	    xid_cursor.down_key))
	    deltay += move;
	else if ((keycode == xid_cursor.h_key) || (keycode == 
	    xid_cursor.left_key))
	    deltax -= move;
	else
	    deltax += move;

/*
    If a valid move can be made then move the cursor.
*/
	if (((event->xkey.x + deltax) < 0) ||
	    ((event->xkey.x + deltax) > maxns) || ((event->xkey.y + deltay) 
	    < 0) || ((event->xkey.y + deltay) > maxnl))
	    ;
	else
	    {
	    XWarpPointer(xid.display,None,None,None,None,None,None,
		deltax,deltay);
	    event->xkey.x += deltax;
	    event->xkey.y += deltay;
	    }
	}
    }

/*
    Create the line/sample strings.
*/
if (w == main_image_bulletinboard)
    {
/*
    Calculate file coordinates for the main image window.
*/
    curx = (((event->xmotion.x + 1) * xid_planes[rplane_idx].skips) +
        xid_planes[rplane_idx].window[SS] - 1) - (xid_planes[rplane_idx].skips
	- 1);
    cury = (((event->xmotion.y + 1) * xid_planes[rplane_idx].skipl) +
        xid_planes[rplane_idx].window[SL] - 1) - (xid_planes[rplane_idx].skipl
	- 1);
    sprintf(lstr,"l: %d",cury);
    sprintf(sstr,"s: %d",curx);
    }
else if (w == zoom_win_bulletinboard)
    {
/*
    Calculate subpixel file coordinates for the zoomed image window.
*/
    if (xid_zoom.flags & ZOOM_MAIN)
        cursor_sub(event->xmotion.x,event->xmotion.y,&xsub,&ysub,
	    xid_planes[rplane_idx].skipl,xid_planes[rplane_idx].skips,
	    xid_planes[rplane_idx].window[SL],
	    xid_planes[rplane_idx].window[SS]);
    else if (xid_zoom.flags & ZOOM_FULLRES)
        cursor_sub(event->xmotion.x,event->xmotion.y,&xsub,&ysub,1,1,
	    xid_fullres[0].window[SL],xid_fullres[0].window[SS]);
    sprintf(lstr,"%lf",ysub);
    sprintf(sstr,"%lf",xsub);
    }
else if (w == fullres_win_bulletinboard)
    {
/*
    Calculate file coordinates for the full resolution image window.
*/
    curx = xid_fullres[0].window[SS] + event->xmotion.x;
    cury = xid_fullres[0].window[SL] + event->xmotion.y;

    sprintf(lstr,"l: %d",cury);
    sprintf(sstr,"s: %d",curx);
    }

/*
    Create the cursor and cursor mask.
*/
if (flag == BIG_CURSOR)
    {
    XSetFunction(xid.display,xid_cursor.pointer_gc,GXcopy);
    XSetForeground(xid.display,xid_cursor.pointer_gc,0);/*0 for bit operation.*/

    XFillRectangle(xid.display,xid_cursor.pointer_pixmap,xid_cursor.pointer_gc,
	15,0,48,25);
    XSetForeground(xid.display,xid_cursor.pointer_gc,1);/*1 for bit operation.*/

    XDrawString(xid.display,xid_cursor.pointer_pixmap,xid_cursor.pointer_gc,15,
	10,lstr,strlen(lstr));
    XDrawString(xid.display,xid_cursor.pointer_pixmap,xid_cursor.pointer_gc,15,
	20,sstr,strlen(sstr));
    }

/*
    Set the foreground color.
*/
XParseColor(xid.display,xid.colormap,wk_colors[xid.cursor_color],&fore);

/*
xid.pointer_cursor = XCreatePixmapCursor(xid.display,xid_cursor.pointer_pixmap,
    xid_cursor.pointer_pixmap,&fore,&back,10,30);
*/
/*
    According to all calculations, the "hot spot" for the crosshair should
    be sample 10 and line 30.  However, this does not appear to give the
    right visual affect (i.e. the line appears to be off by 1).  Therefore,
    unless some other solution can be found, the "hot spot" has been set to
    sample 10 and line 31.
xid.pointer_cursor = XCreatePixmapCursor(xid.display,xid_cursor.pointer_pixmap,
    xid_cursor.pointer_pixmap,&fore,&back,10,31);
*/
if (flag == BIG_CURSOR)
    xid.pointer_cursor = XCreatePixmapCursor(xid.display,xid_cursor.
	pointer_pixmap,xid_cursor.pointer_pixmap,&fore,&back,10,30);
else
    xid.pointer_cursor = XCreatePixmapCursor(xid.display,xid_cursor.
	pointer_pixmap,xid_cursor.pointer_pixmap,&fore,&back,10,10);

/*
    Install the new cursor in the main image window.
*/
XDefineCursor(xid.display,XtWindow(w),xid.pointer_cursor);

/*
    Set the first time flag.
*/
first = FALSE;

/*
    If the cursor tracking line is active, move it accordingly.
*/
if (((xid.num_vert > 0) || (xid.num_horiz > 0)) && (w == 
    main_image_bulletinboard))
    {
/*
    Restore the image data under the horizontal cursor tracking line.
*/
    if (xid.line_y != -1)
	XClearArea(xid.display,XtWindow(w),0,xid.line_y,
	    xid_planes[rplane_idx].dspwin[NS],1,True);

/*
    Restore the image data under the vertical cursor tracking line.
*/
    if (xid.line_x != -1)
	XClearArea(xid.display,XtWindow(w),xid.line_x,0,
	    1,xid_planes[rplane_idx].dspwin[NL],True);

/*
    Move the horizontal cursor tracking line.
*/
    if (xid.num_horiz > 0)
	{
	XDrawLine(xid.display,XtWindow(w),xid.line_gc,0,
	    event->xmotion.y,xid_planes[rplane_idx].dspwin[NS],
	    event->xmotion.y);
	xid.line_y = event->xmotion.y;
	}

/*
    Move the vertical cursor tracking line.
*/
    if (xid.num_vert > 0) 
	{
	XDrawLine(xid.display,XtWindow(w),xid.line_gc,
	    event->xmotion.x,0,event->xmotion.x,
	    xid_planes[rplane_idx].dspwin[NS]);
	xid.line_x = event->xmotion.x;
	}
    }
/*
    Draw any rubber band outlines (if working with graphics data).
*/
graphics_rubberband(w,event->xmotion.x,event->xmotion.y,rplane_idx);
}

cursor_sub(xcursor,ycursor,xsub,ysub,skipl,skips,fsl,fss)
double	*xsub,			/* X location of input point (subpixel
				   accuracy) (O)	      		      */
	*ysub;			/* Y location of input point (subpixel
				   accuracy) (O)	      		      */

long	skipl,			/* Line subsampling factor (I)	 	      */
	skips,			/* Sample subsampling factor (I)	      */
	fsl,			/* Starting line (file coordinate) of the image
				   the coordinate is being calculated for (I) */
	fss,			/* Starting sample (file coordinate) of the 
				   image the coordinate is being calculated 
				   for (I) 				      */
	xcursor,		/* Zero relative X position of crosshair (I). */
	ycursor;		/* Zero relative Y position of crosshair (I). */
/*******************************************************************************
Purpose:
	This routine is used to convert the input crosshair position to a
	line/sample coordinate with subpixel accuracy.

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
    This routine is roughly taken from the LAS function TIEPTS. 
*******************************************************************************/
{
double	x,			/* X coordinate		      		      */
	xfactor,		/* Subpixel factor to add to file coordinate. */
	xyadjust,		/* Adjustment needed to the subpixel value
				   due to short cuts taken when the image 
				   window was enlarged	      		      */
	y,			/* Y coordinate		      		      */
	yfactor;		/* Subpixel factor to add to file coordinate. */

long 	lborder,		/* Number of pixels thrown away from the left
				   edge of the extracted window		      */
	left,			/* Remainder of a division operation	      */
	sl,			/* Starting line of extracted window.	      */
	ss,			/* Startling sample of extracted window.      */
	tborder,		/* Number of pixels thrown away from the top
			  	   edge of the extracted window		      */
	xfil,			/* X file relative coordinate.		      */
	ximg,			/* X image relative coordinate.		      */
	yfil,			/* Y file relative coordinate.		      */
	yimg;			/* Y image relatvie coordinate.		      */
/*
   Determine how many pixels from the window extracted for zooming were thrown
   away by the resampling routine. 
*/
	left = xid_zoom.parms.horiz_dim % 2;
	if (left == 0)
	    lborder = xid_zoom.parms.horiz_dim / 2.0 - 1;
	else
	    lborder = xid_zoom.parms.horiz_dim / 2.0;

	left = xid_zoom.parms.vert_dim % 2;
	if (left == 0)
	    tborder = xid_zoom.parms.vert_dim / 2.0 - 1;
	else
	    tborder = xid_zoom.parms.vert_dim / 2.0;
/*
   Calculate the adjustment needed due to short cuts taken when the image
   window was zoomed.
*/
	xyadjust = ((int)(xid_zoom.parms.factor / 2.0)) * (1.0 / 
	    (double)(xid_zoom.parms.factor));  
/*
   Convert the coordinate to subpixel accuracy. 
*/
	x = (.5 + 1.0 / (double)(2 * xid_zoom.parms.factor) + xcursor /
	    (double)xid_zoom.parms.factor) + xyadjust;
	y = (.5 + 1.0 / (double)(2 * xid_zoom.parms.factor) + ycursor /
	    (double)xid_zoom.parms.factor) + xyadjust;

/*
    Determine the 1-relative image coordinate (based on the zoom factor).
*/
	ximg = (int) (xcursor / (float)xid_zoom.parms.factor) + 1;
	yimg = (int) (ycursor / (float)xid_zoom.parms.factor) + 1;

/*
    Determine the 1-relative file coordinate (based on the subsampling factor
    and the window loaded).
*/
	xfil = (ximg * skips + fss - 1) - (skips - 1);
	yfil = (yimg * skipl + fsl - 1) - (skipl - 1);

/*
    Determine the starting line and sample based on the window extracted and
    factor this into the file coordinate.
*/
	ss = (xid_zoom.window[SS] * skips) + 1;
	sl = (xid_zoom.window[SL] * skipl) + 1;

	xfil = xfil + (ss + lborder - 1);
	yfil = yfil + (sl + tborder - 1);

/*
    Determine the fractional portion of the coordinate value and add this to
    the calculated file coordinate.
*/
	xfactor = x - ximg;
	yfactor = y - yimg;

	*xsub = xfil + xfactor;
	*ysub = yfil + yfactor;

	return;
}
