/*
 *  This is the fullres function source file for the fullres panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <X11/cursorfont.h>
#include "xid.h"
#include "wait.xbm"
#include "../xid_widgets/ProcessM.h"
#include "../xid_widgets/Context.h"

void
fullres_create()

/*******************************************************************************
Purpose:
    This routine calls the routine which allocates memory for the full
    resolution structure, calls the routine which initializes the variables
    defining the default full resolution window size and adds the appropriate 
    event handlers.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
Widget	widget;			/* Widget ID.				      */

long	create = FALSE,		/* Indicates whether or not full resolution
				   pixmaps were created.		      */
	i,			/* Looping index.			      */
	state;			/* Toggle button state.			      */

char	fullres_opt[4];		/* Full resolution window size option.	      */

/*
    Allocate memory for full resolution structure and initialize appropriate
    variables.
*/
fullres_setup();

/*
    Make sure the appropriate flags and variables are set for the default
    size full resolution window.
*/
state = XmToggleButtonGetState(fullres_256_button);
if (state)
    {
    strcpy(fullres_opt,"256");
    widget = fullres_256_button;
    }
else
    {
    state = XmToggleButtonGetState(fullres_512_button);
    if (state)
	{
	strcpy(fullres_opt,"512");
	widget = fullres_512_button;
	}
    else
	{
	state = XmToggleButtonGetState(fullres_full_button);
	if (state)
	    {
	    strcpy(fullres_opt,"FS");
	    widget = fullres_full_button;
	    }
	else
	    {
	    state = XmToggleButtonGetState(fullres_user_button);
	    if (state)
		{
		strcpy(fullres_opt,"US");
		widget = fullres_user_button;
		}
	    }
	}
    }
/* 
    Set appropriate values based on the full resolution size.
*/
fullres_size(widget,fullres_opt,NULL);

/*
    Create full resolution window pixmaps if the the graphics window is 
    active or if the main window graphics pixmaps exist.
*/
if (((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.pixmap[0] != NULL)) &&
    (xid_fullres[0].height != 0) && (xid_fullres[0].width != 0))
    {
    XSetBackground(xid.display,xid_graphics.rb_gc,0);
    XSetForeground(xid.display,xid_graphics.rb_gc,0);

    if (xid_graphics.full_pix[0] == NULL)
	{
	for (i=0; i < USER_GRCOLORS; i++)
	    xid_graphics.full_pix[i] = XCreatePixmap(xid.display,XtWindow(
		fullres_win_bulletinboard),xid_fullres[0].width,
		xid_fullres[0].height,1);
	create = TRUE;
	}

    if (create) 
	{
	for (i=0; i < USER_GRCOLORS; i++)
	    XFillRectangle(xid.display,xid_graphics.full_pix[i],
		xid_graphics.rb_gc,0,0,xid_fullres[0].width,
		xid_fullres[0].height);
	}

    XSetBackground(xid.display,xid_graphics.rb_gc,1);
    XSetForeground(xid.display,xid_graphics.rb_gc,1);
    }

/*
    Add appropriate event handlers if the cursor window is active.
*/
if ((xid_cursor.flags & CURSOR_ACTIVE) && (xid_cursor.flags & CURSOR_MAN))
    XtAddEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
	KeyReleaseMask,FALSE,cursor_track,NULL);
else if ((xid_cursor.flags & CURSOR_ACTIVE) && (xid_cursor.flags & CURSOR_AUTO))
    XtAddEventHandler(fullres_win_bulletinboard,
	ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,
	NULL);
/*
    Add an event handler to detect pointer motion and key release events in
    the full resolution image window.
*/
XtAddEventHandler(fullres_win_bulletinboard,PointerMotionMask|
    KeyReleaseMask,FALSE,cursor_pointer,NULL);
/*
    Add an event handler to detect exposure events in the full resolution 
    image window.
*/
XtAddEventHandler(fullres_win_bulletinboard,ExposureMask,FALSE,
    graphics_expose,NULL);
}

void
fullres_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine closes the full resolution window and full resolution image
    window, sets the main menu option to sensitive, resets bit flags and
    removes the appropriate event handler.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	i,			/* Looping index.			      */
	plane_idx,		/* Index into memory plane structure.	      */
	xwin,			/* X window coordinate.			      */
	ywin;			/* Y window coordinate.			      */

/*
    Erase any full resolution outlines.
*/
if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
    {
/*
    Determine the memory plane index.
*/
    plane_idx = xid_view.rgbplanes[RED] - 1;
    if (plane_idx != -1)
	{
	xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
	    (float)xid_fullres[0].window[SL],&xwin,&ywin);
	XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,ywin-5,
	    xid_fullres[0].width + 10, xid_fullres[0].height + 10,TRUE);
	}
    }

/*
    Free resources.
*/
if (xid_fullres[0].flags & FULLRES_USER)
    XtRemoveEventHandler(main_image_bulletinboard,Button1MotionMask|
	ButtonReleaseMask,FALSE,fullres_drag,NULL);
XFreeGC(xid.display,xid_fullres[0].gc);
xid_fullres[0].gc = (GC)NULL;

for (i=0; i < xid.num_fullres; i++)
    {
    if (xid_fullres[i].image_data != NULL)
	free(xid_fullres[i].image_data);
    if (xid_fullres[i].image_pixmap != NULL)
	XFreePixmap(xid.display,xid_fullres[i].image_pixmap);
    }
free(xid_fullres);
xid_fullres = (struct XID_FULLRES *)NULL;
xid.num_fullres = 0;

/*
    Clear the full resolution window.
*/
XSetWindowBackground(xid.display,XtWindow(fullres_win_bulletinboard),
    xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
XClearWindow(xid.display,XtWindow(fullres_win_bulletinboard));

if (xid_graphics.flags & GR_ACTIVE)
    XtRemoveEventHandler(fullres_win_bulletinboard,LeaveWindowMask|
	EnterWindowMask,FALSE,graphics_expose,NULL);

if ((xid_cursor.flags & CURSOR_ACTIVE) && (xid_cursor.flags & CURSOR_MAN))
    XtRemoveEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
	KeyReleaseMask,FALSE,cursor_track,NULL);
else if ((xid_cursor.flags & CURSOR_ACTIVE) && (xid_cursor.flags & CURSOR_AUTO))
    XtRemoveEventHandler(fullres_win_bulletinboard,ButtonReleaseMask|
	PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

if (xid_zoom.flags & ZOOM_ACTIVE) 
    {
/*
    If a zoomed window which was calculated from the full resolution window
    is currently displayed, clear the zoomed window (i.e., if the full
    resolution window is gone, the corresponding zoomed window no longer 
    makes sense).
*/
    if (xid_zoom.flags & ZOOM_FULLRES)
	{
	if (xid_zoom.flags & ZOOM_ACTIVE)
	    {
	    XSetWindowBackground(xid.display,XtWindow(zoom_win_bulletinboard),
		xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
	    XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));
	    if (xid_zoom.image_pixmap != NULL)
		{
		XFreePixmap(xid.display,xid_zoom.image_pixmap);
		xid_zoom.image_pixmap = (Pixmap)NULL;
		}
	    if (xid_zoom.image_data != NULL)
		{
		free(xid_zoom.image_data);
		xid_zoom.image_data = NULL;
		}
	    }
	}
    }
/*
    No longer need to track the cursor in the full resolution window.
*/
XtRemoveEventHandler(fullres_win_bulletinboard,PointerMotionMask|KeyReleaseMask,
    FALSE,cursor_pointer,NULL);

/*
    Close both the full resolution panel and image window.
*/
XtUnmanageChild(Fullres_panel_bulletinboard);
XtUnmanageChild(Fullres_scrollwin);
XtPopdown(Shell_FullRes);
XtPopdown(Shell_FullResWin);

/*
    Set the full resolution button on the main panel to sensitive.
*/
XtSetSensitive(main_fullres_button,True);

/*
    Reset the active flag.
*/
for (i=0; i < xid.num_fullres; i++)
    xid_fullres[i].flags = xid_fullres[i].flags ^ FULLRES_ACTIVE;
}

void
fullres_size(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate variables and flags based on the full
    resolution size chosen by the user.  If the user chose the user selected
    size, an event handler is added which allows the user to select the size
    of the full resolution window with the cursor.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	i,			/* Looping index.		      	      */
	plane_idx,		/* Index into memory plane structure.	      */
	xwin,			/* X window coordinate.			      */
	ywin;			/* Y window coordinate.			      */

char	height[5],		/* Height text string.			      */
	width[5];		/* Width text string.			      */


/*
    Determine the memory plane index.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;

/*
    Based on the option selected, set the appropriate variables and flags.
*/
if (strcmp(client,"256") == 0)
    {
/*
    Erase any previous full resolution outlines.
*/
    if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
	{
	xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
	    (float)xid_fullres[0].window[SL],&xwin,&ywin);
	XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,ywin-5,
	    xid_fullres[0].width + 10, xid_fullres[0].height + 10,TRUE);
	}
/*
    Remove the event handler that allows the user to select the full
    resolution size with the cursor.
*/
    if (xid_fullres[0].flags & FULLRES_USER)
        XtRemoveEventHandler(main_image_bulletinboard,Button1MotionMask|
	    ButtonReleaseMask,FALSE,fullres_drag,NULL);
    for (i=0; i < xid.num_fullres; i++)
        xid_fullres[i].width = xid_fullres[i].height = 256;
    XmTextSetString(fullres_h_text,"256");
    XmTextSetString(fullres_w_text,"256");
    for (i=0; i < xid.num_fullres; i++)
  	xid_fullres[i].flags = xid_fullres[i].flags & ~FULLRES_USER;
    }
else if (strcmp(client,"512") == 0)
    {
/*
    Erase any previous full resolution outlines.
*/
    if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
	{
	xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
	    (float)xid_fullres[0].window[SL],&xwin,&ywin);
	XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,ywin-5,
	    xid_fullres[0].width + 10, xid_fullres[0].height + 10,TRUE);
	}
/*
    Remove the event handler that allows the user to select the full
    resolution size with the cursor.
*/
    if (xid_fullres[0].flags & FULLRES_USER)
        XtRemoveEventHandler(main_image_bulletinboard,Button1MotionMask|
	    ButtonReleaseMask,FALSE,fullres_drag,NULL);
    for (i=0; i < xid.num_fullres; i++)
        xid_fullres[i].width = xid_fullres[i].height = 512;
    XmTextSetString(fullres_h_text,"512");
    XmTextSetString(fullres_w_text,"512");
    for (i=0; i < xid.num_fullres; i++)
  	xid_fullres[i].flags = xid_fullres[i].flags & ~FULLRES_USER;
    }
else if (strcmp(client,"FS") == 0)
    {
/*
    Erase any previous full resolution outlines.
*/
    if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
	{
	xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
	    (float)xid_fullres[0].window[SL],&xwin,&ywin);
	XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,ywin-5,
	    xid_fullres[0].width + 10, xid_fullres[0].height + 10,TRUE);
	}
/*
    Remove the event handler that allows the user to select the full
    resolution size with the cursor.
*/
    if (xid_fullres[0].flags & FULLRES_USER)
        XtRemoveEventHandler(main_image_bulletinboard,Button1MotionMask|
	    ButtonReleaseMask,FALSE,fullres_drag,NULL);
/*
    Get the display size.
*/
    for (i=0; i < xid.num_fullres; i++)
	{
        xid_fullres[i].width = DisplayWidth(xid.display,xid.screen);
        xid_fullres[i].height= DisplayHeight(xid.display,xid.screen);
	}
    sprintf(width,"%d",xid_fullres[0].width);
    sprintf(height,"%d",xid_fullres[0].height);
    XmTextSetString(fullres_h_text,height);
    XmTextSetString(fullres_w_text,width);
    for (i=0; i < xid.num_fullres; i++)
  	xid_fullres[i].flags = xid_fullres[i].flags & ~FULLRES_USER;
    }
else
    {
    xid_fullres[0].ulx = xid_fullres[0].uly = xid_fullres[0].prevx =
	xid_fullres[0].prevy = -1;
/*
    Add the event handler that allows the user to select the full
    resolution size with the cursor.
*/
    XtAddEventHandler(main_image_bulletinboard,Button1MotionMask|
	    ButtonReleaseMask,FALSE,fullres_drag,NULL);
    for (i=0; i < xid.num_fullres; i++)
	xid_fullres[i].flags = xid_fullres[i].flags | FULLRES_USER;
    }

/*
    Free and recreate the full resolution pixmaps if the graphics window
    is active.
*/
if (((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.full_pix[0] != NULL))
    && !(xid_fullres[0].flags & FULLRES_USER))
    {
    if (xid_graphics.full_pix[0] != NULL)
	for (i=0; i < USER_GRCOLORS; i++)
	    XFreePixmap(xid.display,xid_graphics.full_pix[i]);
/*
    Create full resolution window pixmaps. 
*/
    XSetBackground(xid.display,xid_graphics.rb_gc,0);
    XSetForeground(xid.display,xid_graphics.rb_gc,0);

    for (i=0; i < USER_GRCOLORS; i++)
	{
	xid_graphics.full_pix[i] = XCreatePixmap(xid.display,XtWindow(
	    fullres_win_bulletinboard),xid_fullres[0].width,
	    xid_fullres[0].height,1);
	XFillRectangle(xid.display,xid_graphics.full_pix[i],
	    xid_graphics.rb_gc,0,0,xid_fullres[0].width,
	    xid_fullres[0].height);
	}

    XSetBackground(xid.display,xid_graphics.rb_gc,1);
    XSetForeground(xid.display,xid_graphics.rb_gc,1);
    }
}

void
fullres_update(sl,ss)
long	sl,			/* Starting line of window to load.	      */
	ss;			/* Starting sample of window to load.	      */
/*******************************************************************************
Purpose:
    This routine determines the full resolution window to be loaded and calls
    the appropriate routine to load and display the window.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
Window	imgwin;			/* Window ID for full resolution image window.*/

Cursor	img_cursor;		/* Cursor for image window.		      */

Pixmap	wait_pix;		/* Wait cursor pixmap.			      */

XColor	foregr,			/* Foreground color for cursor.		      */
	backgr;			/* Background color for cursor.	  	      */

long	height,			/* Height of window (main image window 
				   coords).				      */
	i,			/* Looping index.		   	      */
	plane_idx,		/* Index into memory plane structure.	      */
	status,			/* Status indicator.			      */
	width,			/* Width of window (main image window coords).*/
	xwin,			/* X window coordinate of upper left corner.  */
	ywin;			/* Y window coordinate of lower right corner. */

char	outheight[5],		/* Height character string.		      */
	outwidth[5];		/* Width character string.		      */

/*
    Output the user-selected height and width.
*/
if (xid_fullres[0].flags & FULLRES_USER)
    {
    sprintf(outheight,"%d",xid_fullres[0].height);
    sprintf(outwidth,"%d",xid_fullres[0].width);
    XmTextSetString(fullres_h_text,outheight);
    XmTextSetString(fullres_w_text,outwidth);
    }

/*
    Determine the starting line and sample of the window to be extracted.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;
if (!(xid_fullres[0].flags & FULLRES_USER))
    {
    sl = sl - xid_fullres[0].height / 2;
    if (sl < 1)
	sl = 1;
    ss = ss - xid_fullres[0].width / 2;
    if (ss < 1)
	ss = 1;
    }

/*
    Reset starting line and sample if the window falls outside the displayed
    image.
*/
if ((sl + xid_fullres[0].height) > xid_planes[plane_idx].image_info->nl)
    {
    sl = xid_planes[plane_idx].image_info->nl - xid_fullres[0].height + 1;
    if (sl < 1)
	{
	sl = 1;
  	for (i=0; i < xid.num_fullres; i++)
	    xid_fullres[i].height = xid_planes[plane_idx].image_info->nl;
    	}
    }
if ((ss + xid_fullres[0].width) > xid_planes[plane_idx].image_info->ns) 
    {
    ss = xid_planes[plane_idx].image_info->ns - xid_fullres[0].width + 1;
    if (ss < 1)
	{
	ss = 1;
	for (i=0; i < xid.num_fullres; i++)
	    xid_fullres[i].width = xid_planes[plane_idx].image_info->ns;
	}
    }

/*
    Draw a new outline of the full resolution window in the main image window.
*/
if (xid_fullres[0].flags & FULLRES_SHOW)
    {
    XSetForeground(xid.display,xid_fullres[0].gc,xid.color_table[RGBBW_IDX + 
	xid.cursor_color].pixel);
    XSetFunction(xid.display,xid_fullres[0].gc,GXcopy);
    xid_file2win(plane_idx,0,(float)ss,(float)sl,&xwin,&ywin);
    width = xid_fullres[0].width / (float)xid_planes[plane_idx].skips;
    height = xid_fullres[0].height / (float)xid_planes[plane_idx].skipl;
    XDrawRectangle(xid.display,XtWindow(main_image_bulletinboard),
	xid_fullres[0].gc,xwin,ywin,width,height);
    XSetForeground(xid.display,xid_fullres[0].gc,xid_fullres[0].rb_fore);
    XSetFunction(xid.display,xid_fullres[0].gc,GXxor);
    }

/*
    For each full resolution band to be loaded ...
*/
for (i=0; i < xid.num_fullres; i++)
    {
/*
    Determine the memory plane index.
*/
    plane_idx = xid_view.rgbplanes[i] - 1;
/*
    Free any previously allocated memory and allocate the amount needed for
    the new window to be loaded.
*/
    if (xid_fullres[i].image_data != NULL)
	{
	free(xid_fullres[i].image_data);
	xid_fullres[i].image_data = NULL;
	}
    xid_fullres[i].image_data = (unsigned char *) malloc(xid_fullres[i].width * 
	xid_fullres[i].height);
    if (xid_fullres[i].image_data == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
/*
    Set the window, bands and file name in the load structure.
*/
    xid_load.windows[i][SL] = xid_fullres[i].window[SL] = sl; 
    xid_load.windows[i][SS] = xid_fullres[i].window[SS] = ss; 
    xid_load.windows[i][NL] = xid_fullres[i].window[NL] = xid_fullres[i].height;
    xid_load.windows[i][NS] = xid_fullres[i].window[NS] = xid_fullres[i].width;
    xid_load.bands[i][0] = xid_planes[plane_idx].band;
    xid_load.bands[i][1] = 0; 
    xid_load.type = xid_planes[plane_idx].image_type;
    strcpy(xid_load.tnames[i],xid_planes[plane_idx].filename);
    if (xid_load.type == LAS)
	{
	status = c_cmhost(xid_planes[plane_idx].filename,xid_load.hnames[i]);
	if (status != E_SUCC)
	    {
	    xid_message("Error retrieving host file name");
	    return;
	    }
	}
    else
	strcpy(xid_load.hnames[i],xid_load.tnames[i]);
    }
xid_load.totbnd = xid_load.nimgs = xid.num_fullres;

/*
    Display the wait cursor (hourglass) in both the full resolution image 
    window and main panel.
*/
imgwin = XtWindow(fullres_win_bulletinboard);
wait_pix = XCreatePixmapFromBitmapData(xid.display,imgwin,wait_bits,wait_width,
    wait_height,xid.color_table[RGBBW_IDX + WHITE_COL].pixel,
    xid.color_table[RGBBW_IDX + BLACK_COL].pixel,1);
XParseColor(xid.display,xid.colormap,"black",&foregr);
XParseColor(xid.display,xid.colormap,"white",&backgr);
img_cursor = XCreatePixmapCursor(xid.display,wait_pix,wait_pix,
    &foregr,&backgr,1,1);
XDefineCursor(xid.display,imgwin,img_cursor);
XDefineCursor(xid.display,XtWindow(fullres_panel_bulletinboard),img_cursor);
XDefineCursor(xid.display,XtWindow(main_form),img_cursor);

/*
    Set the title for the processing message and processing icon.
*/
XtVaSetValues(load_dialog_shell,XmNtitle,"Load - Processing",NULL);

/*
    Load and display the full resolution window.
*/
status = loadimg(XtWindow(fullres_win_bulletinboard),FALSE);
if (status != XID_S)
    xid_message("Error loading full resolution image");

/*
    Redisplay the default arrow cursor.
*/
XFreeCursor(xid.display,img_cursor);
img_cursor = XCreateFontCursor(xid.display,XC_top_left_arrow);
XDefineCursor(xid.display,imgwin,img_cursor);
XDefineCursor(xid.display,XtWindow(fullres_panel_bulletinboard),img_cursor);
XDefineCursor(xid.display,XtWindow(main_form),img_cursor);

/*
    Turn on the bell to let the user know the image load is complete.
*/
XBell(xid.display,0);

/*
    Free resources.
*/ 
XFreeCursor(xid.display,img_cursor);
XFreePixmap(xid.display,wait_pix);

/*
    If the zoom window is currently active and has been calculated from the
    full resolution window, clear the window and free its resources (the 
    zoomed information is no longer valid because a new full resolution 
    image has been loaded).
*/
if ((xid_zoom.flags & ZOOM_ACTIVE) && (xid_zoom.flags & ZOOM_FULLRES))
    {
    XSetWindowBackground(xid.display,XtWindow(zoom_win_bulletinboard),
	xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
    XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));
    if (xid_zoom.image_pixmap != NULL)
	{
	XFreePixmap(xid.display,xid_zoom.image_pixmap);
	xid_zoom.image_pixmap = (Pixmap)NULL;
	}
    if (xid_zoom.image_data != NULL)
	{
	free(xid_zoom.image_data);
	xid_zoom.image_data = NULL;
	}
    }
}

void
fullres_button(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
	This routine detects that the user has selected the left mouse button
	in the main image window and calls the appropriate routine to update 
	the full resolution window accordingly.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
XButtonEvent	*event;		/* X Event structure.			      */

long		i,		/* Looping index.			      */
		plane_idx,	/* Index to red plane number of the currently
			   	   viewed image.			      */
		sl,		/* Starting line.			      */
		ss,		/* Starting sample.			      */
		xwin,		/* X window coordinate.			      */
		ywin;		/* Y window coordinate.			      */

event = (XButtonEvent *)call;
/*
    Return if the full resolution window is not active or this event was
    not triggered from the main image window.
*/
if ((xid_fullres == NULL) || (w != main_image_bulletinboard))
    return;

if (xid_fullres[0].flags & FULLRES_USER)
    return;

/*
    If button 1 was selected in the main image window, update the full 
    resolution window.
*/
if ((w == main_image_bulletinboard) && (event->button == 1) && (xid.view != -1))
    {
/*
    Determine the memory plane index.
*/
    plane_idx = xid_view.rgbplanes[RED] - 1;

/*
    Erase any previous full resolution outlines.
*/
    if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
	{
	xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
	    (float)xid_fullres[0].window[SL],&xwin,&ywin);
	XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,ywin-5,
	    xid_fullres[0].width + 10, xid_fullres[0].height + 10,TRUE);
	}
    ss = (((event->x + 1) * xid_planes[plane_idx].skips) +
        xid_planes[plane_idx].window[SS] - 1) - (xid_planes[plane_idx].skips
	- 1);
    sl = (((event->y + 1) * xid_planes[plane_idx].skipl) +
        xid_planes[plane_idx].window[SL] - 1) - (xid_planes[plane_idx].skipl
	- 1);
/*
    Load the full resolution window.
*/
    fullres_update(sl,ss);

/*
    Create the full resolution pixmaps if they have not been previously
    created and there is graphics data to display.
*/
    if ((xid_graphics.pixmap[0] != NULL) && (xid_graphics.full_pix[0] == NULL))
	{
/*
    Create full resolution window pixmaps. 
*/
	XSetBackground(xid.display,xid_graphics.rb_gc,0);
	XSetForeground(xid.display,xid_graphics.rb_gc,0);

	for (i=0; i < USER_GRCOLORS; i++)
	    {
	    xid_graphics.full_pix[i] = XCreatePixmap(xid.display,XtWindow(
		fullres_win_bulletinboard),xid_fullres[0].width,
		xid_fullres[0].height,1);
	    XFillRectangle(xid.display,xid_graphics.full_pix[i],
		xid_graphics.rb_gc,0,0,xid_fullres[0].width,
		xid_fullres[0].height);
	    }

	XSetBackground(xid.display,xid_graphics.rb_gc,1);
	XSetForeground(xid.display,xid_graphics.rb_gc,1);
	}
/*
    Update the graphics information.
*/
    if (xid_graphics.pixmap[0] != NULL)
	{
	XSetBackground(xid.display,xid_graphics.rb_gc,0);
	XSetForeground(xid.display,xid_graphics.rb_gc,0);
	for (i=0; i < USER_GRCOLORS; i++)
	    XFillRectangle(xid.display,xid_graphics.full_pix[i],
		xid_graphics.rb_gc,0,0,xid_fullres[0].width,
		xid_fullres[0].height);
	XSetBackground(xid.display,xid_graphics.rb_gc,1);
	XSetForeground(xid.display,xid_graphics.rb_gc,1);
	graphics_update(1,plane_idx);
	}
    }
}

void
fullres_setup()

/*******************************************************************************
Purpose:
    This routine determines the number of full resolution bands to load, 
    allocates the full resolution structures and initializes the appropriate
    variables and flags.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
XGCValues
        values;                 /* Values array for the GC.                   */

unsigned long
        valuemask = 0;          /* Specify no values.                         */

long	i;			/* Looping index.			      */

/*
    Determine the number of full resolution bands to be loaded.
*/
if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) && 
    (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
    xid.num_fullres = 1;
else
    xid.num_fullres = 3;

/*
    Allocate memory for the full resolution structure. 
*/
if (xid_fullres == NULL)
    {
    xid_fullres = (struct XID_FULLRES *) calloc(xid.num_fullres,
        sizeof(struct XID_FULLRES));
    if (xid_fullres == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
    }

/*
    Set appopriate variables and flags.
*/
for (i=0; i < xid.num_fullres; i++)
    {
    xid_fullres[i].flags = (FULLRES_ACTIVE | FULLRES_SHOW); 
    xid_fullres[i].image_data = NULL;
    xid_fullres[i].image_pixmap = (Pixmap)NULL;
    xid_fullres[i].gc = NULL;
    if (xid.visual_info[0].depth == 24)
        xid_fullres[i].rb_fore = 0xffffff;
    else
        xid_fullres[i].rb_fore = 0xff;
    xid_fullres[i].window[SL] = xid_fullres[i].window[SS] = -1;
    }
/*
    Set initial state of "show outline" button.
*/
XmToggleButtonSetState(fullres_outline_toggle,True,False);

/*
    Create the GC.
*/
if (xid_fullres[0].gc == NULL)
    {
    xid_fullres[0].gc = XCreateGC(xid.display,XtWindow(
	main_image_bulletinboard),valuemask,&values);
    XSetLineAttributes(xid.display,xid_fullres[0].gc,2,LineSolid,CapButt,
	JoinMiter);
    XSetForeground(xid.display,xid_fullres[0].gc,xid_fullres[0].rb_fore);
    XSetFunction(xid.display,xid_fullres[0].gc,GXxor);
    }
}

void
fullres_drag(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
    Based on the cursor position, this routine draws a rubber-band rectangle
    outlining the currently selected full resolution area. 

By L. Huewe, USGS/EROS Data Center, 1/93

Algorithm Description:
*******************************************************************************/
{
XEvent  *event;         	/* X event structure defining input event.    */

long    height,                 /* Rectangle height.                          */
	i,			/* Looping index.			      */
	plane_idx,		/* Index into memory plane structure.	      */
	sl,			/* Starting line of window to load.	      */
	ss,			/* Starting line of window to load.	      */
        width,                  /* Rectangle width.                           */
	xwin,			/* X window coordinate.			      */
	ywin;			/* Y window coordinate.			      */

char	context_name[ELEMENT_MAX_LENGTH];
				/* Current active context name.		      */

event = (XEvent *)call;

if (event->type == MotionNotify)
    {
/*
    This statement is necessary until all DG systems are upgraded to dgux
    5.4.2.  Versions previous to this seem to not detect the event
    Button1MotionMask correctly.  The event handler gets fired for any
    motion event not just when button 1 is pressed.
*/
    if (event->xmotion.state != Button1Mask)
	return;
/*
    If the current context is not the "Image" context, do nothing.
*/
    XeContextGetActive((XeContextWidget)context_mouse,context_name);
    if (strcmp(context_name,context_names[CON_IMAGE]) != 0)
	return;
/*
    Erase the previous rubber-band rectangle.
*/
    if ((xid_fullres[0].ulx != -1) && (xid_fullres[0].prevx != -1))
	{
	width = xid_fullres[0].prevx - xid_fullres[0].ulx;
	height = xid_fullres[0].prevy - xid_fullres[0].uly;
	if ((width > 0) && (height > 0))
	    XDrawRectangle(xid.display,XtWindow(w),xid_fullres[0].gc,
		xid_fullres[0].ulx,xid_fullres[0].uly,width,height);
	}

    if (xid_fullres[0].ulx == -1)
	{
	xid_fullres[0].ulx = event->xmotion.x;
	xid_fullres[0].uly = event->xmotion.y;
	}
/*
    Draw the new rubber band rectangle.
*/
    if (xid_fullres[0].ulx != -1)
	{
	width = event->xmotion.x - xid_fullres[0].ulx;
	height = event->xmotion.y - xid_fullres[0].uly;
	if ((width > 0) && (height > 0))
	    {
	    XDrawRectangle(xid.display,XtWindow(w),xid_fullres[0].gc,
		xid_fullres[0].ulx,xid_fullres[0].uly,width,height);
	    xid_fullres[0].prevx = event->xmotion.x;
	    xid_fullres[0].prevy = event->xmotion.y;
	    }
	else
	    xid_fullres[0].prevx = xid_fullres[0].prevy = -1;
	}
    }
else if (event->type == ButtonRelease)
    {
/*
    If the current context is not the "Image" context, do nothing.
*/
    XeContextGetActive((XeContextWidget)context_mouse,context_name);
    if (strcmp(context_name,context_names[CON_IMAGE]) != 0)
	return;
/*
    Determine the memory plane index.
*/
    plane_idx = xid_view.rgbplanes[RED] - 1;

/*
    Erase any previous full resolution outlines.
*/
if ((xid_fullres[0].window[SL] > 0) && (xid_fullres[0].window[SS]))
    {
    xid_file2win(plane_idx,0,(float)xid_fullres[0].window[SS],
	(float)xid_fullres[0].window[SL],&xwin,&ywin);
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),xwin-5,ywin-5,
	xid_fullres[0].width + 10, xid_fullres[0].height + 10,TRUE);
    }

/*
    Erase the rubber-band rectangle.
*/
    if ((xid_fullres[0].ulx != -1) && (xid_fullres[0].prevx != -1))
	{
	width = xid_fullres[0].prevx - xid_fullres[0].ulx;
	height = xid_fullres[0].prevy - xid_fullres[0].uly;
	if ((width > 0) && (height > 0))
	    XDrawRectangle(xid.display,XtWindow(w),xid_fullres[0].gc,
		xid_fullres[0].ulx,xid_fullres[0].uly,width,height);
	}

    if ((xid.view != -1) && (xid_fullres[0].ulx != -1))
        {
	if ((((xid_fullres[0].prevy - xid_fullres[0].uly + 1) * xid_planes[
	    plane_idx].skipl) < 0) || (((xid_fullres[0].prevx - xid_fullres[0].
	    ulx + 1) * xid_planes[plane_idx].skips) < 0))
	    {
	    xid_message("Invalid width or height");
	    xid_fullres[0].ulx = xid_fullres[0].uly = xid_fullres[0].prevx =
		xid_fullres[0].prevy = -1;
	    return;
	    }
        ss = (((xid_fullres[0].ulx + 1) * xid_planes[plane_idx].skips) +
            xid_planes[plane_idx].window[SS] - 1) - (xid_planes[plane_idx].
	    skips - 1);
	sl = (((xid_fullres[0].uly + 1) * xid_planes[plane_idx].skipl) +
	    xid_planes[plane_idx].window[SL] - 1) - (xid_planes[plane_idx].skipl
	    - 1);
	xid_fullres[0].height = (xid_fullres[0].prevy - xid_fullres[0].uly + 1)
	    * xid_planes[plane_idx].skipl;
	xid_fullres[0].width = (xid_fullres[0].prevx - xid_fullres[0].ulx + 1)
	    * xid_planes[plane_idx].skips;
	for (i=1; i < xid.num_fullres; i++)
	    {
	    xid_fullres[i].height = xid_fullres[0].height; 
	    xid_fullres[i].width = xid_fullres[0].width; 
	    }
/*
    Load the full resolution window.
*/
        fullres_update(sl,ss);
/*
    Free and recreate the full resolution pixmaps if the graphics window
    is active or graphics data is displayed.
*/
	if ((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.full_pix[0] != 
	    NULL))
	    {
	    if (xid_graphics.full_pix[0] != NULL)
		for (i=0; i < USER_GRCOLORS; i++)
		    XFreePixmap(xid.display,xid_graphics.full_pix[i]);
/*
    Create full resolution window pixmaps. 
*/
	    XSetBackground(xid.display,xid_graphics.rb_gc,0);
	    XSetForeground(xid.display,xid_graphics.rb_gc,0);

	    for (i=0; i < USER_GRCOLORS; i++)
		{
		xid_graphics.full_pix[i] = XCreatePixmap(xid.display,XtWindow(
		    fullres_win_bulletinboard),xid_fullres[0].width,
		    xid_fullres[0].height,1);
		XFillRectangle(xid.display,xid_graphics.full_pix[i],
		    xid_graphics.rb_gc,0,0,xid_fullres[0].width,
		    xid_fullres[0].height);
		}

	    XSetBackground(xid.display,xid_graphics.rb_gc,1);
	    XSetForeground(xid.display,xid_graphics.rb_gc,1);
	    }
/*
    Update the graphics information.
*/
	if (xid_graphics.pixmap[0] != NULL)
	    {
	    XSetBackground(xid.display,xid_graphics.rb_gc,0);
	    XSetForeground(xid.display,xid_graphics.rb_gc,0);
	    for (i=0; i < USER_GRCOLORS; i++)
		XFillRectangle(xid.display,xid_graphics.full_pix[i],
		    xid_graphics.rb_gc,0,0,xid_fullres[0].width,
		    xid_fullres[0].height);
	    XSetBackground(xid.display,xid_graphics.rb_gc,1);
	    XSetForeground(xid.display,xid_graphics.rb_gc,1);
	    graphics_update(1,plane_idx);
	    }
	}
    xid_fullres[0].ulx = xid_fullres[0].uly = xid_fullres[0].prevx =
	xid_fullres[0].prevy = -1;
    }
}

void
fullres_outline(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether or not the user
    wishes to see an outline in the main image window of the full resolution
    area.

By L. Huewe, USGS/EROS Data Center, 01/93

Algorithm Description:
*******************************************************************************/
{

XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call;

if (acs->set)
    xid_fullres[0].flags |= FULLRES_SHOW;
else
    xid_fullres[0].flags &= ~FULLRES_SHOW;

}
