/*
 *  This is the histo function source file for the histo panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <ctype.h>
#include "xid.h"
#include "typlim.h"

void
histo_create(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine is used to create the GC and set the appropriate default
    flags for the histogram window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
XEvent  *event;         	/* X Event structure.                         */

Window	histo_win;		/* Histogram window ID.			      */

XGCValues 
	values;			/* Values array for the GC.		      */

unsigned long 
	valuemask = 0;		/* Specify no values.			      */

event = (XEvent *)call;
if (event->type == MapNotify)
    {
/*
    Create the working GC for the histogram window.
*/
    histo_win = XtWindow(histo_form);
    xid_histo.gc = XCreateGC(xid.display,histo_win,valuemask,&values);

    XSetForeground(xid.display,xid_histo.gc,
	xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
    XSetBackground(xid.display,xid_histo.gc,
	xid.color_table[RGBBW_IDX + WHITE_COL].pixel);
    XSetFont(xid.display,xid_histo.gc,xid.font_info->fid);

/*
    Set the appropriate histogram flags.
*/
    xid_histo.flags |= HISTO_ACTIVE;
    }
}

void
histo_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine is used to redraw any image data destroyed by the horizontal
    and/or vertical cursor tracking line.  The appropriate event handlers are
    removed and the histogram window is closed.  The main panel button is
    then set to sensitive.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	plane_idx;		/* Memory plane index.			      */

/*
    Remove any event handlers no longer needed.
*/
if (!(xid_histo.flags & HISTO_XY))
    XtRemoveEventHandler(main_image_bulletinboard,PointerMotionMask,
        FALSE,histo_z,NULL);

/*
    Determine which (if any) tracking line was displayed. 
*/
if (xid_histo.flags & HISTO_XZ)
    xid.num_horiz--;
else if (xid_histo.flags & HISTO_YZ)
    xid.num_vert--;

/*
    Restore any image data written over by the vertical or horizontal cursor 
    tracking line if it is no longer needed.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;

if ((xid.num_vert == 0) && (xid.line_x != -1))
    {
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),xid.line_x,0,
	1,xid_planes[plane_idx].dspwin[NL],True);
    xid.line_x = -1;
    xid.flags = xid.flags & (~XID_VERT);
    }
if ((xid.num_horiz == 0) && (xid.line_y != -1))
    {
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,xid.line_y,
	xid_planes[plane_idx].dspwin[NS],1,True);
    xid.line_y = -1;
    xid.flags = xid.flags & (~XID_HORIZ);
    }
if ((xid.num_horiz == 0) && (xid.num_vert == 0) && (xid.line_gc != NULL))
    {
    XFreeGC(xid.display,xid.line_gc);
    xid.line_gc = (GC)NULL;
    }

/*
    Free the hisogram graphics context.
*/
XFreeGC(xid.display,xid_histo.gc);

/*
    Close the histogram window and set the histogram button on the main 
    panel to sensitive.
*/
XtUnmanageChild(Histo_form);
XtPopdown(Shell_Histo);
XtSetSensitive(main_histo_button,True);

/*
    Reset the active flag. 
*/
xid_histo.flags = xid_histo.flags ^ HISTO_ACTIVE;
}

void
histo_expose(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine is used to redraw the histogram window after an expose event. 
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Redraw the histogram window.
*/
histo_draw();
}

void
histo_ilp(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating the object the user
    wishes to histogram (i.e. line, polygon or image).

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Set the flag indicating which object to histogram.
*/
xid_histo.flags = xid_histo.flags & ~(HISTO_IMAGE | HISTO_LINE | HISTO_POLY);
switch (*client)
    {
    case 'I':
	xid_histo.flags = xid_histo.flags | HISTO_IMAGE;
	break;
    case 'L':
	xid_histo.flags = xid_histo.flags | HISTO_LINE;
	break;
    case 'P':
	xid_histo.flags = xid_histo.flags | HISTO_POLY;
	break;
    }
}

void
histo_mo(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user wishes
    to histogram the original image data or the mapped image data.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Set the flag indicating which type of data to histogram.
*/
xid_histo.flags = xid_histo.flags & ~(HISTO_MAPPED | HISTO_ORIG);
if (*client == 'O')
    xid_histo.flags = xid_histo.flags | HISTO_ORIG;
else
    xid_histo.flags = xid_histo.flags | HISTO_MAPPED;

}

void
histo_type(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    selected a distributional or cumulative histogram. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Set the flag indicating the type of histogram to draw. 
*/
xid_histo.flags = xid_histo.flags & ~(HISTO_DIST | HISTO_CUM);
if (*client == 'D')
    xid_histo.flags = xid_histo.flags | HISTO_DIST;
else
    xid_histo.flags = xid_histo.flags | HISTO_CUM;
}

void
histo_update(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine parses the specified mask values and draws the histogram. 

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	i,			/* Looping index.			      */
	nonwhite = FALSE,	/* Indicates whether or not nonwhite space 
				   was found in mask value string.	      */
	status;			/* Status indicator.			      */

char	*maskvals;		/* Pointer to mask value string.	      */

/*
    Determine whether or not any mask values were specified.
*/
maskvals = XmTextGetString(histo_mask_text);
for (i=0; i < strlen(maskvals); i++)
    if (isspace(maskvals[i]) == 0)
 	nonwhite = TRUE;
if (!nonwhite)
    xid_histo.rcount = xid_histo.bcount = 0;
else
    {
/*
    Parse the specified mask values into ranges and/or individual bins.
*/
    if (xid_histo.range != NULL)
	{
	free(xid_histo.range);
	xid_histo.range = NULL;
	}
    if (xid_histo.bin != NULL)
	{
	free(xid_histo.bin);
	xid_histo.bin = NULL;
	}
    status = parse_rb(maskvals,&xid_histo.range,&xid_histo.rcount,
	&xid_histo.bin,&xid_histo.bcount,MINUTINY,MAXUTINY);
    if (status != XID_S)
        xid_histo.rcount = xid_histo.bcount = 0;
    }

/*
    Draw the updated histogram.
*/
XClearWindow(xid.display,XtWindow(histo_drawingarea));
histo_draw();
}

void
histo_xyz(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating which dimension the
    histogram is to be calculated in and adds the appropriate event handlers.  
    It then determines whether the cursor tracking line is on and/or needs
    to be turned on/off.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
XGCValues
	values;                 /* Values array for the GC.                   */

unsigned long
	valuemask = 0;          /* Specify no values.                         */

long	plane_idx;		/* Index to memory plane structure.	      */

/*
    Determine whether or not a tracking line is currently in use. 
*/
if (xid_histo.flags & HISTO_XZ)
    xid.num_horiz--;
else if (xid_histo.flags & HISTO_YZ)
    xid.num_vert--;

/*
    Set the flag indicating the dimension to histogram in.
*/
xid_histo.flags = xid_histo.flags & ~(HISTO_XY | HISTO_XZ | HISTO_YZ);
if (strcmp(client,"XY") == 0)
    xid_histo.flags = xid_histo.flags | HISTO_XY;
else if (strcmp(client,"XZ") == 0)
    xid_histo.flags = xid_histo.flags | HISTO_XZ;
else if (strcmp(client,"YZ") == 0)
    xid_histo.flags = xid_histo.flags | HISTO_YZ;

plane_idx = xid_view.rgbplanes[RED] - 1;
if (!(xid_histo.flags & HISTO_XY))
    {
/*
    Add the appropriate event handler for a Z dimension histogram.
*/
    XtAddEventHandler(main_image_bulletinboard,PointerMotionMask,
        FALSE,histo_z,NULL);
    if (xid_histo.flags & HISTO_XZ)
	xid.num_horiz++;
    else if (xid_histo.flags & HISTO_YZ)
	xid.num_vert++;

/*
    Create a GC for the cursor tracking line.
*/
    if (xid.line_gc == NULL)
        xid.line_gc = XCreateGC(xid.display,XtWindow(main_image_bulletinboard),
	    valuemask,&values);
    XSetForeground(xid.display,xid.line_gc,
	xid.color_table[RGBBW_IDX + WHITE_COL].pixel);
    }
else
/*
    Remove the Z dimension event handler.
*/
    XtRemoveEventHandler(main_image_bulletinboard,PointerMotionMask,
        FALSE,histo_z,NULL);

/*
    If a tracking line is no longer needed, restore the image data.
*/
if ((xid.num_vert == 0) && (xid.line_y != -1))
    {
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),xid.line_x,0,
	1,xid_planes[plane_idx].dspwin[NL],True);
    xid.line_x = -1;
    xid.flags = xid.flags & (~XID_VERT);
    }
if ((xid.num_horiz == 0) && (xid.line_x != -1))
    {
    XClearArea(xid.display,XtWindow(main_image_bulletinboard),0,xid.line_y,
	xid_planes[plane_idx].dspwin[NS],1,True);
    xid.line_y = -1;
    xid.flags = xid.flags & (~XID_HORIZ);
    }
if ((xid.num_horiz == 0) && (xid.num_vert == 0) && (xid.line_gc != NULL))
    {
    XFreeGC(xid.display,xid.line_gc);
    xid.line_gc = (GC)NULL;
    }
}


#define SMALLX 100	/* Smallest acceptable x axis. 			*/
#define SMALLY 25	/* Smallest acceptable y axis. 			*/

#define DSPANNOT 25	/* X coordinate of the Y axis (arbitrary). 	*/


FUNCTION histo_draw()
/*******************************************************************************
Purpose:
    This routine is used to draw the appropriate number of cumulative or
    distributional histograms (typically 1 for black and white images or 
    3 for color images).  Any values specified by the user to be masked out
    are not plotted on the histogram.  The histogram is annotated with
    the mean, standard deviation, min and max pixel values.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
{
Window  histo_win;      /* Histogram window ID.                         */

long	c_width,	/* Calculated font character width.		*/
	c_height;	/* Calculated font character height.		*/

int	yoffset,	/* Location of x axis on screen.		*/
	uscreen,	/* Useable part of screen.			*/
	pscreen,	/* Part of screen for a single component.	*/
	height,		/* Height of the yaxis for single component.	*/
	length,		/* Length of the xaxis.				*/
	border,		/* Area between axis and border of screen.	*/
	yaxis,		/* A screen location on yaxis.			*/
	xaxis,		/* A screen location on the xaxis.		*/
	xshift,		/* Shift x axis for annotation,
			   where xshift is the maximum number
			   of characters between the left
			   margin and the y axis.			*/
	freq,		/* Frequency.					*/
	x1,		/* Screen x location.				*/
	x2,		/* Screen x location.				*/
	y1,		/* Screen y location.				*/
	y2,		/* Screen y location.				*/
	comp,		/* Color component.				*/
	i,j,		/* Variable for loop.				*/
	xoff,		/* X and Y offsets for the histogram axis.	*/
	yoff,
	ymax,		/* Used to label the y axis of the histogram.	*/
	slen,		/* Annotation length for annot.			*/
	totbnd,		/* Total number of bands to histogram.		*/
	bnds[3],	/* Bands to histogram.				*/
	mskflag = FALSE,/* Flag indicating whether mask values were
			   applied to the histogram data.		*/
	zflag = FALSE;	/* Flag indicating histogram in z dimension.	*/

long	*histptr;	/* Pointer to histogram array.			*/

float	maxp = 0.0,	/* Maximum histogram value.			*/
	lnmx = 255;	/* Maximum length for a histogram.		*/

char	annot[50],	/* Annotation for histogram.			*/
	stdeviat[20],	/* Annotation for standard deviation.		*/
	meann[20],	/* Annotation for the mean.			*/
        minstr[20],     /* Annotation for the min band value.		*/
        maxstr[20];     /* Annotation for the max band value.		*/


/*
    Print a warning and return if no image data is currently displayed.
*/
if (xid.view == -1)
    {
    xid_message("No image data currently being viewed");
    return(XID_S);
    }

/*
    Get the window ID of the histogram window.
*/
histo_win = XtWindow(histo_drawingarea);

/*
    Calculate the number of bands to histogram.
*/
if ((xid_histo.flags & HISTO_XZ) || (xid_histo.flags & HISTO_YZ))
    {
    totbnd = 1;
    zflag = TRUE;
    }
else if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) && 
    (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
    {
    bnds[0] = xid_view.rgbplanes[RED];
    totbnd = 1;
    }
else
    for (i=0, totbnd=0; i < 3; i++)
	if (xid_view.rgbplanes[i] != 0)
	    {
	    bnds[totbnd] = xid_view.rgbplanes[i];
	    totbnd++;
	    }
/*
    Calculate the width and height of characters in the working font.
*/
c_width = xid.font_info->max_bounds.rbearing - 
	  xid.font_info->min_bounds.lbearing;

c_height = xid.font_info->max_bounds.ascent + 
	   xid.font_info->max_bounds.descent;

/*
   Assign the working window location offsets.
*/
xshift = c_width * DSPANNOT; 	
xoff = (1 * c_width);
yoff = 0;

/*
   Set the size variables.
*/
xid_histo.xoffset = xoff += xshift;
uscreen = HISTY - yoff;
pscreen = uscreen / totbnd;
height = .70 * pscreen;
xid_histo.xlength = length = ((HISTX - xoff) > lnmx) ? lnmx : HISTX - xoff;
border = (pscreen - height) / 2;

if (totbnd == 1)
    {
    if (!zflag)
        histptr = (long *)xid_planes[bnds[0]-1].histogram;
    else
	histptr = xid_histo.z_histo;
/*
    Mask out the specified values.
*/
    if (xid_histo.bcount != 0)
	{
	for (i=0; i < xid_histo.bcount; i++)
	    *(histptr + xid_histo.bin[i]) = -(*(histptr + xid_histo.bin[i]));
	}
    if (xid_histo.rcount != 0)
	{
	for (i=0; i < xid_histo.rcount; i++)
	    for (j=xid_histo.range[i*2]; j < xid_histo.range[i*2+1] + 1; j++)
	        *(histptr + j) = -(*(histptr + j));
	}
/*
    Calculate the cumulative histogram.  This is done regardless of the option
    chosen because the cumulative histogram is used in percentage calculations.
*/
    for (freq=0; freq < 256; freq++)
	if (*(histptr + freq) > 0)
	    {
	    xid_histo.cum_histo[0][freq] = *(histptr + freq);
	    break;
	    }
	else
	    xid_histo.cum_histo[0][freq] = 0;
    for (freq=freq+1; freq < 256; freq++)
	if (*(histptr + freq) >= 0)
	    xid_histo.cum_histo[0][freq] = *(histptr + freq) + 
		xid_histo.cum_histo[0][freq-1]; 
	else 
	    xid_histo.cum_histo[0][freq] = xid_histo.cum_histo[0][freq-1];
/*
    Find maximum value.
*/
    if (xid_histo.flags & HISTO_CUM)
        maxp = xid_histo.cum_histo[0][255];
    if (xid_histo.flags & HISTO_DIST)
        for (freq = 0; freq <= 255; histptr++, freq++)
            if (maxp < *histptr) maxp = (float) *histptr;
    }
else
    {
    for (comp = 0; comp < totbnd; comp++) /* 1 or 3 */
        {
        histptr = (long *)xid_planes[bnds[comp]-1].histogram;
/*
    Mask out the specified values.
*/
	if (xid_histo.bcount != 0)
	    {
	    for (i=0; i < xid_histo.bcount; i++)
		*(histptr + xid_histo.bin[i]) = -(*(histptr + xid_histo.
		    bin[i]));
	    }
	if (xid_histo.rcount != 0)
	    {
	    for (i=0; i < xid_histo.rcount; i++)
		for (j=xid_histo.range[i*2]; j < xid_histo.range[i*2+1] + 1; 
		    j++)
		    *(histptr + j) = -(*(histptr + j));
	    }
/*
    Calculate the cumulative histogram.  This is done regardless of the option
    chosen because the cumulative histogram is used in percentage calculations.
*/
	for (freq=0; freq < 256; freq++)
	    if (*(histptr + freq) > 0)
		{
		xid_histo.cum_histo[comp][freq] = *(histptr + freq);
		break;
		}
	    else
		xid_histo.cum_histo[comp][freq] = 0;
	for (freq=freq+1; freq < 256; freq++)
	    if (*(histptr + freq) >= 0)
	        xid_histo.cum_histo[comp][freq] = *(histptr + freq) + 
		    xid_histo.cum_histo[comp][freq-1]; 
	    else 
	        xid_histo.cum_histo[comp][freq] = 
		    xid_histo.cum_histo[comp][freq-1];
/*
    Find maximum value.
*/
        if (xid_histo.flags & HISTO_CUM)
	    maxp = xid_histo.cum_histo[comp][255]; 
        if (xid_histo.flags & HISTO_DIST)
            for (freq = 0; freq <= 255; histptr++, freq++)
                if (maxp < *histptr) maxp = (float) *histptr;
        }
    }

/*
    Check for a zero image.
*/
if (maxp == 0) maxp = 1;

/*
    Draw the histograms.
*/
for (comp = 0; comp < totbnd; comp++)
    {
/*
    Draw the X and Y axis.
*/
    yoffset = yoff + ((comp + 1) * pscreen) - border;
    x1 = xoff;
    x2 = xoff + length + 1;
    XSetForeground(xid.display,xid_histo.gc,
	xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
    XDrawLine(xid.display,histo_win,xid_histo.gc,x1,yoffset,x2,yoffset);

/*
    Save the coordinates of the X axis for use in the calculation of 
    percentages.
*/
    xid_histo.xaxis_x[0] = x1;
    xid_histo.xaxis_x[1] = x2;

/*
    Draw the component labels.
*/
    x1 = xoff - xshift;
    y1 = yoffset - (height * .75) - border;
    if (totbnd == 1)
        strcpy(annot,"Black and White");
    else if (comp == 0)
        strcpy(annot,"Red Component");
    else if (comp == 1) 
        strcpy(annot,"Green Component");
    else if (comp == 2)
        strcpy(annot,"Blue Component");
    slen = strlen(annot);
    XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

/*
    Draw the histogram hash marks.  Note that here the value .008 represents 
    8% of the display screen.
*/
    x2 = xoff - (int)(HISTX * .008);
    y2 = yoffset + (int)(HISTY * .008);
    XSetForeground(xid.display,xid_histo.gc,
	xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
    for(i = 1; i <= 4; i++ )
	{
       	yaxis = yoffset - (i * .25 * height) - 1;
       	XDrawLine(xid.display,histo_win,xid_histo.gc,xoff,yaxis,x2,yaxis);
	xaxis = xoff + (i * .25 * length) + 1;
       	XDrawLine(xid.display,histo_win,xid_histo.gc,xaxis,yoffset,xaxis,y2);
        }

/*
    Label the X and Y axis. 
*/
    for (i = 1; i <= 4; i++)
	{
	x1 = xoff + length * (i * .25);
	y1 = yoffset + c_height + 4; 
	freq = (int) 255 * (i * .25);
	sprintf(annot,"%d",freq);
	slen = strlen(annot);
	x1 = x1 - ((slen * c_width) / 2);
	XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

	ymax = (int) maxp * (i * .25);
	sprintf(annot,"%d",ymax);
	slen = strlen(annot);
	x1 = xoff - (slen * c_width);
	x1 = x1 - c_width;
	y1 = yoffset - (height * (i * .25));
	y1 = y1 + (c_height / 2);
	XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);
	}
/*
    Recalculate statistics if any mask values were applied.
*/
    if ((xid_histo.bcount != 0) || (xid_histo.rcount != 0))
	{
	if (!zflag)
	    histptr = (long *)xid_planes[bnds[comp]-1].histogram;
	else
	    histptr = xid_histo.z_histo;
	mskflag = TRUE;
 	xid_histcalc(histptr,&xid_histo.minval,&xid_histo.maxval,
	    &xid_histo.mean,&xid_histo.stdev);
	}
/*
    Draw the histogram statistics.
*/
    x1 = xoff - xshift;
    y1 = yoffset - (height * .75) - border;

    y1 += height * .18;					       /* Output mean */
    strcpy(annot,"Mean    = ");
    if ((!zflag) && (!mskflag))
        sprintf(meann,"%f",xid_planes[bnds[comp]-1].mean);
    else
        sprintf(meann,"%f",xid_histo.mean);
    meann[strlen(meann) - 4] = '\0';
    strcat(annot,meann);
    slen = strlen(annot);
    XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

    y1 += height * .18;					     /* Output stdev. */
    strcpy(annot,"St Dev  = ");
    if ((!zflag) && (!mskflag))
        sprintf(stdeviat,"%f",xid_planes[bnds[comp]-1].stdev);
    else
        sprintf(stdeviat,"%f",xid_histo.stdev);
    stdeviat[strlen(stdeviat) - 4] = '\0';
    strcat(annot,stdeviat);
    slen = strlen(annot);
    XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);
		    
    y1 += height * .18;					        /* Output min */
    strcpy(annot,"Min Pix = ");
    if ((!zflag) && (!mskflag))
        sprintf(minstr,"%d",xid_planes[bnds[comp]-1].minval);
    else
        sprintf(minstr,"%d",xid_histo.minval);
    strcat(annot,minstr);
    slen = strlen(annot);
    XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

    y1 += height * .18;					        /* Output max */
    strcpy(annot,"Max Pix = ");
    if ((!zflag) && (!mskflag))
        sprintf(maxstr,"%d",xid_planes[bnds[comp]-1].maxval);
    else
        sprintf(maxstr,"%d",xid_histo.maxval);
    strcat(annot,maxstr);
    slen = strlen(annot);
    XDrawString(xid.display,histo_win,xid_histo.gc,x1,y1,annot,slen);

/*
    Store appropriate X and Y coordinates for displaying annotation string
    output by histogram percentage calculation.
*/
    xid_histo.annot_x[comp] = x1;
    xid_histo.annot_y1[comp] = y1 + height * .18;
    xid_histo.annot_y2[comp] = xid_histo.annot_y1[comp] + height * .18;

/*
    Set a pointer to the appropriate histogram data.
*/
    if (xid_histo.flags & HISTO_CUM)
	histptr = (long *)xid_histo.cum_histo[comp];
    else if (zflag)
	histptr = xid_histo.z_histo;
    else
        histptr = (long *)xid_planes[bnds[comp]-1].histogram;    

/*
    Plot the histogram data.
*/
    XSetForeground(xid.display,xid_histo.gc,
	xid.color_table[RGBBW_IDX + comp+2].pixel);
    yoffset--;
    for (freq = 0; freq <= 255; histptr++,freq++)
	{
	if (*histptr > 0)
	    {
	    y2 = yoffset - ((*histptr / maxp) * height);
	    x1 = xoff+(((float)freq/(float)255)*length)+ 1;
	    x2 = x1;
	    XDrawLine(xid.display,histo_win,xid_histo.gc,x1,yoffset,x2,y2);
	    }
	}
/*
    Reset masked out values.
*/
    if (!zflag)
        histptr = (long *)xid_planes[bnds[comp]-1].histogram;
    else
	histptr = xid_histo.z_histo;

    if (xid_histo.bcount != 0)
	{
	for (i=0; i < xid_histo.bcount; i++)
	    *(histptr + xid_histo.bin[i]) = -(*(histptr + xid_histo.bin[i]));
	}
    if (xid_histo.rcount != 0)
	{
	for (i=0; i < xid_histo.rcount; i++)
	    for (j=xid_histo.range[i*2]; j < xid_histo.range[i*2+1] + 1; j++)
	        *(histptr + j) = -(*(histptr + j));
	}
    }

return (XID_S);
}

void
histo_z(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine calculates and subsequently draws the histogram in the XZ or
    YZ dimension.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	dtype,		/* Data type.					      */
	i,j,		/* Looping indices.				      */
	ns,		/* Number of values to fill/histogram.		      */
	num = 0,	/* Number of planes to histogram.		      */
	offset,		/* Offset into image data buffer.		      */
	*plane_nums,	/* Plane numbers to histogram.			      */
	zero = 0;	/* Constant zero.			              */

/*
    Calculate the plane numbers to be histogrammed (i.e. histogram only those
    planes which contain image data).
*/
plane_nums = (long *)calloc(xid.max_plane,sizeof(long));
for (i=0; i < xid.max_plane; i++)
    {
    if (xid_planes[i].flags & USED)
	{
	plane_nums[num] = i;
	num++;
	}
    }

/*
    Initialize number of samples, data type and histogram array.
*/
ns = 256;
dtype = ELONG;
c_pxfill(xid_histo.z_histo,&dtype,&ns,&zero);

if (xid_histo.flags & HISTO_XZ)
    {
/*
    Histogram a single line through all bands.
*/
    dtype = EBYTE;
    for (i=0; i < num; i++)
	{
        offset = (xid.line_y * xid_planes[plane_nums[i]].dspwin[NS]); 
	c_pxhist(&xid_planes[plane_nums[i]].dspwin[NS],
	    xid_planes[plane_nums[i]].image_data+offset,&dtype,
	    xid_histo.z_histo,&zero);
	}
    }
else
    {
/*
    Histogram a single column through all bands.
*/
    for (i=0; i < num; i++)
	{
	for (j=0; j < xid_planes[plane_nums[0]].dspwin[NL]; j++)
	    {
	    offset = (j * xid_planes[plane_nums[i]].dspwin[NS]) + xid.line_x;
	    xid_histo.z_histo[*(xid_planes[plane_nums[i]].image_data+offset)]++;
	    }
	}
    }

/*
    Calculate the histogram statistics.
*/
xid_histcalc(xid_histo.z_histo,&xid_histo.minval,&xid_histo.maxval,
    &xid_histo.mean,&xid_histo.stdev);

/*
    Draw the histogram.
*/
XClearWindow(xid.display,XtWindow(histo_drawingarea));
histo_draw();
free(plane_nums);
}

void
histo_pointer(w, client_data, call_data)
Widget  w;
caddr_t client_data;
caddr_t call_data;
/*******************************************************************************
Purpose:
    This routine is used to draw the cursor at the appropriate location in the
    histogram drawing area.  This routine handles both mouse movement and 
    keyboard movement of the cursor.  If the cursor location is within the
    bounds of the X-axis of the histogram, it also outputs the percentage of 
    image data prior to and including the cursor position.
    
By L. Huewe, USGS/EROS Data Center, 02/92

Algorithm Description:
*******************************************************************************/
{
XEvent  *event;                 /* The input event.                           */

float	percent;		/* Represents the percentage of data prior to
				   and including the specified pixel value.   */

long	deltax,         	/* Amount to warp the cursor.                 */
	deltay,
	gray_val,		/* Gray level value at cursor location.	      */
	height,			/* Height of annotation string.		      */
	i,			/* Looping index.			      */
	keycode,        	/* Key code from keyboard.                    */
	maxlen = 15,		/* Maximum length of annotation string.	      */
	move,           	/* Amount of cursor single step.              */
	state,          	/* Keyboard state (Shift key?)                */
	totbnd,			/* Total number of histogram graphs displayed.*/
	width;			/* Width of annotation string.		      */

char	annot[50];		/* Annotation string.			      */

/*
    Get the event values from the widget.
*/
event = (XEvent *)call_data;

/*
    Remove all other MotionNotify events from queue -- this makes for
    smoother scrolling by helping to limit the number of events
    generated while we're acting on the present one.
*/
while (XCheckTypedEvent(xid.display,MotionNotify,event));

/*
    If a a valid cursor key was pressed on the keyboard then move
    the cursor to the new location.
*/
if (event->type == KeyRelease)
    {
/*
    Check the validity of the cursor coordinates.
*/
    if ((event->xkey.x < 0) || (event->xkey.y < 0) ||
	(event->xkey.x > HISTX) || (event->xkey.y > HISTY))
	return;

/*
    Capture the key code and the keyboard state.
*/
    keycode = event->xkey.keycode;
    state   = event->xkey.state;

/*
    Ensure the keys validity.
*/
    if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.j_key) || 
	(keycode == xid_cursor.h_key) || (keycode == xid_cursor.l_key) ||
	(keycode == xid_cursor.left_key) || (keycode == xid_cursor.up_key) ||
	(keycode == xid_cursor.right_key) || (keycode == xid_cursor.down_key))
        {
/*
    Reset the delta X and Y amounts.
*/
	deltax = deltay = 0;

/*
    Check the keyboard state to see if the Shift key was used.
*/
	if (state == ShiftMask)
	    move = 10;		     /* Shift key so use delta of 10.	      */
	else
	    move = 1;

/*
    Set the delta X/Y amount based on the key pressed.
*/
	if ((keycode == xid_cursor.k_key) || (keycode == xid_cursor.up_key))
	    deltay -= move;
	else if ((keycode == xid_cursor.j_key) || (keycode == 
	    xid_cursor.down_key))
	    deltay += move;
	else if ((keycode == xid_cursor.h_key) || (keycode == 
	    xid_cursor.left_key))
	    deltax -= move;
	else
	    deltax += move;

/*
    If a valid move can be made then move the cursor.
*/
	if (((event->xkey.x + deltax) < 0) ||
	    ((event->xkey.x + deltax) > HISTX) || ((event->xkey.y + deltay) 
	    < 0) || ((event->xkey.y + deltay) > HISTY))
	    ;
	else
	    XWarpPointer(xid.display,None,None,None,None,None,None,
		deltax,deltay);
	}
    }

/*
    Calculate the number of histogram graphs displayed.
*/
if ((xid_histo.flags & HISTO_XZ) || (xid_histo.flags & HISTO_YZ))
    totbnd = 1;
else if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) && 
    (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
        totbnd = 1;
else
    for (i=0, totbnd=0; i < 3; i++)
	if (xid_view.rgbplanes[i] != 0)
	    totbnd++;

/*
    Calculate the gray level value corresponding to the cursor position if the
    cursor is within the bounds of the X axis of the histogram.
*/
if ((event->xmotion.x > xid_histo.xaxis_x[0]) && 
    (event->xmotion.x <= xid_histo.xaxis_x[1])) 
	gray_val = ((event->xmotion.x - 1 - xid_histo.xoffset) /
	    (float)xid_histo.xlength) * 255;
else
    return;

/*
    Set the foreground value to black.
*/
XSetForeground(xid.display,xid_histo.gc,xid.color_table[RGBBW_IDX + BLACK_COL].
    pixel);

/*
    Calculate the width and height of characters in the working font.
*/
width = (xid.font_info->max_bounds.rbearing -
    xid.font_info->min_bounds.lbearing) * maxlen;
height = xid.font_info->max_bounds.ascent +
    xid.font_info->max_bounds.descent;

/*
    Output the percentage information.
*/
if (totbnd == 1)
    {
/*
    Clear the old annotation string containing percentage information.
*/
    XClearArea(xid.display,XtWindow(histo_drawingarea),xid_histo.annot_x[0],
        xid_histo.annot_y1[0]-xid.font_info->max_bounds.ascent,width,height,
	False);
    XClearArea(xid.display,XtWindow(histo_drawingarea),xid_histo.annot_x[0],
        xid_histo.annot_y2[0]-xid.font_info->max_bounds.ascent,width,height,
	False);
/*
    Calculate the percentage of image data prior to and including the gray
    level calculated above (this is calculated from the cumulative histogram).
*/
    percent = (float)(((double)xid_histo.cum_histo[0][gray_val] / 
	(double)xid_histo.cum_histo[0][255]) * 100.0);

/*
    Output the percentage information as well as the gray level value 
    corresponding to the cursor position.
*/
    sprintf(annot,"%3.2f%% of Data",percent);
    XDrawString(xid.display,XtWindow(histo_drawingarea),xid_histo.gc,
	xid_histo.annot_x[0],xid_histo.annot_y1[0],annot,strlen(annot));
    sprintf(annot,"  at Pixel %d",gray_val);
    XDrawString(xid.display,XtWindow(histo_drawingarea),xid_histo.gc,
	xid_histo.annot_x[0],xid_histo.annot_y2[0],annot,strlen(annot));
    }
else
    {
    for (i=0; i < totbnd; i++)
	{
/*
    Clear the old annotation string containing percentage information.
*/
        XClearArea(xid.display,XtWindow(histo_drawingarea),xid_histo.annot_x[i],
            xid_histo.annot_y1[i]-xid.font_info->max_bounds.ascent,width,height,
	    False);
        XClearArea(xid.display,XtWindow(histo_drawingarea),xid_histo.annot_x[i],
            xid_histo.annot_y2[i]-xid.font_info->max_bounds.ascent,width,height,
	    False);
/*
    Calculate the percentage of image data prior to and including the gray
    level calculated above (this is calculated from the cumulative histogram).
*/
        percent = (xid_histo.cum_histo[i][gray_val] / 
	    (float)xid_histo.cum_histo[i][255]) * 100;
/*
    Output the percentage information as well as the gray level value 
    corresponding to the cursor position.
*/
        sprintf(annot,"%3.2f%% of Data",percent);
        XDrawString(xid.display,XtWindow(histo_drawingarea),xid_histo.gc,
	    xid_histo.annot_x[i],xid_histo.annot_y1[i],annot,strlen(annot));
        sprintf(annot,"  at Pixel %d",gray_val);
        XDrawString(xid.display,XtWindow(histo_drawingarea),xid_histo.gc,
	    xid_histo.annot_x[i],xid_histo.annot_y2[i],annot,strlen(annot));
	}
    }
}
