/*
 *  This is the load function source file for the load panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */

#include <math.h>
#include <string.h>
#include <X11/cursorfont.h>
#include "xid.h"
#include "wait.xbm"
#include "../xid_widgets/ProcessM.h"

void
load_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine closes the load image panel and sets the load options to 
    sensitive.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Close the file selector and set the load buttons to sensitive.
*/
XtUnmanageChild(Load_form);
XtDestroyWidget(Load_form);
XtPopdown(Shell_Load);
XtSetSensitive(main_load_las,True);
XtSetSensitive(main_load_nonlas,True);
}

void
load_localcancel(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine is called when the user selects cancel from the file
    selector and calls load_destroy to close the file selector window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
/*
    Close the file selector.
*/
load_destroy(w, client, call);
}

void
load_localok(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine retrieves the file name specified by the user, parses the
    name, changes the cursor shape to an hourglass, pops up the processing
    message dialog, loads the appropriate image to the display and updates
    any graphics information.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Modified by L. Huewe, USGS/EROS Data Center, 8/93
    - Modified to output an XID message when xid_getin fails.

Algorithm Description:
*******************************************************************************/
{
Window	imgwin;			/* Window ID for main image window.	      */

Cursor	img_cursor;		/* Cursor for image window.		      */

Pixmap	wait_pix;		/* Wait cursor pixmap.			      */

XColor	foregr,			/* Foreground color for cursor.		      */
	backgr;			/* Background color for cursor.	  	      */

void	load_makelas();		/* Routine to make a LAS specification out of
				   a nonlas (or host) specification.	      */

long    access_mode,            /* File open access mode.                     */
	cflag,                  /* Indicates whether or not a GOF was created.*/
	i,			/* Looping index.			      */
	imglimit,		/* Input image limit.		              */
	lasflg,            	/* Indicates whether a LAS name or host name 
				   was specified.			      */
	length,			/* String length.			      */
	plane_idx,		/* Index into memory plane structure.	      */
	pos,               	/* Position of a specified character in a 
				   string.				      */
	qflag[MAXIMG][2],	/* Indicates whether user specified a window
				   and band numbers for each input image.     */
	same_window,		/* Indicates if specified images have the
				   same window size.			      */
	status;			/* Status indicator.			      */

char	ext[11],		/* File extension.			      */
	*filename,		/* Input specification.		      	      */
	goftyp[6],		/* Type of GOF.				      */
	hname[CMLEN],		/* Host file name of displayed image (first
				   band).				      */
	hostnam[CMLEN],		/* Host name of the GOF.		      */
	lasspec[CMLEN],		/* User specification with LAS names.	      */
	tmpname[3][CMLEN];    	/* Temporary storage for input name.          */


char    *left,             	/* Location of the left paren.      	      */
        *right,            	/* Location of the right paren.     	      */
        *semi;             	/* Location of the semi colon.      	      */

XmFileSelectionBoxCallbackStruct *acs =
	(XmFileSelectionBoxCallbackStruct*)call;

/*
    Retrieve the file name selected and parse it.  This determines its LAS
    name (if applicable), host file name, window specified and bands specified.
*/
if (xid_load.type == LAS)
    {
    XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);
    strcpy(xid_load.image_name,filename);
    if (xid_load.dirmask != NULL)
	XmStringFree(xid_load.dirmask);
    xid_load.dirmask = XmStringCopy(acs->mask);
    XtFree(filename);
    }

/* 
   Determine whether the input name is a LAS name or a host name.
   - If the name contains a bracket ([) or a semicolon (;), it is assumed
     to be a LAS name.
   - If the name contains a slash (/), it is assumed to be a host name.
*/
lasflg = -1;

if (strcspn(xid_load.image_name,"[") != strlen(xid_load.image_name))
    lasflg = TRUE;
else if (strcspn(xid_load.image_name,"/") != strlen(xid_load.image_name))
    lasflg = FALSE;
else if (strcspn(xid_load.image_name,";") != strlen(xid_load.image_name))
    {
    left = strchr(xid_load.image_name,'(');
    right = strchr(xid_load.image_name,')');
    semi = strchr(xid_load.image_name,';');
    if ((left == NULL) || (right == NULL))
        lasflg = TRUE;
    else if (semi > right)
	lasflg = TRUE;
    }

pos = strcspn(xid_load.image_name,"(");
if (pos != strlen(xid_load.image_name))
    {
    strncpy(tmpname[0],xid_load.image_name,pos);
    tmpname[0][pos] = '\0';
    }
else 
    strcpy(tmpname[0],xid_load.image_name);

if (lasflg == -1)
    {
    if (access(tmpname[0],0) != 0)
	lasflg = TRUE;
    else
	lasflg = FALSE;
    }

/*
   If loading a LAS image and the input specification was made with host
   names, convert the image names to LAS names.
*/
if ((xid_load.type == LAS) && (!lasflg))
    load_makelas(xid_load.image_name,lasspec);
else
    strcpy(lasspec,xid_load.image_name);

/*
    Parse the image name, window and band specifications.
*/
imglimit = xid.max_plane;
if (xid_load.type == LAS)
    status = c_getin2(lasspec,xid_load.tnames,xid_load.hnames,
	&xid_load.nimgs,xid_load.windows,xid_load.bands,xid_load.nbands,
	&xid_load.totbnd,&imglimit,&same_window,qflag);
else
    status = xid_getin(xid_load.image_name,xid_load.tnames,xid_load.hnames,
	&xid_load.nimgs,xid_load.windows,xid_load.bands,xid_load.nbands,
	&xid_load.totbnd,&imglimit,&same_window,qflag);
if (status != E_SUCC)
    {
    xid_message("Error parsing image specification");
    return;
    }
/*
    In order to display images from separate files as one image, they must
    all be the same size.
*/
if (!same_window)
    {
    xid_message("Windows of input images are not the same size");
    return;
    }

/*
    Display the wait cursor (hourglass) in both the image window and main
    panel.
*/
imgwin = XtWindow(main_image_bulletinboard);
wait_pix = XCreatePixmapFromBitmapData(xid.display,imgwin,wait_bits,wait_width,
    wait_height,xid.color_table[RGBBW_IDX + WHITE_COL].pixel,
    xid.color_table[RGBBW_IDX + BLACK_COL].pixel,1);
XParseColor(xid.display,xid.colormap,"black",&foregr);
XParseColor(xid.display,xid.colormap,"white",&backgr);
img_cursor = XCreatePixmapCursor(xid.display,wait_pix,wait_pix,
    &foregr,&backgr,1,1);
XDefineCursor(xid.display,imgwin,img_cursor);
XDefineCursor(xid.display,XtWindow(main_form),img_cursor);

/*
    Set the title for the processing message and processing icon.
*/
XtVaSetValues(load_dialog_shell,XmNtitle,"Load - Processing",NULL);

/*
    Close the file selection box.
*/
load_destroy(w, client, call);

/*
    THIS IS TEMPORARY UNTIL THE PROJ INFO PANEL IS IMPLEMENTED.
*/
xid_projinfo.pdist_y = xid_projinfo.pdist_x = -999;
xid_projinfo.line_inc = xid_projinfo.sample_inc = -999;
*xid_projinfo.proj_units = '\0';
	    
/*
    Load the specified image.
*/
xid.view = -1;
loadimg(imgwin,TRUE);

/*
    Redisplay the default arrow cursor.
*/
XFreeCursor(xid.display,img_cursor);
img_cursor = XCreateFontCursor(xid.display,XC_top_left_arrow);
XDefineCursor(xid.display,imgwin,img_cursor);
XDefineCursor(xid.display,XtWindow(main_form),img_cursor);

/*
    Turn on the bell to let the user know the image load is complete.
*/
XBell(xid.display,0);

/*
    Update the graphics information.
*/
if (xid_graphics.pixmap[0] != NULL)
    {
    XSetBackground(xid.display,xid_graphics.rb_gc,0);
    XSetForeground(xid.display,xid_graphics.rb_gc,0);
    for (i=0; i < USER_GRCOLORS; i++)
	XFillRectangle(xid.display,xid_graphics.pixmap[i],xid_graphics.rb_gc,
	    0,0,xid_mem.mem_width,xid_mem.mem_height);
    XSetBackground(xid.display,xid_graphics.rb_gc,1);
    XSetForeground(xid.display,xid_graphics.rb_gc,1);
    plane_idx = xid_view.rgbplanes[RED] - 1;
    graphics_update(0,plane_idx);
    }

/*
    If the graphics panel is displayed, update the GOF name.
*/
if ((xid_graphics.flags & GR_ACTIVE) && (xid_graphics.type != NO_TYPE))
    {
/*
    Write the current record and close the current GOF.
*/

    if ((xid_graphics.type != NO_TYPE) && (xid_graphics.flags & GR_SELECT))
        {
        if (!(xid_element[xid_graphics.current].writeflg))
	    {
	    status = graphics_gofwrite();
	    if (status != XID_S)
	        return;
	    xid_element[xid_graphics.current].writeflg = TRUE;
	    }
        if (xid_graphics.fd != -1)
            c_gofcls(&xid_graphics.fd);
        }

    switch (xid_graphics.type)
	{
	case POINT:
	    strcpy(goftyp,"POINT");
	    break;
	case LINE:
	    strcpy(goftyp,"LINE");
	    break;
	case RECT:
	case POLY:
	    strcpy(goftyp,"POLY");
	    break;
	case TEXT:
	    strcpy(goftyp,"ANNOT");
	    break;
	}
/*
    Determine the default name for the GOF based on the first band of the
    displayed image.
*/
    plane_idx = xid_view.rgbplanes[RED] - 1;
    if (xid_planes[plane_idx].image_type == LAS)
	{
	status = c_cmhost(xid_planes[plane_idx].filename,hname);
	if (status != E_SUCC)
	    {
	    xid_message("Error retrieving host file name");
	    return;
	    }
	}
    else
	strcpy(hname,xid_planes[plane_idx].filename);

    if ((xid_graphics.flags & GR_SELECT) || (xid_graphics.flags & GR_LOAD) ||
	(xid_graphics.flags & GR_EDIT))
	{
	if ((xid_graphics.flags & GR_SELECT) || (xid_graphics.flags & GR_EDIT))
	    access_mode = APPEND;
	else
	    access_mode = READ;
	status = c_gofopn(&xid_graphics.fd,hname,xid_graphics.lname_gof,goftyp,
	    &access_mode,&cflag);
	if (status != E_SUCC)
	    {
	    xid_message("Error opening GOF");
	    xid_graphics.fd = -1;
	    }
    	if (xid_planes[plane_idx].image_type != LAS)
	    strcpy(xid_graphics.lname_gof,hname);
	}
/*
   Set up the GOF extension based on the type specified.
*/
    else
	{
	sprintf(ext,"GOF%c%s",GOF_SEP,goftyp);
	length = strlen(ext);
	c_up2low(ext,&length);

/*
   Get the associated file name.
*/
	c_lsmknm(hname,ext,hostnam);
	if (xid_planes[plane_idx].image_type == LAS)
	    c_cmtae(hostnam,xid_graphics.lname_gof);
	else
	    strcpy(xid_graphics.lname_gof,hostnam);
	}
    XmTextSetString(graphics_gof_text,xid_graphics.lname_gof);
    }

/*
    Free resources.
*/ 
XFreeCursor(xid.display,img_cursor);
XFreePixmap(xid.display,wait_pix);
XmUpdateDisplay(Shell_MainMenu);

/*
    Close the nonlas information panel.
*/
if (xid_load.type != LAS)
    nonlas_destroy(w,client,call);
}

FUNCTION load_display(rgbmem,nplanes,imgwin,shell,win,nl,ns,maxval,rptr,gptr,
    bptr,cmflg,pixptr,loadflg,viewflg,flickerflg)

Widget	shell,			/* Widget ID of image window shell (I).	      */
        win;			/* Widget ID of image window (I).	      */

Window  imgwin;                 /* Window ID for main image window (I).       */

Pixmap	*pixptr;		/* Pointer to pixmap which is to contain the
				   loaded image data (I).		      */

long    cmflg,			/* Indicates whether or not color map should
				   be modified (I).			      */
	flickerflg,		/* Indicates whether or not this load is part
				   of the flicker process (I).		      */
	loadflg,		/* Indicates whether this is being called from
				   the loadimg routine (I).		      */
	*maxval,		/* Maximum image data value (I).	      */
	*nl,			/* Number of lines to load (I).	   	      */
	*ns,			/* Number of samples to load (I).	      */
	*nplanes,		/* Number of planes loaded (I).	      	      */
	*rgbmem,		/* Memory planes to be loaded with image 
				   data (I).				      */
	viewflg;		/* If TRUE, resize viewport to original
				   specification.  If FALSE, base resizing on
				   the current viewport width and height.  (I)*/

unsigned char
	*rptr,			/* Pointer to red image data (I).	      */
	*gptr,			/* Pointer to green image data (I).	      */
	*bptr;			/* Pointer to blue image data (I).	      */

/*******************************************************************************
Purpose:
    This routine outputs the input image data to the current display.  The
    appropriate colormap is defined and installed based on the display type.
    The image data is actually loaded into a pixmap which is used to set
    the background of the main image window.  Load_display also clears any
    currently displayed zoomed or full resolution windows, verifies whether
    or not the profile and cursor Z options are valid and updates the plane
    list displayed by the show function.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
extern void xid_image_makelist();
				/* Function to build a list of plane numbers. */

Arg     args[4];                /* Variable used to set widget resources.     */

Widget	widget;			/* Widget ID.				      */

GC      mask_gc,		/* GC for displaying graphics masks.	      */
	rgb_gc;                 /* Working color GC.                          */

XGCValues
	gc_val;                 /* GC set value array.                        */

XImage  *ximage;                /* X image structure.                         */

Dimension 
	view_height,		/* Viewport height.			      */
	view_width;		/* Viewport width.			      */

static unsigned char
	rr[256],                /* Reserved color values.                     */
	gr[256],
	br[256];

float	scaler;                 /* LUT scaler for actual number of colors.    */

long	buf_size,               /* Number of bytes in a single band.          */
	count,			/* Number of active planes.		      */
	i,			/* Looping index.			      */
	img_idx,		/* Index into colormap of first image color.  */
	num_imgcolors,		/* Number of image colors.		      */
	offset,                 /* Offset used in scaling the 24-bit LUT.     */
	show_flg,		/* Indicates whether or not the show panel
				   is active.				      */
	state,			/* Toggle button state.			      */
	tform[256];		/* Pixel transform array.		      */

unsigned long
	*data;                	/* Pointer to output values for 24-bit data.  */

unsigned char
        *blut,                  /* Blue LUT for composite image.              */
	*composite,             /* Composite RGB 8-bit image or single band
				   black white 8-bit image.                   */
        *comptr,                /* Pointer to composite buffer.               */
	*dataptr,               /* Pointer to image data stored in memory
				   plane.                                     */
	*glut,                  /* Green LUT for composite image.             */
	*rgbptr[3],             /* Pointers to RGB components.                */
	*rlut;                  /* Red LUT for composite image.               */

char    fullres_opt[4],		/* Current full resolution size option.	      */
	points[10];             /* Number of points string.	   	      */

/*
    Calculate the image data buffer size.
*/
buf_size = *nl * *ns;

/*
    Calculate the scaler needed to reduce the image to the appropriate number
    of colors.
*/
if ((*xid.colormap_ptr == xid.colormap) && (*maxval < NUM_IMGCOLORS) &&
    (*nplanes == 1))
    {
    scaler = 1.0;
    num_imgcolors = NUM_IMGCOLORS;
    img_idx = IMG_IDX;
    }
else if (*xid.colormap_ptr == xid.colormap)
    {
    scaler = (256.0 / (float)NUM_IMGCOLORS);       /* (2^8 / # of colors) */
    num_imgcolors = NUM_IMGCOLORS;
    img_idx = IMG_IDX;
    }
else 
    {
    scaler = 1.0;
    num_imgcolors = 256;
    img_idx = 0;
    }
/*
    8-bit monitor -- black and white image.
*/
if ((xid.visual_info[0].visual->class == PseudoColor) && (*nplanes == 1))
    {
    if (!flickerflg)
	{
	comptr = composite = (unsigned char *)malloc(buf_size);
	if (composite == NULL)
	    {
	    xid_message("Insufficient memory available");
	    return(XID_F);
	    }
	}
/*
    Create a linear ramp.
*/
    if (cmflg)
	{
        for (i = 0; i < num_imgcolors; i++)
            {
	    if (!flickerflg)
		xid.colortable_ptr[img_idx + i].blue = xid.colortable_ptr[
		    img_idx + i].green = xid.colortable_ptr[img_idx + i].red = 
		    ((i * scaler) * 257);
	    else
		{
		xid.colortable_ptr[img_idx + i].blue = 
		    xid_flicker.colortable[xid_flicker.current_image][img_idx 
		    + i].blue;
		xid.colortable_ptr[img_idx + i].green = 
		    xid_flicker.colortable[xid_flicker.current_image][img_idx
		    + i].green;
		xid.colortable_ptr[img_idx + i].red = 
		    xid_flicker.colortable[xid_flicker.current_image][img_idx 
		    + i].red;
		}
            xid.color_linear[img_idx + i].blue = xid.color_linear[img_idx + i].
		green = xid.color_linear[img_idx + i].red = ((i * scaler) * 
		257);
            xid.color_linear[img_idx + i].flags =
		xid.colortable_ptr[img_idx + i].flags = DoRed | DoGreen | 
		DoBlue;
            }
	}
/*
    Scale the input data.
*/
    if (!flickerflg)
	{
	dataptr = rptr;
	for (i=0; i < buf_size; i++,comptr++,dataptr++)
	    *comptr = xid.colortable_ptr[(unsigned char) (img_idx + ((float) 
		(*dataptr / scaler)))].pixel; 
	}
/*
    Store the new colormap.
*/
    if (cmflg)
        XStoreColors(xid.display,*xid.colormap_ptr,&xid.colortable_ptr[img_idx],
	    num_imgcolors);
    }
/*
    8-bit monitor -- RGB image.
*/ 
else if (xid.visual_info[0].visual->class == PseudoColor)
    {
/*
    Allocate buffer space for LUTs.
*/
    rlut = (unsigned char *)malloc(256);
    if (rlut == NULL)
	{
	xid_message("Insufficient memory available");
	return(XID_F);
	}
    glut = (unsigned char *)malloc(256);
    if (glut == NULL)
	{
	xid_message("Insufficient memory available");
	free(rlut);
	return(XID_F);
	}
    blut = (unsigned char *)malloc(256);
    if (blut == NULL)
	{
	xid_message("Insufficient memory available");
	free(rlut);
	free(glut);
	return(XID_F);
	}
/*
    Compress the 3 band RGB image into a single band of image data and
    3 LUTs.
*/
    comptr = composite = (unsigned char *)malloc(buf_size);
    if (composite == NULL)
	{
	xid_message("Insufficient memory available");
	free(rlut);
	free(glut);
	free(blut);
	return(XID_F);
	}
    rgbptr[RED] = rptr; 
    rgbptr[GREEN] = gptr; 
    rgbptr[BLUE] = bptr; 
    if (cmflg)
	{
        xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
            num_imgcolors,&composite,rlut,glut,blut,0,rr,gr,br);
/*
    Save this LUT to be used as reserved colors for loading of full
    resolution windows.
*/
	for (i=0; i < 256; i++)
	    {
	    rr[i] = rlut[i];
	    gr[i] = glut[i];
	    br[i] = blut[i];
	    }
	}
    else
        xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
            num_imgcolors,&composite,rlut,glut,blut,num_imgcolors,rr,gr,br);

/*
    Build the new colormap.
*/
    if (cmflg)
	{
        for (i = 0; i < num_imgcolors; i++)
            {
	    if (!flickerflg)
		{
		xid.color_linear[img_idx + i].red = 
		    xid.colortable_ptr[img_idx + i].red = (rlut[i] * 257);
		xid.color_linear[img_idx + i].green =
		    xid.colortable_ptr[img_idx + i].green = (glut[i] * 257);
		xid.color_linear[img_idx + i].blue =
		    xid.colortable_ptr[img_idx + i].blue = (blut[i] * 257);
		}
	    else
		{
		xid.color_linear[img_idx + i].red = (rlut[i] * 257);
		xid.color_linear[img_idx + i].green = (glut[i] * 257);
		xid.color_linear[img_idx + i].blue = (blut[i] * 257);
		xid.colortable_ptr[img_idx + i].red = xid_flicker.colortable[
		    xid_flicker.current_image][img_idx + i].red;
		xid.colortable_ptr[img_idx + i].green = xid_flicker.colortable[
		    xid_flicker.current_image][img_idx + i].green;
		xid.colortable_ptr[img_idx + i].blue = xid_flicker.colortable[
		    xid_flicker.current_image][img_idx + i].blue;
		}
            xid.color_linear[img_idx + i].flags =
		xid.colortable_ptr[img_idx + i].flags = DoRed | DoGreen 
		| DoBlue;
            }
	}

    if (!flickerflg)
        for (i=0; i < buf_size; i++)
	    composite[i] = xid.colortable_ptr[img_idx + composite[i]].pixel;
/*
    Store the new colormap.
*/
    if (cmflg)
        XStoreColors(xid.display,*xid.colormap_ptr,&xid.colortable_ptr[img_idx],
	    num_imgcolors);
    }
else if ((xid.visual_info[0].visual->class == DirectColor) && (*nplanes == 1))
    {
/*
    Allocate buffer space.
*/
    if (!flickerflg)
	{
	data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
	if (data == NULL)
	    {
	    xid_message("Insufficient memory available");
	    return(XID_F);
	    }

/*
    Set a pointer to the image data.
*/
        rgbptr[RED] = rptr; 

/*
    Create the transform array which scales output values such that 56 colormap
    entries can be reserved as working entries.
*/
	for (i = 0; i < 256; i++)
	    tform[i] = (long)(((float)i / scaler) + img_idx);
/*
    Scale the input data.
*/
	for (i = 0; i < buf_size; i++, rgbptr[RED]++)
	    data[i] = 0 + (tform[(*rgbptr[RED])]) + (tform[(*rgbptr[RED])] << 8)
		+ (tform[(*rgbptr[RED])] << 16);
	}
/*
    Assign a gray scale ramp to the image data colors.
*/
    if (*xid.colormap_ptr == xid.colormap)
        offset = (0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE + NUM_WMCOLORS 
	+ NUM_WKCOLOR)));
    else
	offset = 0;

    if (cmflg)
	{
        for (i = img_idx; i < num_imgcolors + img_idx; i++)
	    {
	    if (!flickerflg)
	        xid.colortable_ptr[i].red = xid.colortable_ptr[i].green = 
	            xid.colortable_ptr[i].blue = 
	            (long)((float)((i * scaler) + offset)) << 8;
	    else
		{
		xid.colortable_ptr[i].red = xid_flicker.colortable[xid_flicker.
		    current_image][i].red;
		xid.colortable_ptr[i].green = xid_flicker.colortable[
		    xid_flicker.current_image][i].green;
		xid.colortable_ptr[i].blue  = xid_flicker.colortable[
		    xid_flicker.current_image][i].blue;
		}
	    xid.color_linear[i].red = xid.color_linear[i].green = 
	        xid.color_linear[i].blue = 
	        (long)((float)((i * scaler) + offset)) << 8;
	    xid.color_linear[i].flags = 
		xid.colortable_ptr[i].flags = DoRed | DoGreen | DoBlue;
	    }
        XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);
	}
    }
else if (xid.visual_info[0].visual->class == DirectColor)
    {
/*
    Allocate buffer space.
*/
    if (!flickerflg)
	{
	data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
	if (data == NULL)
	    {
	    xid_message("Insufficient memory available");
	    return(XID_F);
	    }
/*
    Set pointers to the red, green and blue bands of image data.
*/
	rgbptr[RED] = rptr;
	rgbptr[GREEN] = gptr; 
	rgbptr[BLUE] = bptr; 

/*
    Create the transform array which scales output values such that 56 colormap
    entries can be reserved as working entries.
*/
	for (i = 0; i < 256; i++)
	    tform[i] = (long)(((float)i / scaler) + img_idx);
/*
    Scale the input data.
*/
	for (i = 0; i < buf_size; i++, rgbptr[RED]++, rgbptr[GREEN]++, 
	    rgbptr[BLUE]++)
	    data[i] = 0 + (tform[(*rgbptr[RED])]) + (tform[(*rgbptr[GREEN])] 
		<< 8) + (tform[(*rgbptr[BLUE])] << 16);
	}
/*
    Assign a gray scale ramp to the image data colors.
*/
    if (*xid.colormap_ptr == xid.colormap)
        offset = (0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE + NUM_WMCOLORS 
	+ 1)));
    else
	offset = 0;

    if (cmflg)
	{
        for (i = img_idx; i < (img_idx + num_imgcolors); i++)
	    {
	    if (!flickerflg)
		xid.colortable_ptr[i].red = xid.colortable_ptr[i].green = 
		    xid.colortable_ptr[i].blue =
		    (long)((float)((i * scaler) + offset)) << 8;
	    else
		{
		xid.colortable_ptr[i].red = xid_flicker.colortable[xid_flicker.
		    current_image][i].red;
		xid.colortable_ptr[i].green = xid_flicker.colortable[
		    xid_flicker.current_image][i].green;
		xid.colortable_ptr[i].blue  = xid_flicker.colortable[
		    xid_flicker.current_image][i].blue;
		}
	    xid.color_linear[i].red = xid.color_linear[i].green = 
	        xid.color_linear[i].blue =
	        (long)((float)((i * scaler) + offset)) << 8;
	    xid.color_linear[i].flags =
		xid.colortable_ptr[i].flags = DoRed | DoGreen | DoBlue;
	    }
        XStoreColors(xid.display,*xid.colormap_ptr,xid.colortable_ptr,256);
	}
    }

/*
    Create a graphics context for the image window.
*/
gc_val.function = GXcopy;
gc_val.plane_mask = AllPlanes;

rgb_gc = XCreateGC(xid.display,imgwin,GCFunction | GCPlaneMask,&gc_val);

/*
    Allocate buffer space.
*/
if (!flickerflg)
    {
    if ((xid.visual_info[0].visual->class == PseudoColor) &&
	(xid.visual_info[0].depth == 24))
	{
	data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
	if (data == NULL)
	    {
	    xid_message("Insufficient memory available");
	    return(XID_F);
	    }
	for (i=0; i < buf_size; i++)
	    data[i] = composite[i];
	}
/*
    Create an X image.
*/
    if (xid.visual_info[0].depth == 24)
	ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
	    xid.visual_info[0].depth,ZPixmap,0,(char *)data,*ns,*nl,8,0);
    else
	ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
	    xid.visual_info[0].depth,ZPixmap,0,(char *)composite,*ns,*nl,8,0);
/*
    Create a pixmap for the image window.
*/
    *pixptr = XCreatePixmap(xid.display,imgwin,*ns,*nl,xid.visual_info[0].
	depth);
    if (loadflg)
	xid_mem.imgpix = TRUE;
/*
    Draw the image into the image pixmap.
*/
    XPutImage(xid.display,*pixptr,rgb_gc,ximage,0,0,0,0,*ns,*nl);
    }

/*
    Set the current width and height and the minimum and maximum width and
    height of the image window bulletinboard.
*/
XtSetArg(args[0],XmNwidth,*ns);
XtSetArg(args[1],XmNheight,*nl);
XtSetArg(args[2],XmNmaxWidth,*ns);
XtSetArg(args[3],XmNmaxHeight,*nl);
XtSetValues(win,args,4);
XFlush(xid.display);
XtResizeWidget(win,*ns,*nl,0);

/*
    The following size checks and calculations take into an account an 
    offset which represents the width and height of the scrollbar region.
*/

if (viewflg)
    {
    view_width = xid_mem.view_width;
    view_height = xid_mem.view_height;
    }
else
    {
    XtVaGetValues(shell,XmNwidth,&view_width,XmNheight,&view_height,NULL); 
    view_width -= xid_mem.view_xoffset;
    view_height -= xid_mem.view_yoffset;
    }

/*
    If the viewport width is larger than full image width, set the current
    image window width to the full image width. 
*/
if (view_width > *ns)
    XtSetArg(args[0],XmNwidth,*ns + xid_mem.view_xoffset);
else
/*
    Set the current image window width based on the viewport width.
*/
    XtSetArg(args[0],XmNwidth,view_width+xid_mem.view_xoffset);

/*
    If the viewport height is larger than full image height, set the current
    image window height to the full image height. 
*/
if (view_height > *nl)
    XtSetArg(args[1],XmNheight,*nl + xid_mem.view_yoffset);
else
/*
    Set the current image window height based on the viewport height.
*/
    XtSetArg(args[1],XmNheight,view_height+xid_mem.view_yoffset);

/*
    Set the maximum image window width and height based on the full image
    size.
*/
XtSetArg(args[2],XmNmaxWidth,*ns + xid_mem.view_xoffset);
XtSetArg(args[3],XmNmaxHeight,*nl + xid_mem.view_yoffset);
XtSetValues(shell,args,4);
XFlush(xid.display);

/*
    For main image window only, copy any graphics plane masks currently defined.
*/
if (win == main_image_bulletinboard)
    {
/*
    Create the mask graphics context.
*/
    mask_gc = XCreateGC(xid.display, XtWindow(Shell_MainMenu), NULL, NULL);
    for (i=0; i < MASK_MAX; i++)
	if (xid_mask.mask_pixmap[i] != NULL)
	    {
	    XSetBackground(xid.display,mask_gc,xid.color_table[RGBBW_IDX + 
		xid_mask.color[i]].pixel);
	    XSetForeground(xid.display,mask_gc,xid.color_table[RGBBW_IDX + 
		xid_mask.color[i]].pixel);
	    XSetClipMask(xid.display,mask_gc,xid_mask.mask_pixmap[i]);
	    XCopyPlane(xid.display,xid_mask.mask_pixmap[i],*pixptr,
		mask_gc,0,0,xid_mem.mem_width,xid_mem.mem_height,0,0,1);
	    }
/*
    Free the graphics context.
*/
    XFreeGC(xid.display, mask_gc);
    }

/*
    Set the image pixmap to be the background of the image window.
*/
XSetWindowBackgroundPixmap(xid.display,imgwin,*pixptr);

/*
    Clear the window to update the background.
*/
XClearArea(xid.display,imgwin,0,0,0,0,True);

/*
    Store the appropriate information in the composite view structure and
    reset flags and values necessary when loading a new image.
*/
if (loadflg)
    {
/*
    Reset pseudo-color flags and free color table.
*/
    xid_psd.flags = (xid_psd.flags  & ~PSD_NOINIT) | PSD_INIT;
    if (xid_psd.color_psd != NULL)
	{
	free(xid_psd.color_psd);
	xid_psd.color_psd = (XColor *)NULL;
        XmTextSetString(psd_def_text,"");
	}
/*
    Reset slope and offset and free adjusted color table and linear color table.
*/
    xid_adjust.lut_slope = 45.0;
    xid_adjust.lut_offset = 0.0;
    if (xid_adjust.color_adjust != NULL)
	{
	free(xid_adjust.color_adjust);
	xid_adjust.color_adjust = (XColor *)NULL;
	}
    if (xid_adjust.color_linear != NULL)
	{
	free(xid_adjust.color_linear);
	xid_adjust.color_linear = (XColor *)NULL;
	}
/*
    Determine whether or not the profile and cursor Z options are valid. 
*/
    if ((xid_profile.flags & PROF_ACTIVE) || (xid_cursor.flags & CURSOR_ACTIVE))
	{
	for (i=0, count=0; i < xid.max_plane; i++)
	    if (xid_planes[i].flags & USED)
		count++;
	if (count < 2)
	    {
	    if (xid_profile.flags & PROF_ACTIVE)
		XtSetSensitive(profile_z_button,False);
	    if (xid_cursor.flags & CURSOR_ACTIVE)
		XtSetSensitive(cursor_z_button,False);
	    }
	else
	    {
	    if (xid_profile.flags & PROF_ACTIVE)
		XtSetSensitive(profile_z_button,True);
	    if (xid_cursor.flags & CURSOR_ACTIVE)
		XtSetSensitive(cursor_z_button,True);
	    }
	}

/*
    Set the composite structure fields to reflect the currently displayed
    image.
*/
    xid.line_x = xid.line_y = -1;
    xid.view = 0;
    if (*nplanes >= 3)
	{
	xid_view.rgbplanes[RED] = rgbmem[RED];
	xid_view.rgbplanes[GREEN] = rgbmem[GREEN];
	xid_view.rgbplanes[BLUE] = rgbmem[BLUE];
	}
    else if (*nplanes == 1)
	{
	xid_view.rgbplanes[RED] = rgbmem[RED];
	xid_view.rgbplanes[GREEN] = rgbmem[RED];
	xid_view.rgbplanes[BLUE] = rgbmem[RED];
	}
    else if (*nplanes == 2)
	{
	xid_view.rgbplanes[RED] = rgbmem[RED];
	xid_view.rgbplanes[GREEN] = rgbmem[GREEN];
	xid_view.rgbplanes[BLUE] = rgbmem[GREEN];
	}
/*
    If the zoom window is currently active, clear the window and free its
    resources (the zoomed information is no longer valid because a new
    image has been loaded).
*/
    if (xid_zoom.flags & ZOOM_ACTIVE)
	{
        XSetWindowBackground(xid.display,XtWindow(zoom_win_bulletinboard),
	    xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
	XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));
	if (xid_zoom.image_pixmap != NULL)
	    {
	    XFreePixmap(xid.display,xid_zoom.image_pixmap);
	    xid_zoom.image_pixmap = (Pixmap)NULL;
	    }
	if (xid_zoom.image_data != NULL)
	    {
	    free(xid_zoom.image_data);
	    xid_zoom.image_data = NULL;
	    }
	}
/*
    If the full resolution window is currently active, clear the window and 
    free its resources (the full resolution information is no longer valid 
    because a new image has been loaded).
*/
    if (xid_fullres != NULL)
	{
        XSetWindowBackground(xid.display,XtWindow(fullres_win_bulletinboard),
	    xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
	XClearWindow(xid.display,XtWindow(fullres_win_bulletinboard));
/*
    Before freeing the full resolution structure, save the current size
    information.
*/
	state = XmToggleButtonGetState(fullres_256_button);
	if (state)
	    {
	    strcpy(fullres_opt,"256");
	    widget = fullres_256_button;
	    }
	else
	    {
	    state = XmToggleButtonGetState(fullres_512_button);
	    if (state)
		{
	        strcpy(fullres_opt,"512");
		widget = fullres_512_button;
		}
	    else
		{
	        state = XmToggleButtonGetState(fullres_full_button);
		if (state)
		    {
		    strcpy(fullres_opt,"FS");
		    widget = fullres_full_button;
		    }
		else
		    {
	            state = XmToggleButtonGetState(fullres_user_button);
		    if (state)
			{
		        strcpy(fullres_opt,"US");
			widget = fullres_user_button;
			}
		    }
		}
	    }
	for (i=0; i < xid.num_fullres; i++)
	    {
	    if (xid_fullres[i].image_pixmap != NULL)
		{
		XFreePixmap(xid.display,xid_fullres[i].image_pixmap);
		xid_fullres[i].image_pixmap = (Pixmap)NULL;
		}
	    if (xid_fullres[i].image_data != NULL)
		{
		free(xid_fullres[i].image_data);
		xid_fullres[i].image_data = NULL;
		}
	    }
	free(xid_fullres);
	xid_fullres = (struct XID_FULLRES *)NULL;
/*
    Set appropriate variables and flags and allocate structures for full
    resolution window.
*/
	fullres_setup();
	fullres_size(widget,fullres_opt,NULL);
	}

    if (!flickerflg)
	{
/*
    If the show panel is open, update the plane list.
*/
	show_flg = XtIsRealized(Shell_Show);
	if (show_flg)
	    xid_image_makelist(show_redlist,show_greenlist,show_bluelist,
		show_savelist,TRUE);
/*
    If the flicker panel is open, update the plane list.
*/
	if (xid_flicker.flags & FLICKER_ACTIVE)
	    xid_image_makelist(flicker_redlist,flicker_greenlist,
	    flicker_bluelist,flicker_savelist,FALSE);
	}
/*
    If the profile window is active, clear the current profile and redraw the
    profile axis.  This needs to be done because the newly loaded image may
    not be the same size as the previous image.
*/
    if (xid_profile.flags & PROF_ACTIVE)
	{
/*
    Free the appropriate resources.
*/
	XFreeGC(xid.display,xid_profile.gc);
	XFreePixmap(xid.display,xid_profile.info_pixmap);
	XFreePixmap(xid.display,xid_profile.clear_pixmap);

	free(xid_profile.points[RED]);
	free(xid_profile.points[GREEN]);
	free(xid_profile.points[BLUE]);

/*
    Clear the window and create the new profile.
*/
	XClearWindow(xid.display,XtWindow(profile_scrollwin_bulletinboard));
	profile_create();
	XCopyArea(xid.display,xid_profile.info_pixmap,
	    XtWindow(profile_scrollwin_bulletinboard),xid_profile.gc,0,0,
	    xid_profile.width,xid_profile.height,0,0);
	}
/*
    If the power spectrum window is active, clear the current plot and redraw. 
    This needs to be done because the newly loaded image may not be the same 
    size as the previous image.
*/
    if (xid_power.flags & POWER_ACTIVE)
	{
/*
    Free the appropriate resources.
*/
	XFreeGC(xid.display,xid_power.gc);
	XFreePixmap(xid.display,xid_power.info_pixmap);
	XFreePixmap(xid.display,xid_power.clear_pixmap);

	free(xid_power.ent_points);
	free(xid_power.per_points);

	xid_power.maxdb = (float)(20.0 * log10((double)(255.0 * 255.0 * 
	    (*ns))) + 10);

	xid_power.per_npts = (long)(nearest_power2(*ns) / 2.0);
	sprintf(points,"%d",xid_power.per_npts);
	XmTextFieldSetString(power_ppoints_text,points);
/*
   If both the entropy plot and the periodogram are being displayed, force the 
   maximum entropy start frequency, end frequency and number of points to be 
   the same as for the periodogram.
*/
	if (xid_power.flags & POWER_PER)
	    {
	    xid_power.ent_npts = xid_power.per_npts;
	    sprintf(points,"%d",xid_power.per_npts);
	    XmTextFieldSetString(power_epoints_text,points);
	    }
	else
	    {
	    xid_power.ent_npts = (long)(*ns / 2.0);
	    sprintf(points,"%d",xid_power.ent_npts);
	    XmTextFieldSetString(power_epoints_text,points);
	    }
/*
    Clear the window and create the new plot.
*/
	XClearWindow(xid.display,XtWindow(power_scrollwin_bulletinboard));
	power_create();
	XCopyArea(xid.display,xid_power.info_pixmap,
	    XtWindow(power_scrollwin_bulletinboard),xid_power.gc,0,0,
	    xid_power.width,xid_power.height,0,0);
	}
    }

/*
    Free the appropriate resources.
*/
if ((xid.visual_info[0].visual->class == DirectColor) && !flickerflg)
    free(data);
else if (xid.visual_info[0].visual->class == PseudoColor)
    {
    if ((!flickerflg) || (flickerflg && *nplanes == 3))
        free(composite);
    if (*nplanes != 1)
	{
	free(rlut);
	free(glut);
	free(blut);
	}
    }
/*
    The data in the image structure has already been freed above.
*/
if (!flickerflg)
    {
    ximage->data = NULL;
    XDestroyImage(ximage);
    }
XFreeGC(xid.display,rgb_gc);
return(XID_S);
}

void
nonlas_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine closes and destroys the load nonLAS image panel and sets the 
    file selection box to sensitive.
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
*******************************************************************************/
{
/*
    Close the image information panel and set the nonLAS load button to 
    sensitive.
*/
XtPopdown(load_nonlas_dialog);
XtUnmanageChild(nonlas_form);
XtDestroyWidget(load_nonlas_dialog);
XtSetSensitive(load_fileselector,True);
}

/* ARGSUSED */
void
nonlas_apply(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
	Retrieve the image information specified by the user and verify that
	the appropriate information was specified.  Close the nonLAS dialog
	and call the routine to load the image to the display.
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
*******************************************************************************/
{
long	state;			/* Current toggle button state.		      */

char    *tmpstr;                /* Temporary storage for a text string.       */

/*
    Retrieve the number of header bytes, number of bands, number of lines,
    and number of samples in the image.
*/
tmpstr = XmTextFieldGetString(nonlas_sizehdr_text);
if (*tmpstr == '\0')
    {
    xid_message("User must specify number of header bytes");
    return;
    }
sscanf(tmpstr,"%d",&xid_nonlas.header_bytes);
XtFree(tmpstr);

tmpstr = XmTextFieldGetString(nonlas_bands_text);
if (*tmpstr == '\0')
    {
    xid_message("User must specify number of bands");
    return;
    }
sscanf(tmpstr,"%d",&xid_nonlas.nbands);
if (xid_nonlas.nbands == 0)
    {
    xid_message("Image may not contain zero bands");
    return;
    }
XtFree(tmpstr);

tmpstr = XmTextFieldGetString(nonlas_lines_text);
if (*tmpstr == '\0')
    {
    xid_message("User must specify number of lines");
    return;
    }
sscanf(tmpstr,"%d",&xid_nonlas.lines);
if (xid_nonlas.lines == 0)
    {
    xid_message("Image may not contain zero lines");
    return;
    }
XtFree(tmpstr);

tmpstr = XmTextFieldGetString(nonlas_samples_text);
if (*tmpstr == '\0')
    {
    xid_message("User must specify number of samples");
    return;
    }
sscanf(tmpstr,"%d",&xid_nonlas.samples);
if (xid_nonlas.samples == 0)
    {
    xid_message("Image may not contain zero samples");
    return;
    }
XtFree(tmpstr);

state = XmToggleButtonGetState(nonlas_byte_button);
if (state)
    xid_nonlas.dtype = BYTE;
else
    {
    state = XmToggleButtonGetState(nonlas_i2_button);
    if (state)
        xid_nonlas.dtype = I2;
    else
        {
        state = XmToggleButtonGetState(nonlas_i4_button);
        if (state)
            xid_nonlas.dtype = I4;
	else
	    {
	    state = XmToggleButtonGetState(nonlas_real_button);
	    if (state)
		xid_nonlas.dtype = REAL;
	    }
        }
    }

state = XmToggleButtonGetState(nonlas_bsq_button);
if (state)
    xid_nonlas.format = BSQ;
else
    xid_nonlas.format = BIL;

/*
    Load the image.
*/
load_localok(w, client, call);
}

/* ARGSUSED */
void
nonlas_next(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
	This routine is used to switch activity to the input widget.  This is
	used to allow the user to move from one text field to the next by
	simply pressing the "Enter" key (rather than having to click the
	left mouse button in the desired field).
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
*******************************************************************************/
{
/*
    Move to the next item.
*/
XmProcessTraversal(w,XmTRAVERSE_NEXT_TAB_GROUP);
}
 
/* ARGSUSED */
void
nonlas_dialog(w, client, call)
Widget w;
XtPointer client;
XtPointer call;

/*******************************************************************************
Purpose:
	This routine is used to pop up the panel requesting image information
	for a nonLAS image (information such as number of lines, number of
	samples, number of bands, etc.).
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
*******************************************************************************/
{
Position
        popx,           /* X position of pop-up panel.                        */
        popy;           /* Y position of pop-up panel.                        */

char	*filename;      /* Input specification.                       	      */
 
/*SUPPRESS 594*/XmFileSelectionBoxCallbackStruct *acs =
	(XmFileSelectionBoxCallbackStruct*)call;

/*
    Retrieve the file name specified and save the current directory mask.
*/
XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);
strcpy(xid_load.image_name,filename);
if (xid_load.dirmask != NULL)
    XmStringFree(xid_load.dirmask);
xid_load.dirmask = XmStringCopy(acs->mask);
XtFree(filename);

/*
    Create the load nonLAS images popup.
*/
Createload_nonlas(Shell_Load);

/*
    Translate the X/Y position relative to the file selection box to an X/Y 
    position relative to the screen.
*/
XtTranslateCoords(load_form,(Position)0,(Position)0,&popx,&popy);
 
/*
    Position the pop-up panel.
*/
XtMoveWidget(load_nonlas_dialog,popx,popy);
 
/*
    Create the load nonLAS images information panel.
*/
if (xid_load.type == NONLAS)
    {
    XtPopup(load_nonlas_dialog,XtGrabNone);
    XtSetSensitive(load_fileselector,False);
    }
}

void
load_makelas(inspec,lasspec)
char	*inspec,		/* Input specification. (I)		      */
	*lasspec;		/* LAS specification. (O)		      */

/*******************************************************************************
Purpose:
    This routine takes as input a character string containing host image names 
    and window and/or band specifications.  This is converted to a string
    containing LAS names as well as the window and/or band specifications.
    
By L. Huewe, USGS/EROS Data Center, 10/93

Algorithm Description:
*******************************************************************************/
{
long	paren,		/* Position of left paren.			      */
	pflag=FALSE;	/* Indicates whether or not the current image name
			   has been processed.				      */

char	lasname[CMLEN],	/* LAS name specification.			      */
	*ptr,		/* Pointer to input string.			      */
	*tmp_ptr,	/* Pointer to temporary string.		       	      */
	tmpname[CMLEN];	/* Temporary string for holding a single image name.  */

/*
    Initialize strings.
*/
strcpy(lasspec,"");
strcpy(tmpname,"");

/*
    Examine each character in the input string.
*/
for (ptr=inspec,tmp_ptr=tmpname; *ptr != '\0'; ptr++)
    {
/*
    Just copy the character to a temporary string if it is not a "+" or a "("
    (i.e., it must be part of the image name).
*/
    if ((*ptr != '+') && (*ptr != '('))
	{
	*tmp_ptr = *ptr;
	tmp_ptr++;
	}
/*
    A left paren indicates a window and/or band specification.  Convert the 
    host name contained in the temporary string to a LAS name and concatenate
    it to the output string.  Also, concatenate the window and/or band
    specification to the output string.
*/
    else if (*ptr == '(')
	{
	*tmp_ptr = '\0';
	c_cmtae(tmpname,lasname);
	strcat(lasspec,lasname);
	paren = strcspn(ptr,")");
	strncat(lasspec,ptr,paren+1);
	ptr += paren;
	pflag = TRUE;
	}
/*
    A "+" indicates another image name follows.  Convert the current image
    name to a LAS name if it has not already been done and concatenate the
    resulting LAS name and a "+" to the output string.
*/
    else if (*ptr == '+')
	{
	if (!pflag)
	    {
	    *tmp_ptr = '\0';
	    c_cmtae(tmpname,lasname);
	    strcat(lasspec,lasname);
	    }
	strcat(lasspec,"+");
	strcpy(tmpname,"");
	tmp_ptr = tmpname;
	pflag = FALSE;
	}
    }
/*
    If the current image name has not been processed, convert it to a LAS
    name and append it to the output string.
*/
if (!pflag)
    {
    *tmp_ptr = '\0';
    c_cmtae(tmpname,lasname);
    strcat(lasspec,lasname);
    }
return;
}
