#include "xid.h"
#include "../xid_widgets/ProcessM.h"

FUNCTION loadimg(imgwin,loadflg)
Window	imgwin;			/* Window ID for image window (I).	      */
long	loadflg;		/* Indicates whether the image is being
				   loaded into the main image window (I).     */

/*******************************************************************************
Purpose:
    This routine opens and reads an image file (either or LAS or nonLAS).  If 
    necessary, it scales it to BYTE data and subsamples it.  LOADIMG then calls
    the routine LOAD_DISPLAY to output the image to the display screen.
    
By L. Huewe, USGS/EROS Data Center, 11/91
Modified by L. Huewe, USGS/EROS Data Center, 12/92
    - Added the ability to load nonLAS images.
Modified by L. Huewe, USGS/EROS Data Center, 11/93
    - Modified to always pass the maximum value of the image loaded into
      the main image window to load_display.  This ensures the maximum value 
      used to determine how the image is scaled is the same for the main image 
      window and its corresponding full resolution window.
Modified by L. Huewe, USGS/EROS Data Center, 4/94
    - Modified to print a warning message if the maximum value in a full
      resolution window exceeds the maximum value defined in the colormap. 
      This test is applicable only when the maximum value of the data in the 
      main image window is less than NUM_IMGCOLORS which results in each gray 
      level value occupying its own colormap cell (and therefore, there are no 
      colormap cells for values NUM_IMGCOLORS or larger).  If the condition 
      occurs, the offending value is set back to NUM_IMGCOLORS-1.

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
{
Arg	args[3];		/* Variable used to set widget resources.     */

struct	XID_MEMPLANE	
	*memptr;		/* Pointer to current memory plane structure. */

struct	XID_FULLRES
	*fresptr;		/* Pointer to full resolution structure.      */

double  offset,         	/* Offset used to scale non-byte data.        */
	slope,          	/* Slope used to scale non-byte data.         */
	minval,			/* Minimum pixel value of current band.	      */
	maxval;			/* Maximum pixel value of current band.	      */

float	*floatptr;		/* FLOAT data pointer.			      */

int	fdesc;			/* Input image file descriptor.		      */

long	c_eclose(),		/* LAS image close routine.		      */
	c_eopenr(),		/* LAS image open routine.		      */
	c_eread(),		/* LAS image read routine.		      */
	c_minmax(),		/* LAS min/max routine.			      */
	(*close_image)(),	/* Pointer to close routine.		      */
	close_nonlas(),		/* NonLAS image close routine.		      */
	(*minmax_image)(),	/* Pointer to min/max routine.		      */
	minmax_nonlas(),	/* NonLAS min/max routine.		      */
	(*open_image)(),	/* Pointer to open routine.		      */
	open_nonlas(),		/* NonLAS image open routine.		      */
	(*read_image)(),	/* Pointer to read routine.		      */
	read_nonlas();		/* NonLAS image read routine.		      */

long	access = IREAD,		/* Image file open access method.	      */
	bands[2],		/* Bands array for min/max calculation.	      */
	bnd,			/* Band counter for looping.		      */
	checkmax,		/* Flag indicating whether or not to validate
				   values from the full resolution window.    */
	dspwin[4],		/* Display window.			      */
	dtype = ELONG,		/* Data type of histogram buffer.	      */
	*histptr,		/* Pointer to histogram data.		      */
	i,			/* Looping indices.			      */
	img,			/* Image counter for looping.		      */
	lin,			/* Line counter for looping.		      */
	linnum,			/* Line number required by image I/O.	      */
	*longptr,		/* LONG data pointer.			      */
	nbands,			/* Number of bands in the file.		      */
	nbins = 256,		/* Number of histogram bins.		      */
	one = 1,		/* Constant one.			      */
	reset_values,		/* Flag indicating whether or not any full 
				   resolution values were reset.	      */
	*rgbmem,		/* Memory planes to be loaded with image data.*/
	skipl,			/* Subsampling factor in line direction.      */
	skips,			/* Subsampling factor in sample direction.    */
	smp,			/* Sample counter for looping.		      */
	status,			/* Status indicator.			      */
	tbnd,			/* Total number of bands counter.	      */
	user_cancel,		/* Indicates whether or not the user has
				   stopped processing.		      	      */
	wsize,			/* Word size of input image data type.	      */
	zero = 0;		/* Constant zero.			      */

short	*shortptr;		/* SHORT data pointer.			      */

unsigned char
	*dataptr,		/* Pointer to image data stored in memory
				   plane.				      */
	*linebuf;		/* Buffer to read image data into.	      */

char	mesg[200],		/* Error message text.			      */
	title[100];		/* Title for processing message window.	      */

/*
    Free the previously allocated image pixmap.
*/
if (loadflg)
    {
    if (xid_mem.imgpix)
        {
        XFreePixmap(xid.display,xid_mem.image_pixmap);
/*
    Clear the main image window background.
*/
        XSetWindowBackground(xid.display,XtWindow(main_image_bulletinboard),
	    xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
	XClearWindow(xid.display,XtWindow(main_image_bulletinboard));
        xid_mem.imgpix = FALSE;
        xid_mem.image_pixmap = (Pixmap)NULL;
        }
    }
/*
    Free the full resolution pixmap.
*/
else
    for (i=0; i < xid.num_fullres; i++)
        if (xid_fullres[i].image_pixmap != NULL)
	    {
            XFreePixmap(xid.display,xid_fullres[i].image_pixmap);
            XSetWindowBackground(xid.display,XtWindow(
                fullres_win_bulletinboard), xid.color_table[RGBBW_IDX +
                BLACK_COL].pixel);
            XClearWindow(xid.display,XtWindow(fullres_win_bulletinboard));
	    xid_fullres[i].image_pixmap = (Pixmap)NULL;
	    }

/*
    Allocate buffer space for the memory ids.
*/
rgbmem = (long *) calloc(xid_load.totbnd,sizeof(long));
if (rgbmem == NULL)
    {
    xid_message("Insufficient memory available");
    return(XID_F);
    }

if (loadflg)
    {
/*
    Get a new memory plane id for each band and zero out its associated 
    histogram.
*/
    if (xid_load.totbnd > xid.max_plane)
	{
	xid_message("Not enough memory planes available");
	free(rgbmem);
	return(XID_F);
	}

    for (i=0; i < xid_load.totbnd; i++)
	{
	if (xid_getmem(&rgbmem[i]) == XID_F)
	    {
	    xid_message("Not enough memory planes available");
	    free(rgbmem);
	    return(XID_F);
	    }
	c_pxfill(xid_planes[rgbmem[i]-1].histogram,&dtype,&nbins,&zero);
	}

/*******************************************************************************
This condition does not need to be checked for until the user is allowed to
lock a memory plane.  For now it is commented out to save processing time.
********************************************************************************

    Make sure no duplicate memory ids were returned.

    for (i=0; i < xid_load.totbnd-1; i++)
	for (j=i+1; j < xid_load.totbnd; i++)
	    if (rgbmem[i] == rgbmem[j])
		{
		xid_message("Not enough memory planes available");
		return(XID_F);
		}
*/
    }
else
    for (i=0; i < xid.num_fullres; i++)
	c_pxfill(xid_fullres[i].histogram,&dtype,&nbins,&zero);

/*
    Set the appropriate function pointers based on the type of image being
    loaded.
*/
    if (xid_load.type == LAS)
	{
	open_image = c_eopenr;
	read_image = c_eread;
	close_image = c_eclose;
	minmax_image = c_minmax;
	}
    else
	{
	open_image = open_nonlas;
	read_image = read_nonlas;
	close_image = close_nonlas;
	minmax_image = minmax_nonlas;
	}

/*
    For each image and each band, read in the image data and scale it to
    BYTE if necessary.
*/
user_cancel = FALSE;
for (img=0, tbnd = 0; img < xid_load.nimgs; img++)
    {
/*
    Open the input image.
*/
    dtype = nbands = 0;
    status = (*open_image)(&fdesc,xid_load.hnames[img],&nbands,
	&xid_load.windows[img][SL],&xid_load.windows[img][SS],
	&xid_load.windows[img][NL],&xid_load.windows[img][NS],&access,&dtype,
	&zero,&one);
    if (status != E_SUCC)
	{
	sprintf(mesg,"Error opening image file %s",xid_load.hnames[img]);
	xid_message(mesg);
	free(rgbmem);
	return(XID_F);
	}
/*
    Determine the number of bytes/pixel based on the input data type.
*/
    switch(dtype)
    	{
     	case  EBYTE : wsize = 1;
       	    break;
     	case  EWORD : wsize = 2;
            break;
     	case  ELONG :
     	case  EREAL : wsize = 4;
            break;
 	}
/*
    Allocate the input line buffer.
*/
    if ((linebuf = (unsigned char *)malloc(xid_load.windows[img][NS] * wsize)) 
	== NULL)
 	{
	xid_message("Insufficient memory available");
	free(rgbmem);
	return(XID_F);
 	}
/*
    Typecast appropriate pointers.
*/
    shortptr = (short *)linebuf;
    longptr = (long *)linebuf;
    floatptr = (float *)linebuf;

    for (bnd = 0; xid_load.bands[img][bnd] != 0; bnd++, tbnd++)
 	{
/*
    Read the DDR information for the current image (LAS images only).
*/
    	if ((loadflg) && (xid_load.type == LAS))
	    {
	    status = load_projlas(&rgbmem[tbnd],xid_load.hnames[img]);
	    if (status != XID_S)
		{
		free(rgbmem);
		return(XID_F);
		}
	    }
/*
    Load the projection information into the appropriate XID structures
    (nonLAS images).
*/
    	else if ((loadflg) && (xid_load.type != LAS))
	    {
	    status = load_projinfo(&rgbmem[tbnd],xid_load.hnames[img]);
	    if (status != XID_S)
		{
		free(rgbmem);
		return(XID_F);
		}
	    }
/*
    Set the file window and band fields of the memory plane structure.
*/
	if (loadflg)
	    {
	    memptr = &xid_planes[rgbmem[tbnd]-1];
	    memptr->window[SL] = xid_load.windows[img][SL]; 
	    memptr->window[SS] = xid_load.windows[img][SS]; 
	    memptr->window[NL] = xid_load.windows[img][NL]; 
	    memptr->window[NS] = xid_load.windows[img][NS]; 
	    memptr->band = xid_load.bands[img][bnd]; 
	    strcpy(memptr->filename,xid_load.tnames[img]);
	    memptr->image_type = xid_load.type;
	    }
	else
	    fresptr = &xid_fullres[tbnd];
/*
    Determine whether or not subsampling is required and the size of the 
    displayed image.
*/
	if (loadflg)
	    {
	    memptr->dspwin[SL] = memptr->dspwin[SS] = 1;
	    memptr->dspwin[NL] = xid_mem.mem_height;
	    memptr->dspwin[NS] = xid_mem.mem_width;
	    status = xid_samp(xid_load.windows[img],memptr->dspwin,
		&memptr->skipl,&memptr->skips);
	    if (status != E_SUCC)
		{
		free(rgbmem);
		free(linebuf);
		return(XID_F);
		}
	    skipl = memptr->skipl;
	    skips = memptr->skips;
	    dspwin[SL] = memptr->dspwin[SL];
	    dspwin[SS] = memptr->dspwin[SS];
	    dspwin[NL] = memptr->dspwin[NL];
	    dspwin[NS] = memptr->dspwin[NS];
	    }
	else
	    {
	    fresptr->dspwin[SL] = fresptr->dspwin[SS] = 1;
	    fresptr->dspwin[NL] = fresptr->height;
	    fresptr->dspwin[NS] = fresptr->width;
	    status = xid_samp(xid_load.windows[img],fresptr->dspwin,
		&fresptr->skipl,&fresptr->skips);
	    if (status != E_SUCC)
		{
		free(rgbmem);
		free(linebuf);
		return(XID_F);
		}
	    skipl = fresptr->skipl;
	    skips = fresptr->skips;
	    dspwin[SL] = fresptr->dspwin[SL];
	    dspwin[SS] = fresptr->dspwin[SS];
	    dspwin[NL] = fresptr->dspwin[NL];
	    dspwin[NS] = fresptr->dspwin[NS];
	    }

    	if (dtype != EBYTE)
   	    {
/*
    Determine the min/max of the current band and calculate the slope and
    offset required to scale the input data to BYTE.
*/
	    bands[0] = xid_load.bands[img][bnd];
	    bands[1] = 0;
	    status = (*minmax_image)(xid_load.hnames[img],&one,
		xid_load.windows[img],bands,&one,&minval,&maxval);
	    if (status != E_SUCC)
		{
		free(rgbmem);
		free(linebuf);
		xid_message("Error calculating min/max of image");
	        return(XID_F);
		}
	    if (loadflg)
		{
	        status = xid_slpoff(&dtype,&minval,&maxval,&memptr->slope,
		    &memptr->offset);
		slope = memptr->slope;
		offset = memptr->offset;
		}
	    else
		{
	        status = xid_slpoff(&dtype,&minval,&maxval,&fresptr->slope,
		    &fresptr->offset);
		slope = fresptr->slope;
		offset = fresptr->offset;
		}
	    if (status != E_SUCC)
		{
		free(rgbmem);
		free(linebuf);
	        return(XID_F);
		}
	    }
        else
	    {
	    if (loadflg)
		{
	        memptr->slope = 1.0;
	        memptr->offset = 0.0;
		}
	    else
		{
	        fresptr->slope = 1.0;
	        fresptr->offset = 0.0;
		}
	    }

/*
    Read each line of input data, update its associated histogram and scale
    it to BYTE if necessary.
*/
	if (loadflg)
	    {
	    dataptr = memptr->image_data;
	    histptr = memptr->histogram;
	    }
	else
	    {
	    dataptr = fresptr->image_data;
	    histptr = fresptr->histogram;
	    }

/*
    Initialize the input buffer to all zeroes.  If the user cancels part way 
    through the load process. this ensures that there is not garbage in the
    unloaded portion of the buffer.
*/
        bzero(dataptr, dspwin[NL]*dspwin[NS]);
/*
    Determine whether or not the user cancelled the load.
*/
	if (user_cancel)
	    break;
/*
    Update processing message widget.
*/
        sprintf(title,"Processing Band %d of Image %d  ",bnd+1,img+1);
        XtSetArg(args[0], XeNprocessMUpdatePercent, 10);
        XtSetValues(process_widget, args, 1);

        XeProcessMBegin(process_widget,0,dspwin[NL],title,False,True,True);

/*
    If a full resolution window is being loaded, determine whether or not we 
    need to validate the values being loaded.  Validation needs to take place
    only if the maximum image value in the main image window is less than
    NUM_IMGCOLORS.  This is because in this case, there are no colormap
    cells defined for image values greater than or equal to NUM_IMGCOLORS
    and there is the potential that the full resolution image window may
    contain some of these values.
*/
	if ((!loadflg) && (xid_load.totbnd == 1) && (xid_planes[xid_view.
	    rgbplanes[RED] - 1].maxval < NUM_IMGCOLORS))
	    checkmax = TRUE;
	else
	    checkmax = FALSE;
	reset_values = FALSE;

/*
    Read the input image based on its data type.
*/
	switch (dtype)
	    {
	    case EBYTE:
		for (lin = 0, linnum = 1; lin < dspwin[NL];lin++, 
		    linnum += skipl)
	    	    {
/*
    Read a line of image data.
*/
	    	    status = (*read_image)(&fdesc,&xid_load.bands[img][bnd],
			&linnum,linebuf,&one);
		    if (status != xid_load.windows[img][NS])  
			{
		 	sprintf(mesg,"Error reading image file %s",
			    xid_load.hnames[img]);
			xid_message(mesg);
        		free(rgbmem);
		        free(linebuf);
/*
    Close processing message window.
*/
			XeProcessMEnd(process_widget);
			return(XID_F);
			}
/*
    Update processing dialog and check for cancel.
*/
                    if (XeProcessMProcess(process_widget,lin) == 
			XePROCESSM_CANCEL)
		        {
		        user_cancel = TRUE;
                        break;
		        }
/*
    Subsample the image (if necessary) and update the histogram.
*/
	    	    for (smp = 0; smp < xid_load.windows[img][NS]; smp += 
			skips,dataptr++)
			{
/*
    Validate values if necessary.
*/
			if (checkmax)
			    if (linebuf[smp] > (NUM_IMGCOLORS - 1))
				{
				linebuf[smp] = NUM_IMGCOLORS - 1;
				reset_values = TRUE;
				}
			histptr[linebuf[smp]]++;
			*dataptr = linebuf[smp];
			}
	    	    }
		break;

	    case EWORD:
		for (lin = 0, linnum = 1; lin < dspwin[NL];lin++, 
		    linnum += skipl)
	    	    {
/*
    Read a line of image data.
*/
	    	    status = (*read_image)(&fdesc,&xid_load.bands[img][bnd],
			&linnum,linebuf,&one);
                    if (status != xid_load.windows[img][NS]) 
                        {
		 	sprintf(mesg,"Error reading image file %s",
			    xid_load.hnames[img]);
			xid_message(mesg);
                        free(rgbmem);
                        free(linebuf);
/*
    Close processing message window.
*/
			XeProcessMEnd(process_widget);
                        return(XID_F);
                        }
/*
    Update processing dialog and check for cancel.
*/
                    if (XeProcessMProcess(process_widget,lin) == 
			XePROCESSM_CANCEL)
			{
			user_cancel = TRUE;
                        break;
			}
/*
    Subsample the image (if necessary) and update the histogram.
*/
	    	    for (smp = 0; smp < xid_load.windows[img][NS]; smp += 
			skips,dataptr++)
			{
			*dataptr = (unsigned char) ((shortptr[smp] * 
			    slope + offset) + .5);
/*
    Validate values if necessary.
*/
			if (checkmax)
			    if (*dataptr > (NUM_IMGCOLORS - 1))
				{
				*dataptr = NUM_IMGCOLORS - 1;
				reset_values = TRUE;
				}
			histptr[*dataptr]++;
			}
	    	    }
		break;

	    case ELONG:
		for (lin = 0, linnum = 1; lin < dspwin[NL];lin++, 
		    linnum += skipl)
	    	    {
/*
    Read a line of image data.
*/
	    	    status = (*read_image)(&fdesc,&xid_load.bands[img][bnd],
			&linnum,linebuf,&one);
                    if (status != xid_load.windows[img][NS]) 
                        {
		 	sprintf(mesg,"Error reading image file %s",
			    xid_load.hnames[img]);
			xid_message(mesg);
			free(rgbmem);
                        free(linebuf);
/*
    Close processing message window.
*/
			XeProcessMEnd(process_widget);
                        return(XID_F);
                        }
/*
    Update processing dialog and check for cancel.
*/
                    if (XeProcessMProcess(process_widget,lin) == 
			XePROCESSM_CANCEL)
			{
		        user_cancel = TRUE;
                        break;
			}
/*
    Subsample the image (if necessary) and update the histogram.
*/
	    	    for (smp = 0; smp < xid_load.windows[img][NS]; smp += 
			skips,dataptr++)
			{
			*dataptr = (unsigned char) ((longptr[smp] * 
			    slope + offset) + .5);
/*
    Validate values if necessary.
*/
			if (checkmax)
			    if (*dataptr > (NUM_IMGCOLORS - 1))
				{
				*dataptr = NUM_IMGCOLORS - 1;
				reset_values = TRUE;
				}
			histptr[*dataptr]++;
			}
	    	    }
		break;

	    case EREAL:
		for (lin = 0, linnum = 1; lin < dspwin[NL];lin++, 
		    linnum += skipl)
	    	    {
/*
    Read a line of image data.
*/
	    	    status = (*read_image)(&fdesc,&xid_load.bands[img][bnd],
			&linnum,linebuf,&one);
                    if (status != xid_load.windows[img][NS]) 
                        {
		 	sprintf(mesg,"Error reading image file %s",
			    xid_load.hnames[img]);
			xid_message(mesg);
                        free(rgbmem);
                        free(linebuf);
/*
    Close processing message window.
*/
			XeProcessMEnd(process_widget);
                        return(XID_F);
                        }
/*
    Update processing dialog and check for cancel.
*/
                    if (XeProcessMProcess(process_widget,lin) == 
			XePROCESSM_CANCEL)
			{
		        user_cancel = TRUE;
                        break;
			}
/*
    Subsample the image (if necessary) and update the histogram.
*/
	    	    for (smp = 0; smp < xid_load.windows[img][NS]; smp += 
			skips,dataptr++)
			{
			*dataptr = (unsigned char) ((floatptr[smp] * 
			    slope + offset) + .5);
/*
    Validate values if necessary.
*/
			if (checkmax)
			    if (*dataptr > (NUM_IMGCOLORS - 1))
				{
				*dataptr = NUM_IMGCOLORS - 1;
				reset_values = TRUE;
				}
			histptr[*dataptr]++;
			}
	    	    }
		break;
	    } 
/*
    Calculate the associated histogram statistics -- min, max, mean and
    standard deviation.
*/
	if (loadflg)
	    xid_histcalc(memptr->histogram,&memptr->minval,&memptr->maxval,
 	        &memptr->mean,&memptr->stdev);
	else
	    {
	    xid_histcalc(fresptr->histogram,&fresptr->minval,&fresptr->maxval,
 	        &fresptr->mean,&fresptr->stdev);
	    if (reset_values) 
		{
		sprintf(mesg,"Number of image colors in full resolution image exceeds the number of colors in the\n main image window. Values are being clipped to %d.", NUM_IMGCOLORS-1);
		xid_message(mesg);
		}
	    }
    	}
/*
    Close the image.
*/
    status = (*close_image)(&fdesc);
    if (status != E_SUCC)
	{
	sprintf(mesg,"Error closing image file %s",xid_load.hnames[img]);
	xid_message(mesg);
        free(rgbmem);
	free(linebuf);
/*
    Close processing message window.
*/
	XeProcessMEnd(process_widget);
        return(XID_F);
	}
    }
/*
    Done reading the image from disk.
*/
XeProcessMEnd(process_widget);

/*
    Update processing message.
*/
strcpy(title,"Loading image to display  ");
XeProcessMBegin(process_widget,0,1,title,True,False,False);
XeProcessMProcess(process_widget,1);

/*
    Load the image into the image window.
*/
if ((xid_load.totbnd == 1) && loadflg)
    status = load_display(rgbmem,&xid_load.totbnd,imgwin,Shell_MainWin,
	main_image_bulletinboard,&xid_planes[rgbmem[0] - 1].dspwin[NL],
	&xid_planes[rgbmem[RED] - 1].dspwin[NS],&xid_planes[rgbmem[RED] - 1].
	maxval,xid_planes[rgbmem[RED] - 1].image_data,NULL,NULL,TRUE,&xid_mem.
	image_pixmap,TRUE,TRUE,FALSE);
else if ((xid_load.totbnd == 2) && loadflg)
    status = load_display(rgbmem,&xid_load.totbnd,imgwin,Shell_MainWin,
	main_image_bulletinboard,&xid_planes[rgbmem[RED] - 1].dspwin[NL],
	&xid_planes[rgbmem[RED] - 1].dspwin[NS],&xid_planes[rgbmem[RED] - 1].
	maxval,xid_planes[rgbmem[RED] - 1].image_data,xid_planes[rgbmem[GREEN]
	- 1].image_data,xid_planes[rgbmem[GREEN] - 1].image_data,TRUE,&xid_mem.
	image_pixmap,TRUE,TRUE,FALSE);
else if (loadflg)
    status = load_display(rgbmem,&xid_load.totbnd,imgwin,Shell_MainWin,
	main_image_bulletinboard,&xid_planes[rgbmem[RED] - 1].dspwin[NL],
	&xid_planes[rgbmem[RED] - 1].dspwin[NS],&xid_planes[rgbmem[RED] - 1].
	maxval,xid_planes[rgbmem[RED] - 1].image_data,xid_planes[rgbmem[GREEN]
	- 1].image_data,xid_planes[rgbmem[BLUE] - 1].image_data,TRUE,&xid_mem.
	image_pixmap,TRUE,TRUE,FALSE);
else if ((xid_load.totbnd == 1) && !loadflg)
    status = load_display(rgbmem,&xid_load.totbnd,imgwin,Shell_FullResWin,
	fullres_win_bulletinboard,&xid_fullres[0].dspwin[NL],
	&xid_fullres[0].dspwin[NS],&xid_planes[xid_view.rgbplanes[RED] - 1].
	maxval,xid_fullres[0].image_data,NULL,NULL,FALSE,&xid_fullres[0].
	image_pixmap,FALSE,TRUE,FALSE);
else if (!loadflg)
    status = load_display(rgbmem,&xid_load.totbnd,imgwin,Shell_FullResWin,
	fullres_win_bulletinboard,&xid_fullres[0].dspwin[NL],
	&xid_fullres[0].dspwin[NS],&xid_planes[xid_view.rgbplanes[RED] - 1].
	maxval,xid_fullres[RED].image_data,xid_fullres[GREEN].image_data,
	xid_fullres[BLUE].image_data,FALSE,&xid_fullres[0].image_pixmap,FALSE,
	TRUE,FALSE);
/*
    Close processing message window.
*/
XeProcessMEnd(process_widget);

/*
    Change the title to reflect the name of the image loaded into the
    main image window.
*/
if (loadflg)
    {
    XtSetArg(args[0],XmNtitle,xid_load.tnames[0]);
    XtSetValues(Shell_MainWin,args,1);
    }

/*
    Warn the user that image data has been scaled to less than 256 values.
*/
if ((loadflg) && (*xid.colormap_ptr == xid.colormap) && (xid.flags & 
    XID_COLMSG))
    if ((xid_load.totbnd > 1) || (xid_planes[rgbmem[RED] - 1].maxval >
	(NUM_IMGCOLORS - 1)))
	{
	sprintf(mesg,"Image data has been limited to %d gray levels",
	    NUM_IMGCOLORS); 
	xid_message(mesg);
	}

/*
    Free the appropriate resources.
*/
free(linebuf);
free(rgbmem);

return(status);
}

FUNCTION load_projlas(memory,hostname)
long	*memory;		/* Memory plane for which projection infor-
				   mation is being read (I).		      */

char	*hostname;		/* Host name of the LAS image to read DDR 
				   information from (I).		      */
/*******************************************************************************
Purpose:
    This routine copies selected information from the LAS DDR into the
    XID XID_IMGINFO structure.  See the in-line comments for a list of the
    fields copied.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
static	struct	DDR	lasddr;	/* LAS DDR information.			      */

static	char	lastname[CMLEN] = {""};
				/* Last image for which DDR information was 
				   read.				      */

long	i,			/* Looping index.			      */
	status;			/* Status indicator.			      */

/*
    If the image name has changed, retrieve the new DDR information.
*/
if (strcmp(lastname,hostname) != 0)
    {
    status = c_getddr(hostname,&lasddr);
    if (status != E_SUCC)
        return(XID_F);
    strcpy(lastname,hostname);
    }

/*
    Determine whether the DDR contains valid projection and corner
    coordinate information.
*/
if ((lasddr.valid[DDPCV] != 1) || (lasddr.valid[DDZCV] != 1) || 
    (lasddr.valid[DDDCV] != 1) || (lasddr.valid[DDPPV] != 1))
        xid_planes[*memory-1].proj_valid = FALSE;
else
	xid_planes[*memory-1].proj_valid = TRUE;

if (lasddr.valid[DDCCV] != 1)
        xid_planes[*memory-1].corner_valid = FALSE;
else
	xid_planes[*memory-1].corner_valid = TRUE;

/*
    Copy the following information to the image information structure.
	- Projection parameters.
	- Corner coordinates.
	- Projection distance.
	- Projection units.
	- Projection code.
	- Zone code.
	- Datum code.
	- Line and sample increments.
	- Number of lines.
	- Number of samples.
	- Number of bands.
	- Data type.
*/

if (xid_planes[*memory-1].proj_valid)
    {
    for (i=0; i < 15; i++)
	xid_planes[*memory-1].image_info->proj_coef[i] = lasddr.proj_coef[i];

    xid_planes[*memory-1].image_info->proj_code = lasddr.proj_code;
    xid_planes[*memory-1].image_info->zone_code = lasddr.zone_code;
    xid_planes[*memory-1].image_info->datum_code = lasddr.datum_code;
    }

if (xid_planes[*memory-1].corner_valid)
    {
    for (i=0; i < 2; i++)
	{
	xid_planes[*memory-1].image_info->upleft[i] = lasddr.upleft[i];
	xid_planes[*memory-1].image_info->loleft[i] = lasddr.loleft[i];
	xid_planes[*memory-1].image_info->upright[i] = lasddr.upright[i];
	xid_planes[*memory-1].image_info->loright[i] = lasddr.loright[i];
        }
    }

if (lasddr.valid[DDPDV])
    {
    xid_planes[*memory-1].image_info->pdist_y = lasddr.pdist_y;
    xid_planes[*memory-1].image_info->pdist_x = lasddr.pdist_x;
    }
else
    {
    xid_planes[*memory-1].image_info->pdist_y = 0; 
    xid_planes[*memory-1].image_info->pdist_x = 0; 
    }
if (lasddr.valid[DDINCV])
    { 
    xid_planes[*memory-1].image_info->line_inc = lasddr.line_inc;
    xid_planes[*memory-1].image_info->sample_inc = lasddr.sample_inc;
    }
else
    { 
    xid_planes[*memory-1].image_info->line_inc = 0; 
    xid_planes[*memory-1].image_info->sample_inc = 0; 
    }

xid_planes[*memory-1].image_info->nl = lasddr.nl;
xid_planes[*memory-1].image_info->ns = lasddr.ns;
xid_planes[*memory-1].image_info->nbands = lasddr.nbands;
xid_planes[*memory-1].image_info->dtype = lasddr.dtype;
xid_planes[*memory-1].image_info->master_line = lasddr.master_line;
xid_planes[*memory-1].image_info->master_sample = lasddr.master_sample;

if (lasddr.valid[DDPUV])
    strcpy(xid_planes[*memory-1].image_info->proj_units,lasddr.proj_units);
else
    strcpy(xid_planes[*memory-1].image_info->proj_units,"");

return(XID_S);
}

FUNCTION load_projinfo(memory,hostname)
long	*memory;		/* Memory plane for which projection infor-
				   mation is being read (I).		      */

char	*hostname;		/* Host name of the LAS image to read DDR 
				   information from (I).		      */
/*******************************************************************************
Purpose:
    This routine copies appropriate information into the XID XID_IMGINFO 
    structure.
    
By L. Huewe, USGS/EROS Data Center, 12/92

Algorithm Description:
*******************************************************************************/
{
long	i;			/* Looping index.			      */

/*
    Determine whether valid projection information exists.  These flags are
    currently set to FALSE because the ability to specify projection information
    for nonLAS images has not yet been implemented.
*/
xid_planes[*memory-1].proj_valid = xid_planes[*memory-1].corner_valid = FALSE;

/*
    Copy the following information to the image information structure.
	- Projection parameters.
	- Corner coordinates.
	- Projection distance.
	- Projection units.
	- Projection code.
	- Zone code.
	- Datum code.
	- Line and sample increments.
	- Number of lines.
	- Number of samples.
	- Number of bands.
	- Data type.
*/

if (xid_planes[*memory-1].proj_valid)
    {
    for (i=0; i < 15; i++)
	xid_planes[*memory-1].image_info->proj_coef[i] = 
	    xid_projinfo.proj_coef[i];
    xid_planes[*memory-1].image_info->proj_code = xid_projinfo.proj_code;
    xid_planes[*memory-1].image_info->zone_code = xid_projinfo.zone_code;
    xid_planes[*memory-1].image_info->datum_code = xid_projinfo.datum_code;
    }

if (xid_planes[*memory-1].corner_valid)
    {
    for (i=0; i < 2; i++)
	{
	xid_planes[*memory-1].image_info->upleft[i] = xid_projinfo.upleft[i];
	xid_planes[*memory-1].image_info->loleft[i] = xid_projinfo.loleft[i];
	xid_planes[*memory-1].image_info->upright[i] = xid_projinfo.upright[i];
	xid_planes[*memory-1].image_info->loright[i] = xid_projinfo.loright[i];
        }
    }

if ((xid_projinfo.pdist_y != -999) && (xid_projinfo.pdist_x != -999))
    {
    xid_planes[*memory-1].image_info->pdist_y = xid_projinfo.pdist_y;
    xid_planes[*memory-1].image_info->pdist_x = xid_projinfo.pdist_x;
    }
else
    {
    xid_planes[*memory-1].image_info->pdist_y = 0; 
    xid_planes[*memory-1].image_info->pdist_x = 0; 
    }
if ((xid_projinfo.line_inc != -999) && (xid_projinfo.sample_inc != -999))
    { 
    xid_planes[*memory-1].image_info->line_inc = xid_projinfo.line_inc;
    xid_planes[*memory-1].image_info->sample_inc = xid_projinfo.sample_inc;
    }
else
    { 
    xid_planes[*memory-1].image_info->line_inc = 0; 
    xid_planes[*memory-1].image_info->sample_inc = 0; 
    }

xid_planes[*memory-1].image_info->nl = xid_nonlas.lines; 
xid_planes[*memory-1].image_info->ns = xid_nonlas.samples;
xid_planes[*memory-1].image_info->nbands = xid_nonlas.nbands;
xid_planes[*memory-1].image_info->dtype = xid_nonlas.dtype;

if (*xid_projinfo.proj_units == '\0')
    strcpy(xid_planes[*memory-1].image_info->proj_units,
	xid_projinfo.proj_units);
else
    strcpy(xid_planes[*memory-1].image_info->proj_units,"");

return(XID_S);
}
