/*
 *  This is the mask function source file for the mask panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include "xid.h"
#include "typlim.h"
#define	BITMAP	1

GC      mask_gc;			/* Graphics context for copying the
					   mask information to the image 
					   window.			      */

void
mask_apply(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine allocates the bitmap (1-bit pixmap) needed to store the 
    graphics mask, calculates the mask and applies it to the displayed image.
    
By L. Huewe, USGS/EROS Data Center, 03/92

Algorithm Description:
*******************************************************************************/
{
XGCValues
	values;			/* Values array for the GC.		      */

unsigned long
	valuemask = 0;		/* Specify no values.			      */

long	height,			/* Graphics pixmap height.		      */
	i,j,			/* Looping indices.			      */
	idx,			/* Index into memory plane structure.	      */
	init = FALSE,		/* Whether or not to initialize pixmap.       */
	maxval,			/* Maximum gray level value allowed.	      */
	nonwhite = FALSE,	/* Indicates whether or not nonwhite space 
				   was found in range/bin string.	      */
	plane_idx,		/* Image plane index.			      */
	status,			/* Status indicator.			      */
	width;			/* Graphics pixmap width.		      */

char	*grayvals;		/* Pointer to mask value string.	      */

unsigned char
	*imgptr,		/* Pointer to image data buffer.	      */
	lut[256];		/* A LUT indicating which pixel values the
				   mask is to be created for.		      */

/*
    Return if there is no image currently being viewed.
*/
if (xid.view == -1)
    {
    xid_message("No image data currently being viewed");
    return;
    }

/*
    The dislayed image must be black and white (single band) in order to
    generate a graphics mask.
*/
if ((xid_view.rgbplanes[RED] != xid_view.rgbplanes[GREEN]) || (
    xid_view.rgbplanes[GREEN] != xid_view.rgbplanes[BLUE]))
    {
    xid_message("A mask may be calculated for black and white images only");
    return;
    }

/*
    Has a color been selected?
*/
if (xid_mask.current_color == -1) 
    {
    xid_message("No color has been selected for graphics mask");
    return;
    }

/*
    Determine the graphics bitmap width and height.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;
width = xid_mem.mem_width;
height = xid_mem.mem_height;

/*
    Create a bitmap the same size as the image planes.
*/
if (xid_mask.mask_pixmap[xid_mask.graphics_mask] == NULL)
    {
    xid_mask.mask_pixmap[xid_mask.graphics_mask] = XCreatePixmap(xid.display,
	XtWindow(mask_bulletinboard),width,height,BITMAP);
    init = TRUE;
    }

/*
    Create a GC to be used when drawing into the bitmaps.
*/
if (xid_mask.gc == NULL)
    {
    xid_mask.gc = XCreateGC(xid.display,xid_mask.mask_pixmap[xid_mask.
	graphics_mask],valuemask,&values);
    XSetPlaneMask(xid.display,xid_mask.gc,1); 
    }

/*
    Clear the bitmap and set its foreground and background values.
*/
if (init)
    {
    XSetBackground(xid.display,xid_mask.gc,0);
    XSetForeground(xid.display,xid_mask.gc,0);
    XFillRectangle(xid.display,xid_mask.mask_pixmap[xid_mask.graphics_mask],
	xid_mask.gc,0,0,width,height);
    }
XSetBackground(xid.display,xid_mask.gc,1);
XSetForeground(xid.display,xid_mask.gc,1);

/*
    Determine the maximum gray level value allowed.
*/
idx = xid_view.rgbplanes[RED] - 1;
if (xid_planes[idx].maxval < NUM_IMGCOLORS)
    maxval = NUM_IMGCOLORS - 1;
else
    maxval = 255;

/*
    Determine whether or not any gray levels were specified.
*/
grayvals = XmTextGetString(mask_value_text);
for (i=0; i < strlen(grayvals); i++)
    if (isspace(grayvals[i]) == 0)
 	nonwhite = TRUE;
if (!nonwhite)
    xid_mask.num_range = xid_mask.num_bin = 0;
else
    {
/*
    Parse the specified gray level values into ranges and/or individual bins.
*/
    if (xid_mask.range != NULL)
	{
	free(xid_mask.range);
	xid_mask.range = NULL;
	}
    if (xid_mask.bin != NULL)
	{
	free(xid_mask.bin);
	xid_mask.bin = NULL;
	}
    status = parse_rb(grayvals,&xid_mask.range,&xid_mask.num_range,
	&xid_mask.bin,&xid_mask.num_bin,MINUTINY,maxval);
    if (status != XID_S)
        xid_mask.num_range = xid_mask.num_bin = 0;
    }

/*
    Set up a LUT which defines the pixel values where a bit is to be turned
    on in the bitmap.
*/
for (i=0; i < 256; i++)
    lut[i] = 0;
for (i=0; i < xid_mask.num_bin; i++)
    lut[xid_mask.bin[i]] = 1;
for (i=0; i < xid_mask.num_range; i++)
    for (j=xid_mask.range[i*2]; j <= xid_mask.range[i*2+1]; j++)
	lut[j] = 1;

/*
    Turn on the appropriate bits in the bitmap whose pixel values meet the
    criteria set by the user.
*/
imgptr = xid_planes[plane_idx].image_data;
for (i=0; i < xid_planes[plane_idx].dspwin[NL]; i++) 
    for (j=0; j < xid_planes[plane_idx].dspwin[NS]; j++, imgptr++) 
        if (lut[*imgptr] == 1)
	    XDrawPoint(xid.display,xid_mask.mask_pixmap[xid_mask.graphics_mask],
	        xid_mask.gc,j,i);
/*
    Set the clip mask in the GC so that only the values which have been 
    turned on in the bitmap will get copied to the displayed image.
*/
xid_mask.color[xid_mask.graphics_mask] = xid_mask.current_color;
XSetClipMask(xid.display, mask_gc, 
    xid_mask.mask_pixmap[xid_mask.graphics_mask]);

/*
    Copy the bitmap to the image pixmap and clear the image window.  This
    updates the displayed image since the image pixmap is defined as the
    background pixmap of the window.
*/
XCopyPlane(xid.display,xid_mask.mask_pixmap[xid_mask.graphics_mask],
    xid_mem.image_pixmap,mask_gc,0,0,width,height,0,0,1);
XClearWindow(xid.display,XtWindow(main_image_bulletinboard));

/*
    Free the string containing the specified gray level values.
*/
XtFree(grayvals);
}

void
mask_clear(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine clears a single graphics mask by freeing the associated
    bitmap and redisplaying the image, thereby drawing the image with only
    the currently allocated graphics masks.
    
By L. Huewe, USGS/EROS Data Center, 03/92

Algorithm Description:
*******************************************************************************/
{
long	one = 1,		/* Constant one.			      */
	plane_idx;		/* Index into memory plane structure.	      */

/*
    Free the bitmap of the graphics mask to be cleared.
*/
if (xid_mask.mask_pixmap[xid_mask.graphics_mask] != NULL)
    {
    XFreePixmap(xid.display,xid_mask.mask_pixmap[xid_mask.graphics_mask]);
    xid_mask.mask_pixmap[xid_mask.graphics_mask] = (Pixmap)NULL;
    xid_mask.color[xid_mask.graphics_mask] = -1;
    }

/*
    Redisplay the image.  This will cause the image to be redrawn overlaying
    only the active graphics masks.
*/
plane_idx = xid_view.rgbplanes[RED] - 1;
load_display(xid_view.rgbplanes,&one,XtWindow(main_image_bulletinboard),
    Shell_MainWin,main_image_bulletinboard,&xid_planes[plane_idx].dspwin[NL],
    &xid_planes[plane_idx].dspwin[NS],&xid_planes[plane_idx].maxval,
    xid_planes[plane_idx].image_data,NULL,NULL,FALSE,&xid_mem.image_pixmap,
    FALSE,FALSE,FALSE);
}

void
mask_create()
/*******************************************************************************
Purpose:
    This routine sets up initial values required for drawing graphics masks. 
    
By L. Huewe, USGS/EROS Data Center, 03/92

Algorithm Description:
*******************************************************************************/
{
static long	first = TRUE;	/* First time called.			      */

/*
    If this is the first time this function has been called, set the GC
    address to NULL to indicate that a GC needs to be created.
*/
if (first)
    {
/*
    Set initial values.
*/
    xid_mask.rotate_value = 1;
    xid_mask.graphics_mask = 0;
    xid_mask.current_color = -1;

    xid_mask.gc = (GC)NULL;
    first = FALSE;
    }
/*
    Create the graphics context for copying the mask information to the
    image window.
*/
mask_gc = XCreateGC(xid.display, XtWindow(Shell_MainMenu), NULL, NULL);

/*
    Set the foreground and background colors in the mask GC to the selected
    color.
*/
if (xid_mask.current_color != -1)
    {
    XSetBackground(xid.display,mask_gc,xid.color_table[RGBBW_IDX + 
	xid_mask.current_color].pixel);
    XSetForeground(xid.display,mask_gc,xid.color_table[RGBBW_IDX + 
	xid_mask.current_color].pixel);
    }
}

void
mask_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine closes the mask panel and sets the appropriate button on 
    the main panel to sensitive.
    
By L. Huewe, USGS/EROS Data Center, 03/92

Algorithm Description:
*******************************************************************************/
{
/*
    Free the graphics context, close the mask panel and set the mask button 
    on the main panel to sensitive.
*/
XFreeGC(xid.display, mask_gc);
XtUnmanageChild(Mask_bulletinboard);
XtPopdown(Shell_Mask);
XtSetSensitive(main_mask_button,True);

}

void
mask_color(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine sets the foreground and background colors in the default
    GC to the currently selected color.  The currently selected color is
    also highlighted by changing its top and bottom shadow colors to gold.

    Note:  The flaw in this routine is that it assumes the original top
	    and bottom shadow colors are white and gray25.
    
By L. Huewe, USGS/EROS Data Center, 03/92

Algorithm Description:
*******************************************************************************/
{
static 	Widget	previous = NULL;/* Previous Widget ID.			      */

Arg     args[3];        	/* Variable used to set widget resources.     */

Boolean	argok;			/* Indicates whether or not CONVERT was
				   successful.				      */

long	i;			/* Looping indices.			      */

/*
    Find the index into the wk_colors array of the color selected.
*/
for (i=0; i < NUM_CVALS; i++)
    if (strcmp(client,wk_colors[i]) == 0)
	break;

/*
    Set the foreground and background colors in the mask GC to the selected
    color.
*/
xid_mask.current_color = i;
XSetBackground(xid.display,mask_gc,xid.color_table[RGBBW_IDX + i].pixel);
XSetForeground(xid.display,mask_gc,xid.color_table[RGBBW_IDX + i].pixel);

/*
    Set the top and bottom shadow colors of the selected widget to gold to
    indicate to the user what the current color is.
*/
XtSetArg(args[0], XmNtopShadowColor,
    CONVERT(Shell_Mask, "gold", "Pixel", 4, &argok)); 
XtSetArg(args[1], XmNbottomShadowColor,
    CONVERT(Shell_Mask, "gold", "Pixel", 4, &argok)); 
XtSetArg(args[2], XmNshadowThickness,3);
XtSetValues(w,args,3);

/*
    Set the top and bottom shadow colors of the previously selected widget
    back to white and gray25.
*/
if (previous != NULL)
    {
    XtSetArg(args[0], XmNtopShadowColor,
	CONVERT(Shell_Mask, "white", "Pixel", 3, &argok)); 
    XtSetArg(args[1], XmNbottomShadowColor,
	CONVERT(Shell_Mask, "gray25", "Pixel", 3, &argok)); 
    XtSetArg(args[2], XmNshadowThickness,2);
    XtSetValues(previous,args,3);
    }
previous = w;
}

void
mask_rotator(w, client, call)
Widget w;
caddr_t client;
caddr_t call;
/*******************************************************************************
Purpose:
    This routine updates the current graphics mask number. 
    
By L. Huewe, USGS/EROS Data Center, 03/92

Algorithm Description:
*******************************************************************************/
{
char	outval[3];

/*
    Update the graphics mask value and make sure it has not exceeded the
    maximum.
*/
xid_mask.rotate_value++;
xid_mask.rotate_value = (xid_mask.rotate_value > MASK_MAX) ? 
	1 : xid_mask.rotate_value;

/*
    Output the new value.
*/
sprintf(outval,"%d",xid_mask.rotate_value);
xid_mask.graphics_mask = xid_mask.rotate_value - 1;
XmTextSetString(mask_plane_text,outval);
}
