#include "xid.h"
#include "proj.h"

FUNCTION  proj_flush(proj_code,unit,zone,datum,proj_coef)
long	datum,			/* Datum code.  (I)			      */
	proj_code,		/* Projection code. (I)			      */
	unit,			/* Unit code.  (I)			      */
	zone;			/* Zone code.  (I)			      */

double	proj_coef[];		/* Projection parameters.  (I)		      */

/*******************************************************************************
Purpose:
    This routine is used to force the initialization of global values within
    the GCTP routines.  If GCTP routines are called with the same projection
    as the previous call but with different parameters, global values are not
    initialized properly.  This routine forces that initialization by changing
    the zone code and calling c_trans.  This routine would become unnecessary
    if the GCTP routines were corrected to initialize properly or if the
    GCTP routines were replaced with some other package.

By L. Huewe, USGS/EROS Data Center, 10/92

Algorithm Description:
*******************************************************************************/
{
static 	double	projparm[15] = {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,
    0.0,0.0,0.0,0.0}; 		/* Projection parameters.		      */

double	inx = 0.0,		/* Input X coordinate.			      */
	iny = 0.0,		/* Input Y coordinate.			      */
	outx,			/* Output X coordinate.			      */
	outy;			/* Output Y coordinate.			      */

long	outdatum = CLARKE,	/* Output datum code.			      */
	outproj = 0,		/* Output projection code.		      */
	outunit = DEGREES,	/* Output unit code.			      */
	outzone = NULZONE,	/* Output zone code.			      */
	prtprm[2],		/* Print parameters.			      */
	status;			/* Status code.				      */
	
/*
    If the input projection code is UTM, increment the zone.  If the resulting
    zone code is greater than 60, set it back to 1.
*/
if (proj_code == UTM)
    (zone < 59) ? zone++ : (zone = 1);
/*
    Make sure the zone is changed.
*/
else
    (zone == 62) ? (zone = 63) : zone++;

/*
    Initialize the projection package.
*/
prtprm[0] = prtprm[1] = FALSE;
status = c_transinit(&proj_code,&unit,&zone,&datum,proj_coef,&outproj,&outunit,
    &outzone,&outdatum,projparm,prtprm,"");
if (status != E_SUCC)
    return(XID_F);

/*
    Call c_trans (with modified zone code), which forces the initialization
    of the proper variables.
*/
status = c_trans(&proj_code,&unit,&outproj,&outunit,&inx,&iny,&outx,&outy);
if (status != E_SUCC)
    return(XID_F);

return(XID_S);
}
