/*
 *  This is the show function source file for the show panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include "xid.h"

void
show_accept(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine is used to display the appropriate image data in the specified
    RGB order.

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
{
extern void xid_find_composite();
				/* Function to find the composite index of the 
				   specified entry.			      */

Arg	args[1];		/* Variable used to set widget resources.     */

long	composite_index,	/* Composite index.			      */
	i,			/* Looping index.			      */
	nplanes,		/* Number of planes to be displayed.	      */
	show_save = FALSE,	/* Indicates whether or not a saved composite
				   image is being displayed.		      */
	zero;			/* Number of planes which are zero.	      */

/*
    Check whether or not a saved composite view is being displayed.
*/
if (xid_show.planes[RED] == -1)
    {
    if (strcmp(xid_show.composite,"Current") == 0)
	return;
    show_save = TRUE;
/*
    Find the specified view.
*/
    xid_find_composite(xid_show.composite,&composite_index);
    if (composite_index < 0)
	{
        xid_message("Unable to find specified composite image.");
        return;
	}
/*
    Retrieve the plane numbers of the specified view.
*/
    for (i=0; i < 3; i++)
	xid_show.planes[i] = xid_composite[composite_index].rgbplanes[i];
/*
    Deselect all items in the save list.
*/
    XmListDeselectAllItems(show_savelist);
    }
/*
    Deselect all items in the red, green and blue lists.
*/
else
    {
    XmListDeselectAllItems(show_redlist);
    XmListDeselectAllItems(show_greenlist);
    XmListDeselectAllItems(show_bluelist);
    }

/*
    If a value was not specified for one of the display components, copy it
    from a specified component.
*/
for (i=0,zero=0; i < 3; i++)
    if (xid_show.planes[i] == 0)
	{
	zero++;
	switch(i)
	    {
	    case 0:
		xid_show.planes[i] = (xid_show.planes[i+1] > 0) ? 
		    xid_show.planes[i+1] : xid_show.planes[i+2]; 
		break;
	    case 1:
		xid_show.planes[i] = (xid_show.planes[i-1] > 0) ? 
		    xid_show.planes[i-1] : xid_show.planes[i+1]; 
		break;
	    case 2:
		xid_show.planes[i] = (xid_show.planes[i-1] > 0) ? 
		    xid_show.planes[i-1] : xid_show.planes[i-2]; 
		break;
	    }
	}
/*
    If no values were specified, just return.
*/
if (zero == 3)
    return;

/*
    Determine the number of planes to display.
*/
if ((xid_show.planes[RED] == xid_show.planes[GREEN]) && 
    (xid_show.planes[GREEN] == xid_show.planes[BLUE]))
	nplanes = 1;
else
	{
	nplanes = 3;
/*
    Report an error if the specified planes are not all the same size.
*/
	if ((xid_planes[xid_show.planes[RED] - 1].dspwin[NL] != 
	    xid_planes[xid_show.planes[GREEN] - 1].dspwin[NL]) ||
	    (xid_planes[xid_show.planes[GREEN] - 1].dspwin[NL] !=
	    xid_planes[xid_show.planes[BLUE] - 1].dspwin[NL]))
		{
	    	xid_show.planes[RED] = xid_show.planes[GREEN] = 
		    xid_show.planes[BLUE] = 0;
		xid_message("Image planes are different sizes");
		return;
		}
	if ((xid_planes[xid_show.planes[RED] - 1].dspwin[NS] != 
	    xid_planes[xid_show.planes[GREEN] - 1].dspwin[NS]) ||
	    (xid_planes[xid_show.planes[GREEN] - 1].dspwin[NS] !=
	    xid_planes[xid_show.planes[BLUE] - 1].dspwin[NS]))
		{
	    	xid_show.planes[RED] = xid_show.planes[GREEN] = 
		    xid_show.planes[BLUE] = 0;
		xid_message("Image planes are different sizes");
		return;
		}
	}

/*
    Free any previously allocated pixmaps.
*/
if (xid_mem.imgpix)
    {
    XFreePixmap(xid.display,xid_mem.image_pixmap);
    xid_mem.imgpix = FALSE;
    xid_mem.image_pixmap = (Pixmap)NULL;
    }

/*
    Call load_display to display the appropriate image data in the
    appropriate RGB order.
*/
if (nplanes == 1)
    load_display(xid_show.planes,&nplanes,XtWindow(main_image_bulletinboard),
	Shell_MainWin,main_image_bulletinboard,
	&xid_planes[xid_show.planes[RED] - 1].dspwin[NL],
	&xid_planes[xid_show.planes[RED] - 1].dspwin[NS],
	&xid_planes[xid_show.planes[RED] - 1].maxval,
	xid_planes[xid_show.planes[RED] - 1].image_data,NULL,NULL,TRUE,
	&xid_mem.image_pixmap,TRUE,TRUE,FALSE);
else
    load_display(xid_show.planes,&nplanes,XtWindow(main_image_bulletinboard),
	Shell_MainWin,main_image_bulletinboard,
        &xid_planes[xid_show.planes[RED] - 1].dspwin[NL],
        &xid_planes[xid_show.planes[RED] - 1].dspwin[NS],
	&xid_planes[xid_show.planes[RED] - 1].maxval,
	xid_planes[xid_show.planes[RED] - 1].image_data,
	xid_planes[xid_show.planes[GREEN] - 1].image_data,
	xid_planes[xid_show.planes[BLUE] - 1].image_data,TRUE,
	&xid_mem.image_pixmap,TRUE,TRUE,FALSE);

/*
    Apply the appropriate color table.
*/
if (show_save)
    {
    XStoreColors(xid.display,*xid.colormap_ptr,xid_composite[composite_index].
	color_table,256);
    for (i=0; i < 256; i++)
	xid.colortable_ptr[i] = xid_composite[composite_index].color_table[i];

    if (xid.red_from != NULL)
	{
	free(xid.red_from);
	free(xid.red_to);
	}
    xid.red_from = (long *)(calloc(xid_composite[composite_index].red_count, 
	sizeof(long)));
    xid.red_to = (long *)(calloc(xid_composite[composite_index].red_count, 
	sizeof(long)));
    xid.red_count = xid_composite[composite_index].red_count;
    for (i=0; i < xid_composite[composite_index].red_count; i++)
	{
	xid.red_from[i] = xid_composite[composite_index].red_from[i];
	xid.red_to[i] = xid_composite[composite_index].red_to[i];
	}

    if (xid.green_from != NULL)
	{
	free(xid.green_from);
	free(xid.green_to);
	}
    xid.green_from = (long *)(calloc(xid_composite[composite_index].green_count,
	sizeof(long)));
    xid.green_to = (long *)(calloc(xid_composite[composite_index].green_count, 
	sizeof(long)));
    xid.green_count = xid_composite[composite_index].green_count;
    for (i=0; i < xid_composite[composite_index].green_count; i++)
	{
	xid.green_from[i] = xid_composite[composite_index].green_from[i];
	xid.green_to[i] = xid_composite[composite_index].green_to[i];
	}

    if (xid.blue_from != NULL)
	{
	free(xid.blue_from);
	free(xid.blue_to);
	}
    xid.blue_from = (long *)(calloc(xid_composite[composite_index].blue_count,
	sizeof(long)));
    xid.blue_to = (long *)(calloc(xid_composite[composite_index].blue_count, 
	sizeof(long)));
    xid.blue_count = xid_composite[composite_index].blue_count;
    for (i=0; i < xid_composite[composite_index].blue_count; i++)
	{
	xid.blue_from[i] = xid_composite[composite_index].blue_from[i];
	xid.blue_to[i] = xid_composite[composite_index].blue_to[i];
	}
    }
else
    {
    if (xid.red_from != NULL)
	{
	free(xid.red_from);
	free(xid.red_to);
	}
    xid.red_from = (long *)(calloc(2, sizeof(long)));
    xid.red_to = (long *)(calloc(2, sizeof(long)));
    xid.red_count = 2;
    xid.red_from[0] = xid.red_to[0] = 0;
    xid.red_from[1] = xid.red_to[1] = 255;

    if (xid.green_from != NULL)
	{
	free(xid.green_from);
	free(xid.green_to);
	}
    xid.green_from = (long *)(calloc(2, sizeof(long)));
    xid.green_to = (long *)(calloc(2, sizeof(long)));
    xid.green_count = 2;
    xid.green_from[0] = xid.green_to[0] = 0;
    xid.green_from[1] = xid.green_to[1] = 255;

    if (xid.blue_from != NULL)
	{
	free(xid.blue_from);
	free(xid.blue_to);
	}
    xid.blue_from = (long *)(calloc(2, sizeof(long)));
    xid.blue_to = (long *)(calloc(2, sizeof(long)));
    xid.blue_count = 2;
    xid.blue_from[0] = xid.blue_to[0] = 0;
    xid.blue_from[1] = xid.blue_to[1] = 255;
    }

/*
    Change the title to reflect the name of the image displayed in the
    main image window.
*/
XtSetArg(args[0],XmNtitle,xid_planes[xid_show.planes[RED] - 1].filename);
XtSetValues(Shell_MainWin,args,1);

xid_show.planes[RED] = xid_show.planes[GREEN] = xid_show.planes[BLUE] = 0;
}

void
show_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine closes the show panel. 

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
{
/*
    Close the show panel.
*/
XtUnmanageChild(Show_form);
XtPopdown(Shell_Show);

/*
    Set the show button on the main panel to sensitive.
*/
XtSetSensitive(main_show_button,True);
}

void
show_select(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine is called each time the user selects an item from the
    red, green or blue list or from the save list.  The value selected
    is saved until the combination is accepted (using the "Accept" button).

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
long    plane_num;                      /* Plane number selected.             */

char    text[6],                        /* Dummy text variable.               */
        *tmpstr;                        /* Temporary string containing the
                                           value of the selected list element.*/

XmListCallbackStruct *acs=(XmListCallbackStruct*)call;

/*
    Get the item the user selected.
*/
XmStringGetLtoR(acs->item,XmSTRING_DEFAULT_CHARSET,&tmpstr);

/*
   Deselect all items in the save list and extract the plane number from the 
   text string.
*/
if (*client != 'S')
    {
    sscanf(tmpstr,"%s%d",text,&plane_num);
    XmListDeselectAllItems(show_savelist);
    XtSetSensitive(show_delete_button,False);
    }
/*
    Deselect all items in the red, green and blue lists.
*/
else
    {
    XmListDeselectAllItems(show_redlist);
    XmListDeselectAllItems(show_greenlist);
    XmListDeselectAllItems(show_bluelist);
    XtSetSensitive(show_delete_button,True);
    }

/*
    Assign the plane number to the appropriate element in the RGB array.
*/
if (*client == 'R')
    xid_show.planes[RED] = plane_num; 
else if (*client == 'G')
    xid_show.planes[GREEN] = plane_num; 
else if (*client == 'B')
    xid_show.planes[BLUE] = plane_num; 
/*
    If a saved composite name was selected, assign the plane numbers to a
    special value and retrieve the composite name.
*/
else if (*client == 'S')
    {
    xid_show.planes[RED] = xid_show.planes[GREEN] = xid_show.planes[BLUE] = -1;
    xid_show.composite = (char *)malloc(strlen(tmpstr) + 1);
    if (xid_show.composite == NULL)
	{
	xid_message("Insufficient memory available");
	return;
	}
    strcpy(xid_show.composite,tmpstr);
    }

/*
    Free appropriate resources.
*/
free(tmpstr);
}

void
show_save_dialog(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine pops up the dialog box prompting the user for a composite
    name.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{

/*
    Pop up the dialog shell.
*/
XtManageChild(show_dialog_form);
}

void
show_save(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine saves the currently displayed image view with the user
    specified name.  Information saved includes the image plane combination 
    as well as the colormap applied.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
extern void xid_find_composite(),
				/* Function to find the composite index of the 
				   specified entry.			      */
	    xid_get_composite();/* Function to find the first free composite 
				   entry.				      */
XmString
	xmstring;		/* Motif string.			      */

long	dup_index,		/* Index of duplicate entry.		      */
	i,			/* Looping index.			      */
	index,			/* Free composite index.		      */
	len,			/* String length.			      */
	num_composite;		/* Number of composite entries used.	      */

char	*name;			/* Save image name.		 	      */

/*
    Make sure an image is displayed.
*/
if (xid.view == -1)
    {
    xid_message("No image data currently being viewed");
    return;
    }

/*
    Get the next free composite view index.
*/
xid_get_composite(&index,&num_composite);

/*
    Get the saved image name.
*/
name = XmTextFieldGetString(show_save_text);

/*
    Make sure a name was specified.
*/
if (strcmp(name,"") == 0)
    return;

/*
    Check for duplicate name.
*/
xid_find_composite(name,&dup_index);
if (dup_index >= 0)
    {
    xid_message("Composite name already exists");
    return;
    }

/*
    Copy the name into the composite structure.
*/
xid_composite[index].name = (char *)malloc(strlen(name) + 1);
if (xid_composite[index].name == NULL)
    {
    xid_message("Insufficient memory available");
    return;
    }
strcpy(xid_composite[index].name,name);

/*
    Convert the name to lower case.
*/
len = strlen(xid_composite[index].name);
c_up2low(xid_composite[index].name,&len);

/*
    Save the memory plane combination.
*/
for (i=0; i < 3; i++)
    xid_composite[index].rgbplanes[i] = xid_view.rgbplanes[i];

/*
    Save the color table.
*/
for (i=0; i < 256; i++)
    xid_composite[index].color_table[i] = xid.colortable_ptr[i];

/*
    Save the associated break points.
*/
if (xid_composite[index].red_from != NULL)
    {
    free(xid_composite[index].red_from);
    free(xid_composite[index].red_to);
    }
xid_composite[index].red_from = (long *)(calloc(xid.red_count, sizeof(long)));
xid_composite[index].red_to = (long *)(calloc(xid.red_count, sizeof(long)));
xid_composite[index].red_count = xid.red_count;
for (i=0; i < xid.red_count; i++)
    {
    xid_composite[index].red_from[i] = xid.red_from[i];
    xid_composite[index].red_to[i] = xid.red_to[i];
    }

if (xid_composite[index].green_from != NULL)
    {
    free(xid_composite[index].green_from);
    free(xid_composite[index].green_to);
    }
xid_composite[index].green_from = (long *)(calloc(xid.green_count, 
    sizeof(long)));
xid_composite[index].green_to = (long *)(calloc(xid.green_count, sizeof(long)));
xid_composite[index].green_count = xid.green_count;
for (i=0; i < xid.green_count; i++)
    {
    xid_composite[index].green_from[i] = xid.green_from[i];
    xid_composite[index].green_to[i] = xid.green_to[i];
    }

if (xid_composite[index].blue_from != NULL)
    {
    free(xid_composite[index].blue_from);
    free(xid_composite[index].blue_to);
    }
xid_composite[index].blue_from = (long *)(calloc(xid.blue_count, sizeof(long)));
xid_composite[index].blue_to = (long *)(calloc(xid.blue_count, sizeof(long)));
xid_composite[index].blue_count = xid.blue_count;
for (i=0; i < xid.blue_count; i++)
    {
    xid_composite[index].blue_from[i] = xid.blue_from[i];
    xid_composite[index].blue_to[i] = xid.blue_to[i];
    }

/*
    Mark the entry as used.
*/
xid_composite[index].flags |= USED;

/*
    Add this name to the show save list as well as the flicker save list
    if necessary.
*/
xmstring = XmStringCreateLtoR(xid_composite[index].name,
    XmSTRING_DEFAULT_CHARSET);
XmListAddItem(show_savelist,xmstring,2);

if (xid_flicker.flags & FLICKER_ACTIVE)
    XmListAddItem(flicker_savelist,xmstring,1);

/*
    Clear the text field and pop down the dialog shell.
*/
XmTextFieldSetString(show_save_text,"");
XtUnmanageChild(show_dialog_form);

/*
    Free appropriate resources.
*/
XtFree(name);
XmStringFree(xmstring);
}

void
show_info(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine pops up a panel displaying to the user information about
    the selected image plane.  Information reported includes the image file
    name, the window loaded, the band number and the subsampling factor.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
XmString
        xmstring;               /* Motif string.                              */

long	plane_num;		/* Plane number to retrieve information for.  */

char    *tmpstr,		/* Temporary string variable.		      */
	string[100],		/* Image window values.			      */
	text[10];		/* Temporary text string.		      */

XmListCallbackStruct *acs=(XmListCallbackStruct*)call;

/*
    Get the plane number for which the information is to be displayed.
*/
XmStringGetLtoR(acs->item,XmSTRING_DEFAULT_CHARSET,&tmpstr);
sscanf(tmpstr,"%s%d",text,&plane_num);
free(tmpstr);

/*
    Display the title (including the selected plane number).
*/
sprintf(string,"Image Information for Plane %d\n",plane_num);
xmstring = XmStringCreateLtoR(string,XmSTRING_DEFAULT_CHARSET);
XtVaSetValues(info_title,XmNlabelString,xmstring,NULL);
XmStringFree(xmstring);

/*
    Display the image file name.
*/
XmTextFieldSetString(info_filename_text,xid_planes[plane_num-1].filename);

/*
    Display the window that was loaded.
*/
sprintf(string,"(%d, %d, %d, %d)",
    xid_planes[plane_num-1].window[SL],xid_planes[plane_num-1].window[SS],
    xid_planes[plane_num-1].window[NL],xid_planes[plane_num-1].window[NS]);
XmTextSetString(info_window_text,string);

/*
    Display the band number.
*/
sprintf(string,"%d",xid_planes[plane_num-1].band);
XmTextFieldSetString(info_band_text,string);

/*
    Display the subsampling factor.
*/
sprintf(string,"%d",xid_planes[plane_num-1].skipl);
XmTextFieldSetString(info_subsamp_text,string);

/*
    Pop up the dialog shell.
*/
XtManageChild(info_form);
}

void
info_destroy(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine pops down the image plane information panel.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{

/*
    Pop down the dialog shell.
*/
XtUnmanageChild(info_form);
}

void
show_planes(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine highlights or selects the plane numbers representing the
    selected composite view.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
extern void xid_find_composite();
				/* Function to find the composite index of the 
				   specified entry.			      */
XmString
	xmstring;		/* Motif string.			      */

long	composite_index,	/* Index into composite structure.	      */
	i;			/* Looping index.			      */

char	*composite,		/* Composite view name selected.	      */
	planestr[20];		/* Plane number string.			      */

/*
    Highlight the planes representing the selected composite view.
*/
XmListCallbackStruct *acs=(XmListCallbackStruct*)call;

/*
    Get the name of the selected composite view.
*/
XmStringGetLtoR(acs->item,XmSTRING_DEFAULT_CHARSET,&composite);

/*
    "Current" is a special case.  It is not a saved composite view, it is the
    currently displayed image.
*/
if (strcmp(composite,"Current") == 0)
    {
    for (i=0; i < 3; i++)
	{
	sprintf(planestr,"Plane %d",xid_view.rgbplanes[i]);
	xmstring = XmStringCreateLtoR(planestr,XmSTRING_DEFAULT_CHARSET);
	if (i == 0)
	    XmListSelectItem(show_redlist,xmstring,True);
	else if (i == 1)
	    XmListSelectItem(show_greenlist,xmstring,True);
	else
	    XmListSelectItem(show_bluelist,xmstring,True);
	XmStringFree(xmstring);
	}
    }
/*
    Find the composite view index and plane numbers.
*/
else
    {
    xid_find_composite(composite,&composite_index);
    if (composite_index < 0)
	{
	xid_message("Unable to find specified composite image.");
	return;
	}
	
    for (i=0; i < 3; i++)
	{
	sprintf(planestr,"Plane %d",xid_composite[composite_index].
	    rgbplanes[i]);
	xmstring = XmStringCreateLtoR(planestr,XmSTRING_DEFAULT_CHARSET);
	if (i == 0)
	    XmListSelectItem(show_redlist,xmstring,True);
	else if (i == 1)
	    XmListSelectItem(show_greenlist,xmstring,True);
	else
	    XmListSelectItem(show_bluelist,xmstring,True);
	XmStringFree(xmstring);
	}
    }

/*
    Free appropriate resources.
*/
free(composite);
}

void
show_save_cancel(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine pops down the save panel without saving a new composite
    view.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
/*
    Clear the text field and pop down the dialog shell.
*/
XmTextFieldSetString(show_save_text,"");
XtUnmanageChild(show_dialog_form);

return;
}

void
show_delete(w, client, call)
Widget w;
caddr_t client;
caddr_t call;

/*******************************************************************************
Purpose:
    This routine deletes the specified composite view.

By L. Huewe, USGS/EROS Data Center, 07/93

Algorithm Description:
*******************************************************************************/
{
extern void xid_find_composite();
				/* Function to find the composite index of the 
				   specified entry.			      */
XmString
	xmstring;               /* Motif string.                              */

long	composite_index;	/* Composite index.			      */

/*
    Find the composite view to be removed. 
*/
xid_show.planes[RED] = xid_show.planes[GREEN] = xid_show.planes[BLUE] = 0;
if (strcmp(xid_show.composite,"Current") == 0)
    return;
xid_find_composite(xid_show.composite,&composite_index);
if (composite_index < 0)
    {
    xid_message("Unable to find specified composite image.");
    return;
    }

/*
    Remove the entry from the any currently displayed save list.
*/
xmstring = XmStringCreateLtoR(xid_composite[composite_index].name,
    XmSTRING_DEFAULT_CHARSET);
XmListDeleteItem(show_savelist,xmstring);

if (xid_flicker.flags & FLICKER_ACTIVE)
    XmListDeleteItem(flicker_savelist,xmstring);
XmStringFree(xmstring);

/*
    Reset the appropriate fields indicating this composite view has been
    removed.
*/
xid_composite[composite_index].rgbplanes[RED] = xid_composite[composite_index].
    rgbplanes[GREEN] = xid_composite[composite_index].rgbplanes[BLUE] = 0;
xid_composite[composite_index].flags &= (~USED);
free(xid_composite[composite_index].name);
free(xid_composite[composite_index].red_from);
free(xid_composite[composite_index].red_to);
free(xid_composite[composite_index].green_from);
free(xid_composite[composite_index].green_to);
free(xid_composite[composite_index].blue_from);
free(xid_composite[composite_index].blue_to);
}
