#include "xid.h"

void
xid_win2file(plane_idx,win,xwin,ywin,xfil,yfil)

long	plane_idx,		/* Memory plane index. (I)		      */
	win,			/* Window the input coordinates are from. (I)
					0 - Main image window
					1 - Full resolution window
					2 - Zoomed window		      */
	xwin,			/* 0 relative X coordinate. (I)		      */
	ywin;			/* 0 relative Y coordinate. (I)		      */

float	*xfil,			/* Calculated X file coordinate. (O)	      */
	*yfil;			/* Calculated Y file coordinate. (O)	      */

/*******************************************************************************
Purpose:
	This routine converts from window coordinates to file coordinates.
	If the input window coordinates are from the zoomed window, they
	are assumed to be subpixel coordinates and are converted accordingly.

By L. Huewe, USGS/EROS Data Center, 5/91

Algorithm Description:
*******************************************************************************/
{
double	xsub,		/* X coordinate (subpixel accuracy).		      */
	ysub;		/* Y coordinate (subpixel accuracy).		      */

/*
    Main window coordinates.

    fc = (((mwc + 1) * skip) + ss_sl - 1) - (skip - 1)

    fc	  = 	file coordinate
    mwc	  = 	main window coordinate
    skip  = 	subsampling factor
    ss_sl = 	starting line or starting sample
*/
if (win == 0)
    {
    *xfil = (float) ((((xwin + 1) * xid_planes[plane_idx].skips) +
	xid_planes[plane_idx].window[SS] - 1) - (xid_planes[plane_idx].skips
	- 1));
    *yfil = (float) ((((ywin + 1) * xid_planes[plane_idx].skipl) +
	xid_planes[plane_idx].window[SL] - 1) - (xid_planes[plane_idx].skipl
	- 1));
    }
/*
    Full resolution window coordinates.

    fc = ss_sl + frwc

    fc	  = 	file coordinate
    frwc  = 	full resolution window coordinate
    ss_sl = 	starting line or starting sample
*/
else if (win == 1)
    {
    *xfil = (float) (xid_fullres[0].window[SS] + xwin); 
    *yfil = (float) (xid_fullres[0].window[SL] + ywin); 
    }
/*
    Zoom window coordinates.  The routine cursor_sub converts zoom window
    coordinates (subpixel) to file coordinates.
*/
else if (win == 2)
    {
    if (xid_zoom.flags & ZOOM_MAIN)
	cursor_sub(xwin,ywin,&xsub,&ysub,xid_planes[plane_idx].skipl,
	    xid_planes[plane_idx].skips,xid_planes[plane_idx].window[SL],
	    xid_planes[plane_idx].window[SS]);
    else if (xid_zoom.flags & ZOOM_FULLRES)
	cursor_sub(xwin,ywin,&xsub,&ysub,1,1,xid_fullres[0].window[SL],
	    xid_fullres[0].window[SS]);
    *xfil = (float)xsub;
    *yfil = (float)ysub;
    }
}

void
xid_file2win(plane_idx,win,xfil,yfil,xwin,ywin)

long	plane_idx,		/* Memory plane index. (I)		      */
	win,			/* Window the coordinates are to be converted
				   to. (I)
					0 - Main image window
					1 - Full resolution window
					2 - Zoomed window		      */
	*xwin,			/* Calculated 0 relative X coordinate. (0)    */
	*ywin;			/* Calculated 0 relative Y coordinate. (0)    */

float	xfil,			/* X file coordinate. (I)	      	      */
	yfil;			/* Y file coordinate. (I)	      	      */

/*******************************************************************************
Purpose:
	This routine converts file coordinates to the specified window
	coordinates.

By L. Huewe, USGS/EROS Data Center, 5/92

Algorithm Description:
*******************************************************************************/
{
/*
    Convert to main window coordinates.

    mw = ((fc + (skip - 1) - (ss_sl - 1)) / skip) - 1

    mw	  = 	main window coordinate
    fc	  = 	file coordinate
    skip  = 	subsampling factor
    ss_sl =	starting sample or starting line
*/
if (win == 0)
    {
    *xwin = (((long)xfil + (xid_planes[plane_idx].skips - 1) - 
	(xid_planes[plane_idx].window[SS] - 1)) / xid_planes[plane_idx].skips) 
	- 1;
    *ywin = (((long)yfil + (xid_planes[plane_idx].skipl - 1) - 
	(xid_planes[plane_idx].window[SL] - 1)) / xid_planes[plane_idx].skipl) 
	- 1;
    }
/*
    Convert to full resolution window coordinates.

    frw = fc + ss_sl

    frw	  = 	full resolution window coordinate
    fc	  = 	file coordinate
    ss_sl =	starting sample or starting line
*/
else if (win == 1)
    {
    *xwin = ((long)xfil - xid_fullres[0].window[SS]);
    *ywin = ((long)yfil - xid_fullres[0].window[SL]);
    }
/*
    Zoom window coordinates.  The routine xid_sub2win converts file
    coordinates to zoom window coordinates (subpixel). 
*/
else if ((win == 2) && (xid_zoom.flags & ZOOM_MAIN))
    xid_sub2win((double)xfil,(double)yfil,xwin,ywin,xid_planes[plane_idx].skipl,
	xid_planes[plane_idx].skips,xid_planes[plane_idx].window[SL],
	xid_planes[plane_idx].window[SS]);
else if ((win == 2) && (xid_zoom.flags & ZOOM_FULLRES))
    xid_sub2win((double)xfil,(double)yfil,xwin,ywin,1,1,xid_fullres[0].
	window[SL],xid_fullres[0].window[SS]);

}

xid_sub2win(xsub,ysub,xcursor,ycursor,skipl,skips,fsl,fss)
double	xsub,			/* X location of input point (subpixel
				   accuracy) (I)	      		      */
	ysub;			/* Y location of input point (subpixel
				   accuracy) (I)	      		      */

long	skipl,			/* Line subsampling factor (I)	 	      */
	skips,			/* Sample subsampling factor (I)	      */
	fsl,			/* Starting line (file coordinate) of the image
				   the coordinate is being calculated for (I) */
	fss,			/* Starting sample (file coordinate) of the 
				   image the coordinate is being calculated 
				   for (I) 				      */
	*xcursor,		/* Zero relative X position in window. (O)    */
	*ycursor;		/* Zero relative Y position in window. (O)    */
/*******************************************************************************
Purpose:
	This routine converts file coordinates to subpixel coordinates
	(i.e., zoom window coordinates).

By L. Huewe, USGS/EROS Data Center, 5/92

Algorithm Description:
*******************************************************************************/
{
double	x,			/* X coordinate		      		      */
	xfactor,		/* Subpixel factor to add to file coordinate. */
	xyadjust,		/* Adjustment needed to the subpixel value
				   due to short cuts taken when the image 
				   window was enlarged	      		      */
	y,			/* Y coordinate		      		      */
	yfactor;		/* Subpixel factor to add to file coordinate. */

long 	lborder,		/* Number of pixels thrown away from the left
				   edge of the extracted window		      */
	left,			/* Remainder of a division operation	      */
	sl,			/* Starting line of extracted window.	      */
	ss,			/* Startling sample of extracted window.      */
	tborder,		/* Number of pixels thrown away from the top
			  	   edge of the extracted window		      */
	xfil,			/* X file relative coordinate.		      */
	ximg,			/* X image relative coordinate.		      */
	yfil,			/* Y file relative coordinate.		      */
	yimg;			/* Y image relatvie coordinate.		      */
/*
   Determine how many pixels from the window extracted for zooming were thrown
   away by the resampling routine. 
*/
	left = xid_zoom.parms.horiz_dim % 2;
	if (left == 0)
	    lborder = xid_zoom.parms.horiz_dim / 2.0 - 1;
	else
	    lborder = xid_zoom.parms.horiz_dim / 2.0;

	left = xid_zoom.parms.vert_dim % 2;
	if (left == 0)
	    tborder = xid_zoom.parms.vert_dim / 2.0 - 1;
	else
	    tborder = xid_zoom.parms.vert_dim / 2.0;
/*
   Calculate the adjustment needed due to short cuts taken when the image
   window was zoomed.
*/
	xyadjust = ((int)(xid_zoom.parms.factor / 2.0)) * (1.0 / 
	    (double)(xid_zoom.parms.factor));  
/*
    Determine the starting line and sample based on the window extracted and
    factor this into the file coordinate.
*/
	ss = (xid_zoom.window[SS] * skips) + 1;
	sl = (xid_zoom.window[SL] * skipl) + 1;

	xfil = (long)xsub - (ss + lborder - 1);
	yfil = (long)ysub - (sl + tborder - 1);

/*
    Calculate the image coordinate.
*/
	ximg = (xfil + (skips - 1) - (fss - 1)) / skips;
	yimg = (yfil + (skipl - 1) - (fsl - 1)) / skipl;

/*
    Calculate the subpixel portion of the coordinate and factor it into the
    image coordinate.
*/
	xfactor = xsub - (long)xsub;
	yfactor = ysub - (long)ysub;

	x = ximg + xfactor;
	y = yimg + yfactor;

	*xcursor = ((x - xyadjust) - .5 - (1.0 / (2 * xid_zoom.factor))) *
	    xid_zoom.factor;
	*ycursor = ((y - xyadjust) - .5 - (1.0 / (2 * xid_zoom.factor))) *
	    xid_zoom.factor;

	return;
}

long
xid_win2proj(plane_idx,win,xwin,ywin,xproj,yproj)

long	plane_idx,		/* Memory plane index. (I)		      */
	win,			/* Window the input coordinates are from. (I)
					0 - Main image window
					1 - Full resolution window
					2 - Zoomed window		      */
	xwin,			/* 0 relative X coordinate. (I)		      */
	ywin;			/* 0 relative Y coordinate. (I)		      */

double	*xproj,			/* Calculated X projection coordinate. (O)    */
	*yproj;			/* Calculated Y projection coordinate. (O)    */

/*******************************************************************************
Purpose:
	This routine converts from window coordinates to projection coordinates.
	If the input window coordinates are from the zoomed window, they
	are assumed to be subpixel coordinates and are converted accordingly.

By L. Huewe, USGS/EROS Data Center, 9/93

Algorithm Description:
*******************************************************************************/
{
double	coef[6],        /* Coefficients mapping image coordinates to 
			   projection coordinates.                 	      */
	xsub,           /* Current sample (subpixel accuracy).                */
        ysub;           /* Current line (subpixel accuracy).                  */

long    curx,           /* Current X file coordinate.                         */
        cury,           /* Current Y file coordinate.                         */
	icoords[8],     /* Image corner line/sample coordinates.              */
        msgflg = FALSE, /* Specifies that error messages are not to be
                           output by the routine.                             */
	status;		/* Status indicator.			 	      */


/*
    Find the transformation coefficients mapping image coordinates to
    projection coordinates.
*/
icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
icoords[2] = icoords[6] = xid_planes[plane_idx].image_info->nl;
icoords[5] = icoords[7] = xid_planes[plane_idx].image_info->ns;

status = c_rtcoef(xid_planes[plane_idx].image_info->upleft,icoords,coef,
    &msgflg);

/*
    If the coefficients can't be calculated then return. 
*/
if (status != E_SUCC)
    return(XID_F);

/*
    Convert the coordinates to (one-relative) file coordinates.
*/
if (win == 0)
    {
    curx = (((xwin + 1) * xid_planes[plane_idx].skips) + xid_planes[plane_idx].
	window[SS] - 1) - (xid_planes[plane_idx].skips - 1);
    cury = (((ywin + 1) * xid_planes[plane_idx].skipl) + xid_planes[plane_idx].
        window[SL] - 1) - (xid_planes[plane_idx].skipl - 1);

    xsub = (double)curx;
    ysub = (double)cury;
    }
else if (win == 1)
    {
/*
    Calculate the full resolution file coordinate.
*/
    curx = xid_fullres[0].window[SS] + xwin;
    cury = xid_fullres[0].window[SL] + ywin;

    xsub = (double)curx;
    ysub = (double)cury;
    }
else if (win == 2) 
    {
/*
    Calculate the subpixel file coordinates.
*/
    if (xid_zoom.flags & ZOOM_MAIN)
        cursor_sub(xwin,ywin,&xsub,&ysub,xid_planes[plane_idx].skipl,
	    xid_planes[plane_idx].skips,xid_planes[plane_idx].window[SL],
            xid_planes[plane_idx].window[SS]);
    else if (xid_zoom.flags & ZOOM_FULLRES)
        cursor_sub(xwin,ywin,&xsub,&ysub,1,1,xid_fullres[0].window[SL],
	    xid_fullres[0].window[SS]);
    }
/*
    Calculate projection coordinates from the line/sample coordinate.
*/
*yproj = (double)(coef[0] * ysub) + (coef[1] * xsub) + coef[2];
*xproj = (double)(coef[3] * ysub) + (coef[4] * xsub) + coef[5];

return(XID_S);
}
