/*
 *  This is the utility which returns the number of the next available memory 
 *  plane to the calling routine.  GETMEM returns XID_S if an available memory
 *  plane is found and it returns XID_F if no memory planes are available. 
 *  An available memory plane is defined as an unused memory plane or an 
 *  unlocked memory plane.
 *
 *  By L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include "xid.h"

FUNCTION xid_getmem(memory)
long	*memory;		/* The memory plane ID which is available for
			  	   use by the calling function.	(O)	      */

/*******************************************************************************
Purpose:
    This routine returns the next available memory plane ID.  It looks first
    for a free memory plane and if one is not found, an unlocked memory plane.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
{
long	found = FALSE,		/* Indicates whether or not an available memory
				   plane was found.			      */
	i,			/* Looping index.			      */
	plane;			/* Working plane number.		      */

/*
    Set the output memory plane ID based on the current plane number.
*/ 
plane = *memory = ((xid.current_plane + 1) > xid.max_plane) ? 1 : 
    xid.current_plane + 1;

/*
    First look for an unused memory plane.
*/
for (i=0; i < xid.max_plane; i++)
    {
    if (xid_planes[*memory - 1].flags & USED) 
        *memory = ((*memory + 1) > xid.max_plane) ? 1 : *memory + 1;
    else
   	{
	found = TRUE;
	break;
  	}
    }

/*
    Loop until a memory plane is found which is not currently locked.
*/
if (!found)
    {
    *memory = plane;
    for (i=0; i < xid.max_plane; i++)
	{
	if (xid_planes[*memory - 1].flags & LOCK) 
	    *memory = ((*memory + 1) > xid.max_plane) ? 1 : *memory + 1;
	else
	    {
	    found = TRUE;
	    break;
	    }
	}
    }

/*
    Return the memory plane ID found and set its USED flag.
*/
if (found)
    {
    xid.current_plane = *memory;
    xid_planes[*memory-1].flags = xid_planes[*memory-1].flags | USED;
    return(XID_S);
    }
/*
    No free or unlocked memory plane was found.
*/
else
    return(XID_F);
}
