#include "xid.h"
#include <math.h>


FUNCTION xid_histcalc(hist,minb,maxb,mean,stdev)

long	*hist,          /* Actual histogram values. (I)			*/
	*minb,		/* Minimum frequency value. (O)			*/
        *maxb;		/* Maximum frequency value. (O)			*/

double	*mean,		/* Mean frequency value. (O)			*/
        *stdev;		/* Standard deviation of frequency. (O)		*/

/****************************************************************************
 *									    *
 *	Purpose:  To calculate the minimium, maximum, mean and		    *
 *		  standard deviation of a histogram.			    *
 *									    *
 *	By D. Akkerman, USGS/EROS Data Center, 1/91.			    *
 *									    *
 *	Algorithm Description:						    *
 *	    See in-line documentation.					    *
 *	Algorithm Reference:						    *
 *	    Taken from the XLASDSP application.				    *
 *									    *
 ****************************************************************************/
{
long	freq,		/* Loop variable for frequencies. 		*/
        *histptr,       /* Pointer to histogram array. 			*/
        minf,           /* Flag used in min value calculation. 		*/
        maxf;           /* Flag used in max value calculation. 		*/

double  total,          /* Total pixel counts. 				*/
        sumsq,          /* Array of sums of squares of histo values. 	*/
        sum,            /* Array of sums of histogram values. 		*/
        meansq;         /* Square of the mean. 				*/


/*
    Calculate the min, max, mean and std deviation from the histogram.
*/
sum = sumsq = total = 0.0;
histptr = hist;
minf = TRUE;
maxf = FALSE;
for (freq = 0; freq < 256; freq++, histptr++)
    {

/*
    Don't include masked out values in the calculations (i.e. those bins
    with negative counts).
*/
    if (*histptr >= 0)
	{
	sum += *histptr * freq;
	total += *histptr;
	sumsq += (double) *histptr * (freq * freq);
	}
    if (minf == TRUE)
   	if (*histptr > 0)
    	    {
	    *minb = freq;
	    minf = FALSE;
	    maxf = TRUE;
    	    }
    if (maxf == TRUE)
   	if (*histptr > 0)
    	    *maxb = freq;
    }

/*
    Check for an all zero image.
*/
if (total == 0) total = 1;

/*
    Calculate mean and standard deviation from results.
*/
*mean = sum / total;
meansq = (sum * sum) / (total * total);
*stdev = sqrt((sumsq - (total * meansq)) / total);

return (XID_S);
}
