#include "xid.h"

void
xid_image_makelist(red_widget,green_widget,blue_widget,save_widget,current_flg)
Widget	red_widget,		/* Red list widget.  (I)		      */
	green_widget,		/* Green list widget.  (I)		      */
	blue_widget,		/* Blue list widget.  (I)		      */
	save_widget;		/* Save list widget. (I)		      */

long	current_flg;		/* Indicates whether or not to make a special
				   entry in the save list for the currently
				   displayed image.  (I)		      */

/*******************************************************************************
Purpose:
    This routine builds a list of plane numbers available for manipulation
    (i.e., display, flicker, etc.) and places this list into the widgets 
    specifed by red_widget, green_widget and blue_widget.  A list of previously
    saved widgets is also built and placed into the widget specified by 
    save_widget.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
XmString *items;	/* Array of compound strings defining valid plane
			   numbers.					      */

void	xid_get_composite();
			/* Routine to return number of composite entries.     */

int	plane_cnt;	/* Number of valid planes.			      */

long	current,	/* Current index into string array.      	      */
	dummy,		/* Dummy variable (place holder for function call).   */
	i,		/* Looping index.				      */
	idx,		/* Index increment.				      */
	num_composite;	/* Number of saved composite views.		      */

char	tmpstr[50];	/* Temporary string variable.			      */

/*
    Determine the number of planes available for displaying.
*/
for (i=0,plane_cnt=0; i < xid.max_plane; i++)
    {
    if (xid_planes[i].flags & USED)
	plane_cnt++;
    }

/*
    Report a warning and clear all lists if there are no planes available for 
    display.
*/
if (plane_cnt == 0)
    {
    xid_message("No image data available for displaying");
    if (red_widget != NULL)
        XmListDeleteAllItems(red_widget);
    if (green_widget != NULL)
        XmListDeleteAllItems(green_widget);
    if (blue_widget != NULL)
        XmListDeleteAllItems(blue_widget);
    if (save_widget != NULL)
        XmListDeleteAllItems(save_widget);
    return;
    }
/*
    Allocate space for the list of plane numbers.
*/
else if ((items = (XmString *) calloc(plane_cnt,sizeof(XmString))) == NULL)
    {
    xid_message("Insufficient memory available");
    return;
    }

/*
    Assign the list of valid plane numbers to the appropriate widgets.
*/
for (i=0,current=0; i < xid.max_plane; i++)
    {
    if (xid_planes[i].flags & USED)
	{
	sprintf(tmpstr,"Plane %d",i+1);
	items[current] = XmStringCreateLtoR(tmpstr,XmSTRING_DEFAULT_CHARSET);
	current++;
	}
    }

if (red_widget != NULL)
    XtVaSetValues(red_widget,XmNitems,items,XmNitemCount,plane_cnt,NULL);
if (green_widget != NULL)
    XtVaSetValues(green_widget,XmNitems,items,XmNitemCount,plane_cnt,NULL);
if (blue_widget != NULL)
    XtVaSetValues(blue_widget,XmNitems,items,XmNitemCount,plane_cnt,NULL);

/*
    Free appropriate resources.
*/
for (i=0; i < plane_cnt; i++)
    XmStringFree(items[i]);
free(items);

/*
    Assign entries to the save list.
*/
if (save_widget == NULL)
    return;

/*
    Get the number of saved entries and allocate space for them.
*/
xid_get_composite(&dummy,&num_composite);
if ((items = (XmString *) calloc(num_composite+1,sizeof(XmString))) == NULL)
    {
    xid_message("Insufficient memory available");
    return;
    }

/*
    Make a special entry for the current image. 
*/
if ((xid.view != -1) && current_flg)
    {
    items[0] = XmStringCreateLtoR("Current",XmSTRING_DEFAULT_CHARSET);
    idx = 1;
    }
else
    idx = 0;

/*
    Assign the list of valid names to the appropriate widget.
*/
for (i=0, current=0; i < NUM_COMPOSITE; i++)
    if (xid_composite[i].flags & USED)
	{
        items[current+idx] = XmStringCreateLtoR(xid_composite[i].name,
	    XmSTRING_DEFAULT_CHARSET);
	current++;
	}
XtVaSetValues(save_widget,XmNitems,items,XmNitemCount,num_composite+idx,NULL);

/*
    Free appropriate resources.
*/
for (i=0; i < num_composite+1; i++)
    XmStringFree(items[i]);
free(items);

return;
}

long
xid_make_ximage()

/*******************************************************************************
Purpose:
    This routine creates a pixmap for the currently selected flicker image.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
XImage	*ximage;		/* X image structure.			      */

static unsigned char
	rr[256],                /* Reserved color values.                     */
	gr[256],
	br[256];

float	scaler;                 /* LUT scaler for actual number of colors.    */

long    buf_size,               /* Number of bytes in a single band.          */
	i,j,			/* Looping indices.			      */
	img_idx,		/* Index into colormap of first image color.  */
	nl,			/* Number of lines.			      */
	nplanes,		/* Number of planes (bands) to display.       */
	ns,			/* Number of samples.			      */
	num_imgcolors,		/* Number of image colors.		      */
	maxval,			/* Maximum image data value.		      */
	offset,                 /* Offset used in scaling the 24-bit LUT.     */
	tform[256];		/* Pixel transform array.		      */

unsigned long
	*data;                	/* Pointer to output values for 24-bit data.  */

unsigned char
        *blut,                  /* Blue LUT for composite image.              */
	*composite,             /* Composite RGB 8-bit image or single band
				   black white 8-bit image.                   */
        *comptr,                /* Pointer to composite buffer.               */
	*dataptr,               /* Pointer to image data stored in memory
				   plane.                                     */
	*glut,                  /* Green LUT for composite image.             */
	*rgbptr[3],             /* Pointers to RGB components.                */
	*rlut;                  /* Red LUT for composite image.               */

/*
    Set the index into the flicker arrays.
*/
j = xid_flicker.nplanes;

/*
    Calculate the image data buffer size.
*/
nl = xid_planes[xid_flicker.planes[j][0] - 1].dspwin[NL];
ns = xid_planes[xid_flicker.planes[j][0] - 1].dspwin[NS];
buf_size = nl * ns;

/*
    Determine the number of planes to display.
*/
if (((xid_flicker.planes[j][RED]) == (xid_flicker.planes[j][GREEN])) &&
    ((xid_flicker.planes[j][GREEN]) == (xid_flicker.planes[j][BLUE])))
    nplanes = 1;
else
    nplanes = 3;

maxval = xid_planes[xid_flicker.planes[j][RED] - 1].maxval;

/*
    Calculate the scaler needed to reduce the image to the appropriate number
    of colors.
*/
if ((*xid.colormap_ptr == xid.colormap) && (maxval < NUM_IMGCOLORS) &&
    (nplanes == 1))
    {
    scaler = 1.0;
    num_imgcolors = NUM_IMGCOLORS;
    img_idx = IMG_IDX;
    }
else if (*xid.colormap_ptr == xid.colormap)
    {
    scaler = (256.0 / (float)NUM_IMGCOLORS);       /* (2^8 / # of colors) */
    num_imgcolors = NUM_IMGCOLORS;
    img_idx = IMG_IDX;
    }
else 
    {
    scaler = 1.0;
    num_imgcolors = 256;
    img_idx = 0;
    }

/*
    Make sure the non-image portion of the flicker colortable has the
    appropriate values.
*/
for (i=0; i < img_idx; i++)
    {
    xid_flicker.colortable[j][i].pixel = xid.colortable_ptr[i].pixel;
    xid_flicker.colortable[j][i].blue = xid.colortable_ptr[i].blue; 
    xid_flicker.colortable[j][i].green = xid.colortable_ptr[i].green; 
    xid_flicker.colortable[j][i].red = xid.colortable_ptr[i].red; 
    xid_flicker.colortable[j][i].flags = DoRed | DoGreen | DoBlue;
    }

/*
    8-bit monitor -- black and white image.
*/
if ((xid.visual_info[0].visual->class == PseudoColor) && (nplanes == 1))
    {
    comptr = composite = (unsigned char *)malloc(buf_size);
    if (composite == NULL)
	{
	xid_message("Insufficient memory available");
	return(XID_F);
	}
/*
    Create a linear ramp.
*/
    if (!xid_flicker.cflags[j])
	{
	for (i = 0; i < num_imgcolors; i++)
	    {
	    xid_flicker.colortable[j][img_idx + i].pixel =
		xid.colortable_ptr[img_idx + i].pixel;
	    xid_flicker.colortable[j][img_idx + i].blue = xid_flicker.
		colortable[j][img_idx + i].green = xid_flicker.colortable[j]
		[img_idx + i].red = ((i * scaler) * 257);
	    xid_flicker.colortable[j][img_idx + i].flags = DoRed | DoGreen
		| DoBlue;
	    }
	}
/*
    Scale the input data.
*/
    dataptr = xid_planes[xid_flicker.planes[j][0] - 1].image_data;
    for (i=0; i < buf_size; i++,comptr++,dataptr++)
	*comptr = xid_flicker.colortable[j][(unsigned char) (img_idx + 
	    ((float) (*dataptr / scaler)))].pixel; 
    }
/*
    8-bit monitor -- RGB image.
*/ 
else if (xid.visual_info[0].visual->class == PseudoColor)
    {
/*
    Allocate buffer space for LUTs.
*/
    rlut = (unsigned char *)malloc(256);
    if (rlut == NULL)
	{
	xid_message("Insufficient memory available");
	return(XID_F);
	}
    glut = (unsigned char *)malloc(256);
    if (glut == NULL)
	{
	xid_message("Insufficient memory available");
	free(rlut);
	return(XID_F);
	}
    blut = (unsigned char *)malloc(256);
    if (blut == NULL)
	{
	xid_message("Insufficient memory available");
	free(rlut);
	free(glut);
	return(XID_F);
	}
/*
    Compress the 3 band RGB image into a single band of image data and
    3 LUTs.
*/
    comptr = composite = (unsigned char *)malloc(buf_size);
    if (composite == NULL)
	{
	xid_message("Insufficient memory available");
	free(rlut);
	free(glut);
	free(blut);
	return(XID_F);
	}
    rgbptr[RED] = xid_planes[xid_flicker.planes[j][RED] - 1].image_data; 
    rgbptr[GREEN] = xid_planes[xid_flicker.planes[j][GREEN] - 1].image_data;
    rgbptr[BLUE] = xid_planes[xid_flicker.planes[j][BLUE] - 1].image_data; 
    xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
	num_imgcolors,&composite,rlut,glut,blut,0,rr,gr,br);

/*
    Build the new colormap.
*/
    if (!xid_flicker.cflags[j])
	{
	for (i = 0; i < num_imgcolors; i++)
	    {
	    xid_flicker.colortable[j][img_idx + i].pixel =
		xid.colortable_ptr[img_idx + i].pixel;
	    xid_flicker.colortable[j][img_idx + i].red = (rlut[i] * 257);
	    xid_flicker.colortable[j][img_idx + i].green = (glut[i] * 257);
	    xid_flicker.colortable[j][img_idx + i].blue = (blut[i] * 257);
	    xid_flicker.colortable[j][img_idx + i].flags = DoRed | DoGreen 
		| DoBlue;
	    }
	}
    for (i=0; i < buf_size; i++)
	composite[i] = xid_flicker.colortable[j][img_idx + composite[i]].
	    pixel;
    }
else if ((xid.visual_info[0].visual->class == DirectColor) && 
    (nplanes == 1))
    {
/*
    Allocate buffer space.
*/
    data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
    if (data == NULL)
	{
	xid_message("Insufficient memory available");
	return(XID_F);
	}
/*
    Set a pointer to the image data.
*/
    rgbptr[RED] = xid_planes[xid_flicker.planes[j][RED] - 1].image_data; 

/*
    Create the transform array which scales output values such that 56 colormap
    entries can be reserved as working entries.
*/
    for (i = 0; i < 256; i++)
	tform[i] = (long)(((float)i / scaler) + img_idx);
/*
    Scale the input data.
*/
    for (i = 0; i < buf_size; i++, rgbptr[RED]++)
	data[i] = 0 + (tform[(*rgbptr[RED])]) + (tform[(*rgbptr[RED])] << 8)
	    + (tform[(*rgbptr[RED])] << 16);
/*
    Assign a gray scale ramp to the image data colors.
*/
    if (*xid.colormap_ptr == xid.colormap)
	offset = (0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE + 
	    NUM_WMCOLORS + NUM_WKCOLOR)));
    else
	offset = 0;

    if (!xid_flicker.cflags[j])
	{
	for (i = img_idx; i < num_imgcolors + img_idx; i++)
	    {
	    xid_flicker.colortable[j][i].pixel = xid.colortable_ptr[i].
		pixel;
	    xid_flicker.colortable[j][i].red = xid_flicker.colortable[j][i].
		green = xid_flicker.colortable[j][i].blue = 
		(long)((float)((i * scaler) + offset)) << 8;
	    xid_flicker.colortable[j][i].flags = DoRed | DoGreen | DoBlue;
	    }
	}
    }
else if (xid.visual_info[0].visual->class == DirectColor)
    {
/*
    Allocate buffer space.
*/
    data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
    if (data == NULL)
	{
	xid_message("Insufficient memory available");
	return(XID_F);
	}
/*
    Set pointers to the red, green and blue bands of image data.
*/
    rgbptr[RED] = xid_planes[xid_flicker.planes[j][RED] - 1].image_data;
    rgbptr[GREEN] = xid_planes[xid_flicker.planes[j][GREEN] - 1].image_data;
    rgbptr[BLUE] = xid_planes[xid_flicker.planes[j][BLUE] - 1].image_data;

/*
    Create the transform array which scales output values such that 56 colormap
    entries can be reserved as working entries.
*/
    for (i = 0; i < 256; i++)
	tform[i] = (long)(((float)i / scaler) + img_idx);
/*
    Scale the input data.
*/
    for (i = 0; i < buf_size; i++, rgbptr[RED]++, rgbptr[GREEN]++, 
	rgbptr[BLUE]++)
	data[i] = 0 + (tform[(*rgbptr[RED])]) + (tform[(*rgbptr[GREEN])] 
	    << 8) + (tform[(*rgbptr[BLUE])] << 16);
/*
    Assign a gray scale ramp to the image data colors.
*/
    if (*xid.colormap_ptr == xid.colormap)
	offset = (0 - (scaler * (float)(NUM_COLORS + NUM_CSTORE + 
	    NUM_WMCOLORS + 1)));
    else
	offset = 0;

    if (!xid_flicker.cflags[j])
	{
	for (i = img_idx; i < (img_idx + num_imgcolors); i++)
	    {
	    xid_flicker.colortable[j][i].pixel = xid.colortable_ptr[i].
		pixel;
	    xid_flicker.colortable[j][i].red = xid_flicker.colortable[j][i].
		green = xid_flicker.colortable[j][i].blue =
		(long)((float)((i * scaler) + offset)) << 8;
	    xid_flicker.colortable[j][i].flags = DoRed | DoGreen | DoBlue;
	    }
	}
    }

/*
    Allocate buffer space.
*/
if ((xid.visual_info[0].visual->class == PseudoColor) &&
    (xid.visual_info[0].depth == 24))
    {
    data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
    if (data == NULL)
	{
	xid_message("Insufficient memory available");
	return(XID_F);
	}
    for (i=0; i < buf_size; i++)
	data[i] = composite[i];
    free(composite);
    }

/*
    Create an X image.
*/
if (xid.visual_info[0].depth == 24)
    ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
	xid.visual_info[0].depth,ZPixmap,0,(char *)data,ns,nl,8,0);
else
    ximage = XCreateImage(xid.display,xid.visual_info[0].visual,
	xid.visual_info[0].depth,ZPixmap,0,(char *)composite,ns,nl,8,0);

/*
    Create a pixmap and write the image into it.
*/
xid_flicker.pixmaps[j] = XCreatePixmap(xid.display,XtWindow(
    main_image_bulletinboard),ns,nl,xid.visual_info[0].depth);
XPutImage(xid.display,xid_flicker.pixmaps[j],xid_flicker.gc,ximage,0,0,0,0,
   ns,nl);

/*
    Free the appropriate resources.
*/
    if (xid.visual_info[0].visual->class == DirectColor)
	free(data);
    else
	{
	free(composite);
	if (nplanes != 1)
	    {
	    free(rlut);
	    free(glut);
	    free(blut);
	    }
	}

if ((xid.visual_info[0].visual->class == PseudoColor) && (nplanes != 1))
    {
    free(rlut);
    free(glut);
    free(blut);
    }

/*
    The data in the image structure has already been freed above.
*/
ximage->data = NULL;
XDestroyImage(ximage);

return(XID_S);
}

void
xid_find_composite(name,cindex)
long	*cindex;		/* Composite index of input name.  (O)	      */

char	*name;			/* Name of the image composite whose index is
				   to be found.  (I)			      */

/*******************************************************************************
Purpose:
    This routine finds the composite index of the entry with the specified
    composite name.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
long	i;			/* Looping index.			      */

/*
    Initialize the index.
*/
*cindex = -1;

/*
    Search the composite array for a matching name.  If one is found, assign
    the appropriate index to the output parameter.
*/
for (i=0; i < NUM_COMPOSITE; i++)
    if ((xid_composite[i].flags & USED) && (strcmp(name,xid_composite[i].name)
	== 0))
	{
	*cindex = i;
	break;
	}
}

void
xid_get_composite(cindex,num_used)
long	*cindex,		/* Composite index of first free entry.  (O)  */
	*num_used;		/* Number of currently used entries (not
				   including index returned).  (O)	      */


/*******************************************************************************
Purpose:
    This routine finds the first free composite entry and returns it's index
    as well as the number of entries currently in use.

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
long	i;			/* Looping index.			      */

/*
    Initialize values.
*/
*cindex = -1;
*num_used = 0;

/*
    Count the number of composite entries and assign the parameter cindex to
    the index of the first available entry.
*/
for (i=0; i < NUM_COMPOSITE; i++)
    if ((*cindex == -1) && (!(xid_composite[i].flags & USED)))
	*cindex = i;
    else if (xid_composite[i].flags & USED)
	(*num_used)++;

/*
    Warn the user if composite entries are going to be overwritten.
*/
if (*cindex == -1)
    {
    *cindex = 0;
    xid_message("The maximum number of composite views have been saved.\nEntries are now being overwritten.");
    }
}

void
xid_clear_composite(plane_num)
long	plane_num;		/* Plane number that has been initialized. (I)*/

/*******************************************************************************
Purpose:
    This routine removes entries from the composite structure which contain
    plane numbers that have been initialized (cleared).

By L. Huewe, USGS/EROS Data Center, 7/93

Algorithm Description:
*******************************************************************************/
{
XmString	xmstring;	/* Motif format string.		  	      */

long		i;		/* Looping index.			      */

/*
    Loop through the composite structure; for each entry in use, compare the
    list of plane numbers with the plane number that was intialized.  If a 
    match is found, remove the entry from the structure as well as from 
    any displayed list (e.g., on the show panel or the flicker panel).
*/
for (i=0; i < NUM_COMPOSITE; i++)
    if (xid_composite[i].flags & USED)
	{
	if ((xid_composite[i].rgbplanes[RED] == plane_num) || (xid_composite[i].
	    rgbplanes[GREEN] == plane_num) || (xid_composite[i].rgbplanes[BLUE]
	    == plane_num))
	    {
	    xmstring = XmStringCreateLtoR(xid_composite[i].name,
		XmSTRING_DEFAULT_CHARSET);
	    if (XtIsRealized(Shell_Show))
	        XmListDeleteItem(show_savelist,xmstring);
	    if (xid_flicker.flags & FLICKER_ACTIVE)
		XmListDeleteItem(flicker_savelist,xmstring);
	    XmStringFree(xmstring);

	    xid_composite[i].rgbplanes[RED] = xid_composite[i].rgbplanes[GREEN]
		= xid_composite[i].rgbplanes[BLUE] = 0;
	    xid_composite[i].flags &= (~USED);
	    strcpy(xid_composite[i].name,"");
	    }
	}
}
