#include "xid.h"

#define MINOUT	0
#define	MAXOUT	255

FUNCTION xid_slpoff(indtyp,minpix,maxpix,slope,offset)

long	*indtyp;	/* Input data type. (I)				*/

double	*minpix,	/* Minimum pixel value. (I)			*/
	*maxpix,	/* Maximum pixel value. (I)			*/
	*slope,		/* Slope to be computed. (O)			*/
	*offset;	/* Offset to be computed. (O)			*/

/******************************************************************************
 *									      *
 *	Purpose: This routine will take minimum and maximum values of the     *
 *		 input data type and compute the slope and offset needed to   *
 *		 scale all values to the specified range using the following  *
 *		 formula: 						      *
 *			output pixel = input pixel * slope + offset.	      *
 *									      *
 *	By D. Akkerman, USGS/EROS Data Center, 11/89.			      *
 *	 - Adapted from the DMS dtcvrt routine.				      *
 *									      *
 ******************************************************************************/
{
long	one = 1,  	/* Specifies a nonfatal error message.		*/
	status;	  	/* Return status variable.			*/

double	maxin,minin,	/* Maximum & minimum value for input data type. */
	maxout,minout;	/* Maximum & minimum value for output data type.*/


/* Initialize the slope and offset parameters.				*/

*slope = 1.0;
*offset = 0.0;

/* Obtain minimum & maximum for input data type.  Note that the maxin and
   minin parameters are in reverse order to allow the inmnmx routine to
   assign these correctly.						*/

status = c_inmnmx(indtyp,&one,&maxin,&minin);
if (status != E_SUCC)
 {
  c_errmsg("Invalid data type","pcvrt-bdata",&one);
  return(status);
 }

/* Assign the minimum & maximum.					*/

minout = MINOUT;
maxout = MAXOUT;

/* If the input data type is REAL then always scale the range. This is 
   because even a small REAL data range (say 0 to 3) could have a very
   large mantissa and thus require scaling, even though the integer
   portion is within the BYTE range.  					*/

if (*indtyp == EREAL)
   {
    *slope = (maxout - minout)/(*maxpix - *minpix);
    *offset = minout - (*slope * (*minpix));
   }
else if ((*minpix < minout) || (*maxpix > maxout))
        if ((*maxpix - *minpix) <= (maxout - minout))
	    *offset = minout - *minpix;
        else
	    {
	    *slope = (maxout - minout)/(*maxpix - *minpix);
	    *offset = minout - (*slope * (*minpix));
	    }

if ((*slope != 1.0) || (*offset != 0.0))
 {
  printf("Mapped %f : %f    to      0 : 255\n",*minpix,*maxpix);
  printf("Slope = %.10f       Offset = %.10f\n",*slope,*offset);
 }

return(XID_S);
}
