#include "gof.h"

#define MAXELEMENT	512		/* Initial maximum number of graphics 
					   elements.			      */
#define USER_GRCOLORS	8 		/* Maximum number of user specified
					   graphics colors.		      */
#define USER_GRFONTS	9 		/* Maximum number of user specified
					   fonts.		      	      */
#define HANDLE	5			/* Width and height of selection 
					   handles.     		      */

#define NO_TYPE		-1		/* No graphics type has been selected.*/
#define POINT		0		/* Point graphics data.		      */
#define LINE		1		/* Line graphics data.		      */
#define RECT		2		/* Rectangle (polygon) graphics data. */
#define POLY		3		/* Polygon graphics data.	      */
#define TEXT		4		/* Annotation/text graphics data.     */

#define GR_ACTIVE	0x1		/* Active flag.			      */
#define GR_CREATE	0x2		/* Newly created graphics panel.      */
#define GR_SELECT	0x4		/* Select mode for graphics.	      */
#define	GR_LOAD		0x8		/* Load mode for graphics.	      */
#define	GR_LABEL	0x10		/* Flag indicating whether or not a
					   label is currently being placed.   */
#define	GR_CLOSE	0x20		/* Flag indicating whether or not the
					   user has finished selecting the
					   current graphics element.	      */
#define	GR_CANCEL	0x40		/* Flag indicating whether or not 
					   last item was cancelled.	      */
#define GR_EDIT		0x80		/* Edit mode for graphics.	      */

#define GR_ADD		0x1		/* Within Edit mode -- Add mode.      */
#define GR_MOVE		0x2		/* Within Edit mode -- Move mode.     */
#define GR_DELETE	0x4		/* Within Edit mode -- Delete mode.   */
#define GR_EDITLABEL	0x8		/* Within Edit mode -- Change label.  */
#define GR_MODIFY	0x10		/* Within Edit mode -- Indicates the
					   current element has had modifica-
					   tions made.			      */
#define GR_REMOVE	0x20		/* Within Edit mode -- Indicates the
					   current element is to be deleted
					   from the graphics file.	      */

#define GR_ORIG		1		/* Flag indicating a copy of the
					   original state of the current 
					   element is to be made.	      */
#define GR_UNDO		2		/* Flag indicating a copy of the
					   current state of the current 
					   element is to be made.	      */

#define	XID_COL		0		/* Index into xid_atnames array for
					   color.			      */
#define	XID_FONT	1		/* Index into xid_atnames array for
					   font.			      */
#define	XID_LW		2		/* Index into xid_atnames array for
					   line width.			      */
#define	XID_LABX	3		/* Index into xid_atnames array for
					   X label location.		      */
#define	XID_LABY	4		/* Index into xid_atnames array for
					   Y label location.		      */

struct XID_GRAPHICS
 {
 Pixmap		pixmap[USER_GRCOLORS];	/* Graphics pixmap.		      */
 Pixmap         full_pix[USER_GRCOLORS];/* Full resolution window graphics
					   pixmap.                            */
 Pixmap         zoom_pix[USER_GRCOLORS];/* Zoomed window graphics pixmap.     */
 GC		gc;			/* Graphics GC.			      */
 GC		rb_gc;			/* Rubber-band line GC.		      */
 Widget		window;			/* Widget the rubber-band line is
					   drawn in.			      */
 XFontStruct	*font_info[USER_GRFONTS];
					/* Font information for graphics 
					   fonts.			      */
 int		fd;			/* Current file descriptor.	      */
 struct PT_LIST *first;
 struct PT_LIST *cur;
 int		selectx;		/* Last selected point's X coordinate.*/
 int		selecty;		/* Last selected point's Y coordinate.*/
 int		prevx;			/* Previous cursor position (X).      */
 int		prevy;			/* Previous cursor position (Y).      */
 unsigned long	rb_fore;		/* Foreground pixel value for rubber-
					   band line.			      */
 long		type;			/* Current type of graphics data.     */
 long		current;		/* Index of current graphics element. */
 long		max_element;		/* Maximum graphics element number in
					   currently allocated space. 	      */
 long		temp_num;		/* Number of points in temp. array.   */
 long		color;			/* Current color index.		      */
 long		font;			/* Current font index.		      */
 long		line_width;		/* Current line width.		      */
 long		flags;			/* Bit flags
					   - Active flag.
					   - Newly created graphics panel.
					   - Current mode (select or load).
					   - Label placement toggle.
					   - Cancel element flag.	      */
 char           lname_gof[CMLEN];
				  	/* LAS file name of GOF.              */

 char		colors[USER_GRCOLORS][81];
					/* User defined graphics colors.      */
 char		fonts[USER_GRFONTS][81];
					/* User defined fonts.      	      */
 char		attr_types[MAXATT][ATTYP];
					/* Attribute types of current element.*/
 char		attr_values[MAXATT][ATTVAL];
					/* Temporary array of attribute 
					   values.			      */
 };

struct XID_LABEL
 {
 int		x,y;			/* X/Y coordinate of upper left corner
					   of annotation text.		      */
 int		width,height;		/* Width and height of rectangle 
					   surrounding text.	              */
 };

struct GRAPH
 {
 long		type;			/* Type of graphics element.	      */
 long		offset;			/* Offset to the record in the GOF.   */
 long		writeflg;		/* Indicates whether or not the record
					   has been written to the GOF.	      */
 long		color;			/* Color index of graphics element.   */
 long		font;			/* Font index of graphics element.    */
 float		x;			/* X location of point, first vertice
					   of line or polygon, or upper left
					   corner of an annotation string.    */
 float		y;			/* Y location of point, first vertice
					   of line or polygon, or upper left
					   corner of an annotation string.    */
 float		labelx;			/* X location of graphics label.      */
 float		labely;			/* Y location of graphics label.      */
 float		upleft[2];		/* XY location of upper left corner of
					   the bounding rectangle.	      */
 float		loright[2];		/* XY location of lower right corner of
					   the bounding rectangle.	      */
 char		label[KEYL];		/* Graphics label.		      */
 char		attributes[MAXATT][ATTVAL];
					/* Attribute values.		      */
 long		*graph_ptr;		/* Pointer to the structure containing
					   the element specific information.  */
 };

struct LINE_ELEMENT
 {
 long		num_vert;		/* Number of vertices.		      */
 long		line_width;		/* Line width.			      */
 float		*x;			/* X locations of the vertices of
					   the line.			      */
 float		*y;			/* Y locations of the vertices of
					   the line.			      */
 };

struct POLY_ELEMENT
 {
 long		num_vert;		/* Number of vertices.		      */
 long		line_width;		/* Line width.			      */
 long		fill;			/* Flag indicating whether or not
					   the polygon is to be filled.       */
 float		*x;			/* X locations of the vertices of
					   the polygon.			      */
 float		*y;			/* Y locations of the vertices of
					   the polygon.			      */
 };

struct ANNOT_ELEMENT
 {
 long		size;			/* Character size.		      */
 float		angle;			/* Annotation angle.		      */
 char		*string;		/* Annotation string.		      */
 };

struct PT_LIST
 {
 struct PT_LIST *next;
 struct PT_LIST *prev;
 float		x;
 float		y;
 };

struct XID_EDITGRAPH
 {
 float		xfil;			/* File X coordinate of current 
					   vertice.			      */
 float		yfil;			/* File Y coordinate of current
					   vertice.			      */
 float		fxprev;			/* File X coordinate of previous 
					   vertice.			      */
 float		fyprev;			/* File Y coordinate of previous
					   vertice.			      */
 float		fxnext;			/* File X coordinate of next 
					   vertice.			      */
 float		fynext;			/* File Y coordinate of next
					   vertice.			      */
 float		ulx;			/* File X coordinate of upper left
					   corner of original rectangle.      */
 float		uly;			/* File Y coordinate of upper left
					   corner of original rectangle.      */
 long		current;		/* Index of current graphics element. */
 long		delete_idx;		/* Index of graphics element that has
					   been deleted.		      */
 long		vert;			/* Index of current vertice of current
					   graphics element.		      */
 long		prevx;			/* Window X coordinate of previous
					   vertice (move command).	      */
 long		prevy;			/* Window Y coordinate of previous
					   vertice (move command).	      */
 long		nextx;			/* Window X coordinate of next
					   vertice (move command).	      */
 long		nexty;			/* Window Y coordinate of next
					   vertice (move command).	      */
 long		newx;			/* Window X coordinate of new
					   vertice (move command).	      */
 long		newy;			/* Window Y coordinate of new
					   vertice (move command).	      */
 long		new_ulx;		/* New upper left window X coordinate.*/
 long		new_uly;		/* New upper left window Y coordinate.*/
 long		width;			/* Width of new rectangle.	      */
 long		height;			/* Height of new rectangle.	      */
 long		color;			/* Color index of edit handles.       */
 long		writeflg;		/* Indicates whether or not the record 
					   being modified has been written to
					   the GOF.			      */
 long		flags;			/* Bit flags
					   - Edit mode.
					   - Modified flag.		      */
 };

#ifndef _xid_h
extern struct XID_GRAPHICS	xid_graphics;
extern struct XID_LABEL		xid_label;
extern struct GRAPH		*xid_element;
extern struct XID_EDITGRAPH	xid_editgraph;
extern struct GRAPH		xid_editundo;
extern struct GRAPH		xid_editorig;
#else
struct XID_GRAPHICS		xid_graphics;
struct XID_LABEL		xid_label;
struct GRAPH			*xid_element;
struct XID_EDITGRAPH		xid_editgraph;
struct GRAPH			xid_editundo;
struct GRAPH			xid_editorig;
#endif

static	char	xid_atnames[][ATTNAM]={"XID_COL","XID_FONT","XID_LW","XID_LABX",
    "XID_LABY"};
static 	char	xid_attypes[][ATTYP] = {"INT","INT","INT","REA","REA"};
