/*
 *  ProcessM.c
 */

/*
 *  Implementation file for the processing message widget.
 */

/*
 *  Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of USGS not be used in advertising or 
 *  publicity pertaining to distribution of the software without specific, 
 *  written prior permission.  USGS makes no representations about the 
 *  suitability of this software for any purpose.  It is provided "as is" 
 *  without express or implied warranty.
 *
 *  USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS 
 *  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 *  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 *  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *  The Xpm pixmap routine use is:
 *
 *  Copyright 1990-93 GROUPE BULL
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of GROUPE BULL not be used in advertising
 *  or publicity pertaining to distribution of the software without specific,
 *  written prior permission.  GROUPE BULL makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 *
 *  GROUPE BULL disclaims all warranties with regard to this software,
 *  including all implied warranties of merchantability and fitness,
 *  in no event shall GROUPE BULL be liable for any special,
 *  indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits,
 *  whether in an action of contract, negligence or other tortious
 *  action, arising out of or in connection with the use
 *  or performance of this software.
 *
 */

/*
 *  Change log:
 *
 *	D. Akkerman, USGS/EROS Data Center, 7/92
 *	 - inception.
 *
 *	D. Akkerman, USGS/EROS Data Center, 7/93
 *	 - updated for IPC.
 *
 *	D. Akkerman, USGS/EROS Data Center, 12/93
 *	 - Fixed problems with using icon for processing.
 *	 - Added code for more efficient color assignment for "pie" processing.
 *	 - Added code to reduce CPU usage while paused.
 *
 *	L. Huewe, USGS/EROS Data Center, 3/94
 *	 - Fixed a problem in XeProcessMProcess that caused the "init_flag" to
 *	   be set to False at inappropriate times.
 *
 */

/*
 *  Include the required include files.
 */

#include <X11/StringDefs.h>
#include "ProcessMP.h"

/*
 *  Set up the resources for the processing message widget.
 */

#define Offset(field) XtOffset(XeProcessMWidget, processm.field)

static XtResource resources[] =
 {
  {XeNprocessMMessageString, XeCProcessMMessageString, XtRString,
   sizeof(char *), Offset(message_string), XtRString,
   (XtPointer)XePROCESSM_DEFAULT_LABEL},
  {XeNprocessMColors, XeCProcessMColors, XtRImmediate, sizeof(XeProcessMColor*),
   Offset(color_struct), XtRImmediate, (XtPointer)NULL},
  {XeNprocessMUpdatePercent, XeCProcessMUpdatePercent, XtRInt,
   sizeof(XeProcessMIterator), Offset(update_percent), XtRImmediate,
   (XtPointer)XePROCESSM_DEFAULT_UPDATE_PERCENT},
  {XeNprocessMColorList, XeCProcessMColorList, XtRImmediate,
   sizeof(XeProcessMIconColor*), Offset(input_color_list), XtRImmediate,
   (XtPointer)NULL},
  {XeNprocessMVisual, XeCProcessMVisual, XtRVisual, sizeof(Visual*), 
      Offset(visual), XtRImmediate, CopyFromParent},
 };

/*
 *  Class methods.
 */

#ifdef _NO_PROTO
void	Initialize(),
	Destroy();

static Boolean 	SetValues();
#else
void	Initialize(Widget request,
		   Widget new);

static void	Destroy(Widget pmw);

static Boolean	SetValues(Widget old,
			  Widget req,
			  Widget new,
			  ArgList args,
			  Cardinal *num_args);
#endif

/*
 *  Class storage definition.
 */

XeProcessMClassRec xeProcessMClassRec =
 {
	{ /* core_class fields */
	 (WidgetClass) &xmMessageBoxClassRec,        /* ancestor */
	 (String)"XeProcessM",			     /* class name */
	 (Cardinal)sizeof(XeProcessMRec),	     /* widget size */
	 (XtProc)NULL,				     /* class initialize */
	 (XtWidgetClassProc)NULL,		     /* class part init. */
	 (XtEnum)False,				     /* class inited */
	 (XtInitProc)Initialize,		     /* initialize */
	 (XtArgsProc)NULL,			     /* initialize hook */
	 (XtRealizeProc)XtInheritRealize,	     /* realize */
	 (XtActionList)NULL,			     /* actions */
	 (Cardinal)0,				     /* number of actions */
	 (XtResourceList)resources,		     /* resources */
	 (Cardinal)XtNumber(resources),		     /* number of resources */
	 (XrmClass)NULLQUARK,			     /* xrm class */
	 (Boolean)True,				     /* compress motions */
	 (XtEnum)XtExposeCompressMaximal,	     /* compress exposures */
	 (Boolean)True,				     /* compress enter/leave */
	 (Boolean)False,			     /* visibility interest */
	 (XtWidgetProc)Destroy,			     /* destroy */
	 (XtWidgetProc)XtInheritResize,		     /* resize */
	 (XtExposeProc)XtInheritExpose,		     /* expose */
	 (XtSetValuesFunc)SetValues,		     /* set values */
	 (XtArgsFunc)NULL,			     /* set values hook */
	 (XtAlmostProc)XtInheritSetValuesAlmost,     /* set values almost */
	 (XtArgsProc)NULL,			     /* get values hook */
	 (XtAcceptFocusProc)XtInheritAcceptFocus,    /* accept focus */
	 (XtVersionType)XtVersion,		     /* version */
	 (XtPointer)NULL,			     /* callback private */
	 (String)XtInheritTranslations,		     /* translation table */
	 (XtGeometryHandler)XtInheritQueryGeometry,  /* query geometry */
	 (XtStringProc)NULL,			     /* display accelerator */
	 (XtPointer)NULL,			     /* extension */
	},
	{ /* composite_class fields */
	 (XtGeometryHandler)XtInheritGeometryManager,/* geometry manager */
	 (XtWidgetProc)XtInheritChangeManaged,	     /* change managed */
	 (XtWidgetProc)XtInheritInsertChild,	     /* insert child */
	 (XtWidgetProc)XtInheritDeleteChild,	     /* delete child */
	 (XtPointer)NULL,			     /* extension */
	},
	{ /* constraint_class fields */
	 (XtResourceList)NULL,			     /* subresources */
	 (Cardinal)0,				     /* number of subresources*/
	 (Cardinal)0,				     /* record size */
	 (XtInitProc)NULL,			     /* initialize */
	 (XtWidgetProc)NULL,			     /* destroy */
	 (XtSetValuesFunc)NULL,			     /* set values */
	 (XtPointer)NULL,			     /* extension */
	},
	{ /* manager_class fields */
         (String)XtInheritTranslations,              /* default translations  */
         (XmSyntheticResource*)NULL,                 /* syn_resources         */
         (int)0,               			     /* num_syn_resources     */
         (XmSyntheticResource*)NULL,                 /* syn_cont_resources    */
         (int)0,                                     /* num_syn_cont_resources*/
         (XmParentProcessProc)XmInheritParentProcess,/* parent_process        */
         (XtPointer)NULL,                            /* extension             */
	},
	{ /* BulletinBoard class fields */
	 (Boolean)True,                              
						/* always_install_accelerators*/
	 (XmGeoCreateProc)_XmMessageBoxGeoMatrixCreate,/* geo_matrix_create */
	 (XmFocusMovedProc)NULL,                     /* focus moved proc */
	 (XtPointer)NULL,			     /* extension */
	},
	{ /* MessageBox class fields */
	 (int)0,				     /* extension */
	},
	{ /* ProcessM class fields */
	 (XtPointer)NULL,			     /* extension */
	},
 }; /* xeProcessMClassRec */

/*
 *  Define the external hook to the class record.
 */

WidgetClass xeProcessMWidgetClass = (WidgetClass) &xeProcessMClassRec;

/*
 *  Define the private functions.
 */

/*
 *  This procedure will ensure that, if a dialog window is being mapped,
 *  its contents become visible before returning.  It is intended to be
 *  used just before a bout of computing that doesn't service the display.
 *  You should still call XmUpdateDisplay() at intervals during this
 *  computing if possible.
 *
 *  The monitoring of window states is necessary because attempts to map
 *  the dialog are redirected to the window manager (if there is one) and
 *  this introduces a significant delay before the window is actually mapped
 *  and exposed.  This code works under mwm, twm, uwm, and no-wm.  It
 *  doesn't work (but doesn't hang) with olwm if the mainwindow is iconified.
 *
 *  The argument to force_dialog is any widget in the dialog (often it
 *  will be the BulletinBoard child of a DialogShell).
 *
 *  (David Brooks, Systems Engineering, Open Software Foundation)
 *
 *  - adapted for use with DLG-E -- D. Akkerman
 */

static void force_dialog(w)
Widget w;
{
Widget	diashell,
	topshell;

Window	diawindow,
	topwindow;

Display *dpy;

XWindowAttributes xwa;

XEvent	event;

XtAppContext cxt;


/*
 *  Locate the shell we are interested in.  In a particular instance, you
 *  may know these shells already.
 */

 for (diashell = w; !XtIsShell(diashell); diashell = XtParent(diashell))
    ;

/*
 *  Locate its primary window's shell (which may be the same).
 */

 for (topshell = diashell; !XtIsTopLevelShell(topshell);
	topshell = XtParent(topshell))
    ;

 if (XtIsRealized(diashell) && XtIsRealized(topshell))
  {
    dpy = XtDisplay(topshell);
    diawindow = XtWindow(diashell);
    topwindow = XtWindow(topshell);
    cxt = XtWidgetToApplicationContext(diashell);

   /*
    *  Wait for the dialog to be mapped.  It's guaranteed to become so 
    *  unless...
    */

    while (XGetWindowAttributes(dpy, diawindow, &xwa),
           xwa.map_state != IsViewable)
     {

     /*
      *  ...if the primary is (or becomes) unviewable or unmapped, it's
      *  probably iconified, and nothing will happen.
      */

      if (XGetWindowAttributes(dpy, topwindow, &xwa),
          xwa.map_state != IsViewable)
        break;

     /*
      *  At this stage, we are guaranteed there will be an event of some kind.
      *  Beware; we are presumably in a callback, so this can recurse.
      */

      XtAppNextEvent(cxt, &event);
      XtDispatchEvent(&event);
    }
  }

/*
 *  The next XSync() will get an expose event if the dialog was unmapped.
 */

 XmUpdateDisplay(topshell);

 return;
}


/*
 *  This procedure is used to convert a color cell definition of a color
 *  to its hex string equivalent.
 */

static void ConvertPixelToString (w, colormap, pixel, string)
Widget w;
Colormap colormap;
Pixel pixel;
String string;
{
XColor colorcell_def;


/*
 *  Convert the specified pixel to its hex string equivalent.
 */

 colorcell_def.pixel = pixel;
 XQueryColor(XtDisplay(w), colormap, &colorcell_def);
 sprintf(string, "#%x%x%x\0", colorcell_def.red, colorcell_def.green,
                        colorcell_def.blue);

 return;
}


/*
 *  Define the private callbacks for the "ok" and "cancel" button of the
 *  XmMessageBox widget.
 */

static void processm_cancel_callback(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
{
XeProcessMWidget pmw = (XeProcessMWidget)client;


 pmw->processm.user_response = XePROCESSM_CANCEL;

 return;
}


static void processm_pause_callback(w, client, call)
Widget w;
XtPointer client;
XtPointer call;
{
#define NUM_ARGS	25
#define InstanceF(field)	(pmw->processm.field)


XeProcessMWidget pmw = (XeProcessMWidget)client;

Arg args[NUM_ARGS];

Cardinal argcnt;


/*
 *  If pause was selected then change the name of the pause button
 *  and loop until resume is pressed.
 */

 if (InstanceF(pause_set) == False)
  {
   argcnt = 0;
   XtSetArg(args[argcnt], XmNokLabelString, InstanceF(resume_mlabel)); argcnt++;
   XtSetValues((Widget)pmw, args, argcnt);
  
   XtSetSensitive(InstanceF(cancel_button), False);
   InstanceF(pause_set) = True;
   InstanceF(user_response) = XePROCESSM_PAUSE;
  }
 else
  {
   argcnt = 0;
   XtSetArg(args[argcnt], XmNokLabelString, InstanceF(pause_mlabel)); argcnt++;
   XtSetValues((Widget)pmw, args, argcnt);

   XtSetSensitive(InstanceF(cancel_button), True);

   InstanceF(pause_set) = False;
   InstanceF(user_response) = XePROCESSM_NO_RESPONSE;

  /*
   *  Update the "ok" button to ensure that the label change occurs.
   */

   XmUpdateDisplay(InstanceF(ok_button));
   XFlush(XtDisplay((Widget)pmw));

  }


 return;

#undef NUM_ARGS 
#undef InstanceF
}


/*
 *  Private support functions.
 */

/*
 *
 *  This routine is designed to allow the user to interrupt long
 *  processing operations under X.  It works by polling for a specific
 *  event periodically and forcing the server to act on it (namely the 
 *  selection of a cancel button).  This section of code is from original 
 *  work done by Dan Heller and has been adapted for use in DLG-E.  The 
 *  work is freely distributable -- DA.
 *
 */

static XeProcessMResponse CheckForInterrupt(processm_widget)
Widget processm_widget;
{
#define InstanceF(field)	(work->processm.field)


XeProcessMWidget work = (XeProcessMWidget)processm_widget;

XEvent event;

Window dialog_win = XtWindow(processm_widget);


/*
 *  Make sure all our requests get to the server.
 */

 XFlush(XtDisplay(processm_widget));

/*
 *  Let Motif process all pending exposure events for us.
 */

 XmUpdateDisplay(processm_widget);

/*
 *  Check the event queue for pending events.
 */

 while (XCheckMaskEvent(XtDisplay(processm_widget), EVENT_MASK, &event))
  {
   if (event.xany.window == dialog_win)
     XtDispatchEvent(&event);
   else
    ; /* Discard event. */
  }

 return(InstanceF(user_response));

#undef InstanceF
}


/*
 *  Define the processing message widgets methods.
 */

void
#ifdef _NO_PROTO
Initialize(request, new)
Widget		request,
	 	new;
#else
Initialize(Widget request,
	   Widget new)
#endif
{
#define NUM_ARGS		50
#define STRING_SIZE		40
#define InstanceF(f1)		(work->processm.f1)


Widget w;

XeProcessMWidget work = (XeProcessMWidget)new;

Arg args[NUM_ARGS];		/* Xt resource argument list.		*/
 
XmString mstring;		/* Working Motif compound string.	*/

				/* Standard X variable/definitions.	*/
Display *display = XtDisplay(new);

int screen = DefaultScreen(display);

Screen *scr_ptr = XtScreen(new);

String hex_string,		/* String for pixel to color conversion. */
       out_string;

XGCValues values;		/* GC attribute variables.		*/

unsigned long valuemask = 0;

Cardinal argcnt;		/* Xt resource argument count.		*/

long i;				/* Working loop index.			*/

char symbol_name1[STRING_SIZE],	/* Xpm symbols.				*/
     symbol_name2[STRING_SIZE],
     symbol_name3[STRING_SIZE];

XpmAttributes xpm_attributes;

XpmColorSymbol color_symbol[XePROCESSM_XPM_MAX_COLORS];


/*
 *  Initialize working pointers.
 */

 mstring = (XmString)NULL;

 InstanceF(symbol_pixmap) = (Pixmap)NULL;
 InstanceF(shape_mask)    = (Pixmap)NULL;
 InstanceF(dialog_shell)  = (Widget)NULL;

 InstanceF(color_store.num_colors) = 0;

w = new;
while(!XtIsTopLevelShell(w))
    w = XtParent(w);

/*
 *  Check the validity of resources.
 */

 if (InstanceF(color_struct) != NULL)
  {
   if (InstanceF(color_struct->num_colors) > XePROCESSM_MAX_COLORS)
    {
     XtWarning(XePROCESSM_BAD_COLOR_SPEC);
     InstanceF(color_struct->num_colors) = XePROCESSM_MAX_COLORS;
    }
  }

 if (InstanceF(update_percent) > 100)
  {
   XtWarning(XePROCESSM_BAD_UPDATE_PERCENTAGE);
   InstanceF(update_percent) = XePROCESSM_DEFAULT_UPDATE_PERCENT;
  }

/*
 *  Set initial flags for possible processing.
 */

 InstanceF(init_flag)  = False;
 InstanceF(pause_set)  = False;

/*
 *  Force the autoUnmanage resource to False.
 */

 argcnt = 0;
 XtSetArg(args[argcnt], XmNautoUnmanage, False); argcnt++;
 XtSetValues(new, args, argcnt);

/*
 *  If colors have been specified then copy them to the internal array.
 */

 if (InstanceF(color_struct) != NULL)
  {
   for (i = 0; i < InstanceF(color_struct->num_colors); i++)
    InstanceF(color_store.color_array[i]) = 
	InstanceF(color_struct->color_array[i]);

    InstanceF(color_store.num_colors) = InstanceF(color_struct->num_colors);
  }

/*
 *  Set the background pixel string to the background of the panel.
 */

 if (InstanceF(input_color_list) == NULL)
  {
   hex_string = XtMalloc(STRING_SIZE * sizeof(char));
   out_string = XtMalloc(STRING_SIZE * sizeof(char));

   ConvertPixelToString (new, new->core.colormap,
                        new->core.background_pixel,
                        hex_string);

   sprintf(out_string, "- c %s", hex_string);
  }
 else
  {
   for (i = 0; i < XePROCESSM_MAX_COLORS; i++)
        InstanceF(color_list[i]) = InstanceF(input_color_list->pixel_array[i]);
  }

   xpm_attributes.depth = new->core.depth; 
   xpm_attributes.visual = InstanceF(visual);
   xpm_attributes.valuemask = XpmVisual | XpmDepth;

/*
 *  If colors for the symbol have been specified then assign them to the
 *  xpm attribute structure.
 */

 if (InstanceF(input_color_list) != NULL)
  {
   strcpy(symbol_name1, XePROCESSM_COLOR1);
   color_symbol[0].name  = symbol_name1;
   color_symbol[0].value = NULL;
   color_symbol[0].pixel = InstanceF(color_list[0]);

   strcpy(symbol_name2, XePROCESSM_COLOR2);
   color_symbol[1].name  = symbol_name2;
   color_symbol[1].value = NULL;
   color_symbol[1].pixel = InstanceF(color_list[1]);

   strcpy(symbol_name3, XePROCESSM_COLOR3);
   color_symbol[2].name  = symbol_name3;
   color_symbol[2].value = NULL;
   color_symbol[2].pixel = InstanceF(color_list[2]);

   xpm_attributes.colorsymbols = color_symbol;
   xpm_attributes.numsymbols   = (unsigned int)XePROCESSM_MAX_COLORS;
   xpm_attributes.valuemask    |= XpmColorSymbols;
  }

/*
 *  Create a GC for process message drawing.
 */

 values.foreground = BlackPixel(display, screen);
 values.background = new->core.background_pixel;
 values.plane_mask = AllPlanes;
 valuemask	   = (GCForeground | GCBackground | GCPlaneMask);
 
 InstanceF(gc)	   = XCreateGC(display, XtWindow(w), valuemask, &values);

/*
 *  If a processing message string was specified then convert it to a Motif
 *  string, otherwise use a default string.
 */

 if (InstanceF(message_string) != NULL)
  mstring = XmStringCreateLtoR(InstanceF(message_string),
	XmSTRING_DEFAULT_CHARSET);
 else
  mstring = XmStringCreateLtoR(XePROCESSM_DEFAULT_LABEL,
	XmSTRING_DEFAULT_CHARSET);

 argcnt = 0;
 XtSetArg(args[argcnt], XmNmessageString, mstring); argcnt++;
 XtSetValues(new, args, argcnt);
 XmStringFree(mstring);

/*
 *  Create the pixmap to be used when processing time can't be accurately
 *  determined.
 */

 if (InstanceF(input_color_list) == NULL)
  {
   hour_glass[1] = out_string;
   XpmCreatePixmapFromData(display, XtWindow(w), hour_glass,
 		&InstanceF(symbol_pixmap), &InstanceF(shape_mask), 
		&xpm_attributes);
  }
 else
   XpmCreatePixmapFromData(display, XtWindow(w), hour_glass,
 		&InstanceF(symbol_pixmap), &InstanceF(shape_mask), 
		&xpm_attributes);

/*
 *  Create the drawing pixmap for processing draws.
 */

 InstanceF(drawing_pixmap) = XCreatePixmap(display, XtWindow(w), 
	XePROCESSM_PIXMAP_SIZE, XePROCESSM_PIXMAP_SIZE, new->core.depth);

/*
 *  Assign the internal widget IDs.
 */

 InstanceF(ok_button)     = XmMessageBoxGetChild(new,
					XmDIALOG_OK_BUTTON);
 InstanceF(cancel_button) = XmMessageBoxGetChild(new,
					XmDIALOG_CANCEL_BUTTON);
 InstanceF(help_button)   = XmMessageBoxGetChild(new,
					XmDIALOG_HELP_BUTTON);
 InstanceF(symbol_label)  = XmMessageBoxGetChild(new,
					XmDIALOG_SYMBOL_LABEL);
 InstanceF(separator)	  = XmMessageBoxGetChild(new,
					XmDIALOG_SEPARATOR);

/*
 *  Remove the default callbacks for the "ok" and "cancel" buttons and assign
 *  our callbacks.  This allows for the removal of the automatic unmapping
 *  feature and allows for the implementation of a pause capability.
 */

 XtRemoveAllCallbacks(InstanceF(ok_button), XmNactivateCallback);
 XtRemoveAllCallbacks(InstanceF(cancel_button), XmNactivateCallback);

 XtAddCallback(InstanceF(ok_button), XmNactivateCallback,
	processm_pause_callback, (XtPointer)new);
 XtAddCallback(InstanceF(cancel_button), XmNactivateCallback,
	processm_cancel_callback, (XtPointer)new);

/*
 *  Create the labels for the "ok" and "cancel" buttons.
 */

 InstanceF(pause_mlabel)  = XmStringCreateLtoR(XePROCESSM_DEFAULT_OK_PAUSE,
				XmSTRING_DEFAULT_CHARSET);
 InstanceF(resume_mlabel) = XmStringCreateLtoR(XePROCESSM_DEFAULT_OK_RESUME,
				XmSTRING_DEFAULT_CHARSET);

/*
 *  Assign the "ok" button label.
 */

 argcnt = 0;
 XtSetArg(args[argcnt], XmNokLabelString, InstanceF(pause_mlabel)); argcnt++;
 XtSetValues(new, args, argcnt);

/*
 *  Unmanage the "help" button -- you shouldn't really need help on a
 *  processing message (should you?).
 */

 XtUnmanageChild(InstanceF(help_button));

 return;

#undef NUM_ARGS
#undef STRING_SIZE
#undef InstanceF
} /* Initialize */

 
void
#ifdef _NO_PROTO
Destroy(processm_widget)
Widget processm_widget;
#else
Destroy(Widget processm_widget)
#endif
{
XeProcessMWidget pmw = (XeProcessMWidget)processm_widget;


 XmStringFree(pmw->processm.pause_mlabel);
 XmStringFree(pmw->processm.resume_mlabel);

 XFreePixmap(XtDisplay((Widget)pmw),pmw->processm.symbol_pixmap);
 XFreePixmap(XtDisplay((Widget)pmw),pmw->processm.shape_mask);
 XFreePixmap(XtDisplay((Widget)pmw),pmw->processm.drawing_pixmap);

 if (pmw->processm.dialog_shell != NULL)
  XtDestroyWidget(pmw->processm.dialog_shell);

 return;
} /* Destroy */
 

static Boolean
#ifdef _NO_PROTO
SetValues(old, req, new, args, num_args)
Widget  old,
        req,
        new;
ArgList args;
Cardinal *num_args;
#else
SetValues(Widget old,
	  Widget req,
	  Widget new,
	  ArgList args,
	  Cardinal *num_args)
#endif
{
#define NUM_ARGS	25
#define InstanceF(field)	(new_pmw->processm.field)
#define RequestF(field)		(req_pmw->processm.field)


XeProcessMWidget req_pmw = (XeProcessMWidget)req,
		 new_pmw = (XeProcessMWidget)new;

                                /* Standard X variable/definitions.     */
Display *display = XtDisplay(new);

Screen *scr_ptr = XtScreen(new);
 
XmString mstring;		/* Working Motif string.		*/

Arg new_args[NUM_ARGS];		/* Xt resource arguments.		*/

Cardinal argcnt;		/* Resource count.			*/

long i;				/* Working loop index.			*/


/*
 *  Process each of the arguments assigning changes as required.
 */

 for (i = 0; i < *num_args; i++)
  {
   if (strcmp(args[i].name, XeNprocessMMessageString) == 0)
    {
     mstring = XmStringCreateLtoR(new_pmw->processm.message_string,
					XmSTRING_DEFAULT_CHARSET);
     argcnt = 0;
     XtSetArg(new_args[argcnt], XmNmessageString, mstring); argcnt++;
     XtSetValues((Widget)new_pmw, new_args, argcnt); 
     XmStringFree(mstring);
    }

   if (strcmp(args[i].name, XeNprocessMColors) == 0)
    {
     if (RequestF(color_struct->num_colors) > XePROCESSM_MAX_COLORS)
      {
       XtWarning(XePROCESSM_BAD_COLOR_SPEC);
       RequestF(color_struct->num_colors) = XePROCESSM_MAX_COLORS;
      }
     for (i = 0; i < RequestF(color_struct->num_colors); i++)
      InstanceF(color_store.color_array[i])=
	RequestF(color_struct->color_array[i]);

     InstanceF(color_store.num_colors) = RequestF(color_struct->num_colors);
    }

   if (strcmp(args[i].name, XeNprocessMUpdatePercent) == 0)
    {
     if ((XeProcessMIterator)args[i].value > 100)
      {
       XtWarning(XePROCESSM_BAD_UPDATE_PERCENTAGE);
       InstanceF(update_percent) = XePROCESSM_DEFAULT_UPDATE_PERCENT;
      }
     else
      InstanceF(update_percent) = (XeProcessMIterator)args[i].value;
    }

   if (strcmp(args[i].name, XeNprocessMVisual) == 0)
       InstanceF(visual) = (Visual *)args[i].value;
  }
 
 return(True);

#undef NUM_ARGS
#undef InstanceF
#undef RequestF
} /* SetValues */


/*
 *  Define the public functions for the processing message widget.
 */

void
#ifdef _NO_PROTO
XeProcessMBegin(processm_widget, iterator_start, iterator_end,
		process_message, use_symbol, use_pause, use_cancel)
Widget processm_widget;
XeProcessMIterator iterator_start,
		   iterator_end;
String process_message;
Boolean use_symbol,
	use_pause,
	use_cancel;
#else
XeProcessMBegin(Widget processm_widget,
		XeProcessMIterator iterator_start,
		XeProcessMIterator iterator_end,
		String process_message,
		Boolean use_symbol,
		Boolean use_pause,
		Boolean use_cancel)
#endif
{
#define NUM_ARGS		10
#define InstanceF(field)	(work->processm.field)


XeProcessMWidget work = (XeProcessMWidget)processm_widget;

XmString mstring;

Arg args[NUM_ARGS];

Cardinal argcnt;


/*
 *  Perform basic validation of the input parameters.
 */

 if (((iterator_start < 0) || (iterator_end < 0)) && (!use_symbol))
  {
   XtWarning(XePROCESSM_BAD_ITERATOR_VALUE);
   return;
  }
 else if ((iterator_end < iterator_start) && (!use_symbol))
  {
   XtWarning(XePROCESSM_BAD_ITERATOR_VALUE);
   return;
  }
 else if ((iterator_start == iterator_end) && (!use_symbol))
  {
   XtWarning(XePROCESSM_BAD_ITERATOR_VALUE);
   return;
  }

/*
 *  Assign the set color flag for True.  This flag is designed to reduce
 *  the number of color assignments to the absolute minimum for maximum
 *  efficiency in processing the pie graph.
 */

 InstanceF(set_color) = True;

/*
 *  If a processing message string was specified then create it's
 *  Motif equivalent and assign it to the processing message widget.
 */

 if (process_message != NULL)
  {
   mstring = XmStringCreateLtoR(process_message, XmSTRING_DEFAULT_CHARSET);
   argcnt = 0;
   XtSetArg(args[argcnt], XmNmessageString, mstring); argcnt++;
   XtSetValues(processm_widget, args, argcnt);
   XmStringFree(mstring);
  }

/*
 *  If the use_symbol flag has been set then install the default pixmap,
 *  otherwise install the drawing pixmap.
 */

 if (use_symbol == True)
  {
   argcnt = 0;
   XtSetArg(args[argcnt], XmNsymbolPixmap, InstanceF(symbol_pixmap)); argcnt++;
   XtSetValues(processm_widget, args, argcnt);
   InstanceF(use_symbol) = True;
  }
 else
  {
   argcnt = 0;
   XtSetArg(args[argcnt], XmNsymbolPixmap, InstanceF(drawing_pixmap)); argcnt++;
   XtSetValues(processm_widget, args, argcnt);
   InstanceF(use_symbol) = False;
  }

/*
 *  Change the management of the "ok" and "cancel" buttons based on the
 *  parameter settings.
 */

 if (use_pause == True)
  {
   if (!XtIsManaged(InstanceF(ok_button)))
	XtManageChild(InstanceF(ok_button));
  }
 else
  {
   if (XtIsManaged(InstanceF(ok_button)))
	XtUnmanageChild(InstanceF(ok_button));
  }

 if (use_cancel == True)
  {
   if (!XtIsManaged(InstanceF(cancel_button)))
	XtManageChild(InstanceF(cancel_button));
  }
 else
  {
   if (XtIsManaged(InstanceF(cancel_button)))
	XtUnmanageChild(InstanceF(cancel_button));
  }

/*
 *  If both the "ok" and "cancel" buttons are unmanaged then remove the
 *  separator from the widget as well.
 */

 if ((use_pause == False) && (use_cancel == False)) 
  {
   if (XtIsManaged(InstanceF(separator)))
	XtUnmanageChild(InstanceF(separator));
  }
 else
  {
   if (!XtIsManaged(InstanceF(separator)))
	XtManageChild(InstanceF(separator));
  }

/*
 *  Assign/initialize variables for the processing message.
 */

 if (!use_symbol)
  {
   InstanceF(start)		= iterator_start;
   InstanceF(end)		= iterator_end;
   InstanceF(old_angle)		= (90 * 64);
   InstanceF(current_color)	= 0;
   InstanceF(old)		= -1;
  }

 InstanceF(user_response)	= XePROCESSM_NO_RESPONSE;

/*
 *  Clear the working pixmap if required.
 */

 if (!use_symbol)
  {
   XSetForeground(XtDisplay(processm_widget),InstanceF(gc),
		processm_widget->core.background_pixel);

   XFillRectangle(XtDisplay(processm_widget),
	InstanceF(drawing_pixmap), InstanceF(gc),
	0, 0, XePROCESSM_PIXMAP_SIZE, XePROCESSM_PIXMAP_SIZE);

   argcnt = 0;
   XtSetArg(args[argcnt], XmNsymbolPixmap,
	InstanceF(drawing_pixmap)); argcnt++;
   XtSetValues(processm_widget, args, argcnt);

  /*
   *  Retrieve the pixmap ID for the label pixmap -- this is for copying
   *  to during message processing.
   */

   argcnt = 0;
   XtSetArg(args[argcnt], XmNlabelPixmap, &InstanceF(label_pixmap)); argcnt++;
   XtGetValues(InstanceF(symbol_label), args, argcnt);
  }

/*
 *  Set the initialized flag.
 */

 InstanceF(init_flag) = True;

/*
 *  Manage the message processing widget to begin processing.
 */

 XtManageChild(processm_widget);

/*
 *  Force the message processing shell to become active on the screen.
 */

 force_dialog(processm_widget);

 return;

#undef NUM_ARGS
#undef InstanceF
}


XeProcessMResponse
#ifdef _NO_PROTO
XeProcessMProcess(processm_widget, iterator_current)
Widget processm_widget;
XeProcessMIterator iterator_current;
#else
XeProcessMProcess(Widget processm_widget,
		  XeProcessMIterator iterator_current)
#endif
{
#define NUM_ARGS		10
#define InstanceF(field)	(work->processm.field)


XeProcessMWidget work = (XeProcessMWidget)processm_widget;

XExposeEvent xexpose;

XEvent event;

Arg args[NUM_ARGS];

Cardinal argcnt;

XeProcessMResponse user_response;

float percent;


/*
 *  Check if processing widget was previously initialized.
 */

 if (InstanceF(init_flag) != True)
  {
   XtWarning(XePROCESSM_NO_INIT);
   return(XePROCESSM_NO_INITIALIZATION);
  }

 if (((iterator_current == (InstanceF(end) - 1)) ||
     (iterator_current == (InstanceF(end)))) && (!InstanceF(use_symbol)))
   InstanceF(init_flag) = False;

/*
 *  Calculate the percent of the processing completed.
 */

 if (!InstanceF(use_symbol))
  {
   percent = ((float)iterator_current / (float)InstanceF(end));

   if (iterator_current == (InstanceF(end) - 1))
    percent = 1.0;

   InstanceF(percent) = (XeProcessMIterator)(percent * 100.0);
  }

/*
 *  Check the requested color assignment for simple color assignment (0 or 1).
 *  This is the most probable case and should be optimized.
 */

 if (InstanceF(set_color) == True)
  {
   if (InstanceF(color_store.num_colors) == 0)
    {
     XSetForeground(XtDisplay(processm_widget), InstanceF(gc),
         BlackPixel(XtDisplay(processm_widget),
                DefaultScreen(XtDisplay(processm_widget))));
     InstanceF(set_color) = False;
    }
   else if (InstanceF(color_store.num_colors) == 1)
    {
     XSetForeground(XtDisplay(processm_widget), InstanceF(gc),
          InstanceF(color_store.color_array[InstanceF(current_color)]));
     InstanceF(set_color) = False;
    }
  }
 
/*
 *  Update the processing message based on the amount of processing done.
 */

 if (!InstanceF(use_symbol))
  {
   if ((InstanceF(percent) % InstanceF(update_percent)) == 0)
    {
     if (InstanceF(percent) != InstanceF(old))
      {
      /*
       *  If multiple colors are required then there is no choice but to
       *  check and assign a new color each time.  Not as efficient as above,
       *  but there's no choice.
       */

       if ((InstanceF(color_store.num_colors) > 0) &&
	   (InstanceF(set_color) == True))
        {
         XSetForeground(XtDisplay(processm_widget), InstanceF(gc),
	  InstanceF(color_store.color_array[InstanceF(current_color)]));

	 if (InstanceF(percent) != 0)
          InstanceF(current_color)++;

         if (InstanceF(current_color) == InstanceF(color_store.num_colors))
	  InstanceF(current_color) = 0;
        }

      /*
       *  Draw the new arc for the processing message.
       */

       InstanceF(new_angle) = (short)(90 * 64 - (percent * 360.0) * 64);

       XFillArc(XtDisplay(processm_widget),
	InstanceF(drawing_pixmap),
	InstanceF(gc), 0, 0,
	XePROCESSM_PIXMAP_SIZE, XePROCESSM_PIXMAP_SIZE,
	InstanceF(old_angle),
	(InstanceF(new_angle) - InstanceF(old_angle)));

      /*
       *  Copy the working pixmap to the actual label pixmap.
       */

       XCopyArea(XtDisplay(processm_widget), InstanceF(drawing_pixmap),
	InstanceF(label_pixmap), InstanceF(gc), 0, 0,
	XePROCESSM_PIXMAP_SIZE, XePROCESSM_PIXMAP_SIZE,
	0, 0);

      /*
       *  Send the label pixmap an artificial expose event to update it.
       */

       xexpose.type		= Expose;
       xexpose.send_event	= True;
       xexpose.display		= XtDisplay(processm_widget);
       xexpose.window		= XtWindow(InstanceF(symbol_label));
       xexpose.x		= 0;
       xexpose.y		= 0;
       xexpose.width		= XePROCESSM_PIXMAP_SIZE;
       xexpose.height		= XePROCESSM_PIXMAP_SIZE;
       xexpose.count		= 0;
       XSendEvent(XtDisplay(processm_widget), XtWindow(processm_widget),
		True, ExposureMask, (XEvent*)&xexpose);

      /*
       *  Ensure that our artifical event is acted upon immediately.
       */

       while (XCheckMaskEvent(XtDisplay(processm_widget), ExposureMask, &event))
	XtDispatchEvent(&event);

       InstanceF(old_angle) = InstanceF(new_angle);
      }
    }
   InstanceF(old) = InstanceF(percent);
  }

/*
 *  Check if an interrupt was requested.
 */

 user_response = CheckForInterrupt(processm_widget);
 if (user_response == XePROCESSM_CANCEL)
  {
   InstanceF(init_flag) = False;
   return(user_response);
  }

/*
 *  Pause if requested.
 */

 while (user_response == XePROCESSM_PAUSE)
  {
   user_response = CheckForInterrupt(processm_widget);
   sleep(1); /* Reduce CPU usage. */
  }

 return(user_response);

#undef NUM_ARGS
#undef InstanceF
}

 
void
#ifdef _NO_PROTO
XeProcessMEnd(processm_widget)
Widget processm_widget;
#else
XeProcessMEnd(Widget processm_widget)
#endif
{
/*
 *  Unmanage the processing message shell.
 */

 XtUnmanageChild(processm_widget);
 XmUpdateDisplay(processm_widget);

 return;
} 


