/******************************************************************************

NAME:	ABS

FUNCTION:
	ABS is a module which copies a LAS image into another 
	image containing the absolute value of each pixel in the original
	LAS image.  It allows the input image(s) to be sub-windowed
	and sub-banded when copied to the output image.  The input image 
	may be BYTE, INTEGER*2, INTEGER*4, or REAL*4.

PROGRAM HISTORY:
  Version	Date       Author      	Request
  -------	----	   ------       -------
    1.0         4/87       M. Mazzella  adapted from copy by K. Gacke
    1.1		5/92	   S. Nelson	Updated user guide and error messages.
    1.2		3/93       D. Etrheim   If the input image is BYTE, copy the
					image.  This will allow procedure PDF's
					to work when the input images's data 
					type is not always known.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS version 5.0

ALGORITHM (FLOW):
 	The TAE input parameters are retrieved.  Error checking is done on the 
        input image(s) to insure that they all have equal window sizes.  Once 
	the TAE parameters are input, the absolute value of input image is 
	copied to the output image.  After the image has been copied, 
	the min max file and the history file are created/updated to 
	represent the output image.  

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"	
#include "protos.h"


void main
(void)
{

	struct PARBLK vblock;		/* TAE parameter block               */
	static
	long	bands[MAXIMG][MAXBND+1]; /* bands to be processed - all images*/
	long	msgtim;			/* elapsed time between display msg  */
	long	nbands[MAXIMG+1];	/* number of bands per input image   */
	long	nmrimg;			/* number of input images            */
	long	odtype;			/* data type of output image         */
	long	one = 1;		/* value of one			     */
	long	std_out;		/* fortran LUN to standard output    */
	long	totbnd;			/* total number of bands             */
	static
	long	window[MAXIMG][4];	/* window for each input image       */
	static
	char	hosin[MAXIMG][CMLEN];	/* input host image name	     */
	char	hosout[1][CMLEN];	/* output image host name	     */
	char	minmax[4];		/* min max flag                      */
	static
	char	taein[MAXIMG][CMLEN];	/* input image tae name		     */
	char	taeout[1][CMLEN];	/* output image tae name	     */

	c_inlas(&vblock,&std_out);	/* initialize TAE                    */
	getpar(&vblock,taein,hosin,taeout,hosout,&nmrimg,window,bands,nbands,
	       &totbnd,&odtype,&msgtim,minmax);
	process(&vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,
                        &totbnd,odtype,msgtim,minmax);
	c_hstout(&vblock,hosout[0],hosin,&nmrimg," ");

	c_complt("ABS",taeout,&one);
}
