/******************************************************************************

NAME: getpar

FUNCTION: get parameters from tae for abs

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS version 5.0

ALGORITHM (FLOW):

	get input image information and check for errors
	get output image information and check for errors
	get tae globals and check for errors
******************************************************************************/

#include "las.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block            */
    char taein[][CMLEN],	/* pointer to input TAE file descriptor      */
    char hosin[][CMLEN],	/* pointer to input host file descriptor     */
    char taeout[][CMLEN],/* pointer to output TAE file descriptor     */
    char hosout[][CMLEN],/* pointer to output host file descriptor    */
    long *nmrimg,		/* number of input images                    */
    long (*window)[4],		/* window specifications for each input image*/
    long (*bands)[MAXBND+1],	/* band specifications for each input image  */
    long *nbands,		/* number of bands per input image           */
    long *totbnd,		/* total number of bands to be processed     */
    long *odtype,		/* output data type                          */
    long *msgtim,		/* elapsed time between display messages     */
    char *minmax		/* min max flag buffer                       */
)
{
    struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */
    long	imglim;           	/* maximum number of images to be processed  */
    long	qflag[MAXIMG][2];	/* flag indicating if subwindows specified   */
    long	same;               /* flag indicating if images have same window*/
    long	status;             /* function return status code               */
    long	fatal = -1;
    long    nonfatal = 1;
    long	zero = 0;
    struct 	DDR   ddr;


    imglim = MAXIMG;
    status = c_getin(vblock,"IN",&zero,taein,hosin,nmrimg,window,bands,nbands,
                            totbnd,&imglim,&same,qflag);
    if (status != E_SUCC )
       c_errmsg("Fatal error encountered","abs-fatal",&fatal);
    status = c_getddr(hosin[0], &ddr);
    if (status != E_SUCC )
       c_errmsg("Fatal error encountered","abs-fatal",&fatal);
    if (ddr.dtype == EBYTE)
       c_errmsg("Input image is a BYTE image", "abs-type", &nonfatal);
    *odtype = ddr.dtype;
    if (same == 0)
       {
       c_errmsg("Input images do not have same window sizes","abs-window",
                                    &nonfatal);
       c_errmsg("Fatal error encountered","abs-fatal",&fatal);
       }
    status = c_getout(vblock,"OUT",taeout,hosout);
    if (status != E_SUCC )
       c_errmsg("Fatal error encountered","abs-fatal",&fatal);
    val = p_find(vblock,"$msgtim");		/* Retrieve TAE globals              */
    *msgtim = IVAL(*val,0);
    val = p_find(vblock,"$minmax");
    strcpy(minmax,SVAL(*val,0));
    return;
}
