#include <math.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/******************************************************************************

NAME: process

FUNCTION: Process image data for ABS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS version 5.0

ALGORITHM (FLOW):
   open the input images for read access
   open the output image for write access.  
   copies the absolute value of each pixel of the input image to the
   output image by calling a routine based on the data type   

******************************************************************************/

static void absbyte
(
    unsigned char thebuf[],
    long band,
    long ns
);

static void absword
(
    short thebuf[],
    long band,
    long ns
);

static void abslong
(
    long thebuf[],
    long band,
    long ns
);

static void absfloat
(
    float thebuf[],
    long band,
    long ns
);

/*  Variables declared global fo use with the abs functions */

static long offset1[MAXIMG*MAXBND+1];
static long offset2[MAXBND+1];

void process
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block            */
    char hosin[][CMLEN],	/* pointer to input host file descriptor     */
    char hosout[CMLEN],		/* pointer to output host file descriptor    */
    long *nmrimg,           /* number of input images                    */
    long (*window)[4],		/* window specification of input images      */
    long (*bands)[MAXBND+1],/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long *totbnd,		/* total number of bands to be processed     */
    long odtype,		/* data type of output image                 */
    long msgtim,		/* elapsed time between display messages     */
    char *minmax		/* min max flag                              */
)
{
    struct FDESC * fdesc[MAXIMG+1]; 
    struct GDESC * gdesc;
    extern long curent;         /* current line being processed              */
    static  long    opt[3]={0,0,0};/* optional parameter array for EOPENS    */ 
    double line_inc = 1.0;          /* line increment                        */
    double maxval[MAXBND]; 	        /* max pixel value array                 */
    double minval[MAXBND];	        /* min pixel value array                 */
    double samp_inc = 1.0;          /* sample increment                      */
    long	access;			/* image I/O access type                     */
    long	band;			/* loop index counting the bands             */
    long	bufsiz;			/* size of image I/O buffer                  */
    long	image;			/* loop index counting input images          */
    long	size;			/* number of bytes per pixel 	 	     */
    long	msg_flag=ALLCAT;	/* flag for porcessing message       */
    long	nl;			/* number of image lines                     */
    long	ns;			/* number of image samples                   */
    long	one = 1;		/* value of one                          */
    long	sl;			/* starting image line                       */
    long	ss;			/* starting image sample                     */
    long	status;			/* function return status code           */
    long	zero = 0;		/* value of zero                         */
    long	nonfatal = 1;		/* nonfatal error code		 	     */
    long	fatal = -1;		   /* fatal error code                   */
    char 	*thebuf;		   /* image I/O buffer                   */

    access = IREAD;
    nl = window[0][2];
    ns = window[0][3];
    for (band = 0,image = 0;image < *nmrimg; image++)/* open input image(s)  */
       {
       sl = window[image][0];
       ss = window[image][1];
       status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,
                         &nl,&ns,&offset1[band],&access,&odtype,&zero,opt);
       if (status != E_SUCC)
	  c_errmsg("Fatal error encountered","abs-fatal",&fatal);
       band += nbands[image];
       }
    access = IWRITE;				/* open output image         */
    status = c_eopens(&fdesc[*nmrimg],hosout,totbnd,&one,&one,&nl,&ns,offset2,

						    &access,&odtype,&zero,opt);
    if (status != E_SUCC)
       c_errmsg("Fatal error encountered", "abs-fatal",&fatal);
    fdesc[*nmrimg+1] = 0;
    if ((thebuf = c_ealloc(fdesc,&bufsiz)) == NULL)	/* allocate I/O buffer  */
       {
       c_errmsg("Error allocating dynamic memory","abs-alloc",&nonfatal);
       c_errmsg("Fatal error encountered","abs-fatal",&fatal);
       }
    if (c_egroup(fdesc,&gdesc,(unsigned char *) thebuf,
		  &bufsiz) != E_SUCC) /* group images         */
       c_errmsg("Fatal error encountered", "abs-fatal",&fatal);
    switch(odtype)
       {
       case EBYTE:  size = 1; break;
       case EWORD:  size = 2; break;
       case ELONG:	size = 4; break;
       case EREAL:  size = 4; break;
       }
    curent = 1;    /* for processing message utility  */
    c_inittm("ABS", "Processed", "line", "bands", "",&nl,totbnd,&msgtim,
             &msg_flag);
    c_startm();
    if (strcmp(minmax,"YES") == 0)		/* initialize min max arrays  */
       c_inmnmx(&odtype,totbnd,minval,maxval);
    for (curent = 0; curent < nl;curent++)/*copy input image to output image  */
       {
       if (c_estep(&gdesc) != E_SUCC)
	  c_errmsg("Fatal error encountered", "abs-fatal",&fatal);
       for (band = 0; band < *totbnd; band++)
	  {
	  switch(odtype)
	     {
	     case EBYTE:  
	       absbyte( (unsigned char *)thebuf,band,ns);
	       break;
	     case EWORD:  
	       absword( (short *)thebuf,band,ns);
	       break;
	     case ELONG:  
	       abslong( (long *)thebuf,band,ns);
	       break;
	     case EREAL:  
	       absfloat( (float *) thebuf,band,ns);
	       break;
	     }
	  if (strcmp(minmax,"YES") == 0)
	     status = c_pxmnmx((long *)(thebuf + (*(offset1+band) * size)),&ns,
			     &minval[band],&maxval[band],&odtype); 
	  }
       }
    if (c_estep(&gdesc) != E_SUCC)
       c_errmsg("Fatal error encountered", "abs-fatal",&fatal);
    c_stoptm();
    /*
       update output image ddr, min and max
    */
    c_upddr (vblock, hosin, hosout, nmrimg, window, bands, nbands, &line_inc, 
						    &samp_inc, totbnd, &zero);
    if (strcmp(minmax,"YES") == 0)
       if (c_svmnmx(hosout,totbnd,minval,maxval,"YES") != E_SUCC)
	  c_errmsg("Nonfatal error encountered","abs-warn",&nonfatal);
    if (c_egclse(&gdesc) != E_SUCC)	/* close all image files in the group   */
       c_errmsg("Nonfatal error encountered","abs-warn",&nonfatal);
    free(thebuf);
    return;
}


static void absbyte
(
    unsigned char thebuf[],
    long band,
    long ns
)

{
    extern long offset1[];
    extern long offset2[];
    long i;

    for (i = 0; i < ns; i++)
       thebuf[offset2[band] + i] = abs( thebuf[offset1[band] + i]);
}

static void absword
(
    short thebuf[],
    long band,
    long ns
)

{
    extern long offset1[];
    extern long offset2[];
    long i;

    for (i = 0; i < ns; i++)
       thebuf[offset2[band] + i] = abs( thebuf[offset1[band] + i]);
}

static void abslong
(
    long thebuf[],
    long band,
    long ns
)

{
    extern long offset1[];
    extern long offset2[];
    long i;

    for (i = 0; i < ns; i++)
       thebuf[offset2[band] + i] = abs( thebuf[offset1[band] + i]);
}

static void absfloat
(
    float thebuf[],
    long band,
    long ns
)

{
    extern long offset1[];
    extern long offset2[];
    long i;

    for (i = 0; i < ns; i++)
     thebuf[offset2[band] + i] = (float)fabs((double)thebuf[offset1[band] + i]);
}
