#include "add2stat.h"
#include "util.h"
#include "protos.h"

void add_pick
(
    char hos_stout[][CMLEN],     /* output stats file host name       */
    long *numclass_req,          /* no. classes to be picked          */
    long numclass_pick,          /* no. classes in pick stats file    */
    long *numclass_out,          /* no. classes in output stats file  */
    char runtype[]              /* batch/interactive indicator       */
)
/*
*****************************************************************************

NAME:	ADD_PICK

PURPOSE: Adds selected classes from pick statistics file to 
         output statistics file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2-Jun-83  K. Mcgee
   2          6-Feb-89  K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

*****************************************************************************
*/
{
long	all_classes = TRUE;
long	clind[MAXCLS];           /* class indices to be selected     */
long	i, ic, icc, id, idd, j;
long	ns, nshape[9], shape[9];

int term;

char	data_type[26];
char	messg[ERRLEN];
unsigned char	sbuf[16000];             /* data buffer - arbitrarily large  */
char	stname[CMLEN];

long    inf_err = 1;
long    one = 1;
long    zero = 0;

/*
if only some classes are to be copied list all available classes
prompt user for class names and get class indices
----------------------------------------------------------------
*/
if (*numclass_req != numclass_pick) 
   {
   all_classes = FALSE;
   clprompt (&fd_pick, numclass_pick, numclass_req, clind);
   t_clear ();
   }
/*
copy all/requested classes
--------------------------
*/
for (i = 1; i <= *numclass_req; i++)
   {
   if (all_classes) ic = i;
   else ic = clind[i-1];

/*
   get name of the class and number of sites in it
   -----------------------------------------------
*/
   if (c_stget (&fd_pick, "NAME", &ic, &zero, nshape, 
                    (unsigned char *) stname) != E_SUCC)
      {
      c_abort (&block, "Unable to get class information", "msg_noclass",
               "add2stat", hos_stout, &one);
      }
   stname[nshape[2]] = '\0';

   if (c_stget (&fd_pick, "NSITES", &ic, &zero, shape, 
                   (unsigned char *) &ns) != E_SUCC)
      c_abort (&block, "Unable to get site information", "msg_nosite",
               "add2stat", hos_stout, &one);
/*
   if class name already exists in output stats file prompt user for
   new name; abort if batch job
   -----------------------------------------------------------------
*/
   if (c_stfdcl (&fd_base, stname, &icc) !=  E_NOCLN)
      {
      erase_lines (19, 20);
      t_pos (19, 1);
      strcpy (messg, "Duplicate class name found for class: ");
      strcat (messg, stname);
      c_errmsg (messg, "add2stat-dupname", &inf_err); 

      if (runtype[0] ==  'B') 
         c_abort (&block, "Run ADD2STAT interactively", "add2stat-batch", 
                  "add2stat", hos_stout, &one);

      while (1)
         {
         t_pos (21, 1);      
         t_write ("Please enter a new class name ", T_STDCC);
         erase_lines (23, 23);
         t_read (stname, &term);
         if (c_stfdcl (&fd_out, stname, &icc) == E_NOCLN) break;
         erase_lines (21, 23);
         t_pos (20, 1);
         strcpy (messg, "Duplicate output class name: ");
         strcat (messg, stname);
         c_errmsg (messg, "add2stat-dupname", &inf_err); 
         }
      nshape[2] = (long) strlen (stname);
      } 

   id = 1;
   (*numclass_out)++;

/*
   copy all the information in that class
   --------------------------------------
*/
   erase_lines (20, 24);
   while (1)
      {
      if (c_stnode (&fd_pick, &ic, &zero, &id, data_type, shape) != E_SUCC)
         {
         c_stput (&fd_out, "NAME", numclass_out, &zero, nshape, 
                   (unsigned char *) stname);
         break;
         }
      else
         {
         c_stget (&fd_pick, data_type, &ic, &zero, shape, sbuf);
         c_stput (&fd_out, data_type, numclass_out, &zero, shape, sbuf);
         } 
      id++;
      } 

/*
   copy all the information for the sites
   --------------------------------------
*/
   for (j = 1; j <= ns; j++)
      {
      idd = 1;
      while (1)
         {
         if (c_stnode (&fd_pick, &ic, &j, &idd, data_type, shape) != E_SUCC)
            break;
         else
            {
            c_stget (&fd_pick, data_type, &ic, &j, shape, sbuf);
            c_stput (&fd_out, data_type, numclass_out, &j, shape, sbuf);
            idd++;
            } 
         } 
      } 
   } 

/*
close stats files
-----------------
*/
c_stclse (&fd_base); 
c_stclse (&fd_pick); 
c_stclse (&fd_out); 

}
