#include "add2stat.h"
#include "util.h"
#include "protos.h"

void copy_base
(
    char hos_stbase[],      /* base stats file host name         */
    char hos_stpick[],      /* pick stats file host name         */
    char hos_stout[],       /* output stats file host name       */
    long *numclasses_req,   /* no. classes to be picked          */
    long *numclasses_pick,  /* no. classes in pick stats file    */
    long *numclasses_out   /* no. classes in output stats file  */
)
/*
*****************************************************************************

NAME:	COPY_BASE

PURPOSE: copies the bases statistics file to the output statistics file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2-Jun-83  K. Mcgee
   2          6-Feb-89  K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Open base and pick statistics file and get number of classes
   If no. classes requested by user > that in pick file
      get right no. from user
   If total no. classes in output file > MAXCLS abort
   Call STAT_COPY to copy base file to output
   Open output stats file for adding new classes

*****************************************************************************
*/
{
long	access, 
	cols, 
	len, 
	lines, 
	ntot,
	numclasses_base,
	type;

int term;
char	resp[2];
long    fatal_err = -1;

/*
open and get number of classes in pick stats file
-------------------------------------------------
*/
access = IREAD;
c_stopen (&fd_pick, hos_stpick, &access, numclasses_pick);

/*
open and get number of classes in base stats file
-------------------------------------------------
*/
c_stopen (&fd_base, hos_stbase, &access, &numclasses_base);

/*
check number of classes against user request
--------------------------------------------
*/
t_init (&lines, &cols, &type);
if (*numclasses_req  >  *numclasses_pick) 
   {
   t_write ("Number of classes specified > number in file", T_STDCC);
   while (1)
      {
      t_write ("Do you wish to proceed using all classes in file (Y or N)?: ",
               T_PROMPT);
      t_read (resp,&term);      
      len = (long) strlen (resp);
      c_low2up (resp, &len);
      if (resp[0] == 'Y' || resp[0] == 'N') break;
      }

/*
   abort or continue using all classes according to user request
   -------------------------------------------------------------
*/
   if (resp[0] == 'N') 
      c_errmsg ("Too many classes requested.  Restart.", "add2stat-classes",
                &fatal_err);
   else
     *numclasses_req = 0;
   } 

/*
use all classes as default
--------------------------
*/
if (*numclasses_req  ==  0) *numclasses_req = *numclasses_pick;

/*
check if #classes in base + #classes in pick > MAXCLS
-----------------------------------------------------
*/
ntot = *numclasses_req + numclasses_base;
if (ntot  >  MAXCLS) 
   c_errmsg ("Number of classes in output file is > MAXCLS", 
             "add2stat-nclass", &fatal_err); 

/*
copy base stats file to output stats file
-----------------------------------------
*/
stat_copy (hos_stbase, hos_stout);

/*
open output stats file for adding new classes
---------------------------------------------
*/
access = IUPDATE;
if (c_stopen (&fd_out, hos_stout, &access, numclasses_out) !=  E_OPOLD)
   c_errmsg ("Error opening output statistics file", "add2stat-statout",
             &fatal_err); 
}
