#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAXCH   4

void applylut
(
    char hosin[][CMLEN],	/* input image host name		*/
    char hosout[],	/* output image host name		*/
    long num_inbands,	/* no. bands				*/
    long band_num[],	/* band numbers				*/
    long wind[],		/* image window				*/
    float lookup[][256]	/* lookup table radiance values		*/
)
/*
***************************************************************************

NAME: APPLYLUT

PURPOSE: Applies radiance lookup table to input image bands to get
         the output image

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0        06/15/84   K. Gacke        
   2.0        07/28/89   K. A. Narayanan    conversion to 5.0
   3.1        03/04/93   D. Etrheim         Corrected windowing problem

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
extern  struct PARBLK block;                   /* TAE parameter block     */

extern  float   *iobuf;                        /* image i/o buffer        */
extern  struct GDESC * gdesc;                  /* group descriptor        */
extern  long    ipoint[MAXCH + 1],             /* offsets into image i/o  */
                opoint;                        /* for input and output bands */
extern  long    min_max;                       /* minmax option flag      */
extern  long    msgtim;                        /* message interval        */

extern  long    curcat;                        /* parameters for processing */
extern  long    curent;                        /* message utility         */

double	maxval, minval;

float	outval;

long	ftype = EREAL;
long	ii, line;
long	msg_flag = GEN2;
long	one = 1;
long	samp;
long	zero = 0;

unsigned char	bdata;	
/*
initialize the processing message utility and min/max
-----------------------------------------------------
*/
c_inittm ("ALBEDO", "Processed", "line", "", "", &wind[NL], &zero, 
          &msgtim, &msg_flag);
curent = 1;
c_startm ();

c_inmnmx (&ftype, &one, &minval, &maxval);
/*
generate output image using look-up table
-----------------------------------------
*/
for (line = 0; line < wind[NL]; line++, curent++)
   {
   c_estep (&gdesc);
   for (samp = 0; samp < wind[NS]; samp++)
      {
      outval = 0.0;
      for (ii = 0; ii < num_inbands; ii++)
         {
         bdata = (unsigned char) iobuf[ipoint[ii] + samp];
         outval += lookup[band_num[ii]-1][bdata];
         }
      iobuf[opoint+samp] = outval;
      if (min_max)
         {
         maxval = max (maxval, outval);
         minval = min (minval, outval);
         }
      }
   }
c_estep (&gdesc);

c_stoptm(); 
c_egclse (&gdesc);
/*
update output image history and minimum and maximum values
----------------------------------------------------------
*/
c_hstout (&block, hosout, hosin, &one, "");

if (min_max)
   c_svmnmx (hosout, &one, &minval, &maxval, "YES");

}
