#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAXCH   4

void filspec
(
    char hosin[][CMLEN],      /* input image host name              */
    char hosout[][CMLEN],     /* output image host name             */
    long *num_bands,          /* number of bands in each input image*/
    long band_num[][MAXBND+1],/* band numbers                       */
    long wind[][4]            /* input image windows                */
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0       28-July-89  K. Narayanan
   6.1	     23-May-94   D. Eichacker		Added opt[] for eopens

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call C_EOPENS to open input and output images
   Call C_UPDDR to update output image DDR
   Call C_EGROUP to group the images

**************************************************************************
*/
{
extern  struct PARBLK block;                   /* TAE parameter block     */

extern  float   *iobuf;                        /* image i/o buffer        */
extern  struct FDESC * fdesc[3];               /* file descriptors        */
extern  struct GDESC * gdesc;                 /* group descriptor        */
extern  long    ipoint[MAXCH + 1],             /* offsets into image i/o  */
                opoint;                        /* for input and output bands */

extern  char    *prog_name;                     /* program name            */

extern  long    fatal_err;

double	line_inc = 1.0,
	samp_inc = 1.0;

long	access,                      /* file access type                   */
	bufsiz,                      /* image i/o buffer size              */
	comb_flag = COMB,            /* flag for upddr                     */
	one = 1,
	stat,                        /* return status                      */
	type,                        /* file type                          */
	zero = 0;

static  long	opt[3]={0,0,0};	     /* array for eopens                   */

/*
open input and output image files
---------------------------------
*/
access = IREAD;
type = EREAL;
stat = c_eopens (fdesc, hosin[0], band_num[0], &wind[0][SL], &wind[0][SS],
                 &wind[0][NL],&wind[0][NS], ipoint, &access, &type, &zero, opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening input image", "albedo-openin", 
             &fatal_err);

access = IWRITE;
stat = c_eopens(&fdesc[1], hosout[0], &one, &one, &one,
               &wind[0][NL], &wind[0][NS], &opoint, &access, &type, 
               &zero, opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "albedo-openout", 
             &fatal_err);

fdesc[2] = 0;

/*
 update output image DDR based on input images
---------------------------------------------
*/
c_upddr (&block, hosin, hosout[0], &one, wind, band_num, 
         num_bands, &line_inc, &samp_inc, &one, &comb_flag);

/*
set up group containing input and output bands
----------------------------------------------
*/
if ((iobuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "albedo-alloc", 
            prog_name, hosout, &one);
c_egroup (fdesc, &gdesc, (unsigned char *) iobuf, &bufsiz);
}
