#include <stdio.h>
#include "las.h"
#include <math.h>
#include "protos.h"

#define MAXCH   4

void getlut
(
    float lookup[][256],	/* look up table with radiance values	*/
    long dark[],		/* dark values specified by the user	*/
    double sunang,		/* sun angle				*/
    long sat_code,	/* satellite identification code	*/
    long num_bands,	/* no. bands in input image		*/
    long band_num[]	/* band numbers				*/
)
/*
***************************************************************************

NAME: GETLUT

PURPOSE: Computes the radiance tables.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0        06/15/84   K. Gacke        
   2.0        07/28/89   K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

  See PDF/User's Guide for a detailed description and the equations used

  compute mult = sun irradiance multiplied by 10000 to obtain 
	 	albedo values in 100's percent
	      = 10000 * pi / ((sum of solar radiance) * sin(sunangle))

  compute look up table elements using the albedo equation for band i:
      table(i,j) = mult*(j/cscale(i)*rdiffs(i,sat)+rmins(i,sat)

  set up dark values defined by the user

***************************************************************************
*/
{
float	    e, mult;

static	float	rmins[][MAXCH] = {  /* satellite radiance minimums	*/
	0.00 , 0.00 , 0.00 , 0.00 , 
	0.10 , 0.07 , 0.07 , 0.14 , 
	0.08 , 0.06 , 0.06 , 0.11 , 
	0.04 , 0.03 , 0.03 , 0.03 , 
	0.04 , 0.03 , 0.03 , 0.03 , 
	0.02 , 0.04 , 0.04 , 0.10 , 
	0.02 , 0.04 , 0.05 , 0.12 , 
	0.00 , 0.00 , 0.00 , 0.00 };

static	float	rdiffs[][MAXCH] = {  /* satellite detector radiance ranges */
	2.48 , 2.00 , 1.76 , 4.00 , 
	2.00 , 1.49 , 1.33 , 4.01 , 
	2.55 , 1.70 , 1.46 , 3.80 , 
	2.16 , 1.72 , 1.42 , 4.38 , 
	2.55 , 1.76 , 1.46 , 3.80 , 
	2.28 , 1.76 , 1.26 , 3.90 , 
	2.34 , 1.60 , 1.37 , 3.37 , 
	0.00 , 0.00 , 0.00 , 0.00 };

static	float	eirrad[] = {	     /* solar radiances			*/
	17.70 , 15.15 , 12.37 , 24.9};

static	float	cscale[] = {
	127.0 , 127.0 , 127.0 , 127.0};


long	 band, icount, ii, ij;

/*
sum solar radiance values for each channel specified
----------------------------------------------------
*/
for (ii = 0, e = 0; ii < num_bands; ii++) e += eirrad[band_num[ii]-1];

mult = 10000.0 * (3.14159 / (e * sin ((double) sunang)));

/*
calculate look-up table values
------------------------------
*/
for (ii = 0; ii < num_bands; ii++)
   {
   band = band_num[ii] - 1;

   icount = 255 - dark[band];

   for (ij = 0; ij <= icount; ij++)
      lookup [band][ij+dark[band]] = 
             mult * ((((float) ij / cscale[band]) * rdiffs[sat_code][band]) + 
             rmins[sat_code][band]);
   }
/*
set up dark values
------------------
*/
for (ii = 0; ii < num_bands; ii++)
   {
   band = band_num[ii] - 1;
   for (ij = 0; ij < dark[band]; ij++)
      lookup[band][ij] = lookup[band][dark[band]];
   }
}
