#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAXCH   4

void getpar
(
    char hosin[][CMLEN],      /* input image host names           */
    char lasout[][CMLEN],     /* output image las name            */
    char hosout[][CMLEN],     /* output image host name           */
    long image_band_num[][MAXBND+1], /* image band numbers               */
    long *num_inbands,        /* number of input image bands      */
    long wind[][4],             /* input image window               */
    long *satellite_code,     /* satellite identification         */
    float *sunang,             /* sun angle                        */
    long dark[]              /* dark values                      */
)
/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0        06/15/84   K. Gacke        
   2.0        07/28/89   K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
extern  struct PARBLK block;                   /* TAE parameter block     */

extern  long    min_max;                       /* minmax option flag      */
extern  long    msgtim;                        /* message interval        */

extern  long    fatal_err;
extern  long    inf_err;

struct  VARIABLE *var;               /* pointer to a variable in TAE block */

long  count, 
	flag[MAXIMG][2],            /* flag for subwindows                */
	ftype = EREAL,
	i,
	imglimit = 1,               /* maximum number of images           */
	itype = ELONG,
	num_image_bands,            /* number of bands in each input image */
	num_inimages,               /* number of input images           */
	one = 1,
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out,                    /* logical unit for standard output   */
	zero = 0;

char   lasin[1][CMLEN],      	    /* input image las name               */
	minmax[4];                  /* minmax option                      */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input image parameters
---------------------------------------
*/
stat = c_getin (&block, "IN", &zero, lasin, hosin, &num_inimages, wind, 
                image_band_num, &num_image_bands, num_inbands, &imglimit, 
                &same_window, flag);

if (stat != E_SUCC) 
    c_errmsg ("Error accessing the input image", "albedo-in", 
              &fatal_err);

if (*num_inbands  > MAXCH) 
   {
   c_errmsg ("No. of input bands exceeds limit. Reset to 4", "albedo-nbands", 
             &inf_err);
   *num_inbands = MAXCH;
   }
for (i = 0; i < *num_inbands; i++)
   if (image_band_num[0][i] > 4)
      c_errmsg ("Band numbers cannot be > 4", "albedo-bandnum", &fatal_err);
/*
get and validate output image parameters
----------------------------------------
*/
if (c_getout (&block, "OUT", lasout, hosout) != E_SUCC)
   c_errmsg("Error getting output image name", "albedo-out", &fatal_err);
/*
get index for satellite radiance constant
-----------------------------------------
*/
c_getnum (&block, "LANDCC", &one, &itype, (unsigned char *) satellite_code); 
(*satellite_code)--;

/*
get sun angle from the horizon and convert to radians
-----------------------------------------------------
*/
c_getnum (&block, "SUNANG", &one, &ftype, (unsigned char *) sunang);
*sunang *= 3.14159 / 180.;

/*
get dark values
---------------
*/
count = MAXCH;
c_getnum (&block, "DARKLEV", &count, &itype, (unsigned char *) dark);

/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;
}
