
/*******************************************************************************
NAME:     ANNOTATE 
PURPOSE:  program main()
	  ANNOTATE puts annotation text into an image.

PROGRAM HISTORY
---------------------------------------------------------------------------
Date       Programmer     Site   Changes
-----      -------------  ---- ------------------------------------------------
30-nov-81  J. Hwang       sasc
 4-feb-82  J. Hwang,      sasc
 7-jan-88  M. Mazzella    SAR/GSF Conversion to C
12/84      J.Kutler     
01/86      J.Stansbery  
06/89      K.Narayanan         Conversion to 5.0, copied from classmap and 
			       modified
08/89      K.Narayanan         Conversion to 5.0, was passing host name for 
			       completion message
08/90      D.Etrheim      EDC  Corrected to update min/max values, added error 
			       checking on supports 
11/91      J.Fenno        EDC  Added HOR/VERT direction flag for each of the ten
			       annotation strings.  Modified include files to 
			       remove all of the "multiply defined" errors.
09/95      D. Zokaites    EDC  Fixed some problems found via lint:  extra
			       variables, int functions that should be void;
			       deleted "#include <stdio.h>";  added variable
			       use_bgrey;  added some inline comments;  wrote
			       ccell.h;  replaced long tb[] with struct ccell;
			       fixed bugs in image background and foreground;
			       corrected checks for annotation exceeding image
			       size;  deleted some unnecessary variables;
			       consolidated change logs;  added checks for
			       unsupported annotation characters;  rewrote
			       users guide;  called c_lasmsg() instead of
			       c_errmsg();  took valid range off chargrey and
			       backgrnd parms;  corrected min/max updating;
			       made a nonfatal window error fatal;  freed
			       dynamic memory. 
11/99	   D. Hames       EDC  Remove 50000 limits from SL and SS parameters
			       from annochk.pdf, annotate.pdf and annotate_.pdf.
			       This is to support images > 2gig. Add prototypes.
04/00      D. Hames       EDC  Add the 4 fonts found in colorfire into 
                               put_string.  This is an addition to the font 
                               already found in annotate.  Add the parameter 
                               FONT. Modify annotate.pdf to allow an 
                               additional 10 text string lines to be inserted 
                               into an image. Since MAX_ANNOTATIONS was added
			       to protos.h, change the name of protos.h to
			       annotate.h.  Update tests directory to test
			       the additional fonts. CR#1531.
                               
ALGORITHM DESCRIPTION:
   Call GETPAR to get parameters from TAE
   Call FILSPEC set up image i/o specifications
   Call PUT_STRING to insert the annotation
******************************************************************************/

#include "las.h"     /* las header file */ 
#define GLOBALVARS   /* defined only for main(), flags iovars.h */ 
#include "iovars.h"  /* declare variables for image IO */ 
#include "util.h"
#include "annotate.h"

void main
(void) 
{

/* declare variables */ 
   float bgrey;			/* background grey level	  */
   float cgrey[MAX_ANNOTATIONS];/* character grey level		  */

   long cellwide;               /* width of each font character   */
   long cellhigh;               /* heigth of each font character  */
   long image_band_num[MAXIMG][MAXBND + 1]; 
				/* band numbers in each input image*/
   long	image_type;		/* image data type		  */
   long	nlout;			/* output no. lines		  */
   long	nsout;			/* output no. samples per line 	  */
   long num_image_bands[MAXIMG];/* number of bands in each input image*/
   long	num_inimages;           /* number of input images         */
   long	num_strings;		/* no. annotation text strings    */
   long	scale[MAX_ANNOTATIONS];	/* scale factors for strings      */
   long	sl[MAX_ANNOTATIONS];	/* line positions for text strings*/
   long	sp[MAX_ANNOTATIONS];	/* pixel positions for strings    */
   long	textlen[MAX_ANNOTATIONS];	/* lengths of text strings*/
   long use_bgrey;              /* flag, use bgrey?		  */ 

   char	direct[MAX_ANNOTATIONS][5];     /* direction for strings  */
   char font[6];                /* font specified by user         */
   char	subcmd[10];		/* subcommand			  */
   char hosin[MAXIMG][CMLEN];   /* input images host names        */
   char lasin[MAXIMG][CMLEN];   /* input images las names         */
   char noinput[1][CMLEN];     /*parm to c_hstout(), there are no input images*/
   char text[MAX_ANNOTATIONS][264];	 /* annotation text strings*/

/* get input parameters */
   getpar (FALSE, lasin, hosin, &num_inimages, num_image_bands, image_band_num, 
	   &nlout, &nsout, sp, sl, scale, direct, &use_bgrey, &bgrey, cgrey, 
	   text, textlen, &num_strings, subcmd, font, &cellwide, &cellhigh);

/* open image file */
   filspec (hosin, num_inimages, &nlout, &nsout, &image_type);

/* insert annotation in image */
   put_string (sp, sl, textlen, scale, use_bgrey, bgrey, cgrey, direct, text, 
	      &num_strings, num_inimages, num_image_bands, image_band_num, 
	      &nsout, &image_type, font, cellwide, cellhigh);

/* update image history */
   if (c_hstout (&block, hosin[0], noinput, &zero, "") != E_SUCC)
      c_lasmsg (FALSE, "annotate-nonfatal", "Nonfatal error encountered"); 

/* issue completion message */
   if (strcmp (subcmd, "INPLACE") == 0)
      c_complt ("ANNOTATE",(char (*)[256])"", &zero);
   else
      c_complt ("ANNOTATE", lasin, &one);
}


