
/******************************************************************************
NAME:     genchr()
FUNCTION: generate character to given buffer
******************************************************************************/

#include "las.h"    /* las header file */ 
#include "annotate.h"

void genchr
(
    float iobuf[],        	/* I/O : output line 			*/
    long *vcnt, 	        /* I: current line count on the picture */
    struct CCELL *ccell,  	/* I: input info on the character cell  */
    char chrbuf[],        	/* I: character(s) to be expanded       */
    char font[],                /* I: font specified by user            */
    long cellwide,              /* I: width of each font character      */
    long cellhigh               /* I: heigth of each font character     */
)
{
extern unsigned char font1[96][12];
extern unsigned int font2[96][24];
extern unsigned int font3[96][40];
extern unsigned int font4[96][48];
extern unsigned int font5[96][58]; 

/* declare variables */ 
   long	code, lc, scale, x, nchar, index;
   unsigned int	bt; 
   long	pointr, i, k, m;
   long foreground;  		/* flag, is this a foreground pixel? = T or F*/ 
   float pixval;     		/* pixel value to assign to this character   */ 


/* return if table description indicates out of range */
   if ((*vcnt < ccell->table0) || (*vcnt > ccell->table0 + ccell->nlines - 1)) 
      return;

/* get scale */
   index = 1;
   ccell->table8++;
   if (ccell->horizontal) 
      {
      /* set up variables for horizontal text */
	 scale = ccell->nsamps / (ccell->nchars * cellwide);
	 nchar = ccell->nchars;
      }
   else
      {
      /* set up variable for vertical text */
	 scale = ccell->nlines / (ccell->nchars * cellhigh);    
	 if (scale < 1)
	     scale = 1;
	 index = (ccell->table8 - 1) / (scale * cellhigh) + 1;
	 nchar = 1;
      }

/* transfer current line of character dots to buffer */
   pointr=0;      
   lc = ((ccell->table8 - 1) % (scale*cellhigh)) / scale + 1;      

#ifdef vms
   x = 8 * ((lc - 1) / 4) + 4 - lc;   /* bytes in a word have to be reversed */
#else
   x = lc - 1;
#endif

/* loop for all chars */ 
   for (i = 1; i <= nchar; i++)
      {
      /* get ascii character code and convert to an index to the dot table */ 
	 code = chrbuf[index + i - 2] - 32;

      /* if illegal character then use a question mark */ 
	 if (code == -32)
	     code = 0;
	 if ((code > 96) || (code < 0)) 
	    code = 31;

      /* get a byte from the character dot table */ 
         if (strcasecmp(font, "font1") == 0)
	     bt = font1[code][x];
         else if (strcasecmp(font, "font2") == 0)
             bt = font2[code][x];
         else if (strcasecmp(font, "font3") == 0)
             bt = font3[code][x];
         else if (strcasecmp(font, "font4") == 0)
             bt = font4[code][x]; 
         else if (strcasecmp(font, "font5") == 0)
             bt = font5[code][x];
         else   
             {
             c_lasmsg(TRUE, "annotate-gen", 
                      "FONT %s contains an invalid font name", font);
             }

      /* loop for all bits in this byte */ 
	 for ( k=cellwide; k; k--)
	    {
	    /* get a bit from the byte.  If bit is on, set pixel value to
	       foreground, else set it to backround value */ 

	       if ( bit(bt,k) ) 
		  {
		  foreground = TRUE; 
		  pixval = ccell->foreground;  
		  }
	       else
		  {
		  foreground = FALSE; 
		  pixval = ccell->background; 
		  }

	    /* optionally fill buffer with specified value */ 
	       if (foreground || !foreground && ccell->use_back)
		  for (m = 0; m < scale; m++)
		     iobuf[ccell->sample + pointr + m - 1] = pixval; 

	    /* increment buffer index */ 
	       pointr += scale;      
	    }

      }
}



