
/*******************************************************************************
NAME:    getpar()
PURPOSE: Gets parameters
         Note: GETPAR is used by two programs, ANNOCHK and ANNOTATE.
*******************************************************************************/

#include "string.h"  /* for various string functions */ 
#include "las.h"     /* las header file */ 
#include "iovars.h"  /* declare variables for image IO */
#include "typlim.h"  /* for MAXREAL */ 
#include "util.h"
#include "annotate.h"

void getpar
(
    long first,                     /* I: controls parameter checking         */
    char lasin[][CMLEN],            /* O: input image las names               */
    char hosin[][CMLEN],            /* O: images host names                   */
    long *num_inimages,             /* O: number of input images              */
    long num_image_bands[],         /* O: number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* O: band numbers in each input image */
    long *nlout,                    /* O: output no. lines                    */
    long *nsout,                    /* O: output no. samples per line         */
    long sp[],                      /* O: annotation pixel positions          */
    long sl[],                      /* O: annotation line positions           */
    long scale[],                   /* O: scale factors for strings           */
    char direct[][5],               /* O: direction for annotation            */
    long *use_bgrey,                /* O: flag, use bgrey 	              */
    float *bgrey,                   /* O: background grey value               */
    float cgrey[],                  /* O: character grey value                */
    char text[][264],               /* O: annotation text                     */
    long textlen[],                 /* O: lengths of strings                  */
    long *num_strings,              /* O: no. strings                         */
    char subcmd[],                  /* O: subcommand to execute               */
    char font[],		    /* O: font specified by user              */
    long *cellwide,		    /* O: width of each font character        */
    long *cellhigh                  /* O: heigth of each font character       */ 

)

{

/* declare variables */ 
   struct DDR      ddr;            /* holds an image ddr */ 
   struct VARIABLE *var;           /* points to a tae parameter */ 

   double max_grey;                /* max data value */ 
   double max_val[MAXBND];         /* max data value in images */ 
   double min_grey;                /* min data value */ 
   double min_val[MAXBND];         /* min data value in images */ 

   long   bandnum;                 /* loop index, image band */ 
   long   count; 
   long   direction_count;         /* # of input HOR or VERT directions */
   long   five;                    /* location to hold the value 5, =
				      strlen(VERT) + 1 */
   long   flag[MAXIMG][2];         /* flag for subwindows */
   long   ftype = EREAL;           /* image data is processed as r*4 */ 
   long   i;                       /* loop index */ 
   long   icgcnt; 
   long   imagenum;                /* loop index, image number */ 
   long   imglimit = MAXIMG;       /* maximum number of images */
   long   inl;                     /* input number of lines */ 
   long   inp;                     /* input number of pixels (samples) */
   long   inum;
   long   isccnt;
   long   islcnt;                  /* input count of sl */ 
   long   ispcnt;                  /* input count of sp (ss) */ 
   long   itype = ELONG;           /* input parameter type */ 
   long   length; 
   long   maxl;
   long   maxp;
   long   min_max;                 /* flag, update min/max, = T or F */
   long   num_inbands;             /* total number of input image bands */
   long   numstr;                  /* no. annotation strings */
   long   same_window;             /* flag indicating if images have same 
				      window */
   long   std_out;                 /* standard output, parm to c_inlas() */
   long   max_anno=MAX_ANNOTATIONS;/* number of parameter values */ 
   long   wind[MAXIMG][4];         /* image windows */ 
   long   cur;
   long   temp_band[MAXIMG][MAXBND + 1]; 
   long   temp_nb[MAXIMG];

   char   hosout[CMLEN];           /* output image host name */
   char   lasout[CMLEN];           /* output image las name */
   char   minmax[4];               /* minmax flag */
   char   tmptext[MAX_ANNOTATIONS*2][132];   /* buffer for annotation text */

/* initialize program */ 
   c_inlas (&block, &std_out);

/* get annotation text as two strings (132 character maximum each) and
   combine them as one string
----------------------------------------------------------------------*/
   count = MAX_ANNOTATIONS*2;
   length = 132;
   for (i = 0; i < count; i++)
      tmptext[i][0] = '\0';
   numstr = c_getstr (&block, "ANNOTE", &count, &length, (char *)tmptext);
   *num_strings = (numstr + 1) / 2;
   for (i = 0; i < *num_strings; i++)
      {
      strcpy (text[i], tmptext[2*i]);
      strcat (text[i], tmptext[2*i+1]);
      textlen[i] = (long)(strlen (text[i]));
      }

/* get other parameters 
-----------------------*/
   count = MAX_ANNOTATIONS;
   islcnt = c_getnum (&block, "SL", &count, &itype, (unsigned char *)sl);
   ispcnt = c_getnum (&block, "SS", &count, &itype, (unsigned char *)sp);
   if ((isccnt = c_getnum (&block, "SCALFACT", &count, &itype, 
                          (unsigned char *)scale)) == 0)
      {
      isccnt = *num_strings;
      for (i = 0; i < *num_strings; i++)
	 scale[i] = 1;
      }
   five =  5;
   direction_count = c_getstr (&block,"ANOTDIR",&max_anno,&five,(char *)direct);
   if ((icgcnt = c_getnum (&block, "CHARGREY", &count, &ftype, 
		(unsigned char *)cgrey)) == 0)
      {
      icgcnt = *num_strings;
      for (i = 0; i < *num_strings; i++)
	 cgrey[i] = 255.0;
      }
   if (c_getnum (&block, "BACKGRND", &one, &ftype, (unsigned char *)bgrey) == 0)
      *use_bgrey = FALSE;
   else
      *use_bgrey = TRUE; 

   c_getstr (&block, "_SUBCMD", &one, &length, subcmd);

   /*get font size and set cellhigh and cellwide (to use for error checking)*/ 
   c_getstr (&block, "FONT", &one, &length, font);
   if (strcasecmp (font, "font1") == 0)
      {
      *cellwide = 8;
      *cellhigh = 10;
      }
   else if (strcasecmp (font, "font2") == 0)
      {
      *cellwide = 13;
      *cellhigh = 24;
      }
   else if (strcasecmp (font, "font3") == 0)
      {
      *cellwide = 22;
      *cellhigh = 40;
      }
   else if (strcasecmp (font, "font4") == 0)
      {
      *cellwide = 26;
      *cellhigh = 48;
      }
   else if (strcasecmp (font, "font5") == 0)
      {
      *cellwide = 32;
      *cellhigh = 58;
      }
   else 
      c_lasmsg(TRUE, "annotate-font", "Invalid font %s", font);


 
/* if called by ANNOCHK check parameter values
----------------------------------------------*/
if (first)
   {
   /* get the size of the output image
   -----------------------------------*/
   if (strcmp (subcmd, "GENIMG") == 0) 
      {
      c_getnum (&block, "NL", &one, &itype, (unsigned char *)&inl);
      c_getnum (&block, "NS", &one, &itype, (unsigned char *)&inp);
      }
   else
      {                  
      /* get and validate input image parameters
      ------------------------------------------*/
      if (c_getin (&block, "IN", &zero, lasin, hosin, num_inimages, wind, 
		  image_band_num, num_image_bands, &num_inbands, &imglimit, 
                  &same_window, flag) != E_SUCC) 
	 c_lasmsg (TRUE, "annotate-fatal", "Fatal error encountered"); 

      if (strcmp (subcmd, "INOUT") == 0)
	 {
	 if (!same_window)
	    c_lasmsg(TRUE,"annotate-size", "Images must have the same size");
	 inl = wind[0][NL];
	 inp = wind[0][NS];

	 if (c_getout (&block, "OUT", (char (*)[256])lasout, 
			(char (*)[256])hosout) != E_SUCC)
	    c_lasmsg (TRUE, "annotate-fatal", "Fatal error encountered"); 
	 }
      else
	 {
	 for (imagenum = 0; imagenum < *num_inimages; imagenum++)
	    {
	    if (flag[imagenum][0] != 0)
	       c_lasmsg (TRUE, "annotate-wind", 
	                "Window specification not allowed");
	    c_getddr (hosin[imagenum], &ddr);
	    if (imagenum == 0)
	       {
	       inl = ddr.nl;
	       inp = ddr.ns;
	       }
	    else if (ddr.nl != inl || ddr.ns != inp)
	       c_lasmsg(TRUE,"annotate-size", "Images must have the same size");
	    }
	 }
      } 
   if (direction_count < *num_strings)
      c_lasmsg (TRUE, "annotate-number", 
	      "Number of values for ANNOTDIR is less than that for ANNOTE"); 
   if (isccnt < *num_strings)
      c_lasmsg (TRUE, "annotate-number", 
	      "Number of values for SCALFACT is less than that for ANNOTE"); 
   if (islcnt < *num_strings) 
      c_lasmsg (TRUE, "annotate-number", 
	      "Number of values for SL is less than that for ANNOTE"); 
   if (ispcnt < *num_strings)
      c_lasmsg (TRUE, "annotate-number", 
	      "Number of values for SS is less than that for ANNOTE"); 
   if (icgcnt < *num_strings)
      c_lasmsg (TRUE, "annotate-number", 
	      "Number of values for CHARGREY is less than that for ANNOTE"); 

   /* Check input text for unsupported characters 
   ----------------------------------------------*/ 
   for (inum = 0; inum < *num_strings; inum++)
      {
      if (strchr (text[inum], '`'))
	 c_lasmsg(TRUE, "annotate-badchar", "The ` character is unsupported");
      if (strchr (text[inum], '\t'))
	 c_lasmsg(TRUE, "annotate-badchar", "The tab character is unsupported");
      }

   /* Make sure that none of the text extends beyond the image boundaries
   ----------------------------------------------------------------------*/
   for (inum = 0; inum < *num_strings; inum++)
      {
      if ( direct[inum][0] ==  'H' )
	 {
	 maxl = sl[inum] - 1 + scale[inum] * (*cellhigh);   
	 maxp = sp[inum] - 1 + textlen[inum] * scale[inum] * (*cellwide); 
	 } 
      else
	 {
	 maxl = sl[inum] - 1 + textlen[inum] * scale[inum] * (*cellhigh); 
	 maxp = sp[inum] - 1 + scale[inum] * (*cellwide);
	 } 
      if (maxl > inl)
	 c_lasmsg(TRUE, "annotate-textlen", 
		 "Characters in string %d extend beyond the image's bottom",
		 inum+1);

      if (maxp > inp)
	 c_lasmsg(TRUE, "annotate-textlen", 
		 "Characters in string %d extend beyond the image's right side",
		 inum+1);
      
      if (sp[inum] < 1  ||  sl[inum] < 1)
	 c_lasmsg (TRUE, "annotate-fatal", "Program error in getpar()"); 
      }
   }
else /* called by ANNOTATE */ 
   {

   /* get and validate input image parameters
   ------------------------------------------*/
   if (c_getin (&block, "IN", &zero, lasin, hosin, num_inimages, wind, 
	       image_band_num, num_image_bands, &num_inbands, &imglimit, 
	       &same_window, flag) != E_SUCC) 
      c_lasmsg (TRUE, "annotate-fatal", "Fatal error encountered"); 

   if (c_getddr (hosin[0], &ddr) != E_SUCC)
      c_lasmsg (TRUE, "annotate-fatal", "Fatal error encountered"); 
   *nlout = ddr.nl;
   *nsout = ddr.ns;

   /* Get las globals
   ------------------*/
   var    = p_find( &block, "$msgtim");
   msgtim = IVAL( *var, 0);
   var    = p_find( &block, "$minmax");
   strcpy (minmax, SVAL( *var, 0));
   if (minmax[0] == 'Y')
      min_max = TRUE;
   else
      min_max = FALSE;

   /* update image minimum and maximum 
   ----------------------------------*/
   if (min_max)
      {
      /* find min/max in bgrey */ 
	 if (*use_bgrey == TRUE) 
	    max_grey = min_grey = *bgrey;
	 else 
	    {
	    max_grey = (double)MINREAL;
	    min_grey = (double)MAXREAL;
	    }

      /* find min/max in cgrey */ 
	 for (i = 0; i < *num_strings; i++)
	    {
	    if (max_grey < cgrey[i])
	       max_grey = cgrey[i];
	    if (min_grey > cgrey[i])
	       min_grey = cgrey[i];
	    }

      /* process all images 
      -------------------*/ 
      for (imagenum = 0; imagenum < *num_inimages; imagenum++)
	 {

	 /* set window for this image */ 
	    wind[imagenum][SL] = wind[imagenum][SS] = 1;
	    wind[imagenum][NL] = *nlout;
	    wind[imagenum][NS] = *nsout;
	 
	 /* set band numbers */ 
	    for (bandnum = 0; bandnum < num_image_bands[imagenum] ; bandnum++) 
	       temp_band[0][bandnum] = image_band_num[imagenum][bandnum]; 
	    temp_nb[0] = num_image_bands[imagenum]; 

	 /* determine current min/max */ 
	    if (c_minmax ((char (*)[256])hosin[imagenum], &one, wind, temp_band,
			 temp_nb, min_val, max_val) != E_SUCC)
	       {
	       c_lasmsg (FALSE, "annotate-nonfatal", 
			"Nonfatal error encountered");
	       /* can't update min max if c_minmax() failed, so return */
	       return;
	       }

	 /* update min/max with values from cgrey and bgrey */ 
	    for (bandnum = 0; bandnum < num_image_bands[imagenum]; bandnum++)
	       {
	       cur = image_band_num[imagenum][bandnum] - 1;
	       if (max_val[cur] < max_grey)
		  max_val[cur] = max_grey;
	       if (min_val[cur] > min_grey)
		  min_val[cur] = min_grey;
	       } 

	 /* save new values */ 
	    if (c_svmnmx (hosin[imagenum], temp_nb, min_val, max_val, minmax) 
			 != E_SUCC)
	       c_lasmsg (FALSE, "annotate-nonfatal", 
			"Nonfatal error encountered");
	 }
      }
   }
}



