/********************************************************************
NAME: COORDINATES

PURPOSE: Function to compute the coordinates for the image. 

 To get the center of the cell, using formula in the ARC Image
      Integration book, page 2-4. 

***********************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "arc2las.h"

void coordinates
(
  double xdim,         /* I: Cell size in the x dimension           */
  double ydim,         /* I: Cell size in the y dimension           */
  double b_rotation,   /* I: Rotation term to compute x'            */
  double d_rotation,   /* I: Rotation term to compute y'            */
  double ulxmap,       /* I: x coordinate for the upper left cell   */
  double ulymap,       /* I: y coordinate for the upper right cell  */
  struct DDR *ddr      /* O: DDR structure                          */
)
{
    double nl_1;            /* # of lines minus 1                     */
    double ns_1;            /* # of samples minus 1                   */
    double ydim_tmp;        /* Holder of cell size in y dimension     */

    nl_1 = ddr->nl - 1;
    ns_1 = ddr->ns - 1;

    /* Making negative for computations
       --------------------------------*/
    ydim_tmp = ydim * -1;

    /* Compute center of cell
       ----------------------*/
    ddr->upleft[0] = d_rotation * 0 + ydim_tmp * 0 + ulymap;
    ddr->upleft[1] = xdim * 0 + b_rotation * 0 + ulxmap;
    ddr->loleft[0] = d_rotation * 0 + ydim_tmp * nl_1 + ulymap;
    ddr->loleft[1] = xdim * 0 + b_rotation * nl_1 + ulxmap;
    ddr->upright[0] = d_rotation * ns_1 + ydim_tmp * 0 + ulymap;
    ddr->upright[1] = xdim * ns_1 + b_rotation * 0 + ulxmap;
    ddr->loright[0] = d_rotation * ns_1 + ydim_tmp * nl_1 + ulymap;
    ddr->loright[1] = xdim * ns_1 + b_rotation * nl_1 + ulxmap;

    ddr->valid[DDCCV] = VALID;

    /* Setting the pixel size
       ----------------------*/
    ddr->pdist_x = xdim;
    ddr->pdist_y = ydim;

    ddr->valid[DDPDV] = VALID;
}
