/*******************************************************************************
NAME: GETPAR

PURPOSE: gets the user-specified parameters and tae globals for arc2las

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ----------------------------------------
  1.0    12/93  K. Lowell          C   Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "tae.h"
#include "vll.h"
#include "arc2las.h"

void getpar
(
  struct PARBLK *vblk,          /* I: TAE parameter block               */
  char outnam[CMLEN],           /* O: Output image name                 */
  long *msgtim,                 /* O: Message processing time           */
  long *ep3flag,                /* O: Arc 8.3/NO8.3 nameing conv. flag  */
  char *workspace,              /* O: Directory holding ARC information */
  char *geo_type,               /* O: geo dataset type                  */
  char *ingrid,                 /* O: Name of the input ARC image       */
  int *dtype,                   /* O: LAS data type                     */
  int *nbands                   /* O: number of bands in image          */
)
{
    struct VARIABLE *var;       /* Ptr to a variable in the vblk    */

    char ep3file[CMLEN];        /* 8.3 indicator file               */
    char archome[CMLEN];        /* ARC home directory               */

    /* Get the input image NOTE: 
       getnam not used since there are three
       possible extensions for the input file.
       Extensions are assigned based on on the
       band format of INFILE.
       ---------------------------------------*/
    var = p_find(vblk,"WORKSPACE");
    strcpy(workspace,SVAL(*var,0));

    var = p_find(vblk, "GEO_TYPE");
    strcpy(geo_type,SVAL(*var,0));

    var = p_find(vblk,"INGRID");
    strcpy(ingrid,SVAL(*var,0));

    /* Get the output image
       --------------------*/
    var = p_find(vblk,"OUT");
    strcpy(outnam,SVAL(*var,0));

    /* Get the data type
       -----------------*/
    var = p_find(vblk, "DTYPE");
    *dtype = IVAL(*var,0);

    /* Get the number of bands
       -----------------------*/
    var = p_find(vblk, "NBANDS");
    *nbands = IVAL(*var,0);

    /* Retrieve globals
       ----------------*/
    var = p_find(vblk,"$msgtim");
    *msgtim = IVAL(*var,0);

    /* Determine naming convention, 8.3 or NO8.3
       -----------------------------------------*/
    strcpy(archome,getenv("ARCHOME"));
    strcpy(ep3file,archome);
    strcat(ep3file,"/sysgen/no8.3");
    if ((fopen(ep3file,"r")) == NULL)
    {
        c_lasmsg(FALSE,"arc2las-info","8.3 naming convention in use");
        *ep3flag=1;
    }
    else
    {
        c_lasmsg(FALSE,"arc2las-info","NO8.3 naming convention in use");
        *ep3flag=0;
    }
}
