#ifndef INFODEFS_H
#define INFODEFS_H

/* @(#)infodefs.h	1.7 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/*
        Name: infodefs.h
 Description: infolib INFO file definitions.
     History: 12/91  Todd Stellhorn  - original coding
 */

/***************************************************************************/

/* target OS */

#define UNIX_OS_USED
/* #define _POSIX_SOURCE */
/* #define DOS */

/***************************************************************************/

#ifdef UNIX_OS_USED

#include <sys/param.h>

#define OPSYS_SEP             "/"
#define DIR_EXT             "dr9"
#define DATA_EXT            "dat"
#define NIT_EXT             "nit"

                              /* set to ":" for /bin/csh history character */
#define PATH_DELIMITER        ':'

#define PATH_SIZE      MAXPATHLEN         

#endif  /* UNIX */

/***************************************************************************/

#ifdef _POSIX_SOURCE

#include <limits.h>

#define OPSYS_SEP             "/"
#define DIR_EXT             "dr9"
#define DATA_EXT            "dat"
#define NIT_EXT             "nit"

                              /* set to ":" for /bin/csh history character */
#define PATH_DELIMITER        ':' 

#define PATH_SIZE _POSIX_PATH_MAX         

#endif  /* _POSIX_SOURCE */

/***************************************************************************/

#ifdef DOS

#include <dir.h>

#define OPSYS_SEP            "\\"
#define DIR_EXT            ".DR9"
#define DATA_EXT           ".DAT"
#define NIT_EXT            ".NIT"

#define PATH_DELIMITER        '!' /* set to "!" for DOS drive notation */

#define PATH_SIZE          MAXDIR

#endif  /* DOS */

/***************************************************************************/

#define DEFAULT_USER        "ARC"
#define DEFAULT_PATH       "info"

#define ITEMNAME_SIZE          17
#define INTERNALNAME_SIZE      33
#define USERNAME_SIZE           9

#define TMPRECORD_SIZE       4096
#define TMPSTRING_SIZE        128

#define DirEMPTY                1
#define DirNotEMPTY             0

#define DirDataFileTYPE         0
#define DirProgramTYPE          1
#define DirReportTYPE           2
#define DirSpecialFormTYPE      3
#define DirInputFormTYPE        4

/***************************************************************************/

typedef struct
{
      char          InternalName[32];      /* user file name */
      char          ExternalName[8];       /* INFO file name */
      short int     NumberItems;           /* number of items */
      short int     RecordLength;          /* logical record length */
      char          Level0Password[4];     /* level 0 password */
      char          Level1Password[4];     /* level 1 password */
      char          Level2Password[4];     /* level 2 password */
      char          Level3Password[4];     /* level 3 password */
      short int     PrintWidth;            /* report output width */
      short int     EmptyFlag;             /* directory record empty flag */
      long int      NumberRecords;         /* number of records */
      short int     Filler1;               /* not used */
      short int     FileType;              /* directory entry type */
      short int     Filler2;               /* ? */
      char          Filler3[4];            /* ? */
      char          ExternalFlag[2];       /* external data file flag */
      char          Filler4[300];          /* ? and not used */
} DirRecord;

typedef struct
{
      char          ItemName[16];          /* item name */
      short int     ItemWidth;             /* internal item width */
      short int     KeyType;               /* key type */
      short int     ItemPosition;          /* item position */
      short int     ProtectionLevel;       /* item protection level */
      short int     Filler1;               /* ? */
      short int     OutputWidth;           /* item output width */
      short int     NumberDecimals;        /* number of decimal places */
      short int     ItemType;              /* item type */
      short int     ReadOnlyFlag;          /* read only flag */
      short int     KeyLevel;              /* key level */
      short int     IndexNumber;           /* item index number */
      short int     OccuranceCount;        /* number of occurrences */
      short int     Filler2;               /* ? */
      char          AlternateName[16];     /* item alternate name */
      char          Filler3[16];           /* ? */
      short int     ConcordLevel;          /* concordance level */
      short int     AlternateRule;         /* item naming rule */
      char          Filler4[32];           /* ? */
      short int     ConcordCase;           /* concordance case */
      short int     ConcordNumbers;        /* concordance numbers */
      short int     ItemNumber;            /* logical item number */
      char          Filler5[28];           /* not used */
} NitRecord;

/***************************************************************************/
#endif
/*
      end of infodefs.h
 */
