/* @(#)infolib.h	1.7 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infolib.h
 Description: infolib main header.
     History: 12/91  Todd Stellhorn  - original coding
 */

/***************************************************************************/

#ifndef _INFOLIB_H_

#define _INFOLIB_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "infodefs.h"

typedef struct
{
      char         *FileName;          /* user file name */
      char         *DirFilePath;       /* path to DR9 file */
      char         *NitFilePath;       /* path to FileName NIT file */
      FILE         *InfoFILE;          /* file pointer */
      short int     NumberItems;       /* number of (real) items */
      short int     RecordLength;      /* logical record length */
      long int      NumberRecords;     /* number of records */
      short int     AccessFlag;        /* access flag */
      long int      CurrentRecord;     /* current record number */
      char         *IOBuffer;          /* input/output buffer */
} InfoFile;

typedef struct
{
      short int     ItemPosition;      /* item position */
      short int     ItemWidth;         /* item width */
      short int     ItemType;          /* item type */
      short int     NumberDecimals;    /* number of decimal places */
      short int     OutputWidth;       /* item output width */
      char          ItemName[17];      /* item name */
} InfoItemDef;

typedef InfoItemDef TemplateList;

/***************************************************************************/

#define InfoREAD                1      /* file access flags */
#define InfoWRITE               2
#define InfoINIT                3

#define INFO_DATE_TYPE          1      /* INFO item types */
#define INFO_CHARACTER_TYPE     2
#define INFO_INTEGER_TYPE       3
#define INFO_NUMBER_TYPE        4
#define INFO_BINARY_TYPE        5
#define INFO_FLOATING_TYPE      6
#define INFO_LEADFILL_TYPE      7
#define INFO_PACKED_TYPE        8
#define INFO_ZEROFILL_TYPE      9
#define INFO_OVERPUNCH_TYPE    10
#define INFO_TRAILINGSIGN_TYPE 11
#define INFO_RECNO_TYPE        99

#define INFO_RECNO       "$RECNO"

#define CASE_SENSITIVE          1
#define NONCASE_SENSITIVE       2

/***************************************************************************/

/* infolib functions */

/* user level functions */

/* infofile */

InfoFile *InfoOpenFile
(
      char *InfoFilePath, 
      short int AccessType
);
short int InfoCloseFile
(
      InfoFile *InfoFilePtr
);
short int InfoFileExists
( 
      char *InfoFilePath
);
short int InfoReadRecord
(
      InfoFile *InfoFilePtr, 
      long int RecordNumber
);
short int InfoWriteRecord
(
      InfoFile *InfoFilePtr, 
      long int  RecordNumber
);

/* infoitem */

InfoItemDef *InfoGetItemDef
(
      InfoFile *InfoFilePtr, 
      char     *ItemName
);
short int InfoItemExists
(
      InfoFile *InfoFilePtr, 
      char     *ItemName
);
short int InfoDecode
(
      InfoFile    *InfoFilePtr,
      InfoItemDef *ItemDef,
      char        *StringPtr,
      double      *DoublePtr
);
short int InfoEncode
(
      InfoFile    *InfoFilePtr,
      InfoItemDef *ItemDef,
      char        *StringPtr,
      double       DoubleVal
);

/* infoutil */

short int InfoBinarySearch
(
     InfoFile    *InfoFilePtr,
     InfoItemDef *ItemDef,
     char        *SearchString,
     long int     SearchInt,
     long int    *RecordNumber
);
short int InfoSeqSearch
(
     InfoFile    *InfoFilePtr,
     InfoItemDef *ItemDef,
     char        *SearchString,
     long int     SearchInt,
     long int    *RecordNumber
);
TemplateList *InfoGetTemplate
(
     InfoFile *InfoFilePtr
);

/* support functions */

/* infodir */

short int DirGetRecord
(
    char      *DirFilePath,
    short int  RecordStatus,
    char      *FileName,
    DirRecord *ReturnRecord,
    long int  *Position
);
short int DirWriteRecord
(
      char       *DirFilePath,
      DirRecord  *Record,
      long int    Position
);


/* infoetc */

FILE *OpenFile 
( 
      char  *FileName, 
      short int Access
);
short int CloseFile
(
      FILE *FilePtr
);
short int CompareValues
(
      InfoFile     *InfoFilePtr,
      InfoItemDef  *ItemDef,
      char         *SearchString,
      long int      SearchInt,
      short int    *CompareCode
);
short int GetExternalPath
(
      char *DataFilePath,
      char *InfoFilePath,
      char *ExternalFilePath
);
char *IOBufferAllocate
(
      short int BufferSize
);
void InfoFileFree
(
      InfoFile *InfoFilePtr
);
void ItemDefFree
(
      InfoItemDef *ItemDef
);

/* infonit */

short int NitGetRecord
(
      char      *NitFilePath,
      char      *ItemName,
      NitRecord *ReturnRecord,
      long int  *Position 
);

/* infoprs */

short int ParsePathName
(
      char *InString,
      char *PathName,
      char *User,
      char *FileName
);
char *FetchNextWord
(
      char *StringPtr, 
      char Delimiter, 
      char *NextWord
);

/* infostr */

void TerminateString
(
      char      *Source,
      short int  SourceSize,
      char      *Target
);
void StringtoLower
(
      char *StringBuffer
);
void StringtoUpper
(
      char *StringBuffer
);
void FillStringBuffer
(
      char      *InBuffer,
      short int  Position,
      short int  Length,
      char      *OutBuffer
);
void CompressString
(
      char  *String, 
      short  int  Width
);
void ExpandString
(
      char  *String, 
      short  int Width
);
short int CompareStrings
(
      char *String1,
      char *String2,
      short int CompareRule
);

/***************************************************************************/

#endif  /* _INFOLIB_H_ */

/*
      end of infolib.h
 */
