/* @(#)infonit.c	1.7 6/8/92 */
/*
  Notice:  This software is being released into the public
           domain.  It may be freely used whole or in part or 
           distributed as a whole as long as this notice remains
           attached to the source code.  The author and ESRI 
           do not assume any liability or responsibility in the 
           use or distribution of this software.  Direct any 
           questions or comments via email to the author at 
           tstellhorn@esri.com.
            
                              Todd Stellhorn, SMTS
                              ESRI, Applications Development
 */
/***************************************************************************/
/*
        Name: infonit.c
 Description: Contains infolib NIT file functions.  The following
                functions are contained in this module:

                  NitGetRecord

    History: 12/91  Todd Stellhorn  - original coding
 */
/***************************************************************************/

#include "infolib.h"
#include "infodefs.h"

/***************************************************************************/


/****************************************************************************
       Name: NitGetRecord
    Purpose: This function searches the input NIT file for a record
               matching the input INFO item name.  The matched record
               and its file position are returned.
     Inputs: NitFilePath ( char * ) NIT file path
             ItemName ( char * ) INFO item name
             ReturnRecord ( NitRecord * ) NIT record
             Position ( long int * ) Record position
    Outputs: Function - ( short int ) Returned function status
                        0 == error returning record
                        1 == matching record found
               On success, the matching NIT record and NIT file
               position are returned.  On error, the values for 
               ReturnRecord and Position are undefined.
    History: 12/91  Todd Stellhorn  - original coding
****************************************************************************/
short int NitGetRecord
(
      char      *NitFilePath,
      char      *ItemName,
      NitRecord *ReturnRecord,
      long int  *Position
)
{

      char          TmpItemName[ITEMNAME_SIZE];
      char          TmpAlternateName[ITEMNAME_SIZE];
      FILE         *NitFile = ( FILE * ) NULL;
      NitRecord     Record;

      *Position = 0;

      if(( NitFile = OpenFile( NitFilePath, InfoREAD )) == ( FILE * ) NULL )
      {
           return( 0 );
      }

      while( 1 )
      {
           if( fread( &Record, sizeof( NitRecord ), 1, NitFile ) != 1 )
           {
                CloseFile( NitFile );
                return( 0 );
           }

           if( feof( NitFile ))
           {
                CloseFile( NitFile );
                return( 0 );
           }

           TerminateString( Record.ItemName, sizeof ( TmpItemName ),
                TmpItemName );
           TerminateString( Record.AlternateName, sizeof( TmpAlternateName ),
                TmpAlternateName );

           if(( CompareStrings( TmpItemName, ItemName, NONCASE_SENSITIVE )
                == 0 ) || ( CompareStrings( TmpAlternateName, ItemName,
                NONCASE_SENSITIVE ) == 0 ))
           {
                break;
           }

           *Position = *Position + 1;
      }

      if( ! CloseFile( NitFile ))
      {
           return( 0 );
      }

      memcpy( ReturnRecord, &Record, sizeof( NitRecord ));

      return( 1 );
}                                           /* end of NitGetRecord         */

/*                                             end of infonit.c            */
