/* @(#)lci.c	1.0 4/3/93 */

#include <stdio.h>
#include <string.h>
#include "infolib.h"

/*

Program Name: lci (upper case item)

Purpose:      Convert an Info character item to upper case.

Author:       R.W. Klaver

History:      Initial coding 3 April 1993

Remarks:      This program uses lnfolib subroutines to access
              info files.

              The process is to:
                 see if the info file exists
                 open the file
                 see if the info item exists
                 get the defination of the info item
                 make sure the item is of type character
                 loop through the file
                    reading a record
                    decoding the record
                    converting the item to upper case
                    encoding the item
                    writing the record back out to the file
                 closing the file

              The program was compiled with the following command:
                 
                 cc -o uci uci.c -I/edcsnw22/sab/bklaver/infolib
                       /edcsnw22/sab/bklaver/infolib/infolib.a



*/

void main( argc, argv )
int argc;
char **argv;
{
      char         *InfoFilePath, *Item;
      InfoFile     *FilePtr;
      InfoItemDef  *ItemPtr;
      char          RetString[4096];
      double        RetDouble;
      int           j;
      long int      rec;

      /* check the argument...if not complete give usage and exit */

      if( argc != 3 )  {
         printf("\n   Usage: %s <InfoFile> <Item>.\n\n",argv[0]);
         exit( 1 ); 
      }

      InfoFilePath = argv[1]; 
      Item = argv[2]; 

      /* check for error conditions  ... if error print message and exit */

      if( ! InfoFileExists ( InfoFilePath )) {
         printf("\n   Info File %s does not exist.\n\n", InfoFilePath);
         exit( 1 ); 
      }

      if(( FilePtr = InfoOpenFile( InfoFilePath, InfoWRITE )) 
           == ( InfoFile * ) NULL ) {
        printf("\n   Cannot open File %s with rw access.\n\n", InfoFilePath);
        exit( 1 ); 
       }

      if( ! InfoItemExists( FilePtr, Item )) {
         printf("\n   Item %s does not exist.\n\n", Item);
         exit( 1 ); 
      }

      if(( ItemPtr = InfoGetItemDef( FilePtr, Item )) 
           == ( InfoItemDef * ) NULL ) {
           printf("\n   Cannot get Item Defination for %s.\n\n",Item);
           exit( 1 ); 
      }
  
      if( ItemPtr->ItemType != INFO_CHARACTER_TYPE ) {
          printf("\n   Item %s is not Character.\n\n", Item);
          exit( 1 ); 
      }

      printf("   Number of Records is %d\n", FilePtr->NumberRecords);

      /* now can do the real work 

         loop through the file and convert item to lower case */

      for(rec=1; rec<=FilePtr->NumberRecords; rec++) {

         if( ! InfoReadRecord( FilePtr, rec )) {
            printf("\n   Error Reading record %ld.\n\n", rec);
            exit( 1 ); 
         }

         if( ! InfoDecode( FilePtr, ItemPtr, RetString, &RetDouble )) {
            printf("\n   Error Decoding record %ld.\n\n", rec);
            exit( 1 ); 
         }

/*       for(j=0; RetString[j]; j++) RetString[j] = tolower(RetString[j]); */

         StringtoUpper( RetString );

         if( ! InfoEncode( FilePtr, ItemPtr, RetString, &RetDouble )) {
            printf("\n   Error Encoding record %ld.\n\n", rec);
            exit( 1 ); 
         }

         if( ! InfoWriteRecord( FilePtr, rec )) {
            printf("\n   Error Writing record %ld.\n\n", rec);
            exit( 1 ); 
         }

      }  /* end for loop  */
         
      if( ! InfoCloseFile( FilePtr )) {
         printf("\n   Error Closing File %s.\n\n", InfoFilePath);
         exit( 1 );
      }

      printf("   Successful Completion\n");


      exit( 0 );


}  /* end Main  */
