#!/usr/bin/perl
# This perl script will execute the stackstats GRID command and retrieve
# the smallest minimum pixel value and largest maximum pixel value across
# all layers that compose the grid stack.

$arcstack = $ARGV[0];
$output_file = $ARGV[1];
$tmp_file = "tmp_$output_file";
$minmin = 3.5e38; # greater than max possible if R*4 data
$maxmax = -3.5e38; # less than min possible if R*4 data

system "(arc '&run stack_stats $arcstack') >$tmp_file";

# The GRID stackstats command has written the statistics for the layers that
# compose the grid stack to the $tmp_file file. Now parse this file to
# retrieve the smallest minimum pixel value and largest maximum pixel value
# across the layers.

open(FILE, "<$tmp_file") || die "cannot open stack statistics file for read";
$i = 1;
while(<FILE>) {
    # Skip 10 lines of file header information
    if($i > 10) {
        # Once past the header information, first line beginning in '#'
        # indicates end of layer stats.
        if(/#/) {
            last;
        }
        else {
            chomp;
            s/\s+//; # remove leading spaces
            ($layer, $min, $max, $mean, $std) = split(/\s+/);

            # Compare values and save when appropriate
            if($min < $minmin) { $minmin = $min; }
            if($max > $maxmax) { $maxmax = $max; }
        }
    }
    $i++; # keep track of line numbers to allow to skip header stuff
}
close(FILE);
# Delete statistics file created by GRID stackstats command
unlink $tmp_file || print "could not remove temp file $tmp_file\n";

# Write min and max values out to the file for retrieval by PDF file
open(FILE, ">$output_file") || die "cannot open grid info temp file for write";
print FILE "                 Information retrieved from ARC grid stack:\n";
print FILE "  $arcstack\n\n";
print FILE "MINPIX $minmin\n";
print FILE "MAXPIX $maxmax\n\n";
print FILE "--------------------------------------------------------------------\n";
print FILE "This is a temporary file created and used by arc2las when converting\n";
print FILE "an ARC grid or grid stack to a LAS image. If this file remains after\n";
print FILE "arc2las has completed, it should be manually removed.";
close(FILE);

exit;
