/******************************************************************************

NAME:	ASC2TAB

FUNCTION:  ASC2TAB converts an ascii file to a labeled table file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1           10/87      B. Ailts     original development
    5.0         06/89      B. Ailts	LAS 5.0 conversion
    5.1         07/89      B. Ailts     Upper cased attribute names
    5.2         08/89      B. Ailts     Fixed problem with char data on SUN-4
    5.3         09/89      B. Ailts     Fixed bug retreiving LTTYPE
    5.4		11/91      D. Etrheim	ADP #5733 Corrected to skip character
					columns
    5.5		04/93      D. Etrheim	ADP #6101 Correct problem writing shared
					data field.  PRF #6107  Add ODTYPE
    		09/96      D. Etrheim   Initialize some variables to prevent
					accessing uninitialized memory

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Receive the parameter values
Open the input ascii file and output labeled table file 
For each line of the ascii file
   Read the ASCII file
   Write the correct columns to the labeled table file
Close the files
Display the completion message

******************************************************************************/

#include <stdio.h>

#include "las.h"
#include "ltable.h"
#include "util.h"
#include "asc2tab.h"
#include "protos.h"

int main (void)
{
struct PARBLK parb;			/* TAE parameter block */

static struct TAB_DEF tab[MAXCOL];	/* label table descriptot */

FILE *fp;				/* ascii file descriptor */

static union DATA data[MICOL];			/* data buffer */

long dtype[MICOL];			/* dtype of each col of input file */
long fatal = -1;			/* fatal error code		*/
long filenum[MAXCOL];			/* file number to place input column */
long i;					/* counter--current line */
long incol[MICOL];			/* columns needed from input file */
long incol_cnt;				/* no. of cols needed from input file */
long lun;				/* fortran LUN to standard out */
long odtype[MICOL];			/* output dtype of each col in lt */
long ncol;				/* number of columns */
long nonfatal = 1;			/* nonfatal error code		*/
long nrow;				/* number of rows */
long srow;				/* starting row  */
long status;				/* Status */
long tiecol;				/* Column to place in all subfiles */
long totfiles;				/* total number of output files	*/


char *atrname[MAXCOL];			/* attribute names */
char *desc[MAXCOL];			/* attribute descriptions */
char hosin[CMLEN];			/* host input name */
static char hosout[MAXCOL][CMLEN];	/* host output name */
static char ltdesc[MAXCOL][CMLEN];	/* labeled table description */
static char lttype[MAXCOL][CMLEN];	/* labeled table type */

/*  Initialize TAE
------------------*/
c_inlas(&parb,&lun);

/*  Get parameter values
------------------------*/
getpar(&parb,hosin,hosout,&incol_cnt,atrname,dtype,desc,incol,&nrow,
       lttype,ltdesc,&ncol,filenum,&totfiles,&srow,&tiecol,odtype);

/*  Open ascii input file
-------------------------*/
fp = fopen(hosin,"r");			/* Open the ASCII file          */
if (fp == NULL)
   {
   c_errmsg("Error opening input file","asc2tab-open",&nonfatal);
   c_errmsg("Fatal error encountered","asc2tab-fatal",&fatal);
   }

/*  open output labeled table file
----------------------------------*/
labtabop(tab,&incol_cnt,hosout,odtype,desc,atrname,lttype,ltdesc,
         filenum,&totfiles,&tiecol,incol);

/*  Read one line(row) at a time and place in labeled table file
----------------------------------------------------------------*/
status = 1;
for ( i = 0; i < srow; i ++)
   status = readrow(fp,data,&ncol,dtype,incol,odtype);

i = 0;
while ((status != EOF) && (i != nrow))
   {
   writerow(tab,incol,&incol_cnt,odtype,data,filenum,&totfiles,&tiecol,&i);
   status = readrow(fp,data,&ncol,dtype,incol,odtype);
   if (status == 0)
      {
      c_errmsg("Error encountered reading input file","asc2tab-read",&nonfatal);
      for (i = 0; i < totfiles; i++)
         close_tab(&tab[i]);
      c_clean(&parb,"asc2tab",hosout,&totfiles); 
      }
   i = i + 1;
   }

/*  Close the input and output files.
-------------------------------------*/
fclose(fp);

for (i = 0; i < totfiles; i++)
   close_tab(&tab[i]);

/*  Ouptut completion message
-----------------------------*/
c_complt("ASC2TAB",hosout,&totfiles);

return 0;
}
