/******************************************************************************

NAME:	GETPAR

FUNCTION:  GETPAR retrieves the TAE parameter values

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Retreive the output file name
Retreive the input file name
Retreive the the other parameters
Check to see if the number of specified values are equal

******************************************************************************/

#include <string.h>
#include "las.h"
#include "ltable.h"
#include "util.h"
#include "asc2tab.h"
#include "protos.h"

void getpar
(
    struct PARBLK *parb,			/* parameter block structure */
    char *hosin,				/* host input file name descriptor */
    char hosout[][CMLEN],			/* host output file name descriptor */
    long *incol_cnt,			/* input column count */
    char *atrname[],			/* attribute name for each output col */
    long *dtype,				/* input data type for each column */
    char *desc[],				/* descrption for each output column */
    long *incol,				/* input columns needed */
    long *nrow,				/* number of rows needed */
    char lttype[][CMLEN],			/* labeled table type */
    char ltdesc[][CMLEN],			/* labeled table descrption */
    long *ncol,				/* number of total columns */
    long *filenum,				/* file numbers to place columns in */
    long *totfiles,				/* total number of output files	*/
    long *srow,				/* starting row			*/
    long *tiecol,				/* column to place in all files	*/
    long *odtype                           /* output dtype of each col in lt */
)

{ /* getpar */

struct VARIABLE *val;			/* pointer to variable */

long access;				/* type of access of file */
long atrname_cnt;			/* no. of attribute names */
long desc_cnt;				/* no. of attribute descriptions */
long dtype_cnt;				/* no. of data types */
long fatal = -1;			/* fatal error code		*/
long i;					/* counter--current parameter value */
long nbytes;				/* no. of bytes in each attr. name */
long nonfatal = 1;			/* nonfatal error code		*/
long nrow_cnt;				/* number of rows needed */
long odtype_cnt;			/* no. of output data types */
long status;				/* status of function calls	    */
long totfilnum;				/* total no. of values specifed for
					the parameter FILENUM		*/
long zero = 0;				/* value of zero		*/


/* Retrieve output labeled table file name.
-------------------------------------------*/
access = IWRITE;
status = c_getnam(parb,"OUTLT",&zero,hosout[0],"lt",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","asc2tab-fatal",&fatal);

/*  Retrieve input ascii file name.
-----------------------------------*/
access = IREAD;
status = c_getnam(parb,"INFILE",&zero,hosin,"txt",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","asc2tab-fatal",&fatal);

/* Initialize some variables
----------------------------*/
for (i = 0; i < MICOL; i++)
    dtype[i] = incol[i] = odtype[i] = 0;

/*  Retreive parameter values.
------------------------------*/
val = p_find(parb,"LTTYPE");
for (i = 0; i < val->v_count; i++)
   {
   strcpy (lttype[i],SVAL(*val,i));
   nbytes = (long) strlen(lttype[i]);
   c_low2up(lttype[i],&nbytes);
   }

val = p_find(parb,"COMMENT");
for (i = 0; i < val->v_count; i++)
   strcpy (ltdesc[i],SVAL(*val,i));

/*  Retrieve and upper case the attribute names
-----------------------------------------------*/
val = p_find(parb,"ATRNAME");
atrname_cnt = (*val).v_count;
for (i = 0; i < atrname_cnt; i++)
   {
   atrname[i] = SVAL(*val,i);
   nbytes = (long) strlen(atrname[i]);
   c_low2up(atrname[i],&nbytes);
   }

*totfiles = 0;
val = p_find(parb,"FILENUM");
totfilnum = (*val).v_count;
for (i = 0; i < totfilnum; i++)
   {
   filenum[i] = IVAL(*val,i) - 1;
   *totfiles = LASMAX(*totfiles,filenum[i] + 1);
   }

val = p_find(parb,"COLNUM");
if (val->v_count > 0)
   *tiecol = IVAL(*val,0) - 1;
else
   *tiecol = MICOL + 1;

/*  Create the host file names for all the subfiles needed
----------------------------------------------------------*/
for (i = 1; i < *totfiles; i++)
   strcpy(hosout[i],make_lt_name(hosout[0],i));

val = p_find(parb,"IDTYPE");
dtype_cnt = (*val).v_count;
for (i = 0; i < dtype_cnt; i++)
   {
   if (strncmp(SVAL(*val,i),"BYTE",3) == 0)
      *(dtype + i) = EBYTE;
   else if (strncmp(SVAL(*val,i),"I*2",3) == 0)
      *(dtype + i) = EWORD;
   else if (strncmp(SVAL(*val,i),"I*4",3) == 0)
      *(dtype + i) = ELONG;
   else if (strncmp(SVAL(*val,i),"R*4",3) == 0)
      *(dtype + i) = EREAL;
   else if (strncmp(SVAL(*val,i),"R*8",3) == 0)
      *(dtype + i) = EDOUBLE;
   else if (strncmp(SVAL(*val,i),"CHAR",4) == 0)
      *(dtype + i) = C;
   }

val = p_find(parb,"ODTYPE");
odtype_cnt = (*val).v_count;
if (odtype_cnt == 0)
   {
   for (i = 0; i < dtype_cnt; i++)
      *(odtype + i) = *(dtype + i);
   }
else
   {
   if (odtype_cnt != dtype_cnt)
      {
      c_errmsg("IDTYPE and ODTYPE are not equal","asc2tab-dtype",&nonfatal);
      c_errmsg("Fatal error encountered","asc2tab-fatal",&fatal);
      }
   for (i = 0; i < odtype_cnt; i++)
      {
      if (strncmp(SVAL(*val,i),"BYTE",3) == 0)
         *(odtype + i) = EBYTE;
      else if (strncmp(SVAL(*val,i),"I*2",3) == 0)
         *(odtype + i) = EWORD;
      else if (strncmp(SVAL(*val,i),"I*4",3) == 0)
         *(odtype + i) = ELONG;
      else if (strncmp(SVAL(*val,i),"R*4",3) == 0)
         *(odtype + i) = EREAL;
      else if (strncmp(SVAL(*val,i),"R*8",3) == 0)
         *(odtype + i) = EDOUBLE;
      else if (strncmp(SVAL(*val,i),"CHAR",4) == 0)
         *(odtype + i) = C;
      }
   }

val = p_find(parb,"ATRDESC");
desc_cnt = (*val).v_count;
for (i = 0; i < desc_cnt; i++)
   desc[i] = SVAL(*val,i);

val = p_find(parb,"INCOL");
*incol_cnt = (*val).v_count;
for (i = 0; i < *incol_cnt; i++)
   *(incol + i) = IVAL(*val,i) - 1;

val = p_find(parb,"NCOLS");
*ncol = IVAL(*val,0);

val = p_find(parb,"SROW");
*srow = IVAL(*val,0);

val = p_find(parb,"NROWS");
nrow_cnt = (*val).v_count;
*nrow = IVAL(*val,0);

/*  Create correct column array.
-------------------------------*/
if (*incol_cnt == 0)
   {
   *incol_cnt = *ncol;
   for (i = 0; i < *ncol; i++)
      *(incol + i) = i;
   }

/*  Check the count of INCOL, ATRNAME, DTYPE, and DESC
------------------------------------------------------*/
if ((*incol_cnt != atrname_cnt) || (*incol_cnt != dtype_cnt) ||
    (*incol_cnt != desc_cnt) || (*incol_cnt != totfilnum))
   {
   c_errmsg( "ATRNAME, ATRDESC, IDTYPE, FILENUM, and INCOL are not equal"
         ,"asc2tab-equal",&nonfatal);
   c_errmsg("Fatal error encountered","asc2tab-fatal",&fatal);
   }

/* All rows to be proccessed
----------------------------*/
if (nrow_cnt == 0)
   *nrow = 999999;

return;
} /* getpar */
