/******************************************************************************

NAME:	LABTOPOP

FUNCTION:  LABTOPOP opens the output labeled table

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Write vector data for each column needed
Initialize variables needed to open the output labeled table file.
Open the output labeled table file
For each column needed
   Write vector data

******************************************************************************/

#include "las.h"
#include "ltable.h"
#include "asc2tab.h"
#include "protos.h"

void labtabop
(
    struct TAB_DEF *tab,		/* table descriptor */
    long *incol_cnt,		/* number of output columns needed */
    char hosout[][CMLEN],		/* host output file name */
    long *odtype,			/* data type of each output column */
    char *desc[],			/* attribute descrptions of each column */
    char *atrname[],		/* attribute names of each column */
    char lttype[][CMLEN],		/* type of labeled table */
    char ltdesc[][CMLEN],		/* descrption of labeled table */
    long *filenum,			/* file number to place each column */
    long *totfiles,			/* total number of files		*/
    long *tiecol,			/* no. of column to be placed in every file */
    long *incol			/* column numbers needed		*/
)

{ /* labtabop */

long i;				/* counter -- current column*/
long j;				/* counter -- current file		*/

short record[MAXCOL];		/* logical specifying when a record seperator
				   should be used			*/
char sep[MAXCOL];		/* field seperator */

/*  Determine the seperator to be used
--------------------------------------*/
for (i=0; i < *incol_cnt; i++)
   record[filenum[i]] = TRUE;

for (i = *incol_cnt - 1; i >= 0; i--)
   {
   if (record[filenum[i]])
      {
      sep[i] = RECORD_SEP;
      record[filenum[i]] = FALSE;
      }
   else
      sep[i] = FIELD_SEP2;
   }

/*  Initialize variables needed to open the output labeled table file.
----------------------------------------------------------------------*/
tab[0].nsubs = *totfiles - 1;
tab[0].ftype = lttype[0];
tab[0].fdesc = ltdesc[0];
for (i = 1; i < *totfiles; i++)
   {
   tab[0].subfile[i - 1] = hosout[i];
   tab[i].subfile[0] = "\0";
   tab[i].ftype = lttype[i];
   tab[i].fdesc = ltdesc[i];
   }

/*  Open the output labeled table file
--------------------------------------*/
open_tab(&tab[0],hosout[0],IWRITE);
for (i = 1; i < *totfiles; i++)
   open_tab(&tab[i],hosout[i],IWRITE);

/* Write vector data for each column needed
-------------------------------------------*/
for (i=0; i < *incol_cnt; i++)
   {
   if (incol[i] != *tiecol)
      setvec(tab,i,odtype,desc,atrname,filenum[i],sep);
   else
      for (j = 0; j < *totfiles; j++)
         setvec(tab,i,odtype,desc,atrname,j,sep);
   } /* for i */

return;
} /* labtabop */
