/******************************************************************************

NAME:	WRITEROW

FUNCTION:  WRITEROW writes a row to the output labeled table file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Initialize the needed variables
Write each column according the the data type of each column
For each input column needed
   If colunm number does not equal tiecol
      write the column to the correct file
   Else
      Check to see if the value is the same as the previous value
      If different write the value to the main labeled table file
      Write the value to all the subfiles
   Endif
Write the records to the labled table file

******************************************************************************/
#include <string.h>
#include "las.h"
#include "ltable.h"
#include "asc2tab.h"
#include "protos.h"

void writerow
(
    struct TAB_DEF *tab,			/* output labeled table descriptor */
    long *incol,				/* output columns needed */
    long *incol_cnt,			/* number of output columns needed */
    long *odtype,				/* output dtype of columns needed */
    union DATA *data,			/* data buffer */
    long *filenum,				/* file number of column	*/
    long *totfiles,				/* total number of files	*/
    long *tiecol,				/* column to be placed in all files */
    long *row				/* current row			*/
)

{ /* writerow */

struct VECTOR *ptr[MAXCOL];		/* pointer to current vector */

static union DATA save_data;		/* data of previous tie column	*/

long i;					/* counter -- current column */
long j;					/* counter -- current file	*/

short write_flag;			/*  write flag			*/
short write_record = FALSE;		/*  write record flag		*/

/* initialize the needed variables
----------------------------------*/
for (i = 0; i < *totfiles; i++)
   ptr[i] = tab[i].vector;

/* write each column according the the data type of each column
---------------------------------------------------------------*/
for (i = 0; i < *incol_cnt; i++)
   { /* for i */
   if (incol[i] != *tiecol)
      {
      writevec(ptr,incol,odtype,data,filenum[i],i);
      ptr[filenum[i]] = ptr[filenum[i]]->next;
      if (filenum[i] == 0)
         write_record = TRUE;
      }
   else

      /* This assures that only unique data is written to the main labeled
         table file for the column of data that is written to all files
      --------------------------------------------------------------------*/
      {
      write_flag = FALSE;
      switch (*(odtype + *tiecol))
         { /* switch odtype */ 
         case (C): 
                   if ((strcpy(data[*tiecol].chr,save_data.chr) == 0) || 
		       (*row == 0))
		      write_flag = TRUE;
                   break;
         case (I1): 
                   if ((data[*tiecol].byte != save_data.byte) || (*row == 0))
		      write_flag = TRUE;
                   break;
         case (I2): 
                   if ((data[*tiecol].i2 != save_data.i2) || (*row == 0))
		      write_flag = TRUE;
                   break;
         case (I4): 
                   if ((data[*tiecol].i4 != save_data.i4) || (*row == 0))
		      write_flag = TRUE;
                   break;
         case (R4): 
                   if ((data[*tiecol].r4 != save_data.r4) || (*row == 0))
		      write_flag = TRUE;
                   break;
         case (R8): 
                   if ((data[*tiecol].r8 != save_data.r8) || (*row == 0))
		      write_flag = TRUE;
                   break;
         }   /*  switch odtype  tiecol  */

      if (write_flag)
         {
         writevec(ptr,incol,odtype,data,0,i);
         save_data = data[*tiecol];
	 write_record = TRUE;
         ptr[filenum[i]] = ptr[filenum[i]]->next;
         }

      for (j = 1; j < *totfiles; j++)
         {
         writevec(ptr,incol,odtype,data,j,i);
         ptr[j] = ptr[j]->next;
         }
      } /* else */
   } /* for i */

if (write_record)
   put_record(&tab[0]);

for (i = 1; i < *totfiles; i ++)
   put_record(&tab[i]);

return;
} /* writerow */
