#ifndef ASCII2DDR_H
#define ASCII2DDR_H

#include "las.h"

#define PAGE_WIDTH   82         /* ASCII DDR file line width - DSPDDR will
                                   create the file using 80 chars/line but
                                   we need one more char for the '\0'        */

/* funtional prototypes */

void cleanup_files
(
   char out_file[CMLEN]           /* I: name of the output DDR file          */
);

int get_corn_coords
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int get_datum_code
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int get_inc
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int get_mast_coords
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int get_par
(
   struct PARBLK *vblock,        /* I: TAE parameter block                   */
   char   in_hpath[],            /* O: Host path to input ASCII DDR file     */
   char   out_hpath[],           /* O: Host path to output binary DDR file   */
   char   system[],              /* O: Computer system data is on from
                                       value of global $DATASYS              */
   int    *delflg                /* O: Value of env variable $DELFLG         */
);

int get_proj_code
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int get_proj_dist
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int get_proj_parms
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int get_proj_unit
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int get_zone_code
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int parse_ascddr
(
   const char  hpath[CMLEN], /* I: Host path to input DDR file to parse      */
   const char  system[12],   /* I: Computer system data is on from value of
                                   global $DATASYS variable                  */
   struct DDR  *ddr,         /* O: Output DDR structure with DDR values
                                   retrieved from the ASCII DDR file         */
   struct BDDR **bddrs       /* O: Output BDDR structure with Band DDR
                                   values retrieved from the ASCII DDR file  */
);

int det_val_flag
(
   char val_flg_str[13],      /* I: Validity flag (string) */
   long *val_flg_int          /* O: Validity flag (integer) */
);

int read_bddrs
(
   FILE   *fp,                /* I: File pointer for the ASCII DDR file      */
   struct DDR *ddr,           /* I/O: DDR structure - filled in by
                                    read_criticals and read_noncriticals
                                    prior to this routine, though its nbands
                                    value may need to be modified            */
   struct BDDR **bddrs        /* O: Output BDDR structure with BDDR values
                                    retrieved from the ASCII DDR file        */
);

int read_criticals
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

int read_line
(
   FILE *fp,               /* I: File pointer for the ASCII DDR file         */
   char *line_buf          /* O: Buffer to contain line read from file       */
);

int read_noncriticals
(
   FILE   *fp,              /* I: File pointer for the ASCII DDR file        */
   struct DDR *ddr          /* O: Output DDR structure with DDR values
                                  retrieved from the ASCII DDR file          */
);

void signal_cleanup ();

int skip_line
(
   FILE       *fp,               /* I: File pointer for the ASCII DDR file   */
   const int  lines_to_skip      /* I: Number of lines to skip               */
);

#endif
