/******************************************************************************
NAME:        CLEANUP_FILES

PURPOSE:     Remove the outfile DDR that was created by this application, if
             $DELFLG was set.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
$Log: cleanup_files.c,v $
Revision 1.7  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.6.2.1  2001/11/21 19:52:48  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.6  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "diskio.h"
#include "ascii2ddr.h"

extern int g_delflg;   /* Value of env variable $DELFLG                      */

void cleanup_files
(
    char out_file[CMLEN]          /* I: name of the output DDR file          */
)
{
    /* Delete the output DDR file that was created by this application, only
       if $DELFLG was set. Don't care about return status.
    ------------------------------------------------------------------------*/
    if(!g_delflg)
        return;
    else
    {
        c_lasmsg(FALSE, "ascii2ddr-delflg", "$DELFLG is set; deleting output"
                " file created by this application");
        c_dkdel(out_file);
    }
}
