/******************************************************************************
NAME:        READ_LINE

PURPOSE:     Read next line of the ASCII DDR file.

RETURN VALUE:
Type = int
Value         Description
-----         -----------
E_SUCC        Successful completion
E_FAIL        An error occurred during processing

PROGRAM HISTORY:
$Log: read_line.c,v $
Revision 1.7  2001/11/28 21:06:40  shimitz
Implemented changes from peer review and testing. This is formally the
initial released version of the module. CR 2841

Revision 1.6.2.1  2001/11/21 19:52:49  shimitz
Reformatted the files according to the programming standard. Also modified
get_par to retrieve and verify the INFILE and OUT parameters. Made some other
minor changes requested by the user during acceptance testing.

Revision 1.6  2001/06/22 16:21:49  shimitz
Made changes suggested in code peer review session.


PROJECT:     LAS
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ascii2ddr.h"

int read_line
(
    FILE *fp,              /* I: File pointer for the ASCII DDR file         */
    char *line_buf         /* O: Buffer to contain line read from file       */
)
{
    /* Use fgets to retrieve the next line
       -----------------------------------*/
    if(fgets(line_buf, PAGE_WIDTH, fp) == NULL)
    {
        /* Check to see if the NULL was caused by an EOF being
           encountered (!= 0).
           ---------------------------------------------------*/
        if(feof(fp) != 0) /* EOF reached */
        {
            c_lasmsg(FALSE, "ascii2ddr-eof",
                    "Unexpected EOF reached while parsing ASCII DDR file");
        }
        else /* general fgets failure */
        {
            c_lasmsg(FALSE, "ascii2ddr-read",
                    "Error reading from ASCII DDR file");
        }
        return(E_FAIL);
    }

    return(E_SUCC);
}
