#ifndef ASTERHDF2LAS_PROTOS_H
#define ASTERHDF2LAS_PROTOS_H

#include "las.h"

int get_par
(
    struct PARBLK *vblock,    /* I: TAE parameter block                      */
    char infile[],            /* O: Input HDF filename in host format        */
    char outfiles[][CMLEN],   /* O: Output LAS filename(s) in host format    */
    int *num_outs,            /* O: Count of outfiles                        */
    char bands[][3],          /* O: Bands requested by user                  */
    int *num_bands,           /* O: Count of bands                           */
    int band_groupings[],     /* O: Grouping of bands requested by the user  */
    int *num_band_groupings,  /* O: Count of band_groupings                  */
    double window[],          /* O: Window values requested by user          */
    int *num_window,          /* O: Count of window values                   */
    char units[],             /* O: Value from units field (DEG,DMS,PRO,LS)  */
    int *delflg,              /* O: Flag for global value DELFLG (TRUE to
                                    remove intermediate files or FALSE)      */
    long *msgtim,             /* O: Time between display messages            */
    int *process_all_bands,   /* O: Flag indicating if all band in file should
                                    be processed (TRUE or FALSE)             */
    int *window_specified     /* O: Flag indicating if the window field was
                                    specified                                */
);

int nonexisting_bands
(
    char nonex_bands[][3],      /* I: Bands requested that do not exist in
                                      in the HDF file                        */
    int num_nonex_bands,        /* I: Count of nonexisting bands             */
    char bands[][3],            /* I/O: Bands to be processed                */
    int *num_bands,             /* I/O: Count of bands                       */
    int band_groupings[],       /* I/O: The grouping of bands. Each band
                                        grouping represents one LAS image    */
    int *num_band_groupings,    /* I/O: Count of band_groupings              */
    char outfiles[][CMLEN],     /* I/O: Output LAS filename(s)               */
    int *num_outs               /* I/O: Number of output files               */
);

void sort_bands_in_band_groupings
(
    char bands[][3],          /* I/O: Adjusted band values that have
                                      been sorted                            */
    int band_groupings[],     /* I: Band grouping values                     */
    int num_bands,            /* I: Count of adjusted bands                  */
    int num_band_groupings    /* I: Count of adjusted band groupings         */
);

int get_l1a_l1b_metadata
(
    int fileid,               /* I: File id for the file containing
                                    parameter data                           */
    int product_level,        /* I: Flag values can be Level 1A=L1A, or
                                    Level 1B =L1B                            */
    char band_sensor[],       /* I: Indicates if band is VNIR, SWIR or TIR   */ 
    char projectionparameters[],/* I: The tag name that is to be retrieved
                                      ie PROJECTIONPARAMETERS1               */
    char mpmethod[],          /* I: The tag name that is to be retrieved
                                    ie MPMETHOD1                             */
    char *instrument_name,    /* O: Instrument name ie ASTER                 */
    char *projection_name,    /* O: Names in metadata will be either UTM,
                                    LAMCC, PS, EQRECT, SOM                   */
    double projection_values[],/* O: The 15 projection parameters            */
    int ddr_valids[]           /* O: Flags indicating if fields in the DDR
                                    are valid or not (VALID or INVAL)        */
);

int get_dem_metadata
(
    int fileid,               /* I: File id for the file containing
                                    parameter data                           */
    char *instrument_name     /* O: Instrument name ie ASTER                 */
);
int get_l2_metadata
(
    int fileid,               /* I: File id for the file containing
                                    parameter data                           */
    char projectionparameters[],/*I: The tag name that is to be retrieved
                                    ie PROJECTIONPARAMETERS                  */
    char mpmethod[],          /* I: The tag name that is to be retrieved
                                    ie MPMETHOD1                             */
    char *instrument_name,    /* O: Instrument name ie ASTER                 */
    char *projection_name,    /* O: Names in metadata will either be UTM, PS
                                    or EQRECT, SOM                           */
    double projection_values[],/* O: The 15 projection parameters            */
    int ddr_valids[]           /* O: Flags indicating if fields in the DDR
                                     are valid or not (VALID or INVAL)       */
);

int convert_proj_parms_coordinates
(
    double corner_coordinates[], /* I/O: Input lat/lon in degrees, output
                                         y/x in meters                      */
    double projection_values[],  /* I/O: Input angle values in radians output
                                         in degrees/minutes/seconds for all
                                         projections except geographic      */
    int projection_code,         /* I: Projection of the scene              */
    long zone                    /* I: Zone number for UTM                  */
);

int final_output_names
(
    char outfiles[][CMLEN],     /* I/O: Output LAS file names                */
    int *num_outs,              /* I/O: Count of outfiles                    */
    int band_groupings[],       /* I/O: The grouping of bands. Each band
                                        grouping represents one LAS image    */
    char bands[][3],            /* I: Bands to be processed                  */
    int num_bands,              /* I: Count of bands                         */
    int proj_distance[],        /* I/O: Resolution of each band that is to 
                                        be processed                         */
    int *num_band_groupings     /* I/O: Count of band_groupings              */
); 

int convert_verify_window
(
    int window_specified,     /* I: Flag indicating if the window field was
                                    specified (TRUE or FALSE)                */
    double window[],          /* I: Window values requested by user
                                    Image window (If window_units is LS then
                                    sl, ss, nl, ns.  If window_units is DMS or
                                    DEG then UL_lat, UL_long, LR_lat, LR_long.
                                    If window_units is PRO then UL_Y, UL_X,
                                    LR_Y, LR_X.)                             */
    char window_units[],      /* I: Value from units field (LS,DEG,DMS,PRO)  */
    int num_window,           /* I: Count of window values                   */
    int nl,                   /* I: Total number of lines in the image       */
    int ns,                   /* I: Total number of samples in the image     */
    int projection_code,      /* I: Value will be either TM=9, LAMCC=4
                                    PS=6, EQRECT=17, SOM=22, GEO=0           */
    double projection_values[],/* I: Projection parameters of the scene      */
    double corner_coordinates[], /* I: Coordinates of the scene, no windowing
                                       is applied. Value is degrees or meters*/
    int projection_distance,  /* I: Resolution of the image 15,30 or 90 meters*/

    int product_level,        /* I: Values can be Level 1A=L1A, Level 1B=L1B
                                    Level 2 =L2 or Level 3 DEM =L3           */
    long zone,                /* I: UTM zone code                            */
    long valid_window[]       /* O: Window values in line/sample             */
);

int update_ddr
(
    char outfile[],              /* I: LAS file name in host format          */
    long window[],               /* I: Window values in lines/samples        */
    char bands[][3],             /* I: Bands in the LAS image                */
    int band_groupings,          /* I: Number of bands in the LAS image      */
    int data_type,               /* I: Data type of the LAS image
                                       value is either EBYTE or EWORD        */
    int projection_code,         /* I: Value will be either TM=9, LAMCC=4,
                                       PS=6, EQRECT=17, SOM=22, GEO=0        */
    double projection_parameters[], /* I: The 15 parameters in the scene     */
    int window_specified,        /* I: Flag indicating if the window field
                                       was specified (TRUE or FALSE)         */
    double corner_coordinates[], /* I: Coordinates of the scene,windowing isn't
                                       applied. Values are degrees or meters */
    int projection_distance,     /* I: Resolution of the image               */
    char projection_units[],     /* I: Units in degrees or meters            */
    int ddr_valids[],            /* I: Flags indicating if fields in the DDR
                                       are valid or not (0=not valid,1=valid)*/
    char instrument_name[],      /* I: Instrument name ie ASTER              */
    char acquisition_date[],     /* I: Capture date                          */
    char acquisition_time[],     /* I: Capture time                          */
    int orig_nl,                 /* I: Total lines in the image (no windowing
                                       applied)                              */
    int orig_ns,                 /* I: Total samples in the image (no windowing
                                       applied)                              */
    int product_level,           /* I: Values can be Level 1A=L1A, Level 1B=L1B,
                                       Level 2 =L2 or Level 3 DEM =L3        */
    char kinetic_temp[],         /* I: Flag indicating if product is kinetic
                                       temperature                           */
    long utm_zone                /* I: Zone code for UTM projection          */
);

int create_filetable
(
    char filename[],                 /* I: filetable.temp and the current
                                           working directory path            */
    char infile[],                   /* I: Input HDF filename                */
    char working_directory[]         /* I: Current working directory where
                                           the temporary files will be stored*/
);

int recalc_corners
(
    double corners[],         /* I/O: corner points of the image          */
    long nl,                  /* I: original size of the image NL         */
    long ns,                  /* I: original size of the image NS         */
    long window[]             /* I: image window specified                */
);

int cleanup_files
(
    int *num_procd_files,      /* I: number of processed files              */
    char procd_files[][CMLEN], /* I: names of the files that have been 
                                     created by this application            */
    int delflg                 /* I: flag to represent the global DELFLG 
                                     flag (TRUE or FALSE)                   */
);

int delete_image
(
    char img_file[]       /* I: image file to be deleted(with .img extension)*/
);

int latlon2yx
(
    double projection_parameters[], /* I: Projection values as seen in
                                          Projection Transformation Package */
    int projection_code,            /* I: Values can be TM, LAMCC, PS,
                                          EQRECT or SOM                     */
    double lat_lon_coordinates[],   /* I: Corner coordinates in latitude
                                          and longitude                     */
    long zone,                      /* I: UTM zone code                     */
    double y_x_coordinates[]        /* O: Y X corner coordinates in meters  */
);

void signal_cleanup ();

#endif
