/****************************************************************************
NAME:           cleanup_files

PURPOSE:        Cleanup/delete all the image and ddr files that were created
                by this application, if the $DELFLG was set.  If $DELFLG was
                not set then do nothing.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "asterhdf2las_protos.h"

int cleanup_files
(
    int *num_procd_files,      /* I: number of processed files            */
    char procd_files[][CMLEN], /* I: names of the files that have been 
                                     created by this application          */
    int delflg                 /* I: flag to represent the global DELFLG 
                                     flag (TRUE or FALSE)                 */
)

{
    int i;                     /* looping variable                        */
    int num_files;             /* number of files to be deleted           */

    /* If $DELFLG was not set then don't do anything
    ------------------------------------------------*/
    if (delflg == FALSE) 
        return (E_SUCC);

    /* Delete the files that were created by this application, only if
       $DELFLG was set. If an error occurs, then set a flag and continue
       deleting the rest of the files. Loop from the end of the list and
       decrement the num_procd_files counter in case an interrupt occurs
       in the middle of deleting these files.  The filename should be set
       to NULL after the file is deleted.
    -------------------------------------------------------------------*/
    c_lasmsg (FALSE, "asterhdf2las-info", "$DELFLG is set, therefore "
              "deleting all output files created by this application");
    num_files = *num_procd_files;
    for (i = num_files -1; i >= 0; i--)
    {
        if (delete_image (procd_files[i]) != E_SUCC)
        {
            c_lasmsg (FALSE, "asterhdf2las-del",
                      "Error deleting the file: %s", procd_files[i]);
            /* Set the filename to NULL and decrement the num_procd_files 
            counter  */
            strcpy (procd_files[i], "");
            (*num_procd_files)--;
            return (E_FAIL);
        }
    }
    return (E_SUCC);
}
