/******************************************************************************
NAME:           create_filetable

PURPOSE:        This function opens filetable.temp and assigns a logical
                id for the input HDF file, and the intermediate GetAttrtemp 
                and LogStatus files. 
               
RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS

NOTE: The LogStatus and GetAttrtemp files are required for the SDP Toolkit.
      The id's 10100 (for LogStatus) and 10252 (for GetAttrtemp) are defined
      in the SDP Toolkit and SHOULD NOT be changed. The id for the HDF file 
      can be changed but shouldn't be the same as the LogStatus or GetAttrtemp
      id's.
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "asterhdf2las_protos.h"

int create_filetable
(
    char filename[],                 /* I: filetable.temp and the current
                                           working directory path            */
    char infile[],                   /* I/O: Input HDF filename              */
    char working_directory[]         /* I: Current working directory where 
                                           the temporary files will be stored*/
)

{
    FILE *file_ptr;                  /* Pointer used to point to filename    */ 
    char *infile_ptr;                /* Pointer used to point to infile      */ 
    char file_table_contents[CMLEN*3]; /* Buffer used for filetable.temp  
                                          information                        */
    char hdf_file_name[CMLEN];       /* HDF file name (w/o directory path)   */ 
    int len;                         /* Variable used to hold the length of  
                                        a string                             */
    int len_of_file_table_contents;  /* Total buffer size of file contents   */


    /* Write the LogStatus, GetAttrtemp and the HDF file to filetable.temp
       along with the logical id's 10100, 10252, 10308.
    ----------------------------------------------------------------------*/
    sprintf(file_table_contents,
           "10100|LogStatus|%s/LogStatus\n10252|GetAttrtemp|%s/"
           "GetAttrtemp\n10308|",working_directory,working_directory);

    /* Get file name only (excluding directory path) */
    infile_ptr = (char *)strrchr(infile,'/');

    if (infile_ptr == NULL)
    {
        /* There isn't a directory path so copy just the infile */
        strcpy(hdf_file_name,infile);

        /* Add the current working directory to the file name. The full
           directory path and file name is needed for filetable.temp */
        strcpy(infile, working_directory);
        strcat(infile, "/");
        strcat(infile, hdf_file_name);
    }
    else
    {
        /* There is a directory path. Point to the file name only */
        strcpy(hdf_file_name,infile_ptr+1);
    }
    strcat(file_table_contents,hdf_file_name);

    strcat(file_table_contents,"|");

    len = strlen(infile);
    strncat(file_table_contents,infile,len);

    /* To indicate the end of the table, add the following two lines
    ---------------------------------------------------------------*/
    strcat(file_table_contents,"\n0|DUMMY|DUMMY\n\?\n");

    /* Get length of the file table contents
    ---------------------------------------*/
    len_of_file_table_contents = strlen(file_table_contents);

    /* Open filetable.temp
    ---------------------*/
    if ((file_ptr = fopen(filename, "w")) == NULL)
    {
        c_lasmsg(FALSE,"asterhdf2las-file", "Error opening filetable.temp");
        return (E_FAIL);
    }

    /* Write to file
    ----------------*/
    if (fwrite(file_table_contents,1,len_of_file_table_contents,
               file_ptr) != len_of_file_table_contents)
    {
        c_lasmsg(FALSE,"asterhdf2las-writ", "Error writing to filetable.temp");
        return (E_FAIL);
    }

    /* Close file
    ------------*/
    fclose(file_ptr);

    return (E_SUCC);
}    

