/****************************************************************************

NAME:           delete_image

PURPOSE:        Delete the input .img file and the .ddr file that goes
                with it.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "asterhdf2las.h"
#include "las.h"
#include "util.h"    /* c_splitname definition */
#include "diskio.h"  /* c_dkdel definition */

int delete_image
(
    char img_file[]       /* I: image file to be deleted(with .img extension)*/
)

{
    char ddrname[CMLEN];   /* name of the DDR file                           */
    char directory[CMLEN]; /* directory portion of input HDF filename        */
    char extension[CMLEN]; /* extension portion of input HDF file name       */
    char root[CMLEN];      /* root portion of input HDF file name            */

    /* Verify that the extension is .img
    ------------------------------------*/
    c_splitname (img_file, directory, root, extension);
    if (strcmp (extension, "img"))
    {
        c_lasmsg (FALSE, "asterhdf2las-ext",
                    "Image file must have a .img extension: %s", img_file);
        return (E_FAIL);
    }

    /* Delete the .img file
    ------------------------*/
    if (c_dkdel (img_file) != E_SUCC)
    {
        c_lasmsg (FALSE, "asterhdf2las-del", "Error removing the .img file: %s",
                  img_file);
        return (E_FAIL);
    }

    /* Replace the .img in the image file with .ddr
    -----------------------------------------------*/
    strcpy (ddrname, img_file);
    strncpy (&ddrname[strlen (ddrname) - 3], "ddr", 4);

    /* Delete the .ddr file
    ----------------------*/
    if (c_dkdel (ddrname) != E_SUCC)
    {
        c_lasmsg (FALSE, "asterhdf2las-del", "Error removing the .ddr file: %s",
                   ddrname);
        return (E_FAIL);
    }

    return (E_SUCC);
}
