/******************************************************************************
NAME:           get_dem_metadata

PURPOSE:        This function retrieves Level 3 DEM specific information 
                required for LAS DDR. 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS

NOTE: The DEM products are generated using 3N and 3B VNIR bands from the 
      Level 1A products. Therefore the DEM will be in geographic projection
      and there is no projection information.
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "PGS_MET.h"
#include "PGS_tk.h"
#include "mfhdf.h"
#include "PGS_SMF.h"
#include "asterhdf2las_protos.h"

int get_dem_metadata
(
    int fileid,               /* I: File id for the file containing
                                    parameter data                           */
    char *instrument_name     /* O: Instrument name ie ASTER                 */ 
) 

{
    PGSt_SMF_status ret;      /* Return status                               */

    /* Get instrument name from INSTRUMENTSHORTNAME tag
    --------------------------------------------------*/
    ret = PGS_MET_GetPCAttr(fileid,1,"product_summary.0","INSTRUMENTSHORTNAME", 
                            &instrument_name);
    if (ret != PGS_S_SUCCESS)
    {
        c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag "
                 "INSTRUMENTSHORTNAME. Unable to retrieve the sensor type.");
    }

    return (E_SUCC);
}
