/******************************************************************************
NAME:           get_l2_metadata

PURPOSE:        This function retrieves Level 2 metadata specific information 
                required for LAS DDR.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "PGS_MET.h"
#include "PGS_tk.h"
#include "mfhdf.h"
#include "PGS_SMF.h"
#include "asterhdf2las_protos.h"

int get_l2_metadata
(
    int fileid,               /* I: File id for the file containing
                                    parameter data                           */
    char projectionparameters[],/*I: The tag name that is to be retrieved
                                    ie PROJECTIONPARAMETERS                  */
    char mpmethod[],          /* I: The tag name that is to be retrieved
                                    ie MPMETHOD1                             */
    char *instrument_name,    /* O: Instrument name ie ASTER                 */
    char *projection_name,    /* O: Names in metadata will either be UTM, PS
                                    or EQRECT, SOM                           */
    double projection_values[],/* O: The 15 projection parameters            */
    int ddr_valids[]           /* O: Flags indicating if fields in the DDR
                                     are valid or not (VALID or INVAL)       */
) 

{
    PGSt_SMF_status ret;      /* Return status                               */

    /* Get instrument name from INSTRUMENTSHORTNAME tag
    --------------------------------------------------*/
    ret = PGS_MET_GetPCAttr(fileid,1,"product_summary_information",
                            "INSTRUMENTSHORTNAME",&instrument_name);  
    if (ret != PGS_S_SUCCESS) 
    {
        c_lasmsg(FALSE, "asterhdf2las-meta", 
                 "Error retrieving the tag INSTRUMENTSHORTNAME. Unable to "
                  "retrieve the sensor type.");
    }

    /* Get the projection parameters of the first band using tag 
       projectionparameters 
    ------------------------------------------------------------*/
    ret = PGS_MET_GetPCAttr(fileid,1,"level_1_carryover",
                            projectionparameters,projection_values); 

    if (ret != PGS_S_SUCCESS) 
    {
        c_lasmsg(FALSE, "asterhdf2las-meta", "Error retrieving the tag %s. "
                 "Projection parameters are invalid.", projectionparameters);
        ddr_valids[3] = INVAL;
    }
    else
    {
        ddr_valids[3] = VALID;
    }


    /* Get the projection name of the first band using tag mpmethod
    --------------------------------------------------------------*/
    ret = PGS_MET_GetPCAttr(fileid,1,"level_1_carryover",mpmethod,
                            &projection_name);

    if (ret != PGS_S_SUCCESS)
    {
        c_lasmsg(FALSE, "asterhdf2las-meta", 
                 "Error retrieving the tag %s", mpmethod);
        ddr_valids[0] = INVAL;
    }
    return (E_SUCC);
}
