/*******************************************************************************
NAME                           latlon2yx

PURPOSE: Converts the corner coordinates from lat & long in degrees to 
         projection coordinates (y, x) in meters.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:

PROJECT:        LAS

*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "proj.h"
#include "cproj.h"
#include "geompak.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

int latlon2yx
(
    double projection_parameters[], /* I: Projection values as seen in
                                          Projection Transformation Package */
    int projection_code,            /* I: Values can be TM, LAMCC, PS, 
                                          EQRECT, SOM or UTM                */ 
    double lat_lon_coordinates[],   /* I: Corner coordinates in latitude 
                                          and longitude                     */
    long zone,                      /* I: UTM zone code                     */
    double y_x_coordinates[]        /* O: Y X corner coordinates in meters  */
)

{
    int index;                      /* Variable used for looping            */
    long inproj = GEO;              /* Input projection code - always 
                                       geographic                           */
    long inunit = DEGREE;           /* Input unit - always degrees          */
    long inzone = DEFAULT_ZONE;     /* Input zone - always default of 62    */ 
    long indtmnum = WGS84;          /* Input datum - always WGS 84          */
    long outunit = METER;           /* Output unit - always meters          */
    long outzone = DEFAULT_ZONE;    /* Output zone - initialize to 62. It 
                                       will change if the projection is UTM */
    long outdtmnum = WGS84;         /* Output datum - always WGS84          */
    long prtprm[2];                 /* Report destination. Used to write
                                       out projection parameters to a file 
                                       or the terminal                      */
                                               
    double projection_parameters_in[15]; /* Geographic projection parameters*/

    /* Set the geographic projection parameters 
    -------------------------------------------*/
    for (index = 0; index < 15; index++)
        projection_parameters_in[index] = 0.0;

    /* Set to FALSE because we don't want the parameters reported to a file 
       or the terminal
    ----------------------------------------------------------------------*/
    prtprm[0] = prtprm[1] = FALSE;

    /* Set zone for UTM projection 
    -----------------------------*/
    if (projection_code == UTM)
        outzone = zone;

    /* Initialize input geographic and output parameters  
    ---------------------------------------------------*/
    if (c_transinit(&inproj,&inunit,&inzone,&indtmnum,
                    projection_parameters_in,(long *)&projection_code,&outunit,
                    &outzone, &outdtmnum,projection_parameters,
                    prtprm," ") != E_SUCC)
    {
        c_lasmsg(FALSE,"asterhdf2las-proj", 
                 "Error initializing projection parameters");
        return (E_FAIL);
    }
    /* Perform conversion, map lat long to Y X
    -----------------------------------------*/
    for (index = 0; index < NUM_COORDINATES; index=index+2)
    {
        if (c_trans (&inproj,&inunit,(long *)&projection_code,&outunit,
                     &lat_lon_coordinates[index+1], 
                     &lat_lon_coordinates[index],&y_x_coordinates[index+1],
                     &y_x_coordinates[index]) != E_SUCC)
        {
            c_lasmsg(FALSE, "asterhdf2las-proj", "Error converting "
                     "geographic coordinates to meters");
            return (E_FAIL);
        }
    }

    return (E_SUCC);
}
