#ifndef PROCESS_HDF_IMAGE_H
#define PROCESS_HDF_IMAGE_H
#include "las.h"
#include "mfhdf.h"

int process_hdf_image
(
    int32 sd_id,              /* I: SD interface identifier                  */
    int32 sds_index[],        /* I: Index of data set                        */
    double window[],          /* I: Window values requested by user          */
    int projection_code,      /* I: Value will be either TM=9, LAMCC=4,
                                    PS=6, EQRECT=17, SOM=22, GEO=0           */
    double projection_values[], /* I: Projection parameters of the scene     */
    double corner_coordinates[],/* I: Coordinates of the scene, no windowing
                                      is applied. Value is degrees or meters */
    int projection_distance[],/* I: Resolution of each band                  */
    char projection_units[],  /* I: Projection units (degrees or meters)     */
    int ddr_valids[],         /* I: Flags indicating if fields in the DDR are
                                    valid or not (0=not valid, 1=valid)      */
    char outfiles[][CMLEN],   /* I: Output LAS filename(s) in host format    */
    int band_groupings[],     /* I: Number of bands in each band grouping    */
    int nl[],                 /* I: Total number of lines for each band      */
    int ns[],                 /* I: Total number of samples for each band    */
    char infile[],            /* I: Input HDF filename in host format        */
    int window_specified,     /* I: Flag indicating if the window field was
                                    specified (TRUE or FALSE)                */
    int num_band_groupings,   /* I: Count of band groupings                  */
    int data_type[],          /* I: Data type of the each band in the HDF
                                    image. The output image will be the same
                                    data type as the input image.
                                    Value are either EBYTE or EWORD          */
    long msgtim,              /* I: Time between display messages            */
    char window_units[],      /* I: Units the window was specified in        */
    int num_window,           /* I: Count of window values                   */
    char bands[][3],          /* I: Bands to be processed                    */
    char instrument_name[],   /* I: Instrument name ie ASTER                 */
    char acquisition_date[],  /* I: Capture date                             */
    char acquisition_time[],  /* I: Capture time                             */
    int product_level,        /* I: Values can be Level 1A=L1A, Level 1B=L1B
                                    Level 2 =L2 or Level 3 DEM =L3           */
    char kinetic_temp[],      /* I: Flag indicating if product is kinetic
                                    temperature                              */
    long zone,                /* I: Zone code for UTM projection             */
    char processed_files[][CMLEN], /* O: array of filenames created for
                                         output from this application        */
    int *num_processed_files  /* O: count of out files that have been 
                                    processed                                */
);
#endif
