/****************************************************************************
NAME:            signal_cleanup

PURPOSE:         Cleanup/delete all the files that were created by this 
                 application, if the $DELFLG was set.  If $DELFLG was not 
                 set then do nothing.

RETURN VALUE:
Type = void

PROGRAM HISTORY:

PROJECT: LAS

NOTES:
  1) For the signal handler, this must be a void function.  Thus if any
     errors occur in this routine, then will not be checked or returned.
  2) This function cannot have any parameters, since it is passed from the
     main routine to the signal handler.
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "asterhdf2las.h"
#include "asterhdf2las_protos.h"

/* Define the global variables that are used by the cleanup routine when
   the SIGKILL or SIGTERM signals are caught
-------------------------------------------------------------------------*/
extern int g_num_processed_files; 
                              /* Number of files processed for output       */
extern char g_processed_files[NUMFILES][CMLEN];
                              /* Array of filenames that have been created
                                 for output from ASTERHDF2LAS               */
extern int g_delflg;          /* Flag to represent the global DELFLG flag
                                 (TRUE or FALSE)                            */

void signal_cleanup ()
{
    /* Cleanup the files that have been created if $DELFLG is set
    ----------------------------------------------------------*/
    cleanup_files (&g_num_processed_files, g_processed_files, g_delflg);

    exit (EXIT_FAILURE);
}
