/****************************************************************************
NAME:				editpts

PURPOSE:  Edits points based on the "leave-one-out" method.  Points are 
	  edited until the difference between RMS error values between two
	  successive iterations are less than the tolerance value.

****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"
extern long curent;
extern long curcat;


int editpts
(
    struct TPLDATA *tie_data,	/* Tie Point Location data records 	*/
    long npts,			/* Number of points in TPL file 	*/
    long degree,			/* Degree of polynomial to use 		*/
    double tolval,			/* Tolerance value (convergence check)  */
    long *pltflg,            	/* plot flag                            */
    char *plotfile,          	/* plot filename                        */
    FILE *fptr,			/* Print file pointer			*/
    long *msgtim,			/* message time				*/
    float *minrms          	/* Minimum rms value to complete autofit*/
)

{
struct TPLDATA *tt;		/* Ptr to TPL data structure 		*/
double min_rmse;		/* Minimum running RMSE for given test 	*/
double minerr;			/* minimum error			*/
double maxerr;			/* maximum error			*/
double prev_rmse;		/* RMSe value from previous value 	*/
double rmse;			/* Root Mean Squared Error 		*/
double test_rmse;		/* RMSE of each test value 		*/
long flag = 2;			/* message type flag			*/
long index;			/* Index of tie pt to remove 		*/
long i,k;			/* Loop counters 			*/
long npts_used;			/* Current number of points used 	*/
long old_active;		/* Old active flag value 		*/
long pass=1;			/* Non-fatal c_errmsg flag 		*/
long zero = 0;			/* zero					*/


/* Find the number of active points
  --------------------------------*/
for(tt=tie_data, npts_used=0, i=0; i<npts; i++, tt++)
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9)) 
      npts_used++;
if(npts_used==0)
  return(E_FAIL);

/* Edit pts based on the "leave-one-out" method
  --------------------------------------------*/
if (fit(tie_data, npts, npts_used, degree, &prev_rmse, &minerr, 
                  &maxerr, 0, plotfile) == E_FAIL)
   return(E_FAIL);
fprintf(fptr, "Initial fit RMSE (all active points):  %f pixels\n",prev_rmse);
 
curcat = 1;
curent = 1;
c_inittm("AUTOFIT","Processed","interation"," "," ",&npts,&zero,msgtim,&flag);
c_startm();

for(;;) 
   {
   curent++;
   curcat++;
   /* Make sure enough points are available
   --------------------------------------*/
   if (((npts_used<5)&&(degree==1))||((npts_used<8)&&(degree==2))) 
        {
	c_errmsg("Not enough points for polynomial degree","autofit-minpts"
                 ,&pass);
	return(E_FAIL);
	}
   /* Loop through points leaving one out and calculate the new RMSE
   ---------------------------------------------------------------*/
   for(min_rmse=999999999.0,index= -1,tt=tie_data,k=0; k < npts; k++, tt++)
      if((tt->active == 0)||(tt->active == 1)||(tt->active > 9)) 
         {
         old_active=tt->active;
         tt->active=8;
         if (fit(tie_data,npts,npts_used, degree, &test_rmse, 
                           &minerr, &maxerr, 0, plotfile) == E_FAIL)
             return(E_FAIL);
         if (test_rmse < min_rmse) 
            {
            min_rmse = test_rmse; 
            index=k;
            }
         tt->active=old_active;
         }

   /* if the RMSE did not decres error out
   -------------------------------------*/
   if (min_rmse > prev_rmse) 
        {
	c_errmsg("Could not edit to TOLVAL","autofit-wrapback",&pass);
        c_stoptm();
	return(E_SUCC);
	}

   /* print result
   -------------*/
   fprintf(fptr,"Difference between this fit and last:  %lf\n\n",
                 prev_rmse-min_rmse);
   if ((fabs(min_rmse-prev_rmse) <= tolval) || (min_rmse <= *minrms)) 
        {
        if (fit(tie_data, npts, npts_used, degree, &rmse, &minerr, 
                          &maxerr, *pltflg, plotfile) == E_FAIL)
            return(E_FAIL);
	fprintf(fptr,"%d active points remain with RMSE of %f pixels\n", 
		npts_used,rmse);
        c_stoptm();
	return(E_SUCC);
	}

   /* Automatically reject this point
   --------------------------------*/
   fprintf(fptr, "%s [8] Removed from model; Resulting RMSE:  %f\n",
	    tie_data[index].pt_id,min_rmse);
   tie_data[index].active=8;
   npts_used--;
   prev_rmse = min_rmse;
   }
}
