/****************************************************************************
NAME:				getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"
#define TERM 0
#define PFILE 1
#define LP 2


int getpar
(
    struct PARBLK *vblk,	/* TAE variable block 				*/
    char *tpl,		/* Input tie point location file name 		*/
    char *outtpl,		/* Output tie point location file name 		*/
    long *degree,		/* Degree of polynomial 			*/
    float *tolval,		/* Tolerance value 				*/
    long *pltflg,           /* plot flag                                    */
    char *plotfile,         /* plot filename                                */
    long print[],		/* print option                          	*/
    char *prtfile,          /* print file name                              */
    char *filename,         /* print file name                              */
    long *msgtim,		/* message time					*/
    float *minrms          /* Minimum rms value to complete autofit 	*/
)


{
struct VARIABLE *var;		/* pointer to a TAE variable in vblk 	*/
long acc;			/* access flag for c_getnam 		*/
long fatal = -1;		/* Fatal error flag			*/
long count;			/* Loop counter 			*/
long zero = 0;			/* Constant of zero 			*/

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if(c_getnam(vblk,"INTL",&zero,tpl,"tpl",&acc)!=E_SUCC)
   return(E_FAIL);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(vblk,"OUTTL",&zero,outtpl,"tpl",&acc)!=E_SUCC) 
   return(E_FAIL);

/* Get plot file name
-------------------*/
if (c_getcnt(vblk,"OUTPLOT",&count) != E_SUCC)
   return(E_FAIL);
if (count != 0)
   {
   *pltflg = TRUE;
   acc = IWRITE;
   if (c_getnam(vblk,"OUTPLOT",&zero,plotfile,"plt",&acc) != E_SUCC)
      return(E_FAIL);
   }
else
   *pltflg = FALSE;

/* Get the maximum allowable degree of polynomial for model
  --------------------------------------------------------*/
var=p_find(vblk,"PDEGREE");  
*degree=IVAL(*var,0);

/* Get the tolerance value
  -----------------------*/
var=p_find(vblk,"TOLVAL");   
*tolval=RVAL(*var,0);

/* Get print options
------------------*/
strcpy(prtfile,"autofit");
if (c_setprt(vblk,&print[TERM],&print[PFILE],&print[LP],
             prtfile,filename) != E_SUCC)
   c_errmsg("Fatal error encountered","autofit-fatal",&fatal);

/* Retrieve globals
-------------------*/
var = p_find(vblk,"$MSGTIM");
*msgtim = IVAL(*var,0);

/* Get the minimum rms value
  ---------------------------*/
var=p_find(vblk,"STOPRMSE");   
*minrms=RVAL(*var,0);

return(E_SUCC);
}
