#include "las.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block	      */
    float wa[],		        /* weight factor array for denominator images */
    float *wn,			/* weight factor for numerator image  	      */
    float *denom,		/* value to be output when denominator is 0   */
    long *msgtime,		/* elapsed time between display messages      */
    char t1[][CMLEN],		/* pointer to denominator TAE file descriptor */
    char h1[][CMLEN],		/* pointer to denominator host file descriptor*/
    char t2[][CMLEN],		/* pointer to numerator TAE file descriptor   */
    char h2[][CMLEN], 		/* pointer to numerator host file descriptor  */
    char t3[][CMLEN],		/* pointer to output TAE file descriptor      */
    char h3[][CMLEN],		/* pointer to output host file descriptor     */
    long wind1[][4],		/* window specification for denominator images*/
    long wind2[][4],		/* window specification for numerator image   */
    long *nlout,			/* number of lines in output image   	      */
    long *npout,			/* number of pixels per line in output image  */
    long *nib1,			/* number of bands per denominator image      */
    char *minmax,		/* min max flag buffer			      */
    long bands1[][MAXBND + 1], /* band specification for denominator images  */
    long bands2[][MAXBND + 1],	/* band specification for numerator image     */
    long *nim1,			/* number of input images for denominator     */
    long *totbnd1		/* total number of bands in denominator       */
)

/*******************************************************************************
NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        12/20/83   G. Linebaugh, SASC      
     2	       3/20/86   K. Gacke, CSB        
     3            6/87   M. Mazzella        Conversion to NEWLAS
     4         1/26/89   K. Narayanan       Retrocode to 5.0 standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   RunS under TAE.

PROJECT:        LAS 

*******************************************************************************/
{

struct VARIABLE *var;		/* pointer to a TAE variable within vblock    */

long	fatal = -1;
long	i;			/* counter				      */
long	maximg = MAXIMG;	/* maximum number of images to be processed   */
long	nib2[MAXIMG+1];		/* number of bands per numerator image        */
long	nim2;			/* number of images in numerator specification*/
long	nonfatal = 1;
long	one = 1;
long	qflag[MAXIMG][2]; 	/* flag indicating if subwindows specified    */
long    same;			/* flag indicating if images have same window */
long	status;			/* function return status code		      */
long    totbnd2;		/* total number of bands in numerator	      */
long	x;			/* loop index				      */
long	zero = 0;

char outbuf[ERRLEN];		/* message buffer			      */

/*
  get numerator image specification
*/
status = c_getin( vblock, "in", &zero, t2, h2, &nim2, wind2, bands2, nib2, 
                  &totbnd2, &one, &same, qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","bandratio-fatal",&fatal);

if (totbnd2 > 1)
   {
   c_errmsg("Numerator image must be single band", "bandratio-single", &fatal);
   }

/*
  get denominator images specification
*/
status = c_getin( vblock, "in", &one, t1, h1, nim1, wind1, bands1, nib1, totbnd1, 
                &maximg, &same, qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","bandratio-fatal",&fatal);

if (same == 0)
   {
   c_errmsg("Denominator images do not have same window size",
           "bandratio-window",&fatal);
   }

/* 
   check size differences & set up output image size 
*/
if (wind1[0][NS] != wind2[0][NS])
   {
   *npout = wind1[0][NS] < wind2[0][NS] ? wind1[0][NS] : wind2[0][NS];
   sprintf(outbuf, "Number of output samples reset to %d", *npout);
   c_errmsg(outbuf, "informational", &nonfatal);
   }
else
   *npout = wind1[0][NS];

if (wind1[0][NL] != wind2[0][NL])
   {
   *nlout = wind1[0][NL] < wind2[0][NL] ? wind1[0][NL] : wind2[0][NL];
   sprintf(outbuf, "Number of output lines reset to %d", *nlout);
   c_errmsg(outbuf, "informational",&nonfatal);
   }
else
   *nlout = wind1[0][NL];

/*
   get output image name
*/
status = c_getout(vblock, "out", t3, h3);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","bandratio-fatal",&fatal);

/* 
  get weight for the numerator
*/
var = p_find(vblock, "numerwt");
*wn = RVAL (*var, 0);

/*
  get weight array for the denominator
*/
var = p_find(vblock, "denomwt");
i = (*var).v_count;
if (i == 0)
   for (x = 0; x < 24; ++x) wa[x] = 1.0;
else
   for (x = 0; x < i; ++x) wa[x] = RVAL(*var, x);

/*
  get value to be output when denominator is zero 
*/
var = p_find(vblock, "denomval");
*denom = RVAL(*var, 0);

/*
  get TAE globals 
*/
var = p_find(vblock, "$msgtim");
*msgtime = IVAL(*var, 0);

var = p_find(vblock, "$minmax");
strcpy(minmax, SVAL(*var,0));
}
