#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "statio.h"
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAX_ELEMENT   MAXBND*(MAXBND+1)/2
#define MAXW          MAX_ELEMENT+2*MAXBND

float   *mean_matrix;
double  *covinv_matrix;
struct FDESC *  fdesc[MAXIMG+3];   /* Array of file descritors                */
struct GDESC *  gdesc;             /* Group descriptor                        */
 
long    min_max;
long    num_files;              /* no. of files opened                        */
long    inarray[MAXBND],
        outpntr,
        offpntr;                /*  of offsets                                */
char    *thebuf,                /* Buffer for image i/o                       */
        hosin[MAXIMG][CMLEN],   /* Input host file names                      */
        hosout[3][CMLEN],       /* Output host file name                      */
        out[3][CMLEN];          /* names of files opened                      */

FILE *  sdesc[2];               /* File descriptors of stats files            */
static  long    three = 3;      /* Constants for passing to routines          */
struct DDR ddr;
struct  PARBLK vblock;          /* LAS parameter block                        */
unsigned char   *ealloc();


int main
(void) /* bayes */

/***********************************************************************
NAME:	BAYES

FUNCTION:
	Performs Bayesian classification on a multiband image

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0      27-May-82     C. Gurney
    2.0         Jun-87     N. Deodhar/SAR    New LAS & coversion to C
    3.0         Oct-88     B. Allam/SAR      Retrocode
    4.0      23-Mar-89     K. Narayanan      Modified upddr, inittm calls
    5.0      10-Apr-89     K. Narayanan      Enhanced to accept others'
                                             stats files in general
    5.0	     06-Apr-90     D. VanderZee      Numerous performance improvements
					     including adding a mask value for
					     more rapid classification.  Also
					     fixed numerous bugs and cleaned 
				     	     up error messages and corresponding					     pdf documentation.  Bayes now 
					     classifies images 3 times faster 
					     than it did previously.  
    5.0      24-Jul-91     T. Rockvam	     Changed covariance matrices from 
					     float to double to improve accuracy
    5.1	     21-Sep-92	   T. Mittan	     Changed include files to eliminate
					     warnings on link.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under LAS

ALGORITHM (FLOW):
                  Initialize LAS
                  Get input and output parameters
                  Get statistics
                  Setup input and output files
                  Calculate max likelihoods & write output files

******************************************************************************/

{

char	aprior[7],		/* User option for prior probabilities        */
	sthos[CMLEN],		/* Stats file host name                       */
	lasout[3][CMLEN];	/* Output LAS name                            */

static long bands[MAXIMG][MAXBND+1]; /* Band numbers for each input image  */
long	chi,			/* Chi-square or max likelihood indicator flag*/
	clind[MAXCLS],	        /* Class indeces for input stats file         */
	msgtim,			/* Timed processing message time interval     */
	nclass,			/* Number of classes in input stats           */
	nimg,			/* Number of input images                     */
	nuse,			/* Number of classes the user wants to use    */
	mask,			/* flag indicating presence of a mask value   */
	nbands[MAXIMG],		/* Number of bands for each input image       */
	std_out,		/* FORTRAN LUN to standard output             */
	totbnd,			/* Total number of input image bands          */
	wind[1][4];		/* Input image window                         */

float	cconst[MAXCLS];  	/* Class constants                            */
float   maskval;		/* pixel mask value */
float	*mean[MAXCLS];		/* pointers to means for classes */
double	*covinv[MAXCLS];	/* pointers to inverse covariance matices */

num_files = 0;

/* Initiate LAS
   ------------ */
c_inlas (&vblock, &std_out);

/* Get LAS parameters
   ------------------ */
getpar (&vblock, sthos, lasout, nbands, aprior, &totbnd, 
	&chi, wind[0], &nimg, &nuse, &msgtim, bands,&mask,&maskval);

/* Get statistics
   -------------- */
do_stats (mean, covinv, sthos, &nclass, aprior, totbnd, cconst, nuse, 
          clind);

/* Set up files
   ------------ */
setup_files (lasout, nimg, nclass, chi, wind, clind, nbands, bands, mask);

/* Calculate max likelihoods & write output files
   ---------------------------------------------- */
do_calc (nclass, clind, totbnd, cconst, chi, wind[0], msgtim, 
		mean, covinv, mask, maskval);

c_hstout (&vblock, hosout[0], hosin, &nimg, "");
c_hstout (&vblock, hosout[1], hosin, &nimg, "");

c_complt ("BAYES", lasout, &three);

return EXIT_SUCCESS;
}

