#include "bayes.h"
#include "util.h"
#include "pixman.h"

/************************************************************************

NAME:	do_calc

FUNCTION:
	Controls calculation of class membership probabilities
	and generation of classified image, probability image
	and histograms

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0      27-May-82     C. Gurney
    2.0         Jun-87     N. Deodhar/SAR    New LAS & coversion to C
    3.0         Oct-88     B. Allam/SAR      Retrocode
    4.0      23-Mar-89     K. Narayanan      Changed INITTM
    5.0       6-Apr-90     D. VanderZee      Cleaned up, sped up, fixed bugs, 
					     and added masking  

***********************************************************************/

void do_calc
(
    long nclass,			/* i   # classes                             */
    long clind[],		/* i   class indices                         */
    long totbnd,			/* i   # bands                               */
    float cconst[MAXCLS],  	/* i   class constants                       */
    long chi,			/* i   output indicator flag                 */
    long window[4],		/* i   image window specifications           */
    long msgtim,			/* processing line message interval          */
    float *mean[MAXCLS],  	/* i   cluster means                         */
    double *covinv[MAXCLS],  	/* i   cluster inverse covariance matricies  */
    long mask,			/* i   mask value presence flag              */
    double maskval		/* i   pixel value for mask                  */
)
{
long zero = 0;
long one = 1;
long two = 2;
long three = 3;
unsigned char *class;		/* Class indeces                             */
long	chist[MAXCLS],		/* histogram of class assignments            */
	dtype = EREAL,		/* Real data type */
	bdtype = EBYTE,		/* Byte data type */
        ldtype = ELONG,		/* Long data type */
	*hist[MAXCLS], 	        /* histogram of class probabilities          */
        *hist_matrix,           /* buffer for histogram of class probs */
	i,			/* Loop counter on bands                     */
	count,    	        /* beginning pixel of processed line segment  */
	k,  	 		/* Loop counter on classes                   */
	lnk,			/* Loop counter on lines                     */
	msg_flag = GEN2,        /* processing message flag                   */
	n, m,   		/* Loop counter on samples                   */
	nl,			/* Number of input lines                     */
	ns,			/* # pixels of image line                    */
	phist[MAXCLS];		/* histogram of max class probabilities      */
double  xmin, xmax, 		/* min & max values for max likelyhood or
			           chi squared image */ 
        minval, maxval;		/* min & max values for classified image */

static	long	hshape[9] = {0, 1, 256, 0, 0, 0, 0, 0, 0};
				/* Shape vector for histogram                */

float	*g[MAXCLS],	        /* g values                                  */
        *g_matrix,		/* buffer for g values */
	*gvals,			/* Maximum g values                          */
        *x_matrix,		/* buffer for line of pixels */
	*x[MAXBND];	        /* pointers to pixels in current line */
unsigned char	*tmpvals;       /* temporary storage for us in gethist */

extern	long	curent;		/* Processing line message counter           */



hshape[0] = ELONG;
fprintf (stdout, "\n");
nl = window[NL];
ns = window[NS];

/* set number of classes based on presence of mask class */
if (mask)
   nclass += 1;

/* Allocate storage for work buffers
  --------------------------------- */
g_matrix = (float *) malloc (nclass * ns * sizeof (float));
hist_matrix = (long *) malloc (nclass * MAXCLS * sizeof (long));
gvals = (float *) malloc (ns * sizeof (float));
x_matrix = (float *) malloc (totbnd * ns * sizeof (float));
class = (unsigned char *) malloc (ns);
tmpvals = (unsigned char *) malloc (ns);

if ((g_matrix == NULL) || (x_matrix == NULL) || 
   (hist_matrix == NULL) || (gvals == NULL) || 
   (class == NULL) || (tmpvals == NULL))
      iso_err("!Error allocating dynamic memory","bayes-alloc");
/* set up pointer arrays */
for (m=0, k=0, n=0; m < nclass; m++, k+=MAXCLS, n+=ns) 
   {
   g[m] = &g_matrix[n];
   hist[m] = &hist_matrix[k];
   }
for (i=0, n=0; i < totbnd; i++, n+=ns)
   {
   x[i] = &x_matrix[n];
   }

/* Initialize histograms */
count = MAXCLS;
c_pxfill (chist, &ldtype, &count, &zero);
c_pxfill (phist, &ldtype, &count, &zero);
for (n=0; n < nclass; n++)
 c_pxfill (hist[n], &ldtype, &count, &zero); 

/* Update min max values
   --------------------- */
minval = nclass;
maxval = 0;

xmin = 255;
xmax = 0;

/* Set up processing line message
   ------------------------------ */
curent = 1;

c_inittm("BAYES", "Processed", "line", "", "", &nl, &one, &msgtim, &msg_flag);
c_startm ();
for (lnk = 1; lnk <= nl; lnk++)
   {
   curent = lnk;
   if (c_estep (&gdesc) != E_SUCC)
      iso_err ("$", "");

/* Process the lines 
   ----------------- */

   for (i = 0; i < totbnd; i++)
      c_pxcopy (((float *) &thebuf[inarray[i] * sizeof (float)]),
               x[i], &dtype, &ns);

      classify (mean, covinv, x, totbnd, ns, nclass, cconst, 
                g, class, gvals, chi, mask, maskval);

      get_histo (ns, totbnd, nclass, clind, g, class, gvals, hist, chist, 
		phist, cconst, chi, tmpvals);


/* update min and max values for each band
--------------------------------------- */
      if (min_max) {
        c_pxmnmx(&thebuf[offpntr], &ns, &minval, &maxval, &bdtype);
        c_pxmnmx(&thebuf[outpntr], &ns, &xmin, &xmax, &bdtype);
      }
   }
if (c_estep (&gdesc) != E_SUCC)
    iso_err ("$", "");
c_stoptm ();

/* update output image ddr min max fields
 -------------------------------------- */
  if (min_max) {
    c_svmnmx(hosout[0], &one, &minval, &maxval, "YES");
    c_svmnmx(hosout[1], &one, &xmin, &xmax, "YES");
  }

/* Write histograms
   ---------------- */
c_stput (&sdesc[1], "HISTOGRAM", &one, &zero, hshape, chist);
c_stput (&sdesc[1], "HISTOGRAM", &two, &zero, hshape, phist);
for (k = 1; k <= nclass; k++)
   c_stput (&sdesc[1], "HISTOGRAM", &three, &k, hshape, hist[k - 1]);

/* Close files
   ----------- */
c_stclse (&sdesc[1]);
if (c_egclse (&gdesc) != E_SUCC)
   iso_err ("?Error closing image files", "bayes-close");
free ((char *) thebuf);
free (g_matrix);
free (x_matrix);
free (hist_matrix);
free (gvals);
free (class);
free (tmpvals);
   
return;
}
