#include "bayes.h"
#include "string.h"
#include "util.h"

/************************************************************************

NAME:	getpar

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0      27-May-82     C. Gurney
    2.0         Jun-87     N. Deodhar/SAR    New LAS & coversion to C
    3.0         Oct-88     B. Allam/SAR      Retrocode
    5.0       6-Apr-90     D. VanderZee      added mask parameter
    6.1      24-May-94	   D. Eichacker	     added opt[]

FUNCTION:
	Receives parameters from LAS process definition file

***********************************************************************/

void getpar
(
    struct PARBLK *vblock,	/* LAS parameter block                        */
    char sthos[],	        /* Input stats file host name                 */
    char lasout[][CMLEN],	/* Output file LAS names                      */
    long nbands[MAXIMG],		/* Number of bands for each input image       */
    char aprior[],		/* Option for prior probabilities             */
    long *totbnd,		/* # input images                             */
    long *chi,			/* Chi-square or max likelihood indicator flag*/
    long wind[],			/* Input image window                         */
    long *nimg,			/* Number of input images                     */
    long *nuse,			/* Number of classes the user wants to use    */
    long *msgtim,		/* Timed processing message time interval     */
    long bands[][MAXBND+1], 	/* Band nos. for each image		      */
    long *mask,			/* mask flag indicating presence of maskval   */
    float *maskval		/* pixel value of mask */
)
{
long zero = 0;

char	maxchi[4],		/* Chi-square or max likelihood indicator flag*/
        minmax[4],
        runtyp[12],
	lasin[MAXIMG][CMLEN];	/* Input image LAS name                       */

static	long	opt[3]={0,0,0}; /* optional parameter array for EOPENS	      */

long	access,			/* Type of access for files                   */
	indtype,		/* Data type to read input image              */
	band,			/* Counter on bands                           */
	image,			/* Loop counter on input images               */
	imglim = MAXIMG,	/* Maximum number of input images allowed     */
	qflag[MAXIMG][2],	/* Flag indicating if subwindows specified    */
	same,			/* Flag indicating all input windows are same */
	status,			/* Return status                              */
	window[MAXIMG][4];	/* Input image windows for each input image   */

struct	VARIABLE *var, *vr;	/* Pointer into variable block                */

/* Get input image names
   --------------------- */

if (c_getin (vblock, "IN", &zero, lasin, hosin, nimg, window, bands,
       nbands, totbnd, &imglim, &same, qflag) != E_SUCC)
   iso_err ("$", "");

if (!same)
   iso_err ("!Input images must be same size", "bayes-size");

wind[SL] = window[0][SL];
wind[SS] = window[0][SS];
wind[NL] = window[0][NL];
wind[NS] = window[0][NS];
access = IREAD;
indtype = EREAL;
band = 0;

for (image = 0; image < *nimg; image++)
   {
   status = c_eopens (&fdesc[image], hosin[image], bands[image],
                      &window[image][SL], &window[image][SS], &wind[NL], 
		      &wind[NS], &inarray[band], &access, &indtype, &zero, opt);
   if (status != E_SUCC)
      iso_err ("!Error opening input image file", "bayes-open");
   band += nbands[image];
   }
/*
   Get input stats file name
   -------------------------
*/
access = IREAD;

status = c_getnam (vblock, "INSTAT", &zero, sthos, "stat", &access);
if (status != E_SUCC)
   iso_err ("$", "");
/*
   Get output image names
   ---------------------- */

if (c_getout (vblock, "OUT", lasout[0], hosout[0]) != E_SUCC)
   iso_err ("$", "");


if (c_getout (vblock, "OMAXCHI", lasout[1], hosout[1]) != E_SUCC)
   iso_err ("$", "");
/*
   Get output stat file name
   ------------------------- */
access = IWRITE;

status = c_getnam (vblock, "OUTSTAT", &zero, hosout[2], "stat", &access);
if (status != E_SUCC)
   iso_err ("$", "");

strcpy(lasout[2], hosout[2]);

/*
   Get implicit global variable $runtype
   ------------------------------------- */
vr = p_find (vblock, "$RUNTYPE");
strcpy (runtyp, SVAL (*vr, 0));

/*
   Get number of classes
   --------------------- */
var = p_find (vblock, "NCLASS");
if (!(*var).v_count)
  *nuse = 0;
else
  *nuse = IVAL (*var, 0);
if ((*nuse > 0) && (!strncmp(runtyp, "BATCH", 5)))
  iso_err ("!Cannot run in batch when NCLASS is not equal to zero", "bayes-batch");
/*
   Get option for prior probabilities and check
   -------------------------------------------- */
var = p_find (vblock, "APRIORI");
strcpy (aprior, SVAL (*var, 0));

var = p_find (vblock, "MAXCHI");
strcpy (maxchi, SVAL (*var, 0));

if ((strcmp (maxchi, "MAX")) == 0)
   *chi = FALSE;
else
   *chi = TRUE;

/* Get mask value
   -------------- */
var = p_find (vblock, "MASKVAL");
if (!(*var).v_count)
  *mask = FALSE;
else
  {
  *mask = TRUE;
  *maskval = RVAL (*var, 0);
  }

var = p_find (vblock, "$MSGTIM");
*msgtim = IVAL (*var, 0);

var = p_find(vblock, "$MINMAX");
strcpy(minmax, SVAL(*var,0));
if (minmax[0] == 'Y' || minmax[0] == 'y') min_max = TRUE;
else min_max = FALSE;
}
