#include "bayes.h"

void priori_prob
(
    float prior[MAXBND],		/* o   prior probabilities                    */
    long nclass,			/* i   # classes                              */
    char aprior[],		/* i   user option for prior probabilities    */
    long clind[]		/* Class indeces                              */
)

/************************************************************************

NAME:	priori_prob

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0      27-May-82     C. Gurney
    2.0         Jun-87     N. Deodhar/SAR    New LAS & coversion to C
    3.0         Oct-88     B. Allam/SAR      Retrocode
    5           May-90     D. VanderZee      Compare sum to 1.0 +/- .00001 

FUNCTION:
	Finds a priori probabilities according to user option

	Comments-     The prior probabilities entered from the terminal are
	              summed.  if the sum != 1 the user is given a 2nd
	              chance to enter valid values.  if the values still
	              don't sum to 1 then the program aborts with priors

***********************************************************************/
{
long zero = 0;

float	prob,			/* Probability                                */
	sum;			/* Sum of prior probabilities                 */
short	less_than_0;		/* Indicates prior probabilty is less than 0  */
long	count,			/* Number of tries on prob sum                */
	ierr,			/* Return status                              */
	ind,			/* Loop counter on classes                    */
	k,			/* A class index                              */
	lj,			/* Loop counter on class name string          */
	shape[9];		/* stats data descriptor                      */
char	cname[80],		/* class name in character format             */
	lname[200],		/* class name as returned from stats file     */
	pval1[80];		/* A prior probability for the class          */

static	char	messg[2][80] = {"Enter prior probability for class  ",
			   "Prior probabilities must be > 0. Please reenter "};
			/* array to hold terminal i/o messages                */
static	 char 	msg[80];		/* buffer for error messages */

int term;           /* How the string was terminated              */


/* Equal prior probabilities
   ------------------------- */
if ((strcmp (aprior, "EQUAL")) == 0)
   {
   prob = 1.0 / ((float) nclass);
   for (ind = 0; ind < nclass; ind++)
      prior[ind] = prob;
   }
else if ((strcmp (aprior, "SFILE")) == 0)
   {
/*
   Retrieve prior probabilities from stats file
   -------------------------------------------- */
   sum = 0.0;
   for (ind = 0; ind < nclass; ind++)
      {
      k = clind[ind];
      ierr = c_stget (&sdesc[0], "PRIOR_PROBABILITY",&k, &zero, shape, 
		&prior[ind]);
      if (ierr != E_SUCC)
	{
        sprintf(msg,"!No prior probability in statistics file for class # %d"
		,k);
 	iso_err (msg,"bayes-noprior");
	}
      sum += prior[ind];
      }
   if (((sum - 1.0) > .00001) || ((sum - 1.0) < -.00001))
    iso_err("!Sum of prior probabilities for selected classes not equal to 1"
		,"bayes-badprior");
   }
else
   {
/*
   Enter prior probabilities at terminal
   ------------------------------------- */
   count = 0;
   do {
      sum = 0.0;
      t_write ("", T_STDCC);
      for (ind = 0; ind < nclass; ind++)
	 {
	 k = clind[ind];
         strcpy (lname, "");
	 ierr = c_stget (&sdesc[0], "NAME", &k, &zero, shape, lname);
         if (ierr != E_SUCC)
	   {
          sprintf(msg,"!No class name present in statistics file for class # %d"
		,k);
	   iso_err(msg,"bayes-noclass");
	   }
         lname[shape[2]] = '\0';
	 strcpy (cname, lname);
         for (lj = (long) strlen (cname); lj < 80; cname[lj++] = ' ')
            ;
/*
   Change to write the mess 1 and wait for cname
   --------------------------------------------- */
	 t_write (messg[0], T_STDCC);
	 t_write (cname, T_STDCC);
         less_than_0 = TRUE;
         do {
	    t_read (pval1, &term);
            sscanf (pval1, "%f6.4", &prior[ind]);
/*
   Change to mess 2 and get back for the value
   ------------------------------------------- */
	    if (prior[ind] <= 0)
	       t_write (messg[1], T_STDCC);
            else
               less_than_0 = FALSE;
            } while (less_than_0);
	 sum += prior[ind];
	 }
/*
   Check values sum to 1
   (user has a 2nd chance if they don't)
   ------------------------------------- */
      if (((sum - 1.0) > .00001) || ((sum - 1.0) < -.00001))
	 {
	 count++;
	 if (count == 2)
 	    iso_err ("$","");
	 else
      iso_err("?Prior probabilities do not sum to 1. You have one more chance.",
                     "bayes-retry");
	 }
      } while (((sum - 1.0) > .00001) || ((sum - 1.0) < -.00001));
      
/*
   Write values into stats file
   ---------------------------- */
   shape[0] = EREAL;
   shape[1] = 1;
   shape[2] = 1;
   t_write ("", T_STDCC);
   t_write ("Priori probabilities are being written into stats file\n", T_NULL);
   for (ind = 0; ind < nclass; ind++)
      {
      k = clind[ind];
      ierr = c_stput (&sdesc[0], "PRIOR_PROBABILITY", &k, &zero, shape, &prior[ind]);
      if (ierr == E_GNLIO)
        iso_err ("!Error writing to input statistics file", "bayes-write");
      }
   }
return;
}
