#ifndef PROTOS_H
#define PROTOS_H

void classify
(
    float *mean[MAXCLS],          /* i   cluster means */
    double *covinv[MAXCLS],        /* i   inverse covariance matricies for clust */
    float *x[MAXBND],		/* i   image data                             */
    long totbnd,			/* i   number of bands                        */
    long ns,			/* i   number of pixels to be classified      */
    long ncx,			/* i   number of classes                      */
    float cconst[],		/* i   class constants                        */
    float *g[MAXCLS],		/* o   g values                               */
    unsigned char class[],		/* o   class assignments                      */
    float gvals[],		/* o   maximum g values                       */
    long chi,			/* i   output indicator flag                  */
    long mask,			/* i   mask presence indicator               */
    double maskval		/* i   pixel mask value */
);

void do_calc
(
    long nclass,			/* i   # classes                             */
    long clind[],		/* i   class indices                         */
    long totbnd,			/* i   # bands                               */
    float cconst[MAXCLS],  	/* i   class constants                       */
    long chi,			/* i   output indicator flag                 */
    long window[4],		/* i   image window specifications           */
    long msgtim,			/* processing line message interval          */
    float *mean[MAXCLS],  	/* i   cluster means                         */
    double *covinv[MAXCLS],  	/* i   cluster inverse covariance matricies  */
    long mask,			/* i   mask value presence flag              */
    double maskval		/* i   pixel value for mask                  */
);

void do_stats
(
    float *mean[MAXCLS],          /* o   mean values for classes */
    double *covinv[MAXCLS],     /* o   inverse covariance matrices for classes*/
    char sthos[],	        /* i   stats file host name                   */
    long *nclass,		/* o   # classes in stats file                */
    char aprior[6],		/* i   option for prior probabilities         */
    long totbnd,			/* i   # input images                         */
    float cconst[MAXCLS],	        /* o   class constants                        */
    long nuse,			/* i   number of classes                      */
    long clind[MAXCLS]	        /* o   class indices                          */
);

void dyn_alloc
(
    long num_inbands,    /* number of input image bands    */
    long nclass,		/* number of classes */
    float *mean[],	/* means pointers */
    double *covinv[]      /* inverse covariance matrix pointers */
);

void get_histo
(
    long ns,			/* i   # pixels in image line to be processed */
    long totbnd,			/* i   # bands                                */
    long nclass,			/* i   # classes                              */
    long clind[],		/* i   class indices */
    float *g[],	  		/* i   scaled g values                        */
    unsigned char class[], 		/* i   class assignments                      */
    float gvals[],		/* i   scaled maximum g values                */
    long *hist[],		/* i/o histogram of class probabilities       */
    long chist[],		/* i/o histogram of class assignments         */
    long phist[],		/* i/o histogram of max class probabilities */
    float cconst[],	        /* i   class constants                        */
    long chi,			/* i   output indicator flag                  */
    unsigned char *tmpvals	 /* tmp buffer for byte data */
);

void getpar
(
    struct PARBLK *vblock,	/* LAS parameter block                        */
    char sthos[],	        /* Input stats file host name                 */
    char lasout[][CMLEN],	/* Output file LAS names                      */
    long nbands[MAXIMG],		/* Number of bands for each input image       */
    char aprior[],		/* Option for prior probabilities             */
    long *totbnd,		/* # input images                             */
    long *chi,			/* Chi-square or max likelihood indicator flag*/
    long wind[],			/* Input image window                         */
    long *nimg,			/* Number of input images                     */
    long *nuse,			/* Number of classes the user wants to use    */
    long *msgtim,		/* Timed processing message time interval     */
    long bands[][MAXBND+1], 	/* Band nos. for each image		      */
    long *mask,			/* mask flag indicating presence of maskval   */
    float *maskval		/* pixel value of mask */
);

void priori_prob
(
    float prior[MAXBND],		/* o   prior probabilities                    */
    long nclass,			/* i   # classes                              */
    char aprior[],		/* i   user option for prior probabilities    */
    long clind[]		/* Class indeces                              */
);

void setup_files
(
    char lasout[][CMLEN],	/* i   output image LAS names                 */
    long nmrimg,			/* i   number of input images                 */
    long nclass,			/* i   # classes                              */
    long chi,			/* i   indicator flag for output              */
    long wind[][4],		/* i   window specification                   */
    long clind[MAXCLS],		/* i   class indices */
    long nbands[MAXIMG],		/* Number of bands for each input image       */
    long bands[][MAXBND+1], 	/* Band nos. for each image		      */
    long mask			/* i   flag indicating presence of mask value */
);

void iso_err
(
    char *message,
    char *key
);

#endif

