#include "bayes.h"

/************************************************************************

NAME:	setup_files

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
    1.0      27-May-82     C. Gurney
    2.0         Jun-87     N. Deodhar/SAR    New LAS & conversion to C
    3.0         Oct-88     B. Allam/SAR      Retrocode
    4.0      23-Mar-89     K. Narayanan      Changed flag for UPDDR
    5.0       6-Apr-90     D. VanderZee      Cleaned up, fixed up, changed 
					     output images types to byte,  
					     and added masking 
    6.1      24-May-94     D. Eichacker      Added opt[].

FUNCTION:
	Sets up file specification and action groups for input
	and output images and writes headers to output file of probabilitity
	histograms

	Comments-
	   output file #1
	      min value  = 1
	      max value  = nclass
	   output file #2
	      min value  = 0
	      max value  = 255

***********************************************************************/

void setup_files
(
    char lasout[][CMLEN],	/* i   output image LAS names                 */
    long nmrimg,			/* i   number of input images                 */
    long nclass,			/* i   # classes                              */
    long chi,			/* i   indicator flag for output              */
    long wind[][4],		/* i   window specification                   */
    long clind[MAXCLS],		/* i   class indices */
    long nbands[MAXIMG],		/* Number of bands for each input image       */
    long bands[][MAXBND+1], 	/* Band nos. for each image		      */
    long mask			/* i   flag indicating presence of mask value */
) 
{
long zero = 0;
long one = 1;
long two = 2;
long three = 3;

static char	name1[] = {"CLASSES"},	/* name of histo. of classified image */
		name2[] = {"MAX_LIKELIHOOD"},
			/* name of histogram of probability image             */
		name3[] = {"PROBS_FOR_EACH_CLASS"},
			/* name of set of histograms of probs for each class  */
		name4[] = {"CHI_SQUARES"},
			/* name of mask class */
		name5[] = {"MASK CLASS"};

unsigned char	lname[100];	/* Class name from stats file                 */
static	char	msg[80];	/* buffer for error messages */

static	long	ishape[9] = {0, 1, 1, 0, 0, 0, 0, 0, 0},
				/* Stats data descriptor for # classes        */
		shape[9] = {0, 1, 100, 0, 0, 0, 0, 0, 0};
				/* Stats data descriptor for names            */
static	long	opt[3]={0,0,0}; /* optional parameter array for EOPENS        */

long	access,			/* Type of access for files                   */
	bufsiz,			/* Size of image I/O buffer                   */
	ierr,			/* Return status from stat I/O                */
	ind, next,		/* Loop counter on classes                    */
	k,			/* A class index                              */
	nc,			/* Number of classes                          */
	odtype,			/* Output image data type in program          */
	status;			/* Return status                              */
double  y_inc, x_inc;

shape[0]  = EBYTE;
ishape[0] = ELONG;
/*
   Set up output image files
   ------------------------- */
access = IWRITE;
odtype = EBYTE;

status = c_eopens (&fdesc[nmrimg], hosout[0], &one,
                   &one, &one, &wind[0][NL], &wind[0][NS],
                   &offpntr, &access, &odtype, &zero, opt);
if (status != E_SUCC)
   iso_err ("!Error opening output image file", "bayes-open");
strcpy(out[num_files], hosout[0]); 
num_files++;

status = c_eopens (&fdesc[nmrimg + 1], hosout[1], &one,
                 &one, &one, &wind[0][NL], &wind[0][NS],
                 &outpntr, &access, &odtype, &zero, opt);
if (status != E_SUCC)
   iso_err ("!Error opening output image file", "bayes-open");
strcpy(out[num_files], hosout[1]); 
num_files++;

/* update DDR
---------- */
y_inc = 1;
x_inc = 1;
c_upddr(&vblock, hosin, hosout[0], &nmrimg, wind, bands, &nbands[0], 
        &y_inc, &x_inc, &one, &one);

/* Set up access group for input and output
   ---------------------------------------- */
fdesc[nmrimg + 2] = 0;
if ((thebuf = (char *)c_ealloc (fdesc, &bufsiz)) == NULL)
   iso_err ("!Error allocating dynamic memory", "bayes-alloc");

if (c_egroup (fdesc, &gdesc, thebuf, &bufsiz) != E_SUCC)
   iso_err ("$", "");
/*
   Set up output stats file for histograms
   --------------------------------------- */
access = IWRITE;
ierr = c_stopen (&sdesc[1], hosout[2], &access, &nc);
if (ierr != E_OPNEW)
   iso_err ("!Error opening output statistics file", "bayes-open");
strcpy(out[num_files], hosout[2]);
num_files++;

ierr = c_stput (&sdesc[1], "NAME1", &zero, &zero, shape, lasout[0]);
if (ierr != E_DTNEW)
   iso_err ("!Error writing to output statistics file", "bayes-write");
ierr = c_stput (&sdesc[1], "NAME2", &zero, &zero, shape, lasout[1]);
if (ierr != E_DTNEW)
   iso_err ("!Error writing to output statistics file", "bayes-write");

shape[2] = (long) strlen (name1);
ierr = c_stput (&sdesc[1], "NAME", &one, &zero, shape, name1);
if (ierr != E_NOCL)
   iso_err ("!Error writing to output statistics file", "bayes-write");
if (!chi)
   {
   shape[2] = (long) strlen (name2);
   ierr = c_stput (&sdesc[1], "NAME", &two, &zero, shape, name2);
   if (ierr != E_NOCL)
      iso_err ("!Error writing to output statistics file", "bayes-write");
   }
else
   {
   shape[2] = (long) strlen (name4);
   ierr = c_stput (&sdesc[1], "NAME", &two, &zero, shape, name4);
   if (ierr != E_NOCL)
      iso_err ("!Error writing to output statistics file", "bayes-write");
   }
shape[2] = (long) strlen (name3);
ierr = c_stput (&sdesc[1], "NAME", &three, &zero, shape, name3);
if (ierr != E_NOCL)
   iso_err ("!Error writing to output statistics file", "bayes-write");

if (mask)
   ind = nclass + 1;
else 
   ind = nclass;
ierr = c_stput (&sdesc[1], "NCLASS", &three, &zero, ishape, &ind);
if (ierr != E_DTNEW)
   iso_err ("!Error writing to output statistics file", "bayes-write");

if (mask)
   {
   shape[2] = (long) strlen (name5);
   ierr = c_stput (&sdesc[1],"NAME",&three,&one,shape,name5);
   if (ierr == E_GNLIO)
      iso_err ("!Error writing to output statistics file", "bayes-write");
   }
/*
   Read each class name from input stats file and write out to new file
   -------------------------------------------------------------------- */
for (ind = 0; ind < nclass; ind++)
   {
   k = clind[ind];
   ierr = c_stget (&sdesc[0], "NAME", &k, &zero, shape, lname);
   if (ierr != E_SUCC)
     {
     sprintf(msg,"!No class name present in statistics file for class # %d", k);
     iso_err (msg, "bayes-noclass");
     }
   if (mask)
      next = ind + 2;
   else
      next = ind + 1;
   ierr = c_stput (&sdesc[1], "NAME", &three, &next, shape, lname);
   if (ierr == E_GNLIO)
      iso_err ("!Error writing to output statistics file", "bayes-write");
   }
c_stclse (&sdesc[0]);
return;
}
