/* include files, global parameters, and definitions for BDIST      */

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

#define TERM 0
#define LP   1
#define FIL 2

struct	PARBLK	block;                 /* TAE parameter block     */

FILE	*fd;                           /* output file descriptor  */

long	num_outfiles;                  /* number of output files  */
 
char	buf[ERRLEN]; 		       /* error message buffer    */	
char    out_files[1][CMLEN];              /* output file names       */

void main
(void)
/*
**********************************************************************

NAME: BDIST

FUNCTION: Calculate BHATTACHARYYA distances between pairs of classes.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	      23-Mar-83  C. Gurney
   2             Apr-85  Y. Sheu     
   3          19-Jun-89  K. Narayanan     Conversion to 5.0
   4 	      24-Jul-91  T. Rockvam	  Changed covariance matrices from float
					  to double to improve accuracy
   5           1-Sep-92  D. Etrheim       Check status of getnam in getpar.c. 
					Change parameter NFEAT to BANDOPT.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get parameters
   Call STATS to get class statistics
   Call COMPUTE to calculate the distances
   Inform user of successful completion

**********************************************************************
*/
{
double	*covar[MAXBND];		/* covariance matrices			*/
float	*mean[MAXBND];		/* mean vectors				*/

long	feat;			/* no. features requested		*/
long	num_bands;		/* no. bands 				*/
long	num_classes;		/* no. classes processed		*/
long	print[4];		/* print option				*/

char	class_names[MAXCLS][STNAME]; /* class names			*/
char	hos_stat[CMLEN];	/* stats file host name			*/
char	print_file[1][CMLEN];	/* print file				*/

long    zero = 0;
long    one = 1;

/*
get tae parameters
------------------
*/
getpar (hos_stat, &num_classes, print, print_file[0], &feat);

/*
retrieve statistics
-------------------
*/
stats (hos_stat, feat, &num_classes, &num_bands, covar, mean, class_names);

/*
calculate distances
-------------------
*/
compute (num_bands, num_classes, feat, class_names, covar, mean, print);

/*
issue completion message
------------------------
*/
if (print[FIL])
    c_complt ("BDIST", print_file, &one);
else
    {  
    c_complt ("BDIST", print_file, &zero);
    }
}
