#include "bdist.h"
#include "util.h"
#include "protos.h"

void output
(
    char clname[][STNAME], /* class names		*/
    float *b_dist,	/* Bhattacharyya distances	*/
    long nc,		/* no. classes			*/
    long fset[],		/* bands in feature set		*/
    long set,		/* set no.			*/
    long numsets,	/* no. feature sets		*/
    long print[]	/* print option			*/
)
/*
**********************************************************************

NAME: OUTPUT

FUNCTION: Output distances to terminal, line printer, or file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	      23-Mar-83  C. Gurney
   2             Apr-85  Y. Sheu     
   3          24-Jul-89  K. Narayanan     Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
long	i, ind, j, k, kk, 
	n, ncol, nc_per_page, nend, npages, nstart, 
	page;

if (set == 1)
   {
/*
   print class names and numbers
   -----------------------------
*/
   c_txtout ("Classes processed:\n", print, fd);
   if (print[TERM]) ncol = 2;
   else ncol = 3;
   for (i = 0; i < nc; i += ncol)
      {
      buf[0] = '\0';
      for (j = 0; j < ncol; j++)
         {
         if ((ind = i + j) >= nc) break;
         sprintf (&buf[strlen (buf)], "%3d. %-32.32s ", ind+1, clname[ind]);
         }
      c_txtout (buf, print, fd);
      }
   }

/*
print distances in a triangular matrix form
-------------------------------------------
*/
sprintf (buf, "\n\nFeature Set %d - Bands: ", set); 
for (j = 0; j < numsets; j++)
   sprintf (&buf[strlen (buf)], "%3d ", fset[j]);
c_txtout (buf, print, fd);
sprintf (buf, "\nAverage Bhattacharyya distance = %10.2f\n", 
         b_dist[nc*(nc-1)/2]);
c_txtout (buf, print, fd);
c_txtout ("Bhattacharyya distances between classes:\n", print, fd);
if (print[TERM]) nc_per_page = 6;
else nc_per_page = 11;
npages = (nc + nc_per_page - 1)/nc_per_page;
for (page = 0; page < npages; page++)
   {
   nstart = page * nc_per_page;
   sprintf (buf, "\n\n      ");
   ncol = (nstart + nc_per_page) > nc ? nc-nstart-1 : nc_per_page; 
   for ( k = 1; k <= ncol; k++)
      sprintf (&buf[strlen (buf)], "%10d ", nstart+k);
   c_txtout (buf, print, fd);
   sprintf (buf, "     ");
   i = (long) strlen (buf);
   for ( k = i; k < 11*ncol+i; k++) buf[k] = '-';
   buf[k] = '\0';
   c_txtout (buf, print, fd);

   for (k = nstart+1; k < nc; k++)
      {
      sprintf (buf, "%3d | ", k+1);
      kk = k*(k-1)/2;
      nend = k < nstart+ncol ? k : nstart+ncol;
      for (n = nstart; n < nend; n++)
         sprintf (&buf[strlen (buf)], "%10.2f ", b_dist[kk+n]);
      c_txtout (buf, print, fd);
      }      
   }
}
