#include "bdist.h"
#include "protos.h"

void stats
(
    char hosnm[],	/* stats file host name			*/
    long feat,		/* no. features requested		*/
    long *nc,		/* no. classes processed		*/
    long *num_bands,	/* no. bands 				*/
    double *covar[],	/* covariance matrices			*/
    float *mean[],	/* mean vectors				*/
    char clname[][STNAME] /* class names			*/
)
/*
**********************************************************************

NAME: STATS

FUNCTION: Gets class names from file/user, and covariances & means 
          from stats file & checks them

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	      23-Mar-83  C. Gurney
   2             Apr-85  Y. Sheu     
   3           5-Jun-89  K. Narayanan     Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
double	fmean[MAXBND], *fcovar;

double	*covar_buf;		
float	*mean_buf;

long	access, 
	clindex[MAXCLS], 
	cols, count,
	i, ic, jval, k,
	lines, 	nband, nclass, nf, nval, 
	shape[9],
	type;

int term;

FILE * fd_sfin;

char	cname[STNAME];

long    fatal_err = -1;
long    inf_err = 1;
long    zero = 0;
long    one = 1;

/*
initialise xt package
open stats file, check # of classes, and get # of bands
-------------------------------------------------------
*/
t_init (&lines, &cols, &type);
access = IREAD;
if (c_stopen (&fd_sfin, hosnm, &access, &nclass) != E_OPOLD)
   c_errmsg ("Error opening statistics file", "bdist-stopen", &fatal_err);

if (nclass < *nc) 
   g_errmsg ("Number of classes in file less than number requested", 
             "bdist-nclass", &fatal_err);

if (c_stget (&fd_sfin, "MEAN_VECTOR", &one, &zero, shape, 
               (unsigned char *) fmean) != E_SUCC) 
    {
    g_errmsg ("Error getting mean vector", "bdist-mean", &fatal_err);
    }

*num_bands = nband = shape[2];
/*
allocate storage for covariances and means
------------------------------------------
*/
nval = nband*(nband+1)/2;

if ((covar_buf = (double *) malloc (MAXCLS * nval * sizeof (double))) == NULL ||
    (mean_buf = (float *) malloc (MAXCLS * nband * sizeof (float))) == NULL ||
    (fcovar = (double *) malloc (nband * (nband + 1) / 2 * sizeof (double)))
            == NULL)
   g_errmsg ("Error allocating dynamic memory", "bdist-alloc",
             &fatal_err);

for (i = 0; i < MAXCLS; i++)
   {
   covar[i] = &covar_buf[i*nval];
   mean[i] = &mean_buf[i*nband];
   }

/*
get dimensions of covar matrix in stats file
--------------------------------------------
*/
if (feat > 0) nf=feat*(feat+1)/2;
if (*nc == 0) 
   {
   for (k = 0; k < nclass; k++) clindex[k] = k + 1;
   *nc = nclass;
   }
/*
if user is selecting classes then prompt for names
--------------------------------------------------
*/
else 
   {
   sprintf (buf, "%d classes will be processed - ", *nc);
   t_write (buf, T_STDCC);
   for (k=0; k < *nc; k++)
      {
      for (count=0;;)
         {
         sprintf (buf, "Enter name for class %d: ", k+1); 
         t_write (buf, T_PROMPT);
         t_read (cname, &term);
         if (c_stfdcl (&fd_sfin, cname, &ic) == E_SUCC) break;
/*
         if no class found give user 3 attempts to try again
         ---------------------------------------------------
*/
         if (++count < 3)
            {
            jval=3-count;
            sprintf (buf, "No such class. %d more attempts to enter", jval);
            c_errmsg (buf, "bdist-classname", &inf_err);
            }
         else
            {
            g_errmsg ("3 errors in specifying a class name. Restart", 
                      "bdist-class", &fatal_err); 
            }
         }
      clindex[k] = ic;
      }
   }
/*
inform user of number of class statistics being read
----------------------------------------------------
*/
sprintf (buf, "Statistics being read for %d classes", *nc);
c_errmsg (buf, "bdist-read", &inf_err);

/*
get stats from file & check dimensions
--------------------------------------
*/
for (k = 0; k < *nc; k++)
   {
   ic = clindex[k];
   if (c_stget (&fd_sfin, "MEAN_VECTOR", &ic, &zero, shape,
                (unsigned char *) fmean) != E_SUCC)
      {
      sprintf (buf, "Error getting mean vector for class %d", k);
      g_errmsg (buf, "bdist-mean", &fatal_err);
      }
   nband = shape[2];
   if (feat > nband)
      {
      sprintf (buf, "Mean has too few bands for class # %d", k);
      g_errmsg (buf, "bdist-meansiz", &fatal_err);
      }

   for (i = 0; i < nband; i++) mean[k][i] = fmean[i];

   if (c_stget (&fd_sfin, "COVARIANCE_MATRIX", &ic, &zero, shape, 
                (unsigned char *) fcovar) != E_SUCC)
      {
      sprintf (buf, "Error getting covariance matrix for class %d", k);
      g_errmsg (buf, "bdist-cov", &fatal_err);
      }
   if (feat > 0 && shape[2] < nf)
      {
      sprintf (buf, "Covariance matrix has too few bands for class # %d", k);
      g_errmsg (buf, "bdist-covsiz", &fatal_err);
      }
   for (i = 0; i < shape[2]; i++) covar[k][i] = fcovar[i];

   c_stget (&fd_sfin, "NAME", &ic, &zero, shape, (unsigned char *) clname[k]);
   }  

/*
close stats file
----------------
*/
c_stclse (&fd_sfin);
}
