#include <stdio.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "protos.h"

struct  PARBLK  block;                 /* TAE parameter block     */

FILE     *fd;

float   *a_matrix;                     /* dynamic storage for     */
float   *p_matrix;                     /* matrices                */
float   *w_matrix;

long    fdesc[MAXIMG];                 /* file descriptors        */
long    gdesc;                         /* group descriptor        */
long    ipoint[MAXIMG * MAXBND + 1],   /* offsets into image i/o  */
                opoint[MAXBND];        /* for input and output bands */
long     min_max;                       /* minmax option flag      */
long    msgtim;                        /* message interval        */
long    num_outfiles;                  /* number of output files  */

unsigned char   *iobuf;                /* image i/o buffer        */
char    out_files[2][CMLEN];           /* output file names       */

extern long     curent;                /* parameters for processing */
extern long     curcat;                /* message utility         */

void main
(void) 

/*
**********************************************************************

NAME:	CANAL

PURPOSE: Performs canonical analysis/transformation

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  1.0         02/25/83  K. A. Narayanan
  2.0         12/05/88  K. A. Narayanan    Conversion to 5.0
  3.0         03/23/89  K. A. Narayanan    Changed upddr flag
  4.0	      09/24/92	T.    Mittan	   Used c_stclse to close stat file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get parameters
   Call DYN_ALLOC to allocate storage for covariance and other matrices
   Call GETSTAT to get the transformation matrix if available or 
      otherwise compute within- and between-class covariance matrices
   IF transformation matrix is to be computed
      Call GETMAT to compute the transformation matrix
      Call TRANSTAT to copy the input statistics file to the output
         and transform the class and site mean and covariance statistics
   IF images have to be transformed 
      Call FILSPEC to set up I/O file specifications
      Call TRANSIMG to perform the transformation and output the images
      If history was requested
         Call HSTOUT to write out history to output image
   Call DYN_FREE to free dynamic storage
   Call COMPLT to send completion message to terminal

ALGORITHM REFERENCES:

1.  M. S. Bartlett, Further Aspects of the Theory of Multiple
    Regression," Proc. Cambridge Philanthropic Society, 34, 1938.

2.  H. L. Seal, Multivariate Statistical Analysis for Biologists,
    Methuer and Co. Ltd., London, 1964.

3.  D. F. Morrison, Multivariate Statistical Methods, McGraw-Hill,
    New York, N.Y., 1967.

4.  M. S. Bartlett, "Multivariate Analysis," Journal Royal
    Statistical Society, Supplement 9.

**********************************************************************
*/
{
double	amax[MAXBND],            /* input image maximum values       */
	amin[MAXBND];            /* input image minimum values       */

float	*a[MAXBND],              /* pointers to eigen vectors        */
	*p[MAXBND],              /* pointers to eigen vectors        */
	*w[MAXBND];              /* pointers to eigen vectors        */

FILE *fd_sfin;                 /* input stats file descriptor      */
FILE *fd_sfout;                /* input stats file descriptor      */
static	long image_band_num[MAXIMG][MAXBND + 1];
                                 /* band numbers in each input image */
long 	itrns,                   /* transformation matrix availability */
	nclass,                  /* number of classes                */
	nsamp,                   /* number of samples                */
	num_image_bands[MAXIMG], /* number of bands in each input image  */
	num_inbands,             /* total number of input image bands*/
	num_inimages,            /* number of input images           */
	num_outbands,            /* number of bands in output image  */
	num_outcomps,            /* max possible output components   */
	out_type,                /* output image data type           */
	print[3],                /* print parameter option           */
	wind[MAXIMG][4];         /* input image windows              */

static	char hos_in[MAXIMG][CMLEN];
                                 /* input images host names          */
char	hos_out[1][CMLEN],          /* output image host name           */
	las_in[MAXIMG][CMLEN],   /* input images las names           */
	las_out[1][CMLEN],          /* output image las name            */
	las_stout[CMLEN],        /* output stats file las name       */
	print_file[CMLEN];       /* output print file name           */

/*
get parameters
--------------
*/

getpar (las_in, las_stout, hos_in, las_out, hos_out, 
        &num_inimages, num_image_bands, image_band_num, &num_inbands, 
	&itrns, &nclass, &num_outbands, wind, &out_type, &fd_sfin,
        &fd_sfout, print, print_file);

/*
allocate dynamic storage for covariance and transformation matrices 
and get pointers to their rows
--------------------------------------------------------------------
*/
dyn_alloc( num_inbands, p, w, a);

/*
get transformation matrix/statistics
------------------------------------
*/
getstat (a, num_inbands, num_outbands, &num_outcomps,
         p, w, nclass, &nsamp, itrns, print, fd_sfin, fd_sfout);

if (itrns  !=  0) 
   {
/*
   compute transformation matrix if needed
   ---------------------------------------
*/
   getmat (p, w, nclass, nsamp, num_inbands, &num_outcomps, 
           a, print);

/*
   and output transformed statistics
   ---------------------------------
*/
   transtat (a, num_inbands, nclass, num_outcomps, fd_sfout);
   }  

/*
output transformed images if needed
-----------------------------------
*/
if (num_outbands  !=  0) 
   {
/*
   set up files for input and output images
   ----------------------------------------
*/
   filspec (hos_in, hos_out[0], &num_inimages, num_image_bands, 
            image_band_num, num_outbands, &num_outcomps, wind, amax, amin,
            out_type);

/*
   perform the transformation and output the images
   ------------------------------------------------
*/
   transimg (hos_out[0], &num_inbands, &num_outcomps, wind, 
             a, amax, amin, out_type);

}

/*
free storage, update output image history, and terminate processing
-------------------------------------------------------------------
*/
terminate (las_out[0], hos_out[0], hos_in, &num_inimages, num_outbands, 
                    print, print_file, itrns, las_stout);
}
