#include "canal.h"
#include "protos.h"

void donode
(
    long class_index,
    long site_index,
    FILE *fd_in,         /* input stats file descriptor   */
    FILE *fd_out        /* output stats file descriptor  */
)
/*
**********************************************************************

NAME: DONODE                           

PURPOSE: Copies needed data types from input to output statistics file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         08/17/88  K. A. Narayanan    Conversion to 5.0
  1.0         12/24/84  S.Ross 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   If the current node is a class node 
      Get the input class name
      Write the class name to the output file
   If the current node is a site level node
      Get the input site name
      Write the site name to the output file
   For all valid nodes at the current level:
      Get the data type of the current node
      If the data type is a Polygon or
         the Number of points or 
         the Mean vector or
         the Prior probability or
         the Covariance matrix or
         the Correlation matrix
            Get the input data
            Write the data to the output file

**********************************************************************
*/
{
double	*tcov, tmean[MAXBND]; 

float	prior;

long	id, k;
long	npoints, poly[400], shape[9] ;
long	node_stat, stat;

char	*data_type, data_type_buf[80];

unsigned char name[80];

tcov = 0;

/*
----------------------------------------------------------------------
 if necessary, copy the current class node
----------------------------------------------------------------------
*/
data_type = "NAME";

if (site_index == 0  &&  class_index != 0) 
   {
   stat = c_stget (&fd_in, data_type, &class_index, &site_index, shape, name); 
   if (stat != E_SUCC) serrmsg ("class name");
   stat = c_stput (&fd_out, data_type, &class_index, &site_index, shape, name); 
   if (stat != E_NOCL) serrmsg ("class name");
   }  
/*
----------------------------------------------------------------------
 copy the current site node
----------------------------------------------------------------------
*/
if (site_index != 0) 
   {
   stat = c_stget (&fd_in, data_type, &class_index, &site_index, shape, name); 
   if (stat != E_SUCC) serrmsg ("site name"); 
   stat = c_stput (&fd_out, data_type, &class_index, &site_index, shape, name); 
   if (stat != E_NOSI) serrmsg ("site name"); 
   }  
/*
----------------------------------------------------------------------
  loop over stat nodes, copying needed data types
----------------------------------------------------------------------
*/
id = 1;
node_stat = 0;
while (node_stat == 0)
   {
   for (k = 0; k < 9; k++)
      shape[k] = 0;
   node_stat = c_stnode (&fd_in, &class_index, &site_index, &id, 
                         data_type_buf, shape); 
   id++;
/*
   if data exists then
   -------------------
*/
   if (node_stat == 0) 
      {
/*
      copy covariance matrix
      ----------------------
*/
      if (strncmp (data_type_buf, "COVARIANCE", 7) == 0)
         {
         if ((tcov = (double *) malloc (shape[2] * sizeof (double))) == NULL)
            {
            if (fd) fclose (fd);
            c_abort (&block, "Error allocating dynamic memory", "canal-alloc", 
            prog_name, out_files, &num_outfiles);
            }
         data_type = "COVARIANCE_MATRIX";
         stat = c_stget(&fd_in, data_type, &class_index, &site_index, shape, 
                        (unsigned char *) tcov); 
         if (stat != E_SUCC) serrmsg ("covariance matrix");
         stat = c_stput (&fd_out, data_type, &class_index, &site_index, shape, 
                       (unsigned char *) tcov); 
         if (stat != E_DTNEW) serrmsg ("covariance matrix");
         }
/*
      copy mean vector
      ----------------
*/
      else if (strncmp (data_type_buf, "MEAN_VECTOR", 7) == 0)
         {
         data_type = "MEAN_VECTOR";
         stat = c_stget (&fd_in, data_type, &class_index, &site_index, shape, 
                         (unsigned char *) tmean); 
         if (stat != E_SUCC) serrmsg ("mean vector");
         stat = c_stput (&fd_out, data_type, &class_index, &site_index, shape, 
                         (unsigned char *) tmean); 
         if (stat != E_DTNEW) serrmsg ("mean vector");
         }
/*
----------------------------------------------------------------------
 copy npoints
----------------------------------------------------------------------
*/
      else if (strncmp (data_type_buf, "NPOINTS", 7) == 0)
         {
         data_type = "NPOINTS";
         stat = c_stget(&fd_in, data_type, &class_index, &site_index, shape, 
                        (unsigned char *) &npoints); 
         if (stat != E_SUCC) serrmsg ("npoints");
         stat = c_stput (&fd_out, data_type, &class_index, &site_index, shape, 
                         (unsigned char *) &npoints); 
         if (stat != E_DTNEW) serrmsg ("npoints");
         }
/*
----------------------------------------------------------------------
 copy polygon
----------------------------------------------------------------------
*/
      else if (strncmp (data_type_buf, "POLYGON", 7) == 0)
         {
         data_type = "POLYGON";
         stat = c_stget(&fd_in, data_type, &class_index, &site_index, shape, 
                        (unsigned char *) poly); 
         if (shape[2] > 200  ||  stat != 0) serrmsg ("polygon");
         stat = c_stput (&fd_out, data_type, &class_index, &site_index, shape, 
                         (unsigned char *) poly); 
         if (stat != E_DTNEW) serrmsg ("polygon");
         }
/*
----------------------------------------------------------------------
 copy prior probability
----------------------------------------------------------------------
*/
      else if (strncmp (data_type_buf, "PRIOR_P", 7) == 0)
         {
         data_type = "PRIOR_PROBABILITY";
         stat = c_stget(&fd_in, data_type, &class_index, &site_index, shape, 
                        (unsigned char *) &prior); 
         if (stat != E_SUCC) serrmsg ("prior_probability");
         stat = c_stput (&fd_out, data_type, &class_index, &site_index, shape, 
                         (unsigned char *) &prior); 
         if (stat != E_DTNEW) serrmsg ("prior_probability");
         }
      }  
   }  

if (tcov != 0) free (tcov);

}
