
#include "canal.h"
#include "protos.h"

void filspec
(
    char hos_in[][CMLEN],     /* input image host names             */
    char hos_out[],           /* output image host name             */
    long *num_inimages,        /* number of input images             */
    long num_image_bands[],   /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each      */
    long num_outbands,        /* number of output image bands       */
    long *num_outcomps,        /* max possible output bands          */
    long wind[][4],           /* input image windows                */
    double amax[],              /* input images maximum values        */
    double amin[],              /* input images minimum values        */
    long out_type            /* output image data type             */
)
/*
**************************************************************************

NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  1.0         02/25/83  K. A. Narayanan
  2.0         08/17/88  K. A. Narayanan
  3.0         03/23/89  K. A. Narayanan    Changed upddr flag
  6.1	      05/24/94  D. J. Eichacker    Added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   If output data type is BYTE
      Get or computr minimum and maximum values of input image
   Open input and output images
   Group the images

**************************************************************************
*/
{
double	line_inc = 1.0,              /* line and sample increments for     */
	samp_inc = 1.0;              /* output image ddr                   */

long	access,                      /* file access type                   */
	comb_flag = 0,               /* image combination flag             */
	bufsiz,                      /* image i/o buffer size              */
	n, nib,
	one = 1,                     /* constant one                       */
	stat,                        /* return status                      */
	type,                        /* file type                          */
	zero = 0;                    /* constant zero                      */

static	long	opt[3]={0,0,0};      /* option array for c_eopens          */

char	buf[ERRLEN];

if (*num_outcomps < num_outbands) 
   {
   sprintf (buf, "Only the first %d bands are output", *num_outcomps);
   c_errmsg (buf, "canal-outbands", &inf_err);
   }
else
   {
   *num_outcomps = num_outbands;
   }  

/*
get maximum and minimum pixel values of input images
----------------------------------------------------
*/
if (out_type == 1)
   c_minmax (hos_in, num_inimages, wind, image_band_num, num_image_bands, 
             amin, amax);
/*
open input and output image files
---------------------------------
*/
access = IREAD;
type = EREAL;
for (n = 0, nib = 0; n < *num_inimages; n++)
   {
   stat = c_eopens( &fdesc[n], hos_in[n], image_band_num[n], &wind[0][SL], 
                   &wind[0][SS], &wind[0][NL], &wind[0][NS], &ipoint[nib], 
                   &access, &type, &zero, opt);
   if (stat != E_SUCC )
      {
      if (fd) fclose (fd);
      c_abort (&block, "Error opening input image", "canal-open", prog_name,
               out_files, &num_outfiles);
      }

   nib += num_image_bands[n];
   }  

access = IWRITE;
if (out_type == 1) 
   type = EBYTE;
else 
   type = EREAL;
stat = c_eopens(&fdesc[n], hos_out, num_outcomps, &one, &one,
               &wind[0][NL], &wind[0][NS], opoint, &access, &type, 
               &zero, opt);
if (stat != E_SUCC )
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error opening output image", "canal-outopen", prog_name,
               out_files, &num_outfiles);
   }

c_upddr (&block, hos_in, hos_out, num_inimages, wind, image_band_num, 
         num_image_bands, &line_inc, &samp_inc, num_outcomps, &comb_flag);

strcpy (out_files[num_outfiles++], hos_out);

fdesc[++n] = 0;

/*
set up group containing input and output bands
----------------------------------------------
*/
if ((iobuf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error allocating dynamic memory", "canal-alloc", prog_name,
            out_files, &num_outfiles);
   }

c_egroup (fdesc, &gdesc, iobuf, &bufsiz);
}
