#include <string.h>
#include "canal.h"
#include "protos.h"

void getpar
(
    char las_in[][CMLEN],     /* input image las names            */
    char las_stout[],         /* output stats file las name       */
    char hos_in[][CMLEN],     /* input image host names           */
    char las_out[][CMLEN],    /* output image las name            */
    char hos_out[][CMLEN],    /* output image host name           */
    long *num_inimages,       /* number of input images           */
    long num_image_bands[],   /* number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* band numbers in each    */
    long *num_inbands,        /* total number of input image bands   */
    long *itrns,              /* transformation matrix option     */
    long *nclass,             /* no. classes                      */
    long *num_outbands,       /* number of output image bands     */
    long wind[][4],           /* input image windows              */
    long *out_type,           /* output image data type           */
    FILE **fd_sfin,            /* input stats file descriptor      */
    FILE **fd_sfout,           /* output stats file descriptor     */
    long print[],             /* print parameter option           */
    char print_file[]        /* output file                        */
)
/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         12/05/88  K. A. Narayanan    Conversion to 5.0
  1.0         02/25/83  K. A. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Initialize program and start the CPU clock
   Get input statistics file name
   Get number of output bands OUTBANDS requested
   Get matrix avilability indicator
   Get output data type indicator
   If transformation matrix is to be computed 
      Get output statistics file name
      If no output statistics file is specified 
         issue error message and abort
      Open output statistics file
   ELSE 
      If OUTBANDS <= 0 issue error message and abort
   Open input stats file
   Get number of bands in statistics file
   If OUTBANDS = 0 
      Set number of input bands INBANDS = number of bands in stats file
   Else
      Get input image parameters
      If number of input bands INBANDS is < 2 issue error message and abort
      If number of bands from stats file is not the same INBANDS 
         issue error message and abort
      If OUTBANDS > INBANDS inform user and set OUTBANDS=INBANDS
      Get output image name
   Get output destination for statistics

***********************************************************************
*/
{
struct  VARIABLE *var;               /* pointer to a variable in las &block */

long	access,                     /* access type for stats file         */
	dum_nclass,
	flag[MAXIMG][2],            /* flag for subwindows                */
	imglimit = MAXIMG,          /* maximum number of images           */
	m[MAXBND],                  /* mean vector                        */
	mshape[9],                  /* mean shape vector                  */
	one = 1,
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	stat_nbands,                /* no. bands in stats file            */
	std_out,                    /* logical unit for standard output   */
	zero = 0;

char	buf[ERRLEN],
	cnam[80],
	fileutil_file[CMLEN],       /* file for fileutil                  */
	hos_stin[CMLEN],            /* input stats file host name         */
	hos_stout[CMLEN],           /* output stats file host name        */
	minmax[4],                  /* minmax option                      */
	str[8],                     /* temporary string                   */
	trmat[6];                   /* transformation matrix option       */


/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get input statistics file name
------------------------------
*/
access = IREAD;
stat = c_getnam (&block, "INSTAT", &zero, hos_stin, "stat", &access);
if (stat != E_SUCC) 
   c_errmsg ("Error getting input statistics file name", "canal-sfin", &fatal_err);

var = p_find (&block, "OUTBANDS");
*num_outbands = IVAL(*var, 0);

var = p_find (&block, "MATRXOPT");
strcpy (trmat, SVAL(*var, 0));

*itrns = 0;
if (trmat[0]  ==  'C')
   {
   *itrns = 1;
/*
   get output statistics file name and open file
   ---------------------------------------------
*/
   access = IWRITE;

   stat = c_getnam (&block, "OUTSTAT", &zero, hos_stout, "stat", &access);
   if (stat != E_SUCC || strlen (hos_stout) == 0) 
      c_errmsg ("Error getting output statistics file name", "canal-sfout", 
                &fatal_err);
   strcpy(las_stout, hos_stout);

   stat = c_stopen (fd_sfout, hos_stout, &access, &dum_nclass); 
   if (stat != E_OPNEW)
      c_errmsg ("Error opening output statistics file", "canal-sfoutopen", 
                &fatal_err);
   }

else 
   if (*num_outbands  <=  0) 
      c_errmsg ("Zero number of output bands specified for transformation",
               "canal-nobands", &fatal_err);


/*
get number of bands for elements in input stats file
----------------------------------------------------
*/
access = IREAD;
stat = c_stopen (fd_sfin, hos_stin, &access, nclass); 

if (stat !=  E_OPOLD)
   c_errmsg ("Error opening input statistics file", "canal-sfinopen", &fatal_err);

stat = c_stget (fd_sfin, "MEAN_VECTOR", &one, &zero, mshape,
                   (unsigned char *) m); 
if (stat !=  E_SUCC) 
   {
   stat = c_stget (fd_sfin, "NAME", &one, &zero, mshape,
                      (unsigned char *) cnam); 
   sprintf (buf, "Mean vector for class %s not found in statistics file",
            cnam);
   c_errmsg (buf, "canal-nomean", &fatal_err);
   }  
stat_nbands = mshape[2];

if (*num_outbands  ==  0) 
   *num_inbands = stat_nbands;
else
   {
/*
   get and validate input image parameters
   ---------------------------------------
*/
   stat = c_getin (&block, "IN", &zero, las_in, hos_in, num_inimages, wind, 
                   image_band_num, num_image_bands, num_inbands, &imglimit, 
                   &same_window, flag);

   if (stat != E_SUCC || *num_inimages == 0) 
       c_errmsg ("Error getting input image name", "canal-in", &fatal_err);

   if (*num_inbands  <  2) 
      c_errmsg( "No. of input bands should be greater than 1", "canal-ibands", 
                 &fatal_err);

   if (same_window == 0)
       c_errmsg ("Image windows not same size", "canal-samewindow", &fatal_err);

   if (*num_outbands  >  *num_inbands) 
      {
      c_errmsg ("OUTBANDS exceeds no. of input bands;", "canal-obands1", 
                &inf_err);
      sprintf (buf, "reset to %d", *num_inbands);
      c_errmsg (buf, "canal-obands2", &inf_err);
      *num_outbands = *num_inbands;
      }  
/*
   check stats file dimensions against no. of input image bands
   ------------------------------------------------------------
*/
   if (stat_nbands  !=  *num_inbands)
      { 
      sprintf (buf, "Stats file dimension  = %d;", stat_nbands);
      c_errmsg (buf, "canal-sfdim1", &inf_err);
      c_errmsg ("does not match number of bands in input image",
               "canal-sfdim2", &fatal_err);
      }
/*
   get output image name and data type
   -----------------------------------
*/
   stat = c_getout (&block, "OUT", las_out, hos_out);
   if (stat != E_SUCC || strlen (las_out[0]) == 0)
      c_errmsg("Error getting output image name", "canal-out", &fatal_err);

   var = p_find (&block, "ODTYPE");
   strcpy (str, SVAL(*var, 0));
   *out_type = (str[0] == 'B' ? 1 :4);
   }  
/*
Get print option and open print file if necessary
-------------------------------------------------
*/
strcpy (fileutil_file, prog_name);
stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
          print_file);
if (stat != E_SUCC)
   c_errmsg("Fatal error encountered","canal-fatal",&fatal_err);

if ((print[LP]) || (print[FIL]))
   fd = fopen (fileutil_file, "w");
else
   fd = NULL;

/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y' || minmax[0] == 'y') min_max = TRUE;
else min_max = FALSE;

}
