#ifndef PROTOS_H
#define PROTOS_H

void donode
(
    long class_index,
    long site_index,
    FILE *fd_in,         /* input stats file descriptor   */
    FILE *fd_out        /* output stats file descriptor  */
);

void dyn_alloc
(
    long num_inbands,       /* number of input image bands    */
    float *p[],            /* pointers to covariance matrix  */
    float *w[],
    float *a[]        /* pointers to eigen vectors      */
);

void dyn_free
(void);

void egsqrt
(
    float a[],
    long n
);

void filspec
(
    char hos_in[][CMLEN],     /* input image host names             */
    char hos_out[],           /* output image host name             */
    long *num_inimages,
    long num_image_bands[],   /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each      */
    long num_outbands,        /* number of output image bands       */
    long *num_outcomps,        /* max possible output bands          */
    long wind[][4],           /* input image windows                */
    double amax[],              /* input images maximum values        */
    double amin[],              /* input images minimum values        */
    long out_type             /* output image data type             */
);

void getmat
(
    float *p[],        /* between-class covariance matrix */
    float *w[],        /* within-class covariance matrix  */
    long nclass,      /* no. of classes                  */
    long nsamp,       /* total no. of samples            */
    long ibands,      /* no. of input image bands        */
    long *ncomp,      /* max no. of output components    */
    float *a[],        /* transformation matrix           */
    long print[]     /* output destination for info     */
);

void getpar
(
    char las_in[][CMLEN],     /* input image las names            */
    char las_stout[],         /* output stats file las name       */
    char hos_in[][CMLEN],     /* input image host names           */
    char las_out[][CMLEN],    /* output image las name            */
    char hos_out[][CMLEN],    /* output image host name           */
    long *num_inimages,       /* number of input images           */
    long num_image_bands[],   /* number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* band numbers in each    */
    long *num_inbands,        /* total number of input image bands   */
    long *itrns,              /* transformation matrix option     */
    long *nclass,             /* no. classes                      */
    long *num_outbands,       /* number of output image bands     */
    long wind[][4],           /* input image windows              */
    long *out_type,           /* output image data type           */
    FILE **fd_sfin,            /* input stats file descriptor      */
    FILE **fd_sfout,           /* output stats file descriptor     */
    long print[],             /* print parameter option           */
    char print_file[]        /* output file                        */
);

void getstat
(
    float *a[],       /* transformation matrix              */
    long ibands,     /* no. of input image bands           */
    long obands,	    /* no. of output images desired       */
    long *ncomp,     /* no. of bands actually output       */
    float *p[],       /* between-class covariance matrix    */
    float *w[],       /* within-class covariance matrix     */
    long nclass,     /* no. of classes                     */
    long *nsamp,     /* total no. of samples               */
    long itrns,      /* matrix availability indicator      */
    long print[],    /* destination for info output        */
    FILE *fd_sfin,    /* input statistics file descriptor   */
    FILE *fd_sfout   /* output statistics file descriptor  */
);

void newcorel
(
    long ibands,       /* number of input image bands     */
    double tcov[],       /* covariance matrix               */
    long class_index,  /* class index                     */
    long site_index,   /* site index                      */
    FILE * fd_sf        /* stats file descriptor           */
);

void serrmsg
(
    char str[]              /* message to be displayed         */
);

void stat_copy
(
    FILE * fd_in,               /* input statistics file            */
    FILE * fd_out,              /* output statistics file           */
    long nclass              /* number of classes                */
);

void terminate
(
    char las_out[],              /* output image las name         */
    char hos_out[],              /* output image host name        */
    char hos_in[][CMLEN],        /* input images host names       */
    long *num_inimages,          /* number of input images        */
    long num_outbands,           /* number of output bands        */
    long print[],                /* output destination            */
    char print_file[],           /* output file name              */
    long itrns,
    char las_stout[]            /* output stats file las name    */
);

void transimg
(
    char hos_out[],           /* output image host name              */
    long *num_inbands,        /* total no. of input bands            */
    long *num_outbands,       /* total no. of output bands           */
    long wind[][4],           /* input image windows                 */
    float *a[],                /* eigen vectors                       */
    double amax[],              /* input image maximum values          */
    double amin[],              /* input image minimum values          */
    long out_type            /* output image data type              */
);

void transt
(
    FILE * fd,        /* stats file descriptor         */
    long class_index,
    long site_index,
    float *a[],      /* transformation matrix         */
    long ibands,    /* no. input image bands         */
    long ncomp     /* no. output components         */
);

void transtat
(
    float *a[],       /* transformation matrix         */
    long ibands,     /* no. of input image bands      */
    long nclass,     /* no. of classes                */
    long ncomp,      /* no. of output components      */
    FILE * fd_s       /* statistics file descriptor    */
);

#endif

