#include "canal.h"
#include "pixman.h"
#include "protos.h"

void transimg
(
    char hos_out[],           /* output image host name              */
    long *num_inbands,        /* total no. of input bands            */
    long *num_outbands,       /* total no. of output bands           */
    long wind[][4],           /* input image windows                 */
    float *a[],                /* eigen vectors                       */
    double amax[],              /* input image maximum values          */
    double amin[],              /* input image minimum values          */
    long out_type            /* output image data type              */
)

/*
**************************************************************************
NAME: TRANSIMG

PURPOSE: Perform transformation on input image and generate output
         components

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         08/17/88  K. A. Narayanan    Conversion to 5.0
  1.0         02/25/83  K. A. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   If scaling to byte image is required estimate the output maximum and 
      minimum pixel values from the input maximum and minimum pixel 
      values and the values of the eigen vector matrix
   Compute the scale and offset for the linear scaling which maps
      the estimated output minimum and maximum to 0 and 255 respectively
   Perform transformation line by line:
      If output image is byte perform scaling
      If output minmax is requested update minmax values
   If output minmax is requested save the minmax values in the DDR
   Close image files
      
**************************************************************************
*/
{
float	aij,
	emax,
	emin,
	*ip, *op,                     /* image i/o input and output buffer */
                                      /* pointers                          */
	offset,                       /* offset for output image scaling   */
	rmax,                         /* output image est maximum          */
	rmin,                         /* output image est minimum          */
	scale,                        /* factor for sacling output image   */
	sum;
double	output_max[MAXBND],           /* output image maximum values       */
	output_min[MAXBND];           /* output image minimum values       */

long	i, i1, i2, j, n2,
	line,                         /* line number                       */
	msg_flag = 0;                 /* flag for processing message       */

unsigned char	*cpmax,
		*cpmin;

/*
if output image is byte calculate scale and offsets by estimating
output maximum and minimum values for all bands from input maximum
and minimum values and the transformation matrix

Note that eigen vectors are in ascending order and columnwise
------------------------------------------------------------------
*/
if (out_type  ==  1) 
   {
   rmax = -.1e30;                /*  arbitrary low value  */
   rmin = .1e30;		 /*  arbitrary high value */

   for (i = 0; i < *num_outbands; i++)
      {
      emax = 0.; emin = 0.;

      for (j = 0; j < *num_inbands; j++)
         {
/*       compute estimated output maximum and minimum values
         ---------------------------------------------------
*/
         aij = a[i][j];
         if (aij  >  0.0) 
            {
            emax += aij * amax[j];
            emin += aij * amin[j];
            }
         else if (aij  <  0.0)
            {
            emax += aij * amin[j];
            emin += aij * amax[j];
            }  
         }  
      if (emax  >  rmax) rmax = emax;
      if (emin  <  rmin) rmin = emin;
      }  
/*
   compute scaling factor and offset
   ---------------------------------
*/
   scale = 255. / (rmax-rmin);
   offset = (-rmin) * scale;

   }  
/*
initialize and enable processing message utility
------------------------------------------------
*/
c_inittm ("CANAL", "Processed", "line", "bands", " ", &wind[0][NL], 
         num_outbands, &msgtim, &msg_flag);
curent = 1;
c_startm();

/*
initialize output minimum and maximum arrays if needed
------------------------------------------------------
*/
if (min_max) 
   c_inmnmx( &out_type, num_outbands, output_min, output_max);

/*
process each line
-----------------
*/
for (line = 0; line < wind[0][NL]; line++, curent++)
   {
   c_estep( &gdesc);
   for (n2 = 0; n2 < wind[0][NS]; n2++)
      {
      for (i1 = 0; i1 < *num_outbands; i1++)
         {
         sum = 0.0;
         for (i2 = 0; i2 < *num_inbands; i2++)
            {
	    ip = (float *) & iobuf[(ipoint[i2] + n2) * sizeof( float)];
            sum += a[i1][i2] * (*ip);
            }
/*
         scale output values for byte image
         ----------------------------------
*/
         if(out_type == 1)
            {
            sum = sum * scale + offset;
            if (sum < 0) sum = 0;
            else if (sum > 255) sum = 255;

            iobuf[(opoint[i1] + n2)] = sum;
            }
         else
            {
            op = (float *) &iobuf[(opoint[i1] + n2) * sizeof (float)];
            *op = sum;
            }  
         }
      }
/*
   compute output minimum and maximum values if requested
   ------------------------------------------------------
*/
   if (min_max)
      {
      if (out_type == EBYTE)
         {
         cpmax = (unsigned char *) output_max; 
         cpmin = (unsigned char *) output_min;
         }
      for (i1 = 0; i1 < *num_outbands; i1++)
         {
          if (out_type == EBYTE)
             c_pxmnmx( (long *) &iobuf[opoint[i1]], &wind[0][NS], 
                       (double *) (cpmin+i1),(double *) (cpmax+i1), &out_type);
          else
             c_pxmnmx( (long *) (&iobuf[opoint[i1] * sizeof (float)]), 
                      &wind[0][NS], &output_min[i1],&output_max[i1], &out_type);
         }
      }
   }
c_estep( &gdesc);

if (min_max)
   {
   c_svmnmx( hos_out, num_outbands, output_min, output_max, "YES");
   }
c_stoptm();

/*
close image files 
-----------------
*/
c_egclse( &gdesc);
}

