#include "canal.h"
#include "util.h"
#include "protos.h"

void transt
(
    FILE * fd,        /* stats file descriptor         */
    long class_index,
    long site_index,
    float *a[],      /* transformation matrix         */
    long ibands,    /* no. input image bands         */
    long ncomp     /* no. output components         */
)
/*
******************************************************

NAME: TRANST                           

PURPOSE:
     Transforms the mean and covariance statistics and outputs 
     them to the statistics file

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         08/17/88  K. A. Narayanan    Conversion to 5.0
  1.0         02/25/83  K. A. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

******************************************************
*/
{
double	m[MAXBND], x[MAXBND];
double	*x1, *x2, *c, sum;

long	i, j, k, k1;
long	cshape[9], mshape[9];

char	*data_type;
/*
allocate dynamic storage
------------------------
*/
if ((c = (double *) malloc (ibands*(ibands+1)*sizeof (double)/2)) == NULL ||
    (x1 = (double *) malloc (ibands*ibands*sizeof (double))) == NULL ||
    (x2 = (double *) malloc (ibands*ibands*sizeof (double))) == NULL)
   {
   if (fd) c_stclse (&fd);
   c_abort (&block, "Error allocating dynamic memory", "canal-alloc", 
            prog_name, out_files, &num_outfiles);
   }
/*
transform and output mean vector
--------------------------------
*/
data_type =  "MEAN_VECTOR";
c_stget(&fd, data_type, &class_index, &site_index, mshape,(unsigned char *) m);

for (i = 0; i < ncomp; i++)
   {
   sum = 0.0;
   for (j = 0; j < ibands; j++)
      sum += a[i][j] * m[j];
   x[i] = sum;
   }  

mshape[2] = ncomp;
c_stput(&fd, data_type, &class_index, &site_index, mshape,(unsigned char *) x); 

/*
transform and output covariance statistics
------------------------------------------
*/
data_type = "COVARIANCE_MATRIX";
c_stget (&fd, data_type, &class_index, &site_index, cshape,(unsigned char *) c);

for (i = 0, k = 0; i < ibands; i++)
   {
   for (j = 0; j <= i; j++, k++)
      {
      x1[i*ibands+j] = c[k];
      x1[j*ibands+i] = c[k];
      }  
   }  

for (i = 0; i < ncomp; i++)
   {
   for (j = 0; j < ibands; j++)
      {
      sum = 0.0;
      for (k = 0; k < ibands; k++)
         sum += a[i][k] * x1[k*ibands+j];
      x2[i*ibands+j] = sum;
      }  
   }  

for (i = 0, k1 = 0; i < ncomp; i++)
   {
   for (j = 0; j <= i; j++)
      {
      sum = 0.0;
      for (k = 0; k < ibands; k++)
         sum += x2[i*ibands+k] * a[j][k];
      c[k1++] = sum;
      }  
   }  

cshape[2] = ncomp * (ncomp+1) / 2;
c_stput(&fd, data_type, &class_index, &site_index, cshape,(unsigned char *) c); 
/*
compute correlation matrix based on the transformed covariance matrix
---------------------------------------------------------------------
*/
newcorel (ncomp, c, class_index, site_index, fd);

/*
free dynamclass_index storage
--------------------
*/
free (x1);
free (x2);
free (c);
}
