#include "canal.h"
#include "protos.h"

void transtat
(
    float *a[],       /* transformation matrix         */
    long ibands,     /* no. of input image bands      */
    long nclass,     /* no. of classes                */
    long ncomp,      /* no. of output components      */
    FILE *fd_s       /* statistics file descriptor    */
)
/*
**********************************************************************

NAME: TRANSTAT                           

PURPOSE:
    Transforms the class and site mean and covariance
    statistics and outputs them and the transformation matrix
    to the statistics file

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         08/17/88  K. A. Narayanan   Conversion to 5.0
  1.0         02/25/83  K. A. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Open output statistics file
   If error in open inform user and abort
   Put the transformation matrix in statistics file
   Do for number of classes
      Call TRANST to transform and output the class statistics
      Do for number of sites in the class
         Call TRANST to transform and output the site statistics
   Close output statistics file         

**************************************************************
*/
{
long	class_index, site_index;
long	ns;
long	sshape[9];
long	zero = 0;

/*
put transformation matrix in statistics file
--------------------------------------------
first put number of components
------------------------------
*/
sshape[0] = EREAL;
sshape[1] = 2;
sshape[2] = ncomp;
sshape[3] = ibands;
c_stput (&fd_s, "CANAL_TRANS_MATRIX", &zero, &zero, sshape,
            (unsigned char *) a[0]); 

/*
transform class and site mean and covariance statistics
-------------------------------------------------------
*/
for (class_index = 1; class_index <= nclass; class_index++)
   {
/*
   transform and output class statistics
   -------------------------------------
*/
   transt (fd_s, class_index, 0, a, ibands, ncomp);

/*
   transform and output site statistics
   ------------------------------------
*/
   c_stget (&fd_s, "NSITES", &class_index, &zero, sshape,(unsigned char *)&ns);

   for (site_index = 1; site_index <= ns; site_index++)
      transt (fd_s, class_index, site_index, a, ibands, ncomp);
   }  

/*
close statistics file
---------------------
*/
c_stclse (&fd_s); 

}
