/******************************************************************************
FUNCTION:	OPEN_ASSOC

PURPOSE:	Opens the associated files

ALGORITHM 
  Open the associated header file
  Open the associated annotation file
  Open the associated trailer file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include "las.h"     
#include "util.h"
#include "protos.h"

void open_assoc
(
    char *annot_host,	/* annotation host name  			*/
    char *head_host,	/* descriptor for header host name		*/
    char *trail_host	/* descriptor for trailer host name    		*/
)
{
extern long annot_fd;	/* pointer to annotation file descriptor	*/
extern long head_fd;	/* pointer to header file descriptor  		*/
extern long trail_fd;	/* pointer to trailer file descriptor		*/
extern long out_count;	/* number of output images			*/
extern short assoc_flag;/* Flag for associated files open		*/
extern short andsk;	/* annotation file flag				*/
extern short hdsk;	/* header file flag				*/
extern short tdsk;	/* trailer file flag				*/
extern char out[MAXIMG][CMLEN]; /* the output files created		*/

/*
  Open the associated annotation file
*/
if (andsk)
    {
    annot_fd = creat(annot_host,0644);
    strcpy(out[out_count],annot_host);
    out_count++;
    assoc_flag = TRUE;
    }

/*
  Open the associated header file
*/
if (hdsk)
    {
    head_fd = creat(head_host,0644);
    strcpy(out[out_count],head_host);
    out_count++;
    assoc_flag = TRUE;
    }

/*
  Open the associated trailer file
*/
if (tdsk)
    {
    trail_fd = creat(trail_host,0644);
    strcpy(out[out_count],trail_host);
    out_count++;
    assoc_flag = TRUE;
    }

return;
}

/******************************************************************************
FUNCTION:	CLOSE_ASSOC

PURPOSE:	Closes the associated files

ALGORITHM 
  Close the associated header file
  Close the associated annotation file
  Close the associated trailer file

ALGORITHM REFERENCES	none
*******************************************************************************/
void close_assoc
(void)
{
extern long annot_fd;	/* pointer to annotation file descriptor	*/
extern long head_fd;	/* pointer to header file descriptor  		*/
extern long trail_fd;	/* pointer to trailer file descriptor		*/
extern short assoc_flag;/* Flag for associated files open		*/
extern short andsk;	/* annotation file flag		       	 	*/
extern short hdsk;	/* header file flag				*/
extern short tdsk;	/* trailer file flag				*/

/*
  Close the associated header file
*/
if (hdsk)
    close(head_fd);

/*
  Close the associated annotation file
*/
if (andsk)
    close(annot_fd);

/*
  Close the associated trailer file
*/
if (tdsk)
    close(trail_fd);

assoc_flag = FALSE;

return;
}

/******************************************************************************
FUNCTION:	DELETE_ASSOC

PURPOSE:	Deletes the associated files

ALGORITHM 
  Delete the associated header file
  Delete the associated annotation file
  Delete the associated trailer file

ALGORITHM REFERENCES	none
*******************************************************************************/
void delete_assoc
(
    char *annot_host,	/* annotation host name				*/
    char *head_host,	/* header host name				*/
    char *trail_host	/* trailer host name				*/
)
{
extern short andsk;	/* annotation file flag		        	*/
extern short hdsk;	/* header file flag				*/
extern short tdsk;	/* trailer file flag				*/

/*
  Delete the associated header file
*/
if (hdsk)
    c_delfil(head_host);

/*
  Delete the associated annotation file
*/
if (andsk)
    c_delfil(annot_host);

/*
  Delete the associated trailer file
*/
if (tdsk)
    c_delfil(trail_host);

return;
}
