/******************************************************************************
FUNCTION:	CHECK_PIXEL

PURPOSE:	Determine whether or not the window specified intersects
		the quadrants input based on the starting sample and number
		of samples.  If the window does not intersect the quadrants
		input, the array containing the quadrants to be processed
		is adjusted accordingly. 

ALGORITHM 
  Use the remainder of pixels to determine where the window intersects the quads
      The window is entirely in the right half of the image
	If the image shape is 'u' or 'b'
	  The window intersects only one quad
	else
	  The window intersects the right half only

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tm.h"
#include "protos.h"

void check_pixel
(
    long window[],		/* user specified window			*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    long *ns,		/* number of samples in quad			*/
    long quads[],		/* quadrants to process				*/
    long *nquads,		/* number of quads to process			*/
    long *right_fill	/* number of fill pixels on the right		*/
)
{
extern long com_idx[4];	/* index into comment array			*/

long nonfatal = 1;	/* Non-fatal error code				*/
long remain;		/* number of pixels following the start pixel	*/
			/* excluding any right fill values		*/

char mesg[ERRLEN];	/* information message text			*/

/*
  The number of pixels following the starting pixel (not including any
  right fill
*/
remain = (*ns - window[1]) - *right_fill;
if (remain < 0)
    {
    /*
      The window is entirely in the right half of the image
    */
    if ((*imshape == 'u') || (*imshape == 'b')) 
 	{
	/*
	  The window intersects only one quad
	*/
	sprintf (mesg,"Window intersects only 1 quad.  Processing quad %d",
	    quads[0]+1);
	c_errmsg(mesg,"informational",&nonfatal);
	*imshape = 'q';
	quads[0] = quads[0]+1;
	*nquads = 1;
	com_idx[0] = com_idx[1];
	window[1] = *ns - (TMSAMP - window[1]);
	}
    else
	{
	/*
	  The window intersects the right half only
	*/
	sprintf(mesg,"%s%s","Window intersects only right half.  ",
		"Processing quadrants 2 and 4");
 	c_errmsg(mesg,"tipsp-onehalf",&nonfatal);
	*imshape = 'r';
	quads[0] = 2;
	quads[1] = 4;
	*nquads = 2;
	window[1] = *ns - (TMSAMP - window[1]);
	com_idx[0] = com_idx[1];
	com_idx[1] = com_idx[3];
	}
    }

return;
}
