/******************************************************************************
FUNCTION:	CHECK_VOL

PURPOSE:	Check that the appropriate volume is mounted

ALGORITHM 
  Read the volume directory
  Check that the current quad is the same as the quad number from tape
  If not same quad numbers
      Display an error message that the correct volume was not mounted
      Close and dismount the tape
      Prompt the operator to mount the correct volume
      Read the volume directory of the newly mounted tape
      If the correct volume is still not mounted, print an error message
      and stop the process

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "protos.h"

void check_vol
(
    long *unit,		/* tape unit number				*/
    long *cur_quad,		/* current quad number				*/
    long *ex_quad,		/* quadrant expected to be mounted		*/
    char *sceneid,		/* scene id read from text record		*/
    long *format		/* Image file format (BSQ or BIL)		*/
)
{
extern short tape_flag;	/* Flag for tape open				*/

long status;		/* function return status code			*/
long eof = 0;		/* number of EOF marks to be written		*/
long skipopt = 0;	/* skip zero files on open			*/
long nonfatal = 1;	/* Non-Fatal error code				*/
long mode = HANDLE;	/* tape I/O outputs error messages		*/
long access = TREAD;	/* open tape for read access			*/
long rewopt = REWIND;	/* rewind option				*/
long option = REWOFFL;	/* rewind tape and take it offline		*/

char newmsg[CMLEN];	/* new mount message				*/
char tapeid[CMLEN];     /* tape id					*/
char dens[5];		/* tape density in BPI				*/

strcpy (dens,"0");
/*
  Read the volume directory
*/
status = get_voldir(unit,cur_quad,sceneid,format);
if (status != E_SUCC)
    clean_up(unit,status);

/*
  Check that the current quad is the same as the quad number from tape
*/
if (*cur_quad != *ex_quad)
    {
    /*
      Display an error message that the correct volume was not mounted
    */
    c_errmsg("The current volume does not contain the correct quadrant",
	     "tipsp-wrongvol",&nonfatal);

    /*
      Close and dismount the tape
    */
    status = c_tpclos(unit,&mode,&option,&eof);
    if (status != TAPE_SUCC)
	c_errmsg("Error closing tape","tipsp-tpclose",&nonfatal);

    tape_flag = FALSE;

    /*
      Prompt the operator to mount the correct volume
    */
    sprintf(newmsg,"Please mount tape for quad %d",*ex_quad);
    status = c_tpopen(unit,&skipopt,&mode,&rewopt,dens,&access,newmsg,tapeid);
    if (status != TAPE_SUCC)
	clean_up(unit,status);
	  
    tape_flag = TRUE;

    /*
      Read the volume directory of the newly mounted tape
    */
    status = get_voldir(unit,cur_quad,sceneid,format);
    if (status != E_SUCC)
	clean_up(unit,status);
	  
    /* 
      If the correct volume is still not mounted, print an error message
      and stop the process
    */
    if (*cur_quad != *ex_quad)
	clean_up(unit,TIPS_WRONG);
    }

return;
}
