/******************************************************************************
FUNCTION:	CLEAN_UP

PURPOSE:	Cleans up after a fatal errror is encountered

ALGORITHM 
  Output descriptive error message
  Close the input tape
  If the tape was opened, close the input tape
  If the print file was opened, close the print file
  If the image was opened, close the LAS image
  If the associated files were opened, close the associated files
  Clean up any files hanging around
  Output fatal error message

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "protos.h"

void clean_up
(
    long *unit,		/* tape unit number				*/
    long errcode		/* error code					*/
)
{
extern struct PARBLK vblock;	/* TAE parameter block			*/
extern struct FDESC * fdesc[2];		/* image file descriptors 		*/
extern struct GDESC * gdesc;	        /* group file descriptor		*/
extern FILE *out_fp;		/* File pointer to print file		*/

extern char out[MAXIMG][CMLEN]; /* Output files to clean up		*/
extern long out_count;		/* Number of output files to clean up	*/
extern short tape_flag;		/* Flag for tape open			*/
extern short las_flag;		/* Flag for las open			*/
extern short print_flag;	/* Flag for print file open		*/
extern short assoc_flag;	/* Flag for associated files open	*/

long status;	        /* function return status code			*/
long eof = 0;		/* number of EOFs to be written			*/
long nonfatal = 1;	/* Non-fatal error code				*/
long mode = NO_HANDLE;	/* tape I/O does not output error messages	*/
long rewopt = REWOFFL;	/* rewind the tape and take it offline		*/

char msg[100];		/* Message buffer				*/

/*
  Output descriptive error message
*/
printf ("\n");
switch (errcode)
    {
    case TIPS_MOUNT:
        c_errmsg("Error mounting tape","tipsp-mount",&nonfatal);
        break;
        
    case TIPS_READ:
        c_errmsg("Error reading tape","tipsp-tpread",&nonfatal);
        break;

    case TIPS_DENS:
        c_errmsg("Tape density is not 6250 bpi","tipsp-density",&nonfatal);
 	break;

    case TIPS_TYPE:
        c_errmsg("Input tape is not a P tape","tipsp-type",&nonfatal);
        break;

    case TIPS_FORMAT:
	sprintf(msg,"%s%s","Input tape is not band sequential or band ",
		"interleaved by line format");
	c_errmsg(msg,"tipsp-format",&nonfatal);
	break;

    case TIPS_WRONG:
        c_errmsg("The current volume does not contain the correct quadrant",
		 "tipsp-badvol",&nonfatal);
        break;

    case TIPS_OPEN:
        c_errmsg("Error opening output image","tipsp-open",&nonfatal);
        break;

    case TIPS_WRITE:
        c_errmsg("Error writing to output image","tipsp-output",&nonfatal);
        break;

    case TIPS_ALLOC:
        c_errmsg("Error allocating dynamic memory","tipsp-alloc",&nonfatal);
        break;

    case TIPS_GROUP:
        c_errmsg("Error adding output image to I/O group","tipsp-group",
		 &nonfatal);
        break;

    case TIPS_PRT:
        c_errmsg("Error opening print file","tipsp-print",&nonfatal);
        break;

    }

/*
  If the tape was opened, close the input tape
*/
if (tape_flag)
    {
    status = c_tpclos(unit,&mode,&rewopt,&eof);
    if (status != TAPE_SUCC)
       c_errmsg("Error closing tape device","tipsp-tpclose",&nonfatal);
    }

/*
  If the print file was opened, close the print file
*/
if (print_flag)
   fclose(out_fp);

/*
  If the image was opened, close the LAS image
*/
if (las_flag == 1)
    c_egclse(&gdesc);
else if (las_flag == 2)
    c_eclose(&fdesc[0]);
/*
  If the associated files were opened, close the associated files
*/
if (assoc_flag)
    close_assoc();

/*
  Clean up any files hanging around
*/
c_clean(&vblock,"TIPSP",out,&out_count);

return;
}
