/******************************************************************************
FUNCTION:	GET_TRAIL

PURPOSE:	Gets the trailer information found in the supplemental
		logical volume at the end of the tape.

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development
  2.0		12/90		BALTZER		Check return status's

ALGORITHM 
  First, skip over the trailer file descriptor record
  Now, process the trailer record
  If the trailer information is to be saved
	write the record to a file
  Read the EOF mark after the trailer file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "protos.h"

int get_trail
(
    long *unit,		/* tape unit number			      */
    long *flag		/* flag indicating whether to save annotation */
)
{
extern short tdsk;	/* trailer file flag				*/
extern long trail_fd;	/* File descriptor for trailer file	      	*/

long nbytes;		/* number of bytes to be read from tape		*/
long n_written;		/* number of bytes written			*/
long status;		/* function return status code			*/
long nonfatal = 1;	/* Non-fatal error code				*/
long mode = HANDLE;	/* tape I/O outputs error mesages		*/

unsigned char inbuf[4320]; /* Input buffer			        */

/*
  First, skip over the trailer file descriptor record
*/
nbytes = 4320;
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != nbytes)
    return(TIPS_READ);

/* 
  Now, process the trailer record
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != nbytes)
    return(TIPS_READ);

if (*flag == 0)
    {
    if (tdsk)
	{
	n_written = write(trail_fd,inbuf,nbytes);
	if (n_written != nbytes)
	    {
	    c_errmsg("Error writing to associated trailer file",
		     "get_trail-trail",&nonfatal);
	    c_errmsg("Nonfatal error encountered--Image data saved",
		     "get_trail-warn",&nonfatal);
    	    return(TIPS_WRITE);
	    }
	}
    }

/*
  Read the EOF mark after the trailer file
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != TAPE_EOF)
    return(TIPS_READ);

return(E_SUCC);
}
