/******************************************************************************
FUNCTION:	GET_VOLDIR

PURPOSE:	Reads the volume directory

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development
  1.1		4/89		HOLLAREN	Added '\0' to a strings read
						from tape

ALGORITHM 
  Read volume descriptor record
  Check tape type
  Check tape density
  Check band format
  Skip to text record
  Read text record
  Get quadrant number
  Get the scene id
  Read the EOF mark after the directory

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "protos.h"

int get_voldir
(
    long *unit,		/* tape unit number				*/
    long *cur_quad,		/* pointer to current quad number		*/
    char *sceneid,		/* scene id					*/
    long *format		/* image file format (BSQ or BIL)		*/
)
{
long count;		/* number of records to skip over		*/
long nvols;		/* number of volumes				*/
long status;		/* function return status code			*/
long nbytes = 360;	/* number of bytes to be read from tape		*/
long mode = HANDLE;	/* tape I/O outputs error messages		*/

unsigned char inbuf[360]; /* input data buffer				*/
char temp[3];		/* second temporary variable			*/

/*
  Read volume descriptor record
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != nbytes) 
    return(TIPS_READ);

/*
  Check tape type
*/
if (inbuf[48] != 'P')
    return(TIPS_TYPE);

/*
  Check tape density
*/
temp[0] = inbuf[93];
temp[1] = '\0';
sscanf(temp,"%d",&nvols);
if (nvols != 1)
    return(TIPS_DENS);

/*
  Check band format
*/
strncpy(temp,(const char *) &inbuf[162],2);
temp[2] = '\0';
sscanf(temp,"%d",&count);
if (count == 21)
    *format = BSQ;
else if (count == 3)
    *format = BIL;
else
    return (TIPS_FORMAT);

/*
  Skip to text record
*/
status = c_tpfsr(unit,&count,&mode);
if (status != TAPE_SUCC)
    return(TIPS_READ);

/*
  Read text record
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != nbytes)
    return(TIPS_READ);

/*
  Get quadrant number
*/
temp[0] = inbuf[34];
temp[1] = '\0';
sscanf(temp,"%d",cur_quad);

/*
  Get the scene id
*/
if (inbuf[105] != ' ')
    strncpy(sceneid,(const char *) &inbuf[105],10);
else
    strncpy(sceneid,(const char *) &inbuf[106],10);
sceneid[10] = '\0';

/*
  Read the EOF mark after the directory
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != TAPE_EOF) 
    return(TIPS_READ);

return(E_SUCC);
}
