/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the TAE global variables and user-specified parameters

ALGORITHM 
  LAS initialization
  Retrieve the the global parameters message time, min/max flag, and DDR valid
  flags
  Retrieve the input parameters COMMENT, WINDOW, MOSAIC flag, WRTDSK and BANDS
  Sort band list into ascending order
  Check for duplication
  Retrieve QUADNUMS to be processed
  Sort quad list into ascending order
  Check for duplication
  Check for three quad specification
  Check for diagonal quad specification
  Determine the shape of the quadrant specification
  Check the MOSAIC option
  if only one quad is specified
      Get the output names for a single mosaicked output image
      Place the output image in the clean-up array
      If necessary, make host name for an associated header file
      If necessary, make host name for an associated annotation file
      If necessary, make host name for an associated trailer file
  else
     Get the output names for multiple output quad images
     If necessary, make host name for multiple associated header files
     If necessary, make host name for multiple associated annotation files
     If necessary, make host name for multiple associated trailer files
  Retrieve global flag for remounting the TM tape

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tm.h"
#include "util.h"
#include "labserv.h"
#include "protos.h"

void getpar
(
    char hosout[][CMLEN],	  /* output host file				*/
    char taeout[][CMLEN],	  /* output TAE file				*/
    long bands[],		/* band specification				*/
    long *msgtim,		/* elapsed time between display messages	*/
    long *nbands,		/* pointer to number of bands			*/
    long *nquads,		/* pointer to number of quads			*/
    long quads[],		/* quad specification				*/
    char *minmax,		/* min max flag buffer				*/
    char annot_host[][CMLEN], /* annotation host file			*/
    char head_host[][CMLEN],  /* header host file				*/
    char trail_host[][CMLEN], /* trailer host file				*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)   */
    long window[],		/* window to process				*/
    long *wcount,		/* number of window values specified		*/
    char *mflag		  /* mosaicking flag				*/
)

{
extern struct PARBLK vblock;	/* pointer to TAE block			*/
extern char *comment[4];	/* mount messages for each tape		*/
extern char out[MAXIMG][CMLEN];	/* Output files to clean up		*/
extern char tmdrive[CMLEN];	/* Flag for remount of TM tape		*/
extern long out_count;		/* Number of output files to clean up	*/
extern long com_idx[4];		/* index into comment array		*/
extern short andsk;		/* annotation file flag		        */
extern short hdsk;		/* header file flag			*/
extern short tdsk;		/* trailer file flag			*/

struct VARIABLE *val;	   /* pointer to TAE variable within v-block	*/

long gap;		/* loop index -- used in sort			*/
long hold;		/* temporary holding variable -- used in sort	*/
long std_out;		/* LUN for standard output			*/
long i, j;	        /* loop index					*/
long status;		/* function return status code			*/
long flag = 1;		/* flag for mkimg to check file existence 	*/
long fatal = -1;	/* Fatal error code				*/

char base[CMLEN];	/* Base output name without extension		*/
char ext[CMLEN];	/* Extension for associated files		*/
char wrtdsk[4];	 	/* write to disk paramter			*/
char msg[80];		/* Error message string				*/

/*
  LAS initialization
*/
c_inlas(&vblock,&std_out);

/*
  Retrieve the time between processing messages
*/
val = p_find(&vblock,"$MSGTIM");
*msgtim = IVAL(*val,0);

/*
  Retrieve the min/max flag
*/
val = p_find(&vblock,"$MINMAX");
strcpy(minmax,SVAL(*val,0));

/*
  Retrieve COMMENT for mount messages
*/
val = p_find(&vblock,"COMMENT");
for (i = 0; i < val->v_count; i++)
    {
    comment[i] = SVAL(*val,i);
    com_idx[i] = i;
    }

/*
  Retrieve WINDOW
*/
val = p_find(&vblock,"WINDOW");
*wcount = val->v_count;
if (*wcount == 0)
    {
    for (i = 0; i < 4; i++)
   	window[i] = 0;
    }
else
    {
    for (i = 0; i < *wcount; i++)
    	window[i] = IVAL(*val,i);
    }

/*
  Retrieve the MOSAIC option
*/
val = p_find(&vblock,"MOSCFLG");
strcpy(mflag,SVAL(*val,0));

/*
  Retrieve WRTDSK which ancillary files are to be saved on disk
*/
hdsk  = FALSE;
tdsk  = FALSE;
andsk = FALSE;

val = p_find(&vblock,"WRTDSK");
if (val->v_count != 0)
    for (i = 0; i < val->v_count; i++)
    	{
        strcpy(wrtdsk,SVAL(*val,i));
        if (strcmp(wrtdsk,"HDR") == 0)
	    hdsk = TRUE;
        if (strcmp(wrtdsk,"TRA") == 0)
	    tdsk = TRUE;
       	if (strcmp(wrtdsk,"ANB") == 0)
	    andsk = TRUE;
       	}

/*
  Retrieve BANDS to be processed
*/
val = p_find(&vblock,"BANDS");
if (val->v_count == 0)
    {
    for (i = 0; i < MAX_BANDS; i++)
        bands[i] = i + 1;
    *nbands = 7;
    }
else
    {
    *nbands = val->v_count; 
    for (i = 0; i < *nbands; i++)
        bands[i] = IVAL(*val,i);
    }
bands[*nbands] = 0;

/*
  Sort band list into ascending order
*/
for (gap = *nbands / 2; gap > 0; gap /= 2)
    {
    for (i = gap; i < *nbands; i++)
	{
	for (j = i - gap; j >= 0 && bands[j] > bands[j+gap]; j -= gap)
            {
	    hold = bands[j];
 	    bands[j] = bands[j+gap];
  	    bands[j+gap] = hold;
            }
	}
    }

/*
  Check for duplication
*/
for (i = 0; i < *nbands-1; i++)
    {
    if (bands[i] == bands[i+1])
	c_errmsg("Duplicate band numbers were specified","tipsp-duplicate",
		 &fatal);
    }

/*
  Retrieve QUADNUMS to be processed
*/
val = p_find(&vblock,"QUADNUMS");
if (val->v_count == 0)
    {
    for (i = 0; i < MAX_QUADS; i++)
        quads[i] = i + 1;
    *nquads = 4;
    }
else
    {
    *nquads = val->v_count;
    for (i = 0; i < *nquads; i++)
        quads[i] = IVAL(*val,i);
    }

/*
  Sort quad list into ascending order
*/
for (gap = *nquads / 2; gap > 0; gap /= 2)
    {
    for (i = gap; i < *nquads; i++)
	{
	for (j = i - gap; j >= 0 && quads[j] > quads[j+gap]; j -= gap)
	    {
	    hold = quads[j];
 	    quads[j] = quads[j+gap];
  	    quads[j+gap] = hold;
            }
	}
    }

/*
  Check for duplication
*/
for (i = 0; i < *nquads-1; i++)
    {
    if (quads[i] == quads[i+1])
	c_errmsg("Duplicate quadrant numbers were specified","tipsp-duplicate",
		 &fatal);
    }

/*
  Check for three quad specification
*/
if (strcmp(mflag,"YES") == 0)
    {
    if (*nquads == 3)
        c_errmsg("Three quadrant specification not allowed","tipsp-three",
		 &fatal);
    }

/*
  Check for diagonal quad specification
*/
if (strcmp(mflag,"YES") == 0)
    {
    if (*nquads == 2)
        {
        if ((quads[0] == 1) && (quads[1] == 4))
            {
            c_errmsg("Diagonal quadrant specification not allowed",
		     "tipsp-diagonal",&fatal);
            }
        else if ((quads[0] == 2)  && (quads[1] == 3))
            {
            c_errmsg("Diagonal quadrant specification not allowed",
		     "tipsp-diagonal",&fatal);
            }
        }
    }

/*
  Determine the shape of the quadrant specification
*/
if (*nquads == 2)
    {
    if ((quads[0] == 1) && (quads[1] == 3))
        *imshape = 'l';
    else if ((quads[0] == 2) && (quads[1] == 4))
        *imshape = 'r';
    else if ((quads[0] == 1) && (quads[1] == 2))
        *imshape = 'u';
    else 
        *imshape = 'b';
    }
else if (*nquads == 1)
    *imshape = 'q';
else if (*nquads == 4)
    *imshape = 'w';

/*
  Check the MOSAIC option if only one quad is specified
*/
if ((*nquads == 1) && (strcmp(mflag,"YES") == 0))
    {
    sprintf(msg,"%s%s","Mosaicking was indicated, however only one input ",
	    "quadrant was specified");
    c_errmsg(msg,"tipsp-mosaic",&fatal);
    }
else if ((*wcount != 0) && (*nquads > 1))
    strcpy (mflag,"YES");

/*
  Get the output file names
*/
if ((strcmp(mflag,"YES") == 0) || (strcmp(mflag,"ASSOC") == 0))
    {
    /*
      Get the output names for a single mosaicked output image
    */
    status = c_getout(&vblock,"OUT",taeout,hosout);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","tipsp-fatal",&fatal);

    /*
      Place the output image in the clean-up array
    */
    strcpy(out[out_count],hosout[i]);
    out_count++;

    /*
      If necessary, make host name for an associated header file
    */
    if (hdsk)
        c_lsmknm(hosout[0],"tipsp_head",head_host[0]);

    /*
      If necessary, make host name for an associated annotation file
    */
    if (andsk)
        c_lsmknm(hosout[0],"tipsp_annot",annot_host[0]);

    /*
      If necessary, make host name for an associated trailer file
    */
    if (tdsk)
      	c_lsmknm(hosout[0],"tipsp_trail",trail_host[0]);
    }
else
   /*
     Get the output names for multiple output quad images
   */
    {
    val = p_find(&vblock,"OUT");
    strcpy(base,SVAL(*val,0));

    for (i = 0; i < *nquads; i++)
        {
	/*
	  Make the TAE and host names for the output quad image
	*/
        sprintf(ext,".quad%d",quads[i]);

        status = c_mkimg(base,ext,taeout[i],hosout[i],&flag);
        if (status != E_SUCC)
	    c_errmsg("Fatal error encountered","tipsp-fatal",&fatal);

	/*
	  Place the output image in the clean-up array
	*/
	strcpy(out[out_count],hosout[i]);
        out_count++;

	/*
	  If necessary, make host name for associated header file
 	*/
        if (hdsk)
	    c_lsmknm(hosout[i],"tipsp_head",head_host[i]);

	/*
	  If necessary, make host name for associated annotation file
	*/
	if (andsk)
	    c_lsmknm(hosout[i],"tipsp_annot",annot_host[i]);
       
	/*
	  If necessary, make host name for associated trailer file
	*/
	if (tdsk)
	    c_lsmknm(hosout[i],"tipsp_trail",trail_host[i]);

        }
    }

/*
  Retrieve global flag for remounting the TM tape
*/
val = p_find(&vblock,"TMDRIVE");
if (val->v_count == 0)
    strcpy(tmdrive,"MOUNT");
else
    strcpy(tmdrive,SVAL(*val,0));

return;
}
