/******************************************************************************
FUNCTION:	INITQUAD

PURPOSE:	Determines the starting record and last record to read from
		tape and also determines the starting record to write to disk.

ALGORITHM 
  Save the current quad
  Set up which records need to be read from disk and tape according to the
  image shape and current quad number
  Set the tape start record, tape last record, and disk start record
  according to the format of the tape (BIL or BSQ).

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tm.h"     
#include "protos.h"

void initquad
(
    long window[], 		/* user specified window			*/
    long *nl,		/* number of lines in quad			*/
    long *count,		/* loop counter					*/
    char *imshape,	 	/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    long quads[],		/* quadrants to process				*/
    long tape_startrec[],	/* tape record number to start reading		*/
    long *tape_lastrec,	/* last tape record to read			*/
    long disk_rec[],	/* current disk record number			*/
    long disk_startrec[],	/* disk record number to start writing		*/
    long *format		/* Image file format (BSQ or BIL)		*/
)
{
long curquad;		/* current quadrant number			*/

/*
  Save the current quad
*/
curquad = quads[*count];

/*
  Set up which records need to be read from disk and tape according to the
  image shape and current quad number
*/
if ((*imshape == 'l' || *imshape == 'r' || *imshape == 'w') && 
    (curquad == 1 || curquad == 2))
    {
    if (*format == BSQ)
	{
	tape_startrec[*count] = window[0] + 1;
	*tape_lastrec = *nl + 1;
	}
    else
	{
	tape_startrec[*count] = ((window[0] - 1) * 7) + 2;
	*tape_lastrec = (*nl * 7) + 1;
	}
    }
else if ((*imshape == 'l' || *imshape == 'r') && (curquad == 3 || curquad == 4))
    {
    if (*format == BSQ)
	{
	*tape_lastrec = window[2] - disk_rec[*count-1] + 2;
	disk_startrec[*count] = disk_rec[*count-1]; 
	}
    else
	{
	*tape_lastrec = ((window[2] - disk_rec[*count-1] + 1) * 7) + 1;
	disk_startrec[*count] = disk_rec[*count-1]; 
	}
    }
else if (*imshape == 'u' || *imshape == 'b' || *imshape == 'q')
    {
    if (*format == BSQ)
	{
	tape_startrec[*count] = window[0] + 1;
	*tape_lastrec = window[0] + window[2];
	}
    else
	{
	tape_startrec[*count] = ((window[0] - 1) * 7) + 2;
	*tape_lastrec = ((window[0] + window[2] - 1) * 7) + 1;
	}
    }
else if (*imshape == 'w' && (curquad == 3 || curquad == 4))
    {
    if (*format == BSQ)
	{
	disk_startrec[*count] = disk_rec[*count-2];
	*tape_lastrec = window[2] - disk_startrec[*count] + 2;
	}
    else
	{
	disk_startrec[*count] = disk_rec[*count-2];
	*tape_lastrec = ((window[2] - disk_startrec[*count] + 1) * 7) + 1;
	}
    }

return;
}
