#ifndef PROTOS_H
#define PROTOS_H


void open_assoc
(
    char *annot_host,	/* annotation host name  			*/
    char *head_host,	/* descriptor for header host name		*/
    char *trail_host	/* descriptor for trailer host name    		*/
);

void delete_assoc
(
    char *annot_host,	/* annotation host name				*/
    char *head_host,	/* header host name				*/
    char *trail_host	/* trailer host name				*/
);

void check_line
(
    long window[], 		/* user specified window			*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    long *nl,		/* number of lines in quad			*/
    long *ns,		/* numbber of samples in quad			*/
    long quads[],		/* quadrants to process				*/
    long *nquads		/* number of quads to process			*/
);

void check_pixel
(
    long window[],		/* user specified window			*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    long *ns,		/* number of samples in quad			*/
    long quads[],		/* quadrants to process				*/
    long *nquads,		/* number of quads to process			*/
    long *right_fill	/* number of fill pixels on the right		*/
);

void check_vol
(
    long *unit,		/* tape unit number				*/
    long *cur_quad,		/* current quad number				*/
    long *ex_quad,		/* quadrant expected to be mounted		*/
    char *sceneid,		/* scene id read from text record		*/
    long *format		/* Image file format (BSQ or BIL)		*/
);

void check_wind
(
    long *wind,		/* user specified window  (SL,SS,NL,NS)		*/
    long *imsiz,		/* image size  (NL,NS) 				*/
    long *flag		/* flag indicating if window being applied	*/
);

void clean_up
(
    long *unit,		/* tape unit number				*/
    long errcode		/* error code					*/
);

void close_assoc
(
  void
);

int get_annot
(
    long *unit,         /* tape unit number                           */
    long *flag,         /* flag indicating whether to save annotation */
    long bands[],       /* band specification                         */
    char *date,         /* acquisition date                           */
    char *lat_long,     /* lat long at scene center                   */
    char *path_row,     /* WRS path row                               */
    char *scene_id,     /* scene id                                   */
    char *source,       /* data source                                */
    char *resamp,       /* resampling technique                       */
    char *quality       /* overall geometric quality code             */
);

int get_calib
(
    long *unit,         /* tape unit number                             */
    long bands[],       /* Band specification                           */
    long *format        /* image file format (BSQ or BIL)               */
);

int get_head
(
    long *unit,		/* tape unit number				*/
    long *flag,		/* flag indicating whether to save header info	*/
    long *format,		/* image file format (BSQ or BIL)		*/
    long bands[],  		/* Band specification		      	      	*/
    double fcenll[],	/* Full scene center latitude/longitude degrees	*/
    double fcenls[],	/* Full scene center line/sample		*/
    double fcproj[],	/* Full scene center in projection coordinates	*/
    double *angle,		/* Angle of rotation converted into radians	*/
    long *path,		/* WRS path number				*/
    long *utmzone,		/* UTM zone number				*/
    char *direction,	/* Orbital direction				*/
    char *maprj,		/* Map projection identifier			*/
    long *line_center,	/* Line number at scene center			*/
    long *samp_center	/* sample number at scene center		*/
);

int get_trail
(
    long *unit,         /* tape unit number                           */
    long *flag          /* flag indicating whether to save annotation */
);

int get_voldir
(
    long *unit,		/* tape unit number				*/
    long *cur_quad,		/* pointer to current quad number		*/
    char *sceneid,		/* scene id					*/
    long *format		/* image file format (BSQ or BIL)		*/
);

void getpar
(
    char hosout[][CMLEN],	  /* output host file				*/
    char taeout[][CMLEN],	  /* output TAE file				*/
    long bands[],		/* band specification				*/
    long *msgtim,		/* elapsed time between display messages	*/
    long *nbands,		/* pointer to number of bands			*/
    long *nquads,		/* pointer to number of quads			*/
    long quads[],		/* quad specification				*/
    char *minmax,		/* min max flag buffer				*/
    char annot_host[][CMLEN], /* annotation host file			*/
    char head_host[][CMLEN],  /* header host file				*/
    char trail_host[][CMLEN], /* trailer host file				*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)   */
    long window[],		/* window to process				*/
    long *wcount,		/* number of window values specified		*/
    char *mflag		  /* mosaicking flag				*/
);

void initquad
(
    long window[], 		/* user specified window			*/
    long *nl,		/* number of lines in quad			*/
    long *count,		/* loop counter					*/
    char *imshape,	 	/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    long quads[],		/* quadrants to process				*/
    long tape_startrec[],	/* tape record number to start reading		*/
    long *tape_lastrec,	/* last tape record to read			*/
    long disk_rec[],	/* current disk record number			*/
    long disk_startrec[],	/* disk record number to start writing		*/
    long *format		/* Image file format (BSQ or BIL)		*/
);

void proc_assoc
(
    long bands[],		/* band specification				*/
    long *nbands,		/* number of bands				*/
    long quads[],		/* quad specification				*/
    long *nquads,		/* number of quads				*/
    char *annot_host,	/* annotation host name				*/
    char *head_host,	/* header host name				*/
    char *trail_host,	/* trailer host name				*/
    long window[]		/* window specification				*/
);

void proc_window
(
    char hosout[][CMLEN],		/* output host name				*/
    long bands[],		/* band specification				*/
    long *nbands,		/* number of bands				*/
    long quads[],		/* quad specification				*/
    long *nquads,		/* number of quads				*/
    char *annot_host,	/* annotation host name				*/
    char *head_host,	/* header host name				*/
    char *trail_host,	/* trailer host name				*/
    long window[],		/* window specification				*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    char *minmax,		/* minmax flag buffer				*/
    long *msgtim		/* elapsed time between display messages	*/
);

int read_qbil
(
    long *unit,	   	/* tape unit number				*/
    long *nl,		/* number of lines				*/
    long *ns,		/* number of samples per line			*/
    long bands[],		/* band numbers to process			*/
    double outmin[],	/* pointer to array of min values		*/
    double outmax[],	/* pointer to array of max values		*/
    char *minmax		/* minmax flag					*/
);

int read_quad
(
    long *unit,	   	/* tape unit number				*/
    long *nl,		/* number of lines				*/
    long *ns,		/* number of samples per line			*/
    double outmin[],	/* pointer to array of min values		*/
    double outmax[],	/* pointer to array of max values		*/
    char *minmax		/* minmax flag					*/
);

int read_wbil
(
    long *unit,	   	/* tape unit number				*/
    long *count,		/* loop counter				  	*/
    long window[],		/* user specified window               		*/
    long bands[],		/* band numbers to be processed		 	*/
    long quads[],		/* quads to process			  	*/
    long *nsquad,		/* number of samples in the quad	  	*/
    long *lastns,		/* number of samples written to last line 	*/
    long disk_rec[],	/* current disk record number             	*/
    long disk_startrec[],	/* disk record to start writing 	  	*/
    long *tape_rec,		/* current tape record number		  	*/
    long tape_startrec[],	/* tape record number of start reading    	*/
    long *tape_lastrec,	/* tape record number of last record to read	*/
    char *imshape,	 	/* shape of the output image
				w - whole image
                                q - single quad
				u - upper half (quads 1 and 2)
				b - bottom half (quads 3 and 4)
				l - left half (quads 1 and 3)
				r - right half (quads 2 and 4)		*/
    double outmin[],	/* pointer to array of min values		*/
    double outmax[],	/* pointer to array of max values		*/
    char *minmax		/* minmax flag					*/
);

int read_win
(
    long *unit,		/* tape unit number				*/
    long *count,		/* loop counter					*/
    long window[],		/* user specified window			*/
    long *band_num,		/* current output band number			*/
    long quads[],		/* quads to process				*/
    long *nsquad,		/* number of samples in the quad		*/
    long *lastns,		/* number of samples written to last line	*/
    long disk_rec[],	/* current disk record number			*/
    long disk_startrec[],	/* disk record to start writing			*/
    long *tape_rec,		/* current tape record number			*/
    long tape_startrec[],	/* tape record number of start reading		*/
    long *tape_lastrec,	/* tape record number of last record to read	*/
    char *imshape,		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)	*/
    double outmin[],	/* pointer to array of min values		*/
    double outmax[],	/* pointer to array of max values		*/
    char *minmax		/* minmax flag					*/
);

void separate
(
    char hosout[][CMLEN],	/* output host names				*/
    long bands[],		/* band specification				*/
    long *nbands,		/* number of bands				*/
    long quads[],		/* quad specification				*/
    long *nquads,		/* number of quads				*/
    char annot_host[][CMLEN],/* annotation host name(s)			*/
    char head_host[][CMLEN],/* header host names				*/
    char trail_host[][CMLEN],/* trailer host names				*/
    char *minmax,		/* min max flag					*/
    long *msgtim		/* elapsed time between display messages	*/
);

void sepbil
(
    long *unit,		/* Tape unit number			      	*/
    char *hname,	/* host file name of output image    		*/
    long *quad,		/* Current quadrant number		      	*/
    long *nl,		/* Number of lines in the quad		     	*/
    long *ns,		/* Number of samples in the quad	      	*/
    long *nbands,	/* Number of bands being processed	      	*/
    long bands[],  	/* Band specification		      	      	*/
    double outmin[],	/* Array of min values for each output band	*/
    double outmax[],	/* Array of max values for each output band	*/
    char *minmax,	/* Min max flag			      	    	*/
    long *msgtim	/* Seconds between processing messages		*/
);

void sepbsq
(
    long *unit,
   char *hname,
   long *quad,
   long *nl,
   long *ns,
   long *nbands,
   long bands[],
   double outmin[],
   double outmax[],
   char *minmax,
   long *msgtim
);

int swap_tape
(
    long *unit,
   long *quad,
   long *index,
   long *format
);

int mount_first
(
    long *unit,
   long *ex_quad,
   long *nquads,
   long quads[]
);

int mount_tape
(
    long *unit,
   long *curvol,
   long quad
);

int reopen_tape
(
    long *unit
);

void dismount_tape
(
    long *unit
);

void update_ddr
(
    char *hname,		/* Host file name				*/
    long window[],		/* Input window					*/
    long *totbnd,		/* Total number of bands to process		*/
    char *scene_id,		/* Scene id					*/
    char *maprj,		/* Map projection identifier			*/
    char *direction,	/* Orbital direction				*/
    long *path,		/* WRS path number				*/
    long *utmzone,		/* UTM zone number				*/
    double fcproj[],	/* Full scene center in projection coordinates	*/
    double fcenll[],	/* Full scene center latitude/longitude degrees	*/
    double fcenls[],	/* Full scene center line/sample		*/
    double *angle,		/* Angle of rotation converted into radians	*/
    char *date		/* acquisition date				*/
);

void verify_win
(
    long window[], 	/* user specified window 			*/
    char *imshape,		/* shape of the output image
				w - whole image
                                q - single quad
				u - upper half (quads 1 and 2)
				b - bottom half (quads 3 and 4)
				l - left half (quads 1 and 3)
				r - right half (quads 2 and 4)		*/
    long quads[]		/* quadrants to process				*/
);

void winbil
(
    long *unit,		/* Tape unit number				*/
    char *hname,		/* host file name of output image    		*/
    long *count,		/* Loop count					*/
    long *nl,		/* Number of lines in the quadrant		*/
    long *ns,		/* Number of samples in the quadrant		*/
    long *nbands,		/* Number of bands being processed		*/
    long bands[],		/* Band specification		      	 	*/
    long window[],		/* Window to be processed			*/
    long quads[],		/* Quadrant numbers to process			*/
    char *imshape,		/* shape of the output image
				 w - whole image
                                 q - single quad
				 u - upper half (quads 1 and 2)
				 b - bottom half (quads 3 and 4)
				 l - left half (quads 1 and 3)
				 r - right half (quads 2 and 4)		*/
    long *tape_rec,		/* Current tape record          		*/
    long *lastns,		/* Number of samples written to the last line	*/
    long quads_used[],	/* The quadrants actually used			*/
    long *nquads_used,	/* Number of quads used				*/
    double outmin[],	/* Min values for each output band		*/
    double outmax[],	/* Max values for each output band		*/
    char *minmax,		/* Min max flag					*/
    long *msgtim,		/* Number of seconds between processing messages*/
    char *text		/* Text for processing message		    	*/
);

void winbsq
(
    long *unit,		/* Tape unit number				*/
    char *hname,		/* host file name of output image		*/
    long *count,		/* Loop count					*/
    long *nl,		/* Number of lines in the quadrant		*/
    long *ns,		/* Number of samples in the quadrant		*/
    long window[],		/* Window to be processed			*/
    long bands[],		/* Band specification		   		*/
    long *nbands,		/* Number of bands being processed		*/
    long quads[],		/* Quadrant numbers to process			*/
    long quads_used[],	/* The quadrants actually used			*/
    long *nquads_used,	/* Number of quads used				*/
    long *lastns,		/* Number of samples written to the last line	*/
    long *tape_rec,		/* Current tape record                  	*/
    char *imshape, 		/* shape of the output image
				       w - whole image
                                       q - single quad
				       u - upper half (quads 1 and 2)
				       b - bottom half (quads 3 and 4)
				       l - left half (quads 1 and 3)
				       r - right half (quads 2 and 4)   */
    double outmin[],	 /* Array of min values for each output band	*/
    double outmax[],	/* Array of max values for each output band	*/
    char *minmax,		/* Min max flag			      	      	*/
    long *msgtim,		/* Number of seconds between processing message */
    char *text		/* Text for processing message			*/
);

void write_parm
(
    char *date,		/* acquisition date				*/
    char *lat_long,		/* lat long at scene center			*/
    char *path_row,		/* WRS path row					*/
    char *scene_id,		/* scene id					*/
    char *source,		/* data source					*/
    char *resamp,		/* resampling technique				*/
    char *quality,		/* overall geometric quality code		*/
    long *line_center,	/* line number at scene center			*/
    long *samp_center,	/* sample number at scene center		*/
    long window[],		/* window value					*/
    long *nquads,		/* number of quads				*/
    long quads[],		/* quad specification				*/
    char *sceneid		/* scene id read from text record		*/
);

#endif

