/******************************************************************************
FUNCTION:	READ_QBIL

PURPOSE:	Reads in all bands specified from the current quad whose
		image data is in band interleaved by line format.  This
		routine reads in the full image and creates a separate
		image file for each quad.  If a window is being read in or
		the quads are being mosaicked, use the function READ_WBIL.

PROJECT			LAS

ALGORITHM 
  Start the processing message
      Read each line of each band from tape
	      Copy image data to output buffer
	      Calculate the minimum and maximum values if necessary
  Flush the input buffer
  Stop the processing message
  Free the dynamically allocated memory
  Read EOF mark after image file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

int read_qbil
(
    long *unit,	   	/* tape unit number				*/
    long *nl,		/* number of lines				*/
    long *ns,		/* number of samples per line			*/
    long bands[],		/* band numbers to process			*/
    double outmin[],	/* pointer to array of min values		*/
    double outmax[],	/* pointer to array of max values		*/
    char *minmax		/* minmax flag					*/
)
{
extern struct GDESC * gdesc;	/* group file descriptor (seq I/O)	*/
extern long offset[MAXBND+1];	/* offsets into output buffer (seq I/O) */
extern long curent;	/* current line number				*/

extern char *buffer;	/* I/O buffer				*/

long j;			/* loop index					*/
long bndidx;		/* current index into band array		*/
long curlin;		/* current line counter				*/
long status;		/* function return status code			*/
long nbytes = 4320;	/* number of bytes to be read from tape		*/
long dtype = EBYTE;	/* BYTE data type				*/
long mode = HANDLE;	/* tape I/O outputs error messages		*/

unsigned char inbuf[4320]; /* input data buffer				*/

/*
  Start the processing message
*/
curent = 1;
c_startm();

for (curlin = 0; curlin < *nl; curent++)
    {
    if (c_estep(&gdesc) != E_SUCC)
	return (TIPS_WRITE);

    /*
      Read each line of each band from tape
    */
    curlin++;
    bndidx = 0;
    for (j = 0; j < 7; j++)
	{
    	status = c_tpread(unit,inbuf,&nbytes,&mode);
    	if (status != nbytes)
            return(TIPS_READ);

	if ((j+1) == bands[bndidx])
	    {
	    /*
	      Copy image data to output buffer
	    */
    	    c_pxcopy(&inbuf[32],buffer + (*(offset + bndidx)),&dtype,ns);

	    /*
	      Calculate the minimum and maximum values if necessary
	    */
    	    if (strcmp(minmax,"yes") == 0)
        	status = c_pxmnmx(buffer + (*(offset + bndidx)),ns,
				  &outmin[bndidx],&outmax[bndidx],&dtype);
	    bndidx++;
	    }
	}
    }

if (c_estep(&gdesc) != E_SUCC)
    return (TIPS_WRITE);

c_stoptm();

/*
  Free the dynamically allocated memory
*/
free(buffer);

/*
  Read EOF mark after image file
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);

return(E_SUCC);
}
