/******************************************************************************
FUNCTION:	READ_QUAD

PURPOSE:	Reads in a band from the current quad.  This routine reads in
		the full image and creates a separate image file for each quad.
		If a window is being read in or the quads are being mosaicked,
		use the function READ_WIN.

ALGORITHM 
  Start the proccessing message
  Proccess each line
      Read each line from tape
      Copy image data to output buffer
      Calculate the minimum and maximum values if necessary
      Write the line to the disk file
  Stop the proccessing message
  Read EOF mark after image file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

int read_quad
(
    long *unit,	   	/* tape unit number				*/
    long *nl,		/* number of lines				*/
    long *ns,		/* number of samples per line			*/
    double outmin[],	/* pointer to array of min values		*/
    double outmax[],	/* pointer to array of max values		*/
    char *minmax		/* minmax flag					*/
)
{
extern struct FDESC * fdesc[2];	/* image I/O file descriptors 		*/
extern long curent;	/* current line number				*/
extern long curcat;	/* Current band number       	             	*/

long curlin;		/* current line counter				*/
long status;		/* function return status code			*/
long one = 1;		/* Constant one					*/
long dtype = EBYTE;	/* BYTE data type				*/
long mode = HANDLE;	/* tape I/O outputs error messages		*/
long nbytes = 4320;	/* number of bytes to be read from tape		*/

unsigned char inbuf[4320];  /* input data buffer			*/
unsigned char outbuf[4228]; /* output data buffer			*/

/*
  Start the processing message
*/
curent = 1;
c_startm();

/*
  Process all lines
*/
for (curlin = 0; curlin < *nl; curent++)
    {
    /*
      Read each line from tape
    */
    curlin++;
    status = c_tpread(unit,inbuf,&nbytes,&mode);
    if (status != nbytes)
        return(TIPS_READ);

    /*
      Copy image data to output buffer
    */
    c_pxcopy(&inbuf[32],outbuf,&dtype,ns);

    /*
      Calculate the minimum and maximum values if necessary
    */
    if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(outbuf,ns,&outmin[curcat - 1],&outmax[curcat - 1],
			  &dtype);

    /*
      Write the line to the disk file
    */
    status = c_ewrite(&fdesc[0],&curcat,&curlin,outbuf,&one);
    if (status != *ns) 
        return(TIPS_WRITE);

    }

/*
  Stop the proccessing message
*/
c_stoptm();

/*
  Read EOF mark after image file
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);

return(E_SUCC);
}
