/******************************************************************************
FUNCTION:	SEPBIL

PURPOSE:	Processing loop for Band Interleaved by Line images that are
		not being mosaicked.

ALGORITHM 
  Set up current quad number for the display message
  Set up number of lines for the display message
  Open the output image
  Set the image sequential open LAS flag
  Add the host file to the output file list for clean up
  Null terminat the file descriptor array
  Allocate I/O buffer
  Put output image in a processing group
  Read in the image file for the currend quad
  Close all image files in the group
  Process the trailer record

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "util.h"
#include "protos.h"

void sepbil
(
    long *unit,		/* Tape unit number			      	*/
    char *hname,	/* host file name of output image    		*/
    long *quad,		/* Current quadrant number		      	*/
    long *nl,		/* Number of lines in the quad		     	*/
    long *ns,		/* Number of samples in the quad	      	*/
    long *nbands,	/* Number of bands being processed	      	*/
    long bands[],  	/* Band specification		      	      	*/
    double outmin[],	/* Array of min values for each output band	*/
    double outmax[],	/* Array of max values for each output band	*/
    char *minmax,	/* Min max flag			      	    	*/
    long *msgtim	/* Seconds between processing messages		*/
)
{
extern struct FDESC * fdesc[2];		/* image file descriptors 		*/
extern struct GDESC * gdesc;	        /* group file descriptor		*/
extern long offset[MAXBND+1];   /* offsets into output buffer 		*/
extern long out_count;	        /* number of output images		*/
extern short las_flag;		/* Flag for las open			*/
extern char out[MAXIMG][CMLEN]; /* the output files created		*/
extern unsigned char *buffer;	/* I/O buffer				*/

static	long	opt[3]={0,0,0};	/* optional parameter array for EOPENS	*/
long bufsiz;		/* output buffer size				*/
long status;		/* Status indicator			   	*/
long nonfatal = 1;	/* Non-fatal error code				*/
long one = 1;		/* One						*/
long zero = 0;		/* Zero						*/
long flag = ALLCAT;	/* Display message flag--one band at a time 	*/
long dtype = EBYTE;	/* Data type of the image (always BYTE)		*/
long access = IWRITE;	/* Open access for the image file		*/

char text[11];		/* processing message		      		*/

/*
  Set up current quad number for the display message
*/
sprintf(text,"in quad %d",*quad);

/*
  Set up number of lines for the display message
*/
c_inittm("CCTTIPSP","processed","line","bands",text,nl,nbands,msgtim,&flag);

/*
  Open the output image
*/
status = c_eopens(&fdesc[0],hname,nbands,&one,&one,nl,ns,offset,&access,&dtype,
		  &zero,opt);
if (status != E_SUCC)
    clean_up(unit,TIPS_OPEN);

/*
  Set the image sequential open LAS flag
*/
las_flag = 1;

/*
  Add the host file to the output file list for clean up
*/
strcpy(out[out_count],hname);
out_count++;

/*
  Null terminat the file descriptor array
*/
fdesc[1] = 0;

/*
  Allocate I/O buffer
*/
if ((buffer = (unsigned char *) c_ealloc(fdesc,&bufsiz)) == NULL)
    clean_up(unit,TIPS_ALLOC);

/*
  Put output image in a processing group
*/
status = c_egroup(fdesc,&gdesc,buffer,&bufsiz);
if (status != E_SUCC)
    clean_up(unit,TIPS_GROUP);

/*
  Read in the image file for the currend quad
*/
status = read_qbil(unit,nl,ns,bands,outmin,outmax,minmax);
if (status != E_SUCC)
    clean_up(unit,status);

/*
  Close all image files in the group
*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing image group -- Image data saved","tipsp-close",
	    &nonfatal);

las_flag = 0;

/*
  Process the trailer record
*/
status = get_trail(unit,&zero);
if (status != E_SUCC)
    clean_up(unit,status);

return;
}
