/******************************************************************************
FUNCTION:	SWAP_TAPE

PURPOSE:	Dismounts the current tape volume and mounts the tape for the
		quadrant specified making sure the tape is available. 

ALGORITHM 
  Try 2 times to get the correct volume mounted
      Dismount the current tape volume
      If the tape for the quadrant specified is not available, print
      an error
      Prompt for the new tape
      Read the volume directory for the newly mounted tape
      If the wrong volume is mounted and its not the first time through the
      loop, return an error.  Else the correct volume is mounted, break out
      of the loop

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "protos.h"

int swap_tape
(
    long *unit,		/* unit number					*/
    long *quad,		/* quadrant to mount		      		*/
    long *index,	/* index into quads array	      		*/
    long *format	/* Image file format (BSQ or BIL)		*/
)
{
extern short tape_flag;	/* Flag for tape open				*/
extern long com_idx[4];	/* index into comment array			*/
extern char *comment[4];/* mount messages for each tape			*/

char message[ERRLEN];	/* error message buffer		      		*/
char newmsg[CMLEN];	/* buffer storage for mount message   		*/
char sceneid[11];	/* scene id read from text record     		*/
char tapeid[CMLEN];     /* tape id 					*/
char dens[5];		/* tape density			      		*/

long i;			/* looping index		      		*/
long cur_quad;		/* quad number from the tape	      		*/
long status;		/* status indicator		      		*/
long eof = 0;		/* number of EOF marks to write       		*/
long skipopt = 0;	/* number of files to skip on open    		*/
long mode = HANDLE;	/* tape i/o handles errors            		*/
long access = TREAD;	/* open tape for read access	      		*/
long option = REWOFFL;	/* rewind tape and take it off line   		*/
long rewopt = REWIND;	/* rewind tape on open	  	      		*/
long fatal = -1;	/* Fatal error code				*/
long nonfatal = 1;	/* Non-fatal error code				*/

strcpy(dens,"0");
/*
  Try 2 times to get the correct volume mounted
*/
for(i = 0; i < 2; i++)
    {
    /*
      Dismount the current tape volume
    */
    status = c_tpclos(unit,&mode,&option,&eof);
    if (status != TAPE_SUCC)
        c_errmsg("Error closing tape","tipsp-tpclose",&nonfatal);

    tape_flag = FALSE;

    /*
      If the tape for the quadrant specified is not available, print
      an error
    */
    if (strcmp(comment[com_idx[*index]],"NA") == 0)
	{
	sprintf (message,"The tape for quadrant %d is not available",
		 *quad);
	c_errmsg(message,"tipsp-notape",&fatal);
	}

    /*
      Prompt for the new tape
    */
    strcpy(newmsg,comment[com_idx[*index]]);
    status = c_tpopen(unit,&skipopt,&mode,&rewopt,dens,&access,newmsg,tapeid);
    if (status != TAPE_SUCC)
	return (TIPS_OPEN);

    /*
      Read the volume directory for the newly mounted tape
    */
    status = get_voldir(unit,&cur_quad,sceneid,format);
    if (status != E_SUCC)
	return (status);

    /*
      If the wrong volume is mounted and its not the first time through the
      loop, return an error.  Else the correct volume is mounted, break out
      of the loop
    */
    if ((cur_quad != *quad) && i != 0)
	return(TIPS_WRONG);
    else if (cur_quad == *quad)
	break;
    }

return(E_SUCC);
}
