#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"
#include "protos.h"

/******************************************************************************
FUNCTION:	MOUNT_FIRST

PURPOSE:	Finds the first available quadrant and has it mounted

ALGORITHM 
  Loop on the numer of quadrands
      If the comment specified has a valid mount message
	    mount the tape

ALGORITHM REFERENCES	none
*******************************************************************************/
int mount_first
(
    long *unit,			/* unit number				*/
    long *ex_quad,		/* expected quadrant number		*/
    long *nquads,		/* number of quadrants input		*/
    long quads[]		/* quadrant numbers			*/
)
{
extern long com_idx[4];		/* index into comment array		*/
extern char *comment[4];	/* mount messages for each tape		*/
extern char tmdrive[CMLEN];	/* Drive to reopen for TM tape		*/

long i;				/* loop index				*/
long status = 0;	/* status indicator			*/

/*
  Loop on the number of quadrants
*/
for (i = 0; i < *nquads; i++)
    {
    /*
      If the comment specified has a valid mount message; mount the tape
    */
    if (strcmp(comment[com_idx[i]],"NA") != 0)
	{
	if (strcmp(tmdrive,"MOUNT") == 0)
	    status = mount_tape(unit,&i,quads[i]);
	else
	    status = reopen_tape(unit);

	*ex_quad = quads[i];
	break;
	}
     }

return(status);
}

/******************************************************************************
FUNCTION:	MOUNT_TAPE

PURPOSE:	Mounts and open the current input tape volume

ALGORITHM 
  Build string descriptor with appropriate mount message
  Open tape

ALGORITHM REFERENCES	none
*******************************************************************************/
int mount_tape
(
    long *unit,		/* pointer to tape unit number        		*/
    long *curvol,       /* current volume index	              		*/
    long quad		/* quadrant number		      		*/
)
{
extern short tape_flag;	/* Flag for tape open				*/
extern long com_idx[4];	/* index into comment array			*/
extern char *comment[4];/* mount messages for each tape			*/

long access = TREAD;	/* open tape for read access          		*/
long mode = HANDLE;	/* tape I/O outputs error messages    		*/
long rewopt = REWIND;	/* rewind option                      		*/
long fatal = -1;	/* Fatal error code				*/
long skip = 0;		/* skip zero files on open            		*/
long status;		/* function return status code        		*/

char temp[CMLEN];	/* mount message buffer		      		*/
char tapeid[CMLEN];     /* tape id					*/
char message[ERRLEN];	/* error message buffer		      		*/
char density[5];	/* tape density in BPI                		*/

strcpy(density,"0");
/*
  Build string descriptor with appropriate mount message
*/
if (strcmp(comment[com_idx[*curvol]],"NA") == 0)
    {
    sprintf (message,"The tape for quadrant %d is not available",quad);
    c_errmsg (message,"tipsp-notape",&fatal);
    }

strcpy(temp,comment[com_idx[*curvol]]);

/*
  Open tape
*/
status = c_tpopen(unit,&skip,&mode,&rewopt,density,&access,temp,tapeid); 
if (status != TAPE_SUCC)
    clean_up(unit,TIPS_MOUNT);

tape_flag = TRUE;

return(E_SUCC);
}

/******************************************************************************
FUNCTION:	REOPEN_TAPE

PURPOSE: 	Reopens a tape device.  The tape drive name is stored in the
		global "drive".  The drive name is saved in the TAE parameter
		block.

ALGORITHM 
  Reopen the tape drive
  Update the parameter block with the tape drive

ALGORITHM REFERENCES	none
*******************************************************************************/
int reopen_tape
(
    long *unit		/* pointer to tape unit number        		*/
)
{
extern struct PARBLK vblock;	/* TAE parameter block                  */
extern short tape_flag;		/* Flag for tape open			*/
extern char tmdrive[CMLEN];	/* Drive to reopen for the TM tape	*/

long access = TREAD;	/* open tape for read access          		*/
long mode = HANDLE;	/* tape I/O outputs error messages    		*/
long rewopt = REWIND;	/* rewind option                      		*/
long skip = 0;		/* skip zero files on open            		*/
long status;		/* function return status code        		*/

char *drive[1];		/* string descriptor for tape drive		*/
char density[5];	/* tape density in BPI                		*/

strcpy(density,"0");
/*
  Reopen the tape drive
*/
status = c_tpreop(unit,&skip,&mode,&rewopt,density,&access,tmdrive);
if (status != TAPE_SUCC)
    clean_up(unit,TIPS_MOUNT);

tape_flag = TRUE;

/*
  Update the parameter block with the tape drive
*/
strcpy(tmdrive,"MOUNT");
drive[0] = tmdrive;
status = q_string(&vblock,"TMDRIVE",0,drive,P_UPDATE);
status = q_out(&vblock);

return(E_SUCC);
}

/******************************************************************************
FUNCTION:	DISMOUNT_TAPE

PURPOSE:	Rewinds and dismounts the current tape volume.

ALGORITHM 
  Close the tape

ALGORITHM REFERENCES	none
*******************************************************************************/
void dismount_tape
(
    long *unit		/* pointer to tape unit number 			*/
)
{
extern short tape_flag;	/* Flag for tape open				*/
long status;		/* function return status code			*/
long eof = 0;		/* number of EOFs to be written         	*/
long nonfatal = 1;	/* Non-fatal error code				*/
long mode = HANDLE;	/* tape I/O outputs error messages      	*/
long rewopt = REWOFFL;	/* rewind the tape and take it offline  	*/

/*
  Close the tape
*/
status = c_tpclos(unit,&mode,&rewopt,&eof);
if (status != TAPE_SUCC)
    c_errmsg("Error closing tape","tipsp-tpclose",&nonfatal);

tape_flag = FALSE;

return;
}
