/******************************************************************************
FUNCTION:	VERIFY_WIN

PURPOSE:	Verifyies that the window specified is valid

ALGORITHM 
  Set the number of lines and samples in the first quadrant to process
  Based on the shape specified, determine the maximum number of lines and
  samples that may be processed
  Verify the window is valid

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tm.h"
#include "protos.h"

void verify_win
(
    long window[], 	/* user specified window 			*/
    char *imshape,		/* shape of the output image
				w - whole image
                                q - single quad
				u - upper half (quads 1 and 2)
				b - bottom half (quads 3 and 4)
				l - left half (quads 1 and 3)
				r - right half (quads 2 and 4)		*/
    long quads[]		/* quadrants to process				*/
)
{
			/* number of lines in each quad			*/
static long qline[MAX_QUADS] = {2983,2983,2982,2982};

			/* number of samples in each quad		*/
static long qsamp[MAX_QUADS] = {4220,4220,4220,4220};

long flag;		/* flag indicating whether windowing is necessary */
long imgsiz[2];		/* size of the input area			  */
long nl;		/* number of lines in first quad being processed  */
long ns;		/* number samples in first quad being processed   */

/*
  Set the number of lines and samples in the first quadrant to process
*/
nl = qline[quads[0]-1];
ns = qsamp[quads[0]-1];

/*
  Based on the shape specified, determine the maximum number of lines and
  samples that may be processed
*/
if (*imshape == 'q')
    {
    imgsiz[0] = nl;
    imgsiz[1] = ns;
    }
else if (*imshape == 'w')
    {
    imgsiz[0] = TMLINE;
    imgsiz[1] = TMSAMP;
    }
else if ((*imshape == 'u') || (*imshape == 'b'))
    {
    imgsiz[0] = nl;
    imgsiz[1] = TMSAMP;
    }
else 
    {
    imgsiz[0] = TMLINE;
    imgsiz[1] = ns;
    }

/*
  Verify the window is valid
*/
check_wind(window,imgsiz,&flag);

return;
}
