#ifndef _CIB2LAS_H_
#define _CIB2LAS_H_

#include"las.h"
#include"cibframe.h"
#include"get_par.h"

#define NOTAFRAMEFILE -1        /* file is not a frame file, most likely an
                                   overview file, is notincluded in output
                                   image                                    */
#define SAMPLES_FRAME 1536      /* number of samples in a frame file image  */
#define LINES_FRAME 1536        /* number of lines in a frame file image    */
#define SAMPLES_SUBFRAME 256    /* number of samples in a sub-frame image   */
#define LINES_SUBFRAME 256      /* number of lines in a sub-frame file      */


#define RGPF_TOC "A.TOC"        /* RPF Table of Contents filename           */
#define CIB_CENT_MER 0.0        /* Central Meridian (units = degrees) 
                                   CIB Central Meridian at 0.0 degrees      */
#define CIB_FE 0.0              /* False Easting (units = degrees) 
                                   CIB doesn't use False Easting            */
#define CIB_FN 0.0              /* False Northing (units = degrees)   
                                   CIB doesn't use False Northing           */
#define EARTH_RADIUS 6370997.0  /* default radius value for images
                                   where datum is defined                   */
#define CIBMAXIMGSIZE 1E10      /* maximum output image size chosen as
                                   a reasonable value to limit excessive
                                   image sizes                              */
#define MAXTILES 255            /* maximum number of tiles the TILENUM
                                   parameter can accept                     */

/***************************************************************************** 
    Location ID's for sections
 
    from MIL-STD-2411-1
         Registered Data Values for Raster Product Format 
         5.1.1 Section/Component ID Codes 
         pg. 6              
         
****************************************************************************/
#define LOC_HEADER_SECTION                   128
#define LOC_LOCATION_SECTION                 129
#define LOC_COVERAGE_SECTION                 130
#define LOC_COMPRESSION_SECTION              131
#define LOC_COMPRESSION_LOOKUP_SUBSECTION    132
#define LOC_COMPRESSION_PARAMETER_SUBSECTION 133
#define LOC_COLORGRAY_SECTION_SUBHEADER      134
#define LOC_COLORMAP_SUBSECTION              135
#define LOC_IMAGE_DESCR_SUBHEADER            136
#define LOC_IMAGE_DISPLAY_PARAM_SUBHEADER    137
#define LOC_MASK_SUBSECTION                  138
#define LOC_COLOR_CONVERTER_SUBSECTION       139
#define LOC_SPATIAL_DATA_SUBSECTION          140
#define LOC_ATTRIBUTE_SECTION_SUBHEADER      141
#define LOC_ATTRIBUTE_SUBSECTION             142
#define LOC_EXPLICIT_AREAL_TABLE             143
#define LOC_RELATED_IMAGE_SECTION_SUBHEADER  144
#define LOC_RELATED_IMAGE_SUBSECTION         145
#define LOC_REPLACE_UPDATE_SECTION_SUBHEADER 146
#define LOC_REPLACE_UPDATE_TABLE             147
#define LOC_BOUNDARY_SECTION_SUBHEADER       148
#define LOC_BOUNDARY_RECTANGLE_TABLE         149
#define LOC_FRAME_FILE_INDEX_SUBHEADER       150
#define LOC_FRAME_FILE_INDEX_SUBSECTION      151
#define LOC_COLOR_TABLE_SECTION_SUBHEADER    152
#define LOC_COLOR_TABLE_INDEX_RECORD         153

/* structure to hold information about a single frame file
----------------------------------------------------------------------------*/
typedef struct framefile 
{
    char  exists;         /* frame file exists if TRUE                      */ 
    char  * dir,          /* directory frame file stored in                 */
          filename[16],   /* frame file name                                */
          zone;           /* CIB zone frame file located in                 */
    int   res,
          brect_record;   /* boundary rectangle record number 
                             frame file contained within                    */
    long  column,         /* column of frame file in CIB zone               */
          row,            /* row of frame file in CIB zone                  */
          fnum;           /* frame file number in CIB zone 
                             counting starts at row 0 column 0
                             increasing in row major order                  */
          
} FRAMEFILE;

/* structure to hold information on the CIB volume
----------------------------------------------------------------------------*/
typedef struct dims
{
    long  maxrow;      /* highest CIB zone row in volume                    */
    long  maxcolumn;   /* highest CIB zone column in volume                 */
    long  numcolumns;  /* number of columns in CIB zone                     */
    long  numrows;     /* number of rows in CIB zone                        */
} CIB_DATA; 

/* structure to hold information from location section of RPF header. 
---------------------------------------------------------------------------*/
typedef struct location
{
    unsigned short   id;           /* component id                          */
    unsigned int     length;       /* component length                      */
    long             phys_index;   /* component physical location           */
} LOCATION;


/* structure to hold frame file record information read from the RPF Table
   of Contents file A.TOC
----------------------------------------------------------------------------*/
typedef struct frame_entry
{
  int		exists;       /* frame file exists, an image exists at this 
                             location (row & column                         */
  char		*directory;   /* path to frame file under RPF directory         */
  char		filename[16]; /* frame file filename                            */
} FRAME_ENTRY;


int check_endian
(   
    int little_endian /* I: 0xFF - little endian 0x00 - big endian          */
);

int swap_bits              /* returns status of variable do_swap            */
(
    void
);

void swap_endian
(
    void *ptr,             /* I/O: pointer to string to reverse             */
    int count              /* I:   number of characters in string
                                    to reverse                              */
);
                                               
int parse_locations
(
    FILE *fin,             /* I: FILE pointer to TOC file                   */
    int count,             /* I: number of locations to be found            */ 
    char *filename,        /* I: filename of file reading from              */
    LOCATION *locs         /* I/O: structure containng locations to be
                                   found                                    */
);  

FRAMEFILE *read_toc
( 
    char *dir,             /* I: input directory to TOC file                */ 
    char *device,          /* I: drive CD located on                        */
    int  tilenum[MAXTILES],/* I: user-specified boundary rectangles to ingest*/
    char *zone,            /* I/O: CIB zone of requested image              */
    int  *fcount           /* O: number of frame files found                */ 
);

int find_rpf_hdr
(
    FILE    *ffptr         /* I:    input file pointer to TOC file          */ 
);

int get_nitfhdr
(
    FILE    *ffptr         /* I:    input file pointer to TOC file          */ 
);

int get_hdr
(
     FILE                *ffptr,   /* I: frame file pointer                 */
     struct CIBHeader    *cibhdr   /* O: struct. to hold  header info       */
);

int find_loc
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBHeader    *cibhdr  /* O: struct to hold  header info        */

);

int get_loc
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBLocation  *cibloc  /* O: struct to hold location info       */
);

int get_cov
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBCoverage  *cibcov  /* O: struct to hold CIB coverage 
                                         info                               */
);

int get_comphdr
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBCompress  *cibcom  /* O: struct to hold compression
                                                data                        */
);

int get_comptbl
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBCompress  *cibcom  /* O: struct to hold compression
                                         data                               */
);

int get_colhdr
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBColor     *cibcol  /* O: struct to hold color look up
                                         table(s)                           */
);

int get_colmap
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBColor     *cibcol  /* O: struct to hold color look up
                                         table(s)                           */
);

int get_imghdr
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBImage     *cibimg  /* O: struct to hold frame file 
                                         image info                         */
);

int get_imgdsp
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBImage     *cibimg  /* O: struct to hold frame file 
                                         image info                         */
);

int get_imgmask
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBImage     *cibimg  /* O: struct to hold frame file 
                                         image info                         */
);

int get_colcon
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBColor     *cibcol  /* O: struct to hold color look up
                                         table(s)                           */
);

int get_imgpix
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBImage     *cibimg  /* O: struct to hold frame file 
                                         image info                         */
); 

int get_uimgpix
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBImage     *cibimg  /* O: struct to hold frame file 
                                         image info                         */
);

int get_atthdr
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBAttribute *cibatt  /* O: struct to hold attribute info      */
);

int get_atttbl
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBAttribute *cibatt  /* O: struct to hold attribute info      */
);

int get_attcov
(
      FILE                *ffptr,  /* I: frame file pointer                 */
      struct CIBAttribute *cibatt  /* O: struct to hold attribute info      */
);

void bld_img
(
     struct CIBCompress   *cibcom, /* I: struct holding compression info    */
     struct CIBColor      *cibcol, /* I: struct holding CLUT                */
     struct CIBImage      *cibimg  /* O: struct to hold frame file image    */
);

void bld_uimg
(
    struct CIBColor       *cibcol, /* I: struct holding CLUT                */
    struct CIBImage       *cibimg  /* O: struct to hold frame file image    */
);

void wrt_img
(
     struct CIBImage      *cibimg, /* I: struct holding frame file image    */
     unsigned char        *obuffer /* O: buffer to write frame file image
                                         into                               */
);

int read_frame
(
    char *fname,            /* I: path to and name of frame file to read    */
    char fillval,           /* I: specified image fill value                */
    unsigned char *obuffer  /* O: output (array) buffer for frame data      */
);

int read_frame_attributes
(
    FILE *ffptr,                /* I: file pointer to CIB frame file        */
    struct CIBAttribute	*cibatt /* O: structure to hold frame file 
                                      attributes                            */
);

#ifdef DEBUG

    int cib_print
    (
        int type, 
        FRAMEFILE **matrix,
        long minrow,
        long mincolumn,
        INPARAMS *s,
        CIB_DATA *d
    );

    void prt_hdr
    (
         struct CIBHeader     *cibhdr        
    );

    void prt_loc
    (
         struct CIBLocation   *cibloc
    );

    void prt_cov
    (
         struct CIBCoverage   *cibcov
    );

    void prt_com
    (
        struct CIBCompress    *cibcom
    );

    void prt_col
    (
         struct CIBColor      *cibcol
    ); 

    void prt_img
    (
         struct CIBImage      *cibimg
    );

    void prt_att
    (
        struct CIBAttribute   *cibatt
    ); 

    void prt_new
    (
         struct CIBUpdate     *cibnew
    );

    int get_newhdr
    (
          FILE                *ffptr,  /* I: frame file pointer             */
          struct CIBUpdate    *cibnew  /* O: struct to hold new/update info */
    );

    int get_newtbl
    (
         FILE                 *ffptr,  /* I: frame file pointer             */
         struct CIBUpdate     *cibnew  /* O: struct to hold new/update info */
    );
    
#endif /* DEBUG */


#endif
