#ifndef GET_PAR_H
#define GET_PAR_H
#include "las.h" /* for PARBLK defn. */

#define MAXTILES 255

/* used to indicate invalid CIB zone
------------------------------------*/
#define INVALID_ZONE 'Z'

/* structure to hold user input paramater specifications 
--------------------------------------------------------*/
enum window_specifications {SLSS,SLSSNLNS,COORS2,COORS4,DEFAULT};

typedef struct inparams
{
   long  sl;             /* starting line of CIB image to output            */
   long  ss;             /* starting sample of CIB image to output          */
   long  nl;             /* number of lines of CIB image to output          */ 
   long  ns;             /* number of samples of CIB image to output        */

   double nw_lat;        /* upper left latitude (degrees)                   */  
   double nw_y;          /* upper left y proj. coord. (meters)              */  

   double nw_long;       /* upper left longitude (degrees)                  */
   double nw_x;          /* upper left x proj. coord. (meters)              */

   double se_lat;        /* lower right lat. (degrees)                      */
   double se_y;          /* lower right y proj. coord. (meters)             */

   double se_long;       /* lower right long.  (degrees)                    */
   double se_x;          /* lower right x proj. coord. (meters)             */

   int    tilenum[MAXTILES];
                         /* boundary rectangles to ingest - -1 indicates
                            end of list has been reached                    */

   enum   window_specifications type;/* how window was specified            */
   int    fill;           /* fill value for tramsparent pixels, 
                             and empty frames, default = 0 (black)          */
   char   indir[CMLEN];   /* input directory, directory above
                             "/RPF" directory                               */
   char   units[4];       /* user specified input window units
                                 "LS"   line/sample
                                 "DEG"  Degrees
                                 "DMS"  Degree Minutes Seconds
                                       (+DDDMMMSSS.SS)
                                 "PRO"  Projection Coordinates
                                       (meters)                             */
   char minmax;           /* value of LAS global minmax                     */
   long msgtime;          /* value of LAS global msgtime                    */
}INPARAMS;

int get_par
(
    struct PARBLK *vblock, /* I: Pointer to TAE parameter block             */
    char taeout[][CMLEN],  /* O: TAE file name of output image              */
    char comment[CMLEN],   /* O: comment to install CD                      */
    INPARAMS  *inparams    /* O: structure holding Operator input params    */
);

#endif
