#ifndef _NITFACCESS_H_
#define _NITFACCESS_H_
#include"cibframe.h"


/*-------------------------------------------------------------------------- 
   Returns the [location] section address (starting byte in file) in struct
   cibhdr->location field.  This is the location section address of the
   RPF formatted header.  
----------------------------------------------------------------------------*/

int  loc_rpf_header 
(
    FILE *ntfimg  /* I: file pointer to a NITF formatted file */
);


/*-------------------------------------------------------------------------
   Get the address of the locations section of the RPF header located
   in the RPF Table of Contents (TOC) file.

   Returns the [location] section address (starting byte in file) in struct
   cibhdr->location field
----------------------------------------------------------------------------*/
int get_location_address
(
    FILE	    	 *ffptr, /* I: FILE pointer to CIB image frame file     */
    struct CIBHeader *cibhdr /* O: contains TOC information                 */ 
);

#ifdef DEBUG

/* 
	NITF Data Structures for:
		File Header
		Image Subheader

   Code taken from sgibrowse
   
*/

struct NITF_File_Header{
	char		FHDR[10];	/* File Type and Version */
	char		CLEVEL[3];	/* Compliance Level */
	char		STYPE[5];	/* System Type */
	char		OSTAID[11];	/* Originating Station ID */
	char		FDT[15];	/* File Date and Time */
	char		FTITLE[81];	/* File Title */
	char		FSCLAS[2];	/* Security Class */
	char		FSCODE[41];	/* Security Codewords */
	char		FSCTLH[41];	/* File Control and Handling */
	char		FSREL[41];	/* File Releasing Instructions */
	char		FSCAUT[21];	/* File Classification Authority */
	char		FSCTLN[21];	/* File Security Control Number */
	char		FSDWNG[7];	/* File Security Downgrade */
	char		FSDEVT[41];	/* File Downgrading Event */
	char		FSCOP[6];	/* Message Copy Number */
	char		FSCPYS[6];	/* Message Number of Copies */
	char		ENCRYP[2];	/* Encryption */
	char		ONAME[28];	/* Originator's Name */
	char		OPHONE[19];	/* Originator's Phone Number */
	char		FL[13];		/* File Length */
	char		HL[7];		/* NITF File Header Length */
	char		NUMI[4];	/* Number of Images */
	char		LISH001[7];	/* Length of 1st Image Subheader */
	char		LI001[11];	/* Length of 1st Image */
	char		NUMS[4];	/* Number of Symbols */
	char		LSSH001[5];	/* Length of 1st Symbol Subheader */
	char		LS001[7];	/* Length of 1st Symbol */
	char		NUML[4];	/* Number of Labels */
	char		LLSH001[5];	/* Length of 1st Label Subheader */
	char		LL001[4];	/* Length of 1st Label */
	char		NUMT[4];	/* Number of Text Files */
	char		LTSH001[5];	/* Length of 1st Text Subheader */
	char		LT001[6];	/* Length of 1st Text File */
	char		NUMDES[4];	/* Number of Data Extension Segments */
	char		LDSH001[5];	/* Length of 1st DES Subheader */
	char		LD001[10];	/* Length of 1st DES */
	char		NUMRES[4];	/* Number of Reserved Extension Segments */
	char		LRSH001[5];	/* Length of 1st RES Subheader */
	char		LR001[8];	/* Length of 1st RES */
	char		UDHDL[6];	/* User Defined Header Data Length */
	char		UDHD[4];	/* User Defined Header Data */
	char		RETAG[7];	/* Unique Extension type Identifier */
	char		REL[6];		/* Length of REDATA field */
	char		XHDL[6];	/* Extended Header Data Length */
} nitfhdr;
#endif /* DEBUG */

#endif /*  _NITFACCESS_H_ */
