/*****************************************************************************
FILENAME:  swap.c  
FUNCITONS  check_swap()
           swap_endian()

PURPOSE:  Check for a mismatch in endianess between CIB data and the systen
this software is running on.  static int do_swap hold the status of the mis-
match as determined by the function check_swap.  function swap_bits returns
the status of the mismatch, TRUE if mismatched, FALSE otherwise.  Swap will
swap the bit order of the string passed (used to correct mis-match).

PROGRAM HISTORY:

VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
   1.0      B. Gross        2/00            CSB         original development 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:    LAS
ALGORITHM DESCRIPTION:
ALGORITHM REFERENCES:

*****************************************************************************/
#include <stdio.h>
#include "cib2las.h"

/* if TRUE, swap bits, if FALSE, don't
--------------------------------------*/
static int do_swap;  

/* check for mismatch in endianess between host OS and CIB data.  If 
   data doesn't match, set do_swap above to TRUE indicate mismatch 
---------------------------------------------------------------------*/
int check_endian
(   
    int little_endian /* I: CIB Data Formats 0xFF - little endian 
                                             0x00 - big endian              */
)
{
    unsigned char       *c_ptr;
    unsigned short      s;
 
    s = 1;
    
    /* value of *c_ptr will depend on current system's 
       method of endcoding  big endian/ little endian
    -------------------------------------------------*/
    c_ptr = (unsigned char *) &s;
    
    /* operating system is big endian and data
       stored in little endian, swap
    -------------------------------------------*/
    if (*c_ptr == 0 && little_endian)
        do_swap = TRUE;

    /* else if operating system is little endian
       and data stored in big endian, swap
    --------------------------------------------*/
    else if (*c_ptr == 1 && !little_endian)
        do_swap = TRUE;
    else
        do_swap = FALSE;
    return (do_swap);
}

/****************************************************************************
  swap "count" number of bytess, in string passed by ptr 
 ****************************************************************************/
void swap_endian
(
    void *ptr,           /* I: pointer to string containing number to
                               reverse                                      */
    int count            /* I: number of bytes in string to reverse         */
)
{
    unsigned char *end;
    unsigned char temp;
    unsigned char *ch_ptr = (unsigned char *) ptr;

    if(do_swap == TRUE)
    {
        end = &ch_ptr[count - 1];
        while (end > ch_ptr)
        {
          temp = *end;
          *end = *ch_ptr;
          *ch_ptr = temp;
          ch_ptr++, end--;
        }
    }
}
