#include <stdio.h>
#include "las.h"
#include "protos.h"
/******************************************************************************

NAME:	ARANGE

FUNCTION:   Determine the output image size and magnify picture description 
            table accordingly.


PROGRAM HISTORY:
  Version	Date        Author      		Request
  -------	----	    ------                      -------
   1.0        19-Feb-82    J. Hwang, SASC
   2.0         7-Jan-88    M. Mazzella SAR/GSFC       Converted to C
   3.0        20-Dec-90    K. Donaldson               Added comments
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Call IDX function to search the picture description table
      and locate image area description.
   Call INTSCT to adjust window parameter specifications if
      window exceeds the input image in any direction, and
      to set window to default of entire image if window
      was not specified by the user.
   Use IDX function to select ntimes, where ntimes determines
      which reserve size that the output image will be, if
      the truncate option is set, the reserve size will be
      the next smallest size.
   Call MAGTBL to reset the picture description table from
      a 512 x 512 format to a 512*ntimes x 512*ntimes format
      reset imgsiz to equal the reserve display size.
   Call INTSCT to set picture description table so that
      the image rectangle description will be either the 
      entire input image, or, if truncate is set then the
      image rectangle will describe the area of the input
      image that will fit the embedded image area in the
      reserve display selected.
******************************************************************************/

void arange
(
    long window[],	/* user defined window to be applied to input image */
    long trnc,		/* truncation flag: 1 - truncate: select output image*/
    long imgsiz[]	/* input image size */
)

{

long	i,		/* loop counter */
	loc,		/* location of imgsiz in picture description table */
	len,		/* number of entries in picture description table */
	ntimes,		/* number of times to magnify table */
	sizes[10];	/* possible image sizes	*/

#include "externs.h"

/*
Get the image size descritption.
----------------------------------
*/
sizes[0]= 444;
sizes[1]= 436;
sizes[2]= 888;
sizes[3]= 872;
sizes[4]= 1776;
sizes[5]= 1744;
sizes[6]= 3552;
sizes[7]= 3488;
sizes[8]= 7104;
sizes[9]= 6976;

/*
Call IDX function to search the picture description table
and locate image area description.
-----------------------------------------------------------
*/
len = tb[6];
loc = idx(1,0,tb,len,10,TRUE);
loc = (loc - 1) * 10;

/*
Call INTSCT to get the intersection between imgsiz and window and store 
the result in window. 
If window was not specifed, set window = imgsiz .
--------------------------------------------------------------------------
*/
if ((window[0] == 0) || (window[1] == 0) || 
    (window[2] == 0) || (window[3] == 0)) 
   {
   for ( i = 0; i < 4; i++)
      window[i] = imgsiz[i];
   }
else
   intsct (window, imgsiz, FALSE);

/*
Call IDX to select the magnification factor.
Call MAGTBL to magnify picture description table ntimes.
---------------------------------------------------------
*/
ntimes = idx (window[3], window[2], sizes, 5, 2, FALSE) - trnc;
if (ntimes <= 0) 
   ntimes = 1;
magtbl (ntimes);

/*
Now imgsiz contains output image size.
---------------------------------------
*/
imgsiz[2] = tb[2];
imgsiz[3] = tb[3];

/*
Call INTSCT to get the intersection between window and image size in square
description.
Store the result in image description.
---------------------------------------------------------------------------
*/
intsct (&tb[loc], window, TRUE);
}
