#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"
/******************************************************************************

NAME:	BIT

FUNCTION:   Picks a bit value in a given long word.

PROGRAM HISTORY:

  Version	Date        Author 		Request
  -------	----	    ------              -------
   1.0        30-Nov-81   J. Hwang,
   2.0         7-Jan-88   M. Mazzella 		Conversion to C
   3.0        20-Dec-90   K. Donaldson          Added more iformative comments.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):

   If position is if out of range, 
      Print a fatal error message.
   Pick up byte value which includes desired bit by performing a bitwise 
      "AND" with POSth bit of WORD.
   Return TRUE or FALSE depending on the result of the "AND" operation.

******************************************************************************/


long int bit 
(
unsigned char word,         /*  source word  */
unsigned char pos           /*  position of bit desired  */
)

{
extern  struct  PARBLK  block;          /* TAE parameter block */
static  char    *prog_name = "CLASSMAP"; /* Program name */
extern  char    clean_files[2][CMLEN];  /* Clean file names */
extern  long    num_outfiles;           /* Number of output files */


/*
If position is if out of range, print a fatal error message.
--------------------------------------------------------------
*/

if ((pos > 8) || (pos < 1)) 
   c_abort(&block, "Exceeded bit range(1-32) value", "classmap-bitrange",
           prog_name, clean_files, &num_outfiles);

/*
Pick up byte value which includes desired bit by performing a bitwise 
"AND" with POSth bit of WORD.
Return TRUE or FALSE depending on the result of the "AND" operation.
-----------------------------------------------------------------------
*/

if ((1 << (pos - 1)) & word)
   return (TRUE);
else
   return (FALSE);
}
