
#include <stdio.h>
#include "las.h"
#include "protos.h"
/******************************************************************************

NAME:   GENCHR

FUNCTION:   Generate expanded dots of character to given buffer.

PROGRAM HISTORY:
  Version	Date       Author      		 Request
  -------	----	   ------            	 -------
   1.0        4-Feb-82    J. Hwang, SASC
   2.0        7-Jan-88    M. Mazzella SAR/GSFC  Convert to C
   3.0        4-Apr-89    K. Narayanan          Workbuf index was incorrect
   4.0       27-Jun-89    K. Narayanan          Corrected annotation for non-VMS
   5.0       20-Dec-90    K. Donaldson          Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   If table description indicates out of range, 
      Return.
   Set up variables for achar:
      Get the scale factor and the number of character from table.
   Set up variable for dchar:
      Get the scale factor and set the number of character to 1.
   Transfer current line of character dots to buffer:
      Get ascii character code and convert code to index into the dot table.
      If illegal char then use question mark.
      Get a byte of the character dot table.
      If bit is on then, 
         Fill buffer with grey scale as directed in the table scale 
         factor times.
      Otherwise, 
         Use back ground grey from picture description table to fill 
         buffer scale factor times
      Increment buffer index.

******************************************************************************/

int genchr
(
    long vcnt,		/* Current line count on the picture */
    long tab[],		/* Square description  */
    char chrs[]		/* Character(s) to be expanded */
)
{
#include "classmap.h"

unsigned char	bt,	/* Position of bit to be checked */
		code,	/* Character to be written */
		k;	/* Loop counter */

extern unsigned char *workbuf;	/* Storage for line  */
extern long offset[]; 		/* Offsets into imageio buffers */

float   fscale;		/* Scale factor */

long	gsc,		/* Grey scale for tick marks */
	i,		/* Loop counter  */
	index,		/* Index into table for characters to be written */
	lc,		/* Location of index */
	m,		/* Loop counter  */
	nchar,		/* Number of characters */
	pointr,		/* Tic mark indicator */
	status = E_SUCC,/* Return status */
	scale,		/* Scale factor */
	x;

#include "externs.h"

/*
If table description indicates out of range, return.
----------------------------------------------------
*/
if ((vcnt < tab[0]) || (vcnt > tab[0] + tab[2] - 1))
   return(status);

index = 1;
tab[8]++;

/*
Set up variables for achar:
get the scale factor and the number of character from table.
-------------------------------------------------------------
*/
if (!(tab[2] > tab[3]))
   {
   fscale = (float)tab[3] / (float)(tab[6] * 8);
   fscale += 0.5;
   scale = (long)fscale;
   nchar = tab[6];
   }

/*
Set up variable for dchar:
get the scale factor and set the number of character to 1.
-----------------------------------------------------------
*/
else
   {
   fscale = (float)tab[2] / (float)(tab[6] * 10);      
   fscale += 0.5;
   scale = (long)fscale;
   index = index + (tab[8] - 1) / (scale * 10);
   nchar=1;
   }

/*
Transfer current line of character dots to buffer:
get ascii character code and convert code to index into the dot table.
If illegal char then use question mark.
-----------------------------------------------------------------------
*/
lc = ((tab[8] - 1) / scale) + 1;      
pointr = 0;
for (i = 1; i <= nchar; ++i)
   {
   if (chrs[index + i - 2] == 0) 
       code=0;
   else 
       code = chrs[index + i - 2] - 32;
   if (code > 95) 
      code = 31;
#ifdef vms
   x = 8 * ((lc - 1) / 4) + 4 - lc;   /* bytes in a word have to be reversed */
#else
   x = lc - 1;
#endif

/*
Get a byte of the character dot table.
If bit is on then, fill buffer with grey scale as directed in the table
scale factor times.
Otherwise, use back ground grey from picture description table to fill 
buffer scale factor times
Increment buffer index.
-------------------------------------------------------------------------
*/
   bt = dots[code][x];
   for (k = 8; k ; --k)
      {
      gsc = 5;
       if (bit(bt,k) != 1)
         gsc = 4; 
      code = (unsigned char)tab[gsc];
      for (m = 1; m <= scale; ++m)
         {
         workbuf[tab[1] + pointr + m - 1] = code;
         }
      pointr += scale;
      }
   }
return(status);
}
