#include <stdio.h>
#include "las.h"
#include "protos.h"
/******************************************************************************

NAME:  GENTIK

FUNCTION:  Generate tick marks.

PROGRAM HISTORY:
  Version	Date       Author      		  Request
  -------	----	   ------            	  -------
   1.0        4-Feb-82   J. Hwang, SASC
   2.0        8-Jan-88   M. Mazzella, SAR/GSFC    Convert to c
   3.0        4-Apr-89   K. Narayanan             Index was incorrect

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM

   Get tick line count.
   Do until desired pixels are filled.
      If ticks are not across the top or bottom of the image then
         If hcount is beyond horizontal tick length then
            Set grey scale to background.
         Else
            Set grey scale to tick color.
      End-If
      If ticks are across the top or bottom of the image then
         If vcount is beyond vertical tick length then
            Set grey scale to background.
         Else
            Set grey scale to tick color.
      End-If
      Set proper values into buffer.
   End-Do
******************************************************************************/

int gentik
(
    long vcnt,		/* Vertical line count */
    long tab[]		/* Array containing tick information */
)
{

#include "externs.h"

extern unsigned char	*workbuf;	/* Work buffer */

long	count,		/* Place holder for tick mark	*/
	gsc,		/* Grey scale value		*/
	hcount,		/* Horizontal tick mark count	*/
	index,		/* Index into work buffer	*/
	pointr,		/* Pointer to position on line	*/
	twidth,		/* Width of tick mark		*/
	vcount;		/* Vertical line count		*/

/*
If current line count is not in the range of square description then return.
----------------------------------------------------------------------------
*/
if ((vcnt < tab[0]) || (vcnt > tab[0] + tab[2] - 1))
   return(E_SUCC);

/*
Get the tick line count since beginning of the current square.
---------------------------------------------------------------
*/
vcount = vcnt - tab[0];

/*
Do the following until desired pixels are filled.
---------------------------------------------------
*/
for ( hcount = 1; hcount <= tab[3]; ++hcount)
   {
   gsc = 4;                                /*  assume base grey scale */
   pointr = hcount - 1;                    /*  assume it is atic */
   twidth = 7;

/*
If ticks are not across the top or bottom of the image then
If hcount is beyond horizontal tick length then set grey scale to background.
Otherwise, set grey scale to tick color.
-----------------------------------------------------------------------------
*/
   if (!(tab[3] > tab[2]))                 /*  if a tic */
      {
      pointr = vcount;
      twidth = 6;
      if (hcount <= tab[7] )   /*  check if hcount is > h-length of dtic */
      count = pointr % tab[8];
         if (count < tab[twidth] )
            gsc = 5;
      }

/*
If ticks are across the top or bottom of the image then
If vcount is beyond vertical tick length then set grey scale to background.
Otherwise, set grey scale to tick color.
----------------------------------------------------------------------------
*/
   else
      {
      if ( vcount <= tab[6] )
         count = pointr % tab[8];
         if (count < tab[twidth] )
            gsc = 5;
      }

/*
Set proper values into work buffer.
-----------------------------------
*/
   index = tab[1] + hcount - 1;
   workbuf[index] = (unsigned char)tab[gsc];
   }

return(E_SUCC);
}
