#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/******************************************************************************

NAME:   GETIMG

FUNCTION:  Get a line of image data and return proper data.

PROGRAM HISTORY:
  Version	Date       Author      		Request
  -------	----	   ------            	-------
   1.0        19-Feb-82   J. Hwang, SASC
   2.0         8-Jan-88   M. Mazzella SAR/GSFC  Convert to C
   3.0         4-Apr-89   K. Narayanan          Workbuf index was incorrect
   4.0        20-Dec-90   K. Donaldson          Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Read a line of input image.
   Store the input image line in the work buffer.
   Return

******************************************************************************/

int getimg
(
    long vcnt,			/* Vertical line count */
    long tab[]			/* Information about image line */
)

{
#include "classmap.h"

long	i, 			/* Loop counters */
	status = E_SUCC;	/* Return status */

extern long	gdescin,	/* Input image group descriptor */
		offset[];	/* Offsets into image i/o buffers */

extern unsigned char *bufferin,	/* Input image i/o buffer */
		     *workbuf;	/* Image work buffer */

/*
If current line count is not in the range of square description then return.
----------------------------------------------------------------------------
*/
if ((vcnt < tab[0]) || (vcnt > tab[0] + tab[2] - 1))
   return(status);

/*
Read a line of input image.
-----------------------------
*/
status = c_estep((struct GDESC **)&gdescin);
if (status != E_SUCC)
   c_abort (&block, "Error reading input image", "classmap-read", prog_name,
             clean_files, &num_outfiles);

/*
Store the input image line in the work buffer.
-----------------------------------------------
*/
for (i = 0; i < tab[3]; ++i)
   workbuf[i+tab[1]] = bufferin[offset[0]+i];

return(status);
}
