#include <stdio.h>
#include "las.h"
#include "protos.h"
/******************************************************************************

NAME:  GETINF

FUNCTION:   Get information for classmap (including parameters)

PROGRAM HISTORY:
  Version	Date       Author      	     	Request
  -------	----	   ------            	-------
   1.0        19-Feb-82  J. Hwang, SASC
   2.0         8-Jan-88  M. Mazzella, SAR/GSFC  Convert to C
   3.0        20-Dec-90  K. Donaldson           Added comments
   5.0        05-Feb-91  K. Donaldson           Added code to process only
                                                the 1st band if input image
                                                has multiple bands.
   6.0        18-Oct-99  D. Hames               Add parameter ns.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Call GETPAR to get user specified parameters from tae.
   Save the input image size.
   Call ARRANGE to determine the size of the output image, and 
   magnify properly.

******************************************************************************/

void getinf
(
    char hosin[],		/* Host name of input image		*/
    long windin[4],		/* Input image window			*/
    long bands[MAXIMG],		/* Input image bands			*/
    long num_img_bands[][MAXBND+1],  /* Band numbers for each image */
    struct DDR *ddr,		/* Input ddr				*/
    char usrano[],		/* User defined anotation		*/
    char taeout[],		/* Tae name of output image		*/
    char hosout[],		/* Host name of output image		*/
    char clschr[][8],		/* Class names				*/
    unsigned char class[],        /* Array of user defined classes	*/
    char fname[],		/* Facility name			*/
    long *msgtim,		/* Time for the processing message      */
    long imgsiz[],		/* Output image size			*/
    char dltfentr[],		/* User specified dltfentry		*/
    char indltf[],		/* User specified dltf name		*/
    long *usrcol_flg,		/* Flag for user specified colors       */
    long *numcls,		/* Number of input classes		*/
    long *ns			/* number of samples in image           */
)

{
long    trnc,			/* Image truncation flag 		*/
	window[MAXIMG][4];	/* Input image window			*/

/*
Call GETPAR to get user specified parameters.
----------------------------------------------
*/

getpar (hosin, windin, window, bands, num_img_bands, ddr, usrano, taeout, 
        hosout, &trnc, clschr, class, fname, msgtim, indltf, dltfentr, 
        usrcol_flg, numcls);

/*
Save the input image size.
---------------------------
*/
imgsiz[SL] = 1;
imgsiz[SS] = 1;
imgsiz[NL] = ddr->nl;
imgsiz[NS] = ddr->ns;

/*
Save number of samples
----------------------
*/
*ns = ddr->ns;


/*
Call ARRANGE to determine the size of the output image, and magnify properly.
-----------------------------------------------------------------------------
*/
arange (windin, trnc, imgsiz);
}
