#include <stdio.h>
#include "las.h"
#include "protos.h"
/******************************************************************************

NAME:  IDX

FUNCTION:   If eq is true then search the passed table(tbl)
            for a value that matches the value of a, return the index
            that points to the matching value's position in the table

            If eq is false then compare a (np of an image) and b (nl
            of an image) to a table containing a set of reserve sizes,
            return the index to the pair of values in the table that
            are equal to a and b, or that are one step larger than
            a and b

PROGRAM HISTORY:
  Version	Date       Author      	     	Request
  -------	----	   ------            	-------
   1.0       19-Feb-82   J. Hwang, SASC
   2.0        8-Jan-88   M. Mazzella SAR/GSFC  Convert to C
   3.0       20-Dec-90   K. Donaldson          Added more informative comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

      For 1 to table size (tsize)
         Find the requested index in specified table (tbl)
      End-For
      Return the index

******************************************************************************/

long	int idx 
(
long    a,              /* Number of pixels in the image */
long    b,              /* Number of lines in the image */
long    tbl[],          /* Table of values */
long    tsize,          /* Table size */
long    jump,           /* Number of values between each one checked */
long    eq              /* Equivalence flag */
)

{

long	i,		/* Loop counter */
	k;		/* Index into tbl */

#include "externs.h"

/*
Find index for the same value or a value that is greater than or equal to 
but, at the same time, is the smallest value in table.
---------------------------------------------------------------------------
*/
for ( i = 0; i < tsize; i++)
   {
   k = jump * (i + 1) - 1;
   if (((a <= tbl[k-1]) && (b <= tbl[k]) && (!(eq))) || 
       ((eq) && (a == tbl[k]))) 
      break;
   }
return(i+1);
}
