#include <stdio.h>
#include "las.h"
#include "protos.h"
/******************************************************************************

NAME:  INTSCT

FUNCTION:   Get the intersection between two windows.

PROGRAM HISTORY:
  Version	Date       Author      		     Request
  -------	----	   ------            	     -------
   1.0        19-Feb-82   J. Hwang, SASC
   2.0         8-Jan-88   M. Mazzella SAR/GSFC      Convert to C
   3.0        20-Dec-90   K. Donaldson              Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   If size comparison only then
      Take smaller size in both direction (vertical, horizontal)
   Else
      Get the intersection between a and b and 
      Put the result into a.
   End-If
   If no intersection was found then 
      Print and error message and
      Set a = b.
   End-If

******************************************************************************/

void intsct
(
    long a[],		/* First input window and an output window for the */
    long b[],		/* Second input window */
    long abssiz		/* True if absolute size comparison desired */
)
{
#include "classmap.h"
long	k,		/* Loop counter	*/
	m[2];		/* Temporary holding variable */

/*
If size comparison only then take smaller size in both direction
(vertical, horizontal).
-----------------------------------------------------------------
*/
if (abssiz) 
   {
   if (a[2] > b[2]) 
      a[2] = b[2];
   if (a[3] > b[3]) 
      a[3] = b[3];
   }
/*
Otherwise, get the intersection between a and b and put the result into a.
--------------------------------------------------------------------------
*/
else
   {
   if (!((a[0] > b[0] + b[2]) || (b[0] > a[0] + a[2]) || 
         (a[1] > b[1] + b[3]) || (b[1] > a[1] + a[3])))
      {
      for (k = 0; k < 4; ++k)
         {
         if (k <= 1) 
            {
            m[k] = a[k];
            if (a[k] < b[k]) 
               a[k] = b[k];
            }
         else
            {
            if (a[k] + m[k-2] > b[k] + b[k-2]) 
               a[k] = b[k] + b[k-2] - a[k-2];
            else
               a[k] = a[k] + m[k-2] - a[k-2];
            }
         } 
      }
/*
If no intersection was found, print an error message and set a = b.
----------------------------------------------------------------------
*/
   else
      {
      c_errmsg ("No intersection between two windows", "classmap-intsct",
                &inf_err);
      for (k = 0; k < 4; ++k)
         a[k] = b[k];
      }  
   }  
}
