#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"
/*
**********************************************************************

NAME:  LINGEN

FUNCTION:   Generate an output image line.

PROGRAM HISTORY:
  Version	Date       Author      		     Request
  -------	----	   ------            	     -------
   1.0        19-Feb-82   J. Hwang, SASC
   2.0         8-Jul-88   M. Mazzella SAR/GSFC      Convert to C
   3.0        20-Dec-90   K. Donaldson              Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

       For the number of rows in the picure description table
          If line number belongs to the vertical range of the 
          Rectangular description 
             If square type is simple square
                 Call GENSQR to generate simple square
             Else if square type is image
                 Call GETIMG to get a line of image
             Else if square type is character 
                 Call GENCHR to get a line of character dot
             Else if square type is tick
                 Call GENTIK to get a line portion  of tick 
             Else if delay generation then 
                 Delay character generation to wrimg 
             Else
                 Issue an error message and abort

**********************************************************************

*/
void lingen
( 
    long vcnt,		/* Current line count */
    long tbsize 	/* Number of rectangular description in the table */
)
{
#include "classmap.h"

long 	j, 		/* Loop counter and array index */
	l, 		/* Index into picture description table */
	status;		/* Return status */

#include "externs.h"

extern long *buffer;	/* I/O line buffer */

/*
For the entire image description table check if line count belongs to 
the vertical range of the rectangular description.
------------------------------------------------------------------------
*/
for  (j = 0; j < tbsize; ++j)
   {
   l = 10 * j - 1;
   if ((vcnt < tb[l + 1]) || (vcnt > (tb[l + 1] + tb[3+l] - 1)))
      continue;
   switch (tb[10 + l])
      {
      case 0:
         {
/*
If square type is simple square call GENSQR to generate simple square.
----------------------------------------------------------------------
*/
         status = gensqr (vcnt, &tb[1 + l]);
         if (status != E_SUCC)
            c_abort (&block, "Error processing image data", "classmap-lingen",
                       prog_name, clean_files, &num_outfiles);
         break;
         }
      case 1:
         {
/*
If square type is image call GETIMG to get a line of image.
-----------------------------------------------------------
*/
         status = getimg (vcnt, &tb[1+l]); /*, class, clschr);*/
         if (status != E_SUCC)
            c_abort (&block, "Error processing image data", "classmap-lingen",
                       prog_name, clean_files, &num_outfiles);
         break;
         }
      case 2:
         {
/*
If square type is character call GENCHR to get a line of character dot.
-----------------------------------------------------------------------
*/
         status = genchr (vcnt, &tb[1+l], &chr[tb[8+l] - 1]);
         if (status != E_SUCC)
            c_abort (&block, "Error processing image data", "classmap-lingen",
                       prog_name, clean_files, &num_outfiles);
         break;
         }
      case 3:
         {
/*
If square type is tick then call GENTIK to get a line portion of tick.
-----------------------------------------------------------------------
*/
         status = gentik (vcnt, &tb[1+l]);
         if (status != E_SUCC)
            c_abort (&block, "Error processing image data", "classmap-lingen",
                       prog_name, clean_files, &num_outfiles);
         break;
         }
      case -2:
      case -3:
         {
         break;
         }
      default:
         {
/*
If square type is not valid, issue an error message.
-----------------------------------------------------
*/
         c_abort (&block, "Invalid square type specified in square description",
                  "classmap-square", prog_name, clean_files, &num_outfiles);
         break;
         }
      }
   }
}
