#include <stdio.h>
#include "las.h"
#include "protos.h"
/******************************************************************************

NAME:  MAGTBL

FUNCTION:   Magnify square descriptions in the table properly.

PROGRAM HISTORY:
  Version	Date       Author      		 Request
  -------	----	   ------           	 -------
   1.0       19-Feb-82    J. Hwang, SASC
   2.0        8-Jan-88    M. Mazzella SAR/GSFC   Convert to C
   3.0       20-Dec-90    K. Donaldson           Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Set up factor by which to magnify the x, y, xs, ys elements
      of the picture description table(1,2,4,8,16)
   For all the rectange description in the picture definition table:
      Magnify size of square and rearrange coordinates.
      If rectangle describes tick area then
         magnify only tick length, not width.
      End-If
   End-For

******************************************************************************/

void magtbl
(
    long ntimes		/*  Number of times to magmify the table  */
)
{
long	i, j,		/*  Loop counters and array index	  */
	l,		/*  Index into table			  */
	timsqr;		/*  Magnification factor                  */

#include "externs.h"

/*
Set up factor by which to magnify the x, y, xs, ys elements of the 
picture description table(1,2,4,8,16)
--------------------------------------------------------------------
*/
timsqr = 1 << (ntimes - 1);

/*
For all the rectange description in the picture definition table:
--------------------------------------------------------------------
*/
for (i = 1; i <= tb[6]; i++)
   {
   l = (i - 1) * 10;

/*
Magnify size of square and rearrange coordinates.
--------------------------------------------------
*/
   for (j = 0; j < 4; ++j)
      {
      if (j < 2) 
         tb[j+l] = (tb[j+l] - tb[j]) * timsqr + tb[j] - 1;
      else
         tb[j+l] = tb[j+l] * timsqr;
      }

/*
If rectangle describes tick area then magnify only tick length, not width.
---------------------------------------------------------------------------
*/
   if (tb[l+9] == 3)
      {
      if (tb[6+l] > tb[7+l])   /*  atic */
         tb[6+l] = tb[6+l] * timsqr;
      else                          /*  dtic */
         tb[7+l] = tb[7+l] * timsqr;
      }
   }
}
