#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"
/******************************************************************************

NAME: OPNIMG

FUNCTION:   Open disk file via image i/o and allocate space for buffers.

PROGRAM HISTORY:
  Version	Date       Author      		     Request
  -------	----	   ------            	     -------
   1.0        27-Dec-81    J. Hwang, SASC
   2.0         8-Jan-88    M. Mazzella SAR/GSFC     Convert to c
   3.0        20-Mar-89    K. Narayanan STX         Removed unnecessary code
   4.0        20-Dec-90    K. Donaldson             Added comments
   5.0        05-Feb-91    K. Donaldson             Added code to process only
                                                    the 1st band if input image
                                                    has multiple bands.
   6.1	      23-May-94	   D. Eichacker		    Added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

   Open the input image file.
   Allocate dynamic memory for the input buffer.
   Group the input image bands.
   Open the output image file.
   Allocate dynamic memory for the output buffer.
   Group the input image bands.
   Update the output image ddr.

******************************************************************************/

void opnimg
(
    char *hosin,		/* Input file host name	*/
    char *taeout,	/* Output file tae name */
    struct DDR *ddr,	/* Input image ddr */
    long num_img_bands[MAXIMG][MAXBND+1], /* Input image band numbers */
    long window[],	/* Input image window specifications */
    long *offset,	/* Offsets into image i/o buffers */
    char *hosout,	/* Output file host name */
    long windout[]	/* Output image window specifications */
)

{
#include "classmap.h"
double	lineinc,	/* Line increment value */
	sampinc;	/* Sample increment value */

long	access,		/* Access type for imageio */
	bdsout = 3,	/* Number of output bands */
	bufsizin,	/* Size of the input buffer */
	bufsizout,	/* Size of the output buffer */
	comb_flg,	/* Specifies if the image bands are to be combined */
	status;		/* Return status */

extern long	gdescin, 	/* Input group descriptor */
		gdescout,	/* Output group descriptor */		
		fdescin[],  	/* Input file descriptor */
		fdescout[];	/* Output file descriptor */

static long opt[3] = {0,0,0};   /* option array for eopens  */

extern unsigned char	*bufferin,	/* Input image i/o buffer */ 
			*bufferout;	/* Output image i/o buffer */

/*
Open the input image file.
---------------------------
*/
access = IREAD;

status = c_eopens ((struct FDESC **)fdescin, hosin, (long *)num_img_bands, 
		   &window[SL], &window[SS], &window[NL], &window[NS], offset,
	           &access, &ddr->dtype, &zero, opt);
if (status != E_SUCC)
   c_errmsg ("Unable to open input image", "classmap-inimg", &fatal_err);
fdescin[1] = 0;

/*
Allocate dynamic memory for the input buffer.
----------------------------------------------
*/
if ((bufferin = (unsigned char *)c_ealloc((struct FDESC **)fdescin, &bufsizin)) == NULL)
   c_errmsg ("Error allocating dynamic memory", "classmap-alloc",
              &fatal_err);

/*
Group the input image bands.
-----------------------------
*/
status = c_egroup ((struct FDESC **)fdescin, (struct GDESC **)&gdescin,
		    bufferin, &bufsizin);
if (status != E_SUCC)
   c_errmsg ("Unable to group input image", "classmap-ingroup", &fatal_err);

/*
Open the output image file.
----------------------------
*/
access = IWRITE;
status = c_eopens ((struct FDESC **)fdescout, hosout, &bdsout, &windout[SL], 
		   &windout[SS], &windout[NL], &windout[NS], &offset[1],&access,
                   &ddr->dtype, &zero, opt);
if (status != E_SUCC)
   c_errmsg ("Unable to open output image", "classmap-outimg", &fatal_err);

fdescout[1] = 0;
strcpy (clean_files[0], hosout);
strcpy (out_files[0], taeout);
num_outfiles++;

/*
Allocate dynamic memory for the output buffer.
----------------------------------------------
*/
if ((bufferout = (unsigned char *)c_ealloc((struct FDESC**)fdescout,&bufsizout)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "classmap-alloc", 
            prog_name, clean_files, &num_outfiles);

/*
Group the input image bands.
-----------------------------
*/
status = c_egroup ((struct FDESC **)fdescout, (struct GDESC **)&gdescout,
		    bufferout, &bufsizout);
if (status != E_SUCC)
   c_abort (&block, "Unable to group output image", "classmap-outgroup", 
            prog_name, clean_files, &num_outfiles);

status = c_estep((struct GDESC **)&gdescout);
if (status != E_SUCC)
   c_abort (&block, "Error writing output image", "classmap-write", prog_name,
             clean_files, &num_outfiles);

/*
Update the output image ddr.
----------------------------
*/
lineinc = 1.0;
sampinc = 1.0;
comb_flg = NOCOMB;

status = c_upddr (&block,(char (*)[256])hosin,hosout,&one,(long (*)[4])window, 
		  num_img_bands, &one, &lineinc, &sampinc, &bdsout, &comb_flg);
if (status != E_SUCC)
   c_abort (&block, "Error updating the ddr file", "classmap-upddr", prog_name,
            clean_files, &num_outfiles);
}
