#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"
/******************************************************************************

NAME:  PUTIMG

FUNCTION:   Put a line of the picture to disk file.

PROGRAM HISTORY:
  Version	Date       Author      		     Request
  -------	----	   ------		     -------
   1.0        19-Feb-82    J. Hwang, SASC
   2.0         8-Jan-88    M. Mazzella SAR/GSFC
   3.0        20-Dec-90    K. Donaldson             Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):

   For all the bands, 
      Call VARCHR to generate color id (R, G, B).
      If the current line counter is not in vertical range of the 
      grey scale then
         Run the current line through the look up table.
      Otherwise, 
         Store the line in the output buffer.
   End-For
   Write out the output line.

******************************************************************************/

void putimg
(
    long vcnt,			/* Vertical line count		*/
    long gry[]			/* Grey scale lines		*/
)
{
#include "classmap.h"

long	dtype = EBYTE,		/* Data type for pixman	*/
	i,			/* Loop counter	*/
	status,			/* Return status */
	tbindx;			/* Loop counter	*/

extern unsigned char	*bufferout;	/* Output image i/o buffer */
extern unsigned char	*workbuf;	/* Scratch buffer for image i/o	*/

extern long gdescout;	/* Output image group descriptor */
extern long offset[];	/* Offsets into image i/o buffers */

#include "externs.h"

/* 
For all the bands, call VARCHR to generate color id (R, G, B).
If the current line counter is not in vertical range of the grey scale then
run the current line through the look up table.
Otherwise, store the line in the output buffer.
----------------------------------------------------------------------------
*/
for  (tbindx = 1; tbindx <= 3; ++tbindx)
   {
   varchr (vcnt, &tbindx);
   if ((vcnt < gry[0]) || (vcnt > gry[1])) 
      {
      status = c_pxlkup ((long *) workbuf, (long *)&bufferout[offset[tbindx]],
		         &tb[3], &dtype, (long *)coltbl[tbindx - 1], &dtype);
      if (status != E_SUCC)
         c_abort (&block, "Error processing lookup table", "classmap-lut", 
                  prog_name, clean_files, &num_outfiles);
      }
   else
      for (i = tb[1]; i <= tb[3] + tb[1] - 1; ++i)
         bufferout[offset[tbindx] + i - 1] = workbuf[i - 1];
   }

/*
Write out the output line.
---------------------------
*/
status = c_estep((struct GDESC **)&gdescout);
if (status != E_SUCC)
   c_abort (&block, "Error writing output image", "classmap-write", prog_name,
             clean_files, &num_outfiles);
}
