/**************************************************************************
NAME: CLUST2SPEC

PURPOSE: To convert a 6 or 12 Band CLUSTER output BYTE image
         to a Spectrum format image.

PROGRAM HISTORY:
  Version	Date	Author		Request
  -------	----	------		-------
    0.0		12/94	S.Benjamin/Ames ???
    1.0		01/95	K.Lowell	MRLC PRF6282 - Generate MRLC clustered
					products in a Spectrum compatable 
					format
    2.0         12/99   D.Hames         Remove MAXNS from the buffer image_data
					and allocate it using the total number
					of samples in the image. This is to
					support images > 2gig. SRF#893

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Runs under TAE/UNIX

PROJECT: LAS

NOTES: Assumes:  max # clusters is 241, max # bands is 12
                 based on MRLC requirements for image clustering 
                 max # image columns is 10000
                 based on Khoros limitation
************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"     /* Labeled table include file           */
#include "diskio.h"	/* disk allocation include file */
#include "util.h"
#include "protos.h"

#define MAXCLUST 241	/* Maximum number of clusters	*/

void main
(void)
{
struct PARBLK vblk;		/* TAE parameter block			*/
struct TAB_DEF tab;             /* Calibration LT                       */
struct VECTOR *vector;          /* Coeficients from LT                  */

long *fdesc;		/* File descriptor		*/
long acc;		/* Access flag			*/
long one = 1;		/* Constant of one		*/
long opt[2];            /* Option			*/
long std_out;           /* standard out                 */
long nclus;		/* number of clusters		*/
long msgtim;		/* Processing messsage time	*/
long fatal = -1;	/* fatal indicator flag		*/
long nonfatal = 1;	/* nonfatal indicator flag	*/
long dtype;		/* data type			*/
long nl,ns,nb;		/* LAS counters			*/
long status;		/* Return status flag		*/
long curlin;		/* Current processing line	*/
long window[1][4];	/* Processing window		*/
long msg_flag=GEN2;     /* message processing flag      */
long filesize = 0;	/* Esitmated output file size   */

double coef[12];	/* Matrix for cluster means	*/

char  outlist[CMLEN];   /* Output file list        */

extern long curent;
extern long curcat;

char	stat_name[132],out_name[80],hosin[CMLEN];
long	ii,jj,kk,nn,zz;	/* counters				*/
long	count;		/* cluster # and pixel count from stat file	*/
long	kh_head[256];		/* output buffer for viff header	*/
long    *image_data;		/* buffer for binary image              */
long	ncols,nrows,nbands,pxsize_x,pxsize_y;
float	hdrbuf[10000];		/* Stores spectrum hdr as floating point*/
float	means[MAXCLUST][12];	/* Stores band means and count col	*/
long	pxcount[MAXCLUST];	/* integer version of pixel count	*/
FILE	*fdout;			/* Output file descriptor	        */


/* Initialize TAE parameter block
---------------------------------*/
c_inlas(&vblk,&std_out);

/* Get the variables from the TAE variable block
-------------------------------------------------*/
if ((getpar(&vblk,hosin,window,stat_name,out_name,&nclus,&msgtim)) != E_SUCC)
  c_errmsg("Error error encountered","clust2spec-fatal",&fatal);

/* Open input image
---------------------*/
acc = IREAD;
dtype = EBYTE;
opt[0] = opt[1] = 0;
nb = 0;
nl = 0;
ns = 0;

if (status = c_eopenr((struct FDESC **)&fdesc,hosin,&nb,&one,&one,&nl,&ns,&acc,
                        &dtype,opt,&one) != E_SUCC)
  {
  c_errmsg("Fatal error encountered","clust2spec-fatal",&fatal);
  }

/* Allocate space and open the output file
--------------------------------------------*/
filesize = (nl*ns+12000);
if (status = c_dkcre(0,out_name,filesize,0) != NULL);
  {
  if (status == E_FAIL)
    c_errmsg("Fatal error encountered","clust2spec-fatal",&fatal);
  if (status == DISK_EXISTS)
    c_errmsg("Output file already exists","clust2spec-exist",&fatal);
  if (status == DISK_NOSPACE)
    c_errmsg("No available disk space","clust2spec-nodisk",&fatal);
  }
if ((fdout = fopen(out_name,"w")) == (FILE *) NULL)
  {
  c_eclose((struct FDESC **)&fdesc);
  c_errmsg("Error opening output image file","clust2spec-open",&fatal);
  }

nrows = window[0][2];
ncols = window[0][3]; 
pxsize_x = 1.;		/* not used in khoros 1.5	*/
pxsize_y = 1.;		/* not used in khoros 1.5	*/

/* Open the LT file and extract the number of bands contained in
   the original image that was clustered.
----------------------------------------------------------------*/

open_tab(&tab,stat_name,IREAD);
get_vector(&tab);

vector = tab.vector;
for (ii = 0; ii < 6; ii++)
  {
  vector=vector->next;
  }
nbands = vector->lt_size.ncol;  

/* Start to read in cluster means and pixcount  from LT file
-------------------------------------------------------------*/
for (ii=0; ii<nclus;ii++)
  {
  status=get_record(&tab);
  vector = tab.vector;
  for (zz=0;zz<5;zz++) vector=vector->next;
  get_matrix(vector,vector->dtype,(unsigned char *)&count);
  vector=vector->next;
  get_matrix(vector,vector->dtype,(unsigned char *)coef); 
  
  pxcount[ii] = count;
  means[ii][0]=coef[0];
  means[ii][1]=coef[1];
  means[ii][2]=coef[2];
  means[ii][3]=coef[3];
  means[ii][4]=coef[4];
  means[ii][5]=coef[5];
  if (nbands == 12 )
    {
    means[ii][6]=coef[6];
    means[ii][7]=coef[7];
    means[ii][8]=coef[8];
    means[ii][9]=coef[9];
    means[ii][10]=coef[10];
    means[ii][11]=coef[11];
    }
  }

close_tab(&tab);


/* FILL UP OUTPUT BUFFERS			*/
/* Initialize Khoros header, and fill VIFF required fields  			
-------------------------------------------------------------*/
for (jj=0; jj<256; ++jj)
	kh_head[jj] = 0;
kh_head[0] = -1425997565;	/* khoros magic number	*/
kh_head[1] = 33554432;		/* khoros magic number	*/	
kh_head[130] = ncols;		/* number of output samples */
kh_head[131] = nrows;           /* number of output lines */
kh_head[133] = -1;	      	/* not a subimage	*/
kh_head[134] = -1;		/* not a subimage	*/
kh_head[135] = pxsize_x;	/* pixel x size		*/
kh_head[136] = pxsize_y;	/* pixel y size		*/
kh_head[137] = 1;		/* constant of 1	*/
kh_head[139] = 1;		/* constant of 1	*/
kh_head[140] = 1;		/* constant of 1        */
kh_head[141] = 1;		/* constant of 1        */
kh_head[143] = 1;		/* constant of 1        */
kh_head[144] = 5;		/* map storage type (BYTE)*/
kh_head[145] = nbands + 2;	/* number of map columns	*/
kh_head[146] = nclus;		/* number of map rows	*/
kh_head[148] = 1;		/* constant of 1	*/
kh_head[151] = 1;		/*ispare1 = means + count (clusout)	*/	
kh_head[152] = nbands;		/*ispare2 = # columns in mean vector	*/

/* Write out khoros header
---------------------------*/
nn=(long)fwrite(kh_head,1,1024,fdout);

/* Fill Spectrum header with cluster mean centroids
-----------------------------------------------------*/
for (jj=0; jj<nbands; ++jj)
  {
  ii = jj * nclus;
  for (kk=0; kk<nclus; ++kk)
    {
    hdrbuf[ii+kk] = means[kk][jj];
    }
  }
ii=nbands * nclus;
for (kk=0; kk<nclus; ++kk)
		hdrbuf[ii+kk] = (float)pxcount[kk];
ii=(nbands + 1) * nclus;
for (kk=0; kk<nclus; ++kk)
	hdrbuf[ii+kk] = 0;


/* Write out Spectrum header	
--------------------------------*/
ii=(nbands + 2) * nclus;
nn=(long)fwrite(hdrbuf,4,ii,fdout);

/* Allocate memeory
-----------------*/
if ((image_data = (long *)malloc(window[0][3] * sizeof(long))) == NULL)
   c_errmsg("Error allocating dynamic memory","clust2spec-alloc",&fatal); 

/* Open LAS image and copy it into output file 
------------------------------------------------*/
c_inittm("CLUST2SPEC","Processed","line","band"," ",&nrows,&nbands,&msgtim,&msg_flag);
curent = 1;
curcat=1;
c_startm();
for (jj=0; jj<nrows; jj++)
  {
  curlin = jj + 1;
  curent = curlin;
  if (status = c_eread((struct FDESC **)&fdesc,&one,&curlin,
			(unsigned char *)image_data,&one) != ncols) 
    {
    c_errmsg("Error reading input image file","clust2spec-read",&nonfatal);
    c_eclose((struct FDESC **)&fdesc);
    fclose(fdout);
    c_clean(&vblk,"clust2spec",(char (*)[256])out_name,(long *)1);
    }
  if (status = fwrite(image_data,1,ncols,fdout) != ncols)
    {
    c_errmsg("Error writing output image file","clust2spec-write",&nonfatal);
    c_eclose((struct FDESC **)&fdesc);
    fclose(fdout);
    c_clean(&vblk,"clust2spec",(char (*)[256])out_name,(long*)1);
    }
  }

c_stoptm();
	

/* Close the files, free memory and quit 
--------------------------------------*/
free (image_data);
c_dkfree(out_name);
c_eclose((struct FDESC **)&fdesc);
fclose(fdout);
strcpy(outlist,out_name);
c_complt("CLUST2SPEC",(char (*)[256])outlist,&one);
}

