/*******************************************************************************
NAME: GETPAR

PURPOSE: gets the user-specified parameters and tae globals for clust2spec

*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "worgen.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblk,   /* I: pointer to initial TAE parameter block */
    char hosin[],          /* O: Host input image name		        */
    long window[1][4],     /* O: Window of input image			*/
    char stat_name[],      /* O: Input LT name				*/
    char outspec[],        /* O: Output Spectrum image namge		*/
    long *nclust,	   /* O: Number of Clusters in the input scene	*/
    long *msgtim	   /* O: Message processing time		*/
)	

{
struct VARIABLE *var;      /* ptr to a variable in the vblk               */

long status;			/* function return status		*/
long fatal = -1;		/* Fatal error flag			*/
long bands[1][MAXBND+1];	/* Bands to process			*/
long dummy[2];			/* Dummy variable for GETIN		*/
long nbands;			/* Number of bands			*/
long one = 1;			/* Constant of one			*/
long zero = 0;			/* Constant of zero			*/
char taein1[CMLEN];		/* TAE input image name			*/
char hosin1[CMLEN];		/* HOST input image name		*/
char *getenv();                 /* getenv return          */

/* Get the input image 
 ---------------------*/
status = c_getin(vblk,"IN",&zero,(char (*)[256])taein1,(char (*)[256])hosin1,
	 	dummy,window,bands,&nbands,dummy,&one,dummy,(long (*)[2])dummy);
if (status != E_SUCC)
  c_errmsg("Fatal error encountered","clust2spec-fatal",&fatal);
strcpy(hosin,hosin1);   

/* Get the name of the labeled table
-------------------------------------*/
status = c_getnam(vblk,"INLT",&zero,stat_name,"lt",&zero);
if (status != E_SUCC)
  c_errmsg("Fatal error encountered","clust2spec-fatal",&fatal);

/* Get the name of the output spectrum format file
---------------------------------------------------*/
status = c_getnam(vblk,"OUTFILE",&zero,outspec,"spec",&one);
if (status != E_SUCC)
  c_errmsg("Fatal error encountered","clust2las-fatal",&fatal);

/* Get the number of clusters
-----------------------------*/
var = p_find(vblk,"NCLUST");
*nclust = IVAL(*var,0);

/* Retrieve globals
-------------------*/
var = p_find(vblk,"$msgtim");
*msgtim = IVAL(*var,0);

return(E_SUCC);
}
