
/*
 *   Copyright, 1991, The Regents of the University of California.
 *   This software was produced under a U.S. Government contract
 *   (W-7405-ENG-36) by the Los Alamos National Laboratory, which is
 *   operated by the University of California for the U.S. Department
 *   of Energy.  The U.S. Government is licensed to use, reproduce,
 *   and distribute this software.  Neither the Government nor the
 *   University makes any warranty, express or implied, or assumes
 *   any liability responsibility for the use of this software.
 */

#include <stdio.h>
#include "cluster.h"

void SM_add_vector_fl_ul
(
    int dim,				/* Problem dimension */
    int *N,				/* The number of data items */
    float *mean,			/* The mean vector */
    float **cov,			/* The covariance matrix */
    float **vset,			/* Vector set of new vectors */
    int which,			/* The new vector of interest */
    float **scratch		/* TEMPORARY dimensioned [ dim ] [ 2 ] */
)

{
	int i, j;		/* Loop control */
	float m1, m2;	/* Temporary */

	/*
	 *  Check to see if this is the first vector in the set.
	 */
	if ( *N == 0 ) {

		/*  
		 *  Set the mean to this vector, and cov to all zeros.
		 */
		for ( i = 0 ; i < dim ; i ++ ) {
			mean [i] = vset [i][which] ;
			for ( j = i ; j < dim ; j ++ ) cov [i][j] = 0.0 ;
		}
		*N = 1 ;

	} else {

		/*
		 *  Do the recursive updates.
		 */
		m1 = 1.0 - 1.0 / (float) *N ;
		(*N) ++ ;
		m2 = 1 / (float) *N ;

		/*
		 *  Update mean and store values for updating the 
		 *  covariance matrix.
		 */
		for ( i = 0 ; i < dim ; i ++ ) {
			scratch [i][0] = vset [i][which] - mean [i];
			scratch [i][1] = scratch [i][0] * m2 ;
			mean [i] += scratch [i][1] ;
		}

		/*
		 *  Update the covariance matrix.
		 */
		for ( i = 0 ; i < dim ; i ++ ) {
			for ( j = i ; j < dim ; j ++ ) {

				cov [i][j] = m1 * cov [i][j] + 
						scratch [i][1] * scratch [j][0];

			}
		}

	}

	return;
}
