/*******************************************************************************
NAME:		ALLOC_LAS 

PURPOSE:	Allocate space for image buffers

*******************************************************************************/
#include <stdio.h>
#include <malloc.h>
#include "las.h"
#include "cluster.h"


int alloc_las
(
    struct RDESC  rdesc[],  /* File descriptors to the images	     */
    long nmrimg             /* number of input images                    */
)
{
long i;				/* Loop control                              */
long image;			/* Loop index counting for images	     */
long size;			/* Size of the buffer to allocate	     */
long nonfatal = 1;		/* Non-fatal error message status            */

/*
  Allocate space for all the input images.  Use calloc() because it will zero
  all memory that is allocated.  Thus, if the user does not supply one of the
  optional input images, its buffer will be all zeros for the computations.
*/
for (image = 0; image < nmrimg; image++)
    {
    size = rdesc[image].nsamples * rdesc[image].nbands;

    switch (rdesc[image].dtype)
	{
	case EBYTE:
	    rdesc[image].ptr1 =
		(unsigned char *)calloc(size, sizeof(unsigned char));
	    break;
	case EWORD:
	    rdesc[image].ptr2 = (short *)calloc(size, sizeof(short));
	    break;
	case EUWORD:
	    rdesc[image].ptr3 =
		(unsigned short *)calloc(size, sizeof(unsigned short));
	    break;
	case ELONG:
	    rdesc[image].ptr4 = (long *)calloc(size, sizeof(long));
	    break;
	case EULONG:
	    rdesc[image].ptr5 =
		(unsigned long *)calloc(size, sizeof(unsigned long));
	    break;
	case EREAL:
	    rdesc[image].ptr6 = (float *)calloc(size, sizeof(float));
	    if (rdesc[image].ptr6 == NULL)
		{
		c_errmsg("Error allocating memory","alloc_las-alloc",&nonfatal);
		return(E_FAIL);
		}
	    /* calloc does not zero floats properly */
	    for (i = 0; i < size; i++)
		rdesc[image].ptr6[i] = 0.0;
	    break;
	}

    if ((rdesc[image].ptr1 == NULL) && (rdesc[image].ptr2 == NULL) &&
	(rdesc[image].ptr3 == NULL) && (rdesc[image].ptr4 == NULL) &&
	(rdesc[image].ptr5 == NULL) && (rdesc[image].ptr6 == NULL))
	{
	c_errmsg("Error allocating memory","alloc_las-alloc",&nonfatal);
	return(E_FAIL);
	}
    }

return(E_SUCC);
}

