
/*******************************************************************
 * NAME: BIN_SEARCH    
 *
 * PURPOSE: A binary search routine for an array of monotonically
 *          increasing floating point values.  Returns an index to
 *          the nearest match.  Part of the CONSTRUCT program
 ****************************************************************/
#include <math.h>
#include "cluster.h"

int bin_search
(
    float *vals,		/* Array of monotonically increasing fp values */
    int  numdata,		/* Number of values */
    double svalue		/* Value we are searching for */
)

{
int upper, lower, itemp;

lower = 0;
upper = numdata-1;

if ( vals [lower] >= svalue ) return ( lower ) ;

if ( vals [upper] <= svalue ) return ( upper );

while ( (upper - lower) > 1 ) 
  {
  itemp = (upper + lower) / 2;
  if ( vals [itemp] >= svalue ) upper = itemp;
  else if ( vals [itemp] < svalue ) lower = itemp;
  }

if ( upper > lower ) 
  {
  if ( fabs (vals[upper]-svalue) < fabs (vals[lower]-svalue) ) 
    {
    return ( upper );
    } 
  else 
    {
    return ( lower );
    }
  }

return ( upper );
}

long build_indices
(
    float *vals,		/* Array of monotonically increasing values */
    int numdata,		/* Number of values */
    int *indices,		/* Indices computed and returned */
    int min,		/* Minimum index to compute */
    int max,		/* Maximum index to compute */
    double norm		/* Normalization value */
)
{
long i;
float svalue;

for ( i = min ; i <= max ; i ++ ) 
  {
  svalue = (float) i / norm ;
  svalue *= ( vals [numdata-1] - vals [0] );
  svalue += vals [0];
  indices [i] = bin_search ( vals, numdata, svalue );
  }

return (1);
}
