#ifndef CLUSTER_H
#define CLUSTER_H

#include "las.h"
#include "construct.h"
#include "iio.h"

/* Structure containing the necessary information for the input/ouput images
------------------------------------------------------------------------------*/
struct RDESC
    {
    struct FDESC *fd;
    long access;
    long dtype;
    long nlines;
    long nsamples;
    long nbands;
    long bands[MAXBND];
    char host[1][CMLEN];
    unsigned char  *ptr1;
    short	   *ptr2;
    unsigned short *ptr3;
    long 	   *ptr4;
    unsigned long  *ptr5;
    float	   *ptr6;
    };


/*
**  PROTOTYPES
*/

void SM_add_vector_fl_ul
(
    int dim,                    /* Problem dimension */
    int *N,                     /* The number of data items */
    float *mean,                /* The mean vector */
    float **cov,                /* The covariance matrix */
    float **vset,               /* Vector set of new vectors */
    int which,                  /* The new vector of interest */
    float **scratch             /* TEMPORARY dimensioned [ dim ] [ 2 ] */
);

void SM_add_vector_uc_tbl
(
    int dim,                    /* Problem dimension */
    int *N,                     /* The number of data items */
    float *mean,                /* The mean vector */
    float **cov,                /* The covariance matrix */
    unsigned char *new_vec,     /* New vector to add into the statistics */
    float **scratch,            /* TEMPORARY dimensioned [ dim ] [ 2 ] */
    float *conv_table
);

void SM_add_vector_uc_tbl_ul
(
    int dim,                    /* Problem dimension */
    int *N,                     /* The number of data items */
    float *mean,                /* The mean vector */
    float **cov,                /* The covariance matrix */
    unsigned char **vset,       /* Vector set of new vectors */
    int which,                  /* The new vector of interest */
    float **scratch,            /* TEMPORARY dimensioned [ dim ] [ 2 ] */
    float *conv_table
);

float **SM_get_matrix
(
    int dim	/* The dimension of the matrix */
);

int alloc_las
(
    struct RDESC  rdesc[],  /* File descriptors to the images	     */
    long nmrimg             /* number of input images                    */
);

int bandlineread
(
    struct RDESC rdesc[],
    long curl,
    long nmrimg,
    long bandnum,
    unsigned char *bandbuf,
    long numsamp
);

int bin_search
(
    float *vals,            /* Array of monotonically increasing fp values */
    int  numdata,           /* Number of values */
    double svalue           /* Value we are searching for */
);
int
long build_indices
(
    float *vals,            /* Array of monotonically increasing values */
    int  numdata,           /* Number of values */
    int  *indices,          /* Indices computed and returned */
    int  min,               /* Minimum index to compute */
    int  max,               /* Maximum index to compute */
    double norm             /* Normalization value */
);

int close_las
(
    struct RDESC rdesc[],   /* File descriptors to the images            */
    long nmrimg             /* Number of input images                    */
);

long construct_fl 
(
    float *inpix,	/* 1-D array of pixel data of size [pixsiz][ndim] */
    long pixsz,		/* size of pixel image		*/
    int ndim,
    float  **outlut,	/* 2-D array of pixel data of size [ndim][lutsz] */ 
    long lutsz,		/* size of look up table	*/
    long niter, 
    long maxsamp, 
    long ilmax,
    int  *lookup,		/* an array used to speed up the search */
    float  **means,
    long init,		/* a flag to initialize the lut */
    FILE *hist_file_ptr,
    long *count,
    float *vecs_pc, 
    float *pc
);

long construct_sh 
(
    short *inpix,	/* 1-D array of pixel data of size [pixsiz][ndim] */
    long pixsz,		/* size of pixel image		*/
    int ndim,
    float  **outlut,	/* 2-D array of pixel data of size [ndim][lutsz] */ 
    long lutsz,		/* size of look up table	*/
    long niter, 
    long maxsamp, 
    long ilmax,
    int  *lookup,		/* an array used to speed up the search */
    float  **means,
    long init,		/* a flag to initialize the lut */
    FILE *hist_file_ptr,
    long *count,
    float *vecs_pc, 
    float *pc 
);

long construct_uc 
(
    unsigned char *inpix,	/* 1-D array of pixel data of size [pixsiz][ndim] */
    long pixsz,	            /* size of pixel image		*/
    int ndim,
    float  **outlut,    	/* 2-D array of pixel data of size [ndim][lutsz] */ 
    long lutsz,	        	/* size of look up table	*/
    long niter, 
    long maxsamp,
    int  *lookup,		    /* an array used to speed up the search */
    float  **means,
    long init,	        	/* a flag to initialize the lut */
    FILE *hist_file_ptr,
    float *conv_table,
    long *count,
    float **pc_mults_uc, 
    float *vecs_pc, 
    float scale, 
    float *pc,
    long use_log
);

float **contig_matrix
(
    long num_rows,
    long num_cols	/* Size of matrix to allocate */
);

long getdata_uc 
(
    struct RDESC rdesc[],	      /* File descriptors to the images      */
    float **outlut,               /* the output centroids */
    float **means,
    long numlines,                /* the number of lines */
    long numsamp,                 /* the number of samples per line */
    long numbands,                /* the number of bands */
    long iters,                   /* the iterations */
    long numclust,                /* the number of desired clusteres */
    long bnd,                     /* the the band to sort on */
    long ilmax,                   /* the max poss value */
    int  *lookup,                 /* an array used to speed up the search */
    long totalsize,
    FILE *hist_file_ptr,         /* history file */
    float *conv_table,
    long *count,
    float **pc_mults_uc, 
    float *vecs_pc, 
    float scale, 
    float *pc,
    long use_log,
    long mfact,                  /* Multiplicity factor   */
    long window[],		     /* Processing window     */
    long nmrimg  		     /* number of input images         */
);

long getdata_sh 
(
    char *infile,                 /* the input filename   */
    float **outlut,               /* the output centroids */
    float **means,
    long numlines,                /* the number of lines */
    long numsamp,                 /* the number of samples per line */
    long numbands,                /* the number of bands */
    long iters,                   /* the iterations */
    long numclust,                /* the number of desired clusteres */
    long ilmax,                   /* the max poss value */
    int  *lookup,                 /* an array used to speed up the search */
    long totalsize,
    FILE *hist_file_ptr,         /* history file */
    long *count,
    float *vecs_pc, 
    float *pc,
    long mfact                   /* Multiplicity factor   */
);

long getdata_fl 
(
    char *infile,                  /* the input filename   */
    float **outlut,               /* the output centroids */
    float **means,
    long numlines,                /* the number of lines */
    long numsamp,                 /* the number of samples per line */
    long numbands,                /* the number of bands */
    long iters,                   /* the iterations */
    long numclust,                /* the number of desired clusteres */
    long ilmax,                   /* the max poss value */
    int  *lookup,                 /* an array used to speed up the search */
    long totalsize,
    FILE *hist_file_ptr,         /* history file */
    long *count,
    float *vecs_pc, 
    float *pc,
    long mfact                   /* Multiplicity factor   */
);

int getpar
(
    struct PARBLK *vblk,        /* pointer to initial TAE parameter block */
    char hosin[][CMLEN],        /* Host image file names                  */
    long window[][4],           /* Input image window specifications      */
    struct RDESC rdesc[],       /* File information                       */
    construct_struct *construct,/* pointer to construct structure     */
    long *mfact,
    long *msgtim,
    long *nmrimg
);

long lconstruct 
(
    struct PARBLK *vblk,	    /* pointer to initial TAE parameter block */
    struct RDESC rdesc[],       /* file descriptors to the images         */
    char *infile,               /* The input filename */
    char *output,		        /* The output filename */
    FILE *hist_file_ptr,        /* Output history file */
    long numclust,              /* Number of clusters */
    long iters,                 /* Number of algorithm iterations */
    long maxsize,               /* Maximum of ecess memory */
    long use_log,               /* Use logarithm of data */
    long mfact,		            /* multiplicity factor   */
    long msgtim,		        /* Message time		*/
    long window[],              /* Processing window	*/
    long nmrimg 		        /* number of input images  */
);

long lproject 
(
    struct PARBLK *vblk,	   /* pointer to initial TAE parameter block */
    struct RDESC rdesc[],
    char *infile,              /* The input image to project */
    float **outlut,            /* The codebook file to project through */
    char *outfile,             /* The resultant output image file */
    FILE *hist_file_ptr,       /* The history file */
    long numclust,		   /* Number of clusters  */
    struct DDR *ddr,	   /* The DDR structure   */
    long use_log,              /* Use logarithm of data */
    long msgtim,		   /* Processing time     */
    long window[],		   /* Processing window   */
    long nmrimg 	           /* Number of input images */
);

long nn_fl_pc
(
    long dim,                   /* Dimension of the input vector */
    float *x,                   /* Vector to find nearest neighbor of */
    long num_vecs,              /* Number of vectors in the set */
    float **vecs,               /* Set of vectors dimensioned [num_vecs][dim] */
    float *dist,                /* (OUTPUT)Squared distance to nearest vector */
    int  *start,                /* Indices to close vectors */
    float *vecs_pc,             /* Principal component of the vectors */
    double scale,               /* Scale value for finding starting position */
    float *pc,                  /* Principal component */
    long start_length           /* Length of start */
);

long nn_sh_pc
(
    long dim,                   /* Dimension of the input vector */
    short *x,                   /* Vector to find nearest neighbor of */
    long num_vecs,              /* Number of vectors in the set */
    float **vecs,               /* Set of vectors dimensioned [num_vecs][dim] */
    float *dist,                /* (OUTPUT)Squared distance to nearest vector */
    int  *start,                /* Indices to close vectors */
    float *vecs_pc,             /* Principal component of the vectors */
    double scale,               /* Scale value for finding starting position */
    float *pc,                  /* Principal component */
    long start_length           /* Length of start */
);

long nn_uc_pc
(
    long dim,                   /* Dimension of the input vector */
    unsigned char *x,           /* Vector to find nearest neighbor of */
    long num_vecs,              /* Number of vectors in the set */
    float **vecs,               /* Set of vectors dimensioned [num_vecs][dim] */
    float *dist,                /* (OUTPUT)Squared distance to nearest vector */
    float *lut,                 /* Look-Up-Table for char to float conversion */
    int  *start,                /* Indices to close vectors */
    float **pc_mults_uc,        /* Table with multiplication results */
    float *vecs_pc,             /* Principal component of the vectors */
    double scale                /* Scale value for finding starting position */
);

int nn_uc_pc_ul
(
    int dim,                    /* Dimension of the input vector */
    unsigned char **vset,       /* Vector set containing vector in question */
    int which,                  /* The vector of interest */
    int num_vecs,               /* Number of vectors in the set */
    float **vecs,               /* Set of vectors dimensioned [num_vecs][dim] */
    float *lut,                 /* Look-Up-Table for char to float conversion */
    int *start,                 /* Indices to close vectors */
    float **pc_mults_uc,        /* Table with multiplication results */
    float *vecs_pc,             /* Principal component of the vectors */
    double scale                /* Scale value for finding starting position */
);

int open_las
(
    struct RDESC rdesc[],       /* File descriptors to the images            */
    long window[],              /* Window to open input images with      */
    long nmrimg                 /* number of input images                    */
);

float project_uc_uc
(
    struct PARBLK *vblk,        /* pointer to initial TAE parameter block */
    struct RDESC rdesc[],
    float **outlut,             /* Vectors to find nearest */
    long ndim,                  /* Problem dimension */
    long numline,               /* Number of lines */
    long numsamp,               /* Number of columns */
    long lutsz,                 /* Number of vectors to find nearest */
    char *outfile,              /* Output filename */
    int  *lookup,               /* For finding close guess */
    long lookup_size,           /* Size of lookup */
    float ***covar,             /* Covariance matrices */
    float **means,              /* Mean vectors */
    int  *count,                /* Number of elements */
    char *infile,               /* Input file name */
    float *conv_table,          /* For conversion to float */
    float **pc_mult_uc,         /* Mults. for pc times unsigned char */
    float *means_pc,            /* Principal component of vectors in set */
    long msgtim,                /* Message processing time   */
    long window[],              /* Processing window     */
    long nmrimg                 /* Number of input images  */
);

int put2lt
(
    long numbands,
    long numclust,
    float **outlut,
    float ***covar,
    long *pxcnt
);

void bsort3
(
    float **means,              /* The mean vectors indexed [ cluster ] [ i ] */
    long dim,                   /* Size of the vectors */
    long numclust,              /* The number of clusters */
    float *sort_vals            /* The values to sort on */
);

int compareb
(
    float *sort_vals,
    float **means,
    long dim,
    long i1,
    long i2
);

void swap
(
    float *sort_vals,
    float **means,
    long i1,
    long i2,
    long dim
);

long timer(void);

#endif
