/**********************************************************************

NAME: CONSTRUCT

PURPOSE: Generate an initial mean vector for a LAS image
         using the Monte-Carlo sampling method

PROGRAM HISTORY:
Version  Date   Programmer      Code  Reason
-------  -----  -------------   ----  ------
  1.0    01/94  K.Lowell          C   Initial conversion
	 11/95  D.Etrheim	      Modified to correctly trap an error
				      allocating memory.  Also use tae name
				      on c_complt.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:  Algorithm extracted from code provided
  by Mr. Jim White, Los Alamos National Labs.  Code was from
  the Khoros image processing system.

**********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "vll.h"
#include "construct.h"
#include "cluster.h"

int main (void)
{
struct PARBLK vblk;             /* TAE parameter block          */
construct_struct construct;     /* construct structure          */

struct RDESC rdesc[MAXIMG];   /* File information             */

FILE *hist_file_ptr;     /* For history file output */

long use_log;           /* Use logarithm	        */
long fatal = -1;	/* fatal error flag		*/
long std_out;		/* standard out			*/
long mfact = 0;		/* multiplicity factor          */
long nmrimg = 0;	/* number of input images       */
long outcount=1;	/* output file counter          */
long totbnd = 1;
long msgtim;
long window[MAXIMG][4];	/* input image window specification */

static double minval[MAXBND];   /* Minval for DDR	*/
static double maxval[MAXBND];   /* Maxval for DDR	*/

char hosin[MAXIMG][CMLEN];	/* Host input image file name  */
char m_flag[8];			/* MINMAX flag for DDR	*/

/* clear out the rdesc records */
memset(rdesc,0,sizeof(rdesc));

/* Initialize TAE parameter block
---------------------------------*/
c_inlas(&vblk,&std_out);

/* Allocate the construct structure
--------------------------------------*/
/* constructp = (construct_struct *)calloc(1, sizeof(construct_struct));*/

/* Get the variables from the TAE variable block
-------------------------------------------------*/
if ((getpar(&vblk,hosin,window,rdesc,&construct,&mfact,&msgtim,&nmrimg)) != E_SUCC)
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);

/* Open the history file for output
-------------------------------------*/
if ((hist_file_ptr=fopen((construct.h_file), "w"))==NULL)
   c_errmsg("Error opening history file","cluster-open",&fatal);

fprintf(hist_file_ptr,"Output File        = %s\n",construct.o_file);
fprintf(hist_file_ptr,"History File       = %s\n",construct.h_file);



use_log = 0;
if ( construct.log_flag ) 
  {
  if ( construct.log_logic ) use_log = 1;
  }


/* -main_library_call */

if (lconstruct ( &vblk, rdesc, construct.i_file,construct.o_file, 
                 hist_file_ptr,
     		construct.n_int, construct.it_int, construct.max_int, 
     		use_log,mfact,msgtim,window[0],nmrimg) != E_SUCC)
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);

/* Update DDR with MINMAX info
--------------------------------*/
maxval[0] = (double)construct.n_int;
minval[0] = 1.0;
strcpy(m_flag,"YES");
c_svmnmx(construct.o_file,&totbnd,minval,maxval,m_flag);


fclose(hist_file_ptr);
c_complt("CLUSTER",construct.o_file,&outcount);
return(0);
}
