/**************************************************************************
 * NAME: CONTIG_MATRIX 
 *
 * PURPOSE:  Allocate a matrix with "num_rows" rows and "num_cols"
 *         columns contiguously in memory.  This means that all
 *         data in the matrix will be accessible also as a 1-D
 *         array of length "num_rows" * "num_cols".
 * 
 * PROGRAM HISTORY:
 * Version   Date    Programmer        Code   Reason
 * -------   ----    --------------    -----  ------
 *   1.0     11/90   P.Kelly, lanl       C    
 *   2.0     01/94   K.Lowell            C    Conversion to LAS
 *
 * The returned matrix is indexed as:  
 *
 *         returned [ 0 .. num_rows-1 ] [ 0 .. num_cols-1 ]
 *    OR
 *         returned [ 0 .. (num_rows * num_cols) - 1 ]
 *
 */

#include <stdio.h>
#include "cluster.h"

float **contig_matrix
(
    long num_rows,
    long num_cols	/* Size of matrix to allocate */
)

{
float *temp1d, **temp2d;
long jj;
long fatal = -1;		/* fatal error flag		*/


/* Get contiguous memory for all values in matrix.
-------------------------------------------------- */
temp1d = (float *) malloc ( (unsigned) 
		( num_rows * num_cols * sizeof (float) ) ) ;

if ( temp1d == NULL ) 
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }

/* (2)  Get memory for row indices -- to point into temp1d
---------------------------------------------------------- */
temp2d = (float **) malloc ( (unsigned)
		( num_rows * sizeof (float *) ) ) ;

if ( temp2d == NULL ) 
  {
  free(temp1d);
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }
	
/*  (3)  Set up the row pointers into temp1d.
-----------------------------------------------*/
for ( jj = 0 ; jj < num_rows ; jj ++ ) 
  {
  temp2d [jj] = & ( temp1d [jj * num_cols] ) ;
  } 

return ( temp2d ) ;
}
