/**************************************************************************
NAME: GETPAR

PURPOSE: Gets the user-specified parameters and tae globals for CLUSTER

PROGRAM HISTORY:

Version  Date  Programmer      Code  Reason
-------  ----- ----------      ----  ------
  1.0    01/94 K.Lowell          C   Initial conversion
  1.1    08/98 T.Ratliff         C   Modified to initialize dtype
				     field in the RDESC structure

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE.
PROJECT:  LAS
***************************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"
#include "worgen.h"
#include "util.h"
#include "construct.h" 
#include "cluster.h"


int getpar
(
    struct PARBLK *vblk,    /* pointer to initial TAE parameter block */
    char hosin[][CMLEN],	/* Host image file names		  */
    long window[][4],	/* Input image window specifications      */
    struct RDESC rdesc[],	/* File information			  */
    construct_struct *construct, /* pointer to construct structure     */
    long *mfact,
    long *msgtim,
    long *nmrimg
)

{

struct VARIABLE *var;       /* ptr to a variable in the vblk             */

char fileutil_filename[CMLEN]; /* temp tile for fileutil       */
char print_filename[CMLEN]; /* print filename for fileutil     */ 
char logflg[3];        /* logarithm application flag           */
char hosout[1][CMLEN];    /* host file name of output image       */
static char taein[MAXIMG][CMLEN];     /* tae file name of input image    */
char taeout[1][CMLEN];    /* tae file name of output image        */

long element = 0;      /* parameter element number             */
long status;	       /* function return status               */
long maxmem;           /* maximum excess memory to use         */
long print[3]; 
long TERM = 0;
long LP = 1;
long FIL = 2;
static long bands[MAXIMG][MAXBND+1];
long nbands[MAXIMG];
long totbnd = 0;
long same_window;
long qflag[MAXIMG][2]; 
long imglim = MAXIMG;
long image;
long ii = 0;
long nonfatal = 1;
long one = 1;

/* Get the input scene name
---------------------------*/
status = c_getin(vblk,"INPUT",&element,taein,hosin,
                 nmrimg,window,bands,nbands,
                 &totbnd,&imglim,&same_window,qflag);
if (status != E_SUCC)
  return(E_FAIL);
if (same_window != one)
  {
  c_errmsg("Input images do not have same window sizes","cluster-window",&nonfatal);
  return(E_FAIL);
  } 
construct->i_file=strdup(hosin[0]);
construct->i_flag = 1;

for (image=0; image < *nmrimg; image++)
  {
  rdesc[image].access = IREAD;
  rdesc[image].fd = NULL;
  rdesc[image].nlines = window[image][NL];
  rdesc[image].nsamples = window[image][NS];
  rdesc[image].nbands=nbands[image];
  for (ii = 0; ii < nbands[image];ii++)
    rdesc[image].bands[ii] = bands[image][ii];
  strcpy(rdesc[image].host[0], hosin[image]);
  rdesc[image].dtype = 0;
  }

/* Get the output file
------------------------*/
status = c_getout(vblk,"OUTPUT",taeout,hosout);
if (status != E_SUCC)
  return(E_FAIL);
construct->o_file=strdup(hosout[0]);

construct->o_flag = 1;

/* Get the history report file
------------------------------*/
strcpy (fileutil_filename, "cluster");
status = c_setprt (vblk, &print[TERM], &print[FIL], &print[LP],
                  fileutil_filename, print_filename);
if (status != E_SUCC)
  return(E_FAIL);
construct->h_file=strdup(fileutil_filename);
construct->h_flag = 1;

/*Get the number of clusters
------------------------------*/
var = p_find(vblk,"NCLUSTER");
construct->n_int = IVAL(*var,0);
construct->n_flag = 1;

/* Get the number of iterations
---------------------------------*/
var = p_find(vblk,"NITER");
construct->it_int = IVAL(*var,0);
construct->it_flag = 1;

/* Get the amount of additional memory to allocate
----------------------------------------------------*/
var = p_find(vblk,"MALLOC");
maxmem = IVAL(*var,0);
if ( maxmem > 0)
  {
  construct->max_int = IVAL(*var,0);
  construct->max_flag = 1;
  }
else
  {
  construct->max_int = 2;
  construct->max_flag = 1;
  }

/* Get the logarithmic application flag
-----------------------------------------*/
var = p_find(vblk,"LOGFLG");
strcpy(logflg, SVAL(*var, 0));
if (strcmp(logflg,"YES") == 0)
  {
  construct->log_logic = 1;
  construct->log_flag  = 1;
  }
else
  {
  construct->log_logic = 0;
  construct->log_flag  = 0;
  }

/* Get the multiplicity factor
--------------------------------*/
var = p_find(vblk,"MULTFACT");
*mfact=IVAL(*var,0);

/* Retrieve globals
-------------------*/
var = p_find(vblk,"$msgtim");
*msgtim = IVAL(*var,0);


return(E_SUCC);
}


