#ifndef IIO_H
#define IIO_H

/*
 *****************************************************************************
 *
 *  iio.h
 *
 *  Header file with definitions for incremental I/O routines for 
 *  reading and writing VIFF files.  
 *
 *  Written by:  Patrick M. Kelly & James M. White
 *               Computer Research Group C-3
 *               Los Alamos National Laboratory
 *
 *  Date:  11/21/91
 *
 *****************************************************************************
 *
 *  iio_struct contains the following fields:
 *
 *  struct xvimage *im;
 *
 *       The standard internal VIFF image without any image data.  Contains
 *       the image header and map data.  Allocated by iio ().
 *
 *  char *filename;
 *
 *       The filename for the input or output file.  Supplied by the calling
 *       program.
 *
 *  int bufsize;
 *
 *       Indicates the size of the data buffer.  The calling program sets this
 *       to a desired size (0 for default size to be used), and iio () will
 *       reset this value to the actual buffer size allocated.  (Size is in
 *       bytes).
 *       
 *  char *buffer;
 *
 *       The input/output buffer for the data.  Allocated by iio ().
 *
 *  char *singbuf;
 *
 *       The input buffer for a single band of data.  Allocated by iio ().
 *
 *  char **bufptr;
 *
 *       Pointers into "buffer".  When reading or writing data in 
 *       non-interleaved format, these pointers point to the positions
 *       that data from each different band of data begins.  Allocated 
 *       by iio ().
 *
 *  int file;
 *
 *       The Unix file pointer for the file.
 *
 *  off_t *data_beg;
 *
 *        File offsets to the beginning of each band of data in the file.
 *        Allocated and managed by iio ().
 *
 *  off_t *data_cur;
 *
 *        Current file offsets for oeach band of data in the file.  
 *        Allocated and managed by iio (). 
 *
 *  int elem_size;
 *
 *        Size of a single element on the current machine.
 *
 *  int elem_sizef;
 *
 *        Size of a single element in the file.
 *
 *****************************************************************************
 */

struct iio_struct {
	struct xvimage *im;	/* Image w/out any image data allocated/stored */
char *filename;	/* Name of the file */
long bufsize;		/* Length of I/O buffer in bytes */
char *buffer;		/* The I/O buffer */
char *singbuf;		/* Buffer for a single band of data */
char **bufptr;		/* The band pointers into the I/O buffer */
long file;		/* File descriptor of file to do I/O with */
long *data_beg;		/* Beginning positions of data bands */
long *data_cur;		/* Current positions for each data band */
long elem_size;		/* Size of a single element on the current machine */
long elem_sizef;	/* Size of a single element in the file */
};

extern long iio ( /* &iiostr, operation, band, number */ );


#define MAX_BUFFER_SIZE 10000000


#endif
