/*************************************************************************
NAME: LCONSTRUCT.C

PURPOSE: Part of the CONSTRUCT program.  Main processing calls for
         CONSTRUCT occur here.

PROGRAM HISTORY:
Version  Date    Programmer        Code  Reason
-------  -----   ----------        ----  ------
  1.0    01/94   K.Lowell            C   Inital conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
****************************************************************/
#include <stdlib.h>
#include "las.h"
#include <math.h>
#include "cluster.h"

#define MEAN_VECTORS_ONLY 0


long lconstruct 
(
    struct PARBLK *vblk,	    /* pointer to initial TAE parameter block */
    struct RDESC rdesc[],       /* file descriptors to the images         */
    char *infile,                /* The input filename */
    char *output,		        /* THe output filename */
    FILE *hist_file_ptr,        /* Output history file */
    long numclust,              /* Number of clusters */
    long iters,                 /* Number of algorithm iterations */
    long maxsize,               /* Maximum of ecess memory */
    long use_log,               /* Use logarithm of data */
    long mfact,		            /* multiplicity factor   */
    long msgtim,		        /* Message time		*/
    long window[],              /* Processing window	*/
    long nmrimg 		        /* number of input images  */
)
{
float **pc_mults_uc, *vecs_pc,  *pc; /* Processing arrays	*/ 
float t = 0;		     /* Hold float value	*/
float t2 = 0;		     /* Hold float value	*/
struct DDR ddr;		     /* local ddr structure */

float **means;        	     /* Mean vectors */
float **outlut;              /* Mean vectors */
float *ptr;		     /* pointer into lookup table   */

FILE  *fstat;	             /* output stat file pointer   */
 
long fatal = -1;             /* fatal message */ 
long nonfatal = 1;           /* nonfatal error flag	*/
long numbands = 0;           /* Number of data bands in image */
long numlines = 0;           /* Number of rows */
long numsamp = 0;            /* Number of cols */
long exchange;	             /* Exchange flag for outlut sort */

long xx,kk,ii,jj,mm,a,b,c;   /* count indicator */

float *conv_table;           /* float table	*/

char comment [512];          /* Comment for output image */
char meanstr[2000][CMLEN];   /* Cluster means string */
char covstr[512];            /* Covariance string    */
char tmpstr[CMLEN];          /* temp hold string     */

long ilmax; 
int *lookup;
long *count;		     /*         */
long time0, time1;       /* Counters for internal timer */
float hold[6];




/* Do some initializations.
----------------------------*/
if(E_FAIL == c_getddr(infile,&ddr))
  {
  c_errmsg("Error opening DDR","cluster-open",&nonfatal);
  c_errmsg("Fatal error encountered","cluster-fatal",&fatal);
  }
for (ii=0;ii<nmrimg;ii++)
  numbands += rdesc[ii].nbands;
/*
numlines = ddr.nl;
numsamp  = ddr.ns;
*/
numlines = window[NL];
numsamp  = window[NS];

fprintf(hist_file_ptr,"Input File         = %s\n",infile);
fprintf(hist_file_ptr,"Rows               = %d\n",numlines);
fprintf(hist_file_ptr,"Columns            = %d\n",numsamp);
fprintf(hist_file_ptr,"Bands              = %d\n",numbands);
fprintf(hist_file_ptr,"Number of Clusters = %d\n",numclust);
fprintf(hist_file_ptr,"Iterations         = %d\n",iters);
fprintf(hist_file_ptr,"Multiplicity factor= %d\n",mfact);
fflush (hist_file_ptr);

/* Do some memory allocations for mean vectors, etc....
--------------------------------------------------------    */

means = contig_matrix ( numclust, numbands );
outlut = contig_matrix ( numclust, numbands );

if ( ( count = (long *) malloc ( sizeof (long) * numclust ) ) == NULL ) 
  {
  c_errmsg("Memory allocation error","cluster-alloc",&fatal);
  }

/* Initilization for structures to speed up the search time.
------------------------------------------------------------*/
ilmax = 256;
lookup=(int *) malloc((unsigned) (ilmax)*sizeof(long));
if (!lookup)
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }

pc_mults_uc = (float **)contig_matrix ( numbands, 256 );
pc = (float *) malloc ( sizeof(float) * numbands );
if ( pc == NULL ) 
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }
vecs_pc = (float *) malloc ( sizeof(float) * numclust ) ;
if ( vecs_pc == NULL ) 
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }


/*  Build the LUT.
------------------- */
fprintf(hist_file_ptr,"Entering Construction Routine ...\n") ;
fflush (hist_file_ptr);

time0 = timer ();

switch (ddr.dtype) 
     {
      case 1:

         fprintf(hist_file_ptr,"Input is BYTE Data\n");
         fflush (hist_file_ptr);
         conv_table = (float *) malloc ( sizeof(float) * 256 );
         if ( conv_table == NULL ) 
           {
           c_errmsg("Error allocating memory","cluster-alloc",&fatal);
           }

         if ( use_log ) 
           { 
           for ( kk = 0 ; kk < 256 ; kk ++ ) 
             {
             conv_table [kk] = (float) log ( (double) (1 + kk) );
             }
              getdata_uc (
                 rdesc,outlut, means, numlines, numsamp, 
                 numbands, 
                 iters,
                 numclust, 0, ilmax, lookup, maxsize, hist_file_ptr,
                 conv_table,count,pc_mults_uc, vecs_pc, 256.0, pc, use_log,
                 mfact,window,nmrimg) ;
           } 
         else  
           {
           for ( kk = 0 ; kk < 256 ; kk ++ ) 
             {
             conv_table [kk] = (float) kk ;
             }
              getdata_uc (
                 rdesc, outlut, means, numlines, numsamp, numbands, 
                 iters,
                 numclust, 0, ilmax, lookup, maxsize, hist_file_ptr, 
                 conv_table,count,pc_mults_uc, vecs_pc, 256.0, pc, 
                 use_log, mfact,window,nmrimg) ;
           }
         break;

      case 2:

         fprintf(hist_file_ptr,"Input is Short data\n");
	 fflush (hist_file_ptr);

         getdata_sh (
	    infile, outlut, means, numlines, numsamp, numbands, iters,
            numclust, ilmax, lookup, maxsize, hist_file_ptr,
            count, vecs_pc, pc, mfact ) ;

         break;


      case 4:
 
         fprintf(hist_file_ptr,"Input is Floating Point Data\n");
         fflush (hist_file_ptr);

         getdata_fl (
            infile, outlut, means, numlines, numsamp, numbands, iters,
            numclust, ilmax, lookup, maxsize, hist_file_ptr,
            count, vecs_pc, pc, mfact ) ;
        
         break;
 


      default:

         c_errmsg("Invalid data type","cluster-dtype",&fatal);

   }

   time1 = timer ();

   fprintf(hist_file_ptr,"Time to complete = %d seconds.\n", 
        (int)(time1-time0)) ;

   fflush (hist_file_ptr);

/*  (7)  Create an output image with the codebook; no image data.
       ONLY CONTINS MEAN VECTORS AT THIS TIME.
---------------------------------------------------------------------*/

  sprintf(comment,"Cluster centers from %s with %d clusters",infile,numclust);


 if ((fstat = fopen("outstat.lt","w+")) == NULL)
  {
  c_errmsg("Error opening output labeled table file","cluster-open",&fatal);
  }

for (mm = 0; mm < numclust  ; mm++)
  {
  for (a = 0; a < numclust-1; ++a)
    {
    exchange = 0;
    for (jj=0;jj<numbands;jj++)
      {
      t += (outlut[a][jj] * outlut[a][jj]);
      }
    for (b = a+1; b < numclust; ++b)
      {
      for (xx=0;xx<numbands;xx++)
        {
        t2 += (outlut[a][xx] * outlut[a][xx]);
        }
      if (t2 < t)
        {
        c=b;
        for (jj = 0; jj<numbands;jj++)
          {
          hold[jj]=outlut[b][jj];
          }
        exchange=1;
        }
      }
    if (exchange)
      {
      for (ii = 0; ii < numbands; ii ++)
        {
        outlut[c][ii] = outlut[a][ii];
        outlut[a][ii] = hold[ii]; 
        }
      }
    }
  }


ptr = *outlut;

fprintf(fstat,"%s",";\n");
fprintf(fstat,"%s","\"COVAR\",\"\";\n");
fprintf(fstat,"%s","\"INDEX\",\"I4\",\"unique index number\",1,1\n");
fprintf(fstat,"%s","\"LEVEL\",\"C\",\"\",1,6\n");
fprintf(fstat,"%s","\"NBANDS\",\"I4\",\"\",1,1\n");
fprintf(fstat,"%s","\"CLASS\",\"C\",\"\",1,100\n");
fprintf(fstat,"%s","\"SITE\",\"C\",\"\",1,100\n");
fprintf(fstat,"%s","\"NPOINTS\",\"I4\",\"\",1,1\n");
fprintf(fstat,"%s%d\n","\"MEAN\",\"R8\",\"\",1,",numbands);
fprintf(fstat,"%s,%d,%d%s","\"COVAR\",\"R8\",\"\"",numbands,numbands,";\n");


strcpy(tmpstr,"");
for (ii = 0; ii < numclust; ii++)
  {
  strcpy(meanstr[ii],"");
  }

mm = 0;
for (ii = 0; ii < numclust; ii++)
  {
  for (jj = 0; jj < numbands; jj++)
    {
    sprintf(tmpstr,"%f",ptr[mm]);
    strcat(meanstr[ii],tmpstr);
    strcat(meanstr[ii],",");
    mm++;
    strcpy(tmpstr,"");
    }
  }

for (ii = 0; ii < numbands*numbands; ii++)
  {
  strcat(covstr,"0");
  if ((ii + 1) == (numbands*numbands)) 
    {
    strcat(covstr,";\n");
    }
  else
    {
    strcat(covstr,",");
    }
  
  }

for (jj = 0; jj < numclust; jj++)
  {
  if ( jj <= 8)
    {
    kk = jj + 1;
    fprintf(fstat,"%d,%s,%d,%s%d%s,%d,%s%s",kk,"\"CLASS\"",numbands,"\"CLUST0",
            kk,"\",\"\"",10,meanstr[jj],covstr);
    } 
  else
    {
    kk = jj + 1;
    fprintf(fstat,"%d,%s,%d,%s%d%s,%d,%s%s",kk,"\"CLASS\"",numbands,"\"CLUST",
            kk,"\",\"\"",10,meanstr[jj],covstr);
    }
  }

if (fclose(fstat) != NULL)
  {
  c_errmsg("Error closing outstat file","cluster-nonfatal",&nonfatal);
  }
lproject (vblk,rdesc,infile,outlut,output,hist_file_ptr,numclust,
          &ddr,use_log,msgtim,window,nmrimg);

return(E_SUCC);
}

