/**************************************************************
NAME: PROJECT_UC_UC

PURPOSE:  To utilize mean vectors stored in an look up table
          to classify a byte image.  Part of the LPROJECT code
          of the CLUSTER program

****************************************************************/

#include "las.h"
#include "util.h"
#include "iio.h"
#include "cluster.h"

typedef unsigned char IN_DATA_TYPE;

typedef unsigned char OUT_DATA_TYPE;

struct FDESC *fdesc[MAXIMG+1];/* Image file descriptors used for S image I/O */
struct GDESC *gdesc;		  /* Group descriptor used for image I/O */
long offset1[MAXIMG*MAXBND];  /* Offsets into thebuf for input bands */

float project_uc_uc
(
    struct PARBLK *vblk,		/* pointer to initial TAE parameter block */
    struct RDESC rdesc[],
    float **outlut,			/* Vectors to find nearest */
    long ndim,			/* Problem dimension */
    long numline,			/* Number of lines */
    long numsamp,			/* Number of columns */
    long lutsz,			/* Number of vectors to find nearest */
    char *outfile,			/* Output filename */
    int  *lookup,			/* For finding close guess */
    long lookup_size,		/* Size of lookup */
    float ***covar,			/* Covariance matrices */
    float **means,			/* Mean vectors */
    int  *count,			/* Number of elements */
    char *infile,           /* Input file name */
    float *conv_table,		/* For conversion to float */
    float **pc_mult_uc,		/* Mults. for pc times unsigned char */
    float *means_pc,		/* Principal component of vectors in set */
    long msgtim,			/* Message processing time   */
    long window[],			/* Processing window     */
    long nmrimg			/* Number of input images  */
)
{

extern long curent;             /* current processing line                   */
extern long curcat;		/* current processing category               */
OUT_DATA_TYPE *tmpdatout;
float scale;
long k,ii,i,j,isize,isize2;	/* Local loop counters    */
long acc;
long curlin;			/* Current line counter		*/
long band = 0;			/* Current band			*/
long bands[MAXIMG];	        /* Bands array for eopens       */
long cflag;			/* flag to be sent to upddr	*/
long numband[MAXIMG][MAXBND+1]; /* band numbers of each of the input images */
long nbands[MAXIMG];		/* number of bands per image	*/
long status;			/* Status indicator flag	*/
long fatal = -1;		/* Fatal status flag		*/
long nonfatal = 1;		/* Nonfatal status flag		*/
long dtype;			/* Data type			*/
struct FDESC *odesc;			/* Output file pointer		*/
long bufsiz;                 	/* size of image I/O buffer     */
long opt[2];			/* Parameter for eopens	        */
long dwindow[MAXIMG][4];	/* Processing window		*/
long msg_flag=ALLCAT;		/* Message flag for c_inittm    */
long image;			/* Processing loop counter      */

char comment[512];		/* Comment string		*/
float **scratch;		/* Working matrix		*/
long outpix_int;		/* Output pixel value(cluster)  */
long buffer_full;		/* Counter to indicate full write buffer */
long    zero = 0;               /* value of zero                */
long    one = 1;                /* one                          */
long pxcnt[255];	        /* pixel counter- number per class	*/

double line_inc;	        /* number of line increments from input */
double samp_inc;	        /* number of line increments from input */

long num_in_buffer;		/* Counter-number of pixels in write buffer */

char  *thebuf;			/* Processing buffer		*/	
unsigned char  *vbuf[MAXBND];   /* Processing buffer		*/	

scale = ((float) lookup_size) / ( means_pc [lutsz-1] - means_pc [0] );

sprintf(comment,"Clustered from %s with %d clusters",infile,lutsz);

for (ii=0;ii<255;ii++) pxcnt[ii] = 0;


/* Open input and output images
--------------------------------*/
acc = IREAD;
dtype = EBYTE;
opt[0]=0;
opt[1]=0;
for (band=0,image=0;image<nmrimg;image++)
  {
  bands[image]=0;
  bands[image+1]=0;
  if (status = c_eopens(&fdesc[image],rdesc[image].host[0],bands,
                        &window[0],&window[1],&window[2],&window[3],
                        &offset1[band],&acc,&dtype,&zero,opt) != E_SUCC)
    {
    c_errmsg("Error opening image file","cluster-open",&fatal);
    }
  band += rdesc[image].nbands; 
  }

thebuf = c_ealloc(fdesc,&bufsiz);

tmpdatout = (unsigned char *)malloc((unsigned)(sizeof(unsigned char)*numsamp));
  if(!tmpdatout)
  {
  c_errmsg("Error allocating memory","cluster-alloc",&fatal);
  }

status=c_egroup(fdesc,&gdesc,(unsigned char *)thebuf,&bufsiz);

acc = IWRITE;
if (status = c_eopenr(&odesc,outfile,&one,&one,&one,&numline,
                      &numsamp,&acc,&dtype,opt,&one) != E_SUCC)
  {
  c_errmsg("Error opening output image file","cluster-open",&fatal);
  }



/* Allocate Memory 
---------------------- */   
scratch = (float **)contig_matrix(ndim,2);



/* Zero Data 
-------------- */

for(i=0; i < lutsz; ++i)
  {
  for(j=0; j < ndim; ++j)
    {
    for(k=j; k < ndim; ++k)
      {
      covar[i][j][k] = 0.0;
      }
    }
  for(j=0; j < ndim; ++j)
    {
    means[i][j] = 0.0;
    }
  count[i] = 0;
  }


isize = sizeof (IN_DATA_TYPE);
isize2 = sizeof (OUT_DATA_TYPE);

c_inittm("CLUSTER","Processed","line","bands"," ",&numline,&ndim,
         &msgtim,&msg_flag);
curent = 1;
c_startm();

num_in_buffer = 0;
buffer_full = numsamp / isize2;

for (curlin = 1; curlin <= numline; curlin++)
  {
  curent = curlin;
  if (c_estep(&gdesc) != E_SUCC)
    c_errmsg("Error reading input image","cluster-fatal",&fatal);
  for (band = 0; band < ndim; band++)
   {
   vbuf[band] = (unsigned char *)(thebuf +(*(offset1+band) * isize));
   }         

	 for ( i = 0 ; i < numsamp ; i ++ ) 
           {
           outpix_int = nn_uc_pc_ul ( ndim, vbuf, i, lutsz, 
			outlut, conv_table, lookup, pc_mult_uc, 
	                means_pc, scale );
           
           pxcnt[outpix_int]++;

	   tmpdatout [num_in_buffer++] = (OUT_DATA_TYPE) outpix_int;
           if (num_in_buffer == buffer_full)
             {
             c_ewrite(&odesc,&one,&curlin,tmpdatout,&one);
             num_in_buffer = 0;
             }


	   SM_add_vector_uc_tbl_ul (ndim,&count[outpix_int],means[outpix_int], 
		covar[outpix_int], vbuf, i, scratch, conv_table );

  	   }
	}

c_stoptm();

/* Update the DDR
  --------------*/
for (ii=0;ii<ndim;ii++)
  {
  dwindow[ii][0] = one;
  dwindow[ii][1] = one;
  dwindow[ii][2] = numline;
  dwindow[ii][3] = numsamp;
  for (band=0;band<ndim;band++)
    numband[ii][band] = rdesc[ii].bands[band];
  nbands[ii] = rdesc[ii].nbands;
  }

line_inc = (double)1;
samp_inc = (double)1;
cflag = COMB;
status = c_upddr(vblk,rdesc[0].host,outfile,&one,dwindow,numband,
                 nbands,&line_inc,&samp_inc,&one,&cflag);
if (status != E_SUCC)


  c_errmsg("Error updating ddr","cluster-upddr",&nonfatal);

/* Create the HIS file
-----------------------*/
strcat(comment," ");
c_hstout(vblk,outfile,infile,&one,comment);

/* Write out statistics into lableled table
-------------------------------------------*/
if (put2lt(ndim,lutsz,means,covar,pxcnt) != E_SUCC)
  c_errmsg("Error writing output statistics ","cluster-write",&fatal);


/* Close input and output files
--------------------------------*/
c_eclose(&odesc);
c_egclse(&gdesc);	

/* Deallocate Memory 
--------------------------- */     
free(thebuf);

free((char *) *scratch);
free((char *) scratch);


return (E_SUCC);
}
