/******************************************************************************

PROGRAM:	CMALIAS     CMALIAS-CREATE, CMALIAS-DELETE, CMALIAS-LIST

PURPOSE:	Creates (CMALIAS-CREATE), deletes (CMALIAS-DELETE), or
		lists (CMALIAS-LIST) aliases.

FATAL ERROR CONDITIONS:
		Alias name must begin with '$'
		Alias name too long
		Specified alias name already exists
		Specified alias name not found
		Unable to create new alias file
		Unable to open alias file

OTHER ERROR CONDITIONS:
		Specified alias not found

PROGRAM HISTORY:

Version    Date     Author           Code/Contr    Change Request
-------    -----    -------------    ----------    ---------------
  1.0      10/86    C. Greenhagen     CSB          New Development
  5.0	    9/88    B. Davis	      CSB	   5.0 Conversion
  5.1       8/89    B. Ailts          CSB          Checked for lower case before
						   upper casing the subcommand
  5.2       8/89    B. Davis          CSB          Ran input text through
						   getalias when creating

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE 

ALGORITHM(flow)
  call inlas for initialization
  get tae subcommand
  if subcmd is create
     get alias name and text
     call getalias in case an existing alias is part or all of the text
     call putalias to create alias and save on disk
     if the alias file needed to be newly created
	save its name and set number of output file names for complt
  else if subcmd is delete
     get alias name
     call delalias to delete alias
  else if subcmd is list
     call setprt to get print option(s)
     if file option was chosen
        call mknam to create the name for the output disk file
	set number of files and save name for complt
     open the temporary disk file needed for fileutil
     call listalias to write aliases to the temp file
  call complt

******************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include "las.h"
#include "util.h"
#include "alias.h"

#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif

#ifdef gould
#include "diskio.h"
#endif

void main()
{

#define ASZ	9			/* max length of alias name	*/
#define TSZ	CMLEN-1			/* max length of alias text	*/

struct PARBLK vblock;			/* TAE parameter block		*/

struct VARIABLE *in;			/* pointer to variable		*/

FILE *fp;				/*file pointer to tmp fil for fileutil*/

long fatal = 0;				/* value of zero		*/
long file;				/* flag for output to file      */
long i;					/* loop counter			*/
long lnpr;				/* flag for output to printer   */
long newfile;				/* new alias file created?	*/
long numout = 0;			/* # of output files created	*/
long one = 1;				/* value of one			*/
long status;				/* return status of support call*/
long stdlun;				/* logical unit # of std output	*/
long term;				/* flag for output to termial   */

char aliasfile[CMLEN];			/* alias file name		*/
char aliasname[10];			/* alias name			*/
char aliastext[CMLEN];			/* alias text			*/
char newtext[CMLEN];			/* possibly expanded alias text	*/
char outfile[CMLEN];			/* output file names created	*/
char *p;				/* temporary pointer		*/
char pname[CMLEN];			/* Program name			*/
char subcmd[10];			/* subcommand			*/
char tmpfil[1][CMLEN];			/* output file name		*/

/************************************************************************/

c_inlas (&vblock, &stdlun);			/* Initialize		*/

strcpy(pname,"CMALIAS");			/* build program name	*/

strcpy(outfile," ");				/* init output name for complt*/

in = p_find (&vblock,"_SUBCMD");		/* get subcommand	*/
strcpy (subcmd, SVAL (*in,0));

for (i=0, p=subcmd; i<strlen(subcmd); i++,p++)	/* upcase subcommand	*/
    if (islower(*p))
       *p = toupper(*p);

if (strcmp (subcmd,"CREATE") == 0)
    {
    in = p_find (&vblock,"ALIASNAM");
    strcpy (aliasname, SVAL (*in,0));
    if (aliasname[0] != '$')
	{
	c_errmsg ("Alias name must begin with '$'","cmalias-name",&one);
	c_errmsg ("Fatal error encountered","cmalias-fatal",&fatal);
	}
    in = p_find (&vblock,"TEXT");
    strcpy (aliastext, SVAL (*in,0));
    status = c_getalias(aliastext,newtext);
    if (status != E_SUCC)
	c_errmsg ("Fatal error encountered","cmalias-fatal",&fatal);

    status = putalias (aliasname,newtext,aliasfile,&newfile);
    if (status != E_SUCC)
	c_errmsg ("Fatal error encountered","cmalias-fatal",&fatal);
    if (newfile)
	{
	strcpy(outfile,aliasfile);
	numout = numout + 1;
	}
    }

else if (strcmp (subcmd,"DELETE") == 0)
    {
    in = p_find (&vblock,"TEXT");
    strcpy (aliasname, SVAL (*in,0));
    status = delalias (aliasname);
    if (status != E_SUCC)
	c_errmsg ("Fatal error encountered","cmalias-fatal",&fatal);
    }

else if (strcmp (subcmd,"LIST") == 0)
    {
    in = p_find (&vblock,"TEXT");
    strcpy (aliasname, SVAL (*in,0));

    /* get print option, make a file name if necessary, open temp file */

    strcpy(tmpfil[0],"cmalias");
    status = c_setprt(&vblock,&term,&file,&lnpr,tmpfil[0],outfile);
    if (status != E_SUCC)
	c_errmsg ("Fatal error encountered","cmalias-fatal",&fatal);
    if (file == 1)
       numout = numout + 1;
    if ((fp = fopen(tmpfil[0],"w")) == NULL)
	{
	c_errmsg ("Error opening temporary print file","cmalias-fopen",&one);
	c_errmsg ("Fatal error encountered","cmalias-fatal",&fatal);
	}
    status = listalias (fp,aliasname);
    if (status != E_SUCC)
	{
	fclose(fp);
	c_clean(&vblock,pname,tmpfil,&one);
	}
    fclose(fp);
    }

c_complt (pname, outfile, &numout);
}
