/******************************************************************************

PROGRAM:	CMCOPY

PURPOSE:	Copies Cataloged files.

		If both the file part and the extension are specified,
		the new file is named as specified.

		If the filepart is "*", the new file name uses the file part
		of the current file name, and the new extension specified.

		If the extension is "*" or unspecified,	the new file name
		uses the extension of the current file name, and the new
		file part specified.

		If the extension is unspecified, the image and all
		associated files is assumed.

FATAL ERROR CONDITIONS:
		Specification error
		Unable to open alias file
		Specified alias not found: <aliasname>

OTHER ERROR CONDITIONS:
		Output file already exists: filename
	VMS:
		Error writing to file <filename> status = <status>
		Error opening input file: <filename> status = <status>
		Error connecting to input file: <filename> status = <status>
		Error opening output file: <filename> status = <status>
		Error connecting to output file: <filename> status = <status>
	UNIX:
		Error copying file <filename> to <filename>

PROGRAM HISTORY:

Version    Date     Author           Code/Contr  Change Request
-------    -----    -------------    ----------  ---------------
  1.0      10/86    C. Greenhagen     EDC/CSB    New Development
  5.0       9/88    B. Davis	      CSB	 Conversion to 5.0
  5.1       3/89    B. Davis	      CSB	 Increased size of temp file
						 names and extensions in
						 validate.c from 40 to CMLEN
  5.2       8/89    B. Davis	      CSB	 Passed vblock to processfile
						 to enable hstout call, added
						 histry flag to pdf.
  5.3       2/90    B.Davis/D.Hollaren CSB	 Added imgfalg and aftype to
						 consolidate catalog manager
						 name parsing.  Default image
						 names should imply the img,
						 ddr, and his files only.  Extra
						 steps should have to be taken
						 to include more, or to exclude 
						 these.
  5.4      11/90    D. Etrheim        CSB        Update call to processfile.
						 Correct problems copying the
						 history file.
  5.5      01/92    D. Etrheim        CSB	 Check for extension on input
						 but not on output

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE 

ALGORITHM(flow)
  initialize tae
  get parameters
  expand aliases, if any
  call validate to ensure valid parameter names
  call processfile to do the actual copying
  call complt and terminate

******************************************************************************/

#include <stdio.h>
#include <string.h>

#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif
#include "las.h"
#include "util.h"
#include "alias.h"
#include "protos.h"

void main
(void)
{
struct PARBLK vblock;			/* TAE parameter block		*/

struct VARIABLE *in;			/* pointer to variable		*/

long acount;				/* # associated file extensions */
long copyflg = TRUE;			/* dummy value for processfile  */
long fatal = 0;				/* value of zero		*/
long imgset;				/* img & assoc files? (t=1)	*/
long imageflg;				/* Copy image flag		*/
long k;					/* Looping counter		*/
long ncopied = 0;			/* # files copied		*/
long one = 1;				/* value of one			*/
long status;				/* status			*/
long stdlun;				/* logical unit # of std output	*/

char aftype[100][CMLEN];		/* List of associated file extensions */
char name[CMLEN];			/* current cm TAE filename	*/
char newname[CMLEN];			/* new cm TAE filename		*/
char tmpnm[1][CMLEN];			/* output file names for complt	*/
char pname[7];				/* Program name			*/
char tname[CMLEN];			/* name with aliases expanded	*/
char tnewname[CMLEN];			/* newname with aliases expanded*/

/************************************************************************/

c_inlas (&vblock, &stdlun);			/* Initialize		*/

strcpy(pname,"CMCOPY");				/* name for complt	*/

in = p_find (&vblock,"INFILE");			/* get input name	*/
strcpy (name, SVAL (*in,0));

in = p_find (&vblock,"OUTFILE");		/* get new name		*/
strcpy (newname, SVAL (*in,0));

/*
  Convert filenames to host syntax.
*/
c_cmhost(name, tname);
c_cmhost(newname, tnewname);
strcpy(name, tname);
strcpy(newname, tnewname);

/*
  Get the associated file types to be copied
*/
in = p_find (&vblock,"AFTYPE");
strcpy(tmpnm[0], SVAL(*in,0));

if (strcmp(tmpnm[0],"NONE") == 0)
    acount = 0;
else
    acount = (*in).v_count;

for (k = 0; k < acount; k++)
    strcpy(aftype[k], SVAL(*in,k));

/*
  Get the flag to copy image
*/
in = p_find (&vblock,"IMAGEFLG");
if (strcmp(SVAL(*in,0),"YES") == 0)
   imageflg = TRUE;
else
   imageflg = FALSE;


status = c_getalias (name,tname);		/* expand aliases, if	*/
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","cmcopy-fatal",&fatal);

status = c_getalias (newname,tnewname);		/* any, in parameters	*/
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","cmcopy-fatal",&fatal);

validate (tname,tnewname,&imgset);		/* validate input	*/

processfile(&vblock,"cmcopy",tname,tnewname,imgset,imageflg,aftype,&acount,
	    0,&ncopied,copyflg);

if (ncopied == 0)
    c_errmsg ("No files copied","cmcopy-ncopied",&one);
else
    ncopied = 0;

strcpy(tmpnm[0]," ");
c_complt (pname,tmpnm,&ncopied);
}
