/*******************************************************************************

FUNCTION:	VALIDATE

PURPOSE: checks the directory or file specification for validity

	 "*" may be used as the filename or as the extension,		
	    but not both.					
	 "*" may not be used as a part of the filename or
	    as a part of the extension.					
	 If INFILE uses "*" as the filename,			
	    OUTNAME must also use "*" as the filename.	
	 If INFILE uses "*" as the extension,	
	    OUTNAME must also use "*" as the extension.			
	 If INFILE leaves the extension unspecified		
	    (implying image and all associated files),	
	    OUTNAME must also leave the extension unspecified.		
	 For VMS only, if INFILE uses LIMG as the extension,	
	    OUTNAME must also use LIMG as the extension.
	    In this case, nam will be set to "filename;LIMG__*" and	
	    newnam will be set to "filename;*" before returning.
	 						
	 If an image set (image and associated files) is specified,	
	 nam will be set to "filename;" and newnam will be set to
	 "filename;*" before returning.				

FATAL ERROR CONDITIONS:	Specification error occurrs when the following checks
				are not successful

ALGORITHM(flow)
******************************************************************************/


#include <stdio.h>
#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif
#include "las.h"
#include "protos.h"

void validate
(
    char *nam,				/* current TAE name		*/
    char *newnam,				/* new TAE name			*/
    long *imgset				/* img and assoc files? (t=1)	*/
)
     
{
long fatal = 0;				/* value of zero		*/
long one = 1;				/* value of one			*/

char curext[CMLEN];			/* extension of current name	*/
char curfile[CMLEN];			/* file part of current name	*/
char newext[CMLEN];			/* extension of new name	*/
char newfile[CMLEN];			/* file part of new name	*/
char *p;				/* temporary pointer		*/

/*****************************************************************************/

*imgset = 0;

if ((p = strrchr (nam,'/')) == NULL)		/* isolate file part	*/
    strcpy  (curfile,nam);			/* of INFILE		*/
else
    strcpy (curfile,++p);
if ((p = strrchr(curfile, '.')) != NULL)
    *p = NULL;

if ((p = strrchr (nam,'.')) == NULL)		/* isolate extension	*/
    strcpy (curext,"");				/* of INFILE		*/
else
    strcpy (curext,++p);

if ((p = strrchr (newnam,'/')) == NULL)		/* isolate file part	*/
    strcpy  (newfile,newnam);			/* of OUTNAME		*/
else
    strcpy (newfile,++p);
if ((p = strrchr(newfile, '.')) != NULL)
    *p = NULL;

if ((p = strrchr (newnam,'.')) == NULL)		/* isolate extension	*/
   strcpy (newext,"");				/* of OUTNAME		*/
else
    strcpy (newext,++p);

/***  Error conditions  ***/

if (((strchr(curfile,'*') != NULL) && (strlen(curfile) != 1)) ||
    ((strchr(curext,'*')  != NULL) && (strlen(curext)  != 1)) ||
    ((strchr(newfile,'*') != NULL) && (strlen(newfile) != 1)) ||
    ((strchr(newext,'*')  != NULL) && (strlen(newext)  != 1)) ||
    ((curfile[0] == '*')   && (newfile[0] != '*'))    ||
    ((curext[0]  == '*')   && (newext[0]  != '*'))    ||
    ((curfile[0] == '*')   && (curext[0]  == '*'))    ||
    ((newfile[0] == '*')   && (newext[0]  == '*'))    ||
    ((strlen(curext) != 0) && (strlen(newext) == 0))  ||
    ((strlen(curext) == 0) && (strlen(newext) != 0)))
    {
    c_errmsg ("Specification error","cmcopy-name",&one);
    c_errmsg ("Fatal error encountered","cmcopy-fatal",&fatal);
    }

if ((strlen(curext) == 0) && (strlen(newext) == 0))
    {
    *imgset = 1;
    if (strrchr (nam,'.') == NULL)      /* set nam to "filename." */
        strcat(nam, ".");
    if (strrchr (newnam,'.') == NULL)   /* set nam to "filename.*" */
        strcat(newnam, ".*");
    }
}
