/******************************************************************************

PROGRAM:	CMDEL     CMDEL-DIR, CMDEL-FILE

PURPOSE:	Deletes disk directories (CMDEL-DIR) or cataloged files
		(CMDEL-FILE).

FILE REFERENCES:

FATAL ERROR CONDITIONS:
		Alias name too long
		Specified alias name not found
		Unable to open alias file

OTHER ERROR CONDITIONS:
		Specification error

		Error deleting directory from disk: dirname
		Specified directory not found: dirname

		Error deleting file from disk: filename
		Error deleting file from tape: filename
		Specified file not found: filename

PROGRAM HISTORY:

Version  Date   Author         Code/Contr  Change Request
-------  -----  -------------  ----------  ---------------
  1.0    11/86  C.Greenhagen   CSB         New Development
  5.0    08/88  B.Davis        CSB         LAS 5.0 conversion
  5.1    03/89  B.Davis        CSB         Increased size of temp file names
					   and extensions in validate.c from
					   40 to CMLEN
  5.2    08/89  B.Ailts        CSB         Checked if lower case before upper
					   casing subcommand
  5.3    08/89  B.Davis        CSB         Passed vblock to processname for 
					   processfile call.
  5.4    09/89  P.Pease        GSFC        Moved include of las include file to
					   after all system includes.
  5.5    11/89  B.Davis        EDC         Documentation change to pdf
  5.6    02/90  B.Davis &      
		D.Hollaren     CSB         Added imgfalg and aftype to
					   consolidate catalog manager name
					   parsing.  Default image names should
					   imply the img, ddr, and his files
					   only.
					   Extra steps should have to be taken
					   to include more, or to exclude these.
  5.7    10/91  J.Fenno        EDC         Modified support routines and ran
					   through Saber-C.
  5.4    07/92  R.Kruse       EDC/CSB      Remove FILEOPT parameter.
					   CM will not handle offline images.
  5.8    06/97  T.Ratliff      CSB         Made 2 fixes and 1 enhancement. 
					   if pointed-to-file doesn't exist, 
					     .img file will still be deleted.
					   can now delete a large amount of
					     files (> 400).
					   ability to delete a directory that
					     contains files or subdirectories

$Log


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE 

PROJECT:	LAS

ALGOTITHM(flow)
  initialize with inlas, get tae parameters
  for each input file name
     call processname to validate name, expand aliases if any, and process
  call complt to finish

******************************************************************************/


#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include <sys/file.h>
#include "protos.h"

void main
(void)
{
struct PARBLK vblock;			/* TAE parameter block		*/
struct VARIABLE *in;			/* pointer to variable		*/

long acount=0;                          /* # associated file extensions */
long count=0;                           /* # of names specified         */
long i=0;                               /* loop counter                 */
long imageflg=0;                        /* Copy image flag              */
long len=0;                             /* string length                */
long ndel = 0;				/* number of files deleted      */
long one = 1;				/* value of one for errmsg call */
long stdlun=0;                          /* logical unit # of std output */
long verify = 0;			/* verify? (yes=1)              */
long recur = 0;                         /* recursive? (yes=1)           */
long zero = 0;				/* zero for complt call		*/

char aftype[100][CMLEN];		/* List of associated file extensions */
char name[CMLEN];			/* name of file or dir to delete*/
char hname[CMLEN];                    /* host name of file or dir to delete */
char temp[CMLEN];                       /* storage for infile names     */
char dirname[CMLEN];                    /* directory name to delete     */
char olddirname[CMLEN];                 /* checks ifwe are in a loop    */
char *p;				/* temporary pointer		*/
char subcmd[10];			/* subcommand			*/
char tmpnam[CMLEN];			/* temp string			*/
char check[CMLEN];                      /* checks if we've delete dir.  */
char check1[CMLEN];                     /* checks if we've delete dir.  */
char vstring[4];			/* verify option (yes/no)	*/

/************************************************************************/

  c_inlas (&vblock, &stdlun);			/* Initialize		*/

  in = p_find (&vblock,"_SUBCMD");		/* get subcommand	*/
  strcpy (subcmd, SVAL (*in,0));

  for (i = 0, p = subcmd; i < strlen(subcmd); i++,p++)
      if (islower(*p))                            /* upcase subcommand  */
         *p = toupper(*p);

  in = p_find (&vblock,"CONFLG");                 /* get verify option   */
  strcpy (vstring, SVAL (*in,0));
  if ((vstring[0] == 'Y') || (vstring[0] == 'y'))
      verify = 1;
  strcpy(vstring," ");

  if (strcmp (subcmd,"DIR") == 0)     /* delete directory */
  {
    in = p_find (&vblock,"RECURSIVE");   /* get the recursive option */
    strcpy (vstring, SVAL(*in,0));
    if ((vstring[0] == 'Y') || (vstring[0] == 'y'))
        recur = 1;

    if (recur == 0)
    {
      in = p_find (&vblock,"INDIR");             /* get names  */
      count = in->v_count;
      for (i = 0; i < count; i++)
      {
	strcpy(name,SVAL(*in,i));
        c_cmhost(name, hname);
	processname(&vblock,hname,"dir",imageflg,aftype,&acount,verify,&ndel);
      }
      if (ndel == 0)
	c_errmsg("No directories deleted","cmdel-nodir",&one);
    }
    else if (recur == 1)    /* do recursion to delete the directory & files */
    {
      in = p_find (&vblock,"INDIR");   /* get the directory to be deleted */
      count = in->v_count;
      for (i = 0; i < count; i++)
      {
        strcpy(name,SVAL(*in,i));
        c_cmhost(name, hname);

        acount = 1;
        strcpy(aftype[0],"*");     /* set defaults for processname */
        imageflg = 1; 

        memset(olddirname,0,CMLEN);
        strcpy(dirname,hname);
        do
        {
          recurdel(dirname);

         /* Check if we are repeating through a directory. If
           so, we want to exit
           --------------------------------------------------*/
          if (strcmp(dirname,olddirname) != 0)
          {
            strcpy(olddirname,dirname);

          /* Delete all files from the directory
	    ------------------------------------*/
            processname(&vblock,dirname,"file",imageflg,aftype,&acount,verify,
	                &ndel);
	    memset(temp,0,CMLEN);
	    strncpy(temp,dirname,strlen(dirname)-1);
	    strcat(temp,".*");
	    strcpy(dirname,temp);

          /* Delete all . files in the directory
	    ------------------------------------*/
            processname(&vblock,dirname,"file",imageflg,aftype,&acount,verify,
                        &ndel);

          /* Setting up to make call to delete 
             directory
            -------------------------------------*/
            memset(temp,0,CMLEN);
            strncpy(temp,dirname,strlen(dirname)-3); /* chop off "/.*" */
            acount = 0;
            processname(&vblock,temp,"dir",0,aftype,&acount,verify,&ndel);
            acount = 1;

            p = (char *)strrchr(temp,'/');
	    strcpy(check,p);
	    p = (char *)strrchr(hname,'/');
	    strcpy(check1,p);
	    if (strcmp(check,check1) != 0)
              strcpy(dirname,hname);
	    else
	      strcpy(temp,hname);
          }
          else
	    strcpy(temp,hname);
   

        } while (strcmp(temp,hname) != 0 );
        
      }  /* end for count */
      
      if (ndel == 0)
	c_errmsg("No directories deleted","cmdel-nodir",&one);
      
    }  /* end else if recur == 1 */
    
  } /* end if subcmd == dir */

  else if (strcmp (subcmd,"FILE") == 0)   /* delete file */
  {
    /*
      Get the associated file types to be copied
    */
    in = p_find (&vblock,"AFTYPE");
    strcpy(tmpnam, SVAL(*in,0));

    if (strcmp(tmpnam,"NONE") == 0)
      acount = 0;
    else
      acount = (*in).v_count;

    for (i = 0; i < acount; i++)
      strcpy(aftype[i], SVAL(*in,i));

    /*
      Get the flag to copy image
    */
    in = p_find (&vblock,"IMAGEFLG");
    if (strcmp(SVAL(*in,0),"YES") == 0)
      imageflg = TRUE;
    else
      imageflg = FALSE;

     in = p_find (&vblock,"INFILE");			/* get names	*/
     count = in->v_count;

    for (i = 0; i < count; i++)
    {
      strcpy(name,SVAL(*in,i));
      c_cmhost(name, hname);
      len = (long) strlen(hname);
      if (hname[len-1] == '*' && hname[len-2] == '/')
        strcat(hname, ".img");
      processname(&vblock,hname,"file",imageflg,aftype,&acount,verify,&ndel);
    }
    if (ndel == 0)
      c_errmsg ("No files deleted","cmdel-nofile",&one);
    
  } /* end if subcmd == file */

  c_complt("cmdel"," ",&zero);
}
