/*******************************************************************************
NAME                            PROCESSNAME                         

PURPOSE    
  Processname accepts one name from the user-specified parameter
  list (the name may be an alias name referencing a list), and
  extracts and processes one filename or directory name at a time.

FILE REFERENCES                 
  --
   
PROGRAM HISTORY
Version  Date   Author         Code/Contr  Change Request
-------  -----  -------------  ----------  ---------------
  1.0    11/86  C.Greenhagen   CSB         New Development
  5.0    08/88  B.Davis        CSB         LAS 5.0 conversion
  5.1    10/91  J.Fenno        EDC         Updated support routines and ran
                                           through Saber-C.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   must be run under TAE.

PROJECT                LAS                              

******************************************************************************/

#include <stdio.h>

#ifdef unix
#include <sys/file.h>
#endif

#include "las.h"
#include "alias.h"

#include "protos.h"


void processname
(
    struct PARBLK *vblock,                    /* TAE parameter block        */
    char *name,                               /* object name to delete      */
    char *type,                               /* "file" or "dir"            */
    long imageflg,                            /* Process image file         */
    char aftype[][CMLEN],                     /* Assoc file types           */
    long *acount,                             /* # assoc file types         */
    long verify,                              /* verify? (yes=1)            */
    long *ndel                                /* running count of           */
)
                                              /* files deleted              */
{
#define MAXF    300                           /* max #files for dirlst      */

long copyflg = TRUE;                          /* dummy arg for processfile  */
long imgset;                                  /* img & assoc files?         */
long morenames;                               /* more names to process      */
long ndeleted = 0;                            /* # of files deleted by      */
                                              /* one processfile call       */
long status;                                  /* return of support          */
long fatal = 0;                               /* value of zero              */

char *p;                                      /* name pointer               */
char temp[CMLEN];                             /* blank name for processfile */
char tstring[CMLEN];                          /*                            */

/************************************************************************/

strcpy(temp," ");
status = c_getalias (name,tstring);             /* expand aliases,if any*/
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","cmdel-fatal",&fatal);

p = gettoken (tstring,", ");                    /* get name from string */
if (p == NULL)                                  /* no names remaining   */
    morenames = 0;
else
    morenames = 1;

while (morenames)
    {
    if (validate (p,type,&imgset) != E_SUCC)     /* validate input spec. */
        return;

    if (strcmp (type,"dir") == 0)
	processfile(vblock,"cmdeletedir",p,temp,imgset,imageflg,aftype,
		    acount,verify,&ndeleted,copyflg);
    if (strcmp (type,"file") == 0)
	processfile(vblock,"cmdeletefile",p,temp,imgset,imageflg,aftype,
		    acount,verify,&ndeleted,copyflg);


    *ndel += ndeleted;

    p = gettoken (NULL,", ");                   /* get next name        */
    if (p == NULL)                              /* no names remaining   */
        morenames = 0;
    }                                           /* end while (morenames)*/
return;
}
