/*******************************************************************************
NAME                            RECURDEL

PURPOSE
  Recurdel searches the user specified directory for subdirectories.  It
  then returns the lowest subdirectory back to cmdel. 


PROGRAM HISTORY
Version  Date   Author         Code/Contr  Change Request
-------  -----  -------------  ----------  ---------------
  5.2    08/97  T.Ratliff      CSB         Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
    must be run under TAE.

PROJECT               LAS

ALGORITHM
  Using the find command, we find all subdirectories in the user specified
  directory.  Then find the lowest directory and pass it back to cmdel with
  an * at the end of it.

****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

long how_many(char *,long);               /* function counts subdirectories */

void recurdel
(
    char *name                               /* user specified directory */
)

{

long totaldir=0;                          /* loop counter variable */
long j=0;                                 /* loop counter variable */
long subnum=0;                            /* number of subdirectories */
long index=0;                             /* index variable */
long status=0;                            /* status of a function */
long fatal = -1;                          /* used for error statements */
long bufnum[BUFSIZ];                      /* contains # of subdirectories */

char buffer[BUFSIZ][BUFSIZ];              /* path of subdirectories */
char sysstr[BUFSIZ];                      /* system calls */
char hname[CMLEN];                        /* host name */
char temp[CMLEN];                         /* temporary string storage */
char errmsg[CMLEN];                       /* contains error messages  */

FILE *fp;                            /* file pointer */

/*************************************************************************/

  memset(temp,0,CMLEN);                       /* setting temp to null */

/* Changing the format of the path from
  LAS syntax to host name syntax
  -------------------------------------*/
  status = c_cmhost(name,hname);
  if (status != E_SUCC)
    c_errmsg("Error trying to retrieve host name","cmdel-nohost",&fatal);

/* Checking if the directory exists
  ---------------------------------*/
  if ((fp = (fopen(hname,"r"))) == NULL)
  {
    sprintf(errmsg,"The directory %s does NOT exist",name);
    c_errmsg(errmsg,"cmdel-notexist",&fatal);
  }

  fclose(fp);

/* Finding all of the sub-directories in the user
  specified directory
  -----------------------------------------------*/
  sprintf(sysstr,"find %s -type d -print",hname);
  fp = popen(sysstr,"r");
  while (fgets(buffer[totaldir],BUFSIZ,fp))
  {
    /* eliminate the newline character at end of buffer */
    int len = strlen(buffer[totaldir]);
    if (buffer[totaldir][len-1] == '\n')
        buffer[totaldir][len-1] = '\0';
    totaldir++;
  }


/* Finding the number of sub-directories in each 
  sub-directory
  ----------------------------------------------*/
  bufnum[0] = how_many(buffer[0],'/');
  for (j = 1; j < totaldir; j++)
    bufnum[j] = how_many(buffer[j],'/');


/* Finding the lowest sub-directory
   --------------------------------*/
  if (totaldir > 1)
  {
    index = 1;
    subnum = bufnum[index];
    j = 2;
    while (j < totaldir)
    {
      if (bufnum[j] > subnum)
      {
        index = j;
        subnum = bufnum[j];
      }
      j++;
    }
  }

  strcpy(name,buffer[index]);
  strcat(name,"/*");

} /* end recurdel */
/*************************************************************************/

/* Counts the number of subdirectories in a path by counting
   the number of /
------------------------------------------------------------*/

long how_many(char *s,long c)
{
  long n = 0;
  while (s)
  {
    s = strchr(s,(int)c);
    if (s) n++,s++;
  }
  return n;
}
