/*******************************************************************************
NAME                            VALIDATE                            

PURPOSE    
  Validate checks the directory of file specification for validity

FILE REFERENCES                 
  --
   
PROGRAM HISTORY
Version  Date   Author         Code/Contr  Change Request
-------  -----  -------------  ----------  ---------------
  1.0    11/86  C.Greenhagen   CSB         New Development
  5.0    08/88  B.Davis        CSB         LAS 5.0 conversion
  5.1    10/91  J.Fenno        EDC         Modified support routines and ran
                                           through Saber-C.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   must be run under TAE.

PROJECT                LAS                              

ALGORITHM (FLOW)
  For Directories:                                              
     Directory must be specified,                                       
     File must not be specified,                                
     Wildcards are allowed only in the lowest level directory
                                                        
  For Files:                                    
     Wildcards are allowed only in the filename and extension   
                                                        
     If no extension is specified, image and associated files   
  is implied. In this case, imgset is set to 1,         
  and name will be set to "filename;" before returning.
                                                
*******************************************************************************/

#include <stdio.h>
#include <string.h>

#ifdef unix
#include <sys/file.h>
#endif

#include "las.h"

#include "protos.h"


int validate
(
    char *name,                                 /* object name              */
    char *type,                                 /* "dir" or "file"          */
    long *imgset                                /* img & assoc? (1=yes)     */
)
{

long len;                               /* work variables               */
long one = 1;                           /* value of one for errmsg      */

char *p,*q;                             /* temporary pointer            */
char temp[CMLEN];                       /* temporary string             */

/************************************************************************/

*imgset = 0;
len = (long) strlen (name);

if (strcmp (type,"dir") == 0)                           /* directory    */
{
  /* Wildcard only allowed in last defined directory.
   --------------------------------------------------*/
  strcpy(temp, name);
  temp[len-1] = '\0';
  len--;
  p = strrchr (temp, '/');
  if (p != NULL)
    if (strcspn(name,"*") < (len-strlen(p)))            /* illegal      */
    {                                                   /* wildcard     */
      c_errmsg ("Specification error","cmdel-name",&one);
      return (E_FAIL);
    }
}
    
else if (strcmp (type,"file") == 0)             /* file                 */
{
  if (name[len-1] == '/')    /* If slash is last character in directory... */
  {
    c_errmsg ("Specification error","cmdel-name",&one);
    return (E_FAIL);
  }

  /* Make sure that any existing wildcard occurs only in the filename, not
     in a directory.
     ---------------------------------------------------------------------*/
  p = strrchr(name, '/');
  q = strrchr(name, '*');
  if ((p != NULL) && (q != NULL) && (p > q))
  {
    c_errmsg ("Specification error","cmdel-name",&one);
      return (E_FAIL);
  }

  if ((name[len-1] != '*') || (name[len-2] != '/'))
  {
    if (p == NULL)  /* If no directories are part of the name, just point */
      p = name;     /* p at the name                                      */
    if (strcspn(p, ".") >= strlen(p)-1)         /* look for extension */ 
    {                                           /* if none found, */ 
      *imgset = 1;                              /* image & assoc. files */
      if (strrchr(p, '.') == NULL)
        strcat(p, ".");
    }
  }
}
return (E_SUCC);
}
