/*******************************************************************************
NAME:    CMLAS

PURPOSE: Convert a LAS name to a host file name

VERSION  AUTHOR           DATE   REASON
-------  ---------------  -----  -----------------------------------------------
  2.0    J.Fenno          07/92  Development
  2.1    J.Fenno          11/92  Replaced call to c_complt with z_exit
  2.2    D.Lloyd          08/98  Added call to c_imgprs() to remove LAS window
                                 specification from LAS file name.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS/LAS

ALGORITHM REFERENCES:
*******************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "util.h"

main()
{
  long fatal = -1;              /* Fatal error code                        */
  long nonfatal = 1;            /* Non-fatal error code                    */
  long status;                  /* Function return status                  */
  long stdlun;                  /* Standard output                         */

  char hostname[CMLEN];         /* Host file name                          */
  char lasname[CMLEN];          /* LAS file name                           */
  char *str_array[1];           /* Array to send host name to q_string     */
  char caseflg[50];             /* Case sensitive flag (YES/NO)            */

  struct PARBLK vblock;         /* TAE parameter block                     */
  struct VARIABLE *val;         /* Pointer to TAE variable in vblock       */

  /* Variables added for call to c_imgprs
  ---------------------------------------*/
  struct PRS_LST p_list[1];
  long ximages;

  /* Initialize LAS
  -----------------*/
  c_inlas(&vblock, &stdlun);

  /* Get the LAS file name
  ------------------------*/
  val = p_find(&vblock, "INFILE");
  strcpy(lasname, SVAL(*val,0));

  /* Get the CASEFLAG parameter
  -----------------------------*/
  val = p_find(&vblock, "CASEFLAG");
  strcpy(caseflg,SVAL(*val,0));

  /* Parse out the window specification if present
  ------------------------------------------------*/
  if (c_imgprs(lasname,p_list,&ximages) != E_SUCC )
    return E_FAIL;

  /* Get the host file name
  -------------------------*/
  status = c_cmhost(p_list->name, hostname);
  if (status != E_SUCC)
    c_errmsg("Fatal error encountered", "cmhost-fatal", &fatal);

  /* Write the host file name to the TAE parameter block
  ------------------------------------------------------*/
  str_array[0] = hostname;
  if (q_string(&vblock, "OUTFILE", 1, str_array, P_UPDATE) != SUCCESS)
    {
    c_errmsg("Error writing host name to vblock", "cmhost-qstring", &nonfatal);
    c_errmsg("Fatal error encountered", "cmhost-fatal", &fatal);
    }


  /* Send the parameter block to TAE
  ----------------------------------*/
  if (q_out(&vblock) != SUCCESS)
    {
    c_errmsg("Error returning host file name", "cmhost-vblock", &nonfatal);
    c_errmsg("Fatal error encountered", "cmhost-fatal", &fatal);
    }


  /* Display a completion message
  -------------------------------*/
  z_exit(1, "CMHOST completed successfully");

  return E_SUCC;  /* Return to satisfy compiler. */
}
