/*****************************************************************************
FUNCTION:  brief_attribute

PURPOSE:   Selects file names and formats output according to brief option.

PROGRAM HISTORY:
Version  Date   Author          Change Request
-------  -----  -------------   --------------
  ???    ??/??  ???????         Original development
  5.3    10/91  J.Fenno         Added header and run through Saber-C.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
           brief_attribute must be run under TAE.

PROJECT                 LAS

ALGORITHM REFERENCES    none
*****************************************************************************/

#include <stdio.h>
#include <sys/file.h>
#include "las.h"
#include "protos.h"

#define totfil curent           /* processing message counter               */

#define FIRST_IMG  1          /* flags for breaking printing of file groups */
#define SAME_ASSOC 2          /* with blank lines                           */
#define LAST_ASSOC 3
#define NON_IMG    4
#define HEADER     5

int brief_attrib
(

    char *tmpnam,            /* pointer to tmp file name passed to fileutil */
    char *filename,          /* pointer to file name                        */
    long nfiles,             /* number of files for current dircetory       */
    long start,              /* counter for number of files                 */
    FILE *fp,                /* file pointer for temp file sent to fileutil */
    char (*file_array)[CMLEN], /* pointer to array for input file list      */
    long *assoc,             /* pointer to flag to include assoc files      */
    char (*assoc_array)[CMLEN], /* pointer to array for files assoc w img   */
    long *gbl_nfiles,        /* pointer to counter for parameter nfiles     */
    long *totfil_set,        /* flag for adjusting by 1 cause curent init 1 */
    long *prev_file          /* pointer to flg for format of previous file  */
)

{

long header_done = FALSE;    /* flag signifying headers have been written   */
long i;                      /* loop counter                                */
long imgfl_found;            /* flag,img file found in array of assoc files */
long is_image;               /* flag for image file or not                  */
long is_imgfl;               /* flag for image file or not                  */
long j;                      /* loop counter                                */
long nassoc;                 /* number of associated files for an image     */
long second_header = FALSE;  /* flag for writing of second header           */
short status;                /* return status of calls                      */

char dir[CMLEN];             /* directory of current file                   */
char hostnam[CMLEN];         /* host file name                              */
char hostdsc[CMLEN];         /* host file name                              */
char pfile[CMLEN];           /* host file name for print file               */
char *ptr;                   /* pointer to step through name by characters  */
char save_root[CMLEN];       /* prev root name to check against current     */
char sdtype[20];             /* data type of image                          */
char snbands[20];            /* number of bands of image                    */
char ssize[20];              /* size of image                               */
char root[CMLEN];            /* name minus extension                        */
char wd[CMLEN];              /* current working directory name              */

extern long curent;          /* processing message file counter             */

                             /* macro to print with breif attributes        */
#define BRIEF_PRINT()\
{\
ptr = (char *)strrchr(hostdsc, '/');\
ptr++;\
if (is_image)\
   {\
   if ((*prev_file == LAST_ASSOC) || (*prev_file == NON_IMG))\
      fprintf(fp,"\n");\
   }\
else\
   {\
   if (*prev_file == LAST_ASSOC)\
      fprintf(fp,"\n");\
   }\
strcpy(pfile, ptr);\
fprintf(fp,"%-39.39s%14s%9s%10s\n", pfile,ssize,snbands,sdtype);\
}

if ((start == 1) && (!header_done))
        {
        strcpy(dir, filename);
        ptr = (char *)strrchr(dir, '/');
        if (ptr != 0)
                {
                ptr++;
                *ptr = '\0';
                }
        else
                {
                getcwd(wd,CMLEN);
                status = cmtaedir(wd, dir);
                if (status != E_SUCC)
                   return(E_FAIL);
                ptr = (char *)strrchr(dir, '/');
                if (ptr != 0)
                        {
                        ptr++;
                        *ptr = '\0';
                        }
                }
        fprintf(fp,"\n%s\n",dir);
        header_done = TRUE; 
        }
if (*gbl_nfiles == 0)
        fprintf(fp,"\n### NO FILES MATCHED ###\n");

for(i = 0; i < nfiles; i++)
        {
        if (*totfil_set)           /* increment curent                      */
           totfil++;
        else
           *totfil_set = TRUE;

        strcpy(hostdsc,file_array[i]);
                                   /* compare to prev img root name         */
        strcpy(root, hostdsc);
        if ((ptr = strrchr(root, '.')) == NULL)
           return E_FAIL;
        *ptr = '\0';
        if ((strcmp(root, save_root) != 0) || (!(*assoc)))
                {
                strcpy(save_root, root);

                                   /* get the directory name for first file */
                if (i == 0 && start == 1 && !second_header)
                        {
                        second_header = TRUE;
                        fprintf(fp,"\nFile Name                                    NL,NS      Nbands  Datatype\n---------                                    -----      ------  --------\n");
                        *prev_file = HEADER;
                        }

                strcpy(ssize , " ");
                strcpy(snbands , " ");
                strcpy(sdtype , " ");
                                                /* check if it is an image,   */
                                                /* set flags for print format */
                is_image = isimage(hostdsc);
                if (is_image)
                        {
                        status = cmlist_get_assoc(tmpnam,hostdsc,assoc_array,&nassoc);
                        if (status != E_SUCC)
                           return(E_FAIL);
                        j = 0;
                        imgfl_found = FALSE;
                        is_imgfl = FALSE;
                        for (j = 0;j < nassoc; j++)
                           {
                           strcpy(hostnam,assoc_array[j]);
                           /* if image file has not been found in assoc_array */
                           if (!imgfl_found)
                              is_imgfl = isimgfl(hostnam);
                           if (is_imgfl)
                              {
                              getimage(hostnam, ssize, snbands, sdtype);
                              if (status != E_SUCC)
                                 return(E_FAIL);
                              strcpy(hostdsc, hostnam);
                              BRIEF_PRINT()               /* print image file */
                              strcpy(ssize , " ");
                              strcpy(snbands , " ");
                              strcpy(sdtype , " ");
                              imgfl_found = TRUE;
                              is_imgfl = FALSE;
                              *prev_file = FIRST_IMG;
                              }
                           else
                              {
                              strcpy(hostdsc, hostnam);
                              BRIEF_PRINT()          /* print associated file */
                              *prev_file = SAME_ASSOC;
                              }
                           }
                        *prev_file = LAST_ASSOC;
                        }
                else
                        {
                        BRIEF_PRINT()      /* print non image-associated file */
                        *prev_file = NON_IMG;
                        }
                }
        }
return(E_SUCC);
}
