/*******************************************************************************

PROGRAM:   CMLIST

PURPOSE:   Lists catalog files and their attributes if specified.

FILE REFERENCES:
  If printer output is requested, LIST creates an intermediate disk 
  file (list_<time stamp>.pr) to hold the output listing.  After 
  LIST finishes, this file is sent to the line printer and then is 
  automatically deleted. 
   
FATAL ERROR CONDITIONS: At the time of this release, the tape options are not
                        available.  Any input parameters concerning tape files
                        that are not defaulted to disk values result in a fatal
                        error.

PROGRAM HISTORY:
Version  Date   Author        Code/Contr  Change Request
-------  -----  ------------  ----------  --------------
  1.0    10/86  T.Butzer      CSB         New Development
  5.0    07/88  B.Davis       CSB         LAS 5.0 conversion, add documentation,
					  fixed bugs, conform to LAS 5.0
					  catalog manager
  5.1    10/89  P.Pease       GSFC        Changed order of includes to work on
					  IBM-RT.
  5.2    11/89  P.Pease       GSFC        Changed call to c_inlas instead of
					  inlas.
  5.3    10/91  J.Fenno       EDC         Added header and ran through Saber-C.
  5.4	 07/92  R.Kruse       EDC/CSB	  Remove FTYPE and FILEOPT parameters.
					  CM will not handle offline images.
  5.5	 09/92  T.Mittan      EDC/CSB	  Made curent extern to eliminate
					  warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   CMLIST must be run under TAE 

PROJECT:   	       LAS		                

*******************************************************************************/

#include <stdio.h>

#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif

#include "las.h"
#include "util.h"
#include "protos.h"

extern long curent;
#define totfil curent

long assoc; 			/* flag for associated files to be included   */
long curlevel;			/* counter for parameter nlevels	      */
long gbl_nfiles;		/* counter for parameter nfiles		      */
long header_done;		/* flag signifying headers have been written  */
long maxlevel;			/* parameter number of levels of directories  */
long prev_file = TRUE;		/* flg for format if prev file was assoc w img*/
long second_header;		/* flag for writing of second header	      */
long totfil_set;		/* flag for adjusting by 1 cause curent init 1*/

FILE *fp;			/* file pointer for temp file sent to fileutil*/

char assoc_array[MAXFILES][CMLEN]; /* array for files associated with an image*/
char file_array[MAXFILES][CMLEN];  /* array for all files in the input list   */

void main
(void)
{

struct	PARBLK	vblock;		/* parameter block			      */

struct VARIABLE *name;		/* structure for file names	      	      */
struct VARIABLE *in;		/* structure for parameter names	      */
struct VARIABLE *val;	      	/* pointer to TAE variable within v-block     */
				
long fatal = 0;			/* value of zero			      */
long file;			/* flag for output to file		      */
long i;				/* image loop counter			      */
long level;			/* number of directories to levels to search  */
long lnpr;			/* flag for output to lnpr		      */
long msgtm;			/* for tae global msgtim		      */
long nfiles;			/* total number of input files from vblock    */
long numout;			/* number of output files created for complt  */
long one = 1;			/* value of one			 	      */
long status;			/* return status of support calls	      */
long stdlun;			/* logical unit number-LUN for standard output*/
long term;			/* flag for output to terminal		      */
long two = 2;			/* value of two			              */
long zero = 0;			/* value of zero			      */

char attrib[10];		/*flag for what level of attrib info to disply*/
/* char disk[10];		   flag for displaying disk files  	      */
char ent[5];			/* name of entity for processing msg file     */
char filename[CMLEN];		/* current name from the list to be processed */
char hfilename[CMLEN];	/* current (host)name from the list to be processed */
char filnm[1][CMLEN];		/* program name for passing to supports       */
char prognm[7];			/* program name - cmlist		      */
/* char tape[10];		   parameter for searching for files on tape  */
char tmpfil[1][CMLEN];		/* temporary file name for fileutil	      */
char verb[10];			/* verb for processing msg		      */

/******************************************************************************/

c_inlas(&vblock, &stdlun);
strcpy(prognm,"CMLIST");
strcpy(verb,"Processed");
strcpy(ent,"file");

/* get print option, make a file name if necessary, open temp file
------------------------------------------------------------------*/
strcpy(tmpfil[0],"cmlist");
status = c_setprt(&vblock,&term,&file,&lnpr,tmpfil[0],filnm[0]);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","cmlist-fatal",&fatal);
if (file == 1)
   {
   numout = 1;
   }
else
   {
   strcpy(filnm[0]," ");
   numout = 0;
   }

fp = fopen(tmpfil[0],"w");	
if (fp == NULL)
   {
   c_errmsg("Error opening temporary print file","cmlist-open",&one);
   c_errmsg("Fatal error encountered","cmlist-fatal",&fatal);
   }

/* get parameters
-----------------*/
in = p_find(&vblock, "nlevels");
level = IVAL(*in, 0);

in = p_find(&vblock, "listopt");
strcpy(attrib, SVAL(*in,0));

/******************************************************************************
  This section of code is nolonger used, because the two parameters FTYPE and
  FILEOPT were never used and are now removed from the pdf.
 ******************************************************************************/
  /*******************************
     in = p_find(&vblock, "fileopt");
     strcpy(disk, SVAL(*in,0));
     if (strcmp(disk,"ONLINE") != 0)
	{
      c_errmsg("The parameter FILEOPT must contain ONLINE.","cmlist-tape",&one);
      c_errmsg("Tape options not implemented at this time.","cmlist-tape",&one);
	fclose(fp);
	c_clean(&vblock,prognm,tmpfil,&one);
	}

     in = p_find(&vblock, "ftype");
     strcpy(tape, SVAL(*in,0));
     if (strcmp(tape,"ALL") != 0)
	{
	c_errmsg("The parameter FTYPE must contain ALL.","cmlist-tape",&one);
      c_errmsg("Tape options not implemented at this time.","cmlist-tape",&one);
	fclose(fp);
	c_clean(&vblock,prognm,tmpfil,&one);
	}
  *******************************/

name = p_find(&vblock, "infile");
nfiles = name->v_count;

val = p_find(&vblock,"$msgtim");
msgtm = IVAL(*val,0);

/* process files or directories of files
   one at a time, depending on filename
----------------------------------------*/
maxlevel = level;
totfil = 1;
totfil_set = FALSE;
c_inittm(prognm,verb,ent," "," ",&nfiles,&zero,&msgtm,&two);
c_startm();
for (i = 0; i < nfiles; i++)
 {
 curlevel = 0;
 strcpy(filename, SVAL(*name,i));
 c_cmhost(filename, hfilename);
 status = processdisk(tmpfil[0], hfilename, attrib, fp, file_array,
		      &assoc, assoc_array, &curlevel, &maxlevel,
		      &gbl_nfiles, &prev_file, &totfil_set);
 if (status != E_SUCC)
    {
    fclose(fp);
    c_clean(&vblock, prognm, tmpfil, &one);
    }
 }
c_stoptm();

/* close and complete
---------------------*/
fclose(fp);	
c_complt(prognm, filnm, &numout);	
}
