/*****************************************************************************
FUNCTION:  med_attrib

PURPOSE:   Selects file names and formats output according to medium option.

PROGRAM HISTORY:
Version  Date   Author         Change Request
-------  -----  -------------  --------------
  ???    ??/??  ???????        Original development
  5.3    10/91  J.Fenno        Added header and run through Saber-C.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
           med_attrib must be run under TAE.

PROJECT                 LAS

ALGORITHM REFERENCES    none
*****************************************************************************/

#include <stdio.h>
#include <sys/file.h>
#include "las.h"
#include "protos.h" 


#define totfil curent         /* processing message counter                 */

#define FIRST_IMG  1          /* flags for breaking printing of file groups */
#define SAME_ASSOC 2          /* with blank lines                           */
#define LAST_ASSOC 3
#define NON_IMG    4
#define HEADER     5
int med_attrib
(

    char *tmpnam,             /* pointer to tmp name passed to fileutil     */
    char *filename,           /* pointer to file name                       */
    long nfiles,              /* number of files for current dircetory      */
    long start,               /* counter for number of files                */
    FILE *fp,                 /* file pointer for temp file sent to fileutil*/
    char (*file_array)[CMLEN], /* pointer to array of input file list       */
    long *assoc,              /* flag for associated files to be included   */
    char (*assoc_array)[CMLEN], /* pointer to array of assoc files          */
    long *gbl_nfiles,         /* pointer to counter for parameter nfiles    */
    long *totfil_set,         /* flag for adjusting by 1 cause curent init 1*/
    long *prev_file           /* flg for format if prev file was assoc w img*/
)
{



long header_done = FALSE;     /* flag signifying headers have been written  */
long i;                       /* loop counter                               */
long imgfl_found;             /* flag,img file found in array of assoc files*/
long is_image;                /* flag for image file root name or not       */
long is_imgfl = FALSE;        /* flag for image file or not                 */
long j;                       /* loop counter                               */
long lngth;                   /* length of names                            */
long nassoc;                  /* number of associated files for an image    */
long second_header = FALSE;   /* flag for writing of second header          */
short status;                 /* return status of calls                     */

char credate[100];
char dir[CMLEN];              /* directory of current file                  */
char hostdsc[CMLEN];          /* host file name                             */
char hostnam[CMLEN];          /* host file name                             */
char pfile[CMLEN];            /* host file name for print file              */
char *ptr;                    /* pointer to step through name by characters */
char save_root[CMLEN];        /* prev root name to check against current    */
char sdtype[20];              /* data type of image                         */
char snbands[20];             /* number of bands of image                   */
char ssize[20];               /* size of image                              */
char root[CMLEN];             /* name minus extension                       */
char wd[CMLEN];               /* current working directory name             */

extern long curent;           /* processing message file counter            */


                              /* macro to print non and image file names    */
#define MED_PRINT()\
{\
ptr = (char *)strrchr(hostdsc, '/');\
ptr++;\
strcpy(pfile, ptr);\
if (is_image)\
   {\
   if ((*prev_file == LAST_ASSOC) || (*prev_file == NON_IMG))\
      fprintf(fp,"\n");\
   }\
else\
   {\
   if (*prev_file == LAST_ASSOC)\
      fprintf(fp,"\n");\
   }\
if (is_imgfl)\
        {\
        fprintf(fp,"%-39.39s\n", pfile);\
        fprintf(fp," %-16.16s :  %s\n","NL,NS",ssize);\
        fprintf(fp," %-16.16s :  %s\n","Nbands",snbands);\
        fprintf(fp," %-16.16s :  %s\n","Data Type",sdtype);\
        fprintf(fp," %-16.16s :  %s\n","Creation Date",credate);\
        }\
else\
        {\
        fprintf(fp,"%-39.39s\n", pfile);\
        fprintf(fp," %-16.16s :  %s\n","Creation Date",credate);\
        }\
}

if ((start == 1) && (!header_done))
        {
        strcpy(dir, filename);
        ptr = (char *)strrchr(dir, '/');
        if (ptr != 0)
                {
                ptr++;
                *ptr = '\0';
                }
        else
                {
                getcwd(wd,CMLEN);
                status = cmtaedir(wd, dir);
                if (status != E_SUCC)
                   return(E_FAIL);
                ptr = (char *)strrchr(dir, '/');
                if (ptr != 0)
                        {
                        ptr++;
                        *ptr = '\0';
                        }
                }
                fprintf(fp,"\n%s\n",dir);
                header_done = TRUE; 
                
        }
if (*gbl_nfiles == 0)
        fprintf(fp,"\n### NO FILES MATCHED ###\n");

for(i = 0; i < nfiles; i++)
        {
        if (*totfil_set)                      /* increment curent           */
           totfil++;
        else
           *totfil_set = TRUE;

        strcpy(hostdsc,file_array[i]);

                                           /* compare to prev img root name */
        strcpy(root, hostdsc);
        if ((ptr = strrchr(root, '.')) != NULL)
            *(ptr+1) = '\0';
        if ((strcmp(root,save_root) != 0) || (!(*assoc)))
                {
                strcpy(save_root,root);
                                   /* get the directory name for first file */
                if (i == 0 && start == 1 && !second_header)
                        {
                        fprintf(fp,"\nFile Name\n---------\n");
                        second_header = TRUE;
                        *prev_file = HEADER;
                        }

                strcpy(ssize , " ");
                strcpy(snbands , " ");
                strcpy(sdtype , " ");
                                              /* check if it is an image,   */
                                              /* set flags for print format */
                is_image = isimage(hostdsc);
                if (is_image)
                        {
                        status = cmlist_get_assoc(tmpnam,hostdsc,assoc_array,
                                                  &nassoc);
                        if (status != E_SUCC)
                           return(E_FAIL);
                        j = 0;
                        imgfl_found = FALSE;
                        is_imgfl = FALSE;
                        for (j = 0;j < nassoc; j++)
                           {
                           strcpy(hostnam,assoc_array[j]);
                         /* if image file has not been found in assoc_array */
                           if (!imgfl_found)
                              is_imgfl = isimgfl(hostnam);
                           if (is_imgfl)
                              {
                              status = getimage(hostnam,ssize,snbands,sdtype);
                              if (status != E_SUCC)
                                 return(E_FAIL);
                              strcpy(hostdsc, hostnam);
                              getcredate(hostnam, credate);
                              MED_PRINT()               /* print image file */
                              imgfl_found = TRUE;
                              is_imgfl = FALSE;
                              *prev_file = FIRST_IMG;
                              }
                           else
                              {
                              strcpy(hostdsc, hostnam);
                              getcredate(hostnam, credate);
                              MED_PRINT()          /* print associated file */
                              *prev_file = SAME_ASSOC;
                              }
                           }
                        *prev_file = LAST_ASSOC;
                        }
                else
                        {
                        getcredate(hostdsc, credate);
                        MED_PRINT()   /* print non image or associated file */
                        *prev_file = NON_IMG;
                        }
                }
        }
return(E_SUCC);
}
