/*******************************************************************************
FUNCTION:  proc_file

PURPOSE:   Processes one file at a time in the case of a directory
	   specification.  Proc_file sends an array of file names 
	   from a directory list to the appropriate function that
	   formats and writes file names and/or information.

PROGRAM HISTORY:
Version  Date   Author          Change Request
-------  -----  -------------   --------------
  ???    ??/??  ???????         Original development
  5.3    10/91  J.Fenno         Removal of unnecessary variables

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>

#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif

#include "las.h"
#include "protos.h"

int proc_file
(

    char *tmpfil,          /* ptr to temp file containing file names          */
    char *filename,        /* ptr to current file name being processed        */
    char *attrib,          /* ptr to attribute option of current file         */
    FILE *fp,              /* file pointer for temp file sent to fileutil */
    char file_array[][CMLEN],  /* ptr to array for input file list           */
    long *assoc,               /* ptr to flag to include associated files     */
    char assoc_array[][CMLEN], /* array of associated file names              */
    long *curlevel,            /* ptr to counter for parameter nlevels        */
    long *maxlevel,            /* ptr to number of directory levels           */
    long *gbl_nfiles,          /* ptr to counter for parameter nfiles         */
    long *prev_file,       /* ptr to flg for format if prev file was assoc,img*/
    long *totfil_set       /* flag for adjusting by 1 cause curent init 1     */
)

{

#define MAXDIRS 10

int ndirs;              /* number of directories to loop through */

long  j;			/* loop counter				      */
long  nfiles;			/* number of files to loop through            */
long  start;			/* counter for number of files		      */
short status;			/* return status of calls		      */

char  directories[MAXDIRS][CMLEN];/* array of directory names		      */
char  hostname[CMLEN];		/* host name including whole path and file    */
char  filepart[CMLEN];		/* just the file name        		      */
char  *ptr;			/* pointer to step through name by characters */

(*curlevel)++;

strcpy(hostname,filename);

ptr = (char *)strrchr(filename, '/'); 
if (ptr != 0)
   {	
   ptr++;
   strcpy(filepart, ptr);
   }
else
   strcpy(filepart,filename);

*gbl_nfiles = 0;

/* List all of the files that match the hostname
------------------------------------------------*/
start = 1;
do
   {
   status = dirlist(tmpfil, hostname, file_array, &nfiles, MAXFILES, start);
   if (status != E_SUCC)
      return(E_FAIL);

   *gbl_nfiles += nfiles;
   if (strcmp(attrib, "NO") == 0)
      {
      if (*assoc)
	 status = no_attrib(tmpfil, hostname, nfiles, start, fp, file_array,
			    assoc_array, gbl_nfiles, totfil_set, prev_file);
      else
	 status = no_attrib_all(hostname, nfiles, start, fp, file_array,
				gbl_nfiles, totfil_set);
      }
   else if (strcmp(attrib, "BRIEF") == 0)
      status = brief_attrib(tmpfil, hostname, nfiles, start, fp, file_array,
			    assoc, assoc_array, gbl_nfiles, totfil_set, 
			    prev_file);
   else if (strcmp(attrib, "MED") == 0)
      status = med_attrib(tmpfil, hostname, nfiles, start, fp, file_array,
			  assoc, assoc_array, gbl_nfiles, totfil_set, 
			  prev_file);
   else if (strcmp(attrib, "LONG") == 0)
      status = long_attrib(tmpfil, hostname, nfiles, start, fp, file_array,
			   assoc, assoc_array, gbl_nfiles, totfil_set,
			   prev_file);
   else
      status = E_FAIL;

   if (status != E_SUCC)
      return(E_FAIL);
   start = start + nfiles;
   } while (nfiles == MAXFILES);

/* Process all subdirectories. Only 
   process MAXDIRS directories at a time.
-----------------------------------------*/
start = 1;
do
   {
   if ( (*curlevel) < (*maxlevel) )
      {
      status = getdirs(hostname, directories,&ndirs, MAXDIRS, start);
      if (status != E_SUCC)
	 return(E_FAIL);
      if (ndirs == 0)
	 break;
      }
   else
      break;
   start = start + ndirs;
   for (j = 0; j < ndirs; j++)
      {
      status = cmtaedir(directories[j],hostname);
      if (status != E_SUCC)
	 return(E_FAIL);
      strcat(hostname, filepart);

      status = proc_file(tmpfil,    hostname,  attrib,   fp,       file_array,
			 assoc,   assoc_array, curlevel, maxlevel, gbl_nfiles,
			 prev_file, totfil_set);
      if (status != E_SUCC)
	 return(E_FAIL);
      }
   } while (ndirs == MAXDIRS);
(*curlevel)--;
return(E_SUCC);
}
