/*******************************************************************************
FUNCTION:  processdisk

PURPOSE:   Processdisk accepts one "name" from the user-specified parameter
           list ("name" may be an alias name referencing a list), extracts
           and processes one filename or directory name at a time from "name".

PROGRAM HISTORY:
Version  Date   Author          Change Request
-------  -----  -------------   --------------
  ???    ??/??  ???????         Original development
  5.3    10/91  J.Fenno         Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>

#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif

#include "las.h"
#include "alias.h"
#include "protos.h"

int processdisk
(

    char *tmpfil,            /* pointer to temp file name containing names    */
    char *name,              /* pointer to current name being processed       */
    char *attrib,            /* pointer to flag for what info to display      */
    FILE *fp,                /* file pointer for temp file sent to fileutil */
    char (*file_array)[CMLEN],/* pointer to array of files in input list    */
    long *assoc,              /* pointer to flag for assoc files included     */
    char (*assoc_array)[CMLEN],  /* pointer to array of assoc files          */
    long *curlevel,              /* pointer to counter for parameter nlevels */
    long *maxlevel,              /* pointer to parameter number of dir levels */
    long *gbl_nfiles,            /* pointer to counter for parameter nfiles   */
    long *prev_file,         /* flg for format if prev file was assoc w img   */
    long *totfil_set         /* flag for adjusting by 1 cause curent init 1   */
)

{
short first = 1;             /* first time?                                   */
short status;                /* return status of calls                        */

char *filename;              /* pointer to current file name to be processed  */
char *ptr;                   /* pointer for stepping through filename         */
char tstring[CMLEN];         /* expanded string                               */

/* expand aliases, if any
-------------------------*/
status = c_getalias (name,tstring);
if (status != E_SUCC)
   return(E_FAIL);

while (1)
    {
    if (first)
	{
	filename = gettoken (tstring,", ");	/* get name from string	*/
	first = 0;
	}
    else
	filename = gettoken (NULL,", ");	/* get name from string	*/
    if (filename == NULL)
	break;                                  /* no names remaining   */

    *assoc = FALSE;
    ptr = (char *)strrchr(filename, HOST_EXT);

    /* no extension implies image and associated files
    --------------------------------------------------*/
    if (ptr == 0)
	{
	strcat(filename,".img");
	*assoc = TRUE;
	}
#ifdef UNIX
    /* on unix *;* mapped to * to get all files.  Note that if
       an * is used for the file name it has .img concatenated
       on in the if statement so that you look for *.img files.
    -----------------------------------------------------------*/
    else
	{
	if ((*(ptr-1) == '*') && (*(ptr+1) == '*'))
		*ptr = '\0';
        }
#endif

    status = proc_file(tmpfil, filename, attrib, fp, file_array, assoc,
                       assoc_array, curlevel, maxlevel, gbl_nfiles, prev_file,
                       totfil_set);
    if (status != E_SUCC)
       return(E_FAIL);
    }

return(E_SUCC);
}
