/*****************************************************************************

PROGRAM:        CMMKDIR

PURPOSE:        Creates a new directory in the user's directory tree.

                If the directory name begins with the username,
                the full pathname must be specified.

                If the directory name begins with ".", the new directory
                will be a subdirectory of the current working directory.

                If the directory name begins with "-", the new directory
                will be a subdirectory of the current parent directory.

FATAL ERROR CONDITIONS:
                Error creating directory: <directory name>
                Specification error
                Specified alias name not found
                Unable to open alias file


PROGRAM HISTORY:

Version    Date     Author           Code/Contr    Change Request
-------    -----    -------------    ----------    ---------------
  1.0      11/86    C. Greenhagen     EDC/CSB      New Development
  5.o       9/88    B. Davis          CSB          Conversion to 5.0
  5.1       9/89    P. Pease          GSFC         Moved include of all system
                                                 include files before las ones
  5.2       1/91    D. Etrheim        EDC/CSB      Added device specification
                                                   for VMS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under TAE 

ALGORITHM(flow)

*****************************************************************************/

#include <stdio.h>
#include <sys/file.h>
#include <sys/stat.h>
#include "las.h"
#include "util.h"
#include "alias.h"


extern int errno;
#ifndef linux
extern char *sys_errlist[];
#endif


void main
(void)
{
struct PARBLK vblock;                   /* TAE parameter block          */

struct VARIABLE *in;                    /* pointer to variable          */

long nfiles = 0;                        /* # files for complt call      */
long one = 1;                           /* value of one                 */
long status;                            /* status from external calls   */
long stdlun;                            /* logical unit # of std output */
long fatal = 0;                         /* value of zero                */

char hname[255];                        /* host name of new directory   */
char msgtxt[CMLEN];                     /* message text                 */
char name[CMLEN];                       /* TAE name of new directory    */
char outfiles[1][CMLEN];                /* Name of output file          */
char pname[CMLEN];                      /* Program name                 */
char tstring[CMLEN];                    /* temporary string             */

long mode=511;                          /* mode for mkdir (2**9 -1)     */

/************************************************************************/

c_inlas (&vblock, &stdlun);                     /* Initialize           */

strcpy(pname,"CMMKDIR");                        /* build program name   */

in = p_find (&vblock,"INDIR");                  /* get directory name   */ 
strcpy (name, SVAL (*in,0));
status = c_getalias (name,tstring);             /* expand aliases,if any*/
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","cmmkdir-fatal",&fatal);

if (c_cmhost(tstring, hname) != E_SUCC)         /* get host name        */
   c_errmsg ("Fatal error encountered","cmmkdir-fatal",&fatal);

if (mkdir(hname,mode) != 0)
    {
    sprintf (msgtxt,"Error creating directory: %s", hname);
    c_errmsg (msgtxt," ",&one);
    sprintf (msgtxt,"%s",sys_errlist[errno]);
    c_errmsg (msgtxt,"cmmkdir-create",&one);
    c_errmsg ("Fatal error encountered","cmmkdir-fatal",&fatal);
    }
    
nfiles = 1;
strcpy(outfiles[0], hname);
c_complt (pname,outfiles,&nfiles);
}
