/******************************************************************************

PROGRAM:	CMRENAME

PURPOSE:	Renames Cataloged files.

FATAL ERROR CONDITIONS:
		Specification error
		Unable to open alias file
		Specified alias not found: <aliasname>

OTHER ERROR CONDITIONS:
		Output file already exists: filename

PROGRAM HISTORY:

Version  Date   Author         Code/Contr  Change Request
-------  -----  -------------  ----------  ---------------
  1.0    11/86  C.Greenhagen   EDC/CSB     New Development
  5.0    09/88  B.Davis        CSB         Conversion to 5.0
  5.1    03/89  B.Davis        CSB         Increased size of temp file names
					   and extensions in validate.c from 
					   40 to CMLEN
  5.2    03/89  B.Davis        CSB         Passed vblock to processfile for
					   hstout call.
  5.3    09/89  P.Pease        GSFC        Moved all system includes to before
					   any las include.
  5.4    11/90  D.Etrheim      CSB         Update call to processfile.
					   Add option to rename or copy the
					   file.
					   Correct problems renaming the
					   history file.
  5.5    10/91  J.Fenno        EDC         Modified support routines and ran
					   through Saber-C.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE 

ALGORITHM(flow)

		If both the file part and the extension are specified,
		the file is named as specified.

		If the filepart is "*", the new name uses the file part
		of the current name, and the new extension specified.

		If the extension is "*" or unspecified,	the new name
		uses the extension of the current name, and the new
		file part specified.

		If the extension is unspecified, the image and all
		associated files is assumed.

******************************************************************************/


#include <stdio.h>

#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif

#include "las.h"
#include "util.h"
#include "alias.h"
#include "protos.h"

#ifdef gould
#include "diskio.h"
#endif

void main
(void)
{
struct PARBLK vblock;			/* TAE parameter block		*/

struct VARIABLE *in;			/* pointer to variable		*/

long acount;				/* # associated file extensions */
long copyflg;				/* flag if image data pointer is 
					   copied or renamed */
long imageflg;				/* Copy image flag		*/
long imgset;				/* img & assoc files? (t=1)	*/
long i;					/* loop index			*/
long len;				/* string length		*/
long nrenamed = 0;			/* # files renamed		*/
long one = 1;				/* value of one   		*/
long status;				/* return status of support call*/
long stdlun;				/* logical unit # of std output	*/
long verify = 0;			/* verify? (yes=1)		*/
long fatal = 0;				/* value of zero		*/

char aftype[100][CMLEN];		/* List of associated file extensions */
char name[CMLEN];			/* current cm TAE filename	*/
char newname[CMLEN];			/* new cm TAE filename		*/
char tmpnam[1][CMLEN];			/* Array of output file names	*/
char pname[CMLEN];			/* Program name			*/
char tname[CMLEN];			/* name with aliases expanded	*/
char tnewname[CMLEN];			/* newname with aliases expanded*/
char hname[CMLEN];			/* host name with aliases expanded  */
char hnewname[CMLEN];			/*host newname with aliases expanded*/
char vstring[4];			/* verify option string (y/n)	*/
/************************************************************************/

c_inlas (&vblock, &stdlun);			/* Initialize		*/

strcpy(pname,"CMRENAME");			/* build program name	*/

in = p_find (&vblock,"INFILE");			/* get input name	*/
strcpy (name, SVAL (*in,0));

in = p_find (&vblock,"OUTFILE");		/* get new name		*/
strcpy (newname, SVAL (*in,0));

/*
  Get the associated file types to be copied
*/
in = p_find (&vblock,"AFTYPE");
strcpy(tmpnam[0], SVAL(*in,0));

len = (long) strlen(tmpnam[0]);
c_low2up(tmpnam[0],&len);

if (strcmp(tmpnam[0],"NONE") == 0)
    acount = 0;
else
    acount = (*in).v_count;

for (i = 0; i < acount; i++)
    strcpy(aftype[i], SVAL(*in,i));

/*
  Get the flag to copy image
*/
in = p_find (&vblock,"IMAGEFLG");
if (strcmp(SVAL(*in,0),"YES") == 0)
   imageflg = TRUE;
else
   imageflg = FALSE;

/*
  Get the flag to copy or rename the image data 
*/
in = p_find (&vblock,"COPYFLG");
if (strcmp(SVAL(*in,0),"YES") == 0)
   copyflg = TRUE;
else
   copyflg = FALSE;

in = p_find (&vblock,"CONFLG");			/* get verify option	*/
strcpy (vstring, SVAL (*in,0));			/* (YES/NO)		*/
if ((vstring[0]=='Y') || (vstring[0]=='y'))
    verify = 1;

status = c_getalias (name,tname);		/* expand aliases, if	*/
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","cmrename-fatal",&fatal);
status = c_getalias (newname,tnewname);	 	/* any, in parameters	*/
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","cmrename-fatal",&fatal);

if (c_cmhost(tname, hname) != E_SUCC)
   c_errmsg ("Fatal error encountered","cmrename-fatal",&fatal);
if (c_cmhost(tnewname, hnewname) != E_SUCC)
   c_errmsg ("Fatal error encountered","cmrename-fatal",&fatal);
validate (hname,hnewname,&imgset);		/* validate input	*/

processfile (&vblock,"cmrename",hname,hnewname,imgset,imageflg,aftype,&acount,
	     verify,&nrenamed,copyflg);

if (nrenamed == 0)
    c_errmsg ("No files renamed","cmrename-nrenamed",&one);
nrenamed = 0;
strcpy(tmpnam[0]," ");
c_complt (pname,tmpnam,&nrenamed);
}
