/*******************************************************************************

FUNCTION:       VALIDATE

PURPOSE: checks the directory of file specification for validity

FATAL ERROR CONDITIONS: Specification error occurrs when the following checks
                                are not successful

ALGORITHM(flow)
   validate checks the input parameters for validity:                     
                                                                          
        "*" may be used as the filename or as the extension,              
           but not both.                                                  
        "*" may not be used as a part of the filename or                  
           as a part of the extension.                                    
        If INFILE uses "*" as the filename,                                
           OUTFILE must also use "*" as the filename.                     
        If INFILE uses "*" as the extension,                              
           OUTFILE must also use "*" as the extension.                    
        If INFILE leaves the extension unspecified                        
           (implying image and all associated files),                     
           OUTFILE must also leave the extension unspecified.             
        For VMS only, if INFILE uses LIMG as the extension,               
           OUTFILE must also use LIMG as the extension.                   
           In this case, nam will be set to "filename;LIMG__*" and        
           newnam will be set to "filename;*" before returning.           
                                                                           
        If an image set (image and associated files) is specified,        
        nam will be set to "filename;" and newnam will be set to          
        "filename;*" before returning.                                    
                                                                          
*******************************************************************************/


#include <stdio.h>
#include <string.h>

#include <sys/file.h>

#include "las.h"
#include "protos.h"


void validate
(
    char *nam,                          /* current TAE name             */
    char *newnam,                       /* new TAE name                 */
    long *imgset                        /* img and assoc files? (t=1)   */
)
{
long fatal = 0;                         /* value of zero                */
long one = 1;                           /* value of one                 */

char curext[CMLEN];                     /* extension of current name    */
char curfile[CMLEN];                    /* file part of current name    */
char newext[CMLEN];                     /* extension of new name        */
char newfile[CMLEN];                    /* file part of new name        */
char *p;                                /* temporary pointer            */

/************************************************************************/

*imgset = 0;

if ((p = strrchr (nam,'/')) == NULL)            /* isolate file part    */
    strcpy  (curfile,nam);                      /* of INFILE            */
else
    strcpy (curfile,++p);
 
if ((p = strrchr(curfile, '.')) == NULL)
    strcpy (curext,"");                         /* No extension.        */
else
    {
    *p = NULL;
    strcpy(curext, ++p);                        /* Get extension.       */
    }

if ((p = strrchr (newnam,'/')) == NULL)         /* isolate file part    */
    strcpy  (newfile,newnam);                   /* of OUTFILE           */
else
    strcpy (newfile,++p);
 
if ((p = strrchr(newfile, '.')) == NULL)
    strcpy (newext,"");                         /* No extension.        */
else
    {
    *p = NULL;
    strcpy(newext, ++p);                        /* Get extension.       */
    }

/***  Error conditions  ***/

if (((strchr(curfile,'*') != NULL) && (strlen(curfile) != 1)) ||
    ((strchr(curext,'*')  != NULL) && (strlen(curext)  != 1)) ||
    ((strchr(newfile,'*') != NULL) && (strlen(newfile) != 1)) ||
    ((strchr(newext,'*')  != NULL) && (strlen(newext)  != 1)) ||
    ((curfile[0] == '*')   && (newfile[0] != '*'))    ||
    ((curext[0]  == '*')   && (newext[0]  != '*'))    ||
    ((curfile[0] == '*')   && (curext[0]  == '*'))    ||
    ((newfile[0] == '*')   && (newext[0]  == '*'))    ||
    ((strlen(curext) == 0) && (strlen(newext) != 0)))
      {
      c_errmsg ("Specification error","cmrename-name",&one);
      c_errmsg ("Fatal error encountered","cmrename-fatal",&fatal);
      }

if ((strlen(curext) == 0) && (strlen(newext) == 0))
    {
    *imgset = 1;
    if ((p = strrchr (nam,'.')) != NULL)     /* Remove extension.           */
        *p = NULL;
    strcat(nam, ".");                        /* Set nam to "filename.".     */
    
    if ((p = strrchr (newnam,'.')) != NULL)  /* Remove extension.           */
        *p = NULL;
    strcat(newnam, ".*");                    /* Set newnam to "filename.*". */
    }
}
