/******************************************************************************

PROGRAM:	CMSET

PURPOSE:	Sets the user's default directory.

FATAL ERROR CONDITIONS:
		Specification error
		Specified directory does not exist: <directory name>
		Specified alias name not found
		Unable to open alias file

PROGRAM HISTORY:

Version    Date     Author           Code/Contr    Change Request
-------    -----    -------------    ----------    ---------------
  1.0      11/86    C. Greenhagen     EDC/CSB      New Development
  5.0      09/88    B. Davis	      CSB	   Conversion to 5.0
  5.1       1/91    D. Etrheim        CSB          Added device specification
						   for VMS.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE 

ALGORITHM(flow)

******************************************************************************/

#include <stdio.h>
#include <sys/file.h>
#include "las.h"
#include "labserv.h"
#include "util.h"
#include "alias.h"
#include "protos.h"

void main
(void)
{
struct PARBLK vblock;			/* TAE parameter block		*/

struct VARIABLE *in;			/* pointer to variable		*/

long fatal = 0;				/* value of zero 		*/
long one = 1;				/* value of one			*/
long status;				/* return status of calls	*/
long stdlun;				/* logical unit # of std output	*/
long zero = 0;				/* zero for lsstat, complt	*/

char *hname[1];				/* host name of new directory	*/
char msgtxt[CMLEN];			/* message text			*/
char name[CMLEN];			/* TAE name of new directory	*/
char tstring[CMLEN];			/* temporary string		*/
char outfiles[1][CMLEN];			/* Array of output file names	*/
char pname[CMLEN];			/* Program name			*/
char hostnm[CMLEN];			/* host name  			*/
char taenm[CMLEN];			/* TAE name  			*/


/************************************************************************/

c_inlas (&vblock, &stdlun);			/* Initialize		*/

strcpy(pname,"CMSET");				/* build program name	*/

in = p_find (&vblock,"INDIR");			/* get directory name	*/
strcpy (name, SVAL (*in,0));

status = c_getalias (name,tstring);		/* expand aliases,if any*/
if (status != E_SUCC)
    c_errmsg ("Fatal error encountered","cmset-fatal",&fatal);

hname[0] = (char *)malloc (255);
strcpy(taenm,tstring);				/* get host name	*/
status = c_cmhost (taenm,hostnm);
if (status != E_SUCC)
    c_errmsg ("Fatal error encountered","cmset-fatal",&fatal);
 
validate (hostnm);				/* validate input	*/
 
strcpy(hname[0],hostnm);

if ((c_lsstat (hostnm,&zero) != E_SUCC) || (!isdir(hostnm)))
    {
    sprintf (msgtxt,"Specified directory does not exist: %s",hname[0]);
    c_errmsg (msgtxt,"cmset-nodir",&one);
    c_errmsg ("Fatal error encountered","cmset-fatal",&fatal);
    }

status = q_string (&vblock,"INDIR",1,hname,P_UPDATE);
if (status != SUCCESS)
    c_errmsg ("Fatal error encountered","cmset-fatal",&fatal);
status = q_out (&vblock);
if (status != SUCCESS)
    c_errmsg ("Fatal error encountered","cmset-fatal",&fatal);

sprintf (msgtxt, "Setting default directory: %s",hostnm);
c_errmsg (msgtxt," ",&one);

c_complt (pname,outfiles,&zero);
}
