/*******************************************************************************
NAME      		   GETPAR


PURPOSE   To retrieve and edit the parameters.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      DEC 1991       Initial development
T.Mittan	FEB 1992       Finished development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   COMBINE must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   GETPAR retrieves the required TAE parameters from the parameter block. 
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"	
#include "ltable.h"
#include "combine.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,        /* Parameter block	             */
    char inlt1[],        /* first input LT  name	     	     */
    char inlt2[],        /* second input LT  name	     */
    char indat1[],       /* first input file name	     */
    char indat2[],       /* second input file name	     */
    char outfile[],      /* output file name  		     */
    long *incol,         /* input column	           	     */
    long *key_col_dat,	/* search key			     */
    long *key_col_lt,	/* search key			     */
    long *outcol,        /* output column		     */
    char outtyp[],	/* output type			     */
    long *ltflg,		/* lt flag			     */
    long *width_1,	/* column widths		     */
    long *width_2,	/* column withs			     */
    double *out_width,	/* output format width		     */
    char atrnames1[][CMLEN],/* first LT attribute names       */
    char atrnames2[][CMLEN],/* second LT attribute names      */
    long *append_flg,	/* append flag			     */
    long *main_ltflg,	/* lt flag			     */
    long *incol_count,   /* number of input lt atrnames       */
    char outlt[],        /* output LT name  		     */
    char key_name[],     /* first LT keycol attribute name    */
    long *msgtim        /* message time			     */
)
{

struct VARIABLE		*in;		/* Pointer to TAE variable within the
					   parameter block		     */
        long		count;		/* counter			     */
	long		j;		/* loop counter			     */
	long		std_out;	/* Standard output		     */
	long		len;		/* length			     */
        long		mode;		/* access mode			     */
	long		one = 1;	/* constant of one		     */
        long		zero = 0;	/* constant of zero		     */

	char		option[CMLEN];  /* subcommand variable               */
	char		temp[5];	/* temp variable		     */

/* LAS initialization
-------------------*/
c_inlas(vblock, &std_out);

/* Determine which subcomand was used
-----------------------------------*/
in = p_find(vblock, "_SUBCMD");
if (strcmp(SVAL(*in, 0), "2ASC") == 0)
   strcpy(option,"ASC");
else
if (strcmp(SVAL(*in, 0), "ALT") == 0)
   strcpy(option,"ALT");

/* Get the name of the input files, output file, key, and output data type 
------------------------------------------------------------------------*/
if (strcmp(option,"ASC") == 0)
    {
    mode = IREAD;
    if (c_getnam(vblock,"INFILE",&zero,indat1,"txt",&mode) != E_SUCC)
          return(E_FAIL);
    if (c_getnam(vblock,"INFILE",&one,indat2,"txt",&mode) != E_SUCC)
          return(E_FAIL);
    mode = IWRITE;
    if (c_getnam(vblock,"OUTFILE",&zero,outfile,"txt",&mode) != E_SUCC)
          return(E_FAIL);
    in = p_find(vblock, "ODTYPE");
    strcpy(outtyp,SVAL(*in,0));
    }
else
if (strcmp(option,"ALT") == 0)
    {
    mode = IREAD;
    if (c_getnam(vblock,"INFILE",&zero,indat1,"txt",&mode) != E_SUCC)
          return(E_FAIL);
    if (c_getnam(vblock,"INLT",&zero,inlt1,"lt",&mode) != E_SUCC)
          return(E_FAIL);
    *ltflg = TRUE;
    mode = IWRITE;
    if (c_getnam(vblock,"OUTFILE",&zero,outfile,"txt",&mode) != E_SUCC)
          return(E_FAIL);
    in = p_find(vblock, "ODTYPE");
    strcpy(outtyp,SVAL(*in,0));
    }
else
    {
    mode = IREAD;
    if (c_getnam(vblock,"INLT",&zero,inlt1,"lt",&mode) != E_SUCC)
          return(E_FAIL);
    if (c_getnam(vblock,"INLT",&one,inlt2,"lt",&mode) != E_SUCC)
          return(E_FAIL);
    *main_ltflg = TRUE;
    mode = IWRITE;
    if (c_getnam(vblock,"OUTLT",&zero,outlt,"lt",&mode) != E_SUCC)
          return(E_FAIL);
    }

/* get column widths or atrnames depending on subcomand
-----------------------------------------------------*/
if (strcmp(option,"ASC") == 0)
   {
   in = p_find(vblock,"COLWDTH1");
   count = (*in).v_count;
   for (j = 0; j < count; j++)
       width_1[j+1] = IVAL(*in,j);
   width_1[0] = count;

   in = p_find(vblock,"COLWDTH2");
   count = (*in).v_count;
   for (j = 0; j < count; j++)
       width_2[j+1] = IVAL(*in,j);
   width_2[0] = count;
   }
else
if (strcmp(option,"ALT") == 0)
   {
   in = p_find(vblock,"COLWDTH1");
   count = (*in).v_count;
   for (j = 0; j < count; j++)
       width_1[j+1] = IVAL(*in,j);
   width_1[0] = count;
   in = p_find(vblock,"ATRNAME1");
   count = (*in).v_count;
   for (j = 0; j < count; j++)
      {
      strcpy(atrnames1[j],SVAL(*in,j));
      len = (long) strlen(atrnames1[j]);
      c_low2up(atrnames1[j],&len);
      }
   }
else
   {
   in = p_find(vblock,"KEYNAME");
   strcpy(key_name,SVAL(*in,0));
   len = (long) strlen(key_name);
   c_low2up(key_name,&len);
   in = p_find(vblock,"ATRNAME1");
   count = (*in).v_count;
   for (j = 0; j < count; j++)
      {
      strcpy(atrnames1[j],SVAL(*in,j));
      len = (long) strlen(atrnames1[j]);
      c_low2up(atrnames1[j],&len);
      }
   *incol_count = count;
   in = p_find(vblock,"ATRNAME2");
   count = (*in).v_count;
   for (j = 0; j < count; j++)
       {
       strcpy(atrnames2[j],SVAL(*in,j));
       len = (long) strlen(atrnames2[j]);
       c_low2up(atrnames2[j],&len);
       }
   }

/* Get the input column and output column and width of output column 
------------------------------------------------------------------*/
if (strcmp(option,"ASC") == 0)
   {
   in = p_find(vblock,"INCOL");
   *incol = IVAL(*in,0);
   in = p_find(vblock,"OUTCOL");
   *outcol = IVAL(*in,0);
   in = p_find(vblock,"KEYCOL");
   *key_col_dat = IVAL(*in,0);
   *key_col_lt = IVAL(*in,1);
   in = p_find(vblock,"OUTWIDTH");
   count = (*in).v_count;
   if (count == 1)
      *out_width = RVAL(*in,0);
   else
      {
      if (strcmp(outtyp,"STRING") != 0)
         {
         c_errmsg("Invalid OUTWIDTH or DTYPE value","combine-dtype"
       	   ,&one);
         return(E_FAIL);
         }
      else
         *out_width = 10;
      }
   in = p_find(vblock,"APPDFLG");
   strcpy(temp,SVAL(*in,0));
   if (strcmp(temp,"YES") == 0)
       *append_flg = TRUE;
   }
else
if (strcmp(option,"ALT") == 0)
   {
   in = p_find(vblock,"OUTCOL");
   *outcol = IVAL(*in,0);
   in = p_find(vblock,"KEYCOL");
   *key_col_dat = IVAL(*in,0);
   in = p_find(vblock,"OUTWIDTH");
   count = (*in).v_count;
   if (count == 1)
      *out_width = RVAL(*in,0);
   else
      {
      if (strcmp(outtyp,"STRING") != 0)
         {
         c_errmsg("Invalid OUTWIDTH or DTYPE value","combine-dtype"
       	   ,&one);
         return(E_FAIL);
         }
      else
         *out_width = 10;
      }
   in = p_find(vblock,"APPDFLG");
   strcpy(temp,SVAL(*in,0));
   if (strcmp(temp,"YES") == 0)
       *append_flg = TRUE;
   }
else
   {
    in = p_find(vblock,"OUTCOL");
    *outcol = IVAL(*in,0);
    if (*outcol > (*incol_count + 1))
       *outcol = *incol_count + 1;
    }

in = p_find(vblock,"$msgtim");
*msgtim = IVAL(*in,0);
	
return(E_SUCC);

} /* End of getpar */
