/******************************************************************************

NAME:	LABTOPOP

FUNCTION:  LABTOPOP opens the output labeled table

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1           3/92      T. Mittan     original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Write vector data for each column needed
Initialize variables needed to open the output labeled table file.
Open the output labeled table file
For each column needed
   Write vector data

******************************************************************************/

#include "las.h"
#include "ltable.h"
#include "combine.h"
#include "protos.h"

void labtabop
(
    struct TAB_DEF *out_tab,	/* table descriptor 			*/
    struct TAB_DEF *tab,		/* table descriptor 			*/
    long *incol_cnt,		/* number of output columns needed 	*/
    char *hosout,			/* host output file name 		*/
    struct VECTOR *ptr[],		/* ptr to current table position	*/
    char atrname[][CMLEN],		/* attribute names of each column 	*/
    long *outcol,			/* output column			*/
    char *desc,			/* outcol LT descriptor			*/
    long *dtype			/* outcol data type			*/
)

{ 

long i;				/* counter -- current column		*/

short record[MAXCOL];		/* logical specifying when a record seperator
				   should be used			*/
char sep[MAXCOL];		/* field seperator 			*/

/*  Determine the seperator to be used
--------------------------------------*/
for (i=0; i <= *incol_cnt; i++)
   record[0] = TRUE;

for (i = *incol_cnt ; i >= 0; i--)
   {
   if (record[0])
      {
      sep[i] = RECORD_SEP;
      record[0] = FALSE;
      }
   else
      sep[i] = FIELD_SEP2;
   }

/*  Initialize variables needed to open the output labeled table file.
----------------------------------------------------------------------*/
out_tab->nsubs = 0; 
out_tab->ftype = tab->ftype;
out_tab->fdesc = tab->fdesc;

/*  Open the output labeled table file
--------------------------------------*/
open_tab(out_tab,hosout,IWRITE);

/* Write vector data for each column needed
-------------------------------------------*/
for (i=0; i <= *incol_cnt; i++)
      setvec(out_tab,i,ptr,atrname,sep,outcol,desc,dtype);

return;
} /* labtabop */
