/******************************************************************************

NAME: LTFIND	

FUNCTION:  LTFIND reads a labeled table file into an array. 

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         02/92	   T. Mittan    Initial development   

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

******************************************************************************/
#ifdef _IBMR2
#define unix
#endif
#include <stdio.h>
#include <string.h>
#ifdef aiws
#include <fcntl.h>
#endif
#ifdef unix
#include <sys/file.h>
#else
#include <file.h>
#endif
#include "las.h"
#include "ltable.h"
#include "combine.h"
#include "protos.h"

int ltfind
(
    char *inlt,			/* LT name				*/
    char atrname[][CMLEN],		/* names of the specified attributes 	*/
    char lt1_buf[][80],		/* keycol array				*/
    char lt2_buf[][80],		/* incol array				*/
    long *count,			/* number of rows in LT file		*/
    char *desc,			/* Lt description of incol		*/
    long *dtype			/* data type of incol			*/
)

{ 
struct MATRIX *size;		/* matrix size 				*/
struct TAB_DEF tab;		/* table descriptor			*/
struct VECTOR  *ptr[3];         /* pointer to current vector         	*/

union DATA *data;		/* data to be written to ASCII file	*/

long gstatus;			/*  status of get_rec calls		*/
long i;				/*  counter -- current column 		*/
long j;				/*  counter -- current file		*/
long k;				/*  counter -- current attribute value	*/
long nonfatal = 1;		/*  nonfatal error code			*/
long strsize;			/*  size of output string		*/

short found;			/*  found attribute logical		*/
short max_row = 0;		/*  Max no. of rows of all vectors	*/
short max_col = 0;		/*  Max no. of columns of all vectors	*/

char msg[ERRLEN];		/*  error message buffer		*/
char *strbuf;			/*  output string buffer		*/
char *strptr;			/*  output string pointer		*/


/* Opend lt file
---------------*/
open_tab(&tab,inlt,IREAD);
get_vector(&tab);

   /* Locate the attribute names
   -----------------------------*/
   for (i = 0; i < 2; i++)
      {
      ptr[i] = NULL;
      found = FALSE;
      ptr[i] = (struct VECTOR *)get_field_ptr(&tab,atrname[i]);
      if (ptr[i] != NULL)
          found = TRUE;
      if (!found)
         {
         sprintf(msg,"Attribute %s could not be found",atrname[i]);
         c_errmsg(msg,"combine-attribute",&nonfatal);
	 close_tab(&tab);
	 return(E_FAIL);
         }
      }  

   /*  Allocate buffers for the maximum matrix of the data types needed
   --------------------------------------------------------------------*/
   for (i = 0; i < 2; i++)
      {
      size = (struct MATRIX *)&ptr[i]->lt_size;
      max_row = (short) (LASMAX(max_row,size->nrow));
      max_col = (short) (LASMAX(max_col,size->ncol));
      }  /*  for i < 2  */

   /*  Allocate the output buffers
   -------------------------------*/
   data = (union DATA *) calloc(max_row * sizeof(union DATA),max_col);

   /*  Allocate the size of the output string to hold all the output columns
   -------------------------------------------------------------------------*/
   strsize = (max_row + max_col) * 2 * MAXDIG;
   strbuf = (char *) malloc(strsize);

   *dtype = ptr[1]->dtype;
   strcpy(desc,ptr[1]->desc);
/*  read each record of the input labeled table until EOF is reached
--------------------------------------------------------------------*/
do
  {
   /*  Assign the string pointer to the beginning of the string buffer
   ------------------------------------------------------------------*/
   strptr = strbuf;
   for ( i = 0; i < strsize; i++)
      strbuf[i] = ' ';

   gstatus = get_record(&tab);
   if (gstatus == E_FAIL)	/* Read the labeled table file        */
      {
      c_errmsg("Error reading input LT file","combine-read",&nonfatal);
      close_tab(&tab);
      return(E_FAIL);
      }
     

   /* write each column according the the data type of each column
   ---------------------------------------------------------------*/
   for (i = 0; i < 2; i++)
      {
      switch (ptr[i]->dtype)
        {
        case C: sprintf(strptr,"%s",ptr[i]->ptr.c);
            break;
        case I1: get_matrix(ptr[i],I1,(unsigned char *) &data->byte);
   	     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
                   sprintf(strptr,"%hd", *(&data->byte + (k * j) + k));
             break;
        case I2: get_matrix(ptr[i],I2,(unsigned char *) &data->i2);
	     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
                   sprintf(strptr,"%hd",*(&data->i2 + (k * j) + k));
             break;
        case I4: get_matrix(ptr[i],I4,(unsigned char *) &data->i4);
	     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
                   sprintf(strptr,"%d", *(&data->i4 + (k * j) + k));
             break;
        case R4: get_matrix(ptr[i],R4,(unsigned char *) &data->r4);
   	     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
                   sprintf(strptr,"%f", *(&data->r4 + (k * j) + k));
             break;
        case R8: get_matrix(ptr[i],R8,(unsigned char *) &data->r8);
	     for ( j = 0; j < ptr[i]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[i]->lt_size.ncol; k++)
                   sprintf(strptr,"%f", *(&data->r8 + (k * j) + k));
             break;
        case LTDATE: get_lt_date(ptr[i],&data->date);
	     sprintf(strptr,"%2d/%2d/%2d",data->date.month,data->date.day,
	             data->date.year);
             break;
        case LTTIME: get_lt_time(ptr[i],&data->time);
	     sprintf(strptr,"%2d:%2d:%2d",data->time.hour,data->time.minute,
		     data->time.second);
             break;
        } /*  switch  prt[i].dtype  */
      if (i == 0)
         strcpy(lt1_buf[*count],strptr);
      else
         strcpy(lt2_buf[*count],strptr);
     }
  (*count)++;
  }
  while(gstatus == E_SUCC);

/* Free dynamic memory
----------------------*/
free (data);
free (strbuf);

close_tab(&tab);
return(E_SUCC);
} 
