/*
PURPOSE:
 PROCES merges columns from two LT files into an output LT file. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   T.Mittan	    2/92	  EDC		Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    PROCESS must be run under TAE.

PROJECT:			LAS

*****************************************************************************/
#include <stdio.h>
#include <string.h>
#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif
#include "las.h"	
#include "ltable.h"
#include "combine.h"
#include "util.h"
#include "protos.h"

int ltproces
(
    char *inlt1,      	        /* input lt name	             */
    char *inlt2,      	        /* input lt name	             */
    char *outlt,			/* output ltfile		     */
    long *outcol,			/* output column		     */
    char artnames1[][CMLEN],        /* LT attribute names		     */
    char artnames2[][CMLEN],        /* LT attribute names		     */
    long *incol_count,		/* number of input atrnames	     */
    char *key_name,			/* KEYCOL attribute name             */
    long *msgtim
)
{
extern long curent;                     /* curent line                       */

	struct TAB_DEF out_tab;         /* table descriptor                  */
	struct TAB_DEF tab;             /* table descriptor                  */
	struct VECTOR  *ptr[MAXCOL];    /* ptr to first LT record            */

static	union DATA data[MAXCOL];         /* data to be written to ASCII file  */

	short   found;                  /* found attribute logical           */

	long 	count=0;		/* number of rows in second LT file  */
	long	dtype;                  /* data type of second LT column     */
	long	flag = 2;		/* flag for initm 		     */
	long	gstatus;                /* return status for get_record call */
	long	index;			/* index to array		     */
	long 	i;                  /* loop counters                     */
	long    nonfatal = 1;		/* nonfatal error flag		     */
	long	one = 1;
	long	status1;		/* return status indicator	     */
	long	temp = 0;               /* temporary index                   */


static	char	desc[CMLEN];            /* LT description of second LT column*/
static	char	dtemp[CMLEN];           /* temporarty character array        */
static	char	lt1_buf[3250][80];	/* buffer of "KEY STRINGS"           */
static	char	lt2_buf[3250][80];	/* buffer of all enteries in INCOL   */
static	char 	msg[ERRLEN];    	/* error message buffer              */


  /* read data from the second LT file 
  ----------------------------------*/
  status1 = ltfind(inlt2,artnames2,lt1_buf,lt2_buf,&count,desc,&dtype);
  if (status1 == E_FAIL)
       return(E_FAIL);

   open_tab(&tab,inlt1,IREAD);
   get_vector(&tab);

   /* Locate the attribute names for first input lt file 
   ---------------------------------------------------*/
   for (i = 0; i < *incol_count; i++)
      {
      ptr[i] = NULL;
      found = FALSE;
      ptr[i] = (struct VECTOR *)get_field_ptr(&tab,artnames1[i]);
      if (ptr[i] != NULL)
          found = TRUE;
      if (!found)
         {
         sprintf(msg,"Attribute %s could not be found",artnames1[i]);
         c_errmsg(msg,"combine-attribute",&nonfatal);
         close_tab(&tab);
         return(E_FAIL);
         }
      }
      ptr[*incol_count] = NULL;
      found = FALSE;
      ptr[*incol_count] = (struct VECTOR *)get_field_ptr(&tab,key_name);
      if (ptr[*incol_count] != NULL)
          found = TRUE;
      if (!found)
         {
         sprintf(msg,"Attribute %s could not be found",key_name);
         c_errmsg(msg,"combine-attribute",&nonfatal);
         close_tab(&tab);
         return(E_FAIL);
         }

   /*  Allocate buffers for the maximum matrix of the data types needed
   ------------------------------------------------------------------*/

   gstatus = get_record(&tab);
   if (gstatus == E_FAIL)       /* Read the labeled table file        */
      {
      c_errmsg("Error reading input LT file","combine-read",&nonfatal);
      close_tab(&out_tab);
      return(E_FAIL);
      }

    /* open the output labeled table and initilize it
    -----------------------------------------------*/
    labtabop(&out_tab,&tab,incol_count,outlt,ptr,artnames2,outcol,desc,&dtype);

    c_inittm("COMBINE","Processed","line","","",&flag,&one,msgtim,&flag);
    curent = 1;
    c_startm();

/* For each line in the input file read a record from the disk file and merge
   it with the column from the second file, and write to the output file 
---------------------------------------------------------------------------*/
do
  {
   /* read each column  of the first file according the the data type 
   ----------------------------------------------------------------*/
   index = *incol_count + 1;
   for (i = 0; i <= *incol_count; i++)
      {
      switch (ptr[i]->dtype)
        {
        case C: sprintf(data[i].chr,"%s", (char *) &ptr[i]->ptr.c);
	        strcpy(dtemp,data[i].chr);
            break;
        case I1: get_matrix(ptr[i],I1,(unsigned char *) &data[i].byte);
	        sprintf(&dtemp[0],"%hd",data[i].byte);
            break;
        case I2: get_matrix(ptr[i],I2,(unsigned char *) &data[i].i2);
		sprintf(&dtemp[0],"%hd",data[i].i2);
            break;
        case I4: get_matrix(ptr[i],I4,(unsigned char *) &data[i].i4);
		sprintf(&dtemp[0],"%d",data[i].i4);
            break;
        case R4: get_matrix(ptr[i],R4,(unsigned char *) &data[i].r4);
		sprintf(&dtemp[0],"%f",data[i].r4);
            break;
        case R8: get_matrix(ptr[i],R8,(unsigned char *) &data[i].r8);
		sprintf(&dtemp[0],"%f",data[i].r8);
            break;
        }
      }

    /* find "KEY colomn match
    -----------------------*/
    found = FALSE;
    for (i = temp;; i++)
      {
      if (strcmp(lt1_buf[i],dtemp) == 0)
         {
         switch (dtype)
            { 
            case (C):
                  sscanf(lt2_buf[i],"%s%*1c",data[index].chr);
                  break;
            case (I1):
                  sscanf(lt2_buf[i],"%c%*1c",&data[index].byte);
                  break;
            case (I2):
                  sscanf(lt2_buf[i],"%hd%*1c",&data[index].i2);
                  break;
            case (I4):
                  sscanf(lt2_buf[i],"%d%*1c",&data[index].i4);
                  break;
            case (R4):
                  sscanf(lt2_buf[i],"%f%*1c",&data[index].r4);
                  break;
            case (R8):
                  sscanf(lt2_buf[i],"%lf%*1c",&data[index].r8);
                  break;
            } /* case dtype */
        temp = i;
        found = TRUE;
        break;
        }
      else
        if (i == count)
           if (temp == 0)
              break;
           else
              i = -1;
        else
        if (i + 1 == temp)
           break;
      }

      /* if match not found place 0 into output column
      ---------------------------------------------*/
      if (!found)
        {
        switch (dtype)
         { /* switch based on dtype */
         case (C):
                 sscanf(" ","%s%*1c",data[index].chr);
                 break;
         case (I1):
                 sscanf("0","%c%*1c",&data[index].byte);
                 break;
         case (I2):
                 sscanf("0","%hd%*1c",&data[index].i2);
                 break;
         case (I4):
                 sscanf("0","%d%*1c",&data[index].i4);
                 break;
         case (R4):
                 sscanf("0","%f%*1c",&data[index].r4);
                 break;
         case (R8):
                 sscanf("0","%lf%*1c",&data[index].r8);
                 break;
         } /* case dtype */
      }
      /* write vector to lt file
      ------------------------*/
      writevec(&out_tab,ptr,outcol,&dtype,data,incol_count);
   
      /* read next record from first input lt file
      ------------------------------------------*/
      gstatus = get_record(&tab);
      if (gstatus == E_FAIL)       /* Read the labeled table file        */
         {
         c_errmsg("Error reading input LT file","combine-read",&nonfatal);
         close_tab(&out_tab);
         close_tab(&tab);
         return(E_FAIL);
         }
  curent++;
  }
while(gstatus == E_SUCC);

c_stoptm();

/* Close the input and output files
-------------------------------------------------*/
close_tab(&out_tab);
close_tab(&tab);

return(E_SUCC);
} 
