/*
PURPOSE:
 PROCES reads columns from two data files or one data file and on Lt file 
 and merges them to create an output  data file.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   T.Mittan	    2/92	  EDC		Initial development
  5.1      D.Etrheim        4/95          EDC		Remove blanks when
							comparing key strings

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    PROCESS must be run under TAE.

PROJECT:			LAS

*****************************************************************************/
#include <stdio.h>
#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif
#include "las.h"	
#include "ltable.h"
#include "combine.h"
#include "geompak.h"
#include "util.h"
#include "pixman.h"
#include "string.h"
#include "protos.h"

#define SEPCHARS " \"\n"

int proces
(
    char *inlt1,      	        /* input lt name	             */
    char *indat1,	   	        /* first input data name	     */
    char *indat2,	   	        /* secondinput data name	     */
    char *outfile,   	        /* output name	        	     */
    long *incol,			/* input column			     */
    long *key_col_dat,              /* key column of first data column   */
    long *key_col_lt,               /* key column of second data column  */
    long *outcol,			/* output column		     */
    char *outtyp,   	        /* data type of output column	     */
    long *ltflg,			/* input lt file flag		     */
    long *width_1,			/* widths of columns of first in file */
    long *width_2,			/* widths of columns of second in file*/
    double *out_width,		/* format of output column	     */
    char artnames1[][CMLEN],        /* LT attribute names		     */
    long *append_flg,		/* input lt file flag		     */
    long *msgtim			/* message time			     */
)
{
	FILE    *dat1_fd;		     /* First input data file descriptor  */
	FILE    *dat2_fd = NULL;     /* Second input data file descriptor */
	FILE    *out_fd;		     /* output file descriptor	     */

extern long curent;			/* curent line			     */
	long    count = 0;		/* number of values in lt_buf	     */
	long 	dtype;			/* data type of Lt colum	     */
	long	datype = EBYTE;
	long	find_flg = FALSE;	/* flag to indicate data was found   */
	long	flag = 2;		/* inittm flag			     */
	long	key_index1 = 0;		/* index to first input array(keycol)*/
	long	key_index2 = 0;		/* index to second input array(keycol)*/
        long    len;			/* len of output column 	     */
	long    nonfatal = 1;		/* nonfatal error flag		     */
	long	ns = 170;		/* number of samples for pxfill     */
        long    i;			/* counter			     */
	long	in_index = 0;		/* index to second input array (incol)*/
	long	one = 1; 		/* one				     */
	long	out_index = 0;		/* index to first input array (outcol)*/
	int 	ret_int;		/* temp var for output column	     */ 
	long	status1;		/* return status indicator	     */
	long	temp = 0;		/* temp position in array 	     */

	float	ret_real;		/* temp var for output column	     */

	char	desc[CMLEN];		/* descriptor of LT column	     */
	char	format[15];		/* format of output column	     */ 
        char    line_dat[CMLEN];	/* second input buffer		     */
        char    line1_dat[80];		/* second input temp buffer	     */
        char    line_lt[CMLEN];		/* first input buffer		     */
static	char	lt1_buf[3250][80];	/* buffer of "KEY STRINGS"           */
static	char	lt2_buf[3250][80];	/* buffer of all enteries in INCOL   */
        char	out_line[CMLEN];	/* output buffer 		     */
	char 	*status;		/* Status indicator		     */

char *ptr1,*ptr2;

/* Open the input files 
---------------------*/
if (!*ltflg)
   if ((dat2_fd = fopen(indat2,"r")) == NULL)
	{
	c_errmsg("Error opening second input file","combine-open",&nonfatal);
        return(E_FAIL);
	}
if ((dat1_fd = fopen(indat1,"r")) == NULL)
	{
	c_errmsg("Error opening first input file","combine-open",&nonfatal);
        return(E_FAIL);
	}

/* Open the output file
---------------------*/
if ((out_fd = fopen(outfile,"w")) == NULL)
	{
	c_errmsg("Error opening output file","combine-open",&nonfatal);
        return(E_FAIL);
	}

/* Find column locations
----------------------*/
if (*key_col_dat > width_1[0])
	{
	c_errmsg("First KEYCOL not found","combine-key",&nonfatal);
        fclose(dat1_fd);
        fclose(out_fd);
	if (!*ltflg)
           fclose(dat1_fd);
        return(E_FAIL);
	}
    
for (i = 1; i < *key_col_dat; i++)
    key_index1 += width_1[i];

if (*outcol > (width_1[0] + 1))
   for (i = 1; i < (width_1[0] + 1); i++)
      out_index += width_1[i];
else
   for (i = 1; i < *outcol; i++)
      out_index += width_1[i];

if (!*ltflg)
   {
   if (*key_col_lt > width_2[0])
	{
    	c_errmsg("Second KEYCOL not found","combine-key",&nonfatal);
        fclose(dat1_fd);
        fclose(dat2_fd);
        fclose(out_fd);
        return(E_FAIL);
	}
   for (i = 1; i < *key_col_lt; i++)
       key_index2 += width_2[i];
   for (i = 1; i < *incol; i++)
       in_index += width_2[i];
   }
/* read data from either the LT file or the second data file
----------------------------------------------------------*/
      /* if not a LT file read the data file
      ------------------------------------*/
      if (!*ltflg)
        {
        do 
          {
          status = fgets(line_lt,255,dat2_fd);
          strncpy(lt1_buf[count],&(line_lt[key_index2]),width_2[*key_col_lt]);
          lt1_buf[count][width_2[*key_col_lt]] = '\0';
          strncpy(lt2_buf[count],&(line_lt[in_index]),width_2[*incol]);
          lt2_buf[count][width_2[*incol]] = '\0';
          count++;
	  }
        while(status != NULL);
	fclose(dat2_fd);
        }
      else 
        {
        /* it is a LT file find the atribute values and remove any
  	extra blank characters from  line1_dat
        ---------------------------------------------------------*/
        status1 = ltfind(inlt1,artnames1,lt1_buf,lt2_buf,&count,desc,&dtype);
        if (status1 == E_FAIL)
           {
	   fclose(dat1_fd);
    	   fclose(out_fd);
           return(E_FAIL);
           }
        }

c_inittm("COMBINE","Processed","line","","",&flag,&one,msgtim,&flag);
    curent = 1;
    c_startm();

/* For each line in the input file read a record from the disk file and merge
   it with the column from the second file, and write to the output file 
---------------------------------------------------------------------------*/
for(;;)
  {
  if ((status = fgets(line_dat,255,dat1_fd)) == NULL)
     break;

  /* Find input data column
  ------------------------*/  
  strncpy(line1_dat,&(line_dat[key_index1]),width_1[*key_col_dat]);
  line1_dat[width_1[*key_col_dat]] = '\0';

  /* Find input data/lt line
  ------------------------*/
  for(;;)
    {
    find_flg = FALSE;
    if (*ltflg)
      {
      if ((status = (char *)strtok(line1_dat,SEPCHARS)) == NULL)
        {  
        strncpy(line1_dat,&(line_dat[key_index1]),width_1[*key_col_dat]);
        line1_dat[width_1[*key_col_dat]] = '\0';
        }
      else
        strcpy(line1_dat,status);
      }
    /* Find input data column
    -----------------------*/
    for (i = temp;; i++)
      {
      len = (long)strlen(line1_dat);
      ptr1 = rem_blanks(line1_dat,&len);
      len = (long)strlen(lt1_buf[i]);
      ptr2 = rem_blanks(lt1_buf[i],&len);
      if (strcmp(ptr1,ptr2) == 0)
        {
        strcpy(line_lt,lt2_buf[i]);
        find_flg = TRUE;
        temp = i;
        free(ptr1);
        free(ptr2);
        break;
        }
      else
        if (i == count)
           if (temp == 0)
              break;
           else
              i = -1;  
	else
        if (i + 1 == temp)
           break;
   
      free(ptr1);
      free(ptr2);
      }
    if (!find_flg)
      {
      if (*append_flg)
        {
	c_pxfill(line_lt,&datype,&ns," ");
        line_lt[(long) (*out_width) +1 ] = '\0';
        break;
        }
      else
        {
	c_pxfill(line_lt,&datype,&ns," ");
        line_lt[(long) *out_width] = '\0';
        break;
        }
      }
    else
      {
      /* Convert to OUTTYP data type
      ----------------------------*/
      if (strcmp(outtyp,"INTEGER") == 0)
        {
        ret_int = (int ) (*out_width);
        sprintf(format," %d",ret_int);
        format[0] = '%';
        len = (long) strlen(format);
        format[len] = 'd';
        format[len+1] = '\0';
        sscanf(line_lt,"%d",&ret_int);
        sprintf(line_lt,format,ret_int);
        if (*append_flg)
          {
          len = (long) strlen(line_lt);
          line_lt[len] = ' ';
          line_lt[len+1] = '\0';
          }
        }
      else
        if (strcmp(outtyp,"REAL") == 0)
          {
          ret_real = (float ) (*out_width);
          sprintf(format," %11.1f",ret_real);
          format[0] = '%';
          len = (long) strlen(format);
          format[len] = 'f';
          format[len+1] = '\0';
          sscanf(line_lt,"%f",&ret_real);
          sprintf(line_lt,format,ret_real);
          if (*append_flg)
            {
            len = (long) strlen(line_lt);
            line_lt[len] = ' ';
            line_lt[len+1] = '\0';
            }
          }
        else
          if (strcmp(outtyp,"STRING") == 0)
            {
            if (*append_flg)
              {
              out_line[0] = ' ';
              out_line[1] = '\0';
              strcat(out_line,line_lt);
              strcpy(line_lt,out_line);
              len = (long) strlen(line_lt);
              if (*out_width == 0)
                  *out_width = len;
	      c_pxfill(&line_lt[len],&datype,&ns," ");
              line_lt[(long) (*out_width) + 1] = '\0';
              }
            else
              {
              len = (long) strlen(line_lt);
              if (*out_width == 0)
                  *out_width = len;
	      c_pxfill(&line_lt[len],&datype,&ns," ");
              line_lt[(long) (*out_width)] = '\0';
              }
            }
      break;
      }
    }

  /* Format output line and write to output file
  --------------------------------------------*/  
  if (*outcol > 1)
    {
    if (*outcol > width_1[0])
       {
       strncpy(out_line,line_dat,out_index);
       out_line[out_index] = '\0';
       status = (char *)strchr(out_line,'\n');
       if (status != NULL)
          status[0] = ' ';
       strcat(out_line,line_lt);
       len = (long) strlen(out_line);
       out_line[len] = '\n';
       out_line[len+1] = '\0';
       }
    else
       {
       strncpy(out_line,line_dat,out_index);
       out_line[out_index] = '\0';
       strcat(out_line,line_lt);
       strcat(out_line,&(line_dat[out_index]));
       status = (char *)strchr(out_line,'\n');
       if (status != NULL)
          status[0] = ' ';
       len = (long) strlen(out_line);
       out_line[len] = '\n';
       out_line[len+1] = '\0';
       }
    }
  else
    {
    strcpy(out_line,line_lt);
    strcat(out_line,&(line_dat[out_index]));
    status = (char *)strchr(out_line,'\n');
    if (status != NULL)
       status[0] = ' ';
    len = (long) strlen(out_line);
    out_line[len] = '\n';
    out_line[len+1] = '\0';
    }
  if((status1 = fputs(out_line,out_fd)) == EOF)
    {
    c_errmsg("Error writting file encountered","combine-write",&nonfatal);
    fclose(dat1_fd);
    fclose(out_fd);
    return(E_FAIL);
    }
  curent++;
  }
c_stoptm();
/* Close the input and output files
---------------------------------*/
  fclose(dat1_fd);
  fclose(out_fd);

return(E_SUCC);
} 
