#include "complex.h"
#include "util.h"
#include "protos.h"

static long fatal_err = -1;
static long one = 1;

void getparms
(
    char lasin[][CMLEN],		/* Buffer for input las names */
    char lasin2[][CMLEN],	/* Buffer for input las names */
    char hosin[][CMLEN],		/* Buffer for input host names */
    char hosin2[][CMLEN],	/* Buffer for input host names */
    char hosout[][CMLEN], 		/* Buffer for output host names */
    char lasout[][CMLEN],	        /* Buffer for output las names */
    long *nmrimg,
    long *nmrimg2,	/* Number of input images */
    long *totbnd,
    long *totbnd2, 	/* Total no. of bands */
    long *mnmx,			/* Min/max value */
    long *msgtim,		/* Time processing message time interval      */
    long window[MAXIMG][4], 	/* Window specifications */
    long window2[MAXIMG][4] 	/*  for each image */
)

{
/************************************************************************

NAME: GETPAR

FUNCTION:
   to receive the initial input parameters.

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    4.0	      04-OCT-91	  T. ROCKVAM        Moved copy of lasout into out from
					    filspec so lasout doesn't have to be
					    passed there
    3.0       29-DEC-89   B. ALLAM, STX
    2.0       13-NOV-84   K. NARAYANAN SCN 20-363
    1.0       17-SEP-84   B. DEALY
************************************************************************/
long 	status,
     	imglim,			/* max no. of images */
     	qflag[MAXIMG][2],
     	same;			/* flag indicating if images have same window */

char    minmax[3];		/* min/max option */

struct VARIABLE *var;

imglim = MAXIMG;
/*
get input file information
--------------------------
*/
status = c_getin (&block, "IN", &zero, lasin, hosin, nmrimg, window, bands,
                  nbands, totbnd, &imglim, &same, qflag);

if (status != E_SUCC)
  c_errmsg ("Unable to retrieve input image parameter","complex-in",&fatal_err);

status = c_getin (&block, "IN", &one, lasin2, hosin2, nmrimg2, window2, bands2,
                  nbands2, totbnd2, &imglim, &same, qflag);

if (status != E_SUCC)
  c_errmsg ("Unable to retrieve input image parameter","complex-in",&fatal_err);

/*
check that inputs have same number of bands
-------------------------------------------
*/
if (*totbnd != *totbnd2)
  c_errmsg ("IN(1) and IN(2) must have the same number of bands",
	    "complex-bands", &fatal_err); 
/*
get output file information
---------------------------
*/
status = c_getout (&block, "OUT", lasout, hosout); 
if (status != E_SUCC) 
  c_errmsg ("Unable to retrieve output image", "complex-out", &fatal_err);
strcpy (out[num_files++], lasout[0]);

/*
windows must be same size
-------------------------
*/
if (window[0][NL] !=  window2[0][NL]) 
  c_errmsg ("Window must be same size for IN(1) and IN(2)", "complex-size",
            &fatal_err); 
if (window[0][NS] !=  window2[0][NS]) 
  c_errmsg ("Window must be same size for IN(1) and IN(2)", "complex-size",
            &fatal_err); 
if (window[0][SS] !=  window2[0][SS]) 
  c_errmsg ("Window must be same size for IN(1) and IN(2)", "complex-size",
            &fatal_err); 
if (window[0][SL] !=  window2[0][SL]) 
  c_errmsg ("Window must be same size for IN(1) and IN(2)", "complex-size",
            &fatal_err); 

/*
get global values specified by the user.
---------------------------------------
*/
var = p_find (&block, "$MSGTIM");
*msgtim = IVAL (*var, 0);
var = p_find  (&block, "$MINMAX"); /*** NEWLAS call ***/
strcpy(minmax, SVAL(*var,0));
if (minmax[0] == 'Y' || minmax[0] == 'y') *mnmx = TRUE;
else *mnmx = FALSE;
}
