#include "complex.h"
#include "util.h"
#include "protos.h"

static char    *prog_name = "COMPLEX"; /* program name                 */
static long fatal_err = -1;
static long one = 1;

void filspec
(
    char hostin[][CMLEN],         /* input image host names */
    char hostin2[][CMLEN],
    char hosout[CMLEN],	       /* output image name */
    long wind[][4],               /* input image windows */
    long wind2[][4],
    long nimages,
    long nimages2,        /* no. of input images */
    long totbnd		       /* total no. of bands */
)

{
/*
**********************************************************************
NAME: FILSPEC

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    6.1	      23-MAY-94   D. EICHACKER	    Added opt[] for c_eopens.
    4.0	      04-OCT-91   T. ROCKVAM	    Removed unused parameters
    3.0       29-DEC-89   B. ALLAM, STX
    2.0       13-NOV-84   K. NARAYANAN SCN 20-363
    1.0       17-SEP-84   B. DEALY

FUNCTION: sets up the image i/o interface

***********************************************************************
*/

   long    stat,                  /* flag for EOPENS procedure */
           access,                /* file specifications       */
           type,                  /* data type                 */
           np, n, nib,            /* counter variables         */
           bufsiz1, obufsiz, bufsiz; /* image i/o buffer size     */
   double  y_inc, x_inc;          /* line and sample increments*/
static	long	opt[3]={0,0,0};   /* Option array for c_eopens */
/*
 set up input file group
 -----------------------
*/

   access = IREAD;
   type = EREAL;
   for (n = 0, nib = 0; n < nimages ; n++)
   {
     stat = c_eopens(&fdesc[n], hostin[n], bands[n], &wind[n][SL],
                     &wind[n][SS], &wind[n][NL], &wind[n][NS], &ipoint[nib],
                     &access, &type, &zero, opt);
     if (stat != E_SUCC)
       c_errmsg("Error opening input image", "complex-input", &fatal_err);
     nib += 1; 
    }
   fdesc[++n] = 0;
   for (n = 0, nib = 0; n < nimages2; n++)
   {
     stat = c_eopens(&fdesc2[n], hostin2[n], bands2[n], &wind2[n][SL],
                     &wind2[n][SS], &wind2[n][NL], &wind2[n][NS], &ipoint2[nib],
                     &access, &type, &zero, opt);
     if (stat != E_SUCC)
       c_errmsg("Error opening input image", "complex-input", &fatal_err);
     nib += 1; 
    }
   fdesc2[++n] = 0;

/*
 set up output file 
 ------------------
*/
   access = IWRITE;
   np = wind[0][NS] * 2;
   n = 0; 
   stat = c_eopens(&fdescout[n], hosout, &totbnd, &one, &one, &wind[0][NL],
                   &np, opoint, &access, &type, &zero, opt);
   if (stat != E_SUCC)
     c_errmsg("Error opening output image", "complex-out", &fatal_err);
   fdescout[++n] = 0;
/*
set up group containing input and output bands
----------------------------------------------
*/
   if ((inbuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Complex-alloc",
              prog_name, out, &num_files);
   if ((inbuf2 = (float *) c_ealloc(fdesc2, &bufsiz1)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Complex-alloc",
              prog_name, out, &num_files);

   if ((outbuf = (float *) c_ealloc(fdescout, &obufsiz)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Complex-alloc",
              prog_name, out, &num_files);

   if (c_egroup(fdesc, &gdesc,(unsigned char *) inbuf, &bufsiz) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Complex-group",
              prog_name, out, &num_files);
   if (c_egroup(fdesc2, &gdesc2,(unsigned char *) inbuf2, &bufsiz1) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Complex-group",
              prog_name, out, &num_files);

   if (c_egroup(fdescout,&gdescout,(unsigned char *) outbuf,&obufsiz) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Complex-group",
              prog_name, out, &num_files);

/*
update DDR
----------
*/
y_inc = 1;
x_inc = 1;
c_upddr(&block, hostin, hosout, &nimages, wind, bands, nbands, 
        &y_inc, &x_inc, &totbnd, &zero);

}
