#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "compol.h"
#include "protos.h"

extern long    curcat;                        /* message utility         */
extern long    curent;                        /* parameters for processing */

struct PARBLK block;
struct FDESC *    fdescout[2];                   /* output file descriptor  */
float   *inbuf;                         /* image input buffer     */
float   *inbuf2;                        /* image input buffer     */
float   *outbuf;                        /* image output buffer    */
struct GDESC *    gdesc;                         /* group descriptor        */
struct GDESC *    gdesc2;                         /* group descriptor       */
struct GDESC *    gdescout;

int main
(void) /* compol */
{

/*

NAME: COMPOL

FUNCTION:
   Combines two input images on a pixel by pixel basis to form a
   complex output image.

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    5.0	      21-SEP-92   T. Mittan	  Modified include file to eliminate
					  warnings on link.
    4.0       13-Nov-90   W. XIA, STX     fixed bugs after conversion
    3.0       29-DEC-89   B. ALLAM, STX
    2.0       13-NOV-84   K. NARAYANAN 
    1.0       01-JUN-84   B. DEALY

ALGORITHM:
    Initialize LAS
    Get input and output parameters
    Setup input and output file specifications
    Form a complex output image
    Update output file histories 
    Write min and max values for output image

*/

char	hosin[MAXIMG][CMLEN],	/* Buffer for input host names */
 	hosin2[MAXIMG][CMLEN],	/* Buffer for input host names */
	lasin[MAXIMG][CMLEN],	/* Buffer for input las names */
	lasin2[MAXIMG][CMLEN];	/* Buffer for input las names */

char out[1][CMLEN];
char lasout[1][CMLEN];      /* Buffer for output las names */
char hosout[1][CMLEN];   /* Buffer for output host names */

long nbands[MAXBND];    /* number of bands for each specified image */
long ipoint[MAXIMG * MAXBND+1];
long ipoint2[MAXIMG * MAXBND+1];
long one = 1;
long opoint[MAXBND];


static
long	band,			/* Total bands loop counter		*/
	bands2[MAXIMG][MAXBND+1],   /* bands numbers for each specified image */
       	bands[MAXIMG][MAXBND+1];    /* bands numbers for each specified image*/
long	wind[MAXIMG][4], 	/* window specifications */                  
	wind1[MAXIMG][4], 	/* window specifications for output image */
	wind2[MAXIMG][4], 	/* window specifications */
	nmrimg, 		/* Number of input images */
	nmrimg2, 		/* Number of input images */
        mnmx,			/* min/max value */
        std_out, 		/* flag parameter */
	totbnd, 		/* total no. of bands */
	totbnd2, 		/* total no. of bands */
        msgtim;			/* Time processing message time interval      */

double omin[MAXBND]; /* min and max values */
double omax[MAXBND];

/*
Initialize LAS
--------------
*/
c_inlas (&block, &std_out); 

/*
Get input and ouput parameters
------------------------------
*/
getpar (lasin, lasin2, hosin, hosin2,  hosout, &nmrimg, &nmrimg2,
        &totbnd, &totbnd2, lasout, &mnmx, &msgtim, wind, 
        wind2, bands2, bands, nbands);

/*
setup input and output file specifications
------------------------------------------
*/
filspec (hosin, hosin2, hosout[0], wind, wind2, nmrimg, nmrimg2,
         totbnd, bands2, bands, nbands, ipoint, ipoint2, opoint,
         out, wind1);
/*
do the job
----------
*/
comcalc (wind, nmrimg, msgtim, nbands, ipoint, ipoint2, opoint);

/* 
close all the files opened
--------------------------
*/
c_egclse(&gdesc);
c_egclse(&gdesc2);
c_egclse(&gdescout);

/* 
write min and max values for output image
-----------------------------------------
*/
for (band = 0; band < totbnd; ++band)
{
	bands2[0][band] = band + 1;
}
if (mnmx)
  {
  c_minmax (hosout, &one, wind1, bands2, nbands, omin,omax);
  c_svmnmx (hosout[0], &totbnd, omin, omax, "YES");
  }

/*
 update output file histories 
 ----------------------------
*/
c_hstout(&block, hosout[0], hosin, &nmrimg, " ");

/*
 completion messages
 -------------------
*/
c_complt("COMPOL", lasout, &one);

return EXIT_SUCCESS;

}

