#include "las.h"
#include "util.h"
#include "compol.h"
#include "protos.h"

void filspec
(
    char hostin[][CMLEN],         /* input image host names */
    char hostin2[][CMLEN],	       /* input image host namex */
    char hosout[],		       /* output image name */
    long wind[][4],               /* input image windows           */
    long wind2[][4],
    long nimages,
    long nimages2,        /* no. of input images           */
    long totbnd,		       /* total no. of bands */
    long bands2[][MAXBND+1],
    long bands[][MAXBND+1],
    long nbands[],
    long ipoint[],
    long ipoint2[],
    long opoint[],
    char out[][CMLEN],
    long wind1[][4]
)
{
/*
**********************************************************************
NAME: FILSPEC

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    3.0       29-DEC-89   B. ALLAM, STX
    2.0       13-NOV-84   K. NARAYANAN 
    1.0       01-JUN-84   B. DEALY
    6.1       24-May-94   D. Eichacker       Added opt[]

FUNCTION: sets up the image i/o interface

ALGORITHM:
      set up input file specifications

***********************************************************************
*/
   char    *prog_name = "COMPOL"; /* program name	       */

static	long	opt[3]={0,0,0};   /* optional parameter array for EOPENS */

struct FDESC * fdesc[MAXIMG];   /* Input file descriptors */
struct FDESC * fdesc2[MAXIMG];  
   static
   long    stat,                  /* flag for EOPENS procedure */
           access,                /* file specifications       */
  	   comb_flag=1,		/* image combination flag 		   */
           type,                  /* data type                 */
           n, nib,                /* counter variables         */
           bufsiz1, obufsiz, bufsiz; /* image i/o buffer size     */
   long fatal_err = FATAL_ERR;    /* error flag */
   long one = 1, zero = 0;        /* constants */
   double  y_inc, x_inc;          /* line and sample increments*/
   

/*
 set up input file group
 -----------------------
*/

   access = IREAD;
   type = EREAL;
   for (n = 0, nib = 0; n < nimages ; n++)
   {
     stat = c_eopens(&fdesc[n], hostin[n], bands[n], &wind[n][SL],
                     &wind[n][SS], &wind[n][NL], &wind[n][NS], &ipoint[nib],
                     &access, &type, &zero, opt);
     if (stat != E_SUCC)
       c_errmsg("Error opening input image", "Compol-input", &fatal_err);
     nib += 1; 
    }
   fdesc[n] = NULL;
   for (n = 0, nib = 0; n < nimages2; n++)
   {
     stat = c_eopens(&fdesc2[n], hostin2[n], bands2[n], &wind2[n][SL],
                     &wind2[n][SS], &wind2[n][NL], &wind2[n][NS], &ipoint2[nib],
                     &access, &type, &zero, opt);
     if (stat != E_SUCC)
       c_errmsg("Error opening input image", "Compol-input", &fatal_err);
     nib += 1; 
    }
   fdesc2[n] = NULL;
/*
 set up output file 
 ------------------
*/
   wind1[0][SS] = 1;
   wind1[0][SL] = 1;
   wind1[0][NS] = wind[0][NS]*2;
   wind1[0][NL] = wind[0][NL];
     
   access = IWRITE;
   n = 0; 
   stat = c_eopens(&fdescout[n], hosout, &totbnd, &one, &one, &wind1[0][NL],
                   &wind1[0][NS], opoint, &access, &type, &zero, opt);      

   if (stat != E_SUCC)
     c_errmsg("Error opening output image", "Compol-out", &fatal_err);
   fdescout[n+1] = NULL;
   strcpy (out[0], hosout);
/*
set up group containing input and output bands
----------------------------------------------
*/
   if ((inbuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Compol-alloc",
              prog_name, out, &one);
   if ((inbuf2 = (float *) c_ealloc(fdesc2, &bufsiz1)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Compol-alloc",
              prog_name, out, &one);

   if ((outbuf = (float *) c_ealloc(fdescout, &obufsiz)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Compol-alloc",
              prog_name, out, &one);


   if (c_egroup(fdesc, &gdesc,(unsigned char *) inbuf, &bufsiz) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Compol-group",
              prog_name, out, &one);
   if (c_egroup(fdesc2, &gdesc2,(unsigned char *) inbuf2, &bufsiz1) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Compol-group",
              prog_name, out, &one);

   if (c_egroup(fdescout,&gdescout,(unsigned char *) outbuf,&obufsiz) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Compol-group",
              prog_name, out, &one);

/*
update DDR
----------
*/
y_inc = 1;
x_inc = 1;
c_upddr(&block, hostin, hosout, &nimages, wind1, bands, nbands, 
        &y_inc, &x_inc, &totbnd, &comb_flag);

}
