#include "las.h"
#include "util.h"
#include "compol.h"
#include "protos.h"

void getpar
(
    char lasin[][CMLEN],		/* Buffer for input las names */
    char lasin2[][CMLEN],	/* Buffer for input las names */
    char hosin[][CMLEN],		/* Buffer for input host names */
    char hosin2[][CMLEN],	/* Buffer for input host names */
    char hosout[][CMLEN], 		/* Buffer for output host names */
    long *nmrimg,
    long *nmrimg2,	/* Number of input images */
    long *totbnd,
    long *totbnd2, 	/* total no. of bands */
    char lasout[][CMLEN],		/* Buffer for output las names */
    long *mnmx,			/* min/max value */
    long *msgtim,		/* Time processing message time interval      */
    long window[][4],	 	/* window specifications */
    long window2[][4], 		/* for each image */
    long bands2[][MAXBND+1],     /* bands numbers for each specified image */
    long bands[][MAXBND+1],       /* bands numbers for each specified image */
    long nbands[]               /* number of bands for each specified image */
)
{    
/*

NAME: GETPAR

FUNCTION:
   to receive the initial input parameters.

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    2.0       15-DEC-89   B. ALLAM, STX
    1.0       13-JUN-84   B. DEALY
*/
static
long 	status,
     	imglim,			/* max no. of images */
     	qflag[MAXIMG][2],                            
	nbands2[MAXIMG],
     	same;			/* Flag indicating if images have same window */
long    fatal_err = FATAL_ERR;  /* error flag */
long    one = 1, zero = 0;      /* constants */
long    i,j;
char    minmax[3];		/* min/max option */
struct VARIABLE *var;

imglim = MAXIMG;

/*
 get input file information
 --------------------------
*/
status = c_getin (&block, "IN", &zero, lasin, hosin, nmrimg, window, bands,
                  nbands, totbnd, &imglim, &same, qflag);

if (status != E_SUCC)
  c_errmsg ("Unable to retrieve input image parameter", "Compol-in", 
	    &fatal_err);
if (qflag[0][0] == 1)
  c_errmsg ("No window allowed in COMPOL", "Compol-wind", &fatal_err);

status = c_getin (&block, "IN", &one, lasin2, hosin2, nmrimg2, window2, bands2,
                  nbands2, totbnd2, &imglim, &same, qflag);

if (status != E_SUCC)
  c_errmsg ("Unable to retrieve input image parameter", "Compol-in", 
	    &fatal_err);

if (qflag[1][0] == 1)
  c_errmsg ("No window allowed in COMPOL", "Compol-wind", &fatal_err);
                                                                        
/*
  check that inputs have same number of bands
  -------------------------------------------
*/
if (*totbnd != *totbnd2)
  {
  c_errmsg ("IN(1) and IN(2) must have the same number of bands",
	    "Compol-bands", &fatal_err); 
  }  
/*
  check that inputs have same band numbers
  -------------------------------------------
*/
for (j=0; j<*nmrimg; j++)                               
   {
   for (i = 0; i< *totbnd; i++)
      {
      if (bands[j][i] != bands2[j][i])
         {     
         c_errmsg ("IN(1) and IN(2) must have the same band numbers", 
            "Compol-bands", &fatal_err); 
         }
       }     
    } 
/*   
 get output file information
 ---------------------------
*/
status = c_getout (&block, "OUT", lasout, hosout); 
if (status != E_SUCC) 
  c_errmsg ("Unable to retrieve output image", "Compol-out", &fatal_err);

/*
windows must be same size
-------------------------
*/
if (window[0][NL] !=  window2[0][NL]) 
  c_errmsg ("Window must be same size for IN(1) and IN(2)", "Compol-size",
            &fatal_err); 
if (window[0][NS] !=  window2[0][NS]) 
  c_errmsg ("Window must be same size for IN(1) and IN(2)", "Compol-size",
            &fatal_err); 
if (window[0][SS] !=  window2[0][SS]) 
  c_errmsg ("Window must be same size for IN(1) and IN(2)", "Compol-size",
            &fatal_err); 

/*
get global values specified by the user.
---------------------------------------
*/
var = p_find (&block, "$MSGTIM");
*msgtim = IVAL (*var, 0);

var = p_find  (&block, "$MINMAX");
strcpy(minmax, SVAL(*var,0));
if (minmax[0] == 'Y' || minmax[0] == 'y') *mnmx = TRUE;
else *mnmx = FALSE;
}
