/*******************************************************************************
NAME:    GET_DTYPE

PURPOSE: Determines the output data type and it's size

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   SET_DTYPE must be run under TAE.

PROJECT:        LAS

ALGORITHM REFERENCES:     none

*******************************************************************************/
#include "las.h"
#include "util.h"
#include "protos.h"

int get_dtype
(
    char hostin[][CMLEN],  /* I: Pointer to input host file names            */
    long *dtype,           /* O: Data type of the output image                */
    long num_images,       /* I: Number of input images                       */
    long *size,            /* O: Num of bytes required to store the data type */
    long update_mode       /* I: Flag to modify the first input image         */
)

{
char name[CMLEN];          /* Individual names of the input images            */

long i;                    /* Loop control                                    */
long one = 1;              /* One                                             */
long status;               /* Function return status code                     */
long type;                 /* Data type of current image                      */

strcpy(name, hostin[0]);
status = c_gettyp((char (*)[256])name, &one, dtype);
if (status != E_SUCC)
   return(E_FAIL);

/* If not in update mode then find the largest data type of the input images
----------------------------------------------------------------------------*/
if (!update_mode)
   {
   for (i = 1; i < num_images; i++)
      {
      strcpy(name, hostin[i]);
      status = c_gettyp((char (*)[256])name, &one, &type);
      if (status != E_SUCC)
	 return(E_FAIL);

      if (type > *dtype)
	 *dtype = type;
      }
   }

/* Determine the size of the data type
--------------------------------------*/
switch (*dtype)
   {
   case EBYTE: *size = 1; break;
   case EWORD: *size = 2; break;
   case ELONG: *size = 4; break;
   case EREAL: *size = 4; break;
   }

return(E_SUCC);
}
