#include "comsep.h"
#include "util.h"
#include "protos.h"

static long one = 1;
static long fatal_err = -1;

void getpar
(
    char hosin[][CMLEN],		/* Buffer for input host names */
    char hosout[][CMLEN],	/* Buffer for output host names */
    long window[MAXIMG][4], 	/* window specifications */
    long *nmrimg, 		/* Number of input images */
    long *totbnd, 		/* total no. of bands */
    long *mnmx,			/* min/max value */
    long *msgtim		/* Time processing message time interval      */
)
{
/***************************************************************************

NAME: GETPAR

FUNCTION:
   to receive the initial input parameters.

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    5.0       04-OCT-91   T. ROCKVAM     Removed lasout parameter: it is now 
					 retrieved in filspec
    4.0       20-FEB-90   B. ALLAM, STX  Converted complex to real pixel count
    3.0       29-DEC-89   B. ALLAM, STX
    2.0       09-NOV-84   B. DEALY
    1.0       18-SEP-84   B. DEALY
***************************************************************************/
long 	status,
     	img, imglim,		/* max no. of images */
     	qflag[MAXIMG][2],
        access,			/* access mode for files */
     	same;			/* flag indicating if images have same window */

char    lasin[MAXIMG][CMLEN];	/* Buffer for input las names */
char    minmax[3];		/* min/max option */

struct VARIABLE *var;

imglim = MAXIMG;
/*
get input file information
--------------------------
*/
status = c_getin (&block, "IN", &zero, lasin, hosin, nmrimg, window, bands,
                  nbands, totbnd, &imglim, &same, qflag);

if (status != E_SUCC)
 c_errmsg ("Unable to retrieve input image parameter", "comsep-in", &fatal_err);

/*
check input data type
---------------------
*/
for (img = 0; img < *nmrimg; img++)
  {
  c_getddr(hosin[img], &ddr);
  if (ddr.dtype != EREAL)
    c_errmsg ("Invalid data type", "comsep-type", &fatal_err);
/*
  convert from complex to real pixel count
  ----------------------------------------
*/  
  window[img][NS] = window[img][NS] * 2;
/*
  handle default window
  ---------------------
*/
  if (window[img][SS] == 0) window[img][SS] = 1;
  if (window[img][SL] == 0) window[img][SL] = 1;
  if (window[img][NS] == 0) window[img][NS] = ddr.ns;
  if (window[img][NL] == 0) window[img][NL] = ddr.nl;
  window[img][NS] = min(window[img][NS],ddr.ns);
  window[img][NL] = min(window[img][NL],ddr.nl);
  }

/*
get output file information
--------------------------
*/
access = IWRITE;
status = c_getnam (&block, "OUT", &zero, hosout[0], "img",&access);
if (status != E_SUCC) 
  c_errmsg ("Unable to retrieve output image", "comsep-out", &fatal_err);
status = c_getnam (&block, "OUT", &one, hosout[1], "img",&access);
if (status != E_SUCC) 
  c_errmsg ("Unable to retrieve output image", "comsep-out", &fatal_err);

/*
get global values specified by the user.
---------------------------------------
*/
var = p_find (&block, "$MSGTIM");
*msgtim = IVAL (*var, 0);
var = p_find  (&block, "$MINMAX"); /*** NEWLAS call ***/
strcpy(minmax, SVAL(*var,0));
if (minmax[0] == 'Y' || minmax[0] == 'y') *mnmx = TRUE;
else *mnmx = FALSE;

}
