#include "comsep.h"
#include "util.h"
#include "protos.h"

static  char    *prog_name = "COMSEP"; /* program name               */
static long one = 1;
static long fatal_err = -1;


void filspec
(
    char hosin[][CMLEN],          /* input image host names */
    char hosout[][CMLEN],	       /* output image host names */
    long wind[][4],               /* input image windows */
    long nimages,                 /* no. of input images */
    long totbnd		       /* total no. of bands */
)
{
/*
**********************************************************************
NAME: FILSPEC

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    4.0	      04-OCT-91   T. ROCKVAM	    Added call to c_cmtae to get las
					    image name.
    3.0       29-DEC-89   B. ALLAM, STX
    2.0       09-NOV-84   B. DEALY
    1.0       18-SEP-84   B. DEALY
    6.1       24-MAY-94   D. Eichacker, EDC  Added opt[]

FUNCTION: sets up the image i/o interface

***********************************************************************
*/

static	long	opt[3]={0,0,0};   /* optional parameter array for EOPENS */

   long    stat,                  /* flag for EOPENS procedure 	*/
           access,                /* file specifications       	*/
           type,                  /* data type                 	*/
           np, n, nib,            /* counter variables         	*/
           obufsiz1, obufsiz, bufsiz; /* image i/o buffer size 	*/
   double  y_inc, x_inc;          /* line and sample increments	*/

   
/*
 set up input file group
 -----------------------
*/
   access = IREAD;
   type = EREAL;
   for (n = 0, nib = 0; n < nimages ; n++)
   {
     stat = c_eopens(&fdesc[n], hosin[n], bands[n], &wind[n][SL],
                     &wind[n][SS], &wind[n][NL], &wind[n][NS], &ipoint[nib],
                     &access, &type, &zero, opt);
     if (stat != E_SUCC)
       c_errmsg("Error opening input image", "comsep-input", &fatal_err);
     nib += 1; 
    }
   fdesc[++n] = 0;

/*
 set up output file 
 ------------------
*/
   np = wind[0][NS] / 2;
   access = IWRITE;
   n = 0; 
   stat = c_eopens(&fdescout[n], hosout[0], &totbnd, &one, &one, &wind[0][NL],
                   &np, opoint, &access, &type, &zero, opt);
   if (stat != E_SUCC)
     c_errmsg("Error opening output image", "comsep-out", &fatal_err);
   fdescout[++n] = 0;
   strcpy (out[count++], hosout[0]);
   n = 0;
   stat = c_eopens(&fdescout1[n], hosout[1], &totbnd, &one, &one, &wind[0][NL],
                   &np, opoint1, &access, &type, &zero, opt);
   if (stat != E_SUCC)
     c_abort (&block, "Error opening output image", "Comsep-out",
              prog_name, out, &count);
   fdescout1[++n] = 0;
   strcpy (out[count++], hosout[1]);
/*
set up group containing input and output bands
----------------------------------------------
*/
   if ((inbuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Comsep-alloc",
              prog_name, out, &count);

   if ((outbuf = (float *) c_ealloc(fdescout, &obufsiz)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Comsep-alloc",
              prog_name, out, &count);

   if ((outbuf1 = (float *) c_ealloc(fdescout1, &obufsiz1)) == NULL)
     c_abort (&block, "Error allocating dynamic memory", "Comsep-alloc",
              prog_name, out, &count);

   if (c_egroup(fdesc, &gdesc,(unsigned char *)  inbuf, &bufsiz) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Comsep-group",
              prog_name, out, &count);

   if (c_egroup(fdescout, &gdescout,(unsigned char *)  outbuf,
                 &obufsiz) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Comsep-group",
              prog_name, out, &count);

   if (c_egroup(fdescout1, &gdescout1,(unsigned char *)  outbuf1,
                 &obufsiz1) != E_SUCC)
     c_abort (&block, "Error grouping image bands", "Comsep-group",
              prog_name, out, &count);
/*
update DDR
----------
*/
y_inc = 1;
x_inc = 1;
c_upddr(&block, hosin, hosout[0], &nimages, wind, bands, nbands, 
        &y_inc, &x_inc, &totbnd, &zero);

}
