/*******************************************************************************
NAME: AVEROPT	

PURPOSE: To determine which direction the average algorithm should run.

PROGRAM HISTORY:
PROGRAMMER    DATE          REASON
----------    ----          ------
T. Mittan     03/92         Allow average to be taken in either direction

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   AVEROPT must be run under TAE.

PROJECT: LAS		                

ALGORITHM REFERENCES: None
*******************************************************************************/
#include "las.h"
#include "concat.h"

void averopt
(
  long *laver,              /* O: Left average array */
  long out_window[][4],     /* I: Window in output image */
  long *limage              /* I: Current image number */
)
{
  long i;                   /* Loop control variable */
  long j;                   /* Loop control variable */
  long sl;                  /* Starting line */
  long nl;                  /* Number of lines */

  /* Check overlap for each image
  -------------------------------*/
  for (i = 0; i < *limage; i++)
  {
    if ((out_window[i][SL] < (out_window[*limage][SL] +
         out_window[*limage][NL])) && 
        ((out_window[i][SL] + out_window[i][NL]) > out_window[*limage][SL])) 
    {
      if ((out_window[i][SS] <= out_window[*limage][SS]) &&
          ((out_window[i][SS] + out_window[i][NS]) <= (out_window[*limage][SS]
          + out_window[*limage][NS])))
      {
        if (out_window[i][SL] < out_window[*limage][SL])
        {
          sl = 0; 
          if ((out_window[i][SL] + out_window[i][NL]) < 
              (out_window[*limage][SL] + out_window[*limage][NL]))
            nl = out_window[i][SL] + out_window[i][NL] - 
                 out_window[*limage][SL];
          else
            nl = out_window[*limage][NL];
        }
        else
        {
          sl = out_window[i][SL] - out_window[*limage][SL] ;
          if ((out_window[i][SL] + out_window[i][NL]) < 
              (out_window[*limage][SL] + out_window[*limage][NL]))
            nl = out_window[i][NL];
          else
            nl = out_window[*limage][SL] + out_window[*limage][NL] - 
                 out_window[i][SL];
        }
        for (j = sl; j < sl+nl; j++)
          laver[j] = FALSE;
      }
      else
      {
        if ((out_window[i][SS] < (out_window[*limage][SS] + 
             out_window[*limage][NS])) &&
            ((out_window[i][SS] + out_window[i][NS]) >
             (out_window[*limage][SS] + out_window[*limage][NS]))) 
        {
          if (out_window[i][SL] < out_window[*limage][SL])
          {
            sl = 0; 
            if ((out_window[i][SL] + out_window[i][NL]) < 
                (out_window[*limage][SL] + out_window[*limage][NL]))
              nl = out_window[i][SL] + out_window[i][NL] - 
                   out_window[*limage][SL];
            else
              nl = out_window[*limage][NL];
          }
          else
          {
          sl = out_window[i][SL] - out_window[*limage][SL];
          if ((out_window[i][SL] + out_window[i][NL]) < 
              (out_window[*limage][SL] + out_window[*limage][NL]))
            nl = out_window[i][NL];
          else
            nl = out_window[*limage][SL] + out_window[*limage][NL] - 
                 out_window[i][SL];
          }
          for (j = sl; j < sl+nl; j++)
            laver[j] = TRUE;
        }
      }
    }
  } 
}
