#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

extern	long    curcat;                /* message utility         	*/
extern	long    curent;                /* parameters for processing 	*/

struct  PARBLK block;		       /* TAE block			*/
struct  DDR ddr;		       /* DDR structure			*/
long    bands[MAXIMG][MAXBND+1];       /* bands array			*/
long    nbands[MAXIMG];		       /* number of bands per image	*/
struct FDESC *    fdesc[MAXIMG];              /* input file descriptors  	*/
struct FDESC *    fdescout[2];                /* output file descriptor  	*/
float   *inbuf;                        /* image input buffer      	*/
float   *outbuf;                       /* image output buffer     	*/
struct GDESC *    gdesc;               /* group descriptor        	*/
struct GDESC *    gdescout;		       /* output group descriptor	*/
long    ipoint[MAXIMG*MAXBND+1];       /* input array offset		*/
long    opoint[MAXBND];		       /* output array offset		*/

static	long one = 1;		       /* constant of one		*/

int main
(void) /* conj */
{
/*

NAME: CONJ

FUNCTION:
  Initialize LAS
  Get input and output parameters
  Open input and output images
  Process image
  Update history and ddr files

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    5.0	      21-SEP-92	  T. Mittan	    Modified include files to eliminate
					    warnings on link
    4.0       06-FEB-90   B. ALLAM, STX     Fixed problem in process.c
    3.0       30-OCT-89   B. ALLAM, STX
    2.0       29-MAR-84   S. KONDAL, SAR
    1.0       13-DEC-83   J. BAY, SASC
*/

char	hosin[MAXIMG][CMLEN],	/* Buffer for input host names */
	lasin[MAXIMG][CMLEN],	/* Buffer for input las names */
	lasout[1][CMLEN],		/* Buffer for output las names */
	hosout[1][CMLEN]; 		/* Buffer for output host names */
long	wind[MAXIMG][4], 	/* window specifications */
	nmrimg, 		/* Number of input images */
        mnmx,			/* min/max value */
        std_out, 		/* flag parameter */
	totbnd, 		/* total no. of bands */
        img,  	/* band number */
        msgtim;			/* Time processing message time interval      */
double  maxout[MAXBND], minout[MAXBND]; /* min and max samples */

/*
Initialize LAS
--------------
*/
c_inlas (&block, &std_out); /*** NEWLAS call ***/

/*
Get input and ouput parameters
------------------------------
*/
getpar(hosin, lasin, lasout, hosout, wind, &totbnd, &mnmx, &msgtim,
       &nmrimg);

/*
For each image 
--------------
*/
for (img=0; img < nmrimg; img++)
  {
  c_getddr(hosin[img], &ddr);
/*
 convert from complex to real pixel count

*/
  wind[img][NS] = 2 * wind[img][NS];

/*
 handle default window

*/
  if (wind[img][SS] == 0) 
   {
   wind[img][SS] = 1;
   }
  else
   {

/*
 convert starting pixel position from complex to real

*/
   wind[img][SS] = 1 + 2 * (wind[img][SS] - 1);
   }  
  if (wind[img][SL] == 0) wind[img][SL] = 1;
  if (wind[img][NS] == 0) wind[img][NS] = ddr.ns; 
  if (wind[img][NL] == 0) wind[img][NL] = ddr.nl;
/*
 form intersection of input image windows

*/
  wind[img][NS] = min(wind[img][NS], ddr.ns);
  wind[img][NL] = min(wind[img][NL], ddr.nl);
  }

/*
setup input and output file specifications
------------------------------------------
*/
filspec (hosin, hosout[0], wind, nmrimg, totbnd);

/*
process image
-------------
*/
process(wind, msgtim, nmrimg);

/*
 update output file histories 
 ----------------------------
*/
c_hstout(&block, hosout[0], hosin, &nmrimg, " ");

/*
call minmax instead of using above estimates
--------------------------------------------
*/
c_minmax (hosin, &nmrimg, wind, bands, nbands, minout, maxout);


/*
 update output image ddr min max fields
 --------------------------------------
*/
  if (mnmx)  
    c_svmnmx(hosout[0], &totbnd, minout, maxout, "YES");

/*
 completion messages
 -------------------
*/
c_complt("CONJ", lasout, &one);

return EXIT_SUCCESS;

}
