#include "conj.h"
#include "protos.h"

void filspec
(
    char hostin[][CMLEN],         /*input image host name descriptors */
    char hosout[],		       /* output image descriptor */
    long wind[][4],               /*input image windows           */
    long nimages,                 /*no. of input images           */
    long totbnd		       /* total no. of bands */
)
{
/*
**********************************************************************
NAME: FILSPEC

PROGRAM HISTORY:
  Version	Date		Author		Request
  -------	----		------		-------
    
    3.0       30-OCT-89   B. ALLAM, STX
    2.0       29-MAR-84   S. KONDAL, SAR
    1.0       13-DEC-83   J. BAY, SASC

FUNCTION: sets up the image i/o interface

ALGORITHM:
      set up input file specifications

***********************************************************************
*/

static	long	opt[3]={0,0,0};	  /* optional parameter array for EOPENS */

   long    stat,                  /* flag for EOPENS procedure */
           access,                /* file specifications       */
           type,                  /* data type                 */
           n, nib,                /* counter variables         */
           obufsiz, bufsiz;       /* image i/o buffer size     */
   double  y_inc, x_inc;          /* line and sample increments*/

static long zero = 0;
static long one = 1;
static long fatal_err = -1;

/*
 set up input file group
 -----------------------
*/

   access = IREAD;
   type = EREAL;
   for (n = 0, nib = 0; n < nimages ; n++)
   {
     stat = c_eopens(&fdesc[n], hostin[n], bands[n], &wind[n][SL],
                     &wind[n][SS], &wind[n][NL], &wind[n][NS], &ipoint[nib],
                     &access, &type, &zero, opt);
     if (stat != E_SUCC)
       c_errmsg("Error opening input image", "CONJ_input", &fatal_err);
     nib += 1; 
    }
   fdesc[++n] = 0;
/*
 set up output file 
 ------------------
*/

   access = IWRITE;
   n = 0; 
   stat = c_eopens(&fdescout[n], hosout, &totbnd, &one, &one, &wind[0][NL],
                   &wind[0][NS], opoint, &access, &type, &zero, opt);
   if (stat != E_SUCC)
     c_errmsg("Error opening output image", "CONJ-out", &fatal_err);
   fdescout[++n] = 0;
/*
set up group containing input and output bands
----------------------------------------------
*/
   if ((inbuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
     c_errmsg("Error allocating dynamic memory", "CONJ_alloc", &fatal_err);

   if ((outbuf = (float *) c_ealloc(fdescout, &obufsiz)) == NULL)
     c_errmsg("Error allocating dynamic memory", "CONJ_alloc", &fatal_err);

   if (c_egroup(fdesc, &gdesc,(unsigned char *) inbuf, &bufsiz) != E_SUCC)
     c_errmsg("Error grouping image bands", "CONJ_group", &fatal_err);

   if (c_egroup(fdescout,&gdescout,(unsigned char *) outbuf,&obufsiz) != E_SUCC)
     c_errmsg("Error grouping image bands", "CONJ_group", &fatal_err);
/*
update DDR
----------
*/
y_inc = 1;
x_inc = 1;
c_upddr(&block, hostin, hosout, &nimages, wind, bands, nbands, 
        &y_inc, &x_inc, &totbnd, &zero);

}
