/*
*************************************************************************
NAME: aloc_buf

PURPOSE: allocate memory for a 'lines' by 'samps' buffer.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/89   T. Baltzer   
    2            5/99   L. Johnson         Replaced hardwired data sizes with
                                           defined values. Added EDOUBLE type.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

*****************************************************************
	arguments:
	name		description
	----		-----------
	 lines		number of lines in the buffer
	 samps		number of samples in the buffer
	 dtype		the data type of the buffer
	 buff		pointer to buffer

************************************************************************
*/
#include <stdlib.h>
#include "las.h"
#include "constretch.h"


int  aloc_buf
(
    long lines,    /* number of lines in the buffer                 */
    long samps,    /* number of samples in the buffer               */
    long dtype,    /* the data type of the buffer                   */
    char **buff    /* pointer to buffer                             */
)
{
  long inf_err = 1;               /* error status flag           */


  /* NOTE: don't bother type casting the pointer 'buff'.  Upon calling    */
  /*       aloc_buf(), the user can map this pointer into the appropriate */
  /*       pointer type.  Just make sure that the correct number of bytes */
  /*       are allocated.                                                 */

  switch (dtype)
     {
     case EBYTE:
        *buff = (char *)malloc(lines*samps*EBYTESIZE);
        break;

     case EWORD:
        *buff = (char *)malloc(lines*samps*EWORDSIZE);
        break;

     case ELONG:
        *buff = (char *)malloc(lines*samps*ELONGSIZE);
        break;

     case EREAL:
        *buff = (char *)malloc(lines*samps*EREALSIZE);
        break;

     case EDOUBLE:
        *buff = (char *)malloc(lines*samps*EDOUBLESIZE);
        break;
     }

  if (*buff == NULL)
    {
    c_errmsg("Error allocating dynamic memory.","constretch-alloc",&inf_err);
    c_errmsg("Fatal error encountered","constretch-fatal",&inf_err);
    return(0);       /* error return status */
    }

  return(1);    /* successful return status */

}               /* end of aloc_buf()    */
