#ifndef _CONSTRETCH_H_
#define _CONSTRETCH_H_

/* include files, global parameters, and definitions for CONSTRETCH   */

#include "imageio.h"


static	long	inf_err = 1;

#define	MAXRAM	  20971520	/* maximum bytes of available RAM     */


/* Prototypes
   ----------*/
int  aloc_buf
(
    long lines,    /* number of lines in the buffer                 */
    long samps,    /* number of samples in the buffer               */
    long dtype,    /* the data type of the buffer                   */
    char **buff    /* pointer to buffer                             */
);
int  cellmnmx
(
    void *ibuff,             /* input buffer (input)                        */
    long itype,              /* data type (input)                           */
    double *pxminc,          /* min pixel value arrays (output)             */
    double *pxmaxc,          /* max pixel value arrays (output)             */
    long nsbuf,              /* number of samples in input buffer (input)   */
    long *nlc,               /* number of rows in the cell (input)          */
    long *nsc,               /* number of samples in the cell (input)       */
    long ncrows,             /* number of cell rows (input)                 */
    long nccols,             /* number of cell columns (input)              */
    long totccols,           /* total number of cell rows in frame (input)  */
    long nmask,              /* number of mask values(input)                */
    double maskval[][2]      /* mask values(input)                          */
);
int  err_sbuf
(
    long nli,     /* number of lines in the image window (input)        */
    long nsi,     /* number of samples in the image window (input)      */
    long nlk,     /* number of lines in the kernel (input)              */
    long nsk,     /* number of samples in the kernel (input)            */
    long nlb,     /* number of lines in the boundary (input)            */
    long nsb,     /* number of samples in the boundary (input)          */
    long dtype,   /* data type of buffer (input)                        */
    long maxram   /* maximum ram in bytes (input)                       */
);
int extreme
(
    void *obuff,      /* output buffer (input)                              */
    long dtype,       /* input data type (input)                            */
    long nsobuf,      /* number or samples in obuff (input)                 */
    long nlavo,       /* number of lines of valid data in obuff (input)     */
    long nsavo,       /* number of samples of valid data in obuff (input)   */
    long oband,       /* output band number (input)                         */
    double maxbnd[],  /* array of maximum values (input/output)             */
    double minbnd[]   /* array of minimum values (input/output)             */
);
int  free_buf
(
    unsigned char **buff  /* pointer to buffer   */
);
void getpar
(
    struct PARBLK *parblk,   /* TAE parameter block                         */
    char hosin[][CMLEN],     /* host names of input images                  */
    char taeout[],           /* tae name of output image                    */
    char hosout[],           /* host name of output image                   */
    long *totbnd,            /* total number of output bands                */
    long bands[][MAXBND+1],  /* band specification                          */
    long nbands[],           /* number of bands in each image               */
    long *nl,                /* number of lines to be processed             */
    long *ns,                /* number of samples per line to be processed  */
    long window[][4],        /* window specification                        */
    long *nmrimg,            /* number of input images                      */
    long *nrows,             /* number of rows in input images              */
    long *ncols,             /* number of columns in input images           */
    long *itype,             /* data type of input image                    */
    long *otype,             /* output data type                            */
    double *maxval,          /* maximum value for contrast stretch          */
    double *minval,          /* minimum value for contrast stretch          */
    long *min_max,           /* minmax option flag                          */
    long *msgtim,            /*                                             */
    long *bdrflag,           /* contrast stretch border flag                */
    double *thresh,          /* contrast range threshold                    */
    long *nmask,             /* number of mask values                       */
    double maskval[][2]      /* mask values                                 */
);
int  nodemnmx
(
    double *pxminc,  /* min pixel values for cells                          */
    double *pxmaxc,  /* max pixel values for cells                          */
    double *pxminn,  /* min pixel values for nodes                          */
    double *pxmaxn,  /* max pixel values for nodes                          */
    long ncrows,     /* number of cell rows                                 */
    long nccols,     /* number of cell columns                              */
    long bdrflag,    /* contrast stretch border flag                        */
    long dtype,      /* input data type                                     */
    double thresh    /* contrast range threshold                            */
);
int pro_blk
(
    long proflag,                 /* I: Process flag:
                                          0 = get min and max pixel values
                                          1 = Compute the new pixel values  */
    long nmrimg,                  /* I: number of images                    */
    long window[MAXIMG][4],       /* I: window of image                     */
    long bands[MAXIMG][MAXBND+1], /* I: band numbers to be processed        */
    long nbands[MAXIMG],          /* I: number of bands in each image       */
    long totbnd,                  /* I: total number of bands               */
    char hosin[MAXIMG][CMLEN],    /* I: input image host name               */
    char hosout[CMLEN],           /* I: output image host name              */
    long min_max,                 /* I: min, max flag from TAE land         */
    double maxbnd[MAXBND+1],      /* O: array of max pixel values           */
    double minbnd[MAXBND+1],      /* O: array of min pixel values           */
    long nicols,                  /* I: number of cell columns in image     */
    unsigned char *ibuff,         /* I: data buffer to read                 */
    unsigned char *obuff,         /* I: data buffer to write                */
    long itype,                   /* I: input data type                     */
    long otype,                   /* I: output data type                    */
    long *nlib,                   /* I: number of lines in 'ibuff'          */
    long *nsib,                   /* I: number of samples in 'ibuff'        */
    long nlb,                     /* I: number of lines in the boundary     */
    long nsb,                     /* I: number of samples in the boundary   */
    long nli,                     /* I: number of lines in the image        */
    long nsi,                     /* I: number of samples in the image      */
    double *pxminc,               /* I/O: min pixel values for cells        */
    double *pxmaxc,               /* I/O: max pixel values for cells        */
    double *pxminn,               /* I: min pixel values for nodes          */
    double *pxmaxn,               /* I: max pixel values for nodes          */
    long nicells,                 /* I: total number of cells per frame     */
    long ninodes,                 /* I: total number of nodes per frame     */
    long *nlc,                    /* I: number of lines per cell            */
    long *nsc,                    /* I: number of samples per cell          */
    long ncrb,                    /* I: number of cell rows in buffer       */
    long nccb,                    /* I: number of cell columns in buffer    */
    long ncrb1,                   /* I: num. of cell rows in last buffer row*/
    long nccb1,                   /* I: num. of cell cols in last buffer col*/
    long totlb,                   /* I: tot. num. of buffer blocks in y dir */
    long  totsb,                  /* I: tot. num. of buffer blocks in x dir */
    double maxval,                /* I: maximum value for contrast stretch  */
    double minval,                /* I: minimum value for contrast stretch  */
    long nmask,                   /* I: number of mask values               */
    double maskval[][2]           /* I: mask values                         */
);
int  pro_img
(
    long nmrimg,                  /* I: number of images                    */
    long window[MAXIMG][4],       /* I: window of image                     */
    long bands[MAXIMG][MAXBND+1], /* I: band numbers to be processed        */
    long nbands[MAXIMG],          /* I: number of bands in each image       */
    long totbnd,                  /* I: total number of bands               */
    char hosin[MAXIMG][CMLEN],    /* I: input image host name               */
    char hosout[CMLEN],           /* I: output image host name              */
    long itype,                   /* I: input data type                     */
    long otype,                   /* I: output data type                    */
    double maxval,                /* I: maximum value for contrast stretch  */
    double minval,                /* I: minimum value for contrast stretch  */
    long min_max,                 /* I: min, max flag from TAE land         */
    double maxbnd[MAXBND+1],      /* O: array of max pixel values           */
    double minbnd[MAXBND+1],      /* O: array of min pixel values           */
    long nirows,                  /* I: number of cell rows in image        */
    long nicols,                  /* I: number of cell columns in image     */
    long bdrflag,                 /* I: contrast stretch border flag        */
    double thresh,                /* I: contrast range threshold            */
    long nmask,                   /* I: number of mask values               */
    double maskval[][2]           /* I: mask values                         */
);
int  size_buf
(
    long *nli,     /* I: number of lines in the input image                 */
    long *nsi,     /* I: number of samples in the input image               */
    long *nlk,     /* I: number of lines in the kernel                      */
    long *nsk,     /* I: number of samples in the kernel                    */
    long *nlb,     /* I: number of lines in the boundary                    */
    long *nsb,     /* I: number of samples in the boundary                  */
    long *itype,   /* I: input data type                                    */
    long *otype,   /* I: output data type                                   */
    long *maxram,  /* I: maximum buffer memory size in bytes                */
    long *nlib,    /* O: number of lines in the input buffer                */
    long *nsib,    /* O: number of samples in the input buffer              */
    long *nlob,    /* O: number of lines in the output buffer               */
    long *nsob     /* O: number of samples in the output buffer             */
);
int strtch
(
    void *ibuff,         /* input buffer (input)                            */
    long itype,          /* input data type (input)                         */
    void *obuff,         /* output buffer (output)                          */
    long otype,          /* output data type (input)                        */
    double *pxminn,      /* min pixel values for nodes (input)              */
    double *pxmaxn,      /* max pixel values for nodes (input)              */
    long nsbuf,          /* number of samples in input buffer (input)       */
    long *nlc,           /* number of rows in the cell (input)              */
    long *nsc,           /* number of samples in the cell (input)           */
    long ncrows,         /* number of cell rows                             */
    long nccols,         /* number of cell columns                          */
    long totccols,       /* total number of cell rows in frame (input)      */
    double maxval,       /* maximum value for contrast stretch              */
    double minval,       /* minimum value for contrast stretch              */
    long nmask,          /* number of mask values(input)                    */
    double maskval[][2]  /* mask values(input)                              */
);

#endif /* _CONSTRETCH_H_ */
