/*
************************************************************************
NAME: err_sbuf

PURPOSE: error check the input parameters to size_buf()

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           01/90   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

*****************************************************************
	arguments:
name		description
----		-----------
nli		number of lines in the image window (input)
nsi		number of samples in the image window (input)
nlk		number of lines in the kernel (input)
nsk		number of samples in the kernel (input)
nlb		number of lines in the boundary (input)
nsb		number of samples in the boundary (input)
dtype		data type of buffer (input)
maxram		maximum ram in bytes (input)

************************************************************************
*/

#include "las.h"
#include "constretch.h"


int  err_sbuf
(
    long nli,     /* number of lines in the image window (input)        */
    long nsi,     /* number of samples in the image window (input)      */
    long nlk,     /* number of lines in the kernel (input)              */
    long nsk,     /* number of samples in the kernel (input)            */
    long nlb,     /* number of lines in the boundary (input)            */
    long nsb,     /* number of samples in the boundary (input)          */
    long dtype,   /* data type of buffer (input)                        */
    long maxram   /* maximum ram in bytes (input)                       */
)
{
  long	nonfatal = 1; /* nonfatal error status           */


  /* Begin by checking the values of the individual parameters.     */

  if ( nli  < 1 || nsi  < 1 )
     {
     c_errmsg("Invalid window specified","constretch-window",&nonfatal);
     c_errmsg("Fatal error encountered","constretch-fatal",&nonfatal);
     return(E_FAIL);
     }
  if ( nlb < 0 || nsb < 0 || nlk  < 1 || nsk  < 1 )
     {
     c_errmsg("Invalid kernel or boundary","constretch-kerbnd",&nonfatal);
     c_errmsg("Fatal error encountered","constretch-fatal",&nonfatal);
     return(E_FAIL);
     }
  if ( dtype < 0 || dtype > 4 )
     {
     c_errmsg("Invalid data type","constretch-dtype",&nonfatal);
     c_errmsg("Fatal error encountered","constretch-fatal",&nonfatal);
     return(E_FAIL);
     }
  if ( maxram < 1 )
     {
     c_errmsg("Invalid maximum RAM specified","constretch-maxram",&nonfatal);
     c_errmsg("Fatal error encountered","constretch-fatal",&nonfatal);
     return(E_FAIL);
     }

  return(E_SUCC);

}    /*  end of err_sbuf()   */
