/*
******************************************************************************
NAME: extreme

PURPOSE: update the arrays storing the extreme (min/max) values of the
         image (one array element per band)

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1	        06/99    L. Johnson


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

******************************************************************************
	arguments:
	name		description
	----		-----------
	obuff		output buffer (input)
        dtype           input data type (input)
	nsobuf		number or samples in obuff (input)
	nlavo		number of lines of valid data in obuff (input)
	nsavo		number of samples of valid data in obuff (input)
	oband		output band number (input)
	maxbnd		array of maximum values (input/output)
	minbnd		array of minimum values (input/output)

******************************************************************************
*/

#include <stdio.h>
#include "las.h"
#include "constretch.h"


int extreme
(
    void *obuff,      /* output buffer (input)                              */
    long dtype,       /* input data type (input)                            */
    long nsobuf,      /* number or samples in obuff (input)                 */
    long nlavo,       /* number of lines of valid data in obuff (input)     */
    long nsavo,       /* number of samples of valid data in obuff (input)   */
    long oband,       /* output band number (input)                         */
    double maxbnd[],  /* array of maximum values (input/output)             */
    double minbnd[]   /* array of minimum values (input/output)             */
)
{
  long	ii,jj;                       /* counters                            */
  unsigned char	*badd;               /* addresses for different data types  */
  short *wadd;
  long  *ladd;
  float *radd;
  long	obandi;	                     /* output band index                   */
  long  offset;                      /* address offset                      */


  /* Arrays are zero-based. Band numbers are one-based.
  -----------------------------------------------------*/
  obandi = oband - 1;

  /* Must consider data type for pointer incrementation.
  ------------------------------------------------------*/
  switch (dtype)
      {
      case EBYTE:
          for (ii=0; ii<nlavo; ii++)         /* Loop over lines in obuff.   */
              {
              offset = ii*nsobuf;
              badd = (unsigned char *)obuff + offset;
              for (jj=0; jj<nsavo; jj++)     /* Loop over samples in obuff. */
                  {
                  minbnd[obandi] = min(minbnd[obandi], (double)*badd);
                  maxbnd[obandi] = max(maxbnd[obandi], (double)*badd);
                  badd++;
                  }    /* sample loop */
              }    /* line loop   */
          break;

      case EWORD:
          for (ii=0; ii<nlavo; ii++)         /* Loop over lines in obuff.   */
              {
              offset = ii*nsobuf;
              wadd = (short *)obuff + offset;
              for (jj=0; jj<nsavo; jj++)     /* Loop over samples in obuff. */
                  {
                  minbnd[obandi] = min(minbnd[obandi], (double)*wadd);
                  maxbnd[obandi] = max(maxbnd[obandi], (double)*wadd);
                  wadd++;
                  }    /* sample loop */
              }    /* line loop   */
          break;

      case ELONG:
          for (ii=0; ii<nlavo; ii++)         /* Loop over lines in obuff.   */
              {
              offset = ii*nsobuf;
              ladd = (long *)obuff + offset;
              for (jj=0; jj<nsavo; jj++)     /* Loop over samples in obuff. */
                  {
                  minbnd[obandi] = min(minbnd[obandi], (double)*ladd);
                  maxbnd[obandi] = max(maxbnd[obandi], (double)*ladd);
                  ladd++;
                  }    /* sample loop */
              }    /* line loop   */
          break;

      case EREAL:
          for (ii=0; ii<nlavo; ii++)         /* Loop over lines in obuff.   */
              {
              offset = ii*nsobuf;
              radd = (float *)obuff + offset;
              for (jj=0; jj<nsavo; jj++)     /* Loop over samples in obuff. */
                  {
                  minbnd[obandi] = min(minbnd[obandi], (double)*radd);
                  maxbnd[obandi] = max(maxbnd[obandi], (double)*radd);
                  radd++;
                  }    /* sample loop */
              }    /* line loop   */
          break;


      default:
          c_errmsg("Invalid data type.", "constretch-dtype", &inf_err);
          c_errmsg("Fatal error encountered", "constretch-fatal", &inf_err);
          return (E_FAIL);
      }    /* data type switch */

  return(E_SUCC);

}    /* end of extreme() */


