#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"
#define TERM 0
#define LP   1
#define FIL  2

struct PARBLK block;                   /* TAE parameter block     */

FILE    *fd;                           /* output file descriptor  */

float   *expec_table;

long    *contingency_table;
struct FDESC *    fdesc[3];                      /* file descriptors        */
struct GDESC *    gdesc;                         /* group descriptor        */
long    ipoint[2 * MAXBND + 1];        /* offsets into image i/o  */
long    msgtim;                        /* message interval        */
long    num_outfiles;                  /* number of output files  */
long    print[3];                      /* print options           */

char    out_files[CMLEN];              /* output file names       */
static char    *prog_name = "CONTABLE";/* program name            */
unsigned char   *iobuf;                /* image i/o buffer        */

extern	long    curent;                /* parameters for processing */
extern	long    curcat;                /* message utility         */

static  long    zero = 0;	       /* constant of zero	  */
static  long    one = 1;	       /* constant of one	  */

/*
********************************************************************

NAME: CONTABLE                      

PURPOSE: Constructs a contingency table from two classified images

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    5	       9/21/92	  T. Mittan	      Modified include file to eliminate
				 	      warnings on compile
    4          6/06/89    K. Narayanan        Removed calls that sent outputs
                                              to session log and caused beeping
    3         12/05/88    K. Narayanan        Conversion to 5.0
    2          5/14/87    N. Deodhar 
    1          9/16/83    C. Gurney/
   		          K. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get las parameters
   Call FILSPEC to set up image for i/o
   Call TABCALC to read images & construct contingency table
   Call OUTPUT to list results to terminal, lineprinter or file
   Call C_COMPLT to issue a completion message

***********************************************************************
*/
int main
(void) 
{

float	*expec[256];             /* expected values                  */

long	colind[256];             /* number of non-zero columns       */
long	*contab[256];            /* contingency table                */
char	expect_flag[4];          /* leave/omit expected values flag  */
long	image_band_num[2][MAXBND + 1];
                                 /* band numbers in each input image */
long	nc, nr, npix;            /* column, row index, # pixels      */
long	rowind[256];             /* number of non-zero rows          */
long	sum1[256];               /* row sums                         */
long	sum2[256];               /* column sums                      */
long	wind[2][4];              /* input image windows              */

char	hos_in[2][CMLEN];        /* input images host names          */
char	las_in[2][CMLEN];        /* input images las names           */
char	print_filename[1][CMLEN];   /* output file name                 */

/*
get parameters
--------------
*/
getpar (las_in,  hos_in,  print_filename[0], image_band_num, wind, expect_flag);

/*
set up images for i/o
---------------------
*/
filspec (hos_in, image_band_num, wind);

/*
allocate dynamic storage
------------------------
*/
dyn_alloc (contab, expec);

/*
form contingency table
----------------------
*/
tabcalc (&npix, wind, contab, expec, sum1, sum2, colind, rowind, &nc, &nr);

/*
write out results
-----------------
*/
output (contab, expec, sum1, sum2, las_in,  expect_flag, colind, 
        rowind, nc, nr);

/*
free dynamic storage
--------------------
*/
dyn_free ();

/*
inform user about successful completion
---------------------------------------
*/
if (print[FIL])
   c_complt (prog_name, print_filename, &one); 
else
   {
   strcpy(print_filename[0], "\0");
   c_complt (prog_name, print_filename, &zero); 
   }

return EXIT_SUCCESS;
  
}
