#include "contable.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char las_in[][CMLEN],     /* input image las names            */
    char hos_in[][CMLEN],     /* input image host names           */
    char print_filename[],    /* print output file name           */
    long image_band_num[][MAXBND + 1], /* band numbers in each    */
    long wind[][4],           /* input image windows              */
    char expect_flag[]       /* expectation parameter            */
)

/*
************************************************************************
NAME: GETPAR

PURPOSE: Receives and checks las parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    3         12/05/88    K. Narayanan     Conversion to 5.0
    2         05/05/86    B. Kovalick 
    1          9/16/83    C. Gurney/
   		          K. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call INLAS to initialize the program
   Call C_GETIN to get input image names
   If not single band images issue error message and abort
   Get other parameters
   If requested device is printer or file open a file for output

*********************************************************************
*/      
{
static char    *prog_name = "CONTABLE";/* program name            */

long fatal_err = -1;                  /* fatal error flag */
long one = 1;                         /* constant of one  */

struct  VARIABLE   *var;              /* pointer to a variable in TAE block */

long	flag[1][2],                    /* flag for subwindows                */
	i,
	num_image_bands,            /* number of bands in each input image */
	num_inbands,                /* total number of input image bands   */
	num_inimages,               /* number of input images           */
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

char	fileutil_filename[CMLEN];   /* temp file for fileutil             */


/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);
/*
get and validate input image parameters
---------------------------------------
*/
for (i = 0; i < 2; i++)
   {
   stat = c_getin (&block, "IN", &i, las_in[i], hos_in[i], &num_inimages, 
                   wind[i], image_band_num[i], &num_image_bands, &num_inbands, 
                   &one, &same_window, flag);

   if (stat != E_SUCC) 
      c_errmsg ("Error accessing the input file name", "contable-in", 
                 &fatal_err);

   if (num_inbands  != 1) 
      c_errmsg( "Images must be single band", "contable-bands", &fatal_err);
   }

if (wind[0][NS] != wind[1][NS] || wind[0][NL] != wind[1][NL])
   c_errmsg ("Input image sizes do not match", "contable-size", &fatal_err);

var = p_find (&block, "EXVALFLG");
strcpy (expect_flag, SVAL(*var, 0));

/*
Get print option and open print file if necessary
-------------------------------------------------
*/
strcpy (fileutil_filename, prog_name);
stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], 
          fileutil_filename, print_filename);
if (stat != E_SUCC)
   c_errmsg("Fatal error encountered","contable-fatal",&fatal_err);

if (print[LP] || print[FIL]) 
   {
   fd = fopen (fileutil_filename, "w");
   num_outfiles = 1;
   strcpy (out_files[0], fileutil_filename);
   }
else
   {
   num_outfiles = 0;
   fd = NULL;
   }
/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

}
