#include "contable.h"
#include "util.h"
#include "protos.h"

void output
(
    long *contab[],            /* contingency values                 */
    float *expec[],             /* expected values                    */
    long sum1[256],            /* row sums in table                  */
    long sum2[256],            /* column sums in table               */
    char las_in[][CMLEN],      /* input images las_in names          */
    char expect_flag[],        /* expectation flag                   */
    long colind[256],          /* number of non-zero columns         */
    long rowind[256],          /* number of non-zero rows            */
    long nc,                   /* column indices                     */
    long nr                   /* row indices                        */
)
/*
******************************************************************************

NAME: OUTPUT                        

PURPOSE: Outputs contingency table to terminal, line printer, or file

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    4          6/06/89    K. Narayanan     Removed calls that sent output to
                                           session log and caused beeping
    3          8/04/88    K. Narayanan
    2          5/14/87    N. Deodhar 
    1          9/16/83    C. Gurney/
   		          K. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Determine the number of columns based on the output device
   While the starting column is less than or equal to the no. of cols
      Write page headings
      For the number of rows to be printed
         Set row index
         Compute the running row sum
         For the number of columns
            Set the column index 
            Put one column of observed info into a print line
         Print the row
         If this is the last column print the row sum
         If the user wants expected values
            For the number of columns
               Set the column index 
               Put one column of expected info into a print line
            Print the row
      Print the column totals
      If all the columns have been printed print the overall row sum
      Compute the mean, std deviation, skew, and kurtosis 
      Put the mean, std dev, skew, and kurtosis into the print line
      Print the line

**********************************************************************
*/
{
# define term_width 72
# define lp_width 120

double	var;
double	var2;
double	var3;
double	var4;

float	mean[256];         /* column means                  */
float	stdev[256];        /* column standard deviations    */
float	skew[256];         /* column skewness               */
float	kurt[256];         /* column kurtosis               */
float	temp, temp2, temp3, temp4;

long	nclass;
long	total; 
long	first;
long	ibeg, icols, istart;
long	ncols, icnt;
long	nindex;
long	ii, in, ind, j, k, n;
long	off;

char	pline[lp_width];
char	buf[ERRLEN] ;
char	line[lp_width + 5]; 
char	*tbuf;


/*
set values for table outline
----------------------------
*/
nclass = nc > nr ? nc : nr;
icnt = 0;
off = 5;
strcpy (line, "     + ");
for (k = off+2; k < lp_width+off; k += 2)
   {
   line[k] = '-'; 
   line[k+1] = ' ';
   icnt++;
   if (icnt == 5)
      {
      line[k] = '+';
      icnt=0;
      }  
   }  

/*
set start values for printing - if only LP is requested set max no. of
columns and width; otherwise set them all equal to max terminal limits
----------------------------------------------------------------------
*/
if (print[LP] && !print[TERM] && !print[FIL])
   {                                          /*  to line printer */
   ncols = 11;
   }
else                          
   {
   ncols = 6;
   }
istart = 0;
icols = ncols + istart - 1;
/*
write out set of columns in table
---------------------------------
*/
total = 0;
while (istart  <=  nc)
   {
   if (icols > nc)
      {
      icols = nc;
      ncols = nc - istart + 1;
      }  
   nindex = ncols * 10;
   line[nindex+off+1] = '\0';

   tbuf = "\f***Contingency table produced by CONTABLE***"; 
   c_txtout (tbuf, print, fd);

   sprintf (buf, "\n First input image : %s", las_in[0]);
   c_txtout (buf, print, fd);

   sprintf (buf, " Second input image: %s\n", las_in[1]);
   c_txtout (buf, print, fd);

   tbuf = " ...Rows formed by first image, columns by second image";
   c_txtout (tbuf, print, fd);

   tbuf = " ...First row in table = observed values; \
second row = expected values.\n\n";
   c_txtout (tbuf, print, fd);

   for (k = istart; k <= icols; )
      {
      sprintf (buf, "      ");
      for (j = 0; j < ncols; j++)
         {
         sprintf( &buf[strlen( buf)], "%8d  ", colind[k]);
         ++k;
         }
      if (icols == nc) sprintf( &buf[strlen( buf)], "  Rowsum");
      c_txtout( buf, print, fd);
      }  

   c_txtout (line, print, fd);

   for (in = 0; in <= nr; in++)
      {
      n = rowind[in];
      if (istart == 0) total += sum1[n];
      first = TRUE;
      ibeg = off;
      strcpy (pline, "     ");
      for (k = istart; k <= icols; k++)
         {
         ind = colind[k];
         if (first) pline[ibeg] = '!';
         sprintf (&pline[ibeg+1], "%8d !", contab[n][ind]);
         ibeg += 10;
         first = FALSE;
         }  
      c_txtout (pline, print, fd);
      sprintf (buf, "%4d !", n);
      for (k = 0; k < ncols; k++)
         sprintf (&buf[strlen (buf)], "%s", "         !");
      if (icols  ==  nc) 
         sprintf (&buf[strlen (buf)], "%8d", sum1[n]);
      c_txtout (buf, print, fd);

/*    print expected values if requested
      ----------------------------------
*/
      if (expect_flag[0] == 'Y')
         {
         ibeg = off;
         first = TRUE;
         strcpy (pline, "     ");
         for (k = istart; k <= icols; k++)
            {
            ind = colind[k];
            if (first) pline[ibeg] = '!';
            sprintf (&pline[ibeg+1], "%8.2f !", expec[n][ind]);
            ibeg = ibeg+10;
            first = FALSE;
            }  
         c_txtout (pline, print, fd);
         }  
      c_txtout (line, print, fd);
      }  
/*
write out the total in each column
----------------------------------
*/
   pline[0] = '\0';
   ibeg = off+2;
   first = TRUE;
   for (k = istart; k <= icols; k++)
      {
      ind = colind[k];
      if (first) strcpy (pline, "\nTOTAL ");
      sprintf (&pline[ibeg], "%8d  ", sum2[ind]);
      ibeg = ibeg+10;
      first = FALSE;
      }  
   if (icols  ==  nc) 
      sprintf (&pline[ibeg], "%8d  ", total);
   c_txtout (pline, print, fd);                        
/*
compute the mean, std. dev, skew and kurtosis for each column
-------------------------------------------------------------
*/
   for (k = istart; k <= icols; k++)
      {
      j = colind[k];
      mean[j] = ((float) sum2[j]) / (float) (nclass+1);
      var2 = 0.0;
      var3 = 0.0;
      var4 = 0.0;
      for (in = 0; in <= nclass; in++)
         {
         n = rowind[in];
         temp = ((float) contab[n][j]) - mean[j];
         if (temp  !=  0.0) 
            {
            temp2 = temp*temp;
            temp3 = temp*temp2;
            temp4 = temp*temp3;
            var2 = var2+temp2;
            var3 = var3+temp3;
            var4 = var4+temp4;
            }  
         }  
      var = var2/nclass;
      var2 = var2/(nclass+1);
      var3 = var3/(nclass+1);
      var4 = var4/(nclass+1);
      if (var2  !=  0.0) 	       
         {
         stdev[j] = sqrt (var);
         skew[j] = var3/pow (var2, 1.5);
         kurt[j] = var4/(var2*var2)-3.0;
         }
      else
         {
         stdev[j] = 0.0;
         skew[j] = 0.0;
         kurt[j] = 0.0;
         }  
      }  
/*
write the values to the output
------------------------------
*/
   for (ii = 1; ii <= 4; ii++)
      {
      ibeg = off+2;
      first = TRUE;
      for (k = istart; k <= icols; k++)
         {
         ind = colind[k];
         if (ii == 1)
            {
            if (first) strcpy (pline, "\nMEAN  ");
            sprintf (&pline[ibeg], "%8.2f  ", mean[ind]);
            }
         else if (ii == 2)
            {
            if (first) strcpy (pline, "\nSTDEV ");
            sprintf (&pline[ibeg], "%8.2f  ", stdev[ind]);
            }
         else if (ii == 3)
            {
            if (first) strcpy (pline, "\nSKEW  ");
            sprintf (&pline[ibeg], "%8.2f  ", skew[ind]);
            }
         else
            {
            if (first) strcpy (pline, "\nKURT  ");
            sprintf (&pline[ibeg], "%8.2f  ", kurt[ind]);
            }  
         ibeg += 10;
         first = FALSE;
         }  
      c_txtout (pline, print, fd);
      }  
/*
find next set of columns
------------------------
*/
   istart = icols + 1;
   icols = icols + ncols;
   }  
/*
close output file
-----------------
*/
if (fd) fclose (fd);
}
