#include "contable.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void tabcalc
(
    long *npix,                /* # pixels in image                  */
    long wind[][4],            /* image window                       */
    long *contab[],            /* contingency values                 */
    float *expec[],             /* expected values                    */
    long sum1[256],            /* row sums in table                  */
    long sum2[256],            /* column sums in table               */
    long colind[256],          /* number of non-zero columns         */
    long rowind[256],          /* number of non-zero rows            */
    long *nc,                  /* column indices                     */
    long *nr                  /* row indices                        */
)

/*
******************************************************************************

NAME: TABCALC                          

PURPOSE: Reads image and builds contingency table

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    3          8/04/88    K. Narayanan
    2          5/14/87    N. Deodhar 
    1          9/16/83    C. Gurney/
   		          K. Narayanan
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:	
   For each image line
      Call C_ESTEP to read line
      For each pixel in line
         Increment contingency table
         Increment sums of classes in each image
   For each class
      If the row sum is greater than 0 save the class number
      For the number of classes
         If the column sum is greater than 0, save the class number
         Calculate expected value in table
   Set the number of non-zero rows and columns

************************************************************************
*/
{
long	data_type;
long	in1, in2, j, k, l;
long	line;
long 	msg_flag;
long	nclass;
long	num_fill;
long    one = 1;       /* constant of one */
long    zero = 0;      /* constant of zero */
printf ("\n");

/*
initialize and enable processing message utility
------------------------------------------------
*/
msg_flag = GEN2;
c_inittm ("CONTABLE", "Processed", "line", "band", " ", &wind[0][NL], 
          &one, &msgtim, &msg_flag);
curent = 1;
c_startm();

/*
initialize the tables and sum arrays
------------------------------------
*/
data_type = ELONG;
num_fill = 256 * 256;
c_pxfill (contab[0], &data_type, &num_fill, &zero);

num_fill = 256;
c_pxfill (sum1, &data_type, &num_fill, &zero);
c_pxfill (sum2, &data_type, &num_fill, &zero);
/*
read image & set values in table
--------------------------------
*/
nclass = 0;
for (line = 0; line < wind[0][NL]; line++, curent++)
   {
   c_estep (&gdesc);
   for (j = 0; j < wind[0][NS]; j++)
      {
      in1 = iobuf[ipoint[0]+j];
      in2 = iobuf[ipoint[1]+j];
      if (in1 > nclass) nclass = in1;
      if (in2 > nclass) nclass = in2;
      contab[in1][in2]++;
      sum1[in1]++;
      sum2[in2]++;
      }  
   }  
c_estep (&gdesc);
c_stoptm ();
/*
calculate expected values and no. of non-zero classes
-----------------------------------------------------
*/
*npix = wind[0][NS] * wind[0][NL];
*nc = 0;
*nr = 0;
for (k = 0; k <= nclass; k++)
   {
   if (sum1[k] > 0) rowind[(*nr)++] = k;
   for (l = 0; l <= nclass; l++)
      {
      if (k == 0)
         if (sum2[l] > 0) colind[(*nc)++] = l;
      expec[k][l] = (float) sum1[k] * (float) sum2[l] / (float) (*npix);
      }  
   }  
--(*nc);
--(*nr);
}
