/****************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0          12/92     K.Sheffield        Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"


#define TERM 0
#define LP   1
#define FIL  2

extern struct PARBLK block;

void getpar
(
    char cmd[],              /* I: name of the gnuplot command file          */
    char in[],               /* I: name of the input plot file               */
    char data[],             /* I: name of the gnuplot data file             */
    char prtfile[],          /* I: name of the output print file             */
    long *scale,             /* I: residual scaling factor                   */
    char ptype[],            /* I: printer type                              */
    char dspflg[],           /* I: label points string                       */
    char plotopt[],          /* I: residual components string                */
    long *dpi,               /* I: printer resolution in dots per inch       */
    char title[],            /* I: plot title                                */
    long print[3]            /* I: PRINT option flags                        */
)

{
   struct VARIABLE *var;            /* pointer to a variable in TAE block */
   
   long stat,                       /* return status                      */
        std_out;                    /* logical unit for standard output   */

   long fatal_err = -1;             /* fatal error flag                   */
   long len;                        /* string length                      */
   long mode;                       /* file I/O mode                      */
   
   long zero = 0;
   
   
   char fileutil_file[CMLEN];       /* file for fileutil                  */

   /*
     initialize program 
     ------------------ */
   c_inlas (&block, &std_out);
   
   
   /*
     get input data file
     -------------------*/

   mode = IREAD;
   
   if(E_FAIL == c_getnam(&block,"INFILE",&zero,in,"plt",&mode))
     c_errmsg("Fatal error encountered","corrplot-fatal",&fatal_err);
   
   
   /*
     get output command file
     -----------------------*/

   mode = IWRITE;
   
   if(E_FAIL == c_getnam(&block,"CMDFILE",&zero,cmd,"cmd",&mode))
     c_errmsg("Fatal error encountered","corrplot-fatal",&fatal_err);

   /*
     get output command file
     -----------------------*/

   mode = IWRITE;
   
   if(E_FAIL == c_getnam(&block,"CMDFILE",&zero,data,"dat",&mode))
     c_errmsg("Fatal error encountered","corrplot-fatal",&fatal_err);

   /*
     Get print option 
     ---------------- */
   strcpy (prtfile, "corrplot");
   stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], 
		    prtfile, fileutil_file);
   if (stat != E_SUCC)
     c_errmsg("Fatal error encountered","corrplot-fatal",&fatal_err);
   
   /*
     Get plot title
     --------------- */
   var = p_find( &block, "TITLE");
   strcpy(title, SVAL( *var, 0));

   /*
     Get the scale 
     --------------*/
   var = p_find( &block, "SCALFACT");
   *scale =  IVAL( *var, 0);

   /*
     Get printer type
     ---------------- */
   var = p_find( &block, "PTRTYPE");
   strcpy(ptype, SVAL( *var, 0));
   len = (long)strlen(ptype);
   c_up2low(ptype,&len);
   
   /*
     Get printer resolution
     ---------------------- */
   var = p_find( &block, "DPI");
   *dpi =  IVAL( *var, 0);

   /*
     Get labeling option
     ------------------- */
   var = p_find( &block, "DSPFLG");
   strcpy(dspflg, SVAL (*var, 0));
   
   /*
     Get residual component option
     ----------------------------- */
   var = p_find( &block, "PLOTOPT");
   strcpy(plotopt, SVAL (*var, 0));

}
