/****************************************************************************
NAME:				CORRVIEW

PURPOSE:   CORRVIS pulls tie point neighborhoods from reference and search
	   images.  These neighborhoods are expanded, have text and a 
	   a box to hightlight the tie point added, and are written to 
           an output file. 

VERSION	 DATE	AUTHOR	   
-------	 ----	------	  
  1.0	 9/92	D. Steinwand		Original development
  2.0	 1/93	T. Mittan	 	Modified to only use one executable
				 	and released.	
  3.0    12/94	T. Mittan		Modified to use realloc() as tie
					points are read to elimate the
					MAX_TIE_POINT barrier.
  4.0    12/99  D. Hames		Remove MAXNS from buffer in vispts and
					allocate using ns. Modified to support 	
					images > 2gig. Add prototypes. SRF#893

****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "tiepts.h"

#define STAT_FLAG status
#define CHECK(ARG) status = ARG; if(status != E_SUCC) l_abort(status)

void main
(void)
{
struct TPLHEAD tie_head;	/* Tie point location file header record  */
struct PARBLK vblk;		/* TAE parameter block 			  */
struct TPLDATA *tie_data;	/* Ptr to Tie point location data records */
long zero=0;			/* Constant of zero 			  */
long one = 1;			/* constant of one			  */
long std_out;			/* Dummy variable to standard output 	  */
long bands[MAXIMG][MAXBND+1];	/* band array for refin			  */
long bands1[MAXIMG][MAXBND+1];	/* band array for srcin			  */
long bgrey;			/* background grey value		  */
long cgrey;			/* character grey scale value		  */
long npts;			/* Number of points in TPL file 	  */
long maxsamps;			/* Maximum number of samples in output 	  */
long STAT_FLAG;			/* Return status variable 	  	  */
char tpl_name[CMLEN];		/* Input Tie Point Location file name 	  */
char refin[MAXIMG][CMLEN];	/* reference image name			  */
char srcin[MAXIMG][CMLEN];	/* search image name			  */
char outname[CMLEN];		/* output mame				  */
char toname[CMLEN];		/* tae output mame			  */

c_inlas(&vblk, &std_out);

/* Allocate buffer space for tie point records
  ------------------------------------------*/
tie_data=(struct TPLDATA *)malloc((sizeof(struct TPLDATA)*MAX_TIE_POINTS));
if (tie_data == NULL) l_abort(NO_SPACE);

/* get parameters from TAE
------------------------*/
CHECK(getpar(&vblk, tpl_name,bands,bands1,&maxsamps,refin,srcin,outname,toname,
	     &cgrey, &bgrey));

/* get tie points
---------------*/
CHECK(get_tie_points(tpl_name, &tie_head, &tie_data, &npts));

/* create output image
--------------------*/
CHECK(vispts(tie_data, npts, maxsamps,refin,srcin,outname,cgrey,bgrey,
	     bands,bands1));

/* create associated history file
-------------------------------*/
if (c_hstout(&vblk,outname,(char (*)[256])" ",&zero," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered", "testgen-Nonfatal",&one);

/* prouce completion message
--------------------------*/
c_complt("CORRVIEW",(char (*)[256])toname, &one);
}

/****************************************************************************
NAME:				l_abort

PURPOSE:  Handles all fatal error conditions & exits gracefully

VERSION		DATE	AUTHOR
-------		----	------	
  1.0		9/92	D. Steinwand

****************************************************************************/
void l_abort
(
    long status			/* I: Error condition flag */
)
{
long pass = 1;			/* Do not abort processing */
long fatal = -1;		/* Abort processing */

switch(status)
   {
   case NO_SPACE:
	c_errmsg("Error allocating memory","corrview-alloc",&pass);
	break;
   case BAD_TPL:
	c_errmsg("Error opening TPL file","corrview-open",&pass);
	break;
   }
c_errmsg("Fatal error encountered","corrview-fatal",&fatal);
}
