/******************************************************************************

NAME:   genchr

FUNCTION:
   generate expanded dots of character to given buffer

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
   1.0        4-feb-82  j.  hwang,   sasc
   2.0        7-jan-88  m.mazzella SAR/GSFC  convert to c

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

      if vcnt is out of range then return
      if achar then
         get scale factor
         get number of character from table
      else
         get scale factor
         number of character is 1
      end if
      do following number of character times
         get ascii character code
         convert code to index to the dot table
         if illegal char then use question mark
         get a byte of the character dot table
         do i=1,8
            get a bit from this byte
            if bit is on then
               fill buffer with grey scale as directed in the table
               scale factor times.
            else
               use back ground grey from picture description table 
                  to fill buffer scale factor times
            end if
            increment buffer index
         end do
      end do
      return

******************************************************************************/
#include <stdio.h>

#include "las.h"
#include "tiepts.h"

void genchr
(
    char iobuf[], 	/* I/O: output line			*/
    long *vcnt, 	/* I: current line count on the picture */
    long tab[],		/* I:  square description  	*/
    char chrbuf[]	/* I: character(s) to be expanded */
)

{
long	code, lc, scale, x, nchar, index;
char	bt;
long	pointr, i, k, m;
extern unsigned char  dots[96][12];


/*
  return if table description indicates out of range

*/
if ((*vcnt < tab[0]) || (*vcnt > tab[0] + tab[2] - 1)) 
   return;

/*
  get scale

*/
index = 1;
tab[8]++;
if (tab[2] <= tab[3]) 
   {
/*
  set up variables for achar
*/
   scale = tab[3] / (tab[6] * 8);
   nchar = tab[6];
   }
else
   {

/*
  set up variable for dchar
*/
   scale = tab[2] / (tab[6] * 10);    
   if (scale < 1)
       scale = 1;
   index = (tab[8] - 1) / (scale * 10) + 1;
   nchar = 1;
   }


/*
  transfer current line of character dots to buffer

*/
pointr=0;      
lc = ((tab[8] - 1) % (scale*10)) / scale + 1;      

#ifdef vms
x = 8 * ((lc - 1) / 4) + 4 - lc;   /* bytes in a word have to be reversed */
#else
x = lc - 1;
#endif

for (i = 1; i <= nchar; i++)
   {
   code = chrbuf[index + i - 2] - 32;
   if (code == -32)
       code = 0;
   if ((code > 96) || (code < 0)) 
      code = 31;
   bt = dots[code][x];
   for ( k=8; k; k--)
      {
      code = tab[5];
      if (bit(bt,(unsigned char)k) != 1) 
         code = tab[4];
      if (code >= 0)
         for (m = 0; m < scale; m++)
         {
         iobuf[tab[1] + pointr + m - 1] = (char)code;
         }
      pointr += scale;      
      }
   }
return;
}
