/****************************************************************************
NAME:				get_tie_points

PURPOSE:  Reads in tie point & header records from a Tie Point Location file

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		1/89	D. Steinwand	   CSB      LAS 5.0 Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Open the Tie Point Location file & read the header record
	Do until EOF  (keeping track of the number of point read)
	   Read a tie point record and store in main data buffer 
	Close the Tie Point Location file
	Return
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "tiepts.h"

int get_tie_points
(
    char *name,			/* I: Name of the Tie Point Location file */
    struct TPLHEAD *header,	/* I: Tie Point Location header record */
    struct TPLDATA **data,	/* I: Tie Point Location data records */
    long *npts			/* I: Number of tie point data records read */
)
{
int *desc;			/* File descriptor for tie pt file */
long ptyp;			/* Type of tie point data/header record */
long acc;			/* File access flag */
long status;			/* Function return status flag */
long count;			/* Loop counter */
long maxtie;			/* maximum space allocated	*/
struct TPLDATA *data1;          /* TPL data record      */

data1 = *data;
maxtie = MAX_TIE_POINTS;


/* Open the tie point location file 
  --------------------------------*/
ptyp = TPL;
acc = IREAD;
if (c_ptopen((struct PTDESC **)&desc,name,&ptyp,&acc,(long *)header) != E_SUCC) 
   return(BAD_TPL);

/* Read in the tie point location data records
  -------------------------------------------*/
for (count = 0; ; count++)
   {
   if (count + 1 == maxtie)
      {
      maxtie += MAX_TIE_POINTS;
      data1 = (struct TPLDATA *) realloc(data1,
                                  (maxtie * sizeof(struct TPLDATA)));
      *data = data1;
      }

   status = c_ptio((struct PTDESC **)&desc, (long*)&(data1[count]));
   if (status == E_EOF) break;
   if (status == E_FAIL) return(E_FAIL);
   }
*npts = count;

/* Close the Tie Point Location file
  ---------------------------------*/
if (c_ptclse((struct PTDESC **)&desc) != E_SUCC) return(BAD_TPL);

return(E_SUCC);
}
