/****************************************************************************
NAME:				getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

VERSION		DATE	AUTHOR
-------		----	------	
  1.0		9/92	D. Steinwand
  2.0		1/93	T. Mittan		Added parameters chargrey
						and backgrnd

****************************************************************************/
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "tiepts.h"

int getpar
(
    struct PARBLK *vblk,	/* I: TAE variable block 		  */
    char *tpl_name,		/* O: Input tie point location file name  */
    long bands[][MAXBND +1],	/* O: bands in each image		  */
    long bands1[][MAXBND +1],	/* O: bands in each image		  */
    long *maxsamps,		/* O: Maximum number of samples in output */
    char refin[][CMLEN],	/* O: reference image name		  */
    char srcin[][CMLEN],	/* O: search image name			  */
    char honame[],		/* O: output image name			  */
    char toname[CMLEN],		/* O: tae output name			  */
    long *cgrey,		/* O: character grey value		  */
    long *bgrey			/* O: background grey value	          */
)
{
struct VARIABLE *var;		/* pointer to a TAE variable in vblk 	*/
long element;		  	/* array element of IN			*/	
long nmrimg;			/* number of images			*/
long non_fatal = 1;		/* nonfatal error flag			*/
long window[MAXIMG][4];		/* window				*/
long nbands[MAXBND];		/* number of bands			*/
long totbnd;			/* total number of bands		*/
long same;			/* flag for same windows		*/
long qflag[MAXIMG][2];		/* flag for window and band options	*/
long acc;			/* access flag for c_getnam 		*/
long zero = 0;			/* Constant of zero 			*/
long status;			/* return status			*/ 
long imglim;			/* maximum number of images allowed	*/	
long fatal = -1;		/* fatal error flag			*/

char taein[MAXIMG][CMLEN];	/* tae input name			*/

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INTL", &zero, tpl_name, "tpl", &acc) != E_SUCC)
	return(NO_TPL);

/* Retrieve and parse input image specification
-----------------------------------------------*/
element = 0;
imglim = 1;
/* get images
-----------*/
status = c_getin(vblk,"IN",&element,taein,refin,&nmrimg,window,bands,nbands,
                &totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","corrview-fatal",&fatal);
if (totbnd > 1)
   {
   c_errmsg("More than one band specified","corrview-band",&non_fatal);
   c_errmsg("Fatal error encountered","corrview-fatal",&fatal);
   }
if (qflag[0][0] != 0)
   {
   c_errmsg("Windowing not allowed","corrview-window",&non_fatal);
   c_errmsg("Fatal error encountered","corrview-fatal",&fatal);
   }

element = 1;
status = c_getin(vblk,"IN",&element,taein,srcin,&nmrimg,window,bands1,nbands,
                &totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","corrview-fatal",&fatal);
if (totbnd > 1)
   {
   c_errmsg("More than one band specified","corrview-band",&non_fatal);
   c_errmsg("Fatal error encountered","corrview-fatal",&fatal);
   }
if (qflag[0][0] != 0)
   {
   c_errmsg("Windowing not allowed","corrview-window",&non_fatal);
   c_errmsg("Fatal error encountered","corrview-fatal",&fatal);
   }

status = c_getout(vblk,"OUT",(char (*)[256])toname,(char (*)[256])honame);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","corrview-fatal",&fatal);

/* get character grey value
-------------------------*/
var = p_find(vblk, "CHARGREY");
*cgrey = IVAL(*var, 0);

/* get background grey value
--------------------------*/
var = p_find(vblk, "BACKGRND");
*bgrey = IVAL(*var, 0);

/* Get the maximum number of samples in the output image
  -----------------------------------------------------*/
var = p_find(vblk, "OUTNS");
*maxsamps = IVAL(*var, 0);

return(E_SUCC);
}
