/*****************************************************************************
 *									     *
 * 	Include file for structures and defines specific to the TIEPTS 	     *
 *	application.							     *
 *									     *
 *	By: L. Huewe, USGS/EROS Data Center, 6/89			     *
 *									     *
 *****************************************************************************/

#include "geompak.h"		/* Include file used by the geometric 
			 	   registration routines		      */
#include "digitizer.h"		/* Include file used by the digitizing routine*/

/*
   Structure used to store information needed by the interpolated zoom routines
*/
struct ENLARGE
  {
  long *work;			/* Hybrid space buffer 			      */
  long *horiz_table;		/* Horizontal resampling table 		      */
  long *vert_table;		/* Vertical resampling table 		      */
  float alpha;			/* Parametric cubic convolution alpha value   */
  long in_lines;		/* Number of input lines + resampling pad     */
  long in_samps;		/* Number of input samples + resampling pad   */
  long hybrid_samps;		/* Number of samples in hybrid space          */
  long extract_lines;		/* Number of lines to extract from display    */
  long extract_samps;		/* Number of samples to extract from display  */
  long factor;			/* Zoom factor 				      */
  long horiz_dim;		/* Horizontal resampling table dimension      */
  long vert_dim;		/* Vertical resampling table dimension        */
  unsigned char *input;		/* Input space buffer 			      */
  unsigned char *output;	/* Output space buffer 			      */
  char table_file[CMLEN];	/* Resampling weight table file name 	      */
  char method[6];		/* Resampling method 			      */
  };

/*
   Defines for edit flags
*/
#define	TABLE		8	/* Resampling weight table file name	      */


/* 
Function prototypes
-------------------
*/

long bit
(
long word,                      /*  I: source word  */
unsigned char pos               /*  I: position of bit desired  */
);

void l_abort
(
    long status			/* I: Error condition flag */
);

void enlarge
(
    struct ENLARGE *zoom       /* I/O: Enlargement parameters & buffers (I/O) */
);

void horiz_zoom
(
    struct ENLARGE *zoom       /* I/O: Enlargement parameters & buffers (I/O) */
);

void vert_zoom
(
    struct ENLARGE *zoom       /* I/O: Enlargement parameters & buffers (I/O) */
);

void genchr
(
    char iobuf[],       	/* I/O: output line                     */
    long *vcnt,         	/* I: current line count on the picture */
    long tab[],         	/* I:  square description       */
    char chrbuf[]       	/* I: character(s) to be expanded */
);

int get_tie_points
(
    char *name,                 /* I: Name of the Tie Point Location file */
    struct TPLHEAD *header,     /* I: Tie Point Location header record */
    struct TPLDATA **data,      /* I: Tie Point Location data records */
    long *npts                  /* I: Number of tie point data records read */
);

int getpar
(
    struct PARBLK *vblk,        /* I: TAE variable block                  */
    char *tpl_name,             /* O: Input tie point location file name  */
    long bands[][MAXBND +1],    /* O: bands in each image                 */
    long bands1[][MAXBND +1],   /* O: bands in each image                 */
    long *maxsamps,             /* O: Maximum number of samples in output */
    char refin[][CMLEN],        /* O: reference image name                */
    char srcin[][CMLEN],        /* O: search image name                   */
    char honame[],              /* O: output image name                   */
    char toname[CMLEN],         /* O: tae output name                     */
    long *cgrey,                /* O: character grey value                */
    long *bgrey                 /* O: background grey value               */
);

int init_zoom
(
    struct ENLARGE *zoom       /* I/O: Enlargement parameters & buffers (I/O) */
);

int vispts
(
    struct TPLDATA *tie_data,   /* I: Tie Point Location data records   */
    long npts,                  /* I: Number of points in TPL file      */
    long maxsamps,              /* I: Maximum number of samples across  */
    char refin[][CMLEN],        /* I: reference image name              */
    char srcin[][CMLEN],        /* I: search image name                 */
    char outname[],             /* I: output name                       */
    long cgrey,                 /* I: character grey value              */
    long bgrey,                 /* I: background grey value             */
    long bands[][MAXBND+1],     /* I: band array for reference image    */
    long bands1[][MAXBND+1]     /* I: band array for reference image    */
);

