/*****************************************************************************
NAME:				vispts

PURPOSE:  Edits points based on the "leave-one-out" method

VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 9/92	D. Steinwand
  2.0    1/93	T. Mittan	Modified to do all processing in one
				executable.

****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "dot.h"
#include "tiepts.h"

int vispts
(
    struct TPLDATA *tie_data,	/* I: Tie Point Location data records 	*/
    long npts,			/* I: Number of points in TPL file 	*/
    long maxsamps,		/* I: Maximum number of samples across 	*/
    char refin[][CMLEN],	/* I: reference image name		*/
    char srcin[][CMLEN],	/* I: search image name			*/
    char outname[],		/* I: output name			*/
    long cgrey,			/* I: character grey value		*/
    long bgrey,			/* I: background grey value		*/
    long bands[][MAXBND+1],	/* I: band array for reference image	*/
    long bands1[][MAXBND+1]	/* I: band array for reference image	*/
)
{
struct TPLDATA *tt;		/* Ptr to TPL data structure 		*/
struct ENLARGE zoom[2];		/* enlargement buffer			*/
long acc;			/* imageio read/write access		*/
long count = 0;			/* number of point processed		*/
long i,k;			/* Loop counters 			*/
long l,j,z;			/* Loop counters 			*/
long npts_used;			/* Current number of points used 	*/
long sl,ss,nl,ns;		/* image window				*/
long max_ref_sizex;		/* maximum reference chip size in x	*/
long max_ref_sizey;		/* maximum reference chip size in y 	*/
long max_sea_sizex;		/* maximum search chip size in x	*/
long max_sea_sizey;		/* maximum search chip size in y	*/
long num_across;		/* number of scences across		*/
long num_down;			/* number of scences down		*/
long non_fatal = 1;		/* non fatal error flag			*/
long fatal = -1;
long one = 1;			/* constant of one			*/
long zrl,zrs,zsl,zss;		/* number of lines and samples 		*/
long dtype = EBYTE;		/* data type				*/
long opt[3];			/* option array				*/
long scale = 2;			/* scale factor for annotation text	*/
long status;			/* return status			*/
long tb[10];			/* array for annotation text		*/
long len;			/* strlen of annotation text		*/
char *buffer;       		/* data buffer				*/
unsigned char *ptr;
char *ptr1;
int fdesc;			/* imagio file descriptor		*/


/* Loop thru points, finding those which are valid
  -----------------------------------------------*/
max_ref_sizey = max_ref_sizex = 0;
max_sea_sizey = max_sea_sizex = 0;
for(tt=tie_data, npts_used=0, i=0; i<npts; i++, tt++)
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
      {
      if (tt->ref_size[0] > max_ref_sizey) max_ref_sizey = tt->ref_size[0];
      if (tt->ref_size[1] > max_ref_sizex) max_ref_sizex = tt->ref_size[1];
      if (tt->sea_size[0] > max_sea_sizey) max_sea_sizey = tt->sea_size[0];
      if (tt->sea_size[1] > max_sea_sizex) max_sea_sizex = tt->sea_size[1];
      npts_used++;
      }
if(npts_used==0)
  {
  c_errmsg("No tie points are valid","corrview-valid",&non_fatal);
  return(E_FAIL);
  }

max_ref_sizex *= 4;
max_ref_sizey *= 4;
max_sea_sizex *= 4;
max_sea_sizey *= 4;

/* calculate number of subimages across and down the output image
--------------------------------------------------------------*/
num_across = maxsamps / ((max_sea_sizex+100+max_ref_sizex)-50);
if (num_across < 1) 
    num_across = 1;
num_down = npts_used / num_across;
if (num_across * num_down < npts_used) num_down++;

if (num_across > npts_used)
    num_across = npts_used;
  /* background fill output image
  -----------------------------*/
  c_errmsg("Opening OUTPUT image","informational",&non_fatal);
  nl  = num_down*(max_sea_sizey+50) - 49;
  ns  = num_across*(max_sea_sizex+100+max_ref_sizex)-49;
  acc = IWRITE;
  opt[0] = 0;
  opt[1] = NO_COMPRES;
  opt[2] = 0;
  if ((status = c_eopenr((struct FDESC **)&fdesc,outname,&one,&one,&one,&nl,&ns,
		         &acc,&dtype,opt,&one)) == E_FAIL)
     {
     return(E_FAIL);
     }
  /* allocate memeory for buffer
  ----------------------------*/
  if ((buffer = (char *)malloc(ns * sizeof(char))) == NULL)
      c_errmsg("Error allocating dynamic memory","corrview-alloc",&fatal);
  for (i = 0; i < ns; i++)
     buffer[i] = (char)bgrey;
  for (i = 1; i <= nl; i++)
     if ((status = c_ewrite((struct FDESC **)&fdesc,&one,&i,
			(unsigned char *)buffer,&one))==E_FAIL)
       {
       c_eclose((struct FDESC **)&fdesc);
       free (buffer);
       return(E_FAIL);
       }
  c_eclose((struct FDESC **)&fdesc);

/* Calculate windows & subsquent processing : MAIN PROCESSING LOOP
  --------------------------------------------------------------*/
for(tt=tie_data, k=0, i=0; i<npts; i++, tt++)
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
      {
      count++;
      sl=(long)((tt->ref_coord[0] - tt->ref_size[0]/2) + 1);
      ss=(long)((tt->ref_coord[1] - tt->ref_size[1]/2) + 1);
      nl=tt->ref_size[0];
      ns=tt->ref_size[1];

      /*
       We need to initialize stuff for enlargment process 
      -------------------------------------------------*/
      zoom[0].alpha = -.5;
      zoom[0].in_samps = ns;
      zoom[0].in_lines = nl;
      zoom[0].factor = 4;
      strcpy(zoom[0].method,"BI");
      init_zoom(&zoom[0]);

      /* decrement input lines/samples until the lines/samples wich are 
	 to be extracted are less than the number specified and 
         initialize enlargment process
      ---------------------------------------------------------------*/ 
      while (zoom[0].extract_lines > nl)
          {
          zoom[0].in_lines--;
          init_zoom(&zoom[0]);
          }
      while (zoom[0].extract_samps > ns)
          {
          zoom[0].in_samps--;
          init_zoom(&zoom[0]);
          }

      /* Open the reference image and read into the proper buffer
      ---------------------------------------------------------*/ 
      acc    = IREAD;
      opt[0] = 0;
      opt[1] = NO_COMPRES;
      opt[2] = 0;
      if ((status = c_eopenr((struct FDESC **)&fdesc,(char *)refin,&bands[0][0],
		&sl,&ss,&zoom[0].extract_lines,&zoom[0].extract_samps,&acc,
		&dtype,opt,&zoom[0].extract_lines)) == E_FAIL)
         {
         free (buffer);
         return(E_FAIL);
         }
      if ((status = c_eread((struct FDESC **)&fdesc,&bands[0][0],&one,
		zoom[0].input,&zoom[0].extract_lines)) == E_FAIL)
         {
         c_eclose((struct FDESC **)&fdesc);
         free (buffer);
         return(E_FAIL);
         }
      c_eclose((struct FDESC **)&fdesc);

      /* enlarge reference buffer
      -------------------------*/
      enlarge(&zoom[0]);
 
      /* Calculate tie point offset
      ---------------------------*/
      zrl = (long)((tt->ref_coord[0]-sl)+1);
      zrl = (long)(4.0 * ((float)zrl - 0.5) + 0.5);
      zrl = zrl - 7;
      zrs = (long)((tt->ref_coord[1]-ss)+1);
      zrs = (long)(4.0 * ((float)zrs - 0.5) + 0.5);
      zrs = zrs - 7;

       
      /* Engrave tie point box on expanded image
      ----------------------------------------*/
      if ((zrl + 14) > (4 * zoom[0].extract_lines))
         c_errmsg("Error writing tiepoint box","corrview-box",&non_fatal);

      for (j = zrl; j <= (zrl + 14); j++)
        {
        if ((zrl == j) || ((zrl + 14 ) == j))
           {
           for (l = 0; l < 15; l ++) 
              zoom[0].output[zrs + l - 1 + ((j -1) * zoom[0].in_samps * 4)] 
                                    	 = (unsigned char)cgrey;
           }
        else if ((zrl + 7) == j)
           {
           zoom[0].output[zrs -  1 + ((j -1) * zoom[0].in_samps * 4)] 
					= (unsigned char)cgrey;
           zoom[0].output[zrs + 13 + ((j -1) * zoom[0].in_samps * 4)] 
					= (unsigned char)cgrey;
           zoom[0].output[zrs +  6 + ((j -1) * zoom[0].in_samps * 4)] 
					= (unsigned char)cgrey;
           }
        else 
           {
           zoom[0].output[zrs -  1 + ((j -1) * zoom[0].in_samps * 4)] 
					= (unsigned char)cgrey;
           zoom[0].output[zrs + 13 + ((j -1) * zoom[0].in_samps * 4)] 
					= (unsigned char)cgrey;
           }
        }

      /* Calculate search window
      ------------------------*/
      sl=(long)(tt->sea_coord[0] - tt->sea_size[0]/2);
      ss=(long)(tt->sea_coord[1] - tt->sea_size[1]/2);
      nl=tt->sea_size[0];
      ns=tt->sea_size[1];
      
      /* initialize enlargement process
      -------------------------------*/
      zoom[1].alpha = -.5;
      zoom[1].in_samps = ns;
      zoom[1].in_lines = nl;
      zoom[1].factor = 4;
      strcpy(zoom[1].method,"BI");
      init_zoom(&zoom[1]);
      /* Decrement input lines/samples until the lines/samples wich are 
	 to be extracted are less than the number specified and 
         initialize enlargment process
      ---------------------------------------------------------------*/ 
      while (zoom[1].extract_lines > nl)
        {
        zoom[1].in_lines--;
        init_zoom(&zoom[1]);
        }
      while (zoom[1].extract_samps > ns)
        {
        zoom[1].in_samps--;
        init_zoom(&zoom[1]);
        }

      /* Open and read search image into the proper buffer
      --------------------------------------------------*/
      if ((status = c_eopenr((struct FDESC **)&fdesc,(char *)srcin,
		&bands1[0][0],&sl,&ss,&zoom[1].extract_lines, 
		&zoom[1].extract_samps,&acc,&dtype,opt,
		&zoom[1].extract_lines)) == E_FAIL)
         {
         free (buffer);
         return(E_FAIL);
         }
      if ((status = c_eread((struct FDESC **)&fdesc,&bands1[0][0],&one,
		zoom[1].input,&zoom[1].extract_lines)) == E_FAIL)
         {
         c_eclose((struct FDESC **)&fdesc);
         free (buffer);
         return(E_FAIL);
         }
      c_eclose((struct FDESC **)&fdesc);

      /* Enlarge search sub image
      -------------------------*/
      enlarge(&zoom[1]);
 
      /* Calculate tie point offsets
      ----------------------------*/
      zsl = (long)((tt->sea_coord[0]-sl)+1);
      zsl = (long)(4.0 * ((float)zsl - 0.5) + 0.5);
      zsl = zsl - 7;
      zss = (long)((tt->sea_coord[1]-ss)+1);
      zss = (long)(4.0 * ((float)zss - 0.5) + 0.5);
      zss = zss - 7; 

      /* Engrave tie point box on expanded image
      ----------------------------------------*/
      if ((zsl + 14) > (4 * zoom[1].extract_lines))
         c_errmsg("Error writing tiepoint box","corrview-box",&non_fatal);
         
      for (j = zsl; j <=(zsl + 14); j++)
        {
        if ((zsl == j) || ((zsl + 14 ) == j))
           {
           for (l = 0; l < 15; l ++) 
              zoom[1].output[zss + l - 1 + ((j -1) * zoom[1].in_samps * 4)] 
                                    	 = (unsigned char)cgrey;
           }
        else if ((zsl + 7) == j)
           {
           zoom[1].output[zss -  1 + ((j -1) * zoom[1].in_samps * 4)] 
						= (unsigned char)cgrey;
           zoom[1].output[zss + 13 + ((j -1) * zoom[1].in_samps * 4)] 
						= (unsigned char)cgrey;
           zoom[1].output[zss +  6 + ((j -1) * zoom[1].in_samps * 4)] 
						= (unsigned char)cgrey;
           }
        else 
           {
           zoom[1].output[zss -  1 + ((j -1) * zoom[1].in_samps * 4)] 
						= (unsigned char)cgrey;
           zoom[1].output[zss + 13 + ((j -1) * zoom[1].in_samps * 4)] 
						= (unsigned char)cgrey;
           }
        }

      /* Initialize array for annotation text
      -------------------------------------*/
      len   = (long)strlen(tt->pt_id);
      tb[0] = 1;
      tb[1] = max_sea_sizex+25;
      tb[2] = 10 * scale;
      tb[3] = scale * 8 * len; 
      tb[4] = -1.0;
      tb[5] = cgrey;
      tb[6] = len;
      tb[7] = 1;
      tb[8] = 0;
      tb[9] = 2;

      /* CONCAT search and reference images together and write to output
      ----------------------------------------------------------------*/
      nl  = max_sea_sizey;
      ns  = max_sea_sizex+50+max_ref_sizex;
      acc = IUPDATE;
      k   = (count - 1)/num_across;
      sl  = 1+(k*(max_sea_sizey+50));
      k   = ((count - 1) % num_across);
      ss  = 1+(k*(max_sea_sizex+100+max_ref_sizex));
      if ((status = c_eopenr((struct FDESC **)&fdesc,outname,&one,&sl,&ss,&nl,
		&ns,&acc,&dtype,opt,&one)) == E_FAIL)
         {
         free (buffer);
         return(E_FAIL);
         }
      for (j = 1; j <= nl; j++,sl++)
         {
         /* If we are not below search image write it
         -------------------------------------------*/
         if (j <= (zoom[1].in_lines * 4))
           {
           /* Place the search image first
           -----------------------------*/ 
           ptr = &(zoom[1].output[(j - 1) * 4 * zoom[1].in_samps]);
           ptr1 = &buffer[0];
           for (l = zoom[1].in_samps * 4; l--; *(ptr1++) = *(ptr++));

           /* If we are not below reference image write it
           ---------------------------------------------*/
           if (j <= (zoom[0].in_lines * 4))
              {
              /* background fill any empty spaces between images
              ------------------------------------------------*/
              for (l = (zoom[1].in_samps * 4); l <(max_sea_sizex+50); l++)
                  buffer[l] = (char)bgrey; 

              /* Place the reference image 
              --------------------------*/
              l = (max_sea_sizex+50 - 1);
              ptr1 = &buffer[l];
              ptr = &(zoom[0].output[(j - 1) * 4 * zoom[0].in_samps]);
              for (z = zoom[0].in_samps * 4; z--; *(ptr1++) = *(ptr++));
              }
           else
              {
              /* We are below reference image background fill
              ----------------------------------------------*/
              *ptr1 = (char)(zoom[1].in_samps *4);
              for (l = ns - (zoom[1].in_samps * 4); l--; *(ptr1++) 
			             = (char)bgrey); 
              }
           /* Background fill to end of image
           --------------------------------*/
           for (l = zoom[0].in_samps * 4 + (max_sea_sizex +50); l < ns; l++)
               buffer[l] = (char)bgrey; 
           }
         else
           {
           /* We are below both images background fill
           ------------------------------------------*/
           for (l = ns, ptr1 = &(buffer[0]); l-- ; *(ptr1++) 
				= (char)bgrey);
           }
         /* Do actual annotation of text and write out
         -------------------------------------------*/
         if ((j >= (max_ref_sizey+25)) && (j <= (max_ref_sizey+25 + tb[2])))
              {
	      l = j - (max_ref_sizey + 25) + 1;
              genchr(buffer,&l,tb,tt->pt_id);
              }
         if ((status = c_ewrite((struct FDESC **)&fdesc,&one,&j,
			(unsigned char *)buffer,&one)) == E_FAIL)
              {
              c_eclose((struct FDESC **)&fdesc);
	      free (buffer);
              return(E_FAIL);
              }
         }
      c_eclose((struct FDESC **)&fdesc);
      }

/* free dymanic memory 
--------------------*/
free (buffer);
              
return(E_SUCC);
}
