/*
*****************************************************************************

NAME:	COVAR

PURPOSE:
	COVAR computes the mean and the covariance matrix of a multiband
	image and outputs them to a statistics file.

PROGRAM HISTORY:
  Version	Date       Author	   Request
  -------	----	   ------          -------
    5.0		09/92	   T. Mittan	   Modified include file to remove
					   warnings on link
    3.0         11/88      K. A. Narayanan conversion to LAS Version 5.0
    2.0          6/87      J. Lavergne     renamed STAT
    1.0         12/82      K. A. Narayanan 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get and validate input parameters
   Call FILSPEC to open and group input image files
   Call NSTAT to compute mean vector and covariance matrix
      and output to the specified statistics file
   Call COMPLT to issue completion message

*****************************************************************************
*/


#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

struct PARBLK block;                 /* TAE parameter block        */

struct FDESC *    fdesc[MAXIMG + 1];	     /* file descriptor		   */
struct GDESC *    gdesc;	     		     /* group descriptor	   */
long    ipoint[MAXIMG * MAXBND + 1]; /* image i/o buffer pointers  */
unsigned char   *buf;                /* image i/o buffer           */

extern	long    curent;		     /* image io counter 	   */
extern	long    curcat;		     /* image io counter 	   */

static  long    one = 1;	     /* constant of one		   */


int main
(void)
{
static 	long	bands[MAXIMG][MAXBND + 1];    
                                      /* image bands                       */
long	msgtim;                       /* interval between messages         */
long	nimages;                      /* number of images                  */
long	nlin;                         /* number of lines in image          */
long	npin;                         /* number of pixels per line         */
long	num_image_bands[MAXIMG];      /* number of bands in each image     */
long	num_inbands;                  /* total number of input image bands */
long	wind[MAXIMG][4];              /* image windows                     */

static	char	hos_image[MAXIMG][CMLEN];     
                                      /* image host names                  */
char	hos_stat[1][CMLEN];              /* stats file host name              */
char	las_stat[1][CMLEN];              /* stats file las name               */

/*
get parameters                                         
--------------                                     
*/
getpar (las_stat[0], hos_stat[0], hos_image, &nimages, num_image_bands, bands, 
        &num_inbands, wind, &npin, &nlin, &msgtim);

/*
set up file specifications 
--------------------------
*/
filspec (hos_image, &nimages, num_image_bands, bands, &npin, &nlin, wind);

/*
compute and output statistics
-----------------------------
*/
nstat (hos_stat, &num_inbands, &npin, &nlin, &msgtim);

/*
issue completion message
------------------------
*/
c_complt("COVAR", las_stat, &one);

return EXIT_SUCCESS;

}
