#include "covar.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char las_stat[],         /* stats file las name               */
    char hosst[],            /* stats file host name descriptor   */
    char hosnm[][CMLEN],     /* image host name descriptor        */
    long *nimages,           /* number of images specified        */
    long *num_image_bands,   /* number of bands in each image     */
    long bands[MAXIMG][MAXBND + 1],  /* image bands               */
    long *num_inbands,       /* total number of image bands       */
    long wind[][4],          /* image windows                     */
    long *npin,              /* number of pixels in window        */
    long *nlin,              /* number of lines in image window   */
    long *msgtim            /* time between messages             */
)

/*
**************************************************************************

NAME: GETPAR

PURPOSE: gets parameters

PROGRAM HISTORY:
  Version	Date       Author	   Request
  -------	----	   ------          -------
    2.0         11/88      K. A. Narayanan conversion to LAS Version 5.0
    1.0         12/82      K. A. Narayanan 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Call C_GETIN to get input image parameters
   Validate the parameters
   Call P_FIND to get statistics las file name
   Call C_GETNAM to get its host file name

**************************************************************************
*/
{
struct	VARIABLE *var;              /* pointer to a variable in TAE block */

long	access;                     /* access type for getnam             */
long	flag[MAXIMG][2];            /* flag for subwindows                */
long	imglimit = MAXIMG;          /* maximum number of images           */
long	istat;                      /* return status                      */
long	same_window;     /* flag indicating if images have same window    */
long	std_out;                    /* logical unit for standard output   */

char	las_names[MAXIMG][CMLEN];   /* image las names                    */

static long zero = 0;

/*
initialize program
------------------
*/
c_inlas(&block, &std_out);

var = p_find(&block, "$MSGTIM");
*msgtim = IVAL(*var, 0);

/*
get and validate input image name, window, and bands
----------------------------------------------------
*/
istat = c_getin (&block, "IN", &zero, las_names, hosnm, nimages, wind, bands, 
               num_image_bands, num_inbands, &imglimit, &same_window, flag);

if (istat != E_SUCC) 
    c_errmsg ("Error accessing the input file names", "covar-in", &fatal_err);

if (*num_inbands <= 1) 
   c_errmsg ("Input image must contain more than one band", "covar-nbands", 
             &fatal_err);

if (same_window == 0)
    c_errmsg ("Image windows not same size", "covar-samewindow", &fatal_err);
/*
save input image window size
----------------------------
*/
*nlin = wind[0][NL];
*npin = wind[0][NS];

/*
get output statistics file name
-------------------------------
*/
access = IUPDATE;
istat = c_getnam (&block, "OUTSTAT", &zero, hosst, "stat", &access);
if (istat != E_SUCC) 
   c_errmsg ("Error creating the output file names", "covar-outstat", &fatal_err);
strcpy(las_stat, hosst);
}
