#include "covar.h"
#include "util.h"
#include "protos.h"

void nstat
(
    char hosst[][CMLEN],           /* stats file host name descriptor */
    long *nbands,           /* number of bands in input image  */
    long *np,               /* number of pixels/line in image  */
    long *nl,               /* number of lines in image        */
    long *msgtim           /* interval between messages       */
)

/*
**********************************************************************

NAME: NSTAT

PURPOSE: Computes and outputs mean and covariance statistics

PROGRAM HISTORY:
  Version	Date       Author	   Request
  -------	----	   ------          -------
    2.0         11/88      K. A. Narayanan conversion to LAS Version 5.0
    1.0         12/82      K. A. Narayanan 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Call C_INITTM and C_STARTM to initialize and enable processing 
      message utility
   Calculate mean vector and covariance matrix
   Call OUTPUT to output computed statistics
   Call C_STOPTM disable processing message
   Call C_EGCLSE to close image files

***********************************************************************
*/
{
#define	ncov ( *nbands * (*nbands + 1)) / 2   /* no. elements in cov vector */
#define idcov (MAXBND * (MAXBND + 1)) / 2     /* max no. elements in cov    */

static	double	cov[idcov];             /* covariance matrix                */
static	double	smean[MAXBND];          /* mean vector                      */

float	*b1p;                           /* pointers to access data          */
float	*b2p;

long	line;                           /* line number                      */
long	msg_flag;                       /* flag for message                 */
long	num_elements = (*np) * (*nl);   /* number of pixels in image        */
long	size;                           /* data size                        */
long	i, j, m, n, nc;                 /* loop indices                     */

/*
initialize and enable processing message utility
------------------------------------------------
*/
msg_flag = ALLCAT;
c_inittm ("COVAR", "Processed", "line", "bands", " ", nl, nbands, msgtim, 
          &msg_flag);
curent = 1;
c_startm();

size = sizeof (float);

/*
calculate mean vector and covariance matrix
-------------------------------------------
*/
for (i = 0; i < *nbands; i++) smean[i] = 0.0;
for (i = 0; i < ncov; i++) cov[i] = 0.0;

for (line = 0; line < *nl; line++, curent++)
   {
   if (c_estep (&gdesc) != E_SUCC)
      c_errmsg ("Error reading image", "covar-read", &fatal_err);

   for (j = 0; j < *np; j++)
      {
      for (m = 0, nc = 0; m < *nbands; m++)
         {
	 b1p = (float *) &buf[(ipoint[m] + j) * size];
         smean[m] += *b1p; 
         for (n = 0; n <= m; n++)
            {
	    b2p = (float *) &buf[(ipoint[n] + j) * size];
            cov[nc] += (*b1p) * (*b2p);
            nc++;
            }
         }
      }
   }

for (i = 0; i < *nbands; i++)    smean[i] /= num_elements;

for (i = 0, nc = 0; i < *nbands; i++)
   {
   for (j = 0; j <= i; j++)
      {
      cov[nc] = (cov[nc] / num_elements) - (smean[i] * smean[j]);
      nc++;
      }  
   }  

/*
disable processing message
--------------------------
*/
c_stoptm();

/*
output computed statistics
--------------------------
*/
output (hosst, cov, smean, nbands);

/*
close image files
-----------------
*/
c_egclse (&gdesc);
}
