#include "covar.h"
#include "util.h"
#include "protos.h"

void output
(
    char hosst[][CMLEN],                  /* stats file host name      */
    double cov[],                    /* covariance matrix         */
    double smean[],                  /* mean vector               */
    long *nbands                  /* number of bands in image  */
)

/*
**************************************************************************

NAME: OUTPUT

PURPOSE: Outputs mean and covariance to statistics file

PROGRAM HISTORY:
  Version	Date       Author	   Request
  -------	----	   ------          -------
    2.0          7/88      K. A. Narayanan conversion to LAS Version 5.0
    1.0         12/82      K. A. Narayanan 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Call C_STOPEN to open output statistics file
   Set up data types and shape vectors for mean vector and covariance 
      matrix (in vector form)
   Call C_STPUT to output mean and covariance statistics to statistics file
   Call C_STCLSE to close statistics file

**************************************************************************
*/

{
long	access;                         /* stats file access type         */
long	cshape[9];                      /* covariance shape vector        */
FILE *	fd;                             /* stats file descriptor          */
long	ic, is;                         /* class and site indices         */
long	ierr;                           /* stat i/o error return status   */
long	mshape[9];                      /* mean shape vector              */
long	nclass;                         /* number of classes in stats file*/
long	num_files;                      /* number of files counter        */
long	sfile_status;                   /* stats file old/new status      */
static long inf_err = 1;                       
/*
open statistics file
--------------------
*/
access = IUNKNOWN;
sfile_status = c_stopen (&fd, hosst[0], &access, &nclass);
if (sfile_status  ==  E_OPNEW) num_files = 1;
else if (sfile_status != E_OPOLD)
   c_errmsg("Unable to open statistics file", "covar-stopen", &fatal_err);
else num_files = 0;
/*
define data types and shape vectors for mean and covariance
-----------------------------------------------------------
*/
mshape[0] = EDOUBLE;
mshape[1] = 1;
mshape[2] = *nbands;		  

cshape[0] = EDOUBLE;
cshape[1] = 1;
cshape[2] = (*nbands * (*nbands + 1)) / 2;  

/*
write mean and covariance to statistics file
--------------------------------------------
*/
ic = is = 0;
ierr = c_stput (&fd, "MEAN_VECTOR", &ic, &is, mshape, smean);
if (ierr != E_DTOLD && ierr != E_DTNEW)
   {
   c_errmsg ("Error writing mean vector to statistics file", "covar-mean", 
             &inf_err);
   c_clean (&block, "COVAR", hosst, &num_files);
   }

ierr = c_stput (&fd, "COVARIANCE_MATRIX", &ic,&is,cshape, cov);
if (ierr != E_DTOLD && ierr != E_DTNEW)
   {
   c_errmsg ("Error writing covariance matrix to statistics file", "covar-cov", 
             &inf_err);
   c_clean (&block, "COVAR", hosst, &num_files);
   }
/*
close statistics file
---------------------
*/
c_stclse(&fd);

}

