/*******************************************************************************
NAME:           CLEAN_GOF

PURPOSE:        Cleans the deleted records from the input GOF and frees up the
                space used by them.

*******************************************************************************/
#include "las.h"
#include "gof.h"
#include "labserv.h"
#include "protos.h"

int clean_gof
(
    char *ingof,            /* Input GOF name (I) */
    long gtype              /* GOF graphics type to process (I) */
)
{
    union GOF gofrec;     /* GOF record union */
    union GOF gofhdr;     /* GOF header structure */

    /* Valid GOF types */
    static char goftxt[MAXGOF][6] = {"POINT","LINE","POLY","ANNOT"};

    long action = 1;      /* Remove deleted records flag for the close */
    long fatal = -1;      /* Abort flag for error messages */
    long status;          /* Status return variable */

    int infd;             /* Input GOF file descriptor */
    int mode;             /* Mode to open GOF with */
    int adding;           /* Adding to the GOF flag */
    int length;           /* Length to the next record */
    int obyte;            /* Number of bytes read */
    int cflag = 0;        /* Flag indicating whether a GOF was created */

    /* Types of GOF data records to read */
    int type[MAXGOF] = {PNT,HPOLIN,HPOLIN,ANOT}; 

    static int hdrtyp = HDR; /* read header record from GOF */

    char hostnam[CMLEN];  /* Fully qualified host GOF name */
    char lasnam[CMLEN];   /* Fully qualified LAS GOF name */


    /* Open the input graphics overlay file for read access
       ----------------------------------------------------*/
    mode = READ;
    c_cmhost(ingof,hostnam);
    if(c_gofopn(&infd,hostnam,lasnam,goftxt[gtype],&mode,&cflag) != E_SUCC)
        c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);
    strcpy(ingof,hostnam);

    /* Close the GOF
       -------------*/
    if(c_gofcls(&infd) != E_SUCC)
        c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);

    /* Open the GOF as a label services file
       -------------------------------------*/
    mode = 3;
    if(c_lsopen( &infd,hostnam,(long *) &mode)!= E_SUCC)
        c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);

    /* Close the GOF with the option to remove records marked for deletion
       -------------------------------------------------------------------*/
    if(c_lsclos(&infd,hostnam,&action) != E_SUCC)
        c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);

    /* Count the GOF records remaining and update the header record
       ------------------------------------------------------------*/
    if(c_gofopn(&infd,hostnam,lasnam,goftxt[gtype],&mode,&cflag) != E_SUCC)
        c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);
    if(c_gofrd(&infd,&obyte,&length,&hdrtyp,(struct DDR *)NULL,&gofhdr)
       != E_SUCC)
        c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);

    gofhdr.hdrec.nrec = 0;
    status = E_SUCC;
    while (status != E_EOF)
    {
        status = c_gofrd(&infd,&obyte,&length,&type[gtype],(struct DDR *)NULL,
                &gofrec);
        if (status == E_EOF) 
            break;
        else if (status != E_SUCC)
            c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);
        gofhdr.hdrec.nrec++;
    }
    lseek(infd,0,0);
    adding = FALSE;
    if(c_gofwr(&infd,&length,&hdrtyp,(struct DDR *)NULL,&gofhdr,&adding)
               != E_SUCC)
        c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);
    if(c_gofcls(&infd) != E_SUCC)
        c_errmsg ("Fatal error encountered","cpygof-fatal",&fatal);
    return(E_SUCC);
}
