/*******************************************************************************
NAME:           COPY_GOF

PURPOSE:        Copies the records from the input GOF to the output GOF
                (no DELETED records are copied).

*******************************************************************************/

#include "las.h"
#include "gof.h"
#include "typlim.h"
#include "util.h"
#include "protos.h"

int copy_gof
(
    char *ingof,      /* Input GOF name (I) */
    long gtype,       /* GOF graphics type to process (I) */
    char *outgof,     /* Output GOF name (I/O) */
    long *msgtim      /* Number of seconds between processing messages (I) */
)
{
    extern long curent;    /* Current record being processed       */

    union GOF gofrec;      /* GOF record union                     */
    union GOF gofhdr;      /* Input GOF header structure   */  
    union GOF outhdr;      /* Output GOF header structure  */

    /* Valid GOF types */
    static char goftxt[MAXGOF][6] = {"POINT","LINE","POLY","ANNOT"};

    /* Types of GOF data records to read */
    static int type[MAXGOF] = {PNT,FPOLIN,FPOLIN,ANOT};

    static long big = MAXLONG;  /* Used as max entity for processing
                                   message to work correctly */
    static long flag = 2;       /* Type of processing message */
    static long zero = 0;       /* Constant zero */

    long create;           /* GOF create flag */
    long fatal = -1;       /* Fatal flag for errmsg */
    long status;           /* Status return variable */

    char entity[15];       /* Entity being processed */
    char hname[CMLEN];     /* Host name of the GOF to be opened */
    char reclabel[8];      /* GOF record label */
    char lname[CMLEN];     /* lasname output form c_gofopen */

    int outfd;             /* Output GOF file descriptor */
    int obyte;             /* Number of bytes read   */
    int length;            /* Length to the next record */
    int hdrtyp = HDR;      /* Read header record from GOF */ 
    int adding;            /* Adding to the GOF flad */
    int infd;              /* Input GOF file descriptior */
    int goftype;      
    int cflag = 0;         /* Flag indicating whether a GOF was created */
    int mode;              /* Mode to open GOF with  */


    /* Open the input graphics overlay file for read access
       ----------------------------------------------------*/
    mode = READ;
    c_cmhost (ingof,hname);
    if(c_gofopn(&infd,hname,lname,goftxt[gtype],&mode,&cflag) != E_SUCC)
        c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);

    /* Read the GOF header record
       --------------------------*/
    if(c_gofrd(&infd,&obyte,&length,&hdrtyp,(struct DDR *)NULL,&gofhdr)
       != E_SUCC)
        c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);

    /* Open the output graphics overlay file for UPDATE access and write
       out the appropriate header record.
       -----------------------------------------------------------------*/
    mode = UPDATE;
    c_cmhost (outgof,hname);
    if(c_gofopn(&outfd,hname,lname,goftxt[gtype],&mode,&cflag) != E_SUCC)
        c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
    strcpy(outgof,hname);

    if (cflag == TRUE)
    {
        create = adding = TRUE;
        lseek(outfd,0,0);
        gofhdr.hdrec.nrec = 0;
        if(c_gofwr(&outfd,&length,&hdrtyp,(struct DDR *)NULL,&gofhdr,&adding)
                   != E_SUCC)
            c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
    }
    else
    {
        create = adding = FALSE;
        if (c_gofrd(&outfd,&obyte,&length,&hdrtyp,(struct DDR *)NULL,&outhdr)
            != E_SUCC)
            c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
        filhdr(&gofhdr.hdrec,&outhdr.hdrec);
        lseek(outfd,0,0);
        if(c_gofwr(&outfd,&length,&hdrtyp,(struct DDR *)NULL,&outhdr,&adding)
                   != E_SUCC)
            c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
    }

    /* Set up the processing message
       -----------------------------*/
    sprintf(entity,"%s record",goftxt[gtype]);
    c_inittm("CPYGOF","Processed",entity,"","",&big,&zero,msgtim,&flag);
    curent = 0;
    c_startm();

    status = E_SUCC;
    while (status != E_EOF)
    {
        if (type[gtype] == FPOLIN)
        {
            goftype = HPOLIN;
            status = c_gofrd(&infd,&obyte,&length,&goftype,(struct DDR *)NULL,
                             &gofrec);
            goftype = XYPAIRS;
            status = c_gofrd(&infd,&obyte,&length,&goftype,(struct DDR *)NULL,
                             &gofrec);
        }
        else
        {
            goftype = type[gtype];
            status = c_gofrd(&infd,&obyte,&length,&goftype,(struct DDR *)NULL,
                             &gofrec);
        }
        if (status == E_EOF) break;
        if (status != E_SUCC)
            c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);

        /* Do not copy records that are marked for deletion
           ------------------------------------------------*/
        switch(type[gtype])
        {
            case PNT:
                strcpy(reclabel,gofrec.ptrec.label);
                break;
            case FPOLIN:
                strcpy(reclabel,gofrec.plrec.label);
                break;
            case ANOT:
                strcpy(reclabel,gofrec.antrec.label);
                break;
        }
        if (strcmp(reclabel,DELREC) != 0)
        {
            curent++;
            if (!create) 
                filatt(&gtype,&gofhdr.hdrec,&outhdr.hdrec,&gofrec);
            adding = TRUE;
            lseek(outfd,0,2);
            if(c_gofwr(&outfd,&length,&type[gtype],(struct DDR *)NULL,&gofrec,
                       &adding) != E_SUCC)
                c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
        }
    }
    c_stoptm();

    if(c_gofcls(&infd) != E_SUCC)
        c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
    if(c_gofcls(&outfd) != E_SUCC)
        c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
    return(E_SUCC);
}
