/*******************************************************************************
NAME:           CPYGOF

PURPOSE:        Copies/appends the records from a Graphics Overlay 
                File (GOF) to another GOF.

PROGRAM HISTORY:

VERSION DATE    AUTHOR          REASON
------- -----   ------          ----------------------------
 ---    07/86   D. Hollaren     Initial code
 ---    02/89   L. Huewe        DMS v3_0 retrocode
 ---    09/89   L. Huewe        Increased the size of the 
                                variavble subcmd.
 1.0    12/94   D. Eichacker    Cnversion from DMS to LAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        CPYGOF must be run under TAE.

PROJECT:        LAS

ALGORITHM: 

*******************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
    long gtype;             /* Type of GOF */
    long msgtim;            /* Number of seconds between
                               processing messages */
    long nout = 0;          /* Number of output files created */

    char ingof[CMLEN];      /* Input GOF name */
    char out[1][CMLEN];     /* Output GOF names for completion message */
    char outgof[CMLEN];     /* Output GOF name */
    char subcmd[6];         /* Subcommand */ 

    /* Get the user specified parameters
       ---------------------------------*/
    getpar(subcmd,outgof,&gtype,ingof,&msgtim);

    /* Route to the appropriate subroutines for the subcommand
       -------------------------------------------------------*/
    switch (*subcmd)
    {

        /* Copy all the records from the input GOF to the output GOF.
           No deleted records will be copied.
           ----------------------------------------------------------*/
        case 'G':   /* -GOF */
            if (copy_gof(ingof,gtype,outgof,&msgtim) == E_SUCC) 
            {
                strcpy(out[0], outgof);
                nout++;
            }
            break;

            /* Clean the input GOF by removing deleted records and freeing
               up the space that they used.
               -----------------------------------------------------------*/
        case 'C':   /* -CLEAN */
            clean_gof(ingof,gtype); 
            break;
    }

    /* Display a completion message and terminate the application process
       ------------------------------------------------------------------*/
    c_complt("CPYGOF",out,&nout);

    return EXIT_SUCCESS;
}
