/*******************************************************************************
NAME:           FILATT

PURPOSE:        Fill the output GOF record with the attributes from the input
                GOF record.

*******************************************************************************/
#include "las.h"
#include "gof.h"
#include "protos.h"

int filatt
(
    long *gtype,             /* Type of graphics being processed (I) */
    struct HEADER *gofhdr,   /* Input GOF header structure (I) */
    struct HEADER *outhdr,   /* Output GOF header structure (I) */
    union GOF *gofrec        /* GOF record union structure (I/O) */
)
{
    long index;                  /* Index counter for attribute values array */
    long fatal = -1;             /* Fatal flag for errmsg */
    long i;                      /* Looping variable */
    int search = FALSE;          /* Indicates whether routine is searching
                                    on attribute values */ 
    int slist[MAXATT];           /* Search index list for attribute value */
    char attval[MAXATT][ATTIN];  /* Attribute values */

    /* Initalize the attribute value array
       -----------------------------------*/
    for (i = 0; i < MAXATT; i++)
        strcpy(attval[i],"");

    /* Route to the appropriate graphics type
       --------------------------------------*/
    switch (*gtype)
    {
        case 0:              /* Point graphics       */

            /* Generate an attribute value array in the form ATTNAM:ATTVAL
               -----------------------------------------------------------*/
            for (i = 0, index = 0; i < MAXATT; i++)
                if (strlen(gofrec->ptrec.attrb[i]) != 0)
                {
                    sprintf(attval[index],"%s:%s",gofhdr->attrb[i],
                            gofrec->ptrec.attrb[i]);
                    index++;
                }
            if(c_attchk(outhdr,attval,gofrec->ptrec.attrb,slist,&search) 
                    != E_SUCC)
                c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);

            break;

        case 1:
        case 2:              /* Line or Polygon graphics     */

            /* Generate an attribute value array in the form ATTNAM:ATTVAL
               -----------------------------------------------------------*/
            for (i = 0, index = 0; i < MAXATT; i++)
                if (strlen(gofrec->plrec.attrb[i]) != 0)
                {
                    sprintf(attval[index],"%s:%s",gofhdr->attrb[i],
                            gofrec->plrec.attrb[i]);
                    index++;
                }
            if(c_attchk(outhdr,attval,gofrec->plrec.attrb,slist,&search)
                    != E_SUCC)
                c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
            break;

        case 3:              /* Annotation graphics          */

            /* Generate an attribute value array in the form ATTNAM:ATTVAL
               -----------------------------------------------------------*/
            for (i = 0, index = 0; i < MAXATT; i++)
                if (strlen(gofrec->antrec.attrb[i]) != 0)
                {
                    sprintf(attval[index],"%s:%s",gofhdr->attrb[i],
                            gofrec->antrec.attrb[i]);
                    index++;
                }
            if(c_attchk(outhdr,attval,gofrec->antrec.attrb,slist,&search)
                    != E_SUCC)
                c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
            break;
    }
    return(E_SUCC);
}
