/*******************************************************************************
NAME:           FILHDR

PURPOSE:        Fill the output GOF header record with any attribute names from
                the input GOF header not already in the output GOF header.

*******************************************************************************/
#include "las.h"
#include "gof.h"
#include "protos.h"

int filhdr
(
    struct HEADER *gofhdr,      /* Input file header structure (I) */
    struct HEADER *outhdr       /* Output file header structure (I/O) */
)
{
    long i, j;              /* Looping variables */
    long found;             /* Found attribute name flag */
    long fatal = -1;        /* Abort flag for error messages */
    long nonfatal = 1;      /* Report and continue flag for error messages */

    for (j = 0; j < MAXATT; j++)
    {
        /* If the current input attribute is not null
           ------------------------------------------*/
        if (gofhdr->mask[j] != 0)
        {
            found = FALSE;
            /* Search for the attribute name in the output header file
               -------------------------------------------------------*/
            for (i = 0; i < MAXATT; i++)
                if ((outhdr->mask[i] != 0) &&
                        (outhdr->mask[i] == strlen(gofhdr->attrb[j])) &&
                        (strcmp(outhdr->attrb[i],gofhdr->attrb[j]) == 0))
                {
                    found = TRUE;
                    break;
                }
            /* If the attribute was not found in the output header file it
               must be added
               -----------------------------------------------------------*/
            if (!found)
            {
                for (i = 0; i < MAXATT; i++)
                {
                    if (outhdr->mask[i] == 0)
                    {
                        outhdr->mask[i] = strlen(gofhdr->attrb[j]);
                        strcpy(outhdr->attrb[i],gofhdr->attrb[j]);
                        strcpy(outhdr->attyp[i],gofhdr->attyp[j]);
                        found = TRUE;
                        break;
                    }
                }
            }
            /* If there was no room to place the attribute name into the 
               output header an error results and the process terminates
               ---------------------------------------------------------*/
            if (!found)
            {
                c_errmsg("Maximum number of attributes exceeded",
                         "cpygof-maxatt", &nonfatal);
                c_errmsg("Fatal error encountered","cpygof-fatal",&fatal);
            }
        }
    } 
    return(E_SUCC);
}
