/*******************************************************************************
SUBPROGRAM:     GETPAR 

PURPOSE:        Retrieve the user specified parameters.

*******************************************************************************/
#include "las.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

int getpar
(
    char *subcmd,      /* Subcommand */
    char *outgof,      /* Output GOF name */
    long *gtype,       /* GOF graphics types to process */
    char *ingof,       /* Input GOF name */
    long *msgtim       /* Number of seconds between processing messages */
)
{
    struct PARBLK vblock;     /* Parameter block */
    struct VARIABLE *var;     /* Pointer to parameter in variable block */

    long std_out;             /* Standard output pointer */

    /* Initialize LAS and TAE 
       ----------------------*/
    c_inlas(&vblock,&std_out);

    /* Get the subcommand chosen
       -------------------------*/
    var = p_find(&vblock,"_SUBCMD");
    strcpy(subcmd,SVAL(*var,0));

    /* Get the time between processing messages
       ----------------------------------------*/
    var = p_find(&vblock,"$MSGTIM");
    *msgtim = IVAL(*var,0);

    /* Get the input GOF name 
       ----------------------*/
    strcpy(ingof,"");
    var = p_find(&vblock,"INFILE");
    if ((*var).v_count != 0)
        strcpy(ingof,SVAL(*var,0));

    if (strcmp(subcmd,"CLEAN") != 0)
    {
        /* Get the name of the output file
           -------------------------------*/ 
        var = p_find(&vblock,"OUTFILE");
        strcpy (outgof,SVAL(*var,0));
    }

    /* Get the type(s) of GOFs to process
       ----------------------------------*/
    var = p_find(&vblock, "FTYPE");
    if (strcmp(SVAL(*var,0),"POINT") == 0)
        *gtype = 0;
    else if (strcmp (SVAL(*var,0),"LINE") == 0)
        *gtype = 1;
    else if (strcmp (SVAL(*var,0),"POLY") == 0)
        *gtype = 2;
    else if (strcmp (SVAL(*var,0),"ANNOT") == 0)
        *gtype = 3;

    return(E_SUCC);
}
