/****************************************************************************
NAME:                           DCW2TABE
 
PURPOSE:  Generates a set of label tables from a Digital Chart of the
          World (DCW) query.  
 
PROGRAM HISTORY:
VERSION  DATE    AUTHOR     CODE/CONT   REASON
-------  -----   ------     ---------   -----------------------------
  5.1    11/92   J. Hemmer     CSB      LAS 5.1 Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE
 
PROJECT:  LAS
*****************************************************************************/
#include <stdlib.h>
#include <time.h>
#include "dcw2tab.h"
#include "util.h"
#include "protos.h"
 
/*
 *  *** current DCW categories support
 */
struct DCW_COVERAGE dcw_coverage[] = {
    {"Political/Oceans",        "po", 0, 0, 3, FALSE, NULL},
    {"Drainage",                "dn", 0, 0, 3, FALSE, NULL},
    {"Roads",                   "rd", 0, 0, 2, FALSE, NULL},
    {"Railroads",               "rr", 0, 0, 2, FALSE, NULL},
    {"Hypsography",             "hy", 0, 0, 3, FALSE, NULL},
    {"Hypsography-Sup",         "hs", 0, 0, 2, FALSE, NULL},
    {NULL,         		NULL, 0, 0, 0, FALSE, NULL},
    };

int main
(void)
   {
   struct PARBLK  vblk;         /* TAE parameter block			*/
   long           one,		/* constant value of one		*/ 
                  fatal=-1,	/* fatal error flag			*/ 
                  status,	/* Funciton return status		*/ 
                  std_out;	/* Standard output LUN			*/ 

   double         lllat,        /* lower left latitude			*/
		  lllon,	/* lower left longitude			*/
   		  urlat,        /* upper right latitude			*/
		  urlon;	/* upper right longitude		*/

   char           cdpath[CMLEN],/* location of CD drive			*/
	  	  library[21],	/* DCW library name			*/
   		  outlt[1][CMLEN];	/* DCW library name 			*/
   struct 
   DCW_VECTOR	  dcw_vector;	/* set of line for the given query	*/

  /* 
   *  ***  initialize LAS processing
   */
   c_inlas(&vblk, &std_out);
 
  /* 
   *  *** Get all program parameters (TAE & those derrived from files)
   */
   status = getpar(&vblk, cdpath, library, outlt[0], &lllat, &lllon, 
                   &urlat, &urlon, dcw_coverage);

   if (status != E_SUCC)
      c_errmsg("Fatal error encountered", "dcw2tab-fatal", &fatal);
                   
  /*
   *  *** allocate first line memory block
   *         (if this block is exceeded, it will be reallocated within the 
   *         dcw support routines).
   */
   dcw_vector.maxline   = LINEBLOCK;
   dcw_vector.line      = (struct DCW_LINE*)
                           malloc(LINEBLOCK*sizeof(struct DCW_LINE));

   if (dcw_vector.line == NULL)
      c_errmsg("Error allocating dynamic memory","dcw2tab-alloc",&fatal);

  /*
   *  *** perform the dcw query, loading all lines that fit
   */
   dcw_query(cdpath, library, dcw_coverage, &dcw_vector, 
             lllat, lllon, urlat, urlon);

  /*
   *  *** if lines found, write to label table files
   */
   if (dcw_vector.numlines > 0)
      {
      dcw_writelt(outlt[0], dcw_vector.line, dcw_vector.numlines, dcw_coverage, 
                  lllat, lllon, urlat, urlon);
      }

  /*
   *  *** free lines and coverages
   */
   dcw_freelines(&dcw_vector);
   dcw_freecover(dcw_coverage);

  /* 
   *  *** Write the standard completion message and exit
   */
   c_complt("DCW2TAB", outlt, &one);
   
   return EXIT_SUCCESS;
   
   }


