/****************************************************************************
NAME:				getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.1		11/92	J. Hemmer         CSB       Original development
  5.1		 1/93	J. Hemmer         CSB       Set attribute index

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

*****************************************************************************/
#include <stdio.h>
#include "dcw2tab.h"
#include "ltable.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblk,			/* TAE variable block 		   */
    char *cdpath,		/* location of cd drive		   */
    char *library,		/* DCW library			   */
    char *outlt,		/* output label table filename     */
    double *lllat,		/* lower left latitude 		   */
    double *lllon,		/* lower left longitude		   */
    double *urlat,		/* upper right latitude		   */
    double *urlon,		/* upper right longitude	   */
    struct DCW_COVERAGE *dcw_coverage		/* DCW coverage information        */
)
   {
   struct VARIABLE *var;		/* pointer to a TAE variable in vblk */
   char 	coordstr[2][37],        /* coordinate parameter string     */
		punits[4];		/* buffer for coordinate units	   */
   long		acc,			/* access flag for c_getnam	   */
		attribnum,		/* index of feature attribute	   */
		fatal=-1,		/* fatal error flag  		   */
		i,j,			/* loop counter		  	   */
		len,			/* string length		   */
		status,			/* return status		   */
		zero=0;			/* constant of zero		   */
   
  /*
   *  *** get the CD pathname
   */
   var = p_find(vblk, "CDPATH");
   strcpy(cdpath, SVAL(*var, 0));

  /*
   *  *** get the DCW library name
   */
   var = p_find(vblk, "LIBRARY");
   strcpy(library, SVAL(*var, 0));
   len = (long) strlen(library);
   c_up2low(library, &len);

  /*  
   *  *** Retreive the needed parameters.
   */
   var = p_find(vblk,"GEOUNITS");
   strcpy (punits,SVAL(*var,0));
 
  /*
   *  *** define area of interest
   */
   var = p_find(vblk,"LLCOOR");
   strcpy (coordstr[0],SVAL(*var,0));
   strcpy (coordstr[1],SVAL(*var,1));
   sscanf(coordstr[0],"%lf",lllat);
   sscanf(coordstr[1],"%lf",lllon);
 
  /*  
   *  *** make sure coordinates are in degrees
   */
   if (strcmp(punits, "DEG") != 0)
      {
      status = c_decdeg(lllat,punits,"LAT");
      if (status == E_FAIL)
         c_errmsg("Error converting coordinates","dcw2tab-coord",&fatal);
    
      status = c_decdeg(lllon,punits,"LON");
      if (status == E_FAIL)
         c_errmsg("Error converting coordinates","dcw2tab-coord",&fatal);
      }

  /*
   *  *** define area of interest
   */
   var = p_find(vblk,"URCOOR");
   strcpy (coordstr[0],SVAL(*var,0));
   strcpy (coordstr[1],SVAL(*var,1));
   sscanf(coordstr[0],"%lf",urlat);
   sscanf(coordstr[1],"%lf",urlon);
 
  /*
   *  *** make sure coordinates are in degrees
   */
   if (strcmp(punits, "DEG") != 0)
      {
      status = c_decdeg(urlat,punits,"LAT");
      if (status == E_FAIL)
         c_errmsg("Error converting coordinates","dcw2tab-coord",&fatal);
 
      status = c_decdeg(urlon,punits,"LON");
      if (status == E_FAIL)
         c_errmsg("Error converting coordinates","dcw2tab-coord",&fatal);
      }


  /*
   *  *** identify selected features
   */
   i = 0;
   while (dcw_coverage[i].desc != NULL)
      {
      var = p_find(vblk, dcw_coverage[i].name);
      attribnum = 1;
      if (dcw_definecover(cdpath, library, &dcw_coverage[i]) == E_SUCC)
         {
         for (j = 0; j < (*var).v_count; j++)
            {
            dcw_coverage[i].vdt[IVAL(*var,j)-1].selected = TRUE;
            dcw_coverage[i].vdt[IVAL(*var,j)-1].attindex = attribnum++;
            }
         }

      i++;
      }

  /* 
   *  *** Get the output lt file name
   */
   acc = IWRITE;
   if (c_getnam(vblk, "OUTLT", &zero, outlt, "lt", &acc) != E_SUCC)
   	return(E_FAIL);
   
   return(E_SUCC);
   }
