/*****************************************************************************
NAME: GETPROJ

PURPOSE:  Expands a projection code to the projection name

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 1.0     12/97 B.Olson           C    Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/

#include <string.h>
#include "proj.h"
#include "protos.h"

void getproj
(
    long *proj_code,          /* Projection code               */
    char *string,             /* Output Projection name        */
    double proj_coef[15]     /* Projection coefficients       */
)
{
    switch (*proj_code)
    {
        case GEO   : sprintf(string,"Geographic\n");break;
        case UTM   : sprintf(string,"Universal Transverse Mercator\n");break;
        case SPCS  : sprintf(string,"State Plane Coordinates\n");break;
        case ALBERS: sprintf(string,"Albers Conical Equal Area\n");break;
        case LAMCC : sprintf(string,"Lambert Conformal Conic\n");break;
        case MERCAT: sprintf(string,"Mercator\n");break;
        case PS    : sprintf(string,"Polar Stereographic\n");break;
        case POLYC : sprintf(string,"Polyconic\n");break;
        case EQUIDC: sprintf(string,"Equidistant Conic ");
                     if (proj_coef[8] == 0)
                         strcat(string, "A\n");
                     else
                         strcat(string, "B\n");
                     break;
        case TM    : sprintf(string,"Transverse Mercator\n");break;
        case STEREO: sprintf(string,"Stereographic\n");break;
        case LAMAZ : sprintf(string,"Lambert Azimuthal Equal Area\n");break;
        case AZMEQD: sprintf(string,"Azimuthal Equidistant\n");break;
        case GNOMON: sprintf(string,"Gnomonic\n");break;
        case ORTHO : sprintf(string,"Orthographic\n");break;
        case GVNSP : sprintf(string,"General Vertical Near-Side Perspective\n");
                     break;
        case SNSOID: sprintf(string,"Sinusoidal\n");break;
        case EQRECT: sprintf(string,"Equirectangular\n");break;
        case MILLER: sprintf(string,"Miller Cylindrical\n");break;
        case VGRINT: sprintf(string,"Van der Grinten\n");break;
        case HOM   : sprintf(string,"Oblique Mercator ");
                     if (proj_coef[12] == 0)
                         strcat(string,"A ");
                     else
                         strcat(string,"B ");
                     strcat(string,"(Hotine)\n");
                     break;
        case ROBIN : sprintf(string,"Robinson\n");break;
        case SOM   : sprintf(string,"Space Oblique Mercator ");
                     if (proj_coef[12] == 0)
                         strcat(string,"A\n");
                     else
                         strcat(string,"B\n");
                     break;
        case ALASKA:
                     sprintf(string,
                             "Modified Stereographic Conformal--Alaska\n");
                     break;
        case GOOD  : sprintf(string,"Interrupted Goode Homolosine\n");break;
        case MOLL  : sprintf(string,"Mollweide\n");break;
        case IMOLL : sprintf(string,"Interrupted Mollweide\n");break;
        case HAMMER: sprintf(string,"Hammer\n");break;
        case WAGIV : sprintf(string,"Wagner IV\n");break;
        case WAGVII: sprintf(string,"Wagner VII\n");break;
        case OBEQA : sprintf(string,"Oblated Equal Area\n");break;

        default    : sprintf(string,"User Defined\n");break;
    }
}
