/*****************************************************************************

FUNCTION:	GETPAR

PURPOSE: 	Gets the user-specified input parameters and TAE globals
		needed for the DEBAND application.

ALGORITHM REFERENCES:	None.

*****************************************************************************/

#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAX_BAND        7           /* maximum number of bands in TM scene  */

void getpar
(
    struct PARBLK *parblk,	/* TAE parameter block structure    */
    char hosin[][CMLEN],	/* array of host input image names  */
	char taeout[][CMLEN],		/* TAE name of output image	    */
	char hosout[][CMLEN],		/* host name of output image  	    */
	long *totbnd,		/* total bands in specification     */
	long bands[][MAXBND+1],     /* specified bands for each image */
	long *nbands,		/* number of bands for each image   */
	long *nl,			/* number of lines in each image    */
	long *ns,			/* number of samples in each image  */
	long window[MAXIMG][4],	/* specified window for input image */
	long *nmrimg,		/* number of input images	    */
	float *tol,			/* tolerance value for filter	    */
	long *height,		/* height of filter kernel	    */
	long *msgtim,		/* time between processing messages */
	long *minmax		/* flag for minimum/maximum values  */
)
{

struct VARIABLE *var;		/* TAE variable structure		    */

long dtype;			/* Data type of input image(s)		    */
long maximg = MAXIMG;		/* Maximum number of allowed input images   */
long status;			/* Status flag used to check return values  */
long same;			/* Flag to check if window sizes match	    */
long qflag[MAXIMG][2];		/* Specifies if image is windowed/subbanded */
long zero = 0;			/* Constant of zero			    */

char min_max[4];		/* Value of min/max flag		    */
char taein[MAXIMG][CMLEN];	/* TAE names of input image(s)		    */

/* Obtain the values of needed TAE globals.
   ----------------------------------------*/
var = p_find(parblk, "$msgtim");
*msgtim = IVAL(*var, 0);

var = p_find(parblk, "$minmax");
strcpy(min_max, SVAL(*var, 0));
*minmax = (min_max[0] == 'Y') ? TRUE : FALSE;

/* Get input file specification.
   -----------------------------*/
status = c_getin(parblk, "IN", &zero, taein, hosin, nmrimg, window, bands,
	         nbands, totbnd, &maximg, &same, qflag);
if (status != E_SUCC)
    c_lasmsg(TRUE, "deband-fatal", "Fatal error encountered");
if (same != 1)
    c_lasmsg(TRUE, "deband-window",
	     "Input images do not have the same window sizes");

status = c_gettyp(hosin, nmrimg, &dtype);
if (status != E_SUCC)
    c_lasmsg(TRUE, "deband-input", "Input images are not of same data type");

if (dtype != EBYTE)
    c_lasmsg(TRUE, "deband-dtype", "Data type not supported");

*nl = window[0][NL];
*ns = window[0][NS];

/* Get output file name.
   ---------------------*/
status = c_getout(parblk, "OUT", taeout, hosout);
if (status != E_SUCC)
    c_lasmsg(TRUE, "deband-fatal", "Fatal error encountered");

/* Get kernel parameters.
   ----------------------*/
var = p_find(parblk, "TOLVAL");
*tol = RVAL(*var, 0);

var = p_find(parblk, "HEIGHT");
*height = IVAL(*var, 0);
}
