/******************************************************************************
NAME:           DELATT                          

PURPOSE:        DELATT deletes attribute names from the specified graphics
                overlay file.   

*******************************************************************************/
#include "las.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

int delatt
(
    long gtype,            /* GOF type (I) */
    char attrnam[][ATTIN], /* Attribute names and values to delete by (I) */
    long acount,           /* Number of attributes specified to delete by (I) */
    char *ingof            /* Input name (I) */
)
{
    union GOF header;         /* Header record structure */
    union GOF record;         /* GOF record */

    /* Types of GOF's */
    static char gtext[MAXGOF][6] = {"POINT","LINE","POLY","ANNOT"};

    long del[MAXATT];         /* Indices of attributes to delete */
    long fatal = -1;          /* Flag for fatal error message */
    long found;               /* Was the attribute name found ? */
    long index;               /* Index into DEL array */
    long i,j;                 /* Loop indices */
    long nonfatal = 1;        /* Flag for nonfatal error message */
    long pos;                 /* Current file position */
    long status;              /* Status indicator */

    char lname[CMLEN];        /* Fully qualified LAS GOF name */
    char mesg[35];            /* Error message */

    int fd;                   /* File descriptor for GOF */ 
    int type;                 /* Type of record to read */
    int length;               /* Total length of a record and string length */
    int obyte;                /* Number of bytes actually read */
    int addrec = FALSE;       /* Adding a record to the GOF? */
    int cflag;                /* Indicates whether a GOD was created */
    int mode;                 /* File access mode */

    for (i=0; i < acount; i++)
    {
        length = strlen(attrnam[i]);
        c_low2up(attrnam[i],(long *) &length);
    }

    /* Open the specified graphics overlay file for update access
       -----------------------------------------------------------*/
    mode = UPDATE;
    status = c_gofopn(&fd,ingof,lname,gtext[gtype],&mode,&cflag);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","delgof-fatal",&fatal);

    /* Read the header record from the GOF  
       -----------------------------------*/
    type = HDR; 
    status = c_gofrd(&fd,&obyte,&length,&type,(struct DDR *)NULL,&header);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","delgof-fatal",&fatal);
    for (j=0,index=0; j < acount; j++)
    {
        found = FALSE;

        /* Delete the specified attribute names
           ------------------------------------*/
        for (i=0; i < MAXATT; i++)
            if (header.hdrec.mask[i] != 0) 
                if (header.hdrec.mask[i] == strlen(attrnam[j]))
                    if (strcmp(header.hdrec.attrb[i],attrnam[j]) == 0)
                    {
                        header.hdrec.mask[i] = 0;
                        del[index] = i;
                        index ++;
                        found = TRUE;
                        break;
                    }
        /* Write an error message if any of the attribute 
           names to be deleted cannot be found
           ----------------------------------------------*/
        if (!found)
        {
            sprintf (mesg,"Attribute name %s not found",attrnam[j]);
            c_errmsg (mesg,"delgof-notfnd",&nonfatal);
            c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);
        }
    }

    /* Write the record to the graphics overlay file
       ---------------------------------------------*/
    lseek (fd,0,0);
    if (c_gofwr(&fd,&length,&type,(struct DDR *)NULL,&header,&addrec) != E_SUCC)
        c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);


    /* Delete the attribute values associated with the 
       attribute names that were deleted
       -----------------------------------------------*/
    if ((gtype == 0) || (gtype == 3))
    {
        type = PNT;
        pos = (long) lseek (fd,0,1);
        status = c_gofrd(&fd,&obyte,&length,&type,(struct DDR *)NULL,&record);
        while (status != E_EOF)
        {
            if (status != E_SUCC)
                c_errmsg("Fatal error encountered","delgof-fatal",&fatal);
            for (i=0; i < index; i++)
                strcpy (record.ptrec.attrb[del[i]],"");
            lseek (fd,pos,0);
            if (c_gofwr(&fd,&length,&type,(struct DDR *)NULL,&record,&addrec)
                != E_SUCC)
                c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);
            pos = (long) lseek (fd,length,1);
            status = c_gofrd(&fd,&obyte,&length,&type,(struct DDR *)NULL,
                             &record);
        }
    }
    else
    {
        type = HPOLIN;
        pos = (long) lseek (fd,0,1);
        status = c_gofrd(&fd,&obyte,&length,&type,(struct DDR *)NULL,&record);
        while (status != EOF)
        {
            if (status != E_SUCC)
                c_errmsg("Fatal error encountered","delgof-fatal",&fatal);
            for (i=0; i < index; i++)
                strcpy (record.plrec.attrb[del[i]],"");
            lseek (fd,pos,0);
            if (c_gofwr(&fd,&length,&type,(struct DDR *)NULL,&record,&addrec)
                != E_SUCC)
                c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);
            pos = (long) lseek (fd,length,1);
            status = c_gofrd(&fd,&obyte,&length,&type,(struct DDR *)NULL,
                             &record);
        }
    }

    /* Close the GOF
       -------------*/
    if (c_gofcls(&fd) != E_SUCC)
        c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);

    return(E_SUCC);
}
