/******************************************************************************
NAME:           DELGOF                          

PURPOSE:        DELGOF allows deletion of records from the GOF by record number
                or attribute value. DELGOF also allows deletion of attribute
                names from the GOF.

PROGRAM HISTORY:

VERSION  DATE   AUTHOR
-------  ----   ------
1.0      12/88  L. Huewe        DMS v3_0 retrocode.
2.0      10/94  D. Eichacker    converted form DMS to LAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Delgof must be run under TAE.

PROJECT:        LAS

*******************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
    long acount;                 /* Attribute count */
    long gtype;                  /* GOF type to read */
    long nout = 0;               /* Number of output files created */
    long nrec;                   /* Number of records in the GOF */
    long recnum[50];             /* Record numbers to be deleted */

    char attrnam[MAXATT][ATTIN]; /* User defined attribute names */
    char attrval[MAXATT][ATTIN]; /* User specified attribute names and values */
    char ingof[CMLEN];           /* Input GOF name */
    char subcmd[7];              /* User input subcommand */


    /* Get the user specified parameters
       ---------------------------------*/
    getpar (subcmd,&gtype,recnum,&nrec,attrval,attrnam,&acount,ingof);

    if (*subcmd == 'R')
    {
        /* Call the function to mark the record to be deleted 
           --------------------------------------------------*/
        delrec (gtype,recnum,nrec,attrval,acount,ingof);

        /* Clean the input GOF by removing deleted records and freeing
           up the space that they used.
           -----------------------------------------------------------*/
        if (clean_gof(ingof,gtype) == E_SUCC)
            nout++;
    }
    else
    {
        if (delatt(gtype,attrnam,acount,ingof) == E_SUCC)
            nout++;
    }

    /* Display a completion message and terminate the application process
       ------------------------------------------------------------------*/
    c_complt("DELGOF",ingof,&nout);

    return EXIT_SUCCESS;
}
