/******************************************************************************
NAME:           DELREC                          

PURPOSE: Mark the appropriated records for deletion

*******************************************************************************/
#include "las.h"
#include "gof.h"
#include "protos.h"

int delrec
(
    long gtype,            /* GOF type (I) */
    long recnum[],         /* Record numbers to delete (I) */
    long nrec,             /* Number of records to delete (I) */
    char attrval[][ATTIN], /* Attribute names and values to delete by (I) */
    long acount,           /* Number of attributes specified to delete by (I) */
    char *ingof            /* Input name (I) */
)
{
    union GOF gofrecord;         /* GOF record structure */

    /* Types of GOF's */
    static char gtext[MAXGOF][6] = {"POINT","LINE","POLY","ANNOT"};

    long current=0;              /* Current index into RECNUM */
    long fatal = -1;             /* Indicates fatal error */
    long i;                      /* Loop index */
    long nonfatal= 1;            /* Indicates non fatal error */
    long pos;                    /* File position */
    long rec;                    /* Record number read */
    long status;                 /* Status indicator */ 

    char lasnam[CMLEN];          /* Fully qualified LAS GOF name */
    char mesg[65];               /* Error message */
    char sattr[MAXATT][ATTVAL];  /* Verified attributes */

    long delete = FALSE;         /* Delete record ? */

    int gfd;                     /* Graphics overlay file descriptor */
    int rtype;                   /* Type of record to read/write */
    int length;                  /* Number of bytes to next record */
    int obyte;                   /* Number of bytes read/written */
    int addrec = FALSE;          /* Indicates whether adding records to
                                    the GOF */
    int sflag = TRUE;            /* Search flag for c_attchk */
    int slist[MAXATT];           /* List of attribute indices */
    int cflag = 0;               /* Flag indicating whether GOF was created */
    int mode;                    /* File access mode */


    /* Open the GOF for update access and read the header record
       ---------------------------------------------------------*/
    mode = UPDATE;
    if (c_gofopn(&gfd,ingof,lasnam,gtext[gtype],&mode,&cflag) != E_SUCC)
        c_errmsg("Fatal error encountered","delgof-fatal",&fatal);
    rtype = HDR;
    if (c_gofrd(&gfd,&obyte,&length,&rtype,(struct DDR *)NULL,&gofrecord)
        != E_SUCC)
        c_errmsg("Fatal error encountered","delgof-fatal",&fatal);
    for (i=0; i < nrec; i++)
        if (recnum[i] > gofrecord.hdrec.nrec) 
        {
            sprintf (mesg,"Record number %d does not exist",recnum[i]);
            c_errmsg (mesg,"delgof-invrec",&nonfatal);
            c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);
        }

    /* Verify the attribute names and values specified
       ------------------------------------------------*/
    if (c_attchk(&gofrecord.hdrec,attrval,sattr,slist,&sflag) != E_SUCC)
        c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);

    /* Set the record type to read and write
       ---------------------------------------*/
    if (gtype == 0)
        rtype = PNT;
    else if ((gtype == 1) || (gtype == 2))
        rtype = HPOLIN;
    else
        rtype = ANOT;

    /* Save the current file position and read the GOF record
       --------------------------------------------------------*/
    pos = (long) lseek (gfd,0,1);
    status = c_gofrd (&gfd,&obyte,&length,&rtype,(struct DDR *)NULL,&gofrecord);
    rec = 1;
    while (status != E_EOF) 
    {
        if (status != E_SUCC)
            c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);

        /* Determine if the current record is to be deleted
           --------------------------------------------------*/
        if (recnum[current] == rec) 
        {
            delete = TRUE;
            current++;
        }
        else /* Check if the current record has any 
                of the attribute values specified */
            for (i=0; i < acount; i++)
                switch (rtype)
                {
                    case PNT:
                        if ((strcmp(sattr[i],
                                    gofrecord.ptrec.attrb[slist[i]]) == 0) || 
                            ((strcmp(sattr[i],ATHAT) == 0) && 
                             (strcmp(gofrecord.ptrec.attrb[slist[i]],"") != 0)))
                        {
                            delete = TRUE;
                            break;
                        }
                        break;
                    case HPOLIN:
                        if ((strcmp(sattr[i],
                                    gofrecord.plrec.attrb[slist[i]]) == 0) || 
                            ((strcmp(sattr[i],ATHAT) == 0) && 
                             (strcmp(gofrecord.plrec.attrb[slist[i]],"") != 0)))
                        {
                            delete = TRUE;
                            break;
                        }
                        break;
                    case ANOT:
                        if ((strcmp(sattr[i],
                                    gofrecord.antrec.attrb[slist[i]]) == 0) || 
                            ((strcmp(sattr[i],ATHAT) == 0) && 
                             (strcmp(gofrecord.antrec.attrb[slist[i]],
                                     "") != 0)))
                        {
                            delete = TRUE;
                            break;
                        }
                        break;
                }
        /* Delete the record
           -------------------*/
        if (delete)
        {
            lseek (gfd,pos,0);
            switch (rtype)
            {
                case PNT:
                    strcpy (gofrecord.ptrec.label,DELREC);
                    break;
                case HPOLIN:
                    strcpy (gofrecord.plrec.label,DELREC);
                    break;
                case ANOT:
                    strcpy (gofrecord.antrec.label,DELREC);
                    break;
            }
            if (c_gofwr(&gfd,&length,&rtype,(struct DDR *)NULL,&gofrecord,
                        &addrec) != E_SUCC)
                c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);
        }
        /* Read the next record
           ---------------------*/
        pos = (long) lseek (gfd,length,1);
        status = c_gofrd (&gfd,&obyte,&length,&rtype,(struct DDR *)NULL,
                          &gofrecord);
        rec++;
        delete = FALSE;
    }
    /* Close the graphics overlay file
       ---------------------------------*/
    if (c_gofcls(&gfd) != E_SUCC)
        c_errmsg ("Fatal error encountered","delgof-fatal",&fatal);
    return(E_SUCC);
}

