/*******************************************************************************
SUBPROGRAM:                 GETPAR

PURPOSE:        Retrieve the user specified parameters.

*******************************************************************************/
#include <string.h>
#include "las.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

int getpar
(
    char *subcmd,           /* The subcommand */
    long *goftyp,           /* Initial type of GOF to process */
    long recnum[50],        /* Record numbers to delete */
    long *nrec,             /* Number of records to delete */
    char attrval[][ATTIN],  /* User specified attribute names and values */
    char attrnam[][ATTIN],  /* User defined attribute names */
    long *acount,           /* Attribute count */
    char *ingof             /* GOF name whose records are to be deleted */
)
{
    struct PARBLK vblock;       /* Parameter block */
    struct VARIABLE *var;       /* Pointer to the parameter in the
                                   parameter block */

    /* Types of GOF's */
    static char gtext[MAXGOF][6] = {"point","line","poly","annot"};

    char ext[12];               /* Infile extension */

    long fatal = -1;            /* Indicates fatal error */
    long errf = -1;             /* Error flag for Record and atribute
                                   specification */
    long i;                     /* Loop index */
    long std_out;               /* Standard output */
    long zero = 0;              /* the value zero */

    /* Initialize TAE
       --------------*/
    for (i=0; i < 50; i++)
        recnum[i]=-1;
    for (i=0; i < MAXATT; i++)
        strcpy (attrval[i],"");

    c_inlas (&vblock,&std_out);

    /* Get the subcommand
       ------------------*/
    var = p_find(&vblock,"_SUBCMD");
    strcpy(subcmd,SVAL(*var,0));


    /* Get the value of the parameter FTYPE from the parameter block
       -------------------------------------------------------------*/
    var = p_find(&vblock,"FTYPE");
    if (strcmp (SVAL(*var,0),"POINT") == 0)
        *goftyp = 0;
    else if (strcmp (SVAL(*var,0),"LINE") == 0)
        *goftyp = 1;
    else if (strcmp (SVAL(*var,0),"POLY") == 0)
        *goftyp = 2;
    else if (strcmp (SVAL(*var,0),"ANNOT") == 0)
        *goftyp = 3;

    /* Get the value of the parameter INFILE from the parameter block
       --------------------------------------------------------------*/
    strcpy(ext,"gof_");
    strcat(ext,gtext[*goftyp]);

    if (c_getnam(&vblock,"INFILE",&zero,ingof,ext,&zero) != E_SUCC)
        c_errmsg("Fatal error encountered","delgof-fatal",&fatal);

    if (*subcmd == 'R')
    {
        /* Get the value of the parameter RECNUM from the parameter
           block and sort the record numbers in ascending order
           --------------------------------------------------------*/
        var = p_find(&vblock,"RECNUM");
        *nrec = (*var).v_count;
        if (*nrec > 0)
            errf = E_SUCC;
        for (i=0; i < *nrec; i++)
            recnum[i] = IVAL(*var,i);

        /* Sort the record numbers
           -----------------------*/
        sort(recnum,nrec);

        /* Get the value of the parameter ATRVAL from the parameter block
           --------------------------------------------------------------*/
        var = p_find (&vblock,"ATRVAL");
        *acount = (*var).v_count;
        if (*acount > 0 )
            errf = E_SUCC;
        for (i=0; i < *acount; i++)
            strcpy (attrval[i],SVAL(*var,i));

        /* If RECNUM and ATRVAL were not specified error out
           -------------------------------------------------*/

        if (errf != E_SUCC)
            c_errmsg("No record number or attribute value specifed",
                    "delgof-nospec",&fatal);
    }
    else /* subcmd == 'A'  */
    {
        /* Get the attribute names from the parameter block
           ------------------------------------------------*/
        var = p_find(&vblock, "ATRNAME");
        *acount = (*var).v_count;
        for (i=0; i < *acount; i++)
            strcpy (attrnam[i],SVAL(*var,i));
    }
    return(E_SUCC);
}
