/******************************************************************************

NAME:	DELTAB

FUNCTION:
	DELTAB deletes the description and data Labeled Table files.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         11/86      K. Gacke     original development
    1.1          1/87	   K. Gacke     new LT file format
    2.0		 2/89	   B. Ailts	LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM/DESCRIPTION:
	The labeled table file name is retrieved from TAE parameter block
	The labeled table subfile names are read
	For every subfile
	   The subfile is deleted
	Endif
	The labeled table is deleted
	Completion message is displayed
******************************************************************************/
  
#include <stdio.h> 

#include "las.h"
#include "ltable.h"

main()
{
struct TAB_DEF tab;		/*  tab definition structure		*/
struct PARBLK vblock;		/*  TAE parameter block			*/
struct VARIABLE *p_find();	/*  function to locate a parameter within the
				    TAE parameter block			*/
long acc = IREAD;		/*  access to labeled table file	*/
long fatal = -1;		/*  fatal error code			*/
long index;			/*  counter -- current file		*/
long one = 1;			/*  value of one			*/
long stdlun;			/*  standard output			*/
long status;			/*  status of called functions		*/
long zero = 0;			/*  value of zero			*/

char name[CMLEN]; 		/*  Host name of input file		*/
char msg[ERRLEN];		/*  error message buffer		*/

 
c_inlas(&vblock,&stdlun);		/* initialize TAE and LAS             */
status = c_getnam(&vblock,"INLT",&zero,name,"lt",&acc);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","deltab-fatal",&fatal);
sprintf(msg,"Deleting labeled table %s and its associated files", name);
c_errmsg(msg,"informational",&one);
open_tab(&tab,name,IREAD);

/*  delete each subfile  
-----------------------*/
for (index=0; index<tab.nsubs; index++)
   {
   c_delfil(tab.subfile[index]);
   }

/*  Delete the input file
-------------------------*/
c_delfil(name);
c_complt("DELTAB"," ",&zero);
}
