/******************************************************************************
SUBROUTINE CDEMA
 
PURPOSE:
 	Subroutine CDEMA processes DEM type C records in Ascii
      format. The routine reads and converts the data elements in 
      the record and places them into a DEM structure The contents 
      of this DEM structure will be displayed to the user and 
      written to a labeled table file.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
DESCRIPTION/ALGORITHM:

If the buffer reference pointer (BUFREF) is at the end of a block THEN 
   Reset it to the beginning of the next block.
End if

Read the data element (SW2)    from the Ascii buffer
If SW2 == 1
   Read the data element (ARMSEX) from the Ascii buffer
   Read the data element (ARMSEY) from the Ascii buffer
   Read the data element (ARMSEZ) from the Ascii buffer
   Read the data element (SIZE2)  from the Ascii buffer
End if
Read the data element (SW5)    from the Ascii buffer
If SW5 == 1
   Read the data element (RRMSEX) from the Ascii buffer
   Read the data element (RRMSEY) from the Ascii buffer
   Read the data element (RRMSEZ) from the Ascii buffer
   Read the data element (SIZE5)  from the Ascii buffer
End if

return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dementer.h"

void cdema
(
  long *unit,                 /* I: Tape unit number */
  char *record,               /* I: ASCII record buffer */
  long *bufref,               /* I/O: Reference buffer pointer */
  long *bufsiz,               /* I: Current buffer size */
  struct VLL_DEM *vll_dem,    /* I: DEM header structure */
  FILE *infp                  /* I: Input data file pointer */
)
{  /*  cdema  */

    long inc;                 /* */
    long j;                   /* counter -- current block   */

    short nbuf;               /* new buffer flag            */

    /*  Ascii data varies in length but is always in increments of 1024 bytes.
        Set the buffer reference pointer and read the next record
        Read the next record if the C record begins on a new record
        --------------------------------------------------------------------*/
    inc = (long)(*bufsiz / 1024. + .5);
    nbuf = TRUE;
    for (j = 1; j <= inc - 1; j++)
    {
        if((*bufref > (1024*(j-1))) && (*bufref <= (1024*j) - 1))
        {
            *bufref = 1024 * j;
            nbuf = FALSE;
            break;
        }
    }

    if (nbuf)
        newbuf(record,bufref,bufsiz,unit,infp);

    /*  Reset the buffer pointer
        ----------------------------*/
    for (j = 0; j < inc - 1; j++)
        if ((*bufref > (1024 * (j - 1)) + 1) && (*bufref <= (1024 * j)))
            *bufref = 1024 * j;

    /*  Read the confirmation data into the VLL buffer
        --------------------------------------------------*/
    sscanf(&record[*bufref],"%hd",&vll_dem->sw2);
    if (vll_dem->sw2 == 1)
    {
        sscanf(&record[*bufref + 6],"%hd",&vll_dem->armsex);
        sscanf(&record[*bufref + 12],"%hd",&vll_dem->armsey);
        sscanf(&record[*bufref + 18],"%hd",&vll_dem->armsez);
        sscanf(&record[*bufref + 24],"%hd",&vll_dem->size2);
    }

    sscanf(&record[*bufref + 30],"%hd",&vll_dem->sw5);
    if (vll_dem->sw5 == 1)
    {
        sscanf(&record[*bufref + 36],"%hd",&vll_dem->rrmsex);
        sscanf(&record[*bufref + 42],"%hd",&vll_dem->rrmsey);
        sscanf(&record[*bufref + 48],"%hd",&vll_dem->rrmsez);
        sscanf(&record[*bufref + 54],"%hd",&vll_dem->size5);
    }
}  /* cdema  */
