/******************************************************************************
SUBROUTINE CDEMB
 
PURPOSE:
 	Subroutine CDEMB processes DEM type C records in Binary
      format. The routine reads and converts the data elements 
      in the record and places them into the DEM structure.  The
      contents of this structure will be displayed to the user and
      optionally written to a labeled table file.
 
PROJECT:   LAS 
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
DESCRIPTION/ALGORITHM:

Copy data into the DEM structure
 
Convert the members of the C portion of the DEM structure

return
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "dementer.h"

void cdemb
(
  long *unit,                 /* I: Tape unit number */
  short *buff2,               /* I: Input buffer */
  long *bufref,               /* O: Buffer reference pointer */
  struct VLL_DEM *vll_dem,    /* I: Struct containing A and C record info */
  long *outsys,               /* I: Output system */
  FILE *infp                  /* I: Input data file pointer */
)
{  /*  ademb  */

    static long fatal = -1;      /*  fatal error code                    */
    static long insys = SYS_IBM_MVS; /* input system                     */
    static long handle = HANDLE; /*  handle flag of tapeio rountines     */
    static long flag = SYS_USE;  /*  values provided r over/under flow   */
    static long nonfatal = 1;    /*  nonfatal error code                 */
    static long rewopt = REWOFFL; /* rewind option                       */
    static long zero = 0;        /*  value of 0                          */
    static short sover = MAXWORD;/*  value if over flow                  */
    static short sunder = MINWORD;/* value if under flow                 */

    long dtype = EWORD;           /*  data type of data                  */
    long nelem = 10;              /* number of elements                  */
    long status;

    /*  Copy the last 20 bytes into the VLL buffer
        ----------------------------------------------*/
    *bufref += 2;
    c_pxcopy((unsigned char *)&buff2[*bufref],(unsigned char *)&vll_dem->sw2,
            &dtype,&nelem);

    /*  re-fomat the values if needed
        ---------------------------------*/
    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dem->sw2,&dtype,
            &nelem,&sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if (infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }
}  /*  cdemb  */
