/*******************************************************************************
SUBROUTINE CDTT
 
PURPOSE:
 	Subroutine CDTT processes DTT type C data. Data elements
      within the record are read and converted to the host format,
      and are placed into the members of the DTT structure. The
      DTT information is displayed to the user and written to a
      labeled table file.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
ALGORITM DESCRIPTION:
copy last six entries in the DTT structure
convert the last 6 variables
return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "dementer.h"

void cdtt
(
  long *unit,                 /* I: Tape unit number */
  short *buff2,               /* I: Input buffer */
  long *bufref,               /* O: Buffer reference pointer */
  struct VLL_DTT *vll_dtt,    /* I: Struct containing A and C record info */
  long *outsys,               /* I: Output system */
  FILE *infp                  /* I: Input data file pointer */
)
{  /* cdtt  */

    static long fatal = -1;      /*  fatal error code                    */
    static long flag = SYS_USE;  /*  values provided r over/under flow   */
    static long handle = HANDLE; /*  handle flag of tapeio rountines     */
    static long insys = SYS_IBM_MVS; /* input system                     */
    static long nonfatal = 1;    /*  nonfatal error code                 */
    static long rewopt = REWOFFL; /* rewind option                       */
    static long zero = 0;        /*  value of 0                          */
    static short sover = MAXWORD;/*  value if over flow                  */
    static short sunder = MINWORD;/* value if under flow                 */

    long dtype = EWORD;	          /*  data type of variables             */
    long i2byts = 6;              /*  number of WORDS to be copied       */
    long status;                  /*  status of function calls           */

    /*  Copy the last 12 bytes into the VLL buffer
        ----------------------------------------------*/
    *bufref = *bufref + 2;
    c_pxcopy((unsigned char *)&buff2[*bufref],(unsigned char *)&vll_dtt->str1,
            &dtype,&i2byts);

    status = c_pxsys(&insys,outsys,(unsigned char *)&vll_dtt->str1,&dtype,
            &i2byts,&sunder,&sover,&flag);
    if (status != E_SUCC)
    {
        c_errmsg("Error encountered converting data","dementer-convert",
                &nonfatal);
        if ( infp == NULL)
            status = c_tpclos(unit,&handle,&rewopt,&zero);
        else
            fclose(infp);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }
}  /*  cdtt  */
