/******************************************************************************
SUBROUTINE DDRDTED
 
PURPOSE:
		To fill the individual fields of the DDR for DTED data
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
 5.0 05/31/89  B. Ailts		CSB	      Original development
 5.1 07/11/90  D. Etrheim	CSB           Changed proj_units to "seconds"
 5.2 01/17/90  D. Etrheim	CSB           Make zone and datum codes valid
 5.3 12/13/91  D. Etrheim	CSB           Correct the corner coordinates
 
ALGORITHM DESCRIPTION:
Retrieve the DDR information
Retrieve the DDR band information

Place  the values from the DEM structure into the DDR
Copy the SW corner coordinates from the DEM structure

Fill in the DDR band information

Write the DDR information
Write the DDR band information
return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "dementer.h"

void ddrdted
(
  char *hosout,                /* Host output file name */
  struct VLL_DTED *vll_dted    /* DTED header struct */
)
{  /* ddrdted  */

    static long one = 1;            /*  value of 1                          */
    extern struct PARBLK vblock;    /*  TAE parameter block			        */
    struct DDR *ddr;                /*  DDR structure			            */
    struct BDDR *bddr;              /*  BDDR satructure			            */

    long deg;                       /*  degree portion of DMS string        */
    long dtmnum = 0;
    long i;                         /*  counter--current projection parameter */
    long minval;                    /*  minute portion of DMS string        */
    long sec;                       /*  second portion of DMS string        */
    long status;                    /*  status of function calls            */

    char chrtemp[4];                /*  temporary character buffer          */

    /*  Allocate space for the DDR structures
        -----------------------------------------*/
    ddr = (struct DDR *) malloc(sizeof(struct DDR));
    bddr = (struct BDDR *) malloc(sizeof(struct BDDR));

    /*  Retrieve DDR information
        ----------------------------*/
    status = c_getddr(hosout,ddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    status = c_getbdr(hosout,bddr,&one);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    /*  Set master units
        ------------------------*/
    ddr->master_line = 1;
    ddr->master_sample = 1;

    /*  Set projection units
        ------------------------*/
    ddr->valid[DDPUV] = VALID;
    strcpy(ddr->proj_units,"seconds");

    ddr->valid[DDPCV] = VALID;
    ddr->proj_code = 0;

    ddr->valid[DDZCV] = VALID;
    ddr->zone_code = 62;

    /* Get the datum number
       -----------------------*/
    c_dtmnum(vll_dted->datum,&dtmnum);
    if (dtmnum != -1)
    {
        ddr->valid[DDDCV] = VALID;
        ddr->datum_code = dtmnum;
    }
    else
    {
        ddr->valid[DDDCV] = INVAL;
        ddr->datum_code = 0;
    }

    ddr->valid[DDPDV] = VALID;
    ddr->pdist_y = vll_dted->lats / 10;
    ddr->pdist_x = vll_dted->longs / 10;

    ddr->valid[DDPPV] = VALID;
    for (i = 0; i < 15; i ++)
        ddr->proj_coef[i] = 0.0;

    /*  Set the SW corner coordinate
        -------------------------------*/
    /*  Convert to total seconds
        ----------------------------*/
    ddr->valid[DDCCV] = VALID;
    strncpy(chrtemp,&vll_dted->swlat[0],3);
    chrtemp[3] = ' ';
    sscanf(chrtemp,"%d",&deg);
    strncpy(chrtemp,&vll_dted->swlat[3],2);
    chrtemp[2] = ' ';
    sscanf(chrtemp,"%d",&minval);
    strncpy(chrtemp,&vll_dted->swlat[5],2);
    chrtemp[2] = ' ';
    sscanf(chrtemp,"%d",&sec);
    if (vll_dted->swlat[7] == 'N')
        ddr->loleft[0] = (deg * 3600) + (minval * 60) + sec;
    else
        ddr->loleft[0] = ((deg * 3600) + (minval * 60) + sec) * -1;

    strncpy(chrtemp,&vll_dted->swlong[0],3);
    chrtemp[3] = ' ';
    sscanf(chrtemp,"%d",&deg);
    strncpy(chrtemp,&vll_dted->swlong[3],2);
    chrtemp[2] = ' ';
    sscanf(chrtemp,"%d",&minval);
    strncpy(chrtemp,&vll_dted->swlong[5],2);
    chrtemp[2] = ' ';
    sscanf(chrtemp,"%d",&sec);
    if (vll_dted->swlong[7] == 'E')
        ddr->loleft[1] = (deg * 3600) + (minval * 60) + sec;
    else
        ddr->loleft[1] = ((deg * 3600) + (minval * 60) + sec) * -1;

    /*  Calculate the other corner values
        -------------------------------------*/
    ddr->upleft[0] = ddr->loleft[0] + ((ddr->nl - 1) * ddr->pdist_y);

    ddr->upleft[1] = ddr->loleft[1];
    ddr->upright[0] = ddr->upleft[0];

    ddr->upright[1] = ddr->loleft[1] + ((ddr->ns - 1) * ddr->pdist_x);

    ddr->loright[0] = ddr->loleft[0];
    ddr->loright[1] = ddr->upright[1];

    ddr->valid[DDINCV] = VALID;
    ddr->line_inc = 1.0;
    ddr->sample_inc = 1.0;

    strcpy(bddr->source,vll_dted->machin);
    sprintf(bddr->instrument,"reel = %d, file = %d",vll_dted->reel,
            vll_dted->file);
    strcpy(bddr->date,vll_dted->cdate);

    status = c_putddr(hosout,ddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    status = c_putbdr(hosout,bddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    free(ddr);
    free(bddr);
}  /*  ddrdtt  */
