/******************************************************************************
SUBROUTINE DDRDTT
 
PURPOSE:
		To fill the individual fields of the DDR for DTED data
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
 5.0 05/31/89  B. Ailts		CSB	      Original development
 5.1 07/11/90  D. Etrheim	CSB           Changed proj_units to "seconds"
 5.2 01/17/90  D. Etrheim	CSB           Make zone and datum codes valid
 
ALGORITHM DESCRIPTION:
Retrieve the DDR information
Retrieve the DDR band information

Place  the values from the DTT structure into the DDR

Fill in the DDR band information

Write the DDR information
Write the DDR band information
return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "dementer.h"

void ddrdtt
(
  char *hosout,                /* I: Host output file name */
  struct VLL_DTT *vll_dtt,     /* I: DTT header struct */
  long *ddrvflg                /* I: DDR validity flag */
)
{  /* ddrdtt  */

    static long one = 1;            /*  value of 1                      */
    extern struct PARBLK vblock;    /* TAE parameter block              */
    struct DDR *ddr;                /*  DDR structure                   */
    struct BDDR *bddr;              /*  BDDR structure                  */

    long i;                         /*  counter -- current	projection
                                        parameter                       */
    long status;                    /*  status of function calls        */

    /*  Allocate space for the DDR structures
        -----------------------------------------*/
    ddr = (struct DDR *) malloc(sizeof(struct DDR));
    bddr = (struct BDDR *) malloc(sizeof(struct BDDR));

    /*  Retrieve DDR information
        ----------------------------*/
    status = c_getddr(hosout,ddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    status = c_getbdr(hosout,bddr,&one);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    /*  Set master units
        --------------------*/
    ddr->master_line = 1;
    ddr->master_sample = 1;

    /*  Set projection units
        ------------------------*/
    ddr->valid[DDPUV] = VALID;
    strcpy(ddr->proj_units,"seconds");

    ddr->valid[DDPCV] = VALID;
    ddr->proj_code = vll_dtt->mpc;

    ddr->valid[DDZCV] = VALID;
    ddr->zone_code = vll_dtt->mpz;

    ddr->valid[DDDCV] = VALID;
    ddr->datum_code = 0;

    ddr->valid[DDPDV] = *ddrvflg;
    ddr->pdist_y = 0.0;
    ddr->pdist_x = 0.0;

    ddr->valid[DDPPV] = VALID;
    for (i = 0; i < 15; i ++)
        ddr->proj_coef[i] = 0.0;

    ddr->valid[DDCCV] = VALID;
    ddr->upleft[0] = vll_dtt->lanw;
    ddr->upleft[1] = vll_dtt->lanw;
    ddr->loleft[0] = vll_dtt->lasw;
    ddr->loleft[1] = vll_dtt->losw;
    ddr->upright[0] = vll_dtt->lane;
    ddr->upright[1] = vll_dtt->lone;
    ddr->loright[0] = vll_dtt->lase;
    ddr->loright[1] = vll_dtt->lose;

    ddr->valid[DDINCV] = VALID;
    ddr->line_inc = 1.0;
    ddr->sample_inc = 1.0;

    strcpy(bddr->source,"DTT 1:250000");

    status = c_putddr(hosout,ddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    status = c_putbdr(hosout,bddr);
    if (status != E_SUCC)
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);

    free(ddr);
    free(bddr);
}  /*  ddrdtt  */
