#ifndef DEMENTER_H
#define DEMENTER_H

#include "sysdef.h"
#include "typlim.h"
#include "tapeio.h"

#define DTT 1
#define DEM 2
#define DTED 3

#define DEMMAX(A, B)    ((A) > (B) ? (A) : (B)) /* assign maximum of a and b */
#define DEMMIN(A, B)    ((A) < (B) ? (A) : (B)) /* assign minimum of a and b */

/*  The Header structures for each type of data are defined below
    -----------------------------------------------------------------*/
struct VLL_DTT
{
   char sht[13];
   char mps[7];
   char mpe[7];
   short mpc;
   short mpz;
   short evc;
   short xsw;
   short ysw;
   short xnw;
   short ynw;
   short xne;
   short yne;
   short xse;
   short yse;
   long losw;
   long lasw;
   long lonw;
   long lanw;
   long lone;
   long lane;
   long lose;
   long lase;
   short str1;
   short str2;
   short str3;
   short str4;
   short fill;
   short brec;
};

struct VLL_DTED
{
   char fname[18];
   char machin[7];
   char datum[50];
   long reel;
   long file;
   char cdate[7];
   char edate[7];
   long access;
   char swlong[9];
   char swlat[9];
   long longs;
   long lats;
   long acode;
   char scode[4];
   long patchs;
   long points;
};

struct VLL_DEM
{
   char crec[144];
   short lvl;
   short epat;
   short gref;
   short zref;
   double ppar[15];
   short puni;
   short euni;
   short side;
   double corner[4][2];
   double mine;
   double maxe;
   double ccw;
   short cdata;
   float xres;
   float yres;
   float zres;
   short prow;
   short pcol;
   short sw2;
   short armsex;
   short armsey;
   short armsez;
   short size2;
   short sw5;
   short rrmsex;
   short rrmsey;
   short rrmsez;
   short size5;
};

/* constants for DTED data */
#define DTED_HDR_LEN  80
#define DTED_UHL_LEN  80
#define DTED_DSI_LEN  648
#define DTED_ACC_LEN  2700

#define DTED_HDR_OFF  0
#define DTED_UHL_OFF  80
#define DTED_DSI_OFF  160
#define DTED_ACC_OFF  808
#define DTED_DATA_OFF 3508


/* Prototypes for the DEMENTER C files.
   las.h MUST be included before this
   file
--------------------------------------*/


/* adem.c */
void adem
(
  short *buff2,                 /* I: Input buffer */
  long *unit,                   /* I: Tape unit number */
  long *bufsiz,                 /* O: Current size of the buffer */
  long *bufref,                 /* O: Reference pointer to the input buffer */
  long *lines,                  /* O: Number of input lines */
  long *samps,                  /* O: Number of input samples */
  short *form,                  /* O: Format of the data */
  float *deltay,                /* O: Resolution in the Y direction */
  float *deltaz,                /* O: Resolution in the Z direction */
  short *cdata,
  double *ymin,                 /* O: Minimum Y location */ 
  struct VLL_DEM *vll_dem,      /* I: DEM header struct */
  long *outsys,                 /* I: Output system */
  FILE *infp                    /* I: Input data file pointer */
);

/* adema.c */
void adema
(
  struct VLL_DEM *vll_dem,      /* I: Struct defining record A of DEM data */
  char *record,                 /* I: Input ASCII record */
  float *deltay,                /* O: Resolution in the Y direction */
  float *deltaz,                /* O: Resolution in the Z direction */
  long *bufref,                 /* O: Buffer reference pointer */
  short *cdat,
  long *unit,                   /* I: Tape unit */
  FILE *infp                    /* I: Input data file pointer */
);

/* ademb.c */
void ademb
(
  struct VLL_DEM *vll_dem,      /* I: Struct holding record A data */
  short *buff2,                 /* I: Buffer */
  long *bufsiz,                 /* O: Current size of the buffer */
  long *bufref,                 /* O: Buffer reference pointer */
  float *deltay,                /* O: Resolution in the Y direction */
  float *deltaz,                /* O: Resolution in the Z direction */
  short *cdat,
  long *unit,                   /* I: Tape unit number */
  long *outsys,                 /* I: Output system */
  FILE *infp                    /* I: Input data file pointer */
);

/* adted.c */
int adted
(
  long *unit,                   /* I: Tape unit number */
  long *lines,                  /* O: Number of input lines */
  long *samps,                  /* O: Number of input samples */
  struct VLL_DTED *vll_dted,    /* O: DTED header struct */
  FILE *infp                    /* I: Input data file pointer */
);

/* adtt.c */
void adtt
(
  long *unit,                  /* I: Tape unit number */
  short *buff2,                /* I: Buffer */
  long *bufsiz,                /* O: Current buffer size */
  long *bufref,                /* O: Reference buffer pointer */
  long *lines,                 /* O: Number of input lines */
  long *samps,                 /* O: Number of input samples */
  float *deltay,               /* O: Resolution in the Y direction */
  float *deltaz,               /* O: Resolution in the Z direction */
  double *ymin,                /* O: Minimum value in Y direction */
  struct VLL_DTT *vll_dtt,     /* I: DTT header struct */
  long *outsys,                /* I: Output system */
  FILE *infp                   /* I: Input data file pointer */
);

/* bdema.c */
void bdema
(
  unsigned char *patch,   /* I: Temp buffer */
  long *lin,              /* I: Current line */
  char *record,           /* O: Input buffer */
  long *bufsiz,           /* O: Current buffer size */
  long *bufref,           /* O: Buffer reference pointer */
  long *unit,             /* I: Unit number of tape drive */
  long *samps,            /* I: Total number of samples */
  float *deltay,          /* I: Resolution in the Y direction */
  float *deltaz,          /* I: Resolution in the Z direction */
  double *ymin,           /* I: Minimum corner in y direction */
  long *dtype,            /* I: Data type of input data */
  long *bckgnd,           /* I: Background value */
  FILE *infp              /* I: Input data file pointer */
);

/* bdemb.c */
void bdemb
(
  unsigned char *patch,   /* I: Temp buffer */
  long *lin,              /* I: Current line */
  short *buff2,           /* I: Input buffer */
  long *bufsiz,           /* O: Current buffer size */
  long *bufref,           /* O: Buffer reference pointer */
  long *unit,             /* I: Unit number of tape drive */
  long *samps,            /* I: Total number of samples */
  float *deltay,          /* I: Resolution in the Y direction */
  float *deltaz,          /* I: Resolution in the Z direction */
  double *ymin,           /* I: Minimum corner in Y direction */
  long *dtype,            /* I: Data type of input data */
  long *size,             /* I: Size of data type */
  long *outsys,           /* I: Output system */
  long *bckgnd,           /* I: Background value */
  FILE *infp              /* I: Input data file pointer */
);

/* bdted.c */
void bdted
(
  short *patch,        /* O: Patch buffer */
  short *buff2,        /* I: Input buffer */
  long *samps,         /* I: Number of input samples */
  long *unit,          /* I: Tape unit number */
  long *bufsz,         /* O: Current buffer size */
  long *bufref,        /* O: Reference buffer pointer */
  long *dtype,         /* I: Data type of input data */
  long *outsys,        /* I: Output system */
  long *bakgnd,        /* I: Background value */
  FILE *infp,          /* I: Input data file pointer */
  long curpatch        /* I: Current patch being processed */
);

/* bdtt.c */
void bdtt
(
  unsigned char *patch,    /* O: Patch buffer */
  short *buff2,            /* I: Input buffer */
  long *bufsiz,            /* O: Current buffer size */
  long *bufref,            /* O: Reference buffer pointer */
  long *unit,              /* I: Tape unit number */
  long *samps,             /* I: Number of samples */
  float *deltay,           /* I: Resolution in the Y direction */
  double *ymin,            /* I: Minimum Y value */
  long *outsys,            /* I: Output system */
  FILE *infp               /* I: Input data file pointer */
);

/* brecs.c */
void brecs
(
  short *buff2,         /* I: Input buffer */
  long *unit,           /* I: Tape unit number */
  short *form,          /* I: Form of the DEM data */
  long *lines,          /* I: Total number of input lines */
  long *samps,          /* I: Total number of input samples */
  long *bufsiz,         /* I/O: Current buffer size */
  long *bufref,         /* O: Buffer reference pointer */
  char *hosout,         /* O: Output host file name */
  float *deltay,        /* I: Resolution in the Y direction */
  float *deltaz,        /* I: Resolution in the Z direction */
  double *ymin,         /* I: Minimum corner in Y direction */
  long *msgtim,         /* I: Time between display messages */
  long *bckgnd,         /* I: Background value */
  long *outsys,         /* I: Output system */
  char *mmflag,         /* I: Min/Max flag */
  FILE *infp            /* I: Input data file pointer */
);

/* cdema.c */
void cdema
(
  long *unit,                 /* I: Tape unit number */
  char *record,               /* I: ASCII record buffer */
  long *bufref,               /* I/O: Reference buffer pointer */
  long *bufsiz,               /* I: Current buffer size */
  struct VLL_DEM *vll_dem,    /* I: DEM header structure */
  FILE *infp                  /* I: Input data file pointer */
);

/* cdemb.c */
void cdemb
(
  long *unit,                 /* I: Tape unit number */
  short *buff2,               /* I: Input buffer */
  long *bufref,               /* O: Buffer reference pointer */
  struct VLL_DEM *vll_dem,    /* I: Struct containing A and C record info */
  long *outsys,               /* I: Output system */
  FILE *infp                  /* I: Input data file pointer */
);

/* cdtt.c */
void cdtt
(
  long *unit,                 /* I: Tape unit number */
  short *buff2,               /* I: Input buffer */
  long *bufref,               /* O: Reference buffer pointer */
  struct VLL_DTT *vll_dtt,    /* I: DTT structure */
  long *outsys,               /* I: Output system */
  FILE *infp                  /* I: Input data file pointer */
);

/* ddrdem.c */
void ddrdem
(
  char *hosout,               /* I: Host output name */
  struct VLL_DEM *vll_dem,    /* I: DEM header struct */
  long *lines,                /* I: Number of lines in input image */
  long *samps,                /* I: Number of samples in input image */
  long *ddrvflg               /* I: DDR validity flag */
);

/* ddrdted.c */
void ddrdted
(
  char *hosout,                /* Host output file name */
  struct VLL_DTED *vll_dted    /* DTED header struct */
);

/* ddrdtt.c */
void ddrdtt
(
  char *hosout,                /* I: Host output file name */
  struct VLL_DTT *vll_dtt,     /* I: DTT header struct */
  long *ddrvflg                /* I: DDR validity flag */
);

/* dementer.c */
void main
(
  void
);

/* dspdem.c */
void dspdem
(
  FILE *fdesc,                 /* I: File descriptor for output file */
  struct VLL_DEM *vll_dem,     /* I: DEM header struct */
  char *taeout,                /* I: TAE output image name */
  long *lines,                 /* I: Number of input lines */
  long *samps                  /* I: Number of input samples */
);

/* dspdted.c */
void dspdted
(
  FILE *fdesc,                 /* I: File descriptor for output file */
  struct VLL_DTED *vll_dted,   /* I: DTED header struct */
  char *taeout,                /* I: TAE image file name */
  long *lines,                 /* I: Number of input lines */
  long *samps                  /* I: Number of input samples */
);

/* dspdtt.c */
void dspdtt
(
  FILE *fdesc,                 /* I: File descriptor for output file */
  struct VLL_DTT *vll_dtt,     /* I: DTT header struct */
  char *taeout,                /* I: TAE image file name */
  long *lines,                 /* I: Number of input lines */
  long *samps                  /* I: Number of input samples */
);

/* getpar.c */
void getpar
(
  char taeout[][CMLEN],      /* O: Output TAE image file name */
  char *hosout,              /* O: Host image file name */
  char *mess,                /* O: Message for the operator */
  long *bckgnd,              /* O: Background value */
  short *form,               /* O: Format of the tape */
  long *fnum,                /* O: File number on the tape */
  long *msgtim,              /* O: Tie between processing messages */
  char *outlt,               /* O: Output labeled table name */
  long *term,                /* O: Flag for terminal output */
  long *file,                /* O: Flag for file output */
  long *lp,                  /* O: Flag for printer output */
  short *first,              /* O: First file needed */
  short *last,               /* O: Last file needed */
  long *unit,                /* O: Tape unit number */
  char *drive,               /* O: Drive name */
  long *outltf,              /* O: Flag if output labeled table specified */
  char *prtfile,             /* O: Print file name */
  long *outsys,              /* O: Output system */
  char *mmflag,              /* O: Min/Max flag */
  long *ddrvflg,             /* O: DDR validity flag */
  char taein[CMLEN],         /* O: TAE input data file name */
  char *hosin,               /* O: Host input data file name */
  long *diskflg              /* O: Data is on disk flag */
);

/* latlon.c */
void latlon
(
  long *lat,              /* I/O: Latitude in total seconds */
  long *lon,              /* I/O: Longitude in total seconds */
  char *latchar,          /* O: Latitude in DMS format */
  char *lonchar           /* O: Longitude in DMS format */
);

/* newbuf.c */
void newbuf
(
  char *rec,              /* O: Array for ASCII buffer */
  long *bufref,           /* O: Buffer pointer */
  long *bufsz,            /* O: Current buffer size */
  long *unit,             /* I: Tape drive unit number */
  FILE *infp              /* I: Input data file pointer */
);

/* putdem.c */
void putdem
(
  char *taeout,               /* I: LAS image name */
  char *outlt,                /* I: Labeled table name */
  struct VLL_DEM *vll_dem     /* I: DEM header struct */
);

/* putdted.c */
void putdted
(
  char *taeout,               /* I: LAS image name */
  char *outlt,                /* I: Labeled table name */
  struct VLL_DTED *vll_dted   /* I: DTED header struct */
);

/* putdtt.c */
void putdtt
(
  char *taeout,              /* I: LAS image name */
  char *outlt,               /* I: Labeled table name */
  struct VLL_DTT *vll_dtt    /* I: DTT header struct */
);

/* setin.c */
void setin
(
  char *hosout,              /* I: Host output file name */
  double *minele,            /* O: Min element value */
  double *maxele,            /* O: Max element value */
  long *lines,               /* I: Total number of input lines */
  long *samps,               /* I: Total number of input samples */
  unsigned char *temp,       /* O: Temp data buffer */
  char *mmflag,              /* Min/Max flag */
  long *msgtim,              /* Time between display messages */
  long *dtype                /* Data type of input image */
);

/* tapeset.c */
void tapeset
(
  short *form,               /* I: Form on the tape */
  long *fnum,                /* I: File number on tape */
  char *mess,                /* O: Message for the operator */
  long *unit,                /* I: Tape unit number */
  unsigned char *buff2,      /* O: Input buffer */
  short *first,              /* I: First file flag */
  char *drive,               /* I: Drive number */
  long *bufsiz               /* O: Current buffer size */
);

#endif
