/******************************************************************************
SUBROUTINE DSPDEM
 
PURPOSE:
 	This subroutine performs the display of DEM header information
 	to the user. 
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
PROJECT:   LAS
 
ALGORITHM DESCRIPTION:
 	Write the output image name and size to the destination specified
 	Write all of the DEM sturcture members to the destination specified
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dementer.h"

void dspdem
(
  FILE *fdesc,                 /* I: File descriptor for output file */
  struct VLL_DEM *vll_dem,     /* I: DEM header struct */
  char *taeout,                /* I: TAE output image name */
  long *lines,                 /* I: Number of input lines */
  long *samps                  /* I: Number of input samples */
)
{  /* dspdem  */

    long i;                     /*  counter -- current input char*/
    long j;                     /*  counter -- currnet output char*/

    unsigned char blank = FALSE;/*  logical if blank was found  */
    char temp1[80];             /*  temp. buffer of crec        */
    char temp2[80];             /*  temp. buffer of crec        */

    fprintf (fdesc," Header information for image %s\n",taeout);
    fprintf (fdesc," Image is %d lines by %d samples\n",*samps,*lines);
    fprintf (fdesc,"\n TYPE A RECORD DATA \n\n");

    /*  Break the character portion of the A type record into 2 lines.  Break
        at the first space after the 64th character
        ---------------------------------------------------------------------*/
    j = 0;
    for (i = 0; i < 64; i++)
        temp1[i] = vll_dem->crec[i];

    for (;i < 144; i ++)
    {
        if ((vll_dem->crec[i] == ' ') && (!blank))
        {
            blank = TRUE;
            temp1[i] = '\0';
        }
        else if (!blank)
            temp1[i] = vll_dem->crec[i];
        else
        {
            temp2[j] = vll_dem->crec[i];
            j++;
        }
    }
    temp2[j] = '\0';
    fprintf (fdesc," %s\n",temp1);
    fprintf (fdesc," %s\n",temp2);

    /*  Print the DEM header data to the output file
        -------------------------------------------------*/
    fprintf (fdesc," DEM Level                                   %-1d\n",
            vll_dem->lvl);
    if (vll_dem->epat == 1)
        fprintf (fdesc," Elevation pattern is:                 regular\n");
    else if (vll_dem->epat == 2)
        fprintf (fdesc," Elevation pattern is:                 random\n");

    if (vll_dem->gref == 0) 
        fprintf(fdesc," Ground reference is:               Geographic\n");
    else if (vll_dem->gref == 1) 
        fprintf(fdesc," Ground reference is:                      UTM\n");
    else if (vll_dem->gref == 2) 
        fprintf(fdesc," Ground reference is:              State Plane\n");

    fprintf (fdesc," Zone reference =                           %-3d\n",
            vll_dem->zref);

    if (vll_dem->puni == 0)
        fprintf (fdesc," Ground coordinates are in:            radians\n");
    else if (vll_dem->puni == 1)
        fprintf (fdesc," Ground coordinates are in:               feet\n");
    else if (vll_dem->puni == 2)
        fprintf (fdesc," Ground coordinates are in:             meters\n");
    else if (vll_dem->puni == 3)
        fprintf (fdesc," Ground coordinates are in:        arc-seconds\n");

    if (vll_dem->euni == 1)
        fprintf (fdesc," Elevation units are in:                  feet\n");
    else if (vll_dem->euni == 2)
        fprintf (fdesc," Elevation units are in:                meters\n");

    fprintf (fdesc," Number of sides equals                     %-5d\n",
            vll_dem->side);

    if (vll_dem->cdata == 0)
        fprintf (fdesc," Accuracy data is:                     unknown\n");
    else if (vll_dem->cdata == 1)
        fprintf (fdesc," Accuracy data is:                    C record\n");

    fprintf (fdesc," Patch rows equals                          %-5d\n",
            vll_dem->prow);
    fprintf (fdesc," Patch columns equals                       %-5d\n", 
            vll_dem->pcol);

    fprintf (fdesc," Spatial resolution (X) equals       %-1.7E\n",
             vll_dem->xres);
    fprintf (fdesc," Spatial resolution (Y) equals       %-1.7E\n",
             vll_dem->yres);
    fprintf (fdesc," Spatial resolution (Z) equals       %-1.7E\n",
             vll_dem->zres);
    fprintf (fdesc," Minimum elevation equals    %-1.15lE\n",vll_dem->mine);
    fprintf (fdesc," Maximum elevation equals    %-1.15lE\n",vll_dem->maxe);
    fprintf (fdesc," CCW Angle (Radians) equals  %-1.15lE\n",vll_dem->ccw);

    fprintf (fdesc," Projection Parameters:\n");
    for (i = 0; i < 15; i += 3)
    {
        fprintf (fdesc,"     %-1.15lE   ",vll_dem->ppar[i]);
        fprintf (fdesc,"%-1.15lE   ",vll_dem->ppar[i + 1]);
        fprintf (fdesc,"%-1.15lE\n",vll_dem->ppar[i + 2]);
    }  /*  for i < 15  */

    fprintf (fdesc," Corner Coordinates\n");
    if (vll_dem->gref == 1) 
        fprintf(fdesc,
                "                    Northings                     Eastings\n");
    fprintf(fdesc," NW(UL)      %-1.15lE          %-1.15lE\n",
            vll_dem->corner[1][1], vll_dem->corner[1][0]);
    fprintf(fdesc," NE(UR)      %-1.15lE          %-1.15lE\n",
            vll_dem->corner[2][1], vll_dem->corner[2][0]);
    fprintf(fdesc," SW(LL)      %-1.15lE          %-1.15lE\n",
            vll_dem->corner[0][1], vll_dem->corner[0][0]);
    fprintf(fdesc," SE(LR)      %-1.15lE          %-1.15lE\n",
            vll_dem->corner[3][1], vll_dem->corner[3][0]);

    if(vll_dem->cdata == 1)
    {
        fprintf (fdesc,"\n TYPE C RECORD DATA\n");

        if (vll_dem->sw2 == 1)
        {
            fprintf (fdesc," Abs. RMSE statistics are:\n");
            fprintf (fdesc, "                                 %-5d\n",
                     vll_dem->armsex);
            fprintf (fdesc,"                                 %-5d\n",
                     vll_dem->armsey);
            fprintf (fdesc,"                                 %-5d\n",
                     vll_dem->armsez);
            fprintf (fdesc," Sample size used is             %-5d\n",
                     vll_dem->size2);
        }
        else
            fprintf (fdesc," Abs. RMSE statistics are unavailable\n");

        if (vll_dem->sw5 == 1)
        {
            fprintf (fdesc," Real RMSE statistics are:\n");
            fprintf (fdesc,"                                 %-5d\n",
                     vll_dem->rrmsex);
            fprintf (fdesc,"                                 %-5d\n",
                     vll_dem->rrmsey);
            fprintf (fdesc,"                                 %-5d\n",
                     vll_dem->rrmsez);
            fprintf (fdesc," Sample size used is             %-5d\n",
                     vll_dem->size5);
        }
        else
            fprintf (fdesc," Real RMSE statistics are unavailable\n");
    }  /*  if vll_dem->cdata != 1  */
}  /*  dspdem  */
