/******************************************************************************
SUBROUTINE DSPDTED

PURPOSE:
 	This subroutine performs the display of DTED information
 	to the user. 

PROJECT:   LAS

PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
ALGORITHM DESCRIPTION:
 
Output the output image name and size to the destination specified
Output all of the DTED header data records to the destination specified

return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dementer.h"

void dspdted
(
  FILE *fdesc,                 /* I: File descriptor for output file */
  struct VLL_DTED *vll_dted,   /* I: DTED header struct */
  char *taeout,                /* I: TAE image file name */
  long *lines,                 /* I: Number of input lines */
  long *samps                  /* I: Number of input samples */
)
{  /*  dspdted  */

    char cod[32];              /*  Access code buffer               */

    /*  Output the DTED header information to the output file
        ---------------------------------------------------------*/
    fprintf(fdesc," Header information for image %s\n",taeout);
    fprintf(fdesc," Image is %d lines by %d samples\n",*samps,*lines);
    fprintf(fdesc,"\n\n");

    fprintf(fdesc," File name                %s\n",vll_dted->fname);
    fprintf(fdesc," Source hardware          %s\n",vll_dted->machin);
    fprintf(fdesc," Reel number              %-5d\n",vll_dted->reel);
    fprintf(fdesc," File number              %-5d\n",vll_dted->file);
    fprintf(fdesc," Julian creation date     %s\n",vll_dted->cdate);
    fprintf(fdesc," Julian expiration date   %s\n",vll_dted->edate);
    fprintf(fdesc," Reel access              %-5d\n",vll_dted->access);

    fprintf(fdesc," Datum                    %s\n",vll_dted->datum);
    fprintf(fdesc," Latitude,  SW corner     %8s\n",&vll_dted->swlat[0]);
    fprintf(fdesc," Longitude, SW corner     %8s\n",&vll_dted->swlong[0]);

    fprintf(fdesc," Latitude  interval is    %-5d seconds\n", 
            vll_dted->lats / 10);
    fprintf(fdesc," Longitude  interval is   %-5d seconds\n", 
            vll_dted->longs / 10);
    fprintf(fdesc," Vertical accuracy is     %-5d\n", vll_dted->acode);

    if (strncmp(vll_dted->scode,"T",1) == 0) 
        strcpy (cod,"top secret");
    if (strncmp(vll_dted->scode,"S",1) == 0) 
        strcpy(cod,"secret");
    if (strncmp(vll_dted->scode,"C",1) == 0) 
        strcpy(cod,"classified");
    if (strncmp(vll_dted->scode,"U",1) == 0) 
        strcpy(cod,"unclassified");
    if (strncmp(vll_dted->scode,"R",1) == 0) 
        strcpy(cod,"restricted");

    fprintf(fdesc," Security code is         %s\n",cod);
    fprintf(fdesc," Points per patch are     %-5d\n",vll_dted->points);
    fprintf(fdesc," Longitudinal patches are %-5d\n",vll_dted->patchs);
}  /* dspdted  */
