/******************************************************************************
SUBROUTINE DSPDTT
 
PURPOSE:
 	This subroutine performs the display of DTT header information
 	to the user. 

PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
ALGORITHM DESCRIPTION:
 
Output the output image name and size to the destination specified
Output all of the DTT header records to the destination specified

 return
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dementer.h"

void dspdtt
(
  FILE *fdesc,                 /* I: File descriptor for output file */
  struct VLL_DTT *vll_dtt,     /* I: DTT header struct */
  char *taeout,                /* I: TAE image file name */
  long *lines,                 /* I: Number of input lines */
  long *samps                  /* I: Number of input samples */
)
{  /*  dspdtt  */

    char latchar[12];       /*  char buffer containing latitude specs. */
    char lonchar[12];       /*  char buffer containing long. specs. */

    /*  Output DTT information to output file
        -----------------------------------------*/
    fprintf (fdesc," Header information for image %s\n", taeout);
    fprintf (fdesc," Image is %d lines by %d samples\n",*lines,*samps);
    fprintf (fdesc,"\n\n A RECORD DATA IS\n");

    fprintf (fdesc," Sheet number %s\n", vll_dtt->sht);
    fprintf (fdesc," Map series %s\n", vll_dtt->mps);
    fprintf (fdesc," Map edition %s\n", vll_dtt->mpe);
    fprintf (fdesc," Map projection code %d\n", vll_dtt->mpc);
    fprintf (fdesc," Map projection zone %d\n", vll_dtt->mpz);

    if (vll_dtt->evc == 0)
        fprintf (fdesc," Elevation units are in meters\n");
    else if (vll_dtt->evc == 1)
        fprintf (fdesc," Elevation units are in feet\n");

    fprintf (fdesc," X coordinate, SW corner %d\n",vll_dtt->xsw);
    fprintf (fdesc," Y coordinate, SW corner %d\n",vll_dtt->ysw);
    fprintf (fdesc," X coordinate, NW corner %d\n",vll_dtt->xnw);
    fprintf (fdesc," Y coordinate, NW corner %d\n",vll_dtt->ynw);
    fprintf (fdesc," X coordinate, NE corner %d\n",vll_dtt->xne);
    fprintf (fdesc," Y coordinate, NE corner %d\n",vll_dtt->yne);
    fprintf (fdesc," X coordinate, SE corner %d\n",vll_dtt->xse);
    fprintf (fdesc," Y coordinate, SE corner %d\n",vll_dtt->yse);

    latlon(&vll_dtt->lanw,&vll_dtt->lonw,latchar,lonchar);
    fprintf (fdesc," Longitude of NW corner %s\n", lonchar);
    fprintf (fdesc," Latitude of NW corner  %s\n", latchar);
    latlon(&vll_dtt->lasw,&vll_dtt->losw,latchar,lonchar);
    fprintf (fdesc," Longitude of SW corner %s\n", lonchar);
    fprintf (fdesc," Latitude of SW corner  %s\n", latchar);
    latlon(&vll_dtt->lane,&vll_dtt->lone,latchar,lonchar);
    fprintf (fdesc," Longitude of NE corner %s\n", lonchar);
    fprintf (fdesc," Latitude of NE corner  %s\n", latchar);
    latlon(&vll_dtt->lase,&vll_dtt->lose,latchar,lonchar);
    fprintf (fdesc," Longitude of SE corner %s\n", lonchar);
    fprintf (fdesc," Latitude of SE corner  %s\n", latchar);

    fprintf (fdesc,"\n\n C RECORD DATA");
    fprintf (fdesc," Bit string %d\n", vll_dtt->str1);
    fprintf (fdesc," Bit string %d\n", vll_dtt->str2);
    fprintf (fdesc," Bit string %d\n", vll_dtt->str3);
    fprintf (fdesc," Bit string %d\n", vll_dtt->str4);
    fprintf (fdesc," The number one %d\n", vll_dtt->fill);
    fprintf (fdesc," Number of B records is %d\n", vll_dtt->brec);
}  /* dspdtt  */
