/******************************************************************************
SUBROUTINE GETPAR
 
PURPOSE:
 	Subroutine GETPAR recieves the input parameters
      from the screen.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 5.1 07/11/90  D. Etrheim	CSB	      Trap status of c_setprt
 
ALGORITHM DESCRIPTION:
Retrieve an output file name from TM
Set flag if label table is desired
Retrieve the other parameters from the TAE parameter block
Test for non-ascending or duplicate file numbers
Initialize FORM variable according to FTYPE parameter value
Call SETPRT to set integer and file name values from PRINT parameter
Retrieve the output host system

return
******************************************************************************/
#include "las.h"
#include "vll.h"
#include "util.h"
#include "dementer.h"

void getpar
(
  char taeout[][CMLEN],      /* O: Output TAE image file name */
  char *hosout,              /* O: Host image file name */
  char *mess,                /* O: Message for the operator */
  long *bckgnd,              /* O: Background value */
  short *form,               /* O: Format of the tape */
  long *fnum,                /* O: File number on the tape */
  long *msgtim,              /* O: Tie between processing messages */
  char *outlt,               /* O: Output labeled table name */
  long *term,                /* O: Flag for terminal output */
  long *file,                /* O: Flag for file output */
  long *lp,                  /* O: Flag for printer output */
  short *first,              /* O: First file needed */
  short *last,               /* O: Last file needed */
  long *unit,                /* O: Tape unit number */
  char *drive,               /* O: Drive name */
  long *outltf,              /* O: Flag if output labeled table specified */
  char *prtfile,             /* O: Print file name */
  long *outsys,              /* O: Output system */
  char *mmflag,              /* O: Min/Max flag */
  long *ddrvflg,             /* O: DDR validity flag */
  char taein[CMLEN],         /* O: TAE input data file name */
  char *hosin,               /* O: Host input data file name */
  long *diskflg              /* O: Data is on disk flag */
)
{  /*  getpar  */

    static long fatal = -1;      /*  fatal error code                    */
    static long nonfatal = 1;    /*  nonfatal error code                 */
    static long zero = 0;        /*  value of 0                          */

    extern struct PARBLK vblock; /*  TAE parameter block                 */
    struct VARIABLE *var;	     /*  pointer to a specific variable      */

    long acc = IWRITE;           /* File access	                         */
    long sflag = SETN;           /* Flag for SYSSET routine	             */	
    long status;                 /* status of function calls             */

    char syst[DDSYLN];           /* system character string              */
    char strfir[4];              /* buffer for first file                */
    char strlas[4];              /* buffer for last file                 */
    char dtype[5];               /* file type on tape                    */
    char cddrv[3];               /* DDR validity flag string             */

    /*  retrieve TAE parameters
        ---------------------------*/
    var = p_find(&vblock,"FTYPE");
    strcpy(dtype,SVAL(*var,0));
    if (strncmp(dtype,"DTT",3) == 0)
        *form = DTT;
    else if (strncmp(dtype,"DEM",3) == 0)
        *form = DEM;
    else if (strncmp(dtype,"DTED",3) == 0)
        *form = DTED;

    var = p_find(&vblock,"COMMENT");
    if(var->v_count == 1)
    {
        strcpy(mess,SVAL(*var,0));
        *diskflg = 0;
        strcpy(hosin,"");
    }
    else
    {
        var = p_find(&vblock,"INFILE");
        if(var->v_count == 1)
        {
            strcpy(taein,SVAL(*var,0));
            *diskflg = 1;
            if(E_FAIL == c_cmhost(taein,hosin))
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        else
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }


    status = c_getout(&vblock,"OUT",(char (*)[256])taeout[0],
            (char (*)[256])hosout);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);

    status = c_getnam(&vblock,"OUTLT",&zero,outlt,"lt",&acc);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);

    if (strncmp(outlt,"\0",1) == 0)
        *outltf = FALSE;
    else 
        *outltf = TRUE;


    var = p_find(&vblock,"FILENUM");
    *fnum = IVAL(*var,0);

    var = p_find(&vblock,"BACKGRND");
    *bckgnd = IVAL(*var,0);

    var = p_find(&vblock,"$MSGTIM");
    *msgtim = IVAL(*var,0);

    var = p_find(&vblock,"$DDRVFLG");
    strcpy(cddrv,SVAL(*var,0));

    if (strncmp(cddrv,"IN",2) == 0)
        *ddrvflg = INVAL;
    else
        *ddrvflg = UNKNOW;

    var = p_find(&vblock,"FIRST");
    strcpy(strfir,SVAL(*var,0));
    if (strncmp(strfir,"YES",3) == 0)
        *first = TRUE;

    var = p_find(&vblock,"LAST");
    strcpy(strlas,SVAL(*var,0));
    if (strncmp(strlas,"YES",3) == 0)
        *last = TRUE;

    var = p_find(&vblock,"UNIT");
    *unit = IVAL(*var,0);

    var = p_find(&vblock,"DRIVE");
    strcpy(drive,SVAL(*var,0));

    var = p_find(&vblock,"$MINMAX");
    strcpy(mmflag,SVAL(*var,0));

    strcpy (prtfile,"dementer");
    status = c_setprt(&vblock,term,file,lp,prtfile,taeout[1]);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);

    status = c_getsys(syst);
    if (status != E_SUCC)
    {
        c_errmsg("Error retreiving current system","dementer-system",&nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }

    status = c_sysset(syst,outsys,&sflag);
    if (status != E_SUCC)
    {
        c_errmsg("Error retreiving current system","dementer-system",&nonfatal);
        c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
    }
}  /*  getpar  */
