/******************************************************************************
SUBROUTINE LATLON
 
PURPOSE:
 	This subroutine converts the DTT C record latitude and longitude
 	values from total seconds to standard lat and long values.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
ALGORITM DESCRIPTION:
 
Convert the latitude value from total seconds to latitude
Convert the longitude value from total seconds to longitude
 
return
******************************************************************************/
#include <math.h>
#include "las.h"
#include "dementer.h"

void latlon
(
  long *lat,              /* I/O: Latitude in total seconds */
  long *lon,              /* I/O: Longitude in total seconds */
  char *latchar,          /* O: Latitude in DMS format */
  char *lonchar           /* O: Longitude in DMS format */
)
{  /* latlon  */

    double tlat;          /*  temporary latitude            */
    double tlon;          /*  temporary longitude           */
    double t1;		
    double t2;

    long inum;            /*  temporary buffer for integer values*/

    /*  initialize the lat and long character buffers
        -------------------------------------------------*/
    strcpy(latchar," ");
    strcpy(lonchar," ");

    tlat = (double) *lat;
    if (tlat < 0.0)
    {
        strcpy(&latchar[13],"S");
        tlat = fabs(tlat);
    }
    else
        strcpy(&latchar[13],"N");

    t1 = tlat/3600.0;
    inum = (long) t1;
    t2 = tlat - (inum * 3600.);
    t1 = t2 / 60.0;
    inum = (long)t1;
    t2 = t2 - (inum * 60.);
    inum = (long) t2;

    tlon = (double) *lon;
    if (tlon < 0.0)
    {
        strcpy(&lonchar[13],"W");
        tlon = fabs(tlon);
    }
    else
        strcpy(&lonchar[13],"E");

    t1 = tlon / 3600.0;
    inum = (long) t1;
    t2 = tlon - (inum * 3600.);
    t1 = t2 / 60.;
    inum = (long) t1;
    t2 = t2 - (inum * 60.);
    inum = (long) t2;
}  /*  latlon  */
