/*************************************************************************

NAME:  NEWBUF

FUNCTION:
        NEWBUF reads the next physical record on the tape.  This routine
        is referenced by the program DEMENTER.

PROGRAM HISTORY:
  Version	Date	  Author		   Request
  -------       ----      ------    	 	   -------
     1        June 1987   K. Gacke/J. Stansbery    update to standards
     5.0      May  1989   B. Ailts		   LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT:  LAS

ALGORITHM/DESCRIPTION:
        Read the next physical record on tape.  Check the status for
        accuracy.  If the status is greater than or equal to 0, then
        the buffer size should be given the value of status.

**************************************************************************/
#include <stdio.h>
#include "las.h"
#include "dementer.h"

void newbuf
(
  char *rec,              /* O: Array for ASCII buffer */
  long *bufref,           /* O: Buffer pointer */
  long *bufsz,            /* O: Current buffer size */
  long *unit,             /* I: Tape drive unit number */
  FILE *infp              /* I: Input data file pointer */
)
{
    static long fatal = -1;      /*  fatal error code                    */
    static long handle = HANDLE; /* handle flag of tapeio rountines      */
    static long nonfatal = 1;    /*  nonfatal error code                 */
    static long rewopt = REWOFFL;/* rewind option                        */
    static long zero = 0;        /*  value of 0                          */

    long status;			/*  status of function calls                 */
    long size = 32768;		/*  size of tape read                        */

    /*  read the next record into the buffer
        ---------------------------------------*/

    if(infp == NULL)
    {
        status = c_tpread(unit,(unsigned char *)rec,&size,&handle);
        if(status < 0)
        {
            c_errmsg("Error encountered reading tape","dementer-tpread",
                     &nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        if(status >= 0)
            *bufsz = status;
    }
    else
    {
        status = (long)fread(rec,1,size,infp);
        if (status < 0)
        {
            fclose(infp);
            c_errmsg("Error encountered reading disk","dementer-fread",
                     &nonfatal);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        if (status >= 0)
            *bufsz = status;
    }

    /*  Reset the reference buffer pointer
        --------------------------------------*/
    *bufref = 0;
}
