/*************************************************************************

NAME:  PUTDEM

FUNCTION:
	PUTDEM defines the label vector of the labeled table with the
	DEM header information.  This routine is referenced by the
	program DEMENTER.

PROGRAM HISTORY:
  Version	Date	  Author		   Request
  -------       ----      ------    	 	   -------
     1        June 1987   K. Gacke/J. Stansbery    write header information
						   to a labeled table
     5.0      May 1989	  B. Ailts		   LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT:  LAS

ALGORITHM (FLOW):
	Defines the header information of the labeled table (associated
	subfiles, file type, file description, and the label vector).
	Then writes each of the A and C records to the labeled table.
**************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "dementer.h"

void putdem
(
  char *taeout,               /* I: LAS image name */
  char *outlt,                /* I: Labeled table name */
  struct VLL_DEM *vll_dem     /* I: DEM header struct */
)
{
    struct VECTOR *vec;       /* pointer to the linked list field of the LT */
    struct TAB_DEF tab;       /* structure defining the LT */
    char fdesc[CMLEN];        /* file description of the labeled table */
    char temp[CMLEN];         /* temporary description of the LT */

    /* Initialize the label information
       -----------------------------------*/
    tab.nsubs = 0;              /* no associated subfiles */
    tab.ftype = "DEM";          /* define file type LT */
    sprintf(fdesc,"DEM A and C data records");
    tab.fdesc = fdesc;          /* define file description LT */
    open_tab(&tab,outlt,2);     /* open the LT */

    /* define and write the label vectors 
       -------------------------------------*/
    put_vector(&tab,"LASIMAGE",C,"LAS image file name",1,100,FIELD_SEP2);
    vec = tab.vector;
    strcpy(vec->ptr.c,taeout);
    put_vector(&tab,"CHARREC",C,"Character Record",1,144,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dem->crec,144);
    put_vector(&tab,"DEMCODE",I2,"DEM level code",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->lvl);
    put_vector(&tab,"ELEVPAT",I2,"Code defining elevation pattern",
            1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->epat);
    put_vector(&tab,"GRNDREF",I2,
            "Code defining ground planimetric reference system",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->gref);
    put_vector(&tab,"ZONEREF",I2,
            "Code defining zone in ground planimetric reference system",1,1,
             FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->zref);
    put_vector(&tab,"PROJPARM",R8,"Map projection parameters",15,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)vll_dem->ppar);
    sprintf(temp,"%s %s","Code defining unit of measure for ground planimetric",
            "coordinates throughout the file");
    put_vector(&tab,"GRDCOORD",I2,temp,1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->puni);
    sprintf(temp,"%s %s","Code defining unit of measure for elevation",
            "coordinates throughout the file");
    put_vector(&tab,"ELEVUNIT",I2,temp,1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->euni);
    sprintf(temp,"%s %s","Number of sides in polygon which defines the",
            "coverage of the DEM file");
    put_vector(&tab,"NSIDES",I2,temp,1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->side);
    sprintf(temp,"%s %s","A 4x2 array containing the ground coordinates of the",
            "four corners for the DEM");
    put_vector(&tab,"CRNCOORD",R8,temp,4,2,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)vll_dem->corner);
    sprintf(temp,"%s %s","A 2-element array containing minimum and maximum",
            "elevations for the DEM");
    put_vector(&tab,"MIN/MAX",R8,temp,2,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->mine);
    sprintf(temp,"%s %s",
         "Counterclockwise angle (in radians) from primary axis of ground",
         "planimetric reference to primary axis of DEM local reference system");
    put_vector(&tab,"ANGLE",R8,temp,1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->ccw);
    put_vector(&tab,"ACCODE",I2,"Accuracy code for elevations",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->cdata);
    put_vector(&tab,"SPATRES",R4,"DEM spatial resolution (x,y,z)",
            3,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->xres);
    put_vector(&tab,"ROW/COL",I2,
            "Number of rows and columns of profiles in the DEM",2,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->prow);
    put_vector(&tab,"STATS2",I2,
            "Switch indicating availability of statistics in absolute datum",
            1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->sw2);
    put_vector(&tab,"ABSDATUM",I2,
               "RMSE of file datum relative to absolute datum",3,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->armsex);
    put_vector(&tab,"SAMPSIZ2",I2,
            "Sample size on which statistics in absolute datum are based",1,1,
            FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->size2);
    put_vector(&tab,"STATS5",I2,
            "Switch indicating availability of statistics in file datum",1,1,
            FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->sw5);
    put_vector(&tab,"DEMDATUM",I2,"RMSE of DEM data relative to file datum",
            3,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->rrmsex);
    put_vector(&tab,"SAMPSIZ5",I2,
            "DEM sample size on which statistics in file datum are based",1,1,
            RECORD_SEP);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dem->size5);

    put_record(&tab);
    close_tab(&tab);		/* close the LT */
}
