/*************************************************************************

NAME:  PUTDTED

FUNCTION:
	PUTDTED defines the label vector of the labeled table with the
	DTED header information.  This routine is referenced by the
	program DEMENTER.

PROGRAM HISTORY:
  Version	Date	  Author		   Request
  -------       ----      ------    	 	   -------
     1        June 1987   K. Gacke/J. Stansbery    write header information
						   to a labeled table
     2        May 1989    B. Ailts		   LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT:  LAS

ALGORITHM (FLOW):
	Define the header information of the labeled table (associated
	subfiles, file type, file description, and the label vector).
	Then writes each of the A and C records to the
	labeled table.

**************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "dementer.h"

void putdted
(
  char *taeout,               /* I: LAS image name */
  char *outlt,                /* I: Labeled table name */
  struct VLL_DTED *vll_dted   /* I: DTED header struct */
)
{
    struct TAB_DEF tab;	      /* structure defining the LT */
    struct VECTOR *vec;
    char fdesc[CMLEN];        /* file description of the labeled table */

    tab.nsubs = 0;            /* no associated subfiles */
    tab.ftype = "DTED";       /* define file type LT */
    sprintf(fdesc,"DTED A data records");
    tab.fdesc = fdesc;        /* define file description LT */
    open_tab(&tab,outlt,2);   /* open the LT */

    /* define and write the label vector 
       ------------------------------------*/
    put_vector(&tab,"LIMAGE",C,"Cataloged LAS image file",1,100,FIELD_SEP2);
    vec = tab.vector;
    strcpy(vec->ptr.c,taeout);

    put_vector(&tab,"DATUM",C,"Datum Name",1,17,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dted->datum,10);

    put_vector(&tab,"FNAME",C,"File name",1,17,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dted->fname,144);

    put_vector(&tab,"GENMACH",C,"Generating machine",1,6,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dted->machin,144);

    put_vector(&tab,"REEL",I4,"Reel number",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dted->reel);

    put_vector(&tab,"FILE",I4,"File number",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dted->file);

    put_vector(&tab,"CDATE",C,"Creation date",1,6,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dted->cdate,6);

    put_vector(&tab,"EDATE",C,"Expiration date",1,6,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dted->edate,6);

    put_vector(&tab,"ACCESS",I1,"Accessibility",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dted->access);

    put_vector(&tab,"SWLONG",C,"Southwest longitude",1,8,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dted->swlong,8);

    put_vector(&tab,"SWLAT",C,"Southwest latitude",1,8,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dted->swlat,8);

    put_vector(&tab,"LONGS",I4,"Tenths of seconds longitude",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dted->longs);

    put_vector(&tab,"LATS",I4,"Tenths of seconds latitude",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dted->lats);

    put_vector(&tab,"ACCODE",I4,"Accuracy code",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dted->acode);

    put_vector(&tab,"SECCODE",C,"Security code",1,3,FIELD_SEP2);
    vec = vec->next;
    strncpy(vec->ptr.c,vll_dted->scode,3);

    put_vector(&tab,"PATCHES",I4,"Number of patches",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dted->patchs);

    put_vector(&tab,"POINTS",I4,"Number of points per patch",1,1,RECORD_SEP);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dted->points);

    put_record(&tab);		/*  write the record	*/
    close_tab(&tab);		/* close the LT */
}
