/*************************************************************************

NAME:  PUTDTT

FUNCTION:
	PUTDTT defines the label vector of the labeled table with the DTT
	header information.  This routine is referenced by the program
	DEMENTER.

PROGRAM HISTORY:
  Version	Date	  Author		   Request
  -------       ----      ------    	 	   -------
     1        June 1987   K. Gacke/J. Stansbery    write header information
						   to a labeled table
     5.0      May  1989   B. Ailts		   LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT:  LAS

ALGORITHM (FLOW):
	Define the header information of the labeled table (associated
	subfiles, file type, file description, and the label vecotr).
        Then write each of the records to the file

**************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "dementer.h"

void putdtt
(
  char *taeout,              /* I: LAS image name */
  char *outlt,               /* I: Labeled table name */
  struct VLL_DTT *vll_dtt    /* I: DTT header struct */
)
{
    struct TAB_DEF tab;     /* structure defining the LT */
    struct VECTOR *vec;     /* pointer to the linked list field of the LT */
    char fdesc[CMLEN];      /* file description of the labeled table */

    /*  Initialize the header information of the labeled table
        ----------------------------------------------------------*/
    tab.nsubs = 0;			/* no associated subfiles */
    tab.ftype = "DTT";		/* define file type LT */
    sprintf(fdesc,"DTT A and C data records");
    tab.fdesc = fdesc;		/* define file description LT */
    open_tab(&tab,outlt,2);	/* open the LT */

    /* define and write each of the label vectors 
       ---------------------------------------------*/
    put_vector(&tab,"LASIMAGE",C,"LAS image file name",1,100,FIELD_SEP2);
    vec = tab.vector;
    strcpy(vec->ptr.c,taeout);

    put_vector(&tab,"SHEETNO",C,"Sheet number",1,12,FIELD_SEP2);
    vec = vec->next;
    strcpy(vec->ptr.c,vll_dtt->sht);

    put_vector(&tab,"MAPSER",C,"Map series",1,6,FIELD_SEP2);
    vec = vec->next;
    strcpy(vec->ptr.c,vll_dtt->mps);

    put_vector(&tab,"MAPEDIT",C,"Map edition",1,6,FIELD_SEP2);
    vec = vec->next;
    strcpy(vec->ptr.c,vll_dtt->mpe);

    put_vector(&tab,"PROJCODE",I2,"Map projection code number",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->mpc);

    put_vector(&tab,"ZONECODE",I2,"Map projection  zone number",
            1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->mpz);

    put_vector(&tab,"ELEVCODE",I2,
            "Elevation data code number (0 = meters, 1 = feet)",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->evc);

    put_vector(&tab,"YCOORDNW",I2,"Y coordinate of NW map corner",1,1,
               FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->ynw);

    put_vector(&tab,"XCOORDNW",I2,"X coordinate of NW map corner",1,1,
               FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->xnw);

    put_vector(&tab,"YCOORDSW",I2,"Y coordinate of SW map corner",1,1,
               FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->ysw);

    put_vector(&tab,"XCOORDSW",I2,"X coordinate of SW map corner",1,1,
               FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->xsw);

    put_vector(&tab,"YCOORDNE",I2,"Y coordinate of NE map corner",1,1,
               FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->yne);

    put_vector(&tab,"XCOORDNE",I2,"X coordinate of NE map corner",1,1,
               FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->xne);

    put_vector(&tab,"YCOORDSE",I2,"Y coordinate of SE map corner",1,1,
               FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->xse);

    put_vector(&tab,"XCOORDSE",I2,"X coordinate of SE map corner",1,1,
               FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->xse);

    put_vector(&tab,"LATNW",I4,"Latitude of NW map corner",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->lanw);

    put_vector(&tab,"LONGNW",I4,"Longitude of NW map corner",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->lonw);

    put_vector(&tab,"LATSW",I4,"Latitude of SW map corner",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->lasw);

    put_vector(&tab,"LONGSW",I4,"Longitude of SW map corner",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->losw);

    put_vector(&tab,"LATNE",I4,"Latitude of NE map corner",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->lane);

    put_vector(&tab,"LONGNE",I4,"Longitude of NE map corner",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->lone);

    put_vector(&tab,"LATSE",I4,"Latitude of SE map corner",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->lase);

    put_vector(&tab,"LONGSE",I4,"Longitude of SE map corner",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->lose);


    put_vector(&tab,"STR1",I2,"String 1",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->str1);

    put_vector(&tab,"STR2",I2,"String 2",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->str2);

    put_vector(&tab,"STR3",I2,"String 3",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->str3);

    put_vector(&tab,"STR4",I2,"String 4",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->str4);

    put_vector(&tab,"FILL",I2,"Filled with the value 1",1,1,FIELD_SEP2);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->fill);

    put_vector(&tab,"NPROFILE",I2,"Number of profiles",1,1,RECORD_SEP);
    vec = vec->next;
    put_matrix(vec,vec->dtype,&vec->lt_size,(unsigned char *)&vll_dtt->brec);

    put_record(&tab);
    close_tab(&tab);		/* close the LT */
}
