/******************************************************************************
SUBROUTINE SETIN
 
PURPOSE:
 	This subroutine write the input image into a LAS 
        image file.
 
PROJECT:   LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 
 6.1 05/24/94  D. Eichacker	EDC		Added opt[]

 
ALGORITHM/DESCRIPTION:
Open the output image
Call STARTM to enable the processing message utility
For each image line
    For each sample in the image
       Assign a value to the image pixel from the array TEMP
    End for
    Call ESTEP to write an image line
End for
Call STOPTM to disable the processing message utility
 
Close the image file
Save the min/max values to the DDR
******************************************************************************/
#include "las.h"
#include "util.h"
#include "dementer.h"

void setin
(
  char *hosout,              /* I: Host output file name */
  double *minele,            /* O: Min element value */
  double *maxele,            /* O: Max element value */
  long *lines,               /* I: Total number of input lines */
  long *samps,               /* I: Total number of input samples */
  unsigned char *temp,       /* O: Temp data buffer */
  char *mmflag,              /* Min/Max flag */
  long *msgtim,              /* Time between display messages */
  long *dtype                /* Data type of input image */
)
{  /*  setin  */

    extern struct PARBLK vblock;/*  tae parameter block			*/

    float *fbuf;                /*  float buffer pointer		*/
    float *ftemp;               /*  float temporary pointer		*/

    extern long fdesc[2];       /* file descriptor			*/
    extern long gdesc;          /* group descriptor			*/
    extern long offset[2];      /* offsets into buffer array		*/

    long acc = IWRITE;          /*  file access				*/
    long bands = 1;             /*  number of output bands		*/
    long bufsiz;                /*  buffer size				*/
    extern long curent;         /*  curent processing line		*/
    long i;                     /*  counter -- currnet sample		*/
    long j;                     /*  counter -- current line		*/
    long k;                     /*  pixel pointer			*/
    static long opt[3]={0,0,0};	/*  optional parameter array for EOPENS */
    static long one = 1;        /*  value of 1                          */
    static long zero = 0;       /*  value of 0                          */
    long sl = 1;                /*  starting line			*/
    long ss = 1;                /*  starting sample			*/
    long status;                /*  status of function calls		*/

    short *buf;                 /*  word buffer pointer			*/
    short *wtemp;               /*  word temporary pointer		*/

    /*  Set up the output file
        --------------------------*/
    status = c_eopens((struct FDESC **)&fdesc[0],hosout,&bands,&sl,&ss,samps,
            lines,&offset[0],&acc,dtype,&zero,opt);
    if (status != E_SUCC)
    {
        status = c_egclse((struct GDESC **)&gdesc);
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);
    }

    fdesc[1] = 0;
    if (*dtype == EWORD)
        buf = (short *) c_ealloc((struct FDESC **)fdesc,&bufsiz);
    else
        fbuf = (float *) c_ealloc((struct FDESC **)fdesc,&bufsiz);
    if (status != E_SUCC)
    {
        status = c_egclse((struct GDESC **)&gdesc);
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);
    }

    status = c_egroup((struct FDESC **)fdesc,(struct GDESC **)&gdesc,
            (unsigned char *)buf,&bufsiz);
    if (status != E_SUCC)
    {
        status = c_egclse((struct GDESC **)&gdesc);
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);
    }
    status = c_estep((struct GDESC **)&gdesc);
    if (status != E_SUCC)
    {
        status = c_egclse((struct GDESC **)&gdesc);
        c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);
    }

    /*  Copy the TEMP buffer into the LAS I/O buffer - these steps are
        taken to avoid the 90 degree CW rotation that copying the data
        directly into LAS would incur.
        -------------------------------------------------------------------*/

    /*  Create the LAS image
        ------------------------*/
    acc = GEN2;
    c_inittm("DEMENTER","Created","line"," "," ",samps,&one,msgtim,&acc);
    curent = 1;
    c_startm();
    if (*dtype == EWORD)
    {
        wtemp = (short *)temp;
        for (j = 0,curent = 1; j < *samps; j++, curent++)
        {
            for (i = 0,k = *lines - 1;i < *lines; i++,k--)
                *(buf + *offset + k) = *(wtemp + (k * *samps) + j);
            status = c_estep((struct GDESC **)&gdesc);
            if (status != E_SUCC)
            {
                status = c_egclse((struct GDESC **)&gdesc);
                c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);
            }
        }
    }
    else
    {
        ftemp = (float *)temp;
        for (j = 0,curent = 1; j < *samps; j++, curent++)
        {
            for (i = 0,k = *lines - 1;i < *lines; i++,k--)
                *(fbuf + *offset + k) = *(ftemp + (k * *samps) + j);
            status = c_estep((struct GDESC **)&gdesc);
            if (status != E_SUCC)
            {
                status = c_egclse((struct GDESC **)&gdesc);
                c_clean(&vblock,"dementer",(char (*)[256])hosout,&one);
            }
        }
    }

    c_stoptm();
    status = c_egclse((struct GDESC **)&gdesc);

    /*  Update the min/max values
        -----------------------------*/
    c_svmnmx(hosout,&one,minele,maxele,mmflag);
}  /*  setin  */
