/******************************************************************************

SUBROUTINE TAPESET
 
PURPOSE:
 	Subroutine TAPESET positions the tape at the beginning of the
 	file to be processed.
 
PROJECT:  LAS
 
PROGRAM HISTORY:
      
VERSION   DATE   AUTHOR        CODE/CONTR.   CHANGE REQUEST
   1   09/07/84  J. LaVergne      CSB
   9   09/24/87  D. Akkerman      CSB           1)
                                                   Modified to allow for
                                                   multiple file reads
                                                   from a single tape.
 
                                                 2)
                                                   Changed ASCII buffer size
                                                   from 30744 ((1024*30)+24)
                                                   to 32760 ((1024*31)+24+ 
                                                   992).  The factor of 31
                                                   was required for an EDC
                                                   DEM tape.  The added 992
                                                   bytes are included to  
                                                   allow the program to read
                                                   any DEM ASCII format  
                                                   under a factor of 32.
 
                                                 3)
                                                   Corrected problem with
                                                   PUTVLT, which would 
                                                   output the label table
                                                   to a disk file even if
                                                   no filename was specified.

 5.0 05/31/89  B. Ailts		CSB	      LAS 5.0 conversion
 5.1 01/08/93  D. Etrheim       CSB           Correct problem in where
                                              device was returning TAPE_BOT
                                              after c_tpbsr.

 
ALGORITHM DESCRIPTION:
Set COUNT to 1
If FIRST file to be processed then
   Call TPOPEN to open the tape device
   Call XQINTG to send unit number to process
   Call XQSTR to send drive name to process
   Call XQOUT to complete process
Else
   Call TPREOP to reopen the tape device
Endif
Calculate the number of records to skip over (SKIP)
Call TPFSF to skip forward the correct number of files
 
return
******************************************************************************/
#include "las.h"
#include "dementer.h"

void tapeset
(
  short *form,               /* I: Form on the tape */
  long *fnum,                /* I: File number on tape */
  char *mess,                /* O: Message for the operator */
  long *unit,                /* I: Tape unit number */
  unsigned char *buff2,      /* O: Input buffer */
  short *first,              /* I: First file flag */
  char *drive,               /* I: Drive number */
  long *bufsiz               /* O: Current buffer size */
)
{  /*  tapeset  */

    static long fatal = -1;         /*  fatal error code                    */
    static long handle = HANDLE;    /*  handle flag of tapeio rountines     */
    static long nonfatal = 1;       /*  nonfatal error code                 */
    static long rewopt = REWOFFL;   /*  rewind option                       */
    static long zero = 0;           /*  value of 0                          */
    extern struct PARBLK vblock;    /*  TAE parameter block                 */
    long acc = TREAD;               /*  Access of the tape read             */
    long count = 1;                 /*  number of records to skip           */
    char dens[5];                   /*  default density of the tape         */
    char tapeid[CMLEN];             /*  tape id                             */
    long nbytes;                    /*  nbytes to read                      */
    long skip;                      /*  number of files to skip             */
    long status;                    /*  status of function calls            */

    short label = FALSE;            /*  label flag                          */

    strcpy(dens,"0");
    /*  Open the tape unit, determine the blocksize, and skip the
        appropriate files in order to get the tape ready to read.
        -------------------------------------------------------------*/
    if (*first)
    {
        rewopt = REWIND;
        status = c_tpopen(unit,&zero,&handle,&rewopt,dens,&acc,mess,tapeid);
        if (status != TAPE_SUCC)
        {
            c_errmsg("Error opening tape","dementer-tpopen",&nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }

        status = q_intg(&vblock,"UNIT",1,(int *)unit,P_UPDATE);
        if (status != SUCCESS)
        {
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }

        status = q_string(&vblock,"DRIVE",1,&mess,P_UPDATE);
        if (status != SUCCESS)
        {
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }

        status = q_out(&vblock);
        if (status != SUCCESS)
        {
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }

        rewopt = REWOFFL;
    }
    else
        /*  If this isn't the first file to be read then reopen the tape.
            -----------------------------------------------------------------*/
    {
        rewopt = NOREW;
        status = c_tpreop(unit,&zero,&handle,&rewopt,dens,&acc,drive);
        if (status != TAPE_SUCC)
        {
            c_errmsg("Error opening tape","dementer-tpopen",&nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }

        rewopt = REWOFFL;
    }

    if (*form != DTED)
    {
        nbytes = 32760;
        status = c_tpread(unit,buff2,&nbytes,&handle);
        if (status < 0)
        {
            c_errmsg("Error encountered reading tape","dementer-tpread",
                      &nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
        else 
        {
            if (status > 0) 
                *bufsiz = status;
            if (status > 32760)
            {
                c_errmsg("Data set too large","dementer-data",&nonfatal);
                status = c_tpclos(unit,&handle,&rewopt,&zero);
                c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
            }
        }
        if (status == 80)
        {
            skip = (*fnum * 3) - 1;
            label = TRUE;
        }
        else
            skip = *fnum - 1;

        /* Some devices return TAPE_BOT, which seems to be valid since the 
           data does get read in correctly */
        status = c_tpbsr(unit,&count,&handle);
        if (status == TPBSR_BSR)
            /*if ((status != TAPE_SUCC) && (status != TAPE_BOF))*/
        {
            c_errmsg("Error skipping to specified record on tape",
                    "dementer-skip",&nonfatal);
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
    }
    else
        skip = (*fnum - 1) * 3;

    /*  Correct the skip factor for multiple reads.
        -----------------------------------------------*/

    if ((*first) && (*form != DTED) && (label)) 
        skip -= 1;
    if ((!*first) && (*form == DTED))
        skip += 1;
    if (skip != 0)
    {
        status = c_tpfsf(unit,&skip,&handle);
        if (status != TAPE_SUCC)
        {
            status = c_tpclos(unit,&handle,&rewopt,&zero);
            c_errmsg("Error skipping to specified file on tape",
                    "dementer-skip",&nonfatal);
            c_errmsg("Fatal error encountered","dementer-fatal",&fatal);
        }
    }
}  /*  tapeset  */
