/* include files, global parameters, and definitions for DESTRIPE      */

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"
#include "statio.h"

#define NR      0			/* number of lines in local window  */
#define NP      1			/* number of pixels in local window */

#define TERM	0			/* directs output to terminal       */
#define FIL	1			/* directs output to file           */
#define LP	2			/* directs ouput to line printer    */

struct	PARBLK	block;                 /* TAE parameter block     */
struct  DDR	ddr;                   /* ddr                     */

FILE	*fd;			       /* temp file descriptor    */
FILE	*fd1;                          /* output file1 descriptor */
FILE	*fd3;		               /* print file descriptor   */

unsigned char	*inbuf;                /* image input buffer      */
unsigned char	*outbuf1;              /* image output buffer     */

char    out_files[3][CMLEN];           /* output file names       */

long	fdescin[MAXIMG+1];             /* input file descriptors  */
long	fdescout1;                     /* output file1 descriptor  */
long	fdescout2;                     /* output file2 descriptor  */

long	min_max;                       /* minmax option flag      */
long 	msgtim;                        /* message interval        */
long	num_outfiles;                  /* number of output files  */

extern	long	curent;                        /* current entity           */
extern	long	curcat;                        /* current category         */

long	window[MAXIMG][4];    /* input image dimensions (SL SS NL NS) */
long	cell_size[2];	      /* cell size                            */
long	corr_code,	      /* correction codes:                    */
       				      /* 1 - ediesc correction                */
				      /* 2 - smooth correction                */
				      /* 3 - both of above kind               */
				      /* 4 - no correction                    */
				      /* 5 - user's correction                */
		detectors,	      /* number of detectors                  */
		hv_tresh[2], 	      /* horizontal and vertical tresholds    */
		num_stat,	      /* number of statistics:                */
				      /* 1 - before correction                */
    				      /* 2 - before and after correction      */
				      /* 3 - before and after ediesc and      */
                                      /* ediesc and smooth corrections        */
		zero_incl;	      /* 0-include code: 0=include, 1=exclude */

char	correction[6];

void main
(void)  /* destripe */

/*
*****************************************************************************

NAME:	DESTRIPE

PURPOSE: Generates statistics (mean, standard deviation, skewness, and
         kurtosis) of Landsat images and corrects for detector miscalibration.
         It optionally applies a smoothing algorithm.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           Sep-85     S. Kondal/SAR           
   2           July-88    N. Deodhar/SAR     added EDC routines such as
                                             inlas, initm, compl, etc    
   3          9-Mar-89    K. Kunska/STX      conversion to 5.0
   4         24-Mar-89    K. Kunska/STX      fixed display format, processing
                                             message, and smoothing algorithm;
                                             changed LAS parameter names to
                                             agree with the standard.
   5	     19-Jul-89	  K. Kunska/STX      fixed an initialization problem in
                                             ediesc routine.
   5.1       21-Aug-89    J. Marmon/EDC      Changed code to run on pn9050
   5.2        8-NOV-89    P. Pease/GSFC      Changed SMOOTH.C calls to c_startm
                                             & c_stoptm from startm & stoptm.
   5.3        3-Apr-90    K. Kunska/STX      Fixed a min_max problem.
   5.4	     20-Sep-91    T. Rockvam/EDC     Fixed c_complt to output correct
					     file names.
   6.0       19-Oct-99    D. Hames	     Remove reference to MAXNS and
                                             replace with num_samps to support
                                             images >2gig.  Add prototypes.
	

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

*****************************************************************************
*/

{
char    hos_in[MAXIMG][CMLEN],   /* input image host names                    */
	hos_out1[1][CMLEN],	 /* first output image host names             */
	hos_out2[1][CMLEN],      /* second output image host names            */
	print_file[CMLEN],	 /* print file name for FILE option           */
        tae_out1[CMLEN],         /* first output image tae names              */
        tae_out2[CMLEN];         /* second output image tae names             */

float   sratio[16];  		/* stdev sensors / stdev band                 */

double	amax1[MAXBND],	        /* maximum pixel value for hosout1            */
	amax2[MAXBND],		/* maxinum pixel value for hosout2            */
	amin1[MAXBND],		/* minimum pixel value for hosout1            */
	amin2[MAXBND],		/* minimum pixel value for hosout2            */
	band_mean, 		/* band mean                                  */
	sens_std[16],		/* standard deviation of the sensor           */
	sens_mean[16];		/* sensor mean                                */

static long num_img_band[MAXIMG][MAXBND+1]; /* band numbers for each image    */

long    bands_per_image[MAXIMG], /* number of bands per image             */
	cur_band,		/* current band                               */
	data_type, 		/* data type of the image                     */
        fatal_err = -1,		/* fatal error return status                  */
	i,			/* loop counter, array index                  */
	img,			/* current image                              */
	img_limit = MAXIMG,	/* maximum number of images                   */
        image_flag[MAXIMG][2],  /* specifies image/band options               */
        in_band,		/* input band for reading                     */
	istat,			/* type of statistics to be printed           */
	num_inimg,		/* number of input images                     */
	out1_img_band[1][MAXBND+1], /* bands numbers of the output image      */
	out2_img_band[1][MAXBND+1], /* bands numbers of the output image      */
	output_band,		/* output band for ewrite                     */
	output_bands[MAXIMG],   /* output image bands                         */
	outwin[4],		/* output image window specifications         */
	print[3],		/* print options (TERM, LP, FIL)              */
	read_file,		/* flags which file should be read:           */
				/* 0 - IN, 1 - OUT1, 2 - OUT2                 */
	same_wnd,		/* same window size 1=Y, 0=N                  */
	tot_inband,		/* total number of bands to be processed      */
        num_samps;		/* total number of samples in image	      */

long    one = 1;

/*
Get parameters and set up the file specifications
--------------------------------------------------
*/
getpar (hos_in, hos_out1, hos_out2, tae_out1, tae_out2, &data_type, &num_inimg,
	num_img_band, bands_per_image, &tot_inband, img_limit, &same_wnd,
	image_flag, print, print_file, &num_samps);

*output_bands = tot_inband;

filspec (hos_in, hos_out1[0], hos_out2[0], tae_out1, tae_out2, num_inimg,
	 num_img_band, out1_img_band, out2_img_band, bands_per_image,
	 &tot_inband, output_bands);


output_band = 0;
curcat = 0;
for (img = 0; img < num_inimg; img++)
   { 
   for (cur_band = 0; cur_band < bands_per_image[img]; cur_band++)
      {
      curcat++;
      output_band++;

/*
Allocate dynamic memory for the input and buffer
------------------------------------------------
*/
if((inbuf=(unsigned char *)malloc((num_samps) * sizeof(unsigned char)))==NULL)
    c_errmsg ("Error: cannot allocate dynamic memory", "destripe_alloc",
                &fatal_err);

if((outbuf1=(unsigned char *)malloc((num_samps) * sizeof(unsigned char)))==NULL)
    c_errmsg ("Error: cannot allocate dynamic memory", "destripe_alloc",
                &fatal_err);

/*
Call STATIS to calculate statistics before corrections
------------------------------------------------------
*/
      istat = 1;
      read_file = 0;
      in_band = num_img_band[img][cur_band];
      statis (img, in_band, read_file, sratio, &band_mean, sens_mean, 
             sens_std, print, istat, tot_inband, num_samps);

/*
Call EDIESC for edies correction
If necessary, call STATIS to calculate statistics after edies corrections
--------------------------------------------------------------------------
*/
      if (corr_code == 1)
         {
         read_file = 0;
         in_band = num_img_band[img][cur_band];
         ediesc (img, in_band, read_file, output_band, band_mean, sens_mean,
		 sratio, tot_inband);
         if (num_stat == 2)
            {
            istat = 2;
            read_file = 1;
            in_band = out1_img_band[0][output_band-1];
            statis (img, in_band, read_file, sratio, &band_mean, 
                    sens_mean, sens_std, print, istat, tot_inband, num_samps);
            }
         }

/*
Call SMOOTH to apply a smoothing algorithm.
If necessary, call STATIS to calculate statistics after smoothing
------------------------------------------------------------------
*/
      if (corr_code == 2)
         {
         read_file = 0;
         in_band = num_img_band[img][cur_band];
         smooth (img, in_band, read_file, output_band, tot_inband);
         if (num_stat == 2)
            {
            istat = 3;
            read_file = 1;
            in_band = out1_img_band[0][output_band-1];
            statis (img, in_band, read_file, sratio, &band_mean, 
                    sens_mean, sens_std, print, istat, tot_inband, num_samps);
            }
         }

/*
Call EDIESC for edies corrections.
If necessary, call STATIS to calculate statistics after edies correction.
Call SMOOTH to apply a smoothing algorithm.
If necessary, call STATIS to calculate statistics after smooth correction.
--------------------------------------------------------------------------
*/
      if (corr_code == 3)       
         {
         read_file = 0;
         in_band = num_img_band[img][cur_band];
         ediesc (img, in_band, read_file, output_band, band_mean, sens_mean,
		 sratio, tot_inband);

         if (num_stat == 3) 
            {
            istat = 2;
            read_file = 1;
            in_band = out1_img_band[0][output_band-1];
            statis (img, in_band, read_file, sratio, &band_mean, 
                    sens_mean, sens_std, print, istat, tot_inband, num_samps);
            }
         read_file = 1;
         in_band = out1_img_band[0][output_band-1];
         smooth (img, in_band, read_file, output_band, tot_inband);

         if (num_stat == 3)
            {
            istat = 4;
            read_file = 2;
            in_band = out2_img_band[0][output_band-1];
            statis (img, in_band, read_file, sratio, &band_mean, 
                    sens_mean, sens_std, print, istat, tot_inband, num_samps);
            }
         }

/*
Call USERSC to calculate the user's corrections.
--------------------------------------------------
*/ 
      if (corr_code == 5)
         {
         in_band = num_img_band[img][cur_band];
         usersc (img, in_band, output_band, sens_mean, sens_std, tot_inband);
         }

      } /* bands loop */
   } /* img loop   */

/*
Close files
------------
*/
for (i = 0; i < num_inimg; i++)
   c_eclose ((struct FDESC **)&fdescin[i]);
if (corr_code != 4)
   c_eclose ((struct FDESC **)&fdescout1);
if (corr_code == 3)
   c_eclose ((struct FDESC **)&fdescout2);

/*
Update the minmax values
------------------------
*/
if (min_max)
   {
   outwin[SL] = outwin[SS] = 1;
   outwin[NL] = window[0][NL];
   outwin[NS] = window[0][NS];
   if (corr_code != 4)
      {
      c_minmax (hos_out1, &one, (long (*)[4])outwin, out1_img_band,
		output_bands, amin1, amax1);
      c_svmnmx ((char *)hos_out1, output_bands, amin1, amax1, "YES");
      }
   if (corr_code == 3)
      {
      c_minmax (hos_out2, &one, (long (*)[4])outwin, out2_img_band, 
		output_bands, amin2, amax2);
      c_svmnmx ((char *)hos_out2, output_bands, amin2, amax2, "YES");
      }
   }

/*   
Free the dynamic memory.
-----------------------
*/
free (inbuf);
inbuf = NULL;
free (outbuf1);
outbuf1 = NULL;

/*
Generate the history file and print the completion message
-----------------------------------------------------------
*/
if (corr_code  ==  1  ||  corr_code  ==  2  ||  corr_code  ==  5) 
   {
   c_hstout (&block, (char *)hos_out1, hos_in, &num_inimg, " ");
   c_complt ("DESTRIPE", out_files, &num_outfiles); 
   }
else if (corr_code  ==  3) 
   {
   c_hstout (&block, (char *)hos_out1, hos_in, &num_inimg, " ");
   c_hstout (&block, (char *)hos_out2, hos_in, &num_inimg, " ");
   c_complt ("DESTRIPE", out_files, &num_outfiles);
   }
else
   {
   c_complt ("DESTRIPE", out_files, &num_outfiles);
   } 

}        
