#include "destripe.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void ediesc
(
    long img,			/* current image                              */
    long in_band,		/* input band number for eread                */
    long rfile,			/* flags which file should be read            */
    long out_band,		/* output band for ewrite                     */
    double bxbar,		/* band mean                                  */
    double xbar[16], 		/* detector mean                              */
    float sratio[16],		/* stdev sensors / stdev band                 */
    long tot_inbands		/* total number of bands                      */
)

/*
*****************************************************************************

NAME:	EDIESC

PURPOSE: Applies edies correction to the input image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          Oct-84      S. Kendal/SAR
   2         09-Mar-89    K. Kunska          conversion to 5.0    
   3         24-Mar-89    K. Kunska          fixed the processing message.
   4	     19-Jul-89    K. Kunska          fixed an initialization problem.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

*****************************************************************************
*/

{

float	decimal,		/* the first digit after the decimal point    */
	*temp_in,		/* temporary input buffer with float pixels   */
	*temp_out;		/* temporary output buffer with float pixels  */

long	cntary[10],		/* used to determine the element from each    */
				/* column of the round off table that will be */
				/* used in the round off.                     */
	from_type,		/* data type for buffer conversion            */
	i, 			/* loop counter and array index               */
	in_line,		/* input line number for eread                */
	isens, 			/* current sensor                             */
	j,			/* loop counter and array index               */
	line, 			/* line counter                               */
	msg_flag = 1,		/* flag for the processing message utility    */
	num_lines,		/* number of lines in the input image         */
	num_pixels,		/* number of pixels in the input image        */
	out_line,		/* output line number for ewrite              */
	rem,			/* the column to be looked at in the round    */
				/* off table; value will be the first digit   */
				/* after the decimal point.                   */
	temp,			/* pixel value cast as long integer           */
	temp_size,		/* size of the temporary buffers              */
	to_type;		/* data type for buffer conversion            */

unsigned char	roff[1025][10];	/* round off table used to determine if a     */
				/* number is rounded up or down. Added        */
                                /* 11/23/84 for bernstein's statistical       */
                                /* approach.                                  */

long    fatal_err = -1;
long    one = 1;

/*
Initialize the arrays.
----------------------
*/
for (i = 0; i < 10; i++)
   cntary[i] = 0;

for (i = 1; i <= 1024; i++)
   {
   for (j = 1; j <= 5; j++)
      roff[i][j] = 0;
   for (j = 6; j <= 9; j++)
      roff[i][j] = 1;
   }

/*
Call RNDTBL to fill the round off table with 1's.
The first row is filled with 10% 1's, the second with 20% 1's, the third with
30% 1's and so on.
-----------------------------------------------------------------------------
*/
rndtbl (roff);

/* 
Initialize local variables
---------------------------
*/
isens = 0;
num_lines = window[img][NL];
num_pixels = window[img][NS];
out_line = 1;

/*
Allocate dynamic memory for the temporary buffers.
--------------------------------------------------
*/
temp_size = (long)(num_pixels * sizeof (float));
if ((temp_in = (float *) malloc (temp_size)) == NULL ||
    (temp_out = (float *) malloc (temp_size)) == NULL)
    c_errmsg ("Error: cannot allocate dynamic memory", "ediesc_alloc",
               &fatal_err);

/*
Initialize and enable the processing message utility
----------------------------------------------------
*/
c_inittm ("DESTRIPE", "Processed", "line", "band", "for EDIES correction",  
           &num_lines, &tot_inbands, &msgtim, &msg_flag);
curent = 1;
c_startm();


/*
Loop on lines
-------------
*/
for (line = 1; line <= num_lines; line++)
   {
   curent++;
   in_line = line;
   from_type = EBYTE;
   to_type = EREAL;
   if (rfile == 0)
      c_eread ((struct FDESC **)&fdescin[img], &in_band, &in_line, inbuf, &one);
   if (rfile == 1)
      c_eread ((struct FDESC **)&fdescout1, &in_band, &in_line, inbuf, &one);
   if (rfile == 2)
      c_eread ((struct FDESC **)&fdescout2, &in_band, &in_line, inbuf, &one);
   c_pxconv (&from_type, &to_type, inbuf,(unsigned char *)temp_in, &num_pixels);

/*
Perform the sensor test
------------------------
*/
   if (isens > detectors - 1) 
      isens = 0;

/*
Loop on pixels
---------------
*/
   for (i = 0; i < num_pixels; i++)
      {

/*
Perform edies changes
---------------------
*/
      temp_out[i] = (temp_in[i] - xbar[isens]) * sratio[isens] + bxbar;


/*
Perform the round off using the Bernstein's statistical approach.
Determine the column of the round off table to be used.
If the first digit after the decimal point is greater than one and less
than nine then increment the count array for that column.  
This will be the element number in the round off column that will be used to 
do the round off.
If the element number is greater than 1024 set the element to one.
Add the value (1 or 0) at the element number to the output buffer element.
-------------------------------------------------------------------------------
*/
      temp = (long) temp_out[i];
      decimal = 10 * (temp_out[i] - (float)temp);
      rem = (long) decimal;
      if ((rem  >=  1)  &&  (rem  <=  9)) 
         {
         cntary[rem] = cntary[rem] + 1;
         if (cntary[rem]  >  1024) 
            cntary[rem] = 1;
         temp_out[i] = temp_out[i] + roff[cntary[rem]][rem];
         }

      if (temp_out[i] < 0.0)
         temp_out[i] = 0.0;
      if (temp_out[i] > 255.0)
         temp_out[i] = 255.0;
      }

   isens++;


/*
Copy the contents of the output buffer to the output image.
-------------------------------------------------------------
*/
   from_type = EREAL;
   to_type = EBYTE;
   c_pxconv (&from_type, &to_type, (unsigned char *)temp_out, outbuf1, 
	    &num_pixels);
   c_ewrite ((struct FDESC **)&fdescout1, &out_band, &out_line, outbuf1, &one);
   out_line++;
   }

c_stoptm();

/*
Free the dynamic memory
-----------------------
*/
free (temp_in);
free (temp_out);
}
