#include "destripe.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char (*hosin)[CMLEN],    /* input image host names                        */
    char hosout1[],          /* output image host name                        */
    char hosout2[],          /* output image2 host name                       */
    char taeout1[],	     /* output image tae name			      */
    char taeout2[],	     /* output image2 tae name			      */
    long num_inimages,       /* number of input images                        */
    long num_image_bands[][MAXBND + 1], /* band numbers in each input image   */
    long (*out1_img_bands)[MAXBND + 1], /* band numbers for the 1st output img*/
    long (*out2_img_bands)[MAXBND + 1], /* band numbers for the 1st output img*/
    long bands_per_image[],  /* number of bands for each image                */
    long *bands,             /* number of output image bands                  */
    long output_bands[]	     /* output image bands                            */
)

/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0          Oct-84     S. Kendal
   2.0         09-Mar-89   K. Kunska         conversion to 5.0
   3.0         24-Mar-89   K. Kunska         changed parameters passed to
                                             upddr routine.
   3.1	       20-Sep-91   T. Rockvam	     added taeout1 and taeout2 to 
					     be printed in completion message
   6.1	       19-May-94   D. Eichack        Added opt[] parameter used by 
					     eopenr.
**************************************************************************
*/

{

double	line_inc = 1.0,		/* line and sample increments for          */
	samp_inc = 1.0;		/* output image ddr                        */

long	access,			/* file access type                        */
	comb_flag = 0,		/* image combination flag                  */
	maxlines = 1,		/* maximum number of lines to be read and  */
                                /* written into the inbuf and outbuf       */
	n, 
	stat,			/* return status                           */
	type;			/* file type                               */

long    zero = 0;
static	long	opt[3] = {0,0,0};
long    one = 1;

/*
Open input image files
-------------------------
*/
access = IREAD;
for (n = 0; n < num_inimages; n++)
   {
   stat = c_eopenr ((struct FDESC **)&fdescin[n], hosin[n], num_image_bands[n],
		     &window[n][SL], &window[n][SS], &window[n][NL], 
		     &window[n][NS], &access, &zero, opt, &maxlines);
   if (stat != E_SUCC )
      c_abort (&block, "Error opening input image", "filspec-open", "destripe",
               out_files, &num_outfiles);
   }

fdescin[++n] = 0;

/*
Open the  output image files for writing. 
Close the output files.
Note: Nothing is written to the output files at this point. This seemingly 
pointless opening of the files ensures that the associated ddr files are 
created. Then the output files will be open for update. The ddr files must 
exist if the files are to be opened for update.
--------------------------------------------------------------------------
*/
access = IWRITE;
type = EBYTE;
if (corr_code != 4)
   {
   n = 0;
   stat = c_eopenr ((struct FDESC **)&fdescout1, hosout1, bands, &one, &one, 
		   &window[0][NL], &window[0][NS], &access, &type, opt, 
		   &maxlines);
   if (stat != E_SUCC )
      c_abort (&block, "Error opening output image", "filspec-outopen", 
               "destripe", out_files, &num_outfiles);

   c_eclose ((struct FDESC **)&fdescout1);
   }

if (corr_code == 3)
   {
   stat = c_eopenr ((struct FDESC **)&fdescout2, hosout2, bands, &one, &one,
		    &window[0][NL], &window[0][NS], &access, &type, opt, 
		    &maxlines);
   if (stat != E_SUCC )
      c_abort (&block, "Error opening output image", "filspec-outopen", 
               "destripe", out_files, &num_outfiles);

   c_eclose ((struct FDESC **)&fdescout2);
   }


/*
Open the  output image files for update. The output files have been opened 
once before for writing to create the ddr files required by the update option.
Update the ddr files.
----------------------------------------------------------------------------
*/
access = IUPDATE;
if (corr_code != 4)
   {
   stat = c_eopenr ((struct FDESC **)&fdescout1, hosout1, bands, &one, &one, 
		    &window[0][NL], &window[0][NS], &access, &type, opt,
		    &maxlines);
   if (stat != E_SUCC )
      c_abort (&block, "Error opening output image", "filspec-outopen", 
               "destripe", out_files, &num_outfiles);

   for (n = 0; n < *bands; n++)
     out1_img_bands[0][n] = n+1;

   c_upddr (&block, hosin, hosout1, &num_inimages, window, num_image_bands, 
            bands_per_image, &line_inc, &samp_inc, bands, &comb_flag);

   strcpy (out_files[num_outfiles++], taeout1);
   }

if (corr_code == 3)
   {
   stat = c_eopenr ((struct FDESC **)&fdescout2, hosout2, bands, &one, &one,
 		    &window[0][NL], &window[0][NS], &access, &type, opt, 
		    &maxlines);
   if (stat != E_SUCC )
      c_abort (&block, "Error opening output image", "filspec-outopen", 
               "destripe", out_files, &num_outfiles);

   for (n = 0; n < *bands; n++)
     out2_img_bands[0][n] = n+1;

   c_upddr (&block, hosin, hosout2, &one, window, out1_img_bands, 
            output_bands, &line_inc, &samp_inc, bands, &comb_flag);

   strcpy (out_files[num_outfiles++], taeout2);
   }
}
