#include "destripe.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char hosin[][CMLEN],   	/* input image host names                    */
    char hosout1[][CMLEN], 	/* first output image host names             */
    char hosout2[][CMLEN], 	/* second output image host names            */
    char taeout1[],              /* first output image tae names              */
    char taeout2[],              /* second output image tae names             */
    long *data_type,		/* data type of input and output images       */
    long *num_inimg,		/* number of input images                     */
    long (*num_img_band)[MAXBND+1], /* stores band numbers for each image      */
    long *bands_per_image,	/* number of bands per image                  */
    long *tot_inband,		/* total number of bands to be processed      */
    long img_limit,		/* maximum number of images                   */
    long *same_wnd,		/* same window size? 1=Y, 0=N                 */
    long img_flag[MAXIMG][2],	/* image/band options                         */
    long print[],		/* print option (TERM, LP, FIL)               */
    char print_file[],		/* output file                               */
    long *num_samps             /* total number of samples in image           */
)

/*
*****************************************************************************

NAME:	GETPAR

PURPOSE: Retrieves the user supplied parameters from TAE.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          Oct-84      S. Kendal
   2         22-Jul-88    K. Narayanan       revision to make code match the
                                             User's Guide
   3         09-Mar-89    K. Kunska          conversion to 5.0    
   4         24-Mar-89    K. Kunska          changed LAS parameter names to
                                             reflect the standard; added error 
                                             checking on the output image.
   4.1	     20-Sep-91    T. Rockvam	     made a change to print tae image
					     names at top of printout

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

*****************************************************************************
*/

{

char	fileutil_file[CMLEN],	/* file for fileutil                          */
	mnmx[4],		/* minmax flag                                */
	text[80],		/* output text for image names		      */
	zero_flg[4];		/* zero inclusion flag                        */

long	i,			/* loop counter and array index               */
	par_type,		/* type of the parameter                      */
	stat,			/* return status                              */
	std_out;		/* logical unit for standard output           */

struct  DDR	ddr;                   /* ddr                     */

struct VARIABLE	   *var;	/* pointer to a variable in the tae block     */

char    taein[MAXIMG][CMLEN];	/* Tae input file names			      */

long    fatal_err = -1;
long    zero = 0;
long    one = 1;
long    two = 2;

/*
Initialize tae
--------------
*/
c_inlas (&block, &std_out);

/*
Parse the input image specification. 
If the return status is not success, then abort with a fatal error message
---------------------------------------------------------------------------
*/
stat = c_getin (&block, "IN", &zero, taein, hosin, num_inimg, window, 
                num_img_band, bands_per_image, tot_inband, &img_limit, same_wnd,
                img_flag);

if (stat != E_SUCC)
   c_errmsg ("Cannot access input file names", "destripe-getin", 
              &fatal_err);
if (same_wnd == 0)
   c_errmsg ("Image windows differ in size", "destripe_window",
              &fatal_err);

/*
Get the input data type and the image size.  
If the image is not a byte image, then abort with a fatal error message.
-------------------------------------------------------------------------
*/
for (i = 0; i < *num_inimg; i++)
   {
   c_getddr (hosin[i], &ddr);
   *data_type = ddr.dtype;
   if (*data_type != 1)
      c_errmsg ("Data type of input image is not BYTE", "destripe-dtype",
                 &fatal_err);
   }
/*
Get the total number of samples
------------------------------
*/
*num_samps = ddr.ns;

/*
Get the number of detectors and the type of correction.
-------------------------------------------------------
*/
par_type = ELONG;
c_getnum (&block, "NDETECT", &one, &par_type, (unsigned char *)&detectors);
if (window[0][NL] < detectors) 
   c_errmsg ("Number of detectors > number of input image lines",
              "destripe-ndetect", &fatal_err);

var = p_find (&block, "CORRMETH"); 
strcpy (correction, SVAL(*var, 0));

/*
Set the numerical code for correction
--------------------------------------
*/
if (strcmp (correction, "EDIES") == 0)
   corr_code = 1;
if (strcmp (correction, "SMOOTH") == 0)
   corr_code = 2;
if (strcmp (correction, "BOTH") == 0)
   corr_code = 3;
if (strcmp (correction, "NO") == 0)
   corr_code = 4;
if (strcmp (correction, "USER") == 0)
   corr_code = 5;

/*
Get the number of statistics required.
If the number of statistics is not appropriate for the type of correction then
reset the number to the correct value.
-------------------------------------------------------------------------------
*/
c_getnum (&block, "STATOPT", &one, &par_type, (unsigned char *)&num_stat);
if (corr_code == 4 || corr_code == 5)
   num_stat = 1;
if ((corr_code == 1 || corr_code == 2) && num_stat > 2)
   num_stat = 2;

/*
Get the zero inclusion condition
---------------------------------
*/
var = p_find (&block, "ZEROFLG"); 
strcpy (zero_flg, SVAL(*var, 0));
if (strcmp (zero_flg, "YES") == 0)
   zero_incl = 0;
if (strcmp (zero_flg, "NO") == 0)
   zero_incl = 1;


/*
Get the output image parameters
--------------------------------
*/
if (corr_code != 4)
   {
   stat = c_getout (&block, "OUT",(char (*)[256]) taeout1, hosout1);
   if ((stat != E_SUCC) || (strlen(taeout1) == 0))
     c_errmsg ("Correction code requires output image specification",
                "destripe-getout", &fatal_err);
   }

/*
If correction code is BOTH, get the second output image parameters
--------------------------------------------------------------------
*/
if (corr_code == 3)
   {
   stat = c_getout (&block, "OSMOOTH", (char (*)[256])taeout2, hosout2);
   if ((stat != E_SUCC) || (strlen (taeout2) == 0))
      c_errmsg ("Correction code requires output image specification",
                "destripe-getout", &fatal_err);
   }

/*
If the correction code is smooth or both get parameters required for smooth 
algorithm
---------------------------------------------------------------------------
*/
if (corr_code == 2 || corr_code == 3)
   {
   c_getnum (&block, "SMTHRVAL", &two, &par_type,(unsigned char *)hv_tresh);
   c_getnum (&block, "KERNDIM", &two, &par_type, (unsigned char *)cell_size);

   if (cell_size[NR] > window[0][NL] || cell_size[NP] > window[0][NS])
      c_errmsg ("Specified kernel size exceeds the input image size",
                 "destripe-windchk", &fatal_err);
   }

/*
Get the output device for statistics and open the print files
--------------------------------------------------------------
*/
num_outfiles = 0;
strcpy (fileutil_file, "destripe");
stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
          print_file);
if (stat != E_SUCC)
   c_errmsg("Fatal error encountered","destripe-fatal",&fatal_err);

if (print[LP] || print[FIL])
   {
   fd3 = fopen (fileutil_file, "w");
   strcpy (out_files[num_outfiles++], print_file);
   }
else
  fd3 = NULL; 

/*
Print out image names
---------------------
*/
c_txtout("  INPUT IMAGE: ",print,fd3);
for (i = 0; i < *num_inimg; i++)
   {
   if (i == 0)
     sprintf(text,"   %s",taein[i]);
   else
     sprintf(text," + %s",taein[i]);
   c_txtout(text,print,fd3);
   }
if (hosout1[0][0] != '\0')
   {
   c_txtout("  FIRST OUTPUT IMAGE: ",print,fd3);
   sprintf(text,"   %s",taeout1);
   c_txtout(text,print,fd3);
   }
if (hosout2[0][0] != '\0')
   {
   c_txtout("  SECOND OUTPUT IMAGE: ",print,fd3);
   sprintf(text,"   %s",taeout2);
   c_txtout(text,print,fd3);
   }

/*
Get LAS globals
----------------
*/
c_getnum (&block, "$MSGTIM", &one, &par_type, (unsigned char *)&msgtim);
var = p_find(&block, "$MINMAX");
strcpy (mnmx, SVAL(*var, 0));
if (mnmx[0] == 'Y')
   min_max = TRUE;
else
   min_max = FALSE;

}        
