#include "destripe.h"
#include "protos.h"

void prompt
(
    double xbar[],		/* old sensor mean                            */
    double xbarn[],		/* user supplied sensor mean                  */
    double stdev[],		/* sensor standard deviation                  */
    float sratio[],		/* stdev of sensor / stdev of band            */
    float change[]		/* type of change: 1 - no change              */
)
				/*                 2 - mean change            */
				/*                 3 - stdev change           */
				/*                 4 - mean and stdev change  */

/*
*****************************************************************************

NAME:	PROMPT

PURPOSE: Prompts the user for new mean and standard deviation changes for
         selected sensors.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          Oct-84      S. Kendal/SAR
   2         09-Mar-89    K. Kunska          conversion to 5.0    
   3         24-Mar-89    K. Kunska          fixed the display format

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

*****************************************************************************
*/

{

char	*string,		/* pointer to the text buffer used for reading*/
          			/* the user's keyboard input                  */
	text[80];		/* text buffer                                */

float	smean,			/* user supplied sensor mean                  */
	stadev;			/* user supplied sensor standard deviation    */

long	isens,			/* user supplied sensor number                */
	loop_flag1,		/* controls detector prompt for mean changes  */
	loop_flag2,		/* controls detector prompt for stdev changes */
	mean_flag,		/* controls new mean prompt                   */
	std_flag;		/* controls new standard deviation prompt     */

/*
Initialize local flags.
-----------------------
*/
loop_flag1 = TRUE;
loop_flag2 = TRUE;
mean_flag = TRUE;
std_flag = TRUE;
string = text;

/*
Prompt for mean changes
------------------------
*/
printf ("\nMean changes......\n");
while (loop_flag1)
   {
   printf ("Enter detector number (1 - %2d) or carriage return :\n", 
             detectors);
   gets (string);
   if (*string != '\0')
      {
      sscanf (string, "%ld", &isens);
      if (isens < 1 || isens > detectors)
         printf ("Invalid detector number...... Re-enter.\n");
      else
         {
         mean_flag = TRUE;
         while (mean_flag)
            {
            printf ("Old mean = %8.3f .... Enter new value (0 - 255) :\n", 
                    xbar[isens-1]);
            gets (string);
            sscanf (string, "%f", &smean);
            if (smean < 0.0 || smean > 255.0)
               printf ("Invalid mean...... Re-enter.\n");
            else
               {
               xbarn[isens-1] = smean;
               change[isens-1] = 2;
               mean_flag = FALSE;
               }
            }
         }
      }
   else
      loop_flag1 = FALSE;
   }

/*
Prompt for standard deviation changes
--------------------------------------
*/
printf ("Standard deviation changes....\n");
while (loop_flag2)
   {
   printf ("Enter detector number (1 - %2d) or carriage return :\n", 
          detectors);
   gets (string);
   if (*string != '\0')
      {
      sscanf (string, "%ld", &isens);
      if (isens < 1 || isens > detectors)
         printf ("Invalid detector number...... Re-enter.\n");
      else
         {
         std_flag = TRUE;
         while (std_flag)
            {
            printf ("Old stdev = %8.3f ... Enter new value(0-255) :\n", 
                   stdev[isens-1]);
            gets (string);
            sscanf (string, "%f", &stadev);
            if (stadev < 0.0 || stadev > 255.0)
               printf ("Invalid standard deviation......  Re-enter.\n");
            else
               {
               sratio[isens-1] = stadev/stdev[isens-1];
               if (change[isens-1] == 1 || change[isens-1] == 3)
                   change[isens-1] = 3;
               else
                   change[isens-1] = 4;
               std_flag = FALSE;
               }
            }
         }
      }
   else
      loop_flag2 = FALSE;
   }
}
