#include "destripe.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void smooth
(
    long img,			/* current image being processed              */
    long in_band,		/* input band for eread                       */
    long rfile,			/* flags which file needs to be read          */
    long out_band,		/* output band for ewrite                     */
    long tot_inbands		/* total number of bands                      */
)
/*
*****************************************************************************

NAME:	SMOOTH

PURPOSE: Applies a smoothing algorithm to an image.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          Oct-84      S. Kondal/SAR
   2         09-Mar-89    K. Kunska          conversion to 5.0    
   3         24-Mar-89    K. Kunska          changed the algorithm to
                                             reflect C array subscripts.
   4          8-NOV-89    P. Pease           changed call to c_startm &
                                             c_stoptm from startm & stoptm.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

*****************************************************************************
*/

{
double	sum;			/* sum of pixels in cell                      */

float	*buffer,		/* I/O buffer                                 */
	*cbuf[15],		/* circular buffer                            */
 	count,			/* count of pixels used in the average        */
	*temp_in,		/* temporary input buffer                     */
	*temp_out,		/* temporary output buffer                    */
	wavg;			/* average of pixels in the cell              */

long	bad_line,		/* bad line flag                              */
	cbuf_size,		/* size of the cbuf buffer                    */
	cont_flag,		/* continue flag                              */
	edge,			/* edge flag                                  */
	from_type,		/* data type for buffer conversion            */
	i,			/* loop counter and array index               */
	iendl,			/* ending line                                */
	iendp,			/* ending pixel                               */
	imid,			/* middle pixel                               */
	initl,			/* initial line                               */
	initp, 			/* initial pixel                              */
	in_line,		/* input line for eread                       */
	ip, ipix,		/* pixel index into the circular array        */
	isav,			/* row index into the circular buffer         */
	j,			/* loop counter and array index               */
	line,			/* line loop counter                          */
	lmid,			/* middle line                                */
	msg_flag = 1,		/* flag for the processing message utility    */
	num_lines,		/* number of lines in the input image         */
	num_pixels, 		/* number of pixels in the input image        */
	out_line,		/* output line for ewrite                     */
	skip_flag,		/* flag for skipping part of code if necessary*/
	temp_size,		/* size of the temp buffers                   */
	to_type,		/* data type for buffer conversion            */
	write_flag;		/* flag for writing                           */

long    fatal_err = -1;
long    one = 1;

/*
Initialize local  variables
---------------------------
*/
num_lines = window[img][NL];
num_pixels = window[img][NS];
out_line = 1;
isav = 0;
lmid = cell_size[0]/2;
imid = cell_size[1]/2;
initl = lmid;
iendl = num_lines - lmid;
initp = imid;
iendp = num_pixels - imid;

/*
Allocate dynamic memory for the work buffers
--------------------------------------------
*/
cbuf_size = (long)(15 * num_pixels * sizeof(float));
temp_size = (long)(num_pixels * sizeof (float));
if ((buffer = (float *) malloc(cbuf_size)) == NULL ||
    (temp_in = (float *) malloc(temp_size)) == NULL ||
    (temp_out = (float *) malloc(temp_size)) == NULL)
    c_errmsg ("Error: cannot allocate dynamic memory", "smooth_alloc",
               &fatal_err);
for (i = 0, j = 0; i < 15; i++, j+= num_pixels)
   cbuf[i] = &buffer[j];

/*
Initialize and enable the processing message utility
-----------------------------------------------------
*/
c_inittm ("DESTRIPE", "Processed", "line", "band", "for SMOOTH correction", 
          &num_lines, &tot_inbands, &msgtim, &msg_flag);
curent = 1;
c_startm();

/*
For all lines, read the input line and decide if it can be processed.
If line < initial line or line > ending line, (edge) set bad_line flag to true.
If line > initial line (processing in progress), set continue flag to true.
If the line = initial line, fill in the circular buffer and set the
continue flag to true.
-------------------------------------------------------------------------------
*/
in_line = 1;
for (line = 1; line <= num_lines; line++)   
   {
   bad_line = cont_flag = skip_flag = write_flag = FALSE;
   curent++;
   if ((line-1 < initl) || (line > iendl))
      bad_line = TRUE;                     
   else if (line-1 > initl)
      cont_flag = TRUE;                     
   if (line-1 == initl)
      {   
      for (i = initl; i <= cell_size[0]-1; i++)   
         {
         if (rfile == 0)
            c_eread ((struct FDESC **)&fdescin[img], &in_band, &in_line, inbuf,
		     &one);
         if (rfile == 1)
            c_eread ((struct FDESC **)&fdescout1, &in_band, &in_line, inbuf,
		     &one);
         if (rfile == 2)
            c_eread ((struct FDESC **)&fdescout2, &in_band, &in_line, inbuf, 
		     &one);
         in_line++;
         from_type = EBYTE;
         to_type = EREAL;
         c_pxconv (&from_type, &to_type, inbuf,(unsigned char *)temp_in,
		   &num_pixels);
         for (j = 0; j < num_pixels; j++)       
            cbuf[isav][j] = temp_in[j];
         isav++;
         }
      cont_flag = TRUE;
      }
/*
If continue flag is true, process the pixels in the circular buffer.
--------------------------------------------------------------------
*/
   if (cont_flag)
      {
      for (ipix = 0; ipix < num_pixels; ipix++)      
         {
/*
Compare the horizontal and veritcal thresholds with the horizontal and 
vertical differences among the pixels in the cell.
-------------------------------------------------------------------------
*/
         if (ipix < initp || ipix > iendp-1) 
            {
            temp_out[ipix] = cbuf[lmid][ipix];
            }
         else
            {
            edge = FALSE;
            if (lmid > 0 && lmid < cell_size[0]-1) 
               {
               if (abs(cbuf[lmid][ipix]-cbuf[lmid-1][ipix])>hv_tresh[0] || 
                   abs(cbuf[lmid][ipix]-cbuf[lmid+1][ipix])>hv_tresh[0])
                
               if (abs(cbuf[lmid][ipix]-cbuf[lmid][ipix-1]) > hv_tresh[1] || 
                   abs(cbuf[lmid][ipix]-cbuf[lmid][ipix+1]) > hv_tresh[1])
                   edge = TRUE;
               }
            else if (lmid == cell_size[0]-1) 
               {
               if (abs(cbuf[lmid][ipix]-cbuf[lmid-1][ipix])>hv_tresh[0] || 
                  abs(cbuf[lmid][ipix]-cbuf[0][ipix]) > hv_tresh[0]) 
 
              if (abs(cbuf[lmid][ipix]-cbuf[lmid][ipix-1])>hv_tresh[1] || 
                  abs(cbuf[lmid][ipix]-cbuf[lmid][ipix+1]) > hv_tresh[1])
                  edge = TRUE;
               }
            else if (lmid == 0) 
               {
               if (abs(cbuf[lmid][ipix]-
                        cbuf[cell_size[0]-1][ipix]) > hv_tresh[0] || 
                  abs(cbuf[lmid][ipix]-cbuf[lmid+1][ipix]) > hv_tresh[0]) 
 
               if (abs(cbuf[lmid][ipix]-cbuf[lmid][ipix-1]) > hv_tresh[1] || 
                  abs(cbuf[lmid][ipix]-cbuf[lmid][ipix+1]) > hv_tresh[1])
                  edge = TRUE;
               }  /* if */
/*
If the horizontal or vertical differences among the pixels in the cell
are grater than the appropriate horizontal and vertical tresholds, replace
the center pixel with the average of all the pixels in the cell including
the center pixel.
-------------------------------------------------------------------------
*/
            if (!edge) 
               {
               count = 0.0;
               sum = 0.0;
               for (j = 0; j < cell_size[0]; j++)
                  {
                  ip = ipix - imid;
                  for (i = 0; i < cell_size[1]; i++)
                     {
/*
Accumulate the sum of the pixels in the cell
----------------------------------------------
*/
                     sum = sum + cbuf[j][ip];
                     count = count + 1.0;
                     ip++;
                     }
                  }
/*
Compute the average and place it in the output buffer.
------------------------------------------------------
*/
                  wavg = sum/count + 0.5;
                  if (wavg < 0.0) 
                     wavg = 0.0;
                  if (wavg > 255.0) 
                      wavg = 255.0;
                  temp_out[ipix] = wavg;
                  sum = 0.0;
                  count = 0.0;
                  }
/*
If the horizontal or vertical differences among the pixels in the cell
are less than the appropriate horizontal and vertical tresholds, replace
the center pixel in the output buffer with the center pixel in the input buffer.
--------------------------------------------------------------------------------
*/
               else
                  {
                  temp_out[ipix] = cbuf[lmid][ipix];
                  }  /* if */
               }  /* if */
            }   /* loop */

         write_flag = TRUE;       /*go to 800*/
         } /* cont_flag */


/* 
If bad_line flag is true, process the edges.
If current line < initial line, copy the input line to the output buffer.
If current line > endline, copy the saved line to the output buffer. 
---------------------------------------------------------------------------
*/
      if (bad_line)
         {
         if (line-1 < initl)       /*--- (760) ---*/
            {
            if (rfile == 0)
               c_eread ((struct FDESC **)&fdescin[img], &in_band, &in_line, 
			inbuf, &one);
            if (rfile == 1)
               c_eread ((struct FDESC **)&fdescout1, &in_band, &in_line, 
			inbuf, &one);
            if (rfile == 2)
               c_eread ((struct FDESC **)&fdescout2, &in_band, &in_line, inbuf,
			 &one);
            in_line++;
            from_type = EBYTE;
            to_type = EREAL;
            c_pxconv (&from_type, &to_type, inbuf, (unsigned char *)temp_in, 
		      &num_pixels);
            for (j = 0; j < num_pixels; j++)
                {
                cbuf[isav][j] = temp_in[j];
                temp_out[j] = temp_in[j];
                }
            isav++;
            }
         else
            {
            lmid++;
            if (lmid > cell_size[0]-1)
               lmid = 0; 
            for (j = 0; j < num_pixels; j++)
               temp_out[j] = cbuf[lmid][j];
            }  
            write_flag = TRUE;
         }    /* bad_line */
/*
Write the output line to the file
---------------------------------
*/
         if (write_flag)                  /* 800 */
            {
            from_type = EREAL;
            to_type = EBYTE;
            c_pxconv (&from_type, &to_type, (unsigned char *)temp_out, outbuf1,
		       &num_pixels);
            if (rfile == 0)
               c_ewrite ((struct FDESC **)&fdescout1, &out_band, &out_line, 
			outbuf1, &one);
            if (rfile == 1)
               c_ewrite ((struct FDESC **)&fdescout2, &out_band, &out_line,
			 outbuf1, &one);
            out_line++;
            if (line-1 < initl || line > iendl)
               skip_flag = TRUE;         

/*
Read in the next input line and copy it to the circular buffer.
---------------------------------------------------------------
*/
            if (!skip_flag)
               {
               if (in_line <= num_lines)
                  {
                  if (rfile == 0)
                     c_eread ((struct FDESC **)&fdescin[img], &in_band, 
			        &in_line, inbuf, &one);
                  if (rfile == 1)
                     c_eread ((struct FDESC **)&fdescout1, &in_band, &in_line, 
			       inbuf, &one);
                  if (rfile == 2)
                     c_eread ((struct FDESC **)&fdescout2, &in_band, &in_line, 
			       inbuf, &one);
                  in_line++;
                  from_type = EBYTE;
                  to_type = EREAL;
                  c_pxconv (&from_type, &to_type,inbuf,(unsigned char *)temp_in,
 		            &num_pixels);
                  if (isav > cell_size[0]-1)
                     isav = 0;
                  lmid++;
                  if (lmid > cell_size[0]-1)
                     lmid = 0;
                  for (j = 0; j < num_pixels; j++)
                      cbuf[isav][j] = temp_in[j];
                  isav++;
                  }
               } /* skip_flag */
            }    /* write_flag */
         }
   c_stoptm();
/*
Free the dynamic memory
-----------------------
*/
free (buffer);
free (temp_in);
free (temp_out);
}
