#include "destripe.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void statis
(
    long img,			/* current image being processed              */
    long in_band,		/* band number for eread                      */
    long rfile,			/* flags which file should be read            */
    float sratio[],		/* stdev sensors / stdev band                 */
    double *bxbar, 		/* band mean                                  */
    double xbar[], 		/* sensors mean                               */
    double stdev[],		/* sensor's standard deviation                */
    long print[],		/* print options (TERM, LP, FIL)              */
    long istat,			/* type of statistic to be printed            */
    long tot_inbands,		/* total number of bands                      */
    long num_samps		/* total number of samples in the image       */
)

/*
*****************************************************************************

NAME:	STATIS

PURPOSE: Generates statistics (mean, standard deviation, skewness, and
         kurtosis) of Landsat images. The results are displayed on a 
         line printer or terminal.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          Oct-84      S. Kondal/SAR
   2         June-86      B. Kovalick/SAR     #209
   3        09-Mar-89     K. Kunska/STX       conversion to 5.0    
   4        24-Mar-89     K. Kunska/STX       fixed the display format and
                                              the processing message.
   4.1         Apr-89     D. Hollaren         Added statics to run on PN9050
   5         19-Oct-99    D. Hames	      Add variable num_samps and use
					      to replace MAXNS used for
				              allocating memory for temp_in.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

*****************************************************************************
*/

{
char	text[80];		/* text to be send to the output device       */

static double	bmin,		/* band minimum                               */
	bmax, 			/* band maximum                               */
	bkurt,			/* band kurtosis                              */
	bskew, 			/* band skewness                              */
	bsx1, 			/* sum of pixels for the band                 */
	bsx2, 			/* sum of squares for the band                */
	bsx3, 			/* sum of cubes for the band                  */
	bsx4, 			/* sum of quadruples for the band             */
	bxbar2, 		/* square of band mean                        */
	bvar, 			/* band variance                              */
	bstdev,			/* band standard deviation                    */
	senmin, 		/* sensor minimum                             */
	senmax, 		/* sensor maximum                             */
	sx1[16], 		/* sum of input pixels for each sensor        */
	sx2[16], 		/* sum of squares for each sensor             */
	sx3[16], 		/* sum of cubes for each sensor               */
	sx4[16], 		/* sum of quadruples for each sensor          */
	skew[16], 		/* sensor skewness                            */
	skurt[16], 		/* sensor kurtosis                            */
	var,			/* sensor variance                            */
	xbar2, 			/* square of sensor mean                      */
	xcube[16], 		/* cube of the input pixels for each sensor   */
	xquad[16],		/* quadruple of the pixels for each sensor    */
	xsq[16];		/* square of the input pixels for each sensor */

static float	bnsa,		/* number of pixels for the band              */
	nsa[16], 		/* number of pixels for each sensor           */
	*temp_in;       	/* temporary input buffer with float pixels   */

static long	from_type,	/* data type of a buffer (used for conversion)*/
	go_ahead,		/* go ahead flag for staying or exiting loops */
	i, j, n, j1, j2,	/* loop counters and array indexes            */
	in_line,		/* line number for eread                      */
	isens,			/* current sensor                             */
	line, 			/* line counter                               */
	msg_flag = 1,		/* flag for the processing message utility    */
	ncol,			/* output column format                       */
	nsens, 			/* number of sensors specified by the user    */
	num_lines,		/* number of lines in the input image         */
	num_pixels,		/* number of pixels in the input image        */
	nwr[16],		/* used for sensor numbering                  */
	to_type;		/* data type of a buffer (used for conversion)*/

long    fatal_err = -1,         /* fatal error return status                  */
        one = 1;

for (i = 0; i < 16; i++)
   nwr[i] = i + 1;


/* 
Initialize the local variables
------------------------------
*/
num_pixels = window[img][NS];
num_lines = window[img][NL];
from_type = EBYTE;
to_type = EREAL;
*bxbar = 0.;
isens = 0;
bmin = 999;
bmax = 0;
bnsa = 0;
bsx1 = 0.0;
bsx2 = 0.0;
bsx3 = 0.0;
bsx4 = 0.0;
bxbar2 = 0;
bvar = 0;
bstdev = 0;
bskew = 0;
bkurt = 0;

/*
Fill the sensors arrays with zeros
-----------------------------------
*/
for (j = 0; j < detectors; j++)
   {
   nsa[j] = 0;
   sx1[j] = 0;
   xsq[j] = 0.0;
   sx2[j] = 0;
   xcube[j] = 0.0;
   sx3[j] = 0;
   xquad[j] = 0.0;
   sx4[j] = 0;
   xbar[j] = 0;
   stdev[j] = 0;
   skew[j] = 0;
   skurt[j] = 0;
   sratio[j] = 0;
   }

/*
Initialize and enable processing message utility
-------------------------------------------------
*/
c_inittm ("DESTRIPE", "Processed", "line", "band", "for statistics", 
          &num_lines, &tot_inbands, &msgtim, &msg_flag);
curent = 1;
c_startm();

/* Allocate memory for temp_in 
------------------------------
*/
if  ((temp_in=(float *)malloc(num_samps * sizeof(float))) == NULL)
    c_errmsg ("Error allocating dynamic memory", "statis-alloc",&fatal_err);


/*
Loop on lines
-------------
*/
for (line = 1; line <= num_lines; line++)
   {
   curent++;
   in_line = line;
   if (rfile == 0)
      c_eread ((struct FDESC **)&fdescin[img], &in_band, &in_line, inbuf, &one);
   if (rfile == 1)
      c_eread ((struct FDESC **)&fdescout1, &in_band, &in_line, inbuf, &one);
   if (rfile == 2)
      c_eread ((struct FDESC **)&fdescout2, &in_band, &in_line, inbuf, &one);
   c_pxconv (&from_type, &to_type, inbuf,(unsigned char *)temp_in, &num_pixels);
               
/*
Perform a sensor increment and test
------------------------------------
*/
   if (isens > detectors-1) 
      isens = 0;
   senmin = 999;
   senmax = 0;

/*
Loop on samples
----------------
*/
   for (i = 0; i < num_pixels; i++)
      {
      go_ahead = TRUE;
      if (zero_incl > 0 && temp_in[i] <= 0) 
         go_ahead = FALSE;
      if (go_ahead)
         {

/* 
Compute sensor minimum and maximum.
Compute sums, squares, cubes and quadruples of pixels values for each sensor.
-----------------------------------------------------------------------------
*/
         if (temp_in[i] < senmin) 
            senmin = temp_in[i];
         if (temp_in[i] > senmax) 
            senmax = temp_in[i];

         sx1[isens] = sx1[isens] + temp_in[i];
         xsq[isens] = temp_in[i] * temp_in[i];
         sx2[isens] = sx2[isens] + xsq[isens];
         xcube[isens] = xsq[isens] * temp_in[i];
         sx3[isens] = sx3[isens] + xcube[isens];
         xquad[isens] = xcube[isens] * temp_in[i];
         sx4[isens] = sx4[isens] + xquad[isens];
         nsa[isens] = nsa[isens] + 1.0;
         }
      }

/*
Set minimum and maximum for the band
------------------------------------
*/
   if (senmin < bmin) 
      bmin = senmin;
   if (senmax > bmax) 
      bmax = senmax;
   isens++;
   }
c_stoptm();

/*
Compute mean,standard deviation,skewness,kurtosis for the sensors
--------------------------------------------------------------------
*/
for (isens = 0; isens < detectors; isens++)
   {
   bsx1 = bsx1 + sx1[isens];
   bsx2 = bsx2 + sx2[isens];
   bsx3 = bsx3 + sx3[isens];
   bsx4 = bsx4 + sx4[isens];
   bnsa = bnsa + nsa[isens];
   var = 0;
   xbar2 = 0;
   xbar[isens] = sx1[isens] / nsa[isens];
   xbar2 = xbar[isens] * xbar[isens];
   var = sx2[isens] / nsa[isens]-xbar2;
   if (var < 0) 
      var = -var;
   stdev[isens] = sqrt(var);
   if (stdev[isens] !=  0)
      {
      skew[isens] = (sx3[isens] / nsa[isens]-3.0 * (sx2[isens] / 
                     nsa[isens]) * xbar[isens] + 2.0 * xbar2 * 
                     xbar[isens]) / (var * stdev[isens]);
      skurt[isens] = ((sx4[isens] / nsa[isens]-4.0 * (sx3[isens] / 
                     nsa[isens]) * xbar[isens] + 6.0 * (sx2[isens] /
                     nsa[isens]) * xbar2-3.0 * xbar2 * xbar2) / 
                     (var * var))-3.0;
      }
   else
      {
      skew[isens] = 0;
      skurt[isens] = 0;
      }  
   }

/*
Compute mean,standard deviation,skewness,kurtosis for the band
---------------------------------------------------------------
*/
*bxbar = bsx1 / bnsa;
bxbar2 = *bxbar * *bxbar;
bvar = bsx2 / bnsa - bxbar2;
if (bvar < 0) 
   bvar = -bvar;
bstdev = sqrt(bvar);
if (bstdev != 0)
   {
   bskew = (bsx3 / bnsa-3.0 * (bsx2 / bnsa) * *bxbar + 2.0 * bxbar2 * 
            *bxbar) / (bvar * bstdev);
   bkurt = (bsx4 / bnsa-4.0 * (bsx3 / bnsa) * *bxbar + 6.0 * (bsx2 / 
            bnsa) * bxbar2-3.0 * bxbar2 * bxbar2) / (bvar * bvar)-3.0;
   }
else
   {
   bskew = 0;
   bkurt = 0;
   }  

/*
Compute standard deviation ratios
----------------------------------
*/
for (i = 0; i < detectors; i++)
    {
    if (bstdev !=  0 && stdev[i] != 0)
       sratio[i] = bstdev / stdev[i];
    else
       sratio[i] = 0;
    }

/*
Write computed statistics on the device
----------------------------------------
*/
c_txtout("\nSTATISTICS FOR IMAGE NORMALIZATION", print, fd3);
if (istat == 1)
   {
   c_txtout("BEFORE ANY CORRECTION ", print, fd3);
   }
else if (istat == 2)
   {
   c_txtout("AFTER EDIES CORRECTION ", print, fd3);      
   }
else if (istat == 3)
   {
   c_txtout("AFTER SMOOTH CORRECTION ", print, fd3);      
   }
else if (istat == 4)
   {
   c_txtout("AFTER EDIES AND THEN SMOOTH CORRECTION ", print, fd3);      
   }
sprintf(text, "\n          MINIMUM     MAXIMUM        MEAN      STDDEV    SKEWNESS    KURTOSIS ");
c_txtout (text, print, fd3);
sprintf (text, "band %3d %8.3f    %8.3f    %8.3f    %8.3f    %8.3f  %10.3f", 
         in_band, bmin, bmax, *bxbar, bstdev, bskew, bkurt);
c_txtout (text, print, fd3);

ncol = 6;
nsens = (detectors + ncol-1) / ncol;
for (i = 1; i <=  nsens; i++)
   {
   j1 = (i-1) * ncol + 1;
   j2 = j1 + ncol-1;
   if (j2 > detectors)
      j2 = detectors;

   strcpy(text, "       ");
   for (n = j1; n <= j2; n++)
      sprintf (&text[strlen(text)], "DETECTOR%2d  ", nwr[n-1]);
   c_txtout (text, print, fd3);

   sprintf (text, "MEAN     ");
   for (n = j1; n <= j2; n++)
      {
      sprintf (&text[strlen(text)], "%8.3f", xbar[n-1]);
      if (n < j2)
         sprintf (&text[strlen(text)], "    ");
      }
   c_txtout (text, print, fd3);

   sprintf (text, "STDDEV   ");
   for (n = j1; n <= j2; n++)
      {
      sprintf (&text[strlen(text)], "%8.3f", stdev[n-1]);
      if (n < j2)
         sprintf (&text[strlen(text)], "    ");
      }
   c_txtout (text, print, fd3);

   sprintf (text, "SKEWNESS ");
   for (n = j1; n <= j2; n++)
      {
      sprintf (&text[strlen(text)],"%8.3f", skew[n-1]);
      if (n < j2)
         sprintf (&text[strlen(text)], "    ");
      }
   c_txtout (text, print, fd3);

   sprintf (text, "KURT.  ");
   for (n = j1; n <= j2; n++)
      {
      sprintf (&text[strlen(text)], "%10.3f", skurt[n-1]);
      if (n < j2)
         sprintf (&text[strlen(text)], "  ");
      }
   c_txtout (text, print, fd3);

   c_txtout("MULTIPLIER TO NORMALIZE TO :", print, fd3);

   sprintf (text, "STDV RATIO ");
   for (n = j1; n <= j2; n++)
      {
      sprintf (&text[strlen(text)], "%6.3f", sratio[n-1]);
      if (n < j2)
         sprintf (&text[strlen(text)], "      ");
      }
   c_txtout (text, print, fd3);
   c_txtout (" ", print, fd3);      

   }
/*
Free the dynamic memory
-----------------------
*/
free (temp_in);
}
