#include "destripe.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void usersc
(
    long img,			/* current image being processed              */
    long in_band,		/* band number for eread                      */
    long out_band,		/* output band for ewrite                     */
    double xbar[],			/* sensor's mean                              */
    double stdev[],		/* sensor's standard deviation                */
    long tot_inbands		/* total number of bands                      */
)

/*
**************************************************************************
NAME: USERSC

PURPOSE: Performs user defined corrections.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0          Oct-84     S. Kendal
   2.0         09-Mar-89   K. Kunska         conversion to 5.0
   3.0         24-Mar-89   K. Kunska         fixed the processing message

**************************************************************************
*/

{
double	xbarn[16];

float	sratio[16], 
	chng_switch[16], 
	*temp_in,		/* temporary input buffer with float pixels   */
	*temp_out;		/* temporary output buffer with float pixels  */

long	from_type,		/* data type of the inbuf                     */
	i,			/* loop counter and array index               */
	in_line,		/* line number for eread                      */
	isens,			/* sensor index                               */
	line,			/* line counter                               */
	msg_flag = 1,		/* flag for the processing message utility    */
	num_lines,		/* number of lines in the input image         */
	num_pixels,		/* number of pixels in the input image        */
	out_line,		/* line number for ewrite                     */
	temp_size,		/* size of the temporary buffers              */
	to_type;		/* data type of the tempbuf                   */

long    fatal_err = -1;
long    one = 1;

for (i = 0; i < detectors; i++)
   chng_switch[i] = 1;      


num_lines = window[img][NL];
num_pixels = window[img][NS];
out_line = 1;

/* 
Allocate dynamic memory for the temporary buffers
-------------------------------------------------
*/
temp_size = (long)(num_pixels * sizeof (float));
if ((temp_in = (float *) malloc (temp_size)) == NULL ||
    (temp_out = (float *) malloc (temp_size)) == NULL)
    c_errmsg ("Error: cannot allocate dynamic memory", "usersc_alloc",
               &fatal_err);

/*
Prompt for mean and standard deviation changes
-----------------------------------------------
*/

prompt (xbar, xbarn, stdev, sratio, chng_switch);

isens = 0;


/*
Initialize and enable processing message utility
------------------------------------------------
*/
c_inittm ("DESTRIPE", "Processed", "line", "band", "for USER correction",
           &num_lines, &tot_inbands, &msgtim, &msg_flag);
curent  =  1;
c_startm(); 

/*
Loop on lines
-------------
*/
for (line = 1; line <= num_lines; line++)
   {
   curent++;
   in_line = line;
   from_type = EBYTE;
   to_type = EREAL;
   c_eread ((struct FDESC **)&fdescin[img], &in_band, &in_line, inbuf, &one);
   c_pxconv (&from_type, &to_type, inbuf,(unsigned char *)temp_in, &num_pixels);
/*
Perform the sensor test.
-----------------------
*/
   if (isens > detectors-1) 
      isens = 0;

/*
Loop on samples
---------------
*/
   for (i = 0; i < num_pixels; i++)
      {

/*
Perform the appropriate change:
  1- no change
  2- mean change
  3- stanadard deviation change
  4- mean and standard deviation change
-----------------------------------------
*/
      if (chng_switch[isens] == 1)
         {
         temp_out[i] = temp_in[i];
         }
      else if (chng_switch[isens] == 2)
         {
         temp_out[i] = (temp_in[i] + xbarn[isens]) - xbar[isens] + 0.5;
         }
      else if (chng_switch[isens] == 3)
         {
         temp_out[i] = (temp_in[i] - xbar[isens]) * sratio[isens] + 
                      xbar[isens] + 0.5;
         }
      else if (chng_switch[isens] == 4)
         {
         temp_out[i] = (temp_in[i] - xbar[isens]) * sratio[isens] + 
                        xbarn[isens] + 0.5;
         }  

      if (temp_out[i] < 0.0)
         temp_out[i] = 0.0;
      if (temp_out[i] > 255.0)
         temp_out[i] = 255.0;
      }
   isens++;

/* 
Copy the pixels to output image
-------------------------------
*/
   from_type = EREAL;
   to_type = EBYTE;
   c_pxconv (&from_type,&to_type,(unsigned char *)temp_out,outbuf1,&num_pixels);
   c_ewrite ((struct FDESC **)&fdescout1, &out_band, &out_line, outbuf1, &one);
   out_line++;
   }
c_stoptm(); 

/* 
Free the dynamic memory
-----------------------
*/
free (temp_in);
free (temp_out);
}
