/******************************************************************************

NAME:	DESWATH

FUNCTION:
	DESWATH is performs a filtering on tm data.

PROGRAM HISTORY:
  Version	Date       Author      	Request
  -------	----	   ------       -------
    1.0		12/88	   R.McKinney   Written for LAS 5.0
    2.0		03/90	   B.Davis   	changed pxmnmx to c_pxmnmx
    3.0		09/92	   T.Mittan	Added extern to curcat and to curent
					to eliminate warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
 	The TAE input parameters are retrieved.  Error checking is done on the 
        input image(s) to insure that they all have equal window sizes.  Once 
	the TAE parameters are inputted, the input image is copied to the output
	image.  After the image has been filtered, the min max file and the 
 	history file are created/updated to represent the output image.  

ALGORITHM REFERENCES     

   None.

******************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC *  fdesc[MAXIMG+1]; /* image file descriptors used for image I/O */
struct GDESC *  gdesc;		     /* group descriptor used for image I/O       */
long offset1[MAXIMG*MAXBND+1];  /* offsets into "thebuf" for input bands     */
long offset2[MAXBND+1];         /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */
long bands[MAXIMG][MAXBND + 1];
                                /* band specification for first image        */
double minval[MAXBND];          /* min pixel value array		     */
double maxval[MAXBND];          /* max pixel value array		     */

int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		     */

long kernel;			/* number of samples in the kernel           */
long msgtim;			/* elpased time between display messages     */
long nmrimg;			/* number of input images for image          */
long nonfatal = 1;		/* nonfatal error flag			     */
long nbands[MAXIMG];		/* number of bands per input image           */
long odtype;			/* odtype for processing                     */
long one = 1;			/* number one				     */
long std_out;			/* logical unit # for standard output        */
long thresh;			/* smoothing threshold value		     */
long totbnd;			/* total number of bands in first image      */
long window[MAXIMG][4];		/* window specification for first image      */


static char hosin[MAXIMG][CMLEN];/* buffer storage        	   	     */
static char hosout[1][CMLEN];       /* buffer storage        		     */
char    minmax[4];	         /* minmax flag buffer 			     */
static char taein[MAXIMG][CMLEN];/* buffer storage       		     */
static char taeout[1][CMLEN];       /* buffer storage       		     */


c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/*
       get user-specified parameters and TAE globals
*/
getpar(&vblock, taein, hosin, taeout, hosout,
       &nmrimg, bands, nbands, window, &msgtim, 
       minmax, &totbnd,&odtype,&kernel,&thresh);
/*
   perform the filter 
*/
proces(&vblock,hosin,hosout,nmrimg,bands,nbands,
	window,msgtim,minmax,totbnd,odtype,kernel,thresh); 
/*
   create associated history file
*/
if (c_hstout(&vblock, hosout[0], hosin , &nmrimg, " ") != E_SUCC)
   c_errmsg("Error accessing the history information",
		"deswath-history",&nonfatal);

/*
   inform user of successful completion
*/
c_complt("DESWATH", taeout, &one);

return EXIT_SUCCESS;

}
