/******************************************************************************
NAME:   DIF	

FUNCTION:
	DIF performs comparison between two images or between
	multiple pairs of images.

PROGRAM HISTORY:
  Version	Date       Author      	Request
  -------	----	   ------       -------
    3.0         8/88       R. McKinney   LAS 5.0 development
    5.0        12/88       R. McKinney   Update to print correct filename.
    5.0		9/92	   T. Mittan	 Added extern to curent and curcat
					 to eliminate warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS 5.0

ALGORITHM (FLOW):
        Retrieve TAE input parameters.
 	Do error checking on input image(s) to insure equal window sizes.
	Read the input images
        Compare input images.  
        Output to the desired media.
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

#define HALFIMG (MAXIMG / 2)

/*
   The following variables must be declared global for image I/O routines
*/
struct FDESC *  fdesc1[HALFIMG+1];	
                            /* image file descriptors used for image I/O */
struct FDESC *  fdesc2[HALFIMG+1]; 
                            /* image file descriptors used for image I/O */
struct GDESC *  gdesc1;		/* group descriptor used for image I/O       */
struct GDESC *  gdesc2;		/* group descriptor used for image I/O       */
long offset1[HALFIMG*MAXBND+1];/* offsets into "thebuf" for input bands*/
long offset2[HALFIMG*MAXBND+1];/* offsets into "thebuf" for input bands*/
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

int main
(void)
{
struct PARBLK vblock;			/* TAE parameter block               */

static long bands1[HALFIMG][MAXBND+1];	/* bands to be processed - all images*/
static long bands2[HALFIMG][MAXBND+1];	/* bands to be processed - all images*/
long    fyle;				/* flag for output to file	     */
long    lnpr;				/* flag for output to line printer   */
long	msgtim;				/* elapsed time between display msg  */
long	nbands1[HALFIMG];  		/* number of bands per input image   */
long	nbands2[HALFIMG];  		/* number of bands per input image   */
long    nfiles;				/* number of output files created    */
long	nmrimg1;			/* number of input images            */
long	nmrimg2;			/* number of input images            */
long	std_out;			/* LUN to standard output    	     */
long    term;				/* flag for output to terminal       */
long	totbnd1;			/* total number of bands             */
long	totbnd2;			/* total number of bands             */
long	window1[HALFIMG][4];		/* window for each input image       */
long	window2[HALFIMG][4];		/* window for each input image       */

float   tolval; 			/* tolarence value		     */

char    filnm[1][CMLEN];			/* output file name for FILE option  */
char    listopt[6];			/* type of list to produce           */
char	hosin1[HALFIMG][CMLEN];   	/* host names of input images        */
char	hosin2[HALFIMG][CMLEN];   	/* host names of input images        */
char    outfil[1][CMLEN];			/* output file name passed to complt */
char	taein1[HALFIMG][CMLEN];	        /* TAE names of input images         */
char	taein2[HALFIMG][CMLEN];	        /* TAE names of input images         */
char    typ[9];				/* comparison type option            */

/*
   initialization
*/
c_inlas(&vblock,&std_out);

/*
   get parameters and globals
*/
getpar(&vblock,taein1,hosin1,&nmrimg1,window1,bands1,nbands1,
       &totbnd1,taein2,hosin2,&nmrimg2,window2,bands2,nbands2,
       &totbnd2,&msgtim,&term,&fyle,&lnpr,filnm[0],outfil[0],typ, &tolval,
       listopt);
/*
   process image
*/
proces(&vblock,taein1,hosin1,nmrimg1,window1,bands1,nbands1,totbnd1,
	       taein2,hosin2,nmrimg2,window2,bands2,nbands2,
	       msgtim,term,fyle,lnpr,filnm,&nfiles,typ,tolval,
	       listopt);

/* 
   output completion message 
*/
c_complt("DIFF",outfil,&nfiles);

return EXIT_SUCCESS;

}
