/*******************************************************************************
NAME      		  GETPAR 

PURPOSE
	GETPAR will retrieve and perform the final editing of the 
	parameters for the DIFF module.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.McKinney      May  1988       converted to LAS 5.0
R.McKinney      Dec  1988       Updated to print the correct filename.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   DIFF must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   Retrieve the required TAE parameters from the parameter block. 
   Parse input images  so that the window and band specifications are
   known.
******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block            */
    char taein1[][CMLEN],	/* pointer to input TAE file descriptor      */
    char hosin1[][CMLEN],	/* pointer to input host file descriptor     */
    long *nmrimg1,		/* number of input images                    */
    long (*window1)[4],		/* window specifications for each input image*/
    long (*bands1)[MAXBND+1],	/* band specifications for each input image  */
    long nbands1[],		/* number of bands per input image	     */
    long *totbnd1,		/* total number of bands to be processed     */
    char taein2[][CMLEN],	/* pointer to input TAE file descriptor      */
    char hosin2[][CMLEN],	/* pointer to input host file descriptor     */
    long *nmrimg2,		/* number of input images                    */
    long (*window2)[4],		/* window specifications for each input image*/
    long (*bands2)[MAXBND+1],	/* band specifications for each input image  */
    long nbands2[],		/* number of bands per input image	     */
    long *totbnd2,		/* total number of bands to be processed     */
    long *msgtim,		/* elapsed time between display messages     */
    long *term,			/* flag for output to terminal 		     */
    long *fyle,			/* flag for output to file		     */
    long *lnpr,			/* flag for output to line printer	     */
    char *filnm,			/* output filename for FILE option           */
    char *outfil, 		/* pointer to output host file name          */
    char *typ,			/* type of comparison			     */
    float *tolval,		/* tolarence value			     */
    char *listopt		/* type of listing to produce		     */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long    element = 0;		/* element number for getin                  */
long    fatal = -1;		/* fatal error flag			     */
long    i;			/* counter				     */
long	maximg;			/* maximum number of images to be processed  */
long    nonfatal = 1;		/* fatal error flag			     */
long	qflag[MAXIMG][2];	/* flag indicating if subwindows specified   */
long	same;			/* flag indicating if images have same window*/
long	status;			/* function return status code               */

maximg = MAXIMG / 2;

/*
  get first image specification
*/
element = 0;
status = c_getin( vblock, "IN", &element, taein1, hosin1, nmrimg1, window1, 
               bands1, nbands1, totbnd1, &maximg, &same, qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);

/*
  get second image specification
*/
element = 1;
status = c_getin( vblock, "IN", &element, taein2, hosin2, nmrimg2, window2, 
                bands2,nbands2, totbnd2, &maximg, &same, qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);
if (*nmrimg1 != *nmrimg2)
   {
   c_errmsg("Different number of images specified",
           "diff-images",&nonfatal);
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   }

for (i=0; i<*nmrimg1; i++)
  {
if (window1[i][2] != window2[i][2])
   {
   c_errmsg("Images do not have same number of lines",
           "diff-lines",&nonfatal);
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   }
if (window1[i][3] != window2[i][3])
   {
   c_errmsg("Images do not have same number of samples",
           "diff-samples",&nonfatal);
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   }
if (nbands1[i] != nbands2[i])
   {
   c_errmsg("Images do not have same number of bands",
           "diff-bands",&nonfatal);
   c_errmsg("Number of bands reset to minimum","diff-bands",&nonfatal);
   }
  }
/*
   Retrieve the type option
*/
   val = p_find(vblock,"compopt");
   strcpy(typ,SVAL(*val,0)); 
/*
   Retrieve the tolarence value
*/
   val = p_find(vblock,"tolval");
   *tolval = RVAL(*val,0);
/*
   Retrieve the print option
*/
  strcpy(filnm,"diff");
  status = c_setprt(vblock,term,fyle,lnpr,filnm,outfil);
  if (status != E_SUCC)
     c_errmsg("Fatal error encountered","diff-fatal",&fatal);

/*
   Retrieve the type of list to produce
*/
   val = p_find(vblock,"listopt");
   strcpy(listopt,SVAL(*val,0)); 
/*
   Retrieve TAE globals
*/

val = p_find(vblock,"$msgtim");	
*msgtim = IVAL(*val,0);

return;
}
