/*******************************************************************************
NAME      		   PROCES


PURPOSE  PROCES opens the input images for read access and generates a  
	 report of statistics.  

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.McKinney       Aug  1988       converted to LAS 5.0
R.McKinney	 Dec  1988	 Updated to print the correct filename.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   DIFF must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   Open the input images for read access 
   Generate a report showing the differences.
******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include <math.h>
#include "imageio.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC *  fdesc1[]; /* image file descriptors used for image I/O */
extern struct FDESC *  fdesc2[]; /* image file descriptors used for image I/O */
extern struct GDESC *  gdesc1;	/* group descriptor used for image I/O       */
extern struct GDESC *  gdesc2;	/* group descriptor used for image I/O       */
extern long offset1[];          /* offsets into "thebuf" for input bands     */
extern long offset2[];          /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

void proces
(
    struct PARBLK *vblock,	/* TAE parameter block			     */
    char taein1[][CMLEN],       /* TAE file name                             */
    char hosin1[][CMLEN],	/* pointer to input host file descriptor     */
    long nmrimg1,		/* number of input images                    */
    long (*window1)[4],		/* window specification of input images      */
    long (*bands1)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands1,		/* number of bands per input image           */
    long totbnd1,		/* total number of bands to be processed     */
    char taein2[][CMLEN],        /* TAE file name                             */
    char hosin2[][CMLEN],	/* pointer to input host file descriptor     */
    long nmrimg2,		/* number of input images                    */
    long (*window2)[4],		/* window specification of input images      */
    long (*bands2)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands2,		/* number of bands per input image           */
    long msgtim,			/* elapsed time between display messages     */
    long term,			/* flag for output to terminal               */
    long fyle,			/* flag for output to file                   */
    long lnpr,			/* flag for output to line printer           */
    char filnm[][CMLEN],       /* output filename for FILE option           */
    long *nfiles,  		/* number of output files		     */
    char *typ,			/* type of differencing   		     */
    double tolval,      /* tolerance value                           */
    char *listopt		/* type of listing to produce		     */
) 
{

long    access;			/* access mode				     */
long	band;			/* loop index counting the bands             */
long    bandperimg;		/* bands per input image		     */
long    bufsiz;			/* size of image I/O buffer		     */
long    bufsiz2;		/* size of image I/O buffer		     */
long    curlin;			/* current line indicator		     */
long    dtype1;			/* data type of first input image	     */
long    dtype2;			/* data type of second input image	     */
long    fatal = -1;		/* fatal error flag			     */
long    gr[MAXBND];		/* array to keep greater than numbers        */
long    greater;                /* number greater than first image           */
long	image;			/* loop index counting input images          */
long    le[MAXBND];		/* array to keep less than numbers           */
long    less;                   /* number less than first image              */
long    lineflg;                /* new line flag                             */
long    list;			/* list option				     */
long    msg_flag = ALLCAT;      /* flag for processing message		     */
long    nl;			/* number of lines			     */
long    nonfatal = 0;		/* nonfatal error flag			     */
long    ns;			/* number of samples			     */
long    one = 1;		/* one					     */
long    sa[MAXBND];		/* array to keep equal numbers		     */
long    same;                   /* number of equal values		     */
long    sample;			/* sample indicator			     */
long    size1 = 0;			/* size of first image			     */
long    size2 = 0;			/* size of second image			     */
long    sl;			/* starting line			     */
long    ss;			/* starting sample			     */
long    status;			/* status flag				     */
long    tipe; 			/* type of option			     */
long    whichimg;		/* the image to print			     */
long	zero = 0;		/* value of zero                             */

float   ddiff;			/* variable to save the difference           */
float   meanab[MAXBND];		/* mean above reference image                */
float   meanbe[MAXBND];		/* mean below reference image                */
float   pixel1 = 0.0;			/* pixel value of first image		     */
float   pixel2 = 0.0;			/* pixel value of second image		     */
float   varab[MAXBND];		/* variance above reference image            */
float   varbe[MAXBND];          /* variance below reference image            */

char    outmsg[CMLEN];		/* output message to the terminal	     */
unsigned char    *thebuf1;	/* image I/O buffer			     */
unsigned char    *thebuf2;	/* image I/O buffer 			     */

FILE    *fp = NULL;			/* file pointer to print file                */

if (strcmp(listopt,"BRIEF") == 0)
	list = 0;
else
	list = 1;

*nfiles = 0;
if (fyle)
    *nfiles = 1;
if ((lnpr) || (fyle))
     fp = fopen(filnm[0],"w");

if (strcmp(typ,"SIMPLE") == 0)
	tipe = 1;
else
 	tipe = 0;

access = IREAD;

for (band=0,image=0;image<nmrimg1;image++)
   {
   sl = window1[image][0];
   ss = window1[image][1];
   nl = window1[image][2];
   ns = window1[image][3];
   dtype1 = 0;
   status = c_eopens(&fdesc1[image],hosin1[image],bands1[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&dtype1,&zero,&zero);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   band += nbands1[image];
   }
   fdesc1[nmrimg1] = 0;
for (band=0,image=0;image<nmrimg2;image++)
   {
   sl = window2[image][0];
   ss = window2[image][1];
   nl = window2[image][2];
   ns = window2[image][3];
   dtype2 = 0;
   status = c_eopens(&fdesc2[image],hosin2[image],bands2[image],&sl,&ss,&nl,&ns,
                   &offset2[band],&access,&dtype2,&zero,&zero);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   band += nbands2[image];
   }
fdesc2[nmrimg2] = 0;

/* Allocate I/O buffer
----------------------*/
if ((thebuf1 = (unsigned char *) c_ealloc(fdesc1,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","diff-alloc",&nonfatal);
   if ((lnpr) || (fyle))
     {
     fclose(fp);
     c_clean(vblock,"diff",filnm,&one);
     }
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   }
if ((thebuf2 = (unsigned char *) c_ealloc(fdesc2,&bufsiz2)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","diff-alloc",&nonfatal);
   if ((lnpr) || (fyle))
     {
     fclose(fp);
     c_clean(vblock,"diff",filnm,&one);
     }
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc1,&gdesc1,thebuf1,&bufsiz) != E_SUCC)
   {
   if ((lnpr) || (fyle))
     {
     fclose(fp);
     c_clean(vblock,"diff",filnm,&one);
     }
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   }
if (c_egroup(fdesc2,&gdesc2,thebuf2,&bufsiz2) != E_SUCC)
   {
   if ((lnpr) || (fyle))
     {
     fclose(fp);
     c_clean(vblock,"diff",filnm,&one);
     }
   c_errmsg("Fatal error encountered","diff-fatal",&fatal);
   }
/*
   determine the size
*/
switch (dtype1)
   {
   case EBYTE: size1 = 1; break;
   case EWORD: size1 = 2; break;
   case ELONG: size1 = 4; break;
   case EREAL: size1 = 4; break;
   }
switch (dtype2)
   {
   case EBYTE: size2 = 1; break;
   case EWORD: size2 = 2; break;
   case ELONG: size2 = 4; break;
   case EREAL: size2 = 4; break;
   }
/* Setup for processing message utility
---------------------------------------*/
if (list != 1)
 {
 c_inittm("DIFF","Processed","line","bands"," ",&nl,&totbnd1,&msgtim,&msg_flag);
 curent = 1;    
 curcat = 1;
 c_startm();
 }

/* Main processing loop
-----------------------*/
for (band=0;band<totbnd1;band++)
{
   gr[band] = le[band] = sa[band] = 0;
   meanab[band] = meanbe[band] = 0;
   varab[band] = varbe[band] = 0;
}
for (curlin = 0;curlin < nl;curlin ++)
   {
   if (c_estep(&gdesc1) != E_SUCC)
     {
     if ((lnpr) || (fyle))
       {
       fclose(fp);
       c_clean(vblock,"diff",filnm,&one);
       }
     c_errmsg("Fatal error encountered","diff-fatal",&fatal);
     }
   if (c_estep(&gdesc2) != E_SUCC)
     {
     if ((lnpr) || (fyle))
       {
       fclose(fp);
       c_clean(vblock,"diff",filnm,&one);
       }
     c_errmsg("Fatal error encountered","diff-fatal",&fatal);
     }
    whichimg = 0;
    bandperimg = totbnd1 / nmrimg1;
   for (band=0;band<totbnd1;band++)
      {
      lineflg = 1;
      if (band == bandperimg)
         {
         bandperimg += totbnd1 / nmrimg1;
         whichimg += 1;
         }
      same = less = greater = 0;
      for (sample=0;sample<ns;sample++)
	{
	switch (dtype1)
	{
	case EBYTE:
	pixel1 = *(thebuf1 + (*(offset1+band) + sample) * size1);
	break;
	case EWORD:
	pixel1 = (*(short *)(thebuf1 + (*(offset1+band) + sample) * size1));
	break;
	case ELONG:
	pixel1 = (float) (*(long *)(thebuf1 + (*(offset1+band) + sample) * size1));
	break;
	case EREAL:
	pixel1 = (*(float *)(thebuf1 + (*(offset1+band) + sample) * size1));
	break;
	}
	switch (dtype2)
	{
	case EBYTE:
	pixel2 = *(thebuf2 + (*(offset2+band) + sample) * size2);
	break;
	case EWORD:
	pixel2 = (*(short *)(thebuf2 + (*(offset2+band) + sample) * size2));
	break;
	case ELONG:
	pixel2 = (float) (*(long *)(thebuf2 + (*(offset2+band) + sample) * size2));
	break;
	case EREAL:
	pixel2 = (*(float *)(thebuf2 + (*(offset2+band) + sample) * size2));
	break;
	}
/*
  test based upon SIMPLE option
*/
      if (tipe)
      {
	ddiff = pixel1 - pixel2;
	if (pixel1 < pixel2) 
	    {
	    if ((fabs(pixel2 - pixel1)) > tolval) 
		{
	        greater += 1;
		gr[band] += 1;
		meanab[band] -= ddiff;
		varab[band] += (ddiff*ddiff);
            if ((list) && (lineflg))
             {
             if (term)
              {
              if (band == 0 && curlin == 0)
                sprintf(outmsg,"Reference Image: %s",taein1[whichimg]);
              else
                sprintf(outmsg,"\fReference Image: %s",taein1[whichimg]);
              c_pterm(outmsg,&zero);
              sprintf(outmsg,"Compared Image: %s",taein2[whichimg]);
              c_pterm(outmsg,&zero);
              sprintf(outmsg,"Band Number: %d",(band+1));
              c_pterm(outmsg,&zero);
              }
              if ((lnpr) || (fyle))
               {
               fprintf(fp,"\fReference Image: %s\n",taein1[whichimg]);
               fprintf(fp,"Compared Image: %s\n",taein2[whichimg]);
               fprintf(fp,"Band Number: %d\n",(band+1));
               }
             }
                if ((term) && (list))
                  {
                  sprintf(outmsg,"  Line: %d  Sample: %d ",(curlin+1),
			  (sample+1));
        	  c_pterm(outmsg,&zero);
                  sprintf(outmsg,
                  " First value: %f Second value: %f",pixel1,pixel2);
        	  c_pterm(outmsg,&zero);
		  }
                  if (((lnpr) || (fyle)) && (list))
                  {
                  fprintf(fp,"  Line: %d  Sample: %d\n",(curlin+1),
				(sample+1)); 
                  fprintf(fp,"  First value: %f Second value: %f\n",
		  pixel1,pixel2);
	          }
		}
	        else
		{
		same += 1;
		sa[band] += 1;
		}
            lineflg = 0;
	    }
	else
	if (pixel1 > pixel2) 
	    {
	    if ((fabs(pixel1 - pixel2)) > tolval)
		{
	        less += 1;
		le[band] += 1;
		meanbe[band] += ddiff;
		varbe[band] += (ddiff*ddiff);
              if ((list) && (lineflg))
               {
               if (term)
                {
                if (band == 0 && curlin == 0)
                  sprintf(outmsg,"Reference Image: %s",taein1[whichimg]);
                else
                  sprintf(outmsg,"\fReference Image: %s",taein1[whichimg]);
                c_pterm(outmsg,&zero);
                sprintf(outmsg,"Compared Image: %s",taein2[whichimg]);
                c_pterm(outmsg,&zero);
                sprintf(outmsg,"Band Number: %d",(band+1));
                c_pterm(outmsg,&zero);
                }
              if ((lnpr) || (fyle))
               {
               fprintf(fp,"\fReference Image: %s\n",taein1[whichimg]);
               fprintf(fp,"Compared Image: %s\n",taein2[whichimg]);
               fprintf(fp,"Band Number: %d\n",(band+1));
               }
             }
                if ((term) && (list))
                  {
                  sprintf(outmsg,"  Line: %d  Sample: %d ",(curlin+1),
			  (sample+1));
        	  c_pterm(outmsg,&zero);
                  sprintf(outmsg,
                  " First value: %f Second value: %f",pixel1,pixel2);
        	  c_pterm(outmsg,&zero);
		  }
                  if (((lnpr) || (fyle)) && (list))
                  {
                  fprintf(fp,"  Line: %d  Sample: %d\n",(curlin+1),
			  (sample+1)); 
                  fprintf(fp,"  First value: %f Second value: %f\n",
		  pixel1,pixel2);
	          }
		}
	        else
		{
		same += 1;
		sa[band] += 1;
		}
             lineflg = 0;
	    }
	else
	    {
	    same += 1;
	    sa[band] += 1;
            }
	}
/*
  else test it based upon RELATIVE option
*/
  else
      if (tipe == 0)
      {
	ddiff = pixel1 - pixel2;
	if (pixel1 == 0)
	    pixel1 = 1.0;
	ddiff /= pixel1;
	if (pixel1 < pixel2) 
	    {
	    if ((fabs((pixel2 - pixel1)/pixel1)) > tolval) 
		{
	        greater += 1;
		gr[band] += 1;
		meanab[band] -= ddiff;
		varab[band] += (ddiff*ddiff);
              if ((list) && (lineflg))
               {
               if (term)
                {
                if (band == 0 && curlin == 0)
                  sprintf(outmsg,"Reference Image: %s",taein1[whichimg]);
                else
                  sprintf(outmsg,"\fReference Image: %s",taein1[whichimg]);
                c_pterm(outmsg,&zero);
                sprintf(outmsg,"Compared Image: %s",taein2[whichimg]);
                c_pterm(outmsg,&zero);
                sprintf(outmsg,"Band Number: %d",(band+1));
                c_pterm(outmsg,&zero);
                }
              if ((lnpr) || (fyle))
               {
               fprintf(fp,"\fReference Image: %s\n",taein1[whichimg]);
               fprintf(fp,"Compared Image: %s\n",taein2[whichimg]);
               fprintf(fp,"Band Number: %d\n",(band+1));
               }
             }
                if ((term) && (list))
                  {
                  sprintf(outmsg,"  Line: %d  Sample: %d ",(curlin+1),
			  (sample+1));
        	  c_pterm(outmsg,&zero);
                  sprintf(outmsg,
                  " First value: %f Second value: %f",pixel1,pixel2);
        	  c_pterm(outmsg,&zero);
		  }
                  if (((lnpr) || (fyle)) && (list))
                  {
                  fprintf(fp,"  Line: %d  Sample: %d\n",(curlin+1),
				(sample+1)); 
                  fprintf(fp,"  First value: %f Second value: %f\n",
		  pixel1,pixel2);
	          }
		}
	        else
		{
		same += 1;
		sa[band] += 1;
		}
             lineflg = 0;
	    }
	else
	if (pixel1 > pixel2) 
	    {
	    if ((fabs((pixel1 - pixel2)/pixel1)) > tolval)
		{
	        less += 1;
		le[band] += 1;
		meanbe[band] += ddiff;
		varbe[band] += (ddiff*ddiff);
              if ((list) && (lineflg))
               {
               if (term)
                {
                if (band == 0 && curlin == 0)
                  sprintf(outmsg,"Reference Image: %s",taein1[whichimg]);
                else
                  sprintf(outmsg,"\fReference Image: %s",taein1[whichimg]);
                c_pterm(outmsg,&zero);
                sprintf(outmsg,"Compared Image: %s",taein2[whichimg]);
                c_pterm(outmsg,&zero);
                sprintf(outmsg,"Band Number: %d",(band+1));
                c_pterm(outmsg,&zero);
                }
              if ((lnpr) || (fyle))
               {
               fprintf(fp,"\fReference Image: %s\n",taein1[whichimg]);
               fprintf(fp,"Compared Image: %s\n",taein2[whichimg]);
               fprintf(fp,"Band Number: %d\n",(band+1));
               }
             }
                if ((term) && (list))
                  {
                  sprintf(outmsg,"  Line: %d  Sample: %d ",(curlin+1),
			  (sample+1));
        	  c_pterm(outmsg,&zero);
                  sprintf(outmsg,
                  " First value: %f Second value: %f",pixel1,pixel2);
        	  c_pterm(outmsg,&zero);
		  }
                  if (((lnpr) || (fyle)) && (list))
                  {
                  fprintf(fp,"  Line: %d  Sample: %d\n",(curlin+1),
			  (sample+1)); 
                  fprintf(fp,"  First value: %f Second value: %f\n",
		  pixel1,pixel2);
	          }
		}
	        else
		{
		same += 1;
		sa[band] += 1;
	   	}
             lineflg = 0;
	    }
	else
	    {
	    same += 1;
	    sa[band] += 1;
	    }
    }    /* end of RELATIVE loop   */
   }    /* end of sample loop     */
/*
  write difference values to term
*/
 if (same != ns)
   {
   if ((term) && (list))
    {
        sprintf(outmsg,
        "  Less than: %d  Equal to: %d  Greater than: %d",
        less,same,greater);
	c_pterm(outmsg,&zero);
    }
/*
  write difference values to file or printer
*/
   if (((lnpr) || (fyle)) && (list))
    {
	fprintf(fp,
        "  Less than: %d  Equal to: %d  Greater than: %d\n",
        less,same,greater);
    }
   }
  }      /* end of band loop       */
   curent++;
   }    /*  end of curlin loop   */ 

/* Turn off display messages
----------------------------*/
if (list != 1)
   c_stoptm();

/*
  write difference values to term
*/
whichimg = 0;
bandperimg = totbnd1 / nmrimg1;
for (band=0;band<totbnd1;band++)
{
if (band == bandperimg)
   {
   bandperimg += totbnd1 / nmrimg1;
   whichimg += 1;
   }
 switch (gr[band])
    {
    case 0:
      varab[band] = 0;
      meanab[band] = 0;
    break; 
    case 1: 
      varab[band] = 0;
    break;
    default: 
      ddiff=(float)(gr[band]);
      varab[band] = (varab[band] - (meanab[band] * meanab[band]) / ddiff)
			/ (ddiff - 1.0);
      if (varab[band] < 0.0)
 	  varab[band] = 0.0;
      if (varab[band] >= 0.0) 
	  varab[band] = sqrt(varab[band]);
      meanab[band] /= ddiff;
    break;
    } 			 

 switch (le[band])
    {
    case 0:
      varbe[band] = 0;
      meanbe[band] = 0;
    break; 
    case 1: 
      varbe[band] = 0;
    break;
    default: 
      ddiff=(float)(le[band]);
      varbe[band] = (varbe[band] - (meanbe[band] * meanbe[band]) / ddiff)
			/ (ddiff - 1.0);
      if (varbe[band] < 0.0)
 	  varbe[band] = 0.0;
      if (varbe[band] >= 0.0) 
	  varbe[band] = sqrt(varbe[band]);
      meanbe[band] /= ddiff;
   break;
   } 			 

 if (term)
    {
        sprintf(outmsg,"\n");
        c_pterm(outmsg,&zero);
	if (band == 0)
          sprintf(outmsg,"Reference Image: %s\n",taein1[whichimg]);
        else
          sprintf(outmsg,"\fReference Image: %s\n",taein1[whichimg]);
        c_pterm(outmsg,&zero);
        sprintf(outmsg,"Compared Image: %s\n",taein2[whichimg]);
        c_pterm(outmsg,&zero);
        sprintf(outmsg,"Band Number: %d\n",(band+1));
        c_pterm(outmsg,&zero);
	sprintf(outmsg,
	"                 NUMBER OF POINTS AND THEIR DIFFERENCES\n");
	c_pterm(outmsg,&zero);
	sprintf(outmsg,
	"                       COMPARED AGAINST REFERENCE\n");
	c_pterm(outmsg,&zero);
        sprintf(outmsg,
	"                    BELOW      EQUAL TO        ABOVE");
	c_pterm(outmsg,&zero);
	sprintf(outmsg,
	"                    -----      --------        -----");
 	c_pterm(outmsg,&zero);
	sprintf(outmsg,
	"               %10d    %10d   %10d",
	le[band],sa[band],gr[band]);
 	c_pterm(outmsg,&zero);
	sprintf(outmsg,
	"                %f       MEAN       %f",
	meanbe[band],meanab[band]);
	c_pterm(outmsg,&zero);
	sprintf(outmsg,
	"                %f      ST.DEV.     %f\n",
	varbe[band],varab[band]);
	c_pterm(outmsg,&zero);
    }
/*
  write difference values to file or printer
*/
 if ((lnpr) || (fyle))
    {
        fprintf(fp,"Reference Image: %s\n",taein1[whichimg]);
        fprintf(fp,"Compared Image: %s\n",taein2[whichimg]);
        fprintf(fp,"Band Number: %d\n",(band+1));
	fprintf(fp,
	"                 NUMBER OF POINTS AND THEIR DIFFERENCES\n");
	fprintf(fp,
	"                       COMPARED AGAINST REFERENCE\n");
        fprintf(fp,
	"                    BELOW      EQUAL TO        ABOVE\n");
	fprintf(fp,
	"                    -----      --------        -----\n");
	fprintf(fp,
	"               %10d    %10d   %10d\n",
	le[band],sa[band],gr[band]);
	fprintf(fp,
	"                %f       MEAN       %f\n",
	meanbe[band],meanab[band]);
	fprintf(fp,
	"                %f      ST.DEV.     %f\n",
	varbe[band],varab[band]);
    }
 }  /* end the band loop  */

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc1) != E_SUCC)
   c_errmsg("Nonfatal error encountered","diff-warn",&one);
if (c_egclse(&gdesc2) != E_SUCC)
   c_errmsg("Nonfatal error encountered","diff-warn",&one);

/* Free buffer
--------------*/
free(thebuf1);
free(thebuf2);

  if ((lnpr) || (fyle))
  fclose(fp);
return;
}
