/*****************************************************************************
NAME:  diffddr

PURPOSE:
  This module contains the "main" function.

DEVELOPMENT HISTORY:
VERSION      DATE    AUTHOR         CODE/CONT     REASON
---------------------------------------------------------
 1.0         7/00    B. Misterek      CSB         New Development


******************************************************************************/

#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "diffddr.h"


int main(void)
{

    struct PARBLK vblock;            /* TAE parameter block                   */
    
    char infiles[MAXIMG][CMLEN];     /* Input file names                      */
    char progname[CMLEN] = "diffddr";         
    char prtfile[CMLEN];             /* Name of file to output to             */
    char tmpstr[1][CMLEN];           /* Temp string to be used to call c_clean*/
    char *tmpstr2[CMLEN];            /* Temp string to be used with q_string  */
    
    int  bandddr;                    /* Flag to indicate if bands are to 
                                         be compared */
    int  delflg;                     /* Flag to hold the value of the TAE GLOBAL
                                         DELFG  */
    int  i;                          /* General looping variable              */
    long nfiles;                     /* Number of files specified             */
    int  print[3];                   /* Print array to specify where to output*/
   
    long one = 1;                    /* Constant of one                       */
    long std_out;                    /* standard output logical unit number   */


    /*  Initialize TAE processing.
    ------------------------------*/
    c_inlas(&vblock, &std_out);

    /*  Call getpar to get variables from the TAE parameter block.
    --------------------------------------------------------------*/
    if( getpar(&vblock, infiles, &nfiles, &bandddr, print, prtfile, 
               &delflg) != E_SUCC )
    {
        c_lasmsg(TRUE, "diffddr-getpar", "Error reading input parameters");
    }

    /*  Call open_output to open any output files.
    ----------------------------------------------*/
    if( open_output(print, prtfile) != E_SUCC )
    {
        c_lasmsg(TRUE, "diffddr-open","Error opening output file");
    }

    /*  Compare each file from the second to the last with the first.
    -----------------------------------------------------------------*/
    for( i = 1 ; i < nfiles ; i++)
    {
        /*  Set the correct input file names.
        -------------------------------------*/
        update_output(infiles, 0, i);
        
        /*  Call compddr to compare the DDR files.
        -----------------------------------------*/
        if( compddr(infiles, 0, i, bandddr) != E_SUCC)
        {
            /*  If there was a failure, and delflg is set, then
                clean up all files.
            ---------------------------------------------------*/
            close_output();
            if(delflg)
            {
              strcpy(tmpstr[0], prtfile);
              c_clean(&vblock, progname, tmpstr, &one);
            }  
            c_lasmsg(TRUE, "diffddr-compddr", "Error comparing files"); 

        }
    
    }


    if( close_output() != E_SUCC )
    {
        c_lasmsg(TRUE,"diffddr-close","Error occured when closing %s",
                 prtfile);
    }

    /*  Check to see if there were any differences in the files, and
        place either 'YES' or 'NO' in IDENTFLG.
    ----------------------------------------------------------------*/
    if( is_different() == 0)
    {
       tmpstr2[0] = "YES";
    }
    else
    {
       tmpstr2[0] = "NO";
    }

    if(q_string(&vblock, "IDENTFLG", 1, tmpstr2, P_UPDATE) != SUCCESS)
    {
        c_lasmsg(TRUE, "diffddr-q_string", "Error returning IDENTFLG");
    }

    /*  Return the TAE Parameter block.
    -----------------------------------*/
    if(q_out(&vblock) != SUCCESS)
    {
        c_lasmsg(TRUE,"diffddr-q_out","Error returning parameter block");
    }

return EXIT_SUCCESS;
}


