#ifndef DIFFDDR_H
#define DIFFDDR_H

/* TERM, PFILE, and LP are used as references into an array which 
   holds the options that the user may have selected.
----------------------------------------------------------------*/
#define TERM 0                  
#define PFILE 1
#define LP 2

#define TOLERANCE .00001 /* This is the tolerance that will be used when
                            comparing float values in the DDR */

#define DIFF_MSG 1     /* Used with the output function */
#define STATUS_MSG 0  


/* Prototypes
-------------*/

int open_output
(
    int print[],        /* I: Array specifing where to output  */
    char outfile[]      /* I: Output file to open if specified */
);

int is_different(void);

int get_filediff(void);

void update_output
(
    char input_files[][CMLEN],/* I: Array of input filenames */
    int  first,               /* I: Index of first input file in the array */
    int  second               /* I: Index of second input file in the arrray */
);

int output
(
    char *output_string,     /* I: String to be output  */
    int   difference_flag    /* I: Flag to indicate if this is a difference
                                   message. Valid values are DIFF_MSG and
                                   STATUS_MSG*/
);

int close_output(void);

int compband
(
    char input_files[][CMLEN], /* I: Array of input filenames                 */
    int  first,                /* I: Index of first input file in the array   */
    int  second,               /* I: Index of second input file in the array  */
    int  nbands_one,           /* I: Number of bands in the first input image */
    int  nbands_two            /* I: Number of bands in the second input image*/
);

int compddr
(
    char input_files[][CMLEN], /* I: Array of input filenames */
    int  first,                /* I: Index of first input filename in the 
                                     array */
    int  second,               /* I: Index of second input filename in the
                                     array */
    int  bandddr               /* I: Flag to indicate whether to compare bands
                                     1 = YES, 0 = NO  */
);

int getpar
(
    struct PARBLK *vblock,       /* I: TAE Paremeter Block  */
    char infiles[][CMLEN],       /* O: Input filenames      */
    long *nfiles,                /* O: Number of files specified */
    int  *bandddr,               /* O: Flag to indicate if bands are to
                                       be compared */
    int  print[],                /* O: Print array to indicate where to output*/
    char prtfile[],              /* O: Name of file to output to  */
    int  *delflg                 /* O: Gets the value of TAE GLOBAL DELFG 
                                       1 = YES, 0 = NO  */
);


#endif

