/*****************************************************************************
NAME:  getpar

PURPOSE:
  The purpose of this function is to retrieve the parameters from the TAE
  parameter block.

DEVELOPMENT HISTORY:
VERSION      DATE    AUTHOR         CODE/CONT     REASON
---------------------------------------------------------
 1.0         7/00    B. Misterek      CSB         New Development


RETURN VALUE:
  int, status of the fuction(E_SUCC, E_FAIL)

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "diffddr.h"

int getpar
(
    struct PARBLK *vblock,       /* I: TAE Paremeter Block  */
    char infiles[][CMLEN],       /* O: Input filenames      */
    long *nfiles,                /* O: Number of files specified */
    int  *bandddr,               /* O: Flag to indicate if bands are to 
                                      be compared */
    int  print[],                /* O: Print array to indicate where to output*/
    char prtfile[],              /* O: Name of file to output to  */
    int  *delflg                 /* O: Gets the value of TAE GLOBAL DELFG */    
)
{
    struct VARIABLE *var;         /* Pointer to a parameter in vblk */
   
    char tmpstr[CMLEN];           /* Temp string for general use    */
    
    long bands[MAXIMG][MAXBND+1]; /* Array to hold the bands in 
                                     each input image  */
    long i_lim = MAXIMG;          /* Constant of MAXIMG  */
    long nbands[MAXIMG];          /* Number of bands in each image */
    long qflag[MAXIMG][2];        /* Array to pass to c_getin      */
    long same;                    /* Flag to indicate if input images have the
                                     same window size */
    long tot_bands;               /* Integer to hold the total number of bands*/
    long window[MAXIMG][4];       /* Array to hold the windows */
    char taenames[MAXIMG][CMLEN]; /* Array to hold the TAE filenames */
    
    long i = 0;   /* Loop variable */
    int count; /* Count of images */
    long nimg = 0;
    long totimg = 0;

    if(c_getcnt(vblock, "IN", &count) != E_SUCC)
    {
        c_lasmsg(FALSE, "diffddr-getpar", "Error reading parameter IN-count");
        return E_FAIL;
    }

    /*  Call c_getin to retrieve the input images. 
    ----------------------------------------------*/
    for(i = 0 ; i < count ; i++)
    {
        if(c_getin(vblock, "IN", &i, &taenames[totimg], &infiles[totimg], &nimg,
                   &window[totimg], &bands[totimg] , &nbands[totimg], 
                   &tot_bands, &i_lim, &same, qflag) != E_SUCC)
        {
            c_lasmsg(FALSE, "diffddr-getpar","Error reading parameter IN");
            return E_FAIL;
        }

        totimg += nimg;
    }

    *nfiles = totimg;
    
    /*  Use p_find to retrieve the BANDDDR parameter from the TAE block. 
    --------------------------------------------------------------------*/
    var = p_find(vblock, "BANDDDR");
    if( !strcmp("YES", SVAL(*var, 0)) )
        *bandddr = 1;
    else
        *bandddr = 0;
    
    /*  Call c_setprt to retrieve the user's output choices and setup the
        output print file.
    ---------------------------------------------------------------------*/
    strcpy(prtfile, "diffddr");
    if( c_setprt(vblock, &print[TERM], &print[PFILE],
                 &print[LP], prtfile, tmpstr) != E_SUCC)
    {
        c_lasmsg(FALSE, "diffddr-setprt", 
                        "Error initializing print parameters");
        return E_FAIL;
    }


    /*  Retrieve the TAE GLOBAL variable DELFLG and set delflg.
    -----------------------------------------------------------*/
    var = p_find(vblock, "$DELFLG");
    *delflg = IVAL(*var,0);

    
    return E_SUCC;
    
}





