#include <string.h>
#include "diverge.h"
#include "lasmath.h"
#include "protos.h"

void calc
(
    long *numsets,	/* no. feature sets		*/
    long nclass,		/* no. classes			*/
    double *covar[],	/* covariance matrices		*/
    float *mean[],	/* mean vectors			*/
    float *divg,		/* divergences			*/
    char dvgtype[]	/* type of divergence		*/
)

/*
**********************************************************************

NAME: CALC

FUNCTION: Computes divergences

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	        9-Nov-82 C. Gurney
   2            9-Jan-86 B. Kovalick
   3           18-Jul-89 F. Irani/
                         K. Narayanan     Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   See PDF/User's Guide for the detailed algorithm description

**********************************************************************
*/
{
float	avg, det[2],
	dv, tmmn, trmc, y;

long	clpr, diag, fset, inc,
	kk, msize, row;
long	one = 1;

double	*c1, *c2, *ci1, *ci2;

float	*ciadd, *cisub, *csub, *cms,
	*mc, *mmean, *msub;

/*float	*c1, *c2, *ciadd, *cisub, *ci1, *ci2, *csub, *cms,
	*mc, *mmean, *msub;*/

/* 
Allocate memory for work areas. 
--------------------------------------------------------------- 	*/
msize = (*numsets) * (*numsets + 1) / 2 ;

mmean = (float *) malloc((*numsets) * (*numsets) * sizeof(float));
   c1 = (double *) malloc(msize * sizeof(double));
   c2 = (double *) malloc(msize * sizeof(double));
  ci1 = (double *) malloc(msize * sizeof(double));
  ci2 = (double *) malloc(msize * sizeof(double));
 csub = (float *) malloc(msize * sizeof(float));
ciadd = (float *) malloc(msize * sizeof(float));
cisub = (float *) malloc(msize * sizeof(float));
 msub = (float *) malloc((*numsets) * sizeof(float));
  cms = (float *) malloc((*numsets) * sizeof(float));
   mc = (float *) malloc((*numsets) * (*numsets) * sizeof(float));

kk = 0;
avg = 0.;

/*
Do once for each feature set: (e.g. group of bands).
----------------------------------------------------		*/
for (fset = 0; fset < nclass-1; fset++)
{

   /*
   Do for each pair of classes:
   ----------------------------					*/
   for (clpr = fset+1; clpr < nclass; clpr++)
   {
     /* 
     Transfer covariance data to class 1 array
     -----------------------------------------                  */
     for (inc = 0; inc < msize; inc++)
  	  c1[inc] = covar[fset][inc];
     /* 
     Transfer data to class 2 array.
     -------------------------------				*/
      for (inc = 0; inc < msize; inc++)
	    c2[inc] = covar[clpr][inc];

/*
      sprintf (buf,
           "Computing divergence between class %d and class %d",
	    fset+1, clpr+1);
      t_write (buf, T_PROMPT);
*/

      /*
      Subtract c2 from c1.
      --------------------					*/
      for (inc = 0; inc < msize; inc++)
            csub[inc] = c1[inc] - c2[inc];

      /*
      Subtract m1-m2.
      ---------------                            		*/
      for (inc=0; inc < *numsets; inc++) 
       	msub[inc] = mean[fset][inc] - mean[clpr][inc];

      /*
      Invert c1 -> ci1. 
      --------------------						*/
      linv2p (c1, numsets, ci1, det);

      /*
      Invert c2 -> ci2.     
      -----------------						*/
      linv2p (c2, numsets, ci2, det);

      /*
      Subtract: ci2-ci1.
      ------------------                          		*/
      for (inc=0; inc < msize; inc++)
            cisub[inc] = ci2[inc] - ci1[inc];

      /*
      Multiply mc = csub x cisub. 
      --------------------------- 				*/
      vmulss(csub, cisub, numsets, mc, numsets);

      /*
      Add ci1 + ci2.
      --------------						*/
      for (inc = 0; inc < msize; inc++)
            ciadd[inc] = ci1[inc] + ci2[inc];

      /*
      Multiply cms = ciadd x msub: ciadd(numsets x numsets ) msub(numsets).
      ---------------------------------------------------------------	*/
      vmulsf(ciadd, numsets, msub, &one, numsets, cms, numsets);

      /*
      Multiply mmean = cms x transpose msub: 
      --------------------------------------				*/
      vmulfp(cms, msub, numsets, &one, numsets, numsets, numsets,
                   mmean, numsets);

      /*
      Find trace of   mc  = trmc
      and  trace of mean  = tmmn
      --------------------------					*/
      trmc = 0.0;
      tmmn = 0.0;

      for (row=0; row < *numsets; row++)
      {
	diag = row * (*numsets + 1);
	trmc  += mc[diag];
	tmmn  += mmean[diag];
      }

      /*
      Take half of sum of traces:
      ---------------------------				*/
      dv = (trmc + tmmn)/2.0;

      /*
      If user requested transformed divergence, process 'dv' further:
      ---------------------------------------------------------------	*/
      if (strncmp(dvgtype, "STANDARD", 8) == 0)
         divg[kk] = dv;
      else
      {
	y = -dv/8.0;
	y = exp(y);
	divg[kk] = 100.0 * (1.0 - y);
      } 
      avg += divg[kk];
      ++kk;
   } 
}
divg[kk] = avg/(float) kk;


/*
Free allocated space for next user specification.
-------------------------------------------------		*/
free(c1);
free(c2);
free(ciadd);
free(cisub);
free(ci1);
free(ci2);
free(csub);
free(cms);
free(mc);
free(msub);
free(mmean);	

}
