#include "diverge.h"
#include "protos.h"

void compute
(
    long nband,		/* no. bands			*/
    long nc,		/* no. classes processed	*/
    int feat,
    char clname[][STNAME], /* class names		*/
    double *covar[],	/* covariance matrices		*/
    float *mean[],	/* mean vectors			*/
    char div_type[],	/* type of divergence		*/
    long print[]	/* print option			*/
)
/*
**********************************************************************

NAME: COMPUTE

FUNCTION: Calculates divergences

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	        9-Nov-82 C. Gurney
   2            9-Jan-86 B. Kovalick
   3           18-Jul-89 F. Irani/
                         K. Narayanan     Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   if feat > 0 then distances are calculated for every combination 
   of feat bands.
   if feat < 0 the user is prompted to enter abs (feat) 
   feature sets interactively. 
   if feat=0 then all bands are used together in 1 set.
   Call CALC to calculate the distances 
   Call OUTPUT to print out the statistics

**********************************************************************
*/
{
float	*div = NULL; 			/* divergence				*/

double	*scov[MAXCLS];
double	*scov_buf;
float	*smean[MAXCLS];
float	*smean_buf = NULL;

long	fset[MAXBND], 
	i, ij, ind, j, jk, k, kx, 
	m, mi, mij, mj, 
	n, ncom, ni, nj, numsets, nval1, nval2, 
	value;
long	inf_err = 1;

/*
calculate # of feature sets
---------------------------
*/
for (i = 0; i < nband; i++) fset[i] = i + 1;

m=1;
n=1;
if (feat > 0)
   {
   for (k=1; k<=feat; k++) m=m*k;
   for (k=nband-feat+1; k<=nband; k++) n=n*k;
   ncom=n/m;
   }
else if (feat == 0) ncom=1;
else ncom=abs(feat);
/*
process each feature set
------------------------
*/
for (k=1; k<=ncom; k++)
   {
   sprintf (buf, "Processing feature set %d of %d", k, ncom); 
   c_errmsg (buf, "informational", &inf_err);
/*
get band #s for next feature set
--------------------------------
*/
   if (feat < 0)
      {
      value = FALSE;
      while (!value)
         {
         for (;;)
            {
            t_write ("Enter number of bands: ", T_PROMPT);
            scanf ("%d", &numsets);
            if (numsets <= nband) break;
            sprintf (buf, "Maximum band number is %d", nband);
            c_errmsg (buf, "informational", &inf_err);
            }  
         t_write ("Enter band numbers, separated by blanks: ", T_PROMPT);
         for (jk = 0; jk < numsets; jk++) scanf ("%d", &fset[jk]); 

         value = TRUE;
         for (jk = 0; jk < numsets; jk++)
            { 
            if (fset[jk] > nband)
               {
               
               c_errmsg ("Input band(s) larger than number of bands in file", 
                         "informational", &inf_err);
               sprintf (buf, "Maximum band number is %d", nband);
               c_errmsg (buf, "informational", &inf_err);
               value = FALSE;
               break;
               }  
            }
         }
      }
   else if (feat == 0) numsets=nband;
   else
      {
      if (k > 1)
         {
         ind=feat-1;
         kx=1;
         if (fset[feat-1]+1 > nband)
            {
            for (;;)
               {
               if (fset[ind]+kx <= nband) break;
               kx++; ind--;
               }
            fset[ind]++;
            for (jk=ind+1; jk < feat; jk++)
               fset[jk] = fset[jk-1] + 1;
            }
         else
            fset[feat-1]++;
         }  
      numsets = abs (feat);
      if (feat == 0) numsets = nband;
      }  
/*
   reduce covariances & means to correct dimensions
   ------------------------------------------------
*/
   if (k == 1)
      {
      nval1 = numsets*(numsets+1)/2;
      nval2 = nc * (nc - 1) / 2 + 1;
/*
      allocate storage for covariances and means
      ------------------------------------------
*/
      if ((scov_buf = (double *) malloc (nc * nval1 * sizeof (double))) == NULL ||
          (smean_buf = (float *) malloc (nc * numsets * sizeof (float))) == NULL || 
          (div = (float *) malloc (nval2 * sizeof (float))) == NULL)
         g_errmsg ("Error allocating dynamic memory", "diverge-alloc");

      for (i = 0; i < nc; i++)
         {
         scov[i] = &scov_buf[i*nval1];
         smean[i] = &smean_buf[i*numsets];
         }
      }
   for (jk = 0; jk < nc; jk++)
      {
      for (i = 0; i < numsets; i++)
         {
         ni = fset[i] - 1;
         for (j = 0; j <= i; j++)
            {
            ij = i*(i+1)/2 + j;
            nj = fset[j] - 1;
            mi = max (ni, nj);
            mj = min (ni, nj);
            mij = mi*(mi+1)/2 + mj;
            scov[jk][ij] = covar[jk][mij];
            }
         smean[jk][i] = mean[jk][ni];
         }
      }
/*
   process each class pair 
   ------------------------
*/
   calc (&numsets, nc, scov, smean, div, div_type);

   output (clname, div, div_type, nc, fset, k, numsets, print);
   }
}
