/***********************************************************************

NAME: DIVERGE

FUNCTION: Computes standard and transformed divergences between classes

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	        9-Nov-82 C. Gurney
   2            9-Jan-86 B. Kovalick
   3           18-Jul-89 F. Irani/
                         K. Narayanan     Conversion to 5.0
   4	       24-Jul-91 T. Rockvam	  Changed covariance matrices from float
					  to double to improve accuracy
   5           29-Dec-92 D. Etrheim       PRF #5846 Reformatted lp and file 
					  output to be like term output

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get parameters
   Call STATS to get class statistics
   Call COMPUTE to calculate the distances
   Inform user of successful completion

***********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

GLOBAL	struct	PARBLK	block;

GLOBAL	FILE	*fd;                           /* output file descriptor  */

GLOBAL	long	num_outfiles;                  /* number of output files  */
 
GLOBAL  char	buf[ERRLEN]; 		       /* error message buffer    */	
GLOBAL  char    out_files[1][CMLEN];              /* output file names       */

#define TERM 0 
#define LP   1 
#define FIL 2 

int main
(void)
{
double	*covar[MAXBND];		/* covariance matrices			*/
float	*mean[MAXBND];		/* mean vectors				*/

long	feat;			/* no. features requested		*/
long	num_bands;		/* no. bands 				*/
long	num_classes;		/* no. classes processed		*/
long	one = 1;
long	print[4];		/* print option				*/
long	zero = 0;

char	class_names[MAXCLS][STNAME]; /* class names			*/
char	div_type[12];		/* type of divergence			*/
char	hos_stat[CMLEN];	/* stats file host name			*/
char	print_file[1][CMLEN];	/* print file				*/
char    tmpstr[1][CMLEN];       /* temporary string         */

static char    *prog_name = "DIVERGE"; /* program name            */

/*
get tae parameters
------------------
*/
getpar (hos_stat, &num_classes, print, print_file[0], &feat, div_type);

/*
retrieve statistics
-------------------
*/
stats (hos_stat, feat, &num_classes, &num_bands, covar, mean, class_names);

/*
calculate distances
-------------------
*/
compute (num_bands, num_classes, feat, class_names, covar, mean, div_type, 
         print);

/*
issue completion message
------------------------
*/
if (print[FIL])
   c_complt (prog_name, print_file, &one);
else
{
   strcpy(tmpstr[0], "\0");
   c_complt (prog_name, tmpstr, &zero);
}

return EXIT_SUCCESS;

}

