#include "diverge.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char hos_stat[],          /* stats file host name             */
    long *num_classes,        /* number of classes to be used     */
    long print[],	     /* print option                     */
    char print_file[CMLEN],   /* print file name                  */
    long *feat,		     /* number of features		 */
    char dvgtype[]
)
  	
/*
**********************************************************************

NAME: GETPAR

FUNCTION: Gets parameters 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	        9-Nov-82 C. Gurney
   2            9-Jan-86 B. Kovalick
   3           18-Jul-89 F. Irani/
                         K. Narayanan     Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Initialize program
   Get and validate parameters
   If output print file or line printer is specified 
      Open a file
      Print header information

**********************************************************************
*/
{
struct VARIABLE	 *var;

long	access,                     /* access type for getnam             */
	itype,
	sdim,
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */
long	fatal_err = -1;
long	one = 1;
long	zero = 0;

char	fileutil_file[CMLEN];       /* file name for fileutil      */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get sfin parameter
------------------
*/
access = IREAD;
stat = c_getnam (&block, "INSTAT", &zero, hos_stat, "stat", &access);
if (stat != E_SUCC)
   c_errmsg("Fatal error encountered","diverge-fatal",&fatal_err);

itype = ELONG;
if (c_getnum (&block, "NCLASS", &one, &itype, num_classes) == 0)
   *num_classes = 0;
if (c_getnum (&block, "BANDOPT", &one, &itype, feat) == 0) *feat = 0;

sdim = 11;
stat = c_getstr (&block, "TYPEOPT", &one, &sdim, dvgtype);
/*
if batch mode and interactive parameters are specified inform and abort
-----------------------------------------------------------------------
*/
var = p_find (&block, "$runtype");

if ((*num_classes > 0 || *feat < 0) && SVAL (*var, 0)[0] == 'B')
   c_errmsg ("Run interactively with NCLASS > 0 or BANDOPT < 0", 
             "diverge-batch", &fatal_err);

/*
Get print option; if file or line printer open print file 
and print header information
---------------------------------------------------------
*/
num_outfiles = 0;
strcpy (fileutil_file, "diverge");
stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file, 
          print_file);
if (stat != E_SUCC)
   c_errmsg("Fatal error encountered","diverge-fatal",&fatal_err);

if (print[LP] || print[FIL])
   {
   strcpy (out_files[0], fileutil_file);
   num_outfiles++;
   fd = fopen (fileutil_file, "w");
   }
}
