/***************************************************************************
FUNCTION: DOQNEW

PURPOSE: Convert a new format DOQ image into LAS.

PROGRAM HISTORY:
  Version       Date    Author          Reason
  -------       ----    ------          -------
    1.0         3/97    K.Lowell        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Runs under TAE/UNIX

PROJECT:  LAS

ALGORITHM DESCRIPTION: See inline comments
**************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "doq2las.h"

#define REC_SIZE  4000
#define TEST_SIZE   5           /* add 1 byte for carriage return       */
#define PRIMARY_DATUM 0         /* flag values for primary and secondary */
#define SECONDARY_DATUM 1       /* datums                                */

extern long curent;             /* current processing line                   */
extern long curcat;             /* current processing band                   */
int  fdesc[MAXIMG+1];           /* image file descriptors used for image I/O */


/* Internal function prototype */
int get_primary_datum
(
    const char *char_buf,    /* I: Text buffer for header and image records */
    const char *datum        /* I: ASCII value for datum                    */
);


void main
(
  void
)
{
    struct PARBLK parblk;           /* TAE paramter block                   */
    static long bands[1][MAXBND+1]; /* bands to be processed - all images*/
    char    hostdoq_name[CMLEN];    /* input unix disk file name            */
    char    doqname[CMLEN];         /* input TAE disk file name             */
    char    hosin[1][CMLEN];        /* array for MINMAX                     */
    char    hosout[4][CMLEN];       /* array for CLEAN                      */
    char    tname[CMLEN];           /* output tae name                      */
    char    filename[CMLEN];        /* work space for file name creation    */
    char    hostddr_name[CMLEN];    /* output ddr unix disk file name       */
    char    hostimg_name[CMLEN];    /* output img unix disk file name       */
    char    hostdhd_name[CMLEN];    /* output doq header file               */
    char    *char_buf;              /* text buffer for header and image recs*/
    char    dir[CMLEN];             /* directory returned from c_splitname */
    char    root[CMLEN];            /* root returned from c_splitname       */
    char    ext[CMLEN];             /* extension returned from c_splitname  */
    char    temp[CMLEN];            /* hold string for strstr calls         */
    char    abands[10][CMLEN];      /* ASCII value for image bands          */
    char    datum[6];               /* ASCII value for datum "NAD27"or"NAD83" */
    char    datum_hdr[CMLEN];       /* ASCII datum value in DOQ header      */
    char    hor_coor[CMLEN];        /* ASCII horizontal coordinate system   */
    char    *p;                     /* header string hold                   */
    const char underscore = 95;     /* integer value of an underscore       */

    int     input_fd;               /* file descriptor for input doq file
                                       with header                          */
    struct FDESC *output_fd;        /* file descriptor for output image file*/
    FILE    *output_dh;             /* file descriptor for output doq header */
    long    offset[4];              /* file offest                          */
    long    acc=IWRITE;             /* access indicator                     */
    long    rec_size = 0;           /* record size                          */
    long    ii = 0;                 /* loop counter                         */
    long    jj = 0;                 /* loop counter                         */
    long    length = 0;             /* length of string calls               */
    long    ret_stat = 0;           /* return value of function calls       */
    long    fatal=0;                /* fatal message flag                   */
    long    nonfatal=1;             /* nonfatal message flag                */
    long    std_out = 0;            /* logical unit for standard ouput      */
    long    one = 1;                /* value of one                         */
    long    three = 3;              /* value of three                       */
    long    four = 4;               /* value of four                        */
    long    msgtim = 0;             /* elapsed time between display msg     */
    long    msg_flag=ALLCAT;        /* flag for porcessing message          */
    long    bipflag = 0;            /* data is really BIP flag              */
    long    odtype=EBYTE;           /* Output data tpe                      */
    long    endfield,bgnfield;      /* Keyword value limit checks           */
    long    bits=8;                 /* number of bits per pizel             */
    long    opt[MAXBND];            /* option for eopens                    */

    int     datum_flag;             /* Flag denoting primary or secondary 
                                       datum                                */

    double  nw_line_float;
    double  nw_samp_float;

    struct  DDR ddr_struct;         /* LAS ddr structure                    */
    struct  BDDR bddr_struct[5];    /* LAS band independent ddr info        */

    char    quad_name[39];          /* These are all receivers of header    */
    char    quad[3];                /* fields.  Refer to the USGS Document  */
    char    nation[5];              /* "Standards for Digital Orthophotos". */
    char    xy_units_str[12];

    long    nl = 0;
    long    ns = 0;
    long    nb = 0;
    /*long    data_type = 0;*/
    long    band_code = 0;
    long    hor_datum = 0;
    long    ref_sys = 0;
    long    zone_code = 0;

    double  tmp_dbl;
    double  sw_x_coord;
    double  sw_y_coord;
    double  nw_x_coord;
    double  nw_y_coord;
    double  ne_x_coord;
    double  ne_y_coord;
    double  se_x_coord;
    double  se_y_coord;
    double  upper_left_line;
    double  upper_left_samp;
    double  upper_right_line;
    double  upper_right_samp;
    double  lower_left_line;
    double  lower_left_samp;
    double  lower_right_line;
    double  lower_right_samp;
    double  x_pixel_resolution;

    char deg[4];
    char lon[4];
    char proj_opt[100];


    strcpy (deg,"DEG");
    strcpy (lon,"LON");

    /* Initialize TAE parameter block
       ----------------------------------*/
    c_inlas(&parblk,&std_out);

    /* Get LAS parameters
       ----------------------*/
    if ((getpar(&parblk,doqname,hosin[0],tname,datum,proj_opt,
            &msgtim)) != E_SUCC)
        c_errmsg("Fatal error encountered","doq2las-fatal",&fatal);

    /* Create some file names 
       --------------------------*/
    strcpy (hostdoq_name,doqname);
    c_splitname(hosin[0],dir,root,ext);
    strcpy (hostimg_name,dir);
    strcat (hostimg_name,root);
    strcpy (hosout[0],hostimg_name);
    strcat (hosout[0],".img");
    strcpy(hostdhd_name,hostimg_name);
    strcat(hostdhd_name,".dhd");
    strcpy (hosout[1],hostdhd_name);
    strcpy(hostddr_name,hostimg_name);
    strcat(hostddr_name,".ddr");
    strcpy(hosout[2],hostddr_name);
    strcpy(hosout[3],hostimg_name);
    strcat(hosout[3],".his");

    /* This section will be used later if user is given option of
       projection
       ------------------------------------------------------------------*/
    if ((strcmp(proj_opt,"TM") != 0) && (strcmp(proj_opt,"UTM") != 0))
        c_errmsg("Invalid projection option","doq2las-proj",&fatal);

    strcpy (filename,hostdoq_name);
    rec_size = REC_SIZE;
    char_buf = (char *)malloc(sizeof(char) * rec_size);
    if (char_buf == NULL)
        c_errmsg("Error allocating memory","doq2las-alloc",&fatal);

    /* Open the input file
       -------------------*/
    if ((input_fd = open(hostdoq_name, O_RDONLY)) == -1)
        c_errmsg("Error opening DOQ file","doq2las-open",&fatal);

    /* Read the first record
       ---------------------*/
    if (read(input_fd, char_buf, rec_size) != rec_size)
        c_errmsg("Error reading DOQ file","doq2las-read",&fatal);

    if (strncmp(char_buf,"BEGIN_USGS_DOQ_HEADER",21) != 0)
    {
        close(input_fd);
        c_errmsg("Not a new DOQ","doq-new",&fatal);
    }

    /* Extract the number of pixels per line
       -----------------------------------------*/
    if ((p=strstr(char_buf,"SAMPLES_AND_LINES")) != NULL)
    {
        sscanf(&p[17],"%d %d",&ns, &nl);
        ddr_struct.nl = nl;
        ddr_struct.ns = ns;
    }
    else
    {
        close(input_fd); 
        c_errmsg("Unable to locate SAMPLES_AND_LINES","doq2las-data",&fatal);
    }

    /* Realloc the input buffer to the proper size
       -----------------------------------------------*/
    rec_size= ns;
    free(char_buf);
    char_buf = (char *)malloc(sizeof(char) * rec_size);
    if (char_buf == NULL)
    {
        close(input_fd);
        c_errmsg("Error allocating memory","doq2las-alloc",&fatal);
    }

    /* Rewind the file
       ------------------*/
    if (lseek(input_fd,0,SEEK_SET) != 0)
    {
        close(input_fd);
        c_errmsg("Error rewinding the DOQ file","doq2las-rewind",&fatal);
    }

    /* Read the first record with the proper record size
       ---------------------------------------------------*/
    if (read(input_fd, char_buf, rec_size) != rec_size)
    {
        close(input_fd);
        c_errmsg("Error reading DOQ file","doq2las-read",&fatal);
    }


    /* Extract the quad name
       ---------------------------*/
    if ((p=strstr(char_buf,"QUADRANGLE_NAME")) != NULL)
    {
        bgnfield = (long)strcspn(p,"\"");
        endfield = (long)strcspn(&p[bgnfield+1],"\"");
        strncpy(quad_name,&p[bgnfield+1],endfield);
    }
    else
        c_errmsg("Unable to locate QUADRANGLE_NAME","doq2las-data",&nonfatal);

    /* Extract the quadrant name
       ---------------------------*/
    if ((p=strstr(char_buf,"QUADRANT")) != NULL)
    {
        sscanf(&p[8],"%s",quad);
        quad[2] ='\0';
    }
    else
        c_errmsg("Unable to locate QUADRANT","doq2las-data",&nonfatal);

    /* Extract the nation
       ---------------------------*/
    if ((p=strstr(char_buf,"NATION")) != NULL)
        sscanf(&p[6],"%s",nation);
    else
        c_errmsg("Unable to locate NATION","doq2las-data",&nonfatal);


    /* Extract band organization
       -----------------------------*/
    if ((p=strstr(char_buf,"BAND_ORGANIZATION")) != NULL)
    {
        sscanf(&p[17],"%s",temp);
        ii = (long)strlen(temp);
        c_low2up(temp,&ii);
        if ((p=strstr (temp,"SINGLE")) != 0) band_code = 1;
        else if (strcmp (temp,"BSQ") == 0) band_code = 1;
        else if (strcmp (temp,"BIL") == 0) band_code = 1;
        else if (strcmp (temp,"BIP") == 0) band_code = 5;
        else
        {
            close(input_fd);
            c_errmsg("Unsupported BAND_ORGANIZATION","doq2las-supp",&fatal);
        }
    }
    else
    {
        close(input_fd);
        c_errmsg("Unable to locate BAND_ORGANIZATION","doq2las-data",&fatal);
    }

    /* Extract band content  (repeat to get all occurances, for now assume
       B&W or RGB colour
       ------------------------------------------------------*/
    if ((p=strstr(char_buf,"BAND_CONTENT")) != NULL)
    {
        sscanf(&p[12],"%s",abands[nb-1]);
        if (strcmp (abands[nb-1],"BLACK&WHITE") == 0) 
        {
            nb = 1;
            bands[0][0]=1;
        }
        else
        { 
            nb = 3;
            bands[0][0]=1;
            bands[0][1]=2;
            bands[0][2]=3;
        }
        ddr_struct.nbands = nb;
    }
    else
    {
        close(input_fd);
        c_errmsg("Unable to locate BAND_CONTENT","doq2las-data",&fatal);
    }


    /* Extract the bits per pixel
       ---------------------------------*/
    if ((p=strstr(char_buf,"BITS_PER_PIXEL")) != NULL)
    {
        sscanf(&p[14],"%d",&bits);
        if (bits  == 8)
        {
            /*data_type=1; */
            ddr_struct.dtype=1;
        }
        else
        {
            close(input_fd);
            c_errmsg("Unsupported data type","doq2las-supp",&fatal);
        }
    }
    else
    {
        close(input_fd);
        c_errmsg("Unable to locate dtype","doq2las-data",&fatal);
    }


    switch (band_code)
    {
        case 1:                      /* 1 band black & white (B&W)   */
        case 2:                      /* 1 band red                   */
        case 3:                      /* 1 band green                 */
        case 4:                      /* 1 band blue                  */
            nb = 1;
            break;
        case 5:                      /* 3 band color (RGB)           */
            nb = 3;
            msg_flag=ONECAT;
            bipflag = 1;
            break;
        case 6:                      /* 3 band RGB + 1 band B&W      */
            nb = 4;
            msg_flag=ONECAT;
            break;
    }

    /* Extract the horizontal datum based on the user specification.
       ----------------------------------------------------------------*/
    if (strcmp(datum, "NAD27") == 0)
    {
        /* NAD27 (associated with the second horizontal datum) user-specified.
           Check for its existence. */
        if ((p = strstr(char_buf, "SECONDARY_HORIZONTAL_DATUM")) != NULL)
        {
            /* Get secondary datum from header.  If it is NAD27, then use the
               LAS datum number for NAD27 Mean of CONUS (225).  Otherwise, 
               switch to NAD83 (always the DOQ primary datum). */
            sscanf(&p[26], "%s", datum_hdr);
            if (strcmp(datum_hdr, datum) == 0)
            {
                hor_datum = 225;
                datum_flag = SECONDARY_DATUM;
                ret_stat = E_SUCC;
            }
            else
            {
                c_lasmsg(FALSE, "doq2las-datum", "Secondary datum %s not "
                        "supported. Using primary (NAD83).", datum_hdr);

                /* Get primary datum from header.  If it is NAD83, then use the
                   LAS datum number for NAD83 CONUS (219).  Otherwise the header
                   is nonstandard DOQ, so exit. */
                strcpy(datum, "NAD83");
                if ((ret_stat = get_primary_datum(char_buf, datum)) == E_SUCC)
                {
                    hor_datum = 219;
                    datum_flag = PRIMARY_DATUM;
                }
            }
        }
        else
        {
            /* No secondary datum found, so switch to NAD83 (always the DOQ
               primary datum). */
            c_lasmsg(FALSE, "doq2las-datum",
                    "Unable to locate secondary datum (NAD27)."
                    "  Using primary (NAD83).");

            /* Get primary datum from header.  If it is NAD83, then use the
               LAS datum number for NAD83 CONUS (219).  Otherwise the header
               is nonstandard DOQ, so exit. */
            strcpy(datum, "NAD83");
            if ((ret_stat = get_primary_datum(char_buf, datum)) == E_SUCC)
            {
                hor_datum = 219;
                datum_flag = PRIMARY_DATUM;
            }
        }
    }
    else
    {
        /* NAD83 (associated with the primary horizontal datum) user-specified.
           Get primary datum from header.  If it is NAD83, then use the LAS
           datum number for NAD83 CONUS (219).  Otherwise the header is
           nonstandard DOQ, so exit. */
        if ((ret_stat = get_primary_datum(char_buf, datum)) == E_SUCC)
        {
            hor_datum = 219;
            datum_flag = PRIMARY_DATUM;
        }
    }
    if (ret_stat != E_SUCC) 
    {
        free (char_buf);
        close(input_fd);
        c_lasmsg(TRUE, "doq2las-fatal", "Fatal error encountered");
    }

    ddr_struct.datum_code = hor_datum;
    ddr_struct.valid[DDDCV] = VALID;

    /* Extract the horizontal coordinate system 
       -------------------------------------------*/
    if ((p=strstr(char_buf,"HORIZONTAL_COORDINATE_SYSTEM")) != NULL)
    {
        sscanf(&p[28],"%s", hor_coor);
        
        /* The header can have upper and lower case. 
           Change to upper for consistancy
           ----------------------------------------*/
        length = (long)strlen(hor_coor);
        c_low2up(hor_coor,&length);

        if (strncmp(hor_coor,"UTM",3) == 0)
            ref_sys = 1;
        else if (strncmp(hor_coor,"SPCS",4) == 0)
            ref_sys = 2;
        else if (strncmp(hor_coor,"GEOGRAPHIC",10) == 0)
            ref_sys = 0;
        else
            c_errmsg("Unsupported HORIZONTAL_COORDINATE_SYSTEM","doq2las-supp",
                    &nonfatal);
        ddr_struct.proj_code = ref_sys;
        ddr_struct.valid[DDPCV] = VALID;
    }
    else
        c_errmsg("Unable to locate HORIZONTAL_COORDINATE_SYSTEM","doq2las-data",
                &nonfatal);

    /* Extract the zone code 
       -------------------------------------------*/
    if ((p=strstr(char_buf,"COORDINATE_ZONE")) != NULL)
    {
        sscanf(&p[16],"%d",&zone_code);
        ddr_struct.zone_code = zone_code;
        ddr_struct.valid[DDZCV]=VALID;
    }
    else
        c_errmsg("Unable to locate COORDINATE_ZONE","doq2las-data",&nonfatal);

    if (ref_sys == 0)                           /* DOQ header code for GEO */
    {
        ref_sys = 0;                                /* LAS ddr code for GEO */
        ddr_struct.valid[3] = VALID;
    }
    else if (ref_sys == 1)                      /* DOQ header code for UTM */
    {
        if (strcmp(proj_opt,"UTM") == 0) 
        {
            ref_sys = 1;                      /* LAS ddr code for UTM */
            ddr_struct.valid[3] = VALID;
        }
        else
        {
            /* LAS assumes that all UTM's are NAD 27, so a UTM NAD 83 will not 
               work, therefore, generate a TM with NAD 83 and the corresponding
               ddr proj info that is valid for TM, which WILL work with LAS 
               reprojections.  If this becomes an issue later, just put in a 
               flag to optionally convert to TM, rather than always doing so. */

            ref_sys = 9;                        /* LAS ddr code for TM */
            ddr_struct.proj_coef[ 0] = 6378137.0; /* Semi-major axis 
                                                    of ellipsoid*/
            ddr_struct.proj_coef[ 1] = 6356752.31414; /* Semi-minor axis 
                                                         of ellipsoid */
            ddr_struct.proj_coef[ 2] = 0.9996;  /* Scale factor at  
                                                   central meridian */
            tmp_dbl = (6 * abs(zone_code)) - 183;
            /* Longitude of Central Meridian, in degrees */
            c_degdms(&tmp_dbl,&ddr_struct.proj_coef[4],deg,lon);
            ddr_struct.proj_coef[ 6] = 500000.0;          /* False Easting */
            ddr_struct.proj_coef[ 7] = (zone_code < 0) ? 10000000.0 : 0.0;
                                                          /*Fls N'ing*/
            ddr_struct.valid[3] = VALID;
        }
    }
    else if (ref_sys == 2)                    /* DOQ header code for SPCS */
    {
        ref_sys = 2;                              /* LAS ddr code for SPCS */
        ddr_struct.valid[3] = VALID;
    }

    /* Extract the projection units 
       -------------------------------------------*/
    if ((p=strstr(char_buf,"HORIZONTAL_UNITS")) != NULL)
    {
        sscanf(&p[16],"%s",xy_units_str);

        if (c_string2unit(xy_units_str) != E_FAIL) 
        {
            strcpy(ddr_struct.proj_units,xy_units_str);
            ddr_struct.valid[DDPUV] = VALID;
        }
        else
        {
            c_lasmsg(FALSE, "doq2las-units", "Projection units %s are not "
                    "supported by LAS. Setting projection units to invalid", 
                    xy_units_str);
            strcpy(ddr_struct.proj_units,"");
            ddr_struct.valid[DDPUV] = INVAL;
        } 
    }
    else
        c_errmsg("Unable to locate HORIZONTAL_UNITS","doq2las-data",&nonfatal);

    /* Extract the corner coordinates
       -----------------------------------------------*/
    if (datum_flag == SECONDARY_DATUM)
    {
        if ((p=strstr(char_buf,"SECONDARY_NW_QUAD_XY")) != NULL)
            sscanf(&p[20],"%lf %lf",&nw_x_coord,&nw_y_coord);
        else
            c_errmsg("Unable to locate SECONDARY_NW_QUAD_XY", "doq2las-data",
                    &nonfatal);

        if ((p=strstr(char_buf,"SECONDARY_NE_QUAD_XY")) != NULL)
            sscanf(&p[20],"%lf %lf",&ne_x_coord,&ne_y_coord);
        else
            c_errmsg("Unable to locate SECONDARY_NE_QUAD_XY", "doq2las-data",
                    &nonfatal);

        if ((p=strstr(char_buf,"SECONDARY_SW_QUAD_XY")) != NULL)
            sscanf(&p[20],"%lf %lf",&sw_x_coord,&sw_y_coord);
        else
            c_errmsg("Unable to locate SECONDARY_SW_QUAD_XY", "doq2las-data",
                    &nonfatal);

        if ((p=strstr(char_buf,"SECONDARY_SE_QUAD_XY")) != NULL)
            sscanf(&p[20],"%lf %lf",&se_x_coord,&se_y_coord);
        else
            c_errmsg("Unable to locate SECONDARY_SE_QUAD_XY", "doq2las-data",
                    &nonfatal);
    } 
    else /* datum_flag == PRIMARY_DATUM */
    {
        if ((p=strstr(char_buf,"NW_QUAD_CORNER_XY")) != NULL)
            sscanf(&p[17],"%lf %lf",&nw_x_coord,&nw_y_coord);
        else
            c_errmsg("Unable to locate NW_QUAD_CORNER_XY", "doq2las-data",
                    &nonfatal);

        if ((p=strstr(char_buf,"NE_QUAD_CORNER_XY")) != NULL)
            sscanf(&p[17],"%lf %lf",&ne_x_coord,&ne_y_coord);
        else
            c_errmsg("Unable to locate NE_QUAD_CORNER_XY", "doq2las-data",
                    &nonfatal);

        if ((p=strstr(char_buf,"SW_QUAD_CORNER_XY")) != NULL)
            sscanf(&p[17],"%lf %lf",&sw_x_coord,&sw_y_coord);
        else
            c_errmsg("Unable to locate SW_QUAD_CORNER_XY", "doq2las-data",
                    &nonfatal);

        if ((p=strstr(char_buf,"SE_QUAD_CORNER_XY")) != NULL)
            sscanf(&p[17],"%lf %lf",&se_x_coord,&se_y_coord);
        else
            c_errmsg("Unable to locate SE_QUAD_CORNER_XY", "doq2las-data",
                    &nonfatal);
    } 


    /* Reset buffer to correct record size (image sample size)
       -------------------------------------------------------*/
    if (bipflag != 1)
        rec_size = ns;
    else
        rec_size = nb*ns;
    free(char_buf);
    char_buf = (char *)malloc(sizeof(char) * rec_size);
    if (char_buf == NULL)
    {
        close(input_fd);
        c_errmsg("Error allocating memory","doq2las-alloc",&fatal);
    }

    /* Rewind the file
       --------------*/
    if (lseek(input_fd,0,SEEK_SET) != 0)
    {
        close(input_fd);
        c_errmsg("Error rewinding DOQ file","doq2las-rewind",&fatal);
    }

    /* Open the output DOQ header file
       ---------------------------------*/
    if ((output_dh = fopen(hostdhd_name,"w")) == NULL)
    {
        close(input_fd);
        c_errmsg("Error opening DOQ header file","doq2las-open",&fatal);
    }

    /* Read the entire first record to get to the start of the second record
       ---------------------------------------------------------------------*/
    if (read(input_fd, char_buf, rec_size) != rec_size)
    {
        close(input_fd);
        fclose(output_dh);
        c_errmsg("Error reading DOQ file","doq2las-read",&fatal);
    }

    if (fwrite(char_buf,sizeof(char),rec_size, output_dh) != rec_size)
    {
        close(input_fd);
        fclose(output_dh);
        c_errmsg("Error writing DOQ header file","doq2las-write",&fatal);
    }


    /* Extract the x y offsets for origin of image 
       ----------------------------------------------*/
    if (datum_flag == SECONDARY_DATUM)
    {
        if ((p=strstr(char_buf,"SECONDARY_XY_ORIGIN")) != NULL)
            sscanf(&p[19],"%lf %lf",&nw_samp_float, &nw_line_float);
        else
            c_errmsg("Unable to locate SECONDARY_XY_ORIGIN", "doq2las-data",
                    &nonfatal);
    }
    else /* datum_flag == PRIMARY_DATUM */
    {
        if ((p=strstr(char_buf,"XY_ORIGIN")) != NULL)
            sscanf(&p[9],"%lf %lf",&nw_samp_float, &nw_line_float);
        else
            c_errmsg("Unable to locate XY_ORIGIN", "doq2las-data",
                    &nonfatal);
    }

    rec_size= ns;
    if (char_buf == NULL)
    {
        close(input_fd);
        fclose(output_dh);
        c_errmsg("Error allocating memory","doq2las-alloc",&fatal);
    }

    if ((nw_line_float !=0) && (nw_samp_float != 0))
    {
        upper_left_line = nw_line_float;
        upper_left_samp = nw_samp_float;
        upper_right_line = upper_left_line;
        upper_right_samp = upper_left_samp + ns - 1;
        lower_left_line = upper_left_line - nl + 1;
        lower_left_samp = upper_left_samp;
        lower_right_line = upper_left_line - nl + 1;
        lower_right_samp = upper_left_samp + ns - 1;

        ddr_struct.upleft[0] = upper_left_line;
        ddr_struct.upleft[1] = upper_left_samp;
        ddr_struct.loleft[0] = lower_left_line;
        ddr_struct.loleft[1] = lower_left_samp;
        ddr_struct.upright[0] = upper_right_line;
        ddr_struct.upright[1] = upper_right_samp;
        ddr_struct.loright[0] = lower_right_line;
        ddr_struct.loright[1] = lower_right_samp;
        ddr_struct.valid[DDCCV] = VALID;
    }

    /* acquire the horizontal resolution
       -------------------------------------*/
    if ((p=strstr(char_buf,"HORIZONTAL_RESOLUTION")) != NULL)
    {
        sscanf(&p[21],"%lf",&x_pixel_resolution);
        ddr_struct.pdist_y = x_pixel_resolution;
        ddr_struct.pdist_x = x_pixel_resolution; 
        ddr_struct.valid[DDPDV] = VALID;
    }
    else
    {
        c_errmsg("Unable to locate HORIZONTAL_RESOLUTION","doq2las-data",
                  &nonfatal);
    }

    /* fill in misc values in the DDR
       ----------------------------------*/
    ddr_struct.master_line = 0;
    ddr_struct.master_sample = 0;
    ddr_struct.spare = 0; 

    strcpy(ddr_struct.system,"ieee-std");

    ddr_struct.line_inc = 1.0;
    ddr_struct.sample_inc = 1.0;
    ddr_struct.valid[DDINCV] = VALID;

    strcpy(filename,hostimg_name);

    length = (long)strcspn(filename, ".");

    while (length != strlen(filename))
    {
        filename[length] = underscore;
        length = (long)strcspn(filename, ".");
    }

    /* Close the DOA header file
       -----------------------------*/
    if (fclose(output_dh) != 0)
    {
        c_errmsg("Error closing DOQ header file","doq2las-close",&nonfatal);
        close(input_fd);
        c_clean(&parblk,"DOQ2LAS",hosout,&three);
    }

    /* BSQ processing section
       --------------------------*/
    if (bipflag != 1)
    {
        /* Open the output file
           --------------------*/
        opt[0] = 0;
        if (c_eopens(&output_fd,hosin[0],bands[0],&one,&one,&nl,&ns,offset,&acc,
                &odtype,&fatal,opt) == -1)
        {
            c_errmsg("Error opening image file","doq2las-open",&nonfatal);
            close(input_fd);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }

        /* Setup for processing message utility
           ---------------------------------------*/
        c_inittm("DOQ2LAS","Processed","line","bands"," ",&nl,&nb,&msgtim,
                &msg_flag);
        curcat = 1;
        curent = 1;
        c_startm();

        /* Now read the remaining records of the file as image data, writing 
           out to the image disk file
           ------------------------------------------------------------------*/
        for (jj = 1; jj <= nb; jj++)
        {
            curent = 1;
            for (ii = 1; ii <= nl; ii++)
            {
                if (read(input_fd, char_buf, rec_size) != rec_size)
                {
                    c_errmsg("Error reading DOQ file","doq2las-read",&nonfatal);
                    close(input_fd);
                    c_clean(&parblk,"DOQ2LAS",hosout,&three);
                }

                if (c_ewrite(&output_fd,&jj,&ii,(unsigned char *)char_buf,
                        &one) != rec_size)
                {
                    c_errmsg("Error writing image file","doq2las-write",
                              &nonfatal);
                    close(input_fd);
                    c_clean(&parblk,"DOQ2LAS",hosout,&three);
                }
                curent++;
            }
            curcat++;
        }

        free(char_buf);

        /* Turn off display messages
           ----------------------------*/
        c_stoptm();

        if (close(input_fd) != 0)
        {
            c_errmsg("Error closing DOQ file","doq2las-close",&nonfatal);
            close(input_fd);
            c_eclose(&output_fd);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }

        if (c_eclose(&output_fd) != 0)
        {
            c_errmsg("Error closing image file","doq2las-close",&nonfatal);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }
    }
    else /* data type = 2 ie stored bip*/
    {

        /* close the DOQ file to allow stripdoq to use it
           -----------------------------------------------*/
        if (close(input_fd) != 0)
        {
            c_errmsg("Error closing DOQ file","doq2las-close",&nonfatal);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }
        rec_size=nb*ns;
        stripdoqbip(doqname,hosin[0],&ns,&nl,&nb,&rec_size);
    }

    /* section to use if bil is allowed in reality
       char    minmax[4];              minmax flag
       long    dflag = 0;              ddr flag
       long    nheader = 0;            number of header bytes
       nheader = ns;
       strcpy(minmax,"NO");
       strcpy(hosin[0],doqname);
       inbil(&parblk,hosin[0],nl,ns,nb,nheader,hosin,data_type,&msgtim,minmax,
             dflag);
     */

    /* Write out the ddr information
       ----------------------------------*/
    if (c_putddr(hostddr_name, &ddr_struct) != E_SUCC)
    {
        close(input_fd);
        c_clean(&parblk,"DOQ2LAS",hosout,&three);
    }

    for (ii = 0; ii < ddr_struct.nbands; ii++)
    {
        c_intbdr(&bddr_struct[ii]);
        bddr_struct[ii].bandno = ii + 1;
        bddr_struct[ii].valid = INVAL;

        if (c_putbdr(hostddr_name, &bddr_struct[ii]) != E_SUCC)
        {
            c_errmsg("Error returned from c_putbdr","doq2las-ddr",&nonfatal);
            close(input_fd);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }
    }

    if (c_lused(hostddr_name) != E_SUCC)
    {
        c_errmsg("Error returned from c_lused","doq2las-lused",&nonfatal);
        close(input_fd);
        c_clean(&parblk,"DOQ2LAS",hosout,&three);
    }

    /* Create the history file
       ---------------------------*/
    if (c_hstout(&parblk,hosout[0],hosin,&one,"doq2las") != E_SUCC)
        c_errmsg("Error returned from c_hstout","doq2las-lused",&nonfatal);

    c_complt("DOQ2LAS",hosout,&four);
}


/*****************************************************************************
 FUNCTION: get_primary_datum
 
 PURPOSE: Retrieve the primary datum value from the DOQ header.
 
 RETURNS:
          E_SUCC    Successful retrieval of expected datum.
          E_FAIL    Primary datum does not exist or is not expected value.
 
*****************************************************************************/
int get_primary_datum
(
    const char *char_buf,    /* I: Text buffer for header and image records */
    const char *datum        /* I: ASCII value for datum                    */
)
{
    char datum_hdr[CMLEN];     /* ASCII datum value in DOQ header             */
    char *ptr;                 /* header string pointer                       */
    long len;


    /* Check for existence of primary datum.
       ----------------------------------------*/
    if ((ptr = strstr(char_buf, "HORIZONTAL_DATUM")) != NULL)
    {
        /* Get primary datum from header.  If it agrees with the expected
           value, return success.  Otherwise the header is nonstandard DOQ,
           so return failure status.
           -------------------------------------------------------------------*/
        sscanf(&ptr[16], "%s", datum_hdr);
        len = (long)strlen(datum_hdr);
        c_low2up(datum_hdr,&len);
        if (strcmp(datum_hdr, datum) == 0)
            return E_SUCC;
        else
        {
            c_lasmsg(FALSE, "doq2las-datum",
                    "Error: Primary datum not expected value.");
            return E_FAIL;
        }
    }
    else
    {
        /* Nonstandard header.  No primary datum.  Return failure status.
           -----------------------------------------------------------------*/
        c_lasmsg(FALSE, "doq2las-datum",
                "Error: Unable to locate primary datum.");
        return E_FAIL;
    }
}
