/***************************************************************************
FUNCTION: DOQOLD

PURPOSE: Convert a old format DOQ image into LAS.

PROGRAM HISTORY:
  Version       Date    Author          Reason
  -------       ----    ------          -------
    1.0         3/97    K.Lowell        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Runs under TAE/UNIX

PROJECT:  LAS

ALGORITHM DESCRIPTION: See inline comments
**************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "doq2las.h"

#define REC_SIZE  400
#define TEST_SIZE   5           /* add 1 byte for carriage return       */

extern long curent;             /* current processing line                   */
extern long curcat;             /* current processing band                   */
int  fdesc[MAXIMG+1];           /* image file descriptors used for image I/O */

void main
(
  void
)
{
    struct PARBLK parblk;           /* TAE paramter block                   */
    static long bands[1][MAXBND+1]; /* bands to be processed - all images   */
    char    hostdoq_name[CMLEN];    /* input unix disk file name            */
    char    doqname[CMLEN];         /* input TAE disk file name             */
    char    hosin[1][CMLEN];        /* array for MINMAX                     */
    char    hosout[4][CMLEN];       /* array for CLEAN                      */
    char    tname[CMLEN];           /* output tae name                      */
    char    filename[CMLEN];        /* work space for file name creation    */
    char    hostddr_name[CMLEN];    /* output ddr unix disk file name       */
    char    hostimg_name[CMLEN];    /* output img unix disk file name       */
    char    hostdhd_name[CMLEN];    /* output doq header file               */
    char    *char_buf;              /* text buffer for header and image recs*/
    char    dir[CMLEN];             /* directory returned from c_splitname */
    char    root[CMLEN];            /* root returned from c_splitname       */
    char    ext[CMLEN];             /* extension returned from c_splitname  */
    const char capitalE = 69;
    const char underscore = 95;     /* integer value of an underscore       */

    int     input_fd;               /* file descriptor for input doq file
                                       with header                          */

    struct FDESC *output_fd;        /* file descriptor for output image file*/
    FILE    *output_dh;             /* file descriptor for output doq header */
    long    offset[4];              /* file offest                          */
    long    acc=IWRITE;             /* access indicator                     */
    long    rec_size = 0;           /* record size                          */
    long    hrec_size = 400;        /* header record size on output         */
    long    ii = 0;                 /* loop counter                         */
    long    jj = 0;                 /* loop counter                         */
    long    length = 0;             /* length of string calls               */
    long    ret_stat = 0;           /* return value of function calls       */
    long    debug = 0;              /* debug printf flag                    */
    long    fatal=0;                /* fatal message flag                   */
    long    nonfatal = 1;           /* nonfatal flag                        */
    long    std_out = 0;            /* logical unit for standard ouput      */
    long    one = 1;                /* value of one                         */
    long    three = 3;              /* value of three                       */
    long    four = 4;               /* value of four                        */
    long    msgtim = 0;             /* elapsed time between display msg  */
    long    msg_flag=ALLCAT;        /* flag for porcessing message          */
    long    bipflag = 0;            /* data is really BIP flag              */
    long    odtype=EBYTE;           /* Output data tpe                      */
    long    opt[MAXBND];            /* option for eopens                    */


    struct  DDR ddr_struct;         /* LAS ddr structure                    */
    struct  BDDR bddr_struct[5];    /* LAS band independent ddr info        */

    char    quad_name[39];          /* These are all receivers of header    */
    char    quad[3];                /* fields.  Refer to the USGS Document  */
    char    nation[5];              /* "Standards for Digital Orthophotos". */
    char    tmpstr[40];
    char    prod_code[5];
    char    xy_units_str[12];

    long    nl = 0;
    long    ns = 0;
    long    nb = 0;
    long    data_type = 0;
    long    band_code = 0;
    long    hor_datum = 0;
    long    ref_sys = 0;
    long    zone_code = 0;
    long    xy_units = 0;
    long    nw_line_coord = 0;
    long    nw_samp_coord = 0;

    double  tmp_dbl;
    double  rot_angle;
    double  sw_x_coord;
    double  sw_y_coord;
    double  nw_x_coord;
    double  nw_y_coord;
    double  ne_x_coord;
    double  ne_y_coord;
    double  se_x_coord;
    double  se_y_coord;
    double  rec2_field;
    double  upper_left_line;
    double  upper_left_samp;
    double  upper_right_line;
    double  upper_right_samp;
    double  lower_left_line;
    double  lower_left_samp;
    double  lower_right_line;
    double  lower_right_samp;
    double  x_pixel_resolution;
    double  y_pixel_resolution;

    char deg[4];
    char lon[4];
    char proj_opt[100];
    char datum[6];               /* ASCII value for datum "NAD27" or "NAD83" */


    strcpy (deg,"DEG");
    strcpy (lon,"LON");

    /* Initialize TAE parameter block
       ----------------------------------*/
    c_inlas(&parblk,&std_out);

    /* Get LAS parameters (if needed)
       ---------------------------------*/
    if ((getpar(&parblk,doqname,hosin[0],tname,datum,proj_opt,&msgtim))
            != E_SUCC)
    {
        c_errmsg("Fatal error encountered","doq2las-fatal",&fatal);
    }

    /* Create some file names 
       -------------------------------------*/
    strcpy (hostdoq_name,doqname);
    c_splitname(hosin[0],dir,root,ext);
    strcpy (hostimg_name,dir);
    strcat (hostimg_name,root);
    strcpy (hosout[0],hostimg_name);
    strcat (hosout[0],".img");
    strcpy(hostdhd_name,hostimg_name);
    strcat(hostdhd_name,".dhd");
    strcpy (hosout[1],hostdhd_name);
    strcpy(hostddr_name,hostimg_name);
    strcat(hostddr_name,".ddr");
    strcpy(hosout[2],hostddr_name);
    strcpy(hosout[3],hostimg_name);
    strcat(hosout[3],".his");

    if ((strcmp(proj_opt,"TM") != 0) && (strcmp(proj_opt,"UTM") != 0))
    {
        c_errmsg("Invalid projection option","doq2las-proj",&fatal);
    }

    strcpy (filename,hostdoq_name);
    rec_size = REC_SIZE;
    char_buf = (char *) malloc(rec_size);
    if (char_buf == NULL)
    {
        c_errmsg("Error allocating memory","doq2las-alloc",&fatal);
    }


    /* Open the input file
       -------------------*/
    ret_stat = (input_fd = open(hostdoq_name, O_RDONLY));
    if (ret_stat == -1)
    {
        c_errmsg("Error opening doq file","doq2las-open",&fatal);
    }

    /* Read the first record
       ---------------------*/
    ret_stat = read(input_fd, char_buf, rec_size);
    if (ret_stat != rec_size)
    {
        close(input_fd);
        c_errmsg("Error reading doq file","doq2las-read",&fatal);
    }

    /* Convert the text read into field values
       ---------------------------------------*/
    strncpy(quad_name, &char_buf[  0], 38); quad_name[38] = '\0';
    strncpy(quad,      &char_buf[ 38],  2); quad[2]       = '\0';
    strncpy(nation,    &char_buf[ 40],  4); nation[4]     = '\0';
    strncpy(prod_code, &char_buf[137],  4);

    strncpy(tmpstr,    &char_buf[144],  6); tmpstr[ 6] = '\0';
    nl = atoi(tmpstr);

    strncpy(tmpstr,    &char_buf[150],  6); tmpstr[ 6] = '\0';
    ns = atoi(tmpstr);

    strncpy(tmpstr,    &char_buf[156],  3); tmpstr[ 3] = '\0';
    band_code = atoi(tmpstr);
    switch (band_code)
    {
        case 1:                      /* 1 band black & white (B&W)   */
        case 2:                      /* 1 band red                   */
        case 3:                      /* 1 band green                 */
        case 4:                      /* 1 band blue                  */
            nb = 1;
            data_type = 1;
            bands[0][0]=1;
            break;
        case 5:                      /* 3 band color (RGB)           */
            nb = 3;
            data_type = 2;
            msg_flag=ONECAT;
            bipflag = 1;
            bands[0][0]=1;
            bands[0][1]=2;
            bands[0][2]=3;
            break;
        case 6:                      /* 3 band RGB + 1 band B&W      */
            nb = 4;
            data_type = 1;
            msg_flag=ONECAT;
            bands[0][0]=1;
            bands[0][1]=2;
            bands[0][2]=3;
            bands[0][3]=4;
            break;
        case 7:                      /* 1 band B&W + elevation data  */
            nb = 2;
            data_type = 2;
            close(input_fd);
            c_errmsg("Unsupported B&W+Elevation data","doq2las-supp",&fatal);
            break;
        case 8:                      /* 3 band RGB + elevation data  */
            nb = 4;
            data_type = 2;
            close(input_fd);
            c_errmsg("Unsupported RGB+Elevation data","doq2las-supp",&fatal);
            break;
        case 9:                   /* 3 band RGB + 1 band B&W + elevation data */
            nb = 5;
            data_type = 2;
            close(input_fd);
            c_errmsg("Unsupported RGB+B&W+Elevation data","doq2las-supp",
                     &fatal);
            break;
    }
    strncpy(tmpstr,    &char_buf[167],  2); tmpstr[ 2] = '\0';
    hor_datum = atoi(tmpstr);

    /* The principal primary horizontal datum for digital orthophoto
       quarter-quadrangles shall be NAD 83 (code=4) and the principal secondary
       horizontal datum shall be the past principal horizontal datum for the
       geographic area (i.e., NAD 27, Old Hawaiian, Puerto Rico). The principal
       primary and secondary horizontal datum for digital orthophoto quadrangles
       shall be NAD 83, NAD 27, or other approved datums.
     */

    if (hor_datum == 1)                      /* doq header code for NAD 27 */
        hor_datum = 0;                           /* LAS ddr code for NAD 27 */
    else if(hor_datum == 2)                  /* doq header code for WGS 72 */
        hor_datum = 5;                           /* LAS ddr code for WGS 72 */
    else if(hor_datum == 3)                  /* doq header code for WGS 84 */
        hor_datum = 8;                               /* LAS WGS 84 = NAD 83 */
    else if(hor_datum == 4)                  /* doq header code for NAD 83 */
        hor_datum = 8;                           /* LAS ddr code for NAD 83 */
    else if(hor_datum == 5)                  /* doq header code for Old HA */
        hor_datum = 8;                            /* Call it NAD 83 for now */
    else if(hor_datum == 6)                  /* doq header code for P. R.  */
        hor_datum = 8;                            /* Call it NAD 83 for now */
    if (debug)
    {
        printf("horizontal datum is %d\n",hor_datum);
    }

    strncpy(tmpstr,    &char_buf[171], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rot_angle);

    strncpy(tmpstr,    &char_buf[195],  3); tmpstr[ 3] = '\0';
    ref_sys = atoi(tmpstr);

    strncpy(tmpstr,    &char_buf[198],  6); tmpstr[ 6] = '\0';
    zone_code = atoi(tmpstr); /* ignored anyway, TM used proj_coefs, 
                                 not zone_code */

    if (ref_sys == 0)                           /* doq header code for GEO */
    {
        ref_sys = 0;                                /* LAS ddr code for GEO */
        ddr_struct.valid[3] = VALID;
    }
    else if (ref_sys == 1)                      /* doq header code for UTM */
    {
        if (strcmp(proj_opt,"UTM") == 0) 
        {
            ref_sys = 1;                        /* LAS ddr code for UTM */
            ddr_struct.valid[3] = VALID;

        }
        else
        {

            /* LAS assumes that all UTM's are NAD 27, so a UTM NAD 83 will not 
               work, therefore, I will make a TM with NAD 83 and the 
               corresponding ddr proj info that is valid for TM, which WILL 
               work with LAS reprojections. If this becomes an issue later, 
               just put in a flag to optionally convert to TM, rather than 
               always doing so. */
            ref_sys = 9;                       /* LAS ddr code for TM */
            ddr_struct.proj_coef[ 0] = 6378137.0; /* Semi-major axis 
                                                     of ellipsoid */
            ddr_struct.proj_coef[ 1] = 6356752.31414;  /* Semi-minor axis 
                                                          of ellipsoid */
            ddr_struct.proj_coef[ 2] = 0.9996;  /* Scale factor at central 
                                                   meridian */
            tmp_dbl = (6 * abs(zone_code)) - 183;
            /* Longitude of Central Meridian, in degrees */
            c_degdms(&tmp_dbl,&ddr_struct.proj_coef[4],deg,lon);
            ddr_struct.proj_coef[ 6] = 500000.0;         /* False Easting */
            ddr_struct.proj_coef[ 7] = (zone_code < 0) ? 10000000.0 : 0.0;
                                                         /*False N'ing*/
            ddr_struct.valid[3] = VALID;
        }
    }
    else if (ref_sys == 2)                       /* doq header code for SPCS */
    {
        ref_sys = 2;                             /* LAS ddr code for SPCS */
        ddr_struct.valid[3] = VALID;
    }

    strncpy(tmpstr,    &char_buf[204],  3); tmpstr[ 3] = '\0';
    xy_units = atoi(tmpstr);
    if (c_unit2string(xy_units, xy_units_str) != E_SUCC)
    {
        /* c_unit2string displays the error message */
        c_lasmsg(FALSE, "doq2las-warn", "Set xy_units_str to empty");
        strcpy (xy_units_str, " ");
    }

    strncpy(tmpstr,    &char_buf[207], 24); tmpstr[24] = '\0';
    length = (long)strcspn(tmpstr, "D");
    if (length < strlen(tmpstr))
        tmpstr[length] = capitalE;
    sscanf(tmpstr,"%lf",&sw_x_coord);

    strncpy(tmpstr,    &char_buf[231], 24); tmpstr[24] = '\0';
    length = (long)strcspn(tmpstr, "D");
    if (length < strlen(tmpstr))
        tmpstr[length] = capitalE;
    sscanf(tmpstr,"%lf",&sw_y_coord);

    strncpy(tmpstr,    &char_buf[255], 24); tmpstr[24] = '\0';
    length = (long)strcspn(tmpstr, "D");
    if (length < strlen(tmpstr))
        tmpstr[length] = capitalE;
    sscanf(tmpstr,"%lf",&nw_x_coord);

    strncpy(tmpstr,    &char_buf[279], 24); tmpstr[24] = '\0';
    length = (long)strcspn(tmpstr, "D");
    if (length < strlen(tmpstr))
        tmpstr[length] = capitalE;
    sscanf(tmpstr,"%lf",&nw_y_coord);

    strncpy(tmpstr,    &char_buf[303], 24); tmpstr[24] = '\0';
    length = (long)strcspn(tmpstr, "D");
    if (length < strlen(tmpstr))
        tmpstr[length] = capitalE;
    sscanf(tmpstr,"%lf",&ne_x_coord);

    strncpy(tmpstr,    &char_buf[327], 24); tmpstr[24] = '\0';
    length = (long)strcspn(tmpstr, "D");
    if (length < strlen(tmpstr))
        tmpstr[length] = capitalE;
    sscanf(tmpstr,"%lf",&ne_y_coord);

    strncpy(tmpstr,    &char_buf[351], 24); tmpstr[24] = '\0';
    length = (long)strcspn(tmpstr, "D");
    if (length < strlen(tmpstr))
        tmpstr[length] = capitalE;
    sscanf(tmpstr,"%lf",&se_x_coord);

    strncpy(tmpstr,    &char_buf[375], 24); tmpstr[24] = '\0';
    length = (long)strcspn(tmpstr, "D");
    if (length < strlen(tmpstr))
        tmpstr[length] = capitalE;
    sscanf(tmpstr,"%lf",&se_y_coord);


    /* Reset buffer to correct record size (image sample size)
       -------------------------------------------------------*/
    if (bipflag != 1)
        rec_size = ns;
    else
        rec_size = nb*ns;
    free(char_buf);
    char_buf = (char *) malloc(rec_size);
    if (char_buf == NULL)
    {
        close(input_fd);
        c_errmsg("Error allocating memory","doq2las-alloc",&fatal);
    }

    /* Rewind the file
       --------------*/
    ret_stat = (long)lseek(input_fd,0,SEEK_SET);
    if (ret_stat != 0)
    {
        close(input_fd);
        c_errmsg("Error rewinding doq file","doq2las-rewind",&fatal);
    }

    /* Open the output doq header file
       ---------------------------------*/
    strcpy(hostdhd_name,hostimg_name);
    strcat(hostdhd_name,".dhd");
    if ((output_dh = fopen(hostdhd_name,"w")) == NULL)
    {
        close(input_fd);
        c_errmsg("Error opening doq header file","doq2las-open",&fatal);
    }

    /* Read the entire first record to get to the start of the second record
       ---------------------------------------------------------------------*/
    ret_stat = read(input_fd, char_buf, rec_size);
    if (ret_stat != rec_size)
    {
        c_errmsg("Error reading doq file","doq2las-read",&nonfatal);
        close(input_fd);
        fclose(output_dh);
        c_clean(&parblk,"DOQ2LAS",(char (*)[256])hosout[1],&one);
    }

    char_buf[hrec_size]='\n';
    ret_stat = (long)fwrite(char_buf,sizeof(char),hrec_size+1, output_dh);
    if (ret_stat != hrec_size+1)
    {
        c_errmsg("Error writing doq header file","doq2las-write",&nonfatal);
        close(input_fd);
        fclose(output_dh);
        c_clean(&parblk,"DOQ2LAS",(char (*)[256])hosout[1],&one); 
    }

    /* Read the next record
       --------------------*/
    ret_stat = read(input_fd, char_buf, rec_size);
    if (ret_stat != rec_size)
    {
        c_errmsg("Error reading doq file","doq2las-read",&nonfatal);
        close(input_fd);
        fclose(output_dh);
        c_clean(&parblk,"DOQ2LAS",(char (*)[256])hosout[1],&one);
    }

    char_buf[hrec_size]='\n';
    ret_stat = (long)fwrite(char_buf,sizeof(char),hrec_size+1, output_dh);
    if (ret_stat != hrec_size+1)  
    {
        c_errmsg("Error writing doq header file","doq2las-write",&nonfatal);
        close(input_fd);
        fclose(output_dh);
        c_clean(&parblk,"DOQ2LAS",(char (*)[256])hosout[1],&one);
    }

    strncpy(tmpstr,    &char_buf[  0], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rec2_field);
    strncpy(tmpstr,    &char_buf[ 24], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rec2_field);
    strncpy(tmpstr,    &char_buf[ 48], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rec2_field);
    strncpy(tmpstr,    &char_buf[ 72], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rec2_field);
    strncpy(tmpstr,    &char_buf[ 96], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rec2_field);
    strncpy(tmpstr,    &char_buf[120], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rec2_field);
    strncpy(tmpstr,    &char_buf[144], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rec2_field);
    strncpy(tmpstr,    &char_buf[168], 24); tmpstr[24] = '\0';
    sscanf(tmpstr,"%lf",&rec2_field);

    /* Read the next record
       --------------------*/
    ret_stat = read(input_fd, char_buf, rec_size);
    if (ret_stat != rec_size)
    {
        c_errmsg("Error reading doq file","doq2las-read",&nonfatal);
        close(input_fd);
        fclose(output_dh);
        c_clean(&parblk,"DOQ2LAS",(char (*)[256])hosout[1],&one);
    }
    char_buf[hrec_size]='\n';
    ret_stat = (long)fwrite(char_buf,sizeof(char),hrec_size+1, output_dh);
    if (ret_stat != hrec_size+1)  
    {
        c_errmsg("Error writing doq header file","doq2las-write",&nonfatal);
        close(input_fd);
        fclose(output_dh);
        c_clean(&parblk,"DOQ2LAS",(char (*)[256])hosout[1],&one);
    }

    strncpy(tmpstr,    &char_buf[204], 6); tmpstr[ 6] = '\0';
    nw_line_coord = atoi(tmpstr);
    strncpy(tmpstr,    &char_buf[210], 6); tmpstr[ 6] = '\0';
    nw_samp_coord = atoi(tmpstr);

    upper_left_line = nw_y_coord + nw_line_coord - 1;
    upper_left_samp = nw_x_coord - nw_samp_coord + 1;

    upper_right_line = upper_left_line;
    upper_right_samp = upper_left_samp + ns - 1;

    lower_left_line = upper_left_line - nl + 1;
    lower_left_samp = upper_left_samp;

    lower_right_line = upper_left_line - nl + 1;
    lower_right_samp = upper_left_samp + ns - 1;


    /* Read the next record
       --------------------*/
    ret_stat = read(input_fd, char_buf, rec_size);
    if (ret_stat != rec_size)
    {
        c_errmsg("Error reading doq file","doq2las-read",&nonfatal);
        close(input_fd);
        fclose(output_dh);
        c_clean(&parblk,"DOQ2LAS",(char (*)[256])hosout[1],&one);
    }
    char_buf[hrec_size]='\n';
    ret_stat = (long)fwrite(char_buf,sizeof(char),hrec_size+1, output_dh);
    if (ret_stat != hrec_size+1)  
    {
        c_errmsg("Error writing doq header file","doq2las-write",&nonfatal);
        close(input_fd);
        fclose(output_dh);
        c_clean(&parblk,"DOQ2LAS",(char (*)[256])hosout[1],&one);
    }

    strncpy(tmpstr,    &char_buf[ 59], 12); tmpstr[12] = '\0';
    sscanf(tmpstr,"%lf",&x_pixel_resolution);

    strncpy(tmpstr,    &char_buf[ 71], 12); tmpstr[12] = '\0';
    sscanf(tmpstr,"%lf",&y_pixel_resolution);


    ddr_struct.nl = nl;
    ddr_struct.ns = ns;
    ddr_struct.nbands = nb;
    ddr_struct.dtype = 1;

    ddr_struct.master_line = 0;
    ddr_struct.master_sample = 0;

    ddr_struct.valid[0] = VALID;
    ddr_struct.valid[1] = VALID;
    ddr_struct.valid[2] = VALID;
    ddr_struct.valid[4] = VALID;
    ddr_struct.valid[5] = VALID;
    ddr_struct.valid[7] = VALID;

    ddr_struct.proj_code = ref_sys; 
    ddr_struct.zone_code = zone_code;
    ddr_struct.datum_code = hor_datum; 
    ddr_struct.spare = 0; 

    strcpy(ddr_struct.system,"ieee-std");
    strcpy(ddr_struct.proj_units,xy_units_str);

    if (c_string2unit(xy_units_str) != E_FAIL)
    {
        strcpy(ddr_struct.proj_units,xy_units_str);
    }
    else
    {
        c_lasmsg(FALSE, "doq2las-units", "Projection units %s are not "
                "supported by LAS. Setting projection units to invalid",
                xy_units_str);
        strcpy(ddr_struct.proj_units,"");
        ddr_struct.valid[4] = INVAL;
    }

    if ((nw_line_coord !=0) && (nw_samp_coord != 0))
    {
        ddr_struct.upleft[0] = upper_left_line;
        ddr_struct.upleft[1] = upper_left_samp;
        ddr_struct.loleft[0] = lower_left_line;
        ddr_struct.loleft[1] = lower_left_samp;
        ddr_struct.upright[0] = upper_right_line;
        ddr_struct.upright[1] = upper_right_samp;
        ddr_struct.loright[0] = lower_right_line;
        ddr_struct.loright[1] = lower_right_samp;
        ddr_struct.valid[DDCCV] = VALID;
    }

    ddr_struct.pdist_y = y_pixel_resolution;
    ddr_struct.pdist_x = x_pixel_resolution;

    ddr_struct.line_inc = 1.0;
    ddr_struct.sample_inc = 1.0;

    strcpy(filename,hostimg_name);

    length = (long)strcspn(filename, ".");

    while (length != strlen(filename))
    {
        filename[length] = underscore;
        length = (long)strcspn(filename, ".");
    }

    /* Close the DOQ header file
       -----------------------------*/
    ret_stat = fclose(output_dh);
    if (ret_stat != 0)
    {
        c_errmsg("Error closing doq header file","doq2las-close",&nonfatal);
        close(input_fd);
        c_clean(&parblk,"DOQ2LAS",hosout,&three);
    }


    /* BSQ processing section
       --------------------------*/
    if (data_type == 1)
    {

        /* Open the output file
           --------------------*/
        opt[0] = 0;
        ret_stat = c_eopens(&output_fd,hosin[0],bands[0],&one,&one,&nl,&ns,
                offset, &acc,&odtype,&fatal,opt);
        if (ret_stat == -1)
        {
            c_errmsg("Error opening image file","doq2las-open",&nonfatal);
            close(input_fd);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }

        /* Setup for processing message utility
           ---------------------------------------*/
        c_inittm("DOQ2LAS","Processed","line","bands"," ",&nl,&nb,&msgtim,
                 &msg_flag);
        curcat = 1;
        curent = 1;
        c_startm();


        /* Now read the remaining records of the file as image data, writing 
           out to the image disk file
           ----------------------------------------------------------------*/
        for (jj = 1; jj <= nb; jj++)
        {
            curent = 1;
            for (ii = 1; ii <= nl; ii++)
            {
                ret_stat = read(input_fd, char_buf, rec_size);
                if (ret_stat != rec_size)
                {
                    c_errmsg("Error reading doq file","doq2las-read",&nonfatal);
                    close(input_fd);
                    c_eclose(&output_fd);
                    c_clean(&parblk,"DOQ2LAS",hosout,&three);
                }

                ret_stat = c_ewrite(&output_fd,&jj,&ii,
                                    (unsigned char *)char_buf,&one); 
                if (ret_stat != rec_size)
                {
                    c_errmsg("Error writing image file","doq2las-write",
                              &nonfatal);
                    close(input_fd);
                    c_eclose(&output_fd);
                    c_clean(&parblk,"DOQ2LAS",hosout,&three);
                }
                curent++;
            }
            curcat++;
        }

        free(char_buf);

        /* Turn off display messages
           ----------------------------*/
        c_stoptm();

        ret_stat = close(input_fd);
        if (ret_stat != 0)
        {
            c_errmsg("Error closing doq file","doq2las-close",&nonfatal);
            c_eclose(&output_fd);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }

        ret_stat = c_eclose(&output_fd);
        if (ret_stat != 0)
        {
            c_errmsg("Error closing image file","doq2las-close",&nonfatal);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }

    }
    else /* data type = 2 ie stored bip*/
    {

        /* close the doq file to allow stripdoqbip to use it
           ----------------------------------------------------*/
        ret_stat = close(input_fd);
        if (ret_stat != 0)
        {
            c_errmsg("Error closing doq file","doq2las-close",&nonfatal);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }
        rec_size=4*ns*nb;
        stripdoqbip(doqname,hosin[0],&ns,&nl,&nb,&rec_size);
    }

    /* Write out the ddr information
       ----------------------------------*/
    ret_stat = c_putddr(hostddr_name, &ddr_struct);
    if (ret_stat != 0)
    {
        c_errmsg("Error returned from c_putddr","doq2las-ddr",&nonfatal);
        c_clean(&parblk,"DOQ2LAS",hosout,&three);
    }

    for (ii = 0; ii < ddr_struct.nbands; ii++)
    {
        bddr_struct[ii].bandno = ii + 1;
        bddr_struct[ii].valid = INVAL;

        ret_stat = c_putbdr(hostddr_name, &bddr_struct[ii]);
        if (ret_stat != 0)
        {
            c_errmsg("Error returned from c_putbdr","doq2las-ddr",&nonfatal);
            c_clean(&parblk,"DOQ2LAS",hosout,&three);
        }
    }

    ret_stat = c_lused(hostddr_name);
    if (ret_stat != 0)
    {
        c_errmsg("Error returned from c_lused","doq2las-lused",&nonfatal);
    }

    /* Create the history file
       ---------------------------*/
    ret_stat = c_hstout(&parblk,hosout[0],hosin,&one,"doq2las");
    if (ret_stat != 0)
    {
        c_errmsg("Error returned from c_hstout","doq2las-lused",&nonfatal);
    }


    c_complt("DOQ2LAS",hosout,&four);
}
