/*****************************************************************************
NAME: GETPAR

PURPOSE: Get the input parameters

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
     See inline comments
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "doq2las.h"

int getpar
(
    struct PARBLK *parblk,        /* I: TAE parameter block */
    char *doqname,                /* O: Input DOQ file name */
    char *out,                    /* O: Output host image name */
    char *tname,                  /* O: Output TAE image name */
    char *datum,                  /* O: Datum type ("NAD27" or "NAD83") */
    char *proj,                   /* O: Projection type */
    long *msgtim                  /* O: Delay time between display messages */
)
{
    long length;                  /* Variable to store length of a string */
    long status;                  /* Status flag */

    struct VARIABLE *val;


    /* Get the output image name
       ----------------------------*/
    status = c_getout(parblk,"OUT",(char (*)[256])tname,(char (*)[256])out);
    if (status == E_FAIL)
        c_lasmsg(TRUE,"doq2las-fatal","Fatal error encountered");

    /* Get the input DOQ name
       -------------------------*/
    val = p_find(parblk,"DOQNAME");
    strcpy(doqname,SVAL(*val,0));

    /* Get the datum type
       --------------------------*/
    val = p_find(parblk,"DATUM");
    strcpy(datum, SVAL(*val,0));

    /* Get the projection type
       --------------------------*/
    val = p_find(parblk,"PROJ");
    strcpy(proj, SVAL(*val,0));

    /* The header can have upper and lower case.
       Change to upper for consistancy 
       ----------------------------------------*/
    length = (long)strlen(proj);
    c_low2up(proj,&length);


    /* Retrieve globals
       -------------------*/
    val = p_find(parblk,"$msgtim");
    *msgtim = IVAL(*val,0);

    return(E_SUCC);
}
