/*******************************************************************************
NAME:                   INBIL   

PURPOSE:
INBIL reads records from the input BIL file(s) specified and writes them
 to a LAS image file.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    INDAL must be run under TAE.

PROJECT:                        LAS

ALGORITHM REFERENCES:           none
******************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <math.h>
#include "las.h"
#include "doq2las.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc[MAXIMG+1];    /* image file descriptors used for image I/O */
extern long curent;             /* current processing line                   */
extern long curcat;             /* current processing band                   */

void inbil
(
    struct PARBLK *vblock,         /* I: TAE parameter block */
    char *hosout,                  /* I: Host name of output image */
    long nl,                       /* I: Number of lines */
    long ns,                       /* I: Number of samples */
    long nbands,                   /* I: Number of bands */
    long nheader,                  /* I: Number of header bytes */
    char ioname[][CMLEN],          /* I: Array of input image names */
    long odtype,                   /* I: Output data type */
    long msgtim,                   /* I: Message delay time */
    char *minmax,                  /* O: Min/Max flag */
    long dflag                     /* I: DDR flag option */
)
{
    char    hostdoq_name[CMLEN];    /* doqname                            */
    long    acc=IWRITE;             /* Image file open access             */
    long    block;                  /* Loop counter for i/o blocking      */
    long    test=0;                 /* return tester                      */   
    long    block2;                 /* Loop counter for i/o blocking      */
    long    fatal = -1;             /* fatal error flag                   */
    long    flag = ALLCAT;          /* Display message flag               */
    long    i;                      /* index counter                      */
    FILE    *infd;                  /* Input file descriptor              */
    long    nbytes;                 /* number of bytes per image line     */
    long    nonfatal = 1;           /* nonfatal error flag                */
    long    one = 1;                /* the number one for errors          */
    static  long    opt[3]={0,0,0}; /* Image file open option             */
    long    size;                   /* Number of bytes/pixel              */
    long    sl=1;                   /* Image file starting line           */
    long    ss=1;                   /* Image fiel starting sample         */
    long    status;                 /* Status indicator                   */
    char    minmax2[4];             /* local minmax flag                  */

    double *minval;         /* min pixel value array              */
    double *maxval;         /* max pixel value array              */


    char    *thebuf;                /* Input buffer                       */
    char    *header_rec;            /* Header record buffer               */

    struct  DDR ddr;                /* ddr structure                      */
    struct  BDDR bddr[MAXBND];      /* band ddr structure                 */

    /*diable minmax processing for this routine */
    strcpy(minmax2,"NO");
    strcpy(hostdoq_name,ioname[0]);

    if ((minval = (double *) malloc(MAXBND * sizeof(double))) == NULL)
    {
        c_errmsg("Error allocating dynamic memory","doq2las-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","doq2las-fatal",&fatal);
    }

    if ((maxval = (double *) malloc(MAXBND * sizeof(double))) == NULL)
    {
        c_errmsg("Error allocating dynamic memory","doq2las-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","doq2las-fatal",&fatal);
    }

    /* Initialize min/max arrays
       ----------------------------------------*/
    if (strcmp(minmax2,"YES") == 0)
        c_inmnmx(&odtype,&nbands,minval,maxval);

    /* Determine the number bytes/pixel
       ----------------------------------------*/
    switch(odtype)
    {
        case EBYTE:  size = 1; break;
        case EWORD:  size = 2; break;
        case ELONG:   size = 4; break;
        case EREAL:  size = 4; break;
    }


    /* Open the image file for write access
       -------------------------------------------------*/
    status = c_eopenr(&fdesc[0], hosout, &nbands, &sl, &ss, &nl, &ns, &acc,
            &odtype, opt, &one);    
    if ( status != E_SUCC)
        c_errmsg("Fatal error encountered", "doq2las-fatal",&fatal);

    /* Allocate the input buffer
       -------------------------------------------------*/
    if ((thebuf = (unsigned char *) malloc(ns*size)) == NULL)
    {
        c_errmsg("Error allocating dynamic memory","doq2las-alloc", &nonfatal);
        if (c_eclose(fdesc) != E_SUCC)
            c_errmsg("Nonfatal error encountered","doq2las-warn",&nonfatal);
        c_clean(vblock,"doq2las",hosout,&one);
    }
    if (nheader != 0)
    {
        if ((header_rec = (unsigned char *) malloc(nheader)) == NULL)
        {
            c_errmsg("Error allocating dynamic memory","doq2las-alloc", 
                     &nonfatal);
            if (c_eclose(fdesc) != E_SUCC)
                c_errmsg("Nonfatal error encountered","doq2las-warn",&nonfatal);
            c_clean(vblock,"doq2las",hosout,&one);
        }
    }

    /* Initialize the processing message
       ----------------------------------------*/
    curcat=1;
    curent=1;
    c_inittm("DOQIN","Processed","line","bands"," ",&nl, &nbands,&msgtim,&flag);
    c_startm();


    /* Open the input file
       ----------------------*/
    test= (infd = open(hostdoq_name, O_RDONLY));
    if (test == -1)
    {
        if (c_eclose(fdesc) != E_SUCC)
            c_errmsg("Nonfatal error encountered","doq2las-warn", &nonfatal);
        c_errmsg("Error opening input TRANSFER image", "doq2las-inopen",
                 &nonfatal);
        c_clean(vblock,"doq2las",hosout,&one);
    }

    /* Read header record
       ------------------*/
    if (nheader != 0)
    {
        status = read(infd,header_rec,nheader); 
        if (status != nheader)
        {
            if (c_eclose(fdesc) != E_SUCC)
                c_errmsg("Nonfatal error encountered","doq2las-warn", 
                         &nonfatal);
            c_errmsg("Error reading header record of input image",
                    "doq2las-header",&fatal);
            c_clean(vblock,"doq2las",hosout,&one);
        }
    }

    /* For each band in the input file read a line from the DAL file and write
       it to the LAS image file
       ----------------------------------------------------------------------*/
    nbytes = ns * size;
    for (block=0; block < nl; block++, curent++)
    {
        block2 = block + 1;
        for (curcat=1; curcat<=nbands; curcat++)
        {
            if (read(infd,thebuf,nbytes) != nbytes)
            {
                if (c_eclose(fdesc) != E_SUCC)
                    c_errmsg("Nonfatal error encountered","doq2las-warn", 
                             &nonfatal);
                c_errmsg("Error reading input TRANSFER image","doq2las-read",
                        &nonfatal);
                c_clean(vblock,"doq2las",hosout,&one);
            }
            if (strcmp(minmax2,"YES") == 0)
            {
                status = c_pxmnmx(thebuf, &ns, &minval[curcat-1], 
                        &maxval[curcat-1], &odtype);
                if (status != E_SUCC)
                    c_errmsg("Nonfatal error encountered","doq2las-warn",
                             &nonfatal);
            }
            status = c_ewrite(&fdesc[0], &curcat, &block2, thebuf, &one);
            if (status == E_FAIL)
            {
                if (c_eclose(fdesc) != E_SUCC)
                    c_errmsg("Nonfatal error encountered","doq2las-warn", 
                              &nonfatal);
                c_clean(vblock,"doq2las",hosout,&one);
            }
        }                 /* end of curcat loop */
    }                 /* end of block loop */

    /* Free the buffer space previously allocated
       -------------------------------------------------*/
    free(thebuf);

    /* Free the buffer space previously allocated
       -------------------------------------------------*/
    if (nheader != 0)
    {
        free(header_rec);
    }

    /* Update the DDR record file
       -------------------------------------------------*/
    status = c_getddr(hosout,&ddr);
    for (i = 1; i < nbands+1; i++)
        status = c_getbdr(hosout, &bddr[i-1], &i);

    for (i = 0; i < 8; i++)
        ddr.valid[i] = dflag;

    status = c_putddr(hosout,&ddr);

    for (i = 0; i < nbands; i++)
        status = c_putbdr(hosout, &bddr[i]);

    /* Update the associated min/max file
       -------------------------------------------------*/
    if (c_svmnmx (hosout,&nbands,minval,maxval,minmax) != E_SUCC)
        c_errmsg("Nonfatal error encountered","doq2las-warn",&nonfatal);
    free(minval);
    free(maxval);

    /* Close the input and output files
       -------------------------------------------------*/
    close(infd);
    c_eclose(fdesc);
    c_stoptm();
} /* End of inbil */
