/*******************************************************************
    Strip new DOQ 
    pulls apart DOQ into seperate files
    If color then three out files  "r,g,b"
    Then combine the three files for one BSQ file
*********************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "doq2las.h"

extern long curent;             /* current processing line               */
extern long curcat;             /* current processing band               */

void stripdoqbip
(
    char *infile,                  /* I: Input DOQ file */
    char *outfile,                 /* I: Output LAS image file */
    long *ns,                      /* I: Number of samples */
    long *nl,                      /* I: Number of lines */
    long *nb,                      /* I: Number of bands */
    long *hb                       /* I: Header bytes */
)
{
    FILE  *fpin;                    /* input file pointer                   */
    struct FDESC *output_fd;        /* output file pointer                  */

    long  msgtim = 10;              /*elapsed time between display messages */
    long  msg_flag=GEN2;            /* flag for processing message          */
    long  fatal = 0;                /* fatal error flag                     */
    long  ii=0;                     /* increment counters                   */
    long  jj=0;                     /* increment counter                    */
    long  kk=0;                     /* increment counter                    */
    long  one = 1;                  /* constant of one                      */
    long  odtype=EBYTE;             /* data type                            */
    long  acc=IWRITE;               /* access indicator                     */
    long  ret_stat = 0;             /* return status                        */
    long  tnl,tns,tnb,mm,count;     /* temp longs                           */
    long  opt[MAXBND];              /* option for eopenr                    */
    long  line = 0;                 /* current line to write                */
    char  *buf[4];                  /* hold line buffer                     */
    char  *in_buf;                  /* read hold buffer                     */

    if ((fpin=fopen(infile,"rb")) == NULL) 
        c_errmsg("Error opening infile","doq2las-open",&fatal);

    /*rewind(fpin);*/
    ret_stat = fseek(fpin,*hb,0);
    tns = *ns;  
    tnb = *nb; 

    /* Handle Color Band interleaved by pixel  
       ---------------------------------------------*/
    for (ii=0;ii<*nb;ii++)
    {
        buf[ii] = (char *) malloc(sizeof(char)*tns);
        if (buf[ii] == NULL)
        {
            fclose(fpin);
            c_errmsg("Error allocating memory","doq2las-alloc",&fatal);
        }
    } 

    in_buf = (char *) malloc (tns * tnb);
    if (in_buf == NULL)
    {
        fclose(fpin);
        c_errmsg("Error allocating memory","doq2las-alloc",&fatal);
    }

    opt[0] = 0;
    tnl = *nl;
    tns = *ns;
    tnb = *nb;
    count = tns * tnb;
    ret_stat = c_eopenr(&output_fd,outfile,&tnb,&one,&one,&tnl,&tns,&acc,
            &odtype,opt,&one);
    if ( ret_stat != NULL)
    {
        fclose(fpin);
        c_errmsg("Error opening output file","doq2las-open",&fatal);
    }

    /* Setup for processing message utility
       -----------------------------------------*/
    c_inittm("DOQ2LAS","Processed","line","bands"," ",nl,nb,&msgtim,&msg_flag);
    curcat = 1;
    curent = 1;
    c_startm(); 

    for (ii=0;ii<*nl;ii++) 
    {
        ret_stat = (long)fread(in_buf,sizeof(char),count,fpin);
        if (ret_stat != count)
        {
            fclose(fpin);
            c_eclose(&output_fd);
            c_errmsg("Error reading from input file","doq2las-read",&fatal);
        }

        for (jj=0;jj<*ns;jj++) 
        {
            for (kk=0;kk<*nb;kk++) 
            {
                mm = (jj*tnb) + kk;
                buf[kk][jj] = in_buf[mm];
            }
        }
        for (kk=0;kk<*nb;kk++)
        {
            line = ii+1;
            tnb = kk + 1;
            ret_stat = c_ewrite(&output_fd,&tnb,&line,(unsigned char *)buf[kk],
                                &one);
            if (ret_stat == -1)
            {
                fclose(fpin);
                c_eclose(&output_fd);
                c_errmsg("Error writing to output file","doq2las-write",&fatal);
            }
        }
        curent++;
    }
    /* Turn off display messages
       ----------------------------*/
    c_stoptm();

    fclose(fpin);
    c_eclose(&output_fd);
}
