/***************************************************************************
NAME:			DSPANC	

PURPOSE:  Displays EDIPS associated record information

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  5.0		1/93	T. Mittan	Original Development

***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "edips.h"
#include "header.h"
#include "util.h"
#include "labserv.h"
#include "protos.h"

int dspanc
(
    char *hname,			/* filename				*/
    long *acdsk,			/* ancillary flag			*/
    FILE *fptr			/* file pointer				*/
)

{
struct EDPSAC anc;		/* EDIPS annotation file structure	*/
struct EDPSAC_EPH anceph;	/* EDIPS ancillary file ephemeris data	*/
struct EDPSAC_ATT ancatt;	/* EDIPS ancillary file attitude data	*/
struct EDPSAC_RS  ancutm;	/* EDIPS ancillary file horizontal and vertical
				   resampling data for UTM/PS map projections */
struct EDPSAC_RS  ancsom;	/* EDIPS ancillary file horizontal and vertical
				  resampling data for SOM/HOM map projections */

long nonfatal = 1;		/* non fatal error flag			*/
long status;
long outsys;                    /* output system                        */
long sflag = SETN;              /* Flag for SYSSET routine              */
long i;
long j;


char tempfile[CMLEN];
char syst[DDSYLN];              /* system character string              */

/* get current system code
   ----------------------*/
status = c_getsys(syst);
if (status != E_SUCC)
   {
   c_errmsg("Error retreiving current system","dspassoc-system",&nonfatal);
   return(E_FAIL);
   }

status = c_sysset(syst, &outsys, &sflag);
if (status != E_SUCC)
   {
   c_errmsg("Error retreiving current system","dspassoc-system",&nonfatal);
   return(E_FAIL);
   }

/*
fseek(fptr,0,1);
*/
	/* Begin setup of ancillary files
	   -----------------------------*/
if (*acdsk)
  {
  fprintf(fptr,"\nAncillary file information \n");
  fprintf(fptr,"__________________________ \n \n");
  if ((status = c_edips_anc(hname, outsys, &anc, &anceph, &ancatt, &ancutm,
							&ancsom)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }
  c_lsmknm(hname,"edips_ancil",tempfile);

  fprintf(fptr,"Ancillary filename     %s \n \n", tempfile);

  fprintf(fptr,"\nRecord 1 \n");
  fprintf(fptr,"________ \n");

	/* RECORD 1
   	   -------*/
  fprintf(fptr,"r1_record_length       %d \n",anc.r1_record_length);
  fprintf(fptr,"r1_record_number       %d \n",anc.r1_record_number);
  fprintf(fptr,"r1_record_type_code    %d \n",anc.r1_record_type_code);
  fprintf(fptr,"input_pixels_per_line  %d \n",anc.input_pixels_per_line);
  fprintf(fptr,"input_lines_per_image  %d \n",anc.input_lines_per_image);
  fprintf(fptr,"input_inter_pixel_dis  %f \n",anc.input_inter_pixel_dis);
  fprintf(fptr,"input_inter_line_dis   %f \n",anc.input_inter_line_dis);
  fprintf(fptr,"output_pixels_per_line %d \n",anc.output_pixels_per_line);
  fprintf(fptr,"output_lines_per_image %d \n",anc.output_lines_per_image);
  fprintf(fptr,"output_inter_pixl_dis  %f \n",anc.output_inter_pixl_dis);
  fprintf(fptr,"output_inter_line_dis  %f \n",anc.output_inter_line_dis);
  fprintf(fptr,"spacecraft_altitude    %f \n",anc.spacecraft_altitude);
  fprintf(fptr,"input_image_width      %f \n",anc.input_image_width);
  for (i = 0; i < 4; i++)
    fprintf(fptr,"mss_mirror_coef[%d]     %f \n",i,anc.mss_mirror_coef[i]);
  fprintf(fptr,"mss_max_mirror_ang     %f \n",anc.mss_max_mirror_ang);
  fprintf(fptr,"scan_skew_constant     %f \n",anc.scan_skew_constant);
  fprintf(fptr,"tim_bet_mirror_sweeps  %f \n",anc.tim_bet_mirror_sweeps);
  fprintf(fptr,"tim_active_mirror_swp  %f \n",anc.tim_active_mirror_swp);
  fprintf(fptr,"semi_major_axis        %f \n",anc.semi_major_axis);
  fprintf(fptr,"semi_minor_axis        %f \n",anc.semi_minor_axis);
  fprintf(fptr,"earth_curvature_const  %8.10f \n",anc.earth_curvature_const);
for (i = 0; i < 24; i++)
  {
  fprintf(fptr,"mss_samp_delay_const[%d] %f \n",i,anc.mss_samp_delay_const[i]);
  }
for (i = 0; i < 3; i++)
  {
  fprintf(fptr,"mss_band_offsets[%d]     %f \n",i,anc.mss_band_offsets[i]);
  }

	/* RECORD 2
           -------*/
  fprintf(fptr,"\nRecord 2 \n");
  fprintf(fptr,"________ \n");
  fprintf(fptr,"r2_record_length        %d \n",anc.r2_record_length);
  fprintf(fptr,"r2_record_number        %d \n",anc.r2_record_number);
  fprintf(fptr,"r2_record_type_code     %d \n",anc.r2_record_type_code);
  fprintf(fptr,"wrs_frame_number        %s \n",anc.wrs_frame_number);
  fprintf(fptr,"wrs_orbit_number        %s \n",anc.wrs_orbit_number);
  fprintf(fptr,"wrs_ctr_lat             %f \n",anc.wrs_ctr_lat);
  fprintf(fptr,"wrs_ctr_long            %f \n",anc.wrs_ctr_long);
  fprintf(fptr,"center_time             %s \n",anc.center_time);
  fprintf(fptr,"scene_ctr_lat           %f \n",anc.scene_ctr_lat);
  fprintf(fptr,"scene_ctr_long          %f \n",anc.scene_ctr_long);
  for (i = 0; i < 3; i++)
     {
     fprintf(fptr,"sc_earth_cntrd_coords[%d]  %f \n",i,
						anc.sc_earth_cntrd_coords[i]);
     }
  fprintf(fptr,"sc_heading_angle        %f \n",anc.sc_heading_angle);
  fprintf(fptr,"sc_scan_line_coord      %f \n",anc.sc_scan_line_coord);
  fprintf(fptr,"sc_pixel_coord          %f \n",anc.sc_pixel_coord);
  fprintf(fptr,"sc_norm_velocity_err    %f \n",anc.sc_norm_velocity_err);
  fprintf(fptr,"sc_earth_rotat_velcty   %f \n",anc.sc_earth_rotat_velcty);
  fprintf(fptr,"earth_rotation_param    %f \n",anc.earth_rotation_param);

	/* Spacecraft state vector at scene center
	   --------------------------------------*/
  fprintf(fptr,"\nSpacecraft state vector at scene center \n");
  fprintf(fptr,"__________________________________________\n\n");

  fprintf(fptr,"sc_state_vector_pitch   %f \n",anc.sc_state_vector_pitch);
  fprintf(fptr,"sc_state_vector_roll    %f \n",anc.sc_state_vector_roll);
  fprintf(fptr,"sc_state_vector_yaw     %f \n",anc.sc_state_vector_yaw);
  fprintf(fptr,"sc_state_vector_x       %f \n",anc.sc_state_vector_x);
  fprintf(fptr,"sc_state_vector_y       %f \n",anc.sc_state_vector_y);
  fprintf(fptr,"sc_state_vector_z       %f \n",anc.sc_state_vector_z);
  fprintf(fptr,"sc_s_vctr_delta_pitch   %8.10f \n",anc.sc_s_vctr_delta_pitch);
  fprintf(fptr,"sc_s_vctr_delta_roll    %8.10f \n",anc.sc_s_vctr_delta_roll);
  fprintf(fptr,"sc_s_vctr_delta_yaw     %8.10f \n",anc.sc_s_vctr_delta_yaw);
  fprintf(fptr,"sc_s_vctr_delta_x       %f \n",anc.sc_s_vctr_delta_x);
  fprintf(fptr,"sc_s_vctr_delta_y       %f \n",anc.sc_s_vctr_delta_y);
  fprintf(fptr,"sc_s_vctr_delta_z       %f \n",anc.sc_s_vctr_delta_z);
  fprintf(fptr,"cps_in_attitude         %d \n",anc.cps_in_attitude);
  fprintf(fptr,"gcps_in_attitude	%d \n",anc.gcps_in_attitude);
  fprintf(fptr,"cp_correlatns_attempt   %d \n",anc.cp_correlatns_attempt);
  fprintf(fptr,"cps_reject_correlatns   %d \n",anc.cps_reject_correlatns);
  fprintf(fptr,"rms_along_track_error   %f \n",anc.rms_along_track_error);
  fprintf(fptr,"rms_across_track_error  %f \n",anc.rms_across_track_error);
  for (i = 0; i < 25; i++)
   {
   fprintf(fptr,"cps_per_zone[%d]         %d \n",i,anc.cps_per_zone[i]);
   fprintf(fptr,"cp_identificatn[%d]      %s \n",i,anc.cp_identificatn[i]);
   }

	/* Geometric Correction Parameters Ephemeris Data
	   ---------------------------------------------*/
  fprintf(fptr,"\nGeometric Correction Parameters Ephemeris Data \n");
  fprintf(fptr,"______________________________________________ \n \n");
  fprintf(fptr,"time_of_first_set       %s \n",anceph.time_of_first_set);
  fprintf(fptr,"interval_between_sets   %f \n",anceph.interval_between_sets);
  fprintf(fptr,"number_of_sets          %d \n",anceph.number_of_sets);
  for (i = 0; i < 16; i++)
   {
   fprintf(fptr,"spccrft_locatn_x[%d]     %f \n",i,anceph.spccrft_locatn_x[i]);
   fprintf(fptr,"spccrft_locatn_y[%d]     %f \n",i,anceph.spccrft_locatn_y[i]);
   fprintf(fptr,"spccrft_locatn_z[%d]     %f \n",i,anceph.spccrft_locatn_z[i]);
   fprintf(fptr,"spccrft_velocity_x[%d]   %f \n",i,
						anceph.spccrft_velocity_x[i]);
   fprintf(fptr,"spccrft_velocity_y[%d]   %f \n",i,
						anceph.spccrft_velocity_y[i]);
   fprintf(fptr,"spccrft_velocity_z[%d]   %f \n",i,
						anceph.spccrft_velocity_z[i]);
   fprintf(fptr,"data_quality[%d]         %d \n",i,anceph.data_quality[i]);
   }
	/* Geometric Correction Parameters Attitude Data
	   ---------------------------------------------*/
  fprintf(fptr,"\nGeometric Correction Parameters Attitude Data \n");
  fprintf(fptr,"_____________________________________________ \n \n");

  fprintf(fptr,"time_of_first_set       %s \n",ancatt.time_of_first_set);
  fprintf(fptr,"interval_between_sets   %f \n",ancatt.interval_between_sets);
  fprintf(fptr,"number_of_sets          %d \n",ancatt.number_of_sets);

		/* 60 sets of attitude entries
		   --------------------------*/
  for (i = 0; i < 60; i++)
   {
   fprintf(fptr,"pitch_angle[%d]           %8.10f \n",i,ancatt.pitch_angle[i]);
   fprintf(fptr,"roll_angle[%d]            %8.10f \n",i,ancatt.roll_angle[i]);
   fprintf(fptr,"yaw_angle[%d]             %8.10f \n",i,ancatt.yaw_angle[i]);
   fprintf(fptr,"data_quality[%d]          %d \n",i,ancatt.data_quality[i]);
   }
		/* Partial Derivatives for SOM projection
		   --------------------------------------*/
  fprintf(fptr,"\nPartial Derivatives for SOM projection \n");
  fprintf(fptr,"______________________________________ \n");
  for (i = 0; i < 3; i++)
   {
   for (j = 0; j < 5; j++)
    {
    fprintf(fptr,"\n");
    fprintf(fptr,"pd_along_trk_x[%d][%d]     %8.10f \n",i,j,
						ancatt.pd_along_trk_x[i][j]);
    fprintf(fptr,"pd_along_trk_y[%d][%d]     %8.10f \n",i,j,
    						ancatt.pd_along_trk_y[i][j]);
    fprintf(fptr,"pd_acros_trk_x[%d][%d]     %8.10f \n",i,j,
    						ancatt.pd_acros_trk_x[i][j]);
    fprintf(fptr,"pd_acros_trk_y[%d][%d]     %8.10f \n",i,j,
    						ancatt.pd_acros_trk_y[i][j]);
    fprintf(fptr,"pd_altitude_x[%d][%d]      %8.10f \n",i,j,
    						ancatt.pd_altitude_x[i][j]);
    fprintf(fptr,"pd_altitude_y[%d][%d]      %8.10f \n",i,j,
    						ancatt.pd_altitude_y[i][j]);
    fprintf(fptr,"pd_pitch_x[%d][%d]         %8.10f \n",i,j,
    						ancatt.pd_pitch_x[i][j]);
    fprintf(fptr,"pd_pitch_y[%d][%d]         %8.10f \n",i,j,
						ancatt.pd_pitch_y[i][j]);
    fprintf(fptr,"pd_roll_x[%d][%d]          %8.10f \n",i,j,
						ancatt.pd_roll_x[i][j]);
    fprintf(fptr,"pd_roll_y[%d][%d]          %8.10f \n",i,j,
						ancatt.pd_roll_y[i][j]);
    fprintf(fptr,"pd_yaw_x[%d][%d]           %8.10f \n",i,j,
						ancatt.pd_yaw_x[i][j]);
    fprintf(fptr,"pd_yaw_y[%d][%d]           %8.10f \n",i,j,
						ancatt.pd_yaw_y[i][j]);
    }
   }
	/* Multiplicative and additive radiometric correction constants
  	   -----------------------------------------------------------*/
   fprintf(fptr,"\nMultiplicative and additive radiometric correction \n");
   fprintf(fptr,"__________________________________________________ \n");
  for (i = 0; i < 4; i++)
   for (j = 0; j < 6; j++)
    {
    fprintf(fptr,"rc_mult_band_det[%d][%d]   %f \n",i,j,
    						ancatt.rc_mult_band_det[i][j]);
    fprintf(fptr,"rc_add_band_det[%d][%d]    %f \n",i,j,
    						ancatt.rc_add_band_det[i][j]);
    }

/* Find Map Projection Formats for HRS pixel Coordinates and
   VRS pixel Coordinates--data taken from 16 records
   ---------------------------------------------------------*/
  fprintf(fptr,"\nMap Projection HRS and VRS Coordinates \n");
  fprintf(fptr,"______________________________________ \n \n");
   /* UTM/PS data
      ----------*/
  fprintf(fptr,"\nUTM/PS data--records 4 - 11 \n");
  fprintf(fptr,"___________________________ \n");

  fprintf(fptr,"rn4_rec_type_code        %d \n",ancutm.rn4_rec_type_code);
  fprintf(fptr,"rn5_rec_type_code        %d \n",ancutm.rn5_rec_type_code);
  fprintf(fptr,"rn6_rec_type_code        %d \n",ancutm.rn6_rec_type_code);
  fprintf(fptr,"rn7_rec_type_code        %d \n",ancutm.rn7_rec_type_code);
  fprintf(fptr,"rn8_rec_type_code        %d \n",ancutm.rn8_rec_type_code);
  fprintf(fptr,"rn9_rec_type_code        %d \n",ancutm.rn9_rec_type_code);
  fprintf(fptr,"rn10_rec_type_code       %d \n",ancutm.rn10_rec_type_code);
  fprintf(fptr,"rn11_rec_type_code       %d \n",ancutm.rn11_rec_type_code);

  for (i = 0; i < 51; i++)
   {
   for (j = 0; j < 61; j++)
    fprintf(fptr,"hrs_row_coord[%d][%d]      %f \n",i,j,
						ancutm.hrs_row_coord[i][j]);
   fprintf(fptr,"hrs_ln_fill_left[%d]      %f \n",i,ancutm.hrs_ln_fill_left[i]);
   fprintf(fptr,"hrs_ln_fill_rgt[%d]       %f \n",i,ancutm.hrs_ln_fill_rgt[i]); 
   fprintf(fptr,"\n");
   }

  for (i = 0; i < 44; i++)
   for (j = 0; j < 61; j++)
    fprintf(fptr,"vrs_row_coord[%d][%d]      %f \n",i,j,
						ancutm.vrs_row_coord[i][j]);


  fprintf(fptr,"\n");
  fprintf(fptr,"pixel_wrs_cntr           %d \n",ancutm.pixel_wrs_cntr);
  fprintf(fptr,"offset_wrs_cntr          %d \n",ancutm.offset_wrs_cntr);
  fprintf(fptr,"temp_reg_scene_id        %s \n",ancutm.temp_reg_scene_id);
  for (i = 0; i < 4; i++)
   {
   fprintf(fptr,"tr_cur_imag_line[%d]      %d \n",i,ancutm.tr_cur_imag_line[i]);
   fprintf(fptr,"tr_cur_imag_pixel[%d]     %d \n",i,
						ancutm.tr_cur_imag_pixel[i]);
   fprintf(fptr,"tr_ref_imag_line[%d]      %d \n",i,ancutm.tr_ref_imag_line[i]);
   fprintf(fptr,"tr_ref_imag_pixel[%d]     %d \n",i,
						ancutm.tr_ref_imag_pixel[i]);
   }
  fprintf(fptr,"first_overlap_line       %d \n",ancutm.first_overlap_line);
  fprintf(fptr,"first_overlap_pixel      %d \n",ancutm.first_overlap_pixel);
  fprintf(fptr,"second_overlap_line      %d \n",ancutm.second_overlap_line);
  fprintf(fptr,"second_overlap_pixel     %d \n",ancutm.second_overlap_pixel);
  fprintf(fptr,"third_overlap_line       %d \n",ancutm.third_overlap_line);
  fprintf(fptr,"third_overlap_pixel      %d \n",ancutm.third_overlap_pixel);
  fprintf(fptr,"fourth_overlap_line      %d \n",ancutm.fourth_overlap_line);
  fprintf(fptr,"fourth_overlap_pixel     %d \n",ancutm.fourth_overlap_pixel);

  fprintf(fptr,"tick_marks_top           %d \n",ancutm.tick_marks_top);
  fprintf(fptr,"tick_marks_left          %d \n",ancutm.tick_marks_left);
  fprintf(fptr,"tick_marks_right         %d \n",ancutm.tick_marks_right);
  fprintf(fptr,"tick_marks_bottom        %d \n",ancutm.tick_marks_bottom);

  for (i = 0; i < 4; i++)
   fprintf(fptr,"samp_corner_pts[%d]       %d \n",i,ancutm.samp_corner_pts[i]);

  fprintf(fptr,"image_orientatn_ang      %8.10f \n",
						ancutm.image_orientatn_ang);
  fprintf(fptr,"num_sweeps               %d \n",ancutm.num_sweeps);

/* SOM/HOM data
   -----------*/
  fprintf(fptr,"\nSOM/HOM data--records 12 - 19 \n");
  fprintf(fptr,"_____________________________ \n");

  fprintf(fptr,"rn4_rec_type_code        %d \n",ancsom.rn4_rec_type_code);
  fprintf(fptr,"rn5_rec_type_code        %d \n",ancsom.rn5_rec_type_code);
  fprintf(fptr,"rn6_rec_type_code        %d \n",ancsom.rn6_rec_type_code);
  fprintf(fptr,"rn7_rec_type_code        %d \n",ancsom.rn7_rec_type_code);
  fprintf(fptr,"rn8_rec_type_code        %d \n",ancsom.rn8_rec_type_code);
  fprintf(fptr,"rn9_rec_type_code        %d \n",ancsom.rn9_rec_type_code);
  fprintf(fptr,"rn10_rec_type_code       %d \n",ancsom.rn10_rec_type_code);
  fprintf(fptr,"rn11_rec_type_code       %d \n",ancsom.rn11_rec_type_code);

  for (i = 0; i < 51; i++)
   {
   for (j = 0; j < 61; j++)
    fprintf(fptr,"hrs_row_coord[%d][%d]      %f \n",i,j,
						ancsom.hrs_row_coord[i][j]);
   fprintf(fptr,"hrs_ln_fill_left[%d]      %f \n",i,ancsom.hrs_ln_fill_left[i]);
   fprintf(fptr,"hrs_ln_fill_rgt[%d]       %f \n",i,ancsom.hrs_ln_fill_rgt[i]); 
   fprintf(fptr,"\n");
   }

  for (i = 0; i < 44; i++)
   for (j = 0; j < 61; j++)
    fprintf(fptr,"vrs_row_coord[%d][%d]      %f \n",i,j,
						ancsom.vrs_row_coord[i][j]);


  fprintf(fptr,"\n");
  fprintf(fptr,"pixel_wrs_cntr           %d \n",ancsom.pixel_wrs_cntr);
  fprintf(fptr,"offset_wrs_cntr          %d \n",ancsom.offset_wrs_cntr);
  fprintf(fptr,"temp_reg_scene_id        %s \n",ancsom.temp_reg_scene_id);
  for (i = 0; i < 4; i++)
   {
   fprintf(fptr,"tr_cur_imag_line[%d]      %d \n",i,ancsom.tr_cur_imag_line[i]);
   fprintf(fptr,"tr_cur_imag_pixel[%d]     %d \n",i,
						ancsom.tr_cur_imag_pixel[i]);
   fprintf(fptr,"tr_ref_imag_line[%d]      %d \n",i,ancsom.tr_ref_imag_line[i]);
   fprintf(fptr,"tr_ref_imag_pixel[%d]     %d \n",i,
						ancsom.tr_ref_imag_pixel[i]);
   }
  fprintf(fptr,"first_overlap_line       %d \n",ancsom.first_overlap_line);
  fprintf(fptr,"first_overlap_pixel      %d \n",ancsom.first_overlap_pixel);
  fprintf(fptr,"second_overlap_line      %d \n",ancsom.second_overlap_line);
  fprintf(fptr,"second_overlap_pixel     %d \n",ancsom.second_overlap_pixel);
  fprintf(fptr,"third_overlap_line       %d \n",ancsom.third_overlap_line);
  fprintf(fptr,"third_overlap_pixel      %d \n",ancsom.third_overlap_pixel);
  fprintf(fptr,"fourth_overlap_line      %d \n",ancsom.fourth_overlap_line);
  fprintf(fptr,"fourth_overlap_pixel     %d \n",ancsom.fourth_overlap_pixel);

  fprintf(fptr,"tick_marks_top           %d \n",ancsom.tick_marks_top);
  fprintf(fptr,"tick_marks_left          %d \n",ancsom.tick_marks_left);
  fprintf(fptr,"tick_marks_right         %d \n",ancsom.tick_marks_right);
  fprintf(fptr,"tick_marks_bottom        %d \n",ancsom.tick_marks_bottom);

  for (i = 0; i < 4; i++)
   fprintf(fptr,"samp_corner_pts[%d]       %d \n",i,ancsom.samp_corner_pts[i]);

  fprintf(fptr,"image_orientatn_ang      %8.10f \n",
						ancsom.image_orientatn_ang);
  fprintf(fptr,"num_sweeps               %d \n",ancsom.num_sweeps);

  }		/* end of c_edips_anc */

fclose(fptr);

return(E_SUCC);
}		/* end of dspedips */
