/******************************************************************************
NAME:                           DSPASSOC

PURPOSE:      Display the contents of associated files. 

PROGRAM HISTORY:
PROGRAMMER              DATE            REASON
----------              ----            ------
D. Etrheim	       12/92	       Initial development
T. Mittan              12/92           Modified to more generally display
				       contents of associated files 
S. Nelson		1/93	       Added the option to display EDIPS 
				       ancillary file.
S. Nelson		6/94	       Added version 2 SIAT capabilities for
				       CCT-X.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "tipsp.h"
#include "util.h"
#include "protos.h"

#define TERM 0
#define PFILE 1
#define LP 2

int main
(void)
{
char filename[1][CMLEN];	/* print file name				*/
char hname[CMLEN];      /* TAE name of input image                    	*/
char prtfile[CMLEN];    /* print file name                            	*/
char option[10];	/* subcomand option				*/
char tmpstr[1][CMLEN];  /* temporary string */

long fatal = -1;
long print[3];		/* print option					*/
long hdsk = FALSE;	/* header file flag			 	*/
long tdsk = FALSE;	/* trailer file flag			 	*/
long andsk = FALSE;	/* annotation file flag			 	*/
long acdsk = FALSE;	/* ancillary file flag				*/
long uhdsk = FALSE;	/* universal header flag			*/
long lhdsk = FALSE;	/* LANDSAT header flag				*/
long vdsk = FALSE; 	/* volume directory file flag			*/
long ldsk = FALSE;	/* leader file flag				*/
long sdsk = FALSE;	/* supplemental file flag			*/
long idsk = FALSE;	/* image data record flag			*/
long nl;		/* line number for JSC and LTWG			*/
long one = 1;		/* value of one					*/
long zero = 0;		/* value of zero				*/
FILE *fptr;		/* file pointer					*/


/* get parameters
---------------*/
getpar(hname,prtfile,&hdsk,&tdsk,&andsk,&acdsk,&uhdsk,&lhdsk,&vdsk,&ldsk,
	&sdsk,&idsk,filename[0],option,&nl,print);

/* call appropriate subroutine to read file(s)
--------------------------------------------*/
if (strcmp(option,"FFMT") == 0)
    {
    if (dsthdr(hname,prtfile) == E_FAIL)
      c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);
    }
else
if (strcmp(option,"TIPSP") == 0)
   {
   if (dsptips(hname,prtfile,&hdsk,&tdsk,&andsk) == E_FAIL)
      c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);
   }
else
if (strcmp(option,"SIAT") == 0)
   {
   if (dspsiat(hname,prtfile) == E_FAIL)
      c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);
   }
else
if (strcmp(option,"EDIPS") == 0)
   {
   /* open print file
   ----------------*/
   fptr=fopen(prtfile,"w");
   if(!fptr)
      {
      c_lasmsg(TRUE,"dspassoc-print","Error opening print file");
      }

   if (dspedips(hname,&hdsk,&tdsk,&andsk,fptr) == E_FAIL)
      c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);
   if (dspanc(hname,&acdsk,fptr) == E_FAIL)
      c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);
   }
else
if (strcmp(option,"JSC") == 0)
   {
   if (dspjsc(hname,prtfile,&uhdsk,&lhdsk,&acdsk,&nl) == E_FAIL)
      c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);
   }
else
if (strcmp(option,"LTWG") == 0)
   {
   /* open print file
   ----------------*/
   fptr=fopen(prtfile,"w");
   if(!fptr)
      {
      c_lasmsg(TRUE,"dspassoc-print","Error opening print file");
      }

   if (dspltwg(hname, fptr, &vdsk, &ldsk, &tdsk, &sdsk, &idsk, &nl) 
	== E_FAIL)
      c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);
   fclose(fptr);
   }


/* print completion message
-------------------------*/
if (print[PFILE])
   c_complt("DSPASSOC",filename,&one);
else
{
   strcpy(tmpstr[0],"\0");  
   c_complt("DSPASSOC",tmpstr,&zero);
}


return EXIT_SUCCESS;

}
