/***************************************************************************
NAME:			DSPEDIPS	

PURPOSE:  Displays EDIPS associated record information

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  5.0		1/93	T. Mittan	Original Development

***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "edips.h"
#include "header.h"
#include "labserv.h"
#include "util.h"
#include "protos.h"

int dspedips
(
    char *hname,			/* filename				*/
    long *hdsk,			/* header flag				*/
    long *tdsk,			/* trailer flag				*/
    long *andsk,			/* annotation flag			*/
    FILE *fptr			/* file pointer				*/
)

{
struct EDIPSH hdr;   		/* EDIPS header structure               */
struct EDIPST trl;		/* EDIPS trailer structure		*/
struct EDIPSA annot;		/* EDIPS annotation structure		*/

long nonfatal = 1;		/* non fatal error flag			*/
long status;
long outsys;                    /* output system                        */
long sflag = SETN;              /* Flag for SYSSET routine              */
long i;
long j;


char tempfile[CMLEN];
char syst[DDSYLN];              /* system character string              */

/* get current system code
   ----------------------*/
status = c_getsys(syst);
if (status != E_SUCC)
   {
   c_errmsg("Error retreiving current system","dspassoc-system",&nonfatal);
   return(E_FAIL);
   }

status = c_sysset(syst, &outsys, &sflag);
if (status != E_SUCC)
   {
   c_errmsg("Error retreiving current system","dspassoc-system",&nonfatal);
   return(E_FAIL);
   }

if (*hdsk)
  {
  if ((status = c_edips_hdr(hname, outsys, &hdr)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"Header file information \n");
  fprintf(fptr,"_______________________ \n \n");
  c_lsmknm(hname,"edips_headr",tempfile);

  fprintf(fptr,"Header filename     %s \n \n", tempfile);
  /* A. Scene Identification
  -------------------------*/
  fprintf(fptr,"\nScene information \n");
  fprintf(fptr,"_________________ \n");
  fprintf(fptr,"record_number       %d \n",hdr.record_number);
  fprintf(fptr,"record_type_code    %d \n",hdr.record_type_code);
  fprintf(fptr,"record_length       %d \n",hdr.record_length);
  fprintf(fptr,"image_id            %s \n",hdr.image_id);
  fprintf(fptr,"wrs_designator      %s \n",hdr.wrs_designator);
  fprintf(fptr,"tape_generation_day %d \n",hdr.tape_generation_day);
  fprintf(fptr,"tape_generation_mon %d \n",hdr.tape_generation_mon);
  fprintf(fptr,"tape_generation_yr  %d \n",hdr.tape_generation_yr);

    /* B. Spacecraft Description
    ---------------------------*/
  fprintf(fptr,"\nSpacecraft information \n");
  fprintf(fptr,"______________________ \n");
  fprintf(fptr,"sensor_id           %s \n",hdr.sensor_id);
  fprintf(fptr,"mission_number      %d \n",hdr.mission_number);
  fprintf(fptr,"orbit_number        %d \n",hdr.orbit_number);

		/* bitwise munipulation with 4 bytes 1=active, 0=inactive
		   put in temporary storage at first since bits and bands
		   do not correlate */

  for (j = 0; j < 5; j++)
     for (i = 0; i < 6; i++)
        fprintf(fptr,"status_bnd_detect[%d][%d] %d \n",j,i,
                                hdr.status_bnd_detect[j][i]);

  fprintf(fptr,"pix_original_image %d \n",hdr.pix_original_image);

    /* C. Time of Exposure/WRS Designator
    ------------------------------------*/
  fprintf(fptr,"\nTime of Exposure/WRS Designator information \n");
  fprintf(fptr,"___________________________________________ \n");
  fprintf(fptr,"wrs_scan_line       %d \n",hdr.wrs_scan_line);
  fprintf(fptr,"wrs_pixel_number    %d \n",hdr.wrs_pixel_number);
  fprintf(fptr,"center_year         %s \n",hdr.center_year);
  fprintf(fptr,"center_julian_day   %s \n",hdr.center_julian_day);
  fprintf(fptr,"center_hour         %s \n",hdr.center_hour);
  fprintf(fptr,"center_min          %s \n",hdr.center_min);
  fprintf(fptr,"center_sec          %s \n",hdr.center_sec);
  fprintf(fptr,"center_milliseconds %s \n",hdr.center_milliseconds);

   /* D. Data Identification and Characteristics
   ---------------------------------------------*/
  fprintf(fptr,"\nData identification and characteristics \n");
  fprintf(fptr,"_______________________________________ \n");
  fprintf(fptr,"header_length          %d \n",hdr.header_length);
  fprintf(fptr,"number_header_records  %d \n",hdr.number_header_records);
  fprintf(fptr,"header_bytes           %d \n",hdr.header_bytes);

	/* Annotation Data Characteristics
	---------------------------------*/
  fprintf(fptr,"annotat_length         %d \n",hdr.annotat_length);
  fprintf(fptr,"number_annotat_records %d \n",hdr.number_annotat_records);

	/* Ancillary Data Characteristics
	--------------------------------*/
  fprintf(fptr,"ancill_length          %d \n",hdr.ancill_length);
  fprintf(fptr,"number_ancill_records  %d \n",hdr.number_ancill_records);
  fprintf(fptr,"geom_correct           %s \n",hdr.geom_correct);
  fprintf(fptr,"geom_data              %s \n",hdr.geom_data);
  fprintf(fptr,"radiometric_correct    %s \n",hdr.radiometric_correct);
  fprintf(fptr,"radiometric_data       %s \n",hdr.radiometric_data);

	/* Image Data Characteristics
	----------------------------*/
  fprintf(fptr,"image_record_length    %d \n",hdr.image_record_length);
  fprintf(fptr,"calibration_words_line %d \n",hdr.calibration_words_line);
  fprintf(fptr,"image_data_format      %s \n",hdr.image_data_format);

  fprintf(fptr,"interleav_type         %s \n",hdr.interleav_type);
  fprintf(fptr,"bil_interleaving_count %d \n",hdr.bil_interleaving_count);
  fprintf(fptr,"bits_per_pixel         %d \n",hdr.bits_per_pixel);
  fprintf(fptr,"resampling             %d \n",hdr.resampling);
  fprintf(fptr,"map_projection         %d \n",hdr.map_projection);

  fprintf(fptr,"wrs_center_offset      %d \n",hdr.wrs_center_offset);
  fprintf(fptr,"data_justification     %s \n",hdr.data_justification);
  fprintf(fptr,"most_signifcnt_bit     %s \n",hdr.most_signifcnt_bit);
  fprintf(fptr,"pixels_per_line        %d \n",hdr.pixels_per_line);
  fprintf(fptr,"images_per_scene       %d \n",hdr.images_per_scene);
  fprintf(fptr,"mss_band_number        %s \n",hdr.mss_band_number);
  fprintf(fptr,"support_data           %d \n",hdr.support_data);

	/* Trailer Data Characteristics
	------------------------------*/
  fprintf(fptr,"trailer_length         %d \n",hdr.trailer_length);
  fprintf(fptr,"number_trailer_record  %d \n",hdr.number_trailer_record);

	/* Special Purpose Fields
	------------------------*/
  fprintf(fptr,"orbit_direction        %s \n",hdr.orbit_direction);
		/* convert to double--landsat 4
		   ---------------------------*/
  fprintf(fptr,"orientation_angle      %8.10f \n",hdr.orientation_angle);
		/* all mission numbers
		   ------------------*/
  fprintf(fptr,"sensor_mode            %d \n",hdr.sensor_mode);
	/* Temporal Registration Data
	----------------------------*/
  fprintf(fptr,"\nTemporal registration information \n");
  fprintf(fptr,"_________________________________ \n");
  fprintf(fptr,"temporal_scene_id      %s \n",hdr.temporal_scene_id);
  fprintf(fptr,"temp_wrs_designator    %s \n",hdr.temp_wrs_designator);

   for (i = 0; i < 4; i++)
      {
      fprintf(fptr,"cur_imag_line[%d]       %d \n",i,hdr.cur_imag_line[i]);
      fprintf(fptr,"cur_imag_pixel[%d]      %d \n",i,hdr.cur_imag_pixel[i]);
      fprintf(fptr,"ref_imag_line[%d]       %d \n",i,hdr.ref_imag_line[i]);
      fprintf(fptr,"ref_imag_pixel[%d]      %d \n",i,hdr.ref_imag_pixel[i]);
      }
  fprintf(fptr,"first_overlap_line     %d \n",hdr.first_overlap_line);
  fprintf(fptr,"first_overlap_pixel    %d \n",hdr.first_overlap_pixel);
  fprintf(fptr,"second_overlap_line    %d \n",hdr.second_overlap_line);
  fprintf(fptr,"second_overlap_pixel   %d \n",hdr.second_overlap_pixel);
  fprintf(fptr,"third_overlap_line     %d \n",hdr.third_overlap_line);
  fprintf(fptr,"third_overlap_pixel    %d \n",hdr.third_overlap_pixel);
  fprintf(fptr,"fourth_overlap_line    %d \n",hdr.fourth_overlap_line);
  fprintf(fptr,"fourth_overlap_pixel   %d \n",hdr.fourth_overlap_pixel);
  fprintf(fptr,"overlap_pixel_offset   %d \n",hdr.overlap_pixel_offset);
  fprintf(fptr,"quality_code           %s \n",hdr.quality_code); 
  fprintf(fptr,"tick_marks_top         %d \n",hdr.tick_marks_top);   
  fprintf(fptr,"tick_marks_left        %d \n",hdr.tick_marks_left);   
  fprintf(fptr,"tick_marks_right       %d \n",hdr.tick_marks_right);   
  fprintf(fptr,"tick_marks_bottom      %d \n",hdr.tick_marks_bottom);   

	/* Data not available for landsat 1,2 and 3
	   ---------------------------------------*/
  fprintf(fptr,"band_quality           %s \n",hdr.band_quality); 
  fprintf(fptr,"radiometric_calibratn  %d \n",hdr.radiometric_calibratn);
  fprintf(fptr,"calibration_accuracy   %f \n",hdr.calibration_accuracy);

	/* Input Data Quality Indicators
	   ----------------------------*/
		/* Telemetry:
		------------*/
  fprintf(fptr,"\nInput data quality information \n");
  fprintf(fptr,"______________________________ \n");
  fprintf(fptr,"tel_ephemeris_points   %d \n",hdr.tel_ephemeris_points);
  fprintf(fptr,"tel_rejected_ephemeris %d \n",hdr.tel_rejected_ephemeris); 
  fprintf(fptr,"tel_attitude_points    %d \n",hdr.tel_attitude_points);
  fprintf(fptr,"tel_rejected_attitude  %d \n",hdr.tel_rejected_attitude);
  fprintf(fptr,"tel_interval_length    %f \n",hdr.tel_interval_length);
  fprintf(fptr,"tel_ephm_accuracy_alt  %f \n",hdr.tel_ephm_accuracy_alt);
  fprintf(fptr,"tel_ephm_acc_along_trck %f \n",hdr.tel_ephm_acc_along_trck);
  fprintf(fptr,"tel_ephm_acc_acros_trck %f \n",hdr.tel_ephm_acc_acros_trck);
  fprintf(fptr,"tel_att_accuracy_pitch %f \n",hdr.tel_att_accuracy_pitch);
  fprintf(fptr,"tel_att_accuracy_roll  %f \n",hdr.tel_att_accuracy_roll);
  fprintf(fptr,"tel_att_accuracy_yaw   %f \n",hdr.tel_att_accuracy_yaw);

		/* Control Points:
		   --------------*/
  fprintf(fptr,"\nControl point information \n");
  fprintf(fptr,"_________________________ \n");
for (i = 0; i < 4; i++)
   {
   fprintf(fptr,"cp_band_quality[%d]    %s \n",i,hdr.cp_band_quality[i]); 
   }
  fprintf(fptr,"cp_geodetic_points     %d \n",hdr.cp_geodetic_points);
  fprintf(fptr,"cp_registratn_succes   %d \n",hdr.cp_registratn_success);
  fprintf(fptr,"cp_autocorrelatn_init  %f \n",hdr.cp_autocorrelatn_init);
  fprintf(fptr,"cp_err_ellipse_along   %f \n",hdr.cp_err_ellipse_along);
  fprintf(fptr,"cp_err_ellipse_across  %f \n",hdr.cp_err_ellipse_across);
  fprintf(fptr,"cp_autocorrelatn_ref   %f \n",hdr.cp_autocorrelatn_ref);
  fprintf(fptr,"cp_suitability         %f \n",hdr.cp_suitability);

  fprintf(fptr,"data_source            %s \n",hdr.data_source); 
  fprintf(fptr,"uncorrect_ecc_count    %d \n",hdr.uncorrect_ecc_count);
  fprintf(fptr,"bit_error_rate         %d \n",hdr.bit_error_rate);
  fprintf(fptr,"cal_wedge_value_use    %d \n",hdr.cal_wedge_value_use);

for (i = 0; i < 36; i++)
  fprintf(fptr,"cal_wedge_values[%2d]   %d \n",i,hdr.cal_wedge_values[i]);

for (i = 0; i < 6; i++)
   for (j = 0; j < 6; j++)
  fprintf(fptr,"wed_qual_sens_samp[%d][%d] %d \n",i,j,
				hdr.wed_qual_sens_samp[i][j]);

  fprintf(fptr,"wrs_center_lat         %8.10f \n",hdr.wrs_center_lat);
  fprintf(fptr,"wrs_center_long        %8.10f \n",hdr.wrs_center_long);
  fprintf(fptr,"contrast_enhancement   %d \n",hdr.contrast_enhancement);
  fprintf(fptr,"atmospheric_scatter    %d \n",hdr.atmospheric_scatter);
  fprintf(fptr,"edge_enhancement       %d \n",hdr.edge_enhancement);

for (i = 0; i < 4; i++)
    {
      fprintf(fptr,"sensor_gain_opt[%d]   %s \n",i,hdr.sensor_gain_opt[i]); 
      fprintf(fptr,"transmission_type[%d] %s \n",i,hdr.transmission_type[i]); 
    }
  
  }
if (*tdsk)
  {
  fprintf(fptr,"\nTrailer file information \n");
  fprintf(fptr,"________________________ \n\n");
  if ((status = c_edips_trl(hname,&trl)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }
  c_lsmknm(hname,"edips_trail",tempfile);

  fprintf(fptr,"Trailer filename       %s \n \n", tempfile);

  fprintf(fptr,"record_number          %d \n",trl.record_number);
  fprintf(fptr,"record_type            %d \n",trl.record_type);
  fprintf(fptr,"last_scene_data_acqu   %d \n",trl.last_scene_data_acqu);
  fprintf(fptr,"last_scene_hdt         %d \n",trl.last_scene_hdt);
  fprintf(fptr,"destriped_image        %d \n",trl.destriped_image);
  fprintf(fptr,"contrast_stretch       %s \n",trl.contrast_stretch);
  fprintf(fptr,"minimum_contrast_enh   %d \n",trl.minimum_contrast_enh);
  fprintf(fptr,"maximum_contrast_enh   %d \n",trl.maximum_contrast_enh);
  fprintf(fptr,"radiance_value_scatter %d \n",trl.radiance_value_scatter);
  fprintf(fptr,"edge_enh_kernal_size_x %d \n",trl.edge_enh_kernal_size_x);
  fprintf(fptr,"edge_enh_kernal_size_y %d \n",trl.edge_enh_kernal_size_y);

  }		/* end of c_edips_trl */
if (*andsk)
  {
  fprintf(fptr,"\nAnnotation file information \n");
  fprintf(fptr,"___________________________ \n\n");
  if ((status = c_edips_ann(hname,&annot)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }
  c_lsmknm(hname,"edips_annot",tempfile);

  fprintf(fptr,"Annotation filename    %s \n \n", tempfile);

  fprintf(fptr,"record_number          %d \n",annot.record_number);
  fprintf(fptr,"record_type            %d \n",annot.record_type);
  fprintf(fptr,"exposure_day_mon_yr    %s \n",annot.exposure_day_mon_yr);
  fprintf(fptr,"format_ctr_lat_long    %s \n",annot.format_ctr_lat_long);
  fprintf(fptr,"path_row_identifier    %s \n",annot.path_row_identifier);
  fprintf(fptr,"wrs_ctr_lat_long       %s \n",annot.wrs_ctr_lat_long);
  fprintf(fptr,"sensor_band_id_code    %s \n",annot.sensor_band_id_code);
  fprintf(fptr,"transmission           %s \n",annot.transmission);
  fprintf(fptr,"sun_angles             %s \n",annot.sun_angles);
  fprintf(fptr,"correction_applied     %s \n",annot.correction_applied);
	/* Landsat 1-3 only */
  fprintf(fptr,"scale_of_image         %s \n",annot.scale_of_image);
  fprintf(fptr,"projection             %s \n",annot.projection);
  fprintf(fptr,"resample_algorithm     %s \n",annot.resample_algorithm);
  fprintf(fptr,"type_ephemeris_data    %s \n",annot.type_ephemeris_data);
  fprintf(fptr,"process_procedure      %s \n",annot.process_procedure);
  fprintf(fptr,"sensor_gain_opt        %s \n",annot.sensor_gain_opt);
  fprintf(fptr,"mss_transmission       %s \n",annot.mss_transmission);
  fprintf(fptr,"agency_and_project     %s \n",annot.agency_and_project);
  fprintf(fptr,"frame_id_number        %s \n",annot.frame_id_number);

/* Read in Tick Mark data
   ---------------------*/
  fprintf(fptr,"\n Tic mark information \n");
  fprintf(fptr,"______________________ \n\n");

   /* top/bottom edge of image */
  for (i = 0; i < 16; i++)
   {			/* top */
   fprintf(fptr,"tm_top_location[%2d]   %d \n",i, annot.tm_top_location[i]);
   fprintf(fptr,"tm_top_loctn_flg[%2d]  %s \n",i,annot.tm_top_loctn_flg[i]);
   fprintf(fptr,"tm_bot_location[%2d]   %d \n",i, annot.tm_bot_location[i]);
   fprintf(fptr,"tm_bot_loctn_flg[%2d]  %s \n",i,annot.tm_bot_loctn_flg[i]);
   fprintf(fptr,"tm_top_coord[%2d]      %s \n",i,annot.tm_top_coord[i]);
   fprintf(fptr,"tm_bot_coord[%2d]      %s \n",i,annot.tm_bot_coord[i]);
   }

   /* left/right edge of image (two separate sections) */
  for (i = 0; i < 18; i++)
   {
   fprintf(fptr,"tm_left_location[%2d]  %d \n",i,annot.tm_left_location[i]);
   fprintf(fptr,"tm_left_loctn_flg[%2d] %s \n",i,annot.tm_left_loctn_flg[i]);
   fprintf(fptr,"tm_rgt_location[%2d]   %d \n",i,annot.tm_rgt_location[i]);
   fprintf(fptr,"tm_rgt_loctn_flg[%2d]  %s \n",i,annot.tm_rgt_loctn_flg[i]);
   fprintf(fptr,"tm_left_coord[%2d]     %s \n",i,annot.tm_left_coord[i]);
   fprintf(fptr,"tm_rgt_coord[%2d]      %s \n",i,annot.tm_rgt_coord[i]);
   }

  for (i = 0; i < 7; i++)
   {
   fprintf(fptr,"tm_left_location[%d]   %d \n",i,annot.tm_left_location[i+18]);
   fprintf(fptr,"tm_left_loctn_flg[%d]  %s \n",i,
					annot.tm_left_loctn_flg[i+18]);
   fprintf(fptr,"tm_rgt_location[%d]    %d \n",i,annot.tm_rgt_location[i+18]);
   fprintf(fptr,"tm_rgt_loctn_flg[%d]   %s \n",i,annot.tm_rgt_loctn_flg[i+18]);
   fprintf(fptr,"tm_left_coord[%d]      %s \n",i,annot.tm_left_coord[i + 18]);
   fprintf(fptr,"tm_rgt_coord[%d]       %s \n",i,annot.tm_rgt_coord[i + 18]);
   }
  }		/* end of c_edips_ann */

return(E_SUCC);
}		/* end of dspedips */
