/***************************************************************************
NAME:			DSPJSC	

PURPOSE:  Displays JSC associated record information

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  5.0		1/93	T. Mittan	Original Development
  5.1		1/93	S. Nelson	Converted from EDIPS to JSC formats

***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "jsc.h"
#include "header.h"
#include "labserv.h"
#include "protos.h"

int dspjsc
(
    char *hname,			/* filename				*/
    char *prtfile,			/* print file name			*/
    long *uhdsk,			/* universal header flag		*/
    long *lhdsk,			/* landsat header flag			*/
    long *acdsk,			/* ancillary file flag			*/
    long *nl			/* line number of ancillary file	*/
)

{
FILE  *fptr;
struct JSCUH uhdr;   		/* JSC universal header structure       */
struct JSCLH lhdr;		/* JSC landsat header structure		*/
struct JSCRT jscrt4;		/* JSC landsat header Radiometric Transformation
				   band 4				*/
struct JSCRT jscrt5;		/* JSC landsat header Radiometric Transformation
				   band 5				*/
struct JSCRT jscrt6;		/* JSC landsat header Radiometric Transformation
				   band 6				*/
struct JSCRT jscrt7;		/* JSC landsat header Radiometric Transformation
				   band 7				*/
struct JSCRT jscrt8;		/* JSC landsat header Radiometric Transformation
				   band 8				*/
struct JSCIN jscanc;		/* JSC ancillary structure		*/

long nonfatal = 1;		/* non fatal error flag			*/
long i;
long j;


char tempfile[CMLEN];

/* open print file
----------------*/
fptr=fopen(prtfile,"w");
if(!fptr)
   {
   c_errmsg("Error opening print file","dspassoc-print",&nonfatal);
   return(E_FAIL);
   }

/* If Universal Header flag is on, get and display information
   ----------------------------------------------------------*/
if (*uhdsk)
  {
  if (c_jsc_uhdr(hname, &uhdr) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"Universal Header File Information \n");
  fprintf(fptr,"_________________________________ \n \n");
  c_lsmknm(hname,"uhdr",tempfile);
  fprintf(fptr,"Universal header filename     %s \n \n", tempfile);

  fprintf(fptr,"computing_sys_id         %s \n",uhdr.computing_sys_id);
  fprintf(fptr,"tape_library_id          %s \n",uhdr.tape_library_id);
  fprintf(fptr,"sensor_id		 %s \n",uhdr.sensor_id);

  fprintf(fptr,"master_tape_gen_day      %d \n",uhdr.master_tape_gen_day);
  fprintf(fptr,"master_tape_gen_mon	 %d \n",uhdr.master_tape_gen_mon);
  fprintf(fptr,"master_tape_gen_yr	 %d \n",uhdr.master_tape_gen_yr);
  fprintf(fptr,"tape_seq_number          %d \n",uhdr.tape_seq_number);
  fprintf(fptr,"mission_number	 	 %d \n",uhdr.mission_number);
  fprintf(fptr,"row_number		 %d \n",uhdr.row_number);
  fprintf(fptr,"path_number		 %d \n",uhdr.path_number);
  fprintf(fptr,"run_number		 %d \n",uhdr.run_number);
  fprintf(fptr,"orbit_number		 %d \n",uhdr.orbit_number);

  fprintf(fptr,"first_scan_millisec      %d \n",uhdr.first_scan_millisec);
  fprintf(fptr,"first_scan_seconds       %d \n",uhdr.first_scan_seconds);
  fprintf(fptr,"first_scan_minutes       %d \n",uhdr.first_scan_minutes);
  fprintf(fptr,"first_scan_hours         %d \n",uhdr.first_scan_hours);
  fprintf(fptr,"first_scan_days          %d \n",uhdr.first_scan_days);
  fprintf(fptr,"first_scan_month_num     %d \n",uhdr.first_scan_month_num);
  fprintf(fptr,"first_scan_year          %d \n",uhdr.first_scan_year);

		/* Active channels = 1, Other channels = 0 
		   --------------------------------------*/
  fprintf(fptr,"\nActive channels = 1, Other channels = 0\n"); 
  fprintf(fptr,"_______________________________________\n \n");
  for (i = 0; i < 64; i++)
    fprintf(fptr,"channels_active[%d]       %d \n",i,uhdr.channels_active[i]);
	
  fprintf(fptr,"\n");
  fprintf(fptr,"processing_flag          %d \n",uhdr.processing_flag);
  fprintf(fptr,"num_channels_on_tape     %d \n",uhdr.num_channels_on_tape);
  fprintf(fptr,"num_bits_in_picture      %d \n",uhdr.num_bits_in_picture);
  fprintf(fptr,"byt_loctn_strt_of_video_data %d \n",
					uhdr.byt_loctn_strt_of_video_data);
  fprintf(fptr,"byt_loctn_strt_of_calibratn  %d \n",
					uhdr.byt_loctn_strt_of_calibratn);
  fprintf(fptr,"num_video_elemnts_per_scan   %d \n",
					uhdr.num_video_elemnts_per_scan);
  fprintf(fptr,"num_calibratn_elemnts_scan   %d \n",
					uhdr.num_calibratn_elemnts_scan);
  fprintf(fptr,"record_size_in_bytes     %d \n",uhdr.record_size_in_bytes);
  fprintf(fptr,"channels_per_record      %d \n",uhdr.channels_per_record);
  fprintf(fptr,"num_records_per_scan     %d \n",uhdr.num_records_per_scan);
  fprintf(fptr,"num_records_data_set     %d \n",uhdr.num_records_data_set);
  fprintf(fptr,"byts_ancill_data_per_data_set %d \n",
					uhdr.byts_ancill_data_per_data_set);
  fprintf(fptr,"data_order_indicator     %d \n",uhdr.data_order_indicator);
  fprintf(fptr,"start_pixel_number       %d \n",uhdr.start_pixel_number);
  fprintf(fptr,"stop_pixel_number	 %d \n",uhdr.stop_pixel_number);

  fprintf(fptr,"\nAo Coefficients per channel\n");
  fprintf(fptr,"____________________________\n \n");

   for (i = 0; i < 64; i++)
     fprintf(fptr,"a0_coefficient[%d]        %d \n",i,uhdr.a0_coefficient[i]);

  fprintf(fptr,"\nEo Exponents per channel\n");
  fprintf(fptr,"________________________\n \n");

  for (i = 0; i < 64; i++)
     fprintf(fptr,"e0_exponent[%d]           %d \n",i,uhdr.e0_exponent[i]);

  fprintf(fptr,"\nA1 Coeffiecients per channel\n");
  fprintf(fptr,"____________________________\n \n");

  for (i = 0; i < 64; i++)
     fprintf(fptr,"a1_coefficient[%d]        %d \n",i,uhdr.a1_coefficient[i]);

  fprintf(fptr,"\nE1 Exponents per channel\n");
  fprintf(fptr,"________________________\n \n");

  for (i = 0; i < 64; i++)
     fprintf(fptr,"e1_exponent[%d]           %d \n",i,uhdr.e1_exponent[i]);
  fprintf(fptr,"\n");

   for (i = 0; i < 64; i++)
     fprintf(fptr,"color_code_data_chanl[%d] %d \n",i,
						uhdr.color_code_data_chanl[i]);
  fprintf(fptr,"\n");

  fprintf(fptr,"picture_field_width      %d \n",uhdr.picture_field_width);
  fprintf(fptr,"pixel_registration       %d \n",uhdr.pixel_registration);
  fprintf(fptr,"computer_word_size       %d \n",uhdr.computer_word_size);
  fprintf(fptr,"\n");

  for (i = 0; i < 64; i++)
    {
    fprintf(fptr,"lowr_lmt_wavelength[%d]   %s \n",i,
					uhdr.lowr_lmt_wavelength[i]);
    fprintf(fptr,"uppr_lmt_wavelength[%d]   %s \n",i,
					uhdr.uppr_lmt_wavelength[i]);
    }
  fprintf(fptr,"\n");

  fprintf(fptr,"data_sets_per_record      %d \n",uhdr.data_sets_per_record);
  fprintf(fptr,"byte_address_start_second_calibratn %d \n",
				uhdr.byte_address_start_second_calibratn);
  fprintf(fptr,"calibration_elements_second_area    %d \n",
				uhdr.calibration_elements_second_area);
  fprintf(fptr,"calibration_source_indicator        %d \n",
				uhdr.calibration_source_indicator);
  fprintf(fptr,"file_skip_flag           %d \n",uhdr.file_skip_flag);
  fprintf(fptr,"bands_in_first_record    %d \n",uhdr.bands_in_first_record);
  fprintf(fptr,"bytes_per_scan           %d \n",uhdr.bytes_per_scan);
  fprintf(fptr,"pixel_skip_factor	 %d \n",uhdr.pixel_skip_factor);
  fprintf(fptr,"scan_skip_factor	 %d \n",uhdr.scan_skip_factor);

  for (i = 0; i < 4; i++)
    fprintf(fptr,"general_information[%d]   %s \n",i,
				uhdr.general_information[i]);

  fprintf(fptr,"sun_elevation_angle_centre_deg %s \n",
				uhdr.sun_elevation_angle_centre_deg);

  fprintf(fptr,"sun_azimuth_angle_centre_deg   %s \n",
				uhdr.sun_azimuth_angle_centre_deg);

  fprintf(fptr,"number_auxiliary_files   %d \n",uhdr.number_auxiliary_files);
  fprintf(fptr,"start_scan_line		 %d \n",uhdr.start_scan_line);
  fprintf(fptr,"stop_scan_line 		 %d \n",uhdr.stop_scan_line);
  fprintf(fptr,"scan_lins_per_frame_per_thous %d \n",
				uhdr.scan_lins_per_frame_per_thous);

/* Image Annotation Data
   ---------------------*/
  fprintf(fptr,"\nImage Annotation Data\n");
  fprintf(fptr,"_____________________\n \n");

  fprintf(fptr,"ann_satellite_station_id %s \n",uhdr.ann_satellite_station_id);
  fprintf(fptr,"ann_path		 %s \n",uhdr.ann_path);
  fprintf(fptr,"ann_row			 %s \n",uhdr.ann_row);
  fprintf(fptr,"ann_day			 %s \n",uhdr.ann_day);
  fprintf(fptr,"ann_month		 %s \n",uhdr.ann_month);
  fprintf(fptr,"ann_mission		 %s \n",uhdr.ann_mission);
  fprintf(fptr,"ann_days_since_launch    %s \n",uhdr.ann_days_since_launch);
  fprintf(fptr,"ann_hours		 %s \n",uhdr.ann_hours);
  fprintf(fptr,"ann_minutes		 %s \n",uhdr.ann_minutes);
  fprintf(fptr,"ann_tens_of_seconds      %s \n",uhdr.ann_tens_of_seconds);

  fprintf(fptr,"\n");
  fprintf(fptr,"nominal_altitude_met     %d \n",uhdr.nominal_altitude_met);

  fprintf(fptr,"nominal_ground_speed_met_sec %d \n",
				uhdr.nominal_ground_speed_met_sec);
  fprintf(fptr,"scan_type                %d \n",uhdr.scan_type);
  fprintf(fptr,"arc_angle_deg		 %d \n",uhdr.arc_angle_deg);
  fprintf(fptr,"camera			 %d \n",uhdr.camera);
  fprintf(fptr,"input_device		 %d \n",uhdr.input_device);
  fprintf(fptr,"truncation		 %d \n",uhdr.truncation);

		/* Requested channels are bit 1, Other channels are bit 0 
		   -----------------------------------------------------*/
  fprintf(fptr,"\nRequested channels = 1, Other channels = 0\n"); 
  fprintf(fptr,"__________________________________________\n \n");

  for (i = 0; i < 64; i++)
    fprintf(fptr,"channels_requested[%d]    %d \n",i,
				uhdr.channels_requested[i]);
  fprintf(fptr,"\n");

  fprintf(fptr,"processing_mode          %d \n",uhdr.processing_mode);
  fprintf(fptr,"color_select 	 	 %d \n",uhdr.color_select);
  fprintf(fptr,"image_format		 %d \n",uhdr.image_format);
  fprintf(fptr,"repeat_pixels_per_scan   %d \n",uhdr.repeat_pixels_per_scan);
  fprintf(fptr,"repeat_of_scan		 %d \n",uhdr.repeat_of_scan);
  fprintf(fptr,"partial_scan_strt_pixel  %d \n",uhdr.partial_scan_strt_pixel);
  fprintf(fptr,"partial_scan_end_pixel   %d \n",uhdr.partial_scan_end_pixel);
  fprintf(fptr,"sensor_scan_rate  	 %d \n",uhdr.sensor_scan_rate);
  fprintf(fptr,"pixel_size 	 	 %d \n",uhdr.pixel_size);
	
  fprintf(fptr,"angle_of_drift_int	 %d \n",uhdr.angle_of_drift_int);
  fprintf(fptr,"angle_of_drift_fract     %d \n",uhdr.angle_of_drift_fract);
  }

/* If LANDSAT Header flag is on, get and display information
   --------------------------------------------------------*/
if (*lhdsk)
  {
  if (c_jsc_lhdr(hname, &lhdr) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"\n\nLandsat Header File Information \n");
  fprintf(fptr,"_______________________________ \n \n");
  c_lsmknm(hname,"lhdr",tempfile);
  fprintf(fptr,"Landsat header filename     %s \n \n", tempfile);

  /* Begin printing out header information
     ------------------------------------*/
  fprintf(fptr,"%s%s\n",lhdr.productn_syst_id_code,lhdr.id_code_description);
  fprintf(fptr,"%s%s\n",lhdr.mission_id,lhdr.title1);
  fprintf(fptr,"%s%s\n",lhdr.day_num_since_launch,lhdr.title2);
  fprintf(fptr,"%s%s\n",lhdr.orbit_number,lhdr.title3);
  fprintf(fptr,"%s%s\n",lhdr.frame_centre_time,lhdr.title4);
  fprintf(fptr,"%s%s\n",lhdr.frame_centre_lat,lhdr.title5);
  fprintf(fptr,"%s%s\n",lhdr.frame_centre_long,lhdr.title6);
  fprintf(fptr,"%s%s\n",lhdr.utm_zone_number,lhdr.title7);
  fprintf(fptr,"%s%s\n",lhdr.path_number,lhdr.title8);
  fprintf(fptr,"%s%s\n",lhdr.row_number,lhdr.title9);
  fprintf(fptr,"%s%s\n",lhdr.cycle_number,lhdr.title10);

  fprintf(fptr,"%s%s%s\n",lhdr.data_acquisition_ddmmyy,lhdr.title11,
					lhdr.data_acquisition_ddmmmyy);
  fprintf(fptr,"%s%s%s\n",lhdr.master_cct_producd_ddmmyy,lhdr.title12,
					lhdr.master_cct_producd_ddmmmyy);
  fprintf(fptr,"%s%s%s\n",lhdr.date_copy_producd_ddmmyy,lhdr.title13,
					lhdr.date_copy_producd_ddmmmyy);

  fprintf(fptr,"%s%s\n",lhdr.heading,lhdr.title14);
  fprintf(fptr,"%s%s\n",lhdr.seq_number_of_tape,lhdr.title15);
  fprintf(fptr,"%s%s\n",lhdr.tape_start_time,lhdr.title16);

  fprintf(fptr,"\n          %s\n",lhdr.title17);
  fprintf(fptr,"           _____________\n \n");
  fprintf(fptr,"         %s radiometric_calibration\n",
					lhdr.radiometric_calibration);
  fprintf(fptr,"         %s radiometric_levels\n",
					lhdr.radiometric_levels);
  fprintf(fptr,"         %s earth_rotation_correction\n",
					lhdr.earth_rotatn_correctn);
  fprintf(fptr,"         %s mirror_scan_velocity_correction\n",
					lhdr.mirror_scan_velocity_corr);
  fprintf(fptr,"         %s data_representation\n",
					lhdr.data_representation);
  fprintf(fptr,"         %s line_length_correction\n",
					lhdr.line_length_correction);
  fprintf(fptr,"         %s character_code_for_lhdr_file\n",
					lhdr.character_code_for_lhdr);

/* Geometric Transformation Record
   ------------------------------*/
  fprintf(fptr,"\nGeometric Transformation Record\n");
  fprintf(fptr,"_______________________________\n\n");

  fprintf(fptr,"gt_utm_zone_number          %s \n",lhdr.gt_utm_zone_number);
  fprintf(fptr,"gt_utm_coordinate_northing  %s \n",
					lhdr.gt_utm_coordinate_northing);
  fprintf(fptr,"gt_utm_coordinate_easting   %s \n",
					lhdr.gt_utm_coordinate_easting);
  fprintf(fptr,"gt_orientation	 	    %s \n",lhdr.gt_orientation);

/* If Radiometric Transformation flag is on, get and display information
   --------------------------------------------------------------------*/
  if (c_jsc_radtran(hname, &jscrt4, &jscrt5, &jscrt6,
					 &jscrt7, &jscrt8) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"\n\nRadiometric Transformation File Information \n");
  fprintf(fptr,"___________________________________________ \n \n");
  c_lsmknm(hname,"lhdr",tempfile);
  fprintf(fptr,"Landsat header filename     %s \n \n", tempfile);

  fprintf(fptr,"Band 4\n");
  fprintf(fptr,"______\n\n");

  fprintf(fptr,"Detector_1:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt4.radiometric_transformation_detector_1[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_2:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt4.radiometric_transformation_detector_2[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_3:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt4.radiometric_transformation_detector_3[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_4:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt4.radiometric_transformation_detector_4[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_5:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt4.radiometric_transformation_detector_5[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_6:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt4.radiometric_transformation_detector_6[i+16*j]);
    fprintf(fptr,"\n             ");
    }

  fprintf(fptr,"\nBand 5\n");
  fprintf(fptr,"______\n\n");

  fprintf(fptr,"Detector_1:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt5.radiometric_transformation_detector_1[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_2:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt5.radiometric_transformation_detector_2[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_3:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt5.radiometric_transformation_detector_3[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_4:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt5.radiometric_transformation_detector_4[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_5:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt5.radiometric_transformation_detector_5[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_6:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt5.radiometric_transformation_detector_6[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nBand 6\n");
  fprintf(fptr,"______\n\n");

  fprintf(fptr,"Detector_1:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt6.radiometric_transformation_detector_1[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_2:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt6.radiometric_transformation_detector_2[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_3:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt6.radiometric_transformation_detector_3[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_4:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt6.radiometric_transformation_detector_4[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_5:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt6.radiometric_transformation_detector_5[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_6:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt6.radiometric_transformation_detector_6[i+16*j]);
    fprintf(fptr,"\n             ");
    }

  fprintf(fptr,"\nBand 7\n");
  fprintf(fptr,"______\n\n");

  fprintf(fptr,"Detector_1:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt7.radiometric_transformation_detector_1[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_2:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt7.radiometric_transformation_detector_2[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_3:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt7.radiometric_transformation_detector_3[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_4:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt7.radiometric_transformation_detector_4[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_5:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt7.radiometric_transformation_detector_5[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_6:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt7.radiometric_transformation_detector_6[i+16*j]);
    fprintf(fptr,"\n             ");
    }

  fprintf(fptr,"\nBand 8\n");
  fprintf(fptr,"______\n\n");

  fprintf(fptr,"Detector_1:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt8.radiometric_transformation_detector_1[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_2:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt8.radiometric_transformation_detector_2[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_3:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt8.radiometric_transformation_detector_3[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_4:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt8.radiometric_transformation_detector_4[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_5:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt8.radiometric_transformation_detector_5[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\nDetector_6:  ");
  for (j = 0; j < 4; j++)
    {
    for (i = 0; i < 16; i++)
      fprintf(fptr,"%s",jscrt8.radiometric_transformation_detector_6[i+16*j]);
    fprintf(fptr,"\n             ");
    }
  fprintf(fptr,"\n");
  }
if (*acdsk)
  {
  if (c_jsc_anc(&jscanc, hname, nl) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"\nAncillary File Information \n");
  fprintf(fptr,"__________________________ \n \n");
  c_lsmknm(hname,"anc",tempfile);
  fprintf(fptr,"Ancillary filename     %s \n \n", tempfile);

  fprintf(fptr,"Line number of information:  %d \n \n",nl);

  fprintf(fptr,"anc_start                %d \n",jscanc.anc_start);
  fprintf(fptr,"anc_stop                 %d \n",jscanc.anc_stop);
 
  fprintf(fptr,"anc_line_length          %d \n",jscanc.anc_line_length);
  fprintf(fptr,"anc_days      		 %d \n",jscanc.anc_days);
  fprintf(fptr,"anc_hours		 %d \n",jscanc.anc_hours);
  fprintf(fptr,"anc_minutes		 %d \n",jscanc.anc_minutes);
  fprintf(fptr,"anc_seconds		 %d \n",jscanc.anc_seconds);
  fprintf(fptr,"anc_hundreth_seconds	 %d \n",jscanc.anc_hundreth_seconds);

  fprintf(fptr,"anc_sensor		 %d \n",jscanc.anc_sensor);

  for (i = 0; i < 6; i++)
    fprintf(fptr,"anc_cal_band_4[%d]        %d \n",i,jscanc.anc_cal_band_4[i]);
  for (i = 0; i < 6; i++)
    fprintf(fptr,"anc_cal_band_5[%d]        %d \n",i,jscanc.anc_cal_band_5[i]);
  for (i = 0; i < 6; i++)
    fprintf(fptr,"anc_cal_band_6[%d]        %d \n",i,jscanc.anc_cal_band_6[i]);
  for (i = 0; i < 6; i++)
    fprintf(fptr,"anc_cal_band_7[%d]        %d \n",i,jscanc.anc_cal_band_7[i]);

  for (i = 0; i < 6; i++)
    fprintf(fptr,"anc_cal_thermal[%d]        %d \n",i,
						jscanc.anc_cal_thermal[i]);
  }
fclose(fptr);

return(E_SUCC);
}		/* end of dspjsc */
