/***************************************************************************
NAME:			DSPLTWG	

PURPOSE:  Displays LTWG associated record information

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		3/94    D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#ifdef sun
#include <unistd.h>
#endif
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "labserv.h"
#include "protos.h"

int dsplead
(
    char *hname,			/* filename				*/
    FILE *fptr			/* output file pointer			*/
)

{
struct LTWGLFD lfd;		/* LTWG leader file - file descriptor   */
struct LTWGSHR shr;		/* LTWG leader file - scene header rec	*/
struct LTWGMPR mpr;		/* LTWG leader file - map projection rec */
struct LTWGRCR rcr; 		/* LTWG radiometric calibration record	*/

long one = 1;
long nonfatal = 1;		/* non fatal error flag			*/
long status;
long mode = 0;			/* flag value for lsstat		*/
long i;
long loop_count;

FILE  *fin;      		/* input file pointer			*/

char finame[CMLEN];		/* tae name				*/

  c_lsmknm(hname,"lead",finame);
  status = c_lsstat(finame,&mode);    /* make sure file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("LEAD file does not exist on disk","ltwg_lead-stat",&one);
     return(E_FAIL);
     }

  fin=fopen(finame,"r");
  if(!fin)
    {
    c_errmsg("Error opening Leader File","ltwg_lead-open",&nonfatal);
    return(E_FAIL);
    }

       /* print file descriptor record for the leader file */
  if ((status = c_ltwg_lfd(&lfd, fin)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"LEADER FILE - FILE DESCRIPTOR RECORD \n");
  fprintf(fptr,"____________________________________ \n \n");

  fprintf(fptr,"Record Sequence Number                   %d\n",lfd.rec_number);
  fprintf(fptr,"First record sub-type code               %d\n",
	lfd.frst_rec_code);
  fprintf(fptr,"Record type code                         %d\n",lfd.rec_code);
  fprintf(fptr,"Second record sub-type code              %d\n",
	lfd.scnd_rec_code);
  fprintf(fptr,"Third record sub-type code               %d\n",
	lfd.thrd_rec_code);
  fprintf(fptr,"Length of this record                    %d\n",lfd.rec_length);
  fprintf(fptr,"ASCII/EBCDIC flag                        %s\n",lfd.a_e_flag);
  fprintf(fptr,"Control document number                  %s\n",
	lfd.control_doc_num);
  fprintf(fptr,"Control document revision                %s\n",
	lfd.control_doc_rev);
  fprintf(fptr,"File design descriptor revision          %s\n",
	lfd.file_des_rev);
  fprintf(fptr,"Software release Number                  %s\n",
	lfd.rel_num);
  fprintf(fptr,"File number                              %d\n",lfd.file_num);
  fprintf(fptr,"File name                                %s\n",lfd.file_name);
  fprintf(fptr,"Record seq and location type             %s\n",lfd.record_seq);
  fprintf(fptr,"Sequence number location                 %d\n",
	lfd.seq_num_loc);
  fprintf(fptr,"Sequence number field length             %d\n",
	lfd.seq_num_lgt);
  fprintf(fptr,"Record code and location type            %s\n",
	lfd.rec_loc_flg);
  fprintf(fptr,"Record code location                     %d\n",lfd.rec_loc);
  fprintf(fptr,"Record code field length                 %d\n",lfd.rec_lth);
  fprintf(fptr,"Record length and location type          %s\n",
	lfd.rec_lth_flg);
  fprintf(fptr,"Record length location                   %d\n",
	lfd.rec_lth_loc);
  fprintf(fptr,"Record length field length               %d\n",
	lfd.rec_fld_lth);
  fprintf(fptr,"Data interpretation included in descriptor record -      %s\n",
      lfd.data_interp_des);
  fprintf(fptr,"Data interpretation included in other file -             %s \n",
      lfd.data_interp_oth);
  fprintf(fptr,"Data display information included in descriptor record - %s\n",
      lfd.data_disp_des);
  fprintf(fptr,"Data display information included in other file -        %s\n",
      lfd.data_disp_oth);
  fprintf(fptr,"Number of header records                 %d\n",lfd.num_header);
  fprintf(fptr,"Header record length                     %d\n",lfd.lth_header);
  fprintf(fptr,"Number map projection records            %d\n",
	lfd.num_map_proj);
  fprintf(fptr,"Map project record length                %d\n",
	lfd.lth_map_proj);
  fprintf(fptr,"Number of callbration records            %d\n",
	lfd.num_ancill_rec);
  fprintf(fptr,"Calibration record length                %d\n",
	lfd.lth_ancill_rec);
  fprintf(fptr,"Scene identification locator             %s\n",
	lfd.scene_id_loc);
  fprintf(fptr,"WRS identification locator               %s\n",lfd.wrs_id_loc);
  fprintf(fptr,"Mission identification locator           %s\n",
	lfd.miss_id_loc);
  fprintf(fptr,"Sensor identification locator            %s\n",
	lfd.sensor_id_loc);
  fprintf(fptr,"Exposre date-time locator                %s\n",
	lfd.exp_date_loc);
  fprintf(fptr,"Geographic reference locator             %s\n",
	lfd.geog_ref_loc);
  fprintf(fptr,"Image processing locator                 %s\n",
	lfd.imag_proc_loc);
  fprintf(fptr,"Imagery format locator	                 %s\n",
	lfd.imag_for_loc);
  fprintf(fptr,"Band indicator locator                   %s\n",
	lfd.band_ind_loc);
  fprintf(fptr,"Quadrant indicator locator               %s\n",
	lfd.quad_ind_loc);
  fprintf(fptr,"Interpixel & line scale locator          %s\n",
	lfd.inter_scale_loc);
  fprintf(fptr,"Quadrant product vert overlap            %s\n",
 	lfd.quad_ver_ovrlp);
  fprintf(fptr,"Quadrand product horz overlap            %s\n",
	lfd.quad_hor_ovrlp);
  fprintf(fptr,"\n\n");
        /* End file descriptor record */
 
        /* Print scene header records for leader file */
  for (loop_count=1; loop_count<= lfd.num_header; loop_count++)
    {
    if ((status = c_ltwg_shr(&shr, fin, &lfd.lth_header)) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }
  
    fprintf(fptr,"LEADER FILE - SCENE HEADER RECORD %3d\n",loop_count);
    fprintf(fptr,"_____________________________________ \n \n");
     
    fprintf(fptr,"    SCENE PARAMETERS\n");
    fprintf(fptr,"Record Sequence Number                   %d\n",
	shr.rec_number);
    fprintf(fptr,"First record sub-type code               %d\n",	
	shr.frst_rec_code);
    fprintf(fptr,"Record type code                         %d\n",shr.rec_code);
    fprintf(fptr,"Second record sub-type cod               %d\n",
	shr.scnd_rec_code);
    fprintf(fptr,"Third record sub-type cod                %d\n",
	shr.thrd_rec_code);
    fprintf(fptr,"Length of this record	                   %d\n",
	shr.rec_length);
    fprintf(fptr,"Header record sequence number            %d\n",
	shr.header_seq_num);
    fprintf(fptr,"Product type identification              %s\n",
	shr.prod_type);
    fprintf(fptr,"Input scene identification               %s\n",
	shr.in_scene_id);
    fprintf(fptr,"Scene centre latitude                    %f degrees\n",
	  shr.in_scene_centre_lat);
    fprintf(fptr,"Scene centre longitude                   %f degrees\n",
	  shr.in_scene_centre_lon);
    fprintf(fptr,"Line number at scene centre              %f\n",
	shr.in_line_num_cent);
    fprintf(fptr,"Pixel number at scene centre             %f\n",
	shr.in_pixel_num_cent);
    fprintf(fptr,"Input scene centre time                  %s\n",
	shr.time_centre);
    fprintf(fptr,"Time offset from WRS 	                   %e msec\n",
	shr.time_offset);
    fprintf(fptr,"WRS designator (path and row)            %s\n",
	shr.wrs_design);
    fprintf(fptr,"WRS cycle                                %s\n",
	shr.wrs_cycle);
    fprintf(fptr,"Processed scene id	                   %s\n",
	shr.pro_scene_id);
    fprintf(fptr,"Proc scene centre latitude               %f degrees\n",
	shr.pro_scene_centre_lat);
    fprintf(fptr,"Proc scene centre longitude              %f degrees\n",
	shr.pro_scene_centre_lon);
    fprintf(fptr,"Line number procsd scene cent            %f\n",
	shr.pro_line_num_cent);
    fprintf(fptr,"Pixel number procsd scene cent           %f\n",
	shr.pro_pixel_num_cent);
    fprintf(fptr,"Count of overlap lines                   %d\n",
	shr.ovrlap_lines);
    fprintf(fptr,"Count of overlap pixels                  %d\n",
	shr.ovrlap_pixels);
    fprintf(fptr,"    MISSION PARAMETERS\n"); 
    fprintf(fptr,"Mission identification                   %s\n",shr.miss_id);
    fprintf(fptr,"Sensor identification	                   %s\n",
	shr.sensor_id);
    fprintf(fptr,"Orbit number         	                   %d\n",
	shr.orbit_num);
    fprintf(fptr,"Ascending/descending flag                %s\n",
	shr.asc_des_flag);
    fprintf(fptr,"    SENSOR PARAMETERS \n");
    fprintf(fptr,"   Wavelgth ranges (nm)\n");
    fprintf(fptr,"Band  lower limit         upper limit\n");
    fprintf(fptr,"-------------------------------------\n");
    for (i=0; i< 64; i++)
      fprintf(fptr," %2d     %8d            %8d\n",i,shr.wavelgth_range[i][0],
  	  shr.wavelgth_range[i][1]);
    fprintf(fptr,"Number of active bands in image          %d\n",
	shr.num_bands);
    fprintf(fptr,"Number of pixels/line in image           %d\n",
	shr.pixels_per_line);
    fprintf(fptr,"Number of scene lines in image           %d\n",
	shr.num_lines);
    fprintf(fptr,"    PROCESSING PARAMETERS \n");
    fprintf(fptr,"Radiometric calibration design           %s\n",
	shr.rad_cal_des);
    fprintf(fptr,"Radiometric resoultion design            %d\n",
	shr.rad_res_des);
    fprintf(fptr,"Scenic radiometric correction            %s\n",
	shr.scenic_rad_des);
    fprintf(fptr,"Geometric correction designator          %s\n",
	shr.geom_corr_des);
    fprintf(fptr,"Resampling designator                    %s\n",
	shr.resamp_des);
    fprintf(fptr,"Map projection identifier                %s\n",
	shr.map_proj_id);
    fprintf(fptr,"Product processing level                 %d\n",
	shr.process_level);
    fprintf(fptr,"Number map projection records            %d\n",
	shr.num_map_proj_anc);
    fprintf(fptr,"Technique to replace detectors           %s\n",
	shr.replac_detect);
    fprintf(fptr,"Kernal used to replace detector          %s\n",
	shr.kernal_for_replac);
    fprintf(fptr,"Number of radiometric records            %d\n",
	shr.num_rad_ancill);
    fprintf(fptr,"Number of active bands  \n     %s\n",
	shr.active_bands);
    fprintf(fptr,"Interleaving idicator                    %s\n",
	shr.interleave_ind);
    fprintf(fptr,"Detector substitution array\n");
    for (i=0; i<100; i++)
      fprintf(fptr, "           %4d  \n",shr.detector_subst[i]);
    fprintf(fptr,"Detector smoothing array \n");
    fprintf(fptr,"  %s\n",shr.detector_smooth);
    fprintf(fptr,"        Mirror scan velocity profile\n");
    fprintf(fptr,"     Forward scan          Reverse Scan\n");
    fprintf(fptr,"     ----------------------------------\n");
    for (i=0; i<6; i++)
      fprintf(fptr,"    %16.9e     %16.9e\n",shr.mirror_scan_vel[i],
	  shr.mirror_scan_vel[6+i]);
    fprintf(fptr,"       Detector adjustment array     \n");
    fprintf(fptr,"      Reverse             Forward\n");
    fprintf(fptr,"  West       East      West      East\n");
    fprintf(fptr,"-----------------------------------------\n");
    for (i=0; i<16; i++)
      fprintf(fptr,"  %4d       %4d       %4d       %4d\n",
	 shr.dect_adjust[0][i],shr.dect_adjust[1][i],shr.dect_adjust[2][i],
	 shr.dect_adjust[3][i]);
    fprintf(fptr,"\n\n");
    }      /* End of scene header records */ 
 
           /* Print Map projection records */ 
  for(loop_count=1; loop_count<= lfd.num_map_proj; loop_count++)
    {
    if ((status = c_ltwg_mpr(&mpr, fin, &lfd.lth_map_proj)) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }
  
    fprintf(fptr,"LEADER FILE - MAP PROJECTION ANCILLARY RECORD %3d\n",
          loop_count); 
    fprintf(fptr,"__________________________________________________\n\n");
  
    fprintf(fptr,"Record Sequence Number                  %d\n",
	mpr.rec_number);
    fprintf(fptr,"First record sub-type code              %d\n",
	mpr.frst_rec_code);
    fprintf(fptr,"Record type code                        %d\n",mpr.rec_code);
    fprintf(fptr,"Second record sub-type code             %d\n",
	mpr.scnd_rec_code);
    fprintf(fptr,"Third record sub-type code              %d\n",
	mpr.thrd_rec_code);
    fprintf(fptr,"Length of this record                   %d\n",
	mpr.rec_length);
    fprintf(fptr,"    Inut scene related data\n");
    fprintf(fptr,"Nominal number of pixels                %d\n",
	  mpr.nominal_num_pixels);
    fprintf(fptr,"Nominal number of lines                 %d\n",
	mpr.nominal_num_lines);
    fprintf(fptr,"Nominal inter-pixel dist                %f m\n",
	  mpr.nominal_scale_pixel);
    fprintf(fptr,"Nominal inter-line dist                 %f m\n",
	  mpr.nominal_scale_lines);
    fprintf(fptr,"Image skew at centre 	                  %f degrees\n",
	  mpr.image_skew);
    fprintf(fptr,"    UTM/PS Related data for input scene\n");  
    fprintf(fptr,"UTM datum for image WRS centre          %s\n",mpr.utm_datum);
    fprintf(fptr,"UTM zone number for input image         %d\n",mpr.utm_zone);
    fprintf(fptr,"Northing of WRS centre                  %f m\n",
	mpr.wrs_northing);
    fprintf(fptr,"Easting of WRS centre                   %f m\n",
	mpr.wrs_easting);
    fprintf(fptr,"Northing of image centre                %f m\n",
	mpr.input_northing);
    fprintf(fptr,"Easting of image centre                 %f m\n",
	mpr.input_easting);
    fprintf(fptr,"Verticlal offset centre to WRS          %f\n",
	mpr.vert_offset);
    fprintf(fptr,"Horizontal offset center to WRS         %f\n",
	mpr.horz_offset);
    fprintf(fptr,"Orient input image center               %f deg\n",
	mpr.orientation);
    fprintf(fptr,"    Processed scene related data\n");  
    fprintf(fptr,"Product pixels per line                 %f\n",
	mpr.prod_pixel_line);
    fprintf(fptr,"Number of lines per image               %f\n",
	mpr.prod_num_lines);
    fprintf(fptr,"Processed inter-pixel dist              %f m\n",
	  mpr.inter_pixel_scale);
    fprintf(fptr,"Processed inter-line dist               %f m\n",
	  mpr.inter_line_scale);
    fprintf(fptr,"UTM datum for processed image           %s\n",
	mpr.proc_utm_datum);
    fprintf(fptr,"UTM zone number   	                  %d\n",
	mpr.proc_utm_zome);
    fprintf(fptr,"Line num of WRS centre                  %f\n",
	mpr.proc_center_line);
    fprintf(fptr,"Pixel num of WRS centre                 %f\n",
	mpr.proc_center_pix);
    fprintf(fptr,"Orientation of image centre             %f\n",
	mpr.proc_orient);
    fprintf(fptr,"    Sensor data\n"); 
    fprintf(fptr,"Sat. orbital inclin  	                  %f\n",
	mpr.inclination);
    fprintf(fptr,"Spacecraft ascending node               %f\n",
	mpr.ascend_node);
    fprintf(fptr,"Satellite altit.-WRS centre             %f m\n",
	mpr.altitude);
    fprintf(fptr,"Sat grnd speed-WRS centre               %f m/s\n",
	mpr.ground_speed);
    fprintf(fptr,"Satellite heading                       %f deg\n",
	mpr.sat_heading);
    fprintf(fptr,"Cross-track field of view               %f deg\n",
	  mpr.cross_track_fov);
    fprintf(fptr,"Sensor scan rate                        %f scan/sec\n",
	  mpr.scan_rate);
    fprintf(fptr,"Sensor sampling rate 	                  %f samp/sec\n",
	  mpr.samp_rate);
    fprintf(fptr,"Sun elevation angle-centre              %f deg\n",
	mpr.sun_elev);
    fprintf(fptr,"Sun azimuth angle centre                %f deg\n",
	mpr.sun_azimuth);
    fprintf(fptr,"     UTM coord for corners   \n");
    fprintf(fptr,"-------------------------------\n");
    fprintf(fptr,"  Corner 1    %f m    %f m\n",mpr.utm_corner_1[0],
	  mpr.utm_corner_1[1]);
    fprintf(fptr,"  Corner 2    %f m    %f m\n",mpr.utm_corner_2[0],
	  mpr.utm_corner_2[1]);
    fprintf(fptr,"  Corner 3    %f m    %f m\n",mpr.utm_corner_3[0],
	  mpr.utm_corner_3[1]);
    fprintf(fptr,"  Corner 4    %f m    %f m\n",mpr.utm_corner_4[0],
	  mpr.utm_corner_4[1]);
  
    fprintf(fptr,"Latatude and Longitude at corners\n");
    fprintf(fptr,"---------------------------------\n");
    fprintf(fptr,"  Corner 1    %f      %f \n",mpr.lat_long_corner1[0],
  	  mpr.lat_long_corner1[1]);
    fprintf(fptr,"  Corner 2    %f      %f \n",mpr.lat_long_corner2[0],
          mpr.lat_long_corner2[1]);
    fprintf(fptr,"  Corner 3    %f      %f \n",mpr.lat_long_corner3[0],
          mpr.lat_long_corner3[1]);
    fprintf(fptr,"  Corner 4    %f      %f \n",mpr.lat_long_corner4[0],
          mpr.lat_long_corner4[1]);
    fprintf(fptr,"Pixel and line numbers at corners\n");
    fprintf(fptr,"---------------------------------\n");
    fprintf(fptr,"  Corner 1    %f      %f \n",mpr.pix_line_corner1[0],
	  mpr.pix_line_corner1[1]);
    fprintf(fptr,"  Corner 2    %f      %f \n",mpr.pix_line_corner2[0],
          mpr.pix_line_corner2[1]);
    fprintf(fptr,"  Corner 3    %f      %f \n",mpr.pix_line_corner3[0],
          mpr.pix_line_corner3[1]);
    fprintf(fptr,"  Corner 4    %f      %f \n",mpr.pix_line_corner4[0],
          mpr.pix_line_corner4[1]);
    fprintf(fptr,"\n\n");
    }     /* End of map projection records */
  
          /* Print Radiometric Ancillary records */
  for(loop_count=1; loop_count<= lfd.num_ancill_rec; loop_count++)
    {
    if ((status = c_ltwg_rcr(&rcr, fin, &lfd.lth_ancill_rec)) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }
  
    fprintf(fptr,"LEADER FILE - RADIOMETRIC CALIBRATION ANCILLARY RECORD %3d\n"
	,loop_count); 
    fprintf(fptr,"__________________________________________________________");
    fprintf(fptr,"\n\n");
  
    fprintf(fptr,"Record Sequence Number                   %d\n",
	rcr.rec_number);
    fprintf(fptr,"First record sub-type code               %d\n",
	rcr.frst_rec_code);
    fprintf(fptr,"Record type code                         %d\n",rcr.rec_code);
    fprintf(fptr,"Second record sub-type code              %d\n",
	rcr.scnd_rec_code);
    fprintf(fptr,"Third record sub-type code               %d\n",
	rcr.thrd_rec_code);
    fprintf(fptr,"Length of this record                    %d\n",
	rcr.rec_length);
    fprintf(fptr,"Band number	                           %d\n",
	rcr.band_number);   
    fprintf(fptr,"Lower reflectance limit                  %d\n",
	rcr.reflectance_lower);
    fprintf(fptr,"Upper reflectance limit                  %d\n",
	rcr.reflectance_upper);
    fprintf(fptr,"Offset coefficient  	                   %f\n",rcr.a0);
    fprintf(fptr,"Gain coefficient     	                   %f\n",rcr.a1);
    fprintf(fptr,"             Detector transformation tables\n");
    fprintf(fptr,"                        Detector\n");
    fprintf(fptr,"   1      2      3      4      5      6      7      8\n");
    fprintf(fptr,"  ---    ---    ---    ---    ---    ---    ---    ---\n");
    for (i=0; i<256; i++)
       {
       fprintf(fptr,"  %3d    %3d    %3d    %3d    %3d    %3d    %3d    %3d\n",
         rcr.detector_data[0][i],rcr.detector_data[1][i],
	 rcr.detector_data[2][i],rcr.detector_data[3][i],
	 rcr.detector_data[4][i],rcr.detector_data[5][i],
         rcr.detector_data[6][i],rcr.detector_data[7][i]);
       }
    fprintf(fptr,"\n                        Detector\n");
    fprintf(fptr,"   9      10     11     12     13     14     15     16\n");
    fprintf(fptr,"  ---    ---    ---    ---    ---    ---    ---    ---\n");
    for (i=0; i<256; i++)
       {
       fprintf(fptr,"  %3d    %3d    %3d    %3d    %3d    %3d    %3d    %3d\n",
         rcr.detector_data[8][i],rcr.detector_data[9][i],
         rcr.detector_data[10][i],rcr.detector_data[11][i],
         rcr.detector_data[12][i],rcr.detector_data[13][i],
         rcr.detector_data[14][i],rcr.detector_data[15][i]);
       }
    fprintf(fptr,"\n\n");
    }  /* End of Radiometric ancillary records */ 
     /*  End of Leader File  */
  

return(E_SUCC);
}		/* end of dspltwg */
