/***************************************************************************
NAME:			DSPLTWG	

PURPOSE:  Displays LTWG associated record information

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		3/94    D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#ifdef sun
#include <unistd.h>
#endif
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "labserv.h"
#include "util.h"
#include "protos.h"

int dspltwg
(
    char *hname,			/* filename				*/
    FILE *fptr,			/* output file pointer			*/
    long *vdf_flag,		/* Volume directory flag		*/
    long *lead_flag,		/* Leader file flag			*/
    long *trai_flag,		/* Trailer file flag			*/
    long *supp_flag,		/* Supplemental file flag		*/
    long *idr_flag,		/* Image data record flag		*/
    long *line_num		/* line number for image data to be printed*/
)

{
struct LTWGVDR vdr;		/* LTWG volume directory files		*/
struct LTWGFPR fpr;  		/* LTWG file pointer record		*/
struct LTWGTRIV triv;		/* LTWG text record - imagery volume	*/
struct LTWGIFD ifd;		/* LTWG imagery file - file descriptor	*/
struct LTWGIDR idr;   		/* LTWG imagery file - image data record */
struct LTWGTFD tfd;		/* LTWG trailer file - file descriptor  */
struct LTWGTR tr;		/* LTWG trailer records			*/

long one = 1;
long nonfatal = 1;		/* non fatal error flag			*/
long status;
long outsys;                    /* output system                        */
long sflag = SETN;
long mode = 0;			/* flag value for lsstat		*/
long i, j, b = 0;
long loop_count;

FILE  *fin;      		/* input file pointer			*/

char syst[DDSYLN];              /* system character string              */
char finame[CMLEN];		/* tae name				*/

/* get current system code
   ----------------------*/
status = c_getsys(syst);
if (status != E_SUCC)
   {
   c_errmsg("Error retreiving current system","dspassoc-system",&nonfatal);
   return(E_FAIL);
   }

status = c_sysset(syst, &outsys, &sflag);
if (status != E_SUCC)
   {
   c_errmsg("Error retreiving current system","dspassoc-system",&nonfatal);
   return(E_FAIL);
   }

if (*vdf_flag)    /* Print contents of Volume Directory File */
  {
 /* Open volume directory file
    --------------------------*/
  c_lsmknm(hname,"vdf",finame);
  status = c_lsstat(finame,&mode);    /* make sure file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("VDR file does not exist on disk","ltwg_vdr-stat",&one);
     return(E_FAIL);
     }

  fin=fopen(finame,"r");
  if(!fin)
     {
     c_errmsg("Error opening Volume Directory File","ltwg_vdr-open",&nonfatal);
     return(E_FAIL);
     }

  if ((status = c_ltwg_vdr(&vdr, fin)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

       /* Print volume descriptor record */
  fprintf(fptr,"VOLUME DESCRIPTOR RECORD \n");
  fprintf(fptr,"________________________ \n \n");

  fprintf(fptr,"Record sequence number                   %d\n",vdr.rec_number);
  fprintf(fptr,"First record sub-type code               %d\n",
	vdr.frst_rec_code);
  fprintf(fptr,"Record type code              	         %d\n",vdr.rec_code);
  fprintf(fptr,"Second record sub-type code              %d\n",
	vdr.scnd_rec_code);
  fprintf(fptr,"Third record sub-type code               %d\n",
	vdr.thrd_rec_code);
  fprintf(fptr,"Length of this record                    %d\n",vdr.rec_length);
  fprintf(fptr,"ASCII/EBCDIC flag                        %s\n",vdr.a_e_flag);
  fprintf(fptr,"Superstructure control doc               %s\n",
	vdr.control_num);
  fprintf(fptr,"Superstruct contrl doc rev               %s\n",
	vdr.control_rev_num);
  fprintf(fptr,"Superstruct record format rev            %s\n",vdr.format_rev);
  fprintf(fptr,"Software release number                  %s\n",
	vdr.release_num);
  fprintf(fptr,"Tape ID for physical volume              %s\n",vdr.tape_id);
  fprintf(fptr,"Logical volume ID                        %s\n",
	vdr.logical_vol_id);
  fprintf(fptr,"Volume set ID                            %s\n",vdr.vol_set_id);
  fprintf(fptr,"Number of pyhsical volumes               %d\n",
	vdr.num_phy_vol);
  fprintf(fptr,"Phys vol start logical vol               %d\n",
	vdr.start_log_vol);
  fprintf(fptr,"Phys vol end logical vol                 %d\n",
	vdr.end_log_vol);
  fprintf(fptr,"Phys vol containing vol disc             %d\n",vdr.volume_num);
  fprintf(fptr,"1st referenced file in phy vol           %d\n",
	vdr.frst_ref_file);
  fprintf(fptr,"Logical vol num in volume set            %d\n",
	vdr.log_vol_num_set);
  fprintf(fptr,"Logical vol num in Physical vol          %d\n",
	vdr.log_vol_num_phys);
  fprintf(fptr,"Logical vol creation date                %s\n",
	vdr.log_creat_date);
  fprintf(fptr,"Logical vol creation time                %s\n",
	vdr.log_creat_time);
  fprintf(fptr,"Logical vol generating country           %s\n",
	vdr.log_gen_country);
  fprintf(fptr,"Logical vol generating agency            %s\n",
	vdr.log_gen_agency);
  fprintf(fptr,"Logical vol generating facility          %s\n",
	vdr.log_gen_facility);
  fprintf(fptr,"Number pointer records vol dir           %d\n",
	vdr.num_point_rec);
  fprintf(fptr,"Number records in vol directory          %d\n",
	vdr.num_records);
  fprintf(fptr,"\n\n");
     /* End of volume descriptor record */

     /* Print pointer records */
  if (vdr.num_point_rec >= 3)
     b = vdr.num_point_rec/3;

  for (i=1; i<=b; i++)
    {
    for (j=0; j<3; j++)
      {
      if (j == 0)
        {
        if (i == 1)    /* File Pointer Records for Leader File, Imagery File */
          {            /* and Trailer File all follow the same definition */

            fprintf(fptr,"\n\nFILE POINTER RECORD -- LEADER FILE\n");
            fprintf(fptr,"__________________________________ \n \n");
          }
        else
          {
            fprintf(fptr,"\n\nFILE POINTER RECORD -- LEADER FILE - BAND %d\n",
		i);
	    fprintf(fptr,"_________________________________________\n\n");
	  }
	}
      else if (j == 1)
        {
        if (i == 1)
          {
            fprintf(fptr,"\n\nFILE POINTER RECORD -- IMAGERY FILE\n");
            fprintf(fptr,"___________________________________ \n \n");
          }
        else
          {
            fprintf(fptr,"\n\nFILE POINTER RECORD -- IMAGERY FILE - BAND %d\n",
		i);
	    fprintf(fptr,"____________________________________________\n\n");
	  }
	}
      else 
        {
        if (i == 1)
          {
            fprintf(fptr,"\n\nFILE POINTER RECORD -- TRAILER FILE\n");
            fprintf(fptr,"____________________________________ \n \n");
          }
        else
          {
            fprintf(fptr,"\n\nFILE POINTER RECORD -- TRAILER FILE - BAND %d\n",
		i);
	    fprintf(fptr,"____________________________________________\n\n");
	  }
	}

      if ((status = c_ltwg_fpr(&fpr, fin)) == E_FAIL)
         {
         fclose(fptr);
         return(E_FAIL);
         }
     
      fprintf(fptr,"Record sequence number                   %d\n",
	fpr.rec_number);
      fprintf(fptr,"Firstrecord sub-type code                %d\n",
	fpr.frst_rec_code);
      fprintf(fptr,"Record type code                         %d\n",
	fpr.rec_code);
      fprintf(fptr,"Second record sub-type code              %d\n",
	fpr.scnd_rec_code);
      fprintf(fptr,"Third record sub-type code               %d\n",
	fpr.thrd_rec_code);
      fprintf(fptr,"Length of this record                    %d\n",
	fpr.rec_length);
      fprintf(fptr,"ASCII/EBCDIC flag                        %s\n",
	fpr.a_e_flag);
      fprintf(fptr,"File number                              %d\n",fpr.num);
      fprintf(fptr,"File name                                %s\n",fpr.name);
      fprintf(fptr,"File class                               %s\n",fpr.class);
      fprintf(fptr,"File class code                          %s\n",
	fpr.cl_code);
      fprintf(fptr,"File data type                           %s\n",
	fpr.datatyp);
      fprintf(fptr,"File data type code                      %s\n",
	fpr.dt_code);
      fprintf(fptr,"Number of files in File                  %d\n",
	fpr.num_rec);
      fprintf(fptr,"File descriptor length                   %d\n",
	fpr.des_rec_lth);
      fprintf(fptr,"File maximum length                      %d\n",
	fpr.max_lth);
      fprintf(fptr,"File record length type                  %s\n",
	fpr.rec_lth_typ);
      fprintf(fptr,"File record length code                  %s\n",
	fpr.rec_lth_code);
      fprintf(fptr,"Physical vol num, start                  %d\n",
	fpr.vol_num_start);
      fprintf(fptr,"Physical vol num, end                    %d\n",
	fpr.vol_num_end);
      fprintf(fptr,"First record number for volume           %d\n",
	fpr.file_por_frst);
      fprintf(fptr,"Last record number for volume            %d\n",
	fpr.file_por_last);
      fprintf(fptr,"\n\n");
      }
    }   /* End of pointer records */
       
       /* Print volume directory text record for imagery volume */
  if ((status = c_ltwg_triv(&triv, fin)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"VOLUME DIRECTORY TEXT RECORD - IMAGERY VOLUME \n");
  fprintf(fptr,"_____________________________________________ \n \n");

  fprintf(fptr,"Record Sequence Number                   %d\n",
	triv.rec_number); 
  fprintf(fptr,"First record sub-type code               %d\n",
	triv.frst_rec_code); 
  fprintf(fptr,"Record type code                         %d\n",
	triv.rec_code); 
  fprintf(fptr,"Second record sub-type code              %d\n",
	triv.scnd_rec_code); 
  fprintf(fptr,"Third record sub-type code               %d\n",
	triv.thrd_rec_code); 
  fprintf(fptr,"Length of this record                    %d\n",
	triv.rec_length); 
  fprintf(fptr,"ASCII/EBCDIC flag                        %s\n",triv.a_e_flag); 
  fprintf(fptr,"Continuation flag                        %s\n",triv.cont_flag); 
  fprintf(fptr,"Product type: \n     %s\n",triv.product_type); 
  fprintf(fptr,"Location, date & time of creation: \n     %s\n",
	triv.loc_and_date); 
  fprintf(fptr,"Input scene identification: \n     %s\n",triv.in_scene_id); 
  fprintf(fptr,"Physical tape identification:\n     %s\n",triv.tape_id); 
  fprintf(fptr,"WRS & product identification:            %s\n",triv.wrs_id); 
  fprintf(fptr,"level of processing corrections:         %s\n",triv.level_cor); 
  fprintf(fptr,"\n\n");
  }  /*  End of Volume Directory File  */

if (*lead_flag)    /* Print Leader file */
  {
            /* needed to split the leader file out of this file so it would
	       compile on SUN1 */
     dsplead(hname,fptr);
  }   /*  End of Leader File  */
  
if (*trai_flag)   /* Print Trailer Record */
  {
 /* Open Trailer Record
    -------------------*/
  c_lsmknm(hname,"trai",finame);
  status = c_lsstat(finame,&mode);    /* make sure file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("TRAI file does not exist on disk","ltwg_trai-stat",&one);
     return(E_FAIL);
     }

  fin=fopen(finame,"r");
  if(!fin)
     {
     c_errmsg("Error opening Trailer Record","ltwg_trai-open",&nonfatal);
     return(E_FAIL);
     }

         /* Print file descriptor record for trailer file */
  if ((status = c_ltwg_tfd(&tfd, fin)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"TRAILER FILE - FILE DESCRIPTOR RECORD \n"); 
  fprintf(fptr,"_____________________________________\n\n");

  fprintf(fptr,"Record Sequence Number 	                %d\n",tfd.rec_number);
  fprintf(fptr,"First record sub-type code              %d\n",
	tfd.frst_rec_code);
  fprintf(fptr,"Record type code                        %d\n",tfd.rec_code);
  fprintf(fptr,"Second record sub-type code             %d\n",
	tfd.scnd_rec_code);
  fprintf(fptr,"Third record sub-type code              %d\n",
	tfd.thrd_rec_code);
  fprintf(fptr,"Length of this record                   %d\n",tfd.rec_length);
  fprintf(fptr,"ASCII/EBCDIC flag                       %s\n",tfd.a_e_flag);
  fprintf(fptr,"Control document number                 %s\n",
	tfd.control_doc_num);
  fprintf(fptr,"Control document revision               %s\n",
	tfd.control_doc_rev);
  fprintf(fptr,"File design descriptor revision         %s\n",
	tfd.file_des_rev);
  fprintf(fptr,"Software release Number                 %s\n",tfd.rel_num);
  fprintf(fptr,"File number                             %d\n",tfd.file_num);
  fprintf(fptr,"File name                               %s\n",tfd.file_name);
  fprintf(fptr,"Record seq and location type            %s\n",tfd.record_seq);
  fprintf(fptr,"Sequence number location                %d\n",tfd.seq_num_loc);
  fprintf(fptr,"Sequence number field length            %d\n",tfd.seq_num_lgt);
  fprintf(fptr,"Record code and location type           %s\n",tfd.rec_loc_flg);
  fprintf(fptr,"Record code location                    %d\n",tfd.rec_loc);
  fprintf(fptr,"Record code field length                %d\n",tfd.rec_lth);
  fprintf(fptr,"Record length and location type         %s\n",tfd.rec_lth_flg);
  fprintf(fptr,"Record length location                  %d\n",tfd.rec_lth_loc);
  fprintf(fptr,"Record length field length              %d\n",tfd.rec_fld_lth);
  fprintf(fptr,"Data interpretation included in descriptor record -      %s\n",
      tfd.data_interp_des);
  fprintf(fptr,"Data interpretation included in other file -             %s \n",
      tfd.data_interp_oth);
  fprintf(fptr,"Data display information included in descriptor record - %s\n",
      tfd.data_disp_des);
  fprintf(fptr,"Data display information included in other file -        %s\n",
      tfd.data_disp_oth);
  fprintf(fptr,"Number of trailer records               %d\n",tfd.num_trailer);
  fprintf(fptr,"Trailer record length	                %d\n",tfd.lth_trailer);
  fprintf(fptr,"Parity error count locator              %s\n",
	tfd.parity_count_loc);
  fprintf(fptr,"Quality code summary map loc            %s\n",
	tfd.quality_code_loc);
  fprintf(fptr,"\n\n");
         /* End of file descriptor record */

         /* Print Trailer records */
  for(loop_count=1; loop_count<=tfd.num_trailer ; loop_count++)
    {
    if ((status = c_ltwg_tr(&tr, fin, &tfd.lth_trailer)) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }
  
    fprintf(fptr,"TRAILER RECORD %3d\n",loop_count); 
    fprintf(fptr,"__________________\n\n");
  
    fprintf(fptr,"Record Sequence Number                  %d\n",tr.rec_number);
    fprintf(fptr,"First record sub-type code              %d\n",
	tr.frst_rec_code);
    fprintf(fptr,"Record type code                        %d\n",tr.rec_code);
    fprintf(fptr,"Second record sub-type code             %d\n",
	tr.scnd_rec_code);
    fprintf(fptr,"Third record sub-type code              %d\n",
	tr.thrd_rec_code);
    fprintf(fptr,"Length of this record	                  %d\n",tr.rec_length);
    fprintf(fptr,"Trailer record sequence number          %d\n",
	tr.record_seq_num);
    fprintf(fptr,"Sequence of trailer rec in bnd          %d\n",tr.seq_in_band);
    fprintf(fptr,"       TRAILER DATA\n");
    fprintf(fptr,"Histogram of RAW data for 4 detectors for the band \n");
    for (i=0; i<256; i++) 
       fprintf(fptr,"   %8d      %8d      %8d     %8d\n",tr.detect_histo[0][i],
	  tr.detect_histo[1][i],tr.detect_histo[2][i],tr.detect_histo[3][i]); 
    fprintf(fptr,"Parity error count	                  %d\n",
	tr.parity_count);
    fprintf(fptr,"Quality summary\n                       %s\n",
	tr.quality_summary);
    fprintf(fptr,"\n\n");
    }    /* End of individual trailer records */
  }  /* End of trailer record */

if (*supp_flag)   /* Print supplemental file */
   {
	 /* needed to split the supplemental file out from this file so it
	    would compile on SUN1 */
   dspsupp(hname, fptr);
   }
  
if (*idr_flag)    /* Print Imagery file */
  {
 /* Open Image Data Record
    ----------------------*/
  c_lsmknm(hname,"idr",finame);
  status = c_lsstat(finame,&mode);    /* make sure file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("IDR file does not exist on disk","ltwg_idr-stat",&one);
     return(E_FAIL);
     }

  fin=fopen(finame,"r");
  if(!fin)
    {
    c_errmsg("Error opening Image Data Records","ltwg_idr-open",&nonfatal);
    return(E_FAIL);
    }

        /* Print File descriptor record for imagery file */
  if ((status = c_ltwg_ifd(&ifd, fin)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }
 
  fprintf(fptr,"IMAGERY FILE - FILE DESCRIPTOR RECORD \n"); 
  fprintf(fptr,"_____________________________________\n\n");
 
  fprintf(fptr,"Record Sequence Number 	                 %d\n",ifd.rec_number);
  fprintf(fptr,"First record sub-type code               %d\n",
	ifd.frst_rec_code);
  fprintf(fptr,"Record type code                         %d\n",ifd.rec_code);
  fprintf(fptr,"Second record sub-type code              %d\n",
	ifd.scnd_rec_code);
  fprintf(fptr,"Third record sub-type code               %d\n",
	ifd.thrd_rec_code);
  fprintf(fptr,"Length of this record                    %d\n",ifd.rec_length);
  fprintf(fptr,"ASCII/EBCDIC flag                        %s\n",ifd.a_e_flag);
  fprintf(fptr,"Control document number                  %s\n",
	ifd.control_doc_num);
  fprintf(fptr,"Control document revision                %s\n",
	ifd.control_doc_rev);
  fprintf(fptr,"File design descriptor revision          %s\n",
	ifd.file_des_rev);
  fprintf(fptr,"Software release Number                  %s\n",ifd.rel_num);
  fprintf(fptr,"File number                              %d\n",ifd.file_num);
  fprintf(fptr,"File name                                %s\n",ifd.file_name);
  fprintf(fptr,"Record seq and location type             %s\n",ifd.record_seq);
  fprintf(fptr,"Sequence number location                 %d\n",
	ifd.seq_num_loc);
  fprintf(fptr,"Sequence number field length             %d\n",
	ifd.seq_num_lgt);
  fprintf(fptr,"Record code and location type            %s\n",
	ifd.rec_loc_flg);
  fprintf(fptr,"Record code location                     %d\n",ifd.rec_loc);
  fprintf(fptr,"Record code field length                 %d\n",ifd.rec_lth);
  fprintf(fptr,"Record length and location type          %s\n",
	ifd.rec_lth_flg);
  fprintf(fptr,"Record length location                   %d\n",
	ifd.rec_lth_loc);
  fprintf(fptr,"Record length field length               %d\n",
	ifd.rec_fld_lth);
  fprintf(fptr,"Data interpretation included in descriptor record -      %s\n",
      ifd.data_interp_des);
  fprintf(fptr,"Data interpretation included in other file -             %s\n",
      ifd.data_interp_oth);
  fprintf(fptr,"Data display information included in descriptor record - %s\n",
      ifd.data_disp_des);
  fprintf(fptr,"Data display information included in other file -        %s\n",
      ifd.data_disp_oth);
  fprintf(fptr,"Number of image records                  %d\n",
	ifd.number_img_rec);
  fprintf(fptr,"Record length                            %d\n",ifd.record_lth);
  fprintf(fptr,"Number of bits per pixel                 %d\n",
	ifd.bits_per_pixel);
  fprintf(fptr,"Number of pixel/data group               %d\n",
	ifd.pixel_per_group);
  fprintf(fptr,"Number of bytes/data group               %d\n",
	ifd.bytes_per_group);
  fprintf(fptr,"Pixel justification 	                 %s\n",ifd.pix_just);
  fprintf(fptr,"Number of bands this file                %d\n",ifd.num_bands);
  fprintf(fptr,"Lines per band                           %d\n",ifd.lines_band);
  fprintf(fptr,"Left border pixel/line                   %d\n",
	ifd.left_pix_line);
  fprintf(fptr,"Image pixels per line                    %d\n",
	ifd.img_pix_line);
  fprintf(fptr,"Right border pixel/line                  %d\n",
	ifd.right_pix_line);
  fprintf(fptr,"Number of top border lines               %d\n",ifd.top_lines);
  fprintf(fptr,"Number of bottom border lines            %d\n",ifd.bot_lines);
  fprintf(fptr,"Type of file                             %s\n",ifd.file_type);
  fprintf(fptr,"Number of Phy records per line           %d\n",
	ifd.records_line);
  fprintf(fptr,"Phy records/multispectrial line          %d\n",
	ifd.recs_multispec_line);
  fprintf(fptr,"Bytes of prefix data/log rec             %d\n",
	ifd.size_prefix);
  fprintf(fptr,"Bytes of image data/log rec              %d\n",ifd.size_image);
  fprintf(fptr,"Bytes of suffix data/log rec             %d\n",
	ifd.size_suffix);
  fprintf(fptr,"Scan line number locator                 %s\n",
	ifd.line_num_loc);
  fprintf(fptr,"Banc number locator                      %s\n",
	ifd.band_num_loc);
  fprintf(fptr,"Scan location time     	                 %s\n",
	ifd.location_time);
  fprintf(fptr,"Left fill count locator                  %s\n",
	ifd.left_fill_loc);
  fprintf(fptr,"Right fill count locator                 %s\n",
	ifd.right_fill_loc);
  fprintf(fptr,"Unknown field 1	                         %s\n",ifd.unknown1);
  fprintf(fptr,"Unknown field 2	                         %s\n",ifd.unknown2);
  fprintf(fptr,"Left fill/pixel	                         %d\n",ifd.left_fill);
  fprintf(fptr,"Right fill/pixel	                 %d\n",ifd.right_fill);
  fprintf(fptr,"Unknown field 3	                         %s\n",ifd.unknown3);
  fprintf(fptr,"\n\n");
     /* End of file descriptor record */
  
     /* Print image data record for a user specified line */
/* Skip to requested line number and output the data for only that line */
  fseek(fin,(88*(*line_num-1)),SEEK_CUR);  
  if ((status = c_ltwg_idr(&idr, fin)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }
  fprintf(fptr,"IMAGERY FILE - IMAGE LINE DATA RECORD LINE %d\n",*line_num); 
  fprintf(fptr,"________________________________________________\n\n");

  fprintf(fptr,"      PREFIX DATA \n"); 
  fprintf(fptr,"Scan line number                         %d\n",idr.scan_line);
  fprintf(fptr,"Logical band number                      %d\n",idr.band_num);
  fprintf(fptr,"Time in GMT at start of scan             %d ms\n",
	idr.start_time);
  fprintf(fptr,"Count of left fill pixels                %d\n",idr.left_fill);
  fprintf(fptr,"Count of right fill pixels               %d\n",idr.right_fill);
  fprintf(fptr,"      SUFFIX DATA  \n");
  fprintf(fptr,"Sync loss indicator 1-yes 0-no           %d\n",idr.sync_loss);
  fprintf(fptr,"Local use quality code 1                 %d\n",
	idr.local_qual_flag1);
  fprintf(fptr,"Local use quality code 2                 %d\n",
	idr.local_qual_flag2);
  fprintf(fptr,"Detector substitution indcator           %d\n",idr.detect_sub);
  fprintf(fptr,"Calibration pulse width                  %d\n",
	idr.cal_pulse_width);
  fprintf(fptr,"Counted full-scan line length            %d\n",idr.line_lgth);
  fprintf(fptr,"Embedded line length	                 %d\n",
	idr.embedded_lgth);
  fprintf(fptr,"Time error-line start - midscan          %d\n",
	idr.time_err_start);
  fprintf(fptr,"Time error-midscan - line end            %d\n",
	idr.time_err_end);
  fprintf(fptr,"Scan line direction 0-forward 1-reverse  %d\n",
	idr.scan_direction);
  fprintf(fptr,"Current half-scan line length            %d\n",
	idr.half_line_lngth);
  fprintf(fptr,"Satellite time code, start               %s\n",
	idr.sat_time_code);
  fprintf(fptr,"      CALIBRATION INFORMATION \n");
  fprintf(fptr,"Detector identification                  %d\n",idr.detect_id);
  fprintf(fptr,"Calibration lamp value quality           %d\n",
	idr.calib_lamp_qual);
  fprintf(fptr,"Calibration lamp state                   %d\n",
	idr.calib_lamp_state);
  fprintf(fptr,"Calibration state sequence numb          %d\n",
	idr.calib_state_seq);
  fprintf(fptr,"Low level calib before restore           %d thousandths\n",
	  idr.low_calib_value_bef);
  fprintf(fptr,"High level calib                         %d thousandths\n",
	  idr.high_calib_value);
  fprintf(fptr,"Computed gain val 	                 %d millionths\n",
	  idr.comput_gain);
  fprintf(fptr,"Computed bais val 	                 %d millionths\n",
	  idr.comput_bais);
  fprintf(fptr,"Applied gain val 	                 %d millionths\n",
	  idr.app_gain);
  fprintf(fptr,"Applied bais val	                 %d millionths\n",
	  idr.app_bais);
  fprintf(fptr,"Low level calib after restore            %d thousandths\n",
	  idr.low_calib_value_aft);
  fprintf(fptr,"\n\n");
      /* End Image line data record */
  }   /* End Image data record */

return(E_SUCC);
}		/* end of dspltwg */
