/************************************
*  Common includes and definitions  *
************************************/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "siat.h"
#include "labserv.h"
#include "header.h"
#include "protos.h"

/*******************************************************************************
FUNCTION:	dspsiat	

PURPOSE: 	To read the siat file, fill the SIAT structure,
		and display the siat file.	

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
1.0	 T. Mittan	 12/92	 LAS	Original development.

ALGORITHM REFERENCES	none
*******************************************************************************/
int dspsiat
(
    char *name,            	/* TAE name of input image              */
    char *prtfile          	/* print file name                      */
)

{
   FILE *fp;			/* file pointer				*/
   struct SIAT siat;		/* SIAT sturcture declaration		*/

   long  sflag = 0;			/* siat flag				*/
   long  i;			/* array index				*/
   long  nonfatal = 1;		/* nonfatal flag 			*/

   char  hostheader[CMLEN];	/* header name				*/
   struct stat status_buf;	/* structure for file statistics	*/

   long  get_long();
   short get_short();
   float get_fraction();
   float get_frac();
   float get_tick();

  c_lsmknm(name,"cctx_siat",hostheader);/* make associated header file name */

  /* use data from logical tape record
  -----------------------------------*/
  fp = (fopen(hostheader,"r"));
  stat(hostheader,&status_buf);
  fclose(fp);
  if(status_buf.st_size==3658) 
     sflag = 1;
  else 
  if(status_buf.st_size==4168) 
     sflag = 2;
  else 
  if(status_buf.st_size==4138) 
     sflag = 3;
  
   if (c_read_siat(&siat,name) == E_FAIL)
     return(E_FAIL);

  /* open the print file
  ----------------------*/
  fp = fopen(prtfile,"w");
  if (fp == NULL)
     {
     c_errmsg("Error opening print file","dspassoc-open",&nonfatal);
     return(E_FAIL);
     }

  /* Read logical tape information
  ------------------------------*/
  if (sflag != 3)
     {
     fprintf(fp,"Information from logical tape record \n");
     fprintf(fp,"____________________________________ \n\n");
     fprintf(fp,"Siat filename	        %s \n\n", hostheader);
     fprintf(fp,"tape_number            %s \n",siat.tape_number);
     fprintf(fp,"tape_date              %s \n",siat.tape_date);
     fprintf(fp,"tape_siat_number       %s \n",siat.tape_siat_number);
     fprintf(fp,"tape_rbv_number        %s \n",siat.tape_rbv_number);
     fprintf(fp,"tape_mss_number        %s \n",siat.tape_mss_number);
     fprintf(fp,"tape_number_data_files %d \n",siat.tape_number_data_files); 
     fprintf(fp,"tape_number_rbv_vtc    %d \n",siat.tape_number_rbv_vtc);
     fprintf(fp,"tape_number_mss_vtc    %d \n",siat.tape_number_mss_vtc);
     fprintf(fp,"tape_number_rbv_tfc    %d \n",siat.tape_number_rbv_tfc);
     fprintf(fp,"tape_number_mss_tfc    %d \n",siat.tape_number_mss_tfc);
     for (i = 0; i < 64; i++)
         fprintf(fp,"tape_rvb_scene_id[%2d] %s \n",i+1,
					siat.tape_rvb_scene_id[i]);
     for (i = 0; i < 64; i++)
         fprintf(fp,"tape_mss_scene_id[%2d] %s \n",i+1,
					siat.tape_mss_scene_id[i]);
     } 
  else
     {
     fprintf(fp,"Information from logical tape record \n");
     fprintf(fp,"____________________________________ \n\n");
     fprintf(fp,"Siat filename	        %s \n\n",hostheader);
     fprintf(fp,"tape_number            %s \n",siat.tape_number);
     fprintf(fp,"tape_sequence          %s \n",siat.tape_sequence);
     fprintf(fp,"tape_date              %s \n",siat.tape_date);
     fprintf(fp,"tape_siat_number       %s \n",siat.tape_siat_number);
     fprintf(fp,"tape_rbv_number        %s \n",siat.tape_rbv_number);
     fprintf(fp,"tape_mss_number        %s \n",siat.tape_mss_number);
     fprintf(fp,"tape_number_data_files %d \n",siat.tape_number_data_files); 
     fprintf(fp,"tape_number_rbv_vtc    %d \n",siat.tape_number_rbv_vtc);
     fprintf(fp,"tape_number_mss_vtc    %d \n",siat.tape_number_mss_vtc);
     fprintf(fp,"tape_number_rbv_tfc    %d \n",siat.tape_number_rbv_tfc);
     fprintf(fp,"tape_number_mss_tfc    %d \n",siat.tape_number_mss_tfc);
     for (i = 0; i < 64; i++)
         fprintf(fp,"tape_rvb_scene_id[%2d] %s \n",i+1,
					siat.tape_rvb_scene_id[i]);
     for (i = 0; i < 64; i++)
         fprintf(fp,"tape_mss_scene_id[%2d] %s \n",i+1,
					siat.tape_mss_scene_id[i]);
     }
    
  /* use data from calibration modifiers record
  -------------------------------------------*/
  if (sflag != 1)
   {
   fprintf(fp,"\nInformation from Calibration Modifiers record \n");
   fprintf(fp,"_____________________________________________ \n\n");

   /* read calibration modifiers record
   ----------------------------------*/ 
   fprintf(fp,"calibration_id   %d \n",siat.calibration_id);
   fprintf(fp,"calibration_days %d \n",siat.calibration_days);
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band4_low_compressed_multiplicative_sensor[%d] %f \n"
	,i+1,siat.calibration_band4_low_compressed_multiplicative_sensor[i]);
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band4_low_compressed_additive_sensor[%d]       %f \n"
		,i+1,siat.calibration_band4_low_compressed_additive_sensor[i]);
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band4_low_linear_multiplicative_sensor[%d]     %f \n"
	,i+1,siat.calibration_band4_low_linear_multiplicative_sensor[i]);
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band4_low_linear_additive_sensor[%d]           %f \n"
		,i+1,siat.calibration_band4_low_linear_additive_sensor[i]); 
for (i = 0; i < 6; i++)
  fprintf(fp,"calibration_band4_high_compressed_multiplicative_sensor[%d] %f \n"
	,i+1,siat.calibration_band4_high_compressed_multiplicative_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band4_high_compressed_additive_sensor[%d]      %f \n"
		,i+1,siat.calibration_band4_high_compressed_additive_sensor[i]);
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band4_high_linear_multiplicative_sensor[%d]    %f \n"
	,i+1,siat.calibration_band4_high_linear_multiplicative_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band4_high_linear_additive_sensor[%d]          %f \n"
		,i+1,siat.calibration_band4_high_linear_additive_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band5_low_compressed_multiplicative_sensor[%d] %f \n"
	,i+1,siat.calibration_band5_low_compressed_multiplicative_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band5_low_compressed_additive_sensor[%d]       %f \n"
		,i+1,siat.calibration_band5_low_compressed_additive_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band5_low_linear_multiplicative_sensor[%d]     %f \n"
	,i+1,siat.calibration_band5_low_linear_multiplicative_sensor[i]);
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band5_low_linear_additive_sensor[%d]           %f \n"
	,i+1,siat.calibration_band5_low_linear_additive_sensor[i]); 
for (i = 0; i < 6; i++)
  fprintf(fp,"calibration_band5_high_compressed_multiplicative_sensor[%d] %f \n"
	,i+1,siat.calibration_band5_high_compressed_multiplicative_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band5_high_compressed_additive_sensor[%d]      %f \n"
		,i+1,siat.calibration_band5_high_compressed_additive_sensor[i]);
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band5_high_linear_multiplicative_sensor[%d]    %f \n"
	,i+1,siat.calibration_band5_high_linear_multiplicative_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band5_high_linear_additive_sensor[%d]          %f \n"
		,i+1,siat.calibration_band5_high_linear_additive_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band6_low_linear_multiplicative_sensor[%d]     %f \n"
	,i+1,siat.calibration_band6_low_linear_multiplicative_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band6_low_linear_additive_sensor[%d]           %f \n"
		,i+1,siat.calibration_band6_low_linear_additive_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band6_low_compressed_multiplicative_sensor[%d] %f \n"
	,i+1,siat.calibration_band6_low_compressed_multiplicative_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band6_low_compressed_additive_sensor[%d]       %f \n"
	,i+1,siat.calibration_band6_low_compressed_additive_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band7_low_linear_multiplicative_sensor[%d]     %f \n"
	,i+1,siat.calibration_band7_low_linear_multiplicative_sensor[i]); 
for (i = 0; i < 6; i++)
   fprintf(fp,"calibration_band7_low_linear_additive_sensor[%d]           %f \n"
		,i+1,siat.calibration_band7_low_linear_additive_sensor[i]); 
   }

  /* use data from processing record
  --------------------------------*/
  if (sflag != 3)
    {
    fprintf(fp,"\nInformation from Processing Instruction record \n");
    fprintf(fp,"______________________________________________ \n\n");

    fprintf(fp,"processing_scenes_remaining_rbv_vfc %d \n",
			siat.processing_scenes_remaining_rbv_vfc); 
    fprintf(fp,"processing_scenes_remaining_mss_vfc %d \n",
			siat.processing_scenes_remaining_mss_vfc); 
    fprintf(fp,"processing_scenes_remaining_rbv_vt  %d \n",
			siat.processing_scenes_remaining_rbv_vtc); 
    fprintf(fp,"processing_scenes_remaining_mss_vtc %d \n",
			siat.processing_scenes_remaining_mss_vtc); 
    fprintf(fp,"processing_scene_id                 %s \n",
			siat.processing_scene_id);
    fprintf(fp,"processing_preceding_id             %s \n",
			siat.processing_preceding_id);
    fprintf(fp,"processing_succeedingid             %s \n",
			siat.processing_succeedingid);
    fprintf(fp,"processing_mission_number           %d \n",
			siat.processing_mission_number); 
    fprintf(fp,"processing_day_number_from_launch   %d \n",
			siat.processing_day_number_from_launch); 
    fprintf(fp,"processing_hours                    %d \n",
			siat.processing_hours); 
    fprintf(fp,"processing_minutes                  %d \n",
			siat.processing_minutes); 
    fprintf(fp,"processing_seconds                  %d \n",
			siat.processing_seconds); 
    for (i = 0; i < 8; i++)
       {
       fprintf(fp,"processing_band_information[%d]      %s \n",i+1,
			siat.processing_band_information[i]);
       }
    fprintf(fp,"processing_instructions             %s \n",
					siat.processing_instructions);
    fprintf(fp,"processing_mission_number2          %d \n",
			siat.processing_mission_number2); 
    fprintf(fp,"processing_day_number_from_launch2  %d \n",
			siat.processing_day_number_from_launch2); 
    fprintf(fp,"processing_hours2                   %d \n",
			siat.processing_hours2); 
    fprintf(fp,"processing_minutes2  %d \n",siat.processing_minutes2); 
    fprintf(fp,"processing_seconds2  %d \n",siat.processing_seconds2); 
    fprintf(fp,"processing_code_slat %d \n",siat.processing_code_slat); 
    fprintf(fp,"processing_code_mss  %d \n",siat.processing_code_mss); 
    fprintf(fp,"processing_code_psp  %d \n",siat.processing_code_psp); 
    }
  else
    {
    fprintf(fp,"\nInformation from Processing Instruction record \n");
    fprintf(fp,"______________________________________________ \n\n");

    fprintf(fp,"processing_scenes_remaining_rbv_vfc %d \n",
			siat.processing_scenes_remaining_rbv_vfc); 
    fprintf(fp,"processing_scenes_remaining_mss_vfc %d \n",
			siat.processing_scenes_remaining_mss_vfc); 
    fprintf(fp,"processing_scenes_remaining_rbv_vtc %d \n",
			siat.processing_scenes_remaining_rbv_vtc); 
    fprintf(fp,"processing_scenes_remaining_mss_vtc %d \n",
			siat.processing_scenes_remaining_mss_vtc); 
    fprintf(fp,"processing_scene_id                 %s \n",
			siat.processing_scene_id);
    fprintf(fp,"processing_mission_number           %d \n",siat.
			processing_mission_number); 
    fprintf(fp,"processing_day_number_from_launch   %d \n",
			siat.processing_day_number_from_launch); 
    fprintf(fp,"processing_hours     %d \n",siat.processing_hours); 
    fprintf(fp,"processing_minutes   %d \n",siat.processing_minutes); 
    fprintf(fp,"processing_seconds   %d \n",siat.processing_seconds); 
    fprintf(fp,"processing_mission_number2          %d \n",
			siat.processing_mission_number2); 
    fprintf(fp,"processing_day_number_from_launch2  %d \n",
			siat.processing_day_number_from_launch2); 
    fprintf(fp,"processing_hours2    %d \n",siat.processing_hours2); 
    fprintf(fp,"processing_minutes2  %d \n",siat.processing_minutes2); 
    fprintf(fp,"processing_seconds2  %d \n",siat.processing_seconds2); 
    fprintf(fp,"processing_code_slat %d \n",siat.processing_code_slat); 
    fprintf(fp,"processing_code_mss  %d \n",siat.processing_code_mss); 
    fprintf(fp,"processing_code_psp  %d \n",siat.processing_code_psp); 
    }

  /* use data from spacecraft performance record
  --------------------------------------------*/
  if (sflag != 3)  
    {
    fprintf(fp,"\nInformation from Spacecraft Performance record \n");
    fprintf(fp,"________________________________________________ \n\n");

    fprintf(fp,"spacecraft_rbv1_transmission     %s \n",
				siat.spacecraft_rbv1_transmission);
    fprintf(fp,"spacecraft_rbv1_duration         %s \n",
				siat.spacecraft_rbv1_duration);
    fprintf(fp,"spacecraft_rbv1_aperture         %s \n",
				siat.spacecraft_rbv1_aperture);
    fprintf(fp,"spacecraft_rbv2_transmission     %s \n",
				siat.spacecraft_rbv2_transmission);
    fprintf(fp,"spacecraft_rbv2_duration         %s \n",
				siat.spacecraft_rbv2_duration);
    fprintf(fp,"spacecraft_rbv2_aperture         %s \n",
				siat.spacecraft_rbv2_aperture);
    fprintf(fp,"spacecraft_rbv3_transmission     %s \n",
				siat.spacecraft_rbv3_transmission);
    fprintf(fp,"spacecraft_rbv3_duration         %s \n",
				siat.spacecraft_rbv3_duration);
    fprintf(fp,"spacecraft_rbv3_aperture         %s \n",
				siat.spacecraft_rbv3_aperture);
    fprintf(fp,"spacecraft_rbv4_transmission     %s \n",
				siat.spacecraft_rbv4_transmission);
    fprintf(fp,"spacecraft_rbv5_transmission     %s \n",
				siat.spacecraft_rbv5_transmission);
    fprintf(fp,"spacecraft_rbv6_transmission     %s \n",
				siat.spacecraft_rbv6_transmission);
    fprintf(fp,"spacecraft_rbv7_transmission     %s \n",
				siat.spacecraft_rbv7_transmission);
    fprintf(fp,"spacecraft_rbv8_transmission     %s \n",
				siat.spacecraft_rbv8_transmission);
    fprintf(fp,"spacecraft_sensor_gain_band4     %d \n",
				siat.spacecraft_sensor_gain_band4);
    fprintf(fp,"spacecraft_sensor_gain_band5     %d \n",
				siat.spacecraft_sensor_gain_band5);
    fprintf(fp,"spacecraft_sensor_encoding_band4 %d \n",
				siat.spacecraft_sensor_encoding_band4);
    fprintf(fp,"spacecraft_sensor_encoding_band5 %d \n",
				siat.spacecraft_sensor_encoding_band5);
    fprintf(fp,"spacecraft_sensor_encoding_band6 %d \n",
				siat.spacecraft_sensor_encoding_band6);
    fprintf(fp,"spacecraft_tape_id               %s \n",
				siat.spacecraft_tape_id);
    fprintf(fp,"spacecraft_sun_day               %s \n",
				siat.spacecraft_sun_day);
    for (i = 0; i < 24; i++)
      {
      fprintf(fp,"spacecraft_processing_mss_sun_sensor[%2d] %d \n",i+1,
				siat.processing_mss_sun_sensor[i]); 
      }
    fprintf(fp,"spacecraft_sun_day2              %s \n",
				siat.spacecraft_sun_day2);
    fprintf(fp,"spacecraft_sun_flag              %s \n",
				siat.spacecraft_sun_flag);
    }
  else
    {
    fprintf(fp,"\nInformation from Spacecraft Performance record \n");
    fprintf(fp,"________________________________________________ \n\n");

    fprintf(fp,"spacecraft_rbv1_transmission %s \n",
				siat.spacecraft_rbv1_transmission);
    fprintf(fp,"spacecraft_rbv1_duration     %s \n",
				siat.spacecraft_rbv1_duration);
    fprintf(fp,"spacecraft_rbv1_aperture     %s \n",
				siat.spacecraft_rbv1_aperture);
    fprintf(fp,"spacecraft_rbv2_transmission %s \n",
				siat.spacecraft_rbv2_transmission);
    fprintf(fp,"spacecraft_rbv2_duration     %s \n",
				siat.spacecraft_rbv2_duration);
    fprintf(fp,"spacecraft_rbv2_aperture     %s \n",
				siat.spacecraft_rbv2_aperture);
    fprintf(fp,"spacecraft_rbv3_transmission %s \n",
				siat.spacecraft_rbv3_transmission);
    fprintf(fp,"spacecraft_rbv3_duration     %s \n",
				siat.spacecraft_rbv3_duration);
    fprintf(fp,"spacecraft_rbv3_aperture     %s \n",
				siat.spacecraft_rbv3_aperture);
    fprintf(fp,"spacecraft_rbv4_transmission %s \n",
				siat.spacecraft_rbv4_transmission);
    fprintf(fp,"spacecraft_rbv5_transmission %s \n",
				siat.spacecraft_rbv5_transmission);
    fprintf(fp,"spacecraft_rbv6_transmission %s \n",
				siat.spacecraft_rbv6_transmission);
    fprintf(fp,"spacecraft_rbv7_transmission %s \n",
				siat.spacecraft_rbv7_transmission);
    fprintf(fp,"spacecraft_rbv8_transmission %s \n",
				siat.spacecraft_rbv8_transmission);

    fprintf(fp,"spacecraft_sensor_gain_band4 %d \n",
				siat.spacecraft_sensor_gain_band4);
    fprintf(fp,"spacecraft_sensor_gain_band5 %d \n",
				siat.spacecraft_sensor_gain_band5);
    fprintf(fp,"spacecraft_sensor_encoding_band4 %d \n",
				siat.spacecraft_sensor_encoding_band4);
    fprintf(fp,"spacecraft_sensor_encoding_band5 %d \n",
				siat.spacecraft_sensor_encoding_band5);
    fprintf(fp,"spacecraft_sensor_encoding_band6 %d \n",
				siat.spacecraft_sensor_encoding_band6);
    fprintf(fp,"spacecraft_tape_id               %s \n",
				siat.spacecraft_tape_id);
    fprintf(fp,"spacecraft_sun_day               %s \n",
				siat.spacecraft_sun_day);
    for (i = 0; i < 24; i++)
      {
      fprintf(fp,"processing_mss_sun_sensor[%2d] %d \n",i+1,
				siat.processing_mss_sun_sensor[i]); 
      }
    }

  /* use data from annotation record
  --------------------------------*/
  if (sflag != 3) 
    {
    fprintf(fp,"\nInformation from Annotation record \n");
    fprintf(fp,"____________________________________ \n\n");

    fprintf(fp,"annotation_day             %s \n",siat.annotation_day);
    fprintf(fp,"annotation_month           %s \n",siat.annotation_month);
    fprintf(fp,"annotation_year            %s \n",siat.annotation_year);
    fprintf(fp,"annotation_latitude        %s \n",siat.annotation_latitude);
    fprintf(fp,"annotation_longitude       %s \n",siat.annotation_longitude);
    fprintf(fp,"annotation_nadir_latitude  %s \n",
					siat.annotation_nadir_latitude);
    fprintf(fp,"annotation_nadir_longitude %s \n",
					siat.annotation_nadir_longitude);
    fprintf(fp,"annotation_sun_elevation   %s \n",
					siat.annotation_sun_elevation);
    fprintf(fp,"annotation_sun_azimuth     %s \n",siat.annotation_sun_azimuth);
    fprintf(fp,"annotation_heading         %s \n",siat.annotation_heading);
    fprintf(fp,"annotation_rev             %s \n",siat.annotation_rev);
    fprintf(fp,"annotation_data_acquistion %s \n",
					siat.annotation_data_acquistion);
    fprintf(fp,"annotation_orbit_type      %s \n",siat.annotation_orbit_type);
    fprintf(fp,"annotation_scene_id        %s \n",siat.annotation_scene_id);
    fprintf(fp,"annotation_rci_calibration %s \n",
					siat.annotation_rci_calibration);
    fprintf(fp,"annotation_rbv_mode1       %s \n",siat.annotation_rbv_mode1);
    fprintf(fp,"annotation_rbv_shutter1    %s \n",siat.annotation_rbv_shutter1);
    fprintf(fp,"annotation_rbv_mode2       %s \n",siat.annotation_rbv_mode2);
    fprintf(fp,"annotation_rbv_shutter2    %s \n",siat.annotation_rbv_shutter2);
    fprintf(fp,"annotation_rbv_mode3       %s \n",siat.annotation_rbv_mode3);
    fprintf(fp,"annotation_rbv_shutter3    %s \n",siat.annotation_rbv_shutter3);
    fprintf(fp,"annotation_mss_mode        %s \n",siat.annotation_mss_mode);
    }
  else
    {
    fprintf(fp,"\nInformation from Annotation record \n");
    fprintf(fp,"____________________________________ \n\n");

    fprintf(fp,"annotation_day             %s \n",siat.annotation_day);
    fprintf(fp,"annotation_month           %s \n",siat.annotation_month);
    fprintf(fp,"annotation_year            %s \n",siat.annotation_year);
    fprintf(fp,"annotation_latitude        %s \n",siat.annotation_latitude);
    fprintf(fp,"annotation_longitud        %s \n",siat.annotation_longitude);
    fprintf(fp,"annotation_direction       %s \n",siat.annotation_direction);
    fprintf(fp,"annotation_path_row        %s \n",siat.annotation_path_row);
    fprintf(fp,"annotation_nadir_latitude  %s \n",
					siat.annotation_nadir_latitude);
    fprintf(fp,"annotation_nadir_longitude %s \n",
					siat.annotation_nadir_longitude);
    fprintf(fp,"annotation_sensor_code     %s \n",siat.annotation_sensor_code);
    fprintf(fp,"annotation_designated_band %s \n",
					siat.annotation_designated_band);
    fprintf(fp,"annotation_transmission_mode %s \n",
					siat.annotation_transmission_mode);
    fprintf(fp,"annotation_sun_elevation     %s \n",
					siat.annotation_sun_elevation);
    fprintf(fp,"annotation_sun_azimuth       %s \n",
					siat.annotation_sun_azimuth);
    fprintf(fp,"annotation_image_prcessing_codes %s \n",
					siat.annotation_image_prcessing_codes);
    fprintf(fp,"annotation_scene_id          %s \n",siat.annotation_scene_id);
    fprintf(fp,"annotation_rbv               %s \n",siat.annotation_rbv);
    fprintf(fp,"annotation_mss_mode_site     %s \n",
					siat.annotation_mss_mode_site);
    }


  /* use data from RBV computational record
  ---------------------------------------*/
  fprintf(fp,"\nInformation from RBV Computational record \n");
  fprintf(fp,"___________________________________________ \n\n");

  fprintf(fp,"rbv_spacecraft_ddd  %d \n",siat.rbv_spacecraft_ddd);
  fprintf(fp,"rbv_spacecraft_hh   %d \n",siat.rbv_spacecraft_hh);
  fprintf(fp,"rbv_spacecraft_mm   %d \n",siat.rbv_spacecraft_mm);
  fprintf(fp,"rbv_spacecraft_ss   %d \n",siat.rbv_spacecraft_ss);
  fprintf(fp,"rbv_spacecraft_cc   %d \n",siat.rbv_spacecraft_cc);
 
  fprintf(fp,"rbv_greenwich_ddd   %d \n",siat.rbv_greenwich_ddd);
  fprintf(fp,"rbv_greenwich_hh    %d \n",siat.rbv_greenwich_hh);
  fprintf(fp,"rbv_greenwich_mm    %d \n",siat.rbv_greenwich_mm);
  fprintf(fp,"rbv_greenwich_ss    %d \n",siat.rbv_greenwich_ss);
  fprintf(fp,"rbv_greenwich_mmm   %d \n",siat.rbv_greenwich_mmm);

  fprintf(fp,"rbv_altitude_change %f \n",siat.rbv_altitude_change);
  fprintf(fp,"rbv_gmt_date        %s \n",siat.rbv_gmt_date);
  fprintf(fp,"rbv_gmt_time        %s \n",siat.rbv_gmt_time);
  fprintf(fp,"rbv_latitude_format_center  %f \n",
					siat.rbv_latitude_format_center);
  fprintf(fp,"rbv_longitude_format_center %f \n",
					siat.rbv_longitude_format_center);
  fprintf(fp,"rbv_latitude_nadir          %f \n",siat.rbv_latitude_nadir);
  fprintf(fp,"rbv_longitude_nadir         %f \n",siat.rbv_longitude_nadir);
  fprintf(fp,"rbv_altitude        %f \n",siat.rbv_altitude);
  fprintf(fp,"rbv_gmt_exposure    %f \n",siat.rbv_gmt_exposure);
  fprintf(fp,"rbv_flight_heading  %f \n",siat.rbv_flight_heading);
  fprintf(fp,"rbv_pitch           %f \n",siat.rbv_pitch);
  fprintf(fp,"rbv_roll            %f \n",siat.rbv_roll);
  fprintf(fp,"rbv_yaw             %f \n",siat.rbv_yaw);

  /* use data from mss computational record
  ---------------------------------------*/
  fprintf(fp,"\nInformation from MSS Computational record \n");
  fprintf(fp,"___________________________________________ \n\n");

 fprintf(fp,"Spacecraft time at scene center:  %3d:%2d:%2d:%2d:%2d\n",
        siat.mss_timesc_ddd,siat.mss_timesc_hh, siat.mss_timesc_mm, 
	siat.mss_timesc_ss, siat.mss_timesc_cc);
 fprintf(fp,"GMT of scene center:              %3d:%2d:%2d:%2d:%3d\n\n",
        siat.mss_gmtsc_ddd, siat.mss_gmtsc_hh, siat.mss_gmtsc_mm, 
	siat.mss_gmtsc_ss, siat.mss_gmtsc_mmm);

 fprintf(fp,"Normalized altitude change at times:\n");
 fprintf(fp,"  image center - 13.80300:  %f\n", siat.mss_norm_alt_change_ic[0]);
 fprintf(fp,"  image center - 10.35225:  %f\n", siat.mss_norm_alt_change_ic[1]);
 fprintf(fp,"  image center -  6.90150:  %f\n", siat.mss_norm_alt_change_ic[2]);
 fprintf(fp,"  image center -  3.45075:  %f\n", siat.mss_norm_alt_change_ic[3]);
 fprintf(fp,"  image center time:        %f\n", siat.mss_norm_alt_change_ic[4]);
 fprintf(fp,"  image center +  3.45075:  %f\n", siat.mss_norm_alt_change_ic[5]);
 fprintf(fp,"  image center +  6.90150:  %f\n", siat.mss_norm_alt_change_ic[6]);
 fprintf(fp,"  image center + 10.35225:  %f\n", siat.mss_norm_alt_change_ic[7]);
 fprintf(fp,"  image center + 13.80300:  %f\n\n",	
				siat.mss_norm_alt_change_ic[8]);

 fprintf(fp,"Altitude (N.M.) at times:\n");
 fprintf(fp,"  image center - 13.80300:  %f\n", siat.mss_alt[0]);
 fprintf(fp,"  image center - 10.35225:  %f\n", siat.mss_alt[1]);
 fprintf(fp,"  image center -  6.90150:  %f\n", siat.mss_alt[2]);
 fprintf(fp,"  image center -  3.45075:  %f\n", siat.mss_alt[3]);
 fprintf(fp,"  image center time:        %f\n", siat.mss_alt[4]);
 fprintf(fp,"  image center +  3.45075:  %f\n", siat.mss_alt[5]);
 fprintf(fp,"  image center +  6.90150:  %f\n", siat.mss_alt[6]);
 fprintf(fp,"  image center + 10.35225:  %f\n", siat.mss_alt[7]);
 fprintf(fp,"  image center + 13.80300:  %f\n\n", siat.mss_alt[8]);

 fprintf(fp,"Vehicle Roll at image center (rad):  %f\n", 
				siat.mss_vehicle_roll_ic);
 fprintf(fp,"Vehicle Pitch at image center (rad): %f\n", 
				siat.mss_vehicle_pitch_ic);
 fprintf(fp,"Vehicle Yaw at image center (rad):   %f\n\n", 
				siat.mss_vehicle_yaw_ic);
 fprintf(fp,"Roll at times:\n");
 fprintf(fp,"  image center - 13.80300:  %f\n", siat.mss_roll[0]);
 fprintf(fp,"  image center - 10.35225:  %f\n", siat.mss_roll[1]);
 fprintf(fp,"  image center -  6.90150:  %f\n", siat.mss_roll[2]);
 fprintf(fp,"  image center -  3.45075:  %f\n", siat.mss_roll[3]);
 fprintf(fp,"  image center time:        %f\n", siat.mss_roll[4]);
 fprintf(fp,"  image center +  3.45075:  %f\n", siat.mss_roll[5]);
 fprintf(fp,"  image center +  6.90150:  %f\n", siat.mss_roll[6]);
 fprintf(fp,"  image center + 10.35225:  %f\n", siat.mss_roll[7]);
 fprintf(fp,"  image center + 13.80300:  %f\n\n", siat.mss_roll[8]);
 fprintf(fp,"Pitch at times:\n");
 fprintf(fp,"  image center - 13.80300:  %f\n", siat.mss_pitch[0]);
 fprintf(fp,"  image center - 10.35225:  %f\n", siat.mss_pitch[1]);
 fprintf(fp,"  image center -  6.90150:  %f\n", siat.mss_pitch[2]);
 fprintf(fp,"  image center -  3.45075:  %f\n", siat.mss_pitch[3]);
 fprintf(fp,"  image center time:        %f\n", siat.mss_pitch[4]);
 fprintf(fp,"  image center +  3.45075:  %f\n", siat.mss_pitch[5]);
 fprintf(fp,"  image center +  6.90150:  %f\n", siat.mss_pitch[6]);
 fprintf(fp,"  image center + 10.35225:  %f\n", siat.mss_pitch[7]);
 fprintf(fp,"  image center + 13.80300:  %f\n\n", siat.mss_pitch[8]);
 fprintf(fp,"Yaw at times:\n");
 fprintf(fp,"  image center - 13.80300:  %f\n", siat.mss_yaw[0]);
 fprintf(fp,"  image center - 10.35225:  %f\n", siat.mss_yaw[1]);
 fprintf(fp,"  image center -  6.90150:  %f\n", siat.mss_yaw[2]);
 fprintf(fp,"  image center -  3.45075:  %f\n", siat.mss_yaw[3]);
 fprintf(fp,"  image center time:        %f\n", siat.mss_yaw[4]);
 fprintf(fp,"  image center +  3.45075:  %f\n", siat.mss_yaw[5]);
 fprintf(fp,"  image center +  6.90150:  %f\n", siat.mss_yaw[6]);
 fprintf(fp,"  image center + 10.35225:  %f\n", siat.mss_yaw[7]);
 fprintf(fp,"  image center + 13.80300:  %f\n\n", siat.mss_yaw[8]);
 fprintf(fp,"Image skew (rad):  %f\n\n", siat.mss_skew);
 fprintf(fp,"Normalized velocity change:  %f\n\n", siat.mss_norm_vel_change);
 fprintf(fp,"Mean pitch (rad):  %f\n", siat.mss_mean_pitch);
 fprintf(fp,"Mean roll (rad):   %f\n", siat.mss_mean_roll);
 fprintf(fp,"Mean yaw(rad):     %f\n\n", siat.mss_mean_yaw);
 fprintf(fp,"Mean pitch rate (rad/sec):  %f\n", siat.mss_mean_pitch_rate);
 fprintf(fp,"Mean roll rate (rad/sec):   %f\n", siat.mss_mean_roll_rate);
 fprintf(fp,"Mean yaw rate (rad/sec):    %f\n\n", siat.mss_mean_yaw_rate);
 fprintf(fp,"Mean Mean altitude (meters):      %f\n", siat.mss_mean_alt);
 fprintf(fp,"Mean altitude rate (meters/sec):  %f\n\n", siat.mss_mean_alt_rate);
 fprintf(fp,"GMT milliseconds of Day at:\n");
 fprintf(fp,"  image center time - 25 sec:  %d\n", siat.mss_gmt_ms[0]);
 fprintf(fp,"  image center time - 20 sec:  %d\n", siat.mss_gmt_ms[1]);
 fprintf(fp,"  image center time - 15 sec:  %d\n", siat.mss_gmt_ms[2]);
 fprintf(fp,"  image center time - 10 sec:  %d\n", siat.mss_gmt_ms[3]);
 fprintf(fp,"  image center time -  5 sec:  %d\n", siat.mss_gmt_ms[4]);
 fprintf(fp,"  image center time:           %d\n", siat.mss_gmt_ms[5]);
 fprintf(fp,"  image center time +  5 sec:  %d\n", siat.mss_gmt_ms[6]);
 fprintf(fp,"  image center time + 10 sec:  %d\n", siat.mss_gmt_ms[7]);
 fprintf(fp,"  image center time + 15 sec:  %d\n", siat.mss_gmt_ms[8]);
 fprintf(fp,"  image center time + 20 sec:  %d\n", siat.mss_gmt_ms[9]);
 fprintf(fp,"  image center time + 25 sec:  %d\n\n", siat.mss_gmt_ms[10]);
 fprintf(fp,"Latitude (rad) at times of GMT:\n");
 fprintf(fp,"  image center time - 25 sec:  %f\n", siat.mss_latitude[0]);
 fprintf(fp,"  image center time - 20 sec:  %f\n", siat.mss_latitude[1]);
 fprintf(fp,"  image center time - 15 sec:  %f\n", siat.mss_latitude[2]);
 fprintf(fp,"  image center time - 10 sec:  %f\n", siat.mss_latitude[3]);
 fprintf(fp,"  image center time -  5 sec:  %f\n", siat.mss_latitude[4]);
 fprintf(fp,"  image center time:           %f\n", siat.mss_latitude[5]);
 fprintf(fp,"  image center time +  5 sec:  %f\n", siat.mss_latitude[6]);
 fprintf(fp,"  image center time + 10 sec:  %f\n", siat.mss_latitude[7]);
 fprintf(fp,"  image center time + 15 sec:  %f\n", siat.mss_latitude[8]);
 fprintf(fp,"  image center time + 20 sec:  %f\n", siat.mss_latitude[9]);
 fprintf(fp,"  image center time + 25 sec:  %f\n\n", siat.mss_latitude[10]);
 fprintf(fp,"Longitude (rad) at times of GMT:\n");
 fprintf(fp,"  image center time - 25 sec:  %f\n", siat.mss_longitude[0]);
 fprintf(fp,"  image center time - 20 sec:  %f\n", siat.mss_longitude[1]);
 fprintf(fp,"  image center time - 15 sec:  %f\n", siat.mss_longitude[2]);
 fprintf(fp,"  image center time - 10 sec:  %f\n", siat.mss_longitude[3]);
 fprintf(fp,"  image center time -  5 sec:  %f\n", siat.mss_longitude[4]);
 fprintf(fp,"  image center time:           %f\n", siat.mss_longitude[5]);
 fprintf(fp,"  image center time +  5 sec:  %f\n", siat.mss_longitude[6]);
 fprintf(fp,"  image center time + 10 sec:  %f\n", siat.mss_longitude[7]);
 fprintf(fp,"  image center time + 15 sec:  %f\n", siat.mss_longitude[8]);
 fprintf(fp,"  image center time + 20 sec:  %f\n", siat.mss_longitude[9]);
 fprintf(fp,"  image center time + 25 sec:  %f\n\n", siat.mss_longitude[10]);
 fprintf(fp,"Altitude (meters) at times of GMT:\n");
 fprintf(fp,"  image center time - 25 sec:  %f\n", siat.mss_latlon_alt[0]);
 fprintf(fp,"  image center time - 20 sec:  %f\n", siat.mss_latlon_alt[1]);
 fprintf(fp,"  image center time - 15 sec:  %f\n", siat.mss_latlon_alt[2]);
 fprintf(fp,"  image center time - 10 sec:  %f\n", siat.mss_latlon_alt[3]);
 fprintf(fp,"  image center time -  5 sec:  %f\n", siat.mss_latlon_alt[4]);
 fprintf(fp,"  image center time:           %f\n", siat.mss_latlon_alt[5]);
 fprintf(fp,"  image center time +  5 sec:  %f\n", siat.mss_latlon_alt[6]);
 fprintf(fp,"  image center time + 10 sec:  %f\n", siat.mss_latlon_alt[7]);
 fprintf(fp,"  image center time + 15 sec:  %f\n", siat.mss_latlon_alt[8]);
 fprintf(fp,"  image center time + 20 sec:  %f\n", siat.mss_latlon_alt[9]);
 fprintf(fp,"  image center time + 25 sec:  %f\n\n", siat.mss_latlon_alt[10]);
   
  /* use data from image location  record
  ---------------------------------------*/
  
  fprintf(fp,"\nInformation from Image Location record \n");
  fprintf(fp,"________________________________________ \n\n");

  for (i = 0; i < 6; i++)
     fprintf(fp,"image_rbv_top_point[%d]          %f \n",i+1,
					siat.image_rbv_top_point[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_rbv_top_point_ascii[%d]    %s \n",i+1,
					siat.image_rbv_top_point_ascii[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_rbv_left_point[%d]         %f \n",i+1,
					siat.image_rbv_left_point[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_rbv_left_point_ascii[%d]   %s \n",i+1,
					siat.image_rbv_left_point_ascii[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_rbv_right_point[%d]        %f \n",i+1,
					siat.image_rbv_right_point[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_rbv_right_point_ascii[%d]  %s \n",i+1,
					siat.image_rbv_right_point_ascii[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_rbv_bottom_point[%d]       %f \n",i+1,
					siat.image_rbv_bottom_point[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_rbv_bottom_point_ascii[%d] %s \n",i+1,
					siat.image_rbv_bottom_point_ascii[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_mss_top_point[%d]          %f \n",i+1,
					siat.image_mss_top_point[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_mss_top_point_ascii[%d]    %s \n",i+1,
					siat.image_mss_top_point_ascii[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_mss_left_point[%d]         %f \n",i+1,
					siat.image_mss_left_point[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"siat.image_mss_left_point_ascii[%d] %s \n",i+1,
					siat.image_mss_left_point_ascii[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_mss_right_point[%d]        %f \n",i+1,
					siat.image_mss_right_point[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_mss_right_point_ascii[%d]  %s \n",i+1,
					siat.image_mss_right_point_ascii[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_mss_bottom_point[%d]       %f \n",i+1,
					siat.image_mss_bottom_point[i]);
  for (i = 0; i < 6; i++)
     fprintf(fp,"image_mss_bottom_point_ascii[%d] %s \n",i+1,
					siat.image_mss_bottom_point_ascii[i]);
fclose(fp);
return(E_SUCC);
}
