/***************************************************************************
NAME:			DSPSUPP	

PURPOSE:  Displays the first half of the LTWG supplemtal record information

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		3/94    D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#ifdef sun
#include <unistd.h>
#endif
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "labserv.h"
#include "protos.h"

int dspsupp
(
    char *hname,			/* filename				*/
    FILE *fptr			/* output file pointer			*/
)

{
struct LTWGSFD sfd;		/* LTWG supplemental - file descriptor  */
struct LTWGSIH sih;		/* LTWG supplemental - interval header file */
struct LTWGRJR rjr;		/* LTWG raw jitter measurements ancillary */
struct LTWGMTR mtr;		/* LTWG mission telemetry ancillary rec	*/

long one = 1;
long nonfatal = 1;		/* non fatal error flag			*/
long status;
long mode = 0;			/* flag value for lsstat		*/
long i;
long loop_count;

FILE  *fin;      		/* input file pointer			*/

char finame[CMLEN];		/* tae name				*/

 /* Open Supplemental File
    ----------------------*/
  c_lsmknm(hname,"supp",finame);
  status = c_lsstat(finame,&mode);    /* make sure file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("SUPP file does not exist on disk","ltwg_supp-stat",&one);
     return(E_FAIL);
     }

  fin=fopen(finame,"r");
  if(!fin)
     {
     c_errmsg("Error opening Supplemenntal File","ltwg_supp-open",&nonfatal);
     return(E_FAIL);
     }

         /* Print file descriptor record for supp file */
  if ((status = c_ltwg_sfd(&sfd, fin)) == E_FAIL)
     {
     fclose(fptr);
     return(E_FAIL);
     }

  fprintf(fptr,"SUPPLEMENTAL VOLUME - FILE DESCRIPTOR RECORD \n");
  fprintf(fptr,"____________________________________________\n\n");

  fprintf(fptr,"Record Sequence Number                 %d\n",sfd.rec_number);
  fprintf(fptr,"First record sub-type code             %d\n",
	sfd.frst_rec_code);
  fprintf(fptr,"Record type code                       %d\n",sfd.rec_code);
  fprintf(fptr,"Second record sub-type code            %d\n",
	sfd.scnd_rec_code);
  fprintf(fptr,"Third record sub-type code             %d\n",
	sfd.thrd_rec_code);
  fprintf(fptr,"Record length	                       %d\n",sfd.rec_length);
  fprintf(fptr,"ASCII/EBCDIC flag                      %s\n",sfd.a_e_flag);
  fprintf(fptr,"Control document number                %s\n",
	sfd.control_doc_num);
  fprintf(fptr,"Control document revision              %s\n",
	sfd.control_doc_rev);
  fprintf(fptr,"File design descriptor revision        %s\n",sfd.file_des_rev);
  fprintf(fptr,"Software release Number                %s\n",sfd.rel_num);
  fprintf(fptr,"File number                            %d\n",sfd.file_num);
  fprintf(fptr,"File name                              %s\n",sfd.file_name);
  fprintf(fptr,"Record seq and location type           %s\n",sfd.record_seq);
  fprintf(fptr,"Sequence number location               %d\n",sfd.seq_num_loc);
  fprintf(fptr,"Sequence number field length           %d\n",sfd.seq_num_lgt);
  fprintf(fptr,"Record code and location type          %s\n",sfd.rec_loc_flg);
  fprintf(fptr,"Record code location                   %d\n",sfd.rec_loc);
  fprintf(fptr,"Record code field length               %d\n",sfd.rec_lth);
  fprintf(fptr,"Record length and location type        %s\n",sfd.rec_lth_flg);
  fprintf(fptr,"Record length location                 %d\n",sfd.rec_lth_loc);
  fprintf(fptr,"Record length field length             %d\n",sfd.rec_fld_lth);
  fprintf(fptr,"Data interpretation included in descriptor record -      %s\n",
      sfd.data_interp_des);
  fprintf(fptr,"Data interpretation included in other file -             %s \n",
      sfd.data_interp_oth);
  fprintf(fptr,"Data display information included in descriptor record - %s\n",      sfd.data_disp_des);
  fprintf(fptr,"Data display information included in other file -        %s\n",
      sfd.data_disp_oth);
  fprintf(fptr,"Number interval-related headers        %d\n",
	sfd.num_header_rec);
  fprintf(fptr,"Length interval-related header         %d\n",
	sfd.lth_header_rec);
  fprintf(fptr,"Number TM housekeeping records         %d\n",sfd.num_tm_house);
  fprintf(fptr,"Length TM housekeeping records         %d\n",sfd.lth_tm_house);
  fprintf(fptr,"Number ephemeris data records          %d\n",
	sfd.num_ephemeris);
  fprintf(fptr,"Length ephemeris data records          %d\n",
	sfd.lth_ephemeris);
  fprintf(fptr,"Number of scene definition recs        %d\n",
	sfd.num_scene_def);
  fprintf(fptr,"Length of scene definition recs        %d\n",
	sfd.lth_scene_def);
  fprintf(fptr,"Number scene quality data recs         %d\n",
	sfd.num_scene_qual);
  fprintf(fptr,"Length scene quality data recs         %d\n",
	sfd.lth_scene_qual);
  fprintf(fptr,"Number geometric modelling recs        %d\n",
	sfd.num_geometric_mod);
  fprintf(fptr,"Length geometric modelling recs        %d\n",
	sfd.lth_geometric_mod);
  fprintf(fptr,"Number sparse matrices records         %d\n",
	sfd.num_sparse_matrix);
  fprintf(fptr,"Length sparse matrices records         %d\n",
	sfd.lth_sparse_matrix);
  fprintf(fptr,"Number GCD mirror scan start time rec  %d\n",
        sfd.num_gcd_start_time);
  fprintf(fptr,"Length GCD mirror scan start time rec  %d\n",
        sfd.lth_gcd_start_time);
  fprintf(fptr,"Number high freq across-scan lines rec (raw jitter)  %d\n",
        sfd.num_across_scan);
  fprintf(fptr,"Length high freq across-scan lines rec (raw jitter)  %d\n",
        sfd.lth_across_scan);
  fprintf(fptr,"Number high freq cross-scan lines rec   %d\n",
	sfd.lth_cross_scan);
  fprintf(fptr,"Number mission telemetry ancillary rec  %d\n",
	sfd.num_mission_rec);
  fprintf(fptr,"Length mission telemetry ancillary rec  %d\n",
	sfd.lth_mission_rec);
  fprintf(fptr,"Number ground control point recs        %d\n",
	sfd.num_grnd_cont_rec);
  fprintf(fptr,"Length ground control point recs        %d\n",
	sfd.lth_grnd_cont_rec);
  fprintf(fptr,"Interval data start time locator        %s\n",
	sfd.start_time_loc);
  fprintf(fptr,"Interval data stop time locator         %s\n",
	sfd.stop_time_loc);
  fprintf(fptr,"Orbit field locator	                %s\n",
	sfd.orbit_fld_loc);
  fprintf(fptr,"\n\n");
         /* end of file descriptor record */

        /*  Print Interval Header Records */
  for ( loop_count=1; loop_count<= sfd.num_header_rec ; loop_count++)
    {
    if ((status = c_ltwg_sih(&sih, fin, &sfd.lth_header_rec)) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }

    fprintf(fptr,"SUPPLEMENTAL VOLUME - INTERVAL HEADER FILE %3d\n",
	loop_count);
    fprintf(fptr,"______________________________________________\n\n");

    fprintf(fptr,"Record Sequence Number                   %d\n",
	sih.rec_number);
    fprintf(fptr,"First record sub-type code               %d\n",
	sih.frst_rec_code);
    fprintf(fptr,"Record type code                         %d\n",sih.rec_code);
    fprintf(fptr,"Second record sub-type code              %d\n",
	sih.scnd_rec_code);
    fprintf(fptr,"Third record sub-type code               %d\n", 
	sih.thrd_rec_code);
    fprintf(fptr,"Length of this record	                   %d\n",
	sih.rec_length);
    fprintf(fptr,"Interval header record seq num           %d\n",
	sih.header_num);
    fprintf(fptr,"Input scene start time                   %s\n",
	sih.input_start_time);
    fprintf(fptr,"Input scene stop time	                   %s\n",
	sih.input_stop_time);
    fprintf(fptr,"PCD telemetry start time                 %s\n",
	sih.pcd_start_time);
    fprintf(fptr,"PCD telemetry stop time                  %s\n",
	sih.pcd_stop_time);
    fprintf(fptr,"Number of PCD major frames               %d\n",
	sih.pcd_frames);
    fprintf(fptr,"Orbit number	                           %d\n",
	sih.orbit_num);
    fprintf(fptr,"\n\n");
    }   /* End of interval header records */
    
       /* Print TM Housekeeping records and ephemeris records*/
    dspsupp1(fptr, fin, &sfd.num_tm_house, &sfd.lth_tm_house, 
	&sfd.num_ephemeris, &sfd.lth_ephemeris); 

       /* Check if there are any records that we do not have a definition
	      for, print message and skip over record if it exists  */ 
  if (sfd.num_scene_def > 0)
    {
    fprintf(fptr,"There are scene definition (scene header) records");
    fprintf(fptr," in this file,\nhowever, the structure of this record is"); 
    fprintf(fptr," unknown so it is not being printed.\n\n");
    fseek(fin,(sfd.num_scene_def * sfd.lth_scene_def),SEEK_CUR);
    }
  if (sfd.num_scene_qual > 0)
    {
    fprintf(fptr,"There are scene quality data records");
    fprintf(fptr," in this file,\nhowever, the structure of this record is"); 
    fprintf(fptr," unknown so it is not being printed.\n\n");
    fseek(fin,(sfd.num_scene_qual * sfd.lth_scene_qual),SEEK_CUR);
    }
  if (sfd.num_geometric_mod > 0)
    {
    fprintf(fptr,"There are geometric modelling data records");
    fprintf(fptr," in this file,\nhowever, the structure of this record is"); 
    fprintf(fptr," unknown so it is not being printed.\n\n");
    fseek(fin,(sfd.num_geometric_mod * sfd.lth_geometric_mod),SEEK_CUR);
    }
  if (sfd.num_sparse_matrix > 0)
    {
    fprintf(fptr,"There are sparsc matrices records");
    fprintf(fptr," in this file,\nhowever, the structure of this record is"); 
    fprintf(fptr," unknown so it is not being printed.\n\n");
    fseek(fin,(sfd.num_sparse_matrix * sfd.lth_sparse_matrix),SEEK_CUR);
    }
  if (sfd.num_gcd_start_time > 0)
    {
    fprintf(fptr,"There are GCD mirror scan start time records");
    fprintf(fptr," in this file,\nhowever, the structure of this record is"); 
    fprintf(fptr," unknown so it is not being printed.\n\n");
    fseek(fin,(sfd.num_gcd_start_time * sfd.lth_gcd_start_time),SEEK_CUR);
    }
   
     /* Print high freq across scan matrix records (raw jitter) */
  for(loop_count=1; loop_count<= sfd.num_across_scan; loop_count++)
    {
    if ((status = c_ltwg_rjr(&rjr, fin, &sfd.lth_across_scan)) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }
  
    fprintf(fptr,"SUPPLEMENTAL FILE - HIGH FREQUENCY ACROSS-SCAN LINE \n");
    fprintf(fptr,"MATRIX RECORDS (RAW JITTER MEASUREMENT %3d\n",loop_count);
    fprintf(fptr,"___________________________________________________\n\n");
  
    fprintf(fptr,"Record Sequence Number                   %d\n",
	rjr.rec_number);
    fprintf(fptr,"First record sub-type code               %d\n",
	rjr.frst_rec_code);
    fprintf(fptr,"Record type code                         %d\n",rjr.rec_code);
    fprintf(fptr,"Second record sub-type code              %d\n",
	rjr.scnd_rec_code);
    fprintf(fptr,"Third record sub-type code               %d\n",
	rjr.thrd_rec_code);
    fprintf(fptr,"Length of this record	                   %d\n",
	rjr.rec_length);
    fprintf(fptr,"Raw jitter measurements rec num          %d\n",
	rjr.rec_seq_num);
    fprintf(fptr,"Sequence num in PCD major frame          %d\n",rjr.seq_num);
    fprintf(fptr,"Day of year at start PCD STC             %d\n",
	rjr.day_start);
    fprintf(fptr,"Millisec of day start of frame           %d\n",
	rjr.millisec_start);
    fprintf(fptr,"Microsec of day start of frame           %d\n",
	rjr.microsec_start);
    fprintf(fptr,"Tenths of a microsec day  start          %d\n",
	  rjr.tenths_microsec_start);
    fprintf(fptr,"     ADS TEMPERATURE DATA  \n");
    fprintf(fptr,"ADS temp (1) time offset                 %d microsec\n",
	  rjr.ads_temp1_time_offset);
    fprintf(fptr,"ADS temperature 1 	                   %f degrees C\n",
	  rjr.ads_temp1);
    fprintf(fptr,"ADS temp (2) time offset	           %d microsec\n",
	  rjr.ads_temp2_time_offset);
    fprintf(fptr,"ADS temperature 2 	                   %f degrees C\n",
	  rjr.ads_temp2);
    fprintf(fptr,"ADS temp (3) time offset                 %d microsec\n",
	  rjr.ads_temp3_time_offset);
    fprintf(fptr,"ADS temperature 3 	                   %f degrees C\n",
	  rjr.ads_temp3);
    fprintf(fptr,"ADS temp (4) time offset                 %d microsec\n",
	  rjr.ads_temp4_time_offset);
    fprintf(fptr,"ADS temperature 4 	                   %f degrees C\n",
	  rjr.ads_temp4);
    fprintf(fptr,"     ADS MEASUREMENTS  \n");
    fprintf(fptr,"ADS measurment (1) axis 1                %d microsec \n",
	  rjr.ads_measur1[0]);
    fprintf(fptr,"ADS measurment (1) axis 2                %d microsec \n",
	  rjr.ads_measur1[1]);
    fprintf(fptr,"ADS measurment (1) axis 3                %d microsec \n",
	  rjr.ads_measur1[2]);
    fprintf(fptr,"Array of ADS measurments (microradians)\n");
    fprintf(fptr,"    roll          pitch           yaw\n");
    fprintf(fptr,"    ----          -----           ---  \n"); 
    for (i=0; i<1024; i++)
       fprintf(fptr,"   %6d        %6d          %6d\n",
	  rjr.ads_measur[0][i],rjr.ads_measur[1][i],rjr.ads_measur[2][i]);
    fprintf(fptr,"\n\n");
    }  /* End of high freq across scan matrix records (raw jitter) */
  
  if (sfd.num_cross_scan > 0)
    {
    fprintf(fptr,"There are high frequency cross scan matrix records");
    fprintf(fptr," in this file,\nhowever, the structure of this record is"); 
    fprintf(fptr," unknown so it is not being printed.\n\n");
    fseek(fin,(sfd.num_cross_scan * sfd.lth_cross_scan),SEEK_CUR);
    }

       /* Print Mission telemetry records */
  for (loop_count=1; loop_count<= sfd.num_mission_rec; loop_count++)
    {
    if ((status = c_ltwg_mtr(&mtr, fin, &sfd.lth_mission_rec)) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }
  
    fprintf(fptr,"SUPPLEMENTAL VOLUME - MISSION TELEMETRY RECORD - %3d \n",
	loop_count);
    fprintf(fptr,"____________________________________________________\n\n");
  
    fprintf(fptr,"Record Sequence Number                   %d\n",
	mtr.rec_number);
    fprintf(fptr,"First record sub-type code               %d\n",
	mtr.frst_rec_code);
    fprintf(fptr,"Record type code                         %d\n",mtr.rec_code);
    fprintf(fptr,"Second record sub-type code              %d\n",
	mtr.scnd_rec_code);
    fprintf(fptr,"Third record sub-type code               %d\n",
	mtr.thrd_rec_code);
    fprintf(fptr,"Length of this record	                   %d\n",
	mtr.rec_length);
    fprintf(fptr,"Mission telemetry data seq num           %d\n",
	mtr.rec_seq_num);
    fprintf(fptr,"TELEMETRY MAJOR FRAME START TIME\n");
    fprintf(fptr,"Day of year-start PCD frame STC          %d\n",
	mtr.day_start);
    fprintf(fptr,"Millisec of day-start of frame           %d\n",
	mtr.millisec_start);
    fprintf(fptr,"Microsec of day-start of frame           %d\n",
	mtr.microsec_start);
    fprintf(fptr,"Tenths of a microsec-day-start           %d\n",
	  mtr.tenths_microsec_start);
    fprintf(fptr,"EPHEMERIS DATA\n");
    fprintf(fptr,"Flight software time offset              %d\n",
	mtr.eph_time_offset);
    fprintf(fptr,"Spacecraft position x component          %f\n",
	mtr.craft_pos[0]);
    fprintf(fptr,"Spacecraft position y component          %f\n",
	mtr.craft_pos[1]);
    fprintf(fptr,"Spacecraft position z component          %f\n",
	mtr.craft_pos[2]);
    fprintf(fptr,"Spacecraft velocity x component          %f\n",
	mtr.craft_vel[0]);
    fprintf(fptr,"Spacecraft velocity y component          %f\n",
	mtr.craft_vel[1]);
    fprintf(fptr,"Spacecraft velocity z component          %f\n",
	mtr.craft_vel[2]);
    fprintf(fptr,"ATTIDTUDE DATA\n");
    fprintf(fptr,"Flight software time offset              %d\n",
	mtr.att_time_offset);
    fprintf(fptr,"EPA1		                           %f\n",mtr.epa1);
    fprintf(fptr,"EPA2		                           %f\n",mtr.epa2);
    fprintf(fptr,"EPA3		                           %f\n",mtr.epa3);
    fprintf(fptr,"EPA4		                           %f\n",mtr.epa4);
    fprintf(fptr,"ROLL, PITCH, YAW AXIS ANGULAR INCREMENTS\n");
    fprintf(fptr,"Flight software time offset              %d\n",
	mtr.inc_time_offset);
    fprintf(fptr,"Role axis angular increment              %f\n",
	mtr.roll_inc);
    fprintf(fptr,"Pitch axis angular increment             %f\n",
	mtr.pitch_inc);
    fprintf(fptr,"Yaw axis angular increment               %f\n",
	mtr.yaw_inc);
    fprintf(fptr,"ROLL, PITCH, YAW ANGULAR RATE\n");
    fprintf(fptr,"Flight software time offset              %d\n",
	mtr.rate_time_offset);
    fprintf(fptr,"Role axis angular rate                   %f\n",
	mtr.roll_rate);
    fprintf(fptr,"Pitch axis angular rate                  %f\n",
	mtr.pitch_rate);
    fprintf(fptr,"Yaw axis angular rate	                   %f\n",
	mtr.yaw_rate);
    fprintf(fptr,"ROLL, PITCH, YAW ATTITUDE ERROR\n");
    fprintf(fptr,"Flight software time offset              %d\n",
	mtr.err_time_offset);
    fprintf(fptr,"Role axis attitude error                 %f\n",
	mtr.roll_err);
    fprintf(fptr,"Pitch axis attitude error                %f\n",
	mtr.pitch_err);
    fprintf(fptr,"Yaw axis attitude error                  %f\n",mtr.yaw_err);
    fprintf(fptr,"HOUSEKEEPING DATA SAMPLES\n");
    fprintf(fptr,"  Coefficients required to express 12 binary-encoded data\n");
    fprintf(fptr,"      items in engineering units\n");
    fprintf(fptr,"\n       Item 1          Item 2          Item 3          ");
    fprintf(fptr,"Item 4\n"); 
    fprintf(fptr,"    ------------    ------------    ------------");
    fprintf(fptr,"    ------------\n");
    fprintf(fptr,"A0  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item1_coef[0], mtr.item2_coef[0], mtr.item3_coef[0], 
	  mtr.item4_coef[0]);
    fprintf(fptr,"A1  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item1_coef[1], mtr.item2_coef[1], mtr.item3_coef[1], 
	  mtr.item4_coef[1]);
    fprintf(fptr,"A2  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item1_coef[2], mtr.item2_coef[2], mtr.item3_coef[2], 
	  mtr.item4_coef[2]);
    fprintf(fptr,"A3  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item1_coef[3], mtr.item2_coef[3], mtr.item3_coef[3], 
	  mtr.item4_coef[3]);
    fprintf(fptr,"A4  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item1_coef[4], mtr.item2_coef[4], mtr.item3_coef[4], 
	  mtr.item4_coef[4]);
    fprintf(fptr,"A5  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item1_coef[5], mtr.item2_coef[5], mtr.item3_coef[5], 
	  mtr.item4_coef[5]);
    fprintf(fptr,"\n       Item 5          Item 6          Item 7          ");
    fprintf(fptr,"Item 8\n"); 
    fprintf(fptr,"    ------------    ------------    ------------");
    fprintf(fptr,"    ------------\n");
    fprintf(fptr,"A0  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item5_coef[0], mtr.item6_coef[0], mtr.item7_coef[0], 
	  mtr.item8_coef[0]);
    fprintf(fptr,"A1  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item5_coef[1], mtr.item6_coef[1], mtr.item7_coef[1], 
	  mtr.item8_coef[1]);
    fprintf(fptr,"A2  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item5_coef[2], mtr.item6_coef[2], mtr.item7_coef[2], 
	  mtr.item8_coef[2]);
    fprintf(fptr,"A3  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item5_coef[3], mtr.item6_coef[3], mtr.item7_coef[3], 
	  mtr.item8_coef[3]);
    fprintf(fptr,"A4  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item5_coef[4], mtr.item6_coef[4], mtr.item7_coef[4], 
	  mtr.item8_coef[4]);
    fprintf(fptr,"A5  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item5_coef[5], mtr.item6_coef[5], mtr.item7_coef[5], 
	  mtr.item8_coef[5]);
    fprintf(fptr,"\n       Item 9          Item 10         Item 11         ");
    fprintf(fptr,"Item 12\n"); 
    fprintf(fptr,"    ------------    ------------    ------------");
    fprintf(fptr,"    ------------\n");
    fprintf(fptr,"A0  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item9_coef[0], mtr.item10_coef[0], mtr.item11_coef[0], 
	  mtr.item12_coef[0]);
    fprintf(fptr,"A1  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item9_coef[1], mtr.item10_coef[1], mtr.item11_coef[1], 
	  mtr.item12_coef[1]);
    fprintf(fptr,"A2  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item9_coef[2], mtr.item10_coef[2], mtr.item11_coef[2], 
	  mtr.item12_coef[2]);
    fprintf(fptr,"A3  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item9_coef[3], mtr.item10_coef[3], mtr.item11_coef[3], 
	  mtr.item12_coef[3]);
    fprintf(fptr,"A4  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item9_coef[4], mtr.item10_coef[4], mtr.item11_coef[4], 
	  mtr.item12_coef[4]);
    fprintf(fptr,"A5  %12.6f    %12.6f    %12.6f    %12.6f\n",
	  mtr.item9_coef[5], mtr.item10_coef[5], mtr.item11_coef[5], 
	  mtr.item12_coef[5]);
    fprintf(fptr,"\nSample data\n"); 
    fprintf(fptr,"     Calibration currents      Blackbody    Silicon FPA\n");
    fprintf(fptr,"   Lamp 1   Lamp 2   Lamp 3   Temperature   Temperature\n");
    fprintf(fptr,"   ------   ------   ------   -----------   -----------\n");
    for (i=0; i<128; i++)
       fprintf(fptr,"      %d        %d        %d         %d            %d\n",
 	  mtr.samples[0][i],mtr.samples[1][i],mtr.samples[2][i],
	  mtr.samples[3][i],mtr.samples[4][i]);
    fprintf(fptr,"Calib Shutter  Backup shutter  Cold stage FPA  SLC  ");
    fprintf(fptr,"Baffle\n"); 
    fprintf(fptr," Temperature    Temperature     Termperature  Temp  ");
    fprintf(fptr,"Temp\n");
    fprintf(fptr,"-------------  --------------  -------------- ----  ");
    fprintf(fptr,"-----\n");
    for (i=0; i<128; i++)
       {
       fprintf(fptr,"      %d               %d               %d         %d",
          mtr.samples[5][i],mtr.samples[6][i],mtr.samples[7][i],
	  mtr.samples[8][i]);
       fprintf(fptr,"      %d\n", mtr.samples[9][i]);
       }
    fprintf(fptr,"\n\n");
    }  /* End of Mission telemetry records */

  if (sfd.num_grnd_cont_rec > 0)
    {
    fprintf(fptr,"There are ground control point records"); 
    fprintf(fptr," in this file,\nhowever, the structure of this record is"); 
    fprintf(fptr," unknown so it is not being printed.\n\n");
    fseek(fin,(sfd.num_grnd_cont_rec * sfd.lth_grnd_cont_rec),SEEK_CUR);
    }

    /* End of Supplemental records */
  
return(E_SUCC);
}		/* end of dspsupp */
