/***************************************************************************
NAME:			DSPSUPP1

PURPOSE:  Displays the second half of the LTWG supplemental record information

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		3/94    D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#ifdef sun
#include <unistd.h>
#endif
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "protos.h"

int dspsupp1
(
    FILE *fptr,			/* output file pointer			*/
    FILE *fin,			/* input file pointer			*/
    long *num_house,		/* number of TM housekeeping records    */
    long *lth_house,
    long *num_eph,
    long *lth_eph
)

{
struct LTWGTMHK tmhk;		/* LTWG supp file TM housekeeping 	
				        ancillary records		*/
struct LTWGEAA eaa;		/* LTWG ephomeris & attitude ancillary rec*/

long i;
long loop_count;


       /* Print TM Housekeeping records */
  for (loop_count=1; loop_count<= *num_house; loop_count++)
    {
    if (c_ltwg_tmhk(&tmhk, fin, lth_house) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }
  
    fprintf(fptr,"SUPPLEMENTAL VOLUME - TM HOUSEKEEPING RECORD %3d\n",
        loop_count);
    fprintf(fptr,"________________________________________________\n\n");
  
    fprintf(fptr,"Record Sequence Number                   %d\n",
	tmhk.rec_number);
    fprintf(fptr,"First record sub-type code               %d\n",
	tmhk.frst_rec_code);
    fprintf(fptr,"Record type code                         %d\n",
	tmhk.rec_code);
    fprintf(fptr,"Second record sub-type code              %d\n",
	tmhk.scnd_rec_code);
    fprintf(fptr,"Third record sub-type code               %d\n",
	tmhk.thrd_rec_code);
    fprintf(fptr,"Length of this record	                   %d\n",
	tmhk.rec_length);
    fprintf(fptr,"TM housekeeping data record num          %d\n",tmhk.rec_num);
    fprintf(fptr,"PCD major frame identifier               %d\n",
	tmhk.pcd_ident);
    fprintf(fptr,"       PCD MAJOR FRAME START TIME\n");
    fprintf(fptr,"Day of year-start of PCD frame           %d\n",
	tmhk.pcd_start_day);
    fprintf(fptr,"Milliseconds of day-start frame          %d\n",
	tmhk.pcd_start_msec);
    fprintf(fptr,"Microseconds at start of frame           %d\n",
	tmhk.pcd_start_usec);
    fprintf(fptr,"Tenths of a microsecond-start            %d\n",
	tmhk.pcd_start_tenths);
    fprintf(fptr,"       TM HOUSEKEEPING DATA    \n");
    fprintf(fptr,"   Blackbody temperature sample and conversion ");
    fprintf(fptr,"to Degrees C\n"); 
    fprintf(fptr,"Blackbody samp time offset               %d microsec\n",
	  tmhk.bb_time_offset);
    fprintf(fptr,"Blackbody TM housekeeping sample         %d\n",
	tmhk.bb_hk_sample);
    fprintf(fptr,"Blackbody A0 coefficient                 %e\n",tmhk.bb_A0);
    fprintf(fptr,"Blackbody A1 coefficient                 %e\n",tmhk.bb_A1);
    fprintf(fptr,"Blackbody A2 coefficient                 %e\n",tmhk.bb_A2);
    fprintf(fptr,"Blackbody A3 coefficient                 %e\n",tmhk.bb_A3);
    fprintf(fptr,"Blackbody A4 coefficient                 %e\n",tmhk.bb_A4);
    fprintf(fptr,"Blackbody A5 coefficient                 %e\n",tmhk.bb_A5);
    fprintf(fptr,"     Focal plane temperature sample and conversion ");
    fprintf(fptr,"to Degrees C\n");
    fprintf(fptr,"Focal plane samp time offset             %d microsec\n",
	  tmhk.ff_time_offset);
    fprintf(fptr,"Focal plane TM housekeeping sample       %d\n",
	tmhk.ff_hk_sample);
    fprintf(fptr,"Focal plane A0 coefficient               %e\n",tmhk.ff_A0);
    fprintf(fptr,"Focal plane A1 coefficient               %e\n",tmhk.ff_A1);
    fprintf(fptr,"Focal plane A2 coefficient               %e\n",tmhk.ff_A2);
    fprintf(fptr,"Focal plane A3 coefficient               %e\n",tmhk.ff_A3);
    fprintf(fptr,"Focal plane A4 coefficient               %e\n",tmhk.ff_A4);
    fprintf(fptr,"Focal plane A5 coefficient               %e\n",tmhk.ff_A5);
    fprintf(fptr,"     Calib shutter temperature sample and conversion ");
    fprintf(fptr,"to Degrees C \n");
    fprintf(fptr,"Calib shutter samp time offset           %d microsec\n",
	  tmhk.st_time_offset);
    fprintf(fptr,"Calib shutter TM housekeep sample        %d\n",
	tmhk.st_hk_sample);
    fprintf(fptr,"Calib shutter A0 coefficient             %e\n",tmhk.st_A0);
    fprintf(fptr,"Calib shutter A1 coefficient             %e\n",tmhk.st_A1);
    fprintf(fptr,"Calib shutter A2 coefficient             %e\n",tmhk.st_A2);
    fprintf(fptr,"Calib shutter A3 coefficient             %e\n",tmhk.st_A3);
    fprintf(fptr,"Calib shutter A4 coefficient             %e\n",tmhk.st_A4);
    fprintf(fptr,"Calib shutter A5 coefficient             %e\n",tmhk.st_A5);
    fprintf(fptr,"     Baffle temperature sample and conversion to ");
    fprintf(fptr,"Degrees C \n");
    fprintf(fptr,"Baffle samp time offset                  %d microsec\n",
	  tmhk.bt_time_offset);
    fprintf(fptr,"Baffle TM housekeeping sample            %d\n",
	tmhk.bt_hk_sample);
    fprintf(fptr,"Baffle A0 coefficient	                   %e\n",tmhk.bt_A0);
    fprintf(fptr,"Baffle A1 coefficient	                   %e\n",tmhk.bt_A1);
    fprintf(fptr,"Baffle A2 coefficient	                   %e\n",tmhk.bt_A2);
    fprintf(fptr,"Baffle A3 coefficient	                   %e\n",tmhk.bt_A3);
    fprintf(fptr,"Baffle A4 coefficient	                   %e\n",tmhk.bt_A4);
    fprintf(fptr,"Baffle A5 coefficient	                   %e\n",tmhk.bt_A5);
    fprintf(fptr,"     Cold stage temperature sample and conversion ");
    fprintf(fptr,"to Degrees C \n");
    fprintf(fptr,"Cold Stage samp time offset              %d microsec\n",
	  tmhk.cs_time_offset);
    fprintf(fptr,"Cold Stage TM housekeeping sample        %d\n",
	tmhk.cs_hk_sample);
    fprintf(fptr,"Cold Stage A0 coefficient                %e\n",tmhk.cs_A0);
    fprintf(fptr,"Cold Stage A1 coefficient                %e\n",tmhk.cs_A1);
    fprintf(fptr,"Cold Stage A2 coefficient                %e\n",tmhk.cs_A2);
    fprintf(fptr,"Cold Stage A3 coefficient                %e\n",tmhk.cs_A3);
    fprintf(fptr,"Cold Stage A4 coefficient                %e\n",tmhk.cs_A4);
    fprintf(fptr,"Cold Stage A5 coefficient                %e\n",tmhk.cs_A5);
    fprintf(fptr,"     Scan line corrector temperature sample and conversion");
    fprintf(fptr," to Degrees C \n");
    fprintf(fptr,"Line corrector samp time offset          %d microsec\n",
	  tmhk.lc_time_offset);
    fprintf(fptr,"Line corrector TM housekeep sample       %d\n",
	tmhk.lc_hk_sample);
    fprintf(fptr,"Line corrector A0 coefficient            %e\n",tmhk.lc_A0);
    fprintf(fptr,"Line corrector A1 coefficient            %e\n",tmhk.lc_A1);
    fprintf(fptr,"Line corrector A2 coefficient            %e\n",tmhk.lc_A2);
    fprintf(fptr,"Line corrector A3 coefficient            %e\n",tmhk.lc_A3);
    fprintf(fptr,"Line corrector A4 coefficient            %e\n",tmhk.lc_A4);
    fprintf(fptr,"Line corrector A5 coefficient            %e\n",tmhk.lc_A5);
    fprintf(fptr,"     Shutter hub temperature sample and conversion ");
    fprintf(fptr,"to Degrees C \n");
    fprintf(fptr,"Shutter hub samp time offset             %d microsec\n",
	  tmhk.sh_time_offset);
    fprintf(fptr,"Shutter hub TM housekeeping sample       %d\n",
	tmhk.sh_hk_sample);
    fprintf(fptr,"Shutter hub A0 coefficient               %e\n",tmhk.sh_A0);
    fprintf(fptr,"Shutter hub A1 coefficient               %e\n",tmhk.sh_A1);
    fprintf(fptr,"Shutter hub A2 coefficient               %e\n",tmhk.sh_A2);
    fprintf(fptr,"Shutter hub A3 coefficient               %e\n",tmhk.sh_A3);
    fprintf(fptr,"Shutter hub A4 coefficient               %e\n",tmhk.sh_A4);
    fprintf(fptr,"Shutter hub A5 coefficient               %e\n",tmhk.sh_A5);
    fprintf(fptr,"     Relay optics temperature sample and conversion ");
    fprintf(fptr,"to Degrees C \n");
    fprintf(fptr,"Relay optics samp time offset            %d microsec\n",
	  tmhk.ro_time_offset);
    fprintf(fptr,"Relay optics TM housekeep sample         %d\n",
	tmhk.ro_hk_sample);
    fprintf(fptr,"Relay optics A0 coefficient              %e\n",tmhk.ro_A0);
    fprintf(fptr,"Relay optics A1 coefficient              %e\n",tmhk.ro_A1);
    fprintf(fptr,"Relay optics A2 coefficient              %e\n",tmhk.ro_A2);
    fprintf(fptr,"Relay optics A3 coefficient              %e\n",tmhk.ro_A3);
    fprintf(fptr,"Relay optics A4 coefficient              %e\n",tmhk.ro_A4);
    fprintf(fptr,"Relay optics A5 coefficient              %e\n",tmhk.ro_A5);
    fprintf(fptr,"     Unpacked as serial word B  \n");
    fprintf(fptr,"Serial word B samp time offset           %d microsec\n",
	  tmhk.wb_time_offset);
    fprintf(fptr,"Serial word B TM housekeep sample        %d\n",
	tmhk.wb_hk_sample);
    fprintf(fptr,"Serial word B A0 coefficient             %e\n",tmhk.wb_A0);
    fprintf(fptr,"Serial word B A1 coefficient             %e\n",tmhk.wb_A1);
    fprintf(fptr,"Serial word B A2 coefficient             %e\n",tmhk.wb_A2);
    fprintf(fptr,"Serial word B A3 coefficient             %e\n",tmhk.wb_A3);
    fprintf(fptr,"Serial word B A4 coefficient             %e\n",tmhk.wb_A4);
    fprintf(fptr,"Serial word B A5 coefficient             %e\n",tmhk.wb_A5);
    fprintf(fptr,"     Unpacked as serial word D  \n");
    fprintf(fptr,"Serial word D samp time offset           %d microsec\n",
	  tmhk.wd_time_offset);
    fprintf(fptr,"Serial word D TM housekeep sample        %d\n",
	tmhk.wd_hk_sample);
    fprintf(fptr,"Serial word D A0 coefficient             %e\n",tmhk.wd_A0);
    fprintf(fptr,"Serial word D A1 coefficient             %e\n",tmhk.wd_A1);
    fprintf(fptr,"Serial word D A2 coefficient             %e\n",tmhk.wd_A2);
    fprintf(fptr,"Serial word D A3 coefficient             %e\n",tmhk.wd_A3);
    fprintf(fptr,"Serial word D A4 coefficient             %e\n",tmhk.wd_A4);
    fprintf(fptr,"Serial word D A5 coefficient             %e\n",tmhk.wd_A5);
    fprintf(fptr,"     Unpacked as serial word E  \n");
    fprintf(fptr,"Serial word E samp time offset           %d microsec\n",
	  tmhk.we_time_offset);
    fprintf(fptr,"Serial word E TM housekeep sample        %d\n",
	tmhk.we_hk_sample);
    fprintf(fptr,"Serial word E A0 coefficient             %e\n",tmhk.we_A0);
    fprintf(fptr,"Serial word E A1 coefficient             %e\n",tmhk.we_A1);
    fprintf(fptr,"Serial word E A2 coefficient             %e\n",tmhk.we_A2);
    fprintf(fptr,"Serial word E A3 coefficient             %e\n",tmhk.we_A3);
    fprintf(fptr,"Serial word E A4 coefficient             %e\n",tmhk.we_A4);
    fprintf(fptr,"Serial word E A5 coefficient             %e\n",tmhk.we_A5);
    fprintf(fptr,"     Unpacked as serial word F  \n");
    fprintf(fptr,"Serial word F samp time offset           %d microsec\n",
	  tmhk.wf_time_offset);
    fprintf(fptr,"Serial word F TM housekeep sample        %d\n",
	tmhk.wf_hk_sample);
    fprintf(fptr,"Serial word F A0 coefficient             %e\n",tmhk.wf_A0);
    fprintf(fptr,"Serial word F A1 coefficient             %e\n",tmhk.wf_A1);
    fprintf(fptr,"Serial word F A2 coefficient             %e\n",tmhk.wf_A2);
    fprintf(fptr,"Serial word F A3 coefficient             %e\n",tmhk.wf_A3);
    fprintf(fptr,"Serial word F A4 coefficient             %e\n",tmhk.wf_A4);
    fprintf(fptr,"Serial word F A5 coefficient             %e\n",tmhk.wf_A5);
    fprintf(fptr,"     Unpacked as serial word G  \n");
    fprintf(fptr,"Serial word G samp time offset           %d microsec\n",
	  tmhk.wg_time_offset);
    fprintf(fptr,"Serial word G TM housekeep sample        %d\n",
	tmhk.wg_hk_sample);
    fprintf(fptr,"Serial word G A0 coefficient             %e\n",tmhk.wg_A0);
    fprintf(fptr,"Serial word G A1 coefficient             %e\n",tmhk.wg_A1);
    fprintf(fptr,"Serial word G A2 coefficient             %e\n",tmhk.wg_A2);
    fprintf(fptr,"Serial word G A3 coefficient             %e\n",tmhk.wg_A3);
    fprintf(fptr,"Serial word G A4 coefficient             %e\n",tmhk.wg_A4);
    fprintf(fptr,"Serial word G A5 coefficient             %e\n",tmhk.wg_A5);
    fprintf(fptr,"     Unpacked as serial word L  \n");
    fprintf(fptr,"Serial word L samp time offset           %d microsec\n",
	  tmhk.wl_time_offset);
    fprintf(fptr,"Serial word L TM housekeep sample        %d\n",
	tmhk.wl_hk_sample);
    fprintf(fptr,"Serial word L A0 coefficient             %e\n",tmhk.wl_A0);
    fprintf(fptr,"Serial word L A1 coefficient             %e\n",tmhk.wl_A1);
    fprintf(fptr,"Serial word L A2 coefficient             %e\n",tmhk.wl_A2);
    fprintf(fptr,"Serial word L A3 coefficient             %e\n",tmhk.wl_A3);
    fprintf(fptr,"Serial word L A4 coefficient             %e\n",tmhk.wl_A4);
    fprintf(fptr,"Serial word L A5 coefficient             %e\n",tmhk.wl_A5);
    fprintf(fptr,"     Primary mirror temperature sample and conversion ");
    fprintf(fptr,"to Degrees C \n");
    fprintf(fptr,"Primary mirror samp time offset          %d microsec\n",
	  tmhk.pm_time_offset);
    fprintf(fptr,"Primary mirror TM housekeep sample       %d\n",
	tmhk.pm_hk_sample);
    fprintf(fptr,"Primary mirror A0 coefficient            %e\n",tmhk.pm_A0);
    fprintf(fptr,"Primary mirror A1 coefficient            %e\n",tmhk.pm_A1);
    fprintf(fptr,"Primary mirror A2 coefficient            %e\n",tmhk.pm_A2);
    fprintf(fptr,"Primary mirror A3 coefficient            %e\n",tmhk.pm_A3);
    fprintf(fptr,"Primary mirror A4 coefficient            %e\n",tmhk.pm_A4);
    fprintf(fptr,"Primary mirror A5 coefficient            %e\n",tmhk.pm_A5);
    fprintf(fptr,"     Secondary mirror temperature sample and conversion ");
    fprintf(fptr,"to Degrees C \n");
    fprintf(fptr,"Second mirror samp time offset           %d microsec\n",
	  tmhk.sm_time_offset);
    fprintf(fptr,"Second mirror TM housekeep sample        %d\n",
	tmhk.sm_hk_sample);
    fprintf(fptr,"Secondary mirror A0 coefficient          %e\n",tmhk.sm_A0);
    fprintf(fptr,"Secondary mirror A1 coefficient          %e\n",tmhk.sm_A1);
    fprintf(fptr,"Secondary mirror A2 coefficient          %e\n",tmhk.sm_A2);
    fprintf(fptr,"Secondary mirror A3 coefficient          %e\n",tmhk.sm_A3);
    fprintf(fptr,"Secondary mirror A4 coefficient          %e\n",tmhk.sm_A4);
    fprintf(fptr,"Secondary mirror A5 coefficient          %e\n",tmhk.sm_A5);
    fprintf(fptr,"Serial word B	                           %s\n",
	tmhk.ser_wd_b);
    fprintf(fptr,"Serial word D	                           %s\n",
	tmhk.ser_wd_d);
    fprintf(fptr,"Serial word E	                           %s\n",
	tmhk.ser_wd_e);
    fprintf(fptr,"Serial word F	                           %s\n",
	tmhk.ser_wd_f);
    fprintf(fptr,"Serial word G	                           %s\n",
	tmhk.ser_wd_g);
    fprintf(fptr,"Serial word L	                           %s\n",
	tmhk.ser_wd_l);
    fprintf(fptr,"\n\n");
    }     /* End of TM Housekeeping records */

        /* Print Ephemeris and attitude records */
  for (loop_count=1; loop_count<= *num_eph; loop_count++)
    {
    if (c_ltwg_eaa(&eaa, fin, (int *) lth_eph) == E_FAIL)
       {
       fclose(fptr);
       return(E_FAIL);
       }
  
    fprintf(fptr,"SUPPLEMENTAL VOLUME-EPHEMERIS AND ATTITUDE RECORD %3d\n",
	loop_count);
    fprintf(fptr,"_____________________________________________________\n\n");
  
    fprintf(fptr,"Record Sequence Number                   %d\n",
	eaa.rec_number);
    fprintf(fptr,"First record sub-type code               %d\n",
	eaa.frst_rec_code);
    fprintf(fptr,"Record type code                         %d\n",eaa.rec_code);
    fprintf(fptr,"Second record sub-type code              %d\n",
	eaa.scnd_rec_code);
    fprintf(fptr,"Third record sub-type code               %d\n",
	eaa.thrd_rec_code);
    fprintf(fptr,"Length of this record	                   %d\n",
	eaa.rec_length);
    fprintf(fptr,"Ephemeris & attitude record num          %d\n",
	eaa.rec_seq_num);
    fprintf(fptr,"Ephemeris updated (0-NO, 1-YES)          %d\n",
	eaa.ephemeris_flag);
    fprintf(fptr,"Attitude updated (0-NO, 1-YES)           %d\n",
	eaa.attitude_flag);
    fprintf(fptr,"Gyro updated (0-NO, 1-YES)               %d\n",
	eaa.gyro_flag);
    fprintf(fptr,"Gyro drift updated (0-NO,1-YES)          %d\n",
	eaa.gyro_drift_flag);
    fprintf(fptr,"      PCD MAJOR FRAME START TIME\n");
    fprintf(fptr,"Day of year-start of PCD frame           %d\n",
	eaa.pcd_start_day);
    fprintf(fptr,"Milliseconds day-start frame             %d\n",
	eaa.pcd_start_msec);
    fprintf(fptr,"Microseconds-start of PCD frame          %d\n",
	eaa.pcd_start_usec);
    fprintf(fptr,"Tenths of microsecond-start              %d\n",
	eaa.pcd_start_tenths);
    fprintf(fptr,"      EPHEMERIS DATA\n");
    fprintf(fptr,"Ephemeris measure time offset            %d microsec\n",
	  eaa.eph_time_offset);
    fprintf(fptr,"Spacecraft x position component          %22.14e\n",
	eaa.craft_pos[0]);
    fprintf(fptr,"Spacecraft y position component          %22.14e\n",
	eaa.craft_pos[1]);
    fprintf(fptr,"Spacecraft z position component          %22.14e\n",
	eaa.craft_pos[2]);
    fprintf(fptr,"Spacecraft x velosity component          %22.14e\n",
	eaa.craft_vel[0]);
    fprintf(fptr,"Spacecraft y velosity component          %22.14e\n",
	eaa.craft_vel[1]);
    fprintf(fptr,"Spacecraft z velosity component          %22.14e\n",
	eaa.craft_vel[2]);
    fprintf(fptr,"      ATTITUDE DATA\n");
    fprintf(fptr,"Attitude measure time offset             %d microsec\n",
	  eaa.att_time_offset);
    fprintf(fptr,"EPA1		                           %22.14e\n",
	eaa.epa1);
    fprintf(fptr,"EPA2		                           %22.14e\n",
	eaa.epa2);
    fprintf(fptr,"EPA3		                           %22.14e\n",
	eaa.epa3);
    fprintf(fptr,"EPA4		                           %22.14e\n",
	eaa.epa4);
    fprintf(fptr,"      GYRO DATA\n");
    fprintf(fptr,"Gyro measure time offset                 %d microsec\n",
	  eaa.gyro_time_offset);
    fprintf(fptr,"Gyro measurments components (arc_sec of angle)\n");
    fprintf(fptr,"     x component                y component           ");
    fprintf(fptr,"     z component\n");
    fprintf(fptr,"----------------------     ----------------------     "); 
    fprintf(fptr,"----------------------\n");
    for (i=0; i<64; i++)
       fprintf(fptr,"%22.14e     %22.14e     %22.14e\n",eaa.gyro_meas[0][i],
          eaa.gyro_meas[1][i],eaa.gyro_meas[2][i]);
    fprintf(fptr,"      GYRO DRIFT DATA\n");
    fprintf(fptr,"Gyro drift measure time offset           %d microsec\n",
	  eaa.gyro_drift_time_offset);
    fprintf(fptr,"Gyro drift x components                  %22.14e ",
	  eaa.gyro_drift[0]);
    fprintf(fptr,"rads/512 msec\n");
    fprintf(fptr,"Gyro drift y components                  %22.14e ",
	  eaa.gyro_drift[1]);
    fprintf(fptr,"rads/512 msec\n");
    fprintf(fptr,"Gyro drift z components                  %22.14e ",
	  eaa.gyro_drift[2]);
    fprintf(fptr,"rads/512 msec\n");
    fprintf(fptr,"\n\n");
    }   /* End of ephmeris and attitude records */

  
return(E_SUCC);
}		/* end of dspsupp1 */
