/*******************************************************************************
NAME:                           DSPTIPS

PURPOSE:      Display the contents of the TIPSP fields to the
              specified device.

PROGRAM HISTORY:
PROGRAMMER              DATE            REASON
----------              ----            ------
T. Mittan              12/92           New development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tipsp.h"
#include "header.h"
#include "labserv.h"
#include "protos.h"

int dsptips
(
    char *hname,            /* TAE name of input image                    	*/
    char *prtfile,          /* print file name                            	*/
    long *hdsk,		/* header file flag			 	*/
    long *tdsk,		/* trailer file flag			 	*/
    long *andsk		/* annotation file flag			 	*/
)

{
struct TIPSP	tipsp;	/* CCT-PT structure to hold information */
FILE *fp;               /* file pointer to print file 		*/
long nonfatal = 1;	/* nonfatal error flag			*/
long i;			/* array index				*/
long j;			/* array index				*/

char hostheader[CMLEN];	/* associated file names		*/


/* open the print file
----------------------*/
fp = fopen(prtfile,"w");
if (fp == NULL)
   {
   c_errmsg("Error opening print file","dspassoc-open",&nonfatal);
   return(E_FAIL);
   }

   /* header related information
   ---------------------------*/
if (*hdsk)
 {
  /* make appropriate las name
  --------------------------*/
 c_lsmknm(hname,"tipsp_head",hostheader);/* make associated header file name */

 fprintf(fp,"Header information  \n");
 fprintf(fp,"__________________  \n\n");
 fprintf(fp,"header filename        %s \n\n", hostheader);

 if (c_read_head(&tipsp,hname) == E_FAIL)
   {
   fclose(fp);
   return(E_FAIL);
   }

 fprintf(fp,"center_latitude        %f \n",tipsp.head_center_latitude);
 fprintf(fp,"center_longitude       %f \n",tipsp.head_center_longitude);
 fprintf(fp,"center_line            %f \n",tipsp.head_center_line);
 fprintf(fp,"center_sample          %f \n",tipsp.head_center_sample);
 fprintf(fp,"pquadrant_lat          %f \n",tipsp.head_pquadrant_lat);	
 fprintf(fp,"pquadrant_long         %f \n",tipsp.head_pquadrant_long);
 fprintf(fp,"quadrant_line          %f \n",tipsp.head_quadrant_line);
 fprintf(fp,"quadrant_sample        %f \n",tipsp.head_quadrant_sample);	

 fprintf(fp,"average_samples        %d \n",tipsp.head_average_samples);
 fprintf(fp,"bands                  %d \n",tipsp.head_bands);	
 for (i = 0; i < 7; i++)
   fprintf(fp,"bands_present[%1d]       %d \n",i+1,tipsp.head_bands_present[i]);
 fprintf(fp,"center_time_offset     %d \n",tipsp.head_center_time_offset);
 for (i = 0; i< 100; i++)
   fprintf(fp,"detector_map[%2d]       %d \n",i+1,tipsp.head_detector_map[i]);
 fprintf(fp,"hquadrant_overlap      %d \n",tipsp.head_hquadrant_overlap);	
 fprintf(fp,"lines                  %d \n",tipsp.head_lines);		
 fprintf(fp,"orbit_number           %d \n",tipsp.head_orbit_number);
 fprintf(fp,"vquadrant_overlap      %d \n",tipsp.head_vquadrant_overlap);
 fprintf(fp,"wrs_cycle              %d \n",tipsp.head_wrs_cycle);		
 fprintf(fp,"center_time            %s \n",tipsp.head_center_time);		
 fprintf(fp,"dectector_smooth_array %s \n", 
				tipsp.head_dectector_smooth_array); 
 fprintf(fp,"geometric_correction   %s \n",tipsp.head_geometric_correction);	
 fprintf(fp,"interleave             %s \n",tipsp.head_interleave);	
 fprintf(fp,"map_ancillary_records  %s \n",tipsp.head_map_ancillary_records);
 fprintf(fp,"map_projection         %s \n",tipsp.head_map_projection);
 fprintf(fp,"mission_id             %s \n",tipsp.head_mission_id);	
 fprintf(fp,"orbit_direction        %s \n",tipsp.head_orbit_direction);	
 fprintf(fp,"parent_id              %s \n",tipsp.head_parent_id);	
 fprintf(fp,"product_class          %s \n",tipsp.head_product_class);
 fprintf(fp,"product_id             %s \n",tipsp.head_product_id);   	
 fprintf(fp,"radiometric_ancillary_records %s \n",
				tipsp.head_radiometric_ancillary_records);
 fprintf(fp,"radiometric_correction %s \n",tipsp.head_radiometric_correction);
 fprintf(fp,"radiometric_resolution %s \n",tipsp.head_radiometric_resolution);
 fprintf(fp,"resampling_designator  %s \n",tipsp.head_resampling_designator);	
 fprintf(fp,"scene_id               %s \n",tipsp.head_scene_id);		
 fprintf(fp,"sensor_id              %s \n",tipsp.head_sensor_id);	
 fprintf(fp,"wrs_designator         %s \n",tipsp.head_wrs_designator);

/* Map projection related information
-----------------------------------*/
   /* Input scene related information
   --------------------------------*/
 fprintf(fp,"\nMap projection information \n");		
 fprintf(fp,"__________________________ \n\n");		

 fprintf(fp,"nominal_pixels         %d \n",tipsp.map_nominal_pixels);		
 fprintf(fp,"nominal_lines          %d \n",tipsp.map_nominal_lines);	
 fprintf(fp,"nominal_interpixel     %f \n",tipsp.map_nominal_interpixel);
 fprintf(fp,"nominal_interline      %f \n",tipsp.map_nominal_interline);	
 fprintf(fp,"skew                   %f \n",tipsp.map_skew);			

   /* UTM/PS related information
   ---------------------------*/
 fprintf(fp,"utm_datum              %s \n",tipsp.map_utm_datum);		
 fprintf(fp,"utm_zone               %d \n",tipsp.map_utm_zone);		
 fprintf(fp,"utm_wrs_northing       %f \n",tipsp.map_utm_wrs_northing);		
 fprintf(fp,"utm_wrs_easting        %f \n",tipsp.map_utm_wrs_easting);	
 fprintf(fp,"utm_center_northing    %f \n",tipsp.map_utm_center_northing);	
 fprintf(fp,"utm_center_easting     %f \n",tipsp.map_utm_center_easting);
 fprintf(fp,"utm_vertical_offset    %f \n",tipsp.map_utm_vertical_offset);
 fprintf(fp,"utm_horizontal_offset  %f \n",tipsp.map_utm_horizontal_offset);
 fprintf(fp,"utm_rotation           %f \n",tipsp.map_utm_rotation);	


   /* SOM related information
   ------------------------*/
 fprintf(fp,"som_wrs_horizontal     %f \n",tipsp.map_som_wrs_horizontal);
 fprintf(fp,"som_wrs_vertical       %f \n",tipsp.map_som_wrs_vertical);
 fprintf(fp,"som_center_horizontal  %f \n",tipsp.map_som_center_horizontal);	
 fprintf(fp,"som_center_vertical    %f \n",tipsp.map_som_center_vertical);
 fprintf(fp,"som_vertical_offset    %f \n",tipsp.map_som_vertical_offset);	
 fprintf(fp,"som_horizontal_offset  %f \n",tipsp.map_som_horizontal_offset);
 fprintf(fp,"som_rotation           %f \n",tipsp.map_som_rotation);

   /* Processed Quadrant related information
   ---------------------------------------*/
 fprintf(fp,"pixels                 %f \n",tipsp.map_pixels);			
 fprintf(fp,"lines                  %f \n",tipsp.map_lines);		
 fprintf(fp,"corrected_interpixel   %f \n",tipsp.map_corrected_interpixel);	
 fprintf(fp,"corrected_interline    %f \n",tipsp.map_corrected_interline);
 fprintf(fp,"processed_zone         %f \n",tipsp.map_processed_zone);		
 fprintf(fp,"center_line_number     %d \n",tipsp.map_center_line_number);
 fprintf(fp,"center_sample_number   %d \n",tipsp.map_center_sample_number);	
 fprintf(fp,"rotation               %f \n",tipsp.map_rotation);		
 fprintf(fp,"orbit_inclination      %f \n",tipsp.map_orbit_inclination);	
 fprintf(fp,"ascending_longitude    %f \n",tipsp.map_ascending_longitude);
 fprintf(fp,"altitude               %f \n",tipsp.map_altitude);			
 fprintf(fp,"ground_speed           %f \n",tipsp.map_ground_speed);	
 fprintf(fp,"satellite_heading      %f \n",tipsp.map_satellite_heading);
 fprintf(fp,"cross_track_field_of_view %f \n",
				tipsp.map_cross_track_field_of_view);	
 fprintf(fp,"scan_rate              %f \n",tipsp.map_scan_rate);		
 fprintf(fp,"sampling_rate          %f \n",tipsp.map_sampling_rate);
 fprintf(fp,"sun_elevation          %f \n",tipsp.map_sun_elevation);		
 fprintf(fp,"sun_azimuth            %f \n",tipsp.map_sun_azimuth);	

/* Radiometric calibration related information
--------------------------------------------*/
 /* Read data from the Radiometric calibration record which is in the header
 -------------------------------------------------------------------------*/
 fprintf(fp,"\nRadiometric calibration information \n");		
 fprintf(fp,"___________________________________ \n\n");		

 fprintf(fp,"band                  %s \n",tipsp.radio_band);		
 fprintf(fp,"lower_reflectance     %s \n",tipsp.radio_lower_reflectance);
 fprintf(fp,"upper_reflectance     %s \n",tipsp.radio_upper_reflectance);	
 fprintf(fp,"offset                 %f \n",tipsp.radio_offset);		
 fprintf(fp,"gain                   %f \n",tipsp.radio_gain);	
 }

   /* annotation related information
   -------------------------------*/
if (*andsk)
 {
 /* make appropriate las name
 --------------------------*/
 c_lsmknm(hname,"tipsp_annot",hostheader);/* make associated header file name */

 if (c_read_annot(&tipsp,hname) == E_FAIL)
   {
   fclose(fp);
   return(E_FAIL);
   }

 fprintf(fp,"\nAnnotation information \n");		
 fprintf(fp,"______________________ \n\n");		
 fprintf(fp,"Annotation filename    %s \n\n", hostheader);


 fprintf(fp,"acquistion_date        %s \n",tipsp.annot_acquistion_date);	
 fprintf(fp,"agency                 %s \n",tipsp.annot_agency);		
 fprintf(fp,"band_id_code           %s \n",tipsp.annot_band_id_code);
 fprintf(fp,"center_latitude_longitude %s \n",
				tipsp.annot_center_latitude_longitude);
 fprintf(fp,"ephemeris              %s \n",tipsp.annot_ephemeris);		
 fprintf(fp,"geometric_correction_header %s \n",
				tipsp.annot_geometric_correction_header); 
 fprintf(fp,"processing_procedure   %s \n",tipsp.annot_processing_procedure);	
 fprintf(fp,"projection             %s \n",tipsp.annot_projection);	
 fprintf(fp,"resampling_algorithm   %s \n",tipsp.annot_resampling_algorithm);	
 fprintf(fp,"scene_identification   %s \n",tipsp.annot_scene_identification);
 fprintf(fp,"sun_angle              %s \n",tipsp.annot_sun_angle);	
 fprintf(fp,"wrs_latitude_longitude %s \n",tipsp.annot_wrs_latitude_longitude);	
 fprintf(fp,"wrs_path               %s \n",tipsp.annot_wrs_path);		
 
   /* tick mark related information
   ------------------------------*/
 for ( i = 0; i < 8; i++)
    fprintf(fp,"top_tick[%1d]            %d \n",i+1,tipsp.annot_top_tick[i]);	
 for ( i = 0; i < 8; i++)
    fprintf(fp,"left_tick[%1d]           %d \n",i+1,tipsp.annot_left_tick[i]);
 for ( i = 0; i < 8; i++)
    fprintf(fp,"right_tick[%1d]          %d \n",i+1,tipsp.annot_right_tick[i]);	
 for ( i = 0; i < 8; i++)
    fprintf(fp,"bottom_tick[%1d]         %d \n",i+1,tipsp.annot_bottom_tick[i]);

 for ( i = 0; i < 8; i++)
    fprintf(fp,"bottom_tick_ascii[%1d]   %s \n",i+1,
				tipsp.annot_bottom_tick_ascii[i]);	
 for ( i = 0; i < 8; i++)
    fprintf(fp,"left_tick_ascii[%1d]     %s \n",i+1,
				tipsp.annot_left_tick_ascii[i]);
 for ( i = 0; i < 8; i++)
    fprintf(fp,"right_tick_ascii[%1d]    %s \n",i+1,
				tipsp.annot_right_tick_ascii[i]);
 for ( i = 0; i < 8; i++)
    fprintf(fp,"top_tick_ascii[%1d]      %s \n",i+1,
				tipsp.annot_top_tick_ascii[i]);


/* Band quality related information
---------------------------------*/
 fprintf(fp,"\nBand Quality information \n");		
 fprintf(fp,"________________________ \n\n");		

 fprintf(fp,"quality                %s \n",tipsp.band_quality);		

 /* radiometric correction data
 ----------------------------*/
 fprintf(fp,"calibration            %s \n",tipsp.band_calibration);		
 fprintf(fp,"calibration_lamp_use   %s \n",tipsp.band_calibration_lamp_use);
 fprintf(fp,"lamp_mode              %s \n",tipsp.band_lamp_mode);		
 fprintf(fp,"selected_histogram_processing %s \n",
				tipsp.band_selected_histogram_processing);	
 fprintf(fp,"number_of_scenes       %d \n",tipsp.band_number_of_scenes);     
 fprintf(fp,"scene_sequence         %d \n",tipsp.band_scene_sequence);	     
 fprintf(fp,"number_radiometric_corrections %d \n",
				tipsp.band_number_radiometric_corrections);
 fprintf(fp,"number_scans_per_rc    %d \n",tipsp.band_number_scans_per_rc);     
 fprintf(fp,"number_subsegments_per_rc %d \n",
				tipsp.band_number_subsegments_per_rc);
 fprintf(fp,"number_hist_iterations %d \n",tipsp.band_number_hist_iterations);
 fprintf(fp,"minimum_radiance       %f \n",tipsp.band_minimum_radiance);	
 fprintf(fp,"maximum_radiance       %f \n",tipsp.band_maximum_radiance);

   /* Mirror scan correction information
   -----------------------------------*/
 fprintf(fp,"nominal_line_forward   %f \n",tipsp.band_nominal_line_forward);	
 fprintf(fp,"nominal_line_backward  %f \n",tipsp.band_nominal_line_backward);	
 fprintf(fp,"maximum_line_error_forward  %f \n",
				tipsp.band_maximum_line_error_forward);	
 fprintf(fp,"maximum_line_error_backward %f \n",
				tipsp.band_maximum_line_error_backward);
 fprintf(fp,"minimum_line_error_forward  %f \n",
				tipsp.band_minimum_line_error_forward);
 fprintf(fp,"minimum_line_error_backward %f \n",
				tipsp.band_minimum_line_error_backward);	
 fprintf(fp,"mean_line_error_forward     %f \n",
				tipsp.band_mean_line_error_forward);	
 fprintf(fp,"mean_line_error_backward    %f \n",
				tipsp.band_mean_line_error_backward);
 fprintf(fp,"sigma_line_error_forward    %f \n",
				tipsp.band_sigma_line_error_forward);		
 fprintf(fp,"sigma_line_error_backward   %f \n",
				tipsp.band_sigma_line_error_backward);
 fprintf(fp,"nominal_first_line_error_forward  %f \n",
				tipsp.band_nominal_first_line_error_forward); 
 fprintf(fp,"nominal_first_line_error_backward %f \n",
				tipsp.band_nominal_first_line_error_backward);
 fprintf(fp,"maximum_first_line_error_forward  %f \n",
				tipsp.band_maximum_first_line_error_forward);
 fprintf(fp,"maximum_first_line_error_backward %f \n",
				tipsp.band_maximum_first_line_error_backward);
 fprintf(fp,"minimum_first_line_error_forward  %f \n",
				tipsp.band_minimum_first_line_error_forward);
 fprintf(fp,"minimum_first_line_error_backward %f \n",
				tipsp.band_minimum_first_line_error_backward);
 fprintf(fp,"mean_first_line_error_forward     %f \n",
				tipsp.band_mean_first_line_error_forward);	
 fprintf(fp,"mean_first_line_error_backward    %f \n",
				tipsp.band_mean_first_line_error_backward);
 fprintf(fp,"sigma_first_line_error_forward    %f \n",
				tipsp.band_sigma_first_line_error_forward);
 fprintf(fp,"sigma_first_line_error_backward   %f \n",
				tipsp.band_sigma_first_line_error_backward);	
 fprintf(fp,"nom_second_line_error_forward     %f \n",
				tipsp.band_nom_second_line_error_forward);
 fprintf(fp,"nom_second_line_error_backward    %f \n",
				tipsp.band_nom_second_line_error_backward);	
 fprintf(fp,"max_second_line_error_forward     %f \n",
				tipsp.band_max_second_line_error_forward);
 fprintf(fp,"max_second_line_error_backward    %f \n",
				tipsp.band_max_second_line_error_backward);	
 fprintf(fp,"min_second_line_error_forward     %f \n",
				tipsp.band_min_second_line_error_forward);
 fprintf(fp,"min_second_line_error_backward    %f \n",
				tipsp.band_min_second_line_error_backward);	
 fprintf(fp,"mean_second_line_error_forward    %f \n",
				tipsp.band_mean_second_line_error_forward);
 fprintf(fp,"mean_second_line_error_backward   %f \n",
				tipsp.band_mean_second_line_error_backward);	
 fprintf(fp,"sigma_second_line_error_forward   %f \n",
				tipsp.band_sigma_second_line_error_forward);
 fprintf(fp,"sigma_second_line_error_backward  %f \n",
				tipsp.band_sigma_second_line_error_backward);	
 fprintf(fp,"max_nonlin_line_error_forward     %f \n",
				tipsp.band_max_nonlin_line_error_forward);
 fprintf(fp,"max_nonlin_line_error_backward    %f \n",
				tipsp.band_max_nonlin_line_error_backward);	
 fprintf(fp,"min_nonlin_line_error_forward     %f \n",
				tipsp.band_min_nonlin_line_error_forward);
 fprintf(fp,"min_nonlin_line_error_backward    %f \n",
				tipsp.band_min_nonlin_line_error_backward);	
 fprintf(fp,"mean_nonlin_line_error_forward    %f \n",
				tipsp.band_mean_nonlin_line_error_forward);
 fprintf(fp,"mean_nonlin_line_error_backward   %f \n",
				tipsp.band_mean_nonlin_line_error_backward);	
 fprintf(fp,"sigma_nonlin_line_error_forward   %f \n",
				tipsp.band_sigma_nonlin_line_error_forward);
 fprintf(fp,"sigma_nonlin_line_error_backward  %f \n",
				tipsp.band_sigma_nonlin_line_error_backward);	

   /* Control point historical information
   -------------------------------------*/
 fprintf(fp,"quality_codes          %s \n",tipsp.band_quality_codes);		
 fprintf(fp,"scene_id               %s \n",tipsp.band_scene_id);			

 fprintf(fp,"number_scences_in_cp   %f \n",tipsp.band_number_scences_in_cp);	
 fprintf(fp,"sequence_number_in_cp  %f \n",tipsp.band_sequence_number_in_cp);
 fprintf(fp,"number_geodetic_points_in_cp %f \n",
				tipsp.band_number_geodetic_points_in_cp);	
 fprintf(fp,"number_geodetic_points_in_reference %f \n",
				tipsp.band_number_geodetic_points_in_reference);
 fprintf(fp,"average_autocorrelation %f \n",tipsp.band_average_autocorrelation);
 fprintf(fp,"average_initial_peak   %f \n",tipsp.band_average_initial_peak);	
 fprintf(fp,"elipse_along_track_interval_error  %f \n",
				tipsp.band_elipse_along_track_interval_error); 
 fprintf(fp,"elipse_across_track_interval_error %f \n",
				tipsp.band_elipse_across_track_interval_error); 
 fprintf(fp,"elipse_along_track_ref_error       %f \n",
				tipsp.band_elipse_along_track_ref_error);	
 fprintf(fp,"elipse_across_track_ref_error      %f \n",
				tipsp.band_elipse_across_track_ref_error);
 fprintf(fp,"elipse_average_success %f \n",tipsp.band_elipse_average_success);	

   /* Current control point information
   ----------------------------------*/
 fprintf(fp,"number_scenes_interval %d \n",tipsp.band_number_scenes_interval);	
 fprintf(fp,"number_cp_interval     %d \n",tipsp.band_number_cp_interval);	
 fprintf(fp,"number_cp_scene %d \n",tipsp.band_number_cp_scene);	
 fprintf(fp,"number_cp_previous_scenes                %d \n",
				tipsp.band_number_cp_previous_scenes);	
 fprintf(fp,"number_geodetic_cp     %d \n",tipsp.band_number_geodetic_cp);	
 fprintf(fp,"number_cp_correlations_interval          %d \n",
				tipsp.band_number_cp_correlations_interval);	
 fprintf(fp,"number_cp_rejected_interval              %d \n",
				tipsp.band_number_cp_rejected_interval);
 fprintf(fp,"number_cp_rejected_prefiltering_interval %d \n",
			tipsp.band_number_cp_rejected_prefiltering_interval);
 fprintf(fp,"number_cp_rejected_filtering_interval    %d \n",
			tipsp.band_number_cp_rejected_filtering_interval);
 fprintf(fp,"number_cp_correlations_scene             %d \n",
				tipsp.band_number_cp_correlations_scene);	
 fprintf(fp,"number_cp_rejected_scene                 %d \n",
				tipsp.band_number_cp_rejected_scene);		
 fprintf(fp,"number_cp_rejected_prefiltering_scene    %d \n",
			tipsp.band_number_cp_rejected_prefiltering_scene);
 fprintf(fp,"number_cp_rejected_filtering_scene       %d \n",
			tipsp.band_number_cp_rejected_filtering_scene);
 fprintf(fp,"sequence_number_in_interval              %d \n",
				tipsp.band_sequence_number_in_interval);	
 fprintf(fp,"interval_correction    %s \n",tipsp.band_interval_correction);	
 fprintf(fp,"scene_correction       %s \n",tipsp.band_scene_correction);	
 fprintf(fp,"cp_band_mode           %s \n",tipsp.band_cp_band_mode);	
 
 for (i = 0; i < 25;i++)
     fprintf(fp,"band_cp_id[%2d]    %s \n",i+1,tipsp.band_cp_id);

 fprintf(fp,"cp_interval_mode  %s \n",tipsp.band_cp_interval_mode);	
 fprintf(fp,"average_cp_curve  %f \n",tipsp.band_average_cp_curve);	
 fprintf(fp,"average_cp_peak   %f \n",tipsp.band_average_cp_peak);	

 for (i = 0; i < 25; i++)
     fprintf(fp,"cp_line[%2d]       %f \n",i+1,tipsp.band_cp_line[i]);	

 for (i = 0; i < 25; i++)
     fprintf(fp,"cp_sample[%2d]     %f \n",i+1,tipsp.band_cp_sample[i]);

   /* Geometric correction information
   ---------------------------------*/
 fprintf(fp,"geometric_quality %s \n",tipsp.band_geometric_quality);	
 fprintf(fp,"error_matrix_along_track_sigma  %f \n",
				tipsp.band_error_matrix_along_track_sigma);	
 fprintf(fp,"error_matrix_across_track_sigma %f \n",
				tipsp.band_error_matrix_across_track_sigma);
 fprintf(fp,"geometric_along_track_interval  %f \n",
				tipsp.band_geometric_along_track_interval);
 fprintf(fp,"geometric_across_track_interval %f \n",
				tipsp.band_geometric_across_track_interval);	
 fprintf(fp,"geometric_along_track_scene     %f \n",
				tipsp.band_geometric_along_track_scene);
 fprintf(fp,"geometric_across_track_scene    %f \n",
				tipsp.band_geometric_across_track_scene);	
 fprintf(fp,"horizontal_strip_maximum        %f \n",
				tipsp.band_horizontal_strip_maximum);	
 fprintf(fp,"horizontal_strip_minimum        %f \n",
				tipsp.band_horizontal_strip_minimum);
 fprintf(fp,"horizontal_strip_mean           %f \n",
				tipsp.band_horizontal_strip_mean);	
 fprintf(fp,"horizontal_strip_maximum_rotation %f \n",
				tipsp.band_horizontal_strip_maximum_rotation);
 fprintf(fp,"scan_gap_maximum      %f \n",tipsp.band_scan_gap_maximum);		
 fprintf(fp,"scan_gap_minimum      %f \n",tipsp.band_scan_gap_minimum);	
 fprintf(fp,"scan_gap_mean         %f \n",tipsp.band_scan_gap_mean);
 fprintf(fp,"scan_skew_maximum     %f \n",tipsp.band_scan_skew_maximum);	
 fprintf(fp,"scan_skew_minimum     %f \n",tipsp.band_scan_skew_minimum);	
 fprintf(fp,"scan_skew_mean        %f \n",tipsp.band_scan_skew_mean);	
 fprintf(fp,"state_vector_radial_position       %f \n",
				tipsp.band_state_vector_radial_position);	
 fprintf(fp,"state_vector_along_track_position  %f \n",
				tipsp.band_state_vector_along_track_position);
 fprintf(fp,"state_vector_across_track_position %f \n",
				tipsp.band_state_vector_across_track_position);
 fprintf(fp,"state_vector_radial_rate           %f \n",
				tipsp.band_state_vector_radial_rate);	        
 fprintf(fp,"state_vector_along_track_rate      %f \n",
				tipsp.band_state_vector_along_track_rate);
 fprintf(fp,"state_vector_across_track_rate     %f \n",
				tipsp.band_state_vector_across_track_rate);
 fprintf(fp,"state_vector_angle_roll            %f \n",
				tipsp.band_state_vector_angle_roll);	    
 fprintf(fp,"state_vector_angle_pitch           %f \n",
				tipsp.band_state_vector_angle_pitch);	   
 fprintf(fp,"state_vector_angle_yaw             %f \n",
				tipsp.band_state_vector_angle_yaw);	  
 fprintf(fp,"state_vector_roll_angular_rate     %f \n",
				tipsp.band_state_vector_roll_angular_rate);	
 fprintf(fp,"state_vector_pitch_angular_rate    %f \n",
				tipsp.band_state_vector_pitch_angular_rate);
 fprintf(fp,"state_vector_yaw_angular_rate      %f \n",
				tipsp.band_state_vector_yaw_angular_rate);	
 
 for(i = 0; i < 12; i++)
   for (j=0; j < 12; j++)
     fprintf(fp,"state_error_cov_matrix[%2d][%2d] %f \n",i + 1,j + 1,
				tipsp.band_state_error_cov_matrix[i][j]);
 fprintf(fp,"value_for_e                     %f \n",tipsp.band_value_for_e);		
 fprintf(fp,"horizontal_strip_exceeding_maximum %d \n",
				tipsp.band_horizontal_strip_exceeding_maximum);
 fprintf(fp,"horizontal_strip_exceeding_minimum %d \n",
				tipsp.band_horizontal_strip_exceeding_minimum);
 fprintf(fp,"scan_gap_exceeding_maximum      %d \n",
				tipsp.band_scan_gap_exceeding_maximum);	
 fprintf(fp,"scan_gap_exceeding_minimum      %d \n",
				tipsp.band_scan_gap_exceeding_minimum);
 fprintf(fp,"scan_skew_exceeding_maximum     %d \n",
				tipsp.band_scan_skew_exceeding_maximum);	
 fprintf(fp,"scan_skew_exceeding_minimum     %d \n",
				tipsp.band_scan_skew_exceeding_minimum);
 }

   /* trailer information
   --------------------*/
if (*tdsk)
 {
 /* make appropriate las name
 --------------------------*/
 c_lsmknm(hname,"tipsp_trail",hostheader);

 if (c_read_trail(&tipsp,hname) == E_FAIL)
   {
   fclose(fp);
   return(E_FAIL);
   }

 fprintf(fp,"\ntrailer information \n");		
 fprintf(fp,"___________________ \n\n");		
 fprintf(fp,"Trailer filename  %s \n\n", hostheader);

 fprintf(fp,"number_scan_lines %s \n",tipsp.trail_number_scan_lines);	
 fprintf(fp,"quality_indicator_summary      %s \n",
				tipsp.trail_quality_indicator_summary);
 fprintf(fp,"number_major_sync_losses       %s \n",
				tipsp.trail_number_major_sync_losses);
 fprintf(fp,"number_minor_sync_losses       %s \n",
				tipsp.trail_number_minor_sync_losses);	
 fprintf(fp,"number_minor_sync_errors       %s \n",
				tipsp.trail_number_minor_sync_errors);
 fprintf(fp,"number_bit_slips %s \n",tipsp.trail_number_bit_slips);	
 fprintf(fp,"number_time_code_substitutions %s \n",
				tipsp.trail_number_time_code_substitutions);
 fprintf(fp,"quality_summary %s \n",tipsp.trail_quality_summary);		
 }
fclose(fp);
return(E_SUCC);
}
