/*******************************************************************************
NAME:				DSTHDR

PURPOSE:      Display the contents of the FHDR fields to the 
	      specified device.

PROGRAM HISTORY:
PROGRAMMER		DATE		REASON
----------		----		------
D. Etrheim              12/92           New development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:	LAS

ALGORITHM REFERENCES:  
EOSAT Fast Format Document, Version B, October 1, 1991
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "typlim.h"
#include "header.h"
#include "protos.h"

int dsthdr
(
    char *hname,		/* TAE name of input image	     */
    char *prtfile		/* print file name                   */
)

{
FILE *fp;               /* file pointer to print file        */

long  mode = IREAD;	/* file access 			     */
long  nonfatal = 1;	/* non fatal error flag		     */
long temp1,temp2;	/* temp longs for return values      */

char blank[2];		/* blank char  			     */
char header[1536];	/* buffer containing the header file */
char tstr1[80];		/* temp buffer for return values     */
char tstr2[80];		/* temp buffer for return values     */

double dtemp1,dtemp2;   /* temp doubles for return values    */
double projp[15];	/* projection parameters             */

strcpy(blank," ");

/* Open and read header file */
if (c_open_header(header,hname,&mode) != E_SUCC)
     return(E_FAIL);

/* open the print file
----------------------*/
fp = fopen(prtfile,"w");
if (fp == NULL)
   {
   c_errmsg("Error opening print file","dspassoc-open",&nonfatal);
   return(E_FAIL);
   }
   
/* format lines for screen
---------------------------*/
fprintf(fp,"FSTFMT HEADER NAME:%-60s\n",hname);
fprintf(fp,"\n");

c_fhdrrd(header,2,tstr1);
c_fhdrrd(header,4,tstr2);
fprintf(fp,"Product: %-26s WRS: %-9s\n",tstr1,tstr2);

c_fhdrrd(header,6,tstr1);
c_fhdrrd(header,8,tstr2);
fprintf(fp,"Acquisition Date: %-17s Satellite: %-2s\n",tstr1,tstr2);

c_fhdrrd(header,10,tstr1);
c_fhdrrd(header,12,tstr2);
fprintf(fp,"Instrument: %-23s Product Type: %-14s\n", tstr1,tstr2);

c_fhdrrd(header,14,tstr1);
c_fhdrrd(header,17,tstr2);
fprintf(fp,"Product Size: %-21s Geodetic Processing: %-11s\n",tstr1,tstr2);

c_fhdrrd(header,19,tstr1);
c_fhdrrd(header,35,tstr2);
fprintf(fp,"Resampling: %-23s Tape Span Flag: %s \n",tstr1,tstr2);

fprintf(fp,"\n");

c_fhdrrd(header,21,tstr1);
c_fhdrrd(header,23,tstr2);
fprintf(fp,"Gain/Bias Band 1:%-23s  Band 2:%s \n",tstr1,tstr2);

c_fhdrrd(header,25,tstr1);
c_fhdrrd(header,27,tstr2);
fprintf(fp,"%9s Band 3:%-23s  Band 4:%s \n",blank,tstr1,tstr2);

c_fhdrrd(header,29,tstr1);
c_fhdrrd(header,31,tstr2);
fprintf(fp,"%9s Band 5:%-23s  Band 6:%s \n",blank,tstr1,tstr2);

c_fhdrrd(header,33,tstr1);
fprintf(fp,"%9s Band 7: %s\n",blank,tstr1);

c_fhdrrd(header,37,&temp1);
c_fhdrrd(header,39,&temp2);
fprintf(fp,"Start Line: %-5d %17s Lines Per Volume: %d\n",temp1,blank,temp2);

c_fhdrrd(header,41,&dtemp1);
if (dtemp1 == HUGE_VAL) dtemp1 = 0.;
fprintf(fp,"Orientation: %6.2f\n",dtemp1);

fprintf(fp,"\n");

c_fhdrrd(header,43,tstr1);
c_fhdrrd(header,45,&temp2);
fprintf(fp,"Projetion: %s %17s Projection Number: %d\n",tstr1,blank,temp2);
c_fhdrrd(header,47,&temp2);
fprintf(fp,"Zone: %d\n",temp2);

c_fhdrrd(header,49,projp);
fprintf(fp,"Proj. Parm:\n");
fprintf(fp,"A:  %1.14E   %1.14E   %1.14E\n",projp[0],projp[1],projp[2]);
fprintf(fp,"B:  %1.14E   %1.14E   %1.14E\n",projp[3],projp[4],projp[5]);
fprintf(fp,"C:  %1.14E   %1.14E   %1.14E\n",projp[6],projp[7],projp[8]);
fprintf(fp,"D:  %1.14E   %1.14E   %1.14E\n",projp[9],projp[10],projp[11]);
fprintf(fp,"E:  %1.14E   %1.14E   %1.14E\n",projp[12],projp[13],projp[14]);

c_fhdrrd(header,51,tstr1);
fprintf(fp,"Earth Ellipsoid: %s\n",tstr1);

c_fhdrrd(header,53,&dtemp1);
c_fhdrrd(header,55,&dtemp2);
fprintf(fp,"Semi-major Axis: %1.10E Semi-minor Axis: %1.10E\n",dtemp1,dtemp2);

c_fhdrrd(header,57,&dtemp1);
c_fhdrrd(header,59,&temp2);
fprintf(fp,"Pixel Size: %1.14E Pixels per line: %d\n",dtemp1,temp2);

c_fhdrrd(header,61,&temp2);
fprintf(fp,"Lines per image: %d\n",temp2);

fprintf(fp,"\n");

/* Put stuff in LAT/LON order*/
c_fhdrrd(header,65,&dtemp1);
c_fhdrrd(header,63,&dtemp2);
fprintf(fp,"UL:%f  %f \n",dtemp1,dtemp2);

c_fhdrrd(header,73,&dtemp1);
c_fhdrrd(header,71,&dtemp2);
fprintf(fp,"UR:%f  %f \n",dtemp1,dtemp2);

c_fhdrrd(header,81,&dtemp1);
c_fhdrrd(header,79,&dtemp2);
fprintf(fp,"LR:%f  %f \n",dtemp1,dtemp2);

c_fhdrrd(header,89,&dtemp1);
c_fhdrrd(header,87,&dtemp2);
fprintf(fp,"LL:%f  %f \n",dtemp1,dtemp2);

c_fhdrrd(header,69,&dtemp1);
c_fhdrrd(header,67,&dtemp2);
fprintf(fp,"\n");

fprintf(fp,"UL:%1.14E  %1.14E \n",dtemp1,dtemp2);

c_fhdrrd(header,77,&dtemp1);
c_fhdrrd(header,75,&dtemp2);
fprintf(fp,"UR:%1.14E  %1.14E \n",dtemp1,dtemp2);

c_fhdrrd(header,85,&dtemp1);
c_fhdrrd(header,83,&dtemp2);
fprintf(fp,"LR:%1.14E  %1.14E \n",dtemp1,dtemp2);

c_fhdrrd(header,93,&dtemp1);
c_fhdrrd(header,91,&dtemp2);
fprintf(fp,"LL:%1.14E  %1.14E \n",dtemp1,dtemp2);

fprintf(fp,"\n");

c_fhdrrd(header,95,tstr1);
c_fhdrrd(header,97,&temp2);
fprintf(fp,"Bands Present: %s Blocking Factor: %d\n",tstr1,temp2);

c_fhdrrd(header,99,&temp1);
c_fhdrrd(header,101,&temp2);
fprintf(fp,"Record Length: %d Sun Elevation: %d\n",temp1,temp2);

c_fhdrrd(header,103,&temp1);
fprintf(fp,"Sun Azimuth: %d\n",temp1);

fprintf(fp,"\n");

c_fhdrrd(header,107,&dtemp1);
c_fhdrrd(header,105,&dtemp2);
fprintf(fp,"Scene Center: %f %f\n",dtemp1,dtemp2);

c_fhdrrd(header,111,&dtemp1);
c_fhdrrd(header,109,&dtemp2);
fprintf(fp,"Scene Center: %1.14E %1.14E\n",dtemp1,dtemp2);

fprintf(fp,"\n");

c_fhdrrd(header,112,&temp1);
c_fhdrrd(header,113,&temp2);
fprintf(fp,"Scene Center line/pixel: %d %d\n",temp2,temp1);

c_fhdrrd(header,115,&temp1);
fprintf(fp,"Offset: %d\n",temp1);

c_fhdrrd(header,15,tstr1);
fprintf(fp,"Map sheet name: %s\n",tstr1);

c_fhdrrd(header,117,tstr1);
fprintf(fp,"Fast Format version: %s\n",tstr1);
/* close the print file
-----------------------*/
fclose(fp);

return(E_SUCC);
}
