/*******************************************************************************
NAME:				DSTHDR

PURPOSE:      Display the contents of the FHDR fields to the 
	      specified device.

PROGRAM HISTORY:
PROGRAMMER		DATE		REASON
----------		----		------
T. Mittan               12/92           New development
S. Nelson		 1/93		Added Ancillary file to EDIPS fils.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:	LAS

ALGORITHM REFERENCES:  
EOSAT Fast Format Document, Version B, October 1, 1991
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define TERM 0
#define PFILE 1
#define LP 2


void getpar
(
    char *hname,		/* TAE name of input image		      	*/
    char *prtfile,		/* print file name                            	*/
    long *hdsk,		/* header flag					*/
    long *tdsk,		/* trailer flag					*/
    long *andsk,		/* annotation flag				*/
    long *acdsk,		/* ancillary flag				*/
    long *uhdsk,		/* universal header flag			*/
    long *lhdsk,		/* LANDSAT header flag				*/
    long *vdsk,		/* volume directory file flag			*/
    long *ldsk,		/* leader file flag				*/
    long *sdsk,		/* supplemental file flag			*/
    long *idsk,		/* image data records flag			*/
    char *filename,		/* print file name				*/
    char *option,		/* used to determine which subcommand was used	*/
    long *nl,		/* line number of ancillary file of		*/
    long *print		/* output destination                         	*/
)

{
struct PARBLK parblk;  		/* TAE parameter block                 	      */
struct VARIABLE *val;           /* pointer to TAE variable within v-block     */
long  fatal = -1;
long  i;		/* array index					*/
long  mode = IUPDATE;	/* file access 					*/
long  std_out;		/* standard output variable			*/
long  zero = 0;		/* value of zero				*/

char assocfile[10];	/* associate files				*/
char tempfile[CMLEN];	/* associate files				*/


c_inlas(&parblk,&std_out);              /* initialization */

/* Determine which subcomand was used
-----------------------------------*/
val = p_find(&parblk, "_SUBCMD");
strcpy(tempfile,SVAL(*val,0));
if (strcmp(SVAL(*val, 0), "FFMT") == 0)
   strcpy(option,"FFMT");
else
if (strcmp(SVAL(*val, 0), "TIPSP") == 0)
   strcpy(option,"TIPSP");
else
if (strcmp(SVAL(*val, 0), "CCTX") == 0)
   strcpy(option,"SIAT");
else
if (strcmp(SVAL(*val, 0), "EDIPS") == 0)
   strcpy(option,"EDIPS");
else
if (strcmp(SVAL(*val, 0), "JSC") == 0)
   strcpy(option,"JSC");
else
if (strcmp(SVAL(*val, 0), "LTWG") == 0)
   strcpy(option,"LTWG");

if (c_getnam(&parblk,"INFILE",&zero,hname,"img",&mode) != E_SUCC)
   c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);

if ((strcmp(option,"TIPSP") == 0) || (strcmp(option,"EDIPS") == 0)
	|| (strcmp(option,"JSC") == 0) || (strcmp(option,"LTWG") ==0))
  {
  val = p_find(&parblk,"ASFILE");
  if (val->v_count != 0)
    for (i = 0; i < val->v_count; i++)
        {
        strcpy(assocfile,SVAL(*val,i));
        if (strcmp(assocfile,"HDR") == 0)
            *hdsk = TRUE;
        if (strcmp(assocfile,"TRA") == 0)
            *tdsk = TRUE;
        if (strcmp(assocfile,"ANB") == 0)
            *andsk = TRUE;
        if (strcmp(assocfile,"ANC") == 0)
            *acdsk = TRUE;
        if (strcmp(assocfile,"UHDR") == 0)
            *uhdsk = TRUE;
        if (strcmp(assocfile,"LHDR") == 0)
            *lhdsk = TRUE;
        if (strcmp(assocfile,"VDF") == 0)
 	    *vdsk = TRUE;
	if (strcmp(assocfile,"LEAD") == 0)
	    *ldsk = TRUE;
	if (strcmp(assocfile,"TRAI") == 0)
	    *tdsk = TRUE;
	if (strcmp(assocfile,"SUPP") == 0)
	    *sdsk = TRUE;
	if (strcmp(assocfile,"IDR") == 0)
	    *idsk = TRUE;
	}
  }
if ((strcmp(option,"JSC") == 0) && (*acdsk == TRUE))
  {
  val = p_find(&parblk,"LINENUM");
  *nl = IVAL(*val,0);
  }

if (strcmp(option,"LTWG") == 0)
  {
  val = p_find(&parblk,"LINENUM");
  *nl = IVAL(*val,0);
  }

/* get print option
-------------------*/
strcpy(prtfile,"dspfhdr");
if (c_setprt(&parblk,&print[TERM],&print[PFILE],&print[LP],
             prtfile,filename) != E_SUCC)
   c_errmsg("Fatal error encountered","dspassoc-fatal",&fatal);

return;
}
