/*******************************************************************************
NAME:				DSPDDR

PURPOSE:      Display the contents of the DDR file to the specified device.

PROGRAM HISTORY:
PROGRAMMER		DATE		REASON
----------		----		------
K. Zanter	      Feb. 1988		Original development
B. Ailts	      Jul. 1988         LAS 5.0 conversion
B. Ailts	      Dec. 1988		Removed mknam and cmtae -- setprt now
					returns full tae name
					Closed files on fatal errors
B. Ailts	      Mar. 1989		Now accepts alias names
B. Ailts	      Oct. 1989         Changed last accessed field to be
					last modified
P. Pease/GSFC         NOV. 1989         Changed call from clean to c_clean.
D. Etrheim            Nov. 1994         Add output to project parameter file.
C. Gallap             Mar. 1998         Add addition labels on corner coords.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   DSPDDR must be run under TAE.

PROJECT:	LAS

ALGORITHM DESCRIPTION:

ALGORITHM REFERENCES:  The format of the screens used in DSPDDR
		       are nearly identical to those used in EDITDDR.
*******************************************************************************/
#include  <stdio.h>
#include  <stdlib.h>

#include "las.h"
#include "util.h"
#include "protos.h"

#define TERM 0
#define PFILE 1
#define LP 2

int main
(void)
{
FILE *fp = NULL;		/* file pointer to print file		      */

struct PARBLK parblk;		/* TAE parameter block			      */

struct VARIABLE *var;		/* pointer to a TAE variable		      */

struct DDR ddr;			/* data in records 1 and 2 of DDR 	      */
struct BDDR bddr;		/* data in the band records 		      */

long	bands[MAXBND];		/* specified band numbers 		      */
long    fatal = -1;		/* fatal error code			      */
long	i;			/* loop index				      */
long	mode;			/* file access mode			      */
long    one = 1;		/* one					      */
long	print[3];		/* output destination			      */
long	ppfflag = FALSE;	/* flag if output file is a PPF file	      */
long	status;			/* function return status code		      */
long	std_out;		/* logical unit number for standard output    */
long	totbnd;			/* total number of bands to be processed      */
long    zero = 0;		/* zero					      */

char hname[CMLEN];		/* host name of input image		      */
char temp[1][CMLEN];		/* temporary name of output ppf file        */
char filename[1][CMLEN];		/* filename specified by the user	      */
char prtfile[1][CMLEN];		/* print file name			      */
char ppfname[1][CMLEN];		/* ppf file name			      */

c_inlas(&parblk,&std_out);		/* initialization */

/* Get the parameters 
---------------------*/ 
mode = IREAD;
if (c_getnam(&parblk, "IN", &zero, hname, "ddr", &mode) != E_SUCC)
        c_errmsg("Fatal error encountered","dspddr-fatal",&fatal);

mode = IUPDATE;
if (c_getnam(&parblk, "OUTPPF", &zero, ppfname[0], "ppf", &mode) != E_SUCC)
        c_errmsg("Fatal error encountered","dspddr-fatal",&fatal);
if (strcmp(ppfname[0],"") != 0)
   ppfflag = TRUE;

/* get records 1 and 2 of DDR file
----------------------------------*/
status = c_getddr(hname,&ddr);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","dspddr-fatal",&fatal);

/* get band number specification
--------------------------------*/
var = p_find(&parblk,"BANDS");
if ( (*var).v_count == 0)
   {
   totbnd = ddr.nbands;
   for (i = 0; i < totbnd; i++)
       bands[i] = i+1;
   }
else
   {
   totbnd = (*var).v_count;
   for (i = 0; i < totbnd; i++)
       {
       bands[i] = IVAL(*var,i);
       if (bands[i] > ddr.nbands)
	  c_errmsg("Invalid band number","dspddr-bands",&fatal);
       }
   }  /*  bands != null value  */


/* get print option if needed
-----------------------------*/
if (!ppfflag)
   {
   strcpy(prtfile[0],"dspddr");
   status = c_setprt(&parblk,&print[TERM],&print[PFILE],&print[LP],prtfile[0],
                    filename[0]);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","dspddr-fatal",&fatal);

   /* open the print file (if necessary)
   -------------------------------------*/
   if ( (print[PFILE]) || (print[LP]) )
      fp = fopen(prtfile[0],"w");
   }
else
   print[TERM] = print[PFILE] = print[LP] = FALSE;

/* output data for records 1 and 2
----------------------------------*/
if (ppfflag)
   {
   if (ppfimg(ppfname[0],hname,&ddr) != E_SUCC)
      c_clean(&parblk,"dspddr",ppfname,&one);
   }
else
   dstimg(fp,print,hname,&ddr);

/* get and display band dependent records 
-----------------------------------------*/
for (i = 0; i < totbnd; i++)
    {
    if (c_getbdr(hname,&bddr,&bands[i]) != E_SUCC)
       {
       if ((print[PFILE]) || (print[LP]))
          fclose(fp);

       if (ppfflag)
          c_clean(&parblk,"dspddr",ppfname,&one);
       else
          c_clean(&parblk,"dspddr",prtfile,&one);
       }
    if (ppfflag)
       {
       if (ppfbnd(ppfname[0],&bands[i],&bddr) != E_SUCC)
          c_clean(&parblk,"dspddr",ppfname,&one);
       }
    else
       dstbnd(fp,print,hname,&bands[i],&bddr);
    }

/* close the print file (if necessary)
--------------------------------------*/
if ( (print[PFILE]) || (print[LP]) )
   fclose(fp);

/* inform user of successful completion
---------------------------------------*/
if (print[PFILE]) 
   c_complt("DSPDDR",filename,&one);
else if (ppfflag)
   c_complt("DSPDDR",ppfname[0],&one);
else
  {
  strcpy(temp[0], "\0");
   c_complt("DSPDDR",temp,&zero);
  }

return EXIT_SUCCESS;

} /*  main  */
