/*******************************************************************************
NAME:				DSTBND

PURPOSE:      Display the contents of the band record of the DDR file to 
	      the specified device.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:	LAS

ALGORITHM DESCRIPTION:
Format the output strings for each of the BDDR members
If term
   write out each of the output strings to the terminal
If file or lp
   write out each of the output strings to the prtfile

ALGORITHM REFERENCES:  The format of the screens used in DSPDDR
		       are nearly identical to those used in EDITDDR.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define TERM 0
#define PFILE 1
#define LP 2

void dstbnd
(
    FILE *fp,	  	 /* file pointer to print file		              */
    long *print,		 /* output destination			              */
    char *tname,	 	 /* TAE name of input image      */
    long *band,		 /* pointer to current band number		      */
    struct BDDR *bddr	 /* source of data		      		      */
)
{
long	flag;			/* flag for "c_pterm" call		      */

char blank[81];			/* blank line				      */
char line1[CMLEN];		/* line1				      */
char line2[81];			/* line2				      */
char line4[81];			/* line4				      */
char line5[81];			/* line5				      */
char line7[81];			/* line7				      */
char line8[81];			/* line8				      */
char line9[81];			/* line9				      */
char line10[81];		/* line10				      */
char line11[81];		/* line11				      */
char vald[23];			/* validity flag value     		      */

/* set up screen for band information
-------------------------------------*/
sprintf(line1,"IMAGE NAME:%-60s",tname);
sprintf(line2,"   BAND NO:%-3d",*band);
mmval(bddr->valid,vald);
sprintf(line4,
    "        MINIMUM:%-1.14E                       %-15s",
      bddr->minval,vald);
sprintf(line5,
    "        MAXIMUM:%-1.14E                       %-15s",
      bddr->maxval,vald);
sprintf(line7,"    DATA SOURCE:%-31s",bddr->source);
sprintf(line8,"    SENSOR TYPE:%-31s",bddr->instrument);
sprintf(line9,"CAPT. DIRECTION:%-63s",bddr->direction);
sprintf(line10,"           DATE:%-9s",bddr->date);
sprintf(line11,"           TIME:%-7s",bddr->time);

/* set up for a blank line
--------------------------*/
strcpy(blank," ");

/* output to terminal
---------------------*/
if (print[TERM])
   {
   t_clear();
   flag = 0;
   c_pterm(line1,&flag);
   c_pterm(line2,&flag);
   c_pterm(blank,&flag);
   c_pterm(line4,&flag);
   c_pterm(line5,&flag);
   c_pterm(blank,&flag);
   c_pterm(line7,&flag);
   c_pterm(line8,&flag);
   c_pterm(line9,&flag);
   c_pterm(line10,&flag);
   flag = 1;
   c_pterm(line11,&flag);
   }

/* output to print file
-----------------------*/
if ( (print[PFILE]) || (print[LP]) )
   {
   fprintf(fp,"\f");
   fprintf(fp,"%s\n",line1);
   fprintf(fp,"%s\n",line2);
   fprintf(fp,"%s\n",blank);
   fprintf(fp,"%s\n",line4);
   fprintf(fp,"%s\n",line5);
   fprintf(fp,"%s\n",blank);
   fprintf(fp,"%s\n",line7);
   fprintf(fp,"%s\n",line8);
   fprintf(fp,"%s\n",line9);
   fprintf(fp,"%s\n",line10);
   fprintf(fp,"%s\n",line11);
   }

return;
}

/*******************************************************************************
NAME:				MMVAL

PURPOSE:      To create the correct string according to the value of 
	      the minmax valid flag

ALGORITHM DESCRIPTION:
Based on the value of input
   return the correct string

*******************************************************************************/
void mmval
(
    long input,				/* input validity flag numeric value  */
    char str[]				/* validity flag's character value    */
)
{
switch (input)
   {
   case INVAL: sprintf (str,"Valid:INVALID");
	    break;
   case VALID: sprintf (str,"Valid:VALID");
	    break;
   case BOUND: sprintf (str,"Valid:BOUNDED");
	    break;
   default: sprintf (str,"Valid:UNACCEPTABLE");
   } 
return;
}
