/*******************************************************************************
NAME:				DSTIMG

PURPOSE:      Display the contents of the general DDR fields to the 
	      specified device.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:	LAS

ALGORITHM DESCRIPTION:
The output strings are formated for each of the fields of the DDR
If terminal
   Print the strings to the screen
If lp or file
   Print the strings to the prtfile

ALGORITHM REFERENCES:  The format of the screens used in DSPDDR
		       are nearly identical to those used in EDITDDR.
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

#define TERM 0
#define PFILE 1
#define LP 2

void dstimg
(
    FILE *fp,			/* file pointer to print file		      */
    long *print,			/* output destination			      */
    char *tname,		        /* TAE name of input image		      */
    struct DDR *ddr		/* pointer to integer data in record 1	      */
)
{
long	flag;			/* flag for "c_pterm" call		      */

char coord[26];                 /* coordinate label                           */
char dtm_str[45];		/* buffer for the datum string		      */
char line1[CMLEN];		/* buffer for line 1			      */
char line2[81];			/* buffer for line 2			      */
char line3[81];			/* buffer for line 3			      */
char line4[81];			/* buffer for line 4			      */
char line5[81];			/* buffer for line 5			      */
char line6[81];			/* buffer for line 6			      */
char line7[81];			/* buffer for line 7			      */
char line8[81];			/* buffer for line 8			      */
char line9[81];			/* buffer for line 9			      */
char line10[81];		/* buffer for line 10			      */
char line11[81];		/* buffer for line 11		 	      */
char line12[81];		/* buffer for line 12			      */
char line13[81];		/* buffer for line 13			      */
char line14[81];		/* buffer for line 14			      */
char line15[81];		/* buffer for line 15			      */
char line16[81];		/* buffer for line 16			      */
char line17[81];		/* buffer for line 17			      */
char line18[81];		/* buffer for line 18			      */
char line19[81];		/* buffer for line 19			      */
char line20[81];		/* buffer for line 20			      */
char line21[81];		/* buffer for line 21			      */
char proj_str[45];		/* buffer for the projection string	      */
char vald[23];			/* buffer storage for validity flag value     */

/* format lines for first screen
--------------------------------*/
sprintf(line1,"  IMAGE NAME:%-60s",tname);
switch (ddr->dtype)
    {
   case EBYTE: sprintf(line2,
"          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:BYTE",
     ddr->nl,ddr->ns,ddr->nbands);
     break;
   case EWORD:sprintf(line2,
"          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:INTEGER*2",
     ddr->nl,ddr->ns,ddr->nbands);
     	    break;
   case ELONG:sprintf(line2,
"          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:INTEGER*4",
     ddr->nl,ddr->ns,ddr->nbands);
     	    break;
   case EREAL:sprintf(line2,
"          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:REAL*4",
     ddr->nl,ddr->ns,ddr->nbands);
     	    break;
   default:sprintf(line2,
"          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:%-d",
     ddr->nl,ddr->ns,ddr->nbands,ddr->dtype);
            break;
   }

strcpy(coord,"  ");
if ( strstr(ddr->proj_units, "feet")  != NULL ||
     strstr(ddr->proj_units, "meter") != NULL)
  sprintf(coord, "Y                      X  ");
else if ( strstr(ddr->proj_units, "rad")  != NULL ||
          strstr(ddr->proj_units, "sec")  != NULL ||
	  strstr(ddr->proj_units, "deg")  != NULL)
  sprintf(coord, "LAT                  LONG ");

sprintf(line3,"  LAST MODIFIED:         DATE:%-11s  TIME:%-10sSYSTEM:%-11s",
	 ddr->last_used_date, ddr->last_used_time, ddr->system);
pntval(ddr->valid[DDPCV],vald);
c_prostr(&(ddr->proj_code),proj_str);
sprintf(line4,
        "  %-9s%-*s%-*s%-15s","PROJ. CODE:",
        strlen(proj_str),proj_str,(46 - strlen(proj_str))," ",vald);
pntval(ddr->valid[DDZCV],vald);
sprintf(line5,
    "   ZONE CODE:%-4d                                          %-15s",
    ddr->zone_code,vald);
pntval(ddr->valid[DDDCV],vald);
c_dtmstr(&(ddr->datum_code),dtm_str);
sprintf(line6,
        "  %-9s%-*s%-*s%-15s","DATUM CODE:",
        strlen(dtm_str),dtm_str,(46 - strlen(dtm_str))," ",vald);
pntval(ddr->valid[DDPPV],vald);
sprintf(line7,
    "  PROJ. PARM:                                              %-15s",vald);
sprintf(line8,
    "  A:   %1.14E   %1.14E   %1.14E",
   ddr->proj_coef[0],ddr->proj_coef[1],ddr->proj_coef[2]);
sprintf(line9,
    "  B:   %1.14E   %1.14E   %1.14E",
      ddr->proj_coef[3],ddr->proj_coef[4],ddr->proj_coef[5]);
sprintf(line10,
    "  C:   %1.14E   %1.14E   %1.14E",
      ddr->proj_coef[6],ddr->proj_coef[7],ddr->proj_coef[8]);
sprintf(line11,
    "  D:   %1.14E   %1.14E   %1.14E",
      ddr->proj_coef[9],ddr->proj_coef[10],ddr->proj_coef[11]);
sprintf(line12,
    "  E:   %1.14E   %1.14E   %1.14E",
      ddr->proj_coef[12],ddr->proj_coef[13],ddr->proj_coef[14]);
pntval(ddr->valid[DDCCV],vald);
sprintf(line13,
    " CORNER COOR:         %26s           %-15s", coord, vald);
sprintf(line14,"    ULcorner:%-1.14E   %-1.14E",
	ddr->upleft[0],ddr->upleft[1]);
sprintf(line15,"    URcorner:%-1.14E   %-1.14E",
	ddr->upright[0],ddr->upright[1]);
sprintf(line16,"    LLcorner:%-1.14E   %-1.14E",
	ddr->loleft[0],ddr->loleft[1]);
sprintf(line17,"    LRcorner:%-1.14E   %-1.14E",
	ddr->loright[0],ddr->loright[1]);
pntval(ddr->valid[DDPDV],vald);
sprintf(line18,
    "  PROJ. DIST:%-1.14E   %1.14E   %-15s",
      ddr->pdist_y,ddr->pdist_x,vald);
pntval(ddr->valid[DDPUV],vald);
sprintf(line19,
    " PROJ. UNITS:%-10s                                    %-15s",
      ddr->proj_units,vald);
pntval(ddr->valid[DDINCV],vald);
sprintf(line20,
    "   INCREMENT:%-1.14E   %1.14E   %-15s",
      ddr->line_inc,ddr->sample_inc,vald);
sprintf(line21,
    " MASTER COOR:%-5d   %-5d",
      ddr->master_line,ddr->master_sample);

/* output to terminal
---------------------*/
if (print[TERM])
   {
   flag = 0;
   t_clear();
   c_pterm(line1,&flag);
   c_pterm(line2,&flag);
   c_pterm(line3,&flag);
   c_pterm(line4,&flag);
   c_pterm(line5,&flag);
   c_pterm(line6,&flag);
   c_pterm(line7,&flag);
   c_pterm(line8,&flag);
   c_pterm(line9,&flag);
   c_pterm(line10,&flag);
   c_pterm(line11,&flag);
   c_pterm(line12,&flag);
   c_pterm(line13,&flag);
   c_pterm(line14,&flag);
   c_pterm(line15,&flag);
   c_pterm(line16,&flag);
   c_pterm(line17,&flag);
   c_pterm(line18,&flag);
   c_pterm(line19,&flag);
   c_pterm(line20,&flag);
   flag = 1;
   c_pterm(line21,&flag);
   }
   
/* output to print file
-----------------------*/
if ( (print[PFILE]) || (print[LP]) )
   {
   fprintf(fp,"%s\n",line1);
   fprintf(fp,"%s\n",line2);
   fprintf(fp,"%s\n",line3);
   fprintf(fp,"%s\n",line4);
   fprintf(fp,"%s\n",line5);
   fprintf(fp,"%s\n",line6);
   fprintf(fp,"%s\n",line7);
   fprintf(fp,"%s\n",line8);
   fprintf(fp,"%s\n",line9);
   fprintf(fp,"%s\n",line10);
   fprintf(fp,"%s\n",line11);
   fprintf(fp,"%s\n",line12);
   fprintf(fp,"%s\n",line13);
   fprintf(fp,"%s\n",line14);
   fprintf(fp,"%s\n",line15);
   fprintf(fp,"%s\n",line16);
   fprintf(fp,"%s\n",line17);
   fprintf(fp,"%s\n",line18);
   fprintf(fp,"%s\n",line19);
   fprintf(fp,"%s\n",line20);
   fprintf(fp,"%s\n",line21);
   }
return;
}


void pntval
(
    long input,				/* input validity flag numeric value  */
    char str[]				/* validity flag's character value    */
)
{
switch (input)
   {
   case INVAL: sprintf (str,"Valid:INVALID");
	    break;
   case VALID: sprintf (str,"Valid:VALID");
	    break;
   case UNKNOW: sprintf (str,"Valid:UNKNOWN");
	    break;
   default: sprintf (str,"Valid:UNACCEPTABLE");
   } 
return;
}
