/*******************************************************************************
NAME:				PPFBND

PURPOSE:      Writes the contents of the band record of the DDR file to 
	      a project parameter file.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:	LAS

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int ppfbnd
(
    char *ppfname,
    long *band,		 /* pointer to current band number		      */
    struct BDDR *bddr	 /* source of data		      		      */
)
{

char temp[CMLEN];		/* temporary working string                   */
char vald[1][CMLEN];			/* validity flag value     		      */
char str_val[1][CMLEN];		/* Temp array for calls to c_putparm          */
char *ptr;			/* string pointer			      */

long lng_val[20];               /* Temp array for calls to c_putparm          */
long len;			/* String length                              */

double dbl_val[20];             /* Temp array for calls to c_putparm          */

/* Write the band dependent information to the file.  Strings with
   length zero are not written to the file. */
switch (bddr->valid)
   {
   case INVAL: sprintf (vald[0],"INVALID");
            break;
   case VALID: sprintf (vald[0],"VALID");
            break;
   case BOUND: sprintf (vald[0],"BOUNDED");
            break;
   default: sprintf (vald[0],"UNACCEPTABLE");
   }

sprintf(temp,"MININUM_BAND_%d",*band);
dbl_val[0] = bddr->minval;
if (c_putparm(ppfname,temp,str_val,"R",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);
sprintf(temp,"MAXIMUM_BAND_%d",*band);
dbl_val[0] = bddr->maxval;
if (c_putparm(ppfname,temp,str_val,"R",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);
sprintf(temp,"BAND_%d_VALID",*band);
if (c_putparm(ppfname,temp, vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

sprintf(temp,"DATA_SOURCE_%d",*band);
len = (long) strlen(bddr->source);
ptr = rem_blanks(bddr->source,&len);
if (strlen(ptr))
   {
   strcpy(str_val[0], ptr);
   if (c_putparm(ppfname,temp, str_val ,"S",dbl_val,1,lng_val) != E_SUCC)
      return(E_FAIL);
   }
free(ptr);

sprintf(temp,"SENSOR_TYPE_%d",*band);
len = (long) strlen(bddr->instrument);
ptr = rem_blanks(bddr->instrument,&len);
if (strlen(ptr))
   {
   strcpy(str_val[0],ptr);
   if (c_putparm(ppfname,temp, str_val,"S",dbl_val,1,lng_val) 
                 != E_SUCC)
   return(E_FAIL);
   }
free(ptr);

sprintf(temp,"CAP_DIRECTION_%d",*band);
len = (long) strlen(bddr->direction);
ptr = rem_blanks(bddr->direction,&len);
if (strlen(ptr))
   {
   strcpy(str_val[0],ptr);
   if (c_putparm(ppfname,temp, str_val,"S",dbl_val,1,lng_val) 
                 != E_SUCC)
   return(E_FAIL);
   }
free(ptr);

sprintf(temp,"CAP_DATE_%d",*band);
len = (long) strlen(bddr->date);
ptr = rem_blanks(bddr->date,&len);
if (strlen(ptr))
   {
   strcpy(str_val[0],ptr);
   if (c_putparm(ppfname,temp, str_val,"S",dbl_val,1,lng_val) != E_SUCC)
      return(E_FAIL);
   }
free(ptr);

sprintf(temp,"CAP_TIME_%d",*band);
len = (long) strlen(bddr->time);
ptr = rem_blanks(bddr->time,&len);
if (strlen(ptr))
   {
   strcpy(str_val[0], ptr);
   if (c_putparm(ppfname,temp, str_val,"S",dbl_val,1,lng_val) != E_SUCC)
      return(E_FAIL);
   }
free(ptr);

return(E_SUCC);
}
