/*******************************************************************************
NAME:				PPFIMG

PURPOSE:      Write the contents of the general DDR fields to a
	      project parameter file.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:	LAS

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "geompak.h"
#include "protos.h"

int ppfimg
(
    char *ppfname,
    char *tname,		        /* TAE name of input image		     */
    struct DDR *ddr		/* pointer to integer data in record 1	     */
)
{
char  proj_str[45];		/* buffer for the projection string	     */
char  *ptr;			/* string pointer                            */
char   str_val[1][CMLEN];		/* Temp array for calls to c_putparm         */
char vald[1][CMLEN];			/* buffer storage for validity flag value    */
char tmp_filename[CMLEN];       /* Actual file name with the .ppf extension  */
char stprojcoef[50];            /* holds the corrected proj. coef.           */

long  len;			/* string length                             */
long  i = 0;                    /* counter variable                          */
long  nonfatal = 1;             /* Nonfatal error flag                       */
long  lng_val[20];              /* Temp array for calls to c_putparm         */

double dbl_val[20];             /* Temp array for calls to c_putparm         */

FILE *parm_file;                /* File pointer to project parameter file    */

/* Write the DDR information to the project parameter file 
----------------------------------------------------------*/
strcpy(str_val[0], tname);

if (c_putparm(ppfname,"NAME",str_val,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

lng_val[0] = ddr->nl;
if (c_putparm(ppfname,"NL",str_val,"I",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

lng_val[0] = ddr->ns;
if (c_putparm(ppfname,"NS",str_val,"I",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

lng_val[0] = ddr->nbands;
if (c_putparm(ppfname,"NB",str_val,"I",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

switch (ddr->dtype)
    {
    case EBYTE: 
     	strcpy(str_val[0],"BYTE");
     	break;
    case EWORD:
     	strcpy(str_val[0],"INTEGER*2");
     	break;
    case ELONG:
     	strcpy(str_val[0],"INTEGER*4");
	break;
    case EREAL:
      	strcpy(str_val[0],"REAL*4");
	break;
   }
if (c_putparm(ppfname,"DTYPE",str_val,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

strcpy(str_val[0], ddr->last_used_date);
if (c_putparm(ppfname,"DATE", str_val,"S",dbl_val,1,lng_val) 
              != E_SUCC)
   return(E_FAIL);

strcpy(str_val[0], ddr->last_used_time);
if (c_putparm(ppfname,"TIME", str_val,"S",dbl_val,1,lng_val) 
              != E_SUCC)
   return(E_FAIL);

strcpy(str_val[0], ddr->system);
if (c_putparm(ppfname,"SYSTEM", str_val,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

ppfval(ddr->valid[DDPCV],vald[0]);
c_prostr(&(ddr->proj_code),proj_str);

len = (long) strlen(proj_str);
ptr = rem_blanks(proj_str,&len);
if (strlen(ptr))
    {
    strcpy(str_val[0],ptr);
    if (c_putparm(ppfname,"PROJ_CODE",str_val,"S",dbl_val,1,lng_val) != E_SUCC)
        return(E_FAIL);
    }
free(ptr);


if (c_putparm(ppfname,"PROJ_VALID", vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

ppfval(ddr->valid[DDZCV],vald[0]);
lng_val[0] = ddr->zone_code;
if (c_putparm(ppfname,"ZONE_CODE",str_val,"I", dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

if (c_putparm(ppfname,"ZONE_VALID", vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

ppfval(ddr->valid[DDDCV],vald[0]);
lng_val[0] = ddr->datum_code;
if (c_putparm(ppfname,"DATUM_CODE",str_val,"I",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

if (c_putparm(ppfname,"DATUM_VALID", vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

ppfval(ddr->valid[DDPPV],vald[0]);

/* c_putparm is not used because we want to print the
   proj_parm's with 14 decimal places.
-----------------------------------------------------*/

  /* Add the .ppf extension to filename
  -------------------------------------*/
  c_lsmknm(ppfname,"ppf",tmp_filename);

  /* Opening the ppf file and appending the PROJ_PARM's to it
  -----------------------------------------------------------*/
  if ((parm_file = fopen(tmp_filename,"a")) == NULL)
     {
     c_errmsg("Error opening project parameter file","ppfimg-open",&nonfatal);
     return(E_FAIL);
     }
  fprintf(parm_file,"%-20s R %d","PROJ_PARM",15);
  for(i = 0; i < 15; i++)
  {
    FixProjParm(ddr->proj_coef[i],stprojcoef);
    fprintf(parm_file," %s",stprojcoef);
  }
  fprintf(parm_file,"\n");
  fclose(parm_file);

if (c_putparm(ppfname,"PARM_VALID", vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

ppfval(ddr->valid[DDCCV],vald[0]);
if (c_putparm(ppfname,"UL",str_val,"R",ddr->upleft,2,lng_val) != E_SUCC)
   return(E_FAIL);
if (c_putparm(ppfname,"UR",str_val,"R",ddr->upright,2,lng_val) != E_SUCC)
   return(E_FAIL);
if (c_putparm(ppfname,"LL",str_val,"R",ddr->loleft,2,lng_val) != E_SUCC)
   return(E_FAIL);
if (c_putparm(ppfname,"LR",str_val,"R",ddr->loright,2,lng_val) != E_SUCC)
   return(E_FAIL);
if (c_putparm(ppfname,"CORNER_VALID", vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

ppfval(ddr->valid[DDPDV],vald[0]);
dbl_val[0] = ddr->pdist_y;
dbl_val[1] = ddr->pdist_x;
if (c_putparm(ppfname,"PROJ_DIST",str_val,"R",dbl_val,2,lng_val) != E_SUCC)
   return(E_FAIL);
if (c_putparm(ppfname,"DIST_VALID", vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

ppfval(ddr->valid[DDPUV],vald[0]);
len = (long) strlen(ddr->proj_units);
ptr = rem_blanks(ddr->proj_units,&len);
if (strlen(ptr))
   {
   strcpy(str_val[0],ptr);
   if (c_putparm(ppfname,"PROJ_UNITS", str_val,"S",dbl_val,1,lng_val) 
             != E_SUCC)
      return(E_FAIL);
   }
if (c_putparm(ppfname,"UNITS_VALID", vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);
free(ptr);

ppfval(ddr->valid[DDINCV],vald[0]);
dbl_val[0] = ddr->line_inc;
dbl_val[1] = ddr->sample_inc;
if (c_putparm(ppfname,"INCREMENT",str_val,"R",dbl_val,2,lng_val) != E_SUCC)
   return(E_FAIL);
if (c_putparm(ppfname,"INCREMENT_VALID", vald,"S",dbl_val,1,lng_val) != E_SUCC)
   return(E_FAIL);

lng_val[0] = ddr->master_line;
lng_val[1] = ddr->master_sample;
if (c_putparm(ppfname,"MASTER_COOR",str_val,"I",dbl_val,2,lng_val) != E_SUCC)
   return(E_FAIL);

return(E_SUCC);
}

void ppfval
(
    long input,                          /* input validity flag numeric value  */
    char str[]                          /* validity flag's character value    */
)
{
switch (input)
   {
   case INVAL: sprintf (str,"INVALID");
            break;
   case VALID: sprintf (str,"VALID");
            break;
   case UNKNOW: sprintf (str,"UNKNOWN");
            break;
   default: sprintf (str,"UNACCEPTABLE");
   }
return;
}

void FixProjParm
(
    double num,                         /* projection parameter to change      */
    char *newnum                       /* holds the new proj. parm value      */
)
{
  char stnum[50];                   /* holds the number in string form     */
  char temp[50];                    /* temp variable used for processing   */
  char stnewnum[50];                /* holds the new number in string form */
  char *p;                          /* pointer to the exponent of the #    */
  char sign;                        /* either +/- character                */
  long expnum = 0;                  /* exponent of the number              */
  long lenum = 0;                   /* length of the number                */
  long ii = 0;                      /* loop control variable               */
  long index = 0;                   /* loop control variable               */
  long decpos = 0;                  /* what position the decimal is in     */

  /* Making sure the string variables are empty
  ---------------------------------------------*/
  memset(stnum,0,50);
  memset(temp,0,50);
  memset(stnewnum,0,50);

  /* Putting the number into a string like
     it appears in the dspddr screen output.
  ------------------------------------------*/
  sprintf(stnum,"%1.14E",num);

  /* Getting the exponent of the number
  -------------------------------------*/
  p = (char *)strchr(stnum,'E');
  lenum = (long) (strlen(stnum) - strlen(p));
  p++;
  strcpy(temp,p);
  sscanf(temp,"%c%d",&sign,&expnum);

  /* Moving the decimal point in the number
  -----------------------------------------*/
  if (sign == '-')
  {
    stnewnum[0] = '0';
    stnewnum[1] = '.';
    for(index = 2; index < expnum+1; index++)
      stnewnum[index] = '0';

    for(ii = 0; ii < lenum; ii++)
    {
      if (stnum[ii] != '.')
      {
        stnewnum[index] = stnum[ii];
        index++;
      }
    }
  }
  else if (sign == '+')
  {
    index = 0;
    for(ii = 0; ii < lenum; ii++)
    {
      if (stnum[ii] != '.')
      {
        if ((decpos + expnum) < ii)
        {
          stnewnum[index] = '.';
          index++;
          decpos = 99;
        }
        stnewnum[index] = stnum[ii];
        index++;
      }
      else if (stnum[ii] == '.')
        decpos=ii;
    }
  }
  sprintf(newnum,"%s",stnewnum);
}
