/*******************************************************************************
SUBPROGRAM:                    GETPAR 

PURPOSE:        Retrieve the user specified parameters.

*******************************************************************************/
#include "las.h"
#include "gof.h"
#include "util.h"
#include "protos.h"


int getpar
(
    char *subcmd,           /* The subcommand */
    int  *goftyp,           /* The GOF types (by name) specified */
    long *prtfil,           /* Indicates whether GOF information is to be
                               output to a file */
    char *in,               /* The input source file */
    char attrval[][ATTIN],  /* The attribute values specified */
    long *prtpts,           /* Whether to print the points pairs in the 
                               long format */
    long *numrec,           /* Number of records specified */
    long recno[100],        /* The list of records specified */
    long *numattr,          /* Number of attributes specified */
    char *pfile,            /* Temporary file to write to for TERM, LP or
                               user specified file name */
    char *ufile,            /* User specified file name */
    struct PARBLK *vblock   /* TAE variable block */
)
{
    struct VARIABLE *var;   /* Pointer to variable in parameter block */

    long i;                 /* Loop counter */
    long std_out;           /* Standard output */
    long term;              /* Indicates whether GOF information is to be
                               output to the terminal */
    long lp;                /* Indicates whether GOF information is to be
                               output to the line printer */
    long zero = 0;          /* zero */

    char prt[4];            /* Print option */
    char tmpstr[1][CMLEN];


    /* Initialize TAE
       --------------*/
    for (i=0; i < 100; i++)
        recno[i]=0;
    for (i=0; i < MAXATT; i++)
        strcpy (attrval[i],"");

    /* Initialize TAE and LAS
       ---------------------*/
    c_inlas(vblock,&std_out);

    /* Get the subcommand
       ------------------*/
    var = p_find(vblock,"_SUBCMD");
    strcpy(subcmd,SVAL(*var,0));

    /* Get the GOF file type
       ---------------------*/
    var = p_find(vblock,"FTYPE");
    if (strcmp(SVAL(*var,0),"POINT") == 0)
        *goftyp = 0;
    else if (strcmp(SVAL(*var,0),"LINE") == 0)
        *goftyp = 1;
    else if (strcmp(SVAL(*var,0),"POLY") == 0)
        *goftyp = 2;
    else 
        *goftyp = 3;

    /* Get the print destination
       -------------------------*/
    strcpy (pfile,"dspgof");
    if (c_setprt(vblock,&term,prtfil,&lp,pfile,ufile) != E_SUCC)
    {
        strcpy(tmpstr[0],"\0");
        c_clean(vblock,"dspgof",tmpstr,&zero);
    }

    /* Get the input image name
       ------------------------*/
    var = p_find(vblock,"INFILE");
    if ((*var).v_count == 0)
        strcpy(in,"");
    else
        strcpy(in,SVAL(*var,0));

    /* If the subcommand is GOF, then get the attributes, the format
       and the record numbers
       -------------------------------------------------------------*/
    if (*subcmd == 'G')
    {
        var = p_find(vblock,"ATRVAL");
        *numattr = (*var).v_count;
        for (i = 0; i < MAXATT; i++)
        {
            if (i < *numattr)
                strcpy(attrval[i],SVAL(*var,i));
            else
                strcpy(attrval[i], "");
        }
        var = p_find(vblock,"RECNUM");
        *numrec = (*var).v_count;
        for (i = 0; i < *numrec; i++)
            recno[i] = IVAL(*var,i);
    }
    else
    {
        *numattr = 0;
    }

    /* If subcommand is not -DEF, get the LISTFLG
       ------------------------------------------*/
    if ((*subcmd == 'G')) /*-GOF,-REC*/
    {
        var = p_find(vblock,"LISTFLG");
        strcpy(prt,SVAL(*var,0));
        *prtpts = (*prt == 'Y') ? TRUE : FALSE;
    }
    return(E_SUCC);
}
