/******************************************************************************
PROGRAM:                   PRINTHEADER

PURPOSE:        Print the information contained in the header record 
                of the GOF.

*******************************************************************************/

#include "las.h"
#include "gof.h"
#include "protos.h"

/* print out information contained in the header GOF record.
   In the case of -DEF this routine completes all processing.
   ----------------------------------------------------------*/
int printheader
(
    char *subcmd,           /* Subcommand specified (I) */
    char *goftyp,           /* GOF type (I) */
    struct HEADER *gofhdr,  /* Pointer to the header record for the GOF (I) */
    FILE *lfp               /* File pointer to the output file (I) */
)
{
    long i,j;             /* loop counters */
    long numattr;         /* number of attributes */
    long slist[MAXATT];   /* index into attribute list */
    long printed = FALSE; /* whether any information was found/printed */


    fprintf(lfp,"\n                   %-10s FILE\n",goftyp);
    fprintf(lfp,"                   ---------------\n");

    /* If -DEF, print out all Attribute names and values in the header record
       -----------------------------------------------------------------------*/
    if (*subcmd == 'D')
    {
        for (numattr = i = 0; i < MAXATT; i++)
            if (gofhdr->mask[i] != 0) slist[numattr++] = i;
        if (numattr != 0) printed = TRUE;

        fprintf(lfp,
                "Attribute Type     Attribute Type     Attribute Type     Attribute Type\n");
        fprintf(lfp,
                "--------- -------- --------- -------- --------- -------- --------- -------\n");

        for ( j = i = 0;i < numattr; i++,j++)
        {
            if (j == 4)
            {
                fprintf(lfp,"\n");
                j = 0;
            }
            fprintf(lfp,"%9s:%-4s ",gofhdr->attrb[slist[i]],
                    gofhdr->attyp[slist[i]]);
            switch(*(gofhdr->attyp[slist[i]]))
            {
                case 'C':
                case 'c':
                    fprintf(lfp,"(1) ");
                    break;
                case 'I':
                case 'i':
                    fprintf(lfp,"(2) ");
                    break;
                case 'R':
                case 'r':
                    fprintf(lfp,"(3) ");
                    break;
            }   /* Switch on attribute type */
        }       /* For each attribute */
        fprintf(lfp,"\n");
    }  /* If -DEF */
    return(printed);
}
