/******************************************************************************
PROGRAM:                   PRINTREC

PURPOSE:        Print the appropriate information from the input GOF
                record.

*******************************************************************************/

#include "las.h"
#include "gof.h"
#include "protos.h"

int printrec
(
    long rec,               /* The current record number (I) */
    long gtype,             /* GOF type (I) */
    union GOF *gofrec,      /* Pointer to the GOF record (I) */
    struct HEADER *gofhdr,  /* Pointer to the GOF header (I) */
    long prtpts,            /* Whether or not to print out the points (I) */
    FILE *lfp               /* File pointer to the output file (I) */
)
{
    long numattr;          /* Loop counter for number of attributes */
    long j,i;              /* Loop counters */
    long len;              /* String length */
    long pos;              /* Annotation string index */
    long slist[MAXATT];    /* Index into attribute list */
    long printed = FALSE;  /* Was the record printed or not */

    float *tempx;
    float *tempy;


    /* If attributes were specified and matched and the format is short,
       set up the attribute index and count for processing
       Otherwise set up the SLIST to index into the attribute list
       -----------------------------------------------------------------*/
    for (numattr = i = 0; i < MAXATT; i++)
    {
        if (gofhdr->mask[i] != 0) 
            slist[numattr++] = i;
    }

    /* If the format is long, something will be printed
       ------------------------------------------------*/
    printed = TRUE;

    /* Print the appropriate information depending on the GOF type
       -----------------------------------------------------------*/
    switch(gtype)
    {
        case ANOT:
            fprintf(lfp,
                    "Record #: %-5d                           Location: (%5.0f,%5.0f)",
                    rec,gofrec->antrec.y,gofrec->antrec.x);
            fprintf(lfp,"\nAttributes:\n");
            for ( j = i = 0;i < numattr; i++,j++)
            {
                if (j == 3)
                {
                    fprintf(lfp,"\n");
                    j = 0;
                }
                fprintf(lfp,"%9s:%-13s   ",gofhdr->attrb[slist[i]],
                        gofrec->antrec.attrb[slist[i]]);
            }

            fprintf(lfp,"\nAnnotation:      Size:%-3d    Angle:%-.2f\n",
                    gofrec->antrec.size,gofrec->antrec.angle);
            if ((len = (long) strlen(gofrec->antrec.antext)) <= 74)
                fprintf(lfp,"Text:%s\n",gofrec->antrec.antext);
            else
            {
                fprintf(lfp,"Text:%-.74s\n",gofrec->antrec.antext);
                for(i = 0 ;((pos = 74 + i * 79) < len); i++)
                    fprintf(lfp,"%-.79s\n",&gofrec->antrec.antext[pos]);
            }
            fprintf(lfp,"\n");
            break;
        case PNT:
            fprintf(lfp,"Record #: %-5d      Location: (%5.0f,%5.0f)         ",
                    rec,gofrec->ptrec.y,gofrec->ptrec.x);
            fprintf(lfp," Label: %-15s\n",gofrec->ptrec.label);
            fprintf(lfp,"Attributes:\n");
            for ( j = i = 0;i < numattr; i++,j++)
            {
                if (j == 3)
                {
                    fprintf(lfp,"\n");
                    j = 0;
                }
                fprintf(lfp,"%9s:%-13s   ",gofhdr->attrb[slist[i]],
                        gofrec->ptrec.attrb[slist[i]]);
            }
            fprintf(lfp,"\n\n");
            break;
        case FPOLIN:
            fprintf(lfp,
                    "Record #: %-5d                                      ",rec);

            fprintf(lfp," Label: %-15s\n",gofrec->plrec.label);
            fprintf(lfp,"Attributes:\n");
            for ( j = i = 0;i < numattr; i++,j++)
            {
                if (j == 3)
                {
                    fprintf(lfp,"\n");
                    j = 0;
                } 
                fprintf(lfp,"%9s:%-13s   ",gofhdr->attrb[slist[i]],
                        gofrec->plrec.attrb[slist[i]]);
            } 
            if (prtpts)
            {
                fprintf(lfp,"\nPoints:\n");
                for(j=i=0; i < gofrec->plrec.numpts; i++,j++)
                {
                    if (j == 4)
                    {
                        fprintf(lfp,"\n");
                        j = 0;
                    } /* if j == 4 */
                    tempx = gofrec->plrec.x;
                    tempy = gofrec->plrec.y;

                    fprintf(lfp,"%4d (%5.0f,%5.0f) ",i + 1,tempy[i],tempx[i]);
                    /*
                       fprintf(lfp,"%4d (%8.2f,%8.2f) ",
                       i + 1,tempy[0],tempx[0]);
                     */
                }  /* For each point */
            }  /* If user requested to print points */
            fprintf(lfp,"\n\n");
            break;
    }  /* Switch on record type */
    return(printed);
}
