/*****************************************************************************
NAME				 DSPGRID 

PURPOSE    Reads a geometric mapping grid file and displays it on the
           user's terminal, the line printer, or to a file


PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   S. Reichardt   11/82        CSC      Original Development
  4.0      R. Mckinney    12/86        EDC      Misc fixes
  5.0      D. Steinwand   10/88        EDC      LAS 5.0 development,
						C conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   DSPGRID must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION
	Initialize LAS processing
	Get the geometric mapping grid file name
	Get the output device parameter
	Read the geometric mapping grid
	Write the geometric mapping grid to the approiate output device
	Print completion message

ALGORITHM REFERENCES		none
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

int main
(void)
{
struct GEOGRID grid;		/* Geometric mapping grid */
struct PARBLK  vblk;		/* TAE variable block */
FILE *filep;			/* File pointer to output file */
long  std_out;			/* Standard output LUN for inlas */
long  mode = IREAD;		/* IREAD flag for getnam routine */
long  fatal = -1;		/* Fatal error flag for c_errmsg */
long  zero = 0;			/* Constant of zero */
long  access = IUPDATE;		/* Access code for output host file */
long  num_out;			/* Number of output files 0 or 1 */
long  term, file, ln_pr;	/* Print option flags */
char  fname[CMLEN];		/* Host name of geometric mapping grid file */
char  temp_file[CMLEN];		/* Temporary file name for output */
char  filename[CMLEN];		/* Partial file name (wo/ext) for complt */
char  out_file[1][CMLEN];		/* Name of output file for complt */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the name of the geometric mapping grid file
  -----------------------------------------------*/
if(c_getnam(&vblk, "INGRID", &zero, fname, "grid", &mode) != E_SUCC)
  c_errmsg("Fatal error encountered","dspgrid-fatal",&fatal);

/* Get the print option.  Make a file name (if neccessary) and open the
   temporary file
  --------------*/
strcpy(temp_file, "dspgrid");
if(c_setprt(&vblk, &term, &file, &ln_pr, temp_file, filename)!=E_SUCC)
  c_errmsg("Fatal error encountered","dspgrid-fatal",&fatal);
if (file == 1)
   {
   c_mknam(filename, out_file[0], "prt", &access);
   num_out = 1;
   }
else
   {
   strcpy(out_file[0], " ");
   num_out= 0;
   }

filep = fopen(temp_file, "w");

/* Get the geometric mapping grid
  ------------------------------*/
if(c_getgrd(fname, &grid) != E_SUCC)
  c_errmsg("Fatal error encountered","dspgrid-fatal",&fatal);

/* Display geometric mapping grid values 
  -------------------------------------*/
if(show_grid(fname, filep, &grid) != E_SUCC)
  c_clean(&vblk, "DSPGRID", out_file, &num_out);

/* Write completion message and exit program
  -----------------------------------------*/
fclose(filep);
c_complt("DSPGRID", out_file, &num_out);

return EXIT_SUCCESS;

}
