#include <stdio.h>
#include <string.h>
#include "las.h"
#include "protos.h"

#define MAXLINE 132

void prtall
(
    FILE *lfp,			/* Pointer to output file. (I)  	      */
    FILE *fp			/* Pointer to session log file. (I)	      */
)

/******************************************************************************
FUNCTION:	PRTALL

PURPOSE: 	Write all records from the input session log file to the 
		output file.	

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90
	    - LAS v5_0 retrocode.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	DSPLOG must be run under TAE.

PROJECT:	LAS

ALGORITHM:

 	Read a record from the session log file.
	While the session log file contains more records
	    Set a flag indicating the current record is to be written.
	    If the current record is not type PA
		If the current record is type PR
		    Write a blank line.
		    Set a flag indicating the record is not to be written.
		Else IF the current record is type PT
		    Set a flag indicating the record is not to be written.
		Else If the current record is type MS
		    If the record is a "proc" statement
		        Set a flag indicating the record is not to be written.
		Else If the current record is type TS
		    Extract the date and time from the record.
		    Set a flag indicating the record is not to be written.
	    Else
		If the record is one of the type to be written. 
		    Print the date and time stamp.
	    If the record is to be written
		Write the record.
	    Read the next record from the session log file.
	Return to the calling routine.

*******************************************************************************/
{
long	pa = FALSE,		/* Indicates whether a PA record has been 
			           read.				      */
	prt;			/* Indicates whether to print the record.     */

char	dashes[61],		/* String of dashes.			      */
	date_time[81],		/* Date and time of current record.	      */
	*getstat,		/* Status value from fgets.		      */
	rec[133];		/* Session log record.			      */


/*
   Set a string of dashes.
*/
strcpy(dashes,"-----------------------------------------------------------");

/*
   Read the first record from the session log.  Note that because fgets returns
   NULL on both end of file and an error, there is no way to distinguish them.
*/
getstat = fgets(rec,MAXLINE,fp);
while (getstat != '\0')
    {
/*
   Set a flag indicating the current record is to be written to the output file.
*/
    prt = TRUE;
/*
   Determine the type of record read and process accordingly.
*/
    if (strncmp(rec,"PA:",3) != 0)
	{
	if (strncmp(rec,"PR:",3) == 0)
	    {
	    fprintf(lfp,"\n");
	    pa = FALSE;
	    prt = FALSE;
	    }
	else if (strncmp(rec,"PT:",3) == 0)
	    prt = FALSE;
	else if (strncmp(rec,"MS:",3) == 0)
	    {
	    if (strncmp(&rec[5],"proc",4) == 0)
	 	prt = FALSE;	
	    }
	else if (strncmp(rec,"TS:",3) == 0)
	    {

/*
   Extract the date and time from the current record.  The date and time stamp
   has different formats on VMS and UNIX systems.
*/
#ifdef vms
	    strncpy(date_time,&rec[4],11);
	    strncpy(&date_time[12],&rec[16],8);
#else
	    strncpy(date_time,&rec[4],7);
	    date_time[7] = '\0';
	    strncat(date_time,&rec[20],4);
	    date_time[11] = ' ';
	    date_time[12] = '\0';
	    strncat(date_time,&rec[11],8);
#endif
	    date_time[20] = '\0';
	    strcat(date_time,dashes);
	    prt = FALSE;
	    }
	}
    else
	{
	if ((rec[4] == '"') && (!pa))
	    {
/*
   Print the date and time stamp.
*/
	    fprintf (lfp,"\n");
	    fprintf (lfp,"%s\n",date_time);
	    fprintf (lfp,"\n");
	    pa = TRUE;
	    }
	}
    if (prt)
/*
   Print the current record.
*/
	fprintf(lfp,"%s",&rec[4]);
/*
   Read the next record.
*/
    getstat = fgets(rec,MAXLINE,fp);
    }

return;
}
