#include <stdio.h>
#include <string.h>
#include "las.h"
#include "typlim.h"
#include "util.h"
#include "protos.h"

#define MAXLINE 132

int prtsub
(
    FILE *lfp,			/* Pointer to output file. (I)  	      */
    FILE *fp,			/* Pointer to session log file. (I)	      */
    long bdate[],		/* Beginning month, day and year. (I)	      */
    long edate[],		/* Ending month, day and year. (I)	      */
    long btime[],		/* Beginning hours, minutes and seconds. (I)  */
    long etime[]		/* Ending hours, minutes and seconds. (I)     */
)
/******************************************************************************
FUNCTION:	PRTSUB

PURPOSE: 	Write selected records based on the input dates and times
		to the output file.	

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90
	    - LAS v5_0 retrocode.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	DSPLOG must be run under TAE.

PROJECT:	LAS

ALGORITHM:

	Convert starting and ending dates and times to total seconds.
 	Read a record from the session log file.
	While the session log file contains more records
	    Set a flag indicating the current record is to be written.
	    If the current record is not type PA
		If the current record is type PR and it meets the date and
		  time requirements
		    Write a blank line.
		    Set a flag indicating the record is not to be written.
		Else If the current record is type PT and it meets the date
		  and time requirements
		    Set a flag indicating the record is not to be written.
		Else If the current record is type MS and it meets the date
		  and time requirements
		    If the record is a "proc" statement
		        Set a flag indicating the record is not to be written.
		Else If the current record is type TS
		    Extract the date and time from the record.
		    If neither a beginning or ending time was input
			Set the time values to the default (999).
		    Calculate the 2-digit year and numeric month.
   		    If the beginning date was not specified but the beginning 
		      time was
   			Calculate the current record total seconds based on 
			  the default date.
			Determine whether the current total seconds is greater
			  than or equal to the starting total seconds.
			Recalculate the total seconds using the actual date of 
			  the record.
		    Else
			Calculate the current record total seconds.
			Determine whether the current total seconds is greater
			  than or equal to the starting total seconds.
   		    If the ending date was not specified but the ending time 
		      was
			Calculate the current record total seconds based on 
			    the default date. 
		    Determine whether the current total seconds is less than 
		      or equal to the ending total seconds.
		    Set a flag indicating the record is not to be written.
	    Else
		If the record meets all criteria
		    Print the date and time stamp.
	    If the record is to be written and meets all criteria
		Write the record.
	    Read the next record from the session log file.
	Return to the calling routine.

*******************************************************************************/
{
static char calendar[37] = {"janfebmaraprmayjunjulaugsepoctnovdec"};
				/* Months of the year.			      */

long	current,		/* Total number of seconds of current record. */
	day,			/* Day of current record.		      */
	deflt = 999,		/* Default value.			      */
	end,			/* Ending total number of seconds.	      */
	found = FALSE,		/* Indicates whether any records were found to
				   be printed.				      */
	hour,			/* Hour of current record.		      */
	in = FALSE,		/* Indicates whether current record is within
				   the specified range.			      */
	minute,			/* Minute of current record.		      */
	month,			/* Month of current record.		      */
	pa = FALSE,		/* Indicates whether a PA record has been 
			           read.				      */
	prt,			/* Indicates whether to print the record.     */
	sec,			/* Seconds of current record.		      */
	start,			/* Starting total number of seconds.	      */
	three = 3,		/* Constant three.			      */
	year;			/* Year of current record.		      */

char	cmonth[4],		/* Month (3 character abbreviation).	      */
	dashes[61],		/* String of dashes.			      */
	date_time[81],		/* Date and time of current record.	      */
	*getstat,		/* Status value from fgets.		      */
	rec[133];		/* Session log record.			      */

int pos;            /* index into character string.          */
/*
   Set a string of dashes.
*/
strcpy(dashes,"-----------------------------------------------------------");

/* 
   Convert starting and ending dates and times to total seconds.
*/
if ((bdate[0] == 999) && (btime[0] == 999))
    start = MINLONG;
else
    totsec(&bdate[2],&bdate[0],&bdate[1],&btime[0],&btime[1],&btime[2],&start);

if ((edate[0] == 999) && (etime[0] == 999))
    end = MAXLONG;
else 
    totsec(&edate[2],&edate[0],&edate[1],&etime[0],&etime[1],&etime[2],&end);

/*
   Read the first record from the session log.  Note that because fgets returns
   NULL on both end of file and an error, there is no way to distinguish them.
*/
getstat = fgets(rec,MAXLINE,fp);
while (getstat != '\0')
    {
/*
   Set a flag indicating the current record is to be written to the output file.
*/
    prt = TRUE;
/*
   Determine the type of record read and process accordingly.
*/
    if (strncmp(rec,"PA:",3) != 0)
	{
	if ((strncmp(rec,"PR:",3) == 0) && in)
	    {
	    fprintf(lfp,"\n");
	    pa = FALSE;
	    prt = FALSE;
	    }
	else if ((strncmp(rec,"PT:",3) == 0) && in)
	    prt = FALSE;
	else if ((strncmp(rec,"MS:",3) == 0) && in)
	    {
	    if (strncmp(&rec[5],"proc",4) == 0)
	 	prt = FALSE;	
	    }
	else if (strncmp(rec,"TS:",3) == 0)
	    {
/*
   Extract the date and time from the current record.  The date and time stamp 
   has different formats on VMS and UNIX systems.
*/
#ifdef vms
	    strncpy(date_time,&rec[4],20);
	    date_time[20] = '\0';
	    if ((bdate[0] == 999) && (edate[0] == 999))
/*
   Ignore the date.
*/
		day = year = month = 999;
	    else
	        sscanf(date_time,"%d-%3s-%d",&day,cmonth,&year);
#else
	    strncpy(date_time,&rec[4],7);
	    date_time[7] = '\0';
	    strncat(date_time,&rec[20],4);
	    date_time[11] = ' ';
	    date_time[12] = '\0';
	    strncat(date_time,&rec[11],8);
	    date_time[20] = '\0';
	    if ((bdate[0] == 999) && (edate[0] == 999))
/*
   Ignore the date.
*/
		day = year = month = 999;
	    else
	        sscanf(date_time,"%s %d %d",cmonth,&day,&year);
#endif
/*
   Calculate the total number of seconds for the current record.
*/
	    if ((btime[0] == 999) && (etime[0] == 999))
/*
   Ignore the time.
*/
		hour = minute = sec = 999;
	    else
	        sscanf(&date_time[12],"%d:%d:%d",&hour,&minute,&sec);
/*
   Determine the 2-digit year and the numeric month.
*/
	    if (year != 999)
		{
	        year = year % 100;
	        c_up2low(cmonth,&three);
	        c_index(calendar,cmonth,&pos);
	        month = (long) (((pos - 1) + 3) / 3.0);
		}
/*
   If the beginning date was not specified but the beginning time was,
   calculate the current record total seconds based on the default date 
   to check against the starting seconds then recalculate the total
   seconds using the actual date of the record.  
*/
	    if ((bdate[0] == 999) && (btime[0] != 999))
		{
	        totsec(&deflt,&deflt,&deflt,&hour,&minute,&sec,&current);
	        if (current >= start) 
		    in = TRUE;
	        else 
		    in = FALSE;
	        totsec(&year,&month,&day,&hour,&minute,&sec,&current);
		}
	    else
		{
	        totsec(&year,&month,&day,&hour,&minute,&sec,&current);
	        if (current >= start) 
		    in = TRUE;
	        else 
		    in = FALSE;
		}
/*
   If the ending date was not specified but the ending time was, calculate the 
   current record total seconds based on the default date to check against the 
   ending seconds.
*/
	    if ((edate[0] == 999) && (etime[0] != 999))
	        totsec(&deflt,&deflt,&deflt,&hour,&minute,&sec,&current);
	    if (in && (current > end))
		in = FALSE;
	    strcat(date_time,dashes);
	    prt = FALSE;
	    }
	}
    else
	{
	if ((rec[4] == '"') && (!pa) && in)
	    {
/*
   Print the date and time stamp.
*/
	    fprintf (lfp,"\n");
	    fprintf (lfp,"%s\n",date_time);
	    fprintf (lfp,"\n");
	    pa = TRUE;
	    }
	}
    if (prt && in)
/*
   Print the current record.
*/
	{
	fprintf(lfp,"%s",&rec[4]);
	found = TRUE;
	}
/*
   Read the next record.
*/
    getstat = fgets(rec,MAXLINE,fp);
    }

if (found)
    return(E_SUCC);
else
    return(E_FAIL);
}
