#include "las.h"
#include "protos.h"

#define STARTYR	1986

void totsec
(
    long *year,		/* Two-digit year (I).				*/
    long *month,		/* Month (I).					*/
    long *day,		/* Day (I).					*/
    long *hour,		/* Hour (based on 24-hr clock) (I).		*/
    long *minute,	/* Minute (I).					*/
    long *sec,		/* Seconds (I).					*/
    long *tot_sec	/* Total seconds since Jan. 1, 1986 (O).	*/
)
/******************************************************************************
FUNCTION:	TOTSEC

PURPOSE: 	Calculate the total number of seconds since Jan. 1, 1986
		given the input date and time.  This formula is valid for
		Jan. 1, 1986 - Dec. 31, 2085.

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90
	    - LAS v5_0 retrocode.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	DSPLOG must be run under TAE.

PROJECT:	LAS

ALGORITHM:

	If no date was input
	    Set the date to Jan. 1, 1986.
	If no time was input
	    Set the time to 00:00:00.
	Convert the input date to a julian date taking into account leap years.
	Calculate the total number of seconds.  This formula factors in the
	  turn of the century.

*******************************************************************************/
{
static long dtbmth[24] = {0,31,59,90,120,151,181,212,243,273,304,334,
		          0,31,60,91,121,152,182,213,244,274,305,335};
			/* Julian day the first of each month (both
			   non-leap years and leap years.		*/

long	century,	/* Century mark.				*/
	dy,		/* Day.						*/
	hr,		/* Hour.					*/
	julian,		/* Juliand day.					*/
	leap = 0,		/* Indicates whether the input year is a leap 
			   year.					*/
	leapyr = 72,	/* Starting leap year.				*/
	mon,		/* Month.					*/
	mt,		/* Minute.					*/
	sc,		/* Second.				 	*/
	yr;		/* Year.					*/

/*
   Set the date to Jan. 1, 1986 if no date was input.
*/
if (*year == 999)
    {
    yr = STARTYR % 100;
    mon = dy = 1; 
    }
else
    {
    yr = *year;
    mon = *month;
    dy = *day;
    }
/*
   Set the time to 00:00:00 if no time was input.
*/
if (*hour == 999)
    hr = mt = sc = 0;
else
    {
    hr = *hour;
    mt = *minute;
    sc = *sec;
    }
/*
   Convert the input date to a julian date (factor in leap years).
*/
if (leapyr == yr)
    leap = 1;

while (leapyr < yr)
    {
    leapyr = leapyr + 4;
    if (leapyr == yr)
        leap = TRUE;
    else if (leapyr > yr)
	{
        leap = FALSE;
	break;
	}
    }

if (leap) 
    mon = mon + 12;
julian = dtbmth[mon-1] + dy;

/*
   Calculate the total seconds for the input date and time. 
*/
if (yr < (STARTYR % 100))
    century = 100;
else
    century = 0;

*tot_sec = sc + (mt * 60) + (hr * 3600) + ((julian - 1) * 86400) +
    (long)(((yr - (STARTYR % 100) + century) * 365.25) + 0.25) * 86400;

return;
}
