/*****************************************************************************
NAME				 DSPRWT 

PURPOSE    Reads a resampling weight rwt file and displays it on the
           user's terminal, the line printer, or to a file


PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand    2/89        EDC      Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   DSPRWT must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION

	Initialize LAS processing
	Get the resampling weight table file name
	Get the output device parameter
	Read the resampling weight table
	Write the resampling weight table to the approiate output device
	Print completion message
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

int main
(void)
{
struct RWTAB table;		/* Resampling weight table */
struct PARBLK  vblk;		/* TAE variable block */
FILE *filep;			/* File pointer to output file */
long  std_out;			/* Standard output LUN for inlas */
long  mode = IREAD;		/* IREAD flag for getnam routine */
long  fatal = -1;		/* Fatal error flag for c_errmsg */
long  zero = 0;			/* Constant of zero */
long  access = IUPDATE;		/* Access code for output host file */
long  num_out;			/* Number of output files 0 or 1 */
long  term, file, ln_pr;	/* Print option flags */
char  fname[CMLEN];		/* Host name of resampling weight table file */
char  temp_file[CMLEN];		/* Temporary file name for output */
char  filename[CMLEN];		/* Partial file name (wo/ext) for complt */
char  out_file[CMLEN];		/* Name of output file for complt */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the name of the resampling weight table file
  -----------------------------------------------*/
if(c_getnam(&vblk, "INRWT", &zero, fname, "rwt", &mode) != E_SUCC)
  c_errmsg("Fatal error encountered","dsprwt-fatal",&fatal);

/* Get the print option.  Make a file name (if neccessary) and open the
   temporary file
  --------------*/
strcpy(temp_file, "dsprwt");
if(c_setprt(&vblk, &term, &file, &ln_pr, temp_file, filename) != E_SUCC)
  c_errmsg("Fatal error encountered","dsprwt-fatal",&fatal);
if (file == 1)
   {
   c_mknam(filename, out_file, "prt", &access);
   num_out = 1;
   }
else
   {
   strcpy(out_file, " ");
   num_out= 0;
   }

filep = fopen(temp_file, "w");

/* Get the resampling weight table
  ------------------------------*/
if(c_getrwt(fname, &table) != E_SUCC)
  c_errmsg("Fatal error encountered","dsprwt-fatal",&fatal);

/* Display resampling weight table values 
  -------------------------------------*/
show_rwt(fname, filep, &table);

/* Write completion message and exit program
  -----------------------------------------*/
fclose(filep);
c_complt("DSPRWT", out_file, &num_out);

return EXIT_SUCCESS;

}
