/*****************************************************************************
NAME				 show_rwt

PURPOSE    Writes a resampling weight table to a user defined output device.
	   User's terminal, an ASCII file, or the line printer are valid
	   output devices.


PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand    2/89        EDC      Original development

ALGORITHM DESCRIPTION
	Print the name of the resampling weight table file
	Print the each weight table in tabular form
	Return
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

void show_rwt
(
    char *fname,			/* TAE name of print file */
    FILE *filep,			/* File pointer for output */
    struct RWTAB *table		/* Resampling weight table */
)
{
long i, j;			/* Loop counters */

/* Output resampling weight table file name
  ---------------------------------------*/
fprintf(filep,"Resampling weight table file name:\n");
fprintf(filep,"  %s\n\n",fname);

/* Print first table
  -----------------*/
fprintf(filep, "First table dimension:   %d\n\n",table->dim[0]);

for (i = 0; i < 33; i++)
   {
   for (j = 0; j < table->dim[0]; j++)
	fprintf(filep, " %7.4f", table->table1[i][j]);
   fprintf(filep, "\n");
   }

/* Print second table
  ------------------*/
if (table->dim[1] == 0) return;

fprintf(filep, "\f\nSecond table dimension:   %d\n\n",table->dim[1]);

for (i = 0; i < 33; i++, fprintf(filep,"\n"))
   for (j = 0; j < table->dim[1]; j++)
	fprintf(filep, " %7.4f", table->table2[i][j]);

/* Print third table
  -----------------*/
if (table->dim[2] == 0) return;

fprintf(filep, "\f\nThird table dimension:   %d\n\n",table->dim[2]);

for (i = 0; i < 33; i++, fprintf(filep,"\n"))
   for (j = 0; j < table->dim[2]; j++)
	fprintf(filep, " %7.4f", table->table3[i][j]);

return;
}
