/*****************************************************************************

NAME:				EDGEX	

PURPOSE:  Extracts edges from an image

PROGRAM HISTORY:
VERSION		DATE	AUTHOR	      CODE/CONT 
-------		----	------	      ---------
  1.0		4/93	D. Steinwand
  1.1		8/94    D. Eichacker	CSB	Updated to LAS standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
EDGEX must be run under TAE.
EDGEX only works on BYTE images.

*****************************************************************************/
#include "las.h"
#include <stdio.h>
#include <stdlib.h>
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK vblock;           /* TAE parameter block                       */

static long bands[MAXIMG][MAXBND+1];/* bands to be processed - all images    */


long 	fatal = -1;		/* fatal error flag		     	     */
long    msgtim;                 /* elapsed time between display msg  	     */
long    nbands[MAXIMG];         /* number of bands per input image   	     */
long    nmrimg;                 /* number of input images            	     */
long    odtype;                 /* data type of output image         	     */
long    one = 1;                /* value of one                      	     */
long    pixval;			/* Edge pixel value                    	     */
long    status;                 /* function return status code       	     */
long    std_out;                /* LUN to standard output            	     */
long 	threshold;		/* Upper edge limit                  	     */
long    totbnd;                 /* total number of bands             	     */
long    window[MAXIMG][4];      /* window for each input image       	     */

char    hosin[MAXIMG][CMLEN];   /* host names of input images        	     */
char    hosout[1][CMLEN];       /* host name of output image         	     */
char    taein[MAXIMG][CMLEN];   /* TAE names of input images         	     */
char    taeout[1][CMLEN];       /* TAE name of output image          	     */

float   efract;                 /* Fraction of pixels classifed as edges   */

/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);

/* Get parameters and globals
-----------------------------*/
status = getpar(&vblock,taein,hosin,taeout,hosout,&nmrimg,window,bands
       ,nbands, &totbnd,&odtype,&msgtim,&threshold,&efract,&pixval);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","edgex-fatal",&fatal);

/* Set up the input & output images and do the actual processing
----------------------------------------------------------------*/
status = proces (&vblock,hosin,hosout,nmrimg,window,bands,nbands,
                 totbnd,odtype,threshold,efract,pixval,msgtim);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","edgex-fatal",&fatal);

/*  Update history file
-----------------------*/
if (c_hstout(&vblock,hosout[0],hosin,&nmrimg," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","edgex-warn",&one);


/* Close down processing and exit
  ------------------------------*/
c_complt("EDGEX",taeout,&one);

return EXIT_SUCCESS;

}
