/******************************************************************************
NAME:                           GETPAR

ALGORITHM:
   Retrieve the required parameters and globals from the TAE parameter block
   Parse the input image specification by calling GETIN which will validate
    the window and band specifications
   Checking to ensure that all the input images have equal window sizes
   Retreive the output image name
   Retreive the output data type
   Retreive the global parameters
   Return
******************************************************************************/
#include "las.h"
#include <stdio.h>
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,       /* pointer to TAE parameter block            */
    char taein[][CMLEN],         /* TAE file names of input images            */
    char hosin[][CMLEN],         /* host file names of input images           */
    char taeout[][CMLEN],        /* TAE file name of output image             */
    char hosout[][CMLEN],        /* host file name of output image            */
    long *nmrimg,                /* number of input images                    */
    long (*window)[4],           /* window specifications for each input image*/
    long (*bands)[MAXBND+1],     /* band specifications for each input image  */
    long *nbands,                /* number of bands per input image           */
    long *totbnd,                /* total number of bands to be processed     */
    long *odtype,                /* output data type                          */
    long *msgtim,                /* elapsed time between display messages     */
    long *threshold,             /* Upper edge limit                          */
    float *efract,               /* Fraction of pixels classified as edges    */
    long *pixval 		/* Edge pixel value                          */
)

{
struct VARIABLE *val;           /* pointer to a TAE variable within v-block  */

long    element;                /* parameter element number                  */
long    fatal = -1;             /* fatal error flag                          */
long    imglim;                 /* maximum number of images to be processed  */
long    qflag[MAXIMG][2];       /* flag indicating if subwindows specified   */
long    same;                   /* flag indicating if images have same window*/
long    status;                 /* function return status code               */


/* Retrieve and parse input image specification
-----------------------------------------------*/
element = 0;
imglim = 1;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,nbands,
                totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","edgex-fatal",&fatal);
if (window[0][2] < 3 || window[0][3] < 3)
   c_errmsg("Windowed image size is smaller than 3x3","edgex-wind",
            &fatal);

/* Retrieve output image name
-----------------------------*/
status = c_getout(vblock,"OUT",taeout,hosout);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","edgex-fatal",&fatal);

/* output data type
----------------------------*/
*odtype = EBYTE;

val = p_find(vblock,"THRVAL");
if(val->v_count == 0)   /* if THRVAL == NULL */
  *threshold = 256;
else			/* THRVAL was specified */
  *threshold = IVAL(*val,0);

val = p_find(vblock,"EDGEVAL");
*efract = RVAL(*val,0);

val = p_find(vblock,"PIXVAL");
*pixval = IVAL(*val,0);

/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);

return(E_SUCC);
}

