/******************************************************************************
FUNCTION:	UPDATE_DDR

PURPOSE:	Updates the DDR information for an image


PROGRAMMER	  DATE		REASON
----------	  ----		------
A. Weissenberger  07/01         Removed all ifdef's 
******************************************************************************/



#include "las.h"
#include "tae.h"
#include "desc.h"
#include "protos.h"

int updateddr_
(
    char *hname,		/* Host file name			*/
    long window[],		/* Input window				*/
    long *totbnd,		/* Total number of bands to process	*/
    long *linc,		        /* Line increment factor		*/
    long *sinc,		        /* Sample increment factor		*/
    char *stype,		/* sensor type (MSS or RBV)		*/
    char *direction,	        /* Direction of satellite "D" or "A"	*/
    long *path,		        /* WRS path				*/
    long *row,		        /* WRS row				*/
    double wrsll[],		/* WRS center latitude/longitude	*/
    double wrsls[],		/* WRS center line/sample		*/
    double *angle,		/* Image orientation angle		*/
    char *edate,		/* Date of exposure			*/
    char *projec,		/* Projection type			*/
    char *scene,		/* Scene id (contains time of exposure) */
    long *ddrvflag,		/* DDR validity flag			*/
    long *corect,		/* MSS corrected data flag		*/
    long hname_len,		/* Length of the Host file name		*/
    long stype_len,		/* Length of the Host file name		*/
    long direc_len,		/* Length of the Host file name		*/
    long edate_len,		/* Length of the Host file name		*/
    long proj_len,		/* Length of the Host file name		*/
    long scene_len		/* Length of the Host file name		*/
)
{
    char *hname_ptr;
    char *proj_ptr;
    char *scene_ptr;
    char *edate_ptr;
    char *direc_ptr;
    char *stype_ptr;
    long status;


    hname_ptr = squeeze(hname,hname_len);
    direc_ptr = squeeze(direction,direc_len);
    proj_ptr  = squeeze(projec,proj_len);
    stype_ptr = squeeze(stype,stype_len);
    edate_ptr = squeeze(edate,edate_len);
    scene_ptr = squeeze(scene,scene_len);

    status = update_ddr(hname_ptr,window,totbnd,linc,sinc,stype_ptr,direc_ptr,
                        path,row,wrsll,wrsls,edate_ptr,proj_ptr,scene_ptr,
                        ddrvflag);

    free(hname_ptr);
    free(proj_ptr);
    free(scene_ptr);
    free(edate_ptr);
    free(direc_ptr);
    free(stype_ptr);

    return(status);
}
