
C     ******************************************************************
C     PURPOSE: BSQMSS processes a band with MSS image data.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     btot	       total number of output bands
C     fill	       fill value
C     inbnd	       current input band number
C     msgtim	       elapsed time between display messages
C     nl	       number of lines in input image(s)
C     ns	       number of samples per line in input image(s)
C     unit     	       logical unit number of the tape drive
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     eclose	       imageio function to close an image file
C     eread	       imageio function to read an image line
C     i	    	       loop index
C     image(3596)      output image buffer
C     inbuf(3596)      input image buffer
C     lin	       indicates where to begin extracting lines when
C		       the image is larger than MSS size
C     nol	       number of lines
C     nos	       number of samples
C     one              value of one
C     pix	       indicates where to begin extracting pixels when
C		       the image is larger than MSS size
C     pxcopy           function to copy from one buffer to another
C     pxswap	       byte swap a data buffer
C     reclen	       record length
C     record	       current record number
C     status	       function return status code
C     stlin	       indicates how many fill lines need to be inserted
C     stpix	       indicates how many fill pixels need to be inserted
C     tpwrit	       tapeio function to write to tape
C     tpweof	       tapeio function to write EOF mark(s) to tape
C     zero	       value of zero
C
C	Algorithm:  BSQMSS first fills the image with zeros.
C		    Next the processing message is initalized.
C		    Image records are then written to tape.
C		    Processing continues until all images have
C		    been written in a BSQ format.
C     *****************************************************************

      subroutine bsqmss (btot,nl,ns,unit,fill,msgtim,inbnd)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4            
     .                     btot,
     .			   eclose,
     .			   eread,
     .                     fill,
     .                     i,
     .			   inbnd,
     .			   lin,
     .			   msgtim,
     .			   nl,
     .                     nol,
     .                     nos,
     .			   ns,
     .			   one,
     .                     pix,
     .			   pxcopy
      integer*4
     .                     reclen,
     .                     record,
     .                     rfdesc,
     .                     status,
     .                     stlin,
     .                     stpix,
     .                     tpwrit,
     .                     tpweof,
     .                     unit

      character*1	   fillvl,
     .			   image(3596),
     .			   inbuf(3596),
     .			   v127,
     .			   zero

      common/randio/rfdesc
      common/curent/i

      data reclen/3596/

      one = 1

      zero = char(0)
      v127 = char(127)
      fillvl = char(fill)

C     Initialization
C     --------------
      call pxfill(image,EBYTE,reclen,zero)
      stpix = 0
      stlin = 0
      pix = 0
      lin = 0

      if (nl .gt. 2983) then
        lin = (nl - 2983)/2
        nol = 2983
      else
        stlin = (2983 - nl)/2
        nol = nl
      endif

      if (ns .gt. 3548) then
        pix = (ns - 3548)/2
        nos = 3548
      else
        stpix = (3548 - ns)/2
        nos = ns
      endif

C     Set the record type to 237
C     --------------------------

      image(6) = char(237)

C     Fill the image portion of the record with the fill value
C     --------------------------------------------------------
      call pxfill(image(13),EBYTE,3548,fillvl)

C     Initialize and start the display message utility
C     ------------------------------------------------
      i = 1
      call inittm('EDIPSOUT','Processed','line','band',' ',2983,
     +	           btot,msgtim,ONECAT)
      call startm

C     Write the proper number of fill lines to center the image
C     ---------------------------------------------------------
      do 100 i = 1,stlin

C       Set the record number
C       ---------------------
        record = i

        call btcpy(record,image(1),4)

C       Write the fill record to tape
C       -----------------------------
        status = tpwrit (unit,image,reclen,HANDLE)
        if (status .ne. reclen) then
	   status = eclose(rfdesc)
	   call fclean(unit)
	endif
100   continue

C     Write out the actual image lines
C     --------------------------------

      do 400 i = stlin+1,nol+stlin
C       Set the record number
C       ---------------------
        record = i

	call btcpy(record,image(1),4)

C       Read in the image line
C       ----------------------
        status = eread(rfdesc,inbnd,i-stlin+lin,inbuf,one)
        if (status .lt. ESUCC) then
	    status = eclose(rfdesc)
	    call fclean(unit)
	endif

C       Copy the image data from the input buffer to the output buffer
C       --------------------------------------------------------------
        status = pxcopy(inbuf(1 + pix),
     +			image(13 + stpix),EBYTE,nos)

C       Fill the end of the record with 127
C       -----------------------------------
        call pxfill(image(3561),EBYTE,36,v127)

C       Write the image record to tape
C       ------------------------------
        status = tpwrit (unit,image,reclen,HANDLE)
	if (status .ne. reclen) then
	   status = eclose(rfdesc)
	   call fclean(unit)
	endif
400   continue

C     Initialize the image record
C     ---------------------------
      call pxfill(image,EBYTE,reclen,zero)

C     Set the record type to 237
C     --------------------------

      image(6) = char(237)

C     Fill the image portion of the record with the fill value
C     --------------------------------------------------------
      call pxfill(image(13),EBYTE,3548,fillvl)

C     Write the remaining fill lines
C     ------------------------------
      do 500 i = stlin+nol+1,2983

C       Set the record number
C       ---------------------
        record = i

        call btcpy(record,image(1),4)

C       Write the fill record to tape
C       -----------------------------
        status = tpwrit (unit,image,reclen,HANDLE)
	if (status .ne. reclen) then
	   status = eclose(rfdesc)
	   call fclean(unit)
	endif
500   continue

C     Disable the display message utility
C     -----------------------------------
      call stoptm

C     Write the EOF mark
C     ------------------
      status = tpweof (unit,one,HANDLE)
      if (status .ne. TSUCC) then
	 status = eclose(rfdesc)
	 call fclean(unit)
      endif

      return
      end
