
C     *****************************************************************
C     PURPOSE: GETPAR receives the user-specified parameters.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     annot            user-specified annotation record
C     bands(MAXBND+1,MAXIMG)
C		       input band specification
C     bform	       band format indicator
C     bndnum(4)	       output band numbers
C     btot	       total number of output bands
C     dens	       output tape density
C     fill	       fill value
C     hosin(MAXIMG)    input host names
C     msgtim	       elapsed time between display messages
C     nband(MAXIMG)    number of bands in each input image
C     nmrimg	       number of input images
C     parblk(XPRDIM)   parameter block
C     path	       nominal path number
C     prefix	       first portion of associated file names
C     row	       nominal row number
C     stype	       sensor type
C     tlabel	       mount message for first volume
C     window(4,MAXIMG)
C		       input window specification     
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     annot1(4)	       user specified annotation
C     count	       number of values specified for a parameter
C     getin	       function to parse input image specification
C     i		       loop index
C     j	  	       loop index
C     len(4)	       length specification
C     length	       length specification
C     qflag(2,MAXIMG)  flag indicating if a subwindow was specified
C     same	       flag indicating if images have same window sizes
C     status	       function return status code
C     subnam	       specified subcommand name
C     taein(MAXIMG)    input TAE names 
C
C	ALGORITHM:  Retrieve the TAE global MSGTIM
C                   Retrieve the IN, COMMENT, and TAPEDENS parameters
C                   Retrieve WRSPATHN, WRSROWN, annotation, bands 
C                   and fill values
C                   
C     *****************************************************************

      subroutine getpar (parblk,msgtim,prefix,hosin,nmrimg,btot,
     +  		 window,bands,nband,dens,path,row,
     +		         annot,bndnum,fill,bform,stype,tlabel)

	include 'las.fin'

      integer*4            
     .     	           bands(MAXBND+1,MAXIMG),
     .                     bndnum(4),
     .			   btot,
     .                     count,
     .			   elemen,
     .       	           getin,
     .                     fill,
     .			   i,
     .			   j,
     .                     len(4),
     .                     length,
     .			   msgtim,
     .		           nband (MAXIMG),
     .			   nmrimg
      integer*4
     .			   parblk(XPRDIM),
     .                     qflag(2,MAXIMG),
     .			   same,
     .                     status,
     .                     window(4,MAXIMG)

      character*112        annot

      character*(CMLEN)        
     .                     hosin(MAXIMG),
     .                     prefix,
     .                     subnam,
     .			   taein(MAXIMG)

      character*80         tlabel

      character*28         annot1(4)

      character*11         bform
      character*5          dens
 
      character*3
     .			   path,
     .			   row,
     .			   stype
 

C     Get TAE global variable "$msgtim"
C     ---------------------------------
      call xrintg(parblk,'$msgtim',1,msgtim,count,status)

C     Get input file specification (IN)
C     ---------------------------------
      elemen = 1
      status = getin(parblk,'IN',elemen,taein,hosin,nmrimg,window,
     +	             bands,nband,btot,MAXIMG,same,qflag)
      if (status .ne. ESUCC) then
         call errmsg
     +   ('Fatal error encountered--Output tape is not valid',
     +	  'edipsout-fatal',-1)
      endif

      if (same .ne. 1) then
         call errmsg
     +   ('Input images do not have the same window sizes',
     +    'edipsout-window',1)
         call errmsg
     +   ('Fatal error encountered--Output tape is not valid',
     +    'edipsout-fatal',-1)
      endif

C     -----------------------------------------------------------------
C     |If more than four bands were specified, print a warning message|
C     |and adjust the necessary variables so that only the first four |
C     |bands are processed					      |
C     -----------------------------------------------------------------
      if (btot .gt. 4) then
        call errmsg('More than four bands were specified',
     +	 	    'edipsout-extra',1)
        call errmsg
     +  ('Only the first four bands are being processed',
     +	 'edipsout-extra',1)

        btot = 0
        do 20 i = 1,nmrimg
           j = 1
10         continue
           if (bands(j,i) .ne. 0) then
              j = j + 1
	      btot = btot +1
              if (btot .eq. 4) goto 15
              goto 10
	   endif
           nband(i) = j - 1
           goto 20

15         nmrimg = i
           nband(i) = j - 1
           goto 25

20       continue
25       continue

      endif
         
C     Set prefix for associated file names
C     ------------------------------------
      prefix = hosin(1)

C     Get description of the tape (COMMENT)
C     -------------------------------------
      call xrstr(parblk,'COMMENT',1,tlabel,length,count,status)

C     Get output tape density  (DENSITY)
C     ----------------------------------
      call xrstr (parblk,'TAPEDENS',1,dens,length,count,status)

C     Null terminate the string
C     -------------------------
      dens((length+ 1):(length+ 1)) = char(0)

C     Get nominal path number (PATH)
C     ------------------------------
      call xrstr (parblk,'WRSPATHN',1,path,length,count,status)
      if (count .eq. 0) path = ' '

C     Get nominal row number (ROW)
C     ----------------------------
      call xrstr (parblk,'WRSROWN',1,row,length,count,status)
      if (count .eq. 0) row = ' '

C     Get annotation text (ANNOT)
C     ---------------------------
      annot1(1) = ' '
      annot1(2) = ' '
      annot1(3) = ' '
      annot1(4) = ' '
      call xrstr (parblk,'ANNOTE',4,annot1,len,count,status)
      annot = annot1(1) // annot1(2) // annot1(3) // annot1(4)

C     Get band numbers for output image (BANDS)
C     ------------------------------------------
      call xrintg (parblk,'BANDS',4,bndnum,count,status)

C     Get fill value  (FILL)
C     ----------------------
      call xrintg (parblk,'FILLVAL',1,fill,count,status)

C     Determine which subcommand was specified
C     ----------------------------------------
      call xrstr (parblk,'_SUBCMD',1,subnam,len,count,status)
      if (subnam .eq. 'BSQ') then
        bform = 'SEQUENTIAL'
        call xrstr (parblk,'SENSOR',1,stype,length,count,status)
      else
        bform = 'INTERLEAVED'
        stype = 'MSS'
      endif

C     If BIL format was specificed for a single band, print a warning
C     message and change the format indicator to BSQ
C     ---------------------------------------------------------------
      if ((btot .eq. 1) .and. (bform .eq. 'INTERLEAVED')) then
        call errmsg 
     +  ('Image is being processed in band sequential format',
     +   'edipsout-note',1)
        bform = 'SEQUENTIAL'
      endif 

C     Reset indicators for RBV 
C     ------------------------
      if (stype .eq. 'RBV') then
         nmrimg = 1
         btot = 1
         nband(1) = 1
      endif

      return
      end
