
C     *********************************************************************
C     PURPOSE: NXTVOL is used to mount the second tape in a two-volume set.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     dens	       output tape density
C     mstat	       mount status indicator
C     unit	       logical unit number of the tape drive
C     volnum           current output volume number
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     mesg             second volume mount message for user
C     opmesg	       second volume mount message for operator
C     skip	       number of files to skip
C     status           function return status code
C     tpclos	       tapeio function to close a tape device
C     tpopen           tapeio function to open a tape device
C
C     ALGORITHM:
C                   Close the current volume.
C		    Request the operator to mount a tape for second volume
C		    The second volume is opened for processing.
C         
C     ********************************************************************

      subroutine nxtvol (dens,unit,volnum,mstat)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4            
     .   		   mstat,
     . 			   skip,
     .                     status,
     .                     tpclos,
     .                     tpopen,
     .                     unit,
     .                     volnum

      character*(CMLEN)
     .			   mesg,
     .			   opmesg,
     .                     tapeid
      character*5          dens

      data mesg/' Please mount the second tape volume'/

C     Close the current tape volume
C     -----------------------------
      status = tpclos (unit,HANDLE,TRWOFL,2)
      if (status .ne. TSUCC) then
         call errmsg('Warning:  Error closing first tape',
     +	 	     'edipsout-tpcloserr',1)
      endif

C     Inform the user that the next volume is being mounted
C     -----------------------------------------------------
      call errmsg('Second tape volume is being mounted',
     +	          'informational', 1)

C     Open the new tape volume
C     ------------------------
      opmesg = mesg//char(0)
      skip = 0
      status = tpopen (unit,skip,HANDLE,TNOREW,dens,TWRITE,opmesg,
     + tapeid)
      if (status .ne. TSUCC) then
	  mstat = -1
      else
          mstat = 0
      endif

C     Increment the volume counter
C     -------------------------------
      volnum = volnum + 1

      return
      end
