
C     ******************************************************************
C     PURPOSE: RBV processes a band of RBV image data.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     fill	       fill value
C     inbnd	       current input band number
C     msgtim	       elapsed time between display messages
C     nl	       number of lines in input image(s)
C     ns	       number of samples per line in input image(s)
C     unit     	       logical unit number of the tape drive
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     bndnum	       current band number--used for display message
C     eclose	       imageio function to close an image file
C     eread	       imageio function to read an image line
C     i		       line loop index
C     image(5388)      output image buffer
C     inbuf(5388)      input image buffer
C     lin	       indicates where to begin extracting lines
C		       when the image is larger than RBV size
C     reclen	       record length
C     record	       current record number
C     one	       value of one
C     pix	       indicates where to begin extracting pixels
C		       when the image is larger than RBV size
C     pxcopy           function to copy from one buffer to another
C     pxswap	       function to byte swap a data buffer
C     status	       function return status code
C     stlin	       indicates how many fill lines need to be inserted
C     stpix	       indicates how many fill pixels need to be inserted
C     tpwrit	       tapeio function to write to tape
C     tpweof	       tapeio function to write EOF mark(s) to tape
C     zero	       value of zero
C
C	ALGORITHM:  Fill the image with zeros
C		    Initalize processing message
C		    Write image records to tape
C		    Processing continues until all images have
C		       been written in a BSQ format.
C     *****************************************************************

      subroutine rbv (nl,ns,unit,fill,msgtim,inbnd)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4            
     .			   bndnum,
     .			   eclose,
     .			   eread,
     .                     i,
     .			   inbnd,
     .                     fill,
     .                     lin,
     .			   msgtim,
     .			   nl,
     .			   ns,
     .			   rfdesc,
     .                     reclen,
     .		  	   record
      integer*4
     .			   one,
     .                     pix,
     .			   pxcopy,
     .                     status,
     .                     stlin,
     .                     stpix,
     .                     tpwrit,
     .                     tpweof,
     .                     unit

      character*1	   fillvl,
     .			   image(5388),
     .			   inbuf(5388),
     .			   v127,
     .			   zero

      common/randio/rfdesc
      common/curent/i
      common/curcat/bndnum

      data reclen/5388/
  
      one = 1
      bndnum = 1

      zero = char(0)
      v127 = char(127)
      fillvl = char(fill)

C     Initialization
C     --------------
      call pxfill(image,EBYTE,reclen,zero)
      stlin = 0
      stpix = 0
      lin = 0
      pix = 0

      if (nl .gt. 5322) then
        lin = (nl - 5322)/2
        nl = 5322
      else
        stlin = (5322 - nl)/2
      endif

      if (ns .gt. 5322) then
        pix = (ns - 5322)/2
        ns = 5322
      else
        stpix = (5322 - ns)/2
      endif

C     Set record type to 237
C     ----------------------

      image(6) = char(237)

C     Fill the image portion of the record with the fill value
C     --------------------------------------------------------
      call pxfill(image(13),EBYTE,5322,fillvl)

C     Initialize and start the display message utility
C     ------------------------------------------------
      i = 1
      call inittm('EDIPSOUT','Processed','line','band',' ',5322,
     +		  1, msgtim, ONECAT)
      call startm

C     Write the proper number of fill lines to center the image
C     ---------------------------------------------------------
      do 100 i = 1,stlin

C       Set the record number
C       ---------------------
        record = i

        call btcpy(record,image(1),4)

C       Write the image record to tape
C       ------------------------------
        status = tpwrit (unit,image,reclen,HANDLE)
        if (status .ne. reclen) then
	   status = eclose(rfdesc)
  	   call fclean(unit)
	endif
100   continue

C     Process the image lines
C     -----------------------
      do 400 i = stlin + 1,nl + stlin

C       Set the record number
C       ---------------------
        record = i

        call btcpy(record,image(1),4)

C       Read in the image line
C       ----------------------
        status = eread(rfdesc,inbnd,i-stlin+lin,inbuf,one)
        if (status .lt. ESUCC) then
	   status = eclose(rfdesc)
	   call fclean(unit)
        endif

C       Copy the image data from the input buffer to the output buffer
C       --------------------------------------------------------------
        status = pxcopy(inbuf(1 + pix),
     +	   		image(13 + stpix),EBYTE,NS)

C       Fill the end values of the record to 127
C       ---------------------------------------

        image(5335) = char(127)
        image(5336) = char(127)

        call pxfill (image(5338),EBYTE,51,v127)

C       Write the image record to tape
C       ------------------------------
        status = tpwrit (unit,image,reclen,HANDLE)
        if (status .ne. reclen) then
	   status = eclose(rfdesc)
	   call fclean(unit)
	endif
400   continue

C     Initialize the image record
C     ---------------------------
      call pxfill(image,EBYTE,reclen,zero)

C     Set the record type to 237
C     --------------------------

      image(6) = char(237)

C     Fill the image portion of the record with the fill value
C     --------------------------------------------------------
      call pxfill(image(13),EBYTE,5322,fillvl)

C     Write the remaining fill lines
C     ------------------------------
      do 500 i = stlin + nl + 1,5322

C       Set the record number
C       ---------------------
        record = i

        call btcpy(record,image(1),4)

C       Write the fill record to tape
C       -----------------------------
        status = tpwrit (unit,image,reclen,HANDLE)
	if (status .ne. reclen) then
	   status = eclose(rfdesc)
	   call fclean(unit)
	endif
500   continue

C     Disable the display message utility
C     -----------------------------------
      call stoptm

C     Write the EOF mark
C     ------------------
      status = tpweof (unit,one,HANDLE)
      if (status .ne. TSUCC) then
	 status = eclose(rfdesc)
	 call fclean(unit)
      endif

      return
      end
