
C     ******************************************************************
C     PURPOSE:  WRTANT sets up and writes the annotation record to tape.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     annt   	       user-specified annotation record
C     curbnd	       current band number
C     gain(4)	       MSS sensor gain options
C     path	       nominal path number
C     prefix	       first portion of associated file names
C     row	       nominal row number
C     stype	       sensor type
C     trans(4)	       transmission mode
C     unit	       logical unit number of the tape drive
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     annot(5388)      annotation buffer (RBV)
C     annotm(3596)     annotation buffer (MSS)
C     annt1	       First portion of user specified 
C		       annotation string
C     annt2	       Second portion of user specified
C		       annotation string
C     filenm 	       host name of associated annotation file
C     flag	       indicator for whether associated annotation file exists
C     i	               loop index
C     lsmknm	       function to make an associated file name
C     lsstat	       function to check whether associated file exists
C     opn	       indicator for whether associated annotation file is open
C     pthbuf	       nominal path number (for annotation buffer)
C     reclen	       record length
C     recnum	       record number
C     rowbuf	       nominal row number (for annotation buffer)
C     status	       function return status code
C     tpwrit	       tapeio function to write to tape
C     unitan	       logical unit number of annotation file
C
C	Algorithm:  WRTANT fills the annotation buffer with zeros.
C		    The buffer area is filled with appropriate values.
C		    Information is gathered from the annotation file
C		       if one exists.
C	            The annotation file is written to tape.
C		    The files are closed.	
C     ******************************************************************

      subroutine wrtant(stype,path,row,unit,prefix,
     +  		gain,trans,curbnd,annt)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4            
     .                     i,
     .			   lsmknm,
     .			   lsstat,
     .			   reclen,
     .                     recnum,
     .                     status,
     .                     tpwrit,
     .                     unit,
     .  		   unitan

      character*112        annt

      character*(CMLEN)	   filenm,
     .			   prefix

      character*81         annt2

      character*25         annt1

      character*3 	
     .			   path,
     .			   pthbuf,
     .			   row,
     .			   rowbuf,
     .			   stype

      character*1	   annot(5388),
     . 			   annotm(3596),
     .			   curbnd,
     .			   gain(4),
     .			   fillvl,
     .			   trans(4)

      logical              flag,
     .                     opn

      equivalence (annot,annotm)
      equivalence (annot(7),annt1)
      equivalence (annot(41),annt2)
      equivalence (annot(33),pthbuf)
      equivalence (annot(37),rowbuf)
      
C     Initialize the annotation record to blanks
C     ------------------------------------------
      fillvl = char(32)
      call pxfill(annot,EBYTE,121,fillvl)

C     Set record length
C     -----------------
      if (stype .eq. 'MSS') then
        reclen = 3596
      else
        reclen = 5388
      endif

C     Set bytes 122 through record length to zero
C     -------------------------------------------
      do 100 i = 122,reclen
        annot(i) = char(0)
100   continue

C     Set tick mark data to blanks
C     ----------------------------
      do 200 i = 405,413
        annot(i) = char(32)
200   continue
      do 250 i = 423,548
        annot(i) = char(32)
250   continue
      do 300 i = 803,811
        annot(i) = char(32)
300   continue
      do 350 i = 821,964
        annot(i) = char(32)
350   continue
      do 400 i = 1201,1209
        annot(i) = char(32)
400   continue
      do 450 i = 1219,1263
        annot(i) = char(32)
450   continue
      do 500 i = 1599,1608
        annot(i) = char(32)
500   continue
      do 550 i = 1618,1760
        annot(i) = char(32)
550   continue
      do 600 i = 1997,2005
        annot(i) = char(32)
600   continue
      do 650 i = 2015,2059
        annot(i) = char(32)
650   continue
      do 700 i = 2395,2403
        annot(i) = char(32)
700   continue
      do 750 i = 2413,2538
        annot(i) = char(32)
750   continue

C     Set byte 5 to 0
C     ---------------
      annot(5) = char(0)

C     Set record type to 219
C     ----------------------
      annot(6) = char(219)

C     Determine if associated annotation file exists
C     ----------------------------------------------
      status = lsmknm(prefix,'edips_annot',filenm)
      status = lsstat(filenm,0)
      if (status .eq. 1) then
	 flag = .true.
      else
	 flag = .false.
      endif

      if (flag) inquire(file=filenm,opened=opn)

C     Open the associated annotation file if necessary
C     ------------------------------------------------
      if ((flag) .and. (.not. opn)) then
        unitan = 1
        open (unit=unitan,status='old',file=filenm,form='unformatted',
     .    iostat=status)
        if (status .ne. 0) then
   	  call errmsg 
     +    ('Error opening associated annotation file',
     +	   'edipsout-openerr',1)
          call fclean(unit)
        endif

        rewind (unit=unitan)
      endif

C     Read a record from the file if the file exists
C     ----------------------------------------------
      if (flag) then

        if (stype .eq. 'RBV') then
          read (unit=unitan,end=800,err=800) annot
        else 
          read (unit=unitan,end=800,err=800) annotm
        endif

        if (stype .eq. 'MSS') then
          annotm(91) = gain(ichar(curbnd))
          annotm(92) = trans(ichar(curbnd))
          annotm(121) = char(ichar(curbnd) + 48)
        else
          annotm(121) = char(65)
        endif

      endif

800   continue

C     Set the record number
C     ---------------------
      recnum = 2
      call btcpy(recnum,annot(1),4)

C     Write the user specified path row indicator to the annotation record
C     --------------------------------------------------------------------
      if (path .ne. ' ') pthbuf = path
      if (row .ne. ' ') rowbuf = row

C     Write the user specified annotation text to the annotation record
C     -----------------------------------------------------------------
      if (annt .ne. ' ') then
        if (stype .eq. 'MSS') then
          annt1 = annt (1:25)
          annt2 = annt (29:47) // '      ' // annt (57:81) // '   ' // 
     +      annt (85:111)
        else
          annt1 = annt (1:25)
          annt2 = annt (29:53) // annt (57:81) // '   ' //
     +      annt (85:112)
        endif
      endif

C     Write the annotation record to tape
C     -----------------------------------
      if (stype .eq. 'RBV') then
        status = tpwrit (unit,annot,reclen,HANDLE)
        if (status .ne. reclen) call fclean(unit)
      else
        status = tpwrit (unit,annotm,reclen,HANDLE)
        if (status .ne. reclen) call fclean(unit)
      endif

C     Close the associated file
C     -------------------------
      if (flag) close (unit = unitan)

      return
      end

