
C     ******************************************************************
C     PURPOSE: WRTTRA sets up and writes the trailer record to tape.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     bform	       band format indicator
C     bndnum(4)	       output band numbers
C     num	       number of trailer records to output
C     prefix	       first portion of associated file names
C     stype	       sensor type
C     unit	       logical unit number of the tape drive
C     unittr	       logical unit number of associated trailer file
C
C     INTERNAL VARIABLES:
C     Name             Description
C     ----	       -----------
C     count	       used to access proper record from associated trailer file
C     filenm	       host name of associated trailer file
C     flag	       indicator for whether associated trailer file exists
C     i		       loop index
C     lsmknm	       function to make an associated file name
C     lsstat	       function to check whether associated file exists
C     opn	       indicator for whether associated trailer file is open
C     pxswap	       byte swap a data buffer
C     reclen	       record length
C     recnum	       record number
C     status	       function return status code
C     tpwrit	       tapeio function to write to tape
C     trailr(5388)     trailer buffer (RBV)
C     trailm(3596)     trailer buffer (MSS)
C
C	ALGORITHM:  WRTTRA initializes the trailer buffer with zeros.
C		    Information is taken from the trailer file.
C		    The trailer buffer is filled with known values
C		      if the trailer file doesn't exist.
C		    The trailer file is written to tape.
C		    The files are closed.
C     ******************************************************************

      subroutine wrttra (stype,bform,prefix,unit,unittr,bndnum,num)

	include 'las.fin'

	include 'tapeio.fin'

      integer*4            
     .			   bndnum(4),
     .                     count,
     .                     i,
     .		    	   lsmknm,
     .			   lsstat,
     .                     num,
     .			   reclen,
     .                     recnum,
     .                     status,
     .                     tpwrit,
     .                     unit,
     .                     unittr

      character*(CMLEN)	   filenm,
     .			   prefix

      character*11	   bform

      character*3	   stype

      character*1          trailr(5388),
     .			   trailm(3596),
     .			   zero

      logical              flag,
     .                     opn

      equivalence (trailr,trailm)

      zero = char(0)

C     Initialize the trailer record to zeroes
C     ---------------------------------------
      call pxfill(trailr,EBYTE,5388,zero)

C     Set the record length
C     ---------------------
      if (stype .eq. 'MSS') then
        reclen = 3596
      else
        reclen = 5388
      endif
    
C     Set record type to 246
C     ----------------------

      trailm(6) = char(246)

C     Determine if associated trailer file exists
C     -------------------------------------------
      status = lsmknm(prefix,'edips_trail',filenm)
      status = lsstat(filenm,0)
      if (status .eq. 1) then
         flag = .true.
      else
	 flag = .false.
      endif

      if (flag) inquire(file=filenm,opened=opn)
      unittr = 3

C     Open the associated trailer file if necessary
C     ---------------------------------------------
      if ((flag) .and. (.not. opn)) then
        open (unit=unittr,status='old',file=filenm,access='direct',
     .    form='unformatted',recl=reclen,iostat=status)
        if (status .ne. 0) then
          call errmsg ('Error opening associated trailer file',
     +		       'edipsout-openerr',1)
        call fclean(unit)
        endif
   
        rewind (unit=unittr)
      endif

      count = 1

      do 100 i = 1,num
        if (flag) then
C         Read a record from the file if the file exists
C         ----------------------------------------------
          if (bform .eq. 'INTERLEAVED') then
            if (bndnum(count) .eq. i) then
              read (unit=unittr,rec=bndnum(count),err=50) trailm
              count = count + 1
            else
    	      call pxfill(trailr,EBYTE,5388,zero) 

              trailr(6) = char(246)

            endif
          else
            if (stype .eq. 'MSS') then
              read (unit=unittr,rec=bndnum(count),err=50) trailm
            else
              read (unit=unittr,rec=bndnum(count),err=50) trailr
            endif
          endif

        endif

50      continue

C       Set the record number
C       ---------------------
        recnum = i

	call btcpy(recnum,trailr(1),4)

C       write the trailer record to tape
C       --------------------------------
        if (stype .eq. 'MSS') then
          status = tpwrit (unit,trailm,reclen,HANDLE)
	  if (status .ne. reclen) call fclean(unit)
        else
          status = tpwrit (unit,trailr,reclen,HANDLE)
          if (status .ne. reclen) call fclean(unit)
        endif

100   continue

      return
      end
