/****************************************************************************
NAME:				BACKMATCH

PURPOSE:  Compares two tie point location (TPL) files and de-actives points
          if the difference between the backmatched points are outside the
	  tolerance value.

VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 9/92	D. Steinwand

****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

#define NOTSAME	 202

int backmatch
(
    struct TPLDATA *tie_data1,	/* Tie Point Location data records 	*/
    struct TPLDATA *tie_data2,	/* Tie Point Location data records 	*/
    long npts,			/* Number of points in TPL file 	*/
    double tolval			/* Tolerance value 			*/
)
{
struct TPLDATA *tt1;		/* Ptr to TPL data structure 		*/
struct TPLDATA *tt2;		/* Ptr to TPL data structure 		*/
double delta;			/* difference value			*/
double deltax;			/* x difference value			*/
double deltay;			/* y difference value			*/
long i;				/* Loop counters 			*/
long npts_used;			/* Current number of points used 	*/
long pass=1;			/* Non-fatal c_errmsg flag 		*/
char mess[80];			/* Message buffer for c_errmsg routine 	*/

/* Loop thru points and compare
  ----------------------------*/
for(tt1=tie_data1, tt2=tie_data2, npts_used=0, i=0; i<npts; i++,tt1++,tt2++)
   if((tt1->active == 0)||(tt1->active == 1)||(tt1->active > 9))
      {

/* See if points are comparable.  If any one point is not compareable, abort
  -------------------------------------------------------------------------*/
      if ((tt2->ref_coord[0] != tt1->sea_coord[0]) ||
          (tt2->ref_coord[1] != tt1->sea_coord[1]) ||
          (strcmp(tt1->pt_id,tt2->pt_id) != 0)) return(NOTSAME);

/* If a point in the first file is active, but the point is not active
   in the second file, de-activate it!
  -----------------------------------*/
      if((tt2->active != 0)&&(tt2->active != 1)&&(!(tt2->active > 9)))
	  {
	  tt1->active = 8;
          sprintf(mess,"%s [8] Second point not active", tt1->pt_id); 
          c_errmsg(mess," ",&pass); 
          } 

/* See if points are within tolerance
  ----------------------------------*/
      deltax = tt1->ref_coord[1] - tt2->sea_coord[1];
      deltay = tt1->ref_coord[0] - tt2->sea_coord[0];
      delta = sqrt(deltax * deltax + deltay * deltay);
      if (delta > tolval) 
	  {
	  tt1->active = 8;
          sprintf(mess,"%s [8] Backmatched values are not within tolerance", 
			tt1->pt_id);
	  c_errmsg(mess," ",&pass);
 	  }
      npts_used++;
      }

      if(npts_used == 0)
	  {
	  c_lasmsg (FALSE, "editcorr-0points", "Zero points were used");
	  return(E_FAIL);
	  }

return(E_SUCC);
}
