/****************************************************************************
NAME:				DISTANCE

PURPOSE:  Edits points based on mean of displacement from nominal

VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 4/93	D. Steinwand

****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

int distance
(
    struct TPLDATA *tie_data,	/* Tie Point Location data records 	*/
    long npts,			/* Number of points in TPL file 	*/
    double tolval			/* Sigma value--for corr peak edits 	*/
)
{
struct TPLDATA *tt;		/* Ptr to TPL data structure 		*/
double sum = 0.0;		/* Sum accumulator 			*/
double mean;			/* Mean correlation peak value 		*/
double upper_threshold;		/* Upper_Threshold = MEAN + TOLVAL      */
double lower_threshold;		/* Lower_Threshold = MEAN - TOLVAL      */
long i;				/* Loop counters 			*/
long npts_used;			/* Current number of points used 	*/
long pass=1;			/* Non-fatal c_errmsg flag 		*/
char mess[80];			/* Message buffer for c_errmsg routine 	*/

/* Calculate mean of the line and sample displacement values
  ---------------------------------------------------------*/
for(tt=tie_data, npts_used=0, i=0; i<npts; i++, tt++)
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
      {
      sum += tt->displacement;
      npts_used++;
      }

      if(npts_used == 0)
	  {
	  c_lasmsg (FALSE, "editcorr-0points", "Zero points were used");
	  return(E_FAIL);
	  }

mean = sum/(double)npts_used;

/* Report statistics to user
  -------------------------*/
c_errmsg("Correlation Displacement Statistics:"," ",&pass);
sprintf(mess,"   %d active points used in calculations.",npts_used);
c_errmsg(mess," ",&pass);
sprintf(mess,"   Mean Displacement:  %f", mean);
c_errmsg(mess," ",&pass);
sprintf(mess,"   Threshold:  (+/-) %f", tolval);
c_errmsg(mess," ",&pass);

/* Make pts outside threshold inactive--report removed pts to user
  ---------------------------------------------------------------*/
if(tolval>0)
   {
   upper_threshold = mean + tolval;
   lower_threshold = mean - tolval;
   for(tt=tie_data, i=0; i<npts; i++, tt++)
      if(((tt->displacement > upper_threshold) || 
       (tt->displacement < lower_threshold)) && 
       ((tt->active<2) || (tt->active>9)))
         {
         tt->active=5;
	 npts_used--;
         sprintf(mess,"%s [5] Outside specified distance from mean displacment",
                      tt->pt_id);
         c_errmsg(mess," ",&pass);
         }
   }
return(E_SUCC);
}
