/****************************************************************************
NAME:				EDITCORR

PURPOSE:  EDITCORR uses a Tie Point Location file containing tie point pairs
          to derive bivariate polynomials using the method of least squares.
	  Editing of the TPL file is done by the "leave-one-out" method.

VERSION	 DATE	AUTHOR	   
-------	 ----	------	  
  1.0	 9/92	D. Steinwand
  2.0	12/92	T. Mittan		Added option to specify plot file.
					Released to baseline.
  3.0    4/93   D. Steinwand		Added DISTANCE subcommand

****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

#define DIFFNPTS	201
#define MODEL		  1
#define PEAKS		  2
#define COMP		  3
#define BACK		  4
#define DISTANCE 	  5
#define MAXIMUM           6
#define MEDIAN            7

#define CHECK(ARG) status = ARG; if(status != E_SUCC) l_abort(status)

int main
(void)
{
struct TPLHEAD tie_head;	/* Tie point location file header record   */
struct PARBLK vblk;		/* TAE parameter block 			   */
struct TPLDATA *tie_data1;	/* Ptr to Tie point location data records  */
struct TPLDATA *tie_data2;	/* Ptr to Tie point location data records  */
long num_out=1;			/* Number of output files 		   */
long std_out;			/* Dummy variable to standard output 	   */
long npts1;			/* Number of points in first TPL file 	   */
long npts2;			/* Number of points in second TPL file 	   */
long status;			/* return status			   */
long pltflg = FALSE;		/* plot flag			  	   */
long degree;			/* Degree of polynomial used in fit 	   */
long subcmd;			/* Subcommand used 			   */
long actflg;			/* active flag				   */
float maxres;			/* Maximum allowable RMSE 		   */
float sigval;			/* Sigma factor for edit on corr strength  */
float tolval;			/* Tolerance value for COMPARE & BACKMATCH */
char tpl1[CMLEN];		/* Input Tie Point Location file name 	   */
char tpl2[CMLEN];		/* Input Tie Point Location file name 	   */
char outtl[CMLEN];		/* Output Tie Point Location file name 	   */
char outbuf[2][CMLEN];		/* Tae name of output files for complete   */
char plotfile[CMLEN];		/* plot filename			   */

c_inlas(&vblk, &std_out);

/* Allocate buffer space for tie point records
  -------------------------------------------*/
tie_data1=(struct TPLDATA *)malloc((sizeof(struct TPLDATA)*MAX_TIE_POINTS));
if (tie_data1 == NULL) l_abort(NO_SPACE);
tie_data2=(struct TPLDATA *)malloc((sizeof(struct TPLDATA)*MAX_TIE_POINTS));
if (tie_data2 == NULL) l_abort(NO_SPACE);

CHECK(getpar(&vblk,&subcmd,tpl1,tpl2,outtl,&degree,&maxres,&sigval,&tolval,
	     &pltflg,plotfile,&actflg));

if(subcmd == MODEL)
        {
        CHECK(get_tie_points(tpl1, &tie_head, &tie_data1, &npts1));
        CHECK(model(tie_data1, npts1, degree, maxres,pltflg,plotfile,actflg));
        }
if(subcmd == MAXIMUM)
        {
        CHECK(get_tie_points(tpl1, &tie_head, &tie_data1, &npts1));
        CHECK(maximum(tie_data1, npts1, degree, maxres,pltflg,plotfile));
        }
if(subcmd == MEDIAN)
        {
        CHECK(get_tie_points(tpl1, &tie_head, &tie_data1, &npts1));
        CHECK(median(tie_data1, npts1, degree, maxres,pltflg,plotfile));
        }
if(subcmd == DISTANCE)
	{
	CHECK(get_tie_points(tpl1, &tie_head, &tie_data1, &npts1));
	CHECK(distance(tie_data1, npts1, tolval));
        } 
if(subcmd == PEAKS)
        {
        CHECK(get_tie_points(tpl1, &tie_head, &tie_data1, &npts1));
        CHECK(peaks(tie_data1, npts1, sigval));
        }
if(subcmd == COMP)
        {
        CHECK(get_tie_points(tpl2, &tie_head, &tie_data2, &npts2));
        CHECK(get_tie_points(tpl1, &tie_head, &tie_data1, &npts1));
        if(npts1 != npts2) l_abort(DIFFNPTS);
        CHECK(compare_tpl(tie_data1, tie_data2, npts1, tolval));
        }
if(subcmd == BACK)
        {
        CHECK(get_tie_points(tpl2, &tie_head, &tie_data2, &npts2));
        CHECK(get_tie_points(tpl1, &tie_head, &tie_data1, &npts1));
        if(npts1 != npts2) l_abort(DIFFNPTS);
        CHECK(backmatch(tie_data1, tie_data2, npts1, tolval));
        }

CHECK(put_tie_points(outtl, &tie_head, tie_data1, npts1));
strcpy(outbuf[0], outtl);
if (pltflg)
   {
   strcpy(outbuf[1], plotfile);
   num_out = 2;
   c_complt("EDITCORR", outbuf, &num_out);
   }
else
   c_complt("EDITCORR", outbuf, &num_out);

return EXIT_SUCCESS;

}
