/****************************************************************************
NAME:				GETPAR

PURPOSE:  Get user entered parameters from the TAE parameter block

VERSION		DATE	AUTHOR
-------		----	------	
  1.0		9/92	D. Steinwand

****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

#define MODEL		  1
#define PEAKS		  2
#define COMP		  3
#define BACK		  4
#define DISTANCE          5
#define MAXIMUM           6
#define MEDIAN            7


int getpar
(
    struct PARBLK *vblk,	/* TAE variable block 				*/
    long *subcmd,		/* Subcommand used 				*/
    char *t1,		/* Input tie point location file name 		*/
    char *t2,		/* Input tie point location file name (2nd) 	*/
    char *out_tpl,		/* Output tie point location file name 		*/
    long *degree,		/* Degree of polynomial 			*/
    float *maxres,		/* Maximum allowable residual 			*/
    float *sigval,		/* Sigma value--for corr peak edits 		*/
    float *tolval,		/* Tolerance value--for comp. & backmatching 	*/
    long *pltflg,		/* plot flag					*/
    char *plotfile,		/* plot filename				*/
    long *actflag          /* active flag                                  */
)
{
struct VARIABLE *var;		/* pointer to a TAE variable in vblk 	*/
long acc;			/* access flag for c_getnam 		*/
long zero = 0;			/* Constant of zero 			*/
long one = 1;			/* Constant of one 			*/
long count;			/* count of OUTFILE			*/

/* Get the subcommand used
  -----------------------*/
var = p_find(vblk, "_SUBCMD");
if      (strcmp("MODEL",    SVAL(*var,0)) == 0) *subcmd=MODEL;
else if (strcmp("MAXIMUM",  SVAL(*var,0)) == 0) *subcmd=MAXIMUM;
else if (strcmp("MEDIAN",   SVAL(*var,0)) == 0) *subcmd=MEDIAN;
else if (strcmp("DISTANCE", SVAL(*var,0)) == 0) *subcmd=DISTANCE;
else if (strcmp("PEAKS",    SVAL(*var,0)) == 0) *subcmd=PEAKS;
else if (strcmp("BACKMATCH",SVAL(*var,0)) == 0) *subcmd=BACK;
else                                            *subcmd=COMP;

/* Get the input tie point location file names
  -------------------------------------------*/
acc = IREAD;
if(c_getnam(vblk,"INTL",&zero,t1,"tpl",&acc)!=E_SUCC)return(NO_TPL);
if((*subcmd == BACK) || (*subcmd == COMP))
   if(c_getnam(vblk,"INTL",&one,t2,"tpl",&acc)!=E_SUCC)return(NO_TPL);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(vblk, "OUTTL", &zero, out_tpl, "tpl", &acc) != E_SUCC)
	return(BAD_TPL);

/* Get the maximum allowable degree of polynomial for model
  --------------------------------------------------------*/
if ((*subcmd == MODEL) || (*subcmd == MAXIMUM) || (*subcmd == MEDIAN))
   {var=p_find(vblk,"PDEGREE");*degree=IVAL(*var,0);}

/* Get the maximum residual value allowed
  --------------------------------------*/
if ((*subcmd == MODEL) || (*subcmd == MAXIMUM) || (*subcmd == MEDIAN))
   {var=p_find(vblk,"MAXRES");*maxres=RVAL(*var,0);}

/* Get the output plot file 
  -----------------------*/
if ((*subcmd == MODEL) || (*subcmd == MAXIMUM) || (*subcmd == MEDIAN))
   {
   if (c_getcnt(vblk,"OUTFILE",&count) != E_SUCC)
       return(E_FAIL);
   if (count != 0)
      {
      *pltflg = TRUE;
      acc = IWRITE;
      if (c_getnam(vblk,"OUTFILE",&zero,plotfile,"plt",&acc) != E_SUCC)
         return(E_FAIL);
      }
   else
      *pltflg = FALSE;
   }

/* Get plot flag
--------------*/
if (*subcmd == MODEL)
   {
   var=p_find(vblk,"ACTFLG");
   if (strcmp("ACTIVE",SVAL(*var,0)) == 0)
      {
      *actflag = 1;
      }
   else
   if (strcmp("INACTIVE",SVAL(*var,0)) == 0)
      {
      *actflag = 2;
      }
   else
      {
      *actflag = 3;
      }
   }


/* Get the backmatch/compare tolerance value
  -----------------------------------------*/
if ((*subcmd == BACK) || (*subcmd == COMP) || (*subcmd == DISTANCE))
     {var=p_find(vblk,"TOLVAL");*tolval=RVAL(*var,0);}

/* Get the number of std dev from mean to do first-cut edits on correlation
   strength values
  ---------------*/
if (*subcmd == PEAKS){var=p_find(vblk,"NSTDEV");*sigval=RVAL(*var, 0);}

return(E_SUCC);
}
