/****************************************************************************
NAME:				 MEDIAN

PURPOSE:  

VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 4/93	D. Steinwand

****************************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "geompak.h"
#include "lasmath.h"
#include "protos.h"

int median
(
    struct TPLDATA *tie_data,	/* Tie Point Location data records 	*/
    long npts,			/* Number of points in TPL file 	*/
    long degree,			/* Degree of polynomial to use 		*/
    double maxres,			/* Maximum allowable residual 		*/
    long plot,			/* plot flag				*/
    char *plotfile			/* plot filename			*/
)
{
struct TPLDATA *tt;		/* Ptr to TPL data structure 		*/
double mederr;			/* Median error                         */
double test_median;		/* RMSE of each test value 		*/
double min_median;		/* Minimum running RMSE for given test 	*/
double maxerr;			/* Maximum residual error               */
long old_active;		/* Old active flag value 		*/
long index;			/* Index of tie pt to remove 		*/
long i,k;			/* Loop counters 			*/
long npts_used;			/* Current number of points used 	*/
long pass=1;			/* Non-fatal c_errmsg flag 		*/
long num_surfaces;		/* Number of surfaces calculated        */
char mess[80];			/* Message buffer for c_errmsg routine 	*/

/* Find the number of active points
  --------------------------------*/
for(tt=tie_data, npts_used=0, i=0; i<npts; i++, tt++)
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9)) npts_used++;

if(npts_used == 0)
   {
   c_lasmsg (FALSE, "editcorr-0points", "Zero points were used");
   return(E_FAIL);
   }

/* Edit pts based on the "leave-one-out" method
  --------------------------------------------*/
for(num_surfaces=0;;)
   {
   medfit(tie_data, npts, npts_used, degree, &mederr, &maxerr, plot, plotfile);
   if (maxerr <= maxres) 
        {
	sprintf(mess,"%d active points remain", npts_used);
	c_errmsg(mess," ", &pass);
	sprintf(mess,"%d surfaces calculated", num_surfaces);
	c_errmsg(mess," ", &pass);
	sprintf(mess,"Maximum residual error:  %f", maxerr);
	c_errmsg(mess," ", &pass);
	return(E_SUCC);
	}
   if (((npts_used<5)&&(degree==1))||((npts_used<8)&&(degree==2))) 
	{
	c_errmsg("Too few points remain to model","editcorr-minpts",&pass);
	return(E_SUCC);
	}

   min_median = 999999999.0; index= -1;
   for(tt=tie_data, k=0; k < npts; k++, tt++)
      if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
         {
         old_active=tt->active;
         tt->active=8;
         medfit(tie_data,npts,npts_used,degree,&test_median,&maxerr,0,plotfile);
         if(test_median < min_median) {min_median = test_median; index=k;}
         tt->active=old_active;
         num_surfaces++;
         }
   sprintf(mess, "%s [8] Removed from model; Median residual:  %f",
	    tie_data[index].pt_id,min_median);
   c_errmsg(mess," ",&pass);
   tie_data[index].active=8;
   npts_used--;
   }
}

/****************************************************************************
NAME:				FIT

PURPOSE:  Preforms a least-squares fit and calculates residual errors

VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 4/93	D. Steinwand

****************************************************************************/
int medfit
(
    struct TPLDATA *tie_data,	/* Tie Point Location data records 	  */
    long npts,			/* Number of points in TPL file 	  */
    long npts_used,			/* Current number of points used 	  */
    long degree,			/* Degree of polynomial used 		  */
    double *med,			/* Median error                           */
    double *maxres,			/* Maximum residual error                 */
    long plot,			/* Flag--write plot file? 		  */
    char *plotfile			/* plot filename			  */
)
{
struct TPLDATA *tt;		/* Ptr to TPL data structure 		  */
double *v;			/* Work buffer for LSQ fit 		  */
double *b1;			/* Vector containing X coordinates 	  */
double *b2;			/* Vector containing Y coordinates 	  */
double *A;			/* Matrix A (used for LSQ fit) 		  */
double *mag;			/* Magnitude of residuals                 */
double x,y;			/* Calculated x & y locactions 		  */
double delta_x,delta_y;		/* Absolute value of x & y residuals 	  */
double rmse;			/* Root Mean Squared error                */
double rms_acc; 		/* Accumulator for RMSe calc 		  */
double x_residual;		/* Residual in X 			  */
double y_residual;		/* Residual in Y 		 	  */
long i,k;			/* Loop counters 			  */
long pass = 1;			/* nonfatal error flag			  */
long size;			/* Size of vectors (based on poly degree) */
FILE *fptr = NULL;

if(degree==1)size=3; else size=6;
if (plot) fptr = fopen(plotfile,"w");

/* initilize variables
--------------------*/
v =(double *)malloc((sizeof(double )*npts));
if (v == NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","editcorr-alloc",&pass);
   return (E_FAIL);
   }
b1 =(double *)malloc((sizeof(double )*npts));
if (b1 == NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","editcorr-alloc",&pass);
   return (E_FAIL);
   }
b2 =(double *)malloc((sizeof(double )*npts));
if (b2 == NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","editcorr-alloc",&pass);
   return (E_FAIL);
   }
A =(double *)malloc((sizeof(double )*6 * npts));
if (A == NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","editcorr-alloc",&pass);
   return (E_FAIL);
   }
mag =(double *)malloc((sizeof(double )*6 * npts));
if (A == NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","editcorr-alloc",&pass);
   return (E_FAIL);
   }

/* Loop on tie point pairs.  If the tie point is active, use it in the
   fit calculations; first for x, then for y.
  ------------------------------------------*/
for(tt=tie_data, k=0, i=0; i < npts; i++, tt++)
   {
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
      {
      A[k] =               1.0;
      A[k+npts_used] =     tt->ref_coord[1];
      A[k+(2*npts_used)] = tt->ref_coord[0];
      if(degree==2)
           {
           A[k+(3*npts_used)]=tt->ref_coord[1] * tt->ref_coord[1];
           A[k+(4*npts_used)]=tt->ref_coord[1] * tt->ref_coord[0];
           A[k+(5*npts_used)]=tt->ref_coord[0] * tt->ref_coord[0];
           }
      b1[k] = tt->sea_coord[1];
      b2[k] = tt->sea_coord[0];
      k++;
      }
   }
c_qrdecomp(A, npts_used, size, v, 0);
c_qrsolve(A, npts_used, size, v, b1, 0);
c_qrsolve(A, npts_used, size, v, b2, 0);

/* Calculate Residuals
  -------------------*/
rms_acc = 0.0;
*maxres = 0.0;

/* Loop on tie point pairs.  If the tie point is active, evaluate the
   polynomial and calculate residuals & rms errors
  -----------------------------------------------*/
if (plot) fprintf(fptr,"%d\n",npts_used);
for(tt=tie_data, npts_used=0, i=0; i < npts; i++, tt++)
   {
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
      {
       x = c_eval(&degree,b1,&(tt->ref_coord[1]),&(tt->ref_coord[0]));
       y = c_eval(&degree,b2,&(tt->ref_coord[1]),&(tt->ref_coord[0]));
       x_residual = tt->sea_coord[1] - x;
       y_residual = tt->sea_coord[0] - y;
       mag[npts_used] = sqrt(x_residual*x_residual + y_residual*y_residual);
       if(mag[npts_used] > *maxres) *maxres = mag[npts_used];
       delta_x = fabs(x_residual);
       delta_y = fabs(y_residual);
       if (plot) fprintf(fptr,"%s %f %f %f %f %d\n",tt->pt_id,tt->sea_coord[1],
		tt->sea_coord[0], x_residual, y_residual,tt->active);
       rms_acc += (delta_x * delta_x) + (delta_y * delta_y);
       npts_used++;
       }
   }
*med = find_median(mag,npts_used);

/* Calculate residuals
  -------------------*/
rmse = sqrt(rms_acc / npts_used);
if (plot) fprintf(fptr,"%f\n",rmse);
if (plot) fclose(fptr);

free (v);
free (b1);
free (b2);
free (A);
free (mag);

return(E_SUCC);
}

/****************************************************************************
NAME:                            FIND_MEDIAN

PURPOSE:  Finds the median value of a set of residuals

VERSION  DATE   AUTHOR
-------  ----   ------
  1.0    4/93   D. Steinwand

****************************************************************************/
static int compare_double
(
    double *i,
    double *j
)
  {if(*i>*j) return(1); else if(*i<*j) return(-1); else return(0);}

double find_median
(
    double *residual,
    long npts
)
{ 
qsort(residual,npts,sizeof(double),compare_double); 
if ((npts-1)%2 != 0) return((residual[npts/2]+residual[(npts/2)-1])/2.0);
else return(residual[npts/2]); }
