/****************************************************************************
NAME:				PEAKS

PURPOSE:  Edits points based on mean & std dev of correlation peak values

VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 9/92	D. Steinwand

****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

int peaks
(
    struct TPLDATA *tie_data,	/* Tie Point Location data records 	*/
    long npts,			/* Number of points in TPL file 	*/
    double sigval			/* Sigma value--for corr peak edits 	*/
)
{
struct TPLDATA *tt;		/* Ptr to TPL data structure 		*/
double sum = 0.0;		/* Sum accumulator 			*/
double sumsqr = 0.0;		/* Sum squared accumulator 		*/
double mean;			/* Mean correlation peak value 		*/
double stddev;			/* Std Dev of correlation peaks 	*/
double threshold;		/* Threshold = MEAN - (SIGVAL * STDDEV/2) */
long i;				/* Loop counters 			*/
long npts_used;			/* Current number of points used 	*/
long pass=1;			/* Non-fatal c_errmsg flag 		*/
char mess[80];			/* Message buffer for c_errmsg routine 	*/

/* Calculate mean and std dev on the correlation strength values
  -------------------------------------------------------------*/
for(tt=tie_data, npts_used=0, i=0; i<npts; i++, tt++)
   if((tt->active == 0)||(tt->active == 1)||(tt->active > 9))
      {
      sum += tt->strength;
      sumsqr += tt->strength * tt->strength;
      npts_used++;
      }

   if(npts_used == 0)
      {
      c_lasmsg (FALSE, "editcorr-0points", "Zero points were used");
      return(E_FAIL);
      }

mean = sum/(double)npts_used;
stddev = sqrt((sumsqr - (sum*sum/(double)npts_used))/(double)(npts_used-1));

/* Report statistics to user
  -------------------------*/
c_errmsg("Correlation Strength Statistics:"," ",&pass);
sprintf(mess,"   %d active points used in calculations.",npts_used);
c_errmsg(mess," ",&pass);
sprintf(mess,"   Mean & Standard Dev:  %f, %f", mean, stddev);
c_errmsg(mess," ",&pass);

/* Make pts less than threshold inactive--report removed pts to user
  -----------------------------------------------------------------*/
if(sigval>0)
   {
   threshold = mean - sigval*stddev;
   sprintf(mess,"Correlation strength threshold:  %f",threshold);
   c_errmsg(mess, " ", &pass);
   for(tt=tie_data, i=0; i<npts; i++, tt++)
      if((tt->strength < threshold) && ((tt->active<2) || (tt->active>9)))
         {
         tt->active=4;
	 npts_used--;
         sprintf(mess,"%s [4] Strength of peak below threshold value",
                      tt->pt_id);
         c_errmsg(mess," ",&pass);
         }
   }
return(E_SUCC);
}
