/*******************************************************************************
NAME:  DDRHELP 

PURPOSE:  To display the help for EDITDDR.

PROGRAM HISTORY:
Version		Date	Author		Code/contr	Request
-------		----	------		----------	-------
  1.0		11/87	B.Ailts		   CSB		Original 
							development
  2.0		04/88   B.Ailts		   CSB		Replaced ___term
							calls with the
							TAE terminal 
							calls
							Replaced 
							newlas.h with 
							las.h
  1.2		07/88	B.Ailts		   CSB		LAS 5.0 standards
  1.3		12/88   B.Ailts		EDC/CSB		added noaa-11 + radians
  1.4           10/89   B.Ailts		EDC/CSB		Changed last accessed to
							last modified
  1.5		06/92	S.Nelson	EDC/CSB		Added the proj code to
							the list of projections.
							Added map projectiosn
							Hammer and Robinson.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
Receives the number of lines in the screen and the code for which the help 
 is needed.  
Based on the code, the help messages are displayed to the terminal.  
A code of a blank will display general help message.
The screen is displayed until the return key is hit and then the
 control is returned to the calling routine.
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"			/* general newlas includes 	*/
#include "editddr.h"

void ddrhelp
(
  char *ans,                       /* I: Input help code */
  long *line                       /* I: Number of lines in the terminal */
)
{  /* ddrhelp */

long term;				/* terminator of t_input	*/
char blank[81];				/* buffer for a blank line	*/
char retrn[60];				/* buffer for return message	*/
char temp[3];				/* buffer for return input	*/

/*  Initialize variables
------------------------*/
strcpy(blank,"                                                               ");
strcpy(retrn,"Press the return key when finished");

/*  Clear the terminal
----------------------*/
t_clear();

/*  Compare the input code with the known codes and output the correct
    help message to the terminal.
----------------------------------------------------------------------*/
if (strncmp(ans,"in",2) == 0)
   {
   t_output(2,2,"Input image.  The name of the image that the DDR");
   t_output(3,2,"file describes and is associated with.");
   }
else if (strncmp(ans,"nl",2) == 0)
   t_output(2,2,"Number of lines.  The number of lines in the image.");

else if (strncmp(ans,"ns",2) == 0)
   t_output(2,2,
       "Number of samples.  The number of samples per line in the image.");

else if (strncmp(ans,"nb",2) == 0)
   t_output(2,2,"Number of bands.  The number of bands in the image.");

else if (strncmp(ans,"dt",2) == 0)
   {
   t_output(2,2,"Data type.  The data type of the image.");
   t_output(3,2,"  = BYTE: BYTE     (8-bit unsigned integer)");  
   t_output(4,2,"                   (0 through 255)");
   t_output(5,2,"  = I*2: INTEGER*2 (16-bit signed integer)");
   t_output(6,2,"		    (-32768 through 32767)");
   t_output(7,2,"  = I*4: INTEGER*4 (32-bit signed integer)");
   t_output(8,2,"		    (-2147483648 through 2147483648)");
   t_output(9,2,"  = R*4: REAL*4    (32-bit signed real)");
   t_output(10,2,"		    (System dependent)");
   }
else if (strcmp(ans,"mc") == 0)
   {
   t_output(2,2,"Master line and sample.  The line and sample location");
   t_output(3,2,"relative to the master image.");
   }
else if (strcmp(ans,"pc") == 0)
   {
   t_output(2,2, "Projection code.");
   t_output(3,2,"Refer to the GEOMPOD for further details.");
   t_output(4,2,"Valid projection system codes are:");
   t_output(5,2,"  GEO(0)     = geographic");
   t_output(6,2,"  UTM(1)     = Universal Transverse Mercator");
   t_output(7,2,"  SPCS(2)    = State Plane Coordinates");
   t_output(8,2,"  ALBERS(3)  = Albers Conical Equal Area");
   t_output(9,2,"  LAMCC(4)   = Lambert Conformal Conic");
   t_output(10,2,"  MERCAT(5)  = Mercator");
   t_output(11,2,"  PS(6)      = Polar Stereographic");
   t_output(12,2,"  POLYC(7)   = Polyconic");
   t_output(13,2,"  EQUIDC(8)  = Equidistant Conic");
   t_output(14,2,"  TM(9)      = Transverse Mercator");
   t_output(15,2,"  STEREO(10) = Stereographic");
   t_output(16,2,"  LAMAZ(11)  = Lambert Azimuthal Equal Area");
   t_output(17,2,"  AZMEQD(12) = Azimuthal Equidistant");
   t_output(18,2,"  GNOMON(13) = Gnomonic");
   t_output(19,2,"  ORTHO(14)  = Orthographic");

   /*  Split the help messages between two seperate screeens
   ---------------------------------------------------------*/
   t_output(*line,2,retrn);
   t_input(*line,37,temp,&term);
   if (term == P_OVER)
      t_output(*line - 2,2,"[editddr-input] Input is larger than buffer size");
   t_clear();
   t_output(2,2,"  GVNSP(15)  = General Vertical Near-Side Perspective");
   t_output(3,2,"  SNSOID(16) = Sinusoidal");
   t_output(4,2,"  EQRECT(17) = Equirectangular");
   t_output(5,2,"  MILLER(18) = Miller Cylindrical");
   t_output(6,2,"  VGRINT(19) = Van der Grinten");
   t_output(7,2,"  HOM(20)    = Oblique Mercator (Hotine)");
   t_output(8,2,"  ROBIN(21)  = Robinson");
   t_output(9,2,"  SOM(22)    = Space Oblique Mercator (SOM)");
   t_output(10,2,"  ALASKA(23) = Alaska Conformal");
   t_output(11,2,"  GOOD(24)   = Interrupted Goode Homolsine");
   t_output(12,2,"  MOLL(25)   = Mollweide");
   t_output(13,2,"  IMOLL(26)  = Interrupted Mollweide");
   t_output(14,2,"  HAMMER(27) = Hammer");
   t_output(15,2,"  WAGIV(28)  = WAGNER IV");
   t_output(16,2,"  WAGVII(29) = WAGNER VII");
   t_output(17,2,"  OBEQA(30)  = Oblated Equal Area");
   t_output(18,2,"  31+        = User Defined");
   }
else if (strncmp(ans,"pcv",3) == 0)
   {
   t_output(2,2,
      "Projection code validity.  The validity of the projection code value.");
   t_output(3,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(4,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(5,2,"  = UN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"zc") == 0)
   {
   t_output(2,2,
      "Zone code.  The zone to use for UTM and State Plane projection"); 
   t_output(3,2,
      "systems.  Refer to the GEOMPOD documentation for further details.");
   }
else if (strncmp(ans,"zcv",3) == 0)
   {
   t_output(2,2,"Zone code validity.  The validity of the zone code value.");
   t_output(3,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(4,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(5,2,"  = UN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"dc") == 0)
   {
   t_output(2,2,
      "Datum code.  The datum can be used for setting the semimajor");
   t_output(3,2,
      "axis and eccentricity squared for a particular projection.  Refer to");
   t_output(4,2,"the GEOMPOD documentation for further details.");
   t_output(5,2,"The supported datums are:");
   t_output(6,2,"  0 = Clarke 1866"); 
   t_output(7,2,"  1 = Clarke 1880");
   t_output(8,2,"  2 = Bessel");
   t_output(9,2,"  3 = International 1967");
   t_output(10,2,"  4 = International 1909");
   t_output(11,2,"  5 = WGS 72");
   t_output(12,2,"  6 = Everest");
   t_output(13,2,"  7 = WGS 66");
   t_output(14,2,"  8 = GRS 1980");
   t_output(15,2,"  9 = Airy");
   t_output(16,2,"  10 = Modified Everest");
   t_output(17,2,"  11 = Modified Airy");
   t_output(18,2,"  12 = Walbeck");
   t_output(19,2,"  13 = Southeast Asia");

   /*  Split the help messages between two seperate screeens
   ---------------------------------------------------------*/
   t_output(*line,2,retrn);
   t_input(*line,37,temp,&term);
   if (term == P_OVER)
      t_output(*line - 2,2,"[editddr-input] Input is larger than buffer size");
   t_clear();
   t_output(2,2,"  14 = Australian National");
   t_output(3,2,"  15 = Krassovsky");
   t_output(4,2,"  16 = Hough");
   t_output(5,2,"  17 = Mercury 1960");
   t_output(6,2,"  18 = Modified Mercury 1968");
   t_output(7,2,"  19 = Spere of radius - 6370997 meters");
   }
else if (strncmp(ans,"dcv",3) == 0)
   {
   t_output(2,2,"Datum code validity.  The validity of the datum code value.");
   t_output(3,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(4,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(5,2,"  = UN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"pp") == 0)
   {
   t_output(2,2,
      "Projection parameters.  These parameters are required to define"); 
   t_output(3,2,
      "each map projection.  Refer to the GEOMPOD documentation for further");
   t_output(4,2,
      "details.  Each of these parameters may be edited individually by ");
   t_output(5,2,
      "specifying the the code PP, the letter of the row needed, and the");
   t_output(6,2,
      "column number.  Example PPA2 = -34000000");
   t_output(7,2,
      "A row of parameters may be specified by just using the code PP and");
   t_output(8,2,
      "the letter of the row.  Example PPA = 56000000 -34000000 0");
   }
else if (strncmp(ans,"ppv",3) == 0)
   {
   t_output(2,2,
      "Projection parameters validity.  The validity of the projection");
   t_output(3,2,"parameters value.");
   t_output(4,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(5,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(6,2,"  = IN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"cc") == 0) 
   t_output(2,2,
      "Corner coordinates of the image.  Specify each corner seperately.");

else if ((strncmp(ans,"ul",2) == 0) ||
         (strncmp(ans,"ccul",4) == 0))
   {
   t_output(2,2,
      "Upper left corner coordinates.  The location of the upper left");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strncmp(ans,"ll",2) == 0) ||
         (strncmp(ans,"ccll",4) == 0))
   {
   t_output(2,2,
      "Lower left corner coordinates.  The location of the lower left");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strncmp(ans,"ur",2) == 0) ||
         (strncmp(ans,"ccur",4) == 0))
   {
   t_output(2,2,
      "Upper right corner coordinates.  The location of the upper right");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strncmp(ans,"lr",2) == 0) ||
         (strncmp(ans,"cclr",4) == 0))
   {
   t_output(2,2,"Lower right corner coordinates.  The location of the lower");
   t_output(3,2,
      "right corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if (strncmp(ans,"ccv",3) == 0)
   {
   t_output(2,2,
      "Corner coordinates validity.  The validity of the corner coordinate");
   t_output(3,2,"values.");
   t_output(4,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(5,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(6,2,"  = UN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"pd") == 0)
   {
   t_output(2,2,
      "Projection distance in line and sample direction.  The distance (in ");
   t_output(3,2,
    "the line and sample direction) on the ground that one sample represents.");
   }
else if (strncmp(ans,"pdv",3) == 0)
   {
  t_output(2,2,"Projection distance validity.  The validity of the projection");
   t_output(3,2,"distance value.");
   t_output(4,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(5,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(6,2,"  = UN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"pu") == 0)
   {
   t_output(2,2,
      "Projection measurement units.  The measurement system used for the");
   t_output(3,2,"projection dimensions.");
   t_output(4,2,"The current standard projection units include:");
   t_output(5,2,"radians");
   t_output(6,2,"degrees");
   t_output(7,2,"seconds");
   t_output(8,2,"dms");
   t_output(9,2,"feet");
   t_output(10,2,"meters");
   }
else if (strncmp(ans,"puv",3) == 0)
   {
   t_output(2,2,
   "Projection units validity.  The validity of the projection measurement");
   t_output(3,2,"units.");
   t_output(4,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(5,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(6,2,"  = UN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"ic") == 0)
   {
   t_output(2,2,
      "Line and sample increment.  The line and sample increment factor used");
   t_output(3,2,"for sub-sampling or expansion of the image.  A positive");
   t_output(4,2,"value greater than one represents that the image has been");
   t_output(5,2,"sub-sampled.  A value between zero and one represents that");
   t_output(6,2,"the image has been expanded.  A value of zero or one means");
   t_output(7,2,"that the image has not been sub-sampled or expanded");
   }
else if (strncmp(ans,"icv",3) == 0)
   {
   t_output(2,2,
      "Increment validity.  The validity of the line and sample increment");
   t_output(3,2,"values.");
   t_output(4,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(5,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(6,2,"  = UN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"lm") == 0)
   {
   t_output(2,2,
      "Last modified date and time.  This is the date and time that the image was");
   t_output(3,2,
      "last modified.  See LMD and LMT for correct formats to be used.");
   }
else if ((strcmp(ans,"lmd") == 0) ||
         (strcmp(ans,"d") == 0))
   {
   t_output(2,2,
      "Last modified date.  This is the date that the image was last modified.");
   t_output(3,2,
      "The date should be entered using the 'dd-mmm-yy' format.");
   t_output(4,2,
      "Example:  DECEMBER 31, 1986 should be entered as 31-dec-86 with");
   t_output(5,2,"month in lower case.");
   }
else if ((strcmp(ans,"lmt") == 0) ||
         (strcmp(ans,"t") == 0))
   {
   t_output(2,2,
      "Last modified time.  This is the time that the image was last modified.");
   t_output(3,2,
      "The time should be entered using a twenty-four hour clock; seconds are");
   t_output(4,2,
      "separated by a colon.  Example:  1:05:55 p.m. should be entered");
   t_output(5,2,"as 1305:55.");
   }
else if (strcmp(ans,"sy") == 0)
   {
   t_output(2,2,"System.  This is the system whose data format the image");
   t_output(3,2," adheses to.");
   }

else if (strcmp(ans,"min") == 0)
   {
   t_output(2,2,
      "Minimum value.  The minimum intensity value.  The value specified for");
   t_output(3,2,
    "this field will be placed in the band that is currently being displayed.");
   }
else if (strcmp(ans,"max") == 0)
   {
   t_output(2,2,
      "Maximum value.  The maximum intensity value.  The value specified for");
   t_output(3,2,
    "this field will be placed in the band that is currently being displayed.");
   }
else if ((strncmp(ans,"minv",4) == 0) ||
	 (strncmp(ans,"maxv",4) == 0))
   {
   t_output(2,2,"Min/max validity.  The validity of the minimum and maximum");
   t_output(3,2,"values for the band currently being displayed.");
   t_output(4,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(5,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(6,2,"  = BO:  BOUNDED The validity of the value is bounded.");
   }
else if (strncmp(ans,"ds",2) == 0)
   {
   t_output(2,2,
      "Data source.  The source of the data.  The value specified for");  
   t_output(3,2,
    "this field will be placed in the band that is currently being displayed.");
   t_output(4,2,"The current standard data sources include:");
   t_output(5,2,"dem");
   t_output(6,2,"spot");
   t_output(7,2,"landsat-1, landsat-2, landsat-3, landsat-4, landsat-5");
   t_output(8,2,"landsat-7, noaa-6, noaa-7, noaa-8, noaa-9, noaa-10, noaa-11");
   }
else if (strncmp(ans,"st",2) == 0)
   {
   t_output(2,2,
      "Sensor type.  The type of sensor or instrument.  The value");
   t_output(3,2,
      "specified for this field will be placed in the band that is currently");
   t_output(4,2,"being displayed.");
   t_output(5,2,"The current standard sensor types include:");
   t_output(6,2,"dem-ascii");
   t_output(7,2,"dem-binary");
   t_output(8,2,"dted");
   t_output(9,2,"dtt");
   t_output(10,2,"mss");
   t_output(11,2,"rbv");
   t_output(12,2,"tm");
   }
else if (strncmp(ans,"cd",2) == 0)
   {
   t_output(2,2,"Capture direction.  The direction of the capture process.");
   t_output(3,2,"Example:  ascending or descending");
   }
else if (strncmp(ans,"da",2) == 0)
   {
   t_output(2,2,
      "Capture date.  The date on which the data was captured.  The");
   t_output(3,2,
      "date should be entered using the 'dd-mmm-yy' format.");
   t_output(4,2,
      "Example:  December 31, 1986 should be entered as 31-dec-86");
   t_output(5,2,"(with month in lower case).");
   }
else if (strncmp(ans,"ti",2) == 0)
   {
   t_output(2,2,
      "Capture time.  The time at which the data was captured.  The");
   t_output(3,2,
      "time should be entered using a twenty-four hour clock; seconds");
   t_output(4,2,
      "are separated by a colon.  Example:  1:05:55 p.m. should be entered");
   t_output(5,2,"as 1305:55.");
   }

/*  A blank as a code will display the general help statements
--------------------------------------------------------------*/
else if (strncmp(ans," ",1) == 0)
   {
   t_output(2,2,
      "Each of the DDR fields may be edited by specifying the upper case");
   t_output(3,2,
      "letters of a specific field name, followed by an equal sign(=),");
   t_output(4,2,
      "followed by the new value.  Edit codes may be entered in upper or");
   t_output(5,2,"lower case.");
   t_output(6,2,
      "Example:  To change the zone code field to '0' enter ZC = 0.");
   t_output(7,2," ");
   t_output(8,2,
      "The (H)elp command is used to retreive information on any of the");
   t_output(9,2,
      "DDR fields.  Specify the (H)elp command using the following syntax.");
   t_output(10,2,
      "Example: To recieve help on projection codes enter H PC.");
   t_output(11,2," ");
   t_output(12,2,
      "The (P)age command is used to page through the band specific fields.");
   t_output(13,2," ");
   t_output(14,2,
      "The (Q)uit command is used to exit the EDITDDR module.");
   t_output(15,2," ");
   t_output(16,2,
      "The (W)rite command is used to write the changes made to the DDR");
   t_output(17,2,
      "fields to the DDR file associated with the image specified by");
   t_output(18,2,
      "Image Name.  The DDR file will not be changed unless the (W)rite");
   t_output(19,2,
      "command is involked."); 
   } 

/*  An invalid code was entered, display the error message
----------------------------------------------------------*/
else
   {
   t_output(2,2,"[editddr-code] Invalid help command code");
   t_output(3,2," ");
   t_output(4,2,"An invalid help command code was entered.  Retype the");
   t_output(5,2,"command with the correct code.  The upper case letters");
   t_output(6,2,"or a combination of the upper case letters should be used as");
   t_output(7,2,"the edit command code");
   }

/*  Output the return command line.
-----------------------------------*/
t_output(*line,2,retrn);

/*  Wait for the user to hit the return key to continue.
--------------------------------------------------------*/
t_input(*line,37,temp,&term);
if (term == P_OVER)
   t_output(*line - 2,2,"[editddr-input] Input is larger than buffer size");

}  /* ddrhelp */
