/******************************************************************************
NAME: DSPCOM

PURPOSE: DSPCOM is used to display and parse the command line.

PROGRAM HISTORY:
Version	Date       Author       Request
-------	----       ------       -------
1.0     10/87      B.Ailts      Initial development
1.1     07/88      B.Ailts      Las 5.0 standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
   Run under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
  The descriptors and other variables are initialized.
  The band indepenent fields are retrieved and displayed.  
  The band dependent fields are retrived from the DDR file.  
  Until the quit command is issued
     The command line is displayed.  
     The input string is parsed and the correct subroutine is called.  
     If the page command is issued, the next page is displayed.  
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"
#include "editddr.h"

void dspcom
(
  char *hostddr,                   /* I: Host ddr name */
  char *taeout,                    /* I: TAE image name */
  long *write_flag                 /* O: Write flag */
)
{

struct DDR ddr;				/*  integer portion of DDR	*/
struct BDDR bddr[MAXBND];		/*  band portion of DDR	*/

long ans_length;			/*  length of command		*/
long curbnd;				/*  curent band			*/
long edit_flag;				/*  edit flag			*/
long fatal = 0;				/*  fatal error code		*/
long i;					/*  counter--			*/
long line;				/*  number of lines on terminal	*/
long mode;				/*  lsstat mode argument	*/
long nonfatal = 1;			/*  nonfatal error code		*/
long status;				/*  status of in_term		*/
long term;				/*  terminator of input string  */
long totbnd;				/*  total bands in DDR file	*/

char ans[81];				/*  buffer for input string	*/
char blank70[71];			/*  buffer for blank line	*/
char com[80];				/*  buffer for command line	*/
char *ptr;				/*  pointer to char. in input	*/

/*  Initialize the terminal for output
--------------------------------------*/
edtint(&line);

/*  Initialize variables
------------------------*/
curbnd = 0;
edit_flag = FALSE;
strcpy(com,"COMMANDS: (H)ELP (P)AGE (Q)UIT (W)RITE");
strcpy(blank70,"                                                           ");

/*  Check for the existance of the ddr file
-------------------------------------------*/
mode = 0;
status = c_lsstat(hostddr,&mode);
if (status != E_SUCC)
   {
   c_errmsg("Input file could not be found","editddr-input",&nonfatal);
   c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
   }

/*  Retrieve the band independant fields
----------------------------------------*/
status = c_getddr(hostddr,&ddr);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","editddr-fatal",&fatal);

/*  display the band independant fields to terminal
---------------------------------------------------*/
dstimg(taeout,&ddr);
totbnd = ddr.nbands;

/*  Retreive the band dependant fields
--------------------------------------*/
for (i = 1; i < (totbnd + 1); i++)
   {
   status = c_getbdr(hostddr,&(bddr[i - 1]),&i);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
   }

/*  Reset to temporary descriptors to beginning of the buffer
-------------------------------------------------------------*/
do  	/*  while the input string is not equal to quit */
   {
   /*  output the command line
   ---------------------------*/
   t_output(line - 1,2,blank70);
   t_output(line - 1,2,com);
   t_output(line,2,blank70);

   /*  Retrieve the input string
   -----------------------------*/
   t_input(line,2,ans,&term);
   if (term == P_OVER)
      t_output(line - 2,2,"[editddr-input] Input is larger than buffer size");

   /*  Make answer all lower case
   ------------------------------*/
   ans_length = (long)strlen(ans);
   c_up2low(ans,&ans_length);

   /*  blank out the err message line
   ----------------------------------*/
   t_output(line - 2,2,blank70);

   /*  Based on input string branch to correct subroutine
   ------------------------------------------------------*/
   if (strcmp(ans,"q") == 0)
      {
      if (edit_flag)
         disquit(ans,&line);
      else
	 {
         t_output(line - 1,2,blank70);
         t_output(line,2,blank70);
         t_pos(line,1);
	 }
      }
   else if (strncmp(ans,"h",1) == 0) 
      {
      ptr = ans;
      while (strncmp(ptr," ",1) != 0)
         (ptr) ++;
      if (strlen(ptr) > 1)
         (ptr) ++;
      ddrhelp(ptr,&line);

      /*  Re-display the screen after help is displayed
      -------------------------------------------------*/
      if (curbnd == 0)
         dstimg(taeout,&ddr);
      else
         dstbnd(taeout,&(bddr[curbnd - 1]));
      }
   else if (strcmp(ans,"w") == 0)
      {
      status = writeddr(hostddr,&ddr,bddr);
      if (status != E_SUCC)
         {
         t_pos(line - 2,2);
         c_errmsg("Error writing to DDR file","editddr-write",&nonfatal);
         c_errmsg("Fatal error encountered","editddr-fatal",&fatal);
	 }
      else
	 {
         /*  output informational message
         --------------------------------*/
         t_output(line - 2,2,
	          "[editddr-informational] Write completed                ");
         *write_flag = TRUE;
	 edit_flag = FALSE;
	 }  /*  write was successful  */
      }  /*  write option  */
   else if (strcmp(ans,"p") == 0)
      {
      if (curbnd == totbnd)
         {
         dstimg(taeout,&ddr);
         curbnd = 0;
         }
      else
         {
         curbnd++;
         dstbnd(taeout,&(bddr[curbnd - 1]));
         } /* else curbnd > totbnd */
      }  /* if print */
   else 
      {
      if (curbnd == 0)
         edtddr(ans,taeout,hostddr,&line,&ddr,&bddr[0],&edit_flag);
      else
         edtddr(ans,taeout,hostddr,&line,&ddr,&bddr[curbnd - 1],&edit_flag);
      }
   } /* do while not quit */
while (strncmp(ans,"q",1) != 0);

} /* dspcom */
