/******************************************************************************
NAME: DSTBND

PURPOSE: DSTBND displays the band dependant fields to the terminal.

PROGRAM HISTORY:
Version	Date       Author       Request
-------	----       ------       -------
1.0     10/87      B.Ailts      Initial development
1.1     07/88	   B.Ailts	LAS 5.0 standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
   Run under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
  Each of the line strings are formated with the information provided by the
   input arguements.  
  Each of the line strings are then output to a specific line on the terminal.
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "editddr.h"

void dstbnd
(
  char *taeout,                    /* I: TAE image name */
  struct BDDR *bddr                /* I: Current ddr field */
)
{
  char line[81];                   /* Buffer for line */
  char valstr[19];

  void mmval(long,char []);

  /* Clear the screen
  -------------------*/
  t_clear();

  /* Format each line of the output
  ---------------------------------*/
  sprintf(line,"Image Name:%-60s",taeout);
  t_output(1,2,line);
  sprintf(line,"   band no:%-3d",bddr->bandno);
  t_output(2,2,line);
  mmval(bddr->valid,valstr);
  sprintf(line,"        MINimum:%-1.14lE                     %-15s",
          bddr->minval,valstr);
  t_output(4,2,line);
  sprintf(line,"        MAXimum:%-1.14lE                     %-15s",
          bddr->maxval,valstr);
  t_output(5,2,line);
  sprintf(line,"    Data Source:%-32s",bddr->source);
  t_output(7,2,line);
  sprintf(line,"    Sensor Type:%-32s",bddr->instrument);
  t_output(8,2,line);
  sprintf(line,"Capt. Direction:%-64s",bddr->direction);
  t_output(9,2,line);
  sprintf(line,"           DAte:%-9s",bddr->date);
  t_output(10,2,line);
  sprintf(line,"           TIme:%-7s",bddr->time);
  t_output(11,2,line);
}

/******************************************************************************
NAME: MMVAL

PURPOSE: Returns the correct string for the input valid flag

ALGORITHM DESCRIPTION:
  MMVAL returns the correct string for the input valid code.
******************************************************************************/
void mmval
(
  long input,                     /* Input valid code */
  char str[]                      /* Output valid string */
)
{
  /* Copy the correct string based on the valid code
  --------------------------------------------------*/
  switch (input)
  {
    case INVAL:
      sprintf(str,"Valid:INVALID");
      break;
    case VALID:
      sprintf(str,"Valid:VALID");
      break;
    case BOUND:
      sprintf(str,"Valid:BOUNDED");
      break;
    default:
      sprintf(str,"Valid:UNACCEPTABLE");
  }
}
