/******************************************************************************
NAME: DSTIMG

PURPOSE: DSTIMG is used to display the contents of the DDR file to 
         the terminal.

PROGRAM HISTORY:
Version	Date       Author       Request
-------	----       ------       -------
1.0     10/87      B.Ailts      Initial development
1.1     07/88      B.Ailts      LAS 5.0 standards
1.2     10/89      B.Ailts      Changed last used to last modified

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
    Run under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
  Each output line string is formated with the information provided by the
  input arguments
  Each of the line strings is then output to the terminal.
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "editddr.h"

void dstimg
(
  char *taeout,                    /* I: TAE image name */
  struct DDR *ddr                  /* I: Integer portion of DDR */
)
{
char line[81];				/*  buffer of output line 	*/
char proj_str[45];			/*  buffer of projection string */
char dtm_str[46];			/*  buffer of datum string      */
char vald[16];

void pntval(long input,char str[16]);

/*  Clear the terminal
----------------------*/
t_clear();

/*  format each line of output
------------------------------*/
sprintf(line,"  Image Name:%-60s",taeout);
t_output(1,2,line);
switch (ddr->dtype)
    {
   case EBYTE: sprintf(line,
"          NL:%-5d         NS:%-5d          NB:%-3d        DType:BYTE",
     ddr->nl,ddr->ns,ddr->nbands);
     break;
   case EWORD:sprintf(line,
"          NL:%-5d         NS:%-5d          NB:%-3d        DType:INTEGER*2",
     ddr->nl,ddr->ns,ddr->nbands);
     	    break;
   case ELONG:sprintf(line,
"          NL:%-5d         NS:%-5d          NB:%-3d        DType:INTEGER*4",
     ddr->nl,ddr->ns,ddr->nbands);
     	    break;
   case EREAL:sprintf(line,
"          NL:%-5d         NS:%-5d          NB:%-3d        DType:REAL*4",
     ddr->nl,ddr->ns,ddr->nbands);
     	    break;
   default:sprintf(line,
"          NL:%-5d         NS:%-5d          NB:%-3d        DType:%-d",
     ddr->nl,ddr->ns,ddr->nbands,ddr->dtype);
            break;
   }
t_output(2,2,line);

sprintf(line,"   Last Modified:        Date:%-10s   Time:%-10s SYstem:%-10s",
        ddr->last_used_date,ddr->last_used_time,ddr->system);
t_output(3,2,line);

pntval(ddr->valid[DDPCV],vald);
c_prostr(&(ddr->proj_code),proj_str);
sprintf(line,
        "  %-9s%-*s%-*s%-15s","Proj. Code:",
        strlen(proj_str),proj_str,(46 - strlen(proj_str))," ",vald);
t_output(4,2,line);

pntval(ddr->valid[DDZCV],vald);
sprintf(line,
    "   Zone Code:%-4d                                          %-15s",
    ddr->zone_code,vald);
t_output(5,2,line);

pntval(ddr->valid[DDDCV],vald);
c_dtmstr(&(ddr->datum_code),dtm_str);
sprintf(line,
        "  %-9s%-*s%-*s%-15s","Datum Code:",
        strlen(dtm_str),dtm_str,(46 - strlen(dtm_str))," ",vald);
t_output(6,2,line);

pntval(ddr->valid[DDPPV],vald);
sprintf(line,
    "  Proj. Parm:                                              %-15s",vald);
t_output(7,2,line);
sprintf(line,
    "  A:   %1.14lE   %1.14lE   %1.14lE",
   ddr->proj_coef[0],ddr->proj_coef[1],ddr->proj_coef[2]);
t_output(8,2,line);
sprintf(line,
    "  B:   %1.14lE   %1.14lE   %1.14lE",
      ddr->proj_coef[3],ddr->proj_coef[4],ddr->proj_coef[5]);
t_output(9,2,line);
sprintf(line,
    "  C:   %1.14lE   %1.14lE   %1.14lE",
      ddr->proj_coef[6],ddr->proj_coef[7],ddr->proj_coef[8]);
t_output(10,2,line);
sprintf(line,
    "  D:   %1.14lE   %1.14lE   %1.14lE",
      ddr->proj_coef[9],ddr->proj_coef[10],ddr->proj_coef[11]);
t_output(11,2,line);
sprintf(line,
    "  E:   %1.14lE   %1.14lE   %1.14lE",
      ddr->proj_coef[12],ddr->proj_coef[13],ddr->proj_coef[14]);
t_output(12,2,line);

pntval(ddr->valid[DDCCV],vald);
sprintf(line,
    " Corner Corr:                                              %-15s",vald);
t_output(13,2,line);
sprintf(line,"    ULcorner:%-1.14lE   %-1.14lE",
	ddr->upleft[0],ddr->upleft[1]);
t_output(14,2,line);
sprintf(line,"    URcorner:%-1.14lE   %-1.14lE",
	ddr->upright[0],ddr->upright[1]);
t_output(15,2,line);
sprintf(line,"    LLcorner:%-1.14lE   %-1.14lE",
	ddr->loleft[0],ddr->loleft[1]);
t_output(16,2,line);
sprintf(line,"    LRcorner:%-1.14lE   %-1.14lE",
	ddr->loright[0],ddr->loright[1]);
t_output(17,2,line);

pntval(ddr->valid[DDPDV],vald);
sprintf(line,
    "  Proj. Dist:%-1.14lE   %1.14lE   %-15s",
      ddr->pdist_y,ddr->pdist_x,vald);
t_output(18,2,line);

pntval(ddr->valid[DDPUV],vald);
sprintf(line,
    " Proj. Units:%-10s                                    %-15s",
      ddr->proj_units,vald);
t_output(19,2,line);

pntval(ddr->valid[DDINCV],vald);
sprintf(line,
    "   InCrement:%-1.14lE   %1.14lE   %-15s",
      ddr->line_inc,ddr->sample_inc,vald);
t_output(20,2,line);

sprintf(line,
    " Master Corr:%-5d   %-5d    ",
      ddr->master_line,ddr->master_sample);
t_output(21,2,line);

/*  Output each of the line strings to the terminal
---------------------------------------------------*/
t_output(21,2,line);

}

/******************************************************************************
NAME: PNTVAL

PURPOSE: Returns the correct string for the input valid flag

ALGORITHM DESCRIPTION:
  PNTVAL returns the correct string for the input valid code.
******************************************************************************/
void pntval
(
  long input,           /* Input valid code */
  char str[16]          /* Output valid string */
)
{

  /* Copy the correct string based on the valid code
  --------------------------------------------------*/
  switch (input)
  {
    case INVAL:
      sprintf(str,"Valid:INVALID");
      break;
    case VALID:
      sprintf(str,"Valid:VALID");
      break;
    case UNKNOW:
      sprintf(str,"Valid:UNKNOWN");
      break;
    default:
      sprintf(str,"Valid:UNACCEPTABLE");
  }
}
