/******************************************************************************* 
NAME:  EDITDDR 

PURPOSE:  To edit a DDR file.

PROGRAM HISTORY:
Version		Date	Author		Code/contr	Request
-------		----	------		----------	-------
  1.0		11/87	B.Ailts		   CSB		original development
  1.1		12/87	B.Ailts		   CSB		Replace DESC arguments
							with char. strings
							Change include dir.
							specifications
  1.2		07/88   B.Ailts		   CSB		LAS5.0 standards
  1.3           10/89   B.Ailts		   CSB		lower cased input name
						        now works with aliases
							Changed last accessed to
							last modified
  1.4		08/90	D.Etrheim	   CSB		ADP #5521 Changed to 
							accept the projection
							name "utm". Changed to 
						        accept upper case 
						        projection types for
							-TUTOR.
  1.5		05/92	S.Nelson	   CSB		Corrected error
							entering proj. type
						  	in both tutor and
							screen mode.
  1.6		06/92	S. Nelson	   CSB		Added map projections
							Hammer and Robinson.
							Changed STYPE to SENSOR.
  1.7         	10/09	D.Etrheim/CSB	Modified to allow negative zone codes.
                                     	Needed for UTM below the equator.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
TAE is initialized 
input TAE image name and subcommand are retrieved from the parameter block
Depending on the subcommand chosen the following procedures are followed:

-SCREEN
   The host ddr file name is calculated from the input TAE image name
   The contents of this ddr file are read and displayed to the terminal.  
   1. The user can edit any of the fields except the individual band 
      numbers by using the codes shown on the screen in upper case.  
   2. The user can also receive on-line help on any of the fields by 
      using the codes shown in upper case. 
   3. The user can page through all the ddr information.  
   4. The user can write any changes he made to the ddr file.  The
      host ddr file name is re-calculated before writing to the file
      in case the user changed the input image name.
   5. The user may exit this routine.  The completion message will
      be displayed with an output file name only when a ddr file
      is written to.

-TUTOR
   The host ddr file name is calculated from the input TAE image name
   The contents of this ddr file are read in
   Each of the parameters are retrieved from the parameter block
   If the parameter has a value of NULL then the DDR field that pertains to 
    that parameter will not be changed
   If the parameter has a value, then the DDR field that pertains to that 
    parameter will be changed to the new input value
   After all the parameters are retrieved, the fields are written to the DDR 
    file
   After all the fields are written, the completion message is displayed.

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "alias.h"
#include "labserv.h"
#include "editddr.h"

void main
(
  void
)
{  /* main */

struct PARBLK parb;		/*  TAE parameter block		*/
struct VARIABLE *val;			/*  pointer to parameter in vblock */

long one = 1;				/*  one				*/
long status;				/*  status of calls		*/
long std;				/*  standard output		*/
long write_flag = 0;			/*  write flag			*/
long zero = 0;				/*  zero			*/

char ext[4];				/*  'ddr' file extension	*/
char hosin[CMLEN];			/*  buffer for host image name  */
char hostddr[CMLEN];			/*  buffer for host ddr name	*/
char taein[CMLEN];			/*  buffer for TAE image name	*/
static char temp[CMLEN];                /*  temp buffer for TAE image name */
char subcmd[12];			/*  buffer for subcommand parm. */

/*  Initialize TAE and parameter block
--------------------------------------*/
c_inlas(&parb,&std);

/*  Initialize variables
------------------------*/
strcpy(ext,"ddr");

/*  Retreive the input TAE image name
-------------------------------------*/
val = p_find(&parb,"IN");
strcpy(temp,SVAL(*val,0));
 
/* Find alias of input TAE name
-------------------------------*/
status = c_getalias(temp,taein);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","dspddr-fatal",&zero);

/*  Calculate the input host image name
---------------------------------------*/
status = c_cmhost(taein,hosin);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","editddr-fatal",&zero);

/*  Create the host DDR file name
---------------------------------*/
c_lsmknm(hosin,ext,hostddr);

/*  Retrieve the subcommand
---------------------------*/
val = p_find(&parb,"_SUBCMD");
strcpy(subcmd,SVAL(*val,0));

/*  Branch on the basis of the subcommand
-----------------------------------------*/
if (strcmp(subcmd,"TUTOR") == 0)
   {
   edtbat(&parb,hostddr);
   write_flag = 1;
   }
else
   {
   dspcom(hostddr,taein,&write_flag);
   }

/* Output the completion message.  Output file name if a write was completed
----------------------------------------------------------------------------*/
if (!write_flag)
   c_complt("EDITDDR"," ",&zero);
else
   c_complt("EDITDDR",hostddr,&one);

} /* main */
