/******************************************************************************
NAME: EDTINT

PURPOSE: EDTINT is used to initialize the terminal output of TAE.

PROGRAM HISTORY:
Version Date       Author       Request
------- ----       ------       -------
1.0     10/87      B.Ailts      Initial development
1.0     07/88      B.Ailts      Las 5.0 standards

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
     Run under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
  This function initializes the terminal for TAE output
  The terminal characteristics returned from INIT_TERM are checked to be 
    sure this module can use this terminal
  A fatal error is output if the terminal characteristics are not correct.
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "editddr.h"

void edtint
(
  long *lines                      /* I: Number of lines on terminal */
)
{
  long cols;                       /* Number of columns on terminal */
  long type;                       /* Acceptable terminal type */

  /* Initialize the terminal for TAE output
  -----------------------------------------*/
  t_init(lines,&cols,&type);

  /* Output a fatal error if the terminal does not have the correct
     characteristics to run this module using the -PROMPT subcommand
  ------------------------------------------------------------------*/
  if (type == T_NOTCRT)
  {
    c_lasmsg(FALSE,"editddr-terminal","Terminal is not supported by TAE I/O");
    c_lasmsg(TRUE,"editddr-fatal","Fatal error encountered");
  }

  if (type == T_NOTTERM)
  {
    c_lasmsg(FALSE,"editddr-device","Device is not a terminal");
    c_lasmsg(TRUE,"editddr-fatal","Fatal error encountered");
  }

  if (*lines < 24)
  {
    c_lasmsg(FALSE,"editddr-display",
             "Terminal has too few lines of display area");
    c_lasmsg(TRUE,"editddr-fatal","Fatal error encountered");
  }
}
