#include "las.h"
#include "geompak.h"
#include "protos.h"

int check
(
    struct GEOGRID *grid,			/* Geometric mapping package */
    long *line				/* number of lines on terminal*/
)
/******************************************************************************
*
*  NAME:       CHECK
*
*  FUNCTION:   To verify that parameters have been entered if creating an new
*              grid file.
*
*  ALGORITHM DESCRIPTION:
*  If any of the required parameters are zero or null, issue an error message.
*
******************************************************************************/
{ /* check */
long i;					/* loop index			*/
long flag;				/* flag if coefficients were entered */

flag = FALSE;

/* Check Model method
---------------------*/
if (strlen(grid->model) == 0)
   {                              
   t_output(*line - 2,2,
    "[editgrid-create] MM is required when a grid is being created");
   return(E_FAIL);
   }

/* Check number of coefficients 
-------------------------------*/
if (grid->ncoeff == 0)
   {
   t_output(*line - 2,2,
       "[editgrid-create] NC is required when a grid is being created");
   return(E_FAIL);
   }

/*  If one of transformation coefficients is nonzero, assume success
--------------------------------------------------------------------*/ 
for (i = 0; i < grid->ncoeff * 2; i++)
   if (grid->coeffs[i] != 0.0) 
       flag = TRUE;
if (!flag)
   {
   t_output(*line - 2,2,
      "[editgrid-create] FP and SP are required when a grid is being created");
   return(E_FAIL);
   }

return(E_SUCC); 
} /* check */
