/******************************************************************************

NAME:	DSPCOM

FUNCTION: DSPCOM is used to display and parse the command line.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         05/89      D.Etrheim    initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
The terminal and variables are initialized.
The fields are displayed to the terminal.
Until the quit or write command is issued
   The command line is displayed.  
   The input string is parsed and the correct subroutine is called.  
   If the Write command is invoked and the input grid file did not exist
      check that required grid fields were entered.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

void dspcom
(
    struct GEOGRID *grid,			/*  Geometric mapping grid	*/
    char *hosin,				/*  input TAE grid file name	*/
    char *hosout,				/*  output host grid file name	*/
    long *write_flag,			/*  write flag			*/
    long *exist_flag			/*  flag for file existence	*/
)

{ /* dspcom */

long length;				/*  length of command		*/
long edit_flag = FALSE;			/*  edit flag			*/
long fatal = -1;			/*  fatal error code		*/
long line;				/*  number of lines on terminal	*/
long page = 1;				/*  flag for paging		*/
long nonfatal = 1;			/*  nonfatal error code		*/
long status = E_SUCC;			/*  status flag			*/
long term;				/*  terminator of input string  */

char taein[CMLEN];			/*  input TAE grid file name	*/
char ans[81];				/*  buffer for input string	*/
char blank70[71];			/*  buffer for blank line	*/
char com[80];				/*  buffer for command line	*/

/*  Initialize the terminal for output
--------------------------------------*/
edtint(&line);

/*  Initialize variables
------------------------*/
strcpy(com,"COMMANDS: (H)ELP (P)AGE (Q)UIT (W)RITE");
strcpy(blank70,
"                                                                      ");

/*  display the grid fields on the  terminal
--------------------------------------------*/
if (*exist_flag)
   strcpy(taein, hosin); 
else
   taein[0] = (char) NULL;
dstimg(taein,grid,&page); 

/* do while the input string is not equal to quit 
-------------------------------------------------*/
do  	
   {
   /*  output the command line
   ---------------------------*/
   t_output(line - 1,2,blank70);
   t_output(line - 1,2,com);
   t_output(line,2,blank70);

   /*  Retrieve the input string
   -----------------------------*/
   t_input(line,2,ans,&term);
   if (term == P_OVER)
      t_output(line - 2,2,"[editgrid-input] Input is larger than buffer size");

   /*  Make answer all lower case
   ------------------------------*/
   length = (long) strlen(ans);
   c_up2low(ans,&length);

   /*  blank out the error message line
   ------------------------------------*/
   t_output(line - 2,2,blank70);

   /*  Based on input string branch to correct subroutine
   ------------------------------------------------------*/
   if (strcmp(ans,"q") == 0)
      {
      if (edit_flag)
         disquit(ans,&line);
      else
	 {
         t_output(line - 1,2,blank70);
         t_output(line,2,blank70);
         t_pos(line,1);
	 }
      }
   else if (strncmp(ans,"h",1) == 0) 
      {
      gridhelp(ans,&line); 
/* Re-display the screen after help is displayed
   ------------------------------------------------*/
      dstimg(taein,grid,&page); 
      }
   else if (strncmp(ans,"w",1) == 0)
      {
      if (!*exist_flag)
          status = check(grid,&line);
      if (status == E_SUCC)
         {
         status = c_putgrd(hosout,grid);
         if (status != E_SUCC)
            {
            t_pos(line,2);
            c_errmsg("Error writing to grid file","editgrid-write",&nonfatal);
            c_errmsg("Fatal error encountered","editgrid-fatal",&fatal); 
	    }
         else
	    {
            /* successful write, now clear command and input line before exit 
            -----------------------------------------------------------------*/
            t_output(line - 1,2,blank70);
            t_output(line,2,blank70);
            t_pos(line,1);
            *write_flag = TRUE;
            strcpy(ans,"q");
	    }   
         }
      }  
   else if (strcmp(ans,"p") == 0)
          {
          if (page)
             page = 0;
          else
             page = 1;
          dstimg(taein,grid,&page); 
          }
   else 
         edtgrid (ans,&line,grid,&edit_flag);
   } /* do while not quit */
while (strcmp(ans,"q") != 0);

return;
} /* dspcom */
