/******************************************************************************

NAME:	DSTIMG

FUNCTION: DSTIMG is used to display the contents of the grid file to 
	  the terminal.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         5/89       D.Etrheim	initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
Each output line string is formated with the information provided by the
input arguments
Each of the line strings is then output to the terminal.

******************************************************************************/
#include "las.h"
#include "geompak.h"
#include "protos.h"

void dstimg
(
    char *taein,				/*  input TAE grid name	*/
    struct GEOGRID *grid,			/*  Geometric mapping grid	*/
    long *flag				/*  flag for paging		*/
)

{  /* dstimg */

char line[81];				/*  buffer of output line 	*/
char proj_str[45];			/*  buffer of projection string */
char dtm_str[CMLEN];		/*  buffer of datum string	*/
char vald[16];				/*  validity string		*/

/*  Clear the terminal
----------------------*/
t_clear();

/*  format each line of output
------------------------------*/
if (*flag)
   {
   sprintf(line,"Grid Name:%-60s",taein);
   t_output(1,2,line);

   c_prostr(&(grid->code),proj_str);
   sprintf(line,"Proj. Code: %-*s",strlen(proj_str),proj_str);
   t_output(3,2,line);

   dtm_str[0]='\0';
   c_dtmstr(&(grid->datum),dtm_str);
   sprintf(line,"Datum Code: %-s",dtm_str);
   t_output(4,2,line);

   sprintf(line,"Proj. Zone: %-d",grid->zone);
   t_output(5,2,line);

   pntval(grid->proj_valid,vald);
   sprintf(line,"Proj. Validity: %-15s",vald);
   t_output(6,2,line);

   sprintf(line, "Proj. Parm:");
   t_output(7,2,line);
   sprintf(line,
     "A:   %1.14E   %1.14E   %1.14E",
     grid->projprms[0],grid->projprms[1],grid->projprms[2]);
   t_output(8,2,line);
   sprintf(line,
     "B:   %1.14E   %1.14E   %1.14E",
      grid->projprms[3],grid->projprms[4],grid->projprms[5]);
   t_output(9,2,line);
   sprintf(line,
     "C:   %1.14E   %1.14E   %1.14E",
      grid->projprms[6],grid->projprms[7],grid->projprms[8]);
   t_output(10,2,line);
   sprintf(line,
     "D:   %1.14E   %1.14E   %1.14E",
      grid->projprms[9],grid->projprms[10],grid->projprms[11]);
   t_output(11,2,line);
   sprintf(line,
     "E:   %1.14E   %1.14E   %1.14E",
      grid->projprms[12],grid->projprms[13],grid->projprms[14]);
   t_output(12,2,line);

   sprintf(line,"Corner Corr:");
   t_output(14,2,line);
   sprintf(line,"ULcorner: %1.14E   %1.14E",
	grid->corners[0],grid->corners[1]);
   t_output(15,4,line);
   sprintf(line,"URcorner: %1.14E   %1.14E",
	grid->corners[4],grid->corners[5]);
   t_output(16,4,line);
   sprintf(line,"LLcorner: %1.14E   %1.14E",
	grid->corners[2],grid->corners[3]);
   t_output(17,4,line);
   sprintf(line,"LRcorner: %1.14E   %1.14E",
	grid->corners[6],grid->corners[7]);
   t_output(18,4,line);

   sprintf(line,
     "Proj. Dist: %-1.14E   %1.14E",grid->pdist[0],grid->pdist[1]);
   t_output(20,2,line);

   sprintf(line,
     "Proj. Units: %-10s",grid->units);
   t_output(21,2,line);
   } /* page one */
else
   {
   sprintf(line,"Model Method: %-s",grid->model);
   t_output(2,2,line);
   sprintf(line,"Number of Coefficients: %d",grid->ncoeff);
   t_output(3,2,line);
   dstcoef(grid);
   } /* page two */

return;
}  /* dstimg */

/******************************************************************************

NAME:	PNTVAL

FUNCTION: Returns the correct string for the input valid flag

ALGORITHM DESCRIPTION:
PNTVAL returns the correct string for the input valid code.

******************************************************************************/

void pntval
(
    long input,			/*  input valid code			*/
    char str[16]			/*  output valid string			*/
)

{  /* pntval */

/*  copy the correct string based on the valid code
---------------------------------------------------*/
switch (input)
   {
   case GEO_NOTVALID: sprintf (str,"INVALID");
	    break;
   case GEO_VALID: sprintf (str,"VALID");
	    break;
   default: sprintf (str,"UNACCEPTABLE");
   } /* switch (*input) */

return;
}  /* pntval */
