/******************************************************************************

NAME:	EDTINT

FUNCTION: EDTINT is used to initialize the terminal output of TAE.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         05/89      D.Etrheim    initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
This function initializes the terminal for TAE output
The terminal characteristics returned from TERM_INIT are checked to be 
 sure this module can use this terminal
A fatal error is output if the terminal characteristics are not correct.

******************************************************************************/
#include "las.h"
#include "geompak.h"
#include "protos.h"

void edtint
(
    long *lines				/*  number of lines on terminal	*/
)

{  /* edtint */

long cols;				/*  number of columns on terminal*/
long fatal = -1;			/*  fatal error code		 */
long nonfatal = 1;			/*  nonfatal error code		 */
long type;				/*  acceptable terminal type	 */

/*  Initialize the terminal for TAE output
------------------------------------------*/
t_init(lines,&cols,&type);

/*  Output a fatal error if the terminal does not have the correct
characteristics to run this module using the -PROMPT subcommand
------------------------------------------------------------------*/
if (type == T_NOTCRT)
  {
  c_errmsg("Terminal is not supported by TAE I/O","editgrid-terminal",&nonfatal);
  c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
  }

if (type == T_NOTTERM)
   {
   c_errmsg("Device is not a terminal","editgrid-device",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }

if (*lines < 24 )
   {
   c_errmsg("Terminal has too few lines of display area",
           "editgrid-display",&nonfatal);
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }

return;
} /* edtint */
