/******************************************************************************* 
NAME: GETPAR 

PURPOSE:  Get user entered parameters from the TAE parameter block

PROGRAM HISTORY:
Version		Date	Author		Request
-------		----	------		-------
  1.0		5/89	D.Etrheim	initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
If an input grid file was entered, get the host file name
Get the output grid file name and the subcommand from the parameter block

*************************************************************************/
#include "las.h"			
#include "geompak.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *parb,			/*  TAE parameter block		*/
    char *hosin,				/*  Input TAE name		*/
    char *hosout,				/*  output grid file name	*/
    char *subcmd,				/*  subcommand parameter	*/
    long *exist_flag			/*  flag for file existence	*/
)

{  /* getpar */

struct VARIABLE *val;			/*  pointer to parameter in vblock */
long fatal = -1;			/*  fatal error flag for c_errmsg */
long zero = 0;				/*  zero			*/
long mode;				/*  flag for getnam routine     */

/*  Retrieve the input grid name
-------------------------------------*/
val = p_find(parb,"ingrid");
if ((*val).v_count == 0)
    *exist_flag = FALSE;
else
   {
   mode = IREAD;
   if (c_getnam(parb,"INGRID",&zero,hosin,"grid",&mode) != E_SUCC)
       c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);
   }

/* Retrieve the output grid name 
--------------------------------*/
mode = IWRITE;
if (c_getnam(parb,"OUTGRID",&zero,hosout,"grid",&mode) != E_SUCC)
   c_errmsg("Fatal error encountered","editgrid-fatal",&fatal);

/*  Retrieve the subcommand
---------------------------*/
val = p_find(parb,"_SUBCMD");
strcpy(subcmd,SVAL(*val,0));

return;
} /* getpar */
