/*******************************************************************************
NAME:  GRIDHELP 

PURPOSE:  To display the help for EDTGRID.

PROGRAM HISTORY:
Version		Date	Author		Request
-------		----	------		-------
  1.0		05/89	D.Etrheim	initial development
  1.1		06/92	S.Nelson	Added the proj code to the list of
					map projections.
					Added map projections Hammer and
					Robinson.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
Receives the number of lines in the screen and the code for which the help 
 is needed.  
Based on the code, the help messages are displayed to the terminal.  
A code of a blank will display general help message.
The screen is displayed until the return key is hit and then the
 control is returned to the calling routine.
*******************************************************************************/
#include <stdio.h>
#include "las.h"			/* general las includes 	*/
#include "geompak.h"
#include "protos.h"

void gridhelp
(
    char ans[],	  			/* input help code		*/
    long *line				/* number of lines in the terminal */
)

{  /* gridhelp */

long term;				/* terminator of t_input	*/
char retrn[60];				/* buffer for return message	*/
char temp[3];				/* buffer for return input	*/

/*  Initialize variables
------------------------*/
strcpy(retrn,"Press the return key when finished");

/*  Clear the terminal
----------------------*/
t_clear();

/*  Compare the input code with the known codes and output the correct
    help message to the terminal.
----------------------------------------------------------------------*/
if (strcmp(ans,"h pc") == 0)
   {
   t_output(2,2,"Projection code.  Refer to the GEOMPOD for further details.");
   t_output(4,2,"Valid projection system codes are:");
   t_output(5,2,"  GEO(0)     = geographic");
   t_output(6,2,"  UTM(1)     = Universal Transverse Mercator");
   t_output(7,2,"  SPCS(2)    = State Plane Coordinates");
   t_output(8,2,"  ALBERS(3)  = Albers Conical Equal Area");
   t_output(9,2,"  LAMCC(4)   = Lambert Conformal Conic");
   t_output(10,2,"  MERCAT(5)  = Mercator");
   t_output(11,2,"  PS(6)      = Polar Stereographic");
   t_output(12,2,"  POLYC(7)   = Polyconic");
   t_output(13,2,"  EQUIDC(8)  = Equidistant Conic");
   t_output(14,2,"  TM(9)      = Transverse Mercator");
   t_output(15,2,"  STEREO(10) = Stereographic");
   t_output(16,2,"  LAMAZ(11)  = Lambert Azimuthal Equal Area");
   t_output(17,2,"  AZMEQD(12) = Azimuthal Equidistant");
   t_output(18,2,"  GNOMON(13) = Gnomonic");
   t_output(19,2,"  ORTHO(14)  = Orthographic");

   /*  Split the help messages between two separate screens
   ---------------------------------------------------------*/
   t_output(*line,2,retrn);
   t_input(*line,37,temp,&term);
   if (term == P_OVER)
      t_output(*line - 2,2,"[editgrid-input] Input is larger than buffer size");
   t_clear();
   t_output(2,2,"  GVNSP(15)  = General Vertical Near-Side Perspective");
   t_output(3,2,"  SNSOID(16) = Sinusoidal");
   t_output(4,2,"  EQRECT(17) = Equirectangular");
   t_output(5,2,"  MILLER(18) = Miller Cylindrical");
   t_output(6,2,"  VGRINT(19) = Van der Grinten");
   t_output(7,2,"  HOM(20)    = Oblique Mercator (Hotine)");
   t_output(8,2,"  ROBIN(21)  = Robinson");
   t_output(9,2,"  SOM(22)    = Space Oblique Mercator (SOM)");
   t_output(10,2,"  ALASKA(23) = Alaska Conformal");
   t_output(11,2,"  GOOD(24)   = Interrupted Goode Homolsine");
   t_output(12,2,"  MOLL(25)   = Mollweide");
   t_output(13,2,"  IMOLL(26)  = Interrupted Mollweide");
   t_output(14,2,"  HAMMER(27) = Hammer");
   t_output(15,2,"  WAGIV(28)  = Wagner IV");
   t_output(16,2,"  WAGVII(29) = Wagner VII");
   t_output(17,2,"  OBEQA(30)  = Oblated Equal Area");
   t_output(18,2,"  31+        = User Defined");
   }
else if (strcmp(ans,"h pz") == 0)
   {
   t_output(2,2,
      "Projection zone.  The zone to use for UTM and State Plane projection"); 
   t_output(3,2,
      "systems.  Refer to the GEOMPOD documentation for further details.");
   }
else if (strcmp(ans,"h dc") == 0)
   {
   t_output(2,2,
      "Datum code.  The datum can be used for setting the semimajor");
   t_output(3,2,
      "axis and eccentricity squared for a particular projection.  Refer to");
   t_output(4,2,"the GEOMPOD documentation for further details.");
   t_output(5,2,"The supported datums are:");
   t_output(6,2,"  0 = Clarke 1866"); 
   t_output(7,2,"  1 = Clarke 1880");
   t_output(8,2,"  2 = Bessel");
   t_output(9,2,"  3 = International 1967");
   t_output(10,2,"  4 = International 1909");
   t_output(11,2,"  5 = WGS 72");
   t_output(12,2,"  6 = Everest");
   t_output(13,2,"  7 = WGS 66");
   t_output(14,2,"  8 = GRS 1980");
   t_output(15,2,"  9 = Airy");
   t_output(16,2,"  10 = Modified Everest");
   t_output(17,2,"  11 = Modified Airy");
   t_output(18,2,"  12 = Walbeck");
   t_output(19,2,"  13 = Southeast Asia");

   /*  Split the help messages between two separate screens
   ---------------------------------------------------------*/
   t_output(*line,2,retrn);
   t_input(*line,37,temp,&term);
   if (term == P_OVER)
      t_output(*line - 2,2,"[editgrid-input] Input is larger than buffer size");
   t_clear();
   t_output(2,2,"  14 = Australian National");
   t_output(3,2,"  15 = Krassovsky");
   t_output(4,2,"  16 = Hough");
   t_output(5,2,"  17 = Mercury 1960");
   t_output(6,2,"  18 = Modified Mercury 1968");
   t_output(7,2,"  19 = Spere of radius - 6370997 meters");
   }
else if (strcmp(ans,"h pv") == 0)
   {
   t_output(2,2,"Projection validity.  The validity of the projection fields.");
   t_output(3,2,"  = IN:  INVALID The value is known not to be valid.");
   t_output(4,2,"  = VAL: VALID   The value is known to be valid.");
   t_output(5,2,"  = UN:  UNKNOWN The validity of the value is not known.");
   }
else if (strcmp(ans,"h pp") == 0)
   {
   t_output(2,2,
      "Projection parameters.  These parameters are required to define"); 
   t_output(3,2,
      "each map projection.  Refer to the GEOMPOD documentation for further");
   t_output(4,2,
      "details.  Each of these parameters may be edited individually by ");
   t_output(5,2,
      "specifying the the code PP, the letter of the row needed, and the");
   t_output(6,2,
      "column number.  Example PPA2 = -34000000");
   t_output(7,2,
      "A row of parameters may be specified by just using the code PP and");
   t_output(8,2,
      "the letter of the row.  Example PPA = 56000000 -34000000 0");
   }
else if (strcmp(ans,"h cc") == 0)
   { 
   t_output(2,2,
      "Corner coordinates.  Projection coordinates of the resulting output");
   t_output(3,2,"image's four corners.  Specify each corner separately.");
   }                        
else if ((strcmp(ans,"h ul") == 0) || (strcmp(ans,"h ccul") == 0))
   {                                                
   t_output(2,2,
      "Upper left corner coordinates.  The location of the upper left");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strcmp(ans,"h ll") == 0) || (strcmp(ans,"h ccll") == 0))
   {                                                      
   t_output(2,2,
      "Lower left corner coordinates.  The location of the lower left");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strcmp(ans,"h ur") == 0) || (strcmp(ans,"h ccur") == 0))
   {
   t_output(2,2,
      "Upper right corner coordinates.  The location of the upper right");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strcmp(ans,"h lr") == 0) || (strcmp(ans,"h cclr") == 0))
   {                                                      
   t_output(2,2,"Lower right corner coordinates.  The location of the lower");
   t_output(3,2,
      "right corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if (strcmp(ans,"h pd") == 0)
   {
   t_output(2,2,
      "Projection distance in line and sample direction.  The distance (in ");
   t_output(3,2,
    "the line and sample direction) on the ground that one sample represents.");
   }
else if (strcmp(ans,"h pu") == 0)
   {
   t_output(2,2,
      "Projection measurement units.  The measurement system used for the");
   t_output(3,2,"projection dimensions.");
   t_output(4,2,"The current standard projection units include:");
   t_output(5,2,"radians");
   t_output(6,2,"degrees");
   t_output(7,2,"seconds");
   t_output(8,2,"dms");
   t_output(9,2,"feet");
   t_output(10,2,"meters");
   }
else if (strcmp(ans,"h mm") == 0)
   {
   t_output(3,2,"Model method.  Type of model used to generate the geometric");
   t_output(4,2,"mapping grid.  Standard values are POLYNOMIAL, SPACECRAFT,");
   t_output(5,2,"and FINITE ELEMENT.  Other values are allowed, if needed.");
   }
else if (strcmp(ans,"h nc") == 0)
   {
   t_output(3,2,
    "Number of coefficients.  The number of polynomial coefficients.");
   t_output(4,2,                                 
    "The valid entries are 3,6,10,15 for a 1st, 2nd, 3rd, or 4th degree");
   t_output(5,2,"polynomial.  This field is zero if other than a polynomial");
   t_output(6,2,"model was used to generate the mapping grid.");
   }
else if ((strcmp(ans,"h fp") == 0) || (strcmp(ans,"h sp") == 0))
   {
   t_output(3,2,"Transformation coefficients.  The first polynomial is the Y");
   t_output(4,2,"transformation coefficients and the second polynomial is the");
   t_output(5,2,"X transformation coefficients.  Each coefficient is edited");
   t_output(6,2,"separately.  Example FPA1 = .707; FPA2 = -.707");
   t_output(7,2,"or SPB1 = .707; SPB2 = -.707"); 
   } 

/*  A blank as a code will display the general help statements
--------------------------------------------------------------*/
else if (strcmp(ans,"h") == 0) 
   {
   t_output(2,2,
      "Each of the GRID fields may be edited by specifying the upper case");
   t_output(3,2,
      "letters of a specific field name, followed by an equal sign(=),");
   t_output(4,2,
      "followed by the new value.  Edit codes may be entered in upper or");
   t_output(5,2,"lower case.");
   t_output(6,2,
      "Example:  To change the projection zone field to '0' enter PZ = 0.");
   t_output(7,2," ");
   t_output(8,2,
      "The (H)elp command is used to retrieve information on any of the");
   t_output(9,2,
      "GRID fields.  Specify the (H)elp command using the following syntax.");
   t_output(10,2,
      "Example: To receive help on projection codes enter H PC.");
   t_output(11,2," ");
   t_output(12,2,
      "The (P)age command is used to page through the grid fields.");
   t_output(13,2," ");
   t_output(14,2,
      "The (Q)uit command is used to exit the EDITGRID module.");
   t_output(15,2," ");
   t_output(16,2,
      "The (W)rite command is used to write the changes made to the GRID");
   t_output(17,2,
      "fields.  The GRID file will not be created unless the (W)rite");
   t_output(18,2,
      "command is invoked."); 
   } 

/*  An invalid code was entered, display the error message
----------------------------------------------------------*/
else
   {
   t_output(2,2,"[editgrid-code] Invalid help command code");
   t_output(3,2," ");
   t_output(4,2,"An invalid help command code was entered.  Retype the");
   t_output(5,2,"command withe the correct code.  The upper case letters");
   t_output(6,2,"or a combination of the upper case letters should be used as");
   t_output(7,2,"the edit command code");
   }

/*  Output the return command line.
-----------------------------------*/
t_output(*line,2,retrn);

/*  Wait for the user to hit the return key to continue.
--------------------------------------------------------*/
t_input(*line,37,temp,&term);
if (term == P_OVER)
   t_output(*line - 2,2,"[editgrid-input] Input is larger than buffer size");

return;
}  /* gridhelp */
