#include "editstat.h"

/*
	FUNCTION chgsite (shape, pbuf)

**********************************************************************

   name- chgsite                        expires- 12/31/99

   language- C     type- Function

   Version- 1.0   Date- 29-Jun-83  Programmer- K. S. Mcgee
   Revision- 1    Date- 16-Aug-83  Programmer- S. Ross/SASC  scn#20-25
   version- 2.0   Date- 10-Oct-89  Programmer- F. Irani/STX

   Function- allows user to change a column number and/or row number
             associated with a specified site polygon

***********************************************************************/


int chgsite
(
    long shape[],	/* Shape vector for data		*/
    long pbuf[]		/* Polygon data buffer from STATIO	*/
)
	
{

long i;   /* General purpose array index */
long j;   /* General purpose array index */

long	*array,			/* Temporary to hold polygon if nosave 	*/
	col, 			/* col value (X) of polygon vertex	*/
	npages,			/* number of pages to display		*/
	nuflag,			/* Flag that a new point is added	*/
	pstart,  		/* Starting page to display		*/
	pt,			/* Point to change from user		*/
	row,			/* Row (Y) value of polygon vertex	*/
	save,			/* Flage to indicate save data		*/
	start = 0,		/* Start of page			*/
	tpages,			/* Total number of pages		*/
	tnum;			/* Used to calculate number of pages	*/

listem = TRUE;			/* To clear screen 1st time */

/* Allocate space for temporary array.
--------------------------------------*/
array = (long *)malloc(2 * MAXPTS * sizeof(long));

/* Initialize display array to zero.
------------------------------------*/
for (i = 0; i < MAXPTS; i++)
   {
   *(array + i) = 0;
   *(array + MAXPTS + i) = 0;
   }

save = FALSE;

/* Calculate max number of pages.
---------------------------------*/
tpages = (long) (shape[2]/30.0);
tnum   = tpages % 30;
if (tnum  >  0) 
	npages = tpages + 1;
else
	{
	npages = tpages;
	if (npages == 0) 
		npages = 1;
	}
pstart = 1;

/* Move polygon data into temporary buffer.
-------------------------------------------*/
j = 0;
for (i=0; i < shape[2]*2; i+=2)
	{
	*(array + j) = pbuf[i];
	*(array + MAXPTS + j) = pbuf[i+1];
	++j;
	}

for (;;)
  {

/* If invalid entry or more changes display site polygon.
---------------------------------------------------------*/
  disppoly (shape, array, &pstart); 

/* Prompt for pt number  to be changed.
---------------------------------------*/
  sprintf(estring,
  "Enter Point to change, Save, Back or Page[point#] to Page[to point#]:");
  prompt(ostring);
  if (back)
	{
	free (array);
	save = FALSE;
	return(save);
	}

/* Check for page.
------------------*/
  if (strncmp(ostring, "PAGE", len) == 0 ) 
	{
	++pstart;
	if (pstart  >  npages)
		pstart = 1;
	t_lclear (2, 20);
	continue;
	}
  else if (es_index(ostring, "P") == 0)
	{

	/* Check if paging to a particular element.
    	-------------------------------------------*/
	for (i = 0; i < len; ++i)
        	{
		if (ostring[i] >= '0' && ostring[i] <= '9')
			break;
		}

	if ( i < len && sscanf(&ostring[i], "%ld", &pt) == 1)
		{
		if (pt > shape[2]) 
			pt = shape[2];
	     	if (pt < 1) 
			pt = 1;

		start = (pt-1)/30 + 1;
		if (start  !=  pstart) 
		t_lclear (2, 20);
		pstart = start;
		continue;
		}
 	}
  else if (strncmp(ostring, "SAVE", len) == 0 ) 
 	{
	save = TRUE;

	/* Move updated temp array back to pbuf.
	----------------------------------------*/
	j = 0;
	for (i=0; i < shape[2]*2; i+=2)
		{
		pbuf[i]   = *(array + j);
		pbuf[i+1] = *(array + MAXPTS + j);
		++j;
		}
	free (array);
	return(save);
 	}

  nuflag = FALSE;

/* Convert point number to integer.
-----------------------------------*/
  if ( (sscanf(ostring, "%ld", &pt) != 1) || (pt < 0) || (pt > (shape[2]+1)) )
	{
	t_lclear (t_lines - 2, 1);
	t_output (t_lines - 2, 1, "[editstat-number] Invalid point number");
	listem = FALSE;
	continue;
	}
  if (pt  >  shape[2])
	{
	if (shape[2] + 1 > MAXPTS)
		{
		t_lclear(t_lines - 2, 1);
		sprintf(estring,
	"[editstat-maxpoint] Unable to add more points. Maximum of %3d has been reached.", MAXPTS);
		t_output(t_lines - 2, 1,  estring);
		continue;
		}
	t_lclear (t_lines - 2, 1);
	++shape[2];
	pt = shape[2];
	start = 0;
	nuflag = TRUE;
	}

/* Page to selected point if not showing.
-----------------------------------------*/
  if (start != ((pt - 1) / 30) + 1 )
	{
	start = ((pt - 1) / 30) + 1;
	if (start  !=  pstart) 
		t_lclear (2, 20);
	pstart = start;
	listem = TRUE;
	disppoly (shape, array, &pstart);
	}

/* Prompt for column number, row number.
----------------------------------------*/
  for (;;)
    {

    sprintf(estring,
    "Enter ROW COL for Point %ld, Back or Page[point#] to Page[to point#]:",pt);
    prompt(ostring);
    if (back)
	{
	if (nuflag)
		{
    	  	--shape[2];
	  	nuflag = FALSE;
		listem = TRUE;
		disppoly (shape, array, &pstart);
	  	}
	break;
	}

/* Check for page.
------------------*/
    if (es_index(ostring, "P") == 0)
	{
	/* Check if paging to a particular element.
    	-------------------------------------------*/
	if (nuflag) 
	  	{
		--shape[2];
	  	nuflag = FALSE;
	  	}

	for (i = 0; i < len; ++i)
        	{
		if (ostring[i] >= '0' && ostring[i] <= '9')
			break;
		}

	if ( i < len && sscanf(&ostring[i], "%ld", &pt) == 1)
		{
		if (pt > shape[2]) 
			pt = shape[2];
	     	if (pt < 1) 
			pt = 1;
		start = (pt-1)/30 + 1;
		if (start  !=  pstart) 
			t_lclear (2, 20);
		pstart = start;
		}
	else
		{
		++pstart;
		if (pstart  >  npages)
			pstart = 1;
		}
	listem = TRUE;
	disppoly (shape, array, &pstart);
	break;
	}

/* Extract pixel number & row number from user entry.
------------------------------------------------------*/
    t_lclear (t_lines - 2, 22);
    if ( sscanf(ostring, "%ld %ld", &row, &col) != 2)
   	{
	t_lclear (t_lines - 2, 1);
   t_output (t_lines - 2, 1, "[editstat-twoval] Two values must be entered");
	listem = FALSE;
   	continue;
   	}

    if (row < 0 || col < 0)
   	{
   	t_output (t_lines - 2, 1, "[editstat-value] Row and column values must be positive");
	continue;
   	}
    else
   	{
   	*(array + pt - 1) = col;
   	*(array + MAXPTS + pt - 1) = row;
   	}

    start = (pt-1)/30 + 1;
    if (start  !=  pstart) 
	t_lclear (2, 20);

    pstart = start;
    break;
    }
  }

}
