/* ---------------------------------------------------------------------------

   Name- Editstat                         expires- 12/31/83

   Language- C     Type- Main

   Version 	Date	Programmer	Comment
   -------	----	----------	-------
    1.0      20-JUN-83  J. H. Urquhart
    6.0      10-NOV-86  F. Irani/SAR    (ADD EDC V4.0)
    7.0       6-OCT-89  F. Irani        (V5.0 Conversion)
    8.0      26-Jan-89  B. Ailts        Misc. clean up
				        Renamed support routine file names to 
					 match the support names
					Added lp and file to the interactive 
					 LIST command
					Made -sort and -copy subcommands to be 
					 able to run in batch
					Added the class and site number to the 
					 output from list
    9.0	     20-Sep-91  T. Rockvam	Fixed error messages
					Changes COMMAND subcmd to loop until
					 valid response instead of exiting and
					 running again
					Fixed completion message to print after
					 copy and sort are done

   Function- The main module for the subsystem 'EDITSTAT'

   Exit states-  User enters Exit or Quit for a command

   Common areas- editstat.h: the editstat common 
                 contains option and command info.

---------------------------------------------------------------------------
*/

#include <stdlib.h>
#include <string.h>
#include "es_globals.h"
#include "util.h"
#include "protos.h"

int main
(void)

{

long ii;             /* Loop variable  */
long status;         /* Status flag for function calls */
long fatal_err = -1; /* Fatal error flag for c_errmsg  */
long inf_err = 1;    /* Information flag for c_errmsg  */
long zero = 0;       /* Constant of Zero */
long one = 1;        /* Constant of One  */


struct  VARIABLE *var;

char	cmdlst[16][10];

char 	cpy_cls[MAXCLS][STNAME];  /* names of the classes to be copied	*/

char	*cmdptr;		/* Update vblock command		*/

char  tmpstr[1][CMLEN];

long	access,			/* Open file for read, write, etc.	*/
	blist[MAXBND],		/* List of band no. to be copied	*/
	bnum,			/* Bands number				*/
	itype = ELONG,		/* Data type of input number		*/
	ncmds = 16,		/* Number of interactive commands 	*/
	nincls = 0,			/* Number of classes specified to be copied*/
	oclass,			/* Dummy: Output file number of classes */
	pcount = 1,		/* Number of TAE parameters retrieved	*/
	sdim,			/* TAE block character parameter length	*/
	std_out;		/* Standard output for inlas call	*/	

/*
Initialize command list.
------------------------*/
strcpy(cmdlst[0],  "COMBCLASS"); 
strcpy(cmdlst[1],  "DROPCLASS"); 
strcpy(cmdlst[2],  "DROPHIST"); 
strcpy(cmdlst[3],  "DROPSITE");
strcpy(cmdlst[4],  "EDITCOR"); 
strcpy(cmdlst[5],  "EDITCOV" ); 
strcpy(cmdlst[6],  "EDITHIST"); 
strcpy(cmdlst[7],  "EDITMEAN"); 
strcpy(cmdlst[8],  "EDITSITE"); 
strcpy(cmdlst[9],  "EXIT"); 
strcpy(cmdlst[10], "HELP"); 
strcpy(cmdlst[11], "LIST"); 
strcpy(cmdlst[12], "MOVESITE"); 
strcpy(cmdlst[13], "NEWCLASS"); 
strcpy(cmdlst[14], "RENCLASS"); 
strcpy(cmdlst[15], "RENSITE");

/* Initialize TAE and processing line message.
----------------------------------------------*/
c_inlas ( &block, &std_out);
t_init ( &t_lines, &t_cols, &tae_term);

/* Dynamically allocate space for global variables.
---------------------------------------------------*/
dyn_alloc();

/* Initialize Global variables in editstat.h.
---------------------------------------------*/
quit = back = outopen = copy = sort = pcop = FALSE;
strcpy(command,"NONE");

/* Initialize fileutil flags, (once on stay on) and others.
-----------------------------------------------------------*/
printer = 0;
listit[0] = histfla = 0;
prt[0] = 0;
prt[1] = 0;
prt[2] = 0;
pflags[TERM] = 1;
olas[0][0] = prtfile[0] = '\0';

/*  This will need to be used when the plotting software is implemented.
------------------------------------------------------------------------*/
/*
if ( c_getstr( &block, "$PLOTTER", &pcount, &sdim, plotter) != 1)
   {
   sprintf(estring, "Error in global value /$plotter");
   c_errmsg (estring, "EDITSTAT-fatal", &fatal_err);
   }
*/

sdim = 8;
status = c_getstr ( &block, "_SUBCMD", &one, &sdim, subcmd);

/*  Only open file if subcmd = INPLACE,LIST,SORT,EDIT, or COPY
--------------------------------------------------------------*/
if (strcmp(subcmd, "COMMAND") != 0)
  {
  /*
  Always capture input file name (file being operated on).
  --------------------------------------------------------*/
  access = IREAD;
  status = c_getnam( &block, "INSTAT", &zero, hos, "stat", &access);
  strcpy(las, hos);
  if ( status != E_SUCC)
     {
  /* If input file does not exist for READ 
  then getname for WRITE access (new file).
  -----------------------------------------*/
     if (strcmp(subcmd, "INPLACE") == 0)
        {
        t_output(t_lines, 1, "\nEnter <CR> to continue.");
        t_read (estring, &tae_term);
        sprintf(estring, "Create new stats file? [Yes/No]");
        prompt(ostring);
        t_lclear(t_lines - 1, 1);
        if (strncmp(ostring, "Y", 1) != 0)
           z_exit(-1,"FATAL");
        else
           {
           label();
           strcpy(command, "CREATE");
           t_highlight(command);
	   t_pos (t_lines, 1);
           access = IWRITE;
           status = c_getnam(&block, "INSTAT", &zero, hos,"stat", &access);
           if (status != E_SUCC)
              c_errmsg ("Fatal error encountered","editstat-fatal", &fatal_err);
    	   }
        } /* if subcmd = INPLACE  */
     else
        c_errmsg ("Fatal error encountered","editstat-fatal", &fatal_err);
     }

  /* Open the statistics file.
  ----------------------------*/
  inopen = FALSE;

  /*
  If subcmd = LIST, COPY or SORT open for read only access.
  ---------------------------------------------------------*/
  if ((strcmp(subcmd,"INPLACE") == 0) || (strcmp(subcmd,"EDIT") == 0))
	access = IUNKNOWN;
  else
	access = IREAD;

  status = c_stopen ( &inlun, hos, &access, &nclass);
  if (status == E_GNLIO)
      {
      c_errmsg ("Error opening input file","editstat-open",
            &inf_err);
      c_errmsg ("Fatal error encountered","editstat-fatal", &fatal_err);
      }
  else
      inopen = TRUE;
  }

/* Get value for SUBCMD. If = copy get value of OUTSTAT.
(Validity of INSTAT will depend on status of SUBCMD.)
---------------------------------------------------*/
if (strcmp(subcmd, "COPY") == 0)
   {
   copy = TRUE;
   access = IWRITE;
   status = c_getnam( &block, "OUTSTAT", &zero, ohos, "stat", &access);
   strcpy(olas[0], ohos);
   if ( status != E_SUCC)
      {
      c_errmsg("Fatal error encountered", "editstat-fatal", &fatal_err);
      } 

   /* Retrieve POLYFLG parameter.
   ------------------------------*/
   status = c_getstr ( &block, "POLYFLG", &pcount, &sdim, ostring);
   if ( ostring[0] == 'Y' )
      {
      histfla = 1;
      pcop = TRUE;
      } 
   var = p_find(&block,"CLASSNAM");
   strcpy(cpy_cls[0], SVAL(*var,0));
   if (strncmp(cpy_cls[0],"*",1) == 0)
      nincls = 0;
   else if (strcmp(cpy_cls[0],"0") == 0)
      nincls = -1;
   else
      nincls = var->v_count;
   for (ii = 0; ii < nincls; ii++)
      {
      strcpy(cpy_cls[ii], SVAL(*var,ii));
      len = (long) strlen(cpy_cls[ii]);
      c_low2up(cpy_cls[ii], &len);
      }

   }  /*  if subcmd == copy  */

else if (strcmp(subcmd, "SORT") == 0)
   {
   access = IWRITE;
   status = c_getnam( &block, "OUTSTAT", &zero, ohos, "stat", &access);
   strcpy(olas[0], ohos);
   if ( status != E_SUCC)
      {
      c_errmsg("Fatal error encountered", "editstat-fatal", &fatal_err);
      }

   status = c_getnum (&block, "BAND", &one, &itype, &sband);
   if ( status != 1)
      c_errmsg("Fatal error encountered", "editstat-fatal", &fatal_err);

   sdim = 3;
   status = c_getstr ( &block, "SORTOPT", &pcount, &sdim, direction);
   if ( status  != 1)
      c_errmsg("Fatal error encountered", "editstat-fatal", &fatal_err);
   if (strcmp(direction, "ASC") == 0)
      sdir = 1; 	/* ascending */
   else
      sdir = 0; 	/* decending */
 
   /* Set up variables to copy everything in the sort.
   ---------------------------------------------------*/
   nincls = 0;
   bnum = 0;
   sort = TRUE;
   }  /*  if subcmd == SORT  */

else if (strcmp(subcmd, "LIST") == 0)
   {
   listit[0] = TRUE;

   /* es_list(nincls, cpy_cls); */
   statlist();

   /* Update listit flag for fileutil.  (Cleaner check in .PDF).
   -------------------------------------------------------------*/
   if (pflags[LP] || pflags[FIL])
   	fclose(printer);
	
   /* Call complete with appropriate output file names.
   ----------------------------------------------------*/
   sdim = 9;
   status = c_getstr ( &block, "CMD", &pcount, &sdim, command);
   if (strcmp(command,"LIST") != 0)
      {
      if (pflags[FIL])
         c_complt ( prog_name, usrfile, &one);
      else
         {
            strcpy(tmpstr[0],"\0");
            c_complt ( prog_name,tmpstr, &zero);
         }
      }
   }  /*  else if subcmd == LIST  */

/* EDITSTAT-COPY Processing:
----------------------------*/
if (copy || sort)
   {

   /* Get OUTSTAT and open an output file. 
   ---------------------------------------*/
   access  = IWRITE;
   outopen = FALSE;
   status  = c_stopen ( &outlun, ohos, &access, &oclass);

   /* Stop if open error.
   ----------------------*/
   switch (status )
      {
      case E_OPOLD : 
                 label();
                 if ( inopen)
                    status = c_stclse ( &inlun); 
                 if ( outopen)
                    status = c_stclse ( &outlun);
                 c_errmsg("Specified output file already exists.",
			  "editstat-open", &inf_err );
                 c_errmsg("Fatal error encountered", "editstat-fatal", 
		          &fatal_err);
      case E_GNLIO :
                 if ( inopen)
                    status = c_stclse ( &inlun); 
                 label();
                 c_errmsg("Error opening output file","editstat-open", 
	                  &inf_err );
                 c_errmsg("Fatal error encountered", "editstat-fatal", 
		 &fatal_err);
      case E_OPNEW :
		outopen = TRUE;
      }  /* switch status */

   if (copy)
      get_bands(blist,&bnum);
   es_copy(nincls,cpy_cls,blist,bnum);
   strcpy( hos, ohos);
   strcpy( las, olas[0]);

   if ( inopen)
      status = c_stclse ( &inlun);
   inopen = FALSE;
   inlun = outlun;
   }  /*  if "EDITSTAT-COPY") 	*/

/* Make input and output file names upper case for display.
-----------------------------------------------------------*/
len = (long) strlen(las);
c_low2up(las,&len);

len = (long) strlen(olas[0]);
c_low2up(olas[0],&len);

if (strcmp(subcmd, "COMMAND") == 0)
   {
	/* Display heading.
	-------------------*/
        label();
	sprintf( estring, "EDITSTAT       FILE: %s", las);
	t_lclear(1, 1);
	t_output ( 1, 1, estring);

	/* Display available commands.
	------------------------------*/
	t_output ( 3, 1, "       Available commands are :");
	t_output(5,  1,"         COMBCLASS            EXIT");
	t_output(6,  1,"         DROPCLASS            HELP");
	t_output(7,  1,"         DROPHIST             LIST    ");
	t_output(8,  1,"         DROPSITE             MOVESITE");
	t_output(9,  1,"         EDITCOR              NEWCLASS");
	t_output(10, 1,"         EDITCOV              RENCLASS");
	t_output(11, 1,"         EDITHIST             RENSITE");
	t_output(12, 1,"         EDITMEAN ");
	t_output(13, 1,"         EDITSITE");

      for (;;)
	{
      	/* Read user entry.
      	-------------------*/
      	sprintf(estring, "Enter Command");
      	prompt(command);
      	len = (long) strlen(command);
      	c_low2up(command, &len);

      	/*
      	Get full command name for output to vblock.
      	-------------------------------------------*/
      	for (ii = 0; ii < ncmds; ++ii)
      		{
		if (strncmp(command, cmdlst[ii], len) == 0)
			{
			strcpy(command, cmdlst[ii]);
			break;
			}
      		}

      	/*
      	if valid command name was not found, prompt again.
      	--------------------------------------------------*/
      	if (ii == ncmds)
      		{
		sprintf(estring, "%s: No Such Command", command);
		t_lclear(t_lines-2, 1);
		t_output ( t_lines-2, 1, estring);
		listem = FALSE;
      		}
	else
		break;
	}

      /*
      Output the name to the local vblock variable prtfile.
      -----------------------------------------------------*/
      cmdptr = command;
      status = q_string(&block,"CMD",1,&cmdptr,P_UPDATE);
      if (status != SUCCESS)
          {
          c_errmsg("Fatal error encountered", "editstat-fatal", &fatal_err);
          }
      
      status = q_out(&block);
      if (status != SUCCESS)
          {
          c_errmsg("Error writing to PARBLK","editstat-parblk", &inf_err);
          c_errmsg("Fatal error encountered", "editstat-fatal", &fatal_err);
          } 

   }
else if (strcmp(subcmd,"EDIT") == 0)
   {
   sdim = 9;
   status = c_getstr ( &block, "CMD", &pcount, &sdim, command);
   len = (long) strlen(command);

   if ( strncmp( command, "HELP", len) == 0)
        {
        strcpy(command, "HELP");
	t_highlight(command);
	q_init (&block, P_BYTES, P_CONT);
	t_pos (t_lines, 1);
	status = q_dynp(&block, "ESHELP", M_FULLPDF);
	if (status != SUCCESS)
	      {
	      c_errmsg("Bad tae return value, q_dynp",
			"EDITSTAT-warning", &one);
	      }
	status = p_inim(&block, P_BYTES, P_CONT);
	}
   if (strncmp(command, "EDITCOR", len) == 0)
	{
	strcpy(command, "EDITCOR");
	t_highlight(command);
	es_editcor();
	}
   if (strncmp(command, "EDITCOV", len) == 0)
	{
	strcpy(command, "EDITCOV");
	t_highlight(command);
	es_editcovr();
	}
   else if (strncmp(command, "EDITMEAN", len) == 0)
	{
	strcpy(command, "EDITMEAN");
	t_highlight(command);
	es_editmean();
	}
   else if (strncmp(command, "EDITSITE", len) == 0)
	{
	strcpy(command, "EDITSITE");
	t_highlight(command);
	es_editsite();
	}
   else if (strncmp(command, "EDITHIST", len) == 0)
	{
	strcpy(command, "EDITHIST");
	t_highlight(command);
	es_edithist();
	}
   else if (strncmp(command, "COMBCLASS", len) == 0)
	{
	strcpy(command, "COMBCLS");
	t_highlight(command);
	es_combcls();
	}
   else if (strncmp(command, "MOVESITE", len) == 0)
	{
	strcpy(command, "MOVESITE");
	t_highlight(command);
	es_movesite();
	}
   else if (strncmp(command, "DROPCLASS", len) == 0)
	{
	strcpy(command, "DROPCLASS");
	t_highlight(command);
	es_dropcls();
	}
   else if (strncmp(command, "DROPHIST", len) == 0)
	{
	strcpy(command, "DROPHIST");
	t_highlight(command);
	es_drophist();
	}
   else if (strncmp(command, "DROPSITE", len) == 0)
	{
	strcpy(command, "DROPSITE");
	t_highlight(command);
	es_dropsite();
	}
   else if (strncmp(command, "RENCLASS", len) == 0)
	{
	strcpy(command, "RENCLASS");
	t_highlight(command);
	es_rencls();
	}
   else if (strncmp(command, "RENSITE", len) == 0)
	{
	strcpy(command, "RENSITE");
	t_highlight(command);
	es_rensite();
	}
   else if (strncmp(command, "NEWCLASS", len) == 0)
	{
	strcpy(command, "NEWCLASS");
	t_highlight(command);
	es_newclass();
	}
   } /* EDIT options */

/* Call complete with appropriate output file names.
----------------------------------------------------*/
if ( inopen)
    c_stclse ( &inlun);
if ( outopen) 
    c_stclse ( &outlun); 

/* Free space for global variables.
-----------------------------------*/
dyn_free();

len = (long) strlen(command);

if (((strncmp( command, "EXIT", len) == 0) &&
    (strncmp(subcmd, "EDIT",len) == 0)))
   {
      strcpy(tmpstr[0], "\0");
      c_complt ( prog_name,tmpstr, &zero);
   }
else if ((copy) || (sort))
   c_complt ( prog_name, olas, &one);
else
   z_exit(1,"");

return EXIT_SUCCESS;

} /*  Main  */
