#include "editstat.h"
#include "msg.h"

/* FUNCTION es_combcls

------------------------------------------------------------------------
   name- combcls

   language- C

   Version- 1.0    Date-  30-Jun-83  Programmer- S. Ross, SASC
   Revision- 2     Date-  30-May-86  Programmer- N. Deodhar/SAR, #202
   Version- 1.0    Date-  10-Oct-89  Programmer- F. Irani/STX

	  This code was borrowed from combcls to be included in editstat.

   Function- combcls combines two classes at a time in a statistics file
	     as many times as is desired.

************************************************************************

*/

void es_combcls
(void)

{

long i;          /*  General purpose array index    */
long status;     /*  Status flag for function calls */
long zero = 0;   /*  General programmer usage       */


char	combcls[STNAME+1],		/* Name of class to be combined  */
	newclass[STNAME+1];		/* Destination class for combcls */

long	existed,	/* Flags an existg class being added to */
	shape[9],	/* shape vector for class name buffer	*/
	valid;		/* Flag for a valid new class name	*/

listem = TRUE;
status = E_SUCC;

/* Do forever or until QUIT or valid Class/Site specification.
--------------------------------------------------------------*/
for (;;)
  {
  combcls[0] = '\0';
  newclass[0] = '\0';

  /* List classes.
  ----------------*/
  if (listem)
    {
    label();
    combheader(newclass, combcls);
    listcls();
    combheader(newclass, combcls);
    if (quit)
	return;
    }

  /* Prompt for and read class name.
  ----------------------------------*/
  listem = FALSE;
  QPROMPT("Enter a class name (may be new) to contain combined classes",newclass);
  if (quit)
	return;

  if (strncmp(newclass, "0", len) == 0)
    	continue;

  if (es_index(newclass, "*") >= 0)
        {
	t_lclear(t_lines - 2, 1);
	t_output(t_lines - 2, 1,
	"[editstat-invalchar]  Invalid character specified.");
 	continue;
        }

  if (strncmp(newclass, "LIST", len) == 0)
        {
	listem = TRUE;
	continue;
        }

  /* Check naming syntax.
  -----------------------*/
  valid = checkname(newclass);
  if (!valid)
	continue;

  /* Try to get class name as specified.
  --------------------------------------*/
  existed = FALSE;
  status = c_stfdcl (&inlun, newclass, &ic);
  if (status != E_SUCC)
  	  {
	  status = E_NOCL;
	  for (i =1; i <= nclass; ++i)
	        {
		/* If can't find as specified, try for short form.
		--------------------------------------------------*/
		status = c_stget (&inlun, "NAME", &i, &zero, shape, ostring);
		ostring[shape[2]] = '\0';
		status = E_NOCL;
  		if (es_index(ostring, newclass) == 0)
			{
			status = E_SUCC;
			strcpy(newclass, ostring);
			ic = i;
			break;
			}
	        }
  	  }

  sprintf(estring, "[informational] New Class: %s", newclass);
  t_lclear( 2, 1);
  t_output( 2, 1, estring);

  if (status == E_SUCC)
      {
      existed = TRUE;
      for (;;)
        {
        sprintf(estring,
        "Class %s already exists: Add other classes to it? [YES/NO]", 
	newclass);
        prompt(ostring);
        if (quit)
	    return;
	break;
        }
      if (strncmp(ostring, "Yes", len) != 0)
        {
	newclass[0] = '\0';
        combheader(newclass, combcls);
	continue;
        }
      }
  else
  	existed = FALSE;
  
  /* Get name of each class to be combined and merge it.
  ------------------------------------------------------*/
  listem = TRUE;  

  for (;;)
    {
    /* List classes.
    ----------------*/
    if (listem)
      {
      label();
      combheader(newclass, combcls);
      listcls();
      combheader(newclass, combcls);
      if (quit)
  	  return;
      }
  
    /* Get a class name from user.
    ------------------------------*/
    listem = FALSE;
    BPROMPT("Enter a class name to be merged",ostring);
    if (back)
  	break;

    if (strncmp(ostring, "0", len) == 0)
    	continue;

    if (strncmp(ostring, "List", len) == 0)
        {
	listem = TRUE;
	continue;
        }

    /* Ensure that the class to be combined exists.
    -----------------------------------------------*/
    strcpy(combcls, ostring);
    status = c_stfdcl (&inlun, combcls, &ic);
    if (status != E_SUCC)
          {
	  status = E_NOCL;
	  for (i =1; i <= nclass; ++i)
	        {
		/* If can't find as specified, try for short form.
		--------------------------------------------------*/
		status = c_stget (&inlun, "NAME", &i, &zero, shape, ostring);
		ostring[shape[2]] = '\0';
		status = E_NOCL;
  		if (es_index(ostring, combcls) == 0)
			{
			status = E_SUCC;
			strcpy(combcls, ostring);
			ic = i;
			break;
			}
	        }
          }

    if (status != E_SUCC)
        {
	combcls[0] = '\0';
	c_sterr(status,t_lines - 2);
    	continue;
        }
    else
        {
	/* Be sure class is not to be combined with itself.
	---------------------------------------------------*/
	if (strcmp(newclass, ostring) == 0)
 	    {
	    t_lclear( t_lines - 2, 1);
	    t_output (t_lines - 2, 1, "[editstat-dupname] Duplicate name was specified");
	    combcls[0] = '\0';
	    listem = FALSE;
	    continue;
	    }

        combheader(newclass, combcls);

  	t_lclear(t_lines - 2, 1);
  	t_lclear(t_lines - 1, 1);
  	t_lclear(t_lines, 1);
        sprintf(estring, "Class %s will be combined with", newclass);
  	t_output(t_lines - 2, 1, estring);
        sprintf(estring, "Class %s. OK? [YES/NO]", combcls);
       	prompt(ostring);
    	
    	if (strncmp(ostring, "YES", len) != 0)
	    {
	    combcls[0] = '\0';
	    listem = TRUE;
    	    continue;
	    }

	/* Combine the class.
	---------------------*/
	comthm(newclass, combcls);

  	/* If this class existed already, merge results in one less class.
  	------------------------------------------------------------------*/
	if (existed)
	    	--nclass;
	else
		/* Class now exists. Subtract for new class merges.
		---------------------------------------------------*/
		existed = TRUE;

	/* Class is merged.
	-------------------*/
  	combcls[0] = '\0';
        sprintf(estring, "Classes combined.");
  	t_lclear(t_lines - 2, 1);
  	t_output(t_lines - 2, 1, estring);
	t_output(t_lines, 1, "Enter <CR> to continue.");
	t_read (estring, &tae_term);

	listem = TRUE;
    	}
    }

  /* Next Destination class.
  --------------------------*/
  listem = TRUE;

  }

}
