#include "editstat.h"

/*
     FUNCTION es_editcor

**********************************************************************

   name- es_editcorr                    

   language- C

   Version- 1.0    Date- 20-Jun-83  Programmer- R. J. Tighe
   Version- 1.0    Date- 10-Oct-89  Programmer- F. M. Irani

   Function- control module for the editstat es_editcorr (edit
        correlation matrix) function. reads the correlation matrix in
        from disk, passes it to display and editing routines, and
        writes it back to disk (if "save" is entered.)

   comments- uses newcovr to perform editing

*************************************************************************

*/


void es_editcor
(void)

{

long status;         /* Status flag for function calls   */
long inf_err = 1;    /* Information flag for c_errmsg    */
long fatal_err = -1; /* Fatal error flag for c_errmsg    */
long zero = 0;       /* General programmer usage         */

char 	es_node[STNAME+1],		/* Node name (for display)	*/
	data_type[STNAME+1];		/* Data_type name buffer 	*/

double	*cormtrx;			/* Covariance (triangular) matrix */

long	save, 				/* flag to save changes		*/
	shape[9];			/* Matrix shape vector		*/

listem = TRUE;
/*
Do Until Quit.
--------------*/
for (;;)
  {

  /*
  Prompt user for Image level, class and/or site name to process.
  ---------------------------------------------------------------*/
  for (;;)
  	{
	status = getcs(0);
	if (quit) 
	    return;

	if (es_index(class, "*") >= 0)
		{
		t_lclear(t_lines - 2, 1);
		t_output(t_lines - 2, 1, "Invalid character specified");
		continue;
		}

	if (status != E_SUCC)
		{
	  	c_sterr(status,t_lines - 3);
		if (status == E_NOCL)
			{
			t_lclear(t_lines - 2, 1);
			t_output(t_lines - 2, 1, 
			"Use the command NEWCLASS to create a new class.");
			}
		if (status == E_NOSI)
			{
			t_lclear(t_lines - 2, 1);
			t_output(t_lines - 2, 1, 
			"Use the command EDITSITE to create a new site.");
			t_output(t_lines, 1, "Enter <CR> to continue.");
			t_read (estring, &tae_term);
			listem = TRUE;
			}
		continue;
		}

	/*
	GETCS puts name of class, site or "0" (image level) into ostring.
	-----------------------------------------------------------------*/
	if (strcmp(ostring, "0") == 0 )
		{
		if (ic)
			{
			status = 
			c_stget(&inlun, "NAME", &ic, &zero, shape, ostring);
			ostring[shape[2]] = '\0';
			sprintf(es_node, "Class level of class:%s", ostring);
			}
		else 
	 		strcpy(es_node, "Image level");
		}
	else
		strcpy(es_node, ostring);
	break;
  	}

  save = FALSE;

  label();

  /*
  Get the shape vector for the matrix.
  ------------------------------------*/
  data_type[0] = '\0';
  for (id = 1; status != E_NODT; ++id)
  	{
    	status = c_stnode (&inlun, &ic, &is, &id, data_type, shape);
    	if (strcmp(data_type, "CORRELATION_MATRIX") == 0)
    		break;
  	}

  if (strcmp(data_type, "CORRELATION_MATRIX") == 0)
  	{
	/* Allocate space for correlation matrix:  Shape[2] is the 
	size of the stored lower half of triangular matrix.
	---------------------------------------------------*/
	if ((cormtrx = (double *) malloc (shape[2] * sizeof (double))) == NULL )
		{
		c_errmsg ("Error allocating dynamic memory", "editstat-alloc",
			  &inf_err);
		c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
		}

	/* Fetch correlation matrix.
	----------------------------*/
	status = c_stget (&inlun, "CORRELATION_MATRIX", &ic, &is, 
	shape, cormtrx);
	if (status != E_SUCC)
		{
		c_sterr(status,t_lines - 2);
		free (cormtrx);
		continue;
		}
  	}
  else if (status != E_GNLIO)
  	{

  	/*
  	Matrix does not exist. Create new.
  	----------------------------------*/
	shape[0] = EDOUBLE;
	shape[1] = 1;

	/*
	Space is allocated to cormtrx in covar_init.
	--------------------------------------------*/
	covar_init (&cormtrx, shape);
        if (back)
		{
		listem = TRUE;
		continue;
		}
  	}
  else
  	{
	c_sterr(status,t_lines - 2);
	free (cormtrx);
	continue;
  	}

  /*
  Fetch new data (newcovr handles correlation as well as covariance.)
  -------------------------------------------------------------------*/
  newcovr(cormtrx, shape, &save, es_node);
  listem = TRUE;
  if (back)
  	{
	free(cormtrx);
	continue;
  	}

  /*
  Put new data in stat file if indicated.
  ---------------------------------------*/
  if (save) 
  	{
	status = c_stput (&inlun, "CORRELATION_MATRIX",
			 &ic, &is, shape, cormtrx);
	if (status != E_SUCC && status != E_DTOLD && status != E_DTNEW)
		{
		t_lclear(t_lines - 1,1);
		t_lclear(t_lines,1);
		t_pos(t_lines,1);
		c_sterr(status,t_lines - 2);
		}
  	}

  free(cormtrx);

  } /* Do until Quit. */

}	   
