#include <math.h>
#include "editstat.h"
#include "msg.h"

/*
     FUNCTION esc_corre(data_type,shape,blist,num,bandin)

**********************************************************************

   name- esc_corre                       

   language- C

   version- 1.0    date- 3-may-84  programmer- b. dealy (sar)
   revision 2      date-28-may-85  programmer- y.sheu scn#20-480 
   version- 2.0    date- 10-Oct    programmer- F. Irani STX

   function- esc_corre copies and reduces dimensions of the correlation
	     matrix
   	     esccorre was copied from esc_covar and modified to handle 
	     correlation matrices

**********************************************************************
*/


int esc_corre
(
    char data_type[],
    long shape[],
    long blist[],
    long num
)

{

long i;              /* General purpose array index     */
long j;              /* General purpose array index     */
long k;              /* General purpose array index     */
long status;         /* Status flag for function calls  */
long inf_err = 1;    /* Information flag for c_errmsg   */
long fatal_err = -1; /* Fatal error flag for c_errmsg   */

double	*cor_buf,
	*cor_new = NULL;

long	bandin,
    	col,
	cskip,
    	row,
    	rskip;
	
/* Compute dimension of correlation  matrix.
--------------------------------------------*/
bandin = (long) (( sqrt((8.0 * shape[2]) + 1) - 1) /2);

/* Keep any user specified dimensions that are > bandin.
--------------------------------------------------------*/
for (i = 0, j = 0; i < num; i++)
   {
   if ( blist[i] <=  bandin) 
      {
      blist[j] = blist[i];
      ++j;
      }
   else
      {
      --num;
      t_lclear(t_lines - 2, 22);
      if (histfla != 1) 
         {
  	 sprintf(estring,"%4d is an invalid band -- it is ignored", blist[i]);
         c_errmsg(estring,"editstat-invband",&inf_err);
         }
      }
   }

/* Allocate space for starting and final correlation  matricies.
----------------------------------------------------------------*/
if ((( cor_buf = (double *)
      malloc ((bandin * (bandin + 1) / 2) * sizeof (double))) == NULL ) ||
    (( cor_new = (double *) 
      malloc ((bandin * (bandin + 1) / 2) * sizeof (double))) == NULL ))
	 {
         c_errmsg ("Error allocating dynamic memory", "editstat-alloc",
                   &inf_err);
         c_errmsg ("Fatal error encountered", "editstat-fatal", &fatal_err);
	 }

/* Get correlation  matrix.
---------------------------*/
status = c_stget(&inlun, data_type, &ic, &is, shape, cor_buf);
if (status == E_SUCC)
    {
    /* Loop through rows.
    ---------------------*/
    i = -1;  
    j = 0;

    /* See if the row is to be deleted.
    -----------------------------------*/
    for (row = 1; row <= bandin; ++row)
        {
    	rskip = TRUE;
	for (k = 0; k < num; ++k)
		{
	 	if (row == blist[k])
			rskip = FALSE;
		}

    	/* Loop through columns.
    	------------------------*/
    	for (col = 1; col <= bandin; ++col)
		{

    		/* Account by lower triangular matrix.
    		--------------------------------------*/
    		if (col > row)
			break;

        	/* See if the column is to be deleted.
        	--------------------------------------*/
		cskip = TRUE;
		for (k = 0; k < num; ++k)
			{
			if (col == blist[k])
				cskip = FALSE;
			}

    		/* Increment input index whether a copy or not.
    		-----------------------------------------------*/
		++i;

    		/* If no flags are on, copy and increment output index.
    		-------------------------------------------------------*/
		if (!cskip && !rskip)
			{
			cor_new[j] = cor_buf[i];
			++j;
			}
		}
       }
    }
else
    {
    c_errmsg("Error retrieving Correlation Matrix","editstat-retrieve",
              &inf_err);
    c_errmsg ("Fatal error encountered", "editstat-fatal", &fatal_err);
    }
/* Reset shape to reduced dimension.
------------------------------------*/
shape[2] = ((num)*((num)+1))/2;

/* Put reduced correlation matrix out.
--------------------------------------*/
status = c_stput (&outlun, data_type, &ocls, &is, shape, cor_new);
if ((status  >  E_NOSI) && (status != E_DTNEW))
   {
   erwrite("correlation matrix");
   }

free (cor_buf);
free (cor_new);

return(status);

}
