

#include <math.h>
#include "editstat.h"
#include "msg.h"
/*
     FUNCTION esc_covar(data_type,shape,blist,num,bandin,inlun,
                         outlun)

   name- esc_covar                       

   language- C

   version- 1.0    date- 20-jun-83  programmer- g.mcconaughy(sasc)
   revision 2      date  28 may 85  programmer y. sheu scn#20-480
   version- 2.0    date- 10-Oct-89  programmer- F. Irani (Stx)

   function- esc_covar copies and reduces dimensions of the covariance
        matrix

**********************************************************************

*/
int esc_covar
(
    char data_type[],
    long shape[],
    long blist[],
    long num
)

{

long i;               /* General purpose array index     */
long j;               /* General purpose array index     */
long k;               /* General purpose array index     */
long status;          /* Status flag for function calls  */
long inf_err = 1;     /* Information flag for c_errmsg   */
long fatal_err = -1;  /* Fatal error flag for c_errmsg   */

double	*cov_buf,
	*cov_new;

long	bandin,
    	col,
	cskip,
    	row,
    	rskip;
long	size;

/* Compute dimension of covariance matrix.
------------------------------------------*/
bandin =  (long) (( sqrt( (8.0 * shape[2]) + 1) - 1) / 2);

/* Check that user is deleting a sensible number of bands.
----------------------------------------------------------*/
/*
if ( (bandin - num)  <  1 ) 	    
	c_errmsg("Error: Attempt to delete all bands.",
		 "ESC-FAIL",&fatal_err);
*/

/* Keep any user specified dimensions that are > bandin.
--------------------------------------------------------*/
for (i = 0, j = 0; i < num; ++i)
   if (blist[i]  <=  bandin) 
      {
      blist[j] =blist[i];
      ++j;
      }
   else
      {
      --num;
      if (histfla != 1)
          {
	  sprintf(estring,"%4d is an invalid band - it is ignored",blist[i]);
	  c_errmsg(estring,"editstat-invband",&inf_err);
      	  }
      }

/* Allocate space for starting and final covariance matricies.
--------------------------------------------------------------*/
size = (long) ((bandin * (bandin + 1) / 2) * sizeof (double));
cov_buf = (double *) malloc(size);
cov_new = (double *) malloc (size);
if ((cov_buf == NULL) || (cov_new == NULL))
   {
   c_errmsg ("Error allocating dynamic memory", "editstat-alloc",&inf_err);
   c_errmsg ("Fatal error encountered", "editstat-alloc", &fatal_err);
   }

/* Get covariance matrix.
-------------------------*/
status = c_stget (&inlun, data_type, &ic, &is, shape, cov_buf);
if (status  ==  E_SUCC) 
    {
    /* Loop through rows.
    ---------------------*/
    i = -1;  
    j = 0;

    /* See if the row is to be deleted.
    -----------------------------------*/
    for (row = 1; row <= bandin; ++row)
        {
    	rskip = TRUE;
	for (k = 0; k < num; ++k)
		{
	 	if (row == blist[k])
			rskip = FALSE;
		}

    	/* Loop through columns.
    	------------------------*/
    	for (col = 1; col <= bandin; ++col)
		{
    		/* Account only by lower triangular matrix.
    		-------------------------------------------*/
    		if (col > row)
			break;

        	/* See if the column is to be deleted.
        	--------------------------------------*/
		cskip = TRUE;
		for (k = 0; k < num; ++k)
			{
			if (col == blist[k])
				cskip = FALSE;
			}

    		/* Increment input index whether a copy or not.
    		-----------------------------------------------*/
		++i;

    		/* If no flags are on, copy and increment output index.
    		-------------------------------------------------------*/
		if (!cskip && !rskip)
			{
			cov_new[j] = cov_buf[i];
			++j;
			}
		}
       }

    } /* if status == E_SUCC) */
else
    {
    c_errmsg("Error retrieving Covariance Matrix","editstat-retrieve",&inf_err);
    c_errmsg ("Fatal error encountered", "editstat-alloc", &fatal_err);
    }

/* Reset shape to reduced dimension.
------------------------------------*/
shape[2] = ( (num) * ( (num) + 1)) / 2;

/* Put reduced covariance matrix out.
-------------------------------------*/
status = c_stput (&outlun, data_type, &ocls, &is, shape, cov_new);
if ((status  >  E_NOSI) && (status != E_DTNEW))
	{
        erwrite("covariance matrix");
	}

free (cov_buf);
free (cov_new);

return(status);

}
