#include "editstat.h"
#include "msg.h"

/*
     FUNCTION esc_mean(data_type,shape,blist,num)

**********************************************************************

   name- esc_mean

   language- C

   version- 2.0    date- 10/26/89   programmer- F. Irani (STX)
   version- 1.0    date- 20-jun-83  programmer- g.mcconaughy(sasc)

   function- esc_mean copies and reduces mean vectors from a
        statistics file

**********************************************************************

*/
int esc_mean
(
    char data_type[],	/* Data item being processed			*/
    long shape[],	/* Shape Vector for data (length of mean vector)*/
    long blist[],	/* List of bands to be copied  			*/
    long num		/* Number of bands to be copied	    		*/
)

{

long i;              /* General purpose array index    */
long j;              /* General purpose array index    */
long k;              /* General purpose array index    */
long status;         /* Status flag for function calls */
long inf_err = 1;    /* Information flag for c_errmsg  */
long fatal_err = -1; /* Fatal error flag for c_errmsg  */


double	 *inmean,	/* Input vector to copy				*/
	*outmean;	/* Output vector after reduction is any 	*/

long	bandin,		/* Number of bands in input vector 		*/
	copy;		/* Flag to copy a band		   		*/

/* Allocate space for the copy buffers.
---------------------------------------*/
if ( (inmean = (double *) malloc (MAXBND * sizeof (double))) == NULL)
	{
	c_errmsg ("Error allocating input dynamic memory", 
		  "editstat-alloc", &inf_err);
	c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
	}

if ( (outmean = (double *) malloc (MAXBND * sizeof (double))) == NULL)
	{
	free(inmean);
	c_errmsg ("Error allocating output dynamic memory", 
		  "editstat-alloc", &inf_err);
	c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
	}

/* Get the mean.
----------------*/
bandin = 0;
status = c_stget( &inlun, data_type, &ic, &is, shape, inmean);
if (status  !=  E_SUCC)
   {
   erread("the mean");
   free(outmean);
   free(inmean);
   return(status); 
   }
bandin = shape[2];

/*
Throw away any dimensions that the user may put in that is beyond
the dimension of the mean
-----------------------------------------------------------------*/
for (i = 0, j = 0; i < num; ++i)
   {
   if (blist[i]  <=  bandin) 
   	{
         blist[j] = blist[i];
         ++j;
   	}
   else
   	{
         --num;
         sprintf(estring, "%4d is an invalid band -- it is ignored", blist[i]);
         c_errmsg(estring,"editstat-invband",&inf_err);
   	} 	 
   }

/* Reduce mean vector.
----------------------*/
for (i = 0, k = 0; i < bandin; ++i)
	{
	copy = FALSE;
	for (j = 0; j < num; ++j)
		if (i+1 == blist[j])
			{
			copy = TRUE;
			break;
			}
   	if (copy)
		{
		outmean[k] = inmean[i];
   		++k;
		}
	}
shape[2] = k;

/* Put the new mean out.
------------------------*/
status = c_stput (&outlun, data_type, &ocls, &is, shape, outmean);
if ((status  > E_NOSI) && (status != E_DTNEW)) 
	{
   	erwrite("the mean");
	}

free(outmean);
free(inmean);

return(status); 

}
