/* ---------------------------------------------------------------------------

   Name- getsort

   Language- C     

   Version- 1.0   Date-  6-OCT-89  Programmer- F. Irani     (V5.0 Conversion)

   Function: Gets class means for a specified band and sorts 
	     corresponding class indicies ascending or descending
	     by the mean values.

--------------------------------------------------------------------------- */

#include "editstat.h"

void getsort
(
    long *sortlist	/* List of class indicies to be sorted  	*/
)

{

long i;               /* General purpose array index    */ 
long j;               /* General purpose array index    */
long status;          /* Status flag for function calls */
long inf_err = 1;     /* Information flag for c_errmsg  */
long fatal_err = -1;  /* Fatal error flag for c_errmsg  */
long zero = 0;        /* General programmer usage       */ 

double	hold,		/* Temporary variable for sorting 		   */
	*mvector,	/* Temporary space to hold class level mean vector */
	*svector;	/* Temporary space to hold mean values for bands   */

long 	shape[9];	/* Shape vector for mean vector			   */

/* Allocate space for sort arrays.
----------------------------------*/
if ( (mvector = (double *) malloc( MAXBND * sizeof(double))) == NULL )
	{
        c_errmsg ("Error allocating dynamic memory", "editstat-alloc",
                   &inf_err);
        c_errmsg ("Fatal error encountered","editstat-fatal",&fatal_err);
	}
if ( (svector = (double *) malloc( (nclass+1) * sizeof(double))) == NULL )
	{
        c_errmsg ("Error allocating dynamic memory", "editstat-alloc",
                   &inf_err);
        c_errmsg ("Fatal error encountered","editstat-fatal",&fatal_err);
	}

/* Collect all class mean values for specified band.
   The indexes into sortlist always start with 1 instead of zero because
   the first element is reserved for the data at the image level
-----------------------------------------------------------------------*/
for (ic = 1; ic <= nclass; ++ic)
	{
	status = c_stget(&inlun, "MEAN_VECTOR",	&ic, &zero, shape, mvector);
	if (status == E_NODT)
		{
		sprintf(estring,"No mean vector found for class %d.",ic);
		c_errmsg(estring,"editstat-nomean", &inf_err);
		c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
		}	
	if (shape[2] < sband)
		{
		sprintf(estring,"Band specified (%3d) is greater than %3d.",
		        sband, shape[2]);
		c_errmsg(estring,"editstat-bands", &inf_err);
		c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
		}

	svector[ic] = mvector[sband - 1];
	}

/* Sort array of class means - ascending.
-----------------------------------------*/
for (i = 1; i < nclass; ++i)
	{
	for (j = i+1; j <= nclass; ++j)
		{
		if (svector[i] < svector[j])
			continue;

		/* Sort mean values.
		--------------------*/
		hold = svector[i];
		svector[i] = svector[j];
		svector[j] = hold;

		/* Sort class indicie list.
		---------------------------*/
		hold = sortlist[i];
		sortlist[i] = sortlist[j];
		sortlist[j] = (long) hold;
		}
	}

/* If descending order specified then reverse array.
----------------------------------------------------*/
if (!sdir)
   {
   for (i = 1, j = nclass; i < j; ++i, --j)
      {
      hold = sortlist[i];
      sortlist[i] = sortlist[j];
      sortlist[j] = (long) hold;
      }
   }

free (mvector);
free (svector);

return;
}
