#include "editstat.h"

#define IMAGELEVEL	-1
#define ALLITEMS	 0

#define STATS		 1
#define DIRECTORY	-1
#define LISTALL	 0

/* Constant Strings */
#define BLANKLINE " "

/* array of strings for storing datatypes. */
#define MAX_STR_ARRAY 100
extern long lwidth;               /* line width of output listing         */
char *thebuf;	                 /* buffer used when calling stget */

/* Prototypes for static functions in this file  */

static void ls_i4_mat
(
    long data[],  /*  data[m][xdim]         */
    long xdim,   /* X dimension of matrix   */
    char *msg
);

static void ls_r4_mat
(
    float data[], /*  data[m][n]  */
    long xdim,
    long ydim,
    char *msg
);  

static void ls_r8_mat
(
    double data[],
    long dim,
    char *msg
);  

static void ls_r8_vec
(
    double data[],
    long dim,
    char *msg
);  

static void ls_r4_vec
(
    float data[],
    long dim,
    char *msg
);

static void ls_i4_vec
(
    long data[],
    long dim,
    char data_type[],
    char *msg
);

static void ls_i2_vec
(
    long data[],
    long dim,
    char data_type[],
    char *msg
);

static void ls_l1_char
(
    char data[],
    char *msg
);

static void ls_hex
(
    long data[],    /* Undefined data to dump   */
    long shape[],   /* shape vector         */
    char *msg
);
                    

/*===========================================================================*/
/*    name- st_write                                                         */
/*                                                                           */
/*    language- C                                                            */
/*                                                                           */
/*    version- 1.0    date- 30-jun-83   programmer- j. seamone, sasc         */
/*    revision 2.0    date  04-sep-86   programmer  n. deodhar/sar    #211   */
/*    version- 2.0    date- 25-Oct-89   Programmer- F. Irani/ STX            */
/*    version- 2.1    date- 28-Jun-93   Programmer- K. Sheffield/ EDC-STX    */
/*                                                                           */
/*    function- prints header information and selects an output format       */
/*         for one record of a statistics file.*                             */
/*                                                                           */
/*                                                                           */
/*    functions called-                                                      */
/*       ls_l1_char : formats and outputs a character estring                */
/*       ls_r4_vec  : formats and outputs a float vector                     */
/*       ls_r4_mat  : formats and outputs a float matrix                     */
/*       ls_r8_vec  : formats and outputs an double vector                   */
/*       ls_r8_mat  : formats and outputs a double matrix                    */
/*       ls_i4_vec  : formats and outputs an long  vector                    */
/*       ls_i4_mat  : formats and outputs an long matrix                     */
/*       ls_i2_vec  : formats and outputs an short int vector                */
/*       ls_hex     : generates a hexadecimal dump of the invalid record     */
/*                    passed to it                                           */
/*                                                                           */
/*    comments- if the type and dimension of the shape vector is not any of  */
/*       the valid combinations,a hexadecimal dump of the invalid record is  */
/*       generated and processing is terminated.                             */
/*===========================================================================*/

long st_write
(
    long *ic,             /* class number containing the data */
    long *is,                /* site number containing the data  */
    char *datatype,       /* the data type to be output       */
    long shape[9],        /* data dimension array             */
    char *msg               /* output header/message            */
)
{
   int i;
   
   long status;         /* Status flag for function calls */

   long fatal_err = -1; /* Fatal error flag for c_errmsg */

   
   /* 
     The shape vector contains the size/type of the
     the buffer variable needed 
     */
   int	size_of_data = 0;
   
   switch (shape[0]) {
    case EBYTE		: size_of_data = sizeof(unsigned char); break;
    case EWORD		: size_of_data = sizeof(short); break;
    case ELONG		: size_of_data = sizeof(long); break;
    case EREAL		: size_of_data = sizeof(float); break;
    case EDOUBLE		: size_of_data = sizeof(double); break;
    case ECOMP		: size_of_data = 2*sizeof(float); break;
    case EDCOMP		: size_of_data = 2*sizeof(double); break;
      default			: 
	c_errmsg("Shape data type invalid","eslist-fatal", &fatal_err);
      break;
   }
   
   /* determine if the data dimensions are valid. */

   if (shape[1] < 0 || shape[1] > 7)
     c_errmsg("Shape dimension out of range","eslist-fatal", &fatal_err);

   /* determine the buffer size needed for the data and then allocate it */

   for (i = 1; i <= shape[1]; i++)
     size_of_data *= shape[i +1];
   
   if ( (thebuf = (char *)calloc(1,size_of_data+1)) == NULL)
     c_errmsg("Error allocating memory","eslist-malloc", &fatal_err); 
   
   /* get the data from the stat file */

   status = c_stget( &inlun, datatype, ic, is, shape, thebuf);
   if (status != E_SUCC)
   {
      c_errmsg("Error reading stat file","eslist-fatal", &fatal_err);
   }
   
   /* Check type & shape[1] and call appropriate format function
      to format output.  If type & shape[1] are not appropriate,
      dump the data (in hexadecimal).
      -------------------------------------------------------------*/

   switch (shape[1])
   {
      /* 1D data */
    case 1:

      switch (shape[0])
      {
       case EDOUBLE:
	 
	 if ( (strncmp(datatype, "COV", 3) == 0) ||
	     (strncmp(datatype, "COR", 3) == 0) )
	   ls_r8_mat ((double *) thebuf, shape[2],msg);
      
	 if ( (strncmp(datatype, "COV", 3) != 0) &&
	     (strncmp(datatype, "COR", 3) != 0) )
	   ls_r8_vec ((double *) thebuf, shape[2],msg);
	 break;

       case EBYTE:
	 ls_l1_char (thebuf,msg);
	 break;
       case EWORD:
	 ls_i2_vec ((short *) thebuf, shape[2], datatype,msg);
	 break;
       case ELONG:
	 ls_i4_vec ((long *) thebuf, shape[2], datatype,msg);
	 break;
       case EREAL:
	 ls_r4_vec ((float *) thebuf, shape[2],msg);
	 break;
       default:
	 ls_hex (thebuf, shape,msg);
      }
      break;

      /* 2D data */

    case 2:
      switch(shape[0])
      {
       case ELONG:
	 ls_i4_mat ((long *) thebuf, shape[2],msg);
	 break;
       case EREAL:
	 ls_r4_mat ((float *) thebuf, shape[2], shape[3],msg);
	 break;
       default:
	 ls_hex (thebuf, shape,msg);
      }
      break;

    default:
      ls_hex (thebuf, shape,msg);
   }
   
   free(thebuf);
   
   return E_SUCC;
   
}

/*=================================================================*/
/*                                                                 */
/*  name- lsi4mat                                                  */
/*                                                                 */
/*  language- C                                                    */
/*                                                                 */
/*  version- 2.0    date- 30-Oct-89  programmer- F. Irani          */
/*  version- 1.0    date- 30-jun-83  programmer- J.W.Seamone       */
/*                                                                 */
/*                                                                 */
/*  function- ls_i4_mat is a function of ls_format and formats and */
/*  prints an long matrix. *                                       */
/*                                                                 */
/*  PDL:                                                           */
/*  begin ls_i4_mat                                                */
/*                                                                 */
/*  display header                                                 */
/*  do for length of vector                                        */
/*  if a new screen page is reached                                */
/*  reprint header                                                 */
/*  print vector element                                           */
/*  return                                                         */
/*                                                                 */
/*  end   ls_i4_mat                                                */
/*=================================================================*/


static void ls_i4_mat
(
    long data[],  /*  data[m][xdim]  		*/
    long xdim,	 /* X dimension of matrix	*/
    char *msg
)
{
   int i;
   
   char 	header1[81],	/* display header1		*/
   header2[81];	/* display header2		*/
   
   listline(BLANKLINE);
   if(msg != NULL)
     listline(msg);

   sprintf( header1, "POINT #  ROW         COLUMN");
   sprintf( header2, "-------  ----------  ----------");
   
   for (i = 0; i < xdim; ++i)
   {
      /* Print out the header lines at the top of every terminal screen.
         If output is going to other than the display terminal, t_lines
         will be zero.  In that case, write the header lines once,
         followed by all the data.
         ---------------------------------------------------------------*/
      if (i == 0)
      {
	 listline(header1);
	 listline(header2);
      }
      else if (t_lines > 2)
          if (i % (t_lines - 2) == 0)
          {
              listline(header1);
              listline(header2);
          }
      
      /*
	I4 matrix.
	----------				*/
      sprintf( estring, "    %3d%12d%12d", i+1, data[i+xdim], data[i]);
      listline(estring);
   }
   
   return;
   
}      

/*======================================================================*/
/*                                                                      */
/*  name- lsr4mat                                                       */
/*                                                                      */
/*  language- C	type- function                                          */
/*                                                                      */
/*  version- 2.0    date- 30-Oct-89  programmer- F. Irani               */
/*  version- 1.0    date- 30-jun-83  programmer- j.w.seamone            */
/*                                                                      */
/*  this code was borrowed from liststat to be included                 */
/*  in editstat. lsr4mat.cmt was version 1 dated 15-mar-82.             */
/*                                                                      */
/*  function- ls_r4_mat is a function of ls_format and formats and      */
/*  prints an long matrix. *                                            */
/*                                                                      */
/*  parameters-                                                         */
/*  variable                type i/o function                           */
/*  --------                ---- --- --------                           */
/*  data(n,m)               long  i   matrix containing the statistical */
/*  data.                                                               */
/*  n                       long  i   passed dimension of matrix        */
/*  m                       long  i   passed dimension of matrix        */
/*======================================================================*/

static void ls_r4_mat
(
    float data[], /*  data[m][n]  */
    long xdim,
    long ydim,
    char *msg
)

{
   int i,j;
   
   listline(BLANKLINE);
   if(msg != NULL)
     listline(msg);
   
   /*
     r4 matrix.
     ----------*/
   for (j=0; j < ydim; ++j)
   {
      sprintf( estring, "%4d", j+1);
      for (i = 0; i < xdim; ++i)
      {
	 if ( i == 3)
	 {
	    sprintf(ostring, " %16.9f     ", data[(j*xdim) + i] );
	    strcat(estring, ostring);
	 }
	 else
	 {
	    sprintf( ostring, "%16.9f", data[(j*xdim) + i] );
	    strcat(estring, ostring);
	 }
      }
      listline(estring);
   }
   
   return;
}      

/*
 **********************************************************************
 
 name- lsr8mat                        
 
 language- C     type- function
 
 version- 1.0     date- 30-jun-83  programmer- j.w.seamone
 version- 1.1     date- 29-may-86  programmer- n. deodhar, #129
 version- 2.0     date- 30-Oct-89  programmer- F. Irani
 
 this code was borrowed from liststat to be included
 in editstat.  lsr8mat.cmt was version 1 dated 07-mar-83.
 
 function- ls_r8_mat is a function of ls_format.it takes the 
 covariance 'matrix' (actually in vector format)and formats
 and outputs it in a standard covariance matrix format. 
 
 parameters-
 variable                type i/o function
 --------                ---- --- --------
 data(1)                 double  i   vector containing the statistical
 data.
 m                       long  i   passed dimension for vector
 
 **********************************************************************
 
 */
static void ls_r8_mat
(
    double data[],
    long dim,
    char *msg
)


{
   
   double **dispbuf;
   
   long	ice,
   ics,  	/* Input colum start */
   row,  	/* Input colum end   */
   ndim,
   pdim;
   int	i,j,k;

   long inf_err = 1;
   long fatal_err = -1;

   listline(BLANKLINE);
   /*
     Compute Covariance Matrix dimension 'ndim'.
     -------------------------------------------*/
   pdim = (lwidth-3)/14;
   ndim = (long) (( sqrt((8.0 * dim) + 1.0) -1.0) / 2.0 + .009);
   
   /*
     Allocate temporary space for matrix display.
     --------------------------------------------*/
   if ( ( dispbuf = (double **) malloc (ndim * sizeof(double *) ) ) == NULL)
   {
      c_errmsg ("Error allocating dynamic memory","editstat-alloc", &inf_err);
      c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
   }
   
   for (i = 0; i < ndim; ++i)
   {
      if (( dispbuf[i] = (double *) malloc(ndim * sizeof (double) )) == NULL)
      {
	 /* Free any successfully allocated memory.	
	    ------------------------------------------*/
	 for (j = 0; j < i; ++j)
	 {
	    free(dispbuf[j]);
	 }
	 free(dispbuf);
	 c_errmsg ("Error allocating dynamic memory","editstat-alloc", 
		   &inf_err);
	 c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
      }
   }
   
   /*
     Move data from input buffer into output matrix.
     -----------------------------------------------*/
   k = 0;
   for (i = 0; i < ndim; ++i)
     for (j = 0; j <= i; ++j)
     {
        dispbuf[i][j] = data[k]; 
	dispbuf[j][i] = data[k];
        ++k;
     }
   
   if (ndim  <  pdim) 
     pdim = ndim;
   
   if(msg != NULL)
     listline(msg);

   /*
     Write covar matrix including row and column counter.
     ----------------------------------------------------*/
   ics = 0;
   ice = pdim;
   
   while (ics  <  ndim)
   {
      sprintf(estring, "    ");
      for (k = ics; k < ice; ++k)
      {
	 sprintf(ostring, "%13d", k+1);
	 strcat(estring, ostring);
      }
      listline(estring);
      for (row = 0; row < ndim; ++row)
      {
	 sprintf(estring, "%3ld", row+1);
	 for ( k = ics; k < ice; ++k)
	 {
	    sprintf(ostring, " %13e",dispbuf[row][k]);
	    strcat(estring, ostring);
	 }
	 listline(estring);
      }
      ics += pdim;
      ice += pdim;
      if (ice  >  ndim)
	ice = ndim;
   }  		       /* while */
   
   /* 
     Free any successfully allocated memory.	
     ---------------------------------------*/
   for (i = 0; i < ndim; ++i)
   {
      free(dispbuf[i]);
   }
   free(dispbuf);
   
   return;
   
}          

/*
 **********************************************************************
 
 name- lsr8vec                        
 
 language- C     type- function
 
 version-  1.0    date- 30-jun-83  programmer- j.w.seamone
 
 this code was borrowed from liststat to be
 included in editstat.  lsr8vec.cmt was version 1
 dated 30-jun-82.
 
 function- ls_r8_vec is a function of ls_format and formats and
 prints an short vec *
 
 parameters-
 variable                type i/o function
 --------                ---- --- --------
 data(n)                 short  i   vector containing the statistical
 data
 dim                     long  i   passed dimension of vector
 
 **********************************************************************
 
 */
static void ls_r8_vec
(
    double data[],
    long dim,
    char *msg
)
{
   
   int i,j,k;
   
   long col;
   long row;
   long msize;
   
   char *cptr;
   
   
   if(msg != NULL)
   {
      msize = col = (long) strlen(msg);
      strcpy(estring,msg);
   }
   else
   {
      msize = col = 0;
      *estring = '\0';
   }
   
   cptr = estring;
   row = 0;
   
   for(i=0;i<dim;i++)
   {
      sprintf(ostring, " %13e", data[i]);
      k = strlen(ostring);
      col += k;
      
      if(col >= lwidth)
      {
	 listline(estring);
	 cptr = estring;
	 
	 if(row == 0)
	 {
	    j = 0;
	    
	    while(j < msize) 
	    {
	       *cptr = ' ';
	       cptr++;
	       j++;
	    }
	    row++;
	 }
	 else
	   cptr += msize;

	 *cptr = '\0';
	 col = msize;
      }
      strcat(cptr,ostring);
      cptr += k;
   }
   listline(estring);
   
   return;
}          

/*
 **********************************************************************
 
 name- lsr4vec                        
 
 language- C     type- function
 
 version-  1.0    date- 30-jun-83  programmer- j.w.seamone
 
 this code was borrowed from liststat to be
 included in editstat.  lsr4vec.cmt was version 1
 dated 30-jun-82.
 
 function- ls_r4_vec is a function of ls_format and formats and
 prints an short vec *
 
 parameters-
 variable                type i/o function
 --------                ---- --- --------
 data(n)                 short  i   vector containing the statistical
 data
 n                       long  i   passed dimension of vector
 
 **********************************************************************
 
 */
static void ls_r4_vec
(
    float data[],
    long dim,
    char *msg
)
{
   
   int i,j,k;
   
   long col;
   long row;
   long msize;
   
   char *cptr;
   
   
   if(msg != NULL)
   {
      msize = col = (long) strlen(msg);
      strcpy(estring,msg);
   }
   else
   {
      msize = col = 0;
      *estring = '\0';
   }
   
   cptr = estring;
   row = 0;
   
   for(i=0;i<dim;i++)
   {
      sprintf(ostring, " %13e", data[i]);
      k = strlen(ostring);
      col += k;
      
      if(col >= lwidth)
      {
	 listline(estring);
	 cptr = estring;
	 
	 if(row == 0)
	 {
	    j = 0;
	    while(j < msize) 
	    {
	       *cptr = ' ';
	       cptr++;
	       j++;
	    }
	    row++;
	 }
	 else
	   cptr += msize;
	 
	 *cptr = '\0';
      }
      strcat(cptr,ostring);
      cptr += k;
   }
   listline(estring);
   
   return;
}



/*
 **********************************************************************
 
 name- lsi4vec                        
 
 language- C     type- function
 
 version-  1.0    date- 30-jun-83  programmer- j.w.seamone
 
 this code was borrowed from liststat to be
 included in editstat.  lsi4vec.cmt was version 1
 dated 30-jun-82.
 
 function- ls_i4_vec is a function of ls_format and formats and
 prints an short vec *
 
 parameters-
 variable                type i/o function
 --------                ---- --- --------
 data(n)                 short  i   vector containing the statistical
 data
 n                       long  i   passed dimension of vector
 
 **********************************************************************
 
 */
static void ls_i4_vec
(
    long data[],
    long dim,
    char data_type[],
    char *msg
)
   

{
   
   
   char	lines[STNAME];
   
   long 	done,
   page,
   col,
   row,
   ioff,
   ncount;
   
   int	i;
   /*
     Fix up index for data_type = 'histogram'.
     -----------------------------------------*/

   if(msg != NULL)
     listline(msg);

   ioff = 1;
   if ( strncmp(data_type, "HIST", 4) == 0 )
     ioff = 0;
   /*
     Initialize for processing.
     --------------------------*/
   page = row = col = ncount = 0;
   ostring[0] = '\0';
   lines[0] = '\0';
   
   done = FALSE;
   while (!done)
   {
      ++page;
      if ( (page>1) && (ncount<dim) )
      {
	 sprintf( estring, "                             PAGE %ld",page);
	 listline(estring);
      }
      /*
	Begin processing.  Be sure to check for 
	multi-page and partial-page processing.
	---------------------------------------*/
      for (row = 0; row < 16; ++row)
      {
	 for (col=0; col < 4; ++col)
	 {
	    i = (col*16) + row + ((page-1)*64);
	    /*
	      Encode counter & vector.
	      ------------------------*/
	    if (i  <  dim)
	    {
	       sprintf(ostring, "%8ld%12ld", i+ioff, data[i]);
	       strcat(lines, ostring);
	       ++ncount;
	       ostring[0] = '\0';
	    }
	    else if (col == 0)
	    {
	       done = TRUE;
	       break;
	    }
	 } 
	 if (done)
	   break;
	 else
	 {
	    listline(lines);
	    lines[0]  = '\0';
	 }
      }
   }
   return;
}          

/*
 **********************************************************************
 
 name- lsi2vec                        
 
 language- C     type- function
 
 version-  1.0    date- 30-jun-83  programmer- j.w.seamone
 
 this code was borrowed from liststat to be
 included in editstat.  lsi2vec.cmt was version 1
 dated 30-jun-82.
 
 function- ls_i2_vec is a function of ls_format and formats and
 prints an short vec *
 
 parameters-
 variable                type i/o function
 --------                ---- --- --------
 data(n)                 short  i   vector containing the statistical
 data
 n                       long  i   passed dimension of vector
 
 **********************************************************************
 
 */
static void ls_i2_vec
(
    long data[],
    long dim,
    char data_type[],
    char *msg
)
{
   
   
   char	p[20][73];
   
   long 	page,
   col,
   row,
   ioff,
   ij,
   lrow = 0;
   
   int	i,k;

   if(msg != NULL)
     listline(msg);

   /*
     Fix up index for data_type = 'histogram'.
     -----------------------------------------*/
   ioff = 0;
   if ( strncmp(data_type, "HIST", 4) == 0 )
     ioff = -1;
   
   /*
     Initialize for processing.
     --------------------------*/
   page = i = 0;
   
   for (;;)   /*  do forever  	*/
   {
      for (k = 0; k < 20; k++)
        p[k][0] = '\0';
      /*
	Begin processing.  Be sure to check for 
	multi-page and partial-page processing.
	---------------------------------------*/
      for (col=0; col < 55; col +=18)
      {
	 for (row=0; row < 16; ++row)
	 {
	    ++i;
	    if (i  >  dim)
  	      goto ltwo; 		         
	    /*
	      Encode counter & vector.
	      ------------------------*/
	    sprintf(&p[row][col], " %ld", i+ioff);
	    sprintf(ostring, "    %ld", data[i-1]);
	    strcat(p[row],ostring);
	 } 
      }
      
      ++page;
      if (page > 1) 
      {
	 sprintf( estring, "                             PAGE %ld",page); /* 1 */
	 listline(estring);
      }
      
      for (ij = 0; ij < row; ++ ij)
      {
	 sprintf(estring, "%s ", p[ij]); 	/*--- (2) ---*/
	 listline(estring);
      }
      
   }
   
   /*
     Test for end of data/partial-page.
     ----------------------------------*/
 ltwo:
   
   if ( (i % 64)  ==  0)
   {
      return;
   }
   ++page;
   if (page > 1) 
   {
      sprintf( estring, "                             PAGE %ld",page); /* 1 */
      listline(estring);
   }
   
   if ( col == 0 ) 
     lrow = row;
   
   if ( col > 0 ) 
     lrow = 16;
   
   for (row = 0; row < lrow; ++row)
   {
      sprintf(estring, "%s ", p[row]); 	/*--- (2) ---*/
      listline(estring);
   }
   return;
}          

/*
 **********************************************************************
 
 name- lsl1char                       
 
 language- C     type- function
 
 version- 1.0    date- 30-jun-83  programmer- j.w.seamone
 version- 2.0    date- 30-Oct-89  programmer- F. Irani
 
 This code was borrowed from liststat to be included
 in editstat.
 
 function- ls_l1_char is a function of ls_format and formats and
 prints a character string. 
 
 PDL:
 begin ls_l1_char
 copy data buffer into character string via sprintf
 display the character string
 return
 end ls_l1_char
 **********************************************************************
 
 */

static void ls_l1_char
(
    char data[],
    char *msg
)

{
   /*
     L1 character : print data.
     --------------------------			*/
   if(msg != NULL)
     listline(msg);

   sprintf(estring, "%s", data);
   listline(estring);
   return;
}
/*
 **********************************************************************
 
 #include "editstat.h" 
  
 name- lshex                          
 
 language- C     type- function
 
 version- 1.0    date- 30-jun-83   programmer- j.w.seamone
 version- 2.0    date- 30-Oct-89  programmer- F. Irani
 
 function- ls_hex is a function of ls_format and generates a
 hexadecimal dump of an invalid data record . *
 
 PDL:
 begin ls_hex
 
 if data type = EBYTE:
 set n = 1;
 if data type =  EWORD:
 set n = 2;
 if data type =  ELONG:
 set n = 4;
 if data type =  EREAL:
 set n = 4;
 if data type =  EDOUBLE:
 set n = 8;
 if data type =  ECOMP:
 set n = 8;
 if data type =  EDCOMP:
 set n = 16;
 Determine number of characters to print
 for all characters
 display element header
 copy data to hex. character string via sprintf
 display hex. character on a single line
 return
 
 end ls_hex
 **********************************************************************
 
 */
static void ls_hex
(
    long data[],  	/* Undefined data to dump 	*/
    long shape[],	/* shape vector			*/
    char *msg
)

{
   
   int	i,j,k;

   if(msg != NULL)
     listline(msg);

   /*
     Determine the total number of short chars of data.
     -------------------------------------------------- */
   k = shape[2];
   
   /*
     Determine total number of unsigned char elements.
     -------------------------------------------------*/
   for (i = 1; i < shape[1]; ++i)
     k *= shape[i+2];
   
   /*
     For each display row.
     ---------------------*/
   for (j = 0; j < k; ++j)
   {
      sprintf(estring, "%10ld", j);
      listline(estring);
      sprintf( estring, "%16x", data[j]);
      listline(estring);
   } 
   
   return;
   
}  
