#include "editstat.h"
#include <math.h>
/*
     Function newcovr(matrix,shape,save, es_node)

**********************************************************************

   name- newcovr                        

   language- C

   Version- 2.0   Date- 30-Oct-89  Programmer- F. Irani, STX
   Version- 1.0   Date- 20-Jun-83  Programmer- R. J. Tighe

   function- controls the interface with the user. handles displays,
        prompts, error-checking, and editing functions.

*******************************************************************************

*/



void newcovr
(
    double matrix[],  /* Data matrix	*/
    long shape[],   /* Shape vector  */
    long *save,	   /* User wants to save flag 	*/
    char es_node[] /* Name of data level (image, class, site) for display */
)

{

long i;


double  new_value = 0.0, /* new value for matrix */
	root;      	/* Factor for dimension of expanded matrix */

long	col,		/* Column number 	*/
	delpage,  	/* Page index 		*/
	ival,     	/* Matrix index 	*/
	jval,     	/* Matrix index 	*/
	kval,     	/* Matrix index 	*/
	maxdim,   	/* Maximum matrix dimension 		*/
	maxpage,  	/* Maximum pages for this matrix 	*/
	newpage = 1, 	/* Next page index 	*/
	oldpage = 1, 	/* Last page index 	*/
	ok,		/* Flag to indicate valid matrix value	*/
	pdim = 5,    	/* Page dimension 	*/
	row;	     	/* Row number		*/

row = col = 1;

*save = FALSE;

root = ((8.0 * shape[2]) + 1.0);

maxdim = (long) (( sqrt(root) -1.0) / 2.0 + 0.5);

delpage = maxdim / pdim;

if ( maxdim - pdim * delpage != 0) 
	++delpage;

maxpage = delpage * delpage;

/*
Display a page of the covariance matrix.
----------------------------------------*/
ten:
for (;;) 	/* do forever or till back  or save. (10) ---*/
  {
  label();
  dispcovr (&oldpage, &newpage, matrix, &maxdim, &pdim, &delpage, es_node);
  /* Prompt for covariance matrix elements to be changed.
  -------------------------------------------------------*/
  for (;;)
    {
    sprintf(estring,
    "Enter: ROW COL to edit, Back, Save or P[Row Col] to Page[to Row Col]:");

    /* Read user entry.
    -------------------*/
    prompt(ostring);

    /* Check for "Back" or "SAVE".
    ------------------------------*/
    if (back)
    	{   
    	*save = FALSE;
	return;
    	}
    if (strncmp( ostring, "SAVE", len) == 0)
    	{
	*save = TRUE;
	return;
    	}

    /*
    Check for page.
    ---------------*/
    if (es_index(ostring, "P") == 0)
    	{
    	/*
    	Check if paging to a particular element.
    	----------------------------------------*/
	for (i = 0; i < len; ++i)
        	{
		if (ostring[i] >= '0' && ostring[i] <= '9')
			break;
		}

	if ( i < len && 
             sscanf(&ostring[i], "%ld %ld", &row, &col) == 2)
		{
		if (row > maxdim) 
			row = maxdim;
		if (col > maxdim)
			col = maxdim;
	     	if (row < 1) 
			row = 1;
		if (col < 1)
			col = 1;
		oldpage = newpage;
		newpage = ((row-1)/pdim) * delpage + ((col-1)/pdim) + 1;
		if (newpage > maxpage)
			newpage = 1;
		goto ten;
		} 
	else
		{
		++newpage;
		goto ten;
		}
    	}

    /* Extract row and column elements from user entry.
    ---------------------------------------------------*/
    if ( sscanf(ostring, "%ld %ld", &row, &col) != 2)
       {
       t_lclear (t_lines - 2, 1);
       t_output (t_lines - 2, 1,
	"[editstat-twoval]  Two values must be entered");
       continue;
       }  
    if ((row < 1) || (col < 1 ))
       {
       t_lclear (t_lines - 2, 1);
       t_output (t_lines - 2, 1,
       "[editstat-value]  Row and column values must be positive");
       continue;
       }  
    if ((row > maxdim) || (col > maxdim))
       {
       t_lclear (t_lines - 2, 1);
       t_output (t_lines - 2, 1,
       "[editstat-value]  Row and/or column value too large");
       continue;
       }  
    if (newpage != ( ((row-1)/pdim) * delpage + ((col-1)/pdim) + 1) )
        {
	newpage = ((row-1)/pdim) * delpage + ((col-1)/pdim) + 1;
	if (newpage > maxpage)
	newpage = 1;
	label();
	dispcovr(&oldpage, &newpage, matrix, &maxdim, &pdim, &delpage, es_node);
        }
    break;

    }
  for (;;)
    {
    /* Prompt for new covariance matrix value.
    ------------------------------------------*/
    t_lclear (t_lines - 1, 1);
    sprintf(estring, "Enter new value or Back to respecify ROW COL.");

    /* Read new value.
    ------------------*/
    prompt(ostring);
    if (back)
	{
	ok = FALSE;
	break;
	}
    ok = TRUE;

    /* Convert to internal double format.
    -------------------------------------*/
    if ( sscanf(ostring, "%lf", &new_value) != 1)
       {
       t_lclear (t_lines - 2, 1);
       sprintf(estring, "Invalid numeric format");
       t_output (t_lines - 2, 1, estring);
       ok = FALSE;
       continue;
       }

    if ((new_value < 0) && (row == col)) 
       {
       t_output (t_lines - 2, 1,
       "[edistat-diagonal]  No negative values on diagonal");
       ok = FALSE;
       continue;
       }
    break;
    }

  /*
  Change covariance matrix value.
  -------------------------------*/
  if (row > col)
	ival = row;
  else
	ival = col;
  if (row < col)
	jval = row;
  else
	jval = col;
  kval = ((ival*(ival-1)/2) + jval) - 1;

  /*
  Replace value only if it is valid.
  ----------------------------------------------*/
  if (ok) 
  	matrix[kval] = new_value;
  
  /*
  Set page and display again.
  ---------------------------*/
  oldpage = newpage;
  newpage = ((row-1)/pdim) * delpage + ((col-1)/pdim) + 1;
  
  }  /* end forever loop (was 10) */
  
}
