#include "editstat.h"
#include "typlim.h"

/*
**********************************************************************

   name- newhist                        expires- 12/31/99

   language- fortran     type- subroutine

   version- 1.0    date- 14-jun-83  programmer- l. urquhart

   function- displays histogram and prompts user for changes

   parameters-
      variable                type i/o function
      --------                ---- --- --------
      hist(256)               i*4  i/o histogram array
      sflag                   l*4  o   save flag

   subprograms called- 
      disphist - displays half of histogram 
      histind -  extracts first and second element from user input 
      intchar -  converts character string to integer value

**********************************************************************

*/


int newhist
(
    long hist[],		/* Histogram data		*/
    char es_node[]	/* histogram image/class level name buffer  */
)

{

long	h1,		/* Histogram bin change starting range no.	*/
	h2,		/* Histogram bin change ending   range no.	*/
	index,		/* Histogram element index			*/
	new,		/* New value for histogram 			*/
	page,		/* Page number for element			*/
	save;		/* User entered SAVE flag			*/

long i, status;

save = FALSE;

page = index = 0;

/*
Prompt for histogram elements to be changed until quit or save.
---------------------------------------------------------------*/
for (;;)
      {
      /* Display histogram (half).
      ----------------------------*/
      disphist (&page, &hist[index], es_node);

      ostring[0] = '\0';
      sprintf(estring,
      "Enter START END bins to change, Page[page#], Back or Save, ");
      prompt(ostring);
  
      /*
      If p entered, display next page of histogram.
      ---------------------------------------------*/
      if (strncmp (ostring, "P", 1) == 0)
         {
    	 /* Check if paging to a particular page.
    	 ----------------------------------------*/
	 for (i = 0; i < len; ++i)
	 	{
		if (ostring[i] >= '1' && ostring[i] <= '4')
			{
			ostring[i+1] = '\0';
			sscanf(&ostring[i], "%ld",  &page);
			--page;
			break;
			}
	 	}
	 if (i == len)
	 	++page;

	 if (page > 3)
		page = 0;

         index = 64 * page;
         continue;
         }
      
      /*
      Check for back or save.
      -----------------------*/
      if (back)
    	return(save);
          
      if (strncmp(ostring, "SAVE", len) == 0) 
      		{
            	save = TRUE;
            	return(save); 
      		}
      
      h2 = 0;

      /*
      Extract histogram elements from user entry.
      -------------------------------------------*/
      status = sscanf(ostring, "%ld %ld", &h1, &h2);
      if (status != 1 && status != 2)
         {
         continue;
         }

      /* Error if value is invalid.
      -----------------------------*/
      if (h1 < MINUTINY || h1 > MAXUTINY)
        {
      	t_lclear (t_lines - 2, 1);
      	t_output (t_lines - 2, 1,
	"[editstat-range]  Value specified is out of range");
	t_pos(t_lines, 1);
      	continue;
        }
  
      /* Error if second value < first.
      ---------------------------------*/
      if (h2 < h1)
        {
      	t_lclear (t_lines - 2, 1);
      	t_output (t_lines - 2, 1,
	"[editstat-range]  Second value is smaller than first");
	t_pos(t_lines, 1);
      	continue;
        }
  
      /*
      If second value not found, set second to first and return.
      ----------------------------------------------------------*/
      if (h2 == 0)
      		h2 = h1;
      else if (h2 < MINUTINY || h2 > MAXUTINY)
        {
      	t_lclear (t_lines - 2, 1);
      	t_output (t_lines - 2, 1,
	"[editstat-range]  Value specified is out of range");
	t_pos(t_lines, 1);
      	continue;
        }

      /* Prompt for new histogram value.
      ----------------------------------*/
      t_lclear (t_lines - 1, 24);
      ostring[0] = '\0';
      sprintf(estring, "Enter a non-negative new value or p to re-prompt:");
      prompt(ostring);

      /* If p entered, display next page of histogram.
      ------------------------------------------------*/
      if (strncmp (ostring, "PAGE", len) == 0)
         {
    	 /* Increment page.
    	 ------------------
	 ++page;
	 if (page > 3)
		page = 0;
         index = 64 * page;
    	 ------------------*/
         continue;
        }

      /* Check for quit or save.
      --------------------------*/
      if (back)
        	return(save);
          
      if (strncmp(ostring, "SAVE", len) == 0) 
      		{
            	save = TRUE;
            	return(save); 
      		}
  
      /* Convert to integer.
      ----------------------*/
      if (sscanf(ostring, "%ld", &new) != 1)       
         {
         t_lclear (t_lines - 2, 1);
      	 t_output (t_lines - 2, 1, "[editstat-invnum]  Invalid numeric format");
	 t_pos(t_lines, 1);
         continue;
         }

      if (new < 0)
         {
         t_lclear (t_lines - 2, 1);
      	 t_output (t_lines - 2, 1,
	"[editstat-range]  Value specified is out of range");
	 t_pos(t_lines, 1);
         continue;
         }
      
      /*
      Change histogram values.
      ------------------------*/
      for (i = h1; i <= h2; ++i)
         hist[i] = new;

      /*
      Set page to show h2.
      --------------------*/
      page = (long)( h2/64.0);
      index = page * 64;
  
      } 					/* for each page */

}
