#ifndef PROTOS_H
#define PROTOS_H

#define MAXPTS 5000
#include "las.h"
#include "statio.h"

int checkname
(
    char csname[]		/* Class or site name to check */
);

void check_page
(
    long *oldpage,
    long *newpage,
    long *delpage,
    long *pdim,
    long *maxdim,
    long *row,
    long *col,
    long *sizer,
    long *sizec
);

int chgsite
(
    long shape[],	/* Shape vector for data		*/
    long pbuf[]		/* Polygon data buffer from STATIO	*/
);

void comthm
(
    char newclass[],	/* Name of new class to hold the combined classes */
    char combcls[]	/* Name of class to be combined			  */
);

void covar_init
(
    double **matrix,
    long shape[]
);

void dispcovr
(
    long *oldpage,		/* Control for printing pages		*/
    long *newpage,		/* Control for printing pages		*/
    double *matrix,		/* Matrix data to be displayed		*/
    long *maxdim,		/* Maximum dimension for matrix		*/
    long *pdim,			/* Page dimension			*/
    long *delpage,		/* Used to figure page			*/
    char es_node[]		/* Node for which matrix is displayed	*/
);

void disphist
(
    long *page,		/* Histogram display page number (0 or 1) */
    long hist[],		/* Histogram array values		  */
    char es_node[]	/* Name of image, class or site level	*/
);

void es_drophist(void);

void dispmean
(
    double mean[],		/* mean vector data			*/
    long size,		/* Size of Vector			*/
    long index,		/* Index into vector			*/
    char es_node[]	/* image/class/site level of vector being displayed */
);

void disppoly
(
    long shape[], 		/* Length of polygon to display		*/
    long array[2][MAXPTS],	/* Polygon values to be displayed	*/
    long *npages		/* Number of pages to display polygon 	*/
);

int drsi
(void);

void dyn_alloc
(void);

void dyn_free
(void);

void es_copy
(
    long nincls,			/* no of classes to be copied		*/
    char cpy_cls[MAXCLS][STNAME],	/* names of classes to be copied	*/
    long blist[MAXBND],		/* band no to be copied			*/
    long num			/* Index into bands to be copied	*/
);

int es_index
(
    char s[],			/* String to search		*/
    char t[]			/* String to search for		*/
);

int es_pterm
(
    char *buf,			/* character buffer to be output	      */
    long *flag			/* flag to query user for command             */
);

int es_txtout
(
    char buf[],			/* Text to be written out                     */
    long print[],		/* Print options (FILE, LP, TERM)             */
    FILE *fp			/* File descriptor for print FILE or LP       */
);

int esout
(
    struct DESC *d_buf,
    long print[],
    int *fp
);

#ifdef unix
int esout_
(
    char buf[],
    long print[],
    FILE *fp
);
#endif

int esc_copy
(
    char data_type[],
    long shape[]
);

int esc_corre
(
    char data_type[],
    long shape[],
    long blist[],
    long num
);

int esc_covar
(
    char data_type[],
    long shape[],
    long blist[],
    long num
);

void esc_donode
(
    long blist[],
    long num
);

int esc_histo
(
    char data_type[],
    long shape[],
    long blist[],
    long num,
    long hcount
);

int esc_mean
(
    char data_type[],	/* Data item being processed			*/
    long shape[],	/* Shape Vector for data (length of mean vector)*/
    long blist[],	/* List of bands to be copied  			*/
    long num		/* Number of bands to be copied	    		*/
);

void get_bands
(
    long blist[MAXBND],		/* List of bands to be copied */
    long *num			/* Index into bands to be deleted	*/
);

int getcs
(
    long level     /* Highest level of search allowed:
			=0: prompt for image, class or site,
			=1: prompt for  class or site,
			=2: prompt for  site.			*/
);

void getsort
(
    long *sortlist	/* List of class indicies to be sorted  	*/
);

int listsit
(
    char listclass[]	/* Class name -- not a local variable  */
);

long st_write
(
    long *ic,             /* class number containing the data */
    long *is,                /* site number containing the data  */
    char *datatype,       /* the data type to be output       */
    long shape[9],        /* data dimension array             */
    char *msg               /* output header/message            */
);

void move
(
    char oldclass[],	/* Old class location 	*/
    char oldsite[],	/* Old site  location 	*/
    char newclass[],	/* New class location 	*/
    char newsite[]	/* New site  location 	*/
);

void newcovr
(
    double matrix[],  /* Data matrix	*/
    long shape[],   /* Shape vector  */
    long *save,	   /* User wants to save flag 	*/
    char es_node[] /* Name of data level (image, class, site) for display */
);

int newhist
(
    long hist[],		/* Histogram data		*/
    char es_node[]	/* histogram image/class level name buffer  */
);

int newmean
(
    double mean[],		/* Mean vector data 	*/
    long size,		/* Vector length	*/
    char es_node[]	/* Node name 		*/
);

void st_write_img
(void);

void datatype_hdr
(
    char *dtype,
    long shape[9],
    int cstype
);

void datatype_hdr
(
    char *dtype,
   long shape[9],
   int cstype
);

void image_hdr
(void);

void es_combcls
(void);

void es_dropcls
(void);

void es_dropsite
(void);

void es_editcor
(void);

void es_editcovr
(void);

void es_edithist
(void);

void es_editmean
(void);

void es_editsite
(void);

void es_movesite
(void);

void es_newclass
(void);

void es_rencls
(void);

void es_rensite
(void);

int get_classinfo
(
    long *tsites
);

void listcls
(void);

int statlist
(void);

void cull_list
(
    char **list,
   char keeplist[][STNAME],
   long numklist,
   long nlist,
   char type[]
);

void dname_add
(
    char *dname
);

int dname_srch
(
    char *dname
);

void cname_del
(
    char class_names[][STNAME],
   int offset,
   int *no_of_classes
);

void dname_clr
(
    int pos
);

void st_directory
(void);

void dir_data
(
    char *space
);

#ifdef vax
char *strdup
(
    char *str
);
#endif

void c_sterr
(
    long status,	/* Statio call return value to report on */
    long line
);

#endif
