/***********************************************************************
     FUNCTION statlist(nincls, cpy_cls)


   name- statlist                                   

   language- C
   version- 1.0     date- 04-mar-93  programmer- R. Kruse, EDC-software   
   version- 1.0.1   date- 02-jun-93  programmer- K. Sheffield, EDC-software 
   function- 'statlist' provides a formatted output of statistics files   
   	generated by the program 'stats' and/or any of the statistics file   
        manipulation programs.  File, line printer lisitings and/or       
	screen displays are generated of the directory and/or                
	data content (e.g. formatted vectors or matrices)of the file.        

   history- The original source code did not group the data types
	within sections of a report.  The new code prints all image
	level statistics, then prints the class/site information with each
	data type having its own section.  

       09/94  line 696 moved down five lines due to editstat-list bug
              when using large stat files with no training site data.
	      K.Lowell, CSB,EDC
********************************************************************** */
#include "editstat.h"
#include "util.h"

#ifdef vax
char * strdup();
#endif

#define IMAGELEVEL	-1
#define ALLITEMS	 0

#define STATS		 1
#define DIRECTORY	-1
#define LISTALL	 0

/* Constant Strings */
#define BLANKLINE " "

/* array of strings for storing datatypes. */
#define MAX_STR_ARRAY 100
char *str_array[MAX_STR_ARRAY];
int str_count = 0;             /* number of strings in str_array */
int elist_count = 0;           /* the number of datatypes to exclude */

/* long	status;	 status flag for function calls	*/


char dataname[STNAME];		/* data item to list (* for all) */
char sitename[1][STNAME];		/* site to list (* for all) */
char listoption[31];		/* list option to use */

char datatype[STNAME];		/* data item found in stnode */
char tmpstr[STNAME+5];

char **classlist;               /* pointer to buffer for holding class
				   names */
char **sitelist;                /* pointer to buffer for holding site
				   names */
long *numsites;


long ic_spec,ic_temp,is_temp,id_temp;	/* increment: class, site, data */

long shape[9];	                 /* used to retreive data dim. in stnode and 
				    stget */


long class_flg;	                 /* = IMAGELEVEL, ALLITEMS or number of classes
				    specified in the CLASSNAME array */
long site_flg;	                 /* = IMAGELEVEL, ALLITEMS or 1 for one site */
long all_data;	                 /* = ALLITEMS or 1 for one data item */
  
long list_flg;	                 /* = STATS, DIRECTORY or LISTALL */


long lwidth = 80;               /* line width of output listing         */


char classnames[MAXCLS][STNAME];	/* names of the classes to be listed */

long status;          /* Status flag for function calls */
long fatal_err = -1;  /* Fatal error flag for c_errmsg  */ 
long inf_err = -1;    /* Information flag for c_errmsg  */
long zero = 0;        /* General programmer usage       */


int statlist
(void)
{
   
   struct	VARIABLE *var;
   
   long no_of_classnames;
   char *cptr;                          /* pointer to classnames */
   char **sptr;                         /* pointer to sitelist */
   char **sp_tmp;                       /* temporary pointer to sitelist */
   
   long nsites;         /* number of sites for the current class */
   long ns_tmp;
   long tsites;         /* total number of sites                */
   
   

   long	i;		/* general purpose array index 		*/
   
   
   
   
   
   /* 
     Initialize output destination flags per run of LIST.
     ----------------------------------------------------
     */
   pflags[TERM] = pflags[LP] = pflags[FIL] = 0;
   
   strcpy(prtfile, "editstat");
   
   /* Get specification to list at terminal, file and/or line printer.
      -------------------------------------------------------------------*/
   status = c_setprt(&block, &pflags[TERM], &pflags[FIL], &pflags[LP], 
		     prtfile, usrfile[0]);
   if (status != E_SUCC)
   {
      c_errmsg ("Error setting print values.","ESLIST-fatal",&inf_err);
      printf("\n");
      quit = TRUE;
      return(E_FAIL);
   }
   if((printer = fopen(prtfile,"w")) == NULL)
   {
      c_errmsg ("Error setting print values.","ESLIST-fatal",&inf_err);
      quit = TRUE;
      return(E_FAIL);
   }
      
   /*-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-*/
   
   
   /*
     The classnames array is filled in editstat
     ------------------------------------------
     */
   var = p_find(&block,"CLASSNAM");
   strcpy(classnames[0], SVAL(*var,0));
   if (strncmp(classnames[0],"*",1) == 0)
   {
      no_of_classnames = nclass;
      class_flg = 0;
   }
   else if (strcmp(classnames[0],"0") == 0)
   {
      no_of_classnames = 0;
      class_flg = -1;
   }
   else
   {
      class_flg = 1;
      no_of_classnames = var->v_count;
   }
   
   for (i = 0; i < var->v_count; i++)
   {
      strcpy(classnames[i], SVAL(*var,i));
      len = (long) strlen(classnames[i]);
      c_low2up(classnames[i], &len);
   }
   /*-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-*/
   
   
   /*
     Get name of data item being listed.
     -----------------------------------
     */
   var = p_find(&block, "DATAITEM");
   strcpy(dataname, SVAL(*var,0));
   len = (long) strlen(dataname);
   c_low2up(dataname,&len);
   
   if (strncmp(dataname,"*",1) == 0)
     all_data = 0;
   else
     all_data = 1;

   /*
     The data items being excluded 
     ------------------------------
     */
   var = p_find(&block,"XCLDTYPE");
   for (i = 0; i < var->v_count; i++)
   {
      str_array[i] = strdup(SVAL(*var,i));
      len = (long) strlen(str_array[i]);
      c_low2up(str_array[i], &len);
   }
   str_count = elist_count = var->v_count;
   
   

   /*-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-*/
   
   
   /*
     Get site name.
     -----------------
     */
   var = p_find(&block, "SITENAM");
   strcpy(sitename[0], SVAL(*var,0));
   len = (long) strlen(sitename[0]);
   c_low2up(sitename[0],&len);
   
   if (strncmp(sitename[0],"*",1) == 0)
     site_flg = 0;
   else if (strncmp(sitename[0],"0",1) == 0)
     site_flg = -1;
   else
     site_flg = 1;
   /*-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-*/
   
   
   /*
     Get Specification to list directory, Statistics, or both.
     ------------------------------------------------------------
     */
   var = p_find(&block, "LISTOPT");
   strcpy(listoption, SVAL(*var,0));
   len = (long) strlen(listoption);
   c_low2up(listoption,&len);
   
   if (strncmp(listoption, "S", 1) == 0)
     list_flg   = STATS;
   if (strncmp(listoption, "B", 1) == 0) 
     list_flg   = LISTALL;
   if (strncmp(listoption, "D", 1) == 0) 
     list_flg   = DIRECTORY;

   var = p_find(&block,"LWIDTH");
   lwidth = IVAL(*var,0);
   
   /*-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-*/
   
   
   /*
     Process the directory data (if needed)
     --------------------------------------
     */
   
   /* get the class names from the stat file */
   status = get_classinfo(&tsites);

   /* set ic = is = 0 and increment id to find all the data components */
   if (list_flg == DIRECTORY || list_flg == LISTALL) 
   {
      st_directory ();
      
   }
   /*=====================================================================*/
   /*                                                                     */
   /*                                                                     */
   /*     Notes:	. The stat listing is output in the following     */
   /*                     format:                                         */
   /*                                                                     */
   /*       image	: IMAGENAME                                       */
   /*                 mean	XXXXX                                     */
   /*                 cov mat	A B C                                     */
   /*                         B E F                                       */
   /*                         C F G                                       */
   /*                 nbands	YYYYY                                     */
   /*                 ...                                                 */
   /*                 data_n	????                                      */
   /*                                                                     */
   /*       class information                                             */
   /*                 data_item                                           */
   /*                 mean                                                */
   /*                 value                                               */
   /*                 CLASS1	XXX                                       */
   /*                 CLASS2	YYY                                       */
   /*                 ...		..                                */
   /*                 CLASSN	NNN                                       */
   /*                                                                     */
   /*                 cov mat	class1                                    */
   /*                               A B C                                 */
   /*                               B E F                                 */
   /*                               C F G                                 */
   /*                 ...                                                 */
   /*                 cov mat	classN                                    */
   /*                               A B C                                 */
   /*                               B E F                                 */
   /*                               C F G                                 */
   /*                                                                     */
   /*                               ...                                   */
   /*                               data_n	????                      */
   /*       site information                                              */
   /*           CLASS1 information                                        */
   /*           <print data_items for this class only>                    */
   /*           <ie:class info is printed twice>                          */
   /*                                                                     */
   /*           <same as data_item section for classes>                   */
   /*           ...                                                       */
   /*           CLASSN information                                        */
   /*           <print data_items for this class only>                    */
   /*           <ie:class info is printed twice>                          */
   /*                                                                     */
   /*           <same as data_item section for classes>                   */
   /*                                                                     */
   /*       . If a specific data_item is specified then only that         */
   /*       data item is listed for any of the levels above               */
   /*                                                                     */
   /*       . if class_flg = -1 only image level data is printed          */
   /*                                                                     */
   /*       . if site_fg = -1 only image and class data is printed        */
   /*                                                                     */
   /*       . only one site or class can be specified, but many           */
   /*       classes can be specified                                      */
   /*                                                                     */
   /*       The above is the complete description of the report format    */
   /*=====================================================================*/
   
   /*
     Process the image level data
     ----------------------------
     
     Process the class level data
     ----------------------------
     */
   
   if (list_flg == STATS || list_flg == LISTALL) 
   {
      image_hdr();
      
      st_write_img ();
      
      
      /*
	List all class statistics
	-------------------------
	*/

      if(class_flg > 0)
	cull_list(classlist,classnames,no_of_classnames,nclass,"Class");
      

      if ( class_flg >= 0)
      {	/* list all classes */
	 dname_clr(elist_count);
	 dname_add("NAME");
	 
	 is = 0;	/* output only class data */
	 /* nclass global assigned in c_stopen() */
	 
	 for (ic=1; ic <= nclass; ++ic)
	 {	/* class loop */
	    if(classlist[ic-1] == NULL) continue;
	    
	    for (id = 1; (status = c_stnode (&inlun,&ic,&is,&id, datatype,shape)
			  ) == E_SUCC ; id++)
	      
	    { /* data loop */
	       
	       if(all_data && strcmp(datatype,dataname))
		 continue;
	       
	       status = dname_srch(datatype);
	       /*
		 Determine if the datatype needs to be printed
		 ---------------------------------------------
		 */
	       if (status != E_SUCC)
	       {	
		 
		 /* output the info for this datatype from 
		    all classes.  NOTE: the datatype is not
		    in classes found before the current
		    class.  ie. start srch from id to nclass */
		  
		  /* print header for datatype */
		  datatype_hdr(datatype,shape,0);
		  dname_add(datatype);
		  
		  for (ic_temp = ic; ic_temp <= nclass; ic_temp++)
		  {
		     cptr = classlist[ic_temp-1];
		     if(cptr != NULL)
		     {
			sprintf(tmpstr," %3d    %-10s",ic_temp,cptr);
			
			if(E_SUCC == c_stnode (&inlun,&ic_temp,&is,&id,
					       datatype,shape))
			  status = st_write(&ic_temp, &is, 
					    datatype, shape,tmpstr);
		     }
		     
		  }
	       }
	    } /* data loop */
	 } /* class loop */
      }
      
      else
      { /* class_flg == -1 .... done */
	 
	 listline_r(BLANKLINE);
	 listline_r(BLANKLINE);
	 sprintf(estring,"Class statistics not printed");
	 listline_r(estring);
      }
      
      
      /*
	List all site statistics
	-------------------------
	*/

      if(site_flg > 0)
	cull_list(sitelist,sitename,1,tsites,"Site");
      

      if ( site_flg >= 0)
      {	/* list all classes */
	 dname_clr(elist_count);
	 dname_add("NAME");
	 
	 is = 0;	/* output only class data */
	 /* nclass global assigned in c_stopen() */
	 
	 sptr = sitelist;
	 
	 for (ic=1; ic <= nclass; ++ic)
	 {	/* class loop */
	    nsites = numsites[ic-1];
	    if(classlist[ic-1] == NULL) 
	    {
	       sptr += nsites;
	       continue;
	    }
	    
	    for (is=1; is <= nsites; ++is)
	    {	/* site loop */
	    
	       for (id = 1;(status = c_stnode (&inlun,&ic,&is,&id, datatype,
					       shape)) == E_SUCC ; id++)
	      
	       { /* data loop */
	       
		  if(all_data && strcmp(datatype,dataname))
		    continue;
	       
		  status = dname_srch(datatype);
		  /*
		    Determine if the datatype needs to be printed
		    ---------------------------------------------
		    */
		  if (status != E_SUCC)
		  {	
		     
		 /* output the info for this datatype from 
		    all classes and sites.  NOTE: the datatype is not
		    in classes found before the current
		    class.  ie. start srch from id to nclass */
		     
		     /* print header for datatype */
		     datatype_hdr(datatype,shape,1);
		     dname_add(datatype);
		  
		     sp_tmp = sptr;
		     
		     for (ic_temp = ic; ic_temp <= nclass; ic_temp++)
		     {
			ns_tmp = numsites[ic_temp-1];
			cptr = classlist[ic_temp-1];
			if(cptr != NULL)
			{
			   for (is_temp = is; is_temp <= ns_tmp; is_temp++)
			   {
			      if(sp_tmp[is_temp-1] != NULL)
			      {
			
				 sprintf(tmpstr,"%3d:%-3d   %-10s",ic_temp,
					 is_temp, sp_tmp[is_temp-1]);
				 if(E_SUCC == c_stnode (&inlun,&ic_temp,
							&is_temp, &id, datatype
							,shape))
				   status = st_write(&ic_temp, &is_temp, 
						     datatype, shape, tmpstr);
			      }
			   }
			}
			sp_tmp += ns_tmp;
		     }
		  }
		  
	       } /* data loop */
	    } /* site loop */

	    sptr += nsites;
	 } /* class loop */
      }
      
      else
      { /* site_flg == -1 .... done */
	 
	 listline_r(BLANKLINE);
	 listline_r(BLANKLINE);
	 sprintf(estring,"Site statistics not printed");
	 listline_r(estring);
      }
      
   } /* list_flg == DIRECTORY */

  return E_SUCC;
}


/*=================================================================*/
/*  NAME: st_write_img                                             */
/*                                                                 */
/*  PURPOSE: To write out the image level data in the current stat */
/*           file.                                                 */
/*=================================================================*/

void st_write_img
(void)

{
   ic = is = 0;
   id = 0;
   for (id = 0;
	(status = c_stnode(&inlun, &ic, &is, &id, datatype, shape)) == E_SUCC;
	/* != E_SUCC -- no more data items at the image level */
	++id)
   {
      if ( (strncmp(datatype,dataname, len) == 0) ||
	  all_data == ALLITEMS )
      {		/* print data item */
	 st_write(&ic,&is,datatype, shape,NULL);
      }
      
      
   }
   
}
/*===========================================================================*/
/*  NAME:                                                                    */
/* datatype_hdr                                                              */
/*                                                                           */
/*  PURPOSE:                                                                 */
/* This function prints out a header or banner for each data type            */
/* being listed in the stat file.                                            */
/*===========================================================================*/


void datatype_hdr
(
    char *dtype,
    long shape[9],
    int cstype
)
{
   int i,j;
   
   

   listline(BLANKLINE);
   listline(BLANKLINE);
   
   sprintf(estring,"                     %s", dtype);
   listline(estring);
   sprintf(estring,"--------------------------------------------------------------------------------");
   listline(estring);
   listline(BLANKLINE);
   
   
   if(cstype)
     sprintf(estring,"cls:site# site name");
   else
     sprintf(estring,"class#  class name ");
   
   if(shape[1] == 1 && ( shape[0] == EREAL || (shape[0] == EDOUBLE && 
					       strncmp(dtype,"COV",3) && 
					       strncmp(dtype,"COR",3))))
   {
      i = shape[2];
      if(i > 3)
	i = 3;
      
      for(j=0;j<i;j++)
      {
	 sprintf(ostring," band %-8d",j+1);
	 strcat(estring,ostring);
      }
      if(shape[2] > i)
	strcat(estring," .... ");
   }
   listline(estring);
}

/*=================================================================*/
/*                                                                 */
/*  NAME: image_hdr                                                */
/*                                                                 */
/*  PURPOSE: This function prints out header information for the   */
/*           current stat file.                                    */
/*=================================================================*/


void image_hdr
(void)
{
   int i;
   
   sprintf(estring,"Statistics File          : %s",las);
   listline(estring);
   listline(BLANKLINE);
   
   sprintf(estring,"Number of Classes        : %ld", nclass);
   listline(estring);
   
     
   sprintf(estring,"Site(s) specified        : %s", sitename[0]);
   listline(estring);
   
     
   sprintf(estring,"Statistic data items     : %s", dataname);
   listline(estring);
   if(elist_count > 0)
   {
      i = 0;
      
      sprintf(estring,"Excluded data items      : %s", str_array[i]);
      listline(estring);
      i++;
      while (i < elist_count)
      {
	 sprintf(estring,"                         : %s", str_array[i]);
	 listline(estring);
	 i++;
      }
   }
   
   listline(BLANKLINE);
}



/*========================================================================*/
/* NAME: get_classinfo                                                    */
/*                                                                        */
/* PURPOSE: extracts the classnames and the number sites in a stat file.  */
/*                                                                        */
/*   Allocate memory for the class names and the list of sites per class. */
/*   For each class in the stat file, read in it's name and the number of */
/*   sites contain in it.                                                 */
/*========================================================================*/


int get_classinfo
(
    long *tsites
)
{
   
   char buf[STNAME];
   long i,j;
   long total;
   long nsites;
   char **ptr;
   
                                                                             
   if((classlist = (char **) calloc(nclass,sizeof(char *))) == NULL)         
   {                                                                         
      c_errmsg ("Error allocating dynamic memory","editstat-alloc", &inf_err); 
      c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);       
   }                                                                         

   if((numsites = (long *) calloc(nclass,sizeof(long))) == NULL)
   {
      c_errmsg ("Error allocating dynamic memory","editstat-alloc", &inf_err);
      c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
   }
   
   total = 0;
   ptr = classlist;
   
   for(j=0,i=1;i<=nclass;i++,j++)
   {
      /* get the class names */
      c_stget( &inlun, "NAME", &i, &zero, shape, buf);
      buf[shape[2]] = '\0';
      
      *ptr = strdup(buf);
      c_stget( &inlun, "NSITES", &i, &zero, shape, buf);
      switch (shape[0])
      {
       case EBYTE:
	 numsites[j] = *((unsigned char *) buf); break;
       case EWORD:
	 numsites[j] = *((short *) buf); break;
       case ELONG:
	 numsites[j] = *((long *) buf); break;
      }
      total += numsites[j];
      ptr++;
      
   }
   /* The following line of code was moved down five lines to be inside
      the next loop.  Would get a core dump if there were no sites specifed
      in the stat file during an editstat-list option using stats files
      generated by CLUSTER */
   /* *tsites = total; */
  
   
   if(total > 0)
   {
      *tsites = total;
      if((sitelist = (char **) calloc(total,sizeof(char *))) == NULL)
      {
	 c_errmsg("Error allocating dynamic memory","editstat-alloc", &inf_err);
	 c_errmsg("Fatal error encountered","editstat-fatal",&fatal_err);
      }

      ptr = sitelist;
      
      for(i=1;i<=nclass;i++)
      {
	 nsites = numsites[i-1];
	 for(j=1;j<=nsites;j++)
	 {
	    /* get the site names */
	    c_stget( &inlun, "NAME", &i, &j, shape, buf);
	    buf[shape[2]] = '\0';
	    *ptr = strdup(buf);
	    ptr++;
	 }
      }
      
   }
   return(E_SUCC);
}

      
/*===========================================================================*/
/* NAME:cull_list                                                            */
/*                                                                           */
/* PURPOSE:Keeps user specifed classes/sites in the list of all the classes/ */
/*         sites in a stat file.                                             */
/*===========================================================================*/



void cull_list
(
    char **list,            /* names of all the classes/sites          */
    char keeplist[][STNAME],/* names of the classes/sites to be kept */
    long numklist,          /* the number of items to be kept */
    long nlist,             /* the number of items in list   */
    char type[]            /* type of items in the list, eg. "CLASS" or "SITE" */
)

{
   int i, j;
   char **ptr;
   
   ptr = list;

   /* loop through the class list, keeping only the classes specified */
   
   for(i=0;i<nlist;i++)
   {
      for(j=0;j<numklist;j++)
	if(!strcmp(*ptr,keeplist[j])) /* found it, so go on to the next one */
	  break;

      /* haven't found the current item in the keep list, so nuke it */
      if(j == numklist)
      {
	 free(*ptr);
	 *ptr = NULL;
      }
      ptr++;
   }

   /* ok, now see if an item specified is really in the original list */

   for(j=0;j<numklist;j++)
   {
      ptr = list;
      for(i=0;i<nlist;i++,ptr++)
	if(*ptr != NULL && !strcmp(*ptr,keeplist[j])) /* found it, so go on 
	                                                 to the next one */
	  break;

      if(i == nlist)
      {
	 sprintf(estring,"%s %s not found",type, keeplist[j]);
	 c_errmsg(estring,"editstat-class", &inf_err);
      }
   }
}

      
/*===========================================================================*/
/*  NAME: dname_add                                                          */
/*                                                                           */
/*  PURPOSE: Adds a dataitem from a statfile to the list of processed items. */
/*===========================================================================*/


void dname_add
(
    char *dname
)
{
   
   if (str_count < MAX_STR_ARRAY)	
   {
      str_array[str_count] = strdup(dname);
      ++str_count;
   }
   else
   {
      /* print an error here */
      sprintf(estring,"Error allocating string (%s)", dname);
      c_errmsg(estring,"editstat-class", &inf_err);
   }
}

/*=================================================================*/
/* NAME:dname_srch                                                 */
/*                                                                 */
/* PURPOSE: Searches the list of dataitems for a particular item.  */
/*=================================================================*/

int dname_srch
(
    char *dname
)
{
   int	i;
   for (i=0; i< str_count; ++i)
   {
      if ( (status = strcmp(dname,str_array[i])) == 0)
      {
	 return(E_SUCC);			
      }
   }
   return(E_FAIL);
}
/*=================================================================*/
/*  NAME:cname_del                                                 */
/*                                                                 */
/*  PURPOSE:Deletes a specified name from the list of classnames.  */
/*=================================================================*/

void cname_del
(
    char class_names[][STNAME],
    int offset,
    int *no_of_classes
)
{
   int  i;

   for ( i = offset; i < (*no_of_classes -1); ++i)
   {
      class_names[i][0] = '\0' ;
      strcat(class_names[i], class_names[i+1]);
   }
   *class_names[ (*no_of_classes) ] = '\0';
   --(*no_of_classes);

}

/*=================================================================*/
/*  NAME: dname_clr                                                */
/*                                                                 */
/*  PURPOSE: Clears the list of dataitems.                         */
/*=================================================================*/

void dname_clr
(
    int pos
)
{
   int	i;
   
   for (i=pos; i < str_count; ++i)
   {
      free(str_array[i]);
   }
   str_count = pos;
}

/*============================================================================*/
/*  NAME: st_directory                                                        */
/*                                                                            */
/*  PURPOSE: This function lists all the classes, and sites in the current    */
/*           stat file.                                                       */
/*============================================================================*/

void st_directory
(void)
{
   long	nsites;
   char name[STNAME];
   /* char space[2][30] = { "          ", "                   " };  */
   char space[2][40];
   
   strcpy(space[0] , "        ");
   strcpy(space[1] , "                ");
   image_hdr();

   /* list all the classes */

   for ( ic = 1; ic <= nclass; ic++)
   {
      listline(BLANKLINE);
      sprintf(estring,"Class -        %s",classlist[ic-1]);
      listline(estring);
      /* listline(BLANKLINE); */
      
      /* list all the sites associated with this class */
      dir_data(space[0]);
      
      c_stget( &inlun, "NSITES", &ic, &zero, shape, &nsites);
      
      for (is = 1; is < nsites; is++)
      {
	 
	 c_stget( &inlun, "NAME", &ic, &is, shape, name);
	 
	 listline(BLANKLINE);
	 sprintf(estring,"      Site -        %s",name);
	 listline(estring);
	 listline(BLANKLINE);
	 
	 dir_data(space[1]);
      }
   }
   listline(BLANKLINE);
   listline(BLANKLINE);
}

/*===================================================================*/
/*  NAME: dir_data                                                   */
/*                                                                   */
/*  PURPOSE: To extract the class/site name from the stat file for   */
/*           the directory listing.                                  */
/*===================================================================*/

void dir_data
(
    char *space
)
{
   long	id;
   
   for (id = 1; 
	(status = c_stnode( &inlun, &ic, &is,
                        &id, datatype, shape)) == E_SUCC;
	++id)
   {
      if ( (strcmp(datatype,"NAME")) != 0 )
      {
	 sprintf(estring,"%s%s",space, datatype);
	 listline(estring);
      }
   }
}

#ifdef vax
char *strdup
(
    char *str
)
{
   char *ptr;

   ptr = (char *) malloc(strlen(str)+1);
   if(ptr != NULL)
      strcpy(ptr,str);
   return(ptr);
}
#endif

