/******************************************************************************

  NAME:	BAT_TPL	

  FUNCTION: 

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	EDITTIE must be run under TAE

  PROJECT:	LAS

  ALGORITHM DESCRIPTION:
  Allocate space for the tie point records
  Load the header and data records
  Retrieve the parameters relating to the header record and update the
      appropriate header fields
  Get the number of point ids entered
  For each tie point id entered
     Find the location of the tie point in the array
     If not found, add a new record
     Retrieve the parameters relating to the data records
     If enough values were entered 
        update the appropriate data fields
  Write the Tie Point file header and data records
  Free allocated space

  ALGORITHM REFERENCES: None

******************************************************************************/
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int bat_tpl
(
    struct PARBLK *vblock,			/*  TAE variable block		*/
    char hosin[][CMLEN],			/*  input host TPL file names	*/
    char *hosout,				/*  output host TPL file name	*/
    long *combin,				/*  combin two input files	*/
    long *new_file,				/*  TRUE if no input files  	*/
    char *subcmd				/*  subcommand			*/
)

{ /* bat_tpl */

struct PTDESC *tie;				/*  File descriptor		*/

struct TPLHEAD head1;			/*  TPL header record		*/
struct TPLDATA *data;			/*  TPL data records		*/
struct VARIABLE *var;			/*  pointer to TAE variable	*/

long acc = IWRITE;			/*  File access mode		*/
long curent = 0;			/*  Current data record 	*/
long count = 0;				/*  number of data records	*/
long i,j;				/*  loop counter		*/
long found;				/*  flag if ptid is found	*/
long length;				/*  length of command		*/
long line;				/*  dummy variable for load_tpl */
long nonfatal = 1;			/*  nonfatal error code		*/
long pidcnt;				/*  number of point ids entered */
long pptype = TPL;			/*  Type of tie point file	*/
long status;				/*  status flag			*/
long temp;				/*  temporary storage		*/
long temp1;				/*  temporary storage		*/
long maxtie;                            /*  maximum amount space allacated */


char ctemp[20];				/*  temporary character stroage */
char proj_str[10];			/*  projection charater string  */

/* Allocate buffer space for data records
-----------------------------------------*/
data = (struct TPLDATA *)calloc(MAX_TIE_POINTS,(sizeof(struct TPLDATA)));
if (data == NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","edittie-alloc",&nonfatal);
   return(E_FAIL);
   }
maxtie = MAX_TIE_POINTS;

/* Read the input files 
-----------------------*/
if (load_tpl(hosin,combin,new_file,&line,subcmd,&head1,&data,&count,&maxtie) 
            != E_SUCC)
   return(E_FAIL);

/* Get the projection validity
------------------------------*/
var = p_find(vblock,"ppval");
if ((*var).v_count != 0)
   {
   strcpy(ctemp,SVAL(*var,0));
   if (strncmp(ctemp,"IN",2) == 0)
      head1.proj_valid = GEO_NOTVALID;
   else
      head1.proj_valid = GEO_VALID; 
   }

/* Get projection type 
----------------------*/
var = p_find(vblock,"projtype");
if ((*var).v_count != 0)
   {
   strcpy (proj_str,SVAL(*var,0));
   if (strncmp(proj_str,"geo",3) == 0)
      head1.code=0;
   else if (strncmp(proj_str,"u",1) == 0)
      head1.code = 1;
   else if (strncmp(proj_str,"spc",3)==0) 
      head1.code = 2;
   else if (strncmp(proj_str,"alber",5) == 0) 
      head1.code = 3;
   else if ((strncmp(proj_str,"lambert c",9) == 0) || 
            (strncmp(proj_str,"lamc",4) == 0))
      head1.code = 4;
   else if (strncmp(proj_str,"me",2) == 0) 
      head1.code = 5;
   else if (strncmp(proj_str,"ps",2) == 0)
      head1.code = 6;
   else if (strncmp(proj_str,"poly",4) == 0) 
      head1.code = 7;
   else if (strncmp(proj_str,"equid",5) == 0) 
      head1.code = 8;
   else if (strncmp(proj_str,"t",1) == 0) 
      head1.code = 9;
   else if (strncmp(proj_str,"ste",3)==0) 
      head1.code = 10;
   else if ((strncmp(proj_str,"lambert a",9) == 0) || 
            (strncmp(proj_str,"lama",4) == 0))
      head1.code = 11;
   else if (strncmp(proj_str,"az",2) == 0)
      head1.code = 12;
   else if (strncmp(proj_str,"gn",2) == 0) 
      head1.code = 13;
   else if (strncmp(proj_str,"or",2) == 0)
      head1.code = 14;
   else if (strncmp(proj_str,"gvn",3) == 0) 
      head1.code = 15;
   else if (strncmp(proj_str,"sn",2) == 0) 
      head1.code = 16;
   else if (strncmp(proj_str,"eqr",3) == 0)
      head1.code = 17;
   else if (strncmp(proj_str,"mi",2)==0)
      head1.code = 18;
   else if (strncmp(proj_str,"v",1)==0) 
      head1.code = 19;
   else if (strncmp(proj_str,"ho",2)==0) 
      head1.code = 20;
   else if (strncmp(proj_str,"ro",2) == 0)
      head1.code = 21;
   else if (strncmp(proj_str,"so",2) == 0)
      head1.code = 22;
   else if (strncmp(proj_str,"alask",5) == 0)
      head1.code = 23;
   else if (strncmp(proj_str,"good",4) == 0)
      head1.code = 24;
   else if (strncmp(proj_str,"moll",4) == 0)
      head1.code = 25;
   else if (strncmp(proj_str,"imoll",5) == 0)
      head1.code = 26;
   else if (strncmp(proj_str,"ha",2) == 0)
      head1.code = 27;
   else if (strncmp(proj_str,"wagiv",5) == 0)
      head1.code = 28;
   else if (strncmp(proj_str,"wagvii",6) == 0)
      head1.code = 29;
   else if (strncmp(proj_str,"obeqa",5) == 0)
      head1.code = 30;
   else
      {
      sscanf(proj_str,"%d",&temp);
      if ((temp < 0) || (temp > 99))
         {
         c_errmsg("Invalid projection code","edittie-projcode",&nonfatal);
         return(E_FAIL); 
         }
      else
         head1.code = temp;
      }
   }

/* Get zone number 
-----------------*/
var = p_find(vblock,"zonenum");
if ((*var).v_count != 0)
   head1.zone = IVAL(*var,0);

/* Get datum code 
-----------------*/
var = p_find(vblock,"dcode");
if ((*var).v_count != 0)
   head1.datum = IVAL(*var,0);


/* Get projection coefficients 
------------------------------*/
var = p_find(vblock,"projparm");
if ((*var).v_count != 0)
   {
   for (i = 0; i < (*var).v_count ; i++)
       head1.projprms[i] = RVAL(*var,i);
   }

/* Get the four corner coordinates 
---------------------------------*/
var = p_find(vblock,"upleft");
if ((*var).v_count != 0)
   {
   head1.corners[0] = RVAL(*var,0);
   head1.corners[1] = RVAL(*var,1);
   }
var = p_find(vblock,"loleft");
if ((*var).v_count != 0)
   {
   head1.corners[2] = RVAL(*var,0);
   head1.corners[3] = RVAL(*var,1);
   }
var = p_find(vblock,"upright");
if ((*var).v_count != 0)
   {
   head1.corners[4] = RVAL(*var,0);
   head1.corners[5] = RVAL(*var,1);
   }
var = p_find(vblock,"loright");
if ((*var).v_count != 0)
   {
   head1.corners[6] = RVAL(*var,0);
   head1.corners[7] = RVAL(*var,1);
   }

/* Get ground distance in line and sample direction 
---------------------------------------------------*/
var = p_find(vblock,"pdline");
if ((*var).v_count != 0)
   head1.pdist[0] = RVAL(*var,0);

var = p_find(vblock,"pdsamp");
if ((*var).v_count != 0)
   head1.pdist[1] = RVAL(*var,0);

/* Get ground measurement units 
-------------------------------*/
var = p_find(vblock,"punits");
if ((*var).v_count != 0)
   {
   strcpy(head1.units,SVAL(*var,0));
   length = (long)strlen(head1.units);
   c_up2low(head1.units,&length);
   }

/* Get reference mode
---------------------*/
var = p_find(vblock,"reftype");
if ((*var).v_count != 0)
   {
   strcpy(ctemp,SVAL(*var,0));
   if (strcmp(ctemp,"TEMP") == 0)
       strcpy(head1.ref_mode,"CHIP-TEM");
   else if (strcmp(ctemp,"FULL") == 0)
       strcpy(head1.ref_mode,"CHIP-FUL");
   else if (strcmp(ctemp,"PRE") == 0)
       strcpy(head1.ref_mode,"CHIP-PRE");
   else
      strcpy(head1.ref_mode,ctemp);
   }

/* Get correlation mode
-----------------------*/
var = p_find(vblock,"corrmode");
if ((*var).v_count != 0)
   {
   strcpy(head1.corr_type,SVAL(*var,0));
   length = (long)strlen(head1.corr_type);
   c_low2up(head1.corr_type,&length);
   }

/* Get edge threshold 
---------------------*/
var = p_find(vblock,"edgeval");
if ((*var).v_count != 0)
   head1.edge_thres = RVAL(*var,0);

/* Get minimum correlation strength 
----------------------------------*/
var = p_find(vblock,"mincorr");
if ((*var).v_count != 0)
   head1.min_str = RVAL(*var,0);

/* Get maximum difference 
-------------------------*/
var = p_find(vblock,"maxdiff");
if ((*var).v_count != 0)
   head1.max_dis = RVAL(*var,0);

/* Get fitting method 
---------------------*/
var = p_find(vblock,"fitmeth");
if ((*var).v_count != 0)
   {
   strcpy(ctemp,SVAL(*var,0));
   if (strcmp(ctemp,"PARAB") == 0)
      strcpy(head1.peak_method,"PARABOLOID");
   else if (strcmp(ctemp,"GAUSS") == 0) 
      strcpy(head1.peak_method,"GAUSSIAN");
   else if (strcmp(ctemp,"RECIP") == 0) 
      strcpy(head1.peak_method,"RECIPROCAL");
   else if (strcmp(ctemp,"INTEGER") == 0) 
      strcpy(head1.peak_method,"NEAR_INT");
   else  
      strcpy(head1.peak_method,"NO_CORR");
   }

/* Get search image name 
-----------------------*/
var = p_find(vblock,"srchname");
if ((*var).v_count != 0)
   strcpy(head1.sea_name,SVAL(*var,0));

/* Get reference image name 
--------------------------*/
var = p_find(vblock,"refname");
if ((*var).v_count != 0)
   strcpy(head1.ref_name,SVAL(*var,0));

/* Get number of point ids entered.  Find the pt_id in the 
array or add a new record.  Loop through and get all values
entered for the data records.
--------------------------------------------------------------------*/
var = p_find(vblock,"ptid");
pidcnt = (*var).v_count;

for (i = 0; i < pidcnt; i++)
   {
   found = FALSE;
   temp = i * 2;
   temp1 = (i * 2) + 1;
   var = p_find(vblock,"ptid");
   strcpy(ctemp,SVAL(*var,i));
   for (j = 0; j < count; j++)
      {
      if (strcmp(data[j].pt_id,ctemp) == 0)
         {
         curent = j;
         found = TRUE;
         break;
         }
      }
   if (!found) 
      {
      if ((count + 1) > maxtie)
         {
         maxtie += MAX_TIE_POINTS;
         data = (struct TPLDATA *) realloc((char *)data,
                                      (maxtie * sizeof(struct TPLDATA)));
         }
      curent = count;
      strcpy(data[curent].pt_id,ctemp);
      ++count;
      }

   var = p_find(vblock,"coors");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].geo_coord[0] = RVAL(*var,temp);
         data[curent].geo_coord[1] = RVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"srchcoor");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].sea_coord[0] = RVAL(*var,temp);
         data[curent].sea_coord[1] = RVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"refcoors");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].ref_coord[0] = RVAL(*var,temp);
         data[curent].ref_coord[1] = RVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"rmserror");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].rms_error[0] = RVAL(*var,temp);
         data[curent].rms_error[1] = RVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"nomcoors");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].nominal[0] = RVAL(*var,temp);
         data[curent].nominal[1] = RVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"refsize");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].ref_size[0] = IVAL(*var,temp);
         data[curent].ref_size[1] = IVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"srchoff");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].offsets[0] = IVAL(*var,temp);
         data[curent].offsets[1] = IVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"elev");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > i)
         data[curent].elevation = RVAL(*var,i);
      }

   var = p_find(vblock,"scalfact");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > i)
         data[curent].zoom_factor = RVAL(*var,i);
      }

   var = p_find(vblock,"strength");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > i)
         data[curent].strength = RVAL(*var,i);
      }

   var = p_find(vblock,"displace");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > i)
         data[curent].displacement = RVAL(*var,i);
      }

   var = p_find(vblock,"refchip");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > i)
         strcpy(data[curent].ref_chip,SVAL(*var,i));
      }

   var = p_find(vblock,"srchsize");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].sea_size[0] = IVAL(*var,temp);
         data[curent].sea_size[1] = IVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"chipcoor");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].chip_coord[0] = IVAL(*var,temp);
         data[curent].chip_coord[1] = IVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"reqsize");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].req_size[0] = IVAL(*var,temp);
         data[curent].req_size[1] = IVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"cornvals");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].location[0] = IVAL(*var,temp);
         data[curent].location[1] = IVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"lscoors");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > temp1) 
         {
         data[curent].chip_loc[0] = IVAL(*var,temp);
         data[curent].chip_loc[1] = IVAL(*var,temp1);
         }
      }

   var = p_find(vblock,"actflg");
   if ((*var).v_count != 0)
      {
      if ((*var).v_count > i) 
         {
         if (strcmp(SVAL(*var,i),"ACCEPT") == 0)
            data[curent].active = 0;
         else if (strcmp(SVAL(*var,i),"REJECT") == 0)
            data[curent].active = 7;
         else 
            data[curent].active = data[curent].active + 10;
         } 
      }
   }

/* Write the file
----------------*/
if (*new_file && (count < 1))
   {
   c_errmsg("Error creating new Tie Point file","edittie-new",&nonfatal);
   return(E_FAIL);
   }
else
   {
   status = c_ptopen(&tie,hosout,&pptype,&acc,&head1);
   if (status != E_SUCC)
      {
      c_ptclse(&tie);
      return (E_FAIL);
      }
   for (i = 0; i < count; i++)
      {
      status = c_ptio(&tie,&(data[i]));
      if (status != E_SUCC)
         {
         c_ptclse(&tie);
         return (E_FAIL);
         }
      }
   c_ptclse(&tie);
   }

/* free the buffer
------------------*/
free(data);

return(E_SUCC);
} /* bat_tpl */
