/******************************************************************************

  NAME:	DATA_TPL	

  FUNCTION: Display the contents of the data records of a TPL file to the 
            terminal.

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	EDITTIE must be run under TAE

  PROJECT:	LAS

  ALGORITHM DESCRIPTION:
  Each output line string is formated with the information provided by the
  input arguments
  Each of the line strings is then output to the terminal.

  ALGORITHM REFERENCES: None.

******************************************************************************/
#include "las.h"
#include "geompak.h"
#include "protos.h"

void data_tpl
(
    struct TPLDATA *data,			/*  TPL data record		*/
    long *curent				/*  record to display		*/
)

{  /* data_tpl */

char line[81];				/*  buffer of output line 	*/
char temp[60];				/*  temporary buffer		*/

/*  Clear the terminal
----------------------*/
t_clear();

/*  format each line of output
------------------------------*/
sprintf(line,"Point Id: %s",data[*curent].pt_id);
t_output(1,2,line);

actflg(&(data[*curent].active),temp);
sprintf(line,"Active Flag: %s",temp);
t_output(2,2,line); 

sprintf(line,"Geographic Coordinates:  %16.5f   %16.5f",
        data[*curent].geo_coord[0],data[*curent].geo_coord[1]);
t_output(3,2,line);

sprintf(line,"Search Coordinates:      %16.5f   %16.5f",
        data[*curent].sea_coord[0],data[*curent].sea_coord[1]);
t_output(4,2,line);

sprintf(line,"Reference Coordinates:   %16.5f   %16.5f",
        data[*curent].ref_coord[0],data[*curent].ref_coord[1]);
t_output(5,2,line);

sprintf(line,"RMS error:               %16.5f   %16.5f",
        data[*curent].rms_error[0],data[*curent].rms_error[1]);
t_output(6,2,line);

sprintf(line,"Elevation of Point:   %16.5f",data[*curent].elevation);
t_output(8,2,line);

sprintf(line,"Scale Factor:         %16.5f",data[*curent].zoom_factor);
t_output(9,2,line);

sprintf(line,"Strength Of Correlation:     %16.5f",data[*curent].strength);
t_output(10,2,line);

sprintf(line,"Displacement Of Correlation: %16.5f",data[*curent].displacement);
t_output(11,2,line);

sprintf(line,"Nominal Cooridinates: %16.5f  %16.5f",
        data[*curent].nominal[0],data[*curent].nominal[1]);
t_output(12,2,line);

sprintf(line,"Reference Size:        %d     %d",
        data[*curent].ref_size[0],data[*curent].ref_size[1]);
t_output(13,2,line);

sprintf(line,"Search Offsets:        %d     %d",
        data[*curent].offsets[0],data[*curent].offsets[1]);
t_output(14,2,line);

sprintf(line,"Search Size:           %d     %d",
        data[*curent].sea_size[0],data[*curent].sea_size[1]);
t_output(15,2,line);

sprintf(line,"Chip Location:         %d     %d",
        data[*curent].chip_loc[0],data[*curent].chip_loc[1]);
t_output(16,2,line);

sprintf(line,"CHip Coordiantes:      %d     %d",
        data[*curent].chip_coord[0],data[*curent].chip_coord[1]);
t_output(17,2,line);

sprintf(line,"REquested Size:        %d     %d",
        data[*curent].req_size[0],data[*curent].req_size[1]);
t_output(18,2,line);

sprintf(line,"COrner Coordinates:    %d     %d",
        data[*curent].location[0],data[*curent].location[1]);
t_output(19,2,line);

sprintf(line,"Reference Chip Name: %-60.60s",data[*curent].ref_chip);
t_output(21,2,line);

}  /* data_tpl */
