/******************************************************************************

  NAME:	DSP_MTP	

  FUNCTION: Display and parse the command line for a MTP file.

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	EDITTIE must be run under TAE

  PROJECT:	LAS

  ALGORITHM DESCRIPTION:
  Allocate space for the data records
  Load the header and data records
  The fields of the header record are displayed to the terminal
  Until the quit or write command is issued
   The correct command line is displayed.  
   The input string is parsed and the correct subroutine is called.  
  Free the allocated space

  ALGORITHM DESCRIPTION: None.

******************************************************************************/
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int dsp_mtp
(
    char hosin[][CMLEN],			/*  input host MTP file names	*/
    char *hosout,				/*  output host MTP file name	*/
    long *combin,				/*  combin two input files	*/
    long *write_flag,			/*  write flag			*/
    long *new_file,				/*  TRUE if no input files  	*/
    char *subcmd				/*  subcommand			*/
)

{ /* dsp_mtp */

struct PTDESC *tie;				/*  File descriptor		*/

struct MTPHEAD head1;			/*  MTP header record		*/
struct MTPDATA *data;			/*  MTP data records		*/

long acc = IWRITE;			/*  File access mode		*/
long curent;				/*  Current data record 	*/
long com_flag = TRUE;			/*  Flag to determine which screen
					    and command line to display */
long count = 0;				/*  number of data records	*/
long edit_flag = FALSE;			/*  edit flag			*/
long i;					/*  loop counter		*/
long length;				/*  length of command		*/
long line;				/*  lines on the terminal	*/
long nonfatal = 1;			/*  nonfatal error code		*/
long pptype = MTP;			/*  Type of tie point file	*/
long status;				/*  status flag			*/
long temp;				/*  temporary storage		*/
long term;				/*  terminator of input string  */
long maxtie;                            /*  maximum amount space allacated */

char ans[81];				/*  buffer for input string	*/
char blank70[71];			/*  buffer for blank line	*/
char com[80];				/*  buffer for command line	*/
char com2[80];				/*  buffer for command line	*/
char *ptr;				/*  pointer to char. in input	*/

 
/*  Initialize variables
------------------------*/
strcpy(com,"COMMANDS: (H)ELP (Q)UIT (W)RITE (S)WITCH");
strcpy(com2,
"COMMANDS: (H)ELP (P)AGE (Q)UIT (W)RITE (S)WITCH (A)DD (R)EMOVE PT_ID ");
strcpy(blank70,
"                                                                      ");

/* Allocate buffer space for data records
-----------------------------------------*/
data = (struct MTPDATA *)calloc(MAX_TIE_POINTS,(sizeof(struct MTPDATA)));
if (data == NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","edittie-alloc",&nonfatal);
   return(E_FAIL);
   }
maxtie = MAX_TIE_POINTS;

/* Initialize TAE terminal I/0
------------------------------*/
edtint(&line);

/* Read the input files 
-----------------------*/
if (load_mtp(hosin,combin,new_file,&line,subcmd,&head1,data,&count,&maxtie) 
             != E_SUCC)
   return(E_FAIL);

/* display the tie point header record on the terminal
-----------------------------------------------------*/
head_mtp(hosout,&head1); 

/* do while the input string is not equal to quit 
-------------------------------------------------*/
do  	
   {
   /*  output the command line
   ---------------------------*/
   t_output(line - 1,2,blank70);
   if (com_flag)
      t_output(line - 1,2,com);
   else
      t_output(line - 1,2,com2);
   t_output(line,2,blank70);

   /*  Retrieve the input string
   -----------------------------*/
   t_input(line,2,ans,&term);
   if (term == P_OVER)
      t_output(line - 2,2,"[edittie-input] Input is larger than buffer size");

   /*  Make answer all lower case
   ------------------------------*/
   length = (long)strlen(ans);
   c_up2low(ans,&length);

   /*  blank out the error message line
   ------------------------------------*/
   t_output(line - 2,2,blank70);

   /*  Based on input string branch to correct subroutine.
   First check to see if a specific tie point was entered.
   ------------------------------------------------------*/
   if (!com_flag)
      {
      for (i = 0; i < count; i++)
         {
         if (strcmp(data[i].pt_id,ans) == 0)
            {
            curent = --i;
            strcpy(ans,"p");
            break;
            }
         }
      }
   if (strcmp(ans,"s") == 0)
      { /* switch */
      if (com_flag)
         {
         curent = 0;
         data_mtp(data,&curent);
         com_flag = FALSE;
         }
      else
         {
         head_mtp(hosout,&head1);
         com_flag = TRUE;
         }
      }
   else if (strncmp(ans,"a",1) == 0)
      { /* add */
      if ((count + 1) > maxtie)
         {
         maxtie += MAX_TIE_POINTS;
         data = (struct MTPDATA *) realloc((char *)data,
                                      (maxtie * sizeof(struct MTPDATA)));
         }
      ptr = ans;
      while (strncmp(ptr," ",1) != 0)
         (ptr) ++;
      if (strlen(ptr) > 1)
         (ptr) ++; 
      if (strlen(ans) <= 2)
          t_output(line - 2,2,"[edittie-code] Invalid edit command code");
      else if (strlen(ptr) >= 20)
          t_output(line - 2,2,"[edittie-length] String length is too long");
      else if (!c_ckptid(ptr))
          t_output(line - 2,2,"[edittie-add] Tie point already exsits");
      else
         {
         strcpy(data[count].pt_id,ptr);
         curent = count;
         data_mtp(data,&curent);
         ++count;
         edit_flag = TRUE;
         }
      }
   else if (strncmp(ans,"r ",2) == 0)
      { /* remove */
      ptr = ans;
      while (strncmp(ptr," ",1) != 0)
         (ptr) ++;
      if (strlen(ptr) > 1)
         (ptr) ++; 
      if (strlen(ans) <= 2)
           t_output(line - 2,2,"[edittie-code] Invalid edit command code");
      else if (strlen(ptr) >= 20)
           t_output(line - 2,2,"[edittie-length] String length is too long");
      else if (c_ckptid(ptr)) 
           t_output(line - 2,2,"[edittie-remove] Error removing tie point");
      else
         {
         temp = 0;
         for (i = 0; i < count; i++)
            {
            if (strcmp(ptr,data[i].pt_id) != 0)
               {
               data[temp] = data[i];
               ++temp;
               }
             }
         --count;
         edit_flag = TRUE;
         }
      }
   else if (strncmp(ans,"q",1) == 0)
      { /* quit */
      if (edit_flag)
         disquit(ans,&line);
      else
	 {
         t_output(line - 1,2,blank70);
         t_output(line,2,blank70);
         t_pos(line,1);
         }
      }
   else if (strncmp(ans,"h",1) == 0) 
      { /* help */
      ptr = ans;
      while (strncmp(ptr," ",1) != 0)
         (ptr) ++;
      if (strlen(ptr) > 1)
         (ptr) ++; 
      tiehelp(ptr,&line);

      /* Re-display the screen after help is displayed
      ------------------------------------------------*/
      if (com_flag)
         head_mtp(hosout,&head1);
      else
         data_mtp(data,&curent);
      }
   else if (strncmp(ans,"w",1) == 0)
      { /* write */
      if (*new_file && (count < 1))
        t_output(line - 2,2,"[edittie-new] Error creating new Tie Point file"); 
      else
         {
         status = c_ptopen(&tie,hosout,&pptype,&acc,&head1);
         if (status != E_SUCC)
            {
            t_pos(line - 2,2);
            c_ptclse(&tie);
            return (E_FAIL);
            }
         for (i = 0; i < count; i++)
            {
            status = c_ptio(&tie,&(data[i]));
            if (status != E_SUCC)
               {
               t_pos(line - 2,2);
               c_ptclse(&tie);
               return (E_FAIL);
               }
            }
         c_ptclse(&tie);

         /* successful write, now clear command and input line before exit 
         -----------------------------------------------------------------*/
         t_output(line - 1,2,blank70);
         t_output(line,2,blank70);
         t_pos(line,1);
         *write_flag = TRUE;
         strncpy(ans,"q",1);
         }   
      }
   else if (strcmp(ans,"p") == 0)
      { /* page */
      if (!com_flag)
         {
         ++curent;
         if (curent >= count)
            curent = 0;
         data_mtp(data,&curent); 
         }
      }
   else 
     edt_mtp(&head1,data,ans,&line,&curent,&edit_flag);
   } /* do while not quit */
while (strncmp(ans,"q",1) != 0);

/* free the buffer
-----------------*/
free(data);

return(E_SUCC);
} /* dsp_mtp */
