/******************************************************************************

  NAME:	EDT_MTP

  FUNCTION: Edit the contents of the MTP file interactively on the terminal.

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	EDITTIE must be run under TAE

  PROJECT:	LAS

  ALGORITHM DESCRIPTION:
  Compare the edit command code with the codes of each of the tie points fields
  When the correct code is found
     the correct field of the tie point is updated with the new value 
     the new value is output to the correct place on the terminal
     Validity checking is done when possible.  If an incorrect code is input, 
       an error message is output.

  ALGORITHM DESCRIPTION: None.

******************************************************************************/
#include "las.h"
#include "geompak.h"
#include "datum.h"
#include "protos.h"

void edt_mtp
(
    struct MTPHEAD *head,			/* MTP header record		*/
    struct MTPDATA *data,			/* MTP data records		*/
    char *edans_buf,			/* pointer to edit command code	*/
    long *line,				/* number of lines on the screen*/
    long *curent,				/* curent data record		*/
    long *edit_flag			/* edit flag			*/
)

{  /* edt_mtp */

long status;				/* status of called routines	*/
long temp;				/* temporary storage for long var*/

char *eqptr;				/* temporary pointer to command code*/
char blank5[6];				/* buffer to blank out 5 spaces	*/
char blank12[13];			/* buffer to blank out 12 spaces*/
char blank23[24];			/* buffer to blank out 23 spaces*/
char blank50[51];			/* buffer to blank out 50 spaces*/
char blank60[61];			/* buffer to blank out 60 spaces*/
char blank70[71];			/* buffer to blank out 70 spaces*/
char echo[80];				/* temp. buffer to hold output value*/

/*  Initialize the blank strings 
--------------------------------*/
strcpy(blank5,"     ");
strcpy(blank12,"            ");
strcpy(blank23,"                       ");
strcpy(blank50,"                                                  ");
strcpy(blank60,"                                                            ");
strcpy(blank70,
"                                                                      ");
/*  Initialize variables
------------------------*/
status = E_SUCC;

/*  Set temporary pointer to edit command buffer 
------------------------------------------------*/
eqptr = edans_buf;

/*  Find the start of the new value (1st non-space letter past the equal sign)
------------------------------------------------------------------------------*/
status = start_of_line(line,&eqptr,edans_buf);
if (status != E_SUCC)
   return;

/*  Compare the edit command code with each of the known codes of the 
fields.  When the correct code is found change the value of that field
and output the new value to the correct place on the terminal.  Make
sure that the place on the terminal is blanked out first to remove any
artifacts of the previous value.
-------------------------------------------------------------------------*/
if (strncmp(edans_buf,"pv",2) == 0)
   {
   validate(eqptr,echo,&(head->proj_valid),line);
   t_output(3,18,blank12);
   t_output(3,18,echo);
   }  /* if 'pv'  */

else if (strncmp(edans_buf,"cf",2) == 0)
   {
   if ((strcmp(eqptr,"no") == 0) || (strcmp(eqptr,"no-chips") == 0))
      {
      strcpy(head->chip_mode,"NO-CHIPS");
      t_output(3,52,blank12);
      t_output(3,52,head->chip_mode);
      }
   else if ((strcmp(eqptr,"temp") == 0) || (strcmp(eqptr,"chip-tem") == 0))
      {
      strcpy(head->chip_mode,"CHIP-TEM");
      t_output(3,52,blank12);
      t_output(3,52,head->chip_mode);
      }
   else if ((strcmp(eqptr,"full") == 0) || (strcmp(eqptr,"chip-ful") == 0))
      {
      strcpy(head->chip_mode,"CHIP-FUL");
      t_output(3,52,blank12);
      t_output(3,52,head->chip_mode);
      }
   else if ((strcmp(eqptr,"pre") == 0) || (strcmp(eqptr,"chip-pre") == 0))
      {
      strcpy(head->chip_mode,"CHIP-PRE");
      t_output(3,52,blank12);
      t_output(3,52,head->chip_mode);
      }
   else
      {
      t_output(*line - 2,2,"[edittie-chip] Invalid chip mode flag");
      status = E_FAIL;
      }
   } /* 'cf' */

else if (strncmp(edans_buf,"pc",2)== 0)
   {
   /*  output the correct projection based on the new projection code
   ------------------------------------------------------------------*/
   if ((strncmp(eqptr,"geo",3) == 0) || (strncmp(eqptr,"0",1) == 0))
      {
      strcpy(echo,"(0)GEOGRAPHIC");  
      head->code=0;
      }
   else if ((strncmp(eqptr,"u",1) == 0) || (strcmp(eqptr,"1") == 0))
      {
      strcpy(echo,"(1)UTM");  
      head->code = 1;
      }
   else if ((strncmp(eqptr,"spc",3) == 0) || (strcmp(eqptr,"2") == 0))
      {
      strcpy(echo,"(2)STATE PLANE");  
      head->code = 2;
      }
   else if ((strncmp(eqptr,"al",2) == 0) || (strcmp(eqptr,"3") == 0))
      {
      strcpy(echo,"(3)ALBERS CONICAL EQUAL AREA");  
      head->code = 3;
      }
   else if ((strncmp(eqptr,"lambert c",9) == 0) || 
            (strncmp(eqptr,"lamc",4) == 0) || (strcmp(eqptr,"4") == 0))
      {
      strcpy(echo,"(4)LAMBERT CONFORMAL CONIC");  
      head->code = 4;
      }
   else if ((strncmp(eqptr,"me",2) == 0) || (strcmp(eqptr,"5") == 0))
      {
      strcpy(echo,"(5)MERCATOR");  
      head->code = 5;
      }
   else if ((strncmp(eqptr,"ps",2) == 0) || (strcmp(eqptr,"6") == 0))
      {
      strcpy(echo,"(6)POLAR STEREOGRAPHIC");  
      head->code = 6;
      }
   else if ((strncmp(eqptr,"poly",4) == 0) || (strcmp(eqptr,"7") == 0))
      {
      strcpy(echo,"(7)POLYCONIC");  
      head->code = 7;
      }
   else if ((strncmp(eqptr,"equid",5) == 0) || (strcmp(eqptr,"8") == 0))
      {
      strcpy(echo,"(8)EQUIDISTANT CONIC");  
      head->code = 8;
      }
   else if ((strncmp(eqptr,"t",1) == 0) || (strcmp(eqptr,"9") == 0))
      {
      strcpy(echo,"(9)TRANSVERSE MERCATOR");  
      head->code = 9;
      }
   else if ((strncmp(eqptr,"ste",3) == 0) || (strncmp(eqptr,"10",2) == 0))
      {
      strcpy(echo,"(10)STEREOGRAPHIC");  
      head->code = 10;
      }
   else if ((strncmp(eqptr,"lambert a",9) == 0) || 
            (strncmp(eqptr,"lama",4) == 0) || (strncmp(eqptr,"11",2) == 0))
      {
      strcpy(echo,"(11)LAMBERT AZIMUTHAL EQUAL AREA");  
      head->code = 11;
      }
   else if ((strncmp(eqptr,"az",2) == 0) || (strncmp(eqptr,"12",2) == 0))
      {
      strcpy(echo,"(12)AZIMUTHAL EQUIDISTANT");  
      head->code = 12;
      }
   else if ((strncmp(eqptr,"gn",2) == 0) || (strncmp(eqptr,"13",2) == 0))
      {
      strcpy(echo,"(13)GNOMONIC");  
      head->code = 13;
      }
   else if ((strncmp(eqptr,"or",2) == 0) || (strncmp(eqptr,"14",2) == 0))
      {
      strcpy(echo,"(14)ORTHOGRAPHIC");  
      head->code = 14;
      }
   else if ((strncmp(eqptr,"gvn",3) == 0) || (strncmp(eqptr,"15",2) == 0))
      {
      strcpy(echo,"(15)GENERAL VERTICAL NEAR-SIDE PERSPECTIVE"); 
      head->code = 15;
      }
   else if ((strncmp(eqptr,"sn",2) == 0) || (strncmp(eqptr,"16",2) == 0))
      {
      strcpy(echo,"(16)SINUSIODAL");  
      head->code = 16;
      }
   else if ((strncmp(eqptr,"eqr",3) == 0) || (strncmp(eqptr,"17",2) == 0))
      {
      strcpy(echo,"(17)EQUIRECTANGULAR");  
      head->code = 17;
      }
   else if ((strncmp(eqptr,"mi",2) == 0) || (strncmp(eqptr,"18",2) == 0))
      {
      strcpy(echo,"(18)MILLER CYLINDRICAL");  
      head->code = 18;
      }
   else if ((strncmp(eqptr,"v",1) == 0) || (strncmp(eqptr,"19",2) == 0))
      {
      strcpy(echo,"(19)VAN DER GRINTEN");  
      head->code = 19;
      }
   else if ((strncmp(eqptr,"ho",2) == 0) || (strncmp(eqptr,"20",2) == 0))
      {
      strcpy(echo,"(20)OBLIQUE MERCATOR (Hotine)");  
      head->code = 20;
      }
   else if ((strncmp(eqptr,"ro",2)==0) || (strncmp(eqptr,"21",2) == 0))
      {
      strcpy(echo,"(21)ROBINSON");
      head->code = 21;
      }
   else if ((strncmp(eqptr,"so",2)==0) || (strncmp(eqptr,"22",2) == 0))
      {
      strcpy(echo,"(22)SOM");
      head->code = 22;
      }
   else if ((strncmp(eqptr,"alaska",5)==0) || (strncmp(eqptr,"23",2) == 0))
      {
      strcpy(echo,"(23)ALASKA CONFORMAL");
      head->code = 23;
      }
   else if ((strncmp(eqptr,"good",4)==0) || (strncmp(eqptr,"24",2) == 0))
      {
      strcpy(echo,"(24)INTERRUPTED GOODE HOMOLOSINE");
      head->code = 24;
      }
   else if ((strncmp(eqptr,"moll",4)==0) || (strncmp(eqptr,"25",2) == 0))
      {
      strcpy(echo,"(25)MOLLWEIDE");
      head->code = 25;
      }
   else if ((strncmp(eqptr,"imoll",5)==0) || (strncmp(eqptr,"26",2) == 0))
      {
      strcpy(echo,"(26)INTERRUPTED MOLLWEIDE");
      head->code = 26;
      }
   else if ((strncmp(eqptr,"ro",2)==0) || (strncmp(eqptr,"27",2) == 0))
      {
      strcpy(echo,"(27)HAMMER");
      head->code = 27;
      }
   else if ((strncmp(eqptr,"wagiv",5)==0) || (strncmp(eqptr,"28",2) == 0))
      {
      strcpy(echo,"(28)WAGNER IV");
      head->code = 28;
      }
   else if ((strncmp(eqptr,"wagvii",6)==0) || (strncmp(eqptr,"29",2) == 0))
      {
      strcpy(echo,"(29)WAGNER VII");
      head->code = 29;
      }
   else if ((strncmp(eqptr,"obeqa",5)==0) || (strncmp(eqptr,"30",2) == 0))
      {
      strcpy(echo,"(30)OBLATED EQUAL AREA");
      head->code = 30;
       }
   else
      {
      sscanf(eqptr,"%d",&temp);
      if ((temp < 0) || (temp > 99))
         {
         t_output(*line - 2,2,"[edittie-projcode] Invalid projection code");
         status = E_FAIL;
         }
      else
	 {
         strcpy(echo,eqptr);
         head->code = temp;
         }
      }
   if (status == E_SUCC)
      {
      t_output(4,14,blank60);
      t_output(4,14,echo);
      }
   }  /* if 'pc' */

else if (strncmp(edans_buf,"pz",2) == 0)
   {
   sscanf(eqptr,"%d",&temp);
   if ((temp < 0) || (temp > 5400))
      { /* Invalid zone code */
      t_output(*line - 2,2,"[edittie-zone] Invalid zone code");
      status = E_FAIL;
      } 
   else
      { /* valid zone code */
      strcpy(echo,eqptr);
      head->zone = temp;
      t_output(6,14,blank5);
      t_output(6,14,echo);
      }
   }  /* if 'pz' */

else if (strncmp(edans_buf,"dc",2) == 0)
   {
   sscanf(eqptr,"%d",&temp);
   if ((temp < 0) || (temp > MAX_DATUM))
      {
      t_output(*line - 2,2,"[edittie-datum] Invalid datum code");
      status = E_FAIL;
      }
   else
      { /* valid datum code */
      c_dtmstr(&temp,echo);
      head->datum = temp;
      t_output(5,14,blank60);
      t_output(5,14,echo);
      } 
   }  /* if 'dc' */

else if (strncmp(edans_buf,"ppa",3) == 0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(8,7,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[0]));
      sprintf(echo,"%16.5f",head->projprms[0]);
      t_output(8,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(8,26,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[1]));
      sprintf(echo,"%16.5f",head->projprms[1]);
      t_output(8,26,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(8,45,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[2]));
      sprintf(echo,"%16.5f",head->projprms[2]);
      t_output(8,45,echo);
      }
   else
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      {
      t_output(8,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(head->projprms[0]),
             &(head->projprms[1]),&(head->projprms[2]));
      sprintf(echo,"%16.5f   %16.5f   %16.5f",head->projprms[0],
              head->projprms[1],head->projprms[2]);
      t_output(8,7,echo);
      } 
   }  /* if 'ppa' */

else if (strncmp(edans_buf,"ppb",3) == 0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(9,7,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[3]));
      sprintf(echo,"%16.5f",head->projprms[3]);
      t_output(9,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(9,26,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[4]));
      sprintf(echo,"%16.5f",head->projprms[4]);
      t_output(9,26,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(9,45,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[5]));
      sprintf(echo,"%16.5f",head->projprms[5]);
      t_output(9,45,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(9,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(head->projprms[3]),
             &(head->projprms[4]),&(head->projprms[5]));
      sprintf(echo,"%16.5f   %16.5f   %16.5f",head->projprms[3],
              head->projprms[4],head->projprms[5]);
      t_output(9,7,echo);
      }
   }  /* if 'ppb' */

else if (strncmp(edans_buf,"ppc",3) == 0)
   {
   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(10,7,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[6]));
      sprintf(echo,"%16.5f",head->projprms[6]);
      t_output(10,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(10,26,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[7]));
      sprintf(echo,"%16.5f",head->projprms[7]);
      t_output(10,26,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(10,45,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[8]));
      sprintf(echo,"%16.5f",head->projprms[8]);
      t_output(10,45,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(10,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(head->projprms[6]),
             &(head->projprms[7]),&(head->projprms[8]));
      sprintf(echo,"%16.5f   %16.5f   %16.5f",head->projprms[6],
              head->projprms[7],head->projprms[8]);
      t_output(10,7,echo);
      }
   }  /* if 'ppc' */

else if (strncmp(edans_buf,"ppd",3) == 0)
   {

   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(11,7,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[9]));
      sprintf(echo,"%16.5f",head->projprms[9]);
      t_output(11,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(11,26,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[10]));
      sprintf(echo,"%16.5f",head->projprms[10]);
      t_output(11,26,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(11,45,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[11]));
      sprintf(echo,"%16.5f",head->projprms[11]);
      t_output(11,45,echo);
      }
   else
      {
      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      t_output(11,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(head->projprms[9]),
             &(head->projprms[10]),&(head->projprms[11]));
      sprintf(echo,"%16.5f   %16.5f   %16.5f",head->projprms[9],
              head->projprms[10],head->projprms[11]);
      t_output(11,7,echo);
      }
   }  /* if 'ppd' */

else if (strncmp(edans_buf,"ppe",3) == 0)
   {

   /*  set projection parameter based on the column number
   -------------------------------------------------------*/
   if (edans_buf[3] == '1')
      {
      t_output(12,7,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[12]));
      sprintf(echo,"%16.5f",head->projprms[12]);
      t_output(12,7,echo);
      }
   else if (edans_buf[3] == '2')
      {
      t_output(12,26,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[13]));
      sprintf(echo,"%16.5f",head->projprms[13]);
      t_output(12,26,echo);
      }
   else if (edans_buf[3] == '3')
      {
      t_output(12,45,blank23);
      sscanf(eqptr,"%lf",&(head->projprms[14]));
      sprintf(echo,"%16.5f",head->projprms[14]);
      t_output(12,45,echo);
      }
   else

      /*  Set all three projection parameters in the row
      --------------------------------------------------*/
      {
      t_output(12,7,blank70);
      sscanf(eqptr,"%lf %lf %lf",&(head->projprms[12]),
      &(head->projprms[13]),&(head->projprms[14]));
      sprintf(echo,"%16.5f   %16.5f   %16.5f",head->projprms[12],
      head->projprms[13],head->projprms[14]);
      t_output(12,7,echo);
      }
   }  /* if 'ppe' */

else if ((strncmp(edans_buf,"ul",2) == 0) || (strncmp(edans_buf,"ccul",4) == 0))
   {
   t_output(14,14,blank60);
   sscanf(eqptr,"%lf %lf",&(head->corners[0]),&(head->corners[1]));
   sprintf(echo,"%18.5f   %18.5f",head->corners[0], head->corners[1]);
   t_output(14,14,echo);
   }  /* if 'ul' */

else if ((strncmp(edans_buf,"ur",2)==0) || (strncmp(edans_buf,"ccur",4) == 0))
   {
   t_output(15,14,blank60);
   sscanf(eqptr,"%lf %lf",&(head->corners[4]),&(head->corners[5]));
   sprintf(echo,"%18.5f   %18.5f",head->corners[4], head->corners[5]);
   t_output(15,14,echo);
   }  /* if 'ur' */

else if ((strncmp(edans_buf,"ll",2) == 0) || (strncmp(edans_buf,"ccll",4) == 0))
   {
   t_output(16,14,blank60);
   sscanf(eqptr,"%lf %lf",&(head->corners[2]),&(head->corners[3]));
   sprintf(echo,"%18.5f   %18.5f",head->corners[2], head->corners[3]);
   t_output(16,14,echo);
   }  /* if 'll' */

else if ((strncmp(edans_buf,"lr",2) == 0) || (strncmp(edans_buf,"cclr",4) == 0))
   {
   t_output(17,14,blank60);
   sscanf(eqptr,"%lf %lf",&(head->corners[6]),&(head->corners[7]));
   sprintf(echo,"%18.5f   %18.5f",head->corners[6], head->corners[7]);
   t_output(17,14,echo);
   }  /* if 'lr' */

else if (strncmp(edans_buf,"pd",2) == 0)
   {
   t_output(19,14,blank60);
   sscanf(eqptr,"%lf %lf",&(head->pdist[0]), &(head->pdist[1]));
   sprintf(echo,"  %16.5f     %16.5f",head->pdist[0], head->pdist[1]);
   t_output(19,14,echo);
   }  /* if 'pd' */

else if (strncmp(edans_buf,"pu",2) == 0)
   {
   strcpy(echo,eqptr);
   if (strlen(echo) > 12)
      {
      t_output(*line-2,2,"[edittie-length] String length is too long");
      status = E_FAIL;
      }
   else
      {
      t_output(20,15,blank12);
      t_output(20,15,echo);
      strcpy(head->units,eqptr);
      }
   }  /* if 'pu' */

else if (strncmp(edans_buf,"sn",2) == 0)
   {
   strcpy(head->sea_name,eqptr);
   sprintf(echo,"%-60s",head->sea_name); 
   t_output(21,18,blank60);
   t_output(21,18,echo);
   }/* if 'sn' */

else if (strncmp(edans_buf,"rn",2) == 0)
   {
   strcpy(head->ref_name,eqptr);
   sprintf(echo,"%-60s",head->ref_name); 
   t_output(22,18,blank60);
   t_output(22,18,echo);
   } /* if 'rn' */

else if (strncmp(edans_buf,"pi",2) == 0)
   {
   if (strlen(eqptr) > 20)
      {
      t_output(*line - 2,2,"[edittie-length] String length is too long");
      status = E_FAIL;
      }
   else if (c_ckptid(eqptr))
      {
      strcpy(data[*curent].pt_id,eqptr);
      t_output(1,12,blank60);
      t_output(1,12,data[*curent].pt_id);
      }
   else
      {
      t_output(*line - 2,2,"[edittie-add] Tie point already exists");
      status = E_FAIL;
      }
   } /* if 'pi' */

else if (strncmp(edans_buf,"gc",2) == 0)
   {
   sscanf(eqptr,"%lf %lf",&(data[*curent].geo_coord[0]),
                          &(data[*curent].geo_coord[1]));
   sprintf(echo,"%16.5f   %16.5f",data[*curent].geo_coord[0], 
                                 data[*curent].geo_coord[1]);
   t_output(3,26,blank50);
   t_output(3,26,echo);
   } /* if 'gc' */

else if (strncmp(edans_buf,"rcn",3)== 0)
   {
   strcpy(data[*curent].ref_chip,eqptr);
   sprintf(echo,"%-60s",data[*curent].ref_chip);
   t_output(17,23,blank60);
   t_output(17,23,echo);
   } /* if 'rcn' */

else if (strncmp(edans_buf,"rc",2) == 0)
   {
   sscanf(eqptr,"%lf %lf", &(data[*curent].ref_coord[0]),
          &(data[*curent].ref_coord[1]));
   sprintf(echo,"%16.5f   %16.5f",data[*curent].ref_coord[0],
           data[*curent].ref_coord[1]);
   t_output(5,26,blank50);
   t_output(5,26,echo);
   } /* if 'rc' */

else if (strncmp(edans_buf,"ep",2) == 0)
   {
   sscanf(eqptr,"%lf", &(data[*curent].elevation));
   sprintf(echo,"%16.5f",data[*curent].elevation);
   t_output(7,24,blank50);
   t_output(7,24,echo);
   } /* if 'ep' */

else if (strncmp(edans_buf,"sf",2) == 0)
   {
   sscanf(eqptr,"%lf", &(data[*curent].zoom_factor));
   sprintf(echo,"%16.5f",data[*curent].zoom_factor);
   t_output(9,24,blank50);
   t_output(9,24,echo);
   } /* if 'sf' */

else if (strncmp(edans_buf,"nc",2) == 0)
   {
   sscanf(eqptr,"%lf %lf", &(data[*curent].nominal[0]),
          &(data[*curent].nominal[1]));
   sprintf(echo,"%16.5f  %16.5f",data[*curent].nominal[0],
           data[*curent].nominal[1]);
   t_output(11,24,blank50);
   t_output(11,24,echo);
   } /* if 'nc' */

else if (strncmp(edans_buf,"cs",2) == 0)
   {
   sscanf(eqptr,"%d %d", &(data[*curent].chip_size[0]),
          &(data[*curent].chip_size[1]));
   sprintf(echo,"%d     %d",data[*curent].chip_size[0],
           data[*curent].chip_size[1]);
   t_output(13,20,blank50);
   t_output(13,20,echo);
   } /* if 'cs' */

else if (strncmp(edans_buf,"so",2) == 0)
   {
   sscanf(eqptr,"%d %d", &(data[*curent].offsets[0]),
          &(data[*curent].offsets[1]));
   sprintf(echo,"%d     %d",data[*curent].offsets[0],
           data[*curent].offsets[1]);
   t_output(15,20,blank50);
   t_output(15,20,echo);
   } /* if 'so' */

else if (strncmp(edans_buf,"scn",3)== 0)
   {
   strcpy(data[*curent].sea_chip,eqptr);
   sprintf(echo,"%-60s",data[*curent].sea_chip);
   t_output(19,20,blank60);
   t_output(19,20,echo);
   } /* if 'scn' */

else
   {
   t_output(*line - 2,2,"[edittie-code] Invalid edit command code");
   t_output(*line,2,blank60);
   status = E_FAIL;
   } /* invalid code */

/*  Set edit flag to TRUE if no error had occured and a field was edited
------------------------------------------------------------------------*/
if (status == E_SUCC)
   *edit_flag = TRUE;

return;
}  /* edt_mtp */
