/****************************************************************************

  NAME:		GETPAR

  FUNCTION:	Retrive TAE parameters

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	EDITTIE must be run under TAE

  PROJECT:	LAS

  ALGORITHM DESCRIPTION:
  Retrieve the subcommand
  Depending on the subcommand
    retrieve the correct input and output names

  ALGORITHM REFERENCES: None.

******************************************************************************/
#include "las.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,		/* TAE variable block	*/
    char *subcmd,				/* Subcommand		*/
    char *hosout,				/* Output host file name*/
    char hosin[][CMLEN],		/* Input host file names*/
    char *type,					/* Type of tie point file*/
    double *mindist,            /* Minimum separation to allow*/
    long *combin,			    /* Combine two input files*/
    long *new_file				/* Flag if creating a new file*/
)

{ /* getpar */
struct VARIABLE *var;			/* pointer to TAE variable*/

long one = 1;					/* Constant of one	*/
long mode;					/* File access mode	*/
long zero = 0;					/* Constant of zero	*/
long str_len;                   /* string length */

/* Get the sumcommand
---------------------*/
var = p_find(vblock,"_SUBCMD");
strcpy(subcmd,SVAL(*var,0));

/* Get the file type, input and output file names
------------------------------------------------*/
if (strcmp(subcmd,"SCREEN") == 0)
   {
   var = p_find(vblock,"FTYPE");
   strcpy(type,SVAL(*var,0));
   str_len = (long)strlen(type);
   c_up2low(type,&str_len);
   var = p_find(vblock,"INTIE");
   if ((*var).v_count == 0)
      *new_file = TRUE;
   else
      {
      mode = IREAD;
      if (c_getnam(vblock,"INTIE",&zero,hosin[0],type,&mode) != E_SUCC)
          return(E_FAIL);
      if ((*var).v_count == 2)
         {
         if (c_getnam(vblock,"INTIE",&one,hosin[1],type,&mode) != E_SUCC)
             return(E_FAIL);
         *combin = TRUE;
         }
      }
   mode = IWRITE;
   if (c_getnam(vblock,"OUTTIE",&zero,hosout,type,&mode) != E_SUCC)
       return(E_FAIL);
   }
else if (strcmp(subcmd,"RMDUPS") == 0)
   {
   strcpy(type,"tpl");
   var = p_find(vblock,"MINDIST");
   *mindist = RVAL( *var, 0);
   var = p_find(vblock,"INTL");
   if ((*var).v_count == 0)
      return(E_FAIL); 
   else
      {
      mode = IREAD;
      if (c_getnam(vblock,"INTL",&zero,hosin[0],type,&mode) != E_SUCC)
          return(E_FAIL);
      if ((*var).v_count == 2)
         {
         if (c_getnam(vblock,"INTL",&one,hosin[1],type,&mode) != E_SUCC)
             return(E_FAIL);
         *combin = TRUE;
         }
      }
   mode = IWRITE;
   if (c_getnam(vblock,"OUTTL",&zero,hosout,type,&mode) != E_SUCC)
       return(E_FAIL);
   }
else if (strcmp(subcmd,"TPS") == 0) 
   {
   strcpy(type,"tps");
   var = p_find(vblock,"INTS");
   if ((*var).v_count == 0)
      *new_file = TRUE;
   else
      {
      mode = IREAD;
      if (c_getnam(vblock,"INTS",&zero,hosin[0],"tps",&mode) != E_SUCC)
          return(E_FAIL);
      if ((*var).v_count == 2)
         {
         if (c_getnam(vblock,"INTS",&one,hosin[1],type,&mode) != E_SUCC)
             return(E_FAIL);
         *combin = TRUE;
         }
      }
   mode = IWRITE;
   if (c_getnam(vblock,"OUTTS",&zero,hosout,type,&mode) != E_SUCC)
       return(E_FAIL);
   }
else if (strcmp(subcmd,"TPL") == 0) 
   {
   strcpy(type,"tpl");
   var = p_find(vblock,"INTL");
   if ((*var).v_count == 0)
      *new_file = TRUE;
   else
      {
      mode = IREAD;
      if (c_getnam(vblock,"INTL",&zero,hosin[0],type,&mode) != E_SUCC)
          return(E_FAIL);
      if ((*var).v_count == 2)
         {
         if (c_getnam(vblock,"INTL",&one,hosin[1],type,&mode) != E_SUCC)
             return(E_FAIL);
         *combin = TRUE;
         }
      }
   mode = IWRITE;
   if (c_getnam(vblock,"OUTTL",&zero,hosout,type,&mode) != E_SUCC)
       return(E_FAIL);
   }
else  
   {
   strcpy(type,"mtp");
   var = p_find(vblock,"INMT");
   if ((*var).v_count == 0)
      *new_file = TRUE;
   else
      {
      mode = IREAD;
      if (c_getnam(vblock,"INMT",&zero,hosin[0],type,&mode) != E_SUCC)
          return(E_FAIL);
      if ((*var).v_count == 2)
         {
         if (c_getnam(vblock,"INMT",&one,hosin[1],type,&mode) != E_SUCC)
             return(E_FAIL);
         *combin = TRUE;
         }
      }
   mode = IWRITE;
   if (c_getnam(vblock,"OUTMT",&zero,hosout,type,&mode) != E_SUCC)
       return(E_FAIL);
   }

return(E_SUCC);
} /* getpar */
