/******************************************************************************

  NAME:	HEAD_TPL

  FUNCTION: Display the contents of the header record of a TPL file to the 
            terminal.

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

  PROJECT:	LAS

  ALGORITHM DESCRIPTION:
  Each output line string is formated with the information provided by the
  input arguments
  Each of the line strings is then output to the terminal

  ALGORITHM REFERENCES: None.

******************************************************************************/
#include "las.h"
#include "geompak.h"
#include "protos.h"

void head_tpl
(
    char *taeout,				/*  output TAE tpl name		*/
    struct TPLHEAD *head,			/*  TPL header record		*/
    long *page_flag			/*  flag to determine header page */
)
{  /* head_tpl */

char line[81];				/*  buffer of output line 	*/
char proj_str[45];			/*  buffer of projection string */
char dtm_str[46];                       /*  buffer of datum string      */
char vald[16];				/*  validity string		*/

/*  Clear the terminal
----------------------*/
t_clear();

/*  format each line of output
------------------------------*/
if (*page_flag)
   {
   sprintf(line,"TPL File Name: %-60.60s",taeout);
   t_output(2,2,line);

   pntval(head->proj_valid,vald);
   sprintf(line,"Proj. Validity: %-15s   Reference Mode: %s",vald,
           head->ref_mode);
   t_output(4,2,line);

   c_prostr(&(head->code),proj_str);
   sprintf(line,"Proj. Code: %s",proj_str);
   t_output(5,2,line);

   c_dtmstr(&(head->datum),dtm_str);
   sprintf(line,"Datum Code: %s",dtm_str);
   t_output(6,2,line);

   sprintf(line,"Proj. Zone: %-d",head->zone);
   t_output(7,2,line);

   sprintf(line, "Proj. Parm:");
   t_output(9,2,line);
   sprintf(line,"A:   %16.5f   %16.5f   %16.5f",
        head->projprms[0],head->projprms[1],head->projprms[2]);
   t_output(10,2,line);
   sprintf(line,"B:   %16.5f   %16.5f   %16.5f",
         head->projprms[3],head->projprms[4],head->projprms[5]);
   t_output(11,2,line);
   sprintf(line,"C:   %16.5f   %16.5f   %16.5f",
         head->projprms[6],head->projprms[7],head->projprms[8]);
   t_output(12,2,line);
   sprintf(line,"D:   %16.5f   %16.5f   %16.5f",
         head->projprms[9],head->projprms[10],head->projprms[11]);
   t_output(13,2,line);
   sprintf(line,"E:   %16.5f   %16.5f   %16.5f",
         head->projprms[12],head->projprms[13],head->projprms[14]);
   t_output(14,2,line);

   sprintf(line,"Corner Corr:");
   t_output(16,2,line);
   sprintf(line,"ULcorner: %18.5f   %18.5f",head->corners[0],head->corners[1]);
   t_output(17,4,line);
   sprintf(line,"URcorner: %18.5f   %18.5f",head->corners[4],head->corners[5]);
   t_output(18,4,line);
   sprintf(line,"LLcorner: %18.5f   %18.5f",head->corners[2],head->corners[3]);
   t_output(19,4,line);
   sprintf(line,"LRcorner: %18.5f   %18.5f",head->corners[6],head->corners[7]);
   t_output(20,4,line);
   }
else
   {
   sprintf(line,"Proj. Dist:   %16.5f     %16.5f",head->pdist[0],
          head->pdist[1]);
   t_output(2,2,line);

   sprintf(line,"Proj. Units: %-10s",head->units);
   t_output(4,2,line);

   sprintf(line,"Edge Threshold:      %16.5f",head->edge_thres);
   t_output(6,2,line);

   sprintf(line,"Minimum Correlation: %16.5f",head->min_str);
   t_output(7,2,line);

   sprintf(line,"Maximum Difference:  %16.5f",head->max_dis);
   t_output(8,2,line);

   sprintf(line,"Fitting Method:   %s",head->peak_method);
   t_output(10,2,line);

   sprintf(line,"Correlation Mode: %s",head->corr_type);
   t_output(11,2,line);

   sprintf(line,"Search Name:    %-60.60s",head->sea_name);
   t_output(13,2,line);

   sprintf(line,"Reference Name: %-60.60s",head->ref_name);
   t_output(14,2,line);
   }

}  /* head_tpl */
