/****************************************************************************

  NAME:	LOAD_TPS

  FUNCTION: Read a TPS file and combine the data records if necessary.

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	EDITTIE must be run under TAE

  PROJECT:	LAS

  ALGORITHM DESCRIPTION:
  If no input file
     clear the header record
  Else 
     Read in the header record(s)
     If more than one input file 
        compare the header records 
        read in and combine the data records
     Else
        read in the data records

  ALGORITHM DESCRIPTION: None.

*****************************************************************************/
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int load_tps
(
    char hosin[][CMLEN],				/* Input host file names*/
    long *combin,					/* combine two input files*/
    long *new_file,					/* flag if creating a new file*/
    long *line,					/* lines on the terminal*/
    char *subcmd,					/* subcommand		*/
    struct TPSHEAD *head1,				/* TPS header record	*/
    struct TPSDATA **data,				/* TPS data record	*/
    long *count,					/* count of data records */
    long *maxtie					/* maximum space allocated */
)

{ /* load_tps */
struct TPSHEAD head2;				/* TPS header record	*/

struct PTDESC *tie;					/* File descriptor 	*/
struct PTDESC *tie2;				/* File descriptor	*/

long acc;					/* File mode access	*/
long i;						/* loop counter		*/
long nonfatal = 1;				/* Nonfatal error messages*/
long pptype = TPS;				/* File type		*/
long status;					/* Function status	*/
long temp;					/* temp storage 	*/
long eleven = 11;				/* constant value       */
struct TPSDATA *data1;				/* TPS data record	*/

data1 = *data;

/* Open tie point files.  If the input files are 
to be combined, compare the header information.
------------------------------------------------*/
if (*new_file)
   clrtps(head1); 
else
   {
   acc = IREAD;
   if (*combin)
      {
      if (c_ptopen(&tie,hosin[0],&pptype,&acc,head1) != E_SUCC)
          return(E_FAIL);
      if (c_ptopen(&tie2,hosin[1],&pptype,&acc,&head2) != E_SUCC)
          {
          c_ptclse(&tie);
          return(E_FAIL);
          }   
      if ((head1->pdist[0] != head2.pdist[0]) ||
          (head1->pdist[1] != head2.pdist[1]))
         {
         c_errmsg("Projection distance must be the same","edittie-head",
                  &nonfatal);
         c_ptclse(&tie);
         c_ptclse(&tie2);
         return(E_FAIL);
         }
      if (head1->proj_valid != head2.proj_valid)
         {
         c_errmsg("Projection validity must be the same","edittie-head",
                   &nonfatal);
         c_ptclse(&tie);
         c_ptclse(&tie2);
         return(E_FAIL);
         }
      if (head1->code != head2.code)
         {
         c_errmsg("Projection code must be the same","edittie-head",&nonfatal);
         c_ptclse(&tie);
         c_ptclse(&tie2);
         return(E_FAIL);
         }
      if (head1->zone != head2.zone)
         {
         c_errmsg("Projection zone code must be the same","edittie-head",
                  &nonfatal);
         c_ptclse(&tie);
         c_ptclse(&tie2);
         return(E_FAIL);
         }
      if (head1->datum != head2.datum)
         {
         c_errmsg("Projection datum code must be the same","edittie-head",
                  &nonfatal);
         c_ptclse(&tie);
         c_ptclse(&tie2);
         return(E_FAIL);
         }
      if (strcmp(head1->mode,head2.mode) != 0)
         {
         c_errmsg("Tie point mode must be the same","edittie-head",&nonfatal);
         c_ptclse(&tie);
         c_ptclse(&tie2);
         return(E_FAIL);
         }
      c_up2low(head1->units,&eleven);
      c_up2low(head2.units,&eleven);
      if (strcmp(head1->units,head2.units) != 0)
         {
         c_errmsg("Projection units must be the same","edittie-head",
                  &nonfatal);
         c_ptclse(&tie);
         c_ptclse(&tie2);
         return(E_FAIL);
         }
      for (i = 2; i < 15; i++)
         {
         if (head1->projprms[i] != head2.projprms[i])
            {
            c_errmsg("Projection parameters must be the same","edittie-head",
                      &nonfatal);
            c_ptclse(&tie);
            c_ptclse(&tie2);
            return(E_FAIL);
            }
         }

      /* Header records OK, now combine the data records
      --------------------------------------------------*/
      for (i = 0; ; i++)
          {
          if (i+1 == *maxtie)
             {
             *maxtie += MAX_TIE_POINTS;
             data1 = (struct TPSDATA *) realloc(data1,
                                      (*maxtie * sizeof(struct TPSDATA)));
             *data = data1;
             }
          status = c_ptio(&tie,&(data1[i]));
          if (status == E_EOF) break;
          if (status == E_FAIL)
             {
             c_ptclse(&tie);
             c_ptclse(&tie2);
             return(E_FAIL);
             }
          c_ckptid(data1[i].pt_id);
          } 
      temp = i;
      for (i = temp; ; i++)
          {
          if (i+1 == *maxtie)
             {
             *maxtie += MAX_TIE_POINTS;
             data1 = (struct TPSDATA *) realloc(data1,
                                      (*maxtie * sizeof(struct TPSDATA)));
             *data = data1;
             }
          status = c_ptio(&tie2,&(data1[i]));
          if (status == E_EOF) break;
          if (status == E_FAIL)
             {
             c_ptclse(&tie);
             c_ptclse(&tie2);
             return(E_FAIL);
             }
          if (!c_ckptid(data1[i].pt_id))
             {
             if (strncmp(subcmd,"SCREEN",6) == 0)
                { 
                while (!c_ckptid(data1[i].pt_id))
                    renmpt(data1[i].pt_id,line);
                }
             else
                {
                c_errmsg("Error combining tie points","edittie-combine",
                         &nonfatal);
                c_ptclse(&tie);
                c_ptclse(&tie2);
                return(E_FAIL);
                }
             }
          }
         *count = i;
         c_ptclse(&tie);
         c_ptclse(&tie2);
      } 
   else
      {
      if (c_ptopen(&tie,hosin[0],&pptype,&acc,head1) != E_SUCC)
         return(E_FAIL);
      for (i = 0; ; i++)
          {
          if (i+1 == *maxtie)
             {
             *maxtie += MAX_TIE_POINTS;
             data1 = (struct TPSDATA *) realloc(data1,
                                      (*maxtie * sizeof(struct TPSDATA)));
             *data = data1;
             }

          status = c_ptio(&tie,&(data1[i]));
          if (status == E_EOF) break;
          if (status == E_FAIL)
             {
             c_ptclse(&tie);
             return(E_FAIL);
             }
          c_ckptid(data1[i].pt_id);
          }
      *count = i;
      c_ptclse(&tie);
      }
   }

return(E_SUCC);
} /* load_tps */
