/*******************************************************************************

  NAME:  TIEHELP 

  FUNCTION: To display the help for EDITTIE.

  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	EDITTIE must be run under TAE

  PROJECT:  LAS

  ALGORITHM DESCRIPTION:
  Receives the number of lines in the screen and the code for which the help 
   is needed.  
  Based on the code, the help messages are displayed to the terminal.  
  A code of a blank will display general help message.
  The screen is displayed until the return key is hit and then the
   control is returned to the calling routine.

  ALGORITHM REFERENCES: None.

*******************************************************************************/
#include <stdio.h>
#include "las.h"			/* general las includes 	*/
#include "protos.h"

void tiehelp
(
    char *ans,				/* input help code		*/
    long *line				/* number of lines in the terminal */
)

{  /* tiehelp */

long term;				/* terminator of t_input	*/
char retrn[60];				/* buffer for return message	*/
char temp[3];				/* buffer for return input	*/

/*  Initialize variables
------------------------*/
strcpy(retrn,"Press the return key when finished");

/*  Clear the terminal
----------------------*/
t_clear();

/*  Compare the input code with the known codes and output the correct
    help message to the terminal.
----------------------------------------------------------------------*/
if (strcmp(ans,"in") == 0)
   {
   t_output(2,2,
     "Image name.  LAS name of the image in from which points were obtained.");
   t_output(3,2,
     "If the points were obtained from a source other than an image, this");
   t_output(4,2,"field will be blank.");
   }
else if (strcmp(ans,"pc") == 0)
   {
   t_output(2,2,"Projection code.  Refer to the GEOMPOD for further details.");
   t_output(4,2,"Valid projection system codes are:");
   t_output(5,2,"  GEO(0)     = Geographic");
   t_output(6,2,"  UTM(1)     = Universal Transverse Mercator");
   t_output(7,2,"  SPCS(2)    = State Plane Coordinates");
   t_output(8,2,"  ALBERS(3)  = Albers Conical Equal Area");
   t_output(9,2,"  LAMCC(4)   = Lambert Conformal Conic");
   t_output(10,2,"  MERCAT(5)  = Mercator");
   t_output(11,2,"  PS(6)      = Polar Stereographic");
   t_output(12,2,"  POLYC(7)   = Polyconic");
   t_output(13,2,"  EQUIDC(8)  = Equidistant Conic");
   t_output(14,2,"  TM(9)      = Transverse Mercator");
   t_output(15,2,"  STEREO(10) = Stereographic");
   t_output(16,2,"  LAMAZ(11)  = Lambert Azimuthal Equal Area");
   t_output(17,2,"  AZMEQD(12) = Azimuthal Equidistant");
   t_output(18,2,"  GNOMON(13) = Gnomonic");
   t_output(19,2,"  ORTHO(14)  = Orthographic");

   /*  Split the help messages between two separate screens
   ---------------------------------------------------------*/
   t_output(*line,2,retrn);
   t_input(*line,37,temp,&term);
   if (term == P_OVER)
      t_output(*line - 2,2,"[edittie-input] Input is larger than buffer size");
   t_clear();
   t_output(2,2,"  GVNSP(15)  = General Vertical Near-Side Perspective");
   t_output(3,2,"  SNSOID(16) = Sinusoidal");
   t_output(4,2,"  EQRECT(17) = Equirectangular");
   t_output(5,2,"  MILLER(18) = Miller Cylindrical");
   t_output(6,2,"  VGRINT(19) = Van der Grinten");
   t_output(7,2,"  HOM(20)    = Oblique Mercator (Hotine)");
   t_output(8,2,"  ROBIN(21)  = Robinson");
   t_output(9,2,"  SOM(22)    = Space Oblique Mercator (SOM)");
   t_output(10,2,"  ALASKA(23) = Alaska Conformal");
   t_output(11,2,"  GOOD(24)   = Interrupted Goode Homolsine");
   t_output(12,2,"  MOLL(25)   = Mollweide");
   t_output(13,2,"  IMOLL(26)  = Interrupted Mollweide");
   t_output(14,2,"  HAMMER(27) = Hammer");
   t_output(15,2,"  WAGIV(28)  = Wagner IV");
   t_output(16,2,"  WAGVII(29) = Wagner VII");
   t_output(17,2,"  OBEQA(30)  = Oblated Equal Area");
   t_output(18,2,"  31+        = User Defined");
   }
else if (strcmp(ans,"pz") == 0)
   {
   t_output(2,2,
      "Projection zone.  The zone to use for UTM and State Plane projection"); 
   t_output(3,2,
      "systems.  Refer to the GEOMPOD documentation for further details.");
   }
else if (strcmp(ans,"dc") == 0)
   {
   t_output(2,2,
      "Datum code.  The datum can be used for setting the semimajor");
   t_output(3,2,
      "axis and eccentricity squared for a particular projection.  Refer to");
   t_output(4,2,"the GEOMPOD documentation for further details.");
   t_output(5,2,"The supported datums are:");
   t_output(6,2,"  0 = Clarke 1866"); 
   t_output(7,2,"  1 = Clarke 1880");
   t_output(8,2,"  2 = Bessel");
   t_output(9,2,"  3 = International 1967");
   t_output(10,2,"  4 = International 1909");
   t_output(11,2,"  5 = WGS 72");
   t_output(12,2,"  6 = Everest");
   t_output(13,2,"  7 = WGS 66");
   t_output(14,2,"  8 = GRS 1980");
   t_output(15,2,"  9 = Airy");
   t_output(16,2,"  10 = Modified Everest");
   t_output(17,2,"  11 = Modified Airy");
   t_output(18,2,"  12 = Walbeck");
   t_output(19,2,"  13 = Southeast Asia");

   /*  Split the help messages between two separate screens
   ---------------------------------------------------------*/
   t_output(*line,2,retrn);
   t_input(*line,37,temp,&term);
   if (term == P_OVER)
      t_output(*line - 2,2,"[edittie-input] Input is larger than buffer size");
   t_clear();
   t_output(2,2,"  14 = Australian National");
   t_output(3,2,"  15 = Krassovsky");
   t_output(4,2,"  16 = Hough");
   t_output(5,2,"  17 = Mercury 1960");
   t_output(6,2,"  18 = Modified Mercury 1968");
   t_output(7,2,"  19 = Sphere of radius - 6370997 meters");
   }
else if (strcmp(ans,"pv") == 0)
   {
   t_output(2,2,"Projection validity.  The validity of the projection fields.");
   t_output(3,2,"IN  =  INVALID The values is known not to be valid.");
   t_output(4,2,"VAL =  VALID   The values is known to be valid.");
   }
else if (strcmp(ans,"tm") == 0)
   {
   t_output(2,2,
     "Tie point mode.  The type of cordinates in the Tie Point file.");
   t_output(3,2,"Valid values are:");
   t_output(4,2,"  GEO  = Geographic Coordinates");
   t_output(5,2,"  REF  = Reference Image Coordinates");
   t_output(6,2,"  SEA  = Search Image Coordinates");
   t_output(7,2,"  PRO  = Projection Coordinates");
   t_output(8,2,"  USR  = User Defined Coordinates");
   }
else if (strcmp(ans,"pp") == 0)
   {
   t_output(2,2,
      "Projection parameters.  These parameters are required to define"); 
   t_output(3,2,
      "each map projection.  Refer to the GEOMPOD documentation for further");
   t_output(4,2,
      "details.  Each of these parameters may be edited individually by ");
   t_output(5,2,
      "specifying the the code PP, the letter of the row needed, and the");
   t_output(6,2,
      "column number.  Example PPA2 = -34000000");
   t_output(7,2,
      "A row of parameters may be specified by just using the code PP and");
   t_output(8,2,
      "the letter of the row.  Example PPA = 56000000 -34000000 0");
   }
else if (strcmp(ans,"cc") == 0)
   { 
   t_output(2,2,
      "Corner coordinates.  Projection coordinates of the resulting output");
   t_output(3,2,"image's four corners.  Specify each corner separately.");
   }
else if ((strncmp(ans,"ul",2) == 0) || (strncmp(ans,"ccul",4) == 0))
   {
   t_output(2,2,
      "Upper left corner coordinates.  The location of the upper left");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strncmp(ans,"ll",2) == 0) || (strncmp(ans,"ccll",4) == 0))
   {
   t_output(2,2,
      "Lower left corner coordinates.  The location of the lower left");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strncmp(ans,"ur",2) == 0) || (strncmp(ans,"ccur",4) == 0))
   {
   t_output(2,2,
      "Upper right corner coordinates.  The location of the upper right");
   t_output(3,2,
      "corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if ((strncmp(ans,"lr",2) == 0) || (strncmp(ans,"cclr",4) == 0))
   {
   t_output(2,2,"Lower right corner coordinates.  The location of the lower");
   t_output(3,2,
      "right corner.  The coordinates are entered in latitude/longitude or");
   t_output(4,2,"northing/easting order.");
   }
else if (strcmp(ans,"pd") == 0)
   {
   t_output(2,2,
      "Projection distance in line and sample direction.  The distance (in ");
   t_output(3,2,
    "the line and sample direction) on the ground that one line or sample");
   t_output(4,2,"represents.");
   }
else if (strcmp(ans,"pu") == 0)
   {
   t_output(2,2,
      "Projection measurement units.  The measurement system used for the");
   t_output(3,2, "projection dimensions."); 
   t_output(4,2,"The current standard projection units include:");
   t_output(5,2,"radians");
   t_output(6,2,"degrees");
   t_output(7,2,"seconds");
   t_output(8,2,"dms");
   t_output(9,2,"feet");
   t_output(10,2,"meters");
   }
else if (strcmp(ans,"tc") == 0)
   {
   t_output(2,2,
      "Tie point coordinates.  These are either geographic coordinates,");
   t_output(3,2,
      "projection coordinates, or user defined coordinates.  This field");
   t_output(4,2,
      "is valid when tie point mode is set to 'REF', 'PRO', or 'USR'.");
   }
else if (strcmp(ans,"ic") == 0)
   {
   t_output(2,2,
      "Image location of the tie point in the line, sample direction in the");
   t_output(3,2,
      "full image.  This field is used when tie point mode is set to 'REF' or");
   t_output(4,2,
      "'SEA'.");
   }
else if (strcmp(ans,"ep") == 0)
    t_output(2,2,"Elevation of the tie point.");
else if (strcmp(ans,"sf") == 0)
   {
   t_output(2,2,
      "The enlargement factor used when selecting the tie point.");
   t_output(3,2,
      "This gives an indication as to the possible sub-pixel");
   t_output(4,2,
      "accuracy of the tie point.");
   }
else if (strcmp(ans,"cs") == 0)
   {
   t_output(2,2,
      "Desired subimage width and height.");
   }
else if (strcmp(ans,"soc") == 0)
   t_output(2,2,"Strength of correlation.");
else if (strcmp(ans,"so") == 0)
   {
   t_output(2,2,
      "Desired horizontal and vertical search offsets.");
   }
else if (strcmp(ans,"ci") == 0)
   {
   t_output(2,2,
      "The LAS image name of the file containing the subimage chip.  This");
   t_output(3,2,
      "field is blank if a subimage chip file does not exist for this tie");
   t_output(4,2,"point.");
   }
else if (strcmp(ans,"rms") == 0)
   t_output(2,2,"Estimated horizontal and vertical RMS errors");
else if (strcmp(ans,"rm") == 0)
   {
   t_output(2,2,
      "The type of reference coordinates in the Tie Point Location file.");
   t_output(3,2,"Valid values are:");
   t_output(4,2,"IMAGE =  Full Image");
   t_output(5,2,"MAP   =  Area is a map");
   t_output(6,2,"TEMP  =  Temporal registration");
   t_output(7,2,"FULL  =  Exract from full input image");
   t_output(8,2,"PRE   =  Search subimages prescaled or rotated");
   }
else if (strcmp(ans,"cm") == 0)
   {
   t_output(2,2,
      "The type of correlation that was preformed.  Valid values are:"); 
   t_output(3,2,"EDGE   =  Edge correlation");
   t_output(4,2,"GREY   =  Grey correlation");
   t_output(5,2,"PHASE  =  Phase correlation");
   t_output(6,2,
     "MANUAL =  Manual correlation--automatic correlation not performed");
   }
else if (strcmp(ans,"et") == 0)
   t_output(2,2,"The fraction of pixels to be classified as edges.");
else if (strcmp(ans,"mc") == 0)
   t_output(2,2,"The minimum acceptable correlation strength.");
else if (strcmp(ans,"md") == 0)
   {
   t_output(2,2,"The maximum acceptable difference between the nominal tie");
   t_output(3,2,"point location and that given by correlation.");
   }
else if (strcmp(ans,"fm") == 0)
   {
   t_output(2,2,"The correlation peak fitting method.  Valid values are:");
   t_output(3,2,"PARAB   =  Elliptic paraboloid");
   t_output(4,2,"GAUSS   =  Elliptic Gaussian");
   t_output(5,2,
     "RECIP   =  Elliptic paraboloid fit to reciprocal of correlation values");
   t_output(6,2,"INTEGER =  No surface fit");
   t_output(7,2,"NOCORR  =  No correlation");
   }
else if (strcmp(ans,"sn") == 0)
   t_output(2,2,
      "The LAS name of the search image in which points were obtained from.");
else if (strcmp(ans,"rn") == 0)
   t_output(2,2,
     "The LAS name of the reference image in which points were obtained from.");
else if (strcmp(ans,"pi") == 0)
   {
   t_output(2,2,"The tie point identification field describing each point.");
   t_output(3,2,"It must be unique among points in this file.");
   }
else if (strcmp(ans,"gc") == 0)
   t_output(2,2,
      "Geographic, projection, or user defined coordinates of the tie point.");
else if (strcmp(ans,"sc") == 0)
   {
   t_output(2,2,"Refined search image location of the tie pint in the");
   t_output(3,2,"line/sample direction in the full search iamge.");
   }
else if (strcmp(ans,"rc") == 0)
   {
   t_output(2,2,
      "Image location of the tie point in the line/sample direction in the");
   t_output(3,2,
      "reference image or subimage.  This defines the center of the reference");
   t_output(4,2,
      "chip extracted during the correlation process.");
   }
else if (strcmp(ans,"doc") == 0)
   {
   t_output(2,2,
      "Displacement, measured diagonally, from the nominal input image");
   t_output(3,2,
      "location of the tie point to the location obtained by correlation.");
   }
else if (strcmp(ans,"nc") == 0)
   {
   t_output(2,2,"Nominal search image locaiton of the tie point in the");
   t_output(3,2,"line/sample direction in the full search image.");
   }
else if (strcmp(ans,"ss") == 0)
   t_output(2,2,"Actual width and height of the input search subimage.");
else if (strcmp(ans,"rs") == 0)
   t_output(2,2,"Actual width and height of the reference subimage.");
else if (strcmp(ans,"chc") == 0)
   {
   t_output(2,2,"The line/sample coordinates of the tie point relative to");
   t_output(3,2,"the reference chip image."); 
   } 
else if (strcmp(ans,"cl") == 0)
   {
   t_output(2,2,
      "The line/sample coordinate values relative to the full image of the");
   t_output(3,2,"corner of the input subimage.");
   }
else if (strcmp(ans,"coc") == 0)
   {
   t_output(2,2,
      "The line/sample coordinates of the upper left corner of the reference");
   t_output(3,2,"subimage relative to the reference image or reference chip.");
   } 
else if (strcmp(ans,"res") == 0)
   t_output(2,2,"Requested width and height of the reference subimage.");
else if (strcmp(ans,"af") == 0)
   {
   t_output(2,2,"Tie point active flag containing accept/reject codes.");
   t_output(3,2,"Valid values are:");
   t_output(4,2,"ACCEPT   = Accept manually etracted tie point");
   t_output(5,2,"REJECT   = Tie point rejected manually by user");
   t_output(6,2,"REACCEPT = Previously rejected tie point re-accepted");
   } 
else if (strcmp(ans,"rcn") == 0)
   t_output(2,2,
      "The image name of the file containing the subimage reference chip.");
else if (strcmp(ans,"cf") == 0)  
   {
   t_output(2,2,"The type of subimage chip used.  Valid values are:");
   t_output(3,2,"NO   = Chips not used");
   t_output(4,2,"TEMP = Temporal registration");
   t_output(5,2,"FULL = Extract from full input image");
   t_output(6,2,"PRE  = Search subimages prescaled or rotated");
   }
else if (strcmp(ans,"scn") == 0)
   {
   t_output(2,2,"The image name of the file containing the prescaled or");
   t_output(3,2,"rotated subimage search chip."); 
   }
/*  A blank as a code will display the general help statements
--------------------------------------------------------------*/
else if (strncmp(ans," ",1) == 0) 
   {
   t_output(2,2,
     "Each of the tie point header or data fields may be edited by");
   t_output(3,2,
      "specifying the upper case letters of a specific field name, followed");
   t_output(4,2,
      "by an equal sign(=), followed by the new value.  Edit codes may be");
   t_output(5,2,"entered in upper or lower case.");
   t_output(6,2,
      "Example:  To change the projection zone field to '0' enter PZ = 0.");
   t_output(7,2," ");
   t_output(8,2,
      "The (H)elp command is used to retrieve information on any of the");
   t_output(9,2,
      "tie point fields.  Specify the (H)elp command using the following.");
   t_output(10,2,
      "syntax.  Example: To receive help on projection codes enter H PC.");
   t_output(11,2," ");
   t_output(12,2,
      "The (P)age command is used to page through the tie point records.");
   t_output(13,2," ");
   t_output(14,2,
      "The (Q)uit command is used to exit the EDITTIE module.");
   t_output(15,2," ");
   t_output(16,2,
      "The (W)rite command is used to write the changes made to the tie point");
   t_output(17,2,
      "fields.  The tie point file will not be created unless the (W)rite");
   t_output(18,2,
      "command is invoked."); 

   /*  Split the help messages between two separate screens
   ---------------------------------------------------------*/
   t_output(*line,2,retrn);
   t_input(*line,37,temp,&term);
   if (term == P_OVER)
      t_output(*line - 2,2,"[edittie-input] Input is larger than buffer size");
   t_clear();
   t_output(2,2,
      "The (S)witch command is used to switch between displaying the header");
   t_output(3,2,
      "record and the data records.");
   t_output(4,2," ");
   t_output(5,2,
      "The (A)dd command is used to add a tie point.  An empty tie point");
   t_output(6,2,
      "record will then be displayed to the terminal.  Example: To add a tie");
   t_output(7,2,
      "point record tp1, enter A TP1");
   t_output(8,2," ");
   t_output(9,2,
       "The (D)elete command is used to delete a tie point.  To delete a tie");
   t_output(10,2,
       "point tp1, enter D TP1");
   t_output(11,2," ");
   t_output(12,2,
       "Entering a specific tie point identification allows you to display");
   t_output(13,2,
       "that tie point without having to page through any previous records.");
   } 

/*  An invalid code was entered, display the error message
----------------------------------------------------------*/
else
   {
   t_output(2,2,"[edittie-code] Invalid help command code");
   t_output(3,2," ");
   t_output(4,2,"An invalid help command code was entered.  Retype the");
   t_output(5,2,"command withe the correct code.  The upper case letters");
   t_output(6,2,"or a combination of the upper case letters should be used as");
   t_output(7,2,"the edit command code");
   }

/*  Output the return command line.
-----------------------------------*/
t_output(*line,2,retrn);

/*  Wait for the user to hit the return key to continue.
--------------------------------------------------------*/
t_input(*line,37,temp,&term);
if (term == P_OVER)
   t_output(*line - 2,2,"[edittie-input] Input is larger than buffer size");

}  /* tiehelp */
