/*****************************************************************************

  NAME: RENMPT

  FUNCTION:  Rename a point id with user input.

  ALGORITHM DESCRIPTION:
  Output the duplicate point id
  Receive user input and convert to lower case before returning

*****************************************************************************/
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

void renmpt
(
    char *pt_id,				/* point id 			*/
    long *line				/* number of lines on terminal	*/
)
{ /* renmpt */

char blank[71];				/* Blank line			*/
char mess[35];				/* Message buffer 		*/
char mess2[70];				/* Message buffer 		*/
char temp[20];				/* Temporary character storage  */

long length;				/* string length		*/
long term;				/* Terminator of input string	*/

/* Initialize variables
-----------------------*/
strcpy(blank,
"                                                                      ");

sprintf(mess,"Enter a new point identification:");
sprintf(mess2,"Duplicate point identification: %s",pt_id);

t_output(*line - 2,0,blank);
t_output(*line - 1,0,blank);
t_output(*line,0,blank);
t_output(*line - 2,2,mess2);
t_output(*line - 1,2,mess);
t_input(*line,2,temp,&term);

length = (long)strlen(temp);
c_up2low(temp,&length);
strncpy(pt_id,temp,20);

} /* renmpt */

/******************************************************************************

  NAME:	PNTVAL

  FUNCTION: Returns the correct string for the input valid flag

  PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         6/89       D. Etrheim	Initial development

  ALGORITHM DESCRIPTION:
  Returns the correct string for the input valid code.

******************************************************************************/

void pntval
(
    long input,			/*  input valid code			*/
    char str[16]			/*  output valid string			*/
)

{  /* pntval */

/*  copy the correct string based on the valid code
---------------------------------------------------*/
switch (input)
   {
   case GEO_NOTVALID: sprintf (str,"INVALID");
	    break;
   case GEO_VALID: sprintf (str,"VALID");
	    break;
   default: sprintf (str,"UNACCEPTABLE");
   } /* switch (*input) */

}  /* pntval */
/******************************************************************************

  NAME:	START_OF_LINE

  FUNCTION: Parse the command line for an equal (=) sign

  PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         6/89       D. Etrheim	Initial development

  ALGORITHM DESCRIPTION:
  START_OF_LINE parses the edit command string 
  Returns the starting position of the character string which is to the right
  of the equal sign(=)
  A non-fatal error is output if the command string does not contain an equal
  sign

******************************************************************************/

int start_of_line
(
    long *line,				/*  number of lines on the terminal*/
    char **eqptr,				/*  aheadess of the str. right of = */
    char *string				/*  aheadess of command string	*/
)
{  /*  start_of_line  */

/*  Locate the aheadess of the equal sign in the command string
--------------------------------------------------------------*/
*eqptr = strchr(string,'=');

/*  output non-fatal error if equal sign is not found
-----------------------------------------------------*/
if (*eqptr == 0)
   {
   t_output(*line - 2,2,"[edittie-code] Invalid syntax of command code");
   return(E_FAIL);
   }

/* increment aheadess to position just to the right of the equal sign
--------------------------------------------------------------------*/
(*eqptr) ++;		

/*  Skip over the white space between the equal sign and start of string
------------------------------------------------------------------------*/
while (strncmp(*eqptr," ",1) == 0)
   (*eqptr) ++;
 
return(E_SUCC);
}  /* start_of_line */
/******************************************************************************

NAME:	VALIDATE

FUNCTION: Returns the correct string depending on the valid string

  PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         6/89       D. Etrheim	Initial development

ALGORITHM DESCRIPTION:
VALIDATE outputs the correct string 
Updates the vld argument based on the input command string
A non-fatal error is output if the command string is not valid.

******************************************************************************/
void validate
(
    char *eqptr,				/* point to command string	*/
    char *echo,				/* pointer to output string	*/
    long *vld,				/* pointer to valid code	*/
    long *line				/* number of lines on terminal	*/
)

{  /* validate */

/*  Output the correct string and correct valid code based on the 
    input command code
-----------------------------------------------------------------*/
if (strncmp(eqptr,"i",1) == 0)
   {
   *vld = GEO_NOTVALID;
   strcpy(echo,"INVALID");
   }
else if (strncmp(eqptr,"v",1) == 0)
   {
   *vld = GEO_VALID;
   strcpy(echo,"VALID");
   }
else
   {
   /*  If invalid command code was input, output a non-fatal error message
       and output the correct string for the current valid code
   -----------------------------------------------------------------------*/
   if (*vld == GEO_NOTVALID)
      strcpy(echo,"INVALID");
   else 
      strcpy(echo,"VALID");
   t_output(*line-2,2,"[edittie-value] Unacceptable valid value");
   }

}  /* validate */
/*****************************************************************************

  NAME: ACTFLG

  FUNCTION: Find correct line to display.

  PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         6/89       D. Etrheim	Initial development

  PROJECT:  LAS

  ALGORITHM DESCRIPTION:
  Copy correct line to character string

*****************************************************************************/

void actflg
(
    long *active,					/* active flag	*/
    char *line					/* output line	*/
)
{ /* actflg */

switch(*active)
   {
   case 0:  sprintf(line,"Accept manually extracted tie point");
            break;
   case 1:  sprintf(line,"Accept automatically registered tie point");
            break;
   case 2:  sprintf(line,
               "Reject; correlation peak too near edge of search area");
            break;
   case 3:  sprintf(line,
               "Reject; subsidiary peak comparable in strength to main peak");
            break;
   case 4:  sprintf(line,
               "Reject; strength of main peak below minimum specified by user");
            break;
   case 5:  sprintf(line,
               "Reject; displacement exceeds maximum specified by user");
            break;
   case 6:  sprintf(line,"Reject; correlation not attempted");
            break;
   case 7:  sprintf(line,"Tie point rejected manually by user");
            break;
   case 8:  sprintf(line,
               "Tie point rejected automatically during modeling process");
            break;
   case 9:  sprintf(line,
               "Tie point rejected manually during the mmodeling process");
            break;
   default: sprintf(line,"Previously rejected tie point made active");
            break;
     }

} /* actflg */
/*****************************************************************************

  NAME:	CLRTPS

  FUNCTION: Clear a TPS header record.

  PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         6/89       D. Etrheim	Initial development

  ALGORITHM DESCRIPTION:
  Zero out or set to NULL all fields

******************************************************************************/
void clrtps
(
    struct TPSHEAD *head			/* TPS header record */
)

{ /* clrtps */

long i;					/* loop counter */

for (i = 0; i < 8; i++)
   head->corners[i] = 0.0;
head->pdist[0] = head->pdist[1] = 0.0;
for (i = 0; i < 15; i++)
   head->projprms[i] = 0.0;
head->proj_valid = GEO_NOTVALID;
head->code = head->zone = head->datum = 0;
head->mode[0] = head->units[0] = head->name[0] = (char) NULL;

} /* clrtps */

/*****************************************************************************

  NAME: CLRTPL

  FUNCTION: Clear a TPL header record.

  PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         6/89       D. Etrheim	Initial development

  ALGORITHM DESCRIPTION:
  Zero out or set to NULL all fields

******************************************************************************/
void clrtpl
(
    struct TPLHEAD *head			/* TPL header record */
)

{ /* clrtpl */

long i;					/* loop counter */

for (i = 0; i < 8; i++)
   head->corners[i] = 0.0;
head->pdist[0] = head->pdist[1] = 0.0;
for (i = 0; i < 15; i++)
   head->projprms[i] = 0.0;
head->proj_valid = GEO_NOTVALID;
head->code = head->zone = head->datum = 0;
head->edge_thres = head->min_str = head->max_dis = 0.0;
head->ref_mode[0] = head->units[0] = head->ref_name[0] = (char)NULL;
head->peak_method[0] = head->corr_type[0] = head->sea_name[0] = (char)NULL;

} /* clrtpl */

/*****************************************************************************

  NAME: CLRMTP

  FUNCTION: Clear a MTP header record.

  PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         6/89       D. Etrheim	Initial development

  ALGORITHM DESCRIPTION:
  Zero out or set to NULL all fields

******************************************************************************/
void clrmtp
(
    struct MTPHEAD *head			/* MTP header record */
)

{ /* clrmtp */

long i;					/* loop counter */

for (i = 0; i < 8; i++)
   head->corners[i] = 0.0;
head->pdist[0] = head->pdist[1] = 0.0;
for (i = 0; i < 15; i++)
   head->projprms[i] = 0.0;
head->proj_valid = GEO_NOTVALID;
head->code = head->zone = head->datum = 0;
head->chip_mode[0] = head->units[0] = (char)NULL;
head->ref_name[0] = head->sea_name[0] = (char)NULL;

} /* clrmtp */
