
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "facmsg.msg"
#include "protos.h"

struct PARBLK block;                   /* TAE parameter block     */
struct DDR ddr;

FILE    *fd;                           /* output file descriptor  */

double  *cl_sumx;                      /* dynamic storage for     */
double  *cl_sumsq;                     /* sums and sums of squares*/

float   *cl_mean;                      /* and means               */

long    num_files;                     /* no. of files opened     */
struct FDESC *fdesc[MAXIMG];           /* file descriptors        */
struct GDESC *gdesc;                   /* group descriptor        */
long    ipoint[MAXIMG * MAXBND + 1],   /* offsets into image i/o  */
                opoint[MAXBND+1];      /* for input and output bands */
long    min_max;                       /* minmax option flag      */
long    msgtim;                        /* message interval        */
long    print[3];

float   *iobuf;                        /* image i/o buffer        */
char    out[MAXIMG][CMLEN];            /* names of file opened    */

extern long    curcat;                 /* message utility         */
extern long    curent;                 /* parameters for processing */

static  long    zero = 0;	       /* constant of zero	  */
static  long    one = 1;	       /* constant of one	  */

#define TERM 0
#define LP   1
#define FIL 2
#define max_trans  14
#define max_images  14

int main
(void)
{
/*-----------------------------------------------------------------------
NAME: FACTOR

PROGRAM HISTORY:
      Version          date         Author
      -------          ----         ------
         3          15-apr-88      B. Allam 
         2          06-sep-84      J. Seamone 
         1          12-dec-83      S. Ross  

FUNCTION:    FACTOR performs a linear transformation on a multi-band
             image. If requested the function will weight each band 
             separately and modify the output by a user-given scale 
             and offset.

FACTOR Routines:
             GETPARM, INITIO, TFORM, UPLABEL
        
ALGORITHM:

    Call GETPARM to retrieve the Weight and Scale vectors.
    Call INITIO to initialize all input and one output files
    Call TFORM to perform linear transformation.
    Call C_HSTOUT to update the output file history.
    Call UPLABEL to update all output image labels.
    Call C_COMPLT to print out a completion message.

-----------------------------------------------------------------------
*/

  float   fact[max_trans][max_images],      /*band weight vector */
          scale[max_trans][2];              /*scales and offsets */
  double  minval[MAXBND+1], /* array of min values  */
          maxval[MAXBND+1], /* array of max values  */
          y_inc,                            /* line increment for sampling */
          x_inc;                            /* sample increment for sampling */

static long bands[MAXIMG][MAXBND + 1];      /*band no's in each input image */
  long    num_images,                       /*no. of input images    */
          wind[MAXIMG][4],                  /*input image windows    */
          nbands[MAXIMG],                   /*no. of bands in each input image*/
          totbnd,                           /*total no. of input image bands */
          nrep;                             /*no. of transformations */

  char    ihos[MAXIMG][CMLEN],             /*input images host names */
          olas[1][CMLEN],                       /*output image LAS names */
          ohos[1][CMLEN];                    /*output image host names */
        

/*
get parameters
--------------
*/
   getparm(ihos, olas, ohos, &num_images, nbands, bands, &totbnd, wind,
           fact, scale, &nrep);

/*
 perform transformations
 -----------------------
*/
   num_files = 0;
   initio (ihos, ohos[0], &nrep, num_images, nbands, bands, wind);
   tform (totbnd, &nrep, fact, scale, wind, minval, maxval);

/*
 update output file histories 
 ----------------------------
*/
   c_hstout(&block, ohos[0], ihos, &num_images, " ");

/*
update DDR
----------
*/
y_inc = 1;
x_inc = 1;
c_upddr(&block, ihos, ohos[0], &num_images, wind, bands, nbands, &y_inc, &x_inc,
        &nrep, &zero);

/*
 update output image ddr min max fields
 --------------------------------------
*/
  if (min_max)  
    c_svmnmx(ohos[0], &nrep, minval, maxval, "YES");

/*
 completion messages
 -------------------
*/
   c_complt("FACTOR", olas, &one);

return EXIT_SUCCESS;

}
