#include "factor.h"
#include "util.h"


void getparm
(
    char ihos[MAXIMG][CMLEN],              /*input images host names */
    char olas[][CMLEN],                       /*output image LAS names */
    char ohos[][CMLEN],                     /*output image host names */
    long *num_images,                      /*no. of input images    */
    long *nbands,                          /*no. of bands in each input image*/
    long (*bands)[MAXBND + 1],             /*band no's in each input image */
    long *totbnd,                          /*total no. of input image bands */
    long (*wind)[4],                       /*input image windows    */
    float factor[max_trans][max_images],    /* band weight vector */
    float scale[max_trans][2],              /* scales and offsets */
    long *nrep                            /*no. of transformations */
)

{
/*
----------------------------------------------------------------------------
NAME : GETPARM

FUNCTION: GETPARM is called by FACTOR to retrieve user-given inputs
          from LAS.

PROGRAM HISTORY:
  Version          Date            Author
  -------          ----            ------
     3           15-apr-88       b.allam/sar  
     2           22-aug-84       j. seamone/sar   
     1           12-dec-83       s.ross/sasc  



ALGORITHM:

  Initialize program
  Get and validate input and output parameters  
  Calculate factor for each transformation by dividing the weight
                        vector by DENOMVAL (the band divisor array)
  Output the number of transformations to be done.

  
----------------------------------------------------------------------------
*/

float	divide[MAXIMG];                    /* band divide vector */
long    same_window,                       /*flag indicating if images have */
                                           /*same window                    */
        flag[MAXIMG][2],                   /*flag for subwindows            */
        std_out,                           /*logical unit for standard output*/
        stat,                              /*return status */  
        deftrans = 0,                      /*flag for one transaction */
        imglimit = MAXIMG;                 /*maximum number of images */

char    ilas[MAXIMG][CMLEN],                /*input images LAS names */
        minmax[4],                          /*minmax option          */
        buf[ERRLEN];

struct VARIABLE	*vw, *vs, *vd,   /*pointers to VARIABLE structure*/
       *var;
long    i, j, k;                            /*counter variables      */

long    zero = 0;

static char *name[] = {"WEIGHT1", "WEIGHT2", "WEIGHT3", "WEIGHT4", "WEIGHT5", 
                       "WEIGHT6", "WEIGHT7", "WEIGHT8", "WEIGHT9", "WEIGHT10", 
                       "WEIGHT11", "WEIGHT12", "WEIGHT13", "WEIGHT14", 
                       "SCALE1", "SCALE2", "SCALE3", "SCALE4", "SCALE5", 
                       "SCALE6", "SCALE7", "SCALE8", "SCALE9", "SCALE10", 
                       "SCALE11", "SCALE12", "SCALE13", "SCALE14" };

/*
initialize program
-----------------
*/
  c_inlas(&block, &std_out);

/*
get and validate input image parameters
---------------------------------------
*/
  stat = c_getin(&block, "IN", &zero, ilas, ihos, num_images, wind,
                 bands, nbands, totbnd, &imglimit,
                 &same_window, flag);

  if (stat != E_SUCC)
    iso_err(factor_badinp, "factor_badinp");

  if (!same_window)
    iso_err(factor_fstwind, "factor_fstwind");

  if (*totbnd > 14)
   {
    sprintf(buf,"!Input Image may not contain more than 14 bands .Number given is = %3d\n",*nbands);
    iso_err(buf, "factor_nbnd");
   }   


/* get and validate output image parameters  */
/* ----------------------------------------  */
   stat = c_getout(&block, "OUT", olas, ohos);
   if (stat != E_SUCC)
     iso_err(factor_badoutp, "factor-badoutp");

/*
 get divide vector attributes and possibly their values
 ------------------------------------------------------
*/
vd = p_find(&block, "DENOMVAL");

/*
 check the divide values given
 -----------------------------
*/
if ((*vd).v_count) /* if not default divide */
{
   for (i = 0; i < (*vd).v_count; i++)
      divide[i] = RVAL(*vd, i);
   if ((*vd).v_count < *totbnd)
      {
      iso_err (factor_lowdiv, "factor_lowdiv"); 
      for (i = (*vd).v_count; i < *totbnd; i++)
          divide[i] = 1.0;
      }  
   for (i = 0; i < *totbnd; i++)
      if (divide[i] == 0.0) 
         iso_err (factor_zdiv, "factor_zdiv"); 
 }  

/*
 get band weight and factor parameters
 -------------------------------------
*/
*nrep = j = 0;
while ((j < 14) && (!deftrans))
  {
   vw = p_find(&block, name[j]);
   vs = p_find(&block, name[j+14]);
   if (!(*vw).v_count) /* if default weight */
     {
      for (k = 0; k < *totbnd; k++)
         factor[j][k] = 1.0/(float)*totbnd;
      if (!(*vs).v_count) /* if default scale */
         {
         scale[j][0] = 1.0;
         scale[j][1] = 0;
         *nrep = j;
         deftrans = 1;
         }  
     }
    else
        for (i = 0; i < *totbnd; i++)
          factor[j][i] = RVAL(*vw,i);
    if (!(*vs).v_count) /* if default scale */
         {
         scale[j][0] = 1.0;
         scale[j][1] = 0;
         }
    else /* if not default scale */
        {
         scale[j][0] = RVAL(*vs,0);
         scale[j][1] = RVAL(*vs,1);
         if (scale[j][0] == 0.0) 
           {
            sprintf(buf,"!A value of 0.0 was given for parameter SCALFACT%2d\n", j);
            iso_err (buf, "factor_zsc"); 
           }
         }
    j++;
   }  

/*
 adjust weight vectors to account for divide parameter,
 and find out how many transformations to perform.


*/
if (*nrep == 0)
   {
   if (deftrans)
     {
      *nrep = 1;
     }
   else 
      *nrep= j-1;
   }  
sprintf(buf, "Number of transformations to be done is %2d\n", *nrep);
iso_err (buf, "factor_ntrans");
if ((*vd).v_count)   /* if not default divide */
   {
   for (i=0; i < *nrep; i++)
      {
      for (j=0; j < *totbnd; j++)
         factor[i][j] = factor[i][j] / divide[j];
      }  
   }  
/*
Get LAS globals
---------------
*/

var = p_find(&block, "$msgtim");
msgtim = IVAL(*var,0);

var = p_find(&block, "$minmax");
strcpy(minmax, SVAL(*var,0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

}

