#include "factor.h"

void initio
(
    char hosin[][CMLEN],          /*input image host name descriptors */
    char hosout[],                /*output image host name descriptor */
    long *nrep,                        /*no. of transformations        */
    long nimages,                     /*no. of input images           */
    long nbands[],                    /*no. of bands in each input    */
    long image_band_num[][MAXBND +1], /*band numbers in each          */
    long wind[][4]                   /*input image windows           */
)
{
/*
------------------------------------------------------------------------

NAME: INITIO                         

FUNCTION: INITIO is called by FACTOR to initialize the input file
          action group to set up the next output file.

PROGRAM HISTORY:
    Version             date            Author
    -------             ----            ------
       6.1	      23-May-94	       D.Eichacker  Added opt[].
       4              15-apr-88        b.allam/sar  scn#20-140
       3              28-feb-84        s.ross/sar   scn#20-181
       1              12-dec-83        s.ross/sasc  scn#20-105

ALGORITHM:

    Call C_EOPENS to open input and output images
    Call C_EGROUP to group the image

------------------------------------------------------------------------
*/

   long    stat,                  /* flag for EOPENS procedure */
           access,                /* file specifications       */
           type,                  /* data type                 */
           n, nib,                /* counter variables         */
           bufsiz;                /* image i/o buffer size     */

   long    one = 1;
   long    zero = 0;

static	long	opt[3]={0,0,0};   /* option array for c_eopens */
/*
 set up input file group
 -----------------------
*/

   access = IREAD;
   type = EREAL;
   for (n = 0, nib = 0; n < nimages ; n++)
   {
     stat = c_eopens(&fdesc[n], hosin[n], image_band_num[n], &wind[n][SL],
                     &wind[n][SS], &wind[n][NL], &wind[n][NS], &ipoint[nib],
                     &access, &type, &zero, opt);
     if (stat != E_SUCC)
       iso_err(factor_ifilerr, "factor_ifilerr");
     nib += nbands[n]; 
    }
/*
 set up output file 
 ------------------
*/

   access = IWRITE;
 
   stat = c_eopens(&fdesc[n], hosout, nrep, &one, &one, &wind[0][NL],
                   &wind[0][NS], opoint, &access, &type, &zero, opt);
   if (stat != E_SUCC)
     iso_err(factor_ofilerr, "factor_ofilerr");
   strcpy(out[num_files], hosout); 
   num_files = num_files +1;
   fdesc[++n] = 0;
/*
set up group containing input and output bands
----------------------------------------------
*/
   if ((iobuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
     iso_err("Error allocating dynamic memory", "factor_alloc");

   if (c_egroup(fdesc, &gdesc, iobuf, &bufsiz) != E_SUCC)
     iso_err("Error grouping image bands", "factor_grp");
 }
