#include "factor.h"
#include "util.h"
#include "pixman.h"

void tform
(
    long num_inbands,                   /* total no. of input image bands */
    long *nrep,                          /* no. of transformations       */
    float factor[][max_images],          /* band weight vector           */
    float scale[][2],                    /* scales and offsets           */
    long wind[][4],                     /* image window                  */
    double minval[MAXBND+1], /* array of min values  */
    double maxval[MAXBND+1] /* array of max values  */
)
{

/*
-------------------------------------------------------------------------

NAME: TFORM                       

FUNCTION: TFORM is called by FACTOR to perform the linear 
          transformation of the multi-band input image. 

PROGRAM HISTORY:
   Version             Date           Author
   -------             ----           ------  
      3              25-Apr-88      B. Allam/SAR
      2              22-AUG-84      J. Seamone/SAR 
      1              12-SEP-83      S. Ross 

ALGORITHM:

   Compute the value of output pixels for each input image.

--------------------------------------------------------------
*/
float   outpix,        /* temp output pixel value */
        fact;          /* temp hold value         */
long    jk, img, pix,  /* counter variables       */
        msg_flag,      /* format flag             */
        repcnt,        /* no. of transformation   */
        dtype,         /* data type               */
        npix,          /* no. of pixels           */
        nlin;          /* no. of lines            */

/*
initialize and enable processing message utility
------------------------------------------------
*/

   curent = 1;
   msg_flag = ALLCAT;
   c_inittm("FACTOR", "Processed", "line", "bands", " ",
             &wind[0][NL], &num_inbands, &msgtim, &msg_flag);
   c_startm();
   
/*
initialize minimum and maximum arrays if needed
-----------------------------------------------
*/
   if (min_max)
   {
   dtype = EREAL;
   c_inmnmx(&dtype, &num_inbands, minval, maxval);
   }
/*
process a line at a time
------------------------
*/
npix = wind[0][NS];
nlin = wind[0][NL];   
for (jk = 1; jk <= nlin; jk++, curent++)
   {
      c_estep(&gdesc);
      for (repcnt = 0; repcnt < *nrep; repcnt++)
      {
/*
compute output pixel value
--------------------------
*/
       for (pix=0; pix < npix; pix++)
         {
          outpix = 0.0;
          for (img = 0; img < num_inbands; img++)
            {
             fact = iobuf[ipoint[img]+pix] * factor[repcnt][img]
                    * scale[repcnt][0];
             outpix = outpix + fact;
            }
          iobuf[opoint[repcnt]+pix] = outpix+scale[repcnt][1];
         } 
      }
/*
update min and max values for each band
---------------------------------------
*/
      if (min_max)
        for (repcnt=0; repcnt < *nrep; repcnt++)
          c_pxmnmx(&iobuf[opoint[repcnt]], &npix, &minval[repcnt],
                    &maxval[repcnt], &dtype);
    } 

  c_estep(&gdesc);

  c_stoptm();

/*
close opened and grouped image files
------------------------------------
*/
c_egclse(&gdesc);

}
