/******************************************************************************
FUNCTION:	BLDFIL

PURPOSE:	BLDFIL inserts the standard header into a text file that
		is to be written to a line printer.  Form feed characters
		within the text file are recognized.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      05/86    K. Gacke            initial development
  5.0	   06/88    D. Hollaren         - changed cerrmsg to c_errmsg and
					  passed error code by address
					- include las.h instead of tae.h
					- changed declarations to raw C types
  5.2      02/90    B. Davis		Pass value of 1 to z_exit, not address.
					Not all machine addresses are
					interpreted as positive values, up until
					now we were getting lucky.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT			LAS

ALGORITHM
  Initialize LAS
  Get the input parameters.
  Open the input and output files
  Read each record of the input file
	insert the standard header and
	write the record to the output file.
  Close the input and output files
  Terminate the application without outputting a completion message

ALGORITHM REFERENCES	none.

******************************************************************************/
#include <stdio.h>		/* C standard I/O include file       	*/
#include <stdlib.h>
#include "las.h"		/* LAS include			        */
#include "util.h"
#include "protos.h"

#define  MAX_BYTE   200		/* max line length                      */

long width;			/* line printer page width              */
long lp_maxline;		/* max # lines on line pritner page     */
char addr[101];			/* address of facility                  */
char name[101];			/* name of facility running program     */
char infile[CMLEN];		/* input file name              	*/
char outfile[CMLEN];		/* output file name             	*/
char repnum[9];			/* report number                        */
char title1[133];		/* title 1                              */
char title2[133];		/* title 2                              */

int main
(void)
{
struct PARBLK   vblock;		/* TAE parameter block               	*/

FILE *infp,*outfp;		/* input and output file pointers    	*/

long lp_line;			/* line printer line counter         	*/
long lp_page = 0;		/* line printer page counter         	*/
long nonfatal = 1;		/* Nonfatal error code		     	*/
long fatal = -1;		/* Fatal error code		     	*/
long one = 1;			/* value of one                      	*/
long std_out;			/* standard out				*/

char buf[MAX_BYTE];		/* buffer to read data into     	*/
char prog[11];			/* Program exit string			*/
char *ptr;			/* pointer to a character string	*/

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);
c_errmsg("Routing file to user specified destination"," ",&nonfatal);

/*
  Get the input parameters.  No parameters are passed because they have
  been decalared as globals.
*/
getpar(&vblock);

lp_line = lp_maxline;

/*
  Open the input files
*/
infp = fopen(infile,"r");
if (infp == NULL)
    {
    c_errmsg("Error opening input file in print file utility","fileutil-open",
	     &fatal);
    }

/*
  Open the output files
*/
outfp = fopen(outfile,"w");
if (outfp == NULL)
   {
   c_errmsg("Error opening output file in print file utility","fileutil-open",
	    &fatal);
   }

/*
  Read each record of the input file and call prt_lp to insert the standard
  header and write the record to the output file.
*/
for (;;)
   {
   ptr = fgets(buf,MAX_BYTE,infp);
   if (ptr == NULL)
      break; 

   prt_lp(outfp,buf,&lp_line,&lp_page);
   }

/*
  Close the input and output files
*/
fclose(infp);
fflush(outfp);
fclose(outfp);

/*
  Terminate the application without outputting a completion message
*/
z_exit(one,prog);

return EXIT_SUCCESS;
}

/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the input parameters.  No parameters are needed for
		this routine because the variables have all been decalred
		as globals.
******************************************************************************/
void getpar
(
    struct PARBLK *vblock	/* Variable block			     */
)

{
struct VARIABLE *val;		/* pointer to TAE variable           	     */

/*
  Retrieve the installation site name.  (eg. EROS)
*/
val = p_find(vblock,"$fname");
strcpy(name,SVAL(*val,0));

/*
  Retrieve the address of the installation site.  (eg. Sioux Falls, SD)
*/
val = p_find(vblock,"$faddr");
strcpy(addr,SVAL(*val,0));

/*
  Retrieve the input file name to be printed
*/
val = p_find(vblock,"infile");
strcpy(infile,SVAL(*val,0));

/*
  Retrieve the input file name to be printed
*/
val = p_find(vblock,"outfile");
strcpy(outfile,SVAL(*val,0));

/*
  Retrieve the number of lines on the printer page
*/
val = p_find(vblock,"npglins");
lp_maxline = IVAL(*val,0);

/*
  Retrieve the lint printer page width
*/
val = p_find(vblock,"lpwidth");
width = IVAL(*val,0);

/*
  Retrieve the report number parameter
*/
val = p_find(vblock,"repnum");
strcpy(repnum,SVAL(*val,0));

/*
  Retrieve the text string for the standard header titles
*/
val = p_find(vblock,"text");
if ((*val).v_count == 1)
    {
    strcpy(title1,SVAL(*val,0));
    strcpy(title2," ");
    }
else
    {
    strcpy(title1,SVAL(*val,0));
    strcpy(title2,SVAL(*val,1));
    }

return;
}
