/******************************************************************************
FUNCTION:	GENPFIL

PURPOSE:	Generate the name of the output print file

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 5.0       06/88    D. Hollaren		initial development
 5.1       08/89    B. Davis		Pass value of 1 to z_exit, not address.
					Not all machine addresses are
					interpreted as positive values, up until
					now we were getting lucky.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM 
  Initialize LAS
  Get the host file name for the input file
  Set the array of pointers to the host file name
  Write the host file name to the variable block
  Terminate the program without a completion message

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>		/* C standard I/O include file         */
#include <stdlib.h>
#include "las.h"		/* LAS include			       */
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK   vblock;		/* TAE parameter block			*/

struct VARIABLE *val;		/* pointer to TAE variable		*/

char prog[11];			/* Program exit string			*/
char hname[CMLEN];		/* Host file name			*/
char prfile[1][CMLEN];		/* Temporary print file name		*/
char *host[1];			/* Array of pointers to host file name  */
long one = 1;			/* value of one				*/
long status;			/* Status variable			*/
long std_out;			/* standard output device 		*/
long element = 0;		/* INFILE element number to get		*/
long access = IWRITE;		/* Want write access to print file	*/

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);

/*
  Retrieve parameter input file parameter
*/
status = c_getnam(&vblock,"INFILE",&element,hname,"prt",&access);
if (status != E_SUCC)
    {
    val = p_find(&vblock,"PRFILE");
    strcpy(prfile[0],SVAL(*val,0));
    c_clean(&vblock,"fileutil",prfile,&one);
    }

/*
  Set the array of pointers to the host file name
*/
host[0] = hname;

/*
  Output the host file name to the variable block
*/
q_init(&vblock,P_BYTES,P_ABORT);
q_string(&vblock,"OUTFILE",1,host,P_ADD);
q_out(&vblock);

/*
  Terminate the program without a completion message
*/
strcpy(prog," ");
z_exit(one,prog);

return EXIT_SUCCESS;

}
