/******************************************************************************
FUNCTION:	PRT_LP

PURPOSE:	Prints a line to the file that is to be routed to the line
		printer.  It keeps track of the current line it is printing
		to determine when the printer page is filled.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      05/86    K. Gacke            initial development
  5.0	   06/88    D. Hollaren         - changed cerrmsg to c_errmsg and
					  passed error code by address
					- include las.h instead of tae.h
					- changed declarations to raw C types

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

PROJECT			LAS

ALGORITHM
  Search the message buffer for a form feed character and only process the
  buffer up to that point by recursively calling this routine.
	replace form feed with null term 
	print the buffer
	reposition the buffer pointer
	Set line counter to end of page 
  Truncrate the line to be written to the width of the output printer or term
  Write record to the output file.

ALGORITHM REFERENCES	none.

******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <string.h>
#include "las.h"
#include "protos.h"

void prt_lp
(
    FILE *outfp,			/* file pointer to line printer      */
    char *buf,				/* message to be printed             */
    long *lp_line,			/* current line number               */
    long *lp_page			/* current page number               */
)
{

extern long width;			/* line printer page width           */
extern long lp_maxline;			/* max # lines on line pritner page  */
extern char addr[101];			/* address of facility               */
extern char name[101];			/* name of facility running program  */
extern char repnum[9];			/* report number                     */
extern char title1[133];		/* title 1                           */
extern char title2[133];		/* title 2                           */

long	nbyte;				/* number of bytes written           */
long	nonfatal = 1;			/* Nonfatal error code		     */
long	fatal = -1;			/* Fatal error code		     */

char	msg[ERRLEN];			/* message to be outputted           */
char	*ptr;				/* pointer to character string       */

/*
  Search the message buffer for a form feed character and only process the
  buffer up to that point by recursively calling this routine.
*/
if ((int)(ptr = strchr(buf,'\f')) > 0)
   {
   *ptr = '\0';				 /* replace form feed with null term */

   if (strlen(buf) > 0)			 /* print the buffer 		     */
      prt_lp(outfp,buf,lp_line,lp_page); 

   buf = ptr + 1;			 /* reposition the buffer pointer     */
   *lp_line = lp_maxline;		 /* Set line counter to end of page  */
   }

/*
  If the line number if over the maximum lines per page start a new page and
  insert the standard header on the new page.
*/
++(*lp_line);
if (*lp_line > lp_maxline)
   {
   *lp_line = 5;			/* Header takes up 5 lines	*/
   ++(*lp_page);			/* Increment the page counter	*/
   sheadc(outfp,width,name,addr,*lp_page,repnum,title1,title2);
   }

/*
  Truncrate the line to be written to the width of the output printer or term
*/
if (strlen(buf) > width)
   {
   sprintf(msg,"Text line is too long.  Line is truncated to %d characters",
           width);
   c_errmsg(msg,"fileutil-length",&nonfatal);
   *(buf + width + 1) = '\0';
   }

/*
  Write record to the output file.
*/
nbyte = fprintf(outfp,"%s",buf);
if (nbyte == -1)
   c_errmsg("Error writing record to the output file","fileutil-write",&fatal);

return;
}
