/******************************************************************************
FUNCTION:	TERMOUT

PURPOSE:	Outputs a text file to the terminal screen.  It is part of
		the proc FILEUTIL.  It allows the user to issue commands
		to have text outputted line by line, page, scroll, start
		from beginning, or search for specified string.

		NOTE: If the proc is running in batch, terminal output is
		      invalid and output is NOT written to the batch log.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 1.0       06/86    K. Gacke            initial development
 5.0       06/88    D. Hollaren		- changed declarations to raw C types
					- change cerrmsg to c_errmsg and
					  passed error code by address
					- changed to raw C types
 5.2       02/90    B. Davis		Pass value of 1 to z_exit, not address.
					Not all machine addresses are
					interpreted as positive values, up until
					now we were getting lucky.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM 
  Initalize LAS
  Retrieve the file name parameter
  Open the input file
  Set the output buffer address to the input buffer
  Read each record of the input file
	Call pterm to output the line to the terminal screen.
  Close the input file
  Terminate the application without outputting a completion message

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include			     */
#include "util.h"
/* #include "desc.h"			 string descriptor include file    */
#include "protos.h"

#define MAX_BYTE 200

int main
(void)
{
struct PARBLK   vblock;			/* TAE parameter block               */

struct VARIABLE *val;			/* pointer to TAE variable           */

FILE	*infp;			 	/* input file pointer                */

long	one = 1;			/* value of one                      */
long	fatal = -1;			/* Fatal error code		     */
long	std_out;			/* standard out -- fortran LUN       */
long	zero = 0;			/* value of zero                     */

char	buf[MAX_BYTE];			/* buffer to read data into          */
char	prog[11];			/* Program exit string		     */
char	*infile;			/* input file name                   */
char 	*ptr;				/* pointer to character string       */

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);

/*
  Retrieve input file name
*/
val = p_find(&vblock,"infile");
infile = SVAL(*val,0);

/*
  Open the input file
*/
infp = fopen(infile,"r");
if (infp == NULL)
    {
    c_errmsg("Error opening input file in terminal print file utility",
	     "fileutil-open",&fatal);
    }

/*
  Read each record of the input file and call pterm to output the line to
  the terminal screen.
*/
for (;;)
   {
   ptr = fgets(buf,MAX_BYTE,infp);	/* Get a line from the input file */
   if (ptr == NULL)
      {
      c_pterm("EOF",&one);
      break; 
      }

   *(buf + strlen(buf) - 1) = '\0';	/* Replace the new line character */
   c_pterm(ptr,&zero);			/* Write buffer to the terminal   */
   }

/*
  Close the input file
*/
fclose(infp);

/*
  Terminate the application without outputting a completion message
*/
z_exit(one,prog);

return EXIT_SUCCESS;
}
