#include <string.h>
#include <time.h>
#include "film.h"
#include "dltf.h"
#include "util.h"
#include "protos.h"

void annot
(
    long film_type,            /* Film type                 */
    char fname[],              /* Facility name             */
    char hos_in[],             /* Input image host name     */
    char dpfin[],              /* Dltf name                 */
    char dpfentry[]           /* Dltf entry                */
)

{
/*
*****************************************************************************

NAME:	ANNOT

PURPOSE: Produces annotation

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          5-May-82    J. Hwang, SASC
   2          4-Apr-89    K. A. Narayanan    Conversion to 5.0
   3         27-Jun-89    K. A. Narayanan    Corrected history records  
   4         19-Nov-90    K. Kunska          Added comments; passed correct
                                             file descriptor to c_r2dltf;
   5         15-Oct-98    D. Hames           Add code to to comply 
                                             with localtime & y2k



COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

      Get image area, color scale area positions
      While more history is available
	 Get history 
         Update history storage in order to
            save three most recent history records
      If film type is pseudo then
         Get lookup table from input dltf 
         If failed to get lookup table then
            Use default(32 color standard lookup table)
            Set up color scales to standard
            Set default labels along side the color scale.
         Else
            Find 32 different colors in the lookup table read
               from input dltf and place those colors
               in the color scale.
            Set user defined labels along side the color scale.
         Place tags
      Else if film type is black and white 
         Get lookup table from input dltf
         If successful then
            Use user supplied lookup table
        Else
            Use default lookup table
         Set up color scale as grey scale, labelling as 0 to 255
      Else
         Get lookup table from the input dltf
         If successful then
            Use user supplied lookup table 
         Else
            Use default lookup table
         Set up color scale to 32 color standard and set up
            Image square description type to -1.
         Place tags based on parameter input.
      Place annotation lines identifying las as the generation
         facility and giving the generation date in character
         string table
      Place labels in the picture description table.

*****************************************************************************
*/

static char    *prog_name = "FILM";    /* program name            */
long one = 1;  /* Constant of one */

extern char workbuf[]; /* Work buffer */

struct	tm	*loc_time;	/* Structure for time */

struct	ASDLTF dpf_info;	/* DLTF field information */

char	*buf,			/* Buffer for the history record */
	*ctime(),		/* Function to convert time to character str */
	desc[81],		/* User specified description of DLTF entry */
	dpf_type[3], 		/* Data type of the DLTF entry */
	key[16],		/* History record key */
	lookup[3][256];		/* Buffer for data portion of  DLTF entry */

char	*wptr;		/* Pointer to work buffer */

double	offset,			/* Offset of the LUT in the DLTF entry */
	slope;			/* Slope of the LUT in the DLTF entry */

long	access = IREAD,		/* File access code */
	day, month, year,	/* Date elements    */
	default_dpf,		/* Default DPF flag - set to TRUE if DLTF */
				/* file name was not specified            */
	dpf_len,		/* Length of data portion of second DLTF */
                                /* record */
	end_lut[3],		/* Last entry in LUT data */
	fd_dltf,		/* File descriptor for DLTF file */
	fd_hist,		/* File descriptor for history file */
	found,			/* Found flag */
	i, il, index, j, k , l,	/* Loop counters and array indices */
	lbnd,			/* Lookup table band counter */
	len, len1,		/* History record length */
	lut_bands = 0,  /* Number of bands in the lookup table */
	ncount,			/* Counter for \n in the history record */
	secs,			/* Time in seconds */
	start_lut[3],		/* First entry in the LUT data */
	status;			/* Return status */

static char stdlbl[][7] = {
	"BLACK ", "PURP-1", "PURP-2", "PURP-3", 
	"PURP-4", "BLUE-1", "BLUE-2", "BLUE-3", 
	"BLUE-4", "BLUE-5", "GREN-1", "GREN-2", 
	"GREN-3", "GREN-4", "PINK-1", "PINK-2", 
	"RED-1 ", "RED-2 ", "RED-3 ", "RED-4 ", 
	"ORAN-1", "ORAN-2", "YELO-1", "YELO-2", 
	"YELO-3", "BROW-1", "BROW-2", "BROW-3", 
	"BROW-4", "GRAY-1", "GRAY-2", "WHITE "};	/* Standard labels */


/*
Get coordinates of image, color scale, tag area
------------------------------------------------
*/
found = FALSE;
for (i = 0; i < 4; i++)
   {
   aimg[i] = image[i];
   alab[i] = cos001[i];
   }  
if (film_type == FALSE_COL) alab[2] = 32*alab[2];

/*
Get history label (3 lines)
----------------------------
*/
for (i = 0; i < hist1[6]; i++)
   {
   chrbuf[hist1[7]+i-1] = ' ';
   chrbuf[hist2[7]+i-1] = ' ';
   chrbuf[hist3[7]+i-1] = ' ';
   }

/*
Get maximum 3 history records   
-----------------------------
*/
if (c_hsopen (&fd_hist, hos_in, &access) == E_SUCC)
   {
   strcpy (key, " ");
   for (i = 1; i <= 3; i++)
      {
      ncount = 0;
      status = c_hsread (&fd_hist, key, &buf, &len1);
      if (status != E_SUCC && status != E_SMAL) break;
      strncpy ( workbuf, buf, hist1[6]);

      len = 0;
      wptr = workbuf;
      while ((ncount <= 5) && (len < hist1[6]))
         {
         if (*wptr == '\n')
            {
            *wptr = ' ';
            ncount++;
            }
         wptr++;
         len++;
         }
      if (i == 1) strncpy (&chrbuf[hist1[7]-1], workbuf, len);
      if (i == 2) strncpy (&chrbuf[hist2[7]-1], workbuf, len);
      if (i == 3) strncpy (&chrbuf[hist3[7]-1], workbuf, len);
      }
   }
status = c_hsclos (&fd_hist, hos_in);

/*
Get look up table 
-----------------
*/
status = E_SUCC;      
if (strlen (dpfin) == 0) default_dpf = TRUE;
else 
   {
   default_dpf = FALSE;
   if (c_r1dltf (&fd_dltf, dpfin, dpfentry, desc, &dpf_info, 
                 &dpf_len, dpf_type) != E_SUCC)
      c_abort (&block, "Error reading DLTF", "film-dltfread", prog_name,
               clean_files, &one);
   else
      {
      if (dpf_type[0] != 'B' ||
         ((film_type == BW) && (dpf_info.numbnd != 1)) ||
         ((film_type == FALSE_COL) && (dpf_info.numbnd != 1)) ||
         ((film_type == PSEUDO) && (dpf_info.numbnd != 3)))
            c_abort (&block, "Invalid lookup table type", "film-luttype",
                     prog_name, clean_files, &one);
      if ((film_type == BW) || (film_type == FALSE_COL))
          lut_bands = 1;
      if (film_type == PSEUDO)
          lut_bands = 3;
      for (lbnd = 1; lbnd <= lut_bands; lbnd++)
         {
         if (c_r2dltf (&fd_dltf, dpfentry, &lbnd, &start_lut[lbnd-1], 
                       &end_lut[lbnd-1], &dpf_len, lookup[lbnd-1], &slope, 
                       &offset) != E_SUCC)
            c_abort (&block, "Error reading DLTF", "film-dltfread", 
                     prog_name, clean_files, &one);
         }
      }
   }

/*
If film type is pseudo then get lookup table from input dltf.
If failed to get lookup table then use default(32 color standard lookup table).
Set up color scales to standard.
Set default labels along side the color scale.
-------------------------------------------------------------------------------
*/
if (film_type == PSEUDO) 
   {
   if (default_dpf)
      {
      for (i = 0; i < 3; i++)
         {
         for (k = 0; k < 32; k++)
            {
            for (j = 0; j < 8; j++)
               {
               index = (31-k)*8 + j;
/*
-----------------------------------------------------------------------------
        It is not clear what the intention of the following statement was.
        It does not work correctly and messes up the default table. I have
        commented it out for the time being. However it should work
        correctly for 32 classes or less - K. A. Narayanan, April 4, 89
-----------------------------------------------------------------------------
*/
               clut[i][index] = clut[i][32-k];          
               if ((j == 0) && (i == 0)) 
                  {
                  scales[31-k] = (char ) index;  /*  set color scale */
                  }  
               }  
            strcpy (lblchr[k], stdlbl[k]);              
            }
         }  
      alab[2] = 32*alab[2];
      }
/*
If dltf was specified, find 32 different colors in the lookup table read
from input dltf and place those colors in the color scale.
Set user defined labels along side the color scale.
-------------------------------------------------------------------------
*/
   else                
      {
      l = 0;
      for (i = 0; i < 256; i++)
         {
         found = FALSE;
         for (j = 0; j < 3; j++)
            clut[j][i] = lookup[j][i];
         if (i != 0) 
            {
            for (k = 0; k < i; k++)
               if ((clut[0][k] == lookup[0][i]) && 
                  (clut[1][k] == lookup[1][i]) && 
                  (clut[2][k] == lookup[2][i])) found=TRUE;
            }
         if ((!found) || (i == 0))       /*--- (2230) ---*/
            {
            if (l <= 31) scales[l] = (unsigned char )i;
            l++;
            }  
         }  
      if (l < 31) 
         {
         alab[2] = (l+1)*alab[2];
         for (il=l; il < 32; il++)
            strcpy (lblchr[il], "      ");
         }
      else 
         alab[2] = 32*alab[2];
      }  
/*
Place tags.
------------
*/
   chrbuf[colid[7]-1] = tags[0];
   chrbuf[colid[7]] = tags[1];
   chrbuf[colid[7]+1] = tags[2];
   }

/*
If film type is black and white, get lookup table from input dltf.
If successful then use user supplied lookup table.
Otherwise, use default lookup table.
Set up color scale as grey scale, labelling as 0 to 255
-------------------------------------------------------------------
*/
else if (film_type == BW) 
   {
   if (!default_dpf)
      {
      for (i = 0; i < 256; i++) 
          bwlut[i] = lookup[0][i];
      }
   else
      {
      for (i = 0; i < 256; i++) 
         bwlut[i] = (char)i;
      }
   for (i = 0; i < 32; i++)
      {
      if (i == 0) 
         k=0;
      else 
         k = 1;      
      scales[i] = (char) (i*8 - k);
      }  
   }

/*
If film type is false color, get lookup table from the input dltf.
If successful then use user supplied lookup table 
Otherwise, use default lookup table
Set up color scale to 32 color standard.
Set image square description type to -1.
Place tags based on parameter input.
------------------------------------------------------------------
*/
else 
   {
   if (!default_dpf)
      {
      for (i = 0; i < 256; i++) 
         bwlut[i] = lookup[0][i];
      }
   else
      {
      for (i = 0; i < 256; i++) 
         bwlut[i] = (unsigned char) i;
      }
   for (i = 0; i < 32; i++) scales[i] = (char) i;

   image[9] = -1;    /*  square type is delayed */

   chrbuf[colid[7]-1] = tags[0];
   chrbuf[colid[7]] = tags[1];
   chrbuf[colid[7]+1] = tags[2];
   }  

/*
Place annotation lines identifying las as the generation facility and 
giving the generation date in character string table.
-----------------------------------------------------------------------
*/
strncpy (&chrbuf[titled[7]-1],  title, titled[6]);

time (&secs);
loc_time = localtime (&secs);
month = loc_time->tm_mon + 1;
day = loc_time->tm_mday;
year = loc_time->tm_year;
year = year%100;                 /* used to retrieve 2 year digits for
                                            dates after the year 2000 */
 

sprintf (&chrbuf[lstano[7]-1], "%2d/%2d/%02d", month, day, year);
strcpy (&chrbuf[lstano[7]-1+9], fname);

/*
Put 7 characters to each color scales id area in picture description table.
----------------------------------------------------------------------------
*/
strncpy (&chrbuf [labl01[7]-1], lblchr[0], 7);
cos001[4] = scales[0];
strncpy (&chrbuf [labl02[7]-1], lblchr[1], 7);
cos002[4] = scales[1];
strncpy (&chrbuf [labl03[7]-1], lblchr[2], 7);
cos003[4] = scales[2];
strncpy (&chrbuf [labl04[7]-1], lblchr[3], 7);
cos004[4] = scales[3];
strncpy (&chrbuf [labl05[7]-1], lblchr[4], 7);
cos005[4] = scales[4];
strncpy (&chrbuf [labl06[7]-1], lblchr[5], 7);
cos006[4] = scales[5];
strncpy (&chrbuf [labl07[7]-1], lblchr[6], 7);
cos007[4] = scales[6];
strncpy (&chrbuf [labl08[7]-1], lblchr[7], 7);
cos008[4] = scales[7];
strncpy (&chrbuf [labl09[7]-1], lblchr[8], 7);
cos009[4] = scales[8];
strncpy (&chrbuf [labl10[7]-1], lblchr[9], 7);
cos010[4] = scales[9];
strncpy (&chrbuf [labl11[7]-1], lblchr[10], 7);
cos011[4] = scales[10];
strncpy (&chrbuf [labl12[7]-1], lblchr[11], 7);
cos012[4] = scales[11];
strncpy (&chrbuf [labl13[7]-1], lblchr[12], 7);
cos013[4] = scales[12];
strncpy (&chrbuf [labl14[7]-1], lblchr[13], 7);
cos014[4] = scales[13];
strncpy (&chrbuf [labl15[7]-1], lblchr[14], 7);
cos015[4] = scales[14];
strncpy (&chrbuf [labl16[7]-1], lblchr[15], 7);
cos016[4] = scales[15];
strncpy (&chrbuf [labl17[7]-1], lblchr[16], 7);
cos017[4] = scales[16];
strncpy (&chrbuf [labl18[7]-1], lblchr[17], 7);
cos018[4] = scales[17];
strncpy (&chrbuf [labl19[7]-1], lblchr[18], 7);
cos019[4] = scales[18];
strncpy (&chrbuf [labl20[7]-1], lblchr[19], 7);
cos020[4] = scales[19];
strncpy (&chrbuf [labl21[7]-1], lblchr[20], 7);
cos021[4] = scales[20];
strncpy (&chrbuf [labl22[7]-1], lblchr[21], 7);
cos022[4] = scales[21];
strncpy (&chrbuf [labl23[7]-1], lblchr[22], 7);
cos023[4] = scales[22];
strncpy (&chrbuf [labl24[7]-1], lblchr[23], 7);
cos024[4] = scales[23];
strncpy (&chrbuf [labl25[7]-1], lblchr[24], 7);
cos025[4] = scales[24];
strncpy (&chrbuf [labl26[7]-1], lblchr[25], 7);
cos026[4] = scales[25];
strncpy (&chrbuf [labl27[7]-1], lblchr[26], 7);
cos027[4] = scales[26];
strncpy (&chrbuf [labl28[7]-1], lblchr[27], 7);
cos028[4] = scales[27];
strncpy (&chrbuf [labl29[7]-1], lblchr[28], 7);
cos029[4] = scales[28];
strncpy (&chrbuf [labl30[7]-1], lblchr[29], 7);
cos030[4] = scales[29];
strncpy (&chrbuf [labl31[7]-1], lblchr[30], 7);
cos031[4] = scales[30];
strncpy (&chrbuf [labl32[7]-1], lblchr[31], 7);
cos032[4] = scales[31];

}
