#include "film.h"
#include "protos.h"

/******************************************************************************

NAME:	ARANGE

FUNCTION:   Determine the output image size and magnify picture 
            description table accordingly.


PROGRAM HISTORY:
  Version	Date       Author      	               Request
  -------	----	   ------                      -------
   1.0        19-Feb-82  J. Hwang, SASC
   2.0         7-Jan-88  M. Mazzella SAR/GSFC       Converted to C
   3.0        19-Nov-90  K. Kunska                  Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):

        Use IDX function to search the picture description table
           and locate image area description.
        Use IDX function to select ntimes, where ntimes determines
           which reserve size that the output image will be, if
           the truncate option is set, the reserve size will be
           the next smallest size.
        Call MAGTBL to reset the picture description table from
           a 512 x 512 format to a 512*ntimes x 512*ntimes format
        Reset imgsiz to equal the reserve display size.
        Call INTSCT to set picture description table so that
           the image rectangle description will be either the 
           entire input image, or, if truncate is set then the
           image rectangle will describe the area of the input
           image that will fit the embedded image area in the
           reserve display selected.

******************************************************************************/

void arange
(
    long window[],	/* User defined window to be applied to input image */
    long trnc,		/* 1: Truncate: select the largest output image size */
    long imgsiz[]	/* I/O input image size as an input, output image */
)
			/* size as an output.  */

{
long	l,		/* Location of imgsiz in picture description table */
	len,		/* Numer of entries in picture description table */
	ntimes,		/* Number of times to magnify table */
	sizes[10];	/* Possible image sizes	*/

/*
Get image square description idx.
---------------------------------
*/
sizes[0]= 444;
sizes[1]= 436;
sizes[2]= 888;
sizes[3]= 872;
sizes[4]= 1776;
sizes[5]= 1744;
sizes[6]= 3552;
sizes[7]= 3488;
sizes[8]= 7104;
sizes[9]= 6976;
len = tb[6];
l = idx(1,0,tb,len,10,TRUE);
l = (l - 1) * 10;

/*
Magnify picture description table ntimes.
------------------------------------------
*/
ntimes = idx(window[3],window[2],sizes,5,2,FALSE)-trnc;
if (ntimes <= 0) 
   ntimes = 1;
magtbl(ntimes);

/*
Put output image size into imgsiz.
------------------------------------
*/
imgsiz[2] = tb[2];
imgsiz[3] = tb[3];

/*
Get intersection between window and image size in square description -
result will be in  image description.
----------------------------------------------------------------------
*/
intsct(&tb[l],window,TRUE);

}
