#include "film.h"
#include "util.h"
#include "protos.h"

/******************************************************************************

NAME:	BIT

FUNCTION:   Picks a bit value in a given long word

PROGRAM HISTORY:
  Version	Date       Author      	        Request
  -------	----	   ------               -------
   1.0        30-Nov-81  J. Hwang, SASC
   2.0        7-Jan-88   M. Mazzella, SAR/GSFC  Conversion to C
   3.0        3-Jan-89   K. A. Narayanan        Copied from classmap and
                                                changed error message to refer
                                                to film instead of classmap
   4.0       19-Nov-90   K. Kunska              Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):

   If bit is out of valid range, 
      Print error message.
   Pick up byte value which includes desired bit by performing bitwise 
   AND with POSth bit of WORD
   return

******************************************************************************/


long bit
(
    unsigned char	word,		/*  Source word  */
    unsigned char   pos		/*  Position of bit desired  */
)
{

static char    *prog_name = "FILM";    /* program name            */

/*
If bit is out of valid range, print error message.
----------------------------------------------------
*/

if ((pos > 8) || (pos < 1)) 
   {
   c_abort(&block, "Exceeded  bit range(1-32) value", "film-bitrange",
           prog_name, clean_files, &num_outfiles);
   }

/*
Pick up byte value which includes desired bit.
-----------------------------------------------
*/

if ((1 << (pos - 1)) & word)
   return (TRUE);
else
   return(FALSE);
}
