/*****************************************************************************
NAME:	FILM

PURPOSE: Reformat an image for a film product

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          5-May-82    J. Hwang, SASC
   2         14-Jun-88    K. A. Narayanan    
   3          3-Jan-89    K. A. Narayanan    Conversion to 5.0
   4         27-Jun-89    K. A. Narayanan    Corrected GENCHR.C for annotation
                                             on non-VMS systems. modified
                                             ANNOT.C to correctly handle
                                             history records.
   5         29-Aug-89    K. A. Narayanan    Last column of image was not
                                             getting mapped. corrected 	WRIMG.C.
   6         13-Nov-90    K. Kunska          Added comments; corrected for 
                                             compiler's warning that -32 is
                                             always false; replaced c_errmsg 
                                             with c_abort when necessary.
   7         21-Sep-92	  T. Mittan	     Added extern to curent and curcat
					     to eliminate warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Call INIT to initialize arrays, LUTs, and other variables
   Call GETPAR to get parameters
   Call FILSPEC to set up I/O file specifications
   Call ANNOT to place annotation into the picture description table
   Call GENPIC to generate pictures
   Print completion message.

*****************************************************************************
*/


#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct  PARBLK  block;                 /* TAE parameter block     */

FILE    *fd;                           /* output file descriptor  */

struct FDESC *  fdesc[5];              /* file descriptors        */
struct GDESC *  gdescin;               /* group descriptor        */
struct GDESC *  gdescout;              /* group descriptor        */
long    ipoint[3],                     /* offsets into image i/o  */
                opoint[3];             /* for input and output bands */
long    min_max;                       /* minmax option flag      */
long    msgtim;                        /* message interval        */

unsigned char   *inbuf;                /* image i/o buffer        */
unsigned char   *outbuf;               /* image i/o buffer        */
static char    *prog_name = "FILM";    /* program name            */
char    clean_files[2][CMLEN];         /* clean file names */
char    out_files[2][CMLEN];           /* output file names        */

extern long    curent;                 /* parameters for processing */
extern long    curcat;                 /* message utility         */
long    num_outfiles;                  /* number of output files   */

static  long    one = 1;

#define BW 0                           /* black and white color flag */
#define PSEUDO 1                       /* pseudo color flag */
#define FALSE_COL 2                    /* false color flag  */

int main
(void) 
{
static	long	image_band_num[3][MAXBND + 1];
                                 /* band numbers in each input image */

char	dpf_in[CMLEN],           /* Input DLTF                       */
	dpf_entry[13],           /* Entry for the LUT                */
	facility_name[100],      /* Facility address                 */
	hos_out[1][CMLEN],          /* Output image host name           */
	las_out[1][CMLEN];          /* Output image las name            */

double	maxval[MAXBND],		 /* Maximum pixel value in the output image */
	minval[MAXBND];		 /* Minimum pixel value in the input image */

long	band,			 /* Band loop counter */
	film_type,               /* Film type BW, pseudo, or false color */
	imgsiz[1][4],               /* Output image size                */
	num_image_bands[3],      /* Number of bands in each input image  */
	num_inbands,             /* Total number of input image bands*/
	num_inimages,            /* Number of input images           */
	num_outbands,            /* Number of bands in output image  */
	obands[1][MAXBND+1],	 /* Output image bands               */
	truncate,                /* Truncation option                */
	wind[3][4];              /* Input image windows              */

static	char	hos_in[3][CMLEN];/* Input images host names          */
 

/*
Initialize arrays, LUTs, and other variables.
--------------------------------------------
*/
initf ();
num_outfiles = 0;
/*
Get LAS parameters.
-------------------
*/
getpar (hos_in, las_out, hos_out, &num_inimages, num_image_bands, 
        image_band_num, &num_inbands, &num_outbands, wind, &truncate, 
        &film_type, facility_name, dpf_in, dpf_entry);

/*
Open input and output files.
-----------------------------
*/
filspec (hos_in, hos_out[0], las_out[0], &num_inimages, num_image_bands, 
         image_band_num, &num_outbands, wind, truncate, imgsiz[0]);         

/*
Generate annotation.
---------------------
*/
annot (film_type, facility_name, hos_in[0], dpf_in, dpf_entry);

/*
Generate output image.
-----------------------
*/
genpic (hos_in, hos_out[0], &num_inimages, &num_outbands, film_type, imgsiz[0]);

/*
Update output image history and min/max.
-----------------------------------------
*/
if (min_max)
   {
   for (band = 0; band < num_outbands; band++) 
      obands[0][band] = band+1;
   obands[0][num_outbands] = 0;
   c_minmax (hos_out, &one, imgsiz, obands, &num_outbands, minval, maxval);
   c_svmnmx (hos_out[0], &num_outbands, minval, maxval, "YES");
   }

c_hstout (&block, hos_out[0], hos_in, &num_inimages, "");

/*
Print completion message.
--------------------------
*/
c_complt (prog_name, las_out, &one);

return EXIT_SUCCESS;
}
