#include "film.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char hosin[][CMLEN],			/* Input image host names */
    char hosout[],			/* Output image host name */
    char lasout[],			/* Output image las name */
    long *num_inimages,			/* Number of input images */
    long num_image_bands[],		/* Number of bands in each input image*/
    long image_band_num[][MAXBND + 1],	/* Band numbers in each input image */
    long *num_outbands,			/* Number of output image bands */
    long wind[][4],			/* Input image windows */
    long truncate,			/* Truncate/fill option */
    long imgsiz[]			/* Output image size */
)

/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1            7-Dec-82   J. Hwang
   2            3-Jan-89   K. A. Narayanan    Conversion to 5.0
   3           19-Nov-90   K. Kunska          Added comments
   4	       20-Oct-92   S. Nelson	      Changed c_abort to write film in
					      lower case.
   6.1	       23-May-94   D. Eichacker	      Added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open input image files.
   Set up group containing input bands.
   Call ARANGE to get output image size.
   Open output image file.
   Set up group containing output bands.
   Update output image ddr.

**************************************************************************
*/
{
struct	DDR ddr;

long zero = 0;  /* Constant of zero */
long one = 1;   /* Constant of one  */

double	line_inc = 1.0,              /* Line and sample increments for     */
	samp_inc = 1.0;              /* output image ddr                   */

long	access,                      /* File access type                   */
	bufsiz,                      /* Image i/o buffer size              */
	comb_flag = 0,               /* Image combination flag             */
	n, nib,		             /* Loop counter and array index       */
	status,                      /* Return statusus                    */
	type;                        /* File type                          */
static  long	opt[3]={0,0,0};	     /* For c_eopens                       */

/*
Open input image files.
------------------------
*/
access = IREAD;
type = EBYTE;
for (n = 0, nib = 0; n < *num_inimages; n++)
   {
   status = c_eopens (&fdesc[n], hosin[n], image_band_num[n], &wind[0][SL], 
                      &wind[0][SS], &wind[0][NL], &wind[0][NS], &ipoint[nib], 
                      &access, &type, &zero, opt);
   if (status != E_SUCC )
      c_abort (&block, "Error opening input image", "film-openin", "film",
               clean_files, &num_outfiles);

   nib += num_image_bands[n];
   }  
fdesc[++n] = 0;

/*
Set up group containing input bands.
-------------------------------------
*/
if ((inbuf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "film-alloc", "film",
            clean_files, &num_outfiles);
status = c_egroup (fdesc, &gdescin, inbuf, &bufsiz);
if (status != E_SUCC)
   c_abort (&block, "Error grouping input image bands", "film-ingroup",
            "film", clean_files, &num_outfiles);

/*
Call ARANGE to get output image size.
--------------------------------------
*/
c_getddr (hosin[0], &ddr);
imgsiz[0] = 1;
imgsiz[1] = 1;
imgsiz[2] = ddr.nl;
imgsiz[3] = ddr.ns;
arange (wind[0], truncate, imgsiz);

/*
Open output image file.
------------------------
*/
access = IWRITE;
status = c_eopens (fdesc, hosout, num_outbands, &one, &one, &imgsiz[2], 
                   &imgsiz[3], opoint, &access, &type, &zero, opt);
if (status != E_SUCC )
   c_abort (&block, "Error opening output image", "film-openout", "film",
            clean_files, &num_outfiles);

fdesc[1] = 0;
strcpy (out_files[num_outfiles++], lasout);
strcpy (clean_files[num_outfiles++], hosout);

/*
Set up group containing output bands.
--------------------------------------
*/
if ((outbuf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "film-alloc", "film",
            clean_files, &num_outfiles);
status = c_egroup (fdesc, &gdescout, outbuf, &bufsiz);
if (status != E_SUCC)
   c_abort (&block, "Error grouping output image bands", "film-outgroup",
            "film", clean_files, &num_outfiles);

status = c_estep (&gdescout);
if (status != E_SUCC)
   c_abort (&block, "Error writting output image", "film-outwrite",
            "film", clean_files, &num_outfiles);

/*
Update output image ddr.
-------------------------
*/
status = c_upddr (&block, hosin, hosout, num_inimages, wind, image_band_num, 
                  num_image_bands, &line_inc, &samp_inc, num_outbands, 
                  &comb_flag);
if (status != E_SUCC)
   c_abort (&block, "Error updating ddr file", "film-upddr", "film",
            clean_files, &num_outfiles);
}
