#include <stdio.h>
#include "las.h"
#include "externs.h"
#include "protos.h"

/******************************************************************************

NAME:   GENCHR

FUNCTION:   Generate expanded dots of character to given buffer

PROGRAM HISTORY:
  Version	Date       Author      	       Request
  -------	----	   ------              -------
   1.0        4-Feb-82  J. Hwang,   SASC
   2.0        7-Jan-88  M. Mazzella SAR/GSFC  convert to c
   3.0        4-Apr-89  K. Narayanan          Workbuf index was incorrect
   4.0       27-Jun-89  K. Narayanan          Corrected annotation for non-VMS
   5.0       19-Nov-90  K. Kunska             Corrected for compiler warning
                                              that -32 is always false; added
                                              comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):

      If vcnt is out of range then return
      If achar then
         Get scale factor
         Get number of character from table
      Else
         Get scale factor
         Number of character is 1
      End if
      Do following number of character times
         Get ascii character code
         Convert code to index to the dot table
         If illegal char then use question mark
         Get a byte of the character dot table
         Do i=1,8
            Get a bit from this byte
            If bit is on then
               Fill buffer with grey scale as directed in the table
               scale factor times.
            Else
               Use back ground grey from picture description table 
                  to fill buffer scale factor times
            End if
            Increment buffer index
         End do
      End do
      Return

******************************************************************************/
int genchr
(
    long vcnt,		/* Current line count on the picture */
    long tab[],		/* Square description  */
    char chrs[]		/* Character(s) to be expanded */
)

{
unsigned char	bt,	/* Position of bit to be checked */
		code,	/* Character to be written */
		k;	/* Loop counter  */

extern unsigned char workbuf[];  /* Storage for line  */

float	fscale;		/* Floating point scale factor */

long	gsc,		/* Grey scale for tick marks */
	i,		/* Loop counter */
	index,		/* Index into table for character to be written */
	lc,		/* Location of index */
	m,		/* Loop counter */
	nchar,		/* Number of characters */
	pointr,		/* Tic mark indicator */
	status = E_SUCC,/* Error status */
	scale,		/* Integer scale factor */
	x;		/* Temporary variable for byte swapping */


/*
If table description indicates out of range, return.
-----------------------------------------------------
*/
if ((vcnt < tab[0]) || (vcnt > tab[0] + tab[2] - 1))
   return(status);

/*
If achar then get the scale factor and the number of the character from table.
------------------------------------------------------------------------------
*/
index = 1;
tab[8]++;
if (!(tab[2] > tab[3]))
   {
   fscale = (float)tab[3] / (float)(tab[6] * 8);
   fscale += 0.5;
   scale = (long) fscale;
   nchar = tab[6];
   }
/*
If dchar then get the scale factor and set the number of the character to 1.
-----------------------------------------------------------------------------
*/
else
   {
   fscale = (float)tab[2] / (float)(tab[6] * 10);      
   fscale += 0.5;
   scale = (long) fscale;
   index = index + (tab[8] - 1) / (scale * 10);
   nchar=1;
   }

/*
Transfer the current line of character dots to buffer.
-------------------------------------------------------
*/
lc = ((tab[8] - 1) / scale) + 1;      
pointr = 0;

/*
Get ASCII character code.
Convert code to index to the dot table.
If illegal char then use question mark.
-----------------------------------------
*/

for (i = 1; i <= nchar; ++i)
   {
   if (chrs[index + i - 2] == 0)
      code = 0;
   else
      code = chrs[index + i - 2] - 32;
   if ((code > 95))
      code = 31;
#ifdef vms
   x = 8 * ((lc - 1) / 4) + 4 - lc;   /* bytes in a word have to be reversed */
#else
   x = lc - 1;
#endif

/*
Get a byte of the character dot table.
---------------------------------------
*/
   bt = dots[code][x];

/*
For all bytes, get a bit from this byte
If bit is on then fill buffer with grey scale as directed in the table
scale factor times.
If bit is not on then use background grey from picture description table 
to fill buffer scale factor times
Increment buffer index
--------------------------------------------------------------------------
*/
   for ( k = 8; k ; --k)
      {
      gsc = 5;
      if (bit(bt,k) != 1)
         gsc=4; 
      code = (unsigned char) tab[gsc];
      for (m=1; m <= scale; ++m)
         {
         workbuf[tab[1] + pointr + m - 1] = code;
         }
      pointr += scale;
      }
   }
return(status);
}
