#include <stdio.h>
#include "las.h"
#include "protos.h"

/******************************************************************************

NAME:  GENTIK

FUNCTION:  Generate tick marks

PROGRAM HISTORY:
  Version	Date       Author      	        Request
  -------	----	   ------               -------
   1.0        4-Feb-82   J. Hwang, SASC
   2.0        8-Jan-88   M. Mazzella, SAR/GSFC   Convert to c
   3.0        4-Apr-89   K. Narayanan            Index was incorrect
   4.0       19-Nov-90   K. Kunska               Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):

      Get tick line count
      Do until desired pixels filled
         If ticks not are across the top or bottom of the image then
            If hcount is beyond horizontal tick length then
              Set grey scale to background
	    Else
              Set grey scale to tick color
         If ticks are across the top or bottom of the image then
            If vcount is beyond vertical tick length then
              Set grey scale to background
	    Else
              Set grey scale to tick color
         Set proper values into buffer
******************************************************************************/

int gentik
(
    long vcnt,		/* Vertical line count		*/
    long tab[]		/* Array containing tick info	*/
)

{

extern unsigned char	workbuf[]; /* Character work buffer     */

long	count = 0,		/* Place holder for tick mark	*/
    	gsc,			/* Grey scale value		*/
    	hcount,			/* Horizontal tick mark count	*/
    	index,			/* Index into work buffer	*/
    	pointr,			/* Pointer to position on line	*/
    	twidth,			/* Width of tick mark		*/
    	vcount;			/* Vertical line count		*/

if ((vcnt < tab[0]) || (vcnt > tab[0] + tab[2] - 1))
   return(E_SUCC);

/*
Get the tick line count since beginning of current square.
----------------------------------------------------------
*/
vcount = vcnt - tab[0];
/*
Do the following until desired pixels are filled.
-------------------------------------------------
*/
for ( hcount = 1; hcount <= tab[3]; ++hcount)
   {
   gsc = 4;                                /*  assume base grey scale */
   pointr = hcount - 1;                    /*  assume it is atic */
   twidth = 7;
/*
If ticks not are across the top or bottom of the image and
if hcount is beyond horizontal tick length then set grey scale to background.
-----------------------------------------------------------------------------
*/
   if (!(tab[3] > tab[2])) 
      {
      pointr = vcount;
      twidth = 6;
      if (hcount <= tab[7] )
         count = pointr % tab[8];
         if (count < tab[twidth] )
            gsc = 5;
      }
/*  
If ticks are across the top or bottom of the image and
if vcount is beyond vertical tick length then set grey scale to background.
----------------------------------------------------------------------------
*/
   else
      {
      if ( vcount <= tab[6] )
         count = pointr % tab[8];
         if (count < tab[twidth] )
            gsc = 5;
      }

/*
If within the horizontal and vertical tick length then set grey scale to 
tick color.
Set proper values into buffer.
----------------------------------------------------------------------------
*/
   index = tab[1] + hcount - 1;
   workbuf[index] = (unsigned char) tab[gsc];
   }
return(E_SUCC);
}
