#include <string.h>
#include "film.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char hosin[][CMLEN],		/* Input image host name               */
    char lasout[][CMLEN],		/* Output image las name               */
    char hosout[][CMLEN],		/* Output image host name              */
    long *num_inimages,		/* Number of input images              */
    long num_image_bands[],	/* Number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* Band numbers in each input image */
    long *num_inbands,		/* Total number of input image bands   */
    long *num_outbands,		/* Number of output image bands        */
    long wind[][4],		/* Input image windows                 */
    long *truncate,		/* Truncation/fill option              */
    long *film_type,		/* Film type                           */
    char facility_name[],	/* Facility address                    */
    char dpfin[],		/* DLTF                                */
    char dpfentry[]		/* LUT entry in DLTF                   */
)


/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get user supplied parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          5-May-82    J. Hwang
   2          3-Jan-89    K. A. Narayanan    Conversion to 5.0
   3         19-Nov-90    K. Kunska          Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize program 
   Get film type and set output image number of bands
   Get and validate input image parameters
   Get the facility address for output title.
   Get DLTF and DLTF entry.
   Get tags if film type is false color.
   Get truncation parameter.
   Set title to user annotation if defined, otherwise set it to tae name.
   Get labels if film type is pseudo-color.
   Set labels if film type is black and white.
   Get LAS globals.

**********************************************************************
*/
{

long zero = 0;  /* constant of zero */
long fatal_err = -1;  /* Fatal error flag for c_errmsg */
long inf_err = 1;     /* Information flag for c_errmsg */

struct VARIABLE	 *var;		/* Pointer to a variable in TAE block */

char	*ftype,			/* Film type: pseudo-color, false color,  */
				/*            black-and-white             */
	minmax[4];		/* minmax option */

long	access,			/* Access type for getnam             */
	flag[3][2],		/* Flag for subwindows                */
	i,			/* Loop counter                       */
	imglimit = 3,		/* Maximum number of images           */
	len,			/* String length                      */
	same_window,		/* Flag indicating if images have     */
				/* same window                        */
	stat,			/* Return status                      */
	std_out;		/* Logical unit for standard output   */

static char lasin[3][CMLEN];	/* Input image las name               */

for (i = 0; i < 32; i++) 
   strcpy (lblchr[i], "      ");

/*
Initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
Get film type.
---------------
*/
*num_outbands = 3;
ftype = SVAL(*p_find (&block, "COLRFORM"), 0);
if (ftype[0] == 'B') 
   {
   *film_type = BW;
   *num_outbands = 1;
   }
else if (ftype[0] == 'P') 
   *film_type = PSEUDO;
else 
   *film_type = FALSE_COL;

/*
Get and validate input image parameters.
-----------------------------------------
*/
stat = c_getin (&block, "IN", &zero, lasin, hosin, num_inimages, wind,
               image_band_num, num_image_bands, num_inbands, &imglimit,
               &same_window, flag);
if (stat != E_SUCC) 
   c_errmsg ("Error getting input image", "film-in", &fatal_err);

if (*num_inbands > 1 && (*film_type == BW || *film_type == PSEUDO))
   c_errmsg ("Single band image expected for PSEUDO color or B/W option",
             "film-oneband", &fatal_err); 
else if (*film_type == FALSE_COL) 
   if (*num_inbands != 3) 
      c_errmsg ("Three band image expected for FALSE color option",
                "film-threebands", &fatal_err); 
/*
Get the facility address for output title.
--------------------------------------------
*/
strcpy (facility_name, SVAL (*p_find (&block, "$faddr"), 0));
len = (long) strlen (facility_name);
for (i = len; i < lstano[6]; i++) facility_name[i] = ' ';
facility_name[lstano[6]] = '\0';

/*
Get DLTF and DLTF entry.
-------------------------
*/
access = IREAD;
if (c_getnam (&block, "indltf", &zero, dpfin, "dltf", &access) !=  E_SUCC) 
   c_errmsg ("Error getting DLTF", "film-dltf", &fatal_err);
else
   {
   if (strlen (dpfin) == 0)
      c_errmsg ("No DLTF specified - default LUT will be used", "film-deflut", 
                &inf_err); 
   else
      {
      var = p_find (&block, "dltfentr");
      if ((*var).v_count  ==  0) 
         c_errmsg ("No DLTF entry specified", "film-dltfent", &fatal_err); 
      strcpy (dpfentry, SVAL(*var, 0));
      }
   }
   
/*
Get tags if film type is false color.
--------------------------------------
*/
if (*film_type == FALSE_COL) 
   {
   var = p_find (&block, "COLTAGS");
   for (i = 0; i < (*var).v_count; i++)
      tags[i] = SVAL(*var, i)[0];
   }

/*
Get and validate output image parameters.
------------------------------------------
*/
stat = c_getout (&block, "OUT", lasout, hosout);
if (stat != E_SUCC)
   c_errmsg("Error getting output image", "film-out", &fatal_err);

/*
Get truncation parameter.
--------------------------
*/
if (SVAL(*p_find (&block, "trunopt"), 0)[0] == 'T')
    *truncate = 1;
else
    *truncate = 0;

/*
Set title to user annotation if defined, otherwise set it to tae name.
------------------------------------------------------------------------
*/
var = p_find (&block, "text");
if ((*var).v_count != 0) 
   strcpy (title, SVAL(*var, 0));
else 
   strcpy (title, lasin[0]);

for (i = (long) strlen (title); i < titled[6]; i++)
    title[i] = ' ';
title[titled[6]] = '\0';

/*
Get labels if film type is pseudo-color.
------------------------------------------ 
*/
if (*film_type == PSEUDO) 
   {
   var = p_find (&block, "LABEL");
   for (i = 0; i < (*var).v_count; i++)
      strncpy (lblchr[i], SVAL(*var, i),7);
   }

/*
Set labels if film type is black and white.
-------------------------------------------- 
*/
else if (*film_type == BW) 
   {
   strcpy (lblchr[0],  "   0  ");
   strcpy (lblchr[16], "  127 ");
   strcpy (lblchr[31], "  255 ");
   }  

/*
Get LAS globals.
----------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find (&block, "$MINMAX");
strcpy (minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') 
   min_max = TRUE;
else 
   min_max = FALSE;
}
