#include <stdio.h>
#include "las.h"
#include "protos.h"

/******************************************************************************

NAME:  IDX

FUNCTION:   If eq is true then 
               search the passed table(tbl) for a value that matches the
               value of a return the index that points to the  matching 
               value's position in the table.
           If eq is false then 
               compare a (np of an image) and b (nl of an image) to a table
               containing a set of reserve sizes, 
               return the index to the pair of values in the table that are
               equal to a and b, or that are one step larger than a and b.

PROGRAM HISTORY:
  Version	Date       Author      	        Request
  -------	----	   ------               -------
   1.0       19-Feb-82   J. Hwang, SASC
   2.0        8-Jan-88   M. Mazzella SAR/GSFC   Convert to c
   3.0       19-Nov-90   K. Kunska              Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):

      Do i=1, table size(tsize)
         Find requested index in specified table(tbl)
      End-do
      return

******************************************************************************/

long idx
(
    long	a,		/* Number of pixels in an image */
	long    b,		/* Number of lines in an image  */
    long    tbl[],  /* Table of values      */
	long    tsize,  /* Table size           */
    long    jump,	/* Number of values between each one checked	*/
    long    eq      /* Equivalence flag     */

)
{
long	i,		/* Loop counter	  */
	k;		/* Index into tbl */


/*
Find index for the same value or a value that is greater than or 
equal to but is also the smallest value in table given.
------------------------------------------------------------------
*/
for ( i = 0; i < tsize; i++)
   {
   k = jump * (i + 1) - 1;
   if (((a <= tbl[k-1]) && (b <= tbl[k]) && (!(eq))) || 
       ((eq) && (a == tbl[k]))) 
      break;
   }
return(i+1);
}
