#include "film.h"
#include "protos.h"

/******************************************************************************

NAME:  INTSCT

FUNCTION:   Get the intersection between two windows

PROGRAM HISTORY:
  Version	Date       Author      	       Request
  -------	----	   ------              -------
   1.0        19-Feb-82  J. Hwang, SASC
   2.0         8-Jan-88  M. Mazzella SAR/GSFC  Convert to c
   3.0         3-Jan-89  K. Narayanan          Copied from classmap and changed
                                               error message to refer to film
                                               instead of classmap
   4.0        19-Nov-90  K. Kunska             Added comments; fixed error msg

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):

        If size comparison only then
           Take smaller size in both direction
              (vertical,horizontal)
        Else
           Get intersection between a and b and 
           Put the result into a
        If no intersection then 
           Issue an informational message

******************************************************************************/

void intsct
(
    long a[],		/* Input window a, output intersection of a and b */
    long b[],		/* Input window b				  */
    long abssiz		/* True if absolute size comparison desired	  */
)
{
long	k,		/* Loop counter					*/
	m[2];		/* Temporary holding variable			*/

long inf_err = 1;  /* Information flag for c_errmsg */

/*
If size comparison only then take smaller size in both directions
(vertical,horizontal).
------------------------------------------------------------------
*/
if (abssiz) 
   {
   if (a[2] > b[2]) 
      a[2] = b[2];
   if (a[3] > b[3]) 
      a[3] = b[3];
   }
/*
Get intersection between a and b and put the result into a.
-----------------------------------------------------------
*/
else
   {
   if (!((a[0] > b[0] + b[2]) || (b[0] > a[0] + a[2]) || 
    (a[1] > b[1] + b[3]) || (b[1] > a[1] + a[3])))
      {
      for (k = 0; k < 4; ++k)
         {
         if (k <= 1) 
            {
            m[k] = a[k];
            if (a[k] < b[k]) 
               a[k] = b[k];
            }
         else
            {
            if (a[k] + m[k-2] > b[k] + b[k-2]) 
               {
               a[k] = b[k] + b[k-2] - a[k-2];
               }
            else
               {
               a[k] = a[k] + m[k-2] - a[k-2];
               }  /* if */
            }  /* if */
         } /* for */
      return;
      }
/*
If no intersection then issue an informational message
and make a=b.
-------------------------------------------------------
*/
   else
      {
      c_errmsg ("No intersection between two windows", "film-nointsct", 
                 &inf_err);
      for (k = 0; k < 4; ++k)
         a[k] = b[k];
      }  /* else */
   }  /* if */
return;      
}
