#include "film.h"
#include "util.h"
#include "protos.h"

/*
**********************************************************************

NAME:	LINGEN

PURPOSE: Generate a line of picture

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         19-Feb-82    J. Hwang, SASC
   2          3-Jan-89    K. A. Narayanan    Conversion to 5.0
   3         19-Nov-90    K. Kunska          Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

       For the number of rows in the picure description table
          If line number belongs to the vertical range of the 
          Rectangular description 
             If square type is simple square
                 Call GENSQR to generate simple square
             Else if square type is image
                 Call RDIMG to get a line of image
             Else if square type is character 
                 Call GENCHR to get a line of character dot
             Else if square type is tick
                 Call GENTIK to get a line portion  of tick 
             Else if delay generation then 
                 Delay character generation to wrimg 
             Else
                 Issue an error message and abort

**********************************************************************
*/
void lingen
(
    long vcnt,			/* Output image line number      */
    long tbsize			/* Picure description table size */
)
{

long	j, l; 			/* Loop counters and array indexes */

static char    *prog_name = "FILM";    /* program name            */



/*
For the entire image description table check if line count belongs to 
the vertical range of the rectangular description.
------------------------------------------------------------------------
*/
for  (j = 0; j < tbsize; ++j)
   {
   l = 10 * j - 1;
   if ((vcnt < tb[l + 1]) || (vcnt > (tb[l + 1] + tb[3+l] - 1)))
      continue;
   switch (tb[10 + l])
      {
      case 0:
         {
/*
If square type is simple square call GENSQR to generate simple square.
----------------------------------------------------------------------
*/
         gensqr(vcnt,&tb[1 + l]);
         break;
         }
      case 1:
         {
/*
If square type is image call RDIMG to get a line of image.
-----------------------------------------------------------
*/
         rdimg(vcnt,&tb[1+l],0);
         break;
         }
      case 2:
         {
/*
If square type is character call GENCHR to get a line of character dot.
-----------------------------------------------------------------------
*/
         genchr(vcnt, &tb[1+l], &chrbuf[tb[8+l] - 1]);
         break;
         }
      case 3:
         {
/*
If square type is tick then call GENTIK to get a line portion of tick.
-----------------------------------------------------------------------
*/
         gentik(vcnt,&tb[1+l]);
         break;
         }
      case -1:
      case -2:
         {
         break;
         }
      default:
         {
/*
If square type is not valid, issue an error message.
-----------------------------------------------------
*/
         c_abort (&block, "Invalid square type specified in square description",
                  "film-square", prog_name, clean_files, &num_outfiles);
         }
      }
   }
}
