#include <stdio.h>
#include "las.h"
#include "externs.h"
#include "protos.h"

/******************************************************************************

NAME:  magtbl

FUNCTION:   Magnify square descriptions in the table properly.

PROGRAM HISTORY:
  Version	Date       Author      	       Request
  -------	----	   ------              -------
   1.0       19-Feb-82  J. Hwang, SASC
   2.0        8-Jan-88  M. Mazzella SAR/GSFC   Convert to c
   3.0       19-Nov-90  K. Kunska              Added comments

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):

      Set up factor by which to magnify the x,y,xs,ys elements
         of the picture description table(1,2,4,8,16)
      Do i=1, number of rectangle descriptions in the picture
         description table
         For each rectangle adjust x, y descriptors and magnify
            xs, ys descriptors
         If rectangle describes tick area then
            Magnify length of tick not width
         End if
      End do

******************************************************************************/

void magtbl
(
    long ntimes		/* Number of times to magmify the table  */
)

{

long	i, j,		/* Loop counters			  */
	l,		/* Index into table			  */
	timsqr;		/* Number to multiply values in table by */


timsqr = 1 << (ntimes - 1);

/*
Set up factor by which to magnify the x,y,xs,ys elements
of the picture description table(1,2,4,8,16).
-------------------------------------------------------------
*/
for  (i=1; i <= tb[6]; i++)
   {
   l = (i - 1) * 10;

/*
Magnify size of square, and rearrange coordinates.
--------------------------------------------------
*/
   for (j = 0; j < 4; ++j)
      if (j < 2) 
         tb[j+l] = (tb[j+l] - tb[j]) * timsqr + tb[j] - 1;
      else
         tb[j+l] = tb[j+l] * timsqr;

/*
If rectangle describes tick area then magnify length of tick not width.
------------------------------------------------------------------------
*/
   if (tb[l+9] == 3) 
      if (tb[6+l] > tb[7+l])   /*  atic */
         tb[6+l] = tb[6+l] * timsqr;
      else                          /*  dtic */
         tb[7+l] = tb[7+l] * timsqr;
   }
}
