#include "film.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
*****************************************************************************

NAME:	WRIMG

PURPOSE: Writes a line of output image

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          5-May-82    J. Hwang, SASC
   2          3-Jan-89    K. A. Narayanan    Conversion to 5.0
   3         29-Aug-89    K. A. Narayanan    Last column of image was not
                                             getting mapped
   4         19-Nov-90    K. Kunska          Added comments; corrected index
                                             clut

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   For number of output bands
      If film type is pseudo
         Call VCHAR to generate delayed tag
         If current line number is in the range of image 
            Map pixel values using clut
         If current line number is in the range of color scale 
             Map labels using clut
      Else if film type is false color
         Call VCHAR to generate delayed tag
         Call RDIMG to get a line of image
         If current line number is in the range of image
            Map pixel values using bwlut
         If current line number is in the range of color scale 
            Map labels using clut
      Else
         If current line number is in the range of image
            Map pixel values using bwlut
      Write out a line of output image

*****************************************************************************
*/

void wrimg
(
    long vcnt,			/* Current line number          */
    long num_outbands,		/* Number output bands          */
    long film_type,		/* Film type                    */
    long imgsiz[]		/* Output image size            */
)
{

char    *prog_name = "FILM";    /* program name            */

long	dtype = EBYTE,		/* Data type for pixman		*/
	i, 			/* Loop counter                 */
	op,			/* Band offset                  */
	status,			/* Return status                */
	tbindx;			/* Band index                   */

extern unsigned char workbuf[];	/* Work buffer                  */

/*
For all bands:
---------------
*/
for (tbindx = 0; tbindx < num_outbands; tbindx++)
   {
   op = opoint[tbindx];

/*
If film type is pseudo, call VCHAR to generate delayed tag.
-----------------------------------------------------------
*/
   if (film_type == PSEUDO) 
      {
      vchar (vcnt, tbindx);
      for (i = 0; i < imgsiz[3]; i++)
         outbuf[op+i] = workbuf[i];

/*
If current line number is in the range of image, map pixel values using clut.
------------------------------------------------------------------------------
*/
      if ((vcnt >= aimg[0]) && (vcnt <= aimg[2]+aimg[0]-1)) 
         {
         c_pxlkup (&workbuf[aimg[1]], &outbuf[op+aimg[1]], &aimg[3], 
                   &dtype, clut[tbindx], &dtype);
         }  

/*
If current line number is in the range of color scale, map labels using clut.
-----------------------------------------------------------------------------
*/
      if ((vcnt >= alab[0]) && (vcnt <= alab[2]+alab[0]-1)) 
         {
         c_pxlkup (&workbuf[alab[1]], &outbuf[op+alab[1]], &alab[3], 
                   &dtype, clut[tbindx], &dtype);
         }  
      }

/*
If film type is false color, call VCHAR to generate delayed tag.
-----------------------------------------------------------------
*/
   else if (film_type == FALSE_COL) 
      {
      vchar (vcnt, tbindx);
      for (i = 0; i < imgsiz[3]; i++)
         outbuf[op+i] = workbuf[i];

/*
Call RDIMG to get a line of image.
-----------------------------------
*/
      rdimg (vcnt, aimg, tbindx);

/*
If current line number is in the range of image, map pixel values using bwlut.
------------------------------------------------------------------------------
*/
      if ((vcnt >= aimg[0]) && (vcnt <= aimg[2]+aimg[0]-1)) 
         {
         c_pxlkup (&workbuf[aimg[1]], &outbuf[op+aimg[1]], &aimg[3], 
                   &dtype, bwlut, &dtype);
         }  

/*
If current line number is in the range of color, scale map labels using clut.
-----------------------------------------------------------------------------
*/
      if ((vcnt >= alab[0]) && (vcnt <= alab[2]+alab[0]-1)) 
         {
         c_pxlkup (&workbuf[alab[1]], &outbuf[op+alab[1]], &alab[3], 
                  &dtype, clut[tbindx], &dtype);
         }  
      }
   else        

/*
If film type is black and white, and current line number is in the range 
of image, map pixel values using bwlut.
-------------------------------------------------------------------------
*/
      {
      for (i = 0; i < imgsiz[3]; i++)
         outbuf[op+i] = workbuf[i];
      if ((vcnt >= aimg[0]) && (vcnt <= aimg[2]+aimg[0]-1)) 
         {
         c_pxlkup (&workbuf[aimg[1]], &outbuf[op+aimg[1]], &aimg[3], 
                   &dtype, bwlut, &dtype);
         }  
      }  
   }

/*
Write out a line of output image.
----------------------------------
*/
status = c_estep (&gdescout);
if (status != E_SUCC)
   c_abort (&block, "Error writting output image", "film-outwrite",
            prog_name, clean_files, &num_outfiles);
}
