/*
************************************************************************
NAME: app_wrd

PURPOSE: Apply a filter algorithm to a INTEGER*2 buffer of data

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           03/90   T. Baltzer   
    2		04/92	S. Nelson	Added output min/max options, HIGHVAL
					and LOWVAL, for subcommand -HIGH.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

*****************************************************************
	arguments:
	name		description
	----		-----------
	buffer		input buffer (input)
	obuff		output buffer (output)
	histgm		histogram array (input)
	numh		number of elements in 'histgm' (input)
	nsbuf		number of samples in input buffer (input)
	nlav		number of actual valid lines (rows) in input
			buffer (input)
	nsav		number of actual valid samples (columns) in
			input buffer (input)
	nlb		number of lines in the boundary (input)
	nsb		number of samples in the boundary (input)
	fstr		filter structure (input)
	lowval		minimum value for output
	highval		maximum value for output
	out_flag	output min/max flag

************************************************************************
*/
#include <stdio.h>
#include "worgen.h"
#include "filt.h"
#include "protos.h"

int app_wrd
(
    short *buffer,
    short *obuff,
    long *histgm,
    long numh,
    long nsbuf,
    long nlav,
    long nsav,
    long nlb,
    long nsb,
    struct FSTR *fstr,
    double lowval,
    double highval,
    long out_flag[3]
)

{

long	stat;		/* return status */
long	nonfatal;	/* non-fatal error status */


switch (fstr->subcmd[0])
   {
   case 'H': stat = high_wrd(buffer,obuff,histgm,numh,nsbuf,nlav,nsav,
			     nlb,nsb,fstr->nlk,fstr->nsk,fstr->invld,
			     fstr->thresh,fstr->minval,fstr->maxval,
			     fstr->scalf,fstr->offset,fstr->addback,
			     fstr->minpix, lowval, highval, out_flag);
	     break;
   case 'V': stat = vlow_wrd(buffer,obuff,histgm,numh,nsbuf,nlav,nsav,
			     nlb,nsb,fstr->nlk,fstr->nsk,fstr->invld,
			     fstr->thresh,fstr->minval,fstr->maxval,
			     fstr->scalf);
	     break;
   case 'I': stat = ilow_wrd(buffer,obuff,histgm,numh,nsbuf,nlav,nsav,
		   	     nlb,nsb,fstr->nlk,fstr->nsk,
			     fstr->thresh,fstr->minval,fstr->maxval,
			     fstr->scalf);
	     break;
   case 'M': stat = med_wrd(buffer,obuff,histgm,numh,nsbuf,nlav,nsav,
  	                    nlb,nsb,fstr->nlk,fstr->nsk);
	     break;
   case 'D':
   case 'L':
   default : stat = E_FAIL; /* should not get to here, error ! */
	     c_errmsg("Invalid filter type","filter-ftype",&nonfatal);
	     c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
	     break;
   }

return(stat);

}    /*  end of app_wrd()  */
