/*
************************************************************************
NAME: err_sbuf

PURPOSE: error check the input parameters to size_buf()

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           01/90   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

*****************************************************************
	arguments:
name		description
----		-----------
nli		number of lines in the image window (input)
nsi		number of samples in the image window (input)
nlk		number of lines in the kernal (input)
nsk		number of samples in the kernal (input)
nlb		number of lines in the boundary (input)
nsb		number of samples in the boundary (input)
dtype		data type of buffer (input)
maxram		maximum ram in bytes (input)

************************************************************************
*/

#include "worgen.h"
#include "protos.h"

int err_sbuf
(
    long nli,
    long nsi,
    long nlk,
    long nsk,
    long nlb,
    long nsb,
    long dtype,
    long maxram
)

{

long	nonfatal = 1; /* nonfatal error status           */


/* Begin by checking the values of the individual parameters.     */

if ( nli  < 1 || nsi  < 1 )
   {
   c_errmsg("Invalid window specified","filter-window",&nonfatal);
   c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
   return(E_FAIL);
   }
if ( nlb < 0 || nsb < 0 || nlk  < 1 || nsk  < 1 )
   {
   c_errmsg("Invalid kernel or boundary","filter-kerbnd",&nonfatal);
   c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
   return(E_FAIL);
   }
if ( dtype < 0 || dtype > 4 )
   {
   c_errmsg("Invalid data type","filter-dtype",&nonfatal);
   c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
   return(E_FAIL);
   }
if ( maxram < 1 )
   {
   c_errmsg("Invalid maximum RAM specified","filter-maxram",&nonfatal);
   c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
   return(E_FAIL);
   }


return(E_SUCC);

}    /*  end of err_sbuf()   */
