/*
************************************************************************
NAME: med_wrd

PURPOSE: Apply a MEDIAN filter algorithm to a buffer of data (word, I*2
	 data)

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           02/90   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

*****************************************************************
	arguments:
	name		description
	----		-----------
	buffer		input buffer (input)
	obuff		output buffer (output)
	histgm		histogram array (input)
	numh		number of elements in 'histgm' (input)
	nsbuf		number of samples in input buffer (input)
	nlav		number of actual valid lines (rows) in input
			buffer (input)
	nsav		number of actual valid samples (columns) in
			input buffer (input)
	nlb		number of lines (rows) in boundary (input)
	nsb		number of samples (columns) in boundary (input)
	nlk		number of rows in the kernel (input)
	nsk		number of samples in the kernel (input)

************************************************************************
*/
#include "las.h"
#include "pixman.h"
#include "protos.h"

int med_wrd
(
    short *buffer,
    short *obuff,
    long *histgm,
    long numh,
    long nsbuf,
    long nlav,
    long nsav,
    long nlb,
    long nsb,
    long nlk,
    long nsk
)
{

long	nlavo;		/* number of actual valid lines of data in   */
			/* the output buffer			     */
long	nsavo;		/* number of actual valid samples of data in */
			/* the output buffer			     */
long	nsobuf;		/* number of samples in the output buffer    */
short	*oadd;		/* output buffer address */
short	*oradd;		/* address of kernel origon */
short	*ladd;		/* line buffer address */
long	i;     		/* counter */
long	mid;		/* middle pixel */
long	dlong = ELONG;	/* data type	*/
long	dword = EWORD;	/* data type	*/
long	zerofil = 0;	/* variable used to zero fill the histogram  */
long	count;		/* counter */
long	cursam;		/* current sample */
long	stat;		/* return status flag */
long	nonfatal;	/* non-fatal error status */


nlavo = nlav - (2 * nlb);
nsavo = nsav - (2 * nsb);
nsobuf = nsbuf - (2 * nsb);

mid = (nlk * nsk + 1) / 2;

oadd = obuff;
ladd = buffer;

c_pxfill(histgm,&dlong,&numh,&zerofil);

for (i = 0; i < nlk; i++)
   {
   stat = c_pxhist(&nsk,ladd,&dword,histgm,&count);
   if (stat != E_SUCC)
      {
      c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
      return(E_FAIL);
      }
   ladd += nsbuf;
   }

set_medw(histgm,mid,oadd);

oradd = buffer;
cursam = 1;

for (  ;  ;  )
   {
   for (i = 1; i < nlavo; i++)
      {
      movew_d(&oradd,nlk,nsk,nsbuf,histgm);
      oadd += nsobuf;
      set_medw(histgm,mid,oadd);
      }

   if (cursam == nsavo)  break;
   movew_r(&oradd,nsk,nsbuf,histgm);
   cursam++;
   oadd++;
   set_medw(histgm,mid,oadd);

   for (i = 1; i < nlavo; i++)
      {
      movew_u(&oradd,nlk,nsk,nsbuf,histgm);
      oadd -= nsobuf;
      set_medw(histgm,mid,oadd);
      }

   if (cursam == nsavo)  break;
   movew_r(&oradd,nsk,nsbuf,histgm);
   cursam++;
   oadd++;
   set_medw(histgm,mid,oadd);

   }   /* end of infinite loop */

return(E_SUCC);

}    /*  end of med_wrd()  */
