/*
*************************************************************************
NAME: prelo 

PURPOSE: prepares the initial kernel for low filters.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           11/84   J. LaVergne  
    2            1/87	K. Zanter     conversion to unix	 
    3            3/89   K. Narayanan  conversion to c and 5.0 standards
    3.1          4/89   D. Hollaren   Added static to compile on PN9050
    3.2		 2/91   T. Baltzer    reduce unneccessarily large arrays

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Must be run under TAE.

PROJECT: LAS

*****************************************************************

	arguments:
	name		description
	----		-----------
	curbnd		current input band number
	curimg		current input image number
	hosout		out image host name
	kersum		data sum of the kernel
	ns		number of samples per line to be processed
	subcmd		specified subcommand
	xhalf		half of the number of samples in the kernel
	yhalf		half of the number of lines in the kernel

	internal variables:
	name		description
	----		-----------
	buffer		input buffer
	status		function return status code
************************************************************************
*/
#include "filter.h"
#include "filt.h"
#include "util.h"
#include "protos.h"



int prelo
(
    char hosout[][CMLEN],
    long xhalf,
    long yhalf,
    long *ns,
    long *curimg,
    long *curbnd,
    float *kersum
)

{

long    one = 1;

float	*buffer;

long	j;
long	k;
long	l;
long	nsk;
long	status;

float	*sum_buf;
extern  float	*sum;        /* pointer to the middle of sum array  */

nsk = (xhalf * 2) + 1;

buffer = (float *) malloc(*ns * sizeof(float));
if (buffer  ==  NULL) 
   {
   clos_img(&outfd);
   c_errmsg("Error allocating dynamic memory","filter-alloc",&one);
   c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
            prog_name, hosout, &one);
   }
sum_buf = (float *) malloc((*ns  + nsk) * sizeof(float));
if (sum_buf  ==  NULL) 
   {
   clos_img(&outfd);
   c_errmsg("Error allocating dynamic memory","filter-alloc",&one);
   c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
            prog_name, hosout, &one);
   }

sum = &sum_buf[(nsk - 1) / 2];
/*
	initialization 
	--------------
*/
*kersum = 0.0;
for (j = -xhalf; j <= *ns + xhalf; j++)
   {
   sum[j] = 0.0;
   }

/*
	sum up the column values for the initial kernel (including
        mirroring)
	----------------------------------------------------------------
*/
for (j = 1; j <= yhalf + 1; j++)
   {
   status = c_eread (&infd[*curimg], curbnd, &j, buffer, &one);
   if (status  !=  *ns) 
      {
      clos_img(&outfd);
      c_abort (&parblk, "Fatal error encountered", "filter-fatal", 
               prog_name, hosout, &one);
      }

   for (k = 1 - xhalf; k <= *ns + xhalf; k++)
      {
      set (k, &l, *ns);
      sum[k] += buffer[l];
      if (j  !=  1) 
         {
         sum[k] += buffer[l];
         } 
      }
   }

/*
	calculate the sum of the kernel
	--------------------------------------------------------------
*/
for (k = 1 - xhalf; k <= xhalf + 1; k++)
   {
   *kersum += sum[k];
   }

/*
   free dynamic memory
*/
if ( free_buf(buffer) == E_FAIL )  return(E_FAIL);

return(E_SUCC);
}
