/*
************************************************************************
NAME: set_i_byt

PURPOSE: Using the -ILOW filter, calculate the value of a pixel within
	 the output buffer (take into account the threshold parameter,
	 BYTE data).

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           03/90   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

*****************************************************************
	arguments:
	name		description
	----		-----------
	minval		minimum valid value (input)
	maxval		maximum valid value (input)
	thresh		threshold value (input)
	histgm		histogram array (input)
	scalf		scale factor (input)
	iadd		input buffer address (input)
	oadd		output buffer address (output)

************************************************************************
*/
#include "las.h"
#include "typlim.h"
#include "protos.h"

int set_i_byt
(
    double minval,
    double maxval,
    double thresh,
    long *histgm,
    double scalf,
    unsigned char *iadd,
    unsigned char *oadd
)

{

float	outflt;		/* floating point output */
long	kersum;		/* kernel summation */
long	nkers;		/* number of pixels in sum 'kersum' */
long	i;		/* counter */


if (((float)(*iadd) <= maxval) && ((float)(*iadd) >= minval))
   {
   *oadd = *iadd;	 /* valid pixel */
   }
else			 /* invalid pixel */
   {
   if ( (float)(*iadd) - thresh > minval )  /* find min of maximums */
      minval = (float)(*iadd) - thresh;

   if ( (float)(*iadd) + thresh < maxval )  /* find max of minimums */
      maxval = (float)(*iadd) + thresh;

   kersum = 0;				/* sum the appropriate range */
   nkers = 0;				/*  of the histogram 	     */
   for ( i = (long) minval; i <= maxval; i++)
      {
      kersum += (i * histgm[i]);
      nkers += histgm[i];
      }

   if (nkers == 0)
      *oadd = 0;
   else
      {
      outflt = (float) kersum / (float) nkers * scalf;
      if (outflt < MINUTINY)  outflt = MINUTINY;
      else if (outflt > MAXUTINY)  outflt = MAXUTINY;
      *oadd = (unsigned char) outflt;
      }
   }

return(E_SUCC);

}    /*  end of set_i_byt()  */
