/*
************************************************************************
NAME: set_v_wrd

PURPOSE: Using the -VLOW filter, calculate the value of a pixel within
	 the output buffer (take into account the threshold parameter,
	 INTEGER*2 data).

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           03/90   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

*****************************************************************
	arguments:
	name		description
	----		-----------
	minval		minimum valid value (input)
	maxval		maximum valid value (input)
	thresh		threshold value (input)
	histgm		histogram array (input)
	scalf		scale factor (input)
	invld		invalid pixel option (input)
	iadd		input buffer address (input)
	oadd		output buffer address (output)

************************************************************************
*/
#include "las.h"
#include "typlim.h"
#include "protos.h"

int set_v_wrd
(
    double minval,
    double maxval,
    double thresh,
    long *histgm,
    double scalf,
    long invld,
    short *iadd,
    short *oadd
)

{

float	outflt;		/* floating point output */
long	kersum;		/* kernel summation */
long	nkers;		/* number of pixels in sum 'kersum' */
long	i;		/* counter */


if ((float)(*iadd) < minval)		/* less than minimum value */
   {
   switch (invld)
      {
      case 1:   outflt = minval - 1.0;
		if (outflt < MINWORD)  outflt = MINWORD;
		else if (outflt > MAXWORD)  outflt = MAXWORD;
		*oadd = (short) outflt;
		break;
      case 2:	*oadd = 0;
		break;
      case 3:	*oadd = *iadd;
		break;
      }
   }
else if ((float)(*iadd) > maxval)	/* more than maximum value */
   {
   switch (invld)
      {
      case 1:   outflt = maxval + 1.0;
		if (outflt < MINWORD)  outflt = MINWORD;
		else if (outflt > MAXWORD)  outflt = MAXWORD;
		*oadd = (short) outflt;
		break;
      case 2:	*oadd = MAXUTINY;
		break;
      case 3:	*oadd = *iadd;
		break;
      }
   }
else					/* within bounds */
   {
   if ( (float)(*iadd) - thresh > minval )  /* find min of maximums */
      minval = (float)(*iadd) - thresh;

   if ( (float)(*iadd) + thresh < maxval )  /* find max of minimums */
      maxval = (float)(*iadd) + thresh;

   kersum = 0;				/* sum the appropriate range */
   nkers = 0;				/*  of the histogram 	     */
   for ( i = (long) minval; i <= maxval; i++)
      {
      kersum += (i * histgm[i]);
      nkers += histgm[i];
      }

   outflt = (float) kersum / (float) nkers * scalf;
   if (outflt < MINWORD)  outflt = MINWORD;
   else if (outflt > MAXWORD)  outflt = MAXWORD;
   *oadd = (short) outflt;
   }

return(E_SUCC);

}    /*  end of set_v_wrd()  */
