/*
************************************************************************
NAME: vlow_byt

PURPOSE: Apply a -VLOW (threshold) filter algorithm to a buffer of data
	 (byte data)

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           02/90   T. Baltzer   


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: LAS

ALGORITHM DESCRIPTION:

*****************************************************************
	arguments:
	name		description
	----		-----------
	buffer		input buffer (input)
	obuff		output buffer (output)
	histgm		histogram array (input)
	numh		number of elements in 'histgm' (input)
	nsbuf		number of samples in input buffer (input)
	nlav		number of actual valid lines (rows) in input
			buffer (input)
	nsav		number of actual valid samples (columns) in
			input buffer (input)
	nlb		number of lines (rows) in boundary (input)
	nsb		number of samples (columns) in boundary (input)
	nlk		number of rows in the kernel (input)
	nsk		number of samples in the kernel (input)
	invld		invalid pixel option (input)
	thresh		threshold value (input)
	minval		minimum valid value (input)
	maxval		maximum valid value (input)
	scalf		scale factor (input)

************************************************************************
*/
#include "las.h"
#include "pixman.h"
#include "protos.h"

int vlow_byt
(
    unsigned char *buffer,
    unsigned char *obuff,
    long *histgm,
    long numh,
    long nsbuf,
    long nlav,
    long nsav,
    long nlb,
    long nsb,
    long nlk,
    long nsk,
    long invld,
    double thresh,
    double minval,
    double maxval,
    double scalf
)
{

long	nlavo;		/* number of actual valid lines of data in   */
			/* the output buffer			     */
long	nsavo;		/* number of actual valid samples of data in */
			/* the output buffer			     */
long	nsobuf;		/* number of samples in the output buffer    */
unsigned char	*oadd;	/* output buffer address */
unsigned char	*iadd;	/* input buffer address */
unsigned char	*oradd;	/* address of kernel origon */
unsigned char	*ladd;	/* line buffer address */
long	i;     		/* counter */
long	dlong = ELONG;	/* data type	*/
long	dbyte = EBYTE;	/* data type	*/
long	zerofil = 0;	/* variable used to zero fill the histogram  */
long	count;		/* counter */
long	cursam;		/* current sample */
long	stat;		/* return status flag */
long	nonfatal;	/* non-fatal error status */


nlavo = nlav - (2 * nlb);
nsavo = nsav - (2 * nsb);
nsobuf = nsbuf - (2 * nsb);

oadd = obuff;
iadd = buffer + (nlb * nsbuf) + nsb;
ladd = buffer;

c_pxfill(histgm,&dlong,&numh,&zerofil);

for (i = 0; i < nlk; i++)
   {
   stat = c_pxhist(&nsk,ladd,&dbyte,histgm,&count);
   if (stat != E_SUCC)
      {
      c_errmsg("Fatal error encountered","filter-fatal",&nonfatal);
      return(E_FAIL);
      }
   ladd += nsbuf;
   }

set_v_byt(minval,maxval,thresh,histgm,scalf,invld,iadd,oadd);

oradd = buffer;
cursam = 1;

for (  ;  ;  )
   {
   for (i = 1; i < nlavo; i++)
      {
      move_d(&oradd,nlk,nsk,nsbuf,histgm);
      oadd += nsobuf;
      iadd += nsbuf;
      set_v_byt(minval,maxval,thresh,histgm,scalf,invld,
	        iadd,oadd);
      }

   if (cursam == nsavo)  break;
   move_r(&oradd,nlk,nsk,nsbuf,histgm);
   cursam++;
   oadd++;
   iadd++;
   set_v_byt(minval,maxval,thresh,histgm,scalf,invld,
	     iadd,oadd);

   for (i = 1; i < nlavo; i++)
      {
      move_u(&oradd,nlk,nsk,nsbuf,histgm);
      oadd -= nsobuf;
      iadd -= nsbuf;
      set_v_byt(minval,maxval,thresh,histgm,scalf,invld,
	        iadd,oadd);
      }

   if (cursam == nsavo)  break;
   move_r(&oradd,nlk,nsk,nsbuf,histgm);
   cursam++;
   oadd++;
   iadd++;
   set_v_byt(minval,maxval,thresh,histgm,scalf,invld,
	     iadd,oadd);

   }   /* end of infinite loop */

return(E_SUCC);

}    /*  end of vlow_byt()  */
