/*====================================================================
   NAME:                          BSRCH

   FUNCTION:
      BSRCH is a routine which returns the transformed pixel value. A 
      binary search is applied on the look up table to find the pixel 
      value read from the replacement block. Once this value is found, 
      the transformed pixel value is located in the second dimension 
      of the look up table.

   
   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C 
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions
====================================================================*/
#include "las.h"
#include "fixlin.h"


void bsrch(
   long  vmax,         /*  I: maximum value in histogram             */
   float cur,          /*  I: value being searched for               */
   long  hisin,        /*  I: current histogram                      */
   float *val,         /*  O: value associated with 'cur'            */
   float lkup[MAXHIS][2][NMRLEV]  /*  I: look up table               */
   )

   {
   long  high;         /*  high value in histogram                   */
   long  low;          /*  low value in histogram                    */
   long  mid;          /*  middle value in histogram                 */
   long  one = 1;      /*  constant one                              */

   low = 1;
   high = vmax;
   mid = (low + high) / 2;

   while(1)
      {
      if (low > high)
         {
         c_errmsg ("Value is not in look up table","fixlin-novalue",
                      &one);
         break;
         }
      else if (cur < lkup[hisin][0][mid-1])
         {
         high = mid - 1;
         mid = (low + high) / 2;
         }
      else if (cur > lkup[hisin][0][mid-1])
         {
         low = mid + 1;
         mid = (low + high) / 2;
         }
      else if (cur == lkup[hisin][0][mid-1])
         {
         *val = lkup[hisin][1][mid-1];
         break;
         }
      }
   }
