/*====================================================================
   NAME:                          DEFWIN

   FUNCTION:
      Computes the number of lines to be included in the proximity and
      replacement histograms.  The histograms in most cases will 
      include the same number of lines before and after the replacement 
      block.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        1.0              K.GACKE            CSB      initial devel.
        2.0      5/86    B.Ailts            CSB      unix coversion
        3.0      3/89    R.McKinney         CSB      LAS 5.0 conv.
        4.0      7/99    K.Hoback           CSB      FORTRAN to C
                                                     conversion.
        4.1      9/99    G.Schmidt          CSB      Fixed the FtoC
                                                     conversions
====================================================================*/
#include "las.h"
#include "fixlin.h"

void defwin (
   long  *ibefor,      /*  O: number of lines before bad data in IN1 */
   long  *iafter,      /*  O: number of lines after bad datain IN1   */
   long  *rbefor,      /*  O: number of lines before bad datain IN2  */
   long  *rafter,      /*  O: number of lines after bad datain IN1   */
   long  iwind[10][4], /*  I: IN1 window specification of bad data   */
   long  rwind[10][2], /*  I: IN2 window specification of bad data   */
   long  wind1[4],     /*  I: window specification of IN1            */
   long  wind2[4],     /*  I: window specification of IN2            */
   long  bn            /*  I: current histogram                      */
   )

   {
   /*  Compute the number of lines included in proximity histogram
   -------------------------------------------------------------------*/
   if (iwind[bn][0] - iwind[bn][2] <= 0)
      *ibefor = iwind[bn][0] - 1;
   else
      *ibefor = iwind[bn][2];

   if (iwind[bn][0] - 1 + (iwind[bn][2] * 2) > wind1[2])
      *iafter = wind1[2] - (iwind[bn][0] + iwind[bn][2]) + 1;
   else
      *iafter = iwind[bn][2];

   /*  Compute the number of lines included in replacement histogram
   -------------------------------------------------------------------*/
   if ((rwind[bn][0] - iwind[bn][2]) <= 0)
      *rbefor = rwind[bn][0] - 1;
   else
      *rbefor = iwind[bn][2];

   if ((rwind[bn][0] - 1 + (iwind[bn][2] * 2)) > wind2[2])
      *rafter = wind2[2] - (rwind[bn][0] + iwind[bn][2]) + 1;
   else
      *rafter = iwind[bn][2];
   }
