#ifndef FIXLIN_H
#define FIXLIN_H


#undef MAXIMG
#define MAXIMG 1       /*  only allow for one input image each       */
#define BSIZE  1024000 /*  buffer size                               */
#define MAXHIS 10      /*  maximum number of histograms              */
#define NMRLEV 16384   /*  max number of different values in hist    */

/*=====================================================================
   This header file is for fixlin

   note: make sure that the las.h and const.h headers are included in 
         the program before this header file.


       I: indicates input to function.
       O: Output from function.
       I/O: input and output to function.
=====================================================================*/
void bsrch(
   long  vmax,         /*  I: maximum value in histogram             */
   float cur,          /*  I: value being searched for               */
   long  hisin,        /*  I: current histogram                      */
   float *val,         /*  O: value associated with 'cur'            */
   float lkup[MAXHIS][2][NMRLEV]  /*  I: look up table               */
   );

void chkwin(
   long  totblk,       /*  I: number of blocks to be fixed           */
   long  iwind[10][4], /*  I: bad block definition(sl,ss,nl,ns)      */
   long  rwind[10][2], /*  I: replacement definition(sl,ss)          */
   long  wind1[4],     /*  I: window specification of IN1            */
   long  wind2[4],     /*  I: window specification of IN2            */
   long  nmrhis[10]    /*  I: number of histograms to be built       */
   );

void defwin (
   long  *ibefor,      /*  O: number of lines before bad data in IN1 */
   long  *iafter,      /*  O: number of lines after bad datain IN1   */
   long  *rbefor,      /*  O: number of lines before bad datain IN2  */
   long  *rafter,      /*  O: number of lines after bad datain IN1   */
   long  iwind[10][4], /*  I: IN1 window specification of bad data   */
   long  rwind[10][2], /*  I: IN2 window specification of bad data   */
   long  wind1[4],     /*  I: window specification of IN1            */
   long  wind2[4],     /*  I: window specification of IN2            */
   long  bn            /*  I: current histogram                      */
   );

void getpar (
   struct PARBLK *vblock, /*  O: TAE parameter block                 */
   char  taein1[CMLEN],   /*  O: TAE input file name 1               */
   char  hosin1[CMLEN],   /*  O: host input file name 1              */
   char  taein2[CMLEN],   /*  O: TAE input file name 2               */
   char  hosin2[CMLEN],   /*  O: host input file name 2              */
   char  taeout[CMLEN],   /*  O: TAE output file name                */
   char  hosout[CMLEN],   /*  O: host output file name               */
   long  *totblk,      /*  O: total number of blocks to fix          */
   long  iwind[10][4], /*  O: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  O: replacement block (sl,ss)              */
   long  wind1[4],     /*  O: window specification of IN(1)          */
   long  nmrhis[10],   /*  O: number of histograms to build          */
   char  minmx[4],     /*  O: minmax flag                            */
   long  *msgtim,      /*  O: time between display messages          */
   long  band1[MAXBND+1], /*  O: band number of IN(1)                */
   long  band2[MAXBND+1], /*  O: band number of IN(2)                */
   long  wind2[4],     /*  O: window specification of IN(2)          */
   long  *nimg,        /*  O: number of images inputted              */
   long  imglimit,     /*  I: maximum number of images allowed       */
   long  *nbands       /*  O: number of bands per input image        */
   );

void hibyte(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   float pixdiv,       /*  I: dividing point of sub-histograms       */
   long  nmrhis,       /*  I: number of histograms to build          */
   long  bn,           /*  I: current block                          */
   char  hosout[CMLEN],/*  I: host output file name                  */
   long  dtype,        /*  I: data type images                       */
   long  ihist[MAXHIS][NMRLEV], /* O: histogram array                */
   long  rhist[MAXHIS][NMRLEV], /* O: replacement histogram array    */
   float ihst[MAXHIS][NMRLEV], /* O: percentages of histogram values */
   float rhst[MAXHIS][NMRLEV]  /* O: percentages of histogram values */
   );

void hilong(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   float pixdiv,       /*  I: dividing point of sub-histograms       */
   long  nmrhis,       /*  I: number of histograms to build          */
   long  bn,           /*  I: current block                          */
   char  hosout[CMLEN],/*  I: host output file name                  */
   long  dtype,        /*  I: data type images                       */
   long  ihist[MAXHIS][NMRLEV], /* O: histogram array                */
   long  rhist[MAXHIS][NMRLEV], /* O: replacement histogram array    */
   float ihst[MAXHIS][NMRLEV], /* O: percentages of histogram values */
   float rhst[MAXHIS][NMRLEV]  /* O: percentages of histogram values */
   );

void hireal(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  *max1,        /*  O:          */
   long  *max2,        /*  O:          */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   float pixdiv,       /*  I: dividing point of sub-histograms       */
   long  nmrhis,       /*  I: number of histograms to build          */
   long  bn,           /*  I: current block                          */
   char  hosout[CMLEN],/*  I: host output file name                  */
   long  dtype,        /*  I: data type images                       */
   float ihist[MAXHIS][2][NMRLEV], /* O: histogram array             */
   float rhist[MAXHIS][2][NMRLEV]  /* O: replacement histogram array */
   );

void hiword(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   float pixdiv,       /*  I: dividing point of sub-histograms       */
   long  nmrhis,       /*  I: number of histograms to build          */
   long  bn,           /*  I: current block                          */
   char  hosout[CMLEN],/*  I: host output file name                  */
   long  dtype,        /*  I: data type images                       */
   long  ihist[MAXHIS][NMRLEV], /* O: histogram array                */
   long  rhist[MAXHIS][NMRLEV], /* O: replacement histogram array    */
   float ihst[MAXHIS][NMRLEV], /* O: percentages of histogram values */
   float rhst[MAXHIS][NMRLEV]  /* O: percentages of histogram values */
   );

void init_imgs(
   char  hosin1[CMLEN], /*  I: host input file name 1                */
   char  hosin2[CMLEN], /*  I: host input file name 2                */
   long  *dtype,        /*  O: data type images                      */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   long  wind1[4],     /*  I: IN(1) window specification             */
   long  wind2[4],     /*  I: IN(2) window specification             */
   long  *vmin,        /*  O: minimum pixel value                    */
   long  *vmax,        /*  O: maximum pixel value                    */
   long  totblk,       /*  I: number of blocks to fix                */
   long  iwind[10][4], /*  I: bad data block (sl,ss,nl,ns)           */
   long  rwind[10][2], /*  I: replacement block definition(sl,ss)    */
   long  nmrhis[10],   /*  I: number of histograms to be built       */
   long  nimg,         /*  I: number of input images                 */
   long  nbands,       /*  I: number of bands per input image        */
   long  *odtype       /*  O: output image data type                 */
   );

void prbyte(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  totblk,       /*  I: total number of blocks                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  dtype,        /*  I: data type images                       */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  tothis[10],   /*  I: number of sub-histograms per block     */
   char  hosout[CMLEN],/*  I: host output file name                  */
   char  minmx[4],     /*  I: min max flag                           */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   long  vmin,         /*  I: minimum value of IN(1) and IN(2)       */
   long  vmax,         /*  I: maximum value of IN(1) and IN(2)       */
   long  msgtim,       /*  I: elapsed time between display mesaages  */
   long  odtype,       /*  I: output image data type                 */
   long  nbands        /*  I: number of bands in the output image    */
   );

void prlong(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  totblk,       /*  I: total number of blocks                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  dtype,        /*  I: data type images                       */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  tothis[10],   /*  I: number of sub-histograms per block     */
   char  hosout[CMLEN],/*  I: host output file name                  */
   char  minmx[4],     /*  I: min max flag                           */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   long  vmin,         /*  I: minimum value of IN(1) and IN(2)       */
   long  vmax,         /*  I: maximum value of IN(1) and IN(2)       */
   long  msgtim,       /*  I: elapsed time between display mesaages  */
   long  odtype,       /*  I: output image data type                 */
   long  nbands        /*  I: number of bands in the output image    */
   );

void prreal(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  totblk,       /*  I: total number of blocks                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  dtype,        /*  I: data type images                       */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  tothis[10],   /*  I: number of sub-histograms per block     */
   char  hosout[CMLEN],/*  I: host output file name                  */
   char  minmx[4],     /*  I: min max flag                           */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   long  msgtim,       /*  I: elapsed time between display mesaages  */
   long  odtype,       /*  I: output image data type                 */
   long  nbands        /*  I: number of bands in the output image    */
   );

void prword(
   struct PARBLK *vblock, /*  I: TAE parameter block                 */
   long  totblk,       /*  I: total number of blocks                 */
   long  iwind[10][4], /*  I: bad data definition(sl,ss,nl,ns)       */
   long  rwind[10][2], /*  I: replacement block (sl,ss)              */
   long  dtype,        /*  I: data type images                       */
   long  wind1[4],     /*  I: window specification of IN(1)          */
   long  wind2[4],     /*  I: window specification of IN(2)          */
   long  tothis[10],   /*  I: number of sub-histograms per block     */
   char  hosout[CMLEN],/*  I: host output file name                  */
   char  minmx[4],     /*  I: min max flag                           */
   long  band1[MAXBND+1], /*  I: band number of IN(1)                */
   long  band2[MAXBND+1], /*  I: band number of IN(2)                */
   long  vmin,         /*  I: minimum value of IN(1) and IN(2)       */
   long  vmax,         /*  I: maximum value of IN(1) and IN(2)       */
   long  msgtim,       /*  I: elapsed time between display mesaages  */
   long  odtype,       /*  I: output image data type                 */
   long  nbands        /*  I: number of bands in the output image    */
   );

void trans(
   long  vmin,         /*  I: minimum value of IN(1) and IN(2)       */
   long  vmax,         /*  I: maximum value in histogram             */
   long  nmrhis,       /*  I: number of histograms to build          */
   float ihst[MAXHIS][NMRLEV], /*  I: percentage of histogram values */
   float rhst[MAXHIS][NMRLEV], /*  I: percentage of histogram values */
   long  lkup[MAXHIS][NMRLEV]  /*  O: look up table                  */
   );

void trreal(
   long  *max1,        /*  I:  maximum value in ihist                */
   long  *max2,        /*  I:  maximum value in rhist                */
   long  nmrhis,       /*  I:  number of histograms                  */
   float ihist[MAXHIS][2][NMRLEV], /*  I:  histogram array           */
   float rhist[MAXHIS][2][NMRLEV], /* I:  replacement hist. array    */
   float lkup[MAXHIS][2][NMRLEV] /*  O: look up table                */
   );

#endif
