/******************************************************************************
NAME:		FLAG	

PURPOSE:   	FLAG flags regions in an image whose areas are less than 
		user specified thresholds for given classes.  The pixels 
		in these regions are flagged by multipilying them by -1.

			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
	P.Pease		09/87	Made mods to fortran version
5.0	D.VanderZee	10/90	Re-retrocode (All new C version!)
5.1	D.Etrheim	11/91	Changed to call bsearch only if a threshold 
				table is being used.
5.2	S.Nelson	 5/92	Updated user guide and error messages.
				Changed parameter name to standard.
5.3	T.Mittan	 9/92	Added extern to curcat to eliminate warnings
				on link.
				
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
FLAG must be run under TAE.

PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"	
#include "typlim.h"
#include "util.h"
#include "protos.h"

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
struct FDESC  *fdesc[3];		/* image file descriptors used for image I/O */
struct GDESC  *gdesc;		/* group descriptor used for image I/O */
long in_ptr[1];		/* pointer into "iobuf" for input bands */
long out_ptr[1];   	/* pointer into "iobuf" for output bands */
extern long curent;		/* current processing line */


int main
(void)
{
struct PARBLK vblock;			/* TAE parameter block */


long	i;				/* loop counter */
long	j;				/* loop counter */
long	*inlin;				/* pointer to input line */
long	*outlin;			/* pointer to output line */
long 	bands[1][MAXBND+1];	 	/* bands to be processed - all images*/
long	msgtim;				/* elapsed time between display msg */
long	nbands;				/* number of bands per input image */
long	nmrimg;				/* number of input images */
long	std_out;			/* LUN to standard output */
long	totbnd;				/* total number of bands */
long	window[1][4];			/* window for each input image */
long	msg_frmt = 3;			/* format flag for processing message*/
long	zero = 0;			/* number zero for passing by address */
long	one = 1;			/* number one for passing by address */
long	sl = 1;				/* start line for output image */
long 	ss = 1;				/* start sample for output image */
long	rd_acc = IREAD;			/* read access flag */
long	wr_acc = IWRITE;		/* write access flag */
long	cflag = NOCOMB;			/* flag for updating ddr */
long	dtype = ELONG;			/* long data type */
long	open_opt[2];			/* optional open parameter */
long	fatal = -1;			/* fatal error flag */
long	nonfatal = 1;			/* nonfatal error flag */
long	bufsiz;				/* image i/o buffer size */
long	nfile = 1;			/* number of output files */
long	mskcls;				/* pixel value for mask class */
long	d_thresh;			/* default threshold value */
long	*regcls;			/* region class numbers */
long	*regcnt;			/* region counts */
long	maxcls;				/* max class value */
long	mincls;				/* min class value */
long	numregs;			/* number of regions in lt */

char	hosin[1][CMLEN];		/* host names of input images */
char	hosout[1][CMLEN];			/* host name of output image */
char	taeout[1][CMLEN];			/* TAE name of output image */
char	regionlt[CMLEN];		/* region labeled table name */
char	threshlt[CMLEN];		/* threshold labeled table name */
char	minmax[4];			/* min max flag */
unsigned char	*iobuf;			/* buffer for image i/o */

double	minval;				/* min values for output image */
double 	maxval;				/* max values for output image */
double  ln_inc = 1;			/* line increment for ddr */
double  smp_inc = 1;			/* sample increment for ddr */
/*****************************************************************************/

/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);

/* Get parameters and globals
-----------------------------*/
getpar(&vblock,hosin,taeout,hosout,regionlt,threshlt,&nmrimg,
	window,bands,&nbands,&totbnd,&mskcls,&d_thresh,&msgtim,minmax);

/* Initialize processing message utility 
----------------------------------------*/
c_inittm("FLAG","Processed","line"," "," ",&window[0][2],&zero,&msgtim,
	&msg_frmt);

/*****************************************************************************/
/* Read region statistics and threshold values and setup for processing images
------------------------------------------------------------------------------*/
setup(regionlt,threshlt,mskcls,d_thresh,&regcls,&regcnt,&mincls,
	&maxcls,&numregs);

/*****************************************************************************/
open_opt[0] = 0;
/* Open input and output images and group them for sequential i/o 
-----------------------------------------------------------------*/
if (c_eopens(&fdesc[0],hosin[0],bands[0],&window[0][0],&window[0][1],
	&window[0][2],&window[0][3],in_ptr,&rd_acc,&dtype,&zero,
	open_opt) != E_SUCC)
   c_errmsg("Fatal error encountered","flag-fatal",&fatal);

/* Determine data type for output image
---------------------------------------*/
if (maxcls <= MAXUTINY && mincls >= MINUTINY)
   open_opt[0] = EBYTE;
else if (maxcls <= MAXWORD && mincls >= MINWORD)
   open_opt[0] = EWORD;
else 
   open_opt[0] = ELONG;

open_opt[1] = 0;

if (c_eopens(&fdesc[1],hosout[0],&nbands,&sl,&ss,&window[0][2],&window[0][3],
	out_ptr,&wr_acc,&dtype,&zero,open_opt) != E_SUCC)
   c_errmsg("Fatal error encountered","flag-fatal",&fatal);
 
/* Allocate memory for image i/o 
--------------------------------*/
if ((iobuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","flag-alloc",&nonfatal);
   c_clean(&vblock,"flag",hosout,&one);
   }

/* Group the images for i/o
---------------------------*/
if (c_egroup(fdesc,&gdesc,iobuf,&bufsiz) != E_SUCC)
   c_clean(&vblock,"flag",hosout,&one); 

/* Start processing messages
----------------------------*/
curent = 1;
c_startm();
curent = 0;

/*****************************************************************************/
/* Process image--flagging regions whose areas fall below threshold values
--------------------------------------------------------------------------*/
for (i = 0; i < window[0][2]; i++)
   {
   curent++;
   if (c_estep(&gdesc) != E_SUCC)
      c_clean(&vblock,"flag",hosout,&one); 

   inlin = (long *)&iobuf[in_ptr[0]*4];
   outlin = (long *)&iobuf[out_ptr[0]*4];

   for (j = 0; j < window[0][3]; j++) 
      {
      if (inlin[j] > numregs || inlin[j] < 1)  
         {
	 c_errmsg("Region number in image not found in labeled table",
						"flag-regnum",&nonfatal);
         c_clean(&vblock,"flag",hosout,&one); 
 	 }
      outlin[j] = regcls[inlin[j]];
      }
   }
 
/* Write out last line of image
-------------------------------*/ 
if (c_estep(&gdesc) != E_SUCC)
   c_clean(&vblock,"flag",hosout,&one); 

/* Finish up processing with housekeeping chores 
------------------------------------------------*/
/* Close images in i/o group 
----------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_clean(&vblock,"flag",hosout,&one);

/* Free image i/o buffer 
------------------------*/
free(iobuf);
free(regcls);
free(regcnt);

/* Update DDR for output image
------------------------------*/
if (c_upddr(&vblock,hosin,hosout[0],&nmrimg,window,bands,&nbands,
	&ln_inc,&smp_inc,&totbnd,&cflag) != E_SUCC)
   c_errmsg("Fatal error encountered","flag-fatal",&fatal);

/* if minmax to be tracked, then store them
-------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   minval = mincls;
   maxval = maxcls; 
   if (c_svmnmx(hosout[0],&totbnd,&minval,&maxval,minmax) != E_SUCC)
      c_errmsg("Fatal error encountered","flag-fatal",&fatal);
   }
 
/* Update history file 
----------------------*/
c_hstout(&vblock,hosout[0],hosin,&nmrimg," ");

/* Stop processing messages
---------------------------*/
c_stoptm();

/* Output completion message 
----------------------------*/
c_complt("FLAG",taeout,&nfile);

return EXIT_SUCCESS;

}
