/*******************************************************************************
NAME:		GETPAR	

PURPOSE:    	GETPAR gets parameters from tae parameter block	
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	9/90	Re-retrocode (All new C version!)
5.1	S.Nelson	5/92	Updated error messages.  Changed parameter name
				to INLT.
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/

#include "las.h"
#include "util.h"
#include "labserv.h"
#include "protos.h"


void getpar
(
    struct PARBLK *vblock,		/* pointer to TAE parameter block */
    char hosin[][CMLEN],         /* host file names of input images */
    char taeout[][CMLEN],		/* TAE file name of output image */
    char hosout[][CMLEN],		/* host file name of output image */
    char *regionlt,		/* region description labeled table */
    char *threshlt,		/* class threshold labeled table */
    long *nmrimg,		/* number of input images */
    long window[][4],		/* window specifications for each input image*/
    long bands[][MAXBND+1],	/* band specifications for each input image */
    long *nbands,		/* number of bands per input image */
    long *totbnd,		/* total number of bands to be processed */
    long *mskcls,		/* mask class value */
    long *d_thrsh,		/* default threshold value */
    long *msgtim,		/* elapsed time between display messages */
    char *minmax		/* min max flag buffer */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block */

long	idtype;			/* input image data type */
long	access = IREAD;		/* access flag */
long	element = 0;		/* parameter element number */
long    fatal = -1;		/* fatal error flag */
long	imglim = 1;		/* maximum number of images to be processed */
long    one = 1;		/* value of one	*/
long	qflag[1][2];		/* flag indicating if subwindows or bands 
							were specified */
long	same;			/* flag indicating if images have same window*/

char 	taein[1][CMLEN];           /* TAE file names of input images */
char 	msg[ERRLEN+1];		/* message buffer */	

/* Retrieve and parse input image specification 
-----------------------------------------------*/
if (c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,nbands,
		totbnd,&imglim,&same,qflag) != E_SUCC)
   c_errmsg("Fatal error encountered","flag-fatal",&fatal);

/* Check for invalid number of bands 
------------------------------------*/
if (*totbnd != 1)
   c_errmsg("Only single band input allowed","flag-nbands",&fatal);

/* Check for valid input data type
----------------------------------*/
if (c_gettyp(hosin,&one,&idtype) != E_SUCC)
   c_errmsg("Fatal error encountered","flag-fatal",&fatal);

if (idtype != EBYTE && idtype != EWORD && idtype != ELONG)
   {
   sprintf(msg,"Image %s not of type byte, integer*2, or integer*4", hosin[0]);
   c_errmsg(msg,"flag-dtype",&fatal);
   }

/* Make labeled table file name 
-------------------------------*/
c_lsmknm(hosin[0],"lt",regionlt);

/* Retrieve output image name
-----------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) != E_SUCC)
   c_errmsg("Fatal error encountered","flag-fatal",&fatal);

/* Retrieve threshold value labeled table file name
---------------------------------------------------*/
if (c_getnam(vblock,"INLT",&element,threshlt,"lt",&access) != E_SUCC)
   c_errmsg("Fatal error encountered","flag-fatal",&fatal);

/* Retrieve default threshold value
-----------------------------------*/
val = p_find(vblock,"DFLTHRSH");
*d_thrsh = IVAL(*val,0);

/* Retrieve mask class value 
----------------------------*/
val = p_find(vblock,"MASKCLSS");
*mskcls = IVAL(*val,0);
 
/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

return;
}
