/******************************************************************************
NAME:   			CLENUP 

PURPOSE:    	To close all files, free all allocated memory, and if necessary,
	    	call clean upon fatal error.

ALGORITHM:	for number of input images
      		   close input images
   		close output images
   		if buffer space was allocated
      		   free buffer space
   		if fatal error was the reason for calling clenup
      		   call clean

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>

#include "las.h"
#include "util.h"
#include "protos.h"

/*
The following variables must be declared global for image I/0 routines
*/
extern struct FDESC * fdescin[MAXIMG+1];  
                               /* image file descriptors used for image I/O   */
extern struct FDESC * fdescot;            
                               /* image file descriptors used for image I/O   */

int clenup
(
    struct PARBLK *vblock,	/* variable block 			      */
    long *nmrimg,		/* number of input images to close	      */
    char hosout[][CMLEN],		/* pointer to output host file name           */
    char *subcmd,		/* T. A. E. subcommand-perform which flip type*/
    unsigned char **in92buf,	/* input image I/O buffer                     */
    unsigned char **out92buf,	/* output image I/O buffer                    */
    long err			/* code for fatal error or not		      */
)
{
long	i;			/* loop counter for number of input images    */
long	non = 1;		/* flag for non-fatal error message	      */
long	numout = 1;		/* pointer to number of output files	      */

char	pname[5];		/* program name for clean		      */


for (i = 0; i < *nmrimg; i++)
   {
   if (c_eclose(&fdescin[i]) != E_SUCC)		/* close input image file */
      c_errmsg("Nonfatal error encountered -- Image data saved",
	       "flip-warn",&non);
   }

if (c_eclose(&fdescot) != E_SUCC)		/* close output image file */
   c_errmsg("Nonfatal error encountered -- Image data saved","flip-warn",&non);

if ((*subcmd == '9') || (*subcmd == '2'))	/* if memory was allocated    */
   {						/* for buffers, free it up    */
   free(*in92buf);
   free(*out92buf);
   }

if (err == TRUE)
   {
   strcpy(pname,"FLIP");
   c_clean(vblock,pname,hosout,&numout);
   }

return(E_SUCC);
}
