/******************************************************************************
NAME:   			GETPAR 

PURPOSE:	GETPAR retrieves the required TAE parameters from the parameter
		block.  The input image is parsed so that the window and band
		specifications are known.  Error checking is done on the input
		image to insure that all input image has been specified with
		the same window.

ALGORITHM:	call getin to retrieve input image specifications
		check the input windows
		call getout to retrieve output image specifications
		obtain the subcommand and set flag for updating the ddr corner-
			points according to subcommand
		obtain TAE globals

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>

#include "las.h"
#include "util.h"
#include "protos.h"

#define IN 0
			/* index for INput window specs		      */
				/* and 270 images			      */
/*
The following variables must be declared global for image I/0 routines
*/
extern struct FDESC * fdescin[MAXIMG+1];  
                               /* image file descriptors used for image I/O   */
extern struct FDESC * fdescot;            
                               /* image file descriptors used for image I/O   */

int getpar
(
    struct PARBLK *vblock,	/* variable block 			     */
    char *subcmd,			/* T.A.E. subcommand */
    long *rot,			/* rotation for updating ddr cornerpoints     */
    char taein[][CMLEN],		/* pointer to input TAE file array           */
    char hosin[][CMLEN],		/* pointer to input host file array          */
    char taeout[][CMLEN],		/* pointer to output TAE file array          */
    char hosout[][CMLEN],		/* pointer to output host file array         */
    long *nmrimg,			/* number of input images 		     */
    long window[][4],		/* window specifications for input image     */
    long bands[][MAXBND+1],/* band specifications for each input image  */
    long *nbands,			/* number of bands per input image           */
    long *totbnd,			/* total number of bands to be processed     */
    long *msgtim,			/* elapsed time between display messages     */
    char mnmx[]			/* min max flag        		     	     */
)
{
struct  VARIABLE *val;		/* pointer to variable struct 		     */

long  element;			/* parameter element number		     */
long  imglim;			/* maximum number of images to be processed  */
long  non = 1;			/* non-fatal error message flag		     */
long  qflag[MAXIMG][2];		/* flag indicating if subwindows specified   */
long  same;			/* flag indicating if images have same window*/
long  status;			/* return status 		     	     */

element = 0;
imglim = MAXIMG;

/* 
Get the user specified input image specifications from the parameter block
*/
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,
		 bands,nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   return(E_FAIL);

if (same != 1)				/* Verify all input images same size  */
   {
   c_errmsg("Input images must be same size","flip-same",&non);
   return(E_FAIL);
   }

/* 
Get user specified output image specifications from the parameter block
*/
if (c_getout(vblock,"OUT",taeout,hosout) != E_SUCC)
   return(E_FAIL);

val = p_find(vblock, "_SUBCMD"); 	/* Determine the subcommand chosen  */
strcpy(subcmd,SVAL(*val,0));

if (*subcmd == '9')			/* assign rotation code for updating */
   *rot = 1;				/* ddr cornerpoints		     */
else if (*subcmd == '1')
   *rot = 2;
else if (*subcmd == '2')
   *rot = 3;
else if (*subcmd == 'F')
   *rot = 4;
else if (*subcmd == 'M')
   *rot = 5;

val = p_find(vblock,"$MSGTIM");		/* Retrieve TAE globals              */
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$MINMAX");
strcpy(mnmx,SVAL(*val,0));

return(E_SUCC);
}
