#define IN 0			/* index for INput window specs		      */
#define BUFSZ 1024		/* buffer size for stepping through 90 	      */


#include "las.h"
#include "util.h"
#include "protos.h"                

                
                /* and 270 images			      */
/*
The following variables must be declared global for image I/0 routines
*/
extern struct FDESC * fdescin[MAXIMG + 1];
extern struct FDESC * fdescot;

int initio
(
    char *subcmd,		/* T. A. E. subcommand-perform which flip type*/
    char hosin[][CMLEN],		/* pointer to input host file array           */
    char *hosout,		/* pointer to output host file array          */
    long *nmrimg,		/* pointer to number of input images from IN  */
    long window[][4],		/* window specification of input images       */
    long *totbnd,		/* total number of bands to be processed      */
    long *imgln,			/* storage for number of lines in output img  */
    long *imgsmp,		/* storage for number of samples, output img  */
    long *nl,			/* number of image lines                      */
    long *ns,			/* number of image samples                    */
    long *nl2sav,		/*number of input lines to read & save per buf*/
    long *ns2sav,		/* number of input samples to read & save/ln  */
    long *size,       		/* number of bytes per pixel                  */
    long *dtype,			/* data type of images                        */
    long *bufflg,		/* flag to indicate io buffering is needed    */
    unsigned char **in92buf,     	/* input image I/O buffer                     */
    unsigned char **out92buf     	/* output image I/O buffer                    */
)

{
long	access; 		/* image I/O access type                      */
long 	bufsiz = (BUFSZ * BUFSZ);
				/* max buffer size for buffering read/writes  */
long	dumbnd;			/* dummy num bands for open,                  */
long 	i;			/* loop counter				      */
long	non = 1;		/* non-fatal error message flag		      */
long	one = 1;		/* value of one                               */
long	status;			/* function return status code                */
long    opt[3];

opt[0] = opt[1] = opt[2] = 0;
access = IREAD;					/* open input image file      */
*dtype = 0;            

*imgln = window[IN][NL];
*imgsmp = window[IN][NS];

*nl = *imgln;
*ns = *imgsmp;
*nl2sav = *imgln;
*ns2sav = *imgsmp;

for (i = 0; i < *nmrimg; i++)
   {
   dumbnd = 0;
   status = c_eopenr(&fdescin[i],hosin[i],&dumbnd,&window[i][0],&window[i][1],
		     &window[i][2],&window[i][3],&access,dtype,opt,&one);
   if (status != E_SUCC)
      return(E_FAIL);
   }

fdescin[*nmrimg]=0;

if (c_gettyp(hosin,nmrimg,dtype) != E_SUCC)
   {                  
   c_errmsg("Input images must be of same data type","flip-type",&non);
   return(E_FAIL);
   }

switch(*dtype)
   {
   case EBYTE:	*size = 1; break;
   case EWORD:  *size = 2; break;
   case ELONG:  *size = 4; break;
   case EREAL:	*size = 4; break;
   }

access = IWRITE;				/* to open output image file */
if ((*subcmd == '9') || (*subcmd == '2'))	/* first assign correct nl   */
   {						/* and ns, and decide whether*/
                           			/* or not to buffer the image*/
   if ((((*imgsmp) * (*size)) * (*imgln)) > bufsiz)
      {
      if (*imgln > BUFSZ)
	 {
         *nl2sav = BUFSZ;
         if (((*imgsmp) * (*size)) > BUFSZ)
            *ns2sav = BUFSZ / (*size);
         else
	    *ns2sav = (*imgsmp);
	 }
      else
	 {
         *nl2sav = (*imgln);
	 *ns2sav = BUFSZ / (*size);
	 }
      status = c_eopenr(&fdescot,hosout,totbnd,&one,&one,ns,nl,&access,dtype,
		        opt,&one);
      if (status != E_SUCC)
         return(E_FAIL);

      if (c_eclose(&fdescot) != E_SUCC)
         c_errmsg("Nonfatal error encountered--Image data saved",
		  "flip-warn",&non);

      access = IUPDATE;
      status = c_eopenr(&fdescot,hosout,totbnd,&one,&one,ns2sav,nl2sav,&access,
		      dtype,opt,&one);
      if (status != E_SUCC)
         return(E_FAIL);

      (*bufflg) = TRUE;
      }
    else
      {
      status = c_eopenr(&fdescot,hosout,totbnd,&one,&one,ns2sav,nl2sav,
			&access,dtype,opt,&one);
      if (status != E_SUCC)
         return(E_FAIL);

      (*bufflg) = FALSE;
      }
   }
else
   status = c_eopenr(&fdescot,hosout,totbnd,&one,&one,nl,ns,&access,dtype,
		     opt,&one);

if ((*subcmd == '9') || (*subcmd == '2'))		/* assign 2D buffers  */
   {
   *in92buf = (unsigned char *) malloc(bufsiz);		/* allocate I/O buffer*/
   if (*in92buf == NULL)
      {
      c_errmsg("Error allocating dynamic memory","flip-alloc",&non);
      return(E_FAIL);
      }

   *out92buf = (unsigned char *)malloc(bufsiz);	/* allocate I/O buffer*/
   if (*out92buf == NULL)
      {
      c_errmsg("Error allocating dynamic memory","flip-alloc",&non);
      return(E_FAIL);
      }
   }                                 

return(E_SUCC);
}
