/******************************************************************************
NAME:   			PROCES 

PURPOSE:	PROCES actually processes the input image, rearranging output
		pixels a line  at a time, or a buffered number of samples at a
		time for 90 and 270 degree rotations.

ALGORITHM:	in each subcommand case
		   set counters
		   read a number of lines into a buffer
		   rearrange the buffer
		   write ouput buffer
		   reset counters

ALGORITHM REFERENCES:		none
******************************************************************************/
#include <stdio.h>

#include "las.h"
#include "pixman.h"
#include "protos.h"

/*
The following variables must be declared global for image I/0 routines
*/

extern struct FDESC * fdescin[MAXIMG + 1];
                                /* file descriptors for input images	      */
extern struct FDESC * fdescot;	       
                                /* file descriptor for output image	      */
extern long curent;            /* current entity for processing message       */
extern long curcat;            /* current category for processing message     */

int proces
(
    char *subcmd,		/* T. A. E. subcommand-perform which flip type*/
    long *nmrimg,		/* pointer to number of input images specified*/
    long *nbands,		/* number of bands per input image            */
    long *nl,			/* number of image lines                      */
    long *ns,			/* number of image samples                    */
    long *nl2sav,		/* number of input lines to read & save / buf */
    long *ns2sav,		/* number of input samples to read & save /ln */
    long *sabfct,		/* counter for stepping through 1K samples    */
    long *size,       		/* number of bytes per pixel                  */
    long *dtype,			/* data type of images                        */
    unsigned char *inbuf,		/* input image line I/O buffer                */
    unsigned char *outbuf,		/* output image line I/O buffer               */
    unsigned char *in92buf,     	/* input image I/O buffer                     */
    unsigned char *out92buf,     	/* output image I/O buffer                    */
    char *hosout,		/* pointer to output host file name           */
    long *totbnd,		/* pointer to total number of output bands    */
    long bands[][MAXBND+1]  /* array of bands for input image         */
)
{
long    access = IUPDATE;	/* access mode for opening output file        */
long	bndcnt;			/* band for loop counter		      */
long	bytecnt;		/* counter for accessing each byte of buffers */
long	cnt;			/* counter for number of images loop	      */
long    ct;			/* counter				      */
long	hct;			/* horizantal loop counter		      */
long	inct;			/* reverse input counter		      */
long    inlnct;			/* input image lines counter                  */
long    inpxct;       		/* input image pixels counter                 */
long	lnbfct;			/* counter for stepping through 1K lines      */
long	mnmxct = -1;		/* counter for min and max arrays	      */
long	non = 1;		/* non-fatal error message flag		      */
long	one = 1;		/* value of one                               */
long	opln;			/* number of lines for re-opening image	      */
long	opsmp;			/* number of samps for re-opening image	      */
long    otlnct;			/* output image lines counter                 */
long    otpxct;       		/* output image pixels counter                */
long    sabfln;			/* number of lines to read into buffer	      */
long    sabfsmp;		/* number of samples per line to read into buf*/
long    sl;			/* starting line for open		      */
long	status;			/* function return status code                */
long    ss;			/* starting sample for open		      */
long	totnl;			/* tot num lines left in current BUFSZ samps  */
long	totns;			/* tot num smpls left in current BUFSZ lines  */
long	vct;			/* vertiacal loop counter		      */
long    opt[3];

opt[0] = opt[1] = opt[2] = 0;

curcat = 1;
for (cnt = 0; cnt < *nmrimg; cnt++)
   {
   for (bndcnt = 0; bndcnt < nbands[cnt];bndcnt++,curcat++)
      {
      mnmxct++;
      switch (*subcmd)			/* flip image by subcmd */
	 { 				/* curent is the output line counter */
	 case ('F'):		/* flip image on its horizontal axis */
	    for (inlnct = *nl,curent = 1;inlnct > 0;inlnct--,curent++)
	       {       
	       if (c_eread(&fdescin[cnt],&bands[cnt][bndcnt],&inlnct,inbuf,
                   &one) != (*ns))
		  {
		  c_errmsg("Error reading input image","flip-read",&non);
		  return(E_FAIL);
		  }                      
 	       if (c_ewrite(&fdescot,&curcat,&curent,inbuf,&one) != (*ns))
		  {
		  c_errmsg("Error writing output image","flip-write",&non);
		  return(E_FAIL);
		  }
	       }
	    break;
	 case ('M'):		/* mirror image on its vertical axis */
	    for (curent = 1;curent <= (*nl);curent++)
	       {
	       if (c_eread(&fdescin[cnt],&bands[cnt][bndcnt],&curent,inbuf,
                  &one) != (*ns))
		  {
		  c_errmsg("Error reading input image","flip-read",&non);
		  return(E_FAIL);
		  }
	       for (inpxct = (*ns - 1),otpxct = 0;inpxct >= 0;inpxct--,otpxct++)
		  {
		  c_pxcopy((inbuf + (inpxct * (*size))),
			  (outbuf + (otpxct * (*size))),dtype,&one);
		  }
	       if (c_ewrite(&fdescot,&curcat,&curent,outbuf,&one) != (*ns))
		  {
		  c_errmsg("Error writing output image","flip-write",&non);
		  return(E_FAIL);
		  }
	       }
	    break;
	 case ('1'):			/* rotate image clockwise 180 degrees */
	    for (inlnct = (*nl),curent = 1;inlnct > 0;inlnct--,curent++)
	       {
	       if (c_eread(&fdescin[cnt],&bands[cnt][bndcnt],&inlnct,inbuf,
	          &one) != (*ns))
		  {
		  c_errmsg("Error reading input image","flip-read",&non);
		  return(E_FAIL);
		  }
	       for (inpxct = (*ns - 1),otpxct = 0;inpxct >= 0;inpxct--,otpxct++)
		  {
		  c_pxcopy((inbuf + (inpxct * (*size))),
			   (outbuf + (otpxct * (*size))),dtype,&one);
		  }
	       if (c_ewrite(&fdescot,&curcat,&curent,outbuf,&one) != (*ns))
		  {
		  c_errmsg("Error writing output image","flip-write",&non);
		  return(E_FAIL);
		  }
	       }
	    break;
	 case ('9'):			/* rotate image clockwise 90 degrees */
	    curent = 1;			/* a bufsiz window at a time - this  */
	    sl = 1;			/* reduces the number of times an out*/
	    ss = 1;			/* put line is re-written, increasing*/
	    opln = (*ns2sav);		/* performance */
	    opsmp = (*nl2sav);
	    totnl = (*nl);
	    totns = 1;
	    lnbfct = 1;
	    sabfln = (*nl2sav);
	    sabfsmp = (*ns2sav);
	    (*sabfct) = (*ns2sav);
	    while (totns < (*ns))
	       {
	       for (inlnct = totnl,hct = 0;inlnct >= (totnl - sabfln + 1);
		    inlnct--,hct++)
		  {
		  if (c_eread(&fdescin[cnt],&bands[cnt][bndcnt],&inlnct,
                     in92buf,&one) != (*ns))
		     {
		     c_errmsg("Error reading input image","flip-read",&non);
		     return(E_FAIL);
		     }
		  for (inct = 0,vct = (lnbfct - 1);inct < sabfsmp;inct++,vct++)
                     {
		     for (bytecnt = 0; bytecnt < (*size);bytecnt++)
                        {
		        *(out92buf + ((inct * (*nl2sav)) * (*size)) +
				     (hct * (*size)) + bytecnt) = 
		        *(in92buf + (vct * (*size)) + bytecnt);
                        }
                     }   
		  }
	       for (ct = 0,otlnct = 1;ct < opln;ct++,otlnct++)
		  {
		  if (c_ewrite(&fdescot,&curcat,&otlnct,
		      out92buf + (((*nl2sav) * ct) * (*size)),&one) != (opsmp))
		     {
		     c_errmsg("Error writing output image","flip-write",&non);
		     return(E_FAIL);
		     }
		  }
	       /* reset counters		      			      */
	       totnl = totnl - sabfln;
	       if (totnl <= 0) 	    	/* less than buffer of lines left */
		  {
		  if (totns >= ((*ns) - (*ns2sav)))
		     {		    /* less than buffer of samps left */
		     totns = totns + (*ns2sav);
		     curent = curent + sabfsmp - 1;
		     sl = 1;
		     ss = 1;
		     opln = (*ns2sav);
		     opsmp = (*nl2sav);
		     }
		  else		    	/* more than buffer of samps left */
		     {
		     totnl = (*nl);
		     totns = totns + (*ns2sav);
		     ss = 1;
		     sl = sl + sabfsmp;
		     curent = curent + sabfsmp;
		     sabfsmp = (*ns2sav);
		     lnbfct = lnbfct + sabfsmp;
		     sabfln = (*nl2sav);
		     opsmp = sabfln;
		     if (totns >= ((*ns) - (*ns2sav)))
		        opln =  (*ns) - totns + 1;
		     else
		        opln = sabfsmp;
		     }
		  }
	       else		    	/* more than buffer of lines left */
		  {
		  if (totns >= ((*ns) - (*ns2sav)))
		     {		    	/* less than buffer of samps left */
		     ss = ss + sabfln;
		     if (totnl <= (*nl2sav))
		        opsmp = totnl;
		     if (totnl < (*nl2sav))
		        {
			sabfln = totnl;
			sabfsmp = (*ns) - totns + 1;
			}
		     else
		        {
			sabfln = (*nl2sav);
			sabfsmp = (*ns2sav);
			}
		     }
  		   else
		     {
		     ss = ss + sabfln;
		     if (totnl < (*nl2sav))
			 sabfln = totnl;
		     opsmp = sabfln;
		     }
		  }
	    	  if ((totns < (*ns)) || (bndcnt < nbands[cnt]))
		     {
	             if (c_eclose(&fdescot) != E_SUCC)
   		        c_errmsg("Nonfatal error encountered--Image data saved",
		                 "flip-warn",&non);

	             status = c_eopenr(&fdescot,hosout,totbnd,&sl,&ss,
		          	       &opln,&opsmp,&access,dtype,opt,&one);
	             if (status != E_SUCC)
   	                return(E_FAIL);
		     }
	       }
 	    break;
	 case ('2'):			/* rotate image clockwise 270 degrees */
	    curent = 1;			/* a bufsiz window at a time - this  */
	    sl = 1;			/* reduces the number of times an out*/
	    ss = 1;			/* put line is re-written, increasing*/
	    opln = (*ns2sav);		/* performance */
	    opsmp = (*nl2sav);
	    totnl = (*nl);
	    totns = 1;
	    lnbfct = 1;
	    sabfln = (*nl2sav);
	    sabfsmp = (*ns2sav);
	    (*sabfct) = (*ns2sav);
	    while (totns < (*ns))
	       {
	       for (hct = 0,inlnct = (*nl - totnl + 1); hct < sabfln;
		    inlnct++,hct++)
		  {
		  status = c_eread(&fdescin[cnt],&bands[cnt][bndcnt],
                            &inlnct,in92buf,&one);
		  if (status != (*ns))
		     {
		     c_errmsg("Error reading input image","flip-read",&non);
		     return(E_FAIL);
		     }
		  for (vct = 0,inct = (*ns - totns);vct < sabfsmp;inct--,vct++)
                     {
		     for (bytecnt = 0; bytecnt < (*size);bytecnt++)
                        {
  		        *(out92buf + ((vct * (*nl2sav)) * (*size)) +
				     (hct * (*size)) + bytecnt) = 
		        *(in92buf + (inct * (*size)) + bytecnt);
                        }
                      }
		  }
	       for (ct = 0,otlnct = 1;ct < opln;ct++,otlnct++)
		  {
/*       
		  status =  c_ewrite(&fdescot,&bndcnt,&otlnct,
  		      		 out92buf + (((*nl2sav) * ct) * (*size)),&one);
*/
		  status =  c_ewrite(&fdescot,&curcat,&otlnct,
  		      		 out92buf + (((*nl2sav) * ct) * (*size)),&one);
		  if (status != opsmp)                     
		     {
		     c_errmsg("Error writing output image","flip-write",&non);
		     return(E_FAIL);
		     }
		  }
	       /* reset counters		      			      */
	       totnl = totnl - sabfln;
	       if (totnl <= 0) 	    	/* less than buffer of lines left */
		  {
		  if (totns >= ((*ns) - (*ns2sav)))
		     {		    /* less than buffer of samps left */
		     totns = totns + (*ns2sav);
		     curent = curent + sabfsmp - 1;
		     sl = 1;
		     ss = 1;
		     opln = (*ns2sav);
		     opsmp = (*nl2sav);
		     }
		  else		    	/* more than buffer of samps left */
		     {
		     totnl = (*nl);
		     totns = totns + (*ns2sav);
		     ss = 1;
		     sl = sl + sabfsmp;
		     curent = curent + sabfsmp;
		     lnbfct = lnbfct + sabfsmp;
		     sabfln = (*nl2sav);
		     opsmp = sabfln;
		     if (totns >= ((*ns) - (*ns2sav)))
		        opln =  (*ns) - totns + 1;
		     else
		        opln = sabfsmp;
		     sabfsmp = opln;
		     }
		  }
	       else		    	/* more than buffer of lines left */
		  {
		  if (totns >= ((*ns) - (*ns2sav)))
		     {		    	/* less than buffer of samps left */
		     ss = ss + sabfln;
		     if (totnl <= (*nl2sav))
		        opsmp = totnl;
		     if (totnl < (*nl2sav))
		        {
			sabfln = totnl;
			sabfsmp = (*ns) - totns + 1;
			}
		     else
		        {
			sabfln = (*nl2sav);
			sabfsmp = (*ns2sav);
			}
		     }
  		   else
		     {
		     ss = ss + sabfln;
		     if (totnl < (*nl2sav))
			 sabfln = totnl;
		     opsmp = sabfln;
		     }
		  }
	    	  if ((totns < (*ns)) || (bndcnt < nbands[cnt]))
		     {
	             if (c_eclose(&fdescot) != E_SUCC)
   		        c_errmsg("Nonfatal error encountered--Image data saved",
		                 "flip-warn",&non);

	             status = c_eopenr(&fdescot,hosout,totbnd,&sl,&ss,
		          	       &opln,&opsmp,&access,dtype,opt,&one);
	             if (status != E_SUCC)
   	                return(E_FAIL);
		     }
	       }
 	    break;
	}
      }
   }
return(E_SUCC);
}
