/****************************************************************************

NAME: CHANGE_CHANNEL

PURPOSE: To change the channel number (3rd character) of the input filename
         to '2'.  This will be called for Fast-L7A bands of 6H, 7, and 8.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  07/99  G.Schmidt        EDC   Initial development
         10/01  A.Weissenberger  EDC   CR 5057. Formatting and remove
                                       redundant c_lasmsg calls.
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:
****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "fhdr.h"
#include "util.h"
#include "fstfmtin.h"

void main () 
{
    struct PARBLK vblk;        /* TAE parameter block                       */
    long std_out = 0;          /* standard out                              */
    char infile[CMLEN];        /* input file name                           */
    char *slashptr = NULL;     /* ptr to the infile at the last '/'         */

    c_inlas (&vblk, &std_out);

    /* Get the variables from the TAE variable block
       ------------------------------------------------*/
    ccgetpar (&vblk, infile);

    /* Change the channel number to a '2'.  The channel is the 3rd character
       in the basename of the input string.
       --------------------------------------------------------------------*/
    slashptr = (char *) strrchr (infile, '/');
    if (slashptr != NULL)
    {
        slashptr++;    /* now at the beginning of the basename */
        slashptr += 2; /* now points to the 3rd character position in
                          the basename */
    }
    else
        slashptr = &infile[2];

    slashptr[0] = '2';

    /* Pass the string back to TAE
       ------------------------------*/
    if (E_SUCC != str2tae(&vblk,infile,sizeof(infile),"%s",1,"TMPOUT"))
    {
        c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting TMPOUT");
    }

    if (q_out(&vblk) != SUCCESS)
    {
        c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
    }
}
