/***************************************************************************

NAME: CHECK_RES

PURPOSE: To look through the specified bands and determine if they are of
         the same resolution.

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  06/99  G.Schmidt        EDC   Initial development
         10/01  A.Weissenberger  EDC   CR 5057. Remove redundant c_lasmsg
                                       calls, formatting.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES: This routine is only valid for Fast-L7A bands.  The results of this
routine will not necessarily be valid for any other format.

****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "fhdr.h"
#include "util.h"
#include "fstfmtin.h"

void main () 
{
    long std_out = 0;          /* standard out                             */
    long tmp_long;             /* temporary long value                     */
    char bands[MAXBND][6];     /* array of the bands to search for (each band
                                  may be up to 5 characters/digits long)   */
    int nbands;                /* the number of bands in the bands array   */
    long same_res = 1;         /* specifies if the bands are all of the same
                                  resolution (0=no, 1=yes)                 */
    int i;
    struct PARBLK vblk;          /* TAE parameter block     */

    c_inlas (&vblk, &std_out);

    /* Get the variables from the TAE variable block
       ------------------------------------------------*/
    crgetpar (&vblk, bands, &nbands);

    /* Determine if the bands are of the same resolution.  Bands (1-5,7),
       (6,9), and 8 are packaged together. NOTE: Loop through all the bands
       and compare them to the first band.  If there is only one band, then
       same_res is already set to 1.  If same_res ever gets set to 0, then
       we can exit the loop.
       --------------------------------------------------------------------*/
    for (i = 1; i < nbands; i++)
    {
        switch (bands[0][0])
        {
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '7':
                if ((bands[i][0] != '1') && (bands[i][0] != '2') &&
                    (bands[i][0] != '3') && (bands[i][0] != '4') &&
                    (bands[i][0] != '5') && (bands[i][0] != '7'))
                    same_res = 0;
                break;

            case '6':
            case '9':
                if ((bands[i][0] != '6') && (bands[i][0] != '9'))
                    same_res = 0;
                break;

            case '8':
                /* If we are in this loop then there are more than one bands.
                   If there are more than one bands and one of them is band
                   8, then there is no way all the bands can be of the same
                   resolution.
                   --------------------------------------------------------*/
                same_res = 0;
                break;
        }

        /* If this band was not of the same resolution, then we can break out.
           -----------------------------------------------------------------*/
        if (!same_res)
            break;
    }

    /* Pass the same_res flag back to TAE
       -------------------------------------*/
    if (E_SUCC != lng2tae(&vblk,&same_res,1,1,"SAME_RES",&tmp_long))
    {
        c_lasmsg(TRUE,"fstfmtin-lng2tae","Error outputting SAME_RES");
    }

    if (q_out(&vblk) != SUCCESS)
    {
        c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
    }
}
