/****************************************************************************
NAME: GETPAR

PURPOSE: Gets the user-specified parameters and tae globals for
convert_window.

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  06/99  G.Schmidt        EDC   Updated to support Fast-L7A.
         10/01  A.Weissenberger  EDC   CR 5057. Formatting and remove
                                       redundant c_lasmsg
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:

****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "fhdr.h"
#include "fstfmtin.h"

void cwgetpar (
    struct PARBLK *vblk,   /* I: pointer to initial TAE parameter block     */
    char *inhdr,           /* O: name of header file to read                */
    char *htype,           /* O: header type (A,B,C,L7A,SIC)                */
    double window[4],      /* O: input window in units (UL y/lat, UL x/long,
                              LR y/lat, LR x/long)                       */
    char *units,           /* O: units of the input window (DEG, DMS,
                              LS, PRO)*/
    long *resolution,      /* O: L7 resolution type (VISIBLE, THERMAL, PAN) */
    long *same_res)        /* O: are bands of the same resolution?          */

{
    struct VARIABLE *var;      /* ptr to a variable in the vblk             */
    long num;                  /* number of values in the input param       */
    long i;

    /* Get the header type
       ----------------------*/
    var = p_find(vblk, "HTYPE");
    strcpy(htype, SVAL(*var, 0));

    /* Get the input header
       -----------------------*/
    var = p_find(vblk, "INHDR");
    strcpy(inhdr, SVAL(*var, 0));

    /* Get the input window
       -----------------------*/
    var = p_find(vblk, "WINDOW");
    num = var->v_count;
    if (num != 4)
    {
        c_lasmsg(TRUE,"fstfmtin-window","Window must contain 4 parameters");
    }
    for (i = 0; i < num; i++)
        window[i] = RVAL(*var, i);

    /* Get the input units
       ----------------------*/
    var = p_find(vblk, "UNITS");
    strcpy(units, SVAL(*var, 0));

    /* Get the L7 resolution type
       -----------------------------*/
    var = p_find (vblk, "RESOLUTION");
    *resolution = IVAL (*var, 0);

    /* Get the same_res flag
       ------------------------*/
    var = p_find (vblk, "SAME_RES");
    *same_res = IVAL (*var, 0);
}
