/****************************************************************************

NAME: GETPAR

PURPOSE: Get parameters for fhdr2ddr

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  07/99  G.Schmidt        EDC   Updated to support Fast-L7A.
         07/01  A.Weissenberger        CR 5057. Formatting and removal of
                                       redundant c_lasmsg calls.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Initialize las 
   Get input image parameters
   Get input header name
   Get globals

NOTES:

****************************************************************************/
#include <string.h>
#include "fhdr2ddr.h"
#include "util.h"
#include "fstfmtin.h"

void getpar (
    struct PARBLK *vblk,   /* I: pointer to TAE parameter block             */
    char hostimg[CMLEN],   /* O: input image host name                      */
    char hosthdr[CMLEN],   /* O: input header host name                     */
    long wind[4],          /* O: input image window                         */
    char bandlist[][6],    /* O: list of bands read from tape               */
    int *nbands,           /* O: number of bands in bands array             */
    char *htype,           /* O: header type                                */
    long *hflag)           /* O: create fhdr flag                           */

{
    char lashdr[CMLEN];              /* input header las name               */
    long i;                          /* working index value                 */
    long status;                     /* function return status              */
    long access = IREAD;             /* reading the image and header        */
    long zero = 0;

    struct VARIABLE *var;            /* pointer to a variable in TAE block  */


    /* get the image name
       ---------------------*/
    if (c_getnam (vblk, "IN", &zero, hostimg, "img", &access) != E_SUCC)
    {
        c_lasmsg(TRUE,"fstfmtin-host","Error retrieving the host image name");
    } 


    /* get the fast format header name
       ----------------------------------*/
    var = p_find(vblk, "HEADER");
    strcpy(lashdr,SVAL(*var,0));
    i = (long) strlen(lashdr);
    c_up2low(lashdr,&i); 
    status = c_cmhost(lashdr,hosthdr);
    if(status == E_FAIL)
    {
        c_lasmsg(TRUE,"fstfmtin-cmhost","Error returned from cmhost");
    }


    /* determine the header type
       ---------------------------------*/
    var = p_find (vblk, "HTYPE");
    strcpy (htype, SVAL(*var, 0));


    /* get the bands being read from tape
       --------------------------------------*/
    var = p_find(vblk, "BANDS");
    *nbands = var->v_count;
    for(i=0;i < *nbands; i++)
        strcpy(bandlist[i],SVAL(*var,i));


    /* get the window from the original image
       -----------------------------------------*/
    var = p_find(vblk, "WINDOW");
    for (i = 0; i < var->v_count; i++)
        wind[i] = IVAL(*var,i);


    /* get the header copy flag
       ---------------------------*/
    var = p_find(vblk, "HDRFLG");
    if(strcmp("YES",SVAL(*var,0)))
        *hflag = 0;
    else
        *hflag = 1;
}
