/****************************************************************************
NAME: GETPAR

PURPOSE: gets the user-specified parameters and tae globals for locate_cd_hdr

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.3  09/00  A.Shimitz        EDC   Initial development - CR #3519
         10/01  A.Weissenberger  EDC   CR 5057. Formatting and removal of
                                       redundant calls to c_lasmsg.
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:

***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "fhdr.h"
#include "fstfmtin.h"

#define BASNAMLEN 120

void lcdgetpar (
    struct PARBLK *vblk,   /* O: pointer to initial TAE parameter block     */
    char oprmsg[],         /* I: message for the operator for CD mounting   */
    char drivename[],      /* I: name of the drive the CD is on             */
    char basename[][BASNAMLEN],
    /* I: array of basenames of header files - if the
       user did not specify the basename in INFILE
       we will determine the basenames from the
       header files we find                       */
    long *mounted,         /* I: flag indicating if the CD is mounted       */
    char opr_drive[],      /* I: tape drive unit assigned                   */
    long *nnames)          /* O: number of basenames in basename array -
                              at this point it should be either 0 or 1,
                              depending on if the user specified an INFILE
                              or not          */

{
    struct VARIABLE *var;    /* ptr to a variable in the vblk               */
    int i;                   /* looping index                               */

    /* Get the operator message
       ---------------------------*/
    var = p_find(vblk, "OPERATOR_MSG");
    strcpy(oprmsg, SVAL(*var, 0));

    /* Get the mounted flag
       -----------------------*/
    var = p_find(vblk, "MOUNTED");
    *mounted = IVAL(*var, 0);

    if (*mounted == TRUE)
    {
        /* Retrieve the drivename
           -------------------------*/
        var = p_find(vblk, "DRIVE");
        strcpy(drivename, SVAL(*var, 0));

        /* Get the operator drive name
           ------------------------------*/
        var = p_find(vblk, "OPR_DRIVE");
        strcpy(opr_drive, SVAL(*var, 0));
    }

    /* Get the user's basename
       --------------------------*/
    var = p_find(vblk, "BASENAME");
    *nnames = var->v_count;
    for(i = 0; i < *nnames; i++)
        strcpy(basename[i],SVAL(*var,i));
}
