/****************************************************************************

NAME: lng2tae

PURPOSE: Retrieve a long variable from a C program and place it in the TAE 
         variable block.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
         10/01  A.Weissenberger  EDC   CR 5057. Formatting and removal of
                                       redundant c_lasmsg calls.
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:

****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "fstfmtin.h"


int lng2tae (
    struct PARBLK *vblk, /* I: TAE parameter block                         */
    long data[],         /* I: array of longs to transfer                  */
    long skipf,          /* I: skip factor                                 */
    long num_longs,      /* I: number of elements                          */
    char varname[],      /* I: tae name of variable                        */
    long *result)        /* O: value of parameter                          */

{

    long *numbers;       /* the numbers retrieved from the buffer           */
    long *ptr;           /* pointer to walk down the list of numbers
                            retrieved*/
    long i;              /* loop control                                   */
    long index;          /* data index                                      */

    numbers = (long *)malloc(num_longs * sizeof(long));
    if (numbers == NULL)
    {
        c_lasmsg(TRUE,"fstfmtin-alloc","Error allocating dynamic memory.");
        return(E_FAIL);
    }

    ptr = numbers;
    index = 0;
    for (i = 0; i < num_longs; i++)
    {
        *ptr = data[index];
        index += skipf;
        ptr++;
    }

    /* Place the required variables in the TAE variable block
       ---------------------------------------------------------*/
    if (q_intg(vblk, varname, num_longs, (int *)numbers, P_UPDATE) != SUCCESS)
    {
        c_lasmsg(TRUE,"fstfmtin-taevar","Error passing %s to TAE",varname);
        return(E_FAIL);
    }

    *result = *numbers;
    free(numbers);
    return(E_SUCC);
}
