/*****************************************************************************
NAME: GETPAR

PURPOSE: Gets user-specified parameters and tae globals for merge_hdr_bands.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  03/00  G.Schmidt        EDC   Original development
         10/01  A.Weissenberger  EDC   CR 5057. Formatting and removal of
                                       redundant c_lasmsg calls.
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:

*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "fhdr.h"
#include "fstfmtin.h"

void mhbgetpar (
    struct PARBLK *vblk,       /* I: pointer to initial TAE parameter block */
    int *nhdrs,                /* O: number of headers in the fhdrs array   */
    char hdrs[][CMLEN],        /* O: array of fhdr file names to merge
                                  (w/o the header extensions)            */
    char hdr_extension[CMLEN]) /* O: header extension                       */
{
    struct VARIABLE *var;    /* ptr to a variable in the vblk     */
    int count;               /* count of the number of headers    */
    int i;                   /* looping variable                  */

    /* Get the number of headers to be merged
       -----------------------------------------*/
    var = p_find (vblk, "NHDRS");
    *nhdrs = IVAL (*var, 0);

    /* Get the header filenames
       ---------------------------*/
    var = p_find (vblk, "HDRS");
    count = var->v_count;
    if (count != *nhdrs)
    {
        c_lasmsg (FALSE, "fstfmtin-nhdrs",
                  "Error: The number of headers in HDRS does not match the NHDRS value");
        c_lasmsg (TRUE, "fstfmtin-fatal", "Fatal error encountered");
    }

    for (i = 0; i < count; i++)
        strcpy (hdrs[i], SVAL (*var, i));

    /* Get the header extension
       ---------------------------*/
    var = p_find (vblk, "HDR_EXT");
    strcpy (hdr_extension, SVAL (*var, 0));
}
