/*****************************************************************************
NAME: GETPAR

PURPOSE: gets the user-specified parameters and tae globals for read_header

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  05/99  G.Schmidt        EDC   Updated to support Fast-L7A.
    7.3  10/00  A.Shimitz        EDC   Added retrieval of CDPATH parameter,
                                       which is set up by LOCATE_CD_HDR, for
                                       use by read_header.c to reopen the CD
                                       (CR #3519)
         10/01  A.Weissenberger  EDC   CR 5057 Formatting and removal of
                                       redundant c_lasmsg calls.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:

*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "fhdr.h"
#include "fstfmtin.h"

void rhgetpar (
    struct PARBLK *vblk, /* O: pointer to initial TAE parameter block        */
    char oprmsg[],       /* I: message for the operator when requesting tape */
    char drive[],        /* I: tape drive unit assigned                      */
    char opr_drive[],    /* I: tape drive unit assigned                      */
    long *mounted,       /* I: flag indicating if the tape is mounted or not */
    char *outfhdr,       /* I: fhdr file name                                */
    long *mediaflag,     /* I: flag indicating if the image is on disk or CD  
                              rather than on tape                            */
    char hpath[CMLEN],   /* I: host path to image data on disk or just the
                              image basename if on CD                        */
    long *resolution,    /* I: L7 resolution type (VISIBLE, THERMAL, PAN)    */
    long *same_res,      /* I: are bands of the same resolution?             */
    char cdpath[])       /* I: path to the header file on CD                 */

{
    struct VARIABLE *var;    /* ptr to a variable in the vblk                */

    /* Retrieve the tape mounted flag
       ---------------------------------*/
    var = p_find(vblk, "MOUNTED");
    *mounted = IVAL(*var, 0);

    /* Get the operator message
       ---------------------------*/
    var = p_find(vblk, "OPERATOR_MSG");
    strcpy(oprmsg, SVAL(*var, 0));

    if (*mounted == TRUE)
    {
        /* Get the drive name
           ---------------------*/
        var = p_find(vblk, "DRIVE");
        strcpy(drive, SVAL(*var, 0));

        /* Get the operator drive name
           ------------------------------*/
        var = p_find(vblk, "OPR_DRIVE");
        strcpy(opr_drive, SVAL(*var, 0));

        /* Get the path to header file on CD
           ------------------------------------*/
        var = p_find(vblk, "CDPATH");
        strcpy(cdpath, SVAL(*var, 0));
    }

    /* Get the header name
       ----------------------*/
    var = p_find(vblk, "OUTHDR");
    strcpy(outfhdr, SVAL(*var, 0));

    /* Get the mediaflag
       -----------------*/
    var = p_find(vblk, "MEDIAFLAG");
    *mediaflag = IVAL(*var, 0);

    /* Get the hpath
       ----------------*/
    var = p_find(vblk, "HPATH");
    strcpy(hpath, SVAL(*var, 0));

    /* Get the L7 resolution type
       -----------------------------*/
    var = p_find (vblk, "RESOLUTION");
    *resolution = IVAL (*var, 0);

    /* Get the same_res flag
       ------------------------*/
    var = p_find (vblk, "SAME_RES");
    *same_res = IVAL (*var, 0);
}
