/*****************************************************************************

NAME: SRCH_VOL

PURPOSE: To search the specified filenames and return an array of filenames
         that matches the search criteria.  Return the filenames without the
         ".img" extension.  Also return an array with the ".img" extension.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
    7.2  06/99  G.Schmidt        EDC   Initial development
         10/01  A.Weissenberger  EDC   CR 5057 Formatting and removal of
                                       redundant c_lasmsg calls
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES: The calling fstfmtin.pdf routine appends the .img, .ddr, and .his
       extensions to the filenames returned from this routine.  Thus the
       .img extension must be removed from the filenames that are returned.

*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "tapeio.h"
#include "fhdr.h"
#include "util.h"
#include "fstfmtin.h"

void main () 
{
    struct PARBLK vblk;             /* TAE parameter block                   */
    long std_out = 0;               /* standard out                          */
    char srchfile[CMLEN];           /* string to search for                  */
    char found_files[15][CMLEN];    /* array of filenames that matched the
                                       search there will be at most 15
                                       different volumes*/
    char *packed_files=NULL;   /* array to store the found_files all packed
                                  into one array.  each filename may be 80
                                  characters long and there can be up to 15
                                  filenames                                 */
    long nfiles;               /* number of files found in the search       */
    int i, j;                  /* loop variable                             */
    long tmp_long;             /* temp long value                           */
    char *ptr;                 /* temp pointer to point to chars in the
                                  filenames                                 */

    c_inlas (&vblk, &std_out);

    /* Get the variables from the TAE variable block
       ------------------------------------------------*/
    svgetpar (&vblk, srchfile);

    /* This search file is in the TAE form so change it to UNIX form
       ----------------------------------------------------------------*/
    for (i = 0; i < strlen(srchfile); i++)
    {
        if (srchfile[i] == '[' || srchfile[i] == ']' || srchfile[i] == '.')
            srchfile[i] = '/';
        if (srchfile[i] == ';')
            srchfile[i] = '.';
    }

    /* Get the filenames that match the srchfile string
       ---------------------------------------------------*/
    if (c_getfiles (srchfile, found_files, &nfiles, 15, 1, 0) != E_SUCC)
    {
        c_lasmsg (TRUE, "fstfmtin-c_getfiles", "Error searching for header "
                  "file.");
    }
    

    /* Allocate space for the packed_files array.  Allocate 81 chars for
     * each file found.
     --------------------------------------------------------------------*/
    packed_files = malloc (81 * nfiles * sizeof (char));
    if (packed_files == NULL)
    {
        c_lasmsg(TRUE,"fstfmtin-alloc","Error allocating memory");
    }

    /* The found files are in UNIX form and need to be passed back in
     * TAE form.  Also remove the .img extension from each file.
     -----------------------------------------------------------------*/
    for (i = 0; i < nfiles; i++)
    {
        /* Find the last '.' (for the extension) and cut the rest of the name
           -----------------------------------------------------------------*/
        ptr = (char *) strrchr (found_files[i], '.');
        if (ptr != NULL)
            ptr[0] = '\0';

        /* Change the first '/' to '['
           ------------------------------*/
        ptr = (char *) strchr (found_files[i], '/');
        if (ptr != NULL)
            ptr[0] = '[';

        /* Change the last '/' to ']'
           -----------------------------*/
        ptr = (char *) strrchr (found_files[i], '/');
        if (ptr != NULL)
            ptr[0] = ']';

        /* Loop through the filename changing all leftover '/'s to '.'s
           ---------------------------------------------------------------*/
        for (j = 0; j < strlen(found_files[i]); j++)
            if (found_files[i][j] == '/')
                found_files[i][j] = '.';
    }
    
    /* Pass the number of files back to TAE
       ---------------------------------------*/
    if (E_SUCC != lng2tae(&vblk,&nfiles,1,1,"NVOL_FILES",&tmp_long))
    {
        free (packed_files);
        packed_files = NULL;
        c_lasmsg(TRUE,"fstfmtin-lng2tae","Error outputting NVOL_FILES");
    }

    /* Pack all the filenames into one big string, putting enough blank
     * spaces between them to make 80 characters.
     -----------------------------------------------------------------------*/
    strcpy (packed_files, found_files[0]);
    for (i = 1; i < nfiles; i++)
    {
        for (j = 0; j < (80 - strlen(found_files[i-1])); j++)
            strcat (packed_files, " ");
        strcat (packed_files, found_files[i]);
    }

    /* Pass the found filenames back to TAE
       ---------------------------------------*/
    if (E_SUCC != str2tae(&vblk,packed_files,80,"%80s",nfiles,"VOL_FILES"))
    {
        free (packed_files);
        packed_files = NULL;
        c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting VOL_FILES");
    }

    /* Pack all the filenames with the ";img" extension into one big string,
     * putting enough blank spaces between them to make 80 characters.
     -----------------------------------------------------------------------*/
    strcpy (packed_files, found_files[0]);
    strcat (packed_files, ";img");
    for (i = 1; i < nfiles; i++)
    {
        for (j = 0; j < (80 - strlen(found_files[i-1])); j++)
            strcat (packed_files, " ");
        strcat (packed_files, found_files[i]);
        strcat (packed_files, ";img");
    }

    /* Pass the found filenames back to TAE
       ---------------------------------------*/
    if (E_SUCC != str2tae(&vblk,packed_files,80,"%80s",nfiles,"IMGVOL_FILES"))
    {
        free (packed_files);
        packed_files = NULL;
        c_lasmsg(TRUE,"fstfmtin-str2tae","Error outputting IMGVOL_FILES");
    }

    if (q_out(&vblk) != SUCCESS)
    {
        free (packed_files);
        packed_files = NULL;
        c_lasmsg(TRUE,"fstfmtin-qout","Error passing variables to TAE");
    }

    free (packed_files);
    packed_files = NULL;
}
