/*****************************************************************************
NAME: str2tae

PURPOSE: Send a C program string variable to the TAE variable block.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ---------------------------------------
         10/01  A.Weissenberger  EDC   CR 5057 Formatting and removal of
                                       redundant calls to c_lasmsg
                                       
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "fstfmtin.h"

int str2tae (
    struct PARBLK *vblk, /* I: pointer to TAE parameter block              */
    char buffer[],       /* I: an array of characters                      */
    long length,         /* I: length of string to place in vblk.  NO '\0' */
    char format[],       /* I: format of the string                        */
    long num_strings,    /* I: # of strings to place in the vblk           */
    char varname[])      /* I: name of the variable in the vblk            */

{
    char *bigstr;        /* array holding all strings in consecutive order */
    char **bigptr;       /* array of ptrs to the start of each string in
                            bigstr*/
    long big_index;      /* bigstr index                                   */
    long index;          /* index to buffer                                */

    long not_done;       /* loop control                                   */
    long i;              /* loop control                                   */
    long status;         /* integer return status                          */

    /* Allocate an array of characters large enough to hold all of the
       strings consecutively with the '\0' character added to each.
       ------------------------------------------------------------------*/
    bigstr = (char *)malloc(num_strings * (length + 1) * sizeof(char));
    if (bigstr == NULL)
    {
        c_lasmsg(FALSE,"fstfmtin-alloc","Error allocating dynamic memory");
        return(E_FAIL);
    }

    /* Allocate an array of pointers which will point to
       the beginning of each individual string in bigstr.
       -----------------------------------------------------*/
    bigptr = (char **)malloc(num_strings * sizeof(char *));
    if (bigptr == NULL)
    {
        c_lasmsg(FALSE,"fstfmtin-alloc","Error allocating dynamic memory");
        free(bigstr);
        return(E_FAIL);
    }

    /* Copy each of the strings from buffer into bigstr
       ---------------------------------------------------*/
    big_index = 0;
    index = 0;

    for (i = 0; i < num_strings; i++)
    {
        status = sscanf( &buffer[index], format, &bigstr[big_index]);
        if (status == EOF)
        {
            c_lasmsg(FALSE,"fstfmtin-sscanf",
                     "Error retrieving %s from header file", varname);
            free(bigstr);
            free(bigptr);
            return(E_FAIL);
        }
        bigptr[i] = &bigstr[big_index];
        bigstr[big_index + length] = '\0';
        index += length;
        big_index += (length + 1);
    }

    /* When picking up the bands that are present we always pick up 9
       characters.  We do not want to return all 9 characters though.  All we
       want to return are the characters that indicate valid bands
       (1, 2, 3, 4, 5, 6, 7, 8, 9).
       ---------------------------------------------------------------------*/
    if (strcmp(varname, "BANDS_PRESENT") == 0)
    {
        not_done = TRUE;
        i = 0;
        while ((not_done) && (i < num_strings))
        {
            if (*bigptr[i] != ' ')
                i++;
            else
                not_done = FALSE;
        }
        num_strings = i;
    }

    /* Place the strings in the variable block
       ----------------------------------------------------------*/
    if (q_string(vblk, varname, num_strings, bigptr, P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FALSE,"fstfmtin-taevar","Error passing %s to TAE",varname);
        free(bigstr);
        free(bigptr);
        return(E_FAIL);
    }

    free(bigstr);
    free(bigptr);
    return(E_SUCC);
}
