/*******************************************************************************
FUNCTION:   c_open_header   

PURPOSE:    To read or write the header file if it exits.   

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "labserv.h"
#include "fhdr.h"
#include "header.h"
#include "protos.h"

int open_header
(
    char *buf,        /* header buffer */
    char *name,       /* image name */
    long *flag,       /* flag to determine read or write acc */
    long *exist_flag, /* flag if header exists or not */
    char *msgkey      /* key for who called me, used to create error messages */
)
{
    FILE *fp;            /* file pointer */

    char tempkey[30];    /* used to create temporary keys for error messages */
    long status;         /* return status */
    char hostheader[CMLEN];  /* header name */
    long mode = 0;       /* flag value for lsstat */

    if (*flag == IREAD)
    {
        c_lsmknm(name,"fhdr",hostheader); /* make associated header file name */
        status = c_lsstat(hostheader,&mode);/* make sure header file exists  */
        if (status != E_SUCC)
        {
            *exist_flag = FALSE;
            c_lasmsg(FALSE,"informational","FHDR file does not exist on disk");
            c_lasmsg(FALSE,"informational","Creating a template FHDR");
            template_header(buf);
            return(E_SUCC);
        }
        else
            *exist_flag = TRUE;

        if ((fp = (fopen(hostheader,"r"))) == NULL)
        {
            strcpy(tempkey,msgkey);
            strcat(tempkey,"-open");
            c_lasmsg(FALSE,tempkey,"Error opening header");
            return(E_FAIL);
        }
        status = (long)fread(buf,1,HEADER_SIZE,fp);
        fclose(fp);
        if (status != HEADER_SIZE)
        {
            strcpy(tempkey,msgkey);
            strcat(tempkey,"-read");
            c_lasmsg(FALSE,tempkey,"Error reading header");
            return(E_FAIL);
        }
    }
    else if (*flag == IWRITE)
    {
        c_lsmknm(name,"fhdr",hostheader);/* make associated header file name */
        if ((fp = (fopen(hostheader,"w")))== NULL)
        {
            strcpy(tempkey,msgkey);
            strcat(tempkey,"-open");
            c_lasmsg(FALSE,tempkey,"Error opening header");
            return(E_FAIL);
        }
        status = (long)fwrite(buf,1,HEADER_SIZE,fp);
        fclose(fp);
        if (status != HEADER_SIZE)
        {
            strcpy(tempkey,msgkey);
            strcat(tempkey,"-write");
            c_lasmsg(FALSE,tempkey,"Error writing header");
            return(E_FAIL);
        }
    }
    return(E_SUCC);
}

