/*******************************************************************************
FUNCTION:   fill_header 

PURPOSE:    Updates a header file to be placed on disk.  Necessary 
        information is taken from the ddr.  

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "siat.h"
#include "tipsp.h"
#include "edips.h"
#include "geompak.h"
#include "util.h"
#include "header.h"
#include "fhdr.h"
#include "protos.h"

SETPROJKN (projkn);           /* Table of projection keys and names */
SETMONTHDEF (monthdef);       /* Month name-number table */

char thekey[40]; /*GLOBAL USED TO CREATE MSGKEY's*/
char *makekey
(
    char *thiskey,
    char *subkey
)
{
    strcpy(thekey, thiskey);
    strcat(thekey, "-");
    strcat(thekey, subkey);
    return (thekey);
}

int fill_header
(
    char *header,           /* header buffer                        */
    char hosin[][CMLEN],    /* input image name                     */
    char aftype[],          /* asoociate file type                  */
    char key[]              /* msgkey from who called me            */
)
{
    struct DDR ddr;         /* structure containe DDR information   */

    long annot = TRUE;      /* annotation flag                      */
    long head = TRUE;       /* header flag                          */
    long i;                 /* counter in for loop                  */
    long inunits;           /* input projection units               */
    long monthndx;          /* month index                          */
    long outsys;            /* output system                        */
    long outunits;          /* output projection units              */
    long proj_code = 0;     /* set projection code = 0 , "GEO"      */
    long proj_number = 0;   /* set projection code = 0 , "GEO"      */
    long projndx;           /* Projection index                     */
    long prtprm[2];         /* flags for c_transinit                */
    long status;            /* return status                        */
    long sflag = SETN;      /* Flag for SYSSET routine              */
    long temp;              /* temporary variable                   */
    double temp1;           /* double temporary variable            */
    long yr;                /* year */
    long year;              /* year value to convert, added for y2k */

    char buf[20];           /* temporary work buffer                */
    char tempbuf[20];       /* temporary work buffer                */
    char syst[DDSYLN];      /* system character string              */

    struct SIAT siat;       /* SIAT sturcture declaration           */
    struct TIPSP tipsp;     /* TIPSP structure declaration          */
    struct EDIPSH edipsh;   /* EDIPS header structure               */
    struct EDIPSA edipsa;   /* EDIPSA header structure              */
    struct EDPSAC edpsac;   /* EDPSAC header structure              */
    struct EDPSAC_EPH anceph;   /* EDIPS ancillary structure--Ephemeris Data */
    struct EDPSAC_ATT ancatt;   /* EDIPS ancillary structure--Attitude Data  */
    struct EDPSAC_RS  ancutm;   /* EDIPS ancillary structure--Horizontal and 
                                   Vertical resampling for UTM/PS data       */
    struct EDPSAC_RS  ancsom;   /* EDIPS ancillary structure--Horizontal and
                                   Vertical resampling for SOM/HOM data       */



    double lat;         /* latitude variable            */
    double longg;       /* longitude variable           */
    double pixsize;     /* pixel size                   */
    double x1;          /* temporary corner value       */
    double x2;          /* temporary corner value       */

    char **bigptr;          /* pointer to the array of projection parms */
    struct DATUMDEF dtm_info;      /* Datum code */

    /* get ddr*/

    c_getddr(hosin[0],&ddr);

    /* update projection code in header file
       --------------------------------------*/
    if (ddr.valid[DDPCV] == 0)
        c_lasmsg(TRUE,makekey(key,"invalid"),"Invalid projection field in DDR");


    if (ddr.proj_code == 22)
    {
        temp = 21;
        if ((status =c_fhdrup(header,45,(char *)&temp)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    }
    else if ((status =c_fhdrup(header,45,(char *)&ddr.proj_code)) == E_FAIL)
        c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

    projndx = 0;
    while((projndx<PROJCOUNT)&&(ddr.proj_code!=projkn[projndx].projnum))
        projndx++;
    if (projndx!=PROJCOUNT)
        status = c_fhdrup(header,43,projkn[projndx].projname);
    else /*NOT FOUND IN TABLE*/
        status = c_fhdrup(header,43,"GK");

    if (status == E_FAIL)
        c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

    /* update zone code in header file
       --------------------------------*/
    if (ddr.valid[DDZCV] ==0)
        c_lasmsg(TRUE,makekey(key,"invalid"),"Invalid zone field in DDR");
    else if ((status = c_fhdrup(header,47,(char *)&ddr.zone_code)) == E_FAIL)
        c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

    /* update projection parameters in header file
       --------------------------------------------*/
    if (ddr.valid[DDPPV] == 0)
        c_lasmsg(TRUE,makekey(key,"invalid"),
                "Invalid projection parameter fields in DDR");
    else if ((status = c_fhdrup(header,49,(char *)ddr.proj_coef)) == E_FAIL)
        c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

    outunits = 4;
    prtprm[0] = 0;             /* print to terminal = FALSE           */
    prtprm[1] = 0;             /* print to file     = FALSE           */

    if ((inunits = c_string2unit(ddr.proj_units)) == E_FAIL)
    {
        c_lasmsg(FALSE, "fstfmtout-units", 
                "Projection units %s are not supported", ddr.proj_units);
    }

    /* If coordinates are square then "PRODUCT TYPE =MAP ORIENTED"
       -------------------------------------------------------------------*/
    if (ddr.valid[DDCCV] == 0)
        c_lasmsg(TRUE,makekey(key,"invalid"), "Invalid corner field in DDR");

    if(floor(ddr.upleft[0]) == floor(ddr.upright[0])
            && floor(ddr.loleft[0]) == floor(ddr.loright[0])
            && floor(ddr.upleft[1]) == floor(ddr.loleft[1])
            && floor(ddr.upright[1])== floor(ddr.loright[1]) )
    {
        /* Set PRODUCT TYPE to MAP ORIENTED */
        strcpy(buf, "MAP ORIENTED");
        if ((status =c_fhdrup(header,12,buf)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"), "Error updating header");

        /* Set ORIENTATION to 0.00 */
        temp1 = 0;
        if ((status =c_fhdrup(header,41,(char *)&temp1)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"), "Error updating header");
    }

    /* update corner values in header file
       ------------------------------------*/
    if (inunits == outunits)
    {
        if ((status = c_fhdrup(header,63,(char *)&ddr.upleft[1])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,65,(char *)&ddr.upleft[0])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,63,(char *)&ddr.upright[1])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,63,(char *)&ddr.upright[1])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,65,(char *)&ddr.loright[0])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,65,(char *)&ddr.loright[0])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,63,(char *)&ddr.loleft[1])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,65,(char *)&ddr.loleft[0])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    }
    else
    {
        c_transinit(&ddr.proj_code,&inunits,&ddr.zone_code,&ddr.datum_code,
                ddr.proj_coef,&proj_code,&outunits,&ddr.zone_code,
                &ddr.datum_code, ddr.proj_coef,prtprm,"\0");

        c_trans(&ddr.proj_code,&inunits,&proj_code,
                &outunits,&ddr.upleft[1],&ddr.upleft[0],
                &longg,&lat);

        if ((status = c_fhdrup(header,63,(char *)&longg)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,65,(char *)&lat)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

        c_trans(&ddr.proj_code,&inunits,&proj_code,
                &outunits,&ddr.upright[1],&ddr.upright[0],
                &longg,&lat);

        if ((status = c_fhdrup(header,71,(char *)&longg)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,73,(char *)&lat)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

        c_trans(&ddr.proj_code,&inunits,&proj_code,
                &outunits,&ddr.loright[1],&ddr.loright[0],
                &longg,&lat);

        if ((status = c_fhdrup(header,79,(char *)&longg)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,81,(char *)&lat)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

        c_trans(&ddr.proj_code,&inunits,&proj_code,
                &outunits,&ddr.loleft[1],&ddr.loleft[0],
                &longg,&lat);

        if ((status = c_fhdrup(header,87,(char *)&longg)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,89,(char *)&lat)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    }

    /* if input units are meters update northing and eastings
       -------------------------------------------------------*/
    if (inunits == 2)
    {
        if ((status = c_fhdrup(header,67,(char *)&ddr.upleft[1])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,69,(char *)&ddr.upleft[0])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,75,(char *)&ddr.upright[1])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,77,(char *)&ddr.upright[0])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,83,(char *)&ddr.loright[1])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,85,(char *)&ddr.loright[0])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,91,(char *)&ddr.loleft[1])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        if ((status = c_fhdrup(header,93,(char *)&ddr.loleft[0])) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    }

    /* update pixel size in header file
       ---------------------------------*/
    if (inunits == 2)
    {
        if ((status = c_fhdrup(header,57,(char *)&ddr.pdist_y)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    }
    else
    {
        outunits = 2;
        c_transinit(&ddr.proj_code,&inunits,&ddr.zone_code,&ddr.datum_code,
                ddr.proj_coef,&proj_code,&outunits,&ddr.zone_code,
                &ddr.datum_code, ddr.proj_coef,prtprm,"\0");

        c_trans(&ddr.proj_code,&inunits,&proj_code,
                &outunits,&ddr.upleft[1],&ddr.upleft[0],
                &x1,&lat);

        c_trans(&ddr.proj_code,&inunits,&proj_code,
                &outunits,&ddr.upright[1],&ddr.upright[0],
                &x2,&lat);

        pixsize = (x1 - x2)/ddr.ns;
        if ((status = c_fhdrup(header,57,(char *)&pixsize)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    }

    /* put projection parms in proper DMS format
       ------------------------------------------*/
    bigptr = (char **)malloc(15  * sizeof(char *));
    if (bigptr == NULL)
    {
        c_lasmsg(TRUE,makekey(key,"alloc"),"Error allocating dynamic memory.");
        return(E_FAIL);
    }

    sscanf(&header[538],"%d",&proj_number);
    for (i = 0; i < 15; i++)
        bigptr[i] = &header[594 + (i * 24)];
    local_convert_dms(bigptr, proj_number,key);

    free (bigptr);

    /* update datum code values
       -------------------------*/


    if (ddr.valid[DDDCV] == 0)
        c_lasmsg(TRUE,makekey(key,"invalid"),"Invalid datum field in DDR");
    dtm_info.datumnum=ddr.datum_code;
    if (c_getdatum((&dtm_info)) != E_SUCC)
        c_lasmsg(TRUE,makekey(key,"invalid"),"Error getting Datum Info");
    if(dtm_info.datumnum > MINDTM)
    {
        if (status = c_fhdrup(header,51,dtm_info.datumname) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    }
    else
    {
        if ((status = c_fhdrup(header,51,dtm_info.spherename)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    }
    if ((status = c_fhdrup(header,53,(char *)&dtm_info.smajor)) == E_FAIL)
        c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
    if ((status = c_fhdrup(header,55,(char *)&dtm_info.sminor)) == E_FAIL)
        c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

    /* use associated files
       ---------------------*/
    if (strcmp(aftype,"CCTX") == 0)
    {
        if ((status = c_read_siat(&siat,hosin[0])) != E_FAIL)   
        {

            /* update satellite number
               ------------------------*/
            sprintf(buf,"L%d      ",siat.processing_mission_number);
            if ((status = c_fhdrup(header,8,buf)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            /* Update acquisition date
               ------------------------*/
            monthndx = 0;
            while((monthndx<MONTHCOUNT)
                    &&(strcmp(monthdef[monthndx].monthname,
                    siat.annotation_month)))
                monthndx++;

            strcpy(tempbuf,monthdef[monthndx].monthnum);


            sscanf(siat.annotation_year,"%d",&year);
            yr = c_4digityr (year);                 /* add c_4digityr for y2k */
            sprintf(buf,"%4d     ",yr);
            sprintf(&(buf[4]),"%2s   ",tempbuf);
            sprintf(&(buf[6]),"%2s   ",siat.annotation_day);
            if ((status = c_fhdrup(header,6,buf)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            /* update center lattitude and longitude
               --------------------------------------*/
            buf[0] = siat.annotation_longitude[1];
            buf[1] = siat.annotation_longitude[2];
            buf[2] = siat.annotation_longitude[3];
            sscanf((char *)&(siat.annotation_longitude[5]),"%d",&temp);
            sprintf(&buf[3],"0000.0000");
            sscanf(buf,"%lf",&temp1);
            temp1 = temp1/10000;
            temp1 += (double)(temp)/60.0;
            if (siat.annotation_longitude[0] == 'W')
                temp1 = -temp1;
            if ((status = c_fhdrup(header,105,(char *)&temp1)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            buf[0] = siat.annotation_latitude[1];
            buf[1] = siat.annotation_latitude[2];
            sscanf((char *)&(siat.annotation_latitude[4]),"%d",&temp);
            sprintf(&buf[2],"0000.0000");
            sscanf(buf,"%lf",&temp1);
            temp1 = temp1/10000;
            temp1 += (double)temp/60.0;
            if (siat.annotation_latitude[0] == 'S')
                temp1 = -temp1;
            if ((status = c_fhdrup(header,107,(char *)&temp1)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");


            /* update the sun elevation and azimuth
               -------------------------------------*/
            sscanf(&(siat.annotation_sun_elevation[6]),"%d",&temp);
            if ((status = c_fhdrup(header,101,(char *)&temp)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
            sscanf(&(siat.annotation_sun_azimuth[3]),"%d",&temp);
            if ((status = c_fhdrup(header,103,(char *)&temp)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        }
    }
    else if (strcmp(aftype,"TIPSP") == 0)
    {
        if ((status = c_read_head(&tipsp,hosin[0])) == E_FAIL)
            head = FALSE;
        if ((status = c_read_annot(&tipsp,hosin[0])) == E_FAIL)
            annot = FALSE;

        if (head)
        {
            /* update center coordinates
               --------------------------*/
            temp1 = tipsp.head_center_latitude;
            if ((status = c_fhdrup(header,107,(char *)&temp1)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
            temp1 = tipsp.head_center_longitude;
            if ((status = c_fhdrup(header,105,(char *)&temp1)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            temp = (long)tipsp.head_center_line;
            if ((status = c_fhdrup(header,113,(char *)&temp)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
            temp =  (long)tipsp.head_center_sample;
            if ((status = c_fhdrup(header,112,(char *)&temp)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            /* update satellite number
               ------------------------*/
            tempbuf[0] = 'L';
            tempbuf[1] = tipsp.head_mission_id[9];
            if ((status = c_fhdrup(header,8,tempbuf)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            /* update the sun elevation and azimuth
               -------------------------------------*/
            temp = (long)tipsp.map_sun_elevation;
            if ((status = c_fhdrup(header,101,(char *)&temp)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
            temp = (long)tipsp.map_sun_azimuth;
            if ((status = c_fhdrup(header,103,(char *)&temp)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        }
        if (annot)
        {
            /* update acquistion date
               -----------------------*/

            tempbuf[0] = tipsp.annot_acquistion_date[2];
            tempbuf[1] = tipsp.annot_acquistion_date[3];
            tempbuf[2] = tipsp.annot_acquistion_date[4];
            tempbuf[3] = '\0';

            monthndx = 0;
            while((monthndx<MONTHCOUNT)
                    &&(strcmp(monthdef[monthndx].monthname,tempbuf)))
                monthndx++;

            strcpy(tempbuf,monthdef[monthndx].monthnum);

            sscanf(&(tipsp.annot_acquistion_date[5]),"%d",&year);
            yr = c_4digityr (year);                 /* add c_4digityr for y2k */
            sprintf(buf,"%4d   ",yr);
            sprintf(&(buf[4]),"%2s   ",tempbuf);
            sprintf(&(buf[6]),"%2s   ",tipsp.annot_acquistion_date);
            if ((status = c_fhdrup(header,6,buf)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            /* update WRS path/row
               --------------------*/
            strcpy(buf,&(tipsp.annot_wrs_path[1]));
            buf[3] = '/';
            buf[7] = '0';
            buf[8] = '0';
            if ((status = c_fhdrup(header,4,buf)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        }
    }
    else if (strcmp(aftype,"EDIPS") == 0)
    { 
        /* get current system code
           ----------------------*/
        status = c_getsys(syst);
        if (status != E_SUCC)
        {
            c_lasmsg(FALSE,"dspedips-system","Error retreiving current system");
            return(E_FAIL);
        }

        status = c_sysset(syst, &outsys, &sflag);
        if (status != E_SUCC)
        {
            c_lasmsg(FALSE,"dspedips-system","Error retreiving current system");
            return(E_FAIL);
        }

        strcpy(buf,"MSS");
        if ((status = c_fhdrup(header,10,buf)) == E_FAIL)
            c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

        if (( status = c_edips_hdr(hosin[0], outsys, &edipsh)) == E_SUCC)
        {
            /* update WRS path/row
               --------------------*/
            buf[0] = edipsh.wrs_designator[2];
            buf[1] = edipsh.wrs_designator[3];
            buf[2] = edipsh.wrs_designator[4];
            buf[3] = '/';
            buf[4] = edipsh.wrs_designator[5];
            buf[5] = edipsh.wrs_designator[6];
            buf[6] = edipsh.wrs_designator[7];
            buf[7] = '0';
            buf[8] = '0';
            if ((status = c_fhdrup(header,4,buf)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            /* update satellite number
               ------------------------*/
            tempbuf[0] = 'L';
            sprintf(&(tempbuf[1]),"%d",edipsh.mission_number);
            if ((status = c_fhdrup(header,8,tempbuf)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        }

        if ((status = c_edips_ann(hosin[0],&edipsa)) == E_SUCC)
        {
            /* update the sun elevation and azimuth
               -------------------------------------*/
            sscanf(&(edipsa.sun_angles[6]),"%d",&temp);
            if ((status = c_fhdrup(header,101,(char *)&temp)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
            sscanf(&(edipsa.sun_angles[10]),"%d",&temp);
            if ((status = c_fhdrup(header,103,(char *)&temp)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");

            /* update acquistion date
               -----------------------*/
            tempbuf[0] = edipsa.exposure_day_mon_yr[2];
            tempbuf[1] = edipsa.exposure_day_mon_yr[3];
            tempbuf[2] = edipsa.exposure_day_mon_yr[4];
            tempbuf[3] = '\0';

            monthndx = 0;
            while((monthndx<MONTHCOUNT)
                    &&(strcmp(monthdef[monthndx].monthname,tempbuf)))
                monthndx++;

            strcpy(tempbuf,monthdef[monthndx].monthnum);

            sscanf(&(edipsa.exposure_day_mon_yr[5]),"%d",&year);
            yr = c_4digityr (year);     /* add c_4digityr for y2k */
            sprintf(buf,"%4d   ",yr);
            sprintf(&(buf[4]),"%2s   ",tempbuf);
            sprintf(&(buf[6]),"%2s   ",edipsa.exposure_day_mon_yr);
            if ((status = c_fhdrup(header,6,buf)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        }

        if ((status = c_edips_anc(hosin[0], outsys, &edpsac, &anceph, &ancatt,
                &ancutm, &ancsom)) == E_SUCC)
        {
            /* Update scene center lat long
               -----------------------------*/
            temp1 = edpsac.scene_ctr_lat;
            temp1 = temp1 * 360 /(2 * 3.14159265);
            if ((status = c_fhdrup(header,107,(char *)&temp1)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
            temp1 = edpsac.scene_ctr_long;
            temp1 = temp1 * 360 /(2 * 3.14159265);
            if ((status = c_fhdrup(header,105,(char *)&temp1)) == E_FAIL)
                c_lasmsg(TRUE,makekey(key,"update"),"Error updating header");
        }
    } 

    return (E_SUCC);
}


