/******************************************************************************
FUNCTION:   FSTFMTOUT

PURPOSE:    Copies a LAS image to a fast format B magnetic tape.

PROJECT:    LAS

ALGORITHM REFERENCES:   none.

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "fhdr.h"
#include "protos.h"

#define MAX_VOL       15        /* Maximum number of tape volumes            */
                                /* current tape.                             */
/* Global variable declarations
-------------------------------*/
struct PARBLK vblock;           /* TAE parameter block                        */
char  comment[MAX_VOL][CMLEN];  /* Tape mount message to the operator         */
char  density[5];               /* Tape density entered by the user           */
char  *drive[1];                /* Physical tape drive name                   */
char  label_file[CMLEN];        /* File containing tape label info (no ext)   */
struct FDESC *fdesc[MAXIMG+1];  /* Image file descriptors used for image I/O  */
extern long  curcat;            /* Current output band being processed        */
extern long  curent;            /* Current output line being processed        */
long  curvol;                   /* Current tape volume being process          */
long  datasize[NDTYPES] = {0,1,2,4,4,8};/* # of bytes used for each data type */

int main (void)
{
    static long bands[MAXIMG][MAXBND+1];/* list of input bands to be copied   */
    static long window[MAXIMG][4];      /* input window specification         */
    static long nbands[MAXIMG];         /* # bands in each input image        */

    long bands_per_tape;
    long blksiz;                        /* number of lines in output block    */
    long exist_flag;                    /* does header file exist             */
    long flag;                          /* flag to indicate if header file 
                                           exits                              */
    long header_count = HEADER_SIZE;    /* number of bytes in header          */
    long msgtim;                        /* elapsed time between display msg   */
    long mode = HANDLE;
    long nmrimg;                        /* number of input images             */
    long odtype = EBYTE;                /* data type of output image          */
    long one = 1;
    long totbnd;                        /* total number of bands to be copied */
    long unit;                          /* unit number of tape device         */
    long zero = 0;                      /* Value of zero                      */
    long append_flag;                   /* append file flag                   */

    char header[HEADER_SIZE];           /* header buffer                      */
    char taein[MAXIMG][CMLEN];          /* LAS input file names               */
    char hosin[MAXIMG][CMLEN];          /* Host input file names              */
    char out[1][CMLEN];                 /* Output files created               */
    char aftype[CMLEN];                 /* assocative file type               */
    char append_file[CMLEN];            /* append file name                   */

    /* Get the input parameters
       ---------------------------*/
    getpar(taein,hosin,window,bands,nbands,&totbnd,&blksiz,&nmrimg, density,
            &msgtim,aftype,append_file,&append_flag);

    /* Open the input image
       -----------------------*/
    open_image(hosin,&nmrimg,window,&odtype);

    /* Mount a tape or reopen the drive
       -----------------------------------*/
    mount_tape(&unit,TWRITE,FALSE);

    /* create/update header file
       --------------------------*/
    flag = IREAD;
    if (open_header(header,hosin[0],&flag,&exist_flag,"fstfmtout") == E_FAIL)
        c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
    if (!exist_flag)
        fill_header(header,hosin,aftype,"fstfmtout");
    calculate_nbands(&unit,window,&odtype,totbnd,&bands_per_tape);

    update_header(header,window[0][NL],window[0][NS],1,bands_per_tape,&blksiz);

    /* write the updated header file to disk and to tape
       --------------------------------------------------*/
    flag = IWRITE;
    open_header(header,hosin[0],&flag,&exist_flag,"fstfmtout");
    write_tape(&unit,(unsigned char *)header,&header_count,&zero);
    if (c_tpweof(&unit,&one,&mode) != TAPE_SUCC)
        c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");

    /* Copy the input data to tape in BSQ format
       ---------------------------------------------------*/
    cp_bsq(&unit,&nmrimg,window,bands,nbands,&totbnd,&odtype,&blksiz,
            &msgtim,&bands_per_tape,header,append_file,&append_flag);

    /* Close the tape device
       ------------------------*/
    close_tape(&unit,NOREW,1);

    /* Update the parameter block with the current tape volume,
       tape file number and the drive the tape was mounted on.
       -----------------------------------------------------------*/
    q_intg(&vblock,"CURVOL",1,(int*)&curvol,P_UPDATE);
    q_string(&vblock,"DRIVE",1,drive,P_UPDATE);
    q_out(&vblock);

    /* Standard completion message
       ------------------------------*/
    c_complt("FFMTOUT",out,&zero);

    return 0;
}

