/******************************************************************************
FUNCTION:   GETPAR


PURPOSE:    Retrieve the input parameters from the TAE variable block

ALGORITHM 

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "sysdef.h"
#include "protos.h"

#define MAX_VOL     15  /* maximum number of tape volumes            */

void getpar
(
    char taein[][CMLEN],    /* Input LAS file names               */
    char hosin[][CMLEN],    /* Input Host file names              */
    long window[][4],       /* input window specification         */
    long bands[][MAXBND+1], /* list of input bands to be copied   */
    long nbands[],          /* # bands in each input image        */
    long *totbnd,           /* total number of bands to be copied */
    long *blksiz,           /* number of lines in output block    */
    long *nmrimg,           /* number of input images             */
    char density[],         /* tape density in BPI                */
    long *msgtim,           /* elapsed time between display msg   */
    char aftype[],          /* associative file type              */
    char append_file[],     /* append file name                   */
    long *append_flag       /* flag to append data to end of images */
)
{
    extern struct PARBLK vblock;    /* TAE parameter block                   */
    extern long  curvol;            /* current tape volume being process     */
    extern char  comment[][CMLEN];  /* tape mount message to the operator    */
    extern char  *drive[1];         /* physical tape drive name              */
    extern char  label_file[];      /* File containing tape label info       */

    long acc = IREAD;           /* access to INFILE                       */
    long index;                 /* loop index                             */
    long status;                /* function return status code            */
    long std_out;               /* fortran standard out LUN               */
    long same;                  /* flag to check input image windows      */
    long vol;                   /* Volume continuation number             */
    long zero = 0;              /* Value of zero                          */
    long imglim = MAXIMG;       /* maximum number of input images         */

    static long qflag[MAXIMG][2];   /* flag to check for subwindow        */

    struct VARIABLE *val;       /* pointer to a TAE variable              */

    /* Initialize TAE and LAS
       -------------------------*/
    c_inlas(&vblock,&std_out);

    /* Retrieve tape mount message
       ------------------------------*/
    val = p_find(&vblock,"COMMENT");
    for (index = 0; index < val->v_count; index++)
        strcpy(comment[index],SVAL(*val,index));

    for (vol = 1, index = val->v_count; index < MAX_VOL; index++, vol++)
        sprintf(comment[index],"%s%s%d",comment[val->v_count - 1],
                " -- CONTINUATION ",vol);

    /* Retrieve tape density in BPI
       -------------------------------*/
    val = p_find(&vblock,"TAPEDENS");
    strcpy(density,SVAL(*val,0));

    /* Retrieve number of lines in output block
       -------------------------------------------*/
    val = p_find(&vblock,"BLKSIZ");
    *blksiz = IVAL(*val,0);

    /* Retrieve associative file type
       -------------------------------*/
    val = p_find(&vblock,"AFTYPE");
    if (val->v_count == 1)
        strcpy(aftype,SVAL(*val,0));

    /* Retrieve current tape volume
       -------------------------------*/
    val = p_find(&vblock,"CURVOL");
    curvol = IVAL(*val,0);

    /* Retrieve physical tape drive name
       ------------------------------------*/
    val = p_find(&vblock,"DRIVE");
    drive[0] = SVAL(*val,0);

    /* Retrieve number of seconds between processing messages
       ---------------------------------------------------------*/
    val = p_find(&vblock,"$msgtim");
    *msgtim = IVAL(*val,0);

    /* Retrieve input file
       --------------------*/
    val = p_find(&vblock,"INFILE");
    if (val->v_count == 0)
        *append_flag = FALSE;
    else
    {
        status = c_getnam(&vblock,"INFILE",&zero,append_file,"txt",&acc);
        if (status != E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
        *append_flag = TRUE;
    }

    /* Retrieve the input image specifications including
       LAS image name(s), window and bands
       ----------------------------------------------------*/
    status = c_getin(&vblock,"IN",&zero,taein,hosin,nmrimg,window,bands,nbands,
            totbnd,&imglim,&same,qflag);
    if (status != E_SUCC)
        c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");

    if (*totbnd > 7)
        c_lasmsg(TRUE,"fstfmtout-bands",
                 "Number of bands specified greater than 7");

    /* Check to make sure that multiple window specified are all the same size
       ----------------------------------------------------------------------*/
    if (same == 0)
        c_lasmsg(TRUE,"fstfmtout-wind",
                 "Input images do not have same window sizes");

    /* Retrieve the name of the file that contains the tape label information
       ---------------------------------------------------------------------*/
    val = p_find(&vblock,"PARFILE");
    if (val->v_count == 0)
        strcpy(label_file, "");
    else
        strcpy(label_file, SVAL(*val,0));

}

