/******************************************************************************
FUNCTION:   OPEN_IMAGE

PURPOSE:    Opens the image.  If the output band format is BIL, the LAS
        image is opened for sequential processing (process all bands
        at once).  If the output band format is BSQ, the LAS image
        is opened for random processing (process a single band at a
        time).

ALGORITHM 
  Open image for Sequential processing of bands
      Open each image file
      Allocate I/O buffer the first time
      Group the file descriptors
  Else Open image for Random access
      Open each image file

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "protos.h"

void open_image
(
    char hosin[][CMLEN],    /* input host name                                */
    long *nmrimg,           /* number of input images                         */
    long window[][4],       /* input window specification                     */
    long *odtype            /* output image data type                         */
)
{
    extern struct FDESC *fdesc[]; /* image file descriptors used for image I/O*/

    long image;         /* loop index for counting images                 */
    long status;        /* function return status code                    */
    long innb;          /* number of bands in input image                 */
    long zero = 0;      /* value of zero                                  */
    long one  = 1;      /* value of one                                   */
    long access = IREAD;/* image I/O access type of READ                  */

    /*
       Open each image file
     */
    for (image = 0; image < *nmrimg; image++)
    {
        innb = 0;
        status = c_eopenr(&fdesc[image],hosin[image],&innb,&window[image][0],
                &window[image][1],&window[image][2],&window[image][3],
                &access,odtype,&zero,&one);
        if (status != E_SUCC)
            c_lasmsg(TRUE,"fstfmtout-fatal","Fatal error encountered");
    }

}

