#ifndef PROTOS_H_
#define PROTOS_H_

void bksp_file
(
    long *unit          /* tape unit number                           */
);

void c_date_time 
(
   char *date_str,      /* The current date as retrieved from the system   */
   char *time_str       /* The current time as retrieved from the system   */
);

int open_header
(
    char *buf,        /* header buffer                        */
    char *name,       /* image name               */
    long *flag,       /* flag to determine read or write acc  */
    long *exist_flag, /* flag if header exists or not     */
    char *msgkey      /* key for who called me, used to create error messages */
);

void calculate_nbands
(
    long *unit,                 /* Tape unit number                           */
    long window[][4],           /* window for image               */
    long *odtype,       /* data type of output                    */
    long totbnd,                /* total bands to be written              */
    long *bands_per_tape   /* number of bands to be written to one tape  */
);

void local_convert_dms
(
    char **strings,       /* pointer to strings     */
    long proj_number,     /* USGS projection number */
    char *key			  /* key used in error messages */
);

void cp_bsq
(
    long *unit,             /* tape unit number                           */
    long *nmrimg,           /* number of input images                     */
    long window[][4],       /* input window (sl,ss,nl,ns)             */
    long bands[][MAXBND+1],     /* list of input bands to be copied           */
    long nbands[],          /* # bands in each input image                */
    long *totbnd,           /* total number of bands to be copied         */
    long *odtype,           /* output image data type                     */
    long *blksiz,           /* number of image lines in output block      */
    long *msgtim,           /* elapsed time between messages              */
    long *bands_tape,
    char *header,           /* header string                  */
    char append_file[],     /* the append file name               */
    long *append_flag       /* flag  TRUE if file is to be append to tape */
);

char *makekey
(
    char *thiskey,
    char *subkey
);

int fill_header
(
    char *header,
    char hosin[][CMLEN],
    char aftype[],
    char key[]
);

void getpar
(
    char taein[][CMLEN],        /* Input LAS file names               */
    char hosin[][CMLEN],        /* Input Host file names              */
    long window[][4],       /* input window specification             */
    long bands[][MAXBND+1],     /* list of input bands to be copied       */
    long nbands[],          /* # bands in each input image            */
    long *totbnd,           /* total number of bands to be copied         */
    long *blksiz,           /* number of lines in output block        */
    long *nmrimg,           /* number of input images             */
    char density[],                 /* tape density in BPI                        */
    long *msgtim,           /* elapsed time between display msg       */
    char aftype[],          /* associative file type              */
    char append_file[],     /* append file name               */
    long *append_flag      /* flag to append data to end of images       */
);

void open_image
(
    char hosin[][CMLEN],    /* input host name                    */
    long *nmrimg,       /* number of input images                         */
    long window[][4],   /* input window specification                     */
    long *odtype       /* output image data type                         */
);

void mount_tape
(
    long *unit,                 /* Tape unit number                           */
    long access,                /* Open tape for specified  access            */
    int flag                  /* Flag to disable display messages           */
);

void reopen_tape
(
    long *unit,                 /* tape unit number                           */
    long access                 /* open tape for read access                  */
);

int write_tape
(
    long *unit,          /* tape unit number                                   */
    unsigned char *buf,  /* buffer to be written to tape                       */
    long *out_byte,      /* number of bytes to be written                      */
    long *bckrec        /* number of records to back space to get back to the
                           beginning of the bands (BIL only)                  */
);

void close_tape
(
    long *unit,          /* tape unit number                                   */
    long rewopt,         /* rewind option                                      */
    long eofs           /* Number of EOF's to write to tape                   */
);

void update_header
(
    char *header,               /* header buffer                        */
    long nl,            /* number of lines          */
    long ns,            /* number of samples            */
    long start_band,        /* starting band number         */
    long bands_per_tape,    /* ending band number           */
    long *blksiz                /* blocking factor                      */
);

int print_label
(
   char parm_file[]     /* Name of the file that contains the label info     */
);

#endif
