#include "gaustran.h"

void filspec
(
    char hosin[][CMLEN], 		/* host name for input image          */
    char hosout[], 			/* host name for output image         */
    long num_inimages,			/* total number of input images       */
    long num_img_bands[][MAXBND+1],	/* band numbers for each input image  */
    long out_img_bands[][MAXBND+1],	/* band numbers for each output image */
    long out_bands,			/* number of output image bands       */
    long bands_per_img[],		/* number of bands for each image     */
    long ipoint[],
    long opoint[],
    long window[][4]			/* input image dimensions	      */
)

/*
*****************************************************************************

NAME:	 FILSPEC	

PURPOSE: Opens the input and output images. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Mar-85    B. Dealy
   2         16-Oct-89    K. Kunska          conversion to 5.0   
   6.1       25-May-94	  D. Eichacker		added opt[]
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open the input image file.
   Allocate buffer space for the input image files.
   Group the input image files.
   Open the output image file.
   Allocate buffer space for the output image file.
   Group the output image files.
   Update the ddr records.

*****************************************************************************
*/

{
char	*prog_name = "GAUSTRAN";	/* program name 		      */

static	long	opt[3]={0,0,0};		/* optional parameter array for EOPENS*/

long	access,				/* access type                        */
	bufsize,			/* buffer size                        */
	comb_flag,			/* combine flag for 1-band output img */
	i,				/* loop counter and array index       */
	n,				/* loop counter and array index       */
	nib,				/* band index                         */
	stat,				/* return status                      */
	type;				/* data type                          */

long one = 1;
long zero = 0;

struct FDESC * fdescin[MAXIMG+1];  /* input file descriptor  */
struct FDESC * fdescout[2];        /* output file descriptor */

double	line_inc,			/* line increment                     */
	samp_inc;			/* sample increment                   */
/*
Open the input image file.
---------------------------
*/
access = IREAD;
type = EREAL;
for (n = 0, nib = 0; n < num_inimages; n++)
   {
   stat = c_eopens (&fdescin[n], hosin[n], num_img_bands[n], &window[n][SL],
                    &window[n][SS], &window[n][NL], &window[n][NS], 
                    &ipoint[nib], &access, &type, &zero, opt);
   if (stat != E_SUCC)
      c_abort (&block, "Error opening the input image", "gaustran-inopen",
               prog_name, out_files, &one);
   nib += bands_per_img[n];
   }
fdescin[n] = 0;

/*
Allocate buffer space for the input image files.
-------------------------------------------------
*/
if ((inbuf = (float *) c_ealloc (fdescin, &bufsize)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "gaustran-alloc",
            prog_name, out_files, &one);

/* 
Group the input image files.
-----------------------------
*/
c_egroup (fdescin, &gdescin, inbuf, &bufsize);


/*
Open the output image file.
-----------------------------
*/
n = 0;
access = IWRITE;
for (i = 0; i < out_bands; i++)
   out_img_bands[0][i] = i + 1;

stat = c_eopens (&fdescout[n], hosout, &out_bands, &one, &one, &window[n][NL], 
                 &window[n][NS], opoint, &access, &type, &zero, opt);
if (stat != E_SUCC)
   c_abort (&block, "Error opening the output image", "gaustran-outopen",
            prog_name, out_files, &one);
fdescout[++n] = 0;

/*
Allocate buffer space for the output image file.
-------------------------------------------------
*/
if ((outbuf = (float *) c_ealloc (fdescout, &bufsize)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "gaustran-alloc",
            prog_name, out_files, &one);

/* 
Group the output image files.
--------------------------------
*/
c_egroup (fdescout, &gdescout, outbuf, &bufsize);

/*
Update the ddr records.
------------------------
*/
line_inc = 1;
samp_inc = 1;
comb_flag = NOCOMB;

c_upddr (&block, hosin, hosout, &num_inimages, window, num_img_bands,
         bands_per_img, &line_inc, &samp_inc, &out_bands, &comb_flag);

free (fdescin);
free (fdescout);
}

