#include "gaustran.h"

void getpar
(
    char taein[][CMLEN],		/* tae name of the input image              */
    char hosin[][CMLEN],		/* host name of the input image             */
    char taeout[][CMLEN], 		/* tae name of the output image             */
    char hosout[][CMLEN],		/* host name of the output image            */
    long *dtype,			/* data type of the input image              */
    long *bands_per_img, 	/* number of bands per input imgae           */
    long (*img_flag)[2],		/* specified image/band options              */
    long img_limit,		/* maximum number of images                  */
    long (*num_img_band)[MAXBND+1],	/* band numbers for each input image */
    long *num_inimg,		/* total number of input images              */
    long *same_wnd, 		/* same window size? 1=Y 2=N                 */
    long *tot_inbands,		/* total number of input bands               */
    float scalfact[],		/* scale factors for the result of          */
    float expsf[], 		/* scale factors for the exponent           */
    float offset[],  		/* offset to be subtracted from the input   */
    long window[][4]		/* input image dimensions		     */
)

/*
*****************************************************************************

NAME:	GETPAR

PURPOSE: Retrieves the user supplied parameters from TAE.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         18-Mar-85    B. Dealy
   2         16-Oct-89    K. Kunska          conversion to 5.0    
   3         02-Mar-90    K. Kunska          fixed an informational msg display

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   
   Initialize tae.
   Parse the input image specification. 
   If the return status is not success then 
      Abort with a fatal error message.
   Determine the input image type.
   Parse the output image specification.
   If the return status is not success then 
      Abort with a fatal error message.
   Get and validate the user supplied parameters.
   Get the LAS global paramters.

*****************************************************************************
*/

{
char	mnmx[4];		/* minmax flag                               */

long	count,			/* number of values for a parameter          */
        i,			/* loop counter and array index              */
	newcount,		
	partype,		/* data type of a parameter                  */
	status,			/* return status                             */
	std_out;		/* logical unit for standard output          */

long inf_err = 1;
long fatal_err = 1;
long one = 1;
long zero = 0 ;

struct VARIABLE  *var;	/* pointer to a variable in the tae block     */

/*
Initialize tae. 
----------------
*/
img_limit = MAXIMG;
c_inlas (&block, &std_out);

/*
Parse the input image specification.
If the return status is not success then abort with a fatal error message.
---------------------------------------------------------------------------
*/
status = c_getin (&block, "IN", &zero, taein, hosin, num_inimg, window, 
                  num_img_band, bands_per_img, tot_inbands, &img_limit, 
                  same_wnd, img_flag);
if (status != E_SUCC)
   c_errmsg ("Cannot access input file names", "gaustran-getin", 
              &fatal_err);
if (same_wnd == 0)
   c_errmsg ("Image windows differ in size", "gaustran_window",
              &fatal_err);

/*
Determine the input image type.
-------------------------------
*/
for (i = 0; i < *num_inimg; i++)
   {
   status = c_getddr (hosin[i], &ddr);
   if (status != E_SUCC)
      c_errmsg ("DDR cannot be found", "gaustran-getddr", &fatal_err);
   }
*dtype = ddr.dtype;

/*
Parse the output image specification.
If the return status is not success then abort with a fatal error message.
--------------------------------------------------------------------------
*/
status = c_getout (&block, "OUT", taeout, hosout); 
if (status != E_SUCC)
   c_errmsg ("Cannot access output file names", "gaustran-getout", 
              &fatal_err);
strcpy (out_files[zero], taeout[0]);

/*
Get and validate the user supplied parameters.
-----------------------------------------------
*/
/* count = 50; */
/* c_getnum (&block, "SCALFACT", &count, &partype, scalfact); */
var = p_find (&block, "SCALFACT");
count = (*var).v_count;
for (i = 0; i < count; i++)
   scalfact[i] = RVAL(*var, i);
if (count < *tot_inbands)
   {
   newcount = count;
   c_errmsg ("Number of SCALFACT values < number of bands - using default",
              "gaustran-scalfact", &inf_err); 
   for (i = newcount; i < *tot_inbands; i++)
      scalfact[i] = 1.0;
   }
 
/* c_getnum (&block, "EXPSF", &count, &partype, expsf); */
var = p_find (&block, "EXPSF");
count = (*var).v_count;
for (i = 0; i < count; i++)
   expsf[i] = RVAL(*var, i);
if (count < *tot_inbands)
   {
   newcount = count;
   c_errmsg ("Number of EXPSF values < number of bands - using default",
             "gaustran-expsf", &inf_err); 
   for (i = newcount; i < *tot_inbands; i++)
      expsf[i] = 1.0;
   }

/* c_getnum (&block, "OFFSET", &count, &partype, offset);*/
var = p_find (&block, "OFFSET");
count = (*var).v_count;
for (i = 0; i < count; i++)
   offset[i] = RVAL(*var, i);
if (count < *tot_inbands)
   {
   newcount = count;
   c_errmsg ("Number of OFFSET values < number of bands - using default",
             "gaustran-offset", &inf_err); 
   for (i = newcount; i < *tot_inbands; i++)
      offset[i] = 1.0;
   }

/* 
Get the LAS global paramters.
--------------------------------- 
*/
partype = ELONG;
c_getnum (&block, "$MSGTIM", &one, &partype, &msgtim);
var = p_find(&block, "$MINMAX");
strcpy (mnmx, SVAL(*var, 0));
if (mnmx[0] == 'Y')
   min_max = TRUE;
else
   min_max = FALSE;


}
