/******************************************************************************

NAME:   CONVERT_WINDOW

PURPOSE: This routine will read a DDR structure and convert the window values
         to line/sample from their input units value.


PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "cproj.h"
#include "diskio.h"
#include "util.h"
#include "geotiffin.h"

int convert_window
(
   struct DDR *ddr,          /* I: pointer to DDR structure              */
   double window[],          /* I: windowed area of image, its units 
                                   can be DEG, DMS, PRO or LS            */
   long valid_window[],      /* O: long values of the converted and 
                                   validated window in sl, ss, nl, ns    */
   char units[]              /* I: window units (DEG, DMS, PRO, LS)      */ 
)
{

long lngunits = 0;           /* long value of the units
                                DEG=DEGREE(4), DMS=DMS(5), PRO=METER(2) */


/* Convert the string units value to a long value
   DEG = 4, DMS = 5, PRO = 2 (meters).  If units
   are LS then we don't need the lngunits because
   c_window_proj_to_ls will not be called.
-------------------------------------------------*/
if (!strncmp (units, "DEG", 3))
  lngunits = DEGREE;
else if (!strncmp (units, "DMS", 3))
  lngunits = DMS;
else if (!strncmp (units, "PRO", 3))
  lngunits = METER;

/* Convert the window parameters from their units to line/sample.
   This will not be done for units of LS.
-----------------------------------------------------------------*/
if (lngunits != 0)
  {
  if (c_window_proj_to_ls (ddr, window, &lngunits) != E_SUCC)
    {
    c_lasmsg(FATAL,"geotiffin-wndconv","Error converting window coordinates");
    }
  }

/* Check the new window
-----------------------*/
if (c_check_window (ddr, window, valid_window) != E_SUCC)
{
    c_lasmsg(FATAL,"geotiffin-wndchk","Error checking the window coordinates");
}

return (E_SUCC);
}
