/******************************************************************************
NAME:		G2D_GETPAR

PURPOSE:	Get the input parameters for gtif2ddr

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "geotiffin.h"

void g2d_getpar
(
    struct PARBLK *vblock,/* I: TAE parameter block 			      */
    char infile[][CMLEN], /* O: Input geoTIFF file		              */
    char outf[][CMLEN],   /* O: Output host name                              */
    long *bands,	  /* O: List of sorted bands in ascending order       */
    double *window,       /* O: windowed area of image                        */
    long *numouts,        /* O: number of output images                       */
    long *convert_window_flag, /* O: flag indicating a window was specified &
                                     to convert its units to LS(TRUE or FALSE)*/
    long *nl,		  /* O: num of lines in image (no windowing applied)  */
    long *ns,		  /* O: num of samples in image (no windowing applied)*/
    double pixel[],	  /* O: resolution and pixel size of image	      */
    int *mounted,	  /* O: flag indicating a CD is mounted		      */
    long *lan7,		  /* O: flag indicating landsat 7 data		      */
    char cd_dpath[],  /* O: data path to cd                              */
    char sceneid[],	  /* O: sceneid of geoTIFF image                      */
    char coorunits[]      /* O: units of the window                           */

)

{
	struct VARIABLE *val;     /* pointer to a TAE variable 	             */
	char outfile[MAX_FILE][CMLEN]; /* output ddr name in TAE form	     */
	long index;		  /* looping variable		             */

	/* Retrieve output image name
	-----------------------------*/
	val = p_find(vblock, "OUTF");
	*numouts = val->v_count;
	for (index = 0; index < val->v_count; index++)
	{  
		strcpy(outfile[index],SVAL(*val,index));
		/* need outfile in host format */
		c_cmhost(outfile[index],outf[index]);
		strcat(outf[index],".ddr");
    }

	/* Retrieve input geoTIFF file name
	-------------------------------*/
	val = p_find(vblock, "INFILE");
	for (index = 0; index < val->v_count; index++)
    {
		strcpy(infile[index],SVAL(*val,index)); 
    }

	/* Retrieve flag indicating if a window was to be created
	------------------------------------------------------------*/
	val = p_find(vblock, "CONVERT_WIN_FLG");
	*convert_window_flag = IVAL(*val,0);

	/* Retrieve window area to be copied
	----------------------------------*/
	if (*convert_window_flag == TRUE)
    {
		val = p_find(vblock,"WINDOW");
		if(val->v_count != 0)
		{
		   for (index = 0; index < 4; index++)
			 window[index] = RVAL(*val,index);
		}
	   else
      {
      for (index = 0; index < 4; index++)
         window[index] = 0.0;
      }
   }
   else
   {
	   val = p_find(vblock,"OUTWINDOW");
	   if(val->v_count != 0)
      {
      for (index = 0; index < 4; index++)
         window[index] = IVAL(*val,index);
      }
   }

	/* Retrieve sorted bands used to update the bddr
	----------------------------------------------*/
	val = p_find(vblock,"BANDS");
	if(val->v_count != 0)
   {
   for (index = 0; index < val->v_count; index++)
       bands[index] = IVAL(*val,index);
   }

	/* Retrieve total number of lines and samples 
	--------------------------------------------*/
	val = p_find(vblock,"NL");
	if(val->v_count != 0)
	   *nl = IVAL(*val,0);

	val = p_find(vblock,"NS");
	if(val->v_count != 0)
	   *ns = IVAL(*val,0);

	/* Retrieve the resolution (pixel size for x and y)
	-------------------------------------------------*/
	val = p_find(vblock,"RESOLX");
	if(val->v_count != 0)
	   pixel[0] = RVAL(*val,0);

	val = p_find(vblock,"RESOLY");
	if(val->v_count != 0)
	   pixel[1] = RVAL(*val,0);

	/* Retrieve MOUNTED flag
	-----------------------*/
	val = p_find(vblock,"MOUNTED");
	if(val->v_count != 0)
	   *mounted = IVAL(*val,0);

	/* Retrieve LAN7 flag
	--------------------*/
	val = p_find(vblock,"LAN7");
	if(val->v_count != 0)
	   *lan7 = IVAL(*val,0);

	/* Retrieve SCENEID
	-----------------*/
	val = p_find(vblock,"SCENEID");
	if(val->v_count != 0)
		strcpy(sceneid,SVAL(*val,0));               

	/* Retrieve data path to cd
	-------------------------*/
	val = p_find(vblock,"CD_DPATH");
	if(val->v_count != 0)
		strcpy(cd_dpath,SVAL(*val,0));
	else
		strcpy(cd_dpath,"");

	/* Get UNITS
	-----------*/
	val = p_find(vblock,"UNITS");
	if (val->v_count == 1)
		strcpy(coorunits, SVAL(*val,0));
	else
		strcpy(coorunits,"");

}
