/*************************************************************************
NAME: GEOGRAPHIC

PURPOSE:
	Retrieve geographic information from the geoTIFF keys. 

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      01/01  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

**************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "geompak.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "geotiffin.h"

int geographic

(
    char outf[][CMLEN],         /* I: output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer 	*/
    TIFF *in,                   /* I: Input tiff file pointer     	*/
    long *numouts,		/* I: number of output images    	*/
    double window[],		/* I: windowed area of image    	*/
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was 
                                      specified and to convert its 
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],		/* I: resolution of image		*/
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
) 

{
/* variables needed for geoTIFF keys */
double tiepoints[6];            /* corner coordinates values            */
short angularUnits=0;           /* angular unit of measurement          */

/* variables needed for proj parms, geographic does not have any, default to 0*/
double Lat = 0.0;
double Lon = 0.0;
double centerLon = 0.0;
double falseEast = 0.0;
double falseLat = 0.0;
double falseLon = 0.0;
double falseNorth = 0.0;
double geotrans[16];
double natOrigin = 0.0;
double num = 0.0;
double parallel1 = 0.0;
double parallel2 = 0.0;
double semiMajor = 0.0;
double semiMinor = 0.0;

/* for output to ddr */
long datum;			/* GCTP datum code                     */
long proj_code = 0;		/* GCTP projection code                */
long zone_code = 62;            /* UTM or State Plane code, default is 62 */

/* other geoTIFF key stuff */
short geoType = 0;              /* geoTIFF key for datum               */

/* other variables */
int  index;                     /* variable used for looping            */
int one = 1;                    /* constant                             */
int zero = 0;                   /* constant                             */
long ddr_valids[8];             /* array containing valids or invalids  */
long status;                    /* function return status               */

/* Set all the valid flags to VAILD
----------------------------------*/
for (index = 0; index < 8; index++)
    ddr_valids[index] = VALID;

/* Get corner coordinates
------------------------*/
if (get_coordinates(in, pixel[0],&ddr_valids[6], tiepoints, geotrans) != E_SUCC)
   c_lasmsg(FATAL, "geo-coordinates", "Error retrieving corner coordinates");

/* Get other keys
-----------------*/
status = GTIFKeyGet(gtif, GeogAngularUnitsGeoKey, &angularUnits, zero, one);

if (status == 0)
   {
   c_lasmsg(NONFATAL, "geographic-warn",
            "GeogAngularUnitsGeoKey does not exist in geoTIFF file");
   c_lasmsg(NONFATAL, "geographic-warn",
            "Assuming angular units to be degrees");

   /* Assuming degrees so change the value and flag proj. params VALID */
   angularUnits = 9102;
   }

/* Get datum 
-----------*/
GTIFKeyGet(gtif, GeographicTypeGeoKey, &geoType, zero, one);

if (get_datum(&geoType,&datum) != E_SUCC)
       c_lasmsg(FATAL,"geotiffin-datum","Fatal error encountered");

/* Write to DDR
--------------*/ 
if(update_ddr(outf,proj_code,zone_code,datum,semiMajor,semiMinor,parallel1,
           parallel2,centerLon,natOrigin,falseEast,falseNorth,
           Lon,Lat,falseLon,falseLat,num,angularUnits,tiepoints,numouts,window,
           bands,convert_window_flag,nl,ns,pixel,lan7,geotrans,ddr_valids,
           rasterType,valid_window,coorunits) != E_SUCC)
   c_lasmsg(FATAL,"geotiffin-ddr","Fatal error encountered");

return(E_SUCC);
}
