#ifndef PROTOS_H
#define PROTOS_H
#include "xtiffio.h"
#include "geotiffio.h"

#define DUMMY_BAND -1

#define MAX_FILE 256            /* Tiff files most likely store one
				   band per file. MAX_FILE is set 
				   to 256 which is the same number 
				   required to support maximum bands    */
#define FATAL 1 
#define NONFATAL 0 

int alaska
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int albers
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int azmuth
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

void bip2bil
(
    unsigned char *inbuf,       /* O: Processing buffer for strip geoTIFF
                                      images */
    unsigned char *tempbuf,     /* I: temp buffer for image manipulation -
                                      same size as inbuf */
    int imagesize,              /* I: size of line in image                */
    int nbands                  /* I: number of bands in the image         */
);


void cw_getpar
(
    struct PARBLK *vblock,      /* I: TAE parameter block           */
    double *window,             /* O: image window                  */
    char coorunits[]            /* O: units of the window           */
);

int eqrect
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int equidc
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */ 
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int get_datum
(
    short *geotype,               /* I: geoTIFF key number for datum  */
    long *datum                   /* O: LAS datum number              */ 
);

int get_stateplanezone
(
    short *projtype,            /* I: geoTIFF projection type code */
    long *zone,                 /* O: LAS state plane zone code    */
    short *linearunits,         /* I/O: unit of measurement        */
    long ddr_valids[]           /* I/O: array containing valids or invalids */
);

int get_utmzonegen
(
    short *projgeo,             /* I: generic geoTIFF key number for zone*/
    long *zonenum               /* O: LAS zone number                   */
);

int get_zone_datum
(
    short *projtype,            /* I: geoTIFF projection type code     */
    long *zone,                 /* O: LAS zone code                    */
    long *datum,                /* O: LAS datum code                   */
    short *linearunits,         /* I/O: unit of measurement            */
    long ddr_valids[]           /* I/O: array containing valids or invalids */
);

void getpar
(
    struct PARBLK *vblock, /* I: TAE parameter block                          */
    char comment[][CMLEN], /* O: CD mount message to operator                 */
    char dpath[],          /* O: directory path name to location of files     */
    char outpath[],        /* O: directory path name to outimg and bil files  */
    long bands[],          /* O: bands to be processed within image           */
    long *numimg_files,    /* O: number of output files                       */
    long nbands[],         /* O: number of bands to be copied to each img file*/
    char outimg[][CMLEN],  /* O: output image name(s)                         */
    char root[],           /* O: the root or partial root of the tif file 
                                 supplied by the user                         */
    long *totbands,        /* O: total number of bands to be processed        */
    long *process_all_bands/* O: flag to indicate if all the bands need to be
                                 processed                                    */
);

void g2d_getpar
(
    struct PARBLK *vblock,/* I: TAE parameter block                           */
    char infile[][CMLEN], /* O: Input tiff file                               */
    char outf[][CMLEN],   /* O: Output host name                              */
    long *bands,        /* O: List of sorted bands in ascending order         */
    double *window,     /* O: windowed area of image                          */
    long *numouts,      /* O: number of output images                         */
    long *convert_window_flag, /* O: flag indicating a window was specified &
                                     to convert its units to LS(TRUE or FALSE)*/
    long *nl,           /* O: number of lines in image (no windowing applied) */
    long *ns,           /* O: numberof samples in image (no windowing applied)*/
    double pixel[],     /* O: resolution and pixel size of image              */
    int  *mounted,      /* O: flag indicating a CD is mounted                 */
    long *lan7,         /* O: flag indicating landsat 7 data                  */
    char cd_dpath[],    /* O: data path to cd                              */
    char sceneid[],	/* O: sceneid of geoTIFF image			      */
    char coorunits[]
);

void sb_getpar
(
    struct PARBLK *vblock,      /* I: TAE parameter block               */
    char bil_files[][CMLEN],    /* O: array of bil files to be sorted   */
    char tif_files[][CMLEN],    /* O: array of tif files to be sorted   */
    long band_nums[],           /* O: array of sorted band numbers      */
    long *window                /* O: window area                       */
);


int gnomon
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int lamaz
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int lamcc
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);


int lng2tae
(
    struct PARBLK *vblk,/* I: TAE parameter block                     */
    long data[],        /* I: array of longs to transfer              */
    long skipf,         /* I: skip factor                             */
    long num_longs,     /* I: number of elements                      */
    char varname[],     /* I: tae name of variable                    */
    long *result        /* O: value of parameter                      */
);

int mercat
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int miller
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);


int mount_cd
(
    struct PARBLK *vblk,/* I: TAE parameter block                     */
    long *unit,         /* O: cd unit number                          */
    char comment[],     /* I: message to the operator                 */
    char device[],      /* O: device CD was mounted on                */
    char drive[],       /* O: drive name that operator mounted CD     */
    int  *mounted       /* O: mounted flag mounted=1,  not mounted=0  */
);

int nonexisting_bands
(
    char nonexist_bands[][CMLEN], /* I: array of nonexisting bands         */
    long bandnum[],        /* I: band numbers containing their true names  */
    long nbands[],         /* I: num of bands requested by user            */
    long totbands,         /* I: total number of bands to be processed     */
    char outimg[][CMLEN],  /* I: names of output images requested by user  */
    char newouts[][CMLEN], /* O: new output images, reflects nonexisting bands*/
    long new_nbands[],     /* O: new nbands to reflect nonexisting bands   */
    long new_bandnum[],    /* O: new bandnum to reflect nonexisting bands  */
    long lan7,             /* I: flag indicating landsat 7 data            */
    long nonexist_bands_totnum /*I: total number of bands in nonexist_bands*/
);


int om
(
    char outf[][CMLEN],         /* I: Output DDR file                      */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer           */
    TIFF *in,                   /* I: Input tiff file pointer              */
    long *numouts,              /* I: number of output images              */
    double window[],            /* I: windowed area of image               */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image            */
    long *ns,                   /* I: num of samples in the image          */
    double pixel[],             /* I: resolution of image                  */
    long *lan7,                 /* I: flag indicating landsat 7 data       */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int som
(
    char outf[][CMLEN],         /* I: Output DDR file                      */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer           */
    TIFF *in,                   /* I: Input tiff file pointer              */
    long *numouts,              /* I: number of output images              */
    double window[],            /* I: windowed area of image               */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image            */
    long *ns,                   /* I: num of samples in the image          */
    double pixel[],             /* I: resolution of image                  */
    long *lan7,                 /* I: flag indicating landsat 7 data       */
    char sceneid[],		/* I: sceneid of geoTIFF image             */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);


int ortho
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int poly 
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int process_band
(
    char infile[],         /* I: tif file to be processed                   */
    char outfile[],        /* O: Output BIL directory path and file         */
    double pixel[],        /* O: resolution of the image                    */
    uint16 *bitspersample, /* O: input data type                            */
    tsize_t *image_width,  /* O: number of samples                          */
    tsize_t *image_length, /* O: number of lines                            */
    tsample_t *nbands,     /* O: number of bands in the image               */
    long *real_flag,       /* O: Flag for floating-point data 1=true        */
    char outpath[]         /* I: directory path of the output image used
                                 for the output path of the  BIL file       */
);


int ps
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);


int recalc_corners
(
    double corners[],         /* I/O: corner points of the image          */
    long origsize[],          /* I: original size of the image NL,NS      */
    double window[]           /* I: image window specified                */
);


int robin
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int snsoid
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int somb
(
    char outf[][CMLEN],         /* I: Output DDR file                      */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer           */
    TIFF *in,                   /* I: Input tiff file pointer              */
    long *numouts,              /* I: number of output images              */
    double window[],            /* I: windowed area of image               */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image            */
    long *ns,                   /* I: num of samples in the image          */
    double pixel[],             /* I: resolution of image                  */
    long *lan7,                 /* I: flag indicating landsat 7 data       */
    char sceneid[],             /* I: sceneid of the geoTIFF image         */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);


int stereo
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int stpl
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);


int tm
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int update_ddr
(
    char hostddr[][CMLEN],/* I: name of the host ddr to update          */
    long proj_code,       /* I: GCTP projection code                    */
    long zone_code,       /* I: UTM or State Plane code                 */
    long datum,           /* I: GCTP datum code                         */
    double semiMajor,     /* I: semi major axis, sphere or lon/z        */
    double semiMinor,     /* I: semi minor axis                         */
    double parallel1,     /* I: standard parallel1,factor,height,satnum
                                or shapem                               */
    double parallel2,     /* I: standard parallel2,aziand,incang,path
                                or shapen                               */
    double centerLon,     /* I: central long,central merc,azmthpt or asclong */
    double natOrigin,     /* I: origin latitude, true scale, center latitude */
    double falseEast,     /* I: false easting                           */
    double falseNorth,    /* I: false northing                          */
    double Lon,           /* I: long1,PSRev,angle                       */
    double Lat,           /* I: lat1,LRat                               */
    double falseLon,      /* I: long2,PFlag                             */
    double falseLat,      /* I: lat2                                    */
    double num,           /* I: value of 0 or 1                         */
    int linearUnits,      /* I: projection units                        */
    double tiepoints[6],  /* I: corner coordinates                      */
    long *numouts,        /* I: number of output images                 */
    double window[],      /* I: windowed area of image                  */
    long bands[],         /* I: array of sorted used to update BDDR     */
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,             /* I: number of lines in nonwindowed image    */
    long *ns,             /* I: number of samples in nonwindowed image  */
    double pixel[],       /* I: resolution of image                     */
    long *lan7,           /* I: flag indicating landsat 7 data          */
    double geotrans[16],  /* I: model transformation corner coordinates */
    long ddr_valids[8],   /* I: array containing valids or invalids     */
    short *rasterType,    /* I: pixel definition as point or area *     */
    long valid_window[],  /* O: window values in line/sample            */
    char coorunits[]      /* I: window units (DEG,DMS,PRO,LS)           */
); 

int utm
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int vgrint
(
    char outf[][CMLEN],         /* I: Output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int geographic

(
    char outf[][CMLEN],         /* I: output DDR file                   */
    GTIF *gtif,                 /* I: Input geoTIFF file pointer        */
    TIFF *in,                   /* I: Input tiff file pointer           */
    long *numouts,              /* I: number of output images           */
    double window[],            /* I: windowed area of image            */
    long bands[],               /* I: array of sorted bands in ascending order*/
    long *convert_window_flag,  /* I: flag indicating a window was
                                      specified and to convert its
                                      units to LS (TRUE or FALSE)       */
    long *nl,                   /* I: num of lines in the image         */
    long *ns,                   /* I: num of samples in the image       */
    double pixel[],             /* I: resolution of image               */
    long *lan7,                 /* I: flag indicating landsat 7 data    */
    short *rasterType,          /* I: pixel definition as point or area */
    long valid_window[],        /* O: window values in line/sample      */
    char coorunits[]            /* I: window units (DEG,DMS,PRO,LS)     */
);

int get_coordinates
(
    TIFF *in,                   /* I: Input tiff file pointer              */
    double pixel,               /* I: resolution of image                  */
    long *ddr_valids,           /* I: corner coordinate flag value
                                      (VALID or INVAL)                     */
    double tiepoints[],         /* O: corner coordinates values            */
    double geotrans[]           /* O: model transformation corner
                                      coordinates                          */
);

int convert_window
(
   struct DDR *ddr,          /* I: pointer to DDR structure              */
   double window[],          /* I: windowed area of image, its units
                                   can be DEG, DMS, PRO or LS            */
   long valid_window[],      /* O: long values of the converted and
                                   validated window in sl, ss, nl, ns    */
   char units[]              /* I: window units (DEG, DMS, PRO, LS)      */
);

#endif
