/***************************************************************************
NAME: GET_DATUM

PURPOSE:  Given the geoTIFF code, return the LAS datum code.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*****************************************************************************/
#include "las.h"
#include "geotiffin.h"

int get_datum
(
    short *geotype,		  /* I: geoTIFF key number for datum */
    long *datum		          /* O: LAS datum number             */
)
{

	/* Keys ranging from 4200 - 4999 are EPSG GCS Based on EPSG Datum
	---------------------------------------------------------------*/ 
	if      (*geotype == 4205) 
			*datum = 107;	     /* Afgooye                      */ 
	else if (*geotype == 4210)
			*datum = 122;	     /* Arc 1960                     */
	else if (*geotype == 4216)
			*datum = 130;        /* Bermuda 1957                 */
	else if (*geotype == 4218) 
			*datum = 132;  	     /* Bogota                       */
	else if (*geotype == 4219) 
			*datum = 133;  	     /* Bukit Rimpah                 */
	else if (*geotype == 4221) 
			*datum = 135;        /* Campo Inchauspe              */ 
	else if (*geotype == 4222) 
			*datum = 137;  	     /* Cape                         */
	else if (*geotype == 4223) 
			*datum = 140;  	     /* Carthage                     */
	else if (*geotype == 4225)
			*datum = 143; 	     /* Corrego Alegre               */
	else if (*geotype == 4236) 
			*datum = 180; 	     /* Hu Tzu Shan                  */
	else if (*geotype == 4239) 
			*datum = 186;  	     /* Indian 1954                  */
	else if (*geotype == 4240)
			*datum = 189;  	     /* Indian 1975                  */
	else if (*geotype == 4244) 
			*datum = 194;  	     /* Kandawala                    */
	else if (*geotype == 4250) 
			*datum = 200;  	     /* Leigon                       */
	else if (*geotype == 4251) 
			*datum = 201;  	     /* Liberia 1964                 */
	else if (*geotype == 4256) 
			*datum = 207;        /* Mahe 1971                    */
	else if (*geotype == 4266) 
			*datum = 211;  	     /* M'poraloko                   */
	else if (*geotype == 4267) 
            *datum = 225;        /* NAD 27                       */
	else if (*geotype == 4269)        
            *datum = 219;        /* NAD83                        */
	else if (*geotype == 4322) 
			*datum = 316;  	     /* WGS 72                       */
	else if (*geotype == 4326) 
			*datum = 317;  	     /* WGS 84                       */


	/* Keys ranging from 4000-4199 are EPSG GCS Based on Ellipsoid only  
	--------------------------------------------------------------------*/
	else if (*geotype == 4001)
            *datum = 9;          /* Airy 1930                    */
	else if (*geotype == 4002)
            *datum = 11;         /* Airy Modified 1849           */
	else if (*geotype == 4003)
            *datum = 14;         /* Australian National Spheroid */
	else if (*geotype == 4004)
            *datum = 2;          /* Bessel 1841                  */
	else if (*geotype == 4006)
            *datum = 20;         /* Bessel 1841 (Namibia)        */
	else if (*geotype == 4008)
            *datum = 0;          /* Clarke 1866                  */
	else if (*geotype == 4012)
            *datum = 1;          /* Clarke 1880                  */
	else if (*geotype == 4015)
            *datum = 6;          /* Everest 1930 1937 Adjustment */
	else if (*geotype == 4016)
            *datum = 21;         /* Everest 1830 1967 Definition */
                                 /* (Sabah & Sarawak)    */
	else if (*geotype == 4017)
            *datum = 22;         /* Everest 1830 1975 Definition */
                                 /* (India 1956)         */
	else if (*geotype == 4018)
            *datum = 10;         /* Everest 1930 Modelse ified   */
	else if (*geotype == 4019)
            *datum = 8;          /* GRS 1980                     */
	else if (*geotype == 4020)
            *datum = 27;         /* Helmert 1906                 */
	else if (*geotype == 4021)
            *datum = 28;         /* Indonesian National Spheroid */
	else if (*geotype == 4022)
            *datum = 4;          /* International 1924           */
	else if (*geotype == 4023)
            *datum = 3;          /* International 1967           */
	else if (*geotype == 4024)
            *datum = 15;         /* Krassovsky 1940              */
	else if (*geotype == 4025)
            *datum = 7;          /* WGS 66 (NWL9D)               */
	else if (*geotype == 4026)
            *datum = 5;          /* WGS 72 (NWL10D)              */
	else if (*geotype == 4030) 
            *datum = 12;         /* WGS 84                       */
	else 
		c_lasmsg(FATAL,"geotiffin-datum", 
					  "GeoTIFF does not contain valid datum code");

	/* Warn user if the datum code
	----------------------------*/
	if (*datum < 100)
	{
		c_lasmsg(NONFATAL,"geotiffin-datum", "Warning: GeoTIFF does not "
         "contain a valid datum code. The spheroid code will be used instead.");
	}

	return(E_SUCC);
}
