/*****************************************************************************
NAME: GET_STATEPLANEZONE

PURPOSE:  Given the geoTIFF code, return the LAS zone code.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

REFERENCES: This information was obtained from: 

Dr. Niles Ritter (formerly of Jet Propulsion Laboratory)
  Mike Ruth, SPOT Image Corp, Product Development Group
  GeoTIFF Format Specification, specification Version 1.8.1, Available at:
  http://home.earthlink.net/~ritter/geotiff/spec/geotiffhome.html


*****************************************************************************/

#include "las.h" 
#include "geotiffin.h"

int get_stateplanezone
(
    short *projtype,		/* I: geoTIFF projection type code */
    long *zone,			/* O: LAS state plane zone code    */
    short *linearunits,         /* I/O: units of measurement       */
    long ddr_valids[]           /* I/O: array containing valids or invalids */
)		 

{

    if  (*projtype == 26729)      /* Alabama East           */
        *zone = 101;
    else if (*projtype == 26730)  /* Alabama West           */
        *zone = 102;  			
    else if (*projtype == 26748)  /* Arizona East           */
	*zone = 201;
    else if (*projtype == 26749)  /* Arizona Central        */
	*zone = 202;
    else if (*projtype == 26750)  /* Arizona West           */
	*zone = 203;
    else if (*projtype == 26751)  /* Arkansas North         */
	*zone = 301;
    else if (*projtype == 26752)  /* Arkansas South         */
	*zone = 302;
    else if (*projtype == 26741)  /* California I           */
	*zone = 401;
    else if (*projtype == 26742)  /* California II          */
	*zone = 402;
    else if (*projtype == 26743)  /* California III         */
	*zone = 403;
    else if (*projtype == 26744)  /* California IV          */
	*zone = 404;
    else if (*projtype == 26745)  /* California V           */
	*zone = 405;
    else if (*projtype == 26746)  /* California VI          */
	*zone = 406;
    else if (*projtype == 26747)  /* California VII         */
	*zone = 407;
    else if (*projtype == 26753)  /* Colorado North         */
	*zone = 501;
    else if (*projtype == 26754)  /* Colorado Central       */
	*zone = 502;
    else if (*projtype == 26755)  /* Colorado South         */
	*zone = 503;
    else if (*projtype == 26756)  /* Connecticut            */
	*zone = 600;
    else if (*projtype == 26757)  /* Delaware               */
	*zone = 700;
    else if (*projtype == 26758)  /* Florida East           */
	*zone = 901;
    else if (*projtype == 26759)  /* Florida West           */
	*zone = 902;
    else if (*projtype == 26760)  /* Florida North          */
	*zone = 903;
    else if (*projtype == 26766)  /* Georgia East           */
	*zone = 1001;
    else if (*projtype == 26767)  /* Georgia West           */
	*zone = 1002;
    else if (*projtype == 26768)  /* Idaho East             */
	*zone = 1101;
    else if (*projtype == 26769)  /* Idaho Central          */
	*zone = 1102;
    else if (*projtype == 26770)  /* Idaho West             */
	*zone = 1103;
    else if (*projtype == 26771)  /* Illinois East          */
	*zone = 1201;
    else if (*projtype == 26772)  /* Illinois West          */
	*zone = 1202;
    else if (*projtype == 26773)  /* Indiana East           */
	*zone = 1301;
    else if (*projtype == 26774)  /* Indiana West           */
	*zone = 1302;
    else if (*projtype == 26775)  /* Iowa North             */
	*zone = 1401;
    else if (*projtype == 26776)  /* Iowa South             */
	*zone = 1402;
    else if (*projtype == 26777)  /* Kansas North           */
	*zone = 1501;
    else if (*projtype == 26778)  /* Kansas South           */
	*zone = 1502;
    else if (*projtype == 26779)  /* Kentucky North         */
	*zone = 1601;
    else if (*projtype == 26780)  /* Kentucky South         */
	*zone = 1602;
    else if (*projtype == 26781)  /* Louisiana North        */
	*zone = 1701;
    else if (*projtype == 26782)  /* Louisiana South        */
	*zone = 1702;
    else if (*projtype == 26783)  /* Maine East             */
	*zone = 1801;
    else if (*projtype == 26784)  /* Maine West             */ 
	*zone = 1802;
    else if (*projtype == 26785)  /* Maryland               */
	*zone = 1900;
    else if (*projtype == 26786)  /* Massachusetts          */
	*zone = 2001;
    else if (*projtype == 26787)  /* Massachusetts Island   */
	*zone = 2002;
    else if (*projtype == 26801)  /* Michigan East (TM)     */
	*zone = 2101;
    else if (*projtype == 26802)  /* Michigan Central (TM)  */
	*zone = 2102;
    else if (*projtype == 26803)  /* Michigan West (TM)     */
	*zone = 2103;
    else if (*projtype == 26788)  /* Michigan North         */
	*zone = 2111;
    else if (*projtype == 26789)  /* Michigan Central       */
	*zone = 2112;
    else if (*projtype == 26790)  /* Michigan South         */
	*zone = 2113;
    else if (*projtype == 26791)  /* Minnesota North        */
	*zone = 2201;
    else if (*projtype == 26792)  /* Minnesota Central      */
	*zone = 2202;
    else if (*projtype == 26793)  /* Minnesota South        */
	*zone = 2203;
    else if (*projtype == 26794)  /* Mississippi East       */
	*zone = 2301;
    else if (*projtype == 26795)  /* Mississippi West       */
	*zone = 2302;
    else if (*projtype == 26796)  /* Missouri East          */
	*zone = 2401;
    else if (*projtype == 26797)  /* Missouri Central       */
	*zone = 2402;
    else if (*projtype == 26798)  /* Missouri West          */
	*zone = 2403;
    else if (*projtype == 32001)  /* Montana North          */
	*zone = 2501;
    else if (*projtype == 32002)  /* Montana Central        */
	*zone = 2502;
    else if (*projtype == 32003)  /* Montana South          */
	*zone = 2503;
    else if (*projtype == 32005)  /* Nebraska North         */
	*zone = 2601;
    else if (*projtype == 32006)  /* Nebraska South         */
	*zone = 2602;
    else if (*projtype == 32007)  /* Nevada East            */
	*zone = 2701;
    else if (*projtype == 32008)  /* Nevada Central         */
	*zone = 2702;
    else if (*projtype == 32009)  /* Nevada West            */
	*zone = 2703;
    else if (*projtype == 32010)  /* New Hampshire          */
	*zone = 2800;
    else if (*projtype == 32011)  /* New Jersey             */
	*zone = 2900;
    else if (*projtype == 32012)  /* New Mexico East        */
	*zone = 3001;
    else if (*projtype == 32013)  /* New Mexico Central     */
	*zone = 3002;
    else if (*projtype == 32014)  /* New Mexico West        */
	*zone = 3003;
    else if (*projtype == 32015)  /* New York East          */
	*zone = 3101;
    else if (*projtype == 32016)  /* New York Central       */
	*zone = 3102;
    else if (*projtype == 32017)  /* New York West          */
	*zone = 3103;
    else if (*projtype == 32018)  /* New York Long Island   */
	*zone = 3104;
    else if (*projtype == 32019)  /* North Carolina         */
	*zone = 3200;
    else if (*projtype == 32020)  /* North Dakota North     */
	*zone = 3301;
    else if (*projtype == 32021)  /* North Dakota South     */
	*zone = 3302;
    else if (*projtype == 32022)  /* Ohio North             */
	*zone = 3401;
    else if (*projtype == 32023)  /* Ohio South             */
	*zone = 3402;
    else if (*projtype == 32024)  /* Oklahoma North         */
	*zone = 3501;
    else if (*projtype == 32025)  /* Oklahoma South         */
	*zone = 3502;
    else if (*projtype == 32026)  /* Oregon North           */
	*zone = 3601;
    else if (*projtype == 32027)  /* Oregon South           */
	*zone = 3602;
    else if (*projtype == 32028)  /* Pennsylvania North     */
	*zone = 3701;
    else if (*projtype == 32029)  /* Pennsylvania South     */
	*zone = 3702;
    else if (*projtype == 32030)  /* Rhode Island           */
	*zone = 3800;
    else if (*projtype == 32031)  /* South Carolina North   */
	*zone = 3901;
    else if (*projtype == 32033)  /* South Carolina South   */
	*zone = 3902;
    else if (*projtype == 32034)  /* South Dakota North     */
	*zone = 4001;
    else if (*projtype == 32035)  /* South Dakota South     */
	*zone = 4002;
    else if (*projtype == 32036)  /* Tennessee              */
	*zone = 4100;
    else if (*projtype == 32037)  /* Texas North            */
	*zone = 4201;
    else if (*projtype == 32038)  /* Texas North Central    */
	*zone = 4202;
    else if (*projtype == 32039)  /* Texas Central          */
	*zone = 4203;
    else if (*projtype == 32040)  /* Texas South Central    */
	*zone = 4204;
    else if (*projtype == 32041)  /* Texas South            */
	*zone = 4205;
    else if (*projtype == 32042)  /* Utah North             */
	*zone = 4301;
    else if (*projtype == 32043)  /* Utah Central           */
	*zone = 4302;
    else if (*projtype == 32044)  /* Utah North             */
	*zone = 4303;
    else if (*projtype == 32045)  /* Vermont                */
	*zone = 4400;
    else if (*projtype == 32046)  /* Virginia North         */
	*zone = 4501;
    else if (*projtype == 32047)  /* Virginia South         */
	*zone = 4502;
    else if (*projtype == 32048)  /* Washington North       */
	*zone = 4601;
    else if (*projtype == 32049)  /* Washington South       */
	*zone = 4602;
    else if (*projtype == 32050)  /* West Virginia North    */
	*zone = 4701;
    else if (*projtype == 32051)  /* West Virginia South    */
	*zone = 4702;
    else if (*projtype == 32052)  /* Wisconsin North        */
	*zone = 4801;
    else if (*projtype == 32053)  /* Wisconsin Central      */
	*zone = 4802;
    else if (*projtype == 32054)  /* Wisconsin South        */
	*zone = 4803;
    else if (*projtype == 32055)  /* Wyoming East           */
	*zone = 4901;
    else if (*projtype == 32056)  /* Wyoming East Central   */
	*zone = 4902;
    else if (*projtype == 32057)  /* Wyoming West Central   */
	*zone = 4903;
    else if (*projtype == 32058)  /* Wyoming West           */
	*zone = 4904;
    else if (*projtype == 26731)  /* Alaska *zone 1          */
	*zone = 5001;
    else if (*projtype == 26732)  /* Alaska *zone 2          */
	*zone = 5002;
    else if (*projtype == 26733)  /* Alaska *zone 3          */
	*zone = 5003;
    else if (*projtype == 26734)  /* Alaska *zone 4          */
	*zone = 5004;
    else if (*projtype == 26735)  /* Alaska *zone 5          */
	*zone = 5005;
    else if (*projtype == 26736)  /* Alaska *zone 6          */
	*zone = 5006;
    else if (*projtype == 26737)  /* Alaska *zone 7          */
	*zone = 5007;
    else if (*projtype == 26738)  /* Alaska *zone 8          */
	*zone = 5008;
    else if (*projtype == 26739)  /* Alaska *zone 9          */
	*zone = 5009;
    else if (*projtype == 26740)  /* Alaska *zone 10         */
	*zone = 5010;
    else if (*projtype == 26761)  /* Hawaii *zone 1          */
	*zone = 5101;
    else if (*projtype == 26762)  /* Hawaii *zone 2          */
	*zone = 5102;
    else if (*projtype == 26763)  /* Hawaii *zone 3          */
	*zone = 5103;
    else if (*projtype == 26764)  /* Hawaii *zone 4          */
	*zone = 5104;
    else if (*projtype == 26765)  /* Hawaii *zone 5          */
	*zone = 5105;
    else if (*projtype == 32059)  /* Puerto Rico            */
	*zone = 5201;
    else if (*projtype == 32060)  /* Virgin Islands St Croix*/
	*zone = 5202;
    else if (*projtype == 26929)  /* Alabama East           */
	*zone = 101;
    else if (*projtype == 26930)  /* Alabama West           */
	*zone = 102;
    else if (*projtype == 26948)  /* Arizona East           */
	*zone = 201;
    else if (*projtype == 26949)  /* Arizona Central        */
	*zone = 202;
    else if (*projtype == 26950)  /* Arizona West           */
	*zone = 203;
    else if (*projtype == 26951)  /* Arkansas North         */
	*zone = 301;
    else if (*projtype == 26952)  /* Arkansas South         */
	*zone = 302;
    else if (*projtype == 26941)  /* California I           */
	*zone = 401;
    else if (*projtype == 26942)  /* California II          */
	*zone = 402;
    else if (*projtype == 26943)  /* California III         */
	*zone = 403;
    else if (*projtype == 26944)  /* California IV          */
	*zone = 404;
    else if (*projtype == 26945)  /* California V           */
	*zone = 405;
    else if (*projtype == 26946)  /* California VI          */
	*zone = 406;
    else if (*projtype == 26953)  /* Colorado North         */
	*zone = 501;
    else if (*projtype == 26954)  /* Colorado Central       */
	*zone = 502;
    else if (*projtype == 26955)  /* Colorado South         */
	*zone = 503;
    else if (*projtype == 26956)  /* Connecticut            */
	*zone = 600;
    else if (*projtype == 26957)  /* Delaware               */
	*zone = 700;
    else if (*projtype == 26958)  /* Florida East           */
	*zone = 901;
    else if (*projtype == 26959)  /* Florida West           */
	*zone = 902;
    else if (*projtype == 26960)  /* Florida North          */
	*zone = 903;
    else if (*projtype == 26966)  /* Georgia East           */
	*zone = 1001;
    else if (*projtype == 26967)  /* Georgia West           */
	*zone = 1002;
    else if (*projtype == 26968)  /* Idaho East             */
	*zone = 1101;
    else if (*projtype == 26969)  /* Idaho Central          */
	*zone = 1102;
    else if (*projtype == 26970)  /* Idaho West             */
	*zone = 1103;
    else if (*projtype == 26971)  /* Illinois East          */
	*zone = 1201;
    else if (*projtype == 26972)  /* Illinois West          */
	*zone = 1202;
    else if (*projtype == 26973)  /* Indiana East           */
	*zone = 1301;
    else if (*projtype == 26974)  /* Indiana West           */
	*zone = 1302;
    else if (*projtype == 26975)  /* Iowa North             */
	*zone = 1401;
    else if (*projtype == 26976)  /* Iowa South             */
	*zone = 1402;
    else if (*projtype == 26977)  /* Kansas North           */
	*zone = 1501;
    else if (*projtype == 26978)  /* Kansas South           */
	*zone = 1502;
    else if (*projtype == 26979)  /* Kentucky North         */
	*zone = 1601;
    else if (*projtype == 26980)  /* Kentucky South         */
	*zone = 1602;
    else if (*projtype == 26981)  /* Louisiana North        */
	*zone = 1701;
    else if (*projtype == 26982)  /* Louisiana South        */
	*zone = 1702;
    else if (*projtype == 26983)  /* Maine East             */
	*zone = 1801;
    else if (*projtype == 26984)  /* Maine West             */
	*zone = 1802;
    else if (*projtype == 26985)  /* Maryland               */
	*zone = 1900;
    else if (*projtype == 26986)  /* Massachusetts          */
	*zone = 2001;
    else if (*projtype == 26987)  /* Massachusetts Island   */
	*zone = 2002;
    else if (*projtype == 26988)  /* Michigan North         */
	*zone = 2111;
    else if (*projtype == 26989)  /* Michigan Central       */
	*zone = 2112;
    else if (*projtype == 26990)  /* Michigan South         */
	*zone = 2113;
    else if (*projtype == 26991)  /* Minnesota North        */
	*zone = 2201;
    else if (*projtype == 26992)  /* Minnesota Central      */
	*zone = 2202;
    else if (*projtype == 26993)  /* Minnesota South        */
	*zone = 2203;
    else if (*projtype == 26994)  /* Mississippi East       */
	*zone = 2301;
    else if (*projtype == 26995)  /* Mississippi West       */
	*zone = 2302;
    else if (*projtype == 26996)  /* Missouri East          */
	*zone = 2401;
    else if (*projtype == 26997)  /* Missouri Central       */
	*zone = 2402;
    else if (*projtype == 26998)  /* Missouri West          */
	*zone = 2403;
    else if (*projtype == 32100)  /* Montana                */
	*zone = 2500;
    else if (*projtype == 32104)  /* Nebraska               */
	*zone = 2600;
    else if (*projtype == 32107)  /* Nevada East            */
	*zone = 2701;
    else if (*projtype == 32108)  /* Nevada Central         */
	*zone = 2702;
    else if (*projtype == 32109)  /* Nevada West            */
	*zone = 2703;
    else if (*projtype == 32110)  /* New Hampshire          */
	*zone = 2800;
    else if (*projtype == 32111)  /* New Jersey             */
	*zone = 2900;
    else if (*projtype == 32112)  /* New Mexico East        */
	*zone = 3001;
    else if (*projtype == 32113)  /* New Mexico Central     */
	*zone = 3002;
    else if (*projtype == 32114)  /* New Mexico West        */
	*zone = 3003;
    else if (*projtype == 32115)  /* New York East          */
	*zone = 3101;
    else if (*projtype == 32116)  /* New York Central       */
	*zone = 3102;
    else if (*projtype == 32117)  /* New York West          */
	*zone = 3103;
    else if (*projtype == 32118)  /* New York Long Island   */
	*zone = 3104;
    else if (*projtype == 32119)  /* North Carolina         */
	*zone = 3200;
    else if (*projtype == 32120)  /* North Dakota North     */
	*zone = 3301;
    else if (*projtype == 32121)  /* North Dakota South     */
	*zone = 3302;
    else if (*projtype == 32122)  /* Ohio North             */
	*zone = 3401;
    else if (*projtype == 32123)  /* Ohio South             */
	*zone = 3402;
    else if (*projtype == 32124)  /* Oklahoma North         */
	*zone = 3501;
    else if (*projtype == 32125)  /* Oklahoma South         */
	*zone = 3502;
    else if (*projtype == 32126)  /* Oregon North           */
	*zone = 3601;
    else if (*projtype == 32127)  /* Oregon South           */
	*zone = 3602;
    else if (*projtype == 32128)  /* Pennsylvania North     */
	*zone = 3701;
    else if (*projtype == 32129)  /* Pennsylvania South     */
	*zone = 3702;
    else if (*projtype == 32130)  /* Rhode Island           */
	*zone = 3800;
    else if (*projtype == 32133)  /* South Carolina         */
	*zone = 3900;
    else if (*projtype == 32134)  /* South Dakota North     */
	*zone = 4001;
    else if (*projtype == 32135)  /* South Dakota South     */
	*zone = 4002;
    else if (*projtype == 32136)  /* Tennessee              */
	*zone = 4100;
    else if (*projtype == 32137)  /* Texas North            */
	*zone = 4201;
    else if (*projtype == 32138)  /* Texas North Central    */
	*zone = 4202;
    else if (*projtype == 32139)  /* Texas Central          */
	*zone = 4203;
    else if (*projtype == 32140)  /* Texas South Central    */
	*zone = 4204;
    else if (*projtype == 32141)  /* Texas South            */
	*zone = 4205;
    else if (*projtype == 32142)  /* Utah North             */
	*zone = 4301;
    else if (*projtype == 32143)  /* Utah Central           */
	*zone = 4302;
    else if (*projtype == 32144)  /* Utah North             */
	*zone = 4303;
    else if (*projtype == 32145)  /* Vermont                */
	*zone = 4400;
    else if (*projtype == 32146)  /* Virginia North         */
	*zone = 4501;
    else if (*projtype == 32147)  /* Virginia South         */	
	*zone = 4502;
    else if (*projtype == 32148)  /* Washington North       */	
	*zone = 4601;
    else if (*projtype == 32149)  /* Washington South       */	
	*zone = 4602;
    else if (*projtype == 32150)  /* West Virginia North    */
	*zone = 4701;
    else if (*projtype == 32151)  /* West Virginia South    */
	*zone = 4702;
    else if (*projtype == 32152)  /* Wisconsin North        */
	*zone = 4801;
    else if (*projtype == 32153)  /* Wisconsin Central      */
	*zone = 4802;
    else if (*projtype == 32154)  /* Wisconsin South        */
	*zone = 4803;
    else if (*projtype == 32155)  /* Wyoming East           */
	*zone = 4901;
    else if (*projtype == 32156)  /* Wyoming East Central   */
	*zone = 4902;
    else if (*projtype == 32157)  /* Wyoming West Central   */
	*zone = 4903;
    else if (*projtype == 32158)  /* Wyoming West           */
	*zone = 4904;
    else if (*projtype == 26931)  /* Alaska *zone 1          */
	*zone = 5001;
    else if (*projtype == 26932)  /* Alaska *zone 2          */
	*zone = 5002;
    else if (*projtype == 26933)  /* Alaska *zone 3          */
	*zone = 5003;
    else if (*projtype == 26934)  /* Alaska *zone 4          */
	*zone = 5004;
    else if (*projtype == 26935)  /* Alaska *zone 5          */
	*zone = 5005;
    else if (*projtype == 26936)  /* Alaska *zone 6          */
	*zone = 5006;
    else if (*projtype == 26937)  /* Alaska *zone 7          */
	*zone = 5007;
    else if (*projtype == 26938)  /* Alaska *zone 8          */
	*zone = 5008;
    else if (*projtype == 26939)  /* Alaska *zone 9          */
	*zone = 5009;
    else if (*projtype == 26940)  /* Alaska *zone 10         */
	*zone = 5010;
    else if (*projtype == 26961)  /* Hawaii *zone 1          */
	*zone = 5101;
    else if (*projtype == 26962)  /* Hawaii *zone 2          */
	*zone = 5102;
    else if (*projtype == 26963)  /* Hawaii *zone 3          */
	*zone = 5103;
    else if (*projtype == 26964)  /* Hawaii *zone 4          */
	*zone = 5104;
    else if (*projtype == 26965)  /* Hawaii *zone 5          */
	*zone = 5105;
    else if (*projtype == 32161)  /* Puerto Rico Virgin Isl */
	*zone = 5200;
    else
        c_lasmsg(FATAL,"geotiffin-zone",
         "ProjectedCSTypeGeoKey contains a zone which is not supported by LAS");

/* get linear units if it key does not exist 
-------------------------------------------*/
    if (*linearunits == 0)
       {
       if (((*projtype >= 26729) && (*projtype <= 26813)) || 
           ((*projtype >= 32001) && (*projtype <= 32077)))
          *linearunits = 9003;
       else
          *linearunits = 9001;
       ddr_valids[4] = VALID;
       }

return(E_SUCC);
}
