/****************************************************************************
NAME:  GET_ZONE_DATUM

PURPOSE: Given the geoTIFF code ProjectedCSTypeGeoKey, 
         return the LAS datum and zone.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "geotiffin.h"

int get_zone_datum
(
    short *projtype,		/* I: geoTIFF projection type code     */
    long *zone,			/* O: LAS zone code                    */
    long *datum,	        /* O: LAS datum code                   */
    short *linearunits,		/* I/O: unit of measurement	       */
    long ddr_valids[]           /* I/O: array containing valids or invalids */
)

{

char temp[5];
char temp2[3];

/* Given ProjectedCSTypeGeoKey code, get the zone and datum code 
   (zone is last 2 digits of the code)
-----------------------------------------------------------------------*/
if (*projtype != 0)
{
   if((32200 < *projtype) && (*projtype <= 32560)) /* this range shows WGS72*/
     {							
     *datum = 316;
     sprintf(temp,"%ld",*projtype);
     strncpy(temp2,(temp+3),2);
     temp2[2]='\0';
     *zone = atol(temp2);
     if (*linearunits == 0)
        *linearunits = 9001;    
     ddr_valids[4] = VALID;
 }
   else if ((32600 < *projtype) && (*projtype <= 32760)) /* this shows WGS84 */
  {							    
     *datum = 317;
     sprintf(temp,"%ld",*projtype);
     strncpy(temp2,(temp+3),2);
     temp2[2]='\0';
     *zone = atol(temp2);
     if (*linearunits == 0)    
         *linearunits = 9001;
     ddr_valids[4] = VALID;
   }
   else if ((26703 <= *projtype) && (*projtype <= 26722)) /* this shows NAD27 */
     {
     *datum = 225;
     sprintf(temp,"%ld",*projtype);
     strncpy(temp2,(temp+3),2);
     temp2[2]='\0';
     *zone = atol(temp2);
     if (*linearunits == 0)    
        *linearunits = 9001;
     ddr_valids[4] = VALID;
   }
   else if ((26903 <= *projtype) && (*projtype <= 26923)) /* this shows NAD83 */
   {
     *datum = 219;
     sprintf(temp,"%ld",*projtype);
     strncpy(temp2,(temp+3),2);
     temp2[2]='\0';
     *zone = atol(temp2);
     if (*linearunits == 0)    
        *linearunits = 9001;
     ddr_valids[4] = VALID;
   }
   else
   {
     c_lasmsg(FATAL,"geotiffin-zone_datum",
               "geoTIFF has an invalid ProjectedCSTypeGeoKey for UTM");
     }
   }

/* If the zone is located below the equator, then the zone code must be
negative
----------------------------------------------------------------------*/
if (((32301 <= *projtype) && (*projtype <= 32360)) ||
    ((32501 <= *projtype) && (*projtype <= 32560)) ||
    ((32701 <= *projtype) && (*projtype <= 32760)))
    {
    *zone = -*zone;
    }
return(E_SUCC);
}
