/*******************************************************************************
NAME:           GETPAR

PURPOSE:        Get the input parameters for gtif

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "geotiffin.h"

void getpar
(
    struct PARBLK *vblock, /* I: TAE parameter block                          */
    char comment[][CMLEN], /* O: CD mount message to operator                 */
    char dpath[],          /* O: directory path name to location of files     */
    char outpath[],        /* O: directory path name to outimg and bil files  */
    long bands[],          /* O: bands to be processed within image           */
    long *numimg_files,    /* O: number of output files                       */
    long nbands[],         /* O: number of bands to be copied to each img file*/
    char outimg[][CMLEN],  /* O: output image name(s)                         */
    char root[],           /* O: the root or partial root of the tif file 
                                 supplied by the user                         */
    long *totbands,        /* O: total number of bands to be processed        */
    long *process_all_bands/* O: flag to indicate if all the bands need to be
                                 processed                                    */
)

{

    struct VARIABLE *val;  /* pointer to a TAE variable                     */
    struct VARIABLE *val2; /* pointer to a TAE variable                     */ 

    char ext[CMLEN];        /* the extension portion of the tif file name   */ 
    char host_path[CMLEN];  /* host path name                               */
    char out_path[CMLEN];   /* path to output image and bil files           */
    char path[CMLEN];       /* path name entered by user in either tae 
                               or host format                               */
    long index;             /* loop variable                                */      
    long len2;              /* temporary variable for length                */
    long len;               /* temporary variable for length                */
    long total_cds;         /* number of CDs specified                      */
    long total_nbands=0;    /* sum of nbands                                */ 
    long totbands2;         /* total number of bands requested by user      */

    /* Retrieve CD mount message supplied by user
    ----------------------------------------------*/
    val = p_find(vblock,"COMMENT");
    total_cds = val->v_count;
    for (index = 0; index < total_cds; index++)
        strcpy(comment[index],SVAL(*val,index));

    /* Get requested NBANDS and BANDS
    -------------------------------*/
    val = p_find(vblock, "BANDS");
    *totbands = val->v_count;

    /* Set a flag to indicate if the user specified band numbers. If not then
       all the bands will be processed */
    if (*totbands == 0)
        *process_all_bands = TRUE;
    else
        *process_all_bands = FALSE;

    for (index = 0; index < *totbands; index++)
        bands[index] = IVAL(*val,index);
    val2 = p_find(vblock, "NBANDS");
    totbands2 = val2->v_count;

    /* Get sum of NBANDS and do some error checking
    ---------------------------------------------*/
    for (index = 0; index < totbands2; index++)
    {
        nbands[index]=IVAL(*val2,index); 
        total_nbands += nbands[index];
    }

    if ((*totbands == 0) && (totbands2 != 0))
        c_lasmsg(FATAL,"geotiffin-getpar",
                "BANDS cannot be NULL if NBANDS is specified");

    else if ((totbands2 != 0) && (*totbands != total_nbands))
        c_lasmsg(FATAL,"geotiffin-getpar",
                "Total number of BANDS must equal sum of NBANDS");

    /* If the user did not specify NBANDS, set it to the total count of
       the number of bands
    -------------------------------------------------------------------*/
    if (totbands2 == 0)
        nbands[0] = *totbands;

    /* Retrieve input geoTIFF file name
    -----------------------------------*/
    val = p_find(vblock, "INFILE");
    if (val->v_count == 1)
    {
        strcpy(path, SVAL(*val,0));

        /* Copy the root and directory path into two variables
        ---------------------------------------------------------*/
        c_splitname(path,dpath,root,ext);

        /* make sure the extension is ".tif" or empty.  If not, the filename 
           was probably entered without the real extension, but the filename
           has .'s in it */
        las_up2low(ext);
        if ((strlen(ext) != 0) && (strcmp(ext,"tif") != 0))
        {
            char temp_path[CMLEN];
            strcpy(temp_path, path);
            strcat(temp_path, ".tif");
            c_splitname(temp_path,dpath,root,ext);
        }
    }
    else
        strcpy(dpath,"");


    /* Retrieve output image name and so some error checking
    -------------------------------------------------------*/
    val = p_find(vblock,"OUT");
    *numimg_files = val->v_count;

    if (*numimg_files == 0)
        c_lasmsg(FATAL,"geotiffin-getpar","Must specify OUT");

    for (index = 0; index < *numimg_files; index++)
        strcpy(outimg[index],SVAL(*val,index));

    if ((totbands2 != 0) && (totbands2 != *numimg_files))
        c_lasmsg(FATAL,"geotiffin-getpar",
                "Total number of OUTS must equal total count of NBANDS");

    /* Using the TAE path name to outimg get the host datapath less the
       output image file name 
    -----------------------------------------------------------------*/
    c_cmhost(outimg[0],out_path);
    len = (long)strlen(strrchr(out_path,'/')+1);
    len2 = (long)((strlen(out_path))-len);

    /* make sure outpath is empty */
    strcpy(outpath, "");

    strncpy(outpath,out_path,len2);
    outpath[len2] = '\0';
}

