/******************************************************************************
NAME:           GTIF

PURPOSE: Read in tiff images from disk or CD and process them into bil
         files which will be used to ingest through TRANSLAS-IN

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement
2.0      02/00  D. Hames          C    Modify process_band by setting outfile 
                                       to empty. This was done to eliminate
                                       possible corrupt output file names.
                                       SRF#3345
3.0      03/00  D. Hames          C    Add Arc 1960 to get_datum. CR#3454. 
                                       Modify get_zone_datum and get_utmzonegen 
                                       to read the geotiff keys instead of the
                                       upper left corner coordinates for zone 
                                       location. Modify process_band to output 
                                       the bil file in lower case. CR#3474
4.0      04/00  D. Hames          C    Add warning messages to the functions
                                       that extract the projection information 
                                       from the geoTIFF keys. These messages
                                       will warn the user if there are any
                                       missing geoTIFF keys. Also modify these
                                       functions to change the DDR to invalid
                                       if there is required information missing
                                       from the DDR.  CR#3553
5.0      04/00  D. Hames          C    Modify gtif and sort_bands by setting 
                                       nbands array and num_bands array to zero
                                       Set proj units to vaild in get_zone_datum
                                       get_stateplanezone CR#3657. Modify 
                                       process_band and bip2bil to support 
                                       multiband tiled images in contiguous 
                                       format. CR#3677.
6.0      09/00  D. Hames          C    Add NAD 27 to get_datum.
7.0		 09/04	F. Meng			  C    modify gtif by comparing the first tif file
									   found with input file, if not match, use
									   the input file. 
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "geotiffin.h"

int main 
(void)

{
    struct PARBLK vblk;          /* TAE parameter block                     */
    char btemp[MAXBND];          /* variable used to hold band from 
                                    char_bands                              */
    char char_bands[MAXBND][CMLEN]; /* array of bands in string format      */
    char comment[50][CMLEN];     /* CD mount message to the operator        */
    char disk_sceneid[CMLEN];    /* The sceneid retrieved from the file
                                    names located on the disk or CD         */
    char dpath[CMLEN];           /* Path to CD or disk image                */
    char dpath_for_cd[CMLEN];    /* data path of CD                         */
    char device[CMLEN];          /* device CD was mounted on                */
    char files2process[MAX_FILE][CMLEN];/* geoTIFF files that will be 
                                           processed                        */
    char first_tiff_file[CMLEN]; /* first tiff file retrieved from getfiles */
    char finalouts[MAX_FILE][CMLEN]; /* final output image file names       */
    char ftemp[MAXBND];          /* variable used to hold band from 
                                    tiff_file_ary                           */
    char drive[CMLEN];           /* drive name that operator mounted CD     */
    char newouts[MAX_FILE][CMLEN];   /* new outimg used if a band doesn't 
                                        exist                               */
    char next_vol_bands[MAXBND][CMLEN];/* band array that will be used to 
                                          process bands on next CD          */
    char *out_root_ptr;          /* pointer to root of tiff file            */
    char out_dir[CMLEN];         /* directory of tiff file                  */
    char out_root[CMLEN];        /* root of tiff file                       */
    char out_ext[CMLEN];         /* extension of the tiff file              */

    char outfile[CMLEN];         /* Output BIL file for translas            */
    char outfiles[MAX_FILE][CMLEN];  /* bil files to output to tae          */
    char outimg[MAX_FILE][CMLEN];/* Names of output images                  */
    char outpath[CMLEN];         /* Path to output image and bil files      */
    char outtiffs[MAX_FILE][CMLEN];/* geoTIFF file names                    */
    char *ptr;                   /* pointer used to point to chars in 
                                    filenames                               */
    char *tiff_file_ptr;         /* pointer to 1st file found on CD or disk */
    char *string_ptrs[MAX_FILE]; /* used to return variables to TAE         */
    char temp[CMLEN];            /* temp string used to hold file name      */
    char temp2[CMLEN];           /* temp string used to hold band number    */ 
    char temp_dpath[CMLEN];      /* temp path used to retrieve geoTIFF files*/
    char tiff_file_ary[MAX_FILE][CMLEN];/* files retrieved from directory   */ 
    char tmpexten[5];            /* extension used for a multi resolution 
                                    out                                     */ 
    char tmpstr[MAX_FILE][CMLEN];/* temporary string used to hold odtype    */ 
    char usr_sceneid[CMLEN];     /* sceneid supplied by user                */ 
    double pixel[2];             /* resolution of the image                 */ 
    double resolx[MAX_FILE];     /* resolutions, one for ea bil file        */ 
    double resoly[MAX_FILE];     /* resolutions, one for ea bil file        */
    int  mounted;                /* flag indicating if the CD is mounted    */
    long bandnum[MAXBND];        /* band nums containing their true names   */
    long process_all_bands;      /* flag indicating if all bands need to 
                                    be processed                            */
    long c=0;                    /* long value used for looping             */
    long count;                  /* total number of elements                */
    long file_length;            /* length of tif file - excludes directory */
    long files2process_totnum;   /* total number of files to be processed   */
    long group[MAX_FILE];        /* assigned grp #'s to output to tae       */
    long i=0;                    /* long value used for looping             */
    long in_dtype[MAX_FILE];     /* data types to be output to tae          */
    long lan7=FALSE;             /* flag indicating Landsat 7 data          */
    long len;                    /* used to hold the length of a string     */
    long long_bands[MAXBND];     /* bands supplied by user                  */
    long maxnum=1;               /* variable used for looping               */
    long mode=HANDLE;            /* error message option                    */ 
    long n=0;                    /* long value used for looping             */ 
    long nbands[MAXBND];         /* number of bands to be copied to each 
                                    image file                              */ 
    long new_bandnum[MAXBND];    /* new bandnum used if band doesn't exist  */ 
    long new_nbands[MAXBND];     /* new nbands used if a band doesn't exist */ 
    long next_vol_bands_totnum=0;/* total number of bands in next_vol_bands */ 
    long num=0;                  /* long value used for looping             */
    long num_bands[MAXBND];      /* total bands to be output to tae         */
    long num_finalouts;          /* total number of final output images     */
    long num_lines[MAX_FILE];    /* lines to be output to tae               */
    long num_samps[MAX_FILE];    /* samples to be output to tae             */
    long numimg_files;           /* Number of output image files            */
    long p=0;                    /* long value used for looping             */
    long q=0;                    /* long value used for looping             */
    long r=0;                    /* long value used for looping             */
    long rewopt=REWOFFL;         /* rewind option                           */ 
    long root_length;            /* length of the root portion of tif file  */
    long s=0;                    /* long value used for looping             */ 
    long status;                 /* status returned by routines called      */ 
    long stdlun;                 /* Logical unit number for std output      */ 
    long temp_array[MAX_FILE][MAX_FILE]; /* temporary array of longs        */ 
    long tmp_long;               /* Dummy long to obtain lng value          */ 
    long tot_num_files=0;        /* Total number of files, used for looping */ 
    long tot_loops=0;            /* number of times needed to loop          */ 
    long totals=0;               /* long used for tracking totals           */ 
    long totbands;               /* total number of bands to be processed   */ 
    long unit;                   /* Tape unit number                        */ 
    long w=0;                    /* long value used for looping             */ 
    long zero=0;                 /* value of zero                           */
    long real_flag=0;            /* Flag for floating-point data 1=true     */
    tsize_t image_length;        /* number of lines down in the image       */
    tsize_t image_width;         /* number of bytes/samples across the image*/
    uint16 bitspersample;        /* number of bits per sample               */
    uint16 numbands;             /* number of bands in the image            */

    /*  Initialize TAE interface
    -----------------------------*/
    c_inlas(&vblk,&stdlun);

    /* Set nbands array to 0 
    --------------------------*/
    memset(nbands, 0, MAXBND*sizeof(long));

    /* Retrieve the input parameters
    ----------------------------------*/
    getpar(&vblk,comment,dpath,outpath,long_bands,&numimg_files,nbands, 
            outimg,usr_sceneid,&totbands,&process_all_bands);

    /* If the user specified neither a COMMENT nor an INFILE, error out.
    ------------------------------------------------------------------------*/
    if ((strlen(dpath) == 0) && (strlen(usr_sceneid)==0) 
        && (strlen(comment[0]) == 0))
    {
        c_lasmsg(FATAL, "geotiff-comment",
                "Either COMMENT or INFILE must be specified");
    }

    /* Initialize to empty
    ---------------------*/
    strcpy (disk_sceneid, "");

    /* The following loop is necessary in order to support geoTIFF files
       located on multiple CD's.
    -------------------------------------------------------------------*/
    do     
    {
        /* If COMMENT is specified then retrieve the CD's from it
           and ask the operator to mount.
        --------------------------------------------------------*/
 
        if (strlen(comment[c]) != 0)
        {
            if(mount_cd(&vblk,&unit,comment[c],device,drive,
                &mounted) != TAPE_SUCC)
            {
                return (E_FAIL);  
            } 
 
            /* after the CD is mounted use the device for the dpath */
            strcpy(dpath,device);
        }


        /* Retrieve list of geoTIFF files from data path supplied by user
        -----------------------------------------------------------------*/

        /* Establish the sceneid or root of the first file found by using 
        the information supplied by the user. Once this is done, do not
        re-establish it if another CD is mounted. For example, if the
        user specifies "l7" for infile and another CD is mounted, the first
        file on that CD beginning with "l7" will be retrieved and its sceneid
        will be used. This will cause problems if multiple scenes are found on 
        that CD */

        if (disk_sceneid[0] == '\0')
        {
            strcpy(temp_dpath, dpath);
            if (dpath[0] != '\0') /* A data path was provided in INFILE */
            {
                /* Add sceneid to the directory path so c_getfiles can retrieve 
                   a list containing only those geoTIFF file names */
                strcat(temp_dpath,usr_sceneid);
            } 

            strcat(temp_dpath,"*.[Tt][Ii][Ff]");
            if (c_getfiles(temp_dpath,tiff_file_ary,&tot_num_files,
                MAX_FILE,1,0) != E_SUCC)
            {
                /* c_getfiles supplies the error message */
                return (E_FAIL);
            }

            if (tot_num_files >= 1)
            {
                /* Use the first file found out of all the files retrieved */
                tiff_file_ptr = strrchr (tiff_file_ary[0],'/'); 

                if (tiff_file_ptr == NULL)
                    tiff_file_ptr = &tiff_file_ary[0][0];

                else
                    tiff_file_ptr++; /* to step past '/' */

                strcpy(first_tiff_file,tiff_file_ptr); 
            }
            else
            {
                c_lasmsg(NONFATAL,"geotiffin-fatal", 
                       "Error finding GeoTIFF files located in path %s", dpath);
                c_lasmsg(FATAL,"geotiffin-fatal", 
                        "using the file name that begins with %s", usr_sceneid);
            }

			/* If the file specified is not the first file found
			 * make the first file as the file specified, CR10023
			 * ----------------------------------------------------*/
			if (strstr(first_tiff_file, usr_sceneid) ==  NULL)
				strcpy(first_tiff_file, usr_sceneid);

            /* Landsat 7 file naming convention is l7fppprrr_rrrYYYYMMDD_AAA.tif
               l7 = Landsat 7 mission
               f = EMT+ format (1 or 2)
               ppp = starting path
               rrr_rrr = starting and ending rows
               YYYYMMDD = acquisition date 
               AAA = band
               Check if the first file retrieved from getfiles is a Landsat 7 
               image.  If it is, only the sceneid is required.  Landsat 7 
               products contain 1 band per .tif file. If there are additional 
               .tif files in the directory with this sceneid they will be 
               processed according to the user band specifications. The sceneid
               format is l7fppprrr_rrrYYYYMMDD
            -----------------------------------------------------------------*/ 
            if ((strncasecmp(first_tiff_file,"L7",2) == 0) && 
                ((tiff_file_ptr[22] == 'B') || (tiff_file_ptr[22] == 'b')) && 
                (strlen(first_tiff_file) == 29) && 
                (first_tiff_file[9] == '_'))
            {
                lan7 = TRUE;
                /* Copy first 21 chars to get the Landsat 7 sceneid. Do not
                   include _AAA.tif */
                strncpy(disk_sceneid,first_tiff_file,21);
                disk_sceneid[21] = '\0';

                /* The Landsat 7 sceneid is the same for all bands except for  
                   the EMT+ format. f=1 (channel 1) for bands 1 through 6l 
                   and f=2 (for channel 2) for bands 6h through 8. Both naming
                   conventions need to be recognized */
                disk_sceneid[2] = '*';
            }
            /* Not a Landsat 7 image. Use the user input sceneid */
            else 
            {
                if (strcmp (usr_sceneid,"") != 0)
                    strcpy(disk_sceneid,usr_sceneid); 
                else
                {
                    /* INFILE was left blank so retrieve the sceneid from
                       the root portion of the first tif file */
                    file_length = (long)strlen(tiff_file_ptr);
                    if (strrchr (tiff_file_ptr,'_') == 0)
                    {
                        /* Since there isn't an '_' in the filename then we 
                           assume there aren't any other tif files with band 
                           numbers. Take only the first file and set
                           tot_num_files to 1. */
                           tot_num_files = 1;
                    }
                    else
                    {
                        root_length = (long)strlen(strrchr (tiff_file_ptr,'_'));
                        strncpy(disk_sceneid, first_tiff_file,
                                file_length-root_length);
                        disk_sceneid[file_length-root_length] = '\0';
                    }
                }
            }
           
            /* If only 1 file was found then use that file as the data path */
            if (tot_num_files == 1)
                strcpy(dpath, tiff_file_ary[0]);
        }

        if (tot_num_files != 1) 
        {
            /* Now that we have a sceneid that is found on the disk or CD, use 
               c_getfiles again in order to retrieve only the files with this
               sceneid. Also, there is no need to run c_getfiles again if 
               only one file is found.  That way a user can get by with 
               entering one Landsat 7 image without having to specify its band 
               in the parameter BANDS.
            ------------------------------------------------------------------*/
            /* Concat the sceneid to the data path */
            strcat(dpath,disk_sceneid);
            strcat(dpath,"*.[Tt][Ii][Ff]");

            if (c_getfiles(dpath,tiff_file_ary,&tot_num_files,
                MAX_FILE,1,0) != E_SUCC)
            {
                /* c_getfiles supplies the error message */
                return (E_FAIL);
            }
        }

        strcpy(dpath_for_cd,dpath); 

        /* If more than one file was found using the user input sceneid and
           it is not a Landsat 7 image, make sure an _b exists in the file
           name. If it does then process all the bands (in the case were 
           bands were not specified). If it doesn't then process only the 
           first file in the tiff_file_ary array.
        -------------------------------------------------------------------*/
        if ((tot_num_files > 1) && (lan7 == FALSE))
        {
            c_splitname(tiff_file_ary[0],out_dir,out_root,out_ext);
            out_root_ptr = strrchr (out_root,'_');
            if (out_root_ptr == NULL)
                tot_num_files = 1;
            else
            {
                out_root_ptr++; /* to step past '_' */
                if ((out_root_ptr[0] != 'b') && (out_root_ptr[0] != 'B'))
                    tot_num_files = 1;
            }
        }    
                
        /* If the user specifies no bands, then all bands will be processed
           Copy all geoTIFF's in tiff_file_ary to files2process.
        --------------------------------------------------------------------*/
        files2process_totnum = 0;
        if (totbands == 0)
        {
            for (i=0; i<tot_num_files; i++)
            {
				//if (strstr(tiff_file_ary[i], usr_sceneid)
                strcpy(files2process[i], tiff_file_ary[i]);
                files2process_totnum++;
            } 
        }

        /* Retrieve the bands if they were specified 
        ---------------------------------------------*/
        if(next_vol_bands_totnum == 0)    
        {
            if (lan7 == TRUE)
            { 
                /* These are the required band names for landsat7 data */
                for (i=0; i < totbands; i++)
                {
                    if ((long_bands[i]<=5) || (long_bands[i]==7) || 
                       (long_bands[i]==8)) 
                    {
                        sprintf(char_bands[i],"%ld",long_bands[i]);
                        strcat(char_bands[i],"0");
                    }
                    else if (long_bands[i] == 6) 
                    {
                        sprintf(char_bands[i],"%ld",long_bands[i]);
                        strcat(char_bands[i],"1");
                    }
                    else if (long_bands[i] == 61)
                    {
                        strcpy(char_bands[i],"61");
                    }  
                    else if ((long_bands[i] == 9) || (long_bands[i] == 62))
                    {
                        strcpy(char_bands[i],"62");
                    }
                    else
                    {   /* Since a Landsat 7 file specifies a band using
                           ie 40 instead of 4, geotiffin will accept this as 
                           valid band number. We don't want this. We only want
                           to accept 4 as a valid band number.
                           Warn the user of the invalid band number and set a 
                           dummy value in the char_bands array */
                        c_lasmsg(NONFATAL, "geotiffin-band",
                                "Warning band %d does not exist",long_bands[i]);
                        sprintf(char_bands[i],"%ld",DUMMY_BAND);
                    }
                                 
                } 
            } 
            else 
            {
                for (i=0; i < totbands; i++)
                {
                    sprintf(char_bands[i],"%ld",long_bands[i]);
                }  
            }

            /* Copy all char_bands to bandnum which will be passed to tae
            ---------------------------------------------------------------*/
            for (i=0; i < totbands; i++)        
            {
                bandnum[i] = (long)atol(char_bands[i]);
            }
        }
        /* The information in next_vol_bands will be copied to char_bands. 
           This will be used to process geoTIFFs on next CD 
        ---------------------------------------------------------------------*/
        else
        {
            for(q=0; q<MAXBND; q++)
            {
                strcpy(char_bands[q],"");
                strcpy(char_bands[q],next_vol_bands[q]); 
                strcpy(next_vol_bands[q],"");
            }
            totbands = next_vol_bands_totnum;
            next_vol_bands_totnum=0; 
        } 
 
        /* Now that char_bands array is filled with the bands the user 
           requested, compare char_bands[r] to each file in tiff_file_ary.  
           If they match copy geoTIFF to files2process.  If not, go to next 
           band in char_bands[r] and compare to each file in tiff_file_ary 
           until all the bands requested are copied to files2process. If 
           char_band contains a band that does not exist in tiff_file_ary 
           then copy that band to next_vol_bands.  This will be used after 
           the next CD is mounted
        -------------------------------------------------------------------*/
        i=0;
        w=0;
        r=0;
        p=0;
        n=0; 
        strcpy(ftemp,"");
        while (r < totbands)
        {
            len = (long)strlen(char_bands[r]);
            strncpy(btemp,char_bands[r],len);
            btemp[len] = '\0';
            if ((w >= tot_num_files)  && (strcmp(ftemp,btemp) != 0))
            {
                strcpy(next_vol_bands[n],char_bands[r]);
                next_vol_bands_totnum++;
                r++;
                n++;
                i=0;
                w=0; 
                /* char_bands has been incremented by one. Get length of that
                   band */
                len = (long)strlen(char_bands[r]); 
                strncpy(btemp,char_bands[r],len);
                btemp[len] =  '\0';
            }     
            i=0;
            while (i < tot_num_files)
            {
                ptr = (char *) strrchr (tiff_file_ary[w],'_');
                if (ptr == NULL)
                    c_lasmsg(FATAL,"geotiffin-fatal",
                            "Invalid band number in geoTIFF file name");
                else
                {  
                    if (strlen(ptr) < (len+2))
                        c_lasmsg(FATAL,"geotiffin-fatal",
                                "Invalid band number in geoTIFF file name");
                    else  
                    {
                    /* At this point the pointer will be at "_bx.tif",
                       "_bxx.tif" etc. We want the only the band number 
                       (x or xx) copied. */
                        len = (long)strlen(ptr)-2;  /* -2 to not include _b */
                        strncpy(ftemp,ptr+2,len-4); /* +2 so _b is not copied */
                                                    /* -4 to not copy .tif  */
                        ftemp[len-4] = '\0';
                    }
                }   
                if(strcmp(ftemp,btemp) == 0)
                {
                    strcpy(files2process[p],tiff_file_ary[i]);
                    files2process_totnum++;   
                    i++;
                    r++; 
                    p++;    
                    w=0; 
                    break;
                } 
                else
                {
                    i++;
                    w++;
                }
            }
        }

        /* Process geoTIFF files, one at a time 
        ------------------------------------*/
        for (i=0; i < files2process_totnum; i++)
        {
            process_band(files2process[i],outfile, pixel, &bitspersample,
                    &image_width,&image_length,&numbands,&real_flag,outpath);

            /* Copy the output information from process_band into arrays that
               will be passed to TAE.
            ----------------------------------------------------------------*/
            strcpy(outtiffs[num],files2process[i]);
            strcpy(outfiles[num],outfile);
            num_samps[num]=image_width;
            num_lines[num]=image_length;
            in_dtype[num]=bitspersample;
            resolx[num]=pixel[0];
            resoly[num]=pixel[1];
            num_bands[num]=numbands;
            num++;
        }

        /* count is the total number of elements that will be passed to TAE 
        ------------------------------------------------------------------*/
        count = num; 

        /* we need empty arrays in case it's necessary to loop though again 
        -----------------------------------------------------------------*/
        for(q=0; q<MAX_FILE ;q++)
        {
            strcpy(tiff_file_ary[q],"");
            strcpy(files2process[q],"");
        }

        if (mounted == 1)
        {
            /* all necessary information has been read from the individual
               geoTIFF files (ie resolution, nl, ns).  One CD needs to remain 
               open to read the geoTIFF file for the common geographic 
               information, which is used to update the ddr 
            ------------------------------------------------------------*/    

            /* If the user did not specify bands then all bands will be 
               processed on all CD's that were requested */
            if ((totbands == 0) && (strlen(comment[c+1]) != 0))
            {
                status = c_tpclos(&unit,&mode,&rewopt,&zero);
                if (status != TAPE_SUCC)
                    c_lasmsg(NONFATAL,"geotiffin-nonfatal",
                            "Error closing CD");    
                mounted = 0;
            }

            /* if bands have been specified, use next_vol_bands and comment to 
               close the CD, all but the last one */
            else if (totbands != 0)
            {
                if ((next_vol_bands_totnum != 0) && (strlen(comment[c+1]) != 0))
                {
                    status = c_tpclos(&unit,&mode,&rewopt,&zero);
                    if (status != TAPE_SUCC)
                        c_lasmsg(NONFATAL,"geotiffin-nonfatal",
                                "Error closing CD");
                    mounted = 0;
                }
            }
            c++;
        }
        tot_num_files = 0;
    }
    /* If the previous CD is not mounted AND there is request to mount another 
       CD. AND there are bands specified by the user that were not found on the
       previous CD OR the request is to process all bands on all CD's, then loop
    -------------------------------------------------------------------------*/
    while ((mounted == 0) && (strlen(comment[c]) != 0) &&
           ((next_vol_bands_totnum != 0) || (process_all_bands == TRUE)));
    /* end of do while statement */

    /* if there are bands the user requested and they do not exist, update the
       nbands, bandnum, and outimg by creating new ones. Also warn the user
       of the nonexisting bands.
    -------------------------------------------------------------------------*/
    if (next_vol_bands_totnum != 0)
    {
        nonexisting_bands(next_vol_bands,bandnum,nbands,count,outimg,newouts,
                new_nbands,new_bandnum,lan7,next_vol_bands_totnum);

        for(i=0;i<MAXBND;i++)
        {
            nbands[i] = 0;
            nbands[i] = new_nbands[i];
            bandnum[i] = 0;
            bandnum[i] = new_bandnum[i];
        }

        for(i=0;i<MAX_FILE;i++)
        {
            strcpy(outimg[i],"");
            strcpy(outimg[i],newouts[i]);
        }
    }

    /* if bands was not specified by user, we then need a list of all the 
       bands. Get the band numbers from the outfiles array (which contains all 
       files on all CD's or all files on disk and copy to bandnum which will
       be passed through tae
    -----------------------------------------------------------------------*/
    if (totbands == 0)
    {
        for (i=0; i < count; i++)
        {
            ptr = (char *) strrchr (outfiles[i],'/');
            if (ptr == NULL)
                c_lasmsg(FATAL,"geotiffin-fatal",
                        "Output bil file(s) do not exist");
            else
                strcpy(temp,ptr);
            /* if band number does not exit on file then leave char_bands 
               empty */
            if (strrchr(temp, '_') != NULL)   /* indication of a band number */
            {
                ptr = (char *) strrchr (temp, '_');
                if (ptr != NULL)
                    strcpy(temp2,ptr+2);
                len = (long)(strlen(temp2));   
                strncpy(char_bands[i],temp2,len-4); /*4 is the length of .bil*/
                bandnum[i] = (long)atol(char_bands[i]);
            }
        }
    }

    /* if nbands was not specified by the user, then nbands and totbands needs
       to be given the total number of bands that are on CD(s) or disk 
    ------------------------------------------------------------------------*/
    if (nbands[0] == 0)
        nbands[0] = count;
    if (totbands == 0)
        totbands = count;

    /* all bil files are processed now. Number of samples will determine the
       size of the image. Separate the num_samps array into a temporary two
       dimensional array. nbands will decide how the temp_array is grouped.
    ----------------------------------------------------------------------*/
    n=0;
    p=0;
    q=0;
    r=0;
    i=0;
    while (nbands[n] != 0)
    {
        while (i < nbands[n])
        {
            temp_array[q][r] = num_samps[p];
            i++;
            r++;
            p++;
        }
        n++;
        i=0;
        q++;
        r=0;
    }

    /* using the temporary array, assign a group number to each separate image 
       size.  Group will determine which files will be "grouped" together and
       ingested trough TRANSLAS-IN.
    -----------------------------------------------------------------------*/
    totals=0;
    r=0;
    tot_loops=nbands[r];
    i=1;                       /* group number */
    q=0;
    s=0;
    p=0;
    while((temp_array[r][q]) != 0) 
    {
        do
        {
            while (q <= tot_loops)
            {       
                if ((group[q+totals] == 0) && (temp_array[r][q] != 0))
                {
                    group[q+totals] = i;
                    i++;
                    break;
                } 
                else
                    q++;
            } 
            for (p=q; p < tot_loops; p++)
            {
                if (temp_array[r][q] == temp_array[r][p+1])
                    group[p+totals+1] = group[q+totals];
            }
            q=0;
            s++;
        }
        while(s < tot_loops);         /*end of do statement*/
        totals=nbands[r]+totals;
        r++;
        tot_loops=nbands[r];
        s=0;
    }

    /* The user may not know how many image sizes are in a scene, therefore  
       may not request enough outs. Since we cannot put two bands of different 
       sizes (or resolutions) in the same output image, then one will be 
       created using the "_r*" extension.
    -----------------------------------------------------------------------*/
    n=0;
    p=0;
    q=1;
    r=0;
    i=0;
    s=0;
    num=2;
    do
    {
        while(nbands[n] > i)
        {
            if (group[p] == q)
            {
                if (strlen(finalouts[s]) == 0)
                    strcpy(finalouts[s],outimg[r]);
            }
            else if(group[p] >= maxnum)
            {
                if (strlen(finalouts[s]) != 0)
                    s++;        
                strcpy(finalouts[s],outimg[r]);
                sprintf(tmpexten,"_r%d",num);
                strcat(finalouts[s],tmpexten);
                if (num == 2)
                {
                    sprintf(tmpexten,"_r%d",num-1);
                    strcat(finalouts[s-1],tmpexten);
                }
                num++;
                q++;
            }
            i++;
            if(group[p] > maxnum)
                maxnum = group[p];
            p++;
        }
        s++;
        n++;
        num=2;
        i=0;
        r++;
        q++;
    }
    while(group[p] != 0);

    num_finalouts = s;

    /* if outtiffs[0] does not exist, then the file(s) requested by the 
       user does not exist
    --------------------------------------------------------------------*/
    if (strlen(outtiffs[0]) == 0)
    {
        c_lasmsg(FATAL,"geotiffin-fatal","Input geoTIFF file does not exist");
    }

    /* Output values to TAE
    -------------------------*/
    if(E_SUCC != lng2tae(&vblk,&lan7,1,1,"LAN7",&tmp_long))
    {
        c_lasmsg(FATAL, "geotiffin-lng2tae", 
                 "Error passing variable LAN7 to TAE");
    }
    if(E_SUCC != lng2tae(&vblk,num_samps,1,count,"NS",&tmp_long))
    {
        c_lasmsg(FATAL, "geotiffin-lng2tae", 
                 "Error passing variable NS to TAE");
    }
    if(E_SUCC != lng2tae(&vblk,num_lines,1,count,"NL",&tmp_long))
    {
        c_lasmsg(FATAL, "geotiffin-lng2tae", 
                 "Error passing variable NL to TAE");
    }
    if(q_real(&vblk,"RESOLX",count,resolx,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FATAL, "geotiffin-q_real", 
                 "Error passing variable RESOLX to TAE");
    }

    if(q_real(&vblk,"RESOLY",count,resoly,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FATAL, "geotiffin-q_real", 
                 "Error passing variable RESOLY to TAE");
    }

    if(E_SUCC != lng2tae(&vblk,num_bands,1,count,"NB",&tmp_long))
    {
        c_lasmsg(FATAL, "geotiffin-lng2tae", 
                 "Error passing variable NB to TAE");
    }

    if(E_SUCC != lng2tae(&vblk,group,1,count,"GROUP",&tmp_long))
    {
        c_lasmsg(FATAL, "geotiffin-lng2tae", 
                 "Error passing variable GROUP to TAE");
    }
    if(E_SUCC != lng2tae(&vblk,bandnum,1,count,"BANDNUM",&tmp_long))
    {
        c_lasmsg(FATAL, "geotiffin-lng2tae", 
                 "Error passing variable BANDNUM to TAE");
    }

    for (i = 0; i < count; i++)
        string_ptrs[i] = outfiles[i];
    if(q_string(&vblk, "BILFILE",count,string_ptrs,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FATAL, "geotiffin-q_string", 
                 "Error passing variable BILFILE to TAE");
    }

    for (i = 0; i < num_finalouts; i++)
        string_ptrs[i] = finalouts[i];
    if(q_string(&vblk,"FINALOUTS",num_finalouts,string_ptrs,P_UPDATE)!=SUCCESS)
    {
        c_lasmsg(FATAL, "geotiffin-q_string", 
                 "Error passing variable FINALOUTS to TAE");
    }

    string_ptrs[0] = dpath_for_cd;
    if(q_string(&vblk,"CD_DPATH",1,string_ptrs,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FATAL, "geotiffin-q_string", 
                 "Error passing variable FINALOUTS to TAE");
    }


    for(i=0; i<count;i++)
    {
        string_ptrs[i] = tmpstr[i];
        switch (in_dtype[i])
        {
            case 8  : strcpy(tmpstr[i],"BYTE");
                      break;
            case 16 : strcpy(tmpstr[i],"I*2");
                      break;
            case 32 : if( !real_flag ) {
                          strcpy(tmpstr[i],"I*4");
                      }
                      else {
                          strcpy(tmpstr[i],"R*4");
                      }
                      break;
        }
    }
    if(q_string(&vblk,"IDTYPE",count,string_ptrs,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FATAL, "geotiffin-q_string", 
                 "Error passing variable IDTYPE to TAE");
    }

    for (i = 0; i < count; i++)
        string_ptrs[i] = outtiffs[i];
    if(q_string(&vblk, "TIFFILE",count,string_ptrs,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FATAL, "geotiffin-q_string", 
                 "Error passing variable TIFFILE to TAE"); 
    }

    string_ptrs[0] = disk_sceneid;
    if(q_string(&vblk, "SCENEID",1,string_ptrs,P_UPDATE) != SUCCESS)
    {
        c_lasmsg(FATAL, "geotiffin-q_string", 
                "Error passing variable SCENEID to TAE");
    }

    if(q_out(&vblk) != SUCCESS)
        c_lasmsg(FATAL, "geotiffin-qout", "Error passing variables to TAE");

    return (E_SUCC);
}
