/******************************************************************************
NAME:	        MOUNT_CD

PURPOSE:        Physically mounts cd and returns the drive name.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS


*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>     /* gethostname prototype */
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "geotiffin.h"

int mount_cd
(
    struct PARBLK *vblk,/* I: TAE parameter block                     */
    long *unit,         /* O: cd unit number 	                      */
    char comment[],	/* I: message to the operator      	      */
    char device[],	/* O: device CD was mounted on                */
    char drive[],	/* O: drive name that operator mounted CD     */
    int  *mounted       /* O: mounted flag mounted=1,  not mounted=0  */
)
{

char density[5];        /* tape density in BPI                        */
char lhost[CMLEN];	/* local host of CD drive		      */ 
char *string_ptrs[1];   /* used to return variables to tae            */
char tapeid[CMLEN];     /* tape id                                    */
long access = TREAD;    /* open tape for read access                  */
long len;		/* temporary length variable	              */	
long mode = HANDLE;     /* tape I/O outputs error messages            */
long rewopt = NOREW;    /* rewind option                              */
long skip = 0;          /* skip zero files on open                    */
long status;            /* function return status code                */

strcpy(density,"0");

/* Mount the current CD volume. The comment returned containes either 
host@drive, if the CD is remote, and only drive is returned if it is local
-------------------------------------------------------------------------*/
status = c_tpopen(unit,&skip,&mode,&rewopt,density,&access,
                  comment, tapeid);
if (status != TAPE_SUCC)
   return(E_FAIL);

/* set mounted flag */
*mounted = 1;

strcpy(drive, comment);

if (strchr(drive,'@') == NULL)
   { /*this is a local drive */
   if (gethostname(lhost,CMLEN) !=0)
      c_lasmsg(FATAL,"geotiffin-hostname", "Error reading the hostname");
   len = (long)strlen(lhost);
   c_up2low(lhost,&len);
   sprintf(device,"/%s@%s/", lhost, drive);
   }
else /*this is a remote drive */
   {
   sprintf(device,"/%s/", drive);  
   }

/* return the mounted flag and drive right away so they can be unmounted
in the pdf if needed 
----------------------------------------------------------------------*/
if (q_intg(vblk,"MOUNTED",1,mounted,P_UPDATE) != SUCCESS)
    c_lasmsg(FATAL, "geotiffin-q_intg", "Error passing MOUNTED to TAE");

string_ptrs[0] = drive;
if(q_string(vblk, "DRIVE",1,string_ptrs,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-q_string", "Error passing variable DRIVE to TAE");

if(q_out(vblk) != SUCCESS)
   c_lasmsg(FATAL, "geotiffin-qout", "Error passing variables to TAE");

return(E_SUCC);
}

