/*******************************************************************************
NAME:           NONEXISTING_BANDS

PURPOSE:        To warn user of nonexisting bands and create new nband,
		bandnum and outimge parameters to refect the bands that 
		actually exist.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "geotiffin.h"

int nonexisting_bands
(
    char nonexist_bands[][CMLEN], /* I: array of nonexisting bands         */
    long bandnum[],	   /* I: band numbers containing their true names  */
    long nbands[],  	   /* I: num of bands requested by user	           */
    long totbands,         /* I: total number of bands to be processed     */
    char outimg[][CMLEN],  /* I: names of output images requested by user  */
    char newouts[][CMLEN], /* O: new output images, reflects nonexisting bands*/
    long new_nbands[],	   /* O: new nbands to reflect nonexisting bands   */
    long new_bandnum[],	   /* O: new bandnum to reflect nonexisting bands  */
    long lan7,             /* I: flag indicating landsat 7 data    	   */
    long nonexist_bands_totnum /*I: total number of bands in nonexist_bands*/
)

{
long c;			   /* variable used for looping			*/
long i;			   /* variable used for looping			*/
long p;			   /* variable used for looping			*/
long q;			   /* variable used for looping			*/
long temp_bandnum[MAXBND][MAXBND]; /* temp array used for bandnum       */
long tmp_nbands[MAXBND];   /* temporary array used for nbands		*/
long totals;		   /* new total number of nbands		*/


/* nonexist_bands contains all of the bands that the user requested but
do not exist. Warn user that these bands do not exist
----------------------------------------------------------------------*/
i=0;
if (nonexist_bands_totnum != 0)
   {
   while(bandnum[i] !=0)
       {
       for(q=0; q < nonexist_bands_totnum; q++)
           {
           if (bandnum[i] != (long)atol(nonexist_bands[q]))
              ;
           else if((bandnum[i] != 62) && (bandnum[i] != 61))
              {
              if (lan7 == 1)
                  c = bandnum[i]/10;
              else
                  c = bandnum[i];
              /* only issue a warning if the band is a valid number (not a 
                 dummy) but does not exist */
              if (bandnum[i] != DUMMY_BAND) 
                 {
                 c_lasmsg(NONFATAL,"geotiffin-band",
                 "Warning band %d does not exist",c);
                 }
              }
           else if(bandnum[i] == 61)
              {
              c_lasmsg(NONFATAL,"geotiffin-band",
              "Warning band 6 does not exist");
              }
           else if(bandnum[i] == 62)
              {
              c_lasmsg(NONFATAL,"geotiffin-band",
              "Warning band 9 does not exist");
              }
           }
           i++;
       }
   }


/* initialize tmp_nbands and temp_bandnum to 0
---------------------------------------------*/
for(i=0;i<MAXBND;i++)
   tmp_nbands[i] = 0;

for(p=0;p<MAXBND;p++)
   {
   for(i=0;i<MAXBND;i++)
      temp_bandnum[p][i] = 0;
   }   


/* separate the bandnums temporary array (tmp_bandnum) according to 
the original nbands the user requested
-------------------------------------------------------------------*/
if (nonexist_bands_totnum != 0)
   {
   i=0;
   q=0;
   while(nbands[i] != 0)
      {
      p=0;
      c=0;
      while(nbands[i] > p)
         {
         temp_bandnum[i][c] = bandnum[q];
         c++;
         p++;
         q++;
         }
      i++;
      }
   }
/* using the array of bands not found (nonexist_bands) compare that to 
the temp_bandnum and "zero out" the bands that do not exist
-----------------------------------------------------------------------*/
p=0;
q=0;
while (q < nonexist_bands_totnum)
   {
   i=0;
   while ((temp_bandnum[p][i]) != 0) 
      {
      if ((temp_bandnum[p][i]) == (long)atol(nonexist_bands[q]))
         {
         temp_bandnum[p][i] = 0;
         q++;
         i++;
         }
      else
         {
         i++;
         }
      }
   p++;
   }


/* total the new total number of nbands that exist and copy it to a
temporary nbands array
------------------------------------------------------------------*/
p=0;
while(nbands[p] != 0)
   {
   totals=0;
   for(i=0; i < totbands+nonexist_bands_totnum; i++) /*Want total count of 
                                                    bands requested */
      {
      if((temp_bandnum[p][i]) != 0)
         totals++;
      }
   tmp_nbands[p] = totals;
   p++;
   }

/* now copy the temp nbands to the new nbands and the outimg to the new outimg
-----------------------------------------------------------------------------*/
p=0;
i=0;
while(nbands[p] != 0)
   {
   if(tmp_nbands[p] != 0)
     {
     new_nbands[i] = tmp_nbands[p];
     strcpy(newouts[i],outimg[p]);
     i++;
     }
   p++;
   }

/* now copy the temp_bandnum to new_bandnum
------------------------------------------*/
q=0;
for(p=0; p<totbands+nonexist_bands_totnum; p++)
   {
   for(i=0; i<totbands+nonexist_bands_totnum; i++)
      {
      if((temp_bandnum[p][i])  != 0) 
         {
         new_bandnum[q] = temp_bandnum[p][i];
         q++;
         }
       }
   }

return (E_SUCC);
}

