/******************************************************************************

NAME: recalc_corners

PURPOSE: Recalculate the corner points for a subwindow of an image.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "geotiffin.h"

int recalc_corners
(
    double corners[],         /* I/O: corner points of the image          */
    long origsize[],          /* I: original size of the image NL,NS      */
    double window[]           /* I: image window specified                */
)

{
double coefs[6];              /* transformation coefficients              */
double proj_coords[8];        /* image corner projection coordinates      */
long flag = TRUE;             /* c_rtcoef error flag                      */
long img_coords[8];           /* image corner line,sample coordinates     */
long line;                    /* current line                             */
long samp;                    /* current sample                           */
long status;                  /* function return status                   */

/* Copy the corner points in the order c_rtcoef wants.
The corners are (x,y) in UL, UR, LL, LR order
--------------------------------------------------*/
proj_coords[0] = corners[1]; proj_coords[1] = corners[0]; /* UL */
proj_coords[2] = corners[5]; proj_coords[3] = corners[4]; /* LL */
proj_coords[4] = corners[3]; proj_coords[5] = corners[2]; /* UR */
proj_coords[6] = corners[7]; proj_coords[7] = corners[6]; /* LR */

/* set up the image coordinates
---------------------------- */
img_coords[0] = img_coords[1] = 1;                        /* UL */
img_coords[2] = origsize[0];                              /* LL */
img_coords[3] = 1;
img_coords[4] = 1;                                        /* UR */
img_coords[5] = origsize[1];
img_coords[6] = origsize[0];                              /* LR */
img_coords[7] = origsize[1];

/* compute the transformation coefficients
----------------------------------------*/
status = c_rtcoef(proj_coords, img_coords, coefs, &flag);
if(status == E_FAIL)
{
    c_lasmsg(FATAL,"geotiffin-rtcoef",
       "Error computing the transformation coefficients");
}

/* now compute the new corner points
---------------------------------*/
/* UL */
line = (long)window[SL];
samp = (long)window[SS];
corners[0] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
corners[1] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];

/* UR */
samp += (long)window[NS] - 1;
corners[2] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
corners[3] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];

/* LL */
line = (long)window[SL] + (long)window[NL] - 1;
samp = (long)window[SS];
corners[4] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
corners[5] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];

/* LR */
samp += (long)window[NS] - 1;
corners[6] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
corners[7] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];

return (E_SUCC);
}
