/******************************************************************************
NAME:	        SB_GETPAR

PURPOSE:        Get the input parameters for sort_bands

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "geotiffin.h"

void sb_getpar
(
    struct PARBLK *vblock,      /* I: TAE parameter block               */
    char bil_files[][CMLEN],    /* O: array of bil files to be sorted   */
    char tif_files[][CMLEN],    /* O: array of geoTIFF files to be sorted  */
    long band_nums[],           /* O: array of sorted band numbers      */
    long *window		/* O: window area			*/
)

{
struct VARIABLE *val;           /* pointer to a TAE variable         */
long  index;                    /* loop variable                     */

/* Retrieve bil files
--------------------*/
val = p_find(vblock, "BIL_FILES");
for (index = 0; index < val->v_count; index++)
    {
    strcpy(bil_files[index],SVAL(*val,index));
    }

/* Retrieve geoTIFF files
------------------------*/
val = p_find(vblock, "TIF_FILES");
for (index = 0; index < val->v_count; index++)
    {
    strcpy(tif_files[index],SVAL(*val,index));
    }

/* Retrieve band numbers
-----------------------*/
val = p_find(vblock, "BAND_NUMS");
if(val->v_count != 0)
   for (index = 0; index < val->v_count; index++)
       band_nums[index] = IVAL(*val,index);


/* Retrieve window area to be copied
----------------------------------*/
val = p_find(vblock,"OUTWINDOW");
if(val->v_count > 0)
   {
   for (index = 0; index < 4; index++)
       window[index] = IVAL(*val,index);
   }
else
   {
   val = p_find(vblock,"WINDOW");
   if(val->v_count > 0)
     {
     for (index = 0; index < 4; index++)
       /* window[index] = ((double)RVAL(*val,index)); */
          window[index] = (long)((double)RVAL(*val,index));
     }
   else
     {
     for (index = 0; index < 4; index++)
         window[index] = 0;
     }
   }
}


