/*************************************************************************
NAME: SORT_BANDS

PURPOSE:
     Sort tif and bil files in ascending order 

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "geotiffin.h"


void main
(void) 
{
struct PARBLK vblk;             /* TAE parameter block                  */
char bil_files[MAX_FILE][CMLEN];/* array of bil files before sorted	*/
char directory[CMLEN];          /* Directory portion of bil_files       */
char root[CMLEN];		/* Root portion of file name		*/
char extension[CMLEN];		/* Extension portion of the file name   */
char ftemp[CMLEN];              /* band number in file                  */
char *ptr;			/* pointer used to point to chars in filenames*/
char sortedbils[MAX_FILE][CMLEN];/* array of sorted bil files		*/
char sortedtifs[MAX_FILE][CMLEN];/* array of sorted geoTIFF files	*/
char *string_ptrs[MAX_FILE];    /* used to output varialbles to tae     */
char tif_files[MAX_FILE][CMLEN];/* array of tif files before sorted	*/
char tmpstr[CMLEN];		/* temporary window string		*/ 
int  cur_band;                  /* current band number                  */
long band_nums[MAXBND];         /* array of sorted band numbers         */
long i=0;			/* variable used for looping		*/
long k=0;                       /* variable used for looping            */
long p=0;			/* variable used for looping		*/
long std_out;       		/* logical unit number for std output   */
long tmp_long = 0;		/* standard out				*/
long totnum=0;                  /* total number of files in the array	*/
long window[4];			/* windowed area of image 		*/


/*  Initialize TAE interface
-----------------------------*/
c_inlas(&vblk,&std_out);   

/* Set band_nums array to 0
---------------------------*/ 
memset(band_nums, 0, MAXBND*sizeof(long));

/* Retrieve the input parameters
--------------------------------*/
sb_getpar(&vblk,bil_files,tif_files,band_nums,window);  

/* If the band numbers exist sort file in ascending order
-------------------------------------------------------*/
if (band_nums[0] != 0)
   {
   while (band_nums[i] != 0)
         {
         c_splitname(bil_files[p],directory,root,extension);
         ptr = (char *) strrchr(root, '_');
         if (ptr == NULL)
            c_lasmsg(FATAL,"geotiffin-fatal", 
		      "Invalid band specification in output bil file name");
         else
            strcpy(ftemp,ptr+2);

         /* make sure the band specification is in proper format */
         k = 0;
         while(ftemp[k] != '\0')
         {
            if(isdigit(ftemp[k]) == 0)
            {
               c_lasmsg(FATAL,"geotiffin-fatal",
                        "Invalid band specification in output bil file name");
            }
            k++;
         }

         /* convert the string band number from the bil file name to an int */
         if(sscanf(ftemp, "%d", &cur_band) < 1) /* 0 or EOF indicates fail */
         {
            c_lasmsg(FATAL,"geotiffin-fatal",
                     "Invalid band specification in output bil file name");
         }

         if (cur_band == band_nums[i])
	    {
            strcpy(sortedbils[i],bil_files[p]);
	    strcpy(sortedtifs[i],tif_files[p]);
    	    i++;
	    p=0;
            }
         else
	    p++; 
         }
   totnum = i;
   }
/* if bands do not exist, then there is only one geoTIFF file with no band
specification in the geoTIFF name. Sorting is not necessary 
---------------------------------------------------------------------*/
else
   {
   strcpy(sortedbils[0],bil_files[0]);
   strcpy(sortedtifs[0],tif_files[0]);
   totnum = 1;
   }

/* add window to bil file name if out window exists
--------------------------------------------------*/
if (window[0] != 0)
   {
   i=0;
   sprintf(tmpstr,"%ld %ld %ld %ld",window[0],window[1],window[2],window[3]);
   while (strlen (sortedbils[i]) != 0)
	 {
         strcat(sortedbils[i],"(");
         strcat(sortedbils[i],tmpstr);
         strcat(sortedbils[i],")");
	 i++;
	 } 
   }

/* Output values to TAE
----------------------*/
for (i = 0; i < totnum; i++)
    string_ptrs[i] = sortedbils[i];
if(q_string(&vblk,"SORTEDBILS",totnum,string_ptrs,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "sort_bands-q_string", "Error passing SORTEDBILS to TAE");

for (i = 0; i < totnum; i++)
    string_ptrs[i] = sortedtifs[i];
if(q_string(&vblk,"SORTEDTIFS",totnum,string_ptrs,P_UPDATE) != SUCCESS)
   c_lasmsg(FATAL, "sort_bands-q_string", "Error passing SORTEDTIFS to TAE"); 

if (E_SUCC != lng2tae(&vblk,window,1,4,"NEWWINDOW",&tmp_long))
   c_lasmsg(FATAL, "sort_bands-lng2tae", "Error passing NEWWINDOW to TAE");

if(q_out(&vblk) != SUCCESS)
   c_lasmsg(FATAL, "gtif-qout", "Error passing variables to TAE");

}
