/******************************************************************************
FUNCTION:	GETDRIVE


PURPOSE:	Get a tape drive.  Prompt the operator for a tape drive,
		open the tape with the specified density and access, and
		skip the specified number of files.

PROGRAM HISTORY:
  Version	Date       Author      Request
  -------	----	   ------      -------
    5.0		01/92	   D. Hollaren Original development
    5.1		08/94	   D. Etrheim  Modify to allow for longer drive names 

ALGORITHM:
******************************************************************************/

#include <stdio.h>
#include "tae.h"
#include "worgen.h"
#include "tapeio.h"


main()
{
long	unit,			/* Tape drive unit number		*/
	status,			/* Return status variable		*/
	std_out,		/* Standard output			*/
	rewopt,		 	/* Rewind flag                          */
	access,			/* Open tape for write access        	*/
	skip,			/* Number of files on tape to skip	*/
	zero = 0,		/* Constant zero			*/
	mode = HANDLE,		/* Error mode                           */
	fatal = -1;		/* Fatal error code			*/

char	comment[CMLEN],		/* Message to the operator              */
	density[5],		/* Tape density				*/
	drive[CMLEN],		/* Tape drive				*/
	tapeid[CMLEN],		/* Tape id				*/
	out[1][CMLEN];		/* Output files created			*/

char	*array[1];		/* Tape drive assigned			*/

struct PARBLK vblock;		/* TAE parameter block                	*/
struct VARIABLE	*p_find();	/* function to find a TAE variable    	*/
struct VARIABLE	*val;		/* pointer to a TAE variable 	      	*/

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);

/*
  Retrieve the comment that is to be sent to the operator
*/
val = p_find(&vblock,"COMMENT");
strcpy(comment,SVAL(*val,0));

/*
  Retrieve the number of files on tape to skip
*/
val = p_find(&vblock,"SKIPNUM");
skip = IVAL(*val,0);

/*
  Retrieve the density the tape is to be mounted with
*/
val = p_find(&vblock,"TAPEDENS");
strcpy(density,SVAL(*val,0));

/*
  Retrieve the type of tape access desired
*/
val = p_find(&vblock,"ACCOPT");
if (strcmp(SVAL(*val,0),"READ") == 0)
    access = TREAD;
else if (strcmp(SVAL(*val,0),"WRITE") == 0)
    access = TWRITE;
else 
    access = TUPDATE;

/*
  Retrieve the rewind option
*/
val = p_find(&vblock,"REWOPT");
if (strcmp(SVAL(*val,0),"REWIND") == 0)
    rewopt = REWIND;
else
    rewopt = NOREW;

val = p_find(&vblock,"DRIVE");
if ((*val).v_count > 0)
    strcpy(drive,SVAL(*val,0));
else
    drive[0] = '\0';

if (strlen(drive) == 0)
    {
    /*
      Open the tape drive.  Comment gets wiped out when the tape drive
      is assigned
    */
    status = c_tpopen(&unit,&skip,&mode,&rewopt,density,&access,comment,tapeid);
    if (status != TAPE_SUCC)
	c_errmsg("Fatal error encountered","getdive-fatal",&fatal);

    /*
      Return the drive name to the parameter block
    */
    array[0] = comment;
    q_string(&vblock,"DRIVE",1,array,P_UPDATE);

    /*
      Return the tape id to the parameter block
    */
    array[0] = tapeid;
    q_string(&vblock,"TAPEID",1,array,P_UPDATE);

    /*
      Send the variable block to TAE
    */
    q_out(&vblock);
    }
else
    {
    /*
      Re-open the tape device
    */
    status = c_tpreop(&unit,&skip,&mode,&rewopt,density,&access,drive);
    if (status != TAPE_SUCC)
	c_errmsg("Fatal error encountered","getdive-fatal",&fatal);
    }

/*
  Display a completion message
*/
c_complt("GETDRIVE",out,&zero);
}
