#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"	
#include "util.h"

#define MAXLIBS 50		/* Maximum number of TAE libraries            */


int main()
{
char format[CMLEN];		/* Format of output (COM=command LIB=library) */
char libs[MAXLIBS][CMLEN];	/* Libraries of TAE functions                 */
char program[CMLEN];		/* Name of the program to search for          */
char *ptr;			/* Pointer to an environment variable         */
char *ptr2;			/* Temporary pointer for pathnames	      */
char str[CMLEN];		/* Temporary string variable                  */
char *strings[1];		/* Array of string pointers to return to TAE  */

long fatal    = -1;		/* Fatal error flag                           */
long found;			/* Flag to indicate if the program was found  */
long i;				/* Loop control                               */
long nonfatal = 1;		/* Nonfatal error flag                        */
long numlibs;			/* Number of libraries to search              */
long std_out;			/* Standard output                            */

struct PARBLK vblock;		/* TAE parameter block                        */
struct VARIABLE	*parm;		/* Pointer to a TAE variable within vblock    */


/* Initialize LAS
-----------------*/
c_inlas(&vblock,&std_out);


/* Retrieve the program name
----------------------------*/
parm = p_find(&vblock, "PROGRAM");
strcpy(program, SVAL(*parm,0));


/* Retrieve the suffix name
---------------------------*/
parm = p_find(&vblock, "SUFFIX");
if (parm->v_count > 0)
    {
    strcat(program, ".");
    strcat(program, SVAL(*parm,0));
    }


/* Retrieve the output format
-----------------------------*/
parm = p_find(&vblock, "OUTFORM");
strcpy(format, SVAL(*parm,0));


/* Retrieve the application libraries
-------------------------------------*/
parm = p_find(&vblock, "$USERLIB");
for (i = 0, numlibs = 0; (i < parm->v_count) && (numlibs < MAXLIBS); i++)
    strcpy(libs[numlibs++], SVAL(*parm,i));
parm = p_find(&vblock, "$APLIB");
for (i = 0; (i < parm->v_count) && (numlibs < MAXLIBS); i++)
    strcpy(libs[numlibs++], SVAL(*parm,i));


/* Search for the program
-------------------------*/
i = 0;
found = FALSE;
while (!found && (i < numlibs))
    {
    /* If the library starts with a '$', assume it is an environment variable
    -------------------------------------------------------------------------*/
    if (libs[i][0] == '$')
	{

	/* Check for the presence of an additional path following the
	   environment variable, i.e., $ADAPSBASE/prerel
	-------------------------------------------------------------*/
	ptr2 = (char *)strchr(libs[i], '/');
	if (ptr2)
	{
		*ptr2 = (char) 0; /* Nullify the position of the '/' and */
		ptr2++;		  /* position ptr2 at the path extension */
	}

	ptr = getenv(&libs[i][1]);
	if ((ptr != NULL) && (!ptr2))
	    sprintf(str, "%s/%s", ptr, program);
	else if ((ptr != NULL) && (ptr2))
	    sprintf(str, "%s/%s/%s", ptr, ptr2, program);
	else
	    strcpy(str, "/_this_file_should_not_exist_");
	}
    else
	sprintf(str, "%s/%s", libs[i], program);


    /* Determine if the specified file exists in this library
    ---------------------------------------------------------*/
    if (access(str, R_OK))
	i++;
    else
	found = TRUE;
    }


/* If the program was found, return the desired information to the user
-----------------------------------------------------------------------*/
if (found)
    {
    if (strcmp(format, "COM") == 0)
	{
	sprintf(str, "ush %s/%s", libs[i], program);
	strings[0] = str;
	}
    else
	strings[0] = libs[i];

    if ((q_string(&vblock, "OUT", 1, strings, P_UPDATE) != SUCCESS) ||
        (q_out(&vblock) != SUCCESS))
	{
	c_errmsg("Error returning library", "getlib-lib", &nonfatal);
	c_errmsg("Fatal error encountered", "getlib-fatal", &fatal);
	}
    }
else
    {
    sprintf(str, "Library not found for %s", program);
    c_errmsg(str, "getlib-nolib", &nonfatal);
    c_errmsg("Fatal error encountered", "getlib-fatal", &fatal);
    }


/* Issue a completion message
-----------------------------*/
z_exit(1, "GETLIB completed successfully");

return EXIT_SUCCESS;
}
