/*****************************************************************************
NAME				  GETPARM

PURPOSE   Extracts TAE parameters from a project parameter file

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand    12/89       EDC      LAS 5.0 development
  5.1      D. Etrheim       4/90       EDC	ADP #5521  Specify which
                                                parameter on error message
  5.2      D. Etrheim       9/90       EDC	PRF #5788 Add KEYOPT parameter
						to specify either the first or
						last occurence of a key
  5.3      T. Mittan	    ?/94       EDC      Raised limit to 100
  5.4	   K.Lowell	   01/97       EDC      Raised limit to 300

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPARM must run under TAE

PROJECT  GEOREG/LAS

ALGORITHM DESCRIPTION
	Initialize TAE processing with z_init
	Get the name of the project parameter file
	Get the data type of the parameter to be returned
	Get the parameter key
	Open the project parameter file for read
        Read the project parameter file for the key given
	Place the parameter in the TAE Variable block
	Return the variable block to the TAE process
	Call x_exit to terminate proc (NOTE:  INLAS and COMPLT were not used
		here--the function must run silent!

ALGORITHM DESCRIPTION
	Initialize LAS processing
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"

int main()
{
struct PARBLK vblk;			/* TAE parameter block */
struct VARIABLE *var;	/* TAE Parameter access vars */
long fatal = -1;			/* Fatal error message flag */
long zero = 0;				/* Constant of zero */
long one = 1;				/* Constant of one */
long acc;				/* File access code */
long i;					/* Loop counter */
long std_out;				/* Standard output unit number */
long dim;				/* Dimension of variable field */
long savedim = 0;				/* Dimension of variable field */
int val_l[300];				/* Integer values */
long val1_l[300];			/* Integer values */
double val_r[300];			/* Real values */
double val1_r[300];			/* Real values */
char val_c[300][CMLEN];			/* String values */
char val1_c[300][CMLEN];			/* String values */
char dtype[2];				/* Data type--entered by user */
char errstr[80];			/* String for error message */
char option[6];                         /* first or last option */
char pdtype[2];				/* Data type--as read from file */
char savetype[2];			/* Data type--as read from file */
char parm[20];				/* Parm key as read from file */
char parmkey[20];			/* Parm key as entered by user */
char ppf[CMLEN];			/* Name of project parameter file */
char saveparm[20];          		/* Parm key as read from file */
char *strptr[300];			/* Array of ptrs for q_string call */
FILE *ptr;			/* File access var/routine */

/* Initialize TAE processing
 --------------------------*/
c_inlas(&vblk, &std_out);

/* Get the name of the project parameter file
 -------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk, "INPPF", &zero, ppf, "ppf", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","getparm-fatal",&fatal);

/* Get the data type of the parameter to be returned
  -------------------------------------------------*/
var = p_find(&vblk,"DTYPE");
if (strcmp(SVAL(*var,0),"STRING") == 0) strcpy(dtype, "S");
if (strcmp(SVAL(*var,0),"INTEGER") == 0) strcpy(dtype, "I");
if (strcmp(SVAL(*var,0),"REAL") == 0) strcpy(dtype, "R");

/* Get the parmkey of the parameter to be returned
  -------------------------------------------*/
var = p_find(&vblk,"KEY");
strcpy (parmkey, SVAL(*var,0));

/* Get the option of the parameter to be returned
  -------------------------------------------*/
var = p_find(&vblk,"KEYOPT");
strcpy (option, SVAL(*var,0));

/* Open the project parameter file
  -------------------------------*/
ptr = fopen(ppf, "r");
if (ptr == NULL)
   {
   c_errmsg("Error opening project parameter file","getparm-open",&one);
   c_errmsg("Fatal error encountered","getparm-fatal",&fatal);
   }

/* Read the project parameter file for the key of the parameter given
  ------------------------------------------------------------------*/
if (strcmp(option,"LAST") == 0)
   {
   for (;;)
      {
      if (fscanf(ptr, "%s %s %d", parm, pdtype, &dim) != 3) break;
      if (strcmp(parmkey, parm) == 0)
         {
         if (strcmp(pdtype, "I") == 0)
   	    {
   	    for (i = 0; i < dim; i++)
	       fscanf(ptr,"%d", &(val_l[i]));
            }
         if (strcmp(pdtype, "S") == 0)
	    {
	    for (i = 0; i < dim; i++)
	       fscanf(ptr,"%s", &(val_c[i][0]));
            }
         if (strcmp(pdtype, "R") == 0)
	    {
	    for (i = 0; i < dim; i++)
	       fscanf(ptr,"%lf", &(val_r[i]));
            }
         strcpy (saveparm,parm);
         strcpy (savetype,pdtype);
         savedim = dim;
         }

      if (strcmp(parmkey, parm) != 0)
         {
         if (strcmp(pdtype, "I") == 0)
	    {
	    for (i = 0; i < dim; i++)
	       fscanf(ptr,"%d", &(val1_l[i]));
            }
         if (strcmp(pdtype, "S") == 0)
	    {
	    for (i = 0; i < dim; i++)
	       fscanf(ptr,"%s", &(val1_c[i][0]));
            }
         if (strcmp(pdtype, "R") == 0)
	    {
	    for (i = 0; i < dim; i++)
	       fscanf(ptr,"%lf", &(val1_r[i]));
            }
         }
      }
   strcpy (parm,saveparm);
   strcpy (pdtype,savetype);
   dim = savedim;
   }
else
   {
   for (;;)
      {
      if (fscanf(ptr, "%s %s %d", parm, pdtype, &dim) != 3) break;
      if (strcmp(pdtype, "I") == 0)
	 {
	 for (i = 0; i < dim; i++)
	    fscanf(ptr,"%d", &(val_l[i]));
         }
      if (strcmp(pdtype, "S") == 0)
	 {
	 for (i = 0; i < dim; i++)
	    fscanf(ptr,"%s", &(val_c[i][0]));
         }
      if (strcmp(pdtype, "R") == 0)
	 {
	 for (i = 0; i < dim; i++)
	    fscanf(ptr,"%lf", &(val_r[i]));
         }
      if (strcmp(parmkey, parm) == 0) break;
      }
   }

if (strcmp(parm, parmkey) != 0)
   {
   strcpy(errstr,"Parameter ");
   strcat(errstr,parmkey);
   strcat(errstr," not found");
   c_errmsg(errstr,"getparm-read",&one);
   c_errmsg("Fatal error encountered","getparm-fatal",&fatal);
   }

if (strcmp(dtype, pdtype) != 0)
   {
   strcpy(errstr,"Data types not compatible for parameter ");
   strcat(errstr,parmkey);
   c_errmsg(errstr,"getparm-type",&one);
   c_errmsg("Fatal error encountered","getparm-fatal",&fatal);
   }

/* Send back to VBLK
  -----------------*/

if (strcmp(pdtype,"S") == 0)
   {
   for (i = 0; i < dim; i++)
      strptr[i] = &(val_c[i][0]);
   if (q_string(&vblk, "PRETURN", dim, strptr, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","getparm-fatal",&fatal);
   }

if (strcmp(pdtype,"I") == 0)
   if (q_intg(&vblk, "PRETURN", dim, val_l, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","getparm-fatal",&fatal);

if (strcmp(pdtype,"R") == 0)
   if (q_real(&vblk, "PRETURN", dim, val_r, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","getparm-fatal",&fatal);

if (q_out(&vblk) != SUCCESS)
	c_errmsg("Fatal error encountered","getparm-fatal",&fatal);

z_exit(1," ");

return EXIT_SUCCESS;
}
