/******************************************************************************

NAME:   CHECK_ATTRB

FUNCTION:
        CHECK_ATTRB is a function which determines if the current attribute 
        value read from the GOF matches the desired attribute values specified 
        by the user.  If a match is found, a value of TRUE is returned; 
        otherwise a value of FALSE is returned.

ALGORITHM:

        If the user did not specify any attributes
            Return TRUE.
        Else
            For the number of attributes specified by the user
                If the GOF attribute value matches the user specified
                  attribute value OR the user specified to use the
                  record if it has any value for the attribute AND
                  the GOF attribute value is not NULL
                    Return TRUE.
        Return FALSE.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "gof.h"
#include "protos.h"

int check_attrb
(
    char inatr[][ATTVAL],       /* Attribute values of current GOF record (I) */
    char atrval[][ATTVAL],      /* User specified attribute values (I)        */
    int  atrindex[],            /* Indices corresponding to attribute names
                                   in the GOF header record (I)               */
    long numatr         /* Number of attributes specified by the user
                                   numatr(I)                                  */
)
{
long index;                     /* Looping index                              */

/*
If the user did not specify any attributes, always return a value of TRUE
*/
if (numatr == 0)
   return(TRUE);
else
   {
   /*
   Check for a match and return TRUE if one is found.  Note that the search
   function ORs the attribute values.
   */
   for (index=0; index<numatr; index++)
      {
      if ((strcmp(inatr[atrindex[index]],atrval[index]) == 0) ||
         ((strcmp(atrval[index],ATHAT) == 0) && 
          (strcmp(inatr[atrindex[index]],"") != 0)))
         return(TRUE);
      }
   }
return(FALSE);
}
