/******************************************************************************

NAME:   DATA_ANNOT_HDR

FUNCTION:
        DATA_ANNOT_HDR writes the header information of the Data Labeled Table 
        file for annotation records.  It writes the label vector to the LT.

ALGORITHM:

        Write out each of the attribute names (including the data type,
          description, number of rows and columns) for an annotation LT.
        Return to the calling routine.

******************************************************************************/

#include "las.h"
#include "gof.h"
#include "ltable.h"
#include "protos.h"

void data_annot_hdr
(
    struct TAB_DEF *tab         /* Labeled table definition structure (I)     */
)
{
/*
Write out the label vectors for the unique number defining the record,
the X and Y location, the character height, the text rotation angle,
and the annotation string
*/
put_vector(tab,"ANNOTNUM",I4,"unique region number",1,1,FIELD_SEP2); 
put_vector(tab,"X",R4,"column number",1,1,FIELD_SEP2);
put_vector(tab,"Y",R4,"row number",1,1,FIELD_SEP2);
put_vector(tab,"SIZE",I4,"character height in annotation string",1,1,
   FIELD_SEP2);
put_vector(tab,"ANGLE",R4,"angle of annotation string",1,1,FIELD_SEP2);
put_vector(tab,"ANNOT",C,"annotation string",1,ANTLEN,RECORD_SEP);
return;
}
