/******************************************************************************

NAME:   DATA_XY_HDR

FUNCTION:
        DATA_XY_HDR writes the header information of the Data Labeled Table 
        file for XY points (for polygon, line, or point GOFs).  It writes the 
        label vector to the LT.

ALGORITHM:

        If processing a polygon data file (LT type REGXY)
            Write out the attribute definition for REGNUM which is the unique
              number identifying each polygon.
        Else if processing a line data file (LT type ARCXY)
            Write out the attribute definition for ARCNUM which is the unique
              number identifying each line.
        Else if processing a point data file (LT type PNTXY)
            Write out the attribute definition for PNTNUM which is the unique
              number identifying each point.
        Write out the attribute definition for the X location.
        Write out the attribute definition for the Y location. 
        Return to the calling routine.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "gof.h"
#include "ltable.h"
#include "protos.h"

void data_xy_hdr
(
    struct TAB_DEF *tab         /* Labeled table definition structure (I)     */
)
{
/*
Write out the label vectors for the unique number defining the record,
and the X and Y location
*/
if (strcmp(tab->ftype,"REGXY") == 0)
   put_vector(tab,"REGNUM",I4,"unique region number",1,1,FIELD_SEP2);
else if (strcmp(tab->ftype,"ARCXY") == 0)
   put_vector(tab,"ARCNUM",I4,"unique region number",1,1,FIELD_SEP2);
else if (strcmp(tab->ftype,"PNTXY") == 0)
   put_vector(tab,"PNTNUM",I4,"unique region number",1,1,FIELD_SEP2);
put_vector(tab,"X",R4,"column number",1,1,FIELD_SEP2);
put_vector(tab,"Y",R4,"row number",1,1,RECORD_SEP);
return;
}
