/******************************************************************************

NAME:   GOF2TAB

FUNCTION:
        GOF2TAB is an application program which converts a GOF into a
        Labeled Table file.

PROGRAM HISTORY:
PROGRAMMER      DATE    CODE/CONT       REASON
----------      ----    ---------       ------
K. Gacke        10/86   EDC
K. Gacke        1/87    EDC             New LT file format.
K. Gacke        4/87    EDC             Modified LT attribute names from
                                        REGNUM to ARCNUM, PNTNUM, and ANNOTNUM
L. Huewe        1/89    EDC             DMS v3_0 retrocode
D. Etrheim      2/93    EDC             Convert to a LAS application

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:        LAS

ALGORITHM:

        Initialize LAS and TAE.
        Retrieve the input parameters.
        If the GOF type specified is POLY or LINE
            Copy the label and attribute values to the attribute LT and the
              polygon or line data to the data LT.
        Else if the GOF type specified is POINT
            Copy the label and attribute values to the attribute LT and the
              point data to the data LT.
        Else if the GOF type specified is ANNOT
            Copy the label and attribute values to the attribute LT and the
              annotation data to the data LT.
        Write a completion message to the user's terminal.

******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "gof.h"
#include "protos.h"


struct  PARBLK  vblock;         /* TAE parameter block                        */
        char    outlt[2][CMLEN];/* Output Labeled Table Host names            */

int main
(void)
{
struct VARIABLE *val;           /* Pointer to variable in TAE vblock          */

long    access = IREAD;
long    abort = -1;             /* Abort flag for error messages              */
long    index;                  /* Loop index counter                         */
long    length;                 /* String length                              */
long    msgtim;                 /* Number of seconds between messages         */
long    numatr;                 /* Number of attributes in the GOF            */
long    status;                 /* Function return status code                */
long    stdlun;                 /* Standard logical unit number               */
long    tab_access = IWRITE;    /* LT file access                             */
long    two = 2;                /* Value of two                               */
long    zero = 0;               /* Value of zero                              */

char    *desc;                  /* Labeled Table file description             */
char    ext[11];                /* GOF extension                              */
char    *ftype;                 /* GOF file type                              */
char    hgof[CMLEN];            /* Host GOF name                              */
char    htab[CMLEN];            /* Host Labeled Table name                    */
char    inatr[MAXATT][ATTIN];   /* Input GOF attribute array                  */
char    lgof[CMLEN];            /* LAS GOF name                               */
char    *make_lt_name();        /* Make a Labeled Table name                  */

 
/*
Initialize TAE and LAS
*/
c_inlas(&vblock,&stdlun);

/*
Retrieve input parmameters
*/
status = c_getnam(&vblock,"OUTLT",&zero,htab,"lt",&tab_access);
if (status != E_SUCC)
   c_errmsg ("Fatal error encountered","gof2tab-fatal",&abort); 

val = p_find(&vblock,"FTYPE");
ftype = SVAL(*val,0);

sprintf(ext,"gof%c%s",GOF_SEP,ftype);
length = (long)strlen(ext);
c_up2low(ext, &length);
status = c_getnam(&vblock,"INGOF",&zero,hgof,ext,&access);
if (status != E_SUCC)
   c_errmsg ("Fatal error encountered","gof2tab-fatal",&abort);

val = p_find(&vblock,"ATRVAL");
for (numatr=0; numatr<val->v_count; numatr++)
   strcpy(inatr[numatr],SVAL(*val,numatr));
for (index=numatr; index<MAXATT; index++)
   *(inatr[index]) = '\0';

val = p_find(&vblock,"TEXT");
if (val->v_count == 0)
   desc = NULL;
else
   desc = SVAL(*val,0);

val = p_find(&vblock,"$MSGTIM");
msgtim = IVAL(*val,0);

if ((strcmp(ftype,"POLY") == 0) || (strcmp(ftype,"LINE") == 0))
   process_poly_line(hgof,lgof,htab,ftype,inatr,numatr,desc,msgtim);
else if (strcmp(ftype,"POINT") == 0)
   process_point(hgof,lgof,htab,ftype,inatr,numatr,desc,msgtim);
else if (strcmp(ftype,"ANNOT") == 0)
   process_annot(hgof,lgof,htab,ftype,inatr,numatr,desc,msgtim);

c_complt("GOF2TAB",outlt,&two);

return EXIT_SUCCESS;
}
