/******************************************************************************

NAME:   OPEN_LT

FUNCTION:
        Set up the LT file description and type before opening the LT
        and one subfile for write access.

ALGORITHM:

        If a file description was not input
           Generate one and assign the description to the LT structure.
        Set up the appropriate file types based on the type of GOF input.
        Generate the subfile name.
        Open the LT and its subfile for write access.
        Return to the calling routine.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "gof.h"
#include "ltable.h"
#include "protos.h"

void open_lt
(
    struct TAB_INFO *tab,               /* LT file descriptors (O)                    */
    char *las_gof,      /* LAS name of input GOF (I)                  */
    char *host_tab,     /* Host name of output Labeled Table file (I) */
    char *ftype,                /* Input GOF type (I)                         */
    char *desc          /* Labeled Table file description (I)         */
)
{
extern  char    outlt[2][CMLEN];/* Output LT Host names                       */

char    temp[MAX_FIELD_LEN];    /* Temporary variable for file description    */

/*
If a file description was not input, generate one and assign the description
to the LT structure
*/
if (desc == NULL)
   sprintf(temp,"File generated from file %s",las_gof);
else
   strcpy(temp,desc);
tab->desc.fdesc = temp;
tab->data.fdesc = temp;

/*
Set up the appropriate file types based on the type of GOF input
*/
if (strcmp(ftype,"POLY") == 0)                          /* file type        */
   {
   tab->desc.ftype = "REGATT";
   tab->data.ftype = "REGXY";
   }
else if (strcmp(ftype,"LINE") == 0)
   {
   tab->desc.ftype = "ARCATT";
   tab->data.ftype = "ARCXY";
   }
else if (strcmp(ftype,"POINT") == 0)
   {
   tab->desc.ftype = "PNTATT";
   tab->data.ftype = "PNTXY";
   }
else if (strcmp(ftype,"ANNOT") == 0)
   {
   tab->desc.ftype = "ANNOTATT";
   tab->data.ftype = "ANNOTXY";
   }

/*
Generate a subfile name and open the LT for write access
*/
tab->desc.nsubs = 1;
tab->desc.subfile[0] = make_lt_name(host_tab,1);
open_tab(&tab->desc,host_tab,1);
strcpy(outlt[0],host_tab);

/*
Open the subfile for write access also
*/
tab->data.nsubs = 0;
tab->data.subfile[0] = '\0';
open_tab(&tab->data,tab->desc.subfile[0],1);
strcpy(outlt[1],tab->desc.subfile[0]);

return;
}
