/*****************************************************************************
NAME:               EXPYRAMID

PURPOSE:  Up samples with interpolation (Used to create a Lapacian Pyramid)

VERSION     DATE    AUTHOR      CODE/CONT 
  1.0       4/92    D. Steinwand
*****************************************************************************/

#include "ezio.h"

long curcat;                /* current band number */
long curent;                /* current line number */

static void filteru2
(
    unsigned char *in,      /* Input buffer */
    long ns,                /* Number of samples output */
    float *w,               /* Weight vector */
    float *out              /* Results of convolution & downsampling */
);

int main(void)
{

    long in_nl,
    in_ns;                  /* Number of lines, samples (input) */
    long out_nl,
    out_ns;                 /* Number of lines, samples (output) */
    long line,
    i,
    j;                      /* Loop counters */

    float a;                /* A value for kernel construction */
    float *workbuf;         /* Working buffer (results of x conv) */
    float w[5];
    float *pm1,
    *p0,
    *p1,
    *pm1tmp;                /* Pointers into workbuf */

    unsigned char *inbuf;   /* Input buffer */
    unsigned char *outbuf1,
    *outbuf2;               /* Output buffers */

    int total_bands;        /* total bands to process, in bands = out bands */
    int band_number;        /*  Output band number, used for indexing also */
    long bands[MAXIMG][MAXBND+1]; /* band numbers to process */
    int in_image_line;      /* current image line to read */
    int out_image_line;     /* current image line to write */

    /* Initialize processing, images, and buffer space
       -----------------------------------------------*/
    start("EXPYRAMID");
    init_inimg(&in_nl, &in_ns, &total_bands, BYTE, bands);
    out_nl = in_nl * 2;
    out_ns = in_ns * 2;
    init_outimg(out_nl, out_ns, total_bands, BYTE);
    inbuf = (unsigned char *) init_buf(in_ns, BYTE, total_bands);
    workbuf = (float *) init_buf(out_ns*3, FLOAT, total_bands);
    pm1 = workbuf;
    p0 = pm1 + out_ns;
    p1 = p0 + out_ns;
    outbuf1 = (unsigned char *) init_buf(out_ns, BYTE, total_bands);
    outbuf2 = (unsigned char *) init_buf(out_ns, BYTE, total_bands);
    a = get_float("WEIGHT");

    /* Setup convolution kernel and prime image buffers
       ------------------------------------------------*/
    w[2] = a;
    w[3] = w[1] = 0.25;
    w[4] = w[0] = 0.25 - (a / 2.0);

    curent = 1;
    curcat = 1;

    proc_mess(total_bands);

    for(i = 0; i < 5; i++)
        w[i] *= 2.0;

    for (band_number = 1; band_number <= total_bands; band_number++)
    {
        in_image_line = 1;
        out_image_line = 1;

        curcat = band_number;
        curent = out_image_line;

        get_line(inbuf,bands[0][band_number-1],in_image_line);
        in_image_line++;
        filteru2(inbuf ,out_ns, w, pm1);
        get_line(inbuf,bands[0][band_number-1],in_image_line);
        in_image_line++;
        filteru2(inbuf, out_ns, w, p0);

        for(i = 0; i < out_ns; i++)
            outbuf1[i]=0;
        put_line(outbuf1,band_number,out_image_line);  /* Write 1st line */
        out_image_line++;
        curent++;


        /* Loop thru the output image(s), line-by-line, pixel-by-pixel
           -----------------------------------------------------------*/
        for (line=2; line<out_nl-2; line+=2)
        {
            get_line(inbuf,bands[0][band_number-1],in_image_line); 
            in_image_line++;
            filteru2(inbuf,out_ns,w,p1);
            for (j=0; j<out_ns;j++)
            {
                outbuf1[j]=w[0]*pm1[j]; 
                outbuf1[j]+=w[2]*p0[j]; 
                outbuf1[j]+=w[4]*p1[j];
                outbuf2[j]=w[1]*p0[j]; 
                outbuf2[j]+= w[3]*p1[j];
            }
            put_line(outbuf1,band_number,out_image_line);
            out_image_line++;
            curent++;
             /* Write 2 lines of data to disk */
            put_line(outbuf2,band_number,out_image_line);   
            out_image_line++;
            curent++;
            pm1tmp = pm1;
            pm1 = p0;
            p0 = p1;
            p1 = pm1tmp;
        }
        for(i = 0; i < out_ns; i++) 
            outbuf1[i] = 0;

        put_line(outbuf1,band_number,out_image_line);
        out_image_line++;
        curent++;
        put_line(outbuf1,band_number,out_image_line);
        out_image_line++;
        curent++;
        put_line(outbuf1,band_number,out_image_line);
        out_image_line++;
        curent++;
    }
    finish(0);
    return 0;
}

/* Function to apply 5 point convolution in the x dimension and upsample
   results by two.
  ---------------*/
static void filteru2
(
    unsigned char *in,      /* Input buffer */
    long ns,                /* Number of samples output */
    float *w,               /* Weight vector */
    float *out              /* Results of convolution & downsampling */
)
{
    long i,j;

    out[0] = 0.0;
    for(j = 1; j < (ns - 2); j++)
    {
        for(out[j] = 0.0, i = 1; i < 5; i += 2)
            out[j] += w[i] * in[((j + i) - 2) / 2];
        j++;
        for(out[j] = 0.0, i=0; i<5; i+=2)
            out[j] += w[i] * in[((j + i)- 2) / 2];
    }
    out[ns-2] = out[ns-1]=0.0;
}
