/****************************************************************************
NAME:				GRIDFORM

PURPOSE:   Reads foreign geometric mapping grids and converts them into
           the LAS geometric mapping grid format.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		2/89	D. Steinwand	   CSB      Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initailize LAS processing
	Get the input (foreign) grid file name
	Get the output geometric mapping grid file name
	Get the file format of the input grid file
	Read the input grid and pack it into the GEOGRID structure
	Write the geometric mapping grid to disk
	Translate the output grid file name to a LAS name
	Write completion message and exit

ALGORITHM REFERENCES:
*****************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
struct GEOGRID grid;		/* Geometric mapping grid */
char in_name[CMLEN];		/* Input grid file name */
char out_name[CMLEN];		/* Output grid file name */
long one = 1;			/* Constant of one (1) */
struct VARIABLE *var;		/* pointer to a TAE variable in vblk */
struct PARBLK vblk;		/* TAE variable block */
long std_out;			/* standard output for c_inlas */
long acc;			/* access flag for c_getnam */
long status = 0;		/* Status flag */
long zero = 0;			/* Constant of zero */
long fatal = -1;		/* Fatal error flag */

/* Initialize tae parameter blocks, etc
  ------------------------------------*/
c_inlas(&vblk, &std_out);

/* Get the input grid file name
  ----------------------------*/
acc = IREAD;
if (c_getnam(&vblk,"INFILE", &zero, in_name,"grd",&acc) != E_SUCC)
   c_errmsg("Fatal error encountered","gridform-fatal",&fatal);

/* Get the output geometric mapping grid file name
  -----------------------------------------------*/
acc = IWRITE;
if(c_getnam(&vblk,"OUTGRID", &zero, out_name,"grid",&acc) != E_SUCC)
   c_errmsg("Fatal error encountered","gridform-fatal",&fatal);

/* Get the file format of the input grid file and reformat
  -------------------------------------------------------*/
var = p_find(&vblk, "FILEFORM");
if (strcmp(SVAL(*var,0), "LINSAM") == 0) status = linsam(in_name, &grid);
if (strcmp(SVAL(*var,0), "SAMLIN") == 0) status = samlin(in_name, &grid);
if (strcmp(SVAL(*var,0), "OLDSTD") == 0) status = oldstd(in_name, &grid);
if (status != E_SUCC)
   c_errmsg("fatal error encountered","gridform-fatal",&fatal);

/* Write the geometric mapping grid to disk
  ----------------------------------------*/
if(c_putgrd(out_name, &grid) != E_SUCC)
   c_errmsg("fatal error encountered","gridform-fatal",&fatal);

/* Write the standard LAS completion message and exit
  --------------------------------------------------*/
c_complt("GRIDFORM", out_name, &one);

return EXIT_SUCCESS;

}
