/****************************************************************************
NAME:				GRIDGEN

PURPOSE:  Read transformation coefficients from a geometric mapping grid file 
	  and create a new geometric mapping grid file with mapping grid 
	  information.  The input grid file contains number of coefficients 
	  and two sets of coefficients of two polynomials(degree 1 - 4), which 
	  represent a transformation. The transformation information may be 
	  entered manually through the GRIDEDT program.  The output geometric
	  mapping grid file will contain computed mapping grid and other grid
 	  information (DDR, etc) copied from the input grid file.  The mapping 
	  grid will be used by GEOM program to perform geometric rectification
	  for images.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  1.0      	5/84  	M. NISHIHAMA       SAR        N/A
  5.0		10/88	D. Steinwand	   CSB      LAS 5.0 C conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Get parameters from the user
	Get the input grid from disk
	Create a (dimension X dimension) grid from the polynomial in the
		input grid
	Reduce the mapping grid, according to tolerance, if possible
	Calculate errors due to the gridding process
	Write the resulting mapping grid to disk
	Call complete & exit

ALGORITHM REFERENCES:
*****************************************************************************/
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "local_protos.h"

#define STAT_FLAG status
#define CHECK_STAT(ARG) status = ARG; if(status != E_SUCC) gabort(status)

int main(void)
{
struct GEOGRID grid;		/* Geometric mapping grid */
char in_name[CMLEN];		/* Input grid file name */
char out_name[CMLEN];		/* Output grid file name */
double tolerance;		/* Tolerance for linear approximation */
long ngline;			/* Number of lines per output grid cell */
long ngsamp;			/* Number of samples per output grid cell */
int  window[4];			/* Window/size of output grid */
long msgtim;			/* Message time variable */
long one = 1;			/* Constant of one (1) */
long STAT_FLAG;			/* Return status variable */

CHECK_STAT(getpar(in_name,out_name,window,&tolerance,&ngline,&ngsamp,&msgtim));
CHECK_STAT(c_getgrd(in_name, &grid));
CHECK_STAT(make_grid(&grid,window,ngline,ngsamp,tolerance,msgtim));
c_grderr(&grid, &tolerance);
CHECK_STAT(c_putgrd(out_name, &grid));
c_complt("GRIDGEN", out_name, &one);
return (0);
}
