#include "protos.h"
#include "util.h"

void dumpvals
(
    float grey[],     /* I: grey values                            */
    long nbands,      /* I: no. bands                              */
    long dtype,       /* I: image data type                        */
    long isamp,       /* I: sample coordinate                      */
    long line,        /* I: line coordinate                        */
    long header,      /* I: flag indicating whether to display header */
    char comment[],   /* I: user comment                           */
    char field1[],    /* I: annotation info                        */
    char field2[],    /* I: at the beginning of each record        */
    char subc[]       /* I: subcommand                             */
)
/*
***************************************************************************

NAME: DUMPVALS

PURPOSE:  Outputs the specified coordinate information to the term, lp or file

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          4/19/89   K. A. Narayanan    conversion to 5.0
   3         10/26/89   K. A. Narayanan    corrected output to file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{

long	field, i, j, lead, linelen, len, n, num_loops, ngrey;

char	dash[130];

if (dash[0] != '-') 
   for (i = 0; i < 130; i++) dash[i] = '-';

if (print[TERM])
   linelen = 80;                   /*  line width for terminal */
else
   linelen = 132;                  /*  line width for line printer */

if (dtype == EBYTE || dtype == EWORD)/*  width for byte, int*2 field */
   field = 7;
else
   field = 13;                     /*  width for integer*4, real field */

lead = 13;                         /*  leader field width */
                           
   ngrey = 0;  /* ngrey = # greylevels output each time thru loop */
   num_loops = 0;

   while (ngrey < nbands)
      {
      n = (linelen-lead)/field;      /*  number of bands/line */
      if (ngrey+n  >  nbands) n = nbands-ngrey;
      if (header  &&  ngrey == 0)  /* display header info */
         {
         if (subc[0] == 'S')
            {
            sprintf (buf, "\n GREYLEVELS FOR SITE %10s IN CLASS %10s", 
                     field2, field1);
            c_txtout (buf, print, fd);
            }

        if (print[FIL])                        
          {
          sprintf (buf, "\n\n COMMENT: %40s",comment);
          c_txtout (buf, print, fd);
          } 
                                        
         if (n  <=  nbands) 
            {
            c_txtout ("\n\n  LINE  PIXEL          GREY VALUES (BY BAND)",
                      print, fd);
            for (j = 0; j < (len = lead + field - 4); j++) buf[j] = ' ';
            buf[len] = '\0';
            for (i = 0; i < n; i++) 
               {
               sprintf (&buf[strlen (buf)], "%3d", ngrey+i+1);
               for (j = 0; j < field - 3; j++) strcat (buf, " ");
               }
            c_txtout (buf, print, fd);
            strncpy (buf, dash, lead);
            buf[lead] = buf[lead+1] = ' ';
            strncpy (&buf[lead+2], dash, n*field-3);
            buf[lead+2+n*field-3] = '\0';
            c_txtout (buf, print, fd);
            }
         else
            {
            c_txtout ("\n\n  LINE  PIXEL          GREY VALUES (BY BAND)",
                      print, fd);
            strncpy (buf, dash, lead);
            buf[lead] = buf[lead+1] = ' ';
            strncpy (&buf[lead+2], dash, n*field-3);
            buf[lead+2+n*field-3] = '\0';
            c_txtout (buf, print, fd);
            } 
         } 
      if (dtype == EBYTE || dtype == EWORD)
         {
         sprintf (buf, " %5d %5d ", line, isamp);
         for (i = 0; i < n; i++)
         sprintf (&buf[strlen (buf)], "%6d ", (long) (grey[ngrey+i] + 0.5));
         c_txtout (buf, print, fd);
         }
      else if (dtype == ELONG)   /* int*4 */
         {
         sprintf (buf, " %5d %5d ", line, isamp);
         for (i = 0; i < n; i++)
            sprintf (&buf[strlen (buf)], "%12d ", (long) (grey[ngrey+i] + 0.5));
         c_txtout (buf, print, fd);
         }
      else  /* real*4 */
         {
         sprintf (buf, " %5d %5d ", line, isamp);
         for (i = 0; i < n; i++)
            sprintf (&buf[strlen (buf)], "%12.6e ", grey[ngrey+i]);
         c_txtout (buf, print, fd);
         }  
      ngrey += n;
      num_loops++;
      }  
   if (num_loops > 1) 
      c_txtout (" ", print, fd);         /* leave space between  */
                                         /* lines for clarity */      
}
