#include "protos.h"

void filspec
(
    char hosin[][CMLEN],      /* I: input image host names             */
    long num_inimages,        /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* I: band numbers in each      */
    long wind[][4],           /* I: input image windows                */
    long *in_type,            /* O: input image data type              */
    char subc[],              /* I: source of coordinate input         */
    long *xoff,               /* O: master sample coordinate           */
    long *yoff                /* O: master line coordinate             */
)

/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          4/19/89   K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   If statistics file option open input image(s) for random access
   Else open input image(s) for sequential access
   Group the images

**************************************************************************
*/
{
struct	DDR ddr;                     /* image ddr                          */

static	long	first = TRUE;

long	access,                      /* file access type                   */
	bufsiz,                      /* image i/o buffer size              */
	n, nb, nbands,
	stat,                        /* return status                      */
	type;                        /* file type                          */

static long opt[3]={0,0,0}; /* optional parameter array for c_eopen */
long    fatal_err = -1;
long    zero = 0;
long    one = 1;

/*
get input image type and master coordinates
-------------------------------------------
*/
if (first)
   {
   c_getddr (hosin[0], &ddr);
   *in_type = ddr.dtype;
   *xoff = ddr.master_sample ? ddr.master_sample : 1;
   *yoff = ddr.master_line ? ddr.master_line : 1;

   first = FALSE;
   }
/*
open input image files
----------------------
*/
access = IREAD;
type = EREAL;
if (subc[0] == 'S')
   {
/*
   if statistics file option open image for random access
   ------------------------------------------------------
*/
   nb = 0;
   for (n = 0, nbands = 0; n < num_inimages; n++)
      {
      stat = c_eopenr( (struct FDESC **)&fdesc[n], hosin[n], &nb, &wind[0][SL], 
                   &wind[0][SS], &wind[0][NL], &wind[0][NS], 
                   &access, &type, opt, &one);
      if (stat != E_SUCC )
         g_errmsg ("Fatal error encountered ", "grndspot-fatal", &fatal_err);
      nbands += num_image_bands[n];
      }  
   if ((iobuf = (float *) malloc (nbands * wind[0][NS] * sizeof (float))) 
                    == NULL)
         g_errmsg ("Error allocating dynamic memory", "grndspot-alloc", 
                   &fatal_err);
   }

else

   {
/*
   otherwise open image file for sequential access
   -----------------------------------------------
*/
   for (n = 0, nbands = 0; n < num_inimages; n++)
      {
      stat = c_eopens( (struct FDESC **)&fdesc[n], hosin[n], image_band_num[n],
		       &wind[0][SL], &wind[0][SS], &wind[0][NL], &wind[0][NS], 
                       &ipoint[nbands], &access, &type, &zero, opt);
      if (stat != E_SUCC )
         g_errmsg ("Fatal error encountered", "grndspot-fatal", &fatal_err);

      nbands += num_image_bands[n];
      }  

/*
   set up group containing input bands
   -----------------------------------
*/
   if ((iobuf = (float *) c_ealloc((struct FDESC **)fdesc, &bufsiz)) == NULL)
      g_errmsg ("Error allocating dynamic memory", "grndspot-alloc", 
                &fatal_err);
   c_egroup ((struct FDESC **)fdesc, (struct GDESC **)&gdesc, 
	    (unsigned char *)iobuf, &bufsiz);
   }

}
