#include "protos.h"
#include "util.h"
#include "tae.h"

void getcoord 
(
   FILE *clun,			/* I: logical units for coord file,   */ 
   char (*cname)[11],		/* I/O: coordinate name               */ 
   float coord[],		/* O: coordinates                     */ 
   long *numcoord,		/* O: total no. coordinates           */ 
   long *ncoord,		/* I/O: total no. coordinates         */ 
   long *flag			/* I/O: more coordinates indicator    */
)      
/*
***************************************************************************

NAME: GETCOORD

PURPOSE: Gets coordinates from a text file or dynamic tutor

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          4/19/89   K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   If coordinates are not in file then
      Get coordinates through dynamic tutor 
   Else
      Call READNEXT to read next MAX_COORD number of points and their 
         coordinates
      Set FLAG = false if end of file is encountered

***************************************************************************
*/
{
struct	PARBLK	block1;

long	count, ftype, i, k, k1;

char	buf[ERRLEN];

long    inf_err = 1;


if (clun  ==  NULL) 
   {
   q_init (&block1, P_BYTES, P_ABORT);
   q_dynp (&block1, "grndcoord", M_FULLPDF);

   if ((p_inim (&block1, sizeof (struct PARBLK), P_CONT) == SUCCESS))
      {
      count = 50;
      ftype = EREAL;
      *numcoord = c_getnum (&block1, "coord", &count, &ftype, (unsigned char *)coord);
      }
   else
      {
      c_errmsg ("Error getting coordinates", "grndspot-coorderr", 
                 &inf_err);
      *flag = FALSE;
      return;
      }

   if (*numcoord % 2  !=  0) 
      c_errmsg ("Last coordinate pair invalid; ignored", "grndspot-coordign", 
                 &inf_err);
   *numcoord /= 2;

   for (i = 0; i < *numcoord; i++) /* initialize or output will          */
      cname[i][0] = '\0';          /* contain control chars of some sort */

   }
else
   {
   k = 0;
   k1 = 0;
   while (*flag)
      {
      readnext (clun, cname[k], &coord[k1], flag);
      k++;
      k1 += 2;
      }  
   if (*flag) 
      *numcoord = k;
   else
      *numcoord = k-1;
   }  
if (*numcoord  ==  0) return;
else if (*numcoord  ==  1) 
   {
   sprintf (buf, "Processing coordinate  :%5d\n\0", (*ncoord)+1);
   c_errmsg (buf, "grndspot-coord1", &inf_err);
   }
else
   {
   sprintf (buf, "Processing coordinates  :%5d -%5d\n\0", (*ncoord)+1,
            (*ncoord) + (*numcoord));
   c_errmsg (buf, "grndspot-coord2", &inf_err);
   }  

*ncoord += *numcoord;
}
