#include "protos.h"
#include "pixman.h"

void get_greylevel
(
    long numcoord,	/* I: no. coordinates		*/
    float coord[], 	/* I: coordinates		*/
    float grey[], 	/* O: grey levels		*/
    long nbands,	/* I: no. coordinates		*/
    long np,		/* I: no. pixels per line	*/
    long *nl,		/* I: no. lines in image	*/
    long coord_flag[],  /* O: coordinate validity flags */
    long *numinv	/* I/O: no. invalid coordinates	*/
)
/*
***************************************************************************

NAME: GET_GREYLEVEL

PURPOSE: Gets grey levels corresponding the coordinates

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          4/19/89   K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Update number of coordinates for each image line
   Set coordinate validity flags to false if coordinates are invalid
   Do for number of lines in the image
      Read next line
      Do while there are more coordinates for this line
         If next coordinate is on this line and is a valid one 
            get corresponding grey levels
   Close input image bands 

***************************************************************************
*/
{

short	*lineind;

long	i, i1, jpix, k, k1, line, maxline, minline, n;
long	fatal_err = -1;
long	wtype = EWORD;
long    zero = 0;


/*
allocate dynamic memory
*/
lineind = (short *) malloc((*nl) * sizeof(short));
if (lineind == NULL) {
    c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
   }

/*
update number of points for any image line
------------------------------------------
*/
c_pxfill ((unsigned char *)lineind, &wtype, nl, (unsigned char *)&zero); 
minline = *nl;
maxline = 1;

for (k = 0; k < numcoord; k++)
   {
   k1 = 2*k;
   line = (long)(coord[k1] + 0.5);
   jpix = (long)(coord[k1+1] + 0.5);
   if (line  >  0  &&  line  <=  *nl  && 
       jpix  >  0  &&  jpix  <=  np) 
      {
      lineind[line-1]++;
      coord_flag[k] = TRUE;
      minline = min (minline, line);
      maxline = max (maxline, line);
      }
   else
      {
      coord_flag[k] = FALSE;
      (*numinv)++;
      }  
   }  

/*
skip lines outside the user specified range
-------------------------------------------
*/
for (line = 1; line < minline; line++) c_estep ((struct GDESC **)&gdesc);

/*
process lines in the user specified range
-----------------------------------------
*/
for (line = minline; line <= maxline; line++)
   {
   c_estep ((struct GDESC **)&gdesc);
/*
   get grey levels of all points corresponding to this line
   --------------------------------------------------------
*/
   k = 0;
   k1 = 0;
   n = 1;
   while (n  <=  lineind[line-1])
      {
      if (line  ==  (long) (coord[k1] + 0.5)  &&  coord_flag[k]) 
         {
         jpix = (long)(coord[k1+1] + 0.5);
         i1 = nbands * k;
         for (i = 0; i < nbands; i++)
            grey[i1+i] = iobuf[ipoint[i]+jpix-1];
         n++;
         }  
      k1 += 2;
      k++;
      }  

   }  

c_egclse ((struct GDESC **)&gdesc);

/*
Free dynamic memory
*/
free(lineind);
}
