#include "protos.h"
#include "util.h"

void getpar
(
   char lasin[][CMLEN],      /* O: input image las name                */
   char hosst[],             /* O: stats file host name                */ 
   char hosin[][CMLEN],      /* O: input image host names              */
   long *num_inimages,       /* O: number of input images              */
   long num_image_bands[],   /* O: number of bands in each input image */
   long image_band_num[][MAXBND + 1], /* O: band numbers in each input image */ 
   long *num_inbands,        /* O: total number of input image bands   */
   long wind[][4],           /* O: input image windows                 */
   long print[],             /* O: print parameter option              */
   char print_file[],        /* O: output file                         */
   FILE    **clun,           /* O: coordinates file lun                */
   long *transform,          /* O: transformation flag                 */
   float   tmat[][MAX_COEFF],/* O: transformation matrix               */
   long *ncoeff,             /* O: number of coefficients              */ 
   char subc[],	             /* O: subcommand                          */
   long *nclass_asked,       /* O: no. classes requested               */
   char comment[]            /* O: user comments                       */
)

/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          4/19/89   K. A. Narayanan    conversion to 5.0
   3         10/26/89   K. A. Narayanan    corrected/improved window handling

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
struct	VARIABLE *var;               /* pointer to a variable in TAE block */

long	access,                     /* access type for getnam             */
	flag[MAXIMG][2],            /* flag for subwindows                */
	i,
	imglimit = MAXIMG,          /* maximum number of images           */
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

long	itype = ELONG;
long	len, tlen; 

char	buf[ERRLEN],                /* error message buffer               */
	cfin[CMLEN],                /* coordinates file name              */
	fileutil_file[CMLEN],       /* file for fileutil                  */
	hostfin[CMLEN];             /* transformation file host name      */

long    fatal_err = -1;
long    zero = 0;
long    one = 1;


/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input image parameters
---------------------------------------
*/
stat = c_getin (&block, "IN", &zero, lasin, hosin, num_inimages, wind, 
                image_band_num, num_image_bands, num_inbands, &imglimit, 
                &same_window, flag);

if (stat != E_SUCC) 
    c_errmsg ("Error accessing the input image", "grndspot-in", 
              &fatal_err);

for (i = 1; i < *num_inimages; i++)
   {
   if (!same_window ||
       wind[i][SS] != wind[0][SS] || wind[i][SL] != wind[0][SL])
      c_errmsg ("Image windows must be same", "grndspot-wind", 
                &fatal_err);
   }

/*
get the desired subcommand
--------------------------
*/
c_getstr (&block, "_SUBCMD", &one, &len, subc);

/*
get source of input (terminal, file, window, stat file)
and then get the input.
-------------------------------------------------------
*/
access = IREAD;
if (subc[0] == 'F')
   {
   tlen = CMLEN;
   if ((c_getnam (&block, "INFILE", &zero, cfin,"",&access) != E_SUCC)
      || (cfin[0] == '\0'))
      {   
        c_errmsg("Error opening coordinates file", "grndspot-cfile",
                   &fatal_err);
      }
/*
      open coordinates file
      ---------------------
*/
   if ((*clun = fopen (cfin, "r")) == NULL)
      c_errmsg ("Error opening coordinates file", "grndspot-cfile",
                 &fatal_err);
   fgets (buf, 80, *clun);   /* skip the first record   */
   }
else if (subc[0] == 'S')
   {
   stat = c_getnam (&block, "INSTAT", &zero, hosst, "stat", &access);
   if (stat != E_SUCC) 
      c_errmsg ("Error getting statistics file name", "grndspot-sfin", 
                 &fatal_err);
   if (c_getnum(&block,"NCLASS",&one,&itype,(unsigned char *)nclass_asked) == 0)
      *nclass_asked = 0;
   }  
else 
   {
   *clun = NULL;

   if (SVAL (*p_find (&block, "$RUNTYPE"), 0)[0] == 'B')
      c_errmsg ("GRNDSPOT-TERM cannot be run in batch", "grndspot-batch",
                &fatal_err);
   }   
/*
get transformation file
-----------------------
*/
if (subc[0] == 'T' ||  subc[0] == 'F')
   {
   stat = c_getnam (&block, "INGRID", &zero, hostfin, "grid", &access);
   if (stat != E_SUCC) 
      c_errmsg ("Fatal error encountered", "grndspot-fatal", &fatal_err);
   if (strlen (hostfin) >  0) 
      {
      *transform = TRUE;
      gettrmat (hostfin, tmat, ncoeff);
      }
   else *transform = FALSE;
   } 

/*
Get print option and open print file 
------------------------------------
*/
num_outfiles = 0;
strcpy (fileutil_file, "grndspot");
if (c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
              print_file) != E_SUCC)
   c_errmsg ("Error getting print option", "grndspot-print", &fatal_err);
/*
if both TERM and LP are selected use only LP
--------------------------------------------
*/
/*
if (print[LP]) print[TERM] = FALSE;
*/

if ((print[LP]) || (print[FIL]))
   {
   fd = fopen (fileutil_file, "w");
   strcpy (out_files[num_outfiles++], fileutil_file);
   }
else
   fd = NULL;
/*
get the user comment
--------------------
*/
c_getstr (&block, "TEXT", &one, &len, comment);

/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);
}
