#include "protos.h"
#include "util.h"

void getpolygon
(
    long *nclass_used,       /* I: number of classes to process          */
    long icno[],             /* I: set of class indices                  */
    long *isno[MAXCLS],      /* I: set of site indices                   */
    long nsites[],           /* I: number of sites for a class           */
    long area[1024][4],      /* I: array containing site bounding        */
    long nverts[],           /* I: number of vertices per polygon        */
    long nimages,            /* I: no. images                            */
    long num_image_bands[],  /* I: no. bands in each image               */
    long image_band_num[][MAXBND + 1],/* I: band nums in each input image*/
    long nbands,             /* I: no. bands                             */
    long ns,                 /* I: no. samples/line                      */
    long dtype,              /* I: image data type                       */
    char intae[][CMLEN],     /* I: input images las names                */
    char comment[],          /* I: user comment to store in output file  */
    char subc[]              /* I: subcommand                            */
)
/*
***************************************************************************

NAME: GETPOLYGON

PURPOSE: Outputs the greylevels from all the polygons.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1             2/87   B. Kovalick
   2          4/19/89   K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
float	grey[MAXBND];

long	header;
long	i, iband, ic, icl, ien, ii, iline, image, is, isamp, isl, ist, j, k;
long	line, nl, np, numxints, num_poly_pixels; 
long	pixcoord, shape[9], sp, sl;
long	xints[200]; 

char	cname[STNAME], sname[STNAME];

long    fatal_err = -1;
long    inf_err = 1;
long    zero = 0;
long    one = 1;

/*
initialize the image data read for all polygon rectangles
---------------------------------------------------------
*/
sprintf (buf, "\n GRNDSPOT INPUT IMAGE = %s", intae);
c_txtout (buf, print, fd);

/*
loop through the classes.
------------------------
*/
k=0;
for (i=0; i < *nclass_used; i++)
   {
   ic = icno[i];
   c_stget ((FILE **)&fd_sfin,"NAME",&ic,&zero,shape,(unsigned char *)cname);
   icl = shape[2];
   cname[icl] = '\0';

/*
   loop through sites.
   ------------------
*/
   for (j=0; j < nsites[i]; j++, k++, curent++)
      {
      sprintf (buf, "Processing polygon %d of %d of class %s ...",
               j+1, nsites[i], cname);
      c_errmsg (buf, "grndspot-process", &inf_err);

      is = isno[i][j];
      c_stget ((FILE **)&fd_sfin, "NAME", &ic, &is, shape,
                (unsigned char *)sname);
      isl = shape[2];
      sname[isl] = '\0';

      sp = area[k][SS];
      sl = area[k][SL];
      np = area[k][NS];
      nl = area[k][NL];

/*
      for each line in site--find the number of intersections with
      the polygon boundary.
      ------------------------------------------------------------
*/
      header = TRUE;
      num_poly_pixels = 0;
      for (iline = 1; iline <= nl; iline++)
         {
         line = sl + iline - 1;
         for (image = 0; image < nimages; image++)
            for (iband = 0; iband < num_image_bands[image]; iband++)
               c_eread ((struct FDESC **)&fdesc[image], 
			&image_band_num[image][iband], &line, 
                        (unsigned char *)&iobuf[iband * ns], &one);
         sect (xaxis[k], yaxis[k], nverts[k], line, 
               xints, &numxints);

         if (numxints == 1)         /* need at least 2 pts for */
            {
            numxints = 2;           /* looping structure to work */
            xints[1] = xints[0];
            } 

/*
 find interior points in the polygon. 

*/
         for (ii = 1; ii < numxints; ii++)
            {
            if (insde(xaxis[k], yaxis[k], nverts[k], 
                      xints[ii-1], xints[ii], line)) 
               {
               ist = xints[ii-1];
               ien = xints[ii];
               pixcoord = xints[ii-1]; /* actual pixel coord. */
               for (isamp = ist; isamp <= ien; isamp++)
                  {
                  num_poly_pixels++;
                  for (iband = 0; iband < nbands; iband++)
                     grey[iband] = iobuf[iband * ns + isamp - 1];

/*
                  ------ write out the greylevels for this point ----

*/
                  if (num_poly_pixels > 1) header = FALSE;
                  dumpvals (grey, nbands, dtype, 
                            pixcoord, line, header, comment, 
                            cname, sname, subc);
                  pixcoord++;
                  }  
               }  
            }  
         }  
      sprintf (buf, " Number of pixels in polygon = %d\n", num_poly_pixels);
      c_errmsg (buf, "grndspot-polynpix", &inf_err);
      }  
   }  
if (k == 0) 
   g_errmsg ("No sites within image bounds", "grndspot-allout", &fatal_err); 

/*
close all files and free storage
--------------------------------
*/
free (isno_matrix);
free (x_matrix);
free (y_matrix);
free (iobuf);

c_stclse ((FILE **)&fd_sfin); 
for (image = 0; image < nimages; image++) 
    c_eclose ((struct FDESC **)&fdesc[image]);

}
