
/***********************************************************************

NAME: GRNDSPOT

PURPOSE: Obtains greylevels for specified image pixels

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          2/25/87   B. Kovalick        
   3          4/12/89   K. A. Narayanan    Conversion to 5.0
   4         10/26/89   K. A. Narayanan    Modified GETPAR.C, GETWINDOW.C,
                                           and DUMPVALS.C to handle correctly 
                                           window and file output
   5	     10/28/89	B.Ailts		   Added statics to run on PN9050
					   Now allows TERM and LP options at
					   the same time
   6	     03/90	B.Davis		   removed a few statics to run on
					   PN9080
   7         31-JUL-1990  W. Xia           fixed problem when user entered
                                           invalid class name.
   8         05-AUG-1991  T. Baltzer       fixed modified code from GSFC so
                                           that it will actually compile and
					   execute!
   9         01-Sep-1992  D. Etrheim       Check the status of getnam
   10	     11-Jun-1994  D. Eichacker	   Added dynamic allocation for the
					   FILE subcommand.
					   Removed Window parameter for 
					   subcommand WIND.
   11        21-Dec-1999  D. Hames         Remove MAXNL and replace with nl.
					   Modified to support images > 2 gig.
					   Add prototypes. SRF #893

					   

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get parameters from TAE
   Call FILSPEC set up image i/o specifications
   If subcommand = WINDOW 
      Call GETWINDOW to get grey levels of all pixels within
      the user specified window and output them to terminal/printer 
      and/or output file
   Else If subcommand = STAT
      Call STATCHK to open file and check no. of classes
      Call DYN_ALLOC to allocate storage for site numbers for each class
      Call INDICES to get class and site indices
      Call SETAREA to get the bounding rectangles for polygons
      Call GETPOLYGON to process polygons and output greylevels
         to terminal/printer and/or output file
   Else Until all coordinates are taken care of
      Call GETCOORD to get next set of coordinates from TAE or file
      If transformation is required call TRANSFORM to transform the
         input coordinates to (pixel,line)
      Call GET_GREYLEVEL to get the grey levels for these bands
      Call OUTPUT to output coordinates to terminal/printer and/or
         output file
      If user wants more coordinates to be processed
         Call FILSPEC to reopen input image
   Inform user about number of coordinates processed,
      and number of invalid coordinates
   Call COMPLETE to inform user about successful completion

***********************************************************************/
/* defined only in this file, used as a flag in grndspot.h */
#define GLOBALVARS 
#include "protos.h"
#include "util.h"
#include "transform.h"

void main
(void) /* grndspot */
{

FILE	*clun;                    /*  logical units for coord file,   */

CODE	term;

float   *coord;                  /*  pixel, line coords              */
float	*coord1;                 /*  x,y coords                      */
float	*grey;                   /*  grey levels                     */
float	tmat[2][MAX_COEFF];      /*  transformtaion coeffs           */

static	long	flag = TRUE;     /*  flag to indicate more processing*/
static  long	image_band_num[MAXIMG][MAXBND + 1]; 
	                         /*  band numbers in each input image*/
long	area[1024][4];           /*  bounding rectangles             */
long	*coord_flag;   /*  coordinate validity flags       */
long	dtype;                   /*  input image data type           */
long	icno[MAXCLS];            /*  class indices                   */
long	*isno[MAXCLS];           /*  sites in each class             */
long	lmin, lmax;
long	nbands;                  /*  total number of input image bands*/
long	nclass_asked;            /*  # classes asked for stat input  */
long	nclass_used;             /*  actual # of classes used        */
long	ncoeff;                  /*  no. transformation coefficients */
long	ncoord = 0;              /*  total no. coordinates processed */
static long	nsites[MAXCLS];  /*  # sites per class               */
long	numcoord;                /*  no. coordinates                 */
static long num_image_bands[MAXIMG]; /*  number of bands in each input image*/
long	num_inimages;            /*  number of input images          */
long	numinv = 0;              /*  no. invalid coordinates         */
static long nverts[MAX_POLYGON]; /*  no. verts. per polygon          */
long	transform_flag;          /*  transformation flag             */
long	xoff, yoff;              /*  master coordinates              */
static long wind[MAXIMG][4];     /*  input image windows             */

char	ans[4];                  /*  user response                   */
static char	(*cname)[11];         /*  point names                     */
char	comment[41];             /*  comment to store in sas file    */
static	char	hosin[MAXIMG][CMLEN];   
                                 /*  input images host names         */
char	hosst[CMLEN];            /*  input stats file                */
static	char	lasin[MAXIMG][CMLEN];
                                 /*  input images las names          */
char	print_file[CMLEN];       /*  print file name                 */
char	subc[5];                 /*  source of input coordinates     */
char	s[81];

long    fatal_err = -1;
long    inf_err = 1;
long    zero = 0;
long    one = 1;

/*
get tae parameters
------------------	
*/
getpar( lasin, hosst, hosin, &num_inimages, 
                 num_image_bands, image_band_num, &nbands, 
                 wind, print, print_file, 
                 &clun, &transform_flag, tmat, &ncoeff, 
                 subc, &nclass_asked, comment);

/*
allocate memory
---------------	
*/

if (strcmp (subc, "FILE")  == 0)
    {    /* Get the number of lines in the file  */
    while(fgets(s,80,clun) != NULL)
      ncoord++;
    rewind(clun);
    fgets(s,80,clun); /* Ignore First Line of the file */


    cname = (char (*)[11]) malloc(ncoord * 11 * sizeof(char));
    if (cname == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    coord_flag = (long *) malloc(ncoord * sizeof(long));
    if (coord_flag == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    coord = (float *) malloc((2 * ncoord) * sizeof(float));
    if (coord == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    coord1 = (float *) malloc((2 * ncoord) * sizeof(float));
    if (coord1 == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    grey = (float *) malloc((ncoord*MAXBND)* sizeof(float));
    if (grey == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    ncoord = 0;
    }
else
    {
    cname = (char (*)[11]) malloc(MAX_COORD * 11 * sizeof(char));
    if (cname == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    coord_flag = (long *) malloc(MAX_COORD * sizeof(long));
    if (coord_flag == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    coord = (float *) malloc((2 * MAX_COORD)* sizeof(float));
    if (coord == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    coord1 = (float *) malloc((2 * MAX_COORD) * sizeof(float));
    if (coord1 == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    grey = (float *) malloc(MAX_DIM * sizeof(float));
    if (grey == NULL)
      c_errmsg("Error allocating dynamic memory", "grndspot-alloc", &fatal_err);
    }


/*
set up file specifications
--------------------------
*/
filspec( hosin, num_inimages, num_image_bands,
         image_band_num, wind, &dtype, subc, &xoff, &yoff);

/*
process according to specified subcommand
-----------------------------------------
*/
if (strcmp (subc, "WIND")  == 0)
   {

/*
   take the user specified window and process it
   ----------------------------------------------
*/
   getwindow (dtype, nbands, (long *)wind, lasin, comment, subc);

   }
else if (strcmp (subc, "STAT")  == 0)
   {
/*
   ---------------------------------------------------
   get the stat file.
   a maximum of 1023 sites with no more than 201
   vertices can be processed.

   statchk - opens file and checks no. of classes
   dyn_alloc - allocate storage for site numbers for each class
   indices - get class and site indices
   setarea - gets the bounding rectangles for polygons
   getpolygon - processes polygons to output greylevels
   ---------------------------------------------------
*/
   statchk (hosst, nclass_asked, &nclass_used);

   dyn_alloc (nbands, nclass_used, isno);

   indices (nclass_asked, nclass_used, icno, isno, nsites);

   setarea (nclass_used, icno, isno, nsites, area, nverts, &lmin, 
                  &lmax, xoff, yoff, wind);

   getpolygon (&nclass_used, icno, isno, nsites, area, nverts, num_inimages, 
               num_image_bands, image_band_num, nbands, wind[0][NS], dtype, 
               lasin, comment, subc);

   }
else /*  get individual coordinates */
   {

/*
   do until all coordinates are taken care of
   ------------------------------------------

*/
   while (flag)
      {

/*
      get next set of coordinates
      ---------------------------
*/
      getcoord (clun, cname, coord, &numcoord, &ncoord, &flag);

      if (numcoord  >  0) 
         {

/*
         transform them if required
	 --------------------------
*/
         if (transform_flag)
            transform (tmat, ncoeff, coord, coord1, numcoord);

/*
         get grey levels - store numcoord greylevels in grey
	 ----------------------------------------------------
*/
         get_greylevel (numcoord, coord, grey, nbands, wind[0][NS], 
                        &wind[0][NL], coord_flag, &numinv);

/*
         output grey level values of coordinates read in
 	 -----------------------------------------------
*/
         output (clun, transform_flag, cname, coord1, coord, 
                 coord_flag, grey, nbands, numcoord, lasin, 
                 dtype, comment);

         }  

/*
      if coordinates are input from tae inquire if there are more
      -----------------------------------------------------------
*/
      if (clun  ==  NULL) 
         {
         ans[0] = ' ';
         while (! (ans[0]  ==  'y'  ||  ans[0]  ==  'Y'  || 
            ans[0]  ==  'n'  ||  ans[0]  ==  'N'))
            {
            t_write ("$More coordinates to be processed? (Y/N) :", T_PROMPT);
            t_read (ans, &term);
            if (ans[0]  ==  'n'  ||  ans[0]  == 'N') flag = FALSE;
            }  
         }  
/*
         if more coordinates are to be processed reopen the input images
         ---------------------------------------------------------------
*/
         if (flag)
            filspec (hosin,num_inimages, num_image_bands, image_band_num,wind,
		     &dtype,subc, &xoff,&yoff);
      }  
/*
informational messages
----------------------
*/
   sprintf (buf, "Number of coordinates processed: %d", ncoord);
   c_errmsg (buf, "grndspot-ncoord", &inf_err);
   if (numinv  !=  0) 
      {
      sprintf (buf, "Number of coordinates out of range: %d", numinv);
      c_errmsg (buf, "grndspot-nrange", &inf_err);
      }

   }  
/*
free dynamic memory
-------------------
*/
free(grey);
free(coord);
free(coord1);
free(coord_flag);
free(cname);

/*
issue completion message
------------------------
*/
if (print[FIL])
   c_complt ("GRNDSPOT", (char (*)[256])print_file, &one);
else
   c_complt ("GRNDSPOT",(char (*)[256])" ", &zero);
}
