#include "protos.h"

void indices
(
    long nc,                    /* I: number of classes specified by user */
    long ncproc,                /* I: number of classes to be analyzed */
    long icno[MAXCLS],          /* O: set of class indices */
    long *isno[MAXCLS],         /* I/O: set of site indices for each class */
    long nsites[MAXCLS]         /* O: no. of sites to be analyzed for a class */
)
/*
-----------------------------------------------------------
NAME: INDICES

PROGRAM HISTORY:
      Version            Date             Author
      -------            ----             ------
         5              apr-89           K. Narayanan    copied from ZIP and
                                                         deleted unwanted code
         4              mar-89           K. Narayanan    was trying to free
                                                         unallocated space
         3              july-88          B. Allam
         2              feb-87	         R. Mckinney
         1              june-84          Y. Sheu
  
FUNCTION:    Receives user specified class and site names and
             retrieves their associated index numbers from statistics 
             file.

------------------------------------------------------------------------
*/
{
char    numstr[3],           /* reply for all sites */
        stname[MAXCLS][100], /* class name */
        str[3],              /* reply for continue? */
        sname[99][100];      /* site name */
long	shape[9],            /* statistics data descriptor */
        scount,               /* no. of user errors entering site name */
        count = 0,            /* no. of user errors entering class name */
        ierr, stat, cerr, serr,/* status of calls to statistics file */
        ic,                  /* class index number */
        is,                  /* site index number */
        ns;                  /* no. of sites per class */
long    found, resp, name, /* loop control */
        done, numsite, given; /* variables */
long    k, nchk, i, j;     /* counter variables */

long    fatal_err = -1;
long    inf_err = 1;
long    zero = 0;


/*
loop through classes
---------------------
*/
for (i=0; i < ncproc; i++)
  {
   count=0;
   if (nc != 0) 
     {
      found = 0;
/*
not all classes being used => prompt user for name and
check against stats file
-------------------------------------------------------
*/
      while (!found)
        {
         printf("\n$_Enter Class name : ");    
         scanf("%s", stname[i]);
         ierr = c_stfdcl ((FILE **)&fd_sfin, stname[i], &ic); 
         if (ierr == E_GNLIO)
           g_errmsg("general I/O error", "grndspot-i/o", &fatal_err);
/*
---------------------------------------------
 class name not found.
 user is given 3 chances to enter valid name.
---------------------------------------------

*/
         if (ierr == E_NOCLN) 
           {
            count=count+1;
            if (count == 3) 
              g_errmsg("3 Errors for class name.  Restart.", "grndspot-cname",
                        &fatal_err);
            else
             {
              c_errmsg("Class not found.  3 chances to reenter a class name.",
                       "grndspot-cname", &inf_err);
              continue;
             } 
           } 
         else
           found = 1;
/*
--------------------------------------------------------------------
 write class index # into list of class indices if not already given
--------------------------------------------------------------------
*/
         given = FALSE;
         for (nchk = 0; nchk <= i; nchk++)
           if (icno[nchk] == ic) given = TRUE;
         if (given)  
           {
            c_errmsg("Class name already specified.", "grndspot-cname",
                      &inf_err);
            continue;
           } 
         found = 1;
        } 

      icno[i]=ic;
/*
--------------------------------------------------
 prompt user for # of sites and read from terminal
--------------------------------------------------
*/
       numsite = 0;
       while (!numsite)
         {
          printf(" Enter the number of Sites to process for Class %s\n", stname[i]); /*--- (20) ---*/
          printf(" :-");
          scanf("%d", &nsites[i]);
          if (nsites[i] < 0) 
            {
             printf("$_***** INVALID RESPONSE *****\n");    
             continue;
            }  
/*
-----------------------------------------
 check requested # of sites vs. # in file
-----------------------------------------
*/
         stat = c_stget((FILE **)&fd_sfin, "NSITES", &ic, &zero, shape, 
			(unsigned char *)&ns);
         if (stat != E_SUCC)
           g_errmsg ("Error getting no. of sites", "grndspot-nsite",
                      &fatal_err);
         if (ns == 0) 
           {
            c_errmsg ("No sites present for class.", "grndspot-nosite",
                       &inf_err);
            printf("$_Continue? (YES/NO):");
            scanf("%s", str);
            if (str[0] == 'N' || str[0] == 'n')
            g_errmsg ("GRNDSPOT has aborted at your request", "grndspot-abort",
                       &fatal_err);
           }  
         if (nsites[i] == 0) 
           nsites[i] = ns;
         else if (nsites[i] > ns) 
           {
            done = 0;
            while (!done)
              {
    /*25 */    printf("$_Class: %s does not have %d Sites\n", stname[i], nsites[i]);
               printf("$_Do you wish to use all available Sites? (YES/NO):");      /*--- (320) ---*/
               resp = 0;
               while (!resp)
                 {
                  scanf("%s", numstr);      
                  if (numstr[0] == 'y' || numstr[0] == 'Y') 
                    nsites[i] = ns;
                  else if (numstr[0] == 'n' || numstr[0] == 'N')
                    {
                     done = 1; 
                     break;
                    }
                  else
                    {
                     printf("$_***** INVALID RESPONSE *****\n");    
                     printf("$_Respond with YES or NO :");
                     continue;
                    }  
                  resp = 1;
                 } 
               done = 1;
              } 
               if (done)
                 continue;
           }  
         numsite = 1; 
        } 
/*
---------------------------------------------------------------- 
 if not all sites are being used then prompt user for site names
---------------------------------------------------------------- 
*/
      if (nsites[i] != ns) 
        {
         printf ("$_Enter Site names , 1 per line : ");     
         for (j=0; j < nsites[i]; j++)
           {
            scount=0;
            name = 0;
            while (!name)
              {
               scanf("%s", sname[j]);     
               cerr = c_stfdsi ((FILE **)&fd_sfin,stname[i],sname[j],&ic,&is); 
/*
------------------------------------------
 site name not found.
 user given 3 chances to enter valid name.
------------------------------------------
*/
               if (cerr == 3) 
                 {
                  scount=scount+1;
                  if (scount == 3) 
                     g_errmsg ("3 Errors for site name. Restart", 
                               "grndspot-sname", &fatal_err);
                  else                                  
                    {
                     c_errmsg ("Site not found. 3 chances to reenter a site name."
                               , "grndspot-sname", &inf_err);	
                     printf ("\n$_Enter Site name again : ");     
                     continue;
                    }  
                 } 
               name = 1;
              } 
/*
---------------------------------------------
 write site index # into list of site indices
---------------------------------------------
*/
            isno[i][j] = is;
           }  
/*
----------------------------------------------------------------
 all sites are being used. site indices incremented sequentially
----------------------------------------------------------------
*/
        }
      else
        {
         for (j=0; j < ns; j++)
           isno[i][j] = j+1;
        }  
     } 
/*
------------------------------------------------
 all classes and all sites requested.
 class and site indices incremented sequentially
------------------------------------------------
*/
   else
     {
      icno[i] = i+1;
      k = i + 1;
      serr = c_stget ((FILE **)&fd_sfin,"NSITES",&k,&zero,shape,
                      (unsigned char *)&nsites[i]); 
      if (serr != E_SUCC) 
         g_errmsg ("STAT I/O error for sites. Check stats file", 
                   "grndspot-site", &fatal_err);
      for (j=0; j < nsites[i]; j++)
        {
         isno[i][j] = j+1;
        }  
     } 
  } 

}
