#include "protos.h"

long	insde
( 
   long xverts[],		/* I/O: x coordinate of vertices */ 
   long yverts[], 		/* I/O: y coordinate of vertices */ 
   long nverts,			/* I: number of vertices */ 
   long xpos1,			/* I: x-coord of first intersection pt */
   long xpos2,			/* I: x-coord of second intersection pt */ 
   long ypos			/* I: y-coord of test point */ 
)

/*
-----------------------------------------------------------------------
NAME: INSDE 

PROGRAM HISTORY:
      Version           date              Author
      -------           ----              ------
         4             5-apr-89          K. Narayanan copied from ZIP
         3            30-jun-88          B. Allam
         2            11-feb-87          R. Mckinney 
         1            15-mar-82          J. Marvin
 
FUNCTION: determine whether a point is interior to a polygon or not.

---------------------------------------------------------------------
  determines whether a point is interior to a polygon or not.
  the winding number of the test point is computed: consider the
  collection of vectors formed by connecting the test point to
  each of the vertices. add up all the rotation angles between the
  vertices (sequence dependent).  if the sum is smaller than pi,
  the point is exterior.
----------------------------------------------------------------------
*/
{
long	insde,                /* true if point is inside; else false */
	i;                    /* counter */
float	xpos;                 /* x-coord of two intersection points */
double  angsum,               /* sum of rotation angles between vertices */
        angnew;               /* angle between two vertices */

insde = FALSE;

/*
 check if two intersections are the same point or not
 if they are the same point, this point is either a 
 polygon vertex or on the polygon side
------------------------------------------------------
*/
if (xpos1  !=  xpos2)
  {
  xpos = (xpos1+xpos2)/2.;        
  xverts[nverts] = xverts[0];
  yverts[nverts] = yverts[0];
  if ( nverts < 3 ) 
    return(insde);
  else
   {
   angsum = 0.;

   for (i = 1; i <= nverts; i++)
     {

     angnew = angle(xpos, ypos, xverts[i-1], yverts[i-1], xverts[i], yverts[i]);

/*
 check for the polygon line
 if the angle is pi, the mid-point of two intersections is
 lying on the line connecting two vertices.  this means that it is
 on a polygon side line and this line is horizontal
------------------------------------------------------------------             
*/
     if (angnew  !=  3.1415926) /*go to 200;*/
       angsum = angsum + angnew; 
     else
       insde = TRUE;
     } /* for */
   } /* if */
 } /* if */
else insde = TRUE;

if ( abs(angsum) >=  3.1415926 ) 
   insde = TRUE;      /*--- (200) ---*/

return(insde); 
}
