#include "protos.h"
#include "util.h"

void output 
(
   FILE    *clun,           /* I: coordinates file desc            */
   long    transform,       /* I: transformation flag              */
   char    (*cname)[11],    /* I: point name                       */
   float   coord1[],        /* I: X, Y, coordinates                */
   float   coord[],         /* I: line, pixel coordinates          */
   long    coord_flag[],    /* I: coordinate validity flag         */
   float   grey[],          /* I: grey level                       */
   long    nbands,          /* I: total number of input image bands*/ 
   long    numcoord,        /* I: no. coordinates                 */
   char    intae[][CMLEN],  /* I: input images las names           */
   long    dtype,           /* I: image data type                  */
   char    comment[]        /* I: user comment to be stored        */
)

/*
***************************************************************************

NAME: OUTPUT

PURPOSE: Output grey levels on terminal, line printer, and text file

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          4/19/89   K. A. Narayanan    conversion to 5.0
   3	      6/27/94   D. J. Eichacker    Added c_txtout to print
					   COORDINATES OUT OF RANGE
					   message.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
float	x, y;

long	field, i, in, j, jpix, k, k1, lead, len, line, linelen, n, ngrey;

char	dash[131];
char	*buf;

for (i = 0; i < 130; i++) dash[i] = '-';
dash[130] = '\0';

if (print[TERM])
   linelen = 80;                   /*  line width for terminal */
else
   linelen = 132;                  /*  line width for line printer */

if (dtype == EBYTE || dtype == EWORD)/*  width for byte, int*2 field */
   field = 7;
else
   field = 13;                     /*  width for integer*4, real field */
if (print[TERM] || print[LP])       /* lp or term output */
   {                
   if((buf = (char *) malloc ((linelen +1) * sizeof(char))) == NULL)
     {
       c_lasmsg(TRUE, "grndspot-alloc", 
                 "Error allocating dynamic memory");

     }
   sprintf (buf, "\n GRNDSPOT INPUT IMAGE = %s", intae[0]);
   c_txtout (buf, print, fd);
   sprintf (buf, "\n\n COMMENT: %40s",comment);
   c_txtout (buf, print, fd);
   ngrey = 0;                                 
   while (ngrey < nbands)
      {
      in = 0;
      for (k = 0; k < numcoord; k++)
         {
         k1 = 2*k;
         line = (long)(coord[k1] + 0.5);
         jpix = (long)(coord[k1+1] + 0.5);
         x = coord1[k1];
         y = coord1[k1+1];
         if (!transform)            /* no transformation */
            {
            if (clun  ==  NULL)  /* input not from file */
               {
               lead = 11;                     /*  leader field width */
               n = (linelen-lead)/field;      /*  number of bands/line */
               if (ngrey+n  >  nbands) n = nbands-ngrey;
               if (k  ==  0) /* first coord or first on a line */
                  {
                  c_txtout("\n\n  LINE  PIXEL          GREY VALUES", print, fd);
                  len = lead + field - 2; 
                  for (j = 0; j < len; j++) buf[j] = ' ';
                  buf[len] = '\0';       
                  for (i = 0; i < n; i++) 
                     {
                     sprintf (&buf[strlen (buf)], "%3d", ngrey+i+1);
                     for (j = 0; j < field - 3; j++) strcat (buf, " ");
                     }
                  c_txtout (buf, print, fd);
                  strncpy (buf, dash, len = lead+n*field+1);
                  buf[len] = '\0';
                  c_txtout (buf, print, fd);
                  }

               if (coord_flag[k]) 
                  {  
                  if (dtype == EBYTE || dtype == EWORD)
                     {
                     sprintf (buf, " %5d %5d ", line, jpix);
                     for (i = 0; i < n; i++)
                     sprintf (&buf[strlen (buf)], "%6d ", 
                              (long) (grey[in+ngrey+i] + 0.5));
                     c_txtout (buf, print, fd);
                     }
                  else
                     {
                     sprintf (buf, " %5d %5d ", line, jpix);
                     for (i = 0; i < n; i++)
                     sprintf (&buf[strlen (buf)], "%12.6e ", grey[in+ngrey+i]);
                     c_txtout (buf, print, fd);
                     }
                  }
               else
		  {
                  sprintf (buf, " %5d %5d  COORDINATES OUT OF RANGE", 
                           line, jpix);
                  c_txtout (buf, print, fd);
		  }
               }

            else  
/*             
               input is from a file 
               --------------------
*/
               {
               lead = 22;
               n = (linelen-lead)/field;
               if (ngrey+n  >  nbands) n = nbands-ngrey;
               if (k  ==  0)
                  {
                  c_txtout ("\n\n POINT ID    LINE  PIXEL          GREY VALUES",
                           print, fd);
                  len = lead + field - 2; 
                  for (j = 0; j < len; j++) buf[j] = ' ';
                  buf[len] = '\0';
                  for (i = 0; i < n; i++) 
                     {
                     sprintf (&buf[strlen (buf)], "%3d", ngrey+i+1);
                     for (j = 0; j < field - 3; j++) strcat (buf, " ");
                     }
                  c_txtout (buf, print, fd);
                  strncpy (buf, dash, len = lead+n*field+1);
                  buf[len] = '\0';
                  c_txtout (buf, print, fd);
                  }

               if (coord_flag[k]) 
                  {
                  if (dtype == EBYTE || dtype == EWORD)
                     {
                     sprintf (buf, "%10s %5d %5d ", cname[k], line, jpix);
                     for (i = 0; i < n; i++)
                     sprintf (&buf[strlen (buf)], "%6d ", 
                              (long) (grey[in+ngrey+i] + 0.5));
                     c_txtout (buf, print, fd);
                     }
                  else
                     {
                     sprintf (buf, "%10s %5d %5d ", cname[k], line, jpix);
                     for (i = 0; i < n; i++)
                     sprintf (&buf[strlen (buf)], "%12.6e ", grey[in+ngrey+i]);
                     c_txtout (buf, print, fd);
                     }  
                  }
               else
		  {
                  sprintf (buf, "%10s %5d %5d  COORDINATES OUT OF RANGE", 
                           cname[k], line, jpix);
                  c_txtout (buf, print, fd);
		  }
               }  

            }
         else  
/*  
            transformation applied, must print out x, y 
            -------------------------------------------
*/
            {
            if (clun  ==  NULL) 
               {
               lead = 33;
               n = (linelen-lead)/field;
               if (ngrey+n  >  nbands) n = nbands-ngrey;
               if (k  ==  0)
                  {
                  c_txtout 
                  ("\n\n    Y-COORD    X-COORD  LINE PIXEL          GREY VALUES",
                           print, fd);
                  len = lead + field - 2; 
                  for (j = 0; j < len; j++) buf[j] = ' ';
                  buf[len] = '\0';
                  for (i = 0; i < n; i++) 
                     {
                     sprintf (&buf[strlen (buf)], "%3d", ngrey+i+1);
                     for (j = 0; j < field - 3; j++) strcat (buf, " ");
                     }
                  c_txtout (buf, print, fd);
                  strncpy (buf, dash, len = lead+n*field+1);
                  buf[len] = '\0';
                  c_txtout (buf, print, fd);
                  }
               if (coord_flag[k]) 
                  {
                  if (dtype == EBYTE || dtype == EWORD)
                     {
                     sprintf (buf, "%10.2f %10.2f %5d %5d ", x, y, line, jpix);
                     for (i = 0; i < n; i++)
                     sprintf (&buf[strlen (buf)], "%6d ", 
                              (long) (grey[in+ngrey+i] + 0.5));
                     c_txtout (buf, print, fd);
                     }
                  else
                     {
                     sprintf (buf, "%10.2f %10.2f %5d %5d ", x, y, line, jpix);
                     for (i = 0; i < n; i++)
                     sprintf (&buf[strlen (buf)], "%12.6e ", grey[in+ngrey+i]);
                     c_txtout (buf, print, fd);
                     }  
                  }
               else
		  {
                  sprintf (buf, 
                           "%10.2f %10.2f %5d %5d  COORDINATES OUT OF RANGE", 
                           x, y, line, jpix);
                  c_txtout (buf,print,fd);  
		  }
               }
            else  
/*  
               input is from a file 
               --------------------
*/
               {
               lead = 44;
               n = (linelen-lead)/field;
               if (ngrey+n  >  nbands) n = nbands-ngrey;
               if (k  ==  0)
                  {
                  c_txtout 
      ("\n\n POINT ID      Y-COORD    X-COORD   LINE PIXEL          GREY VALUES", 
                     print, fd);
                  len = lead + field - 2; 
                  for (j = 0; j < len; j++) buf[j] = ' ';
                  buf[len] = '\0';
                  for (i = 0; i < n; i++) 
                     {
                     sprintf (&buf[strlen (buf)], "%3d", ngrey+i+1);
                     for (j = 0; j < field - 3; j++) strcat (buf, " ");
                     }
                  c_txtout (buf, print, fd);
                  strncpy (buf, dash, len = lead+n*field+1);
                  buf[len] = '\0';
                  c_txtout (buf, print, fd);
                  }

               if (coord_flag[k]) 
                  {
                  if (dtype == EBYTE || dtype == EWORD)
                     {
                     sprintf (buf, "%10s %10.2f %10.2f %5d %5d ", 
                              cname[k], x, y, line, jpix);
                     for (i = 0; i < n; i++)
                     sprintf (&buf[strlen (buf)], "%6d ", 
                               (long) (grey[in+ngrey+i] + 0.5));
                     c_txtout (buf, print, fd);
                     }
                  else
                     {
                     sprintf (buf, "%10s %10.2f %10.2f %5d %5d ", 
                              cname[k], x, y, line, jpix);
                     for (i = 0; i < n; i++)
                     sprintf (&buf[strlen (buf)], "%12.6e ", grey[in+ngrey+i]);
                     c_txtout (buf, print, fd);
                     }  
                  }
               else
		  {
                  sprintf (buf, 
                       "%10s %10.2f %10.2f %5d %5d  COORDINATES OUT OF RANGE", 
                           cname[k], x, y, line, jpix);
		  c_txtout (buf, print, fd);
		  }
               }  
            }  
         in += nbands;
         }  
      ngrey += n;
      }  
   free(buf);
   buf = NULL;
   }  
/*
text file output 
----------------
*/
if (print[FIL] && !print[TERM] && !print[LP]) /*output goes to a sas type file*/
   {
   /*  Allocate memory for buf.  The size of the buffer should be the
       number of bands multiplied by the field width, plus 42 for
       the leading columns 
   */
   if ((buf = (char *) malloc ((nbands * field + 42)*sizeof(char))) == NULL)
       {
       c_lasmsg(TRUE, "grndspot-alloc",
                 "Error allocating dynamic memory");
       }
   sprintf (buf, "\n GRNDSPOT INPUT IMAGE = %s", intae);
   c_txtout (buf, print, fd);
   sprintf (buf, "\n\n COMMENT: %40s",comment);
   c_txtout (buf, print, fd);
   k1 = 0;
   ngrey = 0;
   for (k = 0; k < numcoord; k++)
      {
      if (coord_flag[k]) 
         {
         line = (long)(coord[k1] + 0.5);
         jpix = (long)(coord[k1+1] + 0.5);
         if (transform)             /* transformation applied */
            {
            x = coord1[k1];
            y = coord1[k1+1];
            if (dtype == EBYTE || dtype == EWORD)
               {
               sprintf (buf, "%10s %8.2f %8.2f %5d %5d ", 
                        cname[k], x, y, line, jpix);
               for (i = 0; i < nbands; i++)
                  sprintf (&buf[strlen (buf)], "%6d ", 
                            (long) (grey[ngrey+i] + 0.5));
               c_txtout (buf, print, fd);
               }
            else if (dtype == ELONG) 
               {
               sprintf (buf, "%10s %8.2f %8.2f %5d %5d ", 
                        cname[k], x, y, line, jpix);
               for (i = 0; i < nbands; i++)
                  sprintf (&buf[strlen (buf)], "%12d ", 
                           (long) (grey[ngrey+i] + 0.5));
               c_txtout (buf, print, fd);
               }
            else /* real*4 */
               {
               sprintf (buf, "%10s %8.2f %8.2f %5d %5d ", 
                        cname[k], x, y, line, jpix);
               for (i = 0; i < nbands; i++)
                  sprintf (&buf[strlen (buf)], "%12.6f ", grey[ngrey+i]);
               c_txtout (buf, print, fd);
               } 
            }
         else 
/* 
            no transformation applied 
            -------------------------
*/
            {
               lead = 11;                     /*  leader field width */
               n = (linelen-lead)/field;      /*  number of bands/line */
               if (ngrey+n  >  nbands) n = nbands-ngrey;
               if (k  ==  0) /* first coord or first on a line */
                  {
                  c_txtout ("\n\n  LINE  PIXEL          GREY VALUES", print, fd);
                  len = lead + field - 2; 
                  for (j = 0; j < len; j++) buf[j] = ' ';
                  buf[len] = '\0';       
                  for (i = 0; i < nbands; i++) 
                     {
                     sprintf (&buf[strlen (buf)], "%3d", i+1);
                     for (j = 0; j < field - 3; j++) strcat (buf, " ");
                     }
                  c_txtout (buf, print, fd);
                  strncpy(buf,dash,len = lead+n*field+1);
                  buf[len] = '\0';
                  c_txtout(buf,print,fd);
                  }

               if (dtype == EBYTE || dtype == EWORD)
                  {
                  sprintf (buf, " %5d %5d ", line, jpix);
                  for (i = 0; i < nbands; i++)
                     sprintf (&buf[strlen (buf)], "%6d ", 
                              (long) (grey[ngrey+i] + 0.5));
                  c_txtout (buf, print, fd);
                  }
               else if (dtype == ELONG) 
                  {
                  sprintf (buf, "%5d %5d ", line, jpix);
                  for (i = 0; i < nbands; i++)
                     sprintf (&buf[strlen (buf)], "%12d ", 
                              (long) (grey[ngrey+i] + 0.5));
                  c_txtout (buf, print, fd);
                  }
               else /* real*4 */
                  {
                  sprintf (buf, "%5d %5d ", line, jpix);
                  for (i = 0; i < nbands; i++)
                     sprintf (&buf[strlen (buf)], "%12.6e ", grey[ngrey+i]);
                  c_txtout (buf, print, fd);
                  } 
            } 
         }  
      k1 += 2;
      ngrey += nbands;
      }  
   free(buf);
   buf = NULL;
   }  
}
