#ifndef PROTOS_H
#define PROTOS_H

#include "grndspot.h"

double  angle
(
   float x0,            /* I: x coordinate of pivot point */
   long y0,             /* I: y coordinate of pivot point */
   long x1,             /* I: x coordinate of first vertex */
   long y1,             /* I: y coordinate of first vertex */
   long x2,             /* I: x coordinate of second vertex */
   long y2              /* I: y coordinate of second vertex */
);

void dumpvals
(
    float grey[],     /* I: grey values                            */
    long nbands,      /* I: no. bands                              */
    long dtype,       /* I: image data type                        */
    long isamp,       /* I: sample coordinate                      */
    long line,        /* I: line coordinate                        */
    long header,      /* I: flag indicating whether to display header */
    char comment[],   /* I: user comment                           */
    char field1[],    /* I: annotation info                        */
    char field2[],    /* I: at the beginning of each record        */
    char subc[]       /* I: subcommand                             */
);

void dyn_alloc
(
    long totbnd,          /* I: total no. of bands */
    long ncls,            /* I: total no. of classes */
    long *isno[MAXCLS]    /* O: site indices */
);

void filspec
(
    char hosin[][CMLEN],      /* I: input image host names             */
    long num_inimages,        /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* I: band numbers in each      */
    long wind[][4],           /* I: input image windows                */
    long *in_type,            /* O: input image data type              */
    char subc[],              /* I: source of coordinate input         */
    long *xoff,               /* O: master sample coordinate           */
    long *yoff                /* O: master line coordinate             */
);

void g_errmsg
(
    char message[],    /* I: error message                         */
    char key[],        /* I: message key                           */
    long *severity     /* I: dummy - included for compatibility    */
);


void getcoord 
(
   FILE *clun,                  /* I: logical units for coord file,   */
   char (*cname)[11],           /* I/O: coordinate name               */
   float coord[],               /* O: coordinates                     */
   long *numcoord,              /* O: total no. coordinates           */
   long *ncoord,                /* I/O: total no. coordinates         */
   long *flag                   /* I/O: more coordinates indicator    */
);


void get_greylevel
(
    long numcoord,      /* I: no. coordinates           */
    float coord[],      /* I: coordinates               */
    float grey[],       /* O: grey levels               */
    long nbands,        /* I: no. coordinates           */
    long np,            /* I: no. pixels per line       */
    long *nl,           /* I: no. lines in image        */
    long coord_flag[],  /* O: coordinate validity flags */
    long *numinv        /* I/O: no. invalid coordinates */
);

void getpar
(
   char lasin[][CMLEN],      /* O: input image las name                */
   char hosst[],             /* O: stats file host name                */
   char hosin[][CMLEN],      /* O: input image host names              */
   long *num_inimages,       /* O: number of input images              */
   long num_image_bands[],   /* O: number of bands in each input image */
   long image_band_num[][MAXBND + 1], /* O: band numbers in each input image */
   long *num_inbands,        /* O: total number of input image bands   */
   long wind[][4],           /* O: input image windows                 */
   long print[],             /* O: print parameter option              */
   char print_file[],        /* O: output file                         */
   FILE    **clun,           /* O: coordinates file lun                */
   long *transform,          /* O: transformation flag                 */
   float   tmat[][MAX_COEFF],/* O: transformation matrix               */
   long *ncoeff,             /* O: number of coefficients              */
   char subc[],              /* O: subcommand                          */
   long *nclass_asked,       /* O: no. classes requested               */
   char comment[]            /* O: user comments                       */
);


void getpoly
(
    long ic,              /* I: class index number */
    long is,              /* I: site index number */
    long xaxis[],
    long yaxis[],         /* O: x and y co-ords of site vertices */
    long *nverts,         /* O: number of vertices */
    long xoff,
    long yoff             /* I: x and y offsets */
);

void getpolygon
(
    long *nclass_used,       /* I: number of classes to process          */
    long icno[],             /* I: set of class indices                  */
    long *isno[MAXCLS],      /* I: set of site indices                   */
    long nsites[],           /* I: number of sites for a class           */
    long area[1024][4],      /* I: array containing site bounding        */
    long nverts[],           /* I: number of vertices per polygon        */
    long nimages,            /* I: no. images                            */
    long num_image_bands[],  /* I: no. bands in each image               */
    long image_band_num[][MAXBND + 1],/* I: band nums in each input image*/
    long nbands,             /* I: no. bands                             */
    long ns,                 /* I: no. samples/line                      */
    long dtype,              /* I: image data type                       */
    char intae[][CMLEN],     /* I: input images las names                */
    char comment[],          /* I: user comment to store in output file  */
    char subc[]              /* I: subcommand                            */
);

void gettrmat
(
   char hostfin[],              /* I: transformation matrix host name   */
   float   tmat[2][MAX_COEFF],  /* O: transformation matrix             */
   long    *ncoeff              /* O: no. coefficients                  */
);

void getwindow
(
    long dtype,           /* I: image data type                       */
    long nbands,          /* I: no. bands                             */
    long wind[4],         /* I: coordinate window                     */
    char intae[][CMLEN],  /* I: input images las names                */
    char comment[],       /* I: user comment to store in output file  */
    char subc[]           /* I: subcommand                            */
);

void indices
(
    long nc,                    /* I: number of classes specified by user */
    long ncproc,                /* I: number of classes to be analyzed */
    long icno[MAXCLS],          /* O: set of class indices */
    long *isno[MAXCLS],         /* I/O: set of site indices for each class */
    long nsites[MAXCLS]         /* O: no. of sites to be analyzed for a class */
);

long    insde
(
   long xverts[],               /* I/O: x coordinate of vertices */
   long yverts[],               /* I/O: y coordinate of vertices */
   long nverts,                 /* I: number of vertices */
   long xpos1,                  /* I: x-coord of first intersection pt */
   long xpos2,                  /* I: x-coord of second intersection pt */
   long ypos                    /* I: y-coord of test point */
);


void output
(
   FILE    *clun,           /* I: coordinates file desc            */
   long    transform,       /* I: transformation flag              */
   char    (*cname)[11],    /* I: point name                       */
   float   coord1[],        /* I: X, Y, coordinates                */
   float   coord[],         /* I: line, pixel coordinates          */
   long    coord_flag[],    /* I: coordinate validity flag         */
   float   grey[],          /* I: grey level                       */
   long    nbands,          /* I: total number of input image bands*/
   long    numcoord,        /* I: no. coordinates                 */
   char    intae[][CMLEN],  /* I: input images las names           */
   long    dtype,           /* I: image data type                  */
   char    comment[]        /* I: user comment to be stored        */
);

void parse
(
    char in[],      /* I: input string                 */
    long *index,    /* I/O: parameter index            */
    char out[],     /* O: output parameter             */
    long *len       /* O: parameter length             */
);

void readnext 
(
   FILE    *lun,         /* I: coordinates file desc        */
   char    cname[],      /* O: point names                  */
   float   coord[2],     /* O: coordinates                  */
   long    *flag         /* I/O: EOF flag                   */
);


void sect
(
    long xverts[],
    long yverts[],        /* I/O: x and y vertices */
    long nverts,          /* I: no. of vertices */
    long line,            /* I: line no. to be checked */
    long sects[],         /* I/O: x coords of intersections */
    long *nsect           /* I/O: no. of intersections */
);

void setarea
(
    long ncx,                  /* I: no. of classes to be analyzed */
    long icno[],               /* I: set of class indices */
    long *isno[MAXCLS],        /* I/O: set of site indices */
    long nsites[],             /* I: no. of sites for a class */
    long area[1024][4],        /* O: array containing site bounding rectangle */
    long nverts[],             /* O: no. of vertices */
    long *lmin,                /* O: min line no. for all site boundings */
    long *lmax,                /* O: max line no. for all site boundings */
    long xoff,
    long yoff,                 /* I: x and y offsets */
    long window[][4]           /* I: window parameters */
);

void statchk
(
    char hostat[],              /* I: stats file host name */
    long nclass,                /* I: no. of classes */
    long *ncproc                /* O: no. of classes processed */
);

void verts
(
    long size[4],           /* O: site bounding rectangle   */
    long xaxis[],
    long yaxis[],           /* I: x and y image coordinates */
    long nverts             /* I: no. of vertices           */
);

#endif
