#include "protos.h"

void setarea
(
    long ncx,                  /* I: no. of classes to be analyzed */
    long icno[],               /* I: set of class indices */
    long *isno[MAXCLS],        /* I/O: set of site indices */
    long nsites[],             /* I: no. of sites for a class */
    long area[1024][4],        /* O: array containing site bounding rectangle */
    long nverts[],             /* O: no. of vertices */
    long *lmin,                /* O: min line no. for all site boundings */
    long *lmax,                /* O: max line no. for all site boundings */
    long xoff,
    long yoff,                 /* I: x and y offsets */
    long window[][4]           /* I: window parameters */
)
/*
-----------------------------------------------------------------------
NAME: SETAREA

PROGRAM HISTORY:
      Version          date                Author
      -------          ----                ------
         4            apr-89              K. Narayanan copied from ZIP
                                                       and made small changes
         3            jul-88              B. Allam
         2            feb -87             R. Mckinney
         1            june-84             Y. Sheu
 
FUNCTION:   To establish which sites need to access image data and
            enter bounding rectangles of these sites into an array.

----------------------------------------------------------------------
*/
{
long	shape[9];             /* shape vector */
long	size[4];              /* site bounding rectangle */
char	sname[80], cname[80]; /* class and site names */
long	k,                    /* counter for no. of sites */
        i, j,                 /* counter variables */
        stat,                 /* status returned */
        ic,                   /* class index number */
        is,                   /* site index number */
        lbot;                 /* line no. of bottom site bounding */

long fatal_err = -1;
long inf_err = 1;
long zero = 0;
/*
loop through classes
--------------------
*/
*lmin = 99999;
*lmax = 0;
k = 0;
for (i= 0; i < ncx; i++)
   {
   ic = icno[i];
   stat = c_stget((FILE **)&fd_sfin, "NAME", &ic, &zero, shape, 
		(unsigned char *)cname);
   if (stat != E_SUCC)
     g_errmsg ("Error reading class names", "grndspot-class", &fatal_err);
/*
loop through sites
------------------
*/
   for (j=0; j < nsites[i]; j++)
      {
      is = isno[i][j];
      stat = c_stget((FILE **)&fd_sfin,"NAME",&ic,&is,shape,
		     (unsigned char *)sname);
      if (stat != E_SUCC)
        g_errmsg ("Error reading site names", "grndspot-sname", &fatal_err);
/*
get site vertices from statistics file 
---------------------------------------
*/
      getpoly (ic, is, xaxis[k], yaxis[k], &nverts[k], xoff, yoff);


/*
get bounding rectangles of site
-------------------------------
*/
      verts (size, xaxis[k], yaxis[k], nverts[k]);

/*
check if outside the window or not
----------------------------------
*/
      if (size[0] > window[0][2]  ||  size[1] > window[0][3] || 
      size[2] < 1  ||  size[3]  <  1) 
         {
         c_errmsg ("Polygon vertex outside image for class and site", 
                   "grndspot-poly", &inf_err);
         isno[i][j] = 0;
         }

/*
find intersections of site rectangles and windows
-------------------------------------------------
*/
      else
         {
         area[k][0] = max(size[0], 1);
         area[k][1] = max(size[1], 1);
         area[k][2] = min(size[2], window[0][2]) - area[k][0]+1;
         area[k][3] = min(size[3], window[0][3]) - area[k][1]+1;

/*
find bounding lines for all sites
---------------------------------
*/
         if (area[k][0]  <  *lmin) *lmin=area[k][0];
         lbot=area[k][0]+area[k][2]-1;
         if (lbot  >  *lmax) *lmax=lbot;
         k = k + 1;
         if (k  >  1023)
           g_errmsg ("Total number of selected sites for all selected classes exceeds 1023",
		     "grndspot-maxsit", &fatal_err);
         }  /* if */
      }  /* do */
   }  /* do */
if (k == 0)
   g_errmsg ("No sites within image bounds", "grndspot-nosite", &fatal_err); 

}
