#include "protos.h"
#include "transform.h"

void transform
(
    float tmat[2][MAX_COEFF],   /* I: transformation matrix      */
    long ncoeff,                /* I: no. of coefficients        */
    float coord[],              /* I/O: coordinates saved        */
    float coord1[],             /* O: X, Y, coordinates          */
    long numcoord               /* I: no. of coordinates         */
)
/*
***************************************************************************

NAME: TRANSFORM

PURPOSE: Transforms using specified transformation matrix. COORD contains 
	input coordinates and will contain transformed coordinates on 
	output. Input coordinates will be saved in COORD1.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          4/19/89   K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Compute degree of polynomial
   Do for number of coefficients
      Perform transformation using the transformation matrix

***************************************************************************
*/
{

float	fx, fy, temp, temp1, temp2;

long	i, j, k, k1, n, ndeg;
/*
compute degree of polynomial
----------------------------
*/
ndeg = (long) sqrt ((double) ((8*ncoeff+1)-1)) / 2 - 0.5;

/*
perform the user specified transformation
-----------------------------------------
*/
k1 = 0;
for (k = 1; k <= numcoord; k++)
   {
   fy = tmat[0][0];
   fx = tmat[1][0];
   n = 1;
   for (i = 1; i <= ndeg; i++)
      {
      for (j = i; j >= 0; j--)
         {
         
         if (coord[k1+1]  ==  0.0  &&  j  ==  0) temp1 = 1.0;
         else temp1 = (float) pow ((double) coord[k1+1], (double) j);
         if (coord[k1]  ==  0.0  &&  (i-j)  ==  0) temp2 = 1.0;
         else temp2 = (float) pow ((double) coord[k1], (double) (i-j));
         temp = temp1*temp2;
         fy = fy+tmat[0][n]*temp;
         fx = fx+tmat[1][n]*temp;
         n++;
         }  
      }  

/*
	   save input coordinates
	   ----------------------
*/
   coord1[k1] = coord[k1];
   coord1[k1+1]= coord[k1+1];

/*
	   and store transformed coordinates in place of input coordinates
	   ---------------------------------------------------------------
*/
   coord[k1] = fy;
   coord[k1+1] = fx;

   k1 += 2;
   }

}
