#include "hindu.h"
#include "protos.h"
/* #define TERM 0
#define LP 1
#define FIL 2 */

void getpar
(
    char taein[][CMLEN], 	/* input image tae name                       */
    char hostin[][CMLEN], 	/* input image host name                      */
    char taeout[][CMLEN], 		/* output image tae name                      */
    char hostout[][CMLEN], 		/* output image host name                     */
    char hostemp[], 		/* temporary image host name                  */
    long *num_inimg,		/* number of input images                     */
    long (*num_img_bands)[MAXBND+1],	/* band numbers for each input image  */
    long *bands_per_img,		/* number of bands in each input image        */
    long *tot_inband,		/* total number of bands                      */
    long img_limit,		/* maximum image limit                        */
    long *same_wnd,		/* same window size: 1=YES, 0=NO              */
    long (*img_flag)[2],		/* flag for the window/band option            */
    long print[],
    char print_file[]		/* print file name                            */
)
/*
*****************************************************************************

NAME: GETPAR

PURPOSE:  Retrives user supplied parameter values and performs appropriate
          validity tests.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         08-Jul-87   N. Deodhar/SAR #212
   3.0         19-May-89   K. Kunska/STX	conversion to 5.0
   5.1         06-aug-93   D. Bridges           Added print parameter

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Call INLAS to initialize TAE.
   Get input image name and the temporary image name.
   Make sure the input image type is BYTE.
   Get the output image file name.
   Get the minimum and maximum acceptable number of clusters.
   Get the number of gray levels for histogram bins.
   Open the print file.
   Get the LAS global parameters: MSGTIM and MINMAX.

*****************************************************************************
*/

{

char    *prog_name = "HINDU"; /* program name                 */
long    one = 1;

char	fileutil_file[CMLEN],	/* file for fileutil                          */
	mnmx[4];		/* minmax parameter                           */

long	access,			/* access type                                */
	clus[2], 		/* minand max acceptable number of clusters   */
	dtype, 			/* input image data type                      */
	i,			/* loop counter and arrray index              */
	par_type,		/* parameter type                             */
	range[3],		/* number of gray levels per histogram bins   */
	stat,			/* return status                              */
	std_out;		/* logical unit for standard output           */

long zero  = 0;     /* Constant of zero   */
long two   = 2;     /* Constant of two    */
long three = 3;     /* Constant of three  */
long fatal_err = -1; /* fatal error flag  */

struct VARIABLE   *var;	/* pointer to a variable in the tae block     */
/*
Initialize LAS.
------------------
*/
c_inlas (&block, &std_out);

/*
Get input image name and the temporary image name.
Make sure the input image type is BYTE.
----------------------------------------------------
*/
access = IWRITE;
c_getnam (&block, "INTEMP", &zero, hostemp, "img", &access);  

stat = c_getin (&block, "IN", &zero, taein, hostin, num_inimg, window, 
                num_img_bands, bands_per_img, tot_inband, &img_limit, 
                same_wnd, img_flag);
if (stat !=  E_SUCC) 
   c_errmsg ("Cannot access the input image name", "hindu-getin", &fatal_err); 

for (i = 0; i < *num_inimg; i++)
   {
   c_getddr (hostin[i], &ddr); 
   dtype = ddr.dtype;
   if (dtype != EBYTE)
      c_errmsg ("Type of the input image must be BYTE", "hindu-dtype", 
                 &fatal_err);
   }

/*
Get the output image file name.
----------------------------------
*/
stat = c_getout (&block, "OUT", taeout, hostout); 
if (stat != E_SUCC) 
   c_errmsg ("Cannot access the output image name", "hindu-getout", 
              &fatal_err); 
num_outfiles = 0;
strcpy (out_files[num_outfiles++], taeout[0]);

/*
Get the minimum and maximum acceptable number of clusters.
Get the number of gray levels for histogram bins.
-------------------------------------------------------------
*/
par_type = ELONG;
c_getnum (&block, "MINMAXCL", &two, &par_type, clus);
minclus = clus[0];
maxclus = clus[1];

c_getnum (&block, "NLEVBIN", &three, &par_type, range); 
init_grid = range[0];
min_grid = range[1];
max_grid = range[2];
if (min_grid > max_grid) 
   c_errmsg ("Minimum gray level value exceeds maximum gray level value", 
             "hindu-grange", &fatal_err); 

/*
Open the print file.
---------------------
*/
strcpy (fileutil_file, prog_name);
if (p_find( &block, "PRINT")->v_count != 0)
  {
  stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
             print_file);
  if (stat != E_SUCC)
     c_errmsg("Fatal error encountered","hindu-fatal",&fatal_err);
  }
if (print[LP] || print[FIL] || print[TERM])
   {
   fd1 = fopen (fileutil_file, "w");
   if (print[FIL])
      strcpy (out_files[num_outfiles++], print_file);
   }
else
   fd1 = NULL;

/* 
Get the LAS global parameters: MSGTIM and MINMAX.
--------------------------------------------------
*/
c_getnum (&block, "$MSGTIM", &one, &par_type, &msgtim);
var = p_find (&block, "$MINMAX");
strcpy (mnmx, SVAL(*var, 0));
if (mnmx[0] == 'Y')
   min_max = TRUE;
else
   min_max = FALSE;

}
