#include "hindu.h"
#include "protos.h"

void lowest
(
    long hnum[], 		/* number of pixels in each histogram bin     */
    long ktab[],			/* location of sorted hnum values             */
    long cell			/* current histogram cell                     */
)

/*
*****************************************************************************

NAME: LOWEST

PURPOSE:  Finds the lowest density value in hnum and interchanges it with 
          the current value at location cell. A corresponding change is 
          also made in the table ktab.


PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         19-May-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   For every occupied bin up and including the current bin, 
      Find the bin with the lowest density.
   Interchange the lowest density and the current density values and their
   locations.

*****************************************************************************
*/

{

long	curdens,		/* density of the current bin                 */
	densloc,		/* location of the current bin density value  */
				/* in hnum array                              */
	hmin,			/* lowest density so far                      */
	j,			/* loop counter and array index               */
	minloc;			/* location of the miminum density            */

/*
Initialize variables.
---------------------
*/
curdens = hnum[cell];
densloc = ktab[cell];
hmin = hnum[0];
minloc = 0;

/*
For every occupied bin up and including the current bin, find the bin with
the lowest density.
---------------------------------------------------------------------------
*/
for (j = 1; j <= cell; j++)
   {
   if (hnum[j] < hmin)
      {
      hmin = hnum[j];
      minloc = j;
      }  
   }  

/*
Interchange the lowest density and the current density values and their
locations.
-----------------------------------------------------------------------
*/
ktab[cell] = ktab[minloc];
hnum[cell] = hmin;
ktab[minloc] = densloc;
hnum[minloc] = curdens;

}
