#include "hindu.h"
#include "protos.h"

void merge
(
    float *hist[MAXBND],		/* multi-dimensional histogram                */
    double histloc[6001],		/* location of each occupied histogram bin    */
    long hnum[6001], 		/* number of pixels in each histogram bin     */
    long nbands,			/* number of input image bands                */
    long ncells,			/* number of occupied histogram bins          */
    long ktab[6001] 		/* original location of hnum values before    */
)
/*
*****************************************************************************

NAME: MERGE

PURPOSE:  Finds histogram bins with higher population neighbours and 
          reassigns bin contents accordingly.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         12-Jan-84   K. McGee		SNC#20-145
   3.0         19-MaY-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize the original cell locations in ktab.
   For all occupied bins, 
      Call LOWEST to find the lowest density bin and interchange it with 
      the current bin (cell).
      Find if this bin has any neighbours of higher density.
      Record the number of neighbours and their locations.
      If any neighbours were found then reassign bin contents accordingly, 
      recalculate histogram values and density.
   End-for

*****************************************************************************
*/

{

double	adrs;			/* address of a bin in histogram space        */

float	sum,			/* recalculated density of a bin              */
	total,			/* total number of pixels in high density     */
				/* neighbours                                 */
	xloc,			/* intermed. result of address recalculation  */
	xval;			/* intermed. result of density recalculation  */

long	band,			/* current band index                         */
	cell,			/* current cell                               */
	i, j,			/* loop counters and array indices            */
	loc,			/* location of the current bin                */
	loc2,			/* location of succesive bins compared with   */
				/* the current bin                            */
	nebdens,		/* location of neighbor density value in hnum */
	nebloc, 		/* location of high density neighbor          */
	nebnum[6001],		/* location of neighbors of histogram bins    */
	nflag,			/* set to true if the bin has neighbors       */
	ngrid,			/* number of histogram bins per band          */
	numn;			/* number of neighbors                        */

/*
Initialize the original cell locations in ktab.
------------------------------------------------
*/
ngrid = (long) ((256.0 / (float)init_grid) + 0.5);
for (cell = 0; cell < ncells; cell++)
   ktab[cell] = cell;

/*
For all occupied bins, call LOWEST to find the lowest density bin and 
interchange it with the current bin (cell).
------------------------------------------------------------------------
*/
for (cell = ncells-1; cell >= 1; cell--)
   {
   lowest (hnum, ktab, cell);
   loc = ktab[cell];
   numn = 0;
   total = 0.0;
/*
Find if this bin has any neighbours of higher density.
---------------------------------------------------------
*/
   for (j = 0; j <= cell-1; j++)
      {
      nflag = FALSE;
      loc2 = ktab[j];
      nebors (histloc, loc, loc2, &nflag, nbands, ngrid);
/*
Record the number of neighbours and their locations.
----------------------------------------------------------
*/
      if (nflag)
         {
         nebnum[numn] = j;
         numn++;
         total = total + (float) hnum[j];
         }  
      }  /* do */

/*
If any neighbours were found then re-assign bin contents accordingly, 
recalculate histogram values and density.
-----------------------------------------------------------------------
*/
   if (numn != 0)
      {
      for (i = 0; i < numn; i++)
         {
         nebloc = nebnum[i];
         nebdens = ktab[nebloc];
         xval = (float)hnum[cell] * (float)hnum[nebloc]/total;
         sum = (float) hnum[nebloc] + xval;
         for (band = 0; band < nbands; band++)
            hist[band][nebdens] = ((hist[band][nebdens] * (float)hnum[nebloc])+ 
                             (hist[band][loc] * xval)) / sum;
         hnum[nebloc] = (long) sum;
         adrs = (hist[0][nebdens] + 0.5) / (float)init_grid + 1;
         for (band = 1; band < nbands; band++)
            {
            xloc = (hist[band][nebdens] + 0.5) / (float)init_grid + 1;
            adrs = (adrs * ngrid) + xloc;
            } 
         histloc[nebdens] = adrs;
         }  /* do */

      hnum[cell] = -hnum[cell];
      }  /* if */
   }  /* do */

}
