#include "hindu.h"
#include "protos.h"

void multhist
(
    float *hist[MAXBND],		/* multi-dimensional histogram gray levels    */
    double histloc[6001],		/* location of each occupied histogram bin in */
    long hnum[6001], 		/* number of pixels in each histogram bin     */
    long *ncells,		/* number of occupied histogram bins          */
    long tot_inbands,		/* total number of input image bands          */
    long npix,			/* number of pixels per line                  */
    long nlin,			/* number of lines in the input image         */
    long repeat, 		/* flag to abort when recalculating histogram */
    long *iterate		/* flag set to true if space allocated for    */
)

/*
*****************************************************************************

NAME: MULTHIST

PURPOSE:  Creates a multidimensional histogram from the input file and
          writes preliminary class assignements to a temporary image file.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         12-Jan-86   K. McGee		SCN#20-145
   3.0         29-Mar-89   K. Kunska/STX	conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables and set tab values according to the number of 
   grey levels per histogram bin.
   Initialize and enable the processing message utility.
   For each line and each pixel
      Find the absolute location of a bin in the histogram space.
      Determine if this location is already occupied by checking the last 
      location found.
      If the bin is occupied, increment density, and write the bin number to the
      temporary output file.
      If the location is new, check if the histogram space is exceeded.
      If out of space, print the apropriate message and exit to the main 
      program so that the histogram can be recalculated.
      If not out of space, set up new location, store the histogram values, 
      density, and address.
      Save the bin number in the temporary output file.
   End-for
   Find the mean grey level in each bin.

*****************************************************************************
*/

{

char    *prog_name = "HINDU"; /* program name                 */
long    one = 1;


char	bfill,			/* blank fill value                           */
	text[80];		/* text buffer for error messages             */

double	adrs;			/* address of a bin in the histogram space    */

long	band,			/* band loop counter and array index          */
	cell,			/* current histogram cell                     */
	ftype,			/* data type of the fill value                */
	i, 			/* loop counter and array index               */
	itab[257], 		/* tab values to assign the gray level to a   */
				/* histogram bin                              */
	last,			/* last histogram bin found                   */
	line,			/* current line                               */
	msg_flag = 3,		/* flag for the processing message utility    */
	ngrid,			/* number of histogram bins per band          */
	pixel,			/* current pixel                              */
	search = TRUE,		/* set to true if current vector of gray      */
                                /* levels has not previously been encountered */
	text_size;		/* size of the text buffer                    */

long inf_err = 1;
long zero = 0;       /*  Constant of zero      */

/*
Initialize variables and set tab values according to the number of 
grey levels per histogram bin.
-------------------------------------------------------------------------
*/
bfill = ' ';
text_size = 80;
ftype = EBYTE;
*ncells = 0;
last = 0;
adrs = 0.0;
ngrid = (long) ((256.0 / (float)init_grid) + 0.5);

for (i = 0; i < 257; i++)
   itab[i] = (long) ((((float)i + 0.5) / (float)init_grid) + 1);

for (i = 0; i < 6001; i++)
   {
   hnum[i] = 0;
   histloc[i] = 0;
   }
   
/*
Initialize and enable the processing message utility.
-----------------------------------------------------
*/
c_inittm ("HINDU", "Processed", "line", " ", " ", &nlin, &zero, &msgtim,
           &msg_flag);
curent = 1;
c_startm();

/*
Read each line & loop over each pixel.
----------------------------------------
*/
for (line = 1; line <= nlin; line++)
   {
   curent++;
   c_estep (&gdescin);
   c_estep (&gdesctmp);
   for (pixel = 0; pixel < npix; pixel++)
      {
/*
Find the absolute location of a bin in the histogram space.
-------------------------------------------------------------
*/
      adrs = itab[inbuf[ipoint[0]+pixel]];
      for (band = 1; band < tot_inbands; band++)
         adrs = adrs * ngrid + itab[inbuf[ipoint[band]+pixel]];

/*
Determine if this location is already occupied by checking the last 
location found.
-------------------------------------------------------------------
*/
      cell = 0;
      search = TRUE;
      if (*ncells >= 1)
         {
         if (adrs == histloc[last])
            {
            hnum[last] = hnum[last] + 1;
            tempbuf[tpoint[0]+pixel] = (unsigned char)last;
            for (band = 0; band < tot_inbands; band++)
               hist[band][last] = hist[band][last] + 
                                  inbuf[ipoint[band]+pixel];
            search = FALSE;
            }  
         while (search && (cell <= *ncells))
            {
/*
If the bin is occupied, increment density, and write the bin number to the
temporary output file.
------------------------------------------------------------------------
*/
            if (adrs == histloc[cell])
               {
               hnum[cell] = hnum[cell] + 1;
               tempbuf[tpoint[0]+pixel] = (unsigned char)cell;
               for (band = 0; band < tot_inbands; band++)
                  hist[band][cell] = hist[band][cell] + 
                                     inbuf[ipoint[band]+pixel];
               last = cell;
               search = FALSE;
               }  /* if */
            cell++;
            }  /* do */
         }  /* if */
/*
If the location is new, check if the histogram space is exceeded.
If out of space, print the apropriate message and exit to the main 
program so that the histogram can be recalculated.
---------------------------------------------------------------------
*/
      if (search)
         {
         if (*ncells > 6000)
            {
            sprintf (text, "Need more histogram space by line %4i pixel %4i", 
                     line, pixel);
            c_errmsg (text, "hindu-hspace", &inf_err); 
            c_pxfill (text, &ftype, &text_size, &bfill);
            if ((init_grid == max_grid) || repeat)
               {
               sprintf (text, "Restart and choose a wider range of values ");
               sprintf (&text[strlen(text)], "for GRANGE");
               c_errmsg (text, "hindu-mhist", &inf_err); 
               c_pxfill (text, &ftype, &text_size, &bfill);
               c_clean (&block, prog_name, tmp_files, &one);
               }
            else
               {
               init_grid = max_grid;
               sprintf (text, "Recalculating histogram with maximum value ");
               sprintf (&text[strlen(text)], "for GRANGE");
               c_errmsg (text, "hindu-mhist", &inf_err);
               c_pxfill (text, &ftype, &text_size, &bfill);
               *iterate = TRUE;
               return;
               }  /* if */
            }  /* if */
/*
If not out of space, set up new location, store the histogram values, 
density, and address.
Save the bin number in the temporary output file.
----------------------------------------------------------------------
*/
         for (band = 0; band < tot_inbands; band++)
            hist[band][*ncells] = inbuf[ipoint[band]+pixel] + 
                                  hist[band][*ncells];
         hnum[*ncells] = 1;
         histloc[*ncells] = adrs;
         tempbuf[tpoint[0]+pixel] = (unsigned char)(*ncells);
         last = *ncells;
         *ncells = *ncells + 1;
         }  /* if */
      }  /* do */
   }  /* do */

c_estep (&gdescin);
c_estep (&gdesctmp);
c_stoptm();

/*
Find the mean grey level in each bin.
------------------------------------
*/
for (band = 0; band < tot_inbands; band++)
   for (i = 0; i < *ncells; i++)
      hist[band][i] = hist[band][i] / (float)(hnum[i]);

}
