#include "hindu.h"
#include "protos.h"

void nebors
(
    double histloc[],		/* location of each occupied histogram bin in */
    long loc,			/* location of the first bin                  */
    long loc2,			/* location of the second bin                 */
    long *flag,			/* set to true if bins are adjacent           */
    long nbands,			/* number of input image bands                */
    long ngrid			/* number of histogram bins per band          */
)
/*
*****************************************************************************

NAME: NEBORS

PURPOSE:  Finds if two histogram bins are adjacent.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         19-May-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   For each band find the distance between the locations of the two bins.
   Set flag to true if the distance between the two bins is 1 (bins are
   adjacent).

*****************************************************************************
*/
	
{

double	bin1,			/* address of bin #1 in histogram space       */
	bin2,			/* address of bin #2 in histogram space       */
	distance,		/* distance between the two bins              */
	offset1, 		/* band offset of bin #1                      */
	offset2;		/* band offset of bin #2                      */

long	band;			/* current band index                         */

/* 
Initialize variables.
---------------------
*/
*flag = TRUE;
distance = 0.0;
bin1 = (long) histloc[loc];
bin2 = (long) histloc[loc2];

/*
For each band find the distance between the locations of the two bins.
Set flag to true if the distance between the two bins is 1 (bins are
adjacent).
------------------------------------------------------------
*/
for (band = 0; band < nbands; band++)
   {
   if (bin1 == bin2)
      return;
   offset1 = bin1 / ngrid;
   offset2 = bin2 / ngrid;
   offset1 = (long)offset1;
   offset2 = (long)offset2;
   distance = distance + (double)abs(bin1 - bin2 - ngrid * (offset1 - offset2));
   if (distance > 1.0)
      {
      *flag = FALSE;
      return;
      }
   else
      {
      bin1 = offset1;
      bin2 = offset2;
      }  
   }  

}
