#include "hindu.h"
#include "protos.h"

void outspec
(
    char hostin[][CMLEN], 		/* host name for input image          */
    char hostout[], 			/* host name for output image         */
    long num_inimages,			/* total number of input images       */
    long num_img_bands[][MAXBND+1],	/* band numbers for each input image  */
    long out_img_bands[][MAXBND+1],	/* band numbers for each output image */
    long out_bands,			/* number of output image bands       */
    long bands_per_img[]		/* number of bands for each image     */
)

/*
*****************************************************************************

NAME: OUTSPEC

PURPOSE:  Opens the output image file.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         08-Jul-86   N. Deodhar/SAR #212
   3.0         19-May-89   K. Kunska/STX	conversion to 5.0
   6.1         24-May-94   D. Eichack/EDC	Added opt[]

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open the output image file.
   Allocate buffer space for the output image file.
   Group the output image files.
   Update the ddr records.

*****************************************************************************
*/

{

long    one = 1;
char    *prog_name = "HINDU"; /* program name                 */


static 	long	opt[3]={0,0,0};		/* option array for eopens	      */
long	access,				/* access type                        */
	bufsize,			/* buffer size                        */
	comb_flag,			/* combine flag for 1-band output img */
	n,				/* loop counter and array index       */
	stat,				/* return status                      */
	type;				/* data type                          */
double	line_inc,			/* line increment                     */
	samp_inc;			/* sample increment                   */

long zero = 0;         /* Constant of zero */

/*
Open the output image file.
-----------------------------
*/
n = 0;
access = IWRITE;
type = EBYTE;
stat = c_eopens (&fdescout[n], hostout, &out_bands, &one, &one, &window[n][NL], 
                 &window[n][NS], opoint, &access, &type, &zero, opt);
if (stat != E_SUCC)
   c_abort (&block, "Error opening the output image", "hindu-outopen",
            prog_name, out_files, &num_outfiles);
fdescout[++n] = 0;

out_img_bands[0][0] = 1;

/*
Allocate buffer space for the output image file.
-------------------------------------------------
*/
if ((outbuf = (unsigned char *) c_ealloc (fdescout, &bufsize)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "hindu-alloc",
            prog_name, out_files, &num_outfiles);

/* 
Group the output image files.
--------------------------------
*/
c_egroup (fdescout, &gdescout, outbuf, &bufsize);

/*
Update the ddr records.
------------------------
*/
line_inc = 1;
samp_inc = 1;
comb_flag = COMB;

c_upddr (&block, hostin, hostout, &num_inimages, window, num_img_bands,
         bands_per_img, &line_inc, &samp_inc, &out_bands, &comb_flag);
}
