#ifndef PROTOS_H
#define PROTOS_H


void classify
(
    char taeout[],		/* output image file tae name                 */
    long npix,			/* number of pixels per line                  */
    long nlin,			/* number of lines in the input image         */
    long ncells,			/* number of occupied histogram bins          */
    long signum,			/* number of significant clusters             */
    float *hist[MAXBND],		/* multi-dimensional histogram values         */
    float *xcentr[MAXBND],	/* cluster centroids                          */
    long nbands,			/* number of bands                            */
    float scale[6001],		/* cluster significance values                */
    long print[]                /* print options                              */
);

void cluster
(
    long npix,			/* number of pixels per line                  */
    long nlin,			/* number of lines in the input image         */
    long ncells,			/* number of occupied histogram bins          */
    long ktab[6001], 		/* locations of sorted hnum values            */
    long hnum[6001], 		/* number of pixels per histogram bin         */
    float *hist[MAXBND], 		/* multi-dimensional histogram gray levels   */
    long *repeat,		/* indicates if the number of clusters is     */
    long nbands,			/* number of bands in the input image         */
    long *signum,		/* number of significant clusters             */
    float *xcentr[MAXBND],	/* cluster centroids                         */
    float scale[6001]		/* array of significance values              */
);

void getpar
(
    char taein[][CMLEN], 	/* input image tae name                       */
    char hostin[][CMLEN], 	/* input image host name                      */
    char taeout[][CMLEN], 		/* output image tae name                      */
    char hostout[][CMLEN], 		/* output image host name                     */
    char hostemp[], 		/* temporary image host name                  */
    long *num_inimg,		/* number of input images                     */
    long (*num_img_bands)[MAXBND+1],	/* band numbers for each input image  */
    long *bands_per_img,		/* number of bands in each input image        */
    long *tot_inband,		/* total number of bands                      */
    long img_limit,		/* maximum image limit                        */
    long *same_wnd,		/* same window size: 1=YES, 0=NO              */
    long (*img_flag)[2],		/* flag for the window/band option            */
    long print[],
    char print_file[]		/* print file name                            */
);

void inspec
(
    char hostin[][CMLEN], 		/* host name for input image          */
    long num_inimages,			/* total number of input images       */
    long num_img_bands[][MAXBND+1],	/* band numbers for each input image  */
    long bands_per_img[]		/* number of bands for each image     */
);

void lowest
(
    long hnum[], 		/* number of pixels in each histogram bin     */
    long ktab[],			/* location of sorted hnum values             */
    long cell			/* current histogram cell                     */
);

void merge
(
    float *hist[MAXBND],		/* multi-dimensional histogram                */
    double histloc[6001],		/* location of each occupied histogram bin    */
    long hnum[6001], 		/* number of pixels in each histogram bin     */
    long nbands,			/* number of input image bands                */
    long ncells,			/* number of occupied histogram bins          */
    long ktab[6001] 		/* original location of hnum values before    */
);

void multhist
(
    float *hist[MAXBND],		/* multi-dimensional histogram gray levels    */
    double histloc[6001],		/* location of each occupied histogram bin in */
    long hnum[6001], 		/* number of pixels in each histogram bin     */
    long *ncells,		/* number of occupied histogram bins          */
    long tot_inbands,		/* total number of input image bands          */
    long npix,			/* number of pixels per line                  */
    long nlin,			/* number of lines in the input image         */
    long repeat, 		/* flag to abort when recalculating histogram */
    long *iterate		/* flag set to true if space allocated for    */
);

void nebors
(
    double histloc[],		/* location of each occupied histogram bin in */
    long loc,			/* location of the first bin                  */
    long loc2,			/* location of the second bin                 */
    long *flag,			/* set to true if bins are adjacent           */
    long nbands,			/* number of input image bands                */
    long ngrid			/* number of histogram bins per band          */
);

void outspec
(
    char hostin[][CMLEN], 		/* host name for input image          */
    char hostout[], 			/* host name for output image         */
    long num_inimages,			/* total number of input images       */
    long num_img_bands[][MAXBND+1],	/* band numbers for each input image  */
    long out_img_bands[][MAXBND+1],	/* band numbers for each output image */
    long out_bands,			/* number of output image bands       */
    long bands_per_img[]		/* number of bands for each image     */
);

void sort
(
    float scale[],		/* cluster significance values                */
    long cnums[],		/* bin number of each significant cluster     */
    long signum			/* number of significant clusters             */
);

void tempfile
(
    char hostemp[],		/* host name for the temporary image          */
    long *temp_bands,		/* number of temporary image bands            */
    char hostin[][CMLEN],	/* host name for the input image              */
    long num_inimg,		/* number of input images                     */
    long num_img_bands[MAXIMG][MAXBND+1],	/* band numbers for each input image  */
    long bands_per_img[]	/* number of bands in each input image        */
);

void tempspec
(
    char hostemp[],			/* host name for temporary image      */
    long tbands				/* bands array for opening temp. img. */
);

#endif
