#include "hindu.h"
#include "protos.h"

void tempfile
(
    char hostemp[],		/* host name for the temporary image          */
    long *temp_bands,		/* number of temporary image bands            */
    char hostin[][CMLEN],	/* host name for the input image              */
    long num_inimg,		/* number of input images                     */
    long num_img_bands[MAXIMG][MAXBND+1],	/* band numbers for each input image  */
    long bands_per_img[]	/* number of bands in each input image        */
)

/*
*****************************************************************************

NAME: TEMPFILE

PURPOSE:  Creates and initializes a temporary image file.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         17-Feb-83   C. Gurney/SASC      
   2.0         08-Jul-86   N. Deodhar/SAR #212
   3.0         19-May-89   K. Kunska/STX	conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open temporary image file for writing.
   Allocate bufer space for the temporary image file.
   Create a file action group for the temporary image file.
   Fill the tmporary file with zeros.
   Update the ddr file.
   Close the temporary image file.

*****************************************************************************
*/

{

char    *prog_name = "HINDU"; /* program name                 */
long    one = 1;


long	access,			/* type of access file is to be opened for    */
	bufsize,		/* buffer size for the temporary image        */
	comb_flag,		/* flags if the input bands will be combined  */
				/* into one output band                       */
	i, n,			/* loop counter and array index               */
	stat,			/* return status                              */
	type,			/* file data type                             */
	zero_fill;		/* fill value                                 */
double	line_inc,		/* line increment                             */
	samp_inc;		/* sample increment                           */
long    opt[3];

long zero = 0;      /* Constant of zero                          */
/*
Open temporary image file for writing.
---------------------------------------
*/
n = 0;
access = IWRITE;
type = EBYTE;
opt[0] = opt[2] = 0;
opt[1] = NO_COMPRES;
stat = c_eopens (&fdesctmp[n], hostemp, temp_bands, &one, &one, 
                 &window[0][NL], &window[0][NS], tpoint, &access, &type, 
                 &zero, opt);
if (stat != E_SUCC)	
   c_abort (&block, "Error opening temporary image", "hindu-tmpopen",
            prog_name, tmp_files, &num_tmpfiles);
fdesctmp[++n] = 0;

/*
Allocate bufer space for the temporary image file.
---------------------------------------------------
*/
if ((tempbuf = (unsigned char *) c_ealloc (fdesctmp, &bufsize)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "hindu-tmpalloc",
            prog_name, tmp_files, &num_tmpfiles);

/*
Create a file action group for the temporary image file.
--------------------------------------------------------
*/
c_egroup (fdesctmp, &gdesctmp, tempbuf, &bufsize);

/*
Fill the tmporary file with zeros.
-----------------------------------
*/
zero_fill = 0;
for (i = 0; i < window[0][NL]; i++)
   {
   c_estep (&gdesctmp);
   c_pxfill (&tempbuf[tpoint[zero]], &type, &window[0][NS], &zero_fill);
   }  

c_estep (&gdesctmp);

/*
Update the ddr file.
--------------------
*/
line_inc = 1;
samp_inc = 1;
comb_flag = COMB;
c_upddr (&block, hostin, hostemp, &num_inimg, window, num_img_bands, 
         bands_per_img, &line_inc, &samp_inc, temp_bands, &comb_flag);

/*
Close the temporary image file.
--------------------------------
*/
c_egclse(&gdesctmp);

}
