/****************************************************************************

NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the HIS function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
J.ROBAR         June 1984       converted from VMS to LAS
C.GREENHAGEN    Mar  1986       updated for standards
R.MCKINNEY      May  1988       converted to LAS 5.0
R.MCKINNEY      Dec  1988       updated to close all files
T.ROCKVAM	Sept 1991	check data type of all input image files

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

******************************************************************************/


#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	
                                  /* image file descriptors used for image I/O*/
extern struct GDESC * gdesc;	  /* group descriptor used for image I/O      */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands */
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands   */
extern long curent;		/* current processing line		    */
extern long curcat;		/* current processing band		    */

void getpar
(
    struct PARBLK *vblock,      /* pointer to TAE block	 	      */
    char hosin[][CMLEN],  	/* host file names of input images	    */
    char taeout[][CMLEN],		/* TAE file name of output image	    */
    char hosout[][CMLEN],		/* host file name of output image	    */
    long *nmrimg, 		 /* number of input images		    */
    long (*bands)[MAXBND+1],         /* band specification 		            */
    long *nbands,		 /* number of bands per input image         */
    long (*window)[4],	         /* window specification		    */
    long *msgtim,		 /* elapsed time between display messages   */
    char *minmax,		 /* min max flag buffer		            */
    long *totbnd,		 /* total number of bands		    */
    char *subnam		 /* subcommand specified		    */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block  */

char     taein[MAXIMG][CMLEN];	/* TAE file names of input images	    */

long     dtype[MAXIMG];		 /* data type of previous input image       */
long     element = 0;		 /* parameter element number		    */
long     fatal = -1;		 /* fatal error flag			    */
long     imglim;		 /* max number of images to be processed    */
long     nonfatal = 1;		 /* value of one			    */
long     qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified */
long     same;		    	 /* flag indicating if images have same wind*/
long     status;	   	 /* function return status code		    */
 
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

/* get the subcommand 
---------------------*/
val = p_find(vblock,"_subcmd");
strcpy(subnam,SVAL(*val,0));

/* Build the string descriptor for TAE variable "IN".
   Call getin to parse the input image specification.
   Getin returns the TAE image name(s), host image name(s),
   bands to be processed, window specification, and number
   of images parsed.
----------------------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","his-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","his-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","his-fatal",&fatal);
   }

/* Check for the correct number of bands
----------------------------------------*/

if (*totbnd != 3)
   {
   c_errmsg("IN must contain exactly three bands","his-bands",
		&nonfatal);
   c_errmsg("Fatal error encountered","his-fatal",&fatal);
   }

/* Get the input data type; make sure all input images
   have same data type
------------------------------------------------------*/
status = c_gettyp(hosin, nmrimg, dtype);
if (status == E_FAIL)
   c_errmsg("Fatal error encountered","his-fatal",&fatal);

/* Insure the datatype is BYTE for RGB2HIS
------------------------------------------*/

if (strcmp(subnam,"RGB2HIS") == 0)
   {
   if (dtype[0] != EBYTE)
      c_errmsg("Invalid input data type--byte only",
		"his-fatal",&fatal);
   }
else

/* Insure the datatype is INTEGER*2 for HIS2RGB
-----------------------------------------------*/

if (strcmp(subnam,"HIS2RGB") == 0)
   {
   if (dtype[0] != EWORD)
      c_errmsg("Invalid input data type--I*2 only",
		"his-fatal",&fatal);
   }
/* Retrieve the TAE output image name from the parameter
   block and store it in "taeout". 
   Create a host file name for the output image and store 
   it in "hosout".
----------------------------------------------------------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","his-fatal",&fatal);

return;
}
