/***************************************************************************

NAME      		  HIS 

PURPOSE Generate an HIS image from an RGB image or viseversa.   

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
J.ROBAR         July 1984       Original development
C.GREENHAGEN    Mar  1986       update for standards 
R.MCKINNEY      July 1988       Conversion to LAS 5.0
R.MCKINNEY      Dec  1988	Updated to close all files
R.MCKINNEY      Dec  1988	Intensity and Saturation are multiplied
				times 255 rather than 100.
B.AILTS		May  1988       Fixed problem when rgb are all equal
T.ROCKVAM	Sept 1991	Added check of data types of all input images;
				removed unused variables
T.Mittan	Sept 1992	Added extern to curent and curcat to eliminate
				warnings on compile
D.Bridges       Feb  1994       Changed floats to doubles so it gives the same
				result on different systems

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   HIS must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM REFERENCES
     J. D. Foley and A. VanDam, Fundamentals of Interactive 
     Computer Graphics, Addison-Wesley Publishing Company, 
     Reading, Massachusetts, 1982, Chap. 17, Section 17.4.5,
     "The HLS Color Model," p. 617.
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC * fdesc[MAXIMG+1];  /* image file descriptors used for image I/O */
struct GDESC * gdesc;		     /* group descriptor used for image I/O       */
long offset1[MAXIMG*MAXBND];    /* offsets into "thebuf" for input bands     */
long offset2[MAXBND];           /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

int main
(void)

{
struct PARBLK	vblock;         /* TAE parameter block    		     */

static long bands[MAXIMG][MAXBND + 1];
                                /* band specification for first image        */
long msgtim;			/* elpased time between display messages     */
long nmrimg;			/* number of input images for image          */
long nonfatal = 1;		/* nonfatal error flag			     */
long nbands[MAXIMG];		/* number of bands per input image           */
long one = 1;			/* number one				     */
long std_out;			/* logical unit # for standard output        */
long totbnd;			/* total number of bands in first image      */
long window[MAXIMG][4];		/* window specification for first image      */

char    *c_ealloc();		/* ealloc function declaration		     */
char	hosin[MAXIMG][CMLEN];	/* buffer storage        	   	     */
char    hosout[1][CMLEN];          /* buffer storage        		     */
char    minmax[4];	        /* minmax flag buffer 			     */
char    subnam[8];		/* subcommand name 			     */
char    taeout[1][CMLEN];          /* buffer storage       		     */


c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/*
       get user-specified parameters and TAE globals
*/
getpar(&vblock, hosin, taeout, hosout,
       &nmrimg, bands, nbands, window, &msgtim, 
       minmax, &totbnd,subnam);
/*
   perform the HIS translation
*/
if (strcmp(subnam,"RGB2HIS") == 0)
{
rgb2his(&vblock,hosin,hosout,nmrimg,bands,nbands,
	window,msgtim,minmax,totbnd); 
}
else
if (strcmp(subnam,"HIS2RGB") == 0)
{
his2rgb(&vblock,hosin,hosout,nmrimg,bands,nbands,
	window,msgtim,minmax,totbnd); 
}
/*
   create associated history file
*/
if (c_hstout(&vblock, hosout[0], hosin , &nmrimg, " ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);

/*
   inform user of successful completion
*/
c_complt("HIS", taeout, &one);

return EXIT_SUCCESS;

}
