/******************************************************************************
NAME:   		      HIS2RGB 

PURPOSE    To proces hue,intensity,saturation bands to red,green,blue. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0      J.ROBAR	   6/84  	  EDC		Conversion from 
							VMS to LAS

  4.0	   C.GREENHAGEN    3/86		  EDC		Updated for standards 

  5.0	   R. McKinney     6/88		  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.0	   R. McKinney    12/88		 EDC		Updated to close all
							files
  5.0	   R. McKinney    12/88		 EDC		Intensity and Saturation
							are times 255 rather
							than 100.
  5.1	   T. Rockvam	  09/91		 EDC		Fixed offset declaration
							to match that in his.c
							Removed unused
							parameters
  5.3      D. Bridges     02/94          EDC            Changed floats to 
							doubles so it gives the
							same result on different
							systems
  6.1	   E. Eichacker   05/94          EDC            added opt[]


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   HIS2RGB must be run under TAE.

PROJECT			LAS

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];
                                 /* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	 /* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands*/
extern long offset2[MAXBND];  /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


void his2rgb
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long (*window)[4],		/* window specification of input images      */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long totbnd			/* total number of bands to be processed     */
)

{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

static	long	opt[3]={0,0,0};	/* optional parameter array for EOPENS	     */
long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    odtype = 0;		/* data type				     */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    sample;			/* sample indicator			     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */

short   *thebuf;	        /* image input buffer                        */

double  red;			/* the red band output			     */
double  red255;			/* the red band output			     */
double  green;			/* the green band output		     */
double  green255;		/* the green band output		     */
double  blue;			/* the blue band output			     */
double  blue255;		/* the blue band output			     */
double  m1;			/* value used for determining RGB            */
double  m2;			/* value used for determining RGB            */
double  scalei;			/* intensity value			     */
double  scales;			/* saturation value                          */
double  hue;			/* hue					     */
double  savehue;		/* save the hue for future processing        */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   odtype = 2;
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      {
      c_errmsg("Unable to open input image","his-open",&nonfatal);
      c_errmsg("Fatal error encountered","his-fatal",&fatal);
      }
   band += nbands[image];

   }

   odtype = 2;

/* Open output image
--------------------*/
access = IWRITE;	
opt[0]=1;
status = c_eopens(&fdesc[nmrimg],hosout[0],&totbnd,&one,&one,&nl,&ns,
         offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
   {
   c_errmsg("Unable to open output image","his-open",&nonfatal);
   c_clean(vblock,"his",hosout,&one);
   }
fdesc[nmrimg+1] = 0;

/* Allocate I/O buffer
----------------------*/
if ((thebuf = (short *) c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","his-alloc",&nonfatal);
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);
   c_clean(vblock,"his",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);
   c_clean(vblock,"his",hosout,&one);
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("HIS","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Main processing loop
-----------------------*/
for (curlin = 0;curlin < nl; curlin++)
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
      c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);
      c_clean(vblock,"his",hosout,&one);
      }

   for (sample = 0; sample < ns; sample++)
        {
	red = green = blue = 0.0;
   	scalei = *(thebuf + (*(offset1+1) + sample));
	scalei /= 255.;
   	scales = *(thebuf + (*(offset1+2) + sample));
	scales /= 255.;
	m2 = 0.0;
	if (scalei <= 0.50) 
		m2 = scalei*(1.0 + scales);
	else if (scalei > 0.50)
	        m2 = scalei+scales-(scalei*scales);
	m1 = 2.0 * scalei - m2;

	hue = *(thebuf + (*(offset1) + sample));

	if (scales == 0.0)
		{
		if (hue == -1.0)
		   {
		   red = scalei;
		   green = scalei;
		   blue = scalei;
		   }
		}
	else
	{
/*
  calculate the red band
*/
	savehue = hue + 120.0;
	if (savehue > 360.0)
	   savehue -= 360.0;
	if (savehue < 0.0)
	   savehue += 360.0;
	if (savehue < 60.0)
	   red = m1 + (m2-m1) * savehue/60.0;
	else
	if (savehue < 180.0)
	   red = m2;
	else
	if (savehue < 240.0)
	   red = m1 + (m2-m1) * (240.0-savehue)/60.0;
	else
	   red = m1;
/*
  calculate the green band
*/
	savehue = hue;
	if (savehue > 360.0)
	   savehue -= 360.0;
	if (savehue < 0.0)
	   savehue += 360.0;
	if (savehue < 60.0)
	   green = m1 + (m2-m1) * savehue/60.0;
	else
	if (savehue < 180.0)
	   green = m2;
	else
	if (savehue < 240.0)
	   green = m1 + (m2-m1) * (240.0-savehue)/60.0;
	else
	   green = m1;
/*
  calculate the blue band
*/
	savehue = hue - 120.0;
	if (savehue > 360.0)
	   savehue -= 360.0;
	if (savehue < 0.0)
	   savehue += 360.0;
	if (savehue < 60.0)
	   blue = m1 + (m2-m1) * savehue/60.0;
	else
	if (savehue < 180.0)
	   blue = m2;
	else
	if (savehue < 240.0)
	   blue = m1 + (m2-m1) * (240.0-savehue)/60.0;
	else
	   blue = m1;
	}
	red255 = red*255.0;
	green255=green*255.0;
	blue255 = blue*255.0;
	if (red255 > 255.0)
	    red = 255.0;
	    else
	    red = red255;
	if (green255 > 255.0)
	    green = 255.0;
	    else
	    green = green255;
	if (blue255 > 255.0)
	    blue = 255.0;
	    else
	    blue = blue255;

	if (red   > 254.5) red   = 254.5;
	if (red   <   0.0) red   =   0.0;
	if (green > 254.5) green = 254.5;
	if (green <   0.0) green =   0.0;
	if (blue  > 254.5) blue  = 254.5;
	if (blue  <   0.0) blue  =   0.0;
        *(thebuf + (*(offset2) + sample))   = red + 0.5; 
	*(thebuf + (*(offset2+1) + sample)) = green + 0.5;
	*(thebuf + (*(offset2+2) + sample)) = blue + 0.5;
      }
	curent ++;
      if (strcmp(minmax,"YES") == 0)
	 {
         status = c_pxmnmx(thebuf + *(offset2),&ns,
                         &minval[0],&maxval[0],&odtype); 
         status = c_pxmnmx(thebuf + *(offset2+1),&ns,
                         &minval[1],&maxval[1],&odtype); 
         status = c_pxmnmx(thebuf + *(offset2+2),&ns,
                         &minval[2],&maxval[2],&odtype); 
	 }
       }
/* Final write
--------------*/
if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
      c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);
      c_clean(vblock,"his",hosout,&one);
      }
/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"his",hosout,&one);


/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);

/* Free buffer
--------------*/
free(thebuf);

return;
}
