/******************************************************************************
NAME:   		      RGB2HIS 

PURPOSE    To proces red,green,blue bands to hue,intensity,saturation. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0      J.ROBAR	   6/84  	  EDC		Conversion from 
							VMS to LAS

  4.0	   C.GREENHAGEN    3/86		  EDC		Updated for standards 

  5.0	   R. McKinney     6/88		  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.0	   R. McKinney    12/88 	 EDC		Updated to close all
							files
  5.0	   R. McKinney    12/88		 EDC		Intensity and Saturation							are times 255 rather
							than 100.
  5.0      R. McKinney    02/89		 EDC		Fixed a wrap around
							problem found in certain
							cases from his to rgb.
  5.1      B. Ailts       05/89          EDC            Fixed the problem when
							all the rgb values are
							equal
  5.2	   T. Rockvam	  09/91		 EDC		Changed declarations for
							offsets to match his.c
							Removed unused
							parameters & variables
  5.3      D. Bridges     02/94          EDC            Changed floats to 
							doubles so it gives the
							same result on different
 							systems
  6.1      D. Eichacker   02/94          EDC            added opot[]

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   RGB2HIS must be run under TAE.

PROJECT			LAS

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	                                                                         /* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	 /* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands*/
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


void rgb2his
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long (*window)[4],		/* window specification of input images      */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long totbnd			/* total number of bands to be processed     */
)

{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

static	long	opt[3]={0,0,0}; /* optional parameter array for EOPENS	     */
long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    odtype = 0;		/* data type				     */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    sample;			/* sample indicator			     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */

short   *thebuf;	        /* image input buffer                        */

double  red;			/* the red band output			     */
double  green;			/* the green band output		     */
double  blue;			/* the blue band output			     */
double  scaler;			/* red value				     */
double  scaleg;			/* green value				     */
double  scaleb;			/* blue value				     */
double  high;			/* maximum red, green, blue		     */
double  low;			/* minimum red, green, blue                  */
double 	intens;		        /* intensity				     */
double  sat;			/* saturation				     */
double  hue = 0.0;			/* hue					     */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   odtype = 2;
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      {
      c_errmsg("Unable to open input image","his-open",&nonfatal);
      c_errmsg("Fatal error encountered","his-fatal",&fatal);
      }
   band += nbands[image];

   }
   odtype = 2;

/* Open output image
--------------------*/
access = IWRITE;	
status = c_eopens(&fdesc[nmrimg],hosout[0],&totbnd,&one,&one,&nl,&ns,
         offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
   {
   c_errmsg("Unable to open output image","his-open",&nonfatal);
   c_clean(vblock,"his",hosout,&one);
   }
fdesc[nmrimg+1] = 0;

/* Allocate I/O buffer
----------------------*/
if ((thebuf = (short *) c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","his-alloc",&nonfatal);
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
   c_errmsg("Nonfatal errror encountered","his-warn",&nonfatal);
   c_clean(vblock,"his",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
   c_errmsg("Nonfatal errror encountered","his-warn",&nonfatal);
   c_clean(vblock,"his",hosout,&one);
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("HIS","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Main processing loop
-----------------------*/
/*
   processing for RGB2HIS 
*/
for (curlin = 0;curlin < nl; curlin++)
   {
   if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);
   c_clean(vblock,"his",hosout,&one);
   }

   for (sample = 0; sample < ns; sample++)
        {
   	scaler = *(thebuf + (*(offset1+0) + sample));
	scaler /= 255.0;
   	scaleg = *(thebuf + (*(offset1+1) + sample));
	scaleg /= 255.0;
   	scaleb = *(thebuf + (*(offset1+2) + sample));
	scaleb /= 255.0;
	    high = scaler;
	if (scaleg > high)
	    high = scaleg;
	if (scaleb > high)
	    high = scaleb;
	    low = scaler;
	if (scaleg < low)
	    low = scaleg;
	if (scaleb < low)
	    low = scaleb;
/*
  calculate the lightness (intensity)
*/
   intens = ((high + low) / 2.0);
/*
  if min = max the achromatic case R=G=B
*/
   if (high == low)
   	{
	sat = 0.0;
	hue = -1.0;
       *(thebuf + (*(offset2+0) + sample)) = hue;
       *(thebuf + (*(offset2+1) + sample)) = (intens*255.);
       *(thebuf + (*(offset2+2) + sample)) = (sat*255.);
	}
/*
  else chromatic case
*/
   else if (high != low)
	{
	if (intens <= 0.5)
	   sat = (high-low)/(high+low);
	else
/*
	   sat = (high-low)/(2 - (high-low));
*/
	   sat = (high-low)/(2 - high - low);
	red = (high-scaler)/(high-low);
	green=(high-scaleg)/(high-low);
	blue =(high-scaleb)/(high-low);
/*
  resulting color between yellow and magenta
*/
	if (scaler == high)
	   hue = blue - green;
/*
  resulting color between cyan and yellow
*/
	else
	if (scaleg == high)
	   hue = 2 + red - blue;
/*
  resulting color between magenta and cyan
*/
	else
	if (scaleb == high)
	   hue = 4 + green - red;
/*
  convert to degrees
*/
  	hue *= 60.0;
/*
  make nonnegative
*/
	if (hue < 0.0)
	   hue += 360.0;
/*
  set the HIS output values
*/
       *(thebuf + (*(offset2+0) + sample)) = hue + 0.5;
       *(thebuf + (*(offset2+1) + sample)) = (intens*255.) + 0.5;
       *(thebuf + (*(offset2+2) + sample)) = (sat*255.) + 0.5;
	}

      }
	curent ++;
      if (strcmp(minmax,"YES") == 0)
	 {
         status = c_pxmnmx(thebuf + *(offset2+0),&ns,
                         &minval[0],&maxval[0],&odtype); 
         status = c_pxmnmx(thebuf + *(offset2+1),&ns,
                         &minval[1],&maxval[1],&odtype); 
         status = c_pxmnmx(thebuf + *(offset2+2),&ns,
                         &minval[2],&maxval[2],&odtype); 
	 }
       }
/* Final write
--------------*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);
   c_clean(vblock,"his",hosout,&one);
   }
/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"his",hosout,&one);

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","his-warn",&nonfatal);

/* Free buffer
--------------*/
free(thebuf);

return;
}
