#include "histeq.h"
#include "protos.h"

void genhist
(
    long window[],	/* input image window               */
    long *parts, 	/* lut length                       */
    long *jump, 		/* lut entries per detector         */
    long *skip, 		/* skip factor                      */
    long ndetect, 	/* no. detectors                    */
    long image_band_num[][MAXBND + 1],
    long *band,  	/* band being processed             */
    long *num_bands,	/* total number of input image bands*/
    long fhist[], 	/* image histogram                  */
    long maxhist[], 	/* max value for each detector      */
    long accdet[],	/* detectors to use for computing   */
    long nuse 		/* no. detectors to correct         */
)

/*
**********************************************************************

NAME: GENHIST

PURPOSE: Computes detector and image histograms

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         17-Jun-82    W. Grunberg   
    2         11-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
long	count, i;
static	long	*ihist;
long	ind, j, k, l, line,
	msg_flag = ONECAT,
	num_val,
	one = 1,
    tempmax = 0,
	type;

/*
allocate storage once for the histogram arrays
----------------------------------------------
*/
num_val= ndetect * 256;
if (*band == 1)
   if ((ihist = (long *) malloc (num_val * sizeof (long))) == NULL ||
       (chist = (long *) malloc (num_val * sizeof (long))) == NULL)
      c_errmsg ("Error allocating dynamic memory", "histeq-alloc", &fatal_err);
/*
initialize the histogram arrays
-------------------------------
*/
type = ELONG;
c_pxfill (ihist, &type, &num_val, &zero); 
c_pxfill (chist, &type, &num_val, &zero); 

num_val= 256;
c_pxfill (fhist, &type, &num_val, &zero); 

num_val= MAX_DET;
c_pxfill (maxhist, &type, &num_val, &zero); 

*parts = ndetect * 256;
*jump = 256;
/*
initialize and enable processing message utility
------------------------------------------------
*/
if (*band == 1)
   c_inittm ("HISTEQ", "Processed", "line", "band", " ", &window[NL], 
             num_bands, &msgtim, &msg_flag);
curent = 1;
curcat = *band;
c_startm();

type = EBYTE;
for (i = 0, line = 1; i < (window[SL]+window[NL]-1)/ndetect; i++)
   {
   for (j=0; j < *parts; j += *jump, curent++, line++)
      {
      c_eread (fdesc, &image_band_num[0][*band-1], &line, iobuf, &one);

      c_pxhiss (&window[NS], iobuf, &type, &ihist[j], &count, skip);
      }  
   }  
c_stoptm ();
/*
create cumulative arrays for sensors 
------------------------------------
*/
l = 0;
for (i = 0; i < *parts; i += *jump)
   {
   chist[i] = ihist[i];
   for (k = i+1; k < i+256; k++)
      {
      chist[k] = ihist[k] + chist[k-1];

/*
find subscript of maximum chist value for each detector
-------------------------------------------------------
*/
      if (tempmax  <  chist[k]) 
         {
         tempmax = chist[k];
         maxhist[l] = k-i;
         }  
      }  
   l++;
   tempmax = 0;
   }  

/*
create full array
-----------------
*/
for (i = 0; i < *parts; i += *jump)
   {
   ind = i/256;
   if (accdet[ind]) 
      for (k = 0; k < 256; k++) fhist[k] += chist[i+k];
   }  

/*
normalize cumulative array
--------------------------
*/
for (i = 0; i < *parts; i += *jump)
   for (l = i; l < i+256; l++) chist[l] *= nuse;

/*
free histogram storage
----------------------
*/
if (*band == *num_bands) free (ihist);
}
