#include "histeq.h"
#include "protos.h"

void genlut
(
    long parts, 		/* lut length                       */
    long jump, 		/* lut entries per detector         */
    long fhist[],	/* image histogram                  */
    long maxhist[],	/* max value for each detector      */
    long ndetect,	/* no. detectors                    */
    long precise,	/* precision                        */
    long detsel[],	/* selected detectors to be corrected */
    long band,		/* band being processed             */
    long num_bands	/* total number of input image bands*/
)

/*
**********************************************************************

NAME: GENLUT

PURPOSE: Generate look up table

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         17-Jun-82    W. Grunberg   
    2         11-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

**********************************************************************
*/
{
long	i, ij, ind, inmax, iover,
	j, k, kl, 
	maxmax, mn,
	next,
	num_val,
	range,
	type;

num_val= ndetect * 256;
/*
allocate storage for lookup tables
----------------------------------
*/
if (band == 1)
   if ((lut = (unsigned char *) malloc (num_val)) == NULL)
      c_errmsg ("Error allocating dynamic memory", "histeq-alloc", &fatal_err);
/*
initialize lookup tables
------------------------
*/
type = EBYTE;
c_pxfill (lut, &type, &num_val, &zero); 

maxmax = 0;
for (kl = 0; kl < ndetect; kl++)
   if (maxmax  <  maxhist[kl]) maxmax = maxhist[kl];
range = (1 << precise) - 1;
mn = 0;
for (i = 0; i < parts; i += jump)
   {
   ind = i/256;
   if (detsel[ind])   
      {
      k=0;
      while (chist[i+k]  <  fhist[0])
         {
         lut[i+k] = 0;
         k++;
         }  
      next = k;
      for (k = next; k <= maxhist[mn]; k++)
         for (j = 0; j <= maxmax-1; j++)
            if (fhist[j]  <=  chist[i+k]  && 
                chist[i+k]  <  fhist[j+1]) lut[i+k] = (unsigned char) j;

      inmax = lut[i+maxhist[mn]-1]+1;
      if (inmax  >  range) inmax=range;
      for (iover = maxhist[mn]; iover < 256; iover++)
         {
         lut[i+iover] = (unsigned char) inmax;
         inmax++;
         if (inmax  >  range) inmax=range;
         }  
      }
   else         
/*    
      detector not requested... generate identity lut
      -----------------------------------------------
*/
      for (ij = 0; ij < 256; ij++) lut[i+ij] = (unsigned char) ij;

   mn++;
   }  

/*
free histogram storage
----------------------
*/
if (band == num_bands) free (chist);
}
