#include <string.h>
#include "histeq.h"
#include "protos.h"

void getpar
(
    char lasin[][CMLEN],      /* input image las name             */
    char hosin[][CMLEN],      /* input image host names           */
    char lasrlut[CMLEN],      /* output image las name            */
    char hosrlut[CMLEN],      /* output image host name           */
    long *num_inimages,       /* number of input images           */
    long num_image_bands[],   /* number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* band numbers in each    */
    long *num_inbands,        /* total number of input image bands   */
    long wind[][4],           /* input image windows              */
    long *ndetect,            /* no. detectors                    */
    long *skip,               /* skip factor                      */
    long *precise,            /* precision                        */
    long detsel[MAX_DET],     /* selected detectors to be corrected */
    long accdet[MAX_DET],     /* detectors to use for computing   */
    long *idet               /* no. detectors to correct         */
)
/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         17-Jun-82    W. Grunberg   
    2         11-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
struct VARIABLE *var;               /* pointer to a variable in TAE block */

long	access,
	count, 
	det_count,
	dets[MAX_DET],              /* detectors to be corrected          */
	adets[MAX_DET],             /* detectors to use for histograms    */
	flag[MAXIMG][2],            /* flag for subwindows                */
	i,
	imglimit = 1,               /* maximum number of images           */
	itype = ELONG,
	msg,
	one = 1,
    same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

long inf_err = 1;
char buf[ERRLEN];               /* print buffer */

char	minmax[4];
/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input image parameters
---------------------------------------
*/
stat = c_getin (&block, "IN", &zero, lasin, hosin, num_inimages, wind, 
                image_band_num, num_image_bands, num_inbands, &imglimit, 
                &same_window, flag);

if (stat != E_SUCC) 
    c_errmsg ("Error accessing the input image", "histeq-in", 
              &fatal_err);

for (i = 1; i < *num_inimages; i++)
   {
   if (!same_window ||
       wind[i][SS] != wind[0][SS] || wind[i][SL] != wind[0][SL])
      c_errmsg ("Image windows must be same", "histeq-wind", 
                &fatal_err);
   }

/*
get output rlut file name
-------------------------
*/
access = IWRITE;
stat = c_getnam (&block, "OUTRLUT", &zero, hosrlut, "rlut", &access);
if (stat != E_SUCC)
   c_errmsg("Error getting output file name", "histeq-out", &fatal_err);
strcpy(lasrlut, hosrlut);

c_getnum (&block, "NDETECT", &one, &itype, (void *)ndetect);
c_getnum (&block, "PIXSAMP", &one, &itype, (void *)skip);
*precise = 8;

/*
determine user detector selection
-set an array of flags for up to 50 detectors
---------------------------------------------
*/
msg = FALSE;
count = MAX_DET;
if ((det_count = c_getnum (&block, "DETNUM", &count, &itype, (void *)dets))
                                                                         == 0)
   for (i = 0; i < *ndetect; i++) detsel[i] = TRUE;
else
   {
   for (i = 0; i < *ndetect; i++) detsel[i] = FALSE;
   for (i = 0; i < det_count; i++)
      {

/*
      store flags in reverse order for tm data
      ----------------------------------------
*/
      if (dets[i] <= *ndetect) 
         {
         if (*ndetect == 16) 
            detsel[*ndetect-dets[i]] = TRUE;
         else
            detsel[dets[i] - 1] = TRUE;
         }  
      else
         msg = TRUE;
      }  
   }  
if (msg)   
   {
   c_errmsg 
      ("Some detectors requested for correction will be ignored;",
       "histeq-detnum1", &inf_err); 
   c_errmsg ("they exceed NDETECT", "histeq-detnum2", &inf_err); 
   }

/*
an array of flags for the histogram accumulation (accdet)
---------------------------------------------------------
*/
msg = FALSE;
count = MAX_DET;
if ((det_count = c_getnum (&block, "CORRDET", &count, &itype, (void *)adets))
                                                                         == 0)
   {
   for (i = 0; i < *ndetect; i++) accdet[i] = TRUE;
   *idet = *ndetect;
   }
else
   {
   *idet = det_count;
   for (i = 0; i < *ndetect; i++) accdet[i] = FALSE;
   for (i = 0; i < det_count; i++)
      {
/*
      store flags in reverse order for tm data
      ----------------------------------------
*/
      if (adets[i] <= *ndetect) 
         {
         if (*ndetect == 16) 
            accdet[*ndetect-adets[i]] = TRUE;
         else
            accdet[adets[i] - 1] = TRUE;
         }
      else
         {
         (*idet)--;
         msg = TRUE;
         }
      }  
   }  
if (msg)   
   {
   c_errmsg 
      ("Some detectors to be used for correction will be ignored;",
       "histeq-corrdett1", &inf_err); 
   c_errmsg ("they exceed NDETECT", "histeq-corrdet2", &inf_err); 
   }
/*
if number of lines not a multiple of number of detectors then adjust 
and issue message
--------------------------------------------------------------------

*/
if (wind[0][NL] % (*ndetect)  !=  0) 
   {
   wind[0][NL] = wind[0][NL] / (*ndetect) * (*ndetect);
   sprintf (buf, "Number of lines adjusted to %d, a multiple of NDETECT",
            wind[0][NL]);
   c_errmsg (buf, "histeq-nlines", &inf_err);
   }  
/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);
var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;
}
