/***********************************************************************

NAME: HISTEQ

PURPOSE: HISTEQ is a radiometric correction program whose
         purpose is to correct images with regular striping due to
         differential detector response within a band in a
         multi-detector sensor.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         17-Jun-82    W. Grunberg   
    2         28-Aug-84    S. Ross       
    3         18-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get parameters from TAE
   Call FILSPEC set up image i/o specifications
   For number of bands
      Call GENHIST to generate detector and image histograms
      Call GENLUT to generate radiometric lookup table
      Call LUTOUT to output lookup table to file
   Call COMPLETE to inform user about successful completion

***********************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

#define MAX_DET 50

struct PARBLK block;                   /* TAE parameter block     */

unsigned        char    *iobuf;        /* image i/o buffer        */

long    *chist;                        /* detector cumulative     */
                                               /* histogram               */
FILE    *fd_sfout;                      /* output rlut file desc   */
struct FDESC * fdesc[MAXIMG];                 /* file descriptors        */
long    min_max;                       /* min max flag            */
long    msgtim;                        /* message interval        */

extern long    curcat;                        /* message utility         */
extern long    curent;                        /* parameters for processing */

static   long    one = 1;

unsigned char   *lut;

int main
(void) 
{
static
long	accdet[MAX_DET],	/* detectors to use for computing   */
                                /* histogram                        */
	band,			/* band being processed             */
	detsel[MAX_DET],
	fhist[256];		/* image histogram                  */
static
long	image_band_num[MAXIMG][MAXBND + 1]; 
	                        /* band numbers in each input image*/
static
long	jump,			/* lut entries per detector         */
	maxhist[MAX_DET],	/* max value for each detector      */
	num_bands,              /* total number of input image bands*/
	ndetect,		/* no. detectors                    */
	num_image_bands[MAXIMG],/* number of bands in each input image*/
	num_inimages,           /* number of input images          */
	nuse,			/* no. detectors to correct         */
	parts,			/* lut length                       */
	precise,		/* precision                        */
	skip,			/* skip factor                      */
	wind[MAXIMG][4];        /* input image windows             */
static
char	hosrlut[CMLEN],         /* output RLUT file host name      */
	lasrlut[1][CMLEN];         /* output RLUT file                */
char	hosin[MAXIMG][CMLEN];   /* input images host names         */
char	lasin[MAXIMG][CMLEN];	/* input images las names          */

/*
get parameters
--------------
*/
getpar (lasin, hosin, lasrlut[0], hosrlut, &num_inimages, 
        num_image_bands, image_band_num, &num_bands, 
        wind, &ndetect, &skip, &precise, detsel, accdet, &nuse);

/*
open input image
----------------
*/
filspec (hosin, num_inimages, num_image_bands, wind);

/*
perform histogram equalization and generate lookup tables
---------------------------------------------------------
*/
for (band = 1; band <= num_bands; band++)
   {
/*
   compute the histograms for the detectors and the whole image
   ------------------------------------------------------------
*/
   genhist (wind[0], &parts, &jump, &skip, ndetect, image_band_num, 
            &band, &num_bands, fhist, maxhist, accdet, nuse);

/*
   generate the lookup tables
   --------------------------
*/
   genlut (parts, jump, fhist, maxhist, ndetect, precise, detsel,
           band, num_bands);

/*
   output look up tables
   ---------------------
*/
   lutout (parts, jump, hosin, image_band_num, &band, hosrlut, wind[0], 
           &ndetect, num_bands);
   }  
/*
issue completion message
------------------------
*/
c_complt ("HISTEQ", lasrlut, &one);

return EXIT_SUCCESS;

}
