#include "histeq.h"
#include "protos.h"

void lutout
(
    long parts, 		/* lut length                       */
    long jump, 		/* lut entries per detector         */
    char hosin[][CMLEN],	/* input images host names          */
    long image_band_num[][MAXBND + 1], /* band numbers       */
    long *band,		/* band processed                   */
    char hosout[],	/* output RLUT file host name       */
    long window[4],	/* input image window               */
    long *ndetect,	/* no. detectors                    */
    long num_bands	/* total number of input image bands*/
)

/*
**********************************************************************

NAME: LUTOUT

PURPOSE: Outputs lookup table to the output rlut file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         17-Jun-82    W. Grunberg   
    2         17-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   if first band
      open stats file for output
      store number of detectors at file level
      get image attributes and store at file level
   get and store band number at the class level -
      if MSS data add eight to band
   place lookup table at the site level, one lookup table per site
   - if MSS data put lookup tables in order 1-number of detectors
   - if TM data put lookup tables in order 16-1
   if last band close the statistics file

**********************************************************************
*/
{
struct	BDDR 	bddr;
struct	DDR 	ddr;

long	access, i, iband, nclass, nsite, 	start;
	
static	long	bandshape[9] = {ELONG, 1, 1, 0, 0, 0, 0, 0, 0};
static	long	detshape[9] = {ELONG, 1, 1, 0, 0, 0, 0, 0, 0};
static	long	filshape[9] = {EWORD, 1, 5, 0, 0, 0, 0, 0, 0};
static	long	lutshape[9] = {EBYTE, 1, 256, 0, 0, 0, 0, 0, 0};

short	filblk[5], mline, msamp;

/*
if first band, open stats file for output, get image related information
and store them in the stats file
------------------------------------------------------------------------
*/
if (*band == 1)
   {
   access = IWRITE;
   if (c_stopen (&fd_sfout, hosout, &access, &nclass) != E_OPNEW)
      c_errmsg ("Error opening statistics file", "histeq-stopen", &fatal_err); 
/*
   put ndetect at file level
   -------------------------
*/
   c_stput (&fd_sfout, "NDETECT", &zero, &zero, detshape, ndetect); 

/*
   get image attributes and store at file level
   --------------------------------------------
*/
   c_getddr (hosin[0], &ddr);

   if (ddr.master_line == 0)
      {
      mline = 1;
      msamp = 1;
      }
   else
      {
      mline = (short) (ddr.master_line);
      msamp = (short) (ddr.master_sample);
      }  

   filblk[0] = (short)  num_bands;
   filblk[1] = (short) (window[SL] + mline - 1);
   filblk[2] = (short)  window[NL];
   filblk[3] = (short) (window[SS] + msamp - 1);
   filblk[4] = (short)  window[NS];

   c_stput (&fd_sfout, "FILBLK", &zero, &zero, filshape, filblk); 
   }
/*
get and store band number at the class level -
if MSS data add eight to iband
----------------------------------------------
*/
c_getbdr (hosin[0], &bddr, &image_band_num[0][*band-1]);
iband = bddr.bandno;
if (*ndetect == 6) iband += 8;

c_stput (&fd_sfout, "BANDNUM", band, &zero, bandshape, &iband); 

/*
place lookup table at the site level, one lookup table per site
   - MSS data put lookup tables in order 1-number of detectors
   - TM data put lookup tables in order 16-1
---------------------------------------------------------------
*/
nsite = 1;
start = 0;
if (*ndetect == 16) 
   {
   start = parts - jump;
   parts = -jump;
   jump = -jump;
   }  
i = start; 
while (i != parts)
   {
   c_stput (&fd_sfout, "DETRLT", band, &nsite, lutshape, &lut[i]); 
   nsite++;
   i += jump;
   }  

/*
if last band has been processed, close the statistics file
----------------------------------------------------------
*/
if (*band == num_bands) 
   {
   c_stclse (&fd_sfout); 
   free (lut);
   }
}
