/*******************************************************************************
FUNCTION:	getpar

PURPOSE:	Retrieve all parameters from TAE.

*******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"


int getpar
(
    struct PARBLK *vblock,	/* Pointer to TAE parameter block             */
    char hosin1[][CMLEN],	/* Host names of images in IN(1)              */
    long *nmrimg1,		/* Number of input images in IN(1)            */
    long window1[][4],		/* Window for each image in IN(1)             */
    long bands1[][MAXBND+1],	/* Band to compare in IN(1)                   */
    long nbands1[],		/* Number of bands for each image in IN(1)    */
    long *totbnd,		/* Total number of bands to be processed      */
    char hosin2[][CMLEN],	/* Host names of images in IN(2)              */
    long *nmrimg2,		/* Number of input images in IN(2)            */
    long window2[][4],		/* Window for each image in IN(2)             */
    long bands2[][MAXBND+1],	/* Band to compare in IN(2)                   */
    long nbands2[],		/* Number of bands for each image in IN(2)    */
    long *msgtim,		/* Elapsed time between processing messages   */
    double *tolval		/* Tolarence value                            */
)
{
char taein[MAXIMG][CMLEN];	/* LAS names of the input images              */

struct VARIABLE *val;		/* Pointer to a TAE variable within v-block   */

long element	= 0;		/* Element number for getin()                 */
long fatal	= -1;		/* Fatal error flag                           */
long maximg;			/* Maximum number of images to be processed   */
long nonfatal	= 1;		/* Nonfatal error flag                        */
long qflag[MAXIMG][2];		/* Flag indicating if subwindows specified    */
long same;			/* Flag indicating if images are same size    */
long status;			/* Function return status                     */
long totbnd2;			/* Total number of bands in second group      */


/*
  The maximum number of images that can be specified for one index of IN
*/
maximg = MAXIMG / 2;


/*
  Get first image specification
*/
element = 0;
status = c_getin(vblock, "IN", &element, taein, hosin1, nmrimg1, window1,
    bands1, nbands1, totbnd, &maximg, &same, qflag);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);


/*
  Ensure images are the same size
*/
if (!same)
    {
    c_errmsg("Not all images are the same size", "imgcmp-size", &nonfatal);
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    }


/*
  Get second image specification
*/
element = 1;
status = c_getin(vblock, "IN", &element, taein, hosin2, nmrimg2, window2,
    bands2, nbands2, &totbnd2, &maximg, &same, qflag);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);


/*
  Ensure images are the same size
*/
if (!same)
    {
    c_errmsg("Not all images are the same size", "imgcmp-size", &nonfatal);
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    }


/*
  Now ensure the two entries of IN are the same size
*/
if ((window1[0][NL] != window2[0][NL]) || (window1[0][NS] != window2[0][NS]))
    {
    c_errmsg("Not all images are the same size", "imgcmp-size", &nonfatal);
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    }


/*
  Ensure both entries have the same number of bands
*/
if (*totbnd != totbnd2)
    {
    c_errmsg("Different number of bands specified", "imgcmp-bands", &nonfatal);
    c_errmsg("Fatal error encountered", "imgcmp-fatal", &fatal);
    }


/*
  Retrieve the tolerence value
*/
val = p_find(vblock,"TOLVAL");
*tolval = RVAL(*val,0);


/*
  Retrieve LAS globals
*/
val = p_find(vblock,"$MSGTIM");	
*msgtim = IVAL(*val,0);

return(E_SUCC);
}
