/**************************************************************************
NAME: FILTER2 

PURPOSE: Searches each line for a valid data point and compiles values
         needed to determine the equations which describe the image edges.
         
PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           8/92      T. Jensen 
    2		1/93	  T. Mittan	      Released to baseline

PROJECT: LAS

*************************************************************************/
#include <math.h>
#include "las.h"
#include "imgcorner.h"

void filter2
(
  long inbuf[],                /* I: Buffer */
  long ns,                     /* I: Number of samples per line to process */
  long iinline,                /* I: Input line number */
  long *ptcntr,                /* I: Corner number */
  float *sumx,                 /* O: Sum of x's */
  float *sumy,                 /* O: Sum of y's */
  float *sumxx,                /* O: Sum of x * x */
  float *sumxy,                /* O: Sum of x * y */
  float *n,                    /* O: Number of elements */
  long value[]                 /* I: Range of values to ignore */
)
{
register long	j;		/* array index				*/
register long   w;		/* array index				*/
long    flag;			/* flag value				*/

flag=FALSE;

/* Find valid data points and determine values to help find equations for the 
   image edges
 ----------------------------------------------------------------------------*/

/*  Left edge of scene -- find valid pts and acquire sums 
----------------------------------------------------------*/
  if(*ptcntr==1)
  {
    for(j=0;j<(ns-18);j++)
    {
    if (((inbuf[j] < value[0]) || (inbuf[j] > value[1])) && 
	((inbuf[j] < value[2]) || (inbuf[j] > value[3])))
       {
        flag=1;
        for(w=1;w<18;w++)
        {
          if((inbuf[j+w]==0)||(inbuf[j+w]>125))
          {
            flag=FALSE;
	    j += w;
	    break;
          }
        }
      }
      if(flag==1)
      {
        *sumx += j;
        *sumxx += (j*j);
	*sumxy += (j*iinline);
        *sumy += iinline;
        *n += 1;
	break;
      }
    }
  }  
/*   Bottem edge of scene -- find valid pts and acquire sums 
-------------------------------------------------------------*/
  if(*ptcntr==2)
  {
    for(j=0;j<(ns-18);j++)
    {
    if (((inbuf[j] < value[0]) || (inbuf[j] > value[1])) && 
	((inbuf[j] < value[2]) || (inbuf[j] > value[3])))
      {
        flag=1;
        for(w=1;w<18;w++)
        {
          if((inbuf[j+w]==0)||(inbuf[j+w]>125))
          {
           flag=FALSE;
	   j += w;
	   break;
          }
        }
      }
      if(flag==1)
      {
        *sumx += j;
        *sumxx += (j*j);
	*sumxy += (j*iinline);
        *sumy += iinline;
        *n += 1;
	break;
      }
    }
  }
/*   Top edge of scene -- find valid pts and acquire sums 
----------------------------------------------------------*/
  if(*ptcntr==3)
  {
    for(j=(ns-18);j>18;--j)
    {
    if (((inbuf[j] < value[0]) || (inbuf[j] > value[1])) && 
	((inbuf[j] < value[2]) || (inbuf[j] > value[3])))
      {
        flag=1;
        for(w=1;w<18;w++)
        {
          if((inbuf[j-w]==0)||(inbuf[j-w]>125))
          {
            flag=FALSE;
	    j -= w;
 	    break;
          }
        }
      }
      if(flag==1)
      {
        *sumx += j;
        *sumxx += (j*j);
	*sumxy += (j*iinline);
        *sumy += iinline;
        *n += 1;
	break;
      }
    }
  }
/*   Right edge of scene -- find valid pts and acquire sums 
------------------------------------------------------------*/
  if(*ptcntr==4)
  {
    for(j=ns;j>18;--j) 
    {
    if (((inbuf[j] < value[0]) || (inbuf[j] > value[1])) && 
	((inbuf[j] < value[2]) || (inbuf[j] > value[3])))
      {
        flag=1;
        for(w=1;w<18;w++)
        {
          if((inbuf[j-w]==0)||(inbuf[j-w]>125))
          {
           flag=FALSE; 
	   j -= w;
	   break;
          }
        }
      }
      if(flag==1)
      {
        *sumx += j;
        *sumxx += (j*j);
	*sumxy += (j*iinline);
        *sumy += iinline;
        *n += 1;
	break;
      }
    }
  }
}
