/***************************************************************************
NAME: GETPAR 

PURPOSE: Get necessary parameters. 

PROGRAM HISTORY:
Version	Date    Author	      Request
-------	----    ------        -------
3       1/93    T. Mittan     Released to baseline

PROJECT: LAS

ALGORITHM DESCRIPTION:
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "imgcorner.h"

#define TERM 0
#define PFILE 1
#define LP 2

int getpar
(
  char hosin[][CMLEN],         /* O: Input image */
  long *totbnd,                /* O: Total bands */
  long bands[][MAXBND+1],      /* O: Band array */
  long nbands[],               /* O: Number of bands */
  long *nl,                    /* O: Number of lines */
  long *ns,                    /* O: Number of samples */
  long window[][4],            /* O: Input window */
  long *nmrimg,                /* O: Number of images */
  char prtfile[],              /* O: Output print file */
  long print[],                /* O: Print flags */
  long *indist,                /* O: Shift distance for corners */
  long *value,                 /* O: Fill value */
  char method[]                /* O: Method for finding points */
)
{
  extern long msgtim;          /* Message display interval */

  extern long fatal;           /* Fatal error flag */
  extern long inf_err;         /* Non fatal error flag */

  extern long zero;            /* Constant of zero */	
  extern long one;             /* Constant of one */
  extern char hosout[CMLEN];   /* Host name of output file */

  struct VARIABLE *var;
  long maximg = MAXIMG;        /* Maximum number of images */
  long qflag[MAXIMG][2];       /* Window flag */
  long same;                   /* Same size window flag */
  long status;                 /* Return status */
  long std_out;                /* Standard output */
  long len;                    /* String length */
  long i;                      /* Array value */
  long acc = IWRITE;           /* Access */

  char taein[MAXIMG][CMLEN];   /* Input TAE name */
  char *trim;                  /* Temporary variable for q_string */
  char temp[CMLEN];            /* Temporary variable for q_string */
  char tmptae[CMLEN];          /* Temp TAE output file name */
  char filename[CMLEN];        /* Process name */
  struct PARBLK parblk;        /* TAE parameter block */

  /* Initialization
  -----------------*/
  c_inlas(&parblk,&std_out);

  /* Get globals                     
  --------------*/
  var = p_find(&parblk, "$msgtim");
  msgtim = IVAL(*var, 0);

  /* Get input file information
  -----------------------------*/
  status = c_getin(&parblk, "IN", &zero, taein, hosin, nmrimg, window, bands, 
                   nbands, totbnd, &maximg, &same, qflag);
  if (status != E_SUCC)
    return(E_FAIL);

  if (same != 1) 
    {
    c_lasmsg(FALSE,"imgcorner-window",
             "Images do not have the same window sizes");
    return(E_FAIL);
    }

  *nl = window[0][NL];
  *ns = window[0][NS];

  /* Get print option
  -------------------*/
  strcpy(prtfile,"imgcorner");
  if (c_setprt(&parblk,&print[TERM],&print[PFILE],&print[LP],
               prtfile,filename) != E_SUCC)
    return(E_FAIL);

  /* Get output file name
  -----------------------*/
  status = c_getnam(&parblk,"OUTLT",&zero,hosout,"trim",&acc);
  if (status  !=  E_SUCC) 
    return(E_FAIL);

  var = p_find(&parblk, "OUTLT");
  strcpy(temp,SVAL(*var, 0));
  c_cmhost(temp,hosout);

/*
  The local version of c_creat_name was removed because it was
    performing the same operation as the support function, except
    for placing a ; instead of a . with the extension

  c_creat_name(hosout,"txt");
  strcpy(temp,hosout);
  len = strlen(temp);
  temp[len - 4] = '\0';
  trim[0] = &temp[0];
  status = q_string(&parblk,"temp",1,trim,P_UPDATE);
  status = q_out(&parblk);
  strcpy(temp,hosout);
  c_cmhost(temp,hosout);
*/

  c_creat_name(hosout,"txt");
  strcpy(tmptae, hosout);
  len = (long)strlen(tmptae);
  tmptae[len-4] = '\0';
  trim = tmptae;
  status = q_string(&parblk,"temp",1,&trim,P_UPDATE);
  status = q_out(&parblk);

  /* Get shift value
  ------------------*/
  var = p_find(&parblk, "OFFSET");
  *indist = IVAL(*var, 0);

  /* get fill value
  ----------------*/
  var = p_find(&parblk, "FILLVAL");
  for (i = 0; i < var->v_count; i++)
    value[i] = IVAL(*var, i);
  if (var->v_count == 1)
    value[1] = value[2] = value[3] = value[0];
  else
  if (var->v_count == 2)
    {
    value[2] = value[0];
    value[3] = value[1];
    }
 else
   if (var->v_count == 3)
     value[3] = value[2];

  /* Get method
  -------------*/
  var = p_find(&parblk, "METHOD");
  strcpy(method,SVAL(*var, 0));

  return(E_SUCC);
}
