/***************************************************************************
NAME: CUT 
 
PURPOSE: Determines the corner coordinates of an image which is
         surrounded by white and black "fill".
 
PROGRAM HISTORY:
Version     Date     Author             Request
-------     ----     ------             -------
3           1/93     T. Mittan          Original development 
 
PROJECT: LAS
***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "imgcorner.h"

int cut
(
  long polybuf[],              /* O: Polynomial corner buffer */
  long nl,                     /* I: Number of lines in image */
  long ns,                     /* I: Number of samples in image */
  long indist,                 /* I: Distance to move corner toward center */
  long sl,                     /* I: Starting line of image */
  long ss,                     /* I: Starting sample of image */
  long value[],                /* I: Fill value range */
  long bands[][MAXBND+1]       /* I: Band array */
)
{
  extern long curent;          /* Global variable for message routine */
  long *buf;                   /* Input image data buffer */
  long k;                      /* Array index */
  long long extr[4];           /* Corner position */
  long line_num;               /* Array index */
  long samp_num;               /* Array index */
  long long pixel_num;	       /* Pixel number = line * samps + samp number */
  long long max_pixel_num;
  long val;                    /* Image data */
  long flag;                   /* Flag value for corner */

  long ur_samp_pos = 0;
  long ll_samp_pos = ns;

  /* Read the image finding the four corners of non-fill data.
     this is done by finding the first non-fill data closes to
     the input image corners.
  ------------------------------------------------------------*/

  /* Allocate the space needed for each line
  ------------------------------------------*/
  buf = (long *)malloc(sizeof(long) * ns);
  if (buf == NULL)
  {
    c_lasmsg(FALSE,"imgcorner-alloc","Could not allocate appropriate space");
    return(E_FAIL);
  }
  
  c_lasmsg(FALSE,"imgcorner-info","In cut function");

  /* Initialize the extr array
  ----------------------------*/
  max_pixel_num = (long long)nl * ns + 1;
  for (k = 0; k < 4; k++) 
    extr[k] = max_pixel_num;
    extr[2] = 0;		/* LR corner */
    // extr[k] = ns + nl + 1;

  /* Main processing loop
  -----------------------*/
  for (line_num = 0; line_num < nl; line_num++)
  {
    get_line(bands[0][0], ns, buf);
    curent++;

    for (samp_num = 0; samp_num < ns; samp_num++)
    {
      flag = TRUE;
      val =  buf[samp_num];

      /* If not fill value
      --------------------*/
      if (((val < value[0]) || (val > value[1])) && ((val < value[2]) ||
           (val > value[3])))
      {
        pixel_num = (long long)line_num * ns + samp_num;
        
	/* Up left corner
        -----------------*/
        // if ((line_num+samp_num) < extr[0])
        if (pixel_num < extr[0])
        {
          /*for (k = samp_num; k < samp_num+25; k++)
            if (((buf[k] >= value[0]) && (buf[k] <= value[1])) || 
                ((buf[k] >= value[2]) && (buf[k] <= value[3])))
            {
              flag = FALSE;
              break;
            }
	    */
          if (flag)
          {
            extr[0] = pixel_num;
            polybuf[0] = (double)(line_num+sl+indist);
            polybuf[1] = (double)(samp_num+ss+indist);
          }
	  continue;
        }

        /* Up right corner
        ------------------*/
        if (samp_num > ur_samp_pos)
        {
          /*for (k = samp_num; k > samp_num-25; k--)
            if ((k <= 0) || ((buf[k] >= value[0]) && 
                (buf[k] <= value[1])) || ((buf[k] >= value[2]) &&
                (buf[k] <= value[3])))
            {
              flag = FALSE;
              break;
            }
	    */
          if (flag)
          {
            // extr[1] = line_num - samp_num;
            extr[1] = pixel_num;
            ur_samp_pos = samp_num;
            polybuf[2] = (double)(line_num+sl+indist);
            polybuf[3] = (double)(samp_num+ss-indist);
          }
        }

        /* Lower right corner
        ---------------------*/
        if ( pixel_num > extr[2])
        {
          /*for (k = samp_num; k > samp_num-25; k--)
            if ((k <= 0) || ((buf[k] >= value[0]) && 
                (buf[k] <= value[1])) || ((buf[k] >= value[2]) && 
                (buf[k] <= value[3])))
            {
              flag = FALSE;
              break;
            }
	    */
          if (flag)
          {
            extr[2] = pixel_num;
            polybuf[4] = (double)(line_num+sl-indist);
            polybuf[5] = (double)(samp_num+ss-indist);
          }
        }

        /* Lower left corner
        --------------------*/
        if (samp_num < ll_samp_pos)
        {
          /*for (k = samp_num; k < samp_num+25; k++)
            if (((buf[k] >= value[0]) && (buf[k] <= value[1])) || 
                ((buf[k] >= value[2]) && (buf[k] <= value[3])))
            {
              flag = FALSE;
              break;
            }
	    */
          if (flag)
          {
            extr[3] = pixel_num;
	    ll_samp_pos = samp_num;
            polybuf[6] = (double)(line_num+sl-indist);
            polybuf[7] = (double)(samp_num+ss+indist);
          }
        }
      }
    }
  }
  polybuf[8] = polybuf[0];
  polybuf[9] = polybuf[1];

  free(buf);
  return(E_SUCC);
}
