/***************************************************************************
NAME:                          IMGCORNER

PURPOSE:     Determines the corner coordinates of a Landsat MSS image
             which is surrounded by white and black "fill".	

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           8/92     T. Jensen       Driver program for IMGCORNER 
   2          11/92     T. Jensen       Improvements to include wider array
                                        of image sizes
   3	       1/93	T. Mittan	Released to baseline

PROJECT: 	LAS

ALGORITHM DESCRIPTION:
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "imgcorner.h"

#define TERM 0
#define PFILE 1
#define LP 2


long line = 1;
struct FDESC *infd;             /* file descriptors          */
long    msgtim;                 /* message display interval  */

extern  long    curcat;         /* parameters for processing */
extern  long    curent;         /* message utility           */
long    fatal = -1;		/* fatal error flag	     */
long    inf_err = 1;		/* non fatal error flag	     */

long    zero = 0;		/* constant of zero 	     */	
long    one = 1;		/* constant of one	     */
char	hosout[CMLEN];		/* host name of output file  */

void main
(
  void
) 
{
FILE 	*outfile;		     /* output data file	      */
FILE	*fptr;			     /* output print file	      */

long		acc;
long	bands[MAXIMG][MAXBND+1];     /* image band numbers    	       */
long		curimg;              /* the current image counter      */
long		i;                   /* a counter                      */
long		intype;              /* input data type                */ 
long		indist;		     /* corner shift distance	       */
long	nbands[MAXIMG];      	     /* number of bands in each image  */
long		nl;                  /* number of lines in image       */
long		nmrimg;              /* number of input images         */ 
long		ns;                  /* number of samples/line in image*/
long		onecat = ONECAT;     /* for inittm                     */
long		print[3];	     /* print flag		       */
long            pn;                  /* polygon number                 */
long		status;              /* function return status code    */ 
long		totbnd;              /* total number of bands          */
long		two = 2;	     /* constant value of two	       */
long		bndin;		     /* band number		       */
long		polybuf[10];         /* corner buffer		       */
long		value[4];	     /* range of fill values	       */
long	window[MAXIMG][4];   	     /* window for each input image    */

char	hosin[MAXIMG][CMLEN];	     /* host names of input images     */
char 	prtfile[CMLEN];		     /* print file name		       */
char	tempbuf[2][CMLEN];	     /* temproary buffer for output    */
char	method[10];	 	     /* method for finding points      */

long *buf;	     	     /* input image buffer	       */

long numbands;			     /* number of input bands	       */
long x[MAXBND + 1][4];		     /* Coords: each corner & band     */
long y[MAXBND + 1][4];		     /* Coords: each corner & band     */
long x1,x2,x3,x4,y1,y2,y3,y4;	     /* corner x's and y's	       */


/* Get user-specified parameters and tae global variables
---------------------------------------------------------*/
status = getpar(hosin,&totbnd,bands,nbands,&nl,&ns,window,&nmrimg,prtfile,
                print,&indist,value,method);
if (status != E_SUCC)
  c_lasmsg(TRUE,"imgcorner-fatal","Fatal error encountered");

/* Allocate the input buffer space
----------------------------------*/
buf = (long *)malloc(sizeof(long) * ns);
if (buf == NULL)
  c_lasmsg(TRUE,"imgcorner-alloc","Could not allocate appropriate space");

/* set data type equal to ELONG
-----------------------------*/
intype = ELONG;

/*Open output file for corner pts   
______________________________________*/
fptr = fopen(prtfile,"w");
if(fptr == NULL)
   {
   c_errmsg("Error opening print file","imgcorner-open",&fatal);
   }

   bndin = 1;
for (curimg = 0; curimg < nmrimg; curimg++)
   {

   /* Open input image
   -------------------*/
   numbands = 0;
   acc = IREAD;
   status=c_eopenr(&infd,hosin[curimg],&numbands,&(window[curimg][SL]),
	&(window[curimg][SS]),&(window[curimg][NL]),&(window[curimg][NS]),
	&acc,&intype,&zero,&one);
   if(status!=E_SUCC)
        c_errmsg("Fatal error encountered","imgcorner-fatal",&fatal);
/* Process each band
--------------------*/
   curent = curcat = 1;

if ((strcmp(method,"NOFILL")) == 0)
  {
  if (bndin == 1)
    {
    c_inittm("IMGCORNER","Processed","line","band"," ",&nl,&one,&msgtim,
             &onecat);
    c_startm ();
    line = 1;
    status = cut(polybuf,nl,ns,indist,window[0][SL],window[0][SS],value,bands);
    if (status != E_SUCC)
      c_lasmsg(TRUE,"imgcorner-fatal","Fatal error encountered");
    bndin++;
    }
  }
else
  {
  if (bndin == 1)
    {
    c_inittm("IMGCORNER","Processed","line","band"," ",&nl,&totbnd,&msgtim,
	   &onecat);
    c_startm ();
    }
  line = 1;
  status = lse(fptr,&curimg,nbands,&nl,&ns,&bndin,value,x,y,bands);
  if (status != E_SUCC)
    c_lasmsg(TRUE,"imgcorner-fatal","Fatal error encountered");

  /* Find smallest polygon coordinates from all of the bands.
  -----------------------------------------------------------*/
  for (x1 = 0, i = 0; i < MAXBND; i++)
     {
     if(x[i][1]>x1)
        x1=x[i][1];
     }
  for (y1 = 0, i = 0; i < MAXBND; i++)
     {
     if(y[i][1]>y1)
         y1=y[i][1];
     }
  for (y2 = 0, i = 0; i < MAXBND; i++)
     {
     if(y[i][2]>y2)
        y2=y[i][2];
     }
  for (x2 = ns, i = 0; i < MAXBND; i++)
     {
     if(x[i][2]<x2)
        x2=x[i][2];
     }
  for (x3 = 0, i = 0; i < MAXBND; i++)
     {
     if(x[i][3]>x3)
        x3=x[i][3];
     }
  for (y3 = ns, i = 0; i < MAXBND; i++)
     {
     if(y[i][3]<y3)
        y3=y[i][3];
     }
  for (x4 = ns, i = 0; i < MAXBND; i++)
     {
     if(x[i][4]<x4)
        x4=x[i][4];
     }
  for (y4 = ns, i = 0; i < MAXBND; i++)
     {
     if(y[i][4]<y4)
        y4=y[i][4];
     }
  }
/* Close input file
--------------------------------*/
   status = c_eclose (&infd);
   if (status  !=  E_SUCC) 
      c_errmsg ("Nonfatal error encountered", "imgcorner-nonfatal", &inf_err);
  }
  c_stoptm();

/* Write results to output file; write the polygon number(which is one each time) and the corner coordinates.  List them starting with the upper left corner
and moving around the polygon in a clockwise fashion: UL, UR, LR, LL.
------------------------------------------------------------------------------*/
   outfile = fopen(hosout,"w");
   if(outfile == NULL)
      {
      c_errmsg("Error opening trim file","imgcorner-open",&fatal);
      }
    if ((strcmp(method,"NOFILL")) == 0)
      {
      pn = 1;
      fprintf(fptr,"\n\n%ld %ld\n",polybuf[0],polybuf[1]); 
      fprintf(fptr,"%ld %ld\n",polybuf[2],polybuf[3]); 
      fprintf(fptr,"%ld %ld\n",polybuf[4],polybuf[5]); 
      fprintf(fptr,"%ld %ld\n",polybuf[6],polybuf[7]); 
      fprintf(outfile,"%ld %ld %ld\n",pn,polybuf[0],polybuf[1]); 
      fprintf(outfile,"%ld %ld %ld\n",pn,polybuf[2],polybuf[3]); 
      fprintf(outfile,"%ld %ld %ld\n",pn,polybuf[4],polybuf[5]); 
      fprintf(outfile,"%ld %ld %ld\n",pn,polybuf[6],polybuf[7]); 
      }
    else
      {
      pn=1;
      y1 += indist;
      x1 += indist;
      y2 += indist;
      x2 -= indist;
      y4 -= indist;
      x4 -= indist;
      y3 -= indist;
      x3 += indist;
      fprintf(outfile,"%ld %ld %ld\n",pn,y1,x1); 
      fprintf(outfile,"%ld %ld %ld\n",pn,y2,x2); 
      fprintf(outfile,"%ld %ld %ld\n",pn,y4,x4); 
      fprintf(outfile,"%ld %ld %ld\n",pn,y3,x3); 
      fprintf(fptr,"\n\n%ld %ld\n",y1,x1); 
      fprintf(fptr,"%ld %ld\n",y2,x2); 
      fprintf(fptr,"%ld %ld\n",y4,x4); 
      fprintf(fptr,"%ld %ld\n",y3,x3); 
      }

free(buf);

/* Close  output file
--------------------------------*/
fclose(outfile);
/* Output completion message
----------------------------*/
/* print completion message
-------------------------*/
if (print[PFILE])
   {
   strcpy(tempbuf[0],hosout);
   strcpy(tempbuf[1],prtfile);
   c_complt("IMGCORNER",tempbuf,&two);
   }
else
   c_complt ("IMGCORNER", hosout, &one);
}


void get_line
(
  long bndin,                  /* I: Band number to read */
  long ns,                     /* I: Number of samples per line */
  long *buf                    /* O: Input image buffer */
)
{
long status;
 
status = c_eread(&infd, &bndin, &line, (void *)buf, &one);
if (status  !=  ns)
  {
   c_eclose(&infd);
   c_errmsg("Fatal error encountered", "imgcorner-fatal",&fatal);
  }   
line++;
}
