/***************************************************************************
                                LSE 

PURPOSE:     Determines the corner coordinates of a Landsat MSS image
             which is surrounded by "fill".	

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   3	       1/93	T. Mittan	   Pulled out of main routine	

PROJECT: 	LAS

ALGORITHM DESCRIPTION:
***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "imgcorner.h"

int lse
(
  FILE *fptr,                  /* I: Output print file */
  long *curimg,                /* I: Current image counter */
  long nbands[MAXIMG],         /* I: Number of bands in each image */
  long *nl,                    /* I: Number of lines in image */
  long *ns,                    /* I: Number of samples/line in image */
  long *bndin,                 /* I: Band number */
  long value[],                /* I: Range of fill values */
  long x[][4],                 /* O: Coords: each corner & band */
  long y[][4],                 /* O: Coords: each corner & band */
  long bands[][MAXBND+1]       /* I: Band array */
)
{
extern  long    line;
extern  long    curcat;         /* parameters for processing */
extern  long    curent;         /* message utility           */

long i;                   	     /* a counter                      */
long *buf;		     	     /* input image buffer	       */
long lineno;			     /* line number to read	       */
long ptcntr;			     /* corner number		       */
float sumx,sumy,sumxx,sumxy,n;       /* Values needed for least sqrs   */
float avgy,avgx;   		     /* averages 		       */
float m1,m2,m3,m4,b1,b2,b3,b4;	     /* slopes, and y-intercepts       */
long x1,x2,x3,x4,y1,y2,y3,y4;	     /* x and y corner values	       */

   buf = (long *)malloc(sizeof(long) * *ns);
   if (buf == NULL)
     {
     c_lasmsg(FALSE,"imgcorner-alloc","Could not allocate appropriate space");
     return(E_FAIL);
     }
     
   c_lasmsg(FALSE,"imgcorner-info","In lse function ");

   for (i = 0; i < MAXBND + 1; i++)
      {
      x[i][1] = 0;
      x[i][2] = *ns;
      x[i][3] = 0;
      x[i][4] = *ns;
      y[i][1] = 0;
      y[i][2] = 0;
      y[i][3] = *nl;
      y[i][4] = *nl;
      }
   for (i = 0; i < nbands[*curimg]; i++, curcat++, (*bndin)++)
      {
      line = 1;

/* Find equation of line for the left edge: m1 = slope, b1= y-intercept
  ----------------------------------------------------------------------*/
      ptcntr=1;
      sumx = 0.0;
      sumxx = 0.0;
      sumxy = 0.0;
      sumy = 0.0;
      n = 0.0;
      curent = 1;
      for (lineno = 1; lineno <= (*nl)/2; lineno++, curent++)
	{
	get_line(bands[*curimg][i], *ns, buf);
        filter2(buf,*ns,lineno,&ptcntr,&sumx,&sumy,&sumxx,&sumxy,&n,value);
	}
      avgy = (sumy/n);
      avgx = (sumx/n);
      m1 = ((n*sumxy-(sumx*sumy))/(n*sumxx-(sumx*sumx)));
      b1= (avgy-m1*avgx);
/* Find equation of line for the bottem edge: m2 = slope, b2 = y-intercept
---------------------------------------------------------------------------*/
      ptcntr=2;
      line = ((85*(*nl))/100);
      curent = line;
      sumx = 0.0;
      sumxx = 0.0;
      sumxy = 0.0;
      sumy = 0.0;
      n = 0.0;
      for(lineno=((85*(*nl))/100); lineno<= *nl; lineno++, curent++)
        {
        get_line(bands[*curimg][i], *ns, buf);
        filter2(buf,*ns,lineno,&ptcntr,&sumx,&sumy,&sumxx,&sumxy,&n,value);
        }
      avgy = (sumy/n);
      avgx = (sumx/n);
      m2 = ((n*sumxy-(sumx*sumy))/(n*sumxx-(sumx*sumx)));
      b2 = (avgy-m2*avgx);
      
/* Find equation of line for the top edge: m3 = slope, b3 = y-intercept 
------------------------------------------------------------------------*/

      ptcntr=3;
      line = 1;
      curent = 1;
      sumx = 0.0;
      sumxx = 0.0;
      sumxy = 0.0;
      sumy = 0.0;
      n = 0.0;
      for(lineno=1; lineno<=((15*(*nl))/100); lineno++,curent++)
        {
        get_line(bands[*curimg][i], *ns, buf);
        filter2(buf,*ns,lineno,&ptcntr,&sumx,&sumy,&sumxx,&sumxy,&n,value);
        }
      avgy = (sumy/n);
      avgx = (sumx/n);
      m3 = ((n*sumxy-(sumx*sumy))/(n*sumxx-(sumx*sumx)));
      b3 = (avgy-m3*avgx);

/* Find equation of line for right edge: m4 = slope, b4 = y-intercept 
-----------------------------------------------------------------------*/

      ptcntr=4;
      line = ((*nl)/2);
      curent = line;
      sumx = 0.0;
      sumxx = 0.0;
      sumxy = 0.0;
      sumy = 0.0;
      n = 0.0;
      for(lineno=((*nl)/2); lineno<= *nl; lineno++,curent++)
        {
        get_line(bands[*curimg][i], *ns, buf);
        filter2(buf,*ns,lineno,&ptcntr,&sumx,&sumy,&sumxx,&sumxy,&n,value);
        }
      avgy = (sumy/n);
      avgx = (sumx/n);
      m4 = ((n*sumxy-(sumx*sumy))/(n*sumxx-(sumx*sumx)));
      b4 = (avgy-m4*avgx);

/* "Shrink" the image dimensions to get rid of "smeared" edge info.
   This is done by adusting the y-intercepts for each line, shifting the line.
------------------------------------------------------------------------------*/
      b1 = b1 + 15;
      b2 = b2 - 15;
      b3 = b3 + 15;
      b4 = b4 - 15;

	fprintf(fptr,"\n\n*******************Band %3d*********************",
                *bndin);

/*Find the intersection points of the lines -- these pts are the corners 
----------------------------------------------------------------------------*/
      x1 = ((b1-b3)/(m3-m1));
      y1 = (m1*x1 + b1);
      if ((x1 < 1) || (x1 > *ns))
          x1 = 1;
      if ((y1 < 1) || (y1 > *nl))
          y1 = 1;
      fprintf(fptr,"\n The upper left corner is: (%ld, %ld) \n",y1,x1);

      x2 = ((b3-b4)/(m4-m3));
      y2 = (m3*x2 + b3);
      if ((x2 > *ns) || (x2 < 1))
         x2 = *ns;
      if ((y2 < 1) || (y2 > *nl))
         y2 = 1;
      fprintf(fptr,"\n The upper right corner is:  (%ld, %ld) \n",y2,x2);

      x3 = ((b1-b2)/(m2-m1));
      y3 = (m2*x3 + b2);
      if ((x3 < 1) || (x3 > *ns))
         x3 = 1;
      if ((y3 > *nl) || (y3 < 1))
         y3 = *nl;
      fprintf(fptr,"\n The lower left corner is:  (%ld, %ld) \n",y3,x3);

      x4 = ((b4-b2)/(m2-m4));
      y4 = (m4*x4 + b4);
      if ((x4 > *ns) || (x4 < 1))
          x4 = *ns;
      if ((y4 > *nl) || (y4 < 1))
          y4 = *nl;
      fprintf(fptr,"\n The lower right corner is:  (%ld, %ld) \n",y4,x4);

/*Save each corner for each band
----------------------------------*/
        x[*bndin][1] = x1;
        y[*bndin][1] = y1;
        x[*bndin][2] = x2;
        y[*bndin][2] = y2;
        x[*bndin][3] = x3;
        y[*bndin][3] = y3;
        x[*bndin][4] = x4;  
        y[*bndin][4] = y4;
      }
  return(E_SUCC);
}
