/****************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize las 
   Get input image parameters
   Get input header name
   Get globals

****************************************************************************/

#include "las.h"
#include "util.h"
#include "fhdr2ddr.h"
#include "protos.h"

void getpar
(
    char lasimg[CMLEN],    /* input image las name                          */
    char hostimg[CMLEN],   /* input image host name                         */
    long wind[4],          /* input image window                            */
    long *nb,              /* number of bands in the image                  */
    char *htype,           /* header type                                   */
    long *hflag           /* create fhdr flag                              */
)


{
   long bands[MAXBND+1];            /* the number of bands in each image    */
   long fatal_err = -1;             /* fatal error flag for c_errmsg        */
   long imgflag[2];                 /* getin image flags                    */
   long numimg;                     /* number of images to get              */
   long one = 1;
   long same_window;                /* flag indicating if the specified images
				       have the same window                 */
   long status;                     /* function return status               */
   long std_out;                    /* logical unit for standard output     */
   long totbnd;                     /* the total number of bands            */
   long zero = 0;
   
   struct VARIABLE *var;            /* pointer to a variable in TAE block   */
   
   /*
     initialize program 
     ------------------ */

   c_inlas (&block, &std_out);
   
   /*
     get and validate input image parameters
     --------------------------------------- */

   status = c_getin (&block, "IN", &zero, lasimg, hostimg, &numimg, wind, 
		   bands, nb, &totbnd, &one, &same_window, imgflag);
   
   if (status != E_SUCC) 
      c_errmsg("Fatal error encountered", "inpeddr-fatal", &fatal_err);
   
   /* subbands, windows, and multiple images are not supported
     --------------------------------------------------------*/
   if ((imgflag[0] != 0) || (imgflag[1] != 0) || (numimg != 1))
     c_errmsg ("Multiple images, sub-bands and windows not allowed",
				"inpeddr-multi", &fatal_err);

/* determine the header type--Currently, the header is assumed to
   be version B. This portion of the code not being deleted to
   allow for other versions in the future. 
--------------------------------------------------------------*/

/*
   var = p_find (&block, "HTYPE");
   strcpy (tmpstr, SVAL(*var, 0));
   *htype = tmpstr[0];
*/
   *htype = 'B';
   
/* Since the header is on disk, the header flag does not apply
   in this application.  hard code to "no copy"
   -----------------------------------------------*/
   
     *hflag = 0;
   
   /*
     Get las globals
     --------------- */
   var = p_find( &block, "$msgtim");
   msgtim = IVAL( *var, 0);
   
}
