#ifndef PROTOS_H
#define PROTOS_H

int datecon
(
    char date[]       /* input  - a date in the form yyyymmdd  (19910101)
		      output - a date in the form dd-mmm-yy (01-jan-91) */
);

void getpar
(
    char lasimg[CMLEN],    /* input image las name                          */
    char hostimg[CMLEN],   /* input image host name                         */
    long wind[4],          /* input image window                            */
    long *nb,              /* number of bands in the image                  */
    char *htype,           /* header type                                   */
    long *hflag           /* create fhdr flag                              */
);

int get_zonenum 
(
    char header[],     /* ptr into the tape's header file for the zone number */
    char proj_type[],  /* string holding the projection type of the image     */
    long *zcode,       /* zone number of the image                  	*/
    double latlong[8], /* array for storing the lat/long for corner points  */
    double *centlat,   /* center latitude				     */
    long *estmzone,	   /* flag for estimating zone			     */
    double *corners,   /* corner for northing and easting computed	     */
    double *corneryx,  /* corner for northing and easting from header
                                                       0:uln, 1:ule
                                                       2:urn, 3:ure
                                                       4:lln, 5:lle
                                                       6:lrn, 6:lre         */
    long datum,		   /* GCTP datum code			  	*/
    long *pcode,	   /* GCTP projection code				*/
    double *projcoef   /* GCTP projection parameters			     */
);

void pcorners
(
    double pixsize,		/* size of image pixel */
    double orientation,		/* Orientation angle of scene (systematic)*/
    double *corners,		/* image data corners-- 0:ul x, 1:ul y
							2:ur x, 3:ur y
							4:ll x, 5:ll y
							6:lr x, 7:lr y    */
    double *corneryx,            /* array for the corner northing and easting  
                                                        0:uln, 1:ule
                                                        2:urn, 3:ure
                                                        4:lln, 5:lle
                                                        6:lrn, 6:lre      */
    long *window		/* image window */
);

int process_hdr
(
    char *hdr,            /* fast format header                               */
    long isize[2],        /* image size (on tape)                             */
    double *pixsize,      /* ground pixel size                                */
    double *orientation,  /* orientation angle of the scene		     */
    double projcoef[15],  /* GCTP projection parameters                       */
    double corners[8],    /* image corner points         0:ul x, 1:ul y
                                                         2:ur x, 3:ur y
                                                         4:ll x, 5:ll y
                                                         6:lr x, 7:lr y       */
    double corneryx[8],   /* array for the corner northing and easting 
                                                         0:uln, 1:ule
                                                         2:urn, 3:ure
                                                         4:lln, 5:lle
                                                         6:lrn, 6:lre      */
    long *pcode,          /* GCTP projection code                             */
    long *zcode,          /* UTM or State Plane code                          */
    long *datum,          /* GCTP datum code                                  */
    char source[32],      /* source of image                                  */
    char inst[32],        /* instrument                                       */
    char date[10]         /* date acquired                                    */
);

void recalc_corners
(
    double corners[8],               /* corner points of the image               */
    long origsize[2],                /* original size of the image NL,NS         */
    long window[4]                  /* image window specified                   */
);

void update_ddr
(
    char hostddr[CMLEN],            /* name of the host ddr to update */
    long window[4],                 /* image window */
    double pixsize,                 /* ground pixel size */
    double proj_coef[15],           /* GCTP projection parameters */
    double corners[8],              /* image projection corner points 
				   (x,y) UL, UR, LL, LR */
    long proj_code,                 /* GCTP projection code */
    long zone_code,                 /* UTM or State Plane code */
    long datum,                     /* GCTP datum code */
    char source[32],                /* source of image */
    char instrument[32],            /* instrument */
    char ac_date[10],               /* acquistion date */
    char ac_time[8]                /* acquistion time */
);

#endif
