/******************************************************************************

NAME: recalc_corners

PURPOSE: Recalculate the corner points for a subwindow of an image.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
       Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
    Reformat image corner points the c_rtcoef wants them.
    Call c_rtcoef to get the projection transformation coefficients.
    Compute the new corner points for that subwindow.

******************************************************************************/
#include "fhdr2ddr.h"
#include "protos.h"

void recalc_corners
(
    double corners[8],      /* corner points of the image               */
    long origsize[2],       /* original size of the image NL,NS         */
    long window[4]          /* image window specified                   */
)
{
   double proj_coords[8];        /* image corner projection coordinates      */
   double coefs[6];              /* transformation coefficients              */

   long fatal = -1;              /* fatal error flag for c_errmsg            */
   long flag = TRUE;             /* c_rtcoef error flag                      */
   long img_coords[8];           /* image corner line,sample coordinates     */
   long line;                    /* current line                             */
   long samp;                    /* current sample                           */
   long status;                  /* function return status                   */
   
   /* Copy the corner points in the order c_rtcoef wants.
      The corners are (x,y) in UL, UR, LL, LR order 
      --------------------------------------------------*/

   proj_coords[0] = corners[1]; proj_coords[1] = corners[0]; /* UL */
   proj_coords[2] = corners[5]; proj_coords[3] = corners[4]; /* LL */
   proj_coords[4] = corners[3]; proj_coords[5] = corners[2]; /* UR */
   proj_coords[6] = corners[7]; proj_coords[7] = corners[6]; /* LR */

   /* set up the image coordinates 
      ---------------------------- */

   img_coords[0] = 1; img_coords[1] = 1;                     /* UL */
   img_coords[2] = origsize[0]; img_coords[3] = 1;           /* LL */
   img_coords[4] = 1; img_coords[5] = origsize[1];           /* UR */
   img_coords[4] = origsize[0]; img_coords[5] = origsize[1]; /* LR */
   
   /* compute the transformation coefficients 
      ----------------------------------------*/

   status = c_rtcoef(proj_coords, img_coords, coefs, &flag);
   if(status == E_FAIL)
     c_errmsg("Fatal error encountered","inpeddr-fatal",&fatal);
   

   /* now compute the new corner points
      ---------------------------------*/

   /* UL */

   line = window[SL];
   samp = window[SS];
   
   corners[0] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
   corners[1] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];
   
   /* UR */

   samp += window[NS] - 1;

   corners[2] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
   corners[3] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];
   
   /* LL */

   line = window[SL] + window[NL] - 1;
   samp = window[SS];
   
   corners[4] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
   corners[5] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];
   
   /* LR */

   samp += window[NS] - 1;

   corners[6] = (coefs[3] * line) + (coefs[4] * samp) + coefs[5];
   corners[7] = (coefs[0] * line) + (coefs[1] * samp) + coefs[2];
}
