/***************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1            7-Dec-82   J. Hwang
   2            3-Jan-89   K. A. Narayanan    conversion to 5.0
   3            5-Apr-89   K. A. Narayanan    added informational message
   5.             May-90   D. VanderZee       fixed numerous bugs
   6.1	       24-May-94   D. Eichacker	      Added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   open input image files
   set up groups for each input image
   open output image file
   set up group for output image
   update output image ddr

***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "insert.h"

void filspec
(
  char hosin[][CMLEN],               /* I: Input image host names */
  char hosout[],                     /* I: Output image host name */
  long *num_inimages,                /* I: Number of input images */
  long num_image_bands[],            /* I: Number of bands in input images */
  long image_band_num[][MAXBND +1],  /* I: Bands numbers in input images */
  long *num_outbands,                /* I: Number of output image bands */
  long intype,                       /* I: Input image data type */
  long wind[][4],                    /* I: Input image window */
  long fill_window[]
)
{
double	line_inc = 1.0,              /* line and sample increments for     */
	samp_inc = 1.0;              /* output image ddr                   */

float	*inbuf;

static long opt[3]={0,0,0};	     /* optional parameter array for EOPENS*/

long	access,                      /* file access type                   */
	bufsiz,                      /* image i/o buffer size              */
	comb_flag = 0,               /* image combination flag             */
	i, img, *ipoint, 
	n, nib, 
	sl,nl,ss,ns,
	stat,                        /* return status                      */
	type;                        /* file type                          */

char	buf[ERRLEN];                 /* error message buffer               */
/*
open input image files
----------------------
*/
access = IREAD;
type = EREAL;
for (i = 0, img = 0; i < 2; i++)
   {
   if (i == 0)
      {
      sl = wind[0][SL] + fill_window[SL] - 1; 
      nl = fill_window[NL];
      ss = wind[0][SS] + fill_window[SS] - 1;
      ns = fill_window[NS];
      ipoint = ipoint1;
      inbuf = inbuf1;
      sprintf (buf, 
          "Window of first input image containing all polygons: %d %d %d %d",
           sl, ss, nl, ns);
      c_errmsg (buf, "insert-wind1", &inf_err);
      }
   else
      {
      sl = wind[1][SL];
      nl = wind[1][NL];
      ss = wind[1][SS];
      ns = wind[1][NS];
      ipoint = ipoint2;
      inbuf = inbuf2;
      }
   opt[0]=0;
   for (n = 0, nib = 0; n < num_inimages[i]; n++, img++)
      {
      stat = c_eopens (&fdesc[n], hosin[img], image_band_num[img], &sl,
                       &ss, &nl, &ns, ipoint+nib, 
                       &access, &type, &zero, opt);
      if (stat != E_SUCC )
         c_errmsg ("Error opening input image", "insert-openin", &fatal_err);

      nib += num_image_bands[img];
      }  
   fdesc[++n] = 0;

/*
   set up group containing input bands
   -----------------------------------
*/
   if ((inbuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
      c_errmsg ("Error allocating dynamic memory", "insert-alloc", &fatal_err);
   c_egroup (fdesc, &gdescin[i], inbuf, &bufsiz);

   if (i == 0) inbuf1 = inbuf;
   else inbuf2 = inbuf;
   }
/*
open output image file
----------------------
*/
access = IWRITE;
opt[0] = intype;
opt[1] = 0;
stat = c_eopens (fdesc, hosout, num_outbands, &one, &one,
               &wind[1][NL], &wind[1][NS], opoint, &access, &type, 
               &zero, opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "insert-openout", &fatal_err);

fdesc[1] = 0;

/*
set up group containing output bands
------------------------------------
*/
if ((outbuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_errmsg ("Error allocating dynamic memory", "film-alloc", &fatal_err);
c_egroup (fdesc, &gdescout, outbuf, &bufsiz);

c_estep (&gdescout);

/*
update output image ddr -----------CHECK------------
-----------------------
*/
c_upddr (&block, hosin, hosout, num_inimages, wind, image_band_num, 
         num_image_bands, &line_inc, &samp_inc, num_outbands, &comb_flag);
}
