/***************************************************************************
NAME: GETPAR

PURPOSE: Retrieves parameter values 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        6-Jan-83   J. Hwang
     2        9-May-85   Y. Sheu 
     3       13-Jan-89   K. Narayanan        Conversion to 5.0   

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
     Initialize the program
     Get parameters
     If second input image is used check for same number of bands and 
        same data type in both images
     Get LAS globals for processing message interval, minmax, and history
     
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "insert.h"

void getpar
(
  char hos_in[][CMLEN],              /* O: Input images host names */
  char las_out[],                    /* O: Output image las name */
  char hos_out[],                    /* O: Output image host name */
  char hos_stat[],                   /* O: Statistics file host name */
  long *num_inbands,                 /* O: Total number of input image bands */
  long num_inimages[2],              /* O: Number of input images */
  long num_image_bands[MAXIMG],      /* O: Number of bands in input images */
  long image_band_num[][MAXBND +1],  /* O: Band numbers in input images */
  long *type,                        /* O: Image data type */
  long wind[][4],                    /* O: Input image windows */
  long in_flag[],                    /* O: Flags for use of images */
  char class[],                      /* O: Class name */
  long *exter_flag,                  /* O: Exterior option */
  long *inter_flag,                  /* O: Interior option */
  long *bound_flag,                  /* O: Boundary option */
  float *exter,                      /* O: Exterior grey value */
  float *inter,                      /* O: Interior grey value */
  float *bound                       /* O: Boundary grey value */
)
{
struct	DDR ddr;
struct	VARIABLE *var;         /* function to find a variable        */

long	bg, eg, ig, pixtype[2];
double  minval, maxval;

long	access,
	flag[MAXIMG][2],             /* flag for subwindows                */
	ftype,
	i, j,
	img,
	imglimit = MAXIMG,          /* maximum number of images           */
	num_bands[2],               /* total number of input image bands*/
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out,                    /* logical unit for standard output   */
	window[MAXIMG][4];          /* image window                       */

char	minmax[4];                  /* minmax option                      */
char	las_in[MAXIMG][CMLEN];	    /* input images las names             */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

in_flag[0] = in_flag[1] = TRUE;
/*
get and validate input images parameters
----------------------------------------
*/
for (i = 0, img = 0; i < 2; i++)
   {   
   stat = c_getin (&block, "IN", &i, las_in[img], hos_in[img], 
                   &num_inimages[i], window, image_band_num[img], 
                   &num_image_bands[img], 
                   &num_bands[i], &imglimit, &same_window, flag[i]);
   if (stat != E_SUCC) 
       c_errmsg ("Error accessing input image", "insert-in",
              &fatal_err);

   if (!same_window)
      c_errmsg ("Same windows must be specified with all images",
                "insert-wind", &fatal_err);

   for (j = 0; j < 4; j++) wind[i][j] = window[0][j];

   c_getddr (hos_in[i], &ddr);
   pixtype[i] = ddr.dtype;

   img += num_inimages[i];
   }

*type = pixtype[0];
c_inmnmx (type, &one, &maxval, &minval);
/*
get and validate output image parameters
----------------------------------------
*/
stat = c_getout (&block, "OUT", las_out, hos_out);
if (stat != E_SUCC)
   c_errmsg ("Error getting output image name", "insert-out", &fatal_err);

/*
get statistics file name
------------------------
*/
access = IREAD;
stat = c_getnam (&block, "INSTAT", &zero, hos_stat, "stat", &access);
if (stat != E_SUCC) 
   c_errmsg ("Error getting statistics file", "insert-sfin", &fatal_err);

strcpy (class, SVAL (*p_find (&block, "CLASSNAM"), 0));
ftype = EREAL;
ig = c_getnum (&block, "INTER", &one, &ftype, inter);
eg = c_getnum (&block, "EXTER", &one, &ftype, exter);
bg = c_getnum (&block, "BOUND", &one, &ftype, bound);

if (ig != 0) 
   {
   if (*inter  <  minval ||  *inter  >  maxval)
      c_errmsg ("INTER value exceeds valid range of image data type", 
                "insert_inter", &fatal_err); 
   if (bg == 0) 
      {
      *bound = *inter;
      bg = 1;
      }
   in_flag[0] = FALSE;
   }
if (eg != 0) 
   {
   if (*exter  <  minval ||  *exter  >  maxval)
      c_errmsg ("EXTER value exceeds valid range of image data type", 
                "insert_exter", &fatal_err); 
   in_flag[1] = FALSE;
   }
else
   {
/*
   check for same number of bands and same data type in both images
   ----------------------------------------------------------------
*/
   if (num_bands[0] != num_bands[1]) 
      c_errmsg ("Input images do not have the same number of bands",
                "insert_bands", &fatal_err); 
   if (pixtype[0] != pixtype[1]) 
      c_errmsg ("Input images are not the same data type", "insert_type",
                &fatal_err); 
   }
if (bg != 0) 
   {
   if (*bound  <  minval ||  *bound  >  maxval)
      c_errmsg ("BOUND value exceeds valid range of image data type", 
                "insert_bound", &fatal_err); 
   }  
/*
if inter, exter, or bound are not used, set flags for call to
polygon 
-------------------------------------------------------------
*/
if (eg == 0) *exter_flag = -1; else *exter_flag = 0;
if (bg == 0) *bound_flag = -2; else *bound_flag = 0;
if (ig == 0) *inter_flag = -2; else *inter_flag = 0;

*num_inbands = num_bands[0];
/*
Get globals
-----------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

}
