/*****************************************************************************
NAME:	INSERT

PURPOSE: Inserts polgonal areas from one image into another.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        6-Jan-83   J. Hwang
     2        9-May-85   Y. Sheu 
     3       13-Jan-89   K. Narayanan        Conversion to 5.0   
     4        5-Apr-89   K. Narayanan        Added informational message
     4.1        Apr-89   D. Hollaren         Removed static to compile on sun
     5.0        May-90   D. VanderZee        Fixed numerous bugs

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get parameters
   Call GETPTS to get polygon coordinates from statistics file
   Call FILSPEC to open input/output images
   Call PROCESS to generate output image
   Call TERMINATE to update ouput image history and terminate program

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "insert.h"

struct  PARBLK  block;                 /* TAE parameter block     */

struct FDESC *fdesc[MAXIMG];                 /* file descriptors        */
struct GDESC *gdescin[2];                    /* group descriptor        */
struct GDESC *gdescout;                      /* group descriptor        */
long    ipoint1[MAXBND],               /* offsets into image i/o  */
                ipoint2[MAXBND],
                opoint[MAXBND];                /* for input and output bands */
long    min_max;                       /* minmax option flag      */
long    msgtim;                        /* message interval        */

float   *inbuf1;                       /* image i/o buffer        */
float   *inbuf2;                       /* image i/o buffer        */
float   *outbuf;                       /* image i/o buffer        */

extern long    curent;                        /* parameters for processing */
extern long    curcat;                        /* message utility         */

void main
(
  void
) 
{
float	bound,                  /* grey level for boundary           */
	exter,                  /* grey level for exterior region    */
	inter;                  /* grey level for interior region    */


long    *ibn_array;
long	*image_band_num[MAXIMG];
                                 /* band numbers in each input image */
long	bound_flag,              /* option for boundary              */
	exter_flag,              /* option for exterior region       */
	fill_window[4],          /* effective polygonal region       */
	in_flag[2],              /* flags for using images           */
	inter_flag,              /* option for interior region       */
	num_image_bands[MAXIMG], /* number of bands in each input image  */
	num_bands,               /* total number of input image bands*/
	num_inimages[2],         /* number of input images           */
	num_polygons,            /* number of polygons               */
	type,                    /* image data type                  */
	wind[2][4];              /* input image windows              */
long	num_points[1000];   /* no. of vertices/polygon          */
long    i,n;

int	*table;    /* polygon vertices                 */

char	class[CMLEN];            /* class name                       */
char	hos_in[MAXIMG][CMLEN];
                                 /* input images host names          */
char	hos_stat[CMLEN];         /* output image host name           */
char	hos_out[CMLEN];          /* output image host name           */
char	las_out[CMLEN];          /* output image las name            */


if ((ibn_array = (long *) malloc(MAXIMG * (MAXBND + 1) * sizeof(long))) == NULL)
      c_errmsg ("Error allocating dynamic memory", "insert-alloc", &fatal_err);

for (i = 0, n = 0; i < MAXIMG; i++, n += (MAXBND + 1)) 
   image_band_num[i] = ibn_array + n; 

/*
get parameters
--------------
*/
getpar (hos_in, las_out, hos_out, hos_stat, &num_bands, num_inimages, 
        num_image_bands, image_band_num, &type, wind, in_flag, class, 
        &exter_flag, &inter_flag, &bound_flag, &exter, &inter, &bound);

/* 
allocate dynamic memory 
*/
if ((table = (int *) malloc(wind[1][NS] * sizeof (int))) == NULL)
      c_errmsg ("Error allocating dynamic memory", "insert-alloc", &fatal_err);

/*
get polygon vertices and "active" fill region
---------------------------------------------
*/
getpts (hos_in, hos_stat, class, &num_polygons, num_points, wind, 
        fill_window, table, in_flag);

/*
open input/output image files
-----------------------------
*/
filspec (hos_in, hos_out, num_inimages, num_image_bands, image_band_num, 
         &num_bands, type, wind, fill_window);

/*
generate output image
---------------------
*/
process (hos_out, fill_window, in_flag, wind, &num_bands, &inter, &exter, 
         &bound, inter_flag, bound_flag, num_polygons, table, num_points);

/*
update output image history and terminate
-----------------------------------------
*/
terminate (las_out, hos_out, hos_in, num_inimages);


free (table);
free (ibn_array);
}
