#ifndef INSERT_H
#define INSERT_H

/* global parameters, and definitions for INSERT    */

extern struct	PARBLK	block;                 /* TAE parameter block     */

extern struct FDESC *fdesc[MAXIMG];             /* file descriptors        */
extern struct GDESC *gdescin[2];                /* group descriptor        */
extern struct GDESC *gdescout;                  /* group descriptor        */
extern long	ipoint1[MAXBND],               /* offsets into image i/o  */
		ipoint2[MAXBND],
		opoint[MAXBND];                /* for input and output bands */
extern long	min_max;                       /* minmax option flag      */
extern long 	msgtim;                        /* message interval        */
 
extern float	*inbuf1;                       /* image i/o buffer        */
extern float	*inbuf2;                       /* image i/o buffer        */
extern float	*outbuf;                       /* image i/o buffer        */
static char    *prog_name = "INSERT";   /* program name            */

extern long	curent;                        /* parameters for processing */
extern long	curcat;                        /* message utility         */

static	long	fatal_err = -1;
static	long	inf_err = 1;

static	long	zero = 0;
static	long	one = 1;


/* Prototypes for the INSERT functions.  This
   file MUST be included after las.h
---------------------------------------------*/

/* filspec.c */
void filspec
(
  char hosin[][CMLEN],               /* I: Input image host names */
  char hosout[],                     /* I: Output image host name */
  long *num_inimages,                /* I: Number of input images */
  long num_image_bands[],            /* I: Number of bands in input images */
  long image_band_num[][MAXBND +1],  /* I: Bands numbers in input images */
  long *num_outbands,                /* I: Number of output image bands */
  long intype,                       /* I: Input image data type */
  long wind[][4],                    /* I: Input image window */
  long fill_window[]
);

/* getpar.c */
void getpar
(
  char hos_in[][CMLEN],              /* O: Input images host names */
  char las_out[],                    /* O: Output image las name */
  char hos_out[],                    /* O: Output image host name */
  char hos_stat[],                   /* O: Statistics file host name */
  long *num_inbands,                 /* O: Total number of input image bands */
  long num_inimages[2],              /* O: Number of input images */
  long num_image_bands[MAXIMG],      /* O: Number of bands in input images */
  long image_band_num[][MAXBND +1],  /* O: Band numbers in input images */
  long *type,                        /* O: Image data type */ 
  long wind[][4],                    /* O: Input image windows */
  long in_flag[],                    /* O: Flags for use of images */
  char class[],                      /* O: Class name */
  long *exter_flag,                  /* O: Exterior option */
  long *inter_flag,                  /* O: Interior option */
  long *bound_flag,                  /* O: Boundary option */
  float	*exter,                      /* O: Exterior grey value */
  float	*inter,                      /* O: Interior grey value */
  float	*bound                       /* O: Boundary grey value */
);

/* getpts.c */
void getpts
(
  char hos_in[][CMLEN],         /* I: Input images host names */
  char hos_sfin[],              /* I: Stats file host name */
  char class[],                 /* I: Class name */
  long *num_polygons,           /* O: Total number of polygons */
  long num_points[],            /* O: No. vertices/polygon */
  long wind[][4],               /* I: Input image windows */
  long fill_window[4],          /* I/O: Effective fill area */
  int	table[],                /* O: Polygon vertices */
  long in_flag[]                /* I: Flags for using input images */
);

/* insert.c */
void main
(
  void
);

/* intsct.c */
void intsct
(
  long a[],                /* O: Input window a, output intersection of two */
  long b[],                /* I: Input window b */
  long abssiz              /* I: True if absolute size comparison desired */
);

/* mark.c */
void mark
(
  long y,                 /* I: Current image line pointer */
  int	table[],          /* I: Sequences of polygon points */
  long npts,              /* I: Number of polygon vertices */
  int	ispt[],           /* O: Buffer to hold mark of intersecion points */
  int	iflag[],          /* I: Flags if mark is from horizontal polygon
                                line segment */
  int	fill[]            /* O: Flags horizontal boundry fill in case the
                                abs. value of slope of polygon line segment
                                is greater than 1. */
);

/* parea.c */
long parea
(
  long in,                /* I: Pixel status in/out */
  long x,                 /* I: X coord of the pixel */
  long y,                 /* I: Y coord of the pixel */
  int	table[],          /* I: Polygon vertices */
  long n,                 /* I: Number of polygons */
  int	isp,              /* I: Number of intersecting line */
  int	next,             /* I: Number of intersecting line segment at
                             next point */
  int	iflag             /* I: Flag indicates that "isp" contains the count
                             of horizontal line segment of polygon */
);

/* polygon.c */
void polygon
(
  long y,                     /* I: Current image line pointer */
  int	table[],              /* I: Sequences of polygon points */
  long nvert,                 /* I: Number of polygon vertices */
  long spix,                  /* I: Starting pixel in the line */
  long np,                    /* I: Number of pixels in the line */
  float	inter,                /* I: Interior value */
  float	bound,                /* I: Boundary value */
  long inter_flag,            /* I: Interior option */
  long bound_flag,            /* I: Boundary option */
  long nbands,                /* I: Number of bands */
  int	fill[],
  int	iflag[], 
  int	ispt[]
);

/* process.c */
void process
(
  char hos_out[],              /* I: Output host file name */
  long fill_window[4],         /* I: Effective polygonal region */
  long in_flag[2],             /* I: Flags for using images */
  long wind[][4],              /* I: Input image windows */
  long *nbands,                /* I: No. bands in the image */
  float	*inter,                /* I: Grey level for interior region */
  float	*exter,                /* I: Grey level for exterior region */
  float	*bound,                /* I: Grey level for boundry */
  long inter_flag,             /* I: Option for interior region */
  long bound_flag,             /* I: Option for boundry */
  long num_polygons,           /* I: No. polygons */
  int	table[],               /* I: Polygon vertices */
  long num_points[]            /* I: No. points/polygon */
);

/* terminate.c */
void terminate
(
  char las_out[], 
  char hos_out[], 
  char hos_in[][CMLEN], 
  long num_inimages[]
);

#endif
