/******************************************************************************
NAME: INTSCT

FUNCTION:
   get the intersection between two windows

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
   1.0        19-feb-82  j. hwang, sasc
   2.0         8-jan-88  m.mazzella SAR/GSFC  convert to c
   3.0         3-jan-89  k. narayanan         copied from classmap and changed
                                              error message to refer to insert
                                              instead of classmap

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):
        if size comparison only then
           take smaller size in both direction
              (vertical,horizontal)
        else
           get intersection between a and b
              and put the result into a
        if no intersection then 
           issue an informational message

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "insert.h"

void intsct
(
  long a[],                /* O: Input window a, output intersection of two */
  long b[],                /* I: Input window b */
  long abssiz              /* I: True if absolute size comparison desired */
)
{
  long k;           /* Loop counter */
  long m[2];        /* Temporary holding variable */

  if (abssiz) 
  {
    if (a[2] > b[2]) 
      a[2] = b[2];
    if (a[3] > b[3]) 
      a[3] = b[3];
  }
  else
  {
    if (!((a[0] > (b[0] + b[2])) || (b[0] > (a[0] + a[2])) || 
          (a[1] > (b[1] + b[3])) || (b[1] > (a[1] + a[3]))))
    {
      for (k = 0; k < 4; ++k)
      {
        if (k <= 1) 
        {
          m[k] = a[k];
          if (a[k] < b[k]) 
            a[k] = b[k];
        }
        else
        {
          if (a[k] + m[k-2] > b[k] + b[k-2]) 
            a[k] = b[k] + b[k-2] - a[k-2];
          else
            a[k] = a[k] + m[k-2] - a[k-2];
         }  /* if */
      } /* for */
      return;
    }
    else
    {
      c_lasmsg(FALSE,"film-info","No intersection between two rectangles");

      /* Make a=b
      -----------*/
      for (k = 0; k < 4; ++k)
        a[k] = b[k];
    }  /* else */
  }  /* if */
}
