/***************************************************************************
NAME:	MARK

PURPOSE: Mark the points that intersect between current image line
         and polygon line segment.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        7-Jan-83   J. Hwang
     2       13-Jan-89   K. Narayanan        Conversion to 5.0   
     5.         May-90   D. VanderZee        fixed numerous bugs

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
      for number of points in a polygon 
         if the polygon line segments intersect with the
         current image line 
            if not horizontal line segment
               get intersection between the polygon line 
               segment and current image line
               if abs(slope of the line segment) < 1
                  get start and end point of horizontal boundary fill
                  mark boundary and boundary fill
               else
                  mark boundary
            else
               if consecutive points are not the same point
                  mark boundary
                  also mark on iflag buffer

****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "insert.h"

void mark
(
  long y,                 /* I: Current image line pointer */
  int table[],          /* I: Sequences of polygon points */
  long npts,              /* I: Number of polygon vertices */
  int ispt[],           /* O: Buffer to hold mark of intersecion points */
  int iflag[],          /* I: Flags if mark is from horizontal polygon
                                line segment */
  int fill[]            /* O: Flags horizontal boundry fill in case the
                                abs. value of slope of polygon line segment
                                is greater than 1. */
)
{
long	i, j, pt1, pt2, str, stp, temp, x;
double	xx, xxx, yoff, yy, yyy, rx;

for (i = 1; i <= npts; i++)
   {
   pt1 = 2*i-1;
   pt2 = 2*i+1;
   if (i == npts) pt2=1;
   if (((table[pt2] >= y) && (table[pt1] <= y)) || 
   ((table[pt2] <= y) && (table[pt1] >= y))) 
      {
      xx = table[pt2-1]-table[pt1-1];
      yy = table[pt2]-table[pt1];
      yoff = y-table[pt1];
      if (yy != 0.)  
         {
/*
         not horizontal line
         -------------------
*/
         rx = xx*yoff/yy + .5 + (double)(table[pt1-1]);
         x = rx;
         xxx = fabs (xx);
         yyy = fabs (yy);
         if (xxx/yyy > 1.) 
            {
            yoff = fabs (yoff);
            str = yoff*xx/(yyy+1)+table[pt1-1]+.5;
            stp = (yoff+1)*xx/(yyy+1)+table[pt1-1]+.5;
            if (str > stp) 
               {
               temp = str; str = stp; stp = temp;
               }
            for (j = str; j <= stp; j++)
               {
               if (j == x) ispt[j-1]++;
               else fill[j-1] = -1;
               }  
            }
         else
            ispt[x-1] = ispt[x-1]+1;
         }
      else
         {
         if (xx != 0.)  
            {
/*
            not same point
            --------------
*/
            str = table[pt1-1]; stp = table[pt2-1];
            if (str > stp) 
               {
               temp = str; str = stp; stp = temp;
               }
            for (j = str; j <= stp; j++)
               {
               ispt[j-1]++;
               if ((j != str) && (j != stp)) iflag[j-1] = 1;
               }  
            }  
         }  
      }  
   }  
}

