/****************************************************************************
NAME:	PAREA

PURPOSE: Determine regional status(interior or exterior) of a
         current point in an image line.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1        7-Jan-83   J. Hwang
     2       13-Jan-89   K. Narayanan        Conversion to 5.0   

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
     if first pixel of line initialize counts
     if the current point is not an element of a horizontal
     line segment of the polygon
        if not boundary clear counters
        else
           if the current point is an extreme point of
           the polygon
              for number of points in the polygon
                 set table pointers
                 if polygon point is same as the current point
                    if line segment starting from the current
                       point ends at above of current image line
                          increment up by one
                    else if line segment starting from the current
                       point ends at below of current image line
                       increment down by one
           accumulate the count for intersecting line segments
           take minimum of up and down
           add minimum of up and down to accumulated count of
           intersecting line segments.
           if above result is odd number set pixel status to inside

***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "insert.h"

long parea
(
  long in,                /* I: Pixel status in/out */
  long x,                 /* I: X coord of the pixel */
  long y,                 /* I: Y coord of the pixel */
  int table[],          /* I: Polygon vertices */
  long n,                 /* I: Number of polygons */
  int isp,              /* I: Number of intersecting line */
  int next,             /* I: Number of intersecting line segment at
                             next point */
  int iflag             /* I: Flag indicates that "isp" contains the count
                             of horizontal line segment of polygon */
)
{
long	both,
	crr, 
	i, 
	new, 
	old, 
	status;              /* pixel status inside/outside       */ 

static	long	down,        /* count for the polygon line        */
                             /* segments that start at the current*/
                             /* point and end at the region below */
                             /* the current image line.           */

	point, 
	up;                  /* count for the polygon line        */
                             /* segments that start at the current*/
                             /* point and end at the region above */
                             /* the current image line            */

	

status = in;
if (x == 1) 
   {
   point = 0;
   up = 0;
   down = 0;
   }  
if (iflag != 1) 
   {
   if (isp <= 0) 
      {
      up = 0;
      down = 0;
      point = 0;
      }
   else
      {
      if (isp > 1) 
         {
         i = 0;
         while (i < n)
            {
            i++;
            crr = 2*i-1;
            old = 2*i-3;
            new = 2*i+1;
            if (i == 1) old = 2*n-1;
            if (i == n) new = 1;
            if ((table[crr-1] == x) && (table[crr] == y)) 
               {
               if (y < table[old]) down++;
               if (y < table[new]) down++;
               if (y > table[old]) up++;
               if (y > table[new]) up++;
               }  
            }  
         }  
      point += isp;
      if (next == 0) 
         {
         both = up < down ? up : down;
         if ((((both+point) >> 1) << 1) != (both+point)) status = !in;
         }  
      }  
   }  
return (status);
}
