/*****************************************************************************
NAME:	POLYGON

PURPOSE: Determine if the pixel is inside or outside the polygons

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
     1       15-Dec-82   J. Hwang
     2       22-Jan-85   F. Irani
     3       13-Jan-89   K. Narayanan        Conversion to 5.0   
     3.1        Apr-89   D. Hollaren         Added static to compile on PN9050
     5.         May-90   D. VanderZee        Fixed numerous bugs

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
       call MARK to mark the intersecting points between
       current image line and the polygon line segments
       if no mark and no action on exterior then return
       for number of pixels in an image line
          determine the regional status of current point
          (interior or exterior) by calling function PAREA.
          if boundary
             clear boundary mark and flags for next process
             if user requested specific pixel value for
                boundary then
                set buffer to the user supplied pixel value
             else if copy from ibuf desired then
                copy from ibuf to outbuf
          if interior
             same as boundary except clearing boundary mark.
          else (exterior)
             if user requested specific pixel value to be
                set for exterior then
                set exterior to specific pixel value supplied
             else
                copy from ibuf to outbuf

****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "insert.h"

void polygon
(
  long y,                     /* I: Current image line pointer */
  int table[],              /* I: Sequences of polygon points */
  long nvert,                 /* I: Number of polygon vertices */
  long spix,                  /* I: Starting pixel in the line */
  long np,                    /* I: Number of pixels in the line */
  float inter,                /* I: Interior value */
  float bound,                /* I: Boundary value */
  long inter_flag,            /* I: Interior option */
  long bound_flag,            /* I: Boundary option */
  long nbands,                /* I: Number of bands */
  int fill[],
  int iflag[],
  int ispt[]
)
{
long	i, 
	inside_flag,              /* status of previous pixel        */
	nb;  

for (i = 0; i < np; i++) fill[i] = iflag[i] = ispt[i] = 0;

inside_flag = FALSE;

/*
mark intersections with current line
------------------------------------
*/
mark (y, table, nvert, ispt, iflag, fill);
/*
examine each pixel whether it is interior or exterior
and assign values accordingly
-----------------------------------------------------
*/
for (i = 1; i <= np; i++)
   {
   inside_flag = parea (inside_flag, i, y, table, nvert, ispt[i-1], ispt[i], 
                        iflag[i-1]);

   if ((ispt[i-1] != 0) || (fill[i-1] == -1))  
      {
/* 
      boundary pixel
      --------------
*/
      ispt[i-1] = 0;
      if (iflag[i-1] != 0) iflag[i-1] = 0;
      if (fill[i-1] != 0) fill[i-1] = 0;
      for (nb = 0; nb < nbands; nb++)
         {
         if (bound_flag >= 0) outbuf[opoint[nb]+i+spix-2] = bound;
         else if (bound_flag <= -2) 
            outbuf[opoint[nb]+i+spix-2] = inbuf1[ipoint1[nb]+i-1];
         }
      }
   else if (inside_flag) 
      {
/* 
      interior pixel
      --------------
*/
      for (nb = 0; nb < nbands; nb++)
         {
         if (inter_flag >= 0) outbuf[opoint[nb]+i+spix-2] = inter;
         else if (inter_flag <= -2) 
            outbuf[opoint[nb]+i+spix-2] = inbuf1[ipoint1[nb]+i-1];
         }
      }  
   }  
}

