/*****************************************************************************
*
*  NAME:  CRTTBL
*
*  FUNCTION: Create the translation table and the Labeled Table
*
*  EXTERNAL REFERENCES:
*	FILE1 - Sorted permutation file
*	FILE3 - Sorted translation file
*	FILE4 - Unsorted translation file
*	
*  PROGRAM HISTORY:
*  Version	Date	Programmer	Code	Request
*  -------	----	----------	----	-------
*   1.0		2/84	K.A.Narayanan	Goddard	
*   4.0	       10/87	D. Akkerman	CSB	PRF #4329
*   5.0	       10/89	D. Etrheim	CSB	LAS 5.0 Conversion
*   5.2         6/90	D. Etrheim	CSB	ADP #5196 Increase performance
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
* 	Must be run under TAE
*
*  PROJECT:  LAS
*
*  ALGORITHM DESCRIPTION:
*	Open the sorted permutation file.  If the translation cannot be held
*  in memory, open the translation file.  Read in a record.  If the attributes
*  do not match the attributes of the next record this is a unique permutation;
*  write the values to the translation table and the LT.  Otherwise, duplicate
*  permutations exist; increment the permutation count and update the 
*  translation table.  Write the last record to the LT and the translation 
*  table.  Sort the translation file.  If the output image is not requested,
*  do not create the translation table.
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltable.h"
#include "util.h"
#include "intersect.h"
#include "protos.h"

int crttbl
(
    struct TAB_DEF *tab,		/* labeled table descriptor	*/
    long *trntbl,			/* translation table		*/
    long *npum,			/* number of permutations	*/
    long *totbnds,			/* total number of bands	*/
    long *maxout,			/* maximum number of permutations 
				   after removing duplicates	*/
    char *file1,			/* sorted permutation file name	*/
    long *imgflg,			/* flag if creating an output image */
    long *addoff,			/* offset to be subtracted	*/
    char *file3			/* sorted permutation file name	*/
)

{
FILE *fp;			/* file descriptor		*/
FILE *fp2;			/* file descriptor		*/
#ifdef vms
struct DESC infile;
struct DESC outfile;
long ncol;
#endif

long i,j,x,			/* loop counters		*/
     nexval,			/* next permutation value	*/
     nexcnt,			/* next permutation count	*/
     nexatt[MAXBND],		/* next permutation list	*/
     next,			/* FALSE if different permutations exist
				   for the same attribute lists */
     non_fatal = 1,		/* for nonfatal error messages  */
     oldatt[MAXBND],		/* current permutation list	*/
     oldval, 			/* current permutation count	*/
     prmcnt,			/* permutation count totals	*/
     result;			/* return values of system sorts */
long nbytes; 			/* number of bytes to read	*/

char file4[20];         	/* temp sort files		*/
char ctemp[3000];		/* temp string for fgets	*/
char command[CMLEN],            /* command for sort             */
     /**datapath, */          /* environment variable DATAPATH       */
     *getenv();

strcpy(file3,"sort2");
c_creat_name(file3,"out");
strcpy(file4,"sort2");
c_creat_name(file4,"dat");

/* Give an informational message; depending on the number 
of permutations this could take awhile
----------------------------------------------------------*/
c_errmsg("Creating the labeled table","intersect-create",&non_fatal);

/* Open the sorted permutation file and read the first record
-------------------------------------------------------------*/
if ((fp = fopen(file1,"r")) == NULL)
     {
     c_errmsg("Error opening temporary file","intersect-open",&non_fatal);
     c_delfil(file1);
     return(E_FAIL);
     }

/*fscanf(fp,"%d %d",&oldval,&prmcnt);
for (i = 0; i < *totbnds; i++)
   fscanf(fp,"%d",&oldatt[i]);*/
/* One read for blank line 
--------------------------*/
fgets(ctemp,5,fp);
nbytes = (*totbnds * 11) + 24;
fgets(ctemp,nbytes,fp);
oldval = atol(ctemp);
prmcnt = atol(&(ctemp[11]));
for (i = 0; i < *totbnds; i++)
   oldatt[i] = atol(&(ctemp[(22 + (i * 11))]));

/* If creating an output image, create a translation table also.  If all 
the permutations cannot be read into memory, create the translation table 
as a file.
-------------------------------------------------------------------------*/
if (*imgflg)
   {
   if (*npum > MAXTBL)
      {
      /* The permutation table will be a file
      ---------------------------------------*/
      if ((fp2 = fopen(file4,"w")) == NULL)
           {
           c_errmsg("Error opening temporary file","intersect-open",&non_fatal);
           fclose(fp);
           c_delfil(file1);
           return(E_FAIL);
           }
      /* Read in the next record
      --------------------------*/
      for (j = 1; j < *npum; j++)
         {
         next = FALSE;
         /*fscanf(fp,"%d %d",&nexval,&nexcnt);
         for (i = 0; i< *totbnds; i++)
            fscanf(fp,"%d",&nexatt[i]);*/
         fgets(ctemp,nbytes,fp);
         nexval = atol(ctemp);
         nexcnt = atol(&(ctemp[11]));
         for (i = 0; i < *totbnds; i++)
            nexatt[i] = atol(&(ctemp[(22 + (i * 11))]));
         for (i = 0; i < *totbnds; i++)
            { 
            /* If the attributes don't match, this is a unique permutation
            --------------------------------------------------------------*/
            if (nexatt[i] != oldatt[i])
               {
               fprintf(fp2,"%11d%11d\n",oldval,*maxout); 
               writelt(tab,maxout,&prmcnt,oldatt,totbnds,addoff);
               oldval = nexval;
               *maxout += 1;
               prmcnt = nexcnt;
               for (x = 0; x < *totbnds; x++)
                  oldatt[x] = nexatt[x];
               next = TRUE;
               break;
               }
            } 

         /* The same attribute list is using two permutations
         ---------------------------------------------------*/
         if (!next)
            {
            prmcnt += nexcnt;
            fprintf(fp2,"%11d%11d\n",nexval,*maxout); 
            }
         }
 
         /* Write the last values to the LT and the translation file
         -----------------------------------------------------------*/
         fprintf(fp2,"%11d%11d\n",oldval,*maxout); 
         fclose(fp2);
         writelt(tab,maxout,&prmcnt,oldatt,totbnds,addoff);

         /* Sort the translation file
         ----------------------------*/
#ifdef unix
         strcpy(command,"sort -n");
         strcat(command,file4);
         strcat(command," -o ");
         strcat(command,file3);
         /*datapath = (char *)getenv(DATAPATH);
         if (datapath != NULL)
           {
           strcat(command," -T ");
           strcat(command,datapath);
           }*/
         result = system(command);
         c_delfil(file4);
         if (result != E_SUCC) 
            {
            c_errmsg("System file sort failed","intersect-system",&non_fatal);
            fclose(fp);
            c_delfil(file1);
            c_delfil(file3);
            return (E_FAIL);
            }
#endif
#ifdef vms
	 ncol = *totbnds;
         infile.addr = file4;
         outfile.addr = file3;
         infile.length = strlen(file4);
         outfile.length = strlen(file3);
         c_errmsg("Sorting the permutation file","intersect-sort",&non_fatal);
         prmsrt (&infile,&outfile,&ncol,&result);
         c_delfil(file4);
         if (result != 1)
            {
            c_errmsg("System file sort failed","intersect-system",&non_fatal);
            fclose(fp);
            c_delfil(file1);
            c_delfil(file3);
            return (E_FAIL);
            }
#endif
      }

   /* The translation table will be contained in memory
   ----------------------------------------------------*/
   else
      { 
      trntbl[0] = 0;

      /* Read in the next record
      --------------------------*/
      for (j = 1; j < *npum; j++)
         {
         next = FALSE;
         /*fscanf(fp,"%d %d",&nexval,&nexcnt);
         for (i = 0; i< *totbnds; i++)
            fscanf(fp,"%d",&nexatt[i]);*/
         fgets(ctemp,nbytes,fp);
         nexval = atol(ctemp);
         nexcnt = atol(&(ctemp[11]));
         for (i = 0; i < *totbnds; i++)
            nexatt[i] = atol(&(ctemp[(22 + (i * 11))]));
         for (i = 0; i < *totbnds; i++)
            { 
            /* If the attributes don't match, this is a unique permutation
            --------------------------------------------------------------*/
            if (nexatt[i] != oldatt[i])
               {
               trntbl[oldval] = *maxout;
               writelt(tab,maxout,&prmcnt,oldatt,totbnds,addoff);
               oldval = nexval;
               *maxout += 1;
               prmcnt = nexcnt;
               for (x = 0; x < *totbnds; x++)
                  oldatt[x] = nexatt[x];
               next = TRUE;
               break;
               }
            } 

         /* The same attribute list is using two permutations
         ---------------------------------------------------*/
         if (!next)
            {
            prmcnt += nexcnt;
            trntbl[nexval] = *maxout;
            }
         }

      /* Write the last values to the LT and the translation table 
      ------------------------------------------------------------*/
      trntbl[oldval] = *maxout;
      writelt(tab,maxout,&prmcnt,oldatt,totbnds,addoff);
      }
   }

/* No output image is being created so just build the Labeled Table.
-------------------------------------------------------------------*/
else
   {
   /* Read in the next record
   --------------------------*/
   for (j = 1; j < *npum; j++)
      {
      next = FALSE;
      /*fscanf(fp,"%d %d",&nexval,&nexcnt);
      for (i = 0; i< *totbnds; i++)
         fscanf(fp,"%d",&nexatt[i]);*/
      fgets(ctemp,nbytes,fp);
      nexval = atol(ctemp);
      nexcnt = atol(&(ctemp[11]));
      for (i = 0; i < *totbnds; i++)
          nexatt[i] = atol(&(ctemp[(22 + (i * 11))]));
      for (i = 0; i < *totbnds; i++)
         { 
         /* If the attributes don't match, this is a unique permutation
         --------------------------------------------------------------*/
         if (nexatt[i] != oldatt[i])
            {
            writelt(tab,maxout,&prmcnt,oldatt,totbnds,addoff);
            oldval = nexval;
            *maxout += 1;
            prmcnt = nexcnt;
            for (x = 0; x < *totbnds; x++)
               oldatt[x] = nexatt[x];
            next = TRUE;
            break;
            }
         } 

      /* The same attribute list is using two permutations
      ---------------------------------------------------*/
      if (!next)
         prmcnt += nexcnt;
      }

   /* Write the last values to the LT 
   ----------------------------------*/
   writelt(tab,maxout,&prmcnt,oldatt,totbnds,addoff);
   }

/* Close and delete the sorted permutation file
-----------------------------------------------*/
fclose(fp);
c_delfil(file1);

return(E_SUCC);
}
