/*****************************************************************************
*
*   NAME: GETRUN 
*
*   FUNCTION:  Construct a run length code table for this sector of each input 
*              image band.
*
*   PROGRAM HISTORY:
*   Version     Date    Programmer      Code    Request
*   -------     ----    ----------      ----    -------
*     1.0       2/84    K.A.Narayanan   Goddard
*     4.0      10/87    D. Akkerman     CSB     PRF #4329
*     5.0      10/89    D. Etrheim      CSB     LAS 5.0 conversion
*     5.2      06/90    D. Etrheim      CSB     ADP #5196 Increase performance
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*   For each band of the input images
*       Initialize the first run length code and value for this band sector
*       For each consecutive pair of pixels in this band sector
*          If the current pixel value matches the previous pixel value then 
*             Increment the length of the current run length code
*          Else
*             Initialize the next run length code and value for this band sector
*
******************************************************************************/
#include "las.h"
#include "protos.h"

void getrun
(
    long *nb, 		/* total no. of bands to be processed */
    long *bufndx,		/* Image buffer index to the current sample */
    long *sectsz, 		/* Size of the current line sector */
    long (*runcnt)[MAXBND],	/* Run length code count table */
    long (*runval)[MAXBND], /* Run length code value table */
    long *thebuf		/* Image I/O buffer */
)
{
/* The following variables have been declared global for image I/O routines
--------------------------------------------------------------------------*/
extern long offset1[MAXIMG*MAXBND];/* offsets for input bands		*/

long runptr;		/* Pointer to next run length code */
long band,i;		/* Loop counters */ 

for (band = 0; band < *nb; band++)
    {
    /* Initialize the first run length code for this band sector
    -----------------------------------------------------------*/
    runptr = 0;
    runcnt[runptr][band] = 1;
    runval[runptr][band] = thebuf[offset1[band] + *bufndx];
    for (i = *bufndx + 1; i < *bufndx + *sectsz; i++)
        {
        if (thebuf[offset1[band] + i] == thebuf[offset1[band] + (i - 1)])
           runcnt[runptr][band] += 1;
        else
           {
           /* Initialize the next run length code for this band sector
           -----------------------------------------------------------*/
           runptr += 1;
           runcnt[runptr][band] = 1;
           runval[runptr][band] = thebuf[offset1[band] + i];
           }
        }
    }
}
