/******************************************************************************
*
*  NAME:	INITLT
*
*  FUNCTION: Open the Labeled table and write the vector records.
*
*  PROGRAM HISTORY:
*  Version	Date	Author		CODE	Request
*  -------	----	------ 		----	-------
*    5.0        10/89	D. Etrheim	CSB	LAS 5.0 Conversion
*    5.1        05/90	D. Etrheim	CSB	ADP #5506  Allow different
*					sl,ss for each image band.  ADP #5196
*					Add LTTYPE and ATRNAME parameters.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
*		Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*	Initialize variables for opening the LT.  Write attribute names
*  for each band, appending the band number to the image name for the 
*  attribute description.  If masking was requested, write the first data
*  record.
*
******************************************************************************/
#include <string.h>
#include "las.h"
#include "ltable.h"
#include "protos.h"

void initlt
(
    struct TAB_DEF *tab,		/* Labeled table descriptor */
    char hosin[][CMLEN],		/* host input image names */
    char *lthos,			/* labeled table host file name */
    long (*bands)[MAXBND + 1],	/* bands to be processed for all images */
    long *nbands,			/* nbands for each input image  */
    long *totbnds,			/* total number of input bands 	*/
    long *mskflg,			/* masking flag			*/
    long *mskcnt,			/* mask count			*/
    char atrnam[][10],		/* attribute names		*/
    char *lttype			/* labeled tabel type		*/
)

{
struct MATRIX size;		/*  size of output matrix		*/
struct VECTOR *ptr;		/* pointer to current vector		*/

char buf1[CMLEN],		/* character buffer for image name	*/
     buf2[7];			/* character buffer for band number	*/

long bnd = 0,			/* current band				*/
     i,				/* loop counter				*/
     image = 0,			/* current image			*/
     zero = 0;			/* constant of zero			*/

/*  Initialize variables needed to open the output labeled table file.
----------------------------------------------------------------------*/
tab->nsubs = 0;
tab->subfile[0] = "\0";
tab->ftype = lttype;
tab->fdesc = "File generated from INTERSECT";
size.nrow = 1;
size.ncol = 1;

/*  Open the output labeled table file
--------------------------------------*/
open_tab(tab,lthos,IWRITE);

/*  Define each vector fields
----------------------------*/
put_vector(tab,"PERMNUM",I4,"INTERSECT permutation code",1,1,FIELD_SEP2);
put_vector(tab,"NPIXELS",I4,"Number of pixels",1,1,FIELD_SEP2);

/* For each attribute description, appended the band number to the 
image name for clarity.
------------------------------------------------------------------*/
for (i = 0; i < *totbnds; i++)
   {
   if (nbands[image] > 1)
      {
      sprintf(buf2,"(:%d)",bands[image][bnd]);
      strcpy(buf1,hosin[image]);
      strcat(buf1,buf2);
      }
   else
      strcpy(buf1,hosin[image]);

   if (i != (*totbnds - 1))
      put_vector(tab,atrnam[i],I4,buf1,1,1,FIELD_SEP2);
   else
      put_vector(tab,atrnam[i],I4,buf1,1,1,RECORD_SEP);

   bnd += 1;
   if (bnd == nbands[image])
      {
      bnd = 0;
      image += 1;
      }
   }

/* If masking was requested, write the first data record
--------------------------------------------------------*/
if (*mskflg)
   {
   ptr = tab->vector;
   put_matrix(ptr,I4,&size,&zero);
   ptr = ptr->next;
   put_matrix(ptr,I4,&size,mskcnt);
   ptr = ptr->next;

   size.nrow = 0;
   size.ncol = 0;
   for (i = 0; i < *totbnds; i++)
      {
      put_matrix(ptr,I4,&size,&zero);
      ptr = ptr->next;
      }
   put_record(tab);
   }

return;
} 
