#ifndef PROTOS_H
#define PROTOS_H

#define TBLSIZ 1024     /* Size of permutation table    */
#include "las.h"
#include "ltable.h"

int crttbl
(
    struct TAB_DEF *tab,		/* labeled table descriptor	*/
    long *trntbl,			/* translation table		*/
    long *npum,			/* number of permutations	*/
    long *totbnds,			/* total number of bands	*/
    long *maxout,			/* maximum number of permutations 
				   after removing duplicates	*/
    char *file1,			/* sorted permutation file name	*/
    long *imgflg,			/* flag if creating an output image */
    long *addoff,			/* offset to be subtracted	*/
    char *file3			/* sorted permutation file name	*/
);

int getpar
(
    struct PARBLK *vblock,		/* TAE parameter block		*/
    char hosin[][CMLEN],		/* Input host file names	*/
    char hosout[][CMLEN],			/* Output host file image name 	*/
    char *lthos,			/* Output LT host file name	*/
    long *nbands,			/* number of bands		*/
    long (*wind)[4],		/* window specification	for each image */
    long *mask,			/* mask values			*/
    long (*bands)[MAXBND+1],	/* band specifications for each input image */
    long *totbnds,			/* total number of bands	*/
    long *nmrimg,			/* number of images being processed */
    long *mskflg,			/* masking flag			*/
    long *imgflg,			/* FASLE if no output image is requested */
    long *msgtim,			/* message timing 		*/
    char atrnam[][10],		/* attribute names		*/
    char *lttype,			/* labeled tabel type		*/
    long *dataflag			 /* data type flag               */
);

void intersect_getprnm
(
    long *forlnk,			/* Permutation record forward link */
    long *bcklnk, 			/* Permutation record previous link */
    FILE *fp,			/* file descriptor to permutation file */
    long *totbnds, 			/* total no. of bands to be processed */
    long (*attr)[TBLSIZ + 1],	/* Attributes defining the permutation */
    long *pcount,			/* Permutation occurrence counter */
    long *prmnum,			/* List of permutation numbers */
    long *curatt,			/* Attributes defining the current permutation*/
    long *mincnt,			/* Shortest current run length code */
    long *prmval,			/* Permutation value */
    long *next,			/* Next permutation number */
    long *addoff			/* offset to be added */
);

void getrun
(
    long *nb, 		/* total no. of bands to be processed */
    long *bufndx,		/* Image buffer index to the current sample */
    long *sectsz, 		/* Size of the current line sector */
    long (*runcnt)[MAXBND],	/* Run length code count table */
    long (*runval)[MAXBND], /* Run length code value table */
    long *thebuf		/* Image I/O buffer */
);

void initlt
(
    struct TAB_DEF *tab,		/* Labeled table descriptor */
    char hosin[][CMLEN],		/* host input image names */
    char *lthos,			/* labeled table host file name */
    long (*bands)[MAXBND + 1],	/* bands to be processed for all images */
    long *nbands,			/* nbands for each input image  */
    long *totbnds,			/* total number of input bands 	*/
    long *mskflg,			/* masking flag			*/
    long *mskcnt,			/* mask count			*/
    char atrnam[][10],		/* attribute names		*/
    char *lttype			/* labeled tabel type		*/
);

int opnlas
(
    char hosin[][CMLEN],		/* input host file names	*/
    char *hosout,			/* output host file name	*/
    long (*wind)[4],		/* window specifications for each input image */
    long (*bands)[MAXBND + 1],	/* bands to be processed for all images */
    long *nbands,			/* number of bands for input images*/
    long *nmrimg,			/* number of input images	*/
    long *imgflg,			/* flag if creating an output image */
    char *hostmp,			/* temp host file name		*/
    long *pass,			/* flag to determine which pass	*/
    long *maxout,			/* maximum output permutation value */
    long *nl,			/* number of lines		*/
    long *ns,			/* number of samples		*/
    long *addoff,			/* offset to be added		*/
    long *range,			/* TRUE if unable to handle the data range */
    long *dataflag		        /* data flag                    */
);

void ovrlay
(
    struct PARBLK *vblock,		/* TAE paramater block		*/
    long *totbnds,			/* total no. of bands to be processed */
    long *mskflg,			/* masking flag			*/
    long *msgtim,			/* for message display		*/
    long *nl,			/* number of lines		*/
    long *ns,			/* number of samples		*/
    long *thebuf,			/* Image I/O buffer		*/
    long *mask,			/* mask values 			*/
    long *npum,			/* number of permutations	*/
    long *imgflg,			/* flag if image is to be created */
    long *mskcnt,			/* number of mask values	*/
    char hostmp[][CMLEN],			/* temp host file name		*/
    long *addoff, 			/* offset to be added */
    char *selfile                  /* sort file name               */
);

void proces
(
    struct PARBLK *vblock,		/* TAE parameter block		*/
    char hosin[][CMLEN],		/* input host file names	*/
    char *hosout,			/* output host file name	*/
    char *lthos,			/* output LT host file name	*/
    long (*wind)[4],		/* window specifications for each input image */
    long (*bands)[MAXBND + 1],	/* bands to be processed for all images */
    long *nbands,			/* number of bands for input images*/
    long *totbnds,			/* total no. of bands to be processed */
    long *nmrimg,			/* number of input images	*/
    long *mask,			/* mask values for each band	*/
    long *mskflg,			/* masking flag			*/
    long *imgflg,			/* FALSE if no output image	*/
    long *msgtim,			/* for message display		*/
    char atrnam[][10],		/* attribute names		*/
    char *lttype,			/* labeled tabel type		*/
    long *dataflag                 /* datatype flag                */
);

int sortbl
(
    long *npum,			/* number of permutations	*/
    long *totbnds,			/* total number of bands	*/
    char *file1,			/* output sorted permutation file */
    long *range,			/* TRUE if unable to handle the data range */
    char *selfile			/* temp unsorted permutation files */
);

int trnimg
(
    long *nl,			/* number of lines		*/
    long *ns,			/* number of samples		*/
    long *thebuf,			/* image I/O buffer		*/
    long *npum,			/* number of permutations	*/
    long *trntbl,			/* translation table		*/
    long *msgtim,
    char *file3                    /* sorted file                  */
);

void writelt
(
    struct TAB_DEF *tab,		/* output labeled table descriptor */
    long *maxout,			/* maximum output permutation thus far */
    long *prmcnt,			/* permutation count			*/
    long *oldatt,			/* attribute list			*/
    long *totbnds,			/* total number of bands		*/
    long *addoff			/* offset to be subtracted		*/
);

#endif
