/*****************************************************************************
*
*  NAME:  SORTBL 
*
*  FUNCTION:  Sort the permutation table
*
*  EXTERNAL REFERENCES:
* 	FILE1 -  Output sorted permutation file
*       FILE2 -  Unsorted permutation file(s)
*
*  PROGRAM HISTORY:
*  Version	Date	Programmer	Code	Request
*  -------	----	----------	----	-------
*    1.0	2/84	K.A.Narayanan	Goddard	
*    4.0       10/87	D. Akkerman	CSB	PRF #4329
*    5.0       10/89	D. Etrheim	CSB	LAS 5.0 conversion
*    5.2       06/90	D. Etrheim	CSB	ADP #5196 Increase performance
*    5.3       10/91    T. Mittan       CSB     Time stamped temporay file
*						and allowed datapath for sort.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*	If the number of image bands used to create the permutations
*  is small enough to allow a system sort, do so.  Otherwise, do a bubble
*  sort on the file. Read in a record and compare with the following record.
*  Write the smallest record to the output file. Stop when no changes were made.
*
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "las.h"
#include "intersect.h"
#include "desc.h"
#include "util.h"
#include "protos.h"

int sortbl
(
    long *npum,			/* number of permutations	*/
    long *totbnds,			/* total number of bands	*/
    char *file1,			/* output sorted permutation file */
    long *range,			/* TRUE if unable to handle the data range */
    char *selfile			/* temp unsorted permutation files */
)

{
FILE *fp;			/* file descriptor		*/
FILE *fp2;			/* file descriptor		*/
#ifdef vms
struct DESC infile;
struct DESC outfile;
long ncol;
#endif

long change,			/* flag if next values written to the file*/
     i,j,x,y,			/* loop counters		*/
     newval,			/* next permutation value	*/
     newcnt,			/* next permutation count	*/
     newatt[MAXBND],		/* next permutation list	*/
     non_fatal = 1,		/* for nonfatal error messages  */
     oldcnt,			/* current permutation value	*/
     oldatt[MAXBND],		/* current permutation list	*/
     oldval, 			/* current permutation count	*/
     result,			/* result of system sorts	*/
     write;			/* flag if current values written to the file*/
long nbytes;			/* number of bytes to read for fgets */

char command[CMLEN],		/* command string for unix system sort */ 
     /**datapath,*/		 	/* environment variable DATAPATH       */ 
     file2[100],		/* temp unsorted permutation files */
     tbuf[5];			/* temp buffer to build command string*/
char *getenv();

char ctemp[3000];		/* temp string for fgets	*/

/* Sort the file.  Use the system sorts if the number of images bands
is less then the number of bands that the system sort will handle. Give 
an informational message as system sorts may take awhile.  If range = TRUE,
the negative data values still exist in the permutation file.  The VMS
system sort will not handle negative data values when represented as
characters; so use the bubble sort instead. 
-------------------------------------------------------------------------*/
strcpy(file2,selfile);
strcpy(file1,"intersect");
c_creat_name(file1,"txt");

#ifdef vms
if ((*totbnds < MXSRT) && !*range) 
   {
   ncol = *totbnds;
   infile.addr = file2;
   outfile.addr = file1;
   infile.length = strlen(file2);
   outfile.length = strlen(file1);
   c_errmsg("Sorting the permutation file","intersect-sort",&non_fatal);
   prmsrt (&infile,&outfile,&ncol,&result);
   c_delfil(file2);
   if (result != 1)
      {
      c_errmsg("System file sort failed","intersect-system",&non_fatal);
      c_delfil(file1);
      return (E_FAIL);
      }
   }
#endif
#ifdef unix
/* Create the command string
----------------------------*/
if (*totbnds < MXSRT)
   {
   strcpy(command,"sort -n");
   for (i = 2; i < *totbnds + 2; i++)
      {
      sprintf(tbuf," +%d",i);
      strcat(command,tbuf);
      }
   strcat(command," ");
   strcat(command,file2);
   strcat(command," -o ");
   strcat(command,file1);
/*if ((datapath = (char *)getenv(DATAPATH)))
   {
   strcat(command," -T ");
   strcat(command,datapath);
   }*/
   c_errmsg("Sorting the permutation file","intersect-sort",&non_fatal);
   result = system(command);
   c_delfil(file2);
   if (result != E_SUCC)
      {
      c_errmsg("System file sort failed","intersect-system",&non_fatal);
      c_delfil(file1);
      return(E_FAIL);
      }
   }
#endif
else
   {
   c_errmsg("Sorting without the system sorts","intersect-slow",&non_fatal);

   nbytes = (*totbnds * 11) + 24;
   for (i = 0; i < *npum; i++)
      {
      /* Open the input and output files
      ----------------------------------*/
      if ((fp = fopen(file2,"r")) == NULL)
         {
         c_errmsg("Error opening temporary file","intersect-open",&non_fatal);
         c_delfil(file2);
         return(E_FAIL);
         }
      if ((fp2 = fopen(file1,"w")) == NULL)
         {
         c_errmsg("Error opening temporary file","intersect-open",&non_fatal);
         fclose(fp);
         c_delfil(file2);
         return(E_FAIL);
         }
      change = FALSE;

      /* Read in the first two records
      --------------------------------*/
      /*fscanf(fp,"%d %d",&oldval,&oldcnt);
      for (x = 0; x < *totbnds; x++)
         fscanf(fp,"%d",&oldatt[x]);*/
      /* read blank line first 
      ------------------------*/
      fgets(ctemp,5,fp);
      fgets(ctemp,nbytes,fp);
      oldval = atol(ctemp);
      oldcnt = atol(&(ctemp[11]));
      for (x = 0; x < *totbnds; x++)
         oldatt[x] = atol(&(ctemp[(22 + (x * 11))]));
      /*fscanf(fp,"%d %d",&newval,&newcnt);
      for (x = 0; x < *totbnds; x++)
         fscanf(fp,"%d",&newatt[x]);*/
      fgets(ctemp,nbytes,fp);
      newval = atol(ctemp);
      newcnt = atol(&(ctemp[11]));
      for (x = 0; x < *totbnds; x++)
         newatt[x] = atol(&(ctemp[(22 + (x * 11))]));
      for (j = 2; j <= *npum; j++)
         {
         write = FALSE;
         for (x = 0; x < *totbnds; x++)
            {
            /* Records are out of order. Switch
            ----------------------------------*/
            if (newatt[x] < oldatt[x])
               {
               fprintf(fp2,"\n%11d%11d",newval,newcnt);
               for (y = 0; y < *totbnds; y++)
                  fprintf(fp2,"%11d",newatt[y]);
               change = TRUE;
               break;
               }
            /* Records are in the correct order
            -----------------------------------*/
            else if (newatt[x] > oldatt[x])
               {
               fprintf(fp2,"\n%11d%11d",oldval,oldcnt);
               for (y = 0; y < *totbnds; y++)
                  fprintf(fp2,"%11d",oldatt[y]);
               oldval = newval;
               oldcnt = newcnt;
               for (y = 0; y < *totbnds; y++)
                  oldatt[y] = newatt[y];
               write = TRUE;
               break;
               }
            }
         /* Records were the same
         ------------------------*/
         if (!write && !change)
            {
            fprintf(fp2,"\n%11d%11d",newval,newcnt);
            for (y = 0; y < *totbnds; y++)
               fprintf(fp2,"%11d",newatt[y]);
            }
         /* Read the next record
         -----------------------*/
         /*fscanf(fp,"%d %d",&newval,&newcnt);
         for (x = 0; x < *totbnds; x++)
            fscanf(fp,"%d",&newatt[x]);*/
         fgets(ctemp,nbytes,fp);
         newval = atol(ctemp);
         newcnt = atol(&(ctemp[11]));
         for (x = 0; x < *totbnds; x++)
            newatt[x] = atol(&(ctemp[(22 + (x * 11))]));
         }

      /* Write the last record to the file and close the files.  Delete the 
         old input file and make the previous output file the new input 
         file to sort. Create a new output file name. 
      ----------------------------------------------------------------*/
      fprintf(fp2,"\n%11d%11d",oldval,oldcnt);
      for (y = 0; y < *totbnds; y++)
         fprintf(fp2,"%11d",oldatt[y]);
      fprintf(fp2,"\n");
      fclose(fp);
      fclose(fp2);
      if (i < *npum)
         c_delfil(file2);
  
      /* Done sorting
      ---------------*/
      if (!change) 
         break;

      strcpy(file2,file1);
      sprintf(file1,"sort%d.out",i);
      }
   }

return(E_SUCC);
}
