/*****************************************************************************
*
*   NAME: TRNIMG 
*
*   FUNCTION: Translate the temporary image 
*
*   EXTERNAL REFERENCES:
*	SORT2.OUT - Sorted translation file	
*
*   PROGRAM HISTORY:
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     1.0       2/84	K.A.Narayanan   Goddard 
*     4.0      10/87	D. Akkerman	CSB	PRF #4329
*     5.0      10/89	D. Etrheim	CSB	LAS 5.0 conversion
*     5.2      06/90	D. Etrheim	CSB	ADP #5196 Increase performance
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*	Translate the temporary image base on the translation table.
*   If the translation table is a file, calculate the number of bytes
*   to read from the translation file to find the translation value.
*	
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "intersect.h"
#include "util.h"
#include "protos.h"

int trnimg
(
    long *nl,			/* number of lines		*/
    long *ns,			/* number of samples		*/
    long *thebuf,			/* image I/O buffer		*/
    long *npum,			/* number of permutations	*/
    long *trntbl,			/* translation table		*/
    long *msgtim,
    char *file3                    /* sorted file                  */
)

{
/* The following variables have been declared global for image I/O routines
--------------------------------------------------------------------------*/
extern struct GDESC * gdesc;	/* group descriptor used for image I/O 		*/
extern long offset1[MAXIMG*MAXBND];/* offsets for input bands		*/
extern long offset2[MAXBND];/* offsets for output 			*/
extern long curent;	/* current line being processed			*/

FILE *fp;			/* File descriptor		*/

long  i,			/* loop counter			*/
      index,			/* offset value for fseek	*/
      msgflg = GEN2,            /* message processing           */
      one = 1;			/* constant of one		*/
char  ctemp[25];		/* temp buffer for fgets */

/* Start process timing message
-------------------------------*/
msgflg = GEN2;
c_inittm("INTERSECT","Translated","line"," "," ",nl,&one,msgtim,&msgflg);
curent = 1;
c_startm();

/* The translation table is in memory
-------------------------------------*/
if (*npum < MAXTBL)
   {
   for (curent = 0; curent < *nl; curent ++)
      {
      if (c_estep(&gdesc) != E_SUCC)
          return(E_FAIL);
      for (i = 0; i < *ns; i++)
         thebuf[offset2[0] + i] = trntbl[thebuf[offset1[0] + i]];
      }
   }
else
   {
   /* The translation table is in a file
   -------------------------------------*/
   if ((fp = fopen(file3,"r")) == NULL)
      {
      c_errmsg("Error opening temporary file","intersect-open",&one);
      c_delfil(file3);
      return(E_FAIL); 
      }
   for (curent = 0; curent < *nl; curent ++)
      {
      if (c_estep(&gdesc) != E_SUCC)
         {
         fclose(fp);
         c_delfil(file3);
         return(E_FAIL); 
         }
      for (i = 0; i < *ns; i++)
         {
         /* If a valid value in the temporary image calculate the
            offset for fseek.  22 bytes per line plus <CR>
         --------------------------------------------------------*/
         if (thebuf[offset1[0] + i] > 0)
            {
            index = (thebuf[offset1[0] + i] -1) * 23;
            fseek(fp,index,0);
            /*fscanf(fp,"%d %d",&dum,&(thebuf[offset2[0] + i]));*/
            fgets(ctemp,23,fp);
            thebuf[offset2[0] + i] = atol(&(ctemp[11]));
            }
         else
            thebuf[offset2[0] + i] = 0;
         }
      }
   c_delfil(file3);
   }

c_stoptm();

/*Final write
-------------*/
if (c_estep(&gdesc) != E_SUCC)
   return(E_FAIL); 

return(E_SUCC);
}
