/******************************************************************************
NAME:   		      CALCULATE 

PURPOSE   To generate the specified bands for an image.  Possible bands are:
          min, max, total bands used, mean, standard deviation, median, and
          mean absolute deviation about the median (MADAM).

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   R. McKinney     12/88	  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file

  5.1      S.Nelson         2/92                        Add Mask Values

  6.1	   D.Eichacker      5/94	EDC		Added opt[]
  7.0      J.Willems        4/97        EDC             Combined updated math2
                                                        with ipmath and added
                                                        meadian and MADAM.
	
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   CALCULATE must be run under TAE.

PROJECT				LAS

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "typlim.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	
                                /* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	/* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND+1];/*offsets into "thebuf" for input bands */
extern long offset2[MAXBND+1];  /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


void calculate
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long (*window)[4],		/* window specification of input images      */
    char procopt[][8],           /* arithematical option(s) to calculate      */
    long optnum,			/* number of mask values		     */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long totbnd,			/* total number of bands to be processed     */
    long odtype,			/* output data type			     */
    long masknum,		/* number of mask values		     */
    float maskval[][2],		/* mask values				     */
    double pixval		        /* pixel value to replace mask values        */
)

{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

static	long	opt[3]={0,0,0}; /* option array for c_eopens                 */
long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	nobnd = 0;		/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long	idtype = 0;		/* type if input data			     */
long	image;			/* loop index counting input images          */
long	i;			/* loop index counting mask values	     */
long	j = 0;			/* loop index incrementing pixel array       */
long	mask_flag;		/* flag for mask value test		     */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    sampin;			/* sample indicator for input		     */
long	sampout;		/* smaple indicator for output		     */
long	sizout = 0;			/* number of bytes per pixel of output data  */
long	sizin = 0;			/* number of bytes per pixel of input data   */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */
long    two = 2;		/* number of bands needed for options        */
long    bandnum = 0;		/* number of output bands		     */

double  stddev = 0.0;			/* standard deviation			     */
double  madam = 0.0;			/* standard deviation about median	     */
double  pix[MAXBND];		/* vector to store pixel values              */

float   mean = 0.0;		/* mean of the bands			     */
float   median = 0.0;	/* mean of the bands			     */
float   temp;			/* temporary storage area		     */
float	temp1;			/* temporary storage area 		     */
float   maxx;                   /* max value                                 */
float   minn;                   /* min value                                 */

char	bndflg[4];		/* flag for total no. of bands               */
char	meanflg[4];		/* flag for mean/st.dev. option              */
char	mnmxflg[4];		/* flag for mean/max option                  */
char	medflg[4];		/* flag for median/madam option              */

unsigned char    *thebuf;	/* image input buffer                        */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&idtype,&zero,opt);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","ipmath-fatal",&fatal);
   band += nbands[image];
   }

/*  Set flags and band number for options selected
-------------------------------------------------*/
for(i = 0; i < optnum; i++)
  {
  if (strcmp(procopt[i],"BANDS") == 0)
    {
    strcpy(bndflg,"YES");
    bandnum++;
    }
  else if (strcmp(procopt[i],"MEAN") == 0)
    {
    strcpy(meanflg,"YES");
    bandnum = bandnum + two;
    }
  else if (strcmp(procopt[i],"MINMAX") == 0)
    {
    strcpy(mnmxflg,"YES");
    bandnum = bandnum + two;
    }
  else if (strcmp(procopt[i],"MEDIAN") == 0)
    {
    strcpy(medflg,"YES");
    bandnum = bandnum + two;
    }
  }

/* Open output image
--------------------*/
if (odtype == 0)
   odtype = idtype;
access = IWRITE;	
status = c_eopens(&fdesc[nmrimg],hosout[0],&bandnum,&one,&one,&nl,&ns,
         offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
      c_errmsg("Fatal error encountered","ipmath-fatal",&fatal);
fdesc[nmrimg+1] = 0;

/* Determine size of input and output image
------------------------------------------*/
switch (odtype)
  {
  case EBYTE: 
    /* If no. of bands are to be output and are 256, it exceed max pixval
    --------------------------------------------------------------------*/
    if (totbnd == 256 && bndflg)
      c_errmsg("BANDS output exceeds 255","ipmath-bands",&nonfatal);
    sizout = 1; break;
  case EWORD: sizout = 2; break;
  case ELONG: sizout = 4; break;
  case EREAL: sizout = 4; break;
  }
switch (idtype)
  {
  case EBYTE: sizin = 1; break;
  case EWORD: sizin = 2; break;
  case ELONG: sizin = 4; break;
  case EREAL: sizin = 4; break;
  }

/* Allocate I/O buffer
----------------------*/
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","ipmath-alloc",&nonfatal);
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
       c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
   c_clean(vblock,"ipmath",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
       c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
   c_clean(vblock,"ipmath",hosout,&one);
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("IPMATH","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Main processing loop
-----------------------*/
/*
   Process each data type separately for time efficiency.
*/
switch (idtype)
  {
case EBYTE:
  for (curlin = 0;curlin < nl; curlin++)
     {
     if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
           c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
        c_clean(vblock,"ipmath",hosout,&one);
        }
     
     /* Increase i/o offsets to increase time efficiency
	-----------------------------------------------*/
     for (band = 0; band < bandnum; band ++)
	offset2[band] *= sizout;
     for (sampin = 0, sampout = 0; sampin < ns; sampin++, sampout += sizout)
	{
	temp = 0.0;
	nobnd = 0;
	j = 0;

	/* Find the sum, mean and std dev if no mask values 
 	  ------------------------------------------------*/
	if (masknum == 0)
	   {
           for (band = 0; band < totbnd; band++)
              {
	      temp += (*(thebuf + (*(offset1 + band) + sampin)));
              if (strcmp(medflg,"YES") == 0)
                {
                pix[j] = (*(thebuf + (*(offset1 + band) + sampin)));
                j++;
                }
              }
           if (strcmp(meanflg,"YES") == 0)
           {
             mean = temp/ (float) totbnd;
	     stddev = 0.0;
           }
           if (strcmp(medflg,"YES") == 0)
           {
	     median = find_median(pix, totbnd);
	     madam = 0.0;
           }
           minn = 255.0;
           maxx = 0.0;
           for (band = 0; band < totbnd; band++)
             {
             temp = (*(thebuf + (*(offset1 + band) + sampin)));
             if (strcmp(meanflg,"YES") == 0)
	        stddev +=(temp - mean) * (temp - mean);
             if (strcmp(medflg,"YES") == 0)
                madam += (temp - median) * (temp - median);
             if (strcmp(mnmxflg,"YES") == 0)
                {
                if (temp > maxx) maxx = temp;
                if (temp < minn) minn = temp;
                }
             }
           if (strcmp(meanflg,"YES") == 0)
             stddev = sqrt(fabs(stddev / totbnd));
           if (strcmp(medflg,"YES") == 0)
             madam = sqrt(fabs(madam / totbnd));
           nobnd = totbnd;
	   }

	/* Test if pixel values are not mask values
	  ----------------------------------------*/
	else
	   {
           for (band = 0; band < totbnd; band++)
              {
              mask_flag = FALSE;
	      temp1 = (*(thebuf + (*(offset1 + band) + sampin)));
              for (i = 0; (i < masknum) && (!mask_flag); i++)
                {
                if (((temp1 >= maskval[i][0]) && (temp1 <= maskval[i][1])))
                   mask_flag = TRUE;
                }
              if (!mask_flag)
                 {
                 temp += temp1;
                 if (strcmp(medflg,"YES") == 0)
                   {
                   pix[j] = (*(thebuf + (*(offset1 + band) + sampin)));
                   j++;
                   }
                 nobnd++;
                 }
               }
             /* If the pixel values are not mask values, calculate dev
              -------------------------------------------------------*/
            minn = 255.0;
            maxx = 0.0;
            if ( nobnd == 0 )
             {
             mean = pixval;
             stddev = pixval;
             median = pixval;
             madam = pixval;
             }
            else
             {
             if (strcmp(meanflg,"YES") == 0)
                {
	        mean = temp/ (float)nobnd;
	        stddev = 0.0;
                }
             if (strcmp(medflg,"YES") == 0)
               {
	       median = find_median(pix, nobnd);
	       madam = 0.0;
               }
             for (band = 0; band < totbnd; band++)
               {
               mask_flag = FALSE;
               temp1 = (*(thebuf + (*(offset1 + band) + sampin)));
               for (i = 0; (i < masknum) && (!mask_flag); i++)
                 {
                 if (((temp1 >= maskval[i][0]) && (temp1 <= maskval[i][1])))
                   mask_flag = TRUE;
                 }
               if (!mask_flag)
                  {
                  if (strcmp(meanflg,"YES") == 0)
       	            stddev += (temp1 - mean) * (temp1 - mean);
                  if (strcmp(medflg,"YES") == 0)
       	            madam += (temp1 - median) * (temp1 - median);
                  if (strcmp(mnmxflg,"YES") == 0)
                    {
                    if (temp1 > maxx) maxx = temp1;
                    if (temp1 < minn) minn = temp1;
                    }
                  }
                }
             if (strcmp(meanflg,"YES") == 0)
               stddev = sqrt(fabs(stddev / nobnd));
             if (strcmp(medflg,"YES") == 0)
               madam = sqrt(fabs(madam / nobnd));
             }
	   }

	switch (odtype)
	   {
	case EBYTE:
          j = 0;
          for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
	       (*(thebuf + (*(offset2 + j) + sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
	       (*(thebuf + (*(offset2 + j) + sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = (unsigned char)nobnd;
               j++;
               }
            }
            break;
	case EWORD:
           j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
	       (*(short *)(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
	       (*(short *)(thebuf+(*(offset2 + j) +sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(short *)(thebuf+(*(offset2 + j)+ sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = (short)nobnd;
               j++;
               }
            }
            break;
        case ELONG:
           j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long)mean;
               j++;
 	       (*(long *)(thebuf+(*(offset2 + j) + sampout))) = (long)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long)median;
               j++;
               (*(long *)(thebuf+(*(offset2 + j) + sampout))) = (long)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long)minn;
               j++;
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long)maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = nobnd;
               j++;
               }
            }
            break;
         case EREAL:
          j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(float *)(thebuf+(*(offset2 + j)+ sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(float *)(thebuf+(*(offset2 + j) + sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = (float)nobnd;
               j++;
               }
            }
           break;
           }
	}
   /* find the minimum and maximum pixel value
   ---------------------------------------*/
   for (band = 0; band < bandnum; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2 + band)),&ns,
                          &minval[band],&maxval[band],&odtype); 
	if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
	}
    curent ++;
    }
  break;
case EWORD:
  for (curlin = 0; curlin < nl; curlin++)
     {
     if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
           c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
        c_clean(vblock,"ipmath",hosout,&one);
        }
     
     /* increase i/o offsets to increase time efficiency
	-----------------------------------------------*/
     for (band = 0; band < totbnd; band++)
	offset1[band] *= sizin;
     for (band = 0; band < bandnum; band ++)
	offset2[band] *= sizout;
     for (sampin = 0, sampout = 0; sampin < (ns * sizin); sampin += sizin,
                                                         sampout += sizout)
	{
	temp = 0.0;
        nobnd = 0;
        j = 0;

	/* Find the sum, mean and std dev if no mask values 
 	  ------------------------------------------------*/
	if (masknum == 0)
	   {
           for (band = 0; band < totbnd; band++)
              {
	      temp += (*(short *)(thebuf + (*(offset1 + band) + sampin)));
              if (strcmp(medflg,"YES") == 0)
                {
                pix[j] = (*(short *)(thebuf + (*(offset1 + band) + sampin)));
                j++;
                }
              }
           if (strcmp(meanflg,"YES") == 0)
             {
	     mean = temp/ (float) totbnd;
	     stddev = 0.0;
             }
           if (strcmp(medflg,"YES") == 0)
             {
             median = find_median(pix, totbnd);
             madam = 0.0;
             }
           minn = 32767.0;
           maxx = -32768.0;
           for (band = 0; band < totbnd; band++)
             {
             temp = (*(short *)(thebuf + (*(offset1+band) + sampin)));
             if (strcmp(meanflg,"YES") == 0)
	       stddev+=(temp - mean) * (temp - mean);
             if (strcmp(medflg,"YES") == 0)
	       madam+=(temp - median) * (temp - median);
             if (strcmp(mnmxflg,"YES") == 0)
                {
                if (temp > maxx) maxx = temp;
                if (temp < minn) minn = temp;
                }
             }
           if (strcmp(meanflg,"YES") == 0)
             stddev = sqrt(fabs(stddev / totbnd));
           if (strcmp(medflg,"YES") == 0)
             madam = sqrt(fabs(madam / totbnd));
           nobnd = totbnd;
	   }

	/* Test if pixel values are not mask values
	  ----------------------------------------*/
	else
	   {
           for (band = 0; band < totbnd; band++)
              {
              mask_flag = FALSE;
	      temp1 = (*(short *)(thebuf + (*(offset1 + band) + sampin)));
              for (i = 0; (i < masknum) && (!mask_flag); i++)
                {
                if (((temp1 >= maskval[i][0]) && (temp1 <=maskval[i][1])))
                   mask_flag = TRUE;
                }
              if (!mask_flag)
                   {
                   temp += temp1;
                   if (strcmp(medflg,"YES") == 0)
                     {
                     pix[j] = (*(short*)(thebuf+(*(offset1 + band) + sampin)));
                     j++;
                     }
                   nobnd++;
                   }
                }

              /* Find the sum, mean, and std dev if not mask value
                 ------------------------------------------------*/
              minn = 32767.0;
              maxx = -32768.0;
              if ( nobnd == 0 )
               {
               mean = pixval;
               stddev = pixval;
               median = pixval;
               madam = pixval;
               }
              else
               {
               if (strcmp(meanflg,"YES") == 0)
                 {
                 mean = temp/ (float) nobnd;
                 stddev = 0.0;
                 }
               if (strcmp(medflg,"YES") == 0)
                 {
                 median = find_median(pix, nobnd);
                 madam = 0.0;
                 }
               for (band = 0; band < totbnd; band++)
                 {
                 mask_flag = FALSE;
                 temp1 = (*(short *)(thebuf + (*(offset1 + band) + sampin)));
                 for (i = 0; (i < masknum) && (!mask_flag); i++)
                  {
                  if (((temp1 >= maskval[i][0]) && (temp1 <= maskval[i][1])))
                    mask_flag = TRUE;
                  }
                 if (!mask_flag)
                   {
                   if (strcmp(meanflg,"YES") == 0)
                      stddev += (temp1 - mean) * (temp1 - mean);
                   if (strcmp(medflg,"YES") == 0)
                      madam += (temp1 - median) * (temp1 - median);
                   if (strcmp(mnmxflg,"YES") == 0)
                      {
                      if (temp1 > maxx) maxx = temp1;
                      if (temp1 < minn) minn = temp1;
                      }
                   }
                }
              if (strcmp(meanflg,"YES") == 0)
               stddev = sqrt(fabs(stddev / nobnd));
              if (strcmp(medflg,"YES") == 0)
               madam = sqrt(fabs(madam / nobnd));
              }
	   }

	switch (odtype)
	   {
	case EBYTE:
          j = 0;
          for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = (unsigned char) nobnd;
               j++;
               }
            }
            break;
        case EWORD:
           j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(short *)(thebuf+(*(offset2 + j) +sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(short *)(thebuf+(*(offset2 + j)+ sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = (short)nobnd;
               j++;
               }
            }
            break;
        case ELONG:
           j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) mean;
               j++;
               (*(long *)(thebuf+(*(offset2 + j) + sampout))) = (long)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) median;
               j++;
               (*(long *)(thebuf+(*(offset2 + j) + sampout))) = (long)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) minn;
               j++;
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = nobnd;
               j++;
               }
            }
            break;
         case EREAL:
          j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(float *)(thebuf+(*(offset2 + j)+ sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(float *)(thebuf+(*(offset2 + j) + sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = (float)nobnd;
               j++;
               }
            }
           break;
           }
	}
   /* find the minimum and maximum pixel value
   ---------------------------------------*/
   for (band = 0; band < bandnum; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2 + band)),&ns,
                          &minval[band],&maxval[band],&odtype); 
	if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
	}
    curent ++;
    }
  break;
case ELONG:
  for (curlin = 0;curlin < nl; curlin++)
     {
     if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
           c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
        c_clean(vblock,"ipmath",hosout,&one);
        }
     
     /* increase i/o offsets to increase time efficiency
	----------------------------------------------*/
     for (band = 0; band < totbnd; band++)
	offset1[band] *= sizin;
     for (band = 0; band < bandnum; band ++)
	offset2[band] *= sizout;
     for (sampin = 0, sampout = 0; sampin < (ns * sizin); sampin += sizin,
							 sampout += sizout)
	{
	temp = 0.0;
        nobnd = 0;
        j = 0;

	/* Find the sum, mean and std dev if no mask values 
 	  ------------------------------------------------*/
	if (masknum == 0)
	   {
           for (band = 0; band < totbnd; band++)
              {
	      temp += (float) ((*(long *)(thebuf + (*(offset1 + band) + sampin))));
              if (strcmp(medflg,"YES") == 0)
                {
                pix[j] = (*(long *)(thebuf + (*(offset1 + band) + sampin)));
                j++;
                }
              }
           if (strcmp(meanflg,"YES") == 0)
             {
             mean = temp/ (float) totbnd;
	     stddev = 0.0;
             }
           if (strcmp(medflg,"YES") == 0)
             {
             median = find_median(pix, totbnd);
             madam = 0.0;
             }
           minn = 65535.0;
           maxx = 0.0;
           for (band = 0; band < totbnd; band++)
             {
             temp =(float) ((*(long *)(thebuf + (*(offset1 + band) + sampin))));
             if (strcmp(meanflg,"YES") == 0)
                stddev +=(temp - mean) * (temp - mean);
             if (strcmp(medflg,"YES") == 0)
                madam += (temp - median) * (temp - median);
             if (strcmp(mnmxflg,"YES") == 0)
                {
                if (temp > maxx) maxx = temp;
                if (temp < minn) minn = temp;
                }
             }
           if (strcmp(meanflg,"YES") == 0)
             stddev = sqrt(fabs(stddev / totbnd));
           if (strcmp(medflg,"YES") == 0)
             madam = sqrt(fabs(madam / totbnd));
           nobnd = totbnd;
	   }

	/* Test if pixel values are mask values
	  ------------------------------------*/
	else
	   {
           for (band = 0; band < totbnd; band++)
              {
              mask_flag = FALSE;
	      temp1 =(float) ( (*(long *)(thebuf + (*(offset1 + band) + sampin))));
              for (i = 0; (i < masknum) && (!mask_flag); i++)
                {
                if (((temp1 >= maskval[i][0]) && (temp1 <= maskval[i][1])))
                   mask_flag = TRUE;
                }
              if (!mask_flag)
                 {
                 temp += temp1;
                 if (strcmp(medflg,"YES") == 0)
                   {
                   pix[j] = (*(long*)(thebuf + (*(offset1 + band) + sampin)));
                   j++;
                   }
                 nobnd++;
                 }
               }
             /* If the pixel values are not mask values, calculate dev
              -------------------------------------------------------*/
              minn = 65535.0;
              maxx = 0.0;
              if ( nobnd == 0 )
               {
               mean = pixval;
               stddev = pixval;
               median = pixval;
               madam = pixval;
               }
              else
                {
                if (strcmp(meanflg,"YES") == 0)
                  {
                  mean = temp/ (float) nobnd;
                  stddev = 0.0;
                  }
                if (strcmp(medflg,"YES") == 0)
                  {
                  median = find_median(pix, nobnd);
                  madam = 0.0;
                  }
                for (band = 0; band < totbnd; band++)
                  {
                  mask_flag = FALSE;
                  temp1 =(float)((*(long *)(thebuf +
                                (*(offset1 + band) + sampin))));
                  for (i = 0; (i < masknum) && (!mask_flag); i++)
                    {
                    if (((temp1 >= maskval[i][0]) && (temp1 <= maskval[i][1])))
                      mask_flag = TRUE;
                    }
                  if (!mask_flag)
                     {
                      if (strcmp(meanflg,"YES") == 0)
                        stddev += (temp1 - mean) * (temp1 - mean);
                      if (strcmp(medflg,"YES") == 0)
                        madam += (temp1 - median) * (temp1 - median);
                      if (strcmp(mnmxflg,"YES") == 0)
                         {
                         if (temp1 > maxx) maxx = temp1;
                         if (temp1 < minn) minn = temp1;
                         }
                      }
                   }
                if (strcmp(meanflg,"YES") == 0)
                 stddev = sqrt(fabs(stddev / nobnd));
                if (strcmp(medflg,"YES") == 0)
                 madam = sqrt(fabs(madam / nobnd));
                }
           }

	switch (odtype)
	   {
        case EBYTE:
          j = 0;
          for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = (unsigned char) nobnd;
               j++;
               }
            }
            break;
        case EWORD:
           j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(short *)(thebuf+(*(offset2 + j) +sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(short *)(thebuf+(*(offset2 + j)+ sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = (short)nobnd;
               j++;
               }
            }
            break;
        case ELONG:
           j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) mean;
               j++;
               (*(long *)(thebuf+(*(offset2 + j) + sampout))) = (long)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long)median;
               j++;
               (*(long *)(thebuf+(*(offset2 + j) + sampout))) = (long)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) minn;
               j++;
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = nobnd;
               j++;
               }
            }
            break;
         case EREAL:
          j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(float *)(thebuf+(*(offset2 + j)+ sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(float *)(thebuf+(*(offset2 + j) + sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = (float)nobnd;
               j++;
               }
            }
           break;
           }
        }
   /* find the minimum and maximum pixel value
   ---------------------------------------*/
   for (band = 0; band < bandnum; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2 + band)),&ns,
                          &minval[band],&maxval[band],&odtype); 
	if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
	}
    curent ++;
    }
  break;
case EREAL:
  for (curlin = 0; curlin < nl; curlin++)
     {
     if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
           c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
        c_clean(vblock,"ipmath",hosout,&one);
        }
     
     /* increase i/o offsets to increase time efficiency
	-----------------------------------------------*/
     for (band = 0; band < totbnd; band++)
	offset1[band] *= sizin;
     for (band = 0; band < bandnum; band ++)
	offset2[band] *= sizout;
     for (sampin = 0, sampout = 0; sampin < (ns * sizin); sampin += sizin,
							 sampout += sizout)
	{
	temp = 0.0;
        nobnd = 0;
        j = 0;

	/* Find the sum, mean and std dev if no mask values 
 	  ------------------------------------------------*/
	if (masknum == 0)
	   {
           for (band = 0; band < totbnd; band++)
              {
	      temp += (*(float *)(thebuf + (*(offset1+band)+sampin)));
              if (strcmp(medflg,"YES") == 0)
                {
                pix[j] = (*(float *)(thebuf + (*(offset1 + band) + sampin)));
                j++;
                }
              }
           if (strcmp(meanflg,"YES") == 0)
             {
	     mean = temp/ (float) totbnd;
	     stddev = 0.0;
             }
           if (strcmp(medflg,"YES") == 0)
             {
             median = find_median(pix, totbnd);
             madam = 0.0;
             }
           minn = 2147483647.0;
           maxx = (-2147483647 - 1);
           for (band = 0; band < totbnd; band++)
             {
             temp = (*(float *)(thebuf + (*(offset1 + band) + sampin)));
             if (strcmp(meanflg,"YES") == 0)
                stddev +=(temp - mean) * (temp - mean);
             if (strcmp(medflg,"YES") == 0)
                madam += (temp - median) * (temp - median);
             if (strcmp(mnmxflg,"YES") == 0)
                {
                if (temp > maxx) maxx = temp;
                if (temp < minn) minn = temp;
                }
             }
           if (strcmp(meanflg,"YES") == 0)
             stddev = sqrt(fabs(stddev / totbnd));
           if (strcmp(medflg,"YES") == 0)
             madam = sqrt(fabs(madam / totbnd));
           nobnd = totbnd;
	   }

	/* Test if pixel values are mask values
	  ------------------------------------*/
	else
	   {
           for (band = 0; band < totbnd; band++)
              {
              mask_flag = FALSE;
	      temp1 = (*(float *)(thebuf + (*(offset1 + band) + sampin)));
              for (i = 0; (i < masknum) && (!mask_flag); i++)
                {
                if (((temp1 >= maskval[i][0]) && (temp1 <= maskval[i][1])))
                   mask_flag = TRUE;
                }
              if (!mask_flag)
                 {
                 temp += temp1;
                 if (strcmp(medflg,"YES") == 0)
                   {
                   pix[j] = (*(float *)(thebuf+(*(offset1 + band) + sampin)));
                   j++;
                   }
                 nobnd++;
                 }
               }
             /* If the pixel values are not mask values, calculate dev
              -------------------------------------------------------*/
              minn = 2147483647.0;
              maxx = (-2147483647 - 1);
              if ( nobnd == 0 )
               {
               mean = pixval;
               stddev = pixval;
               median = pixval;
               madam = pixval;
               }
              else
               {
               if (strcmp(meanflg,"YES") == 0)
                  {
                  mean = temp/ (float)nobnd;
                  stddev = 0.0;
                  }
               if (strcmp(medflg,"YES") == 0)
                  {
                  median = find_median(pix, nobnd);
                  madam = 0.0;
                  }
               for (band = 0; band < totbnd; band++)
                {
                mask_flag = FALSE;
                temp1 = (*(float *)(thebuf + (*(offset1 + band) + sampin)));
                for (i = 0; (i < masknum) && (!mask_flag); i++)
                  {
                  if (((temp1 >= maskval[i][0]) && (temp1 <= maskval[i][1])))
                    mask_flag = TRUE;
                  }
                if (!mask_flag)
                   {
                   if (strcmp(meanflg,"YES") == 0)
                     stddev += (temp1 - mean) * (temp1 - mean);
                   if (strcmp(medflg,"YES") == 0)
                     madam += (temp1 - median) * (temp1 - median);
                   if (strcmp(mnmxflg,"YES") == 0)
                      {
                      if (temp1 > maxx) maxx = temp1;
                      if (temp1 < minn) minn = temp1;
                      }
                  }
                }
              if (strcmp(meanflg,"YES") == 0)
               stddev = sqrt(fabs(stddev / nobnd));
              if (strcmp(medflg,"YES") == 0)
               madam = sqrt(fabs(madam / nobnd));
	      }
	   }

	switch (odtype)
	   {
	case EBYTE:
          j = 0;
          for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(thebuf + (*(offset2 + j) + sampout))) = (unsigned char) nobnd;
               j++;
               }
            }
            break;
	case EWORD:
           j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(short *)(thebuf+(*(offset2 + j) +sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(short *)(thebuf+(*(offset2 + j)+ sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(short *)(thebuf + (*(offset2 + j) + sampout))) = (short)nobnd;
               j++;
               }
            }
            break;
        case ELONG:
           j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long)mean;
               j++;
               (*(long *)(thebuf+(*(offset2 + j) + sampout))) = (long)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long)median;
               j++;
               (*(long *)(thebuf+(*(offset2 + j) + sampout))) = (long)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) minn;
               j++;
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = (long) maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(long *)(thebuf + (*(offset2 + j) + sampout))) = nobnd;
               j++;
               }
            }
            break;
        case EREAL:
          j = 0;
           for (i=0; i< optnum; i++)
            {
              if (strcmp(procopt[i],"MEAN") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = mean;
               j++;
               (*(float *)(thebuf+(*(offset2 + j)+ sampout))) = (float)(stddev);
               j++;
               }
              else if (strcmp(procopt[i],"MEDIAN") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = median;
               j++;
               (*(float *)(thebuf+(*(offset2 + j) + sampout))) = (float)(madam);
               j++;
               }
              else if (strcmp(procopt[i],"MINMAX") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = minn;
               j++;
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = maxx;
               j++;
               }
              else if (strcmp(procopt[i],"BANDS") == 0)
               {
               (*(float *)(thebuf + (*(offset2 + j) + sampout))) = (float)nobnd;
               j++;
               }
            }
           break;
           }
	}
   /* find the minimum and maximum pixel value
   ---------------------------------------*/
   for (band = 0; band < bandnum; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2 + band)),&ns,
                          &minval[band],&maxval[band],&odtype); 
	if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
	}
    curent ++;
    }
  break;
  }

/* Final write
--------------*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
      c_clean(vblock,"ipmath",hosout,&one);
   }
/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = COMB;
status =c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&bandnum,&upddr_flag);
if (status != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);
   c_clean(vblock,"ipmath",hosout,&one);
   }

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&bandnum,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);

/* Free buffer
--------------*/
free(thebuf);
return;
}
/****************************************************************************
NAME:                            FIND_MEDIAN

PURPOSE:  Finds the median value 

VERSION  DATE   AUTHOR
-------  ----   ------
  1.0    4/93   D. Steinwand
  7.0    4/97   J. Willems

****************************************************************************/
static long compare_doubles
(
    double *i,
    double *j
)
{
  if(*i>*j) 
    return(1); 
  else if(*i<*j) 
    return(-1); 
  else 
    return(0);
}

double find_median
(
    double *array,
    long npts
)
{
  qsort(array,npts,sizeof(double),compare_doubles);
  if ((npts-1)%2 != 0) 
    return((array[npts/2]+array[(npts/2)-1])/2.0);
  else return(array[npts/2]); 
}
