/******************************************************************************
NAME      		 IPMATH 

PURPOSE  Perform various arith functions on one input image and create a
	 single output image.  The functions are min, max, mean, standard
         deviation, median, and mean absolute deviation about the median.

PROGRAM HISTORY

VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   R.McKinney      12/88	  EDC		Add UPDDR routine
							Take descriptors out 
							of calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file

  7.0     J.Willems        4/97           EDC           Add functionality for
                                                        FEWS project.  (median,
                                                        MADAM, and total bands
                                                        used)
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   IPMATH must be run under TAE.

PROJECT   	       LAS 5.0		                

LANGUAGE		  C	

ALGORITHM (FLOW)
    The program begins with the standard initialization and retrieval 
of user-specified parameters.  The input and output images are opened
and grouped for sequential processing.  The input image data is converted
to REAL*4 (float) upon input, and the data type of the output image is
always REAL*4.  Space is allocated for the image I/O buffer.  Next, the
initialization for the processing message and min/max arrays is done.
The appropriate outputs are calculated based upon the options entered.
Finally, history files are created for the output image and a completion 
message is output.

ALGORITHM REFERENCES     none

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC * fdesc[MAXIMG+1]; /* image file descriptors used for image i/o */
struct GDESC * gdesc;			/* group descriptor used for image i/o       */
long offset1[MAXIMG*MAXBND+1];/* offsets into thebuf for input bands       */
long offset2[MAXBND+1];	        /* offsets into thebuf for input bands       */
long curent;			/* current processing line      	     */
long curcat;			/* current processing band		     */
long bands[MAXIMG][MAXBND + 1]; /* total number of input bands               */
double  minval[MAXBND];         /* min pixel value array                     */
double  maxval[MAXBND];         /* max pixel value array                     */

int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		     */


static long bands[MAXIMG][MAXBND + 1];       /* band specs for first image   */
long masknum;			/* number of mask values                     */
long optnum;			/* number of options specified               */
long msgtim;			/* elapsed time between display messages     */
long nbands[MAXIMG];		/* number of bands specified per image       */
long nmrimg;			/* total number of input images              */
long nonfatal = 1;		/* nonfatal error flag			     */
long odtype;			/* odtype for output image                   */
long one = 1;			/* integer value of one			     */
long std_out;			/* logical unit # for standard output        */
long totbnd;			/* total number of bands in first image      */
long window[MAXIMG][4];	        /* window specification for first image      */

char hosin[MAXIMG][CMLEN];      /* buffer storage for host name	             */
char hosout[1][CMLEN];          /* buffer storage for output host name       */
char minmax[4];	                /* minmax flag buffer 			     */
char taein[MAXIMG][CMLEN];      /* buffer storage for TAE input name	     */
char taeout[1][CMLEN];          /* buffer storage for TAE output name        */
char procopt[4][8];		/* arithematical option(s) to calculate      */

float maskval[10][2];		/* mask values                               */

float pixval;			/* pixel value to replace a mask value       */

/* initialize LAS and TAE  
------------------------------------------------*/

c_inlas(&vblock, &std_out);  

/* Get user-specified parameters and TAE globals
-----------------------------------------------------*/

getpar(&vblock, taein, hosin, 
       taeout, hosout, &nmrimg, bands, nbands,
       window, procopt, &optnum, &msgtim, minmax, &totbnd,  
       &odtype, maskval, &masknum, &pixval);

/* Calculate Values
---------------------------------------------------*/
calculate(&vblock,hosin,hosout,nmrimg,
        bands,nbands,window,procopt,optnum,msgtim,minmax,
	totbnd,odtype,masknum,maskval,pixval);

 /*  Create associated history file
-------------------------------------------------*/

nmrimg = one;

if (c_hstout(&vblock, hosout[0], hosin, &nmrimg, " ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","ipmath-warn",&nonfatal);


/*  Inform user of successful completion
-------------------------------------------------*/

c_complt("IPMATH", taeout, &one);

return EXIT_SUCCESS;

}
