#include "isoclass.h"
#include "protos.h"

void chain
(void)
/*****************************************************************************
 *
 * NAME		chain
 *
 * PURPOSE	To chain all clusters that are of distance less than chaining
 *		threshold (chnths).
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clknt [MAXCLS]	long		I	# pixels in a cluster
 *	nclust		short		I	Total # of clusters so far
 *
 * ALGORITHM
 *
 *	initialize the chain by unlinking all clusters
 *	for all clusters except last one
 *		if cluster has any members
 *			for all greater clusters
 *				if this cluster has any members
 *					if distance is less than chnths
 *						link the two clusters
 *					end if
 *				end if
 *			end for
 *		end if
 *	end for
 *	give each unique chain its own chain number
 *	output all of the chains
 *
 *****************************************************************************/
{
	short	chain_count,		/* count of the number of chains */
		clust,			/* index when chaining clusters */
	        i,			/* cluster index for distances */
		test;			/* index when testing against clust */
        float   newdist;

	/* initialize chain_list */
	for (i=0; i < maxcls; i++)
           chain_list[i] = UNLINKED;

	/* determine which clusters may be chained */
	for (clust = 0;  clust < nclust - 1;  clust++) {
 		if (clknt [clust] != 0) {
			for (test = clust + 1;  test < nclust;  test++) {
				if (clknt [test] != 0) {
                                        newdist = chain_dist(clust,test);
					if (newdist < chnths)
						link_cluster (clust, test);
                                }
                        }
                 }
         }
	/*
	 * Output any chains if they exist with clusters in each chain
	 * sorted.
	 */
	chain_count = combine_chain ();
	print_chain (chain_count);
}
