#include <math.h>
#include "isoclass.h"
#include "protos.h"

float chain_dist
(
    int i,
    int j
)
/*****************************************************************************
 *
 * NAME		chain_dist
 *
 * PURPOSE	Sums all of the distances between the means for two clusters.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	i		short		I	Cluster #
 *	j		short		I	Cluster #
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clmean [MAXCLS]
 *	       [MAXBND]	float		I	Means for clusters on bands
 *	clstdv [MAXCLS]
 *	       [MAXBND]	float		I	Std Devs for clusters on bands
 *	totbnd		short		I	# bands in input image
 *	sep		float		I	Split separation value
 *
 * RETURN VALUES
 *
 *	Distance (float) between the two clusters
 *
 * ALGORITHM DESCRIPTION
 *
 *	for each band
 *		square the difference between the two means
 *		if sep is 0.0
 *			if neither std dev is not 0.0
 *				divide by product of std devs
 *			else
 *				divide by 0.0001
 *		    	end if
 *		else
 *			divide by sep squared
 *		end if
 *		add square root of result to sum
 *	end for
 *
 *****************************************************************************/
{
	short	k;				/* image index */
	float	dif2,				/* mean difference */
		sumd = 0.0,			/* sum of differences */
		*clmean_cur_i = clmean [i],	/* cursor for clmean [i] */
		*clmean_cur_j = clmean [j],	/* cursor for clmean [j] */
		*clstdv_cur_i = clstdv [i],	/* cursor for clstdv [i] */
		*clstdv_cur_j = clstdv [j];	/* cursor for clstdv [j] */


	for (k = 0;  k < totbnd;  k++)  {
		dif2 = *clmean_cur_i++ - *clmean_cur_j++;
		dif2 *= dif2;					/* square it */
		if (sep == 0.0) 
			if ((*clstdv_cur_i != 0.0) && (*clstdv_cur_j != 0.0))
				dif2 /= *clstdv_cur_i * *clstdv_cur_j;
			else
				dif2 /= 0.0001;
		else
			dif2 /= sep * sep;
		sumd += (dif2);

		clstdv_cur_i++;
		clstdv_cur_j++;
	}
	sumd = sqrt (sumd);

	return (sumd);
}
