#include "isoclass.h"
#include "protos.h"

short combine_chain
(void)
/*****************************************************************************
 *
 * NAME		combine_chain
 *
 * PURPOSE	Gives all members of a chain the same chain identifier number.
 *		Chain identifier numbers are not smaller than MAXCLS.  This
 *		avoids conflicts with links to other clusters.  The chain
 *		assignments are done by going up through the chain list and
 *		seeing what a cluster is linked to.  If it is unlinked, then
 *		it isn't part of a chain.  If it is linked to an unlinked cell,
 *		then the unlinked cell is the beginning of a new chain.  It,
 *		and the current cell are given a new chain number.  If it is
 *		linked to an already chained cluster then this cluster gets
 *		the same chain number.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		---- 		---	-----------
 *	nclust		short		I	# of clusters in image so far
 *
 * RETURN VALUES
 *
 *	Number (short) of chains + MAXCLS
 *
 * ALGORITHM DESCRIPTION
 *
 *	for each cluster
 *		if this cluster is linked
 *			if the cluster this cluster is linked to is unlinked
 *				give that cluster a new chain number
 *				give this cluster the same chain number
 *			else
 *			    	give this cluster the same chain number
 *			end if
 *		end if
 *	end for
 *
 *****************************************************************************/
{
	short	*back_ptr,			/* what chain_cur points at */
		chain_count = MAXCLS,		/* next chain number to use */
		*chain_cur = chain_list,	/* cursor into chain array */
		i;				/* cluster index */


	for (i = 0;  i < nclust;  i++)  {
		if (*chain_cur != UNLINKED)  {
			back_ptr = &(chain_list [*chain_cur]);
	 		if (*back_ptr == UNLINKED)  {
			       	*back_ptr = chain_count;
				*chain_cur = chain_count++;
	 		}  else
	 		       	*chain_cur = *back_ptr;
		}
		chain_cur++;
	}

	return (chain_count);
}
