#include "isoclass.h"
#include "protos.h"

/*****************************************************************************
 *
 * NAME		decide
 *
 * PURPOSE	To decide whether a split or combine operation should be
 *		done on this iteration.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	*itertype	char		I/O	Tag telling if split or combine
 *	iter		short		I	Current iteration #
 *	*alter		short		I/O	Bool : at alternating stage yet?
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	clstdv [MAXCLS]
 *	       [MAXBND]	float		I	Cluster std dev for each band
 *	maxit		short		I	Maximum # of iterations
 *	nclust		short		I	# clusters currently in image
 *	stdmax		float		I	Split determiner
 *	totbnd		short		I	Total # of bands in input image
 *
 * ALGORITHM DESCRIPTION
 *
 *	if alternating
 *		if just combined then
 *			split
 *		else if this is the last iteration then
 *			split
 *		else
 *			combine
 *		end if
 *	else
 *		count # of clusters with std dev greater than stdmax
 *	      	if this is not greater than 20% then
 *			start alternating
 *			combine next
 *		else
 *			keep on splitting
 *		end if
 *	end if
 *
 *****************************************************************************/
void decide
(
    char *itertyp,
    int iter,
    short *alter		/* boolean */
)
{
	short	count = 0,	/* # clusters w/ std devs below stdmax */
		i,		/* cluster counter */
		j;		/* band counter */
	float	pct = 0.20 * nclust; 	/* min # clusters w/ bad std devs */


	if (*alter == TRUE)  {
		/*
		 * Alternate between combine and split operations
		 */
		if (*itertyp == COMBINE) 
			*itertyp = SPLIT;
		/*
		 * if this is to be the last iteration then force it to be a
		 * split iteration
		 */
		else if (iter == maxit - 1) 
			*itertyp = SPLIT;
 		else
 			*itertyp = COMBINE;
	}  else  {
		/*
		 * A series of split iterations is required before the program
		 * alters between iteration types.  This initializes the initial
		 * cluster means if no statistics file is input, or may continue
		 * split operations on means obtained from the stats file if
		 * the stats file does not have 80% of its clusters having
		 * standard deviations below stdmax
		 */
		for (i = 0;  i < nclust;  i++)
			for (j = 0;  j < totbnd;  j++)
				if (clstdv [i] [j] >= stdmax)  {
					count++;
			 		break;
				}

		/*
		 * count is the number of clusters with standard
		 * deviations >= stdmax
		 */
		if (pct >= (float) count)  {
			*alter = TRUE;
			*itertyp = COMBINE;
		}  else
 			*itertyp = SPLIT;
	}
}
