#include "isoclass.h"
#include "protos.h"

void dyn_alloc
(
   double *scov[],              /* I: pointers to covariance matrix  */
   double *clsumx[],            /* I: Sum of pixel values in clusters */
   double *clsumsq[]            /* I: Squared sum of pixel values */
)

/*
*********************************************************************
NAME: DYN_ALLOC

PURPOSE: Allocate storage for covariance matrices 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  1.0         08/17/88  K. A. Narayanan
  2.0         08/31/88  B. Allam

*********************************************************************
*/
{
long	buf_size, t_size, cl_size, c_size, /* size of matrices       */
        i, n, m;

/*
allocate storage for covariance matrices 
-----------------------------------------
*/
t_size = (totbnd *(totbnd+1) /2);
buf_size = (long)(maxcls * t_size * sizeof(double));
cl_size = (long)(maxcls * totbnd * sizeof (double));
c_size = (long)(maxcls * totbnd * sizeof (float));

if ((scov_matrix = (double *) malloc( buf_size)) == NULL ||
    (clsumx_matrix = (double *) malloc( cl_size)) == NULL ||
    (clsumsq_matrix = (double *) malloc( cl_size)) == NULL ||
    (clstdv_matrix = (float *) malloc( c_size)) == NULL ||
    (clmean_matrix = (float *) malloc( c_size)) == NULL) 
    iso_error ("!Error allocating dynamic memory", "isoclass-alloc");
   

/*
initialize the covariance array
*/
for (i=0; i<maxcls * t_size; i++)
   *(scov_matrix + i) = 0.0;

/*
set up pointers for covariance matrices 
----------------------------------------
*/
for (i = 0, n = 0, m = 0; i < maxcls; i++, n += t_size, m += totbnd)
   {
   scov[i] = &scov_matrix[n];
   clsumx[i] = &clsumx_matrix[m];
   clsumsq[i] = &clsumsq_matrix[m];
   clmean[i] = &clmean_matrix[m];
   clstdv[i] = &clstdv_matrix[m];
   }
}
