#include "isoclass.h"
#include "protos.h"

void finish_cov
(
    double *scov[]
)
/*****************************************************************************
 *
 * NAME		finish_cov
 *
 * PURPOSE  	Finishes up the covariance matrix for the current set of
 *		clusters.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	scov [MAXCLS]
 *	     [LOW_HALF]	double		I/O	Covaraince matrix
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----	       	----		---	-----------
 *	clknt [MAXCLS]	long		I	# pixels in each cluster
 *	nclust		short		I	# clusters currently in image
 *	totbnd		short		I	# bands in input image
 *
 * ALGORITHM
 *
 *
 *
 *****************************************************************************/
{                   
	short	i,		/* band counter */
		j,		/* band counter (always less than i) */
	     	k;		/* cluster counter */
	long	clknt_val;	/* particular value in clknt array */
	double	*scov_ptr;	/* pointer into scov array */


	for (k = 0;  k < nclust;  k++)  {
		scov_ptr = scov [k];
		clknt_val = clknt [k];
		for (i = 0;  i < totbnd;  i++)  {
			for (j = 0;  j <= i;  j++)  {
 				if ((*scov_ptr != 0.0) &&
				    (clknt_val > 1))
					*scov_ptr /= (double) (clknt_val - 1.0);
				scov_ptr++;
			}			/* 0 <= j < i */
		}				/* 0 <= i < totbnd */
	}					/* 0 <= k < nclust */
}
