#include "isoclass.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char print_filename[CMLEN],         /* O: output file name  */
    long bands[MAXIMG][MAXBND+1]	/* O: selected band #'s */
)

/*****************************************************************************
 *
 * NAME		getpar
 *
 * PURPOSE	Gets all of the parameters from LAS.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	vblock		struct PARBLK	I	Parameter block passed from LAS
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----   	      	----		---	-----------
 *	bands [MAXIMG]
 *	      [MAXBND]	long		O	Selected bands #'s in files
 *	chnths		float		I/O	Chaining threshold
 *	dlmin		float		I/O	Distance to combine clusters at
 *	do_minmax	short		O	Boolean rep. - do minmax calc.?
 *	host_in		char  	        I/O	Host version - input filenames
 *	host_out	char    	I/O	Host version - output filenames
 *	krn		short		O	Frequency of screen stats
 *	maxcls		short		I/O	Maximum # of allowed clusters
 *	maxit		short		I/O	Maximum # of iterations
 *	minmax [4]	char		I/O	String rep. - do minmax calc.?
 *	msgtime		short		O	Interval between messages
 *	nbands [MAXIMG]	long		O	# of bands in each image file
 *	nimages		long		I/O	# files for input images
 *	nmin		long		I/O	Min # pixels allowed in cluster
 *	print_file	FILE		I	File handle for the report file
 *	sep		float		I/O	Split separation value
 *	*srcstat	char		I/O	Where stats are coming from
 *	stat_host_in	char    	O	Host version - in stat filename
 *	stat_las_in	char    	O	LAS version - in stat filename
 *	stat_host_out	char    	O	Host version - out stat filename
 *	stdmax		float		I/O	Split determiner
 *	las_in		char    	O	LAS version - input filenames
 *	las_out		char    	O	LAS version - output filenames
 *	totbnd		short		I/O	Total # bands in input image
 *	view		short		O	# iterations between prompts
 *	window		long		O	Dimension of images
 *
 *
 * ALGORITHM
 *
 *	get the input image filenames and band numbers
 *	check for any errors in input imgages
 *	get the output image filename
 *	get the output statistics filename
 *	get STATSRC telling where input statistics come from
 *	if input statistics come from a file
 *		get the input statistics filename
 *	end if
 *	get CLUSDIST, MAXCLSTD, SSEPVAL, PRINTIT, VIEWIT, MINCLUST, MAXCLUST, 
 *      CHNTHR, and MAXNUMIT
 *	read the LAS global $msgtim
 *	set history boolean to true if value is "yes" or "YES", false otherwise
 *	copy the string value of the LAS global $minmax
 *	set minmax boolean to true if value is "yes" or "YES", false otherwise
 *	read the LAS global $RUNTYPE
 *	if it is "BATCH" or "ASYNC"
 *		set VIEWIT parameter to only the last iteration
 *	end if
 *	print out input image filename
 *	print out output image filename
 *	if input statistics come from file
 *		print input statistics filename
 *	end if
 *	print output statistics filename
 *	print out values of parameters CLUSDIST, MAXCLSTD, SSEPVAL, MINCLUST, 
 *      CHNTHR,	MAXCLUST, and MAXNUMIT
 *
 *****************************************************************************/
{
	char            fileutil_filename[CMLEN], /* temp file for fileutil */
			*runtyp;		/* method to run program */
	short		i;			/* general counter */
	long		access,			/* kind of access for file */
			imglimit = MAXIMG,	/* maximum number of images */
			qflag [MAXIMG] [2],	/* image flag array */
	    		same_window,		/* flag if all windows same */
			status;			/* return val of ext modules */


	/* get the input image file names */
	status = c_getin (&vblock,"IN",&zero, las_in, host_in, &nimages, window,
			bands, nbands, &totbnd, &imglimit, &same_window,
			qflag);

	/* check for any errors from input images */
	if (status != E_SUCC)
		iso_error ("$", ""); 	/* fatal error */
	if (!same_window)
	  iso_error ("!Error: Input images must be same size","isoclass-size");

	/* get the output image file name */
	status = c_getout (&vblock, "OUT", (char (*)[256])las_out, 
				(char (*)[256])host_out);
	if (status != E_SUCC)
		iso_error ("$", ""); 	/* fatal error */
	else
	 {
        strcpy (out_las[count_las], las_out);
        count_las = count_las +1;
	 }

	/* get the output statistics file name */
	access = IWRITE;
	status = c_getnam (&vblock, "OUTSTAT", &zero, stat_host_out, 
				"stat", &access);
	if (status != E_SUCC)
		iso_error ("$", ""); 	/* fatal error */
	else
         {
        strcpy (out_las[count_las], stat_host_out);
        count_las = count_las +1;
	 }

	/* if there are input statistics, get its file name */
	srcstat = SVAL(*p_find (&vblock, "STATSRC"), 0);
	if (srcstat [0] == 'F')  {
		access = IREAD;
		status = c_getnam (&vblock, "INSTAT", &zero, stat_host_in, 
					"stat", &access);
		if (status != E_SUCC) 
			iso_error ("$", ""); 	/* fatal error */
		else
                    strcpy(stat_las_in, stat_host_in);
	}

	/* read in some more parametes */
	dlmin	= RVAL(*p_find (&vblock, "CLUSDIST"), 0);
	stdmax	= RVAL(*p_find (&vblock, "MAXCLSTD"), 0);
	sep	= RVAL(*p_find (&vblock, "SSEPVAL"), 0);
	krn	= IVAL(*p_find (&vblock, "PRINTIT"), 0);
	view	= IVAL(*p_find (&vblock, "VIEWIT"), 0);
	nmin	= IVAL(*p_find (&vblock, "MINCLUST"), 0);
	maxcls	= IVAL(*p_find (&vblock, "MAXCLUST"), 0);
	chnths	= RVAL(*p_find (&vblock, "CHNTHR"), 0);
	maxit	= IVAL(*p_find (&vblock, "MAXNUMIT"), 0);
                                        
        /*
        Get print option and open print file if necessary
        -------------------------------------------------
        */
        strcpy (fileutil_filename, "ISOCLASS");
		/* print[] is initialized in c_setprt */
        status = c_setprt (&vblock, &print[TERM], &print[FIL], &print[LP], 
                  fileutil_filename, print_filename);
	   if (status != E_SUCC) 
			iso_error ("$", ""); 	/* fatal error */
        print_file = fopen (fileutil_filename, "w");
        if (print_file == NULL)				/* Bad open? */
 	        iso_error ("!Error opening output print file", "isoclass-open");
        strcpy (out[count], fileutil_filename);
	   count = count + 1;
	   /* The print_filename is used here so the name is in LAS format
	    * Also fileutil_filename and print_filename DO NOT correspond to the 
	    * same file name.  They are separate files.  The fileutil_filename will
	    * will be renamed to print_filename in the isoclass.pdf (fileutil).
	    */
	   if (print[FIL] == 1)
		{
		   strcpy (out_las[count_las], print_filename);
		   count_las = count_las +1;
		}

	/* get LAS globals */
	msgtime = IVAL(*p_find (&vblock, "$msgtim"), 0);
	strcpy (minmax, SVAL(*p_find (&vblock, "$minmax"), 0));
	do_minmax = !strcmp (minmax, "yes") || !strcmp (minmax, "YES");

	/* get implicit global variable $runtype */
	runtyp = SVAL(*p_find (&vblock, "$RUNTYPE"), 0);
	if (!strncmp (runtyp, "BATCH", 5) || !strncmp (runtyp, "ASYNC", 5))
		view = maxit;

	/* write out parameters */
	fprintf (print_file, "INPUT IMAGE FOR ISOCLASS \n");
	for (i = 0;  i < nimages;  i++)  {
		fprintf (print_file, "\n     %s\n", las_in[i]);
	}
	fprintf (print_file, "\n\nOUTPUT IMAGE FOR ISOCLASS \n");
	fprintf (print_file, "\n     %s\n", las_out);
	fprintf (print_file,
		 "\n\nSTATISTICS FILE(S) FOR ISOCLASS \n");
	if (srcstat [0] == 'F')
		fprintf (print_file, "\n     IN :  %s\n", stat_las_in);
	fprintf (print_file, "\n     OUT :  %s\n", stat_host_out);
	fprintf (print_file, "\n\n\nPARAMETERS \n");
	fprintf (print_file, "\n  CLUSDIST  MAXCLSTD   SSEPVAL    MINCLUST    CHNTHR  MAXCLUST  MAXNUMIT");
	fprintf (print_file, "\n  --------  --------   -------    --------    ------  --------  --------");
	fprintf (print_file,
	     	  "\n %9.3f %9.3f %9.3f        %4d %9.3f      %4d      %4d\n",
		  dlmin, stdmax, sep, nmin, chnths, maxcls, maxit);
}
