#include "isoclass.h"
#include "protos.h"
              
/*****************************************************************************
 *
 * NAME		isodel
 *
 * PURPOSE	To delete any clusters that do not have the minimum number of
 *		members (nmin).
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *      Bani Allam / STX        24-Jul-89       Freed allocated buffers
 *	Blaine Ailts/EDC	27-Jul-89	Fixed a problem with pointers --
 *						Now will handle images up to
 *						MAXBND with MAXCLASS clusters
 *						correctly
 *
 * PARAMETERS
 *
 *	None
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	nclust		short		I/O	Total # of clusters
 *	clknt [MAXCLS]	long		I/O	# of pixels in each cluster
 *	clmean [MAXCLS]
 *	       [MAXBND]	double		I/O	Cluster means for each band
 *	clstdv [MAXCLS]
 *	       [MAXBND]	double		I/O	Cluster std devs for each band
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	for each cluster
 *		if cluster has has enough members then
 *	       	      	copy cluster mean, std dev, and count into buffers
 *		else
 *			decrement total # of clusters
 *		end if
 *	end for
 *	copy all clusters in buffer back to original arrays
 *
 *****************************************************************************/
void isodel
(void)
{
	short	i, j,				/* cluster counter (old) */
		index;				/* cluster counter (new) */
	long	c_size,
                hclust,
                hclknt[MAXCLS];		/* hold array for clknt */
	float	*hclmean [MAXCLS],	/* hold array for clmean */
                *hclmean_matrix,
                *hclstdv_matrix,
		*hclstdv [MAXCLS];	/* hold array for clstdv */

      c_size = (long)(maxcls * totbnd * sizeof (float));
      if ((hclmean_matrix = (float *) malloc( c_size)) == NULL ||
          (hclstdv_matrix = (float *) malloc( c_size)) == NULL) {
          iso_error ("!Error allocating dynamic memory", "isoclass-alloc");
      }
      for (i = 0, j = 0; i < maxcls; i++, j += totbnd)
       {
        hclmean[i] = &hclmean_matrix[j];
        hclstdv[i] = &hclstdv_matrix[j];
       }

	/*
	 * copy to holding area all statistics that go with clusters
	 * with greater than or equal nmin members (it's packing it)
	 */
	index = 0;
        hclust = nclust;
	for (i = 0;  i < hclust;  i++)
		if (clknt [i] >= nmin)  {
			/* copy into new buffer */
			buff_copy ((char *)clmean [i], (char *)hclmean [index],
			       	   totbnd, (short) sizeof (float));
			buff_copy ((char *)clstdv [i], (char *)hclstdv [index],
				   totbnd, (short) sizeof (float));
			hclknt [index] = clknt [i];
			index++;
		}  else
			/* don't copy (delete it) */
 			nclust--;

	/* copy updated statistics back to original arrays */
	buff_copy ((char *)hclmean[0], (char *)clmean[0], totbnd * maxcls,
		   (short) sizeof (float));
	buff_copy ((char *)hclstdv[0], (char *)clstdv[0], totbnd * maxcls,
		   (short) sizeof (float));
	buff_copy ((char *)hclknt, (char *)clknt, maxcls, (short) sizeof (int));
	free (hclmean_matrix);
	free (hclstdv_matrix);
}
