#include "isoclass.h"
#include "protos.h"

void link_cluster
(
    int lo_clust,	/* number of lower cluster */
    int hi_clust	/* number of higher cluster */
)
/*****************************************************************************
 *
 * NAME	    	link
 *
 * PURPOSE	Links one cluster to another.  To show a link between two
 *		the chain value of the higher cluster contains the value
 *		of the lower cluster.  Of course, the lower cluster may be
 *		linked to yet another lower cluster.  If the higher cluster
 *		is already linked then we link the higher cluster to the
 *		middle cluster (higher of previous link and previous lower
 *		cluster) and link the middle cluster to the new lower cluster
 *		(lower of previous link and previous lower).  Recursion ends
 *		when we try to link a cluster to itself.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	lo_clust	short		I	Cluster #
 *	hi_clust	short		I	Cluster #
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	None
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	if lower cluster is same as higher cluster
 *		return
 *	end if
 *	if higher cluster is unlinked
 *		link to lower cluster
 *	else
 *		link higher cluster to new middle cluster
 *		link new middle cluster to new lower cluster
 *	end if
 *
 *****************************************************************************/
{
	short	old_link = chain_list [hi_clust];
				/* the current value of the higher cluster */

	/* check for end to recursion */
	if (lo_clust == hi_clust)
		return;

	if (old_link == UNLINKED)
		/* UNLINKED, so simply link it */
		chain_list [hi_clust] = lo_clust;
	else  {
		/* already linked, so use the higher of the two and recurse */
	       	chain_list [hi_clust] = max(lo_clust, old_link);
		link_cluster (min(lo_clust, old_link), max(lo_clust, old_link));
	}
}
