#include "isoclass.h"
#include "protos.h"

#define NEW_LINE  22

void print_chain
(
    int top
)
/*****************************************************************************
 *
 * NAME		print_chain
 *
 * PURPOSE	Print the chains to the screen and to the report file.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *      G. McConaughy / SASC    14-MAR-83       Original development
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam / SAR        31-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	top		short		I	Last chain #
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	nclust		short		I	Total # clusters so far
 *	print_file	FILE		I	File descriptor for report file
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	print the number of chains
 *	for each chain number (MAXCLS to top)
 *		print header
 *		for each cluster
 *			if cluster is a member of this chain
 *				if this fills up a line
 *					start a new line
 *				end if
 *				print this cluster number
 *			end if
 *		end for
 *	end for
 *
 *****************************************************************************/
{
	short	col_count = 0,		/* column count for output table */
		i,			/* index into chain_list */
		j;			/* cluster index */
	char	outbuf [80];		/* temporary output buffer (t_write) */


	fprintf (print_file, "\n%d chain(s)\n", top - MAXCLS);
	sprintf (outbuf, "%d chain(s)", top - MAXCLS);
	t_write (outbuf, T_STDCC);

	for (i = MAXCLS;  i < top;  i++)  {
	        col_count = 0;
		fprintf (print_file, "\nChain:  ");
		t_write ("Chain:  ", T_NULL);
		for (j = 0;  j < nclust;  j++)
			if (chain_list [j] == i)  {
				if (col_count++ > NEW_LINE)  {
				 	fprintf (print_file, "\n       ");
					t_write ("", T_STDCC);
					t_write ("        ", T_NULL);
					col_count = 0;
				}
				fprintf (print_file, "%3d", j + 1);
				sprintf (outbuf, "%3d", j + 1);
				t_write (outbuf, T_NULL);
			}
                t_write("", T_STDCC);
	}
}

