#ifndef PROTOS_H
#define PROTOS_H

/* assign.c */
void do_last_iteration
(
   double *clsumx[],		/* O: Sum of pixels in cluster */
   double *clsumsq[],		/* O: Squared sum of pixels in cluster */
   float *mean[],		/* I/O: Cluster mean for each band */
   double *scov[],		/* O: Each cluster's covariance matrix */
   double *min_clust,		/* I/O: Minimum cluster # */
   double *max_clust		/* I/O: Maximum cluster # */
);
void do_iteration
(
   double *clsumx[],		/* O: Sum of pixel vals in each clust */
   double *clsumsq[]		/* O: Squared sum - pix vals in clusts */
);
void assign
(
   int iter,			/* I: Current iteration # */
   double *clsumx[],		/* O: Sum of pixel vals in each clust */
   double *clsumsq[],		/* O: Squared sum - pix vals in clusts */
   double *scov[],		/* O: Each cluster's covariance matrix */
   double *min_clust,		/* I/O: Minimum cluster # */
   double *max_clust		/* I/O: Maximum cluster # */
);

/* chain.c */
void chain
(void);

/* chain_dist.c */
float chain_dist
(
   int i,			/* I: Cluster # */
   int j			/* I: Cluster # */
);

/* clstats.c */
void clstats
(
   double *clsumx[],		/* I: Sum of pixel values in clusters */
   double *clsumsq[],		/* I: Squared sum of pixel values */
   int iter,			/* I: Current iteration # */
   double *scov[]		/* I/O: Covariance matrix */
);

/* comb_chain.c */
short combine_chain
(void);

/* combine.c */
void clear_dist
(void);
void fig_dist
(
   int i,			/* I: Cluster # */
   int j			/* I: Cluster # */
);
short close_enuf
(
    int j			/* I: ??? */
);
void combine
(void);

/* decide.c */
void decide
(
   char *itertyp,		/* I/O: Tag telling if split or combine */	
   int iter,			/* I: Current iteration # */
   short *alter			/* I/O: Bool : at alternating stage yet? */
);

/* doline.c */
float abs_sum
(
   float *vector,		/* I: First element of vector */
   int length,			/* I: # of components in vector */
   int increment		/* I: Distance between components */
);
void offset_pix
(
   float *destination,		/* I/O: Destination array */
   double offset,		/* I: Offset for each pixel */
   int count			/* I: # samples to offset */
);
void doline
(
   float *clmean[],		/* I: Cluster means for each band */
   short cluster[],		/* O: Cluster # each pixel is assigned */
   long in_clust[MAXCLS]	/* O: # pixels assigned to each clust */
);

/* dyn_alloc.c */
void dyn_alloc
(
   double *scov[],		/* I: pointers to covariance matrix  */ 
   double *clsumx[],		/* I: Sum of pixel values in clusters */
   double *clsumsq[]		/* I: Squared sum of pixel values */ 
);

/* fin_cov.c */
void finish_cov
(
    double *scov[]		/* I/O: Covaraince matrix */
);

/* getpar.c */
void getpar
(
   char print_filename[CMLEN], 	/* O: output file name  */
   long bands[MAXIMG][MAXBND+1] /* O: selected band #'s */
);

/* iosetup.c */
void iosetup
(
   int iter,			/* I: Current iteration #  */
   long bands[MAXIMG][MAXBND+1] /* I: Selected band #'s */
);

/* isodel.c */
void isodel
(void);

/* link.c */
void link_cluster
(
   int lo_clust,		/* I: number of lower cluster */
   int hi_clust			/* I: number of higher cluster */
);

/* pixel.c */
void pixel_line_reset
(void);
void pixel_band_next
(void);
float pixel_next
(void);
float pixel_get
(
   int band,			/* I: Band on which pixel lies */
   int sample			/* I: Which pixel to get value of */
);

/* print_ch.c */
void print_chain
(
    int top			/* I: Last chain # */
);

/* rdstats.c */
void rdstats
(void);

/* setup.c */
void setup
(
    float *clmean[]		/* I/O: Cluster means for each band */
);

/* split.c */
void split
(void);

/* utility.c */
void iso_error
(
   char *message,		/* I: Error message and severity */
   char *key
);
short bytes_per_pixel
(
   int data_type		/* I: Tag that determines data type */
);
void buff_copy
(
   char *from,			/* I: Beginning of source buffer */
   char *to,			/* I: Beginning of destination buffer */ 
   long count,			/* I: # of elements to copy */
   int size			/* I: Size of each element */
);

/* wrstats.c */
void init_shapes
(void);
void wrstats
(
   double *scov[],		/* I: Each cluster's covariance matrix */
   float *clmean[]		/* I: Cluster means for each band */
);
#endif
