#include "isoclass.h"
#include "util.h"
#include "protos.h"

/*****************************************************************************
 *
 * NAME		iso_error
 *
 * PURPOSE	To display an error message and exit ISOCLASS if it is fatal.
 *		The first character of the error message determines whether
 *		or not the error is fatal.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *      Bani Allam               8-Aug-88       Retrocode
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	*message	char		I	Error message and severity
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	None
 *
 * RETURN VALUES
 *
 *	None
 *
 *****************************************************************************/
void iso_error
(
    char *message,
    char *key
)
{
	switch (*message++)  {
		case '?' :
		     	c_errmsg (message, key, &inf_err);
		     	break;
		case '!' : {
                        if (gd_out > 0) c_egclse((struct GDESC **)&gd_out);
                        if (print_file) fclose(print_file);
   	 		c_errmsg (message, key, &inf_err);
                        c_clean (&vblock, "isoclass", out, &count);
			break;
                        }
		case '$' : {
                        if (gd_out > 0) c_egclse((struct GDESC **)&gd_out);
                        if (print_file) fclose(print_file);
                        c_clean (&vblock, "isoclass", out, &count);
			}
 	}
}


/*****************************************************************************
 *
 * NAME		bytes_per_pixel
 *
 * PURPOSE	Determines the # of bytes that it takes to represent a pixel
 *		given its data type.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----	    	---	-----------
 *	data_type	short		I	Tag that determines data type
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	None
 *
 * RETURN VALUES
 *
 *	# (short) of bytes it takes to represent a pixel
 *
 *****************************************************************************/
short bytes_per_pixel
(
    int data_type
)
{
	switch (data_type)  {
		case EBYTE:
			return (BYTE_SIZE);
		case EWORD:
			return (WORD_SIZE);
		case ELONG:
			return (LONG_SIZE);
		case EREAL:
			return (REAL_SIZE);
	};
	return(-1); /* This return was put in to avoid a compiler error.
		       It it is not used as an error code. */
}                       


/*****************************************************************************
 *
 * NAME		buff_copy
 *
 * PURPOSE	To copy a segment of memory from one location to another.
 *
 * PROGRAM HISTORY
 *
 *	Programmer		Date		Reason
 *	----------		----		------
 *	Eric Ivancich / SAR	13-Aug-87	C conversion
 *
 * PARAMETERS
 *
 *	Name		Type		I/O	Description
 *	----		----		---	-----------
 *	*from		char		I	Beginning of source buffer
 *	*to		char		I	Beginning of destination buffer
 *	count		long		I	# of elements to copy
 *	size		short		I	Size of each element
 *
 * EXTERNAL VARIABLES REFERENCED
 *
 *	None
 *
 * RETURN VALUES
 *
 *	None
 *
 * ALGORITHM
 *
 *	multiply count by size to get total # of bytes
 *	copy over a byte at a time
 *
 *****************************************************************************/
void buff_copy
(
    char *from,
    char *to,
    long count,
    int size
)
{
	count *= size;
   	while (count--)
 		*to++ = *from++;
}
