/****************************************************************************
NAME:   GETPAR

PURPOSE:  Retreive parameters from TAE and do error checking

***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,          /* TAE parameter block */
    char *header_file,		/* host name of LANDSAT header file */
    char *gridname,			/* host name of output grid file */
    long *print,			/* print utility options */
    char *tmpfil,
    char *prtfile,			/* TAE name of output print file */
    long *msgtim
)
{
struct VARIABLE *var;           /* Pointer to a TAE variable in vblk */

long access = IREAD;		/* file access */
long fatal = -1;
long std_out;                   /* Standard output */
long zero = 0;

/* Initialize LAS processing
  -------------------------*/
c_inlas(vblock, &std_out);

/* Get the landsat header file name 
----------------------------------*/
if (c_getnam(vblock,"INFILE",&zero,header_file,"lhdr",&access) != E_SUCC)
   c_errmsg("Fatal error encountered","jscgrid-fatal",&fatal);

/* Get the output grid file name 
--------------------------------*/
access = IWRITE;
if (c_getnam(vblock,"OUTGRID",&zero,gridname,"grid",&access) != E_SUCC)
   c_errmsg("Fatal error encountered","jscgrid-fatal",&fatal);

/* Get the print option
-----------------------*/
strcpy(tmpfil,"jscgrid");
if (c_setprt(vblock,&print[0],&print[1],&print[3],tmpfil,prtfile) != E_SUCC)
   c_errmsg("Fatal error encountered","jscgrid-fatal",&fatal);

/* Get time between processing messages 
---------------------------------------*/
var = p_find(vblock, "$msgtim");
*msgtim = IVAL(*var, 0);

return;
}
