/******************************************************************************
FUNCTION:	CHECK_WINDOW

PURPOSE:	Checks the validity of the window specification.  If an
		error is detected, the window specification is changed
		to be the intersection of the window specification and
		the actual image.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"

void chk_window
(
    long *outspc,   /* I: output image file specifications    */
    long *window    /* I/O: window area of image to be copied */
)

{
long one = 1; 	/* non-fatal error code              */

/* If NL or NS is 0 then set them to the total number of lines and samples
--------------------------------------------------------------------------*/
if (window[NL] == 0)
    window[NL] = outspc[0];

if (window[NS] == 0)
    window[NS] = outspc[1];

/* If SL or SS is 0 then set them to start at 1,1
-------------------------------------------------*/
if (window[SL] == 0)
    window[SL] = 1;

if (window[SS] == 0)
    window[SS] = 1;

/* Validate the number of lines specified
-----------------------------------------*/
if (window[SL] + window[NL] - 1 > outspc[0])
    {
    c_errmsg("Specified window is too large","jscin-window",&one);
    window[NL] = outspc[0] - window[SL] + 1;
    }

/* Validate the specified number of samples
-------------------------------------------*/
if (window[SS] + window[NS] - 1 > outspc[1])
    {
    c_errmsg("Specified window is too large","jscin-window",&one);
    window[NS] = outspc[1] - window[SS] + 1;
    }

return;
}
/******************************************************************************
FUNCTION:	CHECK_BAND

PURPOSE: 	Checks the validity of the band specification.  If an error is
		detected in the band specification, the program continues by
		not copying the band that is in error.  The routine also sets
		up a boolean array indicating which bands are to be copied.
		This array is returned to the calling program.

ALGORITHM REFERENCES	none
*******************************************************************************/
void chk_bands
(
    long *bands,	/* I/O: bands to be processed within image        */
    long *numbands,	/* I: input image file specifications 	          */
    long *totbnd        /* I/O: total number of bands to be copied        */
)

{
long  index;		/* loop index                        		     */
long  one = 1;		/* Non-fatal error code			             */

char  msg[ERRLEN];	/* buffer space for error message    		     */


/* For each band specified, error check to insure the band exists on tape
-------------------------------------------------------------------------*/
for (index = 0; index < *numbands; index++)
    {
    if (bands[index])
        if ((index + 1) > *numbands)
        {
        sprintf(msg,"Band %d does not exist on tape",index+1);
        c_errmsg(msg,"jscin-band",&one);
        bands[index] = FALSE;
        }
    else
        *totbnd +=1;
    }

return;
}

