/******************************************************************************
FUNCTION:	GETPAR

PURPOSE:	Get the input parameters

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "jscin.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock, 	/* I: TAE parameter block                    */
    long *bands,		/* O: bands to be processed within image     */
    long *msgtim,		/* O: elapsed time between display msg       */
    long *window,		/* O: window of input image                  */
    long *minmax,	        /* O: min max flag                           */
    char hosout[][CMLEN],	/* O: Output host name          	     */
    char taeout[][CMLEN],	/* O: Output TAE name           	     */
    char *comment,		/* O: tape mount message to the operator     */
    long *wrtdsk        	/* O: ancillary files to read in             */
)

{
struct VARIABLE *val;	/* pointer to a TAE variable 	             */

long fatal = -1;	/* Fatal error code		             */
long index;		/* loop index                                */
long std_out;		/* fortran standard out LUN                  */
long temp;		/* temporary variable			     */

/* Initialize LAS
-----------------*/
c_inlas(vblock,&std_out);

/* Retrieve tape mount message supplied by user
----------------------------------------------*/
val = p_find(vblock,"COMMENT");
strcpy(comment,SVAL(*val,0));

/* Retrieve output image name
-----------------------------*/
if (c_getout(vblock,"OUT",(char (*)[256])taeout[0],
		(char (*)[256])hosout[0]) == E_FAIL)
    c_errmsg("Fatal error encountered","jscin-fatal",&fatal);

/* Retrieve window area to be copied
------------------------------------*/
val = p_find(vblock,"WINDOW");
if (val->v_count != 0)
    for (index = 0; index < 4; index++)
	window[index] = IVAL(*val,index);
else
    for (index = 0; index < 4; index++)
	window[index] = 0;

/* Error check WINDOW. Make sure it is greater than or equal to 1
----------------------------------------------------------------*/
if (val->v_count != 0)
    {
    for (index = 0; index < 4; index++)
        {
        if (window[index] < 1)
           c_errmsg("Value for WINDOW must be greater than or equal to 1",
	        	"jscin-window", &fatal);
        }
    }

/* Get the requested bands
-----------------------*/
for (index = 0; index < MAX_BAND;  index++)
    bands[index] = FALSE;
val = p_find(vblock, "BANDS");
if (val->v_count == 0)
    {
    for (index = 0; index < MAX_BAND; index++)
	bands[index] = TRUE;
    }
else
    {
    for (index = 0; index < val->v_count; index++)
	{
	temp= IVAL(*val,index) - 1;
	bands[temp] = TRUE;
	}
    }

/* Get requested ancillary files
--------------------------------*/
wrtdsk[0] = wrtdsk[1] = FALSE;
val = p_find(vblock,"WRTDSK");
for (index = 0; index < val->v_count; index++)
    {
    if (strcmp(SVAL(*val,index),"ANC") == 0)
	wrtdsk[0] = TRUE;
    else if (strcmp(SVAL(*val,index),"HDR") == 0)
	wrtdsk[1] = TRUE;
    }

/* Retrieve number of seconds between processing messages
---------------------------------------------------------*/
val = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*val,0);

/* Retrieve min/max flag
------------------------*/
val = p_find(vblock,"$MINMAX");
if (strcmp(SVAL(*val,0),"YES") == 0)
    *minmax = TRUE;
else
    *minmax = FALSE;
}
