/*******************************************************************************
FUNCTION:	UPDATE_DDR

PURPOSE:	Updates the DDR information for an image

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int update_ddr
(
    char hosout[][CMLEN],	/* I: Host file name			*/
    long *window,		/* I: Input window			*/
    unsigned char *hdr_buf,     /* I: header buffer read from tape      */
    long *totbnd		/* I: Total number of bands to process	*/
)

{
struct DDR ddr;		/* DDR structure			*/
struct BDDR bddr[4];	/* Band DDR structure			*/

long band;		/* Band looping variable		*/
long day;		/* Day of first scan			*/
long hr;		/* hour of first scan			*/
long minutes;		/* minutes of first scan		*/
long month;		/* month of first scan			*/
long osec;		/* Seconds of first scan		*/
long satid;		/* Satellite id				*/
long temp;		/* temp value				*/
long year;		/* Year of first scan			*/

char hrmn[5];		/* Hour and minute of first scan	*/
char instrument[9];	/* inustrument 				*/

static char *cmnth[13] = {"dum","jan","feb","mar","apr","may","jun","jul","aug",
"sep","oct","nov","dec"};

/* Get the date and time of first scan from the header
-----------------------------------------------------*/
osec =  (long)hdr_buf[74];
minutes  =  (long)hdr_buf[75];
hr  =  (long)hdr_buf[76];
day =  (long)hdr_buf[77];
month =  (long)hdr_buf[78];
year =  (long)hdr_buf[79];

sprintf(hrmn,"%d%d",hr,minutes);
squeeze(hrmn,4);

/* Get sensor specifics
-----------------------*/
temp = 8;
c_eb2asc((char *)&hdr_buf[52],&temp);
strncpy(instrument,(const char *)&hdr_buf[52],temp);
instrument[7] = NULL;
satid =  ((long)hdr_buf[64] * 256 + (long)hdr_buf[65]);

/* Get the DDR information from the image file
----------------------------------------------*/
if (c_getddr(hosout[0],&ddr) != E_SUCC)
    return(E_FAIL);

/* Initialize the validity flag
------------------------------*/
ddr.valid[DDINCV] = VALID;

/* Update the values that are known
-----------------------------------*/
ddr.master_line   = window[SL] - 1;
ddr.master_sample = window[SS] - 1;
ddr.line_inc   = 1.0;
ddr.sample_inc = 1.0;

/* Update the Band DDR information
----------------------------------*/
for (band = 1; band <= *totbnd; band++)
    {
    /* Get the Band DDR information from the image
    ----------------------------------------------- */
    if (c_getbdr(hosout[0],&bddr[band-1],&band) != E_SUCC)
        return(E_FAIL);
    
    /* Update the Band DDR information
    ----------------------------------*/
    sprintf(bddr[band-1].source,"Landsat %d",satid);
    strcpy(bddr[band-1].instrument,instrument);
    
    /* Update the date and time
    ---------------------------*/
    sprintf(bddr[band-1].date,"%d-%s-%d",day,cmnth[month],year);
    sprintf(bddr[band-1].time,"%s:%d",hrmn,osec);
    }

/* Put the updated DDR information to the image
-----------------------------------------------*/
if (c_putddr(hosout[0],&ddr) != E_SUCC)
    return(E_FAIL);

/* Put the updated Band DDR information to the image
----------------------------------------------------*/
for (band = 0; band < *totbnd; band++)
    if (c_putbdr((char *)hosout,&bddr[band]) != E_SUCC)
        return(E_FAIL);

return(E_SUCC);
}

