/******************************************************************************
FUNCTION:	JULDATE

PURPOSE:	Conver the julian date to a standard date. 

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 7.0       05/97    J. Willems		initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include <string.h>
#include "las.h"		/* LAS include			       */
#include "util.h"		/* LAS include			       */

int main()
{
struct PARBLK   vblock;        /* TAE parameter block                  */
struct VARIABLE *var;           /* pointer to a TAE variable in vblk    */

char *sptr[1];                  /* pointer array for updating vblock    */
char jdate[6];                  /* five digit julian date               */
char year[3];                   /* two digit year                       */
char date[4];                   /* three digit julian day               */
char outdate[9];                /* output date                          */
long std_out;                   /* Standard output                      */
long yy;                        /* year                                 */
long dd;                        /* date                                 */
long month;                     /* month                                */
long day;                       /* day                                  */

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);

/*
  Get the parameters
*/
var = p_find(&vblock, "JDATE");
strcpy(jdate, SVAL(*var, 0));

if ( strlen( jdate ) != 5 )
    {
    char msg[CMLEN];
    sprintf( msg, "Julian date '%s' must be 5 characters", jdate );
    c_lasmsg( TRUE,"juldate-err", msg );
    return E_FAIL;
    }

strncpy(year, jdate, 2);
year[2] = '\0';
strncpy(date, &jdate[2], 3);
date[3] = '\0';

yy = atoi(year);
dd = atoi(date);
c_julgre(&yy, &dd, &month, &day);

sprintf(outdate, "%02ld %02ld %02ld", month, day, yy);

sptr[0] = outdate;
q_string(&vblock,"DATEOUT",1,sptr,P_UPDATE);
q_out(&vblock);
return E_SUCC;
}
