
#include "karlov.h"

void calc_scale
(
    char hosin[][CMLEN],      /* input image host names             */
    long *num_inimages,       /* number of input images             */
    long num_image_bands[],   /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each      */
    long wind[][4],           /* input image windows                 */
    long *num_inbands,        /* total no. of input bands            */
    long *num_outbands,       /* number of output bands              */
    float *eig_vec[],          /* eigen vectors                       */
    long components[],        /* components in out image            */
    long sub,                 /* subtract option                     */
    float smean[],             /* mean vector                         */
    float *scale,              /* factor for sacling output image   */
    float *offset             /* offset for output image scaling   */
)
/*
**************************************************************************
NAME: FINDMNMX 

PURPOSE: Calculate the scale and offset required to convert the output 
         image to BYTE. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   3.3         23-sep-93   D. Bridges         New 
   6.1	       24-May-94   D. Eichacker	      Added opt[]

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Open input images
   Group the images
   Process the input image and calculate the minimum and maximum pixel
      values for the output image
   Compute the scale and offset for the linear scaling which maps
      the calculated output minimum and maximum to 0 and 255 respectively
   Close the images
**************************************************************************
*/
{

double	outmax,		             /* input image maximum values          */
	outmin;        	             /* input image minimum values          */
long	access,                      /* file access type                   */
	bufsiz,                      /* image i/o buffer size              */
	n, nib,
	stat,                        /* return status                      */
	type;                        /* file type                          */
long    zero = 0;

static long opt[3]={0,0,0};	      /* optional parameter array for EOPENS */

float	*ip,                          /* image i/o input and output buffer */
                                      /* pointers                          */
	sum;

long	i, ii, j, n2,
	line,                         /* line number                       */
	msg_flag = 0,                 /* flag for processing message       */
        fatal_err = -1;

/*
open input image files
----------------------
*/
access = IREAD;
type = EREAL;
for (n = 0, nib = 0; n < *num_inimages; n++)
   {
   stat = c_eopens( &fdesc[n], hosin[n], image_band_num[n], &wind[0][SL], 
                   &wind[0][SS], &wind[0][NL], &wind[0][NS], &ipoint[nib], 
                   &access, &type, &zero, opt);
   if (stat != E_SUCC )
      c_errmsg("Error opening imput image", "karlov-eopens",&fatal_err);

   nib += num_image_bands[n];
   }  

fdesc[++n] = 0;

/*
set up group containing input bands
-----------------------------------
*/
if ((iobuf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
      c_errmsg("Error allocating dynamic memory", "karlov-eopens",&fatal_err);
c_egroup (fdesc, &gdesc, iobuf, &bufsiz);

/*
initialize and enable processing message utility
------------------------------------------------
*/
printf("\n\n");
c_inittm ("KARLOV", "Calculating slope and offset", "line", 
           "bands", " ", &wind[0][NL], num_outbands, &msgtim, &msg_flag);
curent = 1;
c_startm();

/*
initialize output minimum and maximum values 
--------------------------------------------
*/
    outmin = .1e30;                    /* arbitraty high value */
    outmax = -.1e30;                   /* arbitraty low value */
/*
process each line
-----------------
*/
for (line = 0; line < wind[0][NL]; line++, curent++)
   {
   c_estep( &gdesc);
   for (n2 = 0; n2 < wind[0][NS]; n2++)
      {
      for (i = 0; i < *num_outbands; i++)
         {
         sum = 0.0;
         ii = components[i]-1;
         for (j = 0; j < *num_inbands; j++)
            {
	    ip = (float *) &iobuf[(ipoint[j] + n2) * sizeof( float)];
            if(sub == 1)
               sum += eig_vec[ii][j] * (*ip - smean[j]);
            else
               sum += eig_vec[ii][j] * (*ip);
            }
            if (sum < outmin) 
               outmin = sum;
            if (sum > outmax)
               outmax = sum;
         }
      }
   }
c_estep( &gdesc);

/*
   compute scaling factor and offset
   ---------------------------------
*/
if (outmax == outmin)
   c_abort (&block, "Division by zero attempted", "karlov-zerodiv", 
            "karlov", out_files, &num_outfiles);
   *scale = 255. / (outmax - outmin);
   *offset = - (*scale * outmin);

/*
close image files
-----------------
*/
c_egclse( &gdesc);

}

