#include "karlov.h"

void dyn_alloc
(
    long num_inbands,       /* number of input image bands    */
    double *cov[],            /* pointers to covariance matrix  */
    float *eig_vec[]        /* pointers to eigen vectors      */
)
/*
*********************************************************************
NAME: DYN_ALLOC

PURPOSE: Allocate storage for covariance matrix and eigen vectors

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0         15-aug-88   k. a. narayanan    

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

*********************************************************************
*/
{
long	buf_size,                  /* size of cov/eigen matrix       */
        i;
long    fatal_err = -1;

/*
allocate storage for covariance matrix and eigen vectors
--------------------------------------------------------
*/
buf_size = num_inbands * num_inbands;

if (((cov_matrix = (double *) malloc( buf_size * sizeof(double)) ) == NULL) ||
    ((eigen_vectors = (float *) malloc( buf_size * sizeof(float)) ) == NULL))
    c_errmsg( "Error allocating dynamic memory", "karlov-alloc", &fatal_err);

/*
set up pointers for covariance matrix and eigen vectors
-------------------------------------------------------
*/
for (i = 0; i < num_inbands; i++)
   {
   cov[i] = &cov_matrix[i*num_inbands];
   eig_vec[i] = &eigen_vectors[i*num_inbands];
   }
}
