#include <math.h>
#include "karlov.h"

void eigvec
(
    char imgname[][CMLEN],      /* image name                    */
    long mode,                  /* processing mode               */
    float smean[],               /* mean vector                   */
    double *cov[],                /* covariance matrix             */
    long nbands,                /* number of input image bands   */
    float *eig_vec[],            /* eigen vectors                 */
    char class[],               /* class name                    */
    long print[]               /* print option                  */
)

/*
******************************************************************
NAME: EIGVEC

PURPOSE: Computes eigen vectors of covariance matrix

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   3.3         26-mar-93   D. Etrheim         Modified to use the jacobi method
                                              to calculate the eigenvectors
   2.0         15-aug-88   k. a. narayanan    conversion to 5.0
   1.0          7-dec-82   k. a. narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Call TRIDIA to convert covariance matrix to tridiagonal form
   Call EGVCVL to obtain eigen values and vectors
   Call matrans to transpose eigen vectors
   Write the eigen values and vectors and other eigen value statistics 
      to the user's terminal, line printer, or output file
   Compute the product of the eigen vector matrix and the mean vector and
      write the product vector to output file
   Close output file

******************************************************************

*/
{
float	eig_val[MAXBND];               /* eigen values                  */
/*float	doff[MAXBND];*/
float	output[3][MAXBND];             /* temp for eigen stats          */
float	tm[MAXBND];                    /* product of mean and eig       */
float	sum;
float	cumper;

long	i, j, k;
/*long	err;*/
long nrot;

char	buf[200];                      /* buffer for output text       */
static char space[] = " ";

/*
write output header
-------------------
*/
sprintf( buf, "\n Input image: %s", imgname[0]);
c_txtout( buf, print, fd);
if (mode  ==  1)
   c_txtout( "\n Using IMAGE statistics", print, fd);
else if(mode  ==  2)
   {
   sprintf( buf, "\n Using statistics for SINGLE CLASS: %s", class);
   c_txtout( buf, print, fd);
   }
else
   c_txtout( "\n Using TOTAL CLASS statistics", print, fd);

/*
compute eigenvalues and eigen vectors and arrange them in 
descending order
---------------------------------------------------------
*/
nrot = 0;
jacobi(cov,nbands,eig_val,eig_vec,&nrot);
egvorder (eig_val, eig_vec, nbands, nbands);

/*
transpose eigen vectors and display them
----------------------------------------
*/
matrans (eig_vec, eig_vec, nbands, nbands);

for (i = 0; i < nbands; i++)
   {
   sprintf( buf, "\n Eigen Vector  %2d  :", i+1);
   c_txtout( buf, print, fd);

   for (k = 0; k < nbands; )
      {
      buf[0] = '\0';
      for (j = 0; j < 4; j++)
         {
         sprintf( &buf[strlen( buf)], "%15.8f", eig_vec[i][k]);
         if (++k == nbands) break;
         }
      c_txtout( buf, print, fd);
      }
   }

/*
product of mean vector and eigen matrix
---------------------------------------
*/
for (i = 0; i < nbands; i++)
   for (j = 0, tm[i] = 0.0; j < nbands; j++)
      {
      tm[i] = tm[i] + eig_vec[i][j] * smean[j]; 
      }

c_txtout( "\n\n Product of Mean Vector and Eigen Vector Matrix :", print, fd);
for (i = 0; i < nbands; ) 
   {
   buf[0] = '\0';
   for (j = 0; j < 4; j++)
      {
      sprintf( &buf[strlen( buf)], "%15.8f", tm[i]);
      if (++i == nbands) break;
      }
   c_txtout( buf, print, fd);
   }
/*
compute the sum of the eigen values, change negative values
-----------------------------------------------------------
*/
for (i = 0; i < nbands; i++) eig_val[i] = (float)fabs((double)eig_val[i]);

sum = 0.0;

for (i = 0; i < nbands; i++)  sum += eig_val[i];

/*
fill array output  with: eigen values (row 1),percent
variation (row 2), and cumulative percent var. (row 3)       
--------------------------------------------------------
*/
cumper = 0.0;
for (i = 0; i < nbands; i++)
   {
   output[0][i] = eig_val[i];
   output[1][i] = (output[0][i] /sum ) * 100;
   output[2][i] = output[1][i] + cumper;
   cumper = output[2][i];
   }  

sprintf( buf, "\n\n%11sEigenvalue%11sPercent of%11sCumulative", 
         space, space, space);
c_txtout( buf, print, fd);
sprintf( buf, "%32sTotal Data%11sPercent", space, space);
c_txtout( buf, print, fd);
sprintf( buf, "%32sVariation%12sVariation", space, space);
c_txtout( buf, print, fd);
sprintf( buf, "%11s----------%11s----------%11s----------", 
         space, space, space);
c_txtout( buf, print, fd);

/*
write output and close output file
----------------------------------
*/
for (j = 0; j < nbands; j++)
   {
   sprintf (buf, "%4d", j+1);
   sprintf (&buf[strlen( buf)], "%7s%10.4f", space, output[0][j]); 
   sprintf (&buf[strlen( buf)], "%11s%10.2f", space, output[1][j]);
   sprintf (&buf[strlen( buf)], "%11s%10.2f", space, output[2][j]);
   c_txtout( buf, print, fd);
   }  

}

