
#include "karlov.h"

void filspec
(
    char hosin[][CMLEN],      /* input image host names             */
    char hosout[],            /* output image host name             */
    long *num_inimages,       /* number of input images             */
    long num_image_bands[],   /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each      */
    long *num_outbands,        /* number of output image bands       */
    long wind[][4],           /* input image windows                */
    long out_type             /* output image data type             */
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   3.2         23-jul-92   D. Etrheim         Set comb_flag correctly on call
                                              to c_upddr.  Write correct proc
                                              name to history file.  Change
                                              parm name for standards.
   2.0         15-aug-88   k. a. narayanan    conversion to 5.0
   1.0          7-dec-82   k. a. narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   If output data type is BYTE
      Get or compute minimum and maximum values of input image 
   Open input and (optionally) output images
   Group the images

**************************************************************************
*/
{
double	line_inc = 1.0,              /* line and sample increments for     */
	samp_inc = 1.0;              /* output image ddr                   */

long	access,                      /* file access type                   */
	bufsiz,                      /* image i/o buffer size              */
	comb_flag,                   /* image combination flag             */
	n, nib,
	stat,                        /* return status                      */
	type;                        /* file type                          */
long    zero = 0;
long    one = 1;


/*
open input and output image files
---------------------------------
*/
access = IREAD;
type = EREAL;
for (n = 0, nib = 0; n < *num_inimages; n++)
   {
   stat = c_eopens( &fdesc[n], hosin[n], image_band_num[n], &wind[0][SL], 
                   &wind[0][SS], &wind[0][NL], &wind[0][NS], &ipoint[nib], 
                   &access, &type, &zero, &zero);
   if (stat != E_SUCC )
      {
      if (fd) fclose (fd);
      c_abort (&block, "Error opening input image", "karlov-open", "karlov",
               out_files, &num_outfiles);
      }

   nib += num_image_bands[n];
   }  

access = IWRITE;
if (out_type == 1) 
   type = EBYTE;
else 
   type = EREAL;
stat = c_eopens (&fdesc[n], hosout, num_outbands, &one, &one,
               &wind[0][NL], &wind[0][NS], opoint, &access, &type, 
               &zero, &zero);
if (stat != E_SUCC )
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error opening output image", "karlov-outopen", "karlov",
               out_files, &num_outfiles);
   }

/* Set comb_flag based on number of bands output */
if (*num_outbands != nib)
   comb_flag = COMB;
else
   comb_flag = NOCOMB;

c_upddr (&block, hosin, hosout, num_inimages, wind, image_band_num, 
         num_image_bands, &line_inc, &samp_inc, num_outbands, &comb_flag);

strcpy (out_files[num_outfiles++], hosout);

fdesc[++n] = 0;

/*
set up group containing input and output bands
----------------------------------------------
*/
if ((iobuf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error allocating dynamic memory", "karlov-alloc", "karlov",
            out_files, &num_outfiles);
   }
c_egroup (fdesc, &gdesc, iobuf, &bufsiz);
}
