#include "karlov.h"

void getpar
(
    char lasin[][CMLEN],      /* input image las name             */
    char hosst[],             /* stats file host name             */
    char hosin[][CMLEN],      /* input image host names           */
    char lasout[][CMLEN],            /* output image las name            */
    char hosout[][CMLEN],            /* output image host name           */
    long *mode,               /* processing mode parameter        */
    char class[],             /* class name parameter             */
    long *num_inimages,       /* number of input images           */
    long num_image_bands[],   /* number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* band numbers in each    */
    long *num_inbands,        /* total number of input image bands   */
    long *num_outbands,       /* number of output image bands     */
    long wind[][4],           /* input image windows              */
    long components[],        /* components for output image      */
    long *sub,                /* subtract option flag             */
    long *out_type,           /* output image data type           */
    long *termtran,           /* trans matrix option flag         */
    long print[],             /* print parameter option           */
    char print_file[]        /* output file                        */
)
/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   2.0         23-nov-88   k. a. narayanan    conversion to 5.0
   1.0          7-dec-82   k. a. narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize las and start the CPU clock
   Get input image parameters
   Get Transformation input option TRANSRC
   If user wants to use a statistics file for transformation 
      Get statistics file name
      If not specified issue error message and abort
   If TRANSRC=FILE get PRINT option and open output file 
   Get number of components (OUTBANDS) in output image
   If (OUTBANDS = 0)
      If TRANSRC is terminal, issue message and abort
   Else
      If (OUTBANDS > no. of input image bands)
         issue error message and abort
      Get output image name, data type, and components
   Get mean subtraction option SUBMEAN 
   Get STATMODE parameter
   If mode is single class 
      Get CLASS name; if not specified issue error message and abort
   Get globals

***************************************************************************
*/
{
struct VARIABLE	*var;               /* pointer to a variable in TAE block */

long	access,                     /* access type for getnam             */
	flag[MAXIMG][2],            /* flag for subwindows                */
	i,
	imglimit = MAXIMG,          /* maximum number of images           */
	num_comps,                  /* number of components specified     */
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */
long    fatal_err = -1;
long    zero = 0;



char	minmax[4],                  /* minmax option                      */
	fileutil_file[CMLEN],       /* file for fileutil                  */
	str[8],                     /* temporary string buffer            */
	subtr[4],                   /* subtract parameter                 */
	tsource[6];                 /* trans matrix input parameter       */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get tsource, sfin parameters
----------------------------
*/
var = p_find(&block, "TRANSRC");
strcpy (tsource, SVAL(*var, 0));

if (tsource[0] == 'T')  
   *termtran = TRUE;
else
   *termtran = FALSE;

if (! (*termtran)) 
   {
   access = IREAD;
   stat = c_getnam (&block, "INSTAT", &zero, hosst, "stat", &access);
   if (stat != E_SUCC) 
      c_errmsg ("Fatal error encountered", "karlov-fatal", &fatal_err);
   if (strlen(hosst) == 0)
      c_errmsg ("No statistics file specified", "karlov-sfin", &fatal_err);
   } 
/*
get and validate input image parameters
---------------------------------------
*/
stat = c_getin (&block, "IN", &zero, lasin, hosin, num_inimages, wind, 
                image_band_num, num_image_bands, num_inbands, &imglimit, 
                &same_window, flag);

if (stat != E_SUCC) 
      c_errmsg ("Fatal error encountered", "karlov-fatal", &fatal_err);

if (*num_inbands  <  2) 
   c_errmsg ("No. of input bands should be greater than 1", "karlov-nbands",
             &fatal_err);

if (same_window == 0)
    c_errmsg ("Image windows not same size", "karlov-samewindow", &fatal_err);

/*
get number of components desired
--------------------------------
*/
var = p_find (&block, "OUTBANDS");
*num_outbands = IVAL(*var, 0);

if (*num_outbands == 0) 
   {
   if (*termtran)
      c_errmsg ("Zero output components specified for user defined transformation",
               "karlov-zerocomp", &fatal_err);
   }
else
   {
   if (*num_outbands  >  *num_inbands)
      c_errmsg ("No. of output bands should be <= no. of input bands",
                "karlov-obands", &fatal_err);
/*
   get and validate output image parameters
   ----------------------------------------
*/
   stat = c_getout (&block, "OUT", lasout, hosout);
   if (stat != E_SUCC || strlen (lasout[0]) == 0)
      c_errmsg ("Fatal error encountered", "karlov-fatal", &fatal_err);

   var = p_find (&block, "ODTYPE");
   strcpy (str, SVAL(*var, 0));
   *out_type = (str[0] == 'B' ? 1 : 4);

   var = p_find (&block, "COMPS");
   if ((num_comps = (*var).v_count) == 0)
      for (i = 0; i < *num_outbands; i++) components[i] = i+1; 
   else
      {
      if (num_comps != *num_outbands)
         c_errmsg ("Number of components specified not equal to OUTBANDS",
                   "karlov-ncomps", &fatal_err);
      for (i = 0; i < num_comps; i++) components[i] = IVAL(*var, i); 
      }
   }  

/*
Get other parameters
--------------------
*/
var = p_find (&block, "SUBMEAN");
strcpy (subtr, SVAL(*var, 0));
if ((subtr[0]  ==  'Y'))
   *sub = 1;
else
   *sub = 0;

var = p_find(&block, "STATMODE");
strcpy (str, SVAL(*var, 0));
if (str[0] == 'I') *mode = 1;
else if (str[0] == 'S') *mode = 2;
else *mode = 3;

if (*mode  ==  2) 
   {
   var = p_find(&block, "CLASSNAM");
   strcpy (class, SVAL(*var, 0));
   if (var->v_count == 0) 
      c_errmsg("No class name specified", "karlov-class", &fatal_err);
   } 
/*
Get print option and open print file 
------------------------------------
*/
num_outfiles = 0;
strcpy (fileutil_file, "karlov");
stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
          print_file);
if (stat != E_SUCC)
   c_errmsg("Fatal error encountered","karlov-fatal",&fatal_err);

if ((print[LP]) || (print[FIL]))
   {
   fd = fopen (fileutil_file, "w");
   strcpy (out_files[num_outfiles++], fileutil_file);
   }
else
   fd = NULL;
/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;
}
