#include "karlov.h"

int jacobi
(
    double *a[],
    long n,
    float d[],
    float *v[],
    long *iterations
)
{
    long 	end,i,l,m;
    double 	anorm, anormx;
    double 	sinx, sinx2;
    double 	cosx, cosx2;
    double 	sincs;
    double 	thresh;
    float 	*vptr;
    double 	*aptr, *aim, *ail;
	float	*lptr, *mptr;
    double 	range, x, y;
    long	fatal_err = 1;
    long 	iter;

    range = 1.0e-12;


    vptr = v[0];
    for (l=0;l<n;l++) 
        {
        for (m=0;m<n;m++,vptr++)
            *vptr = (m == l) ? 1.0 : 0.0;
        }

    for (l=0;l<n;l++) {
        d[l]=a[l][l];
        }

    anorm=0.0;
    
    for (l=0;l<n;l++) 
        { 
        aptr = &a[l][l+1];
        for (m=l+1;m<n;m++,aptr++)
            anorm += ((*aptr)*(*aptr));
        }

    if (anorm == 0.0) {
        return (-1);
        }

    anorm = sqrt(2.0*anorm);
    anormx = anorm*range/n;

    thresh = anorm;
    iter=0;
    do 
        {
        thresh /= n;
        do
            {
            end = 0;
            lptr = d;
            for ( l=0; l < n-1; l++, lptr++)
                {
                aptr = &a[l][l+1];
            	mptr = lptr+1;
                for ( m=l+1 ; m < n ; m++ , aptr++, mptr++ ) 
                    {
                    if ( fabs(*aptr) >= thresh ) 
                        {
                        end = 1;
                        x = 0.5 * ( d[l] - d[m]);
                        x = 0.5 * ( (*lptr) - (*mptr));
                        y = -(*aptr)/ sqrt((*aptr)*(*aptr) + x*x);
                        if ( x < 0.0)
                            y = -y;

                        sinx = y / sqrt(2.0*(1.0+sqrt(1.0-y*y)));
                        sinx2 = sinx * sinx;
                        cosx2 = 1.0 - sinx2;
                        cosx = sqrt(cosx2);
                        sincs = cosx * sinx;

                        for ( i = 0; i < n; i++) 
                            {
                            if((i != l)&&(i != m))
                                {
                                aim = (i > m) ? &a[m][i] : &a[i][m];
                                ail = (i > l) ? &a[l][i] : &a[i][l];

                                x = (*ail) * cosx - (*aim) * sinx;
                                (*aim) = (*ail) * sinx + (*aim) * cosx;
                                (*ail) = x; 
                                }
                            x = v[i][l] * cosx - v[i][m] * sinx;
                            v[i][m] = v[i][l] * sinx + v[i][m] * cosx;
                            v[i][l] = x;
                            }
                        x = 2.0 * (*aptr) * sincs;
                        y = (*lptr) * cosx2 + (*mptr) * sinx2 - x;
                        x = (*lptr) * sinx2 + (*mptr) * cosx2 + x;
                        *aptr = ((*lptr) - (*mptr)) * sincs + 
                                (*aptr) * (cosx2 - sinx2);
                        (*lptr) = y;
                        (*mptr) = x;
                        }	/* end of if ( a[l][m] >= thresh )  */
                    }	/* end of for ( m=l+1 ; m < n ; m++)  */
                }    /* end of for ( l=0 ; l < n ; l++)  */
            } while ( end );
        iter++;
        } while ( (thresh > anormx) && (iter < 100) );

    if ( iter >= 100)
        c_errmsg("Too many iterations", "karlov-jacobi", &fatal_err);
    *iterations = iter;
    return(0);
}
