#include "karlov.h"

void prcomp
(
    char hosout[],            /* output image host name              */
    long *num_inbands,        /* total no. of input bands            */
    long *num_outbands,       /* total no. of output bands           */
    long components[],        /* components in output image          */
    long wind[][4],           /* input image windows                 */
    float *eig_vec[],          /* eigen vectors                       */
    long sub,                 /* subtract option                     */
    long out_type,            /* output image data type              */
    float smean[],             /* mean vector                         */
    long print[],             /* print option                        */
    float *scale,              /* factor for sacling output image   */
    float *offset             /* offset for output image scaling   */
)

/*
**************************************************************************
NAME: PRCOMP

PURPOSE: Compute the principal components and output them

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   3.3         23-sep-93   D. Bridges         Removed approximation of output
 					      min and max and replaced it with
 					      calc_scale.c for outtype of BYTE
   2.0         23-nov-88   k. a. narayanan    conversion to 5.0
   1.0          7-dec-82   k. a. narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   The following procedures are replaced by calc_scale.c
   --------------------------------------------------------------------------
         If scaling to byte image is required estimate the output maximum and 
             minimum pixel values from the input maximum and minimum pixel 
             values and the values of the eigen vector matrix
         Compute the scale and offset for the linear scaling which maps
             the estimated output minimum and maximum to 0 and 255 respectively
   -------------------------------------------------------------------------- 
   Perform transformation line by line:
      Subtract mean if needed
      If output image is byte perform scaling
      If output minmax is requested  update minmax values
   If output minmax is requested save the minmax values in the DDR
   Close image files
      
**************************************************************************
*/
{
static	double	output_max[MAXBND],   /* output image maximum values       */
	output_min[MAXBND];           /* output image minimum values       */

float	*ip, *op,                     /* image i/o input and output buffer */
                                      /* pointers                          */
	sum;

long	i, ii, j, n2,
	line,                         /* line number                       */
	msg_flag = 0,                 /* flag for processing message       */
	out_pixsize;                  /* output pixel size                 */

char	buf[200];                     /* buffer for output text            */

/*
if output image is byte calculate scale and offsets by estimating
output maximum and minimum values for all bands from input maximum
and minimum values and the transformation matrix

Note that eigen vectors are in ascending order and columnwise
------------------------------------------------------------------
*/

/*
initialize and enable processing message utility
------------------------------------------------
*/
printf("\n\n");
c_inittm ("KARLOV", "Processed", "line", "bands", " ", &wind[0][NL], 
         num_outbands, &msgtim, &msg_flag);
curent = 1;
c_startm();

/*
initialize output minimum and maximum arrays if needed
------------------------------------------------------
*/
if (min_max) 
   {
   c_inmnmx (&out_type, num_outbands, output_min, output_max);
   if (out_type == EBYTE) out_pixsize = 1;
   else out_pixsize = sizeof (float);
   }
/*
process each line
-----------------
*/
for (line = 0; line < wind[0][NL]; line++, curent++)
   {
   c_estep( &gdesc);
   for (n2 = 0; n2 < wind[0][NS]; n2++)
      {
      for (i = 0; i < *num_outbands; i++)
         {
         sum = 0.0;
         ii = components[i]-1;
         for (j = 0; j < *num_inbands; j++)
            {
	    ip = (float *) &iobuf[(ipoint[j] + n2) * sizeof( float)];
            if(sub == 1)
               sum += eig_vec[ii][j] * (*ip - smean[j]);
            else
               sum += eig_vec[ii][j] * (*ip);
            }
/*
         scale output values for byte image
         ----------------------------------
*/
         if(out_type == 1)
            {
            sum = sum * *scale + *offset;
            if (sum < 0) sum = 0;
            else if (sum > 255) sum = 255;

            iobuf[opoint[i] + n2] = sum;
            }
         else
            {
            op = (float *) &iobuf[(opoint[i] + n2) * sizeof (float)];
            *op = sum;
            }  
         }
      }
/*
   compute output minimum and maximum values if requested
   ------------------------------------------------------
*/
   if (min_max)
      {
      for (i = 0; i < *num_outbands; i++)
          c_pxmnmx (&iobuf[opoint[i] * out_pixsize], &wind[0][NS], 
                    &output_min[i], &output_max[i], &out_type);
      }
   }
c_estep( &gdesc);

c_stoptm();

/*
update minimum and maximum values of output image ddr when requested
--------------------------------------------------------------------
*/
if (min_max)
   c_svmnmx (hosout, num_outbands, output_min, output_max, "YES");

c_txtout( "\n Principal components used to create output image are:", 
        print, fd);

for (i = 0; i < *num_outbands; )
   {
   buf[0] = '\0';
   for (ii = 0; ii < 15; ii++)
      {
      sprintf (&buf[strlen( buf)], "%4d", components[i]);
      if (++i == *num_outbands) break;
      }
   c_txtout( buf, print, fd);
   }
c_txtout( "\n", print, fd);

/*
close image files 
-----------------
*/
c_egclse( &gdesc);

}

