#include "kmeans.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void assign
(
    long iter,              /* I: iteration number                 */
    long *maxit,            /* I: maximum no. iterations           */
    float *clmean[],        /* I: class means                      */
    long *nchnge,           /* O: no. pixels changing assignments  */
    float clknt[],          /* O: no. of pixels in each cluster    */
    double *clsumx[],       /* O: sums of pixels                   */
    double *clsumsq[],      /* O: sums of squares of pixels        */
    long *nl,               /* I: no. lines in input image         */
    long *np,               /* I: no. pixels/line in input image   */
    long nclust,            /* I: no. clusters desired             */
    long nbands,            /* I: no. bands in input image         */
    long num_samps          /* I: total num of samples in image    */
)
/*
**********************************************************************

NAME: ASSIGN 

PURPOSE:
   Reads input image and performs cluster assignments on a line by 
   line basis, and writes to the output image

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         25-Jan-83  G. Mcconaughy
    2         23-Sep-88  K. Narayanan
    3         16-Feb-89  K. Narayanan      Conversion to 5.0
    4         20-Oct-99  D. Hames          Add variable num_samps and use to
					   replace MAXNS used for allocating
					   memory for itemp.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   For number of lines in input image 
      Read image line
      Call DOLINE to make a line of cluster assignments
      Accumulate counts and sums for cluster statistics
   Close input and output images

**********************************************************************
*/
{
double	xsq;

float	temp[MAXCLS];             /* temp buffer for cluster counts     */

long	i, j, k, line, nx, type;
long	msg_flag = ONECAT;

unsigned char	*itemp;           /* temp buffer for pixel assignments */

/*
initialize cluster sums, count of assignment change, and cluster counts
-----------------------------------------------------------------------
*/
for (i = 0; i < nbands; i++)
   for (j = 0; j < MAXCLS; j++)
      clsumx[i][j] = clsumsq[i][j] = 0.0;
for (j = 0; j < MAXCLS; j++) clknt[j] = 0.0;
*nchnge = 0;

/*
initialize and enable processing message utility
------------------------------------------------
*/
c_inittm ("KMEANS", "Processed", "line", "iteration ", " ", nl, maxit,
          &msgtim, &msg_flag);
curent = 1;
curcat = iter;
c_startm();

/*
allocate memory
---------------
*/
if((itemp=(unsigned char *)malloc(num_samps * sizeof(unsigned char *)))==NULL)
    c_errmsg ("Error allocating dynamic memory", "kmeans_assign",
                &fatal_err);

/*
perform assignment line by line
-------------------------------
*/
for (line = 0; line < *nl; line++, curent++)
   {
   c_estep ((struct GDESC **)&gdesc_in);
   c_estep ((struct GDESC **)&gdesc_out);
/*
   process the line
   ----------------
*/
   doline (iter, *np, nbands, nclust, clmean, itemp, temp, &nx);
/*
   get line of new cluster assignments, cluster counts, count of 
   assignments changed
   -------------------------------------------------------------
*/
   type = EBYTE;
   c_pxcopy (itemp, &outbuf[opoint], &type, np); 
   for (j = 0; j < nclust; j++) clknt[j] += temp[j];

   *nchnge = nx + *nchnge;
/*
   compute cluster sums based on current assignments
   -------------------------------------------------
*/
   for (j = 0; j < *np; j++)
      {
      for (k = 0; k < nbands; k++)
         {
         xsq = inbuf[ipoint[k]+j];
         clsumx[k][itemp[j]-1] += xsq;
         clsumsq[k][itemp[j]-1] += (xsq*xsq);
         }  
      }  
   }  
/* free buffer
-------------*/
free (itemp);

c_estep ((struct GDESC **)&gdesc_out);
c_stoptm ();
c_egclse ((struct GDESC **)&gdesc_in);
c_egclse ((struct GDESC **)&gdesc_out);
}

