#include "kmeans.h"
#include "protos.h"

void clstats
(
    long nclust,                  /* I: no. clusters                     */
    float clknt[],		  /* I: count of pixels in each cluster  */
    long nbands,                  /* I: no. bands in input image         */
    double *clsumx[], 		  /* I: sums of pixels                   */
    double *clsumsq[],            /* I: sums of squares of pixels        */
    float *clmean[],              /* I/O: class means                    */
    float *clstdv[],              /* I/O: class standard deviations      */
    long iter,                    /* I: iteration number                 */
    long printit,                 /* I: option to print stats            */
    long maxit                    /* I: maximum no. iterations           */
)
/*
**********************************************************************

NAME: CLSTATS

PURPOSE: Computes and prints cluster means and standard deviations


PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         25-Jan-83  G. Mcconaughy
    2         23-Sep-88  K. Narayanan
    3         16-Feb-89  K. Narayanan      Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
float	knt;
double	x, cls;

long	i, j, k;
char	buf1[ERRLEN], buf2[ERRLEN];
/*
compute new cluster means and standard deviations
-------------------------------------------------
*/
for (i = 0; i < nclust; i++)
   {
   knt = clknt[i];
   if (knt  !=  0.0) 
      {
      for (j = 0; j < nbands; j++)
         {
         cls = clsumx[j][i];
         clmean[j][i] = cls/knt;
         x = cls*cls;
         x = knt*clsumsq[j][i]-x;
         if (x < 0.0) x *= -1;
         if (knt > 1)   
            {
            x = x/(knt*(knt-1));
            clstdv[j][i] = sqrt(x);
            }
         else     /* if only 1 pixel, set stdev to zero */
            clstdv[j][i] = 0;
         }  
      }  
   }  

/*
print statistics according to iteration # and 'view' parameter
--------------------------------------------------------------
*/
if ((iter == maxit)  ||  (printit == 0)) 
   {
   for (j = 0; j < nclust; j++)
      {
      fprintf (fd, "CLUSTER NUMBER:  %3d\n", j+1);
      fprintf (fd, "# PIXELS IN CLUSTER:  %10.1f\n", clknt[j]);
      for (i = 0; i < nbands;)
         {
         sprintf (buf1, "CENTER: ");
         sprintf (buf2, "STDEV:  ");
         for (k = 0; k < 14; k++, i++)
            {
            if (i == nbands) break;
            sprintf (&buf1[strlen (buf1)], "%9.2f ", clmean[i][j]);      
            sprintf (&buf2[strlen (buf2)], "%9.2f ", clstdv[i][j]);      
            }
         fprintf (fd, "%s\n",buf1);
         fprintf (fd, "%s\n",buf2);
         fprintf (fd, "\n\n");
         }
      }
   }
}
