#include "kmeans.h"
#include "util.h"
#include "pixman.h"
#include "lasmath.h"
#include "protos.h"
        
void doline
(
    long iter,                  /* I: iteration number            */
    long np,                    /* I: no. pixels/line             */
    long nbands,                /* I: no. bands in input image    */
    long nclust,                /* I: no. clusters                */
    float *clmean[],            /* I: cluster means               */
    unsigned char classign[],   /* I/O: pixel class assignments   */
    float count[],              /* I/O: cluster counts            */
    long *nchange               /* I: no. pixels changing assign  */
)

/*
**********************************************************************

NAME: DOLINE

PURPOSE: Apply minimum distance classifier to one image line

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         25-Jan-83  G. Mcconaughy
    2         23-Sep-88  K. Narayanan
    3         16-Feb-89  K. Narayanan      Conversion to 5.0
    4         08-Aug-90  B. Ailts          Fixed a bug that kept kmeans from
					   working with data above 512 samples

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Process the pixels in the line, MAX_SEGP pixels at a time
      Compute differences and squares of differences from cluster centers
      Compute sum of absolute distance values
      Assign pixel to a cluster with minimum distance
   Compute number of vectors which have changed assignments

**********************************************************************
*/
{
#define MAX_SEGP 512
static	float	*dist_sq;         /* distance square measures       */
static	float	*dist;            /* Euclidian distance measurtes   */
float	mindist,                  /* minimum cluter distance        */
	offset, scale;

static	long	allocated = FALSE;
long	i, ic, imin, index, j, jj, k, l, nb, nnp, type;

/*
allocate dynamic storage once
-----------------------------
*/
if (!allocated)
   {
   if ((dist_sq = (float *) malloc (MAX_SEGP * nbands * sizeof (float))) == 
         NULL || 
       (dist = (float *) malloc (MAX_SEGP * nclust * sizeof (float))) == NULL)
      {
      if (fd) fclose (fd);
      c_egclse ((struct GDESC **)&gdesc_out);
      c_abort (&block, "Error allocating dynamic memory", "kmeans-alloc", 
               prog_name, out_files, &num_outfiles);
      }
   allocated = TRUE;
   }
for (i = 0; i < nclust; i++) count[i] = 0.0;

nb = nbands;
nnp = np;
index = 0;
type = EREAL;

/*
process the pixels in the line, MAX_SEGP pixels at a time
---------------------------------------------------------
*/
for (l = 1; l <= np; l += MAX_SEGP)
   {
   if (np > MAX_SEGP)
      nnp = np - l + 1 < MAX_SEGP ? np - l + 1 : MAX_SEGP;
   else
      nnp = np;

   for (k = 0; k < nclust; k++)
      {
      ic = 0;
      for (j = 0; j < nbands; j++)
         {
/*
         compute difference vectors
         --------------------------
*/
         scale = 1.0;
         offset = -clmean[j][k];
         c_pxline ((long *)&inbuf[ipoint[j]+index],(long *)&dist_sq[ic], 
                   &nnp, &type, &scale, &offset);

/*
         compute square of difference vectors
         ------------------------------------
*/
         scale = 1.0;
         offset = 0.0;
         c_pxmul (&nnp, (long *)&dist_sq[ic], (long *)&dist_sq[ic],
		 (long *)&dist_sq[ic], &scale, &offset, &type);

         ic += nnp;
         }

/*
      compute sum of absolute values
      ------------------------------
*/
      for (i = 0; i < nnp; i++)
         vabsmf (&dist_sq[i], &nb, &nnp, &dist[k*MAX_SEGP+i]);
      }

/*
   find the class with minimum value
   ---------------------------------
*/
   for (j = 0; j < nnp; j++)
      {
      mindist = dist[j];
      imin = 0;

      for (k = 1; k < nclust; k++)
         {
         if (dist[k*MAX_SEGP+j]  <  mindist) 
            {
            mindist = dist[k*MAX_SEGP+j];
            imin = k;
            }  
         }
      jj = j+index;
      classign[jj] = (unsigned char)imin+1;
      count[imin] = count[imin]+1;
      }
   index = index+nnp;
   nnp = np-nnp;
   }

/*
compute number of vectors which have changed assignments
--------------------------------------------------------
*/
if (iter  ==  1) *nchange = np;
else
   for (i = 0, *nchange = 0; i < np; i++)
      if (outbuf[opoint+i] != classign[i]) (*nchange)++;
}
