#include "kmeans.h"
#include "util.h"
#include "protos.h"

void dyn_alloc
(
    long num_inbands,       /* I: number of input image bands      */
    float *clmean[],        /* I/O: pointers to means              */
    float *clstd[],         /* I/O: pointers to standard deviations*/
    double *clsumx[],       /* I/O: pointers to pixel sums         */
    double *clsumsq[]       /* I/O: pointers to sums of squares    */
)
/*
*********************************************************************
NAME: DYN_ALLOC

PURPOSE: Allocate storage for means, sums, and sum squares

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0      16-Feb-89   K. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

*********************************************************************
*/
{
long	buf_size,                  /* size of storage    */
        i, n;

/*
allocate storage for means, sums, and sum squares
-------------------------------------------------
*/
buf_size = num_inbands * MAXCLS;

if ((cl_mean = (float *) malloc( buf_size * sizeof (float))) == NULL ||
    (cl_std = (float *) malloc( buf_size * sizeof (float))) == NULL ||
    (cl_sumx = (double *) malloc( buf_size * sizeof (double))) == NULL ||
    (cl_sumsq = (double *) malloc( buf_size * sizeof (double))) == NULL)
   {
   if (fd) fclose (fd);
   c_abort (&block, "Error allocating dynamic memory", "kmeans-alloc", 
            prog_name, out_files, &num_outfiles);
   }
/*
set up pointers
---------------
*/
for (i = 0, n = 0; i < num_inbands; i++, n += MAXCLS)
   {
   clmean[i] = &cl_mean[n];
   clstd[i] = &cl_std[n];
   clsumx[i] = &cl_sumx[n];
   clsumsq[i] = &cl_sumsq[n];
   }
}
