
#include "kmeans.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char hosin[][CMLEN],      /* I: input image host name descriptors  */
    char hosout[],            /* I: output image host name descriptor  */
    long *num_inimages,       /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* I: band numbers in each      */
    long wind[][4],           /* I: input image windows                */
    long *intype,	      /* O: input image data type	       */ 
    long iter,		      /* I: iteration number                   */
    long *num_samps	      /* O: total num of samples in input image*/
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0       16-Feb-89   K. Narayanan
   2.0       19-Oct-99   D. Hames             Add variable num_samps in order
					      to retrieve total number of 	
					      samples from the DDR.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Open and group input images
   If iteration > 0
      Open output image, for WRITE if iteration = 1, and for
         UPDATE otherwise
      If iteration = 1 update output image DDR
      Group output image

**************************************************************************
*/
{
struct	DDR ddr;

double	line_inc = 1.0,
	samp_inc = 1.0;

long	access,                      /* file access type                   */
	bands[2];
static	long	inbufsiz;            /* image i/o buffer size              */
static	long	outbufsiz;           /* image i/o buffer size              */
long	comb_flag = COMB,            /* flag for upddr                     */
	n, nib,
	stat,                        /* return status                      */
	type;                        /* file type                          */

long opt[3];

opt[0] = opt[2] = 0;
opt[1] = NO_COMPRES;

/*
get input image data type and total number of samples
-----------------------------------------------------
*/
if (iter == 0)
   {
   c_getddr ((char *)hosin, &ddr);
   *intype = ddr.dtype;
   *num_samps = ddr.ns;
   }

/*
open input image files
----------------------
*/
access = IREAD;
type = EREAL;
for (n = 0, nib = 0; n < *num_inimages; n++)
   {
   stat = c_eopens((struct FDESC **)&fdesc[n], hosin[n], image_band_num[n], 
		&wind[0][SL], &wind[0][SS], &wind[0][NL], &wind[0][NS], 
		&ipoint[nib], &access, &type, &zero, &zero);
   if (stat != E_SUCC )
      {
      if (fd) fclose (fd);
      c_abort (&block, "Error opening input image", "kmeans-openin", 
               prog_name, out_files, &num_outfiles);
      }
   nib += num_image_bands[n];
   }  
fdesc[++n] = 0;
/*
allocate storage for image i/o buffer, first time only
-------------------------------------------------------
*/
if (iter == 0)
   {
   if ((inbuf = (float *) c_ealloc((struct FDESC **)fdesc, &inbufsiz)) == NULL)
      {
      if (fd) fclose (fd);
      c_abort (&block, "Error allocating dynamic memory", "kmeans-alloc", 
               prog_name, out_files, &num_outfiles);
      }
   }
/*
set up group containing input bands
-----------------------------------
*/
c_egroup ((struct FDESC **)fdesc, (struct GDESC **)&gdesc_in,
	   (unsigned char *) inbuf, &inbufsiz);

/*
open output image file for iterations > 0
-----------------------------------------
*/
if (iter != 0)
   {
   access = (iter == 1) ? IWRITE : IUPDATE;

   type = EBYTE;
   bands[0] = 1;
   bands[1] = 0;
   stat = c_eopens((struct FDESC **)&fdesc[0], hosout, bands, &one, &one,
                   &wind[0][NL], &wind[0][NS], &opoint, &access, &type, 
                   &zero, opt);
   if (stat != E_SUCC )
      {
      if (fd) fclose (fd);
      c_abort (&block, "Error opening output image", "kmeans-openout", 
               prog_name, out_files, &num_outfiles);
      }

   fdesc[1] = 0;

   if (iter == 1)
      {
      strcpy (out_files[num_outfiles++], hosout);
/*
      allocate storage for image i/o buffer, first time only
     -------------------------------------------------------
*/
      if ((outbuf = (unsigned char *) c_ealloc((struct FDESC **)fdesc, &outbufsiz)) == NULL)
         {
         if (fd) fclose (fd);
         c_abort (&block, "Error allocating dynamic memory", "kmeans-alloc", 
                  prog_name, out_files, &num_outfiles);
         }
/*
      update output image DDR based on input images
      ---------------------------------------------
*/
      c_upddr (&block, hosin, hosout, num_inimages, wind, image_band_num, 
               num_image_bands, &line_inc, &samp_inc, &one, &comb_flag);
      }
/*
   set up group for output image
   -----------------------------
*/
   c_egroup ((struct FDESC **)fdesc,(struct GDESC **) &gdesc_out, outbuf,
	    &outbufsiz);
   }
}
