#include "kmeans.h"
#include "util.h"
#include "protos.h"

#define   TERM   0
#define   LP     1
#define   FIL    2

void getpar
(
    char hos_in[][CMLEN],     /* O: input images host names          */
    char las_out[],           /* O: output image las name            */
    char hos_out[],           /* O: output image host name           */
    long *num_inimages,       /* O: number of input images           */
    long num_image_bands[],   /* O: number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* O: band numbers in each    */
    long *num_inbands,        /* O: total number of input image bands*/
    long wind[][4],           /* O: input image windows              */
    long *nclust,             /* O: no. clusters desired             */
    long *maxit,              /* O: maximum no. iterations           */
    long *iview,              /* O: iteration interval to view stats */
    float *pctcng,            /* O: percentage change threshold      */
    float *clmean[],          /* O: class means                      */
    float *clstd[],           /* O: class standard deviations        */
    double *clsumx[],         /* O: sums of pixels                   */
    double *clsumsq[],        /* O: sums of squares of pixels        */
    char print_file[CMLEN]    /* O: print file name                  */
)

/*
**********************************************************************

NAME: GETPAR

FUNCTION: Gets parameters 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         25-Jan-83  G. Mcconaughy
    2         23-Sep-88  K. Narayanan
    3         16-Feb-89  K. Narayanan      Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Initialize program
   Get and validate parameters
   If output print file or line printer is specified 
      Open a file
      Print header information
   Call DYN_ALLOC to allocate dynamic storage for class means,
      class standard deviations, pixel sums, and sum of squares

**********************************************************************
*/
{

struct	VARIABLE *var;               /* pointer to a variable in las block */

long	flag[MAXIMG][2],            /* flag for subwindows                */
	ftype,
	i, j, k,
	imglimit = MAXIMG,          /* maximum number of images           */
	itype,
	num,
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

char	fileutil_file[CMLEN],       /* file name for fileutil             */
	minmax[4],                  /* minmax option                      */
	las_in[MAXIMG][CMLEN];	    /* input images las names             */

extern long  print[];               /* print options                      */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input image parameters
---------------------------------------
*/
c_getin (&block, "IN", &zero, las_in, hos_in, num_inimages, wind, 
               image_band_num, 
               num_image_bands, num_inbands, &imglimit, &same_window, flag);

if (same_window == 0)
    c_errmsg ("Image windows not same size", "kmeans-window", &fatal_err);

/*
get and validate output image parameters
----------------------------------------
*/
if (c_getout(&block,"OUT",(char (*)[256])las_out,(char(*)[256])hos_out)!=E_SUCC)
   c_errmsg ("Error getting output image name", "kmeans-out", &fatal_err);

num = 1;
itype = ELONG;
ftype = EREAL;
c_getnum (&block, "NCLUST", &num, &itype, (unsigned char *)nclust);
c_getnum (&block, "MAXNUMIT", &num, &itype,(unsigned char *)maxit);
c_getnum (&block, "VIEWIT", &num, &itype, (unsigned char *)iview);
c_getnum (&block, "THRPERC", &num, &ftype, (unsigned char *)pctcng);
/*
Get print option;  open print file and print header information
---------------------------------------------------------------
*/
strcpy (fileutil_file, prog_name);
stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file, 
          print_file);
if (stat != E_SUCC)
   c_errmsg("Fatal error encountered","kmeans-fatal",&fatal_err);

strcpy (out_files[0], fileutil_file);
num_outfiles = 1;
fd = fopen (fileutil_file, "w");

fprintf (fd,"\n\n*** INPUT IMAGE ***\n");
for (i = 0; i < *num_inimages; i++)
   {
   fprintf (fd, "      %s\n", las_in[i]);
   for (j = 0; j < num_image_bands[i];)
      {
      fprintf (fd, "\n     Bands:");
      for (k = 0; k < 20; k++)
         {
         fprintf (fd, " %3d", image_band_num[i][k]);
         if (++j == num_image_bands[i]) break;
         }
      }
      fprintf(fd,"\n");
   }
fprintf (fd, "\n*** OUTPUT IMAGE ***\n");
fprintf (fd, "     %s\n", las_out);
fprintf (fd, "\n*** PARAMETERS ***\n");
fprintf (fd, " NCLUST = %3d MAXNUMIT = %3d THRPERC= %7.3f\n\n",
         *nclust, *maxit, *pctcng);

*pctcng /= 100.0;
/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

/*
allocate dynamic storage for mean, sums, and sums of squares
------------------------------------------------------------
*/
dyn_alloc (*num_inbands, clmean, clstd, clsumx, clsumsq);

}
