/***********************************************************************

NAME: KMEANS

PURPOSE:
   Assigns pixels of a multiband image to clusters using the
   minimum Euclidean distance from cluster means

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         25-Jan-83  G. Mcconaughy
    2         23-Sep-88  K. Narayanan
    3         16-Feb-89  K. Narayanan      Conversion to 5.0
    4         08-Aug-90  B. Ailts          Fixed a bug that kept kmeans from
					   working with data above 512 samples
    5.1       05-Aug-93  D. Bridges        Added TERM to print options
    6         20-Oct-99  D. Hames          Remove references to MAXNS and 
					   replace with num_samps to support
					   images >2gig.  Add prototypes.
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get and validate parameters and allocate
      dynamic storage
   Call CLASSIFY to perform pixel classification and generate 
      output image
   Call TERMINATE to free dynamic storage, update output image history,
      and issue completion message

***********************************************************************/

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "protos.h"

struct PARBLK block;                   /* TAE parameter block     */

FILE    *fd;                           /* output file descriptor  */

double  *cl_sumx;                      /* dynamic storage for     */
double  *cl_sumsq;                     /* sums and sums of squares*/

float   *cl_mean;                      /* means                   */
float   *cl_std;                       /* and standard deviations */
float   *inbuf;                        /* image i/o buffer        */

long    fdesc[MAXIMG];                 /* file descriptors        */
long    gdesc_in;                      /* group descriptor        */
long    gdesc_out;                     /* group descriptor        */
long    ipoint[MAXIMG * MAXBND + 1],   /* offsets into image i/o  */
                opoint;                /* for input and output bands */
long    min_max;                       /* minmax option flag      */
long    msgtim;                        /* message interval        */
long    num_outfiles;                  /* number of output files  */
long    print[3];

unsigned char   *outbuf;               /* image i/o buffer        */
char    out_files[2][CMLEN];           /* output file names       */

extern long    curcat;                 /* parameters for processing */
extern long    curent;                 /* message utility         */

void main
(void) 
{
static double	*clsumx[MAXBND]; /* sums of pixels                   */
static double	*clsumsq[MAXBND];/* sums of squares of pixels        */

float	*clmean[MAXBND];         /* class means                      */
float	*clstd[MAXBND];          /* class standard deviations        */
float	pctcng;                  /* percentage change threshold      */

static 	long	image_band_num[MAXIMG][MAXBND + 1]; 
                                 /* band numbers in each input image */
long	iview,                   /* iteration interval to view statistics */
	maxit,                   /* maximum no. iterations           */
	nclust,                  /* no. clusters desired             */
	num_image_bands[MAXIMG], /* number of bands in each input image  */
	num_inbands,             /* total number of input image bands*/
	num_inimages,            /* number of input images           */
	wind[MAXIMG][4];         /* input image windows              */

static char	hos_in[MAXIMG][CMLEN];   
                                 /* input images host names          */
char	hos_out[CMLEN],          /* output image host name           */
	las_out[CMLEN],          /* output image las name            */
	print_file[CMLEN];       /* output file of cluster statistics*/

/*
get parameters
--------------
*/
getpar (hos_in, las_out, hos_out, &num_inimages, 
        num_image_bands, image_band_num, &num_inbands, 
        wind, &nclust, &maxit, &iview, &pctcng,
        clmean, clstd, clsumx, clsumsq, print_file);
/*
assign pixels to classes and generate output classified image
-------------------------------------------------------------
*/
classify (hos_in, hos_out, &num_inimages, num_image_bands,
          image_band_num, num_inbands, clmean, clstd, 
          clsumx, clsumsq, nclust, &maxit, iview, pctcng, wind);
/*
terminate processing
--------------------
*/
terminate (las_out, hos_out, hos_in, &num_inimages, print_file); 
}
