#ifndef PROTOS_H
#define PROTOS_H

void assign
(
    long iter,              /* I: iteration number                 */
    long *maxit,            /* I: maximum no. iterations           */
    float *clmean[],        /* I: class means                      */
    long *nchnge,           /* O: no. pixels changing assignments  */
    float clknt[],          /* O: no. of pixels in each cluster    */
    double *clsumx[],       /* O: sums of pixels                   */
    double *clsumsq[],      /* O: sums of squares of pixels        */
    long *nl,               /* I: no. lines in input image         */
    long *np,               /* I: no. pixels/line in input image   */
    long nclust,            /* I: no. clusters desired             */
    long nbands,            /* I: no. bands in input image         */
    long num_samps          /* I: total num of samples in image    */
);

void classify
(
    char hos_in[][CMLEN],         /* I: input images host names          */
    char hos_out[],               /* I: output image host name           */
    long *num_inimages,           /* I: number of input images           */
    long num_image_bands[],       /* I: number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* I: band numbers in each        */
    long num_inbands,             /* I: total number of input image bands*/
    float *clmean[],              /* I: class means                      */
    float *clstd[],               /* I: class means                      */
    double *clsumx[], 		  /* I: sums of pixels                   */
    double *clsumsq[],            /* I: sums of squares of pixels        */
    long nclust,                  /* I: no. clusters                     */
    long *maxit,                  /* I: maximum no. iterations           */
    long iview,                   /* I: iteration view interval          */
    double pctcng,                /* I: percentage change threshold      */
    long wind[][4]                /* I: input image windows              */
);

void clstats
(
    long nclust,                  /* I: no. clusters                     */
    float clknt[],		  /* I: count of pixels in each cluster  */
    long nbands,                  /* I: no. bands in input image         */
    double *clsumx[], 		  /* I: sums of pixels                   */
    double *clsumsq[],            /* I: sums of squares of pixels        */
    float *clmean[],              /* I/O: class means                    */
    float *clstdv[],              /* I/O: class standard deviations      */
    long iter,                    /* I: iteration number                 */
    long printit,                 /* I: option to print stats            */
    long maxit                    /* I: maximum no. iterations           */
);

void doline
(
    long iter,                  /* I: iteration number            */
    long np,                    /* I: no. pixels/line             */
    long nbands,                /* I: no. bands in input image    */
    long nclust,                /* I: no. clusters                */
    float *clmean[],            /* I: cluster means               */
    unsigned char classign[],   /* I/O: pixel class assignments   */
    float count[],              /* I/O: cluster counts            */
    long *nchange               /* I: no. pixels changing assign  */
);

void dyn_alloc
(
    long num_inbands,       /* I: number of input image bands      */
    float *clmean[],        /* I/O: pointers to means              */
    float *clstd[],         /* I/O: pointers to standard deviations*/
    double *clsumx[],       /* I/O: pointers to pixel sums         */
    double *clsumsq[]       /* I/O: pointers to sums of squares    */
);

void dyn_free
(void);

void filspec
(
    char hosin[][CMLEN],      /* I: input image host name descriptors  */
    char hosout[],            /* I: output image host name descriptor  */
    long *num_inimages,       /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* I: band numbers in each      */
    long wind[][4],           /* I: input image windows                */
    long *intype,	      /* O: input image data type	       */ 
    long iter,		      /* I: iteration number                   */
    long *num_samps	      /* O: total num of samples in input image*/
);

void getpar
(
    char hos_in[][CMLEN],     /* O: input images host names          */
    char las_out[],           /* O: output image las name            */
    char hos_out[],           /* O: output image host name           */
    long *num_inimages,       /* O: number of input images           */
    long num_image_bands[],   /* O: number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* O: band numbers in each    */
    long *num_inbands,        /* O: total number of input image bands*/
    long wind[][4],           /* O: input image windows              */
    long *nclust,             /* O: no. clusters desired             */
    long *maxit,              /* O: maximum no. iterations           */
    long *iview,              /* O: iteration interval to view stats */
    float *pctcng,            /* O: percentage change threshold      */
    float *clmean[],          /* O: class means                      */
    float *clstd[],           /* O: class standard deviations        */
    double *clsumx[],         /* O: sums of pixels                   */
    double *clsumsq[],        /* O: sums of squares of pixels        */
    char print_file[CMLEN]    /* O: print file name                  */
);

void separ
(
    float *clmean[],       /* I: cluster means                 */
    float *clstdv[],       /* I: cluster standard deviations   */
    long nbands,           /* I: no. bands in input image      */
    long nclust,           /* I: no. clusters                  */
    float clknt[]          /* I: cluster counts                */
);

void setup
(
    long nbands,         /* I: no. bands in input image    */
    long *nl,            /* I: no. lines in input image    */
    long *np,            /* I: and no. pixels/line         */
    long nclust,         /* I: no. clusters                */
    float *clmean[]      /* O: cluster means               */
);

void terminate
(
    char las_out[],		/* I: output image las name            */
    char hos_out[],		/* I: output image host name           */
    char hos_in[][CMLEN],	/* I: input images host names          */
    long *num_inimages,		/* I: number of input images           */
    char print_file[]		/* I: print file name                  */
);
#endif
