#include "kmeans.h"
#include "protos.h"

void separ
(
    float *clmean[],       /* I: cluster means                 */
    float *clstdv[],       /* I: cluster standard deviations   */
    long nbands,           /* I: no. bands in input image      */
    long nclust,           /* I: no. clusters                  */
    float clknt[]          /* I: cluster counts                */
)
/*
**********************************************************************

NAME: SEPAR

PURPOSE: Compute separability information to see if clusters are well
	 separated 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         25-Jan-83  G. Mcconaughy
    2         23-Sep-88  K. Narayanan
    3         16-Feb-89  K. Narayanan      Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
double	di, dj, dij, quot, sumd, sumi, sumj, dif2, sumdij, x; 

long	i, j, k;

/*
compute and print separability information
------------------------------------------
*/
fprintf (fd,"\f\n\n*** CLUSTER SEPARABILITY INFORMATION ***\n\n");
fprintf (fd,"  I  J    D(I,J)      D(I)      D(J)  D(I)+D(J)     QUOT\n");

for (i = 0; i < nclust-1; i++)
   {
   if (clknt[i]  !=  0.0) 
      {
      for (j = i+1; j < nclust; j++)
         {
         if (clknt[j]  !=  0.0) 
            {
            sumd = 0.0;
            sumi = 0.0;
            sumj = 0.0;
            for (k = 0; k < nbands; k++)
               {
               dif2 = (clmean[k][i]-clmean[k][j]);
               dif2 *= dif2;
               sumd = sumd + dif2;
               if (clstdv[k][i]  !=  0.0) 
                  sumi = sumi + (dif2/(clstdv[k][i]*clstdv[k][i]));
               if (clstdv[k][j]  !=  0.0) 
                  sumj = sumj + (dif2/(clstdv[k][j]*clstdv[k][j]));
               }  
            dij = sqrt (sumd);
            x = nbands + 2;
            x = sqrt (x);
            if (sumi  !=  0.0) 
               di = x * sqrt (sumd/sumi);
            else
               di = 0.0;
            if (sumj  !=  0.0) 
               dj = x * sqrt (sumd/sumj);
            else
               dj = 0.0;
            sumdij = di + dj;
            if (sumdij  !=  0.0) 
               quot = dij/sumdij;
            else
               quot = dij;
            
            fprintf (fd, " %2d %2d%10.2f%10.2f%10.2f%10.2f%10.2f\n", 
                     i+1, j+1, dij, di, dj, sumdij, quot);
            }  
         }  
      }  
   }  
if (fd) fclose (fd);
}
