#include "kmeans.h"
#include "util.h"
#include "protos.h"
void setup
(
    long nbands,         /* I: no. bands in input image    */
    long *nl,            /* I: no. lines in input image    */
    long *np,            /* I: and no. pixels/line         */
    long nclust,         /* I: no. clusters                */
    float *clmean[]      /* O: cluster means               */
)

/*
**************************************************************************
NAME: SETUP

PURPOSE: Computes initial cluster means and standard deviations


PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         25-Jan-83  G. Mcconaughy
    2         23-Sep-88  K. Narayanan
    3         16-Feb-89  K. Narayanan      Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

**************************************************************************
*/
{
double	sum1[MAXBND],
	sum2[MAXBND], 
	x, xnum;

float	cntr,
	gval, 
	offset,
	smean[MAXBND], 
	stddev[MAXBND];

long	i, ibnd, iline, isamp, j, k, npix;
long	msg_flag = GEN2;
char	buf1[ERRLEN], buf2[ERRLEN], buf3[ERRLEN];
/*
initialize sums
---------------
*/
for (i = 0; i < nbands; i++)
   sum1[i] = sum2[i] = 0.0;

npix = (*nl) * (*np);
/*
initialize and enable processing message utility
------------------------------------------------
*/
c_inittm ("KMEANS", "Processed", "line", " ", " ", nl, 
         &one, &msgtim, &msg_flag);
curent = 1;
c_startm();
/*
compute intermediate sums
-------------------------
*/
for (iline = 0; iline < *nl; iline++, curent++)
   {
   c_estep ((struct GDESC **)&gdesc_in);
   for (ibnd = 0; ibnd < nbands; ibnd++)
      {
      for (isamp = 0; isamp < *np; isamp++)
         {
         gval = inbuf[ipoint[ibnd]+isamp];
         sum1[ibnd] += gval;
         sum2[ibnd] += gval*gval;
         } 
      } 
   } 
c_egclse ((struct GDESC **)&gdesc_in);
c_stoptm ();
/*
compute overall means and standard deviations
---------------------------------------------
*/
xnum = (double ) npix;
for (i = 0; i < nbands; i++)
   {
   smean[i]  = sum1[i] / xnum;
   x         = sum1[i] * sum1[i];
   x         = xnum * sum2[i] - x;
   x         = x / (xnum * (xnum - 1));
   x         = fabs(x);
   stddev[i] = sqrt (x);	
   }

fprintf (fd,"\n\n*** OVERALL STATISTICS OF INPUT DATA ***\n\n");

for (i = 0; i < nbands;)
   {
   sprintf (buf1, "BAND:       ");
   sprintf (buf2, "MEAN:   ");
   sprintf (buf3, "STDEV:  ");
   for (j = 0; j < 14; j++, i++)
      {
      if (i == nbands) break;
      sprintf (&buf1[strlen (buf1)], "%2d        ", i+1);      
      sprintf (&buf2[strlen (buf2)], "%9.2f ", smean[i]);      
      sprintf (&buf3[strlen (buf3)], "%9.2f ", stddev[i]);
      }
   fprintf (fd, "%s\n", buf1);
   fprintf (fd, "%s\n", buf2);
   fprintf (fd, "%s\n\n", buf3);
   }
/*
compute initial cluster means
-----------------------------
*/
for (i = 0; i < nbands; i++)
   {
   if (smean[i]  <  stddev[i]) stddev[i] = smean[i];
   cntr   = smean[i] + stddev[i];
   offset = (2.0 * stddev[i] / (float) (nclust - 1));
   for (j = 0; j < nclust; j++)
      clmean[i][j] = cntr - (offset * (float)j);
   }  

fprintf (fd,"\f\n\n*** INITIAL CLUSTER CENTROIDS ***\n\n");

for (j = 0; j < nclust; j++)
   {
   fprintf (fd, "CLUSTER:  %3d\n", j+1);
   for (i = 0; i < nbands;)
      {
      fprintf (fd, "MEAN:  ");
      for (k = 0; k < 14; k++, i++)
         {
         if (i == nbands) break;
         fprintf (fd, "%9.2f ", clmean[i][j]);      
         }
      fprintf (fd,"\n\n\n");
      }
   }
}
