/****************************************************************************

NAME: check_res

PURPOSE: Look through the specified bands and determine if they are of the
  same resolution.

RETURN VALUE:
Type = void
Value           Description
-----           -----------

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    06/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include "l7hdf2las.h"

void check_res
(
  int bands[],     /* I: array of bands to search */
  int num_bands,   /* I: number of bands in bands */
  int *same_res    /* O: flag to specify if the bands are of the same
                         resolution (TRUE, FALSE) */
)

{
  int i;  /* looping variable */

  /* Loop through all the bands comparing each one to the first band.  If
     one band is not the same, then exit immediately.  If only one band
     exists, then same_res should be true.
     --------------------------------------------------------------------*/
  *same_res = TRUE;
  for (i = 1; i < num_bands; i++)
  {
    switch (bands[0])
    {
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 7:
        if (bands[i] == 61 || bands[i] == 62 || bands[i] == 8)
        {
          *same_res = FALSE;
          return;
        }
        break;

      case 61:
      case 62:
        if (bands[i] != 61 && bands[i] != 62)
        {
          *same_res = FALSE;
          return;
        }
        break;

      case 8:
        if (bands[i] != 8)
        {
          *same_res = FALSE;
          return;
        }
        break;
    } 
  }

  return;
}
