/****************************************************************************

NAME: copy_vol_files

PURPOSE: Copy all the _vol# files together for the current output file.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "l7hdf2las.h"
#include "cm.h"    /* c_copy_image definition */

int copy_vol_files
(
  struct PARBLK *vblock,   /* I: TAE parameter block */
  char vol_files[][CMLEN], /* I: list of filenames that matched the _vol file
                                 search (Unix format) */
  int nvol_files,          /* I: number of _vol files found in the search */
  char ofile_name[],       /* I: output filename for the merged files */
  long msgtim,             /* I: Elapsed time between display msg for TAE */
  char minmax[4]           /* I: Min max flag for TAE */
)

{
  int bands[MAXIMG][MAXBND+1];
                           /* bands to process - all images.  If all 0s then
                              all bands in all the images will be processed.
                              nbands also needs to be set to 0 for the
                              default. */
  int index, index2;       /* looping variables */
  int nbands[MAXIMG];      /* number of bands per input image.  If bands are
                              all 0s and nbands[i] is 0 then all bands in all
                              the images will be processed */
  int window[MAXIMG][4];   /* window for each image (SL, SS, NL, NS)
                              If all values are 0s then the entire
                              image will be processed.  If the
                              images are not of the same size then
                              processing will be stopped. */

  /* Copy all of the vol_files to hosout using the entire image
     window and all the bands
     ----------------------------------------------------------*/
  for (index = 0; index < nvol_files; index++)
  {
    /* Set the window values to 0s so that the entire image is copied
       --------------------------------------------------------------*/
    window[index][SL] = 0;
    window[index][SS] = 0;
    window[index][NL] = 0;
    window[index][NS] = 0;

    /* Set the band values to 0s so that all the bands are copied
       ----------------------------------------------------------*/
    for (index2 = 0; index2 < MAXBND; index2++)
    {
      bands[index][index2] = 0;
    }
    nbands[index] = 0;
  }

  if (c_copy_image (vblock, vol_files, ofile_name, EXT_COMP,
      nvol_files, window, bands, nbands, 0 /* process all bands */,
      0 /* output to the same data type */, TRUE /* Round the input
      data */, msgtim, minmax) != E_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-copy_image",
      "Error copying the _vol files to one output image");
    return (E_FAIL);
  }

  /* Remove the vol_files .img, .ddr, and .his files
     -----------------------------------------------*/
  for (index = 0; index < nvol_files; index++)
  {
    if (delete_image (vol_files[index]) != E_SUCC)
    {
      /* delete_image prints out an error message */
      return (E_FAIL);
    }
  }

  return (E_SUCC);
}
