/****************************************************************************

NAME: delete_image

PURPOSE: Delete the input .img file and the .ddr/.his files that go with it.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "l7hdf2las.h"
#include "util.h"    /* c_splitname definition */
#include "diskio.h"  /* c_dkdel definition */

int delete_image
(
  char img_file[CMLEN]  /* I: image file to be deleted (with .img extension) */
)

{
  char ddrname[CMLEN];   /* name of the DDR file */
  char directory[CMLEN]; /* directory portion of input HDF filename */
  char extension[CMLEN]; /* extension portion of input HDF file name */
  char hisname[CMLEN];   /* name of the history file */
  char root[CMLEN];      /* root portion of input HDF file name */

  /* Verify that the extension is .img
     ---------------------------------*/
  c_splitname (img_file, directory, root, extension);
  if (strcmp (extension, "img"))
  {
    c_lasmsg (FALSE, "l7hdf2las-extension",
      "Image file must have a .img extension: %s", img_file);
    return (E_FAIL);
  }

  /* Delete the .img file if it exists
     ---------------------------------*/
  if (access (img_file, F_OK) == 0)
  {
    if (c_dkdel (img_file) != E_SUCC)
    {
      c_lasmsg (FALSE, "l7hdf2las-dkdel", "Error removing the .img file: %s",
        img_file);
      return (E_FAIL);
    }
  }

  /* Replace the .img in the image file with .ddr
     --------------------------------------------*/
  strcpy (ddrname, img_file);
  strncpy (&ddrname[strlen (ddrname) - 3], "ddr", 4);

  /* Delete the .ddr file if it exists
     ---------------------------------*/
  if (access (ddrname, F_OK) == 0)
  {
    if (c_dkdel (ddrname) != E_SUCC)
    {
      c_lasmsg (FALSE, "l7hdf2las-dkdel", "Error removing the .ddr file: %s",
        ddrname);
      return (E_FAIL);
    }
  }

  /* Replace the .img in the image file with .his
     --------------------------------------------*/
  strcpy (hisname, img_file);
  strncpy (&hisname[strlen (hisname) - 3], "his", 4);

  /* Delete the .his file if it exists
     ---------------------------------*/
  if (access (hisname, F_OK) == 0)
  {
    if (c_dkdel (hisname) != E_SUCC)
    {
      c_lasmsg (FALSE, "l7hdf2las-dkdel", "Error removing the .his file: %s",
        hisname);
      return (E_FAIL);
    }
  }

  return (E_SUCC);
}
