/****************************************************************************

NAME: delete_links

PURPOSE: Delete the links that were created for CD processing.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    11/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS
  This routine loops from the end of the link_files list when unlinking the
  files.  When a file is unlinked, the num_link_files counter gets
  decremented and the link_files[i] filename is also removed.  This allows
  the updated information to be used if an interrupt occurs while unlinking
  these files.

****************************************************************************/

#include <string.h>
#include <unistd.h>
#include "l7hdf2las.h"

int delete_links
(
  char link_files[][CMLEN],  /* I/O: array of links to be deleted.  These
                                     filenames get set to NULL after being
                                     unlinked. */
  int *num_link_files        /* I/O: number of link files to be deleted.  This
                                     counter gets decremented when a file has
                                     been unlinked. */
)

{
  int i;                /* looping variable */
  int error_occurred;   /* error flag */
  long num_files;       /* number of files to be unlinked */

  /* Loop through all the link_files and unlink them.  If an error occurs,
     then set a flag and continue unlinking the rest of the files.  Loop
     from the end of the list and decrement the num_link_files counter in
     case an interrupt occurs in the middle of unlinking these files.  The
     filename should be set to NULL after the file is unlinked.
     ---------------------------------------------------------------------*/
  error_occurred = FALSE;
  num_files = *num_link_files;  /* num_link_files is decremented in loop */
  for (i = num_files-1; i >= 0; i--)
  {
    if (unlink (link_files[i]) != 0)
    {
      c_lasmsg (FALSE, "l7hdf2las-unlink", "Error unlinking %s",
        link_files[i]);
      error_occurred = TRUE;
    }

    /* Set the filename to NULL and decrement the num_link_files counter
       -----------------------------------------------------------------*/
    strcpy (link_files[i], "");
    (*num_link_files)--;
  }

  if (error_occurred)
    return (E_FAIL);
  else
    return (E_SUCC);
}
