/*******************************************************************************
NAME:		gxx_close_1g_image

PURPOSE:	Close the specified 1G image.

DEVELOPMENT HISTORY:
$Log: gxx_close_1g_image.c,v $
Revision 1.4  2001/09/28 13:34:51  tbeckman
Removed redundant prototypes for some LAS support routines to prevent breaking the build when those prototypes are changed.  Other minor cleanups.

Revision 1.3  2001/01/19 20:36:35  gschmidt
Finished making code review changes.  In addition, added signal handlers
to handle the Ctrl-C introduced by the user.

Revision 1.2  2000/11/08 14:53:20  gschmidt
The gxx and xxx routines had to be updated to compile with the LAS code
and within the LAS system.  Several minor changes were made, but nothing
has been changed to affect the overall flow/working of the original code.

 * 
 *    Rev 1.4   Mon Aug 11 11:04:10 1997   rockvam
 * post-walkthrough changes:
 *  cleanup of $Log: gxx_close_1g_image.c,v $
 *  cleanup of Revision 1.4  2001/09/28 13:34:51  tbeckman
 *  cleanup of Removed redundant prototypes for some LAS support routines to prevent breaking the build when those prototypes are changed.  Other minor cleanups.
 *  cleanup of
 *  cleanup of Revision 1.3  2001/01/19 20:36:35  gschmidt
 *  cleanup of Finished making code review changes.  In addition, added signal handlers
 *  cleanup of to handle the Ctrl-C introduced by the user.
 *  cleanup of
 *  cleanup of Revision 1.2  2000/11/08 14:53:20  gschmidt
 *  cleanup of The gxx and xxx routines had to be updated to compile with the LAS code
 *  cleanup of and within the LAS system.  Several minor changes were made, but nothing
 *  cleanup of has been changed to affect the overall flow/working of the original code.
 *  cleanup of
 *  free hdfid on error
 * 

RETURN VALUE: 
Type = int
Value		Description
-----		-----------
XXXSUCCESS	Successfully closed image
ERROR		Error; image not closed

NOTES:		Need to deattach all the data before closing the file

********************************************************************************
		Landsat 7 Image Assessment System (IAS) Software
		        Property of the U.S. Government
                                NASA/GSFC/EDC
*******************************************************************************/
#include <stdio.h>
#include "xxx_Types.h"
#include "gxx_hdf_io.h"
#include "wrapper.h"

int gxx_close_1g_image (
	HDFIO *hdfid)		/* I-HDF IO structure id	*/
{
intn	hdf_status;		/* HDF function return status	*/
BandNumber band;		/* band loop counter		*/
char	msg[STRLEN];		/* Error message string		*/

if (hdfid == NULL)
    {
    return(XXXSUCCESS);
    }

/*
 * Info about the group
 */
for (band = b1; band <= b8; band++)
    {
    if (hdfid->sds_id[(int)band] != -1)
	{
       /*
	* Done with this SDS
	*/
	hdf_status = SDendaccess(hdfid->sds_id[(int)band]);
	if (hdf_status == FAIL)
	    {
    	    free(hdfid);
    	    hdfid = NULL;
            sprintf(msg,"Error releasing SDS %s", hdfid->sds_name[(int)band]);
	    print_message(FALSE, "gxx_HDFIO", msg);
            return(ERROR);
            }
	}
    }

/*
 * Close down
 */
hdf_status = SDend(hdfid->sd_id);
if (hdf_status == FAIL)
    {
    free(hdfid);
    hdfid = NULL;
    print_message(FALSE, "gxx_HDFIO", "Error ending SDS interface");
    return(ERROR);
    }

Vdetach(hdfid->group_id);
Vend(hdfid->file_id);

hdf_status = Hclose(hdfid->file_id);
if (hdf_status == FAIL)
    {
    free(hdfid);
    hdfid = NULL;
    print_message(FALSE, "gxx_HDFIO", "Error closing HDF file, check attachments");
    return(ERROR);
    }

free(hdfid);

return(XXXSUCCESS);
}
