#include <string.h>
#include "gxx_utility.h"

/******************************************************************************
NAME:           gxx_createname

PURPOSE:
Create a new filename given the directory, root file name, and the desired
extension.

DEVELOPMENT HISTORY:
$Header: /edclxs64/www/adaps/software/src/las/base/app/l7hdf2las/gxx_createname.c,v 1.3 2001/01/19 20:36:35 gschmidt Exp $
AUTHOR        Change-ID    DATE     Description of Change
------        ---------  --------   ---------------------
Gail Schmidt     1.0     05/15/97   Original development.

RETURN VALUE:   Type = int
Value    Description
-----    -----------
none

NOTES:
The last character of the directory path should be a '/'.  A '.' will be
added between the root name and the extension.

******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                                NASA/GSFC/EDC
******************************************************************************/

void gxx_createname (
  const char *directory,      /* I: Directory portion of file name     */
  const char *root,           /* I: Root portion of the file name      */
  const char *extension,      /* I: Extension portion of the file name */
  char *filename)             /* O: New filename                       */
{
  /* Initialize the filename with the directory name.
   */
  strcpy (filename, directory);

  /* Add the root name.
   */
  strcat (filename, root);

  /* Add the '.' after the root.
   */
  strcat (filename, ".");

  /* Add the extension.
   */
  strcat (filename, extension);

  return;
}
