/******************************************************************************
  GPS include file
     this file includes:
       HDF include file
       GPS HDF IO constants and structures
       function prototypes for the IAS HDF IO library
*******************************************************************************/
#ifndef _GXX_HDF_IO_H_
#define _GXX_HDF_IO_H_

#include <string.h>
#include "mfhdf.h"
#include "gxx_const.h"
#include "gxx_structures.h"

/* Note: The _OLD definitions are to support old versions of the data. */
#define IMAGE_CLASS      "Image_Data"
#define IMAGE_CLASS_OLD  "Image"

#define SDS_BAND1	        "B10"
#define SDS_BAND2	        "B20"
#define SDS_BAND3	        "B30"
#define SDS_BAND4	        "B40"
#define SDS_BAND5	        "B50"
#define SDS_BAND6L	        "B60"
#define SDS_BAND6L_OLD          "B6L"
#define SDS_BAND6H	        "B60"
#define SDS_BAND6H_OLD          "B6H"
#define SDS_BAND7	        "B70"
#define SDS_BAND8	        "B80"

#define OUTSIDE 99                /* code for a point outside the image   */
#define LINES_TO_BUFFER 2
/* defines the number of lines to buffer at a time for gxx_gpyramid. */

typedef struct HDFIO {
    int32 file_id;              /* HDF file id                  */
    int32 group_id;             /* HDF group id                 */
    int32 sd_id;                /* HDF SDS interface id         */
    int32 sds_id[NBANDS];       /* HDF SDS id per band          */
    char  sds_name[NBANDS][STRLEN];/* HDF SDS name per band	*/
    char  hdf_dir_name[STRLEN];	/* name of HDF directory file	*/
    char  extdir[STRLEN];	/* external file directory	*/
    } HDFIO;

/* General */
BandNumber gxx_parse_user_band(int user_band_number);
int gxx_get_user_band(BandNumber band_number);

/* 1G I/O */
HDFIO *gxx_open_1g_image(const char *image_filename, AccessMode access_mode, 
	int *nbands, BandNumber bandno[NBANDS], DataType dtype[NBANDS], 
	int nl[NBANDS], int ns[NBANDS]);
int gxx_close_1g_image(HDFIO *hdfid);
int gxx_read_1g_image (const HDFIO *hdfid, BandNumber band, int sl, 
	int ss, int nl, int ns, int subsamp, void *data);
int gxx_write_1g_image(const HDFIO *hdfid, BandNumber band, int sl, 
	int ss, int nl, int ns, int subsamp, void *data);
int32 gxx_open_1g_band(HDFIO *hdfid, BandNumber bandno);
int gxx_create_1g_band(HDFIO *hdfid, BandNumber bandno, DataType dtype, 
	int nl, int ns);
void gxx_create_band_name(const char *HDF_name, const char *type, 
	BandNumber bandno, char *band_name);

#endif
