/******************************************************************************
  GPS structures include file
   this file includes:
     structures for the system table, PCD, CPF, Grid
*******************************************************************************/
#ifndef _GXX_STRUCTURES_H_
#define _GXX_STRUCTURES_H_

#include "gxx_const.h"

typedef struct DBL_XY
    {
    double x;			/* X value                           */
    double y;			/* Y value                           */
    } DBL_XY;

typedef struct VECTOR
    {
    double x;			/* Vector X component			*/
    double y;			/* Vector Y component			*/
    double z;			/* Vector Z component			*/
    } VECTOR;

typedef struct COMPLEX
   {
   double re;
   double im;
   } COMPLEX;

typedef struct LNG_XY
{
  int x;                  /* X value                           */
  int y;                  /* Y value                           */
} LNG_XY;

typedef struct LNG_HW
{
  int height;             /* Height value                      */
  int width;              /* Width value                       */
} LNG_HW;

typedef struct LNG_LS
{
  int line;               /* Line value                        */
  int samp;               /* Sample value                      */
} LNG_LS;
typedef struct DBL_LS
{
  double line;             /* Line value                        */
  double samp;             /* Sample value                      */
} DBL_LS;

typedef struct DBL_LAT_LONG
{
  double lat;              /* Latitude value                    */
  double lng;              /* Longitude value                   */
} DBL_LAT_LONG;

#define COEFS_SIZE 4
typedef struct COEFFICIENTS
{
  double a[COEFS_SIZE];    /* Array of a coefficients           */
  double b[COEFS_SIZE];    /* Array of b coefficients           */
} COEFFICIENTS;

typedef struct CORNERS
{
  DBL_XY upleft;    /* X/Y value of the upper left corner  */
  DBL_XY upright;   /* X/Y value of the upper right corner */
  DBL_XY loleft;    /* X/Y value of the lower left corner  */
  DBL_XY loright;   /* X/Y value of the lower right corner */
} CORNERS;
/*
 * GPS subsystem-wide definitions
 */
#define MAX_BANDS 9

#endif
