/****************************************************************************

NAME: mount_cd

PURPOSE: Mount the specified CD.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include <string.h>
#include "l7hdf2las.h"
#include "las.h"        /* CMLEN for tapeio routines */
#include "tapeio.h"     /* tape routines and defines */
#include "util.h"       /* c_up2low definition */

int mount_cd
(
  char comment[][CMLEN],  /* I: array of user comments for the operator */
  int vol_num,            /* I: volume number of CD to mount (zero-based) */
  int *mounted,           /* O: flag to specify if a CD is mounted */
  char drivename[],       /* O: name of the CD drive in Unix format */
  long *unit              /* O: CD unit number */
)

{
  char density[CMLEN];      /* tape density returned from tapeio routines
                               (not required for CDs) */
  char lhost[CMLEN];        /* local host name */
  char oprmsg[CMLEN];       /* operator message (on input this is the comment
                               field for mounting the CD, on output this is
                               the CD drive name) */
  char tape_id[CMLEN];      /* tape id returned from tapeio routines */
  long cd_access = TREAD;   /* read access */
  long err_mode = HANDLE;   /* print the error messages */
  long length;              /* string length */
  long rewind_flag = NOREW; /* don't rewind the CD */
  long zero = 0;            /* constant zero */

  /* Open and mount the CD
     ---------------------*/
  strcpy (density, "0");
  strcpy (oprmsg, comment[vol_num]);
  if (c_tpopen (unit, &zero /*skip 0 files*/, &err_mode, &rewind_flag,
    density, &cd_access, oprmsg, tape_id) != TAPE_SUCC)
  {
    c_lasmsg (FALSE, "l7hdf2las-tpopen", "Error opening the CD");
    return (E_FAIL);
  }
  *mounted = TRUE;

  /* If the oprmsg doesn't have an @ sign in it then this is a local
     CD drive.  If the oprmsg does have an @ sign then this is a
     remote drive.  If this is a local drive, then we need to get the
     hostname and compute the drive from hostname + @ + DRIVE.
     ----------------------------------------------------------------*/
  if (strchr (oprmsg, '@') == NULL)
  {  /* This is a local tape drive */
    strcpy (drivename, oprmsg);
    if (gethostname (lhost, CMLEN) != 0)
    {
      c_lasmsg (FALSE, "l7hdf2las-hostname", "Error reading the hostname");
      return (E_FAIL);
    }
    length = (long) strlen (lhost);
    c_up2low (lhost, &length);
    sprintf (drivename, "/%s@%s/", lhost, oprmsg);
  }
  else
  {
    /* This is a remote tape drive */
    sprintf (drivename, "/%s/", oprmsg);
  }

  return (E_SUCC);
}
