/*****************************************************************************

NAME: rename_image

PURPOSE: Rename the input file with the name of the output file.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

NOTES:
  This will rename the infile to outfile.  It assumes the infile has the
    ".img" extension on it.  It also renames the .ddr and .his versions of
    these files.

*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "l7hdf2las.h"
#include "util.h"    /* c_splitname definition */

int rename_image
(
  char infile[CMLEN],   /* I: filename of file to be renamed (with .img
                              extension) */
  char outfile[CMLEN]   /* I: name of the new file (with .img extension) */
)

{
  char directory[CMLEN];   /* directory portion of input HDF filename */
  char extension[CMLEN];   /* extension portion of input HDF file name */
  char newname[CMLEN];     /* name of new filename */
  char oldname[CMLEN];     /* name of old filename */
  char root[CMLEN];        /* root portion of input HDF file name */

  /* Verify that the infile extension is .img
     ----------------------------------------*/
  c_splitname (infile, directory, root, extension);
  if (strcmp (extension, "img"))
  {
    c_lasmsg (FALSE, "l7hdf2las-extension",
      "Image file must have a .img extension: %s", infile);
    return (E_FAIL);
  }

  /* Verify that the outfile extension is .img
     -----------------------------------------*/
  c_splitname (outfile, directory, root, extension);
  if (strcmp (extension, "img"))
  {
    c_lasmsg (FALSE, "l7hdf2las-extension",
      "Image file must have a .img extension: %s", outfile);
    return (E_FAIL);
  }

  /* Rename the .img files
     ---------------------*/
  strcpy (oldname, infile);
  strcpy (newname, outfile);
  if (rename (oldname, newname) == -1)
  {
    c_lasmsg (FALSE, "l7hdf2las-rename",
      "Error renaming the .img file %s to %s", oldname, newname);
    return (E_FAIL);
  }

  /* Replace the .img in the input filename and the output filename with .ddr
     ------------------------------------------------------------------------*/
  strcpy (oldname, infile);
  strcpy (newname, outfile);
  strncpy (&oldname[strlen (oldname) - 3], "ddr", 3);
  strncpy (&newname[strlen (newname) - 3], "ddr", 3);

  if (rename (oldname, newname) == -1)
  {
    c_lasmsg (FALSE, "l7hdf2las-rename",
      "Error renaming the .ddr file %s to %s", oldname, newname);
    return (E_FAIL);
  }

  /* Replace the .img in the input filename and the output filename with .his
     ------------------------------------------------------------------------*/
  strcpy (oldname, infile);
  strcpy (newname, outfile);
  strncpy (&oldname[strlen (oldname) - 3], "his", 3);
  strncpy (&newname[strlen (newname) - 3], "his", 3);

  if (rename (oldname, newname) == -1)
  {
    c_lasmsg (FALSE, "l7hdf2las-rename",
      "Error renaming the .his file %s to %s", oldname, newname);
    return (E_FAIL);
  }

  return (E_SUCC);
}
