/****************************************************************************

NAME: signal_cleanup

PURPOSE: Cleanup/delete all the files that were created by this application,
         if the $DELFLG was set.  If $DELFLG was not set then do nothing.
         In addition, if a CD is mounted it must be unmounted, and any
         temporary links must be deleted.

RETURN VALUE:
Type = void

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  7.2    10/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

ALGORITHM DESCRIPTION:

NOTES:
  1) For the signal handler, this must be a void function.  Thus if any
     errors occur in this routine, then will not be checked or returned.
  2) This function cannot have any parameters, since it is passed from the
     main routine to the signal handler.
****************************************************************************/

#include <stdio.h>
#include "l7hdf2las.h"

void signal_cleanup ()
{
  /* Cleanup the files that have been created if $DELFLG is set
     ----------------------------------------------------------*/
  cleanup_files (&g_num_procd_files, g_procd_files, g_delflg);

  /* Delete any temporary links that have been made 
     ----------------------------------------------*/
  if (g_links_created)
    delete_links (g_link_files, &g_num_link_files);

  /* Processing from a CD?
     ---------------------*/
  if (g_mediaflag == CDMEDIA)
  {
    /* Unmount the CD if it is mounted
       -------------------------------*/
    unmount_cd (&g_mounted, &g_unit);
  }

  exit (EXIT_FAILURE);
}
