/****************************************************************************

NAME: sort_bands_in_nbands

PURPOSE: Sort the bands within the nbands group.

RETURN VALUE:
Type = void
Value           Description
-----           -----------

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    06/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

****************************************************************************/

#include "l7hdf2las.h"

void sort_bands_in_nbands
(
  int nbands[],    /* I: array of NBANDS values */
  int num_nbands,  /* I: number of values in nbands */
  int bands[],     /* I/O: on input, array of bands to sort; on output,
                      array of sorted bands */
  int num_bands    /* I: number of bands in bands */
)

{
  int group_start = 0;  /* placeholder of the starting index for the
                           current nbands group */
  int index;            /* index for the current nbands group */
  int index2;           /* index for backwards pass in current nbands group */
  int index_nb;         /* index for the nbands array */
  int swapped;          /* flag to specify if a swap has been made (T or F) */
  int temp;             /* temporary storage variable for swapping */

  /* If num_nbands = num_bands then each band goes into a separate output
     file so there is no need to sort the bands.
     --------------------------------------------------------------------*/
  if (num_nbands != num_bands)
  {
    /* Sort the bands within nbands
       ----------------------------*/
    group_start = 0;

    /* Loop through nbands
       -------------------*/
    for (index_nb = 0; index_nb < num_nbands; index_nb++)
    {
      swapped = TRUE;

      /* Loop through the current nbands set and sort the bands
         ------------------------------------------------------*/
      for (index = group_start; index <= group_start + nbands[index_nb] - 1;
           index++)
      {
        /* If no swaps were made then the sort is finished
           -----------------------------------------------*/
        if (!swapped)
          break;

        /* Start at the end of the group and run a pass backwards in the
           group.
           -------------------------------------------------------------*/
        swapped = FALSE;
        for (index2 = group_start + nbands[index_nb] - 1; index < index2;
             index2--)
        {
          if (bands[index2] != 61 && bands[index2-1] != 61 &&
              bands[index2] != 62 && bands[index2-1] != 62)
          {
            /* Process bands 1,2,3,4,5,7,8 normally
               ------------------------------------*/
            if (bands[index2-1] > bands[index2])
            {
              temp = bands[index2];
              bands[index2] = bands[index2-1];
              bands[index2-1] = temp;
              swapped = TRUE;
            }
          }
          else if (bands[index2-1] > 6 && bands[index2-1] < 9 && 
                   bands[index2] >= 61)
          {
            /* Bands 7 or 8 are greater than 61 or 62
               --------------------------------------*/
            temp = bands[index2];
            bands[index2] = bands[index2-1];
            bands[index2-1] = temp;
            swapped = TRUE;
          }
          else if (bands[index2-1] >= 61 && bands[index2] < 6)
          {
            /* Bands 61 or 62 are greater than 1,2,3,4,5
               -----------------------------------------*/
            temp = bands[index2];
            bands[index2] = bands[index2-1];
            bands[index2-1] = temp;
            swapped = TRUE;
          }
          else if (bands[index2-1] == 62 && bands[index2] == 61)
          {
            /* Swap 62 and 61
               --------------*/
            temp = bands[index2];
            bands[index2] = bands[index2-1];
            bands[index2-1] = temp;
            swapped = TRUE;
          }
        }
      }

      group_start += nbands[index_nb];
    }
  }
}
