/****************************************************************************

NAME: strip_vol_from_string

PURPOSE: Strip the _vol# from the input string.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    08/00  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

NOTES:
  It is assumed that the input filename has an _vol# in it somewhere.  If
    one is not found than E_FAIL is returned from this routine.
  It is also assumed that input filenames are of the format base_res_vol#.img
    or base_vol#.img.
  The # in _vol# may be one or more digits long.

****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "l7hdf2las.h"
#include "util.h"  /* c_splitname definition */

int strip_vol_from_string
(
  char ifile_name[],   /* I: input filename with _vol# */
  char ofile_name[]    /* O: output filename without _vol# */
)

{
  int i;                   /* looping variable */
  char directory[CMLEN];   /* directory portion of input HDF filename */
  char extension[CMLEN];   /* extension portion of input HDF file name */
  char root[CMLEN];        /* root portion of input HDF file name */

  /* Preserve the input filename
     ---------------------------*/
  strcpy (ofile_name, ifile_name);

  /* Split LAS filename into directory, root, and extension
     ------------------------------------------------------*/
  c_splitname (ofile_name, directory, root, extension);

  /* Start at the end of the filename and look for "_vol"
     ----------------------------------------------------*/
  for (i = strlen (root) - 5; i >= 0; i--)
  {
    if (!strncmp (&root[i], "_vol", 4))
    {
      /* The last occurrance of _vol was found so set that position
         to NULL so it is the end of the filename.
         ----------------------------------------------------------*/
      root[i] = '\0';
      break;
    }
  }

  /* Put the filename back together
     ------------------------------*/
  sprintf (ofile_name, "%s%s.%s", directory, root, extension);

  /* Make sure the _vol portion of the filename was found.  If not
     then we have an invalid set of filenames
     -------------------------------------------------------------*/
  if (!strcmp (ofile_name, ifile_name))
  {
    c_lasmsg (FALSE, "l7hdf2las-merge_vol_files",
      "Error finding _vol in the first filename");
    return (E_FAIL);
  }

  return (E_SUCC);
}
