/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME: xxx_CloseODL.c

PURPOSE: Close the ODL tree.

INVOCATION METHOD:
    status = xxx_CloseODL(p_odlTree, p_ErrorMessage)

RETURN VALUE:
    Type = int

Value          Description
-------------- ----------------------------------------------------------------
XXXSUCCESS     ODL tree freed
ERROR          Error freeing ODL tree

ARGUMENT LIST:
Name            Type               Use Description
--------------- ------------------ --- ---------------------------------------
p_odlTree       OBJDESC *           I  ODL tree
p_ErrorMessage  char *              O   Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                    Use    Description
----------------------- ---    -------------------------------------------------
none

EXTERNAL VARIABLES:
Source            Name                Type            Use Description
---------------   ------------------- --------------- --- -----------------------
OdlPrintMessage   ODLErrorMessage     Char *           I  ODL error message

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name            Description
--------------- -------------------------------------------------
OdlFreeTree       Free ODL tree

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
- Error returned form ODLFreeTree

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
T. Raheja	   N/A         R1.0   03/97      Original Implementation

NOTES:
none

Unit xxx_CloseODL PDL:

     Call OdlFreeTree() to free the p_ODLTree
1    IF ODL error THEN
         Build error message in p_ErrorMessage
         RETURN ERROR
2    ENDIF

     RETURN XXXSUCCESS

End Unit xxx_CloseODL
******************************************************************************/

/* System Headers */
#include <stdio.h>
#include <lablib3.h>                 /* prototypes for the ODL functions */

/* Third party control */
#include "xxx_ODL.h"                 /* prototype for the xxx ODL functions */

extern char ODLErrorMessage[];       /* External Variables */

int xxx_CloseODL(OBJDESC *p_lp, char *p_ErrorMessage)
{
	p_lp = OdlFreeTree(p_lp);
	if (p_lp != NULL)
	{
		(void)strcpy(p_ErrorMessage, "Can't Close ODL file:  ");
		(void)strcat(p_ErrorMessage, ODLErrorMessage);
		return ERROR;
	}
	return XXXSUCCESS;
}
