/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_EDCMeta.h

PURPOSE:  Defines EDC Metadata structures, constants, and prototype.

INVOCATION METHOD:
	#include <xxx_EDCMeta.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
none

EXTERNAL VARIABLES:
Source		Name			Type		Use	Description
---------------	----------------	---------------	---	-----------------------
none

DEVELOPMENT HISTORY:
Author			Change-ID	Release	Date		Description of Change
-----------------	-----------	-------	--------	------------------------------
J. kerich		N/A		R1.0	05/05/97	Original Implementation
J. kerich		IASI970103    	R2.0	09/16/97	Update to new DAAC format
J. kerich		IASI980295    	R3.0	03/24/98	Reword TotalScenes comment
J. kerich		IAS980096    	R3.1	07/20/98	Add FormatScanOffset to xxx_ProductMetaData_TYPE

NOTES:
none

******************************************************************************/

#ifndef XXX_EDCMETA_H
#define XXX_EDCMETA_H

#include "xxx_TimeConvert.h"

#define XXX_MAXFILE_SIZE	40

typedef struct xxx_EDCMetaDataFileInfo_TYPE {
	xxxISO8601Time	ProductCreationDateTime;
	char		StationId[20];
} xxx_EDCMetaDataFileInfo_TYPE;

typedef struct xxx_ProductMetaData_TYPE {
	char			ProductType[10];
	char			SpacecraftId[15];
	char			SensorId[15];
	xxxYMDTime		AcquisitionDate;
	long			WRSPath;
	long			StartingRow;
	long			EndingRow;
	long			TotalScenes;     /* Number of scene in DAAC product EndingRow - StartingRow + 1 */
	float			TotalWRSScenes;
	long			NumberOfScans;
	long			StartingSubintervalScan;
	long			EndingSubintervalScan;
	long			FormatScanOffset;
	char			Band_combination[14];
	float			ProductULCornerLat;
	float			ProductULCornerLon;
	float			ProductURCornerLat;
	float			ProductURCornerLon;
	float			ProductLLCornerLat;
	float			ProductLLCornerLon;
	float			ProductLRCornerLat;
	float			ProductLRCornerLon;
        char            	Band1_Gain[2];
        char            	Band2_Gain[2];
        char            	Band3_Gain[2];
        char            	Band4_Gain[2];
        char           		Band5_Gain[2];
        char            	Band6_F1_Gain[2];
        char            	Band6_F2_Gain[2];
        char            	Band7_Gain[2];
        char            	Band8_Gain[2];
	char			Band1_FileName[XXX_MAXFILE_SIZE];
	char			Band2_FileName[XXX_MAXFILE_SIZE];
	char			Band3_FileName[XXX_MAXFILE_SIZE];
	char			Band4_FileName[XXX_MAXFILE_SIZE];
	char			Band5_FileName[XXX_MAXFILE_SIZE];
	char			Band6_F1_FileName[XXX_MAXFILE_SIZE];
	char			Band6_F2_FileName[XXX_MAXFILE_SIZE];
	char			Band7_FileName[XXX_MAXFILE_SIZE];
	char			Band8_F1_FileName[XXX_MAXFILE_SIZE];
	char			Band8_F2_FileName[XXX_MAXFILE_SIZE];
	char			Band8_F3_FileName[XXX_MAXFILE_SIZE];
	char			IC_Data_F1_FileName[XXX_MAXFILE_SIZE];
	char			IC_Data_F2_FileName[XXX_MAXFILE_SIZE];
	char			ScanOffsetsF1Name[XXX_MAXFILE_SIZE];
	char			ScanOffsetsF2Name[XXX_MAXFILE_SIZE];
	char			MSCD_Data_F1_FileName[XXX_MAXFILE_SIZE];
	char			MSCD_Data_F2_FileName[XXX_MAXFILE_SIZE];
	char			PCD_Data_F1_FileName[XXX_MAXFILE_SIZE];
	char			PCD_Data_F2_FileName[XXX_MAXFILE_SIZE];
	char			MetaData_F1_FileName[XXX_MAXFILE_SIZE];
	char			MetaData_F2_FileName[XXX_MAXFILE_SIZE];
	char			MetaData_PS_FileName[XXX_MAXFILE_SIZE];
	char			CPF_FileName[XXX_MAXFILE_SIZE];
	char			Geolocation_FileName[XXX_MAXFILE_SIZE];
	char			HDF_FileName[XXX_MAXFILE_SIZE];
} xxx_ProductMetaData_TYPE;

typedef struct xxx_EDCMetaDataFile_TYPE {
	xxx_EDCMetaDataFileInfo_TYPE	MetadataFileInfo;
	xxx_ProductMetaData_TYPE	ProductMetaData;
} xxx_EDCMetaDataFile_TYPE;

xxx_EDCMetaDataFile_TYPE *xxx_ReadEDCMetadata(char *, char *);

#endif
