/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_Errno.h

PURPOSE:  Include file for error returns from library functions.

INVOCATION METHOD:
	#include <xxx_Errno.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                    Use 	Description
----------------------- --- 	-------------------------------------------------
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. kerich		      R1.0    02/14/97   Original Implementation
T. Raheja		      R1.0    05/20/97   Added error codes
J. kerich	  IASI980424  R3.1    08/11/98   Added XXX_E_FILE_NOT_FOUND & XXX_E_ODL_SYNTAX

NOTES:
Several global library functions may fail for various reasons. If they fail,
the reason for failure will be indicated in the global variable xxx_errno.
This file declares variables for retrieving this error information, and defines
constants for all possible library function failure reasons. This is modelled
after UNIX's errno global variable usage. For more details, see the UNIX man
page for "errno".

******************************************************************************/

#ifndef XXX_ERRNO_H
#define XXX_ERRNO_H

extern char *xxx_errlist[];
extern long xxx_nerr;
extern long xxx_errno;

#define XXX_E_NO_MEMORY				0
#define XXX_E_NULL_INPUT			1
#define XXX_E_NO_MORE_OBJECTS			2
#define XXX_E_OBJECT_NOT_FOUND			3
#define XXX_E_OUT_OF_RANGE			4
#define XXX_E_INVALID_DATA			5
#define XXX_E_TIMEOUT				6
#define XXX_E_FAILURE				7
#define XXX_E_INSUFFICIENT_INPUT		8
#define XXX_E_FILE_NOT_FOUND			9
#define XXX_E_ODL_SYNTAX			10

#endif
