/******************************************************************************
*                              PACOR II SOFTWARE
*                      Property of the U.S. Government
*                              NASA/GSFC/Code 560
*******************************************************************************
*
*    UNIT NAME:
*	xxx_FileLock.h
*
*
*    PURPOSE:
*	Defines the xxx_FileLock prototype and lock macros.
*
*
*    INVOCATION METHOD:
*	#include <xxx_FileLock.h>
*
*
*    EXTERNAL FILE/RECORD REFERENCES:
*
*       NAME		USE	DESCRIPTION
*       ----		---	-------------------------------------
*	fcntl.h		I	Defines UNIX fcntl() function
*
* 
*    CHANGE HISTORY:
*
*				  BUILD/
*    	AUTHOR	      CHANGE-ID	  RELEASE      DATE     DESCRIPTION OF CHANGE
*    	-----------   ---------	  --------   --------   -----------------------
*	J. Kerich     N/A         R1B2       10/15/93   Original Implementation
*       J. kerich                 R1.0       03/24/97   Ported to IAS
*
*
*    NOTES:
*	none
*
******************************************************************************/

#ifndef XXX_FILELOCK_H
#define XXX_FILELOCK_H

#ifndef _FCNTL_H
#include <fcntl.h>
#endif

/* Set read lock, this will prevent another process from write-locking 
   the file */
#define XXX_READ_LOCK(fd, offset, whence, length) \
		xxx_FileLock(fd, F_SETLK, F_RDLCK, offset, whence, length)

/* Set read lock with wait option this will prevent another process from 
   write-locking the file */
#define	XXX_READ_WAIT_LOCK(fd, offset, whence, length) \
		xxx_FileLock(fd, F_SETLKW, F_RDLCK, offset, whence, length)

/* Set write lock, this will prevent another process from writing to the 
   file */
#define	XXX_WRITE_LOCK(fd, offset, whence, length) \
		xxx_FileLock(fd, F_SETLK, F_WRLCK, offset, whence, length)

/* Set write lock with wait option, this will prevent another process from 
   writing to the file */
#define	XXX_WRITE_WAIT_LOCK(fd, offset, whence, length) \
		xxx_FileLock(fd, F_SETLKW, F_WRLCK, offset, whence, length)

/* Unlock file */
#define	XXX_UNLOCK(fd, offset, whence, length) \
		xxx_FileLock(fd, F_SETLK, F_UNLCK, offset, whence, length)

int xxx_FileLock(int, int, short, off_t, short, off_t);

#endif
