/******************************************************************************
*                              PACOR II SOFTWARE
*                      Property of the U.S. Government
*                              NASA/GSFC/Code 560
*******************************************************************************
*
*    UNIT NAME:
*       xxx_GetTempName.c
*
*    PURPOSE:
*       Create a name for a temporary file using the non-POSIX tempnam()
*       system call.
*
*    INVOCATION METHOD:
*       Status = xxx_GetTempName(p_dir, p_pfx, p_tmpnam)
*
*    RETURNED VALUE:
*
*       TYPE = char *
*
*       VALUE           DESCRIPTION
*       ------------    ----------------------------
*       NULL            Fail to generate name            
*       NON-NULL        Generated name
*
*
*    ARGUMENT LIST:
*
*       NAME            TYPE            USE     DESCRIPTION
*       ------------    ---------       ---     -----------------------------------
*        p_dir          const char *     I      Name of the directory in which the file is to be create
*        p_pfx          const char *     I      Favorite initial letter sequences (up to 5 characters)
*
*
*    EXTERNAL FILE/RECORD REFERENCES:
*
*       NAME                    USE     DESCRIPTION
*       ----                    ---     -------------------------------------
*       xxx_GetTempName.h          I      Defines function prototype     
* 
*    EXTERNAL VARIABLES:
*
*       SOURCE         NAME                TYPE    USE     DESCRIPTION
*       ------         ---------------     ----    ---     --------------------
*       None
*
*    EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
*
*       NAME                        PURPOSE
*       ------------------------    --------------------------------------------
*       None 
*
*    ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
*       None
*
*    ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
*       None
*
*    NON-PORTABLE CODE:
*
*       DESCRIPTION     REASON FOR USE          SUGGESTED PORTING PROCEDURES
*       -----------     ---------------         -------------------------------
*       tempnam         Create a name for a     Implement unique name generator 
*                        temporary file
*
*    CHANGE HISTORY:
*
*                                 BUILD/
*       AUTHOR        CHANGE-ID   RELEASE      DATE     DESCRIPTION OF CHANGE
*       -----------   ---------   --------   --------   -----------------------
*       R. Roth       N/A         R1B2       10/11/93   Original Implementation
*	J. kerich                 R1.0       03/24/97   Ported to IAS
*	J. kerich     IASI970057  R2.0       12/05/97   Add process id to temporary 
*							file name to make it more unique
*
*    NOTES:
*      None
*
******************************************************************************/
/******************************************************************************
UNIT xxx_GetTempName PDL
-------------------------------------------------------------------------------

	CALL tempnam() to get a name for a temporary file name
1	IF error THEN
		Call getpid() to get the process id
		Reallocate the temporary file name to include the process id
2		IF reallocation error THEN
			Free memory
			RETURN NULL
3		ENDIF
		Concatenate the process id to temporary file name
4	ENDIF

	RETURN pointer to temporary file name               

-------------------------------------------------------------------------------
END UNIT xxx_GetTempName PDL
******************************************************************************/
/* Feature test switches */
#undef _POSIX_SOURCE

/* System headers */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* Local headers */
#include "xxx_Types.h"

char * xxx_GetTempName(const char *p_dir, const char *p_pfx)
{
	char *p_TempName;
	char *p_Temp;
	char Pid[20];

	if ((p_TempName = tempnam(p_dir, p_pfx)) != NULL)
	{
		(void)sprintf(Pid, "%ld", getpid());
		if ((p_Temp = realloc(p_TempName, strlen(p_TempName)+strlen(Pid)+1)) == NULL)
		{
			free(p_TempName);
			return NULL;
		}
		p_TempName = p_Temp;
		(void)strcat(p_TempName, Pid);
	}

	return(p_TempName);
}
