/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_HDFDirectory.h

PURPOSE:  Defines HDFDirectory structures, constants, and prototype.

INVOCATION METHOD:
	#include <xxx_HDFDirectory.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
none

EXTERNAL VARIABLES:
Source		Name			Type		Use	Description
---------------	----------------	---------------	---	-----------------------
hlimits.h      	VGNAMELENMAX		int		I	Defined max vgroup size	

DEVELOPMENT HISTORY:
Author			Change-ID	Release	Date		Description of Change
-----------------	-----------	-------	--------	------------------------------
J. kerich		N/A		R1.0	04/02/97	Original Implementation
J. kerich     		IASI970103   	R2.0    11/14/97   	Change OBC_ to IC_
J. kerich		IASI990538     	R3.2	11/23/98	Add L1 Vgroups, xxx_HDFDirectory2 & 
      								xxx_CompareObject2 prototypes

NOTES:
	none

******************************************************************************/

#ifndef XXX_HDFDIRECTORY_H
#define XXX_HDFDIRECTORY_H

#include <stdlib.h>
#include <hdfi.h>
#include <hlimits.h>

#define XXX_NUMBER_OF_VGROUPS   13                

/* L0R vgroup names */
#define XXX_VGROUP1    "Scene_Data_30m"
#define XXX_VGROUP2    "Scene_Data_60m"
#define XXX_VGROUP3    "Scene_Data_15m"
#define XXX_VGROUP4    "IC_Data_30m"
#define XXX_VGROUP5    "IC_Data_60m"
#define XXX_VGROUP6    "IC_Data_15m"
#define XXX_VGROUP7    "Scan_Line_Offsets_30m"
#define XXX_VGROUP8    "Scan_Line_Offsets_60m"
#define XXX_VGROUP9    "Scan_Line_Offsets_15m"


/* L1 vgroup names */
#define XXX_VGROUP1_L1 "Scene_Data_Ref"
#define XXX_VGROUP2_L1 "Scene_Data_Thm"
#define XXX_VGROUP3_L1 "Scene_Data_Pan"
#define XXX_VGROUP4_L1 "IC_Data_Ref"
#define XXX_VGROUP5_L1 "IC_Data_Thm"
#define XXX_VGROUP6_L1 "IC_Data_Pan"
#define XXX_VGROUP7_L1 "Scan_Line_Offsets_Ref"
#define XXX_VGROUP8_L1 "Scan_Line_Offsets_Thm"
#define XXX_VGROUP9_L1 "Scan_Line_Offsets_Pan"

/* Shared L0R and L1 vgroup names */
#define XXX_VGROUP10   "PCD"
#define XXX_VGROUP11   "MSCD"
#define XXX_VGROUP12   "Product_Metadata"
#define XXX_VGROUP13   "CPF"

typedef struct xxx_HDFDirInfo_TYPE {
	char VgroupName[VSNAMELENMAX+1];
	char VgroupClass[VSNAMELENMAX+1];
	int32 VgroupId;
} xxx_HDFDirInfo_TYPE;

typedef struct xxx_ObjectInfo_TYPE {
	char   ObjectName[VSNAMELENMAX+1];
	int32  tag;
	int32  ref;
} xxx_ObjectInfo_TYPE;

typedef struct xxx_HDFDirectory_TYPE {
	char                VgroupName[VGNAMELENMAX+1];
	char                VgroupClass[VGNAMELENMAX+1];
	int32               NumberEntries;
	xxx_ObjectInfo_TYPE *p_ObjectInfo;
} xxx_HDFDirectory_TYPE;

typedef struct xxx_HDFDirectoryControl_TYPE {
	int32		       	Count;			/* Number of rows in HDFDirectory format */

	int32                   FileId;
	int32                   vdataId;
	int32                   SDId;
	int32                   sds_index;
	int32                   sds_id;

	xxx_HDFDirectory_TYPE	*p_HDFDirectory;	/* Format PCD */
} xxx_HDFDirectoryControl_TYPE;

#define XXX_FREE_HDFDIRECTORY(HDFDirectoryControl) \
{ \
        int MacroCount; \
        if (HDFDirectoryControl) \
	{ \
		for (MacroCount=0; MacroCount < HDFDirectoryControl->Count; MacroCount++) \
		{ \
			free(HDFDirectoryControl->p_HDFDirectory[MacroCount].p_ObjectInfo); \
		} \
		free(HDFDirectoryControl->p_HDFDirectory); \
		free(HDFDirectoryControl); \
	} \
}

xxx_HDFDirectoryControl_TYPE *xxx_ReadHDFDirectory(char *, char *, char *);
xxx_HDFDirectoryControl_TYPE *xxx_ReadHDFDirectory2(char *, char *, int, char *);
int xxx_CompareObject(char *, char *);
int xxx_CompareObject2(char *, int, char *);

#endif
