/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_LPGSMeta.h

PURPOSE:  Defines LPGS Metadata structures, constants, and prototype.

INVOCATION METHOD:
	#include <xxx_LPGSMeta.h>

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name				Use	Description
-------------------------	---	-------------------------------------------------
none

EXTERNAL VARIABLES:
Source		Name			Type		Use	Description
---------------	----------------	---------------	---	-----------------------
none

DEVELOPMENT HISTORY:
Author			Change-ID	Release	Date		Description of Change
-----------------	-----------	-------	--------	------------------------------
J. kerich		IASI990538     	R3.2	11/23/98	Original Implementation
J. kerich		IASI990550     	R3.2	02/17/99	Add new metadata fields

NOTES:
none

******************************************************************************/

#ifndef XXX_LPGSMETA_H
#define XXX_LPGSMETA_H

#include "xxx_TimeConvert.h"

#define XXX_MAXFILE_SIZE	40

typedef struct xxx_LPGSMetadataFileInfo_TYPE {
	char		RequestId[24];
	xxxISO8601Time	ProductCreationTime;
	char		StationId[4];
	char		Landsat7Xband[2];
	char		GroundStation[4];
	long		LPS_ProcessorNumber;
	char		DateHourContactPeriod[8];
	char		SubintervalNumber[4];
 } xxx_LPGSMetadataFileInfo_TYPE;

typedef struct xxx_LPGSProductMetaData_TYPE {
	char		ProductType[4];
    char        ProcessingSoftware[11];
	char		SpacecraftId[12];
	char		SensorId[8];
	xxxYMDTime	AcquisitionDate;
	long		TotalScenes;	/* how many scenes in metadata */
	long		WRSPath;
	long		StartingRow;
	long		EndingRow;
	char		Band_combination[12];
	double		ProductULCornerLat;
	double		ProductULCornerLon;
	double		ProductURCornerLat;
	double		ProductURCornerLon;
	double		ProductLLCornerLat;
	double		ProductLLCornerLon;
	double		ProductLRCornerLat;
	double		ProductLRCornerLon;
	double		ProductULCornerMapX;
	double		ProductULCornerMapY;
	double		ProductURCornerMapX;
	double		ProductURCornerMapY;
	double		ProductLLCornerMapX;
	double		ProductLLCornerMapY;
	double		ProductLRCornerMapX;
	double		ProductLRCornerMapY;
	long		ProductSamplesPAN;
	long		ProductLinesPAN;
	long		ProductSamplesREF;
	long		ProductLinesREF;
	long		ProductSamplesTHM;
	long		ProductLinesTHM;
	char		Band1_FileName[XXX_MAXFILE_SIZE];
	char		Band2_FileName[XXX_MAXFILE_SIZE];
	char		Band3_FileName[XXX_MAXFILE_SIZE];
	char		Band4_FileName[XXX_MAXFILE_SIZE];
	char		Band5_FileName[XXX_MAXFILE_SIZE];
	char		Band6L_FileName[XXX_MAXFILE_SIZE];
	char		Band6H_FileName[XXX_MAXFILE_SIZE];
	char		Band7_FileName[XXX_MAXFILE_SIZE];
	char		Band8_FileName[XXX_MAXFILE_SIZE];
	char		IC_Data_F1_FileName[XXX_MAXFILE_SIZE];
	char		IC_Data_F2_FileName[XXX_MAXFILE_SIZE];
	char		ScanShifts_F1_FileName[XXX_MAXFILE_SIZE];
	char		ScanShifts_F2_FileName[XXX_MAXFILE_SIZE];
	char		MSCD_FileName[XXX_MAXFILE_SIZE];
	char		PCD_FileName[XXX_MAXFILE_SIZE];
	char		MetaData_LPS1_FileName[XXX_MAXFILE_SIZE];
	char		MetaData_LPS2_FileName[XXX_MAXFILE_SIZE];
	char		MetaData_LPGS_FileName[XXX_MAXFILE_SIZE];
	char		CPF_FileName[XXX_MAXFILE_SIZE];
	char		Geolocation_FileName[XXX_MAXFILE_SIZE];
	char		HDF_Directory_FileName[XXX_MAXFILE_SIZE];
} xxx_LPGSProductMetaData_TYPE;

typedef struct xxx_MinMaxRadiance_TYPE {
	double		MaxDetectedRadianceLevelBand1;
	double		MinDetectedRadianceLevelBand1;
	double		MaxDetectedRadianceLevelBand2;
	double		MinDetectedRadianceLevelBand2;
	double		MaxDetectedRadianceLevelBand3;
	double		MinDetectedRadianceLevelBand3;
	double		MaxDetectedRadianceLevelBand4;
	double		MinDetectedRadianceLevelBand4;
	double		MaxDetectedRadianceLevelBand5;
	double		MinDetectedRadianceLevelBand5;
	double		MaxDetectedRadianceLevelBand6L;
	double		MinDetectedRadianceLevelBand6L;
	double		MaxDetectedRadianceLevelBand6H;
	double		MinDetectedRadianceLevelBand6H;
	double		MaxDetectedRadianceLevelBand7;
	double		MinDetectedRadianceLevelBand7;
	double		MaxDetectedRadianceLevelBand8;
	double		MinDetectedRadianceLevelBand8;
} xxx_MinMaxRadiance_TYPE;

typedef struct xxx_MinMaxPixel_TYPE {
	double		MaxPixelBand1;
	double		MinPixelBand1;
	double		MaxPixelBand2;
	double		MinPixelBand2;
	double		MaxPixelBand3;
	double		MinPixelBand3;
	double		MaxPixelBand4;
	double		MinPixelBand4;
	double		MaxPixelBand5;
	double		MinPixelBand5;
	double		MaxPixelBand6L;
	double		MinPixelBand6L;
	double		MaxPixelBand6H;
	double		MinPixelBand6H;
	double		MaxPixelBand7;
	double		MinPixelBand7;
	double		MaxPixelBand8;
	double		MinPixelBand8;
} xxx_MinMaxPixel_TYPE;

typedef struct xxx_ProductParmeters_TYPE {
	char		CalibrationMethodBias[4];
	char		CalibrationMethodGain[4];
	char           	Band1_Gain[2];
	char            Band2_Gain[2];
        char           	Band3_Gain[2];
        char           	Band4_Gain[2];
        char           	Band5_Gain[2];
        char            Band6_Gain1[2];
        char		Band6_Gain2[2];
        char		Band7_Gain[2];
        char		Band8_Gain[2];
        double		SunAzimuth; 
        double		SunElevation; 
 	char		OutputFormat[4];
} xxx_ProductParmeters_TYPE;

typedef struct xxx_CorrectionsApplied_TYPE {
        char		Striping[2];
        char		Banding[2];
        char		CoherentNoise[2];
        char		MemoryEffect[2];
        char           	ScanCorrelatedShift[2];
        char           	InoperableDetectors[2];
        char		DroppedLines[2];
} xxx_CorrectionsApplied_TYPE;

typedef struct xxx_ProjectionParameters_TYPE {
        char		ReferenceDatum[8];
        char		ReferenceEllipsoid[8];
        double		GridCellSizePan;
        double		GridCellSizeThm;
        double		GridCellSizeRef;
        char		Orientation[4];
        char           	ResamplingOption[4];
        char           	MapProjection[4];
} xxx_ProjectionParameters_TYPE;

typedef struct xxx_LCC_Parameters_TYPE {
	double		LatitudeOfFirstStandardParallel;
	double		LatitudeOfSecondStandardParallel;
	double		LongitudeOfCentrolMeridian;
	double		LatitudeOfProjectionOrigin;
	double		FalseEasting;
	double		FalseNorthing;
	char		FalseEastingNorthingUnits[8];
} xxx_LCC_Parameters_TYPE;

typedef struct xxx_TM_Parameters_TYPE {
	double		ScaleFactorAtCentrolMeridian;
	double		LongitudeOfCentrolMeridian;
	double		LatitudeOfProjectionOrigin;
	double		FalseEasting;
	double		FalseNorthing;
	char		FalseEastingNorthingUnits[8];
} xxx_TM_Parameters_TYPE;

typedef struct xxx_PC_Parameters_TYPE {
	double		LongitudeOfCentrolMeridian;
	double		LatitudeOfProjectionOrigin;
	double		FalseEasting;
	double		FalseNorthing;
	char		FalseEastingNorthingUnits[8];
} xxx_PC_Parameters_TYPE;

typedef struct xxx_PS_Parameters_TYPE {
	double		VerticalLongitudeFromPole;
	double		LatitudeOfTrueScale;
	double		FalseEasting;
	double		FalseNorthing;
	char		FalseEastingNorthingUnits[8];
} xxx_PS_Parameters_TYPE;

typedef struct xxx_OMA_Parameters_TYPE {
	double		LongitudeFirstPointGeodetic;
	double		LatitudeFirstPointGeodetic;
	double		LongitudeSecondPointGeodetic;
	double		LatitudeSecondPointGeodetic;
} xxx_OMA_Parameters_TYPE;

typedef struct xxx_OMB_Parameters_TYPE {
	double		AngleOfAzimuth;
	double		LongitudeAlongProjection;
} xxx_OMB_Parameters_TYPE;

typedef struct xxx_OM_Parameters_TYPE {
	double		ScaleFactorAtCenterOfProjection;
	double		LatitudeOfProjectionOrigin;
	double		FalseEasting;
	double		FalseNorthing;
	char		FalseEastingNorthingUnits[8];
	char		OM_Type[2];
	union s_tag {
		xxx_OMA_Parameters_TYPE	oma;
		xxx_OMB_Parameters_TYPE	omb;
	} s;
} xxx_OM_Parameters_TYPE;

typedef struct xxx_UTM_Parameters_TYPE {
	long		zone_number;
} xxx_UTM_Parameters_TYPE;

typedef struct xxx_Parameters_TYPE {
	long		GroupFound;	/* 0 = LCC, 1 = TM. 2 = PC, 3 = PS, 4 = OM, 5 = UTM, 6 = SOM */
	union u_tag {
		xxx_LCC_Parameters_TYPE	lcc;
		xxx_TM_Parameters_TYPE	tm;
		xxx_PC_Parameters_TYPE	pc;
		xxx_PS_Parameters_TYPE	ps;
		xxx_OM_Parameters_TYPE	om;
		xxx_UTM_Parameters_TYPE	utm;
	} u;
} xxx_Parameters_TYPE;

typedef struct xxx_LPGSMetaDataFile_TYPE {
	xxx_LPGSMetadataFileInfo_TYPE	MetadataFileInfo;
	xxx_LPGSProductMetaData_TYPE	ProductMetaData;
	xxx_MinMaxRadiance_TYPE		MinMaxRadiance;
	xxx_MinMaxPixel_TYPE		MinMaxPixel;
	xxx_ProductParmeters_TYPE	ProductParmeters;
	xxx_CorrectionsApplied_TYPE	CorrectionsApplied;
	xxx_ProjectionParameters_TYPE	ProjectionParameters;
	xxx_Parameters_TYPE		Parameters;
} xxx_LPGSMetaDataFile_TYPE;

xxx_LPGSMetaDataFile_TYPE *xxx_ReadLPGSMetadata(char *, char *);

#endif
