/******************************************************************************
                Landsat 7 Image Assessment System (IAS) Software
                        Property of the U.S. Government
                            NASA / GSFC / Code 510
*******************************************************************************

UNIT NAME:  xxx_RecursiveDeletion.c

PURPOSE:  Delete the Pathname using UNIX rm -rf command.

INVOCATION METHOD:
         status = xxx_RecursiveDeletion(p_Pathname, p_ErrorMessage) 
         
RETURN VALUE:
Type = int

Value          Description
-------------- ----------------------------------------------------------------
XXXSUCCESS     Pathname deleted
ERROR          I/O error deleting pathname

ARGUMENT LIST:
Name            Type            Use          Description
--------------- --------------- ---          ---------------------------------------
p_Pathname        char *        I            Path name
p_ErrorMessage    char *        O            Error message

EXTERNAL FILE/RECORD/DBTABLE REFERENCES:
Name                      Use     Description
------------------------- ---     -------------------------------------------------
file/directory            O       File or directory being deleted

EXTERNAL VARIABLES:
Source          Name                Type            Use Description
--------------- ------------------- --------------- --- -----------------------
none

EXTERNAL UNIT/MACRO/PROCEDURE/FUNCTION REFERENCES:
Name                          Description
----------------------------- -------------------------------------------------
XXX_RIGHT_SHIFT               Used to get the system() status

ABNORMAL TERMINATION CONDITIONS, ERROR MESSAGES:
I/O error

ASSUMPTIONS, CONSTRAINTS, RESTRICTIONS:
none

DEVELOPMENT HISTORY:
Author            Change-ID   Release Date       Description of Change
----------------- ----------- ------- ---------- ------------------------------
J. Kerich                     R1.0    04/18/97   Original Implemention
NOTES:

Unit xxx_RecursiveDeletion.c PDL:

         Build rm -rf command using p_Pathname
         Call system() to run UNIX command
1        IF I/O error THEN
                  Build error message is p_ErrorMessage
                  RETURN ERROR
2        ENDIF

         RETURN XXXSUCCESS

End Unit xxx_RecursiveDeletion.c
******************************************************************************/

/* System headers */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>

/* Local headers */
#include "xxx_Types.h"
#include "xxx_RecursiveDeletion.h"

int xxx_RecursiveDeletion(char *p_Pathname, char *p_ErrorMessage)
{
	char DeleteCommand[MAXPATHLEN];
	int  status;

	(void)sprintf(DeleteCommand, "/bin/rm -fr %s 2>/dev/null 1>/dev/null", p_Pathname);
	if ((XXX_RIGHT_SHIFT(status = system(DeleteCommand))) != 0)
	{
		(void)sprintf(p_ErrorMessage, "Delete failed for %s, status was: %d\n", p_Pathname, status);
		return (ERROR);
	}

	return (XXXSUCCESS);
}
