#include "arctables.h"

/* Define Arc datum abbreviations listed by LAS datum num (- 100) */
const char *arc_datums[] = 
    {
    /* 100 */  "ADI_A", "ADI_B", "ADI_C", "ADI_D", "ADI_E", 
    /* 105 */  "ADI_F", "ADI_M", "AFG",   "AIA",   "AIN_A", 
    /* 110 */  "AIN_B", "AMA",   "ANO",   "ARF_A", "ARF_B", 
    /* 115 */  "ARF_C", "ARF_D", "ARF_E", "ARF_F", "ARF_G",
    /* 120 */  "ARF_H", "ARF_M", "ARS",   "ASC",   "ASM", 
    /* 125 */  "ASQ",   "ATF",   "AUA",   "AUG",   "BAT", 
    /* 130 */  "BER",   "BID",   "BOO",   "BUR",   "CAC", 
    /* 135 */  "CAI",   "CAO",   "CAP",   "CAZ",   "CCD",
    /* 140 */  "CGE",   "CHI",   "CHU",   "COA",   "DAL",
    /* 145 */  "DID",   "DOB",   "EAS",   "ENW",   "EST", 
    /* 150 */  "EUR_A", "EUR_B", "EUR_C", "EUR_D", "EUR_E", 
    /* 155 */  "EUR_F", "EUR_G", "EUR_H", "EUR_I", "EUR_J", 
    /* 160 */  "EUR_K", "EUR_L", "EUR_M", "EUR_S", "EUR_T",
    /* 165 */  "EUS",   "FAH",   "FLO",   "FOT",   "GAA", 
    /* 170 */  "GEO",   "GIZ",   "GRA",   "GSE",   "GUA", 
    /* 175 */  "HEN",   "HIT",   "HER",   "HJO",   "HKD", 
    /* 180 */  "HTN",   "IBE",   "NOTDEFINED",   "IND_B", "IND_I",
    /* 185 */  "NOTDEFINED", "INF_A", "ING_B", "ING_A", "INH_A",
    /* 190 */  "IRL",   "ISG",   "IST",   "JOH",   "KAN", 
    /* 195 */  "KEA",   "KEG",   "NOTDEFINED", "KUS",   "LCF", 
    /* 200 */  "LEH",   "LIB",   "LUZ_A", "LUZ_B", "MAS", 
    /* 205 */  "MER",   "MID",   "MIK",   "MIN_A", "MIN_B",
    /* 210 */  "MOD",   "MPO",   "MVS",   "NAH_A", "NAH_B", 
    /* 215 */  "NAH_C", "NAP",   "NAR_A", "NAR_B", "NAD83", 
    /* 220 */  "NAR_D", "NAR_E", "NAR_H", "NAS_A", "NAS_B", 
    /* 225 */  "NAD27", "NAS_D", "NAS_E", "NAS_F", "NAS_G",
    /* 230 */  "NAS_H", "NAS_I", "NAS_J", "NAS_L", "NAS_N", 
    /* 235 */  "NAS_O", "NAS_P", "NAS_Q", "NAS_R", "NAS_T", 
    /* 240 */  "NAS_U", "NAS_V", "NAS_W", "NSD",   "OEG", 
    /* 245 */  "OGB_A", "OGB_B", "OGB_C", "OGB_D", "OGB_M", 
    /* 250 */  "OHA_A", "OHA_B", "OHA_C", "OHA_D", "OLD_HAWAIIAN",
    /* 255 */  "NOTDEFINED", "NOTDEFINED", "NOTDEFINED", "PHA",   "PIT", 
    /* 260 */  "PLN",   "POS",   "PRP_A", "PRP_B", "PRP_C", 
    /* 265 */  "PRP_D", "PRP_E", "PRP_F", "PRP_G", "PRP_H",
    /* 270 */  "PRP_M", "PTB",   "PTN",   "PUR",   "PUK", 
    /* 275 */  "QAT",   "QUO",   "REU",   "SAE",   "SAN_A", 
    /* 280 */  "SAN_B", "SAN_C", "SAN_D", "SAN_E", "SAN_F", 
    /* 285 */  "SAN_G", "SAN_H", "SAN_I", "SAN_J", "SAN_K",
    /* 290 */  "SAN_L", "SAN_M", "SAO",   "SAP",   "SCK", 
    /* 295 */  "SGM",   "SHB",   "SRL",   "SOA",   "SPK_F", 
    /* 300 */  "SPK_C", "SPK_A", "SPK_E", "SPK_D", "SPK_B", 
    /* 305 */  "SPK_G", "TAN",   "TDC",   "TIL",   "TOY_A",
    /* 310 */  "TOY_B", "TOY_C", "TOY_M", "TRN",   "VOR", 
    /* 315 */  "NWL9D", "WGS72", "WGS84", "WAK",   "YAC",
    /* 320 */  "ZAN" 
    };

/* Define Arc spheroid names listed by LAS spheroid number */
const char *arc_spheroids[] = 
    {
    "CLARKE1866",       /* 0:CLARKE 1866:6378206.4:294.9786982: */
    "CLARKE1880",       /* 1:CLARKE 1880:6378249.145:293.465: */
    "BESSEL",           /* 2:BESSEL:6377397.155:299.1528128: */
    "INT1967",          /* 3:INTERNATIONAL 1967:6378157.5:298.25: */
    "INT1909",          /* 4:INTERNATIONAL 1909:6378388.0:297: */
    "WGS72",            /* 5:WGS 72:6378135.0:298.26: */
    "EVEREST",          /* 6:EVEREST:6377276.3452:300.8017: */
    "WGS66",            /* 7:WGS 66:6378145.0:298.25: */
    "GRS80",            /* 8:GRS 1980:6378137.0:298.257222101: */
    "AIRY",             /* 9:AIRY:6377563.396:299.3249646: */
    "MODEVEREST",       /* 10:MODIFIED EVEREST:6377304.063:300.8017: */
    "MODAIRY",          /* 11:MODIFIED AIRY:6377340.189:299.3249646: */
    "WGS84",            /* 12:WGS84:6378137.0:298.257223563: */
    "SEASIA",           /* 13:SOUTHEAST ASIA(MODIFIED FISCHER 1960)
                             :6378155.0:298.3: */
    "AUSTRALIAN",       /* 14:AUSTRALIAN NATIONAL:6378160.0:298.25: */
    "KRASOVSKY",        /* 15:KRASSOVSKY:6378245.0:298.3: */
    "HOUGH",            /* 16:HOUGH:6378270.0:297.0: */
    "MERCURY1960",      /* 17:MERCURY 1960 (FISCHER 1960):6378166.0:298.3: */
    "MERCURY1968",      /* 18:MODIFIED MERCURY 1968:6378150.0:298.3: */
    "SPHERE",           /* 19:SPHERE:6370997.0:: */
    "BESSELNAMIBIA",    /* 20:BESSEL 1841(NAMIBIA):6377483.865:299.1528128: */
    "EVEREST_SS",       /* 21:EVEREST (SABAH & SARAWAK):6377298.556:300.802:*/
    "EVEREST1956",      /* 22:EVEREST (INDIA 1956):6377301.243:300.8017: */
    "EVEREST1969",      /* 23:EVEREST (MALAYSIA 1969):6377295.664:300.8017: */
    "MODEVEREST",       /* 24:EVEREST (MALAY. & SINGAPORE 1948)
                           :6377304.063:300.8017: */
    "NOTDEFINED",       /* 25:EVEREST (PAKISTAN):6377309.613:300.8017: 
                             (BEST ARC APPROXIMATION:  MODEVEREST) */
    "HAYFORD",          /* 26:HAYFORD:6378388.0:297.0: */
    "HELMERT1906",      /* 27:HELMERT 1906:6378200.0:298.3: */
    "NOTDEFINED",       /* 28:INDONESIAN 1974:6378160.000:298.247: 
                             (BEST ARC APPROXIMATION:  AUSTRALIAN) */
    "SOUTHAMERICAN1969",/* 29:SOUTH AMERICAN 1969:6378160.000:298.25: */
    "NOTDEFINED"        /* 30:WGS 60:6378165.0:298.3: 
                             (BEST ARC APPROXIMATION:  MERCURY1960) */
    };


int num_arc_datums(void) 
/* get total number of datums in arc_datums table */
{
    return sizeof(arc_datums)/sizeof(const char *);
}

int num_arc_spheroids(void) 
/* get total number of spheroids in arc_spheroids table */
{
    return sizeof(arc_spheroids)/sizeof(const char *);
}
