/*******************************************************************************
NAME: CHECKPROJ

PURPOSE:  To check projection information in (*ddr) before creating .prj file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT:    LAS

*******************************************************************************/
#include <stdio.h>
#include <string.h>

#include "las.h"
#include "proj.h"
#include "protos.h"

SETPROJCOEFMAP (coefmap);       /* Coefficient mapping table */

void checkproj
(
    char *projok,
    struct PROJCOEFMAP **cmptr,
    long *projvers,
    struct DDR (*ddr),
    long *pc
)

{
long ii;
char    versflags[11];  /* FLags for projection version */


strcpy(versflags," zothfvxsen");


/*  Determine whether projection data are usable
-------------------------------------------------- */
*projok = TRUE;
*pc = ((*ddr)).proj_code;
if (((*ddr).valid[DDPCV] != VALID) || (*pc == USDEF)
    || (((*ddr).valid[DDPPV] != VALID) && (*pc != GEO) && (*pc != UTM)
    && (*pc != SPCS))
    || (((*ddr).valid[DDZCV] != VALID) && (*pc == SPCS)))
    *projok = FALSE;

/*  For UTM, decide whether to use zone number or lat/long values
-------------------------------------------------------------------*/
else if (*pc == UTM)
    {
    if ((*ddr).valid[DDPPV])
        {
        if (((*ddr).proj_coef[0] == 0.0) && ((*ddr).proj_coef[1] == 0.0))
            {
            if ((*ddr).valid[DDZCV] == VALID)
                strncpy (coefmap[UTM].map, "    ", 4);
            else
                {
                c_lasmsg(FALSE,"las2arc-nozone","UTM projection has neither %s",
                "valid zone code nor internal-point lat/long");
                *projok = FALSE;
                }
            }
        else
            {
            (*ddr).zone_code =(long)(((*ddr).proj_coef[0] + 186.0e+6) / 6.0e+6);
            if (((*ddr).zone_code < 1) || ((*ddr).zone_code > 60))
                {
                (*ddr).valid[DDPPV] = INVAL;
                c_lasmsg(FALSE,"las2arc-badutmzn",
                "(*ddr) proj_coef[0] for UTM gives invalid zone");
                if ((*ddr).valid[DDZCV] != VALID)
                    *projok = FALSE;
                else
                    strncpy (coefmap[UTM].map, "    ", 4);
                }
            else
                (*ddr).valid[DDZCV] = INVAL;
            }
        }
    }

/*  Don't pass meaningless zone code
-------------------------------------*/
else if (((*ddr).valid[DDZCV] == VALID) && (*pc != SPCS))
    (*ddr).valid[DDZCV] = INVAL;

/*  Check whether usable coefficient mapping record is available
-----------------------------------------------------------------*/
if (*projok && ((*ddr).valid[DDPPV] == VALID))
    {
    *cmptr = &coefmap[*pc];
    *projvers = ((*cmptr)->contcnt > 0) ? 1 : 0;
    while ((*cmptr)->contcnt > 0)
        {
        for (ii = 0; (ii < COEFCT)
            && ((*cmptr)->map[ii] != versflags[*projvers]); ii += 2);
        if ((ii < COEFCT) && ((*ddr).proj_coef[ii] == (double) ii))
            (*cmptr)->contcnt = 0;
        else
            {
            (*projvers)++;
            (*cmptr)++;
            for ( ; strcmp (coefmap[*pc].lkey, (*cmptr)->lkey) != 0; (*cmptr)++)
                if (*cmptr  >= &coefmap[ARCPROJCOUNT + EXTRATBLRECS])
                    {
                    *cmptr--;
                    (*cmptr)->contcnt = -1;
                    c_lasmsg(FALSE,"las2arc-badmaptbl",
                    "Mapping table corrupted");
                    }
            }
        }
    if ((*cmptr)->contcnt < 0)
        {
        *projok = FALSE;
        c_lasmsg(FALSE,"las2arc-proj","Unsupported projection type");
        }
    }

return;
}
