/*******************************************************************************
NAME: GETPAR

PURPOSE: gets the user-specified parameters and tae globals for las2arc

PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  ----------------------------------------
  1.0    12/93  K. Lowell          C   Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Must be run under TAE.

PROJECT: LAS

*******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblk,   /* pointer to initial TAE parameter block      */
    char   hosin[],        /* Host input image name                       */
    char   outgrid[],      /* Output Arc Grid location                    */
    long   window[][4],    /* Image processing window                     */
    long   *msgtim,        /* Message processing time                     */
    long   *ep3flag,       /* Arc 8.3/NO8.3 flag                          */
    long   *delflg,        /* Delete intermediate files flag              */
    long   *nbands,        /* Number of bands to read from input image    */
    double *projtol        /* Projection tolerance                        */
)
{
FILE *fp = NULL;
struct VARIABLE *var;      /* ptr to a variable in the vblk               */

long status;               /* function return status                      */
long bands[1][MAXBND+1];   /* Bands to process                            */
long dummy[1][2];          /* Dummy variable for GETIN                    */
long one = 1;              /* Constant of one                             */
long zero = 0;             /* Constant of zero                            */

char taein1[1][CMLEN];     /* TAE input image name                        */
char hosin1[1][CMLEN];     /* HOST input image name                       */
char ep3file[CMLEN];       /* Arc 8.3 flag file                           */
char archome[CMLEN];       /* Arc home directory                          */
char delstr[4];            /* String "YES" or "NO" to set delflg to 1 or 0*/

/* Get the input image NOTE: getnam not used since there are three
possible extensions for the input file.  Extensions are assigned
based on on the band format of INFILE.
-------------------------------------------------------------------*/
var = p_find(vblk, "OUTGRID");
strcpy(outgrid, SVAL(*var, 0));

/* Get the input image
---------------------*/
status = c_getin(vblk,"IN",&zero,taein1,hosin1,dummy[0],window,bands,nbands,
dummy[0],&one,dummy[0],dummy);
if (status != E_SUCC)
    c_lasmsg(TRUE,"las2arc-fatal","Fatal error encountered");
strcpy(hosin,hosin1[0]); 

/* Retrieve globals
-------------------*/
var = p_find(vblk,"$msgtim");
*msgtim = IVAL(*var,0);
var = p_find(vblk,"$PROJTOL");
*projtol = RVAL(*var,0);

/* determine maning convention, 8.3 or NO8.3
---------------------------------------------*/
strcpy(archome,getenv("ARCHOME"));
strcpy(ep3file,archome);
strcat(ep3file,"/sysgen/no8.3");
if ((fp = fopen(ep3file,"r")) == NULL)
    {
    c_lasmsg(FALSE,"las2arc-info","8.3 naming convention in use");
    *ep3flag=1;
    }
else
    {
    fclose(fp);
    c_lasmsg(FALSE,"las2arc-info","NO8.3 naming convention in use");
    *ep3flag=0;
    }


/*  SET DELFLG to 1 if YES or to 0 if NO
( !DELFLG ) will flag main() to create the .stx file 
-----------------------------------------------------------*/
var = p_find(vblk, "DELFLG");
strcpy(delstr, SVAL(*var, 0));

if (!strcmp(delstr,"YES"))
    *delflg=1;
else
    *delflg=0;


return(E_SUCC);
}
