/*****************************************************************************
NAME: GOODKEYSET

PURPOSE:  Set the geoTIFF keys for Interrupted Goode Homolosine projection.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/
#include "las.h"
#include "tiffio.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "protos.h"

void goodkeyset
(
    const struct DDR *ddr,     /* I: LAS image file descriptor record    */
    GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
) 

{
    /* Goode Homolosine is not a known projection in the current
       version of geoTIFF.  The sphere should be all that is
       required for this projection.
       --------------------------------------------------------- */
    GTIFKeySet(gtif, ProjectedCSTypeGeoKey, TYPE_SHORT, 1, 32767);
    GTIFKeySet(gtif, PCSCitationGeoKey, TYPE_ASCII, 0,
            "Goode Homolosine--Incomplete implementation in geoTIFF");

    /* Set the major and minor axis values
       ----------------------------------- */
    set_major_minor(ddr, gtif);

	//CR2912
    GTIFKeySet(gtif, GeographicTypeGeoKey, TYPE_SHORT, 1, 
                   get_geocode(ddr));

    if (ddr->valid[3] == 0)
    {
        /* Set the GeographicTypeGeoKey only when projection coefficients are 
           invalid (for consistency with algorithm in original development)
           ---------------------------------------------------------------- */
        GTIFKeySet(gtif, GeographicTypeGeoKey, TYPE_SHORT, 1, 
                   get_geocode(ddr));
    }
    GTIFKeySet(gtif, GeogLinearUnitsGeoKey, TYPE_SHORT, 1,
            get_linear_units(ddr->proj_units));
}
