#ifndef PROTOS_H
#define PROTOS_H
#include "ltable.h"
#include "xtiffio.h"
#include "geotiffio.h"


void alaskakeyset
(
const struct DDR *ddr,     /* I:  LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor            */
);

void alberskeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void azmeqdkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void eqrectkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void equidckeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void geokeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);


unsigned short get_angular_units
(
    const char *unitstr	  /* I: Units used (in English)              */
);

unsigned short get_geocode
(
    const struct DDR *ddr /* I: LAS image file descriptor record     */
);

unsigned short get_linear_units
(
    const char *unitstr	  /* I: Units used (in English)              */
);

unsigned short get_stateplanezone
(
    const struct DDR *ddr /* I: LAS image file descriptor record     */
);

void get_tiepoints
(
const struct DDR *ddr,       /* I: LAS image file descriptor record     */
double tiepoints[][6],       /* O: coordinate array for GEOTIEPOINTS    */
long *size                   /* O: number of elements in array          */
);

unsigned short get_utmzone 
(
    const struct DDR *ddr    /* I: LAS image file descriptor record     */
);

unsigned short get_utmzonegen
(
    const struct DDR *ddr    /* I: LAS image file descriptor record     */
);

int getpar
(
  struct PARBLK *vblock,     /* I: TAE parameter block                      */
  char          infiles[][CMLEN],/* O: Input LAS file(s)                    */
  long          *num_infiles,/* O: number of input image files              */
  char          outfile[],   /* O: Output host TIFF file                    */
  char          worldfile[], /* O: World file name                          */
  long          *imagelength,/* O: Common length of all input images        */
  long          *imagewidth, /* O: Common width  of all input images        */
  long          *sl,         /* O: Starting line of each LAS input image    */
  long          *ss,         /* O: Starting sample of each LAS input image  */
  long          *nbands,     /* O: Num bands to read from each input image  */
  long          *msgtim,     /* O: # of seconds between processing messages */
  long          bands[][MAXBND+1],/* O: Array to hold the bands for
                                   each input image                         */
  long          window[][4], /* O: Array to hold the windows (sl ss nl ns)  */
  long          *total_band_count/* O: total number of bands in image list  */
);

void gnomonkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void goodkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void gvnspkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void hammerkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void homkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void imollkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void lamazkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void lamcckeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void mercatkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void millerkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void mollkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void obeqakeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void orthokeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void polyckeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void pskeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void robinkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void set_major_minor
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void snsoidkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void somkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void spcskeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void stereokeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void tmkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void utmkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void vgrintkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void wagivkeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

void wagviikeyset
(       
const struct DDR *ddr,     /* I: LAS image file descriptor record    */
GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
);

#endif
