/*****************************************************************************
NAME: UTMKEYSET

PURPOSE:  Set the geoTIFF keys for Universal Transverse Mercator projection.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/
#include "las.h"
#include "datum.h"
#include "tiffio.h"
#include "xtiffio.h"
#include "geotiffio.h"
#include "protos.h"

void utmkeyset
(
    const struct DDR *ddr,     /* I: LAS image file descriptor record    */
    GTIF *gtif                 /* I/O: GeoKey-level descriptor           */
) 

{
    unsigned short ellipse=0;  /* geoTIFF code for ellipse            */

    /* Make sure warning about projection information is generated even if
       get_geocode or get_utmzone is able to return a valid geoTIFF code for
       the datum or zone since UTM is a special case and support for UTM using
       an ellipsoid is not provided in the current geoTIFF spec. */
    if(ddr->datum_code < MINDTM) /* if datum code is an ellipsoid */
    {
        c_lasmsg(FALSE, "las2geotiff-utm",
                 "UTM image uses an ellipsoid. Processing will continue but "
                 "projection information may be incomplete.");
    }

    if ((ddr->datum_code == 5) || (ddr->datum_code == 12) || 
            ((ddr->datum_code >= 217) && (ddr->datum_code <= 222)) ||
            ((ddr->datum_code >= 223) && (ddr->datum_code <= 242)) ||
            (ddr->datum_code == 316) || (ddr->datum_code == 317))
    {
        GTIFKeySet(gtif, GTRasterTypeGeoKey, TYPE_SHORT, 1, RasterPixelIsArea);
        GTIFKeySet(gtif, GTModelTypeGeoKey, TYPE_SHORT, 1, ModelTypeProjected);
        GTIFKeySet(gtif, ProjectedCSTypeGeoKey,TYPE_SHORT, 1,
                get_utmzone(ddr));
    }
    else
    {
        GTIFKeySet(gtif, GTRasterTypeGeoKey, TYPE_SHORT, 1, RasterPixelIsArea);
        GTIFKeySet(gtif, GTModelTypeGeoKey, TYPE_SHORT, 1, ModelTypeProjected);
        GTIFKeySet(gtif, ProjectedCSTypeGeoKey,TYPE_SHORT, 1, 32767);

        /* Let user know ProjectionGeoKey will be Unknown */
        c_lasmsg(FALSE, "las2geotiff-zone",
                 "LAS image contains generic LAS UTM zone. Projection "
                 "information may be incomplete.");
        GTIFKeySet(gtif, ProjectionGeoKey,TYPE_SHORT, 1, get_utmzonegen(ddr));

        ellipse = get_geocode(ddr);
        if (ellipse != 0)
            GTIFKeySet(gtif, GeographicTypeGeoKey,TYPE_SHORT, 1, ellipse);
    }
    if (ddr->valid[4])
        GTIFKeySet(gtif, GeogLinearUnitsGeoKey, TYPE_SHORT, 1,
                get_linear_units(ddr->proj_units));
    GTIFKeySet(gtif, PCSCitationGeoKey, TYPE_ASCII, 0,
            "UTM converted from LAS");
}
