/*****************************************************************************
NAME: CREATEDDR

PURPOSE:  This will create the temporary ddr file and create the filenames
          for the IN parameter for TRANSLAS-OUT.

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 1.0     02/98 T.Ratliff         C    Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int createddr
(
    struct PARBLK *vblock,                   /* TAE parameter block        */
    char *hosout,                            /* UNIX format input file     */
    char *outddr,                            /* temporary ddr file name    */
    char fnames[][CMLEN],                    /* file names for translas    */
    long *nfiles,                            /* # of files in fnames       */
    char *bform                              /* data format of the files   */
)

{
    long nl = 0;                           /* number of lines            */
    long ns = 0;                           /* number of samples          */
    long sl = 0;                           /* starting line              */
    long ss = 0;                           /* starting sample            */
    long nmrimg = 0;                       /* # of images in IN          */
    long totbnd = 0;                       /* total # of bands           */
    long status = 0;                       /* return status of functions */
    long one = 1;                          /* constant value of 1        */
    long fatal = -1;                       /* fatal error constant       */
    long i = 0;                            /* loop counter               */
    long image = 0;                        /* loop index counting images */
    long band = 0;                         /* loop index counting bands  */
    long access = 0;                       /* read/write access type     */
    long zero = 0;                         /* constant value of zero     */
    long type = 0;                         /* data type of image         */
    long upddr_flag = 0;                   /* flag for update ddr        */
    long opt[3];                           /* compression option array   */
    struct FDESC *fdesc[3];                /* image file descriptors     */
    long window[MAXIMG][4];                /* window specs on input image*/
    long bands[MAXIMG][MAXBND+1];          /* bands read for each image  */
    long offset1[MAXIMG*MAXBND];           /* offsets for input bands    */
    long offset2[MAXBND];                  /* offsets for output bands   */
    long nbands[MAXIMG];                   /* number of bands per image  */

    double line_inc = 0.0;                 /* line increment             */
    double samp_inc = 0.0;                 /* sample increment           */

    char hosin[MAXIMG][CMLEN];             /* UNIX format of input images*/
    char taein[MAXIMG][CMLEN];             /* TAE format of input images */
    char path[CMLEN];                      /* path of hosout             */
    char root[CMLEN];                      /* root name of hosout        */
    char exten[CMLEN];                     /* extension of hosout        */

    /* Get parameters and globals
       -----------------------------*/
    getpar(vblock,taein,hosin,hosout,&nmrimg,window,bands,nbands,
            &totbnd,bform);

    /* Creating the new ddr file
       ----------------------------*/
    access = IREAD;
    nl = window[0][NL];
    ns = window[0][NS];
    for (image = 0,band = 0; image < nmrimg; image++)
    {
        sl = window[image][0];
        ss = window[image][1];
        status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,
                &ns,&offset1[band],&access,&type,&zero,&zero);
        band += nbands[image];
    }

    c_splitname(hosout,path,root,exten);
    sprintf(outddr,"%s/t_e_m_p_las",path);
    c_creat_name(outddr,".img");
    access = IWRITE;
    opt[0] = 0;
    opt[1] = NO_COMPRES;
    opt[2] = 0;
    status = c_eopens(&fdesc[nmrimg],outddr,&totbnd,&one,&one,&nl,&ns,
            offset2,&access,&type,&zero,opt);
    fdesc[nmrimg+1] = 0;

    /* Updating the new ddr file
       ----------------------------*/
    line_inc = (double)one;
    samp_inc = (double)one;
    upddr_flag = NOCOMB;
    status = c_upddr(vblock,hosin,outddr,&nmrimg,window,bands,nbands,&line_inc,
            &samp_inc,&totbnd,&upddr_flag);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);

    status = c_eclose(fdesc);

    /* Creating the filenames to separate the bands
       -----------------------------------------------*/
    image = 0;
    if (strcmp(bform,"BSQ") == 0)
    {
        band = 0;
        for (i = 0; i < totbnd; i++)
        {
            sprintf(fnames[i],"%s(%d,%d,%d,%d:%d)",taein[image],
                    window[image][SL],window[image][SS],window[image][NL],
                    window[image][NS],bands[image][band]);
            band++;
            if (band == nbands[image])
            {
                *nfiles += band;
                band = 0;
                image++;
            }
        }
    }
    else
    {
        sprintf(fnames[0],"%s(%d,%d,%d,%d)",taein[image],window[image][SL],
                window[image][SS],window[image][NL],window[image][NS]);
        *nfiles = 1;
    }

    return(E_SUCC);
}
