/*****************************************************************************
NAME: DMS2MET

PURPOSE:  Converts a packed degree-minute-second number to meters and also
          converts pdms to NDF dms format.

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 1.0     01/98 T.Ratliff         C    Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "protos.h"

void dms2met
(
    double tmpdms[],                      /* contains pdms corner points      */
    char llpt[][15],                      /* longitude/latitude corner points */
    double nept[],                        /* northing/easting corner points   */
    struct DDR *ddr                       /* DDR record structure             */
)
{
    struct DDR outddr;                  /* temporary DDR record structure   */
    long prtprm[2];                     /* print flag for c_transinit       */
    long fatal = -1;                    /* fatal error flag                 */
    long status = 0;                    /* return status of functions       */
    long in_units = 0;                  /* units converting from (pdms)     */
    long out_units = 0;                 /* units converting to (meters)     */
    long i = 0;                         /* loop counter                     */

    /* Initializing parms for c_transinit
       ----------------------------------*/
    prtprm[GEO_TERM] = 0;
    prtprm[GEO_FILE] = 0;
    outddr.proj_code = 1;
    out_units = 2;
    in_units = 5;
    outddr.zone_code = ddr->zone_code;
    outddr.datum_code = ddr->datum_code;
    for (i = 0; i < 15; i++)
        outddr.proj_coef[i] = ddr->proj_coef[i];

    status = c_transinit(&ddr->proj_code,&in_units,&ddr->zone_code,
            &ddr->datum_code,ddr->proj_coef,&outddr.proj_code,
            &out_units,&outddr.zone_code,&outddr.datum_code,
            outddr.proj_coef,prtprm,"");
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);

    /* Converting from dms to meters
       --------------------------------*/
    if (c_trans(&ddr->proj_code,&in_units,
            &outddr.proj_code,&out_units,
            &tmpdms[0],&tmpdms[1],&nept[0],&nept[1]) != E_SUCC)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);

    if (c_trans(&ddr->proj_code,&in_units,
            &outddr.proj_code,&out_units,
            &tmpdms[2],&tmpdms[3],&nept[2],&nept[3]) != E_SUCC)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);

    if (c_trans(&ddr->proj_code,&in_units,
            &outddr.proj_code,&out_units,
            &tmpdms[4],&tmpdms[5],&nept[4],&nept[5]) != E_SUCC)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);

    if (c_trans(&ddr->proj_code,&in_units,
            &outddr.proj_code,&out_units,
            &tmpdms[6],&tmpdms[7],&nept[6],&nept[7]) != E_SUCC)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);


    /* Setting Lat/Long value for NDF header
       ----------------------------------------*/
    pdms2dms(tmpdms[0], llpt[0],0);
    pdms2dms(tmpdms[1], llpt[1],1);
    pdms2dms(tmpdms[2], llpt[2],0);
    pdms2dms(tmpdms[3], llpt[3],1);
    pdms2dms(tmpdms[4], llpt[4],0);
    pdms2dms(tmpdms[5], llpt[5],1);
    pdms2dms(tmpdms[6], llpt[6],0);
    pdms2dms(tmpdms[7], llpt[7],1);

}
