/******************************************************************

 NAME: GETPAR
 FUNCTION: Get the input parameters

 COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
              Must be run under TAE

 PROJECT: LAS

*********************************************************************/

#include "las.h"
#include "util.h"
#include "labserv.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,       /* pointer to TAE parameter block            */
    char taein[][CMLEN],         /* TAE file names of input images            */
    char hosin[][CMLEN],         /* host file names of input images           */
    char *hosout,                /* host file name of output image            */
    long *nmrimg,                /* number of input images                    */
    long (*window)[4],           /* window specifications for each input image*/
    long (*bands)[MAXBND+1],     /* band specifications for each input image  */
    long *nbands,                /* number of bands per input image           */
    long *totbnd,                /* total number of bands to be processed     */
    char *bform                  /* data format of the files                  */
)

{
    long element = 0;               /* parameter element number         */
    long fatal = -1;                /* fatal error flag for errmsg      */
    long nonfatal = 1;              /* nonfatal error flag for errmsg   */
    long imglim = 0;                /* read access flag                 */
    long status = 0;                /* status flag                      */
    long same = 0;                  /* checks if window sizes are equal */
    long zero = 0;                  /* constant value 0                 */
    long qflag[MAXIMG][2];          /* indicates if subwindows specified*/

    char taeout[CMLEN];             /* TAE format output name           */
    char msg[CMLEN];                /* message string for c_errmsg      */

    struct VARIABLE *var;

    /* get the NDF header format header name
       ----------------------------------------*/
    status = c_getout(vblock,"OUT",(char (*)[256])taeout,(char (*)[256])hosout);
    if (status == E_FAIL)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);
    status = c_lsstat(hosout,&zero);
    if (status != E_FAIL)
    {
        sprintf(msg,"Error: %s already exists",taeout);
        c_errmsg(msg,"las2ndf-exists",&nonfatal);
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);
    }

    /* Getting the input LAS image name along with
       the sub-banding and sub-windowing information
       ------------------------------------------------*/
    imglim = MAXIMG;
    status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
            nbands, totbnd,&imglim,&same,qflag);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);
    if (same == 0)
    {
        c_errmsg("Input images do not have same window sizes","las2ndf-window",
                &nonfatal);
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);
    }

    /* Getting the data format
       --------------------------*/
    var = p_find(vblock,"BFORM");
    strcpy(bform,SVAL(*var,0));

    return(E_SUCC);
}
