/*****************************************************************************
NAME: SETUP2NHDR

PURPOSE:  This will create the temporary ddr file, create the filenames for
          use in TRANSLAS-OUT, and it will create the NDF header file.

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 1.0     02/98 T.Ratliff         C    Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int main(void)
{
    struct PARBLK vblock;                    /* TAE parameter block        */

    long status = 0;                         /* return status of functions */
    long fatal = -1;                         /* fatal error flag           */
    long i = 0;                              /* loop counter               */
    long nb = 0;                             /* number of bands            */
    long nfiles = 0;                         /* number of files            */
    long std_out = 0;                        /* LUN to standard output     */

    char directory[CMLEN];                   /* UNIX path structure        */
    char root[CMLEN];                        /* name of the file           */
    char exten[CMLEN];                       /* extension of the file      */
    char outddr[CMLEN];                      /* name of temporary ddr file */
    char hosout[CMLEN];                      /* UNIX format of NDF header file*/
    char tmpfile[CMLEN];                     /* output ppf file name       */
    char fnames[300][CMLEN];                 /* array of filenames         */
    char bform[5];                           /* data format of the files   */

    FILE *outf;                              /* file pointer to ppf file   */

    /* Initialize TAE parameter block
       ---------------------------------*/
    c_inlas(&vblock,&std_out);

    /* Creating the temporary ddr file and getting the
       filenames for calls to TRANSLAS-OUT
       --------------------------------------------------*/
    createddr(&vblock,hosout,outddr,fnames,&nfiles,bform);

    /* Creating the NDF header file
       -------------------------------*/
    ddr2nhdr(hosout,outddr,&nb,bform);

    /* Deleting the temporary ddr file
       ----------------------------------*/
    c_delfil(outddr);

    /* Putting the file names into a ppf file
       -----------------------------------------*/
    c_splitname(hosout,directory,root,exten);
    sprintf(tmpfile,"%s%s.ppf",directory,root); 
    outf = fopen(tmpfile,"w");
    if (strcmp(bform,"BSQ") == 0)
    {
        for (i = 0; i < nfiles; i++)
            fprintf(outf,"FNAMES%d            S 1 %s\n",i+1,fnames[i]);
    }
    else
        fprintf(outf,"FNAMES1            S 1 %s\n",fnames[0]);

    fclose(outf);

    /* Sending the number of bands back to TAE
       ------------------------------------------*/
    status = q_intg(&vblock,"NB",1,&nb,P_UPDATE);
    if (status == E_FAIL)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);
    status = q_out(&vblock);
    if (status == E_FAIL)
        c_errmsg("Fatal error encountered","las2ndf-fatal",&fatal);

    return(E_SUCC);
}
