#!/usr/local/bin/perl
#
# FUNCTION:     make_local
#
# USAGE:        make_local <orig filename> <current dir> <home dir>
#
# DESCRIPTION:  This program is used to convert absolute path names in the
#               orig files that are 'diff'ed.  If the path names are all
#               different, the user will have to wade through thousands of
#               lines to look for actual problems.  Instead of overwriting
#               the specified orig file, a new local file is created.
#               NOTE: This will only work on text files.
#
#               Its scheme is as follows:
#
#               <orig filename> is copied to <local filename>
#                 --> origxxxx is copied to localxxxx
#               <current dir> is the path of the current working directory
#               <home dir> is the path of the user's home directory
#
#    xxxxx:     Any occurance of xxxxx in <local filename> will be replaced
#               with $cwd.
#
#    vvvvv:     Any occurance of vvvvv in <local filename> will be replaced
#               with $cwd with $cwd's first '/' removed and subsequent '/'s
#               replaced with '.'s.  (This is due to the VAXish filenaming
#               conventions)
#
#*****************************************************************************

# Check if improper usage of program.
if ($#ARGV != 2)
{
    print "Usage: make_local <orig filename> <current dir> <home dir>\n";
    exit 1;
}

my $orig_filename = $ARGV[0];
my $cwd = $ARGV[1];
my $home = $ARGV[2];

# Determine the local filename by replacing orig* with local*.  This
# replacement should only be at the beginning of the filename.
my $local_filename = $orig_filename;
$local_filename =~ s/^orig/local/;

# Building unix_cwd.  Must add backslashes before slashes.
my @dir_array = split m%/%, $cwd;
my $unix_cwd = join "\\\\/", @dir_array;

# Changing all xxxxx to unix_cwd.
system "sed s/xxxxx/$unix_cwd/g $orig_filename > $local_filename";

# Building vax_cwd.  First checking if below home directory.
my $INHOME = 0;
if ($cwd =~ /^$home/)
{
    $INHOME = 1;
}

my $NUM_SLASHES = 1;
if ($INHOME)
{
    $NUM_SLASHES = split(m%/%, $home) - 1;
}

my $vax_cwd = join ".", @dir_array[$NUM_SLASHES..$#dir_array];

# Changing all vvvvv to vax_cwd.
system "sed s/vvvvv/$vax_cwd/g $local_filename > temp; mv temp $local_filename";
