#include "linerepr.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char taein[][CMLEN],		/* tae name of the input image              */
    char hosin[][CMLEN],		/* host name of the input image             */
    char taeout[][CMLEN],		/* tae name of the output image             */
    char hosout[][CMLEN],		/* host name of the output image            */
    long *dtype,			/* data type of the input image              */
    long *bands_per_img, 	/* number of bands per input imgae           */
    long (*img_flag)[2],		/* specified image/band options              */
    long img_limit,		/* maximum number of images                  */
    long (*num_img_band)[MAXBND+1],	/* band numbers for each input image */
    long *num_inimg,		/* total number of input images              */
    long *same_wnd, 		/* same window size? 1=Y 2=N                 */
    long *tot_inbands,		/* total number of input bands               */
    long *badlin,		/* starting bad line                         */
    long *lineinc		/* bad line increment                        */
)

/*
*****************************************************************************

NAME:	 GETPAR	

PURPOSE: Retrieves the user supplied parameters. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          03-Nov-81    E. Harris     
   2          25-Nov-86    R. McKinney        EDC       
   3          01-Mar-90    K. Kunska          Conversion to 5.0   
   4 	      15-Mae-90    K. Kunska          Corrected a parameter name
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Retrieve tae parameter block.
   Parse the input image specification.
   If the return status is not success then abort with a fatal error message.
   Determine the input image type.
   If the data type exceeds EWORD, abort with an error message.
   Parse the output image specification.
   If the return status is not success then abort with a fatal error message.
   Get starting bad line, and its increment.
   Validate the starting bad line.
   Get the LAS global paramters.

*****************************************************************************
*/

{

char	mnmx[4],		/* minmax flag                               */
	msg[80];		/* error message string                      */

long	i,			/* loop counter and array index              */
	partype,		/* data type of a parameter                  */
	status,			/* return status                             */
	std_out;		/* logical unit for standard output          */

long    fatal_err = -1;
long    zero = 0;
long    one = 1;


struct VARIABLE   *var;	/* pointer to a variable in the tae block     */

/*
Retrieve tae parameter block.
------------------------------
*/
img_limit = MAXIMG;
c_inlas (&block, &std_out); 

/*
Parse the input image specification.
If the return status is not success then abort with a fatal error message.
---------------------------------------------------------------------------
*/
status = c_getin (&block, "IN", &zero, taein, hosin, num_inimg, window, 
                  num_img_band, bands_per_img, tot_inbands, &img_limit, 
                  same_wnd, img_flag);
if (status != E_SUCC)
   c_errmsg ("Cannot access input file names", "lineplot-getin", &fatal_err);
if (same_wnd == 0)
   c_errmsg ("Image windows differ in size", "lineplot_window", &fatal_err);

/*
Determine the input image type.
If the data type exceeds EWORD, abort with an error message.
--------------------------------------------------------------
*/
for (i = 0; i < *num_inimg; i++)
   {
   status = c_getddr (hosin[i], &ddr);
   if (status != E_SUCC)
      c_errmsg ("DDR cannot be found", "lineplot-getddr", &fatal_err);
   }
*dtype = ddr.dtype;
if (*dtype > EWORD)
   c_abort (&block, "REAL and INTEGER*4 data types are not allowed", 
            "linerepr-dtype", prog_name, out_files, &num_outfiles);

/*
Parse the output image specification.
If the return status is not success then abort with a fatal error message.
--------------------------------------------------------------------------
*/
status = c_getout (&block, "OUT", taeout, hosout); 
if (status != E_SUCC)
   c_abort (&block, "Cannot access output file names", "linerepr-getout", 
            prog_name, out_files, &num_outfiles);
num_outfiles = 0;
strcpy (out_files[num_outfiles++], taeout[0]);

/*
Get starting bad line, and its increment.
-------------------------------------------
*/
/* nob = nib; */
partype = ELONG;
c_getnum (&block, "SL", &one, &partype, badlin); 
c_getnum (&block, "LINEINC", &one, &partype, lineinc); 

/* 
Validate the starting bad line.
-------------------------------
*/
if (*badlin > window[0][NL]) 
   {
   sprintf (msg, "Starting line cannot exceed the number of lines ");
   sprintf (&msg[strlen(msg)], "in the input image");
   c_abort (&block, msg, "linerepr-badlin", prog_name, out_files, 
            &num_outfiles);
   }

/* 
Get the LAS global paramters.
--------------------------------- 
*/
c_getnum (&block, "$MSGTIM", &one, &partype, &msgtim);
var = p_find(&block, "$MINMAX");
strcpy (mnmx, SVAL(*var, 0));
if (mnmx[0] == 'Y')
   min_max = TRUE;
else
   min_max = FALSE;

}
