/*******************************************************************************
NAME    		        CALC_PAGES    	    	            

PURPOSE    
  CALC_PAGES calculates the number of printer pages required for output,
  prompting the user for verification if that number is greater than five
  when list is being run interactively.  CALC_PAGES is called by LISIT.

FILE REFERENCES		        
  --
   
PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
S. Ross		Feb. 1984	Version 1.0
N. Deodhar/SAR	Sep. 1987	Version 3.1 - added loginit.for

K. Zanter	Jan. 1986       CIPS conversion to UNIX
K. Gacke        Sept 1986	LAS updates -- processing message utility.
				Modified calls to inlas, getin, and gettyp.  
				Removed calls to CM routines.
B. Davis	Jun. 1988	Conversion to LAS 5.0
T. Mittan       Feb. 1992       Changed MAXLINES_PER_PAGE so that printer
                                does not print only one line on a page

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   LIST must be run under TAE.

PROJECT   	       LAS		                

ALGORITHM(FLOW)
  CALC_PAGES sets the number of pixels per line, then calculates the number of
  pages needed to print the lines.  If the number of pages is more than five
  and the runtype is interactive, the user is prompted using TAE calls for 
  verification that she/he actually wants to print this large of print file,
  and waits for a response.
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "protos.h"

#define pgnum curent
#define MAXLINES_PER_PAGE 52
extern struct FDESC * fdesc[MAXIMG + 1];

void calc_pages
(
    char hosin[][CMLEN],		/* pointer to array of host file names       */
    long *nmrimg,			/* number of input images		     */
    long (*window)[4],		/* window specification	                     */
    long *dtype,			/* data type of images		             */
    long *nbands,	        	/* number of bands in each input image       */
    long *max_pplin,	        /* max number of pixels per line	     */
    long *pages_in_set,		/* number of pages in a set                  */
    long *pages_per_band,	    /* total number of pages for each band       */
    long *pgtot,		        /* total number of output pages              */
    long *lnpr,			/* flag for output to line printer           */
    char *runtype	        	/* current sesson type		  	     */
)
{
long acc = IREAD;		/* read access for opening input images      */
long dumbnd = 0;   	        /*zero for eopenr nbands-use nbands from getin*/
long i;				/* loop counter				     */
long nl[MAXIMG];	        /* number of lines		  */
long ns[MAXIMG];	        /* number of samples		  */
long nt;			/* number of ?			  */	
long one = 1;			/* value of one			  */
long sl[MAXIMG];	        /* starting line		  */
long ss[MAXIMG];	        /* starting sample		  */
long status;		        /* function return status code    */
int  term;	      	        /* how "resp" was terminated      */
long zero = 0;   	        /* value of zero                  */

char mesg[100];		        /* page total message 		  */
char prognm[5]; 		/* program name for error messages*/
char resp[5];		        /* user's response		  */

strcpy(prognm,"list");

/* set the number of output pixels per line */
for (i = 0; i < *nmrimg; i++)
   {
   sl[i] = window[i][0];
   ss[i] = window[i][1];
   nl[i] = window[i][2];
   ns[i] = window[i][3];

   dumbnd = 0;
   status = c_eopenr(&fdesc[i],hosin[i],&dumbnd,&sl[i],&ss[i],&nl[i],&ns[i],
		     &acc,&dtype[i],&zero,&one);
   if (status != E_SUCC) 
      {
      c_errmsg("Error opening input image file","list-open",&one);
      c_errmsg("Fatal error encountered","list-fatal",&zero);
      }
   switch (dtype[i])
      {
      case 1:
         max_pplin[i] = 20;
         break;
      case 2:
         max_pplin[i] = 17;
         break;
      case 3:
         max_pplin[i] = 9;
         break;
      case 4:
         max_pplin[i] = 9;
         break;
      }

/* calculate the number of pages needed */
   nt = ((ns[i] % (max_pplin[i])) == 0) ? ns[i] / (max_pplin[i]) :
						ns[i] / (max_pplin[i]) + 1;
   pages_in_set[i] = ((nl[i] % MAXLINES_PER_PAGE) == 0) ?
                     nl[i] / MAXLINES_PER_PAGE : nl[i] / MAXLINES_PER_PAGE + 1;
   pages_per_band[i] = (pages_in_set[i]) * nt;
   *pgtot = *pgtot + (pages_per_band[i]) * (nbands[i]);
   }

if (*lnpr == 1)
   {
   sprintf (mesg,"Number of printer pages required: %d",*pgtot);
   c_errmsg(mesg," ",&one);
   if (*pgtot > 5)
      {
      if ((*runtype == 'i') || (*runtype == 'I'))
         {
         t_write("Print the image? (Y/N) ",T_PROMPT);
         t_read(resp, &term);
         if ((resp[0] != 'y') && (resp[0] != 'Y'))
            {
            c_errmsg("Try specifying a window."," ",&one);
            c_errmsg("LIST quitting--No output created","list-quit",&zero);
	    exit(-1);
	    }
	 else
	    {
            sprintf (mesg,"%d pages will be printed",*pgtot);
            c_errmsg(mesg," ",&one);
	    }
         }
      }
   }
return;
}
