/*******************************************************************************
NAME    		        CLEANUP    	    	            

PURPOSE    
  CLEANUP calls c_clean after deciding what information to send.
  c_clean deletes the print file, prints a fatal error message,
  and aborts processing.

FILE REFERENCES		        
  If printer or file output is requested, LIST creates an intermediate
  disk file (list_<time stamp>.pr) to hold the output listing.  If    
  cleanup is called, the global delflg is checked within c_clean to
  decide whether or not to delete the file name sent to it by cleanup.
   
PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
S. Ross		Feb. 1984	Version 1.0
N. Deodhar/SAR	Sep. 1987	Version 3.1 - added loginit.for

K. Zanter	Jan. 1986       CIPS conversion to UNIX
K. Gacke        Sept 1986	LAS updates -- processing message utility.
				Modified calls to inlas, getin, and gettyp.  
				Removed calls to CM routines.
B. Davis	Jun. 1988	Conversion to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   LIST must be run under TAE.

PROJECT   	       LAS		                

ALGORITHM (FLOW)
  Cleanup calls c_clean, sending it the temporary filename used for print
or file, otherwise a blank is sent for filename.
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

void cleanup
(
    struct PARBLK *vblock,       /* pointer to TAE block	 	      */
    FILE *fp,	 	 /* file pointer for temp print file	      */
    char *name,	 /* temporary print file name		      */
    long *lnpr,	 /* flag for print output to line printer     */
    long *fyle	 /* flag for print output to disk file	      */
)
{
long    	  one = 1;	 /* value of one			      */
long		  status;	 /* return status			      */
long    	  zero = 0;	 /* value of zero			      */

char tmpstr[1][CMLEN];  /* Temp string */
char prognm[5];

strcpy(prognm,"list");

/* close the print and/or disk file */
if ((*lnpr == 1) || (*fyle == 1))
   {
   status = fclose(fp);
   if (status == EOF)
      {
      c_errmsg("Error closing temporary print file","list-close",&one);
      c_errmsg("Fatal error encountered","list-fatal",&zero);
      }
   strcpy(tmpstr[0],name);
   c_clean(vblock,prognm,tmpstr,&one);
   }
else
   {
   strcpy(tmpstr[0],"\0");
   c_clean(vblock,prognm,tmpstr,&zero);
   }

return;
}
