/*******************************************************************************
NAME    		        GETPAR    	    	            

PURPOSE    
  GETPAR gets the TAE global variables and user specified parameters

FILE REFERENCES		        
  --
   
PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
S. Ross		Feb. 1984	Version 1.0
N. Deodhar/SAR	Sep. 1987	Version 3.1 - added loginit.for

K. Zanter	Jan. 1986       CIPS conversion to UNIX
K. Gacke        Sept 1986	LAS updates -- processing message utility.
				Modified calls to inlas, getin, and gettyp.  
				Removed calls to CM routines.
B. Davis	Jun. 1988	Conversion to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   LIST must be run under TAE.

PROJECT   	       LAS		                

ALGORITHM (FLOW)
  get TAE globals and store in program parameters 
  call c_getin to retrieve parameters
  call c_gettyp to obtain data type 
  call c_setprt to obtain output option(s) 
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,       /* pointer to TAE block	 	      */
    char taein[][CMLEN],		 /* pointer to input TAE file descriptor      */
    char hosin[][CMLEN],   		 /* pointer to input host file descriptor     */
    long *nmrimg, 		 /* pointer to number of input images	      */
    long (*bands)[MAXBND+1],    	 /* band specification 		              */
    long *nbands,        	 /* number of bands for each input image      */
    long *totbnd,		 /* pointer to total number of bands          */
    long (*window)[4],	         /* window specification		      */
    long *term,			 /* flag for output to terminal		      */
    long *fyle,			 /* flag for output to file            	      */
    long *lnpr,			 /* flag for output to line printer           */
    char *filnm,	 	  	 /* output filename for FILE option           */
    char *outfil,        	 /* pointer to output host file name          */
    long *msgtim,		 /* pointer to elapsed time between messages  */
    char *runtype		 /* pointer to session run type flag          */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block    */

long    imglim;			 /* max number of images to be processed      */
long    qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified   */
long    same;		    	 /* flag indicating if images have same window*/
long	status;		   	 /* function return status code		      */
long    zero = 0;		 /* value of zero			      */

char tmpstr[1][CMLEN];   /* Temp string */
char prognm[5];  		 /* program name for error messages           */
/******************************************************************************
  Retrieve the TAE global variables from the parameter block and store them 
  in their respective program variables.
*******************************************************************************/
strcpy(prognm,"list");
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$runtype");
strcpy(runtype,SVAL(*val,0));

/****************************************************************************** 
   Build the string descriptor for TAE variable "IN".  Call getin to parse
   the input image specification.  Getin returns the TAE image name(s), host
   image name(s), bands to be processed, window specification, and number
   of images parsed.
*******************************************************************************/
imglim = MAXIMG;
status = c_getin(vblock,"in",&zero,taein,hosin,nmrimg,window,bands,nbands,
	       totbnd,&imglim,&same,qflag);
if (status != E_SUCC) 
{
   strcpy(tmpstr[0],"\0");
   c_clean(vblock,prognm,tmpstr,&zero);
}
strcpy(filnm,"list");
status = c_setprt(vblock,term,fyle,lnpr,filnm,outfil);
if (status != E_SUCC)
{
   strcpy(tmpstr[0],"\0");
   c_clean(vblock,prognm,tmpstr,&zero);
}
return;
}
