/*******************************************************************************
NAME    		        TERM_WORD    	    	            

PURPOSE    
  TERM_WORD processes a band of a byte image.  The output listing is written
  to the terminal screen.

FILE REFERENCES		        
  --
   
PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
S. Ross		Feb. 1984	Version 1.0
N. Deodhar/SAR	Sep. 1987	Version 3.1 - added loginit.for

K. Zanter	Jan. 1986       CIPS conversion to UNIX
K. Gacke        Sept 1986	LAS updates -- processing message utility.
				Modified calls to inlas, getin, and gettyp.  
				Removed calls to CM routines.
B. Davis	Jun. 1988	Conversion to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  LIST must be run under TAE.

PROJECT   	       LAS		                

ALGORITHM (FLOW)
  initialize window info
  allocate buffer space
  initialize local counters
  calculate pages needed per band
  write header to screen
  for all pages
     while lines this page < max per page, and total lines written < nl
	read a line and increment
	while samps this line < max per line, and total samps written < ns
	   write samps to buffer
	write buffer to screen
     fill rest of screen with blank lines and increment counters
     write a new header
  free buffer space
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"
extern struct FDESC * fdesc[MAXIMG + 1];

void term_word
(
    struct PARBLK *vblock,       /* pointer to TAE block	 	    */
    FILE *fp,			/* file pointer to print file		    */
    long (*window)[4],		/* window specification			    */
    long *curimg,			/* index to current image		    */
    long *bndnum,			/* current band number		  	    */
    long *nmrimg,			/* number of images			    */
    long *nbands,			/* number of image bands		    */
    long *lnpr,			/* pointer to flag for output to printer    */
    long *fyle,			/* pointer to flag for output to file       */
    char *filnm,	    /* pointer temporary print file name        */
    long *mxln			/* max lines for term screen from t_init    */
)
{
long	      curpage;		/* current page number			    */
register long curr_samp;        /* current sample number		    */
long	      first;		/* add 1 page to total, first not counted   */
				/* because of headers			    */
long          i;	        /* loop index 			            */
long          in_line;		/* input line counter			    */
long          in_samp_loc;	/* offset into input image line- by twelves */
long	      k;		/* loop index				    */
long	      last_samp;	/* position of last sample		    */
register long line_num;		/* line number in input image		    */
long          max_pplin = 9;	/* max number of pixels per output line     */
long          nl;		/* number of lines			    */
long          ns;		/* number of samples			    */
long          nt;		/* number screen widths needed to list samps*/
long	      one = 1;		/* value of one                             */
long	      out_lines;	/* number of lines written to output screen */
register long out_samps;	/* number of samples written to output line */
long 	      pages_in_set;	/* number of pages needed to list all lines */
long 	      pages_per_band;	/* total number of pages for each band      */
long	      rem = 0;		/* local var - remainder of mod division    */
long          sl;	     	/* starting line			    */
long          ss;	     	/* starting sample			    */
long          status;		/* function return status code		    */
long	      tmp;		/* temporary used for output of annot. line */
long	      zero = 0;		/* value of zero                            */
register short *bufptr;		/* pointer to input buffer		    */
register short *ptr;		/* pointer to sample to be processed        */
char          msg[150];	        /* string to be outputted to terminal       */
char          prognm[5];        /* program name for error messages          */
char	      temp_buf[80];	/* temporary buffer                         */

strcpy(prognm,"list");
sl = window [*curimg][0];
ss = window [*curimg][1];
nl = window [*curimg][2];
ns = window [*curimg][3];
last_samp = ss + ns - 1;

/* allocate buffer space for image lines */
bufptr = (short *) malloc(ns * 2);
if (bufptr == NULL)
   {
   c_errmsg("Unable to allocate enough buffer space","list-nospace",&one);
   cleanup(vblock,fp,filnm,lnpr,fyle);
   }
/* initialize local variables */ in_line = 1;
line_num = sl;
in_samp_loc = 0;
curpage = 1;
in_samp_loc = 0;
curpage = 1;

/* There are 4 headers, plus 2 footers by pterm, on the first page of each */
/* band.  There is 1 header for each subsequent page of the same band.     */
/* The following lines compute the number of pages needed for the current  */
/* band, taking into consideration the differing number of header lines    */
nt = ((ns % max_pplin) == 0) ? ns / max_pplin : (ns / max_pplin) + 1;
if (nl > (*mxln - 6))
   {
   pages_in_set = ((nl - (*mxln - 6)) % (*mxln - 3) == 0) ?
                  1 + (nl - (*mxln - 6)) / (*mxln - 3) :
                  2 + ((nl - (*mxln - 6)) / (*mxln - 3));
   rem =  ((nl - (*mxln - 6)) % (*mxln - 3));
   if ((rem > 0 ) && (rem < 4))
      pages_per_band = ((pages_in_set * nt) - nt) + 1;
   else
      pages_per_band = (pages_in_set * nt);
   first = 1;
   }
else
   {
   first = 0;
   pages_in_set = 1;
   pages_per_band = pages_in_set * nt;
   }

/* output first annotation line */
tmp = ss;
out_samps = 1;
sprintf (msg,"       ");
while ((out_samps <= (max_pplin)) && (tmp <= last_samp))
      {
      sprintf (temp_buf,"%5d   ",tmp);
      strcat(msg,temp_buf);
      tmp++;
      out_samps++;
      }
c_pterm(msg,&zero);

/* process each output page for this band */
out_lines = 6;
for (i = 0; i < pages_per_band; i++)
    {
    while ((out_lines < (*mxln)) && (in_line <= nl))
          {
          status = c_eread(&fdesc[*curimg],bndnum,&in_line,bufptr,&one);
	  if (status != ns)
	     {
 	     c_errmsg("Error reading input image file","list-read",&one);
   	     cleanup(vblock,fp,filnm,lnpr,fyle);
	     }

          ptr = bufptr + in_samp_loc;
          curr_samp = in_samp_loc + 1;
          out_samps = 1;
 
          sprintf (msg,"%5d ",line_num);
  	  while ((out_samps <= (max_pplin)) && (curr_samp <= ns))
	      {
	      sprintf (temp_buf,"%6d  ",*ptr);
	      strcat(msg,temp_buf);
              ptr++;
   	      curr_samp++;
              out_samps++;
              }
	  if ((out_lines == (*mxln - 1)) &&
	       ((*curimg + 1) == *nmrimg) &&
		 ((*bndnum) == (*nbands)) &&
    		   ((i + 1) == pages_per_band) &&
    		     (in_line <= nl))
	     {
	     c_pterm(msg,&zero);
	     c_pterm("EOF",&one);
	     }
	  else
	     c_pterm(msg,&zero);
          out_lines++;
          in_line++;
          line_num++;
	  }
  
    while (out_lines < (*mxln))		   /* Fill up the last page of a band */
          {				   /* or img to reset counter properly*/
	  if ((out_lines == (*mxln - 1)) &&
	       ((*curimg + 1) == *nmrimg) &&
		 ((*bndnum) == (*nbands)) &&
    		   ((i + 1) == pages_per_band))
	  {
	     c_pterm("EOF",&one);
	  }
	  else
	  {
             sprintf (msg," ");
	     c_pterm(msg,&zero);
	  }
	  out_lines = out_lines + 1;
	  }
  
    if ((in_line) > nl ) 
       {
       in_line = 1;
       line_num = sl;
       }

    curpage++;
    if (curpage >= (pages_in_set + first))
       {
       curpage = 1;
       in_samp_loc = in_samp_loc + (max_pplin);
       if ((rem > 0 ) && (rem < 4))
          first = 0;
       }

/* output header and annotation for next page */
    if ((i + 1) < pages_per_band)
       {
       tmp = in_samp_loc + ss;
       k = 1;
       sprintf (msg,"       ");
       while ((k <= (max_pplin)) && (tmp <= last_samp))
             {
    	     sprintf (temp_buf,"%5d   ",tmp);
	     strcat(msg,temp_buf);
  	     tmp++;
             k++;
             }
	c_pterm(msg,&zero);
        out_lines = 3;
        }
    }
free (bufptr);
return;
}
