/*******************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the LNFILL function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      Jan 1990       converted to LAS 5.0
K.Kimball	Aug. 1991      Updated to 5.0 standards
J.Fenno         Mar  1992      Made OUT a required parameter.  Modified so that
			       image combinations are not allowed for update
			       mode (ie. "img1 + img2").
			       PRF Z0743

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	       LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)

*******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"



void getpar
(
    struct PARBLK *vblock,   /* pointer to TAE block	 	              */
    char *fname,		/* input file name			      */
    char hosin[][CMLEN],  	/* host file names of input images	      */
    char hosout[][CMLEN],		/* host file name of output image	      */
    char *minmax,		/* min max flag buffer		              */
    char taein[][CMLEN],	/* TAE file names of input images	      */
    char taeout[][CMLEN],		/* TAE file name of output image	      */
    long (*bands)[MAXBND+1], /* band specification 		              */
    long *dtype, 	/* data type of input image		      */
    long *maxlfix,		/* maximum number of lines to fix             */
    long *msgtim,		/* elapsed time between display messages      */
    long *nmrimg, 		/* number of input images		      */
    long *nbands,		/* number of bands per input image            */
    long *totbnd,		/* total number of bands		      */
    long (*window)[4],	/* window specification		              */
    short *update	/* True when input image is updated           */
)


{
struct  VARIABLE  *val;	      	 /* Pointer to TAE variable within v-block    */

long     access;		 /* Type of access for opening file           */
long     element = 0;		 /* Parameter element number		      */
long     fatal = -1;		 /* Fatal error flag			      */
long     imglim;		 /* Max number of images to be processed      */
long     nonfatal = 1;		 /* Nonfatal error flag			      */
long     qflag[MAXIMG][2];  	 /* Flag indicating if subwindows specified   */
long     same;		    	 /* Flag indicating if images have same window*/
long     status;	   	 /* Function return status code		      */
long	 zero = 0;               /* Address with the value of zero            */


/* Retrieve the TAE global variables from the parameter block
   and store them in their respective program variables. 
-------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window size","lnfill-size",
            &nonfatal);
   c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
   }

/* Get the file name
--------------------*/
access = IREAD;
status = c_getnam(vblock,"INFILE", &zero, fname,"txt", &access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);


/* Determine if the user has requested update mode or not
---------------------------------------------------------*/
*update = FALSE;
val = p_find(vblock, "OUT");
if (val->v_count == 0)
   {
   *update = TRUE;

   /* Combination of input images is not allowed in update mode
   ------------------------------------------------------------*/
   if (*nmrimg > 1)
      {
      c_errmsg("Combination of images is not allowed in update mode",
	       "lnfill-comb", &nonfatal);
      c_errmsg("Fatal error encountered", "lnfill-fatal", &fatal);
      }

   /* Get the data type
   --------------------*/
   status = c_gettyp(hosin, nmrimg, dtype);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);

   c_errmsg("Input image will be updated"," ", &nonfatal);
   }
else if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);

val = p_find(vblock, "maxlfix");
*maxlfix = IVAL(*val, 0);

/* Retrieve the TAE global variables from the parameter block
   and store them in their respective program variables. 
-------------------------------------------------------------*/
val = p_find(vblock, "$msgtim");
*msgtim = IVAL(*val, 0);

val = p_find(vblock, "$minmax");
strcpy(minmax, SVAL(*val,0));
return;
}
