/*****************************************************************************
NAME:   		       INIT

PURPOSE:  Open input and output images, allocate the buffer, and initialize
          arrays used in processing.
     

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
1.0	   Kelly Kimball   8/91		EDC		Separated from proces 
							routine to create 
							smaller modules.
1.1	   Jim Fenno       3/92		EDC		Placed check of window
							against maxlfix in init
							so an error isn't issued
							for every line.
1.2	   Tom Mittan	  10/92		EDC		Do not allow compression
							because update mode is 
							used.
6.1	   D. Eichacker   05/94		EDC		Added opt[] to first
							and last eopen.
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   lNIT must be run as part of LNFILL.

PROJECT			LAS

ALGORITHM:

ALGORITHM REFERENCES		none
******************************************************************************/
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"


/* The following variables must be declared global for image I/O routines
-------------------------------------------------------------------------*/
extern struct FDESC * fdesc[];	 /* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	 /* group descriptor used for image I/O       */

extern long curcat;		/* current processing band		     */
extern long curent;		/* current processing line		     */
extern long offset1[];          /* offsets into "thebuf" for input bands     */
extern long offset2[];          /* offsets into "thebuf" for output bands    */

void initl
(
    struct PARBLK *vblock,		/* TAE parameter block		      */
    char *fname,			/* Input file name 		      */
    char hosin[][CMLEN],		/* Host names of input images	      */
    char hosout[][CMLEN],		/* Host name of output image	      */
    char *minmax,		/* Min max flag                       */
    double *minval,		/* Min pixel value array              */
    double *maxval,		/* Max pixel value array              */
    long (*bands)[MAXBND+1],	/* Bands to be processed for all 
					   images                             */
    long *dtype,			/* Data type of the input image       */
    long *msgtim,		/* Elapsed time between display 
					   messages                           */
    long *lines,			/* Lines to be fixed		      */
    long *maxlfix,               /* maximum number of lines to fix     */
    long *nbands,		/* Number of bands per input image    */
    long *nblks,			/* Number of blocks to be fixed	      */
    long *nbrln,                 /* Number of lines to be fixed in a 
					   block                              */
    long *nl,			/* Number of image lines              */
    long *nmrimg,		/* Number of input images             */
    long *ns,			/* Number of image samples            */
    long *odtype,		/* Data type of output image          */
    long *size,			/* Number of bytes per pixel 	      */
    long *sl,			/* Starting image line                */
    long *ss,			/* Starting image sample              */
    long *totbnd,		/* Total number of bands to be 
                                           processed                          */
    long (*window)[4],		/* Window specification of input 
                                           images                             */
    short *update		/* True when the input image is to 
					   be updated                         */
)

{


long	access;			/* Image I/O access type                     */
long	band;			/* Loop index counting the bands             */
long	bufsiz;			/* Size of image I/O buffer                  */
long    curlin;			/* Current line being processed		     */
long    fatal = -1;		/* Fatal error flag			     */
long    num_images;		/* Number of images in fdesc to close when
                                   stopping due to fatal error               */
long    i;                      /* Loop control                              */
long	image;			/* Loop index counting input images          */
long	msg_flag=ALLCAT;	/* Flag for porcessing message               */
long	nonfatal = 1;		/* Nonfatal error flag 			     */
long    one = 1;		/* Address with a value of one	             */
long	status;			/* Function return status code               */
long	zero = 0;		/* Value of zero                             */
static	long	opt[3] = {0,0,0};	/* option array for open call	      */


unsigned char *buffer;		/* Buffer of input images 	             */



/* Open input image(s)
----------------------*/
*nl = window[0][2];
*ns = window[0][3];
*sl = window[0][0];
*ss = window[0][1];



if (!*update)
   {
   access = IREAD;
   *nl = window[0][2];
   *ns = window[0][3];

   /* Set output data type to zero so the output image will get
      the data type of the first image
      ---------------------------------------------------------*/

   *odtype = 0;

   for (band=0,image=0; image < *nmrimg; image++)
      {
      *sl = window[image][0];
      *ss = window[image][1];

      status = c_eopens(&fdesc[image],hosin[image],bands[image],sl,ss,nl,ns,
                        &offset1[band],&access,odtype,&zero,opt);
     
      if (status != E_SUCC)
         c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
      band += nbands[image];
      }

   fdesc[*nmrimg] = 0;

   /* Open output image
   --------------------*/
   access = IWRITE;	
   opt[0] = zero;
   opt[1] = NO_COMPRES;
   opt[2] = 0;
   status = c_eopens(&fdesc[*nmrimg],hosout[0],totbnd,&one,&one,nl,ns,
                     offset2,&access,odtype,&zero,opt);

   if (status != E_SUCC)
      c_clean(vblock,"lnfill",hosout,&one);

   fdesc[*nmrimg+1] = 0;

   /* Get the size of the output data type
      -----------------------------*/

   switch(*odtype)
      {
      case EBYTE:  	*size = 1; break;
      case EWORD:	*size = 2; break;
      case ELONG:	*size = 4; break;
      case EREAL:	*size = 4; break;
      }


   /* Allocate I/O buffer
   ----------------------*/
   if ((buffer = (unsigned char *) c_ealloc(fdesc,&bufsiz)) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","lnfill-alloc",&nonfatal);
      c_eclose(&fdesc[*nmrimg]);
      c_clean(vblock,"lnfill",hosout,&one);
      }

   /* Group images
   ---------------*/
   if (c_egroup(fdesc,&gdesc,buffer,&bufsiz) != E_SUCC)
      {
      c_errmsg("Error allocating dynamic memory","lnfill-alloc",&nonfatal);
      c_eclose(&fdesc[*nmrimg]);
      c_clean(vblock,"lnfill",hosout,&one);
      }

   /* Setup for processing message utility
   ---------------------------------------*/
   c_inittm("LNFILL","Copied","line","bands"," ",
            nl,totbnd,msgtim,&msg_flag);
   curent = 1;
   curcat = 1;
    
   c_startm();


   /* copy the records to the output file
   --------------------------------------*/
   for (curlin = 0; curlin < *nl; curlin++)
      {
      if (c_estep(&gdesc) != E_SUCC)
         {
	 c_eclose(&fdesc[*nmrimg]);
	 c_clean(vblock,"lnfill",hosout,&one);
	 }
      for (band=0; band < *totbnd; band++)
	  {
	  c_pxcopy(buffer + (*(offset1+band) * (*size)),
		   buffer + (*(offset2+band) * (*size)),odtype,ns);
         
	  }
       curent++;
       }

   /* Final write
   --------------*/
   if (c_estep(&gdesc) != E_SUCC)
      {
      c_eclose(&fdesc[*nmrimg]);
      c_clean(vblock,"lnfill",hosout,&one);
      }

   /* Turn off display message
   ---------------------------*/
   c_stoptm();

   /* Free buffer
   --------------*/
   free(buffer);

   /* Close all image files in the group
   -------------------------------------*/
   c_egclse(&gdesc);

   /* Open the newly created output image.  Since it was just created
      we know it has just one image, so we do not need to loop 
      through a series of images.
      ---------------------------------------------------------------*/
 
   access = IUPDATE;	
   status = c_eopenr(&fdesc[0],hosout[0],totbnd,&one,&one,nl,ns,
                     &access,odtype,opt,&one);
   

   if (status != E_SUCC)
      {
      c_clean(vblock,"lnfill",hosout,&one);
      }
   
   }    /* if update equals no   */

else
   /* Update the image file
   ------------------------*/
   {
   access = IUPDATE;	
   *odtype = *dtype;

   /* Open all the input images using the data type found in getpar
      -------------------------------------------------------------*/

   for (image = 0; image < *nmrimg; image++)
      {
      status = c_eopenr(&fdesc[image],hosin[image],&nbands[image],
                        &one,&one,nl,ns,&access,odtype,opt,&one);

      if (status != E_SUCC)
         {
         num_images = image - 1;
         close_output(&num_images);
         c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
         }
      }

   /* Get the size of the output data type
      ------------------------------------*/

   switch(*odtype)
      {
      case EBYTE:  	*size = 1; break;
      case EWORD:	*size = 2; break;
      case ELONG:	*size = 4; break;
      case EREAL:	*size = 4; break;
      }

   }


/* Read the input file
----------------------*/
status = rdfil(fname, lines, nblks, nbrln);
if (status != E_SUCC)
   {
   close_output(nmrimg);
   c_clean(vblock,"lnfill",hosout,&one);
   }


/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(odtype,totbnd,minval,maxval);


/* Set the window parameters for the output image
-------------------------------------------------*/
*ss = 1;
*sl = 1;
*nl = window[0][2];
*ns = window[0][3];

/* Check each window and make sure it has fewer lines than maxlfix
------------------------------------------------------------------*/
for (i = 0; i < *nblks; i++)
   if (nbrln[i] > *maxlfix)
      c_errmsg("Warning, input values set to zero","lnfill-maxline",&nonfatal);

/* Initialize the processing message
------------------------------------*/
c_inittm("LNFILL","Processed","line","bands"," ",
         nl,totbnd,msgtim,&msg_flag);

curent = 1;
curcat = 1;
c_startm();

return;
}
