/*******************************************************************************

NAME:    LNFILL 

PURPOSE: To provide a means of fixing bad lines.

PROGRAM HISTORY:

PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      JAN  1984       Original Development
R.MCKINNEY      SEPT 1988       Conversion to LAS 5.0
K.Kimball       Aug. 1991       Updating to 5.0 standards
J.Fenno         Mar  1992       Changed OUT so that is does not default to NULL.
				The user must enter null to enable update mode.
				PRF Z0743
T.Mittan	Sept 1992	Modified so that a blank line is no longer 
				output with the completion message in update 
				mode. Also added extern to curent and curcat
				to eliminate warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   LNFILL must be run under TAE.

PROJECT   	         LAS		                

******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAX_BLOCKS 1000		/* Maximum number of blocks of lines that 
				   can be fixed				     */
					
/* The following variables must be declared global for image I/O routines
   ----------------------------------------------------------------------*/

struct FDESC * fdesc[MAXIMG+1];        /* Image file descriptors used for input 
			                              images 				     */
struct GDESC * gdesc;		     /* Group descriptor used for image I/O       */

extern long curcat;		/* Current processing band		     */
extern long curent;		/* Current processing line		     */
long offset1[MAXIMG*MAXBND+1];  /* Offsets into "thebuf" for input bands     */
long offset2[MAXBND+1];         /* Offsets into "thebuf" for output bands    */


int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		     */

char    fname[CMLEN];		/* Input file name for list of blocks to be
				   fixed				     */
char	hosin[MAXIMG][CMLEN];	/* Input image host name		     */
char    hosout[1][CMLEN];          /* Output image host name		     */
char    minmax[4];	        /* Minmax flag buffer 			     */
char	taein[MAXIMG][CMLEN];	/* Input image LAS name			     */
char    taeout[1][CMLEN];          /* Output image LAS name		     */ 
double  maxval[MAXBND];  	/* Max pixel value array                     */
double  minval[MAXBND];  	/* Min pixel value array                     */


static  long bands[MAXIMG][MAXBND + 1];
                                /* Band specifications		             */
long	dtype;			/* Data type of the input images	     */
long 	lines[MAX_BLOCKS];      /* Lines being fixed in the image	     */
long 	maxlfix;		/* Maximum number of lines to fix            */
long 	msgtim;			/* Elpased time between display messages     */
long 	nbands[MAXIMG];		/* Number of bands per input image           */
long 	nblks;			/* Number of blocks to fix		     */
long 	nbrln[MAX_BLOCKS];      /* Number of lines to be fixed in a block    */
long 	nl;                     /* Number of image lines                     */
long 	nmrimg;			/* Number of input images for image          */
long 	ns;                     /* Number of image samples                   */
long    odtype;			/* Data type of output image		     */
long 	size;                   /* Number of bytes per pixel                 */
long 	sl;                     /* Starting image line                       */
long 	ss;                     /* Starting image sample                     */
long  	std_out;		/* Logical unit # for standard output        */
long    totbnd;			/* Total number of bands in input images     */
long 	window[MAXIMG][4];	/* Window specification for input images     */

short   update;			/* True when the input image is updated      */


c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/* Get user-specified parameters and TAE globals
------------------------------------------------*/
getpar(&vblock, fname, hosin, hosout, minmax, taein, taeout,
       bands, &dtype, &maxlfix, &msgtim, &nmrimg, nbands,
       &totbnd, window, &update);


/* Open i/o images and initalize arrays
---------------------------------------*/
initl(&vblock, fname, hosin, hosout, minmax, minval, maxval, 
     bands, &dtype, &msgtim, lines, &maxlfix, nbands, &nblks, nbrln,
     &nl, &nmrimg, &ns, &odtype, &size, &sl, &ss, &totbnd, 
     window, &update);


/* Perform the rectangular correction
-------------------------------------*/
proces(&vblock, hosout, bands, lines, nbands, &maxlfix, 
       &nblks, nbrln, &nl, &nmrimg, &ns, &odtype, &size, 
       &sl, &ss, &totbnd, window, &update);

/* Update the ddr and close the function
----------------------------------------*/
terminate(&vblock, hosin, hosout, minmax, taeout, maxval, minval,
	  bands, nbands, &nmrimg, &totbnd, window, &update);

return EXIT_SUCCESS;

}
