/*****************************************************************************
NAME:   		       PROCES

PURPOSE    To fix bad rectangular areas within an image. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   R. McKinney	   1/84		  EDC		Original development
  5.0	   R. McKinney     9/88		  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.1     K. Kimball       8/91           EDC           Modularizing LNFILL and
							update to 5.0 standards 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PROCES must be run under TAE.

PROJECT			LAS

ALGORITHM:


ALGORITHM REFERENCES		none
******************************************************************************/
#include "las.h"
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines
-------------------------------------------------------------------------*/
extern struct FDESC * fdesc[];	 /* Image file descriptors used for image I/O */
extern struct GDESC * gdesc;	 /* Group descriptor used for image I/O       */

extern long curcat;		/* Current processing band		     */
extern long curent;		/* Current processing line		     */
extern long offset1[];          /* Offsets into "thebuf" for input bands     */
extern long offset2[];          /* Offsets into "thebuf" for output bands    */

void proces
(
    struct PARBLK *vblock,	 /* TAE parameter block		      	      */
    char hosout[][CMLEN],	 /* Host name of output image		      */
    long (*bands)[MAXBND+1],
    long *lines,		 /* Lines to be fixed                         */
    long *nbands,	 /* Number of bands per input image           */
    long *maxlfix,	 /* Minimum number of lines to fix            */
    long *nblks,		 /* Blocks to be fixed		              */
    long *nbrln,          /* Number of lines to be fixed in a block    */
    long *nl,             /* Number of image lines                     */
    long *nmrimg,	 /* Number of output images		      */
    long *ns,             /* Number of image samples                   */
    long *odtype,	 /* Output data type			      */
    long *size,           /* Number of bytes per pixel                 */
    long *sl,             /* Starting image line                       */
    long *ss,             /* Starting image sample                     */
    long *totbnd,         /* Total number of bands in the output image */
    long (*window)[4],	 /* Window specification of input images      */
    short *update	 /* True when the input image is to be updated*/
)

{


long	band;			/* Loop index counting the bands             */
long    fatal = -1;		/* Fatal error flag                          */
long    fixnofix;		/* Flag to fix lines or not                  */
long    i;			/* Just a counter			     */
long 	image;			/* Iterates through all output images        */
long    inwind[4];              /* Input window areas to fix                 */
long    line;			/* Line to be read			     */
long    line1;			/* Internal variable			     */
long    nbrwind;		/* Number of windows to fix 		     */
long    num_bands;              /* Number of bands in the input image        */
long    num_images;		/* Number of images for output               */
long	nonfatal = 1;		/* Nonfatal error flag			     */
long    one = 1;		/* Address containing value of one           */
long    sample;			/* Sample indicator			     */
long    savewind1;		/* Save the first element of the window      */
long    savewind2;		/* Save the second element of the window     */
long	status;			/* Function return status code               */

float   bottom;		        /* Bottom sample indicator		     */
float   lineinc;		/* Line increment indicator		     */
float   top;		        /* Top sample indicator			     */

unsigned char *buffer1;    	/* Image input buffer                        */
unsigned char *buffer2;         /* Image input buffer                        */
unsigned char *thebuf;      	/* Image input buffer                        */

/* Determine the number of images.  If a new output image is created,
   then it is one.  Otherwise, it is the number of input images.
--------------------------------------------------------------------*/
num_images = 1;
if (*update)
   num_images = *nmrimg;


/* Allocate memory
------------------*/
if ((thebuf = (unsigned char *) malloc(*ns * *size)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","lnfill-alloc",&nonfatal);
   close_output(&num_images);
   if (!*update)
      c_clean(vblock,"lnfill",hosout,&one);
   else
      c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
   }
if ((buffer1 = (unsigned char *) malloc(*ns * *size)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","lnfill-alloc",&nonfatal);
   close_output(&num_images);
   if (!*update)
      c_clean(vblock,"lnfill",hosout,&one);
   else
      c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
   }
if ((buffer2 = (unsigned char *) malloc(*ns * *size)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","lnfill-alloc",&nonfatal);
   close_output(&num_images);
   if (!*update)
      c_clean(vblock,"lnfill",hosout,&one);
   else
      c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
   }


/* Set the number of images to default to be all bands.  This will
   be modified in the loop if the input images are being updated.
------------------------------------------------------------------*/
num_bands = *totbnd;
band = 0;
for (image = 0; image < num_images; image++)
   {
   nbrwind = *nblks;

   /* Set the number of bands for this image if updating the input images
   ----------------------------------------------------------------------*/
   if (*update)
      num_bands = nbands[image];

   for (curcat = 0; curcat < num_bands; curcat++)
      {
      /* If in update mode then set band based on the bands identified at
	 input, otherwise increment band through the total number of bands. 
      ---------------------------------------------------------------------*/
      if (*update)
	 band = bands[image][curcat];
      else
	 band++;

      for (i = 0; i < nbrwind; i++)
	 {
	 inwind[0] = lines[i];
	 inwind[1] = 1;
	 inwind[2] = nbrln[i];
	 inwind[3] = *ns;
	 fixnofix = 0;

	 if (nbrln[i] > *maxlfix)
	    fixnofix = 1;

	 savewind1 = inwind[0] - window[image][0] + 1;
	 savewind2 = inwind[1] - window[image][1] + 1;

	 if (savewind1 == *sl && inwind[2] == *nl &&
	     savewind2 == *ss && inwind[3] == *ns)
	    {
	    c_errmsg("Error specifying window","lnfill-window", &nonfatal);
	    close_output(&num_images);
	    if (!*update)
	       c_clean(vblock,"lnfill",hosout,&one);
	    else
	       c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
	    }

	 /* Read the line prior to the window
	 ------------------------------------*/
	 if (savewind1 != *sl)
	    {
	    line = savewind1 - 1;
	    status = c_eread(&fdesc[image], &band, &line, buffer1, &one);
	    if (status == E_FAIL)
	       {
	       close_output(&image);
	       if (!*update)
		  c_clean(vblock,"lnfill",hosout,&one);
	       else
		  c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
	       }

	    }
	 else
	    {
	    line = savewind1 + inwind[2];
	    status = c_eread(&fdesc[image], &band, &line, buffer1, &one);
	    if (status == E_FAIL)
	       {
	       close_output(&image);
	       if (!*update)
		  c_clean(vblock,"lnfill",hosout,&one);
	       else
		  c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
	       }
	    }

	 /* read the line following the window
	 -------------------------------------*/
	 if ((savewind1 + inwind[2]) != (*nl+1))
	    {
	    line = savewind1 + inwind[2];
	    status = c_eread(&fdesc[image], &band, &line, buffer2, &one);
	    if (status == E_FAIL)
	       {
	       close_output(&image);
	       if (!*update)
		  c_clean(vblock,"lnfill",hosout,&one);
	       else
		  c_errmsg("Fatal error encountered","lnfill-fatal",&fatal);
	       }
	    }
	 else
	    {
	    line = savewind1 - 1;
	    status = c_eread(&fdesc[image], &band, &line, buffer2, &one);
	    if (status == E_FAIL)
	       {
	       close_output(&image);
	       if (!*update)
		  c_clean(vblock,"lnfill",hosout,&one);
	       else
		  c_errmsg("Fatal error encountered", "lnfill-fatal", &fatal);
	       }
	    }

	 line1 = 0;

	 for (curent = savewind1; curent < savewind1 + inwind[2]; curent++)
	    {
	    line1++; 
	    status = c_eread(&fdesc[image], &band, &curent, thebuf, &one);
	    if (status == E_FAIL)
	       {
	       close_output(&image);
	       if (!*update)
		  c_clean(vblock,"lnfill",hosout,&one);
	       else
		  c_errmsg("Fatal error encountered", "lnfill-fatal", &fatal);
	       }


	    /* For the following switch, the code for the loops is repeated
	       to improve performance of the loops by executing the case and 
	       if conditions outside of the loop                            
	    ----------------------------------------------------------------*/
	    switch (*odtype)
	       {
	       case EBYTE:
		  if (fixnofix == 0)
		     {
		     /* the line should be fixed 
		     ---------------------------*/
		     for (sample=(inwind[1]-1); 
		          sample < (inwind[1] + inwind[3] - 1);
			  sample++)
			{
			top = *(buffer1 + sample);
			bottom = *(buffer2 + sample);
			lineinc = (top - bottom) / ((float) inwind[2] + 1);
			*(thebuf + sample) = ((top) - (lineinc * (float) line1));
			}
		     }
		  else
		     {
		     /* The line should be filled with zeros
		     ---------------------------------------*/
		     for (sample=(inwind[1]-1); 
		          sample < (inwind[1] + inwind[3] - 1);
			  sample++)
			*(thebuf + sample) = 0;
		     }
		  break;

	       case EWORD:
		  if (fixnofix == 0)
		     {
		     /* the line should be fixed 
		     ---------------------------*/
		     for (sample=(inwind[1]-1);
		          sample < (inwind[1] + inwind[3] - 1);
			  sample++)
			{
			top = *(short *)(buffer1 + (sample * (*size)));
			bottom = *(short *)(buffer2 + (sample * (*size)));
			lineinc = (top - bottom) / ((float) inwind[2] + 1);
			*(short *)(thebuf + (sample * (*size))) = 
			   ((top) - (lineinc * (float) line1));
			}
		     }
		  else
		     {
		     /* The line should be filled with zeros
		     ---------------------------------------*/
		     for (sample=(inwind[1]-1);
		          sample < (inwind[1] + inwind[3] - 1);
		          sample++)
			*(short *)(thebuf + (sample * (*size))) = 0;
		     }
		  break;

	       case ELONG:
		  if (fixnofix == 0)
		     {
		     /* the line should be fixed 
		     ---------------------------*/
		     for (sample=(inwind[1]-1); 
		          sample < (inwind[1] + inwind[3] - 1);
			  sample++)
			{
			top = (float) (*(long *)(buffer1 + (sample * (*size))));
			bottom = (float) (*(long *)(buffer2 + (sample * (*size))));
			lineinc = (top - bottom) / ((float) inwind[2] + 1);
			*(long *)(thebuf + (sample * (*size))) = 
			   (long) ((top) - (lineinc * (float) line1));
			}
		     }
		  else
		     {
		     /* The line should be filled with zeros
		     ---------------------------------------*/
		     for (sample=(inwind[1]-1); 
		          sample < (inwind[1] + inwind[3] - 1);
		          sample++)
			*(long *)(thebuf + (sample * (*size))) = 0;
		     }
		  break;

	       case EREAL:
		  if (fixnofix == 0)
		     {
		     /* the line should be fixed 
		     ---------------------------*/
		     for (sample=(inwind[1]-1);
		          sample < (inwind[1] + inwind[3] - 1);
			  sample++)
			{
			top = *(float *)(buffer1 + (sample * (*size)));
			bottom = *(float *)(buffer2 + (sample * (*size)));
			lineinc = (top - bottom) / ((float) inwind[2] + 1);
			*(float *)(thebuf + (sample * (*size))) = 
			   ((top) - (lineinc *((float)line1)));
			}
		     }
		  else
		     {
		     /* The line should be filled with zeros
		     ---------------------------------------*/
		     for (sample=(inwind[1]-1);
		          sample < (inwind[1] + inwind[3] - 1);
			  sample++)
			*(float *)(thebuf + (sample * (*size))) = 0;
		     }
		  break;

	       }   /* end case */

	    /* write out the lines with fixed windows
	    -----------------------------------------*/
	    status = c_ewrite(&fdesc[image], &band, &curent, thebuf, &one);
	    if (status == E_FAIL)
	       {
	       c_errmsg("Error writing output image","lnfill-write",&nonfatal);
	       close_output(&image);
	       if (!*update)
		  c_clean(vblock,"lnfill",hosout,&one);
	       else
		  c_errmsg("Fatal error encountered", "lnfill-fatal", &fatal);
	       }
	    } /* curent loop */
	 } /* nbrwind loop */
      } /* curcat loop */
   } /* image loop */


/* Free buffer
--------------*/
free(thebuf);
free(buffer1);
free(buffer2);

return;
}
