/*****************************************************************************
NAME:   		      RDFIL 

PURPOSE    To read in the information describing which blocks to fix and their
           dimensions from the associated file.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   R. McKinney	   1/84		  EDC		Original development
  5.0	   R. McKinney     9/88		  EDC		Updated to 5.0	
  5.1      K. Kimball      8/91           EDC           Added a prolog.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   RDFIL must be run under TAE as part of LNFILL.

PROJECT			LAS

ALGORITHM:


ALGORITHM REFERENCES		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "protos.h"

#define MAX_BLOCKS 1000	/* Maximum number of blocks of lines that will be 
			   fixed					     */

int rdfil
(
    char *fname,	/* File containing specifications for blocks to be
			   fixed					     */
    long *lines,        /* Contains list of line numbers that start each of
			   the blocks of lines to be fixed		     */
    long *nblks,    	/* Number of blocks of lines to be fixed	     */
    long *nbrln	/* Contains list of number of lines in each block 
			   to be fixed					     */
)
{

FILE *fd;		/* Auxiliary file containing the lines to be fixed   */

long i;                 /* Loop index					     */
long nonfatal = 1;      /* Indicates the error is nonfatal		     */
long tlines;            /* Total number of lines to be fixed                 */

/* Open the file
   -------------*/

if ((fd = fopen(fname,"r")) == NULL)
   {
      c_errmsg("Error opening line file",
		"lnfill-open",&nonfatal);
      return (E_FAIL);
   }

/* Get the first line indicating the number of blocks and total number of lines
   ---------------------------------------------------------------------------*/
fscanf(fd,"%d %d",nblks,&tlines);

/* Check for too many blocks
   -------------------------*/

if (*nblks >= MAX_BLOCKS)
   {
   c_errmsg("Too many line correction values", "lnfill-line",&nonfatal);
   return (E_FAIL);
   }

/* Read in the specs on the blocks of lines to be fixed
   ---------------------------------------------------- */

for (i = 0; i < *nblks; i++)
   {
   if (fscanf(fd,"%d %d",&lines[i],&nbrln[i]) != 2)
      {
         c_errmsg("Lines must have a corresponding number of bad lines",
		"lnfill-nbrln",&nonfatal);
         return (E_FAIL);
      }
   }

/* Close the file and return
   -------------------------*/
fclose(fd);
return (E_SUCC);
}
