#include "logical.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char hosin[][CMLEN],      /* input images host names            */
    char hosout[],            /* output image host name             */
    long num_images,          /* number of input images             */
    long num_image_bands[],   /* number of bands in each image      */
    long image_band_num[][MAXBND + 1],
    long wind[][4],           /* input image windows                */
    long *nlout,
    long *nsout,
    double true_val,
    double false_val
)

/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up input and output file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          1/14/85    J. Lavergne
   2          8/ 8/85    J. Davis    
   3          8/ 1/89    K. Narayanan    Conversion to Version 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   if the value of either true_val or false_val is fractional or greater than
      integer*4 can hold then set the output data type to real*4.
   else
      if they are less than -32678 or greater than 32767 then
         set the output data type to integer*4
      else if they are less than 0 or greater than 255 then
         set the output data type to integer*2
      else set the output data type to byte
   open input and output images
   update output image ddr

**************************************************************************
*/
{
double	line_inc = 1.0,
	samp_inc = 1.0;

static  long    fatal_err = -1;
static  long    zero = 0;
static  long    one = 1;


long	access,                      /* file access type                   */
	bufsiz,                      /* image i/o buffer size              */
	comb_flag = COMB,
	conv_type[2],
	i, 
	otype,
	stat,                        /* return status                      */
	type;                        /* file type                          */


/*
find the output data type
*/
if (((float) (int) true_val != true_val)  || 
    ((float) (int) false_val != false_val))    
   otype = EREAL;

else if ((min (true_val, false_val) < -32768)  || 
         (max (true_val, false_val) > 32767)) 
   otype = ELONG;

else if ((min (true_val, false_val) < 0) || 
         (max (true_val, false_val) > 255)) 
   otype = EWORD;

else otype = EBYTE;

/*
find the smallest image size and use it for input and out images
*/
for (i = 0; i < num_images; i++)
   {
   if (i == 0)
      {
      *nlout = wind[0][NL];
      *nsout = wind[0][NS];
      }
   else
      {
      *nlout = min (*nlout, wind[i][NL]);
      *nsout = min (*nsout, wind[i][NS]);
      }
   }

/*
open input and output image files
---------------------------------
*/
access = IREAD;
type = EREAL;
for (i = 0; i < num_images; i++)
   {
   stat = c_eopens (&fdesc[i], hosin[i], image_band_num[i], 
                    &wind[i][SL], &wind[i][SS], nlout, nsout, 
                    &ipoint[i], &access, &type, &zero, &zero);
   if (stat != E_SUCC )
      c_errmsg ("Error opening input image", "logical-inopen", &fatal_err);
      
   }  

access = IWRITE;
conv_type[0] = otype;
conv_type[1] = 0;
stat = c_eopens(&fdesc[i], hosout, &one, &one, &one, nlout, nsout, 
                &opoint, &access, &type, &zero, conv_type);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "logical-outopen", &fatal_err);

fdesc[i+1] = 0;

/*
update output image DDR based on input images
---------------------------------------------
*/
c_upddr (&block, hosin, hosout, &i, wind, image_band_num, num_image_bands,
         &line_inc, &samp_inc, &one, &comb_flag);

/*
set up group containing input and output bands
----------------------------------------------
*/
if ((iobuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "logical-alloc", 
            "LOGICAL", hosout, &one);

c_egroup (fdesc, &gdesc, iobuf, &bufsiz);
}
	
