#include "logical.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char hos_in[][CMLEN],/* input images host names          	*/
    char las_out[][CMLEN],      /* output image las name            	*/
    char hos_out[][CMLEN],      /* output image host name           	*/
    long *num_images,    /* number of images to be compared	*/
    long num_image_bands[], /* no. bands in each image		*/
    long image_band_num[][MAXBND + 1],
    long wind[][4],      /* input image windows              	*/
    long *num_oper, 	/* no. operations			*/
    long action[], 	/* logical operators			*/
    long order[],	/* order of connectors			*/
    long logics[], 	/* logical connectors			*/
    long images[],	/* image sequence			*/
    float *true_val, 	/* output value when true 		*/
    float *false_val, 	/* output value when false		*/
    float values[]	/* comparison values			*/
)
/*
**********************************************************************

NAME: GETPAR

PURPOSE: Retrieves parameter values 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         1/14/85    J. Lavergne
    2         7/28/89    K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
struct VARIABLE	 *var;              /* pointer to a variable in TAE block */

static  long    one = 1;
static  long    fatal_err = -1;

long	count,
	flag[1][2],                    /* flag for subwindows                */
	ftype, i, icount, itype, j, 
	imglimit = 1,               /* maximum number of images           */
	len,
	nbands,
	num_inimages,
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

char	conn[NUM_IMAGES][4],	    /* logical connectors		  */
	las_in[NUM_IMAGES][CMLEN],  /* input images las names             */
	minmax[4],                  /* minmax option                      */
	oper[NUM_IMAGES][3];	    /* logical operators		  */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input images parameters - must be single band
--------------------------------------------------------------
*/
var = p_find (&block, "IN");
*num_images = (*var).v_count;

for (j = 0; j < *num_images; j++)
   {
   stat = c_getin (&block, "IN", &j, las_in[j], hos_in[j], 
                   &num_inimages, wind[j], image_band_num[j], 
                   num_image_bands, &nbands, &imglimit, &same_window, flag);
   if (stat != E_SUCC) 
       c_errmsg ("Error accessing input image", "logical-in", 
              &fatal_err);

   if (nbands != 1) 
      c_errmsg ("Only one band images are allowed", "logical-nbands", 
                &fatal_err);
   }

/*
get output image
----------------
*/
stat = c_getout (&block, "OUT", las_out, hos_out);
if (stat != E_SUCC)
   c_errmsg ("Error getting output image name", "logical-out", 
             &fatal_err);

/*
get and validate other parameters
---------------------------------
*/
count = 10;
ftype = EREAL;
itype = ELONG;

*num_oper = c_getnum (&block, "COMPVAL", &count, &ftype, values);

if ((icount = c_getnum (&block, "IMGSEQ", &count, &itype, images)) == 0)
   for (i = 0; i < *num_oper; i++) images[i] = i + 1;
else
   if (icount != *num_oper)
      c_errmsg ("Number of values in COMPVAL and IMGSEQ must match", 
                "logical-imgseq", &fatal_err);

for (j = 0; j < *num_oper; j++)
   if (images[j] > *num_images) 
      c_errmsg ("IMGSEQ value exceeds the number of input images", 
                "logical-seqval", &fatal_err);

c_getnum (&block, "TRUEVAL", &one, &ftype, true_val);
c_getnum (&block, "FALSEVAL", &one, &ftype, false_val);

if (*true_val == *false_val) 
   c_errmsg ("TRUEVAL and FALSEVAL must have different values", 
             "logical-outval", &fatal_err);

len = 3;
if ((icount = c_getstr (&block, "LOGOPER", &count, &len, oper[0])) == 0)
   for (i = 0; i < *num_oper; i++) strcpy (oper[i], "EQ");
else if (icount != *num_oper)
   c_errmsg ("Number of values in COMPVAL and LOGOPER must match", 
             "logical-operator", &fatal_err);

len = 4;
if ((icount = c_getstr (&block, "LOGCON", &count, &len, conn[0])) == 0)
   for (i = 0; i < *num_oper; i++) strcpy (conn[i], "AND");
else if (icount != *num_oper - 1)
   c_errmsg ("Number of values in LOGCON must be one less than that in COMPVAL",
             "logical-connect", &fatal_err);

if ((icount = c_getnum (&block, "CONORDER", &count, &itype, &order[1])) == 0)
   for (i = 1; i <= *num_oper - 1; i++) order[i] = i;
else if (icount != *num_oper - 1)
   c_errmsg 
      ("Number of values in CONORDER must be one less than that in COMPVAL", 
       "logical-order", &fatal_err);


/*
set the character parameter values into integer arrays
------------------------------------------------------
*/
for (j = 0; j < *num_oper; j++)
   {
   if (strcmp (oper[j], "EQ") == 0) action[j] = 0;
   else if (strcmp (oper[j], "NE") == 0) action[j] = 1;
   else if (strcmp (oper[j], "LT") == 0) action[j] = 2;
   else if (strcmp (oper[j], "GT") == 0) action[j] = 3;
   else if (strcmp (oper[j], "LE") == 0) action[j] = 4;
   else if (strcmp (oper[j], "GE") == 0) action[j] = 5;

   if (strcmp (conn[j], "OR") == 0) logics[j] = 1;
   else logics[j] = 0;
   }

/*
Get globals
-----------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

}
