/***********************************************************************

NAME: LOGICAL

PURPOSE: Performs logical comparisons between input images and constant
         values to produce an output image of true or false values.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         1/14/85    J. Lavergne
    2         4/13/88    K. A. Narayanan    
    3         7/28/89    K. A. Narayanan    conversion to 5.0
    4	      9/23/92	 T. Mittan	    Modified include file to 
					    eliminate warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get and validate parameters 
   Call FILSPEC to set up file specifications
   Call GETRPN to create reverse polish notation
   Call PROCESS to process input images and create output image
   Issue completion message

***********************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define NUM_IMAGES 10

struct PARBLK block;                   /* TAE parameter block     */

float    *iobuf;                        /* image i/o buffer        */

struct FDESC *    fdesc[NUM_IMAGES+2];           /* file descriptors        */
struct GDESC *    gdesc;                         /* group descriptor        */
long     ipoint[NUM_IMAGES],            /* offsets into image i/o  */
                opoint;                        /* for input and output bands */
long     min_max;                       /* minmax option flag      */
long     msgtim;                        /* message interval        */

extern long    curcat;                        /* message utility         */

static  long    one = 1;

int main
(void) 
{
float	false_val, 		/* output value when false		*/
	true_val, 		/* output value when true		*/
	compval[NUM_IMAGES];	/* comparison values			*/

long	log_oper[NUM_IMAGES], 	/* logical operators			*/
	image_band_num[NUM_IMAGES][MAXBND + 1], 
				/* bands in each image in IN element	*/
	images[NUM_IMAGES],	/* sequence of image usage		*/
	index = -1, 		/* index into the rpn expression	*/
	log_conn[NUM_IMAGES], 	/* array of connectors			*/
	nlout,			/* no. lines in output image		*/
	nsout,			/* no. samples per line in output image	*/
	num_image_bands[NUM_IMAGES], /* no. bands in each image		*/
	num_images,         	/* number of images to be compared	*/
	num_oper, 		/* number of operations			*/
	order[NUM_IMAGES+1],	/* order of connectors			*/
	rpn[3*NUM_IMAGES],	/* reverse polish notation		*/
	wind[NUM_IMAGES][4];    /* input image windows              	*/

char	hos_in[NUM_IMAGES][CMLEN], /* input images host names          	*/
	hos_out[1][CMLEN],          /* output image host name           	*/
	las_out[1][CMLEN];          /* output image las name            	*/

/*
get and validate parameters
---------------------------
*/
getpar (hos_in, las_out, hos_out, &num_images, num_image_bands, 
        image_band_num, wind, &num_oper, log_oper, order, log_conn, 
        images, &true_val, &false_val, compval);

/*
open image files
----------------
*/
filspec (hos_in, hos_out[0], num_images, num_image_bands, 
         image_band_num, wind, &nlout, &nsout, true_val, false_val);

/*
set up reverse polish notation
------------------------------
*/
getrpn (num_oper, order, rpn, log_conn, &index);

/*
process images
--------------
*/
process (hos_in, hos_out[0], &num_images, &nlout, nsout, log_oper, rpn, 
         num_oper, images, index, true_val, false_val, compval);

/*
write a standard completion message
-----------------------------------
*/
c_complt ("LOGICAL", las_out, &one); 

return EXIT_SUCCESS;
}

