#include "logical.h"
#include "util.h"
#include "protos.h"

void process
(
    char hos_in[][CMLEN],/* input image host names		*/
    char hos_out[],	/* output image host name		*/
    long *num_images,	/* no. images to be compared		*/
    long *nlout, 	/* no. lines in output image		*/
    long nsout, 		/* no. samples per line in output image	*/
    long action[], 	/* logical operators			*/
    long rpn[], 		/* rpn array				*/
    long num_oper, 	/* no. operations			*/
    long images[], 	/* image sequence			*/
    long index,		/* index into rpn array			*/
    double true_val, 	/* output value when true 		*/
    double false_val, 	/* output value when false		*/
    float values[]	/* comparison values			*/
)
/*
**********************************************************************

NAME: PROCESS

PURPOSE:  Process the image files. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         1/14/85    J. Lavergne
    2         7/28/89    K. A. Narayanan    conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

   for each pixel
      if there is only one comparison, no rpn expression has been built -
         evaluate the comparison and continue to the next pixel
      else
         for each input image 
            call EVAL1 to put the result of each comparison in truth table

         for each element in the rpn expression
            if this element is an operand
               push this value onto the stack
            else (this element is an operator)
               pop the top two stack elements
               call EVAL2 to determine the result of these operands and
               the operator
               push the result onto the stack

         if the top of the stack is TRUE then
            set the output pixel to TRUEVAL
         else
            set the output pixel to FALSEVAL

**********************************************************************
*/
{
double	maxval, minval;

static  long    zero = 0;
static  long    one = 1;
long	false_flag = FALSE;
long	j, k, l, m; 
long	msg_flag = GEN2;
long	stkcnt; 
long	stack[NUM_IMAGES], truth[NUM_IMAGES];
long	true_flag = FALSE;
/*
initialize the processing message utility and min/max
-----------------------------------------------------
*/

c_inittm ("LOGICAL", "Processed", "line", "", "", nlout, &zero, 
          &msgtim, &msg_flag);
curent = 1;
c_startm ();

/*
for each pixel of each line of each image,
evaluate the input pixel : constant expression and store the
result in truth, then use this truth array as input in evaluating
the rpn expression built earlier.
-----------------------------------------------------------------
*/

for (j = 0; j < *nlout; j++, curent++)
   {
   c_estep (&gdesc);
   for (k = 0; k < nsout; k++)
      {
/*
      if the input expression is a simple =,/=,<,>,<=, or >= then no rpn
      expression is needed. the result of the expression can be
      determined directly from subroutine eval1.
      ------------------------------------------------------------------
*/
      if (num_oper == 1) 
         stack[0] = eval1 (iobuf[ipoint[0] + k], action[0], values[0]);

      else
         {
/*
         build the truth array
*/
         for (l = 0; l < num_oper; l++)
            truth[l] = eval1 (iobuf[ipoint[images[l]-1]+k], action[l], 
                              values[l]);

/*
         evaluate the rpn expression
         ---------------------------
*/
         stkcnt = 0;
         for (l = 0; l <= index; l++)
            {
            if (rpn[l] < 0) 
               {
               if (stkcnt != 0) 
                  for (m = stkcnt; m >= 1; m--)
                     stack[m] = stack[m - 1];
               stack[0] = truth[-rpn[l] - 1];
               stkcnt++;
               }
            else
               {
               stack[0] = eval2 (stack[0], rpn[l], stack[1]);
               for (m = 1; m < stkcnt - 1; m++)
                  stack[m] = stack[m + 1];
               stkcnt--;
               }  
            }
         }
      if (stack[0]) 
         {
         iobuf[opoint + k] = true_val;
         if (!true_flag) true_flag = TRUE;
         }
      else 
         {
         iobuf[opoint + k] = false_val;
         if (!false_flag) false_flag = TRUE;
         }
      }
   }
c_estep (&gdesc);

c_stoptm ();
c_egclse (&gdesc);
/*
update the output image min and max, and history
------------------------------------------------
*/
if (true_flag && false_flag)
   {
   maxval  = max (true_val, false_val);
   minval  = min (true_val, false_val);
   }
else
   {
   if (true_flag) maxval = minval = true_val;
   else maxval = minval = false_val;
   }
c_svmnmx (hos_out, &one, &minval, &maxval, "YES");

c_hstout (&block, hos_out, hos_in, num_images, "");
}
