#include "lowcal.h"
#include "util.h"
#include "pixman.h"

void filspec
(
    char (*hosin)[CMLEN],    /* input image host names                        */
    char hosout[],           /* output image host name                        */
    long num_inimages,       /* number of input images                        */
    long num_image_bands[][MAXBND+1], /* band numbers in each input image     */
    long (*out_img_bands)[MAXBND+1],  /* band numbers for the output image    */
    long bands_per_image[],  /* number of bands for each image                */
    long *bands,             /* number of output image bands                  */
    char out_type[],         /* output image data type                        */
    long *dtype		    /* output image data type - numeric value        */
)

/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1.0         01-Jun-84   J. Bay, SAR
   2.0         13-Jan-89   K. Kunska        conversion to 5.0
   3.0         17-Apr-89   K. Kunska        corrected phrasing of error msgs.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Open input image files for random access.
   Get the data type of the input image.
   Open the output image file for random access.
   Update the ddr.

**************************************************************************
*/

{

long    in_type;
long    fatal_err = -1;
long    zero = 0;
long    one = 1;
char    *prog_name = "LOWCAL"; /* program name            */


double	line_inc = 1.0,		/* line and sample increments for          */
	samp_inc = 1.0;		/* output image ddr                        */

long	access,			/* file access type                        */
	comb_flag = 0,		/* image combination flag                  */
	maxlines = 1,		/* maximum number of lines to be read and  */
                                /* written into the inbuf and outbuf       */
	n, 			/* array index and loop counter            */
	opt[2],			/* option array for eopenr                 */
	stat,			/* return status                           */
	type;			/* file type                               */


/*
open input image files
---------------------------------
*/
access = IREAD;
type = EREAL;
for (n = 0; n < num_inimages; n++)
   {
   stat = c_eopenr(&fdescin[n], hosin[n], num_image_bands[n], &inwnd[n][SL], 
                   &inwnd[n][SS], &inwnd[n][NL], &inwnd[n][NS], &access, 
                   &type, &zero, &maxlines);
   if (stat != E_SUCC )
      {
      c_abort (&block, "Error opening input image", "lowcal-open", prog_name,
               out_files, &num_outfiles);
      }
   }

fdescin[++n] = 0;

/*
Get the data type of the input image.
-------------------------------------
*/
stat = c_gettyp (hosin, &num_inimages, &in_type);
if (stat != E_SUCC)
   c_errmsg("Unreadable DDR or data types of images are different",
            "lowcal-gettyp", &fatal_err);

/*
Open the output image file
--------------------------
*/
access = IWRITE;
n = 0;
if (strcmp (out_type, "SAME") == 0)
   opt[n] = in_type;
else if (strcmp (out_type, "BYTE") == 0) 
   opt[0] = EBYTE;
else if (strcmp (out_type, "I*2") == 0)
   opt[0] = EWORD;
else if (strcmp (out_type, "I*4") == 0)
   opt[0] = ELONG;
else
   opt[0] = EREAL;
opt[++n] = 0;

*dtype = opt[0];

stat = c_eopenr (&fdescout, hosout, bands, &one, &one, &inwnd[0][NL], 
                 &inwnd[0][NS], &access, &type, opt, &maxlines);

if (stat != E_SUCC )
   {
   c_abort (&block, "Error opening output image", "lowcal-outopen", prog_name,
               out_files, &num_outfiles);
   }

for (n = 0; n < *bands; n++)
   out_img_bands[0][n] = n+1;

c_upddr (&block, hosin, hosout, &num_inimages, inwnd, num_image_bands, 
         bands_per_image, &line_inc, &samp_inc, bands, &comb_flag);

}


