#include "lowcal.h"
#include "util.h"

void getpar
(
    char taein[][CMLEN], 	/* input image tae names		      */
    char hosin[][CMLEN], 	/* input image host names	 	      */
    char taeout[][CMLEN],		/* output image tae names                     */
    char hosout[][CMLEN],		/* output image host names		      */
    long *num_inimg,		/* number of input images                     */
    long (*num_img_band)[MAXBND+1], /* stores band numbers for each image */
    long *bands_per_image, 	/* number of bands per image                 */
    long *tot_inband,           /* total number of bands to be processed      */
    long img_limit,		/* maximum number of images                   */
    long *same_wnd,		/* same window size? 1=Y, 0=N                 */
    long img_flag[MAXIMG][2],   /* specified image/band options               */
    long *niter,   		/* number of iterations                       */
    double omin[],			/* minimum pixel values                       */
    double omax[],			/* maximum pixel values                       */
    char (*out_type)[5]		/* output data type                           */
)
/*
*****************************************************************************

NAME:	GETPAR

PURPOSE: Get parameters.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         31-May-84    J. Bay, SAR
   2         13-Jan-89    K. Kunska          conversion to 5.0    
   3         17-Apr-89    K. Kunska          fixed minmax option, list of
                                             output images, phrasing of error
                                             messages; ensures that an existing
                                             output image is not overwritten
   4         28-Nov-89    K. Kunska          fixed validity checking for row 
                                             and colunm coefficients

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialize the program.
   Get the local window dimensions.
   Get the subcommand.
   Perform validity checks on extra option parameters.
   Parse the input image specification.
   Get the number of iterations and the output data type.
   For all input images, validate the dimensions.
   Find minimum and maximum pixel values for the quantized mode option.
   Get the output image file name.
   Get LAS gobals.

*****************************************************************************
*/

{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;
long    two = 2;
long    fifteen = 15;


char	*command[13],		/* array of subcommands                       */
/*	dsc[2],	 */		/* data storage code (used in data validation)*/
	mnmx[4],		/* minmax option                              */
	subcmd[20];		/* user supplied subcommand name              */

double	wndsiz;			/* window size (temporary)           	      */

long    found,                  /* flag used to process subcommands           */
	i,			/* loop counter                               */
	icsiz[2],            	/* dimensions of crow and ccol                */
/*	mean,	*/		/* mean from statistic()                      */
	outtype,                /* outyp parameter                            */
	stat,                   /* return status                              */
	std_out,		/* logical unit for standard output           */
/*	stdv,	*/		/* standard deviation from statistic()        */
	valret;                 /* number of values returned by getnum()      */

struct VARIABLE	 *var;	/* pointer of a variable in TAE block         */

img_limit = MAXIMG;

command[0] = "MEAN";
command[1] = "STDEV";
command[2] = "VARIANCE";
command[3] = "QUANT";
command[4] = "MINIMUM";
command[5] = "MAXIMUM";
command[6] = "MEDIAN";
command[7] = "MABS";
command[8] = "SHARP";
command[9] = "KNN";
command[10] = "SELECT";
command[11] = "MSUB";
command[12] = "CONV";

/* initialize program
---------------------
*/
c_inlas(&block, &std_out);

/*
get local window dimensions
----------------------------
*/
outtype = ELONG;
c_getnum (&block, "KERNDIM", &two, &outtype, iwsize);


/*
local window dimensions cannot be 1 by 1
-----------------------------------------
*/
if ((iwsize[NR] == 1) && (iwsize[NP] == 1)) 
   c_errmsg ("Local window dimensions cannot be 1 by 1", "lowcal-window",
	      &fatal_err);

/*
 get subcommand
---------------
*/
var = p_find (&block, "_SUBCMD");
strcpy (subcmd, SVAL(*var, 0));

found = FALSE;
for (i = 0; found != TRUE; i++)
   {
   if (strcmp(subcmd, command[i]) == 0)
      {
      found = TRUE;
      iopt = i + 1;
      if (iopt == 4)
         {
         c_getnum (&block, "NBINS", &one, &outtype, &nbins);
         c_getnum (&block, "MINVAL", &one, &outtype, &mthre);
         }
      else if (iopt == 10)		
         c_getnum (&block, "NNN", &one, &outtype, &kn);
      else if (iopt == 11)
         {
         c_getnum (&block, "NNTHR", &one, &outtype, &jns);
         outtype = EREAL;
         c_getnum (&block, "THRVAL", &one, &outtype, &thresh);
         }
      else if (iopt == 13)
         {
         outtype = EREAL;
         valret = c_getnum (&block, "LINECOEF", &fifteen, &outtype, crow);
         icsiz[0] = valret;
         valret = c_getnum (&block, "SAMPCOEF", &fifteen, &outtype, ccol);
         icsiz[1] = valret;
         }
      }
   }


/*
 do validity checks on extra option parameters
----------------------------------------------
*/
if (iopt == 4) 
   {
   if (nbins == 0) 
      nbins = iwsize[NR] * iwsize[NP];
   if (mthre > iwsize[NR] * iwsize[NP]) 
      c_errmsg ("Threshold value exceeds limit", "lowcal-check", 
                &fatal_err);
   else if (mthre == 0) 
      {
      wndsiz = iwsize[NR] * iwsize[NP];
      mthre = (long)((sqrt(wndsiz)) + 0.5);
      }
   }
else if (iopt == 10) 
   {
   if (kn >= (iwsize[NR] * iwsize[NP])) 
      c_errmsg ("Nearest neighbor value exceeds limit", "lowcal-check", 
                 &fatal_err);
   else if (kn == 0) 
      {
      wndsiz = iwsize[NR] * iwsize[NP];
      kn = (long)((sqrt(wndsiz)) + 0.5);
      }
   }
else if (iopt == 11) 
   {
   if (jns >= (iwsize[NR] * iwsize[NP])) 
      c_errmsg ("Number of  neighbors exceeds limit", "lowcal-check", 
                &fatal_err);
   else if (jns == 0) 
      {
      wndsiz = iwsize[NR] * iwsize[NP];
      jns = (long)((sqrt(wndsiz)) + 0.5);
      }
   }
else if (iopt == 13) 
   {
   if (icsiz[1]  !=  iwsize[NR]) 
      c_errmsg ("Dimension of SAMPCOEF must match KERNDIM(1)", "lowcal-check", 
                &fatal_err);
   if (icsiz[0]  !=  iwsize[NP]) 
      c_errmsg ("Dimension of LINECOEF must match KERNDIM(2)", "lowcal-check", 
                 &fatal_err);
   } 

/*
Parse the input image specification
-----------------------------------
*/
stat = c_getin (&block, "IN", &zero, taein, hosin, num_inimg, inwnd, 
		num_img_band, bands_per_image, tot_inband, &img_limit, 
		same_wnd, img_flag); 

if (stat != E_SUCC)
	c_errmsg ("Cannot access input file names", "lowcal-getin",
                   &fatal_err);
if (*same_wnd == 0)
	c_errmsg ("Image windows not the same size", "lowcal-window",
                   &fatal_err);

/*
Get the number of iterations and the output type
------------------------------------------------
*/
outtype = ELONG;
c_getnum (&block, "NITER", &one, &outtype, niter);
var = p_find (&block, "ODTYPE");
strcpy (out_type[0], SVAL(*var, 0));

/*
Loop on images
--------------
*/
for (i = 0; i < *num_inimg; i++)
   {

/*
Validate window
----------------
*/
   c_getddr (hosin[i], &ddr);
   if (ddr.ns < inwnd[i][NP]) 
      c_errmsg ("Image window must be at least as large as the local window in number of pixels",
		"lowcal-pixels", &fatal_err);
   else if (ddr.nl < inwnd[i][NL]) 
      c_errmsg ("Image window must be at least as large as the local window in the number of lines",
		"lowcal-lines", &fatal_err);

   }

/* 
Find minimum and maximum pixel values for the quantized mode
-------------------------------------------------------------
*/
   if (iopt == 4)
      c_minmax (hosin, num_inimg, inwnd, num_img_band, bands_per_image, 
                omin, omax);   

/* 
Get the output image file name.
---------------------------------
*/
stat = c_getout (&block, "OUT", taeout, hosout);
if (stat != E_SUCC)
   c_errmsg ("Cannot open output image file", "lowcal-getout", &fatal_err);
num_outfiles = 0;
strcpy (out_files[num_outfiles++], taeout[0]);

/* 
Get LAS gobals
--------------
*/
c_getnum (&block, "$MSGTIM", &one, &outtype, &msgtim);
var = p_find (&block, "$MINMAX");
strcpy (mnmx, SVAL(*var, 0));
if (mnmx[0] == 'Y')
   min_max = TRUE;
else
   min_max = FALSE;

}
