#ifndef PROTOS_H
#define PROTOS_H

void convol
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of images                           */
    long num_img_bands[][MAXBND+1],	/* band numbers for input images      */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands		/* total number of bands                      */
);

void filspec
(
    char (*hosin)[CMLEN],    /* input image host names                        */
    char hosout[],           /* output image host name                        */
    long num_inimages,       /* number of input images                        */
    long num_image_bands[][MAXBND+1], /* band numbers in each input image     */
    long (*out_img_bands)[MAXBND+1],  /* band numbers for the output image    */
    long bands_per_image[],  /* number of bands for each image                */
    long *bands,             /* number of output image bands                  */
    char out_type[],         /* output image data type                        */
    long *dtype		    /* output image data type - numeric value        */
);

void getpar
(
    char taein[][CMLEN], 	/* input image tae names		      */
    char hosin[][CMLEN], 	/* input image host names	 	      */
    char taeout[][CMLEN],		/* output image tae names                     */
    char hosout[][CMLEN],		/* output image host names		      */
    long *num_inimg,		/* number of input images                     */
    long (*num_img_band)[MAXBND+1], /* stores band numbers for each image */
    long *bands_per_image, 	/* number of bands per image                 */
    long *tot_inband,           /* total number of bands to be processed      */
    long img_limit,	   /* maximum number of images                   */
    long *same_wnd,	   /* same window size? 1=Y, 0=N                 */
    long img_flag[MAXIMG][2],   /* specified image/band options               */
    long *niter,   		/* number of iterations                       */
    double omin[],			/* minimum pixel values                       */
    double omax[],			/* maximum pixel values                       */
    char (*out_type)[5]		/* output data type                           */
);

void knear
(
    long iter,              /* current iteration                          */
    long niter,     		/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of output images                    */
    long num_img_bands[][MAXBND+1],	/* band numbers for the input images  */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands		/* total number of bands                      */
);

void median
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of input images                     */
    long num_img_bands[][MAXBND+1],	/* band numbers for the input images  */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands		/* total number of bands to be processed      */
);

void quan
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of images                           */
    long num_img_bands[][MAXBND+1],	/* band numbers for input images      */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    double omin[],			/* array of minimum pixel values             */
    double omax[],			/* array of maximum pixel values             */
    long tot_inbands
);

void selavg
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of images                           */
    long num_img_bands[][MAXBND+1],	/* band numbers for input images      */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands		/* total number of bands                      */
);

void sharp
(
    long iter,			/* current iteration                          */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of output images                    */
    long num_img_bands[][MAXBND+1],	/* band numbers for the input images  */
    long out_img_bands[][MAXBND+1],	/* band numbers for the output image  */
    long tot_inbands			/* total number of bands              */
);

void sort2
(
    float b[], 		/* array of absolute differences to be sorted         */
    float c[],		/* array of corresponding original pixel values       */
    long numbr          /* number values to sort (size of local window)       */
);

void varian
(
    long iter,			/* current iteration		              */
    long niter,			/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,           /* number of images                           */
    long num_img_bands[][MAXBND+1],/* band numbers for input images           */
    long out_img_bands[][MAXBND+1],/* band numbers for the output image       */
    long tot_inbands		/* total number of bands                      */
);

void xmean
(
    long iter,			/* current iteration			   */
    long niter,			/* total number of iterations		   */
    long bands_per_image[],	/* number of bands to be processed         */
    long num_img,		/* number of images                        */
    long num_img_bands[][MAXBND+1], /* band numbers for input images        */
    long out_img_bands[][MAXBND+1], /* band numbers for the output image    */
    long tot_inbands		/* total number of bands                   */
);

float xmed
(
    float	box[],			  /* image data for one box                   */
    long	bsize,			  /* local window size                        */
    long 	nhalf             /* half of local window size                */
);

void xminmax
(
    long iter,          		/* current iteration                          */
    long niter,              	/* total number of iterations                 */
    long bands_per_image[],	/* number of bands for each input image       */
    long num_img,		/* number of images                           */
    long num_img_bands[][MAXBND+1],/* band numbers for input images           */
    long out_img_bands[][MAXBND+1],/* band numbers for the output image       */
    long tot_inbands		/* total number of bands                      */
);


#endif
