#include "lowcal.h"

float xmed
(
    float	box[],			  /* image data for one box                   */
    long	bsize,			  /* local window size                        */
    long 	nhalf             /* half of local window size                */
)
/*
*****************************************************************************

NAME:	XMED

PURPOSE: Computer median value of the array box[].

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1	     05-Jun-84    J. Bay, SAR
   2         18-Sep-85    Y. Sheu,           non-ap conversion
   3         13-Jan-89    K. Kunska          conversion to 5.0    

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE
        Called by MEDIAN routine

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   For all pixels in the window
      Read the current pixel.
      Store the pixel in either the xlo or the xhi array depending on the 
      pixel value and the sizes of the arrays.
      Store the median (either the minimum value of the xhi array or the 
      maximum value of the xlo array) in x.
   Return x.
      
*****************************************************************************
*/


{

float	maxlo[3],		  /* max value of xlo array and its index     */
	minhi[3],		  /* min value of xhi array and its index     */
	x,			  /* candidate for median value               */
	xlo[113], 		  /* pixel values lower than median           */
	xhi[113];		  /* pixel values higher than median          */

long 	i, 			  /* loop counter and array index             */
	in,			  /* loop counter and array index             */
	index,
	j,       		  /* loop counter and array index             */
	nhi,			  /* index into xhi array                     */
	nlo;			  /* index into xlo array                     */

nlo = 0;
nhi = 0;
x = box[0];
bsize = bsize - 1;

for (in = 1; in <= bsize; in++)
   {
   if (x < box[in])
      {
      if (nhi >= nhalf) 
         {  
         nlo++;
         xlo[nlo] = x;
         xhi[nhi+1] = box[in];

/*
Find the minimal value of xhi array.
------------------------------------
*/
         minhi[1] = xhi[1];
         minhi[2] = 1;
         for (i = 2; i <= nhi+1; i++)
            {
            if (xhi[i]  <  minhi[1]) 
               {
               minhi[1] = xhi[i];
               minhi[2] = (float) i;
               }
             }
         x = minhi[1];
         index = (long)minhi[2];

/*
Remove the minimal value from the xhi array.
--------------------------------------------
*/
         for (i = index; i <= nhi; i++)
            xhi[i] = xhi[i+1];
         }
      else       /* x < box(in); nhi < nhalf */
         {
         nhi++;
         xhi[nhi] = box[in];
         }
      }
   else if (x > box[in])
      {
      if (nlo >= nhalf) 
         {
         nhi++;
         xhi[nhi] = x;
         xlo[nlo+1] = box[in];

/*
Find the maximum value of xlo array.
------------------------------------
*/
         maxlo[1] = xlo[1];
         maxlo[2] = 1;
         for (j = 2; j <= nlo+1; j++)
            {
            if (xlo[j]  >  maxlo[1]) 
               {
               maxlo[1] = xlo[j];
               maxlo[2] = (float) j;
               }
             }
         x = maxlo[1];
         index = (long)maxlo[2];
/*
Remove the maximum value from the xlo array.
--------------------------------------------
*/
         for (i = index; i <= nlo; i++)
            xlo[i] = xlo[i+1];
         }
      else          /* x > box(in); nlo < nhalf */
         {
         nlo++;
         xlo[nlo] = box[in];
         }
      }
/*
x = box(in):
Place value in the half which currently has fewer entries
---------------------------------------------------------
*/
   else
      { 
      if (nhi > nlo)    /* x = box(in); nhi > nlo */
         {
         nlo++;
         xlo[nlo] = x;
         }
      else    /* x = box(in); nhi <= nlo */
         {
         nhi++;
         xhi[nhi] = x;
         }
      }
   }
return(x);

}  
