/*****************************************************************************
NAME: CALC_COEF

PURPOSE: Calculate the coefficients from the ddr, for the pixel coordinate.

PROGRAM HISTORY:
  Version    Programmer      Date        Reason
  -------    ----------      ----        ------
             K. Hoback       6/99        To get coefficient data only.

*****************************************************************************/
#include "las.h"
#include "ls2pc.h"

int calc_coef
(
  struct DDR *ddr,        /* I: DDR structure */
  double *coef            /* O: Rotation coefficents */
)
{
  double proj_coord[8];   /* Projection coordinates */

  long image_coord[8];    /* Image coordinates line/sample */
  long prt_flag;          /* Print flag */

  /* Make sure project information is valid
  -----------------------------------------*/
  if (ddr->valid[6] == INVAL)
  {
    c_lasmsg(FALSE,"ls2pc-noproj","Projection information not valid");
    return(E_FAIL);
  }
  else
  {
    /* Calculate the rough transformation information
    -------------------------------------------------*/
    proj_coord[0] = ddr->upleft[0];
    proj_coord[1] = ddr->upleft[1];
    proj_coord[2] = ddr->loleft[0];
    proj_coord[3] = ddr->loleft[1];
    proj_coord[4] = ddr->upright[0];
    proj_coord[5] = ddr->upright[1];
    proj_coord[6] = ddr->loright[0];
    proj_coord[7] = ddr->loright[1];

    image_coord[0] = image_coord[1] = 1;
    image_coord[2] = ddr->nl;
    image_coord[3] = 1;
    image_coord[4] = 1;
    image_coord[5] = ddr->ns;
    image_coord[6] = ddr->nl;
    image_coord[7] = ddr->ns;

    prt_flag = TRUE;

    if (c_rtcoef(proj_coord,image_coord,coef,&prt_flag) != E_SUCC)
      return(E_FAIL);
  }

  return(E_SUCC);
}
