/*****************************************************************************
NAME: GETPAR

PURPOSE: Gets the user entered parameters from TAE

PROGRAM HISTORY:
  Version    Programmer	   Date      Reason
  -------    ----------    ----      ------
    1.0      T.Ratliff     10/99     Initial Development

*****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ls2pc.h"

int getpar
(
  struct PARBLK *vblk,          /* I: TAE parameter block */
  struct DDR *ddr,              /* O: Input image DDR structure */
  long *file,                   /* O: PRINT parameter output to a file */
  long *term,                   /* O: PRINT parameter output to a terminal */
  long *lnprt,                  /* O: PRINT parameter output to a printer */
  long *n_ls,                   /* O: Number of line/sample pairs */
  char *outfile,                /* O: Name of the output file */
  char *infile,                 /* O: Name of the input file */
  char *tmpfil,                 /* O: Name of file for FILEUTIL */
  char *prtfile,                /* O: Name of file for PRINT file option */
  long *numfiles,               /* O: Number of output files produced */
  char *filetype                /* O: Type of file that the user requested */
) 
{
  struct VARIABLE *var;         /* Pointer to a TAE variable */
  struct VARIABLE *var1;        /* Pointer to a TAE variable */
  long access;                  /* Read/Write access to a file */
  long bands[1][MAXBND + 1];    /* Bands to be processed within image */
  long one = 1;                 /* Constant value 1 */
  long qflag[1][2];             /* Indicates if subwindows specified */
  long nbands;                  /* Number of bands in each image */
  long num_img;                 /* Number of images in IN */
  long same;                    /* Flag if images have same window */
  long status;                  /* Function return status */
  long totbands;                /* Total number of bands to process */
  long window[1][4];            /* Window specification for image */
  long zero = 0;                /* Constant value 0 */

  char tmp_tae[1][CMLEN];       /* Temp TAE name variable */
  char in_image[1][CMLEN];      /* Name of the input image */

  /* Get the PRINT parameter values
  ---------------------------------*/
  *file = 0;
  *term = 0;
  *lnprt = 0;
  strcpy(tmpfil,"ls2pc");
  status = c_setprt(vblk,term,file,lnprt,tmpfil,prtfile);
  if (status != E_SUCC)
    return(E_FAIL);

  /* Get the name of the input DDR file
  -------------------------------------*/
  status = c_getin(vblk,"IN",&zero,tmp_tae,in_image,&num_img,window,bands,
                   &nbands,&totbands,&one,&same,qflag);
  if (status != E_SUCC)
    return(E_FAIL);

  /* Get the FILETYPE selection
  -----------------------------*/
  var = p_find(vblk, "FILETYPE");
  strcpy(filetype,SVAL(*var,0));
  
  /* Get the OUTFILE name
  -----------------------*/
  access = IWRITE;
  if(!strcmp(filetype,"TIC"))
      status = c_getnam(vblk,"OUTFILE",&zero,outfile,"tic",&access);
  else
      status = c_getnam(vblk,"OUTFILE",&zero,outfile,"tps",&access);
  if (status != E_SUCC)
    return(E_FAIL);

  /* If the PRINT parameter specifies a file
     for output, make sure it is not the
     same name as OUTFILE
  ------------------------------------------*/
  if (*file == 1)
  {
    *numfiles++;
    if (strcmp(prtfile,outfile) == 0)
    {
      c_lasmsg(FALSE,"ls2pc-same",
               "PRINT file name and OUTFILE name are the same");
      return(E_FAIL);
    }
  }

  /* Get the DDR records
  ----------------------*/
  status = c_getddr(in_image[0],ddr);
  if (status != E_SUCC)
    return(E_FAIL);

  var = p_find(vblk,"INFILE");
  if ((*var).v_count > 0)
  {
    /* Get the name of the input file that
       contains the line/sample values
    --------------------------------------*/
    access = IREAD;
    status = c_getnam(vblk,"INFILE",&zero,infile,"txt",&access);
    if (status != E_SUCC)
      return(E_FAIL);
  }
  else
  {
    /* Get the number of lines and samples
    --------------------------------------*/
    var = p_find(vblk,"LINES");
    *n_ls = (*var).v_count;
    var1 = p_find(vblk,"SAMPS");

    /* Make sure the number of lines
       and samples match
    --------------------------------*/
    if (*n_ls != (*var1).v_count)
    {
      c_lasmsg(FALSE,"ls2pc-elements",
               "LINE and SAMPLE arrays must have same number of elements");
      return(E_FAIL);
    }
  }

  return(E_SUCC);
}
