/*****************************************************************************
NAME				 LS2PC 

PURPOSE  Computes projection coordinates from line/sample coordinates 

VERSION    PROGRAMMER	   DATE         REASON
-------    ----------      ----         ------
  1.0      D. Steinwand    10/92        Original development
  2.0	   T. Mittan	   12/92	Release to baseline
           K. Hoback       6/99         Add capability to read 
                                        Lines/Comps from a file.

                                        Change Lines/Samps to use real
                                        number values.

                                        Added a print parameter to control
                                        destination of output.
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "ls2pc.h"

void main
(
  void
)
{
  double coef[6];              /* Rotation coefficients */

  long num_files = 1;          /* Number of files created */
  long n_ls = 0;               /* Number of line/sample pairs */
  long lnprt;                  /* Printer flag for PRINT parameter */
  long term;                   /* Terminal flag for PRINT parameter */
  long file;                   /* File flag for PRINT parameter */
  long status;                 /* Function return status */
  long std_out;                /* Logical unit number for standard output */

  char infile[CMLEN];          /* Input ascii LINE-SAMPLE file */
  char out_files[2][CMLEN];    /* Names of the output files */
  char progname[CMLEN];        /* Name of the program */
  char tmpfil[CMLEN];          /* Temporary file name for fileutil */
  char filetype[4];            /* User selected filetype           */

  struct DDR ddr;              /* DDR record structure */
  struct PARBLK vblk;          /* TAE v-block */

  /* Initialize LAS processing
  ----------------------------*/
  c_inlas(&vblk,&std_out);

  /* Get the TAE parameters
  -------------------------*/
  status = getpar(&vblk,&ddr,&file,&term,&lnprt,&n_ls,out_files[0],infile,
                  tmpfil,out_files[1],&num_files,filetype);
  if (status != E_SUCC)
    c_lasmsg(TRUE,"ls2pc-fatal","Fatal error encountered");
  
  /* Calculate the coefficients
  -----------------------------*/
  status = calc_coef(&ddr,coef);
  if (status != E_SUCC)
      c_lasmsg(TRUE,"ls2pc-fatal","Fatal error encountered");
  
  /*  If the user wants a TIC file.
  ---------------------------------*/
  if(!strcmp(filetype, "TIC"))
  {
      /* Calculate the projection coordinates
      ---------------------------------------*/
      status = process(&vblk,&ddr,n_ls,coef,infile,tmpfil,out_files[0]);
  }
  else /* User wants a TPS file */
  {
      /* Call ls2tps
      --------------*/    
      status = ls2tps(&vblk,&ddr,n_ls,infile,out_files[0], coef);
  }

  if( status != E_SUCC ) 
  {
    strcpy(progname,"ls2pc");
    c_clean(&vblk, progname, out_files, &num_files);
  }


  /* Print compeltion message & exit
  ----------------------------------*/
  c_complt("LS2PC", out_files, &num_files);
}
