/****************************************************************************
SUBPROGRAM:			GETPAR

PURPOSE:	Retrieve the user specified parameters.

PROGRAM HISTORY:
VERSION	     DATE	PROGRAMMER   CODE/CONT	REQUEST
-------      ----  	----------   ---------	-------
  1.0	     7/85	D. Gordon     USGS/EDC	Initial development.
  1.1	    10/92	S. Nelson     USGS/EDC	Made GETPAR separate file.
						Changed from DMS to LAS.
ALGORITHM:
	Initialize TAE and LAS processing.
	Get the DLTF source file name (if specified).
	Get the DLTF entry name.
	Get the output file destination (TERM, LP, File name, or combo.).
	Return.

*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "worgen.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,	/* TAE variable block.				*/
    char *dltfnam,	   /* DLTF entry name. 				*/
    long *term,		/* Terminal output option.			*/
    long *fyle,		/* File output option.				*/
    long *lnptr,		/* Line printer output option.			*/
    char *prtfil,	   /* Print file name.				*/
    char *uoutfil,	   /* Print file name used for the completion
			   message.					*/
    char hostnm[CMLEN]   /* host output file name descriptor 	*/
)

{
struct VARIABLE	*var;   /* Pointer to a variable in the parameter block.*/

long	fatal = -1;	/* Fatal error flag				*/
long	status;		/* Return status indicator.			*/
long	std_out;	/* Standard output.				*/
long	access;		/* Specifiy write access for mknam routine.	*/
long	zero = 0;	/* Specifies no files were created.		*/

char    tmpstr[1][CMLEN];  /* Temp String */

/* Initialize TAE and LAS processing.
  ----------------------------------*/
c_inlas(vblock,&std_out);

/* Get the input file name 
  -----------------------*/
access = IREAD;
status = c_getnam(vblock,"INFILE",&zero,hostnm,"dltf",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","lstdltf-fatal",&fatal);

/* Get the DLTF entry name (if specified).
  ---------------------------------------*/
var = p_find(vblock,"DLTFENTR");

if (var->v_count != 0)
  strcpy(dltfnam,SVAL(*var,0));
else
  strcpy(dltfnam,"");

/* Call the setprt routine to return the term, printer, and file flags.
   The routine also returns the print file name and the name specified by
   the user to be used in the standard completion message.
  --------------------------------------------------------------*/
strcpy(prtfil,"lstdltf");

status = c_setprt(vblock,term,fyle,lnptr,prtfil,uoutfil);

if (status != E_SUCC)
  {
  strcpy(tmpstr[0], "\0");
  c_clean(vblock,"lstdltf",tmpstr,&zero);
  }
return(E_SUCC);
}
