/*******************************************************************************
SUBPROGRAM:			     GETPAR 

PURPOSE:	Retrieve the user specified parameters.

ALGORITHM:

	Initialize LAS and TAE.
	Determine the user-specified destinations.
	Get the list of directories to search for the LT overview text (from
          the TAE globals $USERLIB and $APLIB).
	If no directories are defined by either global
	    Print an error message and abort.
	Return to the main routine.

ALGORITHM REFERENCES:	None
*******************************************************************************/
#include "las.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,		/* Parameter block 		      */
    char libs[][CMLEN],		/* List of libraries to search for
					   LDM overview text		      */
    long *lcount,		/* Number of libraries to search      */
    char ufile[CMLEN]		/* User specified file name	      */
)
{
struct VARIABLE	*var;		/* Pointer to the parameter in the 
					   parameter block   		      */

		long	abort = -1,	/* Abort flag for error messages      */
			i,		/* Looping variable		      */
			lp,		/* Indicates whether the LT overview
					   is to be output to the line printer*/
			noabort = 1,	/* Report and continue flag for error
					   messages			      */
			prtfil,		/* Indicates whether the LT overview
					   is to be output to a text file     */
			std_out,	/* Standard output		      */
			term;		/* Indicates whether the LT overview
                                           is to be output to the terminal    */

		char	pfile[CMLEN];	/* Temporary file to be used for
					   output			      */
/*
   Initialize LAS and TAE
*/
	c_inlas(vblock,&std_out);
/*
   Get the print destination
*/
	strcpy (pfile,"lt");
	 if (c_setprt(vblock,&term,&prtfil,&lp,pfile,ufile) != E_SUCC)
  	     c_errmsg("Error getting print option","lt-print",&abort);
/*
   Get the list of directories to search for the LT Overview text
*/
	*lcount = 0;
	var = p_find(vblock, "$USERLIB");
	if ((*var).v_count != 0)
	    {
	    strcpy (libs[*lcount],SVAL(*var,0));
	    (*lcount)++;
	    }	
	var = p_find(vblock, "$APLIB");
	for (i=0; i < (*var).v_count; i++)
	    {
	    strcpy (libs[*lcount],SVAL(*var,i));
	    (*lcount)++;
	    }	
	if (*lcount == 0)
	    {
	    c_errmsg("Unable to find LT overview text","lt-notfnd",&noabort);
	    c_errmsg ("Fatal error encountered","lt-fatal",&abort);
	    }

	return;
}
