#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"
/*******************************************************************************
NAME:	LT

PURPOSE: Display the LAS labeled table (LT) overview text to the user
         specified destination (terminal, line printer or user-specified
         text file).

PROGRAM HISTORY:
  Version       Date        Author	      Request
 ---------     ------      --------          ---------
    1.0         2/89       L. Huewe          
    2.0		9/90       T. Sturman         Modified for Labeled Table
					      Overview
    3.0        11/91       T. Mittan          Correct to work correctly when
					      PRINT = lp

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	LT must be run under TAE.

PROJECT:	LAS

ALGORITHM:

  	Get the user specified parameters.
	For each directory defined by the TAE globals $USERLIB and $APLIB
	    Build the LT overview text file name.
            If the file exists
		Set a flag indicating the file was found.
		Break from the loop.
	If the LT overview text was not found
	    Print an error message and abort.
	Send the valid file name (containing both the directory path and file
	  name) back to TAE.
	Print a completion message and terminate the application.

ALGORITHM REFERENCES:	None
*******************************************************************************/
int main
(void)
{
struct	PARBLK	vblock;		/* Parameter block 		              */

	long	abort = -1,	/* Abort flag for error messages	      */
		found = FALSE,	/* Indicates whether the text file was found  */
		i,		/* Looping variable			      */
		lcount,		/* Number of libraries to search 	      */
		noabort = 1,	/* Report and continue flag for error messages*/
		one = 1,	/* Constant one				      */
		status,		/* Status indicator			      */
		zero = 0;	/* Constant zero			      */

	char 	*dirptr,	/* Pointer to directory string		      */
		ltxt[CMLEN],	/* Directory location and file containing the
				   LT Overview text			      */
		libs[30][CMLEN],/* List of libraries to search		      */
		*tptr[1],	/* Array of file pointers		      */
		ufile[1][CMLEN];	/* User specified file name	              */

    char  tmpstr[1][CMLEN];  /* Temp String */

/*
   Get the user specified parameters
*/
	strcpy(ufile[0],"");
	getpar(&vblock,libs,&lcount,ufile[0]);
/*
   Find the directory containing the LT text
*/
	for (i=0; i < lcount; i++)
	    {
#ifdef vms
	    if (strcspn(libs[i],"]") != strlen(libs[i]))
	        sprintf(ltxt,"%s%s",libs[i],"lt.overview");
	    else
	        sprintf(ltxt,"%s:%s",libs[i],"lt.overview");
#else
	    if (strcspn(libs[i],"$") != strlen(libs[i]))
		{
	        dirptr = getenv(&libs[i][1]);
	        sprintf(ltxt,"%s/%s",dirptr,"lt.overview");
		}
	    else
	        sprintf(ltxt,"%s/%s",libs[i],"lt.overview");
#endif
/*
   Determine whether the file exists
*/
	    if (access(ltxt,0) == 0)
		{
		found = TRUE;
		break;
		}
	    }
	if (!found)
	    {
	    c_errmsg("Unable to find LT overview text","lt-notfnd",&noabort);
	    c_errmsg ("Fatal error encountered","lt-fatal",&abort);
	    }
/*
   Send the directory and file name found back to TAE
*/
	tptr[0] = ltxt;
	status = q_string(&vblock,"LTXT",1,tptr,P_UPDATE);
	if (status != SUCCESS)
	    {
   	    c_errmsg ("Error setting variable in TAE parameter block",
	        "lt-taeprm",&noabort);
	    c_errmsg ("Fatal error encountered","lt-fatal",&abort);
	    }
	status = q_out(&vblock);
	if (status != SUCCESS)
	    {
   	    c_errmsg ("Error writing TAE parameter block","lt-taeblk",
		&noabort);
	    c_errmsg ("Fatal error encountered","lt-fatal",&abort);
	    }
/*
   Exit the program
*/
	if (strcmp(ufile[0],"") != 0)
	    c_complt("LT",ufile,&one);
	else
        {
        strcpy(tmpstr[0], "\0");
	    c_complt("LT",tmpstr,&zero);
        }

return EXIT_SUCCESS;

}
