/******************************************************************************

FUNCTION:       GETENF

PURPOSE:        Reads in the end files (trailer and supplemental) from tapes 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

ALGORITHM REFERENCES
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "ltwg2in.h"
#include "labserv.h"
#include "protos.h"

int getenf
(
    char hosout[][CMLEN],	/* I/O: output host names 	      	     */
    long *unit,                 /* I/O: tape unit number                     */
    long *wrtdsk,               /* I: ancillary files to read in             */
    long *num_out,              /* I/O: number of output files 		     */
    char comment[][CMLEN],      /* I: tape mount message to the operator     */
    long *num_trailrec,	        /* I: number of trailer records		     */
    long *tapenum,		/* I: current tape mounted		     */
    long *trailsiz,	        /* I: record size of the trailer file	     */
    long *numtapes,	        /* I: number of tapes specified		     */
    long *no_lead,		/* I: flag if there is a leader file on tape    
			           there is not a leader file if an image 
                                   band is split across two tapes	     */
    char *satnam                /* I: satellite name                         */
)

{
FILE *fp;		/* file descriptor			     */
   
long  i=0;		/* loop counter				     */
long  dumlong=0;		/* dummy value                               */
long  dum2 = FALSE;	/* dummy value                               */
long  mode = HANDLE;    /* tape I/O error flag			     */
long  numrecs=0;	/* number of records in the file	     */
long  one = 1;		/* constant of one			     */
long  recsize=0;		/* record size of the file		     */
long  status=0;		/* return status                             */
long  skip=0;		/* number of files to skip		     */
long  tlng1[7];         /* dummy value				     */
long  tlng2[7];	        /* dummy value				     */

unsigned char *in_buf;  /* buffer read from image on tape            */
char tempbuf[10];	/* temp work buffer			     */
char ttype[CMLEN];      /* tape type		                     */	
char tmpstr[7][1];	/* dummy value				     */

/* Mount the last tape if not already mounted.  Some tape sets 
   have these files at the end of each tape, but it is not consistent.
----------------------------------------------------------------------*/
if (*tapenum != *numtapes)
   {
   close_tape(unit);
   if (mount_tape(unit, comment, *numtapes, numtapes,ttype) != E_SUCC)
      return(E_FAIL);
   /* Read the volume descriptor file
    --------------------------------*/
   if (getvdf(unit, &dum2, hosout, &dumlong, &dumlong, &dumlong, 
              &dumlong,tempbuf,&dumlong,&dumlong,&dumlong,
              tmpstr,tlng1,tlng2,&dumlong) != E_SUCC)
       return(E_FAIL);

   /* Skip the leader file and/or the image date portion 
   ----------------------------------------------------*/
   if (*no_lead)
      skip = 1;
   else
      skip = 2;
   if (c_tpfsf(unit,&skip,&mode) != TAPE_SUCC)
      return(E_FAIL);
   }

/* Write the trailer file to disk
----------------------------------*/
if (wrtdsk[TRAI])
   {
   /* Allocate memory for the input buffer
   ---------------------------------------*/
   in_buf = (unsigned char *) malloc(*trailsiz);
   if (in_buf == NULL)
      {
      c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
      return(E_FAIL);
      }

   /* Create the file name
   -----------------------*/
   c_lsmknm(hosout[0],"trai",hosout[*num_out]);
   fp = fopen(hosout[(*num_out)++],"w");

   /* Read and write the records 
   -----------------------------*/
   for (i = 0; i < *num_trailrec; i++)
       {
       status = c_tpread(unit,in_buf,trailsiz,&mode);
       if (status != *trailsiz)
          {
          c_errmsg("Error reading from tape","ltwg2in-read",&one);
          return(E_FAIL);
          }
       fwrite(in_buf,1,*trailsiz,fp);
       }

   /* Close the file
   -----------------*/
   fclose(fp);

   /* One more read to read EOF
   ----------------------------*/
   if (c_tpread(unit,in_buf,trailsiz,&mode) != TAPE_EOF)
      {
      c_errmsg("Error reading from tape","ltwg2in-read",&one);
      return(E_FAIL);
      }

   /* Free allocated space
   -----------------------*/
   free(in_buf);
   }
else
   /* Skip the file completely
   ---------------------------*/
   if (c_tpfsf(unit,&one,&mode) != TAPE_SUCC)
      return(E_FAIL);

/* Spot data does not have a supplemental file
-----------------------------------------------*/
if (wrtdsk[SUPP] && (strncmp(satnam,"SP",2) == 0))
   {
   c_errmsg("No supplemental file","ltwg2in-supp",&one);
   return(E_SUCC);
   }
else if (strncmp(satnam,"SP",2) == 0)
   return(E_SUCC);

      
/* Read the supplemental volume directory file
---------------------------------*/
if (getvdf(unit, &dum2, hosout, &dumlong, &dumlong, &dumlong, 
          &dumlong,tempbuf,&numrecs,&recsize,&dumlong,
	  tmpstr,tlng1,tlng2,&dumlong) != E_SUCC)
    return(E_FAIL);

/* Write the supplemental file to disk
-------------------------------------*/
if (wrtdsk[SUPP])
   {
   /* Allocate memory for the input buffer
   ---------------------------------------*/
   in_buf = (unsigned char *) malloc(recsize);
   if (in_buf == NULL)
      {
      c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
      return(E_FAIL);
      }

   /* Create the file name
   -----------------------*/
   c_lsmknm(hosout[0],"supp",hosout[*num_out]);
   fp = fopen(hosout[(*num_out)++],"w");

   /* Read and write the records.  These are variable length
      so don't check status from tpread
   --------------------------------------------------------*/
   for (i = 0; i < numrecs; i++)
       {
       status = c_tpread(unit,in_buf,&recsize,&mode);
       if (status == TAPE_EOF)
          break;
       fwrite(in_buf,1,status,fp);
       }

   /* Close the file
   -----------------*/
   fclose(fp);

    /* One more read to read EOF
   ----------------------------*/
   if (c_tpread(unit,in_buf,&recsize,&mode) != TAPE_EOF)
      {
      c_errmsg("Error reading from tape","ltwg2in-read",&one);
      return(E_FAIL);
      }

   /* Free allocated space
   -----------------------*/
   free(in_buf);
   }
else
  /* skip supp file completely
  ----------------------------*/
  if (c_tpfsf(unit,&one,&mode) != TAPE_SUCC)
    return(E_FAIL);

/* Read the calibration volume directory file
---------------------------------*/
if (getvdf(unit, &dum2, hosout, &dumlong, &dumlong, &dumlong,
          &dumlong,tempbuf,&numrecs,&recsize,&dumlong,
          tmpstr,tlng1,tlng2,&dumlong) != E_SUCC)
    return(E_FAIL);

/* Write calibration file to disk
----------------------------------*/
if (wrtdsk[CAL])
   {
   /* Allocate memory for the input buffer
   ---------------------------------------*/
   in_buf = (unsigned char *) malloc(recsize);
   if (in_buf == NULL)
      {
      c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
      return(E_FAIL);
      }

   /* Create the file name
   -----------------------*/
   c_lsmknm(hosout[0],"cal",hosout[*num_out]);
   fp = fopen(hosout[(*num_out)++],"w");

   /* Read and write the records. Treat this as a blind loop since
      we may not be able to depend on number of records from volume
      descriptor 
   ----------------------------------------------------------------*/
   for (i = 0; ; i++)
       {
       status = c_tpread(unit,in_buf,&recsize,&mode);
       if (status == TAPE_EOF)
          break;
       if (status != recsize)
         {
         c_errmsg("Error reading from tape","ltwg2in-read",&one);
         return(E_FAIL);
         }
       fwrite(in_buf,1,status,fp);
       }

   /* Close the file
   -----------------*/
   fclose(fp);

   /* Free allocated space
   -----------------------*/
   free(in_buf);
   }
else
  /* skip cal file completely 
  -----------------------------*/
  if (c_tpfsf(unit,&one,&mode) != TAPE_SUCC)
    return(E_FAIL);

/* Skip Null volume descriptor
-------------------------------*/
if (c_tpfsf(unit,&one,&mode) != TAPE_SUCC)
    return(E_FAIL);

/* Read in the work order summary file
---------------------------------------*/
if (wrtdsk[WOR])
  {
  recsize=8192;
  in_buf = (unsigned char *) malloc(recsize);
  if (in_buf == NULL)
    {
    c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
    return(E_FAIL);
    }
  
  /* Create file name
  -------------------*/
  c_lsmknm(hosout[0],"wo",hosout[*num_out]);
  fp = fopen(hosout[(*num_out)++],"w");

  /*Read and write the records
  ----------------------------*/
  for (i=0;;i++)
    {
    status = c_tpread(unit,in_buf,&recsize,&mode);
    if (status == TAPE_EOF)
      break;
    fwrite(in_buf,1,status,fp);
    }
  /* Close the file
  -----------------*/
  fclose(fp);
  
  /* Free allocated space
  -----------------------*/
  free(in_buf);
  }
else
  /* skip work order file completely 
  -----------------------------*/
  if (c_tpfsf(unit,&one,&mode) != TAPE_SUCC)
    return(E_FAIL);

/* Read in the history summary file
---------------------------------------*/
if (wrtdsk[HST])
  {
  recsize=8192;
  in_buf = (unsigned char *) malloc(recsize);
  if (in_buf == NULL)
    {
    c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
    return(E_FAIL);
    }

  /* Create file name
  -------------------*/
  c_lsmknm(hosout[0],"hst",hosout[*num_out]);
  fp = fopen(hosout[(*num_out)++],"w");

  /*Read and write the records
  ----------------------------*/
  for (i=0;;i++)
    {
    status = c_tpread(unit,in_buf,&recsize,&mode);
    if ((status == TAPE_EOF) || (status == TAPE_EOT))
      break;
    fwrite(in_buf,1,status,fp);
    }
  /* Close the file
  -----------------*/
  fclose(fp);

  /* Free allocated space
  -----------------------*/
  free(in_buf);
  }

return(E_SUCC);
}
