/******************************************************************************
FUNCTION:	GETPAR


PURPOSE:	Get the input parameters

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltwg2in.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,  /* I: TAE parameter block                       */
    long *bands,	    /* O: bands to be processed within image        */
    long *msgtim,	    /* O: elapsed time between display msg          */
    long *window,	    /* O: window of input image                     */
    long *minmax,	    /* O: min max flag                              */
    char hosout[][CMLEN],   /* O: Output host name          		    */
    char taeout[][CMLEN],   /* O: Output TAE name           		    */
    char comment[][CMLEN],  /* O: tape mount message to the operator        */
    long *wrtdsk,           /* O: ancillary files to read in                */
    long *numtapes	    /* O: number of tapes specified		    */
)

{
struct VARIABLE *val;	/* pointer to a TAE variable 	             */
   
long fatal = -1;	/* Fatal error code		             */
long index=0;		/* loop index                                */
long std_out=0;		/* standard output 			     */
long temp=0;		/* temporary variable			     */	
long test=0;		/* modulus test parm for input window        */
long tflag=0;		/* thermal specification flag		     */
   
/* Initialize LAS
-----------------*/
c_inlas(vblock,&std_out);
   
/* Retrieve tape mount message supplied by user 
----------------------------------------------*/
val = p_find(vblock,"COMMENT");
*numtapes = val->v_count;
for (index = 0; index < val->v_count; index++)
    strcpy(comment[index],SVAL(*val,index));

/* Retrieve output image name
-----------------------------*/
if (c_getout(vblock,"OUT",(char (*)[256])taeout[0],
		(char (*)[256])hosout[0]) == E_FAIL)
   c_errmsg("Fatal error encountered","ltwg2in-fatal",&fatal);

/* Get the requested bands
--------------------------*/
for (index = 0; index < MAX_BAND;  index++)
    bands[index] = FALSE;
val = p_find(vblock, "BANDS");
if (val->v_count == 0)
   {
   for (index = 0; index < MAX_BAND; index++)
      bands[index] = TRUE;
   }
else
   {
   for (index = 0; index < val->v_count; index++)
      {
      temp= IVAL(*val,index) - 1;
      bands[temp] = TRUE;
      }
   }

/* Set the Thermal band flag
------------------------------*/
if (bands[5] == TRUE) tflag=1;

/* Retrieve window area to be copied
------------------------------------*/
val = p_find(vblock,"WINDOW");
if (val->v_count != 0)
  {
   for (index = 0; index < 4; index++)
     {
     window[index] = IVAL(*val,index);
     if (window[index] < 1)
        c_errmsg("WINDOW specifications must be greater than or equal to 1",
		"ltwg2in-window", &fatal);
     }
   if ((window[0] !=1) && (tflag == 1))
     {
     if ((test = window[0]%4) != 0)
       c_errmsg("Invalid window specification","ltwg2in-wininc",&fatal);
     }
   if ((window[1] !=1) && (tflag == 1))
     {
     if ((test = window[1]%4) != 0)
       c_errmsg("Invalid window specification","ltwg2in-wininc",&fatal);
     }
   if  (tflag == 1)
     {
     if ((test = window[2]%4) != 0)
       c_errmsg("Invalid window specification","ltwg2in-wininc",&fatal);
     }
   if (tflag == 1)
     {
     if ((test = window[3]%4) != 0)
       c_errmsg("Invalid window specification","ltwg2in-wininc",&fatal);
     }
  }
else
   for (index = 0; index < 4; index++)
     window[index] = 0;
   

/* Get requested ancillary files
--------------------------------*/
for (index = 0; index < 8; index++)
    wrtdsk[index] = FALSE;
val = p_find(vblock,"WRTDSK");
for (index = 0; index < val->v_count; index++)
   {
   if (strcmp(SVAL(*val,index),"VDF") == 0)
      wrtdsk[VDF] = TRUE;
   else if (strcmp(SVAL(*val,index),"LEAD") == 0)
      wrtdsk[LEAD] = TRUE;
   else if (strcmp(SVAL(*val,index),"TRAI") == 0)
      wrtdsk[TRAI] = TRUE;
   else if (strcmp(SVAL(*val,index),"SUPP") == 0)
      wrtdsk[SUPP] = TRUE;
   else if (strcmp(SVAL(*val,index),"IDR") == 0)
      wrtdsk[IDR] = TRUE;
   else if (strcmp(SVAL(*val,index),"CAL") == 0)
      wrtdsk[CAL] = TRUE;
   else if (strcmp(SVAL(*val,index),"WOR") == 0)
      wrtdsk[WOR] = TRUE;
   else if (strcmp(SVAL(*val,index),"HST") == 0)
      wrtdsk[HST] = TRUE;
   }

/* Retrieve number of seconds between processing messages
---------------------------------------------------------*/
val = p_find(vblock,"$MSGTIM");
*msgtim = IVAL(*val,0);
   
/* Retrieve min/max flag
------------------------*/
val = p_find(vblock,"$MINMAX");
if (strcmp(SVAL(*val,0),"YES") == 0)
  *minmax = TRUE;
else
  *minmax = FALSE;
}
