/*******************************************************************************
FUNCTION:	GETVDF

PURPOSE:	Read the volume directory file tape and write it to disk if
		specified.  Extract needed information

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "ltwg2in.h"
#include "labserv.h"
#include "protos.h"

int getvdf
(
    long *unit,		/* I: tape unit number				*/
    long *wrtdsk,	/* I: ancillary files to read in 		*/
    char hosout[][CMLEN],/* I/O: Output host names                     	*/
    long *num_out, 	/* I/O: number of output files                	*/
    long *recsize,	/* O: image record length 			*/
    long *numimg_recs,	/* O: number of image records on this tape	*/
    long *num_trailrec,	/* O: number of trailer records			*/
    char *order,	/* O: band order				*/
    long *numlead,	/* O: number of leader files			*/
    long *leadsize,	/* O: size of leader records			*/
    long *trailsiz,	/* O: max size of trailer records		*/
    char bandnum[][1],  /* O: bandnum ascii array			*/
    long bandrecsiz[7], /* O: band record size				*/
    long bandnumrec[7], /* O: number of records per band		*/
    long *cmode		/* O: CEOS mode (1,2,3)				*/
)
{
FILE *fp;		/* file pointer					*/

unsigned char *in_buf;	/* buffer read from data on tape           	*/
char tempbuf[20];       /* temp work buffer				*/

long i=0;		/* loop counter					*/
long mode = HANDLE;	/* tape I/O error flag                        	*/
long numbytes = 360;	/* number of bytes in a volume directory record */
long numrecs=0;		/* number of records in the file		*/
long one = 1;		/* constant of one				*/
long status=0;		/* status flag 					*/


/* Allocate space
-----------------*/
in_buf = (unsigned char *) malloc(numbytes);
if (in_buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","ltwg2in-alloc",&one);
   return(E_FAIL);
   }

/* Create name for volume directory file 
----------------------------------------*/
if (*wrtdsk)
   {
   c_lsmknm(hosout[0],"vdf",hosout[*num_out]);
   fp = fopen(hosout[(*num_out)++],"w");
   }

/* Read and optionally write the volume directory file.
   Get the number of records in this file from the first
   records  Extract other needed information. 
--------------------------------------------------------*/
numrecs = -1;
for (i = 0; ; i++)
    {
    if (i == numrecs)
       break;
    status = c_tpread(unit,in_buf,&numbytes,&mode);
    if (status != numbytes) 
       {
       c_errmsg("Error reading from tape","ltwg2in-read",&one);
       return(E_FAIL);
       }

    switch (i)
      {
      /* Get the number of records in volume directory file
      ----------------------------------------------------*/
      case 0: strncpy(tempbuf,(const char *)&in_buf[164],4);
              tempbuf[4] = NULL;
              numrecs = atol(tempbuf);
              break;

      /* Get the number of records in leader file and size of records
      ---------------------------------------------------------------*/
      case 1: strncpy(tempbuf,(const char *)&in_buf[100],8);
              tempbuf[8] = NULL;
              *numlead = atol(tempbuf);
              strncpy(tempbuf,(const char *)&in_buf[116],8);
              tempbuf[8] = NULL;
              *leadsize = atol(tempbuf);
              break;

      /* Get the record length and band order  Also get the
         number of image records on this tape for BIL.
         if this is the third record of the supplemental vol,
         then break.
      -----------------------------------------------------*/
      case 2: strncpy(tempbuf,(const char *)&in_buf[66],9); 
              tempbuf[9] = NULL;
              if (strcmp(tempbuf,"PROCESSED")==0) 
  		break;
	      strncpy(tempbuf,(const char *)&in_buf[116],8);
              tempbuf[8] = NULL;
              *recsize = atol(tempbuf);
              bandrecsiz[0] = atol(tempbuf);
	      strncpy(tempbuf,(const char *)&in_buf[100],8);
	      bandnumrec[0] = atol(tempbuf); 
	      strncpy(bandnum[0],(const char *)&in_buf[35],1);
	      if (strcmp(bandnum[0],"P") != 0)
		*cmode = 1;
              strncpy(order,(const char *)&in_buf[32],3);
              strncpy(tempbuf,(const char *)&in_buf[152],8);
              tempbuf[8] = NULL;
              /* Subtract the one image file descriptor record */
              *numimg_recs = atol(tempbuf) - 1;
              break;

      /* Get the number and size of the trailer records
      ------------------------------------------------*/
      case 3: strncpy(tempbuf,(const char *)&in_buf[100],8);
              tempbuf[8] = NULL;
              *num_trailrec = atol(tempbuf);
              strncpy(tempbuf,(const char *)&in_buf[116],8);
              tempbuf[8] = NULL;
              *trailsiz = atol(tempbuf);
              break;
      /* Get information about each of the other bands: records per band
         record size, and band number 
      ---------------------------------------------------------------------*/   
      case 5: strncpy(tempbuf,(const char *)&in_buf[116],8);
              tempbuf[8] = NULL;
              bandrecsiz[1] = atol(tempbuf);
              strncpy(tempbuf,(const char *)&in_buf[100],8);
              bandnumrec[1] = atol(tempbuf); 
              strncpy(bandnum[1],(const char *)&in_buf[35],1);
	      break;

      case 8: strncpy(tempbuf,(const char *)&in_buf[116],8); 
              tempbuf[8] = NULL;
              bandrecsiz[2] = atol(tempbuf);
              strncpy(tempbuf,(const char *)&in_buf[100],8);
              bandnumrec[2] = atol(tempbuf); 
              strncpy(bandnum[2],(const char *)&in_buf[35],1); 
              if (strcmp(bandnum[2],"6") ==0)
                *cmode = 2;
              else if (strcmp(bandnum[2],"5") ==0)
                *cmode = 3;
              break;

      case 11: strncpy(tempbuf,(const char *)&in_buf[116],8); 
              tempbuf[8] = NULL;
              bandrecsiz[3] = atol(tempbuf);
              strncpy(tempbuf,(const char *)&in_buf[100],8);
              bandnumrec[3] = atol(tempbuf); 
              strncpy(bandnum[3],(const char *)&in_buf[35],1);
              break;

      case 14: strncpy(tempbuf,(const char *)&in_buf[116],8); 
              tempbuf[8] = NULL;
              bandrecsiz[4] = atol(tempbuf);
              strncpy(tempbuf,(const char *)&in_buf[100],8);
              bandnumrec[4] = atol(tempbuf); 
              strncpy(bandnum[4],(const char *)&in_buf[35],1);
              break;

      case 17: strncpy(tempbuf,(const char *)&in_buf[116],8); 
              tempbuf[8] = NULL;
              bandrecsiz[5] = atol(tempbuf);
              strncpy(tempbuf,(const char *)&in_buf[100],8);
              bandnumrec[5] = atol(tempbuf); 
              strncpy(bandnum[5],(const char *)&in_buf[35],1);
              break;

      case 20:strncpy(tempbuf,(const char *)&in_buf[116],8); 
              tempbuf[8] = NULL;
              bandrecsiz[6] = atol(tempbuf);
              strncpy(tempbuf,(const char *)&in_buf[100],8);
              bandnumrec[6] = atol(tempbuf); 
              strncpy(bandnum[6],(const char *)&in_buf[35],1);
              break;

      default: break;
     }
   
    /* Write to disk
    ----------------*/
    if (*wrtdsk)
       fwrite(in_buf,1,numbytes,fp);
    }

/* Close the file
----------------*/
if (*wrtdsk)
   fclose(fp);

/* One more read for EOF
-----------------------*/
status = c_tpread(unit,in_buf,&numbytes,&mode);
if (status != TAPE_EOF) 
   {
   c_errmsg("Error reading from tape","ltwg2in-read",&one);
   return(E_FAIL);
   }

/*Free allocated space 
----------------------*/
free(in_buf);

return(E_SUCC);
}
