#include "las.h"
#ifndef PROTOS_H
#define PROTOS_H

void calcorn
(
    double clat_l, 	/* I: center point latitude */
    double clon_l,	/* I: center point longitude */
    char *rectec,	/* I: recording technique used */
    double *rtnlat,	/* O: calculated corner point latitudes */
    double *rtnlon,	/* O: calculated corner point longitudes */
    long sat		/* I: satellite number */
);

void chk_window
(
    long *outspc,   /* I: output image file specifications  */
    long *window    /* I/O: window area of image to be copied */
);

void chk_bands
(
    long *bands,	/* I/O: bands to be processed within image	     */
    long *numbands,	/* I: input image file specifications   	     */
    long *totbnd        /* I/O: total number of bands to be copied           */
);

int getenf
(
    char hosout[][CMLEN],	/* I/O: output host names 	      	     */
    long *unit,                 /* I/O: tape unit number                     */
    long *wrtdsk,               /* I: ancillary files to read in             */
    long *num_out,              /* I/O: number of output files 		     */
    char comment[][CMLEN],      /* I: tape mount message to the operator     */
    long *num_trailrec,	        /* I: number of trailer records		     */
    long *tapenum,		/* I: current tape mounted		     */
    long *trailsiz,	        /* I: record size of the trailer file	     */
    long *numtapes,	        /* I: number of tapes specified		     */
    long *no_lead,		/* I: flag if there is a leader file on tape    
			           there is not a leader file if an image 
                                   band is split across two tapes	     */
    char *satnam                /* I: satellite name                         */
);

int getlead
(
    long *unit,		   /* I: tape unit number			*/
    long *wrtdsk,	   /* I: ancillary files to read in		*/
    char hosout[][CMLEN],  /* I/O : Output host names                  	*/
    long *num_out, 	   /* I/O: number of output files             	*/
    long *numlead,	   /* I: number of leader records		*/
    long *leadsize,	   /* I: size of leader records			*/
    double fcenll[],       /* O: Full scene center latitude/longitude degrees */
    double fcenls[],       /* O: Full scene center line/sample          */
    long *path,		   /* O: scene path				*/
    char *direction,	   /* O: satellite direction			*/
    char *maprj,           /* O: map projection				*/
    double pixsiz[],	   /* O: image pixel size			*/
    long *satid,           /* O: satellite id.				*/
    char *sensor,	   /* O: satellite sensor			*/
    double *angle,	   /* O: angle of rotation in radians		*/
    long *utmzone,	   /* O: UTM zone code				*/
    char *satnam,          /* O: satellite name                         */
    char cornlat[][17],	   /* O: corner latitudes                       */
    char cornlong[][17]	   /* O: corner longitudes                      */
);

void getpar
(
    struct PARBLK *vblock,  /* I: TAE parameter block                       */
    long *bands,	    /* O: bands to be processed within image        */
    long *msgtim,	    /* O: elapsed time between display msg          */
    long *window,	    /* O: window of input image                     */
    long *minmax,	    /* O: min max flag                              */
    char hosout[][CMLEN],   /* O: Output host name          		    */
    char taeout[][CMLEN],   /* O: Output TAE name           		    */
    char comment[][CMLEN],  /* O: tape mount message to the operator        */
    long *wrtdsk,           /* O: ancillary files to read in                */
    long *numtapes	    /* O: number of tapes specified		    */
);

int getvdf
(
    long *unit,		/* I: tape unit number				*/
    long *wrtdsk,	/* I: ancillary files to read in 		*/
    char hosout[][CMLEN],/* I/O: Output host names                     	*/
    long *num_out, 	/* I/O: number of output files                	*/
    long *recsize,	/* O: image record length 			*/
    long *numimg_recs,	/* O: number of image records on this tape	*/
    long *num_trailrec,	/* O: number of trailer records			*/
    char *order,	/* O: band order				*/
    long *numlead,	/* O: number of leader files			*/
    long *leadsize,	/* O: size of leader records			*/
    long *trailsiz,	/* O: max size of trailer records		*/
    char bandnum[][1],  /* O: bandnum ascii array			*/
    long bandrecsiz[7], /* O: band record size				*/
    long bandnumrec[7], /* O: number of records per band		*/
    long *cmode		/* O: CEOS mode (1,2,3)				*/
);

void ls_proj
(
   double *angle,	/* I/O: Display rotation angle			   */
   double fcproj[],	/* I: Full scene center projection coords (y,x)	   */
   double fcenls[],	/* I: Full scene center line/sample		   */
   double proj[], 	/* O: Projection coordinates calculated (y,x)	   */
   long linsamp[],	/* I: Line/sample values calculated		   */
   double *pixsiz 	/* I: Pixel size in meters			   */
);

int proces_bsq_m1
(
    char hosout[][CMLEN],   /* I/O: output host names  		      	     */
    long *window,	    /* I: window area of image to be copied 	     */
    long *totbnd,	    /* O: total number of bands to be copied	     */
    long *unit,             /* I/O: tape unit number                         */
    long *wrtdsk,           /* I: ancillary files to read in                 */
    long *num_out,          /* I/O: number of output files 		     */
    long *bands,            /* I: bands to be processed within image         */
    long *msgtim,	    /* I: elapsed time between display msg  	     */
    long *minmax,	    /* I: min max flag                      	     */
    char comment[][CMLEN],  /* I: tape mount message to the operator         */
    long *recsize,          /* I: image record length                        */
    long *tapenum,	    /* I/O: tape number mounted 		     */
    long *numtapes,	    /* I: number of tapes specified 		     */
    long *no_lead,	    /* I/O: flag if successive tapes will have leader 
			            file */
    char ttype[],	    /* I/O: tape type				     */
    long *bandrecsiz,	    /* I: Band record sizes                          */ 
    long *bandnumrec	    /* I: Number of records per band                 */
);

int skip_bsq
(
    long *window,          /* I: window area of image to be copied 	      */
    long *unit,            /* I/O: tape unit number                           */
    char comment[][CMLEN], /* I: tape mount message to the operator           */
    long *tapenum,	   /* I/O: current tape number mounted		      */
    long *numtapes,	   /* I: number of tapes specified		      */
    long *cb,              /* I: current tape band being processed            */
    long *dmod,		   /* O: part of curblk that is needed for the window */
    long *blkfct,	   /* I: blocking factor                              */
    long *ptrband,	   /* I/O: pointer to which band the tape is
			           currently at                               */
    long *swreel,	   /* I/O: flag if need to switch reels               */
    long *bandrecsiz	   /* I: Band record sizes                            */
);

int mount_tape
(
    long *unit,		        /* O: tape unit number                 	      */
    char comment[][CMLEN],	/* I: tape mount message to the operator      */
    long tapenum,		/* I: tape number currently mounted           */
    long *numtapes,		/* I: number of tapes specified               */
    char ttype[]		/* O: tape type	                              */
);

void close_tape
(
    long *unit		/* I: tape unit number                  	      */
);

int update_ddr
(
   char hosout[][CMLEN],      /* I: Host file name       		 */
   long *window,	      /* I: Input window			 */
   long *totbnd,	      /* I: Total number of bands to process     */
   double fcenll[],           /* I: Full scene center latitude/longitude
				    degrees */
   double fcenls[],           /* I: Full scene center line/sample        */
   long *path,	              /* I: path number			         */
   char *direction,	      /* I: sat direction			 */
   char *maprj,		      /* I: map projectin			 */
   double pixsiz[],	      /* I: pixel size			         */
   long *satid,		      /* I: Sat ID			         */
   char *sensor,	      /* I: sensor type			         */
   double *angle,	      /* I: rotation angle			 */
   long *utmzone,	      /* I: UTM zone code			 */
   char *satnam,              /* I: satellite name                       */
   char  cornlat[][17],       /* I: corner latitudes                     */
   char  cornlong[][17]       /* I: corner longitudes                    */
);
#endif
