/******************************************************************************
FUNCTION:       SKIP_BSQ

PURPOSE:        Skip to the correct spot on BSQ tapes.


ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "ltwg2in.h"
#include "util.h"
#include "protos.h"

int skip_bsq
(
    long *window,          /* I: window area of image to be copied 	      */
    long *unit,            /* I/O: tape unit number                           */
    char comment[][CMLEN], /* I: tape mount message to the operator           */
    long *tapenum,	   /* I/O: current tape number mounted		      */
    long *numtapes,	   /* I: number of tapes specified		      */
    long *cb,              /* I: current tape band being processed            */
    long *dmod,		   /* O: part of curblk that is needed for the window */
    long *blkfct,	   /* I: blocking factor                              */
    long *ptrband,	   /* I/O: pointer to which band the tape is
			           currently at                               */
    long *swreel,	   /* I/O: flag if need to switch reels               */
    long *bandrecsiz	   /* I: Band record sizes                            */
)	

{
extern long curcat;     /* curent tape band being processed          */
extern long curent;     /* output line being processed               */
extern long gdesc;	/* group descriptor used for image I/O       */
extern long offset[];	/* offsets into "thebuf" for output bands    */
extern int  fdesc[];	/* image file descriptors used for image I/O */

  
unsigned char *in_buff; /* buffer read from image on tape    */ 
long  nrec=0;		/* number of records to skip		     */
long  recleng=0;          /* record length of image descriptor record  */
long  recsiz=0;		/* record size				     */
long  mode = HANDLE;    /* tape I/O error flag			     */
long  one = 1;		/* constant one			             */
long  status=0;		/* return status                             */
long  skip=0;		/* number of files to skip		     */
long  two=2;		/* constant of two			     */
long  lclblk=0;		/* local block size			     */

char ttype[CMLEN];      /* tape type				     */ 
 
*dmod = 0;		/* modulus offset			     */

/* Switch tapes if needed.  Set the pointer band.
-------------------------------------------------*/
if (*swreel)
   {
/*   if (*cb <= 6) */
   if (*cb <=7) 
      *ptrband = 5;
   else
      *ptrband = 7;
   ++(*tapenum);
   c_stoptm();
   close_tape(unit);
   if (mount_tape(unit, comment, *tapenum, numtapes,ttype) != E_SUCC)
      return(E_FAIL);

   /* Skip the volume directory and leader files
   ---------------------------------------------*/
   status = c_tpfsf(unit,&two,&mode);
   if (status != TAPE_SUCC)
       return(E_FAIL);
   *swreel = FALSE;
   c_startm();
   }

/* Skip to the correct band on tape 
-----------------------------------*/
skip = (*cb - *ptrband) * 3;
status = c_tpfsf(unit,&skip,&mode);
if (status != TAPE_SUCC)
   return(E_FAIL);

/* Skip the file descriptor record for the first band, otherwise
   determine the blking factor for skipping, if needed.  The Thermal
   band will have different values than the other bands
--------------------------------------------------------------------*/
if (*cb != 1)
   {
   recsiz = bandrecsiz[*cb-1];
   in_buff = (unsigned char *) malloc(recsiz);
   if (in_buff == NULL)
     {
     c_errmsg("Error allocating dynamic memory","skip_bsq-alloc",&one);
     return(E_FAIL);
     }
   status = c_tpread(unit,in_buff,&recsiz,&mode);
   recleng = atol((const char *)&in_buff[186]);
   if (status != recleng)
     return(E_FAIL);

   lclblk = recsiz/recleng;
   free(in_buff);
  /* status = c_tpfsr(unit,&one,&mode);
   if (status != TAPE_SUCC)
      return(E_FAIL); */
   }
else
   {
   lclblk = *blkfct;
   } 

/* Skip records on tape to starting line of the window.
------------------------------------------------------*/
if (window[SL] > 1)
   {
   if (*cb != 6)
     {
     nrec = (window[SL] - 1) / lclblk;
     *dmod = (window[SL] - 1) % lclblk;
     }
   else
     {
     nrec = (window[SL]/4 - 1) / lclblk; 
     *dmod =(window[SL]/4 - 1) % lclblk;
     }

   status = c_tpfsr(unit,&nrec,&mode);
   if (status != TAPE_SUCC)
      return(E_FAIL);
   }

/* Free allocated space
-----------------------*/
return(E_SUCC);
}
