#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "string.h"
#include "protos.h"

/******************************************************************************
FUNCTION:	MOUNT_TAPE

PURPOSE:	Physically mounts a tape.  

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/

int mount_tape
(
    long *unit,		        /* O: tape unit number                 	      */
    char comment[][CMLEN],	/* I: tape mount message to the operator      */
    long tapenum,		/* I: tape number currently mounted           */
    long *numtapes,		/* I: number of tapes specified               */
    char ttype[]		/* O: tape type	                              */
)

{
char tdens[CMLEN];	/* tape density					*/
struct TAPEDESC *tdrive;

long access = TREAD;	/* open tape for read access         	      */
long mode = HANDLE;	/* tape I/O outputs error messages   	      */
long rewopt = REWIND;	/* rewind option                     	      */
long skip = 0;		/* skip zero files on open           	      */
long status=0;		/* function return status code       	      */
long one=1;		/* constant of one			      */

char density[5];	/* tape density in BPI               	      */
char tapeid[CMLEN]; 	/* tape id			     	      */

strcpy(density,"0");

if (tapenum > *numtapes)
   {
   c_errmsg("Invalid number of tapes specified","ltwg2in-count",&one);
   return(E_FAIL);
   }

/* Mount the current tape volume
--------------------------------*/
status = c_tpopen(unit,&skip,&mode,&rewopt,density,&access, 
                  comment[tapenum - 1], tapeid); 
if (status != TAPE_SUCC)
   return(E_FAIL);

tdrive = (struct TAPEDESC *) *unit;
strcpy(ttype,tdrive->dtype);
strcpy(tdens,tdrive->density);
return(E_SUCC);
}

/******************************************************************************
FUNCTION:	CLOSE_TAPE

PURPOSE:	Closes the tape device.

ALGORITHM 
  Close the tape

ALGORITHM REFERENCES	none
*******************************************************************************/
void close_tape
(
    long *unit		/* I: tape unit number                  	      */
)

{
long one = 1;		/* value of one                      	      */
long mode = NO_HANDLE;	/* tape I/O outputs error messages   	      */
long rewopt = REWOFFL;	/* rewind option                     	      */
long status;		/* function return status code       	      */

/* Close the tape
-----------------*/
status = c_tpclos(unit,&mode,&rewopt,&one);
if (status != TAPE_SUCC)
   c_errmsg("Nonfatal error encountered","ltwg2in-nonfatal",&one);

return;
}
