/*******************************************************************************
FUNCTION:	UPDATE_DDR

PURPOSE:	Updates the DDR information for an image


ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "proj.h"
#include "tm.h"
#include "protos.h"

int update_ddr
(
   char hosout[][CMLEN],      /* I: Host file name       		 */
   long *window,	      /* I: Input window			 */
   long *totbnd,	      /* I: Total number of bands to process     */
   double fcenll[],           /* I: Full scene center latitude/longitude
				    degrees */
   double fcenls[],           /* I: Full scene center line/sample        */
   long *path,	              /* I: path number			         */
   char *direction,	      /* I: sat direction			 */
   char *maprj,		      /* I: map projectin			 */
   double pixsiz[],	      /* I: pixel size			         */
   long *satid,		      /* I: Sat ID			         */
   char *sensor,	      /* I: sensor type			         */
   double *angle,	      /* I: rotation angle			 */
   long *utmzone,	      /* I: UTM zone code			 */
   char *satnam,              /* I: satellite name                       */
   char  cornlat[][17],       /* I: corner latitudes                     */
   char  cornlong[][17]       /* I: corner longitudes                    */
)
{
struct DDR ddr;		/* DDR structure			*/
struct BDDR bddr[7];	/* Band DDR structure			*/

long band=0;		/* Band looping variable		*/
long inunit = 4;	/* input unit - deg			*/
long outunit = 2;	/* output unit - meters			*/
long inzone = 62;	/* input zone				*/
long outzone = 62;	/* output zone				*/
long indatum = 0;	/* input datum				*/
long outdatum = 0; 	/* output datum				*/
long line_samp[2];	/* line/sample pair			*/
long i=0;			/* loop counter				*/
long inproj=0;		/* input projection			*/
long prtprm[2];         /* report to term and/or file           */
long one = 1;		/* constant of one			*/
long multfact=0;		/* sign value				*/

double inparm[15];      /* input projection parameters          */
double fcproj[2];       /* Full scene center degrees 		*/
double deglat[4];	/* corner corndinates in degrees        */
double deglong[4];	/* corner corndinates in degrees        */
double minutes;		/* minutes				*/
double seconds;		/* seconds				*/

char ctemp[5];          /* temp buffer                          */


/* Get the DDR information from the image file
----------------------------------------------*/
if (c_getddr(hosout[0],&ddr) != E_SUCC)
    return(E_FAIL);

/* Calculate the corner coordinates for these projections
---------------------------------------------------------*/
if ((strncmp(maprj,"SOM",3) == 0) || (strncmp(maprj,"UTM",3) == 0))
   {
   inproj = 0;
   prtprm[0] = 0;
   prtprm[1] = 0;

   for (i = 0; i < 15; i++)
           inparm[i] = 0.0;

   if (strncmp(maprj,"SOM",3) == 0)
      {
      ddr.proj_code = SOM;
      ddr.zone_code = 62;
      if (somprm(path,ddr.proj_coef) != E_SUCC)
          return(E_FAIL);
      }
   else
      {
      ddr.proj_code = UTM;
      ddr.zone_code = *utmzone;
      if (utmprm(utmzone,&fcenll[0],ddr.proj_coef) != E_SUCC)
          return(E_FAIL);
      inzone = *utmzone;
      outzone = *utmzone;
      }

   /* Initialize the Projection Transformation Package
    --------------------------------------------------*/
   if (c_transinit(&inproj,&inunit,&inzone,&indatum,inparm,&ddr.proj_code,&outunit,
                &outzone,&outdatum,ddr.proj_coef,prtprm,"") != E_SUCC)
         return(E_FAIL);

   /* Calculate the center projection coordinates
   ----------------------------------------------*/
   if (c_trans(&inproj,&inunit,&ddr.proj_code,&outunit,
             &fcenll[1],&fcenll[0],&fcproj[1],&fcproj[0]) 
             != E_SUCC)
         return(E_FAIL);

   /* Find the projection coordinates for the corners
   --------------------------------------------------*/
   line_samp[0] = window[SL];
   line_samp[1] = window[SS];
   ls_proj(angle,fcproj,fcenls,ddr.upleft,line_samp,pixsiz);

   line_samp[0] = window[SL] + window[NL] - 1;
   line_samp[1] = window[SS];
   ls_proj(angle,fcproj,fcenls,ddr.loleft,line_samp,pixsiz);

   line_samp[0] = window[SL];
   line_samp[1] = window[SS] + window[NS] - 1;
   ls_proj(angle,fcproj,fcenls,ddr.upright,line_samp,pixsiz);

   line_samp[0] = window[SL] + window[NL] - 1;
   line_samp[1] = window[SS] + window[NS] - 1;
   ls_proj(angle,fcproj,fcenls,ddr.loright,line_samp,pixsiz);

   /* Update the values that are known
   -----------------------------------*/
   ddr.master_line   = window[SL] - 1;
   ddr.master_sample = window[SS] - 1;
   ddr.line_inc   = 1.0;
   ddr.sample_inc = 1.0;
   ddr.pdist_y = pixsiz[0];
   ddr.pdist_x = pixsiz[1];
   strcpy(ddr.proj_units,"meters");

   /* Initialize the validity flags
   --------------------------------*/
   for (i = 0; i < 8; i++)
       ddr.valid[i] = VALID;
   }
else if ((strncmp(maprj,"NNN",3) == 0) || (strncmp(maprj,"YNN",3)==0))
   {
   /* No map projection, not UTM, and not using the reserved field
   ---------------------------------------------------------------*/
   c_errmsg("Corner coordinates will be approximate","ltwg2in-corn",&one);

   /* calcorn will give us the approximate corners
   ---------------------------------------------------*/
   strncpy(ctemp,direction,1);
   ctemp[1] = NULL;
   calcorn (fcenll[0], fcenll[1], ctemp, deglat, deglong, *satid);

   ddr.zone_code = inzone;
   ddr.datum_code = indatum;
   ddr.upleft[0] = deglat[0];
   ddr.upleft[1] = deglong[0];
   ddr.upright[0] = deglat[3];
   ddr.upright[1] = deglong[3];
   ddr.loright[0] = deglat[2];
   ddr.loright[1] = deglong[2];
   ddr.loleft[0] = deglat[1];
   ddr.loleft[1] = deglong[1];
   strcpy(ddr.proj_units,"degrees");

   /* Update some values 
   ---------------------*/
   ddr.master_line   = 0;
   ddr.master_sample = 0;
   ddr.line_inc   = 1.0;
   ddr.sample_inc = 1.0;

   /* Approximate pixel size in degrees 
   ------------------------------------*/
   ddr.pdist_y = abs(ddr.upleft[0] - ddr.loleft[0]) / window[NL];
   ddr.pdist_x = ddr.pdist_y;

   /* Initialize the validity flags
   --------------------------------*/
   for (i = 0; i < 8; i++)
       ddr.valid[i] = VALID;
   }
else if (strncmp(satnam,"SP",2) == 0)
   {
   c_errmsg("Corner coordinates will be approximate","ltwg2in-corn",&one);
 
   /* Set the corners for SPOT Levels 1A and 1B. We have the four
      corners in a character string in DDDMMSS.  Level 2 is registered 
      to a map but we don't have level2 data.  First convert 
      coordinates to total degrees.
   --------------------------------------------------------------*/
   for (i = 0; i < 4; i++)
       {
       if (cornlat[i][0] == 'N')
          multfact = 1;
       else
          multfact = -1;

       strncpy(ctemp,&cornlat[i][1],3);
       ctemp[3] = NULL;
       deglat[i] = (double)atol(ctemp);
       strncpy(ctemp,&cornlat[i][4],2);
       ctemp[2] = NULL;
       minutes = (double)atol(ctemp);
       strncpy(ctemp,&cornlat[i][6],2);
       ctemp[2] = NULL;
       seconds = (double)atol(ctemp);

       deglat[i] = (deglat[i] + (minutes/60) + (seconds/3600)) * multfact;

       if (cornlat[i][0] == 'E')
          multfact = 1;
       else
          multfact = -1;

       strncpy(ctemp,&cornlong[i][1],3);
       ctemp[3] = NULL;
       deglong[i] = (double)atol(ctemp);
       strncpy(ctemp,&cornlong[i][4],2);
       ctemp[2] = NULL;
       minutes = (double)atol(ctemp);
       strncpy(ctemp,&cornlong[i][6],2);
       ctemp[2] = NULL;
       seconds = (double)atol(ctemp);

       deglong[i] = (deglong[i] + (minutes/60) + (seconds/3600)) * multfact;
       }

   /* Fill in DDR fields
   ---------------------*/
   ddr.zone_code = inzone;
   ddr.datum_code = indatum;
   ddr.upleft[0] = deglat[0];
   ddr.upleft[1] = deglong[0];
   ddr.upright[0] = deglat[1];
   ddr.upright[1] = deglong[1];
   ddr.loright[0] = deglat[3];
   ddr.loright[1] = deglong[3];
   ddr.loleft[0] = deglat[2];
   ddr.loleft[1] = deglong[2];
   strcpy(ddr.proj_units,"degrees");

   /* Update some values 
   ---------------------*/
   ddr.master_line   = 0;
   ddr.master_sample = 0;
   ddr.line_inc   = 1.0;
   ddr.sample_inc = 1.0;

   /* Approximate pixel size in degrees 
   ------------------------------------*/
   ddr.pdist_y = abs(ddr.upleft[0] - ddr.loleft[0]) / window[NL];
   ddr.pdist_x = ddr.pdist_y;

   /* Initialize the validity flags
   --------------------------------*/
   for (i = 0; i < 8; i++)
       ddr.valid[i] = VALID;
   }
else
   c_errmsg("Unable to determine projection information","ltwg2in-proj",&one);

/* Update the Band DDR information
----------------------------------*/
for (band = 1; band <= *totbnd; band++)
    {
    /* Get the Band DDR information from the image
    ----------------------------------------------- */
    if (c_getbdr(hosout[0],&bddr[band-1],&band) != E_SUCC)
       return(E_FAIL);

    /* Update the Band DDR information
    ----------------------------------*/
    if (strncmp(satnam,"LS",2) == 0)
       {
       sprintf(bddr[band-1].source,"Landsat %d",*satid);
       strcpy(bddr[band-1].instrument,sensor);

       /* Update the direction of the pass
       -----------------------------------*/
       if (strncmp(direction,"D",1) == 0)
           strcpy(bddr[band-1].direction,"Descending");
       else
           strcpy(bddr[band-1].direction,"Ascending");
       }
    else if (strncmp(satnam,"SP",2) == 0)
       sprintf(bddr[band-1].source,"SPOT %d",*satid);
    }

/* Put the updated DDR information to the image
-----------------------------------------------*/
if (c_putddr(hosout[0],&ddr) != E_SUCC)
   return(E_FAIL);

/* Put the updated Band DDR information to the image
----------------------------------------------------*/
for (band = 0; band < *totbnd; band++)
    if (c_putbdr((char *)hosout,&bddr[band]) != E_SUCC)
       return(E_FAIL);

return(E_SUCC);
}
