/******************************************************************************
FUNCTION:       CDSKIP_BSQ

PURPOSE:        Skip to the correct spot in disk file (using standard C file
                I/O functions)

ALGORITHM REFERENCES    none
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ltwgin.h"

int cdskip_bsq
(
  long *window,                /* Window of image to be copied */
  long *cb,                    /* Current band being processed */
  long *dmod,                  /* Part of curblk needed for the window */
  long *blkfct,                /* Blocking factor */
  FILE *infp,                  /* Input file */
  long reclength,              /* Length of image descrip. record */
  long *recsize,               /* Length of image data records */
  char hpath[CMLEN],           /* Path to image data on disk */
  char filename[][CMLEN],      /* I: filenames on disk or CD */
  char sceneid[],              /* I: naming convention for NLAPS files */
  int nfiles,                  /* I: number of filenames on disk or CD */
  int dir_struct               /* I: flag indicating which directory
                                     structure is used for data */
)
{
    char band[3];           /* character string representing band number */
    char filename_up[MAX_FILES][CMLEN];
                            /* names of files on disk or CD (upper-case) */
    char new_filename[25];  /* name of new image file to open */
    char imgfile[CMLEN];    /* full path to image file on disk */
    int  i;                 /* looping counter */
    long filename_len;      /* length of filename string */
    long nrec;              /* number of records to skip */
    long status;            /* holds result of fseek(); */

    *dmod = 0;

    /* Create the proper filename to open. BSQ image files on CD use the
       filename convention img_xx.001, where xx is the band number the data
       represents, i.e., if current band == 2, the corresponding filename is
       img_02.001. TM data uses the file naming convention of DAT_xx.001.
       Also, NLAPS uses sceneid.ix 
   -------------------------------------------------------------------------*/

    /* Convert filenames on disk or CD to all upper-case for comparisons
    -------------------------------------------------------------------*/
    for(i = 0; i < nfiles; i++)
    {
        strcpy(filename_up[i], filename[i]);
        filename_len = (long)strlen(filename_up[i]);
        c_low2up(filename_up[i], &filename_len);
    }

    /* Create the new_filename string. Look through the array with the
       filenames in upper-case, then append the filename at the same array
       index from the case-sensitive array.
    ---------------------------------------------------------------------*/
    for(i = 0; i < nfiles; i++)
    {
        if (dir_struct == NLAPS)
        {
            if(strncmp(filename_up[i] + SCENELEN, ".I", 2) == 0)
            {
                strncpy(new_filename, sceneid, SCENELEN);
                new_filename[SCENELEN] = '\0';
                sprintf(band, "%1ld", *cb);
                strncat(new_filename, filename[i] + SCENELEN, 2); /* add ".i" */
                strcat(new_filename, band); 
                break;
            }
        }
        else if((dir_struct == ESATM) || (dir_struct == SAR))
        {
            if(strncmp(filename_up[i], "DAT_", 4) == 0)
            {
                sprintf(band, "%02ld", *cb);
                strncpy(new_filename, filename[i], 4); /* add "DAT_" */
                new_filename[4] = '\0';
                strcat(new_filename, band); /* add the band number */
                strcat(new_filename, ".001");
                break;
            }
        }
        else
        {
            if(strncmp(filename_up[i], "IMG_", 4) == 0)
            {
                sprintf(band, "%02ld", *cb);
                strncpy(new_filename, filename[i], 4); /* add "IMG_" */
                new_filename[4] = '\0';
                strcat(new_filename, band); /* add the band number */
                strcat(new_filename, ".001");
                break;
            }
        }
    }

    strcpy(imgfile, hpath);
    strcat(imgfile, new_filename);

    /* Open the image file
    ---------------------*/
    infp = fopen(imgfile, "r");
    if (infp == NULL)
        return (E_FAIL);

    /* Skip the image descriptor record
    ----------------------------------*/
    status = fseek(infp, reclength, 0); /* can't use ANSI C #defines here */ 
    if (status != 0)
        return(E_FAIL);

    /* Skip records to starting line of the specified window
    -------------------------------------------------------*/
    if (window[SL] > 1)
    {
        nrec = (window[SL] - 1) / *blkfct;
        *dmod = (window[SL] - 1) % *blkfct;
        status = fseek(infp, nrec * (*recsize), 1);
        if (status != 0)
            return(E_FAIL);
    }

    return(E_SUCC);
}
