/******************************************************************************
FUNCTION:       CHECK_WINDOW

PURPOSE:        Checks the validity of the window specification.  If an
                error is detected, the window specification is changed
                to be the intersection of the window specification and
                the actual image.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltwgin.h"

void chk_window
(
  long *outspc,                /* Output image file specs */
  long *window                 /* Window of image to be copied */
)
{

    /* If NL or NS is 0 then set them to the total number of
       lines and samples
    --------------------------------------------------------*/
    if (window[NL] == 0)
        window[NL] = outspc[0];

    if (window[NS] == 0)
        window[NS] = outspc[1];

    /* If SL or SS is 0 then set them to start at 1,1 
    -------------------------------------------------*/
    if (window[SL] == 0)
        window[SL] = 1;

    if (window[SS] == 0)
        window[SS] = 1;

    /* Validate the number of lines specified
    -----------------------------------------*/
    if (window[SL] + window[NL] - 1 > outspc[0])
    {
        c_lasmsg(FALSE, "ltwgin-window", "Specified window is too large");
        window[NL] = outspc[0] - window[SL] + 1;
    }

    /* Validate the specified number of samples
    -------------------------------------------*/
    if (window[SS] + window[NS] - 1 > outspc[1])
    {
        c_lasmsg(FALSE, "ltwgin-window", "Specified window is too large");
        window[NS] = outspc[1] - window[SS] + 1;
    }
}

/******************************************************************************
FUNCTION:       CHECK_BAND

PURPOSE:        Checks the validity of the band specification.  If an error is
                detected in the band specification, the program continues by
                not copying the band that is in error.  The routine also sets
                up a boolean array indicating which bands are to be copied.
                This array is returned to the calling program.

ALGORITHM REFERENCES    none
*******************************************************************************/

void chk_bands
(
  long *bands,                /* Bands to be processed within image */
  long *numbands,             /* Input image file specs */
  long *totbnd                /* Total number of bands to copy */
)
{
    long  index;        /* loop index                                        */


    /* For each band specified, error check to ensure the band exists
    -----------------------------------------------------------------*/
    for (index = 0; index < *numbands; index++) 
    {
        if (bands[index])
        {
            if ((index + 1) > *numbands)
            {
                c_lasmsg(FALSE, "ltwgin-band",
                        "Band %d does not exist in image", index+1);
                bands[index] = FALSE;
            }
            else
                *totbnd +=1;
        }
    }
}
