/******************************************************************************

FUNCTION:       GETENF

PURPOSE:        Reads in the end files (trailer and supplemental) from media 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

ALGORITHM REFERENCES
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "labserv.h"
#include "ltwgin.h"

int getenf
(
   char hosout[][CMLEN],         /* I/O: Ouput host names */
   char filename[][CMLEN],       /* I: Names of files on disk or CD */
   int nfiles,                   /* I: Number of filenames on disk or CD */
   int dir_struct,               /* I: Flag indicating which directory
                                       structure is used for data */
   int tapeflag,               /* I: flag indicating media type processing from
                                         TRUE: tape
                                         FALSE: other, including CD, disk */
   long *unit,                   /* I: Tape unit number */
   long *wrtdsk,                 /* I: Ancillary files to read in */
   long *num_out,                /* I/O: Number of output files */
   char comment[][CMLEN],        /* I: Mount message to the operator */
   long *num_trailrec,           /* I: Number of trailer records */
   long *tapenum,                /* I: Tape volume currently mounted */
   long *trailsiz,               /* I: Record size of the trailer file */
   long *media_cnt,              /* I: Number of media specified */
   long *no_lead,                /* I: Flag if there is a leader file on tape
                                       There is not a leader file if an
                                       image band is split across two tapes */
   char *satnam,                 /* I: Satellite name */
   char hpath[CMLEN],            /* I: Host name of input file/directory */
   char sceneid[],               /* I: Naming convention for NLAPS files */
   long ceosid                   /* I: Ground station identifier */
)
{
    FILE *fp=NULL;          /* file descriptor                           */
    FILE *infp=NULL;        /* input file for CD or disk                 */

    long  filename_len;     /* length of filename string */
    long  i;                /* loop counter                              */
    long  dumlong;          /* dummy value                               */
    long  dum2 = FALSE;     /* dummy value                               */
    long  mode = HANDLE;    /* tape I/O error flag                       */
    long  numrecs;          /* number of records in the file             */
    long  one = 1;          /* constant of one                           */
    long  recsize;          /* record size of the file                   */
    long  status;           /* return status                             */
    long  skip;             /* number of files to skip                   */
    long  tmp_len;          /* length of scene number string */

    int rec_num;            /* trailer record currently retrieving */
    int rec_sizes[17];      /* trailer record sizes - these are variable
                               in the SAR format */
    int supp_nfiles;        /* number of filenames returned              */
    int tmp;                /* temp value to pass to mount_media so tapeflag
                               won't get changed during processing somehow */

    char filename_up[MAX_FILES][CMLEN];
                            /* names of files on disk or CD (upper-case) */
    char *in_buf = NULL;    /* buffer read from LGSOWG image */
    char tempbuf[10];       /* temp work buffer                          */
    char ttype[CMLEN];      /* tape type                                 */
    char trafile[CMLEN];    /* formal name of trailer file (on disk)     */
    char scenenum[3];       /* string representation of image number     */
    char tpath[CMLEN];      /* temporary host file path for trailer file */ 
    char supp_filename[10][CMLEN]; /* array of filenames returned */ 
    char supfile[CMLEN];    /* supplemental filename                     */

    /* Mount the last tape if not already mounted.  Some tape sets 
       have these files at the end of each tape, but it is not consistent.
    ----------------------------------------------------------------------*/
    if (tapeflag)
    {
        if (*tapenum != *media_cnt)
        {
            close_media(unit);
            if (mount_media(unit, comment, *media_cnt, media_cnt, ttype, &tmp)
                            != E_SUCC)
                return(E_FAIL);

            /* Read the volume descriptor file
            --------------------------------*/
            if (getvdf(unit,&dum2,filename,nfiles,tapeflag,dir_struct,hpath,
                       sceneid,hosout,&dumlong,&dumlong,&dumlong,&dumlong,
                       tempbuf,&dumlong,&dumlong,&dumlong,&ceosid) != E_SUCC)
                return(E_FAIL);

            /* Skip the leader file and/or the image date portion 
            ----------------------------------------------------*/
            if (*no_lead)
                skip = 1;
            else
                skip = 2;
            if (c_tpfsf(unit,&skip,&mode) != TAPE_SUCC)
                return(E_FAIL);
        }
    }

    /* Write the trailer file to disk
    ----------------------------------*/
    if (wrtdsk[TRAI])
    {
        /* Create the file name
        -----------------------*/
        c_lsmknm(hosout[0],"trai",hosout[*num_out]);
        fp = fopen(hosout[(*num_out)++],"w");
        if (fp == NULL)
        {
            c_lasmsg(FALSE, "ltwgin-write", 
                    "Error opening trailer file for writing");
            return(E_FAIL);
        }

        /* If input image is on CD, open the trailer file
        -------------------------------------------------*/
        if (!tapeflag)
        {
            /* Convert filenames on disk or CD to all upper-case for
               comparisons
            --------------------------------------------------------*/
            for(i = 0; i < nfiles; i++)
            {
                strcpy(filename_up[i], filename[i]);
                filename_len = (long)strlen(filename_up[i]);
                c_low2up(filename_up[i], &filename_len);
            }

            strcpy(trafile, hpath);

            /* Append the trailer filename to the trafile string. Look through
               the array with the filenames in upper-case, then append the
               filename at the same array index from the case-sensitive array.
            ------------------------------------------------------------------*/
            for(i = 0; i < nfiles; i++)
            {
                if (dir_struct == SPOTBIL)
                {
                    if(strncmp(filename_up[i], "TRAI_", 5) == 0)
                    {
                        if(imgnum(hpath, scenenum) != E_SUCC)
                        {
                            c_lasmsg(FALSE, "ltwgin-imgnum", 
                                    "Error retrieving the image"
                                    " number from CD.");
                            return(E_FAIL);
                        }
                        strncat(trafile, filename[i], 5); /* add "trai_" */
                        strcat(trafile, scenenum); /* add the scene number */
                        tmp_len = (long)strlen(scenenum);
                        strncat(trafile, filename[i] + 5 + tmp_len,
                                4); /* add ".dat" */
                    }
                }
                else if (dir_struct == NLAPS)
                {
                    strcpy(tpath,hpath);
                    strcat(tpath,"*.t*");
                    if(c_getfiles(tpath,supp_filename,&supp_nfiles,10,1,10)
                            != E_SUCC)
                    {
                        c_errmsg("Error retrieving filenames from disk",
                                "ltwgin-getfiles", &one);
                        return(E_FAIL);
                    }
                    if (supp_nfiles > 0)
                        strcpy(trafile, supp_filename[0]);
                    else /* try upper-case */
                    {
                        strcpy(tpath,hpath);
                        strcat(tpath,"*.T*");
                        if(c_getfiles(tpath,supp_filename,&supp_nfiles,10,1,10)
                                != E_SUCC)
                        {
                            c_errmsg("Error retrieving filenames from disk",
                                    "ltwgin-getfiles", &one);
                            return(E_FAIL);
                        }
                        if (supp_nfiles > 0)
                            strcpy(trafile, supp_filename[0]);
                    }
                }
                else if((dir_struct == SAFRICAMSS) || (dir_struct == ESATM) ||
                        (dir_struct == SAR))
                {
                    if(strncmp(filename_up[i], "TRA_01.001", 10) == 0)
                    {
                        strcat(trafile, filename[i]);
                        break;
                    }
                }
            }

            infp = fopen(trafile, "r");
            if (infp == NULL)
            {
                c_lasmsg(FALSE, "ltwgin-diskread", 
                        "Error reading trailer file from disk");
                return(E_FAIL);
            }
        }

        /* Read and write the records 
        -----------------------------*/
        for (i = 0; i < *num_trailrec; i++)
        {
            /* Allocate memory for the input buffer
            ---------------------------------------*/
            if(in_buf != NULL)
            {
                free(in_buf);
                in_buf = NULL;
            }
            in_buf = (char *) malloc(*trailsiz);
            if (in_buf == NULL)
            {
                c_lasmsg(FALSE, "ltwgin-alloc",
                         "Error allocating dynamic memory");
                return(E_FAIL);
            }

            if (!tapeflag)
            {
                if (fread(in_buf, 1, *trailsiz, infp) != *trailsiz)
                {
                    c_lasmsg(FALSE, "ltwgin-diskread",
                            "Error reading trailer file from disk");
                    free(in_buf);
                    return(E_FAIL);
                }
            } 
            else
            {
                if(c_tpread(unit,(unsigned char *)in_buf,trailsiz,&mode)
                        != *trailsiz)
                {
                    c_lasmsg(FALSE, "ltwgin-read",
                            "Error reading trailer file from tape");
                    free(in_buf);
                    return(E_FAIL);
                }
            }

            /* Get trailer record sizes if SAR format since they are of
               variable length. First check if the record is present in the
               trailer file (in_buf[xxx] != '0'), and if it is present,
               retrieve the record length from the appropriate location in
               the file descriptor record
            ---------------------------------------------------------------*/
            if((i == 0) && (ceosid == RSAT))
            {
                rec_num = 1; /* retrieving record lengths for
                                records 1...num_trailrec since the file
                                descriptor record is record 0 - this way the
                                rec_num matches up with the i looping
                                variable */

                /* Retrieve size of Data Set Summary record, if present */
                if(in_buf[185] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[186]);
                    rec_num++;
                }
                /* Retrieve size of Map Projection Data record, if present */
                if(in_buf[197] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[198]);
                    rec_num++;
                }
                /* Retrieve size of Platform Position Data record, if present */
                if(in_buf[209] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[210]);
                    rec_num++;
                }
                /* Retrieve size of Attitude Data record, if present */
                if(in_buf[221] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[222]);
                    rec_num++;
                }
                /* Retrieve size of Radiometric Data record, if present */
                if(in_buf[233] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[234]);
                    rec_num++;
                }
                /* Retrieve size of Radiometric Compensation record, if
                   present */
                if(in_buf[245] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[246]);
                    rec_num++;
                }
                /* Retrieve size of Data Quality Summary record, if present */
                if(in_buf[257] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[258]);
                    rec_num++;
                }
                /* Retrieve size of Data Histograms record, if present.
                   If not 0, then there are 2 records. */
                if(in_buf[269] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[270]);
                    rec_num++;
                    /* Do again for second data histogram record */
                    rec_sizes[rec_num] = atol((char *)&in_buf[270]);
                    rec_num++;
                }
                /* Retrieve size of Range Spectra record, if present */
                if(in_buf[281] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[282]);
                    rec_num++;
                }
                /* Retrieve size of Digital Elevation Model Descriptor record,
                   if present */
                if(in_buf[293] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[294]);
                    rec_num++;
                }
                /* Retrieve size of Radar Parameter Data Update record,
                   if present */
                if(in_buf[305] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[306]);
                    rec_num++;
                }
                /* Retrieve size of Annotation Data record, if present */
                if(in_buf[317] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[318]);
                    rec_num++;
                }
                /* Retrieve size of Detailed Processing Parameters record,
                   if present */
                if(in_buf[329] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[330]);
                    rec_num++;
                }
                /* Retrieve size of Calibration Data record, if present */
                if(in_buf[341] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[342]);
                    rec_num++;
                }
                /* Retrieve size of Ground Control Points record, if present */
                if(in_buf[353] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[354]);
                    rec_num++;
                }
                /* Retrieve size of Facility Related Data record, if present */
                if(in_buf[425] != '0')
                {
                    rec_sizes[rec_num] = atol((char *)&in_buf[426]);
                    rec_num++;
                }

                /* Make sure the number of records found to be present
                   here matches the number of records as reported by the
                   VDF file leader file pointer record
                --------------------------------------------------------*/
                if(rec_num != *num_trailrec)
                {
                    c_lasmsg(FALSE, "ltwgin-numtrai",
                            "Number of records doesn't match that reported by"
                            " the VDF file");
                    free(in_buf);
                    return(E_FAIL);
                }

                /* Reset the record number to 1 */
                rec_num = 1;
            }

            if(fwrite(in_buf,1,*trailsiz,fp) != *trailsiz)
            {
                c_lasmsg(FALSE,"ltwgin-write",
                         "Error writing to the trialer file");
                free(in_buf);
                return(E_FAIL);
            }

            /* Update the trailer record size if SAR format
            ------------------------------------------------*/
            if((i < *num_trailrec) && (ceosid == RSAT))
            {
                *trailsiz = rec_sizes[rec_num];
                rec_num++;
            }
        }

        /* Close the file
        -----------------*/
        if(fclose(fp) != 0)
        {
            c_lasmsg(FALSE,"ltwgin-write","Error closing the trailer file");
            free(in_buf);
            return(E_FAIL);
        }

        /* One more read to read EOF
        ----------------------------*/
        if (tapeflag)
        { 
            if (c_tpread(unit,(unsigned char *)in_buf,trailsiz,&mode)
                != TAPE_EOF)
            {
                c_lasmsg(FALSE, "ltwgin-read",
                        "Error reading trailer file from tape");
                free(in_buf);
                return(E_FAIL);
            }
        }

        /* Free allocated space and close infp if needed
        ------------------------------------------------*/
        free(in_buf);
        if (!tapeflag)
            fclose(infp);
    }
    else
    {
        /* Skip the file completely
        ---------------------------*/
        if (tapeflag)
        {
            if (c_tpfsf(unit,&one,&mode) != TAPE_SUCC)
            {
                free(in_buf);
                return(E_FAIL);
            }
        }
    }

    /* Spot data does not have a supplemental file
    -----------------------------------------------*/
    if (wrtdsk[SUPP] && (strncmp(satnam,"SP",2) == 0))
    {
        c_lasmsg(FALSE, "ltwgin-supp", "No supplemental file");
        return(E_SUCC);
    }
    else if (strncmp(satnam,"SP",2) == 0)
        return(E_SUCC);

    /* Japan, South African CD, Canada, and SAR data have no supplemental file
    --------------------------------------------------------------------------*/
    if ((JAPAN) || ((!tapeflag) && (ceosid == HBK)) ||
        (ceosid == CAN) || (ceosid == RSAT))
    {
        if (wrtdsk[SUPP])
            c_lasmsg(FALSE, "ltwgin-supp", "No supplemental file");
        return(E_SUCC);
    }

    /* Read the volume directory file
    ---------------------------------*/
    if (getvdf(unit,&dum2,filename,nfiles,tapeflag,dir_struct,hpath,sceneid,
               hosout,&dumlong,&dumlong,&dumlong,&dumlong,tempbuf,&numrecs,
               &recsize,&dumlong,&ceosid) != E_SUCC)
        return(E_FAIL);

    /* Write the supplemental file to disk
    -------------------------------------*/
    if (wrtdsk[SUPP])
    {
        /* Allocate memory for the input buffer
        ---------------------------------------*/
        in_buf = (char *) malloc(recsize);
        if (in_buf == NULL)
        {
            c_lasmsg(FALSE, "ltwgin-alloc",
                    "Error allocating dynamic memory");
            return(E_FAIL);
        }

        /* Create the file name
           -----------------------*/
        c_lsmknm(hosout[0],"supp",hosout[*num_out]);
        fp = fopen(hosout[(*num_out)++],"w");
        if(fp == NULL)
        {
            c_lasmsg(FALSE,"ltwgin-write",
                    "Error opening the supplemental file for writing");
            free(in_buf);
            return(E_FAIL);
        }

        /* If NLAPS or ESATM copy supplemental file. Append the supplemental
           filename to the supfile string. Look through the array with the
           filenames in upper-case, then append the filename at the same
           array index from the case-sensitive array.
        -------------------------------------------------------------------*/
        for(i = 0; i < nfiles; i++)
        {
            if (dir_struct == NLAPS) 
            {
                if(strncmp(filename_up[i] + SCENELEN, ".SF", 3) == 0)
                {
                    strcpy(supfile, hpath);
                    strncat(supfile, sceneid, SCENELEN);
                    strncat(supfile, filename[i] + SCENELEN, 3); /* add ".sf" */
                    break;
                }
            }
            else if(dir_struct == ESATM)
            {
                if(strncmp(filename_up[i], "DAT_SUP.001", 11) == 0)
                {
                    strcpy(supfile, hpath);
                    strcat(supfile, filename[i]);
                    break;
                }
            }
        }

        if((dir_struct == NLAPS) || (dir_struct == ESATM))
        {
            infp = fopen(supfile, "r");
            if (infp == 0)
            {
                c_lasmsg(FALSE, "ltwgin-diskread",
                        "Error reading SUPP file from disk");
                free(in_buf);
                return(E_FAIL);
            }
        }


        /* Read and write the records.  These are variable length
           so don't check status from tpread. Changed it to not
           throw a fatal error if a problem is encountered when
           reading from tape with Thailand data since it is
           consistently failing here due to the supplementary data
           file being missing (Re: CR 4936).
        -----------------------------------------------------------*/
        for (i = 0; i < numrecs; i++)
        {
            if ((dir_struct == NLAPS) || (dir_struct == ESATM))
            {
                if(fread(in_buf, 1, recsize, infp) != recsize)
                {
                    c_lasmsg(FALSE, "ltwgin-diskread",
                            "Error reading supplemental file from disk");
                    free(in_buf);
                    return(E_FAIL);
                }
                if(fwrite(in_buf,1,recsize,fp) != recsize)
                {
                    c_lasmsg(FALSE,"ltwgin-write",
                            "Error writing to the trialer file");
                    free(in_buf);
                    return(E_FAIL);
                }
            }
            else
            {
                status = c_tpread(unit,(unsigned char *)in_buf,&recsize,&mode);
                if (status == TAPE_EOF)
                    break;
                else if(status != recsize)
                {
                    if(ceosid == THAI)
                    {
                        c_lasmsg(FALSE, "ltwgin-supp", "Thailand data has "
                                 "no supplemental file");
                        break; /* break out if Thailand data since it is missing
                                  the supplementary data file */
                    }
                }
                if(fwrite(in_buf,1,status,fp) != status)
                {
                    c_lasmsg(FALSE,"ltwgin-write",
                            "Error writing to the trialer file");
                    free(in_buf);
                    return(E_FAIL);
                }
            }
        }
        /* Close the file
        -----------------*/
        if(fclose(fp) != 0)
        {
            c_lasmsg(FALSE,"ltwgin-write",
                     "Error closing the supplemental file");
            free(in_buf);
            return(E_FAIL);
        }

        /* Free allocated space
        -----------------------*/
        free(in_buf);
    }

    return(E_SUCC);
}
