/******************************************************************************
FUNCTION:       GETPAR


PURPOSE:        Get the input parameters

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ltwgin.h"

void getpar
(
  struct PARBLK *vblock,             /* TAE parameter block */
  long *bands,                       /* Bands to be processed in image */
  long *msgtim,                      /* Time between display message */
  long *window,                      /* Window of input image */
  long *minmax,                      /* Min/Max flag */
  char hosout[][CMLEN],              /* Output host name */
  char taeout[][CMLEN],              /* Output TAE name */
  char comment[][CMLEN],             /* Mount message to the operator */
  long *wrtdsk,                      /* Ancillary files to read in */
  long *media_cnt,                   /* Number of media specified */
  long *numimg_files,                /* Number of output files */
  char dpath[CMLEN],                 /* TAE input filename */
  int *dirnum                        /* scene number user wishes to ingest */
)
{
    struct VARIABLE *val;   /* pointer to a TAE variable                 */

    long fatal = -1;        /* Fatal error code                          */
    long index;             /* loop index                                */
    long temp;              /* temporary variable                        */ 

    /* Retrieve media mount message supplied by user 
    -----------------------------------------------*/
    val = p_find(vblock,"COMMENT");
    *media_cnt = val->v_count;
    for (index = 0; index < val->v_count; index++)
        strcpy(comment[index],SVAL(*val,index));

    /* Retrieve input file/directory (for disk image input)
    -------------------------------------------------------*/
    val = p_find(vblock, "INFILE");
    if (val->v_count == 1)
        strcpy(dpath, SVAL(*val,0));
    else
        strcpy(dpath,"");

    /* Retrieve directory number(s) (scene numbers) user wishes to ingest
    ---------------------------------------------------------------------*/
    val = p_find(vblock, "DIRNUM");
    if(val->v_count == 1)
        *dirnum = IVAL(*val, 0);
    else
        *dirnum = -1;

    /* Retrieve output image name(s)
    --------------------------------*/
    val = p_find(vblock, "OUT");
    *numimg_files = val->v_count;
    if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
        c_errmsg("Fatal error encountered","ltwgin-fatal",&fatal);

    /* Retrieve window area to be copied
    ------------------------------------*/
    val = p_find(vblock,"WINDOW");
    if (val->v_count != 0)
        for (index = 0; index < 4; index++)
            window[index] = IVAL(*val,index);
    else
        for (index = 0; index < 4; index++)
            window[index] = 0;

    /* Get the requested bands
    --------------------------*/ 
    for (index = 0; index < MAX_BAND;  index++)
        bands[index] = FALSE; 
    val = p_find(vblock, "BANDS");
    if (val->v_count == 0)
    {
        for (index = 0; index < MAX_BAND; index++)
            bands[index] = TRUE;
    }
    else
    {
        for (index = 0; index < val->v_count; index++)
        {
            temp= IVAL(*val,index) - 1;
            bands[temp] = TRUE;
        }
    }

    /* Get requested ancillary files
    --------------------------------*/
    for (index = 0; index < 5; index++)
        wrtdsk[index] = FALSE;
    val = p_find(vblock,"WRTDSK");
    for (index = 0; index < val->v_count; index++)
    {
        if (strcmp(SVAL(*val,index),"VDF") == 0)
            wrtdsk[VDF] = TRUE;
        else if (strcmp(SVAL(*val,index),"LEAD") == 0)
            wrtdsk[LEAD] = TRUE;
        else if (strcmp(SVAL(*val,index),"TRAI") == 0)
            wrtdsk[TRAI] = TRUE;
        else if (strcmp(SVAL(*val,index),"SUPP") == 0)
            wrtdsk[SUPP] = TRUE;
        else if (strcmp(SVAL(*val,index),"IDR") == 0)
            wrtdsk[IDR] = TRUE;
    }

    /* Retrieve number of seconds between processing messages
    ---------------------------------------------------------*/
    val = p_find(vblock,"$MSGTIM");
    *msgtim = IVAL(*val,0);

    /* Retrieve min/max flag
    ------------------------*/
    val = p_find(vblock,"$MINMAX");
    if (strcmp(SVAL(*val,0),"YES") == 0)
        *minmax = TRUE;
    else
        *minmax = FALSE;
}
