#ifndef LTWGIN_H
#define LTWGIN_H

/* Defines for LTWGIN
---------------------*/

#define MAX_BAND 7    /* Maximum number of bands on a LTWG tape currently
                         handled        */
#define MAX_FILES 40  /* Maximum number of files to be retrieved from disk
                         or CD using c_getfiles */

/* increased sceneid length from 8 to 18 */
#define SCENELEN 18    /* Length of characters needed for sceneid       */

/* Values for associated files */
#define VDF  0         
#define LEAD 1
#define TRAI 2
#define SUPP 3
#define IDR  4

/* Values for individual CD directory structures */
#define SAFRICAMSS 1
#define SPOTBIL 2
#define NLAPS 3
#define ESATM 4
#define SAR 5

/* Values for individual ground stations */
#define HBK 1
#define UNK 2
#define TK1 3
#define CAN 4
#define THAI 5
#define SAUDI 6
#define FUCINO 7
#define RSAT 8
#define SWEDEN 9

/* Temporary #defines for processing nonstandard data */
#define JAPAN ceosid == TK1

/* data structure that holds flags to indicate if sscanf was successful in
   retrieving numerical values from strings retrieved from leader file. */
struct DDR_FLAGS
{
   int path;            /* flag for WRS path number */
   int fcenll;          /* flag for scene center lat and long */
   int fcenls;          /* flag for scene center line and sample */
   int utmzone;         /* flag for UTM zone */
   int pixsiz;          /* flag for ground pixel size */
   int proj_corners;    /* flag for projection corners */
   int ls_corners;      /* flag for line/sample corners */
   int angle;           /* flag for rotation angle */
};

/* Prototypes for the functions in ltwgin
   NOTE: las.h must be included ahead of ltwgin.h
-------------------------------------------------*/

/* calcorn.c */
void calcorn
(
  double clat_l,                /* Center point latitude */
  double clon_l,                /* Center point longitude */
  char *rectec,                 /* Recording technique used */
  double *rtnlat,               /* Calculated corner point latitudes */
  double *rtnlon,               /* Calculated corner point longitudes */
  long sat                      /* Satellite number */
);

/* cdskip_bsq.c */
int cdskip_bsq
(
  long *window,                /* Window of image to be copied */
  long *cb,                    /* Current band being processed */
  long *dmod,                  /* Part of curblk needed for the window */
  long *blkfct,                /* Blocking factor */
  FILE *infp,                  /* Input file */
  long reclength,              /* Length of image descrip. record */
  long *recsize,               /* Length of image data records */
  char hpath[CMLEN],           /* Path to image data on disk */
  char filename[][CMLEN],      /* I: filenames on disk or CD */
  char sceneid[],              /* I: naming convention for NLAPS files */
  int nfiles,                  /* I: number of filenames on disk or CD */
  int dir_struct               /* I: flag indicating which directory
                                     structure is used for data */
);

/* check.c */
void chk_window
(
  long *outspc,                /* Output image file specs */
  long *window                 /* Window of image to be copied */
);

/* Line       67 */
void chk_bands
(
  long *bands,                /* Bands to be processed within image */
  long *numbands,             /* Input image file specs */
  long *totbnd                /* Total number of bands to copy */
);

/* det_ds.c */
int det_ds
(
  char hpath[CMLEN],            /* I: Qualified path to image on CD */
  int *ds,                      /* O: Denotes type of structure */
  char sceneid[SCENELEN],       /* O: Naming convention for NLAPS files */
  char filename[MAX_FILES][CMLEN], /* O: Array of filenames extracted */
  int *nfiles                   /* O: Number of filenames returned */
);

/* det_scene.c */
int det_scene
(
   char drivenam[],              /* I: drive media mounted on */
   int dirnum,                   /* I: scene number user wishes to ingest */
   char hpath[CMLEN]             /* O: Host name of input file/directory */
);

/* get_ceosid.c */
int get_ceosid
(
  char idstring[32]             /* Extracted from image VDF file */
);

/* getenf.c */
int getenf
(
   char hosout[][CMLEN],         /* I/O: Ouput host names */
   char filename[][CMLEN],       /* I: Names of files on disk or CD */
   int nfiles,                   /* I: Number of filenames on disk or CD */
   int dir_struct,               /* I: flag indicating which directory
                                       structure is used for data */
   int tapeflag,                 /* I: Indicates processing media type
                                         TRUE: tape
                                         FALSE: other (including CD, disk) */
   long *unit,                   /* I: Tape unit number */
   long *wrtdsk,                 /* I: Ancillary files to read in */
   long *num_out,                /* I/O: Number of output files */
   char comment[][CMLEN],        /* I: Mount message to the operator */
   long *num_trailrec,           /* I: Number of trailer records */
   long *tapenum,                /* I: Tape volume currently mounted */
   long *trailsiz,               /* I: Record size of the trailer file */
   long *media_cnt,              /* I: Number of media specified */
   long *no_lead,                /* I: Flag if there is a leader file on tape
                                       There is not a leader file if an
                                       image band is split across two tapes */
   char *satnam,                 /* I: Satellite name */
   char hpath[CMLEN],            /* I: Host name of input file/directory */
   char sceneid[],               /* I: Naming convention for NLAPS files */
   long ceosid                   /* I: Ground station identifier */
);

/* getlead.c */
int getlead
(
  long *unit,                    /* Tape unit number */
  long *wrtdsk,                  /* Ancillary files to read in */
  char filename[][CMLEN],        /* Filenames on disk or CD */
  int nfiles,                    /* Number of filenames on disk or CD */
  int tapeflag,                  /* I: Indicates processing media type
                                         TRUE: tape
                                         FALSE: other (including CD, disk) */
  int dir_struct,                /* I: flag indicating which directory
                                       structure is used for data */
  char hosout[][CMLEN],          /* Output host names */
  long *num_out,                 /* Number of output files */
  long *numlead,                 /* Number of leader records */
  long *leadsize,                /* Size of leader records */
  double fcenll[],               /* Full scene center lat/long degrees */
  double fcenls[],               /* Full scene center line/sample */
  long *path,                    /* Scene path */
  char *direction,               /* Satellite direction */
  char *maprj,                   /* Map projection */
  double pixsiz[],               /* Image pixel size */
  long *satid,                   /* Satellite id */
  char *sensor,                  /* Satelite sensor */
  double *angle,                 /* Angle of rotation in radians */
  long *datum_code,              /* Datum of the dataset */
  long *utmzone,                 /* UTM zone code */
  char *satnam,                  /* Satellite name */
  char *data_type,               /* Type of data (corrected or uncorrected) */
  struct DDR_FLAGS *ddr_flags,   /* flags to indicate if sscanf was successful
                                    in retrieving numerical values from
                                    strings retrieved from leader file. */
  char cornlat[][17],            /* Corner latitudes */
  char cornlong[][17],           /* Corner longitudes */
  char hpath[CMLEN],             /* Path to image data on disk */
  long ceosid,                   /* Id number of receiving station */
  double proj_corners[8],        /* Projection corners */
  double ls_corners[8],          /* Line/sample corners */
  int *data_set_summ_rec_flg,    /* indicates if the Data Set Summary
                                    record in the SAR format is present */
  int *map_proj_data_rec_flg     /* indicates if the Map Projection Data
                                    record in the SAR format is present */
);

/* getpar.c */
void getpar
(
  struct PARBLK *vblock,             /* TAE parameter block */
  long *bands,                       /* Bands to be processed in image */
  long *msgtim,                      /* Time between display message */
  long *window,                      /* Window of input image */
  long *minmax,                      /* Min/Max flag */
  char hosout[][CMLEN],              /* Output host name */
  char taeout[][CMLEN],              /* Output TAE name */
  char comment[][CMLEN],             /* Mount message to the operator */
  long *wrtdsk,                      /* Ancillary files to read in */
  long *media_cnt,                   /* Number of media specified */
  long *numimg_files,                /* Number of output files */
  char dpath[CMLEN],                 /* TAE input filename */
  int *dirnum                        /* scene number user wishes to ingest */
);

/* getvdf.c */
int getvdf
(
  long *unit,                     /* I: Tape unit number */
  long *wrtdsk,                   /* I: Ancillary files to read in */
  char filename[][CMLEN],         /* I: Names of files on disk or CD */
  int nfiles,                     /* I: Number of filenames on disk or CD */
  int tapeflag,                   /* I: Indicates processing media type
                                          TRUE: tape
                                          FALSE: other (including CD, disk) */
  int dir_struct,                 /* I: flag indicating which directory
                                        structure is used for data */
  char hpath[CMLEN],              /* I: Host path to image data */
  char sceneid[],                 /* I: Naming convention of NLAPS files */
  char hosout[][CMLEN],           /* O: Output host names */
  long *num_out,                  /* O: Number of output files */
  long *recsize,                  /* O: Image record length */
  long *numimg_recs,              /* O: Number of image records on this tape */
  long *num_trailrec,             /* O: Number of trailer records */
  char *order,                    /* O: Band order */
  long *numlead,                  /* O: Number of leader files */
  long *leadsize,                 /* O: Size of leader records */
  long *trailsiz,                 /* O: Max size of trailer records */
  long *ceosid                    /* O: Code for receiving ground station */
);

/* imgnum.c */
int imgnum
(
  char hpath[CMLEN],             /* I: Path to image data */
  char img[3]                    /* O: Image number */
);

/* ls_proj.c */
void ls_proj
(
  double *angle,                 /* Display rotation angle */
  double fcproj[],               /* Full scene center projection coords */
  double fcenls[],               /* Full scene center line/sample coords */
  double proj[],                 /* Projection coordinates calcuated */
  long linsamp[],                /* Line/sample values calculated */
  double *pixsiz                 /* Pixel size in meters */
);

/* proces_bil.c */
int proces_bil
(
   char hosout[][CMLEN],         /* I/O: Output host names */
   long *window,                 /* I/O: Window of image to be copied */
   long *totbnd,                 /* O: Total number of bands to be copied */
   char filename[][CMLEN],       /* I: filenames on disk or CD */
   char sceneid[],               /* I: naming convention of NLAPS files */
   int nfiles,                   /* I: number of filenames on disk or CD */
   int tapeflag,                 /* I: Indicates processing media type
                                         TRUE: tape
                                         FALSE: other (including CD, disk) */
   int dir_struct,               /* I: flag indicating which directory
                                       structure is used for data */
   long *unit,                   /* I: Tape unit number */
   long *wrtdsk,                 /* I: Ancillary files to read in */
   long num_out,                 /* I: Number of output files */
   long numimg_files,            /* I: Number of output image files */
   long *bands,                  /* I/O: Bands to be processed within image */
   long *msgtim,                 /* I: Time between display messages */
   long *minmax,                 /* I: Min/Max flag */
   char comment[][CMLEN],        /* I: Mount message to the operator */
   long *recsize,                /* I: Image record length */
   long *numimg_recs,            /* I: Number of image records on this tape */
   long *tapenum,                /* I/O: Current tape number */
   long *media_cnt,              /* I: Number of media specified */
   long *no_lead,                /* I: Flag if successive tapes have leader
                                       file */
   char ttype[],                 /* I: */
   char hpath[CMLEN],            /* I: Path to image data on disk */
   char *sensor,                 /* I: Satellite sensor */
   long ceosid                   /* I: Integer ID of receiving station */
);

/* proces_bsq.c */
int proces_bsq
(
   char hosout[][CMLEN],         /* I/O: Output host names */
   long *window,                 /* I/O: Window of image to be copied */
   long *totbnd,                 /* O: Total number of bands to be copied */
   char filename[][CMLEN],       /* I: filenames on disk or CD */
   char sceneid[],               /* I: naming convention for NLAPS files */
   int nfiles,                   /* I: number of filenames on disk or CD */
   int dir_struct,               /* I: flag indicating which directory
                                       structure is used for data */
   int tapeflag,                 /* I: Indicates processing media type
                                         TRUE: tape
                                         FALSE: other (including CD, disk) */
   long *unit,                   /* I: Tape unit number */
   long *wrtdsk,                 /* I: Ancillary files to read in */
   long *num_out,                /* I: Number of output files */
   long *numimg_files,           /* I: Number of output image files */
   long *bands,                  /* I/O: Bands to be processed within image */
   long *msgtim,                 /* I: Time between display messages */
   long *minmax,                 /* I: Min/Max flag */
   char comment[][CMLEN],        /* I: Mount message to the operator */
   long *recsize,                /* I: Image record length */
   long *tapenum,                /* I/O: Current tape number */
   long *media_cnt,              /* I: Number of media specified */
   long *no_lead,                /* I: Flag if successive tapes have leader
                                       file */
   char ttype[],                 /* I: */
   char hpath[CMLEN],            /* I: Path to image data on disk */
   char *sensor,                 /* I: Satellite sensor */
   long ceosid                   /* I: Integer ID of receiving station */
);

/* skip_bil.c */
int skip_bil
(
  long *window,                /* I: Window of image to be copied */
  long *unit,                  /* I/O: Tape unit number */
  char filename[][CMLEN],      /* I: filenames on disk or CD */
  int nfiles,                  /* I: number of filenames on disk or CD */
  int tapeflag,                /* I: Indicates processing media type
                                       TRUE: tape
                                       FALSE: other (including CD, disk) */
  int dir_struct,              /* I: flag indicating which directory
                                     structure is used for data */
  char comment[][CMLEN],       /* I: Mount message to the operator */
  long *numimg_recs,           /* I: Number of image records on this tape */
  long *tapenum,               /* I/O: Current tape number */
  long *numbands,              /* I: Number of bands taken from header record */
  long *media_cnt,             /* I: Number of media specified */
  char hpath[CMLEN],           /* I: Path to image data on disk / CD */
  char sceneid[]               /* I: Naming convention of NLAPS files */
);

/* skip_bsq.c */
int skip_bsq
(
  long *window,             /* Window of image to be copied */
  long *unit,               /* Tape unit number */
  char comment[][CMLEN],    /* Mount message to the operator */
  long ceosid,              /* Integer ID of receiving station */
  long *tapenum,            /* Current tape number mounted */
  long *media_cnt,          /* Number of media specified */
  long *cb,                 /* Current tape band being processed */
  long *dmod,               /* Part of curblk needed for the window */
  long *blkfct,             /* Blocking factor */
  long *ptrband,            /* Pointer to the band the tape is currently at */
  long *swreel              /* Flag if need to switch reels */
);

/* tape.c */
int mount_media
(
  long *unit,                  /* Tape unit number */
  char comment[][CMLEN],       /* Mount message to the operator */
  long tapenum, 
  long *media_cnt,             /* I: number of media specified */
  char ttype[],                /* Tape type */
  int *tapeflag                /* I/O: Indicates processing media type
                                         TRUE: tape
                                         FALSE: other (including CD) */
);

/* Line       67 */
void close_media
(
  long *unit              /* Tape unit number */
);

/* update_ddr.c */
int update_ddr
(
  char hosout[][CMLEN],          /* Host file name */
  long *window,                  /* Input window */
  long *totbnd,                  /* Total number of bands to process */
  double fcenll[],               /* Full scene center lat/long degrees */
  double fcenls[],               /* Full scene center line/sample */
  long *path,                    /* Path number */
  char *direction,               /* Sat direction */
  char *maprj,                   /* Map projection */
  double pixsiz[],               /* Pixel size */
  long *satid,                   /* Satellite id */
  char *sensor,                  /* Sensor type */
  char data_type,                /* Type of data (corrected/uncorrected) */
  double *angle,                 /* Rotation angle */
  long *datum,                   /* Datum of the data */
  long *utmzone,                 /* UTM zone code */
  char *satnam,                  /* Satellite name */
  char cornlat[][17],            /* Corner latitudes */
  char cornlong[][17],           /* Corner longitudes */
  double proj_corners[8],        /* Projection corners */
  double ls_corners[8],          /* Line/sample corners */
  long *numimg_files,            /* Number of output image files */
  long ceosid,                   /* ID number of ground station */
  int data_set_summ_rec_flg,     /* indicates if the Data Set Summary
                                    record in the SAR format is present */
  int map_proj_data_rec_flg,     /* indicates if the Map Projection Data
                                    record in the SAR format is present */
  struct DDR_FLAGS *ddr_flags    /* flags to indicate if sscanf was successful
                                    in retrieving numerical values from
                                    strings retrieved from leader file. */
);

#endif
