/******************************************************************************
FUNCTION:       SKIP_BIL

PURPOSE:        Skip to the correct spot on BIL tapes.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "ltwgin.h"

int skip_bil
(
  long *window,                /* I: Window of image to be copied */
  long *unit,                  /* I/O: Tape unit number */
  char filename[][CMLEN],      /* I: filenames on disk or CD */
  int nfiles,                  /* I: number of filenames on disk or CD */
  int tapeflag,                /* I: Indicates processing media type
                                       TRUE: tape
                                       FALSE: other (including CD, disk) */
  int dir_struct,              /* I: flag indicating which directory
                                     structure is used for data */
  char comment[][CMLEN],       /* I: Mount message to the operator */
  long *numimg_recs,           /* I: Number of image records on this tape */
  long *tapenum,               /* I/O: Current tape number */
  long *numbands,              /* I: Number of bands taken from header record */
  long *media_cnt,             /* I: Number of media specified */
  char hpath[CMLEN],           /* I: Path to image data on disk / CD */
  char sceneid[]               /* I: Naming convention of NLAPS files */
)
{
    long  recsize;          /* image record length                       */
    long  mode = HANDLE;    /* tape I/O error flag                       */
    long  nrec;             /* number of records to skip                 */
    long  status;           /* return status                             */
    long  totrec;           /* total number of records to skip           */
    long  dumlong;          /* dummy value                               */
    long  dum2 = FALSE;     /* dummy value                               */

    int tmp;                /* temp value to pass to mount_media so tapeflag
                               won't get changed during processing somehow */

    char tempbuf[10];       /* temp work buffer                          */
    char tempbuf2[3][CMLEN];/* temp work buffer                          */
    char ttype[CMLEN];

    /* Skip records on tape to starting line of the window.
       Mount next tape as needed.
    ------------------------------------------------------*/
    if (window[SL] > 1)
    {
        totrec = nrec = (window[SL] - 1) * *numbands;
        if (totrec > *numimg_recs )
        {
            /* Close current tape, mount next tape, and read
               volume directory file
            -----------------------------------------------*/
            nrec = totrec - *numimg_recs;
            close_media(unit);
            if (mount_media(unit, comment, ++(*tapenum), media_cnt, ttype, &tmp)
                != E_SUCC)
                return(E_FAIL);

            if (getvdf(unit, &dum2, filename, nfiles, tapeflag, dir_struct,
                       hpath, sceneid, tempbuf2, &dumlong, &recsize,
                       numimg_recs, &dumlong, tempbuf, &dumlong, &dumlong,
                       &dumlong, &dumlong) != E_SUCC)
                return(E_FAIL);

            if (totrec > *numimg_recs)
            {
                /* Close current tape, mount next tape, and read
                   volume directory file
                -----------------------------------------------*/
                nrec = totrec - *numimg_recs;
                close_media(unit);
                if (mount_media(unit, comment, ++(*tapenum), media_cnt, ttype,
                                &tmp) != E_SUCC)
                    return(E_FAIL);

                if (getvdf(unit, &dum2, filename, nfiles, tapeflag, dir_struct,
                           hpath, sceneid, tempbuf2, &dumlong, &recsize,
                           numimg_recs, &dumlong, tempbuf, &dumlong, &dumlong,
                           &dumlong, &dumlong) != E_SUCC)
                    return(E_FAIL);
            }
        }

        status = c_tpfsr(unit,&nrec,&mode);
        if (status != TAPE_SUCC)
            return(E_FAIL);
    }

    return(E_SUCC);
}
