/******************************************************************************
FUNCTION:       SKIP_BSQ

PURPOSE:        Skip to the correct spot on BSQ tapes.


ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "ltwgin.h"

int skip_bsq
(
  long *window,             /* Window of image to be copied */
  long *unit,               /* Tape unit number */
  char comment[][CMLEN],    /* Mount message to the operator */
  long ceosid,              /* Integer ID of receiving station */
  long *tapenum,            /* Current tape number mounted */
  long *media_cnt,          /* Number of media specified */
  long *cb,                 /* Current tape band being processed (one-based) */
  long *dmod,               /* Part of curblk needed for the window */
  long *blkfct,             /* Blocking factor */
  long *ptrband,            /* Pointer to the band the tape is currently at */
  long *swreel              /* Flag if need to switch reels */
)
{

    long  nrec;             /* number of records to skip                 */
    long  mode = HANDLE;    /* tape I/O error flag                       */
    long  one = 1;          /* constant one                              */
    long  status;           /* return status                             */
    long  skip;             /* number of files to skip                   */
    long  two=2;            /* constant of two                           */

    int tmp;                /* temp value to pass to mount_media so tapeflag
                               won't get changed during processing somehow */

    char ttype[CMLEN];  

    *dmod = 0;

    /* Switch tapes if needed.  Set the pointer band.
    -------------------------------------------------*/
    if (*swreel)
    {
        if (*cb <= 6)
            *ptrband = 4;
        else
            *ptrband = 7;
        ++(*tapenum);

        c_stoptm();
        close_media(unit);
        if (mount_media(unit, comment, *tapenum, media_cnt, ttype, &tmp)
                        != E_SUCC)
            return(E_FAIL);

        /* Skip the volume directory and leader files
        ---------------------------------------------*/
        status = c_tpfsf(unit,&two,&mode);
        if (status != TAPE_SUCC)
            return(E_FAIL);
        *swreel = FALSE;

        c_startm();
    }

    /* Skip to the correct band on tape. Don't skip files if this is SPOT
       data produced by Sweden (or France) receiving station since
       assumption is it is always written to tape with one band per
       physical volume and previous code will take care of skipping the
       VDF files when new volumes are mounted and the leader file.
       Note: This is an assumption we had to make to be able to process
       any of this data and it was based on the data available at this
       time. No spec. states how many volumes will be used for a scene
       or how may bands will be written to each volume. If other SPOT
       data from either the Sweden or France receiving station is
       encountered that does not follow this assumption is found, this
       logic will have to be re-written to accomodate the format the new
       data follows as well.
    ---------------------------------------------------------------------*/
    if(!(ceosid == SWEDEN))
    {
        skip = (*cb - *ptrband) * 3;
        status = c_tpfsf(unit,&skip,&mode);
        if (status != TAPE_SUCC)
            return(E_FAIL);
    }

    /* Skip the file descriptor record
    ----------------------------------*/
    if (*cb != 1)
    {
        status = c_tpfsr(unit,&one,&mode);
        if (status != TAPE_SUCC)
            return(E_FAIL);
    }

    /* Skip records on tape to starting line of the window.
    ------------------------------------------------------*/
    if (window[SL] > 1)
    {
        nrec = (window[SL] - 1) / *blkfct;
        *dmod = (window[SL] - 1) % *blkfct;
        status = c_tpfsr(unit,&nrec,&mode);
        if (status != TAPE_SUCC)
            return(E_FAIL);
    }

    return(E_SUCC);
}
