#include <stdio.h>
#include <string.h>
#include "las.h"
#include "tapeio.h"
#include "string.h"

/******************************************************************************
FUNCTION:       MOUNT_TAPE

PURPOSE:        Physically mounts a tape.  

ALGORITHM 

ALGORITHM REFERENCES    none
*******************************************************************************/

int mount_media
(
  long *unit,                  /* Tape unit number */
  char comment[][CMLEN],       /* Mount message to the operator */
  long tapenum, 
  long *media_cnt,             /* I: number of media specified */
  char ttype[],                /* Tape type */
  int *tapeflag                /* I/O: Indicates processing media type
                                         TRUE: tape
                                         FALSE: other (including CD) */
)
{
    struct TAPEDESC *tdrive;

    long access = TREAD;    /* open tape for read access                  */
    long mode = HANDLE;     /* tape I/O outputs error messages            */
    long rewopt = REWIND;   /* rewind option                              */
    long skip = 0;          /* skip zero files on open                    */
    long status;            /* function return status code                */
    long one=1;             /* constant of one                            */

    char density[5];        /* tape density in BPI                        */
    char tapeid[CMLEN];     /* tape id                                    */

    strcpy(density,"0");

    if (tapenum > *media_cnt)
    {
        c_errmsg("Invalid number of tapes specified","ltwgin-count",&one);
        return(E_FAIL);
    }

    /* Mount the current tape volume
    --------------------------------*/
    status = c_tpopen(unit,&skip,&mode,&rewopt,density,&access, 
            comment[tapenum - 1], tapeid); 
    if (status != TAPE_SUCC)
        return(E_FAIL);

    tdrive = (struct TAPEDESC *) *unit;
    strcpy(ttype,tdrive->dtype);

    /* Determine if tape or other. Set tapeflag to TRUE if this is a
       tape, otherwise leave at default of FALSE, only indicating the
       media is not a tape (could be CD, DVD, etc.). */
    if(las_is_tape(tdrive))
        *tapeflag = TRUE;

    return(E_SUCC);
}

/******************************************************************************
FUNCTION:       CLOSE_TAPE

PURPOSE:        Closes the tape device.

ALGORITHM 
  Close the tape

ALGORITHM REFERENCES    none
*******************************************************************************/

void close_media
(
  long *unit              /* Tape unit number */
)
{
    long one = 1;           /* value of one                               */
    long mode = NO_HANDLE;  /* tape I/O outputs error messages            */
    long rewopt = REWOFFL;  /* rewind option                              */
    long status;            /* function return status code                */

    /* Close the tape
    -----------------*/
    status = c_tpclos(unit,&mode,&rewopt,&one);
    if (status != TAPE_SUCC)
        c_errmsg("Nonfatal error encountered","ltwgin-nonfatal",&one);

}
