/*****************************************************************************
NAME				  MAGNIFY

PURPOSE   Extracts input image window from full input image specification 

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   J. Bay	   3/84        SAR      Original development
  2.0      R. Wolfe	  12/84	       SAR
  5.0      D. Steinwand    4/89        EDC      LAS 5.0 development
						C conversion
  5.1      D. Etrheim     12/92        EDC      Modified to do error checking 
                                                and to create a unique file name
                                                for the grid

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MAGNIFY must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION
	Initialize LAS processing
	Get the input image name & window entered, if any
	Send the window back to the TAE variable block
	Send the image name (without window spec) back to the TAE v-block
	Write completion message and exit
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"

main()
{
long    fatal = -1;			/* Fatal error flag for c_errmsg */
long	zero = 0;			/* constant zero */
long	one = 1;			/* constant one */
long	std_out;			/* standard output (lun) for inlas */
long	num_img;			/* Number of images entered (0 or 1) */
static long bands[MAXIMG][MAXBND+1];	/* Band numbers */
long	nbands;				/* Number of bands */
long	in_wind[MAXIMG][4];		/* Window entered with input image */
long	totbands;			/* Total number of bands entered */
long 	imglim = MAXIMG;		/* The max number of images allowed */
long 	same;				/* Window flag */
long	qflag[MAXIMG][2];   /* Image option flag */
static char taenam[MAXIMG][CMLEN];	/* TAE image name of input image */
char	*strptr[1];			/* Array of pointers to character */
static char in_image[MAXIMG][CMLEN];	/* Host image name */
struct  PARBLK  vblk;			/* TAE v-block */
char    gridnam[20];			/* Grid name */
char    outtae[1][CMLEN];    	/* Output tae name */
char    outhos[1][CMLEN];		/* Output host name */

/* Initialize LAS processing
 --------------------------*/
c_inlas(&vblk, &std_out);

/* Get the name of the input image file & it's window
 ---------------------------------------------------*/
if (c_getin(&vblk,"IN",&zero,taenam,in_image,&num_img,in_wind,bands,&nbands,
            &totbands,&imglim,&same,qflag) != E_SUCC)
	c_errmsg("Fatal error encountered","magnify-fatal",&fatal);
if (same != 1)
	{
	c_errmsg("Input images must be the same size","magnify-size",&one);
	c_errmsg("Fatal error encountered","magnify-fatal",&fatal);
	}
/* Get output image name
  ---------------------*/
if (c_getout(&vblk,"OUT",outtae,outhos) != E_SUCC)
   c_errmsg("Fatal error encountered","magnify-fatal",&fatal);

/* Create a unique file name for the grid
   --------------------------------------*/
strcpy(gridnam,"magnify");
c_creat_name(gridnam,"tmp");

/* Send back to VBLK
  -----------------*/
if (q_intg(&vblk, "IWIN", 4, in_wind[0], P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","magnify-fatal",&fatal);

strptr[0] = &(taenam[0][0]);
if (q_string(&vblk, "NOWIND", 1, strptr, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","magnify-fatal",&fatal);

strptr[0] = &gridnam[0];
if (q_string(&vblk, "GRIDNAM", 1, strptr, P_UPDATE) != SUCCESS)
	c_errmsg("Fatal error encountered","magnify-fatal",&fatal);

if (q_out(&vblk) != SUCCESS)
	c_errmsg("Fatal error encountered","magnify-fatal",&fatal);

/* Write completion message and exit program
 ------------------------------------------*/
c_complt("MAGNIFY", "", &zero);

return 0;
}
