/*******************************************************************************
NAME      		   GETPAR


PURPOSE   To retrieve and edit the parameters.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
L.HUEWE         Nov  1987       initial LAS programmer
J.DAVIS         July 1985       last LAS 4.0 programmer
R.MCKINNEY      Sept 1988       converted to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MAKEIMG must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   GETPAR retrieves the required TAE parameters from the parameter block.  The
   input images are parsed so that the window and band specifications are
   known.  Error checking is done on the input image(s) to insure that they 
   all have equal window sizes.  
******************************************************************************/


#include <stdio.h>
#include "las.h"	
#include "protos.h"
#include "util.h"

void getpar
(
    struct PARBLK *vblock,     /* I: Parameter block	              */
    long *msgtim,	       /* O: Number of seconds between
					   processing messages        */
    char minmax[],             /* O: Min/max flag		      */
    long *dflag,               /* O: ddr flag			      */
    long *nl,		       /* O: Number of lines in input image   */
    long *ns,		       /* O: Number of samples in input image */
    long *odtype,	       /* O: input data type		      */
    char *taeout,	       /* O: TAE file name of output image    */
    char *hosout,	       /* O: Host file name of output image   */
    char ioname[]              /* O: input file name		      */
)
{

struct	VARIABLE	*in;		/* Pointer to TAE variable within the
					   parameter block		     */

	long 		fatal = -1;	/* fatal error flag		     */
	long		status;		/* Status indicator		     */
	long		std_out;	/* Standard output		     */
	char		dtype[5];	/* Data type buffer                  */
        char            dflag_str[3];   /* DDR update option string          */


/* LAS initialization
------------------------------------------------*/
	c_inlas(vblock, &std_out);

/* Get the name(s) of the input file(s)	and make sure the correct number of
   input names was specified
----------------------------------------------------------------------------*/
	    in = p_find(vblock, "INFILE");
            strcpy(ioname,SVAL(*in,0));

/* Get the name of the output file
-----------------------------------------------------------------*/
	    status = c_getout (vblock,"OUT",(char (*)[256])taeout,
			(char (*)[256])hosout);
	    if (status != E_SUCC)
		c_errmsg ("Invalid output image name",
		       "makeimg-get",&fatal);

/* Get the number of lines, samples and bands in the input image
------------------------------------------------------------------*/

	    in = p_find(vblock, "NL");
	    *nl=IVAL(*in,0);
	    in = p_find(vblock, "NS");
	    *ns=IVAL(*in,0);

/* Error check NL and NS to ensure the values are greater than or equal to 1
---------------------------------------------------------------------------*/
            if (*nl < 1) 
               c_errmsg("NL must be greater or equal to 1","makeimg-nl",&fatal);
            if (*ns < 1) 
               c_errmsg("NS must be greater or equal to 1","makeimg-ns",&fatal);
 

/* Retrieve output data type
--------------------------------------------------*/
	    in = p_find(vblock, "ODTYPE");
            strcpy(dtype,SVAL(*in,0));
            if (strcmp(dtype,"BYTE") == 0)
               *odtype = EBYTE;
            else if (strcmp(dtype,"I*2") == 0)
               *odtype = EWORD;
            else if (strcmp(dtype,"I*4") == 0)
               *odtype = ELONG;
            else if (strcmp(dtype,"R*4") == 0)
               *odtype = EREAL;


/* Get the TAE globals needed
------------------------------------------------*/
	in = p_find(vblock,"$MSGTIM");
	*msgtim = IVAL(*in,0);
	in = p_find(vblock,"$MINMAX");
	strcpy(minmax,SVAL(*in,0));
        in = p_find(vblock, "$DDRVFLG");
        strcpy(dflag_str, SVAL(*in,0));
 
        if (strcmp (dflag_str,"IN") == 0)
            *dflag = INVAL;
        else
            *dflag = UNKNOW;
 
	return;

} /* End of getpar */
