/******************************************************************************

PROGRAM:   	MAKEIMG	

PURPOSE:   To create a LAS 5.0 image from a disk file created
	   either from the VI editor or from the EDT editor
	   under either UNIX or VMS.

PROGRAM HISTORY:
 mode Version   Date        Author           Code/Contr       Change Request
  -------       ----        ------           ----------       --------------
		11/83	    L. Huewe		CSB	        Original LAS
						                development
   
   4.0		07/85       J. Davis		CSB	        Final LAS 4.0
							        development	
  
   5.0          05/88  	    R. McKinney		CSB		Convert to 5.0
 
   5.1          05/92       T. Mittan           EDC      Corrected to allow
                                                         multiple white spaces 

   5.1		09/92	    T. Mittan		CSB      Added extern to curent.

   5.1          08/93       D. Bridges          CSB      Fixed problem reading
                                                         numbers that don't
                                                         completely fit in buff

   6.0          05/94       D. Bridges          CSB      Added option array for
							 opening images
   6.1          08/94       D. Etrheim          CSB      Change 777 to 666 on 
							 file open

   7.0		12/99	    D. Hames		CSB  Remove Valid=(1:50000) from
						     from NL and NS parameters
						     in makeimg.pdf. Add an 
					             error check in getpar to 
						     ensure NL and NS are >= 1.
						     Add prototypes. SRF #893.


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    MAKEIMG must be run under TAE. 

MAKEIMG has been tested on the following systems:
   VAX 11/780 (VMS operating system)
   Gould PowerNode 9050 (UNIX operating system)

PROJECT	       	LAS		                

LANGUAGE	C	

ALGORITHM DESCRIPTION:
    Initialize LAS, retrieve the  
    corresponding parameters.
    Call Process to generate the LAS image.
    Update the output DDR.
    If $MINMAX was set to "YES", save the minmax values.
    Create an output history.
    Call complete to end the routine.
******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "desc.h"
#include "protos.h"
#include "util.h"

/*
   The following variables must be declared global for image I/O routines
*/
int  fdesc;	                /* image file descriptors used for image I/O */
extern long curent;	        /* current processing line		     */
extern long curcat;	        /* current processing band		     */

void main
(void)
{
struct	PARBLK		vblock;		/* Parameter block		     */

	char		hosout[CMLEN];	/* Buffer storage for HOST name      */
	char		minmax[4];	/* Min/max flag			     */
	char 		ioname[CMLEN];  /* input file name                   */
	char		taeout[CMLEN];	/* Buffer storage for TAE name	     */

	long		dflag;	        /* delete flag 			     */
	long		odtype;		/* Data type of the input image	     */
	long		msgtim;		/* Number of seconds between processing
					   messages			     */
	long 		nl;		/* Number of lines in input image    */
	long		ns;		/* Number of samples in input image  */
	long		one = 1;	/* Constant one			     */
	long		zero = 0;	/* zero value			     */

/* Get user specified parameters and TAE globals
-----------------------------------------------------------------------------*/

	getpar (&vblock,&msgtim,minmax,&dflag,&nl,&ns,
	        &odtype,taeout,hosout,ioname);

/* Conversion from disk file to LAS image file
-----------------------------------------------------------------------------*/
	    proces(&vblock,hosout,nl,ns,
		  ioname,odtype,msgtim,minmax,dflag);

/* If output file is a LAS image, create history record
-----------------------------------------------------------------------------*/
	    if (c_hstout(&vblock,hosout,(char (*)[256])" ",&zero," ") != E_SUCC)
	    c_errmsg("Nonfatal error encountered -- Image data saved"
	    	      ,"makeimg-warn",&one);

/* Output completion message for DAL to LAS conversion
-------------------------------------------------------*/
            c_complt("MAKEIMG", (char (*)[256])taeout, &one);

}  /* End of main */
