/******************************************************************************
NAME:			PROCES	

PURPOSE:
 PROCES reads records from the input disk file(s) specified and writes them
 to a LAS image file.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    PROCES must be run under TAE.

PROJECT:			LAS

ALGORITHM DESCRIPTION:
	Open the input disk file for read access 
	Open the output image for write access
	The data is read and converted to an LAS 5.0 image file
	If minmax is set to "YES", then the minmax values are saved
	The DDR is updated 
	The files are closed
ALGORITHM REFERENCES:		none
*****************************************************************************/



/*
#include <ctype.h>
#include <stdedc.h>
*/
#include <stdio.h>
#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif
#ifdef sun
#include <fcntl.h>
#endif

#include <fcntl.h>
#include "las.h"	
#include "util.h"
#include "pixman.h"
#include "protos.h"

#define MAXLEN 1000

/*
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc;	        /* image file descriptors used for image I/O */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

void proces
(
	struct PARBLK *vblock,       /* I: TAE parameter block               */
        char *hosout,                /* I: host name of output image         */
        long  nl,                    /* I: number of lines                   */
        long  ns,                    /* I: number of samples                 */
	char *ioname,   	     /* I: array of input image names	     */
        long  odtype,                /* I: output data type                  */
        long  msgtim,                /* I: message time                      */
	char *minmax,		     /* I: min max flag			     */
	long  dflag 		     /* I: ddr flag option		     */
) 
{
	long 	acc=IWRITE;		/* Image file open access	     */
	long 	flag = ONECAT;		/* Display message flag		     */
	long    i;			/* index counter		     */
	long 	infd;			/* Input file descriptor	     */
	long    j;			/* counter			     */
	long    ky;			/* key for DDVFLG		     */
	long    len;			/* length of line		     */
	long    nbytes;			/* number of bytes read		     */
	long    ndigit = 0;		/* number of digits for a value      */
	long    nonfatal = 1;		/* nonfatal error flag		     */
	long    one = 1;		/* the number one for errors	     */
	long	opt[2];			/* Image file open option	     */
	long    sample;			/* sample indicator		     */
	long	size;			/* Number of bytes/pixel	     */
	long 	sl=1;			/* Image file starting line	     */
	long	ss=1;			/* Image fiel starting sample	     */
	long 	status;			/* Status indicator		     */


static  double minval[MAXBND];		/* min pixel value array	     */
static  double maxval[MAXBND];		/* max pixel value array	     */

	double  atof();	

	char    buffer[MAXLEN];		/* temporary buffer area	     */
        char    line[MAXLEN];		/* input buffer			     */

unsigned char	*thebuf;		/* output buffer		     */

	float   newval;			/* temporary variable		     */

	struct  DDR ddr;		/* ddr structure		     */
static  struct  BDDR bddr[MAXBND];      /* band ddr structure		     */

/* Determine the number bytes/pixel
----------------------------------------*/
	switch(odtype)
   	    {
   	    case EBYTE:  size = 1; break;
   	    case EWORD:  size = 2; break;
   	    case ELONG:	 size = 4; break;
   	    case EREAL:  size = 4; break;
   	    }
/* Set options for opening images 
---------------------------------*/
opt[0] = opt[1] = 0;

/* Open the image file for write access
-------------------------------------------------*/
	status = c_eopenr((struct FDESC **)&fdesc, hosout, &one, &sl, &ss, &nl,
			 &ns, &acc, &odtype, opt, &one);	
	if (status != E_SUCC)
	    c_clean(vblock,"makeimg",(char (*)[256])hosout,&one);


/* Allocate the input and output buffers
-------------------------------------------------*/
	if ((thebuf = (unsigned char *) malloc(ns*size)) == NULL)
	   {
	   if (c_eclose((struct FDESC **)&fdesc) != E_SUCC)
  	   c_errmsg("Nonfatal error encountered","makeimg-warn",&nonfatal);
	   c_clean(vblock,"makeimg",(char (*)[256])hosout,&one);
 	   }
	
/* Initialize min/max arrays
----------------------------------------*/
	if (strcmp(minmax,"YES") == 0)
	    c_inmnmx(&odtype,&one,minval,maxval);

/* Initialize buffer
------------------*/
        for (j = 0; j < MAXLEN; j++)
 		buffer[j] = 0;

/* Initialize the processing message
----------------------------------------*/
	curcat=1;
	curent=1;
	c_inittm("MAKEIMG","Processed","line","bands"," ",&nl,
                &one,&msgtim,&flag);
	c_startm();

/* Open the input file 
----------------------------------*/
	    infd = open(ioname, O_RDONLY, 0666);
	    if (infd < 0)
		{
		c_errmsg("Error opening input file",
	        "makeimg-open",&nonfatal);
	        if (c_eclose((struct FDESC **)&fdesc) != E_SUCC)
  	        c_errmsg("Nonfatal error encountered","makeimg-warn",&nonfatal);
	        c_clean(vblock,"makeimg",(char (*)[256])hosout,&one);
		}

/* For each line in the input file read a record from the disk file and write
   it to the LAS image file
---------------------------------------------------------------------------*/
	   len = MAXLEN;
	   sample = 0;
	   while ((nbytes = read(infd,line,len)) > 0)
	     {
		for (i=0; i<nbytes; i++)
		   {
		   if (line[i] == '\t' || line[i] == '\n' ||
		       line[i] == '\r')
		       line[i] = ' ';
		   if ((line[i] == ' ') && (ndigit  > 0))
		      {
			sscanf(buffer,"%f",&newval);
			for (j=0;j<ndigit;j++)
			    buffer[j] = 0;
			ndigit = 0;
			if (curent <= nl)
			{
			switch (odtype)
			{
			case 1:
			if (newval < 0.0)
			   {
			   c_errmsg("Output data invalid","makeimg-type",
				    &nonfatal);
			   newval = 0.0;
			   }
			if (newval > 255.0)
			   {
			   c_errmsg("Output data invalid","makeimg-type",
				    &nonfatal);
			   newval = 255.0;
			   }
			*(thebuf + sample * size)
				 = newval;
			break;
			case 2:
			if (newval < -32767.0)
			   {
			   c_errmsg("Output data invalid","makeimg-type",
				    &nonfatal);
			   newval = -32767.0;
			   }
			if (newval > 32768.0)
			   {
			   c_errmsg("Output data invalid","makeimg-type",
				    &nonfatal);
			   newval = 32768.0;
			   }
			(*(short *)(thebuf + sample * size))
				 = newval;
			break;
			case 3:
			(*(long *)(thebuf + sample * size))
				 = newval;
			break;
			case 4:
			(*(float *)(thebuf + sample * size))
				 = newval;
			break;
			}
			sample += 1;
		        if (sample == ns)
		        {
		        if (strcmp(minmax,"YES") == 0)
		          c_pxmnmx ((long *)thebuf,&ns,&minval[curcat-1],
			    &maxval[curcat-1],&odtype);
		        status = c_ewrite((struct FDESC **)&fdesc,&curcat,
			                &curent,thebuf,&one);
		        if (status == E_FAIL)
			  {
		          c_errmsg ("Error writing output image",
			    "makeimg-write",&nonfatal);
	                  close(infd);
	                  if (c_eclose((struct FDESC **)&fdesc) != E_SUCC)
  	                  c_errmsg("Nonfatal error encountered","makeimg-warn",
                                   &nonfatal);
	                  c_clean(vblock,"makeimg",(char (*)[256])hosout,&one);
			  }
			sample = 0;
			curent++;
		        }
		       }
		      }  /* end of line[i] == ' ' */
		   else 
                      if (line[i] != ' ')
                        {
                        buffer[ndigit]=line[i];
			ndigit += 1;
                        }
		   }      /* end of i loop   */
	    }
/*
  process the last line even if the number of samples don't agree
*/
        if (sample != ns && sample > 0)
        {
          c_errmsg ("Invalid number of samples",
	    "makeimg-samples",&nonfatal);
        if (strcmp(minmax,"YES") == 0)
          c_pxmnmx ((long *)thebuf,&ns,&minval[curcat-1],
	    &maxval[curcat-1],&odtype);
        status = c_ewrite((struct FDESC **)&fdesc,&curcat,&curent,thebuf,&one);
        if (status == E_FAIL)
	  {
          c_errmsg ("Error writing output image",
	    "makeimg-write",&nonfatal);
	  close(infd);
	  if (c_eclose((struct FDESC **)&fdesc) != E_SUCC)
  	      c_errmsg("Nonfatal error encountered","makeimg-warn",
                       &nonfatal);
          c_clean(vblock,"makeimg",(char (*)[256])hosout,&one);
	  }
	curent++;
        }
/*
  process the remaining lines if there are less lines input
*/
	if (curent <= nl) 
	{
	for (j=curent; j<=nl; j++)
	  {
          if (strcmp(minmax,"YES") == 0)
            c_pxmnmx ((long *)thebuf,&ns,&minval[curcat-1],
	      &maxval[curcat-1],&odtype);
          status =c_ewrite((struct FDESC **)&fdesc,&curcat,&curent,thebuf,&one);
          if (status == E_FAIL)
	  {
          c_errmsg ("Error writing output image",
	    "makeimg-write",&nonfatal);
	  close(infd);
	  if (c_eclose((struct FDESC **)&fdesc) != E_SUCC)
  	      c_errmsg("Nonfatal error encountered","makeimg-warn",
                       &nonfatal);
          c_clean(vblock,"makeimg",(char (*)[256])hosout,&one);
	  }
	  curent++;
	  }
	}
	c_stoptm();

/* Free the buffer space previously allocated
-------------------------------------------------*/
        free (thebuf);

/* Update the DDR record file
-------------------------------------------------*/
	status = c_getddr(hosout,&ddr);

	ky = DDVFLG;

	    status = c_getbdr(hosout, &bddr[0], &one);

	for (i = 0; i < 8; i++)
	    ddr.valid[i] = dflag;

	status = c_putddr(hosout,&ddr);

	    status = c_putbdr(hosout, &bddr[0]);

/* Update the associated min/max file
-------------------------------------------------*/
	if (c_svmnmx (hosout,&one,minval,maxval,minmax) != E_SUCC)
		c_errmsg("Nonfatal error encountered -- Image data saved",
			 "makeimg-warn",&nonfatal);

/* Close the input and output files
-------------------------------------------------*/
	close(infd);
	  if (c_eclose((struct FDESC **)&fdesc) != E_SUCC)
  	      c_errmsg("Nonfatal error encountered","makeimg-warn",
                       &nonfatal);
	return;
} /* End of proces */
