C     *************************************************************************
C     NAME:   			APPLUT 
C
C      
C     PURPOSE: APPLUT applies look up table to an image to create a new
C              image.
C
C     ARGUMENTS:  
C     Name         	Description
C     ----         	-----------
C     bands(MAXBND + 1, MAXIMG)
C			band specs of input image(s)
C     blut(0:257)  	byte look up table
C     curent,il       	line counter
C     curcat,ib       	band counter
C     ddesc        	file descriptor for output dltf name
C     dltfin      	input dltf name
C     dltfout      	output dltf name
C     fdesc        	file descriptor for input image
C     gdesc        	group file descriptor from egroup 
C     hosin       	input host name(s)
C     hosout       	output host name for clean call
C     inasdltf(DLTFSZ) input dltf array from r1dltf
C     indtlen		length of input dltf record type two
C     indtype		data type of input dltf
C     intype       	data type of input image
C     inkey          	key field of record one of input dltf file
C     ldesc		file descriptor for dltf file
C     lut(-32768:32767) look up table
C     lutflg		flag denoting use of multiple input dltf entries
C     ltype        	data type of look up table  
C                  	type = 1 - unsigned byte
C                  	type = 2 - integer * 2  
C     max(MAXBND+1) 	array of real max pixel values for each output band
C     min(MAXBND+1) 	array of real min pixel values for each output band
C     mnmx       	TAE global minmax flag
C     nbands(MAXIMG+1)  number of bands in each input image
C     nmrimg	   	number of input images
C     numout       	number of output host names for clean call
C     offin(MAXBND+1)   offset into input input buffer
C     offout(MAXBND+1)  offset into input output buffer
C     outasdltf(DLTFSZ) output dltf array for w1dltf
C     otype        	data type of output image
C     outdescr		text field of output dltf entry
C     outkey          	key field of record one of output dltf file
C     parblk	   	tae parameter block for clean call
C     thebuf	   	estep buffer
C     window(4,MAXIMG)	input image window
C     totbnd	   	total number of input bands
C
C
C     INTERNAL VARIABLES:
C     Name         	Description
C     ----         	-----------
C     bnd		dltf entry band counter
C     curbnd       	total band counter
C     endpt		ending value of output dltf
C     inenpt		input dltf endpt
C     instpt		input dltf startpt
C     loffin(MAXBND+1)  offset into input lut input buffer
C     loffout(MAXBND+1) offset into input lut output buffer
C     lutmax(MAXBND+1)  max of look up table
C     lutmin(MAXBND+1)  min of look up table
C     i	  	   	I*2 lut counter
C     ibct	   	bands of current image counter
C     nl	   	number of lines from input image window
C     np	   	number of samples from input image window
C     one		value of 1
C     outdtype		data type of output dltf
C     startpt		beginning value of output dltf
C     status       	function return status code
C     tmpbuf(BSIZE) 	temporary buffer for conversion from word to byte
C
C***************************************************************************

      subroutine applut(parblk,hosin,hosout,numout,lut,blut,
     +			ltype,ldesc,dltfin,inasdltf,inkey,indtlen,
     +		        indtype,lutflg,ddesc,dltfout,outkey,outdescr,
     +		        nmrimg,bands,nbands,totbnd,window,intype,otype,
     +			mnmx,min,max)

	include 'las.fin'

	include 'dltf.fin'

      real*8 lutmax
      real*8 lutmin
      real*8 max(MAXBND+1)
      real*8 min(MAXBND+1)
      real*8 mn(MAXBND)
      real*8 mx(MAXBND)
      real*8 offset
      real*8 slope

      integer*4 bands(MAXBND + 1,MAXIMG)
      integer*4 bnd
      integer*4 cldltf
      integer*4 curbnd
      integer*4 ddesc
      integer*4 endpt
      integer*4 eread
      integer*4 ewrite
      integer*4 fdesc(MAXIMG + 1)
      integer*4 fdescin(MAXIMG)
      integer*4 fdescot
      integer*4 i
      integer*4 ib
      integer*4 ibct
      integer*4 il
      integer*4 inasdltf(DLTFSZ)
      integer*4 indtlen
      integer*4 inenpt
      integer*4 instpt
      integer*4 intype
      integer*4 ldesc
      integer*4 ltype
      integer*4 lutflg
      integer*4 minmax
      integer*4 nbands(MAXIMG + 1)
      integer*4 nmrimg
      integer*4 numout
      integer*4 one
      integer*4 offin(MAXBND + 1)
      integer*4 offout(MAXBND + 1)
      integer*4 outasdltf(DLTFSZ)
      integer*4 parblk(XPRDIM)
      integer*4 pxlkup
      integer*4 pxmnmx
      integer*4 rdesc
      integer*4 rddltf
      integer*4 startpt
      integer*4 status
      integer*4 totbnd
      integer*4 nl
      integer*4 np
      integer*4 otype
      integer*4 window(4,MAXIMG)
      integer*4 w2dltf

      integer*2 lut(-32768:32767)

      character*1 blut(0:257)
      character*1 inbuf(BSIZE)
      character*1 outbuf(BSIZE)
      character*1 tmpbuf(BSIZE)

      character*(CMLEN) dltfin
      character*(CMLEN) dltfout
      character*(CMLEN) hosin(MAXIMG)
      character*(CMLEN) hosout
      character*3       indtype
      character*13      inkey
      character*4       mnmx
      character*81      outdescr
      character*3       outdtype
      character*13      outkey
  
      common/inbuf/inbuf
      common/outbuf/outbuf
      common/imgio/fdesc,fdescin,fdescot,rdesc,offin,offout
      common/curent/il
      common/curcat/ib

C-------------
C set constant
C-------------
      one = 1
C---------------------
C set input image size
C---------------------
      nl = window(3,1)
      np = window(4,1)
C----------------------------------------------------------------------
C establish look up table data type, in case its from a dltf, not mklut
C initialize min and max for minmax call within processing loop
C   Use inmnmx to initialize the min and max arrays instead of the do loops
C----------------------------------------------------------------------
      call inmnmx(otype, MAXBND, min, max)
C      if ((intype .eq. 2) .or. (otype .eq. 2)) then
C	 ltype = 2
C        do 91 i=1, MAXBND
C           min(i) = 32767
C           max(i) = -32768
C 91       continue
C      else
C         ltype = 1
C         do 101 i=1, MAXBND
C           min(i) = 255
C           max(i) = 0
C 101        continue
C     end if
C    
      if ((intype .eq. 2) .or. (otype .eq. 2)) then
	 ltype = 2
      else
         ltype = 1
      end if
C---------------------------------------------------------------------
C initialize output dltf file for writing mulitple dltf entries 
C---------------------------------------------------------------------

      if (dltfout .ne. ' ') then
         slope = 1.0
         offset = 0.0
         if (ltype .eq. 1) then
	    outdtype = 'B'
         else if (otype .eq. 2) then
	    outdtype = 'I2'
         else
	    outdtype = 'B'
         end if

	 call initdltf(ddesc,outdtype,outkey,inasdltf,outasdltf,
     +		       outdescr,totbnd,window,dltfin)
      end if

      if (dltfout .ne. ' ') then
	 if (dltfin .eq. ' ') then
            if (ltype .eq. 1) then
    	       startpt = 0
	       endpt = 255
            else
	       lutmax = -32768
	       lutmin = 32767
               status = pxmnmx(lut(-32768),65536,lutmin,lutmax,EWORD)
	       startpt = lutmin
	       endpt = lutmax
            end if
	 end if
      end if
C-------------------------------------
C apply lut
C-------------------------------------
      bnd = 1
      curbnd = 0
      do 300 i = 1, nmrimg
         do 400 ibct = 1, nbands(i)
	    curbnd = curbnd + 1
	    ib = curbnd
       	    if (dltfin .ne. ' ') then
	       status = minmax(hosin(i),one,window(1,i),bands(1,i),
     +                          nbands(i),mn,mx)
	       if (status .ne. ESUCC) call
     +      	  cleanup(parblk,intype,otype,hosout,
     +			  nmrimg,ddesc,numout,dltfout)
	       if (((bnd .eq. 1) .and. (lutflg .eq. 0)) .or.
     +		    (lutflg .eq. 1)) then
                  status = rddltf(otype,ltype,ldesc,inkey,bnd,
     +			      indtype,indtlen,blut(0),lut(-32768),
     +			      mn,mx,instpt,inenpt)
	          if (status .ne. ESUCC) call
     +      	     cleanup(parblk,intype,otype,hosout,
     +			     nmrimg,ddesc,numout,dltfout)
	          startpt = instpt
	          endpt = inenpt
	       end if
 	       bnd = bnd + 1
	    end if
            do 500 il = 1, nl
	       status =  eread(fdescin(i),bands(ibct,i),il,inbuf,1)
	       if (status .ne. np) call
     +      	  cleanup(parblk,intype,otype,hosout,
     +			  nmrimg,ddesc,numout,dltfout)

      	       if ((intype .eq. 2) .or. (otype .eq. 2)) then
	          status = pxlkup(inbuf,outbuf,np,intype,lut(0),ltype)
	       else
	          status = pxlkup(inbuf,outbuf,np,intype,blut(0),ltype)
	       end if
	       if (status .ne. ESUCC) call
     +      	  cleanup(parblk,intype,otype,hosout,
     +			  nmrimg,ddesc,numout,dltfout)

      	       if ((intype .eq. 2) .and. (otype .eq. 1)) then
	          call pxconv(intype,otype,outbuf,tmpbuf,np)
	          status = ewrite(fdescot,curbnd,il,tmpbuf,1)
	       else
	          status = ewrite(fdescot,curbnd,il,outbuf,1)
	       end if

	       if (status .ne. np) call
     +      	  cleanup(parblk,intype,otype,hosout,
     +			  nmrimg,ddesc,numout,dltfout)

               if (mnmx .eq. 'YES') then
                  status = pxmnmx(outbuf,np,min(curbnd),
     +					    max(curbnd),otype)
                  
	          if (status .ne. ESUCC) call
     +      	  cleanup(parblk,intype,otype,hosout,
     +			  nmrimg,ddesc,numout,dltfout)
  	       end if

500         continue

         if (dltfout .ne. ' ') then
            if (ltype .eq. 2) then
               status = w2dltf(ddesc,outdtype,outkey,slope,offset,
     +	                       startpt,endpt,curbnd,lut(startpt))
            else
               status = w2dltf(ddesc,outdtype,outkey,slope,offset,
     +	                       startpt,endpt,curbnd,blut(startpt))
	    end if
	    if (status .ne. ESUCC) call
     +         cleanup(parblk,intype,otype,hosout,
     +		       nmrimg,ddesc,numout,dltfout)
         end if

400      continue
300   continue

      if (dltfout .ne. ' ') then
         status = cldltf(ddesc,dltfout)
	 if (status .ne. ESUCC) call
     +      cleanup(parblk,intype,otype,hosout,
     +		    nmrimg,ddesc,numout,dltfout)
      end if

C---------------------------------------------------------------------
C save look-up table on disk if user-specified parameter name is given
C---------------------------------------------------------------------

      return
      end
