#include "las.h"
#include "util.h"
#include "map.h"


int dtcvrt
(
    int *indtyp,      /* Input data type  (I)                           */
    int *outdtyp,     /* Output data type (I)                           */
    double *minpix,   /* Minimum pixel value (I)                        */
    double *maxpix,   /* Maximum pixel value (I)                        */
    double *slope,    /* Slope to be computed by dtcvrt. (O)            */
    double *offset,   /* Offset to be computed by dtcvrt. (O)           */
    int *printmsg     /* Message flag. (I)                              */
)


/****************************************************************************
 *									    *
 *	Purpose: Convert will take minimum and maximum values of the input  *
 *		 data type and compute the slope and offset needed to scale *
 *		 all values to the output data type using the folowing 	    *
 *		 formula: 						    *
 *			output pixel = input pixel * slope + offset.	    *
 *									    *
 *	Modified by D. Akkerman, USGS/EROS Data Center, 5/88		    *
 *		DMS v1.2 retrocode					    *
 *	Modified by L. Huewe, USGS/EROS Data Center, 8/88		    *
 *	   	Modified to handle minimum and maximum values as all double *
 *		values (rather than the data type of the image.		    *
 *									    *
 ****************************************************************************/
	{
int	one = 1;	  /* Constant one 				      */

char	strng[80];	  /* Output message string 			      */

long	nonfatal = 1;	  /* Flag for non-fatal error			     */

double	maxin,minin,      /* Maximum & minimum value for input data type     */
	maxout,minout;    /* Maximum & minimum value for output data type    */

*slope = 1.0;
*offset = 0.0;

/* Obtain minimum & maximum for input data type */
c_inmnmx((long *)indtyp, (long *)&one, &maxin, &minin);

/* Obtain minimum & maximum for output data type */
c_inmnmx((long *)outdtyp, (long *)&one, &maxout, &minout);

if ((*minpix < minin) || (*maxpix > maxin))
    {
    c_errmsg("DLTF min/max values inconsistent with data type","map-lutmnmx",
              &nonfatal);
    return(E_FAIL);
    }
if ((*minpix < minout) || (*maxpix > maxout))
    if ((*maxpix - *minpix) <= (maxout - minout))
	{
	*offset = minout - *minpix;
	}
    else
	{
	*slope = (maxout - minout)/(*maxpix - *minpix);
	*offset = minout - (*slope * (*minpix));
	}

/* If calling routine requests it, print out scaling factors */

if (*printmsg)
    if ((*slope != 1.0) || (*offset != 0.0))
	{
	sprintf(strng,"Mapped %f : %f    to      0 : 255",*minpix,*maxpix);
	c_errmsg(strng, "informational", (long *)&one);
	sprintf(strng,"Slope = %f       offset = %f",*slope,*offset);
	c_errmsg(strng, "informational", (long *)&one);
	}
return(E_SUCC);

}
