C     *************************************************************************
C     NAME:   			GETPAR 
C
C      
C     PURPOSE:  GETPAR receives the global variables and the user-
C               specified parameters.  
C
C
C     ARGUMENTS:
C     Name             	Description
C     ----	       	-----------
C     bands(MABND + 1,MAXIMG) 
C   		       	input band specification
C     descr	       	output dltf description
C     dltfin	       	input DLTF file name
C     dltfout	       	output DLTF file name
C     from(1000)       	input values to be mapped
C     hosin(MAXIMG)    	input host names
C     hosout	       	output host name
C     inkey	       	key field of input dltf file
C     intype           	data type of input image
C     mnmx             	indicator for whether the user wants the
C		       	min and max determined for each output band
C     msgtim           	how often to display processing message
C     nbands(MAXBND+1) 	number of bands in each input image
C     nft	       	number of FROM/TO values specified
C     nmrimg           	number of input images
C     numout	       	number of output files for complt
C     otype            	data type of output image
C     outkey	       	key field of output dltf file
C     parblk(XPRDIM)   	parameter block
C     rtoff            	offset value of 0.5 or 0 for round/trunc of real data
C     taein(MAXIMG)    	input TAE names
C     taeout           	output TAE name
C     to(1000) 	       	output mapping specifications
C     todescend		flag denoting to values are in descending order
C     totbnd	       	total number of bands to be processed
C     window(4,MAXIMG) 	input window specification
C
C
C     INTERNAL VARIABLES:
C     Name	       	Description
C     ----	       	-----------
C     count            	number of values specified for a parameter
C     fcount           	number of values specified for a file parameter
C     fname	       	input file containing FROM and TO values
C     fromct           	number of values specified for FROM
C     ftlen(1000)       from to length specification
C     i                	loop index
C     kcount		number of values specified for OUTFILE
C     length           	length specification
C     mn(MAXBND)	minimum value of from array
C     mx(MAXBND)	maximum value of from array
C     opt              	value of round/truncate option 
C     qflag(2,MAXIMG)  	flag indicating if a subwindow was specified
C     same             	flag indicating if images have same window size
C     status           	function return status code
C     taelut		tae name of output dltf file
C     tmpfrom(1000)	temp var to get char from, then put in r*8 from
C     tmpto(1000)	temp var to get char to, then put in r*8 to
C     toct             	number of values specified for TO
C     tomax            	maximum value in the TO vector
C     tomin	       	minimum value in the TO vector
C     type		output data type specified in ODTYPE
C
C     *************************************************************************

      subroutine getpar(parblk,msgtim,mnmx,from,to,nft,
     +		        intype,otype,taein,hosin,taeout,hosout,
     +		        dltfin,inkey,dltfout,outkey,descr,numout,
     +			window,bands,nmrimg,nbands,totbnd,rtoff,todescend)

	include 'las.fin'

        include 'typlimsg.fin'

      double precision mx(MAXBND)
      double precision mn(MAXBND)
      double precision tomax
      double precision tomin

      real*8 from(1000)
      real*8 to(1000)

      real rtoff

      integer*4 bands(MAXBND + 1,MAXIMG)
      integer*4 count
      integer*4 fcount
      integer*4 fromct
      integer*4 ftlen(1000)
      integer*4 getin
      integer*4 getnam
      integer*4 getout
      integer*4 gettyp
      integer*4 i
      integer*4 intype
      integer*4 j
      integer*4 kcount
      integer*4 length
      integer*4 minmax
      integer*4 msgtim
      integer*4 nbands(MAXIMG + 1)
      integer*4 nft
      integer*4 nmrimg
      integer*4 numout
      integer*4 otype
      integer*4 parblk(XPRDIM)
      integer*4 pxmnmx
      integer*4 qflag(2,MAXIMG)
      integer*4 same
      integer*4 status
      integer*4	totbnd
      integer*4 toct
      integer*4 todescend
      integer*4 window(4,MAXIMG)

      character*81      descr
      character*(CMLEN) dltfin
      character*(CMLEN) dltfout
      character*(CMLEN) fname
      character*(CMLEN) hosin(MAXIMG)
      character*(CMLEN) hosout
      character*13      inkey
      character*13      outkey
      character*(CMLEN) taein(MAXIMG)
      character*(CMLEN) taelut
      character*(CMLEN) taeout(2)

      character*4 mnmx
      character*5 opt
      character*34 tmpfrom(1000)
      character*34 tmpto(1000)
      character*36 tempchar
      character*4 type

C     Get globals
C     -----------
      call xrintg(parblk,'$msgtim',1,msgtim,count,status)
      call xrstr(parblk,'$minmax',1,mnmx,length,count,status)

C     Get input file specification
C     ----------------------------
      status = getin(parblk,'IN',1,taein,hosin,nmrimg,window,
     +		     bands,nbands,totbnd,MAXIMG,same,qflag)
      if (status .ne. ESUCC) then
	 call errmsg
     +   ('Fatal error encountered','map-fatal',-1)
      end if

      if (same .ne. 1) then
	 call errmsg
     +   ('Input images do not have the same window sizes',
     +    'map-window',1)
         call errmsg('Fatal error encountered','map-fatal',-1)
      end if

C     Get input data type
C     ------------------- 
      status = gettyp(hosin,nmrimg,intype)
      if (status .ne. ESUCC) then
          call errmsg('Input images are not of same data type',
     +		      'map-type',1)
          call errmsg('Fatal error encountered','map-fatal',-1)
      end if

C     Get output file name
C     --------------------
      status = getout(parblk,'OUT',taeout,hosout)
      if (status .ne. ESUCC) then
         call errmsg('Fatal error encountered','map-fatal',-1)
      end if
      numout = 1
      
C     Get output data type
C     -------------------- 
      call xrstr(parblk,'ODTYPE',1,type,length,count,status)
      if (type .eq. 'SAME') then
         otype = intype
      else if (type .eq. 'BYTE') then
	 otype = 1
      else if (type .eq. 'I*2') then
	 otype = 2
      else if (type .eq. 'I*4') then
	 otype = 3
      else
	 otype = 4
      end if

C     Get ROUND/TRUNCATE option, set offset for processing
C     ----------------------------------------------------
      call xrstr(parblk,'ROUNDOPT',1,opt,length,count,status)
      if ((opt .eq. 'ROUND') .and. (intype .eq. 4) .and. (otype .ne. 4))
     + then
	 rtoff = 0.5
      else
	 rtoff = 0.0
      end if

C Get user specifications for FROM, TO, INFILE, INDLTF, OUTFILE, OUTDLTF, TEXT
C ----------------------------------------------------------------------------
C     Get output and input disk lut disk files, if specified
C     ------------------------------------------------------
      call xrstr(parblk,'OUTFILE',1,taelut,length,count,status)
      call xrstr(parblk,'OUTDLTF',1,outkey,length,kcount,status)
      if ((kcount .ne. 1) .and. (count .eq. 1)) then
         call errmsg
     +	   ('Output DLTF file requires key field from OUTDLTF',
     +		   'map-dltfentr',1)
         call errmsg('Fatal error encountered','map-fatal',-1)
      else if (kcount .eq. 1) then
	 if (count .eq. 1) then
	    dltfout = taelut
            if (getnam(parblk,'OUTFILE',1,dltfout,'dltf',IUPDAT) .ne.
     +	       ESUCC)
     +	        then
	        call errmsg('Fatal error encountered','map-fatal',-1)
            end if
            taelut = dltfout
	 else
	    call lsmknm(hosout,'dltf',dltfout)
            taelut = dltfout
	 end if
         call xrstr(parblk,'TEXT',1,descr,length,kcount,status)
         if ((intype .eq. 3) .or. (intype .eq. 4) .or. 
     +       (otype .eq. 3) .or. (otype .eq. 4)) then
	    call errmsg('Output DLTF will not be created',
     +			    'map-create',1)
            dltfout = ' '
         else
	    taeout(2) = taelut
	    numout = 2
         end if
      else
         dltfout = ' '
      end if

      count = 0
      fromct = 0
      toct = 0
      dltfin = ' '
      call xrstr(parblk,'INFILE',1,dltfin,length,fcount,status)
      call xrstr(parblk,'INDLTF',1,inkey,length,kcount,status)
      if (kcount .eq. 1) then
C if this is a dltf file
C ----------------------
         if (fcount .eq. 1) then
            if (getnam(parblk,'INFILE',1,dltfin,'dltf',IREAD).ne.ESUCC)
     +	        then
	        call errmsg('Fatal error encountered','map-fatal',-1)
            end if
	 else
	    if (nmrimg .eq. 1) then
	       call lsmknm(hosin,'dltf',dltfin)
	    else
	        call errmsg('Input DLTF file name cannot be defaulted',
     +			    'map-dltfname',1)
	        call errmsg(' if IN specifies more than one image name',
     +			    'map-dltfname',1)
	        call errmsg('Fatal error encountered','map-fatal',-1)
	    end if
	 end if
      else if ((fcount .eq. 1) .and. (kcount .ne. 1)) then
C else this is a from to pair file
C --------------------------------
	    dltfin = ' '
            count = 1
            fcount = 0
	    fname = dltfin
            if (getnam(parblk,'INFILE',1,fname,'txt',IREAD) .ne. ESUCC)
     +	     then
	       call errmsg('Fatal error encountered','map-fatal',-1)
            end if
      else
         call xrstr(parblk,'FROM',1000,tmpfrom,ftlen,fromct,status)
	 do 05 i = 1,fromct
	    if (index(tmpfrom(i),'.') .eq. 0) then
	       length = index(tmpfrom(i),' ')
	       tempchar = tmpfrom(i)
	       tempchar(length:(length + 1)) = '.0'
	    else
	       tempchar = tmpfrom(i)
	    end if
	    read(tempchar,'(D17.15)') from(i)
05	 continue
         call xrstr(parblk,'TO',1000,tmpto,ftlen,toct,status)
	 do 06 i = 1,toct
	    if (index(tmpto(i),'.') .eq. 0) then
	       length = index(tmpto(i),' ')
	       tempchar = tmpto(i)
	       tempchar(length:(length + 1)) = '.0'
	    else
	       tempchar = tmpto(i)
	    end if
	    read(tempchar,'(D17.15)') to(i)
06	 continue
      end if

C     Check validity of user specifications for FROM and TO parameters
C     ----------------------------------------------------------------
      if (count .eq. 1) then
         call rdfil(fname,from,to,nft)
	 fromct = nft
	 toct = nft
      else if (fcount .eq. 1) then
         return
      else if (fromct.eq.0) then
	 status =  minmax(hosin,nmrimg,window,bands,nbands,mn,mx)
	 if (status .ne. ESUCC) call
     +	    errmsg('Fatal error encountered','map-fatal',-1)
	 do 10 i = 1,nmrimg
	    do 20 j = 1,nbands(nmrimg)
	       if (mn(j) .lt. from(1)) then
	          from(1) = mn(j)
	       end if
	       if (mx(j) .gt. from(2)) then
	          from(2) = mx(j)
	       end if
20	    continue
10       continue
	 fromct = 2
      end if
      if (toct.eq.0) then
         if (otype .eq. EBYTE) then
	    to(1) = MINUTINY
	    to(2) = MAXUTINY
	 end if
         if (otype .eq. EWORD) then
	    to(1) = MINWORD
	    to(2) = MAXWORD
	 end if
         if (otype .eq. ELONG) then
	    to(1) = MAXLONG
	    to(2) = MINLONG
	 end if
 	 if (otype .eq. EREAL) then
	    to(1) = MAXREAL
	    to(2) = MINREAL
	 end if
	 toct = 2
      end if
      if ((fromct.eq.1).or.(toct.eq.1)) then
	 call errmsg
     +   ('FROM and TO must have at least two values',
     +    'map-dimension',1)
         call errmsg
     +   ('Fatal error encountered','map-fatal',-1)
      end if
      if ((fromct).ne.(toct)) then
         call errmsg
     +('The number of FROM values must equal the number of TO values',
     + 'map-unequal',1)
	 call errmsg
     +	 ('Fatal error encountered','map-fatal',-1)
      end if

      if (fcount .ne. 1) then
         nft = fromct
	 todescend = 0
         do 30 i = 2,nft
            if (from(i) .le. from(i-1))  then
               call errmsg
     +         ('FROM values must be specified in ascending order',
     +          'map-order',1)
               call errmsg
     +         ('Fatal error encountered','map-fatal',-1)
            end if
            if (to(i) .lt. to(i-1))  then
	       todescend = 1
            end if
30       continue
      
         tomin = MAXREAL
         tomax = MINREAL
         status = pxmnmx(to,nft,tomin,tomax,EDBLE)
         if (otype.eq.EBYTE) then
            if ((tomin.lt.MINUTINY).or.(tomax.gt.MAXUTINY)) then
                call errmsg
     +      ('TO value is outside the output data type range',
     +	         'map-range',1)
                call errmsg
     +          ('Fatal error encountered','map-fatal',-1)
            end if
         else if (otype.eq.EWORD) then
            if ((tomin.lt.MINWORD).or.(tomax.gt.MAXWORD)) then
                call errmsg
     +	      ('TO value is outside the output data type range',
     +	         'map-range',1)
                call errmsg
     +          ('Fatal error encountered','map-fatal',-1)
            end if
         else if (otype.eq.ELONG) then
            if ((tomin.lt.MINLONG).or.(tomax.gt.MAXLONG)) then
                call errmsg
     +	      ('TO value is outside the output data type range',
     +	         'map-range',1)
                call errmsg
     +          ('Fatal error encountered','map-fatal',-1)
            end if
         else if (otype.eq.EREAL) then
            if ((tomin.lt.MINREAL).or.(tomax.gt.MAXREAL)) then
                call errmsg
     +	      ('TO value is outside the output data type range',
     +	         'map-range',1)
                call errmsg
     +          ('Fatal error encountered','map-fatal',-1)
            end if
         end if
      end if

      return
      end
