C     *************************************************************************
C     NAME:   			INITIO 
C
C      
C     PURPOSE: INITIO opens the image files and groups them for processing.
C
C	       The processing is set up so that the input images are 
C	       always read in as REAL and then converted to the specified
C	       data type upon output.
C
C
C     ARGUMENTS:
C     Name 	       	Description
C     ----	       	-----------
C     bands(MAXBND+1,MAXIMG)
C            	       	input band specification
C     blut(0:257)	byte look up table array
C     ddesc	       	file descriptor for output dltf
C     dltfout	       	host name for output dltf file
C     fdesc(MAXIMG+1)  	file descriptor from eopens stating machine word size
C     fdescin(MAXIMG+1) file descriptor from eopenr for input image(s)
C     fdescot(MAXIMG+1) file descriptor from eopenr for output image
C     hosin(MAXIMG)    	input host names
C     hosout	       	output host name
C     inbuf(BSIZE)	buffer for input lines
C     inkey	       	key field of record one of input dltf file
C     intype	       	data type of input image
C     lut(-32768:32767) I*2 look up table array
C     nbands(MAXIMG + 1)number of bands in each input image
C     nmrimg           	number of input images
C     numout	       	number of names for complt or cleanup
C     outuf(BSIZE)	buffer for outut lines
C     otype	       	data type of output image
C     outkey	       	key field of record one of output dltf file
C     parblk	       	tae parameter block
C     rdesc	       	group descriptor from egroup for R*$ or I*4 data
C     taeout
C     totbnd	       	total number of bands to be processed
C     windw(4,MAXIMG)  	input window specification
C
C     
C     INTERNAL VARIABLES:
C     Name	       	Description
C     ----	       	-----------
C     bnd	       	band loop counter
C     i	               	loop index
C     nfiles	       	number of files opened
C     numbnds		number of bands returned
C     offin(MAXBND+1)  	offset into input buffer
C     offout(MAXBND+1) 	offset into output buffer
C     opt(2)           	option list used for eopens 
C     status           	function return status code
C
C     *************************************************************************

      subroutine initio(parblk,totbnd,bands,windw,nmrimg,hosin,hosout,
     +		        intype,otype,nbands,ddesc,dltfout,outkey,taeout,
     +		        numout,blut,lut)

	include 'las.fin'

	include 'dltf.fin'

      real*4 thebuf(BSIZE)

      integer*4 bnd
      integer*4 bands(MAXBND + 1,MAXIMG)
      integer*4 ckdltf
      integer*4 ddesc
      integer*4 egroup
      integer*4 eopenr
      integer*4 eopens
      integer*4 fdesc(MAXIMG + 1)
      integer*4 fdescin(MAXIMG)
      integer*4 fdescot
      integer*4 i
      integer*4 ib
      integer*4 il
      integer*4 intype
      integer*4 nbands(MAXIMG + 1)
      integer*4 nfiles
      integer*4 nmrimg
      integer*4 numout
      integer*4 numbnds(MAXBND + 1)
      integer*4 offin(MAXBND + 1)
      integer*4 offout(MAXBND + 1)
      integer*4 opt(2)
      integer*4 otype
      integer*4 parblk(XPRDIM) 
      integer*4 rdesc
      integer*4 status
      integer*4 totbnd
      integer*4 windw(4,MAXIMG)

      integer*2 lut(-32768:32767)

      character*1 blut(0:257)
      character*1 inbuf(BSIZE)
      character*1 outbuf(BSIZE)

      character*(CMLEN) dltfout
      character*(CMLEN) hosin(MAXIMG)
      character*(CMLEN) hosout
      character*13      outkey
      character*(CMLEN) taeout(2)

      common/inbuf/inbuf
      common/outbuf/outbuf
      common/thebuf/thebuf
      common/imgio/fdesc,fdescin,fdescot,rdesc,offin,offout
      common/curcat/ib
      common/curent/il

C     Open output dltf if necessary
C     Open the input image file(s)
C     Open the output image file
C     Group the images for sequential processing
C     ----------------------------

      nfiles = 0
      numout = 0
      do 50 i = 1,(MAXBND + 1)
         numbnds(i) = 0
50    continue
      
      if ((otype .lt. 3) .and. (intype .lt. 3)) then
         bnd = 1
         do 60 i = 1,nmrimg
            status = eopenr(fdescin(i), hosin(i), numbnds(i),
     +  	  	     windw(1,i), windw(2,i),
     +			     windw(3,i), windw(4,i),
     +		             IREAD,intype,0,1)
            if (status .ne. ESUCC) call
     +         cleanup(parblk,intype,otype,hosout,
     +		       nmrimg,ddesc,nfiles,dltfout)
            bnd = bnd + nbands(i)
60       continue

         opt(1) = otype
         opt(2) = 0
         status = eopenr(fdescot,hosout,totbnd,1,1,
     +		         windw(3,1),windw(4,1),IWRITE,
     +		         otype,opt,1)
         if (status .ne. ESUCC) call
     +      cleanup(parblk,intype,otype,hosout,
     +		       nmrimg,ddesc,nfiles,dltfout)
	 numout = numout + 1
      else
         bnd = 1
         do 70 i = 1,nmrimg
            status = eopens(fdesc(i), hosin(i), bands(1,i),
     +  	  	     windw(1,i), windw(2,i),
     +			     windw(3,i), windw(4,i),
     +		             offin(bnd),IREAD,EREAL,0,0)
            if (status .ne. ESUCC) call
     +         cleanup(parblk,intype,otype,hosout,
     +		       nmrimg,ddesc,nfiles,dltfout)
            bnd = bnd + nbands(i)
70       continue

         opt(1) = otype
         opt(2) = 0
         status = eopens(fdesc(nmrimg + 1),hosout,totbnd,1,1,
     +		         windw(3,1),windw(4,1),offout(1),IWRITE,
     +		         EREAL,0,opt)
         if (status .ne. ESUCC) call
     +      cleanup(parblk,intype,otype,hosout,
     +		    nmrimg,ddesc,nfiles,dltfout)
	 numout = numout + 1
         fdesc(nmrimg + 2) = 0
         status = egroup(fdesc,rdesc,thebuf,BSIZE)
         if (status .ne. ESUCC) call
     +      cleanup(parblk,intype,otype,hosout,
     +		    nmrimg,ddesc,numout,dltfout)
      end if

      if (dltfout .ne. ' ') then
	 status = ckdltf(dltfout,outkey,ddesc)
         if (status .ne. ESUCC) then
            call cleanup(parblk,intype,otype,hosout,
     +		         nmrimg,ddesc,numout,dltfout)
         end if
         taeout(2) = dltfout
	 numout = numout + 1
	 nfiles = 1
      end if

      return
      end

