C     *************************************************************************
C     NAME        	    	        MAP	
C
C     PURPOSE	 			
C        Performs a piecewise linear transformation on a single- or multi-band
C	 image of any data type.
C
C     EXTERNAL ASSOCIATIONS		none
C
C     FILE REFERENCES			
C        Name         I/O      Purpose of reference
C        ----         ---      --------------------
C        dltfout        o       output look up table
C	 hosout		o	output image
C
C     COMMON BLOCK REFERENCES
C        Name        Field             Type       Description
C        ----        -----             ----       -----------
C        curent      il                I*4        current line
C        curcat      ib                I*4        current band
C
C        seqio       fdesc(MAXIMG+1)   I*4        group file descriptors
C        seqio       fdescin(MAXIMG+1) I*4        input file descriptors
C        seqio       fdescot(MAXIMG+1) I*4        output file descriptor
C
C        seqio       offin(MAXBND+1)   I*4        offsets into 'thebuf'
C						  for input bands
C
C        seqio       offout(MAXBND+1)  I*4        offsets into 'thebuf'
C 						  for output bands
C
C        seqio       rdesc             I*4        group descriptor for R*4 and
C						  I*4 images
C
C        thebuf      thebuf(BSIZE)     R*4        sequential I/O buffer
C        inbuf       inbuf(BSIZE)      R*4        input image random I/O buffer
C        outbuf      outbuf(BSIZE)     R*4        output image random I/O buffer
C
C     DATA STRUCTURES		       none
C
C	PROGRAM HISTORY:
C	Version   Date   	Author	   Code/Contr.	Change Request
C	-------	  ----		------	   -----------	--------------
C	   1    09/14/84        S. Ross        SAR
C	   3	04/14/88        K. Narayanan   SAR      Logic failed for invalid
C                                                       FROM and TO values 
C                                                       for integer images
C							and increase accuracy
C							of the output values
C
C		   12/85	K. Zanter	EDC     CIPS conversion to 1 
C       	   08/86	J. Stansbery    EDC     expand FROM and TO
C							capabilities
C				      			and update to standards
C       	   09/86	K. Zanter       EDC     newLAS updates
C		   12/86	J. Stansbery    EDC     expand FROM and TO
C							capabilities more
C       	   04/88	B. Davis	EDC     incorporated LAS 4.0
C							STRETCH's look up table
C				      			pixel assignment for
C							byte and I*2 data types.
C							Also, it was jointly
C				      			decided to use STRETCH'S
C							pixel calculation for
C							real and I*4, as this
C							algorithm was deemed to
C							be a more accurate
C							equation. The difference
C							was in the calculation
C							of offset. - PRF 4487
C          5.0	   12/88	B. Davis	EDC     Conversion to 5.0, added
C				      			 input DLTF capability
C				      			changed FROM-TO to 1 set
C							 of 100 pairs 
C							added round/truncate
C							 option
C          5.0	   04/89	B. Davis	EDC     After rounding real
C							 pixels, clipped them to
C							 the output from-to
C							 bounds, and in order to
C							 retain as much accuracy
C							 as DLTF logic, made
C							 and to and real
C							 calculations double.
C          5.0	   05/89	B. Davis	EDC     FROM and TO are read in
C							 as strings and then
C							 converted to double 
C							 precision in getpar and
C							 rdfil.
C          5.0	   05/89	B. Davis	EDC     Adjusted round option to
C							 to subract from neg.
C							 values and add to pos.
C							 values.
C          5.1	   10/89	B. Davis	EDC     After r1dltf call, check
C							 against totbnd in order
C							 to correctly handle 
C							 multi-band dltfs.
C          5.2	   12/89	B. Davis	EDC     Makefile for TAE 4.0 -
C							 tae.h in WORINCLUDE,
C							 not TINC
C          5.3	   12/89	B. Davis	EDC     Corrected handling of
C							 multi-band output dltfs
C							 as in 5.1 above.
C          5.4	   01/90	B. Davis	EDC     In previous mod of 4/89
C							 clipping check clipped
C							 all pixels to first to
C							 value if the to pairs
C							 were of descending
C							 magnitude. Made the
C							 clip check more
C							 general.
C          5.5	   02/90	B. Davis	EDC     When setting todescend
C							 getpar, allow adjacent
C							 TO values to be
C							 ascending.
C          5.6	   02/90	B. Davis	EDC     Made conversion of buf
C							 in rdfil to float,
C							 then double, not dir-
C							 ectly to double(sscanf)
C          5.7	   09/90	B. Davis	EDC     pdf change for number
C							 of from to pairs from
C							 INFILE. stated as 100,
C							 really 1000 are allow-
C							 ed. also, put EDBLE in
C							 the call to pxmnmx, to
C							 retain precision for
C							 R*4.
C          5.8	   11/90	B. Davis	EDC     Changed numeric
C							 constants in pwlin.f to
C							 MAXLONG and MINLONG.
C          5.8	   12/90	B. Davis	EDC     minmax call in applut
C							 was passing arrays,
C							 changed to pass
C							 array(img).
C          5.9     05/92	D. Etrheim	EDC	Correct partlut in
C							mklut.f to be set to
C							1000 
C          6.0     03/93        D. Etrheim      EDC     ADP #6069  Allow a DLTF
C						entry consisting of multiple
C						bands to be applied to a single
C						band image
C          6.1     08/93        D. Bridges      EDC     Corrected minmax 
C                				        calculation for BYTE
C     						        and I*2	data types
C          6.2     08/93        D. Bridges      EDC     Corrected problem
C                                                       with initializing ltype
C						        when using a DFLT 
C          6.3     1/99         L.Johnson       EDC     Changed parblk to
C                                                       real*8 for ALL
C                                                       platforms due to
C                                                       apparent alignment
C                                                       problems between
C                                                       Fortran and C.
C
C     COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
C       MAP must be run under TAE.
C
C     PROJECT		        LAS		
C
C     LANGUAGE			FORTRAN-77
C
C     ALGORITHM (FLOW)
C       The user-specified parameters and the necessary global variables 
C       are retrieved.  The scale and offset values are computed for real
C       and integer*4 data, then the piecewise linear mapping is 
C       performed on the input image.  For byte and integer*2 data,
C       a look up table is created for use during the piecewise linear
C       mapping, or read from a dltf file.  The associated min/max and
C       history files are created/updated to represent the output image.
C       Finally, a completion message is written to the terminal.
C
C     ALGORITHM REFERENCES
C	The algorithm for the actual pixel calculation was taken from 
C	LAS 4.0 STRETCH.  There was a difference in the way that offset
C	was calculated in LAS 4.0 STRETCH as compared to that of newLAS
C	MAP, which could produce unequal results due to rounding and 
C	host machine calculation differences.
C	All dltf reading, writing, and utilization was coordinated with
C	LAS 5.0 DMS software to ensure compatiblity.
C
C     *****************************************************************

C     INTERNAL VARIABLES:
C     Name	       Description
C     ----	       -----------
C     bands(MAXBND + 1,MAXIMG)
C      	               input band specification
C     blut(0:255)      byte look up table
C     ddesc	       file descriptor for dltf input file
C     descr	       description field of output dltf file
C     dltfin	       input dltf name
C     dltfout	       output lut name
C     from(1000)       input values to be mapped
C     j                loop index
C     hosin(MAXIMG)    input host names
C     hosout	       output host name
C     hstout           function to create associated history file 
C     inkey	       dltf entry of input dltf file
C     intype	       data type of input image(s)
C     ltype	       data type of look up table
C     lut(-32768:32767)look up table
C     lutbuf(BSIZE)    buffer for lookup table
C     minmax	       indicator for whether the user wants
C            	       the min and max determined for each band
C     msgtim           how often to display processing message
C     nbands(MAXIMG+1) number of bands in each input image
C     nft	       number of FROM/TO values specified
C     nmrimg 	       number of input images
C     numout	       number of names for complt
C     off(999)         offsets for transformation
C     one	       value of one
C     otype	       data type of output image
C     outkey	       dltf entry of output dltf file
C     parblk(XPRDIM)   parameter block
C     rtoff            offset value of 0.5 or 0 for round/trunc of real data
C     sc(999)          scale factors for transformation
C     status           function return status code
C     stdout	       standard output
C     taein(MAXIMG)    input TAE names
C     taeout	       output TAE name
C     to(1000)         output mapping specifications
C     todescend	       flag for noting that to values are descending
C     totbnd	       total number of bands to be processed
C     wndow(4,MAXIMG)  input window specification
C     ******************************************************************

      program map

	include 'las.fin'

	include 'dltf.fin'

      real*8 from(1000)
      real*8 off(999)
      real*8 parblk(XPRDIM)
      real*8 sc(999)
      real*8 to(1000)

      real*4 rtoff
      real*4 thebuf(BSIZE)

      integer*4 bands(MAXBND + 1,MAXIMG)
      integer*4 fdesc(MAXIMG + 1)
      integer*4 fdescin(MAXIMG)
      integer*4 fdescot
      integer*4 hstout
      integer*4 ib
      integer*4 il 
      integer*4 inasdltf(DLTFSZ)
      integer*4 indtlen
      integer*4 intype 
      integer*4 j
      integer*4 ldesc
      integer*4 ltype
      integer*4 lutflg
      integer*4 msgtim
      integer*4 nbands(MAXIMG + 1)
      integer*4 nft
      integer*4 nmrimg
      integer*4 numout
      integer*4 offin(MAXBND + 1)
      integer*4 offout(MAXBND + 1)
      integer*4 otype
      integer*4 one
      integer*4 rdesc
      integer*4 r1dltf
      integer*4 status
      integer*4 stdout
      integer*4 todescend
      integer*4 totbnd
      integer*4 wndow(4,MAXIMG)

      integer*2 lut(-32768:32767)

      character*1 blut(0:257)
      character*1 inbuf(BSIZE)
      character*1 outbuf(BSIZE)

      character*(CMLEN) dltfin
      character*(CMLEN) dltfout
      character*(CMLEN) hosin(MAXIMG)
      character*(CMLEN) hosout
      character*81      indescr
      character*3       indtype
      character*13      inkey
      character*81      outdescr
      character*13      outkey
      character*(CMLEN) taein(MAXIMG)
      character*(CMLEN) taeout(2)
     
      character*4   minmax

      common/inbuf/inbuf
      common/outbuf/outbuf
      common/thebuf/thebuf
      common/imgio/fdesc,fdescin,fdescot,rdesc,offin,offout
      common/curent/il
      common/curcat/ib

C     Initialize LAS program 
C     ----------------------
      call inlas(parblk,stdout)
      one = 1

C     Get user specified parameters
C     -----------------------------
      call getpar(parblk,msgtim,minmax,from,to,nft,
     +	          intype,otype,taein,hosin,taeout,hosout,
     +		  dltfin,inkey,dltfout,outkey,outdescr,numout,wndow,
     +		  bands,nmrimg,nbands,totbnd,rtoff,todescend)

C     -------------------------------------------------------------------
C     Compute scale and offset values if input or output data type is real
C     or I*4,
C     Otherwise create a LUT
C     pwlin will then make actual calculations only for real or I*4,
C     making an assignment from the LUT for byte or I*2
C     -------------------------------------------------------------------
      if ((otype .gt. 2) .or. (intype .gt. 2)) then
          do 10 j = 1, (nft - 1)
             if (from(j + 1) - from(j) .eq. 0) then
                sc(j) = 1
             else
                sc(j) = (to(j + 1) - to(j)) / (from(j + 1) - from(j))
             end if
             off(j) = to(j + 1) - (sc(j) * from(j + 1))
10        continue
      else if (dltfin .eq. ' ') then
          call mklut(ltype,lut,blut,intype,otype,from,to,nft)
      else
         status = r1dltf(ldesc,dltfin,inkey,indescr,inasdltf,
     +		         indtlen,indtype)
	 if (status .ne. ESUCC) then
            call errmsg('Fatal error encountered','map-fatal',0)
	 end if
         if ((inasdltf(NUMBND) .gt. totbnd) .and.
     +        (totbnd .eq. 1)) then
C            call errmsg('Multiple entry DLTF being applied to one band',
C     +                  'informational',1)
             continue
         else if ((inasdltf(NUMBND) .ne. totbnd) .and.
     +	   (inasdltf(NUMBND) .ne. 1)) then
            call errmsg('Improper number of DLTF entries',
     +			'map-entries',1)
            call errmsg('Fatal error encountered','map-fatal',0)
         end if
         if (inasdltf(NUMBND) .eq. 1) then
	    lutflg = 0
	 else
	    lutflg = 1
         end if
      end if

C     Perform piecewise linear mapping function
C     -----------------------------------------
      call pwlin(parblk,hosin,taeout,numout,hosout,wndow,bands,nmrimg,
     +		 from,to,nft,sc,off,intype,otype,msgtim,minmax,nbands,
     +		 totbnd,lut,blut,ltype,ldesc,dltfin,inkey,indescr,
     +		 inasdltf,indtlen,indtype,
     +		 lutflg,dltfout,outkey,outdescr,rtoff,todescend)

C     Generate history 
C     ----------------
      status = hstout(parblk,hosout,hosin,nmrimg,' ')
      if (status .ne. ESUCC) then
         call errmsg('Nonfatal error encountered','map-warn',one)
      endif

C     Print a completion message
C     --------------------------
      call complt('MAP',taeout,numout)

      end 
