C     *******************************************************************
C     PURPOSE:  MKLUT produces a piece-wise linear look up table by
C	        completing a partial look up table provided by the user.
C               The gaps in the partial look up table are filled by linear 
C	        interpolation.  The partial look up table is specified by the 
C               user through the tae parameters from and to which contain 
C               the 'from' and 'to' coordinates, respectively. From and to
C               each may specify up to 100 values. 
C
C     ARGUMENTS:  
C     Name              Description
C     ----              -----------
C     blut(0:255)       byte look up table
C     from		input pixel values
C     intype            input image data type  
C     ltype             data type of look up table  
C                                  (1=Byte, 2=I*2)
C     lut(-32768:32767)	output look up table
C     otype             output image data type  
C     to		output pixel values
C     npairs  		number of from-to pairs
C
C     INTERNAL VARIABLES:
C     Name	        Description
C     ----              -----------
C     curr		lut value counter
C     from(1000)        input pixel values
C     i			from counter
C     iiii		temporary storage for lut
C     j			to counter
C     lutmax		maximum allowed value of lut
C     lutmin		minimum allowed value of lut
C     msg	        error message text
C     otype		image data type
C     partlut(2,1000)   partial lut consisting of from-to pairs
C     shrtlut(0:255)    byte from word look up table
C     to(1000)          output pixel values
C     
C     **************************************************************

      subroutine mklut(ltype,lut,blut,intype,otype,from,to,npairs)

	include 'las.fin'

      real*8 from(1000)        
      real*8 to(1000)

      integer*4 curr
      integer*4 i	
      integer*4 iiii
      integer*4 intype	
      integer*4 j		
      integer*4 ltype 
      integer*2 lut(-32768:32767)
      integer*4 lutmax	
      integer*4 lutmin
      integer*4 npairs  
      integer*4 otype	
      integer*4 partlut(2,1000)   
      integer*2 shrtlut(0:255)

      character*1 blut(0:257)

      character*(CMLEN) msg

C-----------------------------------------------
C pxlkup.c of applut.f creates ltype data in lut
C-----------------------------------------------
C-----------------------------------------
C move mapping pairs into processing array
C-----------------------------------------
      j = 1
      do 200 i = 1,npairs
         partlut(1,j) = from(i)
         partlut(2,j) = to(i)
         j = j + 1
200   continue
C--------------------------------------------------------------------
C set max and min positions according to data type
C change ltype to input data type here, for choosing array type below
C--------------------------------------------------------------------
       if ((otype .eq. 2) .or. (intype .eq. 2)) then
          ltype = EWORD
          lutmax =  32767
          lutmin = -32768      
      else
          ltype = EBYTE
          lutmax = 255
          lutmin = 0
      end if
C
C----------------------------------------------------
C make sure that the partial lut extends to beginning 
C of range but not beyond
C----------------------------------------------------
      if (partlut(1,1).lt.lutmin) then
	msg = 'First LUT position specified in partial LUT is too small'
	call errmsg(msg,'map-lutsml',1)
	call errmsg('Fatal error encountered','map-fatal',-1)
      else if (partlut(1,1).gt.lutmin) then
        do 400 i = npairs,1,-1
            do 500 j = 1,2
               partlut(j,i+1) = partlut(j,i)
500         continue
400     continue
        partlut(1,1) = lutmin
        partlut(2,1) = partlut(2,2)
        npairs = npairs + 1
      end if
C-------------------------------------------------------
C make sure that the partial lut extends to end of range 
C but not beyond
C-------------------------------------------------------
      if (partlut(1,npairs).gt.lutmax) then
	msg = 'Final LUT position specified in partial LUT is too large'
	call errmsg(msg,'map-lutbig',1)
	call errmsg('Fatal error encountered','map-fatal',-1)
      else if (partlut(1,npairs).lt.lutmax) then
         npairs = npairs + 1
         partlut(1,npairs) = lutmax
         partlut(2,npairs) = partlut(2,npairs-1)
      end if
C---------------------------------------------------------------------
C construct lut by linear interpolation between entries in partial lut
C---------------------------------------------------------------------
      curr = 1   
      do 600 i = lutmin,lutmax
          if (partlut(1,curr+1).lt.i) then   
             curr = curr + 1
          end if
          iiii = nint((float(i - partlut(1,curr)) /
     +                 float(partlut(1,curr + 1) - partlut(1,curr))) *
     +                 (partlut(2,curr + 1) - partlut(2,curr)) +
     +                 partlut(2,curr))
          if (ltype .eq. EBYTE) then
             shrtlut(i) = min(max(iiii,lutmin),lutmax)
          else
             lut(i) = min(max(iiii,lutmin),lutmax)
	  endif
600   continue

C-----------------------------
C convert to byte if necessary
C-----------------------------
      if (ltype .eq. EBYTE) then
         call pxconv(2,1,shrtlut,blut,256)
      end if

      return
      end
