C     ******************************************************************
C     PURPOSE:  PWLIN processes the input image by performing a piece-
C	        wise linear mapping.
C
C     ARGUMENTS:
C     Name	       Description
C     ----	       -----------
C     bands(MAXBND + 1,MAXIMG)
C     	 	       input band specification
C     blut(0:255)      byte look up table
C     ddesc	       file descriptor for output dltf file
C     dltfout	       name of the output dltf file
C     from(1000)       input values to be mapped
C     hosin(MAXIMG)    input host names
C     hosout	       output host name
C     inasdltf(DLTFSZ) input dltf array from r1dltf
C     intype           data type of input image
C     inkey	       key field of record one of input dltf file
C     ldesc	       file descriptor for output lut name
C     ltype	       data type of look up table
C     lut(-32768:32767)look up table
C     minmax	       indicator for whether the user wants the
C		       min and max determined for each band
C     msgtim           how often to display processing message
C     nbands(MAXIMG+1) number of bands in each input image
C     nft	       number of FROM/TO values specified
C     nmrimg	       number of input images
C     numout	       number of names for complt or cleanup
C     off(999)         offsets for transformation
C     otype            data type of output image
C     outkey	       key field of record one of output dltf file
C     parblk(XPRDIM)   tae parameter block
C     rtoff            offset value of 0.5 or 0 for round/trunc of real data
C     sc(999)          scale factors for transformation
C     taeout	       output TAE name
C     to(1000)         output mapping specifications
C     todescend	       flag for denoting that to values are descending
C     totbnd	       total number of bands to be processed
C     window(4,MAXIMG) input window specification
C
C     INTERNAL VARIABLES:
C     Name	       Description
C     ----	       -----------
C     egclse	       imageio function to close all files in a group
C     estep	       imageio function to read/write image lines
C     fdesc	       file descriptor pointing to machine word size
C     i                loop index
C     ib	       band loop index
C     il	       line loop index
C     ip	       pixel loop index
C     j	               loop index
C     lninc	       line incrementation value of one for upddr
C     max(MAXBND+1)    array of real max pixel values for each output band
C     min(MAXBND+1)    array of real min pixel values for each output band
C     nl               number of image lines
C     ns 	       number of image samples
C     pix  	       current pixel value
C     pxmnmx	       function to determine the min and max pixel values
C		       of an input array
C     smpinc	       sample incrementation value of one for upddr
C     status           function return status code
C     svmnmx           function to create/update associated min max file
C     tomax	       maximum to pair value
C     tomin	       minimum to pair value
C
C******************************************************************************

      subroutine pwlin(parblk,hosin,taeout,numout,hosout,window,bands,
     +		       nmrimg,from,to,nft,sc,off,intype,otype,msgtim,
     +		       minmax,nbands,totbnd,lut,blut,ltype,ldesc,
     +		       dltfin,inkey,indescr,inasdltf,indtlen,indtype,
     +		       lutflg,dltfout,outkey,outdescr,rtoff,todescend)

	include 'las.fin'

	include 'dltf.fin'

        include 'typlimsg.fin'

      real*8 from(1000)
      real*8 lninc
      real*8 max(MAXBND+1)
      real*8 min(MAXBND+1)
      real*8 off(999)
      real*8 pix
      real*8 realmin
      real*8 realmax
      real*8 sc(999)
      real*8 smpinc
      real*8 tmppix
      real*8 to(1000)

      real*4 rtoff
      real*4 thebuf(BSIZE)

      integer*4 bands(MAXBND + 1,MAXIMG)
      integer*4 ddesc
      integer*4 eclose
      integer*4 egclse
      integer*4 estep
      integer*4 fdesc(MAXIMG + 1)
      integer*4 fdescin(MAXIMG)
      integer*4 fdescot
      integer*4 i
      integer*4 ib
      integer*4 il
      integer*4 inasdltf(DLTFSZ)
      integer*4 indtlen
      integer*4 intype
      integer*4 ip 
      integer*4 j
      integer*4 ldesc
      integer*4 ltype
      integer*4 lutflg
      integer*4 msgtim
      integer*4 nbands(MAXIMG + 1)
      integer*4 nft
      integer*4 nmrimg
      integer*4 numout
      integer*4 nl
      integer*4 ns
      integer*4 offin(MAXBND + 1)
      integer*4 offout(MAXBND + 1)
      integer*4 otype
      integer*4 parblk(XPRDIM) 
      integer*4 pxmnmx
      integer*4 rdesc
      integer*4 status
      integer*4 svmnmx
      integer*4 todescend
      integer*4 tomax
      integer*4 tomin
      integer*4 totbnd
      integer*4 upddr
      integer*4 window(4,MAXIMG)

      integer*2 lut(-32768:32767)

      character*(CMLEN) dltfin
      character*(CMLEN) dltfout
      character*(CMLEN) hosin(MAXIMG)
      character*(CMLEN) hosout
      character*81      indescr
      character*3       indtype
      character*13      inkey
      character*4 	minmax
      character*81      outdescr
      character*13      outkey
      character*(CMLEN) taeout(2)

      character*1 blut(0:257)
      character*1 inbuf(BSIZE)
      character*1 outbuf(BSIZE)

      common/inbuf/inbuf
      common/outbuf/outbuf
      common/thebuf/thebuf
      common/imgio/fdesc,fdescin,fdescot,rdesc,offin,offout
      common/curent/il
      common/curcat/ib

C     Initialize the min and max arrays
C     ---------------------------------
      call inmnmx(otype, MAXBND, min, max)
C      do 10 i = 1,MAXBND
C         min(i) = MAXLONG
C         max(i) = MINLONG
C 10    continue
	
C     Open the image files 
C     --------------------
      call initio (parblk,totbnd,bands,window,nmrimg,hosin,hosout,
     +		   intype,otype,nbands,ddesc,dltfout,outkey,taeout,
     +		   numout,blut,lut)

      nl = window(3,1)
      ns = window(4,1)

C     A check has been added to clip the output pixel values down to
C     the output to range. If the to pairs are in descending order,
C     this check needs to be reversed. This is the reason for 
C     creating tomin and tomax, instead of always using to(1) for
C     min, and to(nft) for max, as was previously done.
C     -------------------------------------------------------------
C      if (todescend .eq. 1) then
C	 tomin = to(nft)
C	 tomax = to(1)
C      else
C	 tomin = to(1)
C	 tomax = to(nft)
C      endif
C
C     the above check was replaced with pxmnmx to allow any arrangement
C     of to values, and then put inside the real processing loop
C     -------------------------------------------------------------
C      realmin = MAXREAL
C      realmax = MINREAL
C
C      call inmnmx(EREAL,1,realmin,realmax)
C      status = pxmnmx(to,nft,realmin,realmax,EREAL)
C      if (status .ne. ESUCC) then
C      	 call cleanup(parblk,intype,otype,hosout,
C     +		      nmrimg,ldesc,numout,dltfout)
C      endif
C      tomin = realmin
C      tomax = realmax

C     Initialize the display message utility
C     --------------------------------------
      il = 1
      ib = 1

C     Process the image data
C     ----------------------
      if ((intype .gt. 2) .or. (otype .gt. 2)) then

          realmin = MAXREAL
          realmax = MINREAL

          status = pxmnmx(to,nft,realmin,realmax,EDBLE)
          if (status .ne. ESUCC) then
      	     call cleanup(parblk,intype,otype,hosout,
     +		          nmrimg,ldesc,numout,dltfout)
          endif

          tomin = realmin
          tomax = realmax

          call inittm('MAP','Processed','line','bands',' ',nl,totbnd,
     +                msgtim,ALLCAT)
          call startm()
          do 200 il = 1,nl
             status = estep(rdesc)
             if (status .ne. ESUCC) then
      		call cleanup(parblk,intype,otype,hosout,
     +			     nmrimg,ldesc,numout,dltfout)
	     end if
             do 150 ib = 1,totbnd
	        do 100 ip = 1,ns
	           pix = thebuf(offin(ib) + ip)
	           if (pix .le. from(1)) then
		      thebuf(offout(ib) + ip) = to(1)
	           else if (pix .ge. from(nft)) then
	              thebuf(offout(ib) + ip) = to(nft)
	           else
	             do 75 j = 1,(nft - 1)
	                if (pix .eq. from(j)) then
		           thebuf(offout(ib) + ip) = to(j)
	                else if (pix .eq. from(nft)) then
		           thebuf(offout(ib) + ip) = to(nft)
	                else if (pix .gt. from(j) .and.
     +			         pix .lt. from(j + 1)) then
      			   tmppix = (sc(j) * pix) + off(j)
			   if (tmppix .lt. 0) then
			      tmppix = tmppix - rtoff
			   else
			      tmppix = tmppix + rtoff
			   end if
			   if (tmppix .lt. tomin) then
		              thebuf(offout(ib) + ip) = tomin
			   else if (tmppix .gt. tomax) then
		              thebuf(offout(ib) + ip) = tomax
			   else
		              thebuf(offout(ib) + ip) = tmppix
			   end if
	                   go to 80
	                end if
75	             continue
80		     continue
	           end if
100             continue
                if (minmax .eq. 'YES') then
                   status = pxmnmx(thebuf(offout(ib) + 1),ns,
     +	    		           min(ib),max(ib),EREAL)
C    There is a problem here in passing the buffer to the bridge routine,
C    the buffer is correct just before calling the bridge routine but 
C    is not correct in the bridge routine.  This causes the min and max
C    to be calculated incorrectly.  May need to rewrite in C to correct 
C    problem.
                end if
150          continue
200       continue
      else
          call inittm('MAP','Processed','line','bands',' ',nl,totbnd,
     +                msgtim,ONECAT)
          call startm()
          call applut(parblk,hosin,hosout,numout,
     +		      lut,blut,ltype,ldesc,dltfin,inasdltf,inkey,
     +		      indtlen,indtype,lutflg,ddesc,dltfout,outkey,
     +		      outdescr,nmrimg,bands,nbands,totbnd,window,
     +		      intype,otype,minmax,min,max)
      end if

C     Flush the buffer
C     ----------------
      if ((intype .gt. 2) .or. (otype .gt. 2)) then
         status = estep(rdesc)
         if (status .ne. ESUCC) then
            call cleanup(parblk,intype,otype,hosout,
     +		         nmrimg,ldesc,numout,dltfout)
         end if
      end if

C     Display the final processing message
C     ------------------------------------
      call stoptm 

C     Close the output image
C     ----------------------
      if ((intype .gt. 2) .or. (otype .gt. 2)) then
         status = egclse(rdesc)
         if (status .ne. ESUCC) then
	    call errmsg
     +      ('Nonfatal error encountered','map-warn',1)
         end if
      else
	 do 300 i = 1, nmrimg
            status = eclose(fdescin(i))
            if (status .ne. ESUCC) then
	       call errmsg
     +         ('Nonfatal error encountered','map-warn',1)
            end if
300	 continue
         status = eclose(fdescot)
         if (status .ne. ESUCC) then
	    call errmsg
     +      ('Nonfatal error encountered','map-warn',1)
         end if
      end if

C     Save ddr information
C     --------------------
      lninc = 1.0
      smpinc = 1.0
      status = upddr(parblk,hosin,hosout,nmrimg,window,bands,nbands,
     +		     lninc,smpinc,totbnd,NOCOMB)
      if (status .ne. ESUCC) then
         call errmsg('Nonfatal error encountered','map-warn',1)
      end if

C     Create the min max file of the output image
C     -------------------------------------------
      if (minmax .eq. 'YES') then
         status = svmnmx(hosout,totbnd,min,max,minmax)
         if (status .ne. ESUCC) then
            call errmsg('Nonfatal error encountered','map-warn',1)
         end if
      end if

      return
      end
