/*******************************************************************************
NAME:   			RDFIL 

PURPOSE:    To read a text file into an array of from - to pairs.   

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   unknown  	   unknown 	   		Original Development
 
  5.0	   B. Davis        12/88	  EDC		conversion to 5.0,
							fixed bugs, included
							error messages

ALGORITHM:
   Get the file name
   Open the file
   Check for from-to to be in pairs
   Check for the max of 1000 pairs
  
   ALGORITHM REFERENCES:		none
*******************************************************************************/
#include <stdio.h>
#include <sys/file.h>
#include <string.h>
#include "las.h"
#include "map.h"

#define MAX_LEN 100
#define MAX_PTS 1000


void rdfil
(
    char  *fname,                       /* host file name               */
    double *from,                       /* pointer to from array        */
    double *to,                         /* pointer to to array          */
    long  *npts                         /* pointer to number of pairs   */
)
{
FILE  *fp;				/* file pointer for text file	*/

double extra;				/* extra			*/

float tmpfrom;				/* for conversion from buf to dbl*/
float tmpto;				/* for conversion from buf to dbl*/

long  length;				/* name length for squeeze	*/
long  negone = -1;			/* fatal flag			*/
long  one = 1;				/* nonfatal flag		*/

char  buf[MAX_LEN];			/* buffer for read		*/
char  *ptr;                             /* to squeeze names for FORTRAN */

length = (long)strlen(fname);
ptr = squeeze(fname,length);
if ((fp = fopen(ptr,"r")) == NULL)
   {
   c_errmsg("Error opening map point file","map-open",&one);
   c_errmsg("Fatal error encountered","map-fatal",&negone);
   }
for (*npts=0;;from++,to++,(*npts)++)
   {
   if (fgets(buf,MAX_LEN,fp) == NULL)
      break;
   if (sscanf(buf,"%f %f %lf",&tmpfrom,&tmpto,&extra) != 2)
      {
      fclose(fp);
      c_errmsg("FROM and TO values specified in text file","map-pairs",&one);
      c_errmsg(" must be in pairs","map-pairs",&one);
      c_errmsg("Fatal error encountered","map-fatal",&negone);
      }
   *from = (double)tmpfrom;
   *to = (double)tmpto;
   }
fclose(fp);
return;
}


void rdfil_
(
    char  *fname,
    double *from,
    double *to,
    long  *npts
)
{
  rdfil(fname,from,to,npts);
}
