/*****************************************************************************
FUNCTION: crefile

PURPOSE:  write out the TO/FROM pairs to the output file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT:        LAS

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "protos.h"

int crefile
(
    char *hosout,                    /* output host file name (TAE)    */
    long percnt,                     /* # of percentages to be matched */
    long to[],                       /* values written to TO column    */
    long from[],                     /* values written to FROM column  */
    long pixflg                     /* pixel flag                     */
)

{
  long i=0;                      /* counter variable               */
  long tempin=0;                 /* temporary variable             */
  long fatal=-1;                 /* error message status           */
  long status=0;                 /* status of function             */

  char houtf[CMLEN];             /* output host file name (UNIX)   */
  char errmsg[CMLEN];            /* contains error messages        */

  FILE *tmpfp;              /* file pointer to output file    */
  FILE *chk;


/* Changing path format from LAS to UNIX
----------------------------------------*/
  status = c_cmhost(hosout,houtf);
  if (status != E_SUCC)
  {
    c_errmsg("Error in retrieving host name","mapfile-host",&fatal);
    return(E_FAIL);
  }

/* Check if the file already exists
-----------------------------------*/
  if ((chk = fopen(houtf,"r")) != NULL)
  {
    fclose(chk);
    sprintf(errmsg,"outfile %s already exists",hosout);
    c_errmsg(errmsg,"mapfile-exists",&fatal);
    return(E_FAIL);
  }

/* Opening the output file for writing
--------------------------------------*/
  if ((tmpfp = fopen(houtf,"w")) != NULL)
  {
    tempin = from[0];
    if (pixflg)
    {
      for (i = 0; i <= percnt; i++)                    /* writing the data */
        fprintf(tmpfp,"%10d%10d\n",from[i],to[i]);
    }
    else
    {
      for (i = 0; i < percnt; i++)                 /* writing the data */
      {
        if ((tempin != from[i]) || (i == 0))
        {
          fprintf(tmpfp,"%10d%10d\n",from[i],to[i]);
          tempin = from[i];
        }
      }
    }
  }
  else
  {
    sprintf(errmsg,"Error creating output file %s",houtf);
    c_errmsg(errmsg,"mapfile-create",&fatal);
    return(E_FAIL);
  }
  fclose(tmpfp);

  return(E_SUCC);
}
