/*****************************************************************************
FUNCTION:   getdata

PURPOSE:    reads in the pixel values and accumulative percentages from
            infile1 and infile2

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT:        LAS

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int getdata
(
    char *infile1,                 /* input host file name      */
    char *infile2,                 /* input host file name      */
    float cper1[],                 /* percentages of INFILE1    */
    float cper2[],                 /* percentages of INFILE2    */
    long val1[],                   /* values of INFILE1         */
    long val2[],                   /* values of INFILE2         */
    long *nval1,                   /* # of values of INFILE1    */
    long *nval2,                   /* # of values of INFILE2    */
    long ctrl                     /* do I read INFILE2         */
)

{
  long i=0;                    /* a counter variable        */
  long status=0;               /* status of func. execution */
  long pixvals=0;              /* pixval from the infiles   */
  long temp3=0;                /* uneeded data from infiles */
  long fatal=-1;               /* error message variable    */
  long vinc1=0;                /* counter for *nval1 above  */
  long vinc2=0;                /* counter for *nval2 above  */

  float temp2=0.0;             /* uneeded data from infiles */
  float cumper=0.0;            /* cum.percnt from infiles   */

  char hname1[CMLEN];          /* contains the host name    */
  char hname2[CMLEN];          /* contains the host name    */
  char temp[BUFSIZ];            /* strings from infile       */
  char errmsg[CMLEN];          /* contains error messages   */

  FILE *tmpfp1;                /* file pointer to INFILE1   */
  FILE *tmpfp2;                /* file pointer to INFILE2   */

  
  /* Changing the format of the path from LAS to UNIX
  ---------------------------------------------------*/
  status = c_cmhost(infile1,hname1);
  if (status != E_SUCC)
  {
    c_errmsg("Error in retrieving host name","mapfile-nohost",&fatal);
    return(E_FAIL);
  }

/* Open INFILE1 for reading
---------------------------*/
  if ((tmpfp1 = fopen(hname1,"r")) != NULL)
  {
  /* Skip first 8 lines of INFILE1
  --------------------------------*/
    for (i = 0; i < 8; i++)
      fgets(temp,BUFSIZ,tmpfp1);

  /* Read the data from INFILE1
  -----------------------------*/
    for (i = 0; i < 32767; i++)
    {
      fscanf(tmpfp1,"%d%d%f%f%s",&pixvals,&temp3,&temp2,&cumper,temp);
      val1[i] = pixvals;
      cper1[i] = cumper;
      vinc1++;

      if ((val1[i] == 0) && (val1[i-1] == 0) && (i > 1))
      {
         vinc1 = vinc1 - 2;
         break;
      }
    }
    *nval1 = vinc1;
    fclose(tmpfp1);
  }
  else
  {
    sprintf(errmsg,"Error reading %s file",hname1);
    c_errmsg(errmsg,"mapfile-read",&fatal);
    return(E_FAIL);
  }

  if (ctrl == 0)
  {
    /* Changing the format of the path from LAS to UNIX
    ---------------------------------------------------*/
    status = c_cmhost(infile2,hname2);
    if (status != E_SUCC)
    {
      c_errmsg("Error in retrieving host name","mapfile-nohost",&fatal);
      return(E_FAIL);
    }
  
    /* Open the INFILE2 for reading
    -------------------------------*/
    if ((tmpfp2 = fopen(hname2,"r")) != NULL)
    {
      /* Skip the first 7 lines of INFILE2
      ------------------------------------*/
      for (i = 0; i < 8; i++)
        fgets(temp,BUFSIZ,tmpfp2);

      /* Read the data from INFILE2
      -----------------------------*/
      for (i = 0; i < 32767; i++)
      {
        fscanf(tmpfp2,"%d%d%f%f%s",&pixvals,&temp3,&temp2,&cumper,temp);
        val2[i] = pixvals;
        cper2[i] = cumper;
        vinc2++;
        if ((val2[i] == 0) && (val2[i-1] == 0) && (i > 1))
        {
           vinc2 = vinc2 - 2;
           break;
        }
      }
      *nval2 = vinc2;
      fclose(tmpfp2);
    }
    else
    {
      sprintf(errmsg,"Error reading %s file",hname2);
      c_errmsg(errmsg,"mapfile-read",&fatal);
      return(E_FAIL);
    }
  }  /* end if ctrl == 0 */

  return(E_SUCC);
}  /* end getdata */
