/***************************************************************************
FUNCTION:    getpar

PURPOSE:     reads in the values from the TAE parameters

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATION:
	      Run under TAE

PROJECT:      LAS

***************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,                  /* TAE parameter block         */
    char *hosout,                           /* output host filename        */
    float *pixval,                          /* value of parameter PIXVAL   */
    long *percnt,                           /* # of input percentages      */
    float per[],                            /* percentages to match        */
    char *subcmd,                           /* value of parameter SUBCMD   */
    long *ctrl,                             /* user controls TO column Y/N */
    long *tovalcnt,                         /* # of values in TO column    */
    long to[],                              /* values for the TO column    */
    char *infile1,                          /* input filename              */
    char *infile2,                          /* input filename              */
    long *pixflg                           /* is there a pixval Y/N       */
)

{
  struct VARIABLE *in;                  /* pointer to parameters       */
  long i=0;                             /* counter variable            */
  long fatal=-1;                        /* used in error message calls */
  char *p;                              /* used to make subcmd all caps*/
  char errmsg[CMLEN];                   /* error message               */


  /* Getting the names of the input file(s)
  -----------------------------------------*/
  in = p_find(vblock,"INFILE");
  strcpy(infile1,SVAL(*in,0));
  p = (char *)strrchr(infile1,';');
  if (p == NULL)
    strcat(infile1,".prt");

  /* Getting outfile name
  -----------------------*/
  in = p_find(vblock,"OUTFILE");
  strcpy(hosout, SVAL(*in, 0));

  /* Getting the pixel value
  --------------------------*/
  in = p_find(vblock,"PIXVAL");
  *pixval = RVAL(*in,0);

  /* Getting the percentages to match to
  --------------------------------------*/
  in = p_find(vblock,"MAPPERC");
  *percnt = in->v_count;
  for(i = 0; i < *percnt; i++)
    per[i] = RVAL(*in,i);

  for(i = 0; i < *percnt-1; i++)
  {
    if (per[i] > per[i+1])
    {
      c_errmsg("Percentages not in ascending order","mapfile-ascend",&fatal);
      return(E_FAIL);
    }
  }
  if ((*pixval == 0.0) && (per[0] == 0.0))
    *pixflg = 0;


  /* Getting the subcommand
  -------------------------*/
  in = p_find(vblock,"_SUBCMD");
  strcpy(subcmd,SVAL(*in,0));
  for(i = 0, p = subcmd; i < strlen(subcmd); i++,p++)
    if (islower(*p))
      *p = toupper(*p);

  if (strcmp(subcmd,"CTRL") == 0)
    *ctrl = 1;

  if (*ctrl == 1)
  {
    in = p_find(vblock,"OUTVAL");
    *tovalcnt = in->v_count;
    if (*tovalcnt != *percnt)
    {
      strcpy(errmsg,"Number of values in OUTVAL and MAPPERC are not equal");
      c_errmsg(errmsg,"mapfile-notequal",&fatal);
      return(E_FAIL);
    }

    /* if subcommand is ctrl, get the users input values
    ----------------------------------------------------*/
    for(i = 0; i < *tovalcnt; i++)
      to[i] = (long) RVAL(*in,i);

    for(i = 0; i < *tovalcnt-1; i++)
    {
      if (to[i] > to[i+1])
      {
       c_errmsg("Percentages not in ascending order","mapfile-ascend",&fatal);
       return(E_FAIL);
      }
    }
  }
  else
  {
    /* Read in second file name
    ---------------------------*/
    in = p_find(vblock,"INFILE");
    strcpy(infile2,SVAL(*in,1));
    p = (char *)strrchr(infile2,';');
    if (p == NULL)
      strcat(infile2,".prt");
  }
  return(E_SUCC);
}
