/*****************************************************************************
FUNCTION:      mapfile

PURPOSE:       reads two histogram files and calculates the TO and FROM values
               needed by MAP based on the percentages given in the parameter 
               MAPPERC.  The TO and FROM values are written to a file that MAP
               can read.

PROGRAM HISTORY:
   Version         Date    Author         Code       Request
   -------         ----    ------         ----       -------
    1.0            1/87    B.Ailts        CSB        Orig. development
    1.1            3/88    B.Ailts        CSB        Made changes for the
                                                     changes that were made to
                                                     PIXCOUNT -- 4528
    1.2            8/88    B.Ailts        CSB        Allow for negative values
                                                     prf 4634    able to null
                                                     out mask value
    1.3           12/88    B.Ailts        CSB        LAS 5.0 conversion
    1.4            5/89    B.Ailts        CSB        Now only outputs one value
                                                     if the value of PIXVAL is
                                                     already in the mapping
                                                     pair
    1.5            07/97   T.Ratliff      CSB        Rewritten into C and 
						     added functionality to
						     allow user to specify
						     values.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT       LAS

******************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
  struct PARBLK vblock;                 /* TAE parameter block             */
  struct VARIABLE *p_find();            /* function to find variables      */

  long numval1=0;                       /* # of values read from INFILE1   */
  long numval2=0;                       /* # of values read from INFILE2   */
  long percnt=0;                        /* # of input percentages          */
  long status=0;                        /* status of function calls        */
  long pixflg=1;                        /* flag if there is a pixel value  */
  long one=1;                           /* zero for complt call            */
  long fatal=-1;                        /* used in error message calls     */
  long stdlun=0;                        /* unit # of std output            */
  long ctrl=0;                          /* user controls TO column (Y/N)   */
  long tovalcnt=0;                      /* # of values in TO column        */
  long value1[32767];                   /* value w/ each percnt of INFILE1 */
  long value2[32767];                   /* value w/ each percnt of INFILE2 */
  long from[100];                       /* values for FROM column          */
  long to[100];                         /* values for TO column            */
  long pixval=0;                        /* value of parameter PIXVAL       */

  float tmp_pixval = 0.0;
  float comper1[32767];                 /* cumul. percentages of INFILE1   */
  float comper2[32767];                 /* cumul. percentages of INFILE2   */
  float per[100];                       /* percentages to match            */

  char infile1[CMLEN];                  /* input filename                  */
  char infile2[CMLEN];                  /* input filename                  */
  char hosout[1][CMLEN];                   /* output host filename            */
  char subcmd[CMLEN];                   /* either ctrl or noctrl           */
  char errmsg[CMLEN];                   /* contains error messages         */
  char *p;


  /* Initialization
  -----------------*/
  c_inlas(&vblock,&stdlun);

  /* Getting parameter values
  ---------------------------*/
  status = getpar(&vblock,hosout[0],&tmp_pixval,&percnt,per,subcmd,&ctrl,
                  &tovalcnt, to, infile1,infile2,&pixflg);
  if (status != E_SUCC)
  {
    c_errmsg("Fatal error encountered","mapfile-fatal",&fatal);
    return EXIT_FAILURE;
  }
  pixval = (long) tmp_pixval;

  /* Reading in the data from the pixcount files
  ----------------------------------------------*/
  status = getdata(infile1,infile2,comper1,comper2,value1,value2,&numval1,
                   &numval2,ctrl);

  if (status != E_SUCC)
  {
    c_errmsg("Fatal error encountered","mapfile-fatal",&fatal);
    return EXIT_FAILURE;
  }

  if (ctrl == 0)
    status = procnoctrl(percnt,&pixflg,numval1,numval2,per,comper1,comper2,
                        value1,value2,to,from,pixval);
  else if (ctrl == 1)
    status = procctrl(percnt,&pixflg,numval1,per,comper1,value1,from,to,
                      pixval);

  if (status != E_SUCC)
  {
    c_errmsg("Fatal error encountered","mapfile-fatal",&fatal);
    return EXIT_FAILURE;
  }

  p = (char *)strchr(hosout[0],';');
  if (p == NULL)
    strcat(hosout[0],";txt");
  status = crefile(hosout[0],percnt,to,from,pixflg);

  if (status != E_SUCC)
  {
    c_errmsg("Fatal error encountered","mapfile-fatal",&fatal);
    return EXIT_FAILURE;
  }

  if (status != E_SUCC)
  {
    strcpy(errmsg,"Error in creating TAE format output file name");
    c_errmsg(errmsg,"mapfile-tae",&fatal);
    c_errmsg("Fatal error encountered","mapfile-fatal",&fatal);
  }
  c_complt("MAPFILE",hosout,&one);

  return EXIT_SUCCESS;

}  /* end of mapfile */
