/*****************************************************************************
FUNCTION:   procnoctrl

PURPOSE:    compares the percentages of the input files with the input
            percentages in order to calculate the output FROM/TO values.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                Run under TAE

PROJECT:        LAS

******************************************************************************/

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "protos.h"

int procnoctrl
(
    long percnt,                  /* # of percentages to be matched      */
    long *pixflg,                 /* flag if there is a pixel value      */
    long numval1,                 /* # of values of INFILE1              */
    long numval2,                 /* # of values of INFILE2              */
    float per[],                  /* percentages to be matched           */
    float comper1[],              /* percentages of INFILE1              */
    float comper2[],              /* percentages of INFILE2              */
    long value1[],                /* values of INFILE1                   */
    long value2[],                /* values of INFILE2                   */
    long to[],                    /* values of TO column of output file  */
    long from[],                 /* values of FROM column of output file */
    long pixval
)

{
  long valcnt1=0;             /* current value position of INFILE1   */
  long valcnt2=0;             /* current value position of INFILE2   */
  long pflg=0;                /* temp variable for pixflg            */
  long compare1=0;            /* logical if difference of two percnt  
                                 is equal or not of INFILE1          */
  long compare2=0;            /* logical if difference of two percnt 
                                 is equal or not of INFILE2          */
  long i=0;                   /* counter variable in for loops       */
  long j=0;                   /* counter variable in for loops       */
  long k=0;                   /* counter variable in for loops       */
  long start=0;               /* what percent to start matching at   */
  float back=0.0;             /* result of fabs(per[i] - comperX[valcntX-1]) */
  float next=0.0;             /* result of fabs(per[i] - comperX[valcntX+1]) */
  float curr=0.0;             /* result of fabs(per[i] - comperX[valcntX]) */


/* Use the lowest value if percent = 0 inorder that the low end of the 
   histogram is not ignored.
----------------------------------------------------------------------*/
  if (per[0] == 0)
  {
    from[0] = value1[0];
    to[0] = value2[0];
    valcnt1++;
    valcnt2++;
    start = 1;
  }

/* Compare the input percentages with the percentages from the input files
   use the FROM/TO values that correspond with closest percentage
--------------------------------------------------------------------------*/
  for (i = start; i < percnt; i++)
  {
    compare1 = FALSE;
    compare2 = FALSE;

    for ( j = valcnt1; j < numval1; j++)
    {
      curr = fabs(per[i] - comper1[valcnt1]);
      next = fabs(per[i] - comper1[valcnt1+1]);
      if (curr < next) 
      {
        back = fabs(per[i] - comper1[valcnt1-1]);
        if (curr == back)
        {
          compare1 = TRUE;
          break;
        }
        else
        {
          from[i] = value1[valcnt1];
          break;
        }
      }
      else
        valcnt1++;
    }

    for (j = valcnt2; j < numval2; j++)
    {
      curr = fabs(per[i] - comper2[valcnt2]);
      next = fabs(per[i] - comper2[valcnt2+1]);
      if (curr < next)
      {
        back = fabs(per[i] - comper2[valcnt2-1]);
        if (curr == back)
        {
          compare2 = TRUE;     
          break;
        }
        else
        {
          from[i] = value2[valcnt2];
          break;
        }
      }
      else
        valcnt2++;
    }

/* If the difference between two actual percentages and the percent needed
   is equal use the one closest to percent found in the second histogram.
--------------------------------------------------------------------------*/
    if ((compare1) && (!compare2))
    {
      if (fabs(comper1[valcnt1] - comper2[valcnt2]) <=
          fabs(comper1[valcnt1+1] - comper2[valcnt2]))
        from[i] = value1[valcnt1];
      else
        from[i] = value1[valcnt1+1];
    }
    else
      from[i] = value1[valcnt1];

/* If the difference between two actual percentages and the percent needed
   is equal use the one closest to percent found in the first histogram.
--------------------------------------------------------------------------*/
    if ((compare2) && (!compare1)) 
    {
      if (fabs(comper2[valcnt2] - comper1[valcnt1]) <=
          fabs(comper2[valcnt2+1] - comper1[valcnt1]))
        to[i] = value2[valcnt2];
      else
        to[i] = value2[valcnt2 + 1];
    }
    else
      to[i] = value2[valcnt2];

  } /* end for i = start to percnt */


/* Place PIXVAL within the correct postion of the TO and FROM columns.
   If from equals the pixval set the to value to pixval.
----------------------------------------------------------------------*/
  pflg = *pixflg;
  if (pflg)
  {
    for (i = 0; i < percnt; i++)
    {
      if (from[i] == pixval)
      {
        to[i] = pixval;
        pflg = FALSE;
        break;
      }
      else if (from[i] > pixval)
      {
        for ( k = percnt; k > i; k--)
        {
          from[k] = from[k-1];
          to[k] = to[k-1];
        }
        from[i] = pixval;
        to[i] = pixval;
        break;
      }
    }
  *pixflg = pflg;
  }
  return(E_SUCC);
}
