/******************************************************************************
FUNCTION:	MAP_GETHOST

PURPOSE:	Get the host file name of a specified LAS file name and
		pass the names back to the TAE parameter block.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
 5.0       06/88    D. Hollaren		initial development
 5.1       05/91    D. Etrheim          Correct call to z_exit
 5.2       01/94    D. Etrheim		Rename to avoid conflict with another
					routine

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
		Run under TAE

PROJECT			LAS

ALGORITHM 
  Initialize LAS
  Get the host file name for the input file
  Set the array of pointers to the host file name
  Write the host file name to the variable block
  Terminate the program without a completion message

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>		/* C standard I/O include file         */
#include "las.h"		/* LAS include			       */

main()
{
struct PARBLK   vblock;		/* TAE parameter block			      */

struct VARIABLE *p_find();	/* function to find TAE variables	      */
struct VARIABLE *val;		/* pointer to TAE variable		      */

char prog[11];			/* Program exit string			      */
char imageflg[4];		/* Flag for LAS or HOST input file name       */
char infile[CMLEN];		/* Input file name			      */
char outfil[CMLEN];		/* Input file name with aliases resolved      */
char hname[CMLEN];		/* Host file name			      */
char msg[ERRLEN];		/* Error message string			      */
char *host[MAXFILES];		/* Array of pointers to host file name        */
char file_array[MAXFILES][CMLEN];/* Array of file names to be output	      */
char *strchr();

long i;				/* Looping variable			      */
long one = 1;			/* value of one				      */
long fatal = -1;		/* Fatal error code			      */
long nonfatal = 1;		/* Non-fatal error code			      */
long element = 0;		/* INFILE element number to get		      */
long access = IREAD;		/* Want read access to print file	      */
long maxfiles = MAXFILES;	/* Maximum number of directory files	      */
long nfiles;			/* Number of direcory files returned	      */
long status;			/* Status variable			      */
long std_out;			/* standard output device 		      */

/*
  Initialize LAS
*/
c_inlas(&vblock,&std_out);

/*
  Retrieve parameter input file parameter
*/
val = p_find(&vblock,"infile");
strcpy(infile,SVAL(*val,0));

/*
  Solve any aliases that may have been entered
*/
status = c_getalias(infile,outfil);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","mapper-fatal",&fatal);

/*
  Retrieve the image flag parameter which determines if the name entered
  is a LAS or HOST file name
*/
val = p_find(&vblock,"imageflg");
strcpy(imageflg,SVAL(*val,0));

/*
  If the input file name is LAS name get the HOST file name
*/
if (strcmp(imageflg,"YES") == 0)
    {
    /*
      Check to see if there is any wildcards or if the user specified
      the extension.  Otherwise default to an extension of "IMG"
    */
    if ((strchr(outfil,'*') == 0) && (strchr(outfil,';') == 0))
	strcat(outfil,";img\0");
	
    /*
      Get the host file name
    */
    status = c_cmhost(outfil,hname);
    if (status != E_SUCC)
	c_errmsg("Fatal error encountered","mapper-fatal",&fatal);
    }
else
    strcpy(hname,outfil);

/*
  Solve any wildcarding
*/
status = dirlist(" ",hname,file_array,&nfiles,maxfiles,one);

/*
  If no files were found print an error message
*/
if (nfiles == 0)
    {
    if (strcmp(imageflg,"YES") == 0)
	sprintf(msg,"No file(s) found for specified LAS  name %s",outfil);
    else
	sprintf(msg,"No file(s) found for specified HOST name %s",outfil);

    c_errmsg(msg,"mapper-found",&nonfatal);
    c_errmsg("Fatal error encountered","mapper-fatal",&fatal);
    }

/*
  Get the addresses of the file names
*/
for (i = 0; i < nfiles; i++)
    host[i] = file_array[i];

/*
  Output the host file name(s) to the variable block
*/
q_init(&vblock,P_BYTES,P_ABORT);
q_string(&vblock,"OUTFILE",nfiles,host,P_ADD);
q_out(&vblock);

/*
  Terminate the program without a completion message
*/
strcpy(prog," ");
z_exit(one,prog);
}
