/*******************************************************************************
PROGRAM:			get_elevation

PURPOSE:  Gets elevation of a digitized point

PROGRAM HISTORY:
	1.0 L. Huewe, USGS/EROS Data Center, 3/89 (ptelev)
	1.1 D. Steinwand                     4/89 Modified for use with MAPPTS

ALGORITHM:
	If no prompting, zero elevation value and return
	Prompt for elevation until satisfactory input has been given
	   get input
	   check value
	   get units specified
	   check units specified
	Convert feet to meters if feet was specified
	Return the elevation

ALGORITHM REFERENCES:	None
*******************************************************************************/
#include <ctype.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "util.h"
#include "protos.h"

#define  FT2M	.3048		/* Feet to meters conversion factor */

double get_elevation
(
    long flag			/* Boolean flag--prompt for elevation? */
)
{
static char defunit[7] = {"meters"}; /* Default elevation units */

double  elev;		/* Elevation */
long	first = TRUE,	/* First time through the loop		      */
	i,		/* Looping variable			      */
	seven = 7,	/* Constant seven			      */
	units_flg,	/* Indicates whether the user specified units */
	units_ok;	/* Whether valid elevation units were entered */

char	istring[81],	/* User response to verification prompt	      */
	text[81],	/* Text to be printed			      */
	units[7];	/* Elevation units			      */

int term;       /* input string terminator  */

/* Check the prompt flag.  If set to no prompting, return an elevation of
   zero.  Otherwise, get the elevation & units
  -------------------------------------------*/
if (flag == FALSE) return(0.0);

/* Get the elevation value
  -----------------------*/
first = TRUE;
do 
    {
    units_ok = TRUE;
    units_flg = FALSE;
    if (!first)
	{
	t_bell();
	t_write("Invalid elevation specification", T_STDCC);
	}
    sprintf(text, "Enter elevation followed by units (default %s)> ",
        defunit);
    t_write(text, T_PROMPT);
    t_read(istring, &term);

/* If nothing was entered, set the elevation to zero.
  -------------------------------------------------*/
    if (*istring == '\0')
	elev = 0.0;
    else
	{
        sscanf(istring,"%lf %s",&elev,units);
	c_up2low(units,&seven);
	}

/* Determine the units specified.  If no value was entered for units, use the 
   default.  If the units entered was not meters or feet, flag an error.
   If a valid value was entered for units, save it as the default for the
   next elevation.
  ---------------*/  
    for (i = 0; i < strlen(istring); i++)
	if ((isdigit(istring[i]) == 0) && (isspace(istring[i]) == 0))
	    {
	    units_flg = TRUE;
	    break;
	    }
    if (!units_flg)
	strcpy(units,defunit);
    else if ((strcmp(units,"meters") != 0) && 
	(strcmp(units,"feet") != 0) && (strcmp(units,"ft") != 0) &&
	(strcmp(units,"m") != 0)) 
	units_ok = FALSE;
    else
	strcpy(defunit,units);
    first = FALSE;
    }
while (!units_ok);

/* Convert from feet to meters.
  ----------------------------*/
if(((strcmp(units,"feet") == 0) || (strcmp(units,"ft") == 0)) && (elev != 0.0))
    elev = elev * FT2M;
return(elev);
}
