/****************************************************************************
NAME:				get_pt_id

PURPOSE:  Gets a point identifier from the user

PROGRAM HISTORY:
	1.0 D. Steinwand USGS/EROS Data Center, 4/89

ALGORITHM DESCRIPTION:
	Increment the point (prompt) counter
	Return default pt id if no prompting
	Do unit statisfactory input has been given:
	   Print prompt
	   Get the input back from the user
	   If input is valid, return
	   If input is not valid, prompt user to re-enter invalid input
*****************************************************************************/
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "geompak.h"
#include "protos.h"

void get_pt_id
(
    long flag,		/* Boolean flag--prompt for pt id's?  */
    char *pt_id
)
{
TEXT mess[80];		/* Buffer for user messages and input */
CODE term;		/* Terminal flag for t_io */
static long pt_num = 0;	/* Prompt number (counter) */
char def_pt_id[20];


/* If no prompting, return the default point id
  --------------------------------------------*/
if (!flag)
  for (;;)
   {
   pt_num++;
   sprintf(pt_id,"tp_%d",pt_num);
   if (c_ckptid(pt_id)) return;
   } 

/* Do until satisfactory user input is given
  -----------------------------------------*/
for (;;)
   {
   pt_num++;
   sprintf(def_pt_id,"tp_%d",pt_num);

/* Prompt user
  -----------*/
#ifdef vms
   t_write(" ",T_STDCC);
#endif
   sprintf(mess,"Enter point id [%s] >  ",def_pt_id);
   t_write(mess,T_NULL);

/* Read user input
  ---------------*/
   t_read(mess, &term);
   
/* Check for end of entry
  ----------------------*/
   if (strlen(mess) == 0)
      {
      strcpy(pt_id, def_pt_id);
      if (c_ckptid(pt_id)) return;
      t_bell();
      t_write(" Point id is not unique; re-enter!", T_STDCC);
      } 

/* For the given units, unpack the fields and check validity.  Break out of
   loop if satisfactory input is given
  -----------------------------------*/
   else
      {
      sscanf(mess, "%s", pt_id);
      if (c_ckptid(pt_id)) return;
      t_bell();
      t_write(" Point id is not unique; re-enter!", T_STDCC);
      }
   }
}
