/****************************************************************************
NAME:				getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		4/89	D. Steinwand	   CSB      New development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Get the input tie point location file name
	Get the output tie point location file name
	Get the output map identifier file name (lt)
	Get the elevation flag
	Get the point id flag
	Return
*****************************************************************************/
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblk,	/* TAE variable block */
    char *in_tps,		/* Input tie point selection file name */
    char *out_tps,		/* Output tie point selection file name */
    char *mapfile,		/* Name of map parameter file */
    long *elev_flag,	/* Boolean flag--are elevations prompted for? */
    long *pt_id_flag	/* Boolean flag--are point ids prompted for? */
)
{
struct VARIABLE *var;		/* pointer to a TAE variable in vblk */
long acc;			/* access flag for c_getnam */
long zero = 0;			/* Constant of zero */

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INTS", &zero, in_tps, "tps", &acc) != E_SUCC)
	return(E_FAIL);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(vblk, "OUTTS", &zero, out_tps, "tps", &acc) != E_SUCC)
	return(E_FAIL);

/* Get the output map identifier file name
  ---------------------------------------*/
acc = IUPDATE;
if(c_getnam(vblk, "INLT", &zero, mapfile, "lt", &acc) != E_SUCC)
	return(E_FAIL);

/* Get the elevation flag
  ----------------------*/
var = p_find(vblk, "ELEVFLG");
if (strcmp(SVAL(*var, 0), "YES") == 0) *elev_flag = TRUE;
else *elev_flag = FALSE;

/* Get the Point id flag
  ---------------------*/
var = p_find(vblk, "PNTFLG");
if (strcmp(SVAL(*var, 0), "YES") == 0) *pt_id_flag = TRUE;
else *pt_id_flag = FALSE;

return(E_SUCC);
}
