#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

/******************************************************************************

NAME: maskstat

FUNCTION: calculate statistics for spectral image from classified image

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
   1.0        20-jan-83  c.gurney/sasc
   2.0        12-feb-88  m.mazzella/SAR/GSFC convert to C
   3.0         6-feb-89  k. narayanan        accomodate support library 
                                             changes and fix handling
                                             multiple images
   4           8-NOV-89  P. Pease            Change call from stoptm to
                                             c_stoptm in MSTATS.C.
   5.0	      23-Sep-92	 T. Mittan	     Modified include file to 
					     eliminate warnings on link
					     Also changed INCLASS to MASKIMG

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):
	get params from tae
	open files
	calculate stats and output to file
	print out completion message
******************************************************************************/

FILE *stfd;			/* stats file				*/
long offc[MAXIMG];		/* offset into image i/o buffer 	*/
long offs[MAXIMG];		/* offset into image i/o buffer 	*/
struct FDESC * fdescc[MAXIMG * MAXBND];	/* file descriptor array		*/
struct FDESC * fdescs[MAXIMG * MAXBND];	/* file descriptor array		*/
struct GDESC * gdescc;			/* group descriptor			*/
struct GDESC * gdescs;			/* group descriptor			*/
unsigned char *bufc;		/* image i/o buffer			*/
unsigned char *bufs;		/* image i/o buffer			*/
long    *maxv_buf;
long    *hist_buf;
double  *sum_buf;
double  *scov_buf;
double  *mean_buf;


int main
(void) /* maskstat */
{
/*
	main driver for maskstat
*/

char	hosin[MAXIMG][CMLEN];	/* host name of spectral image		*/
char	hoscl[1][CMLEN];	        /* host name of classified image	*/
char	hostat[CMLEN];		/* host name of stats file		*/
char	lastat[1][CMLEN];		/* las name of stats file		*/
long	bandscl[1][MAXBND+1];	/* band numbers of classified image	*/
static long	bands[MAXIMG][MAXBND+1];/* band numbers of spectral image	*/
long	dtype;			/* data type of image			*/
long	inbands;		/* total number of bands                */
long	nclass;			/* number of classes in class. image	*/
long	one = 1;		/* 1					*/
long	std_out;		/* standard output for inlas		*/
long	msgtim;			/* timed processing message delay	*/
static long	windin[MAXIMG][4];	/* window for input spectral image	*/
long	windcl[1][4];		/* window for input classified image	*/
long	nbands[MAXIMG];		/* number of bands in spectral image	*/
long	nimgs;			/* number of input images               */
char	name[9];		/* program name for comletion message	*/

struct PARBLK vblock;		/* tae variable block			*/


/* ---------------------------------------------------------------------
	initialize tae 
--------------------------------------------------------------------- */
c_inlas(&vblock, &std_out);

/* ---------------------------------------------------------------------
	get tae parameters
--------------------------------------------------------------------- */
strcpy(name,"MASKSTAT");
msparam(&vblock,hosin,hoscl,lastat[0],hostat,windin,windcl,nbands,&inbands,
&nimgs,bandscl,bands,&msgtim,&dtype);
/*
---------------------------------------------------------------------
	set up files
--------------------------------------------------------------------- */
msfiles(hosin, bands, nbands, nimgs, windin, &dtype, hoscl[0], bandscl[0], 
        windcl[0], hostat, &nclass);
/*
---------------------------------------------------------------------
	calculate stats
--------------------------------------------------------------------- */
mstats(&inbands,nclass,windin[0][2],windin[0][3],msgtim);
/*
---------------------------------------------------------------------
       completion signal
-------------------------------------------------------------------- */
c_egclse(&gdescc);
c_egclse(&gdescs);
c_complt(name, lastat, &one);

return EXIT_SUCCESS;

}
