#include <stdio.h>
#include "las.h"
#include "statio.h"
#include "protos.h"

/******************************************************************************

NAME:  MSFILES

FUNCTION: open files for MASKSTATS

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
   1.0	     20-jan-83   c. gurney/sasc     
   2.0       12-feb-88   m. mazzella/SAR    convert to C

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS v 5.0

ALGORITHM (FLOW):
        open classified image
	open spectral image
	open stats file

******************************************************************************/

void msfiles
(
    char hosin[][CMLEN],	/* spectral host file name		*/
    long bands[][MAXBND+1],	/* band numbers for spectral image	*/
    long nbands[],		/* number of bands in input images      */
    long nimgs,		/* number of input images               */
    long windin[][4],		/* spectral image window		*/
    long *dtyp,
    char *hoscl,		/* classified host file name 		*/
    long *bandscl,		/* band number for classified image	*/
    long windcl[],		/* classified image window		*/
    char *hostat,	/* stats host file name			*/
    long *nclass		/* number of classes in class. image	*/
)
{

extern FILE *stfd;	/* stats file descriptor		*/
extern long offs[];	/* offsets into image i/o buffer	*/
extern long offc[];	/* offsets into image i/o buffer	*/
extern struct FDESC * fdescc[];	/* file descriptor array		*/
extern struct FDESC * fdescs[];	/* file descriptor array		*/
extern struct GDESC * gdescc;	/* group descriptor			*/	
extern struct GDESC * gdescs;	/* group descriptor			*/	
extern unsigned char *bufc;	/* image i/o buffer		*/
extern unsigned char *bufs;	/* image i/o buffer		*/

long no = -1;		/* negitive one for call to c_errmsg	*/
long acc = IREAD;	/* file access code			*/
long buffersiz;		/* size of image i/o buffer		*/
long i, nb;
long dtype = EBYTE;	/* data type of images			*/
long zero = 0;		/* 0					*/
long status;		/* error status				*/
/*
----------------------------------------------------------------------
	get # of classes from input label of classified image
	check for unknown class
----------------------------------------------------------------------
*/
status = c_eopens(fdescc,hoscl,bandscl,&windcl[0],&windcl[1],&windcl[2],
		&windcl[3],offc,&acc,&dtype,&zero,&zero);

if (status)
   c_errmsg("error opening classified image","maskstat-fatal", &no);

for (i = 0, nb = 0; i < nimgs; i++)
   {
   status = c_eopens(fdescs,hosin[i],bands[i],&windin[i][0],&windin[i][1],
   		     &windin[i][2],&windin[i][3],&offs[nb],&acc,dtyp,&zero,&zero);
   nb += nbands[i];
   }

if (status)
   c_errmsg("error opening input image","maskstat-fatal", &no);

fdescs[1] = 0;
fdescc[1] = 0;
bufc =  c_ealloc(fdescc,&buffersiz);
if (bufc == NULL)
   c_errmsg("error allocating space for image i/o","maskstat-fatal", &no);

status = c_egroup(fdescc,&gdescc,bufc,&buffersiz);
if (status)
   c_errmsg("error grouping images","maskstat-fatal", &no);

bufs = c_ealloc(fdescs,&buffersiz);
if (bufs == NULL)
   c_errmsg("error allocating space for image i/o","maskstat-fatal", &no);

status = c_egroup(fdescs,&gdescs,bufs,&buffersiz);
if (status)
   c_errmsg("error grouping images","maskstat-fatal", &no);

/*
---------------------------------------------------------------------
	open stats file
---------------------------------------------------------------------
*/
acc = IWRITE;
status = c_stopen(&stfd,hostat,&acc,nclass);
if (!status)
   c_errmsg("error opening stats file","maskstat-fatal",&no);
return;
}
