#include <stdio.h>
#include <string.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

/******************************************************************************

NAME: msparam

FUNCTION: get parameters from TAE for maskstat

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
   1.0	     20-jan-83   c.gurney/sasc
   2.0       12-feb-88   m.mazzella/SAR/GSFC

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
	get classified image name and check for errors
	get spectral image name and check for errors
	get stats file name and check for errors

******************************************************************************/

void msparam
(
    struct PARBLK *vblock,	/* las parameter block			*/
    char hosin[][CMLEN],		/* spectral host file name		*/
    char hoscl[][CMLEN],		/* classified host file name		*/
    char *lastat,	/* stats las file name			*/
    char *hostat,	/* stats host file name			*/
    long windin[][4],	/* spectral image window		*/
    long windcl[][4],	/* classified image window		*/
    long nbands[],	/* number of bands in spectral image	*/
    long *totbnd,	/* total number of bands		*/
    long *nimgs,		/* number of input images		*/
    long bandscl[][MAXBND+1],	/* band number of classified image	*/
    long bands[][MAXBND+1],/* band numbers for spectral image	*/
    long *msgtim,	/* timed processing message delay	*/
    long *dtype		/* data type of spectral image		*/
)
{

char	lasin[MAXIMG][CMLEN];	/* spectral las name		*/
char	lascl[1][CMLEN];	/* classified las name			*/
char	histry[CMLEN];	/* histry flag				*/
char	minmax[CMLEN];	/* minmax flag				*/
long	acc;		/* file access code			*/
long	i;		/* loop counter				*/
long	imglimit = 1;	/* max number of input images 		*/
long	kk;		/* loop counter				*/
long	nimgcl;		/* number of input classified images	*/
long	nbandscl;	/* number of bands for classified image	*/
long	same_wind;	/* same window flag			*/
long	samewindcl;	/* same window flag			*/
long	status;		/* error status				*/
long	totbndcl;	/* total number of bands classified	*/
long	qflagcl[1][2];	/* 					*/
long	qflag[MAXIMG][2];/* 					*/
long	element = 0;
long	one = 1;	/* 1 for call to c_errmsg		*/
long	no = -1;	/* neg. 1 for call to c_errmsg		*/
struct DDR ddrcl;	/* spectral image ddr			*/

/* -----------------------------------------------------------------------
	get input classified image name
           check windows and data type for consistency 
       get output statistics file
----------------------------------------------------------------------- */
status = c_getin(vblock,"MASKIMG",&element,lascl,hoscl,&nimgcl,windcl,bandscl,
     &nbandscl,&totbndcl,&imglimit,&samewindcl,qflagcl);

if (status != E_SUCC)
   c_errmsg("Error getting MASKIMG","maskstat-fatal",&no);

if (totbndcl > 1) 
    c_errmsg("Multiple bands not allowed for MASKIMG","maskstat-mltbnd", &no);

bandscl[0][1] = 0;
status = c_getddr(hoscl[0],&ddrcl);
if (status != E_SUCC)
   c_errmsg("Error getting ddr for MASKIMG","maskstat-inclas",&no);
if (ddrcl.dtype != EBYTE)
   c_errmsg("MASKIMG must be a classified byte image","maskstat-class",&no);

/*
-----------------------------------------------------------------------
          get input image names
          get # input images.  check # images  <  max_band
----------------------------------------------------------------------- */
imglimit = MAXIMG;
status = c_getin(vblock,"IN",&element,lasin,hosin,nimgs,windin,bands,nbands,
      totbnd, &imglimit,&same_wind,qflag);

if (status != E_SUCC)
   c_errmsg("Error getting input image","maskstat-inerr",&no);

status = c_gettyp(hosin,nimgs,dtype);
if (status != E_SUCC)
   c_errmsg("Error getting data type for input image","maskstat-dtype",&no);
if (*dtype != EBYTE)
   c_errmsg("Input image must be byte image","maskstat-inbyt",&no);

kk=0;
   for (i = 2; i <= 3; i++)
      {
      if (windcl[0][i] > windin[0][i]) 
         {
         windcl[0][i] = windin[0][i];
         kk=1;
         } /* if */
      if (windin[0][i] > windcl[0][i]) 
        {
         windin[0][i] = windcl[0][i];
         kk=2;
         } /* if */
      } /* do */

if (kk == 1) 
  c_errmsg("MASKIMG window truncated to IN size","maskstat-info",&one);
if (kk == 2) 
  c_errmsg("IN window truncated to MASKIMG size","maskstat-info",&one);

acc = IWRITE;
status = c_getnam(vblock,"OUTSTAT",&element,hostat,"stat",&acc);
if (status != E_SUCC)
   c_errmsg("error getting OUTSTAT","maskstat-fatal",&no);
strcpy(lastat, hostat);
c_getgbl(vblock,histry,minmax,msgtim);
return;
}
