#include <stdio.h>
#include "statio.h"
#include "las.h"
#include "util.h"
#include "protos.h"
/******************************************************************************

NAME: mstats

FUNCTION: calculate and write out stats for maskstat

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
  1.0	     20-jan-83   c.gurney/sasc
  2.0        12-feb-88   m.mazzella/SAR/GSFC  convert to C
  3           8-NOV-89   P. Pease/GSFC       Change call to c_stoptm from
                                             stoptm

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):

******************************************************************************/

void mstats
(
    long *nb,	/* number of bands for input spectral image	*/
    long nc,	/* number of classes in classified image	*/
    long nl,	/* number of lines in input image		*/
    long np,	/* number of pixels per line			*/
    long msgtim	/* timed processing message delay		*/
)
{

extern FILE *stfd;	/* stats file descriptor 		*/
extern unsigned char *bufc;	/* image i/o buffer		*/
extern unsigned char *bufs;	/* image i/o buffer		*/
extern long offc[];	/* offsets into image i/o buffer	*/
extern long offs[];	/* offsets into image i/o buffer	*/
extern struct GDESC * gdescs;	/* group descriptor			*/
extern struct GDESC * gdescc;	/* group descriptor			*/
extern long curent;	/* current entity (line) 		*/

long	*hist[MAXCLS]; /* histogram buffer	*/
double	*sum[MAXCLS];	/* sum buffer		*/
double	*scov[MAXCLS];	/* covarience matrix buffer	*/
double	*mean[MAXCLS];	/* mean buffer		*/
double	prod;	/* product for covariance			*/
double	xn;	/* temporary variable				*/
static long	zero = 0;	/* 0				*/
long	npx[MAXCLS];	/* number of pixels per class		*/
long	nshape[9];	/* shape vector for number of pixels	*/
long	mshape[9];	/* shape vector for mean		*/
long	n;		/* loop counter				*/
long	yval;		/* pixel value 				*/
long	xval;		/* pixel value 				*/
long	k;		/* loop counter				*/
long	kx;		/* loop couner				*/
long	m;
long	i;
long	num;
long	j;
long	ic;
long	cshape[9];	/* shape vector for cov. matrix		*/
long	lshape[9];	/* shape vector for name		*/
long	hshape[9]; 	/* shape vector for histrogram		*/	
long	*maxv[MAXCLS];	/* max value per class	*/
long	status;		/* error status				*/
long	allcat=ALLCAT;	/* category flag for timed proc msg	*/
long	no = -1;	/* -1 for calls to c_errmsg		*/
char	cname[3];	/* class name				*/
char	datatype[CMLEN];	/* type of stats data		*/

zeroarrays(maxv,npx,mean,scov,hist,sum,*nb);
initshapes(nshape,mshape,lshape,hshape,cshape,*nb);
/*
----------------------------------------------------------------------
	process each line & pixel
----------------------------------------------------------------------
*/
curent = 1;
c_inittm("MASKSTAT", "Processed", "line", "bands"," ",&nl,nb,&msgtim,
	&allcat);
c_startm();
for (curent = 1; curent <= nl; curent++)
   {
   status = c_estep (&gdescc);
   if (status)
      c_errmsg("Error reading spectral images","maskstat-fatal",&no);
   status = c_estep (&gdescs);
   if (status)
      c_errmsg("Error reading spectral images","maskstat-fatal",&no);
   for (j = 0; j < np; j++)
      {
      n=0;
/*
----------------------------------------------------------------------
	get class # of pixel & current # points in class
----------------------------------------------------------------------
*/
      ic = bufc[offc[0] + j];
      if (ic > nc)
         nc = ic;
      if ((ic <= MAXCLS) && (ic != 0))
         {
         npx[ic-1]++;
         if (npx[ic-1] > 1)
            xn = (double)npx[ic-1] / ((double)npx[ic-1] - 1);
         else
            xn = 0.;
/*
---------------------------------------------------------------------
	for each band, update mean & histogram
---------------------------------------------------------------------
*/
         for (k = 0; k < *nb; ++k)
            {
            xval = bufs[offs[k] + j];
            sum[ic-1][k] += xval;
            mean[ic-1][k] = sum[ic-1][k] / npx[ic-1];
            if (xval > maxv[ic-1][k])
               maxv[ic-1][k] = xval;
            hist[ic-1][k * 256 + xval]++;
/*
----------------------------------------------------------------------
	update sums of products ( lower left triangle)
----------------------------------------------------------------------
*/
            for (m = 0; m <= k; m++)
               {
               n++;
               yval = bufs[offs[m]+j];
               prod = (yval - mean[ic-1][m]) * (xval - mean[ic-1][k]);
               scov[ic-1][n-1] = scov[ic-1][n-1] + (prod * xn);
               }  /* do */
            }  /* do */
         }  /* if */
      }  /* do */
   }  /* do */
c_stoptm();
/*
------------------------------------------------------------------
	get actual covariances
------------------------------------------------------------------
*/
for (k=0; k < nc; ++k)
   {
   num = -1;
   for (i = 1; i <= *nb; i++)
      {
      for (j = 1; j <= i; j++)
         {
         num++;
         if ((scov[k][num] != 0) && (npx[k] > 1))
            scov[k][num] =  scov[k][num] / ((double)npx[k] - 1);
         }  /* do */
      }  /* do */
   }  /* do */
/*
-----------------------------------------------------------------------
	write results to stats file
----------------------------------------------------------------------
*/
mshape[2] = *nb;
cshape[2] = (long) ((double)*nb * ((double)*nb + 1) / 2);
kx=1;
for (k = 1; k <= nc; k++)
   {
   if (npx[k-1] > 0)
      {
      status = c_stput (&stfd, "NPOINTS", &kx, &zero, nshape, &npx[k-1]);
      if (status != E_DTOLD && status != E_DTNEW && status != E_NOCL)
         c_errmsg("error writing stats file (npoints)", "maskstat-fatal", &no);
      status = c_stput (&stfd, "MEAN_VECTOR", &kx, &zero, mshape, 
                        &mean[k-1][0]); 
      if (status != E_DTOLD && status != E_DTNEW && status != E_NOCL)
         c_errmsg("error writing stats file (mean vector)", "maskstat-fatal", 
                  &no);
      status = c_stput (&stfd, "COVARIANCE_MATRIX", &kx, &zero, cshape, 
                        &scov[k-1][0]);
      if (status != E_DTOLD && status != E_DTNEW && status != E_NOCL)
         c_errmsg("error writing stats file(convariance matrix)",
                  "maskstat-fatal", &no);
      sprintf(cname,"%03d",k);
      status = c_stput (&stfd, "NAME", &kx, &zero, lshape, cname); 
      if (status != E_DTOLD && status != E_DTNEW && status != E_NOCL)
         c_errmsg("error writing stats file (name)", "maskstat-fatal", &no);
      for (i=1; i <= *nb; i++)
         {
         sprintf(datatype, "HIST%03d", i);
         hshape[2] = maxv[k-1][i-1] + 1;
         status = c_stput (&stfd, datatype, &kx, &zero, hshape, 
                           &hist[k-1][(i-1) * 256]); 
         if (status != E_DTOLD && status != E_DTNEW && status != E_NOCL)
            c_errmsg("error writing stats file (histogram)", 
                     "maskstat-fatal", &no);
         }  /* do */
      kx++;
      }  /* if */
   }  /* do */
c_stclse(&stfd); 
dyn_free();
return;
}
