#include <stdio.h>
#include "las.h"
#include "bufs.h"
#include "statio.h"
#include "protos.h"

/******************************************************************************

NAME: zeroarrays

FUNCTION: initialize arrays for maskstat

PROGRAM HISTORY:
  Version	Date       Author      	     Request
  -------	----	   ------            -------
   1.0        12-feb-88    m.mazzella SAR/GSFC  orig. dev.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	new LAS

ALGORITHM (FLOW):
         loop on all arrays
            array[loop] = 0
******************************************************************************/

void zeroarrays
(
    long *maxv[MAXCLS],	/* maximum value array	*/
    long npx[MAXCLS], 		/* number of pixels 	*/
    double *mean[MAXCLS],	/* mean array		*/
    double *scov[MAXCLS],	/* covariance array	*/
    long *hist[MAXCLS], /* histogram array	*/
    double *sum[MAXCLS],	/* sum array		*/
    long bands
)
{
long	i;	/*  loop counter				*/
long	j;	/*  loop counter				*/
long	k;	/*  loop counter				*/
long buf_size;
long 	fatal_err = -1;

buf_size = (long) (MAXCLS * bands * sizeof(long));
if ((maxv_buf = (long *) malloc( buf_size)) == NULL )
   c_errmsg( "Error allocating dynamic memory", "maskstat-fatal", &fatal_err);

buf_size = (long) (MAXCLS * bands * sizeof(double));
if ((sum_buf = (double *) malloc( buf_size)) == NULL )
   c_errmsg( "Error allocating dynamic memory", "maskstat-fatal", &fatal_err);

if ((mean_buf = (double *) malloc( buf_size)) == NULL )
   c_errmsg( "Error allocating dynamic memory", "maskstat-fatal", &fatal_err);

buf_size = (long) (MAXCLS * bands * bands * sizeof(double));
if ((scov_buf = (double *) malloc( buf_size)) == NULL )
   c_errmsg( "Error allocating dynamic memory", "maskstat-fatal", &fatal_err);

buf_size = (long) (MAXCLS * bands * 256 * sizeof(long));
if ((hist_buf = (long *) malloc( buf_size)) == NULL )
   c_errmsg( "Error allocating dynamic memory", "maskstat-fatal", &fatal_err);


for (i = 0, j = 0; i < MAXCLS; i++, j += bands)
   {
   mean[i] = &mean_buf[j];
   maxv[i] = &maxv_buf[j];
   sum[i] = &sum_buf[j];
   scov[i] = &scov_buf[j * bands];
   hist[i] = &hist_buf[j * 256];
   }



for ( i = 0; i < MAXCLS; i++)
   {
   npx[i] = 0;
   for ( j = 0; j < bands; j++)
      {
      maxv[i][j] = 0;
      sum[i][j] = 0.0;
      scov[i][j] = 0.0;
      mean[i][j] = 0.0;
      for ( k = 0; k < 256; k++)
         {
         hist[i][j * 256 + k] = 0;
         }
      }
   }
}
