/*******************************************************************************
NAME      		  GETPAR 

PURPOSE  To get the necessary parameters and perform the preliminary
	 edit checking. 

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      Sept. l988      update for standards
R.MCKINNEY	Dec.  1988	Update to close all files
T. Baltzer	Jan.  1991	Set totbnd1, totbnd2 to min
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	         LAS 5.0		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   Getpar first determines the subcommand used.
   Next both of the input images are received.
   Various edit checking is performed for windowing, etc.
   The output image is retrieved.
   Depending upon the subcommand the scale and offset factors
     are retrieved.
   The globals are checked and control returns to the driver.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/


extern struct FDESC * fdesc1[]; /* image file descriptors used for image i/o  */
extern struct FDESC * fdesc2[]; /* image file descriptors used for image i/o  */
extern struct FDESC * fdesc3[]; /* image file descriptors used for image i/o  */
extern struct GDESC * gdesc1;	/* group descriptor used for image i/o        */
extern struct GDESC * gdesc2;	/* group descriptor used for image i/o        */
extern struct GDESC * gdesc3;	/* group descriptor used for image i/o        */
extern long offset1[];          /* offsets into thebuf for input bands        */
extern long offset2[];          /* offsets into thebuf for input bands        */
extern long offset3[];	        /* offsets into thebuf for output bands       */
extern long curent;		/* current processing line		      */
extern long curcat;		/* current processing band		      */

void getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block	      */
    long *msgtim,		/* elapsed time between display messages      */
    char taein1[][CMLEN],	/* pointer to first TAE file descriptor       */
    char hosin1[][CMLEN],	/* pointer to first host file descriptor      */
    char taein2[][CMLEN],	/* pointer to second TAE file descriptor      */
    char hosin2[][CMLEN],	/* pointer to second host file descriptor     */
    char taeout[][CMLEN],		/* pointer to output TAE file descriptor      */
    char hosout[][CMLEN],		/* pointer to output host file descriptor     */
    long (*wind1)[4],		/* window specification for first image       */
    long (*wind2)[4],		/* window specification for second image      */
    long nib1[],         	/* number of bands per first image            */
    long nib2[],         	/* number of bands per second image	      */
    long (*bands1)[MAXBND + 1],   /* band specification for first image       */
    long (*bands2)[MAXBND + 1],   /* band specification for second image      */
    long *nim1,			/* number of input images for first image     */
    long *nim2,			/* number of input images for second image    */
    long *totbnd1,		/* total number of bands in first image       */
    long *totbnd2,		/* total number of bands in second image      */
    float *scale1,		/* first scale factor   		      */
    float *scale2,		/* second scale factor			      */
    char *method,		/* mathematical method of use                 */
    char *minmax,		/* min max flag buffer			      */
    float *offst,		/* offset factor			      */
    long *odtype,		/* output data type			      */
    float *pixval		/* pixel value to be used for a zero divide   */
) 

{
struct VARIABLE *var;		/* pointer to a TAE variable within vblock    */

long    element;		/* element for getin call		      */
long    fatal = -1;		/* fatal error flag			      */
long	maximg = MAXIMG / 2;	/* maximum number of images to be processed   */
long    nonfatal = 1;		/* nonfatal error flag			      */
long	qflag[MAXIMG][2]; 	/* flag indicating if subwindows specified    */
long	reduc;			/* reduce the number of bands by this much    */
long    same;			/* flag indicating if images have same window */
long	status;			/* function return status code		      */

char    dtype[5];		/* data type 				      */


 /* Determine the subcommand chosen
------------------------------------------------------*/

var = p_find(vblock, "_SUBCMD");
strcpy(method, SVAL(*var,0));

 /* Get first image specification
--------------------------------------------------*/

element = 0;
status = c_getin( vblock, "IN", &element, taein1, hosin1, nim1, wind1, bands1, 
               nib1, totbnd1, &maximg, &same, qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","math-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Images do not have same window size",
           "math-window",&nonfatal);
   c_errmsg("Fatal error encountered","math-fatal",&fatal);
   }


 /* Get second image specification
---------------------------------------------------------*/

element = 1;
status = c_getin( vblock, "IN", &element, taein2, hosin2, nim2, wind2, bands2, 
                nib2, totbnd2, &maximg, &same, qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","math-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Images do not have same window size",
           "math-window",&nonfatal);
   c_errmsg("Fatal error encountered","math-fatal",&fatal);
   }


/* Check to determine if the images have the same number of lines and samples
-----------------------------------------------------------------------------*/

if (wind1[0][2] != wind2[0][2])
   {
   c_errmsg("Images do not have same number of lines",
           "math-lines",&nonfatal);
   c_errmsg("Fatal error encountered","math-fatal",&fatal);
   }
if (wind1[0][3] != wind2[0][3])
   {
   c_errmsg("Images do not have same number of samples",
           "math-samples",&nonfatal);
   c_errmsg("Fatal error encountered","math-fatal",&fatal);
   }

/* Check the number of bands in each input image
---------------------------------------------------------*/
if (*totbnd1 != *totbnd2)
   {
   c_errmsg("Images do not have same number of bands",
           "math-bands",&nonfatal);
   c_errmsg("Number of bands reset to minimum","math-bands",&nonfatal);
   if (*totbnd1 > *totbnd2)
      {
      reduc = *totbnd1 - *totbnd2;
      *totbnd1 = *totbnd2;
      for (;;)
	 {
	 nib1[*nim1-1] -= reduc;
	 if ( nib1[*nim1-1] >  0 )  break;
	 if ( nib1[*nim1-1] == 0 )
	    {
	    (*nim1)--;
	    break;
	    }
	 else	/* less than */
	    {
	    reduc = - nib1[*nim1-1];
	    nib1[*nim1-1] = 0;
	    (*nim1)--;
	    }
	 }
      }
   else		/* totbnd2 > totbnd1 */
      {
      reduc = *totbnd2 - *totbnd1;
      *totbnd2 = *totbnd1;
      for (;;)
	 {
	 nib2[*nim2-1] -= reduc;
	 if ( nib2[*nim2-1] >  0 )  break;
	 if ( nib2[*nim2-1] == 0 )
	    {
	    (*nim2)--;
	    break;
	    }
	 else	/* less than */
	    {
	    reduc = - nib2[*nim2-1];
	    nib2[*nim2-1] = 0;
	    (*nim2)--;
	    }
	 }
      }
   }


 /*  Get output image name
--------------------------------------------------------*/

status = c_getout(vblock, "OUT", taeout, hosout);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","math-fatal",&fatal);

 /* Get the output data type; default to the input data type
---------------------------------------------------------------------------*/

var = p_find(vblock,"odtype");
strcpy(dtype,SVAL(*var,0));
if (strcmp(dtype,"SAME") == 0)
	*odtype = 0;
else if (strcmp(dtype,"BYTE") == 0)
        *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
        *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
        *odtype = ELONG;
else if (strcmp(dtype,"R*4") == 0)
        *odtype = EREAL;

 /* Get the scale factors for adding, subtracting, multiplying, dividing
----------------------------------------------------------------------------*/

if ((strcmp (method, "ADD") == 0) || (strcmp (method, "SUB") == 0) || 
   (strcmp (method, "MUL") == 0) || (strcmp (method, "DIV") == 0))
   {
   var = p_find(vblock, "SCALFACT");
   *scale1=RVAL(*var,0);
   if ((strcmp (method, "ADD") == 0) || (strcmp (method, "SUB") == 0))
      {
      *scale2=RVAL(*var,1);
      }

 /* Get the offset for adding, subtracting, multiplying, dividing
-----------------------------------------------------------------------*/

   var = p_find(vblock, "offset");
   *offst=RVAL(*var,0);
   if (strcmp (method, "DIV") == 0)
      {
      var = p_find(vblock,"PIXVAL");
      *pixval = RVAL(*var,0);
      }
   } 

 /* Get TAE globals 
---------------------------------------------------------------*/

var = p_find(vblock, "$MSGTIM");
*msgtim = IVAL(*var, 0);

var = p_find(vblock, "$MINMAX");
strcpy(minmax, SVAL(*var,0));
return;
}
