/******************************************************************************
NAME      		  MATH 

PURPOSE  Perform various math functions on two input images and create a
	 single output image.  The functions are MAX, MIN, ADD, SUBTRACT,
	 MULTIPLY, DIVIDE, AND, OR, and XOR. 

PROGRAM HISTORY

VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   J. Hagedorn	   4/83 	  CSC		Original Development
 
  4.0	   B. Kovalick     8/87		  SAR           Inserted the timing
							message. 

  5.0	   R.McKinney      6/88		  EDC		Add UPDDR routine
							Take descriptors out 
							of calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.0	   R.McKinney	  12/88		  EDC 		Update to close all
							files
  5.1	   T. Mittan	   9/92		  EDC		Added extern to curent
							and curcat to eliminate
							warnings on link
  6.0      D. Bridges      5/94           EDC		Added option array 
							for opening files

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MATH must be run under TAE.

PROJECT   	       LAS 5.0		                

LANGUAGE		  C	

ALGORITHM (FLOW)
    The program begins with the standard initialization and retrieval 
of user-specified parameters.  The input and output images are opened
and grouped for sequential processing.  The input image data is converted
to REAL*4 (float) upon input, and the data type of the output image is
always REAL*4.  Space is allocated for the image I/O buffer.  Next, the
initialization for the processing message and min/max arrays is done.
The appropriate option is performed based upon the subcommand entered.
Finally, associated min/max and history files are created for the output 
image and a completion message is output.

ALGORITHM REFERENCES     none

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define HALFIMG (MAXIMG / 2)
/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC * fdesc1[HALFIMG+1];/*image file descriptors used for image i/o  */
struct FDESC * fdesc2[HALFIMG+1];/*image file descriptors used for image i/o  */
struct FDESC * fdesc3[2];	     /*image file descriptors used for image i/o  */
struct GDESC * gdesc1;			/* group descriptor used for image i/o        */
struct GDESC * gdesc2;			/* group descriptor used for image i/o        */
struct GDESC * gdesc3;			/* group descriptor used for image i/o        */
long offset1[HALFIMG*MAXBND];	/* offsets into thebuf for input bands        */
long offset2[HALFIMG*MAXBND];	/* offsets into thebuf for input bands        */
long offset3[MAXBND];		/* offsets into thebuf for output bands       */
extern long curent;		/* current processing line      	      */
extern long curcat;		/* current processing band		      */
long bands[MAXIMG][MAXBND + 1]; /* total number of input bands                */
double  minval[MAXBND];         /* min pixel value array                      */
double  maxval[MAXBND];         /* max pixel value array                      */

int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		      */

float offst;			/* offset factor 			      */
float scale1;			/* first scale factor			      */
float scale2;			/* second scale factor   		      */

static long bands1[HALFIMG][MAXBND + 1];       /* band specs for first image  */
static long bands2[HALFIMG][MAXBND + 1];       /* band specs for second image */
long band;			/* band counter				      */
long msgtim;			/* elapsed time between display messages      */
long nbands[MAXIMG];		/* number of bands specified per image        */
long nim1;			/* number of input images for first image     */
long nim2;			/* number of input images for second image    */
long nib1[HALFIMG + 1]; 	/* number of bands per first image            */
long nib2[HALFIMG + 1];	        /* number of bands per second image           */
long nmrimg;			/* total number of input images               */
long nonfatal = 1;		/* nonfatal error flag			      */
long numb;                      /* integer for testing method                 */
long odtype;			/* odtype for output image                    */
long one = 1;			/* integer value of one			      */
long std_out;			/* logical unit # for standard output         */
long totbnd1;			/* total number of bands in first image       */
long totbnd2;			/* total number of bands in second image      */
long wind1[HALFIMG][4];	        /* window specification for first image       */
long wind2[HALFIMG][4];	        /* window specification for second image      */

char hosin1[HALFIMG][CMLEN];    /* buffer storage for host name	              */
char hosin2[HALFIMG][CMLEN];    /* buffer storage for second host name        */
char hosout[1][CMLEN];         /* buffer storage for output host name        */
char    minmax[4];	        /* minmax flag buffer 			      */
char taein1[HALFIMG][CMLEN];    /* buffer storage for TAE input name	      */
char taein2[HALFIMG][CMLEN];    /* buffer storage for second input name       */
char taeout[1][CMLEN];          /* buffer storage for TAE output name         */
char totin[MAXIMG][CMLEN];      /* total input image names                    */
char    method[4];		/* mathematical method of use                 */

float pixval;			/* pixel value used when a zero divide        */

/* initialize LAS and TAE  
------------------------------------------------*/

c_inlas(&vblock, &std_out);  

/* Get user-specified parameters and TAE globals
-----------------------------------------------------*/

getpar(&vblock, &msgtim, taein1, hosin1, 
       taein2, hosin2, taeout, hosout, wind1, wind2, 
       nib1, nib2, bands1, bands2, &nim1, &nim2, &totbnd1, &totbnd2,
       &scale1, &scale2, method, minmax, &offst, &odtype, &pixval);

/* Set up for subcommand processing
---------------------------------------------------*/

for (numb = 0; numb < nim1; numb++)
     {
     strcpy (totin[numb], hosin1[numb]);
     nbands[numb] = nib1[numb];
       for (band=0;band<totbnd1; band++)
	   bands[numb][band] = bands1[numb][band];
     }
for (numb = 0; numb < nim2; numb++)
     {
     strcpy (totin[numb+nim1], hosin2[numb]);
     nbands[numb+nim1] = nib2[numb];
       for (band=0;band<totbnd2; band++)
	   bands[numb+nim1][band] = bands2[numb][band];
     }


	     if (strcmp(method,"MAX") == 0)
		numb = 1;
	else if (strcmp(method,"MIN") == 0)
		numb = 2;
	else if (strcmp(method,"ADD") == 0)
		numb = 3;
	else if (strcmp(method,"SUB") == 0)
		numb = 4;
	else if (strcmp(method,"DIV") == 0)
		numb = 5;
	else if (strcmp(method,"MUL") == 0)
		numb = 6;
	else if (strcmp(method,"AND") == 0)
		numb = 7;
	else if (strcmp(method,"OR") == 0)
		numb = 8;
	else if (strcmp(method,"XOR") == 0)
		numb = 9;
	
	switch (numb)
	{

/*	CASE FOR MAX
-------------------------------------------------*/

	case 1:
	mathmax(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax);
	break;

/*	CASE FOR MIN
-------------------------------------------------*/

	case 2:
	mathmin(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax);
	break;

/*	CASE FOR ADD
-------------------------------------------------*/

	case 3:
	mathadd(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax,&scale1,&scale2,&offst);
	break;

/*	CASE FOR SUB
-------------------------------------------------*/

        case 4:
	mathsub(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax,&scale1,&scale2,&offst);
	break;

/*	CASE FOR DIV
-------------------------------------------------*/	

	case 5:
	mathdiv(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax,&scale1,&offst,pixval);
	break;

/*	CASE FOR MUL
-------------------------------------------------*/

	case 6:
	mathmul(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax,&scale1,&offst);
	break;

/*	CASE FOR AND
-------------------------------------------------*/

        case 7:
	mathand(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax);
	break;

/*	CASE FOR OR
-------------------------------------------------*/

	case 8:
	mathor(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax);
	break;
	
/*	CASE FOR XOR
-------------------------------------------------*/

	case 9:
	mathxor(&vblock,hosin1,hosin2,totin,hosout,nim1,nim2,
	        wind1,wind2,bands1,bands2,totbnd1,nbands,
	 	odtype,msgtim,minmax);
	break;
    }

 /*  Create associated history file
-------------------------------------------------*/

nmrimg = nim1 + nim2;

if (c_hstout(&vblock, hosout[0], totin, &nmrimg, " ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&nonfatal);


/*  Inform user of successful completion
-------------------------------------------------*/

c_complt("MATH", taeout, &one);

return EXIT_SUCCESS;

}
