/******************************************************************************
NAME:   		       MATHMIN 

PURPOSE     Compares respective pixel values of the specified input images 
            and outputs the minimum pixel values to one output file. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   J. Hagedorn 	   3/82 	  CSC		Original Development
 
  4.0	   B. Kovalick 	   3/86		  SAR	        Inserted the timing 
							message. 	

  5.0	   R. McKinney     6/88		  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.0	   R.McKinney	  12/88		  EDC 		Update to close all
							files
  6.0	   T.Baltzer 	  01/91		  EDC 		Fix calculation of 
							'band' passed to eopens

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MATHMIN must be run under TAE.

PROJECT			LAS 5.0

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   The two images are MIN'd together
   If $MINMAX was set to "YES", save the minmax values 
   Update the output ddr
   Close the input and output image files

ALGORITHM REFERENCES		none
******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void mathmin
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin1[][CMLEN],	/* host names of input images		     */
    char hosin2[][CMLEN],	/* host names of input images		     */
    char totin[][CMLEN],		/* total input image names		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nim1,			/* number of input images                    */
    long nim2,			/* number of input images                    */
    long (*wind1)[4],		/* window specification of input images      */
    long (*wind2)[4],		/* window specification of input images      */
    long (*bands1)[MAXBND+1],	/* bands to be processed for all images      */
    long (*bands2)[MAXBND+1],	/* bands to be processed for all images      */
    long totbnd1,		/* total number of bands to be processed     */
    long *nbands,		/* number of bands specified for each image  */
    long odtype,			/* data type of output image                 */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax		/* min max flag                              */
)

{

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern struct FDESC * fdesc1[];	 /* image file descriptors used for image I/O */
extern struct FDESC * fdesc2[];	 /* image file descriptors used for image I/O */
extern struct FDESC * fdesc3[];  /* image file descriptors used for output    */
extern struct GDESC * gdesc1;	/* group descriptor used for image I/O       */
extern struct GDESC * gdesc2;	/* group descriptor used for image I/O       */
extern struct GDESC * gdesc3;	/* group descriptor used for image I/O       */
extern long offset1[];          /* offsets into thebuf for input bands       */
extern long offset2[];          /* offsets into thebuf for second bands      */
extern long offset3[];          /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */
extern long bands[][MAXBND + 1];/* total number of input bands               */ 
extern double  minval[];        /* min pixel value array                     */
extern double  maxval[];        /* max pixel value array                     */

double  line_inc;
double  samp_inc;

long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long	bufsiz2;		/* size of image I/O buffer                  */
long	bufsiz3;		/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    dtype = 0;		/* input image data type		     */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long	size;			/* number of bytes per pixel 	 	     */
long	msg_flag = ALLCAT;	/* flag for processing message               */
long 	nl;			/* number of image lines                     */
long    nmrimg;			/* total number of input images              */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    totbnd;			/* total number of bands 		     */
long    totwin[MAXIMG][4];      /* total number of windows                   */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */
long    opt[2];			/* options for opening images		     */


unsigned char *thebuf1;		/* image I/O buffer                          */
unsigned char *thebuf2;		/* image I/O buffer                          */
unsigned char *thebuf3;		/* image I/O buffer                          */

/* Open input image(s)
----------------------*/
access = IREAD;
opt[0] = opt[1] = 0;

for (band=0,image=0;image<nim1;image++)
   {
   totwin[image][0] = sl = wind1[image][0];
   totwin[image][1] = ss = wind1[image][1];
   totwin[image][2] = nl = wind1[image][2];
   totwin[image][3] = ns = wind1[image][3];
   status = c_eopens(&fdesc1[image],hosin1[image],bands1[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&dtype,&zero,opt);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","math-fatal",&fatal);
   band += nbands[image];
   }
   fdesc1[nim1] = 0;
for (band=0,image=0;image<nim2;image++)
   {
   totwin[image+nim1][0] = sl = wind2[image][0];
   totwin[image+nim1][1] = ss = wind2[image][1];
   totwin[image+nim1][2] = nl = wind2[image][2];
   totwin[image+nim1][3] = ns = wind2[image][3];
   status = c_eopens(&fdesc2[image],hosin2[image],bands2[image],&sl,&ss,&nl,&ns,
                   &offset2[band],&access,&dtype,&zero,opt);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","math-fatal",&fatal);
   band += nbands[image+nim1];
   }
fdesc2[nim2] = 0;

/* Open output image
--------------------*/
access = IWRITE;	
opt[0] = odtype;
totbnd = totbnd1;
status = c_eopens(&fdesc3[0],hosout[0],&totbnd,&one,&one,&nl,&ns,offset3,
         &access,&dtype,&zero,opt);
if (status != E_SUCC)
   c_clean(vblock,"math",hosout,&one);
fdesc3[1] = 0;

/* Allocate I/O buffer
----------------------*/

if ((thebuf1 = (unsigned char *)c_ealloc(fdesc1,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","math-alloc",&nonfatal);
   if (c_eclose(&fdesc3[0]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&nonfatal);
   c_clean(vblock,"math",hosout,&one);
   }
if ((thebuf2 = (unsigned char *)c_ealloc(fdesc2,&bufsiz2)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","math-alloc",&nonfatal);
   if (c_eclose(&fdesc3[0]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&nonfatal);
   c_clean(vblock,"math",hosout,&one);
   }
if ((thebuf3 = (unsigned char *)c_ealloc(fdesc3,&bufsiz3)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","math-alloc",&nonfatal);
   if (c_eclose(&fdesc3[0]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&nonfatal);
   c_clean(vblock,"math",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc1,&gdesc1,thebuf1,&bufsiz) != E_SUCC)
   {
   if (c_eclose(&fdesc3[0]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&nonfatal);
   c_clean(vblock,"math",hosout,&one);
   }
if (c_egroup(fdesc2,&gdesc2,thebuf2,&bufsiz2) != E_SUCC)
   {
   if (c_eclose(&fdesc3[0]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&nonfatal);
   c_clean(vblock,"math",hosout,&one);
   }
if (c_egroup(fdesc3,&gdesc3,thebuf3,&bufsiz3) != E_SUCC)
   {
   if (c_eclose(&fdesc3[0]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&nonfatal);
   c_clean(vblock,"math",hosout,&one);
   }

/* Setup for data type
----------------------*/
switch(dtype)
   {
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:	size = 4; break;
   case EREAL:  size = 4; break;
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("MATH","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
curcat = 1;
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&dtype,&totbnd,minval,maxval);

/* Main processing loop
-----------------------*/
for (curlin = 0;curlin < nl;curlin ++)
   {

   if (c_estep(&gdesc1) != E_SUCC)
      {
      if (c_egclse(&gdesc3) != E_SUCC)
      c_errmsg("Nonfatal error encountered","math-warn",&one);
      c_clean(vblock,"math",hosout,&one);
      }
   if (c_estep(&gdesc2) != E_SUCC)
      {
      if (c_egclse(&gdesc3) != E_SUCC)
      c_errmsg("Nonfatal error encountered","math-warn",&one);
      c_clean(vblock,"math",hosout,&one);
      }
   if (c_estep(&gdesc3) != E_SUCC)
      {
      if (c_egclse(&gdesc3) != E_SUCC)
      c_errmsg("Nonfatal error encountered","math-warn",&one);
      c_clean(vblock,"math",hosout,&one);
      }
   for (band=0;band<totbnd;band++)
      {
	c_pxmin(thebuf1 + (*(offset1+band) * size),
		    thebuf2 + (*(offset2+band) * size),
	  	    thebuf3 + (*(offset3+band) * size),
		    &ns,&dtype);
      if (strcmp(minmax,"YES") == 0)
         status = c_pxmnmx(thebuf3 + (*(offset3+band) * size),&ns,
                         &minval[band],&maxval[band],&dtype); 
      }
   curent++;
   }

/* Final write
--------------*/
   if (c_estep(&gdesc3) != E_SUCC)
      {
      if (c_egclse(&gdesc3) != E_SUCC)
      c_errmsg("Nonfatal error encountered","math-warn",&one);
      c_clean(vblock,"math",hosout,&one);
      }

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = COMB;
nmrimg = nim1 + nim2;

status = c_upddr(vblock,totin,hosout[0],&nmrimg,totwin,bands,nbands,
		 &line_inc,&samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"math",hosout,&one);

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&one);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc1) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&one);
if (c_egclse(&gdesc2) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&one);
if (c_egclse(&gdesc3) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math-warn",&one);

/* Free buffer
--------------*/
free(thebuf1);
free(thebuf2);
free(thebuf3);

return;
}
