/****************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the MATH2 function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      DEC  1988       converted to LAS 5.0

S.NELSON	FEB  1992	added masked value


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   The global parameters are first received.
   Next the input images are gotten.
   Checks are made for window size and data types.
   The output image name is received.
   Various validity checks are then made before returning.
******************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	
                                  /* image file descriptors used for image I/O*/
extern struct GDESC * gdesc;	  /* group descriptor used for image I/O      */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands */
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands   */
extern long curent;		/* current processing line		    */
extern long curcat;		/* current processing band		    */

void getpar
(
    struct PARBLK *vblock,      /* pointer to TAE block	 	      */
    char taein[][CMLEN],	/* TAE file names of input images	    */
    char hosin[][CMLEN],  	/* host file names of input images	    */
    char taeout[][CMLEN],		/* TAE file name of output image	    */
    char hosout[][CMLEN],		/* host file name of output image	    */
    long *nmrimg, 		 /* number of input images		    */
    long (*bands)[MAXBND+1],         /* band specification 		            */
    long *nbands,		 /* number of bands per input image         */
    long (*window)[4],	         /* window specification		    */
    char *method, 		 /* method of processing		    */
    long *msgtim,		 /* elapsed time between display messages   */
    char *minmax,		 /* min max flag buffer		            */
    long *totbnd,		 /* total number of bands		    */
    long *odtype,		 /* output data type 			    */
    float *pixval2,		 /* pixel value to be used in a zero divide */
    long *bnd,			 /* band used for division or subtraction   */
    float maskval[][2],		 /* mask values				    */
    long *masknum,		 /* number of masked values		    */
    float *pixval		 /* pixel value to replace mask values      */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block  */

long    element = 0;		 /* parameter element number		    */
long    fatal = -1;		 /* fatal error flag			    */
long    i;			 /* counter-no. of input images	 	    */
long    imglim;			 /* max number of images to be processed    */
long    nonfatal = 1;		 /* value of one			    */
long    qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified */
long    same;		    	 /* flag indicating if images have same wind*/
long    status;			 /* function return status code		    */
long    type;			 /* input data type			    */

char    dtype[5];		 /* data type of previous input image       */
char	mask[10][50];		 /* mask values				    */
char	*ptr;			 /* temporary storage space for pointer	    */
char    *ptr2;			 /* temporary storage space for pointer     */
char 	tempstr[80];		 /* temporary storage space for string	    */

val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

/* Build the string descriptor for TAE variable "IN".
   Call getin to parse the input image specification.
   Getin returns the TAE image name(s), host image name(s),
   bands to be processed, window specification, and number
   of images parsed.
----------------------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","math2-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","math2-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","math2-fatal",&fatal);
   }

/* Retrieve the data type of the input images.
----------------------------------------------*/
status = c_gettyp(hosin,nmrimg,&type);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","math2-fatal",&fatal);

/* Retrieve the TAE output image name from the parameter
   block and store it in "taeout". 
   Create a host file name for the output image and store 
   it in "hosout".
----------------------------------------------------------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","math2-fatal",&fatal);

 /* Determine the subcommand chosen
------------------------------------------------------*/

val = p_find(vblock, "_SUBCMD");
strcpy(method, SVAL(*val,0));

if ((strcmp (method, "DIV") == 0) || (strcmp (method, "SUB") == 0)) 
   {
   val = p_find(vblock, "BAND");
   *bnd=IVAL(*val,0);
   } 

/*
  Retrieve the output data type
--------------------------------------------------------------------------*/
val = p_find(vblock,"ODTYPE");
strcpy(dtype,SVAL(*val,0));
if (strcmp(dtype,"SAME") == 0)
	*odtype = 0;
else if (strcmp(dtype,"BYTE") == 0)
        *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
        *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
        *odtype = ELONG;
else if (strcmp(dtype,"R*4") == 0)
        *odtype = EREAL;
/*
  Retrieve the mask values
--------------------------------------------------------------------------*/
val = p_find(vblock,"MASKVAL");
*masknum=val->v_count;
for (i = 0; i < *masknum; i++)
   {
   strcpy(mask[i],SVAL(*val,i));
  
   /* Locate spaces and delete
     ------------------------*/
   ptr = mask[i];
   ptr2 = mask[i];
   while (*ptr2 != '\0')
      {
      if (!isspace(*ptr2))
	 *(ptr++) = *(ptr2++);
      else
	 ptr2++;
      }
   *ptr = '\0';

   /* Locate the dash and divide mask values accordingly
    ---------------------------------------------------*/
   ptr = (char *)strchr(mask[i],'-');
   if (ptr == mask[i])
      {
      sscanf(mask[i],"-%s",tempstr);
      ptr = (char *)strchr(tempstr,'-');
      }
   if (ptr == NULL)
      {
      sscanf(mask[i],"%f",&maskval[i][0]);
      maskval[i][1] = maskval[i][0];
      }
   else
      {
      sscanf(mask[i],"%f-%f",&maskval[i][0],&maskval[i][1]);
      }

   /* Test to see that range mask values are in ascending order
     ---------------------------------------------------------*/
   if (maskval[i][0] > maskval[i][1])
      {
      c_errmsg("Error in mask value range encountered","math2-mask",
								&nonfatal);
      c_errmsg("Fatal error encountered","math2-fatal",&fatal);
      }
   } 
/*
   Retrieve the pixel values
----------------------------------------------------------------------------*/
val = p_find(vblock,"PIXVAL");
*pixval = RVAL(*val,0);
if (strcmp (method, "DIV") == 0)
   *pixval2 = RVAL(*val,1);
return;
}
