/******************************************************************************

NAME      		 MATH2 

PURPOSE  Perform various arith functions on one input image and create a
	 single output image.  The functions are ADD, SUBTRACT, DIVIDE,
	 and AVERAGE. 

PROGRAM HISTORY

VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   R.McKinney      12/88	  EDC		Add UPDDR routine
							Take descriptors out 
							of calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file

  5.1	   S.Nelson	    2/92	  		Add Mask Values	
  5.2	   T. Mittan	    9/92			Added extern to curent
							and curcat to eliminate
							warnings on link


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MATH2 must be run under TAE.

PROJECT   	       LAS 5.0		                

LANGUAGE		  C	

ALGORITHM REFERENCES     none

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC * fdesc[MAXIMG + 1];/* image file descriptors used for image i/o */
struct GDESC * gdesc;			/* group descriptor used for image i/o       */
long offset1[MAXIMG * MAXBND];	/* offsets into thebuf for input bands       */
long offset2[MAXBND];	        /* offsets into thebuf for output bands      */
extern long curent;		/* current processing line      	     */
extern long curcat;		/* current processing band		     */
long bands[MAXIMG][MAXBND + 1]; /* total number of input bands               */
double  minval[MAXBND];         /* min pixel value array                     */
double  maxval[MAXBND];         /* max pixel value array                     */

int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		     */


static long bands[MAXIMG][MAXBND + 1];       /* band specs for first image   */
long bnd;			/* band used for subtraction and division    */
long masknum;			/* number of mask values		     */
long msgtim;			/* elapsed time between display messages     */
long nbands[MAXIMG];		/* number of bands specified per image       */
long nmrimg;			/* total number of input images              */
long nonfatal = 1;		/* nonfatal error flag			     */
long odtype;			/* odtype for output image                   */
long one = 1;			/* integer value of one			     */
long std_out;			/* logical unit # for standard output        */
long totbnd;			/* total number of bands in first image      */
long window[MAXIMG][4];	        /* window specification for first image      */

char hosin[MAXIMG][CMLEN];      /* buffer storage for host name	             */
char hosout[1][CMLEN];          /* buffer storage for output host name       */
char minmax[4];	        	/* minmax flag buffer 			     */
char taein[MAXIMG][CMLEN];      /* buffer storage for TAE input name	     */
char taeout[1][CMLEN];          /* buffer storage for TAE output name        */
char method[4];			/* arithematical method of use               */

float maskval[10][2];		/* mask values				     */
float pixval2;			/* pixel value used when a zero divide       */
float pixval;			/* pixel value to replace a mask value       */

/* initialize LAS and TAE  
------------------------------------------------*/

c_inlas(&vblock, &std_out);  

/* Get user-specified parameters and TAE globals
-----------------------------------------------------*/

getpar(&vblock, taein, hosin, taeout, hosout, &nmrimg, bands, nbands,
       			window, method, &msgtim, minmax, &totbnd,  &odtype, 
			&pixval2, &bnd, maskval, &masknum, &pixval);

/* Set up for subcommand processing
---------------------------------------------------*/
if (strcmp(method,"ADD") == 0)
   math2add(&vblock,hosin,hosout,nmrimg, bands,nbands,window,
   			msgtim,minmax,totbnd,odtype, masknum,maskval,pixval);

else if (strcmp(method,"SUB") == 0)
   math2sub(&vblock,hosin,hosout,nmrimg, bands,nbands,window, msgtim,
			minmax,totbnd,odtype, bnd,masknum,maskval,pixval);

else if (strcmp(method,"DIV") == 0)
   math2div(&vblock,hosin,hosout,nmrimg, bands,nbands,window,msgtim, minmax,
			totbnd,odtype,pixval2,bnd,pixval,masknum,maskval);

else if (strcmp(method,"AVE") == 0)
   math2ave(&vblock,hosin,hosout,nmrimg, bands,nbands,window,msgtim,
   			minmax,totbnd,odtype,masknum, maskval, pixval);

 /*  Create associated history file
-------------------------------------------------*/

nmrimg = one;

if (c_hstout(&vblock, hosout[0], hosin, &nmrimg, " ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);


/*  Inform user of successful completion
-------------------------------------------------*/

c_complt("MATH2", taeout, &one);

return EXIT_SUCCESS;

}
