/******************************************************************************
NAME:   		      MATH2AVE 

PURPOSE   To generate an average and standard deviation image.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   R. McKinney     12/88	  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file

  5.1      S.Nelson         2/92                        Add Mask Values

  6.1	   D.Eichacker      5/94	EDC		Added opt[]
	
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MATH2AVE must be run under TAE.

PROJECT				LAS

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "typlim.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	
                                 /* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	 /* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands  */
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


void math2ave
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long (*window)[4],		/* window specification of input images      */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long totbnd,			/* total number of bands to be processed     */
    long odtype,			/* output data type			     */
    long masknum,		/* number of mask values		     */
    float maskval[][2],		/* mask values				     */
    float pixval			/* pixel value to replace mask values	     */
)

{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

static	long	opt[3]={0,0,0}; /* option array for c_eopens                 */
long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long	idtype = 0;		/* type if input data			     */
long	image;			/* loop index counting input images          */
long	mask;			/* loop index counting mask values	     */
long	mask_flag;		/* flag for mask value test		     */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    sampin;			/* sample indicator for input		     */
long	sampout;		/* smaple indicator for output		     */
long	sizout = 0;			/* number of bytes per pixel of output data  */
long	sizin = 0;			/* number of bytes per pixel of input data   */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    two = 2;		/* number used to number of output bands     */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */

double  stddev = 0.0;			/* standard deviation			     */

float   mean = 0.0;			/* mean of the bands			     */
float	mntype = 0.0;			/* min value for data type		     */
float	mxtype = 0.0;			/* max value for data type		     */
float   temp;			/* temporary storage area		     */
float	temp1;			/* temporary storage area 		     */

unsigned char    *thebuf;	/* image input buffer                        */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&idtype,&zero,opt);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","math2-fatal",&fatal);
   band += nbands[image];
   }

/* Open output image
--------------------*/
if (odtype == 0)
   odtype = idtype;
access = IWRITE;	
band = two;
status = c_eopens(&fdesc[nmrimg],hosout[0],&band,&one,&one,&nl,&ns,
         offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
      c_clean(vblock,"math2",hosout,&one);
fdesc[nmrimg+1] = 0;

/* Determine size of input and output image
------------------------------------------*/
switch (odtype)
  {
  case EBYTE:
    sizout = 1;
    mxtype = MAXUTINY;
    mntype = MINUTINY;
    break;
  case EWORD:
    sizout = 2;
    mxtype = MAXWORD;
    mntype = MINWORD;
    break;
  case ELONG:
    sizout = 4;
    mxtype = MAXLONG;
    mntype = MINLONG;
    break;
  case EREAL:
    sizout = 4;
    mxtype = MAXREAL;
    mntype = MINREAL;
    break;
  }
switch (idtype)
  {
  case EBYTE: sizin = 1; break;
  case EWORD: sizin = 2; break;
  case ELONG: sizin = 4; break;
  case EREAL: sizin = 4; break;
  }

/* Allocate I/O buffer
----------------------*/
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","math2-alloc",&nonfatal);
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
       c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
   c_clean(vblock,"math2",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
       c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
   c_clean(vblock,"math2",hosout,&one);
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("MATH2","Processed","line","bands"," ",
           &nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Main processing loop
-----------------------*/
/*
   Process each data type separately for time efficiency.
*/
switch (idtype)
  {
case EBYTE:
  for (curlin = 0;curlin < nl; curlin++)
     {
     if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
           c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
        c_clean(vblock,"math2",hosout,&one);
        }
     
     /* Increase i/o offsets to increase time efficiency
	-----------------------------------------------*/
     for (band = 0; band < two; band ++)
	offset2[band] *= sizout;
     for (sampin = 0, sampout = 0; sampin < ns; sampin++, sampout += sizout)
	{
	temp = 0.0;

	/* Find the sum, mean and std dev if no mask values 
 	  ------------------------------------------------*/
	if (masknum == 0)
	   {
           for (band = 0; band < totbnd; band++)
	      temp += (*(thebuf + (*(offset1 + band) + sampin)));
	   mean = temp/ (float) totbnd;
	   stddev = 0.0;
           for (band = 0; band < totbnd; band++)
	      stddev +=((*(thebuf + (*(offset1 + band) + sampin))) - mean)
	                * ((*(thebuf + (*(offset1 + band) + sampin))) - mean);
           stddev = sqrt(fabs(stddev / totbnd));
	   }

	/* Test if pixel values are mask values
	  ------------------------------------*/
	else
	   {
	   mask_flag = FALSE;
           for (band = 0; (band < totbnd) && (!mask_flag); band++)
              {
	      temp1 = (*(thebuf + (*(offset1 + band) + sampin)));
              for (mask = 0; (mask < masknum) && (!mask_flag); mask++)
                {
                if ((temp1 >= maskval[mask][0]) && (temp1 <= maskval[mask][1]))
                   {
                   mask_flag = TRUE;
                   mean = pixval;
		   stddev = pixval;
                   }
                }

              /* Find the sum, mean, and std dev if not mask value
                 ------------------------------------------------*/
              if (!mask_flag)
	  	 {
                 temp += temp1;
		 }
              }
	   if (!mask_flag)
	      {
	      mean = temp/ (float) totbnd;
	      stddev = 0.0;
              for (band = 0; band < totbnd; band++)
	        stddev += ((*(thebuf + (*(offset1 + band) + sampin))) - mean)*
	                 ((*(thebuf + (*(offset1 + band) + sampin))) - mean);
              stddev = sqrt(fabs(stddev / totbnd));
	      }
	   }

	/* Assign pixel values to output
	 ------------------------------*/
	if (idtype != odtype)
	   {
	   if (mean > mxtype)
	      mean = mxtype;
	   else if (mean < mntype)
	      mean = mntype;
	   if (stddev > mxtype)
	      stddev = mxtype;
	   else if (stddev < mntype)
	      stddev = mntype;
	   }
	switch (odtype)
	   {
	case EBYTE:
	   (*(thebuf + (*(offset2) + sampout))) = mean;
	   (*(thebuf + (*(offset2 + 1) + sampout))) = (float)(stddev);
	   break;
	case EWORD:
	   (*(short *)(thebuf + (*(offset2) + sampout))) = mean;
	   (*(short *)(thebuf + (*(offset2 + 1) + sampout))) = (float)(stddev);
	   break;
        case ELONG:
           (*(long *)(thebuf + (*(offset2) + sampout))) = (long) mean;
	   (*(long *)(thebuf + (*(offset2 + 1) + sampout))) = (long)(stddev);
           break;
        case EREAL:
           (*(float *)(thebuf + (*(offset2) + sampout))) = mean;
	   (*(float *)(thebuf + (*(offset2 + 1) + sampout))) = (float)(stddev);
           break;
           }
	}
   /* find the minimum and maximum pixel value
   ---------------------------------------*/
   for (band = 0; band < two; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2 + band)),&ns,
                          &minval[band],&maxval[band],&odtype); 
	if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
	}
    curent ++;
    }
  break;
case EWORD:
  for (curlin = 0; curlin < nl; curlin++)
     {
     if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
           c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
        c_clean(vblock,"math2",hosout,&one);
        }
     
     /* increase i/o offsets to increase time efficiency
	-----------------------------------------------*/
     for (band = 0; band < totbnd; band++)
	offset1[band] *= sizin;
     for (band = 0; band < two; band ++)
	offset2[band] *= sizout;
     for (sampin = 0, sampout = 0; sampin < (ns * sizin); sampin += sizin,
							 sampout += sizout)
	{
	temp = 0.0;

	/* Find the sum, mean and std dev if no mask values 
 	  ------------------------------------------------*/
	if (masknum == 0)
	   {
           for (band = 0; band < totbnd; band++)
	      temp += (*(short *)(thebuf + (*(offset1 + band) + sampin)));
	   mean = temp/ (float) totbnd;
	   stddev = 0.0;
           for (band = 0; band < totbnd; band++)
	     stddev+=((*(short *)(thebuf + (*(offset1 + band) + sampin)))- mean)
	         * ((*(short *)(thebuf + (*(offset1 + band) + sampin)))- mean);
           stddev = sqrt(fabs(stddev / totbnd));
	   }

	/* Test if pixel values are mask values
	  ------------------------------------*/
	else
	   {
	   mask_flag = FALSE;
           for (band = 0; (band < totbnd) && (!mask_flag); band++)
              {
	      temp1 = (*(short *)(thebuf + (*(offset1 + band) + sampin)));
              for (mask = 0; (mask < masknum) && (!mask_flag); mask++)
                {
                if ((temp1 >= maskval[mask][0]) && (temp1 <=maskval[mask][1]))
                   {
                   mask_flag = TRUE;
                   mean = pixval;
		   stddev = pixval;
                   }
                }

              /* Find the sum, mean, and std dev if not mask value
                 ------------------------------------------------*/
              if (!mask_flag)
	  	 {
                 temp += temp1;
		 }
              }
	   if (!mask_flag)
	     {
	     mean = temp/ (float) totbnd;
	     stddev = 0.0;
             for (band = 0; band < totbnd; band++)
	       stddev += ((*(short *)(thebuf+(*(offset1+band)+sampin))) - mean)
		     * ((*(short *)(thebuf+(*(offset1+band)+sampin))) - mean);
              stddev = sqrt(fabs(stddev / totbnd));
	      }
	   }

	/* Assign pixel values to output
	 ------------------------------*/
	if (idtype != odtype)
	   {
	   if (mean > mxtype)
	      mean = mxtype;
	   else if (mean < mntype)
	      mean = mntype;
	   if (stddev > mxtype)
	      stddev = mxtype;
	   else if (stddev < mntype)
	      stddev = mntype;
	   }
	switch (odtype)
	   {
	case EBYTE:
	   (*(thebuf + (*(offset2) + sampout))) = mean;
	   (*(thebuf + (*(offset2 + 1) + sampout))) = (float)(stddev);
	   break;
	case EWORD:
	   (*(short *)(thebuf + (*(offset2) + sampout))) = mean;
	   (*(short *)(thebuf + (*(offset2 + 1) + sampout))) = (float)(stddev);
	   break;
        case ELONG:
           (*(long *)(thebuf + (*(offset2) + sampout))) = (long) mean;
	   (*(long *)(thebuf + (*(offset2 + 1) + sampout))) = (long)(stddev);
           break;
        case EREAL:
           (*(float *)(thebuf + (*(offset2) + sampout))) = mean;
	   (*(float *)(thebuf + (*(offset2 + 1) + sampout))) = (float)(stddev);
           break;
           }
	}
   /* find the minimum and maximum pixel value
   ---------------------------------------*/
   for (band = 0; band < two; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2 + band)),&ns,
                          &minval[band],&maxval[band],&odtype); 
	if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
	}
    curent ++;
    }
  break;
case ELONG:
  for (curlin = 0;curlin < nl; curlin++)
     {
     if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
           c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
        c_clean(vblock,"math2",hosout,&one);
        }
     
     /* increase i/o offsets to increase time efficiency
	----------------------------------------------*/
     for (band = 0; band < totbnd; band++)
	offset1[band] *= sizin;
     for (band = 0; band < two; band ++)
	offset2[band] *= sizout;
     for (sampin = 0, sampout = 0; sampin < (ns * sizin); sampin += sizin,
							 sampout += sizout)
	{
	temp = 0.0;

	/* Find the sum, mean and std dev if no mask values 
 	  ------------------------------------------------*/
	if (masknum == 0)
	   {
           for (band = 0; band < totbnd; band++)
	      temp += (float) (*(long *)(thebuf + (*(offset1 + band) + sampin)));

           /* Find the standard deviation and mean
	     ------------------------------------*/
	   mean = temp/ (float) totbnd;
	   stddev = 0.0;
           for (band = 0; band < totbnd; band++)
	     stddev+=((float)(*(long *)(thebuf+(*(offset1 + band)+ sampin))) - mean)
	         * ((float)(*(long *)(thebuf+(*(offset1 + band) + sampin))) - mean);
           stddev = sqrt(fabs(stddev / totbnd));
	   }

	/* Test if pixel values are mask values
	  ------------------------------------*/
	else
	   {
	   mask_flag = FALSE;
           for (band = 0; (band < totbnd) && (!mask_flag); band++)
              {
	      temp1 = (float) (*(long *)(thebuf + (*(offset1 + band) + sampin)));
              for (mask = 0; (mask < masknum) && (!mask_flag); mask++)
                {
                if ((temp1 >= maskval[mask][0]) && (temp1 <= maskval[mask][1]))
                   {
                   mask_flag = TRUE;
                   mean = pixval;
		   stddev = pixval;
                   }
                }

              /* Find the sum, mean, and std dev if not mask value
                 ------------------------------------------------*/
              if (!mask_flag)
	  	 {
                 temp += temp1;
		 }
              }
	   if (!mask_flag)
	     {
	     mean = temp/ (float) totbnd;
	     stddev = 0.0;
             for (band = 0; band < totbnd; band++)
	       stddev += ((float)(*(long *)(thebuf+(*(offset1+band)+sampin)))- mean)
		      * ((float) (*(long *)(thebuf+(*(offset1+band) + sampin))) - mean);
              stddev = sqrt(fabs(stddev / totbnd));
	      }
	   }

	/* Assign pixel values to output
	 ------------------------------*/
	if (idtype != odtype)
	   {
	   if (mean > mxtype)
	      mean = mxtype;
	   else if (mean < mntype)
	      mean = mntype;
	   if (stddev > mxtype)
	      stddev = mxtype;
	   else if (stddev < mntype)
	      stddev = mntype;
	   }
	switch (odtype)
	   {
	case EBYTE:
	   (*(thebuf + (*(offset2) + sampout))) = mean;
	   (*(thebuf + (*(offset2+1) + sampout))) = (float)(stddev);
	   break;
	case EWORD:
	   (*(short *)(thebuf + (*(offset2) + sampout))) = mean;
	   (*(short *)(thebuf + (*(offset2+1) + sampout))) = (float)(stddev);
	   break;
        case ELONG:
           (*(long *)(thebuf + (*(offset2) + sampout))) = (long) mean;
	   (*(long *)(thebuf + (*(offset2+1) + sampout))) = (long)(stddev);
           break;
        case EREAL:
           (*(float *)(thebuf + (*(offset2) + sampout))) = mean;
	   (*(float *)(thebuf + (*(offset2+1) + sampout))) = (float)(stddev);
           break;
           }
	}
   /* find the minimum and maximum pixel value
   ---------------------------------------*/
   for (band = 0; band < two; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2 + band)),&ns,
                          &minval[band],&maxval[band],&odtype); 
	if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
	}
    curent ++;
    }
  break;
case EREAL:
  for (curlin = 0; curlin < nl; curlin++)
     {
     if (c_estep(&gdesc) != E_SUCC)
        {
        if (c_egclse(&gdesc) != E_SUCC)
           c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
        c_clean(vblock,"math2",hosout,&one);
        }
     
     /* increase i/o offsets to increase time efficiency
	-----------------------------------------------*/
     for (band = 0; band < totbnd; band++)
	offset1[band] *= sizin;
     for (band = 0; band < two; band ++)
	offset2[band] *= sizout;
     for (sampin = 0, sampout = 0; sampin < (ns * sizin); sampin += sizin,
							 sampout += sizout)
	{
	temp = 0.0;

	/* Find the sum, mean and std dev if no mask values 
 	  ------------------------------------------------*/
	if (masknum == 0)
	   {
           for (band = 0; band < totbnd; band++)
	      temp += (*(float *)(thebuf + (*(offset1+band)+sampin)));
	   mean = temp/ (float) totbnd;
	   stddev = 0.0;
           for (band = 0; band < totbnd; band++)
	     stddev+=((*(float *)(thebuf +(*(offset1 + band) + sampin))) - mean)
	         * ((*(float *)(thebuf + (*(offset1 + band) + sampin))) - mean);
           stddev = sqrt(fabs(stddev / totbnd));
	   }

	/* Test if pixel values are mask values
	  ------------------------------------*/
	else
	   {
	   mask_flag = FALSE;
           for (band = 0; (band < totbnd) && (!mask_flag); band++)
              {
	      temp1 = (*(float *)(thebuf + (*(offset1 + band) + sampin)));
              for (mask = 0; (mask < masknum) && (!mask_flag); mask++)
                {
                if ((temp1 >= maskval[mask][0]) && (temp1 <= maskval[mask][1]))
                   {
                   mask_flag = TRUE;
                   mean = pixval;
		   stddev = pixval;
                   }
                }

              /* Find the sum, mean, and std dev if not mask value
                 ------------------------------------------------*/
              if (!mask_flag)
	  	 {
                 temp += temp1;
		 }
              }
	   if (!mask_flag)
	     {
	     mean = temp/ (float) totbnd;
	     stddev = 0.0;
             for (band = 0; band < totbnd; band++)
	       stddev += ((*(float *)(thebuf+(*(offset1+band)+sampin)))- mean)
		      * ((*(float *)(thebuf+(*(offset1+band)+sampin))) - mean);
              stddev = sqrt(fabs(stddev / totbnd));
	      }
	   }

	/* Assign pixel values to output
	 ------------------------------*/
	if (idtype != odtype)
	   {
	   if (mean > mxtype)
	      mean = mxtype;
	   else if (mean < mntype)
	      mean = mntype;
	   if (stddev > mxtype)
	      stddev = mxtype;
	   else if (stddev < mntype)
	      stddev = mntype;
	   }
	switch (odtype)
	   {
	case EBYTE:
	   (*(thebuf + (*(offset2) + sampout))) = mean;
	   (*(thebuf + (*(offset2+1) + sampout))) = (float)(stddev);
	   break;
	case EWORD:
	   (*(short *)(thebuf + (*(offset2) + sampout))) = mean;
	   (*(short *)(thebuf + (*(offset2+1) + sampout))) = (float)(stddev);
	   break;
        case ELONG:
           (*(long *)(thebuf + (*(offset2) + sampout))) = (long) mean;
	   (*(long *)(thebuf + (*(offset2 + 1) + sampout))) = (long)(stddev);
           break;
        case EREAL:
           (*(float *)(thebuf + (*(offset2) + sampout))) = mean;
	   (*(float *)(thebuf + (*(offset2+1) + sampout))) = (float)(stddev);
           break;
           }
	}
   /* find the minimum and maximum pixel value
   ---------------------------------------*/
   for (band = 0; band < two; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2 + band)),&ns,
                          &minval[band],&maxval[band],&odtype); 
	if (status != E_SUCC)
	   c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
	}
    curent ++;
    }
  break;
  }     

/* Final write
--------------*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
      c_clean(vblock,"math2",hosout,&one);
   }
/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = COMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&two,&upddr_flag);
if (status != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);
   c_clean(vblock,"math2",hosout,&one);
   }

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&two,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","math2-warn",&nonfatal);

/* Free buffer
--------------*/
free(thebuf);
return;
}
