/****************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the MATRIX function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      Aug  1988       converted to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   The global parameters are first received.
   Next the input and output images are gotten.
   Various validity checks are then made before returning.
******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	
                                /* image file descriptors used for image I/O*/
extern struct GDESC * gdesc;	/* group descriptor used for image I/O      */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands */
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands   */
extern long curent;		/* current processing line		    */
extern long curcat;		/* current processing band		    */

void getpar
(
    struct PARBLK *vblock,      /* pointer to TAE block	 	      */
    char taein[][CMLEN],	/* TAE file names of input images	    */
    char hosin[][CMLEN],  	/* host file names of input images	    */
    char taeout[][CMLEN],		/* TAE file name of output image	    */
    char hosout[][CMLEN],		/* host file name of output image	    */
    long *nmrimg, 		 /* number of input images		    */
    long (*bands)[MAXBND+1],         /* band specification 		            */
    long *nbands,		 /* number of bands per input image         */
    long (*window)[4],	         /* window specification		    */
    long *msgtim,		 /* elapsed time between display messages   */
    char *minmax,		 /* min max flag buffer		            */
    long *totbnd,		 /* total number of bands		    */
    float matrx[][100],	         /* array containing the coefficients       */
    long *odtype,		 /* output data type 			    */
    long *dim			 /* dimension of matrix			    */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block  */

long     element = 0;		 /* parameter element number		    */
long     fatal = -1;		 /* fatal error flag			    */
long     i,j,k;			 /* counter-no. of input images	 	    */
long     imglim;		 /* max number of images to be processed    */
long     nonfatal = 1;		 /* value of one			    */
long     qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified */
long     same;		    	 /* flag indicating if images have same wind*/
long     status;	   	 /* function return status code		    */
long     xdir;			 /* x direction dimension of matrix         */
long     ydir;			 /* y direction dimension of matrix         */

float    values[100];		 /* values of matrix in x-y direction       */
 
char     dtype[5];		 /* data type of previous input image       */

val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

/* Build the string descriptor for TAE variable "IN".
   Call getin to parse the input image specification.
   Getin returns the TAE image name(s), host image name(s),
   bands to be processed, window specification, and number
   of images parsed.
----------------------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","matrix-getin",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","matrix-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","matrix-fatal",&fatal);
   }
/* Retrieve the TAE output image name from the parameter
   block and store it in "taeout". 
   Create a host file name for the output image and store 
   it in "hosout".
----------------------------------------------------------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","matrix-getout",&fatal);
/*
   Retrieve the size of the matrix
---------------------------------------------------------------------------*/
val = p_find(vblock,"MATDIM");
xdir = IVAL(*val,0);
ydir = IVAL(*val,1);
dim[0] = xdir;
dim[1] = ydir;

if (*totbnd != ydir)
   c_errmsg("Incorrect number of bands","matrix-bands",&fatal);
/*
  Retrieve the matrix values in line sequence
---------------------------------------------------------------------------*/
val = p_find(vblock,"MATVAL");
for (i=0; i<val->v_count;i++)
  values[i] = RVAL(*val,i);
if (((xdir * ydir) != val->v_count) && (xdir * ydir) != 1)
   c_errmsg("Incorrect number of values","matrix-values",&fatal);
if (((xdir * ydir) == 1) && val->v_count != 1)
   c_errmsg("Incorrect number of values","matrix-values",&fatal);
/*
  Put the values in matrix form
--------------------------------------------------------------------------*/
k = 0;
for (i=0; i < xdir; i++)
  {
  for (j=0; j < ydir; j++)
    {
    matrx[i][j] = values[k];
    k++;
    }
  }
/*
  Retrieve the output data type
--------------------------------------------------------------------------*/
val = p_find(vblock,"odtype");
strcpy(dtype,SVAL(*val,0));
if (strcmp(dtype,"SAME") == 0)
	*odtype = 0;
else if (strcmp(dtype,"BYTE") == 0)
        *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
        *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
        *odtype = ELONG;
else if (strcmp(dtype,"R*4") == 0)
        *odtype = EREAL;
return;
}
