/***************************************************************************
NAME      		  MATRIX 

PURPOSE   Apply matrix algebra to an input image.   

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      DEC 1988       Original development
T. Mittan	Sept 1992      Added extern to curent and curcat to eliminate
			       warnings on link

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MATRIX must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)

   The LAS program is initialized.  Next the parameters are
   retrieved thru GETPAR.  The main processing routine is called
   to apply the matrix algebra.  Finally, the
   history file is updated and the routine completed.

ALGORITHM REFERENCES     
   None.

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC * fdesc[MAXIMG+1];  /* image file descriptors used for image I/O */
struct GDESC * gdesc;		     /* group descriptor used for image I/O       */
long offset1[MAXIMG*MAXBND+1];  /* offsets into "thebuf" for input bands     */
long offset2[MAXBND+1];         /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */
long bands[MAXIMG][MAXBND + 1];
                                /* band specification for first image        */
double minval[MAXBND];          /* min pixel value array		     */
double maxval[MAXBND];          /* max pixel value array		     */

int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		     */

long dim[2];			/* dimension of matrix			     */
long msgtim;			/* elpased time between display messages     */
long nmrimg;			/* number of input images for image          */
long nonfatal = 1;		/* nonfatal error flag			     */
long nbands[MAXIMG];		/* number of bands per input image           */
long odtype;			/* odtype for processing                     */
long one = 1;			/* number one				     */
long std_out;			/* logical unit # for standard output        */
long totbnd;			/* total number of bands in first image      */
long window[MAXIMG][4];		/* window specification for first image      */

static float matrx[100][100];	/* array to hold the matrix values           */

static char hosin[MAXIMG][CMLEN];/* buffer storage        	   	     */
static char hosout[1][CMLEN];       /* buffer storage        		     */
char    minmax[4];	         /* minmax flag buffer 			     */
static char taein[MAXIMG][CMLEN];/* buffer storage       		     */
static char taeout[1][CMLEN];       /* buffer storage       		     */


c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/*
       get user-specified parameters and TAE globals
*/
getpar(&vblock, taein, hosin, taeout, hosout,
       &nmrimg, bands, nbands, window, &msgtim, 
       minmax, &totbnd,matrx,&odtype,dim);
/*
   perform the matrix algebra 
*/
proces(&vblock,hosin,hosout,nmrimg,bands,nbands,
	window,msgtim,minmax,totbnd,matrx,odtype,dim); 
/*
   create associated history file
*/
if (c_hstout(&vblock, hosout[0], hosin , &nmrimg, " ") != E_SUCC)
   c_errmsg("Error accessing the history information",
		"matrix-history",&nonfatal);

/*
   inform user of successful completion
*/
c_complt("MATRIX", taeout, &one);

return EXIT_SUCCESS;

}
