/******************************************************************************
NAME:   		      PROCES 

PURPOSE    To proces the matrix algebra. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   R. McKinney     6/88		  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file

  6.1	  D. Eichacker   5/94                           Added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PROCES must be run under TAE.

PROJECT			LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   Apply the matrix algebra to the input image
   Save the minmax values if $MINMAX was set to "YES"
   Update the output ddr
   Close the input and output image files

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	
                                 /* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	 /* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND+1];/* offsets into "thebuf" for input bands*/
extern long offset2[MAXBND+1];  /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


void proces
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long (*window)[4],		/* window specification of input images      */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long totbnd,			/* total number of bands to be processed     */
    float matrx[][100],		/* matrix array for the coefficients         */
    long odtype,			/* output data type			     */
    long *dim		        /* dimension of matrix			     */
)

{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long    i;			/* counter for number of columns	     */
long	image;			/* loop index counting input images          */
long    maxxdim;		/* xdir number of matrices		     */
long    maxydim;		/* ydir number of matrices		     */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    sample;			/* sample indicator			     */
long	size = 0;			/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */

static	long	opt[3]={0,0,0}; /* option array for c_eopens                 */

float   temp;			/* temporary storage area		     */
double  temp2;			/* temporary storage area for i*4 and r*4    */

unsigned char    *thebuf;	/* image input buffer                        */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","matrix-open",&fatal);
   band += nbands[image];

   }

/* Open output image
--------------------*/
access = IWRITE;	
maxxdim = dim[0];
maxydim = dim[1];
band = maxxdim;
status = c_eopens(&fdesc[nmrimg],hosout[0],&band,&one,&one,&nl,&ns,
         offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_clean(vblock,"matrix",hosout,&one);
fdesc[nmrimg+1] = 0;

/* Determine size of output image
--------------------------------*/
switch (odtype)
  {
  case EBYTE: size = 1; break;
  case EWORD: size = 2; break;
  case ELONG: size = 4; break;
  case EREAL: size = 4; break;
  }

/* Allocate I/O buffer
----------------------*/
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","matrix-alloc",&nonfatal);
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","matrix-close",&one);
   c_clean(vblock,"matrix",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","matrix-close",&one);
   c_clean(vblock,"matrix",hosout,&one);
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("MATRIX","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&maxxdim,minval,maxval);

/* Main processing loop
-----------------------*/
/*
   processing for PROCES 
*/
for (curlin = 0;curlin < nl; curlin++)
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","matrix-close",&one);
      c_clean(vblock,"matrix",hosout,&one);
      }

   switch (odtype)
     {
     case 1:
	for (sample=0;sample<ns;sample++)
	{
	   for (i = 0;i < maxxdim; i++)
           {
	   temp = 0.0;
	   for (band = 0;band < maxydim; band++)
		temp +=
		(*(thebuf + (*(offset1+band)+sample) * size))
	         * matrx[i][band];
	   if (temp > 255.0) temp = 255.0;
	   else if (temp < 0.0) temp = 0.0;
	   (*(thebuf + (*(offset2+i)+sample) * size)) = temp;
 	   }
	}
	break;
	case 2:
	for (sample=0;sample<ns;sample++)
	{
	   for (i = 0;i < maxxdim; i++)
           {
	   temp = 0.0;
             for (band = 0; band < maxydim; band++)
		temp +=
		(*(short *)(thebuf + (*(offset1+band)+sample) * size))
	         * matrx[i][band];
	   if (temp > 32767.0) temp = 32767.0;
	   else if (temp < -32768.0) temp = -32768.0;
	   (*(short *)(thebuf + (*(offset2+i)+sample) * size)) = temp;
	   }
	 }
	break;
	case 3:
	for (sample=0;sample<ns;sample++)
	{
	   for (i = 0;i < maxxdim; i++)
           {
	   temp2 = 0.0;
             for (band = 0; band < maxydim; band++)
		temp2 +=
		(float) (*(long *)(thebuf + (*(offset1+band)+sample) * size))
	         * matrx[i][band];
	   if (temp2 > 2147483647.0) temp2 = 2147483647.0;
	   else if (temp2 < -2147483648.0) temp2 = -2147483648.0;
	   (*(long *)(thebuf + (*(offset2+i)+sample) * size)) = (long) temp2;
	   }
	 }
	break;
	case 4:
	for (sample=0;sample<ns;sample++)
	{
	   for (i = 0;i < maxxdim; i++)
           {
	   temp2 = 0.0;
             for (band = 0; band < maxydim; band++)
		temp2 +=
		(*(float *)(thebuf + (*(offset1+band)+sample) * size))
	         * matrx[i][band];
	   if (temp2 > 2147483647.0) temp2 = 2147483647.0;
	   else if (temp2 < -2147483648.0) temp2 = -2147483648.0;
	   (*(float *)(thebuf + (*(offset2+i)+sample) * size)) = temp2;
	   }
	 }
	 break;
        }
   for (band = 0; band < maxxdim; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2+band) * size),&ns,
                          &minval[band],&maxval[band],&odtype); 
	}
    curent ++;
    }
/* Final write
--------------*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","matrix-close",&one);
      c_clean(vblock,"matrix",hosout,&one);
   }
/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&maxxdim,&upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"matrix",hosout,&one);

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&maxxdim,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Error updating the minmax information","matrix-minmax",&one);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing images","matrix-close",&one);

/* Free buffer
--------------*/
free(thebuf);
return;
}
