#include <math.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void assign
(
    long num_inbands,        /* number of input bands           */
    char metric[],           /* distance type: Euclid/cityblock */
    float *clmean[],          /* class means                     */
    long *nclass,            /* number of classes               */
    long clind[],            /* class numbers                   */
    float weights[],          /* weights for each band           */
    long clcount[],          /* pixels in each class            */
    double *clsumx[],          /* sums of pixel values            */
    double *clsumsq[],         /* sums of squares of pixels       */
    double maxdist,            /* distance parameter              */
    float max_dist[],         /* distance for each class         */
    char hos_out[],          /* output image host name          */
    long wind[][4]          /* image window                    */
)

/*
**********************************************************************

NAME: ASSIGN

PURPOSE: Does class assignments 

PROGRAM HISTORY:
  Version   Date       Author         Request
  -------   ----       ------             -------
   3.0       10-Aug-88   K. Narayanan    Conversion to 5.0
   2.0       15-Apr-86   B. Kovalick 
   1.0       12-Aug-85   F. Irani  

    
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
extern struct GDESC * gdesc;
extern long ipoint[MAXIMG * MAXBND + 1];
extern long opoint;
extern long min_max;
extern long msgtim;
extern unsigned char *iobuf;
extern long zero;
extern long one;
extern long curent;

double  *ptr;
double  output_max;
double  output_min;
double  temp1;

float   dist;
float   temp[MAXCLS];
float   rmin;
float   ktemp[MAXCLS];
static  float   fzero = 0.;
float   xval;

long    i, ic, id, im, ip, j, jj, k, count;
long    ftype, ttype;
long    line;
long    msg_flag;
long    nl, np;
long    size;

unsigned char   table[MAXCLS+1];



ftype = ELONG;
ttype = EBYTE;
table[0] = 0;
c_pxconv (&ftype, &ttype, clind, &table[1], nclass);
c_inmnmx (&ftype, &one, &output_min, &output_max);

nl = wind[0][NL];
np = wind[0][NS];

if (metric[0]  ==  'E')    id = 0;
else id = 1;

if (maxdist  ==  0.0)    im = 0;
else if (maxdist  >  0.0)
    {
    im = 1;
    dist = maxdist;
    if (id  ==  0) dist *= dist;
    for (i = 0; i < *nclass; i++) max_dist[i] = dist;
    }
else
    {
    im = 2;
    if (id  ==  0)
        for (i = 0; i < *nclass; i++)
            max_dist[i] *= max_dist[i];
    } 
/*
clear out cluster sums, initialize count of assignment change,
initialize cluster counts, maximum and minimum
-------------------------------------------------------------
*/
size = num_inbands * MAXCLS;
ptr = clsumx[0];
for (i = 0; i < size; i++) *ptr++ = 0.;
ptr = clsumsq[0];
for (i = 0; i < size; i++) *ptr++ = 0.;
ttype = ELONG;
size = MAXCLS;
c_pxfill (clcount, &ttype, &size, &zero);
ttype = EREAL;
c_pxfill (ktemp, &ttype, nclass, &fzero);

output_max = 0;
output_min = *nclass;

/*
initialize and enable processing message utility
------------------------------------------------
*/
msg_flag = 0;
c_inittm ("MINDIST", "Processed", "line", "bands", " ", &wind[0][NL],
&num_inbands, &msgtim, &msg_flag);
curent = 1;
c_startm();

/*
process a line at a time
------------------------
*/
for (line = 0; line < nl; line++, curent++)
    {
    c_estep (&gdesc);
    ttype = EREAL;
    for (ip = 0; ip < np; ip++)
        {
        c_pxfill (temp, &ttype, nclass, &fzero);
        
        for (j = 0; j < *nclass; j++)
            for (i = 0; i < num_inbands; i++)
                {
                temp1 = *(float *) &iobuf[(ipoint[i]+ip) * sizeof (float)]
                - clmean[i][j];
                /*
                compute city block distance / euclidian distance
                ------------------------------------------------
                */
                if (id  !=  0)
                    temp[j] +=  weights[i] * (float)fabs(temp1);
                else
                    temp[j] += weights[i] * temp1 * temp1;
                }
            
            rmin = 1.7e38;
        jj = 0;
        
        for (k = 0; k < *nclass; k++)
            {
            if (temp[k]  <  rmin)
                {
                rmin = temp[k];
                jj = k;
                } 
            } 
        if (im  !=  0)
            if (temp[jj]  >  max_dist[jj]) jj = -1;
        
        iobuf[opoint+ip] = (unsigned char) (jj+1);
        } 
    /*
    find class totals for this line
    -------------------------------
    */
    c_pxfill (ktemp, &ttype, nclass, &fzero);
    
    for (ip = 0; ip < np; ip++)
        for (count = 0; count < *nclass; count++)
            if (iobuf[opoint+ip] == count+1) clcount[count]++;
    /*
    compute cluster sums based on current assignments
    -------------------------------------------------
    */
    for (j = 0; j < np; j++)
        {
        ic = (long ) iobuf[opoint+j];
        if (ic  !=  0)
            {
            for (k = 0; k < num_inbands; k++)
                {
                xval = *((float *) &iobuf[(ipoint[k]+j) * sizeof (float)]);
                clsumx [k][ic-1] += xval;
                clsumsq[k][ic-1] += xval*xval;
                } 
            } 
        } 
    /*
    renumber output pixel values to actual class numbers
    ----------------------------------------------------
    */
    ttype = EBYTE;
    c_pxlkup (&iobuf[opoint], &iobuf[opoint], &np, &ttype, table,
    &ttype);
    if (min_max)
        c_pxmnmx (&iobuf[opoint], &np, &output_min, &output_max, &ttype);
    } 

c_estep (&gdesc);
c_egclse (&gdesc);

c_stoptm ();

/*
update output image ddr min max fields
--------------------------------------
*/
if (min_max)
    c_svmnmx (hos_out, &one, &output_min, &output_max, "YES");
}
