
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "protos.h"

void class_stats
(
    long nclass,             /* number of classes               */
    long clind[],            /* class numbers                   */
    long clcount[],          /* pixels in each class            */
    int num_inbands,
    double *clsumx[],          /* sums of pixel values            */
    double *clsumsq[],         /* sums of squares of pixels       */
    float *clmean[],          /* class means                     */
    char cname[][CMLEN],     /* class names                     */
    long name_exist         /* names exist in statistics file? */
)

/*
**********************************************************************

NAME: CLASS_STATS                        

PURPOSE: Computes class means, standard deviations

PROGRAM HISTORY:
  Version   Date       Author         Request
  -------   ----       ------             -------
    1         25-Mar-83   G.Mcconaughy 
    2         10-Aug-88   K. Narayanan     Conversion to 5.0
    3         19-Sep-91   T. Rockvam       Fixed printout formatting of
                       class statistics

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
extern FILE *fd;

double  count, x;           /* Used to figure std dev   */

long    j, jj, k;           /* Loop indices         */

char    mbuf[132], sbuf[132];       /* Buffers to hold printout for */
char    cbuf[132];          /* mean, std dev,and band number*/

/*
compute new cluster means and standard deviations
-------------------------------------------------
*/
for (k = 0; k < nclass; k++)
    {
    if (clcount[k]  >  1)
        {
        count = clcount[k];
        for (j = 0; j < num_inbands; j++)
            {
            clmean[j][k] = clsumx[j][k] / count;
            x = fabs(count * clsumsq[j][k] - clsumx[j][k] * clsumx[j][k]) /
            (count * (count-1));
            clsumsq[j][k] = sqrt (x);    /* standard deviation */
            } 
        } 
    } 

/*
write out class names/numbers, number of pixels in classs, means,
and standard deviations
-----------------------------------------------------------------
*/
fprintf (fd,"\f\n\n*** CLASS STATISTICS BASED ON OUTPUT IMAGE***\n");
for (k = 0; k < nclass; k++)
    {
    if (name_exist)
        fprintf (fd, "\nCLASS NAME  : %s\n", cname[k]);
    else
        fprintf (fd, "\nCLASS NUMBER: %3d\n", clind[k]);
    fprintf (fd, "    NO. OF PIXELS: %10d\n", clcount[k]);
    if (clcount[k]  !=  0)
        {
        for (j = 0; j < num_inbands;)
            {
            sprintf (mbuf,"    MEAN :  ");
            sprintf (sbuf,"    STDEV:  ");
            sprintf (cbuf,"\n    BAND :  ");
            for (jj = 0; jj < 9; jj++)
                {
                sprintf (&mbuf[strlen (mbuf)], " %11.3e", clmean[j][k]);
                sprintf (&sbuf[strlen (sbuf)], " %11.3e", clsumsq[j][k]);
                sprintf (&cbuf[strlen (cbuf)], " %11d", j+1);
                if (++j == num_inbands) break;
                }
            fprintf (fd,"%s\n",cbuf);
            fprintf (fd,"%s\n",mbuf);
            fprintf (fd,"%s\n",sbuf);
            }
        } 
    } 
if (fd) fclose (fd);
}
