
#include <stdio.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

void dyn_alloc
(
    long num_inbands,       /* number of input image bands    */
    float *clmean[],         /* pointers to means              */
    double *clsumx[],         /* pointers to pixel sums         */
    double *clsumsq[]        /* pointers to sums of squares    */
)
/*
*********************************************************************
NAME: DYN_ALLOC

PURPOSE: Allocate storage for means, sums, and sum squares

PROGRAM HISTORY:
  Version   Date       Author         Request
  -------   ----       ------             -------
   1.0      10-Aug-88    K. Narayanan

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT: LAS

*********************************************************************
*/
{
extern struct PARBLK block;
extern FILE *fd;
extern double *cl_sumx;
extern double *cl_sumsq;
extern float *cl_mean;
extern long num_outfiles;
extern char out_files[2][CMLEN];
extern char *prog_name;

long    buf_size,                  /* size of storage    */
i, n;

/*
allocate storage for means, sums, and sum squares
-------------------------------------------------
*/
buf_size = num_inbands * MAXCLS;

if ((cl_mean = (float *) malloc( buf_size * sizeof (float))) == NULL ||
    (cl_sumx = (double *) malloc( buf_size * sizeof (double))) == NULL ||
    (cl_sumsq = (double *) malloc( buf_size * sizeof (double))) == NULL)
        {
        if (fd) fclose (fd);
        c_abort (&block, "Error allocating dynamic memory", "mindist-alloc",
        prog_name, out_files, &num_outfiles);
        }
/*
set up pointers
---------------
*/
for (i = 0, n = 0; i < num_inbands; i++, n += MAXCLS)
    {
    clmean[i] = &cl_mean[n];
    clsumx[i] = &cl_sumx[n];
    clsumsq[i] = &cl_sumsq[n];
    }
}
