
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char hosin[][CMLEN],      /* input image host name descriptors  */
    char hosout[],            /* output image host name descriptor  */
    long *num_inimages,       /* number of input images             */
    long num_image_bands[],   /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each      */
    long wind[][4]           /* input image windows                */
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version   Date       Author         Request
  -------   ----       ------             -------
   1.0       10-Aug-88   K. Narayanan
   6.1       24-May-94   D. Eichacker       Added opt[]
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT: LAS

**************************************************************************
*/
{
extern struct PARBLK block;
extern FILE *fd;
extern struct FDESC * fdesc[MAXIMG];
extern struct GDESC * gdesc;
extern long ipoint[MAXIMG * MAXBND + 1];
extern long opoint;
extern long num_outfiles;
extern unsigned char *iobuf;
extern char out_files[2][CMLEN];
extern char *prog_name;
extern long zero;
extern long one;




double  line_inc = 1.0,
samp_inc = 1.0;

static  long    opt[3]={0,0,0};      /* otion array for c_eopens           */
long    access,                      /* file access type                   */
bufsiz,                      /* image i/o buffer size              */
comb_flag = COMB,            /* flag for upddr                     */
n, nib,
stat,                        /* return status                      */
type;                        /* file type                          */


/*
open input and output image files
---------------------------------
*/
access = IREAD;
type = EREAL;
for (n = 0, nib = 0; n < *num_inimages; n++)
    {
    stat = c_eopens( &fdesc[n], hosin[n], image_band_num[n], &wind[0][SL],
    &wind[0][SS], &wind[0][NL], &wind[0][NS], &ipoint[nib],
    &access, &type, &zero, opt);
    if (stat != E_SUCC )
        {
        if (fd) fclose (fd);
        c_abort (&block, "Error opening input image", "mindist-openin",
        prog_name, out_files, &num_outfiles);
        }
    nib += num_image_bands[n];
    } 

access = IWRITE;
type = EBYTE;
stat = c_eopens(&fdesc[n], hosout, &one, &one, &one,
&wind[0][NL], &wind[0][NS], &opoint, &access, &type,
&zero, opt);
if (stat != E_SUCC )
    {
    if (fd) fclose (fd);
    c_abort (&block, "Error opening output image", "mindist-openout",
    prog_name, out_files, &num_outfiles);
    }
strcpy (out_files[num_outfiles++], hosout);

fdesc[++n] = 0;

/*
update output image DDR based on input images
---------------------------------------------
*/
c_upddr (&block, hosin, hosout, num_inimages, wind, image_band_num,
num_image_bands, &line_inc, &samp_inc, &one, &comb_flag);

/*
set up group containing input and output bands
----------------------------------------------
*/
if ((iobuf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
    {
    if (fd) fclose (fd);
    c_abort (&block, "Error allocating dynamic memory", "mindist-alloc",
    prog_name, out_files, &num_outfiles);
    }
c_egroup (fdesc, &gdesc, iobuf, &bufsiz);
}
