#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define TERM 0
#define LP 1
#define FIL 2
void getpar
(
    char hos_in[][CMLEN],     /* input images host names          */
    char las_out[][CMLEN],           /* output image las name            */
    char hos_out[][CMLEN],           /* output image host name           */
    char hos_stat[],          /* stats file host name             */
    long *num_inimages,       /* number of input images           */
    long num_image_bands[],   /* number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* band numbers in each    */
    long *num_inbands,        /* total number of input image bands   */
    long wind[][4],           /* input image windows              */
    long *num_classes,        /* number of classes to be used     */
    float weights[],           /* weights for each band            */
    float *maxdist,            /* max distance                     */
    char metric[],            /* distance type                    */
    float *clmean[],           /* class means                      */
    double *clsumx[],           /* sums of pixels                   */
    double *clsumsq[],          /* sums of squares of pixels        */
    char print_file[CMLEN]   /* print file name                 */
)
    
/*
**********************************************************************

NAME: GETPAR

FUNCTION: Gets parameters 

PROGRAM HISTORY:
  Version   Date       Author         Request
  -------   ----       ------             -------
   1          25-Mar-83  G. Mcconaughy 
   2          11-Apr-86  B. Kovalick 
   3           1-Dec-88  K. Narayanan     Conversion to 5.0
   4          13-Jun-89  K. Narayanan     modified to take care of null value
                                          for PRINT
   5          19-Sep-91  T. Rockvam   corrected printout formatting for
                      band numbers and weights

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Initialize program
   Get and validate parameters
   If output print file, terminal, or line printer is specified 
      Open a file
      Print header information
   Call DYN_ALLOC to allocate dynamic storage for class means,
      pixel sums, and sum of squares

**********************************************************************
*/
{
extern struct PARBLK block;
extern FILE *fd;
extern long min_max;
extern long msgtim;
extern long num_outfiles;
extern long print[];
extern char out_files[2][CMLEN];
extern char *prog_name;
extern long zero;


struct VARIABLE *var;               /* pointer to a variable in las block */

long    access,                     /* access type for getnam             */
count,
flag[MAXIMG][2],            /* flag for subwindows                */
ftype,
i, j, k,
imglimit = MAXIMG,          /* maximum number of images           */
itype,
num,
same_window,                /* flag indicating if images have     */
/* same window                        */
stat,                       /* return status                      */
std_out;                    /* logical unit for standard output   */

char    buf[132],
fileutil_file[CMLEN],       /* file name for fileutil      */
minmax[4],                  /* minmax option                      */
las_in[MAXIMG][CMLEN];      /* input images las names             */

    
/*
initialize program
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input image parameters
---------------------------------------
*/
c_getin (&block, "IN", &zero, las_in, hos_in, num_inimages, wind,
image_band_num,
num_image_bands, num_inbands, &imglimit, &same_window, flag);

if (*num_inbands  <  2 || *num_inbands > 50)
    c_lasmsg(TRUE,"mindist-bands","Number of bands in input image must be between 2 and 50");

if (same_window == 0)
    c_lasmsg(TRUE,"mindist-window","Image windows not same size");

/*
get sfin parameter
------------------
*/
access = IREAD;
stat = c_getnam (&block, "INSTAT", &zero, hos_stat, "stat", &access);
if (stat == E_FAIL)
    c_lasmsg(TRUE,"mindist-fatal","Fatal error encountered");

/*
get and validate output image parameters
----------------------------------------
*/
if (c_getout (&block, "OUT", las_out, hos_out) != E_SUCC)
    c_lasmsg(TRUE,"mindist-out","Error getting output image name");

num = 1;
itype = ELONG;
ftype = EREAL;
if (c_getnum (&block, "NCLASS", &num, &itype, (void *)num_classes) == 0)
    *num_classes = 0;
var = p_find( &block, "$runtype");

if (*num_classes > 0 && SVAL (*var, 0)[0] == 'B')
    c_lasmsg(TRUE,"mindist-batch","MINDIST must be run interactively with NCLASS > 0");
c_getnum (&block, "MAXDIST", &num, &ftype, (void *)maxdist);
if (*maxdist < -1.0)
    c_lasmsg(TRUE,"mindist-maxdist","Invalid MAXDIST value specified");
var = p_find (&block, "CALCDIST");
strcpy (metric, SVAL (*var, 0));
count = c_getnum (&block, "WEIGHTS", num_inbands, &ftype, (void *)weights);
if (count  == 0)
    for (i = 0; i < *num_inbands; i++) weights[i] = 1.0;
else if (count < *num_inbands)
    c_lasmsg(TRUE,"mindist-noweights","Weights not specified for some bands");

/*
Get print option; open print file and print header information
--------------------------------------------------------------
*/
num_outfiles = 0;
strcpy (fileutil_file, prog_name);
if (p_find( &block, "PRINT")->v_count != 0)
    {
    stat = c_setprt(&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
    print_file);
    if (stat != E_SUCC)
        c_lasmsg(TRUE,"mindist-fatal","Fatal error encountered");
    }

if (print[LP] || print[FIL] || print[TERM])
    {
    strcpy (out_files[0], fileutil_file);
    num_outfiles++;
    fd = fopen (fileutil_file, "w");
    
    fprintf (fd, "\n\n*** INPUT FILES ***\n");
    for (i = 0; i < *num_inimages; i++)
        {
        fprintf (fd, "      %s\n", las_in[i]);
        sprintf (buf, "\n     Bands:");
        for (j = 0; j < num_image_bands[i];)
            {
            for (k = 0; k < 30; k++)
                {
                sprintf (&buf[strlen (buf)], " %3d", image_band_num[i][j]);
                if (++j == num_image_bands[i]) break;
                }
            fprintf (fd,"%s\n\n",buf);
            }
        }
    fprintf (fd, "*** OUTPUT IMAGE ***\n");
    fprintf (fd, "     %s\n", las_out[0]);
    fprintf (fd, "\n*** INPUT STATISTICS FILE ***\n");
    fprintf (fd, "     %s\n", hos_stat);
    fprintf (fd, "\n*** PARAMETERS ***\n");
    fprintf (fd, "\n     CALCDIST     MAXDIST\n");
    fprintf (fd, "     --------     -------\n");
    fprintf (fd, "\n  %10s  %11.4f\n", metric, *maxdist);
    fprintf (fd, "\n*** WEIGHTS ***\n");
    for (i = 0; i < *num_inbands;)
        {
        buf[0] = '\0';
        for (j = 0; j < 13; j++)
            {
            sprintf (&buf[strlen (buf)], "    %6.2f", weights[i]);
            if (++i == *num_inbands) break;
            }
        fprintf (fd,"%s\n",buf);
        }
    } 

/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y' || minmax[0] == 'y') min_max = TRUE;
else min_max = FALSE;

/*
allocate dynamic storage for mean, sums, and sums of squares
------------------------------------------------------------
*/
dyn_alloc (*num_inbands, clmean, clsumx, clsumsq);

}
