/*******************************************************************************

NAME: MINDIST

PURPOSE:
   Classifies a multispectral input image using a minimum distance measure 
   from class means obtained from an input statistics file

PROGRAM HISTORY:
  Version   Date       Author             Request
  -------   ----       ------             -------
     1       25-Mar-83   G. Mcconaughy
     2       15-Apr-86   B. Kovalick 
     3       23-Nov-88   K. Narayanan      Conversion to 5.0
     4       20-Sep-91   T. Rockvam    	   Fixed formatting of printout 
     5       11-Sep-92   T. Mittan         Eliminated global include file
                       			   to eliminate compiler warnings
     5.1     05-Aug-93   D. Bridges        Added TERM to print parameter 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT: LAS

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "protos.h"

#define TERM 0
#define LP   1
#define FIL 2

struct PARBLK   block;                         /* TAE parameter block        */
FILE            *fd;                           /* output file descriptor     */
double          *cl_sumx;                      /* dynamic storage for        */
double          *cl_sumsq;                     /* sums and sums of squares   */
float           *cl_mean;                      /* and means                  */
struct FDESC    *fdesc[MAXIMG];                /* file descriptors           */
struct GDESC    *gdesc;                       /* group descriptor           */
long            ipoint[MAXIMG * MAXBND + 1],   /* offsets into image i/o     */
                opoint;                        /* for input and output bands */
long            min_max;                       /* minmax option flag         */
long            msgtim;                        /* message interval           */
long            num_outfiles;                  /* number of output files     */
long            print[3];
unsigned char   *iobuf;                        /* image i/o buffer           */
char            out_files[2][CMLEN];           /* output file names          */
char            *prog_name = "MINDIST";        /* program name               */

extern  long    curcat;                        /* parameters for processing  */
extern  long    curent;                        /* message utility            */

long			one=1;
long			zero=0;

int main
(void) 
{

static double   *clsumx[MAXBND],         /* sums of pixels                   */
                *clsumsq[MAXBND];        /* sums of squares of pixels        */

static float    maxdist,                 /* max distance from cluster center */
                max_dist[MAXCLS],        /* max distance for each class      */
                weights[MAXBND],         /* weights for each band            */
                *clmean[MAXBND];         /* class means                      */

long    image_band_num[MAXIMG][MAXBND+1];/* band numbers in each input image */

static long     clcount[MAXCLS],         /* number of pixels in each class   */
                clind[MAXCLS],           /* class numbers to be used         */
                num_classes,             /* number of classes                */
                num_image_bands[MAXIMG], /* # of bands in each input image   */
                num_inbands,             /* total number of input image bands*/
                num_inimages,            /* number of input images           */
                name_exist,              /* class names exist in stats file? */
                wind[MAXIMG][4];         /* input image windows              */

char            class_names[MAXCLS][CMLEN],/* class name parameter            */
                hos_in[MAXIMG][CMLEN];   /* input images host names          */

static char     hos_out[1][CMLEN],       /* output image host name           */
                hos_stat[CMLEN],         /* stats file host name             */
                metric[8],               /* distance type: EUCLID, CITY BLOCK */
                las_out[1][CMLEN],       /* output image las name            */
                print_filename[CMLEN];   /* print file name                  */



/* get parameters
-----------------*/
getpar (hos_in, las_out, hos_out, hos_stat, &num_inimages, num_image_bands, 
	image_band_num, &num_inbands, wind, &num_classes, weights, &maxdist, metric,
	clmean, clsumx, clsumsq, print_filename);


/* read statistics file
------------------------*/
read_stats (hos_stat, num_inbands, clmean, class_names, &name_exist, 
	&num_classes, clind);


/* prompt for distance threshold values if needed
-------------------------------------------------*/
if (maxdist  ==  -1.0)
    thresh (num_classes, class_names, max_dist);


/* set up file specifications
-------------------------------*/
filspec (hos_in, hos_out[0], &num_inimages, num_image_bands, 
         image_band_num, wind);


/* assign pixels to classes
-----------------------------*/
assign (num_inbands, metric, clmean, &num_classes, clind, weights, clcount, 
	clsumx, clsumsq, maxdist, max_dist, hos_out[0], wind);


/* compute and print class statistics if needed
------------------------------------------------*/
if (print[LP] || print[FIL] || print[TERM])
    class_stats (num_classes, clind, clcount, num_inbands, clsumx, clsumsq,
    	clmean, class_names, name_exist);


/* terminate processing
------------------------*/
terminate (las_out, hos_out[0], hos_in, &num_inimages, print_filename);


return EXIT_SUCCESS;

}
