/*******************************************************************************

NAME: READ_STATS

PURPOSE: Gets class statistics from statistic file

PROGRAM HISTORY:
  Version   Date       Author         Request
  -------   ----       ------             -------
     1       25-Mar-83   G. Mcconaughy
     2       10-Aug-88   K. Narayanan      Conversion to 5.0
     3       19-Sep-91   T. Rockvam    Corrected printout formatting for
                       initial class means

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Open statistics file
   If only selected classes are needed call CLPROMPT to get desired 
      class names
   Read and output class statistics
   Close statistics file

*******************************************************************************/
#define TERM 0
#define LP 1
#define FIL 2
#include <stdio.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

void read_stats
(
    char hos_stat[],        /* stats file host name              */
    long num_inbands,       /* number of input bands             */
    float *clmean[],         /* class means                       */
    char cname[][CMLEN],    /* class names                       */
    long *name_exist,       /* class names exist in stats file?  */
    long *nclass,           /* number of classes requested/used  */
    long clind[]           /* class numbers used                */
)

{
extern struct PARBLK block;
extern FILE *fd;
extern long num_outfiles;
extern long print[3];
extern char out_files[2][CMLEN];
extern char *prog_name;
extern long zero;

double  mean[MAXBND];              /* mean vector                       */

long    access;
long    cshape[9], shape[9];       /* shape vectors                     */
FILE *  fd_sfin;                   /* stats file descriptor             */  
long    i, ic, j, jj, k;
long    len;               /* length of class name      */
long    nclass_stats;              /* number of classes in stats file   */
long    prompt;                    /* prompt flag                       */
long    stat;

char    *buf;


/* open stats file
-------------------*/
access = IREAD;
stat = c_stopen (&fd_sfin, hos_stat, &access, &nclass_stats);
if (stat  !=  E_OPOLD)
    {
    if (fd) fclose (fd);
    c_abort (&block, "Error opening statistics file", "mindist-sfopen",
    prog_name, out_files, &num_outfiles);
    }
if (nclass_stats  <  1)
    {
    if (fd) fclose (fd);
    c_abort (&block, "No classes in statistics file", "mindist-noclass",
    prog_name, out_files, &num_outfiles);
    }


/* get class indices - prompt if needed
-----------------------------------------*/
prompt = FALSE;
if (*nclass  ==  0 )
    {
    if (nclass_stats  >  MAXCLS) 
        {
        *nclass = MAXCLS;
        c_lasmsg(FALSE,"mindist-maxcls",
		"Stats file has more than %3d classes; will use first %3d classes",
        *nclass, *nclass);
        }
    else
        *nclass = nclass_stats;
    
    for (i = 0; i < *nclass; i++) clind[i] = i+1;
    }
else if (*nclass > nclass_stats)
    {
    c_lasmsg(FALSE,"mindist-nclass1",
	"NCLASS larger than total number of classes in stats file");
    *nclass = nclass_stats;
    c_lasmsg(FALSE,"mindist-nclass2",
	"Number of classes to be used is now reset to: %3d", *nclass);
    } 
else
    {
    prompt = TRUE;
    clprompt (&fd_sfin, nclass_stats, nclass, clind);
    c_lasmsg(FALSE,"mindist-class","The following classes were selected");
    } 


/* get class(cluster) names and mean vectors
---------------------------------------------*/
if (print[LP] || print[FIL] || print[TERM])
    {
    fprintf (fd, "\f\n\n *** INITIAL CLASS MEANS AS READ ");
    fprintf (fd, "FROM STATISTICS FILE ***\n");
    } 
*name_exist = FALSE;
for (k = 0; k < *nclass; k++)
    {
    ic = clind[k];
    stat = c_stget (&fd_sfin, "NAME", &ic, &zero, cshape, cname[k]);
    if (stat  ==  E_SUCC && !(*name_exist))
        *name_exist = TRUE;
    len = cshape[2];
    cname[k][len] = '\0';
    
    stat = c_stget (&fd_sfin, "MEAN_VECTOR", &ic, &zero, shape, mean);
    if (stat  !=  E_SUCC)
        {
        if (fd) fclose (fd);
        c_abort (&block, "No mean for one of the classes in statistics file",
        "mindist-nomean", prog_name, out_files, &num_outfiles);
        }
    if (shape[2]  !=  num_inbands)
        {
        if (fd) fclose (fd);
        c_abort (&block,
        "Number of bands in mean of statistics file does not match image",
        "mindist-nbands", prog_name, out_files, &num_outfiles);
        }
    for (i = 0; i < shape[2]; i++) clmean[i][k] = mean[i];

    
    /* write out class name (if available, if not class number)
       and mean vector to output if desired
    -----------------------------------------------------------*/
    if (print[LP] || print[FIL] || print[TERM])
        {
        if (*name_exist)
            {
            fprintf (fd, "\nCLASS NAME  :  %s\n", cname[k]);
            }
        else
			{
            fprintf (fd, "\nCLASS: %3d\n", ic);
			}        
		buf = (char *) calloc(12+108*shape[2]+1, sizeof(char));
		if (buf==NULL)
			c_lasmsg(TRUE,"mindist-alloc", "Memory allocation error");
        strcpy (buf, "    MEAN :  ");
        for (j = 0; j < shape[2];)
            {
            for (jj = 0; jj < 9; jj++)
                {
                sprintf (&buf[strlen (buf)], " %11.3e", clmean[j][k]);
                if (++j == shape[2]) break;
                }
            fprintf (fd,"%s\n",buf);
            }
        fprintf(fd,"\n");
        } 
    

    /* if classes have been specified print out class name (or number)
       for session logging
    -------------------------------------------------------------------*/
            if (prompt)
                {
                if (*name_exist)
                    c_lasmsg(FALSE, "i", cname[k]);
                else
                    c_lasmsg(FALSE, "i", " %3d", ic);
                }
    } 


/* close statistics file
--------------------------*/
c_stclse(&fd_sfin);

}
