/*******************************************************************************
NAME      		  GETPAR 

PURPOSE
	GETPAR will retrieve and perform the final editing of the 
	parameters for the MINMAX module.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      May  1988       converted to LAS 5.0
R.MCKINNEY      Dec  1988       Removed the call to mknam

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MINMAX must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   GETPAR retrieves the required TAE parameters from the parameter block.  The
   input images are parsed so that the window and band specifications are
   known.
******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block            */
    char taein[][CMLEN],		/* pointer to input TAE file descriptor      */
    char hosin[][CMLEN],		/* pointer to input host file descriptor     */
    long *nmrimg,		/* number of input images                    */
    long (*window)[4],		/* window specifications for each input image*/
    long (*bands)[MAXBND+1],	/* band specifications for each input image  */
    long nbands[],		/* number of bands per input image	     */
    long *totbnd,		/* total number of bands to be processed     */
    long *msgtim,		/* elapsed time between display messages     */
    char *minmax,		/* min max flag buffer			     */
    long *term,			/* flag for output to terminal 		     */
    long *fyle,			/* flag for output to file		     */
    long *lnpr,			/* flag for output to line printer	     */
    char *filnm,			/* output filename for FILE option           */
    char *outfil 		/* pointer to output host file name          */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block  */

long    element = 0;		/* element number for getin                  */
long    fatal = -1;		/* fatal error flag			     */
long	imglim;			/* maximum number of images to be processed  */
long	qflag[MAXIMG][2];	/* flag indicating if subwindows specified   */
long	same;			/* flag indicating if images have same window*/
long	status;			/* function return status code               */

imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","minmax-fatal", &fatal);

/*
   Retrieve the print option
*/
  strcpy(filnm,"minmax");
  status = c_setprt(vblock,term,fyle,lnpr,filnm,outfil);
  if (status != E_SUCC)
     c_errmsg("Fatal error encountered","minmax-fatal",&fatal);

/*
   Retrieve TAE globals
*/

val = p_find(vblock,"$msgtim");	
*msgtim = IVAL(*val,0);

/*
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));
*/
strcpy(minmax,"YES");

return;
}
