/******************************************************************************

NAME:   MINMAX	

FUNCTION:
	MINMAX is a function which will find the minimum and maximum
	values within an image.

PROGRAM HISTORY:
  Version	Date       Author      	Request
  -------	----	   ------       -------
    1.0 	9/82       G. McConaughy LAS development
    2.0 	8/82       J. Stansbery
    5.0         2/88       R. McKinney   LAS 5.0 development
    5.0        12/88       R. McKinney   Updated to close files.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
         MINMAX	runs under TAE

PROJECT:	LAS 5.0

ALGORITHM (FLOW):
 	The TAE input parameters are retrieved.  Error checking is done on the 
        input image(s) to insure that they all have equal window sizes.  Once 
	the TAE parameters are inputted, the input image is read. 
	After the image has been read, the min max file is created/updated.  

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK vblock;			/* TAE parameter block               */

static long bands[MAXIMG][MAXBND+1];	/* bands to be processed - all images*/
long    fyle;				/* flag for output to file	     */
long    lnpr;				/* flag for output to line printer   */
long	msgtim;				/* elapsed time between display msg  */
long	nbands[MAXIMG];  		/* number of bands per input image   */
long    nfiles;				/* number of output files created    */
long	nmrimg;				/* number of input images            */
long	std_out;			/* LUN to standard output    	     */
long    term;				/* flag for output to terminal       */
long	totbnd;				/* total number of bands             */
long	window[MAXIMG][4];		/* window for each input image       */

char    filnm[CMLEN];			/* output file name for FILE option  */
char	hosin[MAXIMG][CMLEN];   	/* host names of input images        */
char	minmax[4];			/* min max flag                      */
char    outfil[1][CMLEN];			/* output file name passed to complt */
char	taein[MAXIMG][CMLEN];	        /* TAE names of input images         */

/*
   initialization
*/
c_inlas(&vblock,&std_out);

/*
   get parameters and globals
*/
getpar(&vblock,taein,hosin,&nmrimg,window,bands,nbands,
       &totbnd,&msgtim,minmax,&term,&fyle,&lnpr,filnm,outfil[0]);
/*
   process image
*/
proces(taein,hosin,nmrimg,window,bands,nbands,
	   term,fyle,lnpr,filnm,&nfiles);

/* 
   output completion message 
*/
c_complt("MINMAX",outfil,&nfiles);

return EXIT_SUCCESS;

}
