/*******************************************************************************
NAME      		   PROCES


PURPOSE  PROCES opens the input images for read access and generates a  
	 report of statistics.  If appropriate it will update the DDR
	 showing the correct MIN and MAX values.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      May  1988       converted to LAS 5.0
R.MCKINNEY      Dec  1988	Updated to close all files.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   MINMAX must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   PROCES opens the input images for read access and generates the minimum
   and maximum values.  The subroutine c_minmax is called to either
   calculate the min/max or retrieve it from the DDR file.  A report
   is generated going to the terminal, file, and/or printer.
******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

void proces
(
    char taein[][CMLEN],         /* TAE file name                             */
    char hosin[][CMLEN],		/* pointer to input host file descriptor     */
    long nmrimg,			/* number of input images                    */
    long (*window)[4],		/* window specification of input images      */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long term,			/* flag for output to terminal               */
    long fyle,			/* flag for output to file                   */
    long lnpr,			/* flag for output to line printer           */
    char filnm[CMLEN],	        /* output filename for FILE option           */
    long *nfiles  		/* number of output files		     */
)
{
static double minval[MAXBND+1]; /* min pixel value array		     */
static double maxval[MAXBND+1]; /* max pixel value array		     */

long	band;			/* loop index counting the bands             */
long	image;			/* loop index counting input images          */
long    temp;			/* temporary counter	                     */
long    temp2;			/* second temporary counter		     */
long	zero = 0;		/* value of zero                             */

char    outmsg[CMLEN];		/* output message to the terminal	     */
FILE    *fp = NULL;			/* file pointer to print file                */


*nfiles = 0;
if (fyle == 1)
    *nfiles = 1;
if ((lnpr == 1) || (fyle == 1))
     fp = fopen(filnm,"w");
/*
  call minmax to return the min and max values
*/

c_minmax(hosin,&nmrimg,window,bands,nbands,minval,maxval);

/*
  write min max values to term
*/
 temp = temp2 = 0;
 for (image=0; image<nmrimg; image++)
  {
 if (term == 1)
    {
    sprintf(outmsg,"\fImage: %s\n",taein[image]);
    c_pterm(outmsg,&zero);
    for (band=0; band<nbands[image]; band++)
        {
        sprintf(outmsg,
                "  Band Number: %d  Minimum value: %f  Maximum value: %f",
		   (band+1),minval[temp],maxval[temp]);
	c_pterm(outmsg,&zero);
	temp += 1;
	}
    }
/*
  write min max values to file or printer
*/
 if ((lnpr == 1) || (fyle == 1))
    {
    fprintf(fp,"\fImage: %s\n",taein[image]);
    for (band=0; band<nbands[image]; band++)
	{
        fprintf(fp,"  Band Number: %d  Minimum value: %f  Maximum value: %f\n",
		   (band+1),minval[temp2],maxval[temp2]);
	temp2 += 1;
	}
    }
  } 
  if ((lnpr == 1) || (fyle == 1))
  fclose(fp);
return;
}
