
/*******************************************************************************
NAME:     byte_image.c
PURPOSE:  functions to process byte images
          this file is similar to real_image.c 
*******************************************************************************/


#include "msscolor.h" /* application header file */ 
#include "util2.h"



/*******************************************************************************
NAME:     byte_image 
PURPOSE:  create the output byte image
*******************************************************************************/

FUNCTION void byte_image (inimage, outimage)
   struct INIMAGE  *inimage;  /* input:  specs for input image */ 
   struct OUTIMAGE *outimage; /* input:  specs for output image */ 
{

/* declare variables */ 
   long band,                   /* loop index, image band */ 
        line,                   /* loop index, image line */ 
        messform=ALLCAT;        /* output message format, parm to c_inittm() */ 
   unsigned char *inpix[INBANDS],  /* points to row of  input pixels */ 
	        *outpix[OUTBANDS]; /* points to row of output pixels */ 
   struct LUTS luts;               /* luts to speed processing of byte data */ 
   extern long curent;             /* current entity, for processing message */ 

/* initialize luts */ 
   init_luts (&luts); 

/* initialize and start processing line message */ 
   curent = 1; 
   c_inittm ("MSSCOLOR", "Processed", "line", "bands", "", &outimage->nl,
             &outimage->totalbands, &outimage->msgtim, &messform); 
   c_startm (); 

/* process image */ 
   for (line=0;  line<outimage->nl;  line++, curent++)
      {

      /* read a line of data */ 
	 read_bil (line, inimage); 

      /* get inpix and outpix */ 
	 for (band=0;  band<inimage->totalbands;  band++) 
	    inpix[band]  =  inimage->linebuf[band]; 
	 for (band=0;  band<outimage->totalbands;  band++)
	    outpix[band] = outimage->linebuf[band]; 

      /* calculate pixel values for a line of data */ 
	 byte_outvals (&luts, inimage->ns, inpix, outpix); 

      /* update minmax */ 
	 minmax_bil (outimage); 

      /* write a line of data */ 
	 write_bil (line, outimage); 
      }

c_stoptm(); 
}








/*******************************************************************************
NAME:     byte_outvals
PURPOSE:  calculate output pixel values for a line of input data
*******************************************************************************/

FUNCTION void byte_outvals (luts, ns, inpix, outpix)
   struct LUTS *luts;             /* input:  luts for byte data */ 
   long ns;                       /* input:  number of image samples */ 
   unsigned char *inpix[MAXBND];  /* input:  a row of input pixels */ 
   unsigned char *outpix[MAXBND]; /* output:  a row of output pixels */ 

{
/* declare variables */ 
   unsigned char 
	 chan4, chan5, chan6, chan7;   /* input data for satellite channels 
					  4,5,6,7;  corresponds to image bands 
					  1,2,3,4 after subbanding */ 
   float ratio,             /* ratio:  chan5 / chan6 */ 
	 weight,            /* a weight between two types of calculations */ 
         red, green, blue;  /* output data for image bands 1,2,3 */
   long  band,              /* loop index, image band */ 
         sample;            /* loop index, image sample */ 

/* loop for all samples */ 
   for (sample=0;  sample<ns;  sample++)
   {

   /* set variables for input satellite channels */ 
      chan4 = *inpix[0]; 
      chan5 = *inpix[1];
      chan6 = *inpix[2];
      chan7 = *inpix[3]; 

   /* determine ratio between channels 5 and 6 */ 
      ratio = luts->ratlut [chan5][chan6]; 

   /* compute output pixel values based on ratio. 
      the following (float) type casts eliminated warnings from lint. 
   *****************************************************************/

      if (ratio < 0.56)   
	 /* pixel is all vegetation */ 
	 {
	 red   = luts->f0_75 [chan5]; 
	 green = luts->f1_5  [chan4]; 
	 blue  = luts->f1_125 [chan4] - luts->f0_1875 [chan6]; 
	 }

      else if (ratio < 0.65) 
	 /* pixel is a weighted mixture of vegetation and soil */ 
	 {
	 weight = (ratio - 0.56) / (0.65 - 0.56);
	 red    = weight * chan5 + 
		  ( (float)1 - weight) * luts->f0_75 [chan5]; 
	 green  = weight * luts->f2_0 [chan4] + 
		  ( (float)1 - weight) * chan4; 
	 blue   = weight * luts->f1_5 [chan4] - luts->f0_25 [chan6] + 
	 	  ( (float)1 - weight) * luts->f2_0 [chan4] - 
		  luts->f0_35 [chan5] - chan7;

	 red   *= 0.75; 
	 green *= 0.75; 
	 blue  *= 0.75; 
	 }

      else if (ratio < 1.50)
	 /* pixel is soil */ 
	 {
	 red   = luts->f0_5625 [chan5];
	 green = luts->f0_75   [chan4];
	 blue  = (float)0.75 * (luts->f2_0[chan4] - luts->f0_35 [chan5]-chan7 );
	 }

      else 
	 /* pixel is water */ 
	 {
	 red   = luts->f0_75 [chan5];
	 green = luts->f0_75 [chan4];
	 blue  = (float) 0.75 * (luts->f2_0 [chan4] - chan5 ); 
	 }

   /* correct for round-off error */
      if (red  >=0)  red  +=0.5;  else  red  -=0.5; 
      if (green>=0)  green+=0.5;  else  green-=0.5;
      if (blue >=0)  blue +=0.5;  else  blue -=0.5;

   /* convert to byte range */ 
      if (red   > BYTEMAX)   red   = BYTEMAX; 
      if (green > BYTEMAX)   green = BYTEMAX; 
      if (blue  > BYTEMAX)   blue  = BYTEMAX; 

      if (red   < BYTEMIN)   red   = BYTEMIN; 
      if (green < BYTEMIN)   green = BYTEMIN; 
      if (blue  < BYTEMIN)   blue  = BYTEMIN; 

   /* copy rgb data to output buffer */ 
      *outpix[0] = (unsigned char) red; 
      *outpix[1] = (unsigned char) green;
      *outpix[2] = (unsigned char) blue; 

   /* increment pointers to input and output pixels */ 
      for (band=0;  band<INBANDS;  band++)
	 inpix[band] ++;
      for (band=0;  band<OUTBANDS;  band++)
	 outpix[band] ++;

   }
} 
