
/*******************************************************************************
NAME:     get_types.c
PURPOSE:  functions to return types of input and output images 
*******************************************************************************/

#include <ctype.h>     /* for call to isspace() */ 
#include <string.h>    /* for call to strchr() */ 
#include "msscolor.h"  /* application header file */ 
#include "util.h"


/*******************************************************************************
NAME:     get_out_type 
PURPOSE:  Return data type for the output image, implicit input is tae parm 
	  odtype.  Some applications do not have an input image, hence 
	  get_out_type() does not have "struct *inimage" as an input argument.  
*******************************************************************************/

long get_out_type
(
   struct PARBLK *pblock,  /* input:  tae parameter block */ 
   char *parmin            /* input:  name of tae parm for input image.  pass 
                                      NULL if there is no input image*/
)
{
    char *odtype;            /* user value for odtype */
    struct VARIABLE *tparm;  /* points to a tae parm in pblock */ 

    /* set data type based on user value for odtype */ 
    tparm = p_find (pblock, "odtype");             
    odtype = SVAL (*tparm,0);
    if      ( !strcmp (odtype, "SAME"))  return (get_in_type (pblock,parmin)); 
    else if ( !strcmp (odtype, "BYTE"))  return (EBYTE); 
    else if ( !strcmp (odtype, "I*2" ))  return (EWORD); 
    else if ( !strcmp (odtype, "I*4" ))  return (ELONG); 
    else if ( !strcmp (odtype, "R*4" ))  return (EREAL); 
    else
        errclean ("msscolor-fatal", "ODTYPE has an invalid value"); 

    return 0;
}


/*******************************************************************************
NAME:     get_in_type
PURPOSE:  Return data type of the input image. 
	  This function is called by get_out_type().  
*******************************************************************************/

long get_in_type
(
    struct PARBLK *pblock,  /* input:  tae parameter block */ 
    char *parmin            /* input:  name of tae parm for input image */
)
{
    /* declare variables needed for calling c_getin() */ 
    char hostname[MAXIMG][CMLEN],/* host name for image files */ 
    taename[MAXIMG][CMLEN]; /* tae name for image files */ 
    long bands[MAXIMG][MAXBND+1],/* band numbers for each image file */ 
    dtypein,                /* image data type */
    maximg=MAXIMG,          /* max number of images allowed */ 
    nbands[MAXIMG],         /* number of bands in each image file */ 
    nmrimg,                 /* number of disk image files */ 
    one = 1,                /* value of 1 */ 
    qflag[MAXIMG][2],       /* flag, do image components have a band or
                               window specified */ 
    samewindow,             /* flag, do image components have the same 
                               window specified? */ 
    totalbands,             /* total # of bands for all image files */
    window[MAXIMG][4],      /* window (sl ss nl ns) for each image file */
    zero = 0;               /* value of 0 */ 

    /* call c_getin() */ 
    if(c_getin (pblock, parmin, &zero, taename, hostname, &nmrimg, window, 
                bands, nbands, &totalbands, &maximg, &samewindow, qflag) 
            !=E_SUCC)
    {
        errclean ("msscolor-fatal", "Fatal error encountered"); 
    }

    /* get input data type.  
       get datatype of 1st disk image, supporting routines convert other images
       to data type of first image */ 
    if (c_gettyp (hostname, &one, &dtypein) != E_SUCC)
        errclean ("msscolor-fatal", "Fatal error encountered"); 

    /* return type */ 
    return (dtypein);
}





