/*******************************************************************************

NAME:     msscolor.c 

PURPOSE:  The main() for msscolor. 
	  Create a simulated natural color image from a MSS image. 

PROGRAM HISTORY
--------------------------------------------------------------------------------
Date         Programmer       Changes 
-----------  ---------------  --------------------------------------------------
 1-Aug-1974  Pat Chavez       Original development at U.S. Geological Survey 
			      Image Processing Facility / 2255 N Gemini Drive 
			      Flagstaff, AZ  86001
21-Jan-1975  Pat Chavez       Do not divide by zero.
 1-Sep-1979  Randall Davis    Handle byte arrays instead of word arrays.
 4-Oct-1982  Steve Wolfe      Add programmer documentation section.
   Jul-1986  Bill Eme         Transferred from PDP11/73 to MV2
	                      Translated code from MACRO to FORTRAN
   Jul-1987  Bill Eme         Fixed problem with FORTRAN code by changing 
			      algorithm
25-Feb-1993  George Lopez     Separated from douser routine when converted to 
	                      portable code.
22-Apr-1994  Jeff Anderson    Modified equations for pixels classified as
			      vegetation and soils mixture.  Now the equations
			      for soils and vegetation are weighted depending
			      on the value of the 5/6 ratio.  If the ratio is
			      closer to the vegetation cutoff then a higher
			      weight will be applied to the vegetation equation
			      and a lower to the soils and vice versa for a
			      ratio closer to the soil cutoff.
23-Aug-1995  David Zokaites   Acquired MSSBLUE MIPS Fortran application from 
			      Jeff Anderson at USGS in Flagstaff AZ and 
			      converted it to MSSCOLOR LAS application in C;  
			      released application to $LASDITTY. 
13-Sep-1995  David Zokaites   Some minor changes after application received 
			      internal review;  released to $LASBASE.  
*******************************************************************************/

#include "msscolor.h" /* application header file */ 
#include "util2.h"


/* variables common to main() and errclean() */ 
   struct INIMAGE  inimage;  /* info on input las image */ 
   struct OUTIMAGE outimage; /* info on output las image */ 


int main ()                 
{	                    
    /* declare variables */ 
    long one=1,            /* value of one */ 
    std_out,          /* standard output, parm to c_inlas() */ 
    isbyte;           /* flag, are input and output images both byte? */ 
    struct PARBLK pblock;  /* tae parameter block */ 

    /* initialize las and tae */ 
    c_inlas (&pblock, &std_out); 

    /* intialize image status flags used for error handling, must be first for
       correct error handling */ 
    inimage.opened =  inimage.closed = FALSE; 
    outimage.opened = outimage.closed = FALSE;

    /* get types of input and output images */ 
    inimage.dtypein   = get_in_type  (&pblock, "IN"); 
    outimage.dtypeout = get_out_type (&pblock, "IN"); 

    /* set flag.  luts are used to speed processing of byte images, other images 
       are processed as r*4 */ 
    if (inimage.dtypein == EBYTE && outimage.dtypeout == EBYTE)
        isbyte = TRUE;
    else
        isbyte = FALSE;

    /* open input image */ 
    open_bil_in  (&pblock, "IN", (isbyte ? NOTAPPLY : EREAL), &inimage); 
    if (inimage.totalbands != INBANDS)
        errclean ("msscolor-nbands", "IN does not have %d bands", INBANDS); 

    /* open output image */ 
    open_outimage (&pblock, "OUT", inimage.dtypein, (isbyte ? NOTAPPLY : EREAL),
            outimage.dtypeout, BILIO, inimage.nl, inimage.ns,
            OUTBANDS, &outimage);

    /* create image */ 
    if (isbyte) 
        byte_image (&inimage, &outimage); 
    else 
        real_image (&inimage, &outimage); 

    /* end program */ 
    close_inimage  (&inimage);                     /* close image files */ 
    close_outimage (&outimage);                    /* close image files */ 
    finish_out (&pblock, &inimage, &outimage);     /* make ddr & history file */ 
    c_complt ("MSSCOLOR", outimage.taename, &one); /* end program */ 

    return 0;
}



#ifdef ANOTE /* this ifdef'ed function call will pop up from    % lasmsg */ 
   c_lasmsg (ReadThisNote, 
   The unix application lasmsg can not be used directly on this application
   because of the function errclean.  Run tests/lasmsg2 instead.   );
#endif 


