
/*******************************************************************************
NAME:     msscolor.h
PURPOSE:  application header file 
*******************************************************************************/


#include "las.h"   /* las include file */ 
#include "image.h" /* for image data structures and function prototypes */ 

#define INBANDS   4    /* n bands for input images */ 
#define OUTBANDS  3    /* output images are r,g,b for 3 bands */ 
#define LUTMIN    0    /* min index to luts */ 
#define LUTMAX  255    /* max index to luts */ 
#define LUTVALS 256    /* n look-up-table values for byte data */ 
#define BYTEMIN   0.0  /* min byte output value */ 
#define BYTEMAX 255.0  /* max byte output value */ 




/* luts to speed processing of byte data.  luts could be dynamically allocated,
   but they are used for most executions so I did it the easy way. */ 
   struct LUTS 
   {
      /* lut input is data value, lut output is data value * factor where 
	 factor = lut name.  For example, f2_0 [244] = 2.0 * 244.0 = 122.0 */ 
	 float f0_1875 [LUTVALS], 
	       f0_25   [LUTVALS], 
	       f0_35   [LUTVALS], 
	       f0_5625 [LUTVALS], 
	       f0_75   [LUTVALS], 
	       f1_125  [LUTVALS], 
	       f1_5    [LUTVALS], 
	       f2_0    [LUTVALS];

      /* scaled ratio of 2 bands */ 
	 float ratlut[LUTVALS][LUTVALS];  
   };




/* byte_image.c */ 
void byte_image (struct INIMAGE *inimage, struct OUTIMAGE *outimage); 
void byte_outvals (struct LUTS *luts, long ns, 
	   unsigned char *inpix[INBANDS], unsigned char *outpix[OUTBANDS]); 
/* get_types.c */ 
long get_out_type (struct PARBLK *pblock, char *parmin); 
long get_in_type  (struct PARBLK *pblock, char *parmin); 
/* init_luts.c */ 
void init_luts (struct LUTS *fixedlut);
/* real_image.c */ 
void real_image (struct INIMAGE *inimage, struct OUTIMAGE *outimage);
void real_outvals (long roundfix, long ns, float *inpix[INBANDS],
	   float *outpix[OUTBANDS]); 
