
/*******************************************************************************
NAME:     real_image.c
PURPOSE:  functions to process images which are not byte 
          this file is similar to byte_image.c 
*******************************************************************************/

#include "msscolor.h" /* application header file */ 
#include "util2.h"


/*******************************************************************************
NAME:     real_image 
PURPOSE:  create the output image, processing the data as r*4, conversion takes
	  place when data is read from and written to disk  
*******************************************************************************/

FUNCTION void real_image (inimage, outimage)
   struct INIMAGE  *inimage;  /* input:  specs for input image */ 
   struct OUTIMAGE *outimage; /* input:  specs for output image */ 
{

/* declare variables */ 
   float *inpix[INBANDS],   /* points to row of  input pixels */ 
	 *outpix[OUTBANDS]; /* points to row of output pixels */ 
   long band,               /* loop index, image band */ 
        line,               /* loop index, image line */ 
        messform=ALLCAT;    /* output message format, parm to c_inittm() */ 
   extern long curent;      /* current entity, for processing message */ 

/* initialize and start processing line message */ 
   curent = 1; 
   c_inittm ("MSSCOLOR", "Processed", "line", "bands", "", &outimage->nl,
            &outimage->totalbands, &outimage->msgtim, &messform); 
   c_startm (); 

/* process image */ 
   for (line=0;  line<outimage->nl;  line++, curent++)
      {

      /* read a line of data */ 
	 read_bil (line, inimage); 

      /* get inpix and outpix */ 
	 for (band=0;  band<inimage->totalbands;  band++)
	    inpix[band] = (float *) inimage->linebuf[band]; 
	 for (band=0;  band<outimage->totalbands;  band++)
	    outpix[band] = (float *) outimage->linebuf[band]; 

      /* calculate pixel values for a line of data */ 
	 real_outvals (outimage->dtypeout!=EREAL, inimage->ns, inpix, outpix); 

      /* update minmax */ 
	 minmax_bil (outimage); 

      /* write a line of data */ 
	 write_bil (line, outimage); 
      }
c_stoptm(); 
}








/*******************************************************************************
NAME:     real_outvals
PURPOSE:  calculate output pixel values for a line of input data
*******************************************************************************/

FUNCTION void real_outvals (roundfix, ns, inpix, outpix)
   long roundfix;           /* input:  flag, correct for round-off error?  does
                                       output image have an integer data type?*/
   long ns;                 /* input:  number of image samples */ 
   float *inpix[INBANDS];   /* input:  a row of input pixels */ 
   float *outpix[OUTBANDS]; /* output:  a row of output pixels */ 

{
/* declare variables */ 
   float chan4, chan5, chan6, chan7,  /* input data for satellite channels 
					 4,5,6,7;  corresponds to image bands
					 1,2,3,4 after subbanding */ 
	 ratio,                       /* ratio:  chan5 / chan6 */ 
	 weight,                      /* a weight between two types of
	                                 calculations */ 
         red, green, blue;            /* output data for image bands 1,2,3 */
   long  band,                        /* loop index, image band */ 
         sample;                      /* loop index, image sample */ 

/* loop for all samples */ 
   for (sample=0;  sample<ns;  sample++)
   {

   /* set variables for input satellite channels */ 
      chan4 = *inpix[0]; 
      chan5 = *inpix[1];
      chan6 = *inpix[2];
      chan7 = *inpix[3]; 

   /* determine ratio between channels 5 and 6 */ 
      if (chan6 == 0.0)
	 ratio = chan5; 
      else 
	 ratio = chan5 / chan6;  

   /* compute output pixel values based on ratio 
   ********************************************/

      if (ratio < 0.56)   
	 /* pixel is all vegetation */ 
	 {
	 red   = 0.75  * chan5; 
	 green = 1.50  * chan4; 
	 blue  = 1.125 * chan4 - 0.1875 * chan6; 
	 }

      else if (ratio < 0.65) 
	 /* pixel is a weighted mixture of vegetation and soil */ 
	 {
	 weight = (ratio - 0.56) / (0.65 - 0.56);
	 red    = weight * chan5 + 
		  (1.0 - weight) * 0.75 * chan5;
	 green  = weight * 2.0 * chan4 + 
		  (1.0 - weight) * chan4; 
	 blue   = weight * 1.5 * chan4 - 0.25 * chan6 + 
	 	  (1.0 - weight) * 2.0 * chan4 - 
		  0.35 * chan5 - chan7;

	 red   *= 0.75;
	 green *= 0.75;
	 blue  *= 0.75; 
	 }

      else if (ratio < 1.50)
	 /* pixel is soil */ 
	 {
	 red   = 0.5625 * chan5;
	 green = 0.75   * chan4;
	 blue  = 0.75   * (2.0 * chan4 - 0.35 * chan5 - chan7); 
	 }

      else 
	 /* pixel is water */ 
	 {
	 red   = 0.75 * chan5;
	 green = 0.75 * chan4;
	 blue  = 0.75 * (2.0 * chan4 - chan5); 
	 }

   /* optionally correct for round-off error */ 
      if (roundfix) 
	 {
	 if (red  >=0)  red  +=0.5;  else  red  -=0.5;
	 if (green>=0)  green+=0.5;  else  green-=0.5;
	 if (blue >=0)  blue +=0.5;  else  blue -=0.5;
	 }

   /* copy rgb data to output buffer */ 
      *outpix[0] = red; 
      *outpix[1] = green;
      *outpix[2] = blue; 

   /* increment pointers to input and output pixels */ 
      for (band=0;  band<INBANDS;  band++)
	 inpix[band] ++;
      for (band=0;  band<OUTBANDS;  band++)
	 outpix[band] ++;

   }
} 

