/****************************************************************************

NAME:	  MTP2TPL

PURPOSE:  Convert a mtp file to a tpl file

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   		CHANGE
-------	 ----	------	  		------ 
1.0	 3/97	D. Etrheim		Re-created as source code was lost
					by original developer.

*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int main
(void)		
{

struct PARBLK vblk;		/* TAE parameter block */
struct VARIABLE *var;		/* TAE variable pointer */
struct MTPHEAD mtphead;		/* MTP header data */
struct MTPDATA mtpdata;		/* MTP data record */
struct TPLHEAD tplhead;		/* MTP header data */
struct TPLDATA tpldata;		/* MTP data record */
long msgtim;			/* Message timer */
long std_out;			/* Standard output LUN */
long status;			/* Function return status */
long fatal = -1;		/* Fatal flag for errmsg */
long zero = 0;			/* Constant of 0 */
long one = 1;			/* Constant of 1 */
long two = 2;			/* Constant of 2 */
long i;				/* Loop counter */
long ptyp;			/* Type of file used (for Pt I/O) */
long acc=IREAD;			/* Access mode of tie point files */
struct PTDESC * inmtp;			/* File descriptor */
struct PTDESC * outtpl;			/* File descriptor */
char in_mtp[CMLEN];		/* The input merged Tie Point file name */
char out_tpl[1][CMLEN];		/* The output Tie Point Location host name */

extern long curent;

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the input merged tie point file name
  -------------------------------------------*/
if(c_getnam(&vblk, "INMT", &zero, in_mtp, "mtp", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","mtp2tpl-fatal",&fatal);

/* Get the output tie point location file name
  -------------------------------------------*/
acc=IWRITE;
if(c_getnam(&vblk, "OUTTL", &zero, out_tpl[0], "tpl", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","mtp2tpl-fatal",&fatal);

/* Get the message timer
  ---------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);
c_inittm("MTP2TPL","Processed","point number","","",&two,&two, &msgtim, &two);

/* Open input merged Tie Point file
  ---------------------------------*/
ptyp=MTP;
acc=IREAD;
if(c_ptopen(&inmtp, in_mtp, &ptyp, &acc, &mtphead) != E_SUCC) 
	c_errmsg("Fatal error encountered","mtp2tpl-fatal",&fatal);

/* Update the tpl header with the mtp header info  
  ----------------------------------------------*/
clrtpl(&tplhead);
for(i = 0;i<8;i++)
   tplhead.corners[i] = mtphead.corners[i];
tplhead.pdist[0] = mtphead.pdist[0];
tplhead.pdist[1] = mtphead.pdist[1];
for(i = 0; i<15; i++)
   tplhead.projprms[i] = mtphead.projprms[i];
tplhead.proj_valid = mtphead.proj_valid;
tplhead.code = mtphead.code;
tplhead.zone = mtphead.zone;
tplhead.datum = mtphead.datum;
strcpy(tplhead.ref_mode,mtphead.chip_mode);
strcpy(tplhead.units,mtphead.units);
strcpy(tplhead.sea_name,mtphead.sea_name);
strcpy(tplhead.ref_name,mtphead.ref_name);

/* Open the output Tie Point location file
  ---------------------------------------*/
ptyp=TPL;
acc=IWRITE;
if(c_ptopen(&outtpl, out_tpl[0], &ptyp, &acc, &tplhead) != E_SUCC) 
	c_errmsg("Fatal error encountered","mtp2tpl-fatal",&fatal);

/* Initialize some tpl data feilds that do not have 
   corresponding entries in the mtp file
   ------------------------------------------------*/
tpldata.rms_error[0] = tpldata.rms_error[0] = 0.0;
tpldata.strength = tpldata.displacement = 0.0;
tpldata.req_size[0] = tpldata.req_size[1] = 0;
tpldata.sea_size[0] = tpldata.sea_size[1] = 0;
tpldata.req_size[0] = tpldata.req_size[1] = 0;
tpldata.chip_coord[0] = tpldata.chip_coord[1] = 0;
tpldata.location[0] = tpldata.location[1] = 0;
tpldata.chip_loc[0] = tpldata.chip_loc[1] = 0;
tpldata.active = 0;

/* Start the processing message 
   ----------------------------*/
curent = 1;
c_startm();

/* Read file the input mtp file.  Update the corresponding
   fields in the tpl structure.
  --------------------------------------------------------*/
curent = 0;
for (;;)
  {
  /* Read one record until EOF is encountered
  -------------------------------------------*/
  status = c_ptio(&inmtp, &mtpdata);
  if (status == E_EOF) break;
  if (status == E_FAIL)
    c_clean(&vblk,"MTP2TPL",out_tpl,&one);
  
  /* Update the corresponding data fields
     in the tpl file. 
  --------------------------------------*/
  tpldata.ref_coord[0] = tpldata.sea_coord[0] = mtpdata.ref_coord[0];
  tpldata.ref_coord[1] = tpldata.sea_coord[1] = mtpdata.ref_coord[1];
  tpldata.elevation = mtpdata.elevation;
  tpldata.zoom_factor = mtpdata.zoom_factor;
  strcpy(tpldata.pt_id,mtpdata.pt_id);
  strcpy(tpldata.ref_chip,mtpdata.ref_chip);
  tpldata.ref_size[0] = mtpdata.chip_size[0];
  tpldata.ref_size[1] = mtpdata.chip_size[0];
  tpldata.geo_coord[0] = mtpdata.geo_coord[0];
  tpldata.geo_coord[1] = mtpdata.geo_coord[1];
  tpldata.nominal[0] = mtpdata.nominal[0];
  tpldata.nominal[1] = mtpdata.nominal[0];
  tpldata.offsets[0] = mtpdata.offsets[0];
  tpldata.offsets[1] = mtpdata.offsets[1];


  /* Write the tpl data record
  -----------------------------*/
  status = c_ptio(&outtpl, &tpldata);
  if (status == E_FAIL)
    c_clean(&vblk,"MTP2TPL",out_tpl,&one);
  curent++;
  }

/* Stop the processing message */
c_stoptm();


/* Close the files 
   ---------------*/
c_ptclse(&inmtp);
c_ptclse(&outtpl);

c_complt("MTP2TPL", out_tpl, &one);

return EXIT_SUCCESS;

}
