/****************************************************************************
NAME:	  MTPGRID

PURPOSE:  Creates a user specified grid of tie points for the correlation
          process

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   		CHANGE
-------	 ----	------	  		------ 
  1.0	 7/92	D. Steinwand		Development
	 5/95   C. Engebretson     	Update source, PDF, and Makefile
					to baseline standards
	 3/96   David Zokaites,         Added parm PTID, changed name to 
		Gayla Evans             mtpgrid2 

*****************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "geompak.h"

main()		
{
struct PARBLK vblk;		/* TAE parameter block */
struct VARIABLE *p_find();	/* TAE parameter function */
struct VARIABLE *var;		/* TAE variable pointer */
struct MTPHEAD mtphead;		/* MTP header data */
struct MTPDATA mtpdata;		/* MTP data record */
long msgtim;			/* Message timer */
long std_out;			/* Standard output LUN */
long status;			/* Function return status */
long fatal = -1;		/* Fatal flag for errmsg */
long zero = 0;			/* Constant of 0 */
long one = 1;			/* Constant of 1 */
long two = 2;			/* Constant of 2 */
long i,j,k;			/* Loop counters */
long length;			/* Length of PTID parameter */
long ptyp;			/* Type of file used (for Pt I/O) */
long acc=IWRITE;		/* Access mode of TPL file */
long nl,ns;			/* Number of lines and samples in grid */
long xinc,yinc;			/* Grid increment in the x & y dimensions */
long chip_size[2];		/* Reference chip size */
long offset[2];			/* Search offsets */
int *outmtp;			/* File descriptor */
char out_mtp[CMLEN];		/* The output merged Tie Point file name */
char *ptid;                     /* Tie point id */ 
char *id;			/* Modified tie point id */
char *squeeze();		/* Function to remove whitespace */
extern long curent;
extern long curcat;

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the output tie point location file name
  -------------------------------------------*/
if(c_getnam(&vblk, "OUTMT", &zero, out_mtp, "mtp", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","mtpgrid-fatal",&fatal);

/* Get grid parameters
  -------------------*/
var = p_find(&vblk, "NL");
nl = IVAL(*var,0);
var = p_find(&vblk, "NS");
ns = IVAL(*var,0);
var = p_find(&vblk, "SAMPINC");
xinc = IVAL(*var,0);
var = p_find(&vblk, "LINEINC");
yinc = IVAL(*var,0);
var = p_find(&vblk, "REFSIZ");
chip_size[0] = IVAL(*var,0);
chip_size[1] = IVAL(*var,1);
var = p_find(&vblk, "SRCHOFF");
offset[0] = IVAL(*var,0);
offset[1] = IVAL(*var,1);
var = p_find (&vblk, "PTID");
ptid = SVAL (*var,0);

/* Check PTID for legality
  ------------------------*/
if (!strlen(ptid))
    c_lasmsg(TRUE, "mtpgrid-invalid", "Invalid entry for PTID");

length = strlen(ptid);
id = (char *) malloc (strlen(ptid) + 1);
strcpy(id, ptid);
ptid = id;
while (isspace(*ptid)) ptid++;
c_low2up(ptid, &length);
for (i = 0; i < strlen(ptid); ++i) 
    if (!isalpha(ptid[i])) ptid[i] = '\0';

/* Get the message timer
  ---------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);
c_inittm("MTPGRID","Processed","point number","","",&two,&two, &msgtim, &two);

/* Open output Merged Tie Point file
  ---------------------------------*/
for(i = 0; i < 8; i++)
    mtphead.corners[i] = 0.0;
mtphead.pdist[0] = 0.0;
mtphead.pdist[1] = 0.0;
for(i = 0; i < 15; i++)
    mtphead.projprms[i] = 0.0;
mtphead.proj_valid = GEO_NOTVALID;
mtphead.code = 0;
mtphead.zone = 0;
mtphead.datum = 0;
strcpy(mtphead.chip_mode,"NO-CHIPS");
strcpy(mtphead.units,"");
strcpy(mtphead.sea_name,"");
strcpy(mtphead.ref_name,"");
acc = IWRITE; ptyp=MTP;
if(c_ptopen(&outmtp, out_mtp, &ptyp, &acc, &mtphead) != E_SUCC) 
	c_errmsg("Fatal error encountered","mtpgrid-fatal",&fatal);

/* Create grid
  -----------*/
curent = 1; c_startm(); curent = 0;
for (k = 1, i = yinc; i < nl; i += yinc)
    for (j = xinc; j < ns; k++, j += xinc)
	{
	curent++;
	mtpdata.geo_coord[0]=mtpdata.geo_coord[1]=0.0;
	mtpdata.nominal[0]=i;
	mtpdata.nominal[1]=j;
	mtpdata.ref_coord[0]=i;
	mtpdata.ref_coord[1]=j;
	mtpdata.elevation=0.0;
	mtpdata.zoom_factor=0;
	mtpdata.chip_size[0]=chip_size[0];
	mtpdata.chip_size[1]=chip_size[1];
	mtpdata.offsets[0]=offset[0];
	mtpdata.offsets[1]=offset[1];
	sprintf(mtpdata.pt_id,"%s_%d", ptid, k);
	strcpy(mtpdata.ref_chip,"");
	strcpy(mtpdata.sea_chip,"");
	if (c_ptio(&outmtp, &mtpdata) != E_SUCC) 
	    c_errmsg("Fatal error encountered","mtpgrid-fatal",&fatal);
	}
c_stoptm();
free(id);
c_complt("MTPGRID", out_mtp, &one);
}
