/****************************************************************************

NAME:	  PROCESS

PURPOSE:  Process the two mtp files.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   		CHANGE
-------	 ----	------	  		------ 
1.0	 3/97	D. Etrheim		Re-created as source code was lost
					by original developer.

*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int process
(
    long *msgtim,			/* Message timer */
    char in_mtp[][CMLEN],		/* The input merged Tie Point file names */
    char *out_mtp,			/* The output Tie Point Location host name */
    long *size,
    long *offset
)

{
struct MTPHEAD mtphead1;	/* MTP header data */
struct MTPHEAD mtphead2;	/* MTP header data */
struct MTPDATA mtpdata;		/* MTP data record */
long status;			/* Function return status */
long two = 2;			/* Constant of 2 */
long ptyp = MTP;		/* Type of file used (for Pt I/O) */
long acc=IREAD;			/* Access mode of tie point files */
struct PTDESC * inmtp1;			/* File descriptor */
struct PTDESC * inmtp2;			/* File descriptor */
struct PTDESC * outmtp;			/* File descriptor */
char tmpstr[21];

extern long curent;

/* Open input merged Tie Point file
  ------------------------------------------*/
if(c_ptopen(&inmtp1, in_mtp[0], &ptyp, &acc, &mtphead1) != E_SUCC) 
   return(E_FAIL);
if(c_ptopen(&inmtp2, in_mtp[1], &ptyp, &acc, &mtphead2) != E_SUCC) 
   return(E_FAIL);

/* Open the output file. Use the header from
   the first file.
  ---------------------------------------*/
acc=IWRITE;
if(c_ptopen(&outmtp, out_mtp, &ptyp, &acc, &mtphead1) != E_SUCC) 
   return(E_FAIL);

/* Start the processing message 
   ----------------------------*/
c_inittm("MTPZIP","Processed","point number","","",&two,&two, msgtim, &two);
curent = 1;
c_startm();

/* Read file the input mtp file.  
  --------------------------------------------------------*/
curent = 0;
for (;;)
  {
  /* Read one record until EOF is encountered
  -------------------------------------------*/
  status = c_ptio(&inmtp1, &mtpdata);
  if (status == E_EOF) break;
  if (status == E_FAIL)
     return(E_FAIL);
  
  /* Write the mtp data record
  -----------------------------*/
  sprintf(tmpstr,"a%s",mtpdata.pt_id);
  strcpy(mtpdata.pt_id,tmpstr); 
  mtpdata.chip_size[0] = size[0];
  mtpdata.chip_size[1] = size[1];
  mtpdata.offsets[0] = offset[0];
  mtpdata.offsets[1] = offset[1];
  status = c_ptio(&outmtp, &mtpdata);
  if (status == E_FAIL)
     return(E_FAIL);
  curent++;
  }

for (;;)
  {
  /* Read one record until EOF is encountered
  -------------------------------------------*/
  status = c_ptio(&inmtp2, &mtpdata);
  if (status == E_EOF) break;
  if (status == E_FAIL)
     return(E_FAIL);
  
  /* Write the mtp data record
  -----------------------------*/
  sprintf(tmpstr,"b%s",mtpdata.pt_id);
  strcpy(mtpdata.pt_id,tmpstr); 
  mtpdata.chip_size[0] = size[0];
  mtpdata.chip_size[1] = size[1];
  mtpdata.offsets[0] = offset[0];
  mtpdata.offsets[1] = offset[1];
  status = c_ptio(&outmtp, &mtpdata);
  if (status == E_FAIL)
     return(E_FAIL);
  curent++;
  }

/* Stop the processing message */
c_stoptm();

/* Close the files 
   ---------------*/
c_ptclse(&inmtp1);
c_ptclse(&inmtp2);
c_ptclse(&outmtp);

return(E_SUCC);
}
