#include "multiply.h"
#include "util.h"
#include "pixman.h"

void compute
(
    char hos_out[],
    long num_images,
    long *num_bands,
    long *nlout,
    long *nsout,
    long complex
)
/*
***************************************************************************

NAME: COMPUTE

PURPOSE: Performs pixel-by-pixel real or complex multiplication of images

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   2          15-APR-88  K. Narayanan, SAR
   1           9-Jan-84  J. Bay, SAR

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
     Initialize processing message display
     Start processing message display
     For each output image line
        Read next line of input data and write previous line out
        If real data 
           Call c_pxmul to multiply the data
           Update minimum and maximum values if requested 
        Else (complex data)
           For each complex pixel perform complex multiplication
                 Real output = r1*r2 - i1*i2
                 Imaginary output = r1*i2 + r2*i1
     Stop processing message display
     Update minimum and maximum of output (real) image label if needed

***************************************************************************
*/
{
/*
static long	llm = -2147483648, 
	        lum =  2147483647;
*/
double	out_max[MAXBND], out_min[MAXBND];

float	*ip0, *ip1,
	*op,
	scale,
	offset;

long	band, 
	image,
	ipt0, ipt1, 
	line,
	msg_flag,
	opt,
	out_dtype, 
	samp,
	size;



out_dtype = EREAL;

/*
initialize min max arrays for all bands
---------------------------------------
*/
if (min_max)
   c_inmnmx (&out_dtype, num_bands, out_min, out_max);

/*
initialize and start processing message
---------------------------------------
*/
msg_flag = ALLCAT;
c_inittm ("MULTIPLY", "Processed", "line", "bands", " ", nlout, num_bands, 
          &msgtim, &msg_flag);
curent = 1;
c_startm();


/*
process line by line for all bands
----------------------------------
*/
size = sizeof (float);
scale = 1.0;
offset = 0.0;
for (line = 0; line < *nlout; line++, curent++)
   {
   c_estep (&gdesc);
   for (band = 0; band < *num_bands; band++)
      {
      opt = opoint[band] * size;
      for (image = 1; image < num_images; image++)
         {
         if (image == 1) ipt0 = ipoint[0][band] * size;
         else ipt0 = opt;

         ipt1 = ipoint[image][band] * size;

/*
         if not complex perform ordinary multiplication
         ----------------------------------------------
*/
         if (!complex)
            c_pxmul (nsout, &iobuf[ipt0], &iobuf[ipt1], 
                     &iobuf[opt], &scale, &offset, &out_dtype);
         else
/*
            else perform complex multiplication
            -----------------------------------
*/
            for (samp = 0; samp < *nsout; samp += 2)
               {
               ip0 = (float *) &iobuf[ipt0+samp*size]; 
               ip1 = (float *) &iobuf[ipt1+samp*size];
               op = (float *) &iobuf[opt+samp*size]; 

               *op++ = (*ip0) * (*ip1) - (*(ip0+1)) * (*(ip1+1));
               *op = (*ip0) * (*(ip1+1)) + (*(ip0+1)) * (*ip1);
               }  
         }  
/*
      update min and max arrays if needed
      -----------------------------------
*/
      if (min_max)
         c_pxmnmx (&iobuf[opt], nsout, &out_min[band], &out_max[band], &out_dtype);
      }
   }  

c_estep (&gdesc);
c_stoptm();

c_egclse (&gdesc);
/*
update output image ddr min max fields
--------------------------------------
*/
if (min_max) c_svmnmx (hos_out, num_bands, out_min, out_max, "YES");

/*
determine output data type
--------------------------
*/
/*
if (!complex)
   {
   settyp (pixmin, pixmax, itype);
   if ((itype  ==  4)  && ((pixmin  <  llm)  || 
        pixmax  >  lum)) dreal = TRUE;
   } 
if ((dreal)  ||  (complex_flag  ==  "COMP")) 
   {
   odtype = 4;
   }
else if (itype  ==  4) 
   {
   odtype = 3;
   }
else
   {
   odtype = itype;
   } 

if (odtype  ==  4) 
   {
   otype = "REAL*4";
   odtype = 0;
   }
else if (odtype  ==  3) 
   {
   otype = "INTEGER*4";
   }
else if (odtype  ==  2) 
   {
   otype = "INTEGER*2";
   }
else
   {
   otype = "BYTE";
   } 
write(msgtxt, 300)otype;
300 format("Output image will be ", a9);      
cerrmsg (msgtxt, ' ', 1); 
*/
/*
send output data type to procedure for conversion

*/
/*
xqintg (block, "ODTYPE", 1, odtype, xupdat, status);
xqout (block, status);

getddr (taeout, hosout, fixed_label); 
min_pixel_value = pixmin;
max_value = pixmax;
pixel_value_validity = 3;
putddr (taeout, hosout, fixed_label); 

minmax (taeout, hosout, num_bands, 1, omin, omax, mnmx);
*/

}
