#include "multiply.h"
#include "util.h"

void filspec
(
    char hosin[][CMLEN],      /* input image host name descriptors  */
    char hosout[][CMLEN],            /* output image host name descriptor  */
    long num_images,          /* number of input images             */
    long *num_bands,          /* number of bands in each input image*/
    long num_inimages[10],    /* number of images in each IN parameter*/
    long num_image_bands[],
    long image_band_num[][MAXBND + 1],
    long wind[][4],           /* input image windows                */
    long *nlout,
    long *nsout,
    long *odtype	     /* output data type		   */
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up input and output file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          15-APR-88  K. Narayanan, SAR    
   2          20-DEC-91  T. Mittan            Added ODTYPE.
   6.1	      23-May-94  D. Eichacker	      Added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call C_EOPENS to open input and output images
   Call C_EGROUP to group the images

**************************************************************************
*/
{
double	line_inc = 1.0,
	samp_inc = 1.0;

static	long	opt[3] = {0,0,0};    /* option array for c_eopens          */
long	access,                      /* file access type                   */
	band,
	bufsiz,                      /* image i/o buffer size              */
	comb_flag = COMB,
	img,
	i, j,
	stat,                        /* return status                      */
	type;                        /* file type                          */

char *prog_name = "multiply";  /* program name            */
long    fatal_err = -1;
long    zero = 0;
long    one = 1;


/*
open input and output image files
---------------------------------
*/
access = IREAD;
type = EREAL;
opt[0] = 0;
for (i = 0, img = 0; i < num_images; i++)
   {
   for (j = 0, band = 0; j < num_inimages[i]; j++, 
                                             band += num_image_bands[img++])
      {
      stat = c_eopens (&fdesc[img], hosin[img], image_band_num[img], 
                       &wind[img][SL], &wind[img][SS], nlout, nsout, 
                       &ipoint[i][band], &access, &type, &zero, opt);
      if (stat != E_SUCC )
         c_errmsg ("Error opening input image", "multiply-inopen", &fatal_err);
      
      }
   }  

access = IWRITE;
opt[0] = *odtype;
stat = c_eopens(&fdesc[img], hosout[0], num_bands, &one, &one, nlout, nsout, 
                opoint, &access, &type, &zero, opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "multiply-outopen", &fatal_err);

fdesc[img+1] = 0;

/*
update output image DDR based on input images
---------------------------------------------
*/
c_upddr (&block, hosin, hosout[0], &img, wind, image_band_num, num_image_bands,
         &line_inc, &samp_inc, num_bands, &comb_flag);

/*
set up group containing input and output bands
----------------------------------------------
*/
if ((iobuf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "multiply-alloc", 
            prog_name, hosout, &one);

c_egroup (fdesc, &gdesc, iobuf, &bufsiz);
}
