#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "protos.h"

struct PARBLK block;                   /* TAE parameter block     */

FILE     *fd;                           /* output file descriptor  */

struct FDESC *     fdesc[MAXIMG];                 /* file descriptors        */
struct GDESC *     gdesc;                         /* group descriptor        */
long     ipoint[10][MAXBND + 1],        /* offsets into image i/o  */
                opoint[MAXBND + 1];     /* for input and output bands */
long     min_max;                       /* minmax option flag      */
long     msgtim;                        /* message interval        */

unsigned char    *iobuf;                /* image i/o buffer        */

extern long     curcat;                        /* message utility         */
extern long     curent;                        /* parameters for processing */

/*
***************************************************************************

NAME: MULTIPLY

PURPOSE: Performs pixel-by-pixel real or complex multiplication of images

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           9-Jan-84  J. Bay, SAR
   2           5-Dec-88  K. Narayanan, SAR    Conversion to C
   3          21-Mar-89  K. Narayanan, STX    Changed parameter name
   4	      20-DEC-91  T. Mittan	      Added parameter ODTYPE
   5	      23-SEP-92  T. Mittan	      Modified include file to 
					      eliminate warnings on link 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

     Call GETPAR to get and validate user-supplied parameter values 
     Call FILSPEC to set up input and output file specifications
     Call COMPUTE to perform real or complex multiplication and produce
        output image
     Call TERMINATE to update output image history and issue
        successful completion message

**********************************************************************
*/

int main
(void) 
{
long	complex;                 /* real or complex image indicator  */
static	long image_band_num[MAXIMG][MAXBND + 1], 
                                 /* band numbers in each input image */
	nlout,                   /* output image number of lines and */
	nsout,                   /* number of samples per line       */
	num_bands,               /* total number of input image bands*/
	num_image_bands[MAXIMG], 
                                 /* number of bands in each image in INx  */
	num_images,              /* number of input images           */
	num_inimages[10],        /* number of images in each IN      */
	odtype, 		 /* output data type		     */
	wind[MAXIMG][4];         /* input image windows              */

static	char	hos_in[MAXIMG][CMLEN];
                                 /* input images host names          */
char	hos_out[1][CMLEN],          /* output image host name           */
	las_out[1][CMLEN];          /* output image las name            */


/*
get parameters
--------------
*/
getpar (hos_in, las_out, hos_out, &num_images, &num_bands, num_inimages,
        num_image_bands, image_band_num, wind, &nlout, &nsout, &complex,&odtype);

/*
set up input and output file specifications
-------------------------------------------
*/
filspec (hos_in, hos_out, num_images, &num_bands, num_inimages, 
         num_image_bands, image_band_num, wind, &nlout, &nsout,&odtype);

/*
perform multiplication
----------------------
*/
compute (hos_out[0], num_images, &num_bands, &nlout, &nsout, complex);

/*
update output image history and issue completion message
--------------------------------------------------------
*/
terminate (las_out, hos_out[0], hos_in, num_images, num_inimages); 

return EXIT_SUCCESS;

}
