/*--------------------------------------------------------------------------
ROUTINE: get_file_size

PURPOSE:
        Get the size of the file passed in.

PROGRAM HISTORY
VERSION     PROGRAMMER      DATE        REASON
-------     ----------      ----        ------
1.0         T. Beckmann     Jul 1999    Original development

INPUTS:
        filename - host name of file to get the size of
        
OUTPUTS:
        size - the size of the file in bytes
        returns - E_SUCC when a valid datum is found
                  E_FAIL when no valid datum is found
                  
ALGORITHM:
        Get the size of the file and return it
        
NOTES:
none

--------------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "get_file_size.h"

int get_file_size
(
    const char *file,
    long *size
)
{
    struct stat info;

    if (stat(file,&info) < 0)
    {
        c_lasmsg(FALSE, "filesize-error", 
                 "Error encountered reading size of %s", file);
        return (E_FAIL);
    }

    *size = (long)info.st_size;

    return (E_SUCC);
}
