#ifndef READ_NDF_PARAMETERS_H
#define READ_NDF_PARAMETERS_H

typedef enum
{
    INT_PARAM,
    LONG_PARAM,
    FLOAT_PARAM,
    DOUBLE_PARAM,
    STRING_PARAM
} NDF_PARAMETER_TYPE;

#define NDF_ISSUE_WARNING 1
#define NDF_NO_WARNING 0
#define NDF_UNUSED 0

typedef struct 
{
    char name[40];  /* parameter name */
    NDF_PARAMETER_TYPE param_type; /* parameter type */
    void *dest;     /* pointer to destination for data read */
    int dest_size;  /* number of bytes per item read */
    int num_items;  /* number of items expected */
    int *valid_flag_ptr; /* pointer to the valid flag for this item.  If NULL,
                            the parameter is required and not finding it is
                            considered a fatal error */
    int issue_warning; /* flag to issue a warning if the valid_flag_ptr is not
                          NULL and the item is not found.  This allows a 
                          warning message to be automatically generated or
                          surpressed if it is a normal condition. */
} NDF_PARAMETER_DEF_TYPE;

int read_ndf_parameters
(
    const char *filename,     /* I: file to read parameters from */
    NDF_PARAMETER_DEF_TYPE *param_list, 
                              /* I/O: pointer to parameter definition list */
    int num_parameters        /* I: number of parameters in list */
);

#endif

