/****************************************************************************
NAME: GETFILES

PURPOSE: Get the names and the size of the files to write to tape.

PROGRAM HISTORY:

Version   Date     Programmer      Code  Reason
-------   ----     --------------  ----- ------
  1.0     03/98    T.Ratliff        C    Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE

****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

void getfiles
(
    char *hostout,                      /* NDF files in UNIX format    */
    char fnames[][CMLEN],               /* file names to write to tape */
    long filesize[300],                 /* size of target file(s)      */
    double *totsize,                    /* size of all order files     */
    long nb,                            /* number of bands             */
    long *nfiles,                       /* number of files found       */
    char *bform                        /* data format of the files    */
)

{
  long one = 1;                     /* constant value 1            */
  long maxfiles = 300;              /* # of files to return at 1 time */
  long frstcall = 1;                /* first call to c_getfiles    */
  long zero = 0;                    /* constant value 0            */
  long i = 0;                       /* loop counter                */
  long fatal = -1;                  /* fatal error flag            */
  long status = 0;                  /* return status of functions  */
  long tmpsize = 0;                 /* tmp file size holder        */

  char tmphost[CMLEN];              /* tmp  host file name         */
  char path[CMLEN];                 /* path of hostout             */
  char root[CMLEN];                 /* root of hostout             */
  char exten[CMLEN];                /* extension of hostout        */
  char trash[CMLEN];                /* holder for unneeded data    */
  char callstr[CMLEN];              /* command to create a file    */
  char remvstr[CMLEN];              /* command to remove a file    */

  FILE *inf;                        /* file pointer                */


  /* Get the files to write to tape
  ---------------------------------*/
  sprintf(tmphost,"%s.hd",hostout);
  status = c_getfiles(tmphost,fnames[0],&one,maxfiles,frstcall,zero);
  if (status == E_FAIL)
    c_errmsg("Fatal error encountered","ndfout-fatal",&fatal);
  if (strcmp(bform,"BSQ") == 0)
    {
    for (i = 1; i <= nb; i++)
      {
      sprintf(tmphost,"%s.i%d",hostout,i);
      status = c_getfiles(tmphost,fnames[i],&one,maxfiles,frstcall,zero);
      if (status == E_FAIL)
        c_errmsg("Fatal error encountered","ndfout-fatal",&fatal);
      }
    *nfiles = nb + 1;
    }
  else
    {
    sprintf(tmphost,"%s.im",hostout);
    status = c_getfiles(tmphost,fnames[1],&one,maxfiles,frstcall,zero);
    if (status == E_FAIL)
      c_errmsg("Fatal error encountered","ndfout-fatal",&fatal);
    *nfiles = 2;
    }

  /* Find the total size of all files
  -----------------------------------*/
  c_splitname(hostout,path,root,exten);
  sprintf(tmphost,"%st_e_m_p_file_one",path);
  c_creat_name(tmphost,"tmp");
  sprintf(remvstr,"/bin/rm %s",tmphost);
  for (i = 0; i < *nfiles; i++)
    {
    sprintf(callstr,"/bin/ls -l %s > %s",fnames[i],tmphost);
    system(callstr);
    if ((inf = fopen(tmphost,"r")) != NULL)
      {
      fscanf(inf,"%s %s %s %s %d",trash,trash,trash,trash,&tmpsize);
      fclose(inf);
      *totsize += tmpsize;
      filesize[i] = tmpsize / 1000;
      }
    system(remvstr);
    }
  *totsize /= 1000; 
}
