/*****************************************************************************
NAME: GETNBDNS

PURPOSE:  Gets the number of bands and the number of samples in the image.

PROGRAM HISTORY:

Version  Date  Programmer       Code  Reason
-------  ----- ----------       ----  ------
 1.0     03/98 T.Ratliff         C    Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:        LAS
*****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "protos.h"

void getnbdns
(
    char *hostout,                          /* output header file name*/
    long *ns,                               /* number of samples      */
    long *bytes_per_sample,                 /* number of bytes per sample */
    long *nb                               /* number of bands        */
)
{
  long fatal = -1;                      /* fatal error flag       */

  char tmpstr[350];                     /* temp string holder     */ 
  char tmphout[CMLEN];                  /* temp string holder     */ 
  char *p;                              /* pointer to tmpstr      */
  FILE *inf;                            /* input file pointer     */

  /* Open the NDF header file
  ---------------------------*/
  sprintf(tmphout,"%s.hd",hostout);
  if ((inf=fopen(tmphout,"r")) == NULL)
    {
    fclose(inf);
    c_errmsg("Error opening the input header file","ndfout-open",&fatal);
    }

  /* Read the file and look for the 
     pixels per line, and the number of bands
  -------------------------------------------*/
  while (fscanf(inf,"%s",tmpstr) != EOF)
    {
    if (strncmp(tmpstr,"PIXELS_PER_LINE",15) == 0)
      {
      p = (char *)strchr(tmpstr,'=');
      p++;
      *ns = atoi(p);
      }
    else if (strncmp(tmpstr,"NUMBER_OF_BANDS_IN_VOLUME",25) == 0)
      {
      p = (char *)strchr(tmpstr,'=');
      p++;
      *nb = atoi(p);
      }
    else if (strncmp(tmpstr,"PIXEL_FORMAT",12) == 0)
      {
      p = (char *)strchr(tmpstr,'=');
      p++;
      if (strncmp(p,"2BYTEINT",8) == 0)
        *bytes_per_sample = 2;
      else if (strncmp(p,"4BYTEINT",8) == 0)
        *bytes_per_sample = 4;
      else if (strncmp(p,"REAL",4) == 0)
        *bytes_per_sample = 4;
      else
        *bytes_per_sample = 1;
      }
    }
  fclose(inf);
}
