/****************************************************************************
NAME: MKNHDR
 
PURPOSE: To create additional NDF header files, as needed for
         volume spanning of output products.
 
PROGRAM HISTORY:

Version   Date     Programmer      Code  Reason
-------   ----     --------------  ----- ------
  1.0     03/98    T.Ratliff        C    Initial development
 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE
 
NOTE:  This function is based off the mknhdr.c function written by
       Kevin Lowell.
****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include "string.h"
#include "las.h"
#include "protos.h"

void mknhdr
(
    char *hostout,          /* name of the NDF files        */
    long numnhdr,           /* Number of total volumes      */
    long bpv[]             /* bands per volume             */
)
{
  FILE *inf;            /* File pointer                 */
  FILE *outf;           /* File pointer                 */

  long bgnfield = 0;       /* begin of field to scan       */
  long bnd = 0;            /* band number of copy          */
  long endfield = 0;       /* end of field to scan         */
  long fatal = -1;         /* Fatal error flag             */
  long field_length = 0;   /* length of field to copy      */
  long i = 0;              /* loop counter                 */
  long j = 0;              /* loop counter                 */
  long lpv = 0;            /* Lines per volume             */
  long lpf = 0;            /* Lines per file               */
  long tmpbnd = 0;         /* temp number of bands         */
  long totlin = 0;         /* Total number of lines        */
  long totbnd = 0;         /* Total number of bands        */
  long sdf[100];           /* Start data file              */
  long slnum = 1;          /* Start line number            */

  char *p;                 /* pointer to a string          */
  char infile[CMLEN];      /* Input .HD file               */
  char outfile[CMLEN];     /* Output .HX file              */
  char inbuff[CMLEN];      /* input buffer                 */
  char outbuff[CMLEN];     /* output buffer                */
  char band[5];            /* band number characters       */

  /* Initialize sdf array
  -----------------------*/
  for (i = 0; i < 100; i++)
    sdf[i] = 0;

  /* Calculate total number of bands and
     the starting data file for each volume
  -----------------------------------------*/
  sdf[0] = 1;

  for (i = 0; i < numnhdr; i++)
    totbnd += bpv[i];

  for (i = 1; i < numnhdr; i++)
    {
    tmpbnd = 0;
    for (j = 0; j < i; j++)
      tmpbnd += bpv[j];
    sdf[i] = tmpbnd + 1;
    }

  for (i = 1; i <= numnhdr; i++)
    { 
    /* Open the input and output files
    ----------------------------------*/
    sprintf(infile,"%s%s",hostout,".hd");
    if ((inf=fopen(infile,"r")) == NULL)
      c_errmsg("Error opening the input header file","ndfout-open",&fatal);

    sprintf(outfile,"%s%s%d",hostout,".h",i);
    if ((outf=fopen(outfile,"w")) == NULL)
      c_errmsg("Error opening the output header file","ndfout-open",&fatal);

    /* Write the information to the output file,
       changing certain pieces of data along the way
    ------------------------------------------------*/
    while (fscanf(inf,"%s",inbuff) != EOF)
      { 
      if (strncmp(inbuff,"TAPE_SPANNING_FLAG=",19) == 0)
        sprintf(outbuff,"TAPE_SPANNING_FLAG=%d/%d;\n",i,numnhdr);

      else if (strncmp(inbuff,"START_LINE_NUMBER=",18) == 0)
        sprintf(outbuff,"START_LINE_NUMBER=%d;\n",slnum);

      else if (strncmp(inbuff,"START_DATA_FILE=",16) == 0)
        sprintf(outbuff,"START_DATA_FILE=%d;\n",sdf[i-1]);

      else if (strncmp(inbuff,"LINES_PER_VOLUME=",17) == 0)
        {
        p = malloc(sizeof(char));
        p = &inbuff[17];
        totlin = atoi(p);
        lpf = totlin/totbnd;
        lpv = lpf * bpv[i-1];
        sprintf(outbuff,"LINES_PER_VOLUME=%d;\n",lpv);
        slnum += lpv;
        }

      else if  (strncmp(inbuff,"NUMBER_OF_BANDS_IN_VOLUME=",26) == 0)
        sprintf(outbuff,"NUMBER_OF_BANDS_IN_VOLUME=%d;\n",bpv[i-1]);

      else if (strncmp(inbuff,"MAP_PROJECTION_NAME=",20) == 0)
      {
      strcpy(outbuff,inbuff);
      while (strchr(inbuff,';') == NULL)
        {
        fscanf(inf,"%s",inbuff);
        sprintf(outbuff,"%s %s",outbuff,inbuff);
        }
      sprintf(outbuff,"%s\n",outbuff);
      }
      else if (strncmp(inbuff,"BAND",4) == 0)
        {
        if ((p=strstr(inbuff,"BAND")) != NULL)
          {
          bgnfield = (long)strcspn(p,"D");
          endfield = (long)strcspn(p,"_");
          field_length = (endfield - 1) - bgnfield;
          strncpy(band,&p[4],field_length);
          band[field_length] = '\0';
          sscanf(band,"%d",&bnd);
          }
        if (((bnd >= sdf[i-1]) && (bnd < sdf[i]) && (i < numnhdr))  ||
            ((bnd >= sdf[i-1]) && (i == numnhdr)))
          sprintf(outbuff,"%s\n",inbuff);
        else
          strcpy(outbuff,"");
        }
      else
        sprintf(outbuff,"%s\n",inbuff);

      fprintf(outf,"%s",outbuff);  
      } 

    fclose(outf);
    fclose(inf);
    }
}
