/****************************************************************************
NAME: TAELAND

PURPOSE: Writes the number of files per volume, number of volumes, and the
         record size of each file back to TAE.  It also creates a project
         parameter file (.ppf) to hold all the filenames.

PROGRAM HISTORY:

Version   Date     Programmer      Code  Reason
-------   ----     --------------  ----- ------
  1.0     03/98    T.Ratliff        C    Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
  Must be run under UNIX and TAE

****************************************************************************/

#include <stdio.h>
#include "las.h"
#include "protos.h"

void taeland
(
    struct PARBLK *vblk,                    /* TAE parameter block        */
    long nvol,                              /* number of volumes          */
    long fpv[100],                          /* number of files per volume */
    long recdsz[300],                       /* record size of each file   */
    long nfiles,                            /* number of files found      */
    char fnames[][CMLEN],                   /* filenames to write to tape */
    char *hostout                          /* NDF file names in UNIX form*/
)

{
  long fatal = -1;                      /* fatal error flag           */
  long status = 0;                      /* return status of functions */
  long i = 0;                           /* loop counter               */

  char tmpfile[CMLEN];                  /* name of .ppf file          */

  FILE *outf;                           /* pointer to .ppf file       */

  /* Creating a ppf file to store the filenames
  ---------------------------------------------*/
  sprintf(tmpfile,"%s.ppf",hostout);
  outf = fopen(tmpfile,"w");
  for (i = 0; i < nfiles; i++)
    fprintf(outf,"FNAMES%d            S 1 %s\n",i+1,fnames[i]);
  fclose(outf);

  /* Write number of volumes,record sizes,
     and files per volume back to TAE
  ----------------------------------------*/
  status = q_intg(vblk,"NVOL",1,&nvol,P_UPDATE);
  if (status == E_FAIL)
    c_errmsg("Fatal error encountered","ndfout-fatal",&fatal);
  status = q_intg(vblk,"FPV",nvol,fpv,P_UPDATE);
  if (status == E_FAIL)
    c_errmsg("Fatal error encountered","ndfout-fatal",&fatal);
  status = q_intg(vblk,"RECSIZ",nfiles,recdsz,P_UPDATE);
  if (status == E_FAIL)
    c_errmsg("Fatal error encountered","ndfout-fatal",&fatal);
  status = q_out(vblk);
  if (status == E_FAIL)
    c_errmsg("Fatal error encountered","ndfout-fatal",&fatal);

}
