/*****************************************************************************
NAME:                          TAPECHECK

PURPOSE:
   Check for existance of requested files to write to tape.  Also check
   file sizes for tape spanning requirements, and to generate additional
   header files when required to span output media volumes.

PROGRAM HISTORY
Version  Date   Author          Code/Contr   Change Request
-------  -----  --------------  ---------    --------------
  1.0    03/98  T.Ratliff       CSB          Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
    must be run under TAE

PROJECT: LAS

ALGORITM:
  Retrieve the parameters from the parameter block
  Get the number of bands and number of samples from the NDF header file
  Get the names of all the files to copy to tape and calculate total size
  Determine if volume spanning is required and if so, create additional
  header files
  Write variables back to TAE
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "protos.h"

main()
{
  struct PARBLK vblk;                 /* TAE parameter block            */

  long std_out = 0;                   /* standard out                   */
  long ns = 0;                        /* number of samples              */
  long bytes_per_sample = 0;          /* number of bytes per sample     */
  long nb = 0;                        /* number of bands                */
  long i = 0;                         /* loop counter                   */
  long nfiles = 0;                    /* number of files found          */
  long nvol = 0;                      /* number of volumes              */
  long filesize[300];                 /* size of targer files           */
  long fpv[100];                      /* number of files per volume     */
  long recdsz[300];                   /* record size of output files    */
  long bpv[100];                      /* number of bands per volume     */
  
  double totsize = 0.0;               /* size of all order files        */

  char hostout[CMLEN];                /* NDF file names in UNIX format  */
  char tapetype[10];                  /* tape media type                */
  char tapedens[5];                   /* density of tape media          */
  char bform[5];                      /* data format of files           */
  char fnames[300][CMLEN];            /* filenames to write to tape     */
  

  /* Initialize TAE parameter block
  ---------------------------------*/
  c_inlas(&vblk,&std_out);

  /* Get the variables from TAE variable block
  --------------------------------------------*/
  getpar(&vblk,hostout,tapetype,tapedens,bform);

  /* Get the number of samples and
     number of bands
  --------------------------------*/
  getnbdns(hostout,&ns,&bytes_per_sample,&nb);

  /* Initialize the filesize and fpv arrays
  -----------------------------------------*/
  for (i = 0; i < 300; i++)
    filesize[i] = 0;

  for (i = 0; i < 100; i++)
    fpv[i] = 0;

  /* Get the files to write to tape
  ---------------------------------*/
  getfiles(hostout,fnames,filesize,&totsize,nb,&nfiles,bform);

  /* Determining if volume spanning is required,
     if so, create additional header files
  ----------------------------------------------*/
  checktp(tapetype,bform,hostout,fnames,totsize,ns,bytes_per_sample,
          filesize,bpv, recdsz,fpv,&nvol,&nfiles);

  /* Returning variables to TAE
  -----------------------------*/
  taeland(&vblk,nvol,fpv,recdsz,nfiles,fnames,hostout);

  return(E_SUCC);
}
