/*******************************************************************************
NAME		GETPAR 

PURPOSE		To get the necessary parameters and perform the preliminary
		edit checking. 

ALGORITHM:
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblock,	/* pointer to TAE parameter block	             */
    char taein[][CMLEN],/* pointer to first TAE file descriptor              */
    char hosin[][CMLEN],/* pointer to first host file descriptor             */
    char taeout[][CMLEN],/* pointer to output TAE file descriptor             */
    char hosout[][CMLEN],/* pointer to output host file descriptor            */
    long *nmrimg,	       /* number of input images for image           */
    long bands[][MAXBND + 1],   /* band specification for first image         */
    long *nbands,               /* number of bands per image	             */
    long window[][4],	       /* window specification for image             */
    long *msgtim,	       /* elapsed time between display messages      */
    char *minmax,		       /* min max flag buffer			     */
    long *odtype,	       /* output data type			     */
    long *totbnd,	       /* total number of bands in image             */
    float *scale,		       /* scale factor				     */
    float *offset,		       /* offset to be added to the normd calculation*/
    float *rndval,                 /* Rounding option 			     */
    long *normlim,	       /* normalized difference limit		     */
    long *nmasks                  /* Number of mask values                      */
)
{
struct VARIABLE *var;	       /* pointer to a TAE variable within vblock    */

long    element;	       /* element for getin call		     */
long    fatal = -1;	       /* fatal error flag			     */
long	i;		       /* counter				     */
long	maximg = 2;	       /* maximum number of images to be processed   */
long    nonfatal = 1;	       /* nonfatal error flag			     */
long	qflag[MAXIMG][2];      /* flag indicating if subwindows specified    */
long    same;		       /* flag indicating if images have same window */
long	status;		       /* function return status code		     */

/*
  Get the input image name
*/
element = 0;
status = c_getin(vblock, "IN", &element, taein, hosin, nmrimg, window, bands,
    nbands, totbnd, &maximg, &same, qflag);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","normd-fatal",&fatal);


/*
  Check to determine if there are enough bands to do the calculations
*/
if (*nmrimg == 1 && *nbands == 1)
    {
    c_errmsg("Image IN contains less than two bands","normd-in",&nonfatal);
    c_errmsg("Fatal error encountered","normd-fatal",&fatal);
    }


/*
  Check the windows
*/
if (same == 0)
    {
    c_errmsg("Images do not have same window size", "normd-window",&nonfatal);
    c_errmsg("Fatal error encountered","normd-fatal",&fatal);
    }


/*
  Check the bands
*/
for (i = 0; i < *nmrimg; i++) 
    {
    if (nbands[i] != nbands[0])
        {
	c_errmsg("Images do not have same number of bands",
	    "normd-bands", &nonfatal);
        c_errmsg("Fatal error encountered", "normd-fatal", &fatal);
        }
    }


/*
  Get output image name
*/
status = c_getout(vblock, "OUT", taeout, hosout);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","normd-fatal",&fatal);


/*
  Get the output data type; default to the input data type
*/
var = p_find(vblock,"ODTYPE");
if (strcmp(SVAL(*var,0),"BYTE") == 0)
    *odtype = EBYTE;
else if (strcmp(SVAL(*var,0),"I*2") == 0)
    *odtype = EWORD;
else if (strcmp(SVAL(*var,0),"I*4") == 0)
    *odtype = ELONG;
else if (strcmp(SVAL(*var,0),"R*4") == 0)
    *odtype = EREAL;
else
    *odtype = 0;


/*
  Get the offset to be added to the normalized difference calculation
*/
var = p_find(vblock, "OFFSET");
*offset=RVAL(*var,0);


/*
  Get the scale factor
*/
var = p_find(vblock, "SCALFACT");
*scale=RVAL(*var,0);


/*
  Get the rounding option
*/
var = p_find(vblock,"ROUNDOPT");
if (strcmp(SVAL(*var,0),"ROUND") == 0)
    *rndval = 0.5;
else
    *rndval = 0.0;


/*
  Get the max ND value limit.  The default should be set to the actual ND value
  of 1 which is scaled using the user specified values of OFFSET and SCALFACT.
*/
var = p_find(vblock, "NORMLIM");
if (var->v_count)
    *normlim = IVAL(*var,0);
else
    *normlim = (long) ((1 + *offset) * *scale);


/*
  Determine if the input image(s) have mask values
*/
var = p_find(vblock, "SCALOPT");
if (strcmp(SVAL(*var,0), "GLOBAL") == 0)
    *nmasks = 10;
else
    *nmasks = 0;



/*
  Get TAE globals 
*/
var = p_find(vblock, "$MSGTIM");
*msgtim = IVAL(*var, 0);

var = p_find(vblock, "$MINMAX");
strcpy(minmax, SVAL(*var,0));

return(E_SUCC);
}
