#include "overlay.h"
#include "util.h"

void filspec
(
    char hosin[][CMLEN],      /* input image host name descriptors  */
    char hosout[][CMLEN],            /* output image host name descriptor  */
    long *num_bands,          /* number of bands in each input image*/
    long num_image_bands[],
    long image_band_num[][MAXBND + 1],
    long wind[][4],           /* input image windows                */
    long *type
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up input and output file specifications

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call C_EOPENS to open input and output images
   Check for images to be of type byte
   Call C_EGROUP to group the images

**************************************************************************
*/
{
double	line_inc = 1.0,
	samp_inc = 1.0;

long	access,                      /* file access type                   */
	bufsiz,                      /* image i/o buffer size              */
	comb_flag = COMB,
	img,
	stat;                        /* return status                      */

static  long    fatal_err = -1;

long    zero = 0;
long    opt[3] = {0,0,0};
long    one = 1;
char *prog_name = "overlay";  /* program name            */


/*
open input and output image files
---------------------------------
*/
access = IREAD;
for ( img = 0; img < 2; img++)
{
    *type = opt[0];
    stat = c_eopens (&fdesc[img], hosin[img], image_band_num[img], 
                     &wind[img][SL], &wind[img][SS],&wind[img][NL],
                     &wind[img][NS],
                     ipp[img], &access, type, &zero, opt);
    if (stat != E_SUCC )
       c_errmsg ("Error opening input image", "overlay-inopen", &fatal_err);
    if ( *type != EBYTE )
       c_errmsg ("Input image is not byte data type", "overlay-type",
              &fatal_err);
}  

access = IWRITE;
stat = c_eopens(&fdesc[img], hosout[0], num_bands,
                     &wind[img-1][SL], &wind[img-1][SS],&wind[img-1][NL],
                     &wind[img-1][NS],op,&access,type,&zero,opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "overlay-outopen", &fatal_err);

fdesc[img+1] = 0;
/*
update output image DDR based on input images
---------------------------------------------
*/
c_upddr (&vblock, hosin, hosout[0], &img, wind, image_band_num, num_image_bands,
         &line_inc, &samp_inc, num_bands, &comb_flag);

/*
set up group containing input and output bands
----------------------------------------------
*/
if ((buf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_abort (&vblock, "Error allocating dynamic memory", "multiply-alloc", 
            prog_name, hosout, &one);

c_egroup (fdesc, &gdesc, buf, &bufsiz);
}
