#include "overlay.h"
#include "util.h"

void getpar
(
    char hos_in[][CMLEN],     /* input images host names          */
    char las_out[][CMLEN],           /* output image las name            */
    char hos_out[][CMLEN],           /* output image host name           */
    long num_images[],         /* number of images to be overlayed*/
    long *num_bands,          /* total number of input image bands*/
    long num_image_bands[MAXIMG],
    long image_band_num[][MAXBND + 1],
    long wind[MAXIMG][4]     /* input image windows              */
)
  	
/*
**********************************************************************

NAME: GETPAR

PURPOSE: Retrieves parameter values 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
     Initialize the program
     Call get_in to get names of input images
     If number of bands does not agree with both input images 
        issue fatal error message
     Call get_out to get output image name
     
**********************************************************************
*/

{
struct VARIABLE  *var;              /* pointer to a variable in TAE block */

long	flag[MAXIMG][2],            /* flag for subwindows                */
	imglimit = MAXIMG,          /* maximum number of images           */
	nbands,
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

static	char las_in[MAXIMG][CMLEN], /* input images las names             */
             minmax[4];

long    one = 1;
long    zero = 0;
long    fatal_err = -1;
char *prog_name = "overlay";  /* program name            */


char tmpstr[1][CMLEN];  /* Temp string */
strcpy(tmpstr[0],"\0");

/*
initialize program 
------------------ */
c_inlas (&vblock, &std_out);

   stat = c_getin (&vblock,"IN",&zero, las_in, hos_in, 
                   &num_images[0], wind, image_band_num, 
                   &num_image_bands[0], 
                   &nbands, &imglimit, &same_window, flag);
   if (stat != E_SUCC) 
       c_errmsg ("Error accessing input image1 ", "overlay-in", 
              &fatal_err);

   *num_bands = nbands;
   stat = c_getin (&vblock,"IN",&one, las_in[1], hos_in[1], 
                   &num_images[1], wind[1], image_band_num[1], 
                   &num_image_bands[1], 
                   &nbands, &imglimit, &same_window, flag[1]);
   if (stat != E_SUCC) 
       c_errmsg ("Error accessing input image2 ", "overlay-in", 
              &fatal_err);

   if (nbands != *num_bands) 
       c_abort (&vblock, "Input images must have the same number of bands", 
               "overlay-nbands", prog_name, tmpstr, &zero);
/*
get and validate output image parameters 
---------------------------------------- */
stat = c_getout (&vblock, "OUT", las_out, hos_out);
if (stat != E_SUCC)
   c_abort (&vblock, "Error getting output image name", "overlay-out", 
            prog_name, tmpstr, &zero);
/*
Set the output image size as the same size of input images
---------------------------------------------------------- */
if (wind[0][NL] != wind[1][NL] || wind[0][NS] != wind[1][NS] )
   c_abort (&vblock, "Input images are not of same size", "overlay-in", 
            prog_name, tmpstr, &zero);

/* Get globals 
   ___________ */
var = p_find( &vblock, "$msgtim");
msgtim = IVAL( *var, 0);
var = p_find( &vblock, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;
}
