#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "protos.h"

struct PARBLK vblock;                  /* TAE parameter block     */

FILE    *fd;                           /* output file descriptor  */

struct FDESC *    fdesc[MAXIMG];                 /* file descriptors        */
struct GDESC *    gdesc;                         /* group descriptor        */
long    ipp[2][MAXBND + 1],        /* offsets into image i/o  */
                op[MAXBND + 1];            /* for input and output bands */
long    msgtim;                        /* message interval        */
long    min_max;

unsigned char   *buf;                  /* image i/o buffer        */
extern long     curcat;                        /* message utility         */
extern long     curent;                        /* parameters for processing */

/*
***************************************************************************

NAME: OVERLAY

PURPOSE: Creates an output image based on the logical relationship
         between two input images.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   2          24-Mar-89  J. McGuthry, STX     Conversion to C
   3           3-Mar-93  D. Etrheim, EDC      Modify compute.c to exit if
					      no conditions have been entered 
   6.1        18-May-94  D. Eichacker, EDC    Modify overlay for eopens to
					      account for compression.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
     Call GETPAR to get and validate user-supplied parameter values 
     Call FILSPEC to set up input and output file specifications
     Call COMPUTE to perform overlay and produce output imagee
     Call TERMINATE to update output image history and issue
        successful completion message

**********************************************************************
*/

int main
(void) 
{
static	long image_band_num[MAXIMG][MAXBND + 1], 
                                 /* band numbers in each input image */
	num_bands,               /* total number of input image bands*/
	num_image_bands[MAXIMG], 
                                 /* number of bands in each image    */
	num_images[2],           /* number of input images           */
        dtype,                   /* data type of images              */
	wind[MAXIMG][4];         /* input image windows              */


static	char	hos_in[2][CMLEN];
                                 /* input images host names          */
char	hos_out[1][CMLEN],          /* output image host name           */
        las_out[1][CMLEN];          /* output image las name            */

/*
get parameters
-------------- */
getpar (hos_in, las_out, hos_out,num_images, &num_bands,
        num_image_bands, image_band_num, wind);

/*
set up input and output file specifications
------------------------------------------- */
filspec (hos_in, hos_out, &num_bands, 
         num_image_bands, image_band_num, wind,&dtype);

/*
perform overlay
--------------- */
compute (hos_out,&num_bands,&wind[0][NL],
        &wind[0][NS],&dtype);

/*
update output image history and issue completion message
-------------------------------------------------------- */
terminate (las_out, hos_out[0], hos_in, num_images); 

return EXIT_SUCCESS;

}
