/*******************************************************************************
NAME:      		   	FAST_PROC

PURPOSE:    

PROGRAM HISTORY:
PROGRAMMER	  DATE		REASON
----------	  ----		------
B. Wilsey	  06/84		Original development
R. McKinney	  10/87		#4206
B. Ailts	  08/87		newLAS conversion
B. Ailts	  10/88		LAS 5.0 conversion +
				PRF #4595
T. Baltzer  	  04/91		Change eclose() to c_eclose()
				Change egclse() to c_egclse()
D. Eichacker      05/94		Added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PERSPEC must be run under TAE.

PROJECT:        LAS		                

ALGORITHM DESCRIPTION:
Allocate buffer space
Initialization of variables
Open elevation image
Open reflectance image
Open a output file
Initialize constants that will be needed for the perspec calculations
Calculate the viewer's height
Initialize the processing message
Read the elevation data
Read the reflectance data
For each output line
   Fill the output buffer with zeros
   Initialize the variables for each output line
   Retreive elevation data
   If height if greater than previous height
      Retreive reflectance data
      Fill in output buffer with correct number of pixels and pixel value
   When  current line equals number of output lines, rotate the image
   and write to the output image
            Calculate the min/max values of image
Close all the images
Free buffer space

ALGORITHM REFERENCES:
The main processing algorithm was originally taken from:
  B. Fishman and B. Schachter, "Computer Display of Height Fields," 
  Computers and Graphics, Vol.  5, pp. 53-60, 1980.

*******************************************************************************/
#include <math.h>

#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

#define RAD 0.01745329

extern long curent;	/* current line				*/
extern long curcat;	/* currnet band				*/

void fast_proc
(
    struct PARBLK *vblk,	/* I: TAE parameter block		      */
    char *hosin1,		/* I: host name of elevation image            */
    char *hosin2,		/* I: host names of reflectence images	      */
    char *hosout,		/* I: host output name			      */
    long *window1,		/* I: window of elevation image		      */
    long *bands1,		/* I: band specification of first input image */
    long *window2,		/* I: window of reflectance image             */
    long *bands2,		/* I: band specification of second input image*/
    long *totbnd,		/* I: number of bands in second input image   */
    long *out_wind,		/* I: output window specification             */
    float *pixsiz,		/* I: pixel size                              */
    long *viewloc,		/* I: viewer location			      */
    float *viewdir,		/* I/O: viewer direction		      */
    float *foview,		/* I: field of view                           */
    float *vertscl,		/* I: vertical scaling factor                 */
    long *horizn,		/* I: height of horizon 		      */
    long *skyval,		/* I: value of the sky line		      */
    long *odtype,		/* I: output data type			      */
    long *msgtim,		/* I: time between messages		      */
    char *minmax,		/* I: min/max flag			      */
    double *minval,		/* I/O: minimum value of output image	      */
    double *maxval		/* I/O: maximum value of output image	      */
)

{  /*  proces  */

double cosgam;			/* cosine of gamma			*/
double dx;			/* delta x				*/
double dy;			/* delta y				*/
double fov;			/* corrected field of view		*/
double fovt;			/* corrected field of view		*/
double gamma;		
double range;	
double rld2_div_fov;		/* line dimension			*/
double x;			/* the x direction			*/
double y;			/* the y direction			*/

float dh[4];			/* height of pixel			*/
float dh12;			/* height of pixel			*/
float dh34;			/* height of pixel			*/
float dn12;			/* reflectence of pixel			*/
float dn34;			/* reflectence of pixel			*/
float factx;			/* factor in x direction		*/
float facty;			/* factor in y direction		*/
float rld2;			/* line dimension			*/
float tanb;

long acc = IREAD;		/* access code				*/
long bnd = 0;			/* total bands in input image		*/
extern long buffersiz;		/*  size of the data buffer		*/
long buf_siz;			/*  buffer size				*/
long curbnd;			/*  counter -- current band		*/
long cur_line;			/* curent line into temp buffer		*/
long dtype = EWORD;		/* data type of elevation data		*/
long fatal = -1;		/* fatal error code			*/
long i;				/* counter -- current band		*/
long idh;			/*  elevation pixel value		*/
long ix;	
long iy;	
long irow;
long j;				/* counter -- current output line	*/
long ld2;
long line;			/* counter -- curent line		*/
long mirow = 1;			/* minimum row				*/
long mode = 0;			/* mode of status check			*/
long mrow;			/* row to be modified			*/
long nnl;
long nnnl;
long nnnl_m_horizn;
long nonfatal = 1;		/* nonfatal error code			*/
long npix;			/* number of pixels			*/
long ns;			/* number of samples			*/
extern long offset[4];		/* offset of the output buffer		*/
long off_cur;
long one = 1;			/* value of one				*/
static	long	opt[3]={0,0,0}; /* optional parameter array for EOPENR  */
long pos_curcat;		/* position of current category(band)	*/
long pos_iy1;			/* first postion of iy			*/
long pos_iy2;			/* second postion of iy			*/
long status;			/* return value of function calls	*/
long temp;			/* temporary variable -- line of vz value*/
long winldiff;			/* difference of lines between WINDOW1 and
				   WINDOW2				*/
long winsdiff;			/* difference of samples between WINDOW1 and
				   WINDOW2				*/
long vz;			/* vertical hieght of viewer		*/

long zero = 0;			/* value of zero			*/

int fdesc1;			/*  file descriptor of elevation data	*/
int fdesc2;			/*  file descriptor of reflectence data */
extern int fdesc3[2];		/*  file descriptor of output data	*/
extern int gdesc;		/*  group descriptor of output data	*/

short *dembuf;			/*  buffer of elevation data		*/
short dn[4];			/* reflectence of pixel			*/

unsigned char idn;		/* output pixel value			*/
unsigned char *imgbuf;		/* image buffer 			*/
unsigned char *out_buf;		/* temporary output buffer		*/
unsigned char bkgrnd;		/* backround value			*/
unsigned char *buffer;		/* output data buffer			*/

/*  Calculate the difference between WINDOW1 and WINDOW2
--------------------------------------------------------*/
winldiff = window2[0] - window1[0] + 1;
winsdiff = window2[1] - window1[1] + 1;

/*  allocation of image buffers
-------------------------------*/
dembuf = (short *)malloc(window1[2] * window1[3] * 2);
if (dembuf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","perspec-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

imgbuf = (unsigned char *)malloc(*totbnd * window2[2] * window2[3]);
if (imgbuf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","perspec-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

out_buf = (unsigned char *)malloc(*totbnd * out_wind[0] * out_wind[1]);
if (out_buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","perspec-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

/*  Initialization of variables
-------------------------------*/
ns = out_wind[1];
nnl = out_wind[0];

/*  Open elevation image
------------------------*/
status = c_eopenr((struct FDESC **)&fdesc1,hosin1,&one,&window1[0],&window1[1],
		  &window1[2],&window1[3],&acc,&dtype,opt,&window1[2]);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);

/* Open reflectance image
-------------------------*/
dtype = EBYTE;
status = c_eopenr((struct FDESC **)&fdesc2,hosin2,&bnd,&window2[0],&window2[1],
		  &window2[2],&window2[3],&acc,&dtype,opt,&window1[2]);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);

/*  Open a output file
----------------------*/
acc = IWRITE;
status = c_eopens((struct FDESC **)&fdesc3[0],hosout,totbnd,&one,&one,&nnl,&ns,
		  offset,&acc,odtype,&zero,opt);
if (status != E_SUCC)
   {
   if (c_eclose((struct FDESC **)&fdesc3[0]) == E_FAIL)
      c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
   c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
   }

fdesc3[1] = 0;

buffer = (unsigned char *)c_ealloc((struct FDESC **)fdesc3,&buf_siz);
if (buffer == NULL)
   {
   if (c_eclose((struct FDESC **)&fdesc3[0]) == E_FAIL)
      c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
   c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
   }

status = c_egroup((struct FDESC **)fdesc3,(struct GDESC **)&gdesc,buffer,
	          &buf_siz);
if (status != E_SUCC)
   {
   if (c_eclose((struct FDESC **)&fdesc3[0]) == E_FAIL)
      c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
   c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
   }

status = c_estep((struct GDESC **)&gdesc);
if (status != E_SUCC)
   {
   if (c_egclse((struct GDESC **)&gdesc) == E_FAIL)
      c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
   c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
   }

/*  Initialize constants that will be needed for the perspec calculations
-------------------------------------------------------------------------*/
*viewdir = 270 - *viewdir;
if (*viewdir < 0)
   *viewdir = *viewdir + 360;
fovt = (double) (*foview * RAD);
fov = tan(fovt);
ld2 =  nnl/2;
rld2 = nnl/2.0;
rld2_div_fov = rld2 / fov;
*viewdir = *viewdir * RAD;
if ( nnl < ns)
   nnnl = ns;
else
   nnnl = nnl;
nnnl_m_horizn = nnnl - *horizn;

/*  Calculate the viewer's height
---------------------------------*/
if (((viewloc[0] - window1[0] + 1) > 0) && 
    ((viewloc[1] - window1[1] + 1) > 0) &&
    ((viewloc[0] - window1[0] + 1) <= window1[2]) &&
    ((viewloc[1] - window1[1] + 1) <= window1[3]))
   {
   temp = viewloc[0] - window1[0] + 1;
   status = c_eread((struct FDESC **)&fdesc1,&(bands1[0]),&temp,
		     (unsigned char *)dembuf,&one);
   if (status == E_FAIL)
      {
      if (c_egclse((struct GDESC **)&gdesc) == E_FAIL)
         c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
      c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
      }
   vz = (long) *(dembuf + viewloc[1] - window1[1]) + viewloc[2];
   }
else
   vz = viewloc[2];

/* Initialize the processing message
------------------------------------*/
mode = ONECAT;
curent = 1;
curcat = 1;
c_inittm("PERSPEC","Processed","line","band"," ",&nnl,totbnd,msgtim,&mode);

c_startm();

/*  Read the elevation data
---------------------------*/
status = c_eread((struct FDESC **)&fdesc1,&(bands1[0]),&one,
	         (unsigned char *)dembuf,&window1[2]);
if (status == E_FAIL)
   {
   if (c_egclse((struct GDESC **)&gdesc) == E_FAIL)
      c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
   c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
   }

/*  Read the reflectance data
-----------------------------*/
for (curbnd = 0; curbnd < *totbnd; curbnd++)
   {
   curcat = curbnd + 1;
   status = c_eread((struct FDESC **)&fdesc2,&bands2[curbnd],&one,
		    imgbuf + (curbnd * window2[2] * window2[3]),&window2[2]);
   if (status == E_FAIL)
      {
      if (c_egclse((struct GDESC **)&gdesc) == E_FAIL)
         c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
         c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
      }
   }  /*  curbnd < *totbnd  */

cur_line = 0;

/*  for each output line
------------------------*/
for (line = 0; line < nnl; line++)
   {  
   
   /*  Fill the output buffer with zeros
   -------------------------------------*/
   for (curbnd = 0; curbnd < *totbnd; curbnd++)
      {
      curcat = curbnd + 1;
      bkgrnd = (unsigned char)skyval[curbnd];
      c_pxfill(out_buf + (curbnd * ns * nnl) + (cur_line * ns),
	       &dtype,&ns,&bkgrnd);
      }  /*  for curbnd < * totbnd  */

   /*  Initialize the variables for each output line
   -------------------------------------------------*/
   gamma = atan((curent - ld2) * fov / rld2);
   dx = -cos(*viewdir + gamma);
   dy = sin(*viewdir + gamma);
   cosgam = cos(gamma) * *pixsiz;
   mrow = ns;
   range = cosgam;
   x = (double)viewloc[1];
   y = (double)viewloc[0];
   while ((range <= (float)ns * *pixsiz) && (mrow > 1))
      {
      ix = (long)(x);
      iy = (long)(y);
      if (((ix - window1[1] + 1) >= 1) && 
	  ((ix - window1[1] + 1) < window1[2]) && 
	  ((iy - window1[0] + 1) >= 1) && 
	  ((iy - window1[0] + 1) < window1[3]))
	 {  /*  if within limits of output image  */
	 pos_iy1 = ((iy - window1[0] + 1) - 1) * window1[3];
	 pos_iy2 = pos_iy1 + window1[3];
	 dh[0] = *(dembuf + pos_iy1 + (ix - window1[1] + 1) - 1);
	 dh[1] = *(dembuf + pos_iy1 + (ix - window1[1] + 1));
	 dh[2] = *(dembuf + pos_iy2 + (ix - window1[1] + 1) - 1);
	 dh[3] = *(dembuf + pos_iy2 + (ix - window1[1] + 1));
	 factx = x - ix;
	 facty = y - iy;
	 dh12 = dh[0] + (dh[1] - dh[0]) * factx;
	 dh34 = dh[2] + (dh[3] - dh[2]) * factx;
	 idh = (long)(dh12 + (dh34 - dh12) * facty + .5);
	 if (idh != 0)
	    {
	    tanb = (*vertscl * (float)(idh - vz)) / range;
	    mirow = (nnnl_m_horizn - floor((tanb * rld2_div_fov) + .5) < nnnl) ?
	             nnnl_m_horizn - floor((tanb * rld2_div_fov) + .5) : nnnl;
	    irow = mirow > 1 ? mirow : 1;
	    }
	 else
	   irow = mirow;
	 if ( irow < mrow )
	    {
	    pos_iy1 = ((iy - window2[0] + winldiff) - 1) * window2[3];
	    pos_iy2 = pos_iy1 + window2[3];
	    for (curbnd = 0; curbnd < *totbnd; curbnd++)
	       {
	       curcat = curbnd + 1;
	       pos_curcat = curbnd * window2[3] * window1[2];
	       dn[0] = (short) *(imgbuf + pos_curcat + pos_iy1 + 
				((ix - window2[1] + winsdiff) - 1));
	       dn[1] = (short) *(imgbuf + pos_curcat + pos_iy1 + 
				(ix - window2[1] + winsdiff)); 
	       dn[2] = (short) *(imgbuf + pos_curcat + pos_iy2 + 
				((ix - window2[1] + winsdiff) - 1)); 
	       dn[3] = (short) *(imgbuf + pos_curcat + pos_iy2 + 
				(ix - window2[1] + winsdiff)); 
	       dn12 = dn[0] + (dn[1] - dn[0]) * factx;
	       dn34 = dn[2] + (dn[3] - dn[2]) * factx;
	       idn = dn12 + (dn34 - dn12) * facty + .5;
	       npix = mrow - irow + 1;
	       c_pxfill(out_buf + pos_curcat + 
			(cur_line * nnl) + irow - 1,&dtype,&npix,&idn);
	       }  /*  for curbnd < *totbnd  */
	    mrow = irow - 1;
	    }  /* if irow < mrow  */
	 }  /*  if within limits of output image  */
      x += dx;
      y += dy;
      range += cosgam;
      }  /*  while (range <= nw * *pixsiz) && mrow > 1) */

   /*  When  current line equals number of output lines, rotate the image
       and write to the output image
   -----------------------------------------------------------------------*/
   if ( cur_line == nnl - 1)
      {
      cur_line = 0;
      for (i = 0; i < nnl; i++)
         {
	 for (curbnd = 0; curbnd < *totbnd; curbnd++)
	    {
	    curcat = curbnd + 1;
	    pos_curcat = curbnd * ns * nnl;
	    off_cur = *(offset + curbnd);
	    for (j = 0; j < ns; j++)
	       *(buffer + off_cur + nnl - j - 1) = 
	       *(out_buf + pos_curcat + i + (j * nnl));

            /*  Calculate the min/max values of image
	    -----------------------------------------*/
	    if (strcmp(minmax,"YES") == 0)
 	       {
	       status = c_pxmnmx((long *)(buffer + off_cur),&ns,minval,
			          maxval,&dtype);
               if (status != E_SUCC)
                  {
                  if (c_egclse((struct GDESC **)&gdesc) == E_FAIL)
                     c_errmsg("Nonfatal error encountered","perspec-warn",
		              &nonfatal);
                  c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
                  }
	       }  /* if minmax == yes  */
	    }  /* for curbnd < *totbnd  */
         status = c_estep((struct GDESC **)&gdesc);
         if (status != E_SUCC)
            {
            if (c_egclse((struct GDESC **)&gdesc) == E_FAIL)
               c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
            c_clean(vblk,"perspec",(char (*)[256])hosout,&one);
            }
         }  /*  for i < window1[2]  */
      }  /*  if line == window1[2]  */
   else
      cur_line ++;
   curent ++;
   }  /*  for line < nnl  */

c_stoptm();

/*  Close all the images
------------------------*/
status = c_egclse((struct GDESC **)&gdesc);
status = c_eclose((struct FDESC **)&fdesc1);
status = c_eclose((struct FDESC **)&fdesc2);

/*  Free buffer space
---------------------*/
free(dembuf);
free(imgbuf);
free(out_buf);
free(buffer);

return;
}  /*  fast_proc  */
