/*******************************************************************************
NAME:      		   	PROCESL

PURPOSE:    

PROGRAM HISTORY:
PROGRAMMER	  DATE		REASON
----------	  ----		------
B. Wilsey	  06/84		Original development
R. McKinney	  10/87		#4206
B. Ailts	  08/87		newLAS conversion
B. Ailts	  10/88		LAS 5.0 conversion + PRF #4595
D. Eichacker      05/94		Add opt[]

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PERSPEC must be run under TAE.

PROJECT:        LAS		                

ALGORITHM DESCRIPTION:
Calculate the number of lines in the buffer
Allocate space for the buffers
Set the number of lines and samples in the temporary image
Create an arrary of bands for the subsequent output open calls
Allocate space for the row marker for each line of temporary image
Open the input images
Remove the temporary file if it already exists
Calculate the iy scaling factor depending on the viewing angle -- the 
 angle will determine the direction that the input image is read
Initialize constants that will be needed for the perspec calculations
Calculate the height of the viewing location by adding the input Z value
 to the elevation of the viewing location.  The viewing location is
 relative to the full image, not just the window.
Set the first input line and calculate the number of blocks the image
 will be split into in order to proces effeciently
Initialize the processing message
For curcat = 1 to nblock 
   Open a temporary file -- this file will be rotated 90 degrees for the
    final output image -- open for update after first block is processed
   For  band = 0 to *totbnd
   For line = 0 to nnl
      If first time thru block loop
         Fill the output buffer with the fill value specified
      Else
         Read line of output image
      Calculate the initial values
      While range <= ns * *pixsiz && marking row  > 1
         If current line and sample are within the current block
	    A bilinear smoothing is performed on the elevation data
	    If output sample is less than previous output sample
	       For band = 0 to *totbnd
	          A bilinear smoothing is performed on reflectence data
		  Reflectence data is written to output buffer
   Reset the first line read of the next block
   Close the output image and free the buffer space
Close the input images and free the buffer space
Return

ALGORITHM REFERENCES:     
The main processing algorithm was originally taken from:
  B. Fishman and B. Schachter, "Computer Display of Height Fields," 
  Computers and Graphics, Vol.  5, pp. 53-60, 1980.

*******************************************************************************/
#include <math.h>

#include "las.h"
#include "util.h"
#include "pixman.h"
#include "labserv.h"
#include "protos.h"

#define RAD 0.01745329  	/* constant to turn degrees into radians */

extern long curent;		/* current line				*/
extern long curcat;		/* currnet band				*/

void procesl
(
    struct PARBLK *vblk,	/* I: TAE parameter block		      */
    char *hosin1,		/* I: host name of elevation image	      */
    char *hosin2,		/* I: host names of reflectence images	      */
    char *tempout,		/* I: temporary output name		      */
    long *window1,		/* I: window of elevation image		      */
    long *bands1,		/* I: band specification of first input image */
    long *window2,		/* I: window of reflectance image	      */
    long *bands2,		/* I: band specification of second input image*/
    long *totbnd,		/* I: number of bands in second input image   */
    long *out_wind,		/* I: output window specification	      */
    float *pixsiz,		/* I: pixel size			      */
    long *viewloc,		/* I: viewer location			      */
    float *viewdir,		/* I/O: viewer direction		      */
    float *foview,		/* I: field of view			      */
    float *vertscl,		/* I: vertical scaling factor		      */
    long *horizn,		/* I: height of horizon 		      */
    long *skyval,		/* I: value of the sky line		      */
    long *odtype,		/* I: output data type			      */
    long *msgtim		/* I: time between messages		      */
)


{  /*  procesl */

double cosgam;			/* cosine of gamma			*/
double dx;			/* delta x				*/
double dy;			/* delta y				*/
double fov;			/* corrected field of view		*/
double fovt;			/* corrected field of view		*/
double gamma;			/* atan of current position		*/
double range;	
double x;			/* the x direction			*/
double y;			/* the y direction			*/

float dh[4];			/* height of pixel			*/
float dh12;			/* temp. height pixel			*/
float dh34;			/* temp. height pixel			*/
float dn12;			/* temp. reflectence pixel		*/
float dn34;			/* temp. reflectence pixel		*/
float factx;			/* factor in x direction		*/
float facty;			/* factor in y direction		*/
float rld2;			/* line dimension/ 2			*/
float tanb;			/* tangent angle			*/
float tang;			/* temporary angle			*/

long acc = IREAD;		/* access code				*/
long band;			/* curent band				*/
long bands[4];			/* bands to be written to		*/
long band_pos;			/* starting position of band in buffer	*/
long bnd = 0;			/* total bands in input image		*/
extern long buffersiz;		/* size of the data buffer		*/
long buf_nl;			/* number of lines in the buffer	*/
long dn[4];			/* reflectence of pixel			*/
long dtype = EWORD;		/* data type of elevation data		*/
long fatal = -1;		/* fatal error code			*/
long first;			/* if first block			*/
long i;				/* counter -- current band		*/
long idh;			/*  elevation pixel value		*/
long ix;			/* integer of the x value		*/
long iy;			/* integer of the y value		*/
long irow;			/* sample position of output value	*/
long iy_first;			/* first flag of iy 			*/
long iy_first2;			/* first flag of iy 			*/
long iy_last;			/* last flag of iy 			*/
long iy_last2;			/* last flag of iy 			*/
long iy_pos1;			/* first y postion in buffer		*/
long iy_pos2;			/* second y postion in buffer		*/
long ld2;			/* integer value of no. of lines / 2	*/
long line;			/* counter -- curent line		*/
long mirow;			/* minimum row				*/
long mode = 0;			/* mode of status check			*/
long *mrow;			/* marked row that was last modified	*/
long nblock;			/* number of blocks			*/
long nnl;			/* number of lines in temporary file	*/
long nnnl;			/* larger of nnl or ns			*/
long nnnl_m_horizn;		/* nnl minus horizn value		*/
long nonfatal = 1;		/* nonfatal error code			*/
long npix;			/* number of pixels			*/
long ns;			/* number of samples			*/
long one = 1;			/* value of one				*/
static	long	opt[3]={0,0,0}; /* optional parameter array for EOPENR  */
long scaliy;			/* scaling of iy for viewing angle	*/
long sl;			/* starting line			*/
long ss;			/* starting sample			*/
long status;			/* return value of function calls	*/
long temp;			/* position of veiwer location/full image */
long temp_nl;			/* temporary number of lines		*/
long vz;			/* hieght of viewer			*/
long winldiff;			/* difference of lines between WINDOW1 and
				   WINDOW2				*/
long winsdiff;			/* difference of samples between WINDOW1 and
				   WINDOW2				*/

int fdesc1;			/*  file descriptor of elevation data	*/
int fdesc2;			/*  file descriptor of reflectence data */
int fdesc3;			/*  file descriptor of output data	*/

register short *dembuf;		/*  buffer of elevation data		*/
long bkgrnd;			/* backround value			*/
long *buffer;			/* output data buffer			*/
long idn;			/* output pixel value			*/
long *imgbuf;			/* image buffer				*/

char *c_ealloc();		/*  declaration of c_ealloc call	*/

/*  Calculate the difference between WINDOW1 and WINDOW2
--------------------------------------------------------*/
winldiff = window2[0] - window1[0] + 1;
winsdiff = window2[1] - window1[1] + 1;

/*  Calculate the number of lines in the buffer
-----------------------------------------------*/
buf_nl = buffersiz / window1[3];
if (buf_nl > window1[2])
   buf_nl = window1[2];

/*  Set the number of lines and samples in the temporary image
--------------------------------------------------------------*/
ns = out_wind[0];
nnl = out_wind[1];

/*  Allocate space for the buffers
----------------------------------*/
dembuf = (short *)malloc((buf_nl * window1[3] * 2) + 
			 (*totbnd * buf_nl * window2[3] * 4) +
			 (nnl * 4));
if (dembuf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","perspec-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

imgbuf = (long *) dembuf + (buf_nl * window1[3]);
mrow = imgbuf + (*totbnd * buf_nl * window2[3]);

buffer = (long *)malloc(*totbnd * ns * 4);
if (buffer == NULL)
   {
   c_errmsg("Error allocating dynamic memory","perspec-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);
   }

/*  Create an arrary of bands for the subsequent output open calls
------------------------------------------------------------------*/
for (i = 0; i < *totbnd; i++)
   bands[i] = i + 1;
bands[*totbnd] = 0;

/*  Open the input images
-------------------------*/
status = c_eopenr((struct FDESC **)&fdesc1,hosin1,&one,&window1[0],&window1[1],&window1[2],
		  &window1[3],&acc,&dtype,opt,&buf_nl);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);

dtype = ELONG;
status = c_eopenr((struct FDESC **)&fdesc2,hosin2,&bnd,&window2[0],&window2[1],&window2[2],
	          &window2[3],&acc,&dtype, opt,&buf_nl);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","perspec-fatal",&fatal);

/*  Remove the temporary file if it already exists
--------------------------------------------------*/
mode = 0;
status = c_lsstat(tempout,&mode);
if (status == E_SUCC)
   c_delfil(tempout);

/*  Calculate the iy scaling factor depending on the viewing angle -- the 
    angle will determine the direction that the input image is read
-------------------------------------------------------------------------*/
if (*viewdir > 180)
   tang = 360 - *viewdir;
else 
   tang = *viewdir;
scaliy = (long)((float)buf_nl - ((tang / 180) * (float)buf_nl));

/*  Initialize constants that will be needed for the perspec calculations
-------------------------------------------------------------------------*/
*viewdir = 270. - *viewdir;
if (*viewdir < 0) 
   *viewdir = *viewdir + 360.;

fovt = (double) (*foview * RAD);
fov = tan(fovt);
ld2 =  nnl/2;
rld2 = nnl/2.0;
*viewdir = *viewdir * RAD;
if ( nnl < ns)
   nnnl = ns;
else
   nnnl = nnl;
nnnl_m_horizn = nnnl - *horizn;

/*  Calculate the height of the viewing location by adding the input Z value
    to the elevation of the viewing location.  The viewing location is
    relative to the full image, not just the window.
----------------------------------------------------------------------------*/
if (((viewloc[0] - window1[0] + 1) > 0) && 
    ((viewloc[1] - window1[1] + 1) > 0) && 
    ((viewloc[0] - window1[0] + 1) <= window1[2]) &&
    ((viewloc[1] - window1[1] + 1) <= window1[3]))
   {
   temp = viewloc[0] - window1[0] + 1;
   status = c_eread((struct FDESC **)&fdesc1,&bands1[0],&temp,
                    (unsigned char *)dembuf,&one);
   if (status == E_FAIL)
      c_clean(vblk,"perspec",(char (*)[256])tempout,&one);
   vz = *(dembuf + viewloc[1] - window1[1]) + viewloc[2];
   }
else
   vz = viewloc[2];

/*  Set the first input line and calculate the number of blocks the image
    will be split into in order to proces effeciently
-------------------------------------------------------------------------*/
iy_first = (viewloc[0] - window1[0] + 1) - scaliy;
iy_first2 = (viewloc[0] - window2[0] + winldiff) - scaliy;
first = TRUE;
if (tang < 90)
   nblock = (long) ceil((double)(iy_first) / (double)(scaliy)) + 1;
else
   nblock = (long) ceil((double)(window1[2] - iy_first) / (double)buf_nl);

/*  Initialize the processing message
-------------------------------------*/
mode = ONECAT;
curcat = 1;
curent = 1;
ss = 1;
sl = 1;
c_inittm("PERSPEC","Processed","line","block"," ",&nnl,&nblock,msgtim,&mode);

c_startm();

/*  Processing loop for each block
----------------------------------*/
for (curcat = 1; curcat < nblock + 1; curcat++)
   {
   curent = 1;

   /*  Open a temporary file -- this file will be rotated 90 degrees for the
       final output image -- open for update after first block is processed
   -------------------------------------------------------------------------*/
   if (first)
      acc = IWRITE;
   else
      acc = IUPDATE;

   status = c_eopenr((struct FDESC **)&fdesc3,tempout,totbnd,&sl,&ss,&nnl,&ns,&acc,
	             odtype,opt,&one);
   if (status != E_SUCC)
      {
      if (c_eclose((struct FDESC **)&fdesc3) == E_FAIL)
         c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
      c_clean(vblk,"perspec",(char (*)[256])tempout,&one);
      }
   
   /*  Check to make sure that the lines to be read are not outside the input
       images
   --------------------------------------------------------------------------*/
   if (iy_first < 1)
      iy_first = 1;
   else if (iy_first > window1[2])
      iy_first = window1[2];

   if (iy_first2 < 1)
      iy_first2 = 1;
   else if (iy_first2 > window2[2])
      iy_first2 = window2[2];

   if (iy_first + buf_nl - 1 > window1[2])
      temp_nl = window1[2] - iy_first + 1;
   else
      temp_nl = buf_nl;
   
   /* Set the last line read 
      Read the input images
   -------------------------*/
   iy_last = iy_first + temp_nl - 1; 
   if (iy_last > window1[2])
      iy_last = window1[2];

   iy_last2 = iy_first2 + temp_nl - 1; 
   if (iy_last2 > window2[2])
      iy_last2 = window2[2];

   status = c_eread((struct FDESC **)&fdesc1,&bands1[0],&iy_first,
                    (unsigned char *)dembuf,&temp_nl);
   if (status == E_FAIL)
      {
      if (c_eclose((struct FDESC **)&fdesc3) == E_FAIL)
         c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
      c_clean(vblk,"perspec",(char (*)[256])tempout,&one);
      }

   for ( band = 0; band < *totbnd; band++)
      {
      status = c_eread((struct FDESC **)&fdesc2,&bands2[band],&iy_first2,
	               (unsigned char *)imgbuf + (band * window2[3] * buf_nl),
                        &temp_nl);
      if (status == E_FAIL)
         {
         if (c_eclose((struct FDESC **)&fdesc3) == E_FAIL)
            c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
         c_clean(vblk,"perspec",(char (*)[256])tempout,&one);
         }
      }

   /*  Processing loop for each line of the output image
   -----------------------------------------------------*/
   for (line = 0; line < nnl; line++)
      {  
      if (first)
         {
         for (band = 0; band < *totbnd; band++)
            {
            /*  Fill the output buffer with the fill value specified
            --------------------------------------------------------*/
            bkgrnd = (long) skyval[band];
            c_pxfill((unsigned char *)(buffer + (ns * band)),&dtype,&ns,
	             (unsigned char *)&bkgrnd);
            }
         }
      else
         {
         for (band = 1; band <= *totbnd; band++)
            {
            status = c_eread((struct FDESC **)&fdesc3,&band,&curent,
	                    (unsigned char *)(buffer + ((band - 1) * ns)),&one);
            if (status == E_FAIL)
   	       {
   	       if (c_eclose((struct FDESC **)&fdesc3) == E_FAIL)
      	          c_errmsg("Nonfatal error encountered","perspec-warn",
			   &nonfatal);
               c_clean(vblk,"perspec",(char (*)[256])tempout,&one);
   	       }
	    }
         }

      /*  Calculate the initial values
      --------------------------------*/
      gamma = atan((curent - ld2) * fov / rld2);
      dx = -cos(*viewdir + gamma);
      dy = sin(*viewdir + gamma);
      cosgam = cos(gamma) * *pixsiz;
      if (first)
         *(mrow + line) = ns;
      range = cosgam;
      x = (double) viewloc[1];
      y = (double) viewloc[0];

      while ((range <= (float)ns * *pixsiz) && (*(mrow + line) > 1))
         {
         ix = (long)(x);
         iy = (long)(y);
         if (((ix  - window1[1] + 1) >= 1) && 
	     ((ix - window1[1] + 1) < window1[3]) && 
	     ((iy - window1[0] + 1) >= iy_first) && 
	     ((iy - window1[0] + 1 ) < iy_last))
	    {  /*  if within range of image  */
            iy_pos1 = ((iy - window1[0] + 1) - iy_first) * window1[3];
            iy_pos2 = iy_pos1 + window1[3];
	    dh[0] = *(dembuf + iy_pos1 + (ix - window1[1] + 1) - 1);
	    dh[1] = *(dembuf + iy_pos1 + (ix - window1[1] + 1));
	    dh[2] = *(dembuf + iy_pos2 + (ix - window1[1] + 1) - 1);
	    dh[3] = *(dembuf + iy_pos2 + (ix - window1[1] + 1));
	    factx = x - ix;
	    facty = y - iy;

	    /*  a bilinear smoothing is performed on the elevation data
            -----------------------------------------------------------*/
	    dh12 = dh[0] + (dh[1] - dh[0]) * factx;
	    dh34 = dh[2] + (dh[3] - dh[2]) * factx;
	    idh = (long)(dh12 + (dh34 - dh12) * facty + .5);
/*
	    if ( idh != 0)
	       {
*/
	       tanb = (*vertscl * (float)(idh - vz)) / range;
	      mirow = (nnnl_m_horizn - floor((tanb * rld2 / fov) + .5) < nnnl) ?
	               nnnl_m_horizn - floor((tanb * rld2 / fov) + .5) : nnnl;
	       irow = mirow > 1 ? mirow : 1;
/*
	       }
	    else
	       irow = *(mrow + line);
*/

	    if ( irow < *(mrow + line))
	       {
	       for (band = 0; band < *totbnd; band++)
	          {
	          band_pos = band * window2[3] * buf_nl;
	          iy_pos1 = ((iy - window2[0] + winldiff) - iy_first2) * 
			      window2[3];
 	          iy_pos2 = iy_pos1 + window2[3];
	          dn[0] = *(imgbuf + band_pos + iy_pos1 + 
				    ((ix - window2[1] + winsdiff) - 1));
	          dn[1] = *(imgbuf + band_pos + iy_pos1 + 
				    (ix - window2[1] + winsdiff)); 
	          dn[2] = *(imgbuf + band_pos + iy_pos2 + 
				    ((ix  - window2[1] + winsdiff) - 1)); 
	          dn[3] = *(imgbuf + band_pos + iy_pos2 + 
				    (ix - window2[1] + winsdiff)); 

	          /*  a bilinear smoothing is performed on reflectence data
                  ---------------------------------------------------------*/
	          dn12 = (float)dn[0] + (float)(dn[1] - dn[0]) * factx;
	          dn34 = (float)dn[2] + (float)(dn[3] - dn[2]) * factx;
	          idn = (long)(dn12 + (dn34 - dn12) * facty + .5);
	          npix = *(mrow + line) - irow + 1;
	          c_pxfill((unsigned char *)(buffer + (ns * band) + irow - 1),
			   &dtype,&npix,(unsigned char *)&idn);
	          }  /*  for band < *totbnd  */
	       *(mrow + line) = irow - 1;
	       }  /* if irow < *(mrow + line) */
	    }  /*  if within range of image  */
         x += dx;
         y += dy;
         range += cosgam;
         }  /*  while (range <= nw * *pixsiz) && *(mrow + line) > 1) */

      for (i = 1; i <= *totbnd; i++)
         {
         status = c_ewrite((struct FDESC **)&fdesc3,&i,&curent,
                           (unsigned char *)(buffer + (ns * (i - 1))),&one);
         if (status == E_FAIL)
   	    {
   	    if (c_eclose((struct FDESC **)&fdesc3) == E_FAIL)
      	       c_errmsg("Nonfatal error encountered","perspec-warn",&nonfatal);
            c_clean(vblk,"perspec",(char (*)[256])tempout,&one);
   	    }
         }
      curent ++;
      }  /*  for line < nnl  */

   first = FALSE;

   /*  Reset the first line read of the next block
   -----------------------------------------------*/
   if (tang < 90)
      iy_first -= scaliy;
   else
      iy_first += temp_nl - 1;

   if (tang < 90)
      iy_first2 -= scaliy;
   else
      iy_first2 += temp_nl - 1;

   /*  Close the output image
   --------------------------*/
   status = c_eclose((struct FDESC **)&fdesc3);
   }  /*  for curcat < nblock + 1  */

c_stoptm();

/*  Close the input images and free the buffer space
----------------------------------------------------*/
status = c_eclose((struct FDESC **)&fdesc1);
status = c_eclose((struct FDESC **)&fdesc2);

free(dembuf);
free(buffer);

return;
}  /*  procesl  */
