/* ---------------------------------------------------------------------------
NAME: INITIO

FUNCTION:   Opens input and output images

PROGRAM HISTORY:
  Version   Date       Author           Request
  -------   ----       ------           -------
    1.0       06/27/86     J. Stansbery      
    2.0       08/1/89      B. Allam     conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
    Runs under TAE

PROJECT:    LAS

ALGORITHM:
 
   Open the 1st input image.
   If the subcommand is MODIFY, 
      Open the second input image.
   Open the output file.
   Update DDR.
----------------------------------------------------------------------------- */
#include "picshade.h"

void initio
(
    char hosin[][CMLEN],        /* Host name for the elevation image */
    char hosin2[][CMLEN],    /* Host name for the composite image */
    char hosout[],       /* Host name for the output image */
    long num_inimg,      /* Number of input images from IN(1) */
    long num_inimg2,     /* Number of input images from IN(2) */
    long wind[][4],      /* Window specification for the 1st image */
    long wind2[][4],     /* Window specification for the 2nd image */
    long totbnd,         /* Total number of bands in the 1st image */
    long totbnd2,        /* Total number of bands in the 2nd image */
    long otype,          /* Output data type */
    char subcmd[]       /* Subcommand */
)
{

long    zero = 0;
long    one = 1;

double  x_inc,          /* Sample increments */
y_inc;      /* Line increments */

long    access,     /* File access type */
i, n,       /* Loop counter */
opt[2],     /* Optional parameter array  */
status,     /* Return status */
type;       /* Data type */

/* Open the 1st input image.
------------------------- */
type = EREAL;
access = IREAD;
status = c_eopenr(&fdesc[0], hosin[0], &bands_per_img[0], &wind[0][SL],
&wind[0][SS], &wind[0][NL], &wind[0][NS], &access,
&type, &zero, &one);
if (status != E_SUCC)
    msg_err ("!Error opening first input image", "picshade_input");
fdesc[1] = 0;

/* If the subcommand is MODIFY, open the second input image.
---------------------------------------------------------- */
if (!strcmp(subcmd, "MODIFY"))
    {
    for (n = 0; n < num_inimg2 ; n++)
        {
        status = c_eopenr(&fdesc2[n], hosin2[n], &bands_per_img2[n],
        &wind2[n][SL], &wind2[n][SS], &wind2[n][NL],
        &wind2[n][NS], &access, &type, &zero, &one);
        if (status != E_SUCC)
            msg_err ("!Error opening second input image", "picshade_input");
        }
    fdesc2[++n] = 0;
    }

/* Open the output file.
----------------------- */
access = IWRITE;
opt[0] = otype;
opt[1] = 0;
if (!strcmp(subcmd, "MODIFY"))
    {
    status = c_eopenr (&fdescout, hosout, &totbnd2, &one, &one, &wind2[0][NL],
    &wind2[0][NS], &access, &type, opt, &one);
    if (status != E_SUCC)
        msg_err("!Error opening output image","picshade_output");
    obands = totbnd2;
    for (i = 0; i < totbnd2; i++)
        out_img_bands[i] = i + 1;
    }
else
    {
    status = c_eopenr (&fdescout, hosout, &totbnd, &one, &one, &wind[0][NL],
    &wind[0][NS], &access, &type, opt, &one);
    
    if (status != E_SUCC)
        msg_err("!Error opening output image","picshade_output");
    obands = totbnd;
    for (i = 0; i < totbnd; i++)
        out_img_bands[i] = i + 1;
    }
strcpy(out_files[num_files++], hosout);

/* Update DDR.
------------ */
y_inc = 1.0;
x_inc = 1.0;
if (!strcmp(subcmd, "MODIFY"))
    c_upddr(&block, hosin2, hosout, &num_inimg2, wind2, bands2, bands_per_img2,
    &y_inc, &x_inc, &totbnd2, &zero);
else
    c_upddr(&block, hosin, hosout, &num_inimg, wind, bands, bands_per_img,
    &y_inc, &x_inc, &totbnd, &zero);

}

