/*************************************************************************
   NAME:                        GETIM2


   PURPOSE:  GETIM2 gets pixel values and creates histogram of byte,
             ints, and longs in integer range data.

**************************************************************************/
#include <string.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include  "pixman.h"
#include "pixcount.h"


int getim2 (
   long         bndnum,        /*  I: current band number                */
   long         nl,            /*  I: number of lines                    */
   long         ns,            /*  I: number of samples per line         */
   long long    *hist2,        /*  O: histogram array                    */
   char         *subcmd,       /*  I: specified subcommand               */
   long         *mask,         /*  I: mask value                         */
   long         *mskcnt,       /*  O: pixels not in histogram count      */
   long         msgtim,        /*  I: elapsed time between messages      */
   long         totbnd,        /*  I: total number of output bands       */
   long         totcnt,        /*  I: total number of mask values * 2    */
   long         midpt,         /*  I: middle point in histogram array    */
   struct FDESC **fdesc,       /*  I: description of file                */
   long         maxval,        /*  I: maximum pixel value                */ 
   long         minval,        /*  I: minimum pixel value                */
   long long    *maxpixelcount /*  O: the largest total number of pixels */
   )

   {
   extern long curent;         /* current entity being processed         */

   long         flag;          /* flag for functions calls               */
   long         gg;            /* loop counter                           */
   long         i;             /* loop counter                           */
   long         line;          /* current line to be read                */
   long long    maxpixelcount_temp;/* temporary variable used for looping*/
   long         mxns;          /* maximum number of sample read          */
   long         nblk;          /* number of blocks to read               */
   long         nlines;        /* number of lines to read                */
   long         one = 1;       /* constant one                           */
   long         status;        /* function return status code            */

   short        *inbuf = NULL; /* input pixel buffer                     */

   /*  initialize histogram array to all zeroes
   ----------------------------------------------------*/
   memset(hist2,0,MAXBUF*sizeof(long long));

   nlines = MAX_LINES_IN_BLOCK;
   nblk = nl/nlines;
   mxns = nlines * ns;
   curent = 0;

   /*  initialize and start the diplay message utility
   ----------------------------------------------------*/
   line = 1;
   flag = ONECAT;
   c_inittm("PIXCOUNT", "Histogrammed", "line", "band", " ", &nl, &totbnd,
               &msgtim, &flag);

   c_startm();   

   /*  dynamically allocate the array inbuf
   ----------------------------------------------------*/
   inbuf = (short *)malloc(nlines*sizeof(short)*ns);
   if(inbuf == NULL)
      {
      c_errmsg("Error allocating memory","pixcount-alloc",&one);
      return E_FAIL;
      }

   /*  read in and histogram the input lines
   ----------------------------------------------------*/
   for(i = 0; i < nblk; i++)
      {
      status = c_eread(fdesc,&bndnum,&line,(unsigned char *)inbuf,&nlines);
      if (status != mxns)
         {
         free(inbuf);
         c_stoptm();
         return E_FAIL;
         }

      for(gg = 0; gg < mxns; gg++)
         {
         hist2[inbuf[gg]+midpt] ++;
         }
      line += nlines;
      curent = line; 
      }

   /*  process the last block of data
   ----------------------------------------------------*/
   if ((nblk * nlines) < nl)
      {
      nlines = nl - line + 1;
      if (nblk == 0) nlines = nl;
   
      mxns = nlines * ns;
      status = c_eread(fdesc,&bndnum,&line,(unsigned char *)inbuf,&nlines);
      if (status != mxns)
         {
         free(inbuf);
         c_stoptm();
         return E_FAIL;
         }
      for(gg = 0; gg < mxns; gg++)
         {
         hist2[inbuf[gg]+midpt] ++;
         }
      curent = nl;
      }
   line = nl;

   /*  disable the display message utility
   ----------------------------------------------------*/
   c_stoptm();
   
   /*  delete mask value from histogram array
   ----------------------------------------------------*/
   if (strcmp(subcmd, "BCKGRD") == 0)
      {
      *mskcnt = 0;
      for(i = 0; i < totcnt; i += 2)
         for(gg = (mask[i] + midpt); gg <= (mask[i+1] + midpt); gg++)
            {
            *mskcnt += hist2[gg];
            hist2[gg] = 0;
            }
      }

   /*   find the largest number of the unique pixel value
   --------------------------------------------------------*/
   maxpixelcount_temp = hist2[minval + midpt];
   for(i = (minval + midpt + 1); i <= (maxval + midpt); i++)
      {
      if (maxpixelcount_temp < hist2[i])
         maxpixelcount_temp = hist2[i];
      }  
   *maxpixelcount = maxpixelcount_temp;

   /*  free up dynamically allocated memory
   --------------------------------------------------------*/
   free(inbuf);
   return E_SUCC;
   }
