/*************************************************************************
   NAME:                        GETIM4


   PURPOSE:  GETIM4 gets pixel values and creates histogram of integer*4
               (not in integer*2 range) and real*4 data.

**************************************************************************/
#include <string.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "pixcount.h"


int getim4 (
   double histr[2][MAXBIN],    /*  O: R*4 histogram array                */
   long   *icount,             /*  O: number of different pixel values   */
   long   *window,             /*  I: current window specification       */
   long   dtype,               /*  I: data type                          */
   char   *subcmd,             /*  I: specified subcommand               */
   float  *maskr,              /*  I: R*4 mask value                     */
   long   *mskcnt,             /*  O: pixels not in histogram count      */
   long   bndnum,              /*  I: current band number                */
   long   totbnd,              /*  I: total number of output bands       */
   long   msgtim,              /*  I: time between display messages      */
   float  minv,                /*  I: minimum value                      */
   float  maxv,                /*  I: maximum value                      */
   float  rrange,              /*  I: range of values                    */
   long   totcnt,              /*  I: total number of mask values * 2    */
   struct FDESC **fdesc,       /*  I: description of file                */
   long long *maxpixelcount    /*  O: the largest total number of pixels
                                      with a unique pixel value          */
   )

   {

   extern long curent;         /*  current entity being processed        */

   float  *bufr = NULL;        /*  R*4 input pixel buffer                */
   double  rsave1;             /*  variable for minimum value            */
   double  rsave2;             /*  variable for maximum value            */

   long *bufi = NULL;          /*  I*4 input pixel buffer                */
   long   index;               /*  loop index                            */
   long   flag;                /*  flag for function                     */
   long   i;                   /*  loop index                            */
   long   j;                   /*  loop index                            */
   long   line;                /*  line number being read                */
   long long maxpixelcount_temp; /*  temporary variable used for looping */
   long   mxns;                /*  maximum number of samples per read    */
   long   nblk;                /*  number of blocks to read              */
   long   nl;                  /*  number of lines in the image          */
   long   nlines;              /*  number of lines in each read          */
   long   ns;                  /*  number of samples in the image        */
   long   one = 1;             /*  constant one                          */
   long   outpix;              /*  array value where real data is        */
   long   save1;               /*  minimum value for i*4 data            */
   long   save2;               /*  maximum value for i*4 data            */
   long   status;              /*  function return status code           */


   /*    initialize histogram arrays
   ------------------------------------------------------*/
   for(i = 0; i < 2; i++) 
      for(j = 0; j < MAXBIN; j++)
         histr[i][j] = 0;

   *icount = 0;
   nl = window[2];
   ns = window[3];
   nlines = MAX_LINES_IN_BLOCK;
   nblk = nl/nlines;
   mxns = nlines * ns;
   curent = 0;

   /*    initialize and start the diplay message utility
   ------------------------------------------------------*/
   line = 1;
   flag = ONECAT;
   c_inittm("PIXCOUNT", "Histogrammed", "line", "band", " ", &window[2],
              &totbnd, &msgtim, &flag);

   c_startm();  


   if (dtype == ELONG)
      {
      /***************Data type is INTEGER*4**************/

      /*  dynamically allocate the array bufi
      --------------------------------------------------*/
      bufi = (long*)malloc(ELONGSIZE*ns);
      if(bufi == NULL)
         {
         c_errmsg("Error allocating memory","pixcount-alloc",&one);
         return E_FAIL;
         }

      /*  read in, convert, and histogram the input lines
      --------------------------------------------------*/
      for(i = 0; i < nblk; i++)
         {
         status = c_eread(fdesc,&bndnum,&line,(unsigned char *)bufi,&nlines);
         if (status != mxns)
            {
            free(bufi);
            c_stoptm();
            return E_FAIL;
            }

         for(j = 0; j < mxns; j++)
            {
            outpix = (long)(((float)bufi[j] - minv) / rrange * MAXBIN);
            if (outpix < 0) outpix = 0;
            if (outpix > MAXBIN-1) outpix = MAXBIN-1;
            histr[1][outpix] ++;
            /*-----------------------------------------------------
               One is added to value because a zero value indicates
               an empty bin (not used).  Find the minamum pixel value 
               of a range to store with one added for testing for 
               empty bins indicated with a zero.
            ------------------------------------------------------*/
            if((histr[0][outpix] == 0) || (histr[0][outpix] > (bufi[j] + 1)))
               histr[0][outpix] = bufi[j] + 1; 
            }
         line += nlines;
         curent = curent + nlines;
         }

      /*  proces the last block of data
      ----------------------------------------------------------*/
      if ((nblk * nlines) < nl)
         {
         nlines = nl - line + 1;
         if (nblk == 0) nlines = nl;
         mxns = nlines * ns;
         status = c_eread(fdesc,&bndnum,&line,(unsigned char *)bufi,&nlines);
         if (status != mxns)
            {
            free(bufi);
            c_stoptm();
            return E_FAIL;
            }

         for(j = 0; j < mxns; j++)
            {
            outpix = (long)(((float)bufi[j] - minv) / rrange * MAXBIN);
            if (outpix < 0) outpix = 0;
            if (outpix > MAXBIN-1) outpix = MAXBIN-1;
            histr[1][outpix] ++;
            /*-----------------------------------------------------
               One is added to value because a zero value indicates
               an empty bin (not used).
            ------------------------------------------------------*/
            if((histr[0][outpix] == 0) || (histr[0][outpix] > (bufi[j] + 1)))
               histr[0][outpix] = bufi[j] + 1; 
            }
        curent = nl;
        }

      /*  free up dynamically allocated memory
      --------------------------------------------------------*/
      free(bufi);
      }
   else
      { 
      /***************Data type is REAL*4**************/

      /*  dynamically allocate the array inbuf
      ----------------------------------------------------*/
      bufr  = (float*)malloc(nlines*EREALSIZE*ns);

      if(bufr == NULL)
         {
         c_errmsg("Error allocating memory","pixcount-alloc",&one);
         return E_FAIL;
         }

      /*   read in and histogram the input lines
      -------------------------------------------------------------*/
      for(i = 0; i < nblk; i++)
         {
         status = c_eread(fdesc,&bndnum,&line,(unsigned char *)bufr,&nlines);
         if (status != mxns)
            {
            free(bufi);
            c_stoptm();
            return E_FAIL;
            }

         for(j = 0; j < mxns; j++)
            {
            outpix = (long)((bufr[j] - minv) / rrange * MAXBIN);
            if (outpix < 0) outpix = 0;
            if (outpix > MAXBIN-1) outpix = MAXBIN-1;
            histr[1][outpix] ++;
            /*-----------------------------------------------------
               One is added to value because a zero value indicates
               an empty bin (not used).
            ------------------------------------------------------*/
            if((histr[0][outpix] == 0) || (histr[0][outpix] > (bufr[j] + 1)))
               histr[0][outpix] = bufr[j] + 1; 
            }
         line = line + nlines;
         curent = curent + nlines;
         }
      /*   proces the last block of data
      -------------------------------------------------------------*/
      if ((nblk * nlines) < nl)
         {
         nlines = nl - line + 1;
         if (nblk == 0) nlines = nl;
         mxns = nlines * ns;
         status = c_eread(fdesc,&bndnum,&line,(unsigned char *)bufr,&nlines);
         if (status != mxns)
            {
            free(bufi);
            c_stoptm();
            return E_FAIL;
            }

         for(j = 0; j < mxns; j++)
            {
            outpix = (long)((bufr[j] - minv) / rrange * MAXBIN);
            if (outpix < 0) outpix = 0;
            if (outpix > MAXBIN-1) outpix = MAXBIN-1;
            histr[1][outpix] ++;
            /*-----------------------------------------------------
               One is added to value because a zero value indicates
               an empty bin (not used).
            ------------------------------------------------------*/
            if((histr[0][outpix] == 0) || (histr[0][outpix] > (bufr[j] + 1)))
               histr[0][outpix] = bufr[j] + 1; 
            }
         }
      curent = nl;
 
      /*  free up dynamically allocated memory
      --------------------------------------------------------*/
      free(bufr);
      }
   line = nl;

   /*   disable the display message utility
   -------------------------------------------------------------*/
   c_stoptm();

   /*   delete mask value from histogram array
   -------------------------------------------------------------*/
   if (strcmp(subcmd, "BCKGRD") == 0)
      {
      *mskcnt = 0;
      for(i = 0; i < totcnt; i += 2)
         {
         rsave1 = (maskr[i] - minv) / rrange * MAXBIN;
         rsave2 = (maskr[i+1] - minv) / rrange * MAXBIN;
         save1 = (long)rsave1;
         save2 = (long)rsave2;
         if (save1 < 0) save1 = 0;
         if (save2 < 0) save2 = 0;
         if (save1 > MAXBIN-1) save1 = MAXBIN-1;
         if (save2 > MAXBIN-1) save2 = MAXBIN-1;
         for(index = save1; index <= save2; index++)     
            {
            *mskcnt += (long)histr[1][index];
            histr[1][index] = 0.0;
            }
         }
      }


   /*   determine the number of bins with values
   -------------------------------------------------------------*/
   for(i = 0; i < MAXBIN; i++)
      {
      if (histr[0][i] != 0.0)
         {
         rsave1 = histr[0][i];
         rsave2 = histr[1][i];
         histr[0][i] = 0;
         histr[1][i] = 0;
         if (dtype == ELONG)
            {
            histr[0][*icount] = (long long)rsave1;
            histr[1][*icount] = (long long)rsave2;
            }
         else
            {
            histr[0][*icount] = rsave1;
            histr[1][*icount] = rsave2;
            }
         *icount = *icount + 1;
         }
      }

   /*   find the largest number of the unique pixel value
   -------------------------------------------------------*/
   maxpixelcount_temp = (long long)histr[1][0];
   for (i = 0; i <= (long)(maxv-minv); i++)
      {
      if (maxpixelcount_temp < (long long)histr[1][i])
         (maxpixelcount_temp = (long long)histr[1][i]);
      }
   *maxpixelcount = maxpixelcount_temp;
   return E_SUCC;
   }
