/***************************************************************************
    NAME:                          GETPAR


    PURPOSE:  GETPAR gets the user-specified parameters and the TAE
              global variables.

***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "pixcount.h"

int getpar (
   struct PARBLK *parblk,      /*  O: Parameter block                    */
   char *subcmd,               /*  O: specified subcommand               */
   long *msgtim,               /*  O: time between display messages      */
   char taein[MAXIMG][CMLEN],  /*  O: TAE names of input images          */
   char hosin[MAXIMG][CMLEN],  /*  O: host names of input images         */
   long window[MAXIMG][4],     /*  O: window specification               */
   long bands[MAXIMG][MAXBND+1], /*  O: band specification               */
   long *nmrimg,               /*  O: number of input images             */
   long *nbands,               /*  O: number of bands in each image      */
   long *totbnd,               /*  O: total number of output bands       */
   char *option,               /*  O: list option                        */
   char *htype,                /*  O: histogram type                     */
   long *nbins,                /*  O: number of specified bins           */
   float *maskr,               /*  O: mask values not in histogram       */
   long *nomask,               /*  O: mask flag                          */
   long *file,                 /*  O: file output flag                   */
   long *lp,                   /*  O: line printer output flag           */
   long *term,                 /*  O: terminal output flag               */
   char *logopt,               /*  O: log print option                   */
   long *sumry,                /*  O: specified summary option           */
   char *prtfil,               /*  O: print file for setprint            */
   char filena[][CMLEN],       /*  O: file name for disk file            */
   long *astrik,               /*  O: number of occurences               */
   long *sigdig,               /*  O: number of significant digits       */
   long *totcnt                /*  O: number of mask values * 2          */
   )

   {
   struct VARIABLE *in;     /*  pointer to TAE parameter block var.      */
                   

   long count;              /*  number of specified parameter values     */
   long maxcnt = MXCNT;     /*  maximum count for mask values            */
   long ddlen = DDLEN;      /*  length of string in mskstr               */
   long j;                  /*  loop counter                             */
   long maximg = MAXIMG;    /*  Maximum number of images                 */
   long one = 1;            /*  constant one                             */
   long qflag[MAXIMG][2];   /*  flag indicating subwindow was specified  */
   long qq;                 /*  loop counter                             */
   long same;               /*  flag for images with same window sizes   */
   long status;             /*  function return status code              */
   long zero = 0;           /*  constant zero                            */

   char maskstr[MXCNT][DDLEN]; /*  array to house mask values            */
   char sumopt[5];          /*  summary option indicator                 */
   char *chptr;             /*  pointer into maskstr                     */

   /*   get global variable
   ---------------------------------------------------------------------*/
   in = p_find(parblk, "$MSGTIM");
   *msgtim = IVAL(*in, 0);

   /*   get subcommand specified
   ---------------------------------------------------------------------*/
   in = p_find(parblk, "_SUBCMD");
   strcpy(subcmd, SVAL(*in,0));

   /*  get input file specification
   ---------------------------------------------------------------------*/
   status = c_getin(parblk, "IN", &zero, taein, hosin, nmrimg, window, bands,
                  nbands, totbnd, &maximg, &same, qflag);
   if (status != E_SUCC)
      return E_FAIL;

   /*   get the print option
   ---------------------------------------------------------------------*/
   strcpy(prtfil, "pixcount");
   status = c_setprt(parblk,term,file,lp,prtfil,filena[0]);
   if (status != E_SUCC)
      return E_FAIL;

   /*   get the list option
   ---------------------------------------------------------------------*/
   in = p_find(parblk, "LISTOPT");
   strcpy(option, SVAL(*in,0));

   /*   get the histogram type
   ---------------------------------------------------------------------*/
   in = p_find(parblk, "HISTTYPE");
   strcpy(htype, SVAL(*in,0));

   /*   get the number of bins
   ---------------------------------------------------------------------*/
   in = p_find(parblk, "NBINS");
   if(in->v_count == 0)
      *nbins = 0;
   else
      *nbins = IVAL(*in,0);

   /*   get the number of occurences for each astrick
   ---------------------------------------------------------------------*/
   in = p_find(parblk, "NELMTS");
   if(in->v_count == 0)
      *astrik = 0;
   else
      *astrik = IVAL(*in,0);

   /*   get the number of significant digits
   ---------------------------------------------------------------------*/
   in = p_find(parblk, "NSIGDIG");
   if(in->v_count == 0)
      *sigdig = 0;
   else
      *sigdig = IVAL(*in,0);

   /*   get the print type option
   ---------------------------------------------------------------------*/
   in = p_find(parblk, "HISTFORM");
   strcpy(logopt, SVAL(*in,0));

   in = p_find(parblk, "LISTFLG");
   strcpy(sumopt, SVAL(*in,0));

   if (strcmp(sumopt, "YES") == 0)
      *sumry = TRUE;
   else
      *sumry = FALSE;

   /*   get the mask value
   ---------------------------------------------------------------------*/
   *nomask = FALSE;

   if (strcmp(subcmd, "BCKGRD") == 0)
      {
      count = c_getstr(parblk, "MASKVAL", &maxcnt, &ddlen, maskstr[0]);
      j = 0;
      for(qq = 0; qq < count; qq++)
         {
         /*   check to see if dash seperator between mask values.
         ---------------------------------------------------------------*/
         chptr = strchr(&maskstr[qq][1], '-');
         if(chptr == NULL)
            {
            status = sscanf(maskstr[qq], "%f", &maskr[j]);
            if (status != 1)
               {
               c_errmsg("Unable to read mask value", "pixcount-mask", &one);
               return E_FAIL;
               } 
            maskr[j + 1] = maskr[j];
            } 
         else
            {
            status = sscanf(maskstr[qq], "%f-%f", &maskr[j], &maskr[j + 1]);
            if (status != 2)
               {
               c_errmsg("Unable to read mask value", "pixcount-mask", &one); 
               return E_FAIL;
               }
            }
         j += 2;
         }
      *totcnt = count * 2;
      }
   else
      *nomask = TRUE;

   return E_SUCC;
   }
