/****************************************************************************
   NAME:			LOGSCALE

   PURPOSE:   LOGSCALE prints the scale for the logrithmic format.

*****************************************************************************/
#include <string.h>
#include <math.h>
#include "las.h"
#include "pixcount.h"

void logscale 
(
   FILE *fp,                   /* I/O: output file pointer                   */
   long long maxpixelcount,    /* I: the largest total number of pixels
                                  with a unique pixel value                  */
   char scale_string[],        /* O: buffer used for scale line output       */
   char scale_string_top[],    /* O: used for scale #s above scale line      */
   char scale_string_btm[],    /* O: used for scale #s below scale line      */
   float *stars_ratio,         /* O: value used to calculate the number of
                                  stars to output to the histogram           */
   int scale_spacing_flag,     /* I: flag indicating scale spacing           */
   long max_nstars_per_page,   /* I: maximum number of stars that will fit
                                  on a page                                  */
   long long numel,            /* I: number of pixels to be histogrammed     */
   char *htype                 /* I: the type of histogram                   */

)
{
   char log_array_string[CMLEN][CMLEN]; /* values of log array as a string   */
   char *scale_string_ptr=NULL;/* pointer used to point to scale_string      */
   char *scale_string_ptr_top=NULL; /* pointer used for scale_string_top     */
   char *scale_string_ptr_btm=NULL; /* pointer used for scale_string_btm     */
   long i;		       /* loop index                                 */
   long index=0;               /* loop index                                 */
   long lenofvalues[CMLEN];    /* length of each value in the log array      */
   long long log_array[CMLEN]; /* array initialized with numbers that
                                  will be output to histogram                */
   float maxpixelcount_output; /* the log of the maxpixelcount               */
   long position = BOTTOM;     /* flag indicating which scale_string
                                  buffer should be written to                */
   long previous_scalestars = 0; /* previous number of stars that was
                                    output to scale                          */
   long scale_string_pos;      /* position of character in the
                                  scale_string buffers                       */
   long scale_string_pos_top;  /* position of character in the
                                    scale_string_top buffer                  */
   long scale_string_pos_btm;  /* position of character in the
                                       scale_string_btm buffer               */
   long long temp_log_number;  /* temporary value used for looping           */
   long total_log_array_values;/* total number of values in log_array        */
   long total_scalestars;      /* total number of stars that will be
                                  output to scale                            */

   /*    Calculate the log of maxpixelcount or numel. Get the ratio of 
         stars which is the total number of stars that will fit on a page 
         to the log of maxpixelcount_output
   ---------------------------------------------------------------------*/
   if (strcmp (htype, "DIST") == 0)
      maxpixelcount_output = log10(maxpixelcount);
   else
      maxpixelcount_output = log10(numel);

   *stars_ratio = (float)max_nstars_per_page/maxpixelcount_output;

   /*    set log_array values
   -------------------------*/
   temp_log_number = 10;
   while (temp_log_number < maxpixelcount)
   {
      log_array[index] = temp_log_number;
      temp_log_number = 10 * log_array[index];
      index ++;
   }
   total_log_array_values = index;

   /*    add maxpixelcount or numel to log_array
   ---------------------------------------------*/
   if (strcmp(htype,"DIST") == 0)
      log_array[index] = maxpixelcount;
   else
      log_array[index] = numel;      

   /*    get length of each log array value
   ---------------------------------------*/
   for (index = 0; index <= total_log_array_values; index++)
   {
      sprintf(log_array_string[index], "%lld", log_array[index]);
      lenofvalues[index] = (long)strlen(log_array_string[index]);
   }

   /*    set string positions
   -------------------------*/
   if (scale_spacing_flag == 0)
   {
      scale_string_pos = 40;
      scale_string_pos_top=42;
      scale_string_pos_btm=42;
   }
   else
   {
      scale_string_pos = 52;
      scale_string_pos_top=54;
      scale_string_pos_btm=54;
   }

   /*    this functions, logpbin2 and logpbin4, can be called twice
         within pixcount so set the strings to empty
   -----------------------------------------------------------------*/
   strcpy(scale_string, " ");
   strcpy(scale_string_top, " ");
   strcpy(scale_string_btm, " ");

   /*    set pointers
   -------------------*/
   scale_string_ptr = scale_string;
   scale_string_ptr_top = scale_string_top;
   scale_string_ptr_btm = scale_string_btm;


   /*    begin the scale_string
   ---------------------------*/
   if (scale_spacing_flag == 0)
   {
      sprintf(scale_string, "%38s |", " ");

      /*    add extra spaces here so the output scale numbers will be more
      centered with the scale line
      ----------------------------------------------------------------------*/
      sprintf(scale_string_top, "%38s 0  ", " ");
      sprintf(scale_string_btm, "%42s", " ");
   }
   else
   {
      sprintf(scale_string, "%50s |", " ");

      /*    add extra spaces here so the output scale numbers will be more
      centered with the scale line
      ----------------------------------------------------------------------*/
      sprintf(scale_string_top, "%50s 0  ", " ");
      sprintf(scale_string_btm, "%54s", " ");
   }

   /*    loop on each value in log_array
   -------------------------------------*/
   for (index=0; index <= total_log_array_values; index++)
   {
      /* add .5 to round up */
      total_scalestars = (long)((log10(log_array[index]) * *stars_ratio+.5));  

      /* add spaces and '-' to buffers  */
      /* loop on minus one because '|' will be used instead */
      for (i=0; i < (total_scalestars - previous_scalestars-1); i++)
      {
         sprintf(scale_string_ptr + scale_string_pos, "-");
         scale_string_pos ++;

         /* add one position space for centering the output numbers */
         sprintf(scale_string_ptr_top + scale_string_pos_top, " ");
         sprintf(scale_string_ptr_btm + scale_string_pos_btm, " ");
         scale_string_pos_btm ++;
         scale_string_pos_top ++;
      }

      /* '|' takes the place of the last '-' */
      sprintf(scale_string_ptr + scale_string_pos, "|");
      sprintf(scale_string_ptr_top + scale_string_pos_top, " ");
      sprintf(scale_string_ptr_btm + scale_string_pos_btm, " ");

      /* since the numbers on the scale can overlap, every other number
         is output to the top then the bottom of the scale line */
      if (position == BOTTOM)
      {
         sprintf(scale_string_ptr_btm+scale_string_pos_btm-lenofvalues[index],
                 "%lld",log_array[index]);
         sprintf(scale_string_ptr_btm+scale_string_pos_btm, " ");
         position = TOP;
         sprintf(scale_string_ptr_top+scale_string_pos_top+lenofvalues[index],
                  " ");
      }
      else
      {
         sprintf(scale_string_ptr_top+scale_string_pos_top-lenofvalues[index],
                  "%lld",log_array[index]);
         sprintf(scale_string_ptr_top+scale_string_pos_top, " ");
         position = BOTTOM;
         sprintf(scale_string_ptr_btm+scale_string_pos_btm+lenofvalues[index],
                  " ");
      }
      scale_string_pos ++;
      scale_string_pos_top++;
      scale_string_pos_btm++;

      /* add another space for centering purposes */
      if (position == BOTTOM)
      {
         sprintf(scale_string_ptr_top+scale_string_pos_top, " ");
         scale_string_pos_top++;
      }
      else
      {
         sprintf(scale_string_ptr_btm+scale_string_pos_btm, " ");
         scale_string_pos_btm++;
      }
      previous_scalestars = total_scalestars; 
   }

   /* print to file */
   fprintf(fp, "%s\n", scale_string_top);
   fprintf(fp, "%s\n", scale_string);
   fprintf(fp, "%s\n", scale_string_btm);
}
