/*************************************************************************
   NAME:                        MMMSD2


   PURPOSE:  MMMSD2 calculates the mean, mode, median, and standard
             deviation of a single dimensioned array.

**************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "pixcount.h"

void mmmsd2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: mininum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   double  *mean,              /*  O: mean                               */
   double  *stddev,            /*  O: standard deviation                 */
   float   *med,               /*  O: median                             */
   float   *mode,              /*  O: mode                               */
   long    nl,                 /*  I: number of lines                    */
   long    ns,                 /*  I: number of samples per line         */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   long    midpt               /*  I: middle point in histogram array    */
   )

   {
   double  sum;                /*  sum of the pixel values              */
   double  sumsq;              /*  sum of the squares of pixel values   */
   float   temp;               /*  temperary variable                   */

   long long count;            /*  total of the number of pixels        */
   long    i;                  /*  loop index                           */
   long    k;                  /*  loop index                           */
   long long midnum;           /*  half of the total number of pixels   */
   long    mmode;              /*  current mode                         */
   long long moccur;           /*  current maximum number of occurences */
   long long numel;            /*  number of pixels to be histogrammed  */


   /*   initialize local variables
   ---------------------------------------------------------------------*/
   numel = (long long)nl * ns - mskcnt;
   midnum = numel / 2;
   mmode = minval;
   moccur = 0;
   sum = 0;
   sumsq = 0;
   count = 0;

   for(i = (minval + midpt); i <= (maxval + midpt); i++)
      {
      /*   keep a running total of the number of pixels
      ------------------------------------------------------------------*/
      count += hist2[i];

      /*   calculate current mode
      ------------------------------------------------------------------*/
      if (moccur < hist2[i])
         {
         /*  ' - midpt' used to correct for negative value indexing.
         ---------------------------------------------------------------*/
         mmode = i - midpt;
         moccur = hist2[i];
         }

      /*   calculate median
      ------------------------------------------------------------------*/
      if (hist2[i] > 0)
         {
         if (count < (midnum + hist2[i]))
            {
            *med = (float)(i - midpt);
            temp = 0.0;
            if ((long long)(numel / 2.0 + .5) == (numel/2))
               {
               for(k = (i+1); k < (maxval + midpt); k++)
                  if (hist2[k] > 0)
                     {
                     temp = (float) (k - midpt);
                     break;
                     }
               }
            else
               temp = (float)(i - midpt);
            *med = (*med + temp)/2.0;
            }
         }
      sum += ((double)hist2[i] * (i - midpt));
      sumsq += (hist2[i] * ((double)(i - midpt) * (i - midpt)));
      }

   /*   set the mode
   ---------------------------------------------------------------------*/
   *mode = (float)mmode;

   /*   calculate mean and standard deviation
   ---------------------------------------------------------------------*/
   if (numel == 0)
      {
      *mean = 0.0;
      *stddev = 0.0;
      }
   else
      {
      *mean = (double)sum/numel;
      *stddev = ((double)sumsq / numel) - (*mean * *mean);

      /*   get the absolute value of stddev
      --------------------------------------------------------------------*/
      *stddev = sqrt(fabs(*stddev));
      }
   }
