/*************************************************************************
   NAME:                        MMMSD4


   PURPOSE:  MMMSD4 calculates the median, and standard
             deviation for integer*4 (not in integer*2 range) and
             real*4 data.

**************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "pixcount.h"

void mmmsd4 (
   double   histr[2][MAXBIN],  /*  I: R*4 histogram array                */
   long     icount,            /*  I: number of different pixel values   */
   double   *dmean,            /*  O: mean                               */
   double   *dstddv,           /*  O: standard deviation                 */
   float    *med,              /*  O: median                             */
   float    *mode,             /*  O: mode                               */
   long long *moccur,          /*  O: maximum number of occurrences      */
   long     *window,           /*  I: current window specification       */
   long     mskcnt             /*  I: pixels not in histogram count      */
   )

   {
   double   sum;               /*  sum of the pixel values               */
   double   sumsq;             /*  sum of the squares of pixel values    */
 
   float    mmode;             /*  current mode                          */
   float    temp;              /*  temporary value                       */

   long long count;            /*  total of the number of pixels         */
   long     i;                 /*  loop index                            */
   long     k;                 /*  loop index                            */
   long long midnum;           /*  half of the total number of pixels    */
   long long numel;            /*  number of pixels to be histogrammed   */


   /*   initialize local variables
   ----------------------------------------------------------------------*/
   numel = (long long)window[2] * window[3] - mskcnt;
   midnum = numel / 2;
   count = 0;
   sum = 0;
   sumsq = 0;
   *moccur = 0;

   for(i = 0; i < icount; i++)
      {
      /*   keep a running total of the number of pixels
      ----------------------------------------------------------------*/
      count += histr[1][i];

      /*   calculate current mode
      ----------------------------------------------------------------*/
      if (*moccur < histr[1][i])
         {
         mmode = histr[0][i]-1;
         *moccur = (long long)histr[1][i];
         }

      /*   calculate median
      ----------------------------------------------------------------*/
      if (histr[1][i] != 0)
         if (count < (midnum + histr[1][i]))
            {
            *med = histr[0][i]-1;
            if ((long)(numel/2.0 + .5) == (numel/2))
               {
               for(k = (i + 1); k < icount; k++)
                  if (histr[1][k] > 0)
                     {
                     temp = (float) k;
                     break;
                     }
               }
            else
            temp = (float)i;
            *med = (*med + temp) / 2.0;
            }

      if (histr[0][i] != 0)
         {
         temp = histr[0][i]-1; 
         sum += (temp * histr[1][i]);
         sumsq += ((temp * temp) * histr[1][i]);
         }
      }

   /*   set the mode
   ----------------------------------------------------------------------*/
   *mode = mmode;

   /*   calculate mean and standard deviation
   ----------------------------------------------------------------------*/
   if (numel == 0)
      {
      *dmean = 0.0;
      *dstddv = 0.0;
      }
   else
      {
      *dmean = (double)sum/numel;
      *dstddv = ((double)sumsq / numel) - (*dmean * *dmean);

      /*   get the absolute value for dstddv
      --------------------------------------------------------------------*/
      *dstddv = sqrt(fabs(*dstddv));
      }
   }
