#ifndef PIXCOUNT_H
#define PIXCOUNT_H
/*=====================================================================
   This header file is for pixcount

   note: make sure that the las.h header is included in the program 
         before this header file.


       I: indicates input to function.
       O: Output from function.
       I/O: input and output to function. 
=====================================================================*/
#define MAXBIN 16384  /* number of bins for I*4 / R*4 data            */
#define MAXBUF 65536  /* number of bins for I*2 data                  */
#define MIDPT  32768  /* midpoint in I*2 data for negative indecies   */
#define THOUS  1000   /* to allow for 500 mask range values           */
#define DDLEN  80     /* an 80 character length array                 */
#define MXCNT  50     /* 50 strings                                   */
#define MXCHAR 200    /* message string for concatination             */
#define MBINS  33     /* default number of bins used                  */
#define MAXSTARS 72   /* maximum number of start that will fit on a page */
#define BOTTOM 0      /* flag indicating output numbers are to be 
                         placed below the scale line                  */
#define TOP 1         /* flag indicating output numbers are to be
                         placed above the scale line                  */
#define MAX_LINES_IN_BLOCK 20 /* number of lines in a block           */

/*  Program initialization functions
-----------------------------------------------------------------*/
void initio (
   char *host,                 /*  I: host name of input image           */
   long *dtype,                /*  I: data type for opening input image  */
   long *window                /*  I: current window specification       */
   );

int getpar (
   struct PARBLK *parblk,      /*  O: Parameter block                    */
   char *subcmd,               /*  O: specified subcommand               */
   long *msgtim,               /*  O: time between display messages      */
   char taein[MAXIMG][CMLEN],  /*  O: TAE names of input images          */
   char hosin[MAXIMG][CMLEN],  /*  O: host names of input images         */
   long window[MAXIMG][4],     /*  O: window specification               */
   long bands[MAXIMG][MAXBND+1], /*  O: band specification               */
   long *nmrimg,               /*  O: number of input images             */
   long *nbands,               /*  O: number of bands in each image      */
   long *totbnd,               /*  O: total number of output bands       */
   char *option,               /*  O: list option                        */
   char *htype,                /*  O: histogram type                     */
   long *nbins,                /*  O: number of specified bins           */
   float *maskr,               /*  O: mask values not in histogram       */
   long *nomask,               /*  O: mask flag                          */
   long *file,                 /*  O: file output flag                   */
   long *lp,                   /*  O: line printer output flag           */
   long *term,                 /*  O: terminal output flag               */
   char *logopt,               /*  O: log print option                   */
   long *sumry,                /*  O: specified summary option           */
   char *prtfil,               /*  O: print file for setprint            */
   char filena[][CMLEN],       /*  O: file name for disk file            */
   long *astrik,               /*  O: number of occurences               */
   long *sigdig,               /*  O: number of significant digits       */
   long *totcnt                /*  O: number of mask values * 2          */
   );

void logscale
(
   FILE *fp,                   /* I/O: output file pointer                   */
   long long maxpixelcount,    /* I: the largest total number of pixels
                                  with a unique pixel value                  */
   char scale_string[],        /* O: buffer used for scale line output       */
   char scale_string_top[],    /* O: used for scale #s above scale line      */
   char scale_string_btm[],    /* O: used for scale #s below scale line      */
   float *stars_ratio,         /* O: value used to calculate the number of
                                  stars to output to the histogram           */
   int scale_spacing_flag,     /* I: flag indicating scale spacing           */
   long max_nstars_per_page,   /* I: maximum number of stars that will fit
                                  on a page                                  */
   long long numel,            /* I: number of pixels to be histogrammed     */
   char *htype                 /* I: type of histogram                       */
);

/* I*2 and Byte image function headers and I*4 that fits in I*2
------------------------------------------------------------------*/
int getim2 (
   long         bndnum,        /*  I: current band number                */
   long         nl,            /*  I: number of lines                    */
   long         ns,            /*  I: number of samples per line         */
   long long    *hist2,        /*  O: histogram array                    */
   char         *subcmd,       /*  I: specified subcommand               */
   long         *mask,         /*  I: mask value                         */
   long         *mskcnt,       /*  O: pixels not in histogram count      */
   long         msgtim,        /*  I: elapsed time between messages      */
   long         totbnd,        /*  I: total number of output bands       */
   long         totcnt,        /*  I: total number of mask values * 2    */
   long         midpt,         /*  I: middle point in histogram array    */
   struct FDESC **fdesc,       /*  I: description of file                */
   long         maxval,        /*  I: maximum pixel value                */
   long         minval,        /*  I: minimum pixel value                */
   long long    *maxpixelcount /*  O: the largest total number of pixels
                                      with a unique pixel value          */ 
   );

void mmmsd2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: mininum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   double  *mean,              /*  O: mean                               */
   double  *stddev,            /*  O: standard deviation                 */
   float   *med,               /*  O: median                             */
   float   *mode,              /*  O: mode                               */
   long    nl,                 /*  I: number of lines                    */
   long    ns,                 /*  I: number of samples per line         */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   long    midpt               /*  I: middle point in histogram array    */
   );

void tplot2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: minimum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   double  mean,               /*  I: mean                               */
   double  stddev,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   char    *htype,             /*  I: histogram type                     */
   long    astrik,             /*  I: number of occurrences / asterisk   */
   long    midpt               /*  I: middle point in histogram array    */
   );

void pplot2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: minimum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   double  mean,               /*  I: mean                               */
   double  stddev,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   long    *window,            /*  I: current window specification       */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   char    *subcmd,            /*  I: subcommand specified               */
   char    *htype,             /*  I: histogram type                     */
   long    mskcnt,             /*  I: pixels not included in count       */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sumry,              /*  I: summary report option              */
   long    astrik,             /*  I: number of occurrences / asterisk   */
   long    midpt               /*  I: middle point in histogram array    */
   );

void ppbin2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: minimum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   long    *window,            /*  I: current window specification       */
   long    nbins,              /*  I: number of specified bins           */
   double  mean,               /*  I: mean                               */
   double  stddev,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   char    *htype,             /*  I: histogram type                     */
   long    mskcnt,             /*  I: pixels not included in histogram   */
   char    *subcmd,            /*  I: subcommand specified               */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sumry,              /*  I: summary report option              */
   long    prt,                /*  I: print option                       */
   long    astrik,             /*  I: number of occurrences / asterisk   */
   long    midpt               /*  I: middle point in histogram array    */
   );

void logplot2 (
   long long *hist2,           /*  I: histogram array                    */
   long   minval,              /*  I: minimum pixel value                */
   long   maxval,              /*  I: maximum pixel value                */
   double mean,                /*  I: mean                               */
   double stddev,              /*  I: standard deviation                 */
   float  mode,                /*  I: mode                               */
   float  med,                 /*  I: median                             */
   long   *window,             /*  I: current window specification       */
   char   *tname,              /*  I: TAE name of input image            */
   long   bndnum,              /*  I: current band number                */
   char   *subcmd,             /*  I: subcommand specified               */
   char   *htype,              /*  I: histogram type                     */
   long   mskcnt,              /*  I: pixels not in histogram count      */
   FILE   *fp,                 /*  I/O: output file pointer              */
   long   sumry,               /*  I: summary report option              */
   long   midpt,               /*  I: middle point in histogram array    */
   long long maxpixelcount     /*  I: the largest total number of pixels
                                      with a unique pixel value          */
   );

void logpbin2 (
   long long *hist2,           /*  I: histogram array                    */
   long   minval,              /*  I: minimum pixel value                */
   long   maxval,              /*  I: maximum pixel value                */
   long   *window,             /*  I: current window specification       */
   long   nbins,               /*  I: number of specified bins           */
   double mean,                /*  I: mean                               */
   double stddev,              /*  I: standard deviation                 */
   float  mode,                /*  I: mode                               */
   float  med,                 /*  I: median                             */
   char   *tname,              /*  I: TAE name of input image            */
   long   bndnum,              /*  I: current band number                */
   char   *htype,              /*  I: histogram type                     */
   long   mskcnt,              /*  I: pixels not in histogram count      */
   FILE   *fp,                 /*  I/O: output file pointer              */
   long   sumry,               /*  I: summary report option              */
   long   prt,                 /*  I: print option                       */
   long   midpt                /*  I: middle point in histogram array    */
   );

void prpix2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: minimum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   double  mean,               /*  I: mean                               */
   double  stddev,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   long    *window,            /*  I: current window specification       */
   long    mskcnt,             /*  I: pixels not included in count       */
   char    *htype,             /*  I: histogram type                     */
   char    *subcmd,            /*  I: subcommand specified               */
   long    file,               /*  I: file output flag                   */
   long    lp,                 /*  I: line printer output flag           */
   long    term,               /*  I: terminal output flag               */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    midpt               /*  I: middle point in histogram array    */
   );

/* I*4 and Real image function headers 
------------------------------------------------------------------*/
int getim4 (
   double  histr[2][MAXBIN],    /*  O: R*4 histogram array                */
   long   *icount,             /*  O: number of different pixel values   */
   long   *window,             /*  I: current window specification       */
   long   dtype,               /*  I: data type                          */
   char   *subcmd,             /*  I: specified subcommand               */
   float  *maskr,              /*  I: R*4 mask value                     */
   long   *mskcnt,             /*  O: pixels not in histogram count      */
   long   bndnum,              /*  I: current band number                */
   long   totbnd,              /*  I: total number of output bands       */
   long   msgtim,              /*  I: time between display messages      */
   float  minv,                /*  I: minimum value                      */
   float  maxv,                /*  I: maximum value                      */
   float  rrange,              /*  I: range of values                    */
   long   totcnt,              /*  I: total number of mask values * 2    */
   struct FDESC **fdesc,       /*  I: description of file                */
   long long *maxpixelcount    /*  O: the largest total number of pixels
                                      with a unique pixel value          */

   );

void mmmsd4 (
   double   histr[2][MAXBIN],  /*  I: R*4 histogram array                */
   long     icount,            /*  I: number of different pixel values   */
   double   *dmean,            /*  O: mean                               */
   double   *dstddv,           /*  O: standard deviation                 */
   float    *med,              /*  O: median                             */
   float    *mode,             /*  O: mode                               */
   long long *moccur,          /*  O: maximum number of occurrences      */
   long     *window,           /*  I: current window specification       */
   long     mskcnt             /*  I: pixels not in histogram count      */
   );

void tplot4 (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   long    dtype,              /*  I: data type                          */
   char    *htype,             /*  I: histogram type                     */
   long    astrik,             /*  I: number of occurrences per asterisk */
   float   minv,               /*  I: minimum value                      */
   float   maxv,               /*  I: maximum value                      */
   long    sigdig              /*  I: number of significant digits       */
   );

void pplot4 (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   long    *window,            /*  I: current window specification       */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   long    dtype,              /*  I: data type                          */
   char    *subcmd,            /*  I: subcommand specified               */
   char    *htype,             /*  I: histogram type                     */
   long long moccur,           /*  I: maximum number of occurences       */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sumry,              /*  I: summary report option              */
   long    astrik,             /*  I: number of occurrences per asterisk */
   float   rrange,             /*  I: range of values                    */
   long    sigdig              /*  I: number of significant digits       */
   );

void ppbin4 (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    *window,            /*  I: current window specification       */
   long    bndnum,             /*  I: current band number                */
   long    dtype,              /*  I: data type                          */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   char    *htype,             /*  I: histogram type                     */
   char    *subcmd,            /*  I: subcommand specified               */
   long    nbins,              /*  I: number of specified bins           */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sumry,              /*  I: summary report option              */
   long    prt,                /*  I: print option                       */
   long    astrik,             /*  I: number of occurrences per asterisk */
   long    sigdig              /*  I: number of significant digits       */
   );

void logplot4 (
   double histr[2][MAXBIN],    /*  I: histogram array                    */
   long   icount,              /*  I: number of different pixel values   */
   double dmean,               /*  I: mean                               */
   double dstddv,              /*  I: standard deviation                 */
   float  mode,                /*  I: mode                               */
   float  med,                 /*  I: median                             */
   long   *window,             /*  I: current window specification       */
   char   *tname,              /*  I: TAE name of input image            */
   long   bndnum,              /*  I: current band number                */
   long   dtype,               /*  I: data type                          */
   char   *htype,              /*  I: histogram type                     */
   long   mskcnt,              /*  I: pixels not in histogram count      */
   FILE   *fp,                 /*  I/O: output file pointer              */
   long   sumry,               /*  I: summary report option              */
   long   sigdig,              /*  I: number of significant digits       */
   long long maxpixelcount     /*  I: the largest total number of pixels
                                      with a unique pixel value          */
   );

void logpbin4 (
   double   histr[2][MAXBIN],   /*  I: histogram array                    */
   long   icount,              /*  I: number of different pixel values   */
   double dmean,               /*  I: mean                               */
   double dstddv,              /*  I: standard deviation                 */
   float  mode,                /*  I: mode                               */
   float  med,                 /*  I: median                             */
   char   *tname,              /*  I: TAE name of input image            */
   long   *window,             /*  I: current window specification       */
   long   bndnum,              /*  I: current band number                */
   long   dtype,               /*  I: data type                          */
   long   mskcnt,              /*  I: pixels not in histogram count      */
   char   *htype,              /*  I: histogram type                     */
   long   nbins,               /*  I: number of specified bins           */
   FILE   *fp,                 /*  I/O: output file pointer              */
   long   sumry,               /*  I: summary report option              */
   long   prt,                 /*  I: print option                       */
   long   sigdig               /*  I: number of significant digits       */
   );

void prpix4 (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    *window,            /*  I: current window specification       */
   long    bndnum,             /*  I: current band number                */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   char    *htype,             /*  I: histogram type                     */
   char    *subcmd,            /*  I: subcommand specified               */
   long    file,               /*  I: file output flag                   */
   long    lp,                 /*  I: line printer output flag           */
   long    term,               /*  I: terminal output flag               */
   FILE    *fp                 /*  I/O: output file pointer              */
   );

void prpixr (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    *window,            /*  I: current window specification       */
   long    bndnum,             /*  I: current band number                */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   char    *htype,             /*  I: histogram type                     */
   char    *subcmd,            /*  I: subcommand specified               */
   long    file,               /*  I: file output flag                   */
   long    lp,                 /*  I: line printer output flag           */
   long    term,               /*  I: terminal output flag               */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sigdig              /*  I: number of significant digits       */
   );

# endif
