/*************************************************************************
   NAME:                        PPLOT4


   PURPOSE:  PPLOT4 creates the print file with the histogram and data
          of each different pixel value for integer*4 / real*4
          processing.

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "las.h"
#include "pixcount.h"

void pplot4 (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   long    *window,            /*  I: current window specification       */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   long    dtype,              /*  I: data type                          */
   char    *subcmd,            /*  I: subcommand specified               */
   char    *htype,             /*  I: histogram type                     */
   long long moccur,           /*  I: maximum number of occurences       */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sumry,              /*  I: summary report option              */
   long    astrik,             /*  I: number of occurrences per asterisk */
   float   rrange,             /*  I: range of values                    */
   long    sigdig              /*  I: number of significant digits       */
   )

   {
   char    buff[DDLEN];        /*  formatted string output                */

   float   aprcnt;             /*  accumulated percentage of pixel values */
   float   prcnt;              /*  total percentage of pixel values       */
   float   temp;               /*  temporary variable                     */
   float   temp1;              /*  temporary variable                     */

   long    i;                  /*  loop index                             */
   long    itemp;              /*  temporary variable                     */
   long long itemp2;           /*  temporary variable                     */
   long    j;                  /*  loop index                             */
   long    nstars;             /*  stars scaled for a pixel value         */
   long long numel;            /*  number of pixels to be histogrammed    */
   long    page;               /*  number of pages to print               */
   long    roccur;             /*  running total of pixel occurences      */
   long long scale;            /*  scale of * to number of occurences     */
   long    flag;               /*  flag used for pterm                    */


   flag = 0;

   /*---------------------------------------------------------------------
      calculate the required number of pages needed with 60 lines per page.
      plus 1 page of extra information for no summary.
           2 pages extra pages with the summary information.
   ----------------------------------------------------------------------*/
   if (sumry)
      {
      page = (long)(rrange * pow(10,sigdig) / 60 + 2);
      if (page > (icount/60 + 2))
         page = icount/60 + 2;
      }
   else
      {
      page = (long)(rrange * pow(10,sigdig) / 60 + 1);
      if (page > (icount/60 + 1))
         page = icount / 60 + 1;
      }

   c_lasmsg(flag,""," %7d page(s) of data will be printed",page);

   /*  print the heading
   -------------------------------------------------------------------*/
   fprintf(fp,"          IMAGE:  %s     BAND:  %3d\n", tname, bndnum);

   if (strcmp(htype,"DIST") == 0)
      fprintf(fp,"          DISTRIBUTION HISTOGRAM\n");
   else
      fprintf(fp,"          CUMULATIVE HISTOGRAM\n");

   if (strcmp(subcmd,"BCKGRD") == 0)
      fprintf(fp,"          NUMBER OF BACKGROUND VALUES = %10d\n\n",mskcnt);
   else
      fprintf(fp,"\n\n");

   fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",
                 window[0], window[1]);
   fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                 window[2], window[3]);
   fprintf(fp,"    PIXEL VALUE         NO.   TOTAL %%   ACCUM. %%\n");
   fprintf(fp,"    -----------         ---   -------   --------\n");

   /*   initialize accumulative percentage
   -------------------------------------------------------------------*/
   aprcnt = 0;

   /*   calculate the scale
   -------------------------------------------------------------------*/
   numel = (long long)window[2] * window[3] - mskcnt;

   if (astrik == 0)
      {
      if (strcmp(htype,"DIST") == 0)
         scale = (moccur / 80) + 1;
      else
         scale = (numel / 80) + 1;
      }
   else
      scale = astrik;

   roccur = 0;

   /*   process each histogram value
   -------------------------------------------------------------------*/
   i = 0;
   if (numel > 0)
      {
      itemp2 = (long)histr[0][i];
      temp = histr[0][i] - itemp2; 
      temp = (temp * pow(10, sigdig)); 
      temp1 = temp / pow(10, sigdig) + itemp2;
      itemp2 = (long long)histr[1][i];
      for(i = 1; i <= icount; i++)
         {
         temp = histr[0][i]; 
         histr[0][i] -= temp;
         itemp = (long)(histr[0][i] * pow(10, sigdig));
         temp = (float)itemp / pow(10, sigdig) + temp;
         if (temp1 ==  temp)
            {
            itemp2 += (long long)histr[1][i];
            if (i != icount) continue;
            } 

         /*---------------------------------------------------------
           calculate the number of stars according to scale specifications,
           ratio of histogram value to number of total pixels (percent), 
           and accumulative percent
         ----------------------------------------------------------*/
         roccur += itemp2;
         if (strcmp(htype,"DIST") == 0)
            nstars = (long)(itemp2 / scale);
         else
            nstars = (long)(roccur / scale);

         prcnt = (itemp2 / (double)numel) * 100;
         aprcnt += prcnt;

  
         if (dtype == ELONG)
            {
            itemp = (long)temp1;

            /*-----------------------------------------------------------
                 print the histogram
                 itemp is modified to make up for the nonzero bin value
                 histogram count
            -----------------------------------------------------------*/
            if (nstars < 1)
               fprintf(fp," %14d   %11lld %6.2f     %6.2f   :", (itemp - 1), 
                        itemp2, prcnt, aprcnt);
            else
               {
               if (nstars > 80) nstars = 80;
               fprintf(fp," %14d   %11lld %6.2f     %6.2f   :", (itemp - 1), 
                        itemp2, prcnt, aprcnt);
               for(j = 0; j < nstars; j++)
                  fprintf(fp,"*");
               }
            fprintf(fp,"\n");
            }

         if (dtype == EREAL)
            {
            /*---------------------------------------------------------
               generate the format based upon the number of significant
               digits specified.
            -----------------------------------------------------------*/
            sprintf(buff," %c14.%df",'%',sigdig);
            fprintf(fp, buff, (temp1 - 1));

            /*   print the histogram
            -------------------------------------------------------*/
            if (nstars < 1)
               fprintf(fp,"   %11lld %6.2f     %6.2f   :\n", itemp2,
                           prcnt, aprcnt);
            else
               {
               if (nstars > 80) nstars = 80;
               fprintf(fp,"   %11lld %6.2f     %6.2f   :", itemp2,
                           prcnt, aprcnt);
               for(j = 0; j < nstars; j++)
                  fprintf(fp,"*");
               fprintf(fp,"\n");
               }
            }

            itemp2= (long)histr[1][i];
            temp1 = temp;
         }
      }

   /*   print blank lines
   -------------------------------------------------------------------*/
   fprintf(fp,"\n\n\n\n\n");

   /*   print scale specifications
   -------------------------------------------------------------------*/
   fprintf(fp,"  SCALE:   1 * = %7lld OCCURENCES\n\n",scale);

   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   fprintf(fp,"  MEAN= %11.3f          MODE= %#9.3f          MEDIAN= %#9.3f\n",
            dmean, mode, med);
   fprintf(fp,"  STANDARD DEVIATION= %11.3f\n", dstddv);
   }
