/*************************************************************************
   NAME:                        PRPIXR


   PURPOSE:  PRPIXR prints data for REAL*4 input image data.

NOTE:
- The header on the saved file must be 8 lines exactly since the mapfile
  application depends on it.
**************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "pixcount.h"

void prpixr (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    *window,            /*  I: current window specification       */
   long    bndnum,             /*  I: current band number                */
   long    mskcnt,             /*  I: pixels not in histogram count      */
   char    *htype,             /*  I: histogram type                     */
   char    *subcmd,            /*  I: subcommand specified               */
   long    file,               /*  I: file output flag                   */
   long    lp,                 /*  I: line printer output flag           */
   long    term,               /*  I: terminal output flag               */
   FILE    *fp,                /*  I/O: output file pointer              */
   long    sigdig              /*  I: number of significant digits       */
   )

   {
   char    buf[MXCHAR];        /*  message string                        */
   char    buf1[DDLEN];        /*  message string                        */

   double  cum;                /*  cumulative percentage                 */
   double  frac;               /*  fractional percentage                 */
 
   float   temp;               /*  temporary variable                    */

   long    flag;               /*  flag for whether "pterm" query user   */
   long    i;                  /*  loop index                            */
   long    itemp;              /*  temporary variable                    */
   long long itemp2;           /*  temporary variable                    */
   long long numel;            /*  number of pixels to be histogrammed   */


   flag = 0;

   /*  print the heading
   -------------------------------------------------------------------*/
   if (term)
      {
      sprintf(buf,"\n            IMAGE: %s   BAND: %3d", tname, bndnum);
      c_pterm(buf, &flag);

      if (strcmp(htype,"DIST") == 0)
         sprintf(buf,"            DISTRIBUTION HISTOGRAM");
      else
         sprintf(buf,"            CUMULATIVE HISTOGRAM");
      c_pterm(buf, &flag);

      if (strcmp(subcmd,"BCKGRD") == 0)
         sprintf(buf,"            NUMBER OF BACKGROUND VALUES = %10d\n\n",
                     mskcnt);
      else
         sprintf(buf,"\n");
      c_pterm(buf, &flag);

      sprintf(buf,"          STARTING LINE= %7d  STARTING SAMPLE= %7d",
                    window[0], window[1]);
      c_pterm(buf, &flag);
      sprintf(buf,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d",
                    window[2], window[3]);
      c_pterm(buf, &flag);
      sprintf(buf,"\n       PIXEL VALUE        COUNT             FRACTION");
      c_pterm(buf, &flag);
      sprintf(buf,"       -----------        -----             --------");
      c_pterm(buf, &flag);
      }

   if (lp || file)
      {
      fprintf(fp,"          IMAGE:  %s    BAND:  %3d\n", tname, bndnum);

      if (strcmp(htype,"DIST") == 0)
         fprintf(fp,"          DISTRIBUTION HISTOGRAM\n");
      else
         fprintf(fp,"          CUMULATIVE HISTOGRAM\n");

      if (strcmp(subcmd,"BCKGRD") == 0)
         fprintf(fp,"          NUMBER OF BACKGROUND VALUES = %10d\n\n",
                     mskcnt);
      else
         fprintf(fp,"\n\n");

      fprintf(fp,"          STARTING LINE= %7d  STARTING SAMPLE= %7d\n",
                    window[0], window[1]);
      fprintf(fp,"        NUMBER OF LINES= %7d  NUMBER OF SAMPLES= %7d\n",
                    window[2], window[3]);
      fprintf(fp,"       PIXEL VALUE        COUNT             FRACTION\n");
      fprintf(fp,"       -----------        -----             --------\n");
      }

   /*   calculate number of elements
   -------------------------------------------------------------------*/
   numel = (long long) window[2] * window[3] - mskcnt;

   /*   initialize cumulative percentage
   -------------------------------------------------------------------*/
   cum = 0;

   /*   process each histogram value
   -------------------------------------------------------------------*/

   if (numel > 0)
      for(i = 0; i < icount; i++)
         {
         temp = histr[0][i]-1; 
         histr[0][i] -= temp + 1;
         itemp = (long)(histr[0][i] * pow(10,sigdig));
         temp = (float)itemp /pow(10,sigdig) + temp;
         itemp2 = (long long)histr[1][i];
         frac = (histr[1][i] / (double)numel);
         cum += frac;

         /*-----------------------------------------------------------------
            generate the format based upon the number of significant
            digits specified.
         -------------------------------------------------------------------*/
         sprintf(buf1,"  %c#16.%df",'%',sigdig);

         if (lp || file)
            fprintf(fp, buf1, temp);
         if (term)
            sprintf(buf, buf1, temp);

         if (histr[1][i] > 0)
            {
            if (term)
               {
               if (strcmp(htype,"DIST") == 0)
                  sprintf(buf1,"   %11lld       %12.6f", itemp2, frac);
               else
                  sprintf(buf1,"   %11lld       %12.6f", itemp2, cum);
               strcat(buf,buf1);
               c_pterm(buf, &flag);
               }

            if (lp || file)
               {
               if (strcmp(htype,"DIST") == 0)
                  fprintf(fp,"   %11lld       %12.6f\n", itemp2, frac);
               else
                  fprintf(fp,"   %11lld       %12.6f\n", itemp2, cum);
               }
            }
         }

   /*   print blank lines
   -------------------------------------------------------------------*/
   if (term)
      {
      buf[0] = NULL;
      c_pterm(buf, &flag);
      c_pterm(buf, &flag);
      c_pterm(buf, &flag);
      }
   if (lp || file)
      fprintf(fp,"\n\n\n");

   /*   print mean, mode, median, and standard deviation
   -------------------------------------------------------------------*/
   if (term)
      {
      sprintf(buf," MEDIAN             = %#11.3f", med);
      c_pterm(buf, &flag);
      sprintf(buf," MODE               = %#11.3f", mode);
      c_pterm(buf, &flag);
      sprintf(buf," MEAN PIXEL VALUE   = %17.8f", dmean);
      c_pterm(buf, &flag);
      sprintf(buf," STANDARD DEVIATION = %17.8f", dstddv);
      c_pterm(buf, &flag);
      buf[0] = NULL;
      c_pterm(buf, &flag);
      }

   if (lp || file)
      {
      fprintf(fp," MEDIAN             = %#11.3f\n", med);
      fprintf(fp," MODE               = %#11.3f\n", mode);
      fprintf(fp," MEAN PIXEL VALUE   = %17.8f\n", dmean);
      fprintf(fp," STANDARD DEVIATION = %17.8f\n", dstddv);
      }
   }
