/*************************************************************************
   NAME:                        TPLOT2


   PURPOSE:  TPLOT2 prints the histogram and data to the terminal
             for integer*2 processing.

**************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "pixcount.h"

void tplot2 (
   long long *hist2,           /*  I: histogram array                    */
   long    minval,             /*  I: minimum pixel value                */
   long    maxval,             /*  I: maximum pixel value                */
   double  mean,               /*  I: mean                               */
   double  stddev,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   char    *htype,             /*  I: histogram type                     */
   long    astrik,             /*  I: number of occurrences / asterisk   */
   long    midpt               /*  I: middle point in histogram array    */
   )

   {
   long    flag;                /*  flag for whether "pterm" query user  */
   long    i;                   /*  loop index                           */
   long    j;                   /*  loop index                           */
   long    indx;                /*  pointer index for "ivaray" array     */
   long    intrvl;              /*  interval size                        */
   long long ivaray[15];        /*  num. of elements in given intervals  */
   long long moccur;            /*  maximum sum of occurences            */
   long    nstars;              /*  stars scaled for a pixel value       */
   long long roccur;            /*  running total of pixel occurences    */
   long long scale;             /*  scale of * to number of occurences   */
   long long soccur;            /*  sum of occurences in an interval     */

   char    buf[MXCHAR];         /*  message string                       */

   /*   initialize local variables
   ----------------------------------------------------------------------*/
   for(i = 0; i < 15; i++)
      ivaray[i] = 0;
   roccur = 0;
   moccur = 0;
   flag = 0;

   /*   calculate interval size
   ----------------------------------------------------------------------*/
   intrvl = ((maxval - minval + 1) / 15);

   for(i = (minval + midpt), indx = 0; i < (maxval + midpt); i += intrvl, 
          indx++)
      {
      /*   initialize and calculate all occurences in given interval
      -------------------------------------------------------------------*/
      soccur = 0;
      for(j = i; j < (i + intrvl); j++)
         soccur += hist2[j];

      /*   place occurences in array
      -------------------------------------------------------------------*/
      ivaray[indx] = soccur;
      roccur += soccur;

      /*------------------------------------------------------------------
         calculate maximum number of pixels in
         intervals encountered so far
      -------------------------------------------------------------------*/
      if (moccur < soccur) moccur = soccur;
      }

   /*   calculate scale for number of pixels per *
   ----------------------------------------------------------------------*/
   if (astrik == 0)
      {
      if (strcmp(htype,"DIST") == 0)
         scale = (moccur / 60) + 1;
      else
         scale = (roccur / 60) + 1;
      }
   else
      scale = astrik;

   /*   write heading
   ----------------------------------------------------------------------*/
   strcpy(buf, " ");
   c_pterm(buf, &flag);

   if (strcmp(htype,"DIST") == 0)
      sprintf(buf,"  RANGE  OCCURENCES          DISTRIBUTION HISTOGRAM");
   else
      sprintf(buf,"  RANGE  OCCURENCES          ACCUMULATIVE HISTOGRAM");
   c_pterm(buf, &flag);

   sprintf(buf,"  -----  ----------");
   c_pterm(buf, &flag);

   /*   output histogram for each interval
   ----------------------------------------------------------------------*/
   roccur = 0;
   for(i = 0; i < indx; i++)
      {
      if (strcmp(htype,"DIST") == 0)
         nstars = (long)(ivaray[i] / scale);
      else
         nstars = (long)((ivaray[i] + roccur) / scale);

      if (nstars < 1)
         sprintf(buf," %6d %9lld:", (minval + (i * intrvl)), ivaray[i]);
      else
         {
         if (nstars > 60) nstars = 60;
         sprintf(buf," %6d %9lld:", (minval + (i * intrvl)), ivaray[i]);
         for(j = 0; j < nstars; j++)
            strcat(buf,"*");
         }
      c_pterm(buf, &flag);

      roccur = ivaray[i] + roccur;
      }

   /*----------------------------------------------------------------------
      output minimum pixel value, maximum pixel value, interval size,
      scale specifications, mean, mode, median, and standard deviation
   ----------------------------------------------------------------------*/
   strcpy(buf, " ");
   c_pterm(buf, &flag);
   sprintf(buf," IMAGE:  %s  BAND:  %3d", tname, bndnum);
   c_pterm(buf, &flag);
   sprintf(buf," MIN PIXEL VALUE = %10d    MAX PIXEL VALUE = %10d",
                  minval, maxval);
   c_pterm(buf, &flag);
   sprintf(buf," SCALE:   1 * = %7lld OCCURENCES     INTERVAL SIZE = %5d",
                  scale, intrvl);
   c_pterm(buf, &flag);

   sprintf(buf," MEAN= %9.2f MODE= %7.0f. MEDIAN= %9.2f", mean, 
                    mode, med);
   c_pterm(buf, &flag);
   sprintf(buf," STANDARD DEVIATION= %9.2f", stddev);
   c_pterm(buf, &flag);
   strcpy(buf, " ");
   c_pterm(buf, &flag);
   c_pterm(buf, &flag);
   }
