/*************************************************************************
   NAME:                        TPLOT4


   PURPOSE:  TPLOT4 prints the histogram and data to the terminal for
             integer*4 (not in integer*2 range) and real*4 processing.

**************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "pixcount.h"

void tplot4 (
   double  histr[2][MAXBIN],   /*  I: R*4 histogram array                */
   long    icount,             /*  I: number of different pixel values   */
   double  dmean,              /*  I: mean                               */
   double  dstddv,             /*  I: standard deviation                 */
   float   mode,               /*  I: mode                               */
   float   med,                /*  I: median                             */
   char    *tname,             /*  I: TAE name of input image            */
   long    bndnum,             /*  I: current band number                */
   long    dtype,              /*  I: data type                          */
   char    *htype,             /*  I: histogram type                     */
   long    astrik,             /*  I: number of occurrences per asterisk */
   float   minv,               /*  I: minimum value                      */
   float   maxv,               /*  I: maximum value                      */
   long    sigdig              /*  I: number of significant digits       */
   )

   {
   char    buf[MXCHAR];        /*  message string                         */
   char    buf1[DDLEN];        /*  output format buffer                   */
   
   float   rvaray[15];         /*  array elements for R*4 data            */

   long    flag;               /*  flag for whether "pterm" query user    */
   long    i;                  /*  loop index                             */
   long    j;                  /*  loop index                             */
   long    indx;               /*  index counter                          */
   long    intrvl;             /*  interval size                          */
   long long ivaray[15];       /*  array of elements in given intervals   */
   long long moccur;           /*  maximum sum of occurences in interval  */
   long    nstars;             /*  stars scaled for a pixel value         */
   long long roccur;           /*  running total of pixel occurences      */
   long long soccur;           /*  number of occurrences                  */
   long    scale;              /*  scale of * to number of occurences     */
   long    temp;               /*  temporary variable                     */
   long    var1;               /*  temporary variable                     */


   /*   initialize local variables
   ----------------------------------------------------------------------*/
   for(i = 0; i < 15; i++)
      {
      ivaray[i] = 0;
      rvaray[i] = 0;
      }
   flag = 0;
   roccur = 0;
   moccur = 0;

   /*   calculate interval size
   ----------------------------------------------------------------------*/
   intrvl = (icount  / 15) + 1;

   for(i = 0, indx = 0; i < icount; (i += intrvl), indx++)
      {
      /*   initialize occurences in an interval
      -------------------------------------------------------------------*/
      soccur = 0;
      rvaray[indx] = histr[0][i]; 

      /*   calculate all occurences in given interval
      -------------------------------------------------------------------*/
      for(j = i; j < (i + intrvl); j++)
         soccur += histr[1][j];

      /*   place occurences in array
      -------------------------------------------------------------------*/
      ivaray[indx] = soccur;
      roccur += soccur;

      /*------------------------------------------------------------------
         calculate maximum number of pixels in
         intervals encountered so far
      -------------------------------------------------------------------*/
      if (moccur < soccur) moccur = soccur;
      }

   /*   calculate scale for number of pixels per *
   ----------------------------------------------------------------------*/
   if (astrik == 0)
      {
      if (strcmp(htype,"DIST") == 0)
         scale = (long)((moccur / 55) + 1);
      else
         scale = (long)((roccur / 55) + 1);
      }
   else
      scale = astrik;

   /*   write heading
   ----------------------------------------------------------------------*/
   buf[0] = NULL;
   c_pterm(buf, &flag);
   if (strcmp(htype,"DIST") == 0)
      sprintf(buf,
            "            RANGE     OCCURENCES        DISTRIBUTION HISTOGRAM");
   else
      sprintf(buf,
            "            RANGE     OCCURENCES         ACCUMULATIVE HISTOGRAM");
   c_pterm(buf, &flag);

   sprintf(buf,"            -----     ----------");
   c_pterm(buf, &flag);

   /*   output histogram for each interval
   ----------------------------------------------------------------------*/
   roccur = 0;
   if (dtype == ELONG)
      for(i = 0; i <= (indx-1); i++)
         {
         if (strcmp(htype,"DIST") == 0)
            nstars = (long)(ivaray[i] / scale);
         else
            nstars = (long)((ivaray[i] + roccur) / scale);


         if (nstars < 1)
            sprintf(buf," %16d  %11lld:", (long)(rvaray[i] - 1), ivaray[i]);
         else
            {
            if (nstars > 55) nstars = 55;
            sprintf(buf," %16d  %11lld:", (long)(rvaray[i] - 1), ivaray[i]);
            for(j = 0; j < nstars; j++)
               strcat(buf,"*");
            }
         c_pterm(buf, &flag);

         roccur += ivaray[i];
         }

   if (dtype == EREAL)
      {
      for(i = 0; i < indx; i++)
         {
         temp = (long)rvaray[i]; 
         rvaray[i] -= (float)temp;
         var1 = (long)(rvaray[i] * pow(10,sigdig));
         rvaray[i] = (float)var1 / (float)pow(10,sigdig) + (float)temp;

         /*---------------------------------------------------------
            generate the format based upon the number of significant
            digits specified.
         -----------------------------------------------------------*/
         sprintf(buf1," %c#18.%df",'%',sigdig);
         sprintf(buf, buf1, rvaray[i] - 1);

         if (strcmp(htype,"DIST") == 0)
            nstars = (long)(ivaray[i] / scale);
         else
            nstars = (long)((ivaray[i] + roccur) / scale);

         if (nstars < 1)
            {
            sprintf(buf1," %12lld:", ivaray[i]);
            strcat(buf,buf1);
            }
         else
            {
            if (nstars > 55) nstars = 55;
            sprintf(buf1," %12lld:", ivaray[i]);
            strcat(buf,buf1);
            for(j = 0; j < nstars; j++)
               strcat(buf,"*");
            }
         c_pterm(buf, &flag);

         roccur += ivaray[i];
         }
      }

   /*----------------------------------------------------------------------
      output minimum pixel value, maximum pixel value, interval size,
      scale specifications, mean, mode, median, and standard deviation
   ----------------------------------------------------------------------*/
   buf[0] = NULL;
   c_pterm(buf, &flag);
   sprintf(buf," IMAGE:  %s  BAND:  %3d", tname, bndnum);
   c_pterm(buf, &flag);
   sprintf(buf," MIN PIXEL VALUE = %14.3f    MAX PIXEL VALUE = %14.3f",
                  minv, maxv);
   c_pterm(buf, &flag);
   sprintf(buf," SCALE:   1 * = %7d OCCURENCES     INTERVAL SIZE= %14d",
                  scale, intrvl);
   c_pterm(buf, &flag);

   sprintf(buf," MEAN= %10.3f MODE= %9.3f MEDIAN= %9.2f", dmean,
                 mode, med);
   c_pterm(buf, &flag);
   sprintf(buf," STANDARD DEVIATION= %10.2f", dstddv);
   c_pterm(buf, &flag);
   buf[0] = NULL;
   c_pterm(buf, &flag);
   c_pterm(buf, &flag);
   }
