#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "protos.h"

/******************************************************************************
FUNCTION:	CHECK_WINDOW

PURPOSE:	Checks the validity of the window specification.  If an
		error is detected, the window specification is changed
		to be the intersection of the window specification and
		the actual image.

ALGORITHM REFERENCES	none
*******************************************************************************/
int chk_window
(
    long *inspc,    /* I: input image file specifications     */
    long *window    /* I/O: window area of image to be copied */
)

{
long nonfatal = 1; /* non-fatal error code              */

/* If NL or NS is 0 then set them to the total number of lines and samples
--------------------------------------------------------------------------*/
if (window[NL] == 0)
   window[NL] = inspc[0];

if (window[NS] == 0)
   window[NS] = inspc[1];

/* Validate the number of lines specified
-----------------------------------------*/
if (window[SL] + window[NL] - 1 > inspc[0])
   {
   c_errmsg("NL is too large, resetting","pixfill-chckwin",&nonfatal);
   window[NL] = inspc[0] - window[SL] + 1;
   }

/* Validate the specified number of samples
-------------------------------------------*/
if (window[SS] + window[NS] - 1 > inspc[1])
   {
   c_errmsg("NS is too large, resetting","pixfill-chckwin",&nonfatal);
   window[NS] = inspc[1] - window[SS] + 1;
   }

return (E_SUCC);
}
