/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up input and output file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          1/14/85    J. Lavergne
   2          8/ 8/85    J. Davis    
   3          8/ 1/89    K. Narayanan    Conversion to Version 5.0
   6.1        19/5/94    D. Eichacker    Added conv_type[] to 1st eopen

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   open input and output images
   update output image ddr

**************************************************************************
*/
#include "pixsert.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char hosin[][CMLEN],      /* I: input images host names            */
    char hosout[],            /* I: output image host name             */
    long num_images,          /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each image      */
    long *num_bands, 	      /* I: number of image bands              */
    long image_band_num[][MAXBND + 1], /* I: image band numbers        */
    long wind[][4],           /* I: input image windows                */
    long otype,		      /* I: output data type                   */
    long *type                /* I/O:  file type                       */
)


{
double	line_inc = 1.0,
	samp_inc = 1.0;

extern int *fdesc;
extern int *ipoint;
extern int *opoint;

long	access,                      /* file access type                   */
	bufsiz,                      /* image i/o buffer size              */
	comb_flag = NOCOMB,
	i, 
	stat;                        /* return status                      */
long    fatal_err = -1;
long    inf_err = 1;
long    zero = 0;
long    one = 1;
static	long conv_type[3] = {0,0,0};
/*
open input and output image files
---------------------------------
*/

if (num_images > 0)
   {
   access = IREAD;
   for (i = 0; i < num_images; i++)
      {
      stat = c_eopens ((struct FDESC **)&fdesc[i], hosin[i], image_band_num[i], 
                       &wind[i][SL], &wind[i][SS], &wind[i][NL], &wind[i][NS], 
                       (long *)&ipoint[i], &access, type, &zero, 
		       (long *)conv_type);
      if (stat != E_SUCC )
         c_errmsg ("Fatal error encountered ", "pixsert-fatal", &fatal_err);
      }  
   }
else 
     {
	i = 0;
	*type = otype;
     }

access = IWRITE;
conv_type[0] = otype;
conv_type[1] = NO_COMPRES;
conv_type[2] = 0;
stat = c_eopens ((struct FDESC **)&fdesc[i], hosout, num_bands, &one, &one,
		 &wind[0][NL], &wind[0][NS], (long *)opoint, &access, type, 
                 &zero, (long *)conv_type);
if (stat != E_SUCC )
   c_errmsg ("Fatal error encountered ", "pixsert-fatal", &fatal_err);

fdesc[i+1] = 0;

/*
update output image DDR based on input images
---------------------------------------------
*/
if (num_images > 0)
   stat = c_upddr(&block,hosin,hosout,&i,wind,image_band_num,num_image_bands,
            &line_inc, &samp_inc, num_bands, &comb_flag);
   if (stat != E_SUCC )
      c_errmsg ("Nonfatal error encountered", "pixsert-warn", &inf_err);

/*
set up group containing input and output bands
----------------------------------------------
*/
if ((iobuf = ((char *) c_ealloc((struct FDESC **)fdesc, &bufsiz))) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "pixsert-alloc", 
            "pixsert",(char (*)[256]) hosout, &one);

stat = c_egroup ((struct FDESC **)fdesc,(struct GDESC **)&gdesc,
                  (unsigned char *)iobuf,&bufsiz);
if (stat != E_SUCC)
   c_abort(&block, "Fatal error encountered.","pixsert-fatal",
	  "pixsert",(char (*)[256])hosout, &one);
}
	
