/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1          1/01/85    Y. Sheu
    2          7/28/89    K. A. Narayanan    conversion to 5.0
    3          7/29/91    K. Kimball         Removed inadverdent ';' in
                                             a for loop and created loop
                                             to fill bands.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Runs under TAE

PROJECT:        LAS

***************************************************************************
*/

#include "pixsert.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

void getpar
(
    char hosin[][CMLEN],     /* O: input image host names              */
    char lasout[],           /* O: output image las name               */
    char hosout[],           /* O: output image host name              */
    long *num_inimages,      /* O: number of input images              */
    long image_band_num[][MAXBND+1],   /* O: image band numbers        */
    long num_image_bands[],  /* O: number of bands in each input image */
    long *num_bands,         /* O: number of image bands               */
    long wind[][4],          /* O: input image window                  */
    char coord_opt[],        /* O: master/relative coordinates option  */
    char subcmd[],           /* O: subcomand                           */
    long blksiz[],           /* O: size of blocks (NL, NS)             */
    long *radius,            /* O: radius of circle                    */
    long bands[],            /* O: bands to contain the blocks         */
    long *out_type,          /* O: output image data type              */
    double *bkgrnd,          /* O: output image background value       */
    char coordfil[],	     /* O: coordinate file host name           */
    long *grid,              /* O: transformation option               */
    long *ndeg,              /* O: degree of transformation polynomial */
    float aa[],              /* O: transformation coeffs for x         */
    float bb[]    	     /* O: transformation coeffs for y         */
)
{
struct	DDR	ddr;
struct	VARIABLE *var;              /* pointer to a variable in TAE block */

static long access,                 /* access type for getnam             */
	bandp[MAXBND],		    /* bands parameter                    */
	bcnt,                       /* band count                         */
	count,                      /* number of values to get in c_getnum*/
	flag[MAXIMG],               /* flag for subwindows                */
	ftype = EDOUBLE,
	i,
	imglimit = MAXIMG,          /* maximum number of images           */
	itype = ELONG,
	len,                        /* length of string to get in c_getstr*/ 
	ncoeffs,                    /* no. trans. coeffs                  */
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

static	short	dtab[] = {3, 6, 10, 15};

static char hosgrid[CMLEN],             /* transformation grid file host name */
	    lasin[CMLEN],       	/* input image las name               */
	    minmax[4],                  /* minmax option                      */
	    otype[5];                   /* output image data type parameter   */

long    fatal_err = -1;
long    zero = 0;
long    one = 1;

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

len = 6;
c_getstr (&block, "_SUBCMD", &one, &len, subcmd); 

if (subcmd[0]  ==  'I') 
   {
/*
   get and validate input image parameters
   ---------------------------------------
*/
   stat = c_getin (&block, "IN", &zero, (char (*)[256])lasin, hosin,
                   num_inimages, wind, 
                   image_band_num, num_image_bands, num_bands, &imglimit, 
                   &same_window,(long (*)[2])flag);

   if (stat != E_SUCC) 
       c_errmsg ("Fatal error encountered", "pixsert-fatal", &fatal_err);

   if (!same_window)
      c_errmsg ("Images must be of same size", "pixsert-size", &fatal_err);

/*
   set the output image data type to be the same as the input
   image data type
*/
   stat = c_getddr ((char *)hosin, &ddr);
   if (stat != E_SUCC) 
       c_errmsg ("Fatal error encountered", "pixsert-fatal", &fatal_err);
   *out_type = ddr.dtype;

/*
   get coordinate indicator
   ------------------------
*/
   len = 9;
   c_getstr (&block, "COOROPT", &one, &len, coord_opt);
   }
else
   {
   *num_inimages = 0;
/*
   get size of output image
   ------------------------
*/
   itype = ELONG;
   c_getnum (&block, "NL", &one, &itype,(unsigned char *)&wind[0][NL]); 
   c_getnum (&block, "NS", &one, &itype,(unsigned char *)&wind[0][NS]); 

/*
   get number of bands for output image
   ------------------------------------
*/
   c_getnum (&block, "NBANDS", &one, &itype,(unsigned char *)num_bands); 

/*
   get output data type
   --------------------
*/
   len = 5;
   c_getstr (&block, "ODTYPE", &one, &len, otype);
   if (strcmp (otype, "BYTE") == 0) *out_type = EBYTE;
   else if (strcmp (otype, "I*2") == 0) *out_type = EWORD;
   else if (strcmp (otype, "I*4") == 0) *out_type = ELONG;
   else *out_type = EREAL;

/*
   get background gray level for output image
   ------------------------------------------
*/
   stat = c_getnum (&block, "BACKGRND", &one, &ftype,(unsigned char *)bkgrnd);
   if (chkint (*bkgrnd, *out_type))
      c_errmsg ("Invalid background value", "pixsert-bkgrnd", &fatal_err); 
   }  
/*
get and validate output image parameters
----------------------------------------
*/
if (c_getout (&block, "OUT",(char (*)[256])lasout,
             (char (*)[256])hosout)!=E_SUCC)
   c_errmsg("Fatal error encountered", "pixsert-fatal", &fatal_err);

/*
get band numbers into which the blocks are inserted
---------------------------------------------------
*/
count = 256;
bcnt = c_getnum (&block, "BANDS", &count, &itype, (unsigned char *)bandp );

if (bcnt ==  0) 
   c_pxfill ((unsigned char *)bands, &itype, num_bands, (unsigned char *)&one);
else
   {
   if (bcnt  >  *num_bands) 
      c_errmsg ("More band numbers specified than the number of bands in image", 
                "pixsert-numbands", &fatal_err);

   c_pxfill ((unsigned char *)bands, &itype, &bcnt, (unsigned char *)&zero);
   for (i=0; i< bcnt; i++)
      {
      if (bandp[i]  >  *num_bands) 
         c_errmsg ("Band number is greater than number of bands in image",
                   "pixsert-bandnum", &fatal_err);

      bands[bandp[i]-1] = 1;
      }
   } 

/*
get the  region size
first, check for a radius
-------------------------
*/
count = 1;
stat = c_getnum (&block, "RADIUS", &count, &itype, (unsigned char *)radius);
if (stat == 0)
  *radius = 0;
else if (*radius *2 +1 >  wind[0][NL]  || *radius *2 +1 >  wind[0][NS])
   c_errmsg ("Radius size must be less than half the image size",
              "pixsert-circle", &fatal_err);

/*
next, check for block size
--------------------------
*/
count = 2;
stat = c_getnum (&block, "BLKSIZ", &count, &itype,(unsigned char *)blksiz);
if (stat < 2)
  blksiz[0] = blksiz[1] = 0;
else if (blksiz[0]  >  wind[0][NL]  || blksiz[1]  >  wind[0][NS]) 
   c_errmsg ("Region size must be less than image size", "pixsert-blksize",
           &fatal_err); 

/*
make sure one, and only one of radius and blksiz are set
--------------------------------------------------------
*/
if (*radius == 0)
  {
  if (blksiz[0] == 0)
    {
    c_errmsg("Either block size or radius must be specified",
             "pixsert-missing",&fatal_err);
    }
  }
else
  {
  if (blksiz[0] != 0)
    c_errmsg("Either block size or radius must be set to null",
             "pixsert-both",&fatal_err);
  /*
  radius is the only parameter set, so use it to calculate the blksiz
  -------------------------------------------------------------------
  */
  blksiz[0] = blksiz[1] = *radius*2+1;
  }

/*
get coordinates file - must be specified for batch processing
--------------------------------------------------------------
*/
access = IREAD;
if (c_getnam (&block, "INCOORD", &zero, coordfil, "txt", &access) != E_SUCC)
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal_err);

if (strlen (coordfil) == 0)
   {
   if (SVAL (*p_find (&block, "$runtype"), 0)[0] == 'B')
      c_errmsg ("Regions must be specified through INCOORD in batch mode",
                "pixsert-batch", &fatal_err);
   }
/*
get grid file - transforms input coords to different coords
(ex. lat, lon to line,sample)
------------------------------------------------------------
*/
access = IREAD;
if (c_getnam (&block, "INGRID", &zero, hosgrid, "grid", &access) != E_SUCC)   
   c_errmsg("Fatal error encountered","pixsert-fatal",&fatal_err);

if (strlen (hosgrid) == 0)
   *grid = FALSE;
else
   {
   *grid = TRUE;
   gettrmat (hosgrid, &ncoeffs, aa, bb);

   t_write ("   Coefficients from grid file", T_STDCC);
   t_write ("    X Values                      Y Values", T_STDCC);
   for (i = 0; i < ncoeffs; i++)
      {
      sprintf (buf, "      %20.8f   %20.8f", aa[i], bb[i]);
      t_write (buf, T_STDCC);
      } 
   for (i = 0; i < 4; i++)
      {
      if (ncoeffs <= dtab[i]) 
         {
         *ndeg = i+1;
         break;
         } 
      } 
   sprintf (buf, "    Polynomial degree is %2d", *ndeg);
   t_write (buf, T_STDCC);
   } 
/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;
}
