/*
***************************************************************************

NAME: GETTRMAT

PURPOSE: Reads transformation coefficients from grid file

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1          2/15/84   K. A. Narayanan
   2          4/19/89   K. A. Narayanan    conversion to 5.0
   3          8/11/89   K. A. Narayanan    copied from GRNDSPOT and
                                           made minor changes

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM/DESCRIPTION:

   get grid from grid file
   get number of coefficients in transformation polynomial
   get transformation coefficients for X and Y

***************************************************************************
*/
#include "pixsert.h"
#include "geompak.h"
#include "protos.h"

void gettrmat
(
    char hostfin[],           /* I: transformation matrix host name   */
    long *ncoeff,             /* O: no. coefficients                  */
    float a[],		      /* O: transformation coefficients for x */
    float b[]                 /* O: transformation coefficients for y */
)

{
struct	GEOGRID	grid;

long	i;
long    fatal_err = -1;

/*
get grid from grid file
-----------------------
*/
if (c_getgrd (hostfin, &grid) != E_SUCC)
   c_errmsg ("Fatal error encountered", "pixsert-fatal", &fatal_err);

/*
get number of coefficients in transformation polynomial
--------------------------------------------------------
*/
*ncoeff = grid.ncoeff;

/*
read transformation coefficients for y and x
--------------------------------------------
*/
for (i = 0; i < *ncoeff; i++) 
   {
   b[i] = grid.coeffs[i];
   a[i] = grid.coeffs[*ncoeff + i];
   }
}

