/*
***************************************************************************

NAME: PROMPT

PURPOSE: Prompts user for block coordinates and values, and transforms
         and validates them

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1          1/01/85    Y. Sheu
    2          7/28/89    K. A. Narayanan    conversion to 5.0
    3          7/29/91    K. Kimball         Allow commas between input
                                             values and allow 'r' for redo.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Runs under TAE

PROJECT:        LAS

***************************************************************************
*/

#include "pixsert.h"
#include "protos.h"

void prompt
(
    char hosin[][CMLEN],/* I: input image host names		*/
    long wind[][4],	/* I: image window			*/
    long otype, 	/* I: output image data type		*/
    char coord[],	/* I: master/realative coordinate option*/
    long array[][4],	/* O: block information			*/
    double gray[],	/* O: block grey levels			*/
    long tblock,	/* I: total no. blocks			*/
    long *nblock, 	/* I: no. blocks				*/
    long *fend,		/* I/O: flag indicating end of data		*/
    long grid, 		/* I: transformation option		*/
    char subcmd[],	/* I: subcommand				*/
    long blksiz[],	/* I: size of blocks			*/
    float aa[], 	/* I: transformation coefficients for x	*/
    float bb[],		/* I: and y				*/
    long ndeg		/* I: degree of trans. polynomial		*/
)
{
CODE	term;

float   line,                   /* line number                          */
        pixel,                  /* pixel position on line               */
        xin,                    /* input x coordinate                   */
        yin;                    /* input y coordinate                   */

double level;                   /* gray value                           */

char	answer[72];             /* prompt OK or REDO                    */

while (1)
   {
   t_write (
   "\nEnter block coordinates and gray levels (Y,X,GRAY) one block per line.",
            T_STDCC);      
   t_write ("Enter <CR> on a new line when done.\n", T_STDCC);

   while (1)
      {
      if (t_read (buf, &term) != SUCCESS)
         {
         t_write ("\nError in input, please reenter:\n", T_STDCC);
         continue;
         }
      if (strlen (buf) == 0) break;
      sscanf (buf, "%f%*c%f%*c%lf", &line, &pixel, &level);

/*
     if grid file given, transform input coordinates.
     ------------------------------------------------
*/
      if (grid)
         {
         xin = pixel;
         yin = line;
         pixel  = fn (ndeg, aa, xin, yin);
         line  = fn (ndeg, bb, xin, yin);
         }
      if (! chkint (level, otype)) 
         {
         if (!setblk (hosin, pixel, line, coord, level, tblock, nblock, array, 
                      gray, wind, subcmd, blksiz, grid)) 
            continue;

         if (*nblock  ==  MAXREC) 
            {
            sprintf (buf, 
               "Stop entering data, we are going to process the %5d blocks", 
                MAXREC);
            t_write (buf, T_STDCC);
            t_write ("you just specified.  You may enter more data later.", T_STDCC);
            break;
            }
         }
      else
	{
         t_write ("Invalid gray level for output data type, please reenter:", 
                  T_STDCC);
	}
      }
  
   if (*nblock == 0)
      {
      t_write ("No point data specified.  Please reenter:", T_STDCC);      
      continue;
      }
   t_write ("Points OK or REDO [Default: OK]? ", T_PROMPT);
   t_read (answer, &term);
   if (answer[0]  ==  'R' || answer[0] == 'r') 
      {
      *nblock = 0;
      continue;
      }
   else         
      {
      *fend = TRUE;
      break;
      }
   }
}                                                              
