#ifndef PROTOS_H
#define PROTOS_H

#define MAXREC 5000

long chkint
(
        double  grey,         /* I: grey level                   */
        long    type          /* I: image type                   */
);

void copy_image
(
    char hosin[][CMLEN],      /* I: input images host names            */
    char hosout[],            /* I/O: output image host name           */
    long num_images,          /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each image      */
    long image_band_num[][MAXBND + 1], /* I: image band numbers        */
    long *otype,              /* I: output image data type             */
    long wind[][4],           /* I: input image windows                */
    long *num_bands           /* I: no. bands                          */
);

void create_mask
(
    long *radius,            /* I: Radius of circle                   */
    long blksiz[2],          /* I: Block size (NL, NS)                */
    long mask[MAXREC][2]     /* O: Block shape mask (SP, NP)          */
);

void filspec
(
    char hosin[][CMLEN],      /* I: input images host names            */
    char hosout[],            /* I: output image host name             */
    long num_images,          /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each image      */
    long *num_bands, 	      /* I: number of image bands              */
    long image_band_num[][MAXBND + 1], /* I: image band numbers        */
    long wind[][4],           /* I: input image windows                */
    long otype,		      /* I: output data type                   */
    long *type                /* I/O:  file type                       */
);

float fn
(
        long    ndeg,   /* I: degree of polynomial         */
        float   cc[],   /* I: polynomial coefficients      */
        float   x,      /* I: x value                      */
        float   y       /* I: Y VALUE                      */
);

void gen_image
(
    char hosin[][CMLEN],      /* I: input images host names            */
    char hosout[],            /* I: output image host name             */
    long num_images,          /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each image      */
    long image_band_num[][MAXBND + 1], /* I: image band numbers        */ 
    long *otype,              /* I: output image data type             */
    long wind[][4],           /* I: input image windows                */
    long *num_bands,          /* I: no. bands                          */
    double *backgrnd          /* I: output image background            */
);

void getpar
(
    char hosin[][CMLEN],     /* O: input image host names              */
    char lasout[],           /* O: output image las name               */
    char hosout[],           /* O: output image host name              */
    long *num_inimages,      /* O: number of input images              */
    long image_band_num[][MAXBND+1],   /* O: image band numbers        */
    long num_image_bands[],  /* O: number of bands in each input image */
    long *num_bands,         /* O: number of image bands               */
    long wind[][4],          /* O: input image window                  */
    char coord_opt[],        /* O: master/relative coordinates option  */
    char subcmd[],           /* O: subcomand                           */
    long blksiz[],           /* O: size of blocks (NL, NS)             */
    long *radius,            /* O: radius of circle                    */
    long bands[],            /* O: bands to contain the blocks         */
    long *out_type,          /* O: output image data type              */
    double *bkgrnd,          /* O: output image background value       */
    char coordfil[],	     /* O: coordinate file host name           */
    long *grid,              /* O: transformation option               */
    long *ndeg,              /* O: degree of transformation polynomial */
    float aa[],              /* O: transformation coeffs for x         */
    float bb[]    	     /* O: transformation coeffs for y         */
);

void gettrmat
(
    char hostfin[],           /* I: transformation matrix host name   */
    long *ncoeff,             /* O: no. coefficients                  */
    float a[],		      /* O: transformation coefficients for x */
    float b[]                 /* O: transformation coefficients for y */
);

void ins_block
(
    char hosout[],       /* I: output image host name               */
    long wind[][4],      /* I: image window                         */
    long num_bands,      /* I: no. bands                            */
    long bands[],        /* I: bands to contain blocks              */
    long blksiz[2],      /* I: size of full size block              */
    long mask[][2],      /* I: shape of block to insert             */
    long *otype,         /* I: output image data type               */
    long array[][4],     /* I: block information                    */
    double gray[],       /* I: grey levels for blocks               */
    long *nblock         /* I: no. blocks                           */
);

void prompt
(
    char hosin[][CMLEN],/* I: input image host names		*/
    long wind[][4],	/* I: image window			*/
    long otype, 	/* I: output image data type		*/
    char coord[],	/* I: master/realative coordinate option*/
    long array[][4],	/* O: block information			*/
    double gray[],	/* O: block grey levels			*/
    long tblock,	/* I: total no. blocks			*/
    long *nblock, 	/* I: no. blocks				*/
    long *fend,		/* I/O: flag indicating end of data		*/
    long grid, 		/* I: transformation option		*/
    char subcmd[],	/* I: subcommand				*/
    long blksiz[],	/* I: size of blocks			*/
    float aa[], 	/* I: transformation coefficients for x	*/
    float bb[],		/* I: and y				*/
    long ndeg		/* I: degree of trans. polynomial		*/
);

void rdfile
(
    char coordfil[],    /* I: coordinate file host name         */
    char hosin[][CMLEN],/* I: input image host name		*/
    long wind[][4],	/* I: image window		        */
    long otype, 	/* I: output image data type		*/
    char coord[],	/* I: master/relative coordinate option	*/
    long array[][4], 	/* O: block information			*/
    double gray[],	/* O: block gray levels			*/
    long tblock, 	/* I: total no. blocks			*/
    long *nblock, 	/* I: no. blocks			*/
    long *fend, 	/* I/O: flag indicating end of data	*/
    long grid, 		/* I: transformation option		*/
    char subcmd[],	/* I: subcommand			*/
    long blksiz[], 	/* I: block size			*/
    float aa[], 	/* I: transformation coeffs for x       */
    float bb[],         /* I: transformation coeffs for y       */ 
    long ndeg		/* I: degree of trans. polynomial	*/
);

long setblk
(
        char    hosin[][CMLEN], /* I: input image host name                */
        float   pixel,          /* I: pixel coordinate                     */
        float   line,           /* I: line coordinate                      */
        char    coord[],        /* I: master/relative coord. option        */
        double  level,          /* I: grey level for current block         */
        long    tblock,         /* I: total no. blocks                     */
        long    *nblock,        /* I: no. blocks                           */
        long    array[][4],     /* O: block information                    */
        double  gray[],         /* O: block grey levels                    */
        long    wind[][4],      /* I: image window                         */
        char    subcmd[],       /* I: subcommand                           */
        long    blksiz[],       /* I: block size                           */
        long    grid            /* I: transformation option                */
);

void terminate
(
    char las_out[],         /* I: output image host name               */
    char hos_out[],         /* I: output image host name               */
    char hos_in[][CMLEN],   /* I: input images host names              */
    long *num_inimages,     /* I: number of input images               */
    long *num_bands,        /* I: number of bands                      */
    long wind[][4]          /* I: input image windows                  */
);

#endif
