/*
***************************************************************************

NAME: RDFILE

PURPOSE: Reads coordinate file for block coordinates and values,
         and transforms and validates them

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1          1/01/85    Y. Sheu
    2          7/28/89    K. A. Narayanan    conversion to 5.0
    3          7/29/91    K. Kimball         Allow commas between input
                                             values.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Runs under TAE

PROJECT:        LAS

***************************************************************************
*/

#include "pixsert.h"
#include "protos.h"

void rdfile
(
    char coordfil[],    /* I: coordinate file host name         */
    char hosin[][CMLEN],/* I: input image host name		*/
    long wind[][4],	/* I: image window		        */
    long otype, 	/* I: output image data type		*/
    char coord[],	/* I: master/relative coordinate option	*/
    long array[][4], 	/* O: block information			*/
    double gray[],	/* O: block gray levels			*/
    long tblock, 	/* I: total no. blocks			*/
    long *nblock, 	/* I: no. blocks			*/
    long *fend, 	/* I/O: flag indicating end of data	*/
    long grid, 		/* I: transformation option		*/
    char subcmd[],	/* I: subcommand			*/
    long blksiz[], 	/* I: block size			*/
    float aa[], 	/* I: transformation coeffs for x       */
    float bb[],         /* I: transformation coeffs for y       */ 
    long ndeg		/* I: degree of trans. polynomial	*/
)
{
float	
        line,                   /* image line                           */ 
        pixel,                  /* pixel number on image line           */ 
        xin,                    /* image line for coordinate transform  */ 
        yin;                    /* pixel number for coordinate transform*/

double  level;                  /* gray level for region                */
 
long	i;                      /* loop counter                         */

FILE *fd_pf;

long    inf_err = 1;

fd_pf = fopen (coordfil, "r");

t_write ("Reading input coordinates file", T_STDCC);
for (i = 0; i < MAXREC; i++)
   {
   if (fscanf (fd_pf, "%f%*c%f%*c%lf", &line, &pixel, &level) == EOF) 
      {
      *fend = TRUE;      
      break;
      }
/*
   if grid file given, transform input coordinates.
   -----------------------------------------------
*/
   if (grid)
      {
      yin = line;
      xin = pixel;
      pixel = fn (ndeg, aa, xin, yin);
      line = fn (ndeg, bb, xin, yin);
      } 

   if (chkint (level, otype)) 
      {
      sprintf (buf, "Data type of value for block %d invalid", tblock+i+1); 
      c_errmsg (buf, "pixsert-dtype", &inf_err);
      }
   else
      setblk (hosin, pixel, line, coord, level, tblock, nblock, array, gray, 
              wind, subcmd, blksiz, grid);
   }

}                                                              
