/*
***************************************************************************

NAME: SETBLK

PURPOSE: Validates block information against image size

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
    1          1/01/85    Y. Sheu
    2          7/28/89    K. A. Narayanan    conversion to 5.0
    3          7/29/91    K. Kimball         Removed unused variables
                                             nump and numl.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Runs under TAE

PROJECT:        LAS

***************************************************************************
*/

#include "pixsert.h"
#include "protos.h"

long	setblk 
(
        char    hosin[][CMLEN], /* I: input image host name                */
        float   pixel,          /* I: pixel coordinate                     */
        float   line,           /* I: line coordinate                      */
        char    coord[],        /* I: master/relative coord. option        */
        double  level,          /* I: grey level for current block         */
        long    tblock,         /* I: total no. blocks                     */
        long    *nblock,        /* I: no. blocks                           */
        long    array[][4],     /* O: block information                    */
	double  gray[],		/* O: block grey levels			   */
        long    wind[][4],      /* I: image window                         */
        char    subcmd[],       /* I: subcommand                           */
        long    blksiz[],       /* I: block size                           */
        long    grid            /* I: transformation option                */
)
{
struct	DDR ddr;

static	long hcnter,            /* horizontal center of region          */
             nlout,             /* number of lines                      */
             nsout,             /* number of samples                    */
             slout,             /* starting line                        */
             ssout,             /* starting sample                      */
             vcnter,            /* vertical center of region            */
             xoff,              /* x offset                             */
             xxoff,             /* x offset from ddr                    */
	     yoff,              /* y offset                             */
             yyoff;             /* y offset from ddr                    */

long	es,                     /* ending sample                        */
        el,                     /* ending line                          */
        ibcnt,                  /* partial block flag                   */
        ilevel,                 /* gray level                           */
        iline,                  /* intermediate value for line          */
        ipixel,                 /* intermediate value for pixel         */
        numlin,                 /* number of lines                      */
        numsam,                 /* number of samples                    */
	sl,                     /* starting line                        */
        ss,                     /* starting sample                      */
        val,                    /* success flag                         */
        stat;                   /* status flag                          */

long    inf_err = 1;

/*
get offset of master coordinates 
--------------------------------
*/
if (*nblock == 0)
   {
   if (subcmd[0]  ==  'I'  &&  coord[0]  ==  'M') 
      {   
      stat = c_getddr ((char *)hosin, &ddr);
      if (stat != E_SUCC )
         c_errmsg("Nonfatal error encountered","pixsert-warn",&inf_err);
         
      xxoff=ddr.master_sample;
      yyoff=ddr.master_line;
      if (xxoff  <=  0) xxoff = 1;
      if (yyoff  <=  0) yyoff = 1;
      xoff=xxoff+wind[0][SS]-1;
      yoff=yyoff+wind[0][SL]-1;
      }
   else
      {
      xoff=1;
      yoff=1;
      }  

/*
   compute image boundaries in terms of master coordinates
   -------------------------------------------------------
*/
   ssout = 1;
   slout = 1;
   nsout = wind[0][NS];
   nlout = wind[0][NL];
   vcnter = (blksiz[0]-1)/2;
   hcnter = (blksiz[1]-1)/2;
   }
/*
Allow for type differences for line, sample, and gray level
-----------------------------------------------------------
*/
iline = (long)line;
if ((float) iline  <  line) iline++;
ipixel = (long)pixel;                                               
if ((float) ipixel  <  pixel) ipixel++;
if (level < 0)
   ilevel = (long)(level - 0.00005);
else 
   ilevel = (long)(level + 0.00005);                                         
sl = iline - vcnter - yoff + 1;
ss = ipixel - hcnter - xoff + 1;
el = sl + blksiz[0]  - 1;
es = ss + blksiz[1] - 1;
if ((el  <  slout)  ||  (sl  >  nlout)  || (es  <  ssout)  ||  (ss  >  nsout)) 
   {
   val = FALSE;
   if (grid)
      {
      sprintf (buf, "Computed line, sample values = %f %f", line, pixel);
      c_errmsg (buf, "informational", &inf_err);
      }
   sprintf (buf, "Block %d completely outside the image. Respecify.\r\n", 
            *nblock+tblock+1); 
   c_errmsg (buf, "pixsert-noblk", &inf_err);
   } 
else
   {
   ibcnt=0;
   if (sl  <  slout) 
      {
      ibcnt++;
      sl = slout;
      } 
   if (el  >  nlout) 
      {
      ibcnt++;
      el = nlout;                                        
      }  
   if (ss  <  ssout) 
      {
      ibcnt++; 
      ss = ssout;
      }  
   if (es  >  nsout) 
      {
      ibcnt++;
      es = nsout;                                        
      }  
   numsam = es - ss + 1;                                                
   numlin = el - sl + 1;
   val = TRUE;
   if (ibcnt >  0) 
      {
      if (grid)
         {
         sprintf (buf, "Computed line, sample values = %f %f", line, pixel);
         c_errmsg (buf, "informational", &inf_err);
         }
      sprintf (buf, "Warning: Block %d only partially contained in image\r\n", 
               *nblock+tblock+1);
      c_errmsg (buf, "pixsert-partblk", &inf_err);
      }
   gray[*nblock] = ilevel;
   array[*nblock][SS] = ss; 
   array[*nblock][SL] = sl; 
   array[*nblock][NS] = numsam;
   array[*nblock][NL] = numlin;
   (*nblock)++;
   }  
return (val);
}                                                                  
