/*****************************************************************************
*
*  NAME:  GETPRNM
*
*  FUNCTION:  Get a permutation number.
*
*  PROGRAM HISTORY:
*  Version	Date	Programmer	Code	Request
*  -------	----	----------	----	-------
*    5.0       11/89	R.McKinney	DDPS	LAS 5.0 Conversion
*    5.1        7/91    T.MITTAN        LAS     Convert to supported code
*                                               and improve performance.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*  This algorithm places the numbers in proper order by using the input
*  class as a index to the record perm.  Then the count is incremeted,
*  the pixel value is added to the sum.  The square of the pixel value is
*  added to the perm.std for the standard deviation.  The input pixel value
*  is used as an index to perm[].list to increment the list value.  The value
*  in perm[3].list[120] is the number of pixel values from class (3 + minimum 
*  class value) with value 120.  Finally, the min and max are found.
*  NOTE: An offset value is used for setting 
*        perm[class - offset].list[value-offset1]. 
*  Offset value is minimum class number in the classified image.  Offset1 
*  is the minimum value in the image.  
*  
******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "pixstat.h"
#include "protos.h"

void get_permutation_num
(
    struct RECORD *perm,		/* permutation records			*/
    long *curatt,			/* current attributes			*/
    long *minv                     /* minimum offset                       */
)
{


  perm[curatt[0]].count++;
  perm[curatt[0]].sum += curatt[1];  /*add to sum */
  perm[curatt[0]].std += (curatt[1]*curatt[1]); /*add to square of 
                                                            the sums */
  perm[curatt[0]].list[(curatt[1] - minv[1])]++;
  if (perm[curatt[0]].max1 < curatt[1])
      perm[curatt[0]].max1 = curatt[1];
  if (perm[curatt[0]].min1 > curatt[1])
      perm[curatt[0]].min1 = curatt[1];
return;
}
