/******************************************************************************
*
*  NAME:	INITLT
*
*  FUNCTION: Open the Labeled table and write the vector records.
*
*  PROGRAM HISTORY:
*  Version	Date	Author		CODE	Request
*  -------	----	------ 		----	-------
*    1.0        11/89	R.McKinney	DDPS	LAS 5.0 Conversion
*    5.1         7/91   T.Mittan        LAS     convert to baseline code
*                                               and improve performance.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
*		Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*	Initialize variables for opening the LT. 
*
******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "pixstat.h"
#include "protos.h"

void initlt
(
    struct TAB_DEF *tab,		/* Labeled table descriptor */
    char *lthos,			/* labeled table host file name */
    char *statcalc			/* statistical option		*/
)

{
long   non_fatal = 1,		/* nonfatal error message		*/
       acc;        

/*  Initialize variables needed to open the output labeled table file.
----------------------------------------------------------------------*/
tab->nsubs = 0;
tab->subfile[0] = '\0';
tab->ftype = "PERM";
tab->fdesc = "File generated from PIXSTAT";
acc = IWRITE;
/* Give an informational message; depending on the number
of permutations this could take awhile
---------------------------------------------------------*/
c_errmsg("Creating the labeled table","informational",&non_fatal);

/*  Open the output labeled table file
--------------------------------------*/
open_tab(tab,lthos,acc);

/*  Define each vector fields
----------------------------*/
put_vector(tab,"PIXEL",I4,"PIXSTAT pixel value",1,1,FIELD_SEP2);
put_vector(tab,"BAND",I4,"PIXSTAT band number",1,1,FIELD_SEP2);
if (strcmp(statcalc,"MEAN") == 0)
    {
    put_vector(tab,"NPIXELS",I4,"number of pixels",1,1,FIELD_SEP2);
    put_vector(tab,"MEAN",R4,"mean value",1,1,FIELD_SEP2);
    put_vector(tab,"STD",R8,"standard deviation",1,1,RECORD_SEP);
    }
else if (strcmp(statcalc,"MAX") == 0)
    {
    put_vector(tab,"MAX",I4,"maximum pixel value",1,1,FIELD_SEP2);
    put_vector(tab,"MIN",I4,"minimum pixel value",1,1,RECORD_SEP);
    }
else if (strcmp(statcalc,"MODE") == 0)
    {
    put_vector(tab,"NPIXELS",I4,"number of pixels",1,1,FIELD_SEP2);
    put_vector(tab,"MODE",I4,"mode value",1,1,RECORD_SEP);
    }
else if (strcmp(statcalc,"MED") == 0)
    {
    put_vector(tab,"NPIXELS",I4,"number of pixels",1,1,FIELD_SEP2);
    put_vector(tab,"MED",R4,"median value",1,1,FIELD_SEP2);
    put_vector(tab,"MAD",R4,"Median Absolute Deviation",1,1,RECORD_SEP);
    }
else if (strcmp(statcalc,"MAD") == 0)
    {
    put_vector(tab,"NPIXELS",I4,"number of pixels",1,1,FIELD_SEP2);
    put_vector(tab,"MED",R4,"median value",1,1,FIELD_SEP2);
    put_vector(tab,"MAD",R4,"Median Absolute Deviation",1,1,RECORD_SEP);
    }
else if (strcmp(statcalc,"ALL") == 0)
    {
    put_vector(tab,"NPIXELS",I4,"number of pixels",1,1,FIELD_SEP2);
    put_vector(tab,"MEAN",R4,"mean value",1,1,FIELD_SEP2);
    put_vector(tab,"STD",R8,"standard deviation",1,1,FIELD_SEP2);
    put_vector(tab,"MAX",I4,"maximum pixel value",1,1,FIELD_SEP2);
    put_vector(tab,"MIN",I4,"minimum pixel value",1,1,FIELD_SEP2);
    put_vector(tab,"MODE",I4,"mode value",1,1,FIELD_SEP2);
    put_vector(tab,"MED",R4,"median value",1,1,FIELD_SEP2);
    put_vector(tab,"MAD",R4,"median absolute deviation",1,1,RECORD_SEP);
    }
    
return;
} 
