/*****************************************************************************
*
*  NAME:  MEDOPT
*
*  FUNCTION: Create  the Labeled Table
*
*  PROGRAM HISTORY:
*  Version	Date	Programmer	Code	Request
*  -------	----	----------	----	-------
*   5.0	       11/89	R.McKinney 	DDPS	LAS 5.0 Conversion
*   5.1         7/91    T.Mittan        LAS     Convert to baseline code
*                                               and improve performance.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
* 	Must be run under TAE
*
*  PROJECT:  LAS
*
*  ALOGRITHM DESCRIPTION:
*  Loop by class and then by pixel value.  Find the number of pixel values
*  in that class.  Divide by two and add one.  This should be the median
*  value.  If count is even average the two middle values. Write the lable
*  table and loop to next class.
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "pixstat.h"
#include "protos.h"

int medopt
(
    struct TAB_DEF *tab,		/* labeled table descriptor	*/
    long *bnd,			/* total number of bands        */
    struct RECORD *perm,            /* permutation attributes       */
    long *class,                    /* class number                 */
    long *minv,                     /* minimum value offset         */
    long *pixvalue,                 /* pixel value                  */
    long *pxcnt,                    /* number of pixels             */
    long *pixflg,                   /* TRUE if pixvalue option used */
    long *range                    /* Range of values              */
)

{


long i,j,y,k,			/* loop counters		*/
     temp,                      /* temporary holding value      */
     non_fatal= 1;              /* non_fatal flag               */

float *med;			/* med values per band         */
float mid;			/* mid  values per band         */
float med_mad;

long *mad;

struct VECTOR *ptr;		/* pointer to current vector */
struct MATRIX size;		/* matrix size */


/* Initialize the needed variables
----------------------------------*/

size.nrow = 1;
size.ncol = 1;


if ((med = (float *)calloc((*class),(sizeof(float)))) == NULL)
    {
    c_errmsg("error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }

if ((mad = (long *)calloc((2 * *range),(sizeof(long)))) == NULL)
    {
    c_errmsg("Error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }

/* Find median
------------*/
if (*pixflg == 0)
  {
  for (j = 0; j < *class; j++)
    {
    if (perm[j].count > 0)
    {
    mid =  (perm[j].count/2.0) ;
    for(i = 0, y = 0;  (float) i <  mid; y++)
      {
      if (perm[j].list[y] > 0)
        {
        med[j] = (float) y ;
        i += perm[j].list[y];
        if (((float)i <= mid) && (perm[j].count%2) == 0)
           {
           for (k = y + 1; k < *range; k++)
              {
              if (perm[j].list[k] > 0 )
                {
                temp = k;
                break;
                }
              }
           }
        else 
          temp = y;
        }
      }
    med[j] = (med[j] + (float) temp)/2;

     /* Calculate Median Absolute Deviation
     ------------------------------------*/
     for (i = 0; i <(2 *  *range); i++)
         mad[i] = 0;
     for (i = 0; i < *range; i++)
       {
       if (perm[j].list[i] > 0)
          mad[abs( (long)(2 * (med[j] - (float) i)))] += perm[j].list[i];
       }
     for (i = 0,y = 0; i <(2 *  *range); i++)
       {
       if (mad[i] > 0)
          {
          if ((float)y < mid)
            {
            med_mad =  (float) i ;
            y += mad[i];
            if (((float)y<=mid) && ((perm[j].count % 2) == 0))
               {
               for (k = i + 1; k < *range; k++)
                  {
                  if (mad[k] > 0 )
                    {
                    temp = k;
                    break;
                    }
                  }
               }
            else
               temp = i ;
            }
          }
       }

     med_mad = ((med_mad + (float) temp)/4);
     med[j] += (float) minv[1];

   /* Build the Labeled Table.
   -------------------------*/
   /* Read in the next record
   --------------------------*/
		ptr = tab->vector;
                temp = j + 1;
    		put_matrix(ptr,I4,&size,&temp);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,bnd);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[j].count);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&med[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&med_mad);
    		put_record(tab);
   }
  }
 }
else
  {
  for (j = 0; j < *pxcnt; j++)
    {
    if (perm[pixvalue[j]].count >0)
    {
    mid =  (perm[pixvalue[j]].count/2.0);
    for(i = 0, y = 0;  (float)i <  mid; y++)
      {
      med[pixvalue[j]] = (float) y ;
      i += perm[pixvalue[j]].list[y];
      if (((float)i <= mid) && (perm[pixvalue[j]].count%2) == 0)
         {
         for (k = y + 1; k < *range; k++)
            {
            if (perm[pixvalue[j]].list[k] > 0 )
              {
              temp = k;
              break;
              }
            }
         }
      else 
        temp = y;
    }
    med[pixvalue[j]] = (med[pixvalue[j]] + (float) temp)/2;

     /* Calculate Median Absolute Deviation
     ------------------------------------*/
     for (i = 0; i <(2 *  *range); i++)
         mad[0] = 0;
     for (i = 0; i < *range; i++)
       {
       if (perm[pixvalue[j]].list[i] > 0)
          mad[abs( (long)(2 * (med[pixvalue[j]] - (float) i)))] +=
                                                perm[pixvalue[j]].list[i];
       }
     for (i = 0,y = 0; i <(2 *  *range); i++)
       {
       if (mad[i] > 0)
          {
          if ((float) y < mid)
            {
            med_mad =  (float) i ;
            y += mad[i];
            if (((float)y<=mid) && ((perm[pixvalue[j]].count % 2) == 0))
               {
               for (k = i + 1; k < *range; k++)
                  {
                  if (mad[k] > 0 )
                    {
                    temp = k;
                    break;
                    }
                  }
               }
            else
               temp = i ;
            }
          }
       }

     med_mad = ((med_mad + (float) temp)/4);
     med[pixvalue[j]] += (float) minv[1];

   /* Build the Labeled Table.
   -------------------------*/
   /* Read in the next record
   --------------------------*/
		ptr = tab->vector;
    		put_matrix(ptr,I4,&size,&pixvalue[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,bnd);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[pixvalue[j]].count);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&med[pixvalue[j]]);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&med_mad);
    		put_record(tab);
        }
      }
}
free(med);
free(mad);

return(E_SUCC);
}
