/*****************************************************************************
*
*  NAME:  MENOPT
*
*  FUNCTION: Create the translation table and the Labeled Table
*
*	
*  PROGRAM HISTORY:
*  Version	Date	Programmer	Code	Request
*  -------	----	----------	----	-------
*   5.0	       11/89	R.McKinney 	DDPS	LAS 5.0 Conversion
*   5.1         7/91    T.Mittan        LAS     convert to baseline code
*                                               and improve performance.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
* 	Must be run under TAE
*
*  PROJECT:  LAS
*
*  ALOGRITHM DESCRIPTION:
*  Find in the mean = sum/count. Find the standard deviation by the 
*  sample formula.  Write the values to the LT file and then repeat for 
*  next class. 
*
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "ltable.h"
#include "pixstat.h"
#include "protos.h"

int menopt
(
    struct TAB_DEF *tab,		/* labeled table descriptor	*/
    long *bnd,			/* total number of bands	*/
    struct RECORD *perm,           /* permutation count            */
    long *class,                    /* total number of classes      */
    long *pixval,                   /* pixel values (class values)  */
    long *pxcnt,                    /* number of class values       */
    long *pixflg,                   /* true if pixel values specified */
    long *minv                     /* minimum offset               */
)
{
long  j,                        /* loop counter                 */
      temp,                     /* temperary holding value      */
      non_fatal = 1;            /* nonfatal flag                */

float *mean; 			/* mean values per band         */

double stddev;			/* standard deviation holding   */

struct VECTOR *ptr;		/* pointer to current vector */
struct MATRIX size;		/* matrix size */


/* Initialize the needed variables
----------------------------------*/

size.nrow = 1;
size.ncol = 1;


if ((mean = (float *)calloc((*class),(sizeof(float)))) == NULL)
    {
    c_errmsg("error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }

if (*pixflg == 0)
  {
  /*Find mean and standard deviation.
  ---------------------------------*/
  for (j = 0; j < *class; j++)
       {
       if (perm[j].count > 0)
       {
       mean[j] = perm[j].sum / perm[j].count;
       stddev = (perm[j].std - (float)perm[j].count * (mean[j] * mean[j]));
       perm[j].std = sqrt(fabs(stddev / perm[j].count));
       /* Build the Labeled Table.
       -------------------------*/
		ptr = tab->vector;
                temp = j + minv[0];
    		put_matrix(ptr,I4,&size,&temp);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,bnd);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[j].count);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&mean[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,R8,&size,&perm[j].std);
    		put_record(tab);
        }
     }
  }
else
  {
  /*Find mean and standard deviation.
  ---------------------------------*/
  for (j = 0; j < *pxcnt; j++)
       {
       if (perm[pixval[j]].count > 0 )
       {
       mean[pixval[j]] = perm[pixval[j]].sum / perm[pixval[j]].count;
       stddev = (perm[pixval[j]].std - (float) perm[pixval[j]].count *
                (mean[pixval[j]] * mean[pixval[j]]));
       perm[pixval[j]].std = sqrt(fabs(stddev / perm[pixval[j]].count));
       /* Build the Labeled Table.
       -------------------------*/
		ptr = tab->vector;
    		put_matrix(ptr,I4,&size,&pixval[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,bnd);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&perm[pixval[j]].count);
    		ptr = ptr->next;
    		put_matrix(ptr,R4,&size,&mean[pixval[j]]);
    		ptr = ptr->next;
    		put_matrix(ptr,R8,&size,&perm[pixval[j]].std);
    		put_record(tab);
         }
       }
   }
free(mean);
return(E_SUCC);
}
