/*****************************************************************************
*
*  NAME:  MODOPT
*
*  FUNCTION: Create the translation table and the Labeled Table
*
*  PROGRAM HISTORY:
*  Version	Date	Programmer	Code	Request
*  -------	----	----------	----	-------
*   5.0	       11/89	R.McKinney 	DDPS	LAS 5.0 Conversion
*   5.1         7/91    T.Mittan        LAS     Convert to baseline code
*                                               and improve performance.
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
* 	Must be run under TAE
*
*  PROJECT:  LAS
*
*  ALOGRITHM DESCRIPTION:
*  Loop by class and then by pixel value.  For each class find the pixel value
*  that occurs most often.  This is the mode.  Write out the LT table and loop
*  to next class.
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "pixstat.h"
#include "protos.h"

int modopt
(
    struct TAB_DEF *tab,		/* labeled table descriptor	*/
    long *bnd,			/* total number of bands	*/
    struct RECORD *perm,            /* permutation attributes       */
    long *class,                    /* number  of classes           */
    long *range,                    /* range of values              */
    long *minv,                     /* minimum value offset         */
    long *pixvalue,                 /* pixel values                 */
    long *pxcnt,                    /* number of pixel values       */
    long *pixflg                   /* TRUE if pixel value option used */
)

{

long *count,			/* loop counter			*/
     i,j,			/* loop counters		*/
     *mode,			/* mode values per band         */
     temp,                      /* tempary holding value        */
     non_fatal = 1;             /* nonfatal flag                */


struct VECTOR *ptr;		/* pointer to current vector */
struct MATRIX size;		/* matrix size */


if ((mode = (long *)calloc((*class),(sizeof(long)))) == NULL)
    {
    c_errmsg("error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }
if ((count = (long *)calloc((*class),(sizeof(long)))) == NULL)
    {
    c_errmsg("error allocating dynamic memory","pixstat-alloc",&non_fatal);
    return(E_FAIL);
    }

/* Initialize the needed variables
----------------------------------*/

size.nrow = 1;
size.ncol = 1;
 
/* Find mode 
----------*/
if (*pixflg == 0)
{
   for (j = 0; j < *class; j++)
     {
     if (perm[j].count > 0)
      {
     for (i = 0; i < *range; i++)
      {
      if (count[j] < perm[j].list[i])
        {
        count[j] = perm[j].list[i];
        mode[j] = i;
        }
      }
     mode[j] += minv[1];	
  /* Build the Labeled Table.
  -------------------------*/
   /* Read in the next record
   --------------------------*/
		ptr = tab->vector;
                temp = j + minv[0];
    		put_matrix(ptr,I4,&size,&temp);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,bnd);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&count[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&mode[j]);
    		put_record(tab);
     }
   }
  }
else
{
   for (j = 0; j < *pxcnt; j++)
     {
     if (perm[pixvalue[j]].count > 0 )
     {
     for (i = 0; i < *range; i++)
      {
      if (count[pixvalue[j]] < perm[pixvalue[j]].list[i])
        {
        count[pixvalue[j]] = perm[pixvalue[j]].list[i];
        mode[pixvalue[j]] = i;
        }
      }
     mode[pixvalue[j]] += minv[1];
  /* Build the Labeled Table.
  -------------------------*/
   /* Read in the next record
   --------------------------*/
		ptr = tab->vector;
    		put_matrix(ptr,I4,&size,&pixvalue[j]);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,bnd);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&count[pixvalue[j]]);
    		ptr = ptr->next;
    		put_matrix(ptr,I4,&size,&mode[pixvalue[j]]);
         }
   }
}
free(mode);
return(E_SUCC);
}
