/*****************************************************************************
*
*   NAME: OPNLAS 
*
*   FUNCTION:  Open input and output images 
*
*   PROGRAM HISTORY:
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     5.0      11/89	R.McKinney	DDPS	LAS 5.0 conversion
*     5.1       7/91    T.Mittan        LAS     Convert to baseline code
                                                and improve performance. 
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRITPION:
*   For the first pass, open the input image(s).
*
******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "typlim.h"
#include "pixstat.h"
#include "protos.h"

int opnlas
(
    char hosin[][CMLEN],		/* input host file names	*/
    long (*wind)[4],		/* window specifications for each input image */
    long (*bands)[MAXBND + 1],	/* bands to be processed for all images */
    long *nbands,			/* number of bands for input images*/
    long *nmrimg,			/* number of input images	*/
    long *nl,			/* number of lines		*/
    long *ns,			/* number of samples		*/
    long *sl,			/* starting line in input image window	*/
    long *ss,			/* starting sample in input image window */
    long *bandnum                  /* band number                  */
)
{
/* The followin variables have been declared global for image I/O routines
--------------------------------------------------------------------------*/
extern struct FDESC *fdesc[MAXIMG+1]; 
                                  /* image file descriptors used for image I/O*/
extern long offset1[MAXIMG*MAXBND];/* offsets for input bands		*/
       
static  long    opt[3]={0,0,0}; /* optional parameter array for EOPENS     */

long  access = IREAD,		/* file access			*/
      band,			/* band counter			*/ 
      image,			/* image counter		*/
      dtype = ELONG,		/* data type			*/
      status,			/* function return status	*/
      temp,			/* temporary band variable	*/
      zero = 0;			/* zero constant		*/

/* Open the input images
------------------------*/
   *sl = wind[0][SL];
   *ss = wind[0][SS];
   *nl = wind[0][NL];
   *ns = wind[0][NS];

      status = c_eopens(&fdesc[0],hosin[0],bands[0],sl,ss,nl,ns,
                        &offset1[0],&access,&dtype,&zero,opt);
      if (status != E_SUCC)
         return(E_FAIL);
      band = nbands[0];
      (*bandnum)++;
      temp = *bandnum;
      *bandnum -= band;
   if ( *bandnum > 0)
    {
    for (image = 1; image < *nmrimg; image++)
      {
      if (*bandnum <= nbands[image])
       {
       status = c_eopens(&fdesc[1],hosin[image],bands[image],
       &wind[image][SL],&wind[image][SS],&wind[image][NL],&wind[image][NS],
                        &offset1[1],&access,&dtype,&zero,opt);
       if (status != E_SUCC)
         return(E_FAIL);
       break;
       }
    else
      {
      band += nbands[image];
      *bandnum -= nbands[image];
      }
    }
   }
 else
   {
   *bandnum = temp - 1;
   fdesc[1] = 0;
   }
return(E_SUCC);
}
