/*****************************************************************************
*
*   NAME:  PIXSTAT
*
*   FUNCTION:  Compare corresponding pixels from multiple input images and
*              generate a Labeled Table.
*              Each entry in the Labeled Table stores a permutation 
*              code, a count of the number of pixels that have been assigned
*              that permutation value, and a list of the input image 
*              pixel values that compose the permutation.
*
*   PROGRAM HISTORY:
*   Version	Date	Programmer	Code	Request
*   -------	----	----------	----	-------
*     5.0       11/89	R.McKinney	DDPS	LAS 5.0 conversion
*     5.1        7/91   T.Mittan        LAS     Convert to baseline code
*                                               and improve performance.
*     5.2	 9/92	T. Mittan	LAS	Added extern to curent and
*						curcat to eliminate warnings on
*						link
*     5.3       12/92   D. Etrheim      EDC     Correct problem when using
*                                               windows
*     5.4	 5/93	T. Mittan	EDC	Added Median Absolute 
*						Deviation (MAD) option
*     5.5	 6/93	T. Mittan	EDC	Modified Median Absolute 
*						Deviation (MAD) option
*     6.0        3/95   D. Eichacker    EDC     Modified getpar to allow
*						Different starting line
*						and sample to be specified.
*     7.0        5/00   D. Hames        EDC     Modify proces to allocate enough
*                                               memory for permutation records.
*                                               CR#3698.
*
*   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		Must be run under TAE
*
*   PROJECT: LAS
*
*   ALGORITHM DESCRIPTION:
*	Initial the processing session.  Get the input parameters and do
*   validity checking. Process the images by overlaying and creating 
*   permutation codes based on the input pixel values.   
*   Create a Labeled Table. Issue a completion message.
*
******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "pixstat.h"
#include "ltable.h"
#include "protos.h"

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
struct FDESC * fdesc[MAXIMG+1];	
                               /* image file descriptors used for image I/O	*/
struct GDESC * gdesc;		   /* group descriptor used for image I/O		*/
long offset1[MAXIMG*MAXBND];/* offsets for input bands			*/
long offset2[MAXBND];	/* offsets for output				*/
extern long curent;	/* current line being processed			*/
extern long curcat;     /* current catagorey being processed            */

int main
(void)
{ 
struct PARBLK vblock;			/* TAE parameter block		*/

char  hosin[MAXIMG][CMLEN],		/* input host file names	*/
      hosout[1][CMLEN],			/* output list name	        */
      lthos[CMLEN],			/* output LT host file name	*/
      statcalc[5],		        /* statistical option           */
      taeout[2][CMLEN];			/* output TAE names		*/

static long bands[MAXIMG][MAXBND+1];	/* bands to be processed	*/

long  fatal = -1,			/* for fatal error messages	*/
      maskval[100], 			/* mask values for each band	*/ 
      msgtim,				/* message timing value		*/
      imgflg = FALSE,			/* flag if output image 	*/
      mskflg = FALSE,			/* flag if masking is to take place */
      pixflg = FALSE,			/* flag if masking is to take place */
      ltflg = FALSE,                    /* flag if output labled table  */
      mskcnt,				/* number of mask values        */
      pixcnt,				/* number of pixel values	*/
      nbands[MAXIMG],			/* number of bands for each image */
      nmrimg,				/* number of images being processed */
      nout = 1,				/* number of output files	*/
      pixval[100],	    	        /* pixel values for each band	*/ 
      std_out,				/* standard output		*/
      totbnds,				/* total number of bands	*/
      wind[MAXIMG][4];			/* window specification for each image*/

double minval[MAXBND],
       maxval[MAXBND];
      
/*Initialize LAS processing
---------------------------*/
c_inlas(&vblock,&std_out);

/*Get the input parameters
-------------------------*/
if (getpar(&vblock,hosin,hosout[0],lthos,nbands,wind,maskval,bands,&totbnds,
           pixval,statcalc,&nmrimg,&mskflg,&pixflg,&imgflg,&msgtim,
	   &pixcnt,&mskcnt,minval,maxval,&ltflg) != E_SUCC)
   c_errmsg("Fatal error encountered","pixstat-fatal",&fatal);

/* Proces the images
--------------------*/
if (proces(&vblock,hosin,hosout,lthos,wind,bands,nbands,&totbnds,&nmrimg,
          maskval,pixval,statcalc,&mskflg,&pixflg,
          &pixcnt,minval,maxval,&msgtim,&imgflg,&ltflg) != E_SUCC)
   c_errmsg("Fatal error encountered","pixstat-fatal",&fatal);

/* Set up TAE names for completion message
------------------------------------------*/
strcpy(taeout[0], lthos);
if (imgflg)
   {
   strcpy(taeout[1], hosout[0]);
   nout += 1;
   }
c_complt("PIXSTAT",taeout,&nout);

return EXIT_SUCCESS;

} 
