/*******************************************************************************
NAME				CHKWIND

PURPOSE		Validate an image window

PROGRAM HISTORY
	PROGRAMMER	  DATE		REASON
	----------	  ----		------
	D. Hollaren	 10/88		Modified version of C_WNDCHK
					
PROJECT				LAS

ALGORITHM 
	open the image in random mode and get the number of lines and samples
	if the window is 0 0 0 0 then return 1 1 #lines and #samples
	otherwise check that the starting line <= #lines
				 starting sample <= #samples
				 starting line + number lines -1 <= #lines
				 starting sample + number samples -1 <= #samps

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"

int chkwind
(
    long inwind[],	/* Input window to be checked	*/
    long vlwind[],	/* Valid window			*/
    long *nl,
    long *ns	/* Number of lines and samples  */
)
	{

		long	sl=1,		/* a starting line req. by imageio */
			ss=1;		/* a starting samp req. by imageio */

		long	nonfatal=1;	/* non-fatal error message code	   */

		char	err_text[ERRLEN+1];	/* error message text      */

	if ( (inwind[0]==0) && (inwind[1]==0) &&
	     (inwind[2]==0) && (inwind[3]==0) )
		{
		vlwind[0] = sl;
		vlwind[1] = ss;
		vlwind[2] = *nl;
		vlwind[3] = *ns;
		}
	else
		{
		if ( inwind[0] > 0 )
			{
			if ( inwind[0] <= *nl )
				{
				vlwind[0] = inwind[0];
				}
			else
				{
				sprintf(err_text,"%s %d %s %d %s",
					"The starting line", inwind[0],
					"is greater than the number of lines",
					*nl, "in the image");
				c_errmsg(err_text,"plenter-invwnd",&nonfatal);
				c_errmsg("Invalid window specified",
                                         "plenter-invwnd",&nonfatal);
				return E_FAIL;
				};
			}
		else
			{
			sprintf( err_text, " The starting line %d %s",
				inwind[0], "must be positive");
			c_errmsg(err_text,"plenter-invwnd",&nonfatal);
			c_errmsg("Invalid window specified",
                                 "plenter-invwnd",&nonfatal);
			return E_FAIL;
			};

		if ( inwind[1] > 0 )
			{
			if ( inwind[1] <= *ns )
				{
				vlwind[1] = inwind[1];
				}
			else
				{
				sprintf( err_text, "%s %d %s %d %s.",
					"The starting sample", inwind[1],
					"is greater than the number of samples"
					, *ns, "in the image");
				c_errmsg(err_text,"plenter-invwnd",&nonfatal);
			        c_errmsg("Invalid window specified",
                                         "plenter-invwnd",&nonfatal);
				return E_FAIL;
				};
			}
		else
			{
			sprintf( err_text, "The starting sample %d %s",
				inwind[1], "must be positive");
			c_errmsg(err_text,"plenter-invbnd",&nonfatal);
			c_errmsg("Invalid window specified",
                                 "plenter-invwnd",&nonfatal);
			return E_FAIL;
			};

		if ( inwind[2] > 0 )
			{
			if ( (inwind[0] + inwind[2] -1) <= *nl )
				{
				vlwind[2] = inwind[2];
				}
			else
				{
				vlwind[2] = *nl - inwind[0] + 1;
				sprintf( err_text, "%s %d %s %d %s %s %d.",
				"The starting line", inwind[0],
				"and the number of lines", inwind[2],
				"put some of the window outside of the image,",
				"number of lines reset to", vlwind[2]);
				c_errmsg(err_text,"plenter-invwnd",&nonfatal);
			        c_errmsg("Invalid window specified",
                                         "plenter-invwnd",&nonfatal);
				};
			}
		else
			{
			sprintf( err_text, "The number of lines %d %s",
				inwind[2], "must be positive");
			c_errmsg(err_text,"plenter-invwnd",&nonfatal);
			c_errmsg("Invalid window specified",
                                 "plenter-invwnd",&nonfatal);
			return E_FAIL;
			};

		if ( inwind[3] > 0 )
			{
			if ( (inwind[1] + inwind[3] -1) <= *ns )
				{
				vlwind[3] = inwind[3];
				}
			else
				{
				vlwind[3] = *ns - inwind[1] + 1;
				sprintf( err_text, "%s %d %s %d %s %s %d.",
				"The starting sample", inwind[1],
				"and the number of samples", inwind[3],
				"put some of the window outside of the image,",
				"number of samples reset to", vlwind[3]);
				c_errmsg(err_text,"plenter-invwnd",&nonfatal);
			        c_errmsg("Invalid window specified",
                                         "plenter-invwnd",&nonfatal);
				};
			}
		else
			{
			sprintf( err_text, "The number of samples %d %s.",
				inwind[3], "must be positive" );
			c_errmsg(err_text,"plenter-invwnd",&nonfatal);
			c_errmsg("Invalid window specified",
                                 "plenter-invwnd",&nonfatal);
			return E_FAIL;
			}
		}

	return E_SUCC;
	}
