/******************************************************************************
FUNCTION:	NXTSTP

PURPOSE:	Advances tape to next strip or mounts the next volume

ALGORITHM 
  Read the tape until the EOF is encountered.
  If the number of tapes are equal to two and the second stip has been processed
	mount the second tape 
  else if there are four tapes
	mount each tape after processing a strip

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"	/* LAS include				*/
#include "tapeio.h"	/* Tape I/O include			*/
#include "protos.h"

void nxtstp
(
    char comment[][CMLEN],	/* Comment for tape mounts		*/
    long *tapes,		/* Number of tapes			*/
    long *strip,		/* Current strip number			*/
    long *adjlin		/* Adjusted line length			*/
)
{

extern long uni;	/* Tape drive unit number		*/

long size;		/* Buffer size				*/
long status = TAPE_SUCC;/* Status variable			*/
long mode = HANDLE;	/* Tape I/O handles errors		*/
long nonfatal = 1;	/* Non-fatal error code			*/

char buffr[3506];	/* Input buffer				*/

/*
  Read the tape until the EOF is encountered.
*/
size = *adjlin + 56;
while (status >= TAPE_SUCC)
    status = c_tpread(&uni,buffr,&size,&mode);

/*
  If EOF or EOV not encountered some kind of an error occured
*/
if ((status != TAPE_EOF) && (status != TAPE_EOV))
    {
    c_errmsg("Error reading tape","plenter-tpread",&nonfatal);
    clean_up();
    }

/*
  If there are two tapes mount the next volume after the second strip has
  been processed.  If there are four tape mount the next volume after each
  strip.
*/
if ((*tapes == 2) && (*strip == 2))
    nxtvol(comment[1]);
else if ((*tapes == 4) && (*strip < 4))
    nxtvol(comment[*strip]);
  
return;
}

/******************************************************************************
FUNCTION:	NXTVOL

PURPOSE:	dismounts current tape and mount next volume

ALGORITHM 
  Close/dismount the tape
  Mount and open the next tape

ALGORITHM REFERENCES	none
*******************************************************************************/
void nxtvol
(
    char *comment		/* Comment for tape mounts		*/
)
{

extern long uni;	/* Tape drive unit number		*/

long status;		/* Status variable			*/
long mode = HANDLE;	/* Tape I/O handle errors		*/
long rewopt = REWOFFL;	/* Rewind off line			*/
long opt1 = REWIND;	/* Rewind tape befor opening		*/
long inout = TREAD;	/* Read from tape			*/
long eofopt = 1;	/* Number of tape marks			*/
long skip = 0;		/* Number of files to skip		*/
long nonfatal = 1;	/* Non-fatal error code			*/

char dens[5];		/* default Tape density			*/
char tapeid[CMLEN];	/* tape id				*/
 
strcpy(dens,"0");
/*
  Close/dismount the tape
*/
status = c_tpclos(&uni,&mode,&rewopt,&eofopt);
if (status != TAPE_SUCC)
    c_errmsg("Nonfatal error encountered","plenter-warn",&nonfatal);

/*
  Mount and open the next tape (this call will display the same message to
  the operator).
*/
status = c_tpopen(&uni,&skip,&mode,&opt1,dens,&inout,comment,tapeid);
if (status != TAPE_SUCC)
    clean_up();

return;
}
