/******************************************************************************
FUNCTION:	OPEN_LAS

PURPOSE:	Open the image file for the first strip

ALGORITHM 
  The initial time through this routine
      Open the output file for write access
      Allocate buffer space for the image I/O processing
  else
      Open the output file for update access

  Group the open image file(s)

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"	/* LAS include					*/
#include "protos.h"

void open_las
(
    char *hname,		/* image host file name				*/
    long *band,		/* Total number of bands			*/
    long bands[],		/* Which bands are needed			*/
    long *np,		/* Number of samples in output image		*/
    long *nl,		/* Number of lines in output image		*/
    long *initl		/* Initial pass flag				*/
)
{

extern long lstidx;	/* last index counter				*/

/*
  The following variables have been declared global for image I/O routines
*/
extern unsigned char *thebuf;	/* image I/O buffer			*/
extern struct FDESC * fdesc[2];	
                               /* image file descriptors used for image I/O	*/
extern struct GDESC * gdesc;	/* group descriptor used for image I/O		*/
extern long offset1[5]; /* offsets into "thebuf" for output bands	*/
extern long bufsiz;	/* Size of the memory area			*/

long nonfatal = 1;	/* Non-fatal error code				*/
long one    = 1;	/* One						*/
long zero   = 0;	/* Zero						*/
long dtype  = EBYTE;	/* Data type of output				*/
long access = IWRITE;	/* Open the output file with write access	*/
long sp;		/* Starting pixel				*/
long ep;		/* Ending pixel					*/
long status;		/* Status variable				*/
long opt[3];

opt[0] = opt[2] = 0;
opt[1] = NO_COMPRES;
if (*initl)
    {
    access = IWRITE;

    /*
      Open the output file
    */
    status = c_eopens(&fdesc[0],hname,band,&one,&one,nl,np,offset1,&access,
		      &dtype,&zero,opt);
    if (status != E_SUCC)
	clean_up();

    fdesc[1] = 0;

    /*
      Allocate I/O buffer the first time
    */
    if ((thebuf = (unsigned char *) c_ealloc(fdesc,&bufsiz)) == NULL)
	{
	c_errmsg("Error allocating dynamic memory","plenter-alloc",&nonfatal);
	clean_up();
	}
    }
else
    {
    /*
      Calculate the new window starting and ending sample
    sp = skew + lstidx + 1;
    */
    sp = lstidx + 1;
    ep = *np - sp;
    access = IUPDATE;

    /*
      Open the output file
    */
    status = c_eopens(&fdesc[0],hname,bands,&one,&sp,nl,&ep,offset1,&access,
		      &dtype,&zero,opt);
    if (status != E_SUCC)
	clean_up();

    fdesc[1] = 0;
    }

/*
  Group images
*/
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   clean_up();

return;
}

/******************************************************************************
FUNCTION:	CLOSELAS

PURPOSE:	Closes the image file

ALGORITHM 
  Close the LAS image file

ALGORITHM REFERENCES	none
*******************************************************************************/
int close_las
(void)

{

/*
  The following variables have been declared global for image I/O routines
*/
extern struct GDESC * gdesc;	/* group descriptor used for image I/O		*/

long status;		/* Status variable			*/

/*
  Close the LAS file
*/
status = c_egclse(&gdesc);

return(status);
}
