/******************************************************************************
FUNCTION:	PROCESS

PURPOSE: 	Read the tape and process the data and inputs to the image file

ALGORITHM 
ALGORITHM REFERENCES

   Technical Memorandum:
	Generation and Physical Characteristics of the
	LANDSAT-1, -2 and -3 MSS Computer Compatable Tapes

	November 1975	-  Old format of CCT

	December 1977   -  New format of CCT
*******************************************************************************/
#include <stdio.h>
#include <math.h>	/* Math library include				*/
#include "las.h"	/* LAS include					*/
#include "tapeio.h"	/* Tape I/O include				*/
#include "util.h"
#include "pixman.h"
#include "labserv.h"
#include "protos.h"

void process
(
    char *hname,            /* host file name                               */
    long *strip,		/* current strip number				*/
    long bla[],		/* Flags for the bands to process		*/
    long inwin[],		/* sl,ss,nl,ns 					*/
    long *left,		/* left sample of input image 			*/
    long *right,		/* right sample of input image			*/
    long *upscal,		/* scaling factor added to each sample		*/
    long *b4fac,		/* scaling factor multiplied to band 4		*/
    long *linc,		/* line increment 				*/
    long *sinc,		/* sample increment				*/
    long *adjlin,		/* adjusted line length				*/
    float *skwcon,		/* earth skew variable				*/
    long *maxskw,		/* maximum skew					*/
    long *initl,		/* initial pass					*/
    char *earth,		/* correct for earth rotation flag		*/
    char *synth,		/* flag for synthetic pixel removal		*/
    long *msgtim,		/* elapsed time between display messages	*/
    char *minmax,		/* min max flag					*/
    long *np,		/* number of samples in output image		*/
    double *minv,		/* buffer for minimum pixel value/band		*/
    double *maxv,		/* buffer for maximum pixel value/band		*/
    long *band		/* Total number of bands			*/
)
{

char praw[CMLEN];       /* Raw line length host file name               */
extern long uni;        /* Tape drive unit number			*/
extern long outcnt;     /* Number of output files                       */
extern char out[][CMLEN]; /* Output files                                 */

/*
  The following variables have been declared global for image I/O routines
*/
extern unsigned char *thebuf;	/* Image buffer				*/
extern struct GDESC * gdesc;	/* group descriptor used for image I/O		 */
extern long offset1[5]; /* offsets into "thebuf" for output bands	*/
extern long curent;	/* current processing line			*/
extern long lstcnt;	/* last counter					*/
extern long lstidx;	/* last index counter				*/
extern long lstsam;	/* last sample counter				*/
extern long skew;	/* earth's rotational skew			*/

FILE *outfile = NULL; /*ptrs to create output for raw line lengths   */

float ups;		/* float of upscal				*/
float b4f = 1.0;	/* float of b4fac				*/
float fone = 1.0;	/* float of one					*/

long size;		/* buffer size					*/
long count = 0;		/* Sample increment counter 			*/
long i;			/* loop counter-line increment			*/
long j;			/* band counter					*/
long k;			/* loop counter-registration data(band 4)	*/
long index = 0;		/* output buffer's index location		*/
long first;		/* first sample of input image			*/
long last;		/* last sample of input image			*/
long npix;		/* number of pixels				*/
long sample;		/* current sample				*/
long delta;		/* Delta value between synthetic pixels		*/
long skip;		/* add current sample to buffer or not		*/
long relidx;		/* relative index location			*/
long skwerr;		/* temp. used to fiquire skew			*/
long status;		/* function return status code			*/
long done;		/* pxfill is done				*/
long ssamp;		/* Starting sample relative to the strip	*/
long esamp;		/* Ending sample relative to the strip		*/
long bnd;		/* Band element counter				*/
long msgflg = ALLCAT;	/* Processes message format flag		*/
long dtype = EBYTE;	/* Data type of output				*/
long mode = HANDLE;	/* Tape I/O handle errors			*/
long nonfatal = 1;	/* Non-fatal error code				*/

short rawlin;		/* raw line length				*/
short raw[2];		/* 						*/

static unsigned char inbuff[3506]; /* input buffer			*/
static char obuff[4][3506]; /* temporary buffer for band 1		*/
static char tobuff[3506];   /* temporary buffer				*/
static long nxtsam[2340 + 1]; /*temporary storage of samples between removal*/
			      /* Add one as algorithm is not using this array
				 as zero relative.  Problems were occurring
                                 on the SUN with the corner coordinates with
                                 at nxtsam[2340]. */
 
unsigned char fillval = 0;  /* Fill value of zero			*/

/*
  Adjust the starting locations within a strip for the support of processing
  windows
*/
    if (*left < inwin[1])
	ssamp = inwin[1];
    else
	ssamp = *left;

    if (*right > (inwin[1] + inwin[3] - 1))
	esamp = inwin[1] + inwin[3] - 1;
    else
	esamp = *right;

/*
  Determine the starting and ending locations within the input buffer.
  First points to the start of a pixel pair and last points to the
  ending of a pixel pair.
*/
    first = 4 * ((ssamp - *left - 1) + (ssamp % 2));
    last  = 4 * ((esamp - *left + 1) + (esamp % 2));

/*
  First time initailize the some variables
*/
    if (*initl)
	{
        lstsam = 0;
        lstcnt = 0;
        lstidx = 0;
	c_inmnmx(&dtype,band,minv,maxv);
	}
   if(*strip == 1)
        {
         for(i = 1;i <= 2340; i++)
            nxtsam[i] = 0;
        }
/*
  Initailize some other variables each time
*/
    curent = 1;
    done = FALSE;
    ups = (float) *upscal;

/*
  Skip over the lines outside of the specified window by reading records
  from the tape until the desired record has been reached
*/
    for (i = 1; i < inwin[0]; i++)
	{
	size = *adjlin + 56;
	status = c_tpread(&uni,inbuff,&size,&mode);
	if (status < TAPE_SUCC)
	    clean_up();
	}

/*
  Initailize the processing message and start it's output
*/
    c_inittm("PLENTER","Processed","line","bands","",
	     &inwin[2],band,msgtim,&msgflg);
    c_startm();

/*
  Process each line
*/
/*
      If synth pixels to be removed, open file to store raw line lengths 
*/ 
    if((*strip == 1) && (strcmp(synth,"YES") == 0))
       {
        c_lsmknm(hname,"cctx_rll",praw);
        outfile = fopen(praw,"w");
         if(outfile == NULL)
           {
            c_errmsg("Error opening RLL file","plenter-open",&nonfatal);
            clean_up();
           }  
	strcpy(out[outcnt],praw);
	outcnt++;
        fprintf(outfile,"%d\n", *adjlin);
       }
    for (curent = 1; curent <= inwin[2]; curent += (*linc))
	{
	/*
	  Make sure that the specified number of records can be skipped
	  for the line increment
        */
	if ((curent + *linc - 1) > inwin[2])
	    break;

	/*
	  skip over the specified line increment records
	*/
	for (i = 1; i < *linc; i++)
	    {
	    size = *adjlin + 56;
	    status = c_tpread(&uni,inbuff,&size,&mode);
	    if (status < TAPE_SUCC)
		clean_up();
	    }

	/*
	  Initally to setup the writing of the output buffer and there after
	  to write the previous output buffer.
	*/
	status = c_estep(&gdesc);
	if (status != E_SUCC)
	    clean_up();

	/*
	  Set the entire output buffer for each band to zeros on the
	  first pass for each line.
	*/
	if (*initl)
	    {
	    for (bnd = 0, j = 0; j < 4; j++)
		{
		if (bla[j])
		    {
		    c_pxfill(&thebuf[offset1[bnd]],&dtype,np,&fillval);
		    bnd++;
		    }
		}
	    }

	 /*
	   Initialize temporary buffers for each line every time this
	   routine is entered.
	 */
	 if (!done)
	    {
	    c_pxfill(tobuff,&dtype,np,&fillval);
	    for (j = 0; j < 4; j++)
		c_pxfill(obuff[j],&dtype,np,&fillval);
	    }

	/*
	  Read the first record to process from tape
	*/
	size = *adjlin + 56;
	status = c_tpread(&uni,inbuff,&size,&mode);
	if (status < TAPE_SUCC)
	    clean_up();

	/*
	  Calculate the raw line length from the last two bytes of the
	  input line.
	*/
	raw[0] = inbuff[size - 2];
	raw[1] = inbuff[size - 1];
	rawlin = (raw[0] * 256) + raw[1];
        /*
          If synth pixels to be removed, write raw line lengths to file
        */
 
        if((*strip == 1) && (strcmp(synth,"YES") == 0))
           fprintf(outfile,"%d   %d\n",curent,rawlin);
	/*
	  Calculate a delta value between each synthetic pixel and set
	  counters for locating those synthetic pixels
	*/
        if ((*adjlin - (rawlin + 6)) == 0)
           delta = 0;
        else
           delta = rawlin / (*adjlin - (rawlin + 6)) + 1;
	count = lstcnt;
	index = 0;

	/*
	  Process each sample in the strip
	*/
	for (sample = first; sample < last; sample += 8)
	    {
	    /*
	      Increment the sample increment counter
	    */
	    count += 1;

	    /*
	      if the sample counter equals the sample increment value
	      the pixel is to be added to the output buffer
	    */
	    if (*sinc == count)
		skip = FALSE;
	    else
		skip = TRUE;

	    /*
	      Check for synthetic pixels
	    */
	    if (strcmp(synth,"YES") == 0)
		{
		/*
		  Increment a delta sample counter for synthetic pixels
		*/
		nxtsam[curent] += 1;

		/*
		  When the delta sample counter equals the delta value
		  (the number of pixels between each synthetic pixel)
		  skip the next sample, decrement the sample increment
		  counter and reset the delta synthetic counter.
		*/
		if (nxtsam[curent] == delta)
		    {
		    skip = TRUE;
		    count--;
		    nxtsam[curent] = 0;
		    }
		}

	    /*
	      If the input window does not start on an even pixel pair
	      the first sample of the pair is skipped.  Reset the sample
	      increment counter.
	    */
	    if ((sample == first)  && ((ssamp % 2) == 0))
		{
		skip = TRUE;
		count = 0;
		}

	    /*
	      When the skip flag is not set add the first pixel pair for
	      the bands selected to the output band buffer.  Increment
              the sample counter and reset the sample increment counter.
	    */
	    if (!skip)
		{
		if (bla[0])
		    obuff[0][index] = inbuff[sample];
		if (bla[1])
		    obuff[1][index] = inbuff[sample+2];
		if (bla[2])
		    obuff[2][index] = inbuff[sample+4];
		if (bla[3])
		    obuff[3][index] = inbuff[sample+6];
		index++;
		count = 0;
		}

	    /*
	      Increment the sample increment counter
	    */
	    count++;

	    /*
	      If the sample increment counter and the sample increment value
	      are equal set a flag so that the current sample will be added
	      to the output buffer.
	    */
	    if (*sinc == count)
		skip = FALSE;
	    else
		skip = TRUE;

	    /*
	      Check for synthetic pixels
	    */
	    if (strcmp(synth,"YES") == 0)
		{
		/*
		  Increment a delta sample counter for synthetic pixels
		*/
		nxtsam[curent] += 1;

		/*
		  When the delta sample counter equals the delta value
		  (the number of pixels between each synthetic pixel)
		  skip the next sample, decrement the sample increment
		  counter and reset the delta synthetic counter.
		*/
		if (nxtsam[curent] == delta)
		    {
		    skip = TRUE;
		    nxtsam[curent] = 0;
		    count--;
		    }
		}

	    /*
	      If the input window does not end on the last pixel pair
	      the last sample of the pair is skipped.  Decrement the
	      sample increment counter.
	    */
	    if ((sample == last) && ((esamp % 2) != 0))
		{
		skip = TRUE;
		count--;
		}

	    /*
	      When the skip flag is not set add the last pixel pair for
	      the bands selected to the output band buffer.  Increment
              the sample counter and reset the sample increment counter.
	    */
	    if (!skip) 
		{
		if (bla[0])
		    obuff[0][index]  = inbuff[sample+1];
		if (bla[1])
		    obuff[1][index] = inbuff[sample+3];
		if (bla[2])
		    obuff[2][index] = inbuff[sample+5];
		if (bla[3])
		    obuff[3][index] = inbuff[sample+7];
		index++;
		count = 0;
		}

	    /*
              Done with the first sample pass so don't re-initialize variables
	    */
	    done = TRUE;
	    }


	/*
	  Determine the amount of skew to be added to the line
	*/
	if (strcmp(earth,"NO") == 0)
	    {
	    skew = 0;
	    }
	else
	    {
            skwerr = (long) ((float) ((inwin[0] + curent - 2) / 6) * (*skwcon));
            skew = (*maxskw - skwerr) / *sinc;
	    }

	/*
	  Set the relative index pointer to the starting location of the
	  pixels that have just been added and calculate the number of
	  of pixels that were added.
	*/
	relidx = index + skew;
	npix = index;

	/*
	  For each band wanted call PXADD that will add the OFFSET value
	  and multiply each sample of the fourth band by the SCALE FACTOR
	  value.  UPS is the float of upscal and b4f is the float of b4fac
	*/
	for (bnd = 0, j = 0; j < 4; j++)
	    {
	    if (bla[j])
		{

		/*
		  Multiply the fourth band by the scale factor
		*/
		if (j == 3)
		    b4f = (float) (*b4fac);
		else
		    b4f = fone;

		/*
		  Add any offset to the image data
		*/
		c_pxadd(&npix,obuff[j],tobuff,&thebuf[offset1[bnd]+skew],
			&b4f,&fone,&ups,&dtype);

		/*
		  For strip one make sure the adjusted line portion of
		  the image is set to zero
		*/
		if (*strip == 1)
		    {
		    for (k = skew; k < skew + 6; k++)
			{
			if (thebuf[offset1[bnd]+k] == 255)
				thebuf[offset1[bnd]+k] = 0;
			}
		    }

		/*
		  For strip 4  make sure the adjusted line portion of the
		  image is set to zero
		*/
		if (*strip == 4)
		    {
		    for (k = relidx - 6; k < relidx; k++)
			{
			if (thebuf[offset1[bnd]+k] == 255)
				thebuf[offset1[bnd]+k] = 0;
			}
		    }

		/*
		  Calculate the min/max for the image
		*/
		if (strcmp(minmax,"YES") == 0)
		    {
		    status = c_pxmnmx(&thebuf[offset1[bnd]],&relidx,
					&minv[bnd],&maxv[bnd],&dtype);
		    if (status != E_SUCC)
			c_errmsg("Nonfatal error encountered",
				 "plenter-warn",&nonfatal);
		    }
		bnd++;
		}
	    }
	}

/*
  For case where synth pixels removed, close raw line length output file
*/
    if ((*strip == 1) && (strcmp(synth,"YES") == 0))
       fclose(outfile);
/*
  Stop the standard output message
*/
    c_stoptm();

/*
  Write the last line out to the image file
*/
    status = c_estep(&gdesc);
    if (status != E_SUCC)
	clean_up();

/*
  Set global variables to the current values of the sample increment
  counter, sample counter and delta synthetic pixel counter.  This
  will allow the tape to be positioned to the next strip or the next
  tape to be mounted.  These values are saved and used to reset the
  pointers that hold the current position on the input and output files.
*/
    lstcnt = count;
    lstidx += index;

    return;
}
