#ifndef PROTOS_H
#define PROTOS_H

void calcorn
(
    double clat_l, 	/* center point latitude */
    double clon_l,	/* center point longitude */
    char *rectec,	/* recording technique used */
    double *rtnlat,	/* calculated corner point latitudes */
    double *rtnlon,	/* calculated corner point longitudes */
    long sat		/* satellite number */
);

int chkwind
(
    long inwind[],	/* Input window to be checked	*/
    long vlwind[],	/* Valid window			*/
    long *nl,
    long *ns	/* Number of lines and samples  */
);

void clean_up
(void);

void dosiat
(
    char *hname,		/* Host file name			*/
    long *old,		/* Old version of CCTX tapes		*/
    char *direction        /* Direction of the satellite "D" or "A" */
);

void getpar
(
    char tname[][CMLEN],		/* Image file name				*/
    char hname[][CMLEN],		/* Host image file name				*/
    long inwin[],		/* Input window to process			*/
    long *band,		/* Total number of bands needed			*/
    long bands[],		/* Which bands are needed			*/
    long *b4fac,		/* Scaling factor for band 4			*/
    long *linc,		/* Line increment				*/
    long *sinc,		/* Sample increment				*/
    long *upscal,		/* Scaling factor added to each pixel		*/
    long *tapes,		/* Number of tapes needed			*/
    long *skip,		/* Number of files to skip			*/
    char comment[][CMLEN],	/* Tape numbers to be used			*/
    char *wrtdsk,		/* Write id, annotation, SIAT files to disk	*/
    char *earth,		/* Correct for earth rotation flag		*/
    char *synth,		/* Delete synthetic pixels flag			*/
    long bla[],		/* Bands wanted					*/
    char *minmax,		/* Min/Max flag					*/
    long *msgtim,		/* elapsed time between display messages	*/
    long *windflg		/* Window flag					*/
);

void header
(
    char *hname,		/* TAE image file name				*/
    long *strip,		/* current strip number				*/
    long inwin[4],		/* sl,sp,el,ep					*/
    char *wrtdsk,		/* write ID,ANNOTATION,SIAT files to disk	*/
    long *term,		/* flag for terminal output			*/
    long *file,		/* flag for file output				*/
    long *lp,		/* flag for line printer output			*/
    long *initl,		/* initial pass					*/
    long *adjlin,		/* adjusted line length				*/
    float *skwcon,		/* earth skew variable				*/
    long *maxskw,		/* maximum skew					*/
    long *np,		/* number of samples in output image		*/
    long *nl,		/* number of lines in output image		*/
    long *old,		/* old version of CCTX tapes			*/
    char *earth,		/* correct for earth rotation flag		*/
    long *sinc,		/* sample increment				*/
    long *linc,		/* line increment				*/
    FILE *fp,		/* File pointer to print file			*/
    char *scene,		/* scene/frame id				*/
    char *day,		/* Date of exposure  (any day is fine)		*/
    char *month,		/* Month of exposure (Jan is to cold)		*/
    char *year,		/* Year of exposure  (I wouldn't tell you)	*/
    char *center		/* Center  Lat/Long				*/
);

void nxtstp
(
    char comment[][CMLEN],	/* Comment for tape mounts		*/
    long *tapes,		/* Number of tapes			*/
    long *strip,		/* Current strip number			*/
    long *adjlin		/* Adjusted line length			*/
);

void nxtvol
(
    char *comment		/* Comment for tape mounts		*/
);

void open_las
(
    char *hname,		/* image host file name				*/
    long *band,		/* Total number of bands			*/
    long bands[],		/* Which bands are needed			*/
    long *np,		/* Number of samples in output image		*/
    long *nl,		/* Number of lines in output image		*/
    long *initl		/* Initial pass flag				*/
);

int close_las
(void);

void process
(
    char *hname,            /* host file name                               */
    long *strip,		/* current strip number				*/
    long bla[],		/* Flags for the bands to process		*/
    long inwin[],		/* sl,ss,nl,ns 					*/
    long *left,		/* left sample of input image 			*/
    long *right,		/* right sample of input image			*/
    long *upscal,		/* scaling factor added to each sample		*/
    long *b4fac,		/* scaling factor multiplied to band 4		*/
    long *linc,		/* line increment 				*/
    long *sinc,		/* sample increment				*/
    long *adjlin,		/* adjusted line length				*/
    float *skwcon,		/* earth skew variable				*/
    long *maxskw,		/* maximum skew					*/
    long *initl,		/* initial pass					*/
    char *earth,		/* correct for earth rotation flag		*/
    char *synth,		/* flag for synthetic pixel removal		*/
    long *msgtim,		/* elapsed time between display messages	*/
    char *minmax,		/* min max flag					*/
    long *np,		/* number of samples in output image		*/
    double *minv,		/* buffer for minimum pixel value/band		*/
    double *maxv,		/* buffer for maximum pixel value/band		*/
    long *band		/* Total number of bands			*/
);

void update_ddr
(
    char *hname,		/* Host file name			*/
    long inwin[],		/* Input window				*/
    long *totbnd,		/* Total number of bands to process	*/
    long *linc,		/* Line increment factor		*/
    long *sinc,		/* Sample increment factor		*/
    char *scene,		/* Scene id (contains time of exposure) */
    char *day,		/* Day of exposure			*/
    char *month,		/* Month of exposure			*/
    char *year,		/* Year of exposure			*/
    char *direction,	/* Direction of satellite "D" or "A"	*/
    long *old,		/* old version of CCTX tapes		*/
    long *windflg,          /* Window flag                    	*/
    char *center           /* Center  Lat/Long                      */
);

#endif

