/****************************************************************************
NAME:               do_plot

PURPOSE:  Writes a plot file for the GEOREG software

*****************************************************************************/
#include "polyfit.h"

void do_plot
(
    struct TPLDATA *tie_data,   /* Tie Point Location data records */
    struct COEF *coef,      /* Coefficients & modelling stats */
    struct RESIDUAL *residual,  /* Fit residuals */
    char *plotnm               /* plot file name */
)

{
FILE *ptr;
struct TPLDATA *tp;
long act;
long cnt;

/* Open the print file.  This file open is destructive if a file exists
--------------------------------------------------------------------*/
if (strlen(plotnm) == 0)
    return;
else
    ptr = fopen(plotnm, "w");

/* Write maximum line, maximum sample, and RMSe to top of file
-----------------------------------------------------------*/
fprintf(ptr, "%s %lf\n", residual->x_max_ptid, residual->x_max_err);
fprintf(ptr, "%s %lf\n", residual->y_max_ptid, residual->y_max_err);
fprintf(ptr, "%lf\n", residual->ave_rmse);

/* Write each point id, search X and Y coordinate, and their residuals to file
---------------------------------------------------------------------------*/
for (tp = tie_data, cnt = 0; cnt < coef->npts; cnt++, tp++)
    {
    act = tp->active;
    if ((act ==1) || (act == 0) || (act > 9))
        fprintf(ptr,"%s 1 %f %f %f %f %lf %lf\n", tp->pt_id,
        tp->ref_coord[1], tp->ref_coord[0], tp->sea_coord[1],
        tp->sea_coord[0],residual->x_residual[cnt],residual->y_residual[cnt]);
    else
        fprintf(ptr,"%s 0 %f %f %f %f\n", tp->pt_id, tp->ref_coord[1],
        tp->ref_coord[0], tp->sea_coord[1], tp->sea_coord[0]);
    }
fclose(ptr);

}

