/****************************************************************************
NAME:               put_tie_points

PURPOSE:  Writes edited tie points to a new disk file

ALGORITHM DESCRIPTION:
    Initialize parameters for Tie Point Location file write
    Open the Tie Point Location file & write header record
    For each tie point (reguardless of active-flag value)
       Write tie point to disk
    Close the Tie Point Location file
    Return
*****************************************************************************/
#include "polyfit.h"

int put_tie_points
(
    char *name,         /* Name of the Tie Point Location file */
    struct TPLHEAD *header,     /* Tie Point Location header record */
    struct TPLDATA *data,       /* Tie Point Location data records */
    long npts          /* Number of tie point data records read */
)
{
struct PTDESC *desc;/* File descriptor for tie pt file */
long ptyp;          /* Type of tie point data/header record */
long acc;           /* File access flag */
long count;         /* Loop counter */

/* Open the tie point location file
--------------------------------*/
ptyp = TPL;
acc = IWRITE;
if (c_ptopen(&desc, name, &ptyp, &acc, header) != E_SUCC) return(E_FAIL);

/* Write out the tie point location data records
-------------------------------------------*/
for (count = 0; count < npts; count++)
    if (c_ptio(&desc, &(data[count])) != E_SUCC) return(E_FAIL);

/* Close the Tie Point Location file
---------------------------------*/
if (c_ptclse(&desc) != E_SUCC) return(E_FAIL);

return(E_SUCC);
}

