/****************************************************************************
NAME:                           GETSPHEREVAL 

PURPOSE:  Retrieve the semimajor axis and eccentircity squared using the
          input datum. 

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
 7.0		6/98    J. Janssen         CSB      Datum implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Retrieve the datum structure.
	Use the datum structure to set the semimajor axis and the 
  	  eccentricity squared.

ALGORITHM REFERENCES
 	None.
*****************************************************************************/

#include "las.h"
#include "geompak.h"
#include "datum.h" 
#include "worgen.h"
#include "protos.h"

int getsphereval
(
    double *smajorax,		/* semimajor axis */
    double *ecsqval,		/* eccentricity squared */
    long *indatum			/* input datum */
)

{
struct DATUMDEF dtmparm;	/* datum structure */
double flat;			/* flattening for the spheroid */
long pass = E_SUCC;		/* non-fatal code for errmsg routine */

/* Retrieve the datum structure
------------------------------*/
dtmparm.datumnum = *indatum;
if (c_getdatum(&dtmparm) != E_SUCC)
   {
   c_errmsg("Could not retrieve input datum information","datum-info", &pass);
   return(E_FAIL);
   }

/* Use the datum structure to set the semimajor axis and the 
   eccentricity squared.
------------------------------------------------------------*/
*smajorax = dtmparm.smajor;
if ((dtmparm.spherenum == 19) || (dtmparm.spherenum == 31))
   flat = 0.0;
else
   flat = 1/(dtmparm.recip_flat);
*ecsqval = ((2*flat) - (flat*flat));

return(E_SUCC);
} 
