/****************************************************************************
NAME:				 GOODE 

PURPOSE:   Retrieves parameters for the GOODE projection

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  5.1		7/92	S. Nelson 	   CSB	    modified HAMMER.C to be
						    GOODE.C
  7.0		6/98	J. Janssen	   CSB	    Modified due to datum
						    conversion implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Set the zone and projection type
	Retreive the needed parameters
	Get the sphere radius using the input datum
	Return

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"

void goode
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum,			/* projection datum */
    double *prjparms		/* projection parameters */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
long fatal = -1;		/* Fatal error condition */
long one = 1;			/* for error messages */

/*  Set the zone and projection type.
-------------------------------------*/
*prjzone = 62;
*prjtype = 24;

/*  Retreive the needed parameters.
-----------------------------------*/
val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

/*  Get the sphere radius using the input datum.
------------------------------------------------*/
if (getsphereval(prjparms,prjparms + 1,prjdatum) != E_SUCC)
   {
   c_errmsg("Error returned from getsphereval","goode-call",&one);
   c_errmsg("Fatal error encountered","projprm-fatal",&fatal);
   }

}
