/****************************************************************************
NAME:				 STATEPLN 

PURPOSE:  Retrieves parameters for the STATE PLANE  projection

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
 NEWLAS		9/87	B. Ailts	   CSB	    original development
  5.0		1/89	D. Steinwand	   CSB      retrocode to 5.0
  7.0		6/98	J. Janssen	   CSB	    Modified due to datum
						    conversion implementation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Retreive the needed parameters
	Set the projection type
	Return

ALGORITHM REFERENCES		
	Software Documentation for GCTP General Cartographic Transformation
	 Package, U.S. Geological Survey, National Mapping Division, May 1982.

	Software Documentation for GCTP Supplement 1, U.S. Geological Survey,
	 National Mapping Division, May 1982.

	Conversations with John P. Snyder, U.S. Geological Survey,
	 September, 1987.

	Snyder, J.P., Map Projections used by the U.S. Geological Survey:
	 U.S. Geological Survey Bulletin 1532, 1982.

	Snyder, J.P., Map Projections--A Working Manual:  U.S. Geological
	 Survey Professional Paper 1395, 1987.
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"

void statepln
(
    struct PARBLK *parb,		/* TAE parameter block */
    long *prjzone,			/* projection zone */
    long *prjtype,			/* projection type */
    long *prjdatum			/* projection datum */
)
{
struct VARIABLE *val;		/* pointer to TAE variable within V_BLOCK */
struct VARIABLE *valzone;	/* pointer to TAE variable zone within VBLOCK */

/*  Retrieve the parameters needed.
-----------------------------------*/
valzone = p_find(parb,"ZONENUM");
*prjzone = IVAL(*valzone,0);

val = p_find(parb,"DATUM");
*prjdatum = IVAL(*val,0);

/*  Set the projection type
--------------------------*/
*prjtype = 2;
}
