/*****************************************************************************
NAME				  PUTPARM

PURPOSE   Writes TAE parameters to a project parameter file

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  5.0      D. Steinwand    12/89       EDC      LAS 5.0 development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PUTPARM must run under TAE

PROJECT  GEOREG/LAS

ALGORITHM DESCRIPTION
	Initialize TAE processing with z_init
	Get the name of the project parameter file
	Get the data type of the parameter
	Get the parameter key
	Open the project parameter file for append at end
	Get the parameter from TAE and write it to disk
	Close the project parameter file
	Call x_exit to terminate proc (NOTE:  INLAS and COMPLT were not used
		here--the function must run silent!
*****************************************************************************/
#include <stdio.h>
#include "las.h"

#define STRING 1
#define REAL 2
#define INTEGER 3

main()
{
struct PARBLK vblk;			/* TAE parameter block */
struct VARIABLE *var, *p_find();	/* TAE parameter parsing varibles */
long fatal = -1;			/* Fatal error flag */
long zero = 0;				/* Constant of zero */
long one = 1;				/* Constant of one */
long acc;				/* File access variable */
long i;					/* Loop counter */
long parmtype;				/* Parameter data type */
char parmkey[20];			/* Parameter key */
char ppf[CMLEN];			/* Name of project parameter file */
FILE *ptr, *fopen();			/* Disk file I/O var/routine */

/* Initialize TAE processing
 --------------------------*/
z_init(&vblk, P_ABORT);

/* Get the name of the project parameter file
 -------------------------------------------*/
acc = IUPDATE;
if (c_getnam(&vblk, "OUTPPF", &zero, ppf, "ppf", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","putparm-fatal",&fatal);

/* Get the data type of the parameter
  ----------------------------------*/
var = p_find(&vblk,"DTYPE");
if (strcmp(SVAL(*var,0),"STRING") == 0) parmtype = STRING;
if (strcmp(SVAL(*var,0),"INTEGER") == 0) parmtype = INTEGER;
if (strcmp(SVAL(*var,0),"REAL") == 0) parmtype = REAL;

/* Get the parameter key
  ---------------------*/
var = p_find(&vblk,"KEY");
strcpy (parmkey, SVAL(*var,0));

/* Open the project parameter file
  -------------------------------*/
ptr = fopen(ppf, "a+");
if (ptr == NULL)
   {
   c_errmsg("Error opening project parameter file","putparm-open",&one);
   c_errmsg("Fatal error encountered","putparm-fatal",&fatal);
   }

/* Get the parameter and write it to disk
  --------------------------------------*/
if (parmtype == STRING)
   {
   var = p_find(&vblk,"SVAL");
   fprintf(ptr, "%-20s S %d", parmkey, (*var).v_count);
   for (i = 0; i < (*var).v_count; i++)
	fprintf(ptr," %s", SVAL(*var,i));
   fprintf(ptr, "\n");
   }
if (parmtype == INTEGER)
   {
   var = p_find(&vblk,"IVAL");
   fprintf(ptr, "%-20s I %d", parmkey, (*var).v_count);
   for (i = 0; i < (*var).v_count; i++)
	fprintf(ptr," %d", IVAL(*var,i));
   fprintf(ptr, "\n");
   }
if (parmtype == REAL)
   {
   var = p_find(&vblk,"RVAL");
   fprintf(ptr, "%-20s R %d", parmkey, (*var).v_count);
   for (i = 0; i < (*var).v_count; i++)
	fprintf(ptr," %f", RVAL(*var,i));
   fprintf(ptr, "\n");
   }

fclose(ptr);
z_exit(1," ");
}
