#include "radiom2.h"

void filspec
(
    char hosin[][CMLEN],      /* input image host names             */
    char hosout[],            /* output image host name             */
    long *num_inimages,       /* number of input images             */
    long num_image_bands[],   /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each      */
    long wind[][4]           /* input image windows                */
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         8-May-89  K. A. Narayanan    
    6.1      19-May-94  D. Eichacker          Added opt[] for use in eopen

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

**************************************************************************
*/
{
struct	DDR	ddr;

long    fatal_err = -1;
long    one = 1;


double	line_inc = 1.0,
	samp_inc = 1.0;

long	access,                      /* file access type                   */
	comb_flag = NOCOMB,
	n, nb, nbands,
	stat,                        /* return status                      */
	type;                        /* file type                          */
static	long	opt[3] = {0,0,0};     /* eopen option array                 */

/*
open input image files for random access
----------------------------------------
*/
c_getddr (hosin[0], &ddr);
if (ddr.dtype != EBYTE)
   c_errmsg ("Input image must be BYTE", "radiom2-dtype", &fatal_err);

nb = 0;
access = IREAD;
type = EBYTE;
for (n = 0, nbands = 0; n < *num_inimages; n++)
   {
   stat = c_eopenr (&fdesc_in[n], hosin[n], &nb, &wind[0][SL], 
                   &wind[0][SS], &wind[0][NL], &wind[0][NS], 
                   &access, &type, opt, &one);
   if (stat != E_SUCC )
      c_errmsg ("Error opening input image", "radiom2-inopen", &fatal_err);
   nbands += num_image_bands[n];
   }  
if ((inbuf = (unsigned char *) malloc (nbands * wind[0][NS]))
                 == NULL)
   c_errmsg ("Error allocating dynamic memory", "radiom2-alloc", 
              &fatal_err);

access = IWRITE;
stat = c_eopenr (&fdesc_out, hosout, &nbands, &one,
                   &one, &wind[0][NL], &wind[0][NS], 
                   &access, &type, opt, &one);
if (stat != E_SUCC )
c_errmsg ("Error opening output image", "radiom2-openout", &fatal_err);

if ((outbuf = (unsigned char *) malloc (nbands * wind[0][NS])) == NULL)
   c_errmsg ("Error allocating dynamic memory", "radiom2-alloc", 
              &fatal_err);

/*
update output image ddr
-----------------------
*/
c_upddr (&block, hosin, hosout, num_inimages, wind, image_band_num, 
         num_image_bands, &line_inc, &samp_inc, &nbands, &comb_flag);

}
