#include "radiom2.h"
#include "util.h"

void getpar
(
    char hosin[][CMLEN],      /* input image host names           */
    char lasout[][CMLEN],       /* output image las name            */
    char hosout[][CMLEN],       /* output image host name           */
    char hosrlut[CMLEN],      /* input RLUT host name             */
    long *num_inimages,       /* number of input images           */
    long num_image_bands[],   /* number of bands in each input image */
    long image_band_num[][MAXBND + 1], /* band numbers in each    */
    long *num_inbands,        /* total number of input image bands   */
    long wind[][4]           /* input image windows              */
)
/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         10-Feb-82    J. Bay
    2         11-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
struct VARIABLE	*var;               /* pointer to a variable in TAE block */

long    zero = 0;
long    fatal_err = -1;


long	access,
	flag[MAXIMG][2],            /* flag for subwindows                */
	i,
	imglimit = 1,               /* maximum number of images           */
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

char	lasin[MAXIMG][CMLEN],       /* input image las names              */
	minmax[4];                  /* minmax flag                        */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input image parameters
---------------------------------------
*/
stat = c_getin (&block, "IN", &zero, lasin, hosin, num_inimages, wind, 
                image_band_num, num_image_bands, num_inbands, &imglimit, 
                &same_window, flag);

if (stat != E_SUCC) 
    c_errmsg ("Error accessing the input image", "histeq-in", 
              &fatal_err);

for (i = 1; i < *num_inimages; i++)
   {
   if (!same_window ||
       wind[i][SS] != wind[0][SS] || wind[i][SL] != wind[0][SL])
      c_errmsg ("Image windows must be same", "histeq-wind", 
                &fatal_err);
   }

/*
get input RLUT file name
------------------------
*/
access = IREAD;
stat = c_getnam (&block, "INRLUT", &zero, hosrlut, "rlut", &access);
if (stat != E_SUCC)
   c_errmsg("Error getting input file name", "histeq-rlut", &fatal_err);
/*
get output image name
---------------------
*/
c_getout (&block, "OUT", lasout, hosout);

/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);
var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;
}
