/***********************************************************************

NAME: RADIOM2

PURPOSE: RADIOM2 is a las radiometric correction program whose
         purpose is to correct images with regular striping due to
         differential detector response within a band in a
         multi-detector sensor. It uses the radiometric lookup
         tables (RLUTs) generated by HISTEQ.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         10-Feb-82    J. Bay
    2          6-Jun-86    B. Kovalick 
    3         11-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get parameters from TAE
   Call FILSPEC set up image i/o specifications
   Call APPLY_RLUT to apply radiometric lookup table and
      generate output image
   Inform user about successful completion

***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "protos.h"

struct PARBLK block;                   /* TAE parameter block     */

unsigned        char    *inbuf;        /* image i/o buffer        */
unsigned        char    *outbuf;       /* image i/o buffer        */

long    fd_sfin;                       /* input rlut file desc    */
struct FDESC *  fdesc_in[MAXIMG];              /* file descriptors        */
struct FDESC *  fdesc_out;                     /* file descriptors        */
long    min_max;                       /* min max flag            */
long    msgtim;                        /* message interval        */

extern long     curcat;                        /* message utility         */
extern long     curent;                        /* parameters for processing */

static  long    one = 1;

int main
(void) 
{
static
long	image_band_num[MAXIMG][MAXBND + 1], 
	                        /* band numbers in each input image*/
	num_bands,              /* total number of input image bands*/
	num_image_bands[MAXIMG],/* number of bands in each input image*/
	num_inimages,           /* number of input images          */
	wind[MAXIMG][4];        /* input image windows             */

static
char	hosin[MAXIMG][CMLEN],   /* input images host names         */
	hosrlut[CMLEN],         /* output RLUT file host name      */
	hosout[1][CMLEN],		/* input images las names          */
	lasout[1][CMLEN];          /* output image host name          */

/*
get parameters
--------------
*/
getpar (hosin, lasout, hosout, hosrlut, &num_inimages, 
        num_image_bands, image_band_num, &num_bands, wind);

/*
open input and output images
----------------------------
*/
filspec( hosin, hosout[0],&num_inimages, num_image_bands, image_band_num, wind);

/*
apply lookup table and generate output image
--------------------------------------------
*/
apply_rlut (hosin, hosrlut, hosout, wind[0], &num_inimages, &num_bands,
            image_band_num);

/*
issue completion message
------------------------
*/
c_complt ("RADIOM2", lasout, &one);

return EXIT_SUCCESS;

}
