#include "randsamp.h"
#include "pixman.h"
#include "util.h"

int bumpout
(
    float **buffer,	/* work buffer */
    long *band, 		/* band number */
    long rowsin, 	/* number of rows in input image */
    long sprowout, 	/* number of samples in output row */
    long *rowsamp,       /* number of blocks copied */
    long sampsize[],     /* sample block size */
    long inrow, 		/* input image row */
    long *lastsamp, 	/* number of fill blocks in last row */
    long *outline, 	/* output image line number */
    char hnameout[][CMLEN]/*output image name*/
)
/*
***************************************************************************

NAME: BUMPOUT

PURPOSE:   Writes the contents of the staging buffer to output image

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
long	count1, 		/* byte transfer count for output to disk */
	count2, 		/* byte trabsfer count for remaining samples */
	count3, 		/* byte transfer count for last incomplete */
				/* image lines */
	ftype = EREAL,		/* data type of the fill buffer */
	fzero = 0.0,		/* fill value of 0 */
	line, 			/* current line being processed */
	more = TRUE,		/* continuation flag */
	neg_count2,		/* negative value of count2 */
	status,			/* function return value; */
	one = 1;

/*
While there is data in the work buffer, compute transfer counts.
----------------------------------------------------------------
*/
while (more)
   {
   count1 = sampsize[1] * sprowout;
   count2 = sampsize[1] * (*rowsamp - sprowout);
   count3 = sampsize[1] * (*rowsamp);

/*
For each line in the sample block, copy image data from the work 
buffer to the output buffer.
--------------------------------------------------------------------
*/
   for (line = 0; line < sampsize[0]; line++)
      {
      c_pxcopy (buffer[line], outbuf, &ftype, &count1);

/*
If the number of remaining samples exceeds two, left justify 
remaining image data in work buffer.
If the number of remaining samples is less than zero, fill part 
of last line with zeros.
----------------------------------------------------------------
*/
      if (count2 > 0)
         c_pxcopy (&buffer[line][count1], buffer[line], &ftype, &count2);
      else if (count2 < 0) 
         {
         neg_count2 = -count2;
         c_pxfill (&outbuf[count3], &ftype, &neg_count2, &fzero); 
         }  

/*
Write out the output line.
--------------------------
*/
      status = c_ewrite (&fdesc_out, band, outline, outbuf, &one);
               (*outline)++;
      if (status == E_FAIL)
               {
	       if (c_eclose (&fdesc_out) != E_SUCC)
               c_errmsg("Error closing images","randsamp-close",&one);
               if (c_egclse (&gdesc_in) != E_SUCC)
               c_errmsg("Error closing images","randsamp-close",&one);
	       c_clean(&block,"randsamp",hnameout,&one);
               return(E_FAIL);
               }
      }

/*
Update the sample count for the work buffer.
---------------------------------------------
*/
   *rowsamp -= sprowout;
   if (inrow != rowsin-1)
      {
      if (*rowsamp < sprowout) 
        more = FALSE;
      }
   else
      {
      if (*rowsamp <= 0) 
         {
         *lastsamp = -(*rowsamp);
         more = FALSE;
         }  
      }  
   }  
return(E_SUCC);
}	   	        
