#include "randsamp.h"
#include "util.h"

int copysamp
(
    long *band,		/* band number				*/
    long *nbands, 	/* no. bands in input image		*/
    long nsamp, 		/* no. sample blocks			*/
    long wind[][4],	/* input image windows			*/
    long sampsize[], 	/* sample block size			*/
    long nsout, 		/* no. samples/line in output image	*/
    long *nlout, 	/* no. lines in output image		*/
    char hos_out[][CMLEN]/* output image name                    */
)
/*
***************************************************************************

NAME: COPYSAMP

PURPOSE: Copies the random samples from the input image to output.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS


***************************************************************************
*/
{

static	float	**buffer;	/* buffer to hold sample blocks	*/

long	cntsamp, 		/* number of samples in the row */
	dupl_cnt,		/* duplicate samples count */
	i, inrow,		/* current row being processed */
	lastsamp,		/* number of fill blocks in the last row */
	msg_flag = ONECAT,	/* processing message flag */
	outline, 		/* output image line number */
	rowsamp,		/* count of samples written to the current */
				/* output image row */
	rowsin,			/* number of block rows in the input image */
	sline, 			/* index into lines of the sample block */
	sprowin,		/* number of blocks per row in the */
    				/* input image */
	sprowout,		/* number of blocks per row in the */
				/* output image	*/
	status,			/* return value of function */
	strsamp,		/* start sample block number */
	one = 1;

char	buf[ERRLEN];		/* message buffer */

/*
If on the first band, initialize the processing message utility and
call FMATRIX to allocate dynamic memory.
-------------------------------------------------------------------------
*/
if (*band == 1) 
   {
   dupl_cnt = 0;
   for (i = 1; i < nsamp; i++)
      {
      if (samples[i-1] == samples[i])
         dupl_cnt++;
      }
   buffer = fmatrix (sampsize[0], wind[0][NS]+nsout+dupl_cnt);
   c_inittm ("RANDSAMP", "Processed", "line", "band", "", nlout, nbands, 
             &msgtim, &msg_flag);
   curent = 1;
   curcat = *band;
   c_startm ();
   }

/*
Compute the number of block rows in the input image, the number of blocks 
per row in the input image and  the number of blocks per row in the output
image.
---------------------------------------------------------------------------
*/
rowsin = wind[0][NL] / sampsize[0];
sprowin = wind[0][NS] / sampsize[1];
sprowout = nsout / sampsize[1];

/*
Initialize variables.
----------------------
*/
rowsamp = 0;
outline = 1; 
strsamp = 0;
cntsamp = 0;
lastsamp = 0;
curcat = *band;

/*
For all sample rows, call FINDSAMP to determine the number of random 
samples falling in each row.
---------------------------------------------------------------------
*/
for (inrow = 0; inrow < rowsin; inrow++)
   {
   findsamp (inrow, nsamp, sprowin, strsamp, &cntsamp);

/*
For number of lines in the sample block, read the input image and 
call SAMPIO to move samples from input line to work buffer.
---------------------------------------------------------------------
*/
   for (sline = 0; sline < sampsize[0]; sline++)
      {
      status = c_estep (&gdesc_in);
      if (status == E_FAIL)
	  {
          if (c_eclose (&fdesc_out) != E_SUCC)
          c_errmsg("Error closing images","randsamp-close",&one);
          if (c_egclse (&gdesc_in) != E_SUCC)
          c_errmsg("Error closing images","randsamp-close",&one);
	  c_clean(&block,"randsamp", hos_out, &one);
	  }
      if (cntsamp  >  0) 
         sampio (buffer, *band, strsamp, cntsamp, inrow, sprowin, 
                 sampsize, rowsamp, sline);
      }  
   strsamp += cntsamp;
   rowsamp += cntsamp;

/*
If at the last sample or at the last line, call BUMPOUT to write 
image data to disk.
-------------------------------------------------------------------
*/
   if (rowsamp >= sprowout || inrow == rowsin-1) 
      {
      bumpout (buffer, band, rowsin, sprowout, &rowsamp, sampsize, inrow, 
               &lastsamp, &outline, hos_out);
      curent = outline;
      }  
   }  

/*
Close the input files.
______________________
*/
free (inbuf);
status = c_egclse (&gdesc_in);
if (status == E_FAIL)
	{
        if (c_eclose (&fdesc_out) != E_SUCC)
        c_errmsg("Error closing images","randsamp-close",&one);
        c_clean(&block,"randsamp",hos_out,&one);
	}
/*
If on the last band, close output image and call FREE_FMATRIX to 
free dynamic storage.
-------------------------------------------------------------------
*/
if (*band == *nbands)
   {
   c_stoptm ();
   free_fmatrix (buffer, sampsize[0]);
   status = c_eclose (&fdesc_out);
   free (samples);

/*
Print out message for zero-filled blocks.
------------------------------------------ 
*/
   if (lastsamp  >  0) 
      {
      sprintf (buf, "Last %d sample blocks in output image are zero-filled", 
               lastsamp); 
      c_errmsg (buf,"randsamp-last", &one);
      }
   }	      
return(E_SUCC);
}	      
