#include "randsamp.h"
#include "util.h"

int filspec
(
    char hosin[][CMLEN],		/* input image host names */
    char hosout[][CMLEN],		/* output image host name */
    long *num_inimages,		/* number of input images */
    long num_image_bands[],	/* number of bands in each input image */
    long *num_inbands,		/* number of input image bands */
    long image_band_num[][MAXBND + 1], /* band numbers in each input image */
    long wind[][4],		/* input image windows */
    long nsamp,			/* no. sample blocks */
    long *nlout,			/* no. lines in output image */
    long *nsout,			/* no. samples/line in output image */
    long sampsize[],		/* sample block size */
    long band
)


/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

**************************************************************************
*/
{


struct	DDR	ddr;

double	line_inc = 1.0,		/* line and sample increments for */
	samp_inc = 1.0;		/* output image ddr */

long	access,			/* file access type */
	bufsiz=0,		/* image i/o buffer size */
	comb_flag = 0,		/* image combination flag */
	i, n, nib,			/* loop counters */
 	nonfatal = 1,		/* nonfatal error flag */
 	npix,			/* square root of the total number of */
				/* samples */
	num_outfiles=0,         /* number of output files */
	opt[2],			/* output type conversion flag */
	samples_per_row, 	/* number of samples per row in block */
	sample_rows, 		/* number of rows */
	status,			/* return status */
	total_pix,		/* total number of pixels in output image */
	type,			/* file data type */
	one = 1,
	zero = 0;

char	buf[ERRLEN];		/* message buffer */
static	char	*prog_name = "randsamp";

/*
Open input image files.
------------------------
*/
access = IREAD;
type = EREAL;
for (n = 0, nib = 0; n < *num_inimages; n++)
   {
   status = c_eopens (&fdesc[n], hosin[n], image_band_num[n], &wind[n][SL], 
                   &wind[n][SS], &wind[n][NL], &wind[n][NS], &ipoint[nib], 
                   &access, &type, &zero, &zero);
   if (status != E_SUCC )
      c_abort (&block, "Error opening input image", "randsamp-open", prog_name,
               hosout, &num_outfiles);

   nib += num_image_bands[n];
   }  
fdesc[n] = NULL;

/*
Set up group containing input bands.
-------------------------------------
*/
gdesc_in = 0;
if ((inbuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
   {
   for(i = 0 ; i < n ; i++)
   {
       if (c_eclose (&fdesc[i]) != E_SUCC)
           c_errmsg("Error closing images","randsamp-close",&one);
   }
   c_abort (&block, "Error allocating dynamic memory", "randsamp-alloc", 
            prog_name, hosout, &num_outfiles);
   }
status = c_egroup (fdesc, &gdesc_in, inbuf, &bufsiz);
if (status != E_SUCC )
   {
   for(i = 0 ; i < n ; i ++)
   {
       if (c_eclose (&fdesc[i]) != E_SUCC)
           c_errmsg("Error closing images","randsamp-close",&one);
   }
   c_abort (&block, "Error grouping image bands", "randsamp-group", prog_name,
               hosout, &num_outfiles);
   }

/*  
Compute number of lines in output image.
-----------------------------------------
*/
if (band == 1)
   {
   total_pix = nsamp * sampsize[0] * sampsize[1];
   npix = (long) sqrt ((double) total_pix);
   samples_per_row = npix/sampsize[1];                  
   sample_rows = nsamp/samples_per_row;
   if (nsamp - samples_per_row * sample_rows  >  0) 
      sample_rows++;
   *nlout = sample_rows * sampsize[0];
   *nsout = samples_per_row * sampsize[1];
   sprintf (buf, "Output image size (NL, NS): (%d, %d)", *nlout, *nsout);
   c_errmsg (buf, "randsamp-outsize", &one);

/*
Open the output image.
-----------------------
*/
   status  = c_getddr (hosin[0], &ddr);
   if (status == E_FAIL)
      c_errmsg("Error reading ddr file","randsamp-getddr",&nonfatal);
   opt[0]= ddr.dtype;
   opt[1] = 0;
   access = IWRITE;
   status = c_eopenr (&fdesc_out, hosout[0], num_inbands, &one, &one, nlout, 
                   nsout, &access, &type, opt, &one);
   num_outfiles = 1;
   if (status != E_SUCC )
      {
      if (c_egclse (&gdesc_in) != E_SUCC)
         c_errmsg("Error closing input image","randsamp-close",&nonfatal);
      c_abort (&block, "Error opening output image", "randsamp-outopen", 
		prog_name, hosout, &num_outfiles);
      }

   if ((outbuf = (float *) malloc(*nsout * sizeof (float))) == NULL)
      {
      if (c_eclose (&fdesc_out) != E_SUCC)
      c_errmsg("Error closing images","randsamp-close",&one);
      if (c_egclse (&gdesc_in) != E_SUCC)
      c_errmsg("Error closing images","randsamp-close",&one);
      c_abort (&block, "Error allocating dynamic memory", "randsamp-alloc", 
               prog_name, hosout, &num_outfiles);
      }

/*
Update the ddr file.
--------------------
*/
   status = c_upddr (&block, hosin, hosout[0], num_inimages, wind, image_band_num, 
                  num_image_bands, &line_inc, &samp_inc, num_inbands, 
                  &comb_flag);
   if (status != E_SUCC)
      return(E_FAIL);

   }
return(E_SUCC);
}
