#include "randsamp.h"
#include "lasmath.h"

int gensamp
(
    long *nsamp, 	/* no. sample blocks			*/
    long total,		/* total no. sample blocks in image	*/
    long *dseed		/* seed for random no. generator	*/
)
/*
***************************************************************************

NAME: GENSAMP

PURPOSE: Selects additional random samples as needed when the user
         has requested sampling without replacement. After 10 iterations, 
         RANDSAMP will pause and inform the user of the actual percent 
         coverage achieved. The user may elect to stop short of his input 
         percent. This may become necessary because execution time is 
         dependent on the input parameters (and the number of sorts that 
         become necessary when sampling without replacement).

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS


***************************************************************************
*/
{
int     term;

char	option[4];		/* user's response */

long	count, 			/* counts duplicate samples */
	i, 			/* loop counter and array index */
	iter, 			/* iteration counter */
	more = TRUE,		/* continuation flag */
	percent, 		/* percent of coverage achieved so far */
	samp, 			/* current sample being processed */
	sampcnt,		/* counts number of unique samples */
	inf_err = 1;

char	buf[ERRLEN];		/* message buffer */

/*
While in the continue mode, process 10 iterations.
---------------------------------------------------
*/
while (more)
   {
   for (iter = 0; iter < 10; iter++)      
      {
      count = 0;
/*
If there are duplicate samples then regenerate random samples as needed.
--------------------------------------------------------------------------
*/
      for (samp = 0; samp < *nsamp-1; samp++)
         {
         if ((samples[samp]) == (samples[samp+1])) 
            {
            ggud (dseed, total, 1, &samples[samp]);
            count++;
            }  
         }
/*
If there are no duplicate samples, return.
------------------------------------------
*/
      if (count  ==  0) 
         return(E_SUCC);

/*
Sort samples in ascending order.
----------------------------------
*/
      sort (*nsamp, samples-1);
      }

   count = 0;

/*
If we are still here after 10 iterations, check for duplicates and
if there are any zero them out.
--------------------------------------------------------------------
*/
   for (samp = 0; samp < *nsamp-1; samp++)
      {
      if ((samples[samp]) == (samples[samp+1])) 
         {
         (samples[samp]) = 0.0;
         count++;
         }  
      }  
/*
If there are no duplicate samples, return.
------------------------------------------
*/
   if (count  ==  0) 
      return(E_SUCC);

/*
Sort samples in ascending order and increment unique sample count.
-------------------------------------------------------------------
*/
   sort (*nsamp, samples-1);
   sampcnt = *nsamp - count;

/*
Compress zeros out in the random sample list.
----------------------------------------------
*/
   for (i = 0; i < sampcnt; i++) 
      (samples[i]) = (samples[count+i]);

/*
Compute the percentage of the coverage generated so far.
--------------------------------------------------------
*/
   percent = (long) ((float) sampcnt / (float) total * 100.0);

/*
Give user chance to quit the sample selection process.
-------------------------------------------------------
*/
   sprintf (buf, "Sampling of input image after 10 attempts: %d percent", 
            percent);
   c_errmsg (buf, "randsamp-gensamp", &inf_err); 
   t_write ("Do you still wish to continue sample selection (yes/no)? ", 
             T_PROMPT);
   t_read (option, &term);

/*
If the user wishes to continue processing, start at the top.
Otherwise, update the number of samples to be transferred and return.
----------------------------------------------------------------------
*/
   if (option[0] == 'Y' || option[0] == 'y') 
      {
      ggud (dseed, total, count, &samples[sampcnt]);
      sort (*nsamp, (samples-1));
      }
   else
      {
      *nsamp = sampcnt;
      more = FALSE;
      } 
   } 
return(E_SUCC);
}
