#include "randsamp.h"
#include "util.h"

int getpar
(
    char las_in[][CMLEN],     	/* input image las name             	*/
    char hos_in[][CMLEN],     	/* input image host names           	*/
    char las_out[][CMLEN],     	/* output image las name            	*/
    char hos_out[][CMLEN],     	/* output image host name           	*/
    long *num_inimages,       	/* number of input images           	*/
    long *num_inbands,        	/* total number of input image bands	*/
    long num_image_bands[],   	/* number of bands in each input image	*/
    long image_band_num[][MAXBND + 1], /* band numbers in each input image */
    long wind[][4],           	/* input image windows              	*/
    long sampsize[], 	     	/* sample block size	         	*/
    long *percent,               /* sampling percentage                  */
    char duplicate[]	     	/* duplicate samples flag           	*/
)

/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS


***************************************************************************
*/
{
char	minmax[4];		/* minmax option */
static	char	*prog_name = "randsamp";

long	count,			/* counts the number of array elements */
	fatal = -1,             /* fatal flag         */
	flag[MAXIMG][2],	/* flag for subwindows */
	imglimit = MAXIMG,	/* maximum number of images */
	itype = ELONG,		/* parameter data type */
	len,			/* string length */
	num_outfiles = 0,       /* number of output files  */
	same_window,		/* flag indicating if images have same window */
	stat,			/* return status */
	std_out,		/* logical unit for standard output */
	zero = 0;

struct VARIABLE	 *var;          /* pointer to a variable in TAE block */

/*
Initialize LAS.
----------------
*/
c_inlas (&block, &std_out);

/*
Retrieve and validate input image parameters.  If input windows are not
of the same size, abort with an error message.
------------------------------------------------------------------------
*/
stat = c_getin (&block, "IN", &zero, las_in, hos_in, num_inimages, wind, 
                image_band_num, num_image_bands, num_inbands, &imglimit, 
                &same_window, flag);

if (stat != E_SUCC) 
           c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);

if (!same_window)
           c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);

/*
Retrieve and validate output image parameters.
-----------------------------------------------
*/
stat = c_getout (&block, "OUT", las_out, hos_out);
if (stat != E_SUCC)
           c_errmsg("Fatal error encountered","randsamp-fatal",&fatal);

/*
Get the sample size.
--------------------
*/
count = 2;
c_getnum (&block, "KERNDIM", &count, &itype, sampsize);

/*
Check sample size against input image size.
--------------------------------------------
*/
if (wind[0][NS]  <  sampsize[1]  || wind[0][NL]  <  sampsize[0]) 
   c_abort (&block, "Sample block size is larger than input image size", 
            "randsamp-kerndim", prog_name, hos_out, &num_outfiles); 

/*
Get the sampling factor.
------------------------
*/
count = 1;
c_getnum (&block, "SAMPFACT", &count, &itype, percent);

/*
Get the duplication flag.
-------------------------
*/
len = 4;
c_getstr (&block, "DUPFLG", &count, &len, duplicate);

/*
Get LAS globals.
------------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;
return (E_SUCC);
}
