#include "randsamp.h"
#include "util.h"
#include "lasmath.h"

int selsamp
(
    long wind[][4],	/* image windows		*/
    long sampsize[2], 	/* sample block size		*/
    long percent,        /* sampling percentage          */
    char duplicate[],	/* duplication flag		*/
    long *nsamp, 	/* No. sample blocks		*/
    char hos_out[][CMLEN]	/* output image host name	*/
)

/*
***************************************************************************

NAME: SELSAMP

PURPOSE: Selects sample coordinates

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS


***************************************************************************
*/
{
long	dseed = 9274783,	/* seed for random number generation */
	num_outfiles=0,         /* number of output files */
	samples_per_row, 	/* number of samples per row */
	sample_rows, 		/* number of rows */
	total_samples;		/* total number of samples */

static	char 	*prog_name = "randsamp";


/*
Compute total number of samples in input image.
------------------------------------------------
*/
samples_per_row = wind[0][NS] / sampsize[1];
sample_rows = wind[0][NL] / sampsize[0];
total_samples = samples_per_row * sample_rows;

/*
Compute number of random samples required.
-------------------------------------------
*/
#ifdef gould
   *nsamp = (float) percent/100.0 * (float) total_samples + .001;
#else
   *nsamp = (long) ((float) percent/100.0 * (float) total_samples);
#endif
if (*nsamp  ==  0) 
   c_abort (&block, "The number of sample blocks is zero", "randsamp-zero",
            prog_name, hos_out, &num_outfiles); 
 
if ((samples = (long *) malloc (*nsamp * sizeof (long))) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "randsamp-alloc",
            prog_name, hos_out, &num_outfiles); 

/*
Generate random numbers.
-------------------------
*/
ggud (&dseed, total_samples, *nsamp, samples);

/*
Sort random samples into ascending order.
------------------------------------------
*/
sort (*nsamp, samples-1);

/*
If duplicate samples are not allowed, then call GENSAMP to regenerate
random samples as needed.
--------------------------------------------------------------------------
*/
if (duplicate[0] == 'N') 
   gensamp (nsamp, total_samples, &dseed);
return(E_SUCC);
}	
