
/*******************************************************************************
NAME:     errclean() 
PURPOSE:  errclean is called in the event of a fatal error.  It closes any open
	  images;  if $delflg="yes" output images are deleted;  a fatal error
	  message is issued.  This function can be called anytime after
	  getpar() regardless of which images are open, closed, or
	  nonexistent.  This function accepts a variable number of input
	  arguments, much like c_lasmsg does.  Usage:  errclean (messagekey,
	  sprintf format, args to format).
*******************************************************************************/

 
#include <stdarg.h>
#include <stdio.h>
#include "randsel.h"   /* application header file */ 
#include "util.h"
#include "protos.h"


/* vars common to errclean() and main(), declared common to prevent excessive 
   parm passing */ 
   extern struct INIMAGE  inimage,  /* primary input las image */ 
		          interin;  /* intermediate image, as input */ 
   extern struct OUTIMAGE interout, /* intermediate image, as output */ 
                          outimage; /* primary output las image */ 



void errclean (char *mkey, char *format, ...)
{
   va_list argumentp;      /* lists a variable number of function arguments */ 
   char message[3*ERRLEN]; /* text of message printed by c_lasmsg() */

/* get input variables and format message string */ 
   va_start (argumentp,format);  
   vsprintf (message, format, argumentp); 
   va_end (argumentp);

/* close any open image files and free buffers */
   if ( inimage.opened && ! inimage.closed)  close_inimage (&inimage); 
   if ( interin.opened && ! interin.closed)  close_inimage (&interin); 
   if (interout.opened && !interout.closed)  close_outimage(&interout); 
   if (outimage.opened && !outimage.closed)  close_outimage(&outimage); 

/* optionally delete intermediate and output images */ 
   if ( interin.opened || interout.opened)  c_delfil(interout.hostname[0]); 
   if (outimage.opened && outimage.delflg)  c_delfil(outimage.hostname[0]); 

/* issue fatal error and terminate processing */ 
   c_lasmsg (TRUE, mkey, message); 
}



