
/******************************************************************************
NAME:     getpar
PURPOSE:  get most user tae parameters.  other tae parms are referenced by 
	  image data structures
******************************************************************************/


#include <string.h>   /* for call to strchr(), strcmp() */
#include "randsel.h"  /* application header file */ 
#include "util.h"
#include "protos.h"


void getpar 
(
   struct PARBLK *pblock,  /* input:   tae parameter block */ 
   struct UPARMS *uparms   /* output:  user parms */ 
)
{
/* declare variables */ 
   char  hostname[CMLEN],   /* host name for OUT */ 
        *part2,             /* the second part of umask to parse */ 
	 taename[CMLEN],    /* tae name for OUT */
        *umask[MASKCOUNT];  /* maskval as input by the user */ 
   long parmval;            /* for loop index, parameter value */ 
   struct VARIABLE *tparm;  /* finds a tae parm in pblock */ 

/* get easy-to-parse parms */ 
   tparm = p_find (pblock, "fillval");    uparms->fillval  = RVAL (*tparm,0);
   tparm = p_find (pblock, "samperc");    uparms->samperc  = RVAL (*tparm,0);
   tparm = p_find (pblock, "maskband");   uparms->maskband = IVAL (*tparm,0); 

/* get seedval */ 
   tparm = p_find (pblock, "seedval");
   uparms->seedcount = tparm->v_count;
   if (uparms->seedcount)
      uparms->seedval = IVAL (*tparm,0);

/* get user values for maskval */ 
   tparm = p_find (pblock, "maskval");
   uparms->maskcount = tparm->v_count;
   for (parmval=0;  parmval<uparms->maskcount;  parmval++)
      umask[parmval] = SVAL (*tparm,parmval);

/* parse user values of maskval.  User may enter single real numbers or data
   ranges for maskval.  A dash (-) can not be used for data ranges (i.e. 4-5) 
   as it would be confused with the negative sign.  ':' is used instead. */ 

   for (parmval=0;  parmval<uparms->maskcount;  parmval++)
      {
      /* find min mask value */ 
	 if (sscanf (umask[parmval], "%f", &uparms->minmask[parmval] ) != 1)
	    c_lasmsg (TRUE, "randsel-badmask", 
		     "An invalid value of %s was entered for MASKVAL", 
		     umask[parmval] ); 

      /* see if user gave a max value */ 
	 part2 = strchr (umask[parmval], ':');   /* find location of ':' */ 
	 if (part2 == NULL) /* set max to min value */ 
	    uparms->maxmask [parmval] = uparms->minmask [parmval] ;
	 else if (sscanf (++part2, "%f", &uparms->maxmask[parmval] ) != 1)
	    c_lasmsg (TRUE, "randsel-badmask", 
	             "An invalid value of %s was entered for MASKVAL", 
		     umask[parmval] ); 

      /* error check:  is min < max ? */ 
	 if (uparms->maxmask[parmval] < uparms->minmask[parmval]) 
	    c_lasmsg (TRUE, "randsel-mask", 
		   "Maximum value %f for MASKVAL is less than minimum value %f",
		   uparms->maxmask[parmval], uparms->minmask[parmval]); 
      }

/* error check:  user should run las>copy */ 
   if (uparms->samperc == (float)100 && uparms->maskcount == 0)
      c_lasmsg (FALSE, "randsel-copy",
               "The input image will be copied without change");

/* get output image name just to make sure it does not exist on disk.  
   image will be opened in open_bil_out() */ 
   if (c_getout (pblock, "OUT", (char (*)[256])taename, 
                (char (*)[256])hostname) !=E_SUCC)
      c_lasmsg (TRUE, "randsel-fatal", "Fatal error encountered"); 
}



