
/*******************************************************************************
NAME:     image.h
PURPOSE:  header file for image IO 
	  includes data structures for LAS images, 
	  symbolic constants 
*******************************************************************************/



/* types of image IO allowed for */
   enum iotypes 
      {
      BILIO,   /* BIL processing, uses LAS sequential IO */ 
      BSQIO    /* BSQ processing, uses LAS random     IO */ 
      };



 
/* define macros for image data sizes, macros really belong in worgen.h */
#define EBYTESIZE (sizeof(unsigned char))
#define EWORDSIZE (sizeof(short))
#define ELONGSIZE (sizeof(long))
#define EREALSIZE (sizeof(float))



/* a note on data types: 
   An image can potentially have 3 different data types:
   dtypein:  data type as exists in input disk image
   dtypeap:  data type as processed internally by the application
   dtypeout:  data type as will exist in output disk image.  Data types
   can be EBYTE, EWORD, ELONG, EREAL. */ 



/* data structure for one LAS input image, used to facilitate IO.  Because LAS
   image specs allow for grouping image components, (e.g. in="image1 + image2")
   one input image may correspond to multiple disk image files.  The same
   structure is used for all types of IO, although all structure members are 
   not always used. */
   struct INIMAGE
      {
      char hostname[MAXIMG][CMLEN],/* host name for image files */ 
           taename[MAXIMG][CMLEN]; /* tae name for image files */ 
      int  fdarray[MAXIMG+1],      /* file descriptor array */ 
           gdesc;                  /* group descriptor, used for BILIO */ 
      long bands[MAXIMG][MAXBND+1],/* band numbers for each image file */ 
           bufoff[MAXBND],         /* offsets into data buffer, for BILIO */ 
           cblut[MAXIMG*MAXBND],   /* a lut for current band, for BSQIO */ 
 	   dsizeap,                /* # bytes per pixel as seen by the app */
 	   dtypeap,                /* image data type as seen by the app */ 
 	   dtypein,                /* image data type as read = EBYTE, ... */
           fdlut[MAXIMG*MAXBND],   /* a lut for file descriptor, for BSQIO */
           iotype,                 /* type of io, = BILIO, BSQIO */ 
           nbands[MAXIMG],         /* number of bands in each image file */ 
           nmrimg,                 /* number of disk image files */ 
           nl, ns,                 /* size of input image = window[][NL], 
				      window[][NS] */ 
           totalbands,             /* total # of bands for all image files */
           window[MAXIMG][4];      /* window (sl ss nl ns) for each image file*/
      unsigned char *buffer,       /* buffer for image data */ 
	       *linebuf[MAXBND];   /* location in buffer for each line/band of 
				      data, for BILIO */ 
      /* status true/false flags used for error handling */ 
      long closed,                 /* has the image been closed */ 
	   opened;                 /* has the image been opened */
      }; 



/* data structure for one LAS output image, used to facilitate IO, 
   structure includes some tae globals */ 
   struct OUTIMAGE
      {
      char hostname[1][CMLEN],     /* host name for image file, supporting
                                      routines expect a 2-d array */ 
           taename[1][CMLEN];      /* tae name for image, supporting 
				      routines expect a 2-d array */
      int  fdarray[1+1],           /* file descriptor array */ 
           gdesc;                  /* group descriptor for BILIO */ 
      long bufoff[MAXBND],         /* offsets into data buffer, for BILIO */
 	   dsizeap,                /* # bytes per pixel as seen by the app */
 	   dtypeap,                /* image data type as seen by the app */ 
 	   dtypeout,               /* image data type to write = EBYTE, ... */
           iotype,                 /* type of io, = BILIO, BSQIO */ 
           minmax,                 /* tae global $minmax, flag, determine 
				      min/max for ddr? = T,F */
	   msgtim,                 /* tae global $msgtim, time interval for
	                              processing message */ 
           nl, ns,                 /* size of output image */ 
           totalbands;             /* total number of bands */ 
      unsigned char *buffer,       /* buffer for image data */ 
	       *linebuf[MAXBND];   /* location in buffer for each line/band of 
				      data, for BILIO */ 
      double minval[MAXBND],       /* min pixel value, used to update ddr */ 
             maxval[MAXBND];       /* max pixel value, used to update ddr */ 
      /* status flags used for error handing, = true/false */ 
      long closed,                 /* has the image been closed */ 
	   delflg,                 /* tae global $delflg, delete images if err*/
	   opened;                 /* has the image been opened */
      }; 



