
/******************************************************************************
NAME:     image_size
PURPOSE:  return size of output image
          see bottom of file for notes on image size calculations 
******************************************************************************/


#include <math.h>     /* for call to ceil(), sqrt() */ 
#include "randsel.h"  /* application header file */ 
#include "protos.h"


void image_size 
(
   long selcount[],           /* input:  count of n pixels selected for copying 
                                         to output image */ 
   struct INIMAGE *inimage,   /* input:  input image info, needed for nl and 
					 ns */
   struct OUTIMAGE *outimage  /* output: output image info, nl and ns are 
					 updated */
)
{

/* declare variables */ 
   double inpixels,  /* number of pixels in input image, could exceed range of 
		        long data type */ 
	  fraction,  /* fraction of input pixels which were selected */ 
          totsel;    /* total number of pixels selected */ 
   long   line;      /* for loop index, line in input image */ 

/* determine total number of pixels selected */ 
   for (totsel=0.0, line=0;  line<inimage->nl;  line++)
      totsel += (double) selcount[line]; 

/* error check:  output image has zero pixels */ 
   if (totsel == 0.0)
      errclean ("randsel-noout", "No output image was created"); 

/* determine fraction of pixels selected */ 
   inpixels = (double) inimage->nl * (double) inimage->ns;
   fraction = totsel / inpixels; 

/* set image size to maintain input image width to height ratio */ 
   outimage->nl = (long) (inimage->nl * sqrt (fraction) + .4999);  
   outimage->ns = (long) (ceil (totsel / outimage->nl) ); 
   while ( (double)(outimage->nl -1) * (double)outimage->ns > totsel)
      outimage->nl --; 

/* user message: image size */ 
   c_lasmsg (FALSE, "informational", 
	    "Input image size is %d lines %d samples %d bands", 
	    inimage->nl, inimage->ns, inimage->totalbands); 
   c_lasmsg (FALSE, "informational", 
	    "Output image size will be %d lines %d samples %d bands", 
	    outimage->nl, outimage->ns, inimage->totalbands);  

/* error checks */ 
   if (outimage->nl == 0 || outimage->ns == 0                 || 
       outimage->nl * outimage->ns < totsel                   ||
       outimage->nl > inimage->nl || outimage->ns > inimage->ns)
      errclean ("randsel-size", "Error sizing output image"); 
}














#ifdef ThisIsNotUsed
   /* set image size to maintain input image width to height ratio.
      derivation:  
      inpixels = in->nl * in->ns 
      outpixels = fraction * inpixels = fraction * in->nl * in->ns 
      outpixels = fraction * inpixels = sqrt(fraction)* in->nl  *  
					sqrt(fraction)* in->ns
      out->nl is set first, then ns is calculated from the total n pixels.
      For images where nl >> ns, the unusual shape can result in too many image 
      lines.  Hence the while loop.  Don't replace while with if! */ 


   /* This code is not used because it does not accomplish anything.  The
      algorithm in final_image() shifts pixels from one line to another and
      thus makes visual interpretation impossible. */
   /* set image size to maintain visual recognition.  ns must be divided by an
      integer */ 
      double divisor;
      divisor = (double)inpixels / (double)totsel;
      divisor = sqrt (divisor);   /* divide change between nl and ns */ 
      divisor = floor (divisor);  /* round down so image size is never 
				     increased */
      outimage->ns = inimage->ns / divisor; 
      outimage->nl = (long) (ceil (totsel / outimage->ns) ); 
#endif 


