
/******************************************************************************
NAME:     make_final
PURPOSE:  make the final output image, 
	  process image with BIL logic, use sequential IO
******************************************************************************/

#include "randsel.h" /* application header file */ 
#include "util.h"
#include "protos.h"


void make_final
(
   float *fillval,             /* input:  value to fill last image line, float *
                                          (instead of float) to avoid a problem 
				          on sgi platform */
   struct INIMAGE  *inimage,   /* input:  info on input las image */ 
   struct INIMAGE  *interin,   /* input:  info on intermediate las image */
   struct OUTIMAGE *outimage   /* input:  info on output las image */ 
)
{

/* declare vars */ 
   long band,                /* for loop index:  image band */ 
	iinline,              /* for loop index:  input image line */ 
	insamp,              /* for loop index:  input image sample */ 
        messform=ALLCAT,     /* output message format, arg to c_startm() */ 
	outline = 0,         /* count of output image lines */ 
	outsamp = 0;         /* for loop index:  output image sample */ 
   unsigned char *inbuf,     /* location of image pixel in inimage->buffer */
		 *intbuf,    /* location of image pixel in interin->buffer */
		 *outbuf;    /* location of image pixel in outimage->buffer */
   extern long curent;       /* current entity, for processing message */

/* initialize and start processing line message */ 
   c_lasmsg (FALSE, "informational", "Creating output image");
   curent = 1; 
   c_inittm ("RANDSEL", "Processed", "line", "bands", "", &outimage->nl,
            &outimage->totalbands, &outimage->msgtim, &messform); 
   c_startm (); 


/* loop for all input image lines (in and out images are different sizes) */
   for (iinline=0;  iinline<inimage->nl;  iinline++) 
      {
      /* loop for all samples */
	 for (insamp=0;  insamp<inimage->ns;  insamp++)
	    {
	    /* skip this input pixel if it was rejected, selected pixels have
	       a logical true value (SELPIX), intermediate image is byte and 
	       has one band */

	       intbuf = interin->linebuf[0] + insamp * interin->dsizeap;
	       if ( ! *intbuf)
		  continue; 

	    /* loop for all bands */ 
	       for (band=0;  band<inimage->totalbands;  band++)
		  {

		  /* get buffer locations */ 
		     inbuf = inimage->linebuf[band] + insamp * inimage->dsizeap;
		     outbuf=outimage->linebuf[band] +outsamp *outimage->dsizeap;

		  /* copy input pixel to output buffer.  optional data type 
		     conversion occurs when image data is written to disk 
		     image */ 
		     switch (inimage->dtypeap)
			{
			case EBYTE: *         outbuf = *         inbuf;  break; 
			case EWORD: *(short *)outbuf = *(short *)inbuf;  break;
			case ELONG: *(long  *)outbuf = *(long  *)inbuf;  break; 
			case EREAL: *(float *)outbuf = *(float *)inbuf;  break; 
			default:  errclean ("randsel-fatal", 
				  "Program error:  unknown image data type");
			}

		  } /* end loop for band */ 

	       /* increment output sample */
		  outsamp++; 

	       /* at the end of each line, update minmax, write data, ... */ 
		  if (outsamp == outimage->ns)
		     {
			minmax_bil (outimage); /* must come before write */ 
			write_bil  (outimage); 
			outsamp = 0;  
			outline++;
			curent++; 
		     }

	    } /* end loop for input samples */ 

      /* get more input data */ 
	 read_bil (inimage);
	 read_bil (interin); 

      } /* end loop for input lines */ 



/* optionally complete the last image line with fillval 
******************************************************/
   if (outline!=outimage->nl) 
   {
      /* loop for all samples */ 
	 for (  ;  outsamp<outimage->ns;  outsamp++)
	 /* loop for all bands */ 
	    for (band=0;  band<inimage->totalbands;  band++)
	       {
	       /* get buffer location */ 
		  outbuf = outimage->linebuf[band]+ outsamp * outimage->dsizeap;
	       /* copy data to output buffer */ 
		  switch (outimage->dtypeap)
		     {
		     case EBYTE: *          outbuf = *fillval;  break;
		     case EWORD: *(short *) outbuf = *fillval;  break; 
		     case ELONG: *(long  *) outbuf = (long)*fillval;  break;
		     case EREAL: *(float *) outbuf = *fillval;  break;
		     default:  errclean ("randsel-fatal", 
			       "Program error:  unknown image data type");
		     }
	       } 

      /* update minmax and write data */ 
	 minmax_bil (outimage); 
	 write_bil (outimage); 
	 curent++; 
   }

c_stoptm(); 
}




