
/******************************************************************************
NAME:     make_inter
PURPOSE:  make the intermediate byte image, 
          process the images with BSQ logic, use random IO
******************************************************************************/


#include <math.h>    /* for call to pow() */ 
#include "randsel.h" /* application header file */ 
#include "util.h"
#include "protos.h"

void make_inter
(
   struct UPARMS   *uparms,   /* input:   user tae parms */ 
   struct INIMAGE  *inimage,  /* input:   specs for input image */ 
   struct OUTIMAGE *outimage, /* input:   specs for output intermediate image */
   long selcount[]            /* output:  count of n pixels selected for copying
                                          to output image */ 
				       
)
{


/* declare vars */ 
   double randmax;         /* max number output from random() */
   float pixval,           /* input pixel value */ 
         rscalar;          /* scalar for random(), converts random() output to 
			      percent probability of acceptance */
   long line,              /* for loop index:  image line */ 
        makerand,          /* flag, make random numbers, 
			      is uparms->samperc!=100? */ 
	masked,            /* flag, is this pixel masked */ 
	masknum,           /* for loop index, mask number */ 
        messform=GEN2,     /* output message format, parm to c_inittm() */ 
	mb_read,           /* maskband to read, 0 relative, not 1 */ 
	sample,            /* for loop index:  image sample */ 
	zero = 0;          /* value of zero */ 
   unsigned char *inbuf,   /* location of image pixel in inimage->buffer */ 
                 *outbuf;  /* location of image pixel in outimage->buffer */ 
   extern long curent;     /* current entity, for processing message */ 


/* set makerand flag, it saves processing time */ 
   if (uparms->samperc != (float) 100)
      makerand = TRUE; 
   else
      makerand = FALSE;  

/* set mb_read and rscalar */ 
   mb_read = uparms->maskband -1; 
   randmax = pow (2.0, 31.0)  -1.0; 
   rscalar = 100.0 / randmax;  

/* seed random number generation */ 
   seed_rgen (uparms); 

/* initialize and start processing line message */ 
   c_lasmsg (FALSE, "informational", "Creating intermediate image"); 
   curent=1; 
   c_inittm ("RANDSEL", "Analyzed", "line", "", "for MASKBAND",
            &outimage->nl, &zero, &outimage->msgtim, &messform);
   c_startm (); 

/* process image */ 
   for (line=0; line<inimage->nl; line++, curent++)   
      {

      /* initialize selcount */
	 selcount[line] = 0; 

      /* read data */ 
	 read_bsq (line, mb_read, inimage); 

      /* process image line */ 
	 for (sample=0, inbuf=inimage->buffer, outbuf=outimage->buffer; 
	      sample<inimage->ns; 
	      sample++, inbuf+= inimage->dsizeap, outbuf+= outimage->dsizeap)
	    {


	    /* get input pixel value */ 
	       switch (inimage->dtypeap)
		  {
		  case EBYTE: pixval = *          inbuf;  break;
		  case EWORD: pixval = *(short *) inbuf;  break;
		  case ELONG: pixval = *(long  *) inbuf;  break;
		  case EREAL: pixval = *(float *) inbuf;  break;
		  default:  errclean ("randsel-fatal", 
			    "Program error:  unknown image data type");
		  }

	    /* process mask values */ 
	       if (uparms->maskcount)
		  {
		  /* initialize masked flag */ 
		     masked = FALSE; 
		  /* see if input pixels are masked */ 
		     for (masknum=0;  masknum<uparms->maskcount;  masknum++)
			if (pixval >= uparms->minmask[masknum] &&
			    pixval <= uparms->maxmask[masknum]  )
			       {
			       masked = TRUE;
			       break;  
			       }
		  /* reject masked pixels */
		     if (masked)
			{
			*outbuf = REJPIX;
			continue;
			}
		  }

	    /* randomly select some non-masked pixels.  rand() is faster (and 
	       not as good) as random() but has nonstandard output on sun 
	       machines. */ 
#ifndef sun
	       if (makerand && (float)random() * rscalar > uparms->samperc)
#else
	       if (makerand && rand() * rscalar > uparms->samperc)
#endif
		  *outbuf = REJPIX;
	       else
		  {
		  *outbuf = SELPIX;   /* pixel was selected */ 
		  selcount[line]++;   /* increment count of selected pixels */
		  }

	    } /* end loop for each sample */

	 /* write data.  no need to update min/max of intermediate image */
	    write_bsq (line, 0, outimage); 

      } /* end loop for each line */ 

c_stoptm(); 
}


