
/*******************************************************************************
NAME:    RANDSEL 
PURPOSE: program main()
	 RANDSEL copies randomly-selected non-masked pixels from an input to an 
	 output image.
 
PROGRAM HISTORY
-------------------------------------------------------------------------------
  Date      Programmer                          Changes 
--------  ---------------  ----------------------------------------------------
Jul-1995  David Zokaites   Original development 
Aug-1995  David Zokaites   Many minor changes after application received
                           internal review 
Dec-1999  D Hames	   Move function prototypes from randsel.h and image.h
			   into new file protos.h. Remove MAXNL from selcount
			   and allocate using inimage.nl. This was done to 
			   support images greater than 2gig. SRF #893
*******************************************************************************/




#include <string.h>  /* for call to strcpy() */ 
#include "randsel.h" /* application header file */ 
#include "util.h"
#include "protos.h"

/* vars common to main() and errclean().  If other images are processed, 
   change errclean.c */ 
   struct INIMAGE  inimage,  /* primary input las image */ 
		   interin;  /* intermediate image, as input */ 
   struct OUTIMAGE interout, /* intermediate image, as output */ 
                   outimage; /* primary output las image */ 





void main ()                 
{	                    
/* declare vars */ 
   long one = 1,         /* number of output files */ 
	*selcount,       /* count of number pixels selected for copying to 
			    output image, must be long array to avoid loss of
			    precision and type overflow */ 
        std_out;         /* standard output, parm to c_inlas() */ 
   struct PARBLK pblock; /* tae parameter block */ 
   struct UPARMS uparms; /* holds most user tae parms, other parm are in image
                            data structures */ 

/* initialize las and tae, get most user tae parms */ 
   c_inlas (&pblock, &std_out); 
   getpar (&pblock, &uparms); 

/* make intermediate image with random IO.  intermediate is necessary because 
   image size is not known in advance, disk images can not be dynamically 
   resized. */
   /* initialize image status flags, must be first in this section for
      correct error handling */ 
      inimage.opened  = inimage.closed  = FALSE; 
      interin.opened  = interin.closed  = FALSE; 
      interout.opened = interout.closed = FALSE; 
      outimage.opened = outimage.closed = FALSE; 

   /* get name of intermediate image */ 
      name_image (interout.hostname[0], interout.taename[0]); 

   /* open images, input must be opened before output. opening output requires
      knowing input image size. */ 
      open_bsq_in  (&pblock, "IN", 0, &inimage); 
      if (uparms.maskband > inimage.totalbands)
	 errclean ("randsel-badband", 
		  "MASKBAND (%d) exceeds number of bands in IN (%d)",
		  uparms.maskband, inimage.totalbands);
      open_bsq_out (&pblock, NULL, EBYTE, EBYTE, FALSE, inimage.nl, 
      inimage.ns, 1, &interout); 
   
   /* allocate memory */
   if ((selcount = (long *) malloc(inimage.nl * sizeof(long))) == NULL)
      errclean ("randsel-memory",
                "Unable to allocate memory for output image buffer");
 
   /* create image flagging which pixels to copy to final image */ 
      make_inter (&uparms, &inimage, &interout, selcount); 

   /* close images */ 
      close_inimage (&inimage);         /* close image files */ 
      finish_out (&pblock, &interout);  /* close image, make ddr & his file */

/* create final image with sequential IO */ 
   /* reset image status flags, must be first in this section for correct error
      handling, don't reset flags in interout as the program is done with
      interout */ 
      inimage.opened  = inimage.closed  = FALSE; 
      interin.opened  = interin.closed  = FALSE; 
      outimage.opened = outimage.closed = FALSE; 

   /* get name of intermediate image */ 
      strcpy (interin.hostname[0], interout.hostname[0]);
      strcpy (interin.taename [0], interout.taename [0]); 

   /* open input images */ 
      open_bil_in (&pblock, "IN", 0, &inimage); 
      open_bil_in (&pblock, NULL, 0, &interin); 

   /* determine size of output image */ 
      image_size (selcount, &inimage, &outimage); 

   /* open output image */ 
      open_bil_out (&pblock, "OUT", inimage.dtypein, TRUE, &inimage, &outimage);

   /* create output image */ 
      make_final (&uparms.fillval, &inimage, &interin, &outimage); 

   /* close images */ 
      close_inimage (&inimage);        /* close image files */ 
      close_inimage (&interin);        /* close image files */ 
      finish_out (&pblock, &outimage); /* close image, make ddr & history file*/

/* delete intermediate image and end program */ 
   c_delfil (interin.hostname[0]); 
   c_complt ("RANDSEL", outimage.taename, &one);  

/* free buffer */
   free (selcount);  
}







#ifdef ANOTE /* this ifdef'ed function call will pop up from    % lasmsg */ 
   c_lasmsg (ReadThisNote, 
   The unix application lasmsg can not be used directly on this application
   because of the function errclean.  Run tests/lasmsg2 instead.   );
#endif 

