#include "reclass.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char lasin[MAXIMG][CMLEN],	/* O: Input image file LAS name               */
    char hosin[MAXIMG][CMLEN],	/* O: Input image file host name              */
    long bands[][MAXBND+1],	/* O: Band numbers                            */
    long *nimages,              /* O: no. of images                           */
    long nbands[],              /* O: no. of bands in each image              */
    char lasout[CMLEN],		/* O: Output image file LAS name              */
    char hosout[CMLEN],		/* O: Output image file host name             */
    long class[],		/* O: Classes to be weighed                   */
    float weight[],		/* O: Weight given to each class              */
    long nore[],		/* O: Classes not to be smoothed              */
    long *thresh,		/* O: Smoothing threshold value               */
    long wind[][4],		/* O: image window                            */
    long fcellsiz[],		/* O: filter cell size(LAS parameter 'WINDOW')*/
    long *msgtim,		/* O: Processing line message time interval   */
    char minmax[]		/* O: Minmax flag string                      */
)
/******************************************************************************

NAME:	getpar

FUNCTION:
	Get LAS parameters

PROGRAM HISTORY:
  Version	Date       Author      	   Request
  -------	----	   ------          -------
    1.0       13-Oct-83    L. Urquhart
    2.0       24-Dec-87    N. Deodhar/SAR  New LAS & conversion to C
    3.0       12-Sep-88    B. Allam/SAR    Retrocode
    4.0	      23-Sep-92	   T. Mittan	   Used standard parmeter names

ALGORITHM (FLOW):
	Get input image file
	Get output image file
	Get LAS globals $minmax, $msgtim
	Get filter cell window size
	Get class arrays
	Get class weights
	Get noreclass arrays
	Return

******************************************************************************/

{
long	i,			/* Loop index                                 */
	imglimit,		/* Maximum number of input images allowed     */
        defw = 0,
	j,			/* Loop index                                 */
	totbnd,			/* Total number of bands specified            */
	same_wind,		/* Same window flag for getin                 */
        flag = 0,
	qflag[2];		/* Flag indicating if subwindow specified     */

struct VARIABLE *var, *vw, *vt, *vc, *vd, *vn;

/*
   Initialize
   ----------
*/
for (i = 0; i < NUM_CLASSES; i++)
   class[i] = nore[i] = NULL_CLASS;
/*
   Get input image
   ---------------
*/
imglimit = 1;
if (c_getin (&vblock, "IN", &zero, lasin, hosin, nimages, wind, bands,
            nbands,&totbnd,&imglimit,&same_wind,(long (*)[2])qflag) != E_SUCC)
   iso_err ("!Unable to retrieve input image info", "reclass-inerr");

if (totbnd != 1)
   iso_err ("!Input image must be single band", "reclass-numbnd");
/*
   Get output image
   ----------------
*/
if (c_getout(&vblock,"OUT",(char (*)[256])lasout,(char (*)[256])hosout)!=E_SUCC)
   iso_err ("!Error geting output image name", "reclass-outerr");
/*
   Get the important LAS globals
   -----------------------------
*/
var = p_find(&vblock, "$msgtim");
*msgtim = IVAL(*var,0);

var = p_find(&vblock, "$minmax");
strcpy(minmax, SVAL(*var,0));
/*
   Get other parameters
   --------------------
*/
vw = p_find(&vblock, "WINDOW");
fcellsiz[NL] = IVAL(*vw, 0);
fcellsiz[NS] = IVAL(*vw, 1);

vt = p_find(&vblock, "SMTHRVAL");
*thresh = IVAL(*vt, 0);
/*
   Get class numbers and weights
   -----------------------------
*/
   vc = p_find(&vblock, "CLASSNUM");
   if ((*vc).v_count) {
     class_count = (*vc).v_count;
     for (j = 0; j < (*vc).v_count; j++)
       class[j] = IVAL(*vc, j);
   flag = 1;
   }
   else {
     if (!flag)
      defw = 1;
   }
/*
   Get weights
   -----------
*/
if (defw)
  for (i = 0; i < NUM_CLASSES; i++)
    weight[i] = 1.0;
else {
     vd = p_find(&vblock, "WEIGHTS");
     if ((*vd).v_count != class_count)
        iso_err ("!Number of weights don't correspond to number of classes",
                 "reclass-numw");
     if ((*vd).v_count)   
       for (j = 0; j < (*vd).v_count; j++)
         weight[j] = RVAL(*vd, j);
     else
       for (j = 0; j < class_count; j++)
         weight[j] = 1.0;
    }
/*
   Get nore
   --------
*/
   vn = p_find(&vblock, "MASKVAL");
   if ((*vn).v_count)
     for (j = 0; j < (*vn).v_count; j++)
       nore[j] = IVAL(*vn, j);

return;
}
