#include "reclass.h"
#include "pixman.h"
#include "protos.h"

/******************************************************************************

NAME:	process_beginning

FUNCTION:
	Read in some lines and write out some lines as they are in the
	input image.

PROGRAM HISTORY:
  Version	Date       Author      	   Request
  -------	----	   ------          -------
    1.0       13-Oct-83    L. Urquhart
    2.0       24-Dec-87    N. Deodhar/SAR  New LAS & conversion to C
    3.0       12-Sep-88    B. Allam/SAR    Retrocode

ALGORITHM (FLOW):
	Read in the number of lines specified by the cell size
	Write out half that number of lines as they are

******************************************************************************/

void process_beginning
(
    long wind[][4],		/* I: Input image window specified            */
    long fcellsiz[],		/* I: Filter cell size specified              */
    long mnmxflag,		/* I: Minmax flag                             */
    double *minval,		/* I/O: Minimum value                         */
    double *maxval		/* I/O: Maximum value                         */
)
{
long	i,			/* Loop index                                 */
	j;			/* Lines to copy as is                        */

/*
   Loop through fcellsiz[NL] lines
   -------------------------------
*/
for (i = 0; i < fcellsiz[NL]; i++)
   {
/*
   Read line into inbuf and move into buffer
   -----------------------------------------
*/
   if (c_estep ((struct GDESC **)&ingdesc) != E_SUCC)
     {
      if (outgdesc) c_egclse((struct GDESC **)&outgdesc);
      iso_err ("!Error stepping through image", "reclass-instep");
     }
   c_pxcopy ((unsigned char *)&inbuf[inoff],
              (unsigned char *)&buffer[i * wind[0][NS]], &dtype, &wind[0][NS]);
   }
/*
   Write data from edge as it is
   -----------------------------
*/
j = fcellsiz[NL] / 2;
for (i = 0; i < j; i++)
   {
   c_pxcopy ((unsigned char *)&buffer[i * wind[0][NS]], (unsigned char *)&outbuf[outoff], &dtype,
             &wind[0][NS]);
   if (mnmxflag)
      c_pxmnmx ((long *)&outbuf[outoff], &wind[0][NS], minval, maxval, &dtype);
   if (c_estep ((struct GDESC **)&outgdesc) != E_SUCC)
     {
      if (outgdesc) c_egclse((struct GDESC **)&outgdesc);
      iso_err ("!Error stepping through image", "reclass-outstep");
     }
   curent++;
   }
return;
}
