#include "reclass.h"
#include "pixman.h"
#include "protos.h"

/******************************************************************************

NAME:	process_bottom

FUNCTION:
	Read in the bottom lines and write out as they are.

PROGRAM HISTORY:
  Version	Date       Author      	   Request
  -------	----	   ------          -------
    1.0       13-Oct-83    L. Urquhart
    2.0       24-Dec-87    N. Deodhar/SAR  New LAS & conversion to C
    3.0       12-Sep-88    B. Allam/SAR    Retrocode

ALGORITHM (FLOW):
	Read in lines from input image
	Write out lines as they are to output image

******************************************************************************/

void process_bottom
(
    long wind[][4],		/* I: Input image window specified            */
    long fcellsiz[],		/* I: Filter cell size specified              */
    long mnmxflag,		/* I: Minmax flag                             */
    double *minval,		/* I/O: Minimum value                         */
    double *maxval		/* I/O: Maximum value                         */
)
{
long	i,			/* Loop index                                 */
	j;			/* Number of lines already processed          */
/*
   Write out bottom lines
   ----------------------
*/
j = wind[0][NL] - fcellsiz[NL] / 2;

for (i = j; i < wind[0][NL]; i++)
   {
/*
   Move pixels from inbuf into buffer, and back into outbuf
   --------------------------------------------------------
*/
   c_pxcopy ((unsigned char *)&inbuf[inoff], (unsigned char *)&buffer[0],
             &dtype, &wind[0][NS]);
   c_pxcopy ((unsigned char *)&buffer[0], (unsigned char *)&outbuf[outoff], 
              &dtype, &wind[0][NS]);
/*
   Write data as is
   ----------------
*/
   if (mnmxflag)
      c_pxmnmx ((long *)&outbuf[outoff], &wind[0][NS], minval, maxval, &dtype);
   if (c_estep ((struct GDESC **)&outgdesc) != E_SUCC)
     {
      if (outgdesc) c_egclse((struct GDESC **)&outgdesc);
      iso_err ("!Error stepping through image", "reclass-outstep");
     }
   curent++;
/*
   Read line into inbuf
   --------------------
*/
   if (c_estep ((struct GDESC **)&ingdesc) != E_SUCC)
     {
      if (outgdesc) c_egclse((struct GDESC **)&outgdesc);
      iso_err ("!Error stepping through image", "reclass-instep");
     }
   }
return;
}
