/*******************************************************************************
NAME:		GETPAR	

PURPOSE:    	GETPAR gets parameters from tae parameter block	
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	9/90	Re-retrocode (All new C version!)
5.1	S.Nelson	5/92	Updated error messages.
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "reglabel.h"

void getpar
(
  struct PARBLK *vblock,        /* I: Pointer to TAE parameter block */
  char hosin[][CMLEN],          /* O: Host file names of input images */
  char *taeout,                 /* O: TAE file name of output image */
  char *hosout,                 /* O: Host file name of output image */
  char *regionlt,               /* O: Region description labeled table */
  long *nmrimg,                 /* O: Number of input images */
  long window[][4],             /* O: Window specs for each input image */
  long bands[][MAXBND+1],       /* O: Band specs for each input image */
  long *nbands,                 /* O: Number of bands per input image */
  long *totbnd,                 /* O: Total number of bands to be processed */
  long *eightway,               /* O: Check eight-way connectedness flag */
  long *msgtim,                 /* O: Elapsed time between display messages */
  char *minmax                  /* O: Min/Max flag buffer */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block */

long	idtype;			/* input image data type */
long	element = 0;		/* parameter element number */
long    fatal = -1;		/* fatal error flag */
long	nonfatal = 1;		/* nonfatal error flag */
long	imglim = 1;		/* maximum number of images to be processed */
long    one = 1;		/* value of one	*/
long	qflag[1][2];		/* flag indicating if subwindows or bands 
							were specified */
long	same;			/* flag indicating if images have same window*/

char    connect[6];		/* for connectedness string (four | eight) */
char 	msg[ERRLEN+1];		/* message buffer */	
char 	taein[CMLEN];         	/* TAE file names of input images */

/* Retrieve and parse input image specification 
-----------------------------------------------*/
if (c_getin(vblock,"IN",&element,taein,hosin[0],nmrimg,window,bands,nbands,
		totbnd,&imglim,&same,qflag) != E_SUCC)
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);

/* Check for invalid number of bands 
------------------------------------*/
if (*totbnd != 1)
   {
   c_errmsg("Only single band input allowed","reglabel-nbands",&nonfatal);
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);
   }

/* Check for valid input data type
----------------------------------*/
if (c_gettyp(hosin[0],&one,&idtype) != E_SUCC)
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);

if (idtype != EBYTE && idtype != EWORD && idtype != ELONG)
   {
   sprintf(msg,"Image %s not of type byte, integer*2, or integer*4",hosin[0]);
   c_errmsg(msg,"reglabel-dtype",&nonfatal);
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);
   }

/* Retrieve output image name
-----------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) != E_SUCC)
   c_errmsg("Fatal error encountered","reglabel-fatal",&fatal);

/* Make labeled table file name 
-------------------------------*/
c_lsmknm(hosout,"lt",regionlt);

/* Retrieve connectedness flag
------------------------------*/
val = p_find(vblock,"CONNECT");
strcpy(connect,SVAL(*val,0));
if (strcmp(connect,"EIGHT") == 0)
   *eightway = TRUE;
else
   *eightway = FALSE;
 
/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));
}
