/*******************************************************************************
NAME:		LABEL	

PURPOSE:    	LABEL 	First step in labeling contiguous regions in image
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	9/90	Re-retrocode (All new C version!)
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "reglabel.h"

int label
(
  unsigned char *iobuf,       /* I: I/O buffer for image data */
  long nl,                    /* I: Number of lines to process */
  long ns,                    /* I: Number of samples to process */
  long eightway,              /* I: Eight way connectedness flag */
  long *regcls[],             /* O: Class identifier for each region */
  long *regcnt[],             /* O: Count of pixels in each region */
  long *reglnk[],             /* O: Subregion linkage to contiguous region */
  long *gross_nreg,           /* O: Gross number of regions--before linking */
  long *net_nreg              /* O: Net number of regions after linking */
)
{

/* The following variables have been declared global for image i/o 
------------------------------------------------------------------*/
extern 	int	gdesc;		/* group descriptors */	
extern	long	in_ptr[1];	/* pointer into "iobuf" for input bands */
extern	long	out_ptr[1];	/* pointer into "iobuf" for output bands */
extern 	long	curcat;		/* current processing pass */
extern	long	curent;		/* current processing line */

long	i;			/* loop counter */
long	j;			/* loop counter */
long	k;			/* loop counter */
long	nonfatal = 1;		/* nonfatal error flag */
long	ln;			/* line loop counter */
long	match;			/* matched region flag */
long    mxreg;                  /* Maximum number of regions for this image */
long	regnum = 0;		/* current region number */
long	reglbl;			/* region label used in processing */
long	nr;			/* number of runs */
long	*runs;		 	/* indexes into parsed line where runs start */
long	runval;			/* pixel value of current run */
long	strt;			/* start of segment on previous line */  
long	end;			/* end of segment on previous line */  
long	tmplnk1;		/* temporary link value */
long	tmplnk2;		/* temporary link value */

long	*inlin;			/* pointer to input line */
long	*outlin;		/* pointer to output line */
long 	*prvlin;		/* pointer to previous line */

/*****************************************************************************/

/* Alloc storage for region tracking buffers, run pointer array, previous line
------------------------------------------------------------------------------*/

/* The maximum number of regions (maxreg) is arbitrarily set to a number
   that is hopefully large enough to handle any realistic case encountered.
   This is a quick 'n dirty solution.  A better solution should be designed
   when a user encounters this limit, at which point a CR should be
   submitted by the user.
   ------------------------------------------------------------------------*/
mxreg = 100000;
 
if (((*regcls = (long *)malloc(sizeof(long)*(mxreg+1))) == NULL) || 
   ((*regcnt = (long *)malloc(sizeof(long)*(mxreg+1))) == NULL) ||
   ((*reglnk = (long *)malloc(sizeof(long)*(mxreg+1))) == NULL) ||
   ((runs = (long *)malloc(sizeof(long)*(ns+1))) == NULL) ||
   ((prvlin = (long *)malloc(sizeof(long)*ns)) == NULL))
   {
   c_errmsg("Error allocating dynamic memory","4.reglabel-alloc",&nonfatal);
   return (E_FAIL);
   }

/* Initialize region tracking buffers
-------------------------------------*/ 
for (i = 0; i <= mxreg; i++)
   (*regcls)[i] = (*regcnt)[i] = (*reglnk)[i] = 0;

/* Start processing messages
----------------------------*/
curcat = 1;
curent = 1;
c_startm();

/* Read first line in image 
---------------------------*/
if (c_estep(&gdesc) != E_SUCC)
   return (E_FAIL);

/* Set up pointers for cleaner programming 
------------------------------------------*/
inlin = (long *)&iobuf[in_ptr[0]*4]; 
outlin = (long *)&iobuf[out_ptr[0]*4];

/* Parse line to find runs of contiguous pixel values
-----------------------------------------------------*/
parse_ln(inlin,ns,&nr,runs);

/* Label first line
-------------------*/
for (i = 0; i < nr; i++)
/* For all runs in line
-----------------------*/
   {
   /* New region
   -------------*/
   regnum++;

   /* If we have exceeded the maximum number of regions, free memory and
      error out.
      ------------------------------------------------------------------*/
   if (regnum > mxreg)
      {
         /* Number of regions exceeds allocated memory for tracking buffers   
         ------------------------------------------------------------------*/
         /* Reallocate memory for buffers
         --------------------------------*/
         mxreg *= 2;
         *regcls = (long *)realloc(*regcls,sizeof(long)*(mxreg+1));
         *regcnt = (long *)realloc(*regcnt,sizeof(long)*(mxreg+1));
         *reglnk = (long *)realloc(*reglnk,sizeof(long)*(mxreg+1));
         if ((*regcls == NULL) || (*regcnt == NULL) || (*reglnk == NULL))
            {
            c_errmsg("Error allocating dynamic memory","reglabel-alloc",
                     &nonfatal);
            free (*regcls);
            free (*regcnt);
            free (*reglnk);
            free (runs);
            free (prvlin);
            return (E_FAIL);
            }

         /* Initialize newly allocated space for region tracking buffers
         ---------------------------------------------------------------*/
         for (k = reglbl; k <= mxreg; k++)
            (*regcls)[k] = (*regcnt)[k] = (*reglnk)[k] = 0;
      }
   
   /* Store class (pixel) value for region
   ---------------------------------------*/
   (*regcls)[regnum] = inlin[runs[i]]; 

   for (j = runs[i]; j < runs[i+1]; j++)
   /* For all pixels in run
   ------------------------*/
      {
      /* Put region number in output image buffer
      -------------------------------------------*/
      outlin[j] = regnum;
      /* Increment counter for number of pixels in region 
      ---------------------------------------------------*/
      ((*regcnt)[regnum])++;
      }
   }

/* Process the rest of the image
--------------------------------*/
for (ln = 2; ln <= nl; ln++)
/* For the rest of the lines in the image 
-----------------------------------------*/
   {
   /* Copy outlin to pvrlin
   ------------------------*/
   for (i = 0; i < ns; i++)
      prvlin[i] = outlin[i];

   /* Get next line 
   ----------------*/
   if (c_estep(&gdesc) != E_SUCC)
      return (E_FAIL);
   curent++;

   /* Set up pointers for cleaner programming 
   ------------------------------------------*/
   inlin = (long *)&iobuf[in_ptr[0]*4]; 
   outlin = (long *)&iobuf[out_ptr[0]*4];

   /* Parse line to find runs of contiguous pixel values
   -----------------------------------------------------*/
   parse_ln(inlin,ns,&nr,runs);

   for (i = 0; i < nr; i++) 
   /* For all contiguous pixel runs in line   
   ----------------------------------------*/
      {
      match = 0;

      /* Extract pixel value for run 
      ------------------------------*/
      runval = inlin[runs[i]]; 

      /* Establish segment of previous line to check for contiguity
      -------------------------------------------------------------*/
      strt = runs[i];
      end = runs[i+1];
      if (eightway)
         {
         if (strt != 0) strt--;
         if (end != ns) end++; 
	 }

      for (j = strt; j < end; j++)
      /* For all pixels in previous line segment to check 
      ---------------------------------------------------*/
	 {
	 if ((*regcls)[prvlin[j]] == runval)
	 /* Pixel value matches previous line's pixel value
	 --------------------------------------------------*/
	    if (!match)
	    /* First match in a run
	    -----------------------*/
	       {
	       /* Get region label from previous line
	       --------------------------------------*/
	       reglbl = prvlin[j];
	       match = 1;
	       }
	    else
	    /* Run of pixels already matched previous line
	    ----------------------------------------------*/
	       if (prvlin[j] < reglbl)
	       /* Separate regions with same pixel values are joined 
	       -----------------------------------------------------*/
	          /* Keep track of lowest numbered region linked 
	          ----------------------------------------------*/
	          reglbl = prvlin[j];
         }

      if (match)
      /* Match was found on previous line
      -----------------------------------*/
	 {
	 for (j = strt; j < end; j++)
    	    {
	    if ((*regcls)[prvlin[j]] == runval && prvlin[j] != reglbl)
	       {
	       if ((*reglnk)[prvlin[j]] == 0)
	          (*reglnk)[prvlin[j]] = reglbl;
	       else
		  {
	       	  tmplnk1 = prvlin[j];
		  for (;;)
		     {
		     if ((*reglnk)[tmplnk1] == 0)
			break;
		     else
			tmplnk1 = (*reglnk)[tmplnk1];
		     }

		  tmplnk2 = reglbl;
		  for (;;)
		     {
		     if ((*reglnk)[tmplnk2] == 0)
			break;
		     else
			tmplnk2 = (*reglnk)[tmplnk2];
		     }
	         
		  if (tmplnk1 < tmplnk2)
			(*reglnk)[tmplnk2] = tmplnk1;
		  else if (tmplnk2 < tmplnk1)
			(*reglnk)[tmplnk1] = tmplnk2;
	          }
               }	
   	    }
	 }
      else
      /* No pixels in run matched previous line
      -----------------------------------------*/
	 {
	 /* New region
	 -------------*/
	 reglbl = ++regnum;

	 if (reglbl > mxreg)
	 /* Number of regions exceeds allocated memory for tracking buffers   
         ------------------------------------------------------------------*/
	    /* Reallocate memory for buffers
            --------------------------------*/
	    {
            mxreg *= 2;
            *regcls = (long *)realloc(*regcls,sizeof(long)*(mxreg+1));
            *regcnt = (long *)realloc(*regcnt,sizeof(long)*(mxreg+1));
            *reglnk = (long *)realloc(*reglnk,sizeof(long)*(mxreg+1));
            if ((*regcls == NULL) || (*regcnt == NULL) || (*reglnk == NULL))
               {
               c_errmsg("Error allocating dynamic memory","reglabel-alloc",
			&nonfatal);
               free (*regcls);
               free (*regcnt);
               free (*reglnk);
               free (runs);
               free (prvlin);
               return (E_FAIL);
               }

            /* Initialize newly allocated space for region tracking buffers
            ---------------------------------------------------------------*/
            for (k = reglbl; k <= mxreg; k++)
               (*regcls)[k] = (*regcnt)[k] = (*reglnk)[k] = 0;
            }

	 /* Store class (pixel) value for region
	 ---------------------------------------*/
	 (*regcls)[reglbl] = inlin[runs[i]]; 
	 }
      
      for (j = runs[i]; j < runs[i+1]; j++)
      /* For all pixels in run
      ------------------------*/
	 {
	 /* Put region number in output image buffer
	 -------------------------------------------*/
	 outlin[j] = reglbl;
	 /* Increment counter for number of pixels in region 
	 ---------------------------------------------------*/
	 ((*regcnt)[reglbl])++;
	 }
      }
   }

/* Write out last line 
----------------------*/
if (c_estep(&gdesc) != E_SUCC)
   return (E_FAIL);

/* Compute net number of regions after disounting for linkage
-------------------------------------------------------------*/
*net_nreg = *gross_nreg = regnum;
for (i = 1; i <= regnum; i++)
/* For all regions
------------------*/
   if ((*reglnk)[i] != 0)
   /* Region will be linked to another
   -----------------------------------*/
      (*net_nreg)--;

/* Free memory
--------------*/
free (runs);
free (prvlin);

return (E_SUCC);
}
