/*******************************************************************************
NAME:		LINK	

PURPOSE:    	LINK 	Second step in labeling contiguous regions in image
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	9/90	Re-retrocode (All new C version!)
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "ltable.h"
#include "reglabel.h"

int link_region
(
  char *regionlt,             /* Name of region description lt file */
  char hosin[][CMLEN],        /* Name of input image file */
  unsigned char *iobuf,       /* I/O buffer for image data */
  long nl,                    /* Number of lines to process */
  long ns,                    /* Number of samples to process */
  long *regcls[],             /* Class identifier for each region */
  long *regcnt[],             /* Count of pixels in each region */
  long *reglnk[],             /* Subregion linkage to contiguous region */
  long *gross_nreg            /* Gross number of regions--before linking */
)
{

/* The following variables have been declared global for image i/o 
------------------------------------------------------------------*/
extern 	int	gdesc;		/* group descriptors */	
extern	long	in_ptr[1];	/* pointer into "iobuf" for input bands */
extern	long	out_ptr[1];	/* pointer into "iobuf" for output bands */
extern 	long	curcat;		/* current processing pass */
extern	long	curent;		/* current processing line */

long	i;			/* loop counter */
long	smp;			/* sample loop counter */
long	ln;			/* line loop counter */
long	count;			/* count regions as they are linked */
long	tmpreg;			/* temporary region label used in processing */
long	*regseq;		/* region sequence number */
long	nonfatal = 1;		/* nonfatal error flag */
long	access = IWRITE;	/* access flag for labeled table */

long	*inlin;			/* pointer to input line */
long	*outlin;		/* pointer to output line */

char 	msg[ERRLEN+1];		/* message buffer */
char	ch_buf[CMLEN+40];	/* character buffer */
struct TAB_DEF	ltab;		/* labeled table structure */

/*****************************************************************************/

/* Allocate storage for region sequence buffer and run pointers 
---------------------------------------------------------------*/
if ((regseq = (long *)malloc((*gross_nreg+1)*4)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","3.reglabel-alloc",&nonfatal);
   return (E_FAIL);
   }

/* Initialize region tracking buffers
-------------------------------------*/ 
for (i = 0; i <= *gross_nreg; i++)
   regseq[i] = 0;

/* Point all linked regions to their primary region identifier
--------------------------------------------------------------*/
count = 0;
for (i = 1; i <= *gross_nreg; i++)
   if ((*reglnk)[i] == 0)
      regseq[i] = ++count;
   else
      {
      for (tmpreg = (*reglnk)[i]; (*reglnk)[tmpreg] > 0; 
  	  		tmpreg = (*reglnk)[i] = (*reglnk)[tmpreg]);
      	 (*regcnt)[tmpreg] += (*regcnt)[i];
      }


/* Process the image and combine all linked regions
---------------------------------------------------*/
curcat = 2;
curent = 0;
for (ln = 0; ln < nl; ln++)
/* For all lines  
----------------*/
   {
   curent++;
   if (c_estep(&gdesc) != E_SUCC)
      return (E_FAIL);

   /* Set up pointers for cleaner programming 
   ------------------------------------------*/
   inlin = (long *)&iobuf[in_ptr[0]*4];
   outlin = (long *)&(iobuf[out_ptr[0]*4]); 


   for (smp = 0; smp < ns; smp++)
   /* For all samples in line
   --------------------------*/
      {
      if ((*reglnk)[inlin[smp]] > 0)
      /* Region is linked to another one
      ----------------------------------*/
	 /* Assign common region number to linked regions
	 ------------------------------------------------*/
	 outlin[smp] = regseq[(*reglnk)[inlin[smp]]]; 

      else
      /* Region is not linked
      -----------------------*/
	 /* Assign sequential region number to output image  
     	 --------------------------------------------------*/
	 outlin[smp] = regseq[inlin[smp]]; 
      }
   }

/* Write out last line 
----------------------*/
if (c_estep(&gdesc) != E_SUCC)
   return(E_FAIL);

/* Create labeled table containing region label information
-----------------------------------------------------------*/
ltab.nsubs = 0;
ltab.ftype = "REGLAB";
ltab.fdesc = "Region label description file";

open_tab(&ltab,regionlt,access);

put_vector(&ltab,"REGNUM",I4,"Region number",1,1,FIELD_SEP2);
sprintf(ch_buf,"Class value from image: %s",hosin[0]);
put_vector(&ltab,"CLASS",I4,ch_buf,1,1,FIELD_SEP2);
put_vector(&ltab,"COUNT",I4,"Count of pixels in region",1,1,RECORD_SEP);

for (i = 1; i <= *gross_nreg; i++)
   if (regseq[i] != 0)
      {
      put_matrix(ltab.vector,I4,&ltab.vector->lt_size,&regseq[i]);
      put_matrix(ltab.vector->next,I4,&ltab.vector->next->lt_size,
			&(*regcls)[i]);
      put_matrix(ltab.vector->next->next,I4,&ltab.vector->next->next->lt_size,
			&(*regcnt)[i]);
      put_record(&ltab);
      }

close_tab (&ltab);

/* Stop processing message
--------------------------*/
c_stoptm();

/* Write out informational message regarding the number of regions found
------------------------------------------------------------------------*/
sprintf(msg,"Number of regions found: %d",count);
c_errmsg(msg," ",&nonfatal);

/* Free memory
--------------*/
free(regseq);
free(*regcls); 
free(*regcnt); 
free(*reglnk);

return (E_SUCC);
}
