/*******************************************************************************
FUNCTION:	RELDRIVE

PURPOSE:	Release the specified tape drive.  Close the tape, do
		the specified rewind and write the number of specified
		EOF tape marks.

PROGRAM HISTORY:
  Version	Date       Author      Request
  -------	----	   ------      -------
    5.0		01/92	   D.Hollaren  LAS 5.0 conversion
    5.1         03/92      J.Fenno     Problem with VAXA.  c_tpclos is not
				       writing the specified number of eof marks
				       to tape.  Fix is to call c_tpweof first
				       and then call c_tpclos.
    5.2         06/94      T.Baltzer   Do error checking.  Return correct status.

*******************************************************************************/

#include <stdio.h>
#include "tae.h"
#include "worgen.h"
#include "tapeio.h"

main()
{
long	unit,			/* Tape drive unit number            	     */
	rewopt,			/* Rewind option                     	     */
	eofnum,			/* Number of EOF tape marks to write	     */
	access,			/* Access to open tape			     */
	status,			/* Function return status code       	     */
	std_out,		/* Standard output			     */
	zero = 0,		/* Value of zero			     */
	nonfatal = 1,		/* Non-fatal error code			     */
	fatal = -1,		/* Fatal error code			     */
	ropt = NOREW,		/* Rewind on reopen			     */
	mode = HANDLE;		/* Tape I/O writes error message     	     */

char	density[5],		/* Tape density in BPI			     */
	drive[CMLEN],		/* Tape drive string 			     */
	out[1][CMLEN];		/* Output files created			     */

char	*array[1];		/* Tape drive assigned			*/

struct PARBLK vblock;		/* TAE parameter block                	     */
struct VARIABLE	*p_find();	/* Function to find a TAE variable    	     */
struct VARIABLE	*val;		/* Pointer to a TAE variable 	      	     */


/* Initialize LAS
-----------------*/
c_inlas(&vblock,&std_out);

/* Retrieve the rewind option
-----------------------------*/
val = p_find(&vblock,"REWOPT");
if (strcmp(SVAL(*val,0),"REWIND") == 0)
    rewopt = REWIND;
else if (strcmp(SVAL(*val,0),"NOREW") == 0)
    rewopt = NOREW;
else
    rewopt = REWOFFL;

/* Retrieve the number of EOF tape marks to write
-------------------------------------------------*/
val = p_find(&vblock,"NTMARKS");
eofnum = IVAL(*val,0);

/* Set the access mode
----------------------*/
if (eofnum > 0)
    access = TWRITE;
else
    access = TREAD;

/* Set the density to default to be safe
----------------------------------------*/
strcpy (density,"0");

/* Retrieve physical tape drive name
------------------------------------*/
val = p_find(&vblock,"DRIVE");
if ((*val).v_count > 0)
    strcpy(drive,SVAL(*val,0));
else
    drive[0] = '\0';

if (strlen(drive) > 0)
    {
    /* Reopen the tape
    ------------------*/
    ropt = NOREW;
    status = c_tpreop(&unit, &zero, &mode, &ropt, density, &access, drive);
    if (status != TAPE_SUCC)
       {
       c_errmsg("Error reopening tape device", "reldrive-reop", &nonfatal);
       c_errmsg("Fatal error encountered","reldrive-fatal",&fatal);
       }

    /* Write the specified number of tape marks
    -------------------------------------------*/
    if (eofnum > 0)
       {
       status = c_tpweof(&unit, &eofnum, &mode);
       if (status != TAPE_SUCC)
	  {
	  c_errmsg("Error writing eof to tape", "reldrive-eof", &nonfatal);
          c_errmsg("Fatal error encountered","reldrive-fatal",&fatal);
	  }
       }

    /* Rewind and close the tape
    ----------------------------*/
    status = c_tpclos(&unit, &mode, &rewopt, &zero);
    if (status != TAPE_SUCC)
       {
       c_errmsg("Error dismounting the tape","reldrive-dismount",&nonfatal);
       c_errmsg("Fatal error encountered","reldrive-fatal",&fatal);
       }
    }

if (rewopt == REWOFFL)
    {
    /* Re-initialize the drive name and write it to the parameter block
    -------------------------------------------------------------------*/
    drive[0] = '\0';
    array[0] = drive;
    if (q_string(&vblock,"DRIVE",1,array,P_UPDATE) != SUCCESS)
       {
       c_errmsg("Error returning parameter to TAE","reldrive-rparm",&nonfatal);
       c_errmsg("Fatal error encountered","reldrive-fatal",&fatal);
       }


    /* Send the variable block to TAE
    ---------------------------------*/
    if (q_out(&vblock) != SUCCESS)
       {
       c_errmsg("Error returning parameter to TAE","reldrive-rparm",&nonfatal);
       c_errmsg("Fatal error encountered","reldrive-fatal",&fatal);
       }

    }

/* Display a completion message
-------------------------------*/
c_complt("RELDRIVE",out,&zero);
}
