/****************************************************************************

NAME:				GETPAR	

PURPOSE:  Retreive parameters from TAE 

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "constants.h"
#include "protos.h"

int getpar
(
    char in[],			/* The input Tie Point Location file name */
    char out[],			/* The output Tie Point Location file name */
    char host_name[],		/* Name for PRINT parameter 		*/
    long *msgtim,			/* Message timer 			*/
    long *nsamps,			/* Number of samples in scene		*/
    long *nlines,			/* Number of lines in scene		*/
    long *scenewidth,		/* Number of valid image samples	*/
    double *pixsize,		/* Pixel size				*/
    double *datum,			/* datum elevation above sea level	*/
    FILE **fptr,			/* Pointer to output file (report file) */
    char inlt[],			/* Input labeled table file		*/
    long *option			/* Subcommand option			*/
)

{
struct PARBLK vblk;		/* TAE parameter block 			*/
struct VARIABLE *var;		/* Pointer to a TAE variable in vblk 	*/

long term, file, ln_pr;		/* PRINT flags 				*/
long acc;			/* Access flag 				*/
long status;			/* Function return status 		*/
long fatal = -1;		/* Fatal flag for errmsg 		*/
long zero = 0;			/* Constant of zero 			*/
long two = 2;			/* Constant of two 			*/
long std_out;			/* standard output value		*/
char name[CMLEN];		/* Name for PRINT parameter 		*/
char filename[CMLEN];		/* Name for PRINT parameter 		*/

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the message timer and set up processing message
  ---------------------------------------------------*/
var = p_find(&vblk, "$msgtim");
*msgtim = IVAL(*var, 0);

c_inittm("RELIEF2","Processed","point number","","",&two,&two,msgtim,&two);

/* Get the subcommand used
  ----------------------*/
var = p_find(&vblk, "_SUBCMD");
if (strcmp(SVAL(*var, 0), "NONSTAND") == 0)
    *option = NONSTAND;
else
    *option = STANDARD;

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk, "INTL", &zero, in, "tpl", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(&vblk, "OUTTL", &zero, out, "tpl", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);

/* Get image size
---------------*/
var = p_find(&vblk, "PIXSIZ");
*pixsize = RVAL(*var, 0);
if (*option == STANDARD)
    {
    var = p_find(&vblk, "NS");
    *nsamps = IVAL(*var, 0);

    var = p_find(&vblk, "NL");
    *nlines = IVAL(*var, 0);

    var = p_find(&vblk, "NSVAL");
    *scenewidth = IVAL(*var, 0);
    }
else
    {
    acc = IREAD;
    if(c_getnam(&vblk, "INLT", &zero, inlt, "lt", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","relief2-fatal",&fatal);
    }

/* GET DATUM and units
--------------------*/
var = p_find(&vblk, "DATUM");
*datum = RVAL(*var,0);
var = p_find(&vblk, "DATUNIT");
if (strcmp(SVAL(*var,0),"FEET") == 0) *datum *= 0.3048;

/* Get the print option.  Make a file name (if neccessary) and open the
   temporary file
  --------------*/
strcpy(host_name, "relief2");
status = c_setprt(&vblk, &term, &file, &ln_pr, host_name, filename);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","relief2-fatal",&fatal);

acc = IUPDATE;
if (file == 1) 
   {
   c_mknam(filename, name, "prt", &acc);
   }
else strcpy(name, " ");
*fptr = fopen(host_name, "w");

/* Print operating parameters to report
  ------------------------------------*/
fprintf(*fptr,"\nInput point file:   %s\n", in);
fprintf(*fptr,"Output point file:  %s\n\n", out);
fprintf(*fptr,
  "All points are corrected to the reference ellipsiod (Datum = %3.1f)\n\n",
									*datum);

/* Write table header to printer file
  ----------------------------------*/
fprintf(*fptr, "                INPUT               OUTPUT                   REVISED INPUT\n");
fprintf(*fptr, "POINT       LINE    SAMPLE      LINE    SAMPLE    ELEV       LINE    SAMPLE\n");
fprintf(*fptr, "------   -----------------   -----------------  -------   -----------------\n");

return(E_SUCC);
}
