/*******************************************************************

NAME:                          TERAIN_ADJUST 

PURPOSE:  This routine corrects a P-product's sample location due to 
	  elevation, given the sample in question's current location, 
	  the nadir pixel and the elevation above the reference in meters.

*********************************************************************/
#include <math.h>
#include "las.h"
#include "constants.h"
#include "protos.h"

double terrain_adjust
(
    double oldsample,	/* Current sample value			*/
    long nadpix,		/* Center pixel				*/
    long elevation,		/* Elevation at current pixel		*/
    double *datum,		/* Reference elevation			*/
    double *pix		/* pixel size				*/
)
{
double scanangle,	/* angle between the satellite and the pixel	*/
       re,		/* radius of the Earth in meters		*/
       aye,		/* Radius of the Earth plus the satellite height*/
       salt,		/* Satellite height				*/
       dscan,		/* some angle -- see diagrams			*/
       dsample,		/* The change in pixel location			*/
       bee,		/* one side of the triangle used for calculation*/
       dist,		/* distance in meters from center to current pixel */
       eca,		/* the angle measured from the center of the earth*/
       los,		/* another side of the triangle			*/
       pixsize,		/* pixel size					*/
       zdd;		/* an angle used in the calculations		*/

re   = 6378135.0 + *datum;
aye  = 7083165.0; /* re + sat. alt. */
salt =  705000.0;
pixsize = *pix;

dist = (oldsample - nadpix) * pixsize;
eca  = dist / re;
los  = sqrt(aye * aye + re * (re - 2.0 * aye * cos(eca)));
scanangle = asin(re * sin(eca) / los);

/* Linear approximation 
  --------------------*/
if((elevation - *datum) < 1000)
   dscan = (aye * ( elevation - *datum) * scanangle) / (re * (salt - 
	    elevation - *datum)); 

/* With scanangle squared term approximation 
 -----------------------------------------*/
else
   {
   bee = re + elevation - *datum;
   dscan = (aye * ( elevation  - *datum) * scanangle) / (re * ((salt - 
	     elevation - *datum) + aye * scanangle * scanangle / 2.0 * 
	    ((bee * aye) / (re * re) - 1.0)));
   }

zdd = asin(sin(scanangle + dscan) / re * aye);
dsample = -re * (eca + scanangle + dscan - zdd) / pixsize;

return(oldsample - dsample);
}
