/****************************************************************************

NAME:				GETPAR	

PURPOSE:  Get user entered parameters 

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblk,		/* TAE parameter block */
    char *intpl,		/* The input Tie Point Location file name 	*/
    char *outtpl,		/* The output Tie Point Location file name 	*/
    double *elev,		/* the refernece elevation			*/
    char *grid_name,	/* Name of geometric mapping grid file 		*/
    struct GEOGRID *grid,	/* Geometric mapping grid data 			*/
    long *term, 		/* PRINT term flag				*/
    long *file,		/* PRINT file flag				*/
    long *ln_pr,		/* PRINT printer flag 				*/
    char *print_name,	/* Name of the file to be printed		*/
    char *name		/* Name of output print file 			*/
)


{
struct VARIABLE *var;		/* Pointer to a TAE variable in vblk */

long acc;			/* Access flag */
long msgtim;			/* Message timer */
long std_out;			/* Standard output LUN */
long status;			/* Function return status */
long fatal = -1;		/* Fatal flag for errmsg */
long zero = 0;			/* Constant of zero */
long two = 2;			/* Constant of two */
char filename[CMLEN];		/* Name for PRINT parameter 			*/

/* Initialize LAS processing
--------------------------*/
c_inlas(vblk, &std_out);

/* Get the message timer and set up processing message
----------------------------------------------------*/
var = p_find(vblk, "$msgtim");
msgtim = IVAL(*var, 0);

/* Get the input tie point location file name
-------------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INTL", &zero, intpl, "tpl", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","reliefx-fatal",&fatal);

/* Get the name of the geometric mapping grid file and read it
------------------------------------------------------------*/
if (c_getnam(vblk, "INGRID", &zero, grid_name, "grid", &acc) != E_SUCC)
   c_errmsg("Fatal error encountered","reliefx-fatal",&fatal);
if (c_getgrd(grid_name, grid) != E_SUCC)
   c_errmsg("Fatal error encountered","reliefx-fatal",&fatal);

/* Get the output tie point location file name
--------------------------------------------*/
acc = IWRITE;
if(c_getnam(vblk, "OUTTL", &zero, outtpl, "tpl", &acc) != E_SUCC)
	c_errmsg("Fatal error encountered","reliefx-fatal",&fatal);

/* Get the user entered elevation.  This must be entered in meters
----------------------------------------------------------------*/
var = p_find(vblk, "ELEV");
*elev = RVAL(*var, 0);

/* Get the print option.  Make a file name (if neccessary).
---------------------------------------------------------*/
strcpy(print_name, "reliefx");
status = c_setprt(vblk, term, file, ln_pr, print_name, filename);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","reliefx-fatal",&fatal);

acc = IUPDATE;
if (*file) 
   {
   c_mknam(filename, name, "prt", &acc);
   }

/* initialize the processing message
----------------------------------*/
c_inittm("RELIEFX","Processed","point number","","",&two,&two,&msgtim,&two);
}
