/****************************************************************************

NAME:                          TERRAIN_ADJUST 

PURPOSE:  This routine corrects a P-product's sample location due to 
          elevation, given the sample in question's current location, 
          the nadir pixel and the elevation above the reference in meters.

*****************************************************************************/
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

double terrain_adjust
(
    double oldsample,	/* original sample			*/
    long nadpix,		/* center pixel				*/
    double elevation,	/* elevation of point			*/
    double elev		/* relative elevation to adjust to	*/
)
{
double scanangle, re, aye, salt, dscan, dsample;
double bee, dist, eca, los, pixsize, zdd;

re = 6378135.0;		/* Radius of the earth */
salt = 920000.0;	/* Landsat 1,2,3 nominal altitude */
pixsize = 57.2;		/* Nominal pixel size in sample direction */

aye = re + salt;
dist = (oldsample - nadpix) * pixsize;
eca = dist / re;
los = sqrt(aye * aye + re * re - 2.0 * aye * re * cos(eca));
scanangle = asin(re * sin(eca) / los);

/* Linear approximation 
  --------------------*/
if((elevation - elev) < 1000.0) 
    dscan=(aye*(elevation-elev)*scanangle)/(re*(salt-(elevation-elev))); 

/* With scanangle squared term approximation 
  -----------------------------------------*/
else 
   {
   bee = re + (elevation-elev);
   dscan = (aye * (elevation-elev) * scanangle) / (re * ((salt - 
	   (elevation-elev)) + aye * scanangle * scanangle / 2.0 * 
	   ((bee * aye) / (re * re) - 1.0)));
   }
zdd = asin(sin(scanangle + dscan) / re * aye);
dsample = -re * (eca + scanangle + dscan - zdd) / pixsize;
return(oldsample - dsample);
}
