/****************************************************************************
NAME:                           in_frame

PURPOSE:  Computes coefficients mapping from input space projection
          coordinates to input space image coordinates

ALGORITHM DESCRIPTION:
        Find coefficients mapping from input image coordinates to input
          space projection coordinates based on image corner information
          previously read from a DDR
        Invert the coordinates to map the opposite direction
        Return
*****************************************************************************/
#include "remap.h"
#include "local_protos.h"

int in_frame
(
    double *icorners,           /* Corner coordinates from input image DDR */
    long *image,                /* Image coords for icorner or tval */
    double *icoef               /* Coefficients mapping in proj coords to l,s */
)
{
    double temp_cc[6];          /* Temporary coefficients */
    long prt_flag = TRUE;       /* Flag to print error messages in c_rtcoef */

    /* Calculate coefficients mapping from input image coordinates to input
       projection space
       ----------------*/
    if(c_rtcoef(icorners, image, temp_cc, &prt_flag) != E_SUCC) return(E_FAIL);

    /* Invert the coefficients to map from input projection space to input image
       coordinates
       -----------*/
    if(c_invert(temp_cc, icoef) != E_SUCC) return(E_FAIL);

    return(E_SUCC);
}
