#ifndef LOCAL_PROTOS_H
#define LOCAL_PROTOS_H

#include "tae.h"
#include "remap.h"  /* FRAME definition */


int getpar
(
    struct PARBLK *vblk,   /* TAE variable block */
    struct FRAME *frame,   /* Output framing params for GRID subcommand */
    struct PROJ *outp,     /* Output projection parameters */
    struct PROJ *inp,      /* Input projection parameters */
    double *ocoef,         /* Coefficients mapping output l,s to output proj */
    double *icoef,         /* Coeffs mapping input proj space to input l,s */
    char *name,            /* Print file name */
    long *msgtim           /* Message timer */
);

int get_proj_prm
(
    char *proj_name,            /* Projection parameter file name */
    char *inkey,                /* Input projection parameter key */
    char *outkey,               /* Output projection parameter key */
    long in_ddr_info,           /* Boolean flag--proj info from ddr used? */
    long out_ddr_info,          /* Boolean flag--proj info from ddr used? */
    struct PROJ *inp,           /* Input projection parameters */
    struct PROJ *outp           /* Output projection parameters */
);

int use_ddr_info
(
    struct PARBLK *vblk,        /* TAE parameter block */
    struct PROJ *pro,           /* Structure containing projection parameters */
    double *corners,            /* Corner projection coordinates */
    long *image,                /* Corner image coordinates */
    long  in_wind[][4],         /* Image window specification */
    char *flag,                 /* input/output flag            */
    double *pixy,               /* pixel size in y direction    */
    double *pixx                /* pixel size in x direction    */
);

int in_frame
(
    double *icorners,           /* Corner coordinates from input image DDR */
    long *image,                /* Image coords for icorner or tval */
    double *icoef               /* Coefficients mapping in proj coords to l,s */
);

int make_grid
(
    struct FRAME *frame,        /* Output framing parameters */
    struct PROJ *outp,          /* Output projection parameters */
    struct PROJ *inp,           /* Input projection parameters */
    double *ocoef,              /* Coefficients mapping ls to output proj */
    double *icoef,              /* Coefficients mapping input proj to ls */
    long msgtim                 /* Message timing parameter */
);

int out_frame
(
    struct FRAME *frame,        /* Framing parameters */
    struct PROJ *outp,          /* Output projection parameters */
    struct PROJ *inp,           /* Input projection parameters */
    long mode,                  /* Type of framing */
    double *ul,                 /* Upper left corner coordinates */
    double *lr,                 /* Lower right corner coordinates */
    long *ls,                   /* Line/sample coordinates of ul */
    long nl,                    /* Number of lines in output space */
    long ns,                    /* Number of samples in output space */
    long units,                 /* Units code of coordinates entered */
    double angle,               /* Rotatation angle applied to output */
    double pxmin,               /* Maximum & minimum X projection coordinates */
    double pxmax,
    double pymin,               /* Maximum & minimum Y projection coordinates */
    double pymax,
    double *ocoef               /* Coefficients mapping ls to out proj coords */
);

int twopoint
(
    double intie1[2],           /* First input location, line/sample */
    double intie2[2],           /* Second input location, line/sample */
    double outtie1[2],          /* First output location, line/sample */
    double outtie2[2],          /* Second output location, line/sample */
    double scalex,              /* Scale in X (pixel size) */
    double scaley,              /* Scale in Y (pixel size) */
    double icoef[6]             /* Transformation coefficients */
);

#endif
