#include "renumber.h"
#include "typlim.h"

#define ESAME -1

/******************************************************************************

NAME:	getpar

FUNCTION:
	Gets LAS parms ,checks windows, checks for too many band 
	numbers, checks bgval within acceptable range for byte data.
	Set up the lookup table using the background value and the
	'from' and 'to' values.

******************************************************************************/

FUNCTION getpar (lasin, hosin, nimages, bands, nbands, totbnd,
                 lasout, hosout, window, dtype, msgtim, minmax, update)

long	(*bands)[MAXBND + 1],	/* Band numbers for each image                */
	*dtype,			/* Data type of output image                  */
	*msgtim,		/* Processing line message time interval      */
	*nimages,		/* Number of input images                     */
	nbands[],		/* Number of bands for each image             */
	*totbnd,		/* Total number of bands                      */
	(*window)[4],		/* Input image window                         */
	*update;		/* Update flage				      */

char	lasin[][CMLEN],         /* Input image file LAS name                  */
        lasout[],               /* Output image file LAS name                 */
        hosin[][CMLEN],         /* Input image file host name                 */
        hosout[],               /* Output image file host name                */
	minmax[];		/* Minmax flag string                         */
{
char	msg[120],		/* Output message buffer                      */
        infil[CMLEN],           /* Input ascii FROM-TO file                   */
        tmpin[CMLEN];           /* Temporary string for holding IN            */

long	bckgrnd_cnt,		/* Number of background values entered (0, 1) */
	count,			/* Parameter count requested                  */
	fatal = -1,		/* fatal error message flag		      */
	from_cnt,		/* Number of values entered by user           */
	from_val[65536],	/* User supplied FROM values                  */
	i,			/* Loop counter                               */
	imglimit,		/* Maximum number of input images allowed     */
	last_type,		/* Previous image data type                   */
	partype,		/* Data type of parameter to get              */
	qflag[MAXIMG][2],	/* Flag indicating if subwindow specified     */
	same_type,		/* Flag indicating data type same for images  */
	same_wind,		/* Same window flag for getin                 */
	to_cnt, 		/* Number of values entered by user           */
	to_val[65536],		/* User supplied to values                    */
        access,                 /* type of access of file                     */
        odtype,                 /* Output data type (SAME, BYTE, or I*2)      */
        status;                 /* status of function calls                   */


short	bckgrnd,		/* Output background value                    */
	iend,			/* Ending index for lookup table              */
	istart;			/* Starting index for lookup table            */

struct DDR	inddr;		/* DDR structure                              */
struct VARIABLE *p_find(), *var;
long   zero = 0;
long   nonfatal = 1;
FILE   *fp;                     /* FROM-TO ascii file descriptor              */

/*
   Get input and output information
   --------------------------------
*/
imglimit = MAXIMG;

if (c_getin (&vblock, "IN", &zero, lasin, hosin, nimages, window, bands,
           nbands, totbnd, &imglimit, &same_wind, qflag) != E_SUCC)
   c_errmsg("Fatal error encountered", "renumber-fatal",&fatal);

if (!same_wind)
   c_errmsg("All input images must be same size", "renumber-siz",&fatal);

/* If OUT = IN or OUT = NULL, then overwrite the existing input file.
---------------------------------------------------------------------*/
var = p_find(&vblock,"IN");
strcpy(tmpin, SVAL(*var,0));
var = p_find(&vblock,"OUT");
if (var->v_count == 1 && strcmp(SVAL(*var,0), tmpin) != 0)
  {
  if (c_getout (&vblock, "OUT", lasout, hosout) != E_SUCC)
     c_errmsg("Fatal error encountered", "renumber-fatal",&fatal);
  *update = FALSE;
  }
else
  *update = TRUE;


/*
   Get other input parameters
   --------------------------
*/
/* Check if INFILE is specified, ignore FROM and TO parameters if it is 
-----------------------------------------------------------------------*/
access = IREAD;
status = c_getnam(&vblock,"INFILE",&zero,infil,"txt",&access);
if (status != E_SUCC)
   c_errmsg("Error opening INFILE", "renumber-open",&fatal);
if (strlen(infil) != 0)
   {
   printf("opening input file\n");
   fp = fopen(infil,"r");               /* Open the FROM-TO file          */
   if (fp == NULL)
      c_errmsg("Error opening INFILE", "renumber-open",&fatal);
   from_cnt=to_cnt=0;
   do
      {
      status=fscanf(fp,"%d %d",&from_val[from_cnt],&to_val[to_cnt]);
      if (status != EOF)
         {
         if (status !=2)
            {
	    c_errmsg("The FROM and TO values in INFILE must be in pairs",
                      "renumber-pairs",&fatal);
            } 
         if (from_cnt == 65536)
            {
            c_errmsg("INFILE contains too many values", "renumber-points",
                     &fatal);
            }
         from_cnt++;
         to_cnt++;
         }
      }
   while (status != EOF); 
   fclose(fp);
   }
else
   {
   printf("using from-to values\n");
   partype = ELONG;
   count   = 100;
   from_cnt  = c_getnum (&vblock, "FROM", &count, &partype, &from_val[0]);
   to_cnt    = c_getnum (&vblock, "TO",   &count, &partype, &to_val[0]);
   if(to_cnt != from_cnt)
      c_errmsg("Different number of FROM and TO inputs", "renumber-diff",
                &fatal);
   }
count = 1;
partype = EWORD;
bckgrnd_cnt = c_getnum (&vblock, "BACKGRND", &count, &partype, &bckgrnd);

/*
   Get the LAS globals
   -------------------
*/
var = p_find(&vblock, "$msgtim");
*msgtim = IVAL(*var,0);

var = p_find(&vblock, "$minmax");
strcpy(minmax, SVAL(*var,0));


/* Check input image data types.  Set output data type based on input type
   and ODTYPE.
--------------------------------------------------------------------------*/
var = p_find(&vblock, "ODTYPE");
if (strcmp(SVAL(*var,0), "SAME") == 0)
  odtype = ESAME;
else if (strcmp(SVAL(*var,0), "BYTE") == 0)
  odtype = EBYTE;
else
  odtype = EWORD;

same_type = TRUE;

for (i = 0; i < *nimages; i++)
   {
   if (c_getddr (hosin[i], &inddr) != E_SUCC)
      c_errmsg("Fatal error encountered", "renumber-fatal",&fatal);

   if (inddr.dtype != EBYTE && inddr.dtype != EWORD)
      c_errmsg("Input image data type must be byte or INTEGER*2", 
               "renumber-dtype",&fatal);
   if (*update && odtype != ESAME && inddr.dtype != odtype)
      c_errmsg("Cannot update image with different data type",
               "renumber-dtype", &fatal);
   if (inddr.dtype == EWORD && odtype == EBYTE)
      c_errmsg("Input image data I*2 cannot be output as BYTE",
               "renumber-dtype", &fatal);

   *dtype = inddr.dtype;

   if (i == 0)
      last_type = *dtype;
   if (last_type != *dtype)
      same_type = FALSE;
   last_type = *dtype;

   }

if (odtype == EWORD)
   *dtype = EWORD;
else if (!same_type)
   {
   c_errmsg("All input images not same data type. Output will be type INTEGER*2"
	    ,"information",&nonfatal);
   *dtype = EWORD;
   }

/*
  Allocate space for lookup table depending on dtype.
  Determine beginning and ending values of lookup table
  -----------------------------------------------------
*/
if (*dtype == EBYTE)
   {
   lookup_buf = (short *) malloc (256 * sizeof (short));
   lookup = lookup_buf;
   istart = MINUTINY;
   iend = MAXUTINY; 
   }
else
   {
   lookup_buf = (short *) malloc (65536 * sizeof (short));
   lookup = &lookup_buf[32768];
   istart = MINWORD;
   iend = MAXWORD;
   }

if (bckgrnd_cnt && (bckgrnd < istart || bckgrnd > iend))
   {
   sprintf (msg, "BACKGRND must be between %d and %d inclusive",
            istart, iend);
   c_errmsg(msg, "renumber-invbgval",&fatal);
   }

if (lookup == 0)
   c_errmsg("Error allocating memory", "renumber-alloc",&fatal);

/*
   Fill in lookup table starting with background values
   ----------------------------------------------------
*/
   if (bckgrnd_cnt != 0)
      for (i = istart; i <= iend; i++)
         {
         lookup[i] = bckgrnd;
         }
   else 
      for (i = istart; i <= iend; i++)
         {
         lookup[i] = i;
         }
/*
   Set up lookup table
   -------------------
*/
for (i = 0; i < from_cnt; i++)
   {
   if (from_val[i] < istart || from_val[i] > iend ||
       to_val[i] < istart   || to_val[i] > iend)
      {
      sprintf (msg, "FROM and TO values must be between %d and %d inclusive",
               istart, iend);
      c_errmsg(msg, "renumber-range",&fatal);
      }
   
   lookup[from_val[i]] = to_val[i];
   }

return;
}
