#include <stdio.h>
#include "las.h"

/*
   Image I/O globals
   -----------------
*/

short	*lookup,		/* Lookup table to accomodate indices of      */
				/*   -32768 to 32767                          */
	*lookup_buf,		/* actual storage for lookup table            */
	*thebuf;		/* Image I/O buffer                           */
char	*thebuf1;		/* Image I/O buffer                           */

long	fdesc[MAXIMG + 2],	/* File descriptors                           */
	gdesc,			/* Group descriptor                           */
	inoff[MAXBND],		/* Input image I/O buffer indices             */
	outoff[MAXBND],		/* Output image I/O buffer indices            */
        numfiles;               /* No. of files opened                        */

char    out[MAXIMG][CMLEN];     /* Names of files opened                      */
extern long	curent;		/* Processing line message counter            */
struct PARBLK	vblock;		/* LAS parameter block                        */
struct DDR	inddr;		/* DDR structure                              */

/******************************************************************************

NAME:	 RENUMBER

FUNCTION:
	Reassigns individual categories of a byte image 

PROGRAM HISTORY:
  Version	Date       Author	   Request
  -------	----	   ------          -------
    1.0      14-Aug-84	   B. Dealy        Original development
					   byte dtype
    2.0        -Jan-88	   N. Deodhar/SAR  New LAS and conversion to C
    3.0         Aug-88     B. Allam/SAR    Retrocode
    4           Dec-88     P. Pease        'STDOUT' changed to 'STD_OUT'
    5        17_Apr-89     K. Narayanan    Lookup table index was not correctly
                                           handled for non-Vax
    5.1	        JUL-93	   T. Mittan	   Added UPDATE option
    5.2         DEC-93     D. Bridges      Added INFILE to read FROM-TO values
 					   from a text file 
    7.0         MAR-98     LD Johnson      Increased maximum number of pairs
                                           from 1000 to 65,536.
                MAY-98     LD Johnson      Modified to overwrite existing input
                                           image if OUT=IN or OUT=--.
                                           Added ODTYPE to specify the desired
                                           output data type.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under LAS

PROJECT:	new LAS

ALGORITHM (FLOW):

	Do standard initialization
	Get LAS parameters and setup lookup table
	Set up image files for I/O
	Do processing
	Write out imge history, min/max and completion message

******************************************************************************/

main ()       /* renumber */
{
/*
   Renumbers pixel values in byte and short images.
   ------------------------------------------------
*/
double	minval[MAXBND],		/* Output image minimum values                */
        y_inc, x_inc,           /* line and sample increments                 */
	maxval[MAXBND];		/* Output image maximum values                */

static long bands[MAXIMG][MAXBND + 1];	/* Band numbers for input images      */
long	msgtim,			/* Processing line message time interval      */
	nbands[MAXIMG],		/* Number of bands for each image             */
	nimages,		/* Number of input images                     */
	dtype,			/* Data type of output image                  */
	mnmxflag ,	        /* min/max boolean flag                       */
	std_out,		/* Standard output for FORTRAN                */
	totbnd,			/* Total number of input bands                */
	window[MAXIMG][4],	/* Window specs for input images              */
	update;			/* Update flag				      */

char	minmax[4],		/* Minmax flag buffer                         */
	hosin[MAXIMG][CMLEN],	/* Input image file host name                 */
	hosout[CMLEN],		/* Output image file host name                */
	lasin[MAXIMG][CMLEN],	/* Input image file LAS name                  */
	lasout[CMLEN];		/* Output image file LAS name                 */

long    one = 1,        /* For passing by reference                   */
        zero = 0;

numfiles = 0;
mnmxflag = FALSE;
/*
   Initialize LAS
   --------------
*/
c_inlas (&vblock, &std_out);

/*
   Get LAS parameters
   ------------------
*/
getpar (lasin, hosin, &nimages, bands, nbands, &totbnd,
        lasout, hosout, window, &dtype, &msgtim, minmax,&update);
/*
   Initialize minimum and maximum values
*/
if  ((strcmp (minmax, "YES") == 0) || (strcmp (minmax, "yes") == 0))
   {
   mnmxflag = TRUE;
   c_inmnmx (&dtype, &totbnd, minval, maxval);
   }
/*
   Set up image I/O
   ----------------
*/
setup_files (hosin, hosout, nimages, totbnd, bands, nbands,
             dtype, window, &update);
/*
   Do processing
   -------------
*/
c_inittm ("RENUMBER", "Processed", "line", "band", "", &window[0][NL], &totbnd,
         &msgtim, &zero);
curent = 1;
c_startm ();

apply_lut (dtype, totbnd, window, mnmxflag, minval, maxval, &update);

c_stoptm ();

/*
   Close files, free space
   -----------------------
*/
if (c_egclse (&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered", "renumber-warn",&one);

if (update)
  free ((char *) thebuf1);
else
  free ((char *) thebuf);
free ((char *) lookup_buf);
/*
Update DDR
----------
*/
y_inc = 1.0;
x_inc = 1.0;
if (update == FALSE)
   {
   c_upddr(&vblock, hosin, hosout, &nimages, window, bands, nbands, &y_inc, 
	   &x_inc, &totbnd, &zero);
/*
   Handle image history, minmax and completion message
   ---------------------------------------------------
*/
   if (c_svmnmx (hosout, &totbnd, minval, maxval, minmax)
          != E_SUCC)
      c_errmsg("Nonfatal error encountered", "renumber-warn",&one);

   if (c_hstout (&vblock, hosout, hosin, &nimages, " ") != E_SUCC)
      c_errmsg("Nonfatal error encountered", "renumber-warn",&one);
   c_complt ("RENUMBER", lasout, &one);
   }
else
   c_complt ("RENUMBER", lasin, &zero);

}

