#include "renumber.h"

/******************************************************************************

NAME:	setup_files

FUNCTION:
	Set up image files for I/O.

******************************************************************************/

FUNCTION setup_files (hosin, hosout, nimages, totbnd, bands, nbands,
                      dtype, window, update)

char    hosin[][CMLEN],         /* Input image file host name                 */
        hosout[];               /* Output image file host name                */
long	(*bands)[MAXBND + 1],	/* Band numbers for each image                */
	dtype,			/* Data type of output image                  */
	nbands[],		/* Number of bands for each image             */
	nimages,		/* Number of input images                     */
	totbnd,			/* Total number of input bands                */
	(*window)[4],		/* Input image window                         */
	*update;		/* Update flag				      */
{

long	access,			/* Type of file access                        */
	band,			/* Counter on bands                           */
	bufsize,		/* Size of image I/O buffer                   */
	fatal = -1,		/* fatal error message			      */
	image,			/* Counter on images                          */
	intype,			/* Data type of input image                   */
	option[3],		/* Special conversion option on output image  */
	status;			/* Return status                              */

long    one = 1,        /* For passing by reference                   */
        zero = 0;

if (*update)
   {
   access = IUPDATE;
   intype = 0;
   }
else
   {
   access = IREAD;
   intype = EWORD;
   }

/* open input image
-----------------*/

  option[0] = option[1] = option[2] =0;
for (band = 0, image = 0; image < nimages; image++)
   {
   status = c_eopens (&fdesc[image], hosin[image], bands[image],
                    &window[0][SL], &window[0][SS], &window[0][NL], 
		    &window[0][NS], &inoff[band], &access, &intype, 
		    &zero, option);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered", "renumber-warn",&fatal);
   band += nbands[image];
   }

/* Open output image
----------------- */
if (*update == FALSE)
  {
  access = IWRITE;
  option[0] = option[1] = option[2] =0;
  if (dtype != intype)
     option[0] = dtype;

  status = c_eopens (&fdesc[nimages], hosout, &totbnd,
                 &one, &one, &window[0][NL], &window[0][NS],
                 &outoff[0], &access, &intype, &zero, option);
  if (status != E_SUCC)
     c_errmsg("Fatal error encountered", "renumber-fatal",&fatal);
  strcpy(out[numfiles], hosout);
  numfiles++;

  fdesc[nimages + 1] = 0;
  if ((thebuf = (short *) c_ealloc (fdesc, &bufsize)) == NULL)
     c_errmsg("Error allocating dynamic memory", "renumber-alloc",&fatal);

  if (c_egroup (fdesc, &gdesc, thebuf, &bufsize) != E_SUCC) 
     c_errmsg("Fatal error encountered", "renumber-fatal",&fatal);
  }
else
  {
  if ((thebuf1 = (char *) c_ealloc (fdesc, &bufsize)) == NULL)
     c_errmsg("Error allocating dynamic memory", "renumber-alloc",&fatal);

  if (c_egroup (fdesc, &gdesc, thebuf1, &bufsize) != E_SUCC) 
     c_errmsg("Fatal error encountered", "renumber-fatal",&fatal);
  }
}
