/*****************************************************************************
NAME:    get_proj_prm

PURPOSE: Retrieves the projection parameters from the projection parameter
         file.

DEVELOPMENT:  This file was adapted from ticmark/get_proj_prm.c with minimal
              changes.

*****************************************************************************/
#include <string.h>
#include "resample.h"
#include "ltable.h"

int get_proj_prm
(
    char *proj_name,         /* I: Projection parameter file name           */
    char *outkey,            /* I: Output projection parameter key          */
    long out_def,            /* I: Boolean flags--default given by user?    */
    struct PROJ *outp        /* O: Output projection parameters             */
)
{
    struct TAB_DEF tab;             /* Tabular structure of label table     */
    struct VECTOR *keyptr;          /* Pointer to key field                 */
    struct VECTOR *codeptr;         /* Pointer to projection code field     */
    struct VECTOR *zoneptr;         /* Pointer to zone code field           */
    struct VECTOR *unitptr;         /* Pointer to unit of measure field     */
    struct VECTOR *sphptr;         /* Pointer to datum (spheroid) code field*/
    struct VECTOR *parmptr;         /* Pointer to projection coeff. field   */
    long status;                    /* Function return status               */
    long ii;                        /* Loop counter                         */


    /* Open the labeled table file & retrieve the label.
       -------------------------------------------------*/
    open_tab(&tab, proj_name, IREAD);
    get_vector(&tab);

    /* Determine pointers to fields in the file.
       -----------------------------------------*/
    keyptr = get_field_ptr(&tab, "PROJKEY");
    if (keyptr == NULL)
        return(E_FAIL);
    codeptr = get_field_ptr(&tab, "PROJTYPE");
    if (codeptr == NULL)
        return(E_FAIL);
    zoneptr = get_field_ptr(&tab, "PROJZONE");
    if (zoneptr == NULL)
        return(E_FAIL);
    unitptr = get_field_ptr(&tab, "PROJUNITS");
    if (unitptr == NULL)
        return(E_FAIL);
    sphptr = get_field_ptr(&tab, "PROJSPH");
    if (sphptr == NULL)
        return(E_FAIL);
    parmptr = get_field_ptr(&tab, "PROJPARMS");
    if (parmptr == NULL)
        return(E_FAIL);

    /* Loop until key is found or EOF.
       -------------------------------*/
    for (;;)
    {
        status = get_record(&tab);
        if (status == E_FAIL)
            return(E_FAIL);
        if (status == E_EOF)
        {
            close_tab(&tab);
            c_lasmsg(FALSE, "resample-oprojkey", "%s key not found in %s.",
                    outkey, proj_name);
            return (E_FAIL);
        }
        if((strcmp(keyptr->ptr.c, outkey) == 0) || out_def)
            break;
    }

    outp->proj_code = *codeptr->ptr.i4;
    outp->zone_code = *zoneptr->ptr.i4;
    outp->units = *unitptr->ptr.i4;
    outp->datum_code = *sphptr->ptr.i4;
    for (ii = 0; ii < 15; ii++)
        outp->proj_coef[ii] = parmptr->ptr.r8[ii];

    /* Close the file & return.
       ------------------------*/
    close_tab(&tab);
    return(E_SUCC);
}
