/*******************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the RESIZE function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Baltzer	May  1992	initial development (modified from ZOOM)


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	       LAS		                

LANGUAGE		  C	

*******************************************************************************/

#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc1[MAXIMG+1];	
                                /* image file descriptors used for image I/O */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands  */

void getpar
(
    struct PARBLK *vblock,      /* pointer to TAE block	 	      */
    char taein[][CMLEN],	/* TAE file names of input images	      */
    char hosin[][CMLEN],  	/* host file names of input images	      */
    char taeout[][CMLEN],		/* TAE file name of output image	      */
    char hosout[][CMLEN],		/* host file name of output image	      */
    long *nmrimg, 		 /* number of input images		      */
    long (*bands)[MAXBND+1],         /* band specification 		              */
    long *nbands,		 /* number of bands per input image           */
    long (*window)[4],	         /* window specification		      */
    long *msgtim,		 /* elapsed time between display messages     */
    char *minmax,		 /* min max flag buffer		              */
    long *xfact,		 /* factor to enlarge or reduce the x-axis    */
    long *yfact,		 /* factor to enlarge or reduce the y-axis    */
    long *odtype,		 /* data type of input image		      */
    long *totbnd		 /* total number of bands		      */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block    */

long     access;		 /* type of access for file(write,read,update)*/
long     band;			 /* band index				      */
long     dtype;			 /* data type of previous input image         */
long     element = 0;		 /* parameter element number		      */
long     fatal = -1;		 /* fatal error flag			      */
long     i;			 /* counter-no. of input images	 	      */
long     imglim;		 /* max number of images to be processed      */
long     nl;			 /* temporary no. of lines in input image     */
long     nonfatal = 1;		 /* nonfatal error status		      */
long     ns;			 /* temporary no. of samples in input image   */
long     qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified   */
long     same;		    	 /* flag indicating if images have same window*/
long     sl;			 /* starting line of input image              */
long     ss;			 /* starting sample of input image            */
long     status;	   	 /* function return status code		      */
long     zero = 0;		 /* 0					      */
long     opt[2];		 /* options for opening images 		      */
 

/* Retrieve the TAE global variables from the parameter block and store them
   in their respective program variables. 
-----------------------------------------------------------------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

/* Call getin to parse the input image specification.
   Getin returns the TAE image name(s), host image name(s),
   bands to be processed, window specification, and number
   of images parsed.
----------------------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","resize-fatal",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","resize-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","resize-fatal",&fatal);
   }

/* Retrieve the TAE output image name from the parameter
   block and store it in "taeout". 
   Create a host file name for the output image and store 
   it in "hosout".
-----------------------------------------------------------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) == E_FAIL)
   c_errmsg("Fatal error encountered","resize-fatal",&fatal);

/* get XFACT and YFACT (SCALFACT)
----------------------------------*/
val = p_find (vblock,"scalfact");
*yfact = IVAL(*val,0);
*xfact = IVAL(*val,1);

/* check data type of input images, etc.
-----------------------------------------*/
access = IREAD;
opt[0] = opt[1] = 0;
nl = 0;
ns = 0;
sl = 1;
ss = 1;
	
for (band = 0,i = 0; i < *nmrimg; i++)
    {
    dtype = 0;
    status = c_eopens(&fdesc1[i],hosin[i],bands[i],&sl,&ss,&nl,&ns,
		      &offset1[band],&access,&dtype,&zero,opt);
    if (status != E_SUCC)
	 c_errmsg("Fatal error encountered","resize-fatal",&fatal);
    band += nbands[i];
    status = c_eclose(&fdesc1[i]);
    if (status != E_SUCC)
       c_errmsg("Nonfatal error encountered","resize-warn",&nonfatal);
    if (i == 0)
       *odtype = dtype;
    else if (dtype != *odtype)
       {
       c_errmsg("Input images are not the same data type",
  	        "resize-dtype",&nonfatal);
       c_errmsg("Fatal error encountered","resize-fatal",&fatal);
       }	
    }

return;
}
