/******************************************************************************
NAME:   		      PROCES 

PURPOSE     Do the actual image processing (reading and writing).

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T. Baltzer 	   4/92 	  CSC		Original Development

PROJECT			LAS

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"


/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern struct GDESC * gdesc1;	/* group descriptor used for image I/O       */
extern struct GDESC * gdesc2;	/* output group descriptor		     */
extern long offset1[MAXIMG*MAXBND];/* offsets into thebuf for input bands    */
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */

void proces
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosout[][CMLEN],		/* host name of output image		     */
    double minval[MAXBND],	/* min pixel value array		     */
    double maxval[MAXBND],	/* max pixel value array		     */
    long totbnd,			/* total number of bands to be processed     */
    long odtype,			/* data type of output image                 */
    long xfact,			/* reduction factor for the x-axis	     */
    long yfact,			/* reduction factor for the y-axis	     */
    char *minmax,		/* min max flag                              */
    unsigned char *thebuf1,	/* image input buffer                        */
    unsigned char *thebuf2,	/* image output buffer			     */
    long size,			/* number of bytes per pixel 	 	     */
    long nl,			/* number of image lines                     */
    long ns			/* number of image samples                   */
)

{

long	band;			/* loop index counting the bands             */
long    insamp;			/* input sample				     */
long    one = 1;		/* one					     */
long    outpix; 		/* output pixel				     */


/* process all lines of data
----------------------------*/
for (curent = 1; curent <= nl;   )
   {
   if (c_estep(&gdesc1) != E_SUCC)
      {
      c_egclse(&gdesc1);  /* ignore return status, quiting anyway */
      c_egclse(&gdesc2);  /* ignore return status, quiting anyway */
      c_clean(vblock,"resize",hosout,&one);
      }

   if (c_estep(&gdesc2) != E_SUCC)
      {
      c_egclse(&gdesc1);  /* ignore return status, quiting anyway */
      c_egclse(&gdesc2);  /* ignore return status, quiting anyway */
      c_clean(vblock,"resize",hosout,&one);
      }

   /* copy the proper pixels from the input buffer to the output buffer
   --------------------------------------------------------------------*/
   switch (odtype)
      {
      case EBYTE:
         for (band=0;band<totbnd;band++)
            {
            outpix = 0;
	    for (insamp = 0; insamp < ns; insamp++)
	       {
               *(thebuf2 + (*(offset2+band) + outpix)) =
	       *(thebuf1 + (*(offset1+band) + insamp));
	       outpix ++;
	       if (((insamp + 2) % xfact) == 0)
		  insamp++;
	       }
	    }
         break;
      case EWORD:
         for (band=0;band<totbnd;band++)
            {
	    outpix = 0;
	    for (insamp = 0; insamp < ns; insamp++)
	       {
       	       (*(short *)(thebuf2 + (*(offset2+band) + outpix) * size)) =
	       (*(short *)(thebuf1 + (*(offset1+band) + insamp) * size));
	       outpix ++;
	       if (((insamp + 2) % xfact) == 0)
		  insamp++;
	       }
	    }
         break;
      case ELONG:
         for (band=0;band<totbnd;band++)
            {
	    outpix = 0;
	    for (insamp = 0; insamp < ns; insamp++)
	       {
               (*(long *)(thebuf2 + (*(offset2+band) + outpix) * size)) =
	       (*(long *)(thebuf1 + (*(offset1+band) + insamp) * size));
	       outpix ++;
	       if (((insamp + 2) % xfact) == 0)
		  insamp++;
	       }
            }
	 break;
      case EREAL:
         for (band=0;band<totbnd;band++)
            {
	    outpix = 0;
	    for (insamp = 0; insamp < ns; insamp++)
	       {
               (*(float *)(thebuf2 + (*(offset2+band) + outpix) * size)) =
	       (*(float *)(thebuf1 + (*(offset1+band) + insamp) * size));
	       outpix ++;
	       if (((insamp + 2) % xfact) == 0)
		  insamp++;
	       }
	    }
	 break;
      }
      /* update min max values
      -------------------------*/
      if (strcmp(minmax,"YES") == 0)
         {
         for (band=0;band<totbnd;band++)
            c_pxmnmx(thebuf2 + (*(offset2+band) * size),&outpix,
                     &minval[band],&maxval[band],&odtype); 
         }

      /* skip input lines without processing if appropriate
      -----------------------------------------------------*/
      curent++;
      if (((curent % yfact) == 0) && (curent <= nl))
	 {
	 if (c_estep(&gdesc1) != E_SUCC)
	    {
	    c_egclse(&gdesc1);  /* ignore return status, quiting anyway */
	    c_egclse(&gdesc2);  /* ignore return status, quiting anyway */
	    c_clean(vblock,"resize",hosout,&one);
	    }
	 curent++;
	 }
   }

/* Final write
--------------*/
if (c_estep(&gdesc2) != E_SUCC)
   {
   c_egclse(&gdesc1);  /* ignore return status, quiting anyway */
   c_egclse(&gdesc2);  /* ignore return status, quiting anyway */
   c_clean(vblock,"resize",hosout,&one);
   }

return;
}
