/****************************************************************************** 

NAME:	RESIZE 

PURPOSE:	 			
To reduce an image in size by iteratively skipping user specified lines and/or
samples.

EXTERNAL ASSOCIATIONS:	
RESIZE is a module of the LAS system.

FILE REFERENCES:	none

PROGRAM HISTORY:
Version		Date	Author		Code/Conr	Request
-------		----	------		--------	-------
  1.0		4/92	T. Baltzer	   CSB     	Initial development (bulk
							of code modified from ZOOM).

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        RESIZE must be run under TAE.

PROJECT:	LAS		

LANGUAGE:	C

ALGORITHM REFERENCES:	none

*******************************************************************************/
#include <stdio.h> 	/* standard 'C' program include files		     */
#include <stdlib.h>
#include "las.h"        /* include imageio, histio, labserv                  */
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines    */

struct FDESC * fdesc1[MAXIMG+1]; /* image file descriptors used for image I/O */
struct FDESC * fdesc2[2];		 /* output image file descriptors	     */
struct GDESC * gdesc1;		     /* group descriptor used for image I/O       */
struct GDESC * gdesc2;		     /* output group descriptor for image I/O     */
long offset1[MAXIMG*MAXBND];    /* offsets into "thebuf" for input bands     */
long offset2[MAXBND];           /* offsets into "thebuf" for output bands    */

int main
(void)
{

struct  PARBLK    vblock;           /* TAE parameter block                   */

static long bands[MAXIMG][MAXBND+1];/* band specification		     */
long     msgtim;		    /* elapsed time between display messages */
long     nbands[MAXIMG];	    /* number of bands per input image       */
long     nmrimg;		    /* number of input images		     */
long	 nonfatal = 1;		    /* nonfatal error status		     */
long     one = 1; 	            /* value of one			     */
long     odtype;		    /* data type of output image	     */
long	 std_out;		    /* standard output			     */
long	 totbnd;		    /* total number of bands		     */
long     window[MAXIMG][4];         /* window specification		     */
long     xfact;			    /* factor to enlarge or reduce the x-axis*/
long     yfact;			    /* factor to enlarge or reduce the y-axis*/

char    hosin[MAXIMG][CMLEN];       /* buffer storage for "hosin"	     */
char    hosout[1][CMLEN];	            /* buffer storage for "hosout"	     */
char    minmax[4];		    /* min max flag			     */
char    taein[MAXIMG][CMLEN];       /* buffer storage for "taein"	     */
char    taeout[1][CMLEN];	            /* buffer storage for "taeout"	     */


/* initialize TAE 
-----------------*/
c_inlas (&vblock,&std_out);

/* get user specified parameters and global variable values from TAE and
checks window validity 
------------------------------------------------------------------------*/
getpar (&vblock,taein,hosin,taeout,hosout,&nmrimg,bands,nbands,window,
        &msgtim,minmax,&xfact,&yfact,&odtype,&totbnd);

/* set up the input, output images and do the actual processing
----------------------------------------------------------------*/
setup_etc (&vblock,hosin,hosout,nmrimg,window,bands,nbands,
           &totbnd,odtype,xfact,yfact,msgtim,minmax);

/* Update history file
----------------------*/
if (c_hstout(&vblock,hosout[0],hosin,&nmrimg," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered","resize-warn",&nonfatal);

/* Output completion message
----------------------------*/
c_complt("RESIZE",taeout,&one);

return EXIT_SUCCESS;

}

