/******************************************************************************
NAME:   		      SETUP_ETC 

PURPOSE     Prepare the input, output images for processing.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0	   T. Baltzer 	   4/92 	  CSC		Original Development


PROJECT			LAS

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/* The following variables have been declared global for image I/O routines
---------------------------------------------------------------------------*/
extern struct FDESC * fdesc1[MAXIMG+1];
                                /* image file descriptors used for image I/O */
extern struct FDESC * fdesc2[2];	/* output image file descriptor		     */
extern struct GDESC * gdesc1;	/* group descriptor used for image I/O       */
extern struct GDESC * gdesc2;	/* output group descriptor		     */
extern long offset1[MAXIMG*MAXBND];/* offsets into thebuf for input bands    */
extern long offset2[MAXBND];    /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */


void setup_etc
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*window)[4],		/* window specification of input images      */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long *totbnd,		/* total number of bands to be processed     */
    long odtype,			/* data type of output image                 */
    long xfact,			/* reduction factor for the x-axis	     */
    long yfact,			/* reduction factor for the y-axis	     */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax		/* min max flag                              */
)

{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long    bufsiz2;		/* size of image output buffer		     */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nol;			/* number of image output lines		     */
long	nonfatal = 1;		/* nonfatal error flag                       */
long	nos;			/* number of image output samples            */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long	size = 0;			/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */
long    opt[2];			/* options for opening images		     */

unsigned char *thebuf1;		/* image input buffer                        */
unsigned char *thebuf2;		/* image output buffer			     */

/* Open input image(s)
----------------------*/
access = IREAD;
opt[0] = opt[1] = 0;
nl = window[0][2];
ns = window[0][3];
for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc1[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","resize-fatal",&fatal);
   band += nbands[image];
   }
fdesc1[nmrimg] = 0;
/* Open output image
--------------------*/
access = IWRITE;	
nol = nl - (nl / yfact);
nos = ns - (ns / xfact);
status = c_eopens(&fdesc2[0],hosout[0],totbnd,&one,&one,&nol,&nos,
		  offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_clean(vblock,"resize",hosout,&one);
fdesc2[1] = 0;

/* Allocate I/O buffers
-----------------------*/
if ((thebuf1 = (unsigned char *)c_ealloc(fdesc1,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","resize-alloc",&nonfatal);
   c_eclose(&fdesc2[0]); /* don't check return stat, quiting anyway */
   c_clean(vblock,"resize",hosout,&one);
   }

if ((thebuf2 = (unsigned char *)c_ealloc(fdesc2,&bufsiz2)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","resize-alloc",&nonfatal);
   c_eclose(&fdesc2[0]); /* don't check return stat, quiting anyway */
   c_clean(vblock,"resize",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc1,&gdesc1,thebuf1,&bufsiz) != E_SUCC)
   {
   c_eclose(&fdesc2[0]); /* don't check return stat, quiting anyway */
   c_clean(vblock,"resize",hosout,&one);
   }

if (c_egroup(fdesc2,&gdesc2,thebuf2,&bufsiz2) != E_SUCC)
   {
   c_eclose(&fdesc2[0]); /* don't check return stat, quiting anyway */
   c_clean(vblock,"resize",hosout,&one);
   }

/* Setup for data type
----------------------*/
switch(odtype)
   {
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:	size = 4; break;
   case EREAL:  size = 4; break;
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("RESIZE","Processed","line","bands"," ",&nl,totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   if (c_inmnmx(&odtype,totbnd,minval,maxval) != E_SUCC)
      c_errmsg("Nonfatal error encountered","resize-warn",&nonfatal);

/* Do the main processing of the image
--------------------------------------*/
proces(vblock,hosout,minval,maxval,*totbnd,odtype,xfact,yfact,
       minmax,thebuf1,thebuf2,size,nl,ns);

/* Turn off display messages
----------------------------*/
if (curent > nl)
    curent = nl;
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double) (yfact / (yfact - 1.0));
samp_inc = (double) (xfact / (xfact - 1.0));
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,totbnd,&upddr_flag);
if (status != E_SUCC)
   {
   c_eclose(&fdesc2[0]); /* don't check return stat, quiting anyway */
   c_clean(vblock,"resize",hosout,&one);
   }

/* Update min/max values
-------------------------*/
if (c_svmnmx(hosout[0],totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered","resize-warn",&nonfatal);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc1) != E_SUCC)
   c_errmsg("Nonfatal error encountered","resize-warn",&nonfatal);

if (c_egclse(&gdesc2) != E_SUCC)
   c_errmsg("Nonfatal error encountered","resize-warn",&nonfatal);

/* Free buffer
--------------*/
free(thebuf1);
free(thebuf2);

return;
}
