/******************************************************************************
NAME:   		      PROCES 

PURPOSE    To generate the cyan, magenta, yellow, and/or black bands. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   R. McKinney     12/88	  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  6.1	  D. Eichacker    05/94                         Added opt[].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PROCES must be run under TAE.

PROJECT			LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   Calculate the values for each desired band
   Save the minmax values if $MINMAX was set to "YES"
   Update the output ddr
   Close the input and output image files

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern struct FDESC * fdesc[MAXIMG+1];	
                                /* image file descriptors used for image I/O */
extern struct GDESC * gdesc;	/* group descriptor used for image I/O       */
extern long offset1[MAXIMG*MAXBND+1];/* offsets into "thebuf" for input bands*/
extern long offset2[MAXBND+1];  /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */


void proces
(
    struct PARBLK *vblock,	/*  TAE parameter block			     */
    char hosin[][CMLEN],		/* host names of input images		     */
    char hosout[][CMLEN],		/* host name of output image		     */
    long nmrimg,			/* number of input images                    */
    long (*bands)[MAXBND+1],	/* bands to be processed for all images      */
    long *nbands,		/* number of bands per input image           */
    long (*window)[4],		/* window specification of input images      */
    long msgtim,			/* elapsed time between display messages     */
    char *minmax,		/* min max flag                              */
    long totbnd,			/* total number of bands to be processed     */
    long odtype,			/* output data type			     */
    long cyan,			/* cyan band flag			     */
    long magenta,		/* magenta band flag			     */
    long yellow,			/* yellow band flag			     */
    long black			/* black band flag			     */
)

{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long    one = 1;		/* one					     */
long    sample;			/* sample indicator			     */
long	size = 0;		/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    totbnd2;		/* total number of bands to be generated     */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */

static 	long	opt[3]={0,0,0}; /* for c_eopens                              */

float   minn;
float   maxx;
float   red;
float   green;
float   blue;
float   cyn;
float   mag;
float   yel;
float   blk;

unsigned char    *thebuf;	/* image input buffer                        */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      {
      c_errmsg("Error opening input image","rgb2cmyk-open",&nonfatal);
      c_errmsg("Fatal error encountered","rgb2cmyk-fatal",&fatal);
      }
   band += nbands[image];

   }

/* Open output image
--------------------*/
access = IWRITE;	
band = cyan + magenta + yellow + black;  
totbnd2 = band;
status = c_eopens(&fdesc[nmrimg],hosout[0],&band,&one,&one,&nl,&ns,
         offset2,&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      {
      c_errmsg("Error opening output image","rgb2cmyk-open",&nonfatal);
      c_errmsg("Fatal error encountered","rgb2cmyk-fatal",&fatal);
      }
fdesc[nmrimg+1] = 0;

/* Determine size of output image
--------------------------------*/
switch (odtype)
  {
  case EBYTE: size = 1; break;
  case EWORD: size = 2; break;
  case ELONG: size = 4; break;
  case EREAL: size = 4; break;
  }

/* Allocate I/O buffer
----------------------*/
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","rgb2cmyk-alloc",&nonfatal);
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
       c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);
   c_clean(vblock,"rgb2cmyk",hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup(fdesc,&gdesc,thebuf,&bufsiz) != E_SUCC)
   {
   if (c_eclose(&fdesc[nmrimg]) != E_SUCC)
       c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);
   c_clean(vblock,"rgb2cmyk",hosout,&one);
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("RGB2CMYK","Processed","line","bands"," ",
	&nl,&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Main processing loop
-----------------------*/
/*
   processing for PROCES 
*/
maxx = 255.;

switch (odtype)
{
case 1:
for (curlin = 0;curlin < nl; curlin++)
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);
      c_clean(vblock,"rgb2cmyk",hosout,&one);
      }
	for (sample=0;sample<ns;sample++)
	{
        minn = 999.;
	red =	(*(thebuf + (*(offset1)+sample) * size));
	green =	(*(thebuf + (*(offset1+1)+sample) * size));
	blue =	(*(thebuf + (*(offset1+2)+sample) * size));
	cyn =  maxx - red;
	mag =  maxx - green;
  	yel =  maxx - blue;
	if (cyn < minn)
		 minn = cyn;
	if (mag < minn)
		 minn = mag;
	if (yel < minn)
		 minn = yel;
 	blk = minn;	
	if (cyn > 255.0) cyn = 255.0;
	else if (cyn < 0.0) cyn = 0.0;
	if (mag > 255.0) mag = 255.0;
	else if (mag < 0.0) mag = 0.0;
	if (yel > 255.0) yel = 255.0;
	else if (yel < 0.0) yel = 0.0;
	if (blk > 255.0) blk = 255.0;
	else if (blk < 0.0) blk = 0.0;
	if (cyan == 1)
	   (*(thebuf + (*(offset2)+sample) * size)) = cyn;
	if (magenta == 1)
	   (*(thebuf + (*(offset2+cyan)+sample) * size)) = mag;
	if (yellow == 1)
	   (*(thebuf + (*(offset2+cyan+magenta)+sample) * size)) = yel;
	if (black == 1)
	   (*(thebuf + (*(offset2+cyan+magenta+yellow)+sample) * size)) = blk;
	}
    curent ++;
    }
break;

case 2:
for (curlin = 0;curlin < nl; curlin++)
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);
      c_clean(vblock,"rgb2cmyk",hosout,&one);
      }
	for (sample=0;sample<ns;sample++)
	{
        minn = 999.;
	red =	(*(short *)(thebuf + (*(offset1)+sample) * size));
	green =	(*(short *)(thebuf + (*(offset1+1)+sample) * size));
	blue =	(*(short *)(thebuf + (*(offset1+2)+sample) * size));
	cyn =  maxx - red;
	mag =  maxx - green;
  	yel =  maxx - blue;
	if (cyn < minn)
		 minn = cyn;
	if (mag < minn)
		 minn = mag;
	if (yel < minn)
		 minn = yel;
 	blk = minn;	
	if (cyn > 32767.0) cyn = 32767.0;
	else if (cyn < -32768.0) cyn = -32768.0;
	if (mag > 32767.0) mag = 32767.0;
	else if (mag < -32768.0) mag = -32768.0;
	if (yel > 32767.0) yel = 32767.0;
	else if (yel < -32768.0) yel = -32768.0;
	if (blk > 32767.0) blk = 32767.0;
	else if (blk < -32768.0) blk = -32768.0;
	if (cyan == 1)
	   (*(short *)(thebuf + (*(offset2)+sample) * size)) = cyn;
	if (magenta == 1)
	   (*(short *)(thebuf + (*(offset2+cyan)+sample) * size)) = mag;
	if (yellow == 1)
	   (*(short *)(thebuf + (*(offset2+cyan+magenta)+sample) * size)) = yel;
	if (black == 1)
	(*(short *)(thebuf + (*(offset2+cyan+magenta+yellow)+sample) * size)) =
		 blk;
	}
    curent ++;
    }
break;

case 3:
for (curlin = 0;curlin < nl; curlin++)
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);
      c_clean(vblock,"rgb2cmyk",hosout,&one);
      }
	for (sample=0;sample<ns;sample++)
	{
        minn = 999.;
	red =	(float) (*(long *)(thebuf + (*(offset1)+sample) * size));
	green =	(float) (*(long *)(thebuf + (*(offset1+1)+sample) * size));
	blue =	(float) (*(long *)(thebuf + (*(offset1+2)+sample) * size));
	cyn =  maxx - red;
	mag =  maxx - green;
  	yel =  maxx - blue;
	if (cyn < minn)
		 minn = cyn;
	if (mag < minn)
		 minn = mag;
	if (yel < minn)
		 minn = yel;
 	blk = minn;	
	if (cyn > 2147483647.0) cyn = 2147483647.0;
	else if (cyn < -2147483648.0) cyn = -2147483648.0;
	if (mag > 2147483647.0) mag = 2147483647.0;
	else if (mag < -2147483648.0) mag = -2147483648.0;
	if (yel > 2147483647.0) yel = 2147483647.0;
	else if (yel < -2147483648.0) yel = -2147483648.0;
	if (blk > 2147483647.0) blk = 2147483647.0;
	else if (blk < -2147483648.0) blk = -2147483648.0;
	if (cyan == 1)
	   (*(long *)(thebuf + (*(offset2)+sample) * size)) = (long) cyn;
	if (magenta == 1)
	   (*(long *)(thebuf + (*(offset2+cyan)+sample) * size)) = (long) mag;
	if (yellow == 1)
	   (*(long *)(thebuf + (*(offset2+cyan+magenta)+sample) * size)) =(long)yel;
	if (black == 1)
	(*(long *)(thebuf + (*(offset2+cyan+magenta+yellow)+sample) * size)) =
		 (long) blk;
	}
    curent ++;
    }
break;

case 4:
for (curlin = 0;curlin < nl; curlin++)
   {
   if (c_estep(&gdesc) != E_SUCC)
      {
      if (c_egclse(&gdesc) != E_SUCC)
          c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);
      c_clean(vblock,"rgb2cmyk",hosout,&one);
      }
	for (sample=0;sample<ns;sample++)
	{
        minn = 999.;
	red =	(*(float *)(thebuf + (*(offset1)+sample) * size));
	green =	(*(float *)(thebuf + (*(offset1+1)+sample) * size));
	blue =	(*(float *)(thebuf + (*(offset1+2)+sample) * size));
	cyn =  maxx - red;
	mag =  maxx - green;
  	yel =  maxx - blue;
	if (cyn < minn)
		 minn = cyn;
	if (mag < minn)
		 minn = mag;
	if (yel < minn)
		 minn = yel;
 	blk = minn;	
	if (cyn > 2147483647.0) cyn = 2147483647.0;
	else if (cyn < -2147483648.0) cyn = -2147483648.0;
	if (mag > 2147483647.0) mag = 2147483647.0;
	else if (mag < -2147483648.0) mag = -2147483648.0;
	if (yel > 2147483647.0) yel = 2147483647.0;
	else if (yel < -2147483648.0) yel = -2147483648.0;
	if (blk > 2147483647.0) blk = 2147483647.0;
	else if (blk < -2147483648.0) blk = -2147483648.0;
	if (cyan == 1)
	   (*(float *)(thebuf + (*(offset2)+sample) * size)) = cyn;
	if (magenta == 1)
	   (*(float *)(thebuf + (*(offset2+cyan)+sample) * size)) = mag;
	if (yellow == 1)
	   (*(float *)(thebuf + (*(offset2+cyan+magenta)+sample) * size)) = yel;
	if (black == 1)
	(*(float *)(thebuf + (*(offset2+cyan+magenta+yellow)+sample) * size)) =
		 blk;
	}
    curent ++;
    }
break;
}

   for (band = 0; band < totbnd2; band++)
        {
        if (strcmp(minmax,"YES") == 0)
        status = c_pxmnmx(thebuf + (*(offset2+band) * size),&ns,
                          &minval[band],&maxval[band],&odtype); 
	}
/* Final write
--------------*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);
      c_clean(vblock,"rgb2cmyk",hosout,&one);
   }
/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = COMB;
status = c_upddr(vblock,hosin,hosout[0],&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&totbnd2,&upddr_flag);
if (status != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
       c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);
       c_clean(vblock,"rgb2cmyk",hosout,&one);
   }

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout[0],&totbnd2,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Error updating the minmax information","rgb2cmyk-minmax",
		&nonfatal);

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Error closing images","rgb2cmyk-close",&nonfatal);

/* Free buffer
--------------*/
free(thebuf);
return;
}
