/*****************************************************************************
NAME				 EXPLICIT 

PURPOSE    Calculate first order mapping coefficients from user entered
    	   rotation angle, scale factors, and pivot point location.

PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  1.0	   R. White        4/82        CSC      Original Development
  4.0      J. Bay	   2/84	       SAR     
  5.0      D. Steinwand    7/88        EDC      LAS 5.0 development,
						C conversion
  5.01     D. Steinwand    8/89        EDC      Fixed bug with piv pt

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   ROTRNSCL must run under TAE

PROJECT			LAS

ALGORITHM DESCRIPTION

	Convert the rotation angle to radians
	Calculate mapping coefficients
	Return
*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "local_protos.h"

#define PI	3.141592653589793238

void explicit
(
    double angle,			/* Counter-clockwise rotation angle */
    double scale[4],		/* Pre- & Post- scale factors */
    double pivot[4],		/* Pivot point */
    double coef[6]			/* Mapping coefficients */
)
{
    /*--------------------------------------------------------------------------
      Convert rotation angle to radians 
     -------------------------------------------------------------------------*/
    angle *= PI/180.0;

    /*--------------------------------------------------------------------------
      Calculate mapping coeffients 
      ------------------------------------------------------------------------*/
    coef[0] = cos(angle) / (scale[0] * scale[2]);
    coef[1] = sin(angle) / (scale[1] * scale[2]);
    coef[3] = -(sin(angle)) / (scale[0] * scale[3]);
    coef[4] = cos(angle) / (scale[1] * scale[3]);
    coef[2] = pivot[2] - ((pivot[1] * coef[1]) + (pivot[0] * coef[0]));
    coef[5] = pivot[3] - ((pivot[1] * coef[4]) + (pivot[0] * coef[3]));

}
