/****************************************************************************
NAME:				get_ipsf

PURPOSE:   Reads the inverse point spread function from disk

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  5.0	 2/89	D. Steinwand  CSB      LAS 5.0 development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Open the IPSF file for read
	Initialize the ISPF buffer to zero
	Read the IPSF from disk
	Close the file
	Return
*****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"

void get_ipsf
(
    char *infile,		/* Filename */
    double *ipsf		/* Ptr to buffer for inverse point spread function */
)
{
FILE	*fp;		/* File pointer for table file */
long stflag;		/* Status flag */
long i;			/* Loop counter */
long dummy;		/* Non-used value read from file */
long pass = 1;		/* Non-fatal error message; pass */
long fatal = -1;	/* Fatal error message; abort */

/* Open ipsf file for reading
  --------------------------*/ 
fp = fopen(infile,"r");		
if (fp == NULL)
	{
	c_errmsg("Error opening inverse point spread function file",
                 "rtable-open",&pass);
	c_errmsg("Fatal error encountered","rtable-fatal",&fatal);
	}

/* Initialize the ipsf buffer
  --------------------------*/
for (i = 0; i < 1025; ipsf[i++] = 0.0);

/* Read the ipsf file
  ------------------*/
for (i = 0; i < 1025; i++)
   {
   stflag = fscanf(fp, "%d%lf", &dummy, &ipsf[i]);
   if (stflag != 2)
     {
     if (stflag == EOF) break;
     else
	{
	c_errmsg("Error reading inverse point spread function file",
		 "rtable-read",&pass);
	c_errmsg("Fatal error encountered","rtable-fatal",&fatal);
	}
     }
   }

fclose(fp);
return;
}
