/****************************************************************************
NAME:				maketab

PURPOSE:   Create a dimension of the resampling weight table file from an
           inverse point spread function

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT   REASON
-------	 ----	------	   ---------   -----------------------------
  1.0   12/85   D. Meyer      AB       Ditty in support of restoration contract
  5.0	 2/89	D. Steinwand  CSB      LAS 5.0 development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Find the center of the inverse point spread function
	Report the center location and value
	Create the weight table
	Scale weight table so sum of values add to one
	Write to resampling weigth table structure
	Return
*****************************************************************************/
#include "las.h"
#include "geompak.h"
#include "protos.h"

void maketab
(
    double *ipsf,			/* Ptr to buffer containing IPSF */
    long kern_size,			/* Resampling weight table kernel size */
    double table[33][16]		/* Resampling weight table buffer */
)
{
double sum;		/* Sum of weights in each table row */
double rmax;		/* Value of center of inverse point spread function */
long i,j;		/* Loop counters */
long midpt = 0;		/* Center of inverse point spread function */
char mess[80];		/* Buffer for messages */
long pass = 1;		/* Non-fatal error flag--do not abort */
long ifirst;		/* Index to first IPSF loc for given table row */
long iptr;		/* Index to get IPSF val for current table element */
double weight[16];	/* Non scaled resampling weight table column */
double scale;		/* Scale factor--A row of weights must add to one */

/* Find point of maximum so kernel can be centered
  -----------------------------------------------*/
for (sum = 0.0, rmax = -99999.99, i = 0; i < 1025; i++)
   if (ipsf[i] > rmax)
	{
	midpt = i;
	rmax = ipsf[i];
	}

sprintf(mess, "A maximum of %f was found at location %d", rmax, (midpt + 1));
c_errmsg(mess, " ", &pass);

/* Create weight table and write to the table structure
  ----------------------------------------------------*/
ifirst = midpt - ((kern_size / 2) * 32) + 32;
for (i = 0; i < 33; i++, ifirst--)
   {
   for (iptr=ifirst, sum=0, j=0; j < kern_size; iptr += 32, sum += weight[j++])
	weight[j] = ipsf[iptr];
   scale = 1.0 / sum;
   for (j = 0; j < kern_size; j++)
      table[i][j] = weight[j] * scale;
   }
return;
}
