#ifndef PROTOS_H
#define PROTOT_H

#include "las.h"

void calc_normal
(
    double **save_x,
    double **save_y,
    double **save_z,    /* Buffers containing the cartesian coordinates of the
                           current line, previous line and next line. (I)     */
    long current,	/* Index to current line. (I)			      */
    long next,		/* Index to next line. (I)			      */
    long prev,		/* Index to previous line. (I)			      */
    long samp,		/* Current sample. (I)				      */
    double normal[]	/* Average normal of the point of interest (x,y,z) 
			   based on 4 adjacent points. (O)       	      */
);

void findang
(
    double normal[],	/* Normal of the point of interest (x,y,z). (I)       */
    double *point_x,    /* Point of interest (x,y,z). (I)		      */
    double *point_y,
    double *point_z,
    double *rpoint_lat,	/* Point of interest (radians). (I)		      */
    double *rpoint_lon,
    double position[],	/* Aircraft position (x,y,z). (I)		      */
    double *line,	/* Magnitude of line of sight vector (range vector) 
			   in meters. (O) 				      */
    double *incid,	/* Incidence angle (in degrees). (O)		      */
    double *look,	/* Look angle (in degrees). (O)			      */
    double *azm,	/* Azimuth angle (in degrees). (O)		      */
    long option[]       /* Option indicating the information to be output. (I)
                                1 = Calculate incidence angle
                                2 = Calculate radar look angle
                                3 = Calculate line of sight (range) vector 
                                4 = Calculate azimuth angle		      */
);

void findpt
(
    double rplat,	/* Latitude of point of interest (radians). (I)	      */
    double rplon,	/* Longitude of point of interest (radians). (I)      */
    double position[]	/* Aircraft position. (O)	      		      */
);

int get_info
(
    float *ibuf,	/* Input image I/O buffer. (I)	                      */
    float *obuf,	/* Output image I/O buffer. (O)	                      */
    long line,		/* Current line number. (I)			      */
    double position[],	/* Air craft position (x,y,z). (I)		      */
    long window[],	/* Input window. (I)				      */
    struct DDR *ddr,	/* Input image's DDR. (I)			      */
    double coef[],	/* Coefficients mapping image coordinates to projection
			   coordinates. (I)				      */
    long odtype,	/* Output data type. (I)			      */
    long option[],	/* Option indicating the information to be output. (I)
				1 = Calculate incidence angle
				2 = Calculate radar look angle
				3 = Calculate range vector
				4 = Calculate azimuth angle 		      */
    long offset,	/* Value to subtract from the range vector 
			   lengths. (I)					      */
    char *subcmd,	/* Subcommand chosen. (I)			      */
    double **latlon_lat,
    double **latlon_lon,/* Buffer to save the lat/lon (in radians) of previous
		 	   lines  (I/O)					      */
    double **save_x,
    double **save_y,
    double **save_z     /* Buffer to save the cartesian coordinates of previous
			   lines  (I/O)	      				      */
);

void point
(
    struct PARBLK *vblock,/*  TAE parameter block. (I)			      */
    char *hosin,	/* Host name of input image. (I)		      */
    char *hosout,	/* Host name of output image. (I)		      */
    long window[],	/* Window specification of input image. (I)           */
    long band[],	/* Band to be processed. (I)			      */
    long outband,	/* Number of output bands to create. (I)	      */
    long odtype,	/* Data type of output image. (I)                     */
    long msgtim,	/* Elapsed time between processing messages. (I)      */
    char *minmax,	/* Min/max flag. (I)                                  */
    long option[],	/* Option indicating the information to be output. (I)
				1 = Calculate incidence angle
				2 = Calculate radar look angle
				3 = Calculate range vector
				4 = Calculate azimuth angle		      */
    long offset,		/* Value to subtract from the range vector 
			   lengths. (I)					      */
    char *subcmd	/* Subcommand chosen. (I)			      */
);

void getpar
(
    struct PARBLK *vblock,/* Pointer to TAE parameter block. (I/O)	      */
    char taein[CMLEN],	/* TAE file name of input image. (O)           	      */
    char hosin[CMLEN],  /* Host file name of input image. (O)                 */
    char *taeout,       /* TAE file name of output image. (O)		      */
    char *hosout,       /* Host file name of output image. (O)                */
    long window[],      /* Window specifications for input image. (O)         */
    long band[],        /* Band specification for input image. (O)            */
    long *odtype,       /* Output data type. (O)                              */
    double *blat,       /* Beginning latitude of flight path (-LINE) or
                           latitude of specified point (-POINT). (O)          */
    double *blon,       /* Beginning longitude of flight path (-LINE) or
                           longitude of specified point (-POINT). (O)         */
    double *elat,       /* Ending latitude of flight path. (O)                */
    double *elon,       /* Ending longitude of flight path. (O)               */
    double *elev,       /* Elevation (in meters) of flight path (-LINE) or
                           specified point (-POINT). (O)                      */
    long option[],	/* Option indicating the information to be output. (O)
				1 = Calculate incidence angle
				2 = Calculate radar look angle
				3 = Calculate range vector
				4 = Calculate azimuth angle		      */
    long *offset,	/* Value to subtract from the vector lengths. (O)     */
    long *msgtim,       /* Elapsed time between processing messages. (O)      */
    char *minmax,       /* Min/max flag. (O)				      */
    char *subcmd	/* Subcommand chosen. (O)			      */
);

void sph2cart
(
    double lat,		/* Latitude value in radians. (I)		      */
    double lon,		/* Longitude value in radians. (I)		      */
    double elev,	/* Elevation of the point in meters. (I)	      */
    double *x,		/* X value in meters. (O)			      */
    double *y,		/* Y value in meters. (O)			      */
    double *z		/* Z value in meters. (O)		              */
);
#endif
