#include <math.h>
#include "worgen.h"
#include "sartopo.h"
#include "protos.h"

void sph2cart
(
    double lat,     /* Latitude value in radians. (I)           */
    double lon,     /* Longitude value in radians. (I)          */
    double elev,    /* Elevation of the point in meters. (I)    */
    double *x,      /* X value in meters. (O)                   */
    double *y,      /* Y value in meters. (O)                   */
    double *z       /* Z value in meters. (O)                   */
)

/*******************************************************************************
NAME:	  SPH2CART 

PURPOSE:  Convert an input latitude, longitude, elevation coordinate to an
	  X, Y, Z cartesian coordinate.

PROGRAM HISTORY:
	By L. Huewe, USGS/EROS Data Center, 10/90

ALGORITHM:

	Calculate the cosine of the input latitude value.
	Calculate the x, y and z values corresponding to the input latitude,
	  longitude and elevation.
	Return to the calling routine.
		    
*******************************************************************************/
{
    double cos_lat; /* Cosine of input latitude.                */

    cos_lat = cos(lat);
    *x = (elev + EARTH_RAD) * cos(lon) * cos_lat;
    *y = (elev + EARTH_RAD) * sin(lon) * cos_lat;
    *z = (elev + EARTH_RAD) * sin(lat);

    return;
}
