#include "scatter.h"
#include "util.h"
#include "protos.h"

void pageout
(
    char lasin[][CMLEN],		/* Las input file name */
    long nimages,		/* Number of input images */
    long wind[][4],		/* Processing window specification */
    long hist[][114], 		/* Histogram bin populations */
    long row_start, 		/* Start index for rows */
    long row_end, 		/* Stop index for rows */
    char chars[],		/* Plotting characters */
    float rmin[2], 		/* Minimum pixel values for one plot page */
    float rmax[2], 		/* Maximum pixel values for one plot page */
    double xdlt,			/* X-axis increment */
    long page,			/* Counter for suppressed pages */
    long kmin, 			/* Minimum pixel count */
    long kmax,			/* Maximum pixel count */
    long nbrplt,			/* Number of plotting characters */
    char option[5],		/* Scaling option */
    long chrfil, 		/* Character file option flag */
    long mins[94] 		/* Range begin */
)

/*
**********************************************************************
NAME: PAGEOUT

FUNCTION: formats a single page for output

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
     1           Sept-83   R. J. Tighe
     2           Sep-87    D. Etrheim/edc   Prf #4228
     3           May-90    B. Allam	    Conversion to 5.0
     4        16-Nov-90    K. Kunska        Added comments; fixed line printer
                                            output; fixed annotation

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM:

   If on the first page, 
      Determine the frequency bin widths according to the user 
      specifications or calculate them using minimum and maximum
      bin ranges.
   For each row, 
      Initialize the data buffer.
      Read the plotting characters into the data buffer.
      If a non-zero frequency occurs, 
         Compute the bin in which it occurs and 
         Assign it a character symbol.
   Write the formated page to the output device.
   Write the bottom axis.
   If on the last page, 
      write the annotation with the processing window values and statistics.

***********************************************************************
*/
{

char	buf[ERRLEN],		/* Message buffer */
	dbuff[114];		/* Plotting buffer */

float	axval[5], 		/* X-axis values */
	ayval,			/* Y-axis values */
	temp;			/* Intermediate result */

long	bin_freq[94],		/* Bin freqency widths */
	bgg, 			/* Beginnig of a plot line */
	col,			/* Current column index */
	delta, 			/* Intermediate result */
	edd, 			/* End of a plot line */
	i, 			/* Loop counter and array index */
	index, 			/* Loop counter and index into plot buffer */
	count, 			/* Miscelanneous counter */
	row; 			/* Current row index */

/*
If on the first page, determine the frequency bin widths according to
the user specifications or calculate them using minimum and maximum
bin ranges.
----------------------------------------------------------------------
*/
if (page  ==  1) 
   {
   if (chrfil) 
      {
      for (i = 0; i < nbrplt; i++)
         bin_freq[i] = mins[i];
      }
   else      
      {
      temp = (float)(kmax - kmin + 1);
      delta = (long)(temp / (float) nbrplt + 0.5);
      if (delta == 0) delta = 1;
      bin_freq[0] = 0;
      bin_freq[1] = kmin;
      if (nbrplt > 2) 
         {
         for (i = 2; i < nbrplt; i++)
            bin_freq[i] = bin_freq[i-1] + delta;
         }  
      }  
   }
else
   c_txtout ("\f", print, fp);

count = 0;
ayval = rmax[0];

/*
For each row, initialize the data buffer.
------------------------------------------
*/
for (row = row_start; row >= row_end; row--)
   {
   index = 0;
   for (i = 0; i < 114; i++)
     dbuff[i] = ' ';
   for (col = 0; col < 114; col++)
      {

/*
Read the plotting characters into the data buffer.
If a non-zero frequency occurs, compute the bin in which it occurs and 
assign it a character symbol.
-----------------------------------------------------------------------
*/
      dbuff[index] = chars[0];
      for (i = 1; i < nbrplt; i++)
         {
         if (bin_freq[i] > hist[row-1][col]) 
            {
            dbuff[index] = chars[i-1];
            break;
            }  
         dbuff[index] = chars[nbrplt-1];
         }  
      index++;
      }  

/*
Write the formated page to the output device.
--------------------------------------------
*/   
   strcpy(buf, "");
   if (count == 0 || row_end == row) 
      {
      sprintf(&buf[strlen(buf)], " %11.5e I", ayval);
      for (index = 0; index < 114; index++)
         {
         if (dbuff[index] == '%')
           sprintf(&buf[strlen(buf)], "%%");
         else       
           sprintf(&buf[strlen(buf)], "%c", dbuff[index]);
         }
       }
   else
      {
      sprintf(&buf[strlen(buf)], "             I");
      for (index = 0; index < 114; index++)
         {
         if (dbuff[index] == '%')
           sprintf(&buf[strlen(buf)], "%%");
         else       
           sprintf(&buf[strlen(buf)], "%c", dbuff[index]);
         }
      }  
   c_txtout (buf, print, fp);
   count = (count + 1) % 14;
   ayval = ayval - (rmax[0] - rmin[0]) / 56.0;
   }  

/*
Write the bottom axis.
-----------------------
*/
strcpy(buf, "");
count = 0;
for (i = 0; i < 113; i++)
   {
   count = ((count + 1) % 12);
   if (count != 0) 
     dbuff[i] = '-';
   else
     dbuff[i] = 'I';
   }  

axval[0] = rmin[1];
for (i = 1; i < 5; i++)
   axval[i] = axval[i-1] + 24.0 * xdlt;

sprintf(&buf[strlen(buf)], "             I");
for (index = 0; index < 113; index++) 
   sprintf(&buf[strlen(buf)], "%c", dbuff[index]);
c_txtout (buf, print, fp);
strcpy(buf, "");
sprintf(&buf[strlen(buf)], "             %011.5e", axval[0]);
for (i = 1; i < 5; i++) 
   sprintf(&buf[strlen(buf)], "             %011.5e", axval[i]);
c_txtout (buf, print, fp);
strcpy(buf, "");

/*
If on the last page, write the annotation with the processing window
values and statistics.
----------------------------------------------------------------------
*/
if (page == 2) 
   {
   page = 3;
   if (!(strcmp(option, "SAME")))
       {
       sprintf(buf, " THE PLOTS ARE SCALED THE SAME");
       c_txtout (buf, print, fp);
       }
   else
       {
       sprintf(buf, " THE PLOTS ARE SCALED INDEPENDENTLY");
       c_txtout (buf, print, fp);
       }  
   sprintf(buf, " PIXEL VALUES FOR THE FIRST IMAGE ARE PLOTTED");
   sprintf(&buf[strlen(buf)], " ALONG THE Y-AXIS.");
   c_txtout (buf, print, fp);
   sprintf(buf, " PIXEL VALUES FOR THE SECOND IMAGE ARE PLOTTED");
   sprintf(&buf[strlen(buf)], " ALONG THE X-AXIS.");
   c_txtout (buf, print, fp);
   sprintf(buf, " FIRST IMAGE = %s(:%3d)", lasin[0], bands[0][0]);
   sprintf(&buf[strlen(buf)], "     PROCESSED WINDOW = ( %d %d %d %d )",
           wind[0][SL], wind[0][SS], wind[0][NL], wind[0][NS]);
   c_txtout (buf, print, fp);
   if (nimages > 1)
      {
      sprintf(buf, " SECOND IMAGE = %s(:%3d)", lasin[1], bands[1][0]);
      sprintf(&buf[strlen(buf)], "    PROCESSED WINDOW = ( %d %d %d %d )",
              wind[1][SL], wind[1][SS], wind[1][NL], wind[1][NS]);
      }
   else
      {
      sprintf(buf, " SECOND IMAGE = %s(:%3d)", lasin[1], bands[1][0]);
      sprintf(&buf[strlen(buf)], "    PROCESSED WINDOW = ( %d %d %d %d )",
              wind[0][SL], wind[0][SS], wind[0][NL], wind[0][NS]);
      }
   c_txtout (buf, print, fp);
   strcpy(buf, "");
   for (bgg = 0; bgg < nbrplt; bgg+=16)
      {
      edd = bgg + 15;
      if (edd > nbrplt) 
         {
         edd = nbrplt - 1;
         }  
      sprintf(&buf[strlen(buf)], "\n BIN #     :");
      for (i = bgg; i <= edd; i++) 
         sprintf(&buf[strlen(buf)], "     %2d", i+1);
      c_txtout (buf, print, fp);
      strcpy(buf, "");
      sprintf(&buf[strlen(buf)], " SYMBOL    :");
      for (i = bgg; i <= edd; i++)
         {
         if (chars[i] == '%')
           sprintf(&buf[strlen(buf)], "      %%");
         else       
           sprintf(&buf[strlen(buf)], "      %c", chars[i]);
         }
      c_txtout (buf, print, fp);
      strcpy(buf, "");
      sprintf(&buf[strlen(buf)], " MIN COUNT :");
      for (i = bgg; i <= edd; i++) 
         sprintf(&buf[strlen(buf)], " %6d", bin_freq[i]);
      c_txtout (buf, print, fp);
      strcpy(buf, "");
      }  /* do */
   }  /* if */
}	
