#include "scatter.h"
#include "protos.h"
#define MAX_PTS 94

void rdfil
(
    char *fname,			/* Input file with plotter characters */
    char chars[],		/* Plotting characters */
    long *rbegin,		/* Range begin values */
    long *rend,			/* Range end values */
    long *npts			/* Number of plotting characters */
)

/*
**********************************************************************
NAME: RDFIL

FUNCTION:  Reads the user-supplied file with the plotter characters.

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
     1          Sep-83    R. J. Tighe
     2          Sep-87    D. Etrheim/edc    Prf #4228
     3          May-90    B. Allam	    Conversion to 5.0
     4        16-Nov-90   K. Kunska         Added prolog and comments;
                                            Changed calls to open, read, close
                                            to fopen, fscanf, and fclose.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

ALGORITHM:

   Open the point file.
   Read the set of characters, the beginning of the range and the end 
   of the range.
   Close the point file.

**************************************************************************
*/
{

long 	eof_flg,		/* End-of-file flag */
	i,			/* Loop counter and array index */
	status;			/* Return status */

/*
Open the point file.
---------------------
*/
if ((fd = fopen(fname, "r")) == NULL)
   msg_err ("!Error opening map point file", "scatter-open");

/*
Read the set of characters, the beginning of the range and the end 
of the range.
--------------------------------------------------------------------
*/
for (*npts = 0; ; rbegin++, rend++, (*npts)++)
   {
   if (*npts >= MAX_PTS)
      msg_err ("!Too many chars and range values", "scatter-range");
   i = *npts;
   status = fscanf (fd, "%1c %d %d\n", &chars[i], rbegin, rend);
   eof_flg = feof(fd);
   if (eof_flg != 0)
      break;
   if (status != 3)
      msg_err ("!File format error", "scatter-format");
   }

/*
Close the point file.
----------------------
*/
fclose(fd);
}
