#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "statio.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

struct PARBLK block;                   /* TAE parameter block     */

FILE    *fp;                           /* output file descriptor  */
FILE    *fd;                           /* text file descriptor */

struct FDESC * fdesc[MAXIMG];                 /* file descriptors        */
struct GDESC * gdesc;                         /* group descriptor        */
long    ipoint[MAXIMG * MAXBND + 1];   /* offsets into image i/o  */
int     print[3];

long    count;                         /* no. of files opened */
long    bands[MAXIMG][MAXBND+1];
long    nbands[MAXIMG];

char    out[MAXIMG][CMLEN];             /* names of files opened */
char    print_file[CMLEN];
float   *iobuf;                        /* image i/o buffer        */

extern long     curcat;                        /* message utility         */
extern long     curent;                        /* parameters for processing */

#define TERM 0
#define LP   1
#define FIL 2

/****************************************************************************
NAME:  SCATTER

FUNCTION:   SCATTER builds a three-dimensional histogram where pixel 
            values from the two input bands provide the first two 
            dimensions and the histogram bin populations provide the 
            third. The lower left corner of the two-dimensional array 
            is the bin for pixel values (0,0). The pixel value from the 
            first band (band a) provides the y axis bin index. The pixel
            value from the second band (band b) provides the x axis bin 
            index. 

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
     1          Sep-83   R. J. Tighe
     2          Apr-88   P. Pease 	  Change in user_name subroutine to 
                                          allow 12 character names.
     3          Apr-89   B. Allam         Conversion to 5.0
     4        16-Nov-90  K. Kunska        Fixed several little bugs and an
                                          access violation when reading INFILE;
                                          added comments and .dat as default
                                          extention for INFILE
     5	      23-Sep-92	 T. Mittan	  Modified include file to eliminate
					  warnings on link
                                          
ALGORITHM:

   Initialize TAE.
   Call GETPARM to fetch input parameters.
   Call SETUP to open input images.
   Call ACCUM to accumulate the histogram data.
   Call IPRINT to print the histogram data.
   Print the completion message

**********************************************************************
*/
int main
(void) /* scatter */
{

char	chars[95],		/* plotting characters */
	fill_val,		/* Fill value */
	hostin[MAXIMG][CMLEN],	/* Host input file names */
	infile[CMLEN],		/* input file */
	lasin[MAXIMG][CMLEN],	/* Las input file names */
	print_host[CMLEN],	/* Print file name */
	option[5];		/* Scaling option */

float	rmax[2], 		/* Maximum pixel values for plotting */
	rmin[2];		/* Minimum pixel values for plotting */

long	chrfil=0,		/* Infile option */
	fill_num,		/* Number of elements to fill */
	fill_type;		/* Data type of the array to be filled */
static long harray[114][114]; 	/* Histogram bin populations */
long    ik[2],			/* Minmax count */
        kmin, kmax, 		/* Min and max pixel counts */
        maxs[94], mins[94], 	/* Range begin and end */
        msgtim, 		/* Message time interval */
        nbrplt, 		/* Number of plotting characters */
	nimages, 		/* Number of images */
	window[MAXIMG][4],	/* Processing window for images */
	std_out;                /* Logical unit */

/*
Initialize TAE.
---------------
*/
c_inlas (&block, &std_out);

/*
Call GETPARM to fetch input parameters.
-----------------------------------------
*/
fill_type = EBYTE;
fill_num = 94;
fill_val = ' ';
c_pxfill (chars, &fill_type, &fill_num, &fill_val);
getparm (lasin, hostin, window, chars, rmin, rmax, infile, &nbrplt, 
         &msgtim, option, &chrfil, mins, maxs, ik, &nimages);

/*
Call SETUP to open the input images.
-------------------------------------
*/
setup (hostin, window, nimages);

/*
Call ACCUM to accumulate the histogram data.
---------------------------------------------
*/
accum (window, harray, rmin, rmax, &kmin, &kmax, msgtim, ik);

/* close the input images */
c_egclse(&gdesc);
gdesc = NULL;

/*
Call IPRINT to print histogram data.
-------------------------------------
*/
iprint (lasin, nimages, window, harray, chars, rmin, rmax, kmin, kmax, 
        nbrplt, option, chrfil, mins);

count = 0;
c_cmhost(print_file,print_host);
if (print[FIL])
    strcpy(out[count++], print_host);

/*
Print the completion message.
-----------------------------
*/
c_complt("SCATTER", out, &count);

return EXIT_SUCCESS;

}
