#include "search.h"
#include "pixman.h"
#include "protos.h"

void cscio
(
    long *sl,			/* strip line counter                         */
    long npix,			/* number of pixels in the input image        */
    long *samp, 			/* total number of pixels to be processed by  */
    long *strip, 		/* strip number (counter)                     */
    long tot_strip, 		/* total number of strips                     */
    long *pix_rem, 		/* count of pixels left from last strip       */
    long *end_flg, 		/* completion indicator                       */
    float ***data, 		/* data buffer                                */
    long nbands			/* number of input bands                      */
)
/*
*****************************************************************************

NAME: 	CSCIO

PURPOSE:  Fetches pixel data for an image strip.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         01-Oct-84   Y. Sheu
   3.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables and compute the number of samples to be processed by 
   this strip.
   If the number of samples for this strip exceeds 6, 
      Compute the starting sample
      And copy data from the work buffer to the data buffer.
   If strip number is not equal to the strip count, reset the line index.
   Otherwise, reset the strip counter to 0.

*****************************************************************************
*/

{

long	band,			/* current band being processed               */
	bcount, 		/* number of bytes to move for this strip     */
	dtype,			/* data type                                  */
	exit_flg,		/* exit flag                                  */
	line,			/* current line being processed               */
	nsamp,			/* number of pixels to read in for this strip */
	ss;			/* starting pixel for this strip              */

/*
Initialize variables and compute the number of samples to be processed by 
this strip.
---------------------------------------------------------------------------
*/
dtype = EREAL;
exit_flg = FALSE;
nsamp = min((npix - (*strip * 400)), 400);                                   

/*
If the number of samples for this strip exceeds 6, compute the starting sample.
and copy data from the work buffer to the data buffer.
-------------------------------------------------------------------------------
*/
if ((nsamp + *pix_rem) < 6) 
   exit_flg = TRUE;
if (!exit_flg)
   {
   ss = 1 + (*strip * 400);
   bcount = nsamp;
   for (line = 1; line <= 6; line++)         
      {
      for (band = 1; band <= nbands; band++)
         c_pxcopy (&work[band][line][ss], &data[band][line][*pix_rem+1], 
                   &dtype, &bcount);
      *sl = *sl + 1;
      }

/*
If strip number is not equal to the strip count, reset the line index.
Otherwise, reset the strip counter to 0.
-------------------------------------------------------------------------
*/
   *samp =  nsamp + *pix_rem;
   *strip = *strip + 1;
   if (*strip != tot_strip) 
      *sl = *sl - 6;            
   else
      *strip = 0;    
   *pix_rem = 0;            
   }
else
   *end_flg = 1;            
}   

