#include "search.h"
#include "protos.h"

void divmc
(
    float **classtat, 		/* class statistics                           */
    long *init,			/* processsing control parameter              */
    float dv[],			/* pair-wise divergences for all              */
    float ptt,			/* number of pixels in one field              */
    float rfc,			/* scaling factor for separability measure    */
    long *maxp,			/* pointer to last class accepted             */
    long nbands			/* number of input image bands                */
)
/*
*****************************************************************************

NAME: DIVMC

PURPOSE:  Merges and deletes classes according to a sparability criterion
	  provided be the inter-class divergence. When all divergences are 
          smaller than the input threshold then all classes are merged down 
          to just one class. 

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         14-Dec-83   R. J. Tighe	
   3.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Call CALCDIV to calculate the divergence values for all classes.
   Look for the smallest divergence among them.
   If the smallest divergence is less than user defined threshold, 
      Set merge_flg to TRUE.
   Otherwise, 
      Delete sites with a single field of divergence = 4.5*mthresh (user
      defined maximum desired separability for merger)
      Or delete classes with 4, 3, 2, 1 fields with divergences of 1.5, 2.5,
      3.5, 4.5 times mthresh.
   Pack the rest of the classes.
   If merge_flg is set to TRUE, 
      Call MERGS to merge the two classes with the minimum divergence.
   Call DIVRG to calculate the divergences among the new classes after merge.

*****************************************************************************
*/

{

float	pts,			/* number of pixels                           */
	smd;			/* smallest divergence found so far           */

long	ad,
        main_flg,		/* control flag for the main loop             */
	merge_flg,		/* control flag for performing merge          */
	minptr,			/* first class to be accepted                 */
	i, i1,			/* loop counters and array indexes            */
	istp, j,		/*               ,,                           */
	k1, k2, k3,		/*               ,,                           */
	l, l1,			/*               ,,                           */
	loc,			/* location of divergence that could be       */
				/* deleted                                    */
	nrc,			/* number of class to be packed               */
	status,			/* return status                              */
	upbound;		/* last class upper bound                     */

status = 0;
main_flg = merge_flg = FALSE;

/*
Call CALCDIV to calculate the divergence values for all classes.
-----------------------------------------------------------------
*/
if (*init > 0)
   {
   if ((*maxp - 50) == 0)
      main_flg = TRUE;
   else
      {
      minptr = 2;
      *maxp = num_class;
      upbound = *maxp;
      calcdiv (minptr, upbound, classtat, dv, nbands);
      main_flg = TRUE;
      }
   }
else if (*init == 0)
   {
   minptr = num_class;
   *maxp = num_class;
   upbound = *maxp;
   calcdiv (minptr, upbound, classtat, dv, nbands);
   main_flg = TRUE;
   }
else   /* *init < 0 */
   {
   *init = 0;      
   minptr = 2;       
   *maxp = num_class;
   upbound = *maxp;
   calcdiv (minptr, upbound, classtat, dv, nbands);
   main_flg = TRUE;
   }

while (main_flg)
   {
   l = 2;
   smd = dv[1];
   k1 = 2;     
   k2 = 1;     
/*
Look for the smallest divergence among them.
--------------------------------------------
*/
   if (num_class != 2)
      {
      for (i = 3; i <= num_class; i++)
         {
         i1 = i - 1;       
         for (j = 1; j <= i1; j++)
            {
            if ((dv[l] - smd) < 0)
               {
               smd = dv[l];      
               k1 = i;     
               k2 = j;
               }
            l = l + 1;          
            }
         }
      } /* if != 2 */

/* 
If the smallest divergence is less than user defined threshold, set merge_flg
to TRUE.
-----------------------------------------------------------------------------
*/
   if ((smd - mthresh) <= 0)
      /* call merge and continue from there */
      merge_flg = TRUE;
   else
      { 
      if ((*init - 1) > 0)
         break;
      if ((*init - 1) < 0) 
         {
/*
Otherwise, delete sites with a single field of divergence = 4.5*mthresh.
------------------------------------------------------------------------
*/
         ad = (long) (4.5 * rfc * mthresh);
         for (i = 1; i <= 50; i++)
            {
            if ((classtat[i][POP] - ptt) <= 0) 
               {
               for (j = 1; j <= 50; j++)      
                  {
                  if ((j - i) < 0) 
                     {
                     if (classtat[j][POP] > 0) 
                        {
                        loc = (i - 1) * (i - 2) / 2 + j; 
                        if ((dv[loc] - (float) ad) < 0)
                           {
                           classtat[i][POP] = 0.0;
                           break;
                           }
                        }
                     }
                  else if ((j - i) > 0)
                     {
                     if ((classtat[j][POP] - ptt) > 0)
                        {
                        loc = (j - 1) * (j - 2) / 2 + i;
                        if ((dv[loc] - (float) ad) < 0)
                           {
                           classtat[i][POP] = 0.0;
                           break;
                           }
                        }
                     }
                  } /* enddo */
               } /* endif */
            } /* enddo */
         istp = 50;
         } /* endif *init - 1 < 0 */

      else /* if ((*init - 1) == 0) */
         {
/*
Or delete classes with 4, 3, 2, 1 fields with divergences of 1.5, 2.5, 3.5, 
4.5 times mthresh.
----------------------------------------------------------------------------
*/
         for (k1 = 1; k1 <= 4; k1++)      
            {
            pts = ptt * (float) (5 - k1);
            ad = (long) (mthresh * (k1 + 0.5));
            ad = (long) (rfc * (float) ad);     
            for (i = 1; i <= num_class; i++)
               {
               if ((classtat[i][POP] - pts) == 0) 
                  {
                  for (j = 1; j <= num_class; j++)
                     {
                     if ((j - i) < 0)
                        {
                        if ((classtat[j][POP] - pts) >= 0) 
                           {
                           loc = (i - 1) * (i - 2) / 2 + j;
                           if ((dv[loc] - (float) ad) <= 0) 
                              classtat[i][POP] = 0.0;
                           }
                        }
                     else if ((j - i) > 0)
                        {
                        if ((classtat[j][POP] - pts) > 0)
                           {
                           loc = (j - 2) * (j - 1) / 2 + i;
                           if ((dv[loc] - (float) ad) <= 0)
                              classtat[i][POP] = 0.0;
                           }
                        }
                     } /* for */
                  } /* if */
               } /* for */
            } /* for */
          istp = num_class;
          }    
/*
Pack the rest of the classes.
-------------------------------
*/
      nrc = 1;        
      for (i = 1; i <= istp; i++)
         {
         if (classtat[i][POP] > 0) 
            {
            if ((i - nrc) != 0) 
               {
               for (j = 1; j <= POP; j++) 
                  classtat[nrc][j] = classtat[i][j];      
               }
            nrc = nrc + 1;
            }
         }

      num_class = nrc - 1; 
      if ((num_class - 50) >= 0)
         {
         mthresh = mthresh + 1.0;
         merge_flg = FALSE;   
         }
      else /* if ((num_class - 50) < 0)  */
         {
         if (*init == 0) 
            *init =  - 1;
         break; 
         }
      } /* endif smd > 0 */

/*
If merge_flg is set to TRUE, call MERGS to merge the two classes with the 
minimum divergence.
----------------------------------------------------------------------------
*/
   if (merge_flg)
      {
      merge_flg = FALSE;
      mergs (classtat[k2], classtat[k1], nbands, &status);
      if (status == 1) 
         {
         *init = 5;
         break;
         } 
      }

/*
Call DIVRG to calculate the divergences among the new classes after merge.
--------------------------------------------------------------------------
*/
   for (i = 1; i <= POP; i++)
      classtat[k1][i] = classtat[num_class][i];
  
   num_class = num_class - 1;
   l = num_class * (num_class - 1) / 2 + 1;
   k3 = (k1 - 1) * (k1 - 2) / 2;
   
   for (i = 1; i <= num_class; i++)
      {
      if ((i - k1) < 0)
         {
         l1 = k3 + i;      
         dv[l1] = dv[l];       
         }
      else if ((i - k1) > 0)
         {
         l1 = (i - 2) * (i - 1) / 2 + k1;
         dv[l1] = dv[l];       
         }
      l = l + 1;
      }

   k3 = (k2 - 1) * (k2 - 2) / 2;

   for (i = 1; i <= num_class; i++)
      {
      if ((i - k2) < 0)
         {
         l1 = k3 + i;
         divrg (classtat[k2], classtat[i], nbands, &dv[l1]);
         }
      else if ((i - k2) > 0)
         {
         l1 = (i - 2) * (i - 1) / 2 + k2;
         divrg (classtat[k2], classtat[i], nbands, &dv[l1]);      
         }   
      }

      if (num_class == 1) 
         break;
      if (*init <= 0)
         break;
   } /* end while main_flg */
}     

