#include "search.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char hostin[][CMLEN], 		/* host name for input image          */
    long num_inimages,			/* total number of input images       */
    long num_img_bands[][MAXBND+1],	/* band numbers for each input image  */
    long bands_per_img[]		/* number of bands for each image     */
)
/*
*****************************************************************************

NAME: FILSPEC

PURPOSE:  Opens the input image file.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         07-Jul-89   K. Kunska/STX	conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open the input image file.
   Allocate buffer space for the input image files.
   Group the input image files.

*****************************************************************************
*/

{

long	access,				/* access type                        */
	bufsize,			/* buffer size                        */
	n,				/* loop counter and array index       */
	nib,				/* band index                         */
	stat,				/* return status                      */
					/* image bands                        */
	type;				/* data type                          */

long    zero = 0;
long    one = 1;

/*
Open the input image file.
---------------------------
*/
access = IREAD;
type = EREAL;
for (n = 0, nib = 0; n < num_inimages; n++)
   {
   stat = c_eopens (&fdescin[n], hostin[n], num_img_bands[n], &window[n][SL],
                    &window[n][SS], &window[n][NL], &window[n][NS], 
                    &ipoint[nib], &access, &type, &zero, &zero);
   if (stat != E_SUCC)
      c_abort (&block, "Error opening the input image", "search-inopen",
               "search", hostin[n], &one);
   nib += bands_per_img[n];
   }
fdescin[++n] = 0;

/*
Allocate buffer space for the input image files.
-------------------------------------------------
*/
if ((inbuf = (float *) c_ealloc (fdescin, &bufsize)) == NULL)
   c_abort (&block, "Error allocating dynamic memory", "search-alloc", 
            "search", out_files, &num_outfiles);

/*
Group the input image files.
-----------------------------
*/
c_egroup (fdescin, &gdescin, inbuf, &bufsize);

}
