#include "search.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char taein[][CMLEN],		/* input image tae name                       */
    char hostin[][CMLEN], 	/* input image host name                      */
    char taeout[],		/* output image tae name                      */
    char hostout[], 		/* output image host name                     */
    long *num_inimg, 		/* number of input images                     */
    long (*num_img_band)[MAXBND+1], 	/* band numbers for each input image  */
    long *bands_per_img,		/* number of bands in each input image        */
    long *tot_inband,		/* total number of bands                      */
    long img_limit,		/* maximum image limit                        */
    long *same_wnd,		/* same window size? 1=YES, 0=NO              */
    long (*img_flag)[2],		/* flag for the window/band option            */
    char print_file[],		/* print file name                            */
    long *dtype			/* data type of the input image               */
)
{

/*
*****************************************************************************

NAME: GETPAR

PURPOSE:  Retrives user supplied parameter values and performs appropriate
          validity tests.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe      
   2.0         26-Mar-85   Y. Sheu
   3.0         07-Jul-89   K. Kunska		conversion to 5.0
   4.0	       09-Sep-89   K. Kunska		fixed listing of created output
                                                files

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Initialze LAS.
   Get the input image name and data type.
   Get the option for the statistics file.
   If the option is "YES", set savestat flag to TRUE; otherwise set it to FALSE.
   Get the output file name.
   Get the size of the pixel blocks and the type of the interclass 
   distance measure.
   Get the maximum desire separability for merger.
   Set the appropriate default values.
   Get the standard deviation lower and upper bounds.
   Get the coefficient of variation.
   Open the print file.
   Get the LAS global parameters: $MSGTIM and $MINMAX.

*****************************************************************************
*/


char	fileutil_file[CMLEN],	/* file for fileutil routine                  */
	mnmx[4];		/* minmax global parameter                    */

float	stdevlim[2];		/* upper and lower bounds for the standard    */
				/* deviation                                  */
long	access,			/* access type                                */
	i,			/* loop counter and array index               */
	par_type,		/* data type of a parameter                   */
	stat,			/* return status                              */
	std_out;		/* logical unit for standard output           */

struct VARIABLE  *var; 	/* pointer to a variable in the tae block     */

long    fatal_err = -1;         /* fatal error return status          */
long    zero = 0;
long    one = 1;
long    two = 2;

/*
Initialze LAS.
--------------
*/
c_inlas (&block, &std_out); 

/*
Get the input image name and data type.
-----------------------------------------
*/
stat = c_getin (&block, "IN", &zero, taein, hostin, num_inimg, window,
                num_img_band, bands_per_img, tot_inband, &img_limit,
                same_wnd, img_flag); 
if (stat!=E_SUCC)
   c_errmsg ("Cannot access the input image name", "search-getin", &fatal_err);

for (i = 0; i < *num_inimg; i++)
   {
   c_getddr(hostin[i], &ddr);
   *dtype = ddr.dtype;
   }

/* 
Get the output file name.
If the output file name is not null, set savestat flag to TRUE.
----------------------------------------------------------------
*/
num_outfiles = 0;
access = IWRITE;
stat = c_getnam (&block, "OUTSTAT", &zero, hostout, "stat", &access); 
if (stat!=E_SUCC)
   c_errmsg ("Cannot access the statistics file", "search-getnam", 
              &fatal_err);
savestat = FALSE;
if (strlen(hostout) != 0)
   {
   savestat = TRUE;
   strcpy(taeout, hostout);
   strcpy (out_files[num_outfiles++], hostout);
   }

/*
Get the size of the pixel blocks and the type of the interclass 
distance measure.
------------------------------------------------------------------
*/
par_type = ELONG;
c_getnum (&block, "SELMETH", &one, &par_type, (void *)&block_dist); 

/*
Get the maximum desire separability for merger.
Set the appropriate default values.
------------------------------------------------
*/
par_type = EREAL;
c_getnum (&block, "MERGETHR", &one, &par_type, (void *)&mthresh); 

if (mthresh == 0.0) 
   {
   if (block_dist == 0) 
      mthresh = 5.0;
   else
      mthresh = 2.4;
   }  

/*
Get the standard deviation lower and upper bounds.
--------------------------------------------------
*/
c_getnum (&block, "STDEVLIM", &two, &par_type, (void *)stdevlim); 
stdevlb = stdevlim[0];
stdevub = stdevlim[1];

/*
Get the coefficient of variation.
---------------------------------
*/
c_getnum (&block, "COEFVAR", &one, &par_type, (void *)&coefvar); 

/*
Open the print file.
--------------------
*/
strcpy (fileutil_file, "search");
if (p_find (&block, "PRINT")->v_count != 0)
   {
   stat = c_setprt(&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
             print_file);
   if (stat != E_SUCC)
      c_errmsg("Fatal error encountered","search-fatal",&fatal_err);
   }

if (print[LP] || print[FIL] || print[TERM])
   {
   fd1 = fopen (fileutil_file, "w");
   strcpy (out_files[num_outfiles++], print_file);
   }
else
   fd1 = NULL;

/*
Get the LAS global parameters: $MSGTIM and $MINMAX.
---------------------------------------------------
*/
par_type = ELONG;
c_getnum (&block, "$MSGTIM", &one, &par_type, (void *)&msgtim);
var = p_find (&block, "$MINMAX");
strcpy (mnmx, SVAL (*var, 0));
if (mnmx[0] == 'Y')
   min_max = TRUE;
else
   min_max = FALSE;
}
