#include "search.h"
#include "util.h"
#include "protos.h"

void matinv
(
    float **a,			/* matrix to be inverted in place             */
    long size,			/* number of rows and/or columns in the matrix*/
    long *ret_code		/* return code for matrix inversion           */
)
/*
*****************************************************************************

NAME: 	MATINV

PURPOSE:  Inverts a square matrix in place. Terminates with a return code = 1
          if the matrix is singular or near singular.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   For all rows:
      Initialize the pivot.
      If the absolute value of the pivot is greater than 0.01, 
         Find the inverse of the matrix.
      If the matrix is singular, return with an error message (code = 1).
   End-for.   

*****************************************************************************
*/

{

double	temp_pvt;		/* holds the value of pvt in double precision */

float	pvt,			/* pivot value for matrix inversion           */
	temp;

long	j, k, l,		/* loop counters and array indexes            */
	skip_flg;		/* skip flag for the values on the diagonal   */

*ret_code = 0;

/* 
For all rows: initialize the pivot.
If the absolute value of the pivot is greater than 0.01, find the inverse
of the matrix.
------------------------------------------------------------------------------
*/
for (j = 1; j <= size; j++)
   {
   pvt = a[j][j];
   a[j][j] = 1.0;
   temp_pvt = (double)pvt;
   if (fabs(temp_pvt) > 0.01) 
      {
      for (k = 1; k <= size; k++)
         a[j][k] = a[j][k] / pvt;

      for (k = 1; k <= size; k++)
         {
         skip_flg = FALSE;
         if (k == j) 
            skip_flg = TRUE;
         if (!skip_flg)
            {
            temp = a[k][j];
            a[k][j] = 0.0;
            for (l = 1; l <= size; l++)
              a[k][l] = a[k][l] - a[j][l] * temp;
            } /* skip */
         }
      } /* if */
/*
If the matrix is singular, return.
-----------------------------------
*/
   else
      {
      *ret_code = 1;
      c_abort (&block, "Cannot invert a singular matrix", "search-matinv",
               "search", out_files, &num_outfiles);
      }
   } /* for */
}

