#include "search.h"
#include "protos.h"

void mergsd
(
    float a[], 			/* destination class statistics which will    */
    float b[],			/* class containig statistics to be merged -  */
    long bands 			/* number of input image bands                */
)
/*
*****************************************************************************

NAME: 	MERGSD

PURPOSE:  Merges statistics for two classes. It is called only by the SDMC
          routine when the user has requested one of the scaled distance
          options.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    Runs under TAE
    There is a potential loss of numerical precision in the merger of 
    covariance matrices for large values of pt1 and/or pt2.

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Compute merged means.
   Compute merged covariance matrix.
   Save the merged statistics.

*****************************************************************************
*/

{

float	pt,			/* sum of populations of class a and b        */
	pt1,			/* population of class a                      */
	pt2,			/* population of class b                      */
	tm[MAXBND];		/* temporary buffer for merged means          */

long	band,			/* current band number                        */
	i1, j, j1, k1,		/* loop counters and array indexes            */
	l;

pt1 = a[POP];
pt2 = b[POP];
pt = pt1 + pt2;

/*
Compute merged means.
----------------------
*/
l = MEAN;
for (band = 1; band <= bands; band++)
   {
   l++;
   tm[band] = (pt1 * a[l] + pt2 * b[l]) / pt;
   }

/*
Compute merged covariance matrix.
---------------------------------
*/
k1 = 1;
for (band = 1; band <= bands; band++)
   {
   i1 = band + MEAN;
   for (j = 1; j <= band; j++)
      {
      j1 = j + MEAN;
      a[k1] = ( (pt1 - 1) * a[k1] + pt1 * a[i1] * a[j1]
              + (pt2 - 1) * b[k1] + pt2 * b[i1] * b[j1] 
              - pt * tm[band] * tm[j]) / (pt - 1);
      if ((band == j) && (a[k1] < 0.0)) 
         a[k1] = 0.0;
      k1++;
      }
   }

/*
Save the merged statistics.
----------------------------
*/
a[POP] = pt; 
l = MEAN;
for (band = 1; band <= bands; band++)
   {
   l++;
   a[l] = tm[band];
   }
}       

