#include "search.h"
#include <string.h>
#include "pixman.h"
#include "protos.h"

void pageout
(
    long ktf, 			/* total number of fields processed by search */
    long nflds, 			/* number of fields which produced the set of */
    long num_inimg,		/* number of input images                     */
    long class,			/* class index for this class                 */
    float smns[], 		/* mean vector for this class                 */
    float sd[],			/* standard deviation for this class          */
    float zmv[], 			/* coefficient of variation for this class    */
    float zcrl[],			/* correlation matrix for this class          */
    long data_type		/* input image data type                      */
)

/*
*****************************************************************************

NAME: 	PAGEOUT

PURPOSE:  Formats the statistical summary information for one class for 
          output to the printer or file.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         25-Mar-85   Y. Sheu
   3.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Initialize variables.
   Write the number of fields used and the total number of fields processed.
   Write the means statistics for each band.
   Write the standard deviations for each band.
   Write the coefficient of variance for each band.
   Write the covariance matrix.
   Write the correlation matrix.

*****************************************************************************
*/

{

char	*blank,			/* blank fill value for the text buffer       */
	ch[20],			/* band/channel number                        */
	index[4],		/* image index number                         */
	line[132],		/* line of text                               */
	text[132];		/* text buffer                                */
	
double	exp,			/* exponent for pow function                  */
	flds;

long	dtype,			/* data type                                  */
	i, j,			/* loop counters and array indexes            */
	n1, n2,			/*		  ,,			      */
	txtlen;			/* length of the text buffer                  */

/* 
Initialize variables.
----------------------
*/
txtlen = 132;
dtype = EBYTE;
blank = " ";
exp = 0.25;
flds = (double)nflds;
flds = pow (flds, exp);
strcpy(ch, "BAND");
c_pxfill (line, &dtype, &txtlen, blank);
line[0] = '\0';

/* 
Format an output page of statistical summary information for one class.
-----------------------------------------------------------------------
*/
fprintf (fd1,"\f"); 
sprintf (text, "\n          STATISTICAL SUMMARY FOR CLASS %2d ", class);
fprintf (fd1,"%s\n",text);

/*
Write the number of fields used and the total number of fields processed.
--------------------------------------------------------------------------
*/
sprintf (text, "\n   CLASS %2d USED %4d FIELDS OUT OF A TOTAL OF %4d", class,
         nflds, ktf);
sprintf (&text[strlen(text)], " PROCESSED.\n");
sprintf (&text[strlen(text)], "            APRIORI = %10.7f\n", flds);
fprintf (fd1,"%s\n",text);

/*
NOTE:  The following will cause an error if there are too many bands being
processed.  The string 'line' will become too large and cause the error.
Will take a major rewrite to correct this problem.
-------------------------------------------------------------------------
*/
strcat (line, "                                         ");
for (i = 1; i <= num_inimg; i++)
   {
   sprintf (index, "%2d", i); 
   strcpy(&ch[7], index);
   strcpy (&line[strlen(line)], ch);
   strcat (line, "        ");
   }
fprintf (fd1,"%s\n\n",line);

/* 
Write the means statistics for each band.
-----------------------------------------
*/
text[0] = '\0';
sprintf (&text[strlen(text)], "            MEANS                   ");
if (data_type <= 3)
   {
   for (i = 1; i <= num_inimg; i++)
      sprintf (&text[strlen(text)], "%9.2f   ", smns[i]);
   }
else
   {
   for (i = 1; i <= num_inimg; i++)
      sprintf (&text[strlen(text)], "%9.2e   ", smns[i]);
   }
fprintf (fd1,"%s\n",text);
c_pxfill (text, &dtype, &txtlen, blank);

/* 
Write the standard deviations for each band.
--------------------------------------------
*/
text[0] = '\0';
sprintf (&text[strlen(text)], "            STANDARD DEVIATION      ");
if (data_type <= 3)
   {
   for (i = 1; i <= num_inimg; i++)
      sprintf (&text[strlen(text)], "%9.2f   ", sd[i]);
   }
else
   {
   for (i = 1; i <= num_inimg; i++)
      sprintf (&text[strlen(text)], "%9.2e   ", sd[i]);
   }
fprintf (fd1,"%s\n",text);
c_pxfill (text, &dtype, &txtlen, blank);

/*
Write the coefficient of variance for each band.
------------------------------------------------
*/
text[0] = '\0';
sprintf (&text[strlen(text)], "            COEFFICIENT OF VARIANCE ");
if (data_type <= 3)
   {
   for (i = 1; i <= num_inimg; i++)
      sprintf (&text[strlen(text)], "%9.2f   ", zmv[i]);
   }
else
   {
   for (i = 1; i <= num_inimg; i++)
      sprintf (&text[strlen(text)], "%9.2e   ", zmv[i]);
   }
fprintf (fd1,"%s\n",text);
c_pxfill (text, &dtype, &txtlen, blank);
fprintf (fd1,"\n");

/*
Write the covariance matrix.
-----------------------------
*/
sprintf (text, "\n            COVARIANCE MATRIX FOR CLASS %2d", class);
fprintf (fd1,"%s\n",text);
c_pxfill (text, &dtype, &txtlen, blank);
fprintf (fd1,"\n"); 

n1 = 1;
for (i = 1; i <= num_inimg; i++)
   {
   text[0] = '\0';
   strcat (text, "        ");
   n2 = n1 + i - 1;
   if (data_type <= 3)
      {
      for (j = n1; j <= n2; j++)
         sprintf (&text[strlen(text)], "%9.2f   ", scv[j]);
      }
   else
      {
      for (j = n1; j <= n2; j++)
         sprintf (&text[strlen(text)], "%9.2e   ", scv[j]);
      }
   fprintf (fd1,"%s\n",text);
   c_pxfill (text, &dtype, &txtlen, blank);
   n1 = n2 + 1;
   }  

/* 
Write the correlation matrix.
------------------------------
*/
sprintf (text, "\n\n            CORRELATION MATRIX FOR CLASS %2d", class);
fprintf (fd1,"%s\n",text);
c_pxfill (text, &dtype, &txtlen, blank);
fprintf (fd1,"\n"); 

n1 = 1;
for (i = 1; i <= num_inimg; i++)
   {
   text[0] = '\0';
   strcat (text, "        ");
   n2 = n1 + i - 1;
   for (j = n1; j <= n2; j++)
      sprintf (&text[strlen(text)], "%9.2f  ", zcrl[j]);
   fprintf (fd1,"%s\n",text);
   c_pxfill (text, &dtype, &txtlen, blank);
   n1 = n2 + 1;
   }  

}
