#ifndef PROTOS_H
#define PROTOS_H

void calcdist
(
    long minptr,			/* pointer to the first class accepted        */
    long maxptr,			/* pointer to last class accepted             */
    float **classtat, 		/* class statistics                           */
    float dv[],			/* pair-wise divergences for all classes      */
    long nbands 		/* number of input image bands                */
);

void calcdiv
(
    long minprt,			/* pointer to first class accepted            */
    long maxprt,			/* pointer to last class accepted             */
    float **classtat, 		/* class statistics                           */
    float dv[],			/* pair-wise divergences for all              */
    long nbands 		/* number of input image bands                */
);

void cscio
(
    long *sl,			/* strip line counter                         */
    long npix,			/* number of pixels in the input image        */
    long *samp, 			/* total number of pixels to be processed by  */
    long *strip, 		/* strip number (counter)                     */
    long tot_strip, 		/* total number of strips                     */
    long *pix_rem, 		/* count of pixels left from last strip       */
    long *end_flg, 		/* completion indicator                       */
    float ***data, 		/* data buffer                                */
    long nbands			/* number of input bands                      */
);

void divmc
(
    float **classtat, 		/* class statistics                           */
    long *init,			/* processsing control parameter              */
    float dv[],			/* pair-wise divergences for all              */
    float ptt,			/* number of pixels in one field              */
    float rfc,			/* scaling factor for separability measure    */
    long *maxp,			/* pointer to last class accepted             */
    long nbands			/* number of input image bands                */
);

void divrg
(
    float *a, 		/* one of pair of classes for which divergence is to  */
    float *b,		/* the other class for which divergence is to be      */
    long bands,		/* number of input bands                              */
    float *divergence	/* computed output divergence                         */
);

void filspec
(
    char hostin[][CMLEN], 		/* host name for input image          */
    long num_inimages,			/* total number of input images       */
    long num_img_bands[][MAXBND+1],	/* band numbers for each input image  */
    long bands_per_img[]		/* number of bands for each image     */
);

float **fmat2d
(
    long nr,
   long nc
);

float ***fmat3d
(
    long nx,
   long ny,
   long nz
);

void freefmat2d
(
    float **m,
    long nr  	/* no. rows			*/
);

void freefmat3d
(
    float ***m,
    long nx, 	/* x dimension (ex: bands)	*/
    long ny 	/* y dimension (ex: lines)	*/
);

void getpar
(
    char taein[][CMLEN],		/* input image tae name                       */
    char hostin[][CMLEN], 	/* input image host name                      */
    char taeout[],		/* output image tae name                      */
    char hostout[], 		/* output image host name                     */
    long *num_inimg, 		/* number of input images                     */
    long (*num_img_band)[MAXBND+1], 	/* band numbers for each input image  */
    long *bands_per_img,		/* number of bands in each input image        */
    long *tot_inband,		/* total number of bands                      */
    long img_limit,		/* maximum image limit                        */
    long *same_wnd,		/* same window size? 1=YES, 0=NO              */
    long (*img_flag)[2],		/* flag for the window/band option            */
    char print_file[],		/* print file name                            */
    long *dtype			/* data type of the input image               */
);

void header
(
    long num_inimg,		/* number of input images                     */
    long bands_per_img[],	/* number of bands in each input image        */
    char taein[][CMLEN], 	/* input image file tae name                  */
    char taeout[]		/* output statistics file tae name            */
);

void matinv
(
    float **a,			/* matrix to be inverted in place             */
    long size,			/* number of rows and/or columns in the matrix*/
    long *ret_code		/* return code for matrix inversion           */
);

void mergs
(
    float *a, 			/* destination class statistcs to be merged   */
    float *b,			/* class containing statistics to be merged,  */
    long nbands,			/* number of input bands                      */
    long *mer_stat		/* return status                              */
);

void mergsd
(
    float a[], 			/* destination class statistics which will    */
    float b[],			/* class containig statistics to be merged -  */
    long bands 			/* number of input image bands                */
);

void pageout
(
    long ktf, 			/* total number of fields processed by search */
    long nflds, 			/* number of fields which produced the set of */
    long num_inimg,		/* number of input images                     */
    long class,			/* class index for this class                 */
    float smns[], 		/* mean vector for this class                 */
    float sd[],			/* standard deviation for this class          */
    float zmv[], 			/* coefficient of variation for this class    */
    float zcrl[],			/* correlation matrix for this class          */
    long data_type		/* input image data type                      */
);

void process
(
    char taein[][CMLEN],			/* input image tae name               */
    char taeout[],			/* output stat file tae name          */
    long data_type,			/* image data type                    */
    float **classtat,			/* class statistics                   */
    long num_inimg,			/* number of input images             */
    long nbands,				/* total number of input image bands  */
    long num_pixels,			/* number of pixels in the input image*/
    long num_lines, 			/* number of lines in the input image */
    float pts,				/* number of pixels in one field      */
    long bands_per_img[] 		/* number of bands in each image      */
);

void sdis
(
    float *a,			/* one of class pairs for which the scaled    */
    float *b,			/* the other class for which the scaled       */
    long bands,			/* number of input image bands                */
    float *dist			/* output scaled distance                     */
);

void sdmc
(
    float **classtat, 		/* class statistics                           */
    long *init,			/* processsing control parameter              */
    float dv[],			/* pair-wise divergences for all              */
    float ptt,			/* number of pixels in one field              */
    float rfc,			/* scaling factor for separability measure    */
    long *maxptr,		/* pointer to last class accepted             */
    long nbands			/* number of input image bands                */
);

void square
(
    float **a,			/* output sqare array                         */
    long nrows,			/* number of rows in the input square matrix  */
    float *b			/* input array containing matrix in symetric  */
);

void srchio
(
    long nbands, 		/* number of input image bands                */
    long npix 			/* number of pixels                           */
);

void srchstats
(
    char hostout[],		/* output statistics file host name           */
    long nbands,			/* number of input images                     */
    float **classtat		/* final statistics produced by search        */
);

#endif

