#include "search.h"
#include "protos.h"

void sdis
(
    float *a,			/* one of class pairs for which the scaled    */
    float *b,			/* the other class for which the scaled       */
    long bands,			/* number of input image bands                */
    float *dist			/* output scaled distance                     */
)

/*
*****************************************************************************

NAME: 	SDIS

PURPOSE:  Computes the scaled distance between two classes.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-Aug-83   R. J. Tighe
   2.0         07-Jul-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Compute the scaling factor denominator.
   For each band,
      compute the scaling factor and the sum of squares
   End-for
   Compute the scaled distance as the root of sum of squares.

*****************************************************************************
*/

{

double	sud;			/* scaled distance before sqrt operation      */

float	pt1,			/* number of points for array a               */
	pt2,			/* number of points for array b               */
	ptt,			/* scaling factor denomiantor                 */
	scale,			/* scaling factor                             */
	ud;			/* temporary result for scaled distance       */

long	band,			/* current band number                        */
	i1;			/* array index                                */

sud = 0.0;  
pt1 = a[POP]; 
pt2 = b[POP]; 

/*
Compute the scaling factor denominator.
-----------------------------------------
*/
ptt = (pt1 + pt2) / coefvar;  

/*
For each band, compute the scaling factor and the sum of squares.
------------------------------------------------------------------
*/
for (band = 1; band <= bands; band++)
   {
   i1 = band + MEAN;
   scale = (pt1 * a[i1] + pt2 * b[i1]) / ptt;
   if (scale < 1.0) 
      scale = 1.0; 
   ud = (a[i1] - b[i1]) / scale;   
   sud = sud + ud * ud;                
   }

/*
Compute the scaled distance as the root of sum of squares.
----------------------------------------------------------
*/
*dist = (float)sqrt(sud); 

}     

