/* Include files, global parameters, and definitions for SEARCH      */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <statio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

#define NR      0			/* number of lines in local window    */
#define NP      1			/* number of pixels in local window   */

#define TERM	0			/* directs output to terminal         */
#define FIL	1			/* directs output to file             */
#define LP	2			/* directs ouput to line printer      */

struct	PARBLK	block;		/* TAE parameter block                */
struct  DDR	ddr;		/* ddr                                */
FILE	*fd1;			/* print file descriptor              */

struct FDESC * fdescin[MAXIMG+1]; 	/* input file descriptors             */
struct GDESC * gdescin;		/* input group descriptor             */
FILE * sdesc;			/* stats file descriptor              */
float	*inbuf;			/* image input buffer                 */
long	ipoint[MAXIMG*MAXBND+1];/* offsets into input image buffer    */

long	min_max;		/* minmax option flag                 */
long 	msgtim;			/* message interval                   */
long	nclass;			/* number of classes currently in the */
				/* statistics file                    */
long	num_outfiles;		/* number of output files             */
long	print[3];		/* print options: TERM, LP, FIL       */
char    out_files[3][CMLEN]; 	/* output file names                  */

extern	long	curent;			/* current entity                     */
extern	long	curcat;			/* current category                   */

long	window[MAXIMG][4];    /* input image dimensions (SL SS NL NS) */

long	savestat;		/* save statistics flag               */
long	block_dist;		/* pixel block size and interclass    */
				/* distance measure type              */
float	mthresh;		/* maximum desired separability for   */
				/* merger (threshold)                 */
float	stdevlb,		/* standard deviation lower bound     */
	stdevub;	/* standard deviation upper bound     */
float	coefvar;		/* coefficient of variation / 100     */

float	***work;		/* intermediate buffer which holds    */
				/* image lines after they are read    */
				/* from disk; incorporated from the   */
				/* fortran version of search          */
float	*scv;			/* covariance matrix elements for one */
				/* class                              */

long	num_class;		/* number of classes found            */

long	ICOUNT;			/* ICOUNT = tot_inband *              */
				/*          (tot_inband+1) / 2        */
long	INVERSE;		/* INVERSE = ICOUNT		      */
long	MEAN;			/* MEAN = ICOUNT*2                    */
long	POP;			/* POP = MEAN + tot_inband + 1        */

int main
(void) /* search */
{

/*
*****************************************************************************

NAME: SEARCH

PURPOSE:  Performs unsupervised training site selection and creates a 
          statistics file for use in a subsequent classification. An input
          image is processed as a collection of blocks. A block is accepted
          as a training site only if the standard deviations in all bands 
          fall within specified bounds.
          The user has options to produce printed  statistical summary and 
          to create an output statistics file.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         29-AUG-83   J. R. Tighe      
   2.0         25-Mar-85   Y. Sheu		added error msg if processing
						completes with no classes found.
   3.0         05-Jul-89   K. Kunska    	conversion to 5.0
   4.0         09-Sep-89   K. Kunska		fixed processing msg format and
   						output file names
   4.1	       27-Nov-90   D. Akkerman		minor error message fix;
						added fatal error message to
						PDF/User's Guide
   5.1         23-Sep-92   D. Etrheim		Modified global include file
   5.1         04-Aug-93   D. Bridges           Added TERM to print parameter
   
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Call GETPAR to get TAE parameters.
   Call FILSPEC to open the input image file.
   Initialize variables and array dimension constants.
   Allocate the dynamic memory for the class statistics buffer and work buffer.
   Initialize the class statistics and work buffers to 0s.
   Call PROCESS to perform the unsupervised training site selection .
   Close the input image files.
   If statistics are to be saved, 
      call SRCHSTATS to create a stats file and write the statistics.
   Free the dynamic memory.
   Write the standard completion message.

*****************************************************************************
*/

static char	
	hostin[MAXIMG][CMLEN], 		/* input image host name              */
	hostout[CMLEN],			/* output file host name              */
	print_file[CMLEN],		/* print file name                    */
	taein[MAXIMG][CMLEN], 		/* input image tae name               */
	taeout[CMLEN];			/* output file tae name               */

float	**classtat,			/* buffer for the class statistics    */
	pts;				/* number of pixels in each field     */

static long	bands_per_img[MAXIMG],	/* number of bands in each image      */
	data_type,			/* image data type                    */
	dtype,				/* data type for the work buffers     */
	dyn_cols,			/* number of columns in a matrix      */
	dyn_rows,			/* number of rows in a matrix         */
	i,				/* loop counter                       */
	img_flag[MAXIMG][2],		/* flag for window/band option        */
	num_img_bands[MAXIMG][MAXBND+1],/* band numbers for each input image  */
	num_inimg, 			/* number of input images             */
	num_lines, 			/* number of lines in the input image */
	num_pixels,			/* number of pixels in the input image*/
	same_wnd,			/* same window size? 1=YES, 0=NO      */
	tot_inband,			/* total number of input image bands  */
	x, y, z,			/* loop counters and indexes          */
	xdim,				/* x dimension in a 3D work buffer    */
	ydim,				/* y dimension in a 3D work buffer    */
	zdim;				/* z dimension in a 3D work buffer    */

long    zero = 0;

/*
Get TAE parameters.
-------------------
*/
getpar (taein, hostin, taeout, hostout, &num_inimg, num_img_bands,
        bands_per_img, &tot_inband, MAXIMG, &same_wnd, img_flag,
        print_file, &data_type);

/*
Open the input image file.
---------------------------
*/
filspec (hostin, num_inimg, num_img_bands, bands_per_img);

/*
Initialize variables and array dimension constants.
----------------------------------------------------
*/
ICOUNT = tot_inband * (tot_inband + 1) / 2;
INVERSE = ICOUNT;
MEAN = ICOUNT * 2;
POP = MEAN + tot_inband + 1;

if (block_dist == 2) 
   block_dist = -1;                    
pts = 36.0;                                
if (block_dist == 1) 
   pts = 9.0;                    
if (coefvar < 0.0001) 
   coefvar = 0.0001;      

num_lines = window[0][NL];
num_pixels = window[0][NS];

/* 
Allocate the dynamic memory for the class statistics buffer and work buffer.
Initialize the class statistics and work buffers to 0s.
---------------------------------------------------------------------------
*/
dyn_rows = 51;
dyn_cols = POP + 1;
dtype = EREAL;
if ((classtat = fmat2d(dyn_rows, dyn_cols)) == NULL)
   c_abort (&block, "Error allocating dynamic memory for work buffer",
            "search-malloc", "search", out_files, &num_outfiles);
for (i = 0; i < 51; i++)
   c_pxfill (classtat[i], &dtype, &dyn_cols, &zero); 

xdim = tot_inband + 1;
ydim = 7;
zdim = 7006;
if ((work = fmat3d (xdim, ydim, zdim)) == NULL)
   c_abort (&block, "Error allocating dynamic memory for work buffer",
            "search-malloc", "search", out_files, &num_outfiles);
for (x = 0; x < xdim; x++)
   for (y = 0; y < ydim; y++)
      for (z = 0; z < zdim; z++)
         work[x][y][z] = 0.0;

/*
Call PROCESS to perform the unsupervised training site selection .
------------------------------------------------------------------
*/
process (taein, taeout, data_type, classtat, num_inimg, tot_inband, 
         num_pixels, num_lines, pts, bands_per_img);

/*
Close the input image files.
----------------------------
*/
c_egclse (&gdescin);

/*
If statistics are to be saved, call SRCHSTATS to create a stats file 
and write the statistics.
------------------------------------------------------------------------
*/
if (savestat) 
   srchstats (hostout, tot_inband, classtat);

/* 
Free the dynamic memory.
--------------------------
*/
freefmat2d (classtat, dyn_rows);
freefmat3d (work, xdim, ydim);

/*
Write the standard completion message.
---------------------------------------
*/
c_complt ("SEARCH", out_files, &num_outfiles); 

return EXIT_SUCCESS;

}   
