/*****************************************************************************
NAME:   		       OPENFILE

PURPOSE:   Open all of the input images for random reading and writing.
     

PROGRAM HISTORY:
VERSION	 PROGRAMMER   DATE   CODE/CONT  REASON
=======  ==========   =====  =========  =========================================
  5.1    J Fenno      09/91     EDC     Reduction of duplicate code
  6.1	 D Eichacker  05/94     EDC     Added opt[]

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   must be run as part of SEGMREPR.

ALGORITHM:

ALGORITHM REFERENCES:

PROJECT               LAS
*******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "imageio.h"
#include "segmrepr.h"
#include "util.h"
#include "protos.h"

/* Global variables for image i/o routines
==========================================*/
extern int  fdesc[];         /* image file descriptors used for image i/o     */

void openfile
(
    struct PARBLK *vblock,  /* I: tae parameter block                        */
    char hostname[][CMLEN], /* I: host names of images                       */
    char *minmax,           /* I: min/max flag                               */
    double *maxval,         /* O: maximum pixel value array                  */
    double *minval,         /* O: minimum pixel value array                  */
    long nbands[MAXIMG],    /* I: number of bands per image                  */
    long nbrimg,            /* I: number of images                           */
    long odtype,            /* I: data type of images                        */
    long totbnd,            /* I: total number of bands to be processed      */
    long update,            /* I: flag indicating if input images are modified*/
    long window[][NUM_SPECS]/* I: window specification of images             */
)

{
long   access;               /* image i/o access type                         */
long   fatal = -1;           /* fatal error flag                              */
long   image;                /* loop control                                  */
long   nl;                   /* number of lines in all images                 */
long   ns;                   /* number of samples in all images               */
long   one = 1;              /* value of one                                  */
long   sl;                   /* starting line the image                       */
long   ss;                   /* starting sample the image                     */
long   status;               /* return status code for function calls         */
static	long	opt[3] = {0,0,0};    /*  For compression option                  */

nl = window[0][NL];  /* same for all images */
ns = window[0][NS];  /* same for all images */
access  = IUPDATE;	
for (image = 0; image < nbrimg; image++)
   {
   sl = window[image][SL];
   ss = window[image][SS];
   status = c_eopenr((struct FDESC **)&fdesc[image], hostname[image], 
                     &nbands[image], &sl, &ss, &nl, &ns, &access, &odtype,
		     opt, &one);
   if (status != E_SUCC)
      {
      closefile(image - 1);
      if (update)
	 c_errmsg("Fatal error encountered","segmrepr-fatal",&fatal);
      else
	 c_clean(vblock,"segmrepr",(char (*)[256])hostname[0],&one);
      }
   }
fdesc[nbrimg] = 0;

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype, &totbnd, minval, maxval);

return;
}
