/***************************************************************************
NAME      		  SEGMREPR 

PURPOSE:  To provide a means of inserting rectangular groups of
	  pixels into a region of an image.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
E.HARRIS	JAN  1982	Original development
N.DEODHAR	SEPT 1987	Fixed windowing problem
R.MCKINNEY      SEPT 1988       Conversion to LAS 5.0
J.FENNO         SEPT 1991       Ran through Saber-C.
                                Modified to allow i*4 and r*4 data types.
				Added ability to update the input images versus
				creating a new output image.
J.FENNO         Mar  1992       Modified so that OUT is a required parameter.
				Changed so that combinations of images are not
				allowed when in update mode.
				PRF Z0744
D.HAMES		NOV  1999	Remove 50000 limits on number of lines and 
				samples from INWIND(s) in segmrepr.pdf.
				Add prototypes. SRF #893
D.HAMES         JAN  2000       Add an error check to getpar to ensure INWIND(s)
				parameters are positive values. Remove an
				unnecessary setlib in segmrepr.pdf. 


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   SEGMREPR must be run under TAE.

PROJECT   	         LAS		                

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "segmrepr.h"           /* to retrieve segmrepr constants             */
#include "util.h"
#include "protos.h"

/* The following variables must be declared global for image I/O routines
------------------------------------------------------------------------------*/

int  fdesc[MAXIMG+1];           /* image file descriptors used for image I/O  */
extern long curent;		/* current processing line		      */
extern long curcat;		/* current processing band		      */

void main
(void)
{
struct PARBLK	vblock;              /* TAE parameter block                   */

static long bands[MAXIMG][MAXBND + 1];/* band specification for images        */

double maxval[MAXBND];               /* maximum pixel value array             */
double minval[MAXBND];               /* minimum pixel value array             */

long   msgtim;			     /* elpased time between display messages */
long   nbrimg;			     /* number of input images                */
long   nbrwind;                      /* number of repair windows specified    */
long   inwind[MAX_INWINDS][NUM_SPECS]; /* repair windows specified            */
long   nbands[MAXIMG];		     /* number of bands per input image       */
long   odtype;                       /* data type of images                   */
long   size;                         /* number of bytes required for odtype   */
long   std_out;			     /* logical unit # for standard output    */
long   totbnd;			     /* total number of bands                 */
long   update;                       /* flag to update or modify input images */
long   window[MAXIMG][NUM_SPECS];    /* window specification for first image  */

char   hostname[MAXIMG][CMLEN];	     /* host filenames of images to repair    */
char   minmax[4];                    /* minmax flag buffer 		      */
char   option[5];                    /* option of processing		      */
char   taename[MAXIMG][CMLEN];	     /* tae filenames of images to repair     */

float  pixval[MAX_INWINDS];          /* pixel values for constant option      */

c_inlas(&vblock, &std_out);          /* initialize las and tae                */

/* get user-specified parameters and TAE globals
------------------------------------------------*/
getpar(&vblock, hostname, minmax,  option,  taename,  pixval,
       bands,   inwind,   &msgtim, &nbrimg, &nbrwind, nbands,
       &odtype, &size,    &totbnd, &update, window);

/* open all of the images for updating
--------------------------------------*/
openfile(&vblock, hostname, minmax, maxval, minval, nbands,
	 nbrimg,  odtype,   totbnd, update, window);

/* perform the rectangular correction
-------------------------------------*/
if (strcmp(option,"AVER") == 0)
   average(&vblock, hostname, minmax, bands, inwind, msgtim, nbands,
           nbrwind, nbrimg,   odtype, size,  totbnd, update, window); 
else if (strcmp(option,"CONS") == 0)
   constant(&vblock, hostname, minmax, pixval, bands, inwind, msgtim,
	    nbands,  nbrwind,  nbrimg, odtype, size,  totbnd, update,
	    window);
else if (strcmp(option,"HORI") == 0)
   horizontal(&vblock, hostname, minmax, bands, inwind, msgtim, nbands,
	      nbrwind, nbrimg,   odtype, size,  totbnd, update, window); 
else if (strcmp(option,"VERT") == 0)
   vertical(&vblock, hostname, minmax, bands, inwind, msgtim, nbands,
	    nbrwind, nbrimg,   odtype, size,  totbnd, update, window);

/* inform user of successful completion
---------------------------------------*/
c_complt("SEGMREPR", taename, &nbrimg);
}
