/*******************************************************************************
NAME:		GETKERN	

PURPOSE:    	GETKERN returns a 3x3 kernel for a given pixel location 
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	12/90  	Original development	
5.1	S.Nelson	 5/92	Illiminated warnings upon compilation.
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation

		        Note: this function works in conjunction with
			function stepline, which updates the line pointers 
		  	as the program steps through the images.
******************************************************************************/
#include "las.h"
#include "protos.h"

/* The following variables have been declared global 
----------------------------------------------------*/
extern long nline;			/* number of lines to be processed */
extern long ns;			/* number of samples to be processed */
extern long *curlin;		/* current line of input image */
extern long *prvlin;		/* previous line of input image */
extern long *nextln;		/* next line of input image */


/* Function to extract and return 3 X 3 kernel for specified pixel 
------------------------------------------------------------------*/
void getkern
(
    long ln,		/* line that pixel of interest is on */
    long smp,		/* sample of pixel of interest */
    long kern[9]	/* 3x3 kernel indexed as follows: 0   1   2
				        		  3   4   5
  					          	  6   7   8  */
) 

{
/* Center of kernel */ 
kern[4] = curlin[smp];

if (ln == 1)
   /* Top of image */
   {
   kern[0] = kern[1] = kern[2] = 0;
   kern[7] = nextln[smp];
   if (smp == 0)
      /* Left edge of image */ 
      {
      kern[3] = kern[6] = 0;
      kern[5] = curlin[smp + 1];
      kern[8] = nextln[smp + 1];
      }
   else if (smp == ns-1)
      /* Right edge of image */
      {
      kern[5] = kern[8] = 0;
      kern[3] = curlin[smp - 1];
      kern[6] = nextln[smp - 1];
      }
   else 
      /* Neither edge of image */
      {
      kern[3] = curlin[smp - 1];
      kern[5] = curlin[smp + 1];
      kern[6] = nextln[smp - 1];
      kern[8] = nextln[smp + 1];
      }
   }
else if (ln == nline)
   /* Bottom of image */
   {
   kern[6] = kern[7] = kern[8] = 0;
   kern[1] = prvlin[smp];
   if (smp == 0)
      /* Left edge of image */ 
      {
      kern[0] = kern[3] = 0;
      kern[2] = prvlin[smp + 1];
      kern[5] = curlin[smp + 1];
      }
   else if (smp == ns-1)
      /* Right edge of image */
      {
      kern[2] = kern[5] = 0;
      kern[0] = prvlin[smp - 1];
      kern[3] = curlin[smp - 1];
      }
   else 
      /* Neither edge of image */
      {
      kern[0] = prvlin[smp - 1];
      kern[2] = prvlin[smp + 1];
      kern[3] = curlin[smp - 1];
      kern[5] = curlin[smp + 1];
      }
   }
else
   /* Neither top nor bottom of image */
   {
   kern[1] = prvlin[smp];
   kern[7] = nextln[smp];
   if (smp == 0)
      /* Left edge of image */ 
      {
      kern[0] = kern[3] = kern[6] = 0;
      kern[2] = prvlin[smp + 1];
      kern[5] = curlin[smp + 1];
      kern[8] = nextln[smp + 1];
      }
   else if (smp == ns-1)
      /* Right edge of image */ 
      {
      kern[2] = kern[5] = kern[8] = 0;
      kern[0] = prvlin[smp - 1];
      kern[3] = curlin[smp - 1];
      kern[6] = nextln[smp - 1];
      }
   else 
      {
      /* Neither edge of image */
      kern[0] = prvlin[smp - 1];
      kern[2] = prvlin[smp + 1];
      kern[3] = curlin[smp - 1];
      kern[5] = curlin[smp + 1];
      kern[6] = nextln[smp - 1];
      kern[8] = nextln[smp + 1];
      }
   }
}
