/*******************************************************************************
NAME:		GETPAR	

PURPOSE:    	GETPAR gets parameters from tae parameter block	
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	9/90	Re-retrocode (All new C version!)
5.0 	S.Nelson	5/92	Changed parameter name to INLT.  Updated
				error messages.  
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
******************************************************************************/

#include "las.h"
#include "util.h"
#include "protos.h"


void getpar
(
    struct PARBLK *vblock,		/* pointer to TAE parameter block */
    char hosin[][CMLEN],         /* host file names of input images */
    char taeout[][CMLEN],		/* TAE file name of output image */
    char hosout[][CMLEN],		/* host file name of output image */
    char *wgtlt,			/* class pair weight labeled table */
    long *nmrimg,		/* number of input images */
    long window[][4],		/* window specifications for each input image*/
    long bands[][MAXBND+1],		/* band specifications for each input image */
    long *nbands,		/* number of bands per input image */
    long *totbnd,		/* total number of bands to be processed */
    long *d_wgt,			/* default weight value */
    long *fourway,		/* flag indicating whether to enforce strict
				   four-way connectedness criteria */
    long *msgtim,		/* elapsed time between display messages */
    char *minmax		/* min max flag buffer */
)
{
struct VARIABLE	*val;		/* pointer to a TAE variable within v-block */

long	idtype;			/* input image data type */
long	access = IREAD;		/* access flag */
long	element = 0;		/* parameter element number */
long    fatal = -1;		/* fatal error flag */
long	imglim = 1;		/* maximum number of images to be processed */
long    one = 1;		/* value of one	*/
long	qflag[1][2];		/* flag indicating if subwindows or bands 
							were specified */
long	same;			/* flag indicating if images have same window*/

char	connect[6];		/* connectedness parameter */
char 	msg[ERRLEN+1];		/* message buffer */	
char	taein[1][CMLEN];		/* tae name for input image */

/* Retrieve and parse input image specification 
-----------------------------------------------*/
if (c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,nbands,
		totbnd,&imglim,&same,qflag) != E_SUCC)
   c_errmsg("Fatal error encountered","smooth-fatal",&fatal);

/* Check for invalid number of bands 
------------------------------------*/
if (*totbnd != 1)
   c_errmsg("Only single band input allowed","smooth-nbands",&fatal);

/* Check for valid input data type
----------------------------------*/
if (c_gettyp(hosin,&one,&idtype) != E_SUCC)
   c_errmsg("Fatal error encountered","smooth-fatal",&fatal);

if (idtype != EWORD && idtype != ELONG)
   {
   sprintf(msg,"Image %s not of type integer*2 or integer*4", taein[0]);
   c_errmsg(msg,"smooth-dtype",&fatal);
   }

/* Retrieve output image name to make sure that the file does not already exist
   This may look foolish since the program never acutally uses hosout.  But 
   there is method to this madness.  It is done here so that the software 
   doesn't run all the way through, only to find out at the end that the output
   image already exists when copy is run to convert data types. 
------------------------------------------------------------------------------*/
if (c_getout(vblock,"OUT",taeout,hosout) != E_SUCC)
   c_errmsg("Fatal error encountered","smooth-fatal",&fatal);

/* Retrieve weight value labeled table file name
------------------------------------------------*/
if (c_getnam(vblock,"INLT",&element,wgtlt,"lt",&access) != E_SUCC)
   c_errmsg("Fatal error encountered","smooth-fatal",&fatal);

/* Retrieve default threshold value
-----------------------------------*/
val = p_find(vblock,"DFLTWGHT");
*d_wgt = IVAL(*val,0);

/* Retrieve connectedness flag
------------------------------*/
val = p_find(vblock,"CONNECT");
strcpy(connect,SVAL(*val,0));
if (strcmp(connect,"FOUR") == 0)
   *fourway = TRUE;
else
   *fourway = FALSE;

/* Retrieve globals
-------------------*/
val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

return;
}
