#ifndef PROTOS_H
#define PROTOS_H

struct pairweight       /* pairwise weight table structure */
   {
   long fclass;         /* flagged class value */
   long nclass;         /* neighbor class value */
   long weight;         /* weight value */
   };


void getkern
(
    long ln,		/* line that pixel of interest is on */
    long smp,		/* sample of pixel of interest */
    long kern[9]	/* 3x3 kernel indexed as follows: 0   1   2
				        		  3   4   5
  					          	  6   7   8  */
);

void getpar
(
    struct PARBLK *vblock,		/* pointer to TAE parameter block */
    char hosin[][CMLEN],         /* host file names of input images */
    char taeout[][CMLEN],		/* TAE file name of output image */
    char hosout[][CMLEN],		/* host file name of output image */
    char *wgtlt,			/* class pair weight labeled table */
    long *nmrimg,		/* number of input images */
    long window[][4],		/* window specifications for each input image*/
    long bands[][MAXBND+1],		/* band specifications for each input image */
    long *nbands,		/* number of bands per input image */
    long *totbnd,		/* total number of bands to be processed */
    long *d_wgt,			/* default weight value */
    long *fourway,		/* flag indicating whether to enforce strict
				   four-way connectedness criteria */
    long *msgtim,		/* elapsed time between display messages */
    char *minmax		/* min max flag buffer */
);

void randpick
(
    long *limit		/* number of values to choose from */
);

int smoothit
(
    long fourway,	/* flag indicating whether to enforce strict four-way
			   connectedness criteria for reclamation */
    long *changed,	/* flag indicating when pixel values are changed */
    long *neg_pres,	/* flag indicating presence of negative pixels */
    long *minv,		/* min valued pixel */
    long *maxv		/* max valued pixel */
);

int stepline
(
    long ln
);

void readwgts
(
    char *pairwgtlt,		/* pairwise weight values labeled table */
    long dflt			/* user specified default weight */
);

long getwgt
(	/* Returns weight for pair of classes in key */
    struct pairweight *key		/* Key for searching table */
);

int s_compar
(		/* compare routine used by qsort */
    struct pairweight *a,
    struct pairweight *b		/* pairwise weight entry pointers */
);

int compar
(		/* compare routine used by bsearch */
    struct pairweight *a,
    struct pairweight *b		/* pairwise weight entry pointers */
);

#endif
