/*******************************************************************************

NAME:		SMOOTH	

PURPOSE:   	SMOOTH performs area filtering on flagged images

			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
	P.Pease		09/87	Made mods to fortran version
5.0	D.VanderZee	12/90	Re-retrocode (All new C version!)
5.1	S.Nelson	 5/92	Updated user guide and error messages.  Changed
				parameter names to standard. Illiminated 
				warnings upon compilation.
5.2	D.Etrheim	 8/92	Write module name to history file
5.3     D.Etrheim	12/92	Corrected problem where active enteries were
				left in the DISKRSVD file (c_dkfree).  
				
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
SMOOTH must be run under TAE.

PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"	
#include "typlim.h"
#include "util.h"
#include "protos.h"

/* The following variables have been declared global for various routines
-------------------------------------------------------------------------*/
struct FDESC * 	fdesc_in[2];	/* image file descriptors used for image I/O */
struct FDESC * 	fdesc_out[2];	/* image file descriptors used for image I/O */
struct GDESC * 	gdesc_out;	/* group descriptor used for image I/O */
long	out_ptr[1];   	/* pointer into "outbuf" for output data */
long 	nline;             /* number of lines in image to process */
long 	ns;             /* number of samples in image to process */
long	sl;			/* start line for output image */
long 	ss;			/* start sample for output image */
extern long 	curent;		/* current processing line */
long 	*curlin;	/* pointer to current line being processed */
long 	*prvlin;	/* pointer to previous line */
long 	*nextln;	/* pointer to next line */
long 	*outlin;		/* pointer to output line */
unsigned char   *outbuf;        /* buffer for output image */
unsigned char *in_lines[3];


int main
(void)
{
struct PARBLK vblock;		/* TAE parameter block */

long	i;			/* loop counter */
long 	bands[1][MAXBND+1];		/* bands to be processed */
long	msgtim;			/* elapsed time between display msg */
long	nbands;			/* number of bands per input image */
long	nmrimg;			/* number of input images */
long	std_out;		/* LUN to standard output */
long	totbnd;			/* total number of bands */
long	window[1][4];		/* window for each input image */
long	msg_frmt = 3;		/* format flag for processing message*/
long	zero = 0;		/* number zero for passing by address */
long	one = 1;		/* number two for passing by address */
long	rd_acc = IREAD;		/* read access flag */
long	wr_acc = IWRITE;	/* write access flag */
long	cflag = NOCOMB;		/* flag for updating ddr */
long	dtype = ELONG;		/* long data type */
int  	odtype;			/* output data type */
static	long	open_opt[3]={0,0,0};	/* optional open parameter */
long	fatal = -1;		/* fatal error flag */
long	nonfatal = 1;		/* nonfatal error flag */
long	bufsiz;			/* image i/o buffer size */
long	nfile = 1;		/* number of output files */
long	d_wgt;			/* default weight value */
long	fourway;		/* flag indicating whether to enforce strict
				   four-way connectedness criteria */
long	numimgs = 0;		/* number of open images for cleanup */
long	changed;		/* flag indicating pixels were changed */
long	neg_pres;		/* flag indicating negative pixels remain */
long	minv;			/* minimum pixel value */
long	maxv;			/* maximum pixel value */

char	hosin[1][CMLEN];	/* host names of input images */
char	hosout[1][CMLEN];		/* host name of output image */
char	taeout[1][CMLEN];		/* TAE name of output image */
char	temp_1[20];		/* temporary image name */
char	temp_2[20];		/* temporary image name */
char	temp_out[20]; 		/* temporary image name to pass back to pdf */
char    *ptr[1];		/* pointer to temp_out for passing to pdf */
char	temp_cmplt[20];		/* temp image name for completing processes */ 
char	openimgs[2][CMLEN];	/* list of open images for cleanup */
char	wgtlt[CMLEN];		/* pairwise weight labeled table name */
char	minmax[4];		/* min max flag */

double	minval;			/* min values for output image */
double 	maxval;			/* max values for output image */
double  ln_inc = 1;		/* line increment for ddr */
double  smp_inc = 1;		/* sample increment for ddr */
/*****************************************************************************/

/* Initialization
-----------------*/
c_inlas(&vblock,&std_out);

/* Get parameters and globals
-----------------------------*/
getpar(&vblock,hosin,taeout,hosout,wgtlt,&nmrimg,
	window,bands,&nbands,&totbnd,&d_wgt,&fourway,
	&msgtim,minmax);

/* Read pairwise weight table in preparation for processing 
-----------------------------------------------------------*/
readwgts(wgtlt,d_wgt);

/* Initialize processing message utility 
----------------------------------------*/
c_inittm("SMOOTH","Processed","line"," "," ",&window[0][2],&zero,&msgtim,
	&msg_frmt);

/* Start processing messages
----------------------------*/
curent = 1;
c_startm();

/* Create temporary image names
-------------------------------*/
strcpy (temp_1,"smooth_1");
c_creat_name(temp_1,"img");
strcpy (temp_2,"smooth_2");
c_creat_name(temp_2,"img");

fdesc_in[1]=0;
fdesc_out[1]=0;
open_opt[0] = 0;

nline = window[0][2];
ns = window[0][3];

/* Loop until done processing image--alternating between two temp images
------------------------------------------------------------------------*/ 
for (i = 0; ;i++)
   {
   if (i == 0)
      /* First time through--read from input image and write to first temp img
      ------------------------------------------------------------------------*/
      {
      sl = window[0][0];
      ss = window[0][1];

      /* Open input image
      -------------------*/
      if (c_eopenr(&fdesc_in[0],hosin[0],&nbands,&sl,&ss,
	   &nline,&ns,&rd_acc,&dtype,open_opt,&one) != E_SUCC)
         c_errmsg("Fatal error encountered","smooth-fatal",&fatal);

      /* Open output image
      --------------------*/
      if (c_eopens(&fdesc_out[0],temp_1,&nbands,&one,&one,&nline,&ns,
		out_ptr,&wr_acc,&dtype,&zero,open_opt) != E_SUCC)
         c_errmsg("Fatal error encountered","smooth-fatal",&fatal);
      strcpy(openimgs[0],temp_1);
      numimgs = 1;
      }
   else 
      {
      /* Not first time through the image 
      -----------------------------------*/
      sl = ss = 1;

      if ((i % 2) == 1)
         /* Odd numbered pass--read from first temp img and write to second 
         ------------------------------------------------------------------*/
         {
         /* Open input image
         -------------------*/
         if (c_eopenr(&fdesc_in[0],temp_1,&nbands,&sl,&ss,&nline,&ns,
				&rd_acc,&dtype,open_opt,&one) != E_SUCC)
	    c_clean(&vblock,"smooth",openimgs,&numimgs);

         /* Open output image
         --------------------*/
	 if (i > 1) c_delfil(temp_2); 
         if (c_eopens(&fdesc_out[0],temp_2,&nbands,&sl,&ss,&nline,&ns,
		out_ptr,&wr_acc,&dtype,&zero,open_opt) != E_SUCC)
	    c_clean(&vblock,"smooth",openimgs,&numimgs);
         strcpy(openimgs[1],temp_2);
         numimgs = 2;
         }
      else
         /* Even numbered pass--read from second temp img and write to first 
         -------------------------------------------------------------------*/
	 {
         /* Open input image
         -------------------*/
         if (c_eopenr(&fdesc_in[0],temp_2,&nbands,&sl,&ss,&nline,&ns,
		&rd_acc,&dtype,open_opt,&one) != E_SUCC)
	    c_clean(&vblock,"smooth",openimgs,&numimgs);

         /* Open output image
         --------------------*/
	 c_delfil(temp_1); 
         if (c_eopens(&fdesc_out[0],temp_1,&nbands,&sl,&ss,&nline,&ns,
		out_ptr,&wr_acc,&dtype,&zero,open_opt) != E_SUCC)
	    c_clean(&vblock,"smooth",openimgs,&numimgs);
	 }
     }
 
   /* Allocate memory for input image 
   ----------------------------------*/
   if ((in_lines[0] = (unsigned char *) malloc(ns*sizeof(long))) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","smooth-alloc",&nonfatal);
      c_clean(&vblock,"smooth",openimgs,&numimgs);
      }
   if ((in_lines[1] = (unsigned char *) malloc(ns*sizeof(long))) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","smooth-alloc",&nonfatal);
      c_clean(&vblock,"smooth",openimgs,&numimgs);
      }
   if ((in_lines[2] = (unsigned char *) malloc(ns*sizeof(long))) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","smooth-alloc",&nonfatal);
      c_clean(&vblock,"smooth",openimgs,&numimgs);
      }

   /* Allocate memory for output image 
   -----------------------------------*/
   if ((outbuf = (unsigned char *)c_ealloc(fdesc_out,&bufsiz)) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","smooth-alloc",&nonfatal);
      c_clean(&vblock,"smooth",openimgs,&numimgs);
      }
   if (c_egroup(fdesc_out,&gdesc_out,outbuf,&bufsiz) != E_SUCC)
      c_clean(&vblock,"smooth",openimgs,&numimgs);


   /* Process the image reclaiming flagged areas 
   ---------------------------------------------*/
   minv = MAXLONG;
   maxv = MINLONG;
   if (smoothit(fourway,&changed,&neg_pres,&minv,&maxv) == E_FAIL)
      c_clean(&vblock,"smooth",openimgs,&numimgs);


   /* Close input and output images
   --------------------------------*/
   if (c_eclose(&fdesc_in[0]) != E_SUCC)
      c_clean(&vblock,"smooth",openimgs,&numimgs);
   if (c_egclse(&gdesc_out) != E_SUCC)
      c_clean(&vblock,"smooth",openimgs,&numimgs);

   /* Free image buffers
   ---------------------*/
   free(in_lines[0]);
   free(in_lines[1]);
   free(in_lines[2]);
   free(outbuf);

   /* Check to see if smoothing processing is finished--processing is 
      completed when either no more changes were made or there are no 
      more flagged (negative) pixels.
   ------------------------------------------------------------------*/
   if (!changed || !neg_pres)
      {
      if (neg_pres)
	 c_errmsg("Output image still contains flagged pixels",
		"smooth-flag",&nonfatal);
      break;
      }
   }


/* Determine data type for output image
---------------------------------------*/
if (maxv <= MAXUTINY && minv >= MINUTINY)
   odtype = EBYTE;
else if (maxv <= MAXWORD && minv >= MINWORD)
   odtype = EWORD;
else 
   odtype = ELONG;

/* Determine which temp file we ended on
----------------------------------------*/
if (i == 0)
   {
   strcpy(temp_cmplt,temp_1);
   strcpy(temp_out,temp_1);
   }
else 
   if ((i % 2) == 1)
      {
      strcpy(temp_cmplt,temp_2);
      strcpy(temp_out,temp_2);
      c_delfil(temp_1); 
      }
   else
      {
      strcpy(temp_cmplt,temp_1);
      strcpy(temp_out,temp_1);
      c_delfil(temp_2);
      }

/* Truncate the extension ".img" from the output name so COPY and CMDEL work
----------------------------------------------------------------------------*/
temp_out[strlen(temp_out)-4] = '\0';

/* Pass back output file name and data type to pdf for data conversion
----------------------------------------------------------------------*/
if (q_intg(&vblock,"ODTYPE",1,&odtype,P_UPDATE) != SUCCESS)
  {
  c_errmsg("Could not write ODTYPE to PARBLK","smooth-odtype",&nonfatal);
  c_errmsg("Fatal error encountered","smooth-fatal",&fatal);
  }

ptr[0] = temp_out;
if (q_string(&vblock,"TEMPOUT",1,ptr,P_UPDATE) != SUCCESS)
   {
   c_errmsg("Could not write TEMPOUT to PARBLK","smooth-tempout", &nonfatal);
   c_errmsg("Fatal error encountered","smooth-fatal",&fatal);
   }

if (q_out(&vblock) != SUCCESS)
   {
   c_errmsg("Could not pass PARBLK back to pdf","smooth-parblk", &nonfatal);
   c_errmsg("Fatal error encountered","smooth-fatal",&fatal);
   }


/* Finish up processing with housekeeping chores 
------------------------------------------------*/
/* Update DDR for output image
------------------------------*/
if (c_upddr(&vblock,hosin,temp_cmplt,&nmrimg,window,bands,&nbands,
	&ln_inc,&smp_inc,&totbnd,&cflag) != E_SUCC)
   c_errmsg("Fatal error encountered","smooth-fatal",&fatal);

/* If minmax to be tracked, then store them
-------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   {
   minval = minv;
   maxval = maxv; 
   if (c_svmnmx(temp_cmplt,&totbnd,&minval,&maxval,minmax) != E_SUCC)
      c_errmsg("Fatal error encountered","smooth-fatal",&fatal);
   }
 
/* Update history file 
----------------------*/
c_hstout(&vblock,temp_cmplt,hosin,&nmrimg,"SMOOTH");

/* Stop processing messages
---------------------------*/
c_stoptm();

/* Output completion message 
----------------------------*/
c_complt("SMOOTH",temp_cmplt,&nfile);

return EXIT_SUCCESS;

}
