/*******************************************************************************
NAME:		STEPLINE	

PURPOSE:    	STEPLINE steps through both input and output images
		         and updates buffers for processing with
			 getkern.
			 
PROGRAM HISTORY:
Version	Programmer	Date	Reason
-------	----------      ----	------
5.0	D.VanderZee	12/90 	New development	
5.1	S.Nelson	 5/92	Illiminated warnings upon compilation.
				
PROJECT:	LAS
		
ALGORITHM DESCRIPTION: 	See inline documentation
			
			Note: This function works in conjunction with
			function getkern, which retrieves a 3x3 kernel
			for specified pixel.
******************************************************************************/

#include "las.h"
#include "protos.h"

/* The following variables have been declared global for image I/O 
------------------------------------------------------------------*/
extern struct FDESC * fdesc_in[2];/*image file descriptors used for image I/O */
extern struct GDESC * gdesc_out;  /* group descriptor used for image I/O */
extern long out_ptr[1];   	/* pointer into "outbuf" for output bands */
extern long sl;
extern long ss;
extern long nline;			/* number of lines to process */
extern long ns;			/* number of samples to process */
extern unsigned char *in_lines[3];
extern long *curlin;		/* pointer to current line of input image */
extern long *prvlin;		/* pointer to previous line of input image */
extern long *nextln;		/* pointer to next line of input image */
extern long *outlin;		/* pointer to output line of output image */
extern unsigned char	*outbuf;	/* buffer for output image */

/* Function to read line from input image and write line to ouput image
-----------------------------------------------------------------------*/
int stepline
(
    long ln
)

{
static long	prv;
static long	cur;
static long	nxt;

long	one = 1;
long	ln_num;
long	nonfatal = 1;


if (ln == 1)
   /* First line to be processed 
   -----------------------------*/
   {
   prv = 0;
   cur = 1;
   nxt = 2;
  
   /* Read first line 
   ------------------*/
   ln_num = ln + sl - 1;
   if (c_eread(&fdesc_in[0],&one,&ln_num,in_lines[cur],&one) != ns)
      {
      c_errmsg ("Error reading image data","smooth-read",&nonfatal);
      return (E_FAIL); 
      }


   curlin = (long *) in_lines[cur];

   /* Read second line 
   -------------------*/
   ln_num += 1;
   if (c_eread(&fdesc_in[0],&one,&ln_num,in_lines[nxt],&one) != ns)
      {
      c_errmsg ("Error reading image data","smooth-read",&nonfatal);
      return (E_FAIL); 
      }

   nextln = (long *) in_lines[nxt];

   }
else if (ln == nline)
   {
   /* Last line to be processed
   ----------------------------*/
   prv = ((prv + 1) % 3);
   cur = ((cur + 1) % 3);
   
   prvlin = (long *) in_lines[prv];
   curlin = (long *) in_lines[cur];

   }
else if (ln > 1 && ln < nline) 
   {
   /* Any line other than first or last 
   ------------------------------------*/
   prv = ((prv + 1) % 3);
   cur = ((cur + 1) % 3);
   nxt = ((nxt + 1) % 3);

   prvlin = (long *) in_lines[prv];
   curlin = (long *) in_lines[cur];
    
   /* Read the next line 
   ---------------------*/
   ln_num = ln + sl;
   if (c_eread(&fdesc_in[0],&one,&ln_num,in_lines[nxt],&one) != ns) 
      {
      c_errmsg ("Error reading image data","smooth-read",&nonfatal);
      return (E_FAIL); 
      }


   nextln = (long *) in_lines[nxt]; 
   }

/* Also step ouput line
-----------------------*/
if (c_estep(&gdesc_out) != E_SUCC)
   return (E_FAIL); 
outlin = (long *)&outbuf[out_ptr[0]*4];

return (E_SUCC);
}


