
#include "speccomb.h"
#include "protos.h"

void filspec
(
    char hos_patin[][CMLEN],   /* pattern image host name           */
    char hos_in[][CMLEN],      /* input image host name descriptors */
    char hos_out[][CMLEN],     /* output image host name descriptor */
    long *num_images,          /* number of Level II images         */
    long *num_lines_pat,       /* number of lines in pattern image  */
    long *num_samp,            /* number of samples per line        */
    long num_lines[],          /* number of lines in the images     */
    long min_in_gl[],          /* max grey level in pattern image   */
    long max_in_gl[],          /* max grey level in Level II images */
    long *min_pat_gl,          /* min grey level in pattern image   */
    long *max_pat_gl          /* min grey level in Level II images */
)
/*
**************************************************************************

NAME: FILSPEC

PURPOSE: Set up file specifications 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           30-Aug-83  K. A. Narayanan
   2           24-Feb-84  J. Seamone 
   3           18-Aug-88  K. A. Narayanan     Conversion to 5.0
  6.1	       25-May-94  D. Eichacker		Added opt[]

ALGORITHM DESCRIPTION:

   Get maximum grey level in pattern image from DDR; if not available
      compute it
   Get maximum and minimum grey levels in Level II images from DDR; if not 
      available compute them
   Open pattern, Level II, and output image files and set up separate groups

*****************************************************************************
*/
{
long    one = 1;
long    zero = 0;
long    fatal_err = -1;


struct	BDDR	bddr;                /* band dependent DDR                 */

double	line_inc = 1.0,              /* line increment for ddr             */
	pmax, pmin,                  /* temp max/min                       */
	samp_inc = 1.0;              /* sample increment for ddr           */

static	long	opt[3]={0,0,0};	     /* optional parameter array for EOPENS*/

long	access,                      /* file access type                   */
	bands[1][MAXBND + 1],            /* temp array for band number         */
	bufsiz,                      /* image i/o buffer size              */
	comb_flag = 0,               /* image combination flag             */
	n, 
	stat,                        /* return status                      */
	type,                        /* file type                          */
	wind[1][4];                     /* temp window                        */

bands[0][0] = 1; bands[0][1] = 0;
wind[0][SL] = 1; wind[0][SS] = 1; wind[0][NS] = *num_samp;

/*
get pattern image min, max pixel values, using minmax if necessary
------------------------------------------------------------------
*/
c_getbdr (hos_patin[0], &bddr, &one);

if (bddr.valid ==  VALID)
   {
   *max_pat_gl =  (long) bddr.maxval;
   *min_pat_gl =  (long) bddr.minval;
   }
else
   {
   wind[0][NL] = *num_lines_pat; 
   c_minmax (hos_patin, &one, wind, bands, &one, &pmin, &pmax);
   *max_pat_gl = (long) pmax;
   *min_pat_gl = (long) pmin;
   }  
/*
get input image max and min values, using minmax if necessary
-------------------------------------------------------------
*/
for (n = 0; n < *num_images; n++)
   {
   c_getbdr (hos_in[n], &bddr, &one);
   if (bddr.valid == VALID)
      {
      min_in_gl[n] = (long) bddr.minval;
      max_in_gl[n] = (long) bddr.maxval;
      }
   else
      {
      wind[0][NL] = num_lines[n]; 
      c_minmax (hos_in[n], &one, wind, bands, &one, &pmin, &pmax);
      min_in_gl[n] = (long) pmin;
      max_in_gl[n] = (long) pmax;
      }  

   }  
/*
open pattern image
------------------
*/
access = IREAD;
type = EBYTE;

stat = c_eopens (fdesc,hos_patin[0],bands[0],&one,&one,num_lines_pat, num_samp, 
                 &ppoint, &access, &type, &zero, opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening pattern image","speccomb-patopen", &fatal_err);

fdesc[1] = 0;

/*
set up group for pattern image
------------------------------
*/
if ((patin_buf = (unsigned char *) c_ealloc (fdesc, &bufsiz)) == NULL)
   c_errmsg ("Error allocating dynamic memory", "speccomb-alloc", &fatal_err);

c_egroup (fdesc, &gdesc_pat, patin_buf, &bufsiz);

/*
open input image files
----------------------
*/
for (n = 0; n < *num_images; n++)
   {
   stat = c_eopens (fdesc, hos_in[n], bands[0], &one, &one, &num_lines[n], 
                    num_samp, &ipoint[n], &access, &type, &zero, opt);
   if (stat != E_SUCC )
      c_errmsg ("Error opening input image","speccomb-inopen", &fatal_err);

   fdesc[1] = 0;

/*
   set up group for each image
   ---------------------------
*/
   if ((in_buf[n] = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
      c_errmsg ("Error allocating dynamic memory", "speccomb-alloc", &fatal_err);

   c_egroup (fdesc, &gdesc_in[n], in_buf[n], &bufsiz);
   }

/*
open output image
-----------------
*/
access = IWRITE;
stat = c_eopens (fdesc, hos_out[0], &one, &one, &one, num_lines_pat, num_samp, 
                 &opoint, &access, &type, &zero, opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "speccomb-outopen", &fatal_err);

wind[0][NL] = *num_lines_pat; 
c_upddr (&block, hos_patin, hos_out[0], &one, wind, bands, &one, &line_inc, 
         &samp_inc, &one, &comb_flag);

fdesc[1] = 0;

/*
set up group for output image
------------------------------
*/
if ((out_buf = (unsigned char *) c_ealloc (fdesc, &bufsiz)) == NULL)
   c_errmsg ("Error allocating dynamic memory", "speccomb-alloc", &fatal_err);

c_egroup (fdesc, &gdesc_out, out_buf, &bufsiz);
}
