#include "speccomb.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char las_pat[][CMLEN],      /* pattern image LAS name           */
    char hos_pat[][CMLEN],      /* pattern image host name          */
    char las_in[][CMLEN],     /* Level II images LAS names        */
    char hos_in[][CMLEN],     /* Level II images host names       */
    char las_out[][CMLEN],      /* output image LAS name            */
    char hos_out[][CMLEN],      /* output image host name           */
    long *num_images,         /* number of Level II images        */
    long grey_level[],        /* pattern grey level values        */
    long *num_lines_pat,      /* number of lines in pattern image */
    long *num_samp,           /* number of samples per line       */
    long num_lines[],         /* number of lines in images        */
    char print_file[CMLEN]   /* print file name                  */
)
/*
**********************************************************************

NAME: GETPAR

PURPOSE: Gets parameters 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           30-Aug-83  K. A. Narayanan
   2           24-Feb-84  J. Seamone 
   3            6-Dec-88  K. A. Narayanan     Conversion to C
   4           17-Sep-90  B. Ailts            Now allows a user to use the band
						specification of LAS in order
						specify one band of a multi-band
						image for the parameter IN
   5.1         16-Aug-93  D. Bridges          Added PRINT parameter

ALGORITHM DESCRIPTION:

   Initialize program
   Get and validate pattern image parameters
   Get and validate parameters of Level II images 
   Get grey level values in pattern image corresponding to the 
      Level II images
   Get output image name
   Get print option and open print file
   Get las globals

**********************************************************************
*/
{

long    fatal_err = -1;
long    zero = 0;
long    one = 1;

extern  long print[];
extern  char *prog_name;
extern  FILE *fd;

struct	DDR	ddr;                /* image DDR                          */
struct VARIABLE	 *var;              /* pointer to a variable in las block */

long	count,
	flag[1][2],                    /* flag for subwindows                */
	gflag,
	i, j,
	image_band_num[1][MAXBND + 1], /* band numbers in each input image */
	nimages,                    /* no. images specified in image param */
	num_image_bands[MAXIMG],    /* number of bands in each input image */
	num_inbands,                /* total number of input image bands   */
	wind[1][4],                    /* input image windows              */
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out,                    /* logical unit for standard output   */
	type;

char	minmax[4],                  /* minmax option                      */
        fileutil_file[CMLEN];       /* file name for fileutil             */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate pattern image parameters
-----------------------------------------
*/
stat = c_getin (&block, "MASKIMG", &zero, las_pat, hos_pat, &nimages, wind,
               image_band_num, num_image_bands, &num_inbands, &one, 
               &same_window, flag);

if (stat != E_SUCC) 
    c_errmsg ("Error getting pattern image name", "speccomb-pat", &fatal_err);

if (num_inbands  !=  1)
   c_errmsg ("Pattern image must be a single band image",
             "speccomb-patbands", &fatal_err);

*num_lines_pat = wind[0][NL];
*num_samp = wind[0][NS];

c_getddr (hos_pat[0], &ddr);
if (ddr.dtype != EBYTE)
   c_errmsg ("Pattern image must be a BYTE image",
             "speccomb-pattype", &fatal_err);
/*
get and validate output image parameters
----------------------------------------
*/
stat = c_getout (&block, "OUT", las_out, hos_out);
if (stat != E_SUCC)
   c_errmsg("Error getting output image", "speccomb", &fatal_err);

/*
get and validate input level II classified image names and corresponding 
pattern image grey levels
------------------------------------------------------------------------
*/
var = p_find (&block, "IN");
*num_images = (*var).v_count;

for (i = 0; i < *num_images; i++)
   {
   stat = c_getin (&block, "IN", &i, las_in[i], hos_in[i], &nimages, wind,
               image_band_num, num_image_bands, &num_inbands, &one, 
               &same_window, flag);

   if (stat != E_SUCC) 
       c_errmsg ("Error getting Level II classified image name", "speccomb-in", 
                 &fatal_err);

   if (num_inbands  !=  1 || flag[0][0] == 1)
      c_errmsg ("Level II classified image(s) must be single band with no window",
                "speccomb-inbands", &fatal_err);

c_getddr (hos_in[i], &ddr);
if (ddr.dtype != EBYTE)
   c_errmsg ("Level II classified image(s) must be BYTE",
             "speccomb-intype", &fatal_err);

   if (wind[0][NS]  !=  *num_samp ||  wind[0][NL] > *num_lines_pat)
      c_errmsg (
          "Level II Classified image and Pattern image sizes do not match",
          "speccomb-size", &fatal_err);
   num_lines[i] = wind[0][NL];
   }
count = NIMG;
type = ELONG;
if (c_getnum (&block, "MASKVAL", &count, &type, grey_level) != *num_images)
   c_errmsg ("Number of grey levels and number of images do not match",
             "speccomb-maskval", &fatal_err);

/*
get print option and open print file
------------------------------------
*/
strcpy(fileutil_file, prog_name);
stat = c_setprt(&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
              print_file);
if(stat != E_SUCC)
       c_errmsg("Fatal error encountered","speccomb-fatal",&fatal_err);
fd = fopen(fileutil_file, "w");

/*
check for valid grey level values
---------------------------------
*/
gflag = TRUE;
for (i = 0; i < *num_images; i++)
   {
   j = 0;
   while (gflag  &&  j  <  i)
      {
      if (grey_level[i]  ==  grey_level[j]) gflag = FALSE;
      j++;
      }  
   if (!gflag) 
      c_errmsg ("Same grey level values specified for different images",
                "speccomb-sameval", &fatal_err);
   }  

/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y' || minmax[0] == 'y') min_max = TRUE;
else min_max = FALSE;
}
