/* include files, global parameters, and definitions for SPECCOMB      */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "protos.h"

struct PARBLK block;            /* TAE parameter block     */

struct FDESC *	fdesc[2];               /* file descriptors        */
struct GDESC *	gdesc_in[8];            /* image group descriptors */
struct GDESC *	gdesc_out;              /* output group descriptor */
struct GDESC *	gdesc_pat;              /* pattern image group descriptor */
long	ipoint[8];              /* offsets into image i/o  */
long	opoint;                 /* for input and output bands */
long	min_max;                /* minmax option flag      */
long 	msgtim;                 /* message interval        */
long	ppoint;
long    print[3];               /* print options           */
 
unsigned char	*in_buf[8];     /* image i/o buffer        */
unsigned char	*out_buf;       /* output image i/o buffer */
unsigned char	*patin_buf;     /* pattern image i/o buffer*/
char    *prog_name = "SPECCOMB"; /* program name           */

extern	long	curcat;                 /* message utility         */
extern	long	curent;                 /* parameters for processing */

FILE    *fd;                     /* output file descriptor  */

#define TERM 0
#define LP   1
#define FIL  2

#define NIMG 8

int main
(void) 
{

/*
***************************************************************************

NAME: SPECCOMB

FUNCTION:  Combine level I and level II classifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1            1-Sep-83  K. A. Narayanan
   2           25-Mar-87  D. Sullivan  
   3           18-Aug-88  K. A. Narayanan     Conversion to 5.0
   4           17-Sep-90  B. Ailts            Now allows a user to use the band
						specification of LAS in order
						specify one band of a multi-band
						image for the parameter IN
   5           23-Sep-92  D. Etrheim          Modified global include file
   5.1	        8-Oct-92  S. Nelson	      Made correction to combine.c to
					      fix error on printout on sg.
   5.1         16-Aug-93  D. Bridges          Added PRINT parameter

ALGORITHM DESCRIPTION:

   Call GETPAR to get parameters
   Call FILSPEC to set up files
   Call COMBINE to combine the classified images using pattern image
   Call TERMINATE to update output image history and inform user 
      about successful completion

***************************************************************************
*/

long	grey_level[NIMG],        /* class grey level values             */
	max_in_gl[NIMG],         /* max grey level in Level II images   */
	max_pat_gl,              /* max grey level in pattern image     */
	min_in_gl[NIMG],         /* min grey level in Level II images   */
	min_pat_gl,              /* min grey level in pattern image     */
	num_images,              /* number of classified images         */ 
	num_lines[NIMG],         /* number of lines in the images       */
	num_lines_pat,           /* number of lines in pattern image    */
	num_samp;                /* number of samples per line          */

char	hos_in[NIMG][CMLEN],     /* Level II images host names          */
	hos_out[1][CMLEN],          /* output image host name              */
        hos_pat[1][CMLEN],          /* pattern image host name             */
	las_in[NIMG][CMLEN],     /* Level II images las names           */
	las_out[1][CMLEN],          /* output image las name               */
	las_pat[1][CMLEN],          /* pattern image las name              */
        print_filename[CMLEN];   /* print file name                     */

/*
get las parameters
------------------
*/
getpar (las_pat, hos_pat, las_in, hos_in, las_out, hos_out, &num_images, 
        grey_level, &num_lines_pat, &num_samp, num_lines, print_filename);

/*
set up files
------------
*/
filspec (hos_pat, hos_in, hos_out, &num_images, &num_lines_pat, &num_samp, 
         num_lines, min_in_gl, max_in_gl, &min_pat_gl, &max_pat_gl);

/*
insert level II classifications into level I classification
-----------------------------------------------------------
*/
combine (las_pat[0], las_in, hos_out[0], num_samp, &num_lines_pat, 
         num_images, max_in_gl, min_in_gl, min_pat_gl, max_pat_gl, grey_level);


/*
update output image history and issue completion message
--------------------------------------------------------
*/
terminate (las_out, hos_out[0], hos_pat[0], hos_in, num_images);

return EXIT_SUCCESS;

}
