
#include "specstrt.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char hos_patin[],          /* pattern image host name           */
    char hos_in[][CMLEN],      /* input image host name descriptors */
    char hos_out[][CMLEN],     /* output image host name descriptor */
    long pat_image_band_num[],/* pattern image band number          */
    long *num_inimages,        /* number of input images             */
    long num_image_bands[],   /* number of bands in each input image*/
    long out_num_lines[],     /* no. lines in output images         */
    long *num_bands,          /* no. bands in the image             */
    long class_num,            /* class number processed            */
    long num_classes,         /* no. classes to be processed        */
    long grey_level[],         /* class grey level values           */
    long image_band_num[][MAXBND + 1], /* band numbers in each      */
    long wind[][4],           /* input image windows                */
    long wind_pat[4],         /* pattern image window               */
    long *image_type         /* input/output image data type       */
)
/*
**************************************************************************

NAME: FILSPEC

PURPOSE: Set up file specifications and get sizes of output images

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           30-Aug-83  K. A. Narayanan
   2           24-Feb-84  J. Seamone 
   3           18-Aug-88  K. A. Narayanan     Conversion to 5.0
   6.1	       25-May-94  D. Eichacker		added opt[]

ALGORITHM DESCRIPTION:

   If first time:
      Open pattern image
      Call SIZOUT to get the sizes of all the output images
      Close pattern image
   If specified grey level not found in the pattern image return (no output 
      image is generated for that class)
   Otherwise reopen pattern image for subsequent processing
   Open input image files and group them with pattern image
   Open output image file and make a group

*****************************************************************************
*/
{
double	line_inc = 1.0,              /* line and sample increments for     */
	samp_inc = 1.0;              /* output image ddr                   */

static	long	opt[3]={0,0,0};	     /* optional parameter array for EOPENS */
long	access,                      /* file access type                   */
	comb_flag = 0,               /* image combination flag             */
	bufsiz,                      /* image i/o buffer size              */
        fatal_err = -1,
	n, nib,
	one = 1,
	stat,                        /* return status                      */
	type,                        /* file type                          */
	zero = 0;

if (class_num == 0) 
   {
   access = IREAD;
   type = EBYTE;
   stat = c_eopens (fdesc, hos_patin, pat_image_band_num, &wind_pat[SL], 
                    &wind_pat[SS], &wind[0][NL], &wind[0][NS], &ppoint, 
                    &access, &type, &zero, opt);
   if (stat != E_SUCC )
      c_errmsg ("Error opening pattern image","specstrt-patopen", &fatal_err);

   fdesc[1] = 0;

/*
   set up group for pattern image
   ------------------------------
*/
   if ((patin_buf = (unsigned char *) c_ealloc (fdesc, &bufsiz)) == NULL)
      c_errmsg ("Error allocating dynamic memory", "specstrt-alloc", &fatal_err);

   c_egroup (fdesc, &gdesc_pat, patin_buf, &bufsiz);

/*
   compute output image sizes
   --------------------------
*/
   sizout (wind, num_classes, grey_level, out_num_lines);

   c_egclse (&gdesc_pat);
   free (patin_buf);

/*
   get input image data type
   -------------------------
*/
   c_gettyp (hos_in, &one, image_type);
   }

/*
if specified grey level not found in the pattern image no output image is
generated for that class; return
-------------------------------------------------------------------------
*/
if (out_num_lines[class_num] == 0) return;

/*
reopen pattern image for subsequent processing
----------------------------------------------
*/
access = IREAD;
type = EBYTE;
stat = c_eopens (fdesc, hos_patin, pat_image_band_num, &wind_pat[SL], 
                 &wind_pat[SS], &wind[0][NL], &wind[0][NS], &ppoint, 
                 &access, &type, &zero, opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening pattern image","specstrt-patopen", &fatal_err);

fdesc[1] = 0;

/*
set up group for pattern image
------------------------------
*/
if ((patin_buf = (unsigned char *) c_ealloc (fdesc, &bufsiz)) == NULL)
   c_errmsg ("Error allocating dynamic memory", "specstrt-alloc", &fatal_err);

c_egroup (fdesc, &gdesc_pat, patin_buf, &bufsiz);
/*
open input image files
----------------------
*/
access = IREAD;
for (n = 0, nib = 0; n < *num_inimages; n++)
   {
   stat = c_eopens (&fdesc[n], hos_in[n], image_band_num[n], &wind[0][SL], 
                    &wind[0][SS], &wind[0][NL], &wind[0][NS], &ipoint[nib], 
                    &access, image_type, &zero, opt);
   if (stat != E_SUCC )
      c_errmsg ("Error opening input image","specstrt-inopen", &fatal_err);

   nib += num_image_bands[n];
   }  
   
fdesc[n] = 0;

/*
set up group containing input bands
-----------------------------------
*/
if ((in_buf = (unsigned char *) c_ealloc(fdesc, &bufsiz)) == NULL)
   c_errmsg ("Error allocating dynamic memory", "specstrt-alloc", &fatal_err);

c_egroup (fdesc, &gdesc_in, in_buf, &bufsiz);

/*
open output image
-----------------
*/
access = IWRITE;
stat = c_eopens (fdesc, hos_out[class_num], num_bands, &one, &one,
                 &out_num_lines[class_num], &wind[0][NS], opoint, &access, 
                 image_type, &zero, opt);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "specstrt-outopen", &fatal_err);

c_upddr (&block, hos_in, hos_out[0], num_inimages, wind, image_band_num, 
         num_image_bands, &line_inc, &samp_inc, num_bands, &comb_flag);

fdesc[1] = 0;

/*
set up group for output image
------------------------------
*/
if ((out_buf = (unsigned char *) c_ealloc (fdesc, &bufsiz)) == NULL)
   c_errmsg ("Error allocating dynamic memory", "specstrt-alloc", &fatal_err);

c_egroup (fdesc, &gdesc_out, out_buf, &bufsiz);
}

