#include <string.h>
#include "specstrt.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char hos_patin[][CMLEN],    /* input pattern image host name    */
    char hos_in[MAXIMG][CMLEN],/* input image host names          */
    char tae_out[8][CMLEN],   /* output images LAS names          */
    char hos_out[8][CMLEN],   /* output images host names         */
    long *num_inimages,       /* number of input images           */
    long num_image_bands[],   /* number of bands in each input image */
    long image_band_num[][MAXBND + 1],
    long *num_inbands,        /* total number of input image bands   */
    long pat_image_band_num[][MAXBND+1],/* pattern image band number        */
    long grey_level[8],       /* pattern grey level values        */
    long *num_classes,        /* number of classes to be used     */
    long wind_pat[][4],         /* input pattern image window       */
    long wind[][4]           /* input image windows              */
)
/*
**********************************************************************

NAME: GETPAR

PURPOSE: Gets parameters 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           30-Aug-83  K. A. Narayanan
   2           24-Feb-84  J. Seamone 
   3            6-Dec-88  K. A. Narayanan     Conversion to C

ALGORITHM DESCRIPTION:

   Initialize program
   Get and validate input pattern image parameters
   Get and validate input image parameters
   Get class names and corresponding grey level values in pattern image
   Get output image name prefix and generate output image names by 
      appending class names to it
   Get TAE globals

**********************************************************************
*/
{
struct	DDR	ddr;                /* image ddr                          */
struct VARIABLE	 *var;              /* pointer to a variable in TAE block */

long	count,
	flag[MAXIMG][2],            /* flag for subwindows                */
	gflag,
	i, j, 
	imglimit = MAXIMG,          /* maximum number of images           */
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	stat,                       /* return status                      */
	std_out,                    /* logical unit for standard output   */
	type,
	fatal_err = -1,
	one = 1,
	zero = 0;

char	buf[100],
	class[8][CMLEN],            /* class names                        */
	minmax[4],                  /* minmax option                      */
	out[CMLEN];                 /* output image user specified prefix */
static	char tae_in[MAXIMG][CMLEN]; /* input images LAS names             */
char	tae_patin[1][CMLEN];           /* input pattern image LAS name       */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input pattern image parameters
-----------------------------------------------
*/
stat = c_getin (&block, "MASKIMG", &zero, tae_patin,hos_patin, num_inimages, 
               wind_pat,pat_image_band_num, num_image_bands, num_inbands, 
               &one, &same_window, flag);

if (stat != E_SUCC) 
    c_errmsg ("Error accessing pattern image name", "specstrt-patin", 
              &fatal_err);

if (*num_inbands  !=  1)
   c_errmsg( "Multi-band image not allowed for input pattern image", 
             "specstrt-patbands", &fatal_err);

c_getddr (hos_patin[0], &ddr);
if (ddr.dtype != EBYTE)
   c_errmsg ("Pattern image must be a BYTE image", "specstrt-pattype", 
              &fatal_err);
/*
get and validate input image parameters
---------------------------------------
*/
stat = c_getin (&block, "IN", &zero, tae_in, hos_in, num_inimages, wind, 
               image_band_num, 
               num_image_bands, num_inbands, &imglimit, &same_window, flag);

if (stat != E_SUCC) 
    c_errmsg ("Error accessing input image name", "specstrt-in", &fatal_err);

if (same_window == 0)
    c_errmsg ("Image windows not same size", "specstrt-wind", &fatal_err);

if (wind_pat[0][NL] != wind[0][NL] || wind_pat[0][NS] != wind[0][NS])
   {
   if (wind_pat[0][NL] < wind[0][NL]) wind[0][NL] = wind_pat[0][NL];
   if (wind_pat[0][NS] < wind[0][NS]) wind[0][NS] = wind_pat[0][NS];
   c_errmsg ("Pattern and input image sizes do not match ...", 
             "specstrt-size1", &one);
   sprintf (buf, "(NL, NS) = (%d, %d) used", wind[0][NL], wind[0][NS]);
   c_errmsg (buf, "specstrt-size2", &one);
   }
/*
get output class names and their greylevels in pattern image
------------------------------------------------------------ 
*/
var = p_find (&block, "CLASSNAM");
*num_classes = (*var).v_count; 
for (i = 0; i < *num_classes; i++)
   strcpy (class[i], SVAL(*var, i));
count = 8;
type = ELONG;
if (c_getnum (&block, "MASKVAL", &count, &type, grey_level) != *num_classes)
   c_errmsg ("Number of classes and number of grey values are not equal",
             "specstrt-maskval", &fatal_err);

/*
get output image names
----------------------
*/
var = p_find (&block, "OUT");
strcpy (out, SVAL (*var, 0));

/*
generate concatenated file names for output files
-------------------------------------------------
*/
for (i = 0; i < *num_classes; i++)
   {
   strcpy (tae_out[i], out);
   strcat (tae_out[i], ".");
   strcat (tae_out[i], class[i]);
   strcat (tae_out[i], ";img");

   c_cmhost (tae_out[i], hos_out[i]);
   }  

gflag = TRUE;
for (i = 0; i < *num_classes; i++)
   {
   j = 0;
   while (gflag  &&  j  <  i)
      {
      if (grey_level[i]  ==  grey_level[j]) gflag = FALSE;
      j++;
      }  
   if (!gflag) 
      c_errmsg ("Same grey level values specified for different classes",
                "specstrt-sameval", &fatal_err);
   }  

/*
Get TAE globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y' || minmax[0] == 'y') min_max = TRUE;
else min_max = FALSE;

}
