#include "specstrt.h"
#include "protos.h"

void sizout
(
    long wind[][4],          /* input image window        */
    long num_classes,        /* no. classes               */
    long grey_level[8],      /* class grey level values   */
    long out_num_lines[]    /* no. lines in output images*/
)
/*
*********************************************************************

NAME: SIZOUT

PURPOSE: Computes no. lines for output images

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           30-Aug-83  K. A. Narayanan
   2           24-Feb-84  J. Seamone 
   3           18-Aug-88  K. A. Narayanan     Conversion to 5.0

ALGORITHM DESCRIPTION:

   If first class being processed then
      Store the sequence of input class grey level values in a table
   For number of lines in pattern image
      Read the next line
      For number of pixels in the line
         Get the pattern image grey level
	 Determine the output image class number from the table    
         If it is not zero then
	     Increment number of pixels in the output class image
   For number of classes
      Compute no. lines in output class image = 
                       no. pixels in the image / no. pixels per line

*************************************************************************
*/
{
long	class_num, 
	class_table[256],       /* table of class grey levels    */
	gval, 
	i, 
	line, 
	samp;

for (i = 0; i < 256; i++) class_table[i] = 0;

/*
store the sequence of input class grey level values
---------------------------------------------------
*/
for (i = 0; i < num_classes; i++)
   {
   out_num_lines[i] = 0;
   class_table[grey_level[i]] = i+1;
   }  

/*
compute the number of pixels in each class
------------------------------------------
*/
for (line = 0; line <  wind[0][NL]; line++)
   {
   c_estep (&gdesc_pat);

   for (samp = 0; samp < wind[0][NS]; samp++)
      {
      gval = patin_buf[ppoint+samp];
      class_num = class_table[gval];
      if (class_num  !=  0) out_num_lines[class_num-1]++;
      }  
   }  

/*
compute the no. of lines 
------------------------
*/
for (i = 0; i < num_classes; i++)
   out_num_lines[i] = (out_num_lines[i] + wind[0][NS] - 1)/wind[0][NS];

}
