#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct	PARBLK block;			/* TAE parameter block            */

struct FDESC *	fdesc[MAXIMG];			/* file descriptors               */
struct GDESC *  gdesc_in;			/* input imane group descriptor   */
struct GDESC *  gdesc_out;			/* output image group descriptor  */
struct GDESC *  gdesc_pat;			/* pattern image group descriptor */
long    ipoint[MAXIMG * MAXBND + 1], 	/* offsets into image i/o	  */
        opoint[MAXBND],			/* for input and output bands	  */
        ppoint;
long    min_max;			/* minmax option flag		  */
long    msgtim;				/* message interval		  */

unsigned char   *in_buf;		/* input image i/o buffer	  */
unsigned char   *out_buf;		/* output image i/o buffer	  */
unsigned char   *patin_buf;		/* image i/o buffer		  */


int main
(void) 
{

/*
***************************************************************************

NAME: SPECSTRT

FUCNTION: Stratify an input multispectral image to produce seperate class
          multispectral images


PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1           30-Aug-83  K. A. Narayanan
   2           24-Feb-84  J. Seamone 
   3            6-Dec-88  K. A. Narayanan     Conversion to 5.0
   4           24-Jan-92  S. Nelson           Removed warning messages on
                                              compile


ALGORITHM DESCRIPTION:

   Call GETPAR to get parameters
   For number of classses
      (Process one class at a time for easy expansion)
      Call FILSPEC to get all output image sizes first time, 
         and set up pattern, input and output image files
      Call STRAT to produce output class image and update output class 
         image history
   Inform user about successful completion

***************************************************************************
*/

long	class_num,			/* class number processed           */
	grey_level[8];			/* class grey level values          */
static	long image_band_num[MAXIMG][MAXBND + 1]; 
					/* band numbers in each input image */
long	image_type,			/* input/output image type          */
	out_num_lines[8],		/* no. lines in output images       */
	num_classes,			/* number of classes                */
	num_image_bands[MAXIMG],	/* number of bands in each input image*/
	num_bands,			/* total number of input image bands*/
	num_inimages,			/* number of input images           */
	pat_image_band_num[1][MAXBND+1],	/* pattern image band number        */
	wind[MAXIMG][4],		/* input image windows              */
	wind_pat[1][4],			/* pattern image window             */
	zero = 0;

char tmpstr[1][CMLEN];       /*  Temp String */
static	char hos_in[MAXIMG][CMLEN];	/* input images host names	    */
char	hos_patin[1][CMLEN],		/* input pattern image host name    */
	hos_out[8][CMLEN],		/* output image host names          */
	tae_out[8][CMLEN];		/* output image TAE names           */

/*
get tae parameters
------------------
*/
getpar (hos_patin, hos_in, tae_out, hos_out, &num_inimages, num_image_bands, 
       image_band_num, &num_bands, pat_image_band_num, grey_level, 
       &num_classes, wind_pat, wind);

/*
process one class at a time (for easy expansion)
------------------------------------------------
*/
for (class_num = 0; class_num < num_classes; class_num++)
   {

/*
   set up files; and get all out image sizes the first time
   --------------------------------------------------------
*/
   filspec( hos_patin[0], hos_in, hos_out,pat_image_band_num[0], &num_inimages, 
            num_image_bands, out_num_lines, &num_bands, class_num, 
            num_classes, grey_level, image_band_num, 
            wind, wind_pat[0], &image_type);

/*
   stratify input image to get output class image; update its history
   ------------------------------------------------------------------
*/
   strat (wind, num_bands, class_num, &num_classes, out_num_lines, 
          grey_level, tae_out, hos_out, hos_in, &num_inimages, image_type);

   }

/*
issue successful completion message
-----------------------------------
*/
strcpy(tmpstr[0], "\0");
c_complt ("SPECSTRT", tmpstr, &zero);

return EXIT_SUCCESS;

}
