/*******************************************************************************
FUNCTION:	splitname

PURPOSE:	Split a file name into its directory, root, and extension.


Version  Date   Programmer       Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    12/93  J.Fenno          Development
  1.1    08/98  D.Lloyd          Added call to c_imgprs() to remove LAS window
  				 specification from LAS file name.


PROJECT: LAS
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "util2.h"


#define LAS      0
#define HOST     1
#define LAS2HOST 2
#define HOST2LAS 3


int main()
{
char dir[CMLEN];	/* Directory portion of the file specification        */
char ext[CMLEN];	/* Extension portion of the file specification        */
char filename[CMLEN];	/* User supplied file specification                   */
char *ptr[1];		/* Ptr to a string for calling q_string()             */
char root[CMLEN];	/* Root portion of the file specification             */
char str[CMLEN];	/* Temporary string                                   */

long fatal    = -1;	/* Fatal error flag                                   */
long nonfatal =  1;	/* Nonfatal error flag                                */
long std_out;		/* Standard output file                               */

short option;		/* Flag indicating the option being run               */

struct PARBLK vblock;		/* TAE parameter block                        */
struct VARIABLE *parm;		/* Ptr to a TAE variable                      */

/* Variables added for call to c_imgprs
---------------------------------------*/
struct	PRS_LST p_list[1];
int	ximages;


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);


/* Retrieve the option
----------------------*/
parm = p_find(&vblock, "OPTION");
if (strcmp("LAS", SVAL(*parm,0)) == 0)
    option = LAS;
else if (strcmp("HOST", SVAL(*parm,0)) == 0)
    option = HOST;
else if (strcmp("LAS2HOST", SVAL(*parm,0)) == 0)
    option = LAS2HOST;
else
    option = HOST2LAS;


/* Retrieve the file specification and convert it to a host name if necessary
-----------------------------------------------------------------------------*/
parm = p_find(&vblock, "INFILE");
switch (option)
    {
    case LAS:
    case LAS2HOST:
        strcpy( str, SVAL(*parm,0));
	/* Parse out the window specification if present
	------------------------------------------------*/
        if ( c_imgprs( str, p_list, &ximages ) != E_SUCC )
	    return E_FAIL;
	c_cmhost( p_list->name, filename );
	break;

    case HOST:
    case HOST2LAS:
	strcpy(filename, SVAL(*parm, 0));
	break;
    }


/* Split the file name into it parts
------------------------------------*/
c_splitname(filename, dir, root, ext);


/* If there is no directory then get the current directory
----------------------------------------------------------*/
if (strlen(dir) == 0)
    {
    getcwd(dir, CMLEN);
    strcat(dir, "/");     /* add a trailing '/' behind all directories */
    }


/* Return the pieces to TAE
---------------------------*/
ptr[0] = dir;
if (q_string(&vblock, "DIR", 1, ptr, P_UPDATE) != SUCCESS)
    {
    c_errmsg("Error sending parameter to TAE", "splitname-tae", &nonfatal);
    c_errmsg("Fatal error encountered", "splitname-fatal", &fatal);
    }
ptr[0] = root;
if (q_string(&vblock, "ROOT", 1, ptr, P_UPDATE) != SUCCESS)
    {
    c_errmsg("Error sending parameter to TAE", "splitname-tae", &nonfatal);
    c_errmsg("Fatal error encountered", "splitname-fatal", &fatal);
    }
ptr[0] = ext;
if (q_string(&vblock, "EXT", 1, ptr, P_UPDATE) != SUCCESS)
    {
    c_errmsg("Error sending parameter to TAE", "splitname-tae", &nonfatal);
    c_errmsg("Fatal error encountered", "splitname-fatal", &fatal);
    }
if (q_out(&vblock) != SUCCESS)
    {
    c_errmsg("Error sending parameter to TAE", "splitname-tae", &nonfatal);
    c_errmsg("Fatal error encountered", "splitname-fatal", &fatal);
    }


/* Issue a successful completion message
----------------------------------------*/
z_exit(1, "SPLITNAME completed successfully");

return EXIT_SUCCESS;
}
