/******************************************************************************
NAME:           get_datum

PURPOSE:        Fill DDR with datum information.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      02/00  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT:        LAS

ALGORITHM:
If datum does not exist
    Issue a warning message and continue
Else 
    If datum code does not exist in LASTABLES
        Then Issue error message
        Set datum to CLARK 1866
    Else set datum to VALID

Set datum in DDR
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "spothdf2las.h"
#include "geompak.h"

int get_datum
(
    struct DDR *ddr,        /* I/O: local DDR structure                      */
    char datum[]            /* I: datum name retrieved from HDF logfile      */
)
{
    long datum_code = 0;    /* LAS datum code                                */

    /* Check for datum and set in DDR
       --------------------------------*/
    if (strcmp(datum, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-datum", 
                 "Datum does not exist in logfile");
    }
    else
    {
        /* set datum
           -----------*/
        if (E_FAIL == c_dtmnum((char *)datum, &datum_code))
        {
            c_lasmsg(FATAL, "spothdf2las-fatal", "Fatal error encountered");
            return (E_FAIL);
        }
        if (datum_code == -1)
        {
            c_lasmsg(NONFATAL, "spothdf2las-warn",
                    "Datum %s was not found in LASTABLES", datum);
            datum_code = 0;
        }
        else
            ddr->valid[2] = VALID;    /* set datum to valid*/
    }

    ddr->datum_code = datum_code;

    return (E_SUCC);
}
