/******************************************************************************
NAME: 		get_spheroid

PURPOSE: 	Fill DDR with spheroid information.

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      02/00  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: 	LAS

ALGORITHM:
If spheroid name does not exist
    Issue a warning message and continue
Else 
    If spheroid does not exist in LASTABLES
        Then issue an error message
        Set spheroid to CLARK 1866
    Else
        Set spheroid to valid 
          
Set sphere code in DDR

If spherical radius does not exist
    Issue a warning message and continue
Else 
    Set sphere in DDR
    Set projection parameters to valid

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "spothdf2las.h"
#include "geompak.h"

int get_spheroid
(
    struct DDR *ddr,        /* I/O: local DDR structure                      */
    char spheroid[],        /* I: spheroid name retrieved from HDF logfile   */
    char sphere_radius[]    /* I: sphere radius                              */ 
)
{
    int sphere_code = 0;    /* LAS sphere code                               */

    /* Check for datum and set in DDR
       --------------------------------*/
    if (strcmp(spheroid, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-sphere",
                "Sphere does not exist in logfile");
    } 
    else
    {
        /* set sphere 
           ------------*/
        if (E_FAIL == c_dtmnum((char *)spheroid, &sphere_code))
        {
            c_lasmsg(FATAL, "spothdf2las-fatal", "Fatal error encountered");
            return (E_FAIL);
        }
        if (sphere_code == -1)
        {
            c_lasmsg(NONFATAL, "spothdf2las-warn", 
                    "Spheroid %s was not found in LASTABLES",spheroid);
            sphere_code = 0;
        }
        else
            ddr->valid[2] = VALID;    /* set sphere to valid */
    }

    ddr->datum_code = sphere_code; 

    /* Check sphere and set in DDR
       -----------------------------*/
    if (strcmp(sphere_radius, "") == 0)
    {
        c_lasmsg(NONFATAL, "spothdf2las-sphere",
                "Spherical Radius does not exist in logfile");
    }
    else
    {
        ddr->proj_coef[0] = atof(sphere_radius);
        ddr->valid[3] = VALID;    /* set projection parameters to valid */ 
    }
    return (E_SUCC);
}
