/*******************************************************************************
NAME:		getpar

PURPOSE:	Get the input parameters for SPOTHDF2LAS

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      02/00  D. Hames          C    Original developement


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under UNIX and TAE.

PROJECT:         LAS

ALGORITHM:
Initialize TAE interface
Get the TAE global $MSGTIM
Retrieve input Spot HDF file name(s)
Copy total number of HDF input files to numhdf_files
Retrieve output image name(s)
Copy total number of LAS output files to numlas_files
Error check 
	if total number of OUTS is greater than one
	    then number of OUTS must equal number of INFILES
Retrieve logfile name

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "spothdf2las.h"
#include "util.h"

void getpar
(
    struct PARBLK *vblock,      /* I: parameter block	                      */
    char infile[][CMLEN],       /* O: TAE file name(s) of input HDF image(s)  */
    char hosout[][CMLEN],       /* O: host file name(s) of output image(s)    */
    char taeout[][CMLEN],       /* O: TAE file name(s) of output image(s)     */
    int *numhdf_files,          /* O: total number of HDF input files         */
    int *numlas_files,          /* O: total number of LAS output files        */
    long *msgtim,               /* O: # of seconds between processing messages*/
    char logfile[]              /* O: name of file containing DDR information */
)
{
    struct VARIABLE *val;     	/* pointer to TAE variable within the 
                                   parameter block                          */
    long index;                 /* loop index                               */
    long std_out = 0;           /* standard output                          */
    int status = 0;             /* status indicator                         */

    /* Initialize TAE interface
       --------------------------*/
    c_inlas(vblock, &std_out);

    /* Get the TAE globals needed
       ----------------------------*/
    val = p_find(vblock, "$MSGTIM");
    *msgtim = IVAL(*val, 0);

    /* Retrieve input Spot HDF file name
       ----------------------------------*/
    val = p_find(vblock, "INFILE");
    *numhdf_files = val->v_count;

    for (index = 0; index < *numhdf_files; index++)
        strcpy(infile[index], SVAL(*val,index));

    /* Retrieve output image name 
       --------------------------- */
    val = p_find(vblock,"OUT");
    *numlas_files = val->v_count;

    status = c_getout(vblock, "OUT", taeout, hosout);
    if (status != E_SUCC)
        c_lasmsg(FATAL,"spothdf2las-getpar","Fatal error encountered");

    /* The total number of OUTS must equal the total number of INPUTS,
       unless total number of OUTS equals one
       -------------------------------------------------------------------*/
    if (*numlas_files > 1) 
    {
        if (*numlas_files != *numhdf_files)
        {
            c_lasmsg(FATAL, "spothdf2las-file", 
                    "Total number of OUT files must equal total number of " 
                    "INFILE files or specity only one OUT");
        }
    }

    /* Retrieve name of logfile 
       --------------------------*/
    val = p_find(vblock, "LOGFILE");
    strcpy(logfile, SVAL(*val,0));

}
