#ifndef SPOTHDF2LAS_H
#define SPOTHDF2LAS_H

#include "las.h"
#define FATAL 1		
#define NONFATAL 0
#define NUMFILE 20     /* Maximum number of files that a SPOT HDF scene will
		          contain */

#define  HDF_FAIL -1   /* HDF defines SUCCESS 0 and FAIL -1.  There is a 
                          conflict with the TAE and HDF include files 
                          concerning FAIL. For HDF, hdf.h defines FAIL as -1. 
                          In TAE, taeconf.inp defines FAIL as 0.  Therefore, 
                          instead of using FAIL, HDF_FAIL was defined */

void gethdfinfo
(
    char infile[][CMLEN],       /* I: TAE file name(s) of input HDF images    */
    int numhdf_files,           /* I: total number of input HDF files         */
    int numlas_files,           /* I: total number of output LAS files        */
    long nl[],                  /* O: total number of lines                   */
    long ns[],                  /* O: total number of samples                 */
    int idtype[],               /* O: LAS data type to be used                */
    char hdf_name[][CMLEN]      /* O: HDF file descriptons needed for DDR     */
);

int getimage
(
    char infile[],               /* I: input HDF image file name            */
    long nl,                     /* I: total number of lines                */
    long ns,                     /* I: total number of samples              */
    struct FDESC *fdesc,         /* I: image file descriptor                */ 
    char hosout[],               /* I: host name of output image            */
    int idtype                   /* I: size of input data type of HDF image */
);

void getpar
(
    struct PARBLK *vblock,      /* I: parameter block                         */
    char infile[][CMLEN],       /* O: TAE file name(s) of input HDF image(s)  */
    char hosout[][CMLEN],       /* O: host file name(s) of output image(s)    */
    char taeout[][CMLEN],       /* O: TAE file name(s) of output image(s)     */
    int *numhdf_files,          /* O: total number of HDF input files         */
    int *numlas_files,          /* O: total number of LAS output files        */
    long *msgtim,               /* O: # of seconds between processing messages*/
    char logfile[]              /* O: name of file containing DDR information */
);

int process_hdf
(
    char infile[][CMLEN],       /* I: input image name                        */
    char hosout[],              /* I: host name of output image               */
    long msgtim,                /* I: message time                            */
    int numhdf_files,           /* I: total number of HDF input files         */
    long nl,                    /* I: total number of lines                   */
    long ns,                    /* I: total number of samples                 */
    int idtype,                 /* I: input data type of HDF image file       */
    char hdf_name[][CMLEN],     /* I: HDF image file descriptions used in DDR */
    char logfile[]              /* I: file containing geographic information  */
);

int update_ddr
(
    char hosout[],           /* I: name of the host ddr to update             */
    char hdf_name[][CMLEN], /* I: HDF image file descriptions needed for DDR */
    int nbands,             /* I: total number of bands in the LAS image     */
    char logfile[]          /* I: file containing geographic information     */
);

void albers
(
   struct DDR *ddr,         /* I/O: local DDR structure                      */
   char semi_major[],       /* I: spheroid semi major axis of the image      */
   char semi_minor[],       /* I: spheroid semi minor axis of the image      */
   char parallel1[],        /* I: first projection standard parallel         */
   char parallel2[],        /* I: second projection standard parallel        */
   char longitude[],        /* I: projection longitude of the image          */
   char latitude[],         /* I: projection latitude of the image           */
   char false_easting[],    /* I: x coordinate of the origin for projection  */
   char false_northing[],   /* I: y coordinate of the origin for projection  */
   int proj_kilometers      /* I: flag indicating if projection units is in
                                  kilometers */

);

void conic
(
   struct DDR *ddr,         /* I/O: local DDR structure                      */
   char semi_major[],       /* I: spheroid semi major axis of the image      */
   char semi_minor[],       /* I: spheroid semi minor axis of the image      */
   char parallel1[],        /* I: first projection standard parallel         */
   char parallel2[],        /* I: second projection standard parallel        */
   char longitude[],        /* I: projection longitude of the image          */
   char latitude[],         /* I: projection latitude of the image           */
   char false_easting[],    /* I: x coordinate of the origin for projection  */
   char false_northing[],   /* I: y coordinate of the origin for projection  */
   int proj_kilometers      /* I: flag indicating if projection units is in
                                  kilometers */

);

void polar_stereo
(
   struct DDR *ddr,         /* I/O: local DDR structure                      */
   char semi_major[],       /* I: spheroid semi major axis of the image      */
   char semi_minor[],       /* I: spheroid semi minor axis of the image      */
   char longitude[],        /* I: projection longitude of the image          */
   char latitude[],         /* I: projection latitude of the image           */
   char false_easting[],    /* I: x coordinate of the origin for projection  */
   char false_northing[],   /* I: y coordinate of the origin for projection  */
   int proj_kilometers      /* I: flag indicating if projection units is in
                                  kilometers */

);

int get_datum
(
   struct DDR *ddr,         /* I/O: local DDR structure                      */
   char datum[]             /* I: datum name retrieved from HDF logfile      */
);

int get_spheroid
(
   struct DDR *ddr,         /* I/O: local DDR structure                      */
   char spheroid[],         /* I: spheroid name retrieved from HDF logfile   */
   char sphere_radius[]     /* I: sphere radius                              */
);

#endif



