#include "spread.h"
#include "protos.h"
#include "util.h"

void filspec
(
    char hosin[][CMLEN],     /* I: input image host names                     */
    char hosout[],           /* I: output image host name                     */
    long num_inimages,       /* I: number of input images                     */
    long num_image_bands[][MAXBND + 1],/* I: band numbers in each input image */
    long (*out_img_bands)[MAXBND+1], /* O: band numbers for the output image  */
    long bands_per_image[],  /* I: number of bands for each image             */
    long *bands		     /* I: number of output image bands               */
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         15-Feb-85   M. Goldberg/GSFC      
   2.0         31-Aug-87   D. Akkerman/EDC
   3.0         29-Mar-89   K. Kunska		conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open input image files.
   Open the output image file.
   Update the ddr file.

**************************************************************************
*/

{

double	line_inc = 1.0,		/* line and sample increments for          */
	samp_inc = 1.0;		/* output image ddr                        */

long	access,			/* file access type                        */
	comb_flag = 0,		/* image combination flag                  */
	maxlines = 1,		/* maximum number of lines to be read      */
	n, 			/* loop index                              */
	opt[2],			/* option array for eopens                 */
	stat,			/* return status                           */
	type;			/* file type                               */

long    zero = 0;
long    one = 1;



/*
Open input image files.
-------------------------
*/
access = IREAD;
type = ELONG;
for (n = 0; n < num_inimages; n++)
   {
   stat = c_eopenr((struct FDESC **)&fdescin[n], hosin[n], num_image_bands[n], 
                   &window[n][SL], &window[n][SS], &window[n][NL], 
                   &window[n][NS], &access, &type, &zero, &maxlines);
   if (stat != E_SUCC )
      c_abort (&block, "Error opening input image", "spread-open", "spread",
               out_files, &num_outfiles);
   }

fdescin[++n] = 0;

/*
Open the output image file.
--------------------------
*/
access = IWRITE;

n = 0;
opt[0] = EBYTE;
opt[1] = 0;
type = ELONG;

n = 0;
stat = c_eopenr ((struct FDESC **)&fdescout, hosout, bands, &one, &one,
                 &window[0][NL], &window[0][NS], &access, &type, opt,&maxlines);

if (stat != E_SUCC )
   c_abort (&block, "Error opening output image", "spread-outopen", "spread",
               out_files, &num_outfiles);

for (n = 0; n < *bands; n++)
   out_img_bands[0][n] = n + 1;

/*
Update the ddr file.
--------------------
*/
c_upddr (&block, hosin, hosout, &num_inimages, window, num_image_bands, 
         bands_per_image, &line_inc, &samp_inc, bands, &comb_flag);

strcpy (out_files[num_outfiles++], hosout);

}
