#include "spread.h"
#include "protos.h"

void get_rec
(
    long rsize,		     /* I: max dimension of the runend array          */
    long point[],	     /* I: array of pointers to next elements in list */
    long *nextp, 	     /* I/O: pointer to the next record               */
    long *newrec	     /* I/O: pointer to the new record                */
)
/*
*****************************************************************************

NAME: GET_REC

PURPOSE: Removes the new record index value from a stack of available index 
         pointers.  Resets the value of next pointer to point to the next 
         index value.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         15-Feb-85   M. Goldberg/GSFC      
   2.0         31-Aug-87   D. Akkerman/EDC
   3.0         29-Mar-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   If the value of the next pointer exceeds the runend array dimension,
      Terminate with a fatal error message.
   Else
      Remove the newrec index value from a stack of available index pointers.  
      Reset the value of nextp to point to the next index value.

*****************************************************************************
*/

{
long    fatal_err = -1;                /* fatal error return status */

/* 
If the value of the next pointer exceeds the runend array dimension,
terminate with a fatal error message.
----------------------------------------------------------------------
*/

if (*nextp  ==  rsize-1)
   c_errmsg ("Internal array size exceeded", "spread-getrec", &fatal_err);

/*
Remove the newrec index value from a stack of available index pointers.  
Reset the value of nextp to point to the next index value.
------------------------------------------------------------------------
*/

*newrec = *nextp;
printf("returning value of %d\n", *newrec);
*nextp = point[*nextp];

}
