#include "spread.h"
#include "protos.h"

void keep
(
    long run,                   /* I: current run                             */
    long *point, 		/* I/O: pointers to next elements in list     */
    long *root, 		/* I/O: starting values for each line in code */
    long *prior, 		/* I/O: pointer to the prior record           */
    long base                   /* I: starting line for the encoding          */
)

/*
*****************************************************************************

NAME: KEEP

PURPOSE:  Keeps the designated run by resetting the prior record to 
          point to it.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         15-Feb-85   M. Goldberg/GSFC      
   2.0         31-Aug-87   D. Akkerman/EDC
   3.0         29-Mar-89   K. Kunska		conversion to 5.0


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   If this run has already been kept
      Return
   Else
      Reset the pointer from the root or the base.
      Reset the value of 'prior' to the run being kept.

*****************************************************************************
*/

{

long	null = 0;

/*
Run is passed as old or new, and is the index of the internal run-end record.
Prior is the previous record, which needs to point to the current record.  
If this is the first record, prior will not be defined, and the root will 
be used.  
Skip action of this routine if this run has already been kept.
-----------------------------------------------------------------------------
*/
if (run  ==  *prior) 
   return;

/*
Reset the pointer from the root or the base.
--------------------------------------------
*/
if ((root[base]  ==  run)  ||  (*prior  ==  null)) 
   {
   root[base] = run;
   }
else
   {
   point[*prior] = run;
   } 

/*
Reset the value of 'prior' to the run being kept.
-------------------------------------------------
*/
*prior = run;

}
