#ifndef PROTOS_H
#define PROTOS_H

void delrun
(
    long run,		    /* I: current run                                 */
    long *point,	    /* I/O: array of pointers to next elements in list*/
    long *nextp 	    /* I/O: pointer to the next record                */
);

void distance
(
    long target,		/* I: target class for distance search        */
    long radius, 		/* I: radius of search                        */
    long bckgnd,		/* I: background value on output image        */
    long beg_angl, 		/* I: beginning angle for restricted search   */
    long end_angl,		/* I: ending angle for restricted search      */
    long num_inimg,		/* I: total number of images                  */
    long num_img_band[][MAXBND+1],   /* I: band numbers for the input image   */
    long tot_inbands,		/* I: total number of bands                   */
    long bands_per_img[]	/* I: number of bands in each image           */
);

void filspec
(
    char hosin[][CMLEN],     /* I: input image host names                     */
    char hosout[],           /* I: output image host name                     */
    long num_inimages,       /* I: number of input images                     */
    long num_image_bands[][MAXBND + 1],/* I: band numbers in each input image */
    long (*out_img_bands)[MAXBND+1], /* O: band numbers for the output image  */
    long bands_per_image[],  /* I: number of bands for each image             */
    long *bands		     /* I: number of output image bands               */
);

void get_rec
(
    long rsize,		     /* I: max dimension of the runend array          */
    long point[],	     /* I: array of pointers to next elements in list */
    long *nextp, 	     /* I/O: pointer to the next record               */
    long *newrec	     /* I/O: pointer to the new record                */
);

void getpar
(
    char hos_in[][CMLEN], 	/* O: input image host name                  */
    char hos_out[][CMLEN], 	/* O: output image host name                 */
    long *target,		/* O: target class for distance search       */
    long *radius, 		/* O: radius of search                       */
    long mask_target[],		/* O: pixel target values for mask subcommand*/
    long mask_radius[],		/* O: buffer radius for mask subcommand      */
    long *bckgnd,		/* O: background value on output image       */
    long *beg_angl, 		/* O: beginning angle for restricted search  */
    long *end_angl, 		/* O: ending angle for restricted search     */
    long *mask, 		/* O: mask value                             */
    long quad_num[],		/* O: quadrant numbers                       */
    long *cmdopt,		/* O: subcommand option                      */
    long *dtype, 		/* O: data type                              */
    long *bands_per_img,	/* O: number of bands per image              */
    long (*img_flag)[2],	/* O: specified image/band options           */
    long img_limit,		/* O: maximum number of images possible      */
    long (*num_img_band)[MAXBND+1],/* O: stores band numbers for each image  */
    long *num_inimg,		/* O: number of input images                 */
    long *same_wnd,		/* O: same window size? 1=Y, 0=N             */
    long *tot_inband		/* O: total number of bands                  */
);

void keep
(
    long run,                   /* I: current run                             */
    long *point, 		/* I/O: pointers to next elements in list     */
    long *root, 		/* I/O: starting values for each line in code */
    long *prior, 		/* I/O: pointer to the prior record           */
    long base                   /* I: starting line for the encoding          */
);

void runend
(
    long rsize,			/* I: max dimension to runend data array      */
    long line, 		        /* I:current line being accessed during update*/
    long left,			/* I:pointer to the line boundary on left side*/
    long right,			/* I: pointer the line boundary on right side */
    long *end, 			/* O: position in line where code[] ends      */
    long *code,			/* O: codes for output pixel values           */
    long *point,		/* I/O: pointers to next elements in list     */
    long *nextp, 		/* I/O: next pointer                          */
    long nsmp,			/* I: number of samples from the input image  */
    long *root 			/* I/O: starting value in code and end for    */
);

void setup
(
    long radius, 		/* I: radius of search                        */
    long beg_angl, 		/* I: beginning angle for restricted search   */
    long end_angl, 		/* I: ending angle for restricted search      */
    long *nstart,             	/* O: starting indices for radial scans       */
    long *nfin, 		/* O: final indices for radial scans          */
    long *xoff,			/* O: consecutive x offsets for search        */
    long *yoff,			/* O: consecutive y offsets for search        */
    long *dist 			/* O: distances corresponding to x,y values   */
);

void smask
(
    long bckgnd,		/* I: assigned to background pxels in output  */
    long radius[],		/* I: search radius values                    */
    long target[],		/* I: user-selected target values             */
    long mask,			/* I: assigned to buffer pixels in output     */
    long quads[],		/* I: input quadrants to be searched          */
    long bands_per_img[],	/* I: number of bands for each input image    */
    long num_inimg,		/* I: total number of images                  */
    long tot_inbands,		/* I: total number of bands                   */
    long num_img_bands[][MAXBND+1]/* I: band numbers for the input image      */
);

#endif
