/* include files, global parameters, and definitions for SPREAD      */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct	PARBLK	block;                 /* TAE parameter block     */
struct  DDR	ddr;                   /* ddr                     */

FILE	*fd;			       /* temp file descriptor    */

long	*inbuf;                        /* image input buffer      */
long	*outbuf;                       /* image output buffer     */

char    out_files[2][CMLEN];           /* output file names       */

long	fdescin[MAXIMG+1];             /* input file descriptors  */
long	fdescout;                      /* output file2 descriptor */

long	min_max;                       /* minmax option flag      */
long 	msgtim;                        /* message interval        */
long	num_outfiles;                  /* number of output files  */

extern long	curent;                /* current entity           */
extern long	curcat;                /* current category         */


long	tar_cnt;	 	       /* number of pixel values specified   */
 				       /* by the user for -MASK              */
long	qua_cnt;		       /* number of quadrants specified by   */
                                       /* the user for -MASK                 */

long	window[MAXIMG][4];             /* input image dimensions (SL SS NL NS)*/

void main
(void)
{

/*
*****************************************************************************

NAME: SPREAD

PURPOSE: Generates buffer zones of contour distances or mask values
         around target pixels in an image. Subcommand -DIST creates
         an output image whose buffer zones contain the distances of
         the original image pixels from the nearest target pixel in 
         the input image. Subcommand -MASK generates an output image 
         whose buffer zones contain mask values surrounding the 
         target pixels in the input image.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         15-Feb-85   M. Goldberg/GSFC      
   2.0         31-Aug-87   D. Akkerman/EDC
   3.0         29-Mar-89   K. Kunska		conversion to 5.0
   4.0	       20-Jul-89   K. Kunska            fixed a bug in coversion of 
                                                the FORTRAN goto
   5.0         18-Aug-89   J. Marmon            Changes to run on the pn9050
   6.0		3-Apr-90   K. Kunska            Fixed min-max problem
   6.1         31-Aug-92   D. Etrheim		Use LAS name on complt. Cleanup
					of include file.  Rename getrec to 
					get_rec to avoid multiply defined 
					symbol on vms.
   7.0         23-Nov-99   D. Hames     Remove MAXNS and replace with ddr.ns
					Modify smask to initialize rsize using
					number of lines. This is in order to 						support images > 2gig. Add prototypes.
					SRF #893

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to initialize tae and retrieve user-supplied parameters.
   Call FILSPEC to set up file specifications.
   If the user-specified subcomand = DIST, 
      call DISTANCE to perform minimum distance search.
   If the user-specified subcommand = MASK, 
      call SMASK to create masked buffer areas around each pixel in 
      the specified target class.
   Close all images.
   Determine the minimum and maximum values of the output image.
   Write the min, max and the validity flag values to the ddr file.
   Generate the history file and print the completion message.

*****************************************************************************
*/

char	
	hos_in[MAXIMG][CMLEN], 	/* input image host name                      */
	hos_out[MAXIMG][CMLEN]; /* output image host name                     */

static double	amax[MAXBND],	/* maximum pixel value                        */
	amin[MAXBND];		/* minimum pixel value                        */
static
long	
	bands_per_img[MAXIMG],	/* number of bands per image                  */
	bckgnd, 		/* background value on output image           */
	beg_angl,		/* beginning angle for restricted search      */
	cmdopt,			/* subcommand option: 0 = DIST, 1 = MASK      */
	dtype,			/* input image data type                      */
	end_angl, 		/* ending angle for restricted search         */
	i, 			/* loop counter and array index               */
	img_flag[MAXIMG][2],	/* specifies image/band options               */
	img_limit,		/* maximum number of images                   */
	mask,			/* mask value                                 */
	mask_radius[50],	/* maximum radius of the buffer zone          */
	mask_target[50],	/* pixel target values for search             */
	num_inimg,		/* total number of input images               */
	out_bands,		/* number of output bands                     */
	outwin[4],		/* output window specifications               */
	quad_num[4],		/* quadrants to be processed for a user       */
  				/* specified angular search                   */
	radius, 		/* radius of search                           */
	same_wnd,		/* same window size: 1=Y, 0=N                 */
	target, 		/* target class for distance search           */
	tot_inband;		/* total number of bands                      */

long
	*num_img_band,		/* band numbers in each input image   */
	*out_img_band; 		/* bands numbers for the output image */

long	one = 1;
long    inf_err = 1;                   /* info error return status  */

img_limit = MAXIMG;
min_max = FALSE;

/*
Dynamic storage allocation
--------------------------
*/
num_img_band = (long *) malloc(MAXIMG * (MAXBND +1) * sizeof(long));
if (num_img_band == NULL) {
    c_errmsg("Error allocating dynamic memory", "spread-alloc", &inf_err);
   }

out_img_band = (long *) malloc(MAXIMG * (MAXBND +1) * sizeof(long));
if (out_img_band == NULL) {
    c_errmsg("Error allocating dynamic memory", "spread-alloc", &inf_err);
   }


/*
Initialize tae and retrieve user-supplied parameters.
-----------------------------------------------------
*/
getpar (hos_in, hos_out, &target, &radius, mask_target, 
        mask_radius, &bckgnd, &beg_angl, &end_angl, &mask, quad_num, &cmdopt,
        &dtype, bands_per_img, img_flag, img_limit, (long (*)[257])num_img_band,	&num_inimg, &same_wnd, &tot_inband);

/*
More dynamic storage allocation
-------------------------------
*/
inbuf = (long *) malloc(ddr.ns * sizeof(long));
if (inbuf == NULL) {
    c_errmsg("Error allocating dynamic memory", "spread-alloc", &inf_err);
   }

outbuf = (long *) malloc(ddr.ns * sizeof(long));
if (outbuf == NULL) {
    c_errmsg("Error allocating dynamic memory", "spread-alloc", &inf_err);
   }


/*
Set up file specifications.
---------------------------
*/
out_bands = tot_inband;
filspec (hos_in, (char *)hos_out, num_inimg,(long (*)[257])num_img_band,
         (long (*)[257])out_img_band, bands_per_img, &out_bands);


/*
If the user specified subcomand = DIST, call DISTANCE to perform 
minimum distance search.
If the user-specified subcommand = MASK, call SMASK to create masked 
buffer areas around each pixel in the specified target class.
------------------------------------------------------------------------
*/
if (cmdopt == 1)
   distance (target, radius, bckgnd, beg_angl, end_angl, num_inimg,
             (long (*)[257])num_img_band, tot_inband, bands_per_img); 
else
   smask (bckgnd, mask_radius, mask_target, mask, quad_num, bands_per_img, 
           num_inimg, tot_inband, (long (*)[257])num_img_band); 

/*
Close all images.
------------------
*/
for (i = 0; i < num_inimg; i++)
   c_eclose((struct FDESC **)&fdescin[i]);

c_eclose((struct FDESC **)&fdescout);

/* 
Determine the minimum and maximum values of the output image.
Write the min, max and the validity flag values to the ddr file.
-----------------------------------------------------------------
*/
if (min_max)
   {
   outwin[SL] = outwin[SS] = 1;
   outwin[NL] = window[0][NL];
   outwin[NS] = window[0][NS];
   c_minmax (hos_out, &one, (long (*)[4])outwin, (long (*)[257])out_img_band,	
             &out_bands, amin, amax);
   c_svmnmx ((char *)hos_out, &out_bands, amin, amax, "YES");
   }

/*
free the dynamic memory
-----------------------
*/
free(num_img_band);
free(out_img_band);
free(inbuf);
free(outbuf);

/*
Generate the history file and print the completion message.
------------------------------------------------------------
*/
c_hstout (&block, (char *)hos_out, hos_in, &num_inimg, " ");
c_complt ("SPREAD", out_files, &num_outfiles);
}
