#include "stat2tab.h"
#include <string.h>
#include "util.h"
#include "protos.h"

void covar
(
    struct PARBLK *vblock,	/* TAE parameter block			*/
    struct TAB_DEF *tab, 	/* LT file structure		        */
    char ftype[],		/* STAT file data type			*/
    char outfil[][CMLEN],	/* New LT file name			*/
    long *nfile,		/* number of LT files			*/
    char hstat[],		/* STAT host file name			*/
    long level[],		/* Covariance data level		*/
    char *classval[TAE_VAL+1],	/* Class name				*/
    char *siteval[TAE_VAL+1],	/* Site name				*/
    long *msgtim		/* Time between processing messages	*/
)
/*
**********************************************************************

NAME: COVAR                            

PURPOSE: To read the covariance matrix from the statistics file and process
	 the valid data.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1.0         11/86      K. Gacke     original development
    1.1          1/87      K. Gacke     new LT file format
    2.0          9/87      K. Gacke	fixed an infinite loop when processing
					covariance matrix and the covariance
					matrix does not exist at the image level
    2.1		 1/90      J. Marmon    v5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
       Open statistics file
       Read data from stats file until end of file
  	  If successful read from stats file then
	     If data was read then
	        Process data
	     else goto next node
	  else if no more data then
	     go to next node
	  else if site does not exist then
	     go to next node
	  else a fatal error encountered
      Close files.

************************************************************************
*/
{
IMPORT long rec;
long    shape[9];		/* STAT shape buffer for data types	    */
FILE *  fd;			/* STAT file descriptor			    */
long    access;			/* Type of access to open file with         */
long	cur_class;		/* current class in stats file being read   */
long	cur_data;		/* current data in stats file		    */
long	cur_site;		/* current site in stats file being read    */
long	two = 2;		/* used for initializing the time msg.	    */
long 	nclass;			/* Total number of classes		    */
long	status;			/* Return message from functions	    */
long	total_covar = 0;	/* Total number of covariance matrix in file*/
long    len;			/* length of the string data type	    */
char	dtype[26];		/* data type of STAT file		    */
char	msg[256];		/* Buffer for error message		    */
long    nonfatal = 1;

/* 
*  Open STAT file to read from            
*/
access = IREAD;
status = c_stopen(&fd,hstat,&access,&nclass);	
if (status != E_OPOLD)
   {
   c_errmsg("Error opening statistics file","stat2tab-open",&nonfatal);
   close_tab(tab);
   c_clean(vblock,"STAT2TAB",outfil,nfile);
   }
rec = 1;
c_inittm("STAT2TAB","Processed","covariance matrix","","",&two,&two,msgtim,&two);
c_startm();
for (rec=0,cur_class=0,cur_site=0,cur_data=1; cur_class<=nclass; )
  { 
  status = c_stnode(&fd,&cur_class,&cur_site,&cur_data,dtype,shape);
  switch (status)
    {
/* 
*  Info for one data item has been returned   
*/
    case E_SUCC:		
      len = (long) strlen(dtype);
      if (strncmp(dtype,ftype,len) == 0)
         {
         rec++;
         covar_data(vblock,fd,tab,outfil,nfile,&cur_class,&cur_site,
		    level,classval,siteval,shape[2],&total_covar);
         cur_data = 1;
	 cur_class == 0 ? cur_class++ : cur_site++;
         }
      else
         cur_data++;
      break;
/* 
*  No more data items in node                 
*/
    case E_NODT:		
      cur_class == 0 ? cur_class++ : cur_site++;
      cur_data = 1;
      break;
/* 
*  Current site does not exist                
*/
    case E_NOSI:	
      cur_site = 0;
      cur_class++;
      break;
/*
*  Fatal error has happened program terminated
*/
    default:			
      sprintf(msg,"%s %d %s %d.  Return status = %d",
              "Error retrieving STAT node information for class",cur_class,
              "site",cur_site,status);
      c_errmsg(msg,"stat2tab-read ",&nonfatal);
      close_tab(tab);
      c_clean(vblock,"STAT2TAB",outfil,nfile);
      break;
    }
  }
c_stoptm();
close_tab(tab);				/* Close Labeled Table file   */
c_stclse(&fd);					/* Close STAT file            */
return;
}
