#include "stat2tab.h"
#include "util.h"
#include "protos.h"

int get_class_site
(
    struct PARBLK *vblock,		/* TAE parameter block		*/
    FILE *fd,				/* file descriptor		*/
    struct TAB_DEF *tab,		/* LT master file structure	*/
    struct TAB_DEF *subtab,		/* LT subfile structure		*/
    char outfil[][CMLEN],		/* New LT file name(s)		*/
    long *nfile,			/* number of LT files		*/
    char dtype[],			/* data type of STATs file	*/
    long *cur_class,			/* current class 		*/
    long *cur_site,			/* current site			*/
    char *classval[TAE_VAL+1],		/* input class names from param */
    char *siteval[TAE_VAL+1],		/* input site names from param	*/
    char class_name[],			/* class name from STATs file	*/
    char site_name[]			/* site name from STATs file	*/
)
/*
**********************************************************************

NAME: GET_CLASS_SITE                            

PURPOSE:  Returns the current CLASS and SITE names from a STATS file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1.0         11/86      K. Gacke     original development
    1.1          1/87      K. Gacke     new LT file format
    2.0          9/87      K. Gacke	fixed an infinite loop when processing
					covariance matrix and the covariance
					matrix does not exist at the image level
    2.1  	 1/90      J. Marmon    v5.0 conversion 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      If current class is zero then
	 assign class and site names to blanks 
	 return a true value
      else
	 get current class name from stats file
	 If no classes then
	    set class flag to true
         else
	    search class list for class name
	       If found then
		  set class flag to true
	 If site = zero then  	 
	    set site name to blank
	    return class flag
	 else
	    get current site name
	    If no sites then
	       set site flag to true
	    else
	       search site list for site name
	       If found then
		  set site flag to true
      return (class flag && site flag)

************************************************************************
*/
{
long    shape[9];			/* shape buffer			*/
long	class_flag;			/* if class is found or not	*/
long	index;				/* loop counter			*/
long	site_flag;			/* if site is found or not	*/
long	site_index = 0;			/* site index counter		*/
long	status;				/* return measage from calls	*/
long    nonfatal = 1;

if (*cur_class == 0)
   {
   strcpy(class_name,"");
   strcpy(site_name,"");
   return(TRUE);
   }
else
   {
   status = c_stget(&fd,dtype,cur_class,&site_index,shape,class_name);
   if (status != E_SUCC)
      {
      c_errmsg("Error reading STAT class name","stat2tab-nameatr",&nonfatal);
      close_tab(tab);
      if (*nfile == 2)
         close_tab(subtab); 
      c_clean(vblock,"STAT2TAB",outfil,nfile);
      }
   *(class_name + shape[2]) = '\0';
   upcase(class_name);
   if (classval[0] == '\0')
      class_flag = TRUE;
   else
      for (index=0,class_flag=FALSE; classval[index]!='\0'; index++)
         {
         if (strcmp(classval[index],class_name) == 0)
            {
            class_flag = TRUE;
            break;
            }
         }
   }
if (*cur_site == 0)
   {
   strcpy(site_name,"");
   return(class_flag);
   }
else
   {
   status = c_stget(&fd,dtype,cur_class,cur_site,shape,site_name);
   if (status != E_SUCC)
      {
      c_errmsg("Error reading STAT site name","stat2tab-nameatr",&nonfatal);
      close_tab(tab);
      if (*nfile == 2)
         close_tab(subtab); 
      c_clean(vblock,"STAT2TAB",outfil,nfile);
      }
   *(site_name + shape[2]) = '\0';
   upcase(site_name);
   if (siteval[0] == '\0')
      site_flag = TRUE;
   else
      for (index=0,site_flag=FALSE; siteval[index]!='\0'; index++)
         {
         if (strcmp(siteval[index],site_name) == 0)
            {
            site_flag = TRUE;
            break;
            }
         }
   }
return((class_flag) && (site_flag));
}
