#include "stat2tab.h"
#include <string.h>
#include "util.h"
#include "protos.h"

void poly
(
    struct PARBLK *vblock,		/* TAE parameter block		*/
    struct TAB_DEF *tab,		/* LT file structure		*/
    char *ftype,			/* STAT file type		*/
    char *hstat,			/* host stat file name		*/
    char outfil[][CMLEN],		/* new LT file names		*/
    long *nfile,			/* number of LT files		*/
    char *classval[TAE_VAL+1],		/* STAT class names to process  */
    char *siteval[TAE_VAL+1],		/* STAT site names to process	*/
    char *comment,			/* LT comment line		*/
    char *hsubtab,			/* subfile name			*/
    long *msgtim			/* Time between processing msg  */
)
/*
**********************************************************************

NAME: POLY                            

PURPOSE: To read the polygon data from the statistic file and process
	 the valid data.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1.0         11/86      K. Gacke     original development
    1.1          1/87      K. Gacke     new LT file format
    2.0          9/87      K. Gacke	fixed an infinite loop when processing
					covariance matrix and the covariance
					matrix does not exist at the image level
    2.1  	 1/90      J. Marmon    v5.0 conversion 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
       Open statistic file
       Read data from stats file until end of file
	  If successful read then
	     If data exists then
		Process data
	     else go to next node
	  else if no more data in node then
 	     go to next node
	  else if site does not exist then
 	     go to next node
	  else Fatal error, terminate program.
       Close files.

************************************************************************
*/
{
IMPORT long rec;		/* Status number of polygons completed    */
 
struct TAB_DEF subtab;		/* LT file structure subfile		  */
long    shape[9];		/* Shape vector for data		  */
FILE *  fd;			/* STAT file descriptor			  */
long    access;			/* access code for STAT file		  */
long	cur_class;		/* Current class in stats file		  */
long	cur_data;		/* Current data being processed		  */
long	cur_site;		/* Current site in stats file		  */
long	zero= 0;		/* sets the # of subfiles to 0 in subfile */
long    one = 1;		/* the number one used in openning LT     */
long    two = 2;		/* used in initiating time		  */
long 	nclass;			/* Total number of classes in STAT file   */ 
long	status;			/* Return message from function calls     */
long	total_poly = 0;		/* Total number of polygons in stats file */
long    len;			/* String length of data type		  */
char	dtype[26];		/* Data type from stat file		  */ 
long    nonfatal = 1;


/*
*   Open Labeled Table subfile for polygon data
*/
subtab.ftype = "REGXY";
subtab.fdesc = comment;
subtab.nsubs = zero;
open_tab(&subtab,hsubtab,one);
poly_header(tab, &subtab);

/* 
*  Open STAT file             
*/
access = IREAD;
status = c_stopen(&fd,hstat,&access,&nclass);	
if (status != E_OPOLD)
   {
   c_errmsg("Error opening statistics file","stat2tab-open",&nonfatal);
   close_tab(tab);
   close_tab(&subtab); 
   c_clean(vblock,"STAT2TAB",outfil,nfile);
   }

rec = 1;
c_inittm("STAT2TAB","Processed","POLY record","","",&two,&two,msgtim,&two);
c_startm();
for (rec=0,cur_class=1,cur_site=1,cur_data=1; cur_class<=nclass;)
  {
  status = c_stnode(&fd,&cur_class,&cur_site,&cur_data,dtype,shape);
  switch (status)
    {
/* 
*  Info for one data item has been returned   
*/
    case E_SUCC:		
      len = (long) strlen(dtype);
      if (strncmp(dtype,ftype,len) == 0)
         {
	 rec++;
         poly_data(vblock,fd,tab,&subtab,outfil,nfile,dtype,&cur_class,
		   &cur_site,classval,siteval,shape[2],&total_poly);
         cur_data = 1;
         cur_site++;
         }
      else
         cur_data++;
      break;
/* 
*  No more data items in node                 
*/
    case E_NODT:		
      cur_data = 1;
      cur_site++;
      break;
/* 
*  Current site does not exist                
*/
    case E_NOSI:		
      cur_site = 1;
      cur_class++;
      break;
/* 
*  Current class does not exist (error)       
*/
    default:			
      c_errmsg("Error retrieving STAT node information","stat2tab-read",
	       &nonfatal);
      close_tab(tab);
      close_tab(&subtab); 
      c_clean(vblock,"STAT2TAB",outfil,nfile);
      break;
    }
  }
c_stoptm();
close_tab(tab);					/* Close Labeled Table file   */
close_tab(&subtab);				/* Close Labeled Table subfile*/
c_stclse(&fd);					/* Close STAT file            */
return;
}
