#include "stat2tab.h"
#include "util.h"
#include "protos.h"

void poly_data
(
    struct PARBLK *vblock,	/* TAE parameter block			    */
    FILE *fd,			/* file descriptor			    */
    struct TAB_DEF *tab,	/* Definition structure of LT file	    */
    struct TAB_DEF *subtab,	/* Definition structure of LT file	    */
    char outfil[][CMLEN],	/* new LT file names			    */
    long *nfile,		/* number of LT files			    */
    char dtype[],		/* STAT file data type			    */
    long *cur_class,		/* current class in STAT file		    */
    long *cur_site,		/* current site in STAT file		    */
    char *classval[TAE_VAL+1],	/* class name in STAT file		    */
    char *siteval[TAE_VAL+1],	/* site name in STAT file		    */
    long npts,			/* number of item associated with the data  */
    long *total_poly		/* total number of polygons in STAT file    */
)
/*
**********************************************************************

NAME: POLY_DATA                            

PURPOSE: 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1.0         11/86      K. Gacke     original development
    1.1          1/87      K. Gacke     new LT file format
    2.0          9/87      K. Gacke	fixed an infinite loop when processing
					covariance matrix and the covariance
					matrix does not exist at the image level
    2.1  	 1/90      J. Marmon    v5.0 conversion 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      If class and site names are found then
	 write polygon LT file header
	 allocate dynamic buffer
	 get shape vector and buffer for current class & site
	 write all the points for current class & site
      free buffer

************************************************************************
*/
{
static long *buf = NULL;		/* dynamic allocation buffer	   */
static long buf_size = 0;		/* size of dynamic allocation	   */
long    shape[9];			/* shape vector for the data	   */
long	index;				/* loop control counter		   */
long	status;				/* Return status of function call  */
char	class_name[LIN_SIZ];		/* class name returned from STAT file */
char	site_name[LIN_SIZ];		/* site name returned from STAT file  */
long    nonfatal = 1;

/*
*  Get the current class & site from the STAT file
*/
if (get_class_site(vblock,fd,tab,subtab,outfil,nfile,"NAME",cur_class,cur_site,
		   classval,siteval,class_name,site_name))
   {
/*
*  Write the LT header info 
*/
   ++(*total_poly);
   put_i4_field(tab,*total_poly,FIELD_SEP1);
   put_string_field(tab,class_name,FIELD_SEP1,TRUE);
   put_string_field(tab,site_name,RECORD_SEP,TRUE);

   if (buf_size < npts * 2)
      {
      buf_size = npts * 2;
      if (buf != NULL)
         free(buf);
      if ((buf = (LONG *) calloc(buf_size,sizeof(long))) == NULL)
         {
         c_errmsg("Error allocating polygon data buffer","stat2tab-alloc",
		  &nonfatal);
         close_tab(tab);
         close_tab(subtab); 
         c_clean(vblock,"STAT2TAB",outfil,nfile);
         }
      }
/* 
*  Read data from STATS file
*/
   status = c_stget(&fd,dtype,cur_class,cur_site,shape,buf);
   if (status != E_SUCC)
      {
      c_errmsg("Error reading polygon data","stat2tab-fdata",&nonfatal);
      close_tab(tab);
      close_tab(subtab); 
      c_clean(vblock,"STAT2TAB",outfil,nfile);
      }
/* 
*  Write the points into the LT file
*/
   for (index=0; index<(npts*2); index++)
      {
      put_i4_field(subtab,*total_poly,FIELD_SEP1);
      put_i4_field(subtab,buf[index],FIELD_SEP1);
      index = index + 1;
      put_i4_field(subtab,buf[index],RECORD_SEP);
      }
   }
free(buf);
return;
}
