#include "stat2tab.h"
#include <stdlib.h>
#include <string.h>
#include "util.h"
#include "protos.h"

/******************************************************************************

NAME:	STAT2TAB

FUNCTION:
	STAT2TAB is an application program which converts a LAS STAT file
	to a Labeled Table file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         11/86      K. Gacke     original development
    1.1          1/87      K. Gacke     new LT file format
    2.0          9/87      K. Gacke	fixed an infinite loop when processing
					covariance matrix and the covariance
					matrix does not exist at the image level
    2.1 	 1/90      J. Marmon    v5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

LANGUAGE:	C

ALGORITHM 
      Input STATS file is openned
      If POLYgon data in file then
  	 open POLYgon LT file
	 open POLYgon LT subfile
	 If current statistics node contains polygon data then
	    If Class name of current statistics node matches CLASSNAM then
	       If Site name of current stats node matches SITENAM then
 		  transfer data to LT file
      else Covariance data
	 Open Covariance LT file
	 If current stats node contains covaraince matrix then   
	    If current stats level matches (image, class, or site) then
	       If current stats level is class or site level and the class
			  name matches the parameter CLASSNAM then 
  		  If the current level is site level, and the site name
			  matches the parameter SITENAM then 
		     Transfer the data to the LT file

******************************************************************************/

int main
(void)
{
struct PARBLK vblock;		/* TAE parameter block                        */
struct TAB_DEF tab;		/* Pointer to LT file structure		      */
struct VARIABLE *val;		/* Pointer to a variable within TAE vblock    */
char	hstat[CMLEN];		/* Buffer for STAT host file name             */
char	htab[CMLEN];		/* Buffer for Labeled table host file name    */
char	outfil[2][CMLEN];	/* Created LT file names		      */
char	*hsubtab;		/* Buffer for Labeled Table subfile name      */
char	*classval[TAE_VAL+1];	/* STAT class names to be processed           */
char	*siteval[TAE_VAL+1];	/* STAT site names to be processed            */
char	comment[CMLEN];		/* Description of Labeled Table               */
char	ftype[CMLEN];		/* Type of data in the Labeled Table file     */
long    access;			/* access to LT file  read, write	      */
long	index;			/* Loop index counter                         */
long    nfile;			/* Number of files created for completion msg */
long	level[3];		/* Level of STAT file to be processed         */
long 	msgtim;			/* Number of seconds between processing msg.  */
long	std_out;		/* Fortran logical unit number to standard out*/
long	zero = 0;		/* Value of zero                              */
long	one  = 1;		/* Value of one				      */
long    fatal = -1;

/*
*  Get input parameters and subcommand and open LT master file
*/
c_inlas(&vblock,&std_out);
if (c_getnam(&vblock,"INSTAT",&zero, hstat,"stat", &access)== E_FAIL)
   c_errmsg("Fatal error encountered","stat2tab-fatal",&fatal);
access = IWRITE;

if (c_getnam(&vblock, "OUTLT", &zero, htab, "lt", &access) == E_FAIL)
   c_errmsg("Fatal error encountered","stat2tab-fatal",&fatal);

val = p_find(&vblock,"_SUBCMD");
strcpy(ftype,SVAL(*val,0));

val = p_find(&vblock,"COMMENT");
if (val->v_count > 0)
   strcpy(comment,SVAL(*val,0));
else
   strcpy(comment,"");

tab.fdesc = comment;

val = p_find(&vblock,"CLASSNAM");
for (index=0; index<val->v_count; index++)
   {
   classval[index] = SVAL(*val,index);
   upcase(classval[index]);
   }
classval[index] = '\0';

val = p_find(&vblock,"SITENAM");
for (index=0; index<val->v_count; index++)
   {
   siteval[index] = SVAL(*val,index);
   upcase(siteval[index]);
   }
siteval[index] = '\0';
val = p_find(&vblock,"$MSGTIM");
msgtim = IVAL(*val,0);
/*
*  Loads the master LT file name in to the array.  If polygon data is 
*  processed then the subfile name will be added to the array.  This 
*  is done so that when "clean" is called only one variable is passed 
*  into "clean".
*/
strcpy(outfil[0],htab);
/*
*  Process polygon data
*/
if (strcmp(ftype,"POLY") == 0)			
   {
   tab.ftype = "REGATT";
   hsubtab = (char *)make_lt_name(htab,1);
   tab.subfile[0] = hsubtab;
   strcpy(outfil[1],hsubtab);
   tab.nsubs = 1;
   nfile = 2;
   open_tab(&tab,htab,1);
   strcpy(ftype,"POLYGON");
   poly(&vblock,&tab,ftype,hstat,outfil,&nfile,classval,siteval,comment,hsubtab,
	&msgtim);
   }
/*
*  Process covariance data
*/
else if (strcmp(ftype,"COVAR") == 0)
   {
   strcpy(ftype,"COVARIANCE_MATRIX");
   val = p_find(&vblock,"STATSTYP");
/*
*  Check which level is to be processed
*/
   if (val->v_count == 0)
      level[IMAGE_LEV] = level[CLASS_LEV] = level[SITE_LEV] = TRUE;
   else
      {
      level[IMAGE_LEV] = level[CLASS_LEV] = level[SITE_LEV] = FALSE;
      for (index=0; index<val->v_count; index++)
         {
         if (strcmp(SVAL(*val,index),"IMAGE") == 0)
            level[IMAGE_LEV] = TRUE;
         else if (strcmp(SVAL(*val,index),"CLASS") == 0)
            level[CLASS_LEV] = TRUE;
         else if (strcmp(SVAL(*val,index),"SITE") == 0)
            level[SITE_LEV] = TRUE;
         }
      }
   tab.ftype = "COVAR";
   tab.nsubs = 0;
   nfile = 1;
   open_tab(&tab,htab,1);
   covar(&vblock,&tab,ftype,outfil,&nfile,hstat,level,classval,siteval,&msgtim);
   }

c_complt("STAT2TAB",htab,&one);

return EXIT_SUCCESS;

}
