#include "stats.h"

double stats_determ
(
    double *matrix[],		/* I: Determinant to be computed              */
    long norder			/* I: Order of the determinant                */
)

/******************************************************************************

NAME:	determ

FUNCTION:
	Calculate the determinant of a square matrix

*******************************************************************************/

{
static double *array[MAXBND],	/* local temporary array to do calculations on*/
	ret_determ,		/* return value accumulator                   */
	save;			/* temporary intermediate value to            */

long	i,			/* loop counters                              */
	j,			/* loop counters                              */
	k,			/* loop counters                              */
        size, k1;		/* Temporary variable used as loop initial val*/

/* Allocate temporary storage
   -------------------------- */
size = (long)((totbnd * totbnd) * sizeof(double));
arr_matrix = (double *) malloc (size);
for (i=0,j=0; i < totbnd; i++, j+= totbnd)
  array[i] = &arr_matrix[j];

/* Copy input matrix
   ----------------- */
for (i = 0; i < norder; i++)
   for (k = 0; k < norder; k++)
      array[i][k] = matrix[i][k];

ret_determ = 1.0;
for (k = 0; k < norder; k++)
   {
/*
   Interchange columns if diagonal element is zero
   ----------------------------------------------- */
   if (array[k][k] == 0.0)
      {
      for (j = k; j < norder; j++)
         if (array[k][j] != 0.0)
            break;

      if (j == norder)
        {
         ret_determ = 0.0;
         return (ret_determ);
        }
      else
         {
         for (i = k; i < norder; i++)
            {
            save = array[i][j];
            array[i][j] = array[i][k];
            array[i][k] = save;
            }
         ret_determ *= -1.0;
         }
      }
/*
   Subtract row k from lower rows to get diagonal matrix
   ----------------------------------------------------- */
   ret_determ *= array[k][k];
   if (k - norder < 0)
      {
      k1 = k + 1;
      for (i = k1; i < norder; i++)
         for (j = k1; j < norder; j++)
            array[i][j] -= (array[i][k] * array[k][j] / array[k][k]);
      }
   }
   return (ret_determ);
}

