#include "stats.h"
#include "util.h"

void dostats
(
    char mask[],	/* I: Indicates which pixels are inside          */
    long ns,		/* I: Number of pixels in line for a site        */
    char covar[],	/* I: Covariance option                          */
    long *k,		/* I/O: Current number of points                 */
    long *shist[],	/* I/O: Current histogram                        */
    double *scov,	/* I/O: Current covariances                      */
    double smean[],	/* I/O: Current mean                             */
    double sum[],	/* I/O: Current sum of values                    */
    long maxh[],	/* I/O: Current max value                        */
    char subc[],	/* I: Subcommand being used                      */
    long lnk		/* I: Current line number                        */
)

/******************************************************************************

NAME:	dostats

FUNCTION:
	Updates values for histograms, covariances, means,sum of
	values, max. value and number points as data for a site are
	accessed.

ALGORITHM (FLOW):
	DO for the number of pixels in this site window
	   IF the pixel is within the polygon THEN
	      IF the -CUT subcommand is being used THEN
	         DO for the number of bands given
	            IF this pixel should be cut for this band THEN
	               Undo    (Entire multi-pixel is cut)
	            END IF
	         END DO
	      END IF
	      DO for the number of bands
	         Accumulate Mean, Covariance, and Histogram values
	      END DO
	   END IF
	END DO
	Update histogram maximums

*******************************************************************************/

{

float	*ftemp,			/* Temp variable to hold a pixel value        */
	val,			/* Temp variable to hold a pixel value        */
	yval;			/* Temp variable to hold a pixel value        */

static double  
	xmax[MAXBND],		  /*    maximum pixel value        */
        xmin[MAXBND];              /*    minimum pixel value        */  

long	fell_out = FALSE,	/* Flag indicating that it broke out of loop  */
	i,			/* Loop variable                              */
	ival,			/* Temporary variable                         */
	j,			/* Loop variable                              */
	kk,			/* Array index into covar array               */
	lb,			/* Loop variable                              */
	nb;			/* Loop variable                              */

char    mval0 = 0;
double	prod,			/* Current cross product                      */
	xnum;			/* Current value of k/(k-1)                   */

if (lnk == 1)
   c_inmnmx (&in_type, &totbnd, xmin, xmax);
/*
   Do for the number of pixels in the RECTANGULAR site window
   ---------------------------------------------------------- */
for (i = 0; i < ns; i++)
   {
   if (mask[i] != mval0)
      {
      (*k)++;
      if (*k != 1)
         xnum = ((double) *k) / ((double) ((*k) - 1));
      kk = 0;
      fell_out = FALSE;
      if (strcmp (subc, "CUT") == 0)
         {
/*
   For value cutting, test all multi-pixels before computing ANY
   Statistics
   ------------------------------------------------------------- */
         for (nb = 0; nb < totbnd; nb++)
            {
            ftemp = (float *) &thebuf[(offarray[nb] + i) * sizeof (float)];
            val = *ftemp;
            if (cut.bands[nb] == 1 && val <= cut.hival[nb] &&
                val >= cut.loval[nb])
               {
               (*k)--;
               fell_out = TRUE;
               break;
               }
            }
         }
/*
   Update MEAN and HIST for each band
   ---------------------------------- */
      if (!fell_out)   /* did not break out of above loop */
         {
	 for (nb = 0; nb < totbnd; nb++)
	    {
            ftemp = (float *) &thebuf[(offarray[nb] + i) * sizeof (float)];
            val = *ftemp;
	    if (val < xmin[nb])
	       xmin[nb] = val;
	    if (val > xmax[nb])
	       xmax[nb] = val;
	    sum[nb] += val;
	    if (*k > 0)
	       smean[nb] = sum[nb] / *k;
	    ival = (long)(val * scales.sca[nb] + scales.off[nb] + 0.5);
	    if (ival + 1 > maxh[nb])
	       maxh[nb] = ival + 1;
	    shist[nb][ival]++;
/*
   Update covariances
   ------------------ */
	    if (*k > 1 && (covar[0] == 'Y'))
               {
	       for (lb = 0; lb <= nb; lb++)
		  {
                  ftemp = (float *) &thebuf[(offarray[lb] + i) * sizeof (float)];
                  yval = *ftemp;
		  prod = (yval - smean[lb]) * (val - smean[nb]);
		  scov[kk] = scov[kk] + prod * xnum;
		  kk++;
		  }
               }
	    }
         }
      }
   }       /* End of large for loop */

for (j = 0; j < totbnd; j++)
   {
   shist[j][256] = (long) xmin[j];
   shist[j][257] = (long) xmax[j];
   }
}
