#include "stats.h"
#include "util.h"

void getpar
(
    char hstat[],		/* O: Buffer for stat file host name          */
    char hosnm[][CMLEN],	/* O: input host names                        */
    char taenm[][CMLEN],	/* O: input TAE names                         */
    long *nmrimg,		/* O: Number of input images                  */
    long bands_num[][MAXBND+1],	/* O: Band specifications for each input image*/
    long nbands[],		/* O: Number of bands per input image         */
    char mean[],		/* O: Option for mean        (c*4)            */
    char covar[],		/* O: Option for covariance  (c*4)            */
    char hist[],		/* O: Option for histograms  (c*4)            */
    long *nc,			/* O: Number of classes requested             */
    char opt[],			/* O: Option for class or site list  	      */
    char subc[],		/* O: Subcommand to process                   */
    char runtyp[],		/* O: Type of job running(batch,interactive..)*/
    long *msgtim,		/* O: Time processing message time interval   */
    long *num_samps             /* O: Total number of samples in input image  */
)

/******************************************************************************

NAME:	getpar

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       15-Mar-82    C. Gurney
    2.0          Mar-1987  N. Deodhar/SAR   New LAS & conversion to C
    3.0          Dec-1988  B. Allam         Retrocode
    4.0		 Mar-1989  B. Allam         Changed parameter names
    5.0         Jan-1990   B. Ailts         Fixed call to c_minmax

FUNCTION:
	Receives parameters from TAE process definition file.

	Restrictions- number of input images less-than or equal-to 100.
		      number of classes less-than or equal-to MAXCLS.

******************************************************************************/
{
float	hval[50],		/* Array of user entered high values          */
	lval[50],		/* Array of user entered low values           */
	tmax,			/* Temporary max value                        */
	tmin;			/* Temporary min value                        */

static double	maxarr[MAXBND],	/* Max values from minmax file                */
	maxval[2],		/* Max value recieved from minmax             */
	minarr[MAXBND],		/* Min values from minmax file                */
	minval[2];		/* Min value recieved from minmax             */

long	access,			/* Access for stats file                      */
	bnums[50],		/* Band numbers entered by user               */
	dtype[MAXIMG],		/* Data type of the input images              */
	imglim=MAXIMG,		/* Maximum munber of images to be processed   */
	i,			/* Loop index                                 */
	j,			/* Loop index                                 */
	n,			/* Loop index                                 */
	num_bnums,		/* Number of band numbers entered             */
	num_hval,		/* Number of high vals entered                */
	num_lval,		/* Number of low vals entered                 */
	same,			/* Flag indicating if images have same window */
	status,			/* Returned status code                       */
	tbands[2];		/* Band number for calling minmax             */
static long 	valid[MAXBND],	/* Validity flag for min max values           */
	qflag[MAXIMG][2],	/* Flag indicating if subwindows specified    */
	window[MAXIMG][4];	/* Window specification for input images      */
                                           
struct	VARIABLE *var;		/* Pointer into variable block                */

char	msg[ERRLEN];		/* Message buffer for c_errmsg                */
long    one=1;
long    fatal_err = -1;
long    inf_err = 1;
long    zero=0;

/* Get inputs and Subcommand string
   -------------------------------- */
status = c_getin (&vblock, "IN", &zero, taenm, hosnm, nmrimg, window,
              bands_num, nbands, &totbnd, &imglim, &same, qflag);
              
if (status != E_SUCC)
   c_errmsg ("Error accessing input file names", "stats-fatal", &fatal_err);
if (totbnd > MAXBND)
   c_errmsg ("Too may input image bands specified", "stats-fatal", &fatal_err);

/*
get the data type of input image
--------------------------------
*/
c_getddr (hosnm[0], &ddr);  
in_type = ddr.dtype;                           
*num_samps = ddr.ns;

/* Get subcommand
   -------------- */
var = p_find (&vblock, "_SUBCMD");
strcpy (subc, SVAL (*var, 0));

/* Get input stats file name
   ------------------------- */
access = IREAD;
status = c_getnam (&vblock, "INSTAT", &zero, hstat, "stat", &access);
if (status != E_SUCC)
   c_errmsg ("Error accessing input stats file", "stats-fatal", &fatal_err);

/* Obtain option for means, covariances, and histograms
   ---------------------------------------------------- */
var = p_find (&vblock, "MEANFLG");
strcpy (mean, SVAL (*var, 0));

var = p_find (&vblock, "COVARFLG");
strcpy (covar, SVAL (*var, 0));

var = p_find (&vblock, "HISTFLG");
strcpy (hist, SVAL (*var, 0));

/* Obtain number of classes
   ------------------------ */
var = p_find (&vblock, "NCLASS");
if (!(*var).v_count)
  *nc = 0;
else
  *nc = IVAL (*var, 0);

if (*nc > MAXCLS)
   {
   sprintf (msg, "Number of classes must be <= %d", MAXCLS);
   c_errmsg (msg, "stats-fatal", &fatal_err);
   }

/* Get implicit global variable $runtype
   -------------------------------------*/ 
var = p_find (&vblock, "$RUNTYPE");
strcpy (runtyp,SVAL(*var,0));

if ((strcmp (runtyp, "BATCH") == 0) || (strcmp (runtyp, "ASYNC") == 0))
   {
   *nc = 0;
/*   lc[0] = 'N';
   ls[0] = 'N';
*/
   }

/* Obtain class and site list options
   ---------------------------------- */
var = p_find (&vblock,"OPTION");
if (!(*var).v_count)
   strcpy(opt," ");
else
   strcpy (opt,SVAL(*var,0));

/*
   Get processing line message time interval and min-max flag
   ----------------------------------------------------------
*/
var = p_find (&vblock, "$MSGTIM");
*msgtim = IVAL (*var, 0);

if (strcmp (subc, "CUT") == 0)
   {
   var = p_find (&vblock, "BANDS");
   num_bnums = (*var).v_count;

   for (i = 0; i < num_bnums; i++)
      {
      bnums[i] = IVAL (*var, i);
      }

   var = p_find (&vblock, "HIGHVAL");
   num_hval = (*var).v_count;
   for (i = 0; i < num_hval; i++)
      hval[i] = RVAL (*var, i);

   var = p_find (&vblock, "LOWVAL");
   num_lval = (*var).v_count;
   for (i = 0; i < num_lval; i++)
      lval[i] = RVAL (*var, i);

   if (num_bnums != num_hval || num_bnums != num_lval)
      c_errmsg ("Mismatch between number of NBANDS, HIVAL, and LOWVAL values",
               "stats-fatal", &fatal_err);
/*
   Test and order the band numbers and cutting values
   --------------------------------------------------
*/
   for (j = 0; j < num_hval; j++)
      {
      if (lval[j] > hval[j])
         c_errmsg ("Invalid cutting range, LOWVALS must be less than HIVALS",
                  "stats-fatal", &fatal_err);
      if (bnums[j] > totbnd)
         {
         sprintf (msg, "Invalid BANDS specification... band %d not found",
                  bnums[j]);
         c_errmsg (msg, "stats-fatal", &fatal_err);
         }
      cut.bands[bnums[j] - 1] = 1;
      cut.loval[bnums[j] - 1] = lval[j];
      cut.hival[bnums[j] - 1] = hval[j];
      }
   }
/*
   See that that no window was specified for any of the images
   -----------------------------------------------------------
for (i = 0; i < *nmrimg; i++)
   if (qflag[i][1])
      {
      c_errmsg ("Error - No window allowed", "stats-fatal", &fatal_err);
      }
*/
/*
   Make sure all images are the same size
   --------------------------------------
*/
if (!same)
   c_errmsg ("All images must be same size", "stats-fatal", &fatal_err);

master.imsiz[0] = window[0][NL];
master.imsiz[1] = window[0][NS];

master.xoff = 1;
master.yoff = 1;

/*
   Get the data types of the images
   --------------------------------
*/
if ((c_gettyp (hosnm, nmrimg, dtype)) != E_SUCC)
   c_errmsg ("Error getting data types for input images", "stats-fatal", &fatal_err);

/* Get the input min and max if histogramming is requested
   ------------------------------------------------------- */
if (hist[0] == 'Y')
   {
   n = 0;
   for (i = 0; i < *nmrimg; i++)
      {
     c_minmax ((char (*)[256])hosnm[i], &one, (long (*)[4])window[i], 
               (long (*)[257])bands_num[0], &nbands[i],
   	     minarr,maxarr);                            
      for (j = 0; j < nbands[i]; j++)
         {
         if (valid[j] == 0)
            {
            tmin = (float) minarr[j];
            tmax = (float) maxarr[j];
            }
         else if (dtype[i] == EBYTE)
            {
            minarr[i] = 0;
            maxarr[i] = 255;
            tmin =0.;
            tmax = 255.;
            }
         else
            {
            if (n == 0)
               c_errmsg ("MINMAX is being called prior to computing histograms",
                        "stats-info", &inf_err);
            tbands[0] = bands_num[i][j];
            tbands[1] = 0;
            c_minmax ((char (*)[256])hosnm[i], &one, (long (*)[4])window[i],
		       (long (*)[257])tbands, &one,minval, maxval);
            minarr[i] = minval[0];
            maxarr[i] = maxval[0];
            tmin = (float)minval[0];
            tmax = (float)maxval[0];
            }

         if (tmin >= 0.0 && tmax <= 255.0)
            {
            scales.sca[n] = 1.0;
            scales.off[n] = 0.0;
            }
         else if (tmax - tmin <= 255.0)
            {
            scales.sca[n] = 1.0;
            scales.off[n] = -1 * tmin;
            }
         else
            {
            scales.off[n] = -1 * 255.0 * (tmin / (tmax - tmin));
            scales.sca[n] =  255.0 / (tmax - tmin);
            }
         n++;
         }                 
      }
   }
}
