#include "stats.h"

void indices
(
    long nc,		/* I: No. of classes specified by user             */
    long ncx,		/* I: Number of classes to be analyzed             */
    long icno[],	/* O: Set of class indices                         */
    long *isno[MAXCLS],	/* O: Set of site indices for each class           */
    long nsites[MAXCLS]	/* O: Number of sites to be analyzed for a class   */
)

/******************************************************************************

NAME:	indices

FUNCTION:
	receives user specified class and site names and
	retrieves their associated index numbers from statistics
	file.

	Exit states-
	   3 errors for class name. restart
	   3 errors for site name. restart
	   stat i/o error for sites. check stats file.

	Restrictions- number of sites must be less-than or equal-to MAXSIT.

*******************************************************************************/

{
char	inbuf[80],		/* input character buffer                     */
	numstr[80],		/* Response from user to a question           */

	sname[STNAME],		/* Site name                                  */
	stname[STNAME];		/* Class name                                 */

long	classno,		/* Class index number                         */
	count,			/* No. of user errors entering class name     */
	done,			/* Do loop exit flag                          */
	i,			/* Loop variable                              */
	ic,			/* Class index number                         */
	ierr,			/* Status of call to statistics file          */
	is,			/* Site index number                          */
	j,			/* Loop variable                              */
	large_done,		/* Do loop exit flag                          */
	ns,			/* Number of sites per class                  */
	num_sites,		/* Number of sites in a class                 */
	scount,			/* No. of user errors entering site name      */
	shape[9],		/* Statistics data descriptor                 */
	t_err;			/* status from t_read                         */
long    fatal_err = -1;
long    inf_err = 1;
long    zero=0;


/*
   Loop through classes
   --------------------
*/
for (i = 0; i < ncx; i++)
   {
   count = 0;
   if (nc != 0)
      {
/*
   Not all classes being used => prompt user for name and
   check against stats file
   ------------------------------------------------------
*/
      done = FALSE;
      do {
         c_errmsg ("_Enter class name :", "stats-inq", &inf_err);

         strcpy (stname, "");
         t_read (stname, (int *)&t_err);


         if (c_stfdcl ((FILE **)&sdesc, stname, &ic) == 0)
            done = TRUE;
         else
            {
/*
   Class name not found. User is given 3 chances to enter valid name.
   ------------------------------------------------------------------
*/
            count++;
            if (count == 3)
               c_errmsg ("3 errors for class name. restart", "stats-cname", &fatal_err);
            else
               c_errmsg ("Class not found. Total 3 chances. Try again",
                        "stats-warn", &inf_err);
            }
         } while (!done);
/*
   Write class index # into list of class indices
   ----------------------------------------------
*/
      icno[i] = ic;
/*
   Prompt user for # of sites and read from terminal
   -------------------------------------------------
*/
      large_done = FALSE;
      do {
         c_errmsg ("_Enter # of sites for this class :", "stats-inq", &inf_err);

         t_read (inbuf, (int *)&t_err);
	 sscanf (inbuf, "%d", &nsites[i]);
/*
   Check requested # of sites vs. # in file
   ----------------------------------------
*/
         ierr = c_stget ((FILE **)&sdesc, "NSITES", &ic, &zero, shape, 
			  (unsigned char *)&ns);
         if (ierr != E_SUCC)
         c_errmsg ("Stat i/o error for sites. Check stats file",
                  "stats-nsites", &fatal_err);
         if (nsites[i] <= ns)
            large_done = TRUE;
         else
            {
            c_errmsg ("_Class does not have this many sites", "stats-info", &inf_err);
            c_errmsg ("_Do you wish to use all available sites? (YES/NO) :",
                     "stats-inq", &inf_err);

            done = FALSE;
            do {
               t_read (numstr, (int *)&t_err);
               if (numstr[0] != 'Y' && numstr[0] != 'N' &&
                   numstr[0] != 'y' && numstr[0] != 'n')
                  c_errmsg ("_Respond YES or NO :", "stats-inq", &inf_err);
               else
                  done = TRUE;
               } while (!done);

            if (numstr[0] == 'Y' || numstr[0] == 'y')
               {
               nsites[i] = ns;
               large_done = TRUE;
               }
            }
         } while (!large_done);
/*
   If not all sites are being used then prompt user for site names
   ---------------------------------------------------------------
*/
      if (nsites[i] == 0)
         nsites[i] = ns;

      if (nsites[i] != ns)
         {
         c_errmsg ("_Enter site names , 1 per line :", "stats-inq", &inf_err);

         for (j = 0; j < nsites[i]; j++)
            {
            scount = 0;
            strcpy (sname, "");

            done = FALSE;
            do {
               t_read  (sname, (int *)&t_err);
               if (c_stfdsi ((FILE **)&sdesc, stname, sname, &ic, &is) == 0)
                  done = TRUE;
               else
                  {
/*
   Site name not found. User given 3 chances to enter valid name.
   --------------------------------------------------------------
*/
                  scount++;
                  if (scount == 3)
                     c_errmsg ("3 errors for site name. restart",
                              "stats-sname", &fatal_err);
                  else
                     {
                     c_errmsg ("Site not found. Total 3 chances. Try again",
                              "stats-warn", &inf_err);
                     c_errmsg ("_Enter site name again :", "stats-inq", &inf_err);
                     }
                  }
               } while (!done);
/*
   Write site index # into list of site indices
   -------------------------------------------- */
            isno[j][i] = is;
            }
         }
/*
   All sites are being used. Site indices incremented sequentially
   --------------------------------------------------------------- */
      else
         for (j = 0; j < ns; j++)
            isno[j][i] = j + 1;
/*
   All classes and all sites requested.
   Class and site indices incremented sequentially
   ----------------------------------------------- */
      }  /* End of large if statement */
   else
      {
      icno[i] = classno = i + 1;
      if (c_stget ((FILE **)&sdesc, "NSITES", &classno, &zero, shape, 
		  (unsigned char *)&num_sites) != E_SUCC)
         c_errmsg ("Stat i/o error for sites. Check stats file",
                  "stats-nsites", &fatal_err);
      nsites[i] = num_sites;
      for (j = 0; j < nsites[i]; j++)
         isno[j][i] = j + 1;
      }
   }     /* End of large for loop */
}

