#include "stats.h"

long insde
(
    long xverts[],		/* I/O: X coordinates of vertices             */
    long yverts[],		/* I/O: Y coordinates of vertices             */
    long nverts,		/* I: Number of vertices                      */
    long xpos1,			/* I: X coord of 1st intersection pnt         */
    long xpos2,			/* I: X coord of 2nd intersection pnt         */
    long ypos			/* I: Y coordinate of test point              */
)

/******************************************************************************

NAME:	insde

FUNCTION:
	Determine whether a point is interior to a polygon or not.

	Comments- This routine is copied from stats program.
             It is modified to pass xpos as r*4 instead of i*4.
             It is also added the checks for (1) polygon vertex and
                (2) polygon side line

	Determines whether a point is interior to a polygon or not.
	The winding number of the test point is computed: consider the
	collection of vectors formed by connecting the test point to
	each of the vertices. Add up all the rotation angles between the
	vertices (sequence dependent).  If the sum is smaller than PI,
	the point is exterior.

******************************************************************************/
{
long	i,			/* Loop variable                              */
	iverts;			/* number of verticies counting first twice   */

float	angnew,			/* calculated angle                           */
	angsum,			/* Sum of rotation angles between vertices    */
	xpos;			/* X-coord of two intersecton points. It      */
				/* is the x coordinate of test point          */
/*                                            
   Check if two intersections are the same point or not.
   If they are the same point, this point is either a
   polygon vertex or on the polygon side
   ------------------------------------------------------
*/
if (xpos1 == xpos2)
   return (TRUE);

xpos = (xpos1 + xpos2) / 2.0;
iverts = nverts + 1;
xverts[nverts] = xverts[0];
yverts[nverts] = yverts[0];

if (iverts < 3)
   return (FALSE);
angsum = 0.0;

for (i = 1; i < iverts; i++)
   {
   angnew = angle (xpos, ypos, xverts[i-1], yverts[i-1],
                       xverts[i], yverts[i]);
/*
   Check for the polygon line.
   If the angle is PI, the mid-point of two intersections is
   lying on the line connecting two vertices.  This means that it is
   on a polygon side line and this line is horizontal.
   ------------------------------------------------------------------
*/
   if ((fabs(angnew - PI) < .00001))
      return (TRUE);
   angsum += angnew;
   }

if (fabs (angsum) >= PI)
   return (TRUE);
return (FALSE);
}
