#ifndef PROTOS_H
#define PROTOS_H

void classtat
(
    char mean[],	/* I: Option for mean                            */
    char covar[],	/* I: Option for covariance                      */
    char hist[],	/* I: Option for histogram                       */
    long icno[],	/* I: Set of class indices                       */
    long ncx		/* I: Number of classes                          */
);

double stats_determ
(
    double *matrix[],		/* I: Determinant to be computed              */
    long norder			/* I: Order of the determinant                */
);

void dostats
(
    char mask[],	/* I: Indicates which pixels are inside          */
    long ns,		/* I: Number of pixels in line for a site        */
    char covar[],	/* I: Covariance option                          */
    long *k,		/* I/O: Current number of points                 */
    long *shist[],	/* I/O: Current histogram                        */
    double *scov,	/* I/O: Current covariances                      */
    double smean[],	/* I/O: Current mean                             */
    double sum[],	/* I/O: Current sum of values                    */
    long maxh[],	/* I/O: Current max value                        */
    char subc[],	/* I: Subcommand being used                      */
    long lnk		/* I: Current line number                        */
);

void dyn_alloc
(
    double *cov[],		/* O: Covariance matrix                     */  
    long *shist[]		/* O: Site histograms                       */
);

void getpar
(
    char hstat[],		/* O: Buffer for stat file host name          */
    char hosnm[][CMLEN],	/* O: input host names                        */
    char taenm[][CMLEN],	/* O: input TAE names                         */
    long *nmrimg,		/* O: Number of input images                  */
    long bands_num[][MAXBND+1],	/* O: Band specifications for each input image*/
    long nbands[],		/* O: Number of bands per input image         */
    char mean[],		/* O: Option for mean        (c*4)            */
    char covar[],		/* O: Option for covariance  (c*4)            */
    char hist[],		/* O: Option for histograms  (c*4)            */
    long *nc,			/* O: Number of classes requested             */
    char opt[],			/* O: Option for class or site list  	      */
    char subc[],		/* O: Subcommand to process                   */
    char runtyp[],		/* O: Type of job running(batch,interactive..)*/
    long *msgtim,		/* O: Time processing message time interval   */
    long *num_samps             /* O: Total number of samples in input image  */
);

void indices
(
    long nc,		/* I: No. of classes specified by user             */
    long ncx,		/* I: Number of classes to be analyzed             */
    long icno[],	/* O: Set of class indices                         */
    long *isno[MAXCLS],	/* O: Set of site indices for each class           */
    long nsites[MAXCLS]	/* O: Number of sites to be analyzed for a class   */
);

long insde
(
    long xverts[],		/* I/O: X coordinates of vertices             */
    long yverts[],		/* I/O: Y coordinates of vertices             */
    long nverts,		/* I: Number of vertices                      */
    long xpos1,			/* I: X coord of 1st intersection pnt         */
    long xpos2,			/* I: X coord of 2nd intersection pnt         */
    long ypos			/* I: Y coordinate of test point              */
);

void lnames
(
    long nclass,	/* I: Number of classes in statistics file       */
    char list[]		/* I: Indicates whether site listing is required */
);

void openfiles
(
    char hosnm[][CMLEN],	/* I: Buffer for input host names             */
    long nmrimg,		/* I: Number of images                        */
    long bands_num[][MAXBND + 1],/*I: Band number for each band               */
    long nbands[],		/* I: Number of bands for each image          */
    long *sl,		     	/* I: Image window to be read                 */
    long *ss,			/* I: Image window to be read                 */
    long *nl,			/* I: Image window to be read                 */
    long *ns			/* I: Image window to be read                 */
);

float angle
(
    double x0,			/* I: x coordinate of pivot point             */
    long y0,			/* I: y coordinate of pivot point             */
    long x1,			/* I: x coordinate of first vertex            */
    long y1,			/* I: y coordinate of first vertex            */
    long x2,			/* I: x coordinate of second vertex           */
    long y2			/* I: y coordinate of second vertex           */
);

void setarea
(
    char mean[],		/* I: Option for mean                         */
    char covar[],	        /* I: Option for covariance                   */
    char hist[],		/* I: Option for histogram                    */
    long ncx,			/* I: Number of classes to be used            */
    long icno[],		/* I: Set of class indices                    */
    long *isno[MAXCLS],		/* I: Set of site indices                     */
    long nsites[],		/* I: Number of sites for a class             */
    char runtyp[]		/* I: Type of job running(batch,interactive..)*/
);

void sitestat
(
    char mean[],	/* I: Option for mean                            */
    char covar[],	/* I: Option for covariance                      */
    char hist[],	/* I: Option for histogram                       */
    char hosnm[][CMLEN],/* I: Buffer for input host names                */
    long nmrimg,	/* I: Number of images                           */
    long bands_num[][MAXBND + 1],/* I: Band number for each imag         */
    long nbands[],	/* I: Number of bands for each image             */
    long icno[],	/* I: Set of class indices                       */
    long *isno[MAXCLS],	/* I: Set of site indices                        */
    long nsites[],	/* I: Number of sites for a class                */
    long ncx,		/* I: Number of classes to be used               */
    long mdone[],	/* I: Indicates mean written for site            */
    long cdone[],	/* I: Indicates covariance for a site            */
    long hdone[],	/* I: Indicates histogram written for site       */
    char subc[],	/* I: Subcommand to process                      */
    long msgtim,	/* I: Time processing message time interval      */
    long num_samps	/* I: Total number of samples in the input image */
);

void statchk
(
    char hstat[],	/* I: Buffer for stat file host name             */
    long nc,		/* I: Number of classes user requested           */
    long *nclass,	/* I: Number of classes in stats  file           */
    long *ncx		/* O: Number of classes to be processed          */
);

void verts
(
    long ic,		/* I: Class index in statistics file             */
    long is,		/* I: Site index in statistics file              */
    long size[]		/* O: Co-ordinates of site bounding rectangle    */
);

void xsect
(
    long xverts[],	/* I/O: x coordinates of vertices          	      */
    long yverts[],	/* I/O: y coordinates of vertices          	      */
    long nverts,	/* I: number of vertices                 	      */
    long line,		/* I: line number                        	      */
    long xsects[],	/* I: x coordinates of intersections     	      */
    long *nxsect	/* O: number of intersections            	      */
);
#endif
