#include "stats.h"

float angle
(
    double x0,			/* I: x coordinate of pivot point             */
    long y0,			/* I: y coordinate of pivot point             */
    long x1,			/* I: x coordinate of first vertex            */
    long y1,			/* I: y coordinate of first vertex            */
    long x2,			/* I: x coordinate of second vertex           */
    long y2			/* I: y coordinate of second vertex           */
)

/******************************************************************************

NAME:	angle

FUNCTION:
	Finds rotation angle between two given vertices relative
	to given pivot point.  Returns rotation angle in radians.

	Finds the rotation angle between the points (x1,y1) and
	(x2,y2) relative to the central pivot point (x0,y0)

	Calculated by using the ratio of the cross
	product (proportonal to sine(angle)) to the
	dot product (proportional to cosine(angle))

	The sense of the rotation (+ or -) is important.

*******************************************************************************/
{
float	cross,			/* cross product                              */
	dot,			/* dot product                                */
	angle,			/* return value accumulator                   */
	xx1,			/* difference in x for point 1 and vertex     */
	xx2,			/* difference in x for point 2 and vertex     */
	yy1,			/* difference in y for point 1 and vertex     */
	yy2;			/* difference in y for point 2 and vertex     */

xx1 = x1 - x0;
yy1 = (float)(y1 - y0);
xx2 = x2 - x0;
yy2 = (float)(y2 - y0);

cross = xx1 * yy2 - xx2 * yy1;
dot   = xx1 * xx2 + yy1 * yy2;
/*
   Check for 90 degree angle
   -------------------------
*/
if (fabs (dot) == 0.0 )
   angle = 1.5707963;
else
   {
   angle = atan (fabs (cross / dot));
   if (dot < 0.0)
      angle = PI - angle;
   }

if (cross < 0.0)
   angle *= -1.0;
return (angle);
}
