#include "stats.h"
#include "util.h"

void setarea
(
    char mean[],		/* I: Option for mean                         */
    char covar[],	        /* I: Option for covariance                   */
    char hist[],		/* I: Option for histogram                    */
    long ncx,			/* I: Number of classes to be used            */
    long icno[],		/* I: Set of class indices                    */
    long *isno[MAXCLS],		/* I: Set of site indices                     */
    long nsites[],		/* I: Number of sites for a class             */
    char runtyp[]		/* I: Type of job running(batch,interactive..)*/
)

/******************************************************************************

NAME:	setarea

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       15-Mar-82    C. Gurney
    2.0          Mar-1987  N. Deodhar/SAR   New LAS & conversion to C

FUNCTION:
	To establish which sites need to access image data and
	enter bounding rectangles of these sites into an array.

*******************************************************************************/

{
long	dcnt,			/* Site counter                               */
	done,			/* Flag indicating that we are done prompting */
	dummy[258],		/* Histogram                                  */
 	found,			/* Flag indicating that it broke out of loop  */
	i,			/* Loop variable                              */
	ic,			/* A class index                              */
	icerr,			/* Error status from stat I/O function        */
	ierr,			/* Error status from stat I/O function        */
	is,			/* A site index                               */
	j,			/* Loop variable                              */
	k,			/* Site number                                */
	l,			/* Length of user's response                  */
        shape[9],               /* stats data descriptor for histogram      */
	size[4],		/* Site bounding rectangle                    */
	t_err;			/* status from t_read                         */

static long	
	cshape[9],		/* stats data descriptor for covariance       */
	mshape[9];		/* stats data descriptor for mean             */
static  double  *cfile;
static  double	cmean[MAXBND];	/* Mean or covariance                         */
long    t_size;

char	ans[80],		/* User's response                            */
	cname[STNAME],		/* Class name                                 */
	outbuf[ERRLEN],		/* Error message output buffer                */
	sname[STNAME];		/* Site name                                  */
long    fatal_err = -1;
long    inf_err = 1;
long    zero=0;

/*
  Allocate storage for temporary buffers
  --------------------------------------
*/
t_size = (long)((totbnd) * (totbnd+1)/2 * sizeof (double));
cfile = (double *) malloc (t_size);

if (cfile == NULL) 
   {
   c_errmsg("Error allocating dynamic memory","stats-alloc",&inf_err);
   c_errmsg("Fatal error encountered","stats-fatal",&fatal_err);
   }

/*
   Loop through classes.
   --------------------- */
found = FALSE;
ierr = 1;
for (i = 0; i < ncx; i++)
   {
   ic = icno[i];
/*
   Loop through sites.
   ------------------- */
   for (j = 0; j < nsites[i]; j++)
      {
      is = isno[j][i];
/*
   Set flag for mean if mean requested and no mean in stats
   file for this site.
   -------------------------------------------------------- */
      if (mean[0] == 'Y')
         {
         ierr = c_stget ((FILE **)&sdesc, "MEAN_VECTOR", &ic, &is, mshape,
                         (unsigned char *)cmean);

         if (ierr == 0)
            {
            found = TRUE;
            break;
            }
         }
/*
   Set flag for covariance if covariance/correlation requested and no
   covariance/correlation in stats file.
   ------------------------------------------------------------------ */
      if (covar[0] == 'Y')
         {
         ierr = c_stget ((FILE **)&sdesc, "COVARIANCE_MATRIX", &ic, &is, 
                         cshape, (unsigned char *)cfile);
         icerr = c_stget ((FILE **)&sdesc, "CORRELATION_MATRIX", &ic, &is, 
                          cshape, (unsigned char *)cfile);
                                            
         if (ierr == 0 || icerr == 0)
            {
            found = TRUE;
            break;
            }
         }
/*
   See if histograms are present at this site.
   ------------------------------------------- */
      if (hist[0] == 'Y')
         {
         ierr = c_stget ((FILE **)&sdesc, "HIST001", &ic, &is, shape, 
                         (unsigned char *)dummy);

         if (ierr == 0)
            {
            found = TRUE;
            break;
            }
         }
      }
   if (found)             /* broke out of inner loop */
      break;
   }
/*
   Get bounding rectangles of site and write into array area.
   ---------------------------------------------------------- */
k = 0;
for (i = 0; i < ncx; i++)
   {
   dcnt = 0;
   ic = icno[i];

   strcpy (cname, "");
   ierr = c_stget ((FILE **)&sdesc,"NAME",&ic,&zero,shape,
                    (unsigned char *)cname);

   for (j = 0; j < nsites[i]; j++)
      {
      is = isno[j][i];
      k++;
      verts (ic, is, size);
      if (size[SS] < 1 || size[SL] < 1 || 
          size[NS] > master.imsiz[1] || size[NL] > master.imsiz[0] || 
          size[SS] > master.imsiz[1] || size[SL] > master.imsiz[0])
         {
         dcnt++;

         strcpy (sname, "");
         ierr = c_stget ((FILE **)&sdesc, "NAME", &ic, &is, shape, 
                        (unsigned char *)sname);

         sprintf (outbuf, "Polygon vertex for Class: %s, Site: %s is outside image area",
                  cname, sname);
         c_errmsg (outbuf, "stats-vert", &fatal_err);
         isno[j][i] = 0;
         k--;
         dsite[dcnt-1][ic-1] = j+1;
         }
      }
   }
/*
   End condition for area array.
   ----------------------------- */
if (k == 0)
   c_errmsg ("No sites within image bounds", "stats-nosit", &fatal_err);
/*
   If data is present, prompt the user whether or not to overwrite.
   If batch or asynchronous processing, do not prompt - but overwrite
   ------------------------------------------------------------------ */
if (found)
   {
   c_errmsg ("Input statistics file already contains data", "stats-warn", &inf_err);
   done = FALSE;
   do {
      done = TRUE;
      if (strcmp (runtyp, "INTERACTIVE") == 0)
         {
         c_errmsg ("_Do you wish to overwrite this data? (Yes/No) :",
                  "stats-inq", &inf_err);

         t_read  (ans, (int *)&t_err);
         l = (long)strlen(ans);
         c_low2up (ans, &l);
         if (l == 0)
            done = FALSE;
         else if (ans[0] == 'N')
            c_errmsg ("STATS halted by user with no updates to statistics file",
                     "stats-abort", &fatal_err);
         else if (ans[0] == 'Y')
            {
            done = TRUE;
            }
         else 
           {
            c_errmsg ("***** ERROR *****", "stats-warn", &inf_err);
            done = FALSE;
           }
         }    /* if interactive processing */
      } while (!done);
   }
   free(cfile);
}
