#include "stats.h"
#include "util.h"

void statchk
(
    char hstat[],	/* I: Buffer for stat file host name             */
    long nc,		/* I: Number of classes user requested           */
    long *nclass,	/* I: Number of classes in stats  file           */
    long *ncx		/* O: Number of classes to be processed          */
)

/******************************************************************************
 
NAME:	statchk

FUNCTION:
	Opens statistics file; checks number of classes in file
	against user's request.

	Exit states- unable to open stats file. restart
	             too many classes requested. restart

	Comments- If user requests more classes than exist he will be 
	          prompted for action (continue using all classes or abort).
	          if he chooses to abort, prog will exit with "too many
	          classes requested"

*******************************************************************************/

{
char	go_on[4];		/* Does the user wish to go on                */


long	access,			/* stopen access code                         */
	done,			/* whether we are done asking for response    */
	il,			/* error status                               */
	t_err;			/* status from t_read                         */
long    fatal_err = -1;
long    inf_err = 1;


/*
   Open STATS file.
   ---------------- */
access = IUPDATE;
if (c_stopen ((FILE **)&sdesc, hstat, &access, nclass) != E_OPOLD)
   c_errmsg ("Unable to open stats file. restart", "stats-sopen", &fatal_err);
/*
   Check number of classes against user's request
   ---------------------------------------------- */
if (nc > *nclass)
   {
   c_errmsg ("Number of classes requested > the number in the file",
            "stats-info", &inf_err);
   c_errmsg ("_Do you wish to proceed using all classes? (YES/NO) :", 
            "stats-inq", &inf_err);

   done = FALSE;
   do {
      t_read (go_on,(int *)&t_err);
      il = (long)strlen(go_on);
      c_low2up (go_on, &il);

      if (il == 0)
         c_errmsg ("_Respond with YES or NO :", "stats-inq", &inf_err);
      else if (strncmp (go_on, "YES", il) == 0)
         {
         nc = 0;
         done = TRUE;
         }
      else if (strncmp (go_on, "NO", il) == 0)
         c_errmsg ("Too many classes requested. restart", "stats-nclass", &fatal_err);
      else
         c_errmsg ("_Respond with Y or N :", "stats-inq", &inf_err);
      } while (!done);
   }
/*
   Establish actual number of classes to be used
   --------------------------------------------- */
*ncx = nc;
if (nc == 0)
   *ncx = *nclass;
return;
}
