#include "stats.h"

void verts
(
    long ic,		/* I: Class index in statistics file             */
    long is,		/* I: Site index in statistics file              */
    long size[]		/* O: Co-ordinates of site bounding rectangle    */
)
			/* in co-ords of input image                  */

/******************************************************************************

NAME:	verts

FUNCTION:
	Returns site bounding rectangle in terms of input image
	coordinates.

	Exit states-
	   Max # site vertices > 5000. Simplify site shapes.

	Restrictions- The number of polygon vertices must be less-than or
	              equal-to 5000
PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       15-Mar-82    C. Gurney
    2.0          Mar-1987  N. Deodhar/SAR   New LAS & conversion to C
    3.0          DEC-90    W. Xia/STX       Fix xyvert
******************************************************************************/

{
long	ierr,			/* Status code from Stat I/O                  */
	maxx,			/* maximum x value of vertices                */
	maxy,			/* maximum y value of vertices                */
	minx,			/* minimum x value of vertices                */
	miny,			/* minimum y value of vertices                */
	m, n,			/* loop counter                               */
	nl,			/* number of lines in the rectangle           */
	ns,			/* number of pixels per line in rectangle     */
	nv,			/* number of vertices                         */
	shape[9];		/* Statistics data descriptor                 */
static long xyvert[MAX_VERTS];	/* x + y co-ordinates of site vertices        */
				/* in co-ordinates of master image            */
long    fatal_err = -1;

/* Retrieve vertices from stats file.
   ---------------------------------- */
ierr = c_stget((FILE **)&sdesc,"POLYGON",&ic,&is,shape,(unsigned char *)xyvert);
if (ierr != E_SUCC)
   c_errmsg("Fatal error encountered","stats-fatal",&fatal_err);
if (shape[2] > MAX_VERTS)
   c_errmsg ("Max # site vertices > 5000. Simplify site shapes",
            "stats-maxvert", &fatal_err);

minx = 999999;
maxx = 0;
miny = 999999;
maxy = 0;

/* Find min. and max. x and y coordinates.
   --------------------------------------- */
nv = shape[2];
for (n = 0,m = 0; n < nv; n++)
   {
   if (xyvert[m] < minx)
      minx = xyvert[m];
   if (xyvert[m] > maxx)
      maxx = xyvert[m];
   if (xyvert[m+1] < miny)
      miny = xyvert[m+1];
   if (xyvert[m+1] > maxy)
      maxy = xyvert[m+1];
   m = m+2;
   }
/*
   Subtract x and y offsets from master image. Write out
   bounding rectangle in form required for image I/O.
   -----------------------------------------------------
*/
size[SL] = miny - master.yoff + 1;
size[SS] = minx - master.xoff + 1;
nl       = maxy - master.yoff + 1;
ns       = maxx - master.xoff + 1;
size[NL] = nl   - size[SL]    + 1;
size[NS] = ns   - size[SS]    + 1;

}
