#include "stats.h"

void xsect
(
    long xverts[],	/* I/O: x coordinates of vertices          	      */
    long yverts[],	/* I/O: y coordinates of vertices          	      */
    long nverts,	/* I: number of vertices                 	      */
    long line,		/* I: line number                        	      */
    long xsects[],	/* I: x coordinates of intersections     	      */
    long *nxsect	/* O: number of intersections            	      */
)

/******************************************************************************
 
NAME:	xsect

FUNCTION:
	Calculates intersections of a line with the boundaries of a polygon.

	Calculates the intersections of the line number 'line'
	with the line segments connecting the veritces

*******************************************************************************/

{
long	i,			/* Loop variable                      	      */
	itemp,			/* temporary variable                         */
	iverts,			/* number of vertices counting first twice    */
	sorted,			/* Indicates sorting of xsects into           */
				/* ascending order                    	      */
	x1,			/* x coord of previous vertex                 */
	x2,			/* x coord of current vertex                  */
	y1,			/* y coord of previous vertex                 */
	y2;			/* y coord of current vertex                  */

*nxsect = 0;
iverts = nverts + 1;
xverts[iverts-1] = xverts[0];
yverts[iverts-1] = yverts[0];

/* Find intersections of line polygon boundary
   ------------------------------------------- */
for (i = 1; i < iverts; i++)
   {
   y1 = yverts[i-1];
   y2 = yverts[i];
   if ((y1 != y2) && (line >= min (y1, y2)) && (line <= max (y1, y2)))
      {
      (*nxsect)++;
      x1 = xverts[i-1];
      x2 = xverts[i];
      xsects[(*nxsect) - 1] = 
                    ((long)((0.5 + ((float) x1) + ((float) (x2 - x1)) *
                              ((float) (line - y1)) / ((float) (y2 - y1)))));
      }
   }

/* Sort intersections in ascending order
   ------------------------------------- */
do {
   sorted = TRUE;
   for (i = 1; i < *nxsect; i++)
      {
      if (xsects[i-1] > xsects[i])
         {
         itemp       = xsects[i-1];
         xsects[i-1] = xsects[i];
         xsects[i]   = itemp;
         sorted      = FALSE;
         }
      }
   } while (!sorted);
}
