#include "stereo.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char inhos[][CMLEN],     /* input image host names                        */
    char modhos[],          /* modulating image host name		     */
    char outhos[],           /* output image host name                        */
    long *num_inimages,	    /* number of input images			*/
    long num_image_bands[],       /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each input image   */
    long *num_outbands,	/* number of output bands */
    long wind1[][4],	/* input image window */
    long wind2[4],  	/* modulating image window */
    long modimg_band_num[],   /* band numbers in modulating image   */
    long out_type,	/* output image data type */
    long out_img_bands[][MAXBND+1], /* band numbers for the output image     */
    long *modflg               /*flag to indicate if modulating image exits  */
)
	
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         24-Nov-89   W. Xia/STX         conversion to 5.0
   2.0	       05-Feb-91   T. Sturman	      Added Statics for PN

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open input image files.
   Open modulating image files.
   Open the output image file.                                               
   Update the ddr file.
   Group all the images files into a group file
**************************************************************************
*/
{
double	line_inc = 1.0,		/* line and sample increments for          */
	samp_inc = 1.0;		/* output image ddr                        */

extern struct FDESC *fdesc[MAXIMG+2];
extern long *opoint;
extern long *ipoint;

static long 
	fatal_err = -1,
	one = 1,
	zero = 0;

long	access,			/* file access type			*/
	bufsiz,			/* image i/o buffer size		*/
	comb_flag=1,		/* image combination flag		*/
        n, nib,			/* loop index 				*/
        opt[2],			/* optional parameter array for output file */
	stat,			/* return status			*/
	type;			/* file type				*/

char	*prog_name = "STEREO";	/* program name				*/	
/*
Open input image files.                       
-------------------------                      
*/
access = IREAD;
type = EREAL;     
for (n = 0, nib = 0; n < *num_inimages; n++)
{
   stat = c_eopens( &fdesc[n], inhos[n], image_band_num[n], &wind1[0][SL], 
                   &wind1[0][SS], &wind1[0][NL], &wind1[0][NS], 
		   &ipoint[nib], &access, &type, &zero, &zero);

   if (stat != E_SUCC )
      {
      c_errmsg("Error opening input image", "stereo-inopen",
               &fatal_err);      
      }

   nib += num_image_bands[n];
   
}  

/*                                                               
If modulating image exists, open the modulating image files
------------------------------------------------------------
*/

if (*modflg ==1)
 {
  access = IREAD; 
  type = EREAL;
  stat = c_eopens(&fdesc[n], modhos, modimg_band_num, &wind2[SL], &wind2[SS], 
  &wind2[NL], &wind2[NS], &ipoint[nib], &access, &type, &zero, &zero);
          
  if (stat != E_SUCC )
   { 
   c_errmsg ("Error opening modulating image", "stereo-modopen",
              &fatal_err);
   }   
 }
 
/*
Open the output image file.
--------------------------
*/
access = IWRITE;
type = EREAL;

opt[0]=out_type;
opt[1]=0;

if (*modflg == 1)
{
stat = c_eopens (&fdesc[n+1], outhos, num_outbands, &one, &one,
&wind1[0][NL], &wind1[0][NS], opoint, &access, &type, &zero, opt);

if (stat != E_SUCC )       
   {
   c_abort (&block, "Error opening output image", "stereo-eopens",
               prog_name, out_files, &num_outfiles);
   }
 
fdesc[n+2]=0;
}
else
{
stat = c_eopens (&fdesc[n], outhos, num_outbands, &one, &one,
&wind1[0][NL], &wind1[0][NS], opoint, &access, &type, &zero, opt);

if (stat != E_SUCC )       
   {
   c_abort (&block, "Error opening output image", "stereo-eopens",
               prog_name, out_files, &num_outfiles);
   }
        
fdesc[n+1]=0;
}
          
/*
Update the ddr file.
--------------------
*/                                      
c_upddr (&block, inhos, outhos, num_inimages, wind1, image_band_num, 
         num_image_bands, &line_inc, &samp_inc, num_outbands, &comb_flag);

for (n = 0; n < *num_outbands; n++)
   out_img_bands[0][n] = n + 1;

/*                                        
set up group containing input, modulating  and output bands
-----------------------------------------------------------
*/
if ((iobuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
   {
   c_abort (&block, "Error allocating dynamic memory", "stereo-ealloc",
            prog_name, out_files, &num_outfiles);
   }

c_egroup (fdesc, &gdesc, iobuf, &bufsiz);

}
