#include "stereo.h"
#include "util.h"
#include "protos.h"
         
void getpar
(
    char intae[][CMLEN],         /*    input image tae name       */
    char inhos[][CMLEN],         /*    input image host name      */
    char modtae[][CMLEN],           /*    modulating image tae name  */
    char modhos[][CMLEN],           /*    modulating image host name */
    char outtae[][CMLEN], /*    output image tae name      */
    char outhos[][CMLEN], /*    output image host name     */
    long *shift,          /* maximum pixel shift        */
    long *modflg,         /* flag to indicate if modualting image exist */
    long wind1[][4], 		/* input image window		*/
    long *num_inimages,		/* number of input images           */
    long num_image_bands[],	/* number of bands in each input image */
    long image_band_num[][MAXBND+1],  /* band numbers selected for each image
				     	specified */
    long *num_inbands,           /* total number of input image bands   */
    long *num_outbands,		/* number of output image bands     */
    long wind2[][4], 		/* modulating image window 	*/
    long modimg_band_num[][MAXBND + 1], /* band numbers selected for modulating
					 image specified */
    long *out_type           /* output image data type           */
)  
                                     
/*
******************************************************************
NAME: GETPAR

PURPOSE:
	getpar is used to receive the paramter block from tm,
	initialize the catalog manager, and to get the parameters
	from the parameter block.

PROGRAM HISTORY:
  Version   Date       Author                       Request
  -------   ----       ------                       -------
                          
  1.0     07-Nov-84    L. Huewe/CSB
  2.0     26-Mar-86    K. Gacke/CSB                 lasd
  3.0     30-Nov-89    W. Xia                 conversion to 5.0
  4.0	  05-Feb-91    T. Sturman	      Added Statics for PN
                 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS
     
ALGORITHM DESCRIPTION:
                      
   Call c_inlas to initialize TAE.
   Get the input image file name.
   Get the modulating image file name.
   Get the output image file name.
   Get the data type of input images.
   Get the maximum pixel shift.
   Get the LAS global parameters: $maxtim, $minmax.

*************************************************************************
*/
{

/*  local variables */

struct VARIABLE	*var;               /* pointer to a variable in TAE block */
long
	itype,                     
	imglimit=MAXIMG, 	    /* maximum number of image allowed    */
        num_modimg_bands,   	    /* number of bands in modulating image*/
	num_modimg,		    /* number of modulating images 	  */
	num_modbands,		    /* tot num of bands for modulating image */
	same_wnd,	            /* flag if images have same window    */
	stat,                       /* return status                      */
  	std_out;     	       	    /* standard output device             */ 
        
static long
	fatal_err = -1,
	one = 1,
	zero = 0,
 	inimg_flag[MAXIMG][2],	/* flag for subwindow */
        modimg_flag[MAXIMG][2];  /* flag for subwindow */

static char    mnmx[4];        /* minimum and maximum option */
                
/*
initialize a Las program
_________________________
*/
c_inlas(&block,&std_out);
itype = ELONG;
  
/*
get the input image name                       
------------------------
*/
stat=c_getin (&block, "IN1", &zero, intae, inhos, num_inimages, wind1,
	 image_band_num, num_image_bands, num_inbands, &imglimit, &same_wnd,
	 inimg_flag);    
                                              
*num_outbands = *num_inbands;            

if (stat != E_SUCC)                                            
   c_errmsg ("Cannot access input file names", "Stereo-getin",
              &fatal_err);
if (same_wnd == 0)                                  
   c_errmsg ("Image windows differ in size", "Stereo-Getpar", &fatal_err);

/*
get the modulating image name if it exits                       
-----------------------------------------------
*/                                                         
stat=c_getin (&block, "IN2", &zero, modtae, modhos, &num_modimg, wind2,
       	 modimg_band_num, &num_modimg_bands, &num_modbands,&one, &same_wnd,
	 modimg_flag);                 

if (stat != E_SUCC)
   c_errmsg ("Cannot access input modulating file names", "stereo-getin",
              &fatal_err);
if (same_wnd == 0)                                  
   c_errmsg ("Image windows differ in size", "stereo-window", &fatal_err);

if (num_modimg == 0) 
  {
  strcpy(modhos[0], "");  
  *modflg = 0;
  } 
                         
else if (num_modbands > 1)  
  {                              
/*                                  
error if more than one band was specified    
-----------------------------------------
*/
   c_errmsg ("Modulating image may contain only one band","stereo-getpar",
  	      &fatal_err); 
   }
else

/*  
error if input and modulating images are different sizes 
--------------------------------------------------------
*/
   {
     *modflg = 1;
     if ((wind1[0][NS] !=  wind2[0][NS])  ||  (wind1[0][NL] != wind2[0][NL]))  
       {              
        c_errmsg (" Input and modulating images are different sizes",
		 "stereo-getpar ",&fatal_err);                        
       }                          
    }               
      
/*                      
Get the output image file name.
-------------------------------
*/
stat = c_getout (&block, "OUT", outtae, outhos);  

if (stat != E_SUCC)
   c_errmsg ("Cannot access output file name", "stereo-getout", &fatal_err);

num_outfiles = 0;
strcpy (out_files[num_outfiles++], outtae[0]);

/*
get the data type of input image
--------------------------------
*/
c_getddr (inhos[0], &ddr);  
*out_type = ddr.dtype;

/*  
get the maximum pixel shift
---------------------------
*/
c_getnum (&block,  "SHIFT", &one, &itype, shift); 

/*
Get las global parameters
-------------------------         
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$minmax");
strcpy( mnmx, SVAL( *var, 0));
if (mnmx[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

}
