#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct FDESC  *fdesc[MAXIMG+2];				/*  file descriptors		*/
struct GDESC  *gdesc;				/*  group descriptor		*/
long	*ipoint;			/*  offsets into image i/o	*/
long	min_max;			/*  minmax option flag		*/
long	msgtim;				/*  message interval		*/
long	num_outfiles;			/*  number of output files	*/
long	*opoint;			/*  for input and output bands	*/

struct  PARBLK  block;			/* TAE parameter block		*/
struct  DDR     ddr;			/* ddr				*/
FILE    *fd;				/* temp file descriptor		*/

float   *iobuf;				/* image i/o buffer		*/
char    out_files[2][CMLEN];		/* output file names		*/


int main
(void)
{

/*
********************************************************************

NAME: STEREO

PURPOSE: Program stereo takes an input image and optional modulating
	 image and makes a stereo image complement of the initial
    	 image. there must be at least one input and one output image.
	 the modulating image must be the same size as the input image.
	 the input image may be real, integer*4, integer*2, or byte.

PROGRAM HISTORY:

  Version   Date       Author                       Request
  -------   ----       ------                       -------
                          
  1.0     11-Nov-84    L. Huewe/CSB
  2.0     26-Mar-86    K. Gacke/CSB                 lasd
  3.0     30-Nov-89    W. Xia                 conversion to 5.0
  4.0	  05-Feb-91    T. Sturman	      Added Statics for PN
  5.0     24-Jan-92    S. Nelson              Removed warning messages on
                                              compile

                 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
 
call getpar to receive the parameters from the user
call filspec to open input, modulating and output images
if modflg equals one then write a message: "calculating minimum and maximum 
   pixel values for the modulating image "              
   call c_minmax to find minimum and maximum pixel values for modulating image
else
   write a message: "calculating minimum and maximum pixel values for the
   input image " 
   call c_minmax to find minimum and maximum pixel values for input image
call p3dsub to create the stereo pair complement 
call c_minmax to calculate the minimum and maximum values for output image
call c_egclse to close all the images
call c_hstout to generate the history file
call c_complt to print a completion message

*************************************************************************
*/
static long 
	one = 1,
        image_band_num[MAXIMG][MAXBND+1],    /* band numbers selected for each
					          image specified           */
 	wind1[MAXIMG][4], 		     /* input image window 	    */
      	out_img_bands[MAXIMG][MAXBND+1],     /* bands numbers for the output 
                                                image                       */
	num_image_bands[MAXIMG];       	     /* num of bands per input image*/  

long	modimg_band_num[1][MAXBND+1],  	/* band numbers selected for 
              			               modulating image specified */
        modflg,         		/* flag if modualting image exist */
	num_inbands,             /* total number of input image bands*/
	num_inimages,            /* number of input images           */
	num_outbands,            /* number of bands in output image  */
	out_type,                /* output image data type           */
        shift,  	         /* maximum pixel shift              */  
  	wind2[1][4]; 		 /* modulating image window  	*/

long    nonfatal = 1;
long    fatal = -1;

char
        inhos[MAXIMG][CMLEN],		/*  input image host name	*/  
        intae[MAXIMG][CMLEN],		/*  input image tae name	*/
        modhos[1][CMLEN],			/*  modulating image host name	*/  
        modtae[1][CMLEN],			/*  modulating image tae name	*/  
        outhos[1][CMLEN],			/*  output image host name	*/
	outtae[1][CMLEN],			/*  output image tae name	*/  
	*prog_name = "STEREO";		/*  program name		*/

double
	amax[MAXBND],		   	/*  maximum pixel value		*/
        amin[MAXBND];               	/*  minimum pixel value		*/  

/*  allocate memory
------------------*/
if ((ipoint = (long *) malloc(MAXIMG * (MAXBND + 1) * sizeof(long))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","stereo-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","stereo-fatal",&fatal);
   }
if ((opoint = (long *) malloc((MAXBND + 1) * sizeof(long))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","stereo-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","stereo-fatal",&fatal);
   }
/*
get the user specified parameters
---------------------------------
*/                 
                            
getpar(intae, inhos, modtae, modhos, outtae, outhos, &shift, &modflg,
  wind1, &num_inimages, num_image_bands, image_band_num, &num_inbands,
  &num_outbands, wind2, modimg_band_num, &out_type);

/*     
Set up file specifications.
---------------------------
*/

if (num_outbands != 0)                                          
  filspec (inhos, modhos[0], outhos[0],  &num_inimages, num_image_bands, 
  image_band_num, &num_outbands, wind1,wind2[0],modimg_band_num[0], out_type,
  out_img_bands,&modflg);

/*
calculate minimum and maximum pixel values
------------------------------------------
*/
if (modflg  ==  1) 
   {
   c_errmsg("Calculating min max values for the Modulating Image",
     	    "stereo-minmax",&nonfatal);
   c_minmax (modhos, &one, wind2, modimg_band_num, &one, amin, amax);
   }
else
   {
   c_errmsg("Calculating min max values for the Input Image",
    	    "stereo-minmax", &nonfatal);      
   c_minmax (inhos, &num_inimages, wind1, image_band_num, num_image_bands,
   	     amin, amax);       
   }
                               
/*
calculate the stereo pair complement for each line,each band
------------------------------------------------------------
*/                                                        
p3dsub (amax, amin, shift, modflg, wind1, num_inbands, &num_outbands);
  
/*
calculate the minimum and maximum pixel values for the output image
-------------------------------------------------------------------
*/
if (min_max)    
{
	wind1[0][SL] = wind1[0][SS] = 1;
	c_minmax (outhos, &one, wind1, out_img_bands, 
		  &num_inbands, amin, amax);
	c_svmnmx (outhos[0], &num_inbands, amin, amax, "YES");
}


/*
Generate the history file and print the completion message.
------------------------------------------------------------
*/
c_hstout (&block, outhos[0], inhos, &num_inimages, " ");
c_complt (prog_name, out_files, &num_outfiles);

return EXIT_SUCCESS;

}

