/******************************************************************************
NAME:		SUBSTRI

PURPOSE:    	This function extracts a substring of a specified length
	   	starting at a specified position from an input string and
		returns the result to a TAE procedure in an integer name 
		parameter.

	Programmer	  Date		Modifications
	----------	  ----		-------------
	L. Huewe	  05/86		Original development for ADAPS.

PROJECT:	  ADAPS

******************************************************************************/
#include <stdio.h>		/* C standard I/O include file		     */
#include <ctype.h>		/* C standard I/O include file		     */

#include "worgen.h"		/* TAE include file for UNIX		     */
#include "tae.h"		/* TAE include file for UNIX		     */

main()
{
struct PARBLK vblock;		/* TAE parameter block	 		     */

struct VARIABLE *p_find();	/* Function to find TAE variable	     */
struct VARIABLE *var;		  

long colon;			/* Position of the colon		     */
long i;				/* Looping index			     */
long ival[1];			/* Integer value of substring		     */
long length;			/* Length of substring			     */
long start;	        	/* Start of substring			     */
long stdut;  			/* Standard output			     */
long status;			/* Status indicator			     */
long fatal = -1;		/* Indicates a fatal error occured	     */
long nonfatal = 1;		/* Flag for a NONfatal error 		     */

char instr[133];		/* Input string				     */
char outstr[133];		/* Output substring			     */
char spec[8];			/* Substring specification		     */


/*
  Initialize TAE
*/
c_inlas (&vblock,&stdut);


/* 
  Get the input string and specification
*/
var = p_find(&vblock, "STRING");
strcpy (instr,SVAL(*var,0));
var = p_find(&vblock, "SPEC");
strcpy (spec,SVAL(*var,0));


/*
  Extract the start of the substring and the length of the string
*/
colon = strcspn(spec,":");
if (colon == strlen(spec))
    c_errmsg ("Invalid input specification","substr-invspec", &fatal);
    for (i=0; i < strlen(spec); i++)
	{
	if (isdigit(spec[i]) == 0) 
	    if (spec[i] != ':')
		 c_errmsg ("Invalid input specification", "substr-invspec",
			    &fatal);
	}
    if ((colon > 0) && (colon < strlen(spec)-1))
	sscanf (spec,"%d:%d",&start,&length);
    else if (colon == 0)
	{
	start = 1;
	sscanf (spec,":%d",&length);
	}
    else if (colon == strlen(spec)-1)
	{
	sscanf (spec,"%d:",&start);
	length = strlen(instr) - start + 1;
	}
    if ((start+length-1) > strlen(instr))
	c_errmsg ("Invalid input specification","substr-invspec", &fatal);


/*
  Copy the substring into the output string
*/
    strncpy(outstr,&instr[start-1],length);
    outstr[length] = '\0';
    for (i=0; i < length; i++)
	{
	if (isdigit(outstr[i]) == 0)
	    c_errmsg ("Substring contains non-numeric characters",
		"substr-nonum", &fatal);
	}
    ival[0] = 0;
    sscanf (outstr,"%d",&ival[0]);


/* 
Update the value of the name parameter and write out the parameter block
*/
	status = q_intg (&vblock,"OUTVAL",1,ival,P_UPDATE);
	status = q_out (&vblock);

	z_exit(SUCCESS,"SUBSTRI completed successfully");
}
