/****************************************************************************
NAME      		  GETPAR 


PURPOSE		To obtain the necessary parameters for the SYSSAMP function.


PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
B.DEALY         Oct   1984       Original Development
R.MCKINNEY      April 1989       converted to LAS 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
   The global parameters are first received.
   Next the input and output images are gotten.
   Various validity checks are then made before returning.
******************************************************************************/


#include <stdio.h>
#include "las.h"	
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *vblock,       /* I: pointer to TAE block	 	    */
    char taein[][CMLEN],	 /* O: TAE file names of input images	    */
    char hosin[][CMLEN],  	 /* O: host file names of input images	    */
    char *taeout,	 	 /* O: TAE file name of output image	    */
    char *hosout,		 /* O: host file name of output image	    */
    long *nmrimg, 	         /* O: number of input images		    */
    long (*bands)[MAXBND+1],     /* O: band specification 		    */
    long *nbands,		 /* O: number of bands per input image      */
    long (*window)[4],	         /* O: window specification		    */
    long *msgtim,		 /* O: elapsed time between display messages*/
    char *minmax,		 /* O: min max flag buffer		    */
    long *totbnd,		 /* O: total number of bands		    */
    long *kernel,		 /* O: kernel dimensions (nl,ns)            */
    long *skippix,		 /* O: number of pixels to skip		    */
    long *odtype		 /* O: output data type 		    */
)
{
struct  VARIABLE  *val;	      	 /* pointer to TAE variable within v-block  */

long     element = 0;		 /* parameter element number		    */
long     fatal = -1;		 /* fatal error flag			    */
long     imglim;		 /* max number of images to be processed    */
long     nonfatal = 1;		 /* value of one			    */
long     qflag[MAXIMG][2];  	 /* flag indicating if subwindows specified */
long     same;		    	 /* flag indicating if images have same wind*/
long     status;	   	 /* function return status code		    */
long     xdir;			 /* number of pixels in x direction         */
long     ydir;			 /* number of pixels in y direction         */
long     xxdir;			 /* number of pixels in x direction         */
long     yydir;			 /* number of pixels in y direction         */

char     dtype[5];		 /* data type of previous input image       */

val = p_find(vblock,"$msgtim");
*msgtim = IVAL(*val,0);
val = p_find(vblock,"$minmax");
strcpy(minmax,SVAL(*val,0));

/* Build the string descriptor for TAE variable "IN".
   Call getin to parse the input image specification.
   Getin returns the TAE image name(s), host image name(s),
   bands to be processed, window specification, and number
   of images parsed.
----------------------------------------------------------------------------*/
imglim = MAXIMG;
status = c_getin(vblock,"IN",&element,taein,hosin,nmrimg,window,bands,
	       nbands,totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","syssamp-getin",&fatal);
if (same == 0)
   {
   c_errmsg("Input images do not have same window sizes","syssamp-window",
		&nonfatal);
   c_errmsg("Fatal error encountered","syssamp-fatal",&fatal);
   }
/* Retrieve the TAE output image name from the parameter
   block and store it in "taeout". 
   Create a host file name for the output image and store 
   it in "hosout".
----------------------------------------------------------------------------*/
if (c_getout(vblock,"OUT",(char (*)[256])taeout,(char (*)[256])hosout) ==E_FAIL)
   c_errmsg("Fatal error encountered","syssamp-getout",&fatal);

/* Retrieve the kernel dimensions
---------------------------------------------------------------------------*/
val = p_find(vblock,"kerndim");
xdir = IVAL(*val,0);
ydir = IVAL(*val,1);
kernel[0] = xdir;
kernel[1] = ydir;
if (xdir > window[0][2])
   {
   c_errmsg("KERNDIM larger than input image specified",
		"syssamp-kernel",&nonfatal);
   c_errmsg("Fatal error encountered","syssamp-fatal",&fatal);
   }

if (ydir > window[0][3])
   {
   c_errmsg("KERNDIM larger than input image specified",
		"syssamp-kernel",&nonfatal);
   c_errmsg("Fatal error encountered","syssamp-fatal",&fatal);
   }

/* Retrieve the number of pixels to skip
---------------------------------------------------------------------------*/
val = p_find(vblock,"skippix");
xxdir = IVAL(*val,0);
yydir = IVAL(*val,1);
skippix[0] = xxdir;
skippix[1] = yydir;
/*
if (xxdir < 0)
   c_errmsg("Incorrect number of lines to skip",
		"syssamp-lines",&fatal);
if (yydir < 0)
   c_errmsg("Incorrect number of samples to skip",
		"syssamp-samples",&fatal);
*/

if (xxdir < xdir)
   c_errmsg("KERNDIM values are greater than the SKIPPIX values",
		"syssamp-values",&fatal);
if (yydir < ydir)
   c_errmsg("KERNDIM values are greater than the SKIPPIX values",
		"syssamp-values",&fatal);
/*
  Retrieve the output data type
--------------------------------------------------------------------------*/
val = p_find(vblock,"odtype");
strcpy(dtype,SVAL(*val,0));
if (strcmp(dtype,"SAME") == 0)
	*odtype = 0;
else if (strcmp(dtype,"BYTE") == 0)
        *odtype = EBYTE;
else if (strcmp(dtype,"I*2") == 0)
        *odtype = EWORD;
else if (strcmp(dtype,"I*4") == 0)
        *odtype = ELONG;
else if (strcmp(dtype,"R*4") == 0)
        *odtype = EREAL;
return;
}
