/******************************************************************************
NAME:   		      PROCES 

PURPOSE    To proces the extracted windows and create a new image. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  1.0      B. Dealy        10/84          GSFC          Original Development
  5.0	   R. McKinney     6/88		  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file
  5.1	   T. Baltzer      6/92		  EDC		fix nbrsamp calculation
							run thru Saber-C
  6.1      D. Eichacker    5/94           EDC	        added opt[].


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PROCES must be run under TAE.

PROJECT			LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   Apply the syssamp algebra to the input image
   Save the minmax values if $MINMAX was set to "YES"
   Update the output ddr
   Close the input and output image files

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc[MAXIMG+1];	/* image file descriptors used for image IN  */
extern int  fdesc1[2];		/* image file descriptors used for image OUT */
extern int  gdesc;		/* group descriptor used for image IN        */
extern int  gdesc1;		/* group descriptor used for image OUT       */
extern long offset1[MAXIMG*MAXBND];/* offsets into "thebuf" for input bands  */
extern long offset2[MAXBND];	/* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */


void proces
(
    struct PARBLK *vblock,	/* I: TAE parameter block                    */
    char hosin[][CMLEN],	/* I: host names of input images             */
    char *hosout,		/* I: host name of output image	             */
    long nmrimg,		/* I: number of input images                 */
    long (*bands)[MAXBND+1],	/* I: bands to be processed for all images   */
    long *nbands,		/* I: number of bands per input image        */
    long (*window)[4],		/* I: window specification of input images   */
    long msgtim,		/* I: elapsed time between display messages  */
    char *minmax,		/* I: min max flag                           */
    long totbnd,		/* I: total number of bands to be processed  */
    long *kernel,		/* I: kernel dimensions (nl,ns)		     */
    long *skippix,		/* I: number of pixels to skip		     */
    long odtype			/* I: output data type			     */
)
 
{

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

static	long	opt[3]={0,0,0};	/* optional parmeter array for EOPENS	     */
long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bufsiz;			/* size of image IN  buffer                  */
long	bufsiz1;		/* size of image OUT buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag			     */
long    i;			/* counter for number of columns	     */
long	image;			/* loop index counting input images          */
long    j;			/* counter for number of rows	             */
long    maxsamp;		/* number of samples in output image	     */
long    maxline;		/* number of lines in output image           */
long	msg_flag=ALLCAT;	/* flag for processing message               */
long 	nbrsamp;		/* number of sample blocks                   */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long 	one = 1;		/* one  		                     */
long    sample;			/* sample indicator			     */
long	size;			/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    windl;			/* maximum line to retrieve		     */
long    winds;			/* maximum sample to retrieve		     */
long    upddr_flag;	        /* flag for update DDR			     */
long	zero = 0;		/* value of zero                             */

long    left;			/* left side of the sample block	     */
long    right;			/* right side of the sample block	     */
long    top;			/* top of the sample block		     */
long    bottom;			/* bottom of the sample block		     */

float   temp;			/* temporary storage area		     */

double  temp2;			/* temporary storage area for i*4 and r*4    */

unsigned char    *thebuf;	/* image input buffer                        */
unsigned char    *thebuf1;	/* image output buffer                       */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopens((struct FDESC **)&fdesc[image],hosin[image],bands[image],
		    &sl,&ss,&nl,&ns,&offset1[band],&access,&odtype,&zero,opt);
   if (status != E_SUCC)
      {
      c_errmsg("Unable to open input image","syssamp-open",&nonfatal);
      c_errmsg("Fatal error encountered","syssamp-fatal",&fatal);
      }
   band += nbands[image];

   }
fdesc[nmrimg] = 0;

/* Open output image
--------------------*/
access = IWRITE;	
maxline = ((nl-kernel[0])/skippix[0] + 1) * kernel[0];
maxsamp = ((ns-kernel[1])/skippix[1] + 1) * kernel[1];
if (maxline > nl)
   maxline -= kernel[0];
if (maxsamp > ns)
   maxsamp -= kernel[1];
status = c_eopens((struct FDESC **)&fdesc1[0],hosout,&totbnd,&one,&one,&maxline,
                  &maxsamp,offset2,&access,&odtype,&zero,opt);
if (status != E_SUCC)
      {
      c_errmsg("Unable to open output image","syssamp-open",&nonfatal);
      c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
      }
fdesc1[1] = 0;

/* Determine size of output image
--------------------------------*/
switch (odtype)
  {
  case EBYTE: size = 1; break;
  case EWORD: size = 2; break;
  case ELONG: size = 4; break;
  case EREAL: size = 4; break;
  }

/* Allocate I/O buffers
----------------------*/
if ((thebuf = (unsigned char *)c_ealloc((struct FDESC **)fdesc,&bufsiz))==NULL)
   {
   c_errmsg("Error allocating dynamic memory","syssamp-alloc",&nonfatal);
   if (c_egclse((struct GDESC **)&gdesc) != E_SUCC)
       c_errmsg("Error closing images","syssamp-close",&one);
   c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
   }

if ((thebuf1=(unsigned char *)c_ealloc((struct FDESC **)fdesc1,&bufsiz1))==NULL)
   {
   c_errmsg("Error allocating dynamic memory","syssamp-alloc",&nonfatal);
   if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
       c_errmsg("Error closing images","syssamp-close",&one);
   c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
   }

/* Group images
---------------*/ 
if (c_egroup((struct FDESC **)fdesc,(struct GDESC **)&gdesc,thebuf,
		&bufsiz)!=E_SUCC)
   {
   if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
       c_errmsg("Error closing images","syssamp-close",&one);
   c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
   }

if (c_egroup((struct FDESC **)fdesc1,(struct GDESC **)&gdesc1,thebuf1,
              &bufsiz1) != E_SUCC)
   {
   if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
       c_errmsg("Error closing images","syssamp-close",&one);
   c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
   }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("SYSSAMP","Processed","line","bands"," ",&nl,
		&totbnd,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&totbnd,minval,maxval);

/* Calculate the maximum number of lines and samples to use
---------------------------------------------------------*/
windl = (nl / skippix[0]) * skippix[0] + kernel[0];
winds = (ns / skippix[1]) * skippix[1] + kernel[1];
nbrsamp = ns / skippix[1] + 1;
if (windl > nl)
    windl -= kernel[0];
if (winds > ns)
    {
    winds -= kernel[1];
    nbrsamp--;
    }

/* Main processing loop
-----------------------*/

switch (odtype)
{
case 1:
 if (c_estep((struct GDESC **)&gdesc1) != E_SUCC)
    {
    if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
        c_errmsg("Error closing images","syssamp-close",&one);
    c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
    }
 left = top = 0;
 right = left + kernel[1];
 bottom = top + kernel[0];
 while (bottom <= windl)
    {
    for (curlin = top;curlin < bottom; curlin++)
       {
       if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
             c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
          }
       for (band = 0; band < totbnd; band++)
          {
          j = 0;
          for (i = 0; i < nbrsamp; i++)
   	     {
	     left = i * skippix[1];
	     right = left + kernel[1];
	     for (sample = left;sample < right;sample++)
	        {
                temp = *(thebuf + (*(offset1+band)+sample) * size);
	        if (temp > 255.0) temp = 255.0;
	        else if (temp < 0.0) temp = 0.0;
	        *(thebuf1 + (*(offset2+band)+j) * size) = temp;
	        j++;
	        } /* end of sample loop   */
	     }    /* end of nbrsamp loop  */
          }  	/*   end of band loop    */ 
       if (c_estep((struct GDESC **)&gdesc1) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
             c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
          }
       curent ++;
       }		/* end of curlin loop  	*/
       top = bottom;
       bottom = top + skippix[0] - kernel[0];
       if (bottom <= windl)
          {
          for (curlin = top;curlin < bottom; curlin++)
              {
              if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
                 {
                 if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
                    c_errmsg("Error closing images","syssamp-close",&one);
                 c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
                 }
              curent ++;
          }		/*   end of curlin loop  */
          top = bottom;
          bottom = top + kernel[0];
       }		/*   end of if bottom    */
    }		/*   end of while loop   */
break;

case 2:
 if (c_estep((struct GDESC **)&gdesc1) != E_SUCC)
    {
    if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
        c_errmsg("Error closing images","syssamp-close",&one);
    c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
    }
 left = top = 0;
 right = left + kernel[1];
 bottom = top + kernel[0];
 while (bottom <= windl)
    {
    for (curlin = top;curlin < bottom; curlin++)
       {
       if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
              c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
          }
       for (band = 0; band < totbnd; band++)
          {
          j = 0;
          for (i = 0; i < nbrsamp; i++)
             {
             left = i * skippix[1];
          right = left + kernel[1];
          for (sample = left;sample < right;sample++)
             {
             temp =
                (*(short *)(thebuf + (*(offset1+band)+sample) * size));
             if (temp > 32767.0) temp = 32767.0;
             else if (temp < -32768.0) temp = -32768.0;
             (*(short *)(thebuf1 + (*(offset2+band)+j) * size)) = temp;
             j++;
             } /* end of sample loop   */
          }    /* end of nbrsamp loop  */
       }		/*   end of band loop    */
       if (c_estep((struct GDESC **)&gdesc1) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
             c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
          }
       curent ++;
       }		/* end of curlin loop  	*/
    top = bottom;
    bottom = top + skippix[0] - kernel[0];
    if (bottom <= windl)
    {
    for (curlin = top;curlin < bottom; curlin++)
       {
       if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
             c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
          }
       curent ++;
       }		/*   end of curlin loop  */
    top = bottom;
    bottom = top + kernel[0];
    }		/*   end of if bottom    */
 }		/*   end of while loop   */
break;

case 3:
 if (c_estep((struct GDESC **)&gdesc1) != E_SUCC)
    {
    if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
        c_errmsg("Error closing images","syssamp-close",&one);
    c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
    }
 left = top = 0;
 right = left + kernel[1];
 bottom = top + kernel[0];
 while (bottom <= windl)
    {
    for (curlin = top;curlin < bottom; curlin++)
       {
       if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
             c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock, "syssamp",(char(*)[256])hosout,&one);
          }
       for (band = 0; band < totbnd; band++)
          {
          j = 0;
	  for (i = 0; i < nbrsamp; i++)
	     {
	     left = i * skippix[1];
	     right = left + kernel[1];
	     for (sample = left;sample < right;sample++)
	        {
	        temp2 =
	           (*(long *)(thebuf + (*(offset1+band)+sample) * size));
	        if (temp2 > 2147483647.0) temp2 = 2147483647.0;
	        else if (temp2 < -2147483648.0) temp2 = -2147483648.0;
	        (*(long *)(thebuf1 + (*(offset2+band)+j) * size)) = temp2;
                j++;
	        } /* end of sample loop   */
	     }    /* end of nbrsamp loop  */
          }		/*   end of band loop    */
       if (c_estep((struct GDESC **)&gdesc1) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
              c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
          }
       curent ++;
       }		/* end of curlin loop  	*/
    top = bottom;
    bottom = top + skippix[0] - kernel[0];
    if (bottom <= windl)
       {
       for (curlin = top;curlin < bottom; curlin++)
          {
          if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
             {
             if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
                c_errmsg("Error closing images","syssamp-close",&one);
             c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
             }
          curent ++;
          }		/*   end of curlin loop  */
       top = bottom;
       bottom = top + kernel[0];
       }		/*   end of if bottom    */
    }		/*   end of while loop   */
break;

case 4:
 if (c_estep((struct GDESC **)&gdesc1) != E_SUCC)
    {
    if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
        c_errmsg("Error closing images","syssamp-close",&one);
    c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
    }
 left = top = 0;
 right = left + kernel[1];
 bottom = top + kernel[0];
 while (bottom <= windl)
    {
    for (curlin = top;curlin < bottom; curlin++)
       {
       if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
             c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
          }
       for (band = 0; band < totbnd; band++)
          {
          j = 0;
	  for (i = 0; i < nbrsamp; i++)
	     {
	     left = i * skippix[1];
	     right = left + kernel[1];
	     for (sample = left;sample < right;sample++)
	        {
	        temp2 =
                   (*(float *)(thebuf + (*(offset1+band)+sample) * size));
	        if (temp2 > 2147483647.0) temp2 = 2147483647.0;
	        else if (temp2 < -2147483648.0) temp2 = -2147483648.0;
	        *(float *)(thebuf1 + (*(offset2+band)+j) * size) = temp2;
	        j++;
	        } /* end of sample loop   */
	     }    /* end of nbrsamp loop  */
          }		/*   end of band loop	 */
       if (c_estep((struct GDESC **)&gdesc1) != E_SUCC)
          {
          if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
             c_errmsg("Error closing images","syssamp-close",&one);
          c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
          }
       curent ++;
       }		/* end of curlin loop  	*/
    top = bottom;
    bottom = top + skippix[0] - kernel[0];
    if (bottom <= windl)
       {
       for (curlin = top;curlin < bottom; curlin++)
          {
          if (c_estep((struct GDESC **)&gdesc) != E_SUCC)
             {
             if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
                c_errmsg("Error closing images","syssamp-close",&one);
             c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);
             }
          curent ++;
          }		/*   end of curlin loop  */
       top = bottom;
       bottom = top + kernel[0];
    }		/*   end of if bottom    */
 }		/*   end of while loop   */
break;
}

for (band = 0; band < totbnd; band++)
   {
   if (strcmp(minmax,"YES") == 0)
   status = c_pxmnmx((long *)thebuf1 + (*(offset2+band) * size),&ns,
                     &minval[band],&maxval[band],&odtype); 
   }
/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)nl / (double)maxline;
samp_inc = (double)ns / (double)maxsamp;
upddr_flag = NOCOMB;
status = c_upddr(vblock,(char (*)[256])hosin[0],hosout,&nmrimg,window,bands,
		 nbands,&line_inc,&samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"syssamp",(char (*)[256])hosout,&one);

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout,&totbnd,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Error updating the minmax information","syssamp-minmax",&one);

/* Close all image files in the groups
-------------------------------------*/
if (c_egclse((struct GDESC **)&gdesc) != E_SUCC)
   c_errmsg("Error closing images","syssamp-close",&one);
if (c_egclse((struct GDESC **)&gdesc1) != E_SUCC)
   c_errmsg("Error closing images","syssamp-close",&one);

/* Free buffers
--------------*/
free(thebuf);
free(thebuf1);
return;
}
