/***************************************************************************

NAME      		  SYSSAMP 

PURPOSE   Performs systematic pixel or cluster sampling.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
B.DEALY         Oct   1984      Original Development
R.MCKINNEY      April 1989	Conversion to LAS 5.0
T.BALTZER       June  1992	Fix bug in proces()
				Run code thru Saber-C
D.HAMES		Nov   1999      Remove 50000 limits on number of lines and
				samples for KERNDIM and SKIPPIX in syssamp.pdf.
				Add prototypes. Ref SR #893

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   SYSSAMP must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)

   The LAS program is initialized.  Next the parameters are
   retrieved thru GETPAR.  The main processing routine is called
   to apply the sampling.  Finally, the
   history file is updated and the routine completed.

ALGORITHM REFERENCES     

   None.

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/

int  fdesc[MAXIMG+1];           /* image file descriptors used for image IN  */
int  fdesc1[2];                 /* image file descriptors used for image OUT */
int  gdesc;		        /* group descriptor used for image IN        */
int  gdesc1;		        /* group descriptor used for image OUT       */
long offset1[MAXIMG*MAXBND];	/* offsets into "thebuf" for input bands     */
long offset2[MAXBND];		/* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */

void main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		     */

static long bands[MAXIMG][MAXBND + 1]; /* specified bands		     */
long kernel[2];			/* kernel dimensions (nl,ns)		     */
long msgtim;			/* elpased time between display messages     */
long nmrimg;			/* number of input images for image          */
long nonfatal = 1;		/* nonfatal error flag			     */
long nbands[MAXIMG];		/* number of bands per input image           */
long odtype;			/* odtype for processing                     */
long one = 1;			/* number one				     */
long skippix[2];		/* number of pixels to skip (nl,ns)	     */
long std_out;			/* logical unit # for standard output        */
long totbnd;			/* total number of bands in first image      */
long window[MAXIMG][4];		/* window specification for first image      */


static char hosin[MAXIMG][CMLEN];/* buffer storage        	   	     */
static char hosout[CMLEN];       /* buffer storage        		     */
char    minmax[4];	         /* minmax flag buffer 			     */
static char taein[MAXIMG][CMLEN];/* buffer storage       		     */
static char taeout[CMLEN];       /* buffer storage       		     */


c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/*
       get user-specified parameters and TAE globals
*/
getpar(&vblock, taein, hosin, taeout, hosout,
       &nmrimg, bands, nbands, window, &msgtim, 
       minmax, &totbnd,kernel,skippix,&odtype);
/*
   perform the syssamp algebra 
*/
proces(&vblock,hosin,hosout,nmrimg,bands,nbands,
	window,msgtim,minmax,totbnd,kernel,skippix,odtype); 
/*
   create associated history file
*/
if (c_hstout(&vblock, hosout, (char (*)[256])hosin[0] , &nmrimg, " ") != E_SUCC)
   c_errmsg("Error accessing the history information",
		"syssamp-history",&nonfatal);

/*
   inform user of successful completion
*/
c_complt("SYSSAMP", (char (*)[256])taeout, &one);
}
