/******************************************************************************

NAME:	GETPAR

FUNCTION:  GETPAR retrieves the TAE parameter values

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Retreive the output file name
Retreive the input file name
Retreive the the attribute name parameter
Upper case all the attribute names
Retrieve that output columns -- subtract 1 from the values to compensate for 
 zero relative arrays
Calculate total columns.
Find the column of the TIE ATTRIBUTE
Check the count of INCOL, ATRNAME, DTYPE, and DESC

******************************************************************************/

#include <string.h>
#include "las.h"
#include "ltable.h"
#include "util.h"
#include "tab2asc.h"
#include "protos.h"

void getpar
(
    struct PARBLK *parb,		/* parameter block structure */
    char *hosin,			/* host input file name descriptor */
    char *hosout,			/* host output file name descriptor */
    long *noutcol,			/* number of output columns specified */
    long *totcol,			/* total number of columns to write */
    char atrname[][CMLEN],		/* attribute name for each output col */
    long *outcol,			/* output columns specified */
    long *tiecol			/* tie column				*/
)

{ /* getpar */

struct VARIABLE *val;		/* pointer to variable */

long access;			/* type of access of file */
long atrname_cnt;		/* no. of attribute names */
long fatal = -1;		/*  fatal error code			*/
long i;				/* counter--current parameter value */
long nbytes;			/*  no. of bytes in attribute name	*/
long nonfatal = 1;		/*  nonfatal error code			*/
long status;			/*  return status of function calls	*/
long zero = 0;			/*  value of zero			*/

char tieatr[CMLEN];		/*  Tie attribute			*/

/* Retrieve output labeled table file name.
-------------------------------------------*/
access = IWRITE;
status = c_getnam(parb,"OUTFILE",&zero,hosout,"txt",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tab2asc-fatal",&fatal);

/*  Retrieve input ascii file name.
-----------------------------------*/
access = IREAD;
status = c_getnam(parb,"INLT",&zero,hosin,"lt",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tab2asc-fatal",&fatal);

/*  Retreive parameter values.
------------------------------*/
val = p_find(parb,"ATRNAME");
atrname_cnt = (*val).v_count;
for (i = 0; i < atrname_cnt; i++)
   strcpy(atrname[i],SVAL(*val,i));

/*  Upper case the attribute names
----------------------------------*/
for (i = 0; i < atrname_cnt; i++)
   {
   nbytes = (long) strlen(atrname[i]);
   c_low2up(atrname[i],&nbytes);
   }

/* Retrieve that output columns
   Subtract 1 from the values to compensate for zero relative arrays
--------------------------------------------------------------------*/
val = p_find(parb,"OUTCOL");
*noutcol = (*val).v_count;
for (i = 0; i < *noutcol; i++)
   *(outcol + i) = IVAL(*val,i) - 1;

/*  Calculate total columns.
----------------------------*/
*totcol = 0;
for (i = 0; i < *noutcol; i++)
      *totcol = LASMAX(*totcol,*(outcol + i) + 1);

/* Find the column of the TIE ATTRIBUTE
---------------------------------------*/
val = p_find(parb,"TIEATR");
if (val->v_count > 0)
   {
   strcpy(tieatr,SVAL(*val,0));
   for (i = 0; i < atrname_cnt; i++)
      if (strcmp(atrname[i],tieatr) == 0)
         *tiecol = i;
   }
else
   *tiecol = MAXCOL + 1;

/*  Check the count of INCOL, ATRNAME, DTYPE, and DESC
------------------------------------------------------*/
if (*noutcol != atrname_cnt)
   {
   c_errmsg( "ATRNAME and OUTCOL are not equal"
            ,"tab2asc-equal",&nonfatal);
   c_errmsg("Fatal error encountered","tab2asc-fatal",&fatal);
   }

return;
} /* getpar */
