/******************************************************************************

NAME:	TAB2ASC

FUNCTION:  TAB2ASC converts a labeled table file to an ascii file

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development
    5.1         08/89      B. Ailts     Fixed problem with string data on SUN-4
    5.2         08/89      B. Ailts     Fixed with reading a LT file that has
					not subfiles
    5.3         05/90      B. Ailts     Fixed problem with the output record
				        having control characters between valid
				        values,  Fixed problem when tie column
				        was in more than 2 subfiles.
    5.4	        01/91	   T. Mittan	Fixed problem with reading attributes
					which are only in subfiles


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Receive the input parameter values
Open the input labeled table file
Process the data
Close the labeled table file
Display the completion message

******************************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "las.h"
#include "ltable.h"
#include "util.h"
#include "tab2asc.h"
#include "protos.h"

int main
(void)

{

struct PARBLK parb;		/* TAE parameter block */

struct TAB_DEF tab[MAX_SUB + 1]; /* label table descriptor */

long i;				/* counter--current line */
long outcol[MAXCOL];		/* columns needed from input file */
long noutcol;			/* no. of cols needed from input file */
long lun;			/* fortran LUN to standard out */
long one = 1;			/* value of one */
long tiecol;			/*  tie column				*/
long totcol;			/*  total number of columns		*/

char atrname[MAXCOL][CMLEN];	/* attribute names			*/
char hosin[CMLEN];		/* host input name */
char hosout[1][CMLEN];		/* host output name */

 
/*  Initialize tae
------------------*/
c_inlas(&parb,&lun);

/*  Get parameter values
------------------------*/
getpar(&parb,hosin,hosout[0],&noutcol,&totcol,atrname,outcol,&tiecol);

/*  open output labeled table file
----------------------------------*/
labtabop(tab,hosin);

/*  Read one line(row) at a time and place in labeled table file
----------------------------------------------------------------*/
process(&parb,tab,hosout,outcol,&noutcol,&totcol,atrname,&tiecol);

/*  Close the input files.
--------------------------*/
close_tab(&tab[0]);
for (i = 1; i <= tab[0].nsubs; i++)
   close_tab(&tab[i]);

/*  Ouptut completion message
-----------------------------*/
c_complt("TAB2ASC",hosout,&one);

return EXIT_SUCCESS;

}
