/******************************************************************************

NAME:	TIE_COMP

FUNCTION:  TIE_COMP compares the tie column value with its previous value

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development
    5.1         08/89      B. Ailts     Fixed problem with string data on SUN-4

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Based on the data type of the values
Compare the values to see if they are the same as the previous value

******************************************************************************/

#include "las.h"
#include "ltable.h"
#include "tab2asc.h"
#include "protos.h"

void tie_comp
(
    struct VECTOR *ptr[],			/* output labeled table descriptor */
    short *read_flag,		/* logical specifying to read the next record
				   of the main labeled table file	*/
    long *tiecol,			/* tie column			*/
    union DATA *data,		/*  current values from the labeled table */
    union DATA *save_data,		/*  previous values of the tie column	*/
    long *outcol
)

{ /* tie_comp */

long k;				/* counter -- current column of matrix	*/
long j;				/* counter -- current row of matrix	*/

/* Based on data type of the tie column of the labeled table file, compare
   this value with the value of the previous record
---------------------------------------------------------------------------*/
switch (ptr[outcol[*tiecol]]->dtype)
   {
   case C: if (*ptr[outcol[*tiecol]]->ptr.c != save_data->chr)
	       *read_flag = TRUE;
            break;
   case I1: get_matrix(ptr[outcol[*tiecol]],I1,&data->byte);
   	     for ( j = 0; j < ptr[outcol[*tiecol]]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[outcol[*tiecol]]->lt_size.ncol; k++)
		   {
		   if (*(&data->byte + (k * j) + k) != 
			*(&save_data->byte + (k * j) + k))
	              *read_flag = TRUE;
		   }
             break;
   case I2: get_matrix(ptr[outcol[*tiecol]],I2,&data->i2);
	     for ( j = 0; j < ptr[outcol[*tiecol]]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[outcol[*tiecol]]->lt_size.ncol; k++)
		    {
		    if (*(&data->i2 + (k * j) + k) != 
			*(&save_data->i2 + (k * j) + k))
	              *read_flag = TRUE;
			   }
             break;
   case I4: get_matrix(ptr[outcol[*tiecol]],I4,&data->i4);
	    for ( j = 0; j < ptr[outcol[*tiecol]]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[outcol[*tiecol]]->lt_size.ncol; k++)
		   {
		   if (*(&data->i4 + (k * j) + k) != 
			*(&save_data->i4 + (k * j) + k))
		              *read_flag = TRUE;
		   }
             break;
   case R4: get_matrix(ptr[outcol[*tiecol]],R4,&data->r4);
	    for ( j = 0; j < ptr[outcol[*tiecol]]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[outcol[*tiecol]]->lt_size.ncol; k++)
		   {
		   if (*(&data->r4 + (k * j) + k) != 
			*(&save_data->r4 + (k * j) + k))
	              *read_flag = TRUE;
		   }
             break;
   case R8: get_matrix(ptr[outcol[*tiecol]],R8,&data->r8);
	     for ( j = 0; j < ptr[outcol[*tiecol]]->lt_size.nrow; j++)
	        for ( k = 0; k < ptr[outcol[*tiecol]]->lt_size.ncol; k++)
		   {
		   if (*(&data->r8 + (k * j) + k) != 
			*(&save_data->r8 + (k * j) + k))
	              *read_flag = TRUE;
		   }
             break;
   case LTDATE: get_lt_date(ptr[outcol[*tiecol]],&data->date);
		 if ((data->date.month != save_data->date.month) ||
		      (data->date.day != save_data->date.day) ||
		      (data->date.year != save_data->date.year))
	              *read_flag = TRUE;
                 break;
   case LTTIME: get_lt_time(ptr[outcol[*tiecol]],&data->time);
		 if ((data->time.hour != save_data->time.hour) ||
		      (data->time.minute != save_data->time.minute) ||
		      (data->time.second != save_data->time.second))
	              *read_flag = TRUE;
                 break;
   } /*  switch  prt[outcol[*tiecol]].dtype  */
}  /* tie_comp  */
