/******************************************************************************

NAME:	CHECK_LT_ATTRIB

FUNCTION:
	This is a routine which ensures the specified attribute names 
        exist in the LT file.  It also error checks for the attribute data 
        type and matrix size.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

    	Verify the attribute for X location exists and is a scalar value.
    	Verify the attribute for Y location exists and is a scalar value.
   	If processing polygon data
    	    Verify the attribute for region number exists and is a scalar value.
   	If processing line data
    	    Verify the attribute for arc number exists and is a scalar value.
   	If processing annotation data
    	    Verify the attribute for annotation size exists and is a scalar 
		value.
    	    Verify the attribute annotation angle exists and is a scalar value.
    	    Verify the attribute for annotation text exists and has a data 
		type of character.
    	Return to the calling routine.

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

void check_lt_attrib
(
    char *ftype,			/* GOF file type being processed (I)  */
    struct TAB_DEF *regxy,			/* LT data table structure (I) 	      */
    int gof_fd,			/* GOF file descriptor (I)	      */
    char *xname,                 /* X attribute name (I)               */
    char *yname,                 /* Y attribute name (I)               */
    char *ranum,                 /* Region or arc number attribute 
					   name (I)           		      */
    char *size_name,             /* Annotation size attribute name (I) */
    char *angle_name,            /* Annotation angle attribute name (I)*/
    char *annot_name            /* Annotation string attribute name 
					   (I)				      */
)
{
extern	struct 	PARBLK	vblock;		/* TAE parameter block                */
extern	long	nout;			/* Number of output files created     */
extern	char 	hostgof[1][CMLEN]; 	/* HOST name of GOF		      */

struct 	VECTOR 	*ptr;			/* Pointer to LT attribute	      */

long noabort = 1;			/* Report and continue flag for error 
					   messages                           */

char mesg[ERRLEN];			/* Error message text		      */

/*
   Verify each specified attribute exists, has the right data type and that
   that the matrix defining the data is the right size.
*/
if ((ptr = get_field_ptr(regxy,xname)) == NULL)
   {
   sprintf(mesg,"LT attribute name %s not found",xname);
   c_errmsg(mesg,"tab2gof-nofind",&noabort);
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }
if ((ptr->lt_size.nrow != 1) || (ptr->lt_size.ncol != 1))
   {
   sprintf(mesg,"LT attribute %s must be a scalar value",xname);
   c_errmsg(mesg,"tab2gof-scale",&noabort);
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }

if ((ptr = get_field_ptr(regxy,yname)) == NULL)
   {
   sprintf(mesg,"LT attribute name %s not found",yname);
   c_errmsg(mesg,"tab2gof-nofind",&noabort);
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }
if ((ptr->lt_size.nrow != 1) || (ptr->lt_size.ncol != 1))
   {
   sprintf(mesg,"LT attribute %s must be a scalar value",yname);
   c_errmsg(mesg,"tab2gof-scale",&noabort);
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }

if (strcmp(ftype,"POLY") == 0) 
   {
   if ((ptr = get_field_ptr(regxy,ranum)) == NULL)
      {
      sprintf(mesg,"LT attribute name %s not found",ranum);
      c_errmsg(mesg,"tab2gof-nofind",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   if ((ptr->lt_size.nrow != 1) || (ptr->lt_size.ncol != 1))
      {
      sprintf(mesg,"LT attribute %s must be a scalar value",ranum);
      c_errmsg(mesg,"tab2gof-scale",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   }

if (strcmp(ftype,"LINE") == 0) 
   {
   if ((ptr = get_field_ptr(regxy,ranum)) == NULL)
      { 
      sprintf(mesg,"LT attribute name %s not found",ranum);
      c_errmsg(mesg,"tab2gof-nofind",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   if ((ptr->lt_size.nrow != 1) || (ptr->lt_size.ncol != 1))
      {
      sprintf(mesg,"LT attribute %s must be a scalar value",ranum);
      c_errmsg(mesg,"tab2gof-scale",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   }

if (strcmp(ftype,"ANNOT") == 0)
   {
   if ((ptr = get_field_ptr(regxy,size_name)) == NULL)
      {
      sprintf(mesg,"LT attribute name %s not found",size_name);
      c_errmsg(mesg,"tab2gof-nofind",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   if ((ptr->lt_size.nrow != 1) || (ptr->lt_size.ncol != 1))
      {
      sprintf(mesg,"LT attribute %s must be a scalar value",size_name);
      c_errmsg(mesg,"tab2gof-scale",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }

   if ((ptr = get_field_ptr(regxy,angle_name)) == NULL)
      {
      sprintf(mesg,"LT attribute name %s not found",angle_name);
      c_errmsg(mesg,"tab2gof-nofind",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   if ((ptr->lt_size.nrow != 1) || (ptr->lt_size.ncol != 1))
      {
      sprintf(mesg,"LT attribute %s must be a scalar value",angle_name);
      c_errmsg(mesg,"tab2gof-scale",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }

   if ((ptr = get_field_ptr(regxy,annot_name)) == NULL)
      {
      sprintf(mesg,"LT attribute name %s not found",annot_name);
      c_errmsg(mesg,"tab2gof-nofind",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   if (ptr->dtype != C)
      {
      sprintf(mesg,"LT attribute %s must have data type of character",
         annot_name);
      c_errmsg(mesg,"tab2gof-char",&noabort);
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   }
return;
}
