/******************************************************************************

NAME:	GET_LT_TABDEF	

FUNCTION:
	This function returns the pointers to the LT structure definitions 
	for the attribute and data LT files.  If the input LT is the attribute
	file and is not the same file as the data file, one of the subfiles 
	must contain the data.  If the input file is the data file, one of the
	subfiles may contain the attributes.  If not, it is assumed no 
	attribute file exists.  Finally the input file may contain both the 
	attribute values and the data values.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	If the input LT is the attribute file
	    Check each subfile for the data file.
	    If a data file was not found
	       	If the attributes and data are in the same file
		   Set the data pointer the same as the attribute pointer.
		Else
		   Print an error message and abort.
	Else if the input LT is the data file
	    Check each subfile for the attribute file.
	    If an attribute file is not found
	       Set the attribute pointer to null indicating there are no 
	          attributes to copy.
	Else
	    Print an error message indicating the input LT file was not found.
	Return to the calling routine.

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

void get_lt_tabdef
(
    struct TAB_DEF *tab,	  		/* Master LT file structure (I)       */
    char *attrib,	  	/* Attribute LT file type (I)         */
    char *data,	  		/* Data LT file type (I) 	      */
    struct TAB_DEF **attrib_def,  		/* Attribute LT structure to be 
					   returned (O) 		      */
    struct TAB_DEF **data_def,	  	/* Data LT structure to be returned
					   (O)				      */
    int gof_fd			/* GOF file descriptor (I)	      */
)
{
extern	struct 	PARBLK	vblock;		/* TAE parameter block                */
extern	long	nout;			/* Number of output files created     */
extern	char 	hostgof[1][CMLEN]; 	/* HOST name of GOF		      */

static 	struct 	TAB_DEF  sub;		/* LT subfile structure 	      */

	long 	index;			/* Loop counter 		      */
	long 	noabort = 1;		/* Report and continue flag for error 
					   messages                           */

	char 	msg[ERRLEN];		/* Error message buffer 	      */

/*
   If the input LT file is the attribute file, search for the data file.
   If the data file does not exist, check to see if both the attributes and
   data are contained in the same file.  If not, a fatal error is generated.
*/
if (strcmp(tab->ftype,attrib) == 0)
   {				
   *attrib_def = tab;	
   *data_def = NULL;
   for (index=0; index<tab->nsubs; index++)
      {
      open_tab(&sub,tab->subfile[index],0);
      if (strcmp(sub.ftype,data) == 0)
         {
         get_vector(&sub);
         *data_def = &sub;
         break;
         }
      close_tab(&sub);
      }
   if (*data_def == NULL)
      {
/*
   Are the attributes and data in the same file?
*/
      if (strcmp(tab->ftype,data) == 0)
	 *data_def = tab;
      else
         {
         sprintf(msg,"LT file type \"%s\" not found",data);
         c_errmsg(msg,"tab2gof-notfnd",&noabort);
         c_gofcls(&gof_fd);
         c_clean(&vblock,"tab2gof",hostgof,&nout);
         }
      }
   }
else if (strcmp(tab->ftype,data) == 0)
/*
   If the input LT file is the data file, search for the attribute file.
   If the attribute file does not exist, ATTRIB_DEF is set to null
   indicating no attributes are to be copied to the GOF.
*/
   {					
   *data_def = tab;		
   *attrib_def = NULL;	
   for (index=0;index<tab->nsubs;index++)
      {					
      open_tab(&sub,tab->subfile[index],0); 
      if (strcmp(sub.ftype,attrib) == 0)
         {
         get_vector(&sub);
         *attrib_def = &sub;
         break;
         }
      close_tab(&sub);
      }
   }
else
   {
   sprintf(msg,"Input file type \"%s\" not found",attrib);
   c_errmsg(msg,"tab2gof-invlt",&noabort);
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }
return;
}
