/******************************************************************************

NAME:	GET_XY_POINTS	

FUNCTION:
	This function reads the x,y points of the current polygon (or line) 
	from the LT, and stores them in the GOF structure PLREC.  The end 
	of the polygon (or line) is detected when the region number attribute 
	becomes a different value.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	Set the number of points to zero.
	Loop until a new polygon or line is read or end of file is reached
	    If the current polygon or line is not the same as the one
	      just read
		Return to the calling routine.
	    Increment the number of points.
	    If the number of points exceeds the maximum allowed
		Print an error message and abort.
	    Set the x,y values in the GOF structure.
	    Read the next set of points.
	Return to the calling routine.

******************************************************************************/
#define POLYPT 512              /* Number of points in a polygon for initial
                                   allocation */

#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

int get_xy_points
(
    struct TAB_DEF *xy,			/* Data LT file structure (I) 	      */
    struct VECTOR *ptr_reg,		/* Pointer to LT attribute REGNUM (I) */
    struct VECTOR *ptr_x,			/* Pointer to LT attribute X (I)      */
    struct VECTOR *ptr_y,			/* Pointer to LT attribute Y (I)      */
    long *cur_poly,		/* Current value of REGNUM or ARCNUM
					   (I/O)			      */
    struct POLIN *plrec,			/* GOF polygon (or line) structure
					   (I/O)    			      */
    int gof_fd			/* GOF file descriptor (I)	      */
)
{
extern	struct 	PARBLK	vblock;		/* TAE parameter block                */
extern	long	nout;			/* Number of output files created     */
extern	char 	hostgof[1][CMLEN]; 	/* HOST name of GOF		      */

	long	prev_poly;		/* Previous value of REGNUM or ARCNUM */
	long 	index;			/* Loop counter 		      */
	long 	status = 0; 	/* Function return status code        */

long polypt;

polypt = POLYPT;
/*
   Set the number of points to zero
*/
plrec->numpts = 0;
/*
   Extract points from the LT and copy them to the GOF structure until the
   region number or arc number changes.  If the value of the LT attribute 
   for region number or arc number is different than the current polygon or 
   line, return to the calling routine.
*/
for (index=0; ; index++)
   {		
   prev_poly = *cur_poly;
   get_matrix(ptr_reg,I4,(unsigned char *)cur_poly);
   if ((*cur_poly != prev_poly) && (prev_poly != -1)) 
      break;			
   plrec->numpts++;
/*
   Realloc space for points if needed
*/
   if (plrec->numpts > polypt)	
      {
      polypt += POLYPT;
      plrec->x = (float *)realloc(plrec->x, polypt * sizeof(float));
      plrec->y = (float *)realloc(plrec->y, polypt * sizeof(float));
      }
   get_matrix(ptr_x,R4,(unsigned char *)&plrec->x[index]);
   get_matrix(ptr_y,R4,(unsigned char *)&plrec->y[index]);
/*
   Read the next set of points from the LT.  If end of file is detected, return
   to the calling routine.
*/
   status = get_record(xy);
   if (status == E_EOF)
      break;				  
   else if (status != E_SUCC)
      {
      c_gofcls(&gof_fd);
      c_clean(&vblock,"tab2gof",hostgof,&nout);
      }
   }					 
return(status);
}
