/*******************************************************************************

NAME:	GETPAR

FUNCTION:
	Retrieve the user specified parameters.

ALGORITHM:

	Initialize LAS and TAE.
	Retrieve all user specified parameter.
	Retrieve the global variable MSGTIM.
	Return to the calling routine.

*******************************************************************************/
#include "las.h"
#include "gof.h"
#include "ltable.h"
#include "util.h"
#include "protos.h"

void getpar
(
    char *ftype,			/* GOF file type (O)			      */
    char *htab,			/* Labeled table host file name (O)   	      */
    char *appopt,		/* Append option (O)                          */
    char *inattrib[MAXATT+1],	/* List of attribute names to be processed (O)*/
    long *msgtim,		/* Number of seconds between processing
				   messages (O)				      */
    char *xname,			/* X attribute name (O)			      */
    char *yname,			/* Y attribute name (O)			      */
    char *lab_name,		/* Label attribute name (O)		      */
    char *num,			/* Region, arc, point or annotation number 
				   attribute name (O)	      		      */
    char *size_name,		/* Annotation size attribute name (O)	      */
    char *angle_name,		/* Annotation angle attribute name (O)	      */
    char *annot_name,		/* Annotation string attribute name (O)	      */
    char *attr_file,		/* Attribute file type (O)		      */
    char *data_file		/* Data file type (O)			      */
)
{
extern  struct  PARBLK  vblock; /* TAE parameter block                        */
extern  char    hostgof[CMLEN]; /* HOST name of GOF                           */

struct 	VARIABLE *val;		/* Pointer to a variable within TAE vblock    */

long	abort = -1;		/* Abort flag for error messages	      */
long	index;			/* Loop counter                               */
long	length;			/* String length			      */
long	lt_access = IREAD;	/* Access for labeled table file	      */
long	lun;			/* Logical unit number to standard out	      */
long	status;			/* Status indicator			      */
long	zero = 0;		/* Value of zero                              */

char    ext[11];                /* GOF extension                              */
long    access;

/*
   Initialize LAS and TAE 
*/
c_inlas(&vblock,&lun);

/*
   Retrieve input labeled table file name. 
*/
status = c_getnam(&vblock,"INLT",&zero,htab,"lt",&lt_access);
if (status != E_SUCC)
    c_errmsg ("Fatal error encountered","tab2gof-fatal",&abort);

/*
   Determine whether records are to be appended to an existing GOF.
*/
val = p_find(&vblock,"APPDFLG");
strcpy(appopt,SVAL(*val,0));

/*
   Determine which attribute names are to be copied to the GOF.
*/
val = p_find(&vblock,"ATRNAME");
for (index=0; index<val->v_count; index++)
   inattrib[index] = SVAL(*val,index);
inattrib[index] = NULL;

val = p_find(&vblock,"FTYPE");
strcpy(ftype,SVAL(*val,0));

/*
   Retrieve GOF name.
*/
if (appopt[0] == 'N')
   access = IWRITE;
else
   access = IUPDATE;

sprintf(ext,"gof%c%s",GOF_SEP,ftype);
length = (long) strlen(ext);
c_up2low(ext,&length);
status = c_getnam(&vblock,"OUTGOF",&zero,hostgof,ext,&access);
if (status != E_SUCC)
   c_errmsg ("Fatal error encountered","tab2gof-fatal",&abort);

/*
   Retrieve the names of the X and Y value attributes.
*/
val = p_find(&vblock,"LSCATR");
if (val->v_count == 0)
    {
    strcpy(xname,"X");
    strcpy(yname,"Y");
    }
else
    {
    strcpy(xname,SVAL(*val,1));
    length = (long) strlen(xname);
    c_low2up(xname,&length);
    strcpy(yname,SVAL(*val,0));
    length = (long) strlen(yname);
    c_low2up(yname,&length);
    }

/*
   Retrieve the name of the label attribute.
*/
val = p_find(&vblock,"LABATR");
if (val->v_count == 0)
    strcpy(lab_name,"LABEL");
else
    {
    strcpy(lab_name,SVAL(*val,0));
    length = (long) strlen(lab_name);
    c_low2up(lab_name,&length);
    }

if (strcmp(ftype,"POINT") == 0)
    {
/*
   Retrieve the attribute file type. 
*/
    val = p_find(&vblock,"ALTTYPE");
    if (val->v_count == 0)
        strcpy(attr_file,"PNTATT");
    else
        {
        strcpy(attr_file,SVAL(*val,0));
        length = (long) strlen(attr_file);
        c_low2up(attr_file,&length);
        }

/*
   Retrieve the data file type. 
*/
    val = p_find(&vblock,"DLTTYPE");
    if (val->v_count == 0)
        strcpy(data_file,"PNTXY");
    else
        {
        strcpy(data_file,SVAL(*val,0));
    	length = (long) strlen(data_file);
    	c_low2up(data_file,&length);
	}
/*
   Retrieve the name of the point number attribute.
*/
    val = p_find(&vblock,"NUMATR");
    if (val->v_count == 0)
    	strcpy(num,"PNTNUM");
    else
	{
    	strcpy(num,SVAL(*val,0));
    	length = (long) strlen(num);
    	c_low2up(num,&length);
	}
    }
else if (strcmp(ftype,"LINE") == 0)
    {
/*
   Retrieve the attribute file type. 
*/
    val = p_find(&vblock,"ALTTYPE");
    if (val->v_count == 0)
        strcpy(attr_file,"ARCATT");
    else
	{
        strcpy(attr_file,SVAL(*val,0));
        length = (long) strlen(attr_file);
        c_low2up(attr_file,&length);
	}
/*
   Retrieve the data file type. 
*/
    val = p_find(&vblock,"DLTTYPE");
    if (val->v_count == 0)
        strcpy(data_file,"ARCXY");
    else
	{
        strcpy(data_file,SVAL(*val,0));
    	length = (long) strlen(data_file);
    	c_low2up(data_file,&length);
	}

/*
   Retrieve the name of the arc number attribute. 
*/
    val = p_find(&vblock,"NUMATR");
    if (val->v_count == 0)
    	strcpy(num,"ARCNUM");
    else
	{
    	strcpy(num,SVAL(*val,0));
    	length = (long) strlen(num);
    	c_low2up(num,&length);
	}
    }
else if (strcmp(ftype,"POLY") == 0)
    {
/*
   Retrieve the attribute file type. 
*/
    val = p_find(&vblock,"ALTTYPE");
    if (val->v_count == 0)
        strcpy(attr_file,"REGATT");
    else
	{	
        strcpy(attr_file,SVAL(*val,0));
        length = (long) strlen(attr_file);
    	c_low2up(attr_file,&length);
	}

/*
   Retrieve the data file type. 
*/
    val = p_find(&vblock,"DLTTYPE");
    if (val->v_count == 0)
        strcpy(data_file,"REGXY");
    else
	{
        strcpy(data_file,SVAL(*val,0));
    	length = (long) strlen(data_file);
    	c_low2up(data_file,&length);
	}

/*
   Retrieve the name of the region number attribute. 
*/
    val = p_find(&vblock,"NUMATR");
    if (val->v_count == 0)
    	strcpy(num,"REGNUM");
    else
	{
    	strcpy(num,SVAL(*val,0));
    	length = (long) strlen(num);
    	c_low2up(num,&length);
	}
    }
else if (strcmp(ftype,"ANNOT") == 0)
    {
/*
   Retrieve the attribute file type. 
*/
    val = p_find(&vblock,"ALTTYPE");
    if (val->v_count == 0)
        strcpy(attr_file,"ANNOTATT");
    else
	{
        strcpy(attr_file,SVAL(*val,0));
    	length = (long) strlen(attr_file);
    	c_low2up(attr_file,&length);
	}

/*
   Retrieve the data file type. 
*/
    val = p_find(&vblock,"ALTTYPE");
    if (val->v_count == 0)
        strcpy(data_file,"ANNOTXY");
    else
	{
        strcpy(data_file,SVAL(*val,0));
    	length = (long) strlen(data_file);
    	c_low2up(data_file,&length);
	}

/*
   Retrieve the name of the annotation number attribute. 
*/
    val = p_find(&vblock,"NUMATR");
    if (val->v_count == 0)
    	strcpy(num,"ANNOTNUM");
    else
	{
    	strcpy(num,SVAL(*val,0));
    	length = (long) strlen(num);
    	c_low2up(num,&length);
	}

/*
   Retrieve the name of the annotation size attribute.
*/
    val = p_find(&vblock,"SIZATR");
    if (val->v_count == 0)
    	strcpy(size_name,"SIZE");
    else
	{
    	strcpy(size_name,SVAL(*val,0));
    	length = (long) strlen(size_name);
    	c_low2up(size_name,&length);
	}

/*
   Retrieve the name of the annotation angle attribute.
*/
    val = p_find(&vblock,"ANGATR");
    if (val->v_count == 0)
        strcpy(angle_name,"ANGLE");
    else
	{
        strcpy(angle_name,SVAL(*val,0));
    	length = (long) strlen(angle_name);
    	c_low2up(angle_name,&length);
	}

/*
   Retrieve the name of the annotation text attribute.
*/
    val = p_find(&vblock,"TEXTATR");
    if (val->v_count == 0)
    	strcpy(annot_name,"ANNOT");
    else
	{
    	strcpy(annot_name,SVAL(*val,0));
    	length = (long) strlen(annot_name);
    	c_low2up(annot_name,&length);
	}
    }

/*
   Retrieve the global variable MSGTIM. 
*/
val = p_find(&vblock,"$MSGTIM");
*msgtim = IVAL(*val,0);

return;
}
