#ifndef PROTOS_H
#define PROTOS_H

void annotation
(
    int gof_fd,			/* GOF file descriptor (I) 	      */
    struct TAB_DEF *tab,	/* Master LT file structure (I)       */
    char *inattrib[],    	/* Array of attribute names to be 
					           processed (I)                      */
    long msgtim,			/* Number of seconds between messages (I) */
    char *xname,         	/* X attribute name (I)               */
    char *yname,         	/* Y attribute name (I)               */
    char *lab_name,      	/* Label attribute name (I)           */
    char *num,           	/* Region, arc, point or annotation 
					           number attribute name (I)          */
    char *size_name,        /* Annotation size attribute name (I) */
    char *angle_name,       /* Annotation angle attribute name (I)*/
    char *annot_name,       /* Annotation string attribute name(I)*/
    char *attr_file,     	/* Attribute file type (I)            */
    char *data_file     	/* Data file type (I)                 */
);

void check_lt_attrib
(
    char *ftype,		    	/* GOF file type being processed (I)  */
    struct TAB_DEF *regxy,   	/* LT data table structure (I) 	      */
    int gof_fd,			    /* GOF file descriptor (I)	      */
    char *xname,                /* X attribute name (I)               */
    char *yname,                /* Y attribute name (I)               */
    char *ranum,                /* Region or arc number attribute name (I) */ 
    char *size_name,            /* Annotation size attribute name (I) */
    char *angle_name,           /* Annotation angle attribute name (I)*/
    char *annot_name            /* Annotation string attribute name (I) */ 
);

void copy_attrib
(
    char *ftype,		/* GOF file type being processed (I) 	      */
    struct VECTOR *attrib[],	/* Array of LT attribute pointers (I) 	      */
    struct VECTOR *label,		/* LT attribute pointer to LABEL (I) 	      */
    union GOF *gofrec,	/* GOF union for all file types (I) 	      */
    int gof_fd,		/* GOF file descriptor (I) 		      */
    char *lab_name      /* Label attribute name (I)                   */
);

void get_lt_tabdef
(
    struct TAB_DEF *tab,	  		/* Master LT file structure (I)       */
    char *attrib,	             	/* Attribute LT file type (I)         */
    char *data,	  		            /* Data LT file type (I) 	      */
    struct TAB_DEF **attrib_def,  	/* Attribute LT structure to be 
					                   returned (O) 		      */
    struct TAB_DEF **data_def,	  	/* Data LT structure to be returned (O) */
    int gof_fd			            /* GOF file descriptor (I)	      */
);

int get_xy_points
(
    struct TAB_DEF *xy,			/* Data LT file structure (I) 	      */
    struct VECTOR *ptr_reg,		/* Pointer to LT attribute REGNUM (I) */
    struct VECTOR *ptr_x,			/* Pointer to LT attribute X (I)      */
    struct VECTOR *ptr_y,			/* Pointer to LT attribute Y (I)      */
    long *cur_poly,		/* Current value of REGNUM or ARCNUM
					   (I/O)			      */
    struct POLIN *plrec,			/* GOF polygon (or line) structure
					   (I/O)    			      */
    int gof_fd			/* GOF file descriptor (I)	      */
);

void getpar
(
    char *ftype,			/* GOF file type (O)			      */
    char *htab,			    /* Labeled table host file name (O)   	      */
    char *appopt,		    /* Append option (O)                          */
    char *inattrib[MAXATT+1],	/* List of attribute names to be processed (O)*/
    long *msgtim,		    /* Number of seconds between processing
				               messages (O)				      */
    char *xname,			/* X attribute name (O)			      */
    char *yname,			/* Y attribute name (O)			      */
    char *lab_name,		    /* Label attribute name (O)		      */
    char *num,		    	/* Region, arc, point or annotation number 
				               attribute name (O)	      		      */
    char *size_name,		/* Annotation size attribute name (O)	      */
    char *angle_name,		/* Annotation angle attribute name (O)	      */
    char *annot_name,		/* Annotation string attribute name (O)	      */
    char *attr_file,		/* Attribute file type (O)		      */
    char *data_file	    	/* Data file type (O)			      */
);

void point
(
    int gof_fd,	     /* GOF file descriptor (I) 		      */
    struct TAB_DEF *tab, /* LT file structure (I) 		      */
    char *inattrib[],    /* Array of attribute names to be processed (I) */
    long msgtim,		 /* Number of seconds between processing 
				            messages (I)				      */
    char *xname,         /* X attribute name (I)                       */
    char *yname,         /* Y attribute name (I)                       */
    char *lab_name,      /* Label attribute name (I)                   */
    char *num,           /* Region, arc, point or annotation number
                            attribute name (I)                         */
    char *attr_file,     /* Attribute file type (I)                    */
    char *data_file      /* Data file type (I)                         */
);

void poly_line
(
    int gof_fd,		 /* GOF file descriptor (I) 		      */
    char *ftype,		 /* GOF file type (POLY or LINE) (I) 	      */
    struct TAB_DEF *tab, /* LT master file structure (I) 	      */
    char *inattrib[],    /* Array of attribute names to be processed
				            (I)  				      */
    long msgtim,	     /* Number of seconds between processing 
				            messages (I)				      */
    char *xname,         /* X attribute name (I)                       */
    char *yname,         /* Y attribute name (I)                       */
    char *lab_name,      /* Label attribute name (I)                   */
    char *num,           /* Region, arc, point or annotation number 
				            attribute name (I)    		      */
    char *attr_file,     /* Attribute file type (I)                    */
    char *data_file      /* Data file type (I)                         */
);

void put_header
(
    int gof_fd,			    /* GOF file descriptor (I) 	      */
    struct TAB_DEF *regatt,		/* Attribute LT structure (I)         */
    char *inattrib[],		    /* Array of attribute names (I)       */
    struct VECTOR *attrib[],	/* Array of LT attribute pointers (O) */
    struct VECTOR **label,		/* Pointer to LT attribute LABEL (O)  */
    char *lab_name,             /* Label attribute name (I)           */
    char *num			        /* Region, arc, point or annotation
					               number (I)			      */
);

void update_header
(
    int gof_fd,		/* GOF file descriptor (I)	      */
    long nrec			/* Number of records added to the GOF
				     	   (I)				      */
);

#endif

