/******************************************************************************

NAME:	TAB2GOF

FUNCTION:
	TAB2GOF is an application program which converts a Labeled Table (LT)
	file to a DMS Graphics Overlay File (GOF).

PROGRAM HISTORY:
PROGRAMMER      DATE    CODE/CONT       REASON
----------      ----    ---------       ------
K. Gacke	4/87	EDC		Original development 
L. Huewe 	1/89    EDC		DMS v3_0 retrocode.
L. Huewe	12/90	EDC		Modified to allow the user to specify 
					attribute names rather than hard coding 
					them (PRF #5606).
D. Etrheim      3/93    EDC             Convert to a LAS application

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	Retrieve the input parameters.
	Open the labeled table file and retrieve the label vector.
	If the GOF type is POLY 
	    Open the GOF for update access.
	    If the open failed, print an error message and abort.
	    If the file already exists and the parameter APPDFLG is NO
		Print an error message and abort.
	    Get the fully qualified name of the GOF.
	    Copy attribute and polygon data from the LT to the GOF.
	Else if the GOF type is LINE 
	    Open the GOF for update access.
	    If the open failed, print an error message and abort.
	    If the file already exists and the parameter APPDFLG is NO
		Print an error message and abort.
	    Get the fully qualified name of the GOF.
	    Copy attribute and line data from the LT to the GOF.
	Else if the GOF type is POINT 
	    Open the GOF for update access.
	    If the open failed, print an error message and abort.
	    If the file already exists and the parameter APPDFLG is NO
		Print an error message and abort.
	    Get the fully qualified name of the GOF.
	    Copy attribute and point data from the LT to the GOF.
	Else if the GOF type is ANNOT 
	    Open the GOF for update access.
	    If the open failed, print an error message and abort.
	    If the file already exists and the parameter APPDFLG is NO
		Print an error message and abort.
	    Get the fully qualified name of the GOF.
	    Copy attribute and annotation data from the LT to the GOF.
	Close the GOF.
	Write a completion message to the user's terminal.

******************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

struct 	PARBLK	vblock;		/* TAE parameter block                        */
long	nout;			/* Number of output files created	      */
char 	hostgof[1][CMLEN];		/* HOST name of GOF			      */

int main
(void)
{
struct	TAB_DEF	 tab;		/* Definition structure of Labeled Table      */

long	abort = -1;		/* Abort flag for error messages	      */
int 	cflag;			/* Flag indicating whether a GOF was created  */
long	create = FALSE;		/* Flag indicating whether a GOF was created  */
int 	gof_access = UPDATE;	/* GOF file is opened for update access       */
int 	gof_fd;			/* GOF file descriptor                        */
long 	msgtim;			/* Number of seconds between processing msg   */
long	one = 1;		/* Value of one				      */
long	status;			/* Function return status code                */
long	zero = 0;		/* Value of zero                              */

char	angle_name[CMLEN];	/* Annotation angle attribute name    	      */
char	annot_name[CMLEN];	/* Annotation string attribute name           */
char    appopt[4];		/* Append option                              */
char	attr_file[CMLEN];	/* Attribute file type    		      */
char	data_file[CMLEN];	/* Data file type    			      */
char	ftype[6];		/* GOF file type			      */
char	htab[CMLEN];		/* Labeled table host file name    	      */
char	*inattrib[MAXATT+1];	/* List of attribute names to be processed    */
char	lab_name[CMLEN];	/* Label attribute name    		      */
char	lgof[1][CMLEN];		/* GOF LAS name               	              */
char	num[CMLEN];		/* Region, arc, point or annotation number 
				   attribute name    	      		      */
char	size_name[CMLEN];	/* Annotation size attribute name    	      */
char	xname[CMLEN];		/* X attribute name    			      */
char	yname[CMLEN];		/* Y attribute name    			      */
char    tmpstr[1][CMLEN];   /* Temp string */

/*
   Get the user specified parameters.
*/
getpar(ftype,htab,appopt,inattrib,&msgtim,xname,yname,lab_name,
    num,size_name,angle_name,annot_name,attr_file,data_file);

/*
   Open the LT and retrieve the label vector.  The FTYPE chosen determines 
   the type of GOF to be created/updated.  The GOF is opened.  If the GOF 
   already exists and the input parameter APPDFLG is set to NO, a fatal 
   error message is generated.
*/
open_tab(&tab,htab,0);
get_vector(&tab);
nout = 0;

status = c_gofopn(&gof_fd,hostgof[0],lgof[0],ftype,&gof_access,&cflag);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tab2gof-fatal",&abort);
else if (cflag)
   {
   create = TRUE;
   nout = 1;
   }

if (strcmp(ftype,"POLY") == 0)
   poly_line(gof_fd,ftype,&tab,inattrib,msgtim,xname,yname,lab_name,num,
       attr_file,data_file);
else if (strcmp(ftype,"LINE") == 0)
   poly_line(gof_fd,ftype,&tab,inattrib,msgtim,xname,yname,lab_name,num,
       attr_file,data_file);
else if (strcmp(ftype,"POINT") == 0)
   point(gof_fd,&tab,inattrib,msgtim,xname,yname,lab_name,num,attr_file,
        data_file);
else if (strcmp(ftype,"ANNOT")==0) 
   annotation(gof_fd,&tab,inattrib,msgtim,xname,yname,lab_name,num,size_name,
       angle_name,annot_name,attr_file,data_file);

if (c_gofcls(&gof_fd) != E_SUCC)
   c_errmsg ("Fatal error encountered","tab2gof-fatal",&abort);
if (create)
    c_complt("TAB2GOF",lgof,&one);
else
  {
  strcpy(tmpstr[0], "\0");
  c_complt("TAB2GOF",tmpstr,&zero);
  }

return EXIT_SUCCESS;

}
