/******************************************************************************

NAME:	UPDATE_HEADER	

FUNCTION:
	This function updates the number of records field in the GOF 
	header record.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM:

	Position the file pointer at the beginning of the GOF.
	Read the header record.
	If the read fails, print an error message and abort.
	Increment the number of records field by the amount input.
	Position the file pointer at the beginning of the GOF.
	Write the header record with the updated number of records field.
	If the write fails, print an error message and abort.
	Return to the calling routine.

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "gof.h"
#include "util.h"
#include "protos.h"

void update_header
(
    int gof_fd,			/* GOF file descriptor (I)	      */
    long nrec			/* Number of records added to the GOF
					   (I)				      */
)
{
extern	struct 	PARBLK	vblock;		/* TAE parameter block                */
extern	long	nout;			/* Number of output files created     */
extern	char 	hostgof[1][CMLEN]; 	/* HOST name of GOF		      */

static 	union	GOF	gofrec;		/* GOF union for all file types       */

	int 	addrec = FALSE;		/* GOF write boolean flag             */
	int 	gof_type = HDR;		/* GOF record type to be written      */
	int  	length;			/* GOF record length 		      */
	int 	obyte;			/* Number of bytes read from GOF      */

/*
   Set the file pointer to the beginning of the GOF.
*/
lseek(gof_fd,0,0);
/*
   Read the GOF header record.
*/
if (c_gofrd(&gof_fd,&obyte,&length,&gof_type,(struct DDR *)NULL,&gofrec) != E_SUCC)
   {
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }
/*
   Increment the number of records.
*/
gofrec.hdrec.nrec += nrec;

/*
   Set the file pointer back to the beginning of the file.
*/
lseek(gof_fd,0,0);
/*
   Rewrite the header record with the updated number of records field.
*/
if (c_gofwr(&gof_fd,&length,&gof_type,(struct DDR *)NULL, &gofrec,&addrec) != E_SUCC)
   {
   c_gofcls(&gof_fd);
   c_clean(&vblock,"tab2gof",hostgof,&nout);
   }
return;
}
