/*****************************************************************************
*
*  NAME: GET_DATA
*
*  FUNCTION: Read attributes from the labeled table
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include "las.h"
#include "tab2ras.h"
#include "ltable.h"
#include "pixman.h"
#include "protos.h"

void get_data
(
    long **array,               /* O: array where values are to be placed */
    struct VECTOR *ptr,         /* I: */
    long *index,                /* I: index into the array */
    short *buf_ind              /* I: number of times array has been expanded */
)

{
    long dtype = ELONG;     /* data type of the values                      */
    long fatal = -1;        /* for fatal error messages                     */
    long non_fatal = 1;     /* for nonfatal error messages                  */
    long ns;                /* number of elements                           */
    long *tmp_array;        /* temporary new buffer                         */

    /* If there is enough buffer space read the next value
       ------------------------------------------------------*/
    if (*index < BUF_INC * *buf_ind)
        get_matrix(ptr,I4,(unsigned char *)(*array + *index));

    /* Allocate more space and copy the existing elements into the new buffer
       ----------------------------------------------------------------------*/
    else
    {
        (*buf_ind)++;
        tmp_array = (long *)malloc(*buf_ind * BUF_INC * sizeof(long));
        if (tmp_array == NULL)
        {
            c_errmsg("Error allocating dynamic memory","tab2ras-alloc",
                     &non_fatal);
            c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
        }

        ns = *index;
        c_pxcopy((unsigned char *)*array,(unsigned char *)tmp_array,&dtype,&ns);  

        /*  Free the old buffer
            -----------------------*/
        free(*array);

        *array = tmp_array;
        get_matrix(ptr,I4,(unsigned char *)(*array + *index));
    }

    return;
}
