/*****************************************************************************
*
*  NAME: GET_PROJ
*
*  FUNCTION: Read the DDR Projection information from the labeled table 
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include "las.h"
#include "tab2ras.h"
#include "ltable.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

void get_proj
(
    struct TAB_DEF *tab,                /* I: table descriptor */
    long *tab_ind,                      /* I: tab index */
    struct VECTOR *ptr[],               /* I: */
    char hosname[]                      /* I: temporary file names */
)

{ 
    struct DDR ddr;
    struct BDDR bddr;
    long valid;

    long warn = 1;
    long one = 1;

    c_getddr(hosname,&ddr);
    if (c_getbdr(hosname, &bddr,&one) != E_SUCC)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);


    if (ptr[PPARMS] != NULL)
        get_record(&tab[tab_ind[PPARMS]]);
    else if (ptr[PVALID] != NULL)
        get_record(&tab[tab_ind[PVALID]]);
    else if (ptr[PCODE] != NULL)
        get_record(&tab[tab_ind[PCODE]]);
    else if (ptr[PZONE] != NULL)
        get_record(&tab[tab_ind[PZONE]]);
    else if (ptr[PDATUM] != NULL)
        get_record(&tab[tab_ind[PDATUM]]);
    else if (ptr[UNITS] != NULL)
        get_record(&tab[tab_ind[UNITS]]);

    if (ptr[PPARMS] != NULL)
        get_matrix(ptr[PPARMS],R8,(unsigned char *)ddr.proj_coef);

    if (ptr[PVALID] != NULL)
    {
        get_matrix(ptr[PVALID],I4,(unsigned char *)&valid);
        ddr.valid[0] = valid;
        ddr.valid[1] = valid;
        ddr.valid[2] = valid;
        ddr.valid[3] = valid;
        ddr.valid[6] = valid;
    }

    if (ptr[PCODE] != NULL)
        get_matrix(ptr[PCODE],I4,(unsigned char *)&ddr.proj_code);
    if (ptr[PZONE] != NULL)
        get_matrix(ptr[PZONE],I4,(unsigned char *)&ddr.zone_code);
    if (ptr[PDATUM] != NULL)
        get_matrix(ptr[PDATUM],I4,(unsigned char *)&ddr.datum_code);

    if (ptr[UNITS] != NULL)
    {
        if (c_string2unit(ptr[UNITS]->ptr.c) == E_FAIL)
        {
            c_lasmsg(FALSE, "tab2ras-warn", 
                    "Projection Units %s are not supported", ptr[UNITS]->ptr.c);
        }
        else
        {
            strcpy(ddr.proj_units,ptr[UNITS]->ptr.c);
        }
    }     

    if (c_putddr(hosname,&ddr) != E_SUCC)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&warn);
    if(c_putbdr(hosname,&bddr) == E_FAIL)
        c_errmsg("Nonfatal error encountered","tab2ras-warn",&one);

    return;
}
