/*****************************************************************************
*
*  NAME: GTRECI
*
*  FUNCTION: Read a record from the sorted select file
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"
#include  "protos.h"

int gtreci 
(
    FILE *selfp,                /* I: file descriptor for sorted select file */
    long *lineid,               /* O: line id */
    double *value1,             /* I: first value */
    double *value2,             /* I: second value */
    unsigned char *header,      /* O: flag if header record */
    long *prior                 /* O: line priority */
)

{
    long nbytes = REC_SIZE;         /* number of bytes to read from the file */
    long seq;                       /* sequence number */

    char temp[REC_SIZE];            /* buffer for file reads */     
    char *tempptr;                  /* char pointer */

    /* Initialize header and pointer 
       ------------------------------*/
    *header = FALSE;
    tempptr = &temp[0];

    /* Get the next sorted select file record
       -----------------------------------------*/
    if (fgets(temp,nbytes,selfp) == NULL)
        return(E_EOF);
    else
    {
        *lineid = atol(tempptr);
        tempptr += INTLEN;
        seq = atol(tempptr);
        tempptr += INTLEN + 2; /*one is added for the space between numerals */
        sscanf(tempptr,"%lf",value1);                /* atof didn't work???? */
        tempptr += DBLLEN;
        sscanf(tempptr,"%lf",value2);                /* atof didn't work???? */

        /* Determine if this is a header record for a new line segment 
           If so, get the priority.
           -------------------------------------------------------------*/
        if (seq == 0)
        {
            *header = TRUE;
            tempptr += INTLEN; 
            *prior = atol(tempptr);
        }
    }

    return(E_SUCC);
}
