/*****************************************************************************
*
*  NAME: GTSEGI
*
*  FUNCTION: Read the end points of the line segemnt
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"
#include "util.h"
#include "protos.h"

void gtsegi
(
    FILE *fp,                   /* I: file descriptor for sorted select file  */
    char tempfile[][CMLEN],     /* I: temporary file names */
    long *linid,                /* I: line id */
    double *x1,                 /* I: first x coordinate */
    double *y1,                 /* I: first y coordinate */
    double *x2,                 /* I: second x coordinate */
    double *y2                  /* I: second x coordinate */
)

{
    extern struct PARBLK parblk;

    long nfiles = NUMTEMPF;         /* no. of temporary file names */
    long nexlin;                    /* next line id read */
    long nonfatal = 1;              /* for nonfatal error messages */
    long status;                    /* function status */
    long dummy;                     /* dummy value for gtreci */

    unsigned char header;           /* flag if header record is read */
    char msg[ERRLEN];               /* error message buffer */

    /* Retrieve the starting coordinates of the line segment
       -------------------------------------------------------*/
    status = gtreci (fp, &nexlin, y1, x1, &header, &dummy);

    /* Validate the requested retrieval
       -----------------------------------*/
    if (status == E_EOF)
    {
        sprintf(msg,"%s %s %d",
                "Unexpected end of file encountered while retrieving",
                "a segment of line",*linid);
        c_errmsg(msg,"tab2ras-eof",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&nfiles);
    }

    if (header)
    {
        sprintf(msg,"%s %s %d",
                "Unexpected data record encountered while retrieving",
                "a segment of line",*linid);
        c_errmsg(msg,"tab2ras-newrec",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&nfiles);
    }

    if (*linid != nexlin)
    {
        sprintf(msg,"%s %s %d",
                "Unexpected data record encountered while retrieving",
                "a segment of line",*linid);
        c_errmsg(msg,"tab2ras-newrec",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&nfiles);
    }

    /* Retrieve the ending coordinates of the line segment
       -----------------------------------------------------*/
    status = gtreci(fp,&nexlin, y2, x2, &header, &dummy);

    /* Validate the requested retrieval
       -----------------------------------*/
    if (status == E_EOF)
    {
        sprintf(msg,"%s %s %d",
                "Unexpected end of file encountered while retrieving",
                "a segment of line",*linid);
        c_errmsg(msg,"tab2ras-eof",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&nfiles);
    }

    if (header)
    {
        sprintf(msg,"%s %s %d",
                "Unexpected header record encountered while retrieving",
                "a segment of line",*linid);
        c_errmsg(msg,"tab2ras-head",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&nfiles);
    }

    if (*linid != nexlin)
    {
        sprintf(msg,"%s %s %d",
                "Unexpected data record encountered while retrieving",
                "a line segment of line",*linid);
        c_errmsg(msg,"tab2ras-newrec",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&nfiles);
    }

    return;
}
