/*****************************************************************************
*
*  NAME: LINESEL
*
*  FUNCTION: Read the line end point coordinates from the labeled table
*            and write them to the select file
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"
#include "ltable.h"
#include "util.h"
#include "protos.h"

void linesel
(
    struct TAB_DEF *tab,        /* I: table descriptor */
    long *tab_ind,              /* I: tab index */
    struct VECTOR *ptr[],       /* I:  */
    short *id_flag,             /* I: flag if PTID is used instead of INDEX */
    long *pixval,               /* I/O: pixel value */
    char selfile[],             /* I/O: temporary file name */
    long *msgtim,               /* I: time for processing message */
    long *nfile,                /* I: number of LT files */ 
    long *maxweight             /* O: maximum weight            */
)

{
    extern struct PARBLK parblk;
    extern long curent;

    FILE *fp;                       /* File descriptor              */

    double x;                       /* coordinate value */
    double y;                       /* coordinate value */

    long *color_arr;                /* color array */
    long fatal = -1;                /* for fatal error messages */
    long id;                        /* line id */
    long *index_arr;                /* index array */
    long index;                     /* index value */
    long j = 0;                     /* current index value */
    long msgflg = GEN1;             /* flag for processing message */
    long nonfatal = 1;              /* for nonfatal error messages */
    long one = 1;                   /* constant of one */
    long prev_id;                   /* previous line processed */
    long *prior_arr;                /* priority array */
    long prior = 1;                 /* current priority value */    
    long seq;                       /* line sequence number */
    long status;                    /* function status */
    long *weight_arr;               /* weight array */
    long weight = 0;                /* current weight value */

    /*  Open temporary select file 
        -----------------------------*/
    strcpy(selfile,"tab2ras_sel");
    c_creat_name(selfile,"txt");
    if ((fp = fopen(selfile,"w")) == NULL)
    {
        c_errmsg("Error opening select file","tab2ras-open",&nonfatal);
        c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
    }

    /* Read index,pixval,weight,and color values into memory
       ------------------------------------------------------*/
    read_lt(tab,tab_ind,ptr,&index_arr,&color_arr,&weight_arr,&prior_arr,
            id_flag, nfile);

    /* Initialize processing message
       -------------------------------*/
    curent = 1;
    c_inittm("TAB2RAS","Selected","line"," "," ",&j,&j,msgtim,&msgflg);
    c_startm();

    /*  Read the first record from the labeled table that contains the 
        Y/X coordinates 
        ------------------------------------------------------------------*/
    status = get_record(&tab[tab_ind[IDCOOR]]);
    if (status == E_FAIL)
        c_clean(&parblk,"tab2ras",(char (*)[256])selfile,&one);
    get_coor(tab_ind,ptr,&y,&x,&id,&seq);
    prev_id = id;

    j = 1;
    /*  Initialize the attribute values
        -----------------------------------*/
    index = index_arr[j];
    if (tab_ind[LNWGT] != -1)
    {
        if (id_flag[LNWGT])
            weight = weight_arr[id] ;
        else
        {
            if (weight_arr[index] != 0)
                weight = (weight_arr[index]) ;
            else
                weight = weight_arr[index];
        }
    }
    if (tab_ind[PRIOR] != -1)
    {
        if (id_flag[PRIOR])
            prior = prior_arr[id];
        else
            prior = prior_arr[index];
    }
    if (tab_ind[COLOR] != -1)
    {
        if (id_flag[COLOR])
            *pixval = color_arr[id];
        else
            *pixval = color_arr[index];
    }

    /* Write the header record 
       -------------------------*/
    fprintf(fp,"%11d %11d %11d %12d %11d\n",id,0,weight,*pixval,prior);

    *maxweight = weight;

    /*  Read the Y/X coordinates until the EOF is found.  If a new line
        id is read, set the attribute values and write the header record 
        --------------------------------------------------------------------*/
    for (; status != E_EOF;)
    {
        if (id != prev_id)
        {
            prev_id = id;
            j++;
            index = index_arr[j];
            if (tab_ind[LNWGT] != -1)
            {
                if (id_flag[LNWGT])
                    weight = weight_arr[id] ;
                else
                {
                    if (weight_arr[index] != 0)
                        weight = (weight_arr[index]  ) ;
                    else 
                        weight = weight_arr[index];
                }
            }
            if (tab_ind[PRIOR] != -1)
            {
                if (id_flag[PRIOR])
                    prior = prior_arr[id];
                else
                    prior = prior_arr[index];
            }
            if (tab_ind[COLOR] != -1)
            {
                if (id_flag[COLOR])
                    *pixval = color_arr[id];
                else
                    *pixval = color_arr[index];
            }
            fprintf(fp,"%11d %11d %11d %12d %11d\n",id,0,weight,*pixval,prior);

            if (weight > *maxweight)
                *maxweight = weight;
        }

        /* Write the data record and read the next record
           -------------------------------------------------*/
        fprintf(fp,"%11d %11d %17.7lf %18.7lf\n",id,seq,y,x);
        status = get_record(&tab[tab_ind[IDCOOR]]);
        if (status == E_FAIL)
            c_clean(&parblk,"tab2ras",(char (*)[256])selfile,&one);
        get_coor(tab_ind,ptr,&y,&x,&id,&seq);
        curent++;
    }

    c_stoptm();

    /* Close the select file
       -----------------------*/
    fclose (fp);

    /* Free the buffers 
       -------------------*/
    free(index_arr);
    free(color_arr);
    free(prior_arr);
    free(weight_arr);

    return;
}
