/*****************************************************************************
*
*  NAME: LT_OPEN 
*
*  FUNCTION: Open the labeled table files
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include "las.h"
#include "tab2ras.h"
#include "ltable.h"
#include  "protos.h"

void ltopen
(
    struct TAB_DEF *tab,        /* I: table descriptor */
    char ltin[][CMLEN],         /* I: labeled table names */
    char atrname[][CMLEN],      /* I: attribute names */
    struct VECTOR *ptr[],       /* I:   */
    long *tab_ind,              /* O: tab index */
    short *id_flag,             /* O: flag if PTID is used instead of INDEX */
    long *nfile                 /* I: no. of lt files */
)

{

    long fatal = -1;                /* fatal error encountered */
    long i;                         /* loop counter */
    long nonfatal = 1;              /* non fatal error encountered */

    short j;                        /* loop counter */
    short found;                    /* flag if attribute is found */

    char msg[ERRLEN];               /* error message buffer */

    /*  Open the output labeled table file and get the subfile names
        if needed
        ----------------------------------------------------------------*/
    open_tab(&tab[0],ltin[0],IREAD);
    get_vector(&tab[0]);
    if (*nfile == 1)
    {
        for (i = 1; i <= tab[0].nsubs; i++)
        {
            strcpy(ltin[i],tab[0].subfile[i - 1]);
            (*nfile)++;
        }
    }

    for (i = 1; i < *nfile; i++)
    {
        open_tab(&tab[i],ltin[i],IREAD);
        get_vector(&tab[i]);
    }

    /* Initialize variables
       -----------------------*/
    for (i = 0; i < NUMATR; i++)
        ptr[i] = NULL;

    /* Locate the attribute names
       -----------------------------*/
    if (*nfile < 5 )
    {
        for (i = 0; i < NUMATR; i++)
        {
            found = FALSE;
            if (atrname[i][0] != '\0') 
            {
                for(j = (short)*nfile; j >= 0; j--)
                {
                    ptr[i] = (struct VECTOR *)get_field_ptr(&tab[j],atrname[i]);
                    if (ptr[i] != NULL)
                    {
                        found = TRUE;
                        tab_ind[i] = j;
                        break;
                    }
                }

                if (!found)
                {
                    sprintf(msg,"Attribute %s could not be found",atrname[i]);
                    c_errmsg(msg,"tab2ras-attribute",&nonfatal);
                    c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal); 
                }
            }
            else
                tab_ind[i] = -1;
        }
    }
    else
    {
        for (i = 0; i < PROJATR; i++)
        {
            found = FALSE;
            if (atrname[i][0] != '\0') 
            {
                for(j = (short)*nfile; j >= 0; j--)
                {
                    ptr[i] = (struct VECTOR *)get_field_ptr(&tab[j],atrname[i]);
                    if (ptr[i] != NULL)
                    {
                        found = TRUE;
                        tab_ind[i] = j;
                        break;
                    }
                }

                if (!found)
                {
                    sprintf(msg,"Attribute %s could not be found",atrname[i]);
                    c_errmsg(msg,"tab2ras-attribute",&nonfatal);
                    c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
                }
            }
            else
                tab_ind[i] = -1;
        }
    }

    /*  The values for the ID may occur in more the one labeled table file so it
        needs to have two pointers one for the attribute data and one for the
        coordinate data.
        ----------------------------------------------------------------------*/
    ptr[IDCOOR] = (struct VECTOR *)get_field_ptr(&tab[tab_ind[COOR1]],
            atrname[IDCOOR]);
    tab_ind[IDCOOR] = tab_ind[COOR1];
    if (ptr[IDCOOR] == NULL) 
    {
        sprintf(msg,"Attribute %s could not be found in the coordinate file",
                atrname[IDCOOR]);
        c_errmsg(msg,"tab2ras-idcoor",&nonfatal);
        c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal); 
    }

    /*  The index and id values must be in the same lt file in order to provide
        a link between the id and the primary attributes(color,weight,priority)
        ----------------------------------------------------------------------*/
    ptr[INDEX] = (struct VECTOR *)get_field_ptr(&tab[tab_ind[PTID]],
            atrname[INDEX]);
    tab_ind[INDEX] = tab_ind[PTID];
    if (ptr[INDEX] == NULL) 
    {
        sprintf(msg,"Attribute %s could not be found in same file as %s",
                atrname[INDEX],atrname[PTID]);
        c_errmsg(msg,"tab2ras-index",&nonfatal);
        c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
    }

    /*  If the attribute given for weight, color, or prior is in the same 
        labeled table as the ID, then it is assumed that weight, color, or 
        prior is not linked to the coordinates through the INDEX attribute 
        but through the ID attribute
        ----------------------------------------------------------------------*/
    if (tab_ind[PTID] == tab_ind[LNWGT])
        id_flag[LNWGT] = TRUE;
    else
        id_flag[LNWGT] = FALSE;

    if (tab_ind[PTID] == tab_ind[PRIOR])
        id_flag[PRIOR] = TRUE;
    else
        id_flag[PRIOR] = FALSE;

    if (tab_ind[PTID] == tab_ind[COLOR])
        id_flag[COLOR] = TRUE;
    else
        id_flag[COLOR] = FALSE;

    return;
}
