/*****************************************************************************
*
*  NAME: POLYINIT
*
*  FUNCTION: Initialize scan processing for polygon data
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"
#include "util.h"
#include "protos.h"

void polyinit
(
    char tempfile[][CMLEN],     /* I/O: temporary file names */
    double *minval,             /* I: minimum pixel value */
    double *maxval,             /* I: miximum pixel value */
    long *msgtim,               /* I: time for message processing */
    double xnw,                 /* I: x corner coordinate */
    double ynw,                 /* I: y corner coordinate */
    long *listflg,              /* I: flag for printing error messages */
    long nlines,                /* I: number of lines in output image */
    long nsamps,                /* I: number of samples in output image */
    float *pixsiz,              /* I: pixel size */
    double rawynw,              /* I: x corner coordinate */
    double offset               /* I: offset */
)

{
    extern struct PARBLK parblk;
    extern long curent;
    extern long curcat;

    FILE *scanfp;                   /* scan file descriptor */
    FILE *selfp;                    /* select descriptor */

    long linid;                     /* line id */
    long msgflg = GEN1;             /* flag for processing message */
    long lastid;                    /* last line id read */
    long nonfatal = 1;              /* flag for nonfatal errors */
    long numfile = NUMTEMPF;        /* number of temporary files */
    long status;                    /* function status */
    long tdum;                      /* temporary value */
    long zero = 0;                  /* constant of 0 */

    double ddum;                    /* temporary value */
    double ptl;                     /* pixel value to the left*/
    long ptr;                       /* pixel value to the right */
    double x1                       /* line endpoint coordinates */
        , x2
        , y1
        , y2;

    char msg[ERRLEN];               /* error message buffer */
    unsigned char header;           /* flag if header record */

    /* Open the sorted select file
       -----------------------------*/
    if ((selfp = fopen(tempfile[SORTSEL],"r")) == NULL)
    {
        c_errmsg("Error opening sorted select file","tab2ras-open",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }

    /* Open the temporary scan file
       -------------------------------*/
    strcpy(tempfile[SCAN],"tab2ras_scan");
    c_creat_name(tempfile[SCAN],"txt");
    if ((scanfp = fopen(tempfile[SCAN],"w")) == NULL)
    {
        c_errmsg("Error opening scan file","tab2ras-open",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }

    /* Initalize processing message
       -------------------------------*/
    c_inittm("TAB2RAS","Scanned","line"," "," ",&zero,&zero,msgtim,&msgflg);
    curent = 1;
    c_startm();

    /* Initialize scan processing
       -----------------------------*/
    status = gtreci (selfp, &linid, &ddum, &ptl, &header, &ptr);
    if (status == E_EOF)
    {
        sprintf(msg,"%s %s %d",
                "Unexpected end of file encountered while retrieving",
                "a segment of line",linid);
        c_errmsg(msg,"tab2ras-eof",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }
    if (!header)
    {
        sprintf(msg,"%s %s %d",
                "Unexpected data record encountered while retrieving",
                "a segment of line",linid);
        c_errmsg(msg,"tab2ras-head",&nonfatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }

    /* Read each line of the sorted select file 
       ------------------------------------------*/
    do
    {
        /* Get the first line segment */
        gtsegi (selfp,tempfile,&linid, &x1, &y1, &x2, &y2);
        lastid = linid;
        curent++;

        /* Do for each line segment of this line
           ---------------------------------------*/
        linseg: ;

        /* Calculate line/sample locations for the endpoints based the
           coordinate values and write them to the scan file 
           -------------------------------------------------------------*/
        polyscan (scanfp, ptr, (long)ptl, x1, y1, x2, y2, minval, maxval, 
                pixsiz, nlines, nsamps, listflg, xnw, ynw,offset,rawynw);

        /* Traverse the path of the line segment
           ---------------------------------------*/
        x1 = x2;
        y1 = y2;

        /* Check for additional line segments of this line 
           --------------------------------------------------*/
        status = gtreci(selfp, &linid, &y2, &x2, &header, &tdum);
        if (status != E_EOF)
        {
            if (linid == lastid)
                /* process the next line segment of this line 
                   ---------------------------------------------*/
                goto linseg;
            else
            {
                /* Just read in a header record.  Begin processing 
                   the next line 
                   ------------------------------------------------*/
                ptl = x2;
                ptr = tdum;
            }
        }
    }
    while (status != E_EOF);
    c_stoptm();

    /* Scanning is completed.  Close the scan file
       ---------------------------------------------*/
    fclose (scanfp);

    /* delete the sorted select file
       --------------------------------*/
    fclose (selfp);
    c_delfil (tempfile[SORTSEL]);

    return;
}
