#ifndef PROTOS_H
#define PROTOS_H

#include "ltable.h"

long find_path
(
   char directory[]             /* array of directories to search for space */
);


void genrst
(
    char tempfile[][CMLEN],	/* I: temp file names */
    long nlines,		/* I: number of lines in output image */
    long nsamps,		/* I: number of samples in output image */
    double *maxval,		/* I: maximum pixel value */
    double *minval,		/* I: minimum pixel value */
    long msgtim,		/* I: time for processing message */
    long *backgrnd,		/* I: back ground value */
    double *nw,                 /* I: north west corner */
    double *ne,                 /* I: north east corner */
    double *sw,                 /* I: south west corner */
    double *se,                 /* I: south east corner */
    float *pixsiz               /* I: pixel size        */
);

void genrst_pol
(
    char tempfile[][CMLEN],	/* I: temp file names */
    long nlines,		/* I: number of lines in output image */
    long nsamps,		/* I: number of samples in output image */
    double *maxval,		/* I: maximum pixel value */
    double *minval,		/* I: minimum pixel value */
    long msgtim,		/* I: time for processing message */
    long *backgrnd,		/* I: back ground value */
    long mosaic,		/* I: mosaic flag */
    double *nw,                 /* I: north west corner */
    double *ne,                 /* I: north east corner */
    double *sw,                 /* I: south west corner */
    double *se,                 /* I: south east corner */
    float *pixsiz 		/* I: pixel size */
);

void get_coor
(
    long *tab_ind,		/* I: tab index */
    struct VECTOR *ptr[],       /* I:  */
    double *y,			/* I/O: y location */
    double *x,			/* I/O: x location */
    long *id,			/* I: line id */
    long *seq			/* I: line sequence number */
);

void get_corners
(
    struct TAB_DEF *tab,	/* I: table descriptor */
    long *tab_ind,		/* I: tab index */
    struct VECTOR *ptr[],	/* I:  */
    double *corners,		/* I/O: corner coordinates */
    long *nl,			/* I/O: no. of lines in the output image */
    long *ns,			/* I/O: no. of samples in the output image */
    float *pixsiz,		/* I: pixel size */
    double *nw,                 /* O: northwest corner */
    double *ne,                 /* O: northeast corner */
    double *sw,                 /* O: southwest corner */
    double *se,                 /* O: southeast corner */
    double *offset,             /* O: offset */
    double *rawynw              /* O: northwest corner */
);

void get_data
(
    long **array,		/* O: array where values are to be placed */
    struct VECTOR *ptr,         /* I: */
    long *index,		/* I: index into the array */
    short *buf_ind		/* I: number of times array has been expanded */
);

void get_proj
(
    struct TAB_DEF *tab,	/* I: table descriptor */
    long *tab_ind,		/* I: tab index */
    struct VECTOR *ptr[],       /* I: */
    char hosname[]              /* I: temporary file names */
);

void getpar
(
    char ltin[][CMLEN],		/* O: labeled tabel file names */
    char *hosout,		/* O: output host image name */
    char *taeout,		/* O: output tae image name */
    char atrname[][CMLEN],	/* O: attribute names */
    long *pixval,		/* O: pixel value */
    long *nl,			/* O: no. of lines in output images */
    long *ns,			/* O: no. of samples in output images */
    long *backgrnd,		/* O: background value */
    float *pixsiz,		/* O: pixel size  */
    double *cornval,		/* O: cornes values */
    long *msgtim,		/* O: time for processing message */
    long *listflg,		/* O: flag for error messages */
    long *nfile,		/* O: number of labeled table files */
    char *subcmd,		/* O: subcommand */
    long *mosaic,		/* O: mosaic flag */
    char *datpath               /* O: data path  */
);

int gtreci 
(
    FILE *selfp,		/* I: file descriptor for sorted select file */
    long *lineid,		/* O: line id */
    double *value1,		/* I: first value */
    double *value2,		/* I: second value */
    unsigned char *header,	/* O: flag if header record */
    long *prior 		/* O: line priority */
);

void linesel
(
    struct TAB_DEF *tab,        /* I: table descriptor */
    long *tab_ind,              /* I: tab index */
    struct VECTOR *ptr[],       /* I:  */
    short *id_flag,             /* I: flag if PTID is used instead of INDEX */
    long *pixval,               /* I/O: pixel value */
    char selfile[],             /* I/O: temporary file name */
    long *msgtim,               /* I: time for processing message */
    long *nfile,                /* I: number of LT files */
    long *maxweight             /* O: maximum weight            */
);


void ltopen
(
    struct TAB_DEF *tab,	/* I: table descriptor */
    char ltin[][CMLEN],		/* I: labeled table names */
    char atrname[][CMLEN],	/* I: attribute names */
    struct VECTOR *ptr[],       /* I:   */
    long *tab_ind,		/* O: tab index */
    short *id_flag,		/* O: flag if PTID is used instead of INDEX */
    long *nfile			/* I: no. of lt files */
);

void polyinit
(
    char tempfile[][CMLEN],	/* I/O: temporary file names */
    double *minval,		/* I: minimum pixel value */
    double *maxval,		/* I: miximum pixel value */
    long *msgtim,		/* I: time for message processing */
    double xnw,			/* I: x corner coordinate */
    double ynw,			/* I: y corner coordinate */
    long *listflg,		/* I: flag for printing error messages */
    long nlines,		/* I: number of lines in output image */
    long nsamps,		/* I: number of samples in output image */
    float *pixsiz,		/* I: pixel size */
    double rawynw,		/* I: x corner coordinate */
    double offset		/* I: offset */
);

void polyscan
(
    FILE *scanfp,                   /* file descriptor for scan file */
    long ptr,                       /* pixel value for the right region */
    long ptl,                       /* pixel value for the left region */
    double x1,                      /* first x coordinate */
    double y1,                      /* first y coordinate */
    double x2,                      /* second x coordinate */
    double y2,                      /* second y coordinate */
    double *minval,                 /* minimum pixel value */
    double *maxval,                 /* maximum pixel value */
    float *pixsiz,                  /* pixel size */
    long nlines,                    /* no. of lines in output image */
    long nsamps,                    /* no. of samples in output image */
    long *listflg,                  /* flag for printing error messages */
    double xnw,                     /* X coord. of north west corner of image */
    double ynw,                     /* Y coord. of north west corner of image */
    double offset1,                 /* offset */
    double rawynw                   /* Y coord. of north west corner of image */
);


void polysel
(
    struct TAB_DEF *tab,        /* I: table descriptor */
    long *tab_ind,              /* I: tab index */
    struct VECTOR *ptr[],       /* I: */
    short *id_flag,             /* I: flag if PTID is used instead of INDEX */
    long *pixval,               /* I: pixel value */
    char selfile[],             /* I/O: temporary file name */
    long *msgtim,               /* I: time for processing message */
    long *nfile                 /* I: number of LT files */
);

void ptinit
(
    struct TAB_DEF *tab,	/* I: table descriptor */
    long *tab_ind,	        /* I: tab index */
    struct VECTOR *ptr[],       /* I: */
    short *id_flag,		/* I: flag if PTID is used instead of INDEX */
    long *pixval,		/* I/O: pixel value */
    char scanfile[],		/* I/O: temporary file name */
    long *msgtim,		/* I: time for processing message */
    double *minval,		/* I: minimum pixel value */
    double *maxval,		/* I: maximum pixel value */
    float *pixsiz,		/* I: pixel size */
    long nlines,		/* I: number of lines in output image */
    long nsamps,		/* I: number of samples in output image */
    long *listflg,		/* I: flag for printing error messages */
    double xnw,			/* I: x corner coordinate */
    double ynw,			/* I: y corner coordinate */
    long *nfile			/* I: number of LT files */
);

void ptscani
(
    FILE *scanfp,                   /* file descriptor for scan file */
    long *pixval,                   /* pixel value for the point */
    double y,                       /* y coordinate */
    double x,                       /* x coordinate */
    double *minval,                 /* minimum pixel value */
    double *maxval,                 /* maximum pixel value */
    float *pixsiz,                  /* pixel size */
    long nlines,                    /* no. of lines in output image */
    long nsamps,                    /* no. of samples in output image */
    long *listflg,                  /* flag for printing error messages */
    double xnw,                     /* X coord. of north west corner of image */
    double ynw,                     /* Y coord. of north west corner of image */
    long *prior                     /* priority of the point */
);


void read_lt
(
    struct TAB_DEF *tab,        /* I: table descriptor */
    long *tab_ind,              /* I: tab index */
    struct VECTOR *ptr[],       /* I: */
    long **index,               /* O: index array */
    long **color,               /* O: color array */
    long **lnwgt,               /* O: line weight array */
    long **prior,               /* O: priority array */
    short *id_flag,             /* I: flat if PTID is used instead of INDEX */
    long *nfile                 /* I: number of LT files */
);

void scani
(
    FILE *scanfp,                   /* file descriptor for scan file */
    long pixval,                    /* pixel value for the line */
    double x1,                      /* first x coordinate */
    double y1,                      /* first y coordinate */
    double x2,                      /* second x coordinate */
    double y2,                      /* second y coordinate */
    long linwt,                     /* weight of the line */
    double *minval,                 /* minimum pixel value */
    double *maxval,                 /* maximum pixel value */
    float *pixsiz,                  /* pixel size */
    long nlines,                    /* no. of lines in output image */
    long nsamps,                    /* no. of samples in output image */
    long *listflg,                  /* flag for printing error messages */
    double xnw,                     /* X coord. of north west corner of image */
    double ynw,                     /* Y coord. of north west corner of image */
    long *prior                     /* priority of the line */
);


void scnint
(
    char tempfile[][CMLEN],     /* I/O:  temporary file names */
    double *minval,             /* I: minimum pixel value */
    double *maxval,             /* I: miximum pixel value */
    long *msgtim,               /* I: time for message processing */
    double xnw,                 /* I: x corner coordinate */
    double ynw,                 /* I: y corner coordinate */
    long *listflg,              /* I: flag for printing error messages */
    long nlines,                /* I: number of lines in output image */
    long nsamps,                /* I: number of samples in output image */
    float *pixsiz               /* I: pixel size */
);

void sortscan
(
    char tempfile[][CMLEN],		 /* I/O: input select file */
    char *datapath			 /* I: enironment data path */
);

void sortsel
(
    char tempfile[][CMLEN],		/* I/O: input select file */
    char *datapath                      /* I: enironment data path */
);
 
void gtsegi
(
    FILE *fp,                   /* I: file descriptor for sorted select file  */
    char tempfile[][CMLEN],     /* I: temporary file names */
    long *linid,                /* I: line id */
    double *x1,                 /* I: first x coordinate */
    double *y1,                 /* I: first y coordinate */
    double *x2,                 /* I: second x coordinate */
    double *y2                  /* I: second x coordinate */
);

#endif
