/*****************************************************************************
*
*  NAME: PTINIT
*
*  FUNCTION: Initialize processing for point data
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"
#include "ltable.h"
#include "util.h"
#include "protos.h"

void ptinit
(
    struct TAB_DEF *tab,        /* I: table descriptor */
    long *tab_ind,              /* I: tab index */
    struct VECTOR *ptr[],       /* I: */
    short *id_flag,             /* I: flag if PTID is used instead of INDEX */
    long *pixval,               /* I/O: pixel value */
    char scanfile[],            /* I/O: temporary file name */
    long *msgtim,               /* I: time for processing message */
    double *minval,             /* I: minimum pixel value */
    double *maxval,             /* I: maximum pixel value */
    float *pixsiz,              /* I: pixel size */
    long nlines,                /* I: number of lines in output image */
    long nsamps,                /* I: number of samples in output image */
    long *listflg,              /* I: flag for printing error messages */
    double xnw,                 /* I: x corner coordinate */
    double ynw,                 /* I: y corner coordinate */
    long *nfile                 /* I: number of LT files */
)

{
    extern struct PARBLK parblk;
    extern long curent;

    FILE *fp;                       /* File descriptor              */

    double x;                       /* coordinate value */
    double y;                       /* coordinate value */

    long *color_arr;                /* color array */
    long dum;                       /* dummy value */
    long fatal = -1;                /* for fatal error messages */
    long id;                        /* point id */
    long *index_arr;                /* index array */
    long index;                     /* index value */
    long j = 0;                     /* current index value */
    long msgflg = GEN1;             /* flag for processing message */
    long nonfatal = 1;              /* for nonfatal error messages */
    long one = 1;                   /* constant of one */
    long prev_id;                   /* previous point id processed */
    long *prior_arr;                /* priority array */
    long prior = 1;                 /* current priority value */    
    long status;                    /* function status */
    long *weight_arr;               /* weight array */

    /*  Open temporary scan file 
        ---------------------------*/
    strcpy(scanfile,"tab2ras_scan");
    c_creat_name(scanfile,"txt");
    if ((fp = fopen(scanfile,"w")) == NULL)
    {
        c_errmsg("Error opening scan file","tab2ras-open",&nonfatal);
        c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
    }

    /* Read index,pixval,and color values into memory
       ---------------------------------------------------*/
    read_lt(tab,tab_ind,ptr,&index_arr,&color_arr,&weight_arr,&prior_arr,
            id_flag, nfile);

    /* Initialize processing message
       --------------------------------*/
    curent = 1;
    c_inittm("TAB2RAS","Scanned","point"," "," ",&j,&j,msgtim,&msgflg);
    c_startm();

    /*  Read the first record from the labeled table that contains the 
        Y/X coordinates 
        ------------------------------------------------------------------*/
    status = get_record(&tab[tab_ind[IDCOOR]]);
    if (status == E_FAIL)
        c_clean(&parblk,"tab2ras",(char (*)[256])scanfile,&one);
    get_coor(tab_ind,ptr,&y,&x,&id,&dum);
    prev_id = id;

    j = 1;
    /*  Initialize the attribute values
        -----------------------------------*/
    index = index_arr[j];
    if (tab_ind[PRIOR] != -1)
        if (id_flag[PRIOR])
            prior = prior_arr[id];
        else
            prior = prior_arr[index];
    if (tab_ind[COLOR] != -1)
        if (id_flag[COLOR])
            *pixval = color_arr[id];
        else
            *pixval = color_arr[index];

    /*  Read the Y/X coordinates until the EOF is found.  If a new point 
        id is read, set the attribute values and write the header record 
        --------------------------------------------------------------------*/
    for (; status != E_EOF;)
    {
        if (id != prev_id)
        {
            prev_id = id;
            j++;
            index = index_arr[j];
            if (tab_ind[PRIOR] != -1)
                if (id_flag[PRIOR])
                    prior = prior_arr[id];
                else
                    prior = prior_arr[index];
            if (tab_ind[COLOR] != -1)
                if (id_flag[COLOR])
                    *pixval = color_arr[id];
                else
                    *pixval = color_arr[index];
        }

        /* Change y/x to line/sample and write to the scan file 
           ------------------------------------------------------*/
        ptscani(fp, pixval, y, x, minval, maxval, pixsiz, nlines, nsamps, 
                listflg, xnw, ynw, &prior);

        /* Get the next record
           ----------------------*/
        status = get_record(&tab[tab_ind[IDCOOR]]);
        if (status == E_FAIL)
            c_clean(&parblk,"tab2ras",(char (*)[256])scanfile,&one);
        get_coor(tab_ind,ptr,&y,&x,&id,&dum);
        curent++;
    }

    c_stoptm();

    /* Close the scan file 
       ----------------------*/
    fclose (fp);

    /* Free the buffers
       -------------------*/
    free(index_arr);
    free(color_arr);
    free(prior_arr);
    free(weight_arr);

    return;
}
