/*****************************************************************************
*
*  NAME: PTSCANI
*
*  FUNCTION: Calculate the line/sample location for the point and write it
*            to the scan file
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"
#include "protos.h"

void ptscani 
(
    FILE *scanfp,               /* I/O: file descriptor for scan file */

    long *pixval,               /* I: pixel value for the point */ 
    double y,                   /* I: y coordinate */
    double x,                   /* I: x coordinate */
    double *minval,             /* I: minimum pixel value */
    double *maxval,             /* I: maximum pixel value */
    float *pixsiz,              /* I: pixel size */
    long nlines,                /* I: no. of lines in output image */
    long nsamps,                /* I: no. of samples in output image */
    long *listflg,              /* I: flag for printing error messages */
    double xnw,                 /* I: X coord. of north west corner of image */
    double ynw,                 /* I: Y coord. of north west corner of image */
    long *prior                 /* I: priority of the point */
)

{

    double  dbl1 = 0.1e1;           /* double 1.0 */

    long nonfatal = 1;              /* for nonfatal error messages */
    long one = 1;                   /* constant of one */   

    long rastx                      /* x location */
        ,rasty;                     /* y location*/

    char msg[ERRLEN];               /* buffer for error messages */

    /* calculate the y/x coordinate  
       -------------------------------*/
    rasty = (long) (((ynw - y) / pixsiz[0] ) + dbl1);
    rastx = (long) (((x - xnw) / pixsiz[1] ) + dbl1);

    /* Validate the line to see if it lines within the image
       --------------------------------------------------------*/
    if (!*listflg)
    {
        if (rasty > nlines )
            return;
        else if (rasty < one )
            return;
    }
    else
    {
        if (rasty > nlines )
        {
            sprintf(msg,"%s %s %d","Data lies below the output image.",
                    "Data would occur at line",rasty);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            return;
        }
        else if (rasty < one )
        {
            sprintf(msg,"%s %s %lf","Data lies above the output image.",
              "The y coordinate of the northwest corner should be adjusted to ",
              y);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            return;
        }
    }

    /* Validate the sample to see if it lines within the image
       ---------------------------------------------------------*/
    if (!*listflg)
    {
        if (rastx < one )
            return;
        else if (rastx > nsamps )
            return;
    }
    else
    {
        if (rastx > nsamps )
        {
            sprintf(msg,"%s %s %d",
                    "Data lies to the right of the output image.",
                    "Data would occur at sample",rastx);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            return;
        }
        else if (rastx < one )
        {
            sprintf(msg,"%s %s %f","Data lies to the left of the output image.",
              "The x coordinate of the northwest corner should be adjusted to ",
              x);
            c_errmsg(msg,"tab2ras-outside",&nonfatal);
            return;
        }
    }

    /* Log this point to the scan file
       ----------------------------------*/
    fprintf(scanfp,"%10d %10d %10d %10d %11d\n",
            rasty, *prior, rastx , rastx , *pixval);

    /* Determine if pixval is a minimum or maximum value
       -----------------------------------------------------*/
    *maxval = max (*maxval, *pixval );
    *minval = min (*minval, *pixval );

    return;
}
