/*****************************************************************************
*
*  NAME: READ_LT
*
*  FUNCTION: Read the attributes from the labeled table 
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include "las.h"
#include "tab2ras.h"
#include "ltable.h"
#include "protos.h"

void read_lt
(
    struct TAB_DEF *tab,        /* I: table descriptor */
    long *tab_ind,              /* I: tab index */
    struct VECTOR *ptr[],       /* I: */
    long **index,               /* O: index array */
    long **color,               /* O: color array */
    long **lnwgt,               /* O: line weight array */
    long **prior,               /* O: priority array */
    short *id_flag,             /* I: flat if PTID is used instead of INDEX */
    long *nfile                 /* I: number of LT files */
)

{

    struct TAB_DEF *tmp_tab[MAX_SUB];

    long *temp;             /*  temp pointer                                */
    long fatal = -1;        /*  for fatal error messages                    */
    long i;                 /*  counter -- current subfiles                 */
    long id;                /*  id of coordinate pair                       */
    long *id_ptr;           /*  id of coordinate pair                       */
    long j;                 /*  counter -- current record of file           */
    long nonfatal = 1;      /*  nonfatal error code                         */
    long ntabs = 0;         /*  no. of tab                                  */
    long status;            /*  status of called functions                  */
    long zero = 0;          /*  value of zero                               */

    short one = 1;          /*  value of one                                */
    short index_buf_ind = 1;        /* array size increment                 */
    short color_buf_ind = 1;        /* array size increment                 */
    short lnwgt_buf_ind = 1;        /* array size increment                 */
    short prior_buf_ind = 1;        /* array size increment                 */

    /* Initialize and allocate space
       --------------------------------*/
    id_ptr = &id;
    if ((tab[0].nsubs <= 0) && (*nfile > 0))
        tab[0].nsubs = *nfile;
    temp = (long *)malloc(tab[0].nsubs * sizeof(long));
    if (temp == NULL)
    {
        c_errmsg("Error allocating dynamic memory","tab2ras-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
    }

    if (((*index = (long *)malloc(BUF_INC * sizeof(long))) == NULL) ||
            ((*color = (long *)malloc(BUF_INC * sizeof(long))) == NULL) ||
            ((*lnwgt = (long *)malloc(BUF_INC * sizeof(long))) == NULL) ||
            ((*prior = (long *)malloc(BUF_INC * sizeof(long))) == NULL)) 
    {
        c_errmsg("Error allocating dynamic memory","tab2ras-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
    }

    /* Check which labeled tables to read and put in one continous array 
       ---------------------------------------------------------------------*/
    for (i = 0; i <= tab[0].nsubs; i++)
    {
        if ((i == tab_ind[INDEX]) ||
                (i == tab_ind[PRIOR]) ||
                (i == tab_ind[LNWGT]) ||
                (i == tab_ind[COLOR]))
        {
            tmp_tab[ntabs] = &tab[i];
            temp[ntabs] = i;
            ntabs++;
        }
    }

    /* Read the specified labeled tables
       ------------------------------------*/
    for (i = 0; i < ntabs; i++)
    {
        j = 1;
        do
        {
            status = get_record(tmp_tab[i]);
            if (status == E_FAIL)
            {
                c_errmsg("Error reading Labeled Table file","tab2ras-read",
                          &nonfatal);
                c_errmsg("Fatal error encountered","tab2ras-fatal",&fatal);
            }
            if (temp[i] == tab_ind[INDEX])
                get_data(index,ptr[INDEX],&j,&index_buf_ind);
            if (temp[i] == tab_ind[PRIOR])
                if (id_flag[PRIOR])
                {
                    get_data(&id_ptr,ptr[PTID],&zero,&one);
                    get_data(prior,ptr[PRIOR],&id,&prior_buf_ind);
                }
                else
                    get_data(prior,ptr[PRIOR],&j,&prior_buf_ind);

            if (temp[i] == tab_ind[LNWGT])
                if (id_flag[LNWGT])
                {
                    get_data(&id_ptr,ptr[PTID],&zero,&one);
                    get_data(lnwgt,ptr[LNWGT],&id,&lnwgt_buf_ind);
                }
                else
                    get_data(lnwgt,ptr[LNWGT],&j,&lnwgt_buf_ind);

            if (temp[i] == tab_ind[COLOR])
                if (id_flag[COLOR])
                {
                    get_data(&id_ptr,ptr[PTID],&zero,&one);
                    get_data(color,ptr[COLOR],&id,&color_buf_ind);
                }
                else
                    get_data(color,ptr[COLOR],&j,&color_buf_ind);

            j++;
        }
        while (status == E_SUCC);
    }

    /* Free the buffer
       ------------------*/
    free(temp);

    return;
}
