/*****************************************************************************
*
*  NAME: SORTSEL
*
*  FUNCTION: Sort the select file
*
*  PROGRAM HISTORY:
*  Version      Date    Programmer      Code    Request
*  -------      ----    ----------      ----    -------
*     1       02/01/85  K. LETHCOE      CSB
*     2       04/01/91  Ailts/Etrheim   CSB     LAS 5.0 conversion
*     3       12/30/92  D. Etrheim      CSB     Remove DATAPATH from system sort
*
*  COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               Must be run under TAE
*
*  PROJECT: LAS
*
*  ALGORITHM DESCRIPTION:
*       See inline comments 
*
******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "tab2ras.h"
#include "desc.h"
#include "util.h"
#include "protos.h"

void sortsel
(
    char tempfile[][CMLEN],             /* I/O: input select file */
    char *datapath                      /* I: enironment data path */
)

{
    extern struct PARBLK parblk;

#ifdef vms
    struct DESC infile;
    struct DESC outfile;
    long ncol = 2;
#endif

    long non_fatal = 1;             /* for nonfatal error messages  */
    long numfile = NUMTEMPF;        /* output sorted select file */
    long result;                    /* result of system sorts       */

    char command[CMLEN];            /* command string for unix system sort */ 
    char directory[CMLEN];          /* temporary directory name             */


    /* Create the output file name  
       -----------------------------*/
    strcpy(tempfile[SORTSEL],"tab2ras_selsrt");
    c_creat_name(tempfile[SORTSEL],"txt");
    c_errmsg("Sorting the selection file"," ",&non_fatal);

    /* Sort the select file.  This file will be sorted based on 
       the line id and the sequence number 
       -----------------------------------------------------------*/
#ifdef vms
    infile.addr = &tempfile[SELECT];
    outfile.addr = &tempfile[SORTSEL];
    infile.length = strlen(tempfile[SELECT]);
    outfile.length = strlen(tempfile[SORTSEL]);
    prmsrt (&infile,&outfile,&ncol,&result);
    if (result != 1)
    {
        c_errmsg("System file sort failed","tab2ras-system",&non_fatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }
#endif

#ifdef unix
    /* Create the command string
       ----------------------------*/
    if ((datapath = (char *)getenv(DATAPATH)))
    {
        find_path(directory);
        sprintf(command,"sort -n %s -o %s -T %s",tempfile[SELECT],
                tempfile[SORTSEL] ,directory);
    }
    else 
        sprintf(command,"sort -n %s -o %s",tempfile[SELECT],tempfile[SORTSEL]);
    result = system(command);
    if (result != E_SUCC)
    {
        c_errmsg("System file sort failed","tab2ras-system",&non_fatal);
        c_clean(&parblk,"tab2ras",tempfile,&numfile);
    }
#endif

    /* Delete the unsorted select file
       ----------------------------------*/
    c_delfil(tempfile[SELECT]);

    return;
}
