#include "tab2stat.h"
#include <string.h>
#include "protos.h"

int get_cls_site
(
    long level,			/* STAT level being processed                 */
    struct VECTOR *class,	/* Labeled Table record field for CLASS       */
    struct VECTOR *site,	/* Labeled Table record field for SITE        */
    char *classval[TAE_VAL],	/* User specifed CLASS values                 */
    char *siteval[TAE_VAL],	/* User specifed SITE values                  */
    char class_name[],		/* Class value returned to calling routine    */
    char site_name[],		/* Site value returned to calling routine     */
    long *class_counter	/* Number of class values that have been NULL */
)

/******************************************************************************
*
* NAME: Get_Cls_Site
*
* FUNCTION: Determines whether or not the current Labeled Table
* 	    data (e.g. polygon data) should be processed.  If it is to 
*	    be processed, return a value of TRUE; otherwise, it returns 
*	    a value of FALSE.
*
* COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
*           Runs under TAE
*
* PROJECT: LAS
*
* ALGORITHM DESCRIPTION:
*     If level equals image level
*        return true
*     else
*        check class dtype for data type
*	 If all classes are to be processed
*	    set a flag to true
*        else search for class name
*        If class equals null then
*           assign a value
* 	 If class dtype equals numeric then
*	    delimit with quotes
*        check site dtype for data type
*        If all sites are to be processed then
*           set a flag
*	 else search for site name
*	 If site equals null then
*           assign a value
*        If site dtype is numeric then
*           delimit with quotes
*        check level and return.
*
******************************************************************************/

{
long class_flag;		/* Boolean indicating if class value matched  */
long count;			/* Loop counter                               */
long site_flag;			/* Boolean indicating if site value matched   */
char *cptr;			/* Pointer to character string                */
char str_val[MAX_FIELD_LEN];	/* Buffer to hold the string value            */

if (level == IMAGE_LEV)
   return(TRUE);

switch(class->dtype)
   {
   case C:	strcpy(str_val,class->ptr.c);		break;
   case I1:	sprintf(str_val,"%d",*class->ptr.i1);	break;
   case I2:	sprintf(str_val,"%d",*class->ptr.i2);	break;
   case I4:	sprintf(str_val,"%d",*class->ptr.i4);	break;
   case R4:	sprintf(str_val,"%d",*class->ptr.r4);	break;
   case R8:	sprintf(str_val,"%d",*class->ptr.r8);	break;
   }
if (classval[0] == NULL)		/* all classes are to be processed    */
   class_flag = TRUE;
else					/* see if class matches               */
   for (class_flag=FALSE,count=0; classval[count]!=NULL; count++)
      if (strcmp(str_val,classval[count]) == 0)
         class_flag = TRUE;
if (*str_val == '\0')			/* if class value null, assign value  */
   {
   (*class_counter)++;
   sprintf(str_val,"CLASS_%d",*class_counter);
   }
if (class->dtype != C)			/* if numeric, delimit with quotes   */
   sprintf(class_name,"\"%s\"",str_val);
else
   strcpy(class_name,str_val);          /* if name has spaces, replace with  */
					/* underscores.			     */
for (cptr=strchr(class_name,' '); cptr!=NULL; cptr=strchr(class_name,' '))
   *cptr = '_';

switch(site->dtype)
   {
   case C:	strcpy(str_val,site->ptr.c);		break;
   case I1:	sprintf(str_val,"%d",*site->ptr.i1);	break;
   case I2:	sprintf(str_val,"%d",*site->ptr.i2);	break;
   case I4:	sprintf(str_val,"%d",*site->ptr.i4);	break;
   case R4:	sprintf(str_val,"%d",*site->ptr.r4);	break;
   case R8:	sprintf(str_val,"%d",*site->ptr.r8);	break;
   }
if (siteval[0] == NULL)			/* all sites are to be processed      */
   site_flag = TRUE;
else					/* see if site matches                */
   for (site_flag=FALSE,count=0; siteval[count]!=NULL; count++)
      if (strcmp(str_val,siteval[count]) == 0)
         site_flag = TRUE;
if (*str_val == '\0')			/* if site value null, assign value   */
   strcpy(str_val,"SITE");
if (site->dtype != C)			/* if numeric, delimit with quotes   */
   sprintf(site_name,"\"%s\"",str_val);
else
   strcpy(site_name,str_val);		/* if name has spaces, replace with  */
					/* underscores			     */
for (cptr=strchr(site_name,' '); cptr!=NULL; cptr=strchr(site_name,' '))
   *cptr = '_';

switch (level)
   {
   case CLASS_LEV: return(class_flag);
   case SITE_LEV:  return((class_flag) && (site_flag));
   }
return (FALSE);
}
