#include "tab2stat.h"
#include "util.h"
#include "protos.h"

void get_lt_def
(
    struct PARBLK *vblock,	  /* parameter block			   */
    FILE **fd,			  /* file descriptor			   */
    struct TAB_DEF *tab,	  /* master LT file structure */
    char *attrib,	  /* attribute LT file type */
    char *data,	  /* data LT file type */
    struct TAB_DEF **attrib_def,  /* attribute LT structure to be returned */
    struct TAB_DEF **data_def,	  /* data LT structure to be returned */
    char hstat[][CMLEN],		  /* host stat file name		   */
    long *nfile		  /* whether or not a new stat file was made */
)
/*
**********************************************************************

NAME: Get_Lt_Def                            

PURPOSE: This function returns the pointers to the LT structure 
	 definitions for the attribute and data LT files.  

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      If master LT file is the attribute then
         search for data file
	    if LT file types equal then 
	       get the pointer to the structure
         If data file not found then
            return fatal error
      Else if master LT file is the data file then
         search for attribute file
            If file type equals attribute type then
               get the pointer to the structure
         If attribute file not found then
            return a fatal error
      Else 
         bad Labeled table file type.
	
************************************************************************
*/
{
static struct TAB_DEF sub;		/* LT subfile structure */
struct VECTOR *ptr;			/* pointer to search file */
long index;				/* loop counter */
char msg[ERRLEN];			/* error message buffer */

long nonfatal = 1;
long zero = 0;

/* 
 If master LT file is the attribute file, 
 search for the data file.  
*/
ptr = get_field_ptr(tab,attrib);
if (ptr != NULL)
   {					
   *attrib_def = tab;			
   *data_def = NULL;			
   for (index=0; index<tab->nsubs; index++)
      {
      open_tab(&sub,tab->subfile[index],zero);
      get_vector(&sub);
      ptr = get_field_ptr(&sub,data);
      if (ptr != NULL)
         {
         *data_def = &sub;
         break;
         }
      close_tab(&sub);
      }
/** 
 If the data file does not exist, 
 a fatal error is generated.      
**/
   if (*data_def == NULL)
      {
      sprintf(msg,"LT file type \"%s\" not found",data);
      c_errmsg(msg,"tab2stat-file",&nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
   }

/**
 If master LT file is the data file, 
 search for the attribute file.   
**/
else if ((ptr = get_field_ptr(tab,data)) != NULL)
   {					
   *data_def = tab;			
   *attrib_def = NULL;			
   for (index=0;index<tab->nsubs;index++)
      {	
      open_tab(&sub,tab->subfile[index],zero);
      get_vector(&sub);
      ptr = get_field_ptr(&sub,attrib);
      if (ptr != NULL)
         {
         *attrib_def = &sub;
         break;
         }
      close_tab(&sub);
      }
/**  
 If the attribute file does not exist, 
 a fatal error is generated
**/
   if (*attrib_def == NULL)
      {
      sprintf(msg,"LT file type \"%s\" not found",attrib);
      c_errmsg(msg,"tab2stat-file",&nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
   }
/**
 Else bad Labeled table file type, a fatal error is generated.
 The error message about CLASSATR is here because the class attribute
 is used to find which file is the data file and which file is the 
 attribute file.  The attribute comes here before it gets checked
 in process_poly. 
**/
else
   {
   c_errmsg("LT attribute name for 'CLASSATR' not found",
	    "tab2stat-attribute",&nonfatal);
   c_stclse(fd);
   c_clean(vblock,"tab2stat",hstat,nfile);
   }
return;
}
