#include "tab2stat.h"
#include "util.h"
#include "protos.h"

void get_xy_pts
(
    struct PARBLK *vblock,	/* TAE parameter block                        */
    FILE **fd,			/* File descriptor			      */
    struct TAB_DEF *regxy,	/* Labeled Table description structure        */
    struct VECTOR *ptr_reg,	/* Label column for REGNUM                    */
    struct VECTOR *ptr_x,	/* Label column for X                         */
    struct VECTOR *ptr_y,	/* Label column for Y                         */
    long *xy,			/* Array containing the x points              */
    long *npnts,		/* Number of points in the polygon            */
    long *cur_poly,		/* Current polygon being processed            */
    char hstat[][CMLEN],		/* host stat file name			      */
    long *nfile		/* Whether a new STAT file was created or not */
)

/***********************************************************************

NAME: Get_XY_Pts                            

PURPOSE: Reads the x y points of the current polygon from the Labeled
  	 Table, and stores them in the arrays of "x" and "y".  

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      Initialize variables
      Loop until end of polygon or too many points in polygon
   	If current polygon not equall then
	   assign new polygon and exit
        assign x & y points
 	If still current polygon then
	   If too many points then
	      send a fatal error
	   If another record then
	      continue the loop

************************************************************************
*/
{
long flag;			/* boolean variable          */
long *xyptr;			/* array containing X and Y points */
long nonfatal = 1;

/*
 Initialize variables
*/
*npnts = 0;
xyptr = xy;

/* 
 Reads the X and Y values from the current polygon
 and stores them until the end of the polygon is found
*/ 
for (flag=TRUE; flag; xyptr += 2)
   {
   if (*cur_poly != *ptr_reg->ptr.i4)
      {
      *cur_poly = *ptr_reg->ptr.i4;
      flag = FALSE;
      break;
      }
/*
 The actual storing of the values
 -------------------------------- */
switch(ptr_x->dtype)
   {
   case I4:
       *xyptr = *ptr_x->ptr.i4;
       *(xyptr + 1) = *ptr_y->ptr.i4;
       break;
   case R4:
       *xyptr = (long) *ptr_x->ptr.r4;
       *(xyptr + 1) = (long) *ptr_y->ptr.r4;
       break;
   default:
         c_errmsg("Invalid data type for COORSATR","tab2stat-dtype", &nonfatal);
         c_stclse(fd);
         c_clean(vblock,"tab2stat",hstat,nfile);
   }
 
   if (flag)
      {
      (*npnts)++;
      if (*npnts >= MAX_POINTS)
	 {
         c_errmsg("Too many points in LT polygon","tab2stat-bigpoly",
	          &nonfatal);
         c_stclse(fd); 
	 c_clean(vblock,"tab2stat",hstat,nfile);
	 }
      if (get_record(regxy))
         flag = FALSE;
      }
   }

return;
}
