#include "tab2stat.h"
#include "util.h"
#include "protos.h"

void process_covar
(
    struct PARBLK *vblock,	/* TAE parameter block                        */
    FILE **fd,			/* file descriptor			      */
    struct TAB_DEF *tab,	/* Labeled Table description structure        */
    long level[],		/* STAT levels to be processed                */
    char *lev_atr,		/* Label name to be used as the STAT LEVEL    */
    long f_flag,		/* Boolean flag, is FALSE only if IMAGE is    */
    char *class,		/* Label name to be used as the STAT CLASS    */
    char *site,			/* Label name to be used as the STAT SITE     */
    char *classval[TAE_VAL],	/* Label descriptions to be valid class values*/
    char *siteval[TAE_VAL],	/* Label descriptions to be valid site values */
    long nclass,		/* Total number of classes in the STAT file   */
    long *msgtim,		/* Number of seconds between processing msg   */
    char hstat[][CMLEN],		/* host stat file name			      */
    long *nfile		/* Whether a new STAT file was created or not */
)

/*
 **********************************************************************

NAME: Process_Covar                            

PURPOSE: To copy a covariance Labeled Table to a STAT file.  

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      If Level not in labeled table then
	 return a fatal error
      If Class not in labeled table then
	 return a fatal error
      If Site not in labeled table then
	 return a fatal error
      Assign number of points and mean vector
      If covar not in labeled table then 
	 return a fatal error
      Display informational progress message
      Loop, processing all covariance matrix
	 If no more records then
	    break
	 Get current level
	 Check level if valid
	 If level is valid then
	    write matrix to stat file

************************************************************************
*/
{
extern long rec;		/* Current polygon being processed            */

static 
struct VECTOR *covar_matrix,	/* Pointer to value of covariance matrix      */
              *mean_vec,	/* Pointer to value of mean vector            */
              *npoints,		/* Pointer to value of # of points in area    */
              *ptr_class,	/* Pointer to value of class                  */
              *ptr_site,	/* Pointer to value of site                   */
              *stat_level;	/* Pointer to value of statistics level       */
static 
char class_name[LIN_SIZ],	/* STAT class value of current matrix         */
     site_name[LIN_SIZ];	/* STAT site value of current matrix          */
long cur_level;			/* Numeric value of statistics level          */
long valid;			/* Boolean indicating if matrix is copied     */
long nonfatal = 1;
long two = 2;
long class_counter = 0;

/*
 Check for level, class, site in labeled table
 if one is not found then quit procedure
*/
if ((stat_level = get_field_ptr(tab,lev_atr)) == NULL)
   {
   c_errmsg("LT attribute name for 'LEVELATR' not found",
	    "tab2stat-attribute", &nonfatal);
   c_stclse(fd);
   c_clean(vblock,"tab2stat",hstat,nfile);
   }
/*
*  If f_flag is TRUE then class and site attribute names 
*  names must be entered in the parameter list.  This a
*  result of picking something other than just IMAGE in
*  STATSTYP.
*/
if (f_flag)
   {
   if ((ptr_class = get_field_ptr(tab,class)) == NULL)
      {
      c_errmsg("LT attribute name for 'CLASSATR' not found",
	       "tab2stat-attribute",&nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
   if ((ptr_site = get_field_ptr(tab,site)) == NULL)
      {
      c_errmsg("LT attribute name for 'SITEATR' not found",
	       "tab2stat-attribute", &nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
   }
/* 
 Assign number of points and mean vector
*/
npoints = get_field_ptr(tab,"NPOINTS");
mean_vec = get_field_ptr(tab,"MEAN");
if ((covar_matrix = get_field_ptr(tab,"COVAR")) == NULL)
   {
   c_errmsg("LT attribute name 'COVAR' not found","tab2stat-attribute",
	    &nonfatal);
   c_stclse(fd);
   c_clean(vblock,"tab2stat",hstat,nfile);
   }
rec = 1;
c_inittm("TAB2STAT","Processed","Covariance matrix","","",&two,&two,msgtim,&two);
c_startm();

/*
 Process all covariance matrix 
*/
for (rec=0; ; rec++)
   {
   if (get_record(tab))
      break;
   cur_level = get_cur_lvl(stat_level);
   valid = (cur_level != -1) && (level[cur_level]) &&
           (get_cls_site(cur_level,ptr_class,ptr_site,classval,siteval,
                               class_name,site_name,&class_counter));
/*
 If a valid covariance matrix then write it to the stat file
*/
   if (valid)
      stat_covar(vblock,fd,cur_level,class_name,site_name,&nclass,covar_matrix,
                 mean_vec,npoints,hstat,nfile);
   }
c_stoptm();
close_tab(tab);				/* Close Labeled Table file   */
return;
}
