#include "tab2stat.h"
#include "util.h"
#include "protos.h"

void process_poly
(
    struct PARBLK *vblock,	/* TAE parameter block                        */
    FILE **fd,			/* file descriptor			      */
    struct TAB_DEF *tab,	/* Labeled Table description structure        */
    char *class,		/* Label name to be used as the STAT CLASS    */
    char *site,			/* Label name to be used as the STAT SITE     */
    char *classval[TAE_VAL],	/* Label descriptions to be valid class values*/
    char *siteval[TAE_VAL],	/* Label descriptions to be valid site values */
    char *regnm,		/*  */
    char *Xcoor,		/*  */
    char *Ycoor,		/*  */
    long nclass,		/* Total number of classes in the STAT file   */
    long *msgtim,		/* Number of seconds between processing msg   */
    char hstat[][CMLEN],		/* host stat file name			      */
    long *nfile		/* Whether a new STAT file was created or not */
)
/*
**********************************************************************

NAME: Process_Poly                            

PURPOSE: This function copies a polygon Labeled Table to Stat file. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      Dynamic allocation of X and Y arrays
      If Class not found then
 	 return a fatal error
      If Site not found then
	 return a fatal error
      If Regnum not found then
	 return a fatal error
      If X not found then
	 return a fatal error
      If Y not found then
	 return a fatal error
      Get DATA file structure
      Loop, processing polygons
	 If end of file then
	   terminate loop
	 Check for valid polygon
	 If invalid polygon then
	   skip over current polygon
	   and return to top of loop
	 else (valid polygon)
	   get the X,Y points that make up the polygon
	   Search the stat file for the site node
	   Write the information to the stat file
	   If an error writting to file then
	     return a fatal error
      Closs files and free dynamic memory
	   
************************************************************************
*/
{
extern long rec;		/* Current polygon being processed            */

struct TAB_DEF *regatt;		/* attribute LT file structure                */
struct TAB_DEF *regxy;		/* data LT file structure                     */
struct VECTOR *ptr_class;    	/* Label column for "CLASS"                */
struct VECTOR *ptr_site;    	/* Label column for "SITE"                 */
struct VECTOR *ptr_reg;		/* Label column for "REGNUM"                  */
struct VECTOR *ptr_x;		/* Label column for "X"                       */
struct VECTOR *ptr_y;		/* Label column for "Y"                       */

union STAT_SHAPE pshape;	/* STAT shape buffer for data type POLYGON    */

long class_index;		/* Class number to write polygon data into    */
long cur_poly;			/* Current polygon being processed            */
long site_index;		/* Site number to write polygon data into     */
long status;			/* Function return status code                */
long valid;			/* Boolean indicating if polygon is copied    */
long class_counter=0;		/* Number of classes without names	      */
long *xy;		        /* Array containing x values of a polygon     */

char dtype[CMLEN];		/* STAT data type descriptor                  */
char class_name[MAX_FIELD_LEN];	/* STAT class value of current polygon        */
char site_name[MAX_FIELD_LEN];	/* STAT site value of current polygon         */
long nonfatal = 1;
long two = 2;

/*
*  Retrieve the LT table structures for the attribute and data files.  Note
*  that parameter "tab" is one of these table structures.
*/
xy = (long *)malloc(MAX_POINTS * 2);

get_lt_def(vblock,fd,tab,class,Ycoor,&regatt,&regxy,hstat,nfile);
/* 
 Define string descriptor           
*/
strcpy(dtype,"POLYGON");		

strcpy(pshape.dtype,"I*4 ");
pshape.buf[0] = ELONG;
pshape.buf[1] = pshape.buf[3] = 2;

/*
 If Class, or Site, or Regnum not found then return a fatal error
*/ 
if ((ptr_class = get_field_ptr(regatt,class)) == NULL)
   {
   c_errmsg("LT attribute name for 'CLASSATR' not found",
	    "tab2stat-attribute",&nonfatal);
   c_stclse(fd);
   c_clean(vblock,"tab2stat",hstat,nfile);
   }
if ((ptr_site = get_field_ptr(regatt,site)) == NULL)
   {
   c_errmsg("LT attribute name for 'SITEATR' not found",
	    "tab2stat-attribute",&nonfatal);
   c_stclse(fd);
   c_clean(vblock,"tab2stat",hstat,nfile);
   }

if ((ptr_reg = get_field_ptr(regxy,regnm)) == NULL)
   {
   c_errmsg("LT attribute name for 'POLYATR' not found",
	    "tab2stat-attribute",&nonfatal);
   c_stclse(fd);
   c_clean(vblock,"tab2stat",hstat,nfile);
   }
/*
 If X or Y not found then return a fatal error
*/
if ((ptr_x = get_field_ptr(regxy,Xcoor)) == NULL)
   {
   c_errmsg("LT attribute name for 'COORSATR 2' not found",
	    "tab2stat-attribute",&nonfatal);
   c_stclse(fd);
   c_clean(vblock,"tab2stat",hstat,nfile);
   }
if ((ptr_y = get_field_ptr(regxy,Ycoor)) == NULL)
   {
   c_errmsg("LT attribute name for 'COORSATR 1' not found",
	    "tab2stat-attribute",&nonfatal);
   c_stclse(fd);
   c_clean(vblock,"tab2stat",hstat,nfile);
   }

rec = 1;
c_inittm("TAB2STAT","Processed","STAT polygon","","",&two,&two,msgtim,&two);
c_startm();
get_record(regxy);
for (rec=0,cur_poly=1; ; rec++)
   {
   if (get_record(regatt))		/* if end of file, terminate loop     */
      break;
   valid = get_cls_site(SITE_LEV,ptr_class,ptr_site,classval,siteval,
                              class_name,site_name,&class_counter);
   switch (valid)
      {
      case FALSE:    /* an invalid polygon, skip over and loop again */
	skip_xy_pts(regxy,ptr_reg,&cur_poly);
	break;
      case TRUE:
/* 
 Valid polygon, get the x,y points that make up the polygon and
 then search the stat file for the site node.  Write the information
 in the stat file and check for a error writting to the file.
*/
	get_xy_pts(vblock,fd,regxy,ptr_reg,ptr_x,ptr_y,xy,
                      &pshape.buf[2],&cur_poly,hstat,nfile);
	stat_site(vblock,fd,"POLYGON",class_name,site_name,&nclass,
		  &class_index,&site_index,hstat,nfile);
        strcpy(dtype,"POLYGON");
	status = c_stput(fd,dtype,&class_index,&site_index,pshape.buf,xy);
	if ((status != E_DTNEW) &&(status != E_NOCL) && (status != E_NOSI))
	   {
           c_errmsg("Error writing to the statistics file",
		    "tab2stat-write",&nonfatal);
           c_stclse(fd);
           c_clean(vblock,"tab2stat",hstat,nfile);
	   }
	break;
      }
   }
c_stoptm();
close_tab(regxy);				/* Close Labeled Table files  */
close_tab(regatt);

/* Free allocatd space */
free(xy);

return;
}
