#include "tab2stat.h"
#include "util.h"
#include "protos.h"

void stat_class
(
    struct PARBLK *vblock,	/* parameter block		*/
    FILE **fd,			/* file descriptor      	*/
    char data_type[],		/* name of the data item	*/
    char class_name[],		/* STAT class name		*/
    long *nclass,		/* total number of classes in STAT file */
    long *class_index,		/* class name			*/
    long *site_index,		/* site name			*/
    char hstat[][CMLEN],		/* host stat file name		*/
    long *nfile		/* whether or not a new stat file was created*/
)

/*
**********************************************************************

NAME: Stat_Class                            

PURPOSE: To search the STAT file for the class node and return the class
         and site index. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      Allocate dynamic memory for descriptors and buffer
      Set up shape buffers for stat calls
      Search the STAT file for the class name
    	 If the class name already exists then
	   Read the spedified data type to see if the data exists
	   If the data doesn't exist then
	     Data will be inserted
  	   else the data already exists 
	     An ascending number is appended to the class name
	   If an error writting to STAT file then
	     return a fatal error
         Else if class name doesn't exist then
	   A new class is created
	   If an error writting to STAT file then
	     return a fatal error

************************************************************************
*/
{
char *cname;			/* STAT class name descriptor                 */
char *dtype;			/* STAT data type descriptor                  */
union STAT_SHAPE nshape;	/* STAT shape buffer for data type NAME       */
union STAT_SHAPE shape;		/* STAT shape buffer for other data types     */
long *buf;			/* Buffer to read STAT data into              */
long count;			/* Loop index counter                         */
long data_exist;		/* Boolean indicating if data exists          */
long found;			/* Boolean finding the class site names       */
long status;			/* Function return status code                */
char temp[LIN_SIZ];		/* Temporary buffer to hold site name         */
long nonfatal = 1;

/*
*  Dynamic memory allocation
*/
cname = (char *)malloc(CMLEN);
dtype = (char *)malloc(CMLEN);
buf = (long *)malloc(1024);

*site_index = 0;

strcpy(nshape.dtype,"L*1 ");	/* Set up shape buffers for the STAT calls    */
nshape.buf[0] = EBYTE;
nshape.buf[1] = 1;

strcpy(temp,class_name);
strcpy(cname,temp);
/*
*  Search the STAT file for the class name.  Continue until either a unique 
*  class name is found, or the specified data type does not exist for the 
*  specified class name.
*/
for (found=TRUE,data_exist=TRUE,count=1; found && data_exist; count++)
   {
   strcpy(dtype,"NAME");
   status = c_stfdcl(fd,cname,class_index);
   switch (status)
      {
/*
*  If the class name already exists, read the specified data type to see 
*  if the data exists.  If the data does not exist, the data will be inserted, 
*  so boolean "found" is set to false.  If the data already exists, an ascending
*  numeric value is appended to the class name.
*/
      case E_SUCC:
	strcpy(dtype,data_type);
        status = c_stget(fd,dtype,class_index,site_index,shape.buf,buf);
        switch (status)
           {
           case E_NOSI:	data_exist = FALSE;			break;
	   default:	sprintf(temp,"%s_%d",class_name,count);	
			(*nclass)++;
			*class_index = *nclass;
			nshape.buf[2] = STNAME;
			status = c_stput(fd,dtype,class_index,site_index,
					 nshape.buf,temp);
			if ((status != E_DTNEW) && (status != E_NOCL) && 
			    (status != E_NOSI))
			   {
			   c_errmsg("Error creating new CLASS node in statistics file","tab2stat-newclass",&nonfatal);
   			   c_stclse(fd);
   			   c_clean(vblock,"tab2stat",hstat,nfile);
			   }
			found = FALSE;
			break;
	   }
	break;
/*
*  If the class name does not exist, a new class is created and boolean
*  "found" is set to false. 
*/
      case E_NOCLN:
	(*nclass)++;
	*class_index = *nclass;
	nshape.buf[2] = STNAME;
	status = c_stput(fd,dtype,class_index,site_index,nshape.buf,cname);
        if ((status != E_DTNEW) && (status != E_NOCL) && (status != E_NOSI))
	   {
           c_errmsg("Error creating new CLASS node in statistics file",
		    "tab2stat-newclass",&nonfatal);
   	   c_stclse(fd);
   	   c_clean(vblock,"tab2stat",hstat,nfile);
	   }
	found = FALSE;
	break;
      }
   }
free(cname);
free(buf);
free(dtype);
return;
}
