#include "tab2stat.h"
#include <string.h>
#include "util.h"
#include "protos.h"

void stat_covar
(
    struct PARBLK *vblock,	/* TAE parameter block                        */
    FILE **fd,			/* file descriptor		*/
    long cur_level,		/* numeric value of stats type	*/
    char class_name[],		/* STAT class name		*/
    char site_name[],		/* STAT site name		*/
    long *nclass,		/* total # of classes in STAT file	*/
    struct VECTOR *covar_matrix,/* pointer to value of covariance matrix*/
    struct VECTOR *mean_vec,	/* pointer to mean vector in LT 	*/
    struct VECTOR *npoints,	/* pointer to number of points in area  */
    char hstat[][CMLEN],		/* host stat file name			*/
    long *nfile		/* Whether a new STAT file was created or not */
)

/*
**********************************************************************

NAME: Stat_Covar                            

PURPOSE: To write the covariance matrix, mean vector (optional), and number
	 of points (optional) into the STAT file. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      Allocate dynamic memory
      Check level (stats type)
	 Find or create the STAT node in which to be written
	    covariance matrix
	    mean vector
	    number of points
      Check the data for each data type to make sure it doesn't already
                exist in the STAT file
      If write covariance matrix then
  	 write the matrix into the STAT file lower triangle array
	 If error writting to STAT file then
	    return a fatal error
      If write mean vector then
 	 write neam vector
	 If error writting to STAT file then
	    return a fatal error
      If write number of points then
	 write number of points
	 If error writting to STAT file then
	    return a fatal error

************************************************************************
*/
{
char *dtype;			/* data type of the data found		*/
static double *matrix = NULL;	/* buffer to write data to & from	*/
long class_index;		/* index of the class			*/
long col;			/* loop control variable 		*/
long covar_flag;		/* flag, whether to write covariance matrix */
long data_index;		/* index of data item			*/
long index;			/* lower triangle array index counter	*/  
long mean_flag;			/* flag, to write mean vector or not	*/
long npoints_flag;		/* flag, to write # of points or not	*/
long row;			/* triangle array index			*/
long site_index;		/* site index of the node		*/
long size;			/* size of matrix buffer		*/
long shape[9];			/* shape of vector of data found	*/
long status;			/* return message from stat routines	*/
char *data_type;		/* pointer to the STATs data type 	*/
char *msg;	
long nonfatal = 1;

dtype = (char *)malloc(MAX_FIELD_LEN);
msg = (char *)malloc(ERRLEN);

/*if (matrix == NULL)
   {*/
   size =(long) ( covar_matrix->size.nrow * covar_matrix->size.ncol * 
                  sizeof(double));
   if ((matrix = (double *) malloc(size)) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","tab2stat-dynamic",&nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
 /*  }*/
data_type = "COVARIANCE_MATRIX";
switch(cur_level)			/* Depending on the level, find or    */
   {					/* create the STAT node in which the  */
   case IMAGE_LEV:			/* covariance matrix, mean vector,    */
      class_index = site_index = 0;	/* and number of points is to be      */
      break;				/* written.			      */
   case CLASS_LEV:
      stat_class(vblock,fd,data_type,class_name,nclass,&class_index,
		 &site_index,hstat,nfile);
      break;
   case SITE_LEV:
      stat_site(vblock,fd,data_type,class_name,site_name,nclass,&class_index,
		&site_index,hstat,nfile);
      break;
   }
covar_flag = mean_flag = npoints_flag = TRUE;
for (data_index=1;;data_index++)	/* Ensure the data for each data type */
   {					/* does not already exist in STAT file*/
   status = c_stnode(fd,&class_index,&site_index,&data_index,dtype,shape);
   if (status == E_NODT)
       break;
   if (strncmp(dtype,"COVARIANCE_MATRIX",17) == 0)
      {
      covar_flag = FALSE;
      sprintf(msg,"Covariance matrix already exists for class %d site %d.  %s",
           class_index,site_index,"Processing will continue with next matrix.");
      c_errmsg(msg,"informational",&nonfatal);
      }
   if (strncmp(dtype,"MEAN_VECTOR",11) == 0)
      {
      mean_flag = FALSE;
      sprintf(msg,"Mean vector already exists for class %d site %d.  %s",
	   class_index,site_index,"Processing will continue with next matrix.");
      c_errmsg(msg,"informational",&nonfatal);
      }
   if (strncmp(dtype,"NPOINTS",7) == 0)
      {
      npoints_flag = FALSE;
      sprintf(msg,"Number of points already exists for class %d site %d.  %s",
	   class_index,site_index,"Processing will continue with next matrix.");
      c_errmsg(msg,"informational",&nonfatal);
      }
   }

/* 
*  Write covariance matrix into the STAT file.  Note the matrix is     
*  stored in the STAT file as a lower triangle array.                    
*/
if (covar_flag)				
   {					
   size = covar_matrix->size.nrow;	
   for (index=0,row=0; row<size; row++)	
      for (col=0; col<=row; col++,index++)
         *(matrix + index) = *(covar_matrix->ptr.r8 + (row * size) + col);
   shape[0] = EDOUBLE;
   shape[1] = 1;
   shape[2] = index;
   strcpy(dtype,"COVARIANCE_MATRIX");
   status = c_stput(fd,dtype,&class_index,&site_index,shape,matrix);
   if ((status != E_SUCC) && (status != E_DTOLD) && ( status != E_DTNEW) && 
       (status != E_NOCL) && (status != E_NOSI))
      {
      c_errmsg("Error writing to statistics file","tab2stat-write",&nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
   }
/*
*  Write mean vector to the STAT file 
*/
if ((mean_flag) && (mean_vec != NULL))	
   {
   shape[0] = EDOUBLE;
   shape[1] = 1;
   shape[2] = mean_vec->size.ncol;
   strcpy(dtype,"MEAN_VECTOR");
   status = c_stput(fd,dtype,&class_index,&site_index,shape,mean_vec->ptr.r8);
   if ((status != E_SUCC) && (status != E_DTOLD) && ( status != E_DTNEW) && 
       (status != E_NOCL) && (status != E_NOSI))
      {
      c_errmsg("Error writing to statistics file","tab2stat-write",&nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
   }
/* 
*  Write number of points to the STAT file  
*/
if ((npoints_flag) && (npoints != NULL) && (!(npoints->null)))
   {					
   shape[0] = ELONG;       
   shape[1] = 1;
   shape[2] = 1;
   strcpy(dtype,"NPOINTS");
   status = c_stput(fd,dtype,&class_index,&site_index,shape,npoints->ptr.i4);
   if ((status != E_SUCC) && (status != E_DTOLD) && ( status != E_DTNEW) && 
       (status != E_NOCL) && (status != E_NOSI))
      {
      c_errmsg("Error writing to statistics file","tab2stat-write",&nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
   }
free(matrix);
free(msg);
free(dtype);
return;
}
