#include "tab2stat.h"
#include "util.h"
#include "protos.h"

void stat_site
(
    struct PARBLK *vblock,	/* TAE parameter block                        */
    FILE **fd,			/* file descriptor			*/
    char data_type[],		/* name of the data item		*/
    char class_name[],		/* CLASS class name			*/
    char site_name[],		/* STAT site name			*/
    long *nclass,		/* total number of classes in STAT file */
    long *class_index,		/* class name				*/
    long *site_index,		/* site name				*/
    char hstat[][CMLEN],		/* host stat file name			*/
    long *nfile		/* Whether a new STAT file was created or not */
)

/*
**********************************************************************

NAME: Stat_Site                            

PURPOSE: To search the STAT file for a site node and return the class and
	 site index.   

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
      Allocate dynamic memory
      Set up shape buffers
      Search the STAT file for the class & site names and continue until
    		 a unique class site name is found or the data type 
		 doesn't exist.
	 If class, site names exist then
            read data type
	    If data does not exist then
	       data will be inserted
	    If the data already exists then
	       An ascending numeric value is appended the site name
 	 If class name does not exist then 
	    A new class is created
	    Set site to not found
	 If site name does not exist then
	    set site to not found
      If site not found then
	 create a new site
	 If error writting site then
	    return a fatal error 

************************************************************************
*/
{
char cname[CMLEN];		/* STAT class name descriptor                 */
char dtype[CMLEN];		/* STAT data type descriptor                  */
char sname[CMLEN];		/* STAT site name descriptor                  */
union STAT_SHAPE nshape;	/* STAT shape buffer for data type NAME       */
union STAT_SHAPE shape;		/* STAT shape buffer for other data types     */
long *buf;			/* Buffer to read STAT data into              */
long count;			/* Loop index counter                         */
long data_exist;		/* Boolean indicating if data exists          */
long found;			/* Boolean finding the class site names       */
long status;			/* Function return status code                */
long id;			/* Position used in looking for open site     */long valid_site;		/* Boolean used for indecating open site      */
char temp[LIN_SIZ];		/* Temporary buffer to hold site name         */
long nonfatal = 1;

buf = (long *)malloc(1024);

nshape.buf[0] = EBYTE;
nshape.buf[1] = 1;

strcpy(cname,class_name);
strcpy(sname,site_name);
/*
*  Search the STAT file for the class and site names.  Continue until either
*  a unique class site name is found, or the specified data type does not 
*  exist for the specified class site name.
*/
for (found=TRUE,data_exist=TRUE,count=1; found && data_exist; count++)
   {
   strcpy(dtype,"NAME");
   status = c_stfdsi(fd,cname,sname,class_index,site_index);
   switch (status)
      {
/*
*  If the class site names already exist, read the specified data type to see 
*  if the data exists.  If the data does not exists, the data will be inserted, 
*  so boolean "found" is set to false.  If the data already exists, an ascending
*  numeric value is appended to the site name.
*/
      case E_SUCC:
	strcpy(dtype,data_type);
        status = c_stget(fd,dtype,class_index,site_index,shape.buf,buf);
        switch (status)
           {
           case E_NOSI:	data_exist = FALSE;
			break; 
           default:	sprintf(temp,"%s_%d",site_name,count);	
			break; 
           }
 	strcpy(sname, temp);
        break;
/*
*  If the class name does not exist, a new class is created.  This also implies
*  that the site name does not exist, so boolean "found" is set to false.
*/
      case E_NOCLN:
	(*nclass)++;
	*class_index = *nclass;
	*site_index = 0;
	nshape.buf[2] = (long) strlen(cname);
	status = c_stput(fd,dtype,class_index,site_index,nshape.buf,cname);
        if ((status != E_NOSI) && (status != E_DTNEW) && (status != E_NOCL))
           {
           c_errmsg("Error creating new CLASSNAM node in statistics file",
		    "tab2stat-newclass",&nonfatal);
           c_stclse(fd);
           c_clean(vblock,"tab2stat",hstat,nfile);
	   }
        (*site_index)++;
	found = FALSE;
	break;
/*
*  If the site name does not exist, set boolean "found" to false.  An open
*  site is searched for in the class and if a "valid site" is found then
*  index "site index" again and check next site.  When an open site is 
*  found then use the value of "site index" for the position of the site.
*/
      case E_NOSIN:
	found = FALSE;
	for (valid_site=TRUE, id=1; valid_site; ) 
	  {
	  (*site_index)++;
	  status = c_stnode(fd,class_index,site_index,&id,dtype,nshape.buf);
	  if ((status == E_NOSI) || (status == E_NODT))
	    valid_site = FALSE;
	  else if (status == E_GNLIO)
	    {
            c_errmsg("Error looking for open site in statistics file",
	  	     "tab2stat-esite",&nonfatal);
            c_stclse(fd);
            c_clean(vblock,"tab2stat",hstat,nfile);
	    }
	  }
	break;
      }
   }
/*
*  If the site name does not exist, create the new site.
*/
if (found == FALSE)
   {
   nshape.buf[2] = (long) strlen(sname);
   status = c_stput(fd,dtype,class_index,site_index,nshape.buf,sname);
   if ((status != E_DTNEW) && (status != E_NOCL) && (status != E_NOSI))
      {
      c_errmsg("Error creating new SITENAM node in statistics file",
 	       "tab2stat-newsite",&nonfatal);
      c_stclse(fd);
      c_clean(vblock,"tab2stat",hstat,nfile);
      }
   }

/* Free allocated space */
free(buf);

return;
}
