#include "tab2stat.h"
#include <stdlib.h>
#include "util.h"
#include "protos.h"
/******************************************************************************

NAME:	TAB2STAT

FUNCTION:
	TAB2STAT is an application program which converts a Labeled Table
	file to a STAT file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         10/86      K. Gacke     original development
    1.1          1/87      K. Gacke     new LT file format
    1.2		 1/90	   J. Marmon	v5.0 convarsion
    1.3  	 2/90	   B.Ailts      Changed order that X and Y were
				        stored
    1.4          12/91     T.Mittan     Corrected error in parameter passing
    1.5          3/93      D.Etrheim    ADP #6295 Allow polygon coordinates to
					be real numbers

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

LANGUAGE:	C

ALGORITHM (FLOW):
      Input LT file and output STAT file opened
      If the file type of the LT is not a valid LAS stats data type
		  (polygon, covariance matrix) then
	 return a fatal error
      Else if file type equal polygon then
	 process polygons
      Else if file type equal covar then
	 process covariance matrix
      Else Labeled Table file type is invalid
	 return a fatal error
      Close files
      Print completion message.
 
******************************************************************************/

int main
(void)
{
char hstat[1][CMLEN],		/* Host name of STAT file                     */
     htab[CMLEN];		/* Host name of Labeled Table file            */

struct TAB_DEF tab;		/* Definition structure of Labeled Table      */
struct PARBLK vblock;		/* TAE parameter block                        */
struct VARIABLE *val;		/* Pointer to a variable within TAE vblock    */

FILE *    fd;
long	f_flag,			/* True only if IMAGE is picked, site and     */
				/* class values are not needed.		      */
	index,			/* Loop index counter                         */
	level[3],		/* Level of STAT file to be processed         */
	msgtim,			/* Number of seconds between processing msg.  */
	nclass = 0,		/* Total number of classes in STAT file       */
	nfile,			/* Number of files created for completion msg */
	status,			/* Function return status code                */
        std_out,		/* Fortran logical unit number to standard out*/
        access = IREAD;

char	*class,			/* Label that is to be used as the STAT class */
	*classval[TAE_VAL],	/* Label values that are to be STAT classes   */
	*ftyp,			/* Label that is to be used as the file type  */
	*lev_atr,		/* Label that is to be used as the STAT level */
	*regnm,			/* Label that is to be used as the STAT regnum*/
	*site,			/* Label that is to be used as the STAT site  */
	*siteval[TAE_VAL],	/* Label values that are to be STAT sites     */
	*Xcoor,			/* Label to be used as the STAT X coordinate  */
	*Ycoor;			/* Label to be used as the STAT Y coordinate  */
long fatal = -1;
long nonfatal = 1;
long zero = 0;
long one =1;

c_inlas(&vblock,&std_out);

if ((status = c_getnam(&vblock, "INLT", &zero, htab, "lt", &access)) == E_FAIL)
   c_errmsg("Fatal error encountered","tab2stat-fatal",&fatal);

access = IUPDATE;
if ((status = c_getnam(&vblock, "OUTSTAT", &zero, hstat[0], "stat", &access)) ==
	E_FAIL)
   c_errmsg("Fatal error encountered","tab2stat-fatal",&fatal);

val = p_find(&vblock,"STATSTYP");
if (val->v_count == 0)
   {
   level[IMAGE_LEV] = level[CLASS_LEV] = level[SITE_LEV] = TRUE;
   f_flag = TRUE;
   }
else
   {
   level[IMAGE_LEV] = level[CLASS_LEV] = level[SITE_LEV] = FALSE;
   for (index=0; index<val->v_count; index++)
      {
      if (strcmp(SVAL(*val,index),"IMAGE") == 0)
	 {
         level[IMAGE_LEV] = TRUE;
	 f_flag = FALSE;
	 }
      else if (strcmp(SVAL(*val,index),"CLASS") == 0)
 	 {
         level[CLASS_LEV] = TRUE;
	 f_flag = TRUE;
	 }
      else if (strcmp(SVAL(*val,index),"SITE") == 0) 
	 {
  	 level[SITE_LEV] = TRUE;
	 f_flag = TRUE;
	 }
      }
   }
val = p_find(&vblock,"FTYPE");
ftyp = SVAL(*val,0);

/*
*  f_flag is set to FALSE only if IMAGE is choosen as the STATSTYP
*  which means that there will be no class or site data.  For this
*  reason f_flag is checked before capitalizing class or site values.
*/
val = p_find(&vblock,"CLASSATR");
class = SVAL(*val,0);
if (f_flag)
  upcase(class);

val = p_find(&vblock,"SITEATR");
site = SVAL(*val,0);
if (f_flag)
  upcase(site);

val = p_find(&vblock,"LEVELATR");
lev_atr = SVAL(*val,0);
upcase(lev_atr);

val = p_find(&vblock,"POLYNATR");
regnm = SVAL(*val,0);
upcase(regnm);

val = p_find(&vblock,"COORSATR");
Ycoor = SVAL(*val,0);
upcase(Ycoor);

Xcoor = SVAL(*val,1);
upcase(Xcoor);

val = p_find(&vblock,"CLASSNAM");
for (index=0; index<val->v_count; index++)
   {
   classval[index] = SVAL(*val,index);
   upcase(classval[index]);
   }
classval[index] = '\0';

val = p_find(&vblock,"SITENAM");
for (index=0; index<val->v_count; index++)
   {
   siteval[index] = SVAL(*val,index);
   upcase(siteval[index]);
   }
siteval[index] = '\0';

val = p_find(&vblock,"$MSGTIM");
msgtim = IVAL(*val,0);

open_tab(&tab,htab,0);		 	     /* Open Labeled Table file   */
get_vector(&tab);
access = IUNKNOWN;
status = c_stopen(&fd,hstat[0],&access,&nclass);/*       Open STAT file      */
switch (status)
   {
   case E_OPOLD: nfile = 0;	break;
   case E_OPNEW: nfile = 1;	break;
   }
/*
*  Check for which file type the Labeled Table is:
*  POLYgon or COVARariance. Then process that type
*  of file.
*/
if (strcmp(ftyp,"POLY") == 0) 
   {
   if (!(level[SITE_LEV]))
      {
      c_errmsg("Polygon data is required to be at the site level",
	       "tab2stat-level",&nonfatal);
      c_stclse(&fd);
      c_clean(&vblock,"tab2stat",hstat,&nfile);
      }						
/* 
  Process polygon information         
*/
   process_poly(&vblock,&fd,&tab,class,site,classval,siteval,regnm,Xcoor,Ycoor,
	         nclass,&msgtim,hstat,&nfile);
   }
else 	/* Process covariance matrix */
   {
   process_covar(&vblock,&fd,&tab,level,lev_atr,f_flag,class,site,classval,
                  siteval,nclass,&msgtim,hstat,&nfile);
   }

c_stclse(&fd);					/* Close STAT file            */

c_complt("TAB2STAT",hstat,&one);		/* Output completion message  */

return EXIT_SUCCESS;

}
