/******************************************************************************

NAME:	GETPAR

FUNCTION:  GETPAR retrieves the TAE parameter values

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Retreive the output file name
Retreive the input file name
Retreive the the other parameters
Upper case all the attribute names

******************************************************************************/

#include "las.h"
#include "tab2tie.h"
#include "util.h"
#include "geompak.h"
#include "ltable.h"
#include "protos.h"

void getpar
(
    struct PARBLK *parb,		/* parameter block structure */
    char *hosin,			/* host input file name descriptor */
    char *hosout,			/* host output file name descriptor */
    long *atrcnt,			/* number of output columns specified */
    char atrname[][CMLEN],		/* attribute name for each output col */
    long *atrcode			/* the code for each attribute		*/
)

{ /* getpar */

struct VARIABLE *val;			/* pointer to variable */

long access;				/* type of access of file */
long coorcnt;				/* the no. of coor. attributes */
long fatal = -1;			/* fatal error code		*/
long i;					/* counter--current parameter value */
long lsccnt;				/* the no. of l/s coor. attributes */
long nbytes;				/* nbytes of atrnames		*/
long nonfatal = 1;			/* nonfatal error code		*/
long status;				/* status of function calls	*/
long zero = 0;				/* value of zero 		*/

/*  Initialize variables
------------------------*/
*atrcnt = 0;

/* Retrieve output labeled table file name.
-------------------------------------------*/
access = IWRITE;
status = c_getnam(parb,"OUTTS",&zero,hosout,"tps",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tab2tie-fatal",&fatal);

/*  Retrieve input ascii file name.
-----------------------------------*/
access = IREAD;
status = c_getnam(parb,"INLT",&zero,hosin,"lt",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tab2tie-fatal",&fatal);

/*  Retreive parameter values.
------------------------------*/
val = p_find(parb,"COORSATR");
coorcnt = val->v_count;
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSCOORD1;
   (*atrcnt)++;
   }
if ((*val).v_count == 2)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,1));
   atrcode[*atrcnt] = TPSCOORD2;
   (*atrcnt)++;
   }

val = p_find(parb,"LSCATR");
lsccnt = val->v_count;
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSIMGCRD1;
   (*atrcnt)++;
   }
if ((*val).v_count == 2)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,1));
   atrcode[*atrcnt] = TPSIMGCRD2;
   (*atrcnt)++;
   }

/*  Either CORRATR or LSCATR must be specified to create a valid TPS file
-------------------------------------------------------------------------*/
if ((coorcnt == 0) && (lsccnt ==0))
   {
   c_errmsg("Either COOSATR or LSCATR must be specified",
	    "tab2tie-coor",&nonfatal);
   c_errmsg("Fatal error encoutered","tab2tie-fatal",&fatal);
   }

val = p_find(parb,"PVALATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSPROJV;
   (*atrcnt)++;
   }

val = p_find(parb,"CORNATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSUL;
   (*atrcnt)++;
   }
if ((*val).v_count == 4)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,1));
   atrcode[*atrcnt] = TPSLL;
   (*atrcnt)++;
   strcpy(atrname[*atrcnt],SVAL(*val,2));
   atrcode[*atrcnt] = TPSUR;
   (*atrcnt)++;
   strcpy(atrname[*atrcnt],SVAL(*val,3));
   atrcode[*atrcnt] = TPSLR;
   (*atrcnt)++;
   }

val = p_find(parb,"PDATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSDIST1;
   (*atrcnt)++;
   }
if ((*val).v_count == 2)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,1));
   atrcode[*atrcnt] = TPSDIST2;
   (*atrcnt)++;
   }

val = p_find(parb,"PPARMATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSPPARMS;
   (*atrcnt)++;
   }

val = p_find(parb,"PTYPEATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSCODE;
   (*atrcnt)++;
   }

val = p_find(parb,"ZONEATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSZONE;
   (*atrcnt)++;
   }

val = p_find(parb,"DCODEATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSDATUM;
   (*atrcnt)++;
   }

val = p_find(parb,"MODEATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSMODE;
   (*atrcnt)++;
   }

val = p_find(parb,"PUNITATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSUNITS;
   (*atrcnt)++;
   }

val = p_find(parb,"NAMEATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSNAME;
   (*atrcnt)++;
   }

val = p_find(parb,"PTIDATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSPTID;
   (*atrcnt)++;
   }

val = p_find(parb,"ELEVATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSELEV;
   (*atrcnt)++;
   }

val = p_find(parb,"SCALFATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSSCAL;
   (*atrcnt)++;
   }

val = p_find(parb,"REFSZATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSREFSZ1;
   (*atrcnt)++;
   }
if ((*val).v_count == 2)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,1));
   atrcode[*atrcnt] = TPSREFSZ2;
   (*atrcnt)++;
   }

val = p_find(parb,"SRCOFATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSSOFF1;
   (*atrcnt)++;
   }
if ((*val).v_count == 2)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,1));
   atrcode[*atrcnt] = TPSSOFF2;
   (*atrcnt)++;
   }

val = p_find(parb,"CNAMEATR");
if ((*val).v_count > 0)
   {
   strcpy(atrname[*atrcnt],SVAL(*val,0));
   atrcode[*atrcnt] = TPSSNAME;
   (*atrcnt)++;
   }

for (i = 0; i < *atrcnt; i++)
   {
   nbytes = (long) strlen(atrname[i]);
   c_low2up(atrname[i],&nbytes);
   }
return;
} /* getpar */
