/******************************************************************************
NAME:	TAB2TIE

FUNCTION:  TAB2TIE converts a labeled table file to a tie point selection file

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5           06/89      B. Ailts     original development
    5.1         08/89      B. Ailts     Fixed problem with char. data on SUN-4
    5.2		05/90      D. Etrheim	Initialize all fields in the data record
    5.3		10/90      D. Etrheim   ADP #5795 Allow point ID to be I*4 
    5.4		01/94      D. Etrheim   Allow PTID to be any data type

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):

Get parameter values
Open output labeled table file
Read one line(row) at a time and place in labeled table file
Close the input files.
Create the TAE output file name to be used in the completion message.
Ouptut completion message

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "util.h"
#include "tab2tie.h"
#include "protos.h"

int main
(void)
{

struct PARBLK parb;			/* TAE parameter block */

struct TAB_DEF tab[MAX_SUB + 1];	/* label table descriptot */

long atrcnt;				/* attribute count		*/
long atrcode[MAXATR];			/* attribute codes		*/
long i;					/* counter--current line */
long lun;				/* fortran LUN to standard out */
long one = 1;				/* value of one */

char atrname[MAXATR][CMLEN];		/*  attribute names specified	*/
char hosin[CMLEN];			/* host input name */
char hosout[1][CMLEN];			/* host output name */

 
c_inlas(&parb,&lun);

/*  Get parameter values
------------------------*/
getpar(&parb,hosin,hosout[0],&atrcnt,atrname,atrcode);

/*  open output labeled table fle
----------------------------------*/
labtabop(tab,hosin);

/*  Read one line(row) at a time and place in labeled table file
----------------------------------------------------------------*/
process(&parb,tab,hosout,&atrcnt,atrname,atrcode);

/*  Close the input files.
--------------------------*/
for (i = 0; i <= tab[0].nsubs; i++)
   close_tab(&tab[i]);

/*  Ouptut completion message
-----------------------------*/
c_complt("TAB2TIE",hosout,&one);

return EXIT_SUCCESS;

}
