/*******************************************************************************
NAME:    GETPAR

PURPOSE: Retrieve the required information for TAPELABEL.

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    T.Baltzer        06/92  EDC   Development
  1.1    D.Etrheim        09/92  EDC   Change PURGE to NMONTHS for standards
                                       Make change so if not at EROS, printflg
                                       is overridden
  1.2    D.Etrheim        10/92  EDC   Change thebuf to be an array of pointers.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS		                

*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "protos.h"


void getpar
(
    struct PARBLK *vblock,	/* TAE parameter block					*/
    char **thebuf,	/* buffer of information: LIBRARY FORMAT		*/
                    /*  [ 0] code (code for the tape)                  */
                    /*  [ 1] density (density of the tape)             */
                    /*  [ 2] description (description of the tape)     */
                    /*  [ 3] drive (tape drive used)                   */
                    /*  [ 4] project (project the tape was created for)*/
                    /*  [ 5] requestor (initials of the tape creator)  */
                    /*  [ 6] sequence (sequence number of the tape)    */
                    /*  [ 7] tapeid (tape ID number)                   */
                    /*  [ 8] ops error flag                             */
                    /*  [ 9] unused                                     */
                    /*  [10] unused                                     */
                    /*   buffer of information: FREE FORMAT             */
                    /* [0 - 10] text in free format                    */
    long *retain,		/* months to retain beyond current date			*/
    long *printflg,		/* print flag 0 = NO, 1 = YES				*/
    long *formflg		/* format flag 0 = FREE, 1 = STANDARD			*/
)

{
struct VARIABLE *var;      /* pointer to TAE variable within v-block		*/

long i;			   /* counter						*/
long j;                    /* Temp variable					*/
long nonfatal = 1;	   /* non-fatal error status				*/
long seqnum;		   /* sequence number of tape within set		*/
long tottapes;		   /* total number of tapes within set			*/

char tempstr[5];	   /* temporary string					*/
char subcom[5];		   /* sub-command					*/

/* Allocate space */
for (i = 0; i < 11; i++)
    thebuf[i] = (char *)malloc(52);

/* determine the subcommand chosen
-----------------------------------*/
var = p_find(vblock, "_SUBCMD");
strcpy(subcom, SVAL(*var,0));
if (strcmp(subcom,"LIB") == 0)
   *formflg = 1;
else
   *formflg = 0;


/* Retrieve the required parameters
-----------------------------------*/

var = p_find(vblock, "PRINTFLG");
strcpy(tempstr, SVAL(*var, 0));
if (strcmp(tempstr,"YES") == 0)
   *printflg = 1;
else
   *printflg = 0;

/* Make sure we are at EROS Data Center
---------------------------------------*/
var = p_find(vblock,"$FNAME");
if (strcmp("EROS Data Center",SVAL(*var,0)) != 0)
   *printflg = 0;

if (strcmp(subcom,"LIB") == 0)
   {
   var = p_find(vblock, "CODE");
   strcpy(thebuf[0], SVAL(*var, 0));
   var = p_find(vblock, "TAPEDENS");
   strcpy(thebuf[1], SVAL(*var, 0));
   var = p_find(vblock, "DESCRPT");
   strcpy(thebuf[2], SVAL(*var, 0));
   var = p_find(vblock, "TAPEDRIV");
   strcpy(thebuf[3], SVAL(*var, 0));
   var = p_find(vblock, "PROJECT");
   strcpy(thebuf[4], SVAL(*var, 0));
   var = p_find(vblock, "NAME");
   strcpy(thebuf[5], SVAL(*var, 0));
   var = p_find(vblock, "SEQNUM");
   seqnum = IVAL(*var, 0);
   var = p_find(vblock, "NTAPES");
   tottapes = IVAL(*var, 0);
   sprintf(thebuf[6],"%d/%d",seqnum,tottapes);
   var = p_find(vblock, "TAPEID");
   strcpy(thebuf[7], SVAL(*var, 0));
   var = p_find(vblock, "NMONTHS");
   *retain = IVAL(*var, 0);

if (seqnum > tottapes)
   {
   c_errmsg("Sequence number greater than total number","tapelabel-seq",&nonfatal);
   c_errmsg("Nonfatal error encountered","tapelabel-warn",&nonfatal);
   }

   /* For consistency convert all fields to lowercase
   --------------------------------------------------*/
   for (i = 0; i < 8; i++)
      {
      j = (long) strlen(thebuf[i]);
      c_up2low(thebuf[i], &j);
      }
   }

   else  /* free format label (subcommand -FREE) */
   {
   var = p_find(vblock, "TEXT");
   for (i = 0; i < var->v_count; i++)
      strcpy(thebuf[i], SVAL(*var, i));
   }

}
