/*******************************************************************************
NAME:    TAPELABEL

PURPOSE: Generate a tape label.

PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -------------
  1.0    T.Baltzer        06/92  EDC   Development
  1.1    D.Etrheim        09/92  EDC   Change PURGE to NMONTHS for standards
				       Make change so if not at EROS, printflg
				       is overridden
  1.2    D.Etrheim        10/92  EDC   Change thebuf to be an array of pointers.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS		                


*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "protos.h"


int main
(void)
{
struct PARBLK   vblock;       /* TAE parameter block                          */

char *thebuf[11];	/* buffer of information: LIBRARY FORMAT		*/
                          /* [ 0] code (code for the tape)                  */
                          /* [ 1] density (density of the tape)             */
                          /* [ 2] description (description of the tape)     */
                          /* [ 3] drive (tape drive used)                   */
                          /* [ 4] project (project the tape was created for)*/
                          /* [ 5] requestor (initials of the tape creator)  */
                          /* [ 6] sequence (sequence number of the tape)    */
                          /* [ 7] tapeid (tape ID number)                   */
                          /* [ 8] ops error flag				*/
                          /* [ 9] unused					*/
                          /* [10] unused					*/
			/* buffer of information: FREE FORMAT			*/
                          /* [0 - 10] text in free format			*/

char tae_name[1][CMLEN];  /* tae name of label file (returned value)		*/
char *vec[1];		  /* array of pointers for q_string			*/

long fatal = -1;	      /* fatal error status			      */
long formflg;		      /* format flag 0 = FREE , 1 = STANDARD	      */
long nonfatal = 1;	      /* non-fatal error status			      */
long printflg;		      /* print flag 0 = NO , 1 = YES		      */
long retain = 0;	      /* months to retain beyond current date	      */
long std_out;                 /* Logical unit number for standard output      */
long nof;		      /* number of output files			      */


/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);

/* Retrieve the required parameters
-----------------------------------*/
getpar(&vblock,thebuf,&retain,&printflg,&formflg);


/* Create the label
--------------------*/
if ( c_tapelabel(thebuf,retain,printflg,formflg,tae_name[0]) != E_SUCC )
   c_errmsg("Fatal error encountered","tapelabel-fatal",&fatal);

/* Send the name of the tape label file back to TAE land
---------------------------------------------------------*/
vec[0] = tae_name[0];
if (q_string(&vblock,"OUTFILE",1,vec,P_UPDATE) != SUCCESS)
   {
   c_errmsg("Error returning parameter to TAE","tapelabel-rparm",&nonfatal);
   c_errmsg("Fatal error encountered","tapelabel-fatal",&fatal);
   }

if (q_out(&vblock) != SUCCESS)
   {
   c_errmsg("Error returning parameter to TAE","tapelabel-rparm",&nonfatal);
   c_errmsg("Fatal error encountered","tapelabel-fatal",&fatal);
   }


/* Output completion message
----------------------------*/
if (printflg == 0)
   nof = 1;
else
   nof = 0;
c_complt("TAPELABEL",tae_name,&nof);

return EXIT_SUCCESS;

}
