#include <stdio.h>
#include "las.h"
#include "tapeio.h"
#include "protos.h"


/******************************************************************************
FUNCTION:       COMPUTE_NS

PURPOSE:        Reads the first record of the image file to determine the
                actual number of samples per image line.  The tape is back
                spaced one record so that the tape is still at the beginning
                of the image file when the function completes.

ALGORITHM 
  Allocate buffer space
  Determine the number of bytes per image record by reading a record
  from the tape.
  Back up one record on tape for the record just read
  Free the buffer space allocated
  Compute the number of samples per image line from the number of bytes
  per image record

ALGORITHM REFERENCES    none
*******************************************************************************/
int compute_ns
(
    long *unit,                 /* tape unit number                           */
    long *idtype,               /* input image data type                      */
    long *blksiz                /* number of lines in block                   */
)

{
    extern long datasize[NDTYPES];  /* defines number bytes for each data type*/
    long bytes_read;                /* actual # of bytes read from tape       */
    long mode = HANDLE;             /* tape I/O outputs error message         */
    long nbyte;                     /* # of bytes to be read from tape        */
    long ns;                        /* number of samples per image line       */
    long one = 1;                   /* value of one                           */
    long fatal = -1;                /* Fatal error code                       */
    long nonfatal = 1;              /* Non-Fatal error code                   */

    unsigned char   *buf;           /* buffer pointer to read into            */

    struct TAPEDESC *tp;            /* tape drive description                 */

    /* Allocate buffer space
       ------------------------*/
    tp = (struct TAPEDESC *) *unit;
    nbyte = tp->maxbyte;
    buf = (unsigned char *) malloc(nbyte);
    if (buf == NULL)
    {
        c_errmsg("Error allocating dynamic memory","tapein-alloc",&nonfatal);
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);
    }

    /* Determine the number of bytes per image record
       by reading a record from the tape.
       -------------------------------------------------*/
    bytes_read = c_tpread(unit,buf,&nbyte,&mode);

    /* Back up one record on tape for the record just read
       ------------------------------------------------------*/
    c_tpbsr(unit,&one,&mode);

    /* Free the buffer space allocated
       ----------------------------------*/
    free(buf);

    /* Compute the number of samples per image line
       from the number of bytes per image record
       -----------------------------------------------*/
    ns = (bytes_read / datasize[*idtype]) / *blksiz;

    return(ns);
}
