#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "sysdef.h"
#include "protos.h"

#define MAX_VOL 15


/******************************************************************************

FUNCTION:       GETPAR

PURPOSE:        Get the input parameters

ALGORITHM REFERENCES    none

*******************************************************************************/
void getpar
(
    long *access,               /* output image file access                  */
    long bands[][MAXBND+1],     /* bands to be processed within image        */
    long *blksiz,               /* number of lines within input block        */
    long *curbnd,               /* current band to be written to             */
    long *idtype,               /* data type of input image                  */
    long *inhdr,                /* number of header records to skip          */
    long *inspc,                /* Total number lines, samples, and bands    */
    long *insys,                /* Input system data is from                 */
    long *linc,                 /* line increment for subsample              */
    long *m2one,                /* many input files to one output file flag  */
    long *msgtim,               /* elapsed time between display msg          */
    long *odtype,               /* data type of output image                 */
    long *outsys,               /* Output system data is to be put to        */
    long *sinc,                 /* sample increment for subsample            */
    long *skip,                 /* number files to skip prior to copy        */
    long *totbnd,               /* total number of bands to process          */
    long window[][4],           /* window of input image                     */
    short *ddr_flag,            /* flag to copy ddr info                     */
    short *hp_flag,             /* flag for HP computer system               */
    short *minmax,              /* min max flag                              */
    char *bform,                /* input band format (BSQ or BIL)            */
    char *hosout,               /* Output host name                          */
    char *taeout,               /* Output TAE name                           */
    long *compflg,              /* Image compression flag                    */
    char *tddrnam,              /* temporary ddr name                        */
    long *nbands                /* number of bands                           */
)

{
    extern struct PARBLK vblock;
    extern long  curfil;         /* current file being processed              */
    extern long  curvol;         /* current tape volume being process         */
    extern long  nmrfil;         /* total # of files to be processed          */
    extern long  return_EOV;     /* flag to return EOV_detected               */
    extern char comment[][CMLEN];/* tape mount message to the operator        */
    extern char *drive[1];       /* physical tape drive name                  */

    struct VARIABLE *val;        /* pointer to a TAE variable                 */
    struct VARIABLE *val2;       /* pointer to a TAE variable                 */

    long index;                  /* loop index                                */
    long vol;                    /* Volume continuation value                 */
    long std_out;                /* fortran standard out LUN                  */
    long sysflg = SETN;          /* Set system numeric code from system string*/
    long fatal = -1;             /* Fatal error code                          */
    long nonfatal = 1;           /* Nonfatal error code                       */
    long chkflg;                 /* check if image exists or not Flag         */
    long skipf;                  /* number of band numbers to skip            */

    char system[15];             /* Current system                            */
    char type[5];                /* Input/Output type string                  */
    char outfile[CMLEN];         /* output file                               */
    char subcommand[CMLEN];      /* subcommand                                */


    /* Initialize LAS
       -----------------*/
    c_inlas(&vblock,&std_out);

    /* Retrieve tape mount messages supplied by user and create new messages,
       up to MAX_VOL, by adding the string " -- NEXT VOLUME #" onto the last 
       message
       -----------------------------------------------------------------------*/
    val = p_find(&vblock,"COMMENT");
    for (index = 0; index < val->v_count; index++)
        strcpy(comment[index],SVAL(*val,index));
    for (vol = 1, index = val->v_count; index < MAX_VOL; index++, vol++)
        sprintf(comment[index],"%s%s%d",comment[val->v_count - 1],
                " -- NEXT VOLUME ",vol);

    /* Retrieve the current file number being processed (1, 2, 3, ...)
       -------------------------------------------------------------------*/
    val = p_find(&vblock,"CURFIL");
    curfil = IVAL(*val,0);

    /* Retrieve the current subcommand 
       ----------------------------------*/
    val = p_find(&vblock,"_SUBCMD");
    strcpy(subcommand,SVAL(*val,0));

    /* if the command is EOV, set the return_EOV flag */
    if (strcmp(subcommand,"EOV") == 0)
        return_EOV = 1;

    /* Retrieve the "many to one" flag
       ----------------------------------*/
    val = p_find(&vblock,"M2ONE");
    *m2one = IVAL(*val,0);

    /* Retrieve output image name
       -----------------------------*/
    val = p_find(&vblock,"OUT");
    strcpy(outfile,SVAL(*val,0));
    if (!(*m2one) || (*m2one && (curfil == 1)))
        chkflg = 1;
    else 
        chkflg = 0;
    if (c_mkimg(outfile,"",taeout,hosout,&chkflg) == E_FAIL)
        c_errmsg("Fatal error encountered","tapein-fatal",&fatal);

    /* Retrieve number of files to be skipped
       -----------------------------------------*/
    val = p_find(&vblock,"SKIPNUM");
    *skip = IVAL(*val,0);

    /* Retrieve number of header records
       ------------------------------------*/
    val = p_find(&vblock,"NHEADREC");
    *inhdr = IVAL(*val,0);

    /* Retrieve input band format.  If m2one then BFORM must be BSQ
       ---------------------------------------------------------------*/
    val = p_find(&vblock,"BFORM");
    strcpy(bform,SVAL(*val,0));
    if (*m2one && (strcmp(bform, "BSQ") != 0))
    {
        c_errmsg("Invalid band format specification","tapein-bform",&nonfatal);
        c_errmsg("Fatal error encountered", "tapein-fatal", &fatal);
    }

    /* Retrieve ddr copy flag.  If m2one DDRFLG must be NO.
       ------------------------------------------------------*/
    val = p_find(&vblock,"DDRFLG");
    if (strcmp(SVAL(*val,0),"NO") == 0)
        *ddr_flag = FALSE;
    /*
       else if (*m2one)
       {
       c_errmsg("Invalid DDRFLG specification", "tapein-ddrflg", &nonfatal);
     *ddr_flag = FALSE;
     }
     */
    else
        *ddr_flag = TRUE;

    /* Retrieve total number of tape files to be copied
       ---------------------------------------------------*/
    val = p_find(&vblock,"NMRFIL");
    nmrfil = IVAL(*val,0);

    /* Retrieve window area to be copied
       ------------------------------------*/
    val = p_find(&vblock,"WINDOW");
    for (index = 0; index < 4; index++)
        window[0][index] = IVAL(*val,index);

    /* Retrieve the current system
       ------------------------------*/
    c_getsys(system);
    c_sysset(system, outsys, &sysflg);

    /* Retrieve system the input tape was generated on.  Set a flag if the input
       system was from an HP.  If INSYS is null then let it be the same as the
       current system.  If a DDR is present, INSYS will be taken from it later 
       on.
       -----------------------------------------------------------------------*/
    *hp_flag = FALSE;
    val = p_find(&vblock,"INSYS");
    if (val->v_count == 0)
        *insys = *outsys;
    else
    {
        strcpy(system,SVAL(*val,0));
        if (strcmp(system,"PN") == 0)
            *insys = SYS_GOULD_UTX;
        else if (strcmp(system,"VAX") == 0)
            *insys = SYS_VAX_VMS;
        else if (strcmp(system,"SUN") == 0)
            *insys = SYS_SUN_BSD;
        else if (strcmp(system,"IBMR2") == 0)
            *insys = SYS_IBM_AIX;
        else if (strcmp(system,"IEEE") == 0)
            *insys = SYS_IEEE_STD;
        else if (strcmp(system,"HP") == 0)
        {
            *insys = SYS_HP_MPE;
            *hp_flag = TRUE;
        }
    }

    /* Retrieve TOTLINES, TOTSAMPS, and TOTBANDS from the variable block.  If
       any of these variables are null, their values should be set to 0.  This
       will indicate that a DDR must exist along with the image file on tape.
       If values are supplied for these variables and a DDR exists on tape, 
       these values will be overwritten with the values in the DDR.
       -----------------------------------------------------------------------*/
    val = p_find(&vblock,"TOTLINES");
    if (val->v_count == 0)
        inspc[0] = 0;
    else
        inspc[0] = IVAL(*val,0);
    val = p_find(&vblock,"TOTSAMPS");
    if (val->v_count == 0)
        inspc[1] = 0;
    else
        inspc[1] = IVAL(*val,0);
    val = p_find(&vblock,"TOTBANDS");
    if (val->v_count == 0)
        inspc[2] = 0;
    else
        inspc[2] = IVAL(*val,curfil-1);

    /* Retrieve IDTYPE from the variable block.  If IDTYPE is null then assign
       it 0.  This will indicate that a DDR must exist on the tape along with
       the image data.  If a DDR is on the tape and a value was supplied, the
       value from the DDR will be used.
       -----------------------------------------------------------------------*/
    val = p_find(&vblock,"IDTYPE");
    if (val->v_count == 0)
        *idtype = 0;
    else
    {
        strcpy(type,SVAL(*val,0));
        if (strcmp(type,"BYTE") == 0)
            *idtype = EBYTE;
        else if (strcmp(type,"I*2") == 0)
            *idtype = EWORD;
        else if (strcmp(type,"I*4") == 0)
            *idtype = ELONG;
        else if (strcmp(type,"R*4") == 0)
            *idtype = EREAL;
        else
            *idtype = 0;
    }

    /* If the user wants to read all the bands from an image file there are two
       ways this can be done.  First, let NBANDS and BANDS both default to null.
       This indicates that all of the image files being read by TAPEIN are to 
       have all of their bands read.  The second way is for the user to supply 
       all the band numbers manually.  Notice that in a single run of TAPEIN 
       you cannot read some bands from one image and all bands from another 
       image without specifying every band number manually.
       -----------------------------------------------------------------------*/
    val  = p_find(&vblock, "NBANDS");
    val2 = p_find(&vblock, "BANDS");
    if ((val->v_count != 0) && (val2->v_count != 0))
    {
        *totbnd = IVAL(*val, curfil - 1);

        /* BANDS contains all of the band numbers for all image files being read
           from tape.  Retrieve just the band numbers for the current image
           -------------------------------------------------------------------*/
        skipf = 0;
        for (index = 0; index < curfil - 1; index++)
            skipf += IVAL(*val, index);
        for (index = 0; index < *totbnd; index++)
            bands[0][index] = IVAL(*val2, index + skipf);
        for (index = *totbnd; index < MAXBND + 1; index++)
            bands[0][index] = 0;
    }
    else
    {
        if ( ((val->v_count == 0) && (val2->v_count != 0)) ||
                ((val->v_count != 0) && (val2->v_count == 0))  )
            c_errmsg("Both BANDS and NBANDS must be specified",
                    "tapein-bands", &nonfatal);

        *totbnd = 0;
        for (index = 0; index < MAXBND + 1; index++)
            bands[0][index] = 0;
    }

    /* If all image files are being placed in one output image, recalculate 
       totbnd to include all input bands.  The total number of bands is the 
       count of BANDS and/or the sum of all entries in TOTBANDS.  Because we 
       need to know how many total bands there are in the output image when 
       copying the first image file, either BANDS or TOTBANDS must be specified 
       by the user.
       -----------------------------------------------------------------------*/
    if (*m2one)
    {
        val = p_find(&vblock,"BANDS");
        *totbnd = val->v_count;
        if (*totbnd == 0)
        {
            val = p_find(&vblock,"TOTBANDS");
            if (val->v_count != 0)
                for (index = 0; index < nmrfil; index++)
                    *totbnd += IVAL(*val, index);
            else
            {
                c_errmsg("BANDS and TOTBANDS cannot both be NULL",
                        "tapein-multi",&nonfatal);
                c_errmsg("Fatal error encountered", "tapein-fatal", &fatal);
            }
        }
    }

    /* Determine the which band of the output image is to be written.
       If m2one, either NBANDS or TOTBANDS must be specified by the user.
       ---------------------------------------------------------------------*/
    *curbnd = 1;
    if (*m2one)
    {
        val = p_find(&vblock, "NBANDS");
        if (val->v_count == 0)
        {
            *nbands = 0;
            val = p_find(&vblock, "TOTBANDS");
        }
        else
        {
            *nbands = IVAL(*val,curfil-1);
        }
        for (index = 1; index < curfil; index++)
            *curbnd += IVAL(*val,index-1);
    }
    else
    {
        *nbands = *totbnd;
    }

    /* Determine output image access
       --------------------------------*/
    if (curfil == 1)
        *access = IWRITE;
    else if (*m2one)
        *access = IUPDATE;
    else
        *access = IWRITE;

    /* Retrieve output image data type.  If the output type is supposed to be 
       the same as the input data type the value will be assigned later after 
       we look for a DDR.  Assign it 0 to indicate the type needs to be set.
       -----------------------------------------------------------------------*/
    val = p_find(&vblock,"ODTYPE");
    strcpy(type, SVAL(*val, 0));
    if (strcmp(type,"BYTE") == 0)
        *odtype = EBYTE;
    else if (strcmp(type,"I*2") == 0)
        *odtype = EWORD;
    else if (strcmp(type,"I*4") == 0)
        *odtype = ELONG;
    else if (strcmp(type,"R*4") == 0)
        *odtype = EREAL;
    else
        *odtype = 0;

    /* Retrieve the blocking factor (# of image lines/record)
       ---------------------------------------------------------*/
    val = p_find(&vblock,"BLKSIZ");
    *blksiz = IVAL(*val,0);

    /* Retrieve output line increment
       ---------------------------------*/
    val = p_find(&vblock,"LINEINC");
    *linc = IVAL(*val,0);

    /* Retrieve output sample increment
       -----------------------------------*/
    val = p_find(&vblock,"SAMPINC");
    *sinc = IVAL(*val,0);

    /* Retrieve the current tape volume (0, 1, 2, ...)
       --------------------------------------------------*/
    val = p_find(&vblock,"CURVOL");
    curvol = IVAL(*val,0);

    /* Retrieve the physical tape drive name
       ----------------------------------------*/
    val = p_find(&vblock,"DRIVE");
    drive[0] = SVAL(*val,0);

    /* Retrieve number of seconds between processing messages
       ---------------------------------------------------------*/
    val = p_find(&vblock,"$msgtim");
    *msgtim = IVAL(*val,0);

    /* Retrieve min/max flag
       ------------------------*/
    val = p_find(&vblock,"$minmax");
    if (strcmp(SVAL(*val,0),"YES") == 0)
        *minmax = TRUE;
    else
        *minmax = FALSE;

    /* Retrieve image compression flag
       -------------------------------*/
    val = p_find(&vblock,"$compflg");
    if (strcmp(SVAL(*val,0),"ON") == 0)
        *compflg = TRUE;
    else
        *compflg = FALSE;

    /* Get temporary ddr name
       -----------------------*/
    strcpy(tddrnam,"tapein");
    c_creat_name(tddrnam,"ddr");
}
