/*******************************************************************************
NAME:				BDR2STR

PURPOSE:      Copy the contents of the band record of the DDR file to 
	      a string.

PROGRAM HISTORY:
  Version       Date       Author           Request
  -------       ----       ------           -------
   1.0          01/93     K. Sheffield      original development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:	LAS

ALGORITHM DESCRIPTION:
Format the output string for each of the BDDR members

ALGORITHM REFERENCES:  The format of the string created is nearly identical to 
                       those output by DSPDDR.
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "protos.h"

#define BDRSTRSIZE 729         /* ~9 lines * 81 characters/line              */

void bdr2str
(
    struct BDDR *bddr,	        /* source of data	     		      */
    char *string,		/* pointer to output string                   */
    long *len,			/* output string length			      */
    long *band			/* band number                                */
)
{
   char *cptr;                  /* pointer to current location in string      */
   
   char vald[23];		/* buffer storage for validity flag value     */


   cptr = string;
   
   /* set up string for band information
      -------------------------------------*/

   sprintf(cptr,"\f\n   BAND NO:%-3d\n\n",*band);
   cptr += strlen(cptr);
   
   mmval(bddr->valid,vald);
   sprintf(cptr, "        MINIMUM:%-1.14E                       %-15s\n",
	   bddr->minval,vald);
   cptr += strlen(cptr);
   
   sprintf(cptr, "        MAXIMUM:%-1.14E                       %-15s\n\n",
	   bddr->maxval,vald);
   cptr += strlen(cptr);
   
   sprintf(cptr,"    DATA SOURCE:%-31s\n",bddr->source);
   cptr += strlen(cptr);
   
   sprintf(cptr,"    SENSOR TYPE:%-31s\n",bddr->instrument);
   cptr += strlen(cptr);
   
   sprintf(cptr,"CAPT. DIRECTION:%-63s\n",bddr->direction);
   cptr += strlen(cptr);
   
   sprintf(cptr,"           DATE:%-9s\n",bddr->date);
   cptr += strlen(cptr);
   
   sprintf(cptr,"           TIME:%-7s\n",bddr->time);
   cptr += strlen(cptr);
   
   *len = cptr - string + 1;
   
   
   return;
}

/*******************************************************************************
  NAME:				MMVAL
  
  PURPOSE:      To create the correct string according to the value of 
                the minmax valid flag
  
  ALGORITHM DESCRIPTION:
  Based on the value of input
  return the correct string
  
*******************************************************************************/

void mmval
(
    long input,				/* input validity flag numeric value  */
    char str[]				/* validity flag's character value    */
)
{
   switch (input)
   {
    case INVAL: sprintf (str,"Valid:INVALID");
      break;
    case VALID: sprintf (str,"Valid:VALID");
      break;
    case BOUND: sprintf (str,"Valid:BOUNDED");
      break;
    default: sprintf (str,"Valid:UNACCEPTABLE");
   } 
   return;
}
