/*******************************************************************************
FUNCTION: CHECK_TPREAD

PURPOSE:  Checks the status of a tape read.  If the status indicates an error,
	  appropriate action is taken.

ALGORITHM REFERENCES:	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "protos.h"

short check_tpread
(
    long status,         /* status code of tpread call        */
    long *unit,          /* tape unit number                  */
    unsigned char *buf,  /* buffer to read data into          */
    long nbyte,          /* number of bytes to read           */
    long mode           /* error mode flag                   */
)
{
long  one      =  1;    /* value of one                      */
long  nonfatal = 1;     /* Non-fatal error code              */
short not_eof  = TRUE;  /* flag to indicate EOF, EOV or EOT  */


/* Operate on the status passed in
----------------------------------*/
switch (status)
   {
   case TPREAD_READ:
      /* Read error encountered, skip to EOF
      --------------------------------------*/
      c_stoptm();
      c_errmsg("Error reading from tape - Skipping to end of file",
	       " ", &nonfatal);
      skip_file(unit,one);
      break;

   case TAPE_EOF:
      /* Unexpected EOF encountered
      -----------------------------*/
      c_errmsg("Unexpected EOF encountered - LAS image is not complete",
               " ", &nonfatal);
      not_eof = FALSE;
      break;

   case TAPE_EOV:			/* EOV encountered */
   case TAPE_EOT:			/* EOT encountered */
      /* Mount the next tape and continue process
      -------------------------------------------*/
      close_tape(unit,REWOFFL,0);
      mount_tape(unit,TREAD, FALSE);
      c_tpread(unit,buf,&nbyte,&mode);
      break;
   }

return(not_eof);
}
