/******************************************************************************

FUNCTION:	COPY_DDR

PURPOSE:	Copies the DDR record of the LAS image into a DDR file on
		tape.

ALGORITHM 
  Open the temporary DDR file 
      Set the key to read all records from the Label Services file
      Read a record from the Label Services file (DDR)
      Write the Label Services record to tape
  Output an EOF mark
  Close and delete the temp Label Services file containing the DDR info

ALGORITHM REFERENCES	none

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "pixman.h"
#include "labserv.h"
#include "tapeio.h"
#include "vll.h"
#include "sysdef.h"
#include "protos.h"

#define MAX_DDRSIZ 250

void copy_ddr
(
    long *unit,		/* tape unit number                  		     */
    long *insys,		/* Input system data is from		             */
    long *outsys,	        /* Output system data is put to		             */
    char *ddrnam	        /* input host ddr name		     	             */
)
{
   extern long datasize[NDTYPES];/* defines number bytes for each data type  */
   long fd;		/* file descriptor to the label services routine     */
   long clen;		/* Length of the character portion		     */
   long dlen;		/* Length of the data portion			     */
   long odtype;		/* Data type of portion of LS record	             */
   long nelem;		/* number of entities in ddr                         */
   long status;		/* function return status code       	             */
   long one = 1;        /* value of one                      	             */
   long zero = 0;       /* value of zero                      	             */
   long action =  2;	/* Delete the temp DDR file on close	             */
   long mode  = HANDLE; /* tape I/O outputs error messages   	             */
   long cvtflg = SYS_SET;/* Under/over flow flag - let system set            */
  
   char key[KEYL];	/* DDR key to read				     */
   char dtype[TYPL];	/* type of the data portion			     */
   char cbuf[MAX_DDRSIZ];/* Character buffer portion		             */
   
   unsigned char dbuf[MAX_DDRSIZ];/* Buffer to read data portion into        */
   
   /* Open the temporary DDR file 
      ------------------------------*/
   if (c_lsopen(&fd,ddrnam,&zero) != E_SUCC)
     c_lasmsg(TRUE, "tapeout-fatal", "Fatal error encountered");
   
   while(1)
   {
      /* Set the key to read all records from the Label Services file
	 ---------------------------------------------------------------*/
      strcpy(key," ");
      
      clen = MAX_DDRSIZ;
      dlen = MAX_DDRSIZ;
      
      /* Read a record from the Label Services file (DDR)
	 ---------------------------------------------------*/
      status = c_lsread(&fd,key,&clen,&dlen,cbuf,dbuf,dtype);
      if (status == E_EOF)
	break;
      else if (status != E_SUCC)
	c_lasmsg(TRUE, "tapeout-fatal", "Fatal error encountered");
      
      /* Convert the integer portion of the DDR
	 -----------------------------------------*/
      if (strcmp(dtype,"B") == 0)
	odtype = EBYTE;
      else if (strcmp(dtype,"I2") == 0)
	odtype = EWORD;
      else if (strcmp(dtype,"I4") == 0)
	odtype = ELONG;
      else if (strcmp(dtype,"R4") == 0)
	odtype = EREAL;
      else if (strcmp(dtype,"R8") == 0)
	odtype = EDOUBLE;
      else
	c_lasmsg(TRUE, "tapeout-type", "Unsupported data type");
      
      /* Convert the data portion of the DDR Label Services file
	 ----------------------------------------------------------*/
      nelem = dlen / datasize[odtype];
      status = c_pxsys(insys,outsys,dbuf,&odtype,&nelem,&zero,&zero,&cvtflg);
      
      /* Write the Label Services record to tape
	 ------------------------------------------*/
      status = c_tplswr(unit,key,&clen,&dlen,cbuf,(char *) dbuf,dtype);
      if (status != TAPE_SUCC)
	c_lasmsg(TRUE, "tapeout-fatal", "Fatal error encountered");
   }
   
   /* Output an EOF mark
      ---------------------*/
   if (c_tpweof(unit,&one,&mode) != TAPE_SUCC)
     c_lasmsg(TRUE, "tapeout-fatal", "Fatal error encountered");
   
   /* Close and delete the temp Label Services file containing the DDR info
      ----------------------------------------------------------------------*/
   if (c_lsclos(&fd,ddrnam,&action) != E_SUCC)
     c_lasmsg(FALSE, "tapeout-nonfatal", "Nonfatal error encountered");
   
   return;
}
