/*******************************************************************************
NAME:				ddr2str

PURPOSE:      Copy the contents of the general DDR fields to a string.

PROGRAM HISTORY:
  Version       Date       Author           Request
  -------       ----       ------           -------
   1.0          01/93     K. Sheffield      original development


PROJECT:	LAS

ALGORITHM DESCRIPTION:
The output string is formated for each of the fields of the DDR

ALGORITHM REFERENCES:  The format of the string created is nearly identical to 
                       those output by DSPDDR.
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "geompak.h"
#include "protos.h"


void ddr2str
(
    struct DDR *ddr,		/* pointer to integer data in record 1	      */
    char *string,		        /* pointer to output string                   */
    long *len,			/* output string length			      */
    char *tname                    /* TAE name of image                          */
)

{
   char *cptr;                  /* pointer to current location in string      */
   
   char proj_str[45];		/* buffer for the projection string	      */
   char vald[23];		/* buffer storage for validity flag value     */

   cptr = string;
   
   sprintf(cptr,"  IMAGE NAME:%-60s\n",tname);
   cptr += strlen(cptr);
   
   
   /* format lines for first screen
      --------------------------------*/
   
   switch (ddr->dtype)
   {
    case EBYTE: 
      sprintf(cptr,
	      "          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:BYTE\n",
			ddr->nl,ddr->ns,ddr->nbands);
      break;
    case EWORD:
      sprintf(cptr,
	      "          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:INTEGER*2\n",
		       ddr->nl,ddr->ns,ddr->nbands);
      break;
    case ELONG:
      sprintf(cptr,
	      "          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:INTEGER*4\n",
		       ddr->nl,ddr->ns,ddr->nbands);
      break;
    case EREAL:
      sprintf(cptr,
	      "          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:REAL*4\n",
		       ddr->nl,ddr->ns,ddr->nbands);
      break;
    default:
      sprintf(cptr,
	      "          NL:%-5d         NS:%-5d          NB:%-3d        DTYPE:%-d\n",
		    ddr->nl,ddr->ns,ddr->nbands,ddr->dtype);
      break;
   }
   cptr += strlen(cptr);
   sprintf(cptr,"  LAST MODIFIED:         DATE:%-11s  TIME:%-10sSYSTEM:%-11s\n",
	   ddr->last_used_date, ddr->last_used_time, ddr->system);
   cptr += strlen(cptr);
   
   /* generate strings for the projection parameters
      ----------------------------------------------*/

   pntval(ddr->valid[DDPCV],vald);
   c_prostr(&(ddr->proj_code),proj_str);
   sprintf(cptr,
	   "  %-9s%-*s%-*s%-15s\n","PROJ. CODE:",
	   strlen(proj_str),proj_str,(46 - strlen(proj_str))," ",vald);
   cptr += strlen(cptr);

   pntval(ddr->valid[DDZCV],vald);
   sprintf(cptr,
	   "   ZONE CODE:%-4d                                          %-15s\n",
	   ddr->zone_code,vald);
   cptr += strlen(cptr);
   
   pntval(ddr->valid[DDDCV],vald);
   sprintf(cptr,
	 "  DATUM CODE:%-2d                                            %-15s\n",
	   ddr->datum_code,vald);
   cptr += strlen(cptr);
   
   pntval(ddr->valid[DDPPV],vald);
   sprintf(cptr,
	   "  PROJ. PARM:                                              %-15s\n"
	   ,vald);
   cptr += strlen(cptr);

   sprintf(cptr,
	   "  A:   %1.14E   %1.14E   %1.14E\n",
	   ddr->proj_coef[0],ddr->proj_coef[1],ddr->proj_coef[2]);
   cptr += strlen(cptr);

   sprintf(cptr,
	   "  B:   %1.14E   %1.14E   %1.14E\n",
	   ddr->proj_coef[3],ddr->proj_coef[4],ddr->proj_coef[5]);
   cptr += strlen(cptr);

   sprintf(cptr,
	   "  C:   %1.14E   %1.14E   %1.14E\n",
	   ddr->proj_coef[6],ddr->proj_coef[7],ddr->proj_coef[8]);
   cptr += strlen(cptr);

   sprintf(cptr,
	   "  D:   %1.14E   %1.14E   %1.14E\n",
	   ddr->proj_coef[9],ddr->proj_coef[10],ddr->proj_coef[11]);
   cptr += strlen(cptr);

   sprintf(cptr,
	   "  E:   %1.14E   %1.14E   %1.14E\n",
	   ddr->proj_coef[12],ddr->proj_coef[13],ddr->proj_coef[14]);
   cptr += strlen(cptr);

   /* generate the strings for the corner points 
      ------------------------------------------*/

   pntval(ddr->valid[DDCCV],vald);
   sprintf(cptr,
	   " CORNER COOR:                                              %-15s\n"
	   ,vald);
   cptr += strlen(cptr);

   sprintf(cptr,"    ULcorner:%-1.14E   %-1.14E\n", 
	   ddr->upleft[0],ddr->upleft[1]);
   cptr += strlen(cptr);

   sprintf(cptr,"    URcorner:%-1.14E   %-1.14E\n",
	   ddr->upright[0],ddr->upright[1]);
   cptr += strlen(cptr);

   sprintf(cptr,"    LLcorner:%-1.14E   %-1.14E\n",
	   ddr->loleft[0],ddr->loleft[1]);
   cptr += strlen(cptr);

   sprintf(cptr,"    LRcorner:%-1.14E   %-1.14E\n",
	   ddr->loright[0],ddr->loright[1]);
   cptr += strlen(cptr);

   /* generate the strings for the projection distances
      -------------------------------------------------*/

   pntval(ddr->valid[DDPDV],vald);
   sprintf(cptr,
	   "  PROJ. DIST:%-1.14E   %1.14E   %-15s\n",
	   ddr->pdist_y,ddr->pdist_x,vald);
   cptr += strlen(cptr);

   pntval(ddr->valid[DDPUV],vald);
   sprintf(cptr,
	   " PROJ. UNITS:%-10s                                    %-15s\n",
	   ddr->proj_units,vald);
   cptr += strlen(cptr);

   pntval(ddr->valid[DDINCV],vald);
   sprintf(cptr,
	   "   INCREMENT:%-1.14E   %1.14E   %-15s\n",
	   ddr->line_inc,ddr->sample_inc,vald);
   cptr += strlen(cptr);

   sprintf(cptr,
	   " MASTER COOR:%-5d   %-5d\n",
	   ddr->master_line,ddr->master_sample);
   cptr += strlen(cptr);
   
   *len = cptr - string + 1;

   return;
}


/*******************************************************************************
  NAME:				PNTVAL
  
  PURPOSE:      To create the correct string according to the value of a 
                valid flag
  
  ALGORITHM DESCRIPTION:
  Based on the value of input
  return the correct string
  
*******************************************************************************/
void pntval
(
    long input,				/* input validity flag numeric value  */
    char str[]				/* validity flag's character value    */
)
{
   switch (input)
   {
    case INVAL: sprintf (str,"Valid:INVALID");
      break;
    case VALID: sprintf (str,"Valid:VALID");
      break;
    case UNKNOW: sprintf (str,"Valid:UNKNOWN");
      break;
    default: sprintf (str,"Valid:UNACCEPTABLE");
   } 
   return;
}
