#include <stdio.h>
#include "las.h"
#include "util.h"
#include "sysdef.h"
#include "protos.h"

#define MAX_VOL		15	/* maximum number of tape volumes            */

/******************************************************************************

FUNCTION:	GETPAR

PURPOSE:	Retrieve the input parameters from the TAE variable block

ALGORITHM 

ALGORITHM REFERENCES	none

*******************************************************************************/

void getpar
(
    long *insys,			/* Input system				      */
    long *outsys,			/* Output system			      */
    char taein[][CMLEN],		/* Input LAS file names			      */
    char hosin[][CMLEN],		/* Input Host file names		      */
    long window[][4],		/* input window specification		      */
    long bands[][MAXBND+1],		/* list of input bands to be copied	      */
    long nbands[],			/* # bands in each input image		      */
    long *totbnd,			/* total number of bands to be copied	      */
    long *odtype,			/* data type of output image		      */
    long *blksiz,			/* number of lines in output block	      */
    long *linc,			/* line increment for subsample		      */
    long *sinc,			/* sample increment for subsample	      */
    long *nmrimg,			/* number of input images 		      */
    long *skip,			/* # of files to skip prior to copy	      */
    char *bform,			/* band format of output image		      */
    char density[],         /* tape density in BPI               	      */
    long *hp_flag,			/* Flag for HP computer system		      */
    long *ad_flag,			/* Flag for ASCII DDR                         */
    long *aft_flag,			/* Placement Flag for ASCII DDR               */
    long *wrteov_flag,		/* Flag for writing End-of-Volume             */
    long *msgtim,			/* elapsed time between display msg	      */
    long qflag[MAXIMG][2]	/* windowing/banding option flags             */
)
{
extern struct PARBLK vblock;	/* TAE parameter block               	      */
extern int   curvol;		/* current tape volume being process 	      */
extern long  tpfil;		/* # of files on current tape        	      */
extern short ddr_flag;		/* DDR header flag                            */
extern char  comment[][CMLEN];	/* tape mount message to the operator	      */
extern char  *drive[1];         /* physical tape drive name          	      */
extern char  label_file[];      /* File containing tape label info            */
extern char  multivol[4];	/* flag as to how to split tape vol. 	      */

long index;			/* loop index				      */
long status;			/* function return status code		      */
long std_out;			/* fortran standard out LUN		      */
long same;			/* flag to check input image windows	      */
long vol;			/* Volume continuation number		      */
long zero = 0;			/* Value of zero			      */
long sysflg = SETN;		/* Set system value from system string	      */
long fatal = -1;		/* Fatal error code			      */
long nonfatal = 1;		/* Non-fatal error code			      */
long imglim = MAXIMG;		/* maximum number of input images	      */

static char system[15];		/* Current system			      */
static char type[5];		/* Input/Output type string		      */

struct VARIABLE	*val;		/* pointer to a TAE variable		      */

/* Initialize TAE and LAS
-------------------------*/
c_inlas(&vblock,&std_out);

/* Retrieve tape mount message
------------------------------*/
val = p_find(&vblock,"COMMENT");
for (index = 0; index < val->v_count; index++)
   strcpy(comment[index],SVAL(*val,index));

for (vol = 1, index = val->v_count; index < MAX_VOL; index++, vol++)
    sprintf(comment[index],"%s%s%d",comment[val->v_count - 1],
	    " -- CONTINUATION ",vol);

/* Retrieve the current system.
-------------------------------*/
status = c_getsys(system);
status = c_sysset(system,insys,&sysflg);

/* Retrieve system the tape was generated on.
   Set a flag if the input system was from an HP.
-------------------------------------------------*/
val = p_find(&vblock,"OUTSYS");
strcpy(system,SVAL(*val,0));
if (strcmp(system,"IEEE") == 0)
    *outsys = SYS_IEEE_STD;
else if (strcmp(system,"HP") == 0)
    {
    *outsys = SYS_HP_MPE;
    *hp_flag = TRUE;
    }
else
    *outsys = *insys;

/* Retrieve tape density in BPI
-------------------------------*/
val = p_find(&vblock,"TAPEDENS");
strcpy(density,SVAL(*val,0));

/* Retrieve number of file to be skipped
----------------------------------------*/
val = p_find(&vblock,"SKIPNUM");
*skip = IVAL(*val,0);

/* Retrieve input band format
-----------------------------*/
val = p_find(&vblock,"BFORM");
strcpy(bform,SVAL(*val,0));

/* Retrieve output image data type
----------------------------------*/
val = p_find(&vblock,"ODTYPE");
strcpy(type,SVAL(*val,0));
if (strcmp(type,"BYTE") == 0)
    *odtype = 1;
else if (strcmp(type,"I*2") == 0)
    *odtype = 2;
else if (strcmp(type,"I*4") == 0)
    *odtype = 3;
else if (strcmp(type,"R*4") == 0)
    *odtype = 4;
else
    *odtype = 0;

/* Retrieve number of lines in output block
-------------------------------------------*/
val = p_find(&vblock,"BLKSIZ");
*blksiz = IVAL(*val,0);

/* Retrieve output line increment
---------------------------------*/
val = p_find(&vblock,"LINEINC");
*linc = IVAL(*val,0);

/* Retrieve output sample increment
------------------------------------*/
val = p_find(&vblock,"SAMPINC");
*sinc = IVAL(*val,0);

/* Retrieve how to split multi-volume
--------------------------------------*/
val = p_find(&vblock,"MVOLFLG");
strcpy(multivol,SVAL(*val,0));

/* Retrieve DDR flag
--------------------*/
val = p_find(&vblock,"DDRFLG");
if (strcmp(SVAL(*val,0),"YES") == 0)
    ddr_flag = TRUE;
else
    ddr_flag = FALSE;

/* Retrieve current tape volume
-------------------------------*/
val = p_find(&vblock,"CURVOL");
curvol = IVAL(*val,0);

/* Retrieve physical tape drive name
------------------------------------*/
val = p_find(&vblock,"DRIVE");
drive[0] = SVAL(*val,0);

/* Retrieve ASCII DDR Flag
------------------------------------*/
val = p_find(&vblock,"ASCFLG");
if(strcmp("YES",SVAL(*val,0)) == 0)
  *ad_flag = 1;
else
  *ad_flag = 0;

/* Retrieve ASCII DDR Placement Flag
------------------------------------*/
val = p_find(&vblock,"AFTFLG");
if(strcmp("YES",SVAL(*val,0)) == 0)
  *aft_flag = 1;                     /* place the ASCII ddr after the image */
else
  *aft_flag = 0;                     /* place the ASCII ddr before the image */

if(*ad_flag == 0)
  *aft_flag = 0;

/* Retrieve Write End of Volume Flag
------------------------------------*/
val = p_find(&vblock,"WRTEOV");
if(strcmp("YES",SVAL(*val,0)) == 0)
  *wrteov_flag = 1;                 /* write an End of Volume after the image */
else
  *wrteov_flag = 0;    

/* Retrieve number of files on current tape
-------------------------------------------*/
val = p_find(&vblock,"TPFIL");
tpfil = IVAL(*val,0);

/* Retrieve number of seconds between processing messages
---------------------------------------------------------*/
val = p_find(&vblock,"$msgtim");
*msgtim = IVAL(*val,0);

/* Retrieve the input image specifications including
   LAS image name(s), window and bands
----------------------------------------------------*/
status = c_getin(&vblock,"IN",&zero,taein,hosin,nmrimg,window,bands,nbands,
		 totbnd,&imglim,&same,qflag);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","tapeout-fatal",
	    &fatal);

/* Check to make sure that multiple window specified are all the same size
--------------------------------------------------------------------------*/
if (same == 0)
    {
    c_errmsg("Input images do not have same window sizes","tapeout-wind",
	     &nonfatal);
    c_errmsg("Fatal error encountered","tapeout-fatal", &fatal);
    }

/* Set the output data type to the input data type if it was not specified
--------------------------------------------------------------------------*/
if (*odtype == 0)
   {
   status = c_gettyp(hosin,nmrimg,odtype);
   if (status != E_SUCC)
       c_errmsg("Fatal error encountered","tapeout-fatal",&fatal);
   }

/* Retrieve the name of the file that contains the tape label information
-------------------------------------------------------------------------*/
val = p_find(&vblock,"PARFILE");
if (val->v_count == 0)
   strcpy(label_file, "");
else
   strcpy(label_file, SVAL(*val,0));

return;
}
