/*******************************************************************************
NAME:    PRINT_LABEL

PURPOSE: Generate the last tape label stored in the file passed in parm_file.
         This function should be used by any LAS function that automatically
         generates tape labels (copy it from INITLBL).

	 The following parameters must exist in the file and they must be
	 strings:  code, create, density, descrpt, drive, ops, project, name,
	 requestor, sequence, tapeid, hostname, time, and printer.  The 
	 number of months to retain the tape must also exist in the file, 
	 but it is of type integer.


PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Development
  1.1    J.Fenno          03/92  EDC   Added code so that unique file names are
				       used allowing tapeout to be run multiple
				       times from the same working directory.
				       PRF Z0810.
  1.2    J.Matzner	  08/92  EDC   Modified purge date to be an integer of
				       the number of months, from 1 to 60, 
				       before the scratch date of the tape.
				       Added code to check the return of 
				       c_tapelabel.c for either a success or a
				       failure, and that value is returned to 
			               the calling program.
  1.3    D.Etrheim        09/92  EDC   Modified to pass the OPS parameter to
				       c_tapelabel.  Also modified to let
				       c_tapelabel handle remotely mounted
				       tapes.
  1.4    D.Etrheim        10/92  EDC   Modifed thebuf to be an array of 
                                       pointers for call to c_tapelabel

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS		                

*******************************************************************************/
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "tapeio.h"
#include "protos.h"

extern struct PARBLK vblock;       /* TAE Parameter block		      */

int print_label
(
    char parm_file[]     /* Name of the file that contains the label info     */
)
{
struct VARIABLE *var;
char   *thebuf[11];    /* buffer of information: LIBRARY FORMAT            */
                          /* [ 0] code (code for the tape)                */
                          /* [ 1] density (density of the tape)           */
                          /* [ 2] description (description of the tape)   */
                          /* [ 3] drive (tape drive used)                 */
                          /* [ 4] project
                                      (project the tape was created for)      */
                          /* [ 5] requestor (initials of the tape creator)*/
                          /* [ 6] sequence (sequence number of the tape)  */
                          /* [ 7] tapeid (tape ID number)                 */
                          /* [ 8] error flag                              */
                          /* [ 9] unused                                  */
                          /* [10] unused                                  */
char   string[CMLEN];    /* Temporary working string                          */
char   tmp_filename[CMLEN];/* parm_file with the .ppf extension               */
char   tmp_str[15][CMLEN];/* String used for reading from the parameter file  */
char   type[5];          /* Type of the parameter returned by c_getparm       */
char   tae_name[CMLEN];	 /* TAE name of a file				      */

double dbl_val[1];       /* Array of double values passed back from c_getparm */

long   i;                /* Loop control                                      */
long   lng_val[1];       /* Array of long values passed back from c_getparm   */
long   nonfatal =  1;    /* Nonfatal error flag                               */
long   size;             /* Number of elements returned by c_getparm          */
long   retain;   	 /* months to retain after current date              */
                         /* 1-60 months OK, 0 means permanent, > 60 set to 60*/
long   printflg = 1;	 /* print flag 0 = NO 1 =YES			      */
long   formflg = 1;	 /* file format flag (0 = FREE, 1 = STANDARD)	      */

FILE   *file_ptr;        /* Temp file which will contain each label           */

/* Allocate space */
for (i = 0; i < 10; i++)
    thebuf[i] = (char *)malloc(52);

/* Add the ppf extension to parm_file
-----------------------------------------------*/
sprintf(tmp_filename, "%s.ppf", parm_file);

/* If the file parm_file does not exist simply return
-----------------------------------------------------*/
if ((file_ptr = fopen(tmp_filename, "r")) == NULL)
   return(E_SUCC);
else
   fclose(file_ptr);  

/* Make sure we are at EROS Data Center
---------------------------------------*/
var = p_find(&vblock,"$FNAME");
if (strcmp("EROS Data Center",SVAL(*var,0)) != 0)
   printflg = FALSE;
else 
   printflg = TRUE;

/* Write the information for the label to print_file
----------------------------------------------------*/
size = c_getparm(parm_file, "CODE", tmp_str, type, dbl_val, lng_val, 1);
if ((size != 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving CODE", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
strcpy(thebuf[0], tmp_str[0]);

size = c_getparm(parm_file, "DENSITY", tmp_str, type, dbl_val, lng_val, 0);
if ((size != 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving DENSITY", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
strcpy(thebuf[1], tmp_str[0]);

size = c_getparm(parm_file, "DESCRPT", tmp_str, type, dbl_val, lng_val, 1);
if ((size < 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving DESCRPT", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
string[0] = '\0';
for (i = 0; i < size; i++)
   {
   if (i > 0)
      strcat(string, " ");
   strcat(string, tmp_str[i]);
   }
string[51] = '\0';
strcpy(thebuf[2], string);

/* Get the drive.  Remotely mounted tapes, in the from sys@1, will be 
handled in c_tapelabel.
----------------------------------------------------------------------*/
size = c_getparm(parm_file, "DRIVE", tmp_str, type, dbl_val, lng_val, 0);
if ((size != 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving DRIVE", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
strcpy(thebuf[3], tmp_str[0]);

size = c_getparm(parm_file, "PROJECT", tmp_str, type, dbl_val, lng_val, 1);
if ((size != 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving PROJECT", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
strcpy(thebuf[4], tmp_str[0]);

size = c_getparm(parm_file, "NAME", tmp_str, type, dbl_val, lng_val, 1);
if ((size != 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving NAME", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
strcpy(thebuf[5], tmp_str[0]);

size = c_getparm(parm_file, "NMONTHS", tmp_str, type, dbl_val, lng_val, 1);
if ((size != 1) || (strcmp(type, "I") != 0))
   {
   c_errmsg("Error retrieving NMONTHS", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
retain = lng_val[0];

size = c_getparm(parm_file, "SEQUENCE", tmp_str, type, dbl_val, lng_val, 0);
if ((size != 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving SEQUENCE", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
strcpy(thebuf[6], tmp_str[0]);

size = c_getparm(parm_file, "TAPEID", tmp_str, type, dbl_val, lng_val, 0);
if ((size != 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving TAPEID", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
i = (long) strlen(tmp_str[0]);
c_up2low(tmp_str[0], &i);
if (strncmp(tmp_str[0],"sta",3) == 0)
   {
   c_errmsg("Labels not printed for STA tapes","makelbl-none",&nonfatal);
   return(E_SUCC);
   }
strcpy(thebuf[7], tmp_str[0]);

size = c_getparm(parm_file, "OPS", tmp_str, type, dbl_val, lng_val, 0);
if ((size != 1) || (strcmp(type, "S") != 0))
   {
   c_errmsg("Error retrieving OPS", "makelbl-retrieve", &nonfatal);
   return(E_FAIL);
   }
strcpy(thebuf[8], tmp_str[0]);

/* Print the buffer 
-------------------*/
if (c_tapelabel(thebuf,retain,printflg,formflg,tae_name) != E_SUCC) 
    {
    c_errmsg("Error returned from c_tapelabel","c_tplbl-err",&nonfatal);
    return(E_FAIL);
    }
else if (printflg)
    c_errmsg("Tape label printed", "informational", &nonfatal);

return(E_SUCC);
}
