/*******************************************************************************
NAME      		  CREATEC 

PURPOSE  Generate the coefficients from mean values.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.McKinney        Sept 1988     Written for LAS 5.0 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
    CREATEC must be run under TAE.

PROJECT          	 LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
For the mean brightness soil, dark soil, green vegetation, and
dry vegetation calculate the brightness, greenness and wetness
coefficients.
ALGORITHM REFERENCES     none
*******************************************************************************/


#include <stdio.h>
#include <math.h>
#include "las.h"
#include "protos.h"

void createc
(
    long totbnd,			/* total number of bands to be processed     */
    long brght,			/* flag for generation of brightness band    */
    long grn,			/* flag for generation of greenness band     */
    long wt,			/* flag for generation of wetness band       */
    float *bright,		/* brightness coefficients array             */
    float *green,	        /* greenness coefficients array		     */
    float *wet,	        	/* wetness coefficients array		     */
    float *brsoil,		/* mean bright soil array		     */
    float *dksoil,		/* mean dark soil array  		     */
    float *grnveg,		/* mean green vegetation array		     */
    float *dryveg		/* mean dry vegetation array		     */
)

{

long    i;			/* counter                      	     */

float   cbright[6];		/* temporary brightness buffer		     */
float   ccbright=0;		/* temorary brightness variable		     */
float   cgreen[6];		/* temporary greenness buffer		     */
float   ccgreen=0;		/* temporary greennesss variable	     */
float   dcgreen=0;		/* temporary greennesss variable	     */
float   cwet[6];		/* temporary wetness buffer		     */
float   ccwet=0;		/* temporary wetness variable		     */
float   dcwet=0;		/* temporary wetness variable		     */
float   ecwet=0;		/* temporary wetness variable		     */
 

/* Main processing loop
-----------------------*/
/*
  Compute the coefficients
*/
/*******   Calculate brightness     ***********/
	if (brght == 1)
	{
	for (i=0;i<totbnd;i++)
	   {
	   cbright[i] = brsoil[i] - dksoil[i];
	   ccbright += (cbright[i] * cbright[i]);
	   }
	ccbright = sqrt(ccbright);
	for (i=0;i<totbnd;i++)
	   bright[i] = cbright[i] / ccbright;
	}
/*******   Calculate greenness     ***********/
	if (grn == 1)
	{
	for (i=0;i<totbnd;i++)
	   {
	   cgreen[i] = grnveg[i] - dksoil[i];
	   dcgreen += (cgreen[i] * bright[i]);
	   }
	for (i=0;i<totbnd;i++)
	   {
	   cgreen[i] -= (dcgreen * bright[i]);
	   ccgreen += (cgreen[i] * cgreen[i]);
	   }
	ccgreen = sqrt(ccgreen);
	for (i=0;i<totbnd;i++)
	   green[i] = cgreen[i] / ccgreen;
	}
/*******   Calculate wetness     ***********/
	if (wt == 1)
	{
	for (i=0;i<totbnd;i++)
	   {
	   cwet[i] = dryveg[i] - brsoil[i];
           ccwet += (cwet[i] * bright[i]);
           dcwet += (cwet[i] * green[i]);
	   }
	for (i=0;i<totbnd;i++)
	   {
	   cwet[i] -= ((ccwet * bright[i]) + (dcwet * green[i]));
	   ecwet += (cwet[i] * cwet[i]);
	   }
	ecwet = sqrt(ecwet);
	for (i=0;i<totbnd;i++)
	   wet[i] = cwet[i] / ecwet;
	}
return;
}
