/***************************************************************************

NAME      		  TASSELCP 

PURPOSE   Generate a tasseled image from either MSS or TM images.   

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      Sept 1988       Written for LAS 5.0
B.Davis      	FEB  1990       Made unique message keys for the errmsgs in
				getpar.c, and updated the pdf with all of the
				correct error messages, their keys, and
				descriptions.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   TASSELCP must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)

   The LAS program is initialized.  Next the parameters are
   retrieved thru GETPAR.  The main processing routine is called
   to apply the matrix algebra.  Finally, the
   history file is updated and the routine completed.

ALGORITHM REFERENCES     

   None.
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/

struct FDESC * fdesc[MAXIMG+1];  /* image file descriptors used for image I/O */
struct GDESC * gdesc;		     /* group descriptor used for image I/O       */
long offset1[MAXIMG*MAXBND+1];  /* offsets into "thebuf" for input bands     */
long offset2[MAXBND+1];         /* offsets into "thebuf" for output bands    */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

int main
(void)
{
struct PARBLK	vblock;         /* TAE parameter block    		     */

static long bands[MAXIMG][MAXBND + 1];
                                /* band specification for first image        */
long brght;			/* flag for generation of brightness band    */
long fyle; 			/* flag for output to file		     */
long grn;			/* flag for generation of greenness band     */
long i;				/* counter for orthogonality check           */
long lnpr;			/* flag for output to line printer           */
long msgtim;			/* elpased time between display messages     */
long nmrimg;			/* number of input images for image          */
long nonfatal = 1;		/* nonfatal error flag			     */
long nbands[MAXIMG];		/* number of bands per input image           */
long odtype;			/* odtype for processing                     */
long one = 1;			/* number one				     */
long std_out;			/* logical unit # for standard output        */
long term;			/* flag for output to terminal		     */
long totbnd;			/* total number of bands in first image      */
long wt;			/* flag for generation of wetness band       */
long window[MAXIMG][4];		/* window specification for first image      */
long zero = 0;			/* integer value of zero		     */

char    outmsg[CMLEN];		/* output message to the terminal	     */
char    filnm[CMLEN];		/* output file name for FILE option          */
char	hosin[MAXIMG][CMLEN];	/* buffer storage        	   	     */
char    hosout[1][CMLEN];          /* buffer storage        		     */
char    minmax[4];	        /* minmax flag buffer 			     */
char    outfil[CMLEN];		/* output file name passed to complt	     */
char    sub[7];	        	/* processing option			     */
char	taein[MAXIMG][CMLEN];	/* buffer storage       		     */
char    taeout[1][CMLEN];          /* buffer storage       		     */

float   bright[6];		/* brightness coefficients array	     */
float   green[6];		/* greenness coefficients array		     */
float   wet[6];			/* wetness coefficients array		     */
float   brsoil[6];		/* mean bright soil array		     */
float   dksoil[6];		/* mean dark soil array         	     */
float   grnveg[6];		/* mean green vegatation array		     */
float   dryveg[6];		/* mean dry vegatation array		     */
float   ortho; 			/* orthogonality check			     */
float   orthobg;		/* orthogonality for brightness and greenness*/
float   orthobw;		/* orthogonality for brightness and wetness  */
float   orthogw;		/* orthogonality for greenness and wetness   */

FILE    *fp = NULL;		/* file pointer to print file		     */

c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/*
       get user-specified parameters and TAE globals
*/
getpar(&vblock, taein, hosin, taeout, hosout,
       &nmrimg, window, bands, nbands, &totbnd, &msgtim, 
       minmax, &term, &fyle, &lnpr, filnm, outfil,
       sub, &brght, &grn, &wt, bright, green, wet,
       brsoil, dksoil, grnveg, dryveg, &odtype);

/*
   set up output print file
*/
if ((lnpr == 1) || (fyle == 1))
	fp = fopen(filnm,"w");

/*
   perform the creation of the coefficients 
*/
if (strcmp(sub,"CREATE") == 0)
{
createc(totbnd, brght, grn, wt, bright, green, wet,
 	brsoil, dksoil, grnveg, dryveg); 

/*
  print the mean values to the terminal
*/

if (term == 1)
  {
  sprintf(outmsg,
  "\f               BRIGHT SOIL, DARK SOIL, GREEN VEG, AND DRY VEG\n");
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
"               BAND 1     BAND 2     BAND 3     BAND 4     BAND 5     BAND 6");
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
  "BRIGHT SOIL %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f",
  brsoil[0],brsoil[1],brsoil[2],brsoil[3],brsoil[4],brsoil[5]);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
  "DARK SOIL   %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f",
  dksoil[0],dksoil[1],dksoil[2],dksoil[3],dksoil[4],dksoil[5]);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
  "GREEN VEG   %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f",
  grnveg[0],grnveg[1],grnveg[2],grnveg[3],grnveg[4],grnveg[5]);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
  "DRY VEG     %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f",
  dryveg[0],dryveg[1],dryveg[2],dryveg[3],dryveg[4],dryveg[5]);
  c_pterm(outmsg,&zero);
  }

/* 
  print the mean values to the printer or file
*/

if ((lnpr == 1) || (fyle == 1))
  {
  fprintf(fp,
  "\f               BRIGHT SOIL, DARK SOIL, GREEN VEG, AND DRY VEG\n");
  fprintf(fp,
"               BAND 1     BAND 2     BAND 3     BAND 4     BAND 5     BAND 6");
  fprintf(fp,"\n");
  fprintf(fp,
  "BRIGHT SOIL %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
  brsoil[0],brsoil[1],brsoil[2],brsoil[3],brsoil[4],brsoil[5]);
  fprintf(fp,
  "DARK SOIL   %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
  dksoil[0],dksoil[1],dksoil[2],dksoil[3],dksoil[4],dksoil[5]);
  fprintf(fp,
  "GREEN VEG   %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
  grnveg[0],grnveg[1],grnveg[2],grnveg[3],grnveg[4],grnveg[5]);
  fprintf(fp,
  "DRY VEG     %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
  dryveg[0],dryveg[1],dryveg[2],dryveg[3],dryveg[4],dryveg[5]);
  }
}

/*
  generate orthogonality 
*/
  orthobg=orthobw=orthogw=ortho=0;
  for(i=0;i<totbnd;i++)
     {
     orthobg += bright[i] * green[i];
     orthobw += bright[i] * wet[i];
     orthogw += green[i] * wet[i];
     }
     ortho = orthobg + orthobw + orthogw;

/*
  print the coefficients to the terminal
*/

if (term == 1)
  {
  sprintf(outmsg,
  "\f               BRIGHTNESS, GREENNESS, and WETNESS COEFFICIENTS\n");
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
"               BAND 1     BAND 2     BAND 3     BAND 4     BAND 5     BAND 6");
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
  "BRIGHTNESS  %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f",
  bright[0],bright[1],bright[2],bright[3],bright[4],bright[5]);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
  "GREENNESS   %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f",
  green[0],green[1],green[2],green[3],green[4],green[5]);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,
  "WETNESS     %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
  wet[0],wet[1],wet[2],wet[3],wet[4],wet[5]);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,"ORTHOGONALITY BRIGHTNESS AND GREENNESS %15.10f\n",orthobg);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,"ORTHOGONALITY BRIGHTNESS AND WETNESS   %15.10f\n",orthobw);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,"ORTHOGONALITY GREENNESS AND WETNESS    %15.10f\n",orthogw);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,"ORTHOGONALITY CHECK %15.10f\n",ortho);
  c_pterm(outmsg,&zero);
  sprintf(outmsg,"\f");
  c_pterm(outmsg,&zero);
  }

/*
  print the coefficients to the printer or file
*/

if ((lnpr == 1) || (fyle == 1))
  {
  fprintf(fp,
  "\f               BRIGHTNESS, GREENNESS, and WETNESS COEFFICIENTS\n");
  fprintf(fp,
"               BAND 1     BAND 2     BAND 3     BAND 4     BAND 5     BAND 6");
  fprintf(fp,"\n");
  fprintf(fp,
  "BRIGHTNESS  %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
  bright[0],bright[1],bright[2],bright[3],bright[4],bright[5]);
  fprintf(fp,
  "GREENNESS   %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
  green[0],green[1],green[2],green[3],green[4],green[5]);
  fprintf(fp,
  "WETNESS     %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f\n",
  wet[0],wet[1],wet[2],wet[3],wet[4],wet[5]);
  fprintf(fp,"ORTHOGONALITY BRIGHTNESS AND GREENNESS %15.10f\n",orthobg);
  fprintf(fp,"ORTHOGONALITY BRIGHTNESS AND WETNESS   %15.10f\n",orthobw);
  fprintf(fp,"ORTHOGONALITY GREENNESS AND WETNESS    %15.10f\n",orthogw);
  fprintf(fp,"ORTHOGONALITY CHECK %15.10f\n",ortho);
  }

/*
   perform the tasselcap operation 
*/
tassel(&vblock,hosin,hosout,nmrimg,bands,nbands,
	window,msgtim,minmax,totbnd,odtype,
        bright, green, wet, brght, grn, wt);

/*
   create associated history file
*/
if (c_hstout(&vblock, hosout[0], hosin , &nmrimg, " ") != E_SUCC)
   c_errmsg("Nonfatal error encountered",
		"tasselcp-warn",&nonfatal);

/*
   inform user of successful completion
*/
c_complt("TASSELCP", taeout, &one);

if ((lnpr == 1) || (fyle == 1))
     fclose(fp);

return EXIT_SUCCESS;
     
}
