/*******************************************************************************
NAME      		  TESTGEN 


PURPOSE  Allow the user to create test images of specified size and
         characteristics.  The image may be of one of four types:
         UNIFORM, RANDOM, GAUSSIAN, WEDGE.

PROGRAM HISTORY
VERSION     PROGRAMMER	       DATE		REASON
-------     ----------	       ----		------
  1.0       J.HAGEDORN      Nov. 1983       initial LAS programmer
  4.0       J.LAVERGNE      Sept 1987       last LAS 4.0 programmer
  5.0       R.MCKINNEY      May  1988       converted to LAS 5.0
  5.0	    R.MCKINNEY      Dec  1988	    Updated to close all files
  5.1	    B.DAVIS         Mar  1990	    changed pxfill to c_pxfill
  5.2	    D.VanderZee     Sep  1990	    fixed random and gausian image 
					    generators to work with seed values
  5.3       D. Etrheim      Oct  1991       ADP #5621 Fix for fractional step 
					sizes for real images.  Correct NLEVELS
					validation check for byte images.  
					Initialize some fields of the DDR.
  5.4       D. Etrheim      Sep 1992        Change parameters STDDEV and 
					PIXVAL1-6 to STDEV and PIXVAL for 
					standards
  5.5       D. Etrheim      Dec  1992       Fixed to produce same gausian image
                            across all UNIX platforms.  The VAX will produce a
                            different gausian image as it has a different
                            random number generator.  If it is deemed important
                            that they be the same, the algorithm would need to
                            be changed and the LAS support c_randnm would need
                            to be used.
					
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   TESTGEN must be run under TAE.

PROJECT:  	         LAS		                

LANGUAGE:		  C	

ALGORITHM:
    Perform standard initialization and retrieval of user-specified 
      parameters.
    Open and group the output image for sequential processing.
    Allocat space for the image I/O buffer.
    Perform initialization for the processing message and min/max arrays
    Perform the appropriate option based upon the subcommand
      entered. 
    Create associated min/max and history files for the output image 
    Print a completion message.

ALGORITHM REFERENCES:     none

TESTING complete on SUN UNIX (unix), PN9050 UNIX (gould), and
    VAX VMS (vms).

*******************************************************************************/
#include <stdio.h>
#include <desc.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "testgen.h"

#define curlin curent

/*
   The following variables must be declared global for image I/O routines
*/

int  fdesc[MAXIMG + 1];         /* image file descriptors used for image I/O  */
int  gdesc;		        /* group descriptor used for image I/O        */
long offset1[MAXBND+1];     	/* offset into the image i/o buffer           */
extern long curent;		/* current processing line	     */
extern long curcat;		/* current processing band		      */

void main
(
  void
)
{
struct PARBLK	vblock;         /* TAE parameter block    		      */

float ltgrey;                   /* gray level for left or top of wedge image  */
float maxpix;                   /* maximum pixel value for random image       */
float meanval;                  /* mean pixel value for gaussian image        */
float minpix;                   /* minimum pixel value for random image       */
float pixval[256];              /* array of up to 256 uniform values          */
float rbgrey;                   /* gray level for right or bottom of wedge    */
float sigma;                    /* standard deviation for gausian             */

long bands;                     /* band specification for image               */
long dflag;                     /* ddr update option                          */
long nonfatal = 1;		/* nonfatal error flag                        */
long msgtim;                    /* elapsed time between display messages      */
long nlevel;                    /* number of gray levels in wedge             */
long nl;                        /* number of lines in output image            */
long ns;                        /* number of pixels per line in output image  */
long numb;                      /* integers for testing method                */
long one = 1;                   /* value of one                               */
long seedval;                   /* seed for random image generation           */
long std_out;                   /* logical unit # for standard output         */
long zero = 0;                  /* integer value of zero                      */
long odtype;                    /* odtype for processing                      */

char direct[2];                 /* direction of the wedge                     */
char hosout[CMLEN];            /* hostname of output image                    */
char method[4];                 /* method of use                              */
char minmax[4];                 /* minmax flag buffer			      */
char taeout[CMLEN];             /* TAE name of output image                   */

/*
        initialize las and tae
*/
c_inlas(&vblock, &std_out); 

/* 
       get user-specified parameters and TAE globals
*/
getpar(&vblock, &msgtim, taeout, hosout, &nl, 
       &ns, minmax, &dflag, &bands, 
       pixval, &maxpix, &minpix, &seedval, &meanval, 
       &sigma, direct, &ltgrey, &rbgrey, &nlevel, 
       method, &odtype);
/*
       create the image depending upon the option
*/
	     if (strcmp(method,"UNI") == 0)
		numb = 1;
	else if (strcmp(method,"RAN") == 0)
		numb = 2;
	else if (strcmp(method,"GAU") == 0)
		numb = 3;
	else if (strcmp(method,"WED") == 0)
		numb = 4;
	switch (numb)
	{

	case 1:

/*	CASE FOR UNIFORM */

	uniform(&vblock,hosout,nl,ns,bands,odtype,
                pixval,msgtim,minmax,dflag);
	break;
	case 2:

/*	CASE FOR RANDOM */

	randgen(&vblock,hosout,nl,ns,bands,odtype,
		maxpix,minpix,&seedval,msgtim,minmax,dflag);
	break;
	case 3:

/*	CASE FOR GAUSSIAN */

	gausian(&vblock,hosout,nl,ns,bands,odtype,
		meanval,sigma,seedval,msgtim,minmax,dflag);
	break;
	case 4:

/*	CASE FOR WEDGE */

	wedge(&vblock,hosout,nl,ns,bands,odtype,
		direct,ltgrey,rbgrey,nlevel,msgtim,minmax,dflag);
	break;
    }

/*
   create associated history file
*/
if (c_hstout(&vblock,hosout," ",&zero," ") != E_SUCC)
   c_errmsg("Nonfatal error encountered", "testgen-warn",&nonfatal);

/*
   inform user of successful completion
*/
c_complt("TESTGEN",taeout,&one);
}
