/******************************************************************************
NAME:     		  UNIFORM


PURPOSE:     UNIFORM allows testgen to create test images of specified
             size with uniform pixal values.

PROGRAM HISTORY:
VERSION     PROGRAMMER 	     DATE		REASON
-------     ----------	     ----		------
  1.0       J.HAGEDORN      Nov. 1983       initial LAS programmer
  4.0       J.LAVERGNE      Sept 1987       last LAS 4.0 programmer
  5.0       R.MCKINNEY      May  1988       converted to LAS 5.0
  5.0       R.MCKINNEY      Dec  1988	    Updated to close all files
  5.3       D. Etrheim      Oct  1991       Initialize some fields of the DDR.
  6.1       D. Eichacker    May  1994       added opt[]

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   TESTGEN must be run under TAE.

PROJECT:  	         LAS		                

LANGUAGE:		  C	

ALGORITHM:
        UNIFORM opens the input images for read access. 
        Opens the output image for write access.  
        Opens the output image. 
        Writes the value specified by the user to the output image.
    
ALGORITHM REFERENCES:    none

TESTING complete on SUN UNIX (unix), PN9050 UNIX (gould), and
        VAX VMS (vms).

******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "testgen.h"

extern int  fdesc[MAXIMG+1];	/* image file descriptor used for image i/o  */
extern int  gdesc;		/* group descriptor used for image i/o	     */
extern long offset1[MAXBND+1];  /* offset into the image i/o buffer	     */
extern long curent;		/* current processing line		     */ 
extern long curcat;		/* current processing band		     */

void uniform
(
  struct PARBLK *vblock,           /* TAE parameter block */
  char *hosout,                    /* Host output image name */
  long nl,                         /* Number of lines for output image */
  long ns,                         /* Number of samples for output image */
  long bands,                      /* Bands to be processed for all images */
  long odtype,                     /* Output image data type */
  float (pixval)[256],             /* Array of up to 256 uniform values */
  long msgtim,                     /* Time betweeen display messages */
  char *minmax,                    /* Min/Max flag */
  long dflag                       /* DDR flag option */
)
{
static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];	/* max pixel value array                     */

long	access;			/* image I/O access type                     */
long	bufsiz;			/* size of image I/O buffer                  */
long 	curlin;			/* current line being processed              */
long    fatal = -1;		/* fatal error flag                          */
long    i;			/* counter				     */
long    lpixval;                /* pointer for integer pixval conversion     */
long	msg_flag=ALLCAT;	/* flag for porcessing message               */
long    nonfatal = 1;		/* nonfatal error flag			     */
long	one = 1;		/* value of one                              */
long	size;			/* number of bytes per pixel 	 	     */
long	status;			/* function return status code               */
long	zero = 0;		/* value of zero                             */

static  long    opt[3]={0,0,0}; /* optional parameter array for EOPENS       */ 

short   spixval;                 /* pointer for I*2 pixval conversion        */

char    bpixval;                 /* pointer for byte pixval conversion       */
unsigned char *thebuf;		/* image I/O buffer                          */

struct  DDR ddr;		/* ddr structure			     */
static struct BDDR bddr[MAXBND];/* band ddr structure			     */

/*
   open output image
*/
access = IWRITE;	
status = c_eopens(&fdesc[0],hosout,&bands,&one,&one,&nl,&ns,offset1,
         &access,&odtype,&zero,opt);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","testgen-fatal",&fatal);
fdesc[1] = 0;

/*
   allocate I/O buffer
*/
if ((thebuf = (unsigned char *)c_ealloc(fdesc,&bufsiz)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","testgen-alloc",&nonfatal);
   if (c_eclose(&fdesc[0]) != E_SUCC)
      c_errmsg("Nonfatal error encountered","testgen-warn",&nonfatal);
   c_clean(vblock,"testgen",hosout,&one);
   }

/*
   group images
*/
if ((c_egroup(fdesc,&gdesc,thebuf,&bufsiz)) != E_SUCC)
   {
   if (c_eclose(&fdesc[0]) != E_SUCC)
      c_errmsg("Nonfatal error encountered","testgen-warn",&nonfatal);
   c_clean(vblock,"testgen",hosout,&one);
   }

/*
   setup for data type
*/
switch(odtype)
   {
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:	size = 4; break;
   case EREAL:  size = 4; break;
   }

/*
   setup for processing message utility
*/
c_inittm("TESTGEN","Processed","line","bands"," ",&nl,
         &bands,&msgtim,&msg_flag);
curent = 1;    
c_startm();

/*
   initialize min/max arrays
*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&odtype,&bands,minval,maxval);

/*
   main processing loop
*/


	for (curlin = 0; curlin < nl; curlin++)
	{
	if (c_estep(&gdesc) != E_SUCC)
            {
            if (c_egclse(&gdesc) != E_SUCC)
            c_errmsg("Nonfatal error encountered", "testgen-warn",&nonfatal);
            c_clean(vblock,"testgen",hosout,&one);
            }
	for (i = 0; i < bands; i++)
	    {
	    switch (odtype)
	      {
	      case 1: bpixval = pixval[i];
		      c_pxfill(thebuf + (*(offset1+i) * size),&odtype,&ns,
			&bpixval);
		      break;
	      case 2: spixval = pixval[i];
		      c_pxfill(thebuf + (*(offset1+i) * size),&odtype,&ns,
			&spixval);
		      break;
	      case 3: lpixval = pixval[i];
		      c_pxfill(thebuf + (*(offset1+i) * size),&odtype,&ns,
			&lpixval);
		      break;
    	      case 4:
	              c_pxfill(thebuf + (*(offset1+i) * size),&odtype,&ns,
                        &pixval[i]);
	 	      break;
	      }
	if (strcmp(minmax,"YES") == 0)
	    status=c_pxmnmx(thebuf + (*(offset1+i) * size),&ns,
            &minval[i],&maxval[i],&odtype);
	    }
	curent++;
	}
/*
  final buffer flush
*/
if (c_estep(&gdesc) != E_SUCC)
   {
   if (c_egclse(&gdesc) != E_SUCC)
      c_errmsg("Nonfatal error encountered", "testgen-warn",&nonfatal);
   c_clean(vblock,"testgen",hosout,&one);
   }

/*
   turn off display messages
*/
c_stoptm();

/* 
   update DDR
*/

status = c_getddr(hosout, &ddr);

for (i = 1; i < bands+1; i++)
	status = c_getbdr(hosout, &bddr[i-1], &i);

for(i = 0; i < 7; i++)
    ddr.valid[i] = dflag;
ddr.valid[7] = VALID;
ddr.line_inc = (double) one;
ddr.sample_inc = (double) one;
ddr.master_line = one;
ddr.master_sample = one;

status = c_putddr(hosout, &ddr);
if (status != E_SUCC)
   c_errmsg("Nonfatal error encountered", "testgen-warn",&nonfatal);

for (i = 0; i < bands; i++)
    {
    status = c_putbdr(hosout, &bddr[i]);
    if (status != E_SUCC)
       c_errmsg("Nonfatal error encountered", "testgen-warn",&nonfatal);
    }

/*
   create associated min/max file
*/
if (c_svmnmx(hosout,&bands,minval,maxval,minmax) != E_SUCC)
   c_errmsg("Nonfatal error encountered", "testgen-warn",&nonfatal);

/*
   close all image files in the group
*/
if (c_egclse(&gdesc) != E_SUCC)
   c_errmsg("Nonfatal error encountered", "testgen-warn",&nonfatal);

/*
   free buffer
*/
free(thebuf);

}
