#include "texture.h"
#include "protos.h"

void filspec
(
    char hosin[][CMLEN],      /* I: input image host names             */
    char hosout[],            /* I: output image host name             */
    long *num_inimages,       /* I: number of input images             */
    long num_image_bands[],   /* I: number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* I: band numbers in each      */
    long *num_inbands,	      /* I: total number of input image bands  */
    long wind[][4],           /* I: input image windows                */
    long *nlout,	      /* I: output no. lines                   */ 
    long *nsout		      /* I: output no. samples per line        */
)
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         8-May-89  K. A. Narayanan    
    2         6-Sep-89  K. A. Narayanan    initialized line/samp inc
    6.1      20-May-94  D. J. Eichacker    Changed zero to opt[] for use
 					   in eopen.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

**************************************************************************
*/
{
double	line_inc = 1.0,
	samp_inc = 1.0;

long	access,                      /* file access type                   */
	comb_flag = NOCOMB,
	n, nb, 
	stat,                        /* return status                      */
	type;                        /* file type                          */
long    fatal_err = -1;
static	long    opt[3] = {0,0,0};    /* Option array for eopen             */
long    one = 1;

/*
open input and output image files for random access
---------------------------------------------------
*/
nb = 0;
access = IREAD;
type = EREAL;
for (n = 0; n < *num_inimages; n++)
   {
   stat = c_eopenr ((struct FDESC **)&fdesc_in[n], hosin[n], &nb, &wind[0][SL], 
                   &wind[0][SS], &wind[0][NL], &wind[0][NS], 
                   &access, &type, opt, &one);
   if (stat != E_SUCC )
      c_errmsg ("Error opening input image", "texture-open", &fatal_err);
   }  

access = IWRITE;
type = EREAL;
stat = c_eopenr ((struct FDESC **)&fdesc_out, hosout, num_inbands, &one, &one,
		 nlout, nsout, &access, &type, opt, &one);
if (stat != E_SUCC )
   c_errmsg ("Error opening output image", "texture-open", &fatal_err);
if ((outbuf = (float  *) malloc (*num_inbands * *nsout * sizeof(float))) == NULL)
   c_errmsg ("Error allocating dynamic memory", "texture-alloc", 
              &fatal_err);
/*
update output image ddr
-----------------------
*/
c_upddr (&block, hosin, hosout, num_inimages, wind, image_band_num, 
         num_image_bands, &line_inc, &samp_inc, num_inbands, &comb_flag);
}
