#include "texture.h"
#include "protos.h"
#include "util.h"

void getpar
(
    char hosin[][CMLEN],      /* O: input image host names            */
    char lasout[CMLEN],       /* O: output image las name             */
    char hosout[CMLEN],       /* O: output image host name            */
    long *num_inimages,       /* O: number of input images            */
    long num_image_bands[],   /* O: num of bands in each input image  */
    long image_band_num[][MAXBND + 1], /* O: band numbers in each     */
    long *num_inbands,        /* O: total number of input image bands */
    long wind[][4],           /* O: input image windows               */
    long *stepns,             /* O: sample step size		      */
    long *stepnl,             /* O: line step size		      */
    long *angle,              /* O: orientation angle		      */
    long *dist,               /* O: distance between cooccurences     */
    long *wns,                /* O: no. samples/line in kernel	      */
    long *wnl,                /* O: no. lines in kernel 	      */
    long *funct,	      /* O: texture function		      */
    long *nlout,              /* O: output no. lines		      */
    long *nsout,              /* O: output no. samples per line       */
    long print[],             /* O: print option		      */
    char print_file[]         /* O: output print file		      */
)
/*
***************************************************************************

NAME: GETPAR

PURPOSE: Get parameters

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         18-May-89    K. Narayanan    Conversion to 5.0
    2         02-Nov-99    D. Hames	   Add additional error checks for
					   kernel dimension.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT:	LAS

***************************************************************************
*/
{
struct	VARIABLE *var;               /* pointer to a variable in TAE block */

long	count, 
	flag[MAXIMG][2],            /* flag for subwindows                */
	imglimit = MAXIMG,          /* maximum number of images           */
	itype = ELONG,
	kernel_size[2],	            /* kernel dimensions                  */
	same_window,                /* flag indicating if images have     */
                                    /* same window                        */
	slen,
	stat,                       /* return status                      */
	std_out;                    /* logical unit for standard output   */

char	cfunc[5],
	fileutil_file[CMLEN],
	lasin[MAXIMG][CMLEN],       /* input image las names              */
	minmax[4];                  /* minmax flag                        */
long    fatal_err = -1;
long    inf_err = 1;
long    zero = 0;
long    one = 1;

char    buf[ERRLEN];                   /* print buffer          */

/*
initialize program 
------------------
*/
c_inlas (&block, &std_out);

/*
get and validate input image parameters
---------------------------------------
*/
stat = c_getin (&block, "IN", &zero, lasin, hosin, num_inimages, wind, 
                image_band_num, num_image_bands, num_inbands, &imglimit, 
                &same_window, flag);

if (stat != E_SUCC) 
    c_errmsg ("Error accessing the input image", "texture-in", 
              &fatal_err);

if (!same_window)
   c_errmsg ("Images must be same size", "texture-size", &fatal_err);
/*
get output image name
---------------------
*/
if (c_getout (&block,"OUT",(char (*)[256])lasout,(char (*)[256])hosout)!=E_SUCC)
   c_errmsg ("Error opening output image", "texture-out", &fatal_err);
/*
get function 
------------
*/
c_getstr (&block, "TEXTOPT", &one, &slen, cfunc);

if (strncmp (cfunc, "VARI", 4) == 0) *funct = 1;
if (strncmp (cfunc, "SKEW", 4) == 0) *funct = 2;
if (strncmp (cfunc, "KURT", 4) == 0) *funct = 3;
if (strncmp (cfunc, "CORR", 4) == 0) *funct = 4;
if (strncmp (cfunc, "HOMO", 4) == 0) *funct = 5;
if (strncmp (cfunc, "INER", 4) == 0) *funct = 6;
if (strncmp (cfunc, "ENER", 4) == 0) *funct = 7;
if (strncmp (cfunc, "ENTR", 4) == 0) *funct = 8;
/*
get window dimensions
---------------------
*/
count = 2;
c_getnum (&block, "KERNDIM", &count, &itype, (unsigned char *)kernel_size);
*wnl = kernel_size[0];
*wns = kernel_size[1];

/* error check KERNDIM for vaild ranges
---------------------------------------
*/
if (*wnl > 31) 
   c_errmsg ("KERNDIM cannot be larger than 31 in the vertical dimension",
             "texture-kerndim", &fatal_err);

if (*wnl < 3)
   c_errmsg ("KERNDIM cannot be smaller than 3 in the vertical dimension",
             "texture-kerndim", &fatal_err);

if (*wns < 3)
   c_errmsg ("KERNDIM cannot be smaller than 3 in the horizontal dimension",
             "texture-kerndim", &fatal_err);

/*
abort if the image is smaller than the window
---------------------------------------------
*/
if (*wnl  >  wind[0][NL] ||  *wns  >  wind[0][NS]) 
   c_errmsg ("Sliding window is larger than input image (or input window)",
             "texture-swind", &fatal_err); 
/*
*
* get step size
*
*/
c_getnum (&block, "LINEINC", &one, &itype, (unsigned char *)stepnl);
c_getnum (&block, "SAMPINC", &one, &itype, (unsigned char *)stepns);

/*
*
* calculate output lines and samples
*
*/
*nlout = (wind[0][NL] - *wnl)/(*stepnl) + 1;
*nsout = (wind[0][NS] - *wns)/(*stepns) + 1;

if (*funct >= 4) 
   {
/*
get angle and distance for functions 4 - 8 
------------------------------------------
*/
   c_getnum (&block, "ORIANG", &one, &itype, (unsigned char *)angle);
   c_getnum (&block, "SAMPDIST", &one, &itype, (unsigned char *)dist);
/*
check for the proper window size and distance
---------------------------------------------
*/
   if (*dist < 1) *dist=1;
   if ((*wns < *dist+2)  &&  (*angle != 90)) 
      {
      c_errmsg ("Horizontal window size (pixels) too small for adequate sample",
                "texture-horwind", &inf_err);
      c_errmsg ("Increase window size or decrease distance",
                "texture-horwind", &fatal_err);
      }  
   if ((*wnl < *dist+2)  &&  (*angle != 0)) 
      {
      c_errmsg ("Vert. window size (lines) too small for adequate sample",
                "texture-vertwind", &inf_err);
      c_errmsg ("Increase window size or decrease distance",
                "texture-vertwind", &fatal_err);
      }  
   }  
/*
Get print option and open print file 
------------------------------------
*/
strcpy (fileutil_file, "texture");
stat = c_setprt (&block, &print[TERM], &print[FIL], &print[LP], fileutil_file,
          print_file);
if (stat != E_SUCC)
   c_errmsg("Fatal error encountered","texture-fatal",&fatal_err);

if ((print[LP]) || (print[FIL]))
   fd = fopen (fileutil_file, "w");
else
   fd = NULL;

sprintf (buf, "Input image: %s", lasin[0]);
c_txtout (buf, print, fd);
/*
Get las globals
---------------
*/
var = p_find( &block, "$msgtim");
msgtim = IVAL( *var, 0);
var = p_find( &block, "$minmax");
strcpy( minmax, SVAL( *var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;
}
