#include "texture.h"
#include "util.h"
#include "protos.h"

void print_stats
(
    long band,			/* I: band number                     */
    long nlout,			/* I: output no. lines		      */
    long nsout,			/* I: output no. samples per line     */
    double z[],			/* I: texture statistics              */
    long nbands,                /* I: number of bands                 */
    long print[],		/* I: print option		      */
    long fnl,			/* I: number of lines in kernel       */
    long fns,			/* I: number of samples/line in kernel*/
    long stepnl,		/* I: line step size		      */
    long stepns,		/* I: sample step size		      */
    long funct,			/* I: texture function		      */
    int angle,			/* I: orientation angle               */
    int distance,		/* I: distance between cooccurences   */
    long lo_reset,		/* I: no. of low corr values          */
    long hi_reset		/* I: no. of highcorr values          */
)
/*
**********************************************************************

NAME: PRINT_STATS

PURPOSE: Prints texture statistics on terminal, line printer, or
         saves them to a file.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1          1-Sep-83    L. Urquhart
    2          9-Feb-84    G. Mcconaughy
    3         18-May-89    K. Narayanan    Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

**********************************************************************
*/
{
static	double	norm;

double	avg, kurtosis, skew, std, temp, tn, var;

long	i;
long    fatal_err = -1;
char    buf[ERRLEN];                   /* print buffer          */

static	char	fname[][17] = {
 	" VARIANCE       ", " SKEWNESS       ",  " KURTOSIS       ", 
	" CORRELATION    ", " HOMOGENEITY    ",  " INERTIA        ", 
	" ENERGY         ", " ENTROPY        "};

if (band == 1) norm = 0;

tn = (double) (nlout * nsout);
for (i = 0; i < 4; i++) z[i] /= tn;
avg     = z[0];
norm    += (temp = z[0]*z[0]);
var     = z[1] - temp;
if (var < 0.0) 
   {
   sprintf (buf, "--NEGATIVE SQR -- VAR= %12.4g Z1= %12.4g Z2= %12.4g",
           var, z[0], z[1]);
   c_txtout (buf, print, fd);
   c_errmsg ("Negative square root encountered", "texture-negsqr", &fatal_err); 
   }  
skew    = z[2] - 3.*z[1]*z[0] + 2.*(z[0]*temp);
kurtosis= z[3] - 4.*z[2]*z[0] + 6.*z[1]*temp - 3.*(temp*temp);
 if (var > 0.0) 
   {
   kurtosis /= (temp = var*var);
   skew    /= sqrt (temp*var);
   var     *= tn/(tn-1.0);
   std     = sqrt (var);
   }
else
   {
   kurtosis = 0.0;
   skew = 0.0;
   std = 0.0;
   }  

c_txtout ("----------------------------------------------------", 
          print, fd);
sprintf (buf, "---------TEXTURE FUNCTION:%16s----------", fname[funct-1]);
c_txtout (buf, print, fd);
sprintf (buf, "WINDOW:%2dx%2d   STEP:%2dx%2d   ANGLE:%3d   DISTANCE:%3d",
         fnl, fns, stepnl, stepns, angle, distance);
c_txtout (buf, print, fd);
sprintf (buf, "----------Report for Band %2d -----------\n", band);      
c_txtout (buf, print, fd);
/*
print out the number of reset correlation values
------------------------------------------------
*/
if (lo_reset > 0)
   {
   sprintf (buf, 
            "Number of correlation values below -1.0 reset to -1.0 = %6d", 
             lo_reset);
   c_txtout (buf, print, fd);
   }
if (hi_reset > 0)
   {
   sprintf (buf, 
            "Number of correlation values above +1.0 reset to +1.0 = %6d", 
             hi_reset);
   c_txtout (buf, print, fd);
   }
/*
print statistics
----------------
*/
sprintf (buf, "AVERAGE             %12.4g", avg);      
c_txtout (buf, print, fd);
sprintf (buf, "STANDARD DEVIATION  %12.4g", std);
c_txtout (buf, print, fd);
sprintf (buf, "VARIANCE            %12.4g", var);
c_txtout (buf, print, fd);
sprintf (buf, "SKEW COEFFICIENT    %12.4g", skew);
c_txtout (buf, print, fd);
sprintf (buf, "KURTOSIS COEFFICIENT%12.4g", kurtosis);
c_txtout (buf, print, fd);
sprintf (buf, "MINIMUM VALUE       %12.4g", z[4]);
c_txtout (buf, print, fd);
sprintf (buf, "MAXIMUM VALUE       %12.4g\n\n", z[5]);
c_txtout (buf, print, fd);

if (band == nbands)
   {
   if (norm < 0.0) 
      {
      sprintf (buf, "--NEGATIVE SQR -- NORM= %12.4g Z1= %12.4g", norm, z[0]);
      c_txtout (buf, print, fd);
      c_errmsg ("Negative square root encountered", "texture-negsqr", &fatal_err); 
      }  
   norm = sqrt (norm);
   if (nbands > 1) 
      {   
      sprintf (buf, "The NORM for all %3d bands is %12.4g", nbands, norm);
      c_txtout (buf, print, fd);
      }  
   }
}
