#include "texture.h"
#include "util.h"
#include "protos.h"

void textsub
(
    char hosin[][CMLEN],   	/* I: input images host names         */
    char lasout[],         	/* I: output image LAS name           */
    char hosout[],         	/* I: output image host name          */
    long *num_images,           /* I: number of input images          */
    long num_image_bands[],	/* I: num of bands in each input image*/
    long image_band_num[][MAXBND + 1],/* I: num of bands in each input image*/
    long *nbands,               /* I: number of bands                 */
    long stepnl,		/* I: line step size		      */
    long stepns,		/* I: sample step size		      */
    long funct,			/* I: texture function		      */
    long angle,			/* I: orientation angle		      */
    long fnl,			/* I: number of lines in kernel       */
    long fns,			/* I: num of samples/line in kernel   */
    long nl,			/* I: num of lines in input image     */
    long ns,			/* I: num of samples/line in input image */
    long *nlout,		/* I: output no. lines		      */
    long nsout,			/* I: output no. samples per line     */
    long distance,		/* I: distance between cooccurences   */
    long print[],		/* I: print option		      */
    char print_file[]		/* I: output print file		      */
)

/*
**********************************************************************

NAME: TEXTSUB                        

PURPOSE: Performs the analysis for 6 texture functions.

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1          1-Sep-83    L. Urquhart
    2          9-Feb-84    G. Mcconaughy
    3         18-May-89    K. Narayanan    Conversion to 5.0
    4          6-Sep-89    K. Narayanan    Corrected call to COMPLT

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:

       for each band 
 
          for number of lines in output image
             read input lines into work area
             calculate working sums
             calculate requested texture function
             collect sums for summary stats
             write output line

          print summary stats

       close all files
       update output image min and max values in ddr
       create output image history
       issue completion message

**********************************************************************
*/
{
static double pij,	  
	maxval[MAXBND], minval[MAXBND],
	s[9], sum, 
	temp, temp1,  
	v,  
	xa, xm2, xm3, xm4, xt, 
	z[6];

float	corr, xn, 
	*work_buf, *work[31];		/* a cylindrical work area        */

long	band, bsz, code, distns, distnl, 
	hi_reset, i, iband, ii, image, i2, 
	j, jj, jline, j2, kline, 
	line, lo_reset, mlines, msamps, 
	msg_flag = ONECAT,
	myline[31], n, num_files,  
	oband, offlin1, offlin2, offsamp, oline, osamp, outline, 
	samp, *izbuf;
long    one = 1;

char	files[2][CMLEN];


distns=1;
distnl=1;
if (angle != 0) distnl=distance;
if (angle != 90) distns=distance;
if (funct > 6) 
   {
   msamps = fns - distns;
   if (angle  ==  90) msamps = fns;
   mlines = fnl - distnl;
   if (angle  ==   0) mlines = fnl;
   bsz=2*msamps*mlines;
   if ((izbuf = (long *) malloc (bsz * sizeof (long))) == NULL)
      {
      if (fd) fclose (fd);    
      c_eclose ((struct FDESC **)&fdesc_out);
      c_abort (&block, "Error allocating dynamic memory", "texture-alloc", 
               "texture", out_files, &num_outfiles);
      }
   }  

if (funct  <=  3) 
   {
   msamps = fns;
   mlines = fnl;
   offlin1 = 0;
   offsamp = 0;
   offlin2 = 0;
/*
*
* set up offset conditions for summations
* depending on the angle of precedence

*                 0       45      90      135     (degrees)
*     msamps    fns-d    fns-d    fns      fns-d
*     mlines    fnl      fnl-d    fnl-d    fnl-d
*     offlin1   0        d        0        0
*     offsamp   d        d        0        d
*     offlin2   0        0        d        d
*/
   }
else
   {
   msamps = fns - distns;
   if (angle  ==  90) msamps = fns;
   mlines = fnl - distnl;
   if (angle  ==   0) mlines = fnl;
   offlin1 = 0;
   if (angle  ==  45) offlin1 = distnl;
   offsamp = distns;
   if (angle  ==  90) offsamp = 0;
   offlin2 = distnl;
   if (angle  <  90) offlin2 = 0;
   }  
if (funct > 6)
   xn = (float) (msamps * mlines * 2);
else
   xn = (float) (msamps * mlines);

if ((work_buf = (float *) malloc (fnl * ns * sizeof (float))) == NULL)
   {
   if (fd) fclose (fd);    
   c_eclose ((struct FDESC **)&fdesc_out);
   c_abort (&block, "Error allocating dynamic memory", "texture-alloc", 
            "texture", out_files, &num_outfiles);
   }
for (i = 0; i < fnl; i++)
   work[i] = &work_buf[i * ns];

/*
*
* initialize processing message utility
*
*/
c_inittm ("TEXTURE", "Processed", "line", "band", " ", nlout,
           nbands, &msgtim, &msg_flag);
/*
*
* process bands sequentially
*
*/
for (image = 0, oband = 1; image < *num_images; image++)
for (band = 0; band < num_image_bands[image]; band++, oband++)
   {
   iband = image_band_num[image][band];
/*
*
* reset maximum and minimum
*
*/
   lo_reset = 0;
   hi_reset = 0;
   z[5] = 0.0;
   z[4] = 9.e9;
/*
*
* reset global sums
*
*/
   for (i = 0; i < 4; i++) z[i] = 0.0;
/*
*
* for example, the cross product terms are:
*
*           ns-1   nl
*      0 = s u m  s u m  x(k,l)  *  x(k+d,l)
*           k=1    l=1
*
*           ns-1   nl-1
*     45 = s u m  s u m  x(k,l+d)*  x(k+d,l)
*           k=1    l=1
*
*           ns     nl-1
*     90 = s u m  s u m  x(k,l)  *  x(k,l+d)
*           k=1    l=1
*
*           ns-1   nl-1
*    135 = s u m  s u m  x(k,l)  *  x(k+d,l+d)
*           k=1    l=1
*
*     in general
*
*          msamps  mlines
*      x = s u m   s u m  x(k,l+offln1)*x(k+offsamp,l+offlin2)
*           k=1     l=1
*
* initialize work array
*
*/
   curent = 1;
   curcat = oband;
   c_startm();
   for (line = 1; line <= fnl; line++)
      c_eread ((struct FDESC **)&fdesc_in[image], &iband, &line,
               (unsigned char *)work[line-1], &one);
/*
*
*     apply the function to the picture
*
*/
   outline=1;
   for (oline = 0; oline < nl; oline += stepnl, outline++)
      {
      if (outline <= *nlout)
         {
/*
*
* calculate myline as the line index into
* the cylindrical buffer
*
*/
         for (i = 0; i < fnl; i++)
            myline[i] = (oline+i) % fnl;
/*
*
* loop for output samples
*
*/
         osamp = 0;
           
         for (samp = 0; samp < ns; samp += stepns, osamp++)
            {
            if (osamp < nsout)
               {
/*
*
* first initialize
*
*/
               if (funct > 6) n = 0;
               for (i = 0; i < 9; i++) s[i] = 0.0;
               for (ii = 0; ii < msamps; ii++)
                  {
                  i = samp + ii;
                  i2= i + offsamp;
                  for (jj = 0; jj < mlines; jj++)
                     {
                     j = myline[jj + offlin1];
                     j2= myline[jj + offlin2];
/*
*
* calculate working sums
*
*/
                     if (funct <= 6)
                        {
                        s[0] += work[j][i];
                        s[1] += (temp1 = work[j][i] * work[j][i]);
                        s[2] += work[j][i] * temp1;
                        s[3] += temp1 * temp1;
                        s[4] += work[j2][i2];
                        s[5] += work[j2][i2] * work[j2][i2];
                        s[6] += work[j][i] * work[j2][i2];
                        temp = (work[j][i] - work[j2][i2]) *
                               (work[j][i] - work[j2][i2]);
                        s[7] += 1./(1.+temp);
                        s[8] += temp;
                        }
                     else
                        {
/*
*
* inzert precedence & antipredence in izbuf 
*
*/
                        n++;
                        code = (long)(32768 * work[j][i] + work[j2][i2]);
                        inzertion (izbuf, code, n);
                        n++;
                        code = (long)(32768 * work[j2][i2] + work[j][i]);
                        inzertion (izbuf, code, n);
                        }
                     }
                  }
/*
*
* calculate function desired
*
*/
               if (funct <= 6)
                  {
                  for (i = 0; i < 9; i++) s[i] /= xn;
                  xm2 =  s[1] - s[0] * s[0];

/*
*
* variance
*
*/
                  if (funct == 1) 
                     {
                     outbuf[osamp] = xm2*xn/(xn-1.0);
                     }
/*
*
* skewness
*
*/
                  else if (funct == 2) 
                     {
                     xm3 = s[2]-3.*s[1]*s[0]+2.*(s[0]*s[0]*s[0]);
                     outbuf[osamp] = 0.0;
                     if (xm2 > 0.0) outbuf[osamp] = xm3/sqrt(xm2*xm2*xm2);
                     }
/*
*
* kurtosis
*
*/
                  else if (funct == 3) 
                     {
                     xm4 = s[3]-4.*s[2]*s[0]+6.*s[1]*s[0]*s[0]-
                           3.*(s[0]*s[0]*s[0]*s[0]);
                     outbuf[osamp] = 0.0;
                     if (xm2 > 0.0) outbuf[osamp] = xm4/(xm2*xm2);
                     }
/*
*
* correlation
*
*       (see j.h. pollard, numerical & statistical
*                  techniques, chapter 8)
*
*                      s(xy) -  ( (s(x)+s(y))/2 )**2
*                  p= ---------------------------------------
*                     (s(xx)+s(yy))/2 - ( (s(x)+s(y))/2 )**2
*
*/
                  else if (funct == 4) 
                     {
                     outbuf[osamp] = 0.0;
                     xa = (s[0]+s[4])/2.0;
                     xa = xa*xa;                                         
                     xt = (s[1] + s[5])/2 - xa;
                     if (xt  !=  0.) corr = (s[6] - xa)/xt;
/*
                 -----------------------------------------------
                 Areas with little or no variability may result
                 in invalid correlations (outside -1 to +1) due
                 to accumulating round-off errors. If so, reset.
                 B. Kovalick (STX)   January 10, 1988
                 -----------------------------------------------
*/
                     if (corr < -1.0) 
                        {
                        corr = -1.0;
                        lo_reset++;
                        }
                     else if (corr > 1.0)
                        {
                        corr = 1.0;
                        hi_reset++;
                        }
                     outbuf[osamp] = corr;
                     }
/*
*
* homogeneity
*
*/
                  else if (funct == 5) 
                     {
                     outbuf[osamp] = s[7];
                     }
/*
*
* inertia
*
*/
                  else if (funct == 6) 
                     {
                     outbuf[osamp] = s[8];
                     }  
                  }
/*
*
* energy or entropy
*
*/
               else
                  {
                  sum = 0.;
                  code = izbuf[0];
                  pij  = 1.0;
                  for (i = 1; i < n; i++)
                     {
                     if (izbuf[i] ==  code) pij++;
                     else
                        {
                        if (funct  ==  7) sum += pij*pij;
                        if (funct  ==  8) sum -= pij/xn*log(pij/xn);
                        pij = 1.0;
                        code = izbuf[i];
                        }  
                     }
/*
*
* last element 
*
*/
                  if (funct  ==  7) sum += pij*pij;
                  if (funct  ==  8) sum -= pij/xn * log (pij/xn);
                  if (funct  ==  7) sum /= (xn*xn);
                  outbuf[osamp] = sum;
                  }
/*
*
* collect the sums for the summary statistics
*
*/
               v = outbuf[osamp];
               z[0] += v;
               z[1] += (temp1 = v*v);
               z[2] += v*temp1;
               z[3] += temp1*temp1;
               if (v  <  z[4]) z[4] = v;         /* min value    */
               if (v  >  z[5]) z[5] = v;         /* max value    */
               }  
            }
/*
*
* write the output line
*
*/
         c_ewrite ((struct FDESC **)&fdesc_out, &oband, &outline, 
                   (unsigned char *)outbuf, &one);
         curent++;
/*
*
* read in the next lines into the cylindrical work area
*
*/
         if (outline != *nlout)
            {
            for (kline = 1; kline <= stepnl; kline++)
               {
               line  = oline + fnl + kline;
               jline = (line-1) % fnl;
               c_eread ((struct FDESC **)&fdesc_in[image], &iband, &line,
                        (unsigned char *)work[jline], &one);
               }
            }  
         }
      }
/*
*
* calculate and display the summary statistics
*
*/
   c_stoptm ();
   print_stats (oband, *nlout, nsout, z, *nbands, print, fnl, fns,stepnl,stepns,
                      funct, angle, distance, lo_reset, hi_reset);
   minval[oband-1] = z[4];
   maxval[oband-1] = z[5];
   }

/*
*
* update output image min and max values in ddr
*
*/
c_svmnmx (hosout, nbands, minval, maxval, "YES");

/*
*
* close files, free storage
*
*/
free (work_buf);
if (funct > 6) free (izbuf);
c_eclose ((struct FDESC **)&fdesc_out);

/*
*
* create output image history
*
*/
c_hstout (&block, hosout, hosin, num_images, " ");

/*
*
* issue completion message
*
*/
num_files = 1;
strcpy (files[0], lasout);
if (print[FIL])
   {
   strcpy (files[1], print_file);
   num_files++;
   }
c_complt ("TEXTURE", files, &num_files);
}
