/* include files, global parameters, and definitions for TEXTURE */

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "protos.h"

FILE	*fd;                           /* print file descriptor   */
struct PARBLK block;                   /* TAE parameter block     */

float	*outbuf;        	       /* image i/o buffer        */

long	fdesc_in[MAXIMG];              /* file descriptors        */
long	fdesc_out;                     /* file descriptors        */
long	min_max;                       /* min max flag            */
long 	msgtim;                        /* message interval        */
long	num_outfiles;                  /* number of output files  */
 
extern	long	curcat;                /* message utility         */
extern	long	curent;                /* parameters for processing */


char    out_files[2][CMLEN];           /* output file names       */

#define TERM 0
#define LP   1
#define FIL 2

/*
**********************************************************************

NAME: TEXTURE

PURPOSE: 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1          1-Sep-83    L. Urquhart
    2          9-Feb-84    G. Mcconaughy
    3         18-May-89    K. Narayanan    Conversion to 5.0
    4          6-Sep-89    K. Narayanan    Corrected TEXTSUB for argument 
                                           for COMPLT; and FILSPEC to initialize
                                           line/samp inc
    5         19-Sep-92    D. Etrheim      Modifed global include file
    6         02-Nov-99    D. Hames        Remove 50000 limit from TEXTURE.PDF
					   Add additional error checks for 
					   kernel dimension in GETPAR.
					   Add prototypes.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
	Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Call GETPAR to get parameters from TAE
   Call FILSPEC set up image i/o specifications
   Call TEXTSUB to compute texture statistics and generate 
   output image

**********************************************************************
*/

void main
(void) 
{
long	angle = 0;		/* orientation angle		  */
long	dist = 1;		/* distance between cooccurences  */
long	funct;			/* texture function		  */
static long	image_band_num[MAXIMG][MAXBND + 1]; 
	                        /* band numbers in each input image*/
long	nlout;			/* output no. lines		  */
long	nsout;			/* output no. samples per line 	  */
long	num_inbands;            /* total number of input image bands*/
static long	num_image_bands[MAXIMG];/* number of bands in each input image*/
long	num_inimages;           /* number of input images          */
long	print[4];		/* print option			   */
long	stepnl;			/* line step size		   */
long	stepns;			/* sample step size		   */
static long	wind[MAXIMG][4];        /* input image windows             */
long	wnl;			/* no. lines in kernel 		   */
long	wns;			/* no. samples/line in kernel	   */

static char	hosin[MAXIMG][CMLEN];   /* input images host names         */
char	hosout[CMLEN];          /* output image host name          */
char	lasout[CMLEN];          /* output image las name           */
char	print_file[CMLEN];	/* output print file		   */

/*
get parameters
--------------
*/
getpar (hosin, lasout, hosout, &num_inimages, num_image_bands, image_band_num, 
        &num_inbands, wind, &stepns, &stepnl, &angle, &dist, &wns, &wnl, &funct,
        &nlout, &nsout, print, print_file);
/*
open input and output images
----------------------------
*/
filspec (hosin, hosout, &num_inimages, num_image_bands, image_band_num, 
         &num_inbands, wind, &nlout, &nsout);

/*
compute the texture function
----------------------------
*/
textsub (hosin, lasout, hosout, &num_inimages, num_image_bands, image_band_num, 
         &num_inbands, stepnl, stepns, funct, angle, wnl, wns, wind[0][NL], 
         wind[0][NS], &nlout, nsout, dist, print, print_file);
}
