/*****************************************************************************
name: EMBEDPT 

purpose: Place points on an image, using a text file as input.
	 This file will be an ASCII file, of the form
     npts
     line      sample      intensity  
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ticmark.h"

int embedpt
(
    struct PARBLK *vblk,    /* I: TAE parameter block */
    char *infile,           /* I: File with user defined symbol locations */
    char tempfile[][CMLEN], /* O: Temporary sort file names */
    long lines[],           /* I: Lines array for ticmarks */
    long samps[],           /* I: Samps array for ticmarks */
    long zlevel[],          /* I: Intensity array for ticmarks */
    long usr_line[],        /* I: User specified line number array */
    long *usr_samp,         /* I: User specified sample number array */
    long symdim[3],         /* I: Symbol/tic dimensions (h,l,w) */
    float scale,            /* I: Scale factor */
    long mingrey,           /* I: Minimum grey value */
    long maxgrey,           /* I: Maximum grey value */
    FILE *fp,               /* I: Pointer to the report file */
    long print[],           /* I: Array telling where to print report */
    long (*window)[4]       /* I: Window from input image */
)
{
    long    cline;          /* calculated line coordinate                   */
    long    csamp;          /* calculated sample coordinate                 */
    long    bright;         /* symbol brightness value                      */
    long    badtmark;       /* Number of ticmarks outside of image          */
    long    cnt;            /* loop counter                                 */
    long    j;              /* loop counter                                 */
    long    count=0;        /* tic counter                                  */
    long    non_fatal=1;    /* Non fatal error flag	                        */
    long    symbol = TIC;   /* symbol type	default to center               */
    long    line;           /* line number                                  */
    long    samp1;          /* left sample number                           */
    long    samp2;          /* right sample number                          */
    long    result;         /* resulting value	                            */
    long    tmark;          /* Tic mark counter                             */
    float   temp_bright;    /* temporary pixel intensit	                    */
    float   temp_line;      /* temporary line number                        */
    float   temp_samp;      /* temporary sample number                      */
    FILE    *infp;
    FILE    *temp_fp;       /* file pointer                                 */
    char    command[CMLEN];	/* command buffer                               */
    long    numfile = 1;    /* number of files to clean up                  */
    long    cnt2;           /* loop counter                                 */


    /* create file names
       ------------------*/
    strcpy(tempfile[0],"ticmark");
    c_creat_name(tempfile[0],"txt");
    strcpy(tempfile[1],"ticmark");
    c_creat_name(tempfile[1],"dat");

    if ((temp_fp = fopen(tempfile[0],"w")) == NULL)
    {
        c_errmsg("Error opening file","ticmark-open",&non_fatal);
        return(E_FAIL);
    }

    /* get the host name of the user-symbol file
       ------------------------------------------*/
    if ((infp = fopen(infile,"r")) ==NULL)
    {
        c_errmsg("Error opening file","ticcmark-open",&non_fatal);
        return(E_FAIL);
    }


    /* Read the number of points in file
       ----------------------------------*/
    fscanf(infp,"%d \n", &cnt);

    /* Read the line, sample and intensity locations from the file
       and place determine which pixel locations need to be highlighted
       -----------------------------------------------------------------*/
    badtmark = 0;
    tmark = 0;
    for (cnt2=0;cnt2 < cnt;count = 0,cnt2++)
    {
        fscanf(infp,"%f %f %f\n", &temp_line, &temp_samp, &temp_bright);
        cline = (long)temp_line;
        csamp = (long)temp_samp;
        if (temp_bright < (float)mingrey)
            bright = mingrey;
        else
            if (temp_bright > (float)maxgrey)
                bright = maxgrey;
            else
                bright = (long) temp_bright;

        /* Do the scaling
           ---------------*/
        cline = (long) ((float)cline/scale);
        csamp = (long) ((float)csamp/scale);

        if ((cline > 0) && (csamp > 0))
        {
            tmark++;
            if ((print[TERM]) || (print[PFILE]) || (print[LP]))
                badtmark += prtreport(fp,tmark,cline,csamp,"","EMBEDPT",0.0,0.0,
                        window[0][NL],window[0][NS],0,print,FALSE);

            picpak (&cline, &csamp, &symbol, &bright, symdim, lines, samps,
                    zlevel, &count,&usr_line[cnt * WIDTH],
                    &usr_samp[cnt * WIDTH]);
        }

        /* sort the resulting line/sample coordinate values by line number and
           by sample number within each line.  this enables the 'engrave'
           routine to use sequential image i/o.
           ----------------------------------------------------------------- */
        ptsort (&count, lines, samps, zlevel);

        /* create temporary file storage
           ------------------------------*/
        for(j = 0;j < count;j++)
        {
            line = lines[j];
            samp1 = samps[j];
            samp2 = samps[j];

            /* Determine the minimum and maximum pixel locations for this point
               ---------------------------------------------------------------*/
            while (line == lines[j+1])
            {
                j++;
                if ((samp1 > samps[j]) && (samps[j] != 0))
                    samp1 = samps[j];
                if (samp2 < samps[j])
                    samp2 = samps[j];
            }
            /* Write to temporary file
               ------------------------*/
            fprintf(temp_fp,"%10d   %10d  %10d  %10d\n",line,samp1,
                    samp2,bright);
        }
    }
    fclose(temp_fp);

    /* Report on the number of ticmarks that are outside of image
       -------------------------------------------------------------*/
    prtreport(fp,0,0,0,"","",0.0,0.0,0,0,badtmark,print,TRUE);

    /* Create the command string to sort with
       ---------------------------------------*/
    c_errmsg("Sorting the grid file","informational",&non_fatal);
    sprintf(command,"sort -n %s -o %s",tempfile[0],tempfile[1]);
    result = system(command);
    if (result != E_SUCC)
    {
        c_errmsg("System file sort failed","ticmark-system",&non_fatal);
        c_clean(vblk,"ticmark",tempfile,&numfile);
    }

    /* Delete the unsorted select file
       ----------------------------------*/
    c_delfil(tempfile[0]);

    return(E_SUCC);
}
