/**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications.

PROGRAM HISTORY:
VERSION	DATE       AUTHOR         REQUEST
-------	----       ------         -------
1.0     05/90      W. Xia         Conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
      Runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
   Open input image files.
   Open the output image file.                                               
   Update the ddr file.
   Group all the images files into a group file
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ticmark.h"

int filspec
(
    long *num_inimages,             /* I: Number of input images */
    long num_image_bands[],         /* I: Number of bands in each input image */
    long image_band_num[][MAXBND+1],/* I: Band numbers in each input image */
    long num_outbands,              /* I: Number of output bands */
    long out_type,                  /* I: Output image data type */
    long out_img_bands[][MAXBND+1], /* O: Band numbers for the output image */
    long *nlines,                   /* I: Number of lines in mask image */
    long *nsamps,                   /* I: Number of samples in mask image */
    struct DDR *ddr,                /* O: DDR structure */
    struct PARBLK *block,           /* I: TAE parameter block */
    struct FDESC *fdescin[],        /* I/O: Input file descriptors */
    struct FDESC **fdescout,        /* I: Output file descriptors */
    long (*window)[4],              /* I/O: Input image dimen. (SL SS NL NS) */
    char hosout[][CMLEN],           /* I: Output image host name */
    char (*taein)[CMLEN],           /* I: Input image TAE name */
    char (*hosin)[CMLEN]            /* I: Input image host name */
)
{

    double	line_inc = 1.0,  /* line and sample increments for */
        samp_inc = 1.0;	     /* output image ddr */

    long access,             /* file access type */
        maxlines=1,	         /* number of lines to read at a time */
        comb_flag=0,         /* image combination flag */
        i,n,                 /* loop index */
        opt[2],              /* optional parameter array for output file */
        stat,                /* return status */
        type;                /* file type */
    long zero = 0;           /* value of zero */
    long one = 1;            /* value of one */

    /* Open input image files.                       
       -------------------------*/
    type = EREAL;     
    if ( strlen(taein[0]) !=0 )
    {  
        access = IREAD;                                                       
        for (n = 0; n < *num_inimages; n++)
        {  
            stat = c_eopenr( &fdescin[n], hosin[n], image_band_num[n],
                    &window[n][SL], &window[n][SS], &window[n][NL], 
                    &window[n][NS], &access, &type, &zero, &maxlines);            
            if (stat != E_SUCC )
            {
                for (i = 0; i < n; i ++)
                    c_eclose(&fdescin[i]);
                return(E_FAIL);
            }
        }
        fdescin[++n] = 0;
    }

    /* Open the output image file.
       -------------------------- */
    access = IWRITE;

    opt[0]=out_type;             
    opt[1]=0;

    if ( strlen(taein[0]) == 0)
    {
        opt[0]=EBYTE;
        window[0][NS] = *nsamps;
        window[0][NL] = *nlines;
    }  

    stat = c_eopenr (fdescout, hosout[0], &num_outbands, &one, &one,
            &window[0][NL], &window[0][NS], &access, &type, opt,
            &maxlines);
    if (stat != E_SUCC)
        c_abort (block, "Error opening output image", "ticmark-eopenr",
                "ticmark", hosout, &one);

    for (n = 0; n < num_outbands; n++)
        out_img_bands[0][n] = n + 1;

    /* Update the ddr records.
       ------------------------ */
    if (strlen (taein[0]) != 0)
    {                                                            
        stat = c_upddr (block, hosin, hosout[0], num_inimages, window,
                image_band_num, num_image_bands, &line_inc, &samp_inc,
                &num_outbands, &comb_flag);
        if (stat != E_SUCC )
            c_errmsg ("Error updating DDR", "ticmark-upddr",&one);
    }
    else
        update_ddr(ddr,hosout[0]);

    return(E_SUCC);
}
