/*****************************************************************************
NAME:  GETPAR

PURPOSE:  Getpar gets parameters from tae

program history:
VERSION	DATE       AUTHOR            REQUEST
-------	----	   ------            -------
1.0     5/86       D. Steinwand      Prf# 2753,3459
2.0     5/90       W. Xia            Conversion to C
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "ticmark.h"

int getpar
(
    long *nullfi,                   /* O: Null input file flag */
    long *ornull,                   /* O: Null origin (start) coord flag */
    long *num_inbands,              /* O: Total number of input image bands */
    long *bckgnd,                   /* O: Backgound value for mask image */
    char infile[],                  /* O: User defined symbol file name */
    char mode[],                    /* O: Mode of operation--subcmd name */
    double start[2],                /* O: Starting coordinate of first tic */
    double *freq,                   /* O: Frequency of tics */
    long symdim[],                  /* O: Dimensions of symbol */
    long *bright,                   /* O: Grey level of symbol */
    char coform[],                  /* O: Coordinate format */
    long *num_inimages,             /* O: Number of input images */
    long num_image_bands[],         /* O: Number of bands in each input image */
    long *num_outbands,             /* O: Number of output image bands */
    long *out_type,                 /* O: Output data type */
    struct PARBLK *block,           /* I: TAE parameter block */
    long *min_max,                  /* O: Minmax option flag */
    long *msgtim,                   /* O: Message interval */
    long (*window)[4],              /* O: Input image dimen. (SL SS NL NS) */
    long (*image_band_num)[MAXBND+1], /* O: Band numbers for each image */
    char (*taeout)[CMLEN],          /* O: Output image tae name */
    char hosout[][CMLEN],           /* O: Output image host name */
    char (*taein)[CMLEN],           /* O: Input image tae name */
    char hosin[][CMLEN],            /* O: Input image host name */
    long *in_def,                   /* O: Input image flag */
    long *grid_flg,                 /* O: Grid flag */
    float *scale,                   /* O: Scale factor */
    long *mingrey,                  /* O: Minimum grey value */
    long *maxgrey                   /* O: Maximum grey value */
 )
{

    struct	VARIABLE *var;	        /* pointer to a variable in TAE block 	*/

    long    three = 3;              /* value of three */
    long    two = 2;                /* value of two	*/
    long    non_fatal = 1;          /* nonfatal error return status */
    static
        long    len=7, 
        imglimit=MAXIMG,            /* maximum number of image allowed */
        inimg_flag[MAXIMG][2],      /* flag for subwindow */
        same_wnd,                   /* flag indication if images have same 
                                       window */
        itype=ELONG,                /* value of long */
        dotype=EDOUBLE,             /* value of double */
        stat,                       /* return status */
        std_out,                    /* standard output for inlas */
        access;                     /* access type */
    long    zero = 0;               /* constant of zero	*/
    long    one = 1;                /* constant of one */

    static char  mnmx[4];           /* minimum and maximum option*/
    char	grid[4];                /* variable to receive grid value */

    /*initialize a Las program
      ----------------------- */
    c_inlas(block, &std_out);

    /* get the input image name                       
       ------------------------ */
    stat= c_getin (block, "IN", &zero, taein, hosin, num_inimages,
            window, image_band_num, num_image_bands, num_inbands,
            &imglimit, &same_wnd, inimg_flag);   
    if (stat != E_SUCC)
        return (E_FAIL);
    if (*num_inimages > 0)
        *in_def = TRUE;

    if ( strlen (taein[0]) != 0 )
    {
        *nullfi = FALSE;
        if (same_wnd == 0)                                  
        {
            c_errmsg ("Image windows differ in size", "ticmark-window", 
                      &non_fatal);
            return(E_FAIL);
        }

        *num_outbands = *num_inbands;            
        *bckgnd=0;

        /* get the data type of input image
           -------------------------------- */
        if (*num_inimages > 0)
            stat = c_gettyp(&hosin[0],&one,out_type);  
        if (stat != E_SUCC)
            return(E_FAIL);
    }
    else
    {
        c_getnum(block, "BACKGRND", &one, &itype, (void *)bckgnd);
        *nullfi = TRUE;
        *num_inbands = 1;   
        *num_outbands = 1;
        *out_type=EBYTE;
    }

    /* Get the output image file name.
       ------------------------------- */
    stat = c_getout (block, "OUT", taeout, hosout);
    if (stat != E_SUCC)
        return(E_FAIL);

    c_getstr (block, "_SUBCMD", &one, &len, mode);

    /* get parameters not associated with the -embedpts subcommands
       -------------------------------------------------------------*/  
    /* geographic units
       -------------------- */

    if ((strcmp(mode,"EMBEDPT") != 0))
    {
        var = p_find (block, "GEOUNITS");
        strcpy ( coform, SVAL (*var, 0));
    }

    /* get parameters not associated with the -uniform subcommands
       -------------------------------------------------------------- */  
    if ((strcmp(mode,"UNIFORM") != 0))
    {
        access = IREAD;
        stat = c_getnam(block, "INFILE", &zero, infile, "txt",&access);
        if (stat != E_SUCC)                            
            return(E_FAIL);
    }              

    /* get parameters not associated with the -userdef subcommands
       ----------------------------------------------------------- */
    start[0] = 0.0;
    start[1] = 0.0;
    if ((strcmp(mode,"USERDEF") != 0) && (strcmp(mode,"EMBEDPT") != 0))
    {
        if(c_getnum (block, "ORIGIN", &two, &dotype, (void *)start) ==0)
            *ornull = FALSE;    
        if (start[0]==0 && start[1]==0) 
            *ornull = TRUE;

        c_getnum (block, "TICDIST", &one,  &dotype, (void *)freq);
        c_getnum (block, "TICGREY", &one,  &itype, (void *)bright);
        c_getnum (block, "TICSIZ", &three, &itype, (void *)symdim); 
        if ((symdim[2] > symdim[0]) || (symdim[2] > symdim[1])) 
        {
            c_errmsg ("Invalid ticmark dimension","ticmark-dim", &non_fatal);
            return(E_FAIL);
        }
    }  /* if */                                                

    if (strcmp(mode,"EMBEDPT") == 0)
    {
        c_getnum (block, "TICSIZ", &three, &itype, (void *)symdim); 
        if ((symdim[2] > symdim[0]) || (symdim[2] > symdim[1])) 
        {
            c_errmsg ("Invalid ticmark dimension","ticmark-dim", &non_fatal);
            return(E_FAIL);
        }

        var = p_find( block, "SCALFACT");
        *scale = RVAL( *var, 0);
        var = p_find( block, "MINVAL");
        *mingrey = IVAL( *var, 0);
        var = p_find( block, "MAXVAL");
        *maxgrey = IVAL( *var, 0);
    }

    /* get grid flag
       --------------*/
    if (strcmp(mode,"EMBEDPT") != 0)
    {
        var = p_find( block, "GRIDFLG");
        strcpy( grid, SVAL( *var, 0));
        if (grid[0] == 'Y') 
            *grid_flg = TRUE;
        else
            *grid_flg = FALSE;

    }

    /* Get las global parameters
       -------------------------*/
    var = p_find( block, "$msgtim");
    *msgtim = IVAL( *var, 0);

    var = p_find( block, "$MINMAX");
    strcpy( mnmx, SVAL( *var, 0));
    if (mnmx[0] == 'Y') 
        *min_max = TRUE;
    else 
        *min_max = FALSE;
    return(E_SUCC);
}
