/*****************************************************************************
NAME: LINE_SORT 

PURPOSE: Line_sort sorts the pixel values for the grid and writes these
         to a temprorary file

PROGRAM HISTORY:
VERSION    DATE    AUTHOR        REQUEST
-------    ----    ------        -------
1.0        5/92    T. Mittan     Convert to 5.0
*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "desc.h"
#include "ticmark.h"

int line_sort
(
    struct PARBLK *block,          /* I: TAE parameter block                  */
    long grid_line[],              /* I: Grid array of line numbers           */
    long grid_samp[],              /* I: Grid array of sample numbers         */
    long width,                    /* O: Uniform tic width                    */
    char tempfile[][CMLEN],        /* O: Temporary sort file names            */
    long usr_line[],               /* I: User array of line numbers           */
    long usr_samp[],               /* I: User array of sample numbers         */
    long usr_width[],              /* I: User array of widths                 */
    long usr_intensity[],          /* I: User array of intensities            */
    long *nlines,                  /* I: Number of lines                      */
    long *nsamps,                  /* I: Number of samples                    */
    long *bright                   /* I: Uniform tic intensity                */
)
{
    long    cnt,i,j,k;              /* loop counters                        */
    long    col;                    /* column increment                     */
    long    col_minus;              /* previous column increment            */
    long    col_plus;               /* next column increment                */
    long    extra_nl;               /* extra number of lines                */      
    long    extra_samp;             /* extra number of lines                */      
    long    nl;                     /* number of lines                      */
    long    ns;                     /* number of samples                    */
    long    temp_samp;              /* starting sample                      */
    long    temp_samp1;             /* ending sample                        */
    long    first;                  /* first flag                           */
    long    last;                   /* last flag                            */
    long    numfile = 2;            /* number of temporary files            */
    long    non_fatal = 1;          /* nonfatal flag value                  */
    long    result;                 /* status of sort routine               */
    long    row;                    /* row increment                        */
    long    row_minus;              /* previous row increment               */
    long    row_plus;               /* next row increment                   */
    double  slope;                  /* slope of the line                    */
    char    command[CMLEN];         /* command string for unix system sort  */
    FILE    *fp;                    /* file pointer                         */

    /* create file names
       ------------------*/
    strcpy(tempfile[0],"ticmark");
    c_creat_name(tempfile[0],"txt");
    strcpy(tempfile[1],"ticmark");
    c_creat_name(tempfile[1],"dat");

    if ((fp = fopen(tempfile[0],"w")) == NULL)
    {
        c_errmsg("Error opening file","ticmark-open",&non_fatal);
        return(E_FAIL);
    }
    width--;
    /* determine starting sample, ending sample and line number
       ---------------------------------------------------------*/
    for (i = 0; i < NLTIC; i++)
    {
        row_minus = (i - 1) * NSTIC * WIDTH;
        row_plus = (i + 1) * NSTIC * WIDTH;
        row = i * NSTIC * WIDTH;
        for (cnt = 0; cnt < NSTIC; cnt++)
        {
            col = cnt * WIDTH;
            col_minus = (cnt - 1) * WIDTH;
            col_plus  = (cnt + 1) * WIDTH;
            if ((grid_line[row + col] > 0 ) && (grid_samp[row + col] > 0))
            {
                /* Find vertical lines if first or last line extend to edge 
                   of image
                   -------------------------------------------------------*/
                if((i == 0) || ((grid_line[row_minus + col] <= 0) &&
                        (grid_samp[row_minus + col] <= 0)))
                    first = TRUE;
                else
                    first = FALSE;
                if((i == 99) || ((grid_line[row_plus + col] <= 0) &&
                        (grid_samp[row_plus + col] <= 0))) 
                    last = TRUE;
                else
                    last = FALSE;
                if (last)
                    nl =  *nlines - grid_line[row + col] + 1;
                else
                    nl =  grid_line[row_plus + col] - grid_line[row + col];
                if (nl != 0)
                    if (last)
                        slope = (double)(grid_samp[row + col] -
                            grid_samp[row_minus+col])/
                            (double)(grid_line[row+col] -
                            grid_line[row_minus+col]);
                    else
                        slope = (double)(grid_samp[row_plus + col] -
                            grid_samp[row + col])/(double)(nl);
                else
                    slope = 0;
                if (first)
                {
                    extra_samp = (long)((double)grid_line[row + col] * 
                            slope + 0.5);
                    nl = grid_line[row_plus + col];
                    slope = (double)(grid_samp[row_plus + col] -
                            grid_samp[row + col] + extra_samp)/(double)(nl);
                }

                if (nl > *nlines)
                    nl = *nlines;

                /* write data to file
                   -------------------*/
                for (k = 0; k < nl; k++)
                {
                    if (first)
                    {
                        temp_samp = grid_samp[row + col] - extra_samp +
                            (long)(k * slope); 
                        temp_samp1 = grid_samp[row + col+ width] - extra_samp +
                            (long)(k * slope);
                        fprintf(fp,"%10d   %10d  %10d  %10d\n",k + 1,temp_samp,
                                temp_samp1,*bright);
                    }
                    else
                    {
                        temp_samp = grid_samp[row + col] + (long)(k * slope); 
                        temp_samp1 = grid_samp[row + col + width] +
                            (long)(k * slope);
                        fprintf(fp,"%10d   %10d  %10d  %10d\n",
                                grid_line[row+col]+ k, temp_samp,temp_samp1,
                                *bright);
                    }
                }
                /* Find horizontal lines if first or last point extend to 
                   image edge
                   ------------------------------------------------------*/
                if((cnt == 0) || ((grid_line[row + col_minus] <= 0) &&
                        (grid_samp[row + col_minus] <= 0)))
                    first = TRUE;
                else
                    first = FALSE;
                if((i == 99) || ((grid_line[row + col_plus] <= 0) &&
                        (grid_samp[row + col_plus] <= 0))) 
                    last = TRUE;
                else
                    last = FALSE;
                if (last)
                {
                    nl =  grid_line[row + col_minus] - grid_line[row + col];
                    ns =  grid_samp[row + col] - grid_samp[row + col_minus];
                }
                else
                {
                    nl =  grid_line[row + col] - grid_line[row + col_plus];
                    ns =  grid_samp[row + col_plus] - grid_samp[row + col];
                }
                if (nl < 0)
                    nl = -nl;
                else
                    if (nl == 0)
                        nl = 1;
                slope = (double)(ns)/(double)(nl);
                if (first)
                {
                    if (slope > 0)
                        extra_nl = (long)((double)(grid_samp[row + col])/
                                (double)slope+ .5);
                    else
                        extra_nl = 0;
                    nl += abs (extra_nl);
                    slope = ((double) (grid_samp[row + col_plus])/(double)(nl));
                }
                else
                    if (last)
                    {
                        ns = *nsamps - grid_samp[row + col];
                        extra_nl =  (long)((double)ns/slope + 0.5);
                        nl += abs(extra_nl);
                    }

                if (nl > *nlines)
                    nl = *nlines;

                /* write data to file
                   -------------------*/
                for (j = 0; j <= width; j++)
                    for (k = 0; k < nl; k++)
                    {
                        if (first)
                        {
                            temp_samp  = (long)(slope * k + 0.5);
                            temp_samp1 = (long)(0.5 + (k + 1) * slope);
                        }
                        else
                        {
                            temp_samp  = grid_samp[row + col] + 
                                (long)(slope * k + 0.5);
                            temp_samp1 = grid_samp[row + col + j] +
                                (long)(0.5 + (k + 1) * slope);
                        }
                        if (first)
                        {
                            if ((grid_line[row+col+j]+ k - extra_nl) > 0)
                                fprintf(fp,"%10d   %10d  %10d  %10d\n",
                                       grid_line[row+col+j]+ k
                                       - extra_nl,temp_samp,temp_samp1,*bright);
                        }
                        else
                            fprintf(fp,"%10d   %10d  %10d  %10d\n",
                                    grid_line[row+col+j]+k,temp_samp,
                                    temp_samp1,*bright);
                    }
            }
        }
    }
    /* Find the lines and samples for the user-defined grid.
       Because we do not know which points lie on a lat/lon line,
       we assume that all points are independent and draw a 
       verticle and a horizotal line from eac point. As these lines
       get farther from the initial point they will become inacurate.
       ---------------------------------------------------------------*/
    for (i = 0; i < MAXCNT; i++)
    {
        col = i * WIDTH;
        if ((usr_line[col] > 0) && (usr_samp[col] > 0))
        {
            for (j = 0; j < usr_width[i]; j++)
            {
                if (usr_line[col + j] > 0)
                    fprintf(fp,"%10d   %10d  %10d  %10d\n",usr_line[col + j],
                            0,*nsamps,usr_intensity[i]);
                if (usr_samp[col + j] > 0)
                    for (k = 1; k <= *nlines; k++)
                        fprintf(fp,"%10d   %10d  %10d  %10d\n",k,
                                usr_samp[col+j], usr_samp[col+j] ,
                                usr_intensity[i]);
            }
        }
    }
    fclose(fp);

    c_errmsg("Sorting the grid file","informational",&non_fatal);

    /* Create the command string to sort with
       ---------------------------------------*/
#ifdef unix 
    sprintf(command,"sort -n %s -o %s",tempfile[0],tempfile[1]);
    result = system(command);
    if (result != E_SUCC)
    {
        c_errmsg("System file sort failed","ticmark-system",&non_fatal);
        c_clean(block,"ticmark",tempfile,&numfile);
    }
#endif
#ifdef vms
    infile.addr = &tempfile[0];
    outfile.addr = &tempfile[1];
    infile.length = strlen(tempfile[0]);
    outfile.length = strlen(tempfile[1]);
    prmsrt (&infile,&outfile,&ncol,&result);
    if (result != 1)
    {
        c_errmsg("System file sort failed","ticmark-system",&non_fatal);
        c_clean(block,"ticmark",tempfile,&numfile);
    }
#endif

    /* Delete the unsorted select file
       ----------------------------------*/
    c_delfil(tempfile[0]);

    return(E_SUCC);
}
