/*****************************************************************************
NAME: PRTREPORT
 
PURPOSE: Prints a report of the location of the ticmarks.  This report is
           printed to the terminal, file, or printer.
         
PROGRAM HISTORY:
VERSION  DATE       AUTHOR         REQUEST
-------  ----       ------         -------
1.0      3/99       T.Ratliff      Initial development

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "ticmark.h"

int prtreport
(
    FILE *fp,                  /* I: Pointer to report file */
    long tmark,                /* I: Ticmark counter */
    long line,                 /* I: Ticmark line coord */
    long samp,                 /* I: Ticmark sample coord */
    char *unit,                /* I: Unit coords are in */
    char *subcmd,              /* I: Ticmark subcommand */
    double coor1,              /* I: Ticmark coord */
    double coor2,              /* I: Ticmark coord  */
    long lmax,                 /* I: Max number of lines */
    long smax,                 /* I: Max number of sample  */
    long num_badtmark,         /* I: Number of invalid tmarks */
    long print[],              /* I: Print option, term,lp,.. */
    long finalflg              /* I: Flag to print # badtmarks */
)
{
    long bad_flg = FALSE;                  /* Flad indicating a bad tmark   */
    long badtmark = 0;                     /* Bad ticmarks                  */
    long deg[2];                           /* Degrees from DMS coords       */
    long min[2];                           /* Minuets from DMS coords       */
    long sec[2];                           /* Seconds from DMS coords       */
    char line1[CMLEN];                     /* Used to write report to file  */
    char line2[CMLEN];                     /* Used to write report to file  */
    char tmpstr[CMLEN];                    /* Temporary string variable     */
    void breakdms(double,long *,long *,long *);


    /* If subcmd is not EMBEDPT, then report line/sample
       and other coords in units specified by units.
       ----------------------------------------------------*/
    if ((strcmp(subcmd,"EMBEDPT") != 0) && (!finalflg))
    {
        if ((strcmp(unit,"MIN") == 0) || (strcmp(unit,"SEC") == 0))
        {
            /* If in MIN or SEC, coord1, coord2 are in DMS,
               break DMS coords out to get MIN or SEC
               -----------------------------------------------*/
            breakdms(coor1,&deg[0],&min[0],&sec[0]);
            breakdms(coor2,&deg[1],&min[1],&sec[1]);
            if (strcmp(unit,"MIN") == 0)
                sprintf(tmpstr,"DEG:MIN:(%ld:%ld / %ld:%ld)",deg[0],min[0],
                        deg[1], min[1]);
            else if (strcmp(unit,"SEC") == 0)
                sprintf(tmpstr,"DEG:MIN:SEC:(%ld:%ld:%ld / %ld:%ld:%ld)",
                        deg[0],min[0], sec[0],deg[1],min[1],sec[1]);
        }

        /* If the line or sample is greater than the maximum
           lines or samples in the image, increment badtmark
           ----------------------------------------------------*/
        if ((line > lmax) || (samp > smax))
        {
            badtmark++;
            bad_flg = TRUE;
        }

        /* Printing to terminal
           -----------------------*/
        if (print[TERM])
        {
            /* If a bad ticmark, print appropriate message
               ----------------------------------------------*/
            if (bad_flg)
            {
                c_lasmsg(FALSE,"","*TMark #%ld-> L/S:(%ld / %ld)",tmark,
                        line,samp);
                if ((strcmp(unit,"MIN") != 0) && (strcmp(unit,"SEC") != 0))
                    c_lasmsg(FALSE,"","          -> %s:(%lf / %lf)",unit,coor1,
                            coor2);
                else
                    c_lasmsg(FALSE,"","          -> %s",tmpstr);
            }
            else
            {
                c_lasmsg(FALSE,"","TMark #%ld-> L/S:(%ld / %ld)",tmark,
                         line,samp);
                if ((strcmp(unit,"MIN") != 0) && (strcmp(unit,"SEC") != 0))
                    c_lasmsg(FALSE,"","         -> %s:(%lf / %lf)",unit,coor1,
                            coor2);
                else
                    c_lasmsg(FALSE,"","         -> %s",tmpstr);
            }
        }

        /* Printing to a file or the printer
           ------------------------------------*/
        if ((print[PFILE]) || (print[LP]))
        {
            if (bad_flg)
            {
                sprintf(line1," *TMark #%ld-> L/S:(%ld / %ld)",tmark,
                        line,samp);
                if ((strcmp(unit,"MIN") != 0) && (strcmp(unit,"SEC") != 0))
                    sprintf(line2,"           -> %s:(%lf / %lf)",unit,
                            coor1,coor2);
                else
                    sprintf(line2,"           -> %s",tmpstr);
            }
            else
            {
                sprintf(line1," TMark #%ld-> L/S:(%ld / %ld)",tmark,
                        line,samp);
                if ((strcmp(unit,"MIN") != 0) && (strcmp(unit,"SEC") != 0))
                    sprintf(line2,"          -> %s:(%lf / %lf)",unit,
                            coor1,coor2);
                else
                    sprintf(line2,"        -> %s",tmpstr);
            }
            fprintf(fp,"%s\n",line1);
            fprintf(fp,"%s\n",line2);
        }
    }
    /* In the EMBEDPT subcommand.  Display the
       line and sample coords only
       ------------------------------------------*/
    else if (!finalflg)
    {
        /* If line or sample is greater than max lines or
           samples in image, increment badtmark
           -------------------------------------------------*/
        if ((line > lmax) || (samp > smax))
        {
            badtmark++;
            bad_flg = TRUE;
        }

        /* Printing to terminal
           -----------------------*/
        if (print[TERM])
        {
            if (bad_flg)
                c_lasmsg(FALSE,"","*TMark #%ld-> L/S:(%ld / %ld)",tmark,
                         line,samp);
            else
                c_lasmsg(FALSE,"","TMark #%ld-> L/S:(%ld / %ld)",tmark,
                         line,samp);
        }

        /* Printing to file or printer
           ------------------------------*/
        if ((print[PFILE]) || (print[LP]))
        {
            if (bad_flg)
                sprintf(line1," *TMark #%ld-> L/S:(%ld / %ld)",tmark,
                        line,samp);
            else
                sprintf(line1," TMark #%ld-> L/S:(%ld / %ld)",tmark,
                        line,samp);
            fprintf(fp,"%s\n",line1);
        }
    }
    /* Print the number of invalid ticmarks
       ---------------------------------------*/
    else
    {
        /* Printing to terminal
           -----------------------*/
        if (print[TERM])
            c_lasmsg(FALSE,"","Number of INVALID ticmarks: %ld",num_badtmark);

        /* Printing to file or printer
           ------------------------------*/
        if ((print[PFILE]) || (print[LP]))
        {
            sprintf(line1," Number of INVALID ticmarks: %ld",
                    num_badtmark);
            fprintf(fp,"%s\n",line1);
        }
    }
    return(badtmark);
}

void breakdms
(
    double angle,
    long *deg,
    long *min,
    long *sec
)

{
    if (angle < 0)
        angle *= -1;

    *deg = (long)(angle / 1000000);
    angle -= (*deg * 1000000);
    *min = (long)(angle / 1000);
    *sec = (long)angle - (*min * 1000);
}
