#ifndef TICMARK_H
#define TICMARK_H

/* Defines and structure definitions for ticmark
------------------------------------------------*/
#define TIC     1       /* symbol_id flag for ticmarks */
#define ULT     2       /* symbol_id flag for upper left tic */
#define URT     3       /* symbol_id flag for upper right tic */
#define LLT     4       /* symbol_id flag for lower left tic */
#define LRT     5       /* symbol_id flag for lower right tic */

#define MAXCNT  75000
#define NLTIC   100
#define NSTIC   100
#define WIDTH   60

#define DMOD(A,B)       (A) - ((long)((A) / (B)) * (B))

#define TERM 0
#define PFILE 1
#define LP 2

IMPORT  long    curent;                        /* current entity           */
IMPORT  long    curcat;                        /* current category         */

struct PROJ
   {
   long proj_code;              /* Projection code number */
   long units;                  /* Projection units code number */
   long zone_code;              /* Projection zone code (UTM or State Plane) */
   long datum_code;             /* Projection datum code */
   long ns;                     /* Number of samples in image */
   long nl;                     /* Number of lines in image */
   double proj_coef[15];        /* 15 projection "coefficients" */
   };

struct FRAME
   {
   double tol;          /* Tolerance for linear approximation--grid reduce */
   double corners[8];   /* Resulting grid projection corners */
   double o_dist[2];    /* Projection distance in y & x; input space */
   long window[4];      /* Window parameters for output space */
   long ngline;         /* Number of geometric mapping grid lines (rows) */
   long ngsamp;         /* Number of geometric mapping grid samples (cols) */
   long convert;        /* Boolean--use Projection Xform Package to convert? */
  char name[CMLEN];    /* Name of the Geometric Mapping Grid file */
   FILE *fptr;          /* File pointer to print file */
   };

/* Function prototypes for ticmark
----------------------------------*/

void main
(
  void
);

/* embedpt.c */
int embedpt
(
struct PARBLK *vblk,    /* I: TAE parameter block */
char *infile,           /* I: File with user defined symbol locations */
char tempfile[][CMLEN],       /* O: Temporary sort file names */
long lines[],                 /* I: Lines array for ticmarks */
long samps[],                 /* I: Samps array for ticmarks */
long zlevel[],                /* I: Intensity array for ticmarks */
long usr_line[],              /* I: User specified line number array */
long *usr_samp,               /* I: User specified sample number array */
long symdim[3],               /* I: Symbol/tic dimensions (h,l,w) */
float scale,                  /* I: Scale factor */
long mingrey,                 /* I: Minimum grey value */
long maxgrey,                 /* I: Maximum grey value */
FILE *fp,                     /* I: Pointer to the report file */
long print[],                 /* I: Array telling where to print report */
long (*window)[4]             /* I: Window from input image */
);

/* engrave.c */
int engrave
(
long nullfi,                  /* I: Null input file flag */
long nlines,                  /* I: Number of lines in mask image */
long nsamps,                  /* I: Number of samples in mask image */
long bckgnd,                  /* I: Background value for mask image */
long (*image_band_num)[MAXBND+1], /* I: Band numbers selected for each image */
long num_image_bands[],       /* I: Number of bands in each input image */
long *num_inimages,           /* I: Number of images */
long num_inbands,             /* I: Number of bands */
long lines[],                 /* I: Line buffer of ticmarks */
long samps[],                 /* I: Sample buffer of ticmarks */
long zlevel[],                /* I: Brightness level of ticmarks */
float *outbuf,                /* O: Image output buffer */
struct FDESC *fdescin[],      /* I: Input file descriptors */
struct FDESC **fdescout,      /* I: Output file descriptors */
long *msgtim,                 /* I: Message interval */
long (*window)[4],            /* I/O: Input image dimensions (SL SS NL NS) */
long *grid_flg,               /* I: Grid flag */
char tempfile[][CMLEN]        /* I: Temp file name */
);

/* filspec.c */
int filspec
(
long *num_inimages,               /* I: Number of input images */
long num_image_bands[],           /* I: Number of bands in each input image */
long image_band_num[][MAXBND+1],  /* I: Band numbers in each input image */
long num_outbands,                /* I: Number of output bands */
long out_type,                    /* I: Output image data type */
long out_img_bands[][MAXBND+1],   /* O: Band numbers for the output image */
long *nlines,                     /* I: Number of lines in mask image */
long *nsamps,                     /* I: Number of samples in mask image */
struct DDR *ddr,                  /* O: DDR structure */
struct PARBLK *block,             /* I: TAE parameter block */
struct FDESC *fdescin[],          /* I/O: Input file descriptors */
struct FDESC **fdescout,          /* I: Output file descriptors */
long (*window)[4],                /* I/O: Input image dimen. (SL SS NL NS) */
char hosout[][CMLEN],             /* I: Output image host name */
char (*taein)[CMLEN],       /* I: Input image TAE name */
char (*hosin)[CMLEN]        /* I: Input image host name */
);


/* get_proj_prm.c */
int get_proj_prm
(
char *proj_name,        /* I: Projection parameter file name */
char *inkey,            /* I: Input projection parameter key */
long *in_def,                 /* I: Flag for input image */
struct PROJ *inp              /* O: Input projection parameters */
);

/* getpar.c */
int getpar
(
long *nullfi,                       /* O: Null input file flag */
long *ornull,                       /* O: Null origin (start) coord flag */
long *num_inbands,                  /* O: Total number of input image bands */
long *bckgnd,                       /* O: Backgound value for mask image */
char infile[],                      /* O: User defined symbol file name */
char mode[],                        /* O: Mode of operation--subcmd name */
double start[2],                    /* O: Starting coordinate of first tic */
double *freq,                       /* O: Frequency of tics */
long symdim[],                      /* O: Dimensions of symbol */
long *bright,                       /* O: Grey level of symbol */
char coform[],                      /* O: Coordinate format */
long *num_inimages,                 /* O: Number of input images */
long num_image_bands[],             /* O: Number of bands in each input image */
long *num_outbands,                 /* O: Number of output image bands */
long *out_type,                     /* O: Output data type */
struct PARBLK *block,               /* I: TAE parameter block */
long *min_max,                      /* O: Minmax option flag */
long *msgtim,                       /* O: Message interval */
long (*window)[4],                  /* O: Input image dimen. (SL SS NL NS) */
long (*image_band_num)[MAXBND+1],   /* O: Band numbers for each image */
char (*taeout)[CMLEN],              /* O: Output image tae name */
char hosout[][CMLEN],               /* O: Output image host name */
char (*taein)[CMLEN],               /* O: Input image tae name */
char hosin[][CMLEN],                /* O: Input image host name */
long *in_def,                       /* O: Input image flag */
long *grid_flg,                     /* O: Grid flag */
float *scale,                       /* O: Scale factor */
long *mingrey,                      /* O: Minimum grey value */
long *maxgrey                       /* O: Maximum grey value */
);

/* line_sort.c */
int line_sort
(
struct PARBLK *block,          /* I: TAE parameter block                  */
long grid_line[],              /* I: Grid array of line numbers           */
long grid_samp[],              /* I: Grid array of sample numbers         */
long width,                    /* O: Uniform tic width                    */
char tempfile[][CMLEN],        /* O: Temporary sort file names            */
long usr_line[],               /* I: User array of line numbers           */
long usr_samp[],               /* I: User array of sample numbers         */
long usr_width[],              /* I: User array of widths                 */
long usr_intensity[],          /* I: User array of intensities            */
long *nlines,                  /* I: Number of lines                      */
long *nsamps,                  /* I: Number of samples                    */
long *bright                   /* I: Uniform tic intensity                */
);

/* picpak.c */
void picpak
(
long *center_line,       /* I: Center line coordinate of symbol */
long *center_sample,     /* I: Center sample coordinate of symbol */
long *symbol_id,         /* I: Symbol type */
long *bright,            /* I: Grey-level of symbol in output image */
long symdim[],           /* I: Height, Length, and Thickness of each symbol */
long *line_buf,          /* O: Buffer of symbol line coordinates */
long *sample_buf,        /* O: Buffer of symbol sample coordinates */
long *zlevel_buf,        /* O: Buffer of symbol grey-level values */
long *index,             /* I/O: Number of pixels in symbols--buffer index */
long grid_line[],        /* O: Grid line index */
long grid_samp[]         /* O: Grid samp index */
);

/* prtreport.c */
int prtreport
(
FILE *fp,                  /* I: Pointer to report file */
long tmark,                /* I: Ticmark counter */
long line,                 /* I: Ticmark line coord */
long samp,                 /* I: Ticmark sample coord */
char *unit,                /* I: Unit coords are in */
char *subcmd,              /* I: Ticmark subcommand */
double coor1,              /* I: Ticmark coord */
double coor2,              /* I: Ticmark coord  */
long lmax,                 /* I: Max number of lines */
long smax,                 /* I: Max number of sample  */
long num_badtmark,         /* I: Number of invalid tmarks */
long print[],              /* I: Print option, term,lp,.. */
long finalflg              /* I: Flag to print # badtmarks */
);

/* ptsort.c */
void ptsort
(
long *n,                   /* I: Number of elements */
long *keyarr,              /* I/O: Key array to sort on */
long *arr1,                /* I/O: Array to rearrange */
long *arr2                 /* I/O: Array to rearrange */
);

/* swap.c */
void swap
(
long *one,                 /* I/O: Element to switch */
long *two                  /* I/O: Element to switch */
);

/* ticloc.c */
int ticloc
(
struct PARBLK *vblk,  /* I: TAE parameter block */
char *infile,         /* I: file containing user defined symbol locations */
char mode[],          /* I: subcommand option */
double freq,          /* I/O: frequency of tics--same as lat/long fields */
double start[2],      /* I/O: coordinate of first tic */
long symdim[3],       /* I/O: symbol/tic dimensions (h,l,w) */
long bright,          /* I: symbol brightness value */
char coform[],        /* I: format of lat/long coordinate fields */
long ornull,          /* I: origin for uniform tics given was null */
long *lmax,           /* O: number of lines for null input image */
long *smax,           /* O: number of samples for null input image */
long lines[],         /* I: lines array for ticmarks */
long samps[],         /* I: samps array for ticmarks */
long zlevel[],        /* I: intensity array for ticmarks */
struct DDR *ddr,      /* O: DDR structure */
char (*hosin)[CMLEN], /* I: input image host name */
long *in_def,         /* I: flag for input image */
long grid_line[],     /* I: grid line number array */
long grid_samp[],     /* I: grid sample number array */
long *grid_width,     /* O: width of grid */
long usr_line[],      /* I: user specified line number array */
long *usr_samp,       /* I: user specified sample number array */
long usr_width[],     /* O: user specified width  */
long usr_intensity[], /* O: user specified width */
FILE *fp,             /* I: Pointer to the report file */
long print[]          /* I: Print array telling where to print report */
);


/* update_ddr.c */
int update_ddr
(
struct DDR *ddr,               /* I: DDR structure */
char *hosout                   /* I: Output image host name */
);

/* use_ddr_info.c */
int use_ddr_info
(
struct PARBLK *vblk,           /* I: TAE parameter block */
struct PROJ *pro,              /* O: Struct containing projection parameters */
double *corners,               /* O: Corner projection coordinates */
long *image,                   /* O: Corner image coordinates */
long in_wind[][4],             /* O: Image window specification */
double *scaley,                /* O: Line pixel value */
double *scalex,                /* O: Sample pixel value */
long *pixflg                   /* I: Pixsize flag */
);

#endif
