/******************************************************************************
NAME: UPDATE_DDR

PURPOSE: Updates the DDR information for the output image if there is 
         no input image

PROGRAM HISTORY
VERSION  AUTHOR        DATE          REASON
-------  ------        -----         ------
1.0      B. Ailts      10/91         Original development

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "proj.h"
#include "ticmark.h"

int update_ddr
(
    struct DDR *ddr,        /* I: DDR structure */
    char  *hosout           /* I: Output image host name */
)
{
    struct DDR ddr_old;     /* DDR structure                        */
    struct BDDR bddr[MAXBND];/* Band DDR structure                  */

    long i;                 /* Looping variable                     */
    long band;              /* Band looping variable                */
    long status;            /* Status variable                      */
    long fatal = -1;        /* Fatal error code                     */

    /* Get the DDR information from the image file
       ----------------------------------------------*/
    status = c_getddr(hosout,&ddr_old);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","ticmark-fatal",&fatal);

    /* Initialize the validity flags
       --------------------------------*/
    for (i = 0; i < 8; i++)
        ddr_old.valid[i] = ddr->valid[i];

    /* Update the DDR projection values
       -----------------------------------*/
    ddr_old.master_line   = ddr->master_line;
    ddr_old.master_sample = ddr->master_sample;
    ddr_old.line_inc   = ddr->line_inc;
    ddr_old.sample_inc = ddr->sample_inc;
    ddr_old.datum_code = ddr->datum_code;

    ddr_old.zone_code = ddr->zone_code;
    ddr_old.proj_code = ddr->proj_code;
    for (i = 0;i < COEFCT;i++)
        ddr_old.proj_coef[i] = ddr->proj_coef[i];

    strcpy(ddr_old.proj_units,ddr->proj_units);

    ddr_old.pdist_y = ddr->pdist_y;
    ddr_old.pdist_x = ddr->pdist_x;

    /* Calculate projection coordinates based on full scene center
       --------------------------------------------------------------*/
    ddr_old.upleft[0] = ddr->upleft[0];
    ddr_old.upleft[1] = ddr->upleft[1];

    ddr_old.upright[0] = ddr->upright[0];
    ddr_old.upright[1] = ddr->upright[1];

    ddr_old.loright[0] = ddr->loright[0];
    ddr_old.loright[1] = ddr->loright[1];

    ddr_old.loleft[0] = ddr->loleft[0];
    ddr_old.loleft[1] = ddr->loleft[1];

    /* Update the Band DDR information
       ----------------------------------*/
    for (band = 1; band <= ddr_old.nbands; band++)
    {
        /* Get the Band DDR information from the image
           ---------------------------------------------- */
        status = c_getbdr(hosout,&bddr[band-1],&band);
        if (status != E_SUCC)
            c_errmsg("Fatal error encountered","ticmark-fatal",&fatal);
    }

    /* Put the updated DDR information to the image
       ---------------------------------------------- */
    status = c_putddr(hosout,&ddr_old);
    if (status != E_SUCC)
        c_errmsg("Fatal error encountered","ticmark-fatal",&fatal);

    /* Put the updated Band DDR information to the image
       ----------------------------------------------------*/
    for (band = 0; band < ddr_old.nbands; band++)
    {
        status = c_putbdr(hosout,&bddr[band]);
        if (status != E_SUCC)
            c_errmsg("Fatal error encountered","ticmark-fatal",&fatal);
    }

    return(E_SUCC);
}
