/******************************************************************************

NAME:	GETPAR

FUNCTION:  GETPAR retrieves the needed parameter values

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Retrieve the parameter OUTLT 
Create the subfile name
Retrieve the parameter FTYPE 
If FTYPE equals TPS
   Retrieve the input tie point selection parameter 
Else
   Retrieve the input tie point location parameter 
Endif

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

void getpar
(
    struct PARBLK *parb,		/* parameter block structure */
    char *hosin,			/* host input file name descriptor */
    char hosout[][CMLEN],		/* host output file name descriptor */
    char *ftype			/* labeled table type */
)

{ /* getpar */

struct VARIABLE *val;		/* pointer to variable */

long access;			/* type of access of file */
long fatal = -1;		/* fatal error code			*/
long status;			/* return status of function calls	*/
long zero = 0;			/* value of zero (0)			*/

/* Retrieve output labeled table file name.
-------------------------------------------*/
access = IWRITE;
status = c_getnam(parb,"OUTLT",&zero,hosout[0],"lt",&access);
if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tie2tab-fatal",&fatal);

/*  Create the file name of the subfile
---------------------------------------*/
strcpy(hosout[1],make_lt_name(hosout[0],1));

/*  Retrieve input file type.
-----------------------------*/
val = p_find(parb,"FTYPE");
strcpy (ftype,SVAL(*val,0));

/*  Retrieve input ascii file name.
-----------------------------------*/
access = IREAD;
if (strcmp(ftype,"TPS") == 0)
   status = c_getnam(parb,"INTIE",&zero,hosin,"tps",&access);
else
   status = c_getnam(parb,"INTIE",&zero,hosin,"tpl",&access);

if (status != E_SUCC)
   c_errmsg("Fatal error encountered","tie2tab-fatal",&fatal);

return;
} /* getpar */
