/******************************************************************************

NAME:	TIE2TAB

FUNCTION:  TIE2TAB converts a tie point file to a labeled table file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development
    5.1         08/89      B. Ailts     Fixed problem with char. data on SUN-4
    5.1         11/92      D. Etrheim   Correct memory allocation problem that 
                                        occurred with a large TPS file.
    5.2         02/97      D.Etrheim    Change to use strcpy for character data 
                                        to correct problems of freeing the
                                        character pointer.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Receive the parameter values 
If file type is tie point selection file
  Copy the input tie point selection file to the output labeled table file 
Else
  Copy the input tie point location file to the output labeled table file 
Display the completion message 

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int main
(void)

{

char hosin[CMLEN];		/* host input name */
char hosout[2][CMLEN];		/* host output name */

struct PARBLK parb;		/* TAE parameter block */

long lun;			/*  standard output			*/
long two = 2;			/* value of two (2)			*/

char ftype[4];			/* input file type			*/

c_inlas(&parb,&lun);

/*  Get parameter values
------------------------*/
getpar(&parb,hosin,hosout,ftype);

/*  Proces input tie point file
-------------------------------*/
if (strncmp(ftype,"TPS",3) == 0)
   tpsproc(&parb,hosout,hosin);
else
   tplproc(&parb,hosout,hosin);

/*  Ouptut completion message
-----------------------------*/
c_complt("TIE2TAB",hosout,&two);

return EXIT_SUCCESS;

}
