/******************************************************************************

NAME:	TPLLABOP

FUNCTION:  TPLLABOP opens a labeled table file and writes the header
	   information of a tie point location file to the main output 
	   labeled table file

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development
    5.1         08/89      B. Ailts     Fixed problem with char. data on SUN-4

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Write the vectors of the Main labeled table file to hold the header
 information of the tie point location file
Write the vectors of the sub labeled table file to hold the point
 information of the tie point location file
Write the header data of the tie point location file to the main labeled 
 table file 

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "protos.h"

void tpllabop
(
    struct TAB_DEF *tab,		/* table descriptor 			*/
    char hosout[][CMLEN],		/* host output file name 		*/
    struct TPLHEAD *tplhead	/*  header structure of TPL file	*/
)

{ /* tpllabop */

struct VECTOR *ptr;		/* pointer to current vector of labeled table */
struct MATRIX size;		/* size of the matrix			*/

/*  Initialize variables needed to open the output labeled table file.
----------------------------------------------------------------------*/
tab[0].nsubs = 1;
tab[0].ncol = 0;
tab[0].subfile[0] = hosout[1];
tab[0].ftype = "PNTATT";
tab[0].fdesc = "Point attribute information";
tab[1].nsubs = 0;
tab[1].ncol = 0;
tab[1].subfile[0] = "\0";
tab[1].ftype = "PNTXY";
tab[1].fdesc = "Point X Y locations";
size.nrow = 1;
size.ncol = 1;

/*  Open the output labeled table file
--------------------------------------*/
open_tab(&tab[0],hosout[0],IWRITE);
open_tab(&tab[1],hosout[1],IWRITE);

/*  Define each vector field for the main file
----------------------------------------------*/
put_vector(&tab[0],"CORNERS",R8,"Projection corners",2,4,FIELD_SEP2);

put_vector(&tab[0],"PDIST",R8,"Projection distance",1,2,FIELD_SEP2);

put_vector(&tab[0],"PPARMS",R8,"Projection parameters",1,15,FIELD_SEP2);

put_vector(&tab[0],"EDGETHRE",R8,"Edge threshold",1,1,FIELD_SEP2);

put_vector(&tab[0],"MINSTR",R8,"Minimum correlation strength",1,1,FIELD_SEP2);

put_vector(&tab[0],"MAXDIFF",R8,"Maximum correlation difference",
	   1,1,FIELD_SEP2);

put_vector(&tab[0],"PVALID",I4,"Projection validity",1,1,FIELD_SEP2);

put_vector(&tab[0],"PCODE",I4,"Projection code",1,1,FIELD_SEP2);

put_vector(&tab[0],"PZONE",I4,"Projection zone",1,1,FIELD_SEP2);

put_vector(&tab[0],"PDATUM",I4,"Projection datum",1,1,FIELD_SEP2);

put_vector(&tab[0],"REFMODE",C,"Reference mode",1,9,FIELD_SEP2);

put_vector(&tab[0],"UNITS",C,"Projection units",1,12,FIELD_SEP2);

put_vector(&tab[0],"PEAKMETH",C,"Correlation peak fitting method",
	   1,25,FIELD_SEP2);

put_vector(&tab[0],"CORRTYPE",C,"Type of correlation",1,7,FIELD_SEP2);

put_vector(&tab[0],"SEANAME",C,"Name of search images",1,CMLEN,FIELD_SEP2);

put_vector(&tab[0],"REFNAME",C,"Name of reference images",1,CMLEN,RECORD_SEP);

/*  Define each vector field for the subfile
--------------------------------------------*/
put_vector(&tab[1],"PNTNUM",I4,"Unique point number",1,1,FIELD_SEP2);

put_vector(&tab[1],"YGEOCOOR",R8,"Geographic Y coordinate",1,1,FIELD_SEP2);
put_vector(&tab[1],"XGEOCOOR",R8,"Geographic X coordinate",1,1,FIELD_SEP2);

put_vector(&tab[1],"YSEACOOR",R8,"Search Y coordinate",1,1,FIELD_SEP2);
put_vector(&tab[1],"XSEACOOR",R8,"Search X coordinate",1,1,FIELD_SEP2);

put_vector(&tab[1],"YREFCOOR",R8,"Reference Y coordinate",1,1,FIELD_SEP2);
put_vector(&tab[1],"XREFCOOR",R8,"Reference X coordinate",1,1,FIELD_SEP2);

put_vector(&tab[1],"RMSERRV",R8,"RMS vertical error",1,1,FIELD_SEP2);
put_vector(&tab[1],"RMSERRH",R8,"RMS horizontal error",1,1,FIELD_SEP2);

put_vector(&tab[1],"STRENGTH",R8,"Strength of correlation",1,1,FIELD_SEP2);

put_vector(&tab[1],"DISPLAC",R8,"Displacement from nominal image location",
	   1,1,FIELD_SEP2);

put_vector(&tab[1],"NOMINL",R8,"Nominal search line corrdinates",
	   1,1,FIELD_SEP2);
put_vector(&tab[1],"NOMINS",R8,"Nomial search sample corrdinates",
	   1,1,FIELD_SEP2);

put_vector(&tab[1],"ELEV",R8,"Elevation of tie point",1,1,FIELD_SEP2);

put_vector(&tab[1],"ZOOM",R8,"Zoom factor when tie point was selected",1,
	   1,FIELD_SEP2);

put_vector(&tab[1],"REFSZH",I4,"Reference sub-image height",1,1,FIELD_SEP2);
put_vector(&tab[1],"REFSZW",I4,"Reference sub-image width",1,1,FIELD_SEP2);

put_vector(&tab[1],"OFFSETH",I4,"Horizontal search offsets",1,1,FIELD_SEP2);
put_vector(&tab[1],"OFFSETV",I4,"Vertical search offsets",1,1,FIELD_SEP2);

put_vector(&tab[1],"CHIPLOCL",I4,"Line corrdinate of UL corner",1,1,FIELD_SEP2);
put_vector(&tab[1],"CHIPLOCS",I4,"Line corrdinate of UL corner",1,1,FIELD_SEP2);

put_vector(&tab[1],"SEASZH",I4,"Search sub-image height",1,1,FIELD_SEP2);
put_vector(&tab[1],"SEASZW",I4,"Search sub-image width",1,1,FIELD_SEP2);

put_vector(&tab[1],"CHIPCORL",I4,"Line coordinate of tie point",1,1,FIELD_SEP2);
put_vector(&tab[1],"CHIPCORS",I4,"Sample coordinate of tie point",
	   1,1,FIELD_SEP2);

put_vector(&tab[1],"REQSIZEH",I4,"Requested Search sub-image height",
	   1,1,FIELD_SEP2);
put_vector(&tab[1],"REQSIZEW",I4,"Requested Search sub-image width",
	   1,1,FIELD_SEP2);

put_vector(&tab[1],"LOCATL",I4,"Line coordinate of upper corner",
	   1,1,FIELD_SEP2);
put_vector(&tab[1],"LOCATS",I4,"Sample coordinate of upper corner",
	   1,1,FIELD_SEP2);

put_vector(&tab[1],"ACTIVE",I4,"Tie point active flag",1,1,FIELD_SEP2);

put_vector(&tab[1],"REFCHIP",C,"Name of reference chip",1,CMLEN,FIELD_SEP2);

put_vector(&tab[1],"LABEL",C,"User defined label description",1,20,
	   RECORD_SEP);

/*  Write the data to the correct vector of the main labeled table.  This file
    holds the describes the tie point file
------------------------------------------------------------------------------*/
ptr = tab[0].vector;
size.nrow = 2;
size.ncol = 4;
put_matrix(ptr,R8,&size,tplhead->corners);

ptr = ptr->next;
size.nrow = 1;
size.ncol = 2;
put_matrix(ptr,R8,&size,tplhead->pdist);

ptr = ptr->next;
size.ncol = 15;
put_matrix(ptr,R8,&size,tplhead->projprms);

ptr = ptr->next;
size.ncol = 1;
put_matrix(ptr,R8,&size,&tplhead->edge_thres);

ptr = ptr->next;
put_matrix(ptr,R8,&size,&tplhead->min_str);

ptr = ptr->next;
put_matrix(ptr,R8,&size,&tplhead->max_dis);

ptr = ptr->next;
size.ncol = 1;
put_matrix(ptr,I4,&size,&tplhead->proj_valid);

ptr = ptr->next;
put_matrix(ptr,I4,&size,&tplhead->code);

ptr = ptr->next;
put_matrix(ptr,I4,&size,&tplhead->zone);

ptr = ptr->next;
put_matrix(ptr,I4,&size,&tplhead->datum);

ptr = ptr->next;
strcpy(ptr->ptr.c,tplhead->ref_mode);

ptr = ptr->next;
strcpy(ptr->ptr.c,tplhead->units);

ptr = ptr->next;
strcpy(ptr->ptr.c,tplhead->peak_method);

ptr = ptr->next;
strcpy(ptr->ptr.c,tplhead->corr_type);

ptr = ptr->next;
strcpy(ptr->ptr.c,tplhead->sea_name);

ptr = ptr->next;
strcpy(ptr->ptr.c,tplhead->ref_name);

put_record(&tab[0]);

return;
} /* tpllabop */
