/******************************************************************************

NAME:	TPLPROC

FUNCTION:  TPLPROC converts a tie point location file to a labeled table file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Open the input tie point file and output labeled table file
Set up each of the vectors of the ouput labeled table file
For each record in the tie point file
   Read the record
   Write the correct fields of the tie point file to the labeled table file
Close the files

******************************************************************************/
#include <stdio.h>

#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

#define i curent		/* Global for processing message      */

void tplproc
(
    struct PARBLK *vblock,		/* TAE parameter block			*/
    char hosout[][CMLEN],		/* host file name of output file	*/
    char *hosin			/* host file name of input file		*/
)

{  /*  tplproc  */

struct TAB_DEF tab[2];		/* label table descriptot */

struct TPLDATA tpldata;		/* structure to hold the TPL point data	*/
struct TPLHEAD tplhead;		/* structure to hold the TPL header info*/

long acc = IREAD;		/*  file access				*/
long fatal = -1;		/*  fatal error code		*/
long i;				/* counter--current line */
long nonfatal = 1;		/*  nonfatal error code			*/
long pttype;			/*  type of tie point file		*/
long status;			/* Status of function calls		*/
long two = 2;			/* value of two				*/

struct PTDESC * ptdesc;			/*  file descriptor of the tie point file */

/*  Open input tie point file
-----------------------------*/
pttype = TPL;
status = c_ptopen(&ptdesc,hosin,&pttype,&acc,&tplhead);
if (status != E_SUCC)
   {
   c_errmsg("Error opening input file","tie2tab-open",&nonfatal);
   c_errmsg("Fatal error encountered","tie2tab-fatal",&fatal);
   }

/*  open output labeled table file and define each of its vectors
-----------------------------------------------------------------*/
tpllabop(tab,hosout,&tplhead);

/*  Read one line(row) at a time until EOF is reached and place 
    in the labeled table file
----------------------------------------------------------------*/
status = c_ptio(&ptdesc,&tpldata);
if (status == E_FAIL)
   {
   c_errmsg("Error reading input file","tie2tab-read",&nonfatal);
   close_tab(&tab[0]);
   close_tab(&tab[1]);
   c_clean(vblock,"tie2tab",hosout,&two);
   }
i = 1;
while (status != E_EOF)
   {
   writetpl(tab,&tpldata,&i);
   status = c_ptio(&ptdesc,&tpldata);
   if (status == E_FAIL)
      {
      c_errmsg("Error reading input file","tie2tab-read",&nonfatal);
      close_tab(&tab[0]);
      close_tab(&tab[1]);
      c_clean(vblock,"tie2tab",hosout,&two);
      }
   i = i + 1;
   } /*  while not E_EOF  */

/*  Close the input and output files.
-------------------------------------*/
status = c_ptclse(&ptdesc);
close_tab(&tab[0]);
close_tab(&tab[1]);
}  /*  tplproc  */
