/******************************************************************************

NAME:	TPSLABOP

FUNCTION:  TPSLABOP opens a labeled table file and writes the header
	   information of a tie point selection file to the main output 
	   labeled table file

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development
    5.1         08/89      B. Ailts     Fixed problem with char. data on SUN-4

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Write the vectors of the Main labeled table file to hold the header
 information of the tie point selection file
Write the vectors of the sub labeled table file to hold the point
 information of the tie point selection file
Write the header data of the tie point selection file to the main labeled 
 table file 

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "protos.h"

void tpslabop
(
    struct TAB_DEF *tab,		/* table descriptor */
    char hosout[][CMLEN],		/* host output file name */
    struct TPSHEAD *tpshead	/*  header structure of TPS file	*/
)

{ /* tpslabop */

struct VECTOR *ptr;		/*  pointer to the current vector	*/
struct MATRIX size;		/*  size of output matrix		*/

/*  Initialize variables needed to open the output labeled table file.
----------------------------------------------------------------------*/
tab[0].nsubs = 1;
tab[0].ncol = 0;
tab[0].ftype = "PNTATT";
tab[0].fdesc = "Point attribute information";
tab[0].subfile[0] = hosout[1];
tab[1].nsubs = 0;
tab[1].ncol = 0;
tab[1].subfile[0] = "\0";
tab[1].ftype = "PNTXY";
tab[1].fdesc = "Point X Y locations";
size.nrow = 1;
size.ncol = 1;

/*  Open the output labeled table file
--------------------------------------*/
open_tab(&tab[0],hosout[0],IWRITE);
open_tab(&tab[1],hosout[1],IWRITE);

/*  Define each vector field of the main file and write the header information
    of the tie point selection file to it.
------------------------------------------------------------------------------*/
put_vector(&tab[0],"CORNERS",R8,"Projection corners",2,4,FIELD_SEP2);
ptr = tab[0].vector;
size.nrow = 2;
size.ncol = 4;
put_matrix(ptr,R8,&size,tpshead->corners);

put_vector(&tab[0],"PDIST",R8,"Projection distance",1,2,FIELD_SEP2);
ptr = ptr->next;
size.nrow = 1;
size.ncol = 2;
put_matrix(ptr,R8,&size,tpshead->pdist);

put_vector(&tab[0],"PPARMS",R8,"Projection parameters",1,15,FIELD_SEP2);
ptr = ptr->next;
size.ncol = 15;
put_matrix(ptr,R8,&size,tpshead->projprms);

put_vector(&tab[0],"PVALID",I4,"Projection validity",1,1,FIELD_SEP2);
ptr = ptr->next;
size.ncol = 1;
put_matrix(ptr,I4,&size,&tpshead->proj_valid);

put_vector(&tab[0],"PCODE",I4,"Projection code",1,1,FIELD_SEP2);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpshead->code);

put_vector(&tab[0],"PZONE",I4,"Projection zone",1,1,FIELD_SEP2);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpshead->zone);

put_vector(&tab[0],"PDATUM",I4,"Projection datum",1,1,FIELD_SEP2);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpshead->datum);

put_vector(&tab[0],"MODE",C,"Tie point mode",1,4,FIELD_SEP2);
ptr = ptr->next;
strcpy(ptr->ptr.c,tpshead->mode);

put_vector(&tab[0],"UNITS",C,"Projection units",1,12,FIELD_SEP2);
ptr = ptr->next;
strcpy(ptr->ptr.c,tpshead->units);

put_vector(&tab[0],"NAME",C,"Name of images",1,CMLEN,RECORD_SEP);
ptr = ptr->next;
strcpy(ptr->ptr.c,tpshead->name);

/*  Define each vector field of the subfile
-------------------------------------------*/
put_vector(&tab[1],"PNTNUM",I4,"Unique point number",1,1,FIELD_SEP2);

put_vector(&tab[1],"IMAGEY",R8,"Image Y coordinate",1,1,FIELD_SEP2);
put_vector(&tab[1],"IMAGEX",R8,"Image X coordinate",1,1,FIELD_SEP2);

put_vector(&tab[1],"LABEL",C,"User defined label description",1,20,
	   FIELD_SEP2);

put_vector(&tab[1],"COORDY",R8,"Tie point Y coordinates",1,1,FIELD_SEP2);
put_vector(&tab[1],"COORDX",R8,"Tie point X coordinates",1,1,FIELD_SEP2);

put_vector(&tab[1],"ELEV",R8,"Elevation of tie point",1,1,FIELD_SEP2);

put_vector(&tab[1],"ZOOM",R8,"Zoom factor when tie point was selected",1,
	   1,FIELD_SEP2);

put_vector(&tab[1],"CHIPSZEH",I4,"Sub-image height",1,1,FIELD_SEP2);
put_vector(&tab[1],"CHIPSZEW",I4,"Sub-image width",1,1,FIELD_SEP2);

put_vector(&tab[1],"OFFSETH",I4,"Search offsets -- horizontal",1,1,FIELD_SEP2);
put_vector(&tab[1],"OFFSETV",I4,"Search offsets -- vertical",1,1,FIELD_SEP2);

put_vector(&tab[1],"CHIPNAME",C,"Name of Chip",1,CMLEN,RECORD_SEP);

put_record(&tab[0]);

return;
} /* tpslabop */
