/******************************************************************************

NAME:	TPSPROC

FUNCTION:  TPSPROC converts a tie point selection file to a labeled table file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Open the input tie point file and output labeled table file 
Set up the label for the ouput labeled table file
For each record in the tie point file
   Read the record
   Write the correct fields of the tie point file to the labeled table file
Close the files

******************************************************************************/
#include <stdio.h>

#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

#define i curent			/* Global for processing message      */

void tpsproc
(
    struct PARBLK *vblock,		/*  Tae parameter block			*/
    char hosout[][CMLEN],		/*  host output file names		*/
    char *hosin			/*  host input file name		*/
)

{  /*  tpsproc  */

struct TAB_DEF tab[2];		/* label table descriptot */

struct TPSDATA tpsdata;		/*  point data of tie point selection file */
struct TPSHEAD tpshead;		/*  header data of tie point selection file */

long acc = IREAD;		/*  file access type			*/
long fatal = -1;		/*  fatal error code		*/
long i;				/* counter--current line */
long nonfatal = 1;		/*  nonfatal error code			*/
long pttype;			/* tie point file type			*/
long status;			/* Status of function calls		*/
long two = 2;			/* value of two				*/

struct PTDESC * ptdesc;			/* file descriptor of tie point file	*/

/*  Open input tie point file
-----------------------------*/
pttype = TPS;
status = c_ptopen(&ptdesc,hosin,&pttype,&acc,&tpshead);
if (status != E_SUCC)
   {
   c_errmsg("Error opening input file","tie2tab-open",&nonfatal);
   c_errmsg("Fatal error encountered","tie2tab-fatal",&fatal);
   }

/*  open and write the header information to the output labeled table file
--------------------------------------------------------------------------*/
tpslabop(tab,hosout,&tpshead);

/*  Read one line(row) at a time until EOF is reached and place in labeled 
    table file
---------------------------------------------------------------------------*/
status = c_ptio(&ptdesc,&tpsdata);
if (status == E_FAIL)
   {
   c_errmsg("Error reading input file","tie2tab-read",&nonfatal);
   close_tab(&tab[0]);
   close_tab(&tab[1]);
   c_clean(vblock,"tie2tab",hosout,&two);
   }
i = 1;
while (status != E_EOF)
   {
   writetps(tab,&tpsdata,&i);
   status = c_ptio(&ptdesc,&tpsdata);
   if (status == E_FAIL)
      {
      c_errmsg("Error reading input file","tie2tab-read",&nonfatal);
      c_errmsg("Fatal error encountered","tie2tab-fatal",&fatal);
      close_tab(&tab[0]);
      close_tab(&tab[1]);
      c_clean(vblock,"tie2tab",hosout,&two);
      }
   i = i + 1;
   } /*  while not E_EOF  */

/*  Close the input and output files.
-------------------------------------*/
status = c_ptclse(&ptdesc);
close_tab(&tab[0]);
close_tab(&tab[1]);
}  /*  tpsproc  */
