/******************************************************************************

NAME:	WRITETPL

FUNCTION:  WRITETPL write the point data of a tie point location file 
	   to a labeled table file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development
    5.1		02/97      D.Etrheim	Change to use strcpy for character data
                                        to correct problems of freeing the
                                        character pointer.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Write the point data of the tie point location file to the correct vector 
 of the output labeled table subfile

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "protos.h"

void writetpl
(
    struct TAB_DEF *tab,		/* output labeled table descriptor */
    struct TPLDATA *tpldata,	/* point data of tie point location file */
    long *pntnum			/* point number of the tie point	*/
)

{ /* writetpl */

struct VECTOR *ptr;		/* pointer to current vector */
struct MATRIX size;		/* matrix size */

/* initialize the needed variables
----------------------------------*/
ptr = tab[1].vector;

size.nrow = 1;
size.ncol = 1;

/* write each field to the sub file 
------------------------------------*/
put_matrix(ptr,I4,&size,pntnum);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->geo_coord[0]);
ptr = ptr->next;
put_matrix(ptr,R8,&size,&tpldata->geo_coord[1]);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->sea_coord[0]);
ptr = ptr->next;
put_matrix(ptr,R8,&size,&tpldata->sea_coord[1]);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->ref_coord[0]);
ptr = ptr->next;
put_matrix(ptr,R8,&size,&tpldata->ref_coord[1]);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->rms_error[0]);
ptr = ptr->next;
put_matrix(ptr,R8,&size,&tpldata->rms_error[1]);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->strength);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->displacement);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->nominal[0]);
ptr = ptr->next;
put_matrix(ptr,R8,&size,&tpldata->nominal[1]);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->elevation);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpldata->zoom_factor);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpldata->ref_size[0]);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpldata->ref_size[1]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpldata->offsets[0]);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpldata->offsets[1]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpldata->chip_loc[0]);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpldata->chip_loc[1]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpldata->sea_size[0]);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpldata->sea_size[1]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpldata->chip_coord[0]);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpldata->chip_coord[1]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpldata->req_size[0]);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpldata->req_size[1]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpldata->location[0]);
ptr = ptr->next;
put_matrix(ptr,I4,&size,&tpldata->location[1]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpldata->active);
ptr = ptr->next;

strcpy(ptr->ptr.c,tpldata->ref_chip);
ptr = ptr->next;

strcpy(ptr->ptr.c,tpldata->pt_id);

/*  Write the record to the file
--------------------------------*/
put_record(&tab[1]);

return;
} /* writetpl */
