/******************************************************************************

NAME:	WRITETPS

FUNCTION:  WRITETPS write the point data of a tie point selection file 
	   to a labeled table file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0         06/89      B. Ailts     original development
    5.1         02/97      D.Etrheim    Change to use strcpy for character data 
                                        to correct problems of freeing the
                                        character pointer.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Run under TAE

PROJECT:	LAS

ALGORITHM (FLOW):
Write the point data of the tie point selection file to the correct vector 
 of the output labeled table subfile

******************************************************************************/
#include "las.h"
#include "ltable.h"
#include "geompak.h"
#include "protos.h"

void writetps
(
    struct TAB_DEF *tab,		/* output labeled table descriptor */
    struct TPSDATA *tpsdata,	/* point data of the tie point selection file */
    long *pntnum			/* point number				*/
)

{ /* writetps */

struct VECTOR *ptr;		/* pointer to current vector */
struct MATRIX size;		/* matrix size */

/* initialize the needed variables
----------------------------------*/
ptr = tab[1].vector;

size.nrow = 1;
size.ncol = 1;

/* write each field to the sub file 
------------------------------------*/
put_matrix(ptr,I4,&size,pntnum);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpsdata->image_coord[0]);
ptr = ptr->next;
put_matrix(ptr,R8,&size,&tpsdata->image_coord[1]);
ptr = ptr->next;

strcpy(ptr->ptr.c,tpsdata->pt_id);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpsdata->coord[0]);
ptr = ptr->next;
put_matrix(ptr,R8,&size,&tpsdata->coord[1]);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpsdata->elevation);
ptr = ptr->next;

put_matrix(ptr,R8,&size,&tpsdata->zoom_factor);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpsdata->chip_size[0]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpsdata->chip_size[1]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpsdata->offsets[0]);
ptr = ptr->next;

put_matrix(ptr,I4,&size,&tpsdata->offsets[1]);
ptr = ptr->next;

strcpy(ptr->ptr.c,tpsdata->chip_name);

/* Write the recode to the subfile
----------------------------------*/
put_record(&tab[1]);

return;
} /* writetps */
