/****************************************************************************
NAME:				getpar

PURPOSE:  Get user entered parameters from the TAE parameter block

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		CODE/CONT   REASON
-------		----	------		---------   ------
  5.0		12/88	D. Steinwand	   CSB      New development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
	Initialize tae parameter blocks, etc
	Retrieve parameters
	Return

ALGORITHM REFERENCES:
*****************************************************************************/
#include "las.h"
#include "util.h"
#include "geompak.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblk,		/* TAE variable block */
    char *tps_ref,			/* Reference tie point selection file */
    char *tps_search,		/* Search tie point selection file */
    char *outfile,			/* Output file name (host) */
    long *confirm,			/* Confirm flag:  TRUE or FALSE */
    long *mode			/* Processing mode: NOCORR or PRECORR */
)
{
struct VARIABLE *var;		/* pointer to a TAE variable in vblk */
long zero = 0;			/* Constant of 0 */
long one = 1;			/* Constant of 1 */
long acc;			/* File access mode */
char temp_char[8];		/* String containing temp_char */

/* Get and set the mode (temp_char) flag
  --------------------------------------*/
var = p_find(vblk, "_SUBCMD");
strcpy(temp_char, SVAL(*var, 0));
if (strcmp(temp_char,"NOCORR") == 0) *mode = NOCORR;
else *mode = PRECORR;

/* Get confirm mode
  ----------------*/
var = p_find(vblk, "CONFLG");
strcpy(temp_char, SVAL(*var, 0));
if (strcmp(temp_char,"YES") == 0) *confirm = TRUE;
else *confirm = FALSE;

/* Get the reference and search tie point file names
  -------------------------------------------------*/
acc = IREAD;
if(c_getnam(vblk, "INTS", &zero, tps_ref, "tps", &acc) != E_SUCC)
 	return(NO_REF_TPS);
if(c_getnam(vblk, "INTS", &one, tps_search, "tps", &acc)!=E_SUCC)
	return(NO_SEA_TPS);

/* Get the output file name
  ------------------------*/
acc = IWRITE;
if (*mode == PRECORR)
   {
   if(c_getnam(vblk, "OUTMT", &zero, outfile, "mtp", &acc) != E_SUCC)
	return(BAD_OUT_MTP);
   }
else
   {
   if(c_getnam(vblk, "OUTTL", &zero, outfile, "tpl", &acc) != E_SUCC)
	return(BAD_OUT_TPL);
   }
return(E_SUCC);
}
