/*
 *  ProcessM.h
 */

/*
 *  Public declarations/definitions for ProcessM.c
 */

#ifndef _XeProcessM_h
#define _XeProcessM_h

/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or 
 * publicity pertaining to distribution of the software without specific, 
 * written prior permission.  USGS makes no representations about the 
 * suitability of this software for any purpose.  It is provided "as is" 
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS 
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *  Change log:
 *
 *      D. Akkerman, USGS/EROS Data Center, 9/92
 *       - inception.
 *
 */

/*
 *  This module implements a processing message box that an application can
 *  use to provide feedback to the user during long processing operations.
 *  As written, the ProcessM widget is a compound widget encapsulating a
 *  Motif widget tree required for a processing message.  In general, it is
 *  a subclass of the Motif BulletinBoard widget.
 *
 */

/*
 *  Resources:
 *
 *    Name                Class            Data Type     Default        Access
 *
 * XeNlabelString      XeCLabelString      String        "Please Wait"   C,S
 * XeNlabelFont        XeCLabelFont        XFontStruct*  XtDefaultFont   C,S
 * XeNpixmapWidth      XeCPixmapWidth      int           50              C,S
 * XeNpixmapHeight     XeCPixmapHeight     int           50              C,S
 * XeNpixmap           XeCPixmap           Pixmap        NULL            C,S
 * XeNusePixmap        XeCUsePixmap        Boolean       FALSE           C,S
 * XeNdrawColors       XeCDrawColors       XtPointer     NULL            C,S
 * XeNnumColors        XeCNumColors        int           0               C,S
 * XeNsetColors        XeCSetColors        Boolean       FALSE           C,S
 * XeNuseCancel        XeCUseCancel        Boolean       FALSE           C,S
 * XeNusePause         XeCUsePause         Boolean       FALSE           C,S
 * XeNbuttonFont       XeCButtonFont       XFontStruct*  XtDefaultFont   C,S
 * XeNupdatePercent    XeCUpdatePercent    int           20              C,S
 * XeNbackgroundColor  XeCBackgroundColor  int           -99             C,S
 *
 */

/*
 *  Public interfaces.
 */

#include <Xm/BulletinB.h>

/*
 *  Resource definitions.
 */

#define XeNlabelString          "labelString"
#define XeNlabelFont            "labelFont"
#define XeNpixmapWidth          "pixmapWidth"
#define XeNpixmapHeight         "pixmapHeight"
#define XeNpixmap               "pixmap"
#define XeNdrawColors           "drawColors"
#define XeNnumColors            "numColors"
#define XeNsetColors            "setColors"
#define XeNuseCancel            "useCancel"
#define XeNusePause             "usePause"
#define XeNbuttonFont           "buttonFont"
#define XeNupdatePercent        "updatePercent"
#define XeNbackgroundColor      "backgroundColor"
#define XeNusePixmap            "usePixmap"

#define XeCLabelString          "LabelString"
#define XeCLabelFont            "LabelFont"
#define XeCPixmapWidth          "PixmapWidth"
#define XeCPixmapHeight         "PixmapHeight"
#define XeCPixmap               "Pixmap"
#define XeCDrawColors           "DrawColors"
#define XeCNumColors            "NumColors"
#define XeCSetColors            "SetColors"
#define XeCUseCancel            "UseCancel"
#define XeCUsePause             "UsePause"
#define XeCButtonFont           "ButtonFont"
#define XeCUpdatePercent        "UpdatePercent"
#define XeCBackgroundColor      "BackgroundColor"
#define XeCUsePixmap            "UsePixmap"

typedef struct _XeProcessMClassRec *XeProcessMWidgetClass;
typedef struct _XeProcessMRec *XeProcessMWidget;

extern WidgetClass xeProcessMWidgetClass;

/*
 *  Public constants.
 */

#define PROCESSM_MAX_COLORS     10 /* Maximum # of colors for processing.*/

#define MAX_LABEL_SIZE          80 /* Maximum label string length.       */

/*
 *  Public functions.
 */
void XeProcessMInit
(
    XeProcessMWidget processm_widget,
    int iterator_start,
    int iterator_end
);

int XeProcessMUpdate
(
    XeProcessMWidget processm_widget,
    int iterator_current
);

void XeProcessMCenter
(
    XeProcessMWidget processm_widget,
    int *screen_x,
    int *screen_y
);


#endif /* _XeProcessM_h */
