/*
 *  ProcessMP.h
 */

/*
 *  Private declarations/definitions for ProcessM.c
 */

#ifndef _XeProcessMP_h
#define _XeProcessMP_h
 
/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  USGS makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
 
/*
 *  Change log:
 *
 *      D. Akkerman, USGS/EROS Data Center, 9/92
 *       - inception.
 *
 */

#include <stdio.h>
#include <Xm/BulletinBP.h>
#include "ProcessM.h"

/*
 *  Definitions for default resource values.
 */

#define PROCESSM_DEFAULT_LABEL                  "Please Wait"
#define PROCESSM_DEFAULT_PIXMAP_WIDTH           50
#define PROCESSM_DEFAULT_PIXMAP_HEIGHT          50
#define PROCESSM_DEFAULT_PIXMAP                 NULL
#define PROCESSM_DEFAULT_NUM_COLORS             0
#define PROCESSM_DEFAULT_USE_CANCEL             FALSE
#define PROCESSM_DEFAULT_USE_PAUSE              FALSE
#define PROCESSM_DEFAULT_UPDATE_PERCENT         20
#define PROCESSM_DEFAULT_BACKGROUND_COLOR       -99

/*
 *  Working constants.
 */

#define GAP             5

/*
 *  Class definition.
 */

typedef struct _XeProcessMClassPart
 {
  long dummy;
 } XeProcessMClassPart;

typedef struct _XeProcessMClassRec
 {
  CoreClassPart                 core_class;
  CompositeClassPart            composite_class;
  ConstraintClassPart           constraint_class;
  XmManagerClassPart            manager_class;
  XmBulletinBoardClassPart      bulletin_board_class;
  XeProcessMClassPart           processm_class;
 } XeProcessMClassRec;

extern XeProcessMClassRec xeProcessMClassRec;

/*
 *  Instance definition.
 */

typedef struct _XeProcessMPart
 {
  char *label_string,           /* Label string pointer and store.      */
        label_string_store[(MAX_LABEL_SIZE+1)];

  XmString pause_label,         /* Motif Label strings.                 */
           resume_label;

  XFontStruct *label_font,      /* Font structures for the label        */
              *button_font;     /* button widgets.                      */

  XmFontList label_font_list,   /* Motif font lists for the fonts.      */
             button_font_list;

  Boolean cancel_flag,          /* Flag for processing interruption.    */
          pause_flag,           /* Flag for pausing operations.         */
          init_flag,            /* Flag for proper initialization.      */
          use_pixmap,           /* Flag for using specified pixmap.     */
          set_colors;           /* Flag for setting processing colors.  */

  GC    gc;                     /* GC for process drawing.              */

  Pixel *input_colors,          /* Pointer to color array.              */
        draw_colors[PROCESSM_MAX_COLORS],
                                /* Processing colors.                   */
        background_color;       /* Background drawing color.            */

  int   current_color,          /* Current drawing color.               */
        num_colors,             /* Total # of drawing colors.           */
        pixmap_width,           /* Pixmap dimensions.                   */
        pixmap_height,
        update_percent;         /* Percent increments for updates.      */

  Dimension bulletinboard_width,/* Bulletinboard dimensions.            */
            bulletinboard_height,
            cancel_width,       /* Cancel, Pause button widths.         */
            pause_width,
            button_height;      /* PushButton height.                   */

  Widget messageLabel,          /* Internal widgets.                    */
         drawingArea,
         cancelButton,
         pauseButton;

  Pixmap pixmap;                /* DrawingArea background pixmap.       */

  Boolean use_cancel,           /* Booleans for the cancel and pause    */
          use_pause;            /* buttons.                             */

  long percent,                 /* Percent to be drawn.                 */
       start,                   /* Starting iterator amount.            */
       end,                     /* Ending iterator amount.              */
       old;                     /* Old percent value used to limit
                                   message output.                      */

  short new_angle,              /* Angles for processing message draw.  */
        old_angle;

 } XeProcessMPart;

typedef struct _XeProcessMRec
 {
  CorePart              core;
  CompositePart         composite;
  ConstraintPart        constraint;
  XmManagerPart         manager;
  XmBulletinBoardPart   bulletin_board;
  XeProcessMPart        processm;
 } XeProcessMRec;

#endif /* _XeProcessMP_h */
