/*
 *  This is the map function source file for the map panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 * Modified by T Rockvam, USGS/EROS Data Center, 3/94.
 *    - removed duplicate code when resetting slider values
 *
 */
#include <math.h>
#include <stdlib.h>
#include "tiepts.h"
#include "xid.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine modifies the displayed image data 
    to meet the adjust lookup table. This routine is only used in 
    IMAGE to IMAGE mode so that both image displays are not modified
    by the LUT adjustment.

By J. Hemmer, USGS/EROS Data Center, 3/92

Algorithm Description:
*******************************************************************************/
void cmap_adjust_data(void) 
{
   double               rslope;
   int                  i,j,
                        maxval,
                        nl, ns,
                        nplanes,
                        plane_idx;
   short                *newlut;
   unsigned char        *image[3];

  /*
   *   *** if adjustment was not made, return
   */
   if ((tie_adjust.lut_slope == 45.0) && (tie_adjust.lut_offset == 0.0))
      return;

  /*
   *   *** get plane index of respective image display
   */
   if (tie_map.imgidx == REFERENCE)
      plane_idx = 1;
   else
      plane_idx = 4;

  /*
   *  *** determine number of different memory planes
   */
   if (tie.visual_info[0].visual->class == PseudoColor)
      nplanes = 1;
   else if (
     (tie_composite[tie_map.imgidx].rgbplanes[RED] == 
      tie_composite[tie_map.imgidx].rgbplanes[GREEN]) && 
     (tie_composite[tie_map.imgidx].rgbplanes[GREEN] ==
      tie_composite[tie_map.imgidx].rgbplanes[BLUE]))
      nplanes = 1;
   else
      nplanes = 3;
   
   if (tie_adjust.full_lut[tie_map.imgidx] == NULL)
      tie_adjust.full_lut[tie_map.imgidx] = TieMalloc(256*sizeof(short));

   newlut = tie_adjust.full_lut[tie_map.imgidx];

  /*
   *  *** allocate space for temporary image
   */
   nl = tie_planes[plane_idx-1].dspwin[NL];
   ns = tie_planes[plane_idx-1].dspwin[NS];

  /*
   *  *** create LUT based on new adjustment and any previous adjustments
   */
   rslope = DEGTORAD(tie_adjust.lut_slope);
   rslope = tan(rslope);

   for (i = 0; i < 256; i++)
      {
      newlut[i] = (short)(i * rslope + tie_adjust.lut_offset/256.);
      newlut[i] = (newlut[i] < 0)   ?     0 : newlut[i];
      newlut[i] = (newlut[i] > 255)   ? 255 : newlut[i];
      }

   tie_adjust.lut_offset =   0.0;
   tie_adjust.lut_slope  =  45.0;

   for (i = 0; i < nplanes; i++)
      {
      image[i] = TieMalloc (nl * ns);

      memcpy(image[i],
            tie_planes[tie_composite[tie_map.imgidx].rgbplanes[i]-1].image_data,
            nl*ns); 

      for (j = 0; j < nl*ns; j++)
         image[i][j] = (unsigned char)newlut[image[i][j]];
      }

   maxval = tie_planes[plane_idx-1].maxval 
          = newlut[tie_planes[plane_idx-1].maxval];

  /*
   *  *** display adjusted data
   */
   if (tie_map.imgidx == REFERENCE)
      load_display(&plane_idx, &nplanes ,XtWindow(ref_image_bulletinboard),
          Shell_RefImg, ref_image_bulletinboard, &nl, &ns,
          &maxval, image[0], image[1], image[2], TRUE,
          &tie_mem[REFERENCE].image_pixmap,FALSE);
   else
      load_display(&plane_idx,&nplanes,XtWindow(search_image_bulletinboard),
          Shell_SrchImg, search_image_bulletinboard, &nl, &ns,
          &maxval, image[0], image[1], image[2], TRUE,
          &tie_mem[SEARCH].image_pixmap, FALSE);

   for (i = 0; i < nplanes; i++)
      {
      free(image[i]);
      image[i] = NULL;
      }

  /*
   *   *** adjust fullres display
   */
   if ((tie_map.imgidx == REFERENCE) && 
       (tie_ref_fullres != NULL) &&
       (tie_ref_fullres[0].image_data != NULL))
      {
      nl = tie_ref_fullres[0].window[NL];
      ns = tie_ref_fullres[0].window[NS];

      for (i = 0; i < nplanes; i++)
         {
         image[i] = TieMalloc(nl*ns);

         memcpy(image[i], tie_ref_fullres[i].image_data, nl*ns); 
         }

      load_display(&plane_idx, &nplanes ,XtWindow(fullref_win_bulletinboard),
          Shell_RefFull, fullref_win_bulletinboard, &nl, &ns,
          &maxval, image[0], image[1], image[2], TRUE,
          &tie_ref_fullres[0].image_pixmap, FALSE);

      for (i = 0; i < nplanes; i++)
         free(image[i]);
      }
   else if ((tie_map.imgidx == SEARCH) && 
            (tie_srch_fullres != NULL) &&
            (tie_ref_fullres[0].image_data != NULL))
      {
      nl = tie_srch_fullres[0].window[NL];
      ns = tie_srch_fullres[0].window[NS];

      for (i = 0; i < nplanes; i++)
         {
         image[i] = TieMalloc(nl*ns);
         memcpy(image[i], tie_srch_fullres[i].image_data, nl*ns); 
         }

      load_display(&plane_idx, &nplanes, XtWindow(fullsrch_win_bulletinboard),
          Shell_SrchFull, fullsrch_win_bulletinboard, &nl, &ns,
          &maxval, image[0], image[1], image[2], TRUE,
          &tie_srch_fullres[0].image_pixmap, FALSE);

      for (i = 0; i < nplanes; i++)
         free(image[i]);
      }

  /*
   *  *** adjust zoom 
   */
   if ((tie_map.imgidx == REFERENCE) && 
       (tie_zoom[REFERENCE].flags & ZOOM_ACTIVE) && 
       !(tie_zoom[REFERENCE].flags & ZOOM_CLEAR))
      {
      if (!(tie.flags & TIE_ADRG))
        {
        if (tie_zoom[REFERENCE].flags & ZOOM_MAIN)
         zoom_image(ref_image_bulletinboard, zoom_refwin_bulletinboard,
                    tie_planes[tie.rview].dspwin);
        else
         zoom_image(fullref_win_bulletinboard, zoom_refwin_bulletinboard,
                    tie_ref_fullres[0].dspwin);
        }
      }
   else if ((tie_map.imgidx == SEARCH) &&
       (tie_zoom[SEARCH].flags & ZOOM_ACTIVE) && 
       !(tie_zoom[SEARCH].flags & ZOOM_CLEAR))
      {
      if (tie_zoom[SEARCH].flags & ZOOM_MAIN)
         zoom_image(search_image_bulletinboard, zoom_srchwin_bulletinboard,
                    tie_planes[tie.sview].dspwin);
      else
         zoom_image(fullsrch_win_bulletinboard, zoom_srchwin_bulletinboard,
                    tie_srch_fullres[0].dspwin);
      }

  /*
   *  *** redisplay TIE points
   */
   graphic_refresh();
}

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating which component the
    user wishes to map (i.e. red, green, blue or all).

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cmap_component
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
  /*
   *  *** Set the flag indicating which image component to map. 
   */
   tie_map.flags = tie_map.flags & ~(MAP_RED | MAP_GREEN | 
       MAP_BLUE | MAP_ALL);

   switch (*(char *)client)
       {
       case 'R':
           tie_map.flags = tie_map.flags | MAP_RED;
           break;
       case 'G':
           tie_map.flags = tie_map.flags | MAP_GREEN;
           break;
       case 'B':
           tie_map.flags = tie_map.flags | MAP_BLUE;
           break;
       case 'A':
           tie_map.flags = tie_map.flags | MAP_ALL;
           break;
       }
}
   
/*******************************************************************************
Purpose:
    This routine closes the map window as well as the appropriate subpanel,
    sets the main menu option to sensitive and resets the color of the
    working color cell.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void adjust_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XtSetSensitive(main_adjust_button, True);
   XtDestroyWidget(Adjust_bulletinboard);
   XtUnrealizeWidget(Shell_Adjust);

   if (client == NULL)
      {
      if (tie.mode == IM2IM)
      cmap_adjust_data();
      }
   else if (client != NULL)
      {
      XtDestroyWidget(Shell_Adjust);
      Shell_Adjust = NULL;
      }
}

/*******************************************************************************
Purpose:
    This routine is called whenever the user selects the LINEAR button from
    any of the mapping functions.  The current colormap is set to a linear
    ramp.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cmap_linear
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   static unsigned char
           rr[256],            /* Reserved color values.                     */
           gr[256],
           br[256];
   
   XColor  *beg_ptr,            /* Pointer to the start of the appropriate 
                                      XColor structure.                       */
           *work_ptr;           /* Working pointer to appropriate XColor
                                   structure.                                 */

   float   scaler;              /* LUT scaler for actual number of colors.    */

   int     buf_size,            /* Number of bytes in a single band.          */
           i,                   /* Looping index.                             */
           ncolors,             /* Number of colors to store in colormap.     */
           nl, ns,              /* number of lines and samples                */
           nplanes,             /* Number of planes displayed.                */
           plane_idx,           /* memory plane index.                        */
           rp, gp, bp,          /* red, green and blue planes                 */
           offset,              /* Offset used in scaling the 24-bit LUT.     */
           start;               /* Starting index into color table.           */

   unsigned char
        *blut,                  /* Blue LUT for composite image.              */
        *composite,             /* Composite RGB 8-bit image or single band
                                   black white 8-bit image.                   */
        *comptr,                /* Pointer to composite buffer.               */
        *glut,                  /* Green LUT for composite image.             */
        *rgbptr[3],             /* Pointers to RGB components.                */
        *rlut;                  /* Red LUT for composite image.               */

  /*
   *  ***  Determine how many planes are currently being displayed.
   */
   rp = tie_composite[tie_map.imgidx].rgbplanes[RED];
   gp = tie_composite[tie_map.imgidx].rgbplanes[GREEN];
   bp = tie_composite[tie_map.imgidx].rgbplanes[BLUE];

   if ((rp == gp) && (gp == bp))
      nplanes = 1;
   else
      nplanes = 3;

  /*
   *  *** if image to map, just reload
   */
   if (tie.mode == IM2IM)
      {
     /*
      * *** get plane index of respective image display
      */
      if (tie_map.imgidx == REFERENCE)
         plane_idx = 1;
      else
         plane_idx = 4;

      nl = tie_planes[plane_idx-1].dspwin[NL];
      ns = tie_planes[plane_idx-1].dspwin[NS];

     /*
      *  *** display adjusted data
      */
      if (tie_map.imgidx == REFERENCE)
         load_display(&plane_idx, &nplanes ,XtWindow(ref_image_bulletinboard),
             Shell_RefImg, ref_image_bulletinboard, &nl, &ns,
             &tie_planes[plane_idx-1].maxval, 
             tie_planes[rp - 1].image_data,NULL,NULL,TRUE,
             &tie_mem[REFERENCE].image_pixmap,FALSE);
      else
         load_display(&plane_idx,&nplanes,XtWindow(search_image_bulletinboard),
             Shell_SrchImg, search_image_bulletinboard, &nl, &ns,
             &tie_planes[plane_idx-1].maxval, 
             tie_planes[rp - 1].image_data,
             tie_planes[gp - 1].image_data,
             tie_planes[bp - 1].image_data,TRUE,
             &tie_mem[SEARCH].image_pixmap, FALSE);

      free(tie_adjust.full_lut[tie_map.imgidx]);
      tie_adjust.full_lut[tie_map.imgidx] = NULL;

      tie_adjust.lut_offset =   0.0;
      tie_adjust.lut_slope  =  45.0;

     /*
      *  ***  Reset the slider values.
      */
      XtVaSetValues(adjust_contrast_slider,XmNvalue,
              (int)tie_adjust.lut_slope,NULL);
      XtVaSetValues(adjust_bright_slider,XmNvalue,
              (int)(tie_adjust.lut_offset*(100.0/MAXCOLOR)),NULL);

      return;
      }

  /*
   *  ***  Assign number of colors to store and the starting index 
   *       of the image colors based on the type of visual.
   */
   if (tie.visual_info[0].visual->class == PseudoColor)
      {
      start = 0;        
      ncolors = NUM_IMGCOLORS;
      }
   else                                                        /* DirectColor */
      {
      start = 56;
      ncolors = 256;
      }

   if (tie_map.subpanel == Shell_Adjust)
      {
      tie_adjust.lut_offset =   0.0;
      tie_adjust.lut_slope  =  45.0;

     /*
      *  ***  Reset the slider values.
      */
      XtVaSetValues(adjust_contrast_slider,XmNvalue,
              (int)tie_adjust.lut_slope,NULL);
      XtVaSetValues(adjust_bright_slider,XmNvalue,
              (int)(tie_adjust.lut_offset*(100.0/MAXCOLOR)),NULL);

      if (tie_adjust.color_adjust == NULL)
         {
         tie_adjust.color_adjust = (XColor *) calloc(ncolors,sizeof(XColor));
         if (tie_adjust.color_adjust == NULL)
            {
            tie_message(Shell_Adjust, "Insufficient memory available");
            return;
            }
         }

      if (tie_adjust.color_linear == NULL)
         {
         tie_adjust.color_linear = 
                             (XColor *) calloc(NUM_IMGCOLORS, sizeof(XColor));
         if (tie_adjust.color_linear == NULL)
            {
            tie_message(Shell_Adjust, "Insufficient memory available");
            return;
            }
         }
      beg_ptr = &tie_adjust.color_adjust[0];
      tie_adjust.adjust_ptr = &tie_adjust.color_linear[0];
      }

  /*
   *  ***  Calculate the scaler needed to reduce the image to the
   *       appropriate number of colors.
   */
   if ((nplanes == 1) &&
       (tie_planes[tie_composite[tie_map.imgidx].rgbplanes[RED]-1].maxval <
        NUM_IMGCOLORS))
       scaler = 1.0;
   else
       scaler = (256.0 / (float)NUM_IMGCOLORS);      /* (2^8 / # of colors) */

  /*
   *  *** 8-bit monitor -- black and white image.
   */
   if ((tie.visual_info[0].visual->class == PseudoColor) && (nplanes == 1))
      {
     /*
      *  *** Create a linear ramp.
      */
      for (i = 0, work_ptr=beg_ptr + start; i < NUM_IMGCOLORS; i++, work_ptr++)
         {
         work_ptr->pixel = tie.color_table[IMG_IDX + i].pixel;
         work_ptr->blue = work_ptr->green=work_ptr->red = ((i * scaler) * 257);
         work_ptr->flags = DoRed | DoGreen | DoBlue;
         }
  /*
   *  *** Store the new colormap.
   */
   XStoreColors(tie.display,tie.colormap,beg_ptr,NUM_IMGCOLORS);
   }

  /*
   *  *** 8-bit monitor -- RGB image.
   */ 
   else if (tie.visual_info[0].visual->class == PseudoColor)
      {
     /*
      *  *** Allocate buffer space for LUTs.
      */
      rlut = (unsigned char *)malloc(256);
      if (rlut == NULL)
         {
         tie_message(Shell_Adjust, "Insufficient memory available");
         return;
         }

      glut = (unsigned char *)malloc(256);
      if (glut == NULL)
         {
         tie_message(Shell_Adjust, "Insufficient memory available");
         free(rlut);
         return;
         }

      blut = (unsigned char *)malloc(256);
      if (blut == NULL)
         {
         tie_message(Shell_Adjust, "Insufficient memory available");
         free(rlut);
         free(glut);
         return;
         }

     /*
      *  ***  Calculate the image data buffer size.
      */
      buf_size =
       tie_planes[tie_composite[tie_map.imgidx].rgbplanes[RED] - 1].dspwin[NL] *
       tie_planes[tie_composite[tie_map.imgidx].rgbplanes[RED] - 1].dspwin[NS];
   
     /*
      *  ***  Compress the 3 band RGB image into a single band of 
      *       image data and 3 LUTs.
      */
      comptr = composite = (unsigned char *)malloc(buf_size);
      if (composite == NULL)
         {
         tie_message(Shell_Adjust, "Insufficient memory available");
         free(rlut);
         free(glut);
         free(blut);
         return;
         }

      rgbptr[RED] = 
       tie_planes[tie_composite[tie_map.imgidx].rgbplanes[RED] - 1].image_data;
      rgbptr[GREEN] = 
       tie_planes[tie_composite[tie_map.imgidx].rgbplanes[GREEN]-1].image_data;
      rgbptr[BLUE] = 
       tie_planes[tie_composite[tie_map.imgidx].rgbplanes[BLUE] - 1].image_data;

      xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
        NUM_IMGCOLORS,&composite,rlut,glut,blut,0,rr,gr,br);

     /*
      *  ***  Build the linear colormap.
      */
      for (i = 0, work_ptr=beg_ptr+start; i < NUM_IMGCOLORS; i++, work_ptr++)
         {
         work_ptr->pixel = tie.color_table[IMG_IDX + i].pixel;
         work_ptr->red = (rlut[i] * 257);
         work_ptr->green = (glut[i] * 257);
         work_ptr->blue = (blut[i] * 257);
         work_ptr->flags = DoRed | DoGreen | DoBlue;
         }

     /*
      *  ***  Store the colormap.
      */
      XStoreColors(tie.display,tie.colormap,beg_ptr,NUM_IMGCOLORS);
      free(comptr);
      free(rlut);
      free(glut);
      free(blut);
      }
   else if ((tie.visual_info[0].visual->class == DirectColor) && (nplanes == 1))
      {
     /*
      *  *** Assign a gray scale ramp to the image data colors.
      */
      offset = (0 - (scaler * 
               (float)(NUM_COLORS + NUM_CSTORE + NUM_WMCOLORS + 1)));
      for (i = IMG_IDX, work_ptr = beg_ptr + start; 
           i < (IMG_IDX + NUM_IMGCOLORS); i++, work_ptr++)
         {
         work_ptr->pixel = tie.color_table[i].pixel;
         work_ptr->red = work_ptr->green = work_ptr->blue = 
             (unsigned short)((float)((i * scaler) + offset)) << 8;
         work_ptr->flags = DoRed | DoGreen | DoBlue;
         }

     /*
      *  *** Store the colormap.
      */
      XStoreColors(tie.display,tie.colormap,beg_ptr,256);
      }
   else if (tie.visual_info[0].visual->class == DirectColor)
      {
     /*
      *  ***  Assign a gray scale ramp to the image data colors.
      */
      offset = (0 - (scaler*(float)(NUM_COLORS + NUM_CSTORE + NUM_WMCOLORS+1)));

      for (i = IMG_IDX, work_ptr = beg_ptr + start; i < (IMG_IDX + 
            NUM_IMGCOLORS); i++,work_ptr++)
         {
         work_ptr->pixel = tie.color_table[i].pixel;
         work_ptr->red = work_ptr->green = work_ptr->blue =
              (unsigned short)((float)((i * scaler) + offset)) << 8;
         work_ptr->flags = DoRed | DoGreen | DoBlue;
         }
     /*
      *  *** Store the colormap.
      */
      XStoreColors(tie.display,tie.colormap,beg_ptr,256);
      }

   if (tie_map.subpanel == Shell_Adjust)
      {
      for (i=0; i < NUM_IMGCOLORS; i++)
         tie_adjust.color_linear[i] = tie_adjust.color_adjust[start + i];
      }
}

/*******************************************************************************
Purpose:
    This routine updates the value used to increment or decrement the slope
    value.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void adjust_map
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XColor       *color_ptr;     /* Pointer to the start of the appropriate
                                   XColor structure.                          */
   int          bright,         /* Current brightness slider value.           */
                contrast;       /* Current contrast slider value.             */
 
   int          color_idx,      /* Index into color table.                    */
                i,              /* Loop index.                                */
                init = FALSE,   /* Indicates whether or not the adjust color
                                   table needs to be initialized.             */
                ncolors,        /* Number of colors to store in colormap.     */
                start;          /* Starting index in the color table array.   */
 
   int          redval,         /* Working color table values.                */
                greenval,
                blueval;
 
   double       rslope;         /* Slope converted to radians and
                                   the tangent of the result.                 */

  /*
   * Based on the visual type, determine which portion of the colormap needs
   * to be modified and stored.
   */
   if (tie.visual_info[0].visual->class == PseudoColor)
      {
      color_idx = 56;
      start = 0;
      ncolors = NUM_IMGCOLORS;
      }
   else
      {
      color_idx = 0;
      start = 56;
      ncolors = 256;
      }
 
   if (tie_adjust.color_adjust == NULL)
      {
      tie_adjust.color_adjust = (XColor *) calloc(ncolors,sizeof(XColor));
 
      if (tie_adjust.color_adjust == NULL)
         {
         tie_message(Shell_Adjust, "Insufficient memory available");
         return;
         }
 
      tie_adjust.adjust_ptr = &tie.color_table[IMG_IDX];
      init = TRUE;
      }
 
  /*
   *  *** Calculate the new color table.
   */
   if ((tie.visual_info[0].visual->class == PseudoColor) && (init))
       {
      /*
       *  *** Copy the current image colors.
       */
       for (i = 0; i < NUM_IMGCOLORS; i++)
           tie_adjust.color_adjust[i] = tie.color_table[color_idx + i];
       }
    else if (init)
       {
      /*
       *  *** Copy the reserved colors as well as the current image colors.
       */
       for (i = 0; i < 256; i++)
           tie_adjust.color_adjust[i] = tie.color_table[i];
       }
 
  /*
   *  ***  Get the current slope (contrast) and offset (brightness) values.
   */
   XtVaGetValues(adjust_contrast_slider,XmNvalue,&contrast,NULL);
   tie_adjust.lut_slope = (float)contrast;
    
   rslope = DEGTORAD(tie_adjust.lut_slope);
   rslope = tan(rslope);
    
   XtVaGetValues(adjust_bright_slider,XmNvalue,&bright,NULL);
   tie_adjust.lut_offset = (float)bright / 100.0 * MAXCOLOR;

   for (i = 0, color_ptr = tie_adjust.adjust_ptr; i<NUM_IMGCOLORS;
       i++, color_ptr++)
       {
      /*
       *  *** Adjust the appropriate components and assign the new values.
       */
       if ((tie_map.flags & MAP_RED) || (tie_map.flags & MAP_ALL))
           {
           redval = (int)(color_ptr->red*rslope + tie_adjust.lut_offset);
           redval   = (redval < MINCOLOR)   ? MINCOLOR : redval;
           redval   = (redval > MAXCOLOR)   ? MAXCOLOR : redval;
           tie_adjust.color_adjust[start + i].red   = (unsigned short)redval;
           }
       if ((tie_map.flags & MAP_GREEN) || (tie_map.flags & MAP_ALL))
           {
           greenval = (int)(color_ptr->green*rslope + tie_adjust.lut_offset);
           greenval = (greenval < MINCOLOR) ? MINCOLOR : greenval;
           greenval = (greenval > MAXCOLOR) ? MAXCOLOR : greenval;
           tie_adjust.color_adjust[start + i].green = (unsigned short)greenval;
           }
       if ((tie_map.flags & MAP_BLUE) || (tie_map.flags & MAP_ALL))
           {
           blueval  = (int)(color_ptr->blue*rslope + tie_adjust.lut_offset);
           blueval  = (blueval < MINCOLOR)  ? MINCOLOR : blueval;
           blueval  = (blueval > MAXCOLOR)  ? MAXCOLOR : blueval;
           tie_adjust.color_adjust[start + i].blue = (unsigned short)blueval;
           }
 
       tie_adjust.color_adjust[start + i].flags
                     = DoRed | DoGreen | DoBlue;
       }
 
  /*
   *  *** Store and apply the new color table.
   */
   XStoreColors(tie.display,tie.colormap, tie_adjust.color_adjust,ncolors);
 
}

