#include "tiepts.h"
#include "util.h"
#include "protos.h"
#include "manage_cursor.h"
#include "adrgsup.h"

#define TILESIZE        128*128
extern int full_tim[MAX_NTILES][MAX_NTILES]; /*Tile index map for full image*/


/*******************************************************************************
Purpose:
   This routine extracts a 512x512 image from the ADRG image file on CD.
   The image is extract about the given line/sample of the overview image.   
   ADRG images are tiled into 128x128 pixel tiles.  These tile are 
   selected such that the given pixel is closest to the center.

By J. Hemmer, USGS/EROS Data Center, 9/92
Modified by
   D. Etrheim, USGS/EROS Data Center,3/97 
        Build the tile index map for the full image to handle full 
        tiles of blak pixels.
 
Algorithm Description:
*******************************************************************************/
int adrg_extract
(
    int line,           /* overview line of selected point      */
    int sample          /* overviewsample of selected point     */
)
{
   Arg          args[2];        /* resource arguments                   */
   FILE         *img;           /* file descriptor for image file       */
   int          cornertile[2],  /* index of upper left corner tile      */
                i, j,           /* loop counters                        */
                procstat,       /* processing status                    */
                status;         /* return value for las functions       */
   long         las_nlt,        /* las number of lines total            */
                las_nst,        /* las number of samples total          */
                las_next,       /* las file status                      */
                las_band,       /* las band value                       */
                las_nbands,     /* las total number of bands to process */
                las_dtype,      /* las data type                        */
                las_nl,         /* las number of lines in image         */
                las_ns,         /* las number of samples in image       */
                las_sl,         /* las starting line                    */
                las_ss;         /* las starting sample                  */
   struct FDESC *las_fd;        /* las file descriptor                  */
   char         las_fname[CMLEN]; /* name of las filename               */
 
   char         title[100];     /* message panel title                  */
   unsigned     
   char         tile1[TILESIZE], /* ADRG tile for band 1                */
                tile2[TILESIZE], /* ADRG tile for band 2                */
                tile3[TILESIZE]; /* ADRG tile for band 3                */
   double       lat, lon;        /* world coordinates of pixel          */

   int skip;                   /* index to skip on the image */

  /*
   *  *** open ADRG image file
   */
   if ((img = fopen(tie_adrg->image[0].name, "r")) == NULL)
      {
      printf("Cannot open IMGFILE %s\n",tie_adrg->image[0].name);
      return(E_FAIL);
      }
 
  /*
   *  *** skip over header
   */
   fseek(img, 2048, 0);

   las_nbands   = 3;
   las_sl       = 1;
   las_ss       = 1;
   las_nl       = 512;
   las_ns       = 512;
   las_dtype    = EBYTE;
   strcpy(las_fname, "extract.img");

  /*
   *  *** remove previous extract image
   */
   if (!access(las_fname, 0))
      c_delfil(las_fname);

   las_nlt = 128;
   las_nst = 128;
   las_next = CREAT;

  /*
   *  *** get ADRG image line/sample from overview line sample
   */
   adrg_pix2ll(&lat, &lon, line, sample, tie_adrg->ovr_ul[0], 
               tie_adrg->ovr_ul[1], tie_adrg->bsovr, tie_adrg->ascovr);
   adrg_ll2pix(&line, &sample, lat, lon, tie_adrg->image[0].img_ul[0], 
               tie_adrg->image[0].img_ul[1], 
               tie_adrg->image[0].bs, tie_adrg->image[0].asc);

  /*
   *  *** compute tile indeces to point is closest to center
   */
   cornertile[0] = line/128;
   cornertile[1] = sample/128;

   if ((line % 128) < 64)
      cornertile[0] -= 2;
   else
      cornertile[0]--;

   if ((sample % 128) < 64)
      cornertile[1] -= 2;
   else
      cornertile[1]--;

   if (cornertile[0] > (tie_adrg->image[0].nrows - 4)) 
      cornertile[0] = tie_adrg->image[0].nrows - 4;
   if (cornertile[1] > (tie_adrg->image[0].ncols - 4)) 
      cornertile[1] = tie_adrg->image[0].ncols - 4;
   if (cornertile[0] <= 0) cornertile[0] = 1;
   if (cornertile[1] <= 0) cornertile[1] = 1;

  /*
   *  *** required for coordinate transformation
   */
   tie_adrg->extline = cornertile[0]*128;
   tie_adrg->extsamp = cornertile[1]*128;

  /*
   *  *** define wait cursor and display process panel
   */
   show_wait_cursor();
 
  /*
   *   *** display processing dialog
   */
   XtPopup(process_dialog_shell,XtGrabExclusive);

  /*
   *  *** Update the processing icon.
   */
   sprintf(title,"Extracting image from CDROM");;
   XtSetArg(args[0], XeNlabelString, title);
   XtSetArg(args[1], XeNupdatePercent, 10);
   XtSetValues(process_widget, args, 2);
   XeProcessMInit(process_widget, 0, 16);
   XeProcessMUpdate(process_widget,0);
   ForceDialog(process_dialog_shell);


  /*
   *  *** seek to tile (cornertile[0], cornertile[1])
   */
   for (i = 0; i < 4; i++)
      {
     /*
      *  *** seek to tile of interest
      */
      skip = 2048 + (3*TILESIZE) * full_tim[cornertile[1]-1][cornertile[0]+i];
      fseek(img, skip, 0);

      for (j = 0; j < 4; j++)
         {
         procstat = XeProcessMUpdate(process_widget, i*4+j);

         if (procstat < 0)
            break;

        /*
         *  *** read each band of tile
         */
         if ((fread(tile1, TILESIZE, 1, img) == 0) ||
             (fread(tile2, TILESIZE, 1, img) == 0) ||
             (fread(tile3, TILESIZE, 1, img) == 0))
            {
            printf("EOF at (%d %d)\n", i, j);
            return(E_FAIL);
            }

        /*
         *  *** write band 1 
         */
         las_ss = las_nst*j+1;
         las_sl = las_nlt*i+1;
         las_band = 1;

         status = put_buf(tile1, &las_nlt, &las_nst, &las_nlt, &las_nst, 
                       &las_sl, &las_ss, &las_nl, &las_ns, &las_next, 
                       &las_band, &las_nbands, &las_fd, las_fname, 
                       &las_dtype, &las_dtype);
         c_eclose(&las_fd);
         if (status != E_SUCC)
         {
            printf("Error putting buffer to band %d of %s",las_band,las_fname);
            return (E_FAIL);
         }

        /*
         *  *** write band 2 
         */
         las_next = INIT;
         las_band = 2;
         status = put_buf(tile2, &las_nlt, &las_nst, &las_nlt, &las_nst, 
                       &las_sl, &las_ss, &las_nl, &las_ns, &las_next, 
                       &las_band, &las_nbands, &las_fd, las_fname, 
                       &las_dtype, &las_dtype);
         c_eclose(&las_fd);
         if (status != E_SUCC)
         {
            printf("Error putting buffer to band %d of %s",las_band,las_fname);
            return (E_FAIL);
         }

        /*
         *  *** write band 3 
         */
         las_band = 3;
         status = put_buf(tile3, &las_nlt, &las_nst, &las_nlt, &las_nst, 
                       &las_sl, &las_ss, &las_nl, &las_ns, &las_next, 
                       &las_band, &las_nbands, &las_fd, las_fname, 
                       &las_dtype, &las_dtype);
         c_eclose(&las_fd);
         if (status != E_SUCC)
         {
            printf("Error putting buffer to band %d of %s",las_band,las_fname);
            return (E_FAIL);
         }
         }

     /*
      *  *** if loading was cancelled, break
      */
      if (procstat < -1)
         break;
      }
 
   fclose(img);

  /*
   * Close the processing message window and turn on the bell to let the user
   * know the image load is complete.
   */
   XtPopdown(process_dialog_shell);
   unshow_wait_cursor();

   return (E_SUCC);
}
