#include <stdio.h>
#include "tiepts.h"
#include "util.h"
#include "mapsup.h"
#include "adrgsup.h"

#define TILESIZE        16384 /* 128*128 */

long full_tim[MAX_NTILES][MAX_NTILES];  /* Tile index map for full image */

/*******************************************************************************
Purpose:
    This routine transforms the overview image from ADRG to las format.
    In addition all general information concerning the overview image
    and corresponding fullres images area loaded into the tiepts' 
    data structures.
 
By J. Hemmer, USGS/EROS Data Center, 9/92
Modified by
   D. Etrheim, USGS/EROS Data Center,3/97 
        Build the tile index map for the full image to handle full
        tiles of blak pixels.
 
Algorithm Description:
*******************************************************************************/
int adrg_over(void)
{
   FILE         *gen;           /* file descriptor of general info file      */
   FILE         *ovr;           /* file descriptor of overview image         */
   struct FDESC *las_fd;        /* las file descriptor                       */
   long         las_band,       /* band number to process                    */
                las_nbands,     /* number of bands to process                */
                las_next,       /* las next flag                             */
                las_nl,         /* number of lines in buffer                 */
                las_nlt,        /* number of lines in image                  */
                las_ns,         /* number of samples in buffer               */
                las_nst,        /* number of samples in image                */
                las_sl,         /* starting line                             */
                las_ss,         /* starting sample                           */
                las_dtype;      /* image data type                           */
   char         las_fname[CMLEN]; /* las file name for overview image        */

   int          darea,          /* file location of data area                */
                i, j, k,        /* loop counters                             */
                ncol,           /* number of columns of tiles                */
                nrow,           /* numer of rows of tiles                    */
                reclen,         /* length in bytes of a given record         */
                status,         /* return status of las functions            */
                units;          /* coordinate units                          */

   long         nsh,            /* number of horizontal subregions */
                nsv,            /* number of vertical subregions   */
                noz,            /* number of ZDR image files       */
                nos;            /* number of source graphics       */
                
   char         buf[128],               /* workspace for ascii file I/O      */
                ch,                     /* TIM presence flag                 */
                lat_str[14],            /* latitude string                   */
                lon_str[15],            /* longitude string                  */
                ncolstr[4],             /* number of columns string          */
                nrowstr[4];             /* number of rows string             */
   unsigned
   char         *tim;                   /* tile index map                    */
   unsigned     
   char         tile1[TILESIZE],        /* image tile for band 1             */
                tile2[TILESIZE],        /* image tile for band 2             */
                tile3[TILESIZE];        /* image tile for band 2             */
   char         ovrfile[256];           /* overview file pathname            */
   struct 
   TIE_ADRGIMG  tmp;                    /* temp pointer for sort images      */

   long loop1,loop2,count;
   char tempstr[6];             /* tim string*/
               

  /*
   *  *** generate output image filename
   */
   strcpy(ovrfile, tie_adrg->genfile);
   strcpy(strrchr(ovrfile, '.'), ".ovr");

  /*
   *  *** generate output image filename
   */
   if (strrchr(tie_adrg->genfile, '/') != NULL)
      strcpy(las_fname, strrchr(tie_adrg->genfile, '/')+1);

   strcpy(strrchr(las_fname, '.'), "_ovr.img");

   if ((gen = fopen(tie_adrg->genfile, "r")) == NULL)
      {
      printf("Cannot open GENFILE\n");
      return(E_FAIL);
      }

   fread(buf, 5, 1, gen);
   buf[5] = '\0';
   sscanf(buf, "%d", &reclen); 

  /*
   *  *** get address of data area and jump to it
   */
   fseek(gen, reclen+12, 0);
   fread(buf, 5, 1, gen); 
   buf[5] = '\0';
   sscanf(buf, "%d", &darea); 
   fseek(gen, darea-17, 1);

   fseek(gen, 6, 1);            /* DSS 01 */

  /*
   *  *** DRF
   */
   fread(buf, 2, 1, gen); 
   buf[2] = '\0';
   sscanf(buf, "%d", &nsh);

   fread(buf, 2, 1, gen); 
   buf[2] = '\0';
   sscanf(buf, "%d", &nsv);

   fread(buf, 2, 1, gen); 
   buf[2] = '\0';
   sscanf(buf, "%d", &noz);

   fread(buf, 2, 1, gen); 
   buf[2] = '\0';
   sscanf(buf, "%d", &nos);

   fseek(gen, 13, 1);           /* skip to base address of data record */
   fread(buf, 5, 1, gen);
   buf[5] = '\0';
   sscanf(buf, "%d", &darea); 
   fseek(gen, darea-19, 1);     /* jump to data record  */

   fread(buf, 6, 1, gen);       /* OVV 01 */
   fseek(gen, 14, 1);           /* DSI */

  /*
   *  *** get asc and bs for overview image
   */
   fseek(gen, 2, 1);
   fread(buf, 8, 1, gen);
   buf[8] = '\0';
   sscanf(buf, "%d", &tie_adrg->ascovr);
 
   fread(buf, 8, 1, gen);
   buf[8] = '\0';
   sscanf(buf, "%d", &tie_adrg->bsovr);
 
  /* 
   *  *** jump to UL coordinates and read
   */
  /*
   *  *** get UL coordinates
   */
   fread(lon_str, 4, 1, gen);
   lon_str[4] = ' ';
   fread(lon_str+5, 2, 1, gen);
   lon_str[7] = ' ';
   fread(lon_str+8, 5, 1, gen);
   lon_str[13] = ' ';
   lon_str[14] = '\0';
 
   fread(lat_str, 3, 1, gen);
   lat_str[3] = ' ';
   fread(lat_str+4, 2, 1, gen);
   lat_str[6] = ' ';
   fread(lat_str+7, 5, 1, gen);
   lat_str[12] = ' ';
   lat_str[13] = '\0';
   fseek(gen, 1, 1);
 
   tie_adrg->ovr_ul[0] = map_dms2deg(lat_str);
   tie_adrg->ovr_ul[1] = map_dms2deg(lon_str);

  /*
   *  *** read number of rows and columns of tiles
   */
   fseek(gen, 24, 1);
   fread(nrowstr, 3, 1, gen);
   fread(ncolstr, 3, 1, gen);
   nrowstr[3] = '\0';
   ncolstr[3] = '\0';
   sscanf(nrowstr, "%d", &nrow);
   sscanf(ncolstr, "%d", &ncol);

  /*
   *  *** get TIM if present
   */
   fseek(gen, 29, 1);
   fread(&ch, 1, 1, gen);
   fseek(gen, 15*3+2, 1);       /* BDF  */
   tim = (unsigned char*)malloc(nrow*ncol);
 
   if (ch == 'Y')
      {
      for (i=0; i < nrow*ncol; i++)
         {
         fread(buf, 5, 1, gen);
         if (strncmp(buf, "00000", 5) != 0)
            tim[i] = 1;
         else
            tim[i]=0;
         }
      }
   else
      for (i = 0; i < nrow*ncol; i++)
         tim[i] = 1;

  /*
   *  *** allocate space for image information
   */
   adrg_free();
   tie_adrg->nimages = noz;
   tie_adrg->image = (struct TIE_ADRGIMG*)
                      malloc(noz*sizeof(struct TIE_ADRGIMG));
 
  /*
   *  *** get information on each image
   */
   for (k = 0; k < tie_adrg->nimages; k++)
      {
     /*
      *  *** get address of data area and jump to it
      */
      fseek(gen,  13, 1);
      fread(buf,  5, 1, gen);
      buf[5] = '\0';
      sscanf(buf, "%d", &darea);
      fseek(gen, darea-17, 1);
    
      fseek(gen, 6, 1);   /* DSI */

     /*
      *  *** read general information for each image
      */
      fread(buf, 4, 1, gen);                           /* product name */
      fread(buf, 8, 1, gen);       /* image name   */
      buf[8] = '\0';
      strcat(buf, ".img");
      strcpy(tie_adrg->image[k].name, tie_adrg->genfile);
      strcpy(strrchr(tie_adrg->image[k].name, '/'), "/");
      strcat(tie_adrg->image[k].name, buf);

      fseek(gen, 14, 1);   
      fread(buf, 3, 1, gen);       /* units */
      buf[3] = '\0';
      sscanf(buf, "%d", &units);
    
      fseek(gen, 21, 1);                /* lower left coordinate */
    
      fread(lon_str, 4, 1, gen);
      lon_str[4] = ' ';
      fread(lon_str+5, 2, 1, gen);
      lon_str[7] = ' ';
      fread(lon_str+8, 5, 1, gen);
      lon_str[13] = ' ';
      lon_str[14] = '\0';
    
      fread(lat_str, 3, 1, gen);
      lat_str[3] = ' ';
      fread(lat_str+4, 2, 1, gen);
      lat_str[6] = ' ';
      fread(lat_str+7, 5, 1, gen);
      lat_str[12] = ' ';
      lat_str[13] = '\0';
      fseek(gen, 1, 1);

      tie_adrg->image[k].img_ul[0] = map_dms2deg(lat_str);
      tie_adrg->image[k].img_ul[1] = map_dms2deg(lon_str);
    
      fseek(gen, 20, 1);                /* upper right coordinate */

     /*
      *  *** read lower right coordinate
      */
      fread(lon_str, 4, 1, gen);
      lon_str[4] = ' ';
      fread(lon_str+5, 2, 1, gen);
      lon_str[7] = ' ';
      fread(lon_str+8, 5, 1, gen);
      lon_str[13] = ' ';
      lon_str[14] = '\0';
 
      fread(lat_str, 3, 1, gen);
      lat_str[3] = ' ';
      fread(lat_str+4, 2, 1, gen);
      lat_str[6] = ' ';
      fread(lat_str+7, 5, 1, gen);
      lat_str[12] = ' ';
      lat_str[13] = '\0';
      fseek(gen, 1, 1);

      tie_adrg->image[k].img_lr[0] = map_dms2deg(lat_str);
      tie_adrg->image[k].img_lr[1] = map_dms2deg(lon_str);

      fread(buf, 8, 1, gen);
      buf[9] = '\0';
      sscanf(buf, "%d", &tie_adrg->scale);
    
      fseek(gen, 8, 1);
      fread(buf, 8, 1, gen);
      buf[8] = '\0';
      sscanf(buf, "%d", &tie_adrg->image[k].asc);
    
      fread(buf, 8, 1, gen);
      buf[8] = '\0';
      sscanf(buf, "%d", &tie_adrg->image[k].bs);

      /* Reset the ul corner to the corner of the padded extract image */
      fread(lon_str, 4, 1, gen);
      lon_str[4] = ' ';
      fread(lon_str+5, 2, 1, gen);
      lon_str[7] = ' ';
      fread(lon_str+8, 5, 1, gen);
      lon_str[13] = ' ';
      lon_str[14] = '\0';
    
      fread(lat_str, 3, 1, gen);
      lat_str[3] = ' ';
      fread(lat_str+4, 2, 1, gen);
      lat_str[6] = ' ';
      fread(lat_str+7, 5, 1, gen);
      lat_str[12] = ' ';
      lat_str[13] = '\0';
      fseek(gen, 1, 1);

      tie_adrg->image[k].img_ul[0] = map_dms2deg(lat_str);
      tie_adrg->image[k].img_ul[1] = map_dms2deg(lon_str);
      fseek(gen, 64, 1);

      fseek(gen, 24, 1);
      fread(nrowstr, 3, 1, gen);
      nrowstr[3] = '\0';
      fread(ncolstr, 3, 1, gen);
      ncolstr[3] = '\0';
      sscanf(nrowstr, "%d", &tie_adrg->image[k].nrows);
      sscanf(ncolstr, "%d", &tie_adrg->image[k].ncols);

      fseek(gen, 29, 1);
      fread(&ch, 1, 1, gen);
      fseek(gen, 1, 1);

      /* Read the tile index map if it exists, Other wise
         create a dummy one used to find location in the full image */
      if (ch == 'Y')
         {
         fseek(gen, 46, 1);
         for (loop1 = 0; loop1 < tie_adrg->image[k].nrows; loop1++)   
             for (loop2 = 0; loop2 < tie_adrg->image[k].ncols; loop2++)   
                 {
                 fread(tempstr, 5, 1, gen);
                 tempstr[5] = '\0';
                 sscanf(tempstr, "%d", &full_tim[loop2][loop1]);
                 }
         }
       else
         {
         count = 1;
         for (loop1 = 0; loop1 < tie_adrg->image[k].nrows; loop1++)   
             for (loop2 = 0; loop2 < tie_adrg->image[k].ncols; loop2++)   
                 full_tim[loop2][loop1]=count++;
         }

       fread(buf, 3*5, 1, gen);  /* red   band definition */
       fread(buf, 3*5, 1, gen);  /* green band definition */
       fread(buf, 3*5, 1, gen);  /* blue  band definition */
       }

   fclose(gen);



  /*
   *  *** sort images from largest to smallest
   */
   for (i = 0; i < noz; i++)
      for (j = i+1; j < noz; j++)
         {
         if ((tie_adrg->image[i].nrows*tie_adrg->image[i].ncols) <
             (tie_adrg->image[j].nrows*tie_adrg->image[j].ncols))
            {
            tmp = tie_adrg->image[i];
            tie_adrg->image[i] = tie_adrg->image[j];
            tie_adrg->image[j] = tmp;
            }
         }
   
  /*
   *  *** open overview image file
   */
   if ((ovr = fopen(ovrfile, "r")) == NULL)
      {
      printf("Cannot open OVRFILE\n");
      return (E_FAIL);
      }
 
   fseek(ovr, 2048, 0);

   las_nbands   = 3;
   las_sl       = 1;
   las_ss       = 1;
   las_nl       = nrow*128;
   las_ns       = ncol*128;
   las_dtype    = EBYTE;

  /*
   *  *** if this overview image already exists, we are done
   *      The ADRG file names should be unique
   */
   if (!access(las_fname, 0))
      {
      free(tim);
      return (E_FAIL);
      }

  /*
   *   ***  otherwise create las image
   else if (c_eopenr(&las_fd, las_fname, &las_nbands, &las_sl, &las_ss, &las_nl,
                &las_ns, &las_acc, &las_dtype, las_opt, &las_nline) != E_SUCC)
      {
      printf("Cannot open LAS file\n");
      return(0);
      }

printf("\n\n");
   c_eclose(&las_fd);
   */

   las_nlt = 128;
   las_nst = 128;
   las_next = CREAT;

  /*
   *  *** convert ADRG overview to las image, tile by tile
   */
   for (i = 0; i < nrow; i++)
      {
      for (j = 0; j < ncol; j++)
         {
        /*
         *  *** check index for blank tile
         */
         if (tim[i*ncol + j])
            {
            if ((fread(tile1, TILESIZE, 1, ovr) == 0) ||
                (fread(tile2, TILESIZE, 1, ovr) == 0) ||
                (fread(tile3, TILESIZE, 1, ovr) == 0))
               {
               printf("EOF at (%d %d)\n", i, j);
               return(E_FAIL);
               }
            }
         else
            {   
            memset(tile1, 0, TILESIZE);
            memset(tile2, 0, TILESIZE);
            memset(tile3, 0, TILESIZE);
            }   

     /*
      *  *** process band 1
      */
      las_ss = las_nst*j+1;
      las_sl = las_nlt*i+1;
      las_band = 1;

      status = put_buf(tile1, &las_nlt, &las_nst, &las_nlt, &las_nst, 
                       &las_sl, &las_ss, &las_nl, &las_ns, &las_next, 
                       &las_band, &las_nbands, &las_fd, las_fname, 
                       &las_dtype, &las_dtype);
      c_eclose(&las_fd);
      if (status != E_SUCC)
      {
         printf("Error putting buffer to band %d of %s",las_band,las_fname);
         return (E_FAIL);
      }

     /*
      *  *** process band 2
      */
      las_next = INIT;
      las_band = 2;
      status = put_buf(tile2, &las_nlt, &las_nst, &las_nlt, &las_nst, 
                       &las_sl, &las_ss, &las_nl, &las_ns, &las_next, 
                       &las_band, &las_nbands, &las_fd, las_fname, 
                       &las_dtype, &las_dtype);
      c_eclose(&las_fd);
      if (status != E_SUCC)
      {
         printf("Error putting buffer to band %d of %s",las_band,las_fname);
         return (E_FAIL);
      }

     /*
      *  *** process band 3
      */
      las_band = 3;
      status = put_buf(tile3, &las_nlt, &las_nst, &las_nlt, &las_nst, 
                       &las_sl, &las_ss, &las_nl, &las_ns, &las_next, 
                       &las_band, &las_nbands, &las_fd, las_fname, 
                       &las_dtype, &las_dtype);
      c_eclose(&las_fd);
      if (status != E_SUCC)
      {
         printf("Error putting buffer to band %d of %s",las_band,las_fname);
         return (E_FAIL);
      }
         }
      }
 
   fclose(ovr);
   free(tim);

   return (E_SUCC);
}
