#include "tiepts.h"
#include "adrgsup.h"

/*******************************************************************************
Purpose:
    This routine converts an ADRG image pixel to lat/lon.
 
By J. Hemmer, USGS/EROS Data Center, 9/92
 
Algorithm Description:
*******************************************************************************/
void adrg_pix2ll
(
    double *lat,
    double *lon,                /* world coordinates                */
    int l,
    int s,                      /* pixel coordinates to convert     */
    double lat_0,
    double lon_0,               /* corner world coordinate of image */
    int bs,
    int asc             /* conversion parameter for image   */
)
{
   double       rl, rs, rbs, rasc;      /* double representations of parms  */
 
   /* 
    * subtract 1 from line & sample because ADRG conversion 
    * formulas assume 0-relative 
    */
   rl = (double)(l-1);
   rs = (double)(s-1);
   rbs = (double)bs;
   rasc = (double)asc;
 
   *lat = lat_0 - (360.* rl/rbs);
   *lon = lon_0 + (360.* rs/rasc);
}

