#include <math.h>
#include <X11/cursorfont.h>
#include "tiepts.h"
#include "convert.h"
#include "matsup.h"
#include "manage_cursor.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine transforms the overlay vectors for alignment with
    image data.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void align_select
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   Cursor       tcursor,        /* transformation cursor                */
                pointer;        
   Arg          args[1];        /* arguments for Xt routines */
   double       angle,          /* degree of rotation                   */
                deltax=0.0,     /* Amount to warp the cursor.           */
                deltay=0.0,
                scalex,         /* scale factor along X axis            */
                scaley,         /* scale factor along Y axis            */
                move,           /* amount of transformation to apply    */
                rx, ry,         /* reference coordinates                */
                vx, vy;         /* vector coordinates                   */
   int         cx, cy,          /* center of zoom window                */
                fnum,           /* file index of nearest node           */
                line,           /* vector line index.                   */
                pt;             /* vector pt index.                     */
   int          transform;      /* newly selected transform */
   unsigned int state;          /* Keyboard state (Shift key?)          */
   unsigned int keycode;        /* Key code from keyboard.              */

   switch(event->type)
      {
      case KeyRelease:
        /*
         *  *** Capture the key code and the keyboard state.
         */
         keycode = event->xkey.keycode;
         state   = event->xkey.state;

        /*
         *  *** Check the keyboard state to see if the Shift key was used.
         */
         if (state == ShiftMask)
            move = 10.0; 
         else
            move = 1.0;

        /*
         *  *** if in zoom matrix, scale down transformation
         */
         if (w == zoom_srchwin_bulletinboard)
            move /= tie_parms.factor;

        /*
         *  *** Set the delta X/Y amount based on the key pressed.
         */
         if ((keycode == tie_cursor.k_key) || (keycode == tie_cursor.up_key))
            deltay -= (double)move;
         else if ((keycode == tie_cursor.j_key) || 
                  (keycode == tie_cursor.down_key))
            deltay += (double)move;
         else if ((keycode == tie_cursor.h_key) ||  
                  (keycode == tie_cursor.left_key))
            deltax -= (double)move;
         else if ((keycode == tie_cursor.l_key) || (keycode ==
            tie_cursor.right_key))
            deltax += (double)move;
         else
            return;

        /*
         *  *** if rotation or scale, find center of window
         */
         if (tie_vector->transform != TRANSLATE)
            {
            cx = tie_zoom[SEARCH].parms.extract_lines/2*tie_parms.factor;
            cy = tie_zoom[SEARCH].parms.extract_lines/2*tie_parms.factor;

            vector_nnode(zoom_srchwin_bulletinboard, cx, cy, &fnum, &line, &pt);
            vx = tie_vector->line[line].pts[pt*2];
            vy = tie_vector->line[line].pts[pt*2+1];
            vect2full(vx, vy, &rx, &ry);
            rx += tie_srch_fullres[0].window[SS];
            ry += tie_srch_fullres[0].window[SL];
            }

        /*
         *  *** perform transformation
         */
         if (tie_vector->transform == TRANSLATE)
            {
            mat_translate(tie_vector->zmatrix, deltax, deltay);
            }
         else if (tie_vector->transform == ROTATE)
            {
            angle = (deltax + deltay)/(100.0*2.0*PI);
            mat_rotate(tie_vector->zmatrix, angle, rx, ry);
            }
         else if (tie_vector->transform == SCALE)
            {
            scalex = 1.0 + deltax/100.0;
            scaley = 1.0 - deltay/100.0;

            mat_scale(tie_vector->zmatrix, scalex, scaley, rx, ry);
            }

        /*
         *  *** force exposure of fullres and zoom windows
         */ 
         if (XtIsRealized(Shell_SrchFull))
            XClearArea(tie.display, XtWindow(fullsrch_win_bulletinboard),
                    0, 0, 0, 0, True);
       
         if (XtIsRealized(Shell_Zoom))
            XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                    0, 0, 0, 0, True);
         break;

      case ButtonPress:
         tie_vector->pressx1 = tie_vector->pressx2 = event->xbutton.x;
         tie_vector->pressy1 = tie_vector->pressy2 = event->xbutton.y;
         
         /* update the align widget with the newly selected transform 
            Note: there is probably a better way to set the radio buttons, 
            but I couldn't find it. */
         XtSetArg(args[0],XmNset, True);
         switch (event->xbutton.button)
         {
            case ROTATE:
               transform = ROTATE;
               XtSetValues(align_type_rotate,args,1);
               XtSetArg(args[0],XmNset, False);
               XtSetValues(align_type_scale,args,1);
               XtSetValues(align_type_translate,args,1);
               break;
            case SCALE:
               transform = SCALE;
               XtSetValues(align_type_scale,args,1);
               XtSetArg(args[0],XmNset, False);
               XtSetValues(align_type_rotate,args,1);
               XtSetValues(align_type_translate,args,1);
               break;
            default: 
               transform = TRANSLATE;
               XtSetValues(align_type_translate,args,1);
               XtSetArg(args[0],XmNset, False);
               XtSetValues(align_type_rotate,args,1);
               XtSetValues(align_type_scale,args,1);
               break;
         }
         tie_vector->transform = transform;

         XtAddEventHandler(w, PointerMotionMask, FALSE, vector_drag, NULL);
         XtAddEventHandler(w, ButtonReleaseMask, FALSE, vector_drag, NULL);

        /*
         *  *** define cursor based on transformation type
         */
         tcursor = None;
         if (event->xbutton.button == TRANSLATE)
            tcursor = XCreateFontCursor(tie.display, XC_fleur);
         else if (event->xbutton.button == ROTATE)
            tcursor = XCreateFontCursor(tie.display, XC_exchange);
         else if (event->xbutton.button == SCALE)
            tcursor = XCreateFontCursor(tie.display, XC_sizing);

         if (tcursor != None)
         {
            set_window_cursor(tie.display,XtWindow(w),tcursor);
         }
         break;

      case ButtonRelease:
         XtRemoveEventHandler(w, PointerMotionMask,FALSE,vector_drag,NULL);
         XtRemoveEventHandler(w, ButtonReleaseMask,FALSE,vector_drag,NULL);

        /*
         *  *** restore original cursor
         */
         if (w != zoom_srchwin_bulletinboard)
         {
            pointer = XCreateFontCursor(tie.display, XC_arrow);
            set_window_cursor(tie.display,XtWindow(w),pointer);
         }
         else
            zoom_dotcursor(Shell_Zoom);
         break;
      }
}

/*******************************************************************************
Purpose:
    This routine switches the transformation type used for alignment.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void align_trans_type
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   tie_vector->transform = (unsigned char)client;
}

/*******************************************************************************
Purpose:
    This routine accepts  the current alignment.  The nearest point to 
    the center is tied to the underlying line/sample.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by T Rockvam, USGS/EROS Data Center, 3/94.
  - reset rtemp if necessary to use correct coordinates in the rt

Algorithm Description:
*******************************************************************************/
void align_accept
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   double       sx, sy,         /* search image coordinates             */
                vx, vy;         /* vector coordinates                   */
   int  fnum,           /* index of vector file                 */
                line,           /* index of line segment                */
                pt,             /* index of point in line segment       */
                x, y,           /* window coordinates                   */
                zx, zy;         /* zoom coordinates                     */

  /*
   *  *** determine center of zoom window
   */
   x = tie_zoom[SEARCH].parms.extract_lines/2*tie_parms.factor;
   y = tie_zoom[SEARCH].parms.extract_lines/2*tie_parms.factor;

  /*
   *  *** if accepted, tie node nearest to center of window to underlying pixel
   */
   if ((strcmp((char *)client, "ACCEPT") == 0) &&
       (vector_nnode(zoom_srchwin_bulletinboard,x,y,&fnum,&line,&pt) == E_SUCC))
      {
      vx = tie_vector->line[line].pts[pt*2];
      vy = tie_vector->line[line].pts[pt*2+1];

     /*
      *  *** if reselected, reassign reference point
      */
      if (tie.flags & TIE_RESELECTING)
         {
         if (tie_select.rtemp_flag)
            {
            tie_select.rtemp[0] = vx;
            tie_select.rtemp[1] = vy;
            }
         else
            {
            tie_select.rtie[tie_select.reselect][0] = vx;
            tie_select.rtie[tie_select.reselect][1] = vy;
            }
         }
      else
         {
         if (tie_select.rtemp_flag)
            {
            tie_select.rtemp[0] = vx;
            tie_select.rtemp[1] = vy;
            }
        else
            {
            if (tie_select.seltie == tie_select.numalloc)
              {
              tie_select.numalloc += MAX_TIE_POINTS;
              tie_select.elev    =(double *)(realloc(tie_select.elev,
                               tie_select.numalloc*sizeof(double)));
              tie_select.rtie    =(void *)(realloc(tie_select.rtie,
                               2*tie_select.numalloc*sizeof(double)));
              tie_select.crtie    =(void *)(realloc(tie_select.crtie,
                               2*tie_select.numalloc*sizeof(double)));
              tie_select.stie    =(void *)(realloc(tie_select.stie,
                               2*tie_select.numalloc*sizeof(double)));
              tie_select.lsrtie  =(void *)(realloc(tie_select.lsrtie,
                               2*tie_select.numalloc*sizeof(double)));
              tie_select.resid   =(void *)(realloc(tie_select.resid,
                               2*tie_select.numalloc*sizeof(double)));
              tie_select.pt_id   =(void *)(realloc(tie_select.pt_id,
                              tie_select.numalloc*PTIDLEN*sizeof(char)));
              tie_select.mapname =(void *)(realloc(tie_select.mapname,
                               tie_select.numalloc*80*sizeof(char)));
              tie_select.mapid   =(void *)(realloc(tie_select.mapid,
                               tie_select.numalloc*80*sizeof(char)));
              tie_select.disabled=(unsigned char *)(realloc(tie_select.disabled,
                             tie_select.numalloc*sizeof(unsigned char)));
              if ((tie_select.elev == NULL) || (tie_select.rtie == NULL) ||
                  (tie_select.crtie == NULL) || (tie_select.stie == NULL) || 
                  (tie_select.lsrtie == NULL) || (tie_select.resid == NULL) || 
                  (tie_select.pt_id == NULL) || (tie_select.mapname == NULL) ||
                  (tie_select.mapid == NULL) || (tie_select.disabled == NULL))
                  {
                  tie_select.numalloc -= MAX_TIE_POINTS;
                  printf("Error allocating memory\n");
                  XBell(tie.display,0);
                  return;
                  }
              }
            tie_select.rtie[tie_select.seltie][0] = vx;
            tie_select.rtie[tie_select.seltie][1] = vy;
            }
         }

      vector_mark();
      tie_select.curref = TRUE;

     /*
      *  *** transform to alignment
      */
      vect2zoom(vx, vy, &sx, &sy);

      zx = (int)(rint(sx));
      zy = (int)(rint(sy));

      select_srchpt(zx, zy);
      }

   XtUnmanageChild(Align_form);
   XtUnrealizeWidget(Shell_Align);
  
  /*
   *  *** if blending was performed, free pixmap
   */
   if (tie_vector->blend_mask != NULL)
      {
      XFreePixmap(tie.display, tie_vector->blend_mask);
      tie_vector->blend_mask = NULL;
      }

  /*
   *  *** restore selection mode event handling
   */
   XtRemoveEventHandler(fullsrch_win_bulletinboard,
               KeyReleaseMask|ButtonPressMask|ButtonReleaseMask,
               FALSE,align_select,NULL);

   XtRemoveEventHandler(zoom_srchwin_bulletinboard,
               KeyReleaseMask|ButtonPressMask|ButtonReleaseMask,
               FALSE,align_select,NULL);

   XtAddEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask,FALSE,
              zoom_button,NULL);

   XtAddEventHandler(zoom_srchwin_bulletinboard,
               KeyReleaseMask|ButtonPressMask|ButtonReleaseMask,
               FALSE,cursor_select,NULL);

   XtAddEventHandler(fullsrch_win_bulletinboard,
                        PointerMotionMask|KeyReleaseMask,
                        FALSE,cursor_pointer,NULL);

   tie_vector->align = FALSE; 

   XtSetSensitive(main_align_button, True);
   XtUnmanageChild(Align_form);
   XtUnrealizeWidget(Shell_Align);

  /*
   *  *** remove vector overlay vectors
   */
   if (XtIsRealized(Shell_Zoom))
      XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                 0,0,0,0,False);

   tie.flags ^= TIE_ALIGNING;
   zoom_dotcursor(Shell_Zoom);
}

