#include "tiepts.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine displays a message to the user and waits for confirmation.
    A value of 1 is returned for "Yes" response, a value of 2 is returned for
    "No".

By John LaVergne, USGS/EROS Data Center, 6/93 based on a program by Dan Heller
described in 'The X-Journal' May-June 1993 pg 42.
See also Section 5.7.2 of O'Reilly Volume 6a, Motif Programmin Manual.

Algorithm Description:
    The program reproduces the U XtAppMainLoop() function as a mini-main loop,
    that exists until the user verifies seeing the message.  This is useful
    for forcing users to note specific events, and eliminates the problem
    where an important message may be 'lost' temporarily while it is in the
    stack, waiting to be flushed.
*******************************************************************************/
int caution
(
   Widget parent,
   char *str
)
{
   static Widget dialog=0;
   XmString text, yes, no; 
   static int answer;
   extern void response();
   XtAppContext app;
   Arg  args[6];                /* widget arguments                     */

   app = XtWidgetToApplicationContext(parent);
   if (!dialog) 
      {
      yes = XmStringCreateLtoR("Yes",XmSTRING_DEFAULT_CHARSET);
      no  = XmStringCreateLtoR("No",XmSTRING_DEFAULT_CHARSET);
      text = XmStringCreateLtoR("Caution",XmSTRING_DEFAULT_CHARSET);
      XtSetArg(args[0],XmNdialogStyle,XmDIALOG_FULL_APPLICATION_MODAL);
      XtSetArg(args[1],XmNcancelLabelString,no);
      XtSetArg(args[2],XmNokLabelString,yes);
      XtSetArg(args[3],XmNdialogTitle,text);
      XtSetArg(args[4],XtNvisual,tie.visual_info[0].visual);
      XtSetArg(args[5],XtNcolormap,tie.colormap);
      dialog = XmCreateWarningDialog(parent,"dialog",args,6);

      XtUnmanageChild(XmMessageBoxGetChild(dialog,XmDIALOG_HELP_BUTTON));
      XtAddCallback(dialog,XmNokCallback,response,(XtPointer)&answer);
      XtAddCallback(dialog,XmNcancelCallback,response,(XtPointer)&answer);
      XmStringFree(yes);
      XmStringFree(no);
      XmStringFree(text);
      }

   answer = 0;
   text = XmStringCreateLtoR(str,XmSTRING_DEFAULT_CHARSET);
   XtVaSetValues(dialog,XmNmessageString,text,NULL);
   XmStringFree(text);
   XtManageChild(dialog);
   XtPopup(XtParent(dialog),XtGrabNone);

   /* Put the cursor into the Caution window so the user can respond  */
   XWarpPointer(tie.display,None,XtWindow(XtParent(dialog)),None,None,
      None,None,120,110);

   /* Simulate the main loop until the user provides an answer */
   while (answer == 0) XtAppProcessEvent(app,XtIMAll);

   XtPopdown(XtParent(dialog));
   return(answer);
}

/*******************************************************************************
Purpose:  
    The user made a response. Set the answer and then destroy the dialog  

*******************************************************************************/

void response
(
   Widget w,
   int *answer,
   XmAnyCallbackStruct *cbs
)
{
   switch (cbs->reason) 
      {
      case XmCR_OK:
         *answer = 1;
         break;
      case XmCR_CANCEL:
         *answer = 2;
         break;
      }
}
