#include <stdlib.h>
#include "tiepts.h"
#include "util.h"
#include "labserv.h"
#include "ltable.h"
#include "adrgsup.h"
#include "matsup.h"
#include "convert.h"
#include "protos.h"

/*******************************************************************************
NAME: GETXY

PURPOSE: This routine converts from sample/line coordinates to X event
         coordinates for a given display window.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void getxy
(
   Widget w,            /* source widget id                     */
   int samp,            /* source sample                        */
   int line,            /* source line                          */
   int *x,              /* destination x coordinate             */
   int *y               /* destination y coordinate             */
)
{
   int          rplane_idx,     /* reference memory plane index         */ 
                splane_idx;     /* search memory plane index            */ 

   splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
   rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;

  /*
   *  *** reference image full resolution 
   */
   if (w == fullref_win_bulletinboard)
      {
      *x = (samp + (tie_planes[rplane_idx].skips - 1) -
              (tie_planes[rplane_idx].window[SS] - 1))
              / tie_planes[rplane_idx].skips
              - 1;

      *y = (line + (tie_planes[rplane_idx].skipl - 1) -
              (tie_planes[rplane_idx].window[SL] - 1))
              / tie_planes[rplane_idx].skipl
              - 1;
      }
  /*
   *  *** search image full resolution 
   */
   else if (w == fullsrch_win_bulletinboard)
      {
      *x = (samp + (tie_planes[splane_idx].skips - 1) -
              (tie_planes[splane_idx].window[SS] - 1))
              / tie_planes[splane_idx].skips
              - 1;

      *y = (line + (tie_planes[splane_idx].skipl - 1) -
              (tie_planes[splane_idx].window[SL] - 1))
              / tie_planes[splane_idx].skipl
              - 1;
      }
  /*
   *  *** reference overview image
   */
   else if (w == ref_image_bulletinboard)
      {
      *x = ((samp + 1) * tie_planes[rplane_idx].skips)
              + (tie_planes[rplane_idx].window[SS] - 1) 
              - (tie_planes[rplane_idx].skips - 1);

      *y = ((line + 1) * tie_planes[rplane_idx].skipl)
              + (tie_planes[rplane_idx].window[SL] - 1) 
              - (tie_planes[rplane_idx].skipl - 1);
      }
  /*
   *  *** search overview image
   */
   else if (w == search_image_bulletinboard)
      {
      *x = ((samp + 1) * tie_planes[splane_idx].skips)
              + (tie_planes[splane_idx].window[SS] - 1) 
              - (tie_planes[splane_idx].skips - 1);

      *y = ((line + 1) * tie_planes[splane_idx].skipl)
              + (tie_planes[splane_idx].window[SL] - 1) 
              - (tie_planes[splane_idx].skipl - 1);
      }
}

/*******************************************************************************
NAME: GETLINESAMP

PURPOSE: This routine converts from X event coordinates to sample/line 
         coordinates for a given display window.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void getlinesamp
(
   Widget w,            /* source widget id                     */
   int srcx, 
   int srcy,            /* source x and y coordinates           */
   int *samp,           /* sample                               */
   int *line            /* line                                 */
)
{
   int          rplane_idx,     /* reference memory plane index         */ 
                splane_idx;     /* search memory plane index            */ 

   splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
   rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;

  /* 
   *  *** reference full res window
   */
   if (w == fullref_win_bulletinboard)
      {
      *samp = tie_ref_fullres[0].window[SS] + srcx;
      *line = tie_ref_fullres[0].window[SL] + srcy;
      }
  /* 
   *  *** search full res window
   */
   else if (w == fullsrch_win_bulletinboard)
      {
      *samp = tie_srch_fullres[0].window[SS] + srcx;
      *line = tie_srch_fullres[0].window[SL] + srcy;
      }
  /* 
   *  *** reference overview window
   */
   else if (w == ref_image_bulletinboard)
      {
      *samp = tie_planes[rplane_idx].window[SS] + srcx;
      *line = tie_planes[rplane_idx].window[SL] + srcy;
      }
  /* 
   *  *** search overview window
   */
   else if (w == search_image_bulletinboard)
      {
      *samp = tie_planes[splane_idx].window[SS] + srcx;
      *line = tie_planes[splane_idx].window[SL] + srcy;
      }
}

/*******************************************************************************
NAME: REF2SRCH

PURPOSE: This routine converts from reference coordinates to search coordinates
         using the rough transformation.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void ref2srch
(
   double   refx,       /* reference coordinates                    */
   double   refy,       /* reference coordinates                    */
   double  *srchx,      /* search coordinates                       */
   double  *srchy       /* search coordinates                       */
)
{ 
   if (tie.flags & TIE_ADRG)
      {
      refx = tie_adrg->extsamp + 256;
      refy = tie_adrg->extline + 512;

      adrg_pix2ll(&refy, &refx, tie_adrg->extline+256, tie_adrg->extsamp, 
                  tie_adrg->image[0].img_ul[0], tie_adrg->image[0].img_ul[1], 
                  tie_adrg->image[0].bs, tie_adrg->image[0].asc);
      }

   *srchx = c_eval(&tie_rt.degree, tie_rt.samp_coef, &refx, &refy);
   *srchy = c_eval(&tie_rt.degree, tie_rt.line_coef, &refx, &refy);
}

/*******************************************************************************
NAME: FULL2OVER

PURPOSE: This routine converts from fullres coordinates to
         overview coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void full2over
(
   Widget w,            /* overview widget ID                   */
   int fullx,           /* fullres window x coordinate          */
   int fully,           /* fullres window y coordinate          */
   int *x,              /* overview window x coordinate         */
   int *y               /* overwiew window y coordinate         */
)
{
   int          rplane_idx,     /* reference memory plane index         */ 
                splane_idx;     /* search memory plane index            */ 
   double       lat, lon;       /* coordinate for ADRG                  */

   splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
   rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;

  /*
   *  *** if ADRG, convert to lat/lon, then to overview line/sample
   */
   if ((w == ref_image_bulletinboard) && (tie.flags & TIE_ADRG))
      {
      adrg_pix2ll(&lat, &lon, fully, fullx, tie_adrg->image[0].img_ul[0],
                  tie_adrg->image[0].img_ul[1], 
                  tie_adrg->image[0].bs, tie_adrg->image[0].asc);
      adrg_ll2pix(y, x, lat, lon, tie_adrg->ovr_ul[0],
               tie_adrg->ovr_ul[1], tie_adrg->bsovr, tie_adrg->ascovr);
      }
   else if (w == ref_image_bulletinboard)
      {
     /*
      *  *** convert to overview coordinates
      */
      *x = ((int)fullx + (tie_planes[rplane_idx].skips - 1)
                 - (tie_planes[rplane_idx].window[SS] - 1))
                 / tie_planes[rplane_idx].skips - 1;
   
      *y = ((int)fully + (tie_planes[rplane_idx].skipl - 1)
                 - (tie_planes[rplane_idx].window[SL] - 1))
                 / tie_planes[rplane_idx].skipl - 1;
      }
   else
      {
     /*
      *  *** convert to overview coordinates
      */
      *x = ((int)fullx + (tie_planes[splane_idx].skips - 1)
                 - (tie_planes[splane_idx].window[SS] - 1))
                 / tie_planes[splane_idx].skips - 1;

      *y = ((int)fully + (tie_planes[splane_idx].skipl - 1)
                 - (tie_planes[splane_idx].window[SL] - 1))
                 / tie_planes[splane_idx].skipl - 1;
      }
}

/*******************************************************************************
NAME: VECT2FULL

PURPOSE: This routine converts from vector coordinates to fullres window
         coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void vect2full
(
   double vx,           /* vector coordinates (i)               */
   double vy,
   double *fx,          /* fullres window coordinates (o)       */
   double *fy
)
{
  /*
   *  *** convert to search image coordinates
   */
   ref2srch(vx, vy, fx, fy);

  /*
   *  *** apply alignment transformation
   */
   mat_transform(tie_vector->zmatrix, fx, fy);

  /*
   *  *** convert to fullres window coordinates
   */ 
   *fx -= (double)tie_srch_fullres[0].window[SS];
   *fy -= (double)tie_srch_fullres[0].window[SL];
}

/*******************************************************************************
NAME: VECT2IMAGE

PURPOSE: This routine converts from the vector base coordinates to search
         projection defined by search image DDR.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development
 2.0            10/93   J.LaVergne      Added fnum parameter to process the
                                          info for the correct element of 
                                          tie_vectfile array.
                                        Added a test to make sure UTM zones
                                          match before exiting early.
                                          (existing test only checked units
                                           and projections to match)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int vect2image
(
   double pts[],                /* points to convert                    */
   int    npts,                 /* number of points to convert          */
   int    fnum                  /* file number selected (first, second  */
)
{
   double       x, y;           /* ddr coordinates                      */
   int          i;              /* loop counter                         */
   int      status;             /* return value                         */
   long         prtprm[2],      /* dummy for GCTP                       */
                out_units,      /* output coordinate units              */
                splane_idx;     /* search memory plane index            */ 

  /*
   *  *** if an image is loaded, get projection info from DDR
   */
   if (tie_mem[SEARCH].image_pixmap == NULL) 
      return(E_SUCC);

   splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

   if (!tie_planes[splane_idx].proj_valid)
      return(E_SUCC);

  /*
   *  *** determine units of search image
   */
   out_units = tie_planes[splane_idx].units;
   
   if ((tie_vectfile[fnum].inproj == tie_planes[splane_idx].ddr.proj_code) &&
       (tie_vectfile[fnum].inzone == tie_planes[splane_idx].ddr.zone_code) &&
       (tie_vectfile[fnum].indatum == tie_planes[splane_idx].ddr.datum_code) &&
       (tie_vectfile[fnum].inunit == out_units)) return(E_SUCC);             

   prtprm[0] = prtprm[1] = FALSE;
  
  /*
   *  *** initialize GCTP conversion
   */
   status = c_transinit(&tie_vectfile[fnum].inproj,
                     &tie_vectfile[fnum].inunit,
                     &tie_vectfile[fnum].inzone,
                     &tie_vectfile[fnum].indatum,
                      tie_vectfile[fnum].inparm,
                     &tie_planes[splane_idx].ddr.proj_code, 
                     &out_units,
                     &tie_planes[splane_idx].ddr.zone_code,
                     &tie_planes[splane_idx].ddr.datum_code,
                     tie_planes[splane_idx].ddr.proj_coef,
                     prtprm, "");

   if (status != E_SUCC)
   {
    /*tjr tie_message(Shell_Vector, "Error converting to vector coordinates");*/
     printf("Error converting to search image projection coordinates\n");
     return(status);
   }

   for (i = 0; i < npts; i++)
      {
      x = pts[2*i];
      y = pts[2*i+1];

      c_trans(&tie_vectfile[fnum].inproj,
             &tie_vectfile[fnum].inunit,
             &tie_planes[splane_idx].ddr.proj_code, 
             &out_units,
             &x, &y,
             &pts[2*i],
             &pts[2*i+1]
             );
      }

   return(status);
}

/*******************************************************************************
NAME: IMAGE2VECT

PURPOSE: This routine converts from the image projection coordinates to
         vector base coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development
 2.0            10/93   J.LaVergne      Added fnum parameter to process the
                                          info for the correct element of 
                                          tie_vectfile array.
                                        Added a test to make sure UTM zones
                                          match before exiting early.
                                          (existing test only checked units
                                           and projections to match)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int image2vect
(
   double pts[],                /* points to convert                    */
   int    npts,                 /* number of points to convert          */
   int    fnum                  /* file number selected (first, second  */
)
{
   double       x, y;           /* ddr coordinates                      */
   int          i;              /* loop counter                         */
   int      status;             /* return value                         */
   long         prtprm[2],      /* dummy for GCTP                       */
                in_units,       /* output coordinate units              */
                splane_idx;     /* search memory plane index            */ 

  /*
   *  *** if an image is loaded, get projection info from DDR
   */
   if (tie_mem[SEARCH].image_pixmap == NULL) 
      return(E_SUCC);

   splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

   if (!tie_planes[splane_idx].proj_valid)
      return(E_SUCC);

  /*
   *  *** determine units of search image
   */
   in_units = tie_planes[splane_idx].units;
  
   if ((tie_vectfile[fnum].inproj == tie_planes[splane_idx].ddr.proj_code) &&
       (tie_vectfile[fnum].inzone == tie_planes[splane_idx].ddr.zone_code) &&
       (tie_vectfile[fnum].indatum == tie_planes[splane_idx].ddr.datum_code) &&
       (tie_vectfile[fnum].inunit == in_units)) return(E_SUCC);


   prtprm[0] = prtprm[1] = FALSE;
  
  /*
   *  *** initialize GCTP conversion
   */
   status = c_transinit(&tie_planes[splane_idx].ddr.proj_code, 
                     &in_units,
                     &tie_planes[splane_idx].ddr.zone_code,
                     &tie_planes[splane_idx].ddr.datum_code,
                     tie_planes[splane_idx].ddr.proj_coef,
                     &tie_vectfile[fnum].inproj,
                     &tie_vectfile[fnum].inunit,
                     &tie_vectfile[fnum].inzone,
                     &tie_vectfile[fnum].indatum,
                      tie_vectfile[fnum].inparm,
                     prtprm, "");

   if (status != E_SUCC)
   {
    /*tjr tie_message(Shell_Vector, "Error converting to vector coordinates");*/
      printf("Error converting to vector coordinates\n");
      return (status);
   }

   for (i = 0; i < npts; i++)
      {
      x = pts[2*i];
      y = pts[2*i+1];

      c_trans(&tie_planes[splane_idx].ddr.proj_code, 
             &in_units,
             &tie_vectfile[fnum].inproj,
             &tie_vectfile[fnum].inunit,
             &x, &y,
             &pts[2*i],
             &pts[2*i+1]
             );
      }

   return(status);
}

/*******************************************************************************
NAME: PROJ2DEGREES

PURPOSE: This routine converts from image projection coordinates to degrees.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development
 2.0            8/98    C.Olsen         Added the imgindex parameter to 
                                        determine which image DDR to use.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int proj2degrees
(
   double pts[],                /* points to convert                    */
   int    npts,                 /* number of points to convert          */
   int    imgindex              /* REFERENCE or SEARCH                  */ 
)
{
   char mesg[1280];
   double       x, y;           /* ddr coordinates                      */
   int          i;
   int          status;         /* return status                        */
   long         in_units,       /* output coordinate units              */
                outproj,
                outunit,
                outzone,
                plane_idx,      /* memory plane index                   */
                prtprm[2];      /* Print option for the Projection
                                   Transformation Package.              */
   double       outparm[15];

  /*
   *  *** if an image is loaded, get projection info from DDR
   */
   plane_idx = tie_composite[imgindex].rgbplanes[RED] - 1;

   if ((tie_mem[imgindex].image_pixmap == NULL) || 
       (!tie_planes[plane_idx].proj_valid))
      return(E_FAIL);

  /*
   *  *** determine units of search image
   */
   in_units = tie_planes[plane_idx].units;

   outproj = 0;
   outunit = 4; /* degrees */
   outzone = NULZONE;
   for (i = 0; i < 15; i++)
       outparm[i] = 0;
   prtprm[0] = prtprm[1] = FALSE;
  
  /*
   *  *** initialize GCTP conversion
   */
   status = c_transinit(&tie_planes[plane_idx].ddr.proj_code, 
                     &in_units,
                     &tie_planes[plane_idx].ddr.zone_code,
                     &tie_planes[plane_idx].ddr.datum_code,
                     tie_planes[plane_idx].ddr.proj_coef,
                     &outproj,
                     &outunit,
                     &outzone,
                     &tie_planes[plane_idx].ddr.datum_code,
                     outparm,
                     prtprm, ""
                     );

   if (status != E_SUCC)
   {
    /*tjr tie_message(Shell_Vector, "Error converting to vector coordinates");*/
      sprintf(mesg,"Error converting to degrees\nThis image will not support coordinate conversion.\nExit out of TIEPTS to clear coordinate selection");
      tie_message(Shell_Main,mesg);
   }

   for (i = 0; i < npts; i++)
      {
      x = pts[2*i];
      y = pts[2*i+1];

      c_trans(&tie_planes[plane_idx].ddr.proj_code, 
             &in_units,
             &outproj,
             &outunit,
             &x, &y,
             &pts[2*i],
             &pts[2*i+1]
             );
      }

   return(status);
}

/*******************************************************************************
NAME: VECT2ZOOM

PURPOSE: This routine converts points in vector coordinates to
         zoom coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void vect2zoom
(
   double vx,           /* vector coordinate            */
   double vy,
   double *x,           /* zoom coordinates (output)    */
   double *y
)
{
   double       fwx, fwy;       /* fullres window coordinates   */
   int          zx, zy;         /* zoom window coordiantes      */

   vect2full(vx, vy, &fwx, &fwy);

   full2zoom(fwx  + tie_srch_fullres[0].window[SS],
             fwy  + tie_srch_fullres[0].window[SL], &zx, &zy,
             tie_srch_fullres[0].window[SL],
             tie_srch_fullres[0].window[SS]);

   *x = (double)zx;
   *y = (double)zy;
}

/*******************************************************************************
NAME: FULL2ZOOM

PURPOSE: This routine converts fullres coordinates to zoom coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            5/92    L.Huewe         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void full2zoom
(
    double xsub,                /* X location of input point (subpixel
                                   accuracy) (I)                              */
    double ysub,                /* Y location of input point (subpixel
                                   accuracy) (I)                              */
    int    *xcursor,            /* Zero relative X position in window. (O)    */
    int    *ycursor,            /* Zero relative Y position in window. (O)    */
    int    fsl,                 /* Starting line (file coordinate) of the image
                                   the coordinate is being calculated for (I) */
    int    fss                  /* Starting sample (file coordinate) of the 
                                   image the coordinate is being calculated 
                                   for (I)                                    */
)
{
double  x,                      /* X coordinate                               */
        xfactor,                /* Subpixel factor to add to file coordinate. */
        xyadjust,               /* Adjustment needed to the subpixel value
                                   due to short cuts taken when the image 
                                   window was enlarged                        */
        y,                      /* Y coordinate                               */
        yfactor;                /* Subpixel factor to add to file coordinate. */

int     lborder,                /* Number of pixels thrown away from the left
                                   edge of the extracted window               */
        left,                   /* Remainder of a division operation          */
        sl,                     /* Starting line of extracted window.         */
        ss,                     /* Startling sample of extracted window.      */
        tborder,                /* Number of pixels thrown away from the top
                                   edge of the extracted window               */
        xfil,                   /* X file relative coordinate.                */
        ximg,                   /* X image relative coordinate.               */
        yfil,                   /* Y file relative coordinate.                */
        yimg;                   /* Y image relatvie coordinate.               */

  /*
   *  *** Determine how many pixels from the window extracted for zooming 
   *      were thrown away by the resampling routine. 
   */
   left = tie_zoom[SEARCH].parms.horiz_dim % 2;

   if (left == 0)
      lborder = tie_zoom[SEARCH].parms.horiz_dim / 2 - 1;
   else
      lborder = tie_zoom[SEARCH].parms.horiz_dim / 2;

   left = tie_zoom[SEARCH].parms.vert_dim % 2;
   if (left == 0)
      tborder = tie_zoom[SEARCH].parms.vert_dim / 2 - 1;
   else
      tborder = tie_zoom[SEARCH].parms.vert_dim / 2;

  /*
   * Calculate the adjustment needed due to short cuts taken when the image
   * window was zoomed.
   */
   xyadjust = ((int)(tie_parms.factor / 2.0)) * (1.0 / 
               (double)(tie_parms.factor));  

  /*
   * Determine the starting line and sample based on the window extracted 
   * and factor this into the file coordinate.
   */
   ss = tie_zoom[SEARCH].x - tie_zoom[SEARCH].parms.extract_samps / 2 + 1;
   sl = tie_zoom[SEARCH].y - tie_zoom[SEARCH].parms.extract_lines / 2 + 1;

   xfil = (int)xsub - (ss + lborder - 1);
   yfil = (int)ysub - (sl + tborder - 1);

   ximg = (xfil - (fss - 1));
   yimg = (yfil - (fsl - 1));

   xfactor = xsub - (int)xsub;
   yfactor = ysub - (int)ysub;

   x = ximg + xfactor;
   y = yimg + yfactor;

   *xcursor = ((x - xyadjust) - .5 - (1.0 / (2 * tie_parms.factor))) *
              tie_parms.factor;
   *ycursor = ((y - xyadjust) - .5 - (1.0 / (2 * tie_parms.factor))) *
              tie_parms.factor;

   return;
}

/*******************************************************************************
NAME: DEGREES2VECT

PURPOSE: This routine converts from lat/long degrees to vector base
         coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            4/93    L.Hueew         Initial Development
 2.0            10/93   J.LaVergne      Added the fnum parameter to process
                                        the info for the correct(current)
                                        element of tie_vectfile array.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int degrees2vect
(
   double pts[],                /* points to convert                    */
   int    npts,                 /* number of points to convert          */
   int    fnum                  /* file number selected (first, second  */
)
{
   double       x, y,           /* point to convert                     */ 
                parm[15];       /* input projection parameters          */

   int          i;              /* loop counter                         */
   int      status;             /* return value                         */
   long     geo = 0,    /* geographic projection                */
                prtprm[2],      /* dummy for GCTP                       */
                units = 4,      /* output coordinate units(degrees)     */
                zone = NULZONE; /* Zone number.                         */

   if ((tie_vectfile[fnum].inproj == 0) && (tie_vectfile[fnum].inunit == units))
      return(E_SUCC);

   /*
    *  *** initialize variables
    */
   for (i=0; i < 15; i++)
       parm[i] = 0;
   prtprm[0] = prtprm[1] = FALSE;
  
   /*
   *  *** initialize GCTP conversion
   */
   status = c_transinit(&geo,
                     &units,
                     &zone,
                     &tie_vectfile[fnum].indatum,
                     parm,
                     &tie_vectfile[fnum].inproj,
                     &tie_vectfile[fnum].inunit,
                     &tie_vectfile[fnum].inzone,
                     &tie_vectfile[fnum].indatum,
                      tie_vectfile[fnum].inparm,
                     prtprm, "");

   if (status != E_SUCC)
   {
    /*tjr tie_message(Shell_Vector, "Error converting to vector coordinates");*/
      printf("Error converting to vector coordinates\n");
      return (status);
   }
   
   /*
    *  *** Convert each input point from decimal degrees to vector coordinates. 
    */
   for (i = 0; i < npts; i++)
      {
      x = pts[2*i];
      y = pts[2*i+1];

      c_trans(&geo, 
             &units,
             &tie_vectfile[fnum].inproj,
             &tie_vectfile[fnum].inunit,
             &x, &y,
             &pts[2*i],
             &pts[2*i+1]
             );
      }

   return(status);
}

/*******************************************************************************
NAME: DEGREES2IMAGE

PURPOSE: This routine converts from lat/long degrees to image projection
         coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            4/93    L.Huewe         Initial Development
 2.0            11/93   T.Rockvam       Initialized the parms array to zeros

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int degrees2image
(
   double pts[],                /* points to convert                    */
   int    npts,                 /* number of points to convert          */
   int    imgidx                /* REFERENCE or SEARCH                  */
)
{
   double       parms[15],      /* projection parameters                */
                x, y;           /* ddr coordinates                      */

   int          i;              /* loop counter                         */
   int      status;             /* return value                         */
   long         geo = 0,        /* geographic projection                */
                prtprm[2],      /* dummy for GCTP                       */
                out_units,      /* output coordinate units              */
                plane_idx,      /* search memory plane index            */ 
                units = 4,      /* output coordinate units(degrees)     */
                zone = NULZONE; /* Zone number.                         */

  /*
   *  *** if an image is loaded, get projection info from DDR
   */
   if (tie_mem[imgidx].image_pixmap == NULL) 
      return(E_SUCC);

   plane_idx = tie_composite[imgidx].rgbplanes[RED] - 1;

   if (!tie_planes[plane_idx].proj_valid)
      return(E_SUCC);

  /*
   *  *** determine units of image
   */
   out_units = tie_planes[plane_idx].units;
  
   if ((tie_planes[plane_idx].ddr.proj_code == 0) && (out_units == 4))
      return(E_SUCC);

   for (i=0; i < 15; i++)
      parms[i] = 0.0;
   prtprm[0] = prtprm[1] = FALSE;
  
  /*
   *  *** initialize GCTP conversion
   */
   status = c_transinit(&geo,
                     &units,
                     &zone,
                     &tie_planes[plane_idx].ddr.datum_code,
                     parms,
                     &tie_planes[plane_idx].ddr.proj_code, 
                     &out_units,
                     &tie_planes[plane_idx].ddr.zone_code,
                     &tie_planes[plane_idx].ddr.datum_code,
                     tie_planes[plane_idx].ddr.proj_coef,
                     prtprm, "");

   if (status != E_SUCC)
   {
     printf("Error converting to projection coordinates\n");
     return (status);
   }
   
   /*
    *  *** Convert each input point from decimal degrees to proj coordinates.
    */
   for (i = 0; i < npts; i++)
      {
      x = pts[2*i];
      y = pts[2*i+1];
      c_trans(&geo,
             &units,
             &tie_planes[plane_idx].ddr.proj_code, 
             &out_units,
             &x, &y,
             &pts[2*i],
             &pts[2*i+1]
             );
      }

   return(status);
}

/*******************************************************************************
NAME: MAP_PROJ2DEGREES

PURPOSE: This routine converts from the map projection coordinates to
         degrees.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            5/94    T.Rockvam       Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int map_proj2degrees
(
   double pts[],                /* points to convert                    */
   int    npts,                 /* number of points to convert          */
   char   *file_key             /* proj file name and projection key    */
)
{
   struct TAB_DEF tab;          /* Tabular structure of label table     */
   struct VECTOR *keyptr;       /* Pointer to projection key field      */
   struct VECTOR *typeptr;      /* Pointer to projection code field     */
   struct VECTOR *zoneptr;      /* Pointer to projection zone field     */
   struct VECTOR *unitptr;      /* Pointer to units field               */
   struct VECTOR *sphptr;       /* Pointer to spheroid field            */
   struct VECTOR *parmptr;      /* Pointer to proj parms field          */

   char         *file,          /* lt projection file                   */
                *key;           /* key of desired projection            */

   int         i,               /* looping counter                      */
               status,         /* return status                 */
                idpos;          /* @ index                              */
   long inproj,         /* input projection code                */
                inunit,         /* input projection unit                */
                inzone,         /* input projection zone                */
                indatum,        /* input projection datum               */
                outproj,        /* output projection code               */
                outunit,        /* output projection unit               */
                outzone,        /* output projection zone               */
                mode = 4,       /* access mode for lsstat               */
                prtprm[2];      /* GCTP print option                    */

   double       x, y;           /* coordinates                          */
   double       outparm[15];
   double       inparm[15];

  /*
   *  *** parse out the lt filename and the projection key
   */
   if (strcmp(file_key,"") == 0)
       return(E_FAIL);
   idpos = (int)strcspn(file_key,"@");
   if (idpos == strlen(file_key))
       return(E_FAIL);
   else
       {
       file = TieMalloc(idpos+1);
       strncpy(file,file_key,idpos);
       file[idpos] = '\0';
       key = TieMalloc(strlen(file_key)-idpos);
       strncpy(key,file_key+idpos+1,strlen(file_key)-idpos-1);
       key[strlen(file_key)-idpos-1] = '\0';
       }

  /*
   *  *** get the input proj info from the indicated lt and projection key
   */
   if (c_lsstat(file, &mode) != E_SUCC) return(E_FAIL);
   open_tab(&tab, file, IREAD);
   get_vector(&tab);
   keyptr = get_field_ptr(&tab, "PROJKEY");
   if (keyptr == NULL) { close_tab(&tab); return(E_FAIL); }
   typeptr = get_field_ptr(&tab, "PROJTYPE");
   if (typeptr == NULL) { close_tab(&tab); return(E_FAIL); }
   zoneptr = get_field_ptr(&tab, "PROJZONE");
   if (zoneptr == NULL) { close_tab(&tab); return(E_FAIL); }
   unitptr = get_field_ptr(&tab, "PROJUNITS");
   if (unitptr == NULL) { close_tab(&tab); return(E_FAIL); }
   sphptr = get_field_ptr(&tab, "PROJSPH");
   if (sphptr == NULL) { close_tab(&tab); return(E_FAIL); }
   parmptr = get_field_ptr(&tab, "PROJPARMS");
   if (parmptr == NULL) { close_tab(&tab); return(E_FAIL); }

   for (;;)
      {
      status = get_record(&tab);
      if (status == E_FAIL) { close_tab(&tab); return(E_FAIL); }
      if (status == E_EOF) { close_tab(&tab); return(E_FAIL); }
      if (strcmp(keyptr->ptr.c, key) == 0)
           {
           inproj = *typeptr->ptr.i4;
           inunit = *unitptr->ptr.i4;
           inzone = *zoneptr->ptr.i4;
           indatum = *sphptr->ptr.i4;
           for (i = 0; i < 15; i++)
              inparm[i] = parmptr->ptr.r8[i];
           break;
           }
      }
   close_tab(&tab);

  /*
   *  *** set up the output proj info
   */
   outproj = 0;
   outunit = 4; /* degrees */
   outzone = NULZONE;
   for (i = 0; i < 15; i++)
       outparm[i] = 0;
   prtprm[0] = prtprm[1] = FALSE;
 
  /*
   *  *** initialize GCTP conversion
   */
   status = c_transinit(&inproj,&inunit,&inzone,&indatum,inparm,
                     &outproj,&outunit,&outzone,&indatum,outparm,
                     prtprm, "");

   if (status != E_SUCC)
      {
      printf("Error converting to degrees\n");
      return(status);
      }

   for (i = 0; i < npts; i++)
      {
      x = pts[2*i];
      y = pts[2*i+1];

      c_trans(&inproj,&inunit,
             &outproj,&outunit,
             &x,&y,&pts[2*i],&pts[2*i+1]);
      }

   return(E_SUCC);
}

/*******************************************************************************
NAME: PROJ2PROJ

PURPOSE: This routine converts from the one projection coordinate to another
         projection.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            5/94    T.Rockvam       Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int proj2proj
(
   struct DDR *in_ddr,          /* Reference image DDR (not used if I2M) (I) */ 
   struct DDR *out_ddr,         /* Search image DDR (I) */
   double     pts[],            /* points to convert                    */
   int        npts              /* number of points to convert          */
)
{
   long         i,              /* looping counter                      */
                inunit,         /* input projection unit                */
                outunit,        /* output projection unit               */
                status,         /* return status                        */
                prtprm[2];      /* GCTP print option                    */

   double       x, y;           /* coordinates                          */

  /*
   *  *** set up the units & proj info
   */
   if (c_ckunit(in_ddr->proj_units, &inunit) != E_SUCC)
   {
        return (E_FAIL);
   }
   if (c_ckunit(out_ddr->proj_units, &outunit) != E_SUCC)
   {
        return (E_FAIL);
   }

   prtprm[0] = prtprm[1] = FALSE;
 
  /*
   *  *** initialize GCTP conversion
   */
   status = c_transinit(&in_ddr->proj_code,&inunit,&in_ddr->zone_code,
                     &in_ddr->datum_code,in_ddr->proj_coef,
                     &out_ddr->proj_code,&outunit,&out_ddr->zone_code,
                     &out_ddr->datum_code,out_ddr->proj_coef,
                     prtprm, "");

   if (status != E_SUCC)
      {
      printf("Error converting projection coordinates\n");
      return(status);
      }

   for (i = 0; i < npts; i++)
      {
      x = pts[2*i];
      y = pts[2*i+1];

      c_trans(&in_ddr->proj_code,&inunit,
            &out_ddr->proj_code,&outunit,
            &x,&y,&pts[2*i],&pts[2*i+1]);
      }

   return(E_SUCC);
}

/*******************************************************************************
NAME: IMAGE2PROJ

PURPOSE: This routine converts line/sample coordinates to projection
         coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            8/98    C.Olsen         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void image2proj
(
   int    imgindex,         /* SEARCH or REFERENCE                        */
   double pt[],             /* array for projection values                */
   double x, 
   double y                 /* input line/sample values                   */
)
{
   long         icoords[8],     /* image corner line/sample coordinates       */
                msgflg = FALSE, /* specifies that error messages are not to
                                   be output by routine                       */
                plane_idx,      /* memory plane index                         */
                status;         /* status indicator                           */

   plane_idx = tie_composite[imgindex].rgbplanes[RED] - 1;
   icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
   icoords[2] = icoords[6] = tie_planes[plane_idx].ddr.nl;
   icoords[5] = icoords[7] = tie_planes[plane_idx].ddr.ns;
   status = c_rtcoef(tie_planes[plane_idx].ddr.upleft, icoords,
      tie_cursor.coef, &msgflg);
   if (status == E_SUCC)
  /*
   * *** ddr is valid
   */
      {
      pt[1] = (double) (tie_cursor.coef[0] * y) + (tie_cursor.coef[1] * x) +
              tie_cursor.coef[2];
      pt[0] = (double) (tie_cursor.coef[3] * y) + (tie_cursor.coef[4] * x) +
              tie_cursor.coef[5];
      }
   else
      {
      pt[1] = 0.0;
      pt[0] = 0.0;
      }
}

/*******************************************************************************
NAME: PROJ2IMAGE

PURPOSE: This routine converts projection coordinates to line/sample
         coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            8/98    C.Olsen         Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void proj2image
(
   int    imgindex,       /* SEARCH or REFERENCE                        */
   double pt[],           /* array for projection values                */
   double x, 
   double y               /* input line/sample values                   */
)
{
   long         icoords[8],     /* image corner line/sample coordinates       */
                msgflg = FALSE, /* specifies that error messages are not to
                                   be output by routine                       */
                plane_idx,      /* memory plane index                         */
                status;         /* status indicator                           */
   double       temp_cc[10];

   plane_idx = tie_composite[imgindex].rgbplanes[RED] - 1;
   icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
   icoords[2] = icoords[6] = tie_planes[plane_idx].ddr.nl;
   icoords[5] = icoords[7] = tie_planes[plane_idx].ddr.ns;
   status = c_rtcoef(tie_planes[plane_idx].ddr.upleft, icoords,
      temp_cc, &msgflg);
   if (status == E_SUCC)
      {
  /*
   * *** ddr is valid
   */
      status = c_invert(temp_cc,tie_cursor.coef);
      pt[1] = (double) (tie_cursor.coef[0] * y) + (tie_cursor.coef[1] * x) +
              tie_cursor.coef[2];
      pt[0] = (double) (tie_cursor.coef[3] * y) + (tie_cursor.coef[4] * x) +
              tie_cursor.coef[5];
      }
   else
      {
      pt[1] = 0.0;
      pt[0] = 0.0;
      }
}

/*******************************************************************************
NAME: CHECK_DDR

PURPOSE: This routine checks the validity flags of a DDR

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            3/99    T.Ratliff       Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int check_ddr
(
    struct DDR *img_ddr              /* Image DDR to check     */
)
{
  if (img_ddr->valid[0] != VALID) return(E_FAIL);
  if (img_ddr->valid[1] != VALID) return(E_FAIL);
  if (img_ddr->valid[2] != VALID) return(E_FAIL);
  if (img_ddr->valid[3] != VALID) return(E_FAIL);
  if (img_ddr->valid[4] != VALID) return(E_FAIL);
  if (img_ddr->valid[5] != VALID) return(E_FAIL);
  if (img_ddr->valid[6] != VALID) return(E_FAIL);
  return(E_SUCC);
}

/*******************************************************************************
NAME: CONV_LS

PURPOSE: This routine converts lat/long or projection coordinates to
         line/sample coordinates.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            3/99    T.Ratliff       Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void conv_ls
(
  char *ctyp,
  int  idx
)
{
  double tmppt[2];
  tmppt[0] = tie_select.rtie[idx][0];
  tmppt[1] = tie_select.rtie[idx][1];
  if (strcmp(ctyp,"GEO") == 0)
    degrees2image(tmppt,1,REFERENCE);
  proj2image(REFERENCE,tie_select.lsrtie[idx],
             tmppt[0],tmppt[1]);
}

/*******************************************************************************
NAME: VECT2VECT

PURPOSE: This routine converts all vectors to have the same projection
         as the first vector file read.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            4/00    T.Beckmann      Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
int vect2vect
(
   double pts[],                /* points to convert                    */
   int    npts,                 /* number of points to convert          */
   int    fnum                  /* file number selected (first, second  */
)
{
    double x, y;           /* ddr coordinates                      */
    int    i;              /* loop counter                         */
    int    status;         /* return value                         */
    long   prtprm[2];      /* dummy for GCTP                       */

    /* only convert the coordinates if they're not already in the correct
       projection */ 
    if ((tie_vectfile[fnum].inproj == tie_vectfile[0].inproj) &&
        (tie_vectfile[fnum].inzone == tie_vectfile[0].inzone) &&
        (tie_vectfile[fnum].inunit == tie_vectfile[0].inunit) &&
        (tie_vectfile[fnum].indatum == tie_vectfile[0].indatum)) 
    {
        return(E_SUCC);
    }
    
    prtprm[0] = prtprm[1] = FALSE;
  
    /* initialize GCTP conversion to convert to geographic degrees */
    status = c_transinit(&tie_vectfile[fnum].inproj,
                         &tie_vectfile[fnum].inunit,
                         &tie_vectfile[fnum].inzone,
                         &tie_vectfile[fnum].indatum,
                          tie_vectfile[fnum].inparm,
                         &tie_vectfile[0].inproj, 
                         &tie_vectfile[0].inunit,
                         &tie_vectfile[0].inzone,
                         &tie_vectfile[0].indatum,
                          tie_vectfile[0].inparm,
                          prtprm, "");

    if (status != E_SUCC)
    {
        printf("Error converting to search image projection coordinates\n");
        return(status);
    }

    for (i = 0; i < npts; i++)
    {
        x = pts[2*i];
        y = pts[2*i+1];

        c_trans(&tie_vectfile[fnum].inproj,
                &tie_vectfile[fnum].inunit,
                &tie_vectfile[0].inproj, 
                &tie_vectfile[0].inunit,
                &x, &y,
                &pts[2*i],
                &pts[2*i+1]);
    }

    return(status);
}
