/*
 *  This is the main source file for the Tie Point Selection (tiepts) 
 *  application.
 *
 *  By J. Hemmer/D. Akkerman/L. Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *  Modified by J. LaVergne USGS/EROS Data Center 7/93
 *    - Changed the main tiepts window title line to show which mode was 
 *      selected, and fixed an error where the icon name for both images
 *      was 'Reference'.
 *    - Changed the Cursor panel to appear in the lower center of the
 *      display - used to cover the image in the upper left corner.
 *  Modified by T Rockvam, USGS/EROS Data Center, 3/94.
 *    - added shell for dlg line types
 */
#include "tiepts.h"
#include <X11/cursorfont.h>
#include "protos.h"
#include "cr.h"

void Create_Shells (void)
{
   Arg          args[256];      /* X resource arguments.                */
   Widget       vector_form;    /* Return status variable.              */
   long         argcnt;         /* Resource argument count.             */

   /*
    *  Create the main application shell.
    */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNallowShellResize, False); argcnt++;
    if (tie.mode == IM2IM) 
       {
       if (tie.flags == TIE_ADRG)
          XtSetArg(args[argcnt], XmNtitle, "Tie Points - Image to ADRG"); 
       else
          XtSetArg(args[argcnt], XmNtitle, "Tie Points - Image to Image");
       }
    else if (tie.mode == IM2MAP)
       XtSetArg(args[argcnt], XmNtitle, "Tie Points - Image to Map");
    else if (tie.mode == IM2VECT)
       XtSetArg(args[argcnt], XmNtitle, "Tie Points - Image to Vector");
    else if (tie.mode == ONESRCH)
       XtSetArg(args[argcnt], XmNtitle, "Tie Points - Single Search");
    argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "Tie Points"); argcnt++;
    XtSetArg(args[argcnt], XmNx, 478); argcnt++;
    XtSetArg(args[argcnt], XmNy, 559); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
    if (!tie.userplace)
       {
       XtSetArg(args[argcnt],XtNgeometry, "+0-0"); 
       argcnt++;
       }
    Shell_Main = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        applicationShellWidgetClass, tie.display, args, argcnt);

    XtAddCallback(Shell_Main, XmNdestroyCallback,  main_destroy, (XtPointer)1);

   /*
    *  Create the main image window shells.
    */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "Tie Points - Reference Image"); argcnt++;
    if (!tie.userplace)
       {
       XtSetArg(args[argcnt], XtNgeometry, "+0+0"); 
       argcnt++;
       }
    XtSetArg(args[argcnt], XmNiconName, "Reference Image"); argcnt++;
    XtSetArg(args[argcnt], XmNx, 603); argcnt++;
    XtSetArg(args[argcnt], XmNy, 504); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;

    Shell_RefImg = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
    XtAddCallback(Shell_RefImg, XmNdestroyCallback, image_destroy,"Reference");

    XtSetArg(args[0], XmNtitle, "Tie Points - Search Image");
    XtSetArg(args[1], XmNiconName, "Search Image"); 

    if (!tie.userplace)
       {
       if ((tie.mode == IM2IM) || (tie.mode == IM2VECT))
          XtSetArg(args[2], XtNgeometry, "-0+0");
       else
          XtSetArg(args[2], XtNgeometry, "+0+0");
       }
    Shell_SrchImg = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
    XtAddCallback(Shell_SrchImg, XmNdestroyCallback, image_destroy, "Search");

   /*
    *  *** Create the full resolution window shells.
    */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "Reference - Full Res"); argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "Ref Fullres"); argcnt++;
    if (!tie.userplace)
       {
       XtSetArg(args[argcnt], XtNgeometry, "+400-0"); 
       argcnt++;
       }
    XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
    Shell_RefFull = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
    XtAddCallback(Shell_RefFull, XmNdestroyCallback, fullres_destroy, 0);

    XtSetArg(args[0], XmNtitle, "Search - Full Res"); 
    XtSetArg(args[1], XmNiconName, "Srch Fullres");
    if (!tie.userplace)
       {
       if (tie.mode == IM2VECT)
          XtSetArg(args[2], XtNgeometry, "+491+692"); 
       else if (tie.mode == IM2MAP)
          XtSetArg(args[2], XtNgeometry, "-240-350");    
       else
          XtSetArg(args[2], XtNgeometry, "+675-0"); 
       }
    Shell_SrchFull = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
    XtAddCallback(Shell_SrchFull, XmNdestroyCallback, fullres_destroy, 0);

   /*
    *  Create the zoom window shell.
    */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "Tie Points - Zoom"); argcnt++;
    if (!tie.userplace)
       {
       if (tie.mode == IM2VECT)
          XtSetArg(args[argcnt], XtNgeometry, "+491+372");
       else if (tie.mode == IM2MAP)
          XtSetArg(args[argcnt], XtNgeometry, "-128+128"); 
       else
          XtSetArg(args[argcnt], XtNgeometry, "+440+250"); 
       argcnt++;
       }
    XtSetArg(args[argcnt], XmNiconName, "Zoom"); argcnt++;
    XtSetArg(args[argcnt], XmNx, 796); argcnt++;
    XtSetArg(args[argcnt], XmNy, 462); argcnt++;
    XtSetArg(args[argcnt], XtNvisual,tie.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt], XtNcolormap,tie.colormap); argcnt++;
    XtSetArg(args[argcnt], XmNkeyboardFocusPolicy, XmPOINTER); argcnt++;
    Shell_Zoom = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);

   /*
    *  *** Create the load panel shell.
    */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNallowShellResize, False); argcnt++;
    if (!tie.userplace)
       {
       XtSetArg(args[argcnt], XtNgeometry, "+350-0"); 
       argcnt++;
       }
    XtSetArg(args[argcnt], XmNtitle, "Tie Points - Load"); argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "Load"); argcnt++;
    XtSetArg(args[argcnt], XmNx, 350); argcnt++;
    XtSetArg(args[argcnt], XmNy, 375); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
    Shell_Load = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
    Load_fileselector = NULL;
    XtAddCallback(Shell_Load, XmNdestroyCallback, load_destroy, (XtPointer)1);

   /*
    *  *** Create the cursor panel shell.
    */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "Tie Points - Cursor"); argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "Cursor"); argcnt++;
    XtSetArg(args[argcnt], XmNx, 83); argcnt++;
    XtSetArg(args[argcnt], XmNy, 70); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
    if (!tie.userplace)
       {
       XtSetArg(args[argcnt],XtNgeometry, "+355+500"); 
       argcnt++;
       }
    Shell_Cursor = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
    XtAddCallback(Shell_Cursor, XmNdestroyCallback, cursor_destroy, 0);

   /*
    *  *** Create the edit panel shell.
    */
    argcnt = 0;
    XtSetArg(args[argcnt], XmNtitle, "Tie Points - Edit"); argcnt++;
    XtSetArg(args[argcnt], XmNiconName, "Edit"); argcnt++;
    XtSetArg(args[argcnt], XmNx, 83); argcnt++;
    XtSetArg(args[argcnt], XmNy, 70); argcnt++;
    XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
    XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
    if (!tie.userplace)
       {
       XtSetArg(args[argcnt],XtNgeometry, "+0-0");
       argcnt++;
       }
    Shell_Edit = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
    XtAddCallback(Shell_Edit, XmNdestroyCallback, edit_destroy, 0);

   /*
    *  *** adjust map shell
    */
    Shell_Adjust = NULL;

  /*
   *  *** Create Map Parameters shell
   */
   if (tie.mode == IM2MAP)
      {
      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "Map Parameters"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "Register"); argcnt++;
      XtSetArg(args[argcnt], XmNx, 129); argcnt++;
      XtSetArg(args[argcnt], XmNy, 203); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      if (!tie.userplace)
         {
         XtSetArg(args[argcnt],XtNgeometry, "+0+0"); 
         argcnt++;
         }
      Shell_MParm = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           topLevelShellWidgetClass, tie.display, args, argcnt);
      XtAddCallback(Shell_MParm, XmNdestroyCallback, help_destroy, 0);
      Mapparm_form = Createmapparm_form(Shell_MParm);

     /*
      *  *** Create manual entry shell
      */
      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "Manual Entry"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "Manual"); argcnt++;
      if (!tie.userplace)
         {
         XtSetArg(args[argcnt], XmNgeometry, "-0-0");
         argcnt++;
         }
      XtSetArg(args[argcnt], XmNx, 759); argcnt++;
      XtSetArg(args[argcnt], XmNy, 30); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      Shell_Manual = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
      Manual_form = NULL;

     /*
      *  *** Create manual entry shell
      */
      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "Select Available Maps"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "Getmap"); argcnt++;
      if (!tie.userplace)
         {
         XtSetArg(args[argcnt], XmNgeometry, "-0-0");
         argcnt++;
         }
      XtSetArg(args[argcnt], XmNx, 759); argcnt++;
      XtSetArg(args[argcnt], XmNy, 30); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      Shell_Getmap = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
      Getmap_form = NULL;
      }

  /*
   *  *** Create vector display panel
   */
   if (tie.mode == IM2VECT)
      {
      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "Vector Display"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "Vector"); argcnt++;
      if (!tie.userplace)
         {
         XtSetArg(args[argcnt], XmNgeometry, "-0-0");
         argcnt++;
         }
      XtSetArg(args[argcnt], XmNx, 759); argcnt++;
      XtSetArg(args[argcnt], XmNy, 30); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      Shell_Vector = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
      Vector_form = NULL;

      Color_req = NULL;

      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "Vector Alignment"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "Align"); argcnt++;
      if (!tie.userplace)
         {
         if (tie.mode == IM2VECT)
            XtSetArg(args[argcnt], XtNgeometry, "+786+502");
         else
            XtSetArg(args[argcnt], XtNgeometry, "+0-0");
         argcnt++;
         }
      XtSetArg(args[argcnt], XmNx, 759); argcnt++;
      XtSetArg(args[argcnt], XmNy, 30); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      Shell_Align = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
      Align_form = Createalign_form(Shell_Align);

      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "DCW Line Types"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "DCWLT"); argcnt++;
      if (!tie.userplace)
         {
         XtSetArg(args[argcnt], XmNgeometry, "-0-0");
         argcnt++;
         }
      XtSetArg(args[argcnt], XmNx, 759); argcnt++;
      XtSetArg(args[argcnt], XmNy, 30); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      Shell_DCWlt = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
      Dcwlt_form = NULL;

      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "DLG Line Types"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "DLGLT"); argcnt++;
      if (!tie.userplace)
         {
         XtSetArg(args[argcnt], XmNgeometry, "-0-0");
         argcnt++;
         }
      XtSetArg(args[argcnt], XmNx, 759); argcnt++;
      XtSetArg(args[argcnt], XmNy, 30); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      Shell_DLGlt = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
      Dlglt_form = NULL;

      argcnt = 0;
      XtSetArg(args[argcnt], XmNtitle, "DCW Line Status"); argcnt++;
      XtSetArg(args[argcnt], XmNiconName, "DCWLT"); argcnt++;
      if (!tie.userplace)
         {
         XtSetArg(args[argcnt], XmNgeometry, "-20-20");
         argcnt++;
         }
      XtSetArg(args[argcnt], XmNx, 759); argcnt++;
      XtSetArg(args[argcnt], XmNy, 30); argcnt++;
      XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
      XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
      Shell_DCWls = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
           applicationShellWidgetClass, tie.display, args, argcnt);
      Dcwls_form = NULL;
      }

  /*
   *  *** Create the help panel shell.
   */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "Tie Points - Help"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Help"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 129); argcnt++;
   XtSetArg(args[argcnt], XmNy, 203); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
   Shell_Help = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
   XtAddCallback(Shell_Help, XmNdestroyCallback, help_destroy, 0);

  /*
   *  *** Create the about panel shell.
   */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "Tie Points - About"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "About"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 79); argcnt++;
   XtSetArg(args[argcnt], XmNy, 387); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
   Shell_About = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        topLevelShellWidgetClass, tie.display, args, argcnt);
   XtAddCallback(Shell_About, XmNdestroyCallback, about_destroy, 0);

  /*
   *  *** Create the show panel shell.
   */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "Tie Points - Show"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Show"); argcnt++;
   XtSetArg(args[argcnt], XmNx, 759); argcnt++;
   XtSetArg(args[argcnt], XmNy, 30); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
   if (!tie.userplace)
      {
      XtSetArg(args[argcnt], XtNgeometry, "+350-0"); 
      argcnt++;
      }

   Shell_Show = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        applicationShellWidgetClass, tie.display, args, argcnt);

  /*
   *  *** Create the Parameter panel
   */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "Tie Points - Parameters  "); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Parms"); argcnt++;
   if (!tie.userplace)
      {
      XtSetArg(args[argcnt], XmNgeometry, "-0-0"); 
      argcnt++;
      }
   XtSetArg(args[argcnt], XmNx, 759); argcnt++;
   XtSetArg(args[argcnt], XmNy, 30); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
   Shell_Parm = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        applicationShellWidgetClass, tie.display, args, argcnt);
   Parm_bulletinboard = Createparm_form(Shell_Parm);

  /*
   *  *** Create the Graphics panel
   */
   argcnt = 0;
   XtSetArg(args[argcnt], XmNtitle, "Tie Points - Graphics"); argcnt++;
   XtSetArg(args[argcnt], XmNiconName, "Graphics"); argcnt++;
   if (!tie.userplace)
      {
      XtSetArg(args[argcnt], XmNgeometry, "+200-200"); 
      argcnt++;
      }
   XtSetArg(args[argcnt], XmNx, 759); argcnt++;
   XtSetArg(args[argcnt], XmNy, 30); argcnt++;
   XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
   XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
   Shell_Graphic = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
        applicationShellWidgetClass, tie.display, args, argcnt);
   Graphic_form = NULL;

  /*
   *  Create and manage the main panel shell and main panel window.
   */
   Main_shell = Createmain_shell(Shell_Main);
   XtManageChild(Main_shell);
   process_dialog_shell = Create_processing_dialog(Shell_Main);
   XtManageChild(process_dialog_shell);

   if (tie.mode != IM2VECT)
      {
      Create_image_scrollwin(Shell_RefImg, &tie_mem[REFERENCE],
                &ref_image_scrollwin, 
                &ref_image_bulletinboard);
      XtManageChild(ref_image_scrollwin);
      }
   else
      {
      vector_form = Create_vector_overview(Shell_RefImg, 
                                           &ref_image_bulletinboard);
      XtManageChild(vector_form);
      }
    

   Create_image_scrollwin(Shell_SrchImg, &tie_mem[SEARCH],
                &search_image_scrollwin, 
                &search_image_bulletinboard);
   XtManageChild(search_image_scrollwin);
}
