/*
 *  This is the cursor function source file for the cursor panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <math.h>
#include "tiepts.h"
#include <X11/cursorfont.h>
#include "convert.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    selected automatic or manual mode.  It then adds the appropriate event
    handler based on that mode.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_am
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{

}

/*******************************************************************************
Purpose:
    This routine marks the cursor on the graphics planes of the respective
    window.

By J. Hemmer, USGS/EROS Data Center, 1/92

Algorithm Description:
*******************************************************************************/
void cursor_mark
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   int       rplane_idx,        /* image plane index                          */
             rectwidth,         /* rectangle width                            */
             rectheight;        /* rectangle height                           */

  /*
   *  ***  get memory plane index for the given image
   */
   if (w == ref_image_bulletinboard)
      rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;
   else if (w == search_image_bulletinboard)
      rplane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
   else
      return;

   if ((w == ref_image_bulletinboard) && (tie.flags & TIE_ADRG))
      {
      rectwidth  = 32;
      rectheight = 32;
      }
   else if ((tie_planes[rplane_idx].skips != 1) ||
            (tie_planes[rplane_idx].skipl != 1))
      {
      rectwidth  = tie_parms.fwidth/tie_planes[rplane_idx].skips;
      rectheight = tie_parms.fheight/tie_planes[rplane_idx].skipl;
      }
   else
      rectheight = rectwidth  = 64;

   if ((w == ref_image_bulletinboard) && (tie.refrect_x != -1))
      XDrawRectangle(tie.display, XtWindow(w), tie.rect_gc,
         tie.refrect_x - rectwidth/2, tie.refrect_y - rectheight/2,
         rectwidth, rectheight);
   else if ((w == search_image_bulletinboard) && (tie.srchrect_x != -1))
      XDrawRectangle(tie.display, XtWindow(w), tie.rect_gc,
         tie.srchrect_x - rectwidth/2, tie.srchrect_y - rectheight/2,
         rectwidth, rectheight);

   if ((tie.mode == IM2VECT) && (w == ref_image_bulletinboard) && 
       (tie_vector->pressx1 != -1))
      {
      XDrawRectangle(tie.display, XtWindow(ref_image_bulletinboard), 
                      tie.rect_gc,
                      tie_vector->pressx1, tie_vector->pressy1,
                      tie_vector->pressx2 - tie_vector->pressx1,
                      tie_vector->pressy2 - tie_vector->pressy1);
      }

  /*
   *  *** do not redraw sliding rectangle
   */
   if (w == ref_image_bulletinboard)
      tie.refrect_x = -1;
   else if (w == search_image_bulletinboard)
      tie.srchrect_x = -1;
}

/*******************************************************************************
Purpose:
    This routine resets the color of the cursor.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void cursor_setcolor
(
    int i
)
{
   XParseColor(tie.display,tie.colormap,wk_colors[i],&tie_cursor.color);

  /*
   *  *** if zoom panel is up, redefine dot cursor
   */
   if (XtIsRealized(Shell_Zoom))
      zoom_dotcursor(Shell_Zoom);

   if (tie.mode == IM2VECT)
      zoom_dotcursor(ref_image_bulletinboard);
}

/*******************************************************************************
Purpose:

By J. Hemmer, USGS/EROS Data Center, 11/91

Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - changed to draw rubber rectangles only when the coordinates are positive (
   ie., box is only drawn when the cursor is moved down or to the right of
   the cursor's starting location.

Algorithm Description:
*******************************************************************************/
void cursor_select
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   static int   finder = FALSE; /* if cursor finder is ON                     */
   int          deltax,         /* Amount to warp the cursor.                 */
                deltay,
                maxnl,          /* Maximum line value.                        */
                maxns,          /* Maximum sample value.                      */
                move;           /* Amount of cursor single step.              */
   unsigned int state;          /* Keyboard state (Shift key?)                */
   unsigned int keycode;        /* Key code from keyboard.                    */

   maxnl = tie_parms.factor * tie_zoom[SEARCH].parms.in_lines - 1;
   maxns = tie_parms.factor * tie_zoom[SEARCH].parms.in_samps - 1;
   deltax = deltay = 0;

   switch(event->type)
      {
      case KeyRelease:
        /*
         *  *** Check the validity of the cursor coordinates.
         */
         if ((event->xkey.x < 0) || (event->xkey.y < 0) ||
             (event->xkey.x > maxns) || (event->xkey.y > maxnl))
             return;

        /*
         *  *** Capture the key code and the keyboard state.
         */
         keycode = event->xkey.keycode;
         state   = event->xkey.state;

        /*
         *  *** if Return/Enter, select point
         */
         if (keycode == tie_cursor.ret_key)
            {
            if (w == zoom_srchwin_bulletinboard)
               select_srchpt(event->xkey.x, event->xkey.y);
            else
               select_refpt(w, event->xkey.x, event->xkey.y);
            break;
            }
          
        /*
         *  *** Check the keyboard state to see if the Shift key was used.
         */
         if (state == ShiftMask)
            move = 10;               /* Shift key so use delta of 10.         */
         else
            move = 1;

        /*
         *  *** Set the delta X/Y amount based on the key pressed.
         */
         if ((keycode == tie_cursor.k_key) || (keycode == tie_cursor.up_key))
            deltay -= move;
         else if ((keycode == tie_cursor.j_key) || (keycode ==
            tie_cursor.down_key))
            deltay += move;
         else if ((keycode == tie_cursor.h_key) || (keycode ==
            tie_cursor.left_key))
            deltax -= move;
         else if ((keycode == tie_cursor.l_key) || (keycode ==
            tie_cursor.right_key))
            deltax += move;

        /*
         *  *** If a valid move can be made then move the cursor.
         */
         if (((event->xkey.x + deltax) < 0) ||
             ((event->xkey.x + deltax) > maxns) || ((event->xkey.y + deltay)
             < 0) || ((event->xkey.y + deltay) > maxnl))
             ;
         else if (deltax | deltay)
             XWarpPointer(tie.display,None,None,None,None,None,None,
                 deltax,deltay);
         break;

      case ButtonPress:
        /*
         *  *** if image to vector, draw rubber rectangle about selected region
         */
         if ((w == ref_image_bulletinboard) && (tie.mode == IM2VECT)) 
            {
            if (event->xbutton.button == 2)
               {
               tie_vector->pressx1 = event->xbutton.x;
               tie_vector->pressy1 = event->xbutton.y;
               tie_vector->pressx2 = -1;
               tie_vector->pressy2 = -1;
               XtAddEventHandler(w, ButtonMotionMask,
                              FALSE, vector_drag, NULL);
               }
            else if (event->xbutton.button == 3)
               vector_unzoom(w,client,NULL);
            }
         else if (event->xbutton.button == 2)
            {
            zoom_findercursor();
            finder = TRUE;
            }
         break;

      case ButtonRelease:
         if ((w == ref_image_bulletinboard) && 
             (tie.mode == IM2VECT) && (event->xbutton.button == 2))
            {
            if (((tie_vector->pressx2 - tie_vector->pressx1) > 0) &&
                ((tie_vector->pressy2 - tie_vector->pressy1) > 0))
               {
               XDrawRectangle(tie.display, XtWindow(ref_image_bulletinboard), 
                  tie.rect_gc,
                  tie_vector->pressx1, tie_vector->pressy1,
                  tie_vector->pressx2 - tie_vector->pressx1,
                  tie_vector->pressy2 - tie_vector->pressy1);

               vector_zoom();
               tie_vector->pressx1 = tie_vector->pressy1 = -1;
               }
            XtRemoveEventHandler(ref_image_bulletinboard, ButtonMotionMask,
                              FALSE, vector_drag, NULL);
            }
         else if ((tie.mode != ONESRCH) && (event->xbutton.button == 1))
            {
            if (w == zoom_srchwin_bulletinboard)
               select_srchpt(event->xbutton.x, event->xbutton.y);
            else
               select_refpt(w, event->xbutton.x, event->xbutton.y);
            }
         else if (tie.mode == ONESRCH) 
            {
            select_srchpt(event->xbutton.x, event->xbutton.y);
            }
        /*
         *  *** if second button, remove finder cursor
         */
         else if ((event->xbutton.button == 2) && (finder))
            {
            finder = FALSE;
            zoom_dotcursor(Shell_Zoom);
            }

         break;

      };
}

/*******************************************************************************
Purpose:
    This routine adds the appropriate event handler and sets the appropriate
    bit flags based on default values.  It also calls the cursor setup
    routine which defines the appropriate pixmaps for displaying the cursor
    information.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_create
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   int          clear_bits;     /* Variable used to clear individual bits.    */
                           
   if ((event->type == CreateNotify) && !(tie_cursor.flags & CURSOR_ACTIVE))
      {
     /*
      *  *** Add cursor tracking event handler
      */ 
      if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
         XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask|
               PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

      XtAddEventHandler(search_image_bulletinboard,ButtonReleaseMask|
               PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

      if (tie_zoom[REFERENCE].flags & ZOOM_ACTIVE)
         {
         if ((tie.mode == IM2IM) & !(tie.flags & TIE_ADRG))
            XtAddEventHandler(zoom_refwin_bulletinboard,ButtonReleaseMask|
               PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

         XtAddEventHandler(zoom_srchwin_bulletinboard,ButtonReleaseMask|
               PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
         }

      if (tie_ref_fullres != NULL)
         XtAddEventHandler(fullref_win_bulletinboard,ButtonReleaseMask|
               PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

      if (tie_srch_fullres != NULL)
         XtAddEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask|
               PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

      cursor_setup();
      }

  /*
   *  *** Set the appropriate flags.
   */
   tie_cursor.flags = (tie_cursor.flags | CURSOR_XY | CURSOR_ORIG | 
                       CURSOR_AUTO | CURSOR_ACTIVE);

   clear_bits = ~(CURSOR_Z | CURSOR_MAPPED | CURSOR_MAN | CURSOR_UNSCAL | 
                 CURSOR_PROJ | CURSOR_LATLONG);

   tie_cursor.flags = tie_cursor.flags & clear_bits;
}

/*******************************************************************************
Purpose:
    This routine closes the cursor window, sets the main menu option to
    sensitive, resets bit flags and removes the appropriate event handler.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
  /*
   *  *** Close the cursor window and set the cursor button on the main panel
   *      to sensitive.
   */
   XtUnmanageChild(Cursor_bulletinboard);
   XtUnrealizeWidget(Shell_Cursor);
   XtDestroyWidget(Cursor_bulletinboard);
   Cursor_bulletinboard = NULL;
   XtSetSensitive(main_cursor_button,True);

  /*
   *  ***  Reset the active flag and remove the appropriate event handler.
   */
   tie_cursor.flags = tie_cursor.flags ^ CURSOR_ACTIVE;

   if (tie_cursor.flags & CURSOR_AUTO)
      {
      if ((tie.mode == IM2IM) & !(tie.flags & TIE_ADRG))
         XtRemoveEventHandler(ref_image_bulletinboard, ButtonReleaseMask|
              PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

      XtRemoveEventHandler(search_image_bulletinboard, ButtonReleaseMask|
              PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

      if (tie_zoom[REFERENCE].flags & ZOOM_ACTIVE)
         {
         if ((tie.mode == IM2IM) & !(tie.flags & TIE_ADRG))
            XtRemoveEventHandler(zoom_refwin_bulletinboard,ButtonReleaseMask|
                 PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

         XtRemoveEventHandler(zoom_srchwin_bulletinboard,ButtonReleaseMask|
              PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
         }

      if (tie_ref_fullres != NULL)
         XtRemoveEventHandler(fullref_win_bulletinboard,ButtonReleaseMask|
            PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);

      if (tie_srch_fullres != NULL)
         {
         XtRemoveEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask|
                   PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
         }
      }
   else
      {
      if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
         XtRemoveEventHandler(ref_image_bulletinboard,ButtonReleaseMask|
                            KeyReleaseMask,FALSE,cursor_track,NULL);

      XtRemoveEventHandler(search_image_bulletinboard,ButtonReleaseMask|
                            KeyReleaseMask,FALSE,cursor_track,NULL);

      if (tie_zoom[REFERENCE].flags & ZOOM_ACTIVE)
         {
         if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
            XtRemoveEventHandler(zoom_refwin_bulletinboard,ButtonReleaseMask|
                                 KeyReleaseMask,FALSE,cursor_track,NULL);

         XtRemoveEventHandler(zoom_srchwin_bulletinboard,ButtonReleaseMask|
                              KeyReleaseMask,FALSE,cursor_track,NULL);
         }

      if (tie_ref_fullres != NULL)
         {
         XtRemoveEventHandler(fullref_win_bulletinboard,ButtonReleaseMask|
                              KeyReleaseMask,FALSE,cursor_track,NULL);
         }

      if (tie_srch_fullres != NULL)
         {
         XtRemoveEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask|
                              KeyReleaseMask,FALSE,cursor_track,NULL);
         }
      }

  /*
   *  ***  Free resources no longer needed.
   */
   if (tie_cursor.info_pixmap != NULL)
      XFreePixmap(tie.display,tie_cursor.info_pixmap);
   if (tie_cursor.clear_pixmap != NULL)
      XFreePixmap(tie.display,tie_cursor.clear_pixmap);

   XFreeGC(tie.display,tie_cursor.gc);
}

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    chosen to have latitude/longitude values displayed.  If the toggle
    has been unset, the corresponding text value is set to null.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_latlong
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];        /* Variable used to set widget resources.     */

   XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call_data;

  /*
   *  ***  Set the appropriate flag.
   */
   tie_cursor.flags  = tie_cursor.flags ^ CURSOR_LATLONG;

  /*
   *  If the lat/long toggle has been unset, set its corresponding text value
   *  to a null string.
   */
   if (!(acs->set))
      {
      XtSetArg(args[0],XmNvalue,"");
      XtSetValues(cursor_ll_text,args,1);
      }
}

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    selected original or mapped gray level values to be displayed.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_mo
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
  /*
   *  *** Set the appropriate flag.
   */
   tie_cursor.flags  = tie_cursor.flags ^ (CURSOR_ORIG | CURSOR_MAPPED);
}

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    chosen to have projection coordinates displayed.  If the toggle has been 
    unset, the corresponding text value is set to null.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_proj
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg     args[1];        /* Variable used to set widget resources.          */

   XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call_data;

  /*
   *  *** Set the appropriate flag.
   */
   tie_cursor.flags  = tie_cursor.flags ^ CURSOR_PROJ;

  /*
   *  *** If the projection coordinate toggle has been unset, set its 
   *      corresponding text value to a null string.
   */
   if (!(acs->set))
      {
      XtSetArg(args[0],XmNvalue,"");
      XtSetValues(cursor_pc_text,args,1);
      }
}

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    chosen to have unscaled pixel values displayed.  If the toggle has been 
    unset, the corresponding text value is set to null.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_unscaled
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg     args[1];        /* Variable used to set widget resources.         */

   XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call_data;

  /*
   *  ***  Set the appropriate flag.
   */
   tie_cursor.flags  = tie_cursor.flags ^ CURSOR_UNSCAL;

  /*
   *  If the unscaled  coordinate toggle has been unset, set its corresponding
   *  text value to a null string.
   */
   if (!(acs->set))
      {
      XtSetArg(args[0],XmNvalue,"");
      XtSetValues(cursor_un_text,args,1);
      }
}

/*******************************************************************************
Purpose:
    This routine sets the appropriate flag indicating whether the user has
    selected to cursor in the XY dimension or the Z dimension.  If the
    Z dimension has been selected, the scale widget is made sensitive and
    its maximum value is set to the number of planes that can be cursored
    through.  The cursor mode is also set to manual and the appropriate
    event handler is added.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_xyz
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];   /* Variable used to set widget resources.         */
   int          i;         /* Looping index.                                 */

  /*
   *  ***  If Z dimension, determine the number of valid planes.
   */
   if (*(char*)client == 'Z')
      {
      for (i=0; i < tie.max_plane; i++)
         {
         if (tie_planes[i].flags & USED)
            tie_cursor.max_zplane = i + 1;
         }
      }

  /*
   *  ***  Set the appropriate flags.
   */
   tie_cursor.flags  = tie_cursor.flags ^ (CURSOR_XY | CURSOR_Z); 

   if (*(char *)client == 'Z')
      {
     /*
      *  *** Allow the scale widget to be sensitive and set its maximum value.
      */
      XtSetSensitive(cursor_scale,True);
      XtSetArg(args[0],XmNmaximum,tie_cursor.max_zplane);
      XtSetValues(cursor_scale,args,1);
      tie_cursor.zplane = 1;

     /*
      *  ***  If the cursor mode is currently automatic, 
      *       reset it to manual and add the appropriate event handler.
      */
      if (tie_cursor.flags & CURSOR_AUTO)
         {
         XtSetArg(args[0],XmNset,False);
         XtSetValues(cursor_auto_button,args,1);
         XtSetArg(args[0],XmNset,True);
         XtSetValues(cursor_man_button,args,1);
         tie_cursor.flags  = tie_cursor.flags ^ (CURSOR_AUTO | CURSOR_MAN); 
         XtRemoveEventHandler(w,ButtonReleaseMask|
                  PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
         XtAddEventHandler(w,
                  ButtonReleaseMask|KeyReleaseMask,FALSE,cursor_track,NULL);
         }
      }
   else
     /*
      *  *** Do not allow the scale if cursoring in the XY dimension.
      */
      XtSetSensitive(cursor_scale,False);
}

/*******************************************************************************
Purpose:
    This routine is used to calculate the specified information to be output
    to the cursor window.  Once the information is calculated, it is written
    to the pixmap and then copied to the cursor window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
void cursor_track
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   Arg     args[1];             /* Variable used to set widget resources.     */
   Window       cursor_win;     /* Cursor window ID.                          */
   static       
   char         prev_name[CMLEN] = {""},
                                /* Filename of image last processed.          */
                proj_name[CMLEN];
                                /* Projection system name.                    */
   double       latitude,       /* Calculated latitude value.                 */
                longitude,      /* Calculated longitude value.                */
                projx,          /* X projection coordinate.                   */
                projy,          /* Y projection coordinate.                   */
                r_unscale,      /* Unscaled red value.                        */
                g_unscale,      /* Unscaled green value.                      */
                b_unscale,      /* Unscaled blue value.                       */
                xsub,           /* Current sample (subpixel accuracy).        */
                ysub;           /* Current line (subpixel accuracy).          */
   int          bplane_idx,     /* Index to blue plane number of the currently
                                   viewed image.                              */
                curx,           /* Current X file coordinate.                 */
                cury,           /* Current Y file coordinate.                 */
                gplane_idx,     /* Index to green plane number of the currently
                                   viewed image.                              */
                i,j,k,          /* Looping indices.                           */
                imgindex,       /* REFERENCE or SEARCH flag.                  */
                matrix[3][3][3],/* Matrix of current pixel values.            */
                maxnl,          /* Maximum line value.                        */
                maxns,          /* Maximum sample value.                      */
                nbands,         /* Number of bands displayed.                 */
                offset,         /* Offset into image buffer for current 
                                   pixel value.                               */
                pos,            /* Character position within a string.        */
                rplane_idx,     /* Index to red plane number of the currently
                                   viewed image.                              */
                status;         /* Status indicator.                          */
   long icoords[8];     /* Image corner line/sample coordinates.      */
   long msgflg = FALSE; /* Specifies that error messages are not to be
                                   output by the routine.                     */
   long prtprm[2];      /* Print option for the Projection 
                           Transformation Package.                    */

   char         coords[132],    /* Current X/Y file coord in character format.*/
                nsstr[15],      /* North/South latitude string.               */
                ewstr[15],      /* East/West logitude string.                 */
                tmpstr[CMLEN];  /* Temporary string variable.                 */

  /*
   *  *** If no image is currently being viewed, return
   */
   if ((w == ref_image_bulletinboard) &&
       (tie_planes[tie.rview].image_data == NULL))
      return;
   else if ((w == search_image_bulletinboard) && 
            (tie_planes[tie.sview].image_data == NULL))
      return;
   else if ((w == zoom_srchwin_bulletinboard) && 
            (tie_zoom[SEARCH].image_pixmap == NULL)) 
      return;
   else if ((w == zoom_refwin_bulletinboard) && 
            (tie_zoom[REFERENCE].image_pixmap == NULL)) 
      return;
   else if ((w == zoom_refwin_bulletinboard) && (tie_cursor.flags & CURSOR_Z)) 
      return;
   else if ((w == fullref_win_bulletinboard) && 
            (tie_ref_fullres->image_data == NULL))
      return;
   else if ((w == fullsrch_win_bulletinboard) && 
            (tie_srch_fullres->image_data == NULL))
      return;
   else if ((w == fullref_win_bulletinboard) && (tie_cursor.flags & CURSOR_Z)) 
      return;

  /*
   * Determine the appropriate memory indices based on the type of
   * cursoring (XY dimension or Z dimension).
   */
   if (tie_cursor.flags & CURSOR_XY)
      {
      if ((w == ref_image_bulletinboard)   || 
          (w == fullref_win_bulletinboard) ||
          (w == zoom_refwin_bulletinboard))
         {
         rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;
         gplane_idx = tie_composite[REFERENCE].rgbplanes[GREEN] - 1;
         bplane_idx = tie_composite[REFERENCE].rgbplanes[BLUE] - 1;
         }
      else if ((w == search_image_bulletinboard) || 
               (w == fullsrch_win_bulletinboard) ||
               (w == zoom_srchwin_bulletinboard))
         {
         rplane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
         gplane_idx = tie_composite[SEARCH].rgbplanes[GREEN] - 1;
         bplane_idx = tie_composite[SEARCH].rgbplanes[BLUE] - 1;
         }
      }
   else
      {
      rplane_idx = tie_cursor.zplane - 1;
      for (i=0; i < tie.max_plane; i++)
         {
         if (!(tie_planes[rplane_idx].flags & USED)) 
            rplane_idx = ((rplane_idx + 1) > tie_cursor.max_zplane) ? 1 : 
                          rplane_idx + 1;
         else
            break;
         }

      gplane_idx = bplane_idx = rplane_idx; 
      }

  /*
   *  ***  Determine the maximum number of lines and samples in the window.
   */
   if ((w == ref_image_bulletinboard) || (w == search_image_bulletinboard))
      {
      maxnl = tie_planes[rplane_idx].dspwin[NL] - 1;
      maxns = tie_planes[rplane_idx].dspwin[NS] - 1;
      }
   else if ((w == zoom_refwin_bulletinboard) || 
            (w == zoom_srchwin_bulletinboard))
      {
      maxnl = tie_parms.factor * tie_zoom[SEARCH].parms.in_lines;
      maxns = tie_parms.factor * tie_zoom[SEARCH].parms.in_samps;

      if ((rplane_idx == gplane_idx) && (gplane_idx == bplane_idx))
         nbands = 1;
      else
         nbands = 3;
      }
   else if (w == fullref_win_bulletinboard)
      {
      maxnl = tie_ref_fullres[0].dspwin[NL];
      maxns = tie_ref_fullres[0].dspwin[NS];

      if ((rplane_idx == gplane_idx) && (gplane_idx == bplane_idx))
         nbands = 1;
      else
         nbands = 3;
      }
   else if (w == fullsrch_win_bulletinboard)  
      {
      maxnl = tie_srch_fullres[0].dspwin[NL];
      maxns = tie_srch_fullres[0].dspwin[NS];

      if ((rplane_idx == gplane_idx) && (gplane_idx == bplane_idx))
         nbands = 1;
      else
         nbands = 3;
      }

  /*
   *  ***  Can projection information be calculated if requested?
   */
   if (!(tie_planes[rplane_idx].proj_valid))
      tie_cursor.proj_valid = FALSE;

  /*
   * Do new coefficients need to be calculated and does the geographic 
   * transformation package need to be reinitialized?
   */
   if ((strcmp(tie_planes[rplane_idx].filename,prev_name) != 0) && 
       ((tie_cursor.flags & CURSOR_PROJ) || 
        (tie_cursor.flags & CURSOR_LATLONG)) &&
        (tie_planes[rplane_idx].proj_valid))
      {
     /*
      *  Find the transformation coefficients mapping image coordinates to
      *  projection coordinates.
      */
      icoords[0] = icoords[1] = icoords[3] = icoords[4] = 1;
      icoords[2] = icoords[6] = tie_planes[rplane_idx].ddr.nl;
      icoords[5] = icoords[7] = tie_planes[rplane_idx].ddr.ns;

      status = c_rtcoef(tie_planes[rplane_idx].ddr.upleft,icoords,
                       tie_cursor.coef,&msgflg);
     /*
      * If the coefficients can't be calculated then set the DDR validity
      * flag to false.
      */
      if (status != E_SUCC)
         tie_cursor.proj_valid = FALSE;
      else
         tie_cursor.proj_valid = TRUE;

     /*
      * Initialize the Projection Tranformation Package and verify the
      * unit code.
      */
      if (tie_cursor.proj_valid)
         {
         status = c_ckunit(tie_planes[rplane_idx].ddr.proj_units,
                           &tie_cursor.outunit);
         if (status != E_SUCC)
           tie_cursor.proj_valid = FALSE;
         }

     /*
      *  ***  Initialize the geographic transformation package.
      */
      if (tie_cursor.proj_valid)
        {
         tie_cursor.geo = GEO;
         tie_cursor.outdatum = tie_planes[rplane_idx].ddr.datum_code;
         tie_cursor.outzone = tie_planes[rplane_idx].ddr.zone_code;
         tie_cursor.outproj = DEGREE;
         prtprm[0] = prtprm[1] = FALSE;

         status = c_transinit(&tie_planes[rplane_idx].ddr.proj_code,
                           &tie_cursor.outunit,
                           &tie_planes[rplane_idx].ddr.zone_code,
                           &tie_planes[rplane_idx].ddr.datum_code,
                           tie_planes[rplane_idx].ddr.proj_coef,
                           &tie_cursor.geo,
                           &tie_cursor.outproj,
                           &tie_cursor.outzone,
                           &tie_cursor.outdatum,
                           tie_cursor.projparm,prtprm,"");

         if (status != E_SUCC)
             tie_cursor.proj_valid = FALSE;
         }
 
     /*
      *  ***  Determine the projection name based on the projection code.
      */
      if (tie_cursor.proj_valid)
         {
         c_prostr(&tie_planes[rplane_idx].ddr.proj_code,tmpstr);
         pos = strcspn(tmpstr,")");
         strcpy(proj_name,&tmpstr[pos+1]);
         }
      }

   if ((tie_cursor.flags & CURSOR_PROJ) || 
       (tie_cursor.flags & CURSOR_LATLONG))
      strcpy(prev_name,tie_planes[rplane_idx].filename);

  /*
   *  Remove all other MotionNotify events from queue -- this makes for
   *  smoother scrolling by helping to limit the number of events
   *  generated while we're acting on the present one.
   */
   if (!(event->xmotion.send_event))
      while (XCheckTypedEvent(tie.display,MotionNotify,event));

  /*
   *  ***  Check the validity of the cursor coordinates.
   */
   if ((event->xmotion.x < 0) || (event->xmotion.y < 0) || 
       (event->xmotion.x > maxns) || (event->xmotion.y > maxnl))
      return;

  /*
   *  ***  Get the new cursor coordinates.
   */
   tie.x = event->xmotion.x;
   tie.y = event->xmotion.y;

   cursor_win = XtWindow(cursor_drawingarea);

  /*
   *  ***  Clear the cursor pixmap for the new information.
   */
   XCopyArea(tie.display,tie_cursor.clear_pixmap,tie_cursor.info_pixmap,
             tie_cursor.gc,0,0,CURSOR_WIDTH,CURSOR_HEIGHT,0,0);

  /*
   *  ***  Convert the coordinates to (one-relative) file coordinates.
   */
   if ((w == ref_image_bulletinboard) || (w == search_image_bulletinboard))
      {
      getxy(w, event->xmotion.x, event->xmotion.y, &curx, &cury);
     /*
      *  *** Set the image and file coordinate strings.
      */
      sprintf(coords,"%d   %d",event->xmotion.y + 1,event->xmotion.x + 1);
      XtSetArg(args[0],XmNvalue,coords);
      XtSetValues(cursor_ls_text,args,1);
 
      sprintf(coords,"%d   %d",cury,curx);
      XtSetArg(args[0],XmNvalue,coords);
      XtSetValues(cursor_fc_text,args,1);
 
      xsub = (double)curx;
      ysub = (double)cury;
      }
   else if ((w == zoom_refwin_bulletinboard) || 
            (w == zoom_srchwin_bulletinboard))
      {
      if ((tie_zoom[REFERENCE].flags & ZOOM_FULLRES) &&
          (w == zoom_refwin_bulletinboard))
         {
         cursor_sub(REFERENCE, event->xmotion.x,event->xmotion.y,
                    &xsub,&ysub,1,1,
                    tie_ref_fullres[0].window[SL],
                    tie_ref_fullres[0].window[SS]);
         }
      else if ((tie_zoom[SEARCH].flags & ZOOM_FULLRES) &&
               (w == zoom_srchwin_bulletinboard))
         {
         cursor_sub(SEARCH, event->xmotion.x,event->xmotion.y,
                       &xsub,&ysub,1,1,
                       tie_srch_fullres[0].window[SL],
                       tie_srch_fullres[0].window[SS]);
         }

     /*
      *  ***  Set the image and file coordinate strings.
      */
      sprintf(coords,"%d   %d",((int)(event->xmotion.y / tie_parms.factor) + 1),
             ((int)(event->xmotion.x /tie_parms.factor) + 1));
      XtSetArg(args[0],XmNvalue,coords);
      XtSetValues(cursor_ls_text,args,1);

      sprintf(coords,"%.4lf   %.4lf",ysub,xsub);
      XtSetArg(args[0],XmNvalue,coords);
      XtSetValues(cursor_fc_text,args,1);
      }
   else if ((w == fullref_win_bulletinboard) || 
            (w == fullsrch_win_bulletinboard))
      {
      getlinesamp(w, event->xmotion.x, event->xmotion.y, &curx, &cury);
     /*
      *  *** Set the image and file coordinate strings.
      */
      sprintf(coords,"%d   %d",(event->xmotion.y  + 1),(event->xmotion.x + 1));
      XtSetArg(args[0],XmNvalue,coords);
      XtSetValues(cursor_ls_text,args,1);

      sprintf(coords,"%d   %d",cury,curx);
      XtSetArg(args[0],XmNvalue,coords);
      XtSetValues(cursor_fc_text,args,1);

      xsub = (double)curx;
      ysub = (double)cury;
      }

  /*
   *  ***  Calculate the projection coordinates and/or lat/lon coordinates.
   */
   if ((tie_cursor.flags & CURSOR_PROJ) || (tie_cursor.flags & CURSOR_LATLONG)) 
      {
      if (tie_cursor.proj_valid)
         {
        /* 
         *  ***  Calculate projection coord from the line/sample coord
         */
         projy = (double)(tie_cursor.coef[0] * ysub) + (tie_cursor.coef[1] * 
                          xsub) + tie_cursor.coef[2];
         projx = (double)(tie_cursor.coef[3] * ysub) + (tie_cursor.coef[4] * 
                          xsub) + tie_cursor.coef[5];
         sprintf(coords,"Y: %12.3f X: %12.3f %s %s",projy,projx,proj_name,
                 tie_planes[rplane_idx].ddr.proj_units);
         }
      else
         strcpy(coords,"Invalid DDR fields");

      if (tie_cursor.flags & CURSOR_PROJ)
         {
        /*
         *  *** Write out the projection coordinates calculated.
         */
         XtSetArg(args[0],XmNvalue,coords);
         XtSetValues(cursor_pc_text,args,1);
         }

      if (tie_cursor.flags & CURSOR_LATLONG)
         {
         if (tie_cursor.proj_valid) 
            {
           /*
            *  *** Calculate latitude/longitude coordinates from 
            *      the projection coordinates.
            */
            status = c_trans(&tie_planes[rplane_idx].ddr.proj_code,
                            &tie_cursor.outunit,
                            &tie_cursor.geo,
                            &tie_cursor.outproj,
                            &projx,&projy,
                            &longitude,&latitude);
           if (status == IN_BREAK)
               {
               strcpy(coords,"In break area");
               if (tie_cursor.flags & CURSOR_PROJ)
                   {
                   XtSetArg(args[0],XmNvalue,"In break area");
                   XtSetValues(cursor_pc_text,args,1);
                   }
               }
            else if (status != E_SUCC)
               strcpy(coords,"Error calculating latitude/longitude values");
            else
               {
              /* 
               *  *** Determine the appropriate latitude/longitude labels.
               */
               if (latitude > 0)
                  strcpy(nsstr,"North");
               else if (latitude < 0)
                  strcpy(nsstr,"South");
               else
                  strcpy(nsstr,"Equator");

               if (longitude > 0)
                  strcpy(ewstr,"East");
               else if (longitude < 0)
                  strcpy(ewstr,"West");
               else
                  strcpy(ewstr,"Prime Meridian");

               latitude = fabs(latitude);
               longitude = fabs(longitude);

               sprintf(coords,"%12.5f %s %12.5f %s",latitude,nsstr,longitude,
                        ewstr);
               }
            }
         else
            strcpy(coords,"Invalid DDR fields");

        /*
         *  *** Write out the latitude/longitude coordinates calculated.
         */
         XtSetArg(args[0],XmNvalue,coords);
         XtSetValues(cursor_ll_text,args,1);
         }
      }

  /*
   *  *** Set the matrix of gray level values.
   */
   for (i=0; i < 3; i++)
      {
      matrix[RED][0][i] = matrix[RED][1][i] = matrix[RED][2][i] = 0;
      matrix[GREEN][0][i] = matrix[GREEN][1][i] = matrix[GREEN][2][i] = 0;
      matrix[BLUE][0][i] = matrix[BLUE][1][i] = matrix[BLUE][2][i] = 0;
      }

   if (event->xmotion.y == 0)
      {
      matrix[RED][0][0] = matrix[RED][0][1] = matrix[RED][0][2] = -1;
      matrix[GREEN][0][0] = matrix[GREEN][0][1] = matrix[GREEN][0][2] = -1;
      matrix[BLUE][0][0] = matrix[BLUE][0][1] = matrix[BLUE][0][2] = -1;
      }
   else if (event->xmotion.y == maxnl)
      {
      matrix[RED][2][0] = matrix[RED][2][1] = matrix[RED][2][2] = -1;
      matrix[GREEN][2][0] = matrix[GREEN][2][1] = matrix[GREEN][2][2] = -1;
      matrix[BLUE][2][0] = matrix[BLUE][2][1] = matrix[BLUE][2][2] = -1;
      }

   if (event->xmotion.x == 0)
      {
      matrix[RED][0][0] = matrix[RED][1][0] = matrix[RED][2][0] = -1;
      matrix[GREEN][0][0] = matrix[GREEN][1][0] = matrix[GREEN][2][0] = -1;
      matrix[BLUE][0][0] = matrix[BLUE][1][0] = matrix[BLUE][2][0] = -1;
      }
   else if (event->xmotion.x == maxns)
      {
      matrix[RED][0][2] = matrix[RED][1][2] = matrix[RED][2][2] = -1;
      matrix[GREEN][0][2] = matrix[GREEN][1][2] = matrix[GREEN][2][2] = -1;
      matrix[BLUE][0][2] = matrix[BLUE][1][2] = matrix[BLUE][2][2] = -1;
      }
  
   if ((w == ref_image_bulletinboard) || (w == search_image_bulletinboard))
      {
      for (i=0; i < 3; i++)
         for (j=0; j < 3; j++)
            {
            offset = ((event->xmotion.y + (i - 1)) * 
                     tie_planes[rplane_idx].dspwin[NS]) + (event->xmotion.x) + 
                     (j - 1);
         if (matrix[RED][i][j] != -1)
            {
            matrix[RED][i][j] = (int) (*(tie_planes[rplane_idx].image_data
                                       + offset)); 
            matrix[GREEN][i][j] = (int) (*(tie_planes[gplane_idx].
                                       image_data + offset));
            matrix[BLUE][i][j] = (int) (*(tie_planes[bplane_idx].
                                       image_data + offset));
            }
         }
      }
   else if ((w == zoom_refwin_bulletinboard) || 
            (w == zoom_srchwin_bulletinboard))
      {
      if (w == zoom_srchwin_bulletinboard)
         imgindex = SEARCH;
      else
         imgindex = REFERENCE;

      for (i=0; i < 3; i++)
         for (j=0; j < 3; j++)
            {
            offset = ((event->xmotion.y + (i - 1)) * maxns) + (event->xmotion.x)
                        + (j - 1);
            for (k=0; k < nbands; k++)
               {
               if ((matrix[k][i][j] != -1) && (nbands != 1))
                  matrix[k][i][j] = (int) (*(tie_zoom[imgindex].image_data + 
                                           maxns * maxnl * k + offset)); 
               else if ((matrix[k][i][j] != -1) && (nbands == 1))
                  {
                  matrix[RED][i][j] = (int)
                           (*(tie_zoom[imgindex].image_data+ offset)); 
                  matrix[GREEN][i][j] = (int) 
                           (*(tie_zoom[imgindex].image_data + offset)); 
                  matrix[BLUE][i][j] = (int) 
                           (*(tie_zoom[imgindex].image_data + offset)); 
                  }
               }
            }
         }
   else if (w == fullref_win_bulletinboard) 
      {
      for (i=0; i < 3; i++)
         for (j=0; j < 3; j++)
            {
            offset = ((event->xmotion.y + (i - 1)) * 
                tie_ref_fullres[0].dspwin[NS]) + (event->xmotion.x) + 
                (j - 1);
            if ((matrix[RED][i][j] != -1) && (nbands != 1))
               {
               matrix[RED][i][j] = (int) 
                          (*(tie_ref_fullres[RED].image_data + offset)); 
               matrix[GREEN][i][j] = (int) 
                          (*(tie_ref_fullres[GREEN].image_data + offset));
               matrix[BLUE][i][j] = (int) 
                          (*(tie_ref_fullres[BLUE].image_data + offset));
               }
            else if ((matrix[RED][i][j] != -1) && (nbands == 1))
               {
               matrix[RED][i][j] = (int) 
                         (*(tie_ref_fullres[RED].image_data + offset)); 
               matrix[GREEN][i][j] = (int) 
                         (*(tie_ref_fullres[RED].image_data + offset));
               matrix[BLUE][i][j] = (int) 
                         (*(tie_ref_fullres[RED].image_data + offset));
               }
            }
         }
   else if (w == fullsrch_win_bulletinboard)
      {
      for (i=0; i < 3; i++)
         for (j=0; j < 3; j++)
            {
            offset = ((event->xmotion.y + (i - 1)) *
                tie_srch_fullres[0].dspwin[NS]) + (event->xmotion.x) +
                (j - 1);
            if ((matrix[RED][i][j] != -1) && (nbands != 1))
               {
               matrix[RED][i][j] = (int)
                       (*(tie_srch_fullres[RED].image_data + offset));
               matrix[GREEN][i][j] = (int)
                       (*(tie_srch_fullres[GREEN].image_data + offset));
               matrix[BLUE][i][j] = (int)
                       (*(tie_srch_fullres[BLUE].image_data + offset));
               }
            else if ((matrix[RED][i][j] != -1) && (nbands == 1))
               {
               matrix[RED][i][j] = (int)
                    (*(tie_srch_fullres[RED].image_data + offset));
               matrix[GREEN][i][j] = (int)
                    (*(tie_srch_fullres[RED].image_data + offset));
               matrix[BLUE][i][j] = (int)
                    (*(tie_srch_fullres[RED].image_data + offset));
               }
            }
         }


  /*
   *  ***  Determine the unscaled gray level value.
   */
   if (tie_cursor.flags & CURSOR_UNSCAL)
      {
      r_unscale = ((matrix[RED][1][1] - tie_planes[rplane_idx].offset) / 
                   tie_planes[rplane_idx].slope);
       g_unscale = ((matrix[GREEN][1][1] - tie_planes[gplane_idx].offset) / 
                   tie_planes[gplane_idx].slope);
       b_unscale = ((matrix[BLUE][1][1] - tie_planes[bplane_idx].offset) / 
                   tie_planes[bplane_idx].slope);
       sprintf(coords,"%f   %f   %f",r_unscale,g_unscale,b_unscale);

     /*
      *  *** Write out the unscaled coordinate values.
      */
      XtSetArg(args[0],XmNvalue,coords);
      XtSetValues(cursor_un_text,args,1);
      }

  /*
   *  ***  Set the foreground and background colors and write out the 
   *       gray level values.
   */
   XSetForeground(tie.display,tie_cursor.gc,
                  tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
   XSetBackground(tie.display,tie_cursor.gc,
                  tie.color_table[RGBBW_IDX + WHITE_COL].pixel);

   sprintf(coords,"%4d  %4d  %4d",matrix[RED][0][0],matrix[RED][0][1],
           matrix[RED][0][2]);
      XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,6,20,coords,
               strlen(coords));
   
   sprintf(coords,"%4d  %4d  %4d",matrix[RED][1][0],matrix[RED][1][1],
           matrix[RED][1][2]);
      XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,6,40,coords,
               strlen(coords));
   
   sprintf(coords,"%4d  %4d  %4d",matrix[RED][2][0],matrix[RED][2][1],
           matrix[RED][2][2]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,6,60,coords,
               strlen(coords));
   
   sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][0][0],matrix[GREEN][0][1],
           matrix[GREEN][0][2]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,150,20,coords,
               strlen(coords));

   sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][1][0],matrix[GREEN][1][1],
               matrix[GREEN][1][2]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,150,40,coords,
               strlen(coords));

   sprintf(coords,"%4d  %4d  %4d",matrix[GREEN][2][0],matrix[GREEN][2][1],
               matrix[GREEN][2][2]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,150,60,coords,
               strlen(coords));

   sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][0][0],matrix[BLUE][0][1],
           matrix[BLUE][0][2]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,294,20,coords,
               strlen(coords));

   sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][1][0],matrix[BLUE][1][1],
           matrix[BLUE][1][2]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,294,40,coords,
               strlen(coords));

   sprintf(coords,"%4d  %4d  %4d",matrix[BLUE][2][0],matrix[BLUE][2][1],
           matrix[BLUE][2][2]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,294,60,coords,
               strlen(coords));

  /*
   * Set the foreground to white and write out the target pixel value.  The
   * background values of the target pixel have already been set to red,
   * green and blue in the routine cursor_setup.
   */
   XSetForeground(tie.display,tie_cursor.gc,
                  tie.color_table[RGBBW_IDX + WHITE_COL].pixel);

   sprintf(coords,"%4d",matrix[RED][1][1]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,54,40,coords,
               strlen(coords));

   sprintf(coords,"%4d",matrix[GREEN][1][1]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,198,40,coords,
               strlen(coords));

   sprintf(coords,"%4d",matrix[BLUE][1][1]);
   XDrawString(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,342,40,coords,
               strlen(coords));

  /*
   *  ***  Copy the updated information to the cursor window.
   */
   XCopyArea(tie.display,tie_cursor.info_pixmap,cursor_win,tie_cursor.gc,0,0,
       CURSOR_WIDTH,CURSOR_HEIGHT,0,0);
}

/*******************************************************************************
Purpose:
    This routine is used to create the appropriate pixmaps and GC's needed
    to display cursor information.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
void cursor_setup(void)
{
   Window       cursor_win;     /* Cursor window ID.                          */
   XGCValues    values;         /* Values array for the GC.                   */
   XRectangle   rectangles[3];  /* Array for rectangle information.           */
   unsigned long valuemask = 0; /* Specify no values.                         */

  /*
   *  ***  Determine the cursor window ID and create the appropriate pixmaps.
   */
   cursor_win = XtWindow(cursor_drawingarea);
   tie_cursor.info_pixmap = XCreatePixmap(tie.display,cursor_win,CURSOR_WIDTH,
                                     CURSOR_HEIGHT,tie.visual_info[0].depth);
   tie_cursor.clear_pixmap = XCreatePixmap(tie.display,cursor_win,CURSOR_WIDTH,
                                     CURSOR_HEIGHT,tie.visual_info[0].depth);

  /*
   *  *** Create the working GC for the cursor window.
   */
   tie_cursor.gc = XCreateGC(tie.display,cursor_win,valuemask,&values);
   XSetFont(tie.display,tie_cursor.gc,tie.font_info->fid);
   XSetLineAttributes(tie.display,tie_cursor.gc,0,LineSolid,CapRound,JoinRound);
   
  /*
   *  ***  Assign the rectangle information.
   */
   rectangles[0].x = 0; rectangles[1].x = 144; rectangles[2].x = 288;
   rectangles[0].y = rectangles[1].y = rectangles[2].y = 0;
   rectangles[0].width = rectangles[1].width = rectangles[2].width = 144;
   rectangles[0].height = rectangles[1].height = rectangles[2].height = 70;
   
  /*
   *  ***  Clear the cursor pixmap.
   */
   XSetForeground(tie.display,tie_cursor.gc,
                  tie.color_table[RGBBW_IDX + WHITE_COL].pixel);
   XFillRectangle(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,0,0,
                  CURSOR_WIDTH,CURSOR_HEIGHT);

  /*
   *  ***  Draw the outlines into the cursor pixmap.
   */
   XSetForeground(tie.display,tie_cursor.gc,
                  tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
   XDrawRectangles(tie.display,tie_cursor.info_pixmap,
                   tie_cursor.gc,rectangles,3);

  /*
   * Draw the background boxes for the colored "target" (center)
   * pixel values.
   */
   rectangles[0].x = 59; rectangles[1].x = 203; rectangles[2].x = 347;
   rectangles[0].y = rectangles[1].y = rectangles[2].y = 28;
   rectangles[0].width = rectangles[1].width = rectangles[2].width = 30;
   rectangles[0].height = rectangles[1].height = rectangles[2].height = 16;

   XSetForeground(tie.display,tie_cursor.gc,
     tie.color_table[RGBBW_IDX + RED_COL].pixel);
   XFillRectangle(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,
      rectangles[0].x,rectangles[0].y,rectangles[0].width,rectangles[0].height);
   XSetForeground(tie.display,tie_cursor.gc,
      tie.color_table[RGBBW_IDX + GREEN_COL].pixel);
   XFillRectangle(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,
      rectangles[1].x,rectangles[1].y,rectangles[1].width,rectangles[1].height);
   XSetForeground(tie.display,tie_cursor.gc,
      tie.color_table[RGBBW_IDX + BLUE_COL].pixel);
   XFillRectangle(tie.display,tie_cursor.info_pixmap,tie_cursor.gc,
      rectangles[2].x,rectangles[2].y,rectangles[2].width,rectangles[2].height);

  /*
   *  ***  Fill in the cursor window by copying the information from the
   *       cursor pixmap to the cursor window.  Copy the same information to
   *       the clear pixmap.
   */
   XCopyArea(tie.display,tie_cursor.info_pixmap,cursor_win,tie_cursor.gc,0,0,
            CURSOR_WIDTH,CURSOR_HEIGHT,0,0);
   XCopyArea(tie.display,tie_cursor.info_pixmap,tie_cursor.clear_pixmap,
             tie_cursor.gc,0,0,CURSOR_WIDTH,CURSOR_HEIGHT,0,0);
}

/*******************************************************************************
Purpose:
    This routine is used to redraw the cursor window after an expose event. 
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void cursor_expose
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   Window       cursor_win;     /* Cursor window ID.                          */

  /*
   *  ***  Copy the current pixmap to the cursor window.
   */
   cursor_win = XtWindow(cursor_drawingarea);
   XCopyArea(tie.display,tie_cursor.info_pixmap,cursor_win,tie_cursor.gc,
             0,0,CURSOR_WIDTH,CURSOR_HEIGHT,0,0);
}

#define BIG_CURSOR   1
#define SMALL_CURSOR 2
/*******************************************************************************
Purpose:
    This routine is used to draw the crosshair in the appropriate image 
    window at the appropriate location.  The line/sample position is drawn 
    as part of the defined cursor.  This routine handles both mouse movement
    and keyboard movement of the cursor.  It also moves the cursor tracking
    line to the appropriate position if a function such as a profile or
    Z dimension histogram is currently active.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
void cursor_pointer
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
static   int   cursor_height, /* Cursor height.                   */
      cursor_width,  /* Cursor width.                    */
      flag;    /* Indicates which cursor to use (large or
               small).                 */

   static 
   Cursor       pointer_cursor; /* Cursor ID of pointer cursor.               */
   static char  lstr[20],       /* Line coordinate string.                    */
                sstr[20];       /* Sample coordinate string.                  */
   static int   first = TRUE;   /* Is this the first time this function was
                                   called ?                                   */
   double       xsub,           /* Current sample (subpixel accuracy).        */
                ysub;           /* Current line (subpixel accuracy).          */

   int          curx,           /* Current sample.                            */
                cury,           /* Current line.                              */
                deltax,         /* Amount to warp the cursor.                 */
                deltay,
                maxnl,          /* Maximum line value.                        */
                maxns,          /* Maximum sample value.                      */
                xc, yc,         /* coordinate of rectangle                    */
                move,           /* Amount of cursor single step.              */
                rplane_idx,     /* Index to red plane number of the currently
                                   viewed image.                              */
                rectwidth,      /* width of sliding rectangle                 */
                rectheight;     /* width of sliding rectangle                 */
   unsigned int keycode,        /* Key code from keyboard.                    */
                state;          /* Keyboard state (Shift key?)                */

  /*
   *  ***  Make sure an image is currently being viewed.
   */
   if ((w == ref_image_bulletinboard) && 
       (tie_planes[tie.rview].image_data == NULL))
      return;
   else if ((w == search_image_bulletinboard) && 
            (tie_planes[tie.sview].image_data == NULL))
      return;
   else if ((w == zoom_refwin_bulletinboard) && 
            (tie_zoom[REFERENCE].image_pixmap == NULL)) 
      return;
   else if ((w == zoom_srchwin_bulletinboard) && 
            (tie_zoom[SEARCH].image_pixmap == NULL)) 
      return;
   else if ((w == fullref_win_bulletinboard) && (tie_ref_fullres == NULL)) 
      return;
   else if ((w == fullsrch_win_bulletinboard) && (tie_srch_fullres == NULL)) 
      return;
   else if ((w == fullref_win_bulletinboard) && (tie_ref_fullres != NULL)) 
      if (tie_ref_fullres[0].image_data == NULL)
         return;
   else if ((w == fullsrch_win_bulletinboard) && (tie_srch_fullres != NULL)) 
      if (tie_srch_fullres[0].image_data == NULL)
         return;

  /*
   * If this is the first time then assign the fore/background colors,
   * clear the working pixmaps and draw the cursor crosshair.
   */
   if (first)
      {
      pointer_cursor = None;
     /*
      * Load the appropriate font and create a pixmap and GC for the pointer 
      * cursor plane.
      */
      if ((tie_cursor.cursor_font = 
               XLoadQueryFont(tie.display,wk_fonts[POINTERFONT])) == NULL)
         {
         tie_message(Shell_Cursor, "Unable to load pointer font");
         return;
         }
    if (strcmp(ServerVendor(tie.display),"Silicon Graphics") == 0)
   {
   cursor_height = 21;
   cursor_width = 21;
   flag = SMALL_CURSOR;
   }
    else
   {
   cursor_height = 41;
   cursor_width = 63;
   flag = BIG_CURSOR;
   }
      tie_cursor.pointer_pixmap=XCreatePixmap(tie.display,XtWindow(w),cursor_width,cursor_height,1);
      tie_cursor.pointer_gc= XCreateGC(tie.display,tie_cursor.pointer_pixmap,
                                       0,0);
      XSetFont(tie.display,tie_cursor.pointer_gc,tie_cursor.cursor_font->fid);

     /*
      *  ***  Set the foreground and background colors.
      */
      XSetFunction(tie.display,tie_cursor.pointer_gc,GXcopy);
      XSetForeground(tie.display,tie_cursor.pointer_gc,0);
                                               /* 0 for bit operation.*/

      XFillRectangle(tie.display,tie_cursor.pointer_pixmap,
                     tie_cursor.pointer_gc, 0,0,cursor_width,cursor_height);
      XSetForeground(tie.display,tie_cursor.pointer_gc,1); 
                                              /* 1 for bit operation. */

     /*
      *  *** Draw the crosshair.
      */
    if (flag == BIG_CURSOR)
   {
      XDrawLine(tie.display,tie_cursor.pointer_pixmap,tie_cursor.pointer_gc,10,
                20,10,40);
      XDrawLine(tie.display,tie_cursor.pointer_pixmap,tie_cursor.pointer_gc,0,
                30, 20,30);
   }
    else
   {
   XDrawLine(tie.display, tie_cursor.pointer_pixmap, tie_cursor.pointer_gc,
       10, 0, 10, 20);
   XDrawLine(tie.display, tie_cursor.pointer_pixmap, tie_cursor.pointer_gc,
       0, 10, 20, 10);
   }
   }


  /*
   * Remove all other MotionNotify events from queue -- this makes for
   * smoother scrolling by helping to limit the number of events
   * generated while we're acting on the present one.
   */
   while (XCheckTypedEvent(tie.display,MotionNotify,event));

  /*
   * Set the appropriate indices into the memory plane structure and determine
   * the maximum number of lines and samples in the window.
   */
   if (w == ref_image_bulletinboard)
      {
      rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;
      maxnl = tie_planes[rplane_idx].dspwin[NL] - 1;
      maxns = tie_planes[rplane_idx].dspwin[NS] - 1;
      }
   else if (w == search_image_bulletinboard)
      {
      rplane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
      maxnl = tie_planes[rplane_idx].dspwin[NL] - 1;
      maxns = tie_planes[rplane_idx].dspwin[NS] - 1;
      }
   else if ((w == zoom_refwin_bulletinboard) ||
            (w == zoom_srchwin_bulletinboard))
      {
      maxnl = tie_parms.factor * tie_zoom[REFERENCE].parms.in_lines;
      maxns = tie_parms.factor * tie_zoom[REFERENCE].parms.in_samps;
      }
   else if (w == fullref_win_bulletinboard)
      {
      maxnl = tie_ref_fullres[0].dspwin[NL];
      maxns = tie_ref_fullres[0].dspwin[NS];
      }
   else if (w == fullsrch_win_bulletinboard)
      {
      maxnl = tie_srch_fullres[0].dspwin[NL];
      maxns = tie_srch_fullres[0].dspwin[NS];
      }

  /*
   * If a a valid cursor key was pressed on the keyboard then move
   * the cursor to the new location.
   */
   if (event->type == KeyRelease)
      {
     /*
      *  *** Check the validity of the cursor coordinates.
      */
      if ((event->xkey.x < 0) || (event->xkey.y < 0) ||
          (event->xkey.x > maxns) || (event->xkey.y > maxnl))
         return;

     /*
      *  *** Capture the key code and the keyboard state.
      */
      keycode = event->xkey.keycode;
      state   = event->xkey.state;

     /*
      *  *** Ensure the keys validity.
      */
      if ((keycode == tie_cursor.k_key) || (keycode == tie_cursor.j_key) || 
        (keycode == tie_cursor.h_key) || (keycode == tie_cursor.l_key) ||
        (keycode == tie_cursor.left_key) || (keycode == tie_cursor.up_key) ||
        (keycode == tie_cursor.right_key) || (keycode == tie_cursor.down_key) ||
        (keycode == tie_cursor.ret_key))
         {
        /*
         *  ***  Reset the delta X and Y amounts.
         */
         deltax = deltay = 0;

        /*
         *  *** Check the keyboard state to see if the Shift key was used.
         */
         if (state == ShiftMask)
            move = 10;               /* Shift key so use delta of 10.         */
         else
            move = 1;

        /*
         *  *** Set the delta X/Y amount based on the key pressed.
         */
         if ((keycode == tie_cursor.k_key) || (keycode == tie_cursor.up_key))
            deltay -= move;
         else if ((keycode == tie_cursor.j_key) || (keycode == 
                  tie_cursor.down_key))
            deltay += move;
         else if ((keycode == tie_cursor.h_key) || (keycode == 
                  tie_cursor.left_key))
            deltax -= move;
         else if ((keycode == tie_cursor.l_key) || 
                  (keycode == tie_cursor.right_key))
            deltax += move;
/******
         else if ((keycode == tie_cursor.ret_key) && 
                  (tie.flags & TIE_SELECTING))
            {
            cursor_mark(w, NULL, event);
            fullres_button(w, NULL, event);
            }
 ******/

        /*
         *  *** If a valid move can be made then move the cursor.
         */
         if (((event->xkey.x + deltax) < 0) ||
             ((event->xkey.x + deltax) > maxns) || ((event->xkey.y + deltay) 
              < 0) || ((event->xkey.y + deltay) > maxnl)) 
            ;
         else
            XWarpPointer(tie.display,None,None,None,None,None,None,
                deltax,deltay);
         }
      }

  /*
   *  ***  Create the line/sample strings.
   */
   if ((w == ref_image_bulletinboard) || (w == search_image_bulletinboard))
      {
      getxy(w, event->xmotion.x, event->xmotion.y, &curx, &cury);
      sprintf(lstr,"l: %d",cury);
      sprintf(sstr,"s: %d",curx);
      }
   else if (w == zoom_refwin_bulletinboard) 
      {
      if (tie_zoom[REFERENCE].flags & ZOOM_FULLRES)
         cursor_sub(REFERENCE, event->xmotion.x,event->xmotion.y,
                    &xsub,&ysub,1,1, tie_ref_fullres[REFERENCE].window[SL],
                    tie_ref_fullres[REFERENCE].window[SS]);
      else
         cursor_sub(REFERENCE, event->xmotion.x,event->xmotion.y,
                 &xsub,&ysub,1,1,
                 tie_planes[tie.rview].window[SL],
                 tie_planes[tie.rview].window[SS]);

      sprintf(lstr,"%f",ysub);
      sprintf(sstr,"%f",xsub);
      }
   else if (w == zoom_srchwin_bulletinboard)
      {
      if (tie_zoom[SEARCH].flags & ZOOM_FULLRES)
         cursor_sub(SEARCH, event->xmotion.x,event->xmotion.y,
               &xsub,&ysub,1,1,
               tie_ref_fullres[SEARCH].window[SL],
               tie_ref_fullres[SEARCH].window[SS]);
      else
         cursor_sub(SEARCH, event->xmotion.x,event->xmotion.y,
               &xsub,&ysub,1,1,
               tie_planes[tie.sview].window[SL],
               tie_planes[tie.sview].window[SS]);

      sprintf(lstr,"%f",ysub);
      sprintf(sstr,"%f",xsub);
      }
   else if ((w == fullref_win_bulletinboard) ||
            (w == fullsrch_win_bulletinboard))
      {
      getlinesamp(w, event->xmotion.x, event->xmotion.y, &curx, &cury);

      sprintf(lstr,"l: %d",cury);
      sprintf(sstr,"s: %d",curx);
      }

  /*
   *  ***  Create the cursor and cursor mask.
   */
if (flag == BIG_CURSOR)
    {
   XSetFunction(tie.display,tie_cursor.pointer_gc,GXcopy);
   XSetForeground(tie.display,tie_cursor.pointer_gc,0);  
                                                /* 0 for bit operation. */

   XFillRectangle(tie.display,tie_cursor.pointer_pixmap,
                  tie_cursor.pointer_gc,15, 0,48,25);
   XSetForeground(tie.display,tie_cursor.pointer_gc,1);  
                                                /* 1 for bit operation. */

   XDrawString(tie.display,tie_cursor.pointer_pixmap,tie_cursor.pointer_gc,
               15,10, lstr,strlen(lstr));
   XDrawString(tie.display,tie_cursor.pointer_pixmap,tie_cursor.pointer_gc,
               15,20, sstr,strlen(sstr));
    }

  /*
   * According to all calculations, the "hot spot" for the crosshair should
   * be sample 10 and line 30.  However, this does not appear to give the
   * right visual affect (i.e. the line appears to be off by 1).  Therefore,
   * unless some other solution can be found, the "hot spot" has been set to
   * sample 10 and line 31.
   pointer_cursor = XCreatePixmapCursor(tie.display,tie_cursor.pointer_pixmap,
                          tie_cursor.pointer_pixmap,&tie_cursor.color,
                          &tie_cursor.color,10,31);
   */
if (flag == BIG_CURSOR)
    pointer_cursor = XCreatePixmapCursor(tie.display,tie_cursor.
   pointer_pixmap,tie_cursor.pointer_pixmap,&tie_cursor.color,&tie_cursor.color,
   10,30);
else
    pointer_cursor = XCreatePixmapCursor(tie.display,tie_cursor.
   pointer_pixmap,tie_cursor.pointer_pixmap,&tie_cursor.color,&tie_cursor.color,
   10,10);


  /*
   *  ***  Install the new cursor in the main image window.
   */
   set_window_cursor(tie.display,XtWindow(w),pointer_cursor);

  /*
   *  ***  Set the first time flag.
   */
   first = FALSE;

   if ((tie.flags & TIE_RECTCUR) &&
      ((w == ref_image_bulletinboard) || (w == search_image_bulletinboard))) 
      {   
      if ((w == ref_image_bulletinboard) && (tie.flags & TIE_ADRG))
         {
         rectwidth  = 32;
         rectheight = 32;
         }
      else if ((tie_planes[rplane_idx].skips != 1) || 
               (tie_planes[rplane_idx].skipl != 1))
         {
         rectwidth  = tie_parms.fwidth/tie_planes[rplane_idx].skips;
         rectheight = tie_parms.fheight/tie_planes[rplane_idx].skipl;
         }
      else
         rectheight = rectwidth  = 64;
   
      if (w == ref_image_bulletinboard)
         {
         xc = event->xmotion.x - rectwidth/2;
         yc = event->xmotion.y - rectheight/2;
   
         if (tie.refrect_x != -1)
            {
            XDrawRectangle(tie.display, XtWindow(w), tie.rect_gc,
               tie.refrect_x - rectwidth/2, tie.refrect_y - rectheight/2,
               rectwidth, rectheight);
            }
   
         if (xc < 0)
            xc = 0;
         else if (xc > (tie_planes[rplane_idx].dspwin[NS] - rectwidth))
            xc = tie_planes[rplane_idx].dspwin[NS]-rectwidth;

         if (yc < 0)
            yc = 0;
         else if (yc > (tie_planes[rplane_idx].dspwin[NL] - rectheight))
            yc = tie_planes[rplane_idx].dspwin[NL]-rectwidth;
      
         XDrawRectangle(tie.display,XtWindow(w),tie.rect_gc,
                        xc, yc, rectwidth, rectheight);

         tie.refrect_x = xc+rectwidth/2;
         tie.refrect_y = yc+rectheight/2;
         }
      else if (w == search_image_bulletinboard)
         {
         xc = event->xmotion.x - rectwidth/2;
         yc = event->xmotion.y - rectheight/2;

         if (xc < 0)
            xc = 0;
         else if (xc > (tie_planes[rplane_idx].dspwin[NS] - rectwidth))
            xc = tie_planes[rplane_idx].dspwin[NS]-rectwidth;

         if (yc < 0)
            yc = 0;
         else if (yc > (tie_planes[rplane_idx].dspwin[NL] - rectheight))
            yc = tie_planes[rplane_idx].dspwin[NL]-rectwidth;
      
/*
   *** WORK AROUND ALERT! ***
         The following if statement was originally after the assignment of
         of xc and yc above.  However, this caused segementation faults and
         bus errors in certain cases.  Moving the code here seemed to fix
         the problem.  This is an UGLY solution but the problem went away
         when compiled with debug and even when it was not compiled with
         debug and printf statements were inserted making it difficult 
         (that's an understatement!) to track.  At some point when TIEPTS
         is cleaned up and optimized more, this problem should be looked into
         again.
 */
         if (tie.srchrect_x != -1)
            {
            XDrawRectangle(tie.display, XtWindow(w), tie.rect_gc,
                  tie.srchrect_x - rectwidth/2, tie.srchrect_y - rectheight/2,
                  rectwidth, rectheight);
            }

         XDrawRectangle(tie.display, XtWindow(w), tie.rect_gc,
                        xc, yc, rectwidth, rectheight);

         tie.srchrect_x = xc+rectwidth/2;
         tie.srchrect_y = yc+rectheight/2;
         }
      }
}

/*******************************************************************************
Purpose:
        This routine is used to convert the input crosshair position to a
        line/sample coordinate with subpixel accuracy.

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
    This routine is roughly taken from the LAS function TIEPTS. 
*******************************************************************************/
void cursor_sub
(
    int  zoomidx,       /* Zoom window index.                         */
    int  xcursor,       /* Zero relative X position of crosshair (I). */
    int  ycursor,       /* Zero relative Y position of crosshair (I). */
    double *xsub,       /* X location of input point (subpixel accuracy) (O) */
    double *ysub,       /* Y location of input point (subpixel accuracy) (O) */
    int  skipl,         /* Line subsampling factor (I)                */
    int  skips,         /* Sample subsampling factor (I)              */
    int  fsl,           /* Starting line (file coordinate) of the image
                       the coordinate is being calculated for (I) */
    int  fss            /* Starting sample (file coordinate) of the 
                       image the coordinate is being calculated for (I) */
)
{
   double       x,              /* X coordinate                               */
                xfactor,        /* Subpixel factor to add to file coordinate. */
                xyadjust,       /* Adjustment needed to the subpixel value
                                   due to short cuts taken when the image 
                                   window was enlarged                        */
                y,              /* Y coordinate                               */
                yfactor;        /* Subpixel factor to add to file coordinate. */

   int          lborder,        /* Number of pixels thrown away from the left
                                   edge of the extracted window               */
                left,           /* Remainder of a division operation          */
                sl,             /* Starting line of extracted window.         */
                ss,             /* Startling sample of extracted window.      */
                tborder,        /* Number of pixels thrown away from the top
                                   edge of the extracted window               */
                xfil,           /* X file relative coordinate.                */
                ximg,           /* X image relative coordinate.               */
                yfil,           /* Y file relative coordinate.                */
                yimg;           /* Y image relatvie coordinate.               */

  /*
   * Determine how many pixels from the window extracted for zooming were thrown
   * away by the resampling routine. 
   */
   left = tie_zoom[zoomidx].parms.horiz_dim % 2;
   if (left == 0)
      lborder = tie_zoom[zoomidx].parms.horiz_dim / 2 - 1;
   else
      lborder = tie_zoom[zoomidx].parms.horiz_dim / 2;

   left = tie_zoom[zoomidx].parms.vert_dim % 2;
   if (left == 0)
      tborder = tie_zoom[zoomidx].parms.vert_dim / 2 - 1;
   else
      tborder = tie_zoom[zoomidx].parms.vert_dim / 2;

  /*
   * Calculate the adjustment needed due to short cuts taken when the image
   * window was zoomed.
   */
   xyadjust = ((int)(tie_parms.factor / 2.0)) * (1.0 / 
              (double)(tie_parms.factor));  

  /*
   *  *** Convert the coordinate to subpixel accuracy. 
   */
   x = (.5 + 1.0 / (double)(2 * tie_parms.factor) + xcursor /
       (double)tie_parms.factor) + xyadjust;
   y = (.5 + 1.0 / (double)(2 * tie_parms.factor) + ycursor /
       (double)tie_parms.factor) + xyadjust;
   
  /*
   *  *** Determine the 1-relative image coordinate (based on the zoom factor).
   */
   ximg = (int) (xcursor / (float)tie_parms.factor) + 1;
   yimg = (int) (ycursor / (float)tie_parms.factor) + 1;
   
  /*
   *  Determine the 1-relative file coordinate (based on the subsampling 
   *  factor and the window loaded).
   */
   xfil = (ximg * skips + fss - 1) - (skips - 1);
   yfil = (yimg * skipl + fsl - 1) - (skipl - 1);
   
  /*
   *  Determine the starting line and sample based on the window extracted and
   *  factor this into the file coordinate.
   */
   ss = tie_zoom[zoomidx].x - tie_zoom[zoomidx].parms.extract_samps / 2;
   if (ss < 0)
       ss = 0;
   ss = (ss * skips) + 1;

   sl = tie_zoom[zoomidx].y - tie_zoom[zoomidx].parms.extract_lines / 2;
   if (sl < 0)
       sl = 0;
   sl = (sl * skipl) + 1;
   
   xfil = xfil + (ss + lborder - 1);
   yfil = yfil + (sl + tborder - 1);
   
  /*
   *  Determine the fractional portion of the coordinate value and add this to
   *  the calculated file coordinate.
   */
   xfactor = x - ximg;
   yfactor = y - yimg;
   
   *xsub = xfil + xfactor;
   *ysub = yfil + yfactor;
   
}
