#include "tiepts.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine deselects all line type for the given coverage.
    This is a callback routine for the 'None' button.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
 
PROJECT:  LAS
*******************************************************************************/
void dcw_line_none
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i;              /* loop counter                         */

  /*
   *  *** set each flag to false
   */
   for (i = 0; i < dcw_coverage[(int)client].numvdt; i++)
      dcw_coverage[(int)client].vdt[i].selected = FALSE;

  /*
   *  *** remove line type selection panel
   */
   XtUnrealizeWidget(Shell_DCWlt);
   XtDestroyWidget(Dcwlt_form);
   Dcwlt_form = NULL;
   XtSetSensitive(dcw_feature_pull, True);
}

/*******************************************************************************
Purpose:
    This identifies the user selected line types.
    This is a callback routine for the 'Accept' button.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by T Rockvam, USGS/EROS Data Center, 3/94.
  - initialize attindex to indicate line color & style have not been reset
 
Algorithm Description:
*******************************************************************************/
void dcw_line_accept
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i,              /* loop counter                         */
                *selected,      /* list of indeces of selected type     */
                count;          /* number of selected type              */

  /*
   *  *** reset all selection flags
   */
   for (i = 0; i < dcw_coverage[(int)client].numvdt; i++)
      dcw_coverage[(int)client].vdt[i].selected = FALSE;

  /*
   *  *** get selected types and set corresponding flag
   */
   if (XmListGetSelectedPos(dcw_lt_list, &selected, &count))
      {
      for (i = 0; i < count; i++)
         dcw_coverage[(int)client].vdt[selected[i]-1].selected = TRUE;

      XtFree((XtPointer)selected);
      }

   for (i = 0; i < dcw_coverage[(int)client].numvdt; i++)
      {
      if (!dcw_coverage[(int)client].vdt[i].selected)
           dcw_coverage[(int)client].vdt[i].attindex = -1;
      }

  /*
   *  *** remove line type selection panel
   */
   XtUnrealizeWidget(Shell_DCWlt);
   XtDestroyWidget(Dcwlt_form);
   Dcwlt_form = NULL;
   XtSetSensitive(dcw_feature_pull, True);
}

/*******************************************************************************
Purpose:
    This routine selects all line type of the given coverage.
    This is a callback routine for the 'ALL' button.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void dcw_line_all
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i;              /* loop counter                         */

  /*
   *  *** set all flags to TRUE
   */
   for (i = 0; i < dcw_coverage[(int)client].numvdt; i++)
      dcw_coverage[(int)client].vdt[i].selected = TRUE;

  /*
   *  *** remove line type selection panel
   */
   XtUnrealizeWidget(Shell_DCWlt);
   XtDestroyWidget(Dcwlt_form);
   Dcwlt_form = NULL;
   XtSetSensitive(dcw_feature_pull, True);
}
