#include "tiepts.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine deselects all line status types for the given line type.
    This is a callback routine for the 'None' button.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
 
PROJECT:  LAS
 
Algorithm Description:
*******************************************************************************/
void dcw_status_none
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i;              /* loop counter                         */

  /*
   *  *** set each flag to false
   */
   for (i = 0; i < dcw_coverage[(int)client].numstatus; i++)
      dcw_coverage[(int)client].status[i].selected = FALSE;

  /*
   *  *** remove line type selection panel
   */
   XtUnrealizeWidget(Shell_DCWls);
   XtDestroyWidget(Dcwls_form);
   Dcwls_form = NULL;
}

/*******************************************************************************
Purpose:
    This identifies the user selected line status types.
    This is a callback routine for the 'Accept' button.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void dcw_status_accept
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i,              /* loop counter                         */
                *selected,      /* list of indeces of selected type     */
                count;          /* number of selected type              */

  /*
   *  *** reset all selection flags
   */
   for (i = 0; i < dcw_coverage[(int)client].numstatus; i++)
      dcw_coverage[(int)client].status[i].selected = FALSE;

  /*
   *  *** get selected types and set corresponding flag
   */
   if (XmListGetSelectedPos(dcw_ls_list, &selected, &count))
      {
      for (i = 0; i < count; i++)
         dcw_coverage[(int)client].status[selected[i]-1].selected = TRUE;

      XtFree((XtPointer)selected);
      }

  /*
   *  *** remove line type selection panel
   */
   XtUnrealizeWidget(Shell_DCWls);
   XtDestroyWidget(Dcwls_form);
   Dcwls_form = NULL;
}

/*******************************************************************************
Purpose:
    This routine selects all line status types of the given line type.
    This is a callback routine for the 'ALL' button.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void dcw_status_all
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i;              /* loop counter                         */

  /*
   *  *** set all flags to TRUE
   */
   for (i = 0; i < dcw_coverage[(int)client].numstatus; i++)
      dcw_coverage[(int)client].status[i].selected = TRUE;

  /*
   *  *** remove line type selection panel
   */
   XtUnrealizeWidget(Shell_DCWls);
   XtDestroyWidget(Dcwls_form);
   Dcwls_form = NULL;
}
 

