#include <stdio.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include "tiepts.h"
#include "digsup.h"


/*******************************************************************************
Purpose:
   This sets the communication paramters for the digitizer port.

By J. Hemmer, USGS/EROS Data Center, 2/92
   T. Beckmann, USGS/EROS Data Center - Modified to use Posix compliant 
        routines for setting the baud rate instead of using ioctl and directly 
        manipulating the termio structure.  This was done since a termio 
        structure layout on the sgi platform was causing problems.  Using 
        Posix routines prevents structure changes from causing problems in
        the future and it is portable to more systems.
Algorithm Description:
*******************************************************************************/
int dig_open
(
    char *portid
)
{
   struct termios Line_attrib;  /* communications line attributes        */
   int          fd;             /* file descriptor                       */

   fd = open(portid, 2);

   if (fd < 0)
      {
      fprintf(stderr, "Could not open port: %s\n", portid);
      return(NULL);
      }
   
   if (tcgetattr(fd,&Line_attrib) < 0)
      {
      fprintf(stderr, "Could not get port attributes: %d\n", errno);
      close(fd);
      return(NULL);
      }
   
  /*
   *  *** set buad rate
   */
   if ((cfsetispeed(&Line_attrib, B9600) != 0) || 
       (cfsetospeed(&Line_attrib, B9600) != 0))
   {
      fprintf(stderr, "Could not set port speed: %d\n", errno);
      close(fd);
      return(NULL);
   }
   
   if (tcsetattr(fd,TCSANOW,&Line_attrib) < 0)
      {
      fprintf(stderr, "Count not set port attributes: %d\n", errno);
      close(fd);
      return(NULL);
      }

   return(fd);
}
