#include <math.h>
#include "tiepts.h"
#include "convert.h"
#include "tpssup.h"
#include "matsup.h"
#include "protos.h"

/*******************************************************************************
Purpose:
   This function displays the residuals for each of the tie points.
   The residuals panels doubles as the rough transformation acceptance
   panel and general image display.  For the rough transformation, the
   user may select a Tie Point from the list and reselect it.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void edit_display
(
    int botflag /* flag to scroll to bottom of list           */
)
{
   Arg          args[2];        /* resource argument variable                 */
   int          i,              /* loop counter                               */
                count,          /* number of items already in list            */
                nvis;           /* number of items visible in list            */
   double       labs,           /* Absolute value of line residual            */
                temppt[2],      /* Absolute value of line residual            */
                new_rms,        /* RMS error after the new point is added to
                                   the model                                  */
                sabs;           /* Absolute value of sample residual          */

   char         tmpstr[132];    /* temporary string displaying one line       */
   XmString     xmstr;          /* temporary Xm string variable               */

   XtSetArg(args[0], XmNitemCount, &count);
   XtSetArg(args[1], XmNvisibleItemCount, &nvis);
   XtGetValues(edit_list, args, 2);

  /*
   *  *** for each of the tie points, place residuals in scroll list
   */
   for (i=0; i < tie_select.numtie; i++)
      {
      labs = fabs(tie_select.resid[i][0]);
      sabs = fabs(tie_select.resid[i][1]);
      new_rms = sqrt ((labs * labs) + (sabs * sabs));
      temppt[0] = tie_select.rtie[i][0];
      temppt[1] = tie_select.rtie[i][1];

      if (tie.mode == IM2VECT)
         proj2degrees(temppt,1,SEARCH);

      if (!tie_select.disabled[i])
         sprintf(tmpstr,"%6s %9.2f %9.2f %9.2f %9.2f %9.3f %9.3f %9.3f",
              tie_select.pt_id[i], 
              temppt[1], temppt[0],
              tie_select.stie[i][1],tie_select.stie[i][0],
              tie_select.resid[i][1], tie_select.resid[i][0],new_rms);
      else
         sprintf(tmpstr,"DISABLE%9.2f %9.2f %9.2f %9.2f",
              temppt[1],temppt[0],
              tie_select.stie[i][1],tie_select.stie[i][0]);

      xmstr = XmStringCreateLtoR(tmpstr, XmSTRING_DEFAULT_CHARSET);

      if (i < count)
         XmListReplaceItemsPos(edit_list, &xmstr, 1, i+1); 
      else
         XmListAddItem(edit_list, xmstr, 0); 

      XmStringFree(xmstr);

      }

   if (!XtIsRealized(Shell_Edit))
      {
      XtRealizeWidget(Shell_Main);
      XtManageChild(Edit_form);
      }

  /*
   *  *** scroll to bottom of the list
   */
   if (botflag && (tie_select.numtie >= nvis))
      XmListSetPos(edit_list, tie_select.numtie - nvis + 2);
}

/*******************************************************************************
Purpose:
    This routine disables/enables a selected point.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void edit_disable
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          count,          /* number of items selected                   */
                enabled,        /* number of enabled points                   */
                i,              /* loop counter                               */
                *selected;      /* list of items selected in list             */

   if (tie_rt.calc_rt != RT_CAL)
      return;

  /*
   *  *** get index of selected item
   */
   if (!XmListGetSelectedPos(edit_list, &selected, &count))
      return;
  
   if (!tie_select.disabled[selected[0]-1])
      {
      enabled = 0;

      for (i = 0; i < tie_select.numtie; i++)
          {
          if (!tie_select.disabled[i])
             enabled++;

          if (enabled == 5)
             break;
          }

      if (enabled < 5)
         {
         tie_message(Shell_Edit, "Must have at least four points enabled\n");
         return;
         }

      tie_select.disabled[selected[0]-1] = TRUE;
      }
   else
      tie_select.disabled[selected[0]-1] = FALSE;

   tie_rt.degree = tie_parms.order;
   tps_Qfit(tie_select.numtie, tie_select.rtie, tie_select.stie,
            tie_select.disabled, &tie_rt.degree, tie_rt.line_coef,
            tie_rt.samp_coef);
   edit_compute();
   edit_display(FALSE);
   XmListSelectPos(edit_list,selected[0],True);
   XtFree((XtPointer)selected);
   if ((Shell_Stats != NULL) && XtIsRealized(Shell_Stats))
      stats_display(TRUE);
}

/*******************************************************************************
Purpose:
    This routine closes the residual display window.  If the residuals
    represent the rough transformation, this function is invoked for
    the reselection of a point.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void edit_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
  /*
   *  *** remove general residual display
   */
   XtUnmanageChild(Edit_form);
   XtUnrealizeWidget(Shell_Edit);
   XtSetSensitive(main_edit_button, True);

   if (tie.flags & TIE_RESELECTING)
      {
      XtSetSensitive(edit_reselect_button, True);
      tie.flags ^= TIE_RESELECTING;
      }
} 

/*******************************************************************************
Purpose:
    This routine performs a reselection of a previously selected point
    in the rough transformation.  The fullres and zoom panels are updated,
    and the reference point is selected.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void edit_delete
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          *selected,      /* list of items selected in list             */
                count;          /* number of items selected                   */

   if ((tie_rt.calc_rt != RT_CAL) || (tie_select.seltie == 4))
      {
      tie_message(Shell_Edit, "Cannot delete when only 4 points selected");
      return;
      }

  /*
   *  *** if an item was selected from the list
   */
   if (!XmListGetSelectedPos(edit_list, &selected, &count))
      {
      XBell(tie.display, 0);
      return;
      }

   if (tie.flags & TIE_RESELECTING)
      {
      XtSetSensitive(edit_reselect_button, True);
      tie.flags ^= TIE_RESELECTING;
      }

   erase_pt(selected[0]-1,SEARCH);

   tps_delete(selected[0]-1);
   tps_update();
   XmListDeletePos(edit_list, selected[0]);
   tie_rt.degree = tie_parms.order;
   tps_Qfit(tie_select.numtie, tie_select.rtie, tie_select.stie,
            tie_select.disabled, &tie_rt.degree, tie_rt.line_coef,
            tie_rt.samp_coef);
   edit_compute();
   edit_display(FALSE);
   if ((Shell_Stats != NULL) && XtIsRealized(Shell_Stats))
      stats_display(TRUE);
   XtVaGetValues(edit_list,XmNitemCount,&count,NULL);
   if (count > 0)
       {
       /* trigger a singleSelectionCallback, edit_showstatus */
       if (count >= selected[0])
           XmListSelectPos(edit_list,selected[0],True);
       else
           XmListSelectPos(edit_list,count,True);
       }
   if (tie.mode == IM2VECT)
      XClearArea(tie.display, XtWindow(ref_image_bulletinboard),
                 0, 0, 0, 0, True); /* force exposure to redraw pts */
   XtFree((XtPointer)selected);

}

/*******************************************************************************
Purpose:
    This routine performs a reselection of a previously selected point
    in the rough transformation.  The fullres and zoom panels are updated,
    and the reference point is selected.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void edit_reselect
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XEvent       xe;
   int          *selected,      /* list of items selected in list             */
                count;          /* number of items selected                   */
   double       rsl,            /* Starting reference line.                   */
                rss,            /* Starting reference sample.                 */
                ssl,            /* Starting search line.                      */
                sss;            /* Starting search sample.                    */

  /*
   *  *** if an item was selected from the list
   */
   if (!XmListGetSelectedPos(edit_list, &selected, &count))
      {
      XBell(tie.display, 0);
      return;
      }

   if (!(tie_parms.flags & PARMS_LINESAMP))
     {
     if (tie.mode != IM2IM)
       {
       rss = tie_select.rtie[selected[0]-1][0];
       rsl = tie_select.rtie[selected[0]-1][1];
       tie_select.saveref[0] = tie_select.rtie[selected[0]-1][0];
       tie_select.saveref[1] = tie_select.rtie[selected[0]-1][1];
       }
     else
       {
       rss = tie_select.lsrtie[selected[0]-1][0];
       rsl = tie_select.lsrtie[selected[0]-1][1];
       tie_select.saveref[0] = tie_select.lsrtie[selected[0]-1][0];
       tie_select.saveref[1] = tie_select.lsrtie[selected[0]-1][1];
       }
     }
   else
     {
     rss = tie_select.rtie[selected[0]-1][0];
     rsl = tie_select.rtie[selected[0]-1][1];
     tie_select.saveref[0] = tie_select.rtie[selected[0]-1][0];
     tie_select.saveref[1] = tie_select.rtie[selected[0]-1][1];
     }

   XtSetSensitive(edit_reselect_button, False);
   tie.flags |= TIE_RESELECTING;
   tie_select.reselect = selected[0]-1;
   XtFree((XtPointer)selected);
   tie_select.curref = TRUE;

  /*
   *  *** update reference display
   */
   if (tie.mode == IM2IM)
      {
     /*
      *  *** update fullres and zoom windows
      */
      xe.xbutton.type = ButtonRelease;
      xe.xbutton.button = 1;

      if (!XtIsRealized(Shell_RefFull))
         XtRealizeWidget(Shell_RefFull);
   
      fullres_update(ref_image_bulletinboard,
                     &tie.num_ref_fullres, tie_ref_fullres,
                     (int)rsl, (int)rss);
   
      xe.xbutton.x = (int)rss - tie_ref_fullres[0].window[SS];
      xe.xbutton.y = (int)rsl - tie_ref_fullres[0].window[SL];

      zoom_button(fullref_win_bulletinboard, 0, &xe, NULL);

      if (tie.flags & TIE_ADRG)
         select_refpt(ref_image_bulletinboard, 
             tie_parms.factor*tie_zoom[REFERENCE].parms.in_samps/2,
             tie_parms.factor*tie_zoom[REFERENCE].parms.in_lines/2);
      else
         select_refpt(zoom_refwin_bulletinboard, 
             tie_parms.factor*tie_zoom[REFERENCE].parms.in_samps/2,
             tie_parms.factor*tie_zoom[REFERENCE].parms.in_lines/2);
      }

  /*
   *  ***  update search display
   */
   ref2srch(rss, rsl, &sss, &ssl);

  /*
   *  *** update fullres and zoom windows
   */
   full2over(search_image_bulletinboard,(int)sss,(int)ssl,&xe.xbutton.x,
             &xe.xbutton.y);

   if (!XtIsRealized(Shell_SrchFull))
      XtRealizeWidget(Shell_SrchFull);

   if (tie.mode == IM2VECT)
      mat_identity(tie_vector->zmatrix);

   fullres_update(search_image_bulletinboard, &tie.num_srch_fullres,
                  tie_srch_fullres, (int)ssl, (int)sss);

   xe.xbutton.button = 1;
   xe.xbutton.x = (int)sss - tie_srch_fullres[0].window[SS];
   xe.xbutton.y = (int)ssl - tie_srch_fullres[0].window[SL];
   zoom_button(fullsrch_win_bulletinboard, 0, &xe,NULL);

  /*
   *  *** if image to map, reset reference coordinate
   */
   if (tie.mode == IM2MAP)
      {
      tie_select.rtie[tie_select.reselect][0] = rss;
      tie_select.rtie[tie_select.reselect][1] = rsl;
      tie_select.curref = TRUE;
      }

  /*
   *  *** erase previous search point from display
   */
   erase_pt(tie_select.reselect,SEARCH);
}

/*******************************************************************************
Purpose:
    This routine changes the label on the edit_disable_button depending
    on whether the currently selected point is enabled or disabled.
 
By T. Rockvam, USGS/EROS Data Center, 3/94
 
Algorithm Description:
*******************************************************************************/
void edit_showstatus
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XmString     xmstr;          /* temporary Xm string variable               */
   int          *selected,      /* list of items selected in list             */
                count;          /* number of items selected                   */

  /*
   *  *** if an item was selected from the list
   */
   if (!XmListGetSelectedPos(edit_list, &selected, &count))
      return;

   if (tie_select.disabled[selected[0]-1])
      XtVaSetValues(edit_disable_button,XmNlabelString,
         (xmstr=XmStringCreateLtoR("Enable",XmSTRING_DEFAULT_CHARSET)),NULL);
   else
      XtVaSetValues(edit_disable_button,XmNlabelString,
         (xmstr=XmStringCreateLtoR("Disable",XmSTRING_DEFAULT_CHARSET)),NULL);

   XmStringFree(xmstr);
}

/*******************************************************************************
Purpose:
    This routine calculates the residuals of the current set of tie points.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void edit_compute(void)
{
   int          i;              /* loop counter                               */
   double       calc_line,      /* Calculated line value                      */
                calc_samp;      /* Calculated sample value                    */

   for (i=0; i < tie_select.numtie; i++)
      {
      if (tie_select.disabled[i])
         continue;

      calc_samp = c_eval(&tie_rt.degree, tie_rt.samp_coef, 
                         &tie_select.rtie[i][0], &tie_select.rtie[i][1]);
      calc_line = c_eval(&tie_rt.degree, tie_rt.line_coef, 
                         &tie_select.rtie[i][0], &tie_select.rtie[i][1]);

      tie_select.resid[i][0] = tie_select.stie[i][0] - calc_samp;
      tie_select.resid[i][1] = tie_select.stie[i][1] - calc_line;
      }

}
