/*
 *  This is the fullres function source file for the fullres panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <X11/cursorfont.h>
#include "tiepts.h"
#include "xid.h"
#include "convert.h"
#include "matsup.h"
#include "adrgsup.h"
#include "manage_cursor.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine initializes the fullres window parameters.

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
void fullres_create(void)
{
   char         fsiz[4];

   fullres_setup();
   sprintf(fsiz,"%ld",tie_parms.fwidth);
   fullres_size(NULL, fsiz, NULL);

  /*
   *  ***  Add appropriate event handlers if the cursor window is active.
   */
   if ((tie_cursor.flags & CURSOR_ACTIVE) && (tie_cursor.flags & CURSOR_MAN))
      {
      if (tie.mode == IM2IM)
         XtAddEventHandler(fullref_win_bulletinboard,ButtonReleaseMask|
              KeyReleaseMask,FALSE,cursor_track,NULL);
      XtAddEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask|
              KeyReleaseMask,FALSE,cursor_track,NULL);
      }
   else if ((tie_cursor.flags & CURSOR_ACTIVE) && 
            (tie_cursor.flags & CURSOR_AUTO))
      {
      if (tie.mode == IM2IM)
         XtAddEventHandler(fullref_win_bulletinboard,
            ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,
            cursor_track, NULL);
      XtAddEventHandler(fullsrch_win_bulletinboard,
         ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,
         NULL);
      }
}

/*******************************************************************************
Purpose:
    This routine removes the fullres windows and frees allocated space.

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
void fullres_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int  i;                      /* Looping index.                             */

  /*
   *   *** free search fullres pixmaps and data
   */  
   for (i=0; i < tie.num_srch_fullres; i++)
      {
      if (tie_srch_fullres[i].image_data != NULL)
         free(tie_srch_fullres[i].image_data);
      if (tie_srch_fullres[i].image_pixmap != NULL)
         XFreePixmap(tie.display,tie_srch_fullres[i].image_pixmap);
      }

   free(tie_srch_fullres);
   tie_srch_fullres = (struct TIE_FULLRES *)NULL;
   tie.num_srch_fullres = 0;

  /*
   *   *** free reference fullres pixmaps and data
   */  
   for (i=0; i < tie.num_ref_fullres; i++)
      {
      if (tie_ref_fullres[i].image_data != NULL)
         free(tie_ref_fullres[i].image_data);
      if (tie_ref_fullres[i].image_pixmap != NULL)
         XFreePixmap(tie.display,tie_ref_fullres[i].image_pixmap);
      }

   free(tie_ref_fullres);
   tie_ref_fullres = (struct TIE_FULLRES *)NULL;
   tie.num_ref_fullres = 0;

  /*
   *  ***  Remove any event handlers no longer needed. 
   */
   XtRemoveEventHandler(ref_image_bulletinboard,ButtonReleaseMask,FALSE,
          fullres_button,NULL);

   if ((tie_cursor.flags & CURSOR_ACTIVE) && (tie_cursor.flags & CURSOR_MAN))
      {
      XtRemoveEventHandler(fullref_win_bulletinboard,ButtonReleaseMask|
              KeyReleaseMask,FALSE,cursor_track,NULL);
      XtRemoveEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask|
              KeyReleaseMask,FALSE,cursor_track,NULL);
      }
   else if ((tie_cursor.flags & CURSOR_ACTIVE) && 
            (tie_cursor.flags & CURSOR_AUTO))
      {
      XtRemoveEventHandler(fullref_win_bulletinboard,ButtonReleaseMask|
               PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
      XtRemoveEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask|
               PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,NULL);
      }

   if (tie_zoom[REFERENCE].flags & ZOOM_ACTIVE) 
      {
      XtRemoveEventHandler(fullref_win_bulletinboard,ButtonReleaseMask,FALSE,
                           zoom_button,NULL);
      XtRemoveEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask,FALSE,
                           zoom_button,NULL);
      }

  /*
   *  *** clear reference zoom window and free allocated data
   */
   if (tie_zoom[REFERENCE].flags & (ZOOM_FULLRES | ZOOM_MAIN))
      {
      XSetWindowBackground(tie.display, XtWindow(zoom_refwin_bulletinboard),
             tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
      XClearWindow(tie.display,XtWindow(zoom_refwin_bulletinboard));

      if (tie_zoom[REFERENCE].image_pixmap != NULL)
         {
         XFreePixmap(tie.display,tie_zoom[REFERENCE].image_pixmap);
         tie_zoom[REFERENCE].image_pixmap = (Pixmap)NULL;
         }

      if (tie_zoom[REFERENCE].image_data != NULL)
         {
         free(tie_zoom[REFERENCE].image_data);
         tie_zoom[REFERENCE].image_data = NULL;
         }
      }

  /*
   *  *** clear zoom window and free allocated data
   */
   if (tie_zoom[SEARCH].flags & (ZOOM_FULLRES | ZOOM_MAIN))
      {
      XSetWindowBackground(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
      XClearWindow(tie.display,XtWindow(zoom_srchwin_bulletinboard));
   
      if (tie_zoom[SEARCH].image_pixmap != NULL)
         {
         XFreePixmap(tie.display,tie_zoom[SEARCH].image_pixmap);
         tie_zoom[SEARCH].image_pixmap = (Pixmap)NULL;
         }
   
      if (tie_zoom[SEARCH].image_data != NULL)
         {
         free(tie_zoom[SEARCH].image_data);
         tie_zoom[SEARCH].image_data = NULL;
         }
      }
   
   XtRemoveEventHandler(fullref_win_bulletinboard,PointerMotionMask
       |KeyReleaseMask, FALSE,cursor_pointer,NULL);
   XtRemoveEventHandler(fullsrch_win_bulletinboard,PointerMotionMask
       |KeyReleaseMask, FALSE,cursor_pointer,NULL);
   
  /*
   *  *** remove windows
   */
   XtUnmanageChild(Fullref_scrollwin);
   XtUnmanageChild(Fullsrch_scrollwin);
   XtUnmanageChild(Zoom_scrollwin);
   XtUnrealizeWidget(Shell_RefFull);
   XtUnrealizeWidget(Shell_SrchFull);
   XtSetSensitive(main_select_button,True);
   
  /*
   *  *** reset fullres flags
   */
   for (i=0; i < tie.num_ref_fullres; i++)
       tie_ref_fullres[i].flags = tie_ref_fullres[i].flags ^ FULLRES_ACTIVE;
   for (i=0; i < tie.num_srch_fullres; i++)
       tie_srch_fullres[i].flags = tie_srch_fullres[i].flags ^ FULLRES_ACTIVE;
}
   
/*******************************************************************************
Purpose:
    This routine changes the size of the corresponding fullres windows.

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
void fullres_size
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i;                      /* Looping index.                     */ 
   char         height[5],
                width[5];

  /*
   *  *** 256 x 256
   */
   if (strcmp((char*)client,"256") == 0)
      {
      tie_parms.fwidth = tie_parms.fheight = 256;
  
      for (i=0; i < tie.num_ref_fullres; i++)
         tie_ref_fullres[i].flags = tie_ref_fullres[i].flags & ~FULLRES_USER;
      for (i=0; i < tie.num_srch_fullres; i++)
         tie_srch_fullres[i].flags = tie_srch_fullres[i].flags & ~FULLRES_USER;
      }
  /*
   *  *** 512 x 512
   */
   else if (strcmp((char*)client,"512") == 0)
      {
      tie_parms.fwidth = tie_parms.fheight = 512;

      for (i=0; i < tie.num_srch_fullres; i++)
         tie_parms.fwidth = tie_parms.fheight = 512;

      for (i=0; i < tie.num_ref_fullres; i++)
         tie_ref_fullres[i].flags = tie_ref_fullres[i].flags & ~FULLRES_USER;
      for (i=0; i < tie.num_srch_fullres; i++)
         tie_srch_fullres[i].flags = tie_srch_fullres[i].flags & ~FULLRES_USER;
      }
  /*
   *  *** full screen
   */
   else if (strcmp((char*)client,"FS") == 0)
      {
      tie_parms.fwidth = DisplayWidth(tie.display,tie.screen);
      tie_parms.fheight= DisplayHeight(tie.display,tie.screen);

      sprintf(width,"%ld",tie_parms.fwidth);
      sprintf(height,"%ld",tie_parms.fheight);
  
      for (i=0; i < tie.num_ref_fullres; i++)
         tie_ref_fullres[i].flags = tie_ref_fullres[i].flags & ~FULLRES_USER;
      for (i=0; i < tie.num_srch_fullres; i++)
         tie_srch_fullres[i].flags = tie_srch_fullres[i].flags & ~FULLRES_USER;
      }
  /*
   *  *** user specific
   */
   else
      {
      for (i=0; i < tie.num_ref_fullres; i++)
         tie_ref_fullres[i].flags = tie_ref_fullres[i].flags | FULLRES_USER;
      for (i=0; i < tie.num_srch_fullres; i++)
         tie_srch_fullres[i].flags = tie_srch_fullres[i].flags | FULLRES_USER;
      }
}

/*******************************************************************************
Purpose:
    This routine updates a given fullres window

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
void fullres_update
(
   Widget       w,              /* Widget id of display bulletinboard         */
   int          *num_fullres,   /* number of fullres components               */
   struct TIE_FULLRES *tie_fullres,     /* fullres structure for ref/search image */
   int  sl,             /* Starting line of window to load.           */
   int  ss              /* Starting sample of window to load.         */
)
{
   int          i,              /* Looping index.                             */
                height,         /* height of fullres image.                   */
                imglimit,       /* Input image limit.                         */
                qflag[MAXIMG][2],/* Indicates whether user specified a window
                                   and band numbers for each input image.     */
                same_window,     /* Indicates if specified images have the
                                   same window size.                          */

                plane_idx,      /* Index into memory plane structure.         */
                status,         /* Status indicator.                          */
                width;          /* width of fullres image.                    */
   char         outheight[5],
                outwidth[5];

  /*
   *  *** erase outline and generate overview expose event
   */
   erase_fulloutline(w);

  /*
   *  *** user specified size
   */
   if (tie_fullres[0].flags & FULLRES_USER)
      {
      tie_parms.fheight = (tie_parms.fheight > 9999) ? 
                    DisplayHeight(tie.display,tie.screen) : 
                            tie_parms.fheight;
      tie_parms.fheight = (tie_parms.fheight < 0) ? 
                    DisplayHeight(tie.display,tie.screen) : 
                            tie_parms.fheight;
      tie_parms.fwidth = (tie_parms.fwidth > 9999) ? 
                    DisplayWidth(tie.display,tie.screen) : 
                            tie_parms.fwidth;
      tie_parms.fwidth = (tie_parms.fwidth < 0) ? 
                    DisplayWidth(tie.display,tie.screen) : 
                            tie_parms.fwidth;
      sprintf(outheight,"%ld",tie_parms.fheight);
      sprintf(outwidth,"%ld",tie_parms.fwidth);
      }

  /*
   *  ***  Determine the starting line and sample of the window to be extracted.
   */
   if (w == ref_image_bulletinboard)
      {
      tie.refrect_x = -1;
      tie.refrect_y = -1;
      plane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;
      if (tie.flags & TIE_ADRG)
         {
         height = 512;
         width = 512;
         }
      else
         {
         height = tie_parms.fheight;
         width = tie_parms.fwidth;
         }
      }
   else if (w == search_image_bulletinboard)
      {
      tie.srchrect_x = -1;
      tie.srchrect_y = -1;
      plane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
      height = tie_parms.fheight;
      width = tie_parms.fwidth;
      }

   sl = sl - height / 2;

   if (sl < 1)
      sl = 1;

   ss = ss - width / 2;

   if (ss < 1)
      ss = 1;

  /*
   * Reset starting line and sample if the window falls outside the displayed
   * image.
   */
   if ((sl + height) > tie_planes[plane_idx].ddr.nl)
      {
      sl = tie_planes[plane_idx].ddr.nl - height - 1;
      if (sl < 1)
         {
         sl = 1;
         height = tie_planes[plane_idx].ddr.nl;
         }
      }
   if ((ss + width) > tie_planes[plane_idx].ddr.ns) 
      {
      ss = tie_planes[plane_idx].ddr.ns - width - 1;
      if (ss < 1)
         {
         ss = 1;
         width = tie_planes[plane_idx].ddr.ns;
         }
      }

   for (i=0; i < *num_fullres; i++)
      {
      if (w == ref_image_bulletinboard)
         plane_idx = tie_composite[REFERENCE].rgbplanes[i] - 1;
      else if (w == search_image_bulletinboard)
         plane_idx = tie_composite[SEARCH].rgbplanes[i] - 1;

      if (tie_fullres[i].image_data != NULL)
         {
         free(tie_fullres[i].image_data);
         tie_fullres[i].image_data = NULL;
         }

      tie_fullres[i].image_data = (unsigned char *) malloc(width*height);
      if (tie_fullres[i].image_data == NULL)
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return;
         }


      tie_load.windows[i][SL] = tie_fullres[i].window[SL] = sl; 
      tie_load.windows[i][SS] = tie_fullres[i].window[SS] = ss; 
      tie_load.windows[i][NL] = tie_fullres[i].window[NL] = height;
      tie_load.windows[i][NS] = tie_fullres[i].window[NS] = width;
      tie_load.bands[i][0] = tie_planes[plane_idx].band;
      tie_load.bands[i][1] = 0; 

      if ((w == ref_image_bulletinboard) && (tie.flags & TIE_ADRG))
         {
         strcpy(tie_load.image_name, "extract.img");
         c_cmhost("extract;img", tie_load.hnames[i]);
         imglimit = tie.max_plane;

         status = c_getin2(tie_load.image_name,tie_load.tnames,tie_load.hnames,
             &tie_load.nimgs,tie_load.windows,tie_load.bands,tie_load.nbands,
             &tie_load.totbnd,&imglimit,&same_window,qflag);

         tie_fullres[i].window[SL] = tie_adrg->extline;
         tie_fullres[i].window[SS] = tie_adrg->extsamp;
         tie_load.windows[i][NL] = tie_fullres[i].window[NL] = 512;
         tie_load.windows[i][NS] = tie_fullres[i].window[NS] = 512;
         tie_load.bands[i][0] = tie_planes[plane_idx].band;
         tie_load.bands[i][1] = 0; 
         }
      else
         {
         strcpy(tie_load.tnames[i],tie_planes[plane_idx].filename);
         status = c_cmhost(tie_planes[plane_idx].filename,tie_load.hnames[i]);
         tie_load.windows[i][SL] = tie_fullres[i].window[SL] = sl; 
         tie_load.windows[i][SS] = tie_fullres[i].window[SS] = ss; 
         }

      if (status != E_SUCC)
         {
         tie_message(Shell_Main, "Error retrieving host file name");
         return;
         }
      }

   tie_load.totbnd = tie_load.nimgs = *num_fullres;

  /*
   * Display the wait cursor (hourglass) in both the full resolution image 
   * window and main panel.
   */
   show_wait_cursor();

  /*
   *  *** Display the processing message and processing icon.
   */
   XtPopup(process_dialog_shell,XtGrabExclusive);

  /*
   *  *** load data from disk
   */
   if (w == ref_image_bulletinboard)
      status = loadlas(Shell_RefFull, fullref_win_bulletinboard, 
                       &tie_mem[REFERENCE], tie.rview, FALSE);
   else 
      status = loadlas(Shell_SrchFull, fullsrch_win_bulletinboard, 
                       &tie_mem[SEARCH], tie.sview, FALSE);

   if (status != TIE_S)
      tie_message(Shell_Main, "Error loading full resolution image");

  /*
   *  ***  Redisplay the default arrow cursor.
   */
   unshow_wait_cursor();

  /*
   * Close the processing message window and turn on the bell to let the user
   * know the image load is complete.
   */
   XtPopdown(process_dialog_shell);
}

/*******************************************************************************
Purpose:
    This routine is the callback procedure for fullres updates.

By L. Huewe, USGS/EROS Data Center, 12/91
Modified by T Rockvam, USGS/EROS Data Center, 3/94.
  - allow creation of fullres window on MB3 for im2vect

Algorithm Description:
*******************************************************************************/
void fullres_button
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   XEvent       xe;             /* X Event structure.                         */

   int          plane_idx;      /* Index to red plane number of the currently
                                   viewed image.                              */
   int          fnum,
                line,
                pt;
   double       rsl=0.0,        /* Starting reference line.                   */
                ssl,            /* Starting search line.                      */
                sss,            /* Starting search sample.                    */
                rss=0.0;        /* Starting reference sample.                 */
   double       tmpls[2];

   if (event->xbutton.button != 1)
      {
      if ((tie.mode != IM2VECT) || 
          (w != search_image_bulletinboard) || 
          (event->xbutton.button != 3))
         return;
      }

   /*
    *  *** Return if no TPS file is open.
    */
   if (!(tie.flags & TIE_OPEN))
       {
       tie_message(Shell_Main,"No TPS file open");
       return;
       }

   if ((tie.mode == IM2IM) || (w == search_image_bulletinboard))
      cursor_mark(w, client, event, continue_to_dispatch);

  /*
   *  *** if image to vector, reset alignment transformation
   */
   if (tie.mode == IM2VECT)
      mat_identity(tie_vector->zmatrix);

  /*
   *  If button 1 was selected in the main image window, update the full 
   *  resolution window.
   */
   if (w == ref_image_bulletinboard)
      {
      if (tie.flags & TIE_ADRG)
         {
         rss = 0;
         rsl = 0;

         if (!XtIsRealized(Shell_RefFull))
            XtRealizeWidget(Shell_RefFull);
         else
            XRaiseWindow(tie.display, XtWindow(Shell_RefFull));

         adrg_extract(event->xbutton.y+1, event->xbutton.x+1);
         fullres_update(w, &tie.num_ref_fullres, tie_ref_fullres, 
                        (int)rsl, (int)rss);
         }
      else if ((tie.mode != IM2MAP) && (tie.mode != IM2VECT))
         {
         plane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;
         rss = (((event->xbutton.x + 1) * tie_planes[plane_idx].skips) +
                  tie_planes[plane_idx].window[SS] - 1) - 
                  (tie_planes[plane_idx].skips - 1);
         rsl = (((event->xbutton.y + 1) * tie_planes[plane_idx].skipl) +
                  tie_planes[plane_idx].window[SL] - 1) - 
                  (tie_planes[plane_idx].skipl - 1);

         if (!XtIsRealized(Shell_RefFull))
            XtRealizeWidget(Shell_RefFull);

         fullres_update(w, &tie.num_ref_fullres, tie_ref_fullres, 
                        (int)rsl, (int)rss);
         XRaiseWindow(tie.display, XtWindow(Shell_RefFull));

         if (!(tie_parms.flags & PARMS_LINESAMP))
           {
           image2proj(REFERENCE,tmpls,rss,rsl);
           if (tie_parms.flags & PARMS_LATLONG)
             proj2degrees(tmpls,1,REFERENCE);
           rss = tmpls[0];
           rsl = tmpls[1];
           }
         }
      else if (tie.mode == IM2MAP)
         {
        /*
         *  *** for some reason, values above 0x7fff come out negative????
         */
         if (!tie_parms.manual)
            {
            event->xbutton.x &= 0x0000ffff;
            event->xbutton.y &= 0x0000ffff;
            }
   
         rss = tie_select.rtie[tie_select.seltie][0];
         rsl = tie_select.rtie[tie_select.seltie][1];
         }
      else if (tie.mode == IM2VECT)
         {
         if (vector_nnode(w, event->xbutton.x, event->xbutton.y, &fnum, &line, 
                          &pt) == E_SUCC)
            {
            rss = tie_vector->line[line].pts[pt*2];
            rsl = tie_vector->line[line].pts[pt*2+1];
            }
         }
   
     /*
      *  *** if rough trans computed, automatic update search image
      */
      if (tie_rt.calc_rt != RT_SEL)
         {
         sss = 0.0;
         ssl = 0.0;
         ref2srch(rss, rsl, &sss, &ssl);

        /*
         *  *** if digitized point is already on fullres display,
         *         just update the zoom window
         */
         getxy(fullsrch_win_bulletinboard, (int)sss, (int)ssl, &xe.xbutton.x,
               &xe.xbutton.y);
         xe.xbutton.type = ButtonRelease;

         if (!XtIsRealized(Shell_SrchFull))
            XtRealizeWidget(Shell_SrchFull);

         if ((tie.mode != IM2MAP) && (tie.mode != IM2VECT) && 
             !(tie.flags &TIE_ADRG))
            {
            fullres_update(search_image_bulletinboard, 
                           &tie.num_srch_fullres, 
                           tie_srch_fullres, (int)ssl, (int)sss);
            XWarpPointer(tie.display, None, 
                   XtWindow(fullref_win_bulletinboard), 
                   None, None, None, None,
                   tie_parms.fwidth/2, tie_parms.fheight/2);
  
            XRaiseWindow(tie.display, XtWindow(Shell_SrchFull));
            }
         else
            {
            if (!fullres_within((int)sss, (int)ssl))
               fullres_update(search_image_bulletinboard, 
                           &tie.num_srch_fullres, 
                           tie_srch_fullres, (int)ssl, (int)sss);

            xe.xbutton.button = 1;
            xe.xbutton.x = (int)sss - tie_srch_fullres[0].window[SS] + 1;
            xe.xbutton.y = (int)ssl - tie_srch_fullres[0].window[SL] + 1;
            zoom_button(fullsrch_win_bulletinboard, NULL, &xe,
                        continue_to_dispatch);
            }
         }
      else if (tie.mode != IM2MAP) 
         {
         XWarpPointer(tie.display, None, 
                 XtWindow(Shell_SrchImg), 
                 None, None, None, None,
                 tie_mem[SEARCH].view_width/2, tie_mem[SEARCH].view_height/2);
         }

      if ((tie.mode == IM2IM) && (tie_zoom[REFERENCE].flags & ZOOM_ACTIVE)
          && !(tie.flags & TIE_ADRG))
         zoom_clear(); 
      }
   else if (w == search_image_bulletinboard)
      {
      plane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
      sss = (((event->xbutton.x + 1) * tie_planes[plane_idx].skips) +
                tie_planes[plane_idx].window[SS] - 1) - 
                (tie_planes[plane_idx].skips - 1);
      ssl = (((event->xbutton.y + 1) * tie_planes[plane_idx].skipl) +
                tie_planes[plane_idx].window[SL] - 1) - 
                (tie_planes[plane_idx].skipl - 1);

      if (!XtIsRealized(Shell_SrchFull))
         XtRealizeWidget(Shell_SrchFull);

      fullres_update(w, &tie.num_srch_fullres, tie_srch_fullres, 
                     (int)ssl, (int)sss);

      if ((tie.mode == IM2IM) && XtIsRealized(Shell_RefFull))
         XWarpPointer(tie.display, None, XtWindow(fullref_win_bulletinboard), 
                 None, None, None, None,
                 tie_parms.fwidth/2, tie_parms.fheight/2);
      else if (tie.mode != IM2IM)
         XWarpPointer(tie.display, None, 
                 XtWindow(fullsrch_win_bulletinboard), 
                 None, None, None, None,
                 tie_parms.fwidth/2, tie_parms.fheight/2);

       if ((tie.mode == IM2IM) && (tie_zoom[SEARCH].flags & ZOOM_ACTIVE))
          zoom_clear();

       XRaiseWindow(tie.display, XtWindow(Shell_SrchFull));
       }
}

int fullres_within
(
    int x,
    int y       /* selected line and sample */
)
{
   int          dx, dy;         /* boundary of zoom window  */

   dx = tie_parms.zsize/2;
   dy = tie_parms.zsize/2;

   if ((x > (tie_srch_fullres[0].window[SS] + dx)) &&
       (x < (tie_srch_fullres[0].window[SS] + 
                     tie_srch_fullres[0].window[NS] - dx)) &&
       (y > (tie_srch_fullres[0].window[SL] + dy)) &&
       (y < (tie_srch_fullres[0].window[SL] + 
                     tie_srch_fullres[0].window[NL] - dy)))
      return(TRUE);
   else
      return(FALSE);
}

void fullres_setup(void)
{
   int          i;              /* Looping index.                             */

   if (tie.mode != IM2IM)
      tie.num_ref_fullres = 0;
   else if (
         (tie_composite[REFERENCE].rgbplanes[RED] == tie_composite[REFERENCE].
         rgbplanes[GREEN]) && (tie_composite[REFERENCE].rgbplanes[GREEN] ==
         tie_composite[REFERENCE].rgbplanes[BLUE]))
      tie.num_ref_fullres = 1;
   else
      tie.num_ref_fullres = 3;

   if ((tie_composite[SEARCH].rgbplanes[RED] == tie_composite[SEARCH].
       rgbplanes[GREEN]) && (tie_composite[SEARCH].rgbplanes[GREEN] ==
       tie_composite[SEARCH].rgbplanes[BLUE]))
       tie.num_srch_fullres = 1;
   else
       tie.num_srch_fullres = 3;

  /*
   *  ***  Allocate memory for the window structure. 
   */
   if (tie.mode == IM2IM)
      {
      tie_ref_fullres = (struct TIE_FULLRES *) calloc(tie.num_ref_fullres,
                     sizeof(struct TIE_FULLRES));

      if (tie_ref_fullres == NULL) 
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return;
         }
      }

   tie_srch_fullres = (struct TIE_FULLRES *) calloc(tie.num_srch_fullres,
       sizeof(struct TIE_FULLRES));

   if (tie_srch_fullres == NULL) 
      {
      tie_message(Shell_Main, "Insufficient memory available");
      return;
      }

   for (i=0; i < tie.num_ref_fullres; i++)
      {
      tie_ref_fullres[i].flags         = FULLRES_ACTIVE;
      tie_ref_fullres[i].image_data    = NULL;
      tie_ref_fullres[i].image_pixmap  = NULL;
      }

   for (i=0; i < tie.num_srch_fullres; i++)
      {
      tie_srch_fullres[i].flags        = FULLRES_ACTIVE;
      tie_srch_fullres[i].image_data   = NULL;
      tie_srch_fullres[i].image_pixmap = NULL;
      }
}
