#include <stdlib.h>
#include "tiepts.h"
#include "appres.h"
#include "cr.h"
#include "convert.h"
#include "adrgsup.h"
#include "protos.h"

extern TieAppres tie_appres;

#define BITMAP 0x1

/*******************************************************************************
Purpose:
    This routine initializes the graphics pixmaps and context.

By J. Hemmer, USGS/EROS Data Center, 5/92

Algorithm Description:
*******************************************************************************/
void graphic_init(void)
{
    XGCValues    values;             /* Values array for the GC.             */
    unsigned 
        long     valuemask = 0;      /* Specify no values.                   */
    int          i;

    /*
     *  *** if image to image or vector , initialize reference graphics planes
     */
    if (tie.mode == IM2IM)
        tie_mem[REFERENCE].graphics_pixmap = XCreatePixmap(tie.display,
                XtWindow(ref_image_bulletinboard),tie_mem[REFERENCE].mem_width,
                tie_mem[REFERENCE].mem_height, BITMAP);

    /*
     *  *** create each vector plane
     */
    if (tie.mode == IM2VECT)
    {
        for (i = 0; i < NUM_CVALS; i++)
            tie_vector->pixmap[i] = NULL;
    }

    tie_mem[SEARCH].graphics_pixmap = XCreatePixmap(tie.display,
            XtWindow(search_image_bulletinboard),tie_mem[SEARCH].mem_width,
            tie_mem[SEARCH].mem_height, BITMAP);

    /*
     *  *** initialize one bit GC
     */
    tie.bit_gc = XCreateGC(tie.display,tie_mem[SEARCH].graphics_pixmap,
            valuemask,&values);
    /*
       This statement caused problems on X terminals -- the graphics expose did
       not work properly.
       XSetPlaneMask(tie.display,tie.bit_gc,1);
     */

    /*
     *  *** clear the bitmaps
     */
    XSetBackground(tie.display,tie.bit_gc,0);
    XSetForeground(tie.display,tie.bit_gc,0);

    if (tie.mode == IM2IM)
        XFillRectangle(tie.display,tie_mem[REFERENCE].graphics_pixmap, 
                tie.bit_gc, 0,0,tie_mem[REFERENCE].mem_width,
                tie_mem[REFERENCE].mem_height);

    XFillRectangle(tie.display,tie_mem[SEARCH].graphics_pixmap,
            tie.bit_gc,0,0,tie_mem[SEARCH].mem_width,
            tie_mem[SEARCH].mem_height);

    /*
     *  *** create point selection GC
     */
    tie_select.gc = XCreateGC(tie.display,
            RootWindow(tie.display, tie.screen),
            NULL, &values);

    for (i = 0; i < NUM_COLORS; i++)
        if (strcasecmp(wk_colors[i],tie_appres.cursor_color) == 0)
            break;
    if (i == NUM_COLORS) i = 0;
    XSetForeground(tie.display, tie_select.gc,
            tie.color_table[RGBBW_IDX + i].pixel);
    cursor_setcolor(i); /* set the cursor color to match */

    XSetBackground(tie.display,tie_select.gc,
            tie.color_table[RGBBW_IDX + GREEN_COL].pixel);
    XSetFillStyle(tie.display,tie_select.gc, FillSolid);

    /*
     *  *** set tie point label font
     */
    if ((tie_select.font_info = 
                XLoadQueryFont(tie.display,wk_fonts[MEDIUMFONT])) != NULL)
        XSetFont(tie.display, tie_select.gc, tie_select.font_info->fid);
    else
        tie_select.font_info = tie.font_info; 

    XSetFont(tie.display, tie_select.gc, tie_select.font_info->fid);
    XSetFont(tie.display, tie.bit_gc, tie_select.font_info->fid);

    /*
     *  *** if image to vector, clear reference window
     */
    if (tie.mode == IM2VECT)
    {
        XSetWindowBackground(tie.display, XtWindow(ref_image_bulletinboard),
                tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
        XClearWindow(tie.display, XtWindow(ref_image_bulletinboard));
    }
}

/*******************************************************************************
Purpose:
    This routine display the current tie points in the reference overview
    and fullres panels.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void display_pt
(
    int          pt             /* index of tie point to be displayed         */
)
{
    int         wx, wy;         /* window coords of tie point                 */
    int         tmppt[2];

    XSetBackground(tie.display,tie.bit_gc,1);
    XSetForeground(tie.display,tie.bit_gc,1);

    /*
     *   *** display in reference image window
     */
    if (tie.mode == IM2IM)
    {
        if (tie.flags & TIE_ADRG)
            adrg_ll2pix(&wy, &wx, tie_select.rtie[pt][1],tie_select.rtie[pt][0],
                    tie_adrg->ovr_ul[0], tie_adrg->ovr_ul[1], 
                    tie_adrg->bsovr, tie_adrg->ascovr);
        else
        {
            if (!(tie_parms.flags & PARMS_LINESAMP))
            {
                tmppt[0] = (int)tie_select.lsrtie[pt][0];
                tmppt[1] = (int)tie_select.lsrtie[pt][1];
            }
            else
            {
                tmppt[0] = (int)tie_select.rtie[pt][0];
                tmppt[1] = (int)tie_select.rtie[pt][1];
            }

            getxy(fullref_win_bulletinboard,tmppt[0],tmppt[1],&wx,&wy);

            XDrawPoint(tie.display, tie_mem[REFERENCE].graphics_pixmap,
                    tie.bit_gc, wx, wy);
            XDrawString(tie.display, tie_mem[REFERENCE].graphics_pixmap,
                    tie.bit_gc,wx+5,wy-5, tie_select.pt_id[pt],
                    strlen(tie_select.pt_id[pt]));
            XDrawPoint(tie.display, XtWindow(ref_image_bulletinboard),
                    tie_select.gc, wx, wy);
            XDrawString(tie.display, XtWindow(ref_image_bulletinboard),
                    tie_select.gc,wx+5,wy-5, tie_select.pt_id[pt],
                    strlen(tie_select.pt_id[pt]));
        }
    }

    /*
     *   *** display in search image window
     */
    getxy(fullsrch_win_bulletinboard, (long)tie_select.stie[pt][0],
            (long)tie_select.stie[pt][1], &wx, &wy);

    XDrawPoint(tie.display, XtWindow(search_image_bulletinboard),
            tie_select.gc, wx, wy);
    XDrawString(tie.display, XtWindow(search_image_bulletinboard),tie_select.gc,
            wx+5,wy-5, tie_select.pt_id[pt], strlen(tie_select.pt_id[pt]));
    XDrawPoint(tie.display, tie_mem[SEARCH].graphics_pixmap,
            tie.bit_gc, wx, wy);
    XDrawString(tie.display, tie_mem[SEARCH].graphics_pixmap, tie.bit_gc,
            wx+5,wy-5, tie_select.pt_id[pt], strlen(tie_select.pt_id[pt]));

    /*
     *  *** if fullres window available, and point visible, display in fullres
     */
    if (XtIsRealized(Shell_RefFull))
    {
        if (tie.flags & TIE_ADRG)
        {
            adrg_ll2pix(&wy, &wx, tie_select.rtie[pt][1],tie_select.rtie[pt][0],
                    tie_adrg->image[0].img_ul[0], tie_adrg->image[0].img_ul[1],
                    tie_adrg->image[0].bs, tie_adrg->image[0].asc);

            wx -= tie_ref_fullres[0].window[SS];
            wy -= tie_ref_fullres[0].window[SL];
        }
        else
        {
            wx = (int)(tie_select.rtie[pt][0] - tie_ref_fullres[0].window[SS]);
            wy = (int)(tie_select.rtie[pt][1] - tie_ref_fullres[0].window[SL]);
        }

        if ((wx > 0) && (wy > 0) &&
                (wx < tie_ref_fullres[0].dspwin[NS]) &&
                (wy < tie_ref_fullres[0].dspwin[NL]))
        {
            XDrawPoint(tie.display, XtWindow(fullref_win_bulletinboard),
                    tie_select.gc, wx, wy);
            XDrawString(tie.display, XtWindow(fullref_win_bulletinboard),
                    tie_select.gc, wx+5,wy-5, tie_select.pt_id[pt],
                    strlen(tie_select.pt_id[pt]));
        }
    }

    /*
     *  *** if fullres window available, and point visible, display in fullres
     */
    if (XtIsRealized(Shell_SrchFull))
    {
        wx = (int)(tie_select.stie[pt][0] - tie_srch_fullres[0].window[SS]);
        wy = (int)(tie_select.stie[pt][1] - tie_srch_fullres[0].window[SL]);

        if ((wx > 0) && (wy > 0) &&
                (wx < tie_srch_fullres[0].dspwin[NS]) &&
                (wy < tie_srch_fullres[0].dspwin[NL]))
        {
            XDrawPoint(tie.display, XtWindow(fullsrch_win_bulletinboard),
                    tie_select.gc, wx, wy);
            XDrawString(tie.display, XtWindow(fullsrch_win_bulletinboard),
                    tie_select.gc, wx+5,wy-5, tie_select.pt_id[pt],
                    strlen(tie_select.pt_id[pt]));
        }
    }

    return;
}

/*******************************************************************************
Purpose:
    This routine erases the given tie point in the reference overview
    and fullres panels.

By T. Rockvam, USGS/EROS Data Center, 2/95

Algorithm Description: reverse of display_pt
*******************************************************************************/
void erase_pt
(
    int pt,             /* index of tie point to be displayed          */
    int srchwin         /* Flag to delete points in search window      */
)
{
    int         wx, wy;  /* window coords of tie point                 */
    int         tmppt[2];

    XSetFunction(tie.display,tie.bit_gc,GXclear);

    /*
     *   *** erase in reference image window
     */
    if (tie.mode == IM2IM)
    {
        if (tie.flags & TIE_ADRG)
            adrg_ll2pix(&wy, &wx, tie_select.rtie[pt][1],tie_select.rtie[pt][0],
                    tie_adrg->ovr_ul[0], tie_adrg->ovr_ul[1], 
                    tie_adrg->bsovr, tie_adrg->ascovr);
        else
        {
            if (!(tie_parms.flags & PARMS_LINESAMP))
            {
                tmppt[0] = tie_select.lsrtie[pt][0];
                tmppt[1] = tie_select.lsrtie[pt][1];
            }
            else
            {
                tmppt[0] = (int)tie_select.rtie[pt][0];
                tmppt[1] = (int)tie_select.rtie[pt][1];
            }
            getxy(fullref_win_bulletinboard, tmppt[0], tmppt[1], &wx, &wy);

            XDrawPoint(tie.display, tie_mem[REFERENCE].graphics_pixmap,
                    tie.bit_gc, wx, wy);
            XDrawString(tie.display, tie_mem[REFERENCE].graphics_pixmap,
                    tie.bit_gc,wx+5,wy-5, tie_select.pt_id[pt],
                    strlen(tie_select.pt_id[pt]));
        }
    }

    if (srchwin == SEARCH)
    {
        /*
         *   *** erase in search image window
         */
        getxy(fullsrch_win_bulletinboard, (int)tie_select.stie[pt][0],
                (int)tie_select.stie[pt][1], &wx, &wy);

        XDrawPoint(tie.display, tie_mem[SEARCH].graphics_pixmap,
                tie.bit_gc, wx, wy);
        XDrawString(tie.display,tie_mem[SEARCH].graphics_pixmap,tie.bit_gc,
                wx+5,wy-5,tie_select.pt_id[pt],strlen(tie_select.pt_id[pt]));
    }

    XSetFunction(tie.display,tie.bit_gc,GXcopy);
    graphic_refresh();
}

/*******************************************************************************
Purpose:
    This routine process X expose events on the reference and search
    overview displays.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void graphic_expose
(
    Widget        w,
    XtPointer client,
    XEvent*   event,
    Boolean*  continue_to_dispatch
)
{
    XExposeEvent   *xe;  /* expose event pointer                              */

    xe = &event->xexpose;

    /*
     *   *** copy from graphic pixmap to image widget window
     */

    if (w == ref_image_bulletinboard) 
    {
        XSetClipMask(tie.display, tie_select.gc, 
                tie_mem[REFERENCE].graphics_pixmap);
        XCopyPlane(tie.display, tie_mem[REFERENCE].graphics_pixmap, 
                XtWindow(ref_image_bulletinboard), tie_select.gc,
                xe->x, xe->y, xe->width, xe->height, 
                xe->x, xe->y, 1);

        XSetClipMask(tie.display, tie_select.gc, None);

        if (xe->count == 0) 
            display_fulloutline(w); 
    }
    else if (w == search_image_bulletinboard)
    {
        XSetClipMask(tie.display, tie_select.gc,
                tie_mem[SEARCH].graphics_pixmap);
        XCopyPlane(tie.display, tie_mem[SEARCH].graphics_pixmap,
                XtWindow(search_image_bulletinboard), tie_select.gc,
                xe->x, xe->y, xe->width, xe->height,
                xe->x, xe->y, 1);
        XSetClipMask(tie.display, tie_select.gc, None);

        if (xe->count == 0)
        {
            if (tie.mode == IM2MAP) 
            {
                if (tie_maplist != NULL)
                    display_mapoutline();
                if (tie_maplist != NULL)
                    display_maps();
            }
            else if (tie.mode == IM2VECT)
            {
                /* If this is a rough transformation, the vector outline is 
                   displayed only if a valid datum exists in both the reference
                   file and search image or if the ref file and srch image both
                   have an invalid datum. If not then display the dashed 
                   outline of the vector map only after the 4th point is picked.
                   (The rough transformation is checked in display_vectoutline).
                   This is done to prevent the junk that is displayed in the
                   search image when a transformation is not possible between 
                   the search and reference projections. */ 
                if (((tie_vector->srch_datum_exists == FALSE) &&
                   (tie_vectfile[0].indatum >= 100)) ||
                   ((tie_vector->srch_datum_exists == TRUE) &&
                   (tie_vectfile[0].indatum < 100)))
                {
                    if (tie_select.seltie >= 4)
                        display_vectoutline();  
                }
            }
            /* This shows an outline of a box in the fullres window. The 
               tiepts zoom window displays the contents of this box. */
            display_fulloutline(w);
        } 
    }
}

/*******************************************************************************
Purpose:
    This routine removes the graphic panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void graphic_destroy
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XtUnmanageChild(Graphic_form);
    XtUnrealizeWidget(Shell_Graphic);
    XtDestroyWidget(Graphic_form);
    Graphic_form = NULL;
    XtSetSensitive(main_graphic_button, True);
}

/*******************************************************************************
Purpose:
    This routine changes the color of currently defined graphics.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void graphic_setcolor
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Arg          args[1];
    static
    Widget       color_button = NULL;    /* selected attribute color button */

    /*
     *  *** if NULL client, display color requestor
     */
    if (color_button == NULL)
    {
        color_button = w;
        Color_req = Createcolor_req(Shell_Graphic, graphic_setcolor);
        XtManageChild(Color_req);
    }
    /*
     *  *** otherwise, set color and remove color requestor
     */
    else
    {
        if (XtIsRealized(Shell_SrchFull))
            XClearArea(tie.display, XtWindow(fullsrch_win_bulletinboard),
                    0,0,0,0, TRUE);

        if (XtIsRealized(Shell_Zoom))
            XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                    0,0,0,0, TRUE);

        XtSetArg(args[0],XmNforeground, COLOR(Shell_SrchImg, 
                    wk_colors[(int)client]));
        XtSetValues(color_button, args, 1);
        XSetForeground(tie.display, tie_select.gc, 
                tie.color_table[RGBBW_IDX + (int)client].pixel);

        XtUnmanageChild(Color_req);
        XtDestroyWidget(Color_req);
        color_button = NULL;
        graphic_refresh();
    }
}


/*******************************************************************************
Purpose:
    This routine toggles the tie point graphics.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void graphic_toggle
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    /*
     *  *** toggle reference image graphics
     */
    if ((strcmp((char*)client, "REFON") == 0) && 
            !(tie.graphics & REFIMAGE_GRAPHICS))
    {
        tie.graphics ^= REFIMAGE_GRAPHICS;
        XClearArea(tie.display, XtWindow(ref_image_bulletinboard), 
                0, 0, 0, 0, True);
    }
    else if ((strcmp((char*)client, "REFOFF") == 0) && 
            (tie.graphics & REFIMAGE_GRAPHICS))
    {
        tie.graphics ^= REFIMAGE_GRAPHICS;
        if (XtIsRealized(Shell_RefImg))
            XClearWindow(tie.display, XtWindow(ref_image_bulletinboard));
        if (XtIsRealized(Shell_RefFull))
            XClearWindow(tie.display, XtWindow(fullref_win_bulletinboard));
    }
    /*
     *  *** toggle search image graphics
     */
    else if ((strcmp((char*)client, "SRCHON") == 0) && 
            !(tie.graphics & SRCHIMAGE_GRAPHICS))
    {
        tie.graphics ^= SRCHIMAGE_GRAPHICS; 
        XClearArea(tie.display, XtWindow(search_image_bulletinboard), 
                0, 0, 0, 0, True);
    }
    else if ((strcmp((char*)client, "SRCHOFF") == 0) && 
            (tie.graphics & SRCHIMAGE_GRAPHICS))
    {
        tie.graphics ^= SRCHIMAGE_GRAPHICS; 
        if (XtIsRealized(Shell_SrchImg))
            XClearWindow(tie.display, XtWindow(search_image_bulletinboard));
        if (XtIsRealized(Shell_SrchFull))
            XClearWindow(tie.display, XtWindow(fullsrch_win_bulletinboard));
    }
}

/*******************************************************************************
Purpose:
    This routine process X expose events on the reference and search
    full resolution displays.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void graphic_full_expose
(
    Widget        w,
    XtPointer client,
    XEvent*   event,
    Boolean*  continue_to_dispatch
)
{
    XExposeEvent   *xe;          /* expose event pointer                     */
    int            wx, wy;       /* window coords of tie point               */
    int            i;

    xe = &event->xexpose;

    /*
     *   *** ignore all but last event and then display all
     */
    if (xe->count > 0)
        return;

    /*
     *  *** if reference fullres window available, refresh
     */
    if ((w == fullref_win_bulletinboard) && (tie.graphics & REFIMAGE_GRAPHICS))
    {
        for (i = 0; i < tie_select.seltie; i++)
        {
            if (tie.flags & TIE_ADRG)
            {
                adrg_ll2pix(&wy, &wx, tie_select.rtie[i][1], 
                        tie_select.rtie[i][0],
                        tie_adrg->image[0].img_ul[0], 
                        tie_adrg->image[0].img_ul[1],
                        tie_adrg->image[0].bs, tie_adrg->image[0].asc);

                wx -= tie_ref_fullres[0].window[SS];
                wy -= tie_ref_fullres[0].window[SL];
            }
            else
            {
                wx = (int)(tie_select.rtie[i][0] 
                        - tie_ref_fullres[0].window[SS]);
                wy = (int)(tie_select.rtie[i][1] 
                        - tie_ref_fullres[0].window[SL]);
            }

            /*
             *  *** if point is within fullres bounds
             */
            if ((wx > 0) && (wy > 0) &&
                    (wx < tie_ref_fullres[0].dspwin[NS]) &&
                    (wy < tie_ref_fullres[0].dspwin[NL]))
            {
                XDrawPoint(tie.display, XtWindow(fullref_win_bulletinboard),
                        tie_select.gc, wx, wy);
                XDrawString(tie.display, XtWindow(fullref_win_bulletinboard),
                        tie_select.gc, wx+5,wy-5, tie_select.pt_id[i],
                        strlen(tie_select.pt_id[i]));
            }
        }
    }

    /*
     *  *** if fullres window available, refresh
     */
    else if ((w == fullsrch_win_bulletinboard) && 
            (tie.graphics & SRCHIMAGE_GRAPHICS))
    {
        for (i = 0; i < tie_select.seltie; i++)
        {
            wx = (int)(tie_select.stie[i][0] - tie_srch_fullres[0].window[SS]);
            wy = (int)(tie_select.stie[i][1] - tie_srch_fullres[0].window[SL]);

            /*
             *  *** if point is within fullres bounds
             */
            if ((wx > 0) && (wy > 0) &&
                    (wx < tie_srch_fullres[0].dspwin[NS]) &&
                    (wy < tie_srch_fullres[0].dspwin[NL]))
            {
                XDrawPoint(tie.display, XtWindow(fullsrch_win_bulletinboard),
                        tie_select.gc, wx, wy);
                XDrawString(tie.display, XtWindow(fullsrch_win_bulletinboard),
                        tie_select.gc, wx+5,wy-5, tie_select.pt_id[i],
                        strlen(tie_select.pt_id[i]));
            }
        }

        if ((tie.mode == IM2VECT) && (w == fullsrch_win_bulletinboard))
        {
            /* Display the vector and image together in the zoom image only if
               a valid datum exists in both the reference file and search image 
               or if the ref file and srch image both have an invalid datum. If 
               not then don't display the vector until the 4th point is picked. 
               This prevents the junk that is displayed in the search image.*/
            if (((tie_vector->srch_datum_exists == FALSE) && 
               (tie_vectfile[0].indatum >= 100)) ||
               ((tie_vector->srch_datum_exists == TRUE) &&
               (tie_vectfile[0].indatum < 100)))
            {
                if (tie_select.seltie >= 4) 
                    vector_overlay(fullsrch_win_bulletinboard);
            }
            else
                vector_overlay(fullsrch_win_bulletinboard);
        } 
    }
}

/*******************************************************************************
Purpose:
    This routine process X expose events on the reference zoom
    resolution displays.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void graphic_zoom_expose
(
    Widget        w,
    XtPointer client,
    XEvent*   event,
    Boolean*  continue_to_dispatch
)
{
    if (tie_select.curref)
    {
        XDrawPoint(tie.display, XtWindow(zoom_refwin_bulletinboard),
                tie_select.gc, 
                tie_select.saveref[0],
                tie_select.saveref[1]);
        XDrawArc(tie.display, XtWindow(zoom_refwin_bulletinboard),
                tie_select.gc,
                tie_select.saveref[0]-4, 
                tie_select.saveref[1]-4, 
                9, 9, 0, 360*64);
    }
}

/*******************************************************************************
Purpose:
    This routine draws a outline of all maps defined in the map catalog.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to display both the name of the map and the map ID.

Algorithm Description:
*******************************************************************************/
void display_maps(void)
{
    struct TIE_MAPLIST  *temp;
    XPoint      pt[4];
    XFontStruct *font_info;
    XGCValues   values;         /* Values array for the GC.                 */
    GC          gc;
    int         breakpt,
                height,
                i,
                lb, 
                middle,
                rb,
                theight, 
                tleft, 
                twidth, 
                width;
    char        *nameid;        /* Map name and id.                   */

    if (tie_maplist == NULL)
        return;

    temp = tie_maplist;

    /*
     *  *** create point selection GC
     */
    gc = XCreateGC(tie.display, RootWindow(tie.display, tie.screen),
            NULL, &values);

    XSetForeground(tie.display, gc,
            tie.color_table[RGBBW_IDX + tie_select.mapcolor].pixel);
    XSetBackground(tie.display, gc,
            tie.color_table[RGBBW_IDX + tie_select.mapcolor].pixel);
    XSetFillStyle(tie.display, gc, FillSolid);


    if ((font_info = XLoadQueryFont(tie.display,wk_fonts[SMALLFONT])) != NULL)
        XSetFont(tie.display, gc, font_info->fid);
    else
        font_info = tie_select.font_info;

    while (temp != NULL)
    {
        width  = temp->outline[1][0] - temp->outline[0][0];
        height = temp->outline[3][1] - temp->outline[0][1];

        XDrawLine(tie.display, 
                XtWindow(search_image_bulletinboard),gc,
                temp->outline[0][0], temp->outline[0][1], 
                temp->outline[1][0], temp->outline[1][1]);
        XDrawLine(tie.display, 
                XtWindow(search_image_bulletinboard),gc,
                temp->outline[1][0], temp->outline[1][1], 
                temp->outline[2][0], temp->outline[2][1]);
        XDrawLine(tie.display, 
                XtWindow(search_image_bulletinboard),gc,
                temp->outline[2][0], temp->outline[2][1], 
                temp->outline[3][0], temp->outline[3][1]);
        XDrawLine(tie.display, 
                XtWindow(search_image_bulletinboard),gc,
                temp->outline[3][0], temp->outline[3][1], 
                temp->outline[0][0], temp->outline[0][1]);

        /*
         *  *** Append the map ID to the map name separated by the @ character.
         */
        nameid = TieMalloc(strlen(temp->name) + strlen(temp->id) + 3);
        strcpy(nameid,temp->name);
        if (strcmp(temp->id,"") != 0)
        {
            strcat(nameid," @");
            strcat(nameid,temp->id);
        }
        twidth  = XTextWidth(font_info, nameid, strlen(nameid));
        theight = font_info->ascent + font_info->descent;


        /*
         *  *** display map name
         */
        if (temp->dispname)
        {
            /*
             *  *** if text does not fit, look for convenient space and break 
             *         into two lines.
             */
            if ((twidth < width) || (strchr(nameid, ' ') == NULL))
            {
                tleft = (width - 
                        XTextWidth(font_info, nameid, strlen(nameid))
                        )/2;

                XDrawString(tie.display, XtWindow(search_image_bulletinboard), 
                        gc, temp->outline[0][0]+tleft, 
                        temp->outline[0][1]+height/2,
                        nameid, strlen(nameid));
            }
            else
            {
                middle = (int)strlen(nameid)/2;

                for (lb = middle; lb > 0; lb--)
                    if (nameid[lb] == ' ')
                        break;

                for (rb = middle; rb < (int)strlen(nameid); rb++)
                    if (nameid[rb] == ' ')
                        break;

                if ((lb > 0) && (abs(lb-middle) < abs(rb-middle)))
                    breakpt = lb;             
                else
                    breakpt = rb;             

                tleft = (width - XTextWidth(font_info, nameid, breakpt-1))/2;

                XDrawString(tie.display, XtWindow(search_image_bulletinboard), 
                        gc, 
                        temp->outline[0][0]+tleft, 
                        temp->outline[0][1]+height/2-theight,
                        nameid, breakpt);

                tleft  = (width - XTextWidth(font_info, 
                            nameid+breakpt, strlen(nameid)-breakpt))/2;

                XDrawString(tie.display, XtWindow(search_image_bulletinboard), 
                        gc, temp->outline[0][0]+tleft, 
                        temp->outline[0][1]+height/2,
                        nameid+breakpt, strlen(nameid) - breakpt);
            }
        }

        /*
         *  *** display map name
         */
        if (temp->selected)
        {
            for (i = 0; i < 4; i++)
            {
                pt[i].x = temp->outline[i][0];
                pt[i].y = temp->outline[i][1];
            }
            XFillPolygon(tie.display, XtWindow(search_image_bulletinboard),
                    tie_select.gc, pt, 4, Convex,
                    CoordModeOrigin);
        }

        temp = temp->next;
    }

    if (font_info != tie_select.font_info)
    {
        XSetFont(tie.display, gc, tie_select.font_info->fid);
        XFreeFont(tie.display, font_info);
    }

    XFreeGC(tie.display, gc);
}


/*******************************************************************************
Purpose:
    This routine draws a dashed outline of the currently registered
    map based on the rough transformation.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void display_mapoutline(void)
{
    int         i;
    int         x0, x1, x2,
                y0, y1, y2;
    double      tempx, 
                tempy,
                mycoord[2],
                rss,
                rsl,
                sss,
                ssl;

    /*
     *  *** make sure rough transformation is computed
     */
    if ((tie_rt.calc_rt == RT_SEL) || (tie_mapparm.outline[0][0] < 0))
        return;

    XSetLineAttributes(tie.display, tie_select.gc, 4, LineSolid,
            CapButt, JoinBevel);

    tempx = (double)tie_mapparm.outline[0][0];
    tempy = (double)tie_mapparm.outline[0][1];

    /*
     *  *** use registration coefficients to convert to digitizer coordinates
     */
    rss = c_eval(&tie_mapparm.order, tie_mapparm.coef_x, &tempx, &tempy);
    rsl = c_eval(&tie_mapparm.order, tie_mapparm.coef_y, &tempx, &tempy);

    /*
     *  *** convert to search line and sample
     */
    if (tie_mapparm.units == MAP_PROJ)
    {
        mycoord[0] = rss;
        mycoord[1] = rsl;
        map_proj2degrees(mycoord,1,tie_mapparm.mapdesc);
        rss = mycoord[0];
        rsl = mycoord[1];
    }
    ref2srch(rss, rsl, &sss, &ssl);

    /*
     *  *** convert to overview coordinates
     */
    full2over(search_image_bulletinboard, (int)sss, (int)ssl, &x0, &y0);

    x1 = x0;
    y1 = y0;

    for (i = 1; i < 4; i++)
    {
        tempx = (double)tie_mapparm.outline[i][0];
        tempy = (double)tie_mapparm.outline[i][1];

        /*
         *  *** convert to digitizer coordinates
         */
        rss = c_eval(&tie_mapparm.order, tie_mapparm.coef_x, &tempx, &tempy);
        rsl = c_eval(&tie_mapparm.order, tie_mapparm.coef_y, &tempx, &tempy);

        if (tie_mapparm.units == MAP_PROJ)
        {
            mycoord[0] = rss;
            mycoord[1] = rsl;
            map_proj2degrees(mycoord,1,tie_mapparm.mapdesc);
            rss = mycoord[0];
            rsl = mycoord[1];
        }
        ref2srch(rss, rsl, &sss, &ssl);

        /*
         *  *** convert to overview coordinates
         */
        full2over(search_image_bulletinboard, (int)sss, (int)ssl, &x2, &y2);

        XDrawLine(tie.display, XtWindow(search_image_bulletinboard),
                tie_select.gc, x1, y1, x2, y2); 

        x1 = x2;
        y1 = y2;
    }

    XDrawLine(tie.display, XtWindow(search_image_bulletinboard), tie_select.gc,
            x0, y0, x1, y1);
    XSetLineAttributes(tie.display, tie_select.gc, 0, LineSolid,
            CapButt, JoinBevel);
}


/*******************************************************************************
Purpose:
    This routine draws a dashed outline of the currently loaded
    vector map in the search image panel,  based on the rough transformation.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void display_vectoutline(void)
{
    int          x0, x1, x2,
                 y0, y1, y2;
    double       sx, sy,
                 vx, vy;


    /*
     *  *** make sure rough transformation is computed
     */
    if ((tie_rt.calc_rt == RT_SEL) || (tie_vector->numfiles == 0))
        return;

    XSetLineAttributes(tie.display, tie_select.gc, 0, LineOnOffDash,
            CapButt, JoinBevel);

    vx = tie_vector->llx;
    vy = tie_vector->lly;
    ref2srch(vx, vy, &sx, &sy);
    full2over(search_image_bulletinboard, (int)sx, (int)sy, &x0, &y0);

    x1 = x0;
    y1 = y0;

    vx = tie_vector->urx;
    vy = tie_vector->lly;
    ref2srch(vx, vy, &sx, &sy);
    full2over(search_image_bulletinboard, (int)sx, (int)sy, &x2, &y2);
    XDrawLine(tie.display, XtWindow(search_image_bulletinboard),tie_select.gc,
            x1, y1, x2, y2); 

    x1 = x2;
    y1 = y2;
    vx = tie_vector->urx;
    vy = tie_vector->ury;
    ref2srch(vx, vy, &sx, &sy);
    full2over(search_image_bulletinboard, (int)sx, (int)sy, &x2, &y2);
    XDrawLine(tie.display, XtWindow(search_image_bulletinboard),tie_select.gc,
            x1, y1, x2, y2); 

    x1 = x2;
    y1 = y2;
    vx = tie_vector->llx;
    vy = tie_vector->ury;
    ref2srch(vx, vy, &sx, &sy);
    full2over(search_image_bulletinboard, (int)sx, (int)sy, &x2, &y2);
    XDrawLine(tie.display, XtWindow(search_image_bulletinboard),tie_select.gc,
            x1, y1, x2, y2); 
    XDrawLine(tie.display, XtWindow(search_image_bulletinboard),tie_select.gc,
            x0, y0, x2, y2); 
    XSetLineAttributes(tie.display, tie_select.gc, 0, LineSolid,
            CapButt, JoinBevel);
}

/*******************************************************************************
Purpose:
    This routine erases the dashed outline of the currently registered
    map based on the previous rough transformation.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void erase_mapoutline(void)
{
    int         i,
                lrx, lry,       /* lower right coordinate               */
                ovx, ovy,       /* lower right coordinate               */
                ulx, uly;       /* upper left coordinate                */
    double      tempx,
                tempy,
                rss,
                rsl,
                sss,
                ssl;

    /*
     *  *** make sure rough transformation is computed
     */
    if ((tie_rt.calc_rt == RT_SEL) || (tie_mapparm.outline[0][0] < 0))
        return;

    tempx = (double)tie_mapparm.outline[0][0];
    tempy = (double)tie_mapparm.outline[0][1];

    /*
     *  *** convert to digitizer coordinates
     */
    rss = c_eval(&tie_mapparm.order, tie_mapparm.coef_x, &tempx, &tempy);
    rsl = c_eval(&tie_mapparm.order, tie_mapparm.coef_y, &tempx, &tempy);

    /* use a local version of ref2srch with old coefs */
    sss = c_eval(&tie_rt.degree, tie_rt.old_scoef, &rss, &rsl);
    ssl = c_eval(&tie_rt.degree, tie_rt.old_lcoef, &rss, &rsl);

    /*
     *  *** convert to overview coordinates
     */
    full2over(search_image_bulletinboard, (int)sss, (int)ssl, &ulx, &uly);
    lrx = ulx;
    lry = uly;

    for (i = 1; i < 4; i++)
    {
        tempx = (double)tie_mapparm.outline[i][0];
        tempy = (double)tie_mapparm.outline[i][1];

        /*
         *  *** convert to digitizer coordinates
         */
        rss = c_eval(&tie_mapparm.order, tie_mapparm.coef_x, &tempx, &tempy);
        rsl = c_eval(&tie_mapparm.order, tie_mapparm.coef_y, &tempx, &tempy);

        /* use a local version of ref2srch with old coefs */
        sss = c_eval(&tie_rt.degree, tie_rt.old_scoef, &rss, &rsl);
        ssl = c_eval(&tie_rt.degree, tie_rt.old_lcoef, &rss, &rsl);

        /*
         *  *** convert to overview coordinates
         */
        full2over(search_image_bulletinboard, (int)sss, (int)ssl, &ovx, &ovy);

        ulx = (ovx < ulx) ? ovx : ulx;
        uly = (ovy < uly) ? ovy : uly;
        lrx = (ovx > lrx) ? ovx : lrx;
        lry = (ovy > lry) ? ovy : lry;
    }

    XClearArea(tie.display, XtWindow(search_image_bulletinboard),
            ulx, uly, ulx - lrx +1, lry - lrx + 1, True);
}


/*******************************************************************************
Purpose:
    This routine erases the dashed outline of the current vector display.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void erase_vectoutline(void)
{
    int          ulx, uly,
                 lrx, lry;

    double       sx, sy,
                 vx, vy;


    /*
     *  *** make sure rough transformation is computed
     */
    if ((tie_rt.calc_rt == RT_SEL) || (tie_vector->numfiles == 0))
        return;

    /*
     *  *** get upper left coordinate
     */
    vx = tie_vector->llx;
    vy = tie_vector->ury;
    ref2srch(vx, vy, &sx, &sy);
    full2over(search_image_bulletinboard, (int)sx, (int)sy, &ulx, &uly);

    /*
     *  *** get lower right coordinate
     */
    vx = tie_vector->urx;
    vy = tie_vector->lly;
    ref2srch(vx, vy, &sx, &sy);
    full2over(search_image_bulletinboard, (int)sx, (int)sy, &lrx, &lry);

    XClearArea(tie.display, XtWindow(search_image_bulletinboard),
            0, 0, 0, 0, True);
}


/*******************************************************************************
Purpose:
    This routine refreshes all graphics.  It simply clears the window
    areas and corresponding exposure events will result.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void graphic_refresh(void)
{
    if (tie.mode == IM2IM)
        if (XtIsRealized(Shell_RefImg))
        {
            XClearArea(tie.display, XtWindow(ref_image_bulletinboard), 0, 0, 
                    0, 0, True);
        }

    XClearArea(tie.display, XtWindow(search_image_bulletinboard), 0,0,0,0,True);

    if (XtIsRealized(Shell_RefFull))
        XClearArea(tie.display, XtWindow(fullref_win_bulletinboard),0,0,0,
                0,True);
    if (XtIsRealized(Shell_SrchFull))
        XClearArea(tie.display,XtWindow(fullsrch_win_bulletinboard),0,0,0,
                0,True);
    if (XtIsRealized(Shell_Zoom))
    {
        if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
            XClearArea(tie.display,XtWindow(zoom_refwin_bulletinboard),0,0,0,
                    0,True);
        XClearArea(tie.display,XtWindow(zoom_srchwin_bulletinboard),0,0,0,
                0,True);
    }

    /*
     *  *** if image to vector, display selected points
     */
    if (tie.mode == IM2VECT)
        vector_displaypts();
    else if (tie.mode == IM2MAP)
    {
        if (tie_maplist != NULL)
            display_mapoutline();
        if (tie_maplist != NULL)
            display_maps();
    }
}

/*******************************************************************************
Purpose:
    This routine draws a dashed outline of the currently zoomed
    area of the fullres window.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void display_zoomoutline
(
    int imgindex
)
{
    Window   window;
    int      x, y, xoff, yoff;

    if (imgindex == REFERENCE)
    {
        if (!XtIsRealized(Shell_RefFull))
            return;
        window = XtWindow(fullref_win_bulletinboard);
        XClearArea(tie.display, window, 0, 0, 0, 0, True);
    }
    else
    {
        if (!XtIsRealized(Shell_SrchFull))
            return;
        window = XtWindow(fullsrch_win_bulletinboard);
        XClearArea(tie.display, window, 0, 0, 0, 0, True);
    }

    x = tie_zoom[imgindex].x;
    y = tie_zoom[imgindex].y;
    xoff = tie_parms.zsize/2;
    yoff = tie_parms.zsize/2;

    XDrawLine(tie.display, window, tie_select.gc,
            x-xoff, y-yoff, x+xoff, y-yoff);
    XDrawLine(tie.display, window, tie_select.gc,
            x+xoff, y-yoff, x+xoff, y+xoff);
    XDrawLine(tie.display, window, tie_select.gc,
            x+xoff, y+yoff, x-xoff, y+yoff);
    XDrawLine(tie.display, window, tie_select.gc,
            x-xoff, y+yoff, x-xoff, y-yoff);
}

/*******************************************************************************
Purpose:
    This routine draws a dashed outline of the current fullres 
    area on the image overview window.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void display_fulloutline
(
        Widget    w
)
{
    GC        gc;              /* X Graphic Context structure                */
    XGCValues values;          /* X Graphic Context values                   */
    int       rplane_idx,      /* image plane index                          */
              xc, yc,          /* rectangle coordinate                       */
              rectwidth,       /* rectangle width                            */
              rectheight;      /* rectangle height                           */

    /*
     *  ***  get memory plane index for the given image
     */
    if ((w == ref_image_bulletinboard) &&
            (tie_ref_fullres[0].image_data != NULL))
    {
        rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;

        /*
         *  *** convert to overview coordinates
         */
        full2over(w, tie_ref_fullres[0].window[SS]-1, 
                tie_ref_fullres[0].window[SL]-1,
                &xc, &yc);
    }
    else if ((w == search_image_bulletinboard) &&
            (tie_srch_fullres[0].image_data != NULL))
    {
        rplane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

        /*
         *  *** convert to overview coordinates
         */
        full2over(w, tie_srch_fullres[0].window[SS]-1, 
                tie_srch_fullres[0].window[SL]-1,
                &xc, &yc);
    }
    else
        return;

    if ((w == ref_image_bulletinboard) && (tie.flags & TIE_ADRG))
    {
        rectwidth  = rectheight = 32;
    }
    else
    {
        rectwidth  = tie_parms.fwidth/tie_planes[rplane_idx].skips;
        rectheight = tie_parms.fheight/tie_planes[rplane_idx].skipl;
    }

    /*
     *  ***  create marker graphic context
     */
    gc = XCreateGC(tie.display, XtWindow(w), NULL, &values);

    XSetBackground(tie.display,gc,
            tie.color_table[RGBBW_IDX + RED_COL].pixel);
    XSetForeground(tie.display,gc,
            tie.color_table[RGBBW_IDX + RED_COL].pixel);

    XDrawRectangle(tie.display, XtWindow(w), gc,
            xc, yc, rectwidth, rectheight);
    XFreeGC(tie.display, gc);
}

/*******************************************************************************
Purpose:
    This routine draws a dashed outline of the current fullres
    area on the image overview window.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void erase_fulloutline
(
    Widget    w
)
{
    int       rplane_idx,       /* image plane index                          */
              xc, yc,           /* rectangle coordinate                       */
              rectwidth,        /* rectangle width                            */
              rectheight;       /* rectangle height                           */
    /*
     *  ***  get memory plane index for the given image
     */
    if ((w == ref_image_bulletinboard) &&
            (tie_ref_fullres[0].image_data != NULL))
    {
        rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;

        /*
         *  *** convert to overview coordinates
         */
        full2over(w, tie_ref_fullres[0].window[SS]-1,
                tie_ref_fullres[0].window[SL]-1,
                &xc, &yc);
        /****
          XClearArea(tie.display, XtWindow(fullref_win_bulletinboard),
          0, 0, 0, 0, True);
         ****/
    }
    else if ((w == search_image_bulletinboard) &&
            (tie_srch_fullres[0].image_data != NULL))
    {
        rplane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

        /*
         *  *** convert to overview coordinates
         */
        full2over(w, tie_srch_fullres[0].window[SS]-1,
                tie_srch_fullres[0].window[SL]-1,
                &xc, &yc);
        /*****
          XClearArea(tie.display, XtWindow(fullsrch_win_bulletinboard),
          0, 0, 0, 0, True);
         *****/
    }
    else
    {
        XClearArea(tie.display, XtWindow(w), 0, 0, 1, 1, True);
        return;
    }

    rectwidth  = tie_parms.fwidth/tie_planes[rplane_idx].skips;
    rectheight = tie_parms.fheight/tie_planes[rplane_idx].skipl;

    XClearArea(tie.display, XtWindow(w), xc-2, yc-2, rectwidth+4, rectheight+4,
            True);
}

