/*
 *  This is the layout function source file for the layout panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */

#include "tiepts.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine dynamically allocates the amount of memory needed to 
    accomodate the user specified layout.  This includes memory for storing
    image data as well as image related information.

By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
int layout_allocplanes(void)
{
int     error = FALSE,          /* Has a memory allocation error occurred.    */
        i;                      /* Looping indices.                           */

  /*
   *  *** Allocate memory for the number of composite views to store.
   */
   tie_composite = (struct TIE_COMPOSITE *) calloc(2,
                   sizeof(struct TIE_COMPOSITE));

   if (tie_composite == NULL)
      {
      tie_message(Shell_Main, "Insufficient memory available");
      return(TIE_F);
      }
  /*
   *  *** Allocate memory for the number of memory planes specified.
   */
   tie_planes = (struct TIE_MEMPLANE *) calloc(tie_layout.mem_depth,
                sizeof(struct TIE_MEMPLANE));

   if (tie_planes == NULL)
      {
      tie_message(Shell_Main, "Insufficient memory available");
      free(tie_composite);
      error = TRUE;
      }
   else
      {
      for (i=0; ((i < tie_layout.mem_depth) && (!error)); i++)
         {
         tie_planes[i].flags = 0;
         tie_planes[i].image_data = NULL;
         }
      }

  /*
   *  *** If an error occurred, free any previously allocated memory.
   */
   if (error)
      {
      free(tie_planes);
      return(TIE_F);
      }
   else
      return(TIE_S);
   }
