/*
 *  This is the load function source file for the load panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <X11/cursorfont.h>
#include "tiepts.h"
#include "xid.h"
#include "convert.h"
#include "adrgsup.h"
#include "manage_cursor.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine closes the file selector window and sets the main menu 
    load option to sensitive.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
void load_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
  /*
   *  Close the file selector and set the load button on the main panel to 
   *  sensitive.
   */
   XtUnmanageChild(Load_fileselector);
   XtDestroyWidget(Load_fileselector);
   XtUnrealizeWidget(Shell_Load);
   XtSetSensitive(main_load_button,True);
}

/*******************************************************************************
Purpose:
    This routine is called when the user selects cancel from the file
    selector and calls load_destroy to close the file selector window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
void load_localcancel
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
  /*
   *  Close the file selector.
   */
   load_destroy(w, client, call_data);
}

/*******************************************************************************
Purpose:
    This routine is called when the user selects ok from the file
    selector and loads the image into the proper display window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
void load_localok
(
   Widget        w,
   XtPointer client_data,
   XtPointer call_data
)
{
   char *filename;              /* Input specification.                       */
   char imgfile[256];
   char *extptr;
   int  status;
   
   XmFileSelectionBoxCallbackStruct *acs =
           (XmFileSelectionBoxCallbackStruct*)call_data;
   XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);

   strcpy(imgfile, filename);

   /* Getting the directory path of
      the selected file
   --------------------------------*/
   if (tie_load.dirmask != NULL)
     XmStringFree(tie_load.dirmask);
   tie_load.dirmask = XmStringCopy(acs->mask);

  /*
   *  Close the file selection box.
   */
   load_destroy(w, client_data, call_data);

  /*
   *   *** if image loaded, get/verify output tps file
   */
   if ((int)client_data == REFERENCE)
      {
      if ((strrchr(filename, '.') != NULL) && 
          (strcmp(strrchr(filename, '.'), ".gen") == 0))
         {
        /*
         *  *** generate image file name
         */
         if (strrchr(filename, '/') != NULL)
            strcpy(imgfile,  strrchr(filename, '/')+1);
         
         extptr = strrchr(imgfile, '.');
         strcpy(extptr, "_ovr.img");
         strcpy(tie_adrg->genfile, filename);         
         adrg_over();
         }

      if ((load_image(ref_image_bulletinboard, imgfile) == E_SUCC) &&
          (!(tie.flags & TIE_SELECTING)))
         tps_entry((int)client_data);
      }
   else 
      {
      if ((load_image(search_image_bulletinboard, imgfile) == E_SUCC) &&
          (!(tie.flags & TIE_SELECTING)))
         tps_entry((int)client_data);

     /*
      *  ***  if image to vector, reset bounds, clear any existing vectors
      */
      if (tie.mode == IM2VECT) 
         { 
         vector_bounds();

         if (tie_vector->numlines > 0)
            {
            vector_free();
            XClearWindow(tie.display, XtWindow(ref_image_bulletinboard));
            }
         }
      }

  /*
   *  Free the filename string
   */
   XtFree(filename);

   if ((int)client_data == REFERENCE)
     {
     status = check_ddr(&tie_planes[tie.rview].ddr);
     if (status == E_SUCC)
       tie.rvalid = TRUE;
     else
       {
       tie.rvalid = FALSE;
       tie_parms.flags &= 0;
       tie_parms.flags ^= PARMS_LINESAMP;
       tie_message(Shell_Main, "DDR not able to support coordinate conversions.\nOutput points limited to line/sample only");
       XtVaSetValues(parm_ls_button,XmNset,True,XmNsensitive,False,NULL);
       XtVaSetValues(parm_proj_button,XmNset,False,XmNsensitive,False,NULL);
       XtVaSetValues(parm_latlong_button,XmNset,False,XmNsensitive,False,NULL);
       if (tie.mode == IM2IM)
         XtSetSensitive(parm_multiref_button,False);
       }
     }
   else if ((int)client_data == SEARCH)
     {
     status = check_ddr(&tie_planes[tie.sview].ddr);
     if (status == E_SUCC)
       tie.svalid = TRUE;
     else
       tie.svalid = FALSE;

     if ((!tie.svalid) && (tie.mode == IM2VECT))
       {
       tie_parms.flags &= 0;
       tie_parms.flags ^= PARMS_PROJ;
       tie_message(Shell_Main,"DDR not able to support coordinate conversions.\nOutput points limited to projection units.");
       XtVaSetValues(parm_oc_proj,XmNset,True,XmNsensitive,False,NULL);
       XtVaSetValues(parm_oc_latlong,XmNset,False,XmNsensitive,False,NULL);
       }
     }
}
   

/*******************************************************************************
Purpose:
    This routine is called to load an image file into memory and
    display it in the given display window.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
int load_image
(
   Widget   client,             /* ID of image display bulletinboard          */
   char    *filename    /* Input specification.                       */
)
{
   int     imglimit,            /* Input image limit.                         */
           qflag[MAXIMG][2],    /* Indicates whether user specified a window
                                   and band numbers for each input image.     */
           same_window,         /* Indicates if specified images have the
                                   same window size.                          */
           status;              /* Status indicator.                          */

  /*
   *  Parse selected filename.  This determines its LAS
   *  name, host file name, window specified and bands specified.
   */
   strcpy(tie_load.image_name,filename);
   imglimit = tie.max_plane;
   status = c_getin2(tie_load.image_name,tie_load.tnames,tie_load.hnames,
       &tie_load.nimgs,tie_load.windows,tie_load.bands,tie_load.nbands,
       &tie_load.totbnd,&imglimit,&same_window,qflag);

   if (status != E_SUCC)
     return(status);

  /*
   *  Display the wait cursor (hourglass) in both the image window and main
   *  panel.
   */
   show_wait_cursor();

  /*
   *  *** Display the processing message and processing icon.
   */
   XtPopup(process_dialog_shell,XtGrabExclusive);

  /*
   *  *** Load the specified LAS image.
   */
   if ((Widget)client == ref_image_bulletinboard)
      loadlas(Shell_RefImg, client, &tie_mem[REFERENCE], tie.rview, TRUE);
   else if ((Widget)client == search_image_bulletinboard)
      loadlas(Shell_SrchImg, client, &tie_mem[SEARCH], tie.sview, TRUE);

  /*
   *  *** Redisplay the default arrow cursor.
   */
   unshow_wait_cursor();

  /*
   *  Close the processing message window and turn on the bell to let the user
   *  know the image load is complete.
   */
   XtPopdown(process_dialog_shell);
   XBell(tie.display,0);
   
   return(status);
}

int load_display
(
   int  *rgbmem,        /* Memory planes to be loaded with image data (I). */
   int  *nplanes,       /* Number of planes loaded (I).               */
   Window imgwin,       /* Window ID for main image window (I).       */
   Widget shell,        /* Widget ID of image window shell (I).       */
   Widget win,          /* Widget ID of image window (I).             */
   int  *nl,            /* Number of lines to load (I).               */
   int  *ns,            /* Number of samples to load (I).             */
   int  *maxval,        /* Maximum image data value (I).              */
   unsigned char *rptr, /* Pointer to red image data (I).             */
   unsigned char *gptr, /* Pointer to green image data (I).           */
   unsigned char *bptr, /* Pointer to blue image data (I).            */
   int     cmflg,       /* Indicates whether or not color map should
                           be modified (I).                           */
   Pixmap *pixptr,      /* Pointer to pixmap which is contain the
                           loaded image data (I).                     */
   int  loadflg         /* Indicates whether this is being called from
                           the loadlas routine (I).                   */
)
{
   Arg     args[4];             /* Variable used to set widget resources.     */
   Arg     getargs[4];          /* Variable used to set widget resources.     */
   GC      rgb_gc;              /* Working color GC.                          */
   XGCValues
           gc_val;              /* GC set value array.                        */
   XImage  *ximage;             /* X image structure.                         */

   static unsigned char
           rr[256],             /* Reserved color values.                     */
           gr[256],
           br[256];

   float        scaler;         /* LUT scaler for actual number of colors.    */

   int  buf_size,               /* Number of bytes in a single band.          */
           i,                   /* Looping index.                             */
           curview;             /* Composite index of image display           */
   
   unsigned int
           *data,               /* Pointer to output values for 24-bit data.  */
           bindex,              /* blue lookup table index                    */
           gindex,              /* green lookup table index                   */
           rindex;              /* red lookup table index                     */
   
   short   *fulllut;            /* Fullres adjusted lut                       */
   unsigned char
           *blut,               /* Blue LUT for composite image.              */
           *composite,          /* Composite RGB 8-bit image or single band
                                   black white 8-bit image.                   */
           *dataptr,            /* Pointer to image data stored in memory
                                   plane.                                     */
           *glut,               /* Green LUT for composite image.             */
           *rgbptr[3],          /* Pointers to RGB components.                */
           *rlut;               /* Red LUT for composite image.               */
   Dimension   
           shadow,border,hilight;
   
  /*
   *  *** Calculate the image data buffer size.
   */
   buf_size = *nl * *ns;
   
/* ----------------       8-bit monitor -- black and white image.  ----------*/

   if ((tie.visual_info[0].visual->class == PseudoColor) && 
      ((*nplanes < 3) || (tie.mode == IM2IM)))
      {
      if (tie.mode == IM2IM)
         *nplanes = 1;

      composite = (unsigned char *)malloc(buf_size);

      if (composite == NULL)
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return(TIE_F);
         }

     /*
      *  *** if fullres and adjusted, map to adjusted lut
      */
      fulllut = NULL;

      if (!loadflg)
         {
         if ((win == fullref_win_bulletinboard) ||
             (win == zoom_refwin_bulletinboard))
              fulllut = tie_adjust.full_lut[REFERENCE]; 
         else if ((win == fullsrch_win_bulletinboard) ||
             (win == zoom_srchwin_bulletinboard))
            fulllut = tie_adjust.full_lut[SEARCH]; 
         }
     /*
      *  Calculate the scaler needed to reduce the image to the 
      *  appropriate number of colors.
      */
      if (*maxval < NUM_IMGCOLORS)
         scaler = 1.0;
      else
         scaler = (256.0 / (float)NUM_IMGCOLORS);    /* (2^8 / # of colors) */

     /*
      *  *** Create a linear ramp.
      */
      if (cmflg)
         {
         for (i = 0; i < NUM_IMGCOLORS; i++)
            {
            tie.color_table[IMG_IDX + i].blue = 
            tie.color_table[IMG_IDX + i].green = 
            tie.color_table[IMG_IDX + i].red = 
                    (unsigned short)(((float)i * scaler) * 257);
            tie.color_table[IMG_IDX + i].flags = DoRed | DoGreen | DoBlue;
            }
         }
     /*
      *  *** Scale the input data.
      */
      dataptr = rptr;

      if (fulllut != NULL)
         {
         for (i=0; i < buf_size; i++,dataptr++)
            {
            rindex = (int) (IMG_IDX + (((float)fulllut[*dataptr]) / scaler)); 

            rindex = (rindex < IMG_IDX) ? IMG_IDX : rindex;
            rindex = (rindex > 255) ? 255 : rindex;

            composite[i] = tie.color_table[rindex].pixel; 
            }
         }
      else
         for (i=0; i < buf_size; i++,dataptr++)
             {
             rindex = (int) (IMG_IDX + (((float) *dataptr) / scaler)); 

             rindex = (rindex < IMG_IDX) ? IMG_IDX : rindex;
             rindex = (rindex > 255) ? 255 : rindex;

             composite[i] = tie.color_table[rindex].pixel;
             }

     /*
      *  *** Store the new colormap.
      */
      if (cmflg)
          XStoreColors(tie.display,tie.colormap,&tie.color_table[IMG_IDX],
              NUM_IMGCOLORS);
      }

/* --------------------   8-bit monitor -- RGB image.  ------------------*/

   else if (tie.visual_info[0].visual->class == PseudoColor)
      {
     /*
      *  *** Allocate buffer space for LUTs.
      */
      rlut = (unsigned char *)malloc(256);
      if (rlut == NULL)
          {
          tie_message(Shell_Main, "Insufficient memory available");
          return(TIE_F);
          }
      glut = (unsigned char *)malloc(256);
      if (glut == NULL)
          {
          tie_message(Shell_Main, "Insufficient memory available");
          free(rlut);
          return(TIE_F);
          }
      blut = (unsigned char *)malloc(256);
      if (blut == NULL)
          {
          tie_message(Shell_Main, "Insufficient memory available");
          free(rlut);
          free(glut);
          return(TIE_F);
          }
     /*
      *  Compress the 3 band RGB image into a single band of image data and
      *  3 LUTs.
      */
      composite = (unsigned char *)malloc(buf_size);
      if (composite == NULL)
         {
         tie_message(Shell_Main, "Insufficient memory available");
         free(rlut);
         free(glut);
         free(blut);
         return(TIE_F);
         }

      rgbptr[RED] = rptr; 
      rgbptr[GREEN] = gptr; 
      rgbptr[BLUE] = bptr; 

      if (cmflg)
         {
         xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
                     NUM_IMGCOLORS,&composite,rlut,glut,blut,0,rr,gr,br);
        /*
         *  *** Save this LUT to be used as reserved colors for loading of full
         *      resolution windows.
         */
         for (i=0; i < 256; i++)
            {
            rr[i] = rlut[i];
            gr[i] = glut[i];
            br[i] = blut[i];
            }
         }
      else
         xid_rgb2lut(1,&rgbptr[RED],&rgbptr[GREEN],&rgbptr[BLUE],&buf_size,
              NUM_IMGCOLORS,&composite,rlut,glut,blut,NUM_IMGCOLORS,rr,gr,br);

     /*
      *  *** Build the new colormap.
      */
      if (cmflg)
         {
         for (i = 0; i < NUM_IMGCOLORS; i++)
            {
            tie.color_table[IMG_IDX + i].red = (rlut[i] * 257);
            tie.color_table[IMG_IDX + i].green = (glut[i] * 257);
            tie.color_table[IMG_IDX + i].blue = (blut[i] * 257);
            tie.color_table[IMG_IDX + i].flags = DoRed | DoGreen | DoBlue;
            }
         }

      for (i=0; i < buf_size; i++)
         composite[i] = tie.color_table[IMG_IDX + composite[i]].pixel;

     /*
      *  *** Store the new colormap.
      */
      if (cmflg)
         XStoreColors(tie.display,tie.colormap,&tie.color_table[IMG_IDX],
            NUM_IMGCOLORS);
      }

/* --------------------   24-bit monitor -- BW image.  ------------------*/

   else if ((tie.visual_info[0].visual->class == DirectColor)&&(*nplanes < 3))
      {
     /*
      *  *** Allocate buffer space.
      */
      data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
      if (data == NULL)
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return(TIE_F);
         }

     /*
      *  *** if fullres and adjusted, map to adjusted lut
      */
      fulllut = NULL;

      if (!loadflg)
         {
         if ((win == fullref_win_bulletinboard) ||
             (win == zoom_refwin_bulletinboard))
              fulllut = tie_adjust.full_lut[REFERENCE];
         else if ((win == fullsrch_win_bulletinboard) ||
             (win == zoom_srchwin_bulletinboard))
            fulllut = tie_adjust.full_lut[SEARCH];
         }

     /*
      *  Create the transform array which scales output values such that 
      *  56 colormap entries can be reserved as working entries.
      */
      if (*maxval < NUM_IMGCOLORS)
         scaler = 1.0; 
      else
         scaler = (256.0 / (float) NUM_IMGCOLORS);

     /*
      *  *** Create a linear ramp.
      */
      if (cmflg)
         {
         for (i = 0; i < NUM_IMGCOLORS; i++)
            {
            tie.color_table[IMG_IDX + i].blue = 
            tie.color_table[IMG_IDX + i].green = 
            tie.color_table[IMG_IDX + i].red = 
                    (unsigned short)(((float)i * scaler) * 257);
            tie.color_table[IMG_IDX + i].flags = DoRed | DoGreen | DoBlue;
            }
         }

     /*
      *  *** Set a pointer to the image data.
      */
      rgbptr[RED] = rptr; 

      if (fulllut != NULL)
         {
         for (i=0; i < buf_size; i++,rgbptr[RED]++)
            {
            rindex = (int) (IMG_IDX + ((float)
                      (fulllut[*rgbptr[RED]] / scaler)));

            rindex = (rindex < IMG_IDX) ? IMG_IDX : rindex;
            rindex = (rindex > 255) ? 255 : rindex;

            data[i] = (rindex << tie.red_shift) +  (rindex << tie.green_shift) 
                    + (rindex << tie.blue_shift);
            }
         }
      else
         for (i=0; i < buf_size; i++,rgbptr[RED]++)
             {
             rindex = (int) (IMG_IDX + ((float)
                    (*rgbptr[RED]/ scaler)));

             rindex = (rindex < IMG_IDX) ? IMG_IDX : rindex;
             rindex = (rindex > 255) ? 255 : rindex;

             data[i] = (rindex << tie.red_shift) +  (rindex << tie.green_shift) 
                     + (rindex << tie.blue_shift);
             }

     /*
      *  *** Store the new colormap.
      */
      if (cmflg)
          XStoreColors(tie.display,tie.colormap,&tie.color_table[IMG_IDX],
              NUM_IMGCOLORS);

      }

/* --------------------   24-bit monitor -- Color image.  ------------------*/

   else if (tie.visual_info[0].visual->class == DirectColor)
      {
     /*
      *  *** Allocate buffer space.
      */
      data = (unsigned long *)malloc(buf_size * sizeof(unsigned long));
      if (data == NULL)
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return(TIE_F);
         }

     /*
      *  *** if fullres and adjusted, map to adjusted lut
      */
      fulllut = NULL;

      if (!loadflg)
         {
         if ((win == fullref_win_bulletinboard) ||
             (win == zoom_refwin_bulletinboard))
              fulllut = tie_adjust.full_lut[REFERENCE];
         else if ((win == fullsrch_win_bulletinboard) ||
             (win == zoom_srchwin_bulletinboard))
            fulllut = tie_adjust.full_lut[SEARCH];
         }

     /*
      *  Create the transform array which scales output values such that
      *  56 colormap entries can be reserved as working entries.
      */
      if (*maxval < NUM_IMGCOLORS)
         scaler = 1.0;
      else
         scaler = (256.0 / (float) NUM_IMGCOLORS);

     /*
      *  *** Create a linear ramp.
      */
      if (cmflg)
         {
         for (i = 0; i < NUM_IMGCOLORS; i++)
            {
            tie.color_table[IMG_IDX + i].blue = 
            tie.color_table[IMG_IDX + i].green = 
            tie.color_table[IMG_IDX + i].red = 
                    (unsigned short)(((float)i * scaler) * 257);
            tie.color_table[IMG_IDX + i].flags = DoRed | DoGreen | DoBlue;
            }
         }

     /*
      *  *** Set pointers to the red, green and blue bands of image data.
      */
      rgbptr[RED] = rptr;
      rgbptr[GREEN] = gptr; 
      rgbptr[BLUE] = bptr; 

      if (fulllut != NULL)
         {
         for (i=0; i < buf_size; i++,
              rgbptr[RED]++,rgbptr[GREEN]++,rgbptr[BLUE]++)
            {
            rindex = (int) (IMG_IDX + ((float)
                      (fulllut[*rgbptr[RED]] / scaler)));

            rindex = (rindex < IMG_IDX) ? IMG_IDX : rindex;
            rindex = (rindex > 255) ? 255 : rindex;

            gindex = (int) (IMG_IDX + ((float)
                      (fulllut[*rgbptr[GREEN]] / scaler)));

            gindex = (gindex < IMG_IDX) ? IMG_IDX : gindex;
            gindex = (gindex > 255) ? 255 : gindex;

            bindex = (int) (IMG_IDX + ((float)
                      (fulllut[*rgbptr[BLUE]] / scaler)));

            bindex = (bindex < IMG_IDX) ? IMG_IDX : bindex;
            bindex = (bindex > 255) ? 255 : bindex;

            data[i] = (rindex << tie.red_shift) +  (gindex << tie.green_shift) 
                    + (bindex << tie.blue_shift);
            }
         }
      else
         for (i = 0; i < buf_size; i++, rgbptr[RED]++, rgbptr[GREEN]++, 
                                rgbptr[BLUE]++)
            {
            rindex = (int) (IMG_IDX + ((float)
                      (*rgbptr[RED] / scaler)));

            rindex = (rindex < IMG_IDX) ? IMG_IDX : rindex;
            rindex = (rindex > 255) ? 255 : rindex;

            gindex = (int) (IMG_IDX + ((float)
                      (*rgbptr[GREEN] / scaler)));

            gindex = (gindex < IMG_IDX) ? IMG_IDX : gindex;
            gindex = (gindex > 255) ? 255 : gindex;

            bindex = (int) (IMG_IDX + ((float) 
                      (*rgbptr[BLUE] / scaler)));

            bindex = (bindex < IMG_IDX) ? IMG_IDX : bindex;
            bindex = (bindex > 255) ? 255 : bindex;

            data[i] = (rindex << tie.red_shift) + (gindex << tie.green_shift) 
                    + (bindex << tie.blue_shift);
            }

     /*
      *  *** Store the new colormap.
      */
      if (cmflg)
          XStoreColors(tie.display,tie.colormap,&tie.color_table[IMG_IDX],
              NUM_IMGCOLORS);
      }

  /*
   *  *** Create a graphics context for the image window.
   */
   gc_val.function = GXcopy;
   gc_val.plane_mask = AllPlanes;

   rgb_gc = XCreateGC(tie.display,imgwin,GCFunction | GCPlaneMask,&gc_val);

  /*
   *  *** Create an X image.
   */
   if (tie.visual_info[0].visual->class == DirectColor)
       ximage = XCreateImage(tie.display,tie.visual_info[0].visual,
             tie.visual_info[0].depth,ZPixmap,0,(char *)data,*ns,*nl,8,0);
   else
       ximage = XCreateImage(tie.display,tie.visual_info[0].visual,
             tie.visual_info[0].depth,ZPixmap,0,(char *)composite,*ns,*nl,8,0);

  /*
   *  *** Create a pixmap for the image window.
   */
   *pixptr = XCreatePixmap(tie.display,imgwin,*ns,*nl,tie.visual_info[0].depth);

   if (loadflg)
      {
      if (win == ref_image_bulletinboard)
         {
         tie_mem[REFERENCE].imgpix = TRUE;
         }
      else if (win == search_image_bulletinboard)
         {
         tie_mem[SEARCH].imgpix = TRUE;
         }
      }

  /*
   *  *** Draw the image into the image pixmap.
   */
   XPutImage(tie.display,*pixptr,rgb_gc,ximage,0,0,0,0,*ns,*nl);

  /*
   *  Set the current width and height and the minimum and maximum width and
   *  height of the image window bulletinboard.
   */
   XtSetArg(args[0],XmNwidth,*ns);
   XtSetArg(args[1],XmNheight,*nl);
   XtSetArg(args[2],XmNmaxWidth,*ns);
   XtSetArg(args[3],XmNmaxHeight,*nl);
   XtSetValues(win,args,4);
   XFlush(tie.display);
   
/*----------------------------------------------------------------------
 *  The following size checks and calculations take into an account an 
 *  offset which represents the width and height of the scrollbar region.
 *----------------------------------------------------------------------*/

  /*
   *  If the viewport width is larger than full image width, set the current
   *  image window width to the full image width. 
   */
   if (tie_mem[SEARCH].view_width > *ns)
      XtSetArg(args[0],XmNwidth,*ns + tie_mem[SEARCH].view_xoffset);
   else
      XtSetArg(args[0],XmNwidth,
              tie_mem[SEARCH].view_width+tie_mem[SEARCH].view_xoffset);

  /*
   *  If the viewport height is larger than full image height, set the current
   *  image window height to the full image height. 
   */
   if (tie_mem[SEARCH].view_height > *nl)
      XtSetArg(args[1],XmNheight,*nl + tie_mem[SEARCH].view_yoffset);
   else
      XtSetArg(args[1],XmNheight,
             tie_mem[SEARCH].view_height+tie_mem[SEARCH].view_yoffset);

  /*
   *  Set the maximum image window width and height based on the full image
   *  size.
   */
   if ((win == zoom_refwin_bulletinboard)||(win == zoom_srchwin_bulletinboard))
      {
      if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
         {
         XtSetArg(args[0],XmNwidth,2*(*ns + 2*5));
         XtSetArg(args[1],XmNheight,*nl+2*10);
         XtSetArg(args[2],XmNmaxWidth,2*(*ns+2*10));
         XtSetArg(args[3],XmNmaxHeight,*nl+2*10);
         }
      else
         {
         XtSetArg(args[0],XmNwidth,(*ns + 5));
         XtSetArg(args[1],XmNheight,*nl+10);
         XtSetArg(args[2],XmNmaxWidth,(*ns+2*10));
         XtSetArg(args[3],XmNmaxHeight,*nl+2*10);
         }
      }
   else
      {
      XtSetArg(getargs[0],XmNshadowThickness,&shadow);
      XtSetArg(getargs[1],XmNborderWidth,&border);
      XtSetArg(getargs[2],XmNhighlightThickness,&hilight);
      XtGetValues(win,getargs,3);
   
      XtSetArg(args[2],XmNmaxWidth,*ns+4);
      XtSetArg(args[3],XmNmaxHeight,*nl+4);
      }
   
   XtSetValues(shell,args,4);
   XFlush(tie.display);
   
  /*
   *  *** Set the image pixmap to be the background of the image window.
   */
   XSetWindowBackgroundPixmap(tie.display,imgwin,*pixptr);

  /*
   *  *** Clear the window to update the background.
   */
   XClearArea(tie.display,imgwin, 0,0,0,0, True);

  /*
   *  Store the appropriate information in the composite view structre and
   *  reset flags and values necessary when loading a new image.
   */
   if (loadflg)
      {
      if (tie_adjust.color_adjust != NULL)
         {
         free(tie_adjust.color_adjust);
         tie_adjust.color_adjust = (XColor *)NULL;
         }
      if (tie_adjust.color_linear != NULL)
         {
         free(tie_adjust.color_linear);
         tie_adjust.color_linear = (XColor *)NULL;
         }

      tie.line_x = tie.line_y = -1;

      if (win == ref_image_bulletinboard)
         curview = REFERENCE;
      else if (win == search_image_bulletinboard)
         curview = SEARCH;

      if (*nplanes >= 3)
         {
         tie_composite[curview].rgbplanes[RED] = rgbmem[RED];
         tie_composite[curview].rgbplanes[GREEN] = rgbmem[GREEN];
         tie_composite[curview].rgbplanes[BLUE] = rgbmem[BLUE];
         }
      else 
         {
         tie_composite[curview].rgbplanes[RED] = rgbmem[RED];
         tie_composite[curview].rgbplanes[GREEN] = rgbmem[RED];
         tie_composite[curview].rgbplanes[BLUE] = rgbmem[RED];
         }
/*****
      else if (*nplanes == 2)
         {
         tie_composite[curview].rgbplanes[RED] = rgbmem[RED];
         tie_composite[curview].rgbplanes[GREEN] = rgbmem[GREEN];
         tie_composite[curview].rgbplanes[BLUE] = rgbmem[GREEN];
         }
 ******/
     }

  /*
   *  *** Free the appropriate resources.
   */
   if (tie.visual_info[0].visual->class == DirectColor)
      free(data);
   else
      {
      free(composite);
      if (*nplanes >= 3)
         {
         free(rlut);
         free(glut);
         free(blut);
         }
      }

   ximage->data = NULL;         /* data was freed above */
   XDestroyImage(ximage);
   XFreeGC(tie.display,rgb_gc);
   
   return(TIE_S);
}
