#include "tiepts.h"
#include "pixman.h"
#include "xid.h"
#include "ddr.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine opens and reads the input LAS image.  If necessary, it scales 
    it to BYTE data and subsamples it.  LOADLAS then calls the routine 
    LOAD_DISPLAY to output the image to the display screen.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
    This routine is roughly taken from the prototype function XLASDSP.
*******************************************************************************/
int loadlas
(
    Widget  shell,          /* Widget ID for image shell                 */
    Widget  bulletinboard,  /* Widget ID for image display window (I).   */
    struct  TIE_MEMLAYOUT *mem, /* Memory layout for this image              */
    int     view,       /* Composite view index                      */
    int     loadflg     /* Indicates whether the image is being
                               loaded into the main image window (I).    */
)
{
Arg     args[2];                /* Variable used to set widget resources.     */

struct  TIE_MEMPLANE    
        *memptr;                /* Pointer to current memory plane structure. */

struct  TIE_FULLRES
        *fresptr;               /* Pointer to full resolution structure.      */

double  offset,                 /* Offset used to scale non-byte data.        */
        slope,                  /* Slope used to scale non-byte data.         */
        minval,                 /* Minimum pixel value of current band.       */
        maxval;                 /* Maximum pixel value of current band.       */

float   *floatptr;              /* FLOAT data pointer.                        */

struct FDESC *fdesc;            /* Input image file descriptor.               */

long    access = IREAD,         /* Image file open access method.             */
        bnd,                    /* Band counter for looping.                  */
        dspwin[4],              /* Display window.                            */
        dtype = ELONG,          /* Data type of histogram buffer.             */
        i,                      /* Looping indices.                           */
        img,                    /* Image counter for looping.                 */
        lin,                    /* Line counter for looping.                  */
        linnum,                 /* Line number required by image I/O.         */
        *longptr,               /* LONG data pointer.                         */
    nbands=0,           /* Number of bands in the file.               */
        nbins = 256,            /* Number of histogram bins.                  */
        one = 1,                /* Constant one.                              */
        plane1,                 /* Index of first plane number                */
        skipl,                  /* Subsampling factor in line direction.      */
        skips,                  /* Subsampling factor in sample direction.    */
        smp,                    /* Sample counter for looping.                */
        status,                 /* Status indicator.                          */
        tbnd,                   /* Total number of bands counter.             */
        wsize,                  /* Word size of input image data type.        */
        zero = 0;               /* Constant zero.                             */
int bands[2];           /* Bands array for min/max calculation.       */
int     *histptr;               /* Pointer to histogram data.                 */
int *rgbmem;            /* Memory planes to be loaded with image data.*/

short   *shortptr;              /* SHORT data pointer.                        */

unsigned char
        *dataptr,               /* Pointer to image data stored in memory
                                   plane.                                     */
        *linebuf;               /* Buffer to read image data into.            */

char    title[100];             /* Title for processing message window.       */

/*
 *  ***  Free the previously allocated image pixmap.
 */
if (loadflg)
    {
    if (tie_load.totbnd > 3)
       tie_load.totbnd = 3;
    
    if (mem->imgpix)
        {
        XFreePixmap(tie.display,mem->image_pixmap);
        mem->imgpix = FALSE;
        }
    }
else
    {
    if (bulletinboard == fullref_win_bulletinboard)
       {
       for (i=0; i < tie.num_ref_fullres; i++)
           if (tie_ref_fullres[i].image_pixmap != NULL)
               {
               XFreePixmap(tie.display,tie_ref_fullres[i].image_pixmap);
               tie_ref_fullres[i].image_pixmap = (Pixmap)NULL;
               }
       }

    if (bulletinboard == fullsrch_win_bulletinboard)
       {
       for (i=0; i < tie.num_srch_fullres; i++)
           if (tie_srch_fullres[i].image_pixmap != NULL)
               {
               XFreePixmap(tie.display,tie_srch_fullres[i].image_pixmap);
               tie_srch_fullres[i].image_pixmap = (Pixmap)NULL;
               }
       }
    }

/*
 *  ***  Allocate buffer space for the memory ids.
 */
rgbmem = (int *) calloc(tie_load.totbnd,sizeof(int));
if (rgbmem == NULL)
    {
    tie_message(Shell_Main, "Insufficient memory available");
    return(TIE_F);
    }

if (loadflg)
    {
    for (plane1 = 0; plane1 < 3-tie_load.totbnd; plane1++)
        if (tie_planes[view+plane1].image_data == NULL)
           break;
   /*
    *   *** allocate any unallocated planes
    */
    for (i = plane1; i < plane1+tie_load.totbnd; i++)
       {
       if (tie_planes[view + i].image_data == NULL)
          if ((tie.flags & TIE_ADRG) && 
              (bulletinboard == ref_image_bulletinboard))
             {
             tie_planes[view + i].image_data = (unsigned char *) malloc(
                    tie_load.windows[0][NL] * tie_load.windows[0][NS]);
             if (tie_planes[view + i].image_data == NULL)
                {
                tie_message(Shell_Main, "Insufficient memory available");
                return(TIE_F);
                }
             }
          else
             {
             tie_planes[view + i].image_data = (unsigned char *) malloc(
                    tie_layout.mem_width * tie_layout.mem_height);
             if (tie_planes[view + i].image_data == NULL)
                {
                tie_message(Shell_Main, "Insufficient memory available");
                return(TIE_F);
                }
             }

       c_pxfill(tie_planes[view + i].histogram,&dtype,&nbins,&zero);
       }
    }
else
    {
    plane1 = 0;

    if (bulletinboard == fullref_win_bulletinboard)
       for (i=0; i < tie.num_ref_fullres; i++)
           c_pxfill(tie_ref_fullres[i].histogram,&dtype,&nbins,&zero);
    else if (bulletinboard == fullsrch_win_bulletinboard)
       for (i=0; i < tie.num_srch_fullres; i++)
           c_pxfill(tie_srch_fullres[i].histogram,&dtype,&nbins,&zero);
    }

/*
 *  For each image and each band, read in the image data and scale it to
 *  BYTE if necessary.
 */
for (img=0, tbnd = plane1; img < tie_load.nimgs; img++)
    {
   /*
    *  *** tiepts is set to handle no more than 3 bands
    */
    if (tbnd > 2)
       break;

   /*
    *  *** Open the input image.
    */
    dtype = 0;

    status = c_eopenr(&fdesc,tie_load.hnames[img],&nbands,
        &tie_load.windows[img][SL],&tie_load.windows[img][SS],
        &tie_load.windows[img][NL],&tie_load.windows[img][NS],&access,&dtype,
        &zero,&one);

    if (status != E_SUCC)
        {
        return(TIE_F);
        }

   /*
    *  *** Determine the number of bytes/pixel based on the input data type.
    */
    switch(dtype)
        {
        case  EBYTE : wsize = 1;
            break;
        case  EWORD : wsize = 2;
            break;
        case  ELONG :
        case  EREAL : wsize = 4;
            break;
        }

   /*
    *  *** Allocate the input line buffer.
    */
    if ((linebuf = (unsigned char *)malloc(tie_load.windows[img][NS] * wsize)) 
        == NULL)
        {
        tie_message(Shell_Main, "Insufficient memory available");
        return(TIE_F);
        }
   /*
    *  *** Typecast appropriate pointers.
    */
    shortptr = (short *)linebuf;
    longptr = (long *)linebuf;
    floatptr = (float *)linebuf;

    for (bnd = 0; tie_load.bands[img][bnd] != 0; bnd++, tbnd++)
        {
        if (tbnd > 2)
           break;

       /*
        *  *** Read the DDR information for the current image.
        */
        if (loadflg)
            {
            status = load_projinfo(view+tbnd+1,tie_load.hnames[img]);
            if (status != TIE_S)
                {
                return(TIE_F);
                }
            }
       /*
        *  *** Set the LAS window and band fields of the memory plane structure.
        */
        if (loadflg)
            {
            memptr = &tie_planes[view+tbnd];
            memptr->window[SL] = tie_load.windows[img][SL]; 
            memptr->window[SS] = tie_load.windows[img][SS]; 
            memptr->window[NL] = tie_load.windows[img][NL]; 
            memptr->window[NS] = tie_load.windows[img][NS]; 
            memptr->band = tie_load.bands[img][bnd]; 
            strcpy(memptr->filename,tie_load.tnames[img]);
            strcpy(memptr->image_name,tie_load.image_name);
            }
        else
            {
            if (bulletinboard == fullref_win_bulletinboard)
               fresptr = &tie_ref_fullres[tbnd];
            else if (bulletinboard == fullsrch_win_bulletinboard)
               fresptr = &tie_srch_fullres[tbnd];
            }
/*
 *  Determine whether or not subsampling is required and the size of the 
 *  displayed image.
 */
        if (loadflg)
            {
            memptr->dspwin[SL] = memptr->dspwin[SS] = 1;
            if ((tie.flags & TIE_ADRG) &&
                (bulletinboard == ref_image_bulletinboard))
               {
               memptr->dspwin[NL] = (int)tie_load.windows[NL];
               memptr->dspwin[NS] = (int)tie_load.windows[NS];
               }
            else
               {
               memptr->dspwin[NL] = (int)mem->mem_height;
               memptr->dspwin[NS] = (int)mem->mem_width;
               }

            status = xid_samp(tie_load.windows[img],memptr->dspwin,
                &memptr->skipl,&memptr->skips);
            if (status != E_SUCC)
                {
                free(linebuf);
                return(TIE_F);
                }
            skipl = memptr->skipl;
            skips = memptr->skips;
            dspwin[SL] = memptr->dspwin[SL];
            dspwin[SS] = memptr->dspwin[SS];
            dspwin[NL] = memptr->dspwin[NL];
            dspwin[NS] = memptr->dspwin[NS];
            }
        else
            {
            fresptr->dspwin[SL] = fresptr->dspwin[SS] = 1;
            if ((bulletinboard == fullref_win_bulletinboard) &&
                (tie.flags && TIE_ADRG))
               {
               fresptr->dspwin[NL] = 512;
               fresptr->dspwin[NS] = 512;
               }
            else
               {
               fresptr->dspwin[NL] = tie_parms.fheight;
               fresptr->dspwin[NS] = tie_parms.fwidth;
               }

            status = xid_samp(tie_load.windows[img],fresptr->dspwin,
                &fresptr->skipl,&fresptr->skips);
            if (status != E_SUCC)
                {
                free(linebuf);
                return(TIE_F);
                }
            skipl = fresptr->skipl;
            skips = fresptr->skips;
            dspwin[SL] = fresptr->dspwin[SL];
            dspwin[SS] = fresptr->dspwin[SS];
            dspwin[NL] = fresptr->dspwin[NL];
            dspwin[NS] = fresptr->dspwin[NS];
            }

        if (dtype != EBYTE)
            {
/*
 *  Determine the min/max of the current band and calculate the slope and
 *  offset required to scale the input data to BYTE.
 */
            bands[0] = tie_load.bands[img][bnd];
            bands[1] = 0;
            status = c_minmax(tie_load.hnames[img],&one,tie_load.windows[img],
                bands,&one,&minval,&maxval);
            if (status != E_SUCC)
                {
                free(linebuf);
                return(TIE_F);
                }
            if (loadflg)
                {
                status = xid_slpoff(&dtype,&minval,&maxval,&memptr->slope,
                    &memptr->offset);
                slope = memptr->slope;
                offset = memptr->offset;
                }
            else
                {
                status = xid_slpoff(&dtype,&minval,&maxval,&fresptr->slope,
                    &fresptr->offset);
                slope = fresptr->slope;
                offset = fresptr->offset;
                }
            if (status != E_SUCC)
                {
                free(linebuf);
                return(TIE_F);
                }
            }
        else
            {
            if (loadflg)
                {
                memptr->slope = 1.0;
                memptr->offset = 0.0;
                }
            else
                {
                fresptr->slope = 1.0;
                fresptr->offset = 0.0;
                }
            }

/*
 *  Read each line of input data, update its associated histogram and scale
 *  it to BYTE if necessary.
 */
        if (loadflg)
            {
            dataptr = memptr->image_data;
            histptr = memptr->histogram;
            }
        else
            {
            dataptr = fresptr->image_data;
            histptr = fresptr->histogram;
            }

       /*
        *  *** Update the processing
        */
        sprintf(title,"Processing band %d ... ",bnd+1);
        XtSetArg(args[0], XeNlabelString, title);
        XtSetArg(args[1], XeNupdatePercent, 10);
        XtSetValues(process_widget, args, 2);
        XeProcessMInit(process_widget,0,dspwin[NL]);
        XeProcessMUpdate(process_widget,0);
        ForceDialog(process_dialog_shell);
        memset(dataptr, 0, dspwin[NL]*dspwin[NS]);

        switch (dtype)
            {
            case EBYTE:
                for (lin = 0, linnum = 1; lin < dspwin[NL];lin++, 
                    linnum += skipl)
                    {
                    status = c_eread(&fdesc,&tie_load.bands[img][bnd],&linnum,
                        linebuf,&one);
                    if (status != tie_load.windows[img][NS])  
                        {
                        free(linebuf);
                        return(TIE_F);
                        }

                   /*
                    *  *** update processing dialog, and check for cancel
                    */
                    if (XeProcessMUpdate(process_widget,lin) < 0)
                       break;

                    for (smp = 0; smp < tie_load.windows[img][NS]; smp += 
                        skips,dataptr++)
                        {
                        histptr[linebuf[smp]]++;
                        *dataptr = linebuf[smp];
                        }
                    }
                break;

            case EWORD:
                for (lin = 0, linnum = 1; lin < dspwin[NL];lin++, 
                    linnum += skipl)
                    {
                    status = c_eread(&fdesc,&tie_load.bands[img][bnd],&linnum,
                        linebuf,&one);
                    if (status != tie_load.windows[img][NS]) 
                        {
                        free(linebuf);
                        return(TIE_F);
                        }

                   /*
                    *  *** update processing dialog, and check for cancel
                    */
                    if (XeProcessMUpdate(process_widget,lin) < 0)
                       break;

                    for (smp = 0; smp < tie_load.windows[img][NS]; smp += 
                        skips,dataptr++)
                        {
                        *dataptr = (unsigned char) ((shortptr[smp] * 
                            slope + offset) + .5);
                        histptr[dataptr[smp]]++;
                        }
                    }
                break;

            case ELONG:
                for (lin = 0, linnum = 1; lin < dspwin[NL];lin++, 
                    linnum += skipl)
                    {
                    status = c_eread(&fdesc,&tie_load.bands[img][bnd],&linnum,
                        linebuf,&one);
                    if (status != tie_load.windows[img][NS]) 
                        {
                        free(linebuf);
                        return(TIE_F);
                        }

                   /*
                    *  *** update processing dialog, and check for cancel
                    */
                    if (XeProcessMUpdate(process_widget,lin) < 0)
                       break;

                    for (smp = 0; smp < tie_load.windows[img][NS]; smp += 
                        skips,dataptr++)
                        {
                        *dataptr = (unsigned char) ((longptr[smp] * 
                            slope + offset) + .5);
                        histptr[dataptr[smp]]++;
                        }
                    }
                break;

            case EREAL:
                for (lin = 0, linnum = 1; lin < dspwin[NL];lin++, 
                    linnum += skipl)
                    {
                    status = c_eread(&fdesc,&tie_load.bands[img][bnd],&linnum,
                        linebuf,&one);
                    if (status != tie_load.windows[img][NS]) 
                        {
                        free(linebuf);
                        return(TIE_F);
                        }

                   /*
                    *  *** update processing dialog, and check for cancel
                    */
                    if (XeProcessMUpdate(process_widget,lin) < 0)
                       break;

                    for (smp = 0; smp < tie_load.windows[img][NS]; smp += 
                        skips,dataptr++)
                        {
                        *dataptr = (unsigned char) ((floatptr[smp] * 
                            slope + offset) + .5);
                        histptr[dataptr[smp]]++;
                        }
                    }
                break;
            } 

   /*
    *  Calculate the associated histogram statistics -- min, max, mean and
    *  standard deviation.
    */
        if (loadflg)
            {
            xid_histcalc(memptr->histogram,&memptr->minval,&memptr->maxval,
                  &memptr->mean,&memptr->stdev);
            tie_planes[view+tbnd].flags = 
                  tie_planes[view+tbnd].flags | USED;
            }
        else
            xid_histcalc(fresptr->histogram,&fresptr->minval,&fresptr->maxval,
                &fresptr->mean,&fresptr->stdev);
        }
    status = c_eclose(&fdesc);
    if (status != E_SUCC)
        {
        free(linebuf);
        return(TIE_F);
        }
    }

/*
 *  Load the image into the image window.
 */
for (i = 0; i < tie_load.totbnd; i++)
   rgbmem[i] = view + plane1 + i + 1;

if ((tie_load.totbnd < 3) && loadflg)
    status = load_display(rgbmem,&tie_load.totbnd,XtWindow(bulletinboard),
        shell, bulletinboard, &tie_planes[rgbmem[0] - 1].dspwin[NL],
        &tie_planes[rgbmem[RED] - 1].dspwin[NS],&tie_planes[rgbmem[RED] - 1].
        maxval,tie_planes[rgbmem[RED] - 1].image_data,NULL,NULL,TRUE,
        &mem->image_pixmap,TRUE);
else if (loadflg)
    status = load_display(rgbmem,&tie_load.totbnd,XtWindow(bulletinboard),
        shell, bulletinboard,&tie_planes[rgbmem[RED] - 1].dspwin[NL],
        &tie_planes[rgbmem[RED] - 1].dspwin[NS],&tie_planes[rgbmem[RED] - 1].
        maxval,tie_planes[rgbmem[RED] - 1].image_data,tie_planes[rgbmem[GREEN]
        - 1].image_data,tie_planes[rgbmem[BLUE] - 1].image_data,TRUE,
        &mem->image_pixmap,TRUE);
else if ((tie_load.totbnd < 3) && !loadflg)
    {
    if (bulletinboard == fullref_win_bulletinboard)
       status = load_display(rgbmem,&tie_load.totbnd,XtWindow(bulletinboard),
           shell, bulletinboard,&tie_ref_fullres[0].dspwin[NL],
           &tie_ref_fullres[0].dspwin[NS],&tie_ref_fullres[0].maxval,
           tie_ref_fullres[0].image_data,NULL,NULL,FALSE,
           &tie_ref_fullres[0].image_pixmap, FALSE);
    else if (bulletinboard == fullsrch_win_bulletinboard)
       status = load_display(rgbmem,&tie_load.totbnd,XtWindow(bulletinboard),
           shell, bulletinboard,&tie_srch_fullres[0].dspwin[NL],
           &tie_srch_fullres[0].dspwin[NS],&tie_srch_fullres[0].maxval,
           tie_srch_fullres[0].image_data,NULL,NULL,FALSE,
           &tie_srch_fullres[0].image_pixmap, FALSE);
    }
else if (!loadflg)
    {
    if (bulletinboard == fullref_win_bulletinboard)
       status = load_display(rgbmem,&tie_load.totbnd,XtWindow(bulletinboard),
           shell, bulletinboard,&tie_ref_fullres[0].dspwin[NL],
           &tie_ref_fullres[0].dspwin[NS],&tie_ref_fullres[0].maxval,
           tie_ref_fullres[RED].image_data,tie_ref_fullres[GREEN].image_data,
           tie_ref_fullres[BLUE].image_data,FALSE,
           &tie_ref_fullres[0].image_pixmap,FALSE);
    else
       status = load_display(rgbmem,&tie_load.totbnd,XtWindow(bulletinboard),
           shell, bulletinboard,&tie_srch_fullres[0].dspwin[NL],
           &tie_srch_fullres[0].dspwin[NS],&tie_srch_fullres[0].maxval,
           tie_srch_fullres[RED].image_data,tie_srch_fullres[GREEN].image_data,
           tie_srch_fullres[BLUE].image_data,FALSE,
           &tie_srch_fullres[0].image_pixmap,FALSE);
    }
/*
 *  *** Free the appropriate resources.
 */
free(rgbmem);
free(linebuf);

return(status);
}


/*******************************************************************************
Purpose:
    This routine copies selected information from the LAS DDR into the
    TIE TIE_IMGINFO structure.  See the in-line comments for a list of the
    fields copied.
    
By L. Huewe, USGS/EROS Data Center, 11/91

Algorithm Description:
*******************************************************************************/
int load_projinfo
(
    int  memory,          /* Memory plane for which projection infor-
                             mation is being read (I).                */
    char *hostname        /* Host name of the LAS image to read DDR 
                             information from (I).                    */
)
{
    /* initialize the projection invalid */
    tie_planes[memory-1].proj_valid = FALSE;

    /*
     *  *** If the image name has changed, retrieve the new DDR information.
     */
    if (c_getddr(hostname, &tie_planes[memory-1].ddr) != E_SUCC)
        return(E_FAIL);

    /*
     *  *** Determine whether the DDR contains valid projection information.
     */
    if ((tie_planes[memory-1].ddr.valid[DDPCV] != 1) || 
        (tie_planes[memory-1].ddr.valid[DDZCV] != 1) || 
        (tie_planes[memory-1].ddr.valid[DDDCV] != 1) || 
        (tie_planes[memory-1].ddr.valid[DDPPV] != 1) ||
        (tie_planes[memory-1].ddr.valid[DDCCV] != 1) ||
        (tie_planes[memory-1].ddr.valid[DDPUV] != 1))
    {
        tie_planes[memory-1].proj_valid = FALSE;
    }
    else
    {
        /* cache the units */
        if (c_ckunit(tie_planes[memory-1].ddr.proj_units, 
                     &tie_planes[memory-1].units) != E_SUCC)
        {
            return (E_FAIL);
        }
        
        tie_planes[memory-1].proj_valid = TRUE;
    }
    
    return(E_SUCC);
}
