/*
 *  This is the main function source file for the main panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *  Modified by J. LaVergne, USGS/EROS Data Center, 7/93
 *    - Changed so that (on Select) when entry mode is manual, the cursor goes 
 *      into the Manual window rather than the Search or Reference windows.  
 *    - IM2MAP - changed so the 'Show Maps' button is not activated by the 
 *      Select button when in manual mode.
 *    - Changed so that the Load button is deactivated after the Select button
 *      or the Parameters button is pressed. 
 *  Modified by T Rockvam, USGS/EROS Data Center, 3/94.
 *    - added MB3 event handler for im2vect in search image window (create
 *      fullres window and load appropriate vectors)
 *  Modified by C. Olsen, USGS/EROS Data Center, 8/98
 *    - Modified main_select() to call tps_update() in IM2IM mode. (SRF #1935)
 *  Modified by C. Olsen, USGS/EROS Data Center, 9/98
 *    - Modified main_select() to set coordinate type buttons to insensitive
 *      after entering select mode.  (SRF #1958)
 *  Modified by A. Shimitz, USGS/EROS Data Center, 12/00
 *    - Modified main_help() to use the existing netscape window, if one exists,
 *      using the "-remote" option. (CR #4159)
 */
#include "tiepts.h"
#include <X11/cursorfont.h>
#include <signal.h>
#include <sys/wait.h>
#include "tpssup.h"
#include "adrgsup.h"
#include "cr.h"
#include "manage_cursor.h"
#include "protos.h"

void main_about
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    About_bulletinboard = Createabout_bulletinboard(Shell_About);
    XtManageChild(About_bulletinboard);
    XtRealizeWidget(Shell_About);
    XtSetSensitive(main_about_button,False);
}

void main_register
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XtManageChild(Mapparm_form);
    XtRealizeWidget(Shell_MParm);
    XtSetSensitive(main_register_button,False);
    if (register_dialog_form == NULL)
        register_dialog_form  = Createregister_form(Shell_Main); 

    register_setup();
}

void main_maplist
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Cursor  pointer;

    if (tie_rt.calc_rt == RT_SEL)
    {
        tie_message(Shell_Main, 
                    "Must have rough transformation to outline maps");
        return;
    }

    pointer = XCreateFontCursor(tie.display, XC_crosshair);
    set_window_cursor(tie.display,XtWindow(search_image_bulletinboard),pointer);

    XtSetSensitive(main_map_button, False);
    Getmap_form = Createmaplist_form(Shell_Getmap, tie_select.mapcolor);
    XmTextSetString(maplist_catalog_text, tie_mapparm.catalog);
    XmTextSetString(maplist_file_text, tie_mapparm.mapfile);
    XtManageChild(Getmap_form);
    XtRealizeWidget(Shell_Getmap);

    XtRemoveEventHandler(search_image_bulletinboard,ButtonReleaseMask,FALSE,
            fullres_button,NULL);
    XtAddEventHandler(search_image_bulletinboard,ButtonReleaseMask,FALSE,
            maplist_select,NULL);

    tie.flags ^= TIE_RECTCUR;

    if (strcmp(tie_mapparm.catalog, "") != 0)
        maplist_define();
}

void main_stats
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Arg     args[32];              /* X resource arguments.                */
    int     argcnt;

    if (tie_rt.calc_rt == RT_SEL)
        return;

    XtSetSensitive(main_stats_button,False);

    if (Shell_Stats == NULL)
    {
        /*
         *  *** Create the stats panel shell.
         */
        argcnt = 0;
        XtSetArg(args[argcnt], XmNtitle, "Tie Points - VERIFY Statistics"); 
        argcnt++;
        XtSetArg(args[argcnt], XmNiconName, "Verify"); argcnt++;
        XtSetArg(args[argcnt], XmNgeometry, "-0-0"); argcnt++;
        XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
        XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
        Shell_Stats = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
                topLevelShellWidgetClass, tie.display, args, argcnt);
        XtAddCallback(Shell_Stats, XmNdestroyCallback, stats_destroy, 0);
        Stats_form = Createstats_form(Shell_Stats);
        XtManageChild(Stats_form);
        XtRealizeWidget(Shell_Stats);
    }
    else
        XtRealizeWidget(Shell_Stats);

    stats_display(TRUE);
}

void main_align
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    if (!XtIsRealized(Shell_Zoom))
        return;

    XtManageChild(Align_form);
    XtRealizeWidget(Shell_Align);

    XtRemoveEventHandler(fullsrch_win_bulletinboard,
            PointerMotionMask|KeyReleaseMask,
            FALSE,cursor_pointer,NULL);

    XtRemoveEventHandler(zoom_srchwin_bulletinboard,
            KeyReleaseMask|ButtonPressMask|ButtonReleaseMask,
            FALSE,cursor_select,NULL);

    XtRemoveEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask,FALSE,
            zoom_button,NULL);

    XtAddEventHandler(fullsrch_win_bulletinboard,
            KeyReleaseMask|ButtonPressMask|ButtonReleaseMask,
            FALSE,align_select,NULL);

    XtAddEventHandler(zoom_srchwin_bulletinboard,
            KeyReleaseMask|ButtonPressMask|ButtonReleaseMask,
            FALSE,align_select,NULL);

    tie_vector->align = TRUE; 
    tie_vector->blend = 0;
    tie_vector->blend_mask = NULL;

    if (!XtIsManaged(Align_form))
        XtManageChild(Align_form);
    XtRealizeWidget(Shell_Align);
    XtSetSensitive(main_align_button,False);

    XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard),
            0, 0, 1, 1, True);
    tie.flags |= TIE_ALIGNING;

    XmScaleSetValue(vect_blend_slide, 0);
}

void main_vector
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Cursor cursor;

    if (Vector_form == NULL)
    {
        if (tie_parms.vsource == DLGOPT)
            Vector_form = Createdlgvect_form(Shell_Vector);
        else if (tie_parms.vsource == WDB)
        {
            Vector_form = Createwdbvect_form(Shell_Vector);
            vector_wdbattribs();
        }
        else if (tie_parms.vsource == DCW)
        {
            Vector_form = Createdcwvect_form(Shell_Vector);
        }
    }

    if (!XtIsManaged(Vector_form))
    {
        XtManageChild(Vector_form);
    }

    /*
     *  *** If image was loaded, set bounds on panel
     */
    if (tie_mem[SEARCH].image_pixmap != NULL)
        vector_setbounds();

    XtRealizeWidget(Shell_Vector);
    XtSetSensitive(main_vector_button,False);

    /* set the cursor for the vector form */
    cursor = XCreateFontCursor(tie.display,XC_top_left_arrow);
    set_window_cursor(tie.display,XtWindow(Shell_Vector),cursor);
}

void main_cursor
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Cursor_bulletinboard = Createcursor_bulletinboard(Shell_Cursor);
    XtManageChild(Cursor_bulletinboard);
    XtRealizeWidget(Shell_Cursor);
    XtSetSensitive(main_cursor_button,False);
    /****
      XtSetSensitive(cursor_scale,False);
     ***/
}

void main_destroy
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    int          i;

    for (i = 0; i < tie_layout.mem_depth; i++)
        if (tie_planes[i].image_data != NULL)
            free(tie_planes[i].image_data);

    if (tie_adjust.color_adjust != NULL)
    {
        free(tie_adjust.color_adjust);
        tie_adjust.color_adjust = NULL;
    }

    /*
     *  *** Free the appropriate resources before exiting the application.
     */
    if (tie_mem[REFERENCE].imgpix)
        XFreePixmap(tie.display,tie_mem[REFERENCE].image_pixmap);
    if (tie_mem[SEARCH].imgpix)
        XFreePixmap(tie.display,tie_mem[SEARCH].image_pixmap);
    if (tie_zoom[REFERENCE].image_pixmap != NULL)
        XFreePixmap(tie.display,tie_zoom[REFERENCE].image_pixmap);
    if (tie_zoom[SEARCH].image_pixmap != NULL)
        XFreePixmap(tie.display,tie_zoom[SEARCH].image_pixmap);

    if (tie_zoom[REFERENCE].image_data != NULL)
        free(tie_zoom[REFERENCE].image_data);
    if (tie_zoom[SEARCH].image_data != NULL)
        free(tie_zoom[SEARCH].image_data);


    if (tie_zoom[REFERENCE].parms.horiz_table != NULL)
    {
        free(tie_zoom[REFERENCE].parms.horiz_table);
        free(tie_zoom[REFERENCE].parms.input);
        free(tie_zoom[REFERENCE].parms.output);
        free(tie_zoom[REFERENCE].parms.work);
    }

    if (tie_zoom[SEARCH].parms.horiz_table != NULL)
    {
        free(tie_zoom[SEARCH].parms.horiz_table);
        free(tie_zoom[SEARCH].parms.input);
        free(tie_zoom[SEARCH].parms.output);
        free(tie_zoom[SEARCH].parms.work);
    }

    if ((tie.mode == IM2IM) && (tie_ref_fullres != NULL))
    {
        for (i=0; i < tie.num_ref_fullres; i++)
            if (tie_ref_fullres[i].image_data != NULL)
                free(tie_ref_fullres[i].image_data);

        free(tie_ref_fullres);
    }

    if (tie_srch_fullres != NULL)
    {
        for (i=0; i < tie.num_srch_fullres; i++)
            if (tie_srch_fullres[i].image_data != NULL)
                free(tie_srch_fullres[i].image_data);

        free(tie_srch_fullres);
    }


    XFreePixmap(tie.display, tie_mem[SEARCH].graphics_pixmap);

    if (tie.mode == IM2IM)
    {
        if (tie.flags & TIE_ADRG)
        {
            adrg_free();
            free(tie_adrg);
        }

        XFreePixmap(tie.display, tie_mem[REFERENCE].graphics_pixmap);
        if (tie_adjust.full_lut[REFERENCE] != NULL)
            free(tie_adjust.full_lut[REFERENCE]);
        if (tie_adjust.full_lut[SEARCH] != NULL)
            free(tie_adjust.full_lut[SEARCH]);
    }

    XFreeGC(tie.display, tie.bit_gc);
    XFreeGC(tie.display, tie_select.gc);

    if (tie_cursor.pointer_pixmap != NULL)
    {
        XFreePixmap(tie.display,tie_cursor.pointer_pixmap);
        XFreeGC(tie.display,tie_cursor.pointer_gc);
    }


    if (tie.flags & TIE_RECTCUR) 
        if (tie.rect_gc != NULL)
            XFreeGC(tie.display,tie.rect_gc);

    /*
     *  *** free dynamic structure for vector mode
     */
    if (tie.mode == IM2VECT)
    {
        if (tie_vector->numlines > 0) 
            vector_free();

        if (tie_vector->nattrib > 0)
            vector_freeattrib();

        i=0;
        while (dcw_coverage[i].desc != NULL)
        {
            if (dcw_coverage[i].numvdt != 0)
            {
                free(dcw_coverage[i].vdt);
                dcw_coverage[i].numvdt = 0;
            }
            if (dcw_coverage[i].numstatus != 0)
            {
                free(dcw_coverage[i].status);
                dcw_coverage[i].numstatus = 0;
            }

            i++;
        }

        for (i = 0; i < NUM_CVALS; i++)
        {
            if (tie_vector->pixmap[i] != NULL)
                XFreePixmap(tie.display, tie_vector->pixmap[i]);
            tie_vector->pixmap[i] = NULL;
        }


        free(tie_vector->line);
        free(tie_vector);
        free(tie_vectfile);
    }

    free(tie_planes);
    free(tie_composite);

    XtUnmanageChild(main_form); 
    XtUnrealizeWidget(Shell_Main);
    XtDestroyWidget(Shell_Main);

    if (XtIsRealized(Shell_RefImg)) 
    {
        XtUnmanageChild(ref_image_bulletinboard); 
        remove_window_cursor(tie.display,XtWindow(Shell_RefImg));
        XtUnrealizeWidget(Shell_RefImg);
        XtDestroyWidget(Shell_RefImg);
    }

    if (XtIsRealized(Shell_SrchImg)) 
    {
        XtUnmanageChild(search_image_bulletinboard); 
        remove_window_cursor(tie.display,XtWindow(Shell_SrchImg));
        XtUnrealizeWidget(Shell_SrchImg);
        XtDestroyWidget(Shell_SrchImg);
    }

    if (XtIsRealized(Shell_RefFull)) 
    {
        XtUnmanageChild(fullref_win_bulletinboard); 
        XtUnrealizeWidget(Shell_RefFull);
        XtDestroyWidget(Shell_RefFull);
    }

    if (XtIsRealized(Shell_SrchFull)) 
    {
        XtUnmanageChild(fullsrch_win_bulletinboard); 
        XtUnrealizeWidget(Shell_SrchFull);
        XtDestroyWidget(Shell_SrchFull);
    }

    if (XtIsRealized(Shell_Parm)) 
    {
        XtUnmanageChild(Parm_bulletinboard); 
        XtUnrealizeWidget(Shell_Parm);
        XtDestroyWidget(Shell_Parm);
    }

    if (tie.mode  == IM2MAP)
    {
        XtDestroyWidget(Shell_MParm);
        XtDestroyWidget(Shell_RefImg);

        if (tie_mapparm.digid != NULL)
        {
            XtRemoveInput(tie_mapparm.digid);
            close(tie_mapparm.portfd);
        }

        /*
         * Check the datums of the maps in the ref tps file  
         * Default code is "Mixed";
         * if they are all the same reset the tps header
         */
        if ((!tie_parms.manual) && (tie_select.ref_desc != NULL) && 
                (tie_select.srch_desc != NULL))
            tps_checkdatum();
    }

    if (tie_select.ref_desc != NULL)
        c_ptclse(&tie_select.ref_desc);
    if (tie_select.srch_desc != NULL)
        c_ptclse(&tie_select.srch_desc);

    XFreeFont(tie.display, tie_select.font_info);

    /*
       Kill the Mosaic process if one is running.
     */
    if (tie.help_pid != -1)
    {
        kill(tie.help_pid, SIGTERM);
    }
    exit_tiepts(0);
}

void main_help
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    char    *home_dir,      /* Temporary file name.                           */
            *help_dir,      /* Directory containing help files.               */
            help_file[100], /* Help file directory and name.                  */
            home_file[100], /* Home document directory and name.              */
            lock_file[100], /* Full path to location of the netscape lock file*/
            netscape_exe[CMLEN];  /* Mosaic path and executable.              */

    /*
       Get the value of the environment variable LASDOC in order to build
       the directory location of the help files.
     */
    help_dir = getenv("LASDOC");
    if (help_dir == '\0')
    {
        tie_message(Shell_Main, "Error retrieving help file directory");
        return;
    }

    /*
       Get the value of the environment variable HOME in order to determine
       if there is a netscape lock file in existance, which tells us if the
       user already has an active netscape process.
     */
    home_dir = getenv("HOME");
    if (home_dir == '\0')
    {
        tie_message(Shell_Main, "Error retrieving home directory");
        return;
    }

    strcpy(help_file,help_dir);
    strcpy(home_file,help_dir);
    strcat(help_file,"/tiepts/");
    strcat(home_file,"/tiepts/");
    if (client == NULL)
        strcat(help_file,"introduction.html");
    else
        strcat(help_file,(char*)client);
    strcat(home_file,"introduction.html");

    /*
       exec a Netscape process if one is not currently active.
     */
    sprintf(lock_file, "%s%s", home_dir, "/.netscape/lock");
    if (readlink(lock_file, (char *)NULL, 0) == -1)
    {
        /* lock file does not exist */
        tie.help_pid = fork();

        if (tie.help_pid == 0)
        {
            execlp("netscape", "netscape", help_file, (char *)NULL);  
            /* cast to (char *) REQUIRED */
            perror("netscape");
            exit(1);
        }
    }
    /*
       Otherwise, just load the page in the netscape window already active.
     */
    else
    {
        /* lock file does exist */
        sprintf(netscape_exe, "netscape -raise -remote 'openFile(%s)'", 
                help_file);
        if(system(netscape_exe) == -1)
            tie_message(Shell_Main,"Error communicating with netscape process");
    }
}

void main_select
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    int         rplane_idx,             /* memory plane of reference image */
                splane_idx,             /* memory plane of search image    */
                status,                 /* status indicator                */
                index;                  /* variable used for looping       */

    /* If not in Multiple Reference mode,
       set Load button to insensitive
       -------------------------------------*/
    if (!(tie_parms.flags & PARMS_MULTI) && (tie.mode == IM2IM))
        XtSetSensitive(main_load_button,False);

    if (tie.mode == IM2IM)
        rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;

    splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

    /*
     *  *** if either output TPS file is undefined, return
     */
    if ((tie.mode != ONESRCH) && strcmp(tie_parms.outtps[REFERENCE], "") == 0)
    {
        tie_message(Shell_Main, "Must define reference output TPS filename");
        return;
    }
    else if (strcmp(tie_parms.outtps[SEARCH], "") == 0)
    {
        tie_message(Shell_Main, "Must define search output TPS filename");
        return;
    }

    tie.flags |= TIE_SELECTING;
    tie.graphics = REFIMAGE_GRAPHICS | SRCHIMAGE_GRAPHICS;

    /*
     *  *** if images not loaded
     */
    if ((tie.mode == IM2IM) &&
            ((tie_planes[tie.sview].image_data == NULL) ||
             (tie_planes[tie.rview].image_data == NULL)))
    {
        /*
         *  *** if input TPS defined, load images from headers
         */
        if ((strcmp(tie_parms.intps[REFERENCE], "")  == 0) ||
                (strcmp(tie_parms.intps[SEARCH], "")     == 0))
        {
            tie_message(Shell_Main, 
                    "Must load ref and search images or set TPS filenames");
            tie.flags ^= TIE_SELECTING;
            return;
        }
        else
        {
            if (tps_loadimage(ref_image_bulletinboard, REFERENCE) != E_SUCC)
            {
                tie_message(Shell_Main, "Error loading reference image");
                tie.flags ^= TIE_SELECTING;
                return;
            }
            if (tps_loadimage(search_image_bulletinboard, SEARCH) != E_SUCC)
            {
                tie_message(Shell_Main, "Error loading search image");
                tie.flags ^= TIE_SELECTING;
                return;
            }
            rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;
            splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
        }
    }
    else if ((tie.mode != IM2IM) &&
            (tie_planes[tie.sview].image_data == NULL))
    {
        if (strcmp(tie_parms.intps[SEARCH], "") == 0)
        {
            tie_message(Shell_Main, 
                    "Must load search images or set search TPS filenames");
            tie.flags ^= TIE_SELECTING;
            return;
        }
        else if (tps_loadimage(search_image_bulletinboard, SEARCH) != E_SUCC)
        {
            tie.flags ^= TIE_SELECTING;
            return;
        }
        splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
    }

    /*
     *  *** create and display fullres windows
     */
    if (tie.mode == IM2IM)
        Createfullres_scrollwin(Shell_RefFull, &Fullref_scrollwin,
                &fullref_win_bulletinboard);

    Createfullres_scrollwin(Shell_SrchFull, &Fullsrch_scrollwin,
            &fullsrch_win_bulletinboard);

    fullres_create();

    /*
     *  *** create zoom window
     */
    Zoom_scrollwin = Createzoom_scrollwin(Shell_Zoom);
    zoom_create();

    /*
     *  *** create verify window
     */
    verify_dialog_form = Createverify_form(Shell_Zoom); 

    /*
     *  *** create edit window
     */
    Edit_form  = Createedit_form(Shell_Edit); 

    /*
     *  *** Open the TPS files; return if an error occurs.
     */
    if (tie.mode == IM2VECT)
    {
        /* Since rplane_idx has not been set yet when the mode is IM2VECT 
           then initialize it to 0 */
           rplane_idx = 0;

        /* Update the reference tiepoints file with the reference projection
           information from the first DLG file. This is for DLG files only */
        if (tie_parms.vsource == DLGOPT)
        {
            if (c_unit2string(tie_vectfile[0].inunit,
                        tie_planes[rplane_idx].ddr.proj_units) == E_FAIL)
            {
                /* Error message is returned from c_unit2string */
                printf("Error retrieving projection units\n");
                exit(1);
            }
            tie_planes[rplane_idx].ddr.proj_code = tie_vectfile[0].inproj;
            tie_planes[rplane_idx].ddr.zone_code = tie_vectfile[0].inzone;
            tie_planes[rplane_idx].ddr.datum_code = tie_vectfile[0].indatum;
            for (index = 0; index < 15; index++)
            {
                tie_planes[rplane_idx].ddr.proj_coef[index] =
                        tie_vectfile[0].inparm[index];
            }
        }
        status = tps_open(tie_planes[rplane_idx].ddr.proj_units,
                          &tie_planes[rplane_idx].ddr);
    }
    else if (tie.mode == IM2IM)
        status = tps_open("", &tie_planes[rplane_idx].ddr);
    else 
        status = tps_open("", &tie_planes[splane_idx].ddr);
    if (status != E_SUCC)
    {
        tie.flags &= ~TIE_SELECTING;
        return;
    }

    /*
     *  *** initialize point generation identifier
     */
    tie_select.genpt = 1;
    tie_select.curref = FALSE;

    tie.flags |= TIE_RECTCUR;
    tie.refrect_x = -1;
    tie.refrect_y = -1;
    tie.srchrect_x = -1;
    tie.srchrect_y = -1;
    XtSetSensitive(main_graphic_button,True);

    if (tie.mode == IM2VECT)
    {
        zoom_dotcursor(ref_image_bulletinboard);
        tie_vector->pressx1 = -1;
        tie_vector->pressy1 = -1;
        XtSetSensitive(main_align_button, True);

        if (tie_vector->numfiles != 0)
            vector_displaypts(); 
    }

    XtAddEventHandler(search_image_bulletinboard, FocusChangeMask,
            FALSE, cursor_mark,NULL);
    XtAddEventHandler(search_image_bulletinboard,ExposureMask,FALSE,
            graphic_expose, NULL);
    XtAddEventHandler(fullsrch_win_bulletinboard,ExposureMask,FALSE,
            graphic_full_expose, NULL);

    /*
     *  *** define event handlers for reference images
     */
    if (tie.mode == IM2IM)
    {
        if (!(tie.flags & TIE_ADRG))
            XtAddEventHandler(zoom_refwin_bulletinboard,ExposureMask,FALSE,
                    graphic_zoom_expose, NULL);

        XtAddEventHandler(ref_image_bulletinboard, FocusChangeMask,
                FALSE, cursor_mark,NULL);
        XtAddEventHandler(ref_image_bulletinboard,ExposureMask,FALSE,
                graphic_expose, NULL);
        XtAddEventHandler(fullref_win_bulletinboard,ExposureMask,FALSE,
                graphic_full_expose, NULL);

        XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,FALSE,
                fullres_button,NULL);

        if (tie.flags & TIE_ADRG)
            XtAddEventHandler(fullref_win_bulletinboard, ButtonReleaseMask,
                    FALSE, cursor_select, NULL);
        /*
         * *** update the tps files
         */
        if (tie_rt.calc_rt != RT_SEL)
            tps_update();

        XtSetSensitive(parm_ls_button, False);
        XtSetSensitive(parm_proj_button, False);
        XtSetSensitive(parm_latlong_button, False);
        XtSetSensitive(parm_multiref_button, False);
        if (tie_parms.flags & PARMS_MULTI)
            XtSetSensitive(load_search,False);
    }
    else if (tie.mode == IM2MAP)
    {
        if (!tie_parms.manual) XtSetSensitive(main_map_button, True);
            XtSetSensitive(main_select_button, True);  

        /*
         *  *** digitizer event handler
         */
        XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,FALSE,
                cursor_select, NULL);

        if (!tie_parms.manual && tie_mapparm.registered)
            XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,FALSE,
                    fullres_button,NULL);
        else if (tie_parms.manual)
            manual_display(w, client, call_data);
    }
    else if (tie.mode == IM2VECT)
    {
        XtAddEventHandler(zoom_srchwin_bulletinboard,ExposureMask,FALSE,
                vector_zexpose, NULL);

        XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,FALSE,
                fullres_button,NULL);

        XtAddEventHandler(search_image_bulletinboard,
                ButtonPressMask|ButtonReleaseMask,FALSE,
                vector_region,NULL);

        XtAddEventHandler(search_image_bulletinboard,ButtonReleaseMask,FALSE,
                vector_window,NULL);

        XtSetSensitive(parm_oc_proj, False);
        XtSetSensitive(parm_oc_latlong, False);
    }

    XtAddEventHandler(search_image_bulletinboard,ButtonReleaseMask,FALSE,
            fullres_button,NULL);

    /*
     *  *** manage image bulletinboards
     */
    if (Fullref_scrollwin != NULL)
        XtManageChild(Fullref_scrollwin);

    XtManageChild(Fullsrch_scrollwin);
    XtManageChild(Zoom_scrollwin);

    XtSetSensitive(main_select_button,False); 
    if (tie.mode != ONESRCH) 
    {
        XtSetSensitive(main_stats_button,True);
        XtSetSensitive(main_edit_button,True);
    }
    /*
     *   ***  place the pointer in the proper window:
     *        IM2IM, IM2ADRG, IM2VEC - move to the Reference window
     *        IM2MAP (digitizer mode) - stay in the main selection window 
     *        IM2MAP (manual mode) - move to the manual entry window
     *        ONESRCH - move to the Search window
     */
    if ((tie.mode == IM2IM) || (tie.mode == IM2VECT))
        XWarpPointer(tie.display, None, XtWindow(Shell_RefImg),
                None, None, None, None,
                tie_mem[REFERENCE].view_width/2, 
                tie_mem[REFERENCE].view_height/2);
    else if ((tie.mode == IM2MAP) && (tie_parms.manual))
        XWarpPointer(tie.display, None, XtWindow(Shell_Manual),
                None, None, None, None,150,100);
    else if (tie.mode == ONESRCH)   
        XWarpPointer(tie.display, None, XtWindow(Shell_SrchImg),
                None, None, None, None,
                tie_mem[SEARCH].view_width/2, 
                tie_mem[SEARCH].view_height/2);

}

void image_destroy
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    main_destroy(w, client, call_data);
}

void main_graphic
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    if (Graphic_form == NULL)
        Graphic_form = Creategraphic_form(Shell_Graphic);
    XtManageChild(Graphic_form);
    XtRealizeWidget(Shell_Graphic);
    XtSetSensitive(main_graphic_button,False);
}

void main_loop
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XEvent event;
    Pixmap img1, img2;
    Window imgwin;
    int    i;

    imgwin = XtWindow(ref_image_bulletinboard);
    img1 = XCreatePixmap(tie.display,imgwin,
            tie_mem[REFERENCE].view_width, tie_mem[REFERENCE].view_height,
            tie.visual_info[0].depth);
    XCopyArea(tie.display, tie_mem[REFERENCE].image_pixmap,img1,
            DefaultGC(tie.display, tie.screen), 0, 0,
            tie_mem[REFERENCE].view_width, tie_mem[REFERENCE].view_height,
            0, 0);
    img2 = XCreatePixmap(tie.display,imgwin,
            tie_mem[REFERENCE].view_width, tie_mem[REFERENCE].view_height,
            tie.visual_info[0].depth);
    XCopyArea(tie.display, tie_mem[SEARCH].image_pixmap,img2,
            DefaultGC(tie.display, tie.screen), 0, 0,
            tie_mem[REFERENCE].view_width, tie_mem[REFERENCE].view_height,
            0, 0);

    for(i = 0; i < 20; i++)
    {
        XSetWindowBackgroundPixmap(tie.display,imgwin,img1);
        /*****
          XCopyArea(tie.display, img2, tie_mem[REFERENCE].image_pixmap,
          DefaultGC(tie.display, tie.screen), 0, 0,
          tie_mem[REFERENCE].view_width, tie_mem[REFERENCE].view_height,
          0, 0);
         *****/
        XClearWindow(tie.display, imgwin);
        while(XPending(tie.display))
            XNextEvent(tie.display, &event);
        sleep(1);

        XSetWindowBackgroundPixmap(tie.display,imgwin,img2);
        /*****
          XCopyArea(tie.display, img1, tie_mem[REFERENCE].image_pixmap,
          DefaultGC(tie.display, tie.screen), 0, 0,
          tie_mem[REFERENCE].view_width, tie_mem[REFERENCE].view_height,
          0, 0);
         *****/
        XClearWindow(tie.display, imgwin);
        while(XPending(tie.display))
            XNextEvent(tie.display, &event);
        sleep(1);
    }

    XSetWindowBackgroundPixmap(tie.display,imgwin,
            tie_mem[REFERENCE].image_pixmap);
    XFreePixmap(tie.display, img1);
    XFreePixmap(tie.display, img2);
}

void main_load
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    if (((int)client == REFERENCE) && (tie.flags & TIE_ADRG))
        Load_fileselector = Createload_fileselector("adrg_fileselector",
                Shell_Load, client);
    else
        Load_fileselector = Createload_fileselector("load_fileselector",
                Shell_Load, client);
    XtManageChild(Load_fileselector);
    XtRealizeWidget(Shell_Load);
    XtSetSensitive(main_load_button,False);
}

void main_adjust
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Arg     args[256];              /* X resource arguments.                */
    int     argcnt;                 /* number of resource arguments         */

    if ((int)client == SEARCH)
    {
        if (tie_mem[SEARCH].image_pixmap == NULL)
            return;
        else
            tie_map.imgidx = SEARCH;
    }
    else
    {
        if (tie_mem[REFERENCE].image_pixmap == NULL)
            return;
        else
            tie_map.imgidx = REFERENCE;
    }

    if (Shell_Adjust == NULL)
    {
        /*
         *  Create the adjust panel shell.
         */
        argcnt = 0;
        XtSetArg(args[argcnt], XmNallowShellResize, False); argcnt++;
        XtSetArg(args[argcnt], XmNtitle, "Tie Points - Adjust"); argcnt++;
        XtSetArg(args[argcnt], XmNiconName, "Map-Adjust"); argcnt++;
        XtSetArg(args[argcnt], XmNx, 381); argcnt++;
        XtSetArg(args[argcnt], XmNy, 118); argcnt++;
        XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
        XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
        if (!tie.userplace)
        {
            XtSetArg(args[argcnt],XtNgeometry, "+0-0");
            argcnt++;
        }
        Shell_Adjust = XtAppCreateShell( BX_APP_NAME, BX_APP_CLASS,
                topLevelShellWidgetClass, tie.display, args, argcnt);
        XtAddCallback(Shell_Adjust,XmNdestroyCallback,adjust_destroy,"WMCLOSE");
    }

    tie_map.subpanel = Shell_Adjust;

    Adjust_bulletinboard = 
        Createadjust_bulletinboard(Shell_Adjust, tie_map.imgidx);
    /*
     *  ***  Reset the slider values.
     */
    XtVaSetValues(adjust_contrast_slider,XmNvalue,
            (int)tie_adjust.lut_slope,NULL);
    XtVaSetValues(adjust_bright_slider,XmNvalue,
            (int)(tie_adjust.lut_offset*(100.0/MAXCOLOR)),NULL);

    XtManageChild(Adjust_bulletinboard);
    XtRealizeWidget(Shell_Adjust);
    XtSetSensitive(main_adjust_button,False);
}

void main_edit
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    /*
     *  *** if rough trans. calculated, display residuals
     */
    if (tie_rt.calc_rt != RT_SEL)
    {
        XtManageChild(Edit_form);
        XtRealizeWidget(Shell_Edit);
        XtSetSensitive(main_edit_button,False);
        edit_display(TRUE);
    }
}

void main_show
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    int     image_idx;

    /*
     *  ***  reference image uses planes 1, 2, and 3
     */
    if ((int)client  == REFERENCE)
    {
        if (tie_mem[REFERENCE].image_pixmap == NULL)
            return;

        Show_form = Createshow_form(Shell_Show, ref_image_bulletinboard);
        image_idx = tie.rview;
    }
    /*
     *  ***  search uses planes 4, 5, and 6
     */
    else
    {
        if (tie_mem[SEARCH].image_pixmap == NULL)
            return;
        Show_form = Createshow_form(Shell_Show, search_image_bulletinboard);
        image_idx = tie.sview;
    }

    show_create(image_idx);
    XtManageChild(Show_form);
    XtRealizeWidget(Shell_Show);
    XtSetSensitive(main_show_button,False);
}

void main_parm
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XtManageChild(Parm_bulletinboard);
    XtRealizeWidget(Shell_Parm);
    XtSetSensitive(main_parm_button,False);
    XtSetSensitive(main_load_button,False);

    /*
     *  ***  set items on parm panel to their defined values
     */
    parm_setup();
}

void help_destroy
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XtUnrealizeWidget(Shell_Help);
    XtSetSensitive(main_help_button,True);
}

void about_destroy
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XtUnmanageChild(About_bulletinboard);
    XtDestroyWidget(About_bulletinboard);
    XtUnrealizeWidget(Shell_About);
    XtSetSensitive(main_about_button,True);
}

/*******************************************************************************
Purpose:
    This routine is triggered each time a child process is killed.  This allows
    TIEPTS to keep track of whether or not a new Mosaic process needs to be
    fired up when the user requests help or whether the current process can
    be updated.

By L. Huewe, USGS/EROS Data Center, 12/93

Algorithm Description:
*******************************************************************************/
void help_reset(void)
{
    int     child_pid = 0,          /* Child process ID.                      */
            status;                 /* Status indicator.                      */

    char    fname[25];              /* Temporary file name.                   */

    signal (SIGCLD,SIG_IGN);

    /*
       Remove temporary file.
     */
    sprintf(fname,"/tmp/Mosaic.%d",tie.help_pid);
    unlink(fname);

    /*
       Wait for the child process to die so that we don't have defunct processes
       hanging around.
     */
    if (tie.help_pid != -1)
    {
        child_pid = waitpid(tie.help_pid, &status, WNOHANG);
        if(child_pid != 0)
        {
            tie.help_pid = -1;
        }
    }

    /*
       Reset the signal handler.
     */
    signal(SIGCLD,help_reset);
}
