#include "tiepts.h"
#include "cr.h"
#include "mapsup.h"
#include "convert.h"
#include "protos.h"
#include "cproj.h"
#include "datum.h"

/*******************************************************************************
Purpose:
    This routine displays the map coordinate manual entry panel.

By J. Hemmer, USGS/EROS Data Center, 2/92
 Modified by J. LaVergne, USGS/EROS Data Center, 7/93
   - Corrected an error where user-input point coordinates (in DMS) were 
     converted to DD  but stored in a long variable.  This truncated the values      to whole degrees, which kept the transformation from being refined.
Algorithm Description:
*******************************************************************************/
void manual_display
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    if (Manual_form == NULL)
    {
        Manual_form = Createmanual_form(Shell_Manual); 
        XtManageChild(Manual_form);
    }

    if (!XtIsRealized(Shell_Manual))
        XtRealizeWidget(Shell_Manual);

    /*  Initialize tie_mapparm.units to DMS because it the first
     *  Coordinate Type option. DMS is assumed until another
     *  option is selected.
     */
    tie_mapparm.units = DMS; 
}


/*******************************************************************************
Purpose:
    This routine submits a manual entry coordinate from the user

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by John LaVergne, USGS/EROS Data Center 6/93
    -  changed to get real values into tie_select.rtie (they get
       truncated when they are put into tie_select.saveref)
Modified by John LaVergne, USGS/EROS Data Center 12/93
    -  added checks for formatting of user-entered coordinates

Algorithm Description:

NOTE:
    The output reference tps file will reference the first datum selected
    by the user and the Geographic projection.

*******************************************************************************/
void manual_spark
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XEvent  xe;
    int status;                 /* Function return status                  */
    int index;                  /* Value used for looping                  */
    double out_x;               /* Output X projection coordinate          */
    double out_y;               /* Output Y projection coordinate          */
    double current_proj_coef[15]; /* Input projection parameters           */
    double out_proj_coef[15];   /* Used to update the reference tps file   */
    long current_proj_code;     /* Input projection code                   */
    long current_proj_units;    /* Input projection units                  */   
    long current_zone_code;     /* Input zone code                         */
    long out_proj_code;         /* Used to update the reference tps file   */
    long out_proj_units;        /* Used to update the reference tps file   */
    long out_zone_code;         /* Used to update the reference tps file   */
    long prtprm[2];             /* Write out projection parms to a 
                                   file or terminal (TRUE or FALSE) 
                                   prtprm[GEO_TERM], prtprm[GEO_FILE]      */
    char major_minor_string[15];/* Temporary string value used to store
                                   the semi major and semi minor axis      */
    char *manual_x_ptr;         /* Pointer to Lat/X                        */
    char *manual_y_ptr;         /* Pointer to Lon/Y                        */
    char *temp_ptr;             /* Pointer used for XmTextGetString calls  */

    /* 
     * Initialize current zone, projection code and projection parameters
     * to Geographic. These values will be updated if the user decided to
     * enter the points in using projection.
     */
    current_zone_code = 0;
    current_proj_code = GEO;
    for (index = 0; index < 15; index ++)
        current_proj_coef[index] = 0.0;

    /*
     * Make sure the user enters the Lat/Y and Lon/X
     */
    manual_x_ptr = XmTextGetString(manual_x);
    manual_y_ptr = XmTextGetString(manual_y);
    if ((strlen(manual_x_ptr) == 0) || (strlen(manual_y_ptr) == 0))
    {
        tie_message(Shell_Manual,"Both Lat/Y and Lon/X must be entered");
        return;
    } 

    /*
     *  If the datum is required and one was not provided then issue a message
     */
    if ((strcmp(tie_parms.datumreq, "Required") == 0) &&
            (tie_mapparm.datum[0] == NULL))
    {
        tie_message(Shell_Manual,"Must enter datum value in datum "
                "selection box");
        return;
    }

    if (tie_select.seltie == tie_select.numalloc)
    {
        tie_select.numalloc += MAX_TIE_POINTS;
        tie_select.elev    =(double *)(realloc(tie_select.elev,
                    tie_select.numalloc*sizeof(double)));
        tie_select.rtie    =(void *)(realloc(tie_select.rtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.crtie    =(void *)(realloc(tie_select.crtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.stie    =(void *)(realloc(tie_select.stie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.lsrtie  =(void *)(realloc(tie_select.lsrtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.resid   =(void *)(realloc(tie_select.resid,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.pt_id   =(void *)(realloc(tie_select.pt_id,
                    tie_select.numalloc*PTIDLEN*sizeof(char)));
        tie_select.mapname =(void *)(realloc(tie_select.mapname,
                    tie_select.numalloc*80*sizeof(char)));
        tie_select.mapid   =(void *)(realloc(tie_select.mapid,
                    tie_select.numalloc*80*sizeof(char)));
        tie_select.disabled=(unsigned char *)(realloc(tie_select.disabled,
                    tie_select.numalloc*sizeof(unsigned char)));
        if ((tie_select.elev == NULL) || (tie_select.rtie == NULL) ||
                (tie_select.crtie == NULL) || (tie_select.stie == NULL) || 
                (tie_select.lsrtie == NULL) || (tie_select.resid == NULL) || 
                (tie_select.pt_id == NULL) || (tie_select.mapname == NULL) || 
                (tie_select.mapid == NULL) || (tie_select.disabled == NULL))
        {
            tie_select.numalloc -= MAX_TIE_POINTS;
            printf("Error allocating memory\n");
            XBell(tie.display,0);
            return;
        }
    }

    /*
     *  Convert the LAT/Y and LON/X to degrees 
     */ 
    if (tie_mapparm.units == MAP_DMS) 
    {
        tie_select.rtie[tie_select.seltie][0] =
            map_dms2deg(manual_x_ptr);
        tie_select.rtie[tie_select.seltie][1] =
            map_dms2deg(manual_y_ptr);
    }
    else if (tie_mapparm.units == MAP_MINUTES)
    {
        tie_select.rtie[tie_select.seltie][0] =
            map_min2deg(manual_x_ptr,"LON");
        tie_select.rtie[tie_select.seltie][1] =
            map_min2deg(manual_y_ptr,"LAT");
    }
    else if (tie_mapparm.units == MAP_SECONDS)
    {
        tie_select.rtie[tie_select.seltie][0] =
            map_sec2deg(manual_x_ptr,"LON");
        tie_select.rtie[tie_select.seltie][1] =
            map_sec2deg(manual_y_ptr,"LAT");
    }
    else if (tie_mapparm.units == MAP_PROJ)
    {
        /*
         * Get Lon/X and Lat/Y 
         */
        status = sscanf(manual_x_ptr, "%lf",
                &tie_select.rtie[tie_select.seltie][0]);

        if (status != 1)
        {
            tie_message(Shell_Manual, "Invalid Lon/X value");
            return;
        }

        status = sscanf(manual_y_ptr, "%lf",
                &tie_select.rtie[tie_select.seltie][1]);

        if (status != 1)
        {
            tie_message(Shell_Manual, "Invalid Lat/Y value");
            return;
        }

        /* 
         *  Get projection code 
         */

        temp_ptr = XmTextGetString(proj_code_value);
        status = sscanf(temp_ptr, "%d", &current_proj_code);
        XtFree(temp_ptr);
        temp_ptr = NULL;

        if (status != 1)
        {
            tie_message(Shell_Manual,"Must enter projection code");
            return;
        }

        /*
         * If the user inputs 0 for geographic, return an error because
         * the user should use either DMS, Degrees, Minutes or Seconds
         * instead of Projection (Y,X).
         */
        if (current_proj_code == 0)
        {
            tie_message(Shell_Manual,"Projection code 0 was entered. Re-select "
                       "the Coordinate Type to either DMS, Degrees, Minutes or "
                       "Seconds.");
            return;
        }

        /*
         *  Verify that the projection code entered is valid
         */
        if ((current_proj_code > MAXPROJ)||(current_proj_code < 0))
        {
            tie_message(Shell_Manual,"Invalid projection code entered");
            return;
        }

        /* 
         *  Get zone code which is required for UTM and State Plane. Otherwise
         *  set zone to default value.
         */

        temp_ptr = XmTextGetString(zone_code_value);
        status = sscanf(temp_ptr, "%d", &current_zone_code);
        XtFree (temp_ptr);
        temp_ptr = NULL;

        if (status != 1)
        {
            if ((current_proj_code == UTM) || (current_proj_code == SPCS))
            {
                tie_message(Shell_Manual,"Must enter zone code");
                return;
            }
        }

        if ((current_proj_code != UTM) && (current_proj_code != SPCS)) 
        {
            if ((current_zone_code != 0) && (current_zone_code != 62))
            {
                tie_message(Shell_Manual,
                        "Invalid zone code. The combination of the\n"
                        "projection code and zone code are not valid.\n"
                        "Re-enter either the projection code or zone code");
                return;
            }
            else 
                current_zone_code = 62;
        }

        /* 
         * UTM and State Plane use zone codes instead of specific 
         * projection parameters. Set the flag to false to indicate 
         * that the semi major and minor axis should not be included 
         * in the first 2 parameters. Then set them to empty.
         */ 
        if ((current_proj_code == UTM) || (current_proj_code == SPCS))
        {
            tie_mapparm.set_semi_major_minor = FALSE;
            /*
             * There is one instance where the first 2 parameters are required.
             * It is when zone is undefined (code number 62) for projection UTM.
             * Since the user is required to enter the Lon/Z and Lat/Z, do not
             * set the first two projection parameters to empty.
             */ 
            if (current_zone_code != 62)
            {
                XmTextSetString(proj_parms[0], "");
                XmTextSetString(proj_parms[1], ""); 
            }
            /*
             * Set the remaining projection parameters to empty
             */
            for (index = 2; index < 15; index++)
                XmTextSetString(proj_parms[index], "");
        }
        else
            tie_mapparm.set_semi_major_minor = TRUE; 

        /*
         *  Error check the projection and zone codes entered by the user to 
         *  verify that the combination of the two is valid.
         */     
        if ((current_proj_code == UTM) && ((current_zone_code > 62) 
           || (current_zone_code < -60) || (current_zone_code == 61)
           || (current_zone_code == 0)))
        {
            tie_message(Shell_Manual,
                    "Zone code value must be either between -60 and -1, "
                    "1 and 60 or 62 for UTM projection");
            return;
        }
        else if((current_proj_code == SPCS) &&
                ((current_zone_code > 5400) || 
                (current_zone_code < 101)))
        {
            tie_message(Shell_Manual,
                    "Zone code value must be between 101 and 5400 for State "
                    "Plane projection");
            return;
        }

        /* If the user entered projection UTM and zone 62, then the first two 
         * projection parameters are required. (NOTE: the values need to be
         * Lon/Z and Lat/Z in DMS format instead of the semi major and minor  
         * values.
         */
        if ((current_proj_code == UTM) && (current_zone_code == 62))
        {
            for (index = 0; index < 2; index ++)
            {
                temp_ptr = XmTextGetString(proj_parms[index]); 
                status = sscanf(temp_ptr, "%lf", &current_proj_coef[index]);
                XtFree (temp_ptr);
                temp_ptr = NULL;

                if (status != 1) 
                {
                    tie_message(Shell_Manual, "Lon/Z and Lat/Z must be entered "
                            "in the first two projection parameters");
                    return;
                }
            }
        }

        /*
         * Update the first two parameters with semi major and minor axis 
         * in the manual entry box. Also update the tie_parms structure
         * which is needed for c_trans.
         */
        else if (tie_mapparm.set_semi_major_minor == TRUE)
        {
            sprintf(major_minor_string, "%lf", tie_mapparm.datum_info.smajor);
            current_proj_coef[0] = tie_mapparm.datum_info.smajor;
            XmTextSetString(proj_parms[0], major_minor_string);

            sprintf(major_minor_string, "%lf", tie_mapparm.datum_info.sminor);
            current_proj_coef[1] = tie_mapparm.datum_info.sminor;
            XmTextSetString(proj_parms[1], major_minor_string);
        } 

        /*
         * Copy the rest of the parameters into the tie_parms structure
         */
        for (index = 2; index < 15; index ++)
        {
            /* The status of sscanf is not checked here. If a projection 
             * parameter is left empty, 0.0 is assigned as its value. This 
             * value is already set in tie_parms structure when it was 
             * initialized with 0.0
             */
            temp_ptr = XmTextGetString(proj_parms[index]);
            sscanf(temp_ptr, "%lf", &current_proj_coef[index]);
            XtFree (temp_ptr);
            temp_ptr = NULL;
        }

    }
    else /* already in degrees */
    {
        /*
         * Get Lon/X and Lat/Y 
         */
        status = sscanf(manual_x_ptr, "%lf",
                &tie_select.rtie[tie_select.seltie][0]);

        if (status != 1)
        {
            tie_message(Shell_Manual, "Invalid Lon/X value");
            return;
        }

        status = sscanf(manual_y_ptr, "%lf",
                &tie_select.rtie[tie_select.seltie][1]);

        if (status != 1)
        {
            tie_message(Shell_Manual, "Invalid Lat/Y value");
            return;
        }
    }

    /* Free memory */
    XtFree(manual_x_ptr);
    manual_x_ptr = NULL;
    XtFree(manual_y_ptr);
    manual_y_ptr = NULL;

    /* Start the conversion process! */

    /* 
     *  Since the LAT/Y and LON/X have already been converted (from DMS, 
     *  seconds or minutes) to degrees, then the input units can only be 
     *  meters or degrees
     */
    if (tie_mapparm.units == MAP_PROJ)
        current_proj_units = METER;
    else
        current_proj_units = DEGREE;

    /* 
     *  Copy datum into the structure. Since all points need to
     *  reference the first datum, copy it to out_datum_code.
     */

    if (tie_mapparm.out_datum_code == 0)
        tie_mapparm.out_datum_code = tie_mapparm.current_datum;

    /* 
     *  The output value will always be in degrees
     */
    out_proj_code = GEO;
    out_zone_code = 62;
    out_proj_units = DEGREE;

    /* Do not write out the projection parms to a file or terminal */
    prtprm[0] = prtprm[1] = FALSE;

    /* 
     *  Initialize projection and datum information
     */
    status = c_transinit(&current_proj_code, &current_proj_units, 
              &current_zone_code, &tie_mapparm.current_datum, 
              current_proj_coef, &out_proj_code, &out_proj_units, 
              &out_zone_code, &tie_mapparm.out_datum_code, out_proj_coef,
              prtprm, "");

    if (status != E_SUCC)
    {
        tie_message(Shell_Manual, 
                   "Error initializing projection parameters\n "
                   "Verify the projection parameters are correct\n");
        return;
    }
 
    /* 
     *  Convert the projection coordinates
     */
    status = c_trans(&current_proj_code,&current_proj_units,
              &out_proj_code,&out_proj_units,
              &tie_select.rtie[tie_select.seltie][0], 
              &tie_select.rtie[tie_select.seltie][1], &out_x, &out_y);

    if (status != E_SUCC)
        tie_message(Shell_Manual,
               "Error converting search image projection coordinates\n "
               "Verify the projection parameters are correct\n");

    /* copy results back to tie_select */
    tie_select.rtie[tie_select.seltie][0] = out_x;
    tie_select.rtie[tie_select.seltie][1] = out_y;

    /*
     *  Call tps_update to update the reference tps file with the datum 
     *  information(and projection information - if the projection is MAP_PROJ).
     *  All of these projection and datum checks are done in this routine.
     */
    tps_update(); 

    strcpy(tie_mapparm.mapname,"");
    strcpy(tie_mapparm.mapid,"");
    tie_select.saveref[0] = tie_select.rtie[tie_select.seltie][0];
    tie_select.saveref[1] = tie_select.rtie[tie_select.seltie][1];

    tie_select.curref = TRUE;

    /*
     *  *** if overview image is already fullres, send event to zoom
     */
    if (tie_rt.calc_rt != RT_SEL)
    {
        xe.xbutton.type = ButtonRelease;
        xe.xbutton.button = 1;
        xe.xbutton.x = 0;
        xe.xbutton.y = 0;

        fullres_button(ref_image_bulletinboard, NULL, &xe, 0);
    }
}

/*******************************************************************************
Purpose:
    This routine is the callback to the map coordinate manual entry panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void manual_coord
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    int index;     /* Value used for looping */

    if (client == NULL)
        return;

    /*
     *  Set the semi major and semi minor axis to empty. We want to remove 
     *  these values in case the map coordinate "projection" was chosen first 
     *  and now a different map coordinate is selected.
     */

    XmTextSetString(proj_parms[0], "");
    XmTextSetString(proj_parms[1], "");

    /*
     *  Enable projection parameter fields if "PROJ" is selected. Otherwise
     *  disable it since projection parameters are not needed.
     */
    if (strcmp((char*)client, "PROJ") == 0)
    {
        XtSetSensitive(proj_code_value,True);
        XtSetSensitive(proj_code_label,True);
        XtSetSensitive(zone_code_value,True);
        XtSetSensitive(zone_code_label,True);
        XtSetSensitive(proj_parm_label,True);
        for (index = 0; index < 15; index++)
        {
            XtSetSensitive(proj_parms[index],True); 
            XtSetSensitive(proj_parm_number[index],True);
        }
    }
    else
    {
        XtSetSensitive(proj_code_value,False);
        XtSetSensitive(proj_code_label,False);
        XtSetSensitive(zone_code_value,False);
        XtSetSensitive(zone_code_label,False);
        XtSetSensitive(proj_parm_label,False);
        for (index = 0; index < 15; index++)
        {
            XtSetSensitive(proj_parms[index],False); 
            XtSetSensitive(proj_parm_number[index],False);
        }
    }

    /*  Initialize tie_mapparm.units to DMS because it the first 
     *  Coordinate Type option. DMS is assumed until another
     *  option is selected.
     */
    tie_mapparm.units = MAP_DMS;

    if (strcmp((char*)client, "DMS") == 0) 
    {
        tie_mapparm.units = MAP_DMS; 
    }
    else if (strcmp((char*)client, "DEGREES") == 0) 
    {
        tie_mapparm.units = MAP_DEGREES;
    }
    else if (strcmp((char*)client, "MINUTES") == 0)
    {
        tie_mapparm.units = MAP_MINUTES;
    }
    else if (strcmp((char*)client, "SECONDS") == 0) 
    {
        tie_mapparm.units = MAP_SECONDS;
    }
    else if (strcmp((char*)client, "PROJ") == 0)
    {
        tie_mapparm.units = MAP_PROJ;
    }
}

/*******************************************************************************
Purpose:
    This routine traverses the map coordinate manual entry panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void manual_traverse
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XmProcessTraversal(Manual_form, XmTRAVERSE_NEXT_TAB_GROUP);
}

/*******************************************************************************
Purpose: 
    This routine displays the datum requestor panel.

By D. Hames, USGS/EROS Data Center, 4/01

Algorithm Description:
*******************************************************************************/
void manual_datum_show
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Datum_selector = Createdatum_selector(Shell_Manual); 
    XtSetSensitive(man_datum_button, False);
    XtManageChild(Datum_selector);
}

/*******************************************************************************
Purpose:
   Select a Datum value from the selector

By D. Hames, USGS/EROS Data Center, 4/01

Algorithm Description:
*******************************************************************************/
void manual_datum_select
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    char *datumvalue;                /* Temporary storage used to 
                                        hold the datum string value  */
    char *proj_code_value_ptr;       /* Pointer used for 
                                        XmTextGetString call         */
    char semi_major_string[CMLEN];   /* Temporary storage used to 
                                        hold the string value of the 
                                        semi major axis              */
    char semi_minor_string[CMLEN];   /* Temporary storage used to 
                                        hold the string value of the 
                                        semi minor axis              */
    int status;                      /* Function return status       */
    long proj_code;                  /* projection value retrieved
                                        from manual entry            */

    XmFileSelectionBoxCallbackStruct *acs =
        (XmFileSelectionBoxCallbackStruct*)call_data;

    if ((int)client == TRUE)
    {
        /*
         *  Retrieve the datum value
         */
        XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&datumvalue);

        /*
         *  Set the text to this value
         */
        XmTextSetString(man_datum_text, datumvalue);

        /*
         *  Copy string datum value to tie_mapparm structure
         */
        strcpy(tie_mapparm.datum, datumvalue);

        XtFree(datumvalue);
        datumvalue = NULL;

        /*
         *  Extract datum number from ( ) at beginning of string
         */
        if (tie_mapparm.datum[0] == '(') 
            tie_mapparm.current_datum = atoi(&tie_mapparm.datum[1]);
        else
        {
            tie_message(Shell_MParm, 
                    "Unable to retrieve datum number from input datum value.\n"
                    "Re-select the datum from the datum selection box");
            return;
        }

        if (tie_mapparm.units == MAP_PROJ) 
        {
            /*
             *  Copy the datum code to the datum_info structure
             */
            tie_mapparm.datum_info.datumnum = tie_mapparm.current_datum;

            /*
             *  Given the datum number, retrieve semi major and minor values
             */
            if (c_getdatum(&tie_mapparm.datum_info) != E_SUCC)
                tie_message(Shell_Manual,
                            "Unable to retrieve datum information\n");

            /*
             * Get proj code which is used to indicate if a semi major and
             * minor axis are required.
             */
            proj_code_value_ptr = XmTextGetString(proj_code_value);
            status = sscanf(proj_code_value_ptr, "%d", &proj_code);
            XtFree (proj_code_value_ptr);
            proj_code_value_ptr = NULL;

            /* 
             *  If the projection code was entered check if a semi major and
             *  minor axis are required. Otherwise assume they are required
             *  until the projection is picked. This error checking is done
             *  in manual_spark.
             */
            if (status == 1)
            {
                /*
                 * UTM and State Plane use zone codes instead of specific 
                 * projection parameters. Set the flag to false to indicate 
                 * that the semi major and minor axis should not be included 
                 * in the first 2 parameters.
                 */ 
                if ((proj_code == UTM) || (proj_code == SPCS))
                    tie_mapparm.set_semi_major_minor = FALSE;
                else
                    tie_mapparm.set_semi_major_minor = TRUE; 
            }
            else
                tie_mapparm.set_semi_major_minor = TRUE;

            /*
             *  Set the semi major and semi minor axis to empty. We want to 
             *  remove these values in case a different datum was selected 
             *  prior to this one. 
             */
            XmTextSetString(proj_parms[0], ""); 
            XmTextSetString(proj_parms[1], "");  

            /*
             *  Update the first two parameter values in the manual entry
             *  panel.
             */ 
            if (tie_mapparm.set_semi_major_minor == TRUE) 
            {
                sprintf(semi_major_string, "%lf",tie_mapparm.datum_info.smajor);
                sprintf(semi_minor_string, "%lf",tie_mapparm.datum_info.sminor);
                XmTextSetString(proj_parms[0], semi_major_string);
                XmTextSetString(proj_parms[1], semi_minor_string);
            } 
        }
    }

    /*
     *  Resensitize the button and remove the selector
     */
    XtSetSensitive(man_datum_button, True);
    XtDestroyWidget(Datum_selector);

}

