#include "tiepts.h"
#include <math.h>
#include <stdlib.h>
#include <sys/file.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "cr.h"
#include "mapsup.h"
#include "convert.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine sets the maps which are in the selected mapfile to
    "selected".

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to read and process both the map name and map ID.
Modified by J. LaVergne, USGS/EROS Data Center, 7/93
  - Corrected an error in maplist_info for 63360 scale maps (the map width
    was being calculated incorrectly.
Algorithm Description:
*******************************************************************************/
void maplist_init(void)
{
   char                 *mapfile;       /* selected map file            */

  /*
   *   *** make sure maps are present in list
   */
   if (tie_maplist == NULL)
      return;

   mapfile =  XmTextGetString(maplist_file_text);
  
  /*
   *  *** make sure map exists
   */
   if ((strcmp(mapfile, "") == 0) || (access(mapfile, 0)))
      return;
      
   display_maps();

   XtFree(mapfile);
}

/*******************************************************************************
Purpose:
    This routine reads a map catalog and draws an outline of all 
    maps which reside on the current image.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Center, 5/93
  - Modified to read and process both the map name and map ID.
Modified by J. LaVergne, USGS/EROS Data Center, 7/93
  - Fixed errors where dimensions of name and state did not match the
    structure defined in tiepts.h. (caused failures on SG systems).

Algorithm Description:
*******************************************************************************/
void maplist_define(void)
{
   FILE         *fd;            /* file descriptor for map catalog         */
   char         buf[81],        /* string buffer for reading ASCII lines   */
                *catalog,       /* name of catalog file                    */
                district[3],    /* state or country code                   */
                id[7],          /* map id                                  */
                name[38];       /* name of map                             */

   int          i,                      /* loop counter                 */
                llx, lly,               /* lower left coordinates       */
                lrx, lry,               /* lower right coordinates      */
                nx, ny,                 /* number of spark points       */
                scale,                  /* scale of map                 */
                splane_idx,             /* search image memory plane    */
                ulx, uly,               /* upper left coordinates       */
                urx, ury;               /* upper right coordinates      */
        
   double       clat,                   /* center lat of map            */
                clon,                   /* center lon of map            */
                maph,
                mapvx,
                mapvy,
                mapw,                   /* map dimension in degrees     */
                ssl,                    /* search line coordinate       */
                sss;                    /* search sample coordinate     */

   struct TIE_MAPLIST   *temp,          /* pointer to node in linked list  */
                        *last;          /* pointer to end of linked list   */

  /*
   *  *** get name of map catalog and open it
   */
   catalog = XmTextGetString(maplist_catalog_text);

   if (strcmp(catalog, "") == 0) 
      {
      tie_message(Shell_Getmap, "Must enter catalog filename");
      return;
      }
   else if (access(catalog, 0))
      {
      tie_message(Shell_Getmap, "Catalog file does not exist");
      return;
      }
   else if ((fd = fopen(tie_mapparm.catalog, "r")) == NULL)
      {
      tie_message(Shell_Getmap, "Cannot open map catalog");
      return;
      }

   splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

   if (tie_maplist != NULL)
      maplist_free();

  /*
   *  *** read and display map outlines
   */
   while (fgets(buf, 81, fd) != NULL)
      {
      if ((int)strlen(buf) < 49)
         continue;

     /*
      *  *** get center lat lon values
      */
      sscanf(buf+46, "%lf %lf", &clat, &clon);

      if ((int)strlen(buf) < 74)
         scale = 24000;
      else
         sscanf(buf+74, "%d", &scale);

     /*
      *   *** extract state or country
      */
      strncpy(district,buf+43,2);
      district[2]='\0';

     /*
      *  *** get map dimension in degrees
      */
     if (maplist_info(scale,district,&mapw,&maph,&mapvx,&mapvy,&nx,&ny)==E_FAIL)
         break;

     /*
      *  *** compute corner coordindates
      */
      ref2srch(clon-(mapw/2.0), clat+(maph/2.0), &sss, &ssl);
      full2over(search_image_bulletinboard, (int)sss, (int)ssl, &ulx, &uly);

      ref2srch(clon+(mapw/2.0), clat+(maph/2.0), &sss, &ssl);
      full2over(search_image_bulletinboard, (int)sss, (int)ssl, &urx, &ury);

      ref2srch(clon-(mapw/2.0), clat-(maph/2.0), &sss, &ssl);
      full2over(search_image_bulletinboard, (int)sss, (int)ssl, &llx, &lly);

      ref2srch(clon+(mapw/2.0), clat-(maph/2.0), &sss, &ssl);
      full2over(search_image_bulletinboard, (int)sss, (int)ssl, &lrx, &lry);

     /*
      *  *** if outside image, continue
      */
      if (((lrx < 0) && (urx < 0))                     ||
          ((llx >= tie_planes[splane_idx].dspwin[NS])  && 
           (llx >= tie_planes[splane_idx].dspwin[NS])) ||
          ((lly < 0) && (lry < 0))                     ||
          ((uly >= tie_planes[splane_idx].dspwin[NL])  &&
           (ury >= tie_planes[splane_idx].dspwin[NL])))
         continue;

     /*
      *  *** create new linked list node
      */
      temp = TieMalloc(sizeof(struct TIE_MAPLIST));
      temp->selected = NULL;
      temp->dispname = XmToggleButtonGetState(maplist_name_on);
      temp->next = NULL;

     /*
      *   *** extract name
      */
      strncpy(name, buf, 37);
      i = 36;
      while(name[i--] == ' ');
      name[i+2]='\0';
      strcpy(temp->name, name);

     /*
      *   *** extract id
      */
      strncpy(id,buf+37,6);
      id[6] = '\0';
      if (strcmp(id,"      ") == 0)
          strcpy(id,"");
      else
         {
         i = 5;
         while(id[i--] == ' ');
         id[i+2]='\0';
         }
      strcpy(temp->id, id);

     /*
      *   *** extract state or country
      */
      strcpy(temp->state, district);

      temp->scale = scale;
      temp->clat = clat;
      temp->clon = clon;
      temp->outline[0][0] = ulx;
      temp->outline[0][1] = uly;
      temp->outline[1][0] = urx;
      temp->outline[1][1] = ury;
      temp->outline[2][0] = lrx;
      temp->outline[2][1] = lry;
      temp->outline[3][0] = llx;
      temp->outline[3][1] = lly;
      temp->next = NULL;

     /*
      *  *** add node to list
      */
      if (tie_maplist == NULL)
         {
         tie_maplist = temp;
         last = temp;
         }
      else
         {
         last->next = temp;
         last = temp;
         }
      }

   if (tie_maplist == NULL)
      tie_message(Shell_Getmap, "No maps cataloged for this image");
   else
      XClearArea(tie.display, XtWindow(search_image_bulletinboard), 
                 0,0,0,0, True);

   XtFree(catalog);
}


/*******************************************************************************
Purpose:
   Frees space allocate for linked list of maps.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void maplist_free(void)
{
   struct TIE_MAPLIST   *temp;  /* pointer to node in linked list       */

   while (tie_maplist != NULL)
      {
      temp = tie_maplist;
      tie_maplist = tie_maplist->next;
      free(temp);
      }
}

/*******************************************************************************
Purpose:
   Handles the selection of a map from the search display.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void maplist_name
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   struct TIE_MAPLIST   *temp;          /* pointer to node in linked list       */

   temp = tie_maplist;

   if (temp == NULL)
      return;

   while (temp != NULL)
      {
      temp->dispname = (int)client;
      temp = temp->next;
      }

   XClearArea(tie.display, XtWindow(search_image_bulletinboard),0,0,0,0,True);
   display_maps();
}

/*******************************************************************************
Purpose:
   Handles the selection of a map from the search display.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void maplist_select
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   struct TIE_MAPLIST *temp;    /* pointer to node in linked list       */
   int          height,         /* height of map in pixels              */
                i,              /* loop counter                         */
                ulx, uly,       /* upper left coordinate                */
                lrx, lry,       /* lower right coordinate               */
                width;          /* width of map in pixels               */
   char         *mapfile;       /* map file name                        */
   XPoint       pt[4];          /* 4 corner values of map               */

   temp = tie_maplist;
   
  /*
   *   *** for each item in the list
   */
   while (temp != NULL)
      {
     /*
      *  *** if mouse click with map coordinates toggle selection
      */
      if ((event->xbutton.x >= temp->outline[0][0]) &&
          (event->xbutton.x <= temp->outline[1][0]) &&
          (event->xbutton.y >= temp->outline[0][1]) &&
          (event->xbutton.y <= temp->outline[2][1]))
         {
         ulx = lrx = temp->outline[0][0];
         uly = lry = temp->outline[0][1];

         for (i = 1; i < 4; i++)
            {
            ulx = (temp->outline[i][0] < ulx) ? temp->outline[i][0] : ulx;
            uly = (temp->outline[i][1] < uly) ? temp->outline[i][1] : uly;
   
            lrx = (temp->outline[i][0] > lrx) ? temp->outline[i][0] : lrx;
            lry = (temp->outline[i][1] > lry) ? temp->outline[i][1] : lry;
            }

         width  = lrx - ulx;
         height = lry - uly;

         if (event->xbutton.button == 2)
            {
            if (temp->dispname)
               {
               if (!temp->selected)
               temp->dispname = False;
               }
            else
               temp->dispname = True;

            XClearArea(tie.display, XtWindow(search_image_bulletinboard),
                       ulx, uly, width+2, height+2, True);
            }
         else if ((event->xbutton.button == 1) && (temp->selected))
            {
            if (temp->dispname)
               {
               temp->selected = FALSE;
               XClearArea(tie.display, XtWindow(search_image_bulletinboard),
                       ulx, uly, width+2, height+2, True);
               }
            else
               {
               temp = temp->next;
               continue;
               }
            }
         else if (event->xbutton.button == 1)
            {
            if (temp->dispname)
               {
               for (i = 0; i < 4; i++)
                  {
                  pt[i].x = temp->outline[i][0];
                  pt[i].y = temp->outline[i][1];
                  }
               XFillPolygon(tie.display, XtWindow(search_image_bulletinboard),
                         tie_select.gc, pt, 4, Convex, 
                         CoordModeOrigin);

               temp->selected = TRUE;
               }
            else
               {
               temp = temp->next;
               continue;
               }
            }

       /*
        * if a map file is specified in the register panel, place the map name 
        * in the appropriate text field and issue the routine to update 
        * parameters associated with that map
        */
        else if (event->xbutton.button == 3)
            {
            if (XtIsRealized(Shell_MParm))
               if (temp->dispname)
                  {
                  mapfile = XmTextGetString(reg_mapfile_text);
                  if (strcmp(mapfile,"") != 0)
                      get_map_rec(mapfile,temp->name,temp->id);
                  }
               else
                  {
                  temp = temp->next;
                  continue;
                  }
            }

         break;
         }

      temp = temp->next;
      }
}

/*******************************************************************************
Purpose:
   This function is the callback routine for the Map Catalog button on the
   map selection panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void maplist_catfile
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Catalog_fileselector = Createcatalog_fileselector(Shell_Getmap);
   XtSetSensitive(maplist_catalog_button, False);
   XtManageChild(Catalog_fileselector);
}

/*******************************************************************************
Purpose:
   This function is the callback routine for the Map Catalog button on the
   map selection panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void maplist_cattext
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   char         *catalog;

   catalog = XmTextGetString(maplist_catalog_text);

   if (strcmp(catalog, "") == 0)
      return;
  
   if (strcmp(catalog, tie_mapparm.catalog) != 0)
      {
      strcpy(tie_mapparm.catalog, catalog);
      maplist_define();
      }

   XtFree(catalog);
}

/*******************************************************************************
Purpose:
   This function is the callback routine for the Map File button on the
   map selection panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void maplist_mapfile
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Map_fileselector = Createmap_fileselector(Shell_Getmap);
   XtSetSensitive(maplist_file_button, False);
   XtManageChild(Map_fileselector);
}


/*******************************************************************************
Purpose:
   This routine remove the map selection panel and store the selected
   maps in the map file.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS DatA Center, 5/93
  - Modified to read and process both the map name and map ID.

Algorithm Description:
*******************************************************************************/
void maplist_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   struct 
   TIE_MAPLIST          *temp;          /* pointer to node in linked list */
   struct 
   TIE_MAPPARMS         parms;          /* map paramters record           */
   char                 *mapfile;       /* select map file                */
   int                  nx, ny;         /* number of spark points       */
   double               maph,           /* map height in degrees          */
                        mapvx,          /* map verify y offset            */
                        mapvy,          /* map verify x offset            */
                        mapw;           /* map width in degrees           */

  /*
   *  *** get map filename from text widget
   */
   mapfile =  XmTextGetString(maplist_file_text);
 
   if ((strcmp((char*)client, "CANCEL") != 0) && (strcmp(mapfile, "") == 0))
      {
      tie_message(Shell_Getmap, "Must enter map filename");
      return;
      }

   strcpy(tie_mapparm.mapfile, mapfile);

  /*
   *  *** remove panel
   */
   XtUnmanageChild(Getmap_form);
   XtDestroyWidget(Getmap_form);
   XtUnrealizeWidget(Shell_Getmap);

   XtSetSensitive(main_map_button, True);


   XtAddEventHandler(search_image_bulletinboard,ButtonReleaseMask,FALSE,
              fullres_button,NULL);
   XtRemoveEventHandler(search_image_bulletinboard,ButtonReleaseMask,FALSE,
              maplist_select,NULL);

   tie.flags |= TIE_RECTCUR;


  /*
   *  *** if not cancelled, write selected maps to file
   */
   if (strcmp((char*)client, "CANCEL") != 0)
      {
      temp = tie_maplist;

      while (temp != NULL)
         {
         if ((strcmp((char*)client, "ALL") == 0) || (temp->selected))
            if (map_read(mapfile, temp->name, temp->id, &parms) != E_SUCC)
               {
              /*
               *  *** get map dimension in degrees
               */
               if (maplist_info(temp->scale,temp->state, &mapw, &maph, &mapvx, 
                                &mapvy, &nx, &ny) == E_FAIL)
                  break;

               parms.ul_x  = temp->clon - (mapw/2.0);
               parms.ul_y  = temp->clat + (maph/2.0);
               maplist_round(&parms.ul_x);
               maplist_round(&parms.ul_y);
               
               parms.lr_x  = parms.ul_x + mapw;
               parms.lr_y  = parms.ul_y - maph;
               parms.verify_x = parms.ul_x + mapvx;
               parms.verify_y = parms.lr_y + mapvy;
               parms.npts_x = nx;
               parms.npts_y = ny;
               parms.inc_x =  fabs(parms.ul_x - parms.lr_x) /
                             ((double)(parms.npts_x - 1));
               parms.inc_y = -fabs(parms.ul_y - parms.lr_y) /
                             ((double)(parms.npts_y - 1));
               parms.units = MAP_DEGREES;
               parms.scale = temp->scale;
               strcpy(parms.mapfile, mapfile);
               strcpy(parms.mapname, temp->name);
               strcpy(parms.mapid, temp->id);
               strcpy(parms.mapst_cntry, temp->state);
               strcpy(parms.datum, "");
               strcpy(parms.maptype, "Quadrangle");
               strcpy(parms.mapdate, "");
               strcpy(parms.mapdesc, "");
      
               map_writelt(parms);
               }
   
         temp = temp->next;
         }
      }
  
  /*
   *  *** free linked list
   */
   maplist_free();
   XtFree(mapfile);
   XClearArea(tie.display,XtWindow(search_image_bulletinboard),0,0,0,0,True);
}


int maplist_info
(
   int  s,
   char *state,
   double *w, 
   double *h, 
   double *vx, 
   double *vy,
   int  *nx, 
   int  *ny
)
{
  /*
   *  *** get map scale
   */
/*****
   XtSetArg(args[0], XmNmenuHistory, &scale);
   XtGetValues(maplist_scale_option, args, 1);
 ****/

   if (s == 12000)
      {
      *w = 3.75/60.0;
      *h = 3.75/60.0;
      *vx = 1.25/60.0;
      *vy = 1.25/60.0;
      *nx = 2;
      *ny = 2;
      }
   else if (s == 24000)
      {
      *w = 7.5/60.0;
      *h = 7.5/60.0;
      *vx = 2.5/60.0;
      *vy = 2.5/60.0;
      *nx = 2;
      *ny = 2;
      }
   else if (s == 25000)
      {
      *w = 7.5/60.0;
      *h = 7.5/60.0;
      *vx = 2.5/60.0;
      *vy = 2.5/60.0;
      *nx = 2;
      *ny = 2;
      }
   else if (s == 50000)
      {
      /* Canada maps 15'h x 30'w */
      if (strcmp(state,"CN") == 0)
         {
         *w = 30./60.0;
         *h = 15./60.0;
         *vx = 5.0/60.0;
         *vy = 5.0/60.0;
         *nx = 2;
         *ny = 2;
         }
      /* Central America maps 10'h x 15'w */
      /* Costa Rica, Dominican Republic, El Salvador, Guatemala,
         Haiti, Honduras, Nicaragua, Panama */
      else if ((strcmp(state,"CR") == 0) ||
               (strcmp(state,"DR") == 0) ||
               (strcmp(state,"ES") == 0) ||
               (strcmp(state,"GT") == 0) ||
               (strcmp(state,"HA") == 0) ||
               (strcmp(state,"HO") == 0) ||
               (strcmp(state,"NI") == 0) ||
               (strcmp(state,"PN") == 0))
         {
         *w = 15./60.0;
         *h = 10./60.0;
         *vx = 5.0/60.0;
         *vy = 5.0/60.0;
         *nx = 2;
         *ny = 2;
         }
      /* US and Mexico maps 15'h x 20'w */
      else
         {
         *w = 20./60.0;
         *h = 15./60.0;
         *vx = 5.0/60.0;
         *vy = 5.0/60.0;
         *nx = 2;
         *ny = 2;
         }
      }
   else if (s == 62500)
      {
      *w = 15./60.0;
      *h = 15./60.0;
      *vx = 5.0/60.0;
      *vy = 5.0/60.0;
      *nx = 2;
      *ny = 2;
      }
   else if (s == 63360)
      {
      /* Alaska maps 15'h x 22'30''w */
      if (strcmp(state,"AK") == 0)
         {
         *w = 22.50/60.0;
         *h = 15./60.0;
         *vx = 7.5/60.0;
         *vy = 5.0/60.0;
         *nx = 2;
         *ny = 2;
         }
      else
         {
         /* all others 15'h x 30'w */
         *w  = 30.0/60.0;
         *h  = 15.0/60.0;
         *vx =  5.0/60.0;
         *vy =  5.0/60.0;
         *nx =  2;
         *ny =  2;
         }
      }
   else if (s == 100000)
      {
      *w = 40.0/60.0;
      *h = 0.5;
      *vx = 10.0/60.0;
      *vy = 10.0/60.0;
      *nx = 3;
      *ny = 3;
      /* Madagascar maps (MG) will be in Laborde projection so user
         will have to reset the coords */
      }
   else if (s == 250000)
      {
      *w =  2.0;
      *h =  1.0;
      *vx = 45.0/60.0;
      *vy = 15.0/60.0;
      *nx = 5;
      *ny = 3;
      }
   else
      return(E_FAIL);

   return(E_SUCC);
}

void maplist_round
(
   double       *coord
)
{
   double       degrees,
                minutes,
                seconds;

   degrees = find_deg(*coord);
   minutes = (*coord - degrees)*60.0;
   seconds = minutes - floor(minutes);
   
   if (seconds > .75)
      *coord = degrees + ceil(minutes)/60.0;
   else if  (seconds > .25)
      *coord = degrees + (floor(minutes)+.5)/60.0;
   else 
      *coord = degrees + (floor(minutes))/60.0;
}

/*******************************************************************************
Purpose:
   Callback for map file requestor OK button.  This routine is used
   for both map selection and registration.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void mapfile_ok
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   char         *filename;
   struct stat mystat;

   XmFileSelectionBoxCallbackStruct *acs =
             (XmFileSelectionBoxCallbackStruct*)call_data;

  /*
   *  Retrieve the file name selected and parse it.  This determines its LAS
   *  name, host file name, window specified and bands specified.
   */
   XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);

  /*
   *  *** if file exists, set the mapfile text item
   */
   if ((int)strlen(filename) > 0)
      {
      stat(filename,&mystat);
      if (!S_ISDIR(mystat.st_mode))
         {
         if (XtIsRealized(Shell_MParm))
            XmTextSetString(reg_mapfile_text, filename);
         if (XtIsRealized(Shell_Getmap))
            XmTextSetString(maplist_file_text, filename);
         strcpy(tie_mapparm.mapfile, filename);
         }
      else
         {
         XBell(tie.display, 0);
         return;
         }
      }

   if (XtIsRealized(Shell_MParm))
      XtSetSensitive(reg_mapfile_button, True);
   if (XtIsRealized(Shell_Getmap))
      {
      XtSetSensitive(maplist_file_button, True);
      maplist_init();
      XClearArea(tie.display,XtWindow(search_image_bulletinboard),0,0,0,0,True);
      }

   XtDestroyWidget(Map_fileselector);
   XtFree(filename);
}

/*******************************************************************************
Purpose:
   Cancel the map file requestor.  This routine is used by bith the map
   selection and registration panels.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void mapfile_cancel
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   if (XtIsRealized(Shell_MParm))
      XtSetSensitive(reg_mapfile_button, True);
   if (XtIsRealized(Shell_Getmap))
      XtSetSensitive(maplist_file_button, True);

   XtDestroyWidget(Map_fileselector);
}



/*******************************************************************************
Purpose:
   Callback for map catalog file requestor OK button.  

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void catfile_ok
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   char         *filename;
   struct stat mystat;

   XmFileSelectionBoxCallbackStruct *acs =
             (XmFileSelectionBoxCallbackStruct*)call_data;

  /*
   *  Retrieve the file name selected and parse it.  This determines its 
   *  host file name.
   */
   XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);

  /*
   *  *** if file exists, set the mapfile text item
   */
   if ((int)strlen(filename) > 0)
      {
      stat(filename,&mystat);
      if (!S_ISDIR(mystat.st_mode))
         {
         XmTextSetString(maplist_catalog_text, filename);
         strcpy(tie_mapparm.catalog, filename);
         }
      else
         {
         XBell(tie.display, 0);
         return;
         }
      }

   XtSetSensitive(maplist_catalog_button, True);
   maplist_define();
   XtDestroyWidget(Catalog_fileselector);
   XtFree(filename);
}

/*******************************************************************************
Purpose:
   Cancel the map catalog file requestor.  

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void catfile_cancel
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XtSetSensitive(maplist_catalog_button, True);
   XtDestroyWidget(Catalog_fileselector);
}


/*******************************************************************************
Purpose:
    This routine changes the color of currently defined graphics.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void maplist_setcolor
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];
   static
   Widget       color_button = NULL;    /* selected attribute color button */
 
  /*
   *  *** if NULL client, display color requestor
   */
   if (color_button == NULL)
      {
      color_button = w;
      Color_req = Createcolor_req(Shell_Getmap, maplist_setcolor);
      XtManageChild(Color_req);
      }
  /*
   *  *** otherwise, set color and remove color requestor
   */
   else
      {
      XtSetArg(args[0],XmNforeground, COLOR(Shell_Getmap,
               wk_colors[(int)client]));
      XtSetValues(color_button, args, 1);
      XtUnmanageChild(Color_req);
      XtDestroyWidget(Color_req);
      color_button = NULL;
      tie_select.mapcolor = (int)client;
      display_maps();
      }
}
