#include <math.h>
#include "tiepts.h"
#include "mapsup.h"

/****************************************************************************
NAME:                   find_deg, find_min, find_sec    

PURPOSE:  Extracts deg, min, or sec portions of an angle

PROGRAM HISTORY:
VERSION  DATE   AUTHOR     CODE/CONT   REASON
-------  ----   ------     ---------   -----------------------------
  5.0    5/89   D. Steinwand  CSB      LAS 5.0 (original) Development
  5.1   10/93   J. LaVergne   CSB      Found a problem when the input angle
        is less than zero and negative. If the degree is rounded up, the 
        sign is lost.  Even with this fix, it should be noted that the sign 
        is lost when the the degree is zero.  It is up to the programmer 
        calling this routine to test for this condition.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:   must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
        Extract portion of angle
        Return
*****************************************************************************/
int find_deg
(
   double angle                 /* Angle in total degrees */
)
{
   int sign;                    /* Sign of angle */
   int deg;                     /* Degrees portion of angle */
   int minute;                  /* Minutes portion of angle */
   double sec;                  /* Seconds portion of angle */

   sign = 1;
   if (angle < 0) sign = -1;

   deg = (int)fabs(angle);
   minute =  (fabs(angle) - deg) * 60.0;
   sec =  (((fabs(angle) - deg) * 60.0) - minute) * 60.0;

   if (sec >= 59.995) 
      minute++;

   if (minute >= 60) 
      deg++;

   deg *= sign;
   return(deg);
}

int find_min
(
   double angle                 /* Angle in total degrees */
)
{
   double sec;                  /* Seconds portion of angle */
   int minute;                  /* Minutes portion of angle */

   angle = fabs(angle);
   angle -= (int)angle;
   minute = angle * 60.0;
   sec = ((angle * 60.0) - minute) * 60.0;

   if (sec > 59.995) 
      minute++;

   if (minute == 60)
      minute = 0;

   return(minute);
}
   
double find_sec
(
   double angle                 /* Angle in total degrees */
)
{

   angle = fabs(angle);
   angle -= (int)angle;
   angle *= 60.0;
   angle -= (int)angle;
   angle *= 60.0;

   if (angle > 59.995) 
      angle -= 60.0;

   return(angle);
}
