#include "tiepts.h"
#include "labserv.h"
#include "ltable.h"
#include "mapsup.h"


#define MAXID  500

/*******************************************************************************
Purpose:
    This routine generates the list of map ids present in the given
    map file.

By J. Hemmer, USGS/EROS Data Center, 4/92
Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to read and process both a map name and a map ID.

Algorithm Description:
*******************************************************************************/
int map_getidlist
(
    char        *mapfile,       /* Map file name                          */
    Widget      idlist          /* scrollable list of map ids in mapfile  */
)
{
   Arg            args[2];      /* resource argument variable             */
   struct TAB_DEF tab;          /* Tabular structure of label table       */
   struct VECTOR  *mapnmptr;    /* Pointer to map name field              */
   struct VECTOR  *mapidptr;    /* Pointer to map id field                */
   int            count,        /* counter list                           */
                  curlen,       /* current string length                  */
                  i, j,         /* loop counter                           */
                  listmin,      /* index of smallest item in list         */
                  status;       /* Function return status                 */
   long           mode=0;       /* Assess mode for lsstat                 */
   char           *list[MAXID], /* list of ids to insert                  */
                  *tempptr;     /* temp pointer for sorting               */
   XmString       xmstr;        /* temporary Xm string variable           */


  /* 
   *  *** Open the labeled table file & retrieve the label
   */
   if (c_lsstat(mapfile, &mode) != E_SUCC) 
      return(E_FAIL);

   open_tab(&tab, mapfile, IREAD);
   get_vector(&tab);

  /*
   *  ***  Determine pointers to fields in the file
   */ 
   mapnmptr = get_field_ptr(&tab, "NAME");
   mapidptr = get_field_ptr(&tab, "ID");

   if (mapnmptr == NULL) 
      { 
      close_tab(&tab); 
      return(E_FAIL); 
      }

  /*
   *  *** get number of items in list
   */
   XtSetArg(args[0], XmNitemCount, &curlen);
   XtGetValues(idlist, args, 1);

   count = 0; 

  /*
   *  *** create list
   */
   for (;count <= MAXID;)
      {
      status = get_record(&tab);
      if (status == E_FAIL) 
         break;

      if (status == E_EOF) 
         break;

      list[count] = (char*)malloc(strlen(mapnmptr->ptr.c)+1);
      strcpy(list[count], mapnmptr->ptr.c);

      /*
       *  *** Process the map ID if there is one present.
       */
      if (mapidptr != NULL)
          if (strcmp(mapidptr->ptr.c,"") != 0)
              {
              list[count] = (char*)realloc(list[count],strlen(mapnmptr->ptr.c) +
                  strlen(mapidptr->ptr.c) + 3);
              strcat(list[count], " @"); 
              strcat(list[count], mapidptr->ptr.c);
              }
      count++;
      }

  /* 
   *  *** Close the file & return
   */ 
   close_tab(&tab);

  /*
   *  *** insert into list alphabetically
   */
   for (i = 0; i < count; i++)
      {
     /*
      *  *** find smallest
      */
      listmin = i;
      for (j = i+1; j < count; j++)
         if (strcmp(list[j], list[listmin]) < 0)
            listmin = j; 

     /*
      *  *** insert in list widget
      */
      xmstr = XmStringCreateLtoR(list[listmin], XmSTRING_DEFAULT_CHARSET);
 
     /*
      *  *** add to list, this procedure is need because of bug in DG motif
      */
      if ((i+1) <= curlen)
         XmListReplaceItemsPos(idlist, &xmstr, 1, i+1);
      else
         XmListAddItem(idlist, xmstr, 0);

      XmStringFree(xmstr);

     /*
      *  *** free string item
      */
      tempptr = list[listmin];
      list[listmin] = list[i];
      free(tempptr);
      }

  /*
   *  *** remove any remaining members (DG bug)
   */
   if (count < curlen)
      XmListDeleteItemsPos(idlist, curlen-count, count+1);

   return(E_SUCC);
}

