#include <unistd.h>
#include "tiepts.h"
#include "ltable.h"
#include "mapsup.h"

/*****************************************************************************
 NAME:  map_updatelt

 PURPOSE:  
    Updates parameters to the map labeled table file.

By J. Hemmer, USGS/EROS Data Center
Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to read and process both a map name and a map ID.
*****************************************************************************/
int map_updatelt(void)
{
   struct MATRIX size;          /* matrix structure of labeled table       */
   struct TAB_DEF itab;         /* tabular structure of input labeled table*/
   struct TAB_DEF otab;         /* tabular structure of output labeled table*/
   struct VECTOR *ptr;          /* pointer to a vector of label table      */
   struct VECTOR *nameptr;      /* pointer to name field of labeled table  */
   struct VECTOR *idptr;        /* pointer to id field of labeled table    */
   struct VECTOR *stptr;        /* pointer to state or country field of 
                                   labeled table                           */
   struct VECTOR *unitptr;      /* pointer to units field                  */
   struct VECTOR *ul_yptr;      /* pointer to ul corner y field            */
   struct VECTOR *ul_xptr;      /* pointer to ul corner x field            */
   struct VECTOR *lr_yptr;      /* pointer to lr corner y field            */
   struct VECTOR *lr_xptr;      /* pointer to lr corner x field            */
   struct VECTOR *ver_yptr;     /* pointer to verify pt y field            */
   struct VECTOR *ver_xptr;     /* pointer to verify pt x field            */
   struct VECTOR *npts_yptr;    /* pointer to npts in y field              */
   struct VECTOR *npts_xptr;    /* pointer to npts in x field              */
   struct VECTOR *scaleptr;     /* pointer to scale field                  */
   struct VECTOR *datumptr;     /* pointer to datum field                  */
   struct VECTOR *typeptr;      /* pointer to map type field               */
   struct VECTOR *dateptr;      /* pointer to map date field               */
   struct VECTOR *descptr;      /* pointer to map description field        */

   char   backup[256];          /* backup map filename                     */
   char   fdescbuf[80];         /* Data buffer                             */
   char   ftypebuf[80];         /* Data buffer                             */
   int    status;               /* file IO status                          */
   
  /*
   *  *** create backup file
   */
   strcpy(backup, tie_mapparm.mapfile);
   strcat(backup, ".bak");

   unlink(backup);
   if (rename(tie_mapparm.mapfile, backup) < 0)
      return(E_FAIL);

  /* 
   *  *** Initialize the variables needed.
   */
   itab.nsubs = 0;
   itab.subfile[0] = (char*)malloc(2);
   strcpy(itab.subfile[0],"");
   strcpy(ftypebuf, "MAPFILE");
   itab.ftype = ftypebuf;
   strcpy(fdescbuf, "Map identifier parameter file");
   itab.fdesc = fdescbuf;
   itab.ncol = 6;
   
   otab.nsubs = 0;
   otab.subfile[0] = (char*)malloc(2);
   strcpy(otab.subfile[0],"");
   strcpy(ftypebuf, "MAPFILE");
   otab.ftype = ftypebuf;
   strcpy(fdescbuf, "Map identifier parameter file");
   otab.fdesc = fdescbuf;
   otab.ncol = 6;
   
  /*  
   *  *** Open input label table file  
   *      and determine pointers to fields in the file 
   */ 
   open_tab(&itab, backup, IREAD); 
   get_vector(&itab); 

   nameptr = get_field_ptr(&itab, "NAME");
   idptr = get_field_ptr(&itab, "ID");
   stptr = get_field_ptr(&itab, "STATE_COUNTRY");
   unitptr = get_field_ptr(&itab, "UNITS");
   ul_yptr = get_field_ptr(&itab, "UL_Y");
   ul_xptr = get_field_ptr(&itab, "UL_X");
   lr_yptr = get_field_ptr(&itab, "LR_Y");
   lr_xptr = get_field_ptr(&itab, "LR_X");
   ver_yptr = get_field_ptr(&itab, "VER_Y");
   ver_xptr = get_field_ptr(&itab, "VER_X");
   npts_yptr = get_field_ptr(&itab, "NPTS_Y");
   npts_xptr = get_field_ptr(&itab, "NPTS_X");
   scaleptr = get_field_ptr(&itab, "SCALE");
   datumptr = get_field_ptr(&itab, "DATUM");
   typeptr = get_field_ptr(&itab, "TYPE");
   dateptr = get_field_ptr(&itab, "DATE");
   descptr = get_field_ptr(&itab, "DESC");

  /*  
   *  *** Open output label table file  
   *      and set up output vectors
   */ 
   open_tab(&otab, tie_mapparm.mapfile, IWRITE); 
   put_vector(&otab,"NAME",C,"Map name",1,20,FIELD_SEP2);
   put_vector(&otab,"ID",C,"Map identifier",1,80,FIELD_SEP2);
   put_vector(&otab,"STATE_COUNTRY",C,"State or country identifier",1,80,
      FIELD_SEP2);
   put_vector(&otab,"UNITS",I4,"Coordinate unit code 4=deg 6=xy",1,1,
      FIELD_SEP2);
   put_vector(&otab,"UL_Y",R8,"Upper left Y/latitude coordinate",1,1,
      FIELD_SEP2);
   put_vector(&otab,"UL_X",R8,"Upper left X/longitude coordinate",1,1,
      FIELD_SEP2);
   put_vector(&otab,"LR_Y",R8,"Lower right Y/latitude coordinate",1,1,
      FIELD_SEP2);
   put_vector(&otab,"LR_X",R8,"Lower right X/longitude coordinate",1,1,
      FIELD_SEP2);
   put_vector(&otab,"VER_Y",R8,"Verify pt Y/latitude coordinate",1,1,
      FIELD_SEP2);
   put_vector(&otab,"VER_X",R8,"Verify pt X/longitude coordinate",1,1,
      FIELD_SEP2);
   put_vector(&otab,"NPTS_Y",I4,"Number of tics in Y",1,1,FIELD_SEP2);
   put_vector(&otab,"NPTS_X",I4,"Number of tics in X",1,1,FIELD_SEP2);
   put_vector(&otab,"SCALE",I4,"Map scale in 1:x",1,1,FIELD_SEP2);
   put_vector(&otab,"DATUM",C,"Map datum",1,80,FIELD_SEP2);
   put_vector(&otab,"TYPE",C,"Map type",1,80,FIELD_SEP2);
   put_vector(&otab,"DATE",C,"Map date",1,10,FIELD_SEP2);
   put_vector(&otab,"DESC",C,"Map description",1,80,RECORD_SEP);
   
  /*  
   *  ***   Copy from in to out; when we find the update record
   *        use tie_mapparm info instead of that from itab
   */
   for (;;)
      {
      status = get_record(&itab);

      if (status == E_EOF) 
         break;

      if ((status == E_FAIL)  || (idptr == NULL))
         {
         close_tab(&itab);
         close_tab(&otab);
         return(E_FAIL);
         }

      if ((strcmp(nameptr->ptr.c, tie_mapparm.mapname) == 0) && 
          (strcmp(idptr->ptr.c, tie_mapparm.mapid) == 0))
         {
        /*
         * *** Take info from the current map & update the lt
         */
         ptr = otab.vector; strcpy(ptr->ptr.c, tie_mapparm.mapname);
         ptr = ptr->next; strcpy(ptr->ptr.c, tie_mapparm.mapid);
         ptr = ptr->next; strcpy(ptr->ptr.c, tie_mapparm.mapst_cntry);
         size.nrow = size.ncol = 1;
         ptr = ptr->next; put_matrix(ptr,I4,&size,&tie_mapparm.units);
         ptr = ptr->next; put_matrix(ptr,R8,&size,&tie_mapparm.ul_y);
         ptr = ptr->next; put_matrix(ptr,R8,&size,&tie_mapparm.ul_x);
         ptr = ptr->next; put_matrix(ptr,R8,&size,&tie_mapparm.lr_y);
         ptr = ptr->next; put_matrix(ptr,R8,&size,&tie_mapparm.lr_x);
         ptr = ptr->next; put_matrix(ptr,R8,&size,&tie_mapparm.verify_y);
         ptr = ptr->next; put_matrix(ptr,R8,&size,&tie_mapparm.verify_x);
         ptr = ptr->next; put_matrix(ptr,I4,&size,&tie_mapparm.npts_y);
         ptr = ptr->next; put_matrix(ptr,I4,&size,&tie_mapparm.npts_x);
         ptr = ptr->next; put_matrix(ptr,I4,&size,&tie_mapparm.scale);
         ptr = ptr->next; strcpy(ptr->ptr.c,tie_mapparm.datum);
         ptr = ptr->next; strcpy(ptr->ptr.c,tie_mapparm.maptype);
         ptr = ptr->next; strcpy(ptr->ptr.c,tie_mapparm.mapdate);
         ptr = ptr->next; strcpy(ptr->ptr.c,tie_mapparm.mapdesc);
         }
      else
         {
        /*
         * *** Just copy info from the old map file
         */
         ptr = otab.vector; strcpy(ptr->ptr.c, nameptr->ptr.c);
         ptr = ptr->next; strcpy(ptr->ptr.c, idptr->ptr.c);
         ptr = ptr->next; strcpy(ptr->ptr.c, stptr->ptr.c);
         size.nrow = size.ncol = 1;
         ptr = ptr->next; put_matrix(ptr,I4,&size,unitptr->ptr.i4);
         ptr = ptr->next; put_matrix(ptr,R8,&size,ul_yptr->ptr.r8);
         ptr = ptr->next; put_matrix(ptr,R8,&size,ul_xptr->ptr.r8);
         ptr = ptr->next; put_matrix(ptr,R8,&size,lr_yptr->ptr.r8);
         ptr = ptr->next; put_matrix(ptr,R8,&size,lr_xptr->ptr.r8);
         ptr = ptr->next; put_matrix(ptr,R8,&size,ver_yptr->ptr.r8);
         ptr = ptr->next; put_matrix(ptr,R8,&size,ver_xptr->ptr.r8);
         ptr = ptr->next; put_matrix(ptr,I4,&size,npts_yptr->ptr.i4);
         ptr = ptr->next; put_matrix(ptr,I4,&size,npts_xptr->ptr.i4);
         ptr = ptr->next; put_matrix(ptr,I4,&size,scaleptr->ptr.i4);
         ptr = ptr->next; strcpy(ptr->ptr.c,datumptr->ptr.c);
         ptr = ptr->next; strcpy(ptr->ptr.c,typeptr->ptr.c);
         ptr = ptr->next; strcpy(ptr->ptr.c,dateptr->ptr.c);
         ptr = ptr->next; strcpy(ptr->ptr.c,descptr->ptr.c);
         }

      put_record(&otab);
      }

  /* 
   *  *** Close the labeled table files
   */
   close_tab(&itab);
   close_tab(&otab);
   return(E_SUCC);
}
