#include "tiepts.h"
#include "ltable.h"
#include "mapsup.h"

/*****************************************************************************
NAME:                            MAP_WRITELT

PURPOSE:  Opens and writes the parameters to the output labeled table file.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.0           5/89    D. Steinwand       CSB      New development
  5.02          8/89    D. Steinwand       CSB      Fixed bug for SUN 4
                4/93    L. Huewe           CSB      Modified to allow reading
                                                    and processing of both a
                                                    map name and map ID.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   PROJPRM must be run under TAE

PROJECT:  LAS

ALGORITHM DESCRIPTION:
        Initialize the variables needed.
        Open output label table file.
        Write the parameter values to the output matrix.
        Write the matrix record to the output labeled table file.
        Close the labeled table file
        Return
*****************************************************************************/
void map_writelt
(
   struct TIE_MAPPARMS parms    /* map record                           */
)
{
    struct MATRIX size;             /* matrix structure of label table */
    struct TAB_DEF tab;             /* tabular structure of label table */
    struct VECTOR *ptr;             /* pointer to each vector of label table */
    char fdescbuf[80];              /* Data buffer */
    char ftypebuf[80];              /* Data buffer */


    /* Initialize the variables needed.
    -----------------------------------*/
    tab.nsubs = 0;
    tab.subfile[0] = (char*)malloc(2);
    strcpy(tab.subfile[0],"");
    strcpy(ftypebuf, "MAPFILE");
    tab.ftype = ftypebuf;
    strcpy(fdescbuf, "Map identifier parameter file");
    tab.fdesc = fdescbuf;
    tab.ncol = 6;

    /*  Open output label table file.
    ---------------------------------*/
    open_tab(&tab,parms.mapfile,IUPDATE);

    put_vector(&tab,"NAME",C,"Map name",1,20,FIELD_SEP2);
    put_vector(&tab,"ID",C,"Map identifier",1,80,FIELD_SEP2);
    put_vector(&tab,"STATE_COUNTRY",C,"State or country identifier",1,80,
               FIELD_SEP2);
    put_vector(&tab,"UNITS",I4,"Coordinate unit code 4=deg 6=xy",1,1,
               FIELD_SEP2);
    put_vector(&tab,"UL_Y",R8,"Upper left Y/latitude coordinate",1,1,
               FIELD_SEP2);
    put_vector(&tab,"UL_X",R8,"Upper left X/longitude coordinate",1,1,
               FIELD_SEP2);
    put_vector(&tab,"LR_Y",R8,"Lower right Y/latitude coordinate",1,1,
               FIELD_SEP2);
    put_vector(&tab,"LR_X",R8,"Lower right X/longitude coordinate",1,1,
               FIELD_SEP2);
    put_vector(&tab,"VER_Y",R8,"Verify pt Y/latitude coordinate",1,1,
               FIELD_SEP2);
    put_vector(&tab,"VER_X",R8,"Verify pt X/longitude coordinate",1,1,
               FIELD_SEP2);
    put_vector(&tab,"NPTS_Y",I4,"Number of tics in Y",1,1,FIELD_SEP2);
    put_vector(&tab,"NPTS_X",I4,"Number of tics in X",1,1,FIELD_SEP2);
    put_vector(&tab,"SCALE",I4,"Map scale in 1:x",1,1,FIELD_SEP2);
    put_vector(&tab,"DATUM",C,"Map datum",1,80,FIELD_SEP2);
    put_vector(&tab,"TYPE",C,"Map type",1,80,FIELD_SEP2);
    put_vector(&tab,"DATE",C,"Map date",1,10,FIELD_SEP2);
    put_vector(&tab,"DESC",C,"Map description",1,80,RECORD_SEP);

    /*  Write the parameter values to the output matrix.
    ----------------------------------------------------*/
    size.nrow = 1;
    size.ncol = 20;
    ptr = tab.vector;
    strcpy(ptr->ptr.c, parms.mapname);

    size.nrow = 1;
    size.ncol = 80;
    ptr = ptr->next;
    strcpy(ptr->ptr.c, parms.mapid);

    size.nrow = 1;
    size.ncol = 80;
    ptr = ptr->next;
    strcpy(ptr->ptr.c, parms.mapst_cntry);

    size.ncol = 1;

    ptr = ptr->next;
    put_matrix(ptr,I4,&size,&parms.units);

    ptr = ptr->next;
    put_matrix(ptr,R8,&size,&parms.ul_y);

    ptr = ptr->next;
    put_matrix(ptr,R8,&size,&parms.ul_x);

    ptr = ptr->next;
    put_matrix(ptr,R8,&size,&parms.lr_y);

    ptr = ptr->next;
    put_matrix(ptr,R8,&size,&parms.lr_x);

    ptr = ptr->next;
    put_matrix(ptr,R8,&size,&parms.verify_y);

    ptr = ptr->next;
    put_matrix(ptr,R8,&size,&parms.verify_x);

    ptr = ptr->next;
    put_matrix(ptr,I4,&size,&parms.npts_y);

    ptr = ptr->next;
    put_matrix(ptr,I4,&size,&parms.npts_x);

    ptr = ptr->next;
    put_matrix(ptr,I4,&size,&parms.scale);

    size.ncol = 80;
    ptr = ptr->next;
    strcpy(ptr->ptr.c,parms.datum);

    size.ncol = 80;
    ptr = ptr->next;
    strcpy(ptr->ptr.c,parms.maptype);

    size.ncol = 10;
    ptr = ptr->next;
    strcpy(ptr->ptr.c,parms.mapdate);

    size.ncol = 80;
    ptr = ptr->next;
    strcpy(ptr->ptr.c,parms.mapdesc);

    /*  Write the matrix record to the output labeled table file.
    -------------------------------------------------------------*/
    put_record(&tab);

    /*  Close the labeled table file
    --------------------------------*/
    close_tab(&tab);
}

