#include "tiepts.h"
#include "protos.h"

/*******************************************************************************
NAME: TIE_MESSAGE

PURPOSE: This routine displays the given string in a message box.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development
 2.0            6/93    J.LaVergne      Changed from a message box to an
                                        information box, and added an event
                                        handler to ensure window isn't covered

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void tie_message
(
  Widget parent,             /* Parent shell widget                     */
  char *str                  /* Message string                          */
)
{
  Arg args[10];               /* Variable used to set widget resourcees  */
  int argcnt;                /* Number of widget resources              */
  Widget message;             /* Message widget variable                 */
  XmString xmstr1;            /* Title of the window                     */
  XmString xmstr2;            /* Message displayed in the window         */
  extern void tie_msgraise();
            
  /* Make sure parent checks out
  ------------------------------*/
  if ((parent == NULL) || (!XtIsRealized(parent)))
    parent = Shell_Main;

  /* Create message dialog shell
  ------------------------------*/
  argcnt = 0;
  XtSetArg(args[argcnt],XtNvisual,tie.visual_info[0].visual); argcnt++;
  XtSetArg(args[argcnt],XtNcolormap,tie.colormap); argcnt++;
  XtSetArg(args[argcnt], XmNdialogStyle, XmDIALOG_MODELESS);
           argcnt++;
  XtSetArg(args[argcnt],XmNdialogType, XmDIALOG_INFORMATION); argcnt++;
  XtSetArg(args[argcnt], XmNdialogTitle,
           (xmstr1=XmStringCreateLtoR("Message",
           XmSTRING_DEFAULT_CHARSET))); argcnt++;
  XtSetArg(args[argcnt], XmNmessageString,
           (xmstr2=XmStringCreateLtoR(str,
           XmSTRING_DEFAULT_CHARSET))); argcnt++;

  message = XmCreateInformationDialog(parent,"messagebox",args, argcnt);

  XtAddCallback(message, XmNcancelCallback, tie_msgcancel, (caddr_t)0);
  XmStringFree(xmstr1);
  XmStringFree(xmstr2);

  /* Remove help and cancel buttons
  ---------------------------------*/
  XtUnmanageChild(XmMessageBoxGetChild(message, XmDIALOG_HELP_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(message, XmDIALOG_CANCEL_BUTTON));

  XtManageChild(message);
  XBell(tie.display, 0);

  /* Add an event handler to make sure the message box
     doesn't get covered up by another window
  ----------------------------------------------------*/
  XtAddEventHandler(message,VisibilityChangeMask,FALSE,tie_msgraise,message);
}

/*******************************************************************************
NAME: TIE_MSGCANCEL

PURPOSE: This routine removes and destroys the message box.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development
 2.0            6/93    J.LaVergne      Added a call to remove event handler
                                        that was added in tie_message.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void tie_msgcancel
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
  XtRemoveEventHandler(w,VisibilityChangeMask,FALSE,tie_msgraise,NULL);
  XtDestroyWidget(w);
}

/*******************************************************************************
NAME: TIE_MSGRAISE

PURPOSE: This routine raises the message box if it becomes covered.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            6/93    J.LaVergne      Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void tie_msgraise
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
  XRaiseWindow(tie.display,XtWindow(XtParent((Widget)client)));
}
