#include <stdlib.h>
#include <X11/cursorfont.h>
#include "tiepts.h"
#include "proj.h"
#include "cr.h"
#include "manage_cursor.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine initiazes tiepts based on the given mode of operation:

        Image to Image
        Image to Map
        Image to Vector
        One Search

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by T Rockvam, USGS/EROS Data Center, 3/94.
  - initialize vector filename
Modified by C. Olsen, USGS/EROS Data Center, 8/98.
  - allocate memory for the converted tie points (SRF #1935)

Algorithm Description:
*******************************************************************************/
void mode_init
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];
   int          i;
   XGCValues    values;
   Cursor cursor;

  /*
   *   *** if image to image, reserve space for 2 images
   */
   if (w != NULL)
      {
      XtUnmanageChild(Mode_form);
      XtUnrealizeWidget(Shell_Mode);
      XtDestroyWidget(Shell_Mode);
      }

   tie.mode = (int)client;

  /*
   *  *** define planes for two or one image, respectively
   */
   if ((tie.mode == IM2IM) || (tie.mode == IM2ADRG))
      {
      tie_layout.mem_depth = 6;
      tie.max_plane = 6;
      tie.sview = 3;

      if (tie.mode == IM2ADRG)
         {
         tie.mode = IM2IM;
         tie.flags |= TIE_ADRG;
         tie_adrg = TieMalloc(sizeof(struct TIE_ADRGPARM));
         tie_adrg->nimages = 0;
         tie_adrg->image = NULL;
         }
      }
   else
      {
      tie_layout.mem_depth = 3;
      tie.max_plane = 3;
      tie.sview = 0;
      }

   if (tie.mode == IM2VECT)
      {
      tie_vector = TieMalloc(sizeof(struct TIE_VECTOR));
      tie_vectfile = TieMalloc(VECTOR_FILE_BLOCK_SIZE *
                               sizeof(struct TIE_VECTFILE));

     /*
      *  *** initialize to Digital Chart of the World
      */
      tie_vectfile[0].filename = NULL;
      tie_vectfile[0].inproj = GEO;
      tie_vectfile[0].indatum = CLARKE;
      tie_vectfile[0].inunit = 4;               /* degrees */
      tie_vectfile[0].inzone =  NULZONE;
      tie_vectfile[0].corner[0][0] = 0.0;       /* initialize corner values */
      tie_vectfile[0].corner[0][1] = 0.0;
      tie_vectfile[0].corner[1][0] = 0.0;
      tie_vectfile[0].corner[1][1] = 0.0;
      tie_vectfile[0].corner[2][0] = 0.0;
      tie_vectfile[0].corner[2][1] = 0.0;
      tie_vectfile[0].corner[3][0] = 0.0;
      tie_vectfile[0].corner[3][1] = 0.0;
 
      for (i = 0; i < 15; i++)
         tie_vectfile[0].inparm[i] = 0.0;

      i = 0;
      while (dlg_attrib[i].mincode != 0)
         i++;

      tie_vector->abort     = FALSE;
      tie_vector->nattrib   = i;
      tie_vector->numfiles  = 0;
      tie_vector->maxfile   = VECTOR_FILE_BLOCK_SIZE;
      tie_vector->numlines  = 0;
      tie_vector->maxline   = LINEBLOCK;
      tie_vector->line      = TieMalloc(LINEBLOCK*sizeof(struct TIE_LINE));
      tie_vector->line_att  = TieMalloc(tie_vector->nattrib);
      tie_vector->color_att = TieMalloc(tie_vector->nattrib);
      tie_vector->enabled   = TieMalloc(tie_vector->nattrib);
      tie_vector->align     = FALSE;  /* selection by nodes */
      tie_vector->transform = TRANSLATE;
      tie_vector->view_height = 
      tie_vector->view_width  = 512;
      tie_vector->blend_mask = NULL;
      tie_vector->dcwlib    = -1;
      }
   else if (tie.mode == IM2MAP)
      tie_select.mapcolor = GREEN_COL;

   tie_mem[REFERENCE].imgpix = FALSE;
   tie_mem[SEARCH].imgpix = FALSE;
   layout_allocplanes();

  /*  
   *  *** Allocate memory for ref & srch tie points, elev, 
   *      residuals, pt ids, mapnames, mapids, disabled flags
   */
   tie_select.numalloc = MAX_TIE_POINTS;
   tie_select.elev    = (double *)(malloc(tie_select.numalloc*
                                 sizeof(double)));
   tie_select.rtie    = (void *)(malloc(2*tie_select.numalloc*
                                 sizeof(double)));
   tie_select.crtie    = (void *)(malloc(2*tie_select.numalloc*
                                 sizeof(double)));
   tie_select.stie    = (void *)(malloc(2*tie_select.numalloc*
                                 sizeof(double)));
   tie_select.lsrtie   = (void *)(malloc(2*tie_select.numalloc*
                                 sizeof(double)));
   tie_select.resid   = (void *)(malloc(2*tie_select.numalloc*
                                 sizeof(double)));
   tie_select.pt_id   = (void *)(malloc(tie_select.numalloc*
                                 PTIDLEN*sizeof(char)));
   tie_select.mapname = (void *)(malloc(tie_select.numalloc*
                                 80*sizeof(char)));
   tie_select.mapid   = (void *)(malloc(tie_select.numalloc*
                                 80*sizeof(char)));
   tie_select.disabled= (unsigned char *)(malloc(tie_select.numalloc*
                                 sizeof(unsigned char)));
   if ((tie_select.elev == NULL) || (tie_select.rtie == NULL) || 
       (tie_select.crtie == NULL) || (tie_select.stie == NULL) || 
       (tie_select.lsrtie == NULL) || (tie_select.resid == NULL) || 
       (tie_select.pt_id == NULL) || (tie_select.mapname == NULL) || 
       (tie_select.mapid == NULL) || (tie_select.disabled == NULL))
        {
        printf("Error allocating memory\n");
        exit_tiepts(0);
        }

  /*
   *  *** create all the top level shells
   */
   parm_init();
   Create_Shells();

  /*
   *  *** create sliding rectangle GC
   */
   values.plane_mask = 1;
   tie.rect_gc = XCreateGC(tie.display, RootWindow(tie.display, tie.screen),
                           NULL, &values);
   XSetFunction(tie.display,tie.rect_gc,GXxor);
   XSetLineAttributes(tie.display, tie.rect_gc, 2, LineSolid,
                     CapButt, JoinBevel);
   XSetBackground(tie.display,tie.rect_gc,
               tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
  /*
   *  *** set foreground to all 1's
   */
   if (tie.visual_info[0].visual->class == PseudoColor)
      XSetForeground(tie.display,tie.rect_gc, 0xff);
   else
      XSetForeground(tie.display,tie.rect_gc, 0xffffff);
 
  /*
   *  Activate the main panel and main window shells.
   */
   XtRealizeWidget(Shell_Main);
   XtRealizeWidget(Shell_SrchImg);

   if ((tie.mode == IM2MAP) || (tie.mode == ONESRCH))
      {
      XtSetArg(args[0], XmNmappedWhenManaged, False);
      XtSetValues(Shell_RefImg, args, 1);
      }

   XtRealizeWidget(Shell_RefImg);

   /* set the cursor for the main windows */
   cursor = XCreateFontCursor(tie.display,XC_top_left_arrow);
   set_window_cursor(tie.display,XtWindow(main_form),cursor);
   set_window_cursor(tie.display,XtWindow(Shell_SrchImg),cursor);
   set_window_cursor(tie.display,XtWindow(Shell_RefImg),cursor);

  /*
   *  *** initialize graphics
   */
   graphic_init();
}
