
/*
 *  By J. Hemmer, USGS/EROS Data Center, 4/92.
 *    - Tie Point Selection , v1.0.
 *  Modified by J. LaVergne, USGS/EROS Data Center, 7/93
 *    - Corrected an error in register_guesspt for 63360 scale maps (the 
 *      map width was being calculated incorrectly.
 *  Modified by T Rockvam, USGS/EROS Data Center, 3/94.
 *    - check for datum value entered; added datum options selector
 *  Modified by T Rockvam, USGS/EROS Data Center, 12/94.
 *    - added check for find_deg on angle between 0.0 and -1.0.  The actual 
 *      limit is 0.999998611, which is 59 min, 59.9949 seconds.  Since we 
 *      print %5.2f sec, anything greater will be rounded up when printed.
 *
 */
#include "tiepts.h"
#include "util.h"
#include "ltable.h"
#include <math.h>
#include <stdlib.h>
#include "appres.h"
#include "cr.h"
#include "digsup.h"
#include "tpssup.h"
#include "mapsup.h"
#include "protos.h"

extern TieAppres tie_appres;
/*******************************************************************************
Purpose:
   This routine remove the map parameter panel and save the entered 
   parameters to a map file.

By J. Hemmer, USGS/EROS Data Center, 4/91
Modified by L. Huewe, USGS/EROS Data Center, 4/93
    - Modified to read and process both the map name and the map ID from the
      map catalog file.

Algorithm Description:
*******************************************************************************/
void mapparm_destroy
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Arg  args[1];                /* widget arguments                     */
    struct TIE_MAPPARMS parms;   /* used to determine if a new map       */
    struct TIE_MAPLIST*  temp;   /* temp pointer in linked list          */
    char         *mapfile;       /* mapfile entered in text item         */
    char         *mapnameid;     /* map name and ID entered in text item */
    char         *mapname;       /* map name                             */
    char         *mapid;         /* map id                               */
    char         *tmp;           /* temp pointer for extracting items    */
    char         dir[CMLEN],
                 rt_fname[CMLEN],
                 ext[CMLEN];
    Widget       coord;          /* selected coord. units                */
    static Widget rough_accept = NULL;
    size_t       idpos;          /* map id position in the string        */

    double       in_out_proj_coef[15]; /* projection coefficients 
                                          needed for datum conversion    */
    double       out_x;          /* Output X projection coordinate       */
    double       out_y;          /* Output Y projection coordinate       */
    int          active;
    int          index;          /* variable used for looping            */
    int          status;
    long         in_out_proj_code; /* input and output projection code
                                      needed for datum conversion        */
    long         in_out_zone_code; /* input and output zone code needed 
                                      for datum conversion               */
    long         in_out_proj_units; /* input and output projection units 
                                       needed for datum conversion       */
    long         prtprm[2];      /* Write out projection parms to a
                                    file or terminal (TRUE or FALSE)
                                    prtprm[GEO_TERM], prtprm[GEO_FILE]   */


    /*
     *  *** if accept or cancel, remove panel
     */
    if ((client != NULL) && 
            ((strcmp((char*)client, "CANCEL") == 0) || 
             (strcmp((char*)client, "ACCEPT")==0)))
    {
        /*
         *  *** if map was previously registered, restore event handler
         */
        if (tie.flags & TIE_SELECTING)
        {
            XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,
                    FALSE, cursor_select,NULL);

            XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,
                    FALSE, fullres_button,NULL);
        }

        /*
         *  *** if cancel, do not write to map file
         */
        if (strcmp((char*)client, "CANCEL") == 0)
        {
            XtUnrealizeWidget(Shell_MParm);
            XtSetSensitive(main_register_button, True);
            return;
        }
    }

    mapfile = XmTextGetString(reg_mapfile_text);
    mapnameid   = XmTextGetString(reg_mapid_text);
    if (strcmp(mapnameid, "") == 0)
    {
        tie_message(Shell_MParm,"Must enter map id");
        return;
    }

    /*
     * *** Parse the map id and map name.  It is in the form mapname@mapid
     */
    idpos = strcspn(mapnameid,"@");
    if (idpos == strlen(mapnameid))
    {
        mapid = (char *)malloc(2);
        strcpy(mapid,"");
        mapname = TieMalloc(strlen(mapnameid)+1);
        strcpy(mapname,mapnameid);
    }
    else
    {
        mapname = TieMalloc(idpos);
        strncpy(mapname,mapnameid,idpos-1);
        mapname[idpos-1] = '\0';
        mapid = TieMalloc(strlen(mapnameid)-idpos);
        strncpy(mapid,mapnameid+idpos+1,strlen(mapnameid)-idpos-1);
        mapid[strlen(mapnameid)-idpos-1] = '\0';
    }

    /*
     *  *** copy string values from text items
     */
    strcpy(tie_mapparm.mapfile, mapfile);
    strcpy(tie_mapparm.mapname, mapname);
    strcpy(tie_mapparm.mapid, mapid);
    strcpy(tie_mapparm.mapst_cntry, "");
    /* set up mapst_cntry if we know it */
    temp = tie_maplist;
    while (temp != NULL)
    {
        if (strcmp(temp->name,mapname) == 0)
        {
            strcpy(tie_mapparm.mapst_cntry, temp->state);
            break;
        }
        temp = temp->next;
    }

    strcpy(tie_mapparm.datum, XmTextGetString(reg_datum_text));

    if ((strcmp(tie_parms.datumreq, "Required") == 0) &&
            (tie_mapparm.datum[0] == NULL))
    {
        tie_message(Shell_MParm,"Must enter datum value");
        XmProcessTraversal(reg_datum_text, XmTRAVERSE_CURRENT);
        return;
    }

    if (strcmp(tie_parms.datumreq, "Required") == 0)
    {
        /* 
         *  *** Extract datum number from ( ) at beginning of string 
         */
        if (tie_mapparm.datum[0] == '(') 
            tie_mapparm.current_datum = atoi(&tie_mapparm.datum[1]);
        else
        {
            tie_message(Shell_MParm,
                    "Unable to retrieve datum number from input datum value.\n"
                    "Re-select the datum from the datum selection box");
            return;
        }

        /*
         *  *** Check if the datum value was changed and set datum flag. 
         */
        if ((tie_mapparm.out_datum_code != 0) &&
           (tie_mapparm.current_datum != tie_mapparm.out_datum_code))
            tie_mapparm.first_datum_selected = FALSE;
        else
        {
            tie_mapparm.first_datum_selected = TRUE;

            /* Since this is the first datum chosen, this is the one that will
               be applied to all maps in the IM2MAP digitizer mode */
            tie_mapparm.out_datum_code = tie_mapparm.current_datum;
        }
    }
    else
        /* Set to TRUE so a datum conversion will on occur */
        tie_mapparm.first_datum_selected = TRUE;

    strcpy(tie_mapparm.mapdate, XmTextGetString(reg_date_text));
    strcpy(tie_mapparm.mapdesc, XmTextGetString(reg_desc_text));
    strcpy(tie_mapparm.maptype, XmTextGetString(reg_maptype_text));

    /*
     *  *** set coordinate units
     */
    XtSetArg(args[0], XmNmenuHistory, &coord);
    XtGetValues(reg_coord_option, args, 1);

    /* The user can either enter the points in DMS, Projection or User Defined -
       which is actually degrees */
    if (coord == reg_coord_geog)
        tie_mapparm.units = MAP_DEGREES;
    else if (coord == reg_coord_proj)
    {
        tie_mapparm.units = MAP_PROJ;
        if (tie_mapparm.mapdesc[0] == NULL)
        {
            tie_message(Shell_MParm,"Must enter projection description");
            XmProcessTraversal(reg_desc_text, XmTRAVERSE_CURRENT);
            return;
        }
    }
    /*
     * If using user-defined coords, we need to redo the rt going from user-def 
     * coords to srch img (previous rt goes from degrees lat/long to srch img)
     * This means if RT_DDR, ignore the previous pts, see if there is a .rt
     * file, otherwise manually select 4 pts for a new rt.
     */
    else
    {
        if ((client != NULL) && (strcmp((char*)client, "REGISTER") == 0))
        {
            if (tie_rt.calc_rt == RT_DDR)
            {
                if (caution(Shell_MParm,
                    "The Rough Transformation must be recalculated "
                    "for\nuser-defined coordinates.  All previously "
                    "selected\npoints will be lost.  Do you want to continue?") 
                         == 2)
                    return;

                tie_rt.calc_rt = RT_SEL;
                tie_mapparm.units = MAP_USERDEF;
                c_splitname(
                  tie_planes[tie_composite[SEARCH].rgbplanes[RED]-1].image_name,
                  dir,rt_fname,ext);
                strcat(rt_fname,".rt");
                tie_rt.rtfp = fopen(rt_fname,"r+");
                if (tie_rt.rtfp != NULL)
                {
                    tie_select.seltie = tie_select.numtie = 0;
                    tie_select.genpt = 1;
                    tps_update();
                    status = fscanf(tie_rt.rtfp,
                            "%d,%lf,%lf,%d,%d,%d,%lf,%lf,%d,%d,%d,%*s",
                            &active,&tie_select.rtie[tie_select.numtie][0],
                            &tie_select.rtie[tie_select.numtie][1], 
                            &tie_rt.rtrzm[tie_select.numtie],
                            &tie_rt.rtroff[tie_select.numtie][0],
                            &tie_rt.rtroff[tie_select.numtie][1],
                            &tie_select.stie[tie_select.numtie][0],
                            &tie_select.stie[tie_select.numtie][1],
                            &tie_rt.rtszm[tie_select.numtie],
                            &tie_rt.rtsoff[tie_select.numtie][0],
                            &tie_rt.rtsoff[tie_select.numtie][1]);


                    while (status != EOF)
                    {
                        if (active)
                        {
                            tie_select.pt_id[tie_select.numtie][0] = '\0';
                            if (tie_select.numtie == 4)
                                break;
                            tie_select.numtie++;
                            tie_select.seltie++;
                        }
                        status = fscanf(tie_rt.rtfp,
                                "%d,%lf,%lf,%d,%d,%d,%lf,%lf,%d,%d,%d,%*s",
                                &active,&tie_select.rtie[tie_select.numtie][0],
                                &tie_select.rtie[tie_select.numtie][1],
                                &tie_rt.rtrzm[tie_select.numtie],
                                &tie_rt.rtroff[tie_select.numtie][0],
                                &tie_rt.rtroff[tie_select.numtie][1],
                                &tie_select.stie[tie_select.numtie][0],
                                &tie_select.stie[tie_select.numtie][1],
                                &tie_rt.rtszm[tie_select.numtie],
                                &tie_rt.rtsoff[tie_select.numtie][0],
                                &tie_rt.rtsoff[tie_select.numtie][1]);
                    }

                    if (tie_select.numtie >= 4)
                    {
                        tie_rt.degree = tie_parms.order;
                        status = tps_Qfit(tie_select.numtie, tie_select.rtie,
                                tie_select.stie,tie_select.disabled,
                                &tie_rt.degree, tie_rt.line_coef,
                                tie_rt.samp_coef);
                        if (status != E_SUCC)
                        {
                            tie_select.numtie = 0;
                            tie_message(Shell_SrchImg,
                               "The next four points selected will be used to "
                               "calculate\nthe rough transformation.  These "
                               "points should be\nselected from the corners of "
                               "the Search Image.");
                        }
                        else
                        {
                            if (!rough_accept)
                                rough_accept = Createrough_form(Shell_SrchImg);
                            XtManageChild(rough_accept);
                            edit_compute();
                            for (idpos = 0; idpos < tie_select.numtie; idpos++)
                                display_pt(idpos);
                        }
                    }
                    else
                    {
                        /*
                         * Less than 4 point in rt file
                         */
                        sprintf(rt_fname,"You must select %d more points "
                            "to\ncomplete the rough transformation.",
                            4-tie_select.numtie);
                        tie_message(Shell_SrchImg,rt_fname);
                    }
                }
                else
                {
                    tie_select.seltie = tie_select.numtie = 0;
                    tie_select.genpt = 1;
                    tps_update();
                    tie_rt.rtfp = fopen(rt_fname,"w");
                    if (tie_rt.rtfp == NULL)
                    {
                        tie_message(Shell_MParm,
                                "Error opening rough transformation file");
                        return;
                    }

                    tie_message(Shell_SrchImg,"The next four points selected "
                              "will be used to calculate\nthe rough "
                              "transformation.  These points should be\n"
                              "selected from the corners of the Search Image.");
                }
            } /* RT_DDR */

            if (XtIsRealized(Shell_Getmap))
            {
                XtUnmanageChild(Getmap_form);
                XtDestroyWidget(Getmap_form);
                XtUnrealizeWidget(Shell_Getmap);
                XtSetSensitive(main_map_button, True);
                if (tie_maplist != NULL)
                    maplist_free();
                XClearArea(tie.display,XtWindow(search_image_bulletinboard),
                        0,0,0,0,TRUE);
                tie.flags |= TIE_RECTCUR;
            }

            /*
             * Set up event handlers to select from the srch image
             */
            XtRemoveEventHandler(search_image_bulletinboard,ButtonReleaseMask,
                    FALSE,maplist_select,NULL);
            XtAddEventHandler(search_image_bulletinboard,ButtonReleaseMask,
                    FALSE,fullres_button,NULL);
        }

        tie_mapparm.units = MAP_USERDEF;
    } /* MAP_USERDEF */

    /*
     *   *** convert coordinates from "dd mm ss.ss" to packed format
     */
    if (((int)strlen(XmTextGetString(reg_ul_x)) <= 0) ||
            ((int)strlen(XmTextGetString(reg_ul_y)) <= 0) ||
            ((int)strlen(XmTextGetString(reg_lr_x)) <= 0) ||
            ((int)strlen(XmTextGetString(reg_lr_y)) <= 0) ||
            ((int)strlen(XmTextGetString(reg_verify_x)) <= 0) ||
            ((int)strlen(XmTextGetString(reg_verify_y)) <= 0))
    {
        tie_message(verify_dialog_shell,"Incomplete corner and verify points");
        return;
    }
    if (tie_mapparm.units == MAP_DEGREES)
    {
        tie_mapparm.ul_x     = map_dms2deg(XmTextGetString(reg_ul_x));
        tie_mapparm.ul_y     = map_dms2deg(XmTextGetString(reg_ul_y));
        tie_mapparm.lr_x     = map_dms2deg(XmTextGetString(reg_lr_x));
        tie_mapparm.lr_y     = map_dms2deg(XmTextGetString(reg_lr_y));
        tie_mapparm.verify_x = map_dms2deg(XmTextGetString(reg_verify_x));
        tie_mapparm.verify_y = map_dms2deg(XmTextGetString(reg_verify_y));
    }
    else
    {
        sscanf(XmTextGetString(reg_ul_x),     "%lf", &tie_mapparm.ul_x);
        sscanf(XmTextGetString(reg_ul_y),     "%lf", &tie_mapparm.ul_y);
        sscanf(XmTextGetString(reg_lr_x),     "%lf", &tie_mapparm.lr_x);
        sscanf(XmTextGetString(reg_lr_y),     "%lf", &tie_mapparm.lr_y);
        sscanf(XmTextGetString(reg_verify_x), "%lf", &tie_mapparm.verify_x);
        sscanf(XmTextGetString(reg_verify_y), "%lf", &tie_mapparm.verify_y);
    }

    /*
     * Check if the datum has changed from the first datum selected.
     * If so, convert the coordinates from the current datum to the
     * first datum selected.
     */
    if (tie_mapparm.first_datum_selected == FALSE) 
    {
         /*  The input and output value will always be in degrees */
        in_out_proj_code = GEO;
        in_out_zone_code = 62;
        in_out_proj_units = DEGREE;
        for (index = 0; index < 15; index++)
            in_out_proj_coef[index] = 0.0;
     
        /* Do not write out the projection parms to a file or terminal */
        prtprm[0] = prtprm[1] = FALSE;

        /*
         *  Initialize projection and datum information
         */

        status = c_transinit(&in_out_proj_code, &in_out_proj_units,
                  &in_out_zone_code, &tie_mapparm.current_datum,
                  in_out_proj_coef, &in_out_proj_code, &in_out_proj_units,
                  &in_out_zone_code, &tie_mapparm.out_datum_code,
                  in_out_proj_coef, prtprm, "");

        if (status != E_SUCC)
        {
            tie_message(Shell_Manual,
                       "Error initializing projection parameters\n ");
            return;
        }

        /*  Convert the upper left coordinates */
        status = c_trans(&in_out_proj_code,&in_out_proj_units,
                  &in_out_proj_code,&in_out_proj_units, &tie_mapparm.ul_x,
                  &tie_mapparm.ul_y,&out_x, &out_y);

        if (status != E_SUCC)
        {
            tie_message(Shell_Manual,
                   "Error converting projection coordinates\n ");
            return;
        }

         /* copy results back to tie_mapparm */
         tie_mapparm.ul_x = out_x;
         tie_mapparm.ul_y = out_y;

        /*  Convert the lower right coordinates */
        status = c_trans(&in_out_proj_code,&in_out_proj_units,
                  &in_out_proj_code,&in_out_proj_units, &tie_mapparm.lr_x,
                  &tie_mapparm.lr_y,&out_x, &out_y);

        if (status != E_SUCC)
            tie_message(Shell_Manual,
                   "Error converting projection coordinates\n ");

         /* copy results back to tie_mapparm */
         tie_mapparm.lr_x = out_x;
         tie_mapparm.lr_y = out_y;


       /*  Convert the verify points */
        status = c_trans(&in_out_proj_code,&in_out_proj_units,
                  &in_out_proj_code,&in_out_proj_units, &tie_mapparm.verify_x,
                  &tie_mapparm.verify_y,&out_x, &out_y);

        if (status != E_SUCC)
            tie_message(Shell_Manual,
                   "Error converting projection coordinates\n ");

         /* copy results back to tie_mapparm */
         tie_mapparm.verify_x = out_x;
         tie_mapparm.verify_y = out_y;
    } 

    /*
     *  *** store scale as integer values
     */
    tmp = XmTextGetString(reg_scale_text);
    if(((int)strlen(tmp)<= 0) || (sscanf(tmp+2, "%d", &tie_mapparm.scale) != 1))
    {
        tie_message(Shell_MParm,"Must enter scale value");
        XmProcessTraversal(reg_scale_text, XmTRAVERSE_CURRENT);
        return;
    }
    XtFree(tmp);

    /*
     *  *** number of points along the x axis
     */
    tmp = XmTextGetString(reg_npts_x);
    if(((int)strlen(tmp) <= 0) || (sscanf(tmp, "%d", &tie_mapparm.npts_x) != 1))
    {
        tie_message(Shell_MParm,"Must enter number of x points");
        XmProcessTraversal(reg_npts_x, XmTRAVERSE_CURRENT);
        return;
    }
    XtFree(tmp);

    /*
     *  *** number of points along the y axis
     */
    tmp = XmTextGetString(reg_npts_y);
    if(((int)strlen(tmp) <= 0) || (sscanf(tmp, "%d", &tie_mapparm.npts_y) != 1))
    {
        tie_message(Shell_MParm,"Must enter number of y points");
        XmProcessTraversal(reg_npts_y, XmTRAVERSE_CURRENT);
        return;
    }

    tie_mapparm.inc_x =  fabs(tie_mapparm.ul_x - tie_mapparm.lr_x) /
        ((double)(tie_mapparm.npts_x - 1));
    tie_mapparm.inc_y = -fabs(tie_mapparm.ul_y - tie_mapparm.lr_y) /
        ((double)(tie_mapparm.npts_y - 1));
    XtFree(tmp);

    /*
     *  *** if this map name and ID do not exist in the file, save parameters
     */
    if ((strcmp(mapfile, "") != 0) && (strcmp(mapname, "") != 0))
    {
        if (map_read(mapfile, mapname, mapid, &parms) != E_SUCC)
            map_writelt(tie_mapparm);
        else if (mapparm_changed(parms))
            if (map_updatelt() != E_SUCC)
                tie_message(Shell_MParm, "Could not update Map File");
    }

    if ((client != NULL) && (strcmp((char*)client, "REGISTER") == 0))
    {
         /* 
          *  *** Open digitizer commnunication port
          */
        if ((tie_mapparm.digid == NULL) && 
            ((tie_mapparm.portfd = dig_open(tie_parms.port)) != NULL))
        {
            tie_mapparm.digid = XtAppAddInput(tie.context, tie_mapparm.portfd, 
                    (XtPointer)XtInputReadMask,dig_spark,NULL);
        }
        register_map();
    }

    XtFree(mapfile);
    XtFree(mapnameid);
    free(mapname);
    free(mapid);
    XtUnrealizeWidget(Shell_MParm);
    XtSetSensitive(main_register_button, True);
}
 
/*******************************************************************************
Purpose:
   This routine checks if any map parameters have changed.  

By J. Hemmer, USGS/EROS Data Center, 4/91

Algorithm Description:
*******************************************************************************/
int mapparm_changed
(
    struct TIE_MAPPARMS parms          /* parameters from map file */
)
{
    if (strcmp(parms.datum, tie_mapparm.datum) ||
            strcmp(parms.maptype, tie_mapparm.maptype) ||
            strcmp(parms.mapdate, tie_mapparm.mapdate) ||
            strcmp(parms.mapdesc, tie_mapparm.mapdesc) ||
            (parms.units != tie_mapparm.units) ||
            (parms.ul_x != tie_mapparm.ul_x) ||
            (parms.ul_y != tie_mapparm.ul_y) ||
            (parms.lr_x != tie_mapparm.lr_x) ||
            (parms.lr_y != tie_mapparm.lr_y) ||
            (parms.verify_x != tie_mapparm.verify_x) ||
            (parms.verify_y != tie_mapparm.verify_y) ||
            (parms.npts_x != tie_mapparm.npts_x) ||
            (parms.npts_y != tie_mapparm.npts_y) ||
            (parms.scale != tie_mapparm.scale))
        return(TRUE);
    else
        return(FALSE);
}

/*******************************************************************************
Purpose:
   Initialize the map paramters to their default values.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_init(void)
{
    tie_mapparm.ul_x      = 0.0;
    tie_mapparm.ul_y      = 0.0;
    tie_mapparm.lr_x      = 0.0;
    tie_mapparm.lr_y      = 0.0;
    tie_mapparm.inc_x     = 0.0;
    tie_mapparm.inc_y     = 0.0;
    tie_mapparm.verify_x  = 0.0;
    tie_mapparm.verify_y  = 0.0;
    tie_mapparm.units     = MAP_DEGREES;
    tie_mapparm.npts_x    = 0;
    tie_mapparm.npts_y    = 0;
    tie_mapparm.scale     = 24000;
    strcpy(tie_mapparm.datum,   ""); 
    strcpy(tie_mapparm.mapname,   "");
    strcpy(tie_mapparm.maptype, "Quadrangle");
    strcpy(tie_mapparm.mapdate, "");
    strcpy(tie_mapparm.mapdesc, "");
    strcpy(tie_mapparm.mapfile, "");
    strcpy(tie_mapparm.catalog, "");
    tie_mapparm.registered = FALSE;
    tie_mapparm.outline[0][0] = -1;
    tie_mapparm.order = FIRST_ORDER;
    tie_mapparm.digid = NULL;
    tie_mapparm.first_datum_selected = TRUE;
}

/*******************************************************************************
Purpose:
   Set all map paramters items to their predefined values.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_setup(void)
{
    char   buf[100];

    /*
     *  *** if selecting tie points, disable event handler
     */
    if (tie.flags & TIE_SELECTING)
    {
        tie_select.curref = FALSE;
        XtRemoveEventHandler(ref_image_bulletinboard,ButtonReleaseMask,
                FALSE, cursor_select,NULL);

        XtRemoveEventHandler(ref_image_bulletinboard,ButtonReleaseMask,FALSE,
                fullres_button,NULL);
    }

    XmTextSetString(reg_maptype_text, tie_mapparm.maptype);

    sprintf(buf, "1:%d", tie_mapparm.scale);
    XmTextSetString(reg_scale_text, buf);

    if (strcmp(tie_mapparm.mapfile, "") != 0)
        XmTextSetString(reg_mapfile_text, tie_mapparm.mapfile);
}

/*******************************************************************************
Purpose:
   Set the type of map from the option menu.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_maptype
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XmTextSetString(reg_maptype_text, (char*)client);
}

/*******************************************************************************
Purpose:
   Set the map scale from the option menu.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_scale
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XmTextSetString(reg_scale_text, (char*)client);
}

/*******************************************************************************
Purpose:
   Display the map file requestor panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_mapfile_button
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Map_fileselector = Createmap_fileselector(Shell_MParm);
    XtSetSensitive(reg_mapfile_button, False);
    XtManageChild(Map_fileselector);
}

/*******************************************************************************
Purpose:
   Display the datum requestor panel.

By T. Rockvam, USGS/EROS Data Center, 3/94

Algorithm Description:
*******************************************************************************/
void register_datum_show
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    Datum_selector = Createdatum_selector(Shell_MParm);
    XtSetSensitive(reg_datum_button, False);
    XtManageChild(Datum_selector);
}

/*******************************************************************************
Purpose:
   Select a Datum value from the selector

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_datum_select
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    char *datumvalue;

    XmFileSelectionBoxCallbackStruct *acs =
        (XmFileSelectionBoxCallbackStruct*)call_data;

    if ((int)client == TRUE)
    {
        /*
         *  Retrieve the datum value 
         */
        XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&datumvalue);

        /*
         *  Set the text to this value
         */
        XmTextSetString(reg_datum_text, datumvalue);
        XtFree(datumvalue);
    }

    /*
     *  Resensitize the button and remove the selector
     */
    XtSetSensitive(reg_datum_button, True);
    XtDestroyWidget(Datum_selector);
   
}

/*******************************************************************************
Purpose:
   Define and display the map id requestor panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_mapid_button
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    char    *mapfile;

    mapfile = XmTextGetString(reg_mapfile_text);
    if (strcmp(mapfile, "") == 0)
        return;

    Mapid_selector = Createmapid_selector(Shell_MParm);

    if (reg_mapid_list != NULL)
        map_getidlist(mapfile, reg_mapid_list);

    XtSetSensitive(reg_mapid_button, False);
    XtManageChild(Mapid_selector);
    XtFree(mapfile);
}

/*******************************************************************************
Purpose:
   Get selected map namd and id from requestor panel.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to read and process a map name and map ID.

Algorithm Description:
*******************************************************************************/
void register_mapid_ok
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    size_t       idpos;          /* Position of map id in string.            */

    char         *mapfile;       /* Map file name.                           */
    char         *nameid;        /* Map name and id.                         */
    char         *mapid;         /* Map id.                                  */
    char         *mapname;       /* Map name.                                */

    XmCommandCallbackStruct *acs = (XmCommandCallbackStruct*)call_data;

    /*
     *  Retrieve the id name from the map file. 
     */
    XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&nameid);

    XmTextSetString(reg_mapid_text, nameid);
    XtSetSensitive(reg_mapid_button, True);
    XtDestroyWidget(Mapid_selector);

    if ((int)strlen(nameid) > 0)
    {
        /*
         * *** Parse the map id and map name.
         */
        idpos = strcspn(nameid,"@");
        if (idpos == strlen(nameid))
        {
            mapid = (char *)malloc(2);
            strcpy(mapid,"");
            mapname = TieMalloc(strlen(nameid)+1);
            strcpy(mapname,nameid);
        }
        else
        {
            mapname = TieMalloc(idpos);
            strncpy(mapname,nameid,idpos-1);
            mapname[idpos-1] = '\0';
            mapid = TieMalloc(strlen(nameid)-idpos);
            strncpy(mapid,nameid+idpos+1,strlen(nameid)-idpos-1);
            mapid[strlen(nameid)-idpos-1] = '\0';
        }
        mapfile = XmTextGetString(reg_mapfile_text);
        get_map_rec(mapfile, mapname, mapid);
        XtFree(mapfile);
        XtFree(mapname);
        XtFree(mapid);
    }

    XtFree(nameid);
}

/*******************************************************************************
Purpose:
   Cancel map id requestor.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_mapid_cancel
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    XtSetSensitive(reg_mapid_button, True);
    XtDestroyWidget(Mapid_selector);
}

/*******************************************************************************
Purpose:
   Skip to next text item.  If map id and map file are set, 
   load remaining parameters. 

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Cetner, 493
  - Modified to read and process both the map name and map ID.

Algorithm Description:
*******************************************************************************/
void register_mapparm_text
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    int       i;                 /* Looping index.                           */
    size_t    idpos;             /* Map id position.                         */

    char      *mapfile;          /* Map file name.                           */
    char      *mapnameid;        /* Map name and id.                         */
    char      *mapname;          /* Map name.                                */
    char      *mapid;            /* Map id.                                  */

    XmProcessTraversal(Mapparm_form, XmTRAVERSE_NEXT_TAB_GROUP);

    /*
     *  *** if file and id are entered, load from map file
     */
    if ((w == reg_mapfile_text) || (w == reg_mapid_text))
    {
        mapfile = XmTextGetString(reg_mapfile_text);
        mapnameid   = XmTextGetString(reg_mapid_text);

        if ((strcmp(mapfile, "") != 0) && 
                (strcmp(mapnameid, "") != 0))
        {
            if (!access(mapfile, 0))
            {
                /*
                 * *** Parse the map id and map name.
                 */
                idpos = strcspn(mapnameid,"@");
                if (idpos == strlen(mapnameid))
                {
                    mapid = (char *)malloc(2);
                    strcpy(mapid,"");
                    mapname = TieMalloc(strlen(mapnameid)+1);
                    strcpy(mapname,mapnameid);
                }
                else
                {
                    mapname = TieMalloc(idpos);
                    strncpy(mapname,mapnameid,idpos-1);
                    i = idpos-2;
                    while(mapname[i--] == ' ');
                    mapname[i+2]='\0';
                    mapid = TieMalloc(strlen(mapnameid)-idpos);
                    strncpy(mapid,mapnameid+idpos+1,strlen(mapnameid)-idpos-1);
                    mapid[strlen(mapnameid)-idpos-1] = '\0';
                }
                get_map_rec(mapfile, mapname, mapid);
                free(mapname);
                free(mapid);
            }
        }
        XtFree(mapfile);
        XtFree(mapnameid);
    }
    else if (w == reg_ul_x)
    {
        if ((int)strlen(XmTextGetString(reg_scale_text)) > 2)
            register_guesspt();
    }
}


/*******************************************************************************
Purpose:
   Get a map record from the current file and id.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Cetner, 4/93
  - Modified to read and process both a map name and a map ID.

Algorithm Description:
*******************************************************************************/
void get_map_rec
(
    char *mapfile,
    char *mapname,
    char *mapid
)
{
    Arg    args[1];      /* resource argument variable                 */
    char   dms[20];
    char   tmp[20];
    char   *nameid;      /* Map name and id.                           */

    struct TIE_MAPPARMS  parms;

    /*
     *  *** if invalid file or id, return
     */
    if (map_read(mapfile, mapname, mapid, &parms) != E_SUCC)
    {
        XBell(tie.display, 0);
        return;
    }

    XmTextSetString(reg_date_text,    parms.mapdate);
    XmTextSetString(reg_desc_text,    parms.mapdesc);
    XmTextSetString(reg_maptype_text, parms.maptype);
    nameid = TieMalloc(strlen(mapname) + strlen(mapid) + 3);
    strcpy(nameid,mapname);
    if (strcmp(mapid,"") != 0)
    {
        strcat(nameid," @");
        strcat(nameid,mapid);
    }
    XmTextSetString(reg_mapid_text, nameid); 

    /*
     *  *** set coordinate units
     */
    tie_mapparm.units = parms.units;
    if (tie_mapparm.units == MAP_USERDEF)
        XtSetArg(args[0], XmNmenuHistory, reg_coord_user);
    else if (tie_mapparm.units == MAP_PROJ)
        XtSetArg(args[0], XmNmenuHistory, reg_coord_proj);
    else
        XtSetArg(args[0], XmNmenuHistory, reg_coord_geog);
    XtSetValues(reg_coord_option, args, 1);

    /*
     *  *** upper left x
     */
    if (tie_mapparm.units == MAP_DEGREES)
    {
        if ((parms.ul_x < 0.0) && (parms.ul_x > -0.999998611))
            sprintf(dms, " -0 %2d %4.2f",
                    find_min(parms.ul_x),
                    find_sec(parms.ul_x));
        else
            sprintf(dms, "%3d %2d %4.2f", find_deg(parms.ul_x), 
                    find_min(parms.ul_x),
                    find_sec(parms.ul_x));
    }
    else
        sprintf(dms, "%12.2f", parms.ul_x);

    XmTextSetString(reg_ul_x, dms);

    /*
     *  *** upper left y
     */
    if (tie_mapparm.units == MAP_DEGREES)
    {
        if ((parms.ul_y < 0.0) && (parms.ul_y > -0.999998611))
            sprintf(dms, " -0 %2d %4.2f",
                    find_min(parms.ul_y),
                    find_sec(parms.ul_y));
        else
            sprintf(dms, "%3d %2d %4.2f", find_deg(parms.ul_y), 
                    find_min(parms.ul_y),
                    find_sec(parms.ul_y));
    }
    else
        sprintf(dms, "%12.2f", parms.ul_y);

    XmTextSetString(reg_ul_y, dms);

    /*
     *  *** lower right x
     */
    if (tie_mapparm.units == MAP_DEGREES)
    {
        if ((parms.lr_x < 0.0) && (parms.lr_x > -0.999998611))
            sprintf(dms, " -0 %2d %4.2f",
                    find_min(parms.lr_x),
                    find_sec(parms.lr_x));
        else
            sprintf(dms, "%3d %2d %4.2f", find_deg(parms.lr_x), 
                    find_min(parms.lr_x),
                    find_sec(parms.lr_x));
    }
    else
        sprintf(dms, "%12.2f", parms.lr_x);

    XmTextSetString(reg_lr_x, dms);

    /*
     *  *** lower right y
     */
    if (tie_mapparm.units == MAP_DEGREES)
    {
        if ((parms.lr_y < 0.0) && (parms.lr_y > -0.999998611))
            sprintf(dms, " -0 %2d %4.2f",
                    find_min(parms.lr_y),
                    find_sec(parms.lr_y));
        else
            sprintf(dms, "%3d %2d %4.2f", find_deg(parms.lr_y), 
                    find_min(parms.lr_y),
                    find_sec(parms.lr_y));
    }
    else
        sprintf(dms, "%12.2f", parms.lr_y);

    XmTextSetString(reg_lr_y, dms);

    /*
     *  *** verify x
     */
    if (tie_mapparm.units == MAP_DEGREES)
    {
        if ((parms.verify_x < 0.0) && (parms.verify_x > -0.999998611))
            sprintf(dms, " -0 %2d %4.2f",
                    find_min(parms.verify_x),
                    find_sec(parms.verify_x));
        else
            sprintf(dms, "%3d %2d %4.2f", find_deg(parms.verify_x),
                    find_min(parms.verify_x),
                    find_sec(parms.verify_x));
    }
    else
        sprintf(dms, "%12.2f", parms.verify_x);

    XmTextSetString(reg_verify_x, dms);

    /*
     *  *** verify y
     */
    if (tie_mapparm.units == MAP_DEGREES)
    {
        if ((parms.verify_y < 0.0) && (parms.verify_y > -0.999998611))
            sprintf(dms, " -0 %2d %4.2f",
                    find_min(parms.verify_y),
                    find_sec(parms.verify_y));
        else
            sprintf(dms, "%3d %2d %4.2f", find_deg(parms.verify_y),
                    find_min(parms.verify_y),
                    find_sec(parms.verify_y));
    }
    else
        sprintf(dms, "%12.2f", parms.verify_y);

    XmTextSetString(reg_verify_y, dms);

    XmTextSetString(reg_datum_text, parms.datum);

    sprintf(tmp, "1:%d", parms.scale);
    XmTextSetString(reg_scale_text, tmp);

    sprintf(tmp, "%d", parms.npts_x);
    XmTextSetString(reg_npts_x, tmp);

    sprintf(tmp, "%d", parms.npts_y);
    XmTextSetString(reg_npts_y, tmp);
}


/*******************************************************************************
Purpose:
   Register the map defined in the Map Paramters.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_map(void)
{
    Arg          args[1];        /* resource argument variable               */
    char         tmpstr[132];    /* temporary string displaying one line     */
    XmString     xmstr;          /* temporary Xm string variable             */

    XmListDeleteAllItems(register_list);

    /*
     *  *** set polynomial order for least squares
     */
    if (tie_mapparm.npts_x * tie_mapparm.npts_y < 7)
        tie_mapparm.order = FIRST_ORDER;
    else
        tie_mapparm.order = SECOND_ORDER;

    /*
     *  *** update first item in list
     */
    if (tie_mapparm.units == MAP_DEGREES)
    {
        if ((tie_mapparm.ul_y < 0.0) && (tie_mapparm.ul_y > -0.999998611))
            sprintf(tmpstr, "  1    -0 %2d %5.2f ",
                    find_min(tie_mapparm.ul_y),
                    find_sec(tie_mapparm.ul_y));
        else
            sprintf(tmpstr,"  1   %3d %2d %5.2f ",find_deg(tie_mapparm.ul_y),
                    find_min(tie_mapparm.ul_y),
                    find_sec(tie_mapparm.ul_y));
        if ((tie_mapparm.ul_x < 0.0) && (tie_mapparm.ul_x > -0.999998611))
            sprintf(&tmpstr[19], "  -0 %2d %5.2f",
                    find_min(tie_mapparm.ul_x),
                    find_sec(tie_mapparm.ul_x));
        else
            sprintf(&tmpstr[19],"%4d %2d %5.2f",find_deg(tie_mapparm.ul_x),
                    find_min(tie_mapparm.ul_x),
                    find_sec(tie_mapparm.ul_x));
    }
    else
    {
        sprintf(tmpstr, "  1   %12.2f  %12.2f", 
                tie_mapparm.ul_y,
                tie_mapparm.ul_x);
    }

    xmstr = XmStringCreateLtoR(tmpstr, XmSTRING_DEFAULT_CHARSET);
    XmListAddItem(register_list, xmstr, 0);
    XmStringFree(xmstr);
    XmListSelectPos(register_list, 1, False);
    XtSetArg(args[0],XmNdialogStyle,XmDIALOG_MODELESS);
    XtSetValues(register_dialog_form, args, 1);

    if (!XtIsManaged(register_dialog_form))
        XtManageChild(register_dialog_form);

    XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,FALSE,
            register_point, NULL);

    tie_mapparm.mode = SELECT;
}



/*******************************************************************************
Purpose:
   Remove the registration panel.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_destroy
(
    Widget        w,
    XtPointer client,
    XtPointer call_data
)
{
    /*
     *  *** if OK button, check if registration completed
     */
    if ((client != NULL) && (strcmp((char*)client, "OK") == 0))
    {
        if (tie_mapparm.mode != SELECT)
        {
            XmListSetPos(register_list, 1);
            tie_mapparm.registered = True;
            XtUnmanageChild(register_dialog_form);
            XtRemoveEventHandler(ref_image_bulletinboard,ButtonReleaseMask,
                    FALSE, register_point, NULL);
        }
        else
        {
            XBell(tie.display, 0);
            return;
        }
    }
    else if ((client != NULL) && (strcmp((char*)client, "CANCEL") == 0))
    {
        XmListSetPos(register_list, 1);
        tie_mapparm.registered = False;
        XtUnmanageChild(register_dialog_form);
        XtRemoveEventHandler(ref_image_bulletinboard,ButtonReleaseMask,FALSE,
                register_point, NULL);
    }


    /*
     *  *** if was selecting, restore previous event handlers
     */
    if (tie.flags & TIE_SELECTING)
    {
        XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,
                FALSE, cursor_select,NULL);
        XtAddEventHandler(ref_image_bulletinboard,ButtonReleaseMask,
                FALSE, fullres_button,NULL);
    }
}

/*******************************************************************************
Purpose:
   This routine is the callback from sparked points.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_point
(
    Widget        w,
    XtPointer client,
    XEvent*   event,
    Boolean*  continue_to_dispatch
)
{
    Arg          args[2];        /* Variable used to set widget resourcees   */
    char         tmpstr[132];    /* temporary string displaying one line     */
    XmString     xmstr;          /* temporary Xm string variable             */
    static
        double   spark[26][2];   /* Digitizer coordinates to fit             */
    static
        double   mappts[26][2];  /* map points to digitize                   */
    int          count,          /* number of points sparked                 */
                 xspark,         /* increment in x direction                 */
                 yspark;         /* increment in y direction                 */
    int          nsel = 0,       /* number of items selected                 */
                 nvis,           /* number of visible items in the list      */
                 *index;         /* list of selected items in list           */

    XtSetArg(args[0], XmNitemCount, &count);
    XtSetArg(args[1], XmNvisibleItemCount, &nvis);
    XtGetValues(register_list, args, 2);

    /*
     *   ***  if not registering map, verify selected item in list
     */
    if (tie_mapparm.mode != SELECT)
    {
        XmListGetSelectedPos(register_list, &index, &nsel);

        if (nsel > 0)
            count = index[0];
        else
            return;
    }

    /*
     *  *** for some reason, values above 0x7fff come out negative????
     */
    if (!tie_parms.manual)
    {
        spark[count-1][0] = (double)(event->xbutton.x & 0x0000ffff);
        spark[count-1][1] = (double)(event->xbutton.y & 0x0000ffff);
    }
    else
    {
        spark[count-1][0] = (double)event->xbutton.x;
        spark[count-1][1] = (double)event->xbutton.y;
    }


    xspark = count % tie_mapparm.npts_x;
    yspark = count / tie_mapparm.npts_x;

    if (count == 1)
    {
        mappts[0][0] = tie_mapparm.ul_x;
        mappts[0][1] = tie_mapparm.ul_y;
    }

    mappts[count][0] = tie_mapparm.ul_x + (tie_mapparm.inc_x * xspark);
    mappts[count][1] = tie_mapparm.ul_y + (tie_mapparm.inc_y * yspark);

    /*
     *  *** if not all sparked, display next requested point
     */
    if (count < tie_mapparm.npts_x*tie_mapparm.npts_y)
    {
        if (tie_mapparm.units == MAP_DEGREES)
        {
            if ((mappts[count][1] < 0.0) && (mappts[count][1] > -0.999998611))
                sprintf(tmpstr, "%3d    -0 %2d %5.2f ",count+1,
                        find_min(mappts[count][1]),
                        find_sec(mappts[count][1]));
            else
                sprintf(tmpstr, "%3d   %3d %2d %5.2f ",count+1,
                        find_deg(mappts[count][1]),
                        find_min(mappts[count][1]),
                        find_sec(mappts[count][1]));
            if ((mappts[count][0] < 0.0) && (mappts[count][0] > -0.999998611))
                sprintf(&tmpstr[19], "  -0 %2d %5.2f",
                        find_min(mappts[count][0]),
                        find_sec(mappts[count][0]));
            else
                sprintf(&tmpstr[19],"%4d %2d %5.2f",find_deg(mappts[count][0]),
                        find_min(mappts[count][0]),
                        find_sec(mappts[count][0]));
        }
        else
            sprintf(tmpstr, "%3d   %12.2f  %12.2f",
                    count+1, mappts[count][1], mappts[count][0]);
    }
    /*
     *  *** if all points are sparked, spark verification point
     */
    else if (count == tie_mapparm.npts_x*tie_mapparm.npts_y)
    {
        mappts[count][0] = tie_mapparm.verify_x;
        mappts[count][1] = tie_mapparm.verify_y;

        if (tie_mapparm.units == MAP_DEGREES)
        {
            if ((tie_mapparm.verify_y <0.0) && 
                (tie_mapparm.verify_y > -0.999998611))
                sprintf(tmpstr, "%3d    -0 %2d %5.2f ",count+1,
                        find_min(tie_mapparm.verify_y),
                        find_sec(tie_mapparm.verify_y));
            else
                sprintf(tmpstr, "%3d   %3d %2d %5.2f ",count+1,
                        find_deg(tie_mapparm.verify_y),
                        find_min(tie_mapparm.verify_y),
                        find_sec(tie_mapparm.verify_y));
            if ((tie_mapparm.verify_x <0.0) && 
                (tie_mapparm.verify_x > -0.999998611))
                sprintf(&tmpstr[19], "  -0 %2d %5.2f",
                        find_min(tie_mapparm.verify_x),
                        find_sec(tie_mapparm.verify_x));
            else
                sprintf(&tmpstr[19],"%4d %2d %5.2f",
                        find_deg(tie_mapparm.verify_x),
                        find_min(tie_mapparm.verify_x),
                        find_sec(tie_mapparm.verify_x));
        }
        else
            sprintf(tmpstr, "VERPT %12.2f  %12.2f",
                    tie_mapparm.verify_y,
                    tie_mapparm.verify_x);
    }
    else 
        tie_mapparm.mode = EDIT;

    /*
     *  *** if not editing selected points, add next point to list
     */
    if (tie_mapparm.mode != EDIT)
    {
        xmstr = XmStringCreateLtoR(tmpstr, XmSTRING_DEFAULT_CHARSET);
        XmListAddItem(register_list, xmstr, 0);
        XmStringFree(xmstr);
        XmListSelectPos(register_list, 0, False);

        /*
         *  *** scroll to bottom of list
         */
        if (count >= nvis)
            XmListSetPos(register_list, count - nvis + 2);
    }
    /*
     *  *** otherwise compute residuals and display
     */
    else 
    {
        register_resid(spark, mappts);
        XtSetArg(args[0], XmNitemCount, &count);
        XtGetValues(register_list, args, 1);

        /*
         *  *** select next item in list
         */
        if ((nsel > 0) && (index[0] < count))
        {
            XmListDeselectPos(register_list, index[0]);
            XmListSetPos(register_list, index[0]);
            XmListSelectPos(register_list, index[0]+1, False);
        }
    }
}
 

/*******************************************************************************
Purpose:
   This routine is the callback from sparked points.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_resid
(
    double spark[][2],
    double mappts[][2]   /* map coordinates sparked                  */
)
{
    int          i,              /* loop counters                             */
                 npts;           /* total number of register points           */
    double       x,y,            /* Calcualated map coordinates               */
                 res_lat,        /* Residual in Y/latitude direction          */
                 res_lon,        /* Residual in X/longitude direction         */
                 temp_x,         /* temporary x storage                       */
                 temp_y;         /* temporary y storage                       */ 
    char         tmpstr[132];    /* temporary string displaying one line      */
    XmString     xmstr;          /* temporary Xm string variable              */

    npts = tie_mapparm.npts_x * tie_mapparm.npts_y;

    tps_Qfit(npts, spark, mappts, NULL, &tie_mapparm.order, 
            tie_mapparm.coef_y, tie_mapparm.coef_x);

    /* 
     *  For each point, calculate the map coordinate with the polynomial.  Then,
     *  calculate the residual between the actual map point and the calculated
     *  map point.  Write the actual map point, the calculated map point, and
     *  the residual to the screen
     */ 
    for (i = 0; i < npts+1; i++)
    {
        temp_x = spark[i][0];
        temp_y = spark[i][1];
        x = c_eval(&tie_mapparm.order, tie_mapparm.coef_x, 
                   &temp_x, &temp_y);
        y = c_eval(&tie_mapparm.order, tie_mapparm.coef_y, 
                   &temp_x, &temp_y);

        if (tie_mapparm.units == MAP_DEGREES)
        {
            res_lat = (fabs(y) - fabs(mappts[i][1])) * 3600.0;
            res_lon = (fabs(x) - fabs(mappts[i][0])) * 3600.0;
        }
        else
        {
            res_lat = fabs(y) - fabs(mappts[i][1]);
            res_lon = fabs(x) - fabs(mappts[i][0]);
        }

        /*
         *  *** update item in registration list
         */
        if (tie_mapparm.units == MAP_DEGREES)
        {
            if ((mappts[i][1] < 0.0) && (mappts[i][1] > -0.999998611))
                sprintf(tmpstr, "%3d    -0 %2d %5.2f ",i+1,
                        find_min(mappts[i][1]),find_sec(mappts[i][1]));
            else
                sprintf(tmpstr, "%3d   %3d %2d %5.2f ",i+1,
                        find_deg(mappts[i][1]),
                        find_min(mappts[i][1]),find_sec(mappts[i][1]));
            if ((mappts[i][0] < 0.0) && (mappts[i][0] > -0.999998611))
                sprintf(&tmpstr[19], "  -0 %2d %5.2f  ",
                        find_min(mappts[i][0]),find_sec(mappts[i][0]));
            else
                sprintf(&tmpstr[19], "%4d %2d %5.2f  ",find_deg(mappts[i][0]),
                        find_min(mappts[i][0]),find_sec(mappts[i][0]));
            if ((y < 0.0) && (y > -0.999998611))
                sprintf(&tmpstr[34], " -0 %2d %5.2f ",find_min(y), find_sec(y));
            else
                sprintf(&tmpstr[34], "%3d %2d %5.2f ",find_deg(y),find_min(y),
                        find_sec(y));
            if ((x < 0.0) && (x > -0.999998611))
                sprintf(&tmpstr[47],"  -0 %2d %5.2f  ",find_min(x),find_sec(x));
            else
                sprintf(&tmpstr[47], "%4d %2d %5.2f  ",find_deg(x),find_min(x),
                        find_sec(x));
            sprintf(&tmpstr[61], "%8.2f %8.2f",res_lat,res_lon);
        }
        else 
            sprintf(tmpstr,"%3d   %10.1f %10.1f  %10.1f %10.1f  %8.2f %8.2f",
                   (i + 1), mappts[i][1], mappts[i][0], y, x, res_lat, res_lon);

        if (i == npts)
            strncpy(tmpstr, "VERPT", 5);

        xmstr = XmStringCreateLtoR(tmpstr, XmSTRING_DEFAULT_CHARSET);
        XmListReplaceItemsPos(register_list, &xmstr, 1, i+1);
        XmStringFree(xmstr);
    }

    /*
     *  *** define outline of map
     */
    tie_mapparm.outline[0][0] = (int)spark[0][0];
    tie_mapparm.outline[0][1] = (int)spark[0][1];

    tie_mapparm.outline[1][0] = (int)spark[tie_mapparm.npts_x-1][0];
    tie_mapparm.outline[1][1] = (int)spark[tie_mapparm.npts_x-1][1];

    tie_mapparm.outline[2][0] = (int)spark
        [tie_mapparm.npts_x*tie_mapparm.npts_y-1][0];
    tie_mapparm.outline[2][1] = (int)spark
        [tie_mapparm.npts_x*tie_mapparm.npts_y-1][1];

    tie_mapparm.outline[3][0] = (int)spark
        [tie_mapparm.npts_x*(tie_mapparm.npts_y-1)][0];
    tie_mapparm.outline[3][1] = (int)spark
        [tie_mapparm.npts_x*(tie_mapparm.npts_y-1)][1];

    graphic_refresh();

    return;
}

/*******************************************************************************
Purpose:
   This routine is the callback from sparked points.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void register_guesspt(void)
{
    int         nx,             /* number of spark points in y          */
                ny,             /* number of spark points in x          */
                scale;          /* present map scale setting            */
    double      ul_x,           /* upper left x coordinate              */
                ul_y,           /* upper left y coordinate              */
                lr_x,           /* lower right x coordinate             */
                lr_y,           /* lower right y coordinate             */
                verify_x,       /* lower right x coordinate             */
                verify_y;       /* lower right y coordinate             */
    char        dms[20],        /* degrees minutes seconds string       */
                tmp[4];         /* string defining points to spark      */

    if ((tie_parms.manual) || (tie_mapparm.units != MAP_DEGREES))
        return;

    sscanf(XmTextGetString(reg_scale_text)+2, "%d", &scale); 
    ul_x     = map_dms2deg(XmTextGetString(reg_ul_x));
    ul_y     = map_dms2deg(XmTextGetString(reg_ul_y));


    switch (scale)
    {
        case  12000:
            lr_x = ul_x + 3.725/60.;
            lr_y = ul_y - 3.725/60.;
            verify_x = ul_x + 1.25/60.;
            verify_y = lr_y + 1.25/60.;
            nx = 2;
            ny = 2;
            break;

        case  24000:
            lr_x = ul_x + 7.5/60.;
            lr_y = ul_y - 7.5/60.;
            verify_x = ul_x + 2.5/60.;
            verify_y = lr_y + 2.5/60.;
            nx = 2;
            ny = 2;
            break;

        case  25000:
            lr_x = ul_x + 10./60.;
            lr_y = ul_y - 7.5/60.;
            verify_x = ul_x + 2.5/60.;
            verify_y = lr_y + 2.5/60.;
            nx = 2;
            ny = 2;
            break;

        case  50000:
            lr_x = ul_x + 20./60.;
            lr_y = ul_y - 15./60.;
            verify_x = ul_x + 5.0/60.;
            verify_y = lr_y + 5.0/60.;
            nx = 2;
            ny = 2;
            break;

        case  62500:
            lr_x = ul_x + 15./60.;
            lr_y = ul_y - 15./60.;
            verify_x = ul_x + 5.0/60.;
            verify_y = lr_y + 5.0/60.;
            nx = 4;
            ny = 4;
            break;

        case  63360:
            lr_x = ul_x + 30/60.;
            lr_y = ul_y - 15./60.;
            verify_x = ul_x + 5.0/60.;
            verify_y = lr_y + 5.0/60.;
            nx = 5;
            ny = 4;
            break;

        case 100000:
            lr_x = ul_x + 40.0/60.0;
            lr_y = ul_y - 0.5;
            verify_x = ul_x + 10.0/60.0;
            verify_y = lr_y + 10.0/60.0;
            nx = 3;
            ny = 3;
            break;

        case 250000:
            lr_x =  ul_x + 2.0;
            lr_y =  ul_y - 1.0;
            verify_x = ul_x + 45.0/60.0;
            verify_y = lr_y + 15.0/60.0;
            nx = 5;
            ny = 3;
            break;

        default:
            return;
    } 

    /*
     *  *** set lower right point
     */
    if ((lr_x < 0.0) && (lr_x > -0.999998611))
        sprintf(dms, " -0 %2d %4.2f",
                find_min(lr_x),
                find_sec(lr_x));
    else
        sprintf(dms, "%3d %2d %4.2f", find_deg(lr_x),
                find_min(lr_x),
                find_sec(lr_x));

    XmTextSetString(reg_lr_x, dms);

    if ((lr_y < 0.0) && (lr_y > -0.999998611))
        sprintf(dms, " -0 %2d %4.2f",
                find_min(lr_y),
                find_sec(lr_y));
    else
        sprintf(dms, "%3d %2d %4.2f", find_deg(lr_y),
                find_min(lr_y),
                find_sec(lr_y));
    XmTextSetString(reg_lr_y, dms);

    /*
     *  *** set verify point
     */
    if ((verify_x < 0.0) && (verify_x > -0.999998611))
        sprintf(dms, " -0 %2d %4.2f",
                find_min(verify_x),
                find_sec(verify_x));
    else
        sprintf(dms, "%3d %2d %4.2f", find_deg(verify_x),
                find_min(verify_x),
                find_sec(verify_x));
    XmTextSetString(reg_verify_x, dms);

    if ((verify_y < 0.0) && (verify_y > -0.999998611))
        sprintf(dms, " -0 %2d %4.2f",
                find_min(verify_y),
                find_sec(verify_y));
    else
        sprintf(dms, "%3d %2d %4.2f", find_deg(verify_y),
                find_min(verify_y),
                find_sec(verify_y));
    XmTextSetString(reg_verify_y, dms);

    /*
     *  *** set number of points to spark
     */
    sprintf(tmp, "%3d", nx);
    tmp[3] = '\0';
    XmTextSetString(reg_npts_x, tmp);

    sprintf(tmp, "%3d", ny);
    tmp[3] = '\0';
    XmTextSetString(reg_npts_y, tmp);
}
