#include "tiepts.h"
#include "convert.h"
#include "cr.h"
#include "tpssup.h"
#include "protos.h"

#ifndef MIN
#define MIN(a,b)        ((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b)        ((a) > (b) ? (a) : (b))
#endif

/******************************************************************************
Purpose:
    This routine initializes the rough transformation.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by J. LaVergne USGS/EROS Data Center, 7/93
  - Changed 'More than 4 Points in RT File..' message from informational to
    warning (to force a response). 
  - Changed calls to 'caution' into 'tie_message' calls, because DGs that are
    not upgraded blow up tiepts.
 Algorithm Description:
******************************************************************************/
void init_rt(void)
{
  double ref_yx[4][2];     /* Reference tie pts calculated
                              from input images' DDRs                    */ 
  double srch_yx[4][2];    /* Search tie pts calculated
                              from input images' DDRs                    */
  double x_coef[6];
  double y_coef[6];
  double temp_x;
  double temp_y;
  double minlat;
  double maxlat;
  double minlon;
  double maxlon;
  char rt_fname[CMLEN];    /* filename for rough trans. selected points  */
  char tstring[CMLEN];     /* temporary string variable                  */
  char *coordtyp;          /* Type of coordinates in .rt file            */
  int  active;             /* flag indicating rt point is active         */
  int  ddrtie;             /* number of tie points acquired from DDR     */
  int  ext;                /* index of filename extension                */
  int  i;                  /* loop counter                               */
  int  length;             /* string length variable                     */
  long nonfatal;           /* non fatal error flag                       */
  int  rplane_idx;         /* index of search plane                      */
  int  splane_idx;         /* index of search plane                      */
  int  ddr_status;         /* ddr return status                          */
  int  status;             /* function return status                     */

  Widget rough_accept;     /* Accept window for rough trans points       */

  if (tie.mode == IM2IM)
    rplane_idx = tie_composite[REFERENCE].rgbplanes[RED] - 1;

  splane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

  tie_rt.degree = tie_parms.order;

  /* If selecting on a single image,
     display points and return
  ----------------------------------*/
  if (tie.mode == ONESRCH)
    {
    for (i = 0; i < tie_select.seltie; i++)
      display_pt(i);
    return;
    }

  /* If enough points recovered from
     input TPS file, calc rough trans
  -----------------------------------*/
  if (tie_select.numtie > 3)
    {
    tie_rt.calc_rt = RT_CAL;
    tps_Qfit(tie_select.numtie, tie_select.rtie, tie_select.stie, 
             tie_select.disabled, &tie_rt.degree, tie_rt.line_coef,
             tie_rt.samp_coef);

    /* If image to vector, convert points to image coordinates
    ----------------------------------------------------------*/
    if (tie.mode == IM2VECT)
      {
      /* Get the sl,ss of the window 
      ------------------------------*/
      i = tie_composite[SEARCH].rgbplanes[RED] - 1;
      tie_vectfile[0].corner[0][0] = tie_planes[i].window[1];
      tie_vectfile[0].corner[0][1] = tie_planes[i].window[0];
      tie_vectfile[0].corner[1][0] = tie_planes[i].window[1];
      tie_vectfile[0].corner[1][1] = tie_planes[i].window[0] + 
                                     tie_planes[i].window[2] - 1.0;
      tie_vectfile[0].corner[2][0] = tie_planes[i].window[1] +
                                     tie_planes[i].window[3] - 1.0;
      tie_vectfile[0].corner[2][1] = tie_planes[i].window[0];
      tie_vectfile[0].corner[3][0] = tie_planes[i].window[1] +
                                     tie_planes[i].window[3] - 1.0;
      tie_vectfile[0].corner[3][1] = tie_planes[i].window[0] + 
                                     tie_planes[i].window[2] - 1.0;

      /* Create polynomial conversion from
         pixel to image coordinates
      ------------------------------------*/
      status = tie_parms.order;
      tps_Qfit(tie_select.numtie, tie_select.stie, tie_select.rtie,
               NULL, &status, y_coef, x_coef);

      /* Convert sl,ss,el,es to lat/long 
      ----------------------------------*/
      for (i = 0; i < 4; i++)
        {
        temp_x = tie_vectfile[0].corner[i][0];
        temp_y = tie_vectfile[0].corner[i][1];
     
        tie_vectfile[0].corner[i][0] = c_eval(&status,x_coef,&temp_x,&temp_y);
        tie_vectfile[0].corner[i][1] = c_eval(&status,y_coef,&temp_x,&temp_y);
        }
      proj2degrees(tie_vectfile[0].corner, 4, SEARCH);

      /* Set vector min/max lat/long values
      -------------------------------------*/ 
      minlon = maxlon = tie_vectfile[0].corner[0][0];
      minlat = maxlat = tie_vectfile[0].corner[0][1];
      for (i = 1; i < 4; i++)
        {
        minlon=MIN(minlon,tie_vectfile[0].corner[i][0]);
        maxlon=MAX(maxlon,tie_vectfile[0].corner[i][0]);
        minlat=MIN(minlat,tie_vectfile[0].corner[i][1]);
        maxlat=MAX(maxlat,tie_vectfile[0].corner[i][1]);
        }
      if (((tie_vectfile[0].corner[0][0] > 0)  || 
           (tie_vectfile[0].corner[1][0] > 0)) &&
          ((tie_vectfile[0].corner[2][0] < 0)  || 
           (tie_vectfile[0].corner[3][0] < 0)))
        {
        if ((tie_vectfile[0].corner[0][0] > 0) &&
            (tie_vectfile[0].corner[1][0] > 0))
          minlon = MIN(tie_vectfile[0].corner[0][0],
                       tie_vectfile[0].corner[1][0]);
        else
          minlon = MAX(tie_vectfile[0].corner[0][0],
                       tie_vectfile[0].corner[1][0]);
        if ((tie_vectfile[0].corner[2][0] < 0) &&
            (tie_vectfile[0].corner[3][0] < 0))
          maxlon = MAX(tie_vectfile[0].corner[2][0],
                       tie_vectfile[0].corner[3][0]);
        else
          maxlon = MIN(tie_vectfile[0].corner[2][0],
                       tie_vectfile[0].corner[3][0]);
        }
      tie_vectfile[0].corner[1][0] = minlon;
      tie_vectfile[0].corner[1][1] = minlat;
      tie_vectfile[0].corner[2][0] = maxlon;
      tie_vectfile[0].corner[2][1] = maxlat;

      /* Set the vector text fields for
         the known area of interest 
      ---------------------------------*/
      vector_setbounds();
      }
    }
  else
    {
    /* Else try to calculate rough trans
       from input images DDRs
    ------------------------------------*/
    if (tie.mode == IM2IM)
      {
      if (!(tie.flags & TIE_ADRG))
        ddr_status = tps_auto_rt(&tie_planes[rplane_idx].ddr, 
                                 &tie_planes[splane_idx].ddr,
                                 tie.mode, "degrees",
                                 ref_yx, srch_yx);
      else
        ddr_status = tps_auto_rt(&tie_planes[rplane_idx].ddr, 
                                 &tie_planes[splane_idx].ddr,
                                 tie.mode, "degrees",
                                 ref_yx, srch_yx);
      }
    else if ((tie.mode == IM2MAP) && (tie_mapparm.units != MAP_USERDEF))
      ddr_status = tps_auto_rt(&tie_planes[rplane_idx].ddr, 
                               &tie_planes[splane_idx].ddr,
                               tie.mode, "degrees", 
                               ref_yx, srch_yx);
    else if (tie.mode == IM2VECT)
      ddr_status = tps_auto_rt(&tie_planes[splane_idx].ddr, 
                               &tie_planes[splane_idx].ddr,
                               tie.mode, "degrees",
                               ref_yx, srch_yx);
    else
      ddr_status = E_FAIL;
         
    if (ddr_status == E_SUCC)
      {
      tie_rt.calc_rt = RT_DDR;
      tie_select.seltie = tie_select.numtie;
      ddrtie = 4 - tie_select.numtie;
      for (i = 0; i < ddrtie; i++)
        {
        /* Load data into appropriate structures
           and convert if necessary
        ----------------------------------------*/
        if (tie.mode == IM2IM)
          {
          if (tie_parms.flags & PARMS_LINESAMP)
            {
            tie_select.rtie[tie_select.numtie+i][0] = ref_yx[4-ddrtie+i][0];
            tie_select.rtie[tie_select.numtie+i][1] = ref_yx[4-ddrtie+i][1];
            }
          else
            {
            tie_select.lsrtie[tie_select.numtie+i][0] = ref_yx[4-ddrtie+i][0];
            tie_select.lsrtie[tie_select.numtie+i][1] = ref_yx[4-ddrtie+i][1];
            }
          if ((tie_parms.flags & PARMS_PROJ) ||
              (tie_parms.flags & PARMS_LATLONG) &&
             !(tie_parms.flags & PARMS_LINESAMP))
            {
            image2proj(REFERENCE,tie_select.rtie[tie_select.numtie+i],
                       ref_yx[4-ddrtie+i][0],ref_yx[4-ddrtie+i][1]);
            if (tie_parms.flags & PARMS_LATLONG)
              proj2degrees(tie_select.rtie[tie_select.numtie+i],1,REFERENCE);
            } 
          }
        else
          {
          tie_select.rtie[tie_select.numtie+i][0] = ref_yx[4-ddrtie+i][0];
          tie_select.rtie[tie_select.numtie+i][1] = ref_yx[4-ddrtie+i][1];
          }
        tie_select.stie[tie_select.numtie+i][0] = srch_yx[4-ddrtie+i][0];
        tie_select.stie[tie_select.numtie+i][1] = srch_yx[4-ddrtie+i][1];
        tie_select.disabled[tie_select.numtie+i] = FALSE;
        tie_select.pt_id[tie_select.numtie+i][0] = '\0';
        }
      tie_select.numtie += ddrtie;

      /* Calculate rough trans. coefficients
      --------------------------------------*/
      status = tps_Qfit(tie_select.numtie, tie_select.rtie, tie_select.stie,
                        tie_select.disabled, &tie_rt.degree,
                        tie_rt.line_coef, tie_rt.samp_coef);

      /* If all the sample or line cofficients are zero,
         the rough transformation calculation failed.
         The user will be required to select the rough
         transformation points.
      --------------------------------------------------*/
      if ((tie_rt.samp_coef[0] == 0) && 
          (tie_rt.samp_coef[1] == 0) && 
          (tie_rt.samp_coef[2] == 0))
        {
        tie_rt.calc_rt = RT_SEL;
        tie_select.seltie = tie_select.numtie = ddrtie = 0;
        }
      if ((tie_rt.line_coef[0] == 0) &&  
          (tie_rt.line_coef[1] == 0) && 
          (tie_rt.line_coef[2] == 0))
        {
        tie_rt.calc_rt = RT_SEL;
        tie_select.seltie = tie_select.numtie = ddrtie = 0;
        }
      }
    else 
      tie_rt.calc_rt = RT_SEL;

    /* If the user must select the rough trans,
       then the ddr is useless, set status accordingly
    --------------------------------------------------*/
    if (tie_rt.calc_rt == RT_SEL)
      ddr_status = E_FAIL;
    }

  /* Create the srchimgname.rt filename
  -------------------------------------*/
  if (tie_rt.calc_rt == RT_SEL)
    {
    tie_select.numtie = 0;
    length = (int)strlen(tie_planes[splane_idx].image_name);
    for (i=length-1; i >= 0; i--)
      if (tie_planes[splane_idx].image_name[i] == DIR_CHAR)
        break;

    if (i < 0)
      i = 0;

    for (ext=length-1; ext >= 0; ext--)
      if (tie_planes[splane_idx].image_name[ext] == '.')
        break;

    if (ext < i)
      ext = length;

    strncpy(rt_fname,(tie_planes[splane_idx].image_name)+i+1,ext-i-1);
    rt_fname[ext-i-1] = '\0';
    strcat(rt_fname,".rt");

    /* Open the srchimgname.rt file for updating
    --------------------------------------------*/
    tie_rt.rtfp = fopen(rt_fname,"r+");

    /* If the file does not exist, create it
    -----------------------------------------*/
    if (tie_rt.rtfp == NULL)
      {
      tie_rt.rtfp = fopen(rt_fname,"w");
      if (tie_rt.rtfp == NULL)
        c_errmsg("Error opening rough transformation file",
                 "tiepts-rtfil",&nonfatal);
      }
    else
      {
      /* Otherwise, read in the existing points
      -----------------------------------------*/
      do
        {
        status = fscanf(tie_rt.rtfp,
                        "%d,%lf,%lf,%d,%d,%d,%lf,%lf,%d,%d,%d,%s",
              &active,&tie_select.rtie[tie_select.numtie][0],
              &tie_select.rtie[tie_select.numtie][1],
              &tie_rt.rtrzm[tie_select.numtie], 
              &tie_rt.rtroff[tie_select.numtie][0],
              &tie_rt.rtroff[tie_select.numtie][1],
              &tie_select.stie[tie_select.numtie][0],
              &tie_select.stie[tie_select.numtie][1],
              &tie_rt.rtszm[tie_select.numtie],
              &tie_rt.rtsoff[tie_select.numtie][0], 
              &tie_rt.rtsoff[tie_select.numtie][1],tstring);

        /* Get the coordinate type
        --------------------------*/
        coordtyp = strchr(tstring,',');
        coordtyp++;

        if (status != EOF)
          {
          if (active)
            {
            tie_select.disabled[tie_select.numtie] = FALSE;
            tie_select.pt_id[tie_select.numtie][0] = '\0';
            if (tie_select.numtie == 4)
              {
              tie_message(Shell_Main, "More than 4 Points in RT File..");
              break;
              }

            tie_select.numtie++;
            tie_select.seltie++;
            }
          else
            tie_select.disabled[tie_select.numtie] = TRUE;
          }
        } while (status != EOF);

      } 

    /* If less than 4 points were read from the
       srchimgname.rt file, tell user they must
       select the rough trans points
    -------------------------------------------*/
    if (tie_select.numtie < 4)
      tie_message(Shell_SrchImg, "The Rough Transformation cannot be calculated.  The \nnext four points selected will be used to calculate it.\nThese points should be selected from the corners of\nthe Search Image.");

    else
      {
      /* Calculate the rough transformation
      -------------------------------------*/
      status = tps_Qfit(tie_select.numtie, tie_select.rtie, tie_select.stie,
                        tie_select.disabled,&tie_rt.degree,tie_rt.line_coef, 
                        tie_rt.samp_coef);

      if (status != E_SUCC)
        {
        /* Error occurred calculating the rough trans
           reset variables and tell user they must
           select the rough trans points
        ---------------------------------------------*/
        tie_rt.calc_rt = RT_SEL;
        tie_select.numtie = 0;
        tie_message(Shell_SrchImg, "The Rough Transformation cannot be calculated.  The \nnext four points selected will be used to calculate it.\nThese points should be selected from the corners of\nthe Search Image.");
        } 
      else
        {
        /* Display accept form for rough trans points
           that were in the srchimgname.rt file
        ---------------------------------------------*/
        rough_accept = Createrough_form(Shell_Main);
        XtManageChild(rough_accept);
        edit_compute();

        /* Display points in appropriate window
        ---------------------------------------*/
        for (i = 0; i < tie_select.numtie; i++)
          {
          if (strcmp(coordtyp,"REF") != 0)
            conv_ls(coordtyp,i);
          display_pt(i);
          }
        }
      } 
    }

  if (tie_rt.calc_rt != RT_SEL)
    {
    edit_compute();

    for (i = 0; i < tie_select.numtie; i++)
      display_pt(i);
    }
}


/*******************************************************************************
Purpose:
    This routine removes the rough transformation acceptance panel
    and set the initial 4 tie points.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void rough_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];
   Widget       units;
   char         *pt_id;
   char         *elev;
   int          dup;
   int          i,j;
   double x_coef[6],y_coef[6],temp_x,temp_y;
   double minlat,maxlat,minlon,maxlon;

  /*
   * The user is no longer selecting rough transformation points
   * if more than 3 points have been selected.
   */
   if (tie_rt.rtfp != NULL)
      {
      fclose(tie_rt.rtfp);
      tie_rt.rtfp = NULL;
      }

   dup = FALSE;

   for (i = 0; i < 4; i++)
      {
      pt_id =  XmTextGetString(rough_id[i]);
      if (strcmp(pt_id, tie_select.pt_id[i]) && !tps_checkid(pt_id))
         {
         XmTextSetString(rough_id[i], tie_select.pt_id[i]);
         dup = TRUE;
         }
      else
         strcpy(tie_select.pt_id[i], pt_id);

      XtFree(pt_id);
      }

   if (!dup)
      {
     /*
      *  *** get units (feet/meters)
      */
      XtSetArg(args[0], XmNmenuHistory, &units);
      XtGetValues(rough_units_option, args, 1);

      for (i = 0; i < 4; i++)
         {
         elev = XmTextGetString(rough_elev[i]);
         sscanf(elev, "%lf", &tie_select.elev[i]);
         XtFree(elev);
   
        /*
         *  *** if units are in feet, convert to meters
         */
         if (units == rough_units_feet)
            tie_select.elev[i] *= FEET2CM/100.;
         }

      /*tps_dump();*/
      tie_rt.calc_rt = RT_CAL;
      XtDestroyWidget((Widget)client);

      /* set vector min/max lat/long values */ 
      if (tie.mode == IM2VECT)
            {
           /*
            * *** get the sl,ss of the window 
            */
            i = tie_composite[SEARCH].rgbplanes[RED] - 1;
            tie_vectfile[0].corner[0][0] = tie_planes[i].window[1];
            tie_vectfile[0].corner[0][1] = tie_planes[i].window[0];
            tie_vectfile[0].corner[1][0] = tie_planes[i].window[1];
            tie_vectfile[0].corner[1][1] = tie_planes[i].window[0] + 
                                           tie_planes[i].window[2] - 1.0;
            tie_vectfile[0].corner[2][0] = tie_planes[i].window[1] +
                                           tie_planes[i].window[3] - 1.0;
            tie_vectfile[0].corner[2][1] = tie_planes[i].window[0];
            tie_vectfile[0].corner[3][0] = tie_planes[i].window[1] +
                                           tie_planes[i].window[3] - 1.0;
            tie_vectfile[0].corner[3][1] = tie_planes[i].window[0] + 
                                           tie_planes[i].window[2] - 1.0;

           /*
            *  *** create polynomial conversion from pixel to image coordinates
            */
            j = tie_parms.order;
            tps_Qfit(tie_select.numtie, tie_select.stie, tie_select.rtie,
                     NULL, &j, y_coef, x_coef);

            /*
             * *** convert sl,ss,el,es to lat/long 
             */
            for (i = 0; i < 4; i++)
               {
               temp_x = tie_vectfile[0].corner[i][0];
               temp_y = tie_vectfile[0].corner[i][1];
      
               tie_vectfile[0].corner[i][0] =
                        c_eval(&j, x_coef, &temp_x, &temp_y);
               tie_vectfile[0].corner[i][1] =
                        c_eval(&j, y_coef, &temp_x, &temp_y);
               }
            proj2degrees(tie_vectfile[0].corner, 4, SEARCH);

            minlon = maxlon = tie_vectfile[0].corner[0][0];
            minlat = maxlat = tie_vectfile[0].corner[0][1];
            for (i = 1; i < 4; i++)
               {
               minlon=MIN(minlon,tie_vectfile[0].corner[i][0]);
               maxlon=MAX(maxlon,tie_vectfile[0].corner[i][0]);
               minlat=MIN(minlat,tie_vectfile[0].corner[i][1]);
               maxlat=MAX(maxlat,tie_vectfile[0].corner[i][1]);
               }
            if (((tie_vectfile[0].corner[0][0] > 0)  || 
                 (tie_vectfile[0].corner[1][0] > 0)) &&
                ((tie_vectfile[0].corner[2][0] < 0)  || 
                 (tie_vectfile[0].corner[3][0] < 0)))
               {
               if ((tie_vectfile[0].corner[0][0] > 0) &&
                   (tie_vectfile[0].corner[1][0] > 0))
                  minlon=MIN(tie_vectfile[0].corner[0][0],
                             tie_vectfile[0].corner[1][0]);
               else
                  minlon=MAX(tie_vectfile[0].corner[0][0],
                             tie_vectfile[0].corner[1][0]);
               if ((tie_vectfile[0].corner[2][0] < 0) &&
                   (tie_vectfile[0].corner[3][0] < 0))
                  maxlon=MAX(tie_vectfile[0].corner[2][0],
                             tie_vectfile[0].corner[3][0]);
               else
                  maxlon=MIN(tie_vectfile[0].corner[2][0],
                             tie_vectfile[0].corner[3][0]);
               }
            tie_vectfile[0].corner[1][0] = minlon;
            tie_vectfile[0].corner[1][1] = minlat;
            tie_vectfile[0].corner[2][0] = maxlon;
            tie_vectfile[0].corner[2][1] = maxlat;

          /*
           * *** set the vector text fields for the known area of interest 
           */
           vector_setbounds();
           }
      }
   else
      XBell(tie.display, 0);
}
