#include "tiepts.h"
#include "adrgsup.h"
#include "convert.h"
#include "tpssup.h"
#include "protos.h"
/*******************************************************************************
Purpose:
    This routine processes a selected reference.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by C. Olsen, USGS/EROS Data Center, 8/98
  - Modified to allocate memory for the converted tie points.  (SRF #1935)

Algorithm Description:
*******************************************************************************/
void select_refpt
(
    Widget w,                      /* Widget where action happend        */
    int x,                         /* X coord of window                  */
    int y                          /* Y coord of window                  */
)
{
    int fnum;                      /* Index of vector file               */
    int index;                     /* Variable used for looping          */
    int line;                      /* Line index of nearest pt           */ 
    int pt;                        /* Index of nearest pt                */
    int ptindex;                   /* Index of new/reselected point      */
    int status;                    /* Function return status             */
    long inunit;                   /* Input units - value is degrees. 
                                      If necessary, it was converted  
                                      to degrees.                        */
    long inproj;                   /* Input projeciton code - value is
                                      geograhpic                         */
    long inzone;                   /* Input zone - value is 62           */
    long indtmnum;                 /* Current datum selected by user     */
    long outunit;                  /* Output projection units code       */
    long outproj;                  /* Output projection code             */
    long outzone;                  /* Output projection zone number      */
    long outdtmnum;                /* Output datum number first one 
                                      selected by user                   */
    long prtprm[2];                /* Write out projection parms to a
                                      file or terminal (TRUE or FALSE)
                                      prtprm[GEO_TERM], prtprm[GEO_FILE] */
    double proj_coef_out[15];      /* Output projection coef's - all 
                                      values set to 0.0                  */
    double tempx;                  /* Temp digitizer x coordinate        */
    double tempy;                  /* Temp digitizer y coordinate        */

    /*
     *  *** if reselecting, assign to previous point,else at new point
     */
    if (tie.flags & TIE_RESELECTING)
        ptindex = tie_select.reselect;
    else
        ptindex = tie_select.seltie;

    if (ptindex == tie_select.numalloc)
    {
        tie_select.numalloc += MAX_TIE_POINTS;
        tie_select.elev    =(double *)(realloc(tie_select.elev,
                    tie_select.numalloc*sizeof(double)));
        tie_select.rtie    =(void *)(realloc(tie_select.rtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.crtie    =(void *)(realloc(tie_select.crtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.stie    =(void *)(realloc(tie_select.stie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.lsrtie  =(void *)(realloc(tie_select.lsrtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.resid   =(void *)(realloc(tie_select.resid,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.pt_id   =(void *)(realloc(tie_select.pt_id,
                    tie_select.numalloc*PTIDLEN*sizeof(char)));
        tie_select.mapname =(void *)(realloc(tie_select.mapname,
                    tie_select.numalloc*80*sizeof(char)));
        tie_select.mapid   =(void *)(realloc(tie_select.mapid,
                    tie_select.numalloc*80*sizeof(char)));
        tie_select.disabled=(unsigned char *)(realloc(tie_select.disabled,
                    tie_select.numalloc*sizeof(unsigned char)));
        if ((tie_select.elev == NULL) || (tie_select.rtie == NULL) ||
                (tie_select.crtie == NULL) || (tie_select.stie == NULL) || 
                (tie_select.lsrtie == NULL) || (tie_select.resid == NULL) || 
                (tie_select.pt_id == NULL) || (tie_select.mapname == NULL) || 
                (tie_select.mapid == NULL) || (tie_select.disabled == NULL))
        {
            tie_select.numalloc -= MAX_TIE_POINTS;
            printf("Error allocating memory\n");
            XBell(tie.display,0);
            return;
        }
    }

    /*
     *  ***  if image to vector, find nearest point
     */
    if ((tie.mode == IM2VECT) && (w == ref_image_bulletinboard))
    {
        if (vector_nnode(w, x, y, &fnum, &line, &pt) == E_SUCC)
        {
            if (tie_select.curref)
                vector_unmark(); 

            /*
             * Always set rtemp so that in case the user is reselecting a point,
             * and then tries to redefine the boundaries of the vector area, to
             * prevent the inverse rt from being screwed up because one of the
             * rtie has been reset without resetting the corresponding
             * stie.
             */
            tie_select.rtemp[0] = tie_vector->line[line].pts[pt*2];
            tie_select.rtemp[1] = tie_vector->line[line].pts[pt*2+1];
            tie_select.rtemp_flag = TRUE;

            vector_mark();
            tie_select.curref = TRUE;
        }
    }
    /*
     *  ***  if image to map, and digitized point
     */
    else if ((tie.mode == IM2MAP) && (w == ref_image_bulletinboard))
    {
        /*
         *  *** calculate map coordinate from digitized points
         */
        if (!tie_parms.manual)
        {
            tempx = (double)(x & 0x0000ffff);
            tempy = (double)(y & 0x0000ffff);
        }
        else
        {
            tempx = (double)x;
            tempy = (double)y;
        }


        tie_select.saveref[0] = tie_select.rtie[ptindex][0] =
            c_eval(&tie_mapparm.order, tie_mapparm.coef_x, &tempx, &tempy);

        tie_select.saveref[1] = tie_select.rtie[ptindex][1] =
            c_eval(&tie_mapparm.order, tie_mapparm.coef_y, &tempx, &tempy);

        if (tie_mapparm.units == MAP_PROJ)
        {
            status = map_proj2degrees(tie_select.rtie[ptindex],1,
                    tie_mapparm.mapdesc);
            if (status != E_SUCC)
                tie_message(Shell_SrchImg,
                        "Error converting projection\ncoordinates to degrees");
            tie_select.saveref[0] = tie_select.rtie[ptindex][0];
            tie_select.saveref[1] = tie_select.rtie[ptindex][1];
        }
        /*
         * The first datum that is chosen is the one that needs to be applied
         * to all points. Check the flag value. If it indicates another datum
         * was selected (probably due to another map that was registered in 
         * another datum) then convert the current point from the current datum
         * to the first datum selected.
         */

        if (tie_mapparm.first_datum_selected == FALSE)
        {   
            /* The output point value will be in degrees. NOTE: Input values
             * are in degrees because register.c converts them using the
             * input mapfile. 
             */
            inunit = outunit = DEGREE;
            inproj = outproj = GEO;
            inzone = outzone = 62;
            indtmnum = tie_mapparm.current_datum;
            outdtmnum = tie_mapparm.out_datum_code;
            prtprm[0] = prtprm[1] = 0;
            for (index = 0; index < 15; index++)
                proj_coef_out[index] = 0.0;

            status = c_transinit(&inproj, &inunit, &inzone, &indtmnum,
                             tie_parms.ddr.proj_coef, &outproj, &outunit, 
                             &outzone, &outdtmnum, proj_coef_out, prtprm, "");

            if (status != E_SUCC)
            {
               fprintf(stderr, "Error initializing projection parameters\n");
               return;
            }

            status = c_trans(&inproj,&inunit,&outproj,&outunit,
                            &tie_select.rtie[ptindex][0],
                            &tie_select.rtie[ptindex][1], &tempx, &tempy);

            if (status != E_SUCC)
                tie_message(Shell_Manual,
                      "Error converting search image projection coordinates\n "
                      "Verify the projection parameters are correct\n");

            /* copy results back to tie_select */
            tie_select.saveref[0] = tie_select.rtie[ptindex][0] = tempx;
            tie_select.saveref[0] = tie_select.rtie[ptindex][1] = tempy;

            /* 
             *  Call tps_update to update the reference tps file with 
             *  the datum information. 
             */
            tps_update();
        }

        tie_select.curref = TRUE;
    }
    else if (w == zoom_refwin_bulletinboard)
    {
        /*
         *  *** if reference, reset new reference point and mark it
         */
        if (tie_zoom[REFERENCE].flags & ZOOM_CLEAR)
        {
            XBell(tie.display, 0);
            return;
        }

        /*
         *  *** if old point exist, clear it
         */
        if (tie_select.curref)
            XClearArea(tie.display, XtWindow(zoom_refwin_bulletinboard),
                    tie_select.saveref[0]-5, tie_select.saveref[1]-5,
                    12, 12, True);
        else
        {
            XClearArea(tie.display, XtWindow(zoom_refwin_bulletinboard),
                    x, y, 1, 1, True);
            tie_select.curref = TRUE;
        }

        XWarpPointer(tie.display, None, XtWindow(zoom_srchwin_bulletinboard),
                None, None, None, None, x, y);

        tie_select.saveref[0] = x;
        tie_select.saveref[1] = y;
    }
    /*
     *  *** ADRG points are selected from the fullres window
     */
    else if (tie.flags & TIE_ADRG)
    {
        /*
         *  *** if old point exist, clear it
         */
        if (tie_select.curref)
            XClearArea(tie.display, XtWindow(ref_image_bulletinboard),
                    tie_select.saveref[0]-5, tie_select.saveref[1]-5,
                    12, 12, True);
        else
            XClearArea(tie.display, XtWindow(ref_image_bulletinboard),
                    x, y, 1, 1, True);

        tie_select.curref = TRUE;

        x += tie_ref_fullres[0].window[SS];
        y += tie_ref_fullres[0].window[SL];

        adrg_pix2ll(&tie_select.rtie[ptindex][1],
                &tie_select.rtie[ptindex][0],
                y, x, tie_adrg->image[0].img_ul[0],
                tie_adrg->image[0].img_ul[1], 
                tie_adrg->image[0].bs, tie_adrg->image[0].asc);

        tie_select.saveref[0] = (int)tie_select.rtie[ptindex][0];
        tie_select.saveref[1] = (int)tie_select.rtie[ptindex][1];
    }
}



/*******************************************************************************
Purpose:
    This routine processes a selected search point.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by T Rockvam, USGS/EROS Data Center, 3/94.
   - corrected # parms to tps_verify call

Algorithm Description:
*******************************************************************************/
void select_srchpt
(
    int x,
    int y
)
{
    int          ptindex, /* index of new/reselected point        */
                 ss, sl;  /* starting sample and line             */

    /*
     *  *** if reselecting, assign to previous point,else at new point
     */
    if (tie.flags & TIE_RESELECTING)
        ptindex = tie_select.reselect;
    else
        ptindex = tie_select.seltie;

    if (ptindex == tie_select.numalloc)
    {
        tie_select.numalloc += MAX_TIE_POINTS;
        tie_select.elev    =(double *)(realloc(tie_select.elev,
                    tie_select.numalloc*sizeof(double)));
        tie_select.rtie    =(void *)(realloc(tie_select.rtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.crtie    =(void *)(realloc(tie_select.crtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.stie    =(void *)(realloc(tie_select.stie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.lsrtie  =(void *)(realloc(tie_select.lsrtie,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.resid   =(void *)(realloc(tie_select.resid,
                    2*tie_select.numalloc*sizeof(double)));
        tie_select.pt_id   =(void *)(realloc(tie_select.pt_id,
                    tie_select.numalloc*PTIDLEN*sizeof(char)));
        tie_select.mapname =(void *)(realloc(tie_select.mapname,
                    tie_select.numalloc*80*sizeof(char)));
        tie_select.mapid   =(void *)(realloc(tie_select.mapid,
                    tie_select.numalloc*80*sizeof(char)));
        tie_select.disabled=(unsigned char *)(realloc(tie_select.disabled,
                    tie_select.numalloc*sizeof(unsigned char)));
        if ((tie_select.elev == NULL) || (tie_select.rtie == NULL) ||
                (tie_select.crtie == NULL) || (tie_select.stie == NULL) || 
                (tie_select.lsrtie == NULL) || (tie_select.resid == NULL) || 
                (tie_select.pt_id == NULL) || (tie_select.mapname == NULL) || 
                (tie_select.mapid == NULL) || (tie_select.disabled == NULL))
        {
            tie_select.numalloc -= MAX_TIE_POINTS;
            printf("Error allocating memory\n");
            XBell(tie.display,0);
            return;
        }
    }

    if ((tie.mode != ONESRCH) && (!tie_select.curref))
    {
        XBell(tie.display, 0);
        return;
    }

    /*
     *  *** if in single image mode
     */
    if (tie.mode == ONESRCH)
    {
        /*
         *  *** get coord from fullres window
         */
        ss = tie_srch_fullres[0].window[SS];
        sl = tie_srch_fullres[0].window[SL];

        tie_select.rtie[ptindex][0] = 0.0;
        tie_select.rtie[ptindex][1] = 0.0;

        /*
         *  *** save search zoom coordinates
         */
        tie_select.savesrch[0] = x;
        tie_select.savesrch[1] = y;

        cursor_sub(SEARCH, x, y, &tie_select.stie[ptindex][0],
                &tie_select.stie[ptindex][1], 1, 1,sl, ss);
    }
    /*
     *  *** if search, and reference select, mark and verify
     */
    else if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
    {
        XDrawPoint(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                tie_select.gc, x, y);
        XDrawArc(tie.display,XtWindow(zoom_srchwin_bulletinboard),tie_select.gc,
                x-4, y-4, 9, 9, 0, 360*64);

        ss = tie_ref_fullres[0].window[SS];
        sl = tie_ref_fullres[0].window[SL];

        cursor_sub(REFERENCE, tie_select.saveref[0], tie_select.saveref[1], 
                &tie_select.rtie[ptindex][0], 
                &tie_select.rtie[ptindex][1], 1, 1, sl, ss);
        if ((tie_parms.flags & PARMS_PROJ) || (tie_parms.flags & PARMS_LATLONG))
        {
            /* If user selected PROJECTION or LATLONG
               get those coordinates into crtie variable
               for output with LINE/SAMPLE coords
               --------------------------------------------*/
            image2proj(REFERENCE,tie_select.crtie[ptindex],
                    tie_select.rtie[ptindex][0],
                    tie_select.rtie[ptindex][1]);
            if (tie_parms.flags & PARMS_LATLONG)
                proj2degrees(tie_select.crtie[ptindex],1,REFERENCE);

            /* If only PROJECTION or LATLONG is selected
               then move line/sample coords to lsrtie variable
               and place converted coords(crtie) into
               rtie variable
               --------------------------------------------------*/
            if (!(tie_parms.flags & PARMS_LINESAMP))
            {
                tie_select.lsrtie[ptindex][0] = tie_select.rtie[ptindex][0];
                tie_select.lsrtie[ptindex][1] = tie_select.rtie[ptindex][1];
                tie_select.rtie[ptindex][0] = tie_select.crtie[ptindex][0];
                tie_select.rtie[ptindex][1] = tie_select.crtie[ptindex][1];
            }
        }

        /*
         *  *** get coord from fullres window
         */
        ss = tie_srch_fullres[0].window[SS];
        sl = tie_srch_fullres[0].window[SL];

        /*
         *  *** save search zoom coordinates
         */
        tie_select.savesrch[0] = x;
        tie_select.savesrch[1] = y;

        cursor_sub(SEARCH, x, y, &tie_select.stie[ptindex][0], 
                &tie_select.stie[ptindex][1], 1, 1,sl, ss);
    } 
    /*
     *  *** if  image to map
     */
    else if ((tie.mode == IM2MAP) || (tie.mode == IM2VECT) ||
            (tie.flags & TIE_ADRG))
    {
        XDrawPoint(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                tie_select.gc, x, y);
        XDrawArc(tie.display,XtWindow(zoom_srchwin_bulletinboard),tie_select.gc,
                x-4, y-4, 9, 9, 0, 360*64);

        /*
         *  *** get coord from fullres window
         */
        ss = tie_srch_fullres[0].window[SS];
        sl = tie_srch_fullres[0].window[SL];

        /*
         *  *** save search zoom coordinates
         */
        tie_select.savesrch[0] = x;
        tie_select.savesrch[1] = y;

        cursor_sub(SEARCH, x, y, &tie_select.stie[ptindex][0],
                &tie_select.stie[ptindex][1], 1, 1,sl, ss);

        if ((tie.mode == IM2VECT) && (tie_select.rtemp_flag))
        {
            tie_select.rtie[ptindex][0] = tie_select.rtemp[0];
            tie_select.rtie[ptindex][1] = tie_select.rtemp[1];
            tie_select.rtemp_flag = FALSE;
        }
    }

    if (!(tie.flags & TIE_RESELECTING))
    {
        tie_select.seltie++;

        if (tie_rt.calc_rt != RT_DDR)
            tie_select.numtie++;
    }

    tie_select.disabled[ptindex] = FALSE;

    tps_verify(tie_rt.calc_rt, &tie_select.numtie, &ptindex,
            tie_select.rtie, tie_select.stie);
}
