/*
 *  This is the show function source file for the show panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include "tiepts.h"
#include "protos.h"

/*******************************************************************************
Purpose:
    This routine applies the selected memory planes to an image display 
    panel.

By L. Huewe, USGS/EROS Data Center, 11/92

Algorithm Description:
*******************************************************************************/
void show_apply
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];        /* Variable used to set widget resources.     */
   Widget       image;          /* Image display                              */

   int          i,              /* Looping index.                             */
                img_idx,        /* REFERENCE / SEARCH overview                */
                nplanes,        /* Number of planes to be displayed.          */
                zero;           /* Number of planes which are zero.           */

   if (client == NULL)
      return;

   image = (Widget)client;
   if (image == ref_image_bulletinboard)
      img_idx = REFERENCE;
   else
      img_idx = SEARCH;

   for (i=0,zero=0; i < 3; i++)
      if (tie_show.planes[i] == 0)
         {
         zero++;
         switch(i)
            {
            case 0:
               tie_show.planes[i] = (tie_show.planes[i+1] > 0) ? 
                  tie_show.planes[i+1] : tie_show.planes[i+2]; 
               break;

            case 1:
                tie_show.planes[i] = (tie_show.planes[i-1] > 0) ? 
                   tie_show.planes[i-1] : tie_show.planes[i+1]; 
                break;
            case 2:
                tie_show.planes[i] = (tie_show.planes[i-1] > 0) ? 
                   tie_show.planes[i-1] : tie_show.planes[i-2]; 
                break;
            }
         }

   if (zero == 3)
      return;

   if ((tie_show.planes[RED] == tie_show.planes[GREEN]) && 
       (tie_show.planes[GREEN] == tie_show.planes[BLUE]))
      nplanes = 1;
   else
      nplanes = 3;

   if (tie_mem[REFERENCE].imgpix)
      {
      XFreePixmap(tie.display,tie_mem[REFERENCE].image_pixmap);
      tie_mem[REFERENCE].imgpix = FALSE;
      }

   if (nplanes == 1)
      load_display(tie_show.planes,&nplanes,XtWindow(image), Shell_RefImg,image,
           &tie_planes[tie_show.planes[RED] - 1].dspwin[NL],
           &tie_planes[tie_show.planes[RED] - 1].dspwin[NS],
           &tie_planes[tie_show.planes[RED] - 1].maxval,
           tie_planes[tie_show.planes[RED] - 1].image_data,NULL,NULL,TRUE,
           &tie_mem[img_idx].image_pixmap,TRUE);
   else
      load_display(tie_show.planes,&nplanes,XtWindow(image), Shell_RefImg,image,
           &tie_planes[tie_show.planes[RED] - 1].dspwin[NL],
           &tie_planes[tie_show.planes[RED] - 1].dspwin[NS],
           &tie_planes[tie_show.planes[RED] - 1].maxval,
           tie_planes[tie_show.planes[RED] - 1].image_data,
           tie_planes[tie_show.planes[GREEN] - 1].image_data,
           tie_planes[tie_show.planes[BLUE] - 1].image_data,TRUE,
           &tie_mem[img_idx].image_pixmap,TRUE);

   tie_show.planes[RED] = tie_show.planes[GREEN] = tie_show.planes[BLUE] = 0;

   XtSetArg(args[0],XmNlabelString,XmStringCreateLtoR("BW",
       XmSTRING_DEFAULT_CHARSET));
   XtSetValues(show_bw_button,args,1);

   XtSetArg(args[0],XmNlabelString,XmStringCreateLtoR("Red",
       XmSTRING_DEFAULT_CHARSET));
   XtSetValues(show_red_button,args,1);

   XtSetArg(args[0],XmNlabelString,XmStringCreateLtoR("Green",
       XmSTRING_DEFAULT_CHARSET));
   XtSetValues(show_green_button,args,1);

   XtSetArg(args[0],XmNlabelString,XmStringCreateLtoR("Blue",
       XmSTRING_DEFAULT_CHARSET));
   XtSetValues(show_blue_button,args,1);
}

/*******************************************************************************
Purpose:
   Initialize the memory plane display panel.  The reference image may
   access planes 1, 2, and 3.  The search image may access plane 4, 5, and 6.

By L. Huewe, USGS/EROS Data Center, 11/92

Algorithm Description:
*******************************************************************************/
void show_create
(
    int image_idx       /* Plane index for given image                        */
)
{
   Arg      args[3];    /* Variable used to set widget resources.             */
   
   XmString *planes;    /* Array of compound strings defining valid plane
                              numbers.                                        */
   
   int      i,          /* Looping index.                                     */
            plane_idx,  /* Plane string index.                                */
            plane_cnt;  /* Number of valid planes.                            */
    
   char     tmpstr[50]; /* Temporary string variable.                         */
   
   for (i=image_idx,plane_cnt=0; i < image_idx+3; i++)
      if (tie_planes[i].flags & USED)
         plane_cnt++;

   if (plane_cnt == 0)
      {
      tie_message(Shell_Show, "No image data available for displaying");
      return;
      }
   else if ((planes = (XmString *) calloc(plane_cnt,sizeof(XmString))) == NULL)
      {
      tie_message(Shell_Show, "Insufficient memory available");
      return;
      }

   plane_idx = 0;

  /*
   *  *** set textlist items to corresponding memory plane indeces
   */
   for (i=image_idx; i < image_idx+3; i++)
      {
      if (tie_planes[i].flags & USED)
         {
         sprintf(tmpstr,"Plane %d",i+1);
         planes[plane_idx++] = 
                   XmStringCreateLtoR(tmpstr,XmSTRING_DEFAULT_CHARSET);
         }
      }

   XtSetArg(args[0],XmNitems,planes);
   XtSetArg(args[1],XmNitemCount,plane_cnt);

   if (plane_cnt < 4)
      XtSetArg(args[2],XmNvisibleItemCount,plane_cnt);
   else
      XtSetArg(args[2],XmNvisibleItemCount,4);

   XtSetValues(show_plane_list,args,3);

   for (i=0; i < plane_cnt; i++)
      XmStringFree(planes[i]);

   free(planes);
   tie_show.planes[RED] = tie_show.planes[GREEN] = tie_show.planes[BLUE] = 0;

   return;
}

/*******************************************************************************
Purpose:
    This remove Show panel.

By L. Huewe, USGS/EROS Data Center, 11/92

Algorithm Description:
*******************************************************************************/
void show_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XtUnmanageChild(Show_form);
   XtUnrealizeWidget(Shell_Show);
   XtDestroyWidget(Show_form);
   Show_form = NULL;
   XtSetSensitive(main_show_button,True);
}

/*******************************************************************************
Purpose:
    This routine is the callback for selecting a plane from the show textlist.

By L. Huewe, USGS/EROS Data Center, 11/92

Algorithm Description:
*******************************************************************************/
void show_plane_select
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];        /* Variable used to set widget resources.     */

   static 
   char         labels[4][6] = {"Red","Green","Blue","BW"};
                                /* Default labels for radio buttons.          */

   static 
   XmString     xmlabels[4];    /* Motif strings for radio button labels.     */

   static 
   int          first = TRUE;   /* Indicates whether or not first time into
                                   the routine.                               */

   int          i,              /* Looping index.                             */
                plane;          /* Plane number selected.                     */
   char         *planestr;      /* Temporary string variable.                 */

   XmListCallbackStruct *acs=(XmListCallbackStruct*)call_data;

   if (first)
      {
      for (i=0; i < 4; i++)
          xmlabels[i] = XmStringCreateLtoR(labels[i],XmSTRING_DEFAULT_CHARSET);
      first = FALSE;
      }

   XmStringGetLtoR(acs->item,XmSTRING_DEFAULT_CHARSET,&planestr);
   if (tie_show.component != BW)
      {
      sscanf(planestr,"Plane %d",&plane);
      if (plane == tie_show.planes[tie_show.component])
         {
         tie_show.planes[tie_show.component] = 0;
         XtSetArg(args[0],XmNlabelString,xmlabels[tie_show.component]);
         }
      else
         {
         tie_show.planes[tie_show.component] = plane;
         XtSetArg(args[0],XmNlabelString,xmlabels[BW]);
         XtSetValues(show_bw_button,args,1);
         XtSetArg(args[0],XmNlabelString,acs->item);
         }

      switch (tie_show.component)
         {
         case RED:
            XtSetValues(show_red_button,args,1);
            break;

         case GREEN:
            XtSetValues(show_green_button,args,1);
            break;

         case BLUE:
            XtSetValues(show_blue_button,args,1);
            break;
         }
      }
   else
      {
      sscanf(planestr,"Plane %d",&plane);
      if ((plane == tie_show.planes[RED]) && (plane == tie_show.planes[GREEN])
         && (plane == tie_show.planes[BLUE]))
         {
         tie_show.planes[RED] = tie_show.planes[GREEN] 
                              = tie_show.planes[BLUE] 
                              = 0;
         XtSetArg(args[0],XmNlabelString,xmlabels[3]);
         XtSetValues(show_bw_button,args,1);
         }
      else
         {
         tie_show.planes[RED] = tie_show.planes[GREEN] 
                              = tie_show.planes[BLUE] 
                              = plane;
         XtSetArg(args[0],XmNlabelString,acs->item);
         XtSetValues(show_bw_button,args,1);
         XtSetArg(args[0],XmNlabelString,xmlabels[RED]);
         XtSetValues(show_red_button,args,1);
         XtSetArg(args[0],XmNlabelString,xmlabels[GREEN]);
         XtSetValues(show_green_button,args,1);
         XtSetArg(args[0],XmNlabelString,xmlabels[BLUE]);
         XtSetValues(show_blue_button,args,1);
         }
      }
 
   XtFree(planestr);
}

/*******************************************************************************
Purpose:
    This routine is called when a color plane toggle button is selected.

By L. Huewe, USGS/EROS Data Center, 11/92

Algorithm Description:
*******************************************************************************/
void show_component
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   if (*(char*)client == 'R')
      tie_show.component = RED;
   else if (*(char*)client == 'G')
      tie_show.component = GREEN;
   else if (*(char*)client == 'B')
      tie_show.component = BLUE;
   else
      tie_show.component = BW;
}
