#ifndef TIEPTS_H
#define TIEPTS_H

/*
 *  This is the main include file for the Tie Point Selection application.  It
 *  contains references for all other include files required by the application
 *  and is designed to be called once in each source file.
 *
 *  This file originated from the X Image Display application.
 *
 *  by J. Hemmer, USGS/EROS Data Center, 2/92.
 *    - Tie Point Selection, v1.0.
 *  Modified by J. LaVergne, USGS/EROS Data Center, 7/93
 *    - changed the dimension of the TIE_RT structure elements from 4 to 5 
 *      to handle reading the 5th line of data from an .rt file.
 *  Modified by T Rockvam, USGS/EROS Data Center, 3/94
 *    - removed unneeded items; fixed wdb file structure; other misc 1.1 changes
 *  Modified by C. Olsen, USGS/EROS Data Center, 8/98
 *    - added flags for coordinate types, new members to TIE_STRUCT - rvalid
 *      and svalid, TIE_SELECT - crtie and cstie, and TIE_PARMS - flags
 *      (SRF #1935)
 *  Modified by C. Olsen, USGS/EROS Data Center, 9/98
 *    - added character array (coord) in TIE_PARMS to hold IM2VECT coordinate
 *      type  (SRF #1958)
 */
#include <stdio.h>
#include <stdlib.h>

/*
 * Required MOTIF include files.
 */
#include <X11/Xlib.h>
#include <X11/StringDefs.h>

#include <Xm/Xm.h>
#include <X11/Shell.h>
#include <X11/keysym.h>
#include <Xm/ArrowB.h>
#include <Xm/ArrowBG.h>
#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>
#include <Xm/CascadeBG.h>
#include <Xm/Command.h>
#include <Xm/CutPaste.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/DrawnB.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/List.h>
#include <Xm/MainW.h>
#include <Xm/MenuShell.h>
#include <Xm/MessageB.h>
#include <Xm/PanedW.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>
#include <Xm/SelectioB.h>
#include <Xm/SeparatoG.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include "ProcessM.h"

/*
 *  *** Declaration of all widget variables.
 */
#include "widget.h"
#include "las.h"
#include "datum.h"
#include "geompak.h"
#include "dcw.h"
#include "proj.h"
#include "ptio.h"

/*******************************************************************************
                          Application Defines
*******************************************************************************/
/*
 *  *** Macros.
 */
#define DEGTORAD(x)     (x)/57.29577    /* Degrees to radians function.       */

/*
 *  *** Working debug flag.
 */

#define TIE_DEBUG
#define TIEPTS 

/*
 *  *** Application name and class constants.
 */
#define BX_APP_NAME "tie"
#define BX_APP_CLASS "TIEpts"

/*
 *  *** Modes of operation.
 */
#define IM2IM           0       /* image to image mode                        */
#define IM2MAP          1       /* image to map mode                          */
#define IM2VECT         2       /* image to vector mode                       */
#define ONESRCH         3       /* single search image mode                   */
#define IM2ADRG         4       /* image to ARG digitized raster graphics     */

/*
 *  *** Rough transformation states
 */
#define RT_SEL          0       /* rough trans selection state                */
#define RT_DDR          1       /* rough trans calculated from ddr            */
#define RT_CAL          2       /* rough trans calculated from selected pts   */
#define RT_FIL          3       /* rough trans calc from temp file            */

/*
 *  *** Vector file types
 */
#define DLG             0       /* digital line graph file                    */
#define WDB             1       /* world data base data file                  */
#define DCW             2       /* digital chart of the world                 */
#define DLGOPT          3       /* digital line graph optional format         */

#define MAXATTRIB       25      /* maximum number of vector files loaded      */
#define VECTOR_FILE_BLOCK_SIZE 150 /* size to grow tie_vectfile allocation by 
                                      when more memory is needed */

#define WDBFEATURES     6       /* number of WDB features                     */

/*
 *  *** Vector Transformations
 */
#define TRANSLATE       1       
#define ROTATE          2
#define SCALE           3       

/*
 *  *** graphic display status
 */
#define REFIMAGE_GRAPHICS    0x1        /* display graphics over ref image    */
#define SRCHIMAGE_GRAPHICS   0x2        /* display graphics over search image */

/*
 *  *** General constants.
 */
#define DIR_CHAR       '/'      /* reference image index                      */
#define REFERENCE       0       /* reference image index                      */
#define SEARCH          1       /* search image index                         */
#define NAMLEN          CMLEN   /* Maximum file name length.                  */
#define TIE_S           E_SUCC  /* Success!                                   */
#define TIE_F           E_FAIL  /* Failure ...                                */
#define NUM_COLORS      8       /* Total number of working colors.            */
#define BLACK_COL       0       /* Indices into the working color array.      */
#define WHITE_COL       1
#define RED_COL         2
#define GREEN_COL       3
#define BLUE_COL        4
#define NUM_WMCOLORS    12      /* # of colorcells used to minimize flashing. */
#define WM_IDX          0       /* Index into color table for window manager
                                   colors.                                    */
#define NUM_CVALS       8       /* # of colorcells used for drawing colors.   */
#define NUM_LINES       6       /* # of line styles used for drawing vectors  */
#define RGBBW_IDX       12      /* Index into color table for drawing colors. */
#define NUM_WKCOLOR     1       /* # of work colorcells for PSD/MASK.         */
#define WORK_IDX        20      /* Index into color table for work color cell.*/
#define NUM_CSTORE      35      /* # of colorcells in color store.            */
#define CSTORE_IDX      21      /* Index into color table for stored colors.  */
#define NUM_IMGCOLORS   200     /* # of image data colorcells.                */
#define IMG_IDX         56      /* Index into color table for image colors.   */
#define RGB_DATABASE    "/usr/lib/X11/rgb.txt"
                                /* X RGB color database.                      */
#define NUM_VIEW        2       /* Number of composite views to hold
                                   information for.                           */
#define MAX_NTILES      256     /* Maximum number of tiles for ADRG */

#define MAXCOLOR 65535          /* Maximum color value.                       */
#define MINCOLOR 0              /* Minimum color value.                       */

/*
 *  *** Font constants
 */
#define DEFFONT         0       /* default font index                   */
#define POINTERFONT     1       /* cursor coordinate font               */
#define SMALLFONT       2       /* general small font index             */
#define MEDIUMFONT      3       /* general medium font index            */
#define LARGEFONT       4       /* general large font index             */

/* 
 *  *** Boolean values.
 */
#ifndef TRUE
#define FALSE           0
#define TRUE            1
#endif

/*
 *  *** Array indices.
 */
#define RED             0
#define GREEN           1
#define BLUE            2
#define BW              3

/* 
 *  *** Projection Information Package defines.
 */
#define DEGREES         4       /* Output projection units.             */
#define CLARKE          0       /* Output datum code; Clarke 1866.      */
#define NULZONE         62      /* Output zone code; GCTP constant that
                                   signifies a null zone.               */

#define FEET2CM         30.48   /* Feet to meters centimeters           */

/*
 *  ***  Default image plane size and number and viewport size.
 */
#define DEF_HEIGHT      1024
#define DEF_WIDTH       1024
#define DEF_VHEIGHT      512
#define DEF_VWIDTH       512
#define DEF_NPLANE         6

#define DEF_MAP_CATALOG "./catalog.txt"

/*
 *  *** Bit flags.
 */
#define USED    0x1             /* Has data been loaded into plane?           */
#define LOCK    0x2             /* Is the data in this plane locked?          */

#define TIE_SELECTING   0x2     /* Indicates selecting points.                */
#define TIE_RECTCUR     0x4     /* Indicates sliding rectangle.               */
#define TIE_ALIGNING    0x8     /* Indicates aligning vectors.                */
#define TIE_ADRG        0x10    /* Indicates aligning vectors.                */
#define TIE_RESELECTING 0x20    /* Indicates reselecting an existing point.   */
#define TIE_OPEN        0x40    /* Indciates TPS files successfully opened.   */

/*
 *  *** Cursor defines.
 */
#define CURSOR_ACTIVE   0x1     /* Cursor active flag.                        */

#define CURSOR_XY       0x2     /* Display values in XY dimension.            */
#define CURSOR_Z        0x4     /* Display values in Z dimension.             */

#define CURSOR_MAPPED   0x8     /* Display mapped gray level values.          */
#define CURSOR_ORIG     0x10    /* Display original gray level values.        */

#define CURSOR_AUTO     0x20    /* Update requested information automatically.*/
#define CURSOR_MAN      0x40    /* Update requested information based on user
                                   request.                                   */

#define CURSOR_PROJ     0x80    /* Display projection coordinates.            */
#define CURSOR_LATLONG  0x100   /* Display latitude/longitude values.         */
#define CURSOR_UNSCAL   0x200   /* Display unscaled data values.              */

#define CURSOR_HEIGHT    70     /* Height of cursor information drawing area. */
#define CURSOR_WIDTH    434     /* Width of cursor information drawing area.  */

/*
 *  *** Parameters window defines.
 */
#define PARMS_LINESAMP  0x1     /* Indicates line/sample values.              */
#define PARMS_PROJ      0x2     /* Indicates projection coordinates.          */
#define PARMS_LATLONG   0x4     /* Indicates latitude/longitude values.       */
#define PARMS_MULTI     0x8     /* Indicates multiple reference images.       */

/*
 *  *** Full resolution window defines.
 */
#define FULLRES_ACTIVE  0x1     /* Full resolution window active flag.        */
#define FULLRES_USER    0x2     /* User entered window size flag.             */

/*
 *  *** map unit types
 */
#define MAP_RADIANS     0       /* map coordinates in radians           */
#define MAP_FEET        1       /* map coordinates in feet              */
#define MAP_METERS      2       /* map coordinates in meters            */
#define MAP_SECONDS     3       /* map coordinates in seconds           */
#define MAP_DEGREES     4       /* map coordinates in degrees           */
#define MAP_DMS         5       /* map coordinates in degrees/min/sec   */
#define MAP_USERDEF     6       /* map coordinates in degrees/min/sec   */
#define MAP_PROJ        7       /* map coordinates in projection coords */
#define MAP_MINUTES     8       /* map coordinates in minutes           */

/*
 *  *** General Color Map defines.
 */
#define MAP_RED         0x1     /* Map red component of the image.            */
#define MAP_GREEN       0x2     /* Map green component of the image.          */
#define MAP_BLUE        0x4     /* Map blue component of the image.           */
#define MAP_ALL         0x8     /* Map all components of the image.           */

/*
 *  Zoom defines.
 */
#define ZOOM_ACTIVE     0x1     /* Zoom window active flag.                   */
#define ZOOM_CLEAR      0x2     /* Zoom window clear flag.                    */

#define ZOOM_MAIN       0x40    /* Main window is zoomed.                     */
#define ZOOM_FULLRES    0x80    /* Full resolution window is zoomed.          */

#define ZOOM_WINX        64     /* Number of samples in zoom window.          */
#define ZOOM_WINY        64     /* Number of lines in zoom window.            */


/*
 *  *** polynomial order definitions
 */
#define FIRST_ORDER       1     /* first order polynomial                    */
#define SECOND_ORDER      2     /* second order polynomial                   */
#define THIRD_ORDER       3     /* second order polynomial                   */

/*
 *  *** map registration modes
 */
#define SELECT          0x1     /* selecting points                          */
#define VERIFY          0x2     /* selecting verify point                    */
#define EDIT            0x3     /* reselecting points                        */

/*
 *  *** Utility defines.
 */
#define PTIDLEN         20      /* Maximum # of colors available in the
                                   color list.                                */
#define MAX_COLORS      1000    /* Maximum # of colors available in the
                                   color list.                                */

/*
 *  *** Allocation defines and prototypes
 */
#define TieMalloc(x) tie_malloc((x), __FILE__, __LINE__)
#define TieCalloc(x, y) tie_calloc((x), (y), __FILE__, __LINE__)
#define TieRealloc(x, y) tie_realloc((x), (y), __FILE__, __LINE__)

void *tie_malloc(size_t, char *, int);
void *tie_calloc(size_t, size_t, char *, int);
void *tie_realloc(void *, size_t, char *, int);

/******************************************************************************* "Generic" Structures
*******************************************************************************/

/*
 *  ***  Main application structure.
 */
struct TIE_STRUCT
 {
 Display        *display;       /* Pointer for X display structure.           */
 XtAppContext   context;        /* Application context.                       */
 Pixmap         icon_pixmap;    /* Pixmap for the application icon.           */
 Colormap       colormap;       /* Working color map.                         */
 XVisualInfo    *visual_info;   /* Visual information structure.              */
 Visual         *visual;        /* Default visual.                            */
 XColor         color_table[256];
                                /* The current color table.                   */
 int            red_shift;      /* shift value: 0, 8, 16, or 24 */
 int            green_shift;    /* shift value: 0, 8, 16, or 24 */
 int            blue_shift;     /* shift value: 0, 8, 16, or 24 */
 XColor         color_workclear;/* Used to reset color_workcell.              */
 XFontStruct    *font_info;     /* General font information.                  */
 GC             bit_gc;         /* graphic plane GC.                          */
 GC             rect_gc;        /* Sliding rectangle GC.                      */

 char           color_name[NUM_CSTORE][25];
                                /* Color store color strings; used for
                                   comparisons.                               */
 int            cindex,         /* Index into color store.                    */
                current_plane,  /* Current plane number.                      */
                flags,          /* Bit flags                                  */
                graphics,       /* Graphic display flags                      */
                line_x,         /* Tracking line X position.                  */
                line_y,         /* Tracking line Y position.                  */
                logo,           /* Boolean for display of logo                */
                max_plane,      /* Maximum plane number.                      */
                mode,           /* Op mode (IM2IM or IM2MAP).                 */
                numimgs[2],     /* Number of images loaded                    */
                num_ref_fullres,/* The number of TIE_FULLRES structures
                                   allocated for reference image.             */
                num_srch_fullres,/* The number of TIE_FULLRES structures
                                   allocated for search image.                */
                refrect_x,      /* Sliding Rectangle X position (ref image)   */
                refrect_y,      /* Sliding Rectangle Y position (ref image)   */
                rvalid,         /* Reference ddr validity.                    */
                rview,          /* Reference composite index.                 */
                screen,         /* X screen value.                            */
                srchrect_x,     /* Sliding Rectangle X position (srch image)  */
                srchrect_y,     /* Sliding Rectangle Y position (srch image)  */
                svalid,         /* Search ddr validity.                       */
                sview,          /* Search composite index.                    */
                userplace,      /* Allow user to place all panels             */
                x,              /* Current pointer X position.                */
                y;              /* Current pointer Y position.                */
 pid_t          help_pid;       /* Process ID of help process.                */
 };

/*
 *  *** Tie point selection information
 */
struct TIE_SELECT
   {
   GC           gc;             /* Point selection GC.                        */
   XFontStruct  *font_info;     /* Font structure for points                  */
   struct PTDESC *ref_desc,     /* Reference tps output descriptor            */
                 *srch_desc;    /* Search tps output descriptor               */
   double       *elev;          /* Tie point elevation                        */
   double       (*rtie)[2];     /* Reference tie point selections             */
   double       (*stie)[2];     /* Search tie point selections                */
   double       (*resid)[2];    /* Residual values for each tie point         */
   double       (*crtie)[2];    /* Converted reference tie points             */
   double       (*lsrtie)[2];   /* Tie points in line/sample format           */
   double       rtemp[2];       /* Temporary storage for reference tie point in
                                   Image to vector mode                       */
   char         (*pt_id)[PTIDLEN]; /* Unique point identifiers                */
   char         (*mapname)[80]; /* Map name                                   */
   char         (*mapid)[80];   /* Map id                                     */
   unsigned 
   char         *disabled;      /* Point is currently disabled                */
   int          curref,         /* Reference pt selected flag                 */
                genpt,          /* New reference point                        */
                mapcolor,       /* Color of map outlines                      */
                saveref[2],     /* new reference point                        */
                savesrch[2],    /* new search point                           */
                numalloc,       /* Number of tie pts allocated                */
                numtie,         /* Number of tie pts                          */
                seltie,         /* Number of tie pts selected by user         */
                reselect,       /* index of point being reselected            */
                rtemp_flag;     /* has the rtemp field been set?              */
   };

/*
 *  *** Rough transformation information
 */
struct TIE_RT
   {
   double       line_coef[6],   /* Coef. for mapping line coordinates         */
                samp_coef[6],   /* Coef. for mapping sample coordinates       */
                old_lcoef[6],   /* temporary storage of last coef             */
                old_scoef[6];   /* temporary storage of last coef             */
   int          calc_rt;        /* Rough trans. calculation state             */
   int          degree;         /* degree of polynomial fit                   */
   int          rtroff[5][2],   /* Rough transformation reference offsets     */
                rtrzm[5],       /* Rough transformation reference zoom factors*/
                rtsoff[5][2],   /* Rough transformation search offsets        */
                rtszm[5];       /* Rough transformation search zoom factors   */
   FILE         *rtfp;          /* Rough trans. selection file descriptor     */
   };

/*
 *  *** Image plane structure.
 */
struct TIE_MEMPLANE
 {
 double         offset;         /* Offset used to scale non-byte data.        */
 double         slope;          /* Slope used to scale non-byte data.         */
 double         mean;           /* Mean data value.                           */
 double         stdev;          /* Standard deviation.                        */
 int            plane_number;   /* Memory plane number.                       */
 int            window[4];      /* Image window.                              */
 int            dspwin[4];      /* Display window.                            */
 int            band;           /* Band number.                               */
 int            image_type;     /* Image type -- See defines.                 */
 int            histogram[256]; /* Histogram data.                            */
 int            minval;         /* Minimum value.                             */
 int            maxval;         /* Maximum value.                             */
 int            skipl;          /* Line skip factor.                          */
 int            skips;          /* Sample skip factor.                        */
 int            proj_valid;     /* Does image_info contain valid projection
                                   information?                               */
 struct DDR     ddr;            /* Information about image (projection 
                                   information, nl, ns, nbands).              */
 long           units;          /* cached translation of the ddr.units member
                                   to prevent repeated conversion of it */
 int            flags;          /* Bit flags 
                                   - Has data been loaded into this plane?
                                   - Is the data in this plane locked?        */
 unsigned char  *image_data;    /* Original image data.                       */
 char           image_name[NAMLEN];
                                /* Image filename (UNIX format).              */
 char           filename[NAMLEN];
                                /* Image filename (LAS format).               */
 };

/*
 *  *** Tie Points Parameters
 */
struct TIE_PARMS
  {
  unsigned char sholab;         /* Flag for display of point labels           */
  unsigned char vsource;        /* vector data source                         */
  int           factor,         /* Zoom factor.                               */
                fheight,        /* Height of full resolution window.          */
                fwidth;         /* Width of full resolution window.           */
  int           order;          /* Order of Least Squares Fit                 */
  int           refoff[2],      /* Reference offset                           */
                srchoff[2],     /* Search offset                              */
                zsize;          /* Zoom window size                           */ 
  char          coord[13];      /* Coordinate type                            */
  char          epoint[13];     /* Elevation point requirement                */
  char          datumreq[13];   /* Datum value requirement                    */
  char          method[6];      /* Resampling method                          */
  char          intps[2][256];  /* input TPS filenames                        */
  char          outtps[2][256]; /* output TPS filenames                       */
  char          port[256];      /* digitizer port pathname                    */
  char          cd[256];        /* cd pathname                                */
  float         alpha;          /* Parametric cubic convolution alpha value   */
  unsigned char manual;         /* Flag denoting manual entry                 */
  int           flags;          /* Flag denoting coordinate type(s)           */
  struct DDR ddr;               /* Information about image (projection
                                   information, datum, zone)                  */
  };

/*
 *  ***  Memory layout structure.
 */
struct TIE_MEMLAYOUT
 {
 Pixmap         image_pixmap;   /* Pixmap for displayed image data.           */
 Pixmap         graphics_pixmap;/* Pixmap for displayed graphics data.        */
 int            imgpix;         /* Has an image pixmap been created?          */
 int            mem_width;      /* Width of the memory plane.                 */
 int            mem_height;     /* Height of the memory plane.                */
 int            view_width;     /* Width of the viewport.                     */
 int            view_height;    /* Height of the viewport.                    */
 int            view_xoffset;   /* Offset to be added to viewport width to 
                                   allow the appropriate image size to show 
                                   through.                                   */
 int            view_yoffset;   /* Offset to be added to viewport height to 
                                   allow the appropriate image size to show 
                                   through.                                   */
 };

/*
 *  *** Composite structure.
 */
struct TIE_COMPOSITE
 {
 int            rgbplanes[3];   /* Red, green and blue planes used to form 
                                   this composite.                            */
 };

/*******************************************************************************
                      X Image Display Application Structures
*******************************************************************************/

/*
 *  *** Memory plane layout structure.
 */
struct TIE_LAYOUT
 {
 int            mem_width;      /* Memory plane width.                        */
 int            mem_height;     /* Memory plane height.                       */
 int            mem_depth;      /* Memory plane depth.                        */
 int            view_width;     /* Viewport width.                            */
 int            view_height;    /* Viewport height.                           */
 };

/*
 *  *** Load structure.
 */
struct TIE_LOAD
 {
 int            windows[MAXIMG][4];
                                /* Window specification for each input image. */
 int            bands[MAXIMG][MAXBND+1];
                                /* Band numbers for each input image.         */
 int            nbands[MAXIMG]; /* Number of bands for each input image.      */
 int            totbnd;         /* Total number of bands specified.           */
 int            nimgs;          /* Number of images from input specification. */
 int            skipl;          /* Line skip factor.                          */
 int            skips;          /* Sample skip factor.                        */
 char           image_name[NAMLEN];
                                /* Image name.                                */
 char           tnames[MAXIMG][CMLEN];
                                /* LAS names from the input specification.    */
 char           hnames[MAXIMG][CMLEN];
                                /* Host names from the input specification.   */
 XmString       dirmask;        /* Directory path of input file               */
 };


/*
 *  *** Cursor structure.
 */
struct TIE_CURSOR
 {
 Pixmap         info_pixmap;    /* Pixmap for cursor information.             */
 Pixmap         clear_pixmap;   /* Pixmap for clearing the cursor window.     */
 Pixmap         pointer_pixmap; /* Pixmap for cursor pointer.                 */
 GC             gc;             /* Cursor window GC.                          */
 GC             pointer_gc;     /* Pointer GC.                                */
 XColor         color;          /* Cursor color                               */
 XFontStruct    *cursor_font;   /* Pointer to font structure for pointer
                                   cursor.                                    */
 KeyCode        h_key,          /* 'h' or left key.                           */
                j_key,          /* 'j' or down key.                           */
                k_key,          /* 'k' or up key.                             */
                l_key,          /* 'l' or right key.                          */
                left_key,       /* Left arrow key.                            */
                down_key,       /* Down arrow key.                            */
                up_key,         /* Up arrow key.                              */
                right_key,      /* Right arrow key.                           */
                ret_key;        /* Enter key.                                 */
 double         coef[6];        /* Coefficients mapping image coordinates
                                   to projection coordinates.                 */
 double         projparm[15];   /* Projection paramters for GCTP.             */
 int            zplane;         /* Current plane for Z dimension.             */
 int            max_zplane;     /* Maximum plane for Z dimension.             */
 int            proj_valid;     /* Is projection information valid?           */
 long           geo;            /* Output projection system.                  */
 long           outdatum;       /* Output datum code.                         */
 long           outproj;        /* Output projection units.             */
 long           outunit;        /* Output unit code for projection 
                                   coordinates.                               */
 long           outzone;        /* Output zone code.                          */
 int            flags;          /* Bit flags
                                   - Active flag
                                   - Dimension to display values in
                                   - Update mode
                                   - Display projection coordinates?
                                   - Display lat/long values?
                                   - Display unscaled data values?            */
 };

/*
 *  *** Full-resolution window structure.
 */
struct TIE_FULLRES
 {
 Pixmap         image_pixmap;   /* Full resolution window pixmap.             */
 double         offset;         /* Offset used to scale non-byte data.        */
 double         slope;          /* Slope used to scale non-byte data.         */
 double         mean;           /* Mean data value.                           */
 double         stdev;          /* Standard deviation.                        */
 int            dspwin[4];      /* Display window.                            */
 int            histogram[256]; /* Histogram data.                            */
 int            minval;         /* Minimum value.                             */
 int            maxval;         /* Maximum value.                             */
 int            skipl;          /* Line skip factor.                          */
 int            skips;          /* Sample skip factor.                        */
 int            window[4];      /* Full resolution window loaded.             */
 int            flags;          /* Bit flags
                                   - Active flag                              
                                   - User entered window size flag            */
 unsigned char  *image_data;    /* Image data loaded.                         */
 };

/*
 *  *** General map structure.
 */
struct TIE_CMAP
 {
 Widget         subpanel;       /* Map subpanel currently selected.           */
 int            imgidx;         /* Current Image Display index                */
 int            flags;          /* Bit flags
                                   - Color component to map                   */
 };

/*
 *  *** ARC Digitized Raster Graphics Data
 */
struct TIE_ADRGIMG
 {
 char           name[CMLEN];    /* general information filename              */
 int            asc,            /* East-West pixel spacing for fullres       */
                bs;             /* North-South pixel spacing for fullres     */
 int            ncols,          /* number of columns of tiles                */
                nrows;          /* number of rows of tiles                   */
 double         img_ul[2],      /* image upper-left coordinate               */
                img_lr[2];      /* image lower-right coordinate              */
 };

struct TIE_ADRGPARM
 {
 char           genfile[CMLEN]; /* general information filename              */
 int            ascovr,         /* East-West pixel spacing for overview      */
                bsovr,          /* North-South pixel spacing for overview    */
                extline,        /* extraction line                           */
                extsamp,        /* extraction sample                         */
                nimages,        /* number of images on CD                    */
                scale;          /* map scale                                 */
 double         ovr_ul[2];      /* overview upper-left coordinate            */
 struct 
 TIE_ADRGIMG    *image;         /* image information                         */
 };
 
/*
 *  *** Vector Map structure.
 */
#define TIE_LINE DCW_LINE

struct TIE_AREA
 {
 int            id;             /* feature code or identification           */
 int            attrib;         /* feature code or identification           */
 int            npts;           /* number of points in the line             */
 int            *nodes;         /* list of node indeces                     */
 };

struct TIE_VECTFILE
 {
 char           *filename;      /* name of dlg file                     */
 int            startnode;      /* index of first node from file        */
 int            endnode;        /* index of last node from file         */
 int            startarea;      /* index of first areafrom file         */
 int            endarea;        /* index of last area from file         */
 int            startline;      /* index of first areafrom file         */
 int            endline;        /* index of last area from file         */
 double         corner[4][2];   /* corner coordinates                   */

 long           inproj;         /* Input projection zone                */
 long           inunit;         /* Input units                          */
 long           inzone;         /* Input projection zone                */
 double         inparm[15];     /* Input projection parameters          */
 long           indatum;        /* Input datum code.                    */
 };

struct TIE_VECTOR
 {
 Pixmap         pixmap[NUM_CVALS]; 
                                /* Pixmap for vector display.               */
 Pixmap         blend_mask;     /* bitmap of vector display                 */
 unsigned char  type;           /* vector file type                         */
 unsigned char  *color_att;     /* color of each attribute                  */
 unsigned char  *line_att;      /* line style of each attribute             */
 unsigned char  *enabled;       /* attributes that are enabled              */
 unsigned char  align;          /* flag denoting selection by alignment     */
 unsigned char  transform;      /* current vector transformation            */
 unsigned char  blend;          /* blend factor                             */
 int            nattrib;        /* number of different attributes           */
 double         llx, lly,       /* lower left x and y                       */
                tmatrix[3][3],  /* transformation matrix                    */
                urx, ury,       /* upper right x and y                      */
                zmatrix[3][3];  /* zoom transformation matrix               */
 struct
 TIE_LINE       *line;          /* list of lines                            */
 int            maxline;        /* number of lines currently allocated      */
 int            numlines;       /* number of lines defined                  */
 int            maxfile;        /* number of files currently allocated      */
 int            numfiles;       /* number of files loaded                   */
 int            pressx1,pressy1,/* starting point of button press           */
                pressx2,pressy2;/* current point of button drag             */
 short          view_height,    /* height of viewing area                   */
                view_width;     /* number of files loaded                   */
 unsigned char  abort;          /* flag to abort loading of vectors         */
 int            dcwlib;         /* index of current dcw library             */
 int            srch_datum_exists; /* Global flag used for IM2VECT indicating
                                   if the datum in the search image is valid
                                   and is greater than 100 (not at spheroid)*/
 };

/*
 *  *** Map Paramters
 */
struct TIE_MAPPARMS
   {
   double ul_x;           /* Upper left map corner X coordinate           */
   double ul_y;           /* Upper left map corner Y coordinate           */
   double lr_x;           /* Lower right map corner X coordinate          */
   double lr_y;           /* Lower right map corner Y coordinate          */
   double inc_x;          /* Increment in X                               */
   double inc_y;          /* Increment in Y                               */
   double verify_x;       /* Verify point X coordinate                    */
   double verify_y;       /* Verify point Y coordinate                    */
   double coef_x[6];      /* Coefficients mapping sample coordinates      */
   double coef_y[6];      /* Coefficients mapping line coordinates        */
   int    npts_x;         /* Number  of coordinates in X                  */
   int    npts_y;         /* Number  of coordinates in Y                  */
   int    units;          /* Map units                                    */
   int    scale;          /* Annotation field--scale of map               */
   char   mapname[80];    /* Map name                                     */
   char   mapid[80];      /* Map id                                       */
   char   mapst_cntry[80];/* Map state or country indicator               */ 
   char   datum[80];      /* Annotation field--map datum                  */
   char   maptype[80];    /* Annotation field--type of map                */
   char   mapdate[10];    /* Annotation field--date of map                */
   char   mapdesc[80];    /* Annotation field--map description & notes    */
   char   mapfile[256];   /* Map file pathname                            */
   char   catalog[256];   /* Map file pathname                            */
   unsigned char mode;    /* current mode of map registration             */
   unsigned char registered;  
                          /* Flag denoting map is registered entry        */
   int    order;          /* order of polynomial                          */
   int    outline[4][2];  /* outline of map on image                      */
   XtInputId digid;       /* digitizer input id                           */
   int    portfd;         /* digitizer port file descriptor               */
   int    gotdatums;      /* Flag to indicate datums have been read       */
   int    dtm_cnt;        /* Number of datums read into xm_dtm            */
   XmString *xm_dtm;      /* Datum strings                                */
   struct DATUMDEF datum_info; 
                          /* Needed for call to c_getdatum. Used to store
                             datum code and semi major and minor axis.    */
   int set_semi_major_minor;
                          /* Flag used to indicate if the projection
                             parameters should include the semi major 
                             and minor axis                               */
   long out_datum_code;   /* The output datum code that references all 
                             the picked points - used in c_transinit      */
   long current_datum;    /* The most recent datum number chosen          */
   int first_datum_selected; /* Flag to indicate if this is the first
                                datum selected and used                   */ 
   };

/*
 *  *** Map outline structures
 */
struct TIE_MAPLIST
   {
   char         name[38];  /* name of map                               */
   char         state[3];  /* state map covers                          */
   char         id[7];     /* map ID                                    */
   double       clat,      /* center lat lon coordinates                */
                clon;
   int          outline[4][2],
                           /* outline of map on image                   */
                scale;     /* map scale                                 */
   struct 
   TIE_MAPLIST  *next;     /* pointer to next item in linked list       */
   unsigned char        
                selected;  /* denotes map select for output             */
   unsigned char        
                dispname;  /* flag to display map name                  */
   };

/*
 *  *** Adjust structure.
 */
struct TIE_ADJUST
 {
 XColor         *color_adjust;  /* Current adjusted color table.              */
 XColor         *color_linear;  /* Linear color table.                        */
 XColor         *adjust_ptr;    /* Pointer to the color table to adjust.      */
 short          *full_lut[2];   /* Adjust color LUT                           */
 float          slope;          /* Current slope increment.                   */
 float          offset;         /* Current offset increment.                  */
 float          lut_slope;      /* New LUT slope adjustment.                  */
 float          lut_offset;     /* New LUT offset adjustment.                 */
 };

/*
 *  *** Structure used to store information needed by the 
 *      interpolated zoom routines
 */
struct ENLARGE
  {
  int  *work;                   /* Hybrid space buffer                        */
  int  *horiz_table;            /* Horizontal resampling table                */
  int  *vert_table;             /* Vertical resampling table                  */
  int  in_lines;                /* Number of input lines + resampling pad     */
  int  in_samps;                /* Number of input samples + resampling pad   */
  int  hybrid_samps;            /* Number of samples in hybrid space          */
  int  extract_lines;           /* Number of lines to extract from display    */
  int  extract_samps;           /* Number of samples to extract from display  */
  int  horiz_dim;               /* Horizontal resampling table dimension      */
  int  vert_dim;                /* Vertical resampling table dimension        */
  unsigned char *input;         /* Input space buffer                         */
  unsigned char *output;        /* Output space buffer                        */
  char table_file[CMLEN];       /* Resampling weight table file name          */
  };

/*
 *  *** Zoom structure.
 */
struct TIE_ZOOM
 {
 Pixmap         image_pixmap;   /* Pixmap for zoomed image data.              */
 struct ENLARGE parms;          /* Zoom enlargement parameters.               */
 int            x;              /* Center of zoomed window sample.            */
 int            y;              /* Center of zoomed window line.              */
 int            flags;          /* Bit flags
                                   - Active flag
                                   - Resampling method
                                   - Zoom type
                                   - Zoom window                              */
 unsigned char  *image_data;    /* Zoomed image data.                         */
 };

/*
 *  *** Show structure.
 */
struct TIE_SHOW
 {
 int            planes[3];      /* Image planes to be displayed.              */
 int            component;      /* Component currently selecting.             */
 };

struct DLG_LINEATTRIB
   {
   int          mincode;        /* minimum code value                        */
   int          maxcode;        /* maximum code value                        */
   char         *desc;          /* major feature code                        */
   unsigned 
   char         present;        /* boolean denoting presence in current file */
   short        color;          /* working color index                       */
   short        style;          /* dash style index                          */
   };

struct WDB_LINEATTRIB
   {
   char         *desc;          /* major feature code                        */
   unsigned
   char         enabled,        /* boolean denoting loaded toggled on        */
                loaded;         /* boolean denoting loaded into memory       */
   short        color;          /* working color index                       */
   short        style;          /* dash style index                          */
   };

/*
 *  *** WBD file catalog
 */

struct WDB_FILES
   {
   char         *continent;     /* continent category                        */
   char         *filename[6];   /* corresponding names                       */
   };

#ifndef _tiepts_h

/*
 *  *** Declare the global structures.
 */
extern struct TIE_STRUCT     tie;
extern struct TIE_ADRGPARM   *tie_adrg;
extern struct TIE_MEMLAYOUT  tie_mem[2];
extern struct TIE_LAYOUT     tie_layout;
extern struct TIE_LOAD       tie_load;
extern struct TIE_CURSOR     tie_cursor;
extern struct TIE_PARMS      tie_parms;
extern struct TIE_FULLRES    *tie_ref_fullres;
extern struct TIE_FULLRES    *tie_srch_fullres;
extern struct TIE_RT         tie_rt;
extern struct TIE_SELECT     tie_select;
extern struct TIE_CMAP       tie_map;
extern struct TIE_MAPPARMS   tie_mapparm;
extern struct TIE_MAPLIST    *tie_maplist;
extern struct TIE_ADJUST     tie_adjust;
extern struct TIE_ZOOM       tie_zoom[2];
extern struct TIE_SHOW       tie_show;
extern struct TIE_MEMPLANE   *tie_planes;
extern struct TIE_COMPOSITE  *tie_composite;
extern struct TIE_VECTOR     *tie_vector;
extern struct TIE_VECTFILE   *tie_vectfile;
extern struct DLG_LINEATTRIB dlg_attrib[MAXATTRIB];
extern struct WDB_FILES      wdb_files[6];
extern struct WDB_LINEATTRIB wdb_attrib[WDBFEATURES];
extern struct DCW_CD         dcw_cd[DCW_NUMCD];
extern struct DCW_COVERAGE   dcw_coverage[DCW_NUMCOVER];
extern struct DCW_TILEREF    *dcw_tileref;
extern char *wk_colors[];
extern char *wk_fonts[];
extern char dash_pattern[6][2];
#else
struct TIE_STRUCT     tie;
struct TIE_ADRGPARM   *tie_adrg;
struct TIE_MEMLAYOUT  tie_mem[2];
struct TIE_LAYOUT     tie_layout;
struct TIE_LOAD       tie_load;
struct TIE_CURSOR     tie_cursor;
struct TIE_PARMS      tie_parms;
struct TIE_FULLRES    *tie_ref_fullres;
struct TIE_FULLRES    *tie_srch_fullres;
struct TIE_RT         tie_rt;
struct TIE_SELECT     tie_select;
struct TIE_CMAP        tie_map;
struct TIE_MAPPARMS   tie_mapparm;
struct TIE_ADJUST     tie_adjust;
struct TIE_MAPLIST    *tie_maplist;
struct TIE_ZOOM       tie_zoom[2];
struct TIE_SHOW       tie_show;
struct TIE_MEMPLANE   *tie_planes;
struct TIE_COMPOSITE  *tie_composite;
struct TIE_VECTOR     *tie_vector;
struct TIE_VECTFILE   *tie_vectfile;

char *wk_colors[] = {"black","white","red","green","blue",
    "yellow","magenta","cyan"};
                                /* Working color strings.               */

char *wk_fonts[] = {
    "8x13bold",
    "6x10",
    "-adobe-times-medium-r-normal--10-100-75-75-p-54-iso8859-1",
    "-adobe-times-medium-r-normal--14-140-75-75-p-74-iso8859-1",
    "-adobe-times-bold-r-normal--18-180-75-75-p-98-iso8859-1"
    };
                                /* Working font strings.               */

char dash_pattern[][2] = {
    {10,  0},
    {10,  1},
    {10,  3},
    {10,  5},
    { 5, 10},
    { 1, 10}
    };

/*
 *  *** this structure defines the contents of each of the DCW CDs
 *      and their geographic bounds.  This info is also available
 *      on each of the CD's in the Library Attribute Table (LAT.)
 */
struct DCW_CD dcw_cd[] = {
    {"North America",           "noamer",       -180.0,   5.0,    -5.0,   90.0},
    {"Europe/Northern Asia",    "eurnasia",      -50.0,  35.0,  -165.0,   90.0},
    {"South America/Africa",    "soamafr",      -180.0, -90.0,   180.0,   30.0},
    {"South Asia/Australia",    "sasaus",         25.0, -55.0,  -130.0,   55.0}
    };

/*
 *  *** this structure defines the contents of each of the DCW 
 *      coverages of interest for tiepts.  This information could
 *      be read from the CAT. file in the root of the CD.   However,
 *      I am only interested in these coverages.
 *      
 */
struct DCW_COVERAGE dcw_coverage[] = {
    {"Drainage",                "dn", 0, 0, 3, TRUE, NULL},
    {"Hypsography",             "hy", 0, 0, 3, FALSE, NULL},
    {"Hypsography-Sup",         "hs", 0, 0, 2, FALSE, NULL},
    {"Physiography",            "ph", 0, 0, 2, FALSE, NULL},
    {"Political/Oceans",        "po", 0, 0, 3, FALSE, NULL},
    {"Roads",                   "rd", 0, 0, 2, FALSE, NULL},
    {"Railroads",               "rr", 0, 0, 2, FALSE, NULL},
    {"Transportation",          "ts", 0, 0, 2, FALSE, NULL},
    {"Utilities",               "ut", 0, 0, 2, FALSE, NULL},
    {NULL,                      NULL, 0, 0, 0, FALSE, NULL},
    };

struct WDB_FILES wdb_files[] = {
    {"AFRICA",          "ea1","ea2","ea3","ea4","ea5","af1"},
    {"ASIA",            "as1","as2","as3","as4","as5","as6"},
    {"EUROPE",          "ea1","ea2","ea3","ea4","ea5","eu1"},
    {"NORTH AMERICA",   "na1","na2","na3","am1","am2", NULL},
    {"SOUTH AMERICA",   "sa1","sa2","sa3","am1","am2","sa4"},
    {NULL,               NULL, NULL, NULL, NULL, NULL, NULL},
    };

struct WDB_LINEATTRIB wdb_attrib[] = {
   {"Coasts, Islands, Lakes",   TRUE,  0, 0, 0},
   {"Rivers",                   TRUE,  0, 0, 0},
   {"Inter. Boundaries",        FALSE, 0, 0, 0},
   {"Polit. Boundaries",        FALSE, 0, 0, 0},
   {"Railroads",                FALSE, 0, 0, 0},
   {"Roads",                    FALSE, 0, 0, 0},
   };

struct DLG_LINEATTRIB dlg_attrib[] = {
   {20000,      20000,  "Hypsography",          FALSE, 0, 0},

   {50000,      50000,  "Gen. Hydrography",     FALSE, 0, 0},
   {50200,      50207,  "Shoreline",            FALSE, 0, 0},
   {50412,      50412,  "Stream",               FALSE, 0, 0},
   {50415,      50415,  "Aqueduct",             FALSE, 0, 0},
   {50421,      50421,  "Lake or Pond",         FALSE, 0, 0},
   {50605,      50606,  "Bank",                 FALSE, 0, 0},

   {70000,      70000,  "Surface Cover",        FALSE, 0, 0},

   {90000,      90000,  "Boundary",             FALSE, 0, 0},

   {150000,     150000, "Pipe & Trans. Lines",  FALSE, 0, 0},

   {170000,     170000, "Gen. Roads & Trails",  FALSE, 0, 0},
   {170201,     170204, "Class 1 Road",         FALSE, 0, 0},
   {170205,     170208, "Class 2 Road",         FALSE, 0, 0},
   {170209,     170209, "Class 3 Road",         FALSE, 0, 0},
   {170210,     170210, "Class 4 Road",         FALSE, 0, 0},
   {170211,     170212, "Trail",                FALSE, 0, 0},

   {180000,     180000, "Railroads",            FALSE, 0, 0},

   {190000,     190000, "Misc. Transmission",   FALSE, 0, 0},
   {190201,     190201, "Pipeline",             FALSE, 0, 0},
   {190202,     190202, "Power Line",           FALSE, 0, 0},
   {190203,     190203, "Telephone Line",       FALSE, 0, 0},
   {190400,     190402, "Power Plant/Station",  FALSE, 0, 0},
   {190403,     190408, "Landing Strip/Pad",    FALSE, 0, 0},

   {200000,     200000, "Manmade Feature",      FALSE, 0, 0},
   {NULL,       NULL,                           NULL},
   };

#endif
#endif
