#include "tiepts.h"
#include "util.h"
#include "mapsup.h"
#include "tpssup.h"

/*******************************************************************************
Purpose:

By T. Rockvam, USGS/EROS Data Center, 9/94

*******************************************************************************/
void tps_checkdatum(void)
{
   struct TPSHEAD header;   /* TPS file header record structure           */
   struct TPSDATA ptdata;   /* TPS file data record structure             */
   struct TIE_MAPPARMS parms; /* map parms                                    */
   struct PTDESC *indesc;   /* Input tie point file descriptor            */
   long   all_same,         /* Are the map datums all the same?           */
          idpos,          /* map id position                            */
          ftype = TPS,    /* Type of tie point file                     */
          inacc = IREAD,  /* Input file access mode                     */
          outacc = IWRITE,/* Output file access mode                    */
          status;
   long   dtmnum;
   char   *backup=NULL;
   char   *mapid=NULL;
   char   *mapname=NULL;
   char   mydatum[80];

  /*
   *  *** make backup file for output reference 
   */
   c_ptclse(&tie_select.ref_desc);
   backup = (char*)malloc(strlen(tie_parms.outtps[REFERENCE])+strlen(".bak")+1);
   strcpy(backup, tie_parms.outtps[REFERENCE]);
   strcat(backup, ".bak");
   unlink(backup);
   rename(tie_parms.outtps[REFERENCE], backup); 

  /*
   *  *** Open the old ref TPS file for read access
   *      and check the datums
   *      If they are all NAD83, set header.projprms[14] = 3.0
   *      If they are all NAD27, set header.projprms[14] = 2.0
   *      If they are different, set header.projprms[14] = 1.0 (mixed)
   *      If they are all the same, but not NAD27 or NAD83, 
   *                             set header.projprms[14] = 0.0 (unknown)
   */
   c_ptopen(&indesc,backup,&ftype,&inacc,(long *) &header);
   all_same = TRUE;

  /*
   * Get the datum from the first map
   */
   status = c_ptio(&indesc,(long *) &ptdata);
   if (status == E_SUCC)
       {
       idpos = (long)strcspn(ptdata.chip_name,"@");
       if (idpos == strlen(ptdata.chip_name))
           {
           mapid = (char *)malloc(2);                 
           strcpy(mapid,"");
           mapname = (char *)malloc(strlen(ptdata.chip_name)+1); 
           strcpy(mapname,ptdata.chip_name);
           }
       else
           {
           mapname = (char *)malloc(idpos);
           strncpy(mapname,ptdata.chip_name,idpos-1);
           mapname[idpos-1] = '\0';
           mapid = (char *)malloc(strlen(ptdata.chip_name)-idpos);
           strncpy(mapid,ptdata.chip_name+idpos+1,
                   strlen(ptdata.chip_name)-idpos-1);
           mapid[strlen(ptdata.chip_name)-idpos-1] = '\0';
           }
       if (map_read(tie_mapparm.mapfile,mapname,mapid,&parms) != E_SUCC)
           parms.datum[0] = '\0';
       if ((mapname[0] == '\0') && (mapname[0] == '\0'))
           parms.datum[0] = '\0';
       strcpy(mydatum,parms.datum);
       idpos=(long)strlen(mydatum);
       c_low2up(mydatum,&idpos);

      /*
       * Compare the remaining maps
       */
       status = c_ptio(&indesc,(long *) &ptdata);
       }

   while (status == E_SUCC)
      {
      idpos = (long)strcspn(ptdata.chip_name,"@");
      if (idpos == strlen(ptdata.chip_name))
          {
          mapid = (char *)malloc(2);                  
          strcpy(mapid,"");
          mapname = (char *)malloc(strlen(ptdata.chip_name)+1); 
          strcpy(mapname,ptdata.chip_name);
          }
      else
          {
          mapname = (char *)malloc(idpos);
          strncpy(mapname,ptdata.chip_name,idpos-1);
          mapname[idpos-1] = '\0';
          mapid = (char *)malloc(strlen(ptdata.chip_name)-idpos);
          strncpy(mapid,ptdata.chip_name+idpos+1,
                  strlen(ptdata.chip_name)-idpos-1);
          mapid[strlen(ptdata.chip_name)-idpos-1] = '\0';
          }
      if (map_read(tie_mapparm.mapfile,mapname,mapid,&parms) != E_SUCC)
          parms.datum[0] = '\0';
       if ((mapname[0] == '\0') && (mapname[0] == '\0'))
           parms.datum[0] = '\0';
      idpos=(long)strlen(parms.datum);
      c_low2up(parms.datum,&idpos);
      if (strcmp(parms.datum,mydatum) != 0)
          {
          all_same = FALSE;
          break;
          }
      status = c_ptio(&indesc,(long *) &ptdata);
      }
   c_ptclse(&indesc);

  /*
   *  *** Open the new ref TPS file for write access
   *      with the updated header
   */
   c_ptopen(&indesc,backup,&ftype,&inacc,(long *) &header);
   free(backup);
   if (all_same)
       {
       /* extract datum number from ( ) at beginning of string */
       dtmnum = -1;
       if (mydatum[0] == '(')
       {
           dtmnum = atoi(&mydatum[1]);
       }
       else
       {
           c_lasmsg(FALSE,"tiepts-datum","datum string is unrecognized: %s",
                    mydatum);
       }
       
       if (dtmnum > -1)
         {
         strcpy(parms.datum,mydatum);
         header.datum = dtmnum;
         if (strcmp(mydatum,"NAD27") == 0)
           header.projprms[14] = 2.0;
         else if (strcmp(mydatum,"NAD83") == 0)
           header.projprms[14] = 3.0;
         else
           header.projprms[14] = 0.0; /* all the same unknown datum */
         }
       }
   else
       header.projprms[14] = 1.0;

   c_ptopen(&tie_select.ref_desc,tie_parms.outtps[REFERENCE],
             &ftype,&outacc,(long *) &header);

  /*
   *  *** Copy the points
   */
   for (;;)
      {
      status = c_ptio(&indesc,(long *) &ptdata);
      if (status == E_EOF) break;
      if (status == E_FAIL) break;

      c_ptio(&tie_select.ref_desc,(long *) &ptdata);
      }

   c_ptclse(&indesc);
}
