/*******************************************************************************
NAME: TPS_DUMP

PURPOSE: This routine writes a TPS record to the output TPS files.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include <stdio.h>
#include "tiepts.h"
#include "protos.h"
#include "tpssup.h"

void tps_dump(void)
{
  struct TPSDATA ptdata;              /* TPS file data structure         */
  int  i;                             /* Loop counter                    */
  int  loop;                          /* Loop counter                    */
  int  ptindex;                       /* Point index of new point        */
  double tmpdata;                     /* temporary data variable		 */
   
  ptindex = tie_select.seltie;

  if ((tie_rt.calc_rt == RT_SEL) && (tie_select.seltie <= 4) &&
      (tie.mode != ONESRCH))
    {
    for (i = 0; i < tie_select.seltie; i++)
      {
      tie_rt.rtrzm[i] = tie_parms.factor;
      tie_rt.rtszm[i] = tie_parms.factor;
      tie_rt.rtroff[i][0] = tie_parms.refoff[0];
      tie_rt.rtroff[i][1] = tie_parms.refoff[1];
      tie_rt.rtsoff[i][0] = tie_parms.srchoff[0];
      tie_rt.rtsoff[i][1] = tie_parms.srchoff[1];
      }
    }
  else if ((tie_rt.calc_rt != RT_SEL) || (tie.mode == ONESRCH))
    display_pt(ptindex-1);

  if (tie.mode == ONESRCH)
    loop = 1;
  else if ((tie_select.seltie == 4) && (tie_rt.calc_rt == RT_SEL))
    loop = 4;
  else if ((tie_rt.calc_rt == RT_CAL) || (tie_rt.calc_rt == RT_DDR))
    loop = 1;
  else
    loop = 0;

  for (i = 0; i < loop; i++)
    {
    /* Reference Tie points
    -----------------------*/
    ptdata.coord[0] = ptdata.coord[1] = 0.0;
    ptdata.offsets[0] = ptdata.offsets[1] = 16;
    strcpy(ptdata.chip_name,"");

    /* Must change from sample/line(x,y)
       to line/sample(y,x)
    ------------------------------------*/
    if ((tie.mode == IM2MAP) || (tie.mode == IM2VECT) ||
        (tie.flags & TIE_ADRG))
      {
      ptdata.coord[1] = tie_select.rtie[ptindex-loop+i][0];
      ptdata.coord[0] = tie_select.rtie[ptindex-loop+i][1];
      ptdata.image_coord[1] = 0.0;
      ptdata.image_coord[0] = 0.0;
      if (tie.mode == IM2MAP)
        {
        strcpy(ptdata.chip_name,tie_select.mapname[ptindex-loop+i]);
        if (strcmp(tie_select.mapid[ptindex-loop+i],"") != 0)
          {
          strcat(ptdata.chip_name," @");
          strcat(ptdata.chip_name,tie_select.mapid[ptindex-loop+i]);
          }
        }
      else if (tie.mode == IM2VECT)
        {
        if (tie_parms.flags & PARMS_LATLONG)
          {
          tmpdata = ptdata.coord[0];
          ptdata.coord[0] = ptdata.coord[1];
          ptdata.coord[1] = tmpdata;
          proj2degrees(ptdata.coord, 1, SEARCH);
          tmpdata = ptdata.coord[0];
          ptdata.coord[0] = ptdata.coord[1];
          ptdata.coord[1] = tmpdata;
          }
 	    }   
      }
    else if (tie.mode == IM2IM)
      {
      /* Checking if line/sample has been selected
      --------------------------------------------*/
      if (tie_parms.flags & PARMS_LINESAMP)
        {
        ptdata.image_coord[1] = tie_select.rtie[ptindex-loop+i][0];
        ptdata.image_coord[0] = tie_select.rtie[ptindex-loop+i][1];
        }
      else
        {
        ptdata.image_coord[1] = 0.0;
        ptdata.image_coord[0] = 0.0;
        }
      /* Checking if Projection or LatLong has been selected
      ------------------------------------------------------*/
      if ((tie_parms.flags & PARMS_PROJ) || (tie_parms.flags & PARMS_LATLONG))
        {
        if (tie_parms.flags & PARMS_LINESAMP)
          {
          ptdata.coord[1] = tie_select.crtie[ptindex-loop+i][0];
          ptdata.coord[0] = tie_select.crtie[ptindex-loop+i][1];
          }
        else
          {
          ptdata.coord[1] = tie_select.rtie[ptindex-loop+i][0];
          ptdata.coord[0] = tie_select.rtie[ptindex-loop+i][1];
          }
        }
      else
        {
        ptdata.coord[1] = 0.0;
        ptdata.coord[0] = 0.0;
        }
      }
    else
      {
      ptdata.image_coord[1] = tie_select.rtie[ptindex-loop+i][0];
      ptdata.image_coord[0] = tie_select.rtie[ptindex-loop+i][1];
      ptdata.coord[1] = 0.0;
      ptdata.coord[0] = 0.0;
      }
    ptdata.elevation = tie_select.elev[ptindex-loop+i];

    if (tie_rt.calc_rt == RT_SEL)
      {
      ptdata.zoom_factor = tie_rt.rtrzm[i];
      ptdata.chip_size[0] = tie_rt.rtroff[i][0];
      ptdata.chip_size[1] = tie_rt.rtroff[i][1];
      }
    else
      {
      ptdata.zoom_factor = tie_parms.factor;
      ptdata.chip_size[0] = tie_parms.refoff[0];
      ptdata.chip_size[1] = tie_parms.refoff[1];
      }

    strcpy(ptdata.pt_id,tie_select.pt_id[ptindex-loop+i]);
    if (tie.mode != ONESRCH)
      if (c_ptio(&tie_select.ref_desc,(long *) &ptdata) != E_SUCC)
        fprintf(stderr,"ERROR writing data to reference .tps file!\n");

    /* Search Tie points
    --------------------*/
    ptdata.chip_size[0] = ptdata.chip_size[1] = 0;

    /* Must change from sample/line(x,y)
       to line/sample(y,x)
    ------------------------------------*/
    ptdata.image_coord[0] = tie_select.stie[ptindex-loop+i][1];
    ptdata.image_coord[1] = tie_select.stie[ptindex-loop+i][0];
    ptdata.coord[1] = 0.0;
    ptdata.coord[0] = 0.0;

    if (tie_rt.calc_rt == RT_SEL)
      {
      ptdata.zoom_factor = tie_rt.rtszm[i];
      ptdata.offsets[0] = tie_rt.rtsoff[i][0];
      ptdata.offsets[1] = tie_rt.rtsoff[i][1];
      }
    else
      {
      ptdata.zoom_factor = tie_parms.factor;
      ptdata.offsets[0] = tie_parms.srchoff[0];
      ptdata.offsets[1] = tie_parms.srchoff[1];
      }

    if (c_ptio(&tie_select.srch_desc,(long *) &ptdata) != E_SUCC)
      fprintf(stderr,"ERROR writing point data to search .tps file!\n");
    }
}
