/*******************************************************************************
NAME: TPS_FILEOPEN

PURPOSE: This routine opens the output TPS files for writing, and if needed
         will open the input TPS files for reading.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            3/99    T.Ratliff       Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include "tiepts.h"
#include "protos.h"
#include "tpssup.h"

int tps_fileopen
(
    struct PTDESC **indesc,                /* Input TPS file descriptor     */
    struct TPSHEAD *hdr,                   /* TPS file header record struct */
    char *filename,                        /* Input file names              */
    char *mesg,                            /* Error message(s) to display   */
    int  imgidx                            /* Image index (REF or SEARCH)   */
)
{
  long ftype = TPS;                      /* Type of tiepoint file         */
  int  status;                           /* Return status of function     */
  int  len;                              /* Length of message string      */
  long inacc = IREAD;                    /* Input file access mode        */
  long outacc = IWRITE;                  /* Output file access mode       */
  char ftlmsg[1280];                     /* Error message text            */

  /* Open the input tps file, if one listed
  -----------------------------------------*/
  if (strcmp(tie_parms.intps[imgidx],"") != 0)
    {
    c_ptopen(indesc,tie_parms.intps[imgidx],&ftype,&inacc,(long *) hdr);
    if (strcmp(filename,hdr->name) != 0)
      {
      len = (int)strlen(mesg);
      mesg += len;
      sprintf(mesg,"The input image %s is not the same as the image\n referenced by the input TPS file(%s)\n",filename,hdr->name);
      }
    }

  /* Open the output Reference tps file
  -------------------------------------*/
  if (imgidx == REFERENCE)
    {
    status = c_ptopen(&tie_select.ref_desc,tie_parms.outtps[imgidx],
                      &ftype,&outacc,(long *) hdr);
    if (status != E_SUCC)
      {
      sprintf(&ftlmsg[strlen(mesg)],"Unable to open output ref tpsfile.\nPlease check permissions\n");
      tie_select.ref_desc = NULL;
      tie_message(Shell_Main,ftlmsg);
      tie.flags &= ~TIE_OPEN;
      return(E_FAIL);
      }
    }
  else
    {
    /* Open the output Search tps file
    ----------------------------------*/
    status = c_ptopen(&tie_select.srch_desc,tie_parms.outtps[imgidx],
                      &ftype,&outacc,(long *) hdr);
    if (status != E_SUCC)
      {
      sprintf(&ftlmsg[strlen(ftlmsg)],"Unable to open output srch tps file.\nPlease check permissions\n");
      tie_select.srch_desc = NULL;
      tie_message(Shell_Main,ftlmsg);
      tie.flags &= ~TIE_OPEN;
      unlink(tie_parms.outtps[REFERENCE]);
      return(E_FAIL);
      }
    }

  return(E_SUCC);
}
