/*******************************************************************************
NAME: TPS_SETHEADER

PURPOSE: This routine sets the header record to the correct values.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            3/99    T.Ratliff       Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include "tiepts.h"
#include "util.h"
#include "tpssup.h"

void tps_setheader
(
    char *units,
    struct TPSHEAD *hdr
)
{
  int  j;                         /* Loop control variable              */
  long length;                    /* Length of a string of characters   */

  /* Based on the mode of TIEPTS, the user selected options, and
     valid data, set the header information accordingly
  --------------------------------------------------------------*/
  if ((tie.mode == IM2MAP) || (tie.mode == IM2VECT) ||
      (tie.flags & TIE_ADRG))
    {
    length = (long)strlen(units);
    c_up2low(units,&length);

    if ((strcmp(units,"feet") == 0) || (strcmp(units,"meters") == 0) ||
        (strcmp(units,"user_def") == 0))
      {
      strcpy(hdr->mode,"USR");

      if ((tie.mode == IM2VECT) && (hdr->proj_valid != GEO_VALID))
        {
        int plane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
        
        if ((tie_parms.vsource == DLGOPT) && !(tie_parms.flags & PARMS_LATLONG))
          {
          strcpy(hdr->mode,"USR");
          hdr->code = tie_planes[plane_idx].ddr.proj_code;
          hdr->zone = tie_planes[plane_idx].ddr.zone_code;
          for (j = 0; j < 15; j++)
            hdr->projprms[j] = tie_planes[plane_idx].ddr.proj_coef[j];
          }
        else
          {
          strcpy(hdr->mode,"GEO");
          strcpy(hdr->units,"degrees"); 
          hdr->code = 0;
          hdr->zone = 62;
          for (j = 0; j < 15; j++)
            hdr->projprms[j] = 0.0;
          }
        hdr->datum = tie_planes[plane_idx].ddr.datum_code;
        hdr->proj_valid = GEO_VALID;

        for (j = 0; j < 8; j++)
          hdr->corners[j] = 0.0;
        hdr->pdist[0] = hdr->pdist[1] = 1.0;
        }
      }
    else
      {
      strcpy(hdr->mode,"GEO");
      strcpy(hdr->units,"degrees");
      for (j = 0; j < 8; j++)
        hdr->corners[j] = 0.0;
      hdr->pdist[0] = hdr->pdist[1] = 1.0;
      for (j = 0; j < 15; j++)
        hdr->projprms[j] = 0.0;
      hdr->code = hdr->datum = 0;
      hdr->zone = 62;
      hdr->proj_valid = GEO_VALID;
      }

    if ((tie.mode == IM2MAP) && (tie_mapparm.units == MAP_USERDEF))
      {
      strcpy(hdr->mode,"USR");
      hdr->proj_valid = GEO_NOTVALID;
      }
    else if ((tie.mode == IM2VECT) && (tie_parms.flags & PARMS_LATLONG))
      geo_header(hdr);
    }
  else if (tie.mode == IM2IM)
    {
    if (tie_parms.flags & PARMS_LINESAMP)
      strcpy(hdr->mode,"REF");
    else if (tie_parms.flags & PARMS_LATLONG)
      strcpy(hdr->mode,"GEO");
    else if (tie_parms.flags & PARMS_PROJ)
      strcpy(hdr->mode,"PRO");
    }
  else
    strcpy(hdr->mode,"REF");
}
