/*******************************************************************************
NAME: TPS_UPDATE

PURPOSE: This routine updates the TPS output files.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
#include "tiepts.h"
#include "tpssup.h"

void tps_update(void)
{
    struct TPSHEAD header;          /* TPS file header record structure     */
    struct TPSDATA ptdata;          /* TPS file data record structure       */
    struct PTDESC *indesc;          /* Input tps file descriptor            */
    int  i;                         /* Loop counter                         */
    long ftype = TPS;               /* Type of tie point file               */
    long inacc = IREAD;             /* Input file access mode               */
    long outacc = IWRITE;           /* Output file access mode              */
    char *backup;                   /* Name of backup output files          */
    double tmpdata;                 /* temporary data storage               */

    /* Make backup file for output Reference
       ----------------------------------------*/
    c_ptclse(&tie_select.ref_desc);
    backup = (char*)malloc(strlen(tie_parms.outtps[REFERENCE])+5);
    strcpy(backup,tie_parms.outtps[REFERENCE]);
    strcat(backup,".bak");
    unlink(backup);
    rename(tie_parms.outtps[REFERENCE],backup);

    /* Open the Reference backup TPS file and
       read the header record
       -----------------------------------------*/
    c_ptopen(&indesc,backup,&ftype,&inacc,(long *)&header);
    c_ptclse(&indesc);
    if ((tie.mode == IM2MAP) && (tie_mapparm.units == MAP_USERDEF))
    {
        header.proj_valid = GEO_NOTVALID;
        strcpy(header.mode,"USR");
    }
 
    /* Image to Map has the option for a user to select and switch datums in 
       the middle of picking points. These points are converted to reflect the
       new datums in select.c and manual.c. Update the header file with the 
       first datum for IM2MAP. 
       ---------------------------------------------------------------------*/
    if (tie.mode == IM2MAP)
       header.datum = tie_mapparm.out_datum_code;

    /* Open the output Reference TPS file
       and write the header record
       -------------------------------------*/
    c_ptopen(&tie_select.ref_desc,tie_parms.outtps[REFERENCE],&ftype,&outacc,
            (long *)&header);
    free(backup);

    /* Make backup file for output Search
       -------------------------------------*/
    c_ptclse(&tie_select.srch_desc);
    backup = (char*)malloc(strlen(tie_parms.outtps[SEARCH])+5);
    strcpy(backup,tie_parms.outtps[SEARCH]);
    strcat(backup,".bak");
    unlink(backup);
    rename(tie_parms.outtps[SEARCH],backup);

    /* Open the Search TPS file for write access
       and write the header to it.
       --------------------------------------------*/
    c_ptopen(&indesc,backup,&ftype,&inacc,(long *)&header);
    c_ptclse(&indesc);
    c_ptopen(&tie_select.srch_desc,tie_parms.outtps[SEARCH],&ftype,&outacc,
            (long *)&header);
    free(backup);

    for (i = 0; i < tie_select.seltie; i++)
    {
        /* Reference Tie point
           ----------------------*/
        ptdata.coord[0] = ptdata.coord[1] = 0.0;
        ptdata.offsets[0] = ptdata.offsets[1] = 16;
        if (tie.mode == IM2MAP)
        {
            strcpy(ptdata.chip_name,tie_select.mapname[i]);
            if (strcmp(tie_select.mapid[i],"") != 0)
            {
                strcat(ptdata.chip_name," @");
                strcat(ptdata.chip_name,tie_select.mapid[i]);
            }
        }
        else
            strcpy(ptdata.chip_name,"");

        /* Must change from sample/line(x,y)
           to line/sample(y,x)
           ------------------------------------*/
        if ((tie.mode == IM2MAP) || (tie.mode == IM2VECT) ||
                (tie.flags & TIE_ADRG))
        {
            ptdata.coord[1] = tie_select.rtie[i][0];
            ptdata.coord[0] = tie_select.rtie[i][1];
            ptdata.image_coord[1] = 0.0;
            ptdata.image_coord[0] = 0.0;
            if ((tie.mode == IM2VECT) && (tie_parms.flags & PARMS_LATLONG))
            {
                tmpdata = ptdata.coord[0];
                ptdata.coord[0] = ptdata.coord[1];
                ptdata.coord[1] = tmpdata;
                proj2degrees(ptdata.coord, 1, SEARCH);
                tmpdata = ptdata.coord[0];
                ptdata.coord[0] = ptdata.coord[1];
                ptdata.coord[1] = tmpdata;
            }
        }
        else if (tie.mode == IM2IM)
        {
            /* Checking if line/sample has been selected
               --------------------------------------------*/
            if (tie_parms.flags & PARMS_LINESAMP)
            {
                ptdata.image_coord[1] = tie_select.rtie[i][0];
                ptdata.image_coord[0] = tie_select.rtie[i][1];
            }
            else
            {
                ptdata.image_coord[1] = 0.0;
                ptdata.image_coord[0] = 0.0;
            }

            /* Checking if Projection or LatLong has been selected
               ------------------------------------------------------*/
            if ((tie_parms.flags & PARMS_PROJ) 
                || (tie_parms.flags & PARMS_LATLONG))
            {
                if (tie_parms.flags & PARMS_LINESAMP)
                {
                    ptdata.coord[1] = tie_select.crtie[i][0];
                    ptdata.coord[0] = tie_select.crtie[i][1];
                }
                else
                {
                    ptdata.coord[1] = tie_select.rtie[i][0];
                    ptdata.coord[0] = tie_select.rtie[i][1];
                }
            }
            else
            {
                ptdata.coord[1] = 0.0;
                ptdata.coord[0] = 0.0;
            }
        }
        else
        {
            ptdata.image_coord[1] = tie_select.rtie[i][0];
            ptdata.image_coord[0] = tie_select.rtie[i][1];
            ptdata.coord[1] = 0.0;
            ptdata.coord[0] = 0.0;
        }

        ptdata.elevation = tie_select.elev[i];
        ptdata.zoom_factor = tie_parms.factor;
        ptdata.chip_size[0] = tie_parms.refoff[0];
        ptdata.chip_size[1] = tie_parms.refoff[1];

        strcpy(ptdata.pt_id,tie_select.pt_id[i]);

        if (tie.mode != ONESRCH)
            c_ptio(&tie_select.ref_desc,(long *)&ptdata);

        /* Search Tie points
           --------------------*/
        ptdata.chip_size[0] = ptdata.chip_size[1] = 0;

        /* Must change from sample/line(x,y)
           to line/sample(y,x)
           ------------------------------------*/
        ptdata.image_coord[1] = tie_select.stie[i][0];
        ptdata.image_coord[0] = tie_select.stie[i][1];
        ptdata.coord[1] = 0.0;
        ptdata.coord[0] = 0.0;

        ptdata.zoom_factor = tie_parms.factor;
        ptdata.offsets[0] = tie_parms.srchoff[0];
        ptdata.offsets[1] = tie_parms.srchoff[1];
        c_ptio(&tie_select.srch_desc,(long *)&ptdata);
    }
}
