/*
 *  This is the utility source file for the X Image Display application.
 *  It contains procedure routines that are generic in nature and not
 *  necessarily tied to a specific function source file.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *
 *      Routine                         Description
 *      -------                         -----------
 *      COLOR                           Returns pixel value for color string.
 *      FONT_LIST                       Returns a font list pointer for
 *                                       a given font specification.
 *      XCreatePixmapFromData           XPM utility for using XPM in C.
 *      XPM_PIXMAP                      XPM utility for creating pixmaps
 *                                      from XPM specifications.
 *      MENU_POST                       Simple Motif menu handler.
 *      STRING_TABLE                    Motif string table creation utility.
 *      FREE_STRING_TABLE               Motif string table deletion utility.
 *      TIE_StringToPixel               Resource converter to convert
 *                                       widget color string specifications to
 *                                       pixel values.
 *      CompareISOLatin1                 ISOLatin character checker used with
 *                                       the TIE_StringToPixel resource
 *                                       converter.
 *      getcolor                        Manages color specifications for the
 *                                       color store used by tie.
 *
 */
#include <ctype.h>
#include "tiepts.h"
#include "protos.h"

/*
 *  Includes for the TIE_StringToPixel resource converter.
 */

#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include "protos.h"

/*
 * ANSI C requires static functions to be declared.
 */

static int CompareISOLatin1
(
    char *first,
    char *second
);

static Status getcolor
(
    char        *str,           /* Input color specification. (I)               */
    XColor  *color_cell /* Color cell to be assigned. (O)               */
);

/*
 *  Resource name variables for the TIE_StringToPixel resource converter.
 */

static String XtNwrongParameters = "wrongParameters";

/*
 *  Convenience routines; these originate from the ICS BX application.
 */
#ifndef IGNORE_COLOR
unsigned int COLOR
(
    Widget w,
    char *name
)
{
    XrmValue    fromVal, toVal;
    unsigned long *pixel;

    fromVal.size = sizeof(char*);
    fromVal.addr = name;

    XtConvert(w, XmRString, &fromVal, XmRPixel, &toVal);
    pixel = (unsigned long*)toVal.addr;
    if( pixel == NULL )
    {
        fromVal.addr = XtDefaultBackground;
        XtConvert(w, XmRString, &fromVal, XmRPixel, &toVal);
        pixel = (unsigned long*)toVal.addr;
    }
    return(*pixel);
}
#endif

#ifndef IGNORE_FONT
XmFontList FONT_LIST
(
    Widget      w,
    char        *name
)
{
    XrmValue    fromVal, toVal;
    XmFontList  *fontListPtr;

    fromVal.size = sizeof(char*);
    fromVal.addr = name;

    XtConvert(w, XmRString, &fromVal, XmRFontList, &toVal);
    fontListPtr = (XmFontList*)toVal.addr;
    if(fontListPtr == NULL)
    {
        fromVal.addr = "fixed";
        XtConvert(w, XmRString, &fromVal, XmRFontList, &toVal);
        fontListPtr = (XmFontList*)toVal.addr;
    }
    return(*fontListPtr);
}

#endif
#ifndef IGNORE_XPM_PIXMAP
/* Copyright 1989 GROUPE BULL -- See licence conditions in file COPYRIGHT */
/*****************************************************************************
 ****  Create utility for XPM file format... [ X PixMap ]
 ****
 ****  Pixmap XCreatePixmapFromData(dpy, d, cmap, w, h, depth, n, c, col, pix)
 ****
 ****  original:  Daniel Dardailler - Bull RC (89/02/22) daniel@mirsa.inria.fr
 ****  contributors:
 ****             James Bash - AT&T Bell Labs            jmb@attunix.att.com
 *****************************************************************************
 ****  Version 1.2:  Improved storage mgmt & fatal error handling...
 ****  Version 1.1:  extended chars_per_pixel support... [ Read, Create ]
-------------------------------------------------------------------------------
WHO:    Richard Hess                    CORP:   Consilium
TITLE:  Project Manager                 VOICE:  [415] 691-6342
      [ User Interface Services ]       USNAIL: 640 Clyde Court
UUCP:   ...!uunet!cimshop!rhess                 Mountain View, CA 94043
-------------------------------------------------------------------------------
 */

#include <stdio.h>

/* Copyright 1989 GROUPE BULL -- See licence conditions in file COPYRIGHT */
/**************************************************************************
 ****  Constant used in xpm library functions :
 ****
 ****  int XWritePixmapFile(dpy, cmap, filename, pixmap, w, h)
 ****  int XReadPixmapFile(dpy, d, cmap, filename, w, h, depth, pixmap)
 ****  Pixmap XCreatePixmapFromData(dpy, d, cmap, w, h, depth, n, c, col, pix)
 ****
 ****  Daniel Dardailler - Bull RC (89/02/22) e-mail: daniel@mirsa.inria.fr
 **************************************************************************/

#define XPM_FORMAT 1

/* we keep the same codes as for Bitmap management */
#ifndef _XUTIL_H_
#ifdef VMS
#include "decw$include:Xutil.h"
#else
#include <X11/Xutil.h>
#endif
#endif

#define PixmapSuccess        BitmapSuccess 
#define PixmapOpenFailed     BitmapOpenFailed
#define PixmapFileInvalid    BitmapFileInvalid
#define PixmapNoMemory       BitmapNoMemory


typedef struct _Cixel {
     char c1,c2 ;
   } Cixel ;                    /* 2 chars for one pixel value */

typedef struct _CmapEntry {
     Cixel cixel ;               
     int red,green,blue ;
   } CmapEntry ;                    


#define MAXPRINTABLE 93             /* number of printable ascii chars 
                                       minus \ and " for string compat. */

char * printable = " .XoO+@#$%&*=-;:?>,<1234567890qwertyuipasdfghjklzxcvbnmMNBVCZASDFGHJKLPIUYTREWQ!~^/()_`'][{}|" ;
           /* printable begin with a space, so in most case, due to
              my algorythm, when the number of different colors is
              less than MAXPRINTABLE, it will give a char follow by 
              "nothing" (a space) in the readable xpm file */


/**[ XCreatePixmapFromData ]**************************************************
 *
 *  This function allows you to include in your C program (using #include)
 *  a pixmap file in XPM format that was written by XWritePixmapFile
 * (THIS VERSION SUPPORTS : '_chars_per_pixel 1', '_chars_per_pixel 2')
 *****************************************************************************/
Pixmap XCreatePixmapFromData
(
     Display *display,
     Drawable d,
     Colormap colormap,
     unsigned int width,
     unsigned int height,
     int depth,
     unsigned int ncolors,
     unsigned int chars_per_pixel,
     char ** colors,           /* array of colormap entries  "cc","#RRGGBB" */
     char ** pixels            /* array of pixels lines    "cc..cc00cccc.." */
)
{
  Pixmap pixmap = NULL;
  GC Gc = NULL;
  XGCValues xgcv;
  XImage *xim = NULL;

  CmapEntry * cmap = NULL;
  int * Tpixel = NULL;

  char c1, c2;
  XColor xcolor;
  Status status;
  int i,j,p;
  
#ifdef DEBUG
#define DEBUG fprintf
#else
#define DEBUG
#endif

#undef RETURN
#define RETURN(code, flag) \
  { if (cmap)   free (cmap); \
    if (Tpixel) free (Tpixel); \
    if (Gc)     XFreeGC(display, Gc); \
    if (xim)    XDestroyImage(xim); \
    if (flag && pixmap) XFreePixmap(display, pixmap); \
    return (code); }

  if (ncolors > (MAXPRINTABLE*MAXPRINTABLE)) {
/*    DEBUG (stderr,"Too many different colors, version 1");*/
    RETURN (PixmapFileInvalid, 0);
  }

  if ((chars_per_pixel < 1) || (chars_per_pixel > 2)) {
/*    DEBUG (stderr,"version 1.1 handles only 1 or 2 chars_per_pixel");*/
    RETURN (PixmapFileInvalid, 0);
  }

  /* now we construct cmap and Tpixel from colors array parameter */
  cmap = (CmapEntry *) malloc(ncolors*sizeof(CmapEntry)) ;
  Tpixel = (int *) malloc(ncolors*sizeof(int)) ;
  if ((cmap == NULL) || (Tpixel == NULL)) {
    RETURN (PixmapNoMemory, 0);
  }

  if (!colors) {
/*    DEBUG (stderr,"colors not defined");*/
    RETURN (PixmapFileInvalid, 0);
  }

  switch (chars_per_pixel)
    {
    case 1:
      for (i=0; i<2*ncolors ; i+=2)
        {
          if (sscanf(colors[i],"%c", &c1) != 1) {
/*          DEBUG (stderr,
                   "bad colormap entry : must be '\"c\" , \"colordef\",'"); */
            RETURN (PixmapFileInvalid, 0);
          }
          if (strchr(printable,c1))
            {
              cmap[i/2].cixel.c1 = c1 ;
              if (!XParseColor(display,colormap,colors[i+1],&xcolor)) {
/*              DEBUG (stderr,
                       "bad colordef specification : #RGB or colorname");*/
                RETURN (PixmapFileInvalid, 0);
              }
/*            XAllocColor(display,colormap,&xcolor);
              XLookupColor(display,colormap,colors[i+1],&xcolor,&rgbdef); */
              status = getcolor(colors[i+1],&xcolor);
              if (status == 0)
               {
/*              DEBUG (stderr,"bad cixel value : must be printable");*/
                RETURN (PixmapFileInvalid, 0);
               }
              Tpixel[i/2] = xcolor.pixel ;
            } else {
/*            DEBUG (stderr,"bad cixel value : must be printable");*/
              RETURN (PixmapFileInvalid, 0);
            }
        }
      break;
    case 2:
      for (i=0; i<2*ncolors ; i+=2)
        {
          if (sscanf(colors[i],"%c%c", &c1,&c2) != 2) {
/*          DEBUG (stderr,
                   "bad colormap entry : must be '\"cC\" , \"colordef\",'"); */
            RETURN (PixmapFileInvalid, 0);
          }
          if ((strchr(printable,c1)) &&
              (strchr(printable,c2))) {
            cmap[i/2].cixel.c1 = c1 ;
            cmap[i/2].cixel.c2 = c2 ;
            if (!XParseColor(display,colormap,colors[i+1],&xcolor)) {
/*            DEBUG (stderr,"bad colordef specification : #RGB or colorname");*/
              RETURN (PixmapFileInvalid, 0);
            }
/*          XAllocColor(display,colormap,&xcolor);
            XLookupColor(display,colormap,colors[i+1],&xcolor,&rgbdef); */
              status = getcolor(colors[i+1],&xcolor);
              if (status == 0)
               {
/*                DEBUG (stderr,"bad cixel value : must be printable");*/
                RETURN (PixmapFileInvalid, 0);
               }
            Tpixel[i/2] = xcolor.pixel ;
          } else {
/*          DEBUG (stderr,"bad cixel value : must be printable");*/
            RETURN (PixmapFileInvalid, 0);
          }
        }
      break;
    }
  
  pixmap = XCreatePixmap(display,d,width,height,depth);
  Gc = XCreateGC(display,pixmap,0,&xgcv);
  xim = XGetImage(display,pixmap,0,0,width,height,AllPlanes,ZPixmap);
  
  if (!pixels) {
/*    DEBUG (stderr,"pixels not defined");*/
    RETURN (PixmapFileInvalid, 1);
  }

  j = 0 ;
  while (j < height)
    {  
      if (strlen(pixels[j]) != (chars_per_pixel*width)) {
/*      DEBUG (stderr,"bad pixmap line length %d",strlen(pixels[j]));*/
        RETURN (PixmapFileInvalid, 1);
      }
      switch (chars_per_pixel)
        {
        case 1:
          for (i=0; i< (width) ; i++)
            {
              c1 = pixels[j][i] ;
              for (p = 0 ; p < ncolors ; p++)
                if (cmap[p].cixel.c1 == c1) break ;
              if (p == ncolors) {
/*              DEBUG (stderr,"cixel \"%c\" not in previous colormap",c1);*/
                RETURN (PixmapFileInvalid, 1);
              }
              XPutPixel(xim,i,j,Tpixel[p]);
            }
          break;
        case 2:
          for (i=0; i< (2*width) ; i+=2)
            {
              c1 = pixels[j][i] ;
              c2 = pixels[j][i+1] ;
              for (p = 0 ; p < ncolors ; p++)
                if ((cmap[p].cixel.c1 == c1)&&(cmap[p].cixel.c2 == c2)) break ;
              if (p == ncolors) {
/*              DEBUG (stderr,"cixel \"%c%c\" not in previous colormap",c1,c2);*/
                RETURN (PixmapFileInvalid, 1);
              }
              XPutPixel(xim,i/2,j,Tpixel[p]);
            }
          break;
        }
      j++ ;
    }
  
  XPutImage(display,pixmap,Gc,xim,0,0,0,0,width,height);

  RETURN (pixmap, 0);
}

/****<eof>********************************************************************/
Pixmap XPM_PIXMAP
(
    Widget       w,
    unsigned int width,
    unsigned int height,
    unsigned int ncolors,
    unsigned int cpp,
    char         **colors,
    char         **pixels
)
{
        Colormap        colormap;
        int             depth;
        Arg             args[10];
        int             argcnt;
        Pixmap          pixmap;

        argcnt = 0;     
        XtSetArg(args[argcnt], XmNdepth, &depth); argcnt++;
        XtSetArg(args[argcnt], XmNcolormap, &colormap); argcnt++; 
        XtGetValues(w, args, argcnt);

        pixmap = XCreatePixmapFromData(XtDisplay(w), 
                DefaultRootWindow(XtDisplay(w)),
                colormap,
                width,
                height,
                depth,
                ncolors,
                cpp,
                colors,
                pixels);

        switch(pixmap)
        {
        case BitmapOpenFailed:
        case BitmapFileInvalid:
        case BitmapNoMemory:
            tie_message(Shell_Main, "Could not create pixmap.");
            return(XmUNSPECIFIED_PIXMAP);
        default:
            return(pixmap);
        }
}
#endif

#ifndef IGNORE_MENU_POST
void
MENU_POST(p, m, e)
Widget  p;
Widget  m;
XButtonEvent *e;
{
    Arg args[2];
    int argcnt;
    int button;

    argcnt = 0;
    XtSetArg(args[argcnt], XmNwhichButton, &button);
    argcnt++;
    XtGetValues(m, args, argcnt);
    if( e->button != button) return;
    XmMenuPosition(m, e);
    XtManageChild(m);
}
#endif
#ifndef IGNORE_STRING_TABLE
#include <stdarg.h>

XmString* STRING_TABLE(int num_strings, ...)
{
    va_list     ap;
    XmString    *array;
    int         i;

    va_start(ap, num_strings);
    array = (XmString*)XtMalloc((num_strings + 1) * sizeof(XmString*));
    for(i = 0;  i < num_strings; i++ )
    {
        array[i] = XmStringCreateLtoR(va_arg(ap, char*),
            XmSTRING_DEFAULT_CHARSET);
    }
    array[num_strings] = (XmString)0;
    va_end(ap);
    return(array);
}

void FREE_STRING_TABLE
(
    XmString *table
)
{
    int i;

    for( i = 0; table[i] != 0 ; i++)
    {
        XmStringFree(table[i]);
    }
    XtFree((XtPointer)table);
}
#endif

/******************************************************************************
 *                                                                            *
 *  The TIE_StringToPixel resource converter routine is taken from the        *
 *  CvtStringToPixel found in the Converters.c Xt source file.                *
 *                                                                            *
 ******************************************************************************/

/***********************************************************
Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

#define done(type, value) \
        {                                                       \
            if (toVal->addr != NULL) {                          \
                if (toVal->size < sizeof(type)) {               \
                    toVal->size = sizeof(type);                 \
                    return False;                               \
                }                                               \
                *(type*)(toVal->addr) = (value);                \
            }                                                   \
            else {                                              \
                static type static_val;                         \
                static_val = (value);                           \
                toVal->addr = (XtPointer)&static_val;           \
            }                                                   \
            toVal->size = sizeof(type);                         \
            return True;                                        \
        }

Boolean TIE_StringToPixel
(
    Display     *dpy,
    XrmValue    *args,
    Cardinal    *num_args,
    XrmValue    *fromVal,
    XrmValue    *toVal,
    XtPointer   *converter_data
)
{
    static XColor   screenColor;
    Screen          *screen;
    Colormap        colormap;
    Status          status;
    String          params[1];
    Cardinal        num_params=1;
    String          str = (String)fromVal->addr;

    if (*num_args != 2)
     XtAppErrorMsg(XtDisplayToApplicationContext(dpy), XtNwrongParameters,
        "cvtStringToPixel", "XtToolkitError",
        "String to pixel conversion needs screen and colormap arguments",
        (String *)NULL, (Cardinal *)NULL);

    screen = *((Screen **) args[0].addr);
    colormap = *((Colormap *) args[1].addr);

    if (CompareISOLatin1(str, XtDefaultBackground) == 0)
     {
        *converter_data = NULL;                 /* converter_data unused */
        done(Pixel, WhitePixelOfScreen(screen));/* could be black also   */
     }

    if (CompareISOLatin1(str, XtDefaultForeground) == 0)
     {
        *converter_data = NULL;                 /* converter_data unused */
        done(Pixel, BlackPixelOfScreen(screen));/* could be white also   */
     }

    if (*str == '#')    /* some color rgb definition */
     {
        status = getcolor((char *)str,&screenColor);

        if (status == 0)
         {
            params[0] = str;
            XtAppWarningMsg(XtDisplayToApplicationContext(dpy), "badFormat",
                "cvtStringToPixel", "XtToolkitError",
                "RGB color specification \"%s\" has invalid format",
                params, &num_params);
            *converter_data = NULL;             /* converter_data unused */
            return(False);                      /* return bad status     */
         }
        else
         {
            *converter_data = NULL;             /* converter_data unused */
            done(Pixel, screenColor.pixel);
         }

     }
    else  /* some color name */
     {
       status = getcolor((char *)str,&screenColor);
       if (status == 0)
        {
          String msg, type;
          params[0] = str;
        /* Server returns a specific error code but Xlib discards it.  Ugh */
          if (*str == '#' || (getcolor((char *)str,&screenColor)))
           {
            type = "noColormap";
            msg = "Cannot allocate colormap entry for \"%s\"";
           }
          else
           {
            type = "badValue";
            msg = "Color name \"%s\" is not defined in server database";
           }

          XtAppWarningMsg(XtDisplayToApplicationContext(dpy), type, 
                "cvtStringToPixel", "XtToolkitError", msg, params, &num_params);
          *converter_data = NULL;               /* converter_data unused */
          return(False);                        /* return bad status     */
        }
       else
        {
          *converter_data = NULL;               /* converter_data unused */
          done(Pixel, screenColor.pixel);
        }
    }
}

/******************************************************************************
 *                                                                            *
 *                                                                            *
 ******************************************************************************/

/*
 *  ISOLatin character checker taken from the Converters.c source file.
 *  This is used with the TIE_StringToPixel resource converter routine.
 */

static int CompareISOLatin1
(
    char *first,
    char *second
)
{
    register unsigned char *ap, *bp;

    for (ap = (unsigned char *) first, bp = (unsigned char *) second;
         *ap && *bp; ap++, bp++) {
        register unsigned char a, b;

        if ((a = *ap) != (b = *bp)) {
            /* try lowercasing and try again */

            if ((a >= XK_A) && (a <= XK_Z))
              a += (XK_a - XK_A);
            else if ((a >= XK_Agrave) && (a <= XK_Odiaeresis))
              a += (XK_agrave - XK_Agrave);
            else if ((a >= XK_Ooblique) && (a <= XK_Thorn))
              a += (XK_oslash - XK_Ooblique);

            if ((b >= XK_A) && (b <= XK_Z))
              b += (XK_a - XK_A);
            else if ((b >= XK_Agrave) && (b <= XK_Odiaeresis))
              b += (XK_agrave - XK_Agrave);
            else if ((b >= XK_Ooblique) && (b <= XK_Thorn))
              b += (XK_oslash - XK_Ooblique);

            if (a != b) break;
        }
    }
    return (((int) *bp) - ((int) *ap));
}

/******************************************************************************
 *                                                                            *
 *      Purpose:  This function is designed to input an X color specification *
 *                string and return a pixel value that corresponds to a       *
 *                an entry in the color store (tie.color_table[CSTORE_IDX]).  *
 *                The routine has been written for the X Image Display        *
 *                application and makes use of application-dependent globals. *
 *                                                                            *
 *      By D. Akkerman, USGS/EROS Data Center, 9/91.                          *
 *                                                                            *
 *      Algorithm(flow):  See in-line documentation.                          *
 *                                                                            *
 ******************************************************************************/
static Status getcolor
(
    char        *str,           /* Input color specification. (I)               */
    XColor  *color_cell /* Color cell to be assigned. (O)               */
)
{
Status  status,                 /* Return status variable.              */
        zero = 0,               /* Failure.                             */
        nonzero = 1;            /* Success.                             */

static int first = TRUE;        /* First time initialization flag.      */

int     i,                      /* Loop index.                          */
        retpix;                 /* Working search flag.                 */


/*
 *  Initialize the color store string table (once).
 */

if (first)
 {
  for (i = 0; i < NUM_CSTORE; i++)
    strcpy(tie.color_name[i],"");
  first = FALSE;
 }

/*
 *  Search the color store string table for a matching specification.
 */

/*
#ifdef TIE_DEBUG
printf("getcolor: %s\n",str);
#endif
*/

retpix = FALSE;
for (i = 0; i < tie.cindex; i++)
 if ((strcmp(tie.color_name[i],str)) == 0)
  {
   retpix = TRUE;
   break;
  }

/*
 *  If a match was found then assign the output values and return.
 */

if (retpix)
 {
  color_cell->pixel = tie.color_table[CSTORE_IDX + i].pixel;
  color_cell->red   = tie.color_table[CSTORE_IDX + i].red;
  color_cell->green = tie.color_table[CSTORE_IDX + i].green;
  color_cell->blue  = tie.color_table[CSTORE_IDX + i].blue;
  color_cell->flags = tie.color_table[CSTORE_IDX + i].flags;
/*
#ifdef TIE_DEBUG
printf("getcolor: pixel value = %d\n",color_cell->pixel);
#endif
*/
  return(nonzero);      /* For X compatability.         */
 }
else    /* Else create a new entry.                             */
 {
  status = XParseColor(tie.display,tie.colormap,(char *)str,color_cell);
  if (status != 0)
   {
    tie.color_table[CSTORE_IDX + tie.cindex].red   = color_cell->red;
    tie.color_table[CSTORE_IDX + tie.cindex].green = color_cell->green;
    tie.color_table[CSTORE_IDX + tie.cindex].blue  = color_cell->blue;
    tie.color_table[CSTORE_IDX + tie.cindex].flags = color_cell->flags;
    color_cell->pixel = tie.color_table[CSTORE_IDX + tie.cindex].pixel;

    strcpy(tie.color_name[tie.cindex],str);

    XStoreColor(tie.display,tie.colormap,
        &tie.color_table[CSTORE_IDX + tie.cindex]);

    tie.cindex++;
    if (tie.cindex >= NUM_CSTORE)
        tie.cindex = 0;
/*
#ifdef TIE_DEBUG
printf("getcolor: pixel value = %d\n",color_cell->pixel);
#endif
*/
    return(nonzero);    /* For X compatability.         */
   }
  else
   {
    color_cell->red = color_cell->green = color_cell->blue = 0;
    color_cell->flags = 0;
    color_cell->pixel = 0;
    return(zero);       /* For X compatability.         */
   }
 }
}

#include "typlim.h"

#define DASH    -1              /* Indicates the last char was a dash.        */
#define COMMA   -2              /* Indicates the last char was a comma.       */
#define NOPUNCT 0               /* Indicates last char was not punc.          */

int parse_rb
(
    char *str,          /* String to be parsed. (I)                           */
    int  **range,       /* Range values specified. (O)                        */
    int  *rcount,       /* Number of range values specified. (O)              */
    int  **bin,         /* Bin values specified. (O)                          */
    int  *bcount        /* Number of bin values specified. (O)                */
)
{

int     i,              /* Looping index.                                     */
        status=E_SUCC,  /* Status indicator.                                  */
        dash,           /* Position of a dash in the string.                  */
        comma,          /* Position of a comma in the string.                 */
        previous=0,     /* Type of previous character.                        */
        store,          /* Temporary storage variable.                        */
        position,       /* Position of last value parsed.                     */
        match = TRUE;   /* Indicates range value was matched.                 */

char    *tptr,          /* Text pointer.                                      */
        last[4];        /* Last string read.                                  */

tptr = str;
*rcount = *bcount = 0;
while (*tptr != '\0')
    {
    if (*tptr == '-')
        (*rcount)++;
    else if ((*tptr == ',') || (*tptr == ' ') || (*tptr == '\n'))
        (*bcount)++;
    tptr++;
    }
if ((*rcount == 0) && (*bcount == 0))
    *bcount = 1;

if (*rcount != 0) 
    *range = (int *) calloc((*rcount * 2),sizeof(int));
if (*bcount != 0) 
    *bin = (int *) calloc((*bcount+1),sizeof(int));

/*
    Validate the input specification.
*/
if (!isdigit(str[0]))
    {
    tie_message(Shell_Main,"First character of the input specification must be numeric");
    status = TIE_F;
    }
else
    for (i=0; i < (int)strlen(str); i++)
        {
        if (isalpha(str[i]))
            {
            tie_message(Shell_Main,"Alphabetic characters are not valid");
            status = TIE_F;
            break;
            }
        else if (ispunct(str[i]))
            {
            if ((str[i] != '-') && (str[i] != ',')) 
                {
                tie_message(Shell_Main,
"Punctuation characters other than (-) or (,) are not valid");
                status = TIE_F;
                break;
                }
            if (previous != NOPUNCT)
                {
                tie_message(Shell_Main,"Two consecutive punctuation marks are not valid");
                status = TIE_F;
                break;
                }
            if (str[i] == '-')
                {
                match = FALSE;
                previous = DASH;        
                }
            else
                previous = COMMA;
            }
        else if (!isspace(str[i]))
            {
            if (previous == DASH)
                match = TRUE;
            previous = NOPUNCT;
            } 
        }

/*
    Error was encountered so return.
*/

if (status != TIE_S) return(status);

/*
    Only one end of a range was specified (ie. " - 100" or "100 - ")
*/
if (!match)
    {
    tie_message(Shell_Main,"Invalid range specification");
    return (TIE_F);
    }
        
/*
    Parse the string into bin and range values.
*/
dash = strcspn (str,"-");
comma = strcspn (str,",");
tptr = str;
*bcount = *rcount = 0;

/*
    While there is more string to parse.
*/
while (*tptr != '\0')
    {
    if ((dash == strlen(tptr)) && (comma == strlen(tptr)))
        {
/*
    Read a bin value.
*/
        sscanf (tptr,"%d",(*bin + *bcount));
        if ((*(*bin + *bcount) > MAXUTINY) || (*(*bin + *bcount) < MINUTINY))
            {
            tie_message(Shell_Main,"Invalid bin value specified");
            return (TIE_F);
            }
        sprintf (last,"%d",*(*bin + *bcount));
        status = substr(last,tptr,&position);
        if (status != TIE_S)
           return(status);
        tptr[position+strlen(last)] = '\0';
        (*bcount)++;
        return (TIE_S);
        }
    else if (dash < comma)
        {
/*
    Read range values.
*/
        sscanf (tptr,"%d - %d",(*range + *rcount),(*range + *rcount + 1));
        if ((*(*range + *rcount) > MAXUTINY)  || (*(*range + *rcount) < 
            MINUTINY) || (*(*range + *rcount + 1) > MAXUTINY) || (*(*range + 
            *rcount + 1) < MINUTINY))
            {
            tie_message(Shell_Main,"Invalid value specified in the range");
            return (TIE_F);
            }
        sprintf (last,"%d",*(*range + *rcount + 1));

/*
    Switch order if range[0] is not less than or equal to range[1].
*/
        if (*(*range + *rcount) > *(*range + *rcount + 1))
            {
            store = *(*range + *rcount);
            *(*range + *rcount) = *(*range + *rcount + 1);
            *(*range + *rcount + 1) = store;
            }
        (*rcount)++;
        }
    else if (comma < dash)
        {
/*
    Read a bin value.
*/
        sscanf (tptr,"%d ,",(*bin + *bcount));
        if ((*(*bin + *bcount) > MAXUTINY) || (*(*bin + *bcount) < MINUTINY))
            {
            tie_message(Shell_Main,"Invalid bin value specified");
            return (TIE_F);
            }
        sprintf (last,"%d",*(*bin + *bcount));
        (*bcount)++;
        }
/*
    Strip off the portion of the string processed.
*/
    if (comma != strlen(tptr))
        tptr = tptr + comma + 1;
    else 
        {
        status = substr(last,tptr,&position);
        if (status != E_SUCC)
           return(status);
        tptr[position+strlen(last)] = '\0';
        tptr += comma;
        }
    dash = strcspn (tptr,"-");
    comma = strcspn (tptr,",");
    }
return (TIE_S);
}

/************************************************************************
 *                                                                      *
 *    Purpose:  DTSUBS checks if string2 is a substring of string1 and  *
 *              returns the position (1-relative) of the substring      *
 *              in string1.  If string2 is not a substring of string1,  *
 *              position = zero.                                        *
 *                                                                      *
 *    By:  S. Contractor, Code 933.1/NASA/GSFC, 5/24/84                 *
 *    Modifications:                                                    *
 *      D. Maynard, NASA/GSFC RMS, Code 933.1, August 16, 1984          *
 *         Rewritten in C language to improve execution efficiency and  *
 *         bypass some undesirable quirks of the FORTRAN INDEX function.*
 *      D. Hollaren, USGS/EDC Sept 18, 1985                             *
 *         Rewritten to be a useful routine.                            *
 *      C. Greenhagen, USGS/EROS Data Center, 5/88                      *
 *         DMS v3_0 retrocode.                                          *
 *                                                                      *
 ***********************************************************************/
int substr
(
    char string1[],                             /* string for comparison    (I) */
    char string2[],                             /* possible substring       (I) */
    int *pos                            /* position of string2 in   (O) */
)
                                        /* string1 (0 if not a substr)  */

{
int     len1, len2;
char    *s1, *addr;

/* Set pointer to start of string1, determine the length of both strings. */

s1 = string1;
len1 = strlen(string1);
len2 = strlen(string2);

for( ; ; )
        {
        /*  Find the address of the first occurrence in the search      */
        /*  string of the first character from string2.                 */

        addr = strchr (s1,*string2);

        /*  If the address returned is zero then the first character    */
        /*  from string2 is not in the search string.  Set the position */
        /*  variable to zero indicating that string2 is not a substring */
        /*  of string1.                                                 */

        if (addr == 0)
                {
                *pos = 0;
                break;
                }

        /*  If string2 is equal to the portion of string1 starting at   */
        /*  the address just found above and having the same length as  */
        /*  string2, then string2 is a substring of string1.  Calculate */
        /*  the position of the substring in string1.                   */

        /*  If string2 is not a substring at the address just found,    */
        /*  increment the pointer into string1 to start at the next     */
        /*  digit (address found above plus one).  This will in effect  */
        /*  shorten the search string so that this portion of string1   */
        /*  can be searched for the substring string2.                  */

        if (strncmp(addr,string2,len2) == 0)
                {
                *pos = len1 - strlen(addr) + 1;
                break;
                }
        else
                {
                s1 = addr + 1;
                }
        }
        
return (E_SUCC);
}
