/*
 *  *  Modified by T Rockvam, USGS/EROS Data Center, 3/94.
 *    - removed vector_file_* functions used by dlg loading
 */
#include <math.h>
#include <sys/signal.h>
#include <limits.h>     
#include <stdlib.h>
#include "tiepts.h"
#include "proj.h"
#include <X11/cursorfont.h>
#include "mapsup.h"
#include "cr.h"
#include "matsup.h"
#include "convert.h"
#include "wdbsup.h"
#include "tpssup.h"
#include "dlgsup.h"
#include "manage_cursor.h"
#include "protos.h"


#ifndef MIN
#define MIN(a,b)        ((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b)        ((a) > (b) ? (a) : (b))
#endif

/* tjr - temporary global because of division of vector.c and vector_tmp.c */
/* static Widget        *attrib_form = NULL; */
extern Widget   *attrib_form;


#define BITMAP 0x1

#define OVERLAY_CACHE_ALLOC_BLOCK 20
/* block size for growing the overlay cache */

#define OVERLAY_CACHE_WINDOW_PADDING 256
/* defines the number of pixels to grab outside the full res window for the
   vector overlay cache */

/*******************************************************************************
Purpose:
    This routine changes initializes the color requestor for altering
    the vector background color.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_background_button
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Color_req = Createcolor_req(Shell_Vector, vector_background_setcolor);
   XtManageChild(Color_req);
   XtSetSensitive(w, False);
}

/*******************************************************************************
Purpose:
    This routine assigns the vector background to the selected color.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_background_setcolor
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];

   XtUnmanageChild(Color_req);
   XtDestroyWidget(Color_req);
   Color_req = NULL;

   XtSetSensitive(vect_background_button, True);
   XtSetArg(args[0], XmNforeground, COLOR(Shell_Vector, 
            wk_colors[(int)client]));
   XtSetValues(vect_background_button, args, 1);

   XSetWindowBackground(tie.display, XtWindow(ref_image_bulletinboard),
                 tie.color_table[RGBBW_IDX + (int)client].pixel);
   XClearArea(tie.display, XtWindow(ref_image_bulletinboard), 0, 0, 0, 0, True);
}

/*******************************************************************************
Purpose:
    This routine blends the vectors into the zoom image.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_blend(void)
{
   GC           gc;                     /* graphics context             */
   XImage       *vectimg,               /* overlay bitmap               */
                *rastimg;               /* raster image                 */ 
   int          height,                 /* height of zoom window        */
                i, j,                   /* loop counters                */
                lastpix,                /* last pixel value             */
                red, green, blue,       /* blending color of pixel      */
                pix,                    /* resulting pixel value        */
                vectpixel,              /* vector blending value        */
                width;                  /* width of zoom window         */
   unsigned long rastpix,               /* raster pixel value           */
                overpix;                /* vector pixel value           */
   
   double       vectper,                /* percentage of vector         */
                rastper;                /* percentage of raster         */


  /*
   *   *** determine raster and vector percentages
   */
   rastper = ((double)tie_vector->blend)/100.;
   vectper = 1.0 - rastper;

  /*
   *  *** determine size of window
   */
   width = tie_parms.factor * tie_zoom[SEARCH].parms.in_samps;
   height = tie_parms.factor * tie_zoom[SEARCH].parms.in_lines;

  /*
   *  *** get images from pixmaps
   */
   rastimg = XGetImage(tie.display, tie_zoom[SEARCH].image_pixmap,
                       0, 0, width, height, AllPlanes, ZPixmap);
   vectimg = XGetImage(tie.display, tie_vector->blend_mask,
                       0, 0, width, height, 0x1, ZPixmap);

   gc = DefaultGC(tie.display, tie.screen);

   vectpixel = (int)(255*vectper);
   lastpix = -1;

   for (i = 0; i < width; i++)
      for (j = 0; j < height; j++)
          {
          if ((overpix = XGetPixel(vectimg, i, j)) > 0)
             {
             rastpix = XGetPixel(rastimg, i, j);

             if (tie.visual_info[0].visual->class == DirectColor)
                {
                red   = (rastpix & 0x0000ff) * rastper + vectpixel;
                green = ((rastpix & 0x00ff00) >>  8) * rastper + vectpixel;
                blue  = ((rastpix & 0xff0000) >> 16) * rastper + vectpixel;

                pix =  red + (green << 8) + (blue << 16);
                }
             else
                pix = (int)(rastpix*rastper) + vectpixel;

            /*
             *  *** try to minimize color sets
             */
             if (pix != lastpix)
                {
                lastpix = pix;
                XSetForeground(tie.display, gc, pix);
                }

             XDrawPoint(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                        gc, i, j);
             }
          }

   XDestroyImage(rastimg);
   XDestroyImage(vectimg);
}


/*******************************************************************************
Purpose:
    This routine changes the vector blend factor.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_blend_slider
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[10];       /* Resource argument                          */
   int          blend;          /* blending factor                            */

  /*
   *  ***  Get the slider value
   */
   XtSetArg(args[0],XmNvalue,&blend);
   XtGetValues(w, args, 1);
   
   if (tie_vector->blend == blend)
      return;

   tie_vector->blend = blend;

  /*
   *  *** if blending is zero, restore normal vectors
   */
   if ((tie_vector->blend_mask != NULL) && (tie_vector->blend == 0))
      {
      XFreePixmap(tie.display, tie_vector->blend_mask);
      tie_vector->blend_mask = NULL;
      }

   if (tie_vector->blend_mask != NULL) 
      vector_blend();
   else
      XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard),
              0, 0, 0, 0, True);
}

/*******************************************************************************
Purpose:
    This routine closes the vector manipulation. 

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   remove_window_cursor(tie.display,XtWindow(Shell_Vector));
   XtUnrealizeWidget(Shell_Vector);
   XtSetSensitive(main_vector_button, True);
}

/*******************************************************************************
Purpose:
    This routine displays the selected tie points on the vector display
    window.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_displaypts(void)
{
   int          i,
                x, y;
   double       vx, vy;

  /*
   *  *** if rough transformation defined, display selected tie points
   */
   if ((tie_rt.calc_rt == RT_SEL) || (tie_vector->numfiles == 0))
      return;

   for (i = 0; i < tie_select.seltie; i++)
       {
       vx = tie_select.rtie[i][0];
       vy = tie_select.rtie[i][1];
       mat_transform(tie_vector->tmatrix, &vx, &vy);
       x = (int)(rint(vx));
       y = (int)(rint(vy));

       XDrawPoint(tie.display, XtWindow(ref_image_bulletinboard),
           tie_select.gc, x, tie_vector->view_height - y);
       XDrawString(tie.display, XtWindow(ref_image_bulletinboard),
           tie_select.gc, x+5, tie_vector->view_height - y-5, 
           tie_select.pt_id[i], strlen(tie_select.pt_id[i]));
       }
}

/*******************************************************************************
Purpose:
    This routine is the event handler to draw a rubber rectangle about
    a region of the vector display to be zoomed.

By J. Hemmer, USGS/EROS Data Center, 6/92
Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - changed to draw rubber rectangles only when the coordinates are positive (
   ie., box is only drawn when the cursor is moved down or to the right of
   the cursor's starting location.

Algorithm Description:
*******************************************************************************/
void vector_drag
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   int          fnum,           /* file index of nearest node           */
                line,           /* line index of nearest node           */
                pt;             /* point index of nearest node          */
   double       angle,          /* rotation angle                       */
                deltax,         /* total movement in x direction        */
                deltay,         /* total movement in y direction        */
                scalex,         /* scale in x direction                 */
                scaley,         /* scale in x direction                 */
                rx, ry,         /* reference coordinates                */
                vx, vy;         /* vector coordinates                   */

   if (tie_vector->numlines == 0) 
      return;

  /*
   *  *** if reference overview, select region to zoom
   */
   if (w == ref_image_bulletinboard)
      {
      if (((tie_vector->pressx2 - tie_vector->pressx1) > 0) &&
          ((tie_vector->pressy2 - tie_vector->pressy1) > 0))
          XDrawRectangle(tie.display, XtWindow(ref_image_bulletinboard), 
                  tie.rect_gc,
                  tie_vector->pressx1, tie_vector->pressy1,
                  tie_vector->pressx2 - tie_vector->pressx1, 
                  tie_vector->pressy2 - tie_vector->pressy1);

      tie_vector->pressx2 = event->xmotion.x;
      tie_vector->pressy2 = event->xmotion.y;

      if (((tie_vector->pressx2 - tie_vector->pressx1) > 0) &&
          ((tie_vector->pressy2 - tie_vector->pressy1) > 0))
         XDrawRectangle(tie.display, XtWindow(ref_image_bulletinboard), 
                  tie.rect_gc,
                  tie_vector->pressx1, tie_vector->pressy1,
                  tie_vector->pressx2 - tie_vector->pressx1, 
                  tie_vector->pressy2 - tie_vector->pressy1);
      }
  /*
   *  *** otherwise, perform transformation on vector overlay
   */
   else
      {
      deltax = (double)(event->xmotion.x - tie_vector->pressx1);
      deltay = (double)(event->xmotion.y - tie_vector->pressy1);
      tie_vector->pressx1 = event->xmotion.x; 
      tie_vector->pressy1 = event->xmotion.y;

      if (w == zoom_srchwin_bulletinboard)
         {
         deltax /= (double)tie_parms.factor;
         deltay /= (double)tie_parms.factor;
         }

     /*
      *  *** if rotation or scale, find center of window
      */
      if (tie_vector->transform != TRANSLATE)
         {
         vector_nnode(zoom_srchwin_bulletinboard, 
                      tie_zoom[SEARCH].parms.extract_lines/2,
                      tie_zoom[SEARCH].parms.extract_lines/2,
                      &fnum, &line, &pt);

         vx = tie_vector->line[line].pts[pt*2];
         vy = tie_vector->line[line].pts[pt*2+1];
         vect2full(vx, vy, &rx, &ry);
         rx += tie_srch_fullres[0].window[SS];
         ry += tie_srch_fullres[0].window[SL];
         }

     /*
      *  *** perform transformation
      */
      if (tie_vector->transform == TRANSLATE)
         {
         mat_translate(tie_vector->zmatrix, deltax, deltay);
         }
      else if (tie_vector->transform == ROTATE)
         {
         angle = (deltax + deltay)/(100.0*2.0*PI);
         mat_rotate(tie_vector->zmatrix, angle, rx, ry);
         }
      else if (tie_vector->transform == SCALE)
         {
         scalex = 1.0 + deltax/100.0;
         scaley = 1.0 - deltay/100.0;

         mat_scale(tie_vector->zmatrix, scalex, scaley, rx, ry);
         }

     /*
      *  *** force exposure of fullres and zoom windows
      */ 
      XClearArea(tie.display, XtWindow(fullsrch_win_bulletinboard),
                 0, 0, 0, 0, True);
    
      if (XtIsRealized(Shell_Zoom))
         XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                    0, 0, 0, 0, True);
      }
}

/*******************************************************************************
Purpose:
    This routine is the event handler to refresh the vector map display when
    corrupted or resized.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void vector_expose
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XExposeEvent            *xe;         /* X event                        */
   XmDrawingAreaCallbackStruct  *cbs;   /* callback structure             */
   GC           gc;                     /* graphics context               */
   int          i;                      /* loop counter                   */
   double       sx, sy;                 /* scale factors                  */
   short        height,                 /* dimension of display window    */
                width;

   if (tie_vector->numlines <= 0)
      return;

   cbs = (XmDrawingAreaCallbackStruct*)call_data;

  /*
   *   *** update if resized, or no more pending refreshes
   */
   if (cbs->reason == XmCR_RESIZE)
      {
     /*
      *  *** get new dimensions 
      */
      XtVaGetValues(w,
         XmNheight,             &height,
         XmNwidth,              &width,
         NULL);

      sx = ((double)width)/((double)(tie_vector->view_width));
      sy = ((double)height)/((double)(tie_vector->view_height));

      mat_scale(tie_vector->tmatrix, sx, sy, 0.0, 0.0);

      tie_vector->view_width  = width;
      tie_vector->view_height = height;
    
     /*
      *  *** since size changed, destroy current pixmaps
      *      they will be recreated to new size in redrawall
      */
      for (i = 0; i < NUM_CVALS; i++)
         {
         if (tie_vector->pixmap[i] != NULL)
            XFreePixmap(tie.display, tie_vector->pixmap[i]);
         tie_vector->pixmap[i] = NULL;
         }

      XClearWindow(tie.display, XtWindow(ref_image_bulletinboard));
      vector_redrawall();
      }
   else if (cbs->reason == XmCR_EXPOSE)
      {
      xe = (XExposeEvent*)cbs->event;
      gc = DefaultGC(tie.display, tie.screen);

     /*
      *  *** overlay individual planes
      */
      for (i = 0; i < NUM_CVALS; i++)
         {
         if (tie_vector->pixmap[i] != NULL)
            {
            XSetClipMask(tie.display, gc, tie_vector->pixmap[i]);
            XSetForeground(tie.display, gc, COLOR(Shell_Vector, wk_colors[i]));
            XCopyPlane(tie.display, tie_vector->pixmap[i],
                    XtWindow(ref_image_bulletinboard), gc,
                    xe->x, xe->y, xe->width, xe->height,
                    xe->x, xe->y, 1);
            }
         }

     /*
      *  *** display selected points
      */
      XSetClipMask(tie.display, gc, None);
      vector_displaypts();

      if (tie_select.curref)
         vector_mark();
      }
}

/*******************************************************************************
Purpose:
    This routine returns the index of the file containing a given
    vector line.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
int vector_fileline
(
    int line
)
{
   int  i;

   for (i = 0; i < tie_vector->numfiles; i++)
      {
      if ((tie_vectfile[i].startline <= line) &&
          (tie_vectfile[i].endline >= line))
         return(i);
      }

   return(0);
}

/*******************************************************************************
Purpose:
    This routine displays the vector file selector.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_file_button
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Widget       wdblist;

  /*
   *   ***  if data source is DLG
   */
   if (tie_parms.vsource == DLGOPT)
      {
      Vect_fileselector = Createvect_fileselector(Shell_Vector, client);
      XtSetSensitive(dlg_file_button, False);
      }

  /*
   *   ***  else if data source is WDB
   */
   else if (tie_parms.vsource == WDB)
      {
      Vect_fileselector = Creategeneral_selector(Shell_Vector, &wdblist,
                                   vector_wdbload);
      wdb_filelist(wdblist);
      XtSetSensitive(wdb_cont_button, False);
      }
  /*
   *   ***  else if data source is DCW 
   */
   else if (tie_parms.vsource == DCW)
      return;

   XtManageChild(Vect_fileselector);
}

/*******************************************************************************
Purpose:
    This routine is the callback for canceling a vector file selection.

By J. Hemmer, USGS/EROS Data Center, 2/92
Removed for v1.1 and re-added in v1.2.

Algorithm Description:
*******************************************************************************/
void vector_file_cancel
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XtUnmanageChild(Vect_fileselector);
   XtDestroyWidget(Vect_fileselector);

   if (tie_parms.vsource == DLGOPT)
      XtSetSensitive(dlg_file_button, True);
   else if (tie_parms.vsource == WDB)
      XtSetSensitive(wdb_cont_button, True);
}

/*******************************************************************************
Purpose:
    This routine is the callback for accepting or canceling a vector
    file selection.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to check the status from the routine dlg_optread.
Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - added fnum to process the information for the correct (current)
   element of the tie_vectfile array.
Modified by T. Rockvam, USGS/EROS Data Center, 11/93
 - added manual flag to the dlg_optread call
Removed for v1.1 - No more loading dlg files by name
Re-added for v1.2 - Allow loading of non-standard DLG files by name

Algorithm Description:
*******************************************************************************/
void vector_file_ok
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i,              /* loop counter                         */
                status;         /* status indicator                     */
   char         *filename;      /* selected filename                    */

   XmFileSelectionBoxCallbackStruct *acs =
             (XmFileSelectionBoxCallbackStruct*)call_data;

   /* Make sure there are some features selected */
   status = FALSE;
   i = 0;
   while (dlg_attrib[i].mincode != 0)
   {
      if (dlg_attrib[i].present == TRUE)
      {
         status = TRUE;
         break;
      }
      i++;
   }
   if (!status)
   {
      tie_message(Shell_Vector, "No Feature requested");
      return;
   }

   /* Retrieve the file name */
   XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);

   /* free all the previously read vectors */
   vector_free();

   /* load the dlg file, ignoring its return status since if an error happened
      it will have printed a message already and a no lines message will be
      generated when the display is updated */
   load_dlg_file(filename);
   
   XtFree(filename);

   /* update the dlg display window */
   update_dlg_display();

   /* remove the file selector window and allow more selection */
   XtUnmanageChild(Vect_fileselector);
   XtDestroyWidget(Vect_fileselector);
   XtSetSensitive(dlg_file_button, True);
}

/*******************************************************************************
Purpose:
    This routine is the callback for accepting all vector files.

By T. Rockvam, USGS/EROS Data Center, 11/93
Removed for v1.1 and re-added in v1.2.

Algorithm Description:
*******************************************************************************/
void vector_file_all

(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[2];        /* widget resource arguments            */
   int          i,              /* loop counters                        */
                status,         /* status indicator                     */
                file_count = 0; /* number of selection files            */
   XmStringTable file_list;     /* list of file names                   */
   char         *filename;      /* selected filename                    */
   char         title[100];     /* processing message                   */
   char         loaded = FALSE; /* flag if files were loaded            */

   /* Make sure there are some features selected */
   status = FALSE;
   i = 0;
   while (dlg_attrib[i].mincode != 0)
   {
      if (dlg_attrib[i].present == TRUE)
      {
         status = TRUE;
         break;
      }
      i++;
   }
   if (!status)
   {
      tie_message(Shell_Vector, "No Feature requested");
      return;
   }

   /* Retrieve the list of files */
   XtVaGetValues(w,XmNfileListItemCount,&file_count,
                   XmNfileListItems,&file_list,NULL);

   if (file_count > 0)
   {
      /* free all previously read vectors and clear the screen */
      if (tie_vector->numlines > 0)
         vector_free();
      XClearArea(tie.display,XtWindow(search_image_bulletinboard),0,0,0,0,
                 False);

      /* display the processing message and processing icon */
      XtPopup(process_dialog_shell,XtGrabNone);
      sprintf(title,"Loading DLG files ... ");
      XtSetArg(args[0], XeNlabelString, title);
      XtSetArg(args[1], XeNupdatePercent, 1);
      XtSetValues(process_widget, args, 2);
      XeProcessMInit(process_widget,0,file_count);
      XeProcessMUpdate(process_widget,0);
      ForceDialog(process_dialog_shell);
   }

   /* process each file */
   for (i = 0; i < file_count; i++)
   {
      if (XeProcessMUpdate(process_widget,i) < 0)
      {
         XFlush(tie.display);
         XtPopdown(process_dialog_shell);
         break;
      }

      XmStringGetLtoR(file_list[i],XmSTRING_DEFAULT_CHARSET,&filename);

      /* if one of the files is already loaded just continue on */
      if (vector_isloaded(filename))
      {
         XtFree(filename);
         continue;
      }

      /* read the dlg file */
      status = load_dlg_file(filename);
      XtFree(filename);
      if (status != E_SUCC)
         continue;
      loaded = TRUE;
   }

   /* update the dlg display */
   update_dlg_display();

   /* remove the file selector window and allow more selection */
   XtUnmanageChild(Vect_fileselector);
   XtDestroyWidget(Vect_fileselector);
   XtSetSensitive(dlg_file_button, True);

   /* Close the processing message window and turn on the bell to let the user
      know the DLG load is complete. */
   XFlush(tie.display);
   XtPopdown(process_dialog_shell);
   XBell(tie.display,0);

}

/*******************************************************************************
Purpose:
    This routine finds the lower left and upper right coordinates 
    with respect to the image projection.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to allow the user to specify a window for clipping DLG data.

Algorithm Description:
*******************************************************************************/
void vector_dlgbounds(void)
{
   int          i, j;
   double       pt[8];

   /* Convert corner coordinates to image coordinates */
   pt[0] = tie_vectfile[0].corner[0][0]; 
   pt[1] = tie_vectfile[0].corner[0][1];
   degrees2vect(pt,1,0);
   tie_vector->urx = tie_vector->llx = pt[0];
   tie_vector->ury = tie_vector->lly = pt[1];

   /* Find min and max values in the image projection coordinates */
   for (i = 0; i < tie_vector->numfiles; i++)
   {
      for (j = 0; j < 4; j++)
      {
         pt[2*j]   = tie_vectfile[i].corner[j][0];
         pt[2*j+1] = tie_vectfile[i].corner[j][1];
      }

      degrees2vect(pt, 4, 0);

      for (j = 0; j < 4; j++)
      {
         if (pt[2*j] < tie_vector->llx)
            tie_vector->llx = pt[2*j];

         if (pt[2*j] > tie_vector->urx)
            tie_vector->urx = pt[2*j];

         if (pt[2*j+1] < tie_vector->lly)
            tie_vector->lly = pt[2*j+1];

         if (pt[2*j+1] > tie_vector->ury)
            tie_vector->ury = pt[2*j+1];
      }
   }
}

/*******************************************************************************
Purpose:
    This routine frees the allocated space for replaced vectors.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void vector_free(void)
{
   int          i, j;           /* loop counter         */

   /* erase outline on search image */
   erase_vectoutline();

   /* free lines */
   for (i = 0; i < tie_vector->numlines; i++)
      free(tie_vector->line[i].pts);
   tie_vector->numlines = 0;

   /* free vector file information */
   for (i = 0; i < tie_vector->numfiles; i++)
   {
      free(tie_vectfile[i].filename);
      tie_vectfile[i].filename = NULL;
   }

   tie_vector->numfiles = 0;

   /* reset WDB loaded flags */
   if (tie_parms.vsource == WDB)
   {
      for (i = 0; i < WDBFEATURES; i++)
         wdb_attrib[i].loaded = FALSE;
   }
   else if (tie_parms.vsource == DCW)
   {
      i=0;
      while(dcw_coverage[i].desc != NULL)
      {
         for (j = 0; j < dcw_coverage[i].numvdt; j++)
             dcw_coverage[i].vdt[j].loaded = FALSE;
         i++;
      }
   }

   free(tie_vector->line);
   tie_vector->maxline   = LINEBLOCK;
   tie_vector->line      = TieMalloc(LINEBLOCK*sizeof(struct TIE_LINE));
}

/*******************************************************************************
Purpose:
    This routine returns TRUE if the given vector file is already loaded.

RETURNS:
    TRUE if vector file is already loaded
    FAALSE if vector file is not already loaded

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
int vector_isloaded
(
   char         *filename
)
{
   int          i;

   for (i = 0; i < tie_vector->numfiles; i++)
   {
      if (strcmp(filename, tie_vectfile[i].filename) == 0)
         return(TRUE);
   }

   return(FALSE);
}

/*******************************************************************************
Purpose:
    This routine redraws the given line type button.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void vector_line_expose
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   GC           gc;             /* X Graphic Context structure                */
   XGCValues    values;         /* X Graphic Context values                   */
   Dimension    bwidth,         /* border width of drawn button               */
                height,         /* height of button                           */
                shthick,        /* shadow thickness drawn button              */
                width;          /* width of button                            */
   char* dash;          /* dash pattern                               */

   dash = (char*)client;

   XtVaGetValues(w, 
         XmNborderWidth,        &bwidth,
         XmNshadowThickness,    &shthick,
         XmNwidth,              &width,
         XmNheight,             &height,
         NULL);

  /*
   *  ***  create marker graphic context
   */
   values.line_width = 2;

   if (dash[1] == 0)
      values.line_style = LineSolid;
   else
      values.line_style = LineOnOffDash;

   gc = XCreateGC(tie.display, XtWindow(w), GCLineStyle | GCLineWidth, &values);

   if (dash[1] != 0)
      XSetDashes(tie.display, gc, 0, dash, 2);

   XClearArea(tie.display, XtWindow(w), 
              bwidth+shthick+2, bwidth+shthick+2, 
              width - 2*(bwidth+shthick+2), 
              height - 2*(bwidth+shthick+2), 
              False);
   XDrawLine(tie.display, XtWindow(w), gc, 
             bwidth + shthick+2, height/2, 
             width - (bwidth + shthick+2) , 
             height/2);
   XFreeGC(tie.display, gc);
}

/*******************************************************************************
Purpose:
    This marks the current points selected using the alignment method.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void vector_mark(void)
{
   double       vx, vy;         /* vector coordinates           */
   int          x, y;           /* window coordinates           */

   if (tie_select.rtemp_flag)
      {
      vx = tie_select.rtemp[0];
      vy = tie_select.rtemp[1];
      }
   else
      {
      vx = tie_select.rtie[tie_select.seltie][0];
      vy = tie_select.rtie[tie_select.seltie][1];
      }
   mat_transform(tie_vector->tmatrix, &vx, &vy);

   x = (int)(rint(vx));
   y = tie_vector->view_height - (int)(rint(vy));
   XDrawArc(tie.display, XtWindow(ref_image_bulletinboard), tie_select.gc,
                x-2, y-2, 5, 5, 0, 360*64);
}

/*******************************************************************************
Purpose:
    This routine findes the closest line and node to the given coordinates.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
int vector_nnode
(
   Widget       w,
   int x, 
   int y,
   int *fnum,
   int *line,
   int *node
)
{
   double       dist,           /* distance to given node               */
                mindist,        /* current minimum distance node        */
                cx, cy,         /* cursor coordinates                   */
                tempx, tempy,   /* temporary variable for computing distatnce */
                vx, vy;         /* vector coordinates                   */
   int          attrib,         /* attribute index                      */
                i, j;           /* loop counters                        */

   if (tie_vector->numlines <= 0)
      return(E_FAIL);

  /*
   *  *** transform cursor coordinates to overview coordinates
   */
   cx = (double)x;
   if (w == ref_image_bulletinboard)
      cy = (double)(tie_vector->view_height - y);
   else
      cy = (double)y;

   *line = -1;

  /*
   *  *** for each line
   */
   for (i = 0; i < tie_vector->numlines; i++)
      {
      attrib = tie_vector->line[i].attrib;
      if (!tie_vector->enabled[attrib])
         continue;

     /*
      *  *** if first line, compute distance to first node
      */
      if (*line < 0)
         {
         *line = i;
         *node  = 0;
         vx = tie_vector->line[i].pts[0];
         vy = tie_vector->line[i].pts[1];

         if (w == zoom_srchwin_bulletinboard)
            {
            tempx = vx;
            tempy = vy;
            vect2zoom(tempx, tempy, &vx, &vy);
            }
         else
            mat_transform(tie_vector->tmatrix, &vx, &vy);
 
         mindist = sqrt((cx - vx)*(cx - vx) + (cy-vy)*(cy-vy));
         }

     /*
      *  *** for each point in the line
      */
      for (j = 0; j < tie_vector->line[i].npts; j++)
         {
         vx = tie_vector->line[i].pts[2*j];
         vy = tie_vector->line[i].pts[2*j+1];

         if (w == zoom_srchwin_bulletinboard)
            {
            tempx = vx;
            tempy = vy;
            vect2zoom(tempx, tempy, &vx, &vy);
            }
         else
            mat_transform(tie_vector->tmatrix, &vx, &vy);

         dist = sqrt((cx - vx)*(cx - vx) + (cy-vy)*(cy-vy));

         if (dist < mindist)
            {
            *line = i;
            *node = j;
            mindist = dist;
            }
         }
      }

   *fnum = vector_fileline(*line);

   if (*line >= 0)
      return(E_SUCC);
   else
      return(E_FAIL);
}

/*******************************************************************************
Purpose:
    This routine draws the vector on top of the zoom or fullres image.
    If blending mode and drawing in zoom window, only draws vectors
    to a bitmap.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void vector_overlay
(
   Widget       w
)
{
    GC        gc;             /* graphics context for overlay vectors */
    XGCValues values;         /* graphics context values              */
    XPoint    *pts=NULL;      /* list of points in a line             */
    int       pts_allocated=0;/* number of points allocated for pts   */
    double    rx, ry,         /* reference coordinates                */
              sx, sy;         /* search coordinates                   */

    int       attrib,         /* vector attribute index               */
              blending=FALSE, /* blending flag                        */
              dindex,         /* dash pattern index                   */
              i, j,           /* loop counters                        */
              plane;          /* graphics plane                       */
    int       last_dindex=-1; /* saved index of last dash pattern so 
                                 some X state changes can be skipped  */
    int       last_plane=-1;  /* saved index of plane so some color   
                                 changes can be optimized out         */
    int       index;          /* loop index */
    int       max_loop;       /* loop ending condition */
    int       use_cache;      /* flag to use the line cache */
    int       invalidate_cache = 0; /* flag to invalidate the cache */
    int       in_window;      /* flag to indicate the current line is in
                                 the full res window when building the 
                                 cache */
    int end_line, end_sample; /* last line and sample in the cache window */

    typedef struct
    {
        int   fullres_window[4]; /* fullres_window covered by cache */
        int   *lines;            /* array of lines contained in cache */
        int   num_lines;         /* number of lines in lines array */
        int   size_lines;        /* number of lines that fit in lines array */
        int   vector_lines;      /* number of total vector lines */
        int   vector_files;      /* total number of vector files */
        unsigned char *enabled_attrib; /* array of attributes */
        int   num_attrib;        /* number of enabled attributes */
        int   ref_x;             /* reference x location for detecting when 
                                    cache no longer contains vectors needed */
        int   ref_y;             /* reference y location for detecting when 
                                    cache no longer contains vectors needed */
    } LINE_CACHE;
    /* data tracked for the line cache */

    static LINE_CACHE cache = {{-1,-1,-1,-1},NULL,0,0,0,0,NULL,0,0,0};
        /* cache for lines that are visible */


    if ((w == zoom_srchwin_bulletinboard) && (!(tie.flags & TIE_ALIGNING)))
        return;
    else if ((w == zoom_srchwin_bulletinboard) && (tie_vector->blend))
    {
        blending = TRUE;

        if (tie_vector->blend_mask == NULL)
        {
            tie_vector->blend_mask = XCreatePixmap(tie.display, 
                  XtWindow(zoom_srchwin_bulletinboard),
                  tie_parms.factor * tie_zoom[SEARCH].parms.in_samps,
                  tie_parms.factor * tie_zoom[SEARCH].parms.in_lines, 0x1);
        }
    }


    /* if blending, draw vectors in bitmap */
    if (!blending)
    {
        gc = XCreateGC(tie.display, XtWindow(w), NULL, &values);
        XSetBackground(tie.display,gc,
                       tie.color_table[RGBBW_IDX + RED_COL].pixel);
        XSetForeground(tie.display,gc,
                       tie.color_table[RGBBW_IDX + RED_COL].pixel);
    }
    else
    {
        gc = XCreateGC(tie.display, tie_vector->blend_mask, NULL, &values);
        XSetPlaneMask(tie.display, gc, 1);
        XSetForeground(tie.display, gc, 0);
        XSetBackground(tie.display, gc, 0);
        XFillRectangle(tie.display, tie_vector->blend_mask, gc, 0, 0,
                       tie_parms.factor * tie_zoom[SEARCH].parms.in_samps,
                       tie_parms.factor * tie_zoom[SEARCH].parms.in_lines);

        XSetBackground(tie.display, gc, 1);
        XSetForeground(tie.display, gc, 1);
    }

    /* check if the cache invalidation conditions */
    /* window moved? */
    if ((cache.fullres_window[SL] != tie_srch_fullres[0].window[SL]) ||
        (cache.fullres_window[SS] != tie_srch_fullres[0].window[SS]) ||
        (cache.fullres_window[NL] != tie_srch_fullres[0].window[NL]) ||
        (cache.fullres_window[NS] != tie_srch_fullres[0].window[NS]))
    {
        invalidate_cache = 1;
    }
    /* number of lines, files, or attributes changed? */
    else if ((cache.vector_lines != tie_vector->numlines) ||
            (cache.vector_files != tie_vector->numfiles) ||
            (cache.num_attrib != tie_vector->nattrib))
    {
        invalidate_cache = 1;
    }
    /* enabled attributes changed? */
    else
    {
        for (i = 0; i < cache.num_attrib; i++)
        {
            if (cache.enabled_attrib[i] != tie_vector->enabled[i])
            {
                invalidate_cache = 1;
                break;
            }
        }
    }
    /* moved beyond the cache limit? */
    if (!invalidate_cache)
    {
        double x = cache.fullres_window[SS];
        double y = cache.fullres_window[SL];

        mat_transform(tie_vector->zmatrix,&x,&y);
        if ((abs((int)x - cache.ref_x) > OVERLAY_CACHE_WINDOW_PADDING) ||
            (abs((int)y - cache.ref_y) > OVERLAY_CACHE_WINDOW_PADDING))
        {
            /* cache doesn't cover the current area, so regenerate it */
            invalidate_cache = 1;
        }
    }

    /* prepare for the loop to either use the cached set of lines found in
       an earlier call, or search the whole set of lines. */
    if (!invalidate_cache)
    {
        /* cache is okay, so use it */
        max_loop = cache.num_lines;
        use_cache = 1;
    }
    else
    {
        double x,y;
        /* cache needs to be regenerated, so use full set of lines */
        max_loop = tie_vector->numlines;
        use_cache = 0;

        /* update cache tracking values */
        cache.fullres_window[SL] = tie_srch_fullres[0].window[SL];
        cache.fullres_window[SS] = tie_srch_fullres[0].window[SS];
        cache.fullres_window[NL] = tie_srch_fullres[0].window[NL];
        cache.fullres_window[NS] = tie_srch_fullres[0].window[NS];
        cache.num_lines = 0;
        cache.vector_lines = tie_vector->numlines;
        cache.vector_files = tie_vector->numfiles;
        cache.num_attrib = tie_vector->nattrib;
        cache.enabled_attrib = realloc(cache.enabled_attrib,
                                  cache.num_attrib * sizeof (unsigned char));
        if (cache.enabled_attrib == NULL)
        {
            tie_message(Shell_Vector,"Memory allocation error - exit TIEPTS");
            return;
        }
        for (i = 0; i < cache.num_attrib; i++)
        {
            cache.enabled_attrib[i] = tie_vector->enabled[i];
        }

        /* set the reference point for the vect2full transformation.  This
           is used to detect when the overlay has moved to be outside the 
           extents of the cache. */
        x = cache.fullres_window[SS];
        y = cache.fullres_window[SL];
        mat_transform(tie_vector->zmatrix,&x,&y);
        cache.ref_x = (int)x;
        cache.ref_y = (int)y;
        
        /* set end line and sample to include in cache */
        end_line = cache.fullres_window[NL] + OVERLAY_CACHE_WINDOW_PADDING;
        end_sample = cache.fullres_window[NS] + OVERLAY_CACHE_WINDOW_PADDING;
    }

    /* loop through the line segments */
    for (index = 0; index < max_loop; index++)
    {
        /* get the line index to use based on whether the cache is being
           used */
        if (use_cache)
            i = cache.lines[index];
        else
            i = index;
          
        /* set color and dash pattern */
        attrib = tie_vector->line[i].attrib;

        /* if toggled off, skip to next line */
        if (!tie_vector->enabled[attrib])
            continue;

        plane = tie_vector->color_att[attrib];

        /* if more memory is needed for the points, allocate it */
        if (pts_allocated < tie_vector->line[i].npts)
        {
            pts_allocated = tie_vector->line[i].npts;
            pts = (XPoint*)realloc(pts, pts_allocated * sizeof(XPoint)); 
            if (pts == NULL)
            {
                tie_message(Shell_Vector,
                            "Memory allocation error - exit TIEPTS");
                return;
            }
        }

        if (!blending)
        {
            if (plane != last_plane)
            {
                last_plane = plane;
                XSetForeground(tie.display, gc, 
                               COLOR(Shell_Vector, wk_colors[plane]));
            }
        }
    
        /* set line dash pattern */
        dindex = tie_vector->line_att[attrib];

        /* if the line dash pattern has changed, update the X server state */
        if (dindex != last_dindex)
        {
            last_dindex = dindex;

            if (dindex > 0)
            {
                XSetLineAttributes(tie.display, gc, 0, LineOnOffDash,
                                   CapButt, JoinBevel);
                XSetDashes(tie.display, gc, 0, dash_pattern[dindex], 2);
            }
            else
                 XSetLineAttributes(tie.display, gc, 0, LineSolid,
                                    CapButt, JoinBevel);
        }

        /* for each point in line segment */
        in_window = 0;
        for (j = 0; j < tie_vector->line[i].npts; j++)
        {
            rx = tie_vector->line[i].pts[j*2];
            ry = tie_vector->line[i].pts[j*2+1];
   
            /* determine coordinates of this node of line segment */
            if (w == fullsrch_win_bulletinboard)
                vect2full(rx, ry, &sx, &sy);
            else if (w == zoom_srchwin_bulletinboard)
                vect2zoom(rx, ry, &sx, &sy);

            pts[j].x = (int)(rint(sx));
            pts[j].y = (int)(rint(sy));

            /* if not using the cache, check if this line falls in the window 
               so it can be added to the cache */
            if (!use_cache)
            {
                /* determine if each line is really visible */
                if ((pts[j].x >= -OVERLAY_CACHE_WINDOW_PADDING) && 
                      (pts[j].x <= end_line) &&
                    (pts[j].y >= -OVERLAY_CACHE_WINDOW_PADDING) && 
                      (pts[j].y <= end_sample))
                {
                    /* line is visible */
                    in_window = 1;
                }
            }
        } /* for each point */
      
        /* if in window add line to cache */
        if (in_window)
        {
            if (cache.num_lines >= cache.size_lines)
            {
                /* grow the size of the cache */
                cache.size_lines += OVERLAY_CACHE_ALLOC_BLOCK;
                cache.lines = realloc(cache.lines, 
                                      cache.size_lines * sizeof(int));
                if (cache.lines == NULL)
                {
                    tie_message(Shell_Vector,
                                "Memory allocation error - exit TIEPTS");
                    return;
                }
            }

            cache.lines[cache.num_lines] = i;
            cache.num_lines++;
        }
      
        if (!blending)
        {
            XDrawLines(tie.display, XtWindow(w), gc, pts, 
                       tie_vector->line[i].npts, CoordModeOrigin);
        }
        else
        {
            XDrawLines(tie.display, tie_vector->blend_mask, gc, pts, 
                       tie_vector->line[i].npts, CoordModeOrigin);
        }
    } /* for each line */

    free(pts);

    XFreeGC(tie.display, gc);

    if ((blending) && (tie_vector->blend > 0))
        vector_blend();
}

/*******************************************************************************
Purpose:
    This routine is the redraws all vectors after a load or resize.

By J. Hemmer, USGS/EROS Data Center, 6/92

Modified by T. Rockvam, USGS/EROS Data Center, 11/93
 - fixed an overflow error that occurred when the user drew a zoom window
   in the reference panel too many times 

Algorithm Description:
*******************************************************************************/
void vector_redrawall(void)
{
   double               vx,vy;          /* vector coordinates             */
   XPoint*              pts;            /* list of points in a line       */
   int                  attrib,         /* attribute index                */
                        dindex,         /* line dash index of line        */
                        plane;          /* color index of line            */
   int  i, j;

   XSetForeground(tie.display,tie.bit_gc, 0);

   for (i = 0; i < NUM_CVALS; i++)
   {
      if (tie_vector->pixmap[i] != NULL)
      {
         XFillRectangle(tie.display,tie_vector->pixmap[i],
                     tie.bit_gc, 0, 0, tie_vector->view_width, 
                     tie_vector->view_height);
      }
   }

   XSetForeground(tie.display,tie.bit_gc, 1);

   /* for each line */
   for (i = 0; i < tie_vector->numlines; i++)
   {
      /* set color and dash pattern */
      attrib = tie_vector->line[i].attrib;
      if (!tie_vector->enabled[attrib])
         continue;

      plane = tie_vector->color_att[attrib];
      dindex = tie_vector->line_att[attrib];

      /* if this plane was not previously used, create it */
      if (tie_vector->pixmap[plane] == NULL)
      {
         tie_vector->pixmap[plane] = XCreatePixmap(tie.display,
              XtWindow(ref_image_bulletinboard),tie_vector->view_width,
              tie_vector->view_height, BITMAP);
         XSetForeground(tie.display,tie.bit_gc, 0);
         XFillRectangle(tie.display,tie_vector->pixmap[plane],
                     tie.bit_gc, 0, 0, tie_vector->view_width, 
                     tie_vector->view_height);
         XSetForeground(tie.display,tie.bit_gc, 1);
      }

      if (dindex > 0)
      {
         XSetLineAttributes(tie.display, tie.bit_gc, 0, LineOnOffDash,
                     CapButt, JoinBevel);
         XSetDashes(tie.display, tie.bit_gc, 0, dash_pattern[dindex], 2);
      }
      else
         XSetLineAttributes(tie.display, tie.bit_gc, 0, LineSolid,
                     CapButt, JoinBevel);

      pts = TieMalloc(tie_vector->line[i].npts*sizeof(XPoint));
      vx = tie_vector->line[i].pts[0];
      vy = tie_vector->line[i].pts[1];
      mat_transform(tie_vector->tmatrix, &vx, &vy);

      /* check for overflow */
      if (vx < (double)SHRT_MIN)
         pts[0].x = SHRT_MIN;
      else if (vx > (double)SHRT_MAX)
         pts[0].x = SHRT_MAX;
      else
         pts[0].x = (int)(rint(vx));

      if (((double)(tie_vector->view_height)-vy) < (double)SHRT_MIN)
         pts[0].y = SHRT_MIN;
      else if (((double)(tie_vector->view_height)-vy) > (double)SHRT_MAX)
         pts[0].y = SHRT_MAX;
      else
         pts[0].y = tie_vector->view_height - (int)(rint(vy));

      /* for each point of the line */
      for (j = 1; j < tie_vector->line[i].npts; j++)
      {
         vx = tie_vector->line[i].pts[j*2];
         vy = tie_vector->line[i].pts[j*2+1];
         mat_transform(tie_vector->tmatrix, &vx, &vy);

         /* check for overflow */
         if (vx < (double)SHRT_MIN)
            pts[j].x = SHRT_MIN;
         else if (vx > (double)SHRT_MAX)
            pts[j].x = SHRT_MAX;
         else
            pts[j].x = (int)(rint(vx));
         if (((double)(tie_vector->view_height)-vy) < (double)SHRT_MIN)
            pts[j].y = SHRT_MIN;
         else if (((double)(tie_vector->view_height)-vy) > (double)SHRT_MAX)
            pts[j].y = SHRT_MAX;
         else
            pts[j].y = tie_vector->view_height - (int)(rint(vy));
      }

      XDrawLines(tie.display, tie_vector->pixmap[plane], tie.bit_gc, 
                 pts, tie_vector->line[i].npts, CoordModeOrigin);
      free(pts);
   }

   /* force exposure event */
   XClearArea(tie.display, XtWindow(ref_image_bulletinboard), 0,0,0,0, True);
}

/*******************************************************************************
Purpose:
    This routine is the redraws a single plane.  This is required after
    a color or line change or toggle.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void vector_redrawplane
(
    int plane
)
{
   XPoint               *pts;           /* list of points in a line       */
   double               vx,vy;          /* vector coordinates             */
   int                  attrib,         /* attribute index                */
                        cindex,         /* color plane index              */
                        dindex;         /* dash pattern index             */
   GC                   gc;
   int  i, j;

  /*
   *  *** if this plane has not been used, create it
   */
   if (tie_vector->pixmap[plane] == NULL)
      tie_vector->pixmap[plane] = XCreatePixmap(tie.display,
           XtWindow(ref_image_bulletinboard),tie_vector->view_width,
           tie_vector->view_height, BITMAP);

  /*
   *  *** clear the bit plane
   */
   XSetBackground(tie.display,tie.bit_gc,0);
   XSetForeground(tie.display,tie.bit_gc,0);

   XFillRectangle(tie.display,tie_vector->pixmap[plane],
                  tie.bit_gc, 0,0, tie_vector->view_width, 
                  tie_vector->view_height);
   XSetBackground(tie.display,tie.bit_gc,1);
   XSetForeground(tie.display,tie.bit_gc,1);

  /*
   *  *** for each line
   */
   for (i = 0; i < tie_vector->numlines; i++)
      {
      attrib = tie_vector->line[i].attrib;
      cindex = tie_vector->color_att[attrib];
      dindex = tie_vector->line_att[attrib];
  
     /*
      *  *** if not in the given plane, continue
      */
      if ((cindex != plane) || (!tie_vector->enabled[attrib]))
         continue; 

     /*
      *  *** set line dash pattern
      */
      if (dindex > 0)
         {
         XSetLineAttributes(tie.display, tie.bit_gc, 0, LineOnOffDash,
                     CapButt, JoinBevel);
         XSetDashes(tie.display, tie.bit_gc, 0, dash_pattern[dindex], 2);
         }
      else
         XSetLineAttributes(tie.display, tie.bit_gc, 0, LineSolid,
                     CapButt, JoinBevel);

      pts = TieMalloc(tie_vector->line[i].npts*sizeof(XPoint));

      vx = tie_vector->line[i].pts[0];
      vy = tie_vector->line[i].pts[1];
      mat_transform(tie_vector->tmatrix, &vx, &vy);
      pts[0].x = (int)(rint(vx));
      pts[0].y = tie_vector->view_height - (int)(rint(vy));

     /*
      *  *** for each point draw a connecting vector
      */
      for (j = 1; j < tie_vector->line[i].npts; j++)
         {
         vx = tie_vector->line[i].pts[j*2];
         vy = tie_vector->line[i].pts[j*2+1];
         mat_transform(tie_vector->tmatrix, &vx, &vy);
         pts[j].x = (int)(rint(vx));
         pts[j].y = tie_vector->view_height - (int)(rint(vy));
         }

      XDrawLines(tie.display, tie_vector->pixmap[plane], tie.bit_gc, 
                 pts, tie_vector->line[i].npts, CoordModeOrigin);
      free(pts);
      }

  /*
   *  *** copy updated plane to display
   */
   gc = DefaultGC(tie.display, tie.screen);
   XSetClipMask(tie.display, gc, tie_vector->pixmap[plane]);
   XSetForeground(tie.display, gc, COLOR(Shell_Vector, wk_colors[plane]));
   XCopyPlane(tie.display, tie_vector->pixmap[plane],
              XtWindow(ref_image_bulletinboard), gc,
              0, 0, tie_vector->view_width, tie_vector->view_height,
              0, 0, 1);
}


/*******************************************************************************
Purpose:
    This routine assigns the geographic boundaries of the vector data 
    to the user selected areas of interest.  This region is selected
    by pressing and dragging within the search image.

By J. Hemmer, USGS/EROS Data Center, 2/93

Modified by L. Huewe, USGS/EROS Data Center, 4/93
  - Modified to allow selecting a region for DLG data. 

Modified by J. LaVergne, USGS/EROS Data Center, 9/93
  - Fixed problems where the rubber rectangle drew random lines on screen.
    Problem was that negative values were being sent to the XDraw routine.
    Also changed so the rectangle outline stays onscreen until Update is
    clicked (or another rectangle is drawn) so the user can see the area.

Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - added a 0 (fnum) parameter to the image2vect call to process the 
   information for the correct (current) element of the tie_vectfile array. 

Algorithm Description:
*******************************************************************************/
void vector_region
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   double       temp_x,
                temp_y,
                x_coef[6],
                y_coef[6];
   static  
   int          i,
                order,
                plane_idx,
                ses, 
                sel,
                sss, 
                ssl,
                xpress, ypress,
                xdrag,  ydrag;

  /*
   *  *** must check because of bug in DG server
   */
   if (event->xbutton.button != 2)
      return;

   if (event->type == ButtonPress)
      {
/* JLaVergne - try removing the old rubber rectangle from here */ 
      if (((xdrag - xpress) > 0) && ((ydrag - ypress) > 0))
         XDrawRectangle(tie.display, XtWindow(search_image_bulletinboard),
                  tie.rect_gc, xpress, ypress,
                  xdrag - xpress, ydrag - ypress); 
      xpress = xdrag = event->xbutton.x;
      ypress = ydrag = event->xbutton.y;  

     /*
      *  *** remove event handlers for search images
      */
      XtRemoveEventHandler(search_image_bulletinboard,
                     PointerMotionMask|KeyReleaseMask,
                     FALSE,cursor_pointer,NULL);

      XtAddEventHandler(search_image_bulletinboard, PointerMotionMask, 
                        FALSE, vector_region, NULL);
      }
  /*
   *  *** if release event, redefine vector boundaries
   */
   else if (event->type == ButtonRelease)
      {
     /*
      *  *** erase the last rubber rectangle 
      */
/* JLaVergne - try leaving it there for now 
      if (((xdrag - xpress) > 0) && ((ydrag - ypress) > 0))
         XDrawRectangle(tie.display, XtWindow(search_image_bulletinboard),
                  tie.rect_gc, xpress, ypress,
                  xdrag - xpress, ydrag - ypress);                           */
     /*
      *  *** if the rubber rectangle is invalid, quit now
      */
      if (((xdrag - xpress) <= 0) || ((ydrag - ypress) <= 0)) return;  

/* JLaVergne
      display_vectoutline();
      XClearArea(tie.display,XtWindow(search_image_bulletinboard),
                    0,0,0,0, True);                                   */

      plane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
      sss = (((event->xbutton.x + 1) * tie_planes[plane_idx].skips) +
                tie_planes[plane_idx].window[SS] - 1) -
                (tie_planes[plane_idx].skips - 1);
      ssl = (((event->xbutton.y + 1) * tie_planes[plane_idx].skipl) +
                tie_planes[plane_idx].window[SL] - 1) -
                (tie_planes[plane_idx].skipl - 1);

      ses = (((xpress + 1) * tie_planes[plane_idx].skips) +
                tie_planes[plane_idx].window[SS] - 1) -
                (tie_planes[plane_idx].skips - 1);
      sel = (((ypress + 1) * tie_planes[plane_idx].skipl) +
                tie_planes[plane_idx].window[SL] - 1) -
                (tie_planes[plane_idx].skipl - 1);

     /*
      *  *** create polynomial conversion from pixel to image coordinates
      */
      order = tie_parms.order;
      tps_Qfit(tie_select.numtie, tie_select.stie, tie_select.rtie,
               NULL, &order, y_coef, x_coef);

     /*
      *  *** reset vector corners to pixel values
      */
      if (sss < ses)
         {
         tie_vectfile[0].corner[0][0] = sss;
         tie_vectfile[0].corner[1][0] = sss;
         tie_vectfile[0].corner[2][0] = ses;
         tie_vectfile[0].corner[3][0] = ses;
         }
      else
         {
         tie_vectfile[0].corner[0][0] = ses;
         tie_vectfile[0].corner[1][0] = ses;
         tie_vectfile[0].corner[2][0] = sss;
         tie_vectfile[0].corner[3][0] = sss;
         }

     if (ssl > sel)
         {
         tie_vectfile[0].corner[0][1] = sel;
         tie_vectfile[0].corner[1][1] = ssl;
         tie_vectfile[0].corner[2][1] = sel;
         tie_vectfile[0].corner[3][1] = ssl;
         }
      else
         {
         tie_vectfile[0].corner[0][1] = ssl;
         tie_vectfile[0].corner[1][1] = sel;
         tie_vectfile[0].corner[2][1] = ssl;
         tie_vectfile[0].corner[3][1] = sel;
         }

     /*
      *  *** convert corners from pixels to image coordinates
      */
      for (i = 0; i < 4; i++)
         {
         temp_x = tie_vectfile[0].corner[i][0];
         temp_y = tie_vectfile[0].corner[i][1];

         tie_vectfile[0].corner[i][0] = 
                    c_eval(&order, x_coef, &temp_x, &temp_y);
         tie_vectfile[0].corner[i][1] = 
                    c_eval(&order, y_coef, &temp_x, &temp_y);
         }

     /*
      *  *** convert the image coordinates to LAT/LON
      */
      proj2degrees(tie_vectfile[0].corner, 4, SEARCH);

     /*
      *  *** set bounds on the vector form
      */
      vector_setbounds();

     /*
      *  *** add event handlers for search images
      */
      XtRemoveEventHandler(search_image_bulletinboard, PointerMotionMask, 
                           FALSE, vector_region, NULL);
      XtAddEventHandler(search_image_bulletinboard,
                     PointerMotionMask|KeyReleaseMask,
                     FALSE,cursor_pointer,NULL);
      }
   else if (event->type == MotionNotify)
      {
      if ((xdrag != xpress) || (ydrag != ypress))  
         if (((xdrag - xpress) > 0) && ((ydrag - ypress) > 0))
            XDrawRectangle(tie.display, XtWindow(search_image_bulletinboard),
                  tie.rect_gc, xpress, ypress,
                  xdrag - xpress, ydrag - ypress);

      xdrag = event->xmotion.x;
      ydrag = event->xmotion.y;
      if (((xdrag - xpress) > 0) && ((ydrag - ypress) > 0))
         XDrawRectangle(tie.display, XtWindow(search_image_bulletinboard),
                  tie.rect_gc,
                  xpress, ypress,
                  xdrag - xpress, ydrag - ypress);
      }
}

/*******************************************************************************
Purpose:
    This routine scales the vector matrix to fit the reference display.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_scaletofit
(
   double       matrix[3][3]
)
{
   double       sx, sy;
   double       ll[2],          /* point for conversion to image proj */
                ur[2];

   ll[0] = tie_vector->llx;
   ll[1] = tie_vector->lly;
   ur[0] = tie_vector->urx;
   ur[1] = tie_vector->ury;

   mat_identity(matrix);
   mat_translate(matrix, -1.0*ll[0], -1.0*ll[1]);

   sx = ((double)tie_vector->view_width)/(ur[0] - ll[0]);
   sy = ((double)tie_vector->view_height)/(ur[1] - ll[1]);

   /* allow for margin around vectors */
   sx /= 1.1;
   sy /= 1.1;

   /* scale the vectors to fit the display and translate for the margin */
   mat_scale(matrix, sx, sy, 0.0, 0.0);
   mat_translate(matrix, .05*(double)tie_vector->view_width, 
                         .05*(double)tie_vector->view_height);
}

/*******************************************************************************
Purpose:
    This resets the vector panel per a change in vector source.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void vector_setsource(void)
{
   if (Vector_form != NULL)
      {
      if (XtIsRealized(Shell_Vector))
         {
         XtUnmanageChild(Vector_form);
         XtUnrealizeWidget(Shell_Vector);
         XtSetSensitive(main_vector_button, True);
         }

      XtDestroyWidget(Vector_form);

      if (attrib_form != NULL)
         free(attrib_form);

      attrib_form = NULL;
      Vector_form = NULL;
   
      if (tie_vector->numlines > 0)
         {
         vector_free();
         XClearWindow(tie.display, XtWindow(ref_image_bulletinboard));
         if (XtIsRealized(Shell_SrchFull))
            XClearWindow(tie.display, XtWindow(fullsrch_win_bulletinboard));
         }
      }

  /*
   *  *** if image loaded, reset bounds
   */
   if (tie_mem[SEARCH].image_pixmap != NULL)
      vector_bounds();
}

/*******************************************************************************
Purpose:
    This unmarks the current points selected using the alignment method.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void vector_unmark(void)
{
   double       vx, vy;         /* vector coordinates           */
   int          x, y;           /* window coordinates           */

   if (tie_select.rtemp_flag)
      {
      vx = tie_select.rtemp[0];
      vy = tie_select.rtemp[1];
      }
   else
      {
      vx = tie_select.rtie[tie_select.seltie][0];
      vy = tie_select.rtie[tie_select.seltie][1];
      }
   mat_transform(tie_vector->tmatrix, &vx, &vy);
   x = (int)(rint(vx));
   y = tie_vector->view_height - (int)(rint(vy));

   XClearArea(tie.display, XtWindow(ref_image_bulletinboard), 
              x-2, y-2, 6, 6, True);
}

/*******************************************************************************
Purpose:
    This routine is the callback for the Unzoom button.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by T. Rockvam, USGS/EROS Data Center, 9/94
 - Added the "Whole image" unzoom option (unzooms the srch image window
   and corresponding min/max lat/long in the vector panel instead of the 
   vectors in the vector display)

Algorithm Description:
*******************************************************************************/
void vector_unzoom
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
/*
 * Reset the min/max lat/longs to the whole image 
 */
if ((int)client == 1)
   {
   /* if valid ddr use that otherwise use rt */
   if (tie_planes[tie_composite[SEARCH].rgbplanes[RED]-1].proj_valid)
      vector_bounds();
   else
      {
      int i;
      double    minlat,minlon,maxlat,maxlon;
      double x_coef[6],y_coef[6],temp_x,temp_y;

      i = tie_composite[SEARCH].rgbplanes[RED] - 1;
      tie_vectfile[0].corner[0][0] = tie_planes[i].window[1];
      tie_vectfile[0].corner[0][1] = tie_planes[i].window[0];
      tie_vectfile[0].corner[1][0] = tie_planes[i].window[1];
      tie_vectfile[0].corner[1][1] = tie_planes[i].window[0] + 
                                            tie_planes[i].window[2] - 1.0;
      tie_vectfile[0].corner[2][0] = tie_planes[i].window[1] +
                                            tie_planes[i].window[3] - 1.0;
      tie_vectfile[0].corner[2][1] = tie_planes[i].window[0];
      tie_vectfile[0].corner[3][0] = tie_planes[i].window[1] +
                                            tie_planes[i].window[3] - 1.0;
      tie_vectfile[0].corner[3][1] = tie_planes[i].window[0] + 
                                            tie_planes[i].window[2] - 1.0;
      i = tie_parms.order;
      tps_Qfit(tie_select.numtie, tie_select.stie, tie_select.rtie,
                     NULL, &i, y_coef, x_coef);
      for (i = 0; i < 4; i++)
         {
         temp_x = tie_vectfile[0].corner[i][0];
         temp_y = tie_vectfile[0].corner[i][1];
         tie_vectfile[0].corner[i][0] =
               c_eval(&i, x_coef, &temp_x, &temp_y);
         tie_vectfile[0].corner[i][1] =
               c_eval(&i, y_coef, &temp_x, &temp_y);
         }
      proj2degrees(tie_vectfile[0].corner, 4, SEARCH);
      minlon = maxlon = tie_vectfile[0].corner[0][0];
      minlat = maxlat = tie_vectfile[0].corner[0][1];
      for (i = 1; i < 4; i++)
         {
         minlon=MIN(minlon,tie_vectfile[0].corner[i][0]);
         maxlon=MAX(maxlon,tie_vectfile[0].corner[i][0]);
         minlat=MIN(minlat,tie_vectfile[0].corner[i][1]);
         maxlat=MAX(maxlat,tie_vectfile[0].corner[i][1]);
         }
      if (((tie_vectfile[0].corner[0][0] > 0)  || 
           (tie_vectfile[0].corner[1][0] > 0)) &&
          ((tie_vectfile[0].corner[2][0] < 0)  || 
           (tie_vectfile[0].corner[3][0] < 0)))
         {
         if ((tie_vectfile[0].corner[0][0] > 0) &&
             (tie_vectfile[0].corner[1][0] > 0))
            minlon=MIN(tie_vectfile[0].corner[0][0],
                              tie_vectfile[0].corner[1][0]);
         else
            minlon=MAX(tie_vectfile[0].corner[0][0],
                              tie_vectfile[0].corner[1][0]);
         if ((tie_vectfile[0].corner[2][0] < 0) &&
             (tie_vectfile[0].corner[3][0] < 0))
            maxlon=MAX(tie_vectfile[0].corner[2][0],
                      tie_vectfile[0].corner[3][0]);
         else
            maxlon=MIN(tie_vectfile[0].corner[2][0],
                      tie_vectfile[0].corner[3][0]);
         }
      tie_vectfile[0].corner[1][0] = minlon;
      tie_vectfile[0].corner[1][1] = minlat;
      tie_vectfile[0].corner[2][0] = maxlon;
      tie_vectfile[0].corner[2][1] = maxlat;
             vector_setbounds();
      }
   }
else
   {
   vector_scaletofit(tie_vector->tmatrix);
   XClearArea(tie.display, XtWindow(ref_image_bulletinboard), 
              0, 0, 0, 0, False);
   vector_redrawall();
   }
}


/*******************************************************************************
Purpose:
    This routine creates the vector attributes display list for the WDB. 

By J. Hemmer, USGS/EROS Data Center, 6/92
Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - changed OFF to Off and ON to On
Modified by T Rockvam, USGS/EROS Data Center, 3/94.
 - free attribs on first call to clear out previous vector info

Algorithm Description:
*******************************************************************************/
void vector_wdbattribs(void)
{
   Arg          args[512];              /* widget resource arguments    */
   int          argcnt;                 /* widget argument counter      */
   XmString     xmstr;                  /* Motif compound strings       */
   Widget attribute_desc;               /* attribite description widget */
   Widget attribute_color;              /* attribute color widget       */
   Widget attribute_enable;             /* attribute enabled widget     */
   Widget attribute_line;               /* attribute dash style widget  */
   int          i;                      /* loop counter                 */
  
  /*
   *  *** if attribute form not previously created
   */
   if (attrib_form == NULL) 
      {
      vector_freeattrib();

      tie_vector->nattrib = WDBFEATURES;

      tie_vector->line_att  = TieMalloc(tie_vector->nattrib);
      tie_vector->color_att = TieMalloc(tie_vector->nattrib);
      tie_vector->enabled   = TieMalloc(tie_vector->nattrib);

      attrib_form = TieMalloc(tie_vector->nattrib*sizeof(Widget));

      for (i = 0; i < WDBFEATURES; i++)
         {
         tie_vector->line_att[i]  = 0;
         tie_vector->color_att[i] = GREEN_COL;
         tie_vector->enabled[i] = wdb_attrib[i].enabled;
   
         argcnt = 0;
         XtSetArg(args[argcnt], XmNwidth,    360); argcnt++;
         XtSetArg(args[argcnt], XmNheight,    30); argcnt++;
   
         attrib_form[i] = XtCreateWidget("attribute_form",
                      xmFormWidgetClass,
                      vect_attrib_rowcol,
                      args,
                      argcnt);
   
        /*
         *  *** attribute description
         */
         argcnt = 0;
         XtSetArg(args[argcnt], XmNalignment, XmALIGNMENT_BEGINNING); argcnt++;
         XtSetArg(args[argcnt], XmNlabelString, (xmstr=XmStringCreateLtoR(
                      wdb_attrib[i].desc,
                      XmSTRING_DEFAULT_CHARSET))); argcnt++;
         XtSetArg(args[argcnt], XmNrecomputeSize,    False); argcnt++;
         XtSetArg(args[argcnt], XmNtopAttachment,    XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNleftAttachment,   XmATTACH_FORM); argcnt++;
         attribute_desc = XtCreateWidget("desc",
                      xmLabelWidgetClass,
                      attrib_form[i],
                      args,
                      argcnt);
         XmStringFree( xmstr );
         XtManageChild(attribute_desc);
   
        /*
         *  *** attribute color 
         */
         argcnt = 0;
         XtSetArg(args[argcnt], XmNlabelString, (xmstr=XmStringCreateLtoR(
                      "Color",
                      XmSTRING_DEFAULT_CHARSET))); argcnt++;
         XtSetArg(args[argcnt], XmNforeground, 
                  COLOR(Shell_Vector, wk_colors[GREEN_COL])); argcnt++;
         XtSetArg(args[argcnt], XmNrecomputeSize,    False); argcnt++;
         XtSetArg(args[argcnt], XmNtopAttachment,    XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNrightAttachment,XmATTACH_POSITION); argcnt++;
         XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_POSITION); argcnt++;
         XtSetArg(args[argcnt], XmNleftPosition,     (int)50); argcnt++;
         XtSetArg(args[argcnt], XmNrightPosition,    (int)70); argcnt++;
         attribute_color = XtCreateWidget("attribute_color",
                      xmPushButtonWidgetClass,
                      attrib_form[i],
                      args,
                      argcnt);
      
         XmStringFree( xmstr );
         XtAddCallback(attribute_color, XmNactivateCallback,
                    vector_attribute_color, (caddr_t)i);
         XtManageChild(attribute_color);
   
        /*
         *  *** attribute line style
         */
         argcnt = 0;
         XtSetArg(args[argcnt], XmNrecomputeSize,    False); argcnt++;
         XtSetArg(args[argcnt], XmNtopAttachment,    XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNrightAttachment,XmATTACH_POSITION); argcnt++;
         XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_POSITION); argcnt++;
         XtSetArg(args[argcnt], XmNleftPosition,     (int)70); argcnt++;
         XtSetArg(args[argcnt], XmNrightPosition,    (int)90); argcnt++;
         XtSetArg(args[argcnt], XmNpushButtonEnabled,True); argcnt++;
         attribute_line = XtCreateWidget("attribute_line",
                      xmDrawnButtonWidgetClass,
                      attrib_form[i],
                      args,
                      argcnt);
   
         XtAddCallback(attribute_line, XmNactivateCallback,
                    vector_attribute_line, (caddr_t)i);
         XtAddCallback(attribute_line, XmNexposeCallback,
                    vector_line_expose, (caddr_t)dash_pattern[0]);
         XtManageChild(attribute_line);
   
        /*
         *  *** attribute toggle
         */
         argcnt = 0;
         if (tie_vector->enabled[i])
            XtSetArg(args[argcnt], XmNlabelString, (xmstr=XmStringCreateLtoR(
                      "On",
                      XmSTRING_DEFAULT_CHARSET))); 
         else
            XtSetArg(args[argcnt], XmNlabelString, (xmstr=XmStringCreateLtoR(
                      "Off",
                      XmSTRING_DEFAULT_CHARSET))); 
         argcnt++;
         XtSetArg(args[argcnt], XmNrecomputeSize,    False); argcnt++;
         XtSetArg(args[argcnt], XmNtopAttachment,    XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNbottomAttachment, XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNrightAttachment,  XmATTACH_FORM); argcnt++;
         XtSetArg(args[argcnt], XmNleftAttachment, XmATTACH_POSITION); argcnt++;
         XtSetArg(args[argcnt], XmNleftPosition,     (int)90); argcnt++;
         XtSetArg(args[argcnt], XmNpushButtonEnabled,True); argcnt++;
         attribute_enable = XtCreateWidget("attribute_line",
                      xmPushButtonWidgetClass,
                      attrib_form[i],
                      args,
                      argcnt);
   
         XmStringFree(xmstr);
         XtAddCallback(attribute_enable, XmNactivateCallback,
                    vector_attribute_enable, (caddr_t)i);
         XtManageChild(attribute_enable);

         XtManageChild(attrib_form[i]);
         }
      }
}


/*******************************************************************************
Purpose:
    This routine updates the min/max values (minimum bounding rectangle) 
    for clipping the vector data.

By J. Hemmer, USGS/EROS Data Center, 2/92

Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - added a test for scenes that cross the International Date Line 
 - added a 0 (fnum) parameter to the  image2vect call to process the 
   information for the correct (current) element of the tie_vectfile array. 
Modified by T. Rockvam, USGS/EROS Data Center, 12/93
 - Modified to calculate the corners of the window loaded by the user

Algorithm Description:
*******************************************************************************/
int vector_bounds(void)
{
   int         i,              /* loop counter                         */
                plane_idx;      /* search memory plane index            */

   double       minlat, minlon,
                maxlat, maxlon;

   plane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;

   if (!tie_planes[plane_idx].proj_valid)
      return(E_FAIL);

   tie_vectfile[0].corner[0][0] = tie_planes[plane_idx].ddr.upleft[1] +
      (tie_planes[plane_idx].window[1]-1.0)*tie_planes[plane_idx].ddr.pdist_x;
   tie_vectfile[0].corner[0][1] = tie_planes[plane_idx].ddr.upleft[0] -
      (tie_planes[plane_idx].window[0]-1.0)*tie_planes[plane_idx].ddr.pdist_y;
   tie_vectfile[0].corner[1][0] = tie_planes[plane_idx].ddr.loleft[1] +
      (tie_planes[plane_idx].window[1]-1.0)*tie_planes[plane_idx].ddr.pdist_x;
   tie_vectfile[0].corner[1][1] = tie_planes[plane_idx].ddr.loleft[0] +
      (tie_planes[plane_idx].ddr.nl -
      (tie_planes[plane_idx].window[0]+tie_planes[plane_idx].window[2]-1.0)) *
      tie_planes[plane_idx].ddr.pdist_y;
   tie_vectfile[0].corner[2][0] = tie_planes[plane_idx].ddr.upright[1] -
      (tie_planes[plane_idx].ddr.ns -
      (tie_planes[plane_idx].window[1]+tie_planes[plane_idx].window[3]-1.0)) *
      tie_planes[plane_idx].ddr.pdist_x;
   tie_vectfile[0].corner[2][1] = tie_planes[plane_idx].ddr.upright[0] -
      (tie_planes[plane_idx].window[0]-1.0)*tie_planes[plane_idx].ddr.pdist_y;
   tie_vectfile[0].corner[3][0] = tie_planes[plane_idx].ddr.loright[1] -
      (tie_planes[plane_idx].ddr.ns -
      (tie_planes[plane_idx].window[1]+tie_planes[plane_idx].window[3]-1.0)) *
      tie_planes[plane_idx].ddr.pdist_x;
   tie_vectfile[0].corner[3][1] = tie_planes[plane_idx].ddr.loright[0] +
      (tie_planes[plane_idx].ddr.nl -
      (tie_planes[plane_idx].window[0]+tie_planes[plane_idx].window[2]-1.0)) *
      tie_planes[plane_idx].ddr.pdist_y;

   /* convert to DDR corners to LAT/LON */
   proj2degrees(tie_vectfile[0].corner, 4, SEARCH);

   /* make sure corners are absolute bounds */
   minlon = maxlon = tie_vectfile[0].corner[0][0];
   minlat = maxlat = tie_vectfile[0].corner[0][1];
   for (i = 0; i < 4; i++)
   {
      if (minlon > tie_vectfile[0].corner[i][0])
         minlon = tie_vectfile[0].corner[i][0];
      if (maxlon < tie_vectfile[0].corner[i][0])
         maxlon = tie_vectfile[0].corner[i][0];

      if (minlat > tie_vectfile[0].corner[i][1])
         minlat = tie_vectfile[0].corner[i][1];
      if (maxlat < tie_vectfile[0].corner[i][1])
         maxlat = tie_vectfile[0].corner[i][1];
   }

   /* Test for scenes that cross the International Date Line (180): 
      1) if either the upper left or lower left lon is positive, and
         either the upper right or lower right lon is negative, then the IDL is
         somewhere in the image.
      If the IDL was found, calculate min and max lon values:
      2) for the min lon: if both of the leftmost lons are positive, 
         pick the minimum. otherwise use the positive one.
      3) for the max lon: if both of the rightmost lons are negative, 
                  pick the maximum, otherwise use the negative one.
   */
   if (((tie_vectfile[0].corner[0][0] > 0)  || 
        (tie_vectfile[0].corner[1][0] > 0)) &&
       ((tie_vectfile[0].corner[2][0] < 0)  || 
        (tie_vectfile[0].corner[3][0] < 0)))
   {
      if ((tie_vectfile[0].corner[0][0] > 0) &&
          (tie_vectfile[0].corner[1][0] > 0))
      {
         if (tie_vectfile[0].corner[0][0] < tie_vectfile[0].corner[1][0])
            minlon = tie_vectfile[0].corner[0][0];
         else
            minlon = tie_vectfile[0].corner[1][0];
      }
      else
      {
         if (tie_vectfile[0].corner[0][0] > 0)
            minlon = tie_vectfile[0].corner[0][0];
         else
            minlon = tie_vectfile[0].corner[1][0];
      }
      if ((tie_vectfile[0].corner[2][0] < 0) &&
          (tie_vectfile[0].corner[3][0] < 0))
      {
         if (tie_vectfile[0].corner[2][0] > tie_vectfile[0].corner[3][0])
            maxlon = tie_vectfile[0].corner[2][0];
         else
            maxlon = tie_vectfile[0].corner[3][0];
      }
      else
      {
         if (tie_vectfile[0].corner[2][0] < 0)
            maxlon = tie_vectfile[0].corner[2][0];
         else
            maxlon = tie_vectfile[0].corner[3][0];
      }
   }

   /* redefine lower left and upper right corners */
   tie_vectfile[0].corner[1][0] = minlon;
   tie_vectfile[0].corner[1][1] = minlat;
   tie_vectfile[0].corner[2][0] = maxlon;
   tie_vectfile[0].corner[2][1] = maxlat;

   /* set bounds on vector panel */
   if (XtIsRealized(Shell_Vector))
      vector_setbounds();

   return(E_SUCC);
}

/*******************************************************************************
Purpose:
   Assign the min and max bounds of the vector panel to the currently
   defined boundaries. Ie., write the minimum bounding rectangle values
   into the Min/Max, Lat/Lon text fields in the Vector panel.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - added checks to correct an error in find_deg, where the sign is lost
   if the degree sent to the routine is zero.
 
Algorithm Description:
*******************************************************************************/
void vector_setbounds(void)
{
   char         dms[20];

   if (Vector_form != NULL)
   {
       if ((tie_vectfile[0].corner[1][1] <  0.0) &&
           (tie_vectfile[0].corner[1][1] > -0.999998611))
           sprintf(dms,"-0 %2d %4.2f", find_min(tie_vectfile[0].corner[1][1]),
                                       find_sec(tie_vectfile[0].corner[1][1]));
       else 
         sprintf(dms, "%3d %2d %4.2f", find_deg(tie_vectfile[0].corner[1][1]),
                                       find_min(tie_vectfile[0].corner[1][1]),
                                       find_sec(tie_vectfile[0].corner[1][1]));
      XmTextSetString(vect_min_lat, dms);


      if ((tie_vectfile[0].corner[1][0] <  0.0) &&
          (tie_vectfile[0].corner[1][0] > -0.999998611))
           sprintf(dms,"-0 %2d %4.2f", find_min(tie_vectfile[0].corner[1][0]),
                                       find_sec(tie_vectfile[0].corner[1][0]));
      else
         sprintf(dms, "%3d %2d %4.2f", find_deg(tie_vectfile[0].corner[1][0]),
                                       find_min(tie_vectfile[0].corner[1][0]),
                                       find_sec(tie_vectfile[0].corner[1][0]));
      XmTextSetString(vect_min_lon, dms);


      if ((tie_vectfile[0].corner[2][1] <  0.0) &&
          (tie_vectfile[0].corner[2][1] > -0.999998611))
           sprintf(dms,"-0 %2d %4.2f", find_min(tie_vectfile[0].corner[2][1]),
                                       find_sec(tie_vectfile[0].corner[2][1]));
      else
         sprintf(dms, "%3d %2d %4.2f", find_deg(tie_vectfile[0].corner[2][1]),
                                       find_min(tie_vectfile[0].corner[2][1]),
                                       find_sec(tie_vectfile[0].corner[2][1]));
      XmTextSetString(vect_max_lat, dms);


      if ((tie_vectfile[0].corner[2][0] <  0.0) &&
          (tie_vectfile[0].corner[2][0] > -0.999998611))
           sprintf(dms,"-0 %2d %4.2f", find_min(tie_vectfile[0].corner[2][0]),
                                       find_sec(tie_vectfile[0].corner[2][0]));
      else
         sprintf(dms, "%3d %2d %4.2f", find_deg(tie_vectfile[0].corner[2][0]),
                                       find_min(tie_vectfile[0].corner[2][0]),
                                       find_sec(tie_vectfile[0].corner[2][0]));
      XmTextSetString(vect_max_lon, dms);
   }
}

/*******************************************************************************
Purpose:
   Skip to next text item.  

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_tabover
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XmProcessTraversal(Vector_form, XmTRAVERSE_NEXT_TAB_GROUP);
}

/*******************************************************************************
Purpose:
    This routine handles the selection of a WDB continent selection for
    loading.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by T Rockvam, USGS/EROS Data Center, 3/94.
  - fixed to used the correct wdb files

Algorithm Description:
*******************************************************************************/
void vector_wdbload
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   char         *filename;      /* selected filename                    */

   XmFileSelectionBoxCallbackStruct *acs =
             (XmFileSelectionBoxCallbackStruct*)call_data;

  /*
   *  Retrieve the file name selected and parse it.  This determines its LAS
   *  name, host file name, window specified and bands specified.
   */
   if (client)
      {                                         
      XmStringGetLtoR(acs->value,XmSTRING_DEFAULT_CHARSET,&filename);
      XmTextSetString(wdb_cont_text, filename);
      XtFree(filename);                 
      }                         

   XtUnmanageChild(Vect_fileselector);
   XtDestroyWidget(Vect_fileselector);

   XtSetSensitive(wdb_cont_button, True);
}

/*******************************************************************************
Purpose:
    This routine handles updates to the WDB display.

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - added a 0 (fnum) parameter to the vect2image call to process the 
   information for the correct (current) element of the tie_vectfile array. 

Algorithm Description:
*******************************************************************************/
void vector_wdbupdate
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   FILE         *fseq, *frel;   /* file descriptors to seq and rel files   */
   char         *continent;     /* selected continent                      */
   char         filename[100];  /* name of continent file                  */
   float        maxlat, maxlon, /* bounding coordinates for loading        */
                minlat, minlon,
                myminlon, mymaxlon,
                *pts;           /* temp array of points for loading        */
   int         cindex,          /* continent index                         */
                crossflg=FALSE, /* crossing IDL flag                       */
                updated,        /* boolean flag denoting update to display */
                ranks[20],      /* flags denoting depth of loading         */
                npts,           /* number of points loaded at one time     */
                pt,             /* total points of a segment loaded        */
                totpts,         /* number of points in a segment           */
                i, j;           /* loop counters                           */
   double       temppt[2];      /* temp array for point conversion         */


   updated = FALSE;

  /*
   *  *** if no vectors loaded, then update display
   */
   if (tie_vector->numfiles == 0)
      updated = TRUE;

   continent = XmTextGetString(wdb_cont_text);

   if (strcmp(continent, "") == 0)
      {
      tie_message(Shell_Vector, "Must select a continent");
      return;
      }


  /*
   *  *** find index of selected continent
   */
   i = 0;
   while (wdb_files[i].continent != NULL)
      {
      if (strcasecmp(continent, wdb_files[i].continent) == 0)
         break;

      i++;
      }

   XtFree(continent);

   
  /*
   *  *** if unknown continent, exit
   */
   if (wdb_files[i].continent == NULL)
      {
      tie_message(Shell_Vector, "Unknown continent");
      return;
      }
   else
      cindex = i;
  
  /*
   *  *** if continent file is different, update display
   */
   if ((!updated) && 
       (strcmp(tie_vectfile[0].filename, wdb_files[cindex].continent) != 0))
      updated = TRUE;

   if ((strcmp(XmTextGetString(vect_max_lat), "") == 0) ||
       (strcmp(XmTextGetString(vect_max_lon), "") == 0) ||
       (strcmp(XmTextGetString(vect_min_lat), "") == 0) ||
       (strcmp(XmTextGetString(vect_min_lon), "") == 0))
      {
      tie_message(Shell_Vector, "Must define Lat/Lon boundaries");
      return;
      }

   minlat = (float)map_dms2deg(XmTextGetString(vect_min_lat));
   minlon = (float)map_dms2deg(XmTextGetString(vect_min_lon));
   maxlat = (float)map_dms2deg(XmTextGetString(vect_max_lat));
   maxlon = (float)map_dms2deg(XmTextGetString(vect_max_lon));

   if (maxlon < minlon) 
      {
      mymaxlon = minlon;
      myminlon = maxlon;
      crossflg = TRUE;
      }
   else
      {
      myminlon = minlon;
      mymaxlon = maxlon;
      }

  /*
   *  *** if boundary changed, update display
   */
   if ((!updated) &&
       ((minlon != tie_vectfile[tie_vector->numfiles-1].corner[1][0]) ||
        (minlat != tie_vectfile[tie_vector->numfiles-1].corner[1][1]) ||
        (maxlon != tie_vectfile[tie_vector->numfiles-1].corner[2][0]) ||
        (maxlat != tie_vectfile[tie_vector->numfiles-1].corner[2][1])))
      updated = TRUE;

  /*
   *   *** if attribute enabled that is not loaded, update display
   */
   if (!updated)
      {
      for (i = 0; i < WDBFEATURES; i++)
         if (tie_vector->enabled[i] && (!wdb_attrib[i].loaded))
            {
            updated = TRUE;
            break;
            }
      }

  /*
   *  *** if no changes, do not update display
   */
   if (!updated)
      return;

  /*
   *  *** if changes were made, free current vectors
   */
   if (tie_vector->numlines > 0)
      vector_free();

  /*
   *  *** load each requested attribute 
   */
   for (i = 0; i < WDBFEATURES; i++)
      {
      if (tie_vector->enabled[i])
         {
         c_wdbrank(i+1,0,0,ranks); /* select all features */

         wdb_attrib[i].loaded = TRUE;

         if (wdb_files[cindex].filename[i] == NULL)
            filename[0] = NULL;
         else
            sprintf(filename, "%s", wdb_files[cindex].filename[i]);

         if ((filename[0] != NULL) && 
             (c_wdbopen(filename, &fseq, &frel) != E_FAIL))
            {
            while (c_wdbseq(fseq, ranks, crossflg, &minlat, &myminlon, 
                            &maxlat, &mymaxlon, &totpts, frel))
               {
               tie_vector->line[tie_vector->numlines].pts =
                             TieMalloc(totpts*2*sizeof(double));

               tie_vector->line[tie_vector->numlines].npts = totpts;
               tie_vector->line[tie_vector->numlines].attrib = i;
               pts = TieMalloc(100*sizeof(float));

               npts = totpts;
               pt = 0;

               while (totpts > 0)
                  {
                  c_wdbrel(frel, &totpts, &npts, pts);
                  for (j = 0; j < npts; j++)
                     {
                     tie_vector->line[tie_vector->numlines].pts[2*pt] 
                           = pts[2*j+1];
                     tie_vector->line[tie_vector->numlines].pts[2*pt+1] 
                           = pts[2*j];
                     pt++;
                     }
                  }

               free(pts);
               tie_vector->numlines++;
   
              /*
               *  *** if maximum allocated, allocate new block
               */
               if (tie_vector->numlines == tie_vector->maxline)
                  {
                  tie_vector->maxline += LINEBLOCK;
                  tie_vector->line = TieRealloc(tie_vector->line,
                                             tie_vector->maxline*
                                             sizeof(struct TIE_LINE));
                  }

              /*
               *  *** convert points into image projection coordinats
               */
               vect2image(tie_vector->line[tie_vector->numlines-1].pts,
                          tie_vector->line[tie_vector->numlines-1].npts,0);

               }
            c_wdbclose(fseq, frel);
            }
         }
      }

  /*
   *  *** fit on screen
   */
   if (tie_vector->numlines > 0)
      {
      if (tie_vectfile[0].filename) 
         free(tie_vectfile[0].filename);
      tie_vector->numfiles = 1;
      tie_vectfile[0].filename = 
            TieMalloc(strlen(wdb_files[cindex].continent)+1);
      strcpy(tie_vectfile[0].filename, wdb_files[cindex].continent);

     /*
      *  *** reset bounding corner to image proj coordinates
      */
      temppt[0] = tie_vectfile[0].corner[1][0] = minlon;
      temppt[1] = tie_vectfile[0].corner[1][1] = minlat;
      vect2image(temppt, 1, 0);
      tie_vector->llx = temppt[0];
      tie_vector->lly = temppt[1];

      temppt[0] = tie_vectfile[0].corner[2][0] = maxlon;
      temppt[1] = tie_vectfile[0].corner[2][1] = maxlat;
      vect2image(temppt, 1, 0);
      tie_vector->urx = temppt[0];
      tie_vector->ury = temppt[1];

      vector_scaletofit(tie_vector->tmatrix);
      XClearArea(tie.display, XtWindow(ref_image_bulletinboard),
              0, 0, 0, 0, False);
      vector_redrawall();
      }
   else
      tie_message(Shell_Vector, "No lines found");
}

/*******************************************************************************
Purpose:
    This routine handles exposure event to the zoom window display.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_zexpose
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   XExposeEvent   *xe;  /* expose event pointer                              */

   xe = &event->xexpose;

  /*
   *  *** if transformation is calculated and no pending exposures, redraw
   */
   if ((tie_rt.calc_rt == RT_SEL) || (xe->count != 0))
      return;

   vector_overlay(zoom_srchwin_bulletinboard);
}

/*******************************************************************************
Purpose:
    This routine zooms the vector display per the selected region.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_zoom(void)
{
   int          llx, lly,       /* lower left coordinate of  region     */
                urx, ury,       /* upper right coordinate of region     */
                zh,             /* height of region                     */
                zw;             /* width of region                      */
   double       sx,             /* scaling factor in x direction        */
                sy;             /* scaling factor in y direction        */

  /*
   *  *** determine size of region
   */
   zw = abs(tie_vector->pressx1 - tie_vector->pressx2);
   zh = abs(tie_vector->pressy1 - tie_vector->pressy2);

  /*
   *  *** if too small, continue
   */
   if ((zw < 10) && (zh < 10))
      return;

  /*
   *  *** determine lower left and upper right corners
   */
   if (tie_vector->pressx1 < tie_vector->pressx2)
      {
      llx = tie_vector->pressx1;
      urx = tie_vector->pressx2;
      }
   else
      {
      llx = tie_vector->pressx2;
      urx = tie_vector->pressx1;
      }

   if (tie_vector->pressy1 < tie_vector->pressy2)
      {
      lly = tie_vector->pressy1;
      ury = tie_vector->pressy2;
      }
   else
      {
      lly = tie_vector->pressy2;
      ury = tie_vector->pressy1;
      }

  /*
   *   *** compute scaling factor to fit window and apply to trans matrix
   */
   sx = ((double)tie_vector->view_width)/((double)(urx - llx));
   sy = ((double)tie_vector->view_height)/((double)(ury - lly));

   mat_translate(tie_vector->tmatrix, (double)-llx, 
                 (double)ury - tie_vector->view_height);

  /*
   *  *** zoom by greater amount so don't loss aspect ratio
   */
   mat_scale(tie_vector->tmatrix, sx, sy, 0.0, 0.0);
   XClearArea(tie.display, XtWindow(ref_image_bulletinboard),
              0, 0, 0, 0, False);
   vector_redrawall();
}
