#include <math.h>
#include <sys/signal.h>
#include "tiepts.h"
#include "proj.h"
#include "dcw.h"
#include <X11/cursorfont.h>
#include "convert.h"
#include "cr.h"
#include "dlgsup.h"
#include "mapsup.h"
#include "tpssup.h"
#include "manage_cursor.h"
#include "protos.h"

/* tjr - temporary global because of division of vector.c and vector_tmp.c */
/* static Widget        *attrib_form = NULL; */
extern Widget   *attrib_form;

#define BITMAP 0x1

/*******************************************************************************
Purpose:
    This routine removes all dynamically allocated structures associated
    with the attributes panel.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - n 
Algorithm Description:
*******************************************************************************/
void vector_freeattrib(void)
{
   int   i;             /* loop counter                           */

  /*
   *  *** if attributes have been displayed, remove the individual forms
   */
   if (attrib_form != NULL)
      {
      for (i = 0; i < tie_vector->nattrib; i++)
/* JgL - force removal of previous item (next line & brackets) */
         {
         XtUnmanageChild(attrib_form[i]);
         XtDestroyWidget(attrib_form[i]);
         }

      free(attrib_form);
      attrib_form = NULL;
      }
 
  /*
   *  *** free line, color, and enabled lists
   */
   if (tie_vector->nattrib > 0)
      {
      free(tie_vector->line_att);
      free(tie_vector->color_att);
      free(tie_vector->enabled);
      tie_vector->nattrib = 0;
      }
}

/*******************************************************************************
Purpose:
    This routine displays the requested attributes.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by T Rockvam, USGS/EROS Data Center, 3/94.
   - added code to save previous color & line style of DCW attribs
 
Algorithm Description:
*******************************************************************************/
void vector_dcwattrib(void)
{
   Arg          args[10];               /* widget resource arguments    */
   int          argcnt;                 /* widget argument counter      */
   int          i, j;                   /* loop counter                 */
   int          attribnum;              /* attribute counter            */
 
unsigned char *old_line_att;
unsigned char *old_color_att;

old_line_att  = TieMalloc(tie_vector->nattrib);
old_color_att = TieMalloc(tie_vector->nattrib);

for (i = 0; i < tie_vector->nattrib; i++)
   {
   old_line_att[i]  = tie_vector->line_att[i];
   old_color_att[i] = tie_vector->color_att[i];
   }

  /*
   *  *** remove old form and start fresh
   */
   vector_freeattrib();

  /*
   *  *** allocated required number of forms
   */
   i=0;
   while(dcw_coverage[i].desc != NULL)
      {
      for (j = 0; j < dcw_coverage[i].numvdt; j++)
         {
         if (dcw_coverage[i].vdt[j].selected)
            tie_vector->nattrib++;
         }
      i++;
      }

   attrib_form = TieMalloc(tie_vector->nattrib*sizeof(Widget));
   tie_vector->line_att  = TieMalloc(tie_vector->nattrib);
   tie_vector->color_att = TieMalloc(tie_vector->nattrib);
   tie_vector->enabled   = TieMalloc(tie_vector->nattrib);

   attribnum = 0;

   i=0;
   while(dcw_coverage[i].desc != NULL)
      {
      if (dcw_coverage[i].numvdt == 0)
         {
         i++;
         continue;
         }

      for (j = 0; j < dcw_coverage[i].numvdt; j++)
         {
        /*
         *  *** if this attribute is not requested, continue
         */
         if (!dcw_coverage[i].vdt[j].selected)
            continue;

        /* 
         *  *** if the attrib was set before save the linestyle & color 
         */
         if (dcw_coverage[i].vdt[j].attindex != -1)             
            {                                           
            tie_vector->line_att[attribnum]  =  
                old_line_att[dcw_coverage[i].vdt[j].attindex];
            tie_vector->color_att[attribnum] =          
                old_color_att[dcw_coverage[i].vdt[j].attindex];
            }                                           
         else   
            {
            tie_vector->line_att[attribnum]  = 0;
            tie_vector->color_att[attribnum] = GREEN_COL;
            }
         tie_vector->enabled[attribnum]   = TRUE;
         
         dcw_coverage[i].vdt[j].attindex = attribnum;

         argcnt = 0;
         XtSetArg(args[argcnt], XmNwidth,    460); argcnt++;
         XtSetArg(args[argcnt], XmNheight,    30); argcnt++;
   
         attrib_form[attribnum] = XtCreateWidget("attribute_form",
                      xmFormWidgetClass,
                      vect_attrib_rowcol,
                      args,
                      argcnt);
   
         Createattrib_form(attrib_form[attribnum], 
                           dcw_coverage[i].vdt[j].desc, attribnum, 
                           (int)tie_vector->color_att[attribnum],
                           (int)tie_vector->line_att[attribnum]);
         XtManageChild(attrib_form[attribnum++]);
         }

      i++;
      }

   free(old_line_att);
   free(old_color_att);
}

/*******************************************************************************
Purpose:
    This routine sets the dcw library of interest.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void vector_dcwlib
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XmString     xmstr;          /* temporary Xm string variable               */
   int          i, j;           /* loop counters */
  /*
   *  *** if library changed, unset all loaded flags
   */
   if ((int)client != tie_vector->dcwlib)
      {
      tie_vector->dcwlib = (int)client;
      XtVaSetValues(dcw_library_label,XmNlabelString,
              (xmstr=XmStringCreateLtoR(dcw_cd[(int)client].label, 
              XmSTRING_DEFAULT_CHARSET)),NULL);

      i=0;
      while(dcw_coverage[i].desc != NULL)
         {
         for (j = 0; j < dcw_coverage[i].numvdt; j++)
            dcw_coverage[i].vdt[j].loaded = FALSE;
         i++;
         }
      }
}

/*******************************************************************************
Purpose:
    This routine displays the status (ie., highlighted or not) of each 
    line type available for a given feature.  (DCW Line Types panel)
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void vector_dcwlinestatus
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   XmString     xmstr;          /* temporary Xm string variable               */
   int          i;

   if ((!dcw_coverage[(int)client].getstatus) ||
       (dcw_coverage[(int)client].numstatus == 0))
      return;

   Dcwls_form = Createdcwls_form(Shell_DCWls, (int)client);
   XmListDeleteAllItems(dcw_ls_list);

  /*
   *  *** display each of the line type in a scrollable list
   */
   for (i = 0; i < dcw_coverage[(int)client].numstatus; i++)
      {
      xmstr = XmStringCreateLtoR(dcw_coverage[(int)client].status[i].desc,
                                 XmSTRING_DEFAULT_CHARSET);
      XmListAddItem(dcw_ls_list, xmstr, 0);
      XmStringFree(xmstr);
      }

   XtManageChild(Dcwls_form);
   XtRealizeWidget(Shell_DCWls);

  /*
   *  *** highlight any previously selected line status types
   */
   XmListDeselectAllItems(dcw_ls_list);
 
   for (i = 0; i < dcw_coverage[(int)client].numstatus; i++)
      {
      if (dcw_coverage[(int)client].status[i].selected)
         XmListSelectPos(dcw_ls_list, i+1, True);
      }
}

/*******************************************************************************
Purpose:
    This routine displays the line types available for a given Feature.  Ie.,
    list the individual options within a particular feature.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void vector_dcwlinetype
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i;              /* loop counter                               */
   XmString     xmstr;          /* temporary Xm string variable               */


  /*
   *  *** make sure a CD is found
   */
   if ((dcw_getcd(tie_parms.cd, &tie_vector->dcwlib, dcw_cd)) == E_FAIL)
      {
      tie_message(Shell_Vector, "Cannot read CD\n");
      return;
      }

  /*
   *  *** if vdtlist is undefined, get line type from VDT file
   */
   if (dcw_coverage[(int)client].numvdt == 0)
      { 
      dcw_definecover(tie_parms.cd, dcw_cd[tie_vector->dcwlib].lib, 
                       &dcw_coverage[(int)client]);
      for (i = 0; i < dcw_coverage[(int)client].numvdt; i++)
         dcw_coverage[(int)client].vdt[i].attindex = -1; 
      }

  /*
   *  *** create line type form
   */
   Dcwlt_form = Createdcwlt_form(Shell_DCWlt, (int)client);
   XmListDeleteAllItems(dcw_lt_list);

  /*
   *  *** display each of the line type in a scrollable list
   */
   for (i = 0; i < dcw_coverage[(int)client].numvdt; i++)
      {
      xmstr = XmStringCreateLtoR(dcw_coverage[(int)client].vdt[i].desc,
                                 XmSTRING_DEFAULT_CHARSET);
      XmListAddItem(dcw_lt_list, xmstr, 0);
      XmStringFree(xmstr);
      }

   XtManageChild(Dcwlt_form);
   XtRealizeWidget(Shell_DCWlt);

  /*
   *  *** highlight any previously selected line types
   */
   XmListDeselectAllItems(dcw_lt_list);

   for (i = 0; i < dcw_coverage[(int)client].numvdt; i++)
      {
      if (dcw_coverage[(int)client].vdt[i].selected)
         XmListSelectPos(dcw_lt_list, i+1, True);
      }

   XtSetSensitive(dcw_feature_pull, False);
}

/*******************************************************************************
Purpose:
    This routine displays all the line types available for DLG data.
 
By T. Rockvam, USGS/EROS Data Center, 2/94
 
Algorithm Description:
*******************************************************************************/
void vector_dlglinetype
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   int          i;              /* loop counter                               */
   XmString     xmstr;          /* temporary Xm string variable               */

  /*
   *  *** create line type form
   */
   Dlglt_form = Createdlglt_form(Shell_DLGlt);
   XmListDeleteAllItems(dlg_lt_list);

  /*
   *  *** display each of the line type in a scrollable list
   */
   i = 0;
   while (dlg_attrib[i].mincode != 0)
      {
      xmstr = XmStringCreateLtoR(dlg_attrib[i].desc, XmSTRING_DEFAULT_CHARSET);
      XmListAddItem(dlg_lt_list, xmstr, 0);
      XmStringFree(xmstr);
      i++;
      }

   XtManageChild(Dlglt_form);
   XtRealizeWidget(Shell_DLGlt);

  /*
   *  *** highlight any previously selected line types
   */
   XmListDeselectAllItems(dlg_lt_list);

   i = 0;
   while (dlg_attrib[i].mincode != 0)
      {
      if (dlg_attrib[i].present)
         XmListSelectPos(dlg_lt_list, i+1, True);
      i++;
      }

   XtSetSensitive(dlg_feature_button, False);
}

/*******************************************************************************
Purpose:
    This routine handles updates to the DCW display.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Modified by J. LaVergne, USGS/EROS Data Center, 10/93
 - Modified to check that a library name has been entered
 - added a 0 (fnum) parameter to the vect2image call to process the 
   information for the correct (current) element of the tie_vectfile array. 

Algorithm Description:
*******************************************************************************/
void vector_dcwupdate
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[2];                /* resource arguments               */
   int          i, j, k, l, r, c;
   int          *active,                /* flag denoting something requested
                                           for this coverage                */
                anything;               /* flag denoting anything requested  */
   int          numline,                /* number of requested lines found  */
                numtile,                /* number of tiles in AFT           */
                numlft,                 /* number of lft records            */
                recnum,                 /* record number in LFT             */
                numconvert,             /* number of lines converted into 
                                           image projection coordinates     */
                tileid,                 /* id of tile of interest           */
                updated,                /* flag denoting query was updated  */
                *line,                  /* list of selected line id's       */
                *type;                  /* list of selected line types      */
   double       maxlat, maxlon,         /* bounding coordinates for loading */
                minlat, minlon, tmp_latlon;
   char         minlontile, maxlontile,
                minlattile, maxlattile, /* 15 degree tile boundaries        */
                startcol, startrow,     /* starting 5 degree row/column     */
                endcol, endrow,         /* ending 5 degree row/column       */
                tmp_latlontile, tmp_rowcol;   /* temporary boundary values  */
   char         tilename[7],            /* pathname of tile of interest     */
                title[100];             /* Title for processing message window*/

   double       temppt[2];              /* temp array for point conversion  */

  /*
   *  *** FOR NOW, reset each time update is requested
   */
   tie_vector->abort = TRUE;
 
  /*
   *  *** if nothing loaded, set updated flag
   */
   if ((tie_vector->numlines == 0) || (tie_vector->abort))
      {
      updated = TRUE;
      if (tie_vector->abort)
         {
         vector_free();
         XClearArea(tie.display,XtWindow(search_image_bulletinboard),
                    0,0,0,0, True);
         }
      }
   else
      updated = FALSE;

  /*
   *  *** check that bounds are entered
   */
   if ((strcmp(XmTextGetString(vect_max_lat), "") == 0) ||
       (strcmp(XmTextGetString(vect_max_lon), "") == 0) ||
       (strcmp(XmTextGetString(vect_min_lat), "") == 0) ||
       (strcmp(XmTextGetString(vect_min_lon), "") == 0))
      {
      tie_message(Shell_Vector, "Must define Lat / Lon boundaries");
      return;
      }
 
   anything = FALSE;
  /*
   *  *** count number of coverages and allocate active flag space
   */
   i=0;
   while(dcw_coverage[i].desc != NULL)
     i++;
   active = TieMalloc(i*sizeof(int));

   i=0;
   while(dcw_coverage[i].desc != NULL)
      {
      active[i] = FALSE;

     /*
      *  *** if any line types selected, set active flag
      */
      for (j = 0; j < dcw_coverage[i].numvdt; j++)
         if (dcw_coverage[i].vdt[j].selected)
            {
            active[i] = TRUE;
            break;
            }

      anything |= active[i];

      i++;
      }

  /*
   *  *** if nothing requested, print message 
   */
   if (!anything)
      {
      tie_message(Shell_Vector, "No Feature requested");
      free(active);
      return;
      }

  /*
   *  *** extract bounds from vector panel
   */
   minlat = map_dms2deg(XmTextGetString(vect_min_lat));
   minlon = map_dms2deg(XmTextGetString(vect_min_lon));
   maxlat = map_dms2deg(XmTextGetString(vect_max_lat));
   maxlon = map_dms2deg(XmTextGetString(vect_max_lon));

  /*
   *  *** if boundary changed, update display
   */
   if ((minlon != tie_vectfile[0].corner[1][0]) ||
        (minlat != tie_vectfile[0].corner[1][1]) ||
        (maxlon != tie_vectfile[0].corner[2][0]) ||
        (maxlat != tie_vectfile[0].corner[2][1]))
      {
      if (!updated && (tie_vector->numlines > 0))
         vector_free();
    
      if (!updated)
         updated = TRUE;
      }
 
  /*
   *  *** reset bounding corner to image proj coordinates
   */
   temppt[0] = tie_vectfile[0].corner[1][0] = minlon;
   temppt[1] = tie_vectfile[0].corner[1][1] = minlat;
   vect2image(temppt, 1, 0);
   tie_vector->llx = temppt[0];
   tie_vector->lly = temppt[1];
 
   temppt[0] = tie_vectfile[0].corner[2][0] = maxlon;
   temppt[1] = tie_vectfile[0].corner[2][1] = maxlat;
   vect2image(temppt, 1, 0);
   tie_vector->urx = temppt[0];
   tie_vector->ury = temppt[1];

  /*
   *  *** get 15 degree tile boundaries
   */
   minlontile = dcw_lontile(minlon, TRUE);
   maxlontile = dcw_lontile(maxlon, FALSE);
   minlattile = dcw_lattile(minlat, TRUE);
   maxlattile = dcw_lattile(maxlat, FALSE);

  /*
   *  *** determine 5 minute boundaries
   */
   startcol = dcw_getrowcol(minlon, TRUE);
   endcol   = dcw_getrowcol(maxlon, FALSE);
   startrow = dcw_getrowcol(minlat, TRUE);
   endrow   = dcw_getrowcol(maxlat, FALSE);


  /*
   *  *** If the max lon value is less than the min lon value, we need to 
   *  *** process DCW tiles in two steps (...-Z, and Z-...).    
   */
   tmp_latlontile = tmp_rowcol = '0';
   if (maxlontile < minlontile) 
      {
      tmp_rowcol = endcol;
      tmp_latlon = maxlon;
      tmp_latlontile = maxlontile;
      endcol = '3';
      maxlon = 180.0;
      maxlontile = 'Z';
      }

   if ((dcw_getcd(tie_parms.cd, &tie_vector->dcwlib, dcw_cd)) == E_FAIL)
      {
      tie_message(Shell_Vector, "Cannot read CD\n");
      free(active);
      return;
      }

  /*
   *  *** if no updates detected, check all selection against loadings
   */
   if (!updated)
      {
      i=0;
      while(dcw_coverage[i].desc != NULL)
         {
         if (updated)
            break;

         for (j = 0; j < dcw_coverage[i].numvdt; j++)
            {
            if (dcw_coverage[i].vdt[j].selected && 
                !dcw_coverage[i].vdt[j].loaded)
               {
               updated = TRUE;
               break;
               }
            }

         i++;
         }
      }
      
  /*
   *  *** if no updates detected, return
   */
   if (!updated)
      {
      tie_message(Shell_Vector, "Nothing Updated???");
      free(active);
      return;
      }

  /*
   *  *** if still lines loaded, free attribute form
   */
   if (tie_vector->numlines > 0)
      vector_freeattrib();

   vector_dcwattrib();

  /*
   *  *** set up abort capabilities
   */
   tie_vector->abort = FALSE;

  /*
   *  Display the wait cursor (hourglass) in both the image window and main
   *  panel.
   */
   show_wait_cursor();

  /*
   *  *** Display the processing message and processing icon.
   */
   XtPopup(process_dialog_shell,XtGrabExclusive);
   sprintf(title,"Loading DCW tiles ... ");
   XtSetArg(args[0], XeNlabelString, title);
   XtSetArg(args[1], XeNupdatePercent, 1);
   XtSetValues(process_widget, args, 2);
   XeProcessMInit(process_widget,0, 
              (maxlontile-minlontile+1)*
              (maxlattile-minlattile+1)*
              9+1);
   XeProcessMUpdate(process_widget,0);
   ForceDialog(process_dialog_shell);
   numtile = 0;


  /*
   *  *** for each longitude 15 minute tile
   */
   for (i = minlontile; i <= maxlontile; i++)
      {
      if (tie_vector->abort)
         break;

     /*
      *  *** for each latitude 15 minute tile
      */
      for (j = minlattile; j <= maxlattile; j++)
         {
         if (tie_vector->abort)
            break;

        /*
         *  *** for each 5 minute latitude within this 15 minute tile
         */
         for (r = '1'; r < '4'; r++)
            {
            if (tie_vector->abort)
               break;

            if (((j == minlattile) && (r < startrow)) ||
                ((j == maxlattile) && (r > endrow)))
               {
              /*
               *  *** update processing dialog, and check for cancel
               */
               numtile += 3;
               if (XeProcessMUpdate(process_widget,numtile) < 0)
                  tie_vector->abort = TRUE;
               continue;
               }

           /*
            *  *** for each 5 minute longitude within this 15 minute tile
            */
            for (c = '1'; c < '4'; c++)
               {
              /*
               *  *** update processing dialog, and check for cancel
               */
               numtile++;
               if (XeProcessMUpdate(process_widget,numtile) < 0)
                  {
                  tie_vector->abort = TRUE;
                  break;
                  }

               if (((i == minlontile) && (c < startcol)) ||
                   ((i == maxlontile) && (c > endcol)))
                  continue;
 
              /*
               *  *** generate tile pathname
               */
               tilename[0] = i - ('A' - 'a');
               tilename[1] = '/';
               tilename[2] = j - ('A' - 'a');
               tilename[3] = '/';
               tilename[4] = c;
               tilename[5] = r;
               tilename[6] = '\0';

              /*
               *  *** get tile ID from tile Area Feature Table
               */
               if ((tileid=dcw_gettileid(tie_parms.cd, 
                         dcw_cd[tie_vector->dcwlib].lib, tilename)) == E_FAIL)
                  continue;

              /*
               *  *** for each coverage type
               */
               k=0;
               while(dcw_coverage[k].desc != NULL)
                  {
                 /*
                  *  *** check abort flag
                  */
                  if (tie_vector->abort)
                     break;

                 /*
                  *  *** if nothing requested, skip to next coverage
                  */
                  if (!active[k])
                     {
                     k++;
                     continue;
                     }

                 /*
                  *  *** find offset into Line Feature Table for this tile
                  */
                  if (dcw_getoffset(tie_parms.cd,dcw_cd[tie_vector->dcwlib].lib,
                                    dcw_coverage[k].name, tileid, &recnum, 
                                    &numlft) == E_SUCC)
                     {
                     numline = dcw_linequery(tie_parms.cd, 
                                        dcw_cd[tie_vector->dcwlib].lib, 
                                        &dcw_coverage[k], &line, &type, tileid, 
                                        recnum, numlft);

                    /*
                     *  *** extract requested lines from the tile
                     */
                     if (numline > 0)
                        {
                        numconvert = tie_vector->numlines;
                        dcw_readlines(tie_parms.cd, 
                                 dcw_cd[tie_vector->dcwlib].lib, 
                                 &dcw_coverage[k], &tie_vector->line, 
                                 &tie_vector->numlines, &tie_vector->maxline,
                                 tilename, line, type, numline, minlat, minlon,
                                 maxlat, maxlon, TRUE);

                       /*
                        *  *** convert lines to image proj coordinates
                        */
                        for (l = numconvert; l < tie_vector->numlines; l++)
                           vect2image(tie_vector->line[l].pts, 
                                      tie_vector->line[l].npts, 0);
                        }

                     free(line);
                     free(type);
                     }

                  k++;
                  }
               }
            }
         }
      }

  /* 
   *  *** If DCW tiles are being processed in two pieces, do part two now 
   */
   if (tmp_latlontile != '0')
      {
      endcol = tmp_rowcol;
      maxlon = tmp_latlon;
      maxlontile = tmp_latlontile;
      tmp_rowcol = startcol;
      tmp_latlon = minlon;
      tmp_latlontile = minlontile;
      startcol = '1';
      minlon = -180.0;
      minlontile = 'A';
      XtPopdown(process_dialog_shell);
      XtPopup(process_dialog_shell,XtGrabExclusive);
      sprintf(title,"Loading DCW tiles ... ");
      XtSetArg(args[0], XeNlabelString, title);
      XtSetArg(args[1], XeNupdatePercent, 1);
      XtSetValues(process_widget, args, 2);
      XeProcessMInit(process_widget,0,
              (maxlontile-minlontile+1)*
              (maxlattile-minlattile+1)*
              9+1);
      XeProcessMUpdate(process_widget,0);
      ForceDialog(process_dialog_shell);
      numtile = 0;

     /*
      *  *** for each longitude 15 minute tile
      */
      for (i = minlontile; i <= maxlontile; i++)
         {
         if (tie_vector->abort)
            break;

        /*
         *  *** for each latitude 15 minute tile
         */
         for (j = minlattile; j <= maxlattile; j++)
            {
            if (tie_vector->abort)
               break;

           /*
            *  *** for each 5 minute latitude within this 15 minute tile
            */
            for (r = '1'; r < '4'; r++)
               {
               if (tie_vector->abort)
                  break;


               if (((j == minlattile) && (r < startrow)) ||
                   ((j == maxlattile) && (r > endrow)))
                  {
                 /*
                  *  *** update processing dialog, and check for cancel
                  */
                  numtile += 3;
                  if (XeProcessMUpdate(process_widget,numtile) < 0)
                     tie_vector->abort = TRUE;
                  continue;
                  }

              /*
               *  *** for each 5 minute longitude within this 15 minute tile
               */
               for (c = '1'; c < '4'; c++)
                  {
                 /*
                  *  *** update processing dialog, and check for cancel
                  */
                  numtile++;
                  if (XeProcessMUpdate(process_widget,numtile) < 0)
                     {
                     tie_vector->abort = TRUE;
                     break;
                     }

                  if (((i == minlontile) && (c < startcol)) ||
                      ((i == maxlontile) && (c > endcol)))
                     continue;

                 /*
                  *  *** generate tile pathname
                  */
                  tilename[0] = i - ('A' - 'a');
                  tilename[1] = '/';
                  tilename[2] = j - ('A' - 'a');
                  tilename[3] = '/';
                  tilename[4] = c;
                  tilename[5] = r;
                  tilename[6] = '\0';

                 /*
                  *  *** get tile ID from tile Area Feature Table

                  */
                  if ((tileid=dcw_gettileid(tie_parms.cd,
                         dcw_cd[tie_vector->dcwlib].lib, tilename)) == E_FAIL)
                     continue;

                 /*
                  *  *** for each coverage type
                  */
                  k=0;
                  while(dcw_coverage[k].desc != NULL)
                     {
                    /*
                     *  *** check abort flag
                     */
                     if (tie_vector->abort)
                        break;

                    /*
                     *  *** if nothing requested, skip to next coverage
                     */
                     if (!active[k])
                        {
                        k++;
                        continue;
                        }

                    /*
                     *  *** find offset into Line Feature Table for this tile
                     */
                     if (dcw_getoffset(tie_parms.cd,
                         dcw_cd[tie_vector->dcwlib].lib, dcw_coverage[k].name,
                         tileid, &recnum, &numlft) == E_SUCC)
                        {
                        numline = dcw_linequery(tie_parms.cd,
                                        dcw_cd[tie_vector->dcwlib].lib,
                                        &dcw_coverage[k], &line, &type, tileid,
                                        recnum, numlft);

                       /*
                        *  *** extract requested lines from the tile
                        */
                        if (numline > 0)
                           {
                           numconvert = tie_vector->numlines;
                           dcw_readlines(tie_parms.cd,
                                 dcw_cd[tie_vector->dcwlib].lib,
                                 &dcw_coverage[k], &tie_vector->line,
                                 &tie_vector->numlines, &tie_vector->maxline,
                                 tilename, line, type, numline, minlat, minlon,
                                 maxlat, maxlon, TRUE);

                          /*
                           *  *** convert lines to image proj coordinates
                           */
                           for (l = numconvert; l < tie_vector->numlines; l++)
                              vect2image(tie_vector->line[l].pts,
                                         tie_vector->line[l].npts, 0);
                           }

                        free(line);
                        free(type);
                        }

                     k++;
                     }
                  }
               }
            }
         }
      startcol = tmp_rowcol;
      minlon = tmp_latlon;
      minlontile = tmp_latlontile;
      }

   free(active);

  /*
   *  *** Redisplay the default arrow cursor.
   */
   unshow_wait_cursor();

  /*
   *  Close the processing message window and turn on the bell to let the user
   *  know the image load is complete.
   */
   XtPopdown(process_dialog_shell);
   XBell(tie.display,0);
 
  /*
   *  *** set loaded flags on selected items
   */
   i=0;
   while(dcw_coverage[i].desc != NULL)
      {
      for (j = 0; j < dcw_coverage[i].numvdt; j++)
         if (dcw_coverage[i].vdt[j].selected)
            {
            if (!tie_vector->abort)
               dcw_coverage[i].vdt[j].loaded = TRUE;
            else
               dcw_coverage[i].vdt[j].loaded = FALSE;
            }
      i++;
      }

   if (tie_vector->numlines > 0)
      {
      tie_vector->numfiles = 1;
      tie_vectfile[0].filename =
                            TieMalloc(strlen(dcw_cd[tie_vector->dcwlib].lib)+1);
      strcpy(tie_vectfile[0].filename, dcw_cd[tie_vector->dcwlib].lib);
      vector_scaletofit(tie_vector->tmatrix);
      XClearArea(tie.display, XtWindow(ref_image_bulletinboard),
                 0, 0, 0, 0, False);
      vector_redrawall();
      }
   else if (!tie_vector->abort)
      tie_message(Shell_Vector, "No Lines found");
}

/*******************************************************************************
Purpose:
    This routine handles updates to the DCW display.
 
By J. Hemmer, USGS/EROS Data Center, 2/92
 
Algorithm Description:
*******************************************************************************/
void vector_dcwwrite
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   struct
   TIE_LINE     *geo;          /* lines converted to geographic coordinates */
   int          i;
   char         ltpath[256];

   if (tie_vector->numlines == 0)
      {
      tie_message(Shell_Vector, "No Lines are Drawn");
      return;
      }
  /*
   *  *** generate lt path name 
   */
   i = 0;
   do
      {
      i++;
      sprintf(ltpath, "dcw_%d.lt", i);
      }
   while (!access(ltpath, 0));

   geo = TieMalloc(tie_vector->numlines*sizeof(struct TIE_LINE));

   for (i = 0; i < tie_vector->numlines; i++)
      {
     /*
      *  *** temporarily store points and convert to image coordinates
      */
      geo[i].npts = tie_vector->line[i].npts;
      geo[i].attrib = tie_vector->line[i].attrib;
      geo[i].pts = (double*)malloc(2*geo[i].npts*sizeof(double));
      geo[i].npts = tie_vector->line[i].npts;
      memcpy(geo[i].pts, tie_vector->line[i].pts, 2*geo[i].npts*sizeof(double));
      proj2degrees(geo[i].pts, geo[i].npts, SEARCH);
      }

   dcw_writelt(ltpath, geo, tie_vector->numlines, 
               dcw_coverage,
               tie_vectfile[0].corner[1][1], tie_vectfile[0].corner[1][0], 
               tie_vectfile[0].corner[2][1], tie_vectfile[0].corner[2][0]);

   for (i = 0; i < tie_vector->numlines; i++)
      free(geo[i].pts);
   free(geo);

   tie_message(Shell_Vector, "DCW lines written to LT file");
}


/*******************************************************************************
Purpose:
    This routine dumps the current dlg lines to a labeled table
 
By T. Rockvam, USGS/EROS Data Center, 3/94
 
Algorithm Description:
*******************************************************************************/
void vector_dlgwrite
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   struct
   TIE_LINE     *geo;          /* lines converted to geographic coordinates */
   int          i;
   char         ltpath[256];

   if (tie_vector->numlines == 0)
      {
      tie_message(Shell_Vector, "No Lines are Drawn");
      return;
      }
  /*
   *  *** generate lt path name 
   */
   i = 0;
   do
      {
      i++;
      sprintf(ltpath, "dlg_%d.lt", i);
      }
   while (!access(ltpath, 0));

   geo = TieMalloc(tie_vector->numlines*sizeof(struct TIE_LINE));
   
   for (i = 0; i < tie_vector->numlines; i++)
      {
     /*
      *  *** temporarily store points and convert to image coordinates
      */
      geo[i].npts = tie_vector->line[i].npts;
      geo[i].attrib = tie_vector->line[i].attrib;
      geo[i].pts = TieMalloc(2*geo[i].npts*sizeof(double));
      memcpy(geo[i].pts, tie_vector->line[i].pts, 2*geo[i].npts*sizeof(double));
      proj2degrees(geo[i].pts, geo[i].npts, SEARCH);
      }

   dlg_writelt(ltpath, geo, tie_vector->numlines,
               tie_vectfile[0].corner[1][1], tie_vectfile[0].corner[1][0], 
               tie_vectfile[0].corner[2][1], tie_vectfile[0].corner[2][0]);

   for (i = 0; i < tie_vector->numlines; i++)
      free(geo[i].pts);
   free(geo);

   tie_message(Shell_Vector, "DLG lines written to LT file");
}

/*******************************************************************************
Purpose:
    This routine creates the dlg vector attributes display list. 

By J. Hemmer, USGS/EROS Data Center, 6/92
Modified by T. Rockvam, USGS/EROS Data Center, 3/94
 - Modified to use for dlg vectors

Algorithm Description:
*******************************************************************************/
void vector_dlgattribs(void)
{
   Arg          args[10];               /* widget resource arguments    */
   int          argcnt;                 /* widget argument counter      */
   int          i;                      /* loop counter                 */
  
  /*
   *  *** if attribute form not previously created
   */
   if (attrib_form == NULL) 
      {
      vector_freeattrib();
      while (dlg_attrib[tie_vector->nattrib].mincode != 0)
         tie_vector->nattrib++;
      tie_vector->line_att  = (unsigned char*)malloc(tie_vector->nattrib);
      tie_vector->color_att = (unsigned char*)malloc(tie_vector->nattrib);
      tie_vector->enabled   = (unsigned char*)malloc(tie_vector->nattrib);

      attrib_form = (Widget*)malloc(tie_vector->nattrib*sizeof(Widget));

      for (i = 0; i < tie_vector->nattrib; i++)
         {
         tie_vector->line_att[i]  = 0;
         tie_vector->color_att[i] = GREEN_COL;
         tie_vector->enabled[i] = TRUE;
   
         argcnt = 0;
         XtSetArg(args[argcnt], XmNwidth,    460); argcnt++;
         XtSetArg(args[argcnt], XmNheight,    30); argcnt++;
         XtSetArg(args[argcnt], XmNresizePolicy, XmRESIZE_NONE); argcnt++;
   
         attrib_form[i] = XtCreateWidget("attribute_form",
                      xmFormWidgetClass,
                      vect_attrib_rowcol,
                      args,
                      argcnt);
   
         Createattrib_form(attrib_form[i], dlg_attrib[i].desc, i, 
                           (int)tie_vector->color_att[i],       
                           (int)tie_vector->line_att[i]);       
         }
      }

  /*
   *   *** manage set of widget for attributes present in vector display
   */
   for (i = 0; i < tie_vector->nattrib; i++)
      {
      if ((!dlg_attrib[i].present) && (XtIsManaged(attrib_form[i])))
         XtUnmanageChild(attrib_form[i]);
      else if ((dlg_attrib[i].present) && (!XtIsManaged(attrib_form[i])))
         XtManageChild(attrib_form[i]);
      }
}

/*******************************************************************************
Purpose:
    This routine changes the color of a given line attribute.
    It is called twice per color change, once to initialize the
    color requestor, and once to assign the selected color.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_attribute_color
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];                /* widget resource              */
   int          cindex,                 /* color index                  */
                oldplane;               /* previous color plane         */
   static
   short        attrib;                 /* attribute index              */
   static   
   Widget       color_button = NULL;    /* selected attribute color button */

  /*
   *  *** if NULL client, display color requestor
   */
   if (color_button == NULL)
      {
      attrib = (short)client;
      color_button = w;
      Color_req = Createcolor_req(Shell_Vector, vector_attribute_color);
      XtManageChild(Color_req);
      }
  /*
   *  *** otherwise, set color and remove color requestor
   */
   else
      {
      cindex = (int)client;

      if (cindex != tie_vector->color_att[attrib])
         {
         oldplane = tie_vector->color_att[attrib];
         tie_vector->color_att[attrib] = cindex;
         if (tie_parms.vsource == DLGOPT)
           dlg_attrib[attrib].color = cindex;
         vector_redrawplane(oldplane);
         vector_redrawplane(tie_vector->color_att[attrib]);

         if (XtIsRealized(Shell_SrchFull))
            XClearArea(tie.display, XtWindow(fullsrch_win_bulletinboard), 
                    0,0,0,0, TRUE);

         if (XtIsRealized(Shell_Zoom))
            XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard), 
                    0,0,0,0, TRUE);
            
         XtSetArg(args[0],XmNforeground,
                  COLOR(Shell_Vector, wk_colors[cindex]));
         XtSetValues(color_button, args, 1);
         }

      XtUnmanageChild(Color_req);
      XtDestroyWidget(Color_req);
      color_button = NULL;
      }
}

/*******************************************************************************
Purpose:
    This routine toggles the display of a given attribute.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_attribute_enable
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[1];
   XmString     xmstr;

  /*
   *  *** if attributes is currently enabled, disable
   */
   if (tie_vector->enabled[(int)client])
      {
      XtSetArg(args[0], XmNlabelString, (xmstr=XmStringCreateLtoR(
                   "Off", XmSTRING_DEFAULT_CHARSET))); 
      tie_vector->enabled[(int)client] = FALSE;
      XtSetValues(w, args, 1);
      XmStringFree( xmstr );
      vector_redrawplane(tie_vector->color_att[(int)client]);

     /* 
      *  *** force display windows to refresh graphics
      */
      XClearArea(tie.display, XtWindow(ref_image_bulletinboard), 
               0,0,0,0, TRUE);

      }
  /*
   *  *** otherwise, enable it
   */
   else
      {
      XtSetArg(args[0], XmNlabelString, (xmstr=XmStringCreateLtoR(
                   "On", XmSTRING_DEFAULT_CHARSET))); 
      tie_vector->enabled[(int)client] = TRUE;
      XtSetValues(w, args, 1);
      XmStringFree( xmstr );

      if ((tie_parms.vsource == WDB) && (!wdb_attrib[(int)client].loaded))
         vector_wdbupdate(w,client,call_data);
      else
         vector_redrawplane(tie_vector->color_att[(int)client]);
      }

  /*
   *  *** update fullres window overlays
   */
   if (XtIsRealized(Shell_SrchFull))
      XClearArea(tie.display, XtWindow(fullsrch_win_bulletinboard), 
              0,0,0,0, TRUE);

  /*
   *  *** update zoom window overlays
   */
   if (XtIsRealized(Shell_Zoom))
      XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard), 
              0,0,0,0, TRUE);
}

/*******************************************************************************
Purpose:
    This routine changes the line pattern of a given line attribute.
    This routines is called twice per line style change:  once to initialize
    the line style requestor, and once to assign the selected line style.

By J. Hemmer, USGS/EROS Data Center, 2/92

Algorithm Description:
*******************************************************************************/
void vector_attribute_line
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   static
   Widget       line_button = NULL;     /* selected attribute line button */
   static
   int          attrib;                 /* selecte attribute index        */

  /*
   *  *** if line_button is NULL, display line requestor
   */
   if (line_button == NULL)
      {
      line_button = w;
      attrib = (int)client;
      Line_req = Createline_req(Shell_Vector, vector_attribute_line);
      XtManageChild(Line_req);
      }
  /*
   *  *** otherwise, set line and remove line requestor
   */
   else
      {
      XtRemoveCallback(line_button, XmNexposeCallback,
                 vector_line_expose, 
                 (caddr_t)dash_pattern[tie_vector->line_att[attrib]]);
      tie_vector->line_att[attrib] = (int)client;
      if (tie_parms.vsource == DLGOPT)
        dlg_attrib[attrib].style = (int)client;
      vector_redrawplane(tie_vector->color_att[attrib]);

     /*
      *  *** force display windows to refresh graphics
      */
      XClearArea(tie.display, XtWindow(ref_image_bulletinboard),
                 0,0,0,0, TRUE);

     /*
      *  *** update search fullres overlays
      */
      if (XtIsRealized(Shell_SrchFull))
         XClearArea(tie.display, XtWindow(fullsrch_win_bulletinboard),
                 0,0,0,0, TRUE);

     /*
      *  *** update zoom window overlays
      */
      if (XtIsRealized(Shell_Zoom))
         XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                 0,0,0,0, TRUE);

      XtAddCallback(line_button, XmNexposeCallback,
                 vector_line_expose, 
                 (caddr_t)dash_pattern[tie_vector->line_att[attrib]]);
      XClearArea(tie.display, XtWindow(line_button),0,0,0,0,TRUE);
      XtUnmanageChild(Line_req);
      XtDestroyWidget(Line_req);
      line_button = NULL;
      }
}

/*******************************************************************************
Purpose:
    This routine automatically loads one full res window's worth of vector
    data.  (This routine is only called in IM2VECT mode.)

By T. Rockvam, USGS/EROS Data Center, 3/94

Algorithm Description:
*******************************************************************************/
void vector_window
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   int          plane_idx,      /* Index to red plane number of the currently
                                   viewed image.                              */
                i,              /* Looping counter                            */
                order;          /* polynomial order                           */

   double       ssl,            /* Starting search line.                      */
                sss,            /* Starting search sample.                    */
                sel,            /* Ending search line.                        */
                ses,            /* Ending search sample.                      */
                y_coef[6],
                x_coef[6],
                temp_y,
                temp_x;

   if (event->xbutton.button != 3)
      return;
   if (Vector_form == NULL)
      {
      tie_message(Shell_Vector, "No Feature requested");
      return;
      }

   /*
    * *** get the sl,ss of the window 
    */
   plane_idx = tie_composite[SEARCH].rgbplanes[RED] - 1;
   sss = (((event->xbutton.x + 1) * tie_planes[plane_idx].skips) +
             tie_planes[plane_idx].window[SS] - 1) - 
             (tie_planes[plane_idx].skips - 1);
   ssl = (((event->xbutton.y + 1) * tie_planes[plane_idx].skipl) +
             tie_planes[plane_idx].window[SL] - 1) - 
             (tie_planes[plane_idx].skipl - 1);

   sss -= tie_parms.fwidth / 2.0;
   ssl -= tie_parms.fheight / 2.0;
   ses = sss + tie_parms.fwidth;
   sel = ssl + tie_parms.fheight;

   /*
    * *** reset vector corners to pixel values 
    */
   if (sss < ses)
      {
      tie_vectfile[0].corner[0][0] = sss;
      tie_vectfile[0].corner[1][0] = sss;
      tie_vectfile[0].corner[2][0] = ses;
      tie_vectfile[0].corner[3][0] = ses;
      }
   else
      {
      tie_vectfile[0].corner[0][0] = ses;
      tie_vectfile[0].corner[1][0] = ses;
      tie_vectfile[0].corner[2][0] = sss;
      tie_vectfile[0].corner[3][0] = sss;
      }
   if (ssl > sel)
      {
      tie_vectfile[0].corner[0][1] = sel;
      tie_vectfile[0].corner[1][1] = ssl;
      tie_vectfile[0].corner[2][1] = sel;
      tie_vectfile[0].corner[3][1] = ssl;
      }
   else
      {
      tie_vectfile[0].corner[0][1] = ssl;
      tie_vectfile[0].corner[1][1] = sel;
      tie_vectfile[0].corner[2][1] = ssl;
      tie_vectfile[0].corner[3][1] = sel;
      }

   /*
    * *** convert sl,ss,el,es to lat/long 
    */
   order = tie_parms.order;
   tps_Qfit(tie_select.numtie, tie_select.stie, tie_select.rtie,
            NULL, &order, y_coef, x_coef);
   for (i = 0; i < 4; i++)
      {
      temp_x = tie_vectfile[0].corner[i][0];
      temp_y = tie_vectfile[0].corner[i][1];

      tie_vectfile[0].corner[i][0] =
                 c_eval(&order, x_coef, &temp_x, &temp_y);
      tie_vectfile[0].corner[i][1] =
                 c_eval(&order, y_coef, &temp_x, &temp_y);
      }
   if (tie_parms.vsource == DLGOPT)
      proj2degrees(tie_vectfile[0].corner, 4, SEARCH);
   else
      image2vect(tie_vectfile[0].corner, 4, 0);

   /*
    * *** set the vector text fields for this area of interest 
    */
   vector_setbounds();

   /*
    * *** update the vectors for this area of interest 
    */
   if (tie_parms.vsource == DLGOPT)
      vector_dlgupdate(w,client,NULL);
   else if (tie_parms.vsource == DCW)
      vector_dcwupdate(w,client,NULL);
   else
      vector_wdbupdate(w,client,NULL);

}
