#include <ctype.h>
#include "tiepts.h"
#include "cr.h"
#include "matsup.h"
#include "tpssup.h"
#include "protos.h"


/*******************************************************************************
Purpose:
    This routine closes the tie point accept window. 

By J. Hemmer, USGS/EROS Data Center, 2/92
Modified by J. LaVergne USGS/EROS Data Center 7/93
  - Changed the 'Accept/Reject Selected Point' window to be positioned below 
    the 'Tie Points - Zoom' window, so the user can see the selected point 
    while deciding to accept or reject it.
Modified by J. LaVergne USGS/EROS Data Center 7/93
  - Changed the elevation field to be required if the user selects that option
    in the parameters panel.  Also changed to verify that the elevation value
    entered by the user contains valid information.
  - Changed calls to 'caution' into 'tie_message' calls, because DGs that are
    not upgraded blow up tiepts.
  - Changed to verify that both an elevation value and a point id are entered,
    and that the point id is <= 20 characters long.  Also changed the point
    id field to be uneditable when reselecting a point.
  Algorithm Description:
******************************************************************************/
void verify_destroy
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
   Arg          args[10];       /* Resource argument                         */
   Widget       units;          /* Elevation units (feet/meters)             */
   Widget       rough_accept;   /* rough trans. point id/elev entry panel    */
   int          i,              /* loop counter                              */
                ptindex;        /* point index of new/reselected point       */
   char         *ptid;          /* point identification string               */
   char         *elev;          /* point elevation string                    */
   char         *ptr;           /* string pointer                            */
   char         coordtyp[5];

  /*
   *  *** make nonrelative fields visible, otherwise shell will resize
   */
   if (!XtIsManaged(verify_id))
      {
      XtManageChild(verify_id);
      XtManageChild(verify_id_label);
      XtManageChild(verify_elev_label);
      XtManageChild(verify_elev);
      }

  /*
   *   *** if point was accepted
   */
   if (w == verify_accept)
      {
      if (tie_parms.flags & PARMS_LINESAMP)
        strcpy(coordtyp,"REF");
      else if (tie_parms.flags & PARMS_LATLONG)
        strcpy(coordtyp,"GEO");
      else if (tie_parms.flags & PARMS_PROJ)
        strcpy(coordtyp,"PRO");
     /*
      *  *** Write the selected point to the rough transformation file.
      */
      if ((tie_rt.calc_rt == RT_SEL) && (tie.mode != ONESRCH))
      {
         fprintf(tie_rt.rtfp,
             "1,%lf,%lf,%d,%d,%d,%lf,%lf,%d,%d,%d,%s,%s\n",
             tie_select.rtie[tie_select.seltie-1][0],
             tie_select.rtie[tie_select.seltie-1][1],
             tie_parms.factor, tie_parms.refoff[0],tie_parms.refoff[1],
             tie_select.stie[tie_select.seltie-1][0], 
             tie_select.stie[tie_select.seltie-1][1],
             tie_parms.factor, tie_parms.srchoff[0], tie_parms.srchoff[1],
             "image",coordtyp);

         if (tie.mode == IM2MAP) 
            {
            strcpy(tie_select.mapname[tie_select.seltie-1],tie_mapparm.mapname);
            strcpy(tie_select.mapid[tie_select.seltie-1],tie_mapparm.mapid);
            }

        /*
         *  *** if 4 pts, calculate/display the rough transformation.
         */
         if (tie_select.numtie == 4)
            {
            tie_rt.degree = tie_parms.order;
            tps_Qfit(tie_select.numtie, tie_select.rtie, tie_select.stie, 
                     tie_select.disabled, &tie_rt.degree, tie_rt.line_coef,
                     tie_rt.samp_coef);
            rough_accept = Createrough_form(Shell_Main);
            XtManageChild(rough_accept);
            for (i = 0; i < tie_select.numtie; i++)
               display_pt(i);
            edit_compute();
            }

        /*
         *  *** lower zoom window and remove acceptance panel
         */
         XLowerWindow(tie.display, XtWindow(Shell_Zoom));
         tie_select.curref = FALSE;
         tie_select.pt_id[tie_select.seltie-1][0] = '\0';

         if (tie.mode == IM2VECT)
            vector_unmark();
         }

     /*
      *  *** otherwise, store point in output TPS file
      */
      else
         {
         if ((tie.mode != ONESRCH) && 
             (tie_rt.calc_rt != RT_CAL) && (tie_select.seltie > 3))
            tie_rt.calc_rt = RT_CAL;

         ptid = XmTextGetString(verify_id);
         elev = XmTextGetString(verify_elev);

        /*
         *  *** Make sure we got an id and an elevation value 
         */
         if (*elev == NULL)
            {
            /* JgL - use tie_message, caution fails */
            tie_message(verify_dialog_shell,"Enter elevation value");
            XmProcessTraversal(verify_elev, XmTRAVERSE_CURRENT);
            return;
            }                                                          
         if (*ptid == NULL)
            {
            tie_message(verify_dialog_shell,"Enter Point ID");
            XmProcessTraversal(verify_id, XmTRAVERSE_CURRENT);
            return;
            }
         ptid = (char *)squeeze(ptid,(long)strlen(ptid));
         if ((ptid == NULL) || ((int)strlen(ptid) > 20) || 
                     ((int)strlen(ptid) == 0))
            {
            tie_message(verify_dialog_shell,
                        "Point ID must be 1 to 20 characters");
            XmProcessTraversal(verify_id, XmTRAVERSE_CURRENT);
            return;
            }

        /*
         *  ***  Make sure elevation value is valid.  Testing is minimal
         *  ***  for now, just look for numbers and valid characters.  Various
         *  ***  tests could and maybe should be added, such as multiple 
         *  ***  entries of valid characters, embedded blanks, etc.
         */
         for (ptr = elev; *ptr != '\0'; ptr++)
            {
            if ((isdigit(*ptr) == 0) && (*ptr != ' ') && (*ptr != '-') && 
                (*ptr != '+') && (*ptr != '.'))
               {
               /* JgL - use tie_message, caution fails */
               tie_message(verify_dialog_shell,"Invalid elevation value");
               XmProcessTraversal(verify_elev, XmTRAVERSE_CURRENT);
               return;
               }
            }
        
        /*
         *  *** get units (feet/meters)
         */
         XtSetArg(args[0], XmNmenuHistory, &units);
         XtGetValues(verify_units_option, args, 1);

        /*
         *  *** if pt_id is valid, store in TPS file
         */
         tie_select.numtie--;
         if (tps_checkid(ptid) || (tie.flags & TIE_RESELECTING))
            {
            tie_select.numtie++;
            if (tie.flags & TIE_RESELECTING)
               ptindex = tie_select.reselect;
            else
               {
               ptindex = tie_select.seltie-1;
               strcpy(tie_select.pt_id[ptindex], ptid);
               if (tie.mode == IM2MAP) 
                  {
                  strcpy(tie_select.mapname[ptindex],tie_mapparm.mapname);
                  strcpy(tie_select.mapid[ptindex],tie_mapparm.mapid);
                  }

               }

            sscanf(elev, "%lf", &tie_select.elev[ptindex]);

           /*
            *  *** if units are in feet, convert to meters
            */
            if (units == verify_units_feet)
               tie_select.elev[ptindex] *= FEET2CM/100.;

           /*
            *  *** dump new records to TPS files
            */
            if (tie.flags & TIE_RESELECTING)
               tps_update();
            else
               tps_dump();

           /*
            *  *** compute new residuals
            */
            if (tie_rt.calc_rt != RT_SEL)
               {
               edit_compute();
               if (tie.mode == IM2MAP) 
                  erase_mapoutline();
               else if (tie.mode == IM2VECT)
                  {
                  erase_vectoutline();
                 /*
                  *  *** redraw overlay vectors
                  */
                  mat_identity(tie_vector->zmatrix);
                  XClearArea(tie.display, XtWindow(zoom_srchwin_bulletinboard),
                             0, 0, 0, 0, True);
                  XClearArea(tie.display, XtWindow(fullsrch_win_bulletinboard),
                             0, 0, 0, 0, True);
                  }
               else if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
                  {
                  XClearWindow(tie.display,  
                               XtWindow(zoom_refwin_bulletinboard));
                  XClearWindow(tie.display,  
                               XtWindow(zoom_srchwin_bulletinboard));
                  }
               }
   
           /*
            *  *** if residuals are display, update them
            */
            if (XtIsManaged(Edit_form))
               edit_display(TRUE);

           /*
            *  *** if stats are displayed, update them
            */
            if ((Shell_Stats != NULL) && XtIsRealized(Shell_Stats))
               stats_display(TRUE);

            XLowerWindow(tie.display, XtWindow(Shell_Zoom));
            tie_select.curref = FALSE;
            }
        /*
         *  *** if pt_id is invalid,  display message and request new id
         */
         else
            {
            tie_select.numtie++;
            tie_message(verify_dialog_shell, "ID is not unique");
            XtFree(ptid);
            XtFree(elev);
            XmProcessTraversal(verify_id, XmTRAVERSE_CURRENT);
            return;
            }

         XtFree(ptid);
         XtFree(elev);
         }
      }
   else if (w == verify_reject)
      {
      if (!(tie.flags & TIE_RESELECTING))
         {
         if (tie_select.genpt > 1)
            tie_select.genpt--;

         tie_select.seltie--;

         if (tie_rt.calc_rt != RT_DDR)
            tie_select.numtie--;
         }

      if ((tie_rt.calc_rt != RT_SEL) && (tie.mode != ONESRCH))
         for (i=0; i < 6; i++)
            {
            tie_rt.line_coef[i] = tie_rt.old_lcoef[i];
            tie_rt.samp_coef[i] = tie_rt.old_scoef[i];
            }

      XClearWindow(tie.display,  XtWindow(zoom_srchwin_bulletinboard));
      XWarpPointer(tie.display, None,
                   XtWindow(zoom_srchwin_bulletinboard),
                   None, None, None, None,
                   tie_select.savesrch[0], tie_select.savesrch[1]);
      }

   XtUnmanageChild(verify_dialog_form);     
   XtPopdown(verify_dialog_shell);
   if (tie.flags & TIE_RESELECTING)
      {
      display_pt(tie_select.reselect);
      if (tie.mode == IM2VECT)
         XClearArea(tie.display,XtWindow(ref_image_bulletinboard),
           0,0,0,0,True);
      XtSetSensitive(edit_reselect_button, True);
      tie.flags ^= TIE_RESELECTING;
      }
}

/*******************************************************************************
NAME: VERIFY_DISPLAY

PURPOSE: This displays the tie point accept/reject window.

PROGRAM HISTORY:
VERSION         DATE    AUTHOR          REASON
-------         ----    -----------     ----------
 1.0            2/92    J.Hemmer        Initial Development
 2.0            9/98    C.Olsen         Modified to display and store lat/long
                                        or projection values in IM2IM or
                                        IM2VECT mode.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        TIEPTS must be run under X Windows.

PROJECT: TIEPTS

ALGORITHM DESCRIPTION:
*******************************************************************************/
void verify_display
(
    double rx,                   /* reference x coordinate                 */
    double ry,                   /* reference y coordinate                 */
    double sx,                   /* search x coordinate                  */
    double sy,                   /* search y coordinate                  */
    double oldx,                 /* old residual x coordinate              */
    double oldy,                 /* old residual u coordinate              */
    double old_rms,              /* old residual magnitude                 */
    double newx,                 /* new residual x coordinate              */
    double newy,                 /* new residual magnitude                 */
    double new_rms              /* new residual magnitude                 */
)
{
  double pt[2];                /* temporary coordinates for conversion   */
  Arg args[2];                 /* Resource argument variable for widgets */
  char tmpstr[80];             /* temporary string                       */
  int  ptindex;                /* point index of new/reselected point    */     
  Dimension  width, height;
  Position   popx, popy;
  void display_coords(Widget,char *);

  if (tie.flags & TIE_RESELECTING)
    ptindex = tie_select.reselect;
  else
    ptindex = tie_select.seltie-1;

  if (!(tie_parms.flags & PARMS_LINESAMP))
    {
    pt[0] = tie_select.lsrtie[ptindex][0];
    pt[1] = tie_select.lsrtie[ptindex][1];
    }
  else
    {
    pt[0] = rx;
    pt[1] = ry;
    }

  if (tie.mode == IM2VECT)
    {
    if ((tie_parms.flags & PARMS_LATLONG) && (tie.svalid))
      {
      double temppt[2];
      temppt[0] = tie_select.rtie[ptindex][0];
      temppt[1] = tie_select.rtie[ptindex][1];
      proj2degrees(temppt,1,SEARCH);
      sprintf(tmpstr,"Lat: %12.5f", temppt[1]);
      display_coords(verify_y,tmpstr);
      sprintf(tmpstr,"Lon: %12.5f", temppt[0]);
      display_coords(verify_x,tmpstr);
      }
    else
      {
      sprintf(tmpstr, "Y:   %12.2f", tie_select.rtie[ptindex][1]);
      display_coords(verify_y,tmpstr);
      sprintf(tmpstr,"X:   %12.2f", tie_select.rtie[ptindex][0]);
      display_coords(verify_x,tmpstr);
      }

    /* Display the LINE/SAMPLE coords for the
       search image
    -----------------------------------------*/
    sprintf(tmpstr, "Line:                %9.2f",sy);
    display_coords(verify_reference,tmpstr);
    sprintf(tmpstr,"Sample:              %9.2f",sx);
    display_coords(verify_search,tmpstr);
    }   
  else if (tie.mode == IM2IM)
    {
    if (tie_parms.flags & PARMS_LINESAMP)
      {
      /* Display reference/search line coordinates
      --------------------------------------------*/
      sprintf(tmpstr,"Line:   %9.2f    %9.2f",pt[1],sy);
      display_coords(verify_reference,tmpstr);
   
      /* Display reference/search sample coordinates
      ----------------------------------------------*/
      sprintf(tmpstr,"Sample: %9.2f    %9.2f",pt[0],sx);
      display_coords(verify_search,tmpstr);
  
      /* If LAT/LONG or PROJECTION selected, 
         display those coordinates
      --------------------------------------*/
      if ((tie_parms.flags & PARMS_LATLONG) || (tie_parms.flags & PARMS_PROJ))
        {
        if (tie_parms.flags & PARMS_LATLONG)
          {
          /* Set strings to lat/long values (ref only)
          --------------------------------------------*/
          sprintf(tmpstr,"Lat: %12.5f", tie_select.crtie[ptindex][1]);
          display_coords(verify_y,tmpstr);
          sprintf(tmpstr,"Lon: %12.5f", tie_select.crtie[ptindex][0]);
          display_coords(verify_x,tmpstr);
          }
        else
          {
          /* Set strings to projection coordinates (ref only)
          ---------------------------------------------------*/
          sprintf(tmpstr,"Y:   %12.2f", tie_select.crtie[ptindex][1]);
          display_coords(verify_y,tmpstr);
          sprintf(tmpstr,"X:   %12.2f", tie_select.crtie[ptindex][0]);
          display_coords(verify_x,tmpstr);
          }
        }
      }
    else if (tie_parms.flags & PARMS_LATLONG)
      {
      /* User selected to output LAT/LONG values
         only. (ref only)
      --------------------------------------------*/
      sprintf(tmpstr, "Lat: %12.5f", tie_select.rtie[ptindex][1]);
      display_coords(verify_y,tmpstr);
      sprintf(tmpstr,"Lon: %12.5f", tie_select.rtie[ptindex][0]);
      display_coords(verify_x,tmpstr);
      }
    else
      {
      /* User selected to output PROJECTION values
         only. (ref only)
      --------------------------------------------*/
      sprintf(tmpstr,"Y:   %12.2f", tie_select.rtie[ptindex][1]);
      display_coords(verify_y,tmpstr);
      sprintf(tmpstr,"X:   %12.2f", tie_select.rtie[ptindex][0]);
      display_coords(verify_x,tmpstr);
      }
    }
  else
    { 
    /* Display reference/search line coordinates
    --------------------------------------------*/
    sprintf(tmpstr,"Line:   %9.2f    %9.2f",pt[1],sy);
    display_coords(verify_reference,tmpstr);
   
    /* Display reference/search sample coordinates
    ----------------------------------------------*/
    sprintf(tmpstr,"Sample: %9.2f    %9.2f",pt[0],sx);
    display_coords(verify_search,tmpstr);
    } 

  /* If not selecting rough trans. points
  ---------------------------------------*/
  if ((tie_rt.calc_rt != RT_SEL)  || (tie.mode == ONESRCH))
    {
    /* Allow entry of ID and elevation
    ----------------------------------*/
    XtSetArg(args[0],XmNsensitive,TRUE);
    XtSetValues(verify_elev,args,1);
    XtSetValues(verify_elev_label,args,1);
    if (tie.flags & TIE_RESELECTING) 
      {
      XtSetArg(args[1],XmNeditable,FALSE);
      XtSetValues(verify_id,args,2);
      XtSetValues(verify_id_label,args,2);
      }
    else
      {
      XtSetValues(verify_id,args,1);
      XtSetValues(verify_id_label,args,1);
      }

    if (tie.mode != ONESRCH)
      {
      /* Display residual lines
      -------------------------*/
      sprintf(tmpstr,"Line:     %9.2f %9.2f",oldy,newy);
      display_coords(verify_res_line,tmpstr);
   
      /* Display residual samples 
      ---------------------------*/
      sprintf(tmpstr,"Sample:   %9.2f %9.2f",oldx,newx);
      display_coords(verify_res_samp,tmpstr);
      
      /* Display residual magnitudes
      ------------------------------*/
      sprintf(tmpstr, "Magnitude:%9.2f %9.2f", old_rms, new_rms);
      display_coords(verify_res_mag,tmpstr);
      } 

    /* Display ID and elevation
    ---------------------------*/
    if (tie.flags & TIE_RESELECTING)
      {
      XmTextSetString(verify_id, tie_select.pt_id[tie_select.reselect]);
      sprintf(tmpstr, "%6.2f", tie_select.elev[tie_select.reselect]);
      XmTextSetString(verify_elev, tmpstr);
      }
    else
      {
      tie_select.numtie--;
      do 
        {
        sprintf(tmpstr, "pt_%d", tie_select.genpt++);
        }
      while(!tps_checkid(tmpstr));
      tie_select.numtie++;

      XmTextSetString(verify_id, tmpstr);

      /* Set the elev value default, based on the user's option
      ---------------------------------------------------------*/
      XmTextSetString(verify_elev, "0.0");
      if (strcmp(tie_parms.epoint,"Required") == 0) 
        XmTextSetString(verify_elev, "");
      }
    }

  /* Otherwise, do not allow entry of ID and elevation
  ----------------------------------------------------*/
  else if (XtIsManaged(verify_id))
    {
    /* Display residual lines
    -------------------------*/
    sprintf(tmpstr,"Line: ");
    display_coords(verify_res_line,tmpstr);
   
    /* Display residual samples
    ---------------------------*/
    sprintf(tmpstr,"Sample:");
    display_coords(verify_res_samp,tmpstr);

    /* Display residual magnitudes
    ------------------------------*/
    sprintf(tmpstr,"Magnitude");
    display_coords(verify_res_mag,tmpstr);

    XtSetArg(args[0], XmNsensitive, False);
    XtSetValues(verify_id, args, 1);
    XtSetValues(verify_id_label, args, 1);
    XtSetValues(verify_elev, args, 1);
    XtSetValues(verify_elev_label, args, 1);
    }

  XtSetArg(args[0],XmNheight,&height);
  XtSetArg(args[1],XmNwidth,&width);
  XtGetValues(Shell_Zoom,args,2);
  XtTranslateCoords(Shell_Zoom,(Position)(width/2)-301,(Position)height+20,
                    &popx,&popy);
  XtMoveWidget(verify_dialog_shell,popx,popy);
  XtManageChild(verify_dialog_form);
  XtPopup(verify_dialog_shell,XtGrabExclusive);
}

void display_coords
(
    Widget widget,
    char *tmpstr
)
{
  Arg args[2];                 /* Resource argument variable for widgets */
  XmString xmstr;              /* Temporary Xm string variable           */

  XtSetArg(args[0],XmNlabelString,
           (xmstr = XmStringCreateLtoR(tmpstr,XmSTRING_DEFAULT_CHARSET)));
  XtSetValues(widget,args,1);
  XmStringFree(xmstr);  
}
