/*******************************************************************************
NAME:           WDBOPEN

PURPOSE:        Open the specified WDBII files.

                The WDBII sequential file serves as look-up table
                for retrieving geographic coordinates from the
                WDBII relative file.

PROGRAM HISTORY:
Version      Date    Programmer         Request
-------      ----    ----------         -------
  1.0        10/91   D. Hollaren        Initial development

*******************************************************************************/
#include <stdio.h>
#include "tiepts.h"
#include "wdbsup.h"

int c_wdbopen
(
    char        *filenam,               /* file root name to be opened               */
    FILE        **fp_seq,               /* file pointer to the WDBII sequential file */
    FILE        **fp_rel                /* file pointer to the WDBII relative file   */
)
{
char    tmpfil[CMLEN],          /* temporary string to build the file name   */
        msg[ERRLEN],            /* error message text                        */
        disk[CMLEN];            /* disk location of World Data Bank II files */

long    nonfatal = 1;           /* non-fatal error code                      */

/*
  Get the location of the World Data Bank II files
*/
if (getenv("WDBII") != NULL)
    strcpy(disk,getenv("WDBII"));
else
    {
    c_errmsg("Error retrieving value of WDBII","wdbopen-find",&nonfatal);
    return(E_FAIL);
    }

/*
  Generate the name of the WDBII sequential file
*/
#ifdef VMS
    sprintf(tmpfil,"%s%s.seq",disk,filenam);
#else
    sprintf(tmpfil,"%s/%s.seq",disk,filenam);
#endif

/*
  Open the WDBII sequential file
*/
*fp_seq = fopen(tmpfil,"r");
if (*fp_seq == NULL)
    {
    sprintf (msg,"Error opening the file %s",tmpfil);
    c_errmsg(msg,"wdbopen-open",&nonfatal);
    return(E_FAIL);
    }

/*
  Generate the name of the WDBII relative file
*/
#ifdef VMS
    sprintf(tmpfil,"%s%s.rel",disk,filenam);
#else
    sprintf(tmpfil,"%s/%s.rel",disk,filenam);
#endif

/*
  Open the WDBII relative file
*/
*fp_rel = fopen(tmpfil,"r");
if (*fp_rel == NULL)
    {
    sprintf (msg,"Error opening the file %s",tmpfil);
    c_errmsg(msg,"wdbopen-open",&nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
