/*******************************************************************************
NAME:           WDBRANK

PURPOSE:        Sort the desired features into WDB2 rankings

PROGRAM HISTORY:
Version      Date    Programmer         Request
-------      ----    ----------         -------
  1.0        10/91   D. Hollaren        Initial development

*******************************************************************************/
#include <stdio.h>
#include "tiepts.h"
#include "wdbsup.h"

int c_wdbrank
(
    int ftype,          /* type of feature                      */
    int fcount,         /* number of features                   */
    int feature[],      /* feature classifications to process   */
    int ranks[] /* sorted rankings desired from WDBII   */
)
{
char    msg[ERRLEN];    /* error message text                   */

long    nonfatal = 1;   /* non-fatal error code                 */
int     i;              /* looping variable                     */

/*
  Feature types
*/
static char     ftext[6][4] = {"CIL",   /* Coastlines, islands, lakes   */
                               "RIV",   /* Rivers                       */
                               "BDY",   /* International boundaries     */
                               "PDY",   /* Internal boundaries          */
                               "RRS",   /* Railroads                    */
                               "RDS"};  /* Roads                        */

/*
  Maximum number or ranks for each feature
                          (CIL,RIV,BDY,PDY,RRR,RDS)
*/
static long max_rank[6] = { 15, 12,  3,  3, 19, 13};

if (fcount == 0)
    {
    /*
      Select all ranks for processing
    */
    for (i = 0; i < max_rank[ftype-1]; i++)
        ranks[i] = TRUE;
    }
else if (ftype == 0) /* do entire area file */
    {
    ranks[0] = TRUE;
    }
else
    {
    /*
      Initialize ranks
    */
    for (i = 0; i < max_rank[ftype-1]; i++)
        ranks[i] = FALSE;

    /*
      Set only the ranks that were selected
    */
    for (i = 0; i < fcount; i++)
        {
        /*
          Check the bounds of valid features for the feature type
        */
        if ((feature[i] < 1) || (feature[i] > max_rank[ftype-1]))
            {
            sprintf(msg,"Invalid feature selection %d for the %s feature type",
                    feature[i],ftext[ftype-1]);
            c_errmsg(msg,"wdbrank-class",&nonfatal);
            return(E_FAIL);
            }

        ranks[feature[i]-1] = TRUE;
        }
    }

return(E_SUCC);
}
