#include "tiepts.h"
#include "xid.h"

/*****************************************************************************
 *                                                                           *
 *      Purpose: Calculate the sampling factors need to load an image into   *
 *               a memory window.                                            *
 *                                                                           *
 *      By: D. Hollaren, USGS/EROS Data Center, 9/11/85                      *
 *              Original development for DMS.                                *
 *      Modified by: D. Akkerman, USGS/EROS Data Center, 5/88                *
 *              DMS v3_0 retrocode.                                          *
 *      Modified by: L. Huewe, USGS/EROS Data Center, 9/88                   *
 *              Calculation of the number of lines and samples required for  *
 *              a particular subsampling factor was modified to reflect the  *
 *              change in the subsampling method made in cxddrop.            *
 *      Modified by D. Akkerman, USGS/EROS Data Center, 11/89.               *
 *              Adapted from the cxdsamp routine for use in the LAS Pixar    *
 *              prototype system.                                            *
 *      Modified by K. Sheffield, USGS/EROS Data Center, 7/90.               *
 *              Recoded the subsampling calculations.                        *
 *      Modified by D. Akkerman, USGS/EROS Data Center, 7/90.                *
 *              Adapted for use with the xlasdsp application.                *
 *                                                                           *
 *      Algorithm (flow):                                                    *
 *                                                                           *
 *      See in-line documentation.                                           *
 *                                                                           *
 *****************************************************************************/
int xid_samp
(
    int inpwdw[],       /* File window image is taken from. (I)         */
    int memwdw[],       /* Pane window image is to be loaded into. (I/O)*/
    int *skipl,         /* Sampling factor in the line direction. (O)   */
    int *skips          /* Sampling factor in the sample direction. (O) */
)

{
int     nl,     /* Number lines calculated with different sampling factors.   */
        ns;     /* Number samples calculated with different sampling factors. */


/* Decide the amount of input sampling needed in the line direction.    */

        *skipl = ((inpwdw[2] - 1) / memwdw[2]) + 1;
        nl = (inpwdw[2] - 1) / *skipl + 1;

/* Decide the amount of input sampling needed in the sample direction.  */

        *skips = ((inpwdw[3] - 1) / memwdw[3]) + 1;
        ns = (inpwdw[3] - 1) / *skips + 1;

/* Added to make the sampling factors the same in both the line and
   sample directions.  This may be commented out if both factors need
   not be the same.

   Determine whether to use the sampling factor for the number of lines
   or to use the sampling factor for the number of samples.             */

        if (*skipl > *skips)
                {
                /* Use the line sampling factor to calculate the number
                   of samples.                                          */

                *skips = *skipl;
                ns = (inpwdw[3] - 1) / *skips + 1;
                }
        else
                {
                /* Use the sample sampling factor to calculate the number
                   of lines.                                            */

                *skipl = *skips;
                nl = (inpwdw[2] - 1) / *skipl + 1;
                }

/* Now adjust the memory window to reflect the sampling being done.     */

memwdw[2] = nl;
memwdw[3] = ns;

return (E_SUCC);
}
