/*
 *  This is the zoom function source file for the zoom panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include "tiepts.h"
#include "pixman.h"
#include "bitmaps/crosshair.xbm"
#include "convert.h"
#include "manage_cursor.h"
#include "protos.h"

void zoom_clear(void)
{
   if (XtIsRealized(Shell_Zoom))
      {
      if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
         {
         XSetWindowBackground(tie.display,
                 XtWindow(zoom_refwin_bulletinboard),
                 tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
         XClearWindow(tie.display, XtWindow(zoom_refwin_bulletinboard));
         tie_zoom[REFERENCE].flags |= ZOOM_CLEAR;
         }

      XSetWindowBackground(tie.display,
                 XtWindow(zoom_srchwin_bulletinboard),
                 tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
      XClearWindow(tie.display, XtWindow(zoom_srchwin_bulletinboard));
      XLowerWindow(tie.display, XtWindow(Shell_Zoom));
      tie_zoom[SEARCH].flags |= ZOOM_CLEAR;
      tie_select.curref = FALSE;
      }
}

/*******************************************************************************
Purpose:
   This routine defines the zoom cursor as a single pixel.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void zoom_dotcursor
(
   Widget       w
)
{
   Cursor       dot_cursor;     /* Single pixel cursor                        */
   Pixmap       dot_pixmap;     /* Single pixel pixmap                        */

   dot_pixmap = XCreateBitmapFromData(tie.display,
                          RootWindow(tie.display, tie.screen),
                          crosshair_bits, crosshair_width, crosshair_height);
   
   dot_cursor = XCreatePixmapCursor(tie.display, 
                      dot_pixmap, 
                      dot_pixmap, 
                      &tie_cursor.color, &tie_cursor.color, 7, 7);

   if (w == Shell_Zoom)
   {
      if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
      {
         set_window_cursor(tie.display,XtWindow(zoom_refwin_bulletinboard),
                           dot_cursor);
      }
      
      set_window_cursor(tie.display,XtWindow(zoom_srchwin_bulletinboard),
                        dot_cursor);
   }
   else
   {
      set_window_cursor(tie.display, XtWindow(w), dot_cursor);
   }

   XFreePixmap(tie.display, dot_pixmap);
}

/*******************************************************************************
Purpose:
   This routine defines the zoom cursor as a single pixel inside a box.

By J. Hemmer, USGS/EROS Data Center, 6/92

Algorithm Description:
*******************************************************************************/
void zoom_findercursor(void)
{
   GC           gc;
   XGCValues    values;         /* X Graphic Context values                   */
   Cursor       finder_cursor;  /* Single pixel cursor                        */
   Pixmap       finder_pixmap;  /* Single pixel pixmap                        */

   finder_pixmap = XCreatePixmap(tie.display,
                                 RootWindow(tie.display, tie.screen),
                                 11,11,1);
   values.plane_mask = 1;
   gc = XCreateGC(tie.display, finder_pixmap, GCPlaneMask, &values);

   XSetLineAttributes(tie.display,gc,0, LineSolid,CapRound,JoinRound);
   XSetForeground(tie.display,gc,0);
   XFillRectangle(tie.display,finder_pixmap, gc, 0, 0, 10, 10);
   XSetForeground(tie.display,gc,1);
   XDrawRectangle(tie.display,finder_pixmap, gc, 0, 0, 10, 10);
   XDrawPoint(tie.display,finder_pixmap, gc, 5, 5);

   finder_cursor = XCreatePixmapCursor(tie.display, finder_pixmap,
                                   finder_pixmap, &tie_cursor.color,
                                   &tie_cursor.color, 5, 5);

   if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
   {
      set_window_cursor(tie.display,XtWindow(zoom_refwin_bulletinboard),
                        finder_cursor);
   }
   
   set_window_cursor(tie.display,XtWindow(zoom_srchwin_bulletinboard),
                     finder_cursor);

   XFreePixmap(tie.display, finder_pixmap);
   XFreeGC(tie.display, gc);
}

/*******************************************************************************
Purpose:
    This routine initializes the parameter structure used later by the
    routine zoom_enlarge.  Checks are made to be sure that enough data exists
    to extract the amount required by the enlargement routine.

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
void zoom_create(void)
{
   tie_zoom[REFERENCE].image_pixmap =  NULL;
   tie_zoom[SEARCH].image_pixmap =  NULL;
   tie_zoom[REFERENCE].image_data = NULL;
   tie_zoom[SEARCH].image_data = NULL;

  /*
   *  ***  Add appropriate event handlers if the cursor window is active.
   */
   if ((tie_cursor.flags & CURSOR_ACTIVE) && (tie_cursor.flags & CURSOR_MAN))
      {
      if (zoom_refwin_bulletinboard != NULL)
         XtAddEventHandler(zoom_refwin_bulletinboard,
                        ButtonReleaseMask|KeyReleaseMask,
                        FALSE,cursor_track,NULL);
      if (zoom_srchwin_bulletinboard != NULL)
         XtAddEventHandler(zoom_srchwin_bulletinboard,
                        ButtonReleaseMask|KeyReleaseMask,
                        FALSE,cursor_track,NULL);
      }
   else if ((tie_cursor.flags & CURSOR_ACTIVE) && 
            (tie_cursor.flags & CURSOR_AUTO))
      {
      if (zoom_refwin_bulletinboard != NULL)
         XtAddEventHandler(zoom_refwin_bulletinboard,
              ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,
              FALSE,cursor_track, NULL);
      if (zoom_srchwin_bulletinboard != NULL)
         XtAddEventHandler(zoom_srchwin_bulletinboard,
             ButtonReleaseMask|PointerMotionMask|KeyReleaseMask,
             FALSE,cursor_track, NULL);
      }

  /*
   *  *** Add appropriate event handlers if the full res window is active.
   */
   if (fullref_win_bulletinboard != NULL)
      XtAddEventHandler(fullref_win_bulletinboard,ButtonReleaseMask,FALSE,
           zoom_button,NULL);

   if (fullsrch_win_bulletinboard != NULL)
      XtAddEventHandler(fullsrch_win_bulletinboard,ButtonReleaseMask,FALSE,
           zoom_button,NULL);

  /*
   *  ***  Set active flag.
   */
   tie_zoom[REFERENCE].flags = ZOOM_ACTIVE;
   tie_zoom[SEARCH].flags = ZOOM_ACTIVE;
   tie_zoom[REFERENCE].flags |= ZOOM_CLEAR;
   tie_zoom[SEARCH].flags |= ZOOM_CLEAR;
}

/*******************************************************************************
Purpose:
    This routine determines the action to take based on the mouse button 
    selected.  Button 1 pressed in the main image window indicates the zoomed
    window is to be updated.  Buttons 2 and 3 pressed in any TIE window 
    increase and decrease the zoom factor respectively.
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
void zoom_button
(
   Widget        w,
   XtPointer client,
   XEvent*   event,
   Boolean*  continue_to_dispatch
)
{
   double       rs, rl,         /* reference image line and sample            */
                ss, sl;         /* search image line and sample               */
   int          templ,          /* ref line value                             */
                temps;          /* ref sample value                           */
   double       tempx,          /* digitizer x coord                          */
                tempy;          /* digitizer y coord                          */
   double       tmpll[2];       /* Temp holder of lat/long values             */

  /*
   *   *** verify existance of data to zoom
   */
   if (w == fullref_win_bulletinboard) 
      {
      if ((tie_ref_fullres == NULL) || 
          (tie_ref_fullres[REFERENCE].image_data == NULL))
         return;

      tie_zoom[REFERENCE].flags = tie_zoom[REFERENCE].flags 
                                    & ~(ZOOM_MAIN | ZOOM_CLEAR)
                                    | ZOOM_FULLRES;
      }
   else if (w == fullsrch_win_bulletinboard)
      {
      if ((tie_srch_fullres == NULL) 
         || (tie_srch_fullres[REFERENCE].image_data == NULL))
         return;
      tie_zoom[SEARCH].flags = tie_zoom[SEARCH].flags 
                                       & ~(ZOOM_MAIN | ZOOM_CLEAR)
                                       | ZOOM_FULLRES;
      }
   else if (w == ref_image_bulletinboard) 
      { 
      tie_zoom[REFERENCE].flags = tie_zoom[REFERENCE].flags
                                       & ~(ZOOM_FULLRES | ZOOM_CLEAR)
                                       | ZOOM_MAIN;
      }
   else if (w == search_image_bulletinboard)
      {
      tie_zoom[SEARCH].flags = tie_zoom[SEARCH].flags
                                       & ~(ZOOM_FULLRES | ZOOM_CLEAR)
                                       | ZOOM_MAIN;
      }
   
  /*
   *  *** display zoom window if not present, and define dot cursor
   */
   if (!XtIsRealized(Shell_Zoom))
      {
      XtRealizeWidget(Shell_Zoom);
      zoom_dotcursor(Shell_Zoom);
      }
   else
      XRaiseWindow(tie.display, XtWindow(Shell_Zoom));
   
  /*
   *  *** If button 1 was selected in the main image window, 
   *              update the zoomed window.
   */
   if ((event->xbutton.type == KeyRelease) || (event->xbutton.button == 1)) 
       {
       if ((w == ref_image_bulletinboard) ||
           ((w == fullref_win_bulletinboard) && (tie_ref_fullres != NULL)))
          {
          if ((tie.mode != IM2MAP) && (tie.mode != IM2VECT))
             {
             tie_zoom[REFERENCE].x = event->xbutton.x;
             tie_zoom[REFERENCE].y = event->xbutton.y;

             if (w == ref_image_bulletinboard)
                zoom_image(w,zoom_refwin_bulletinboard,
                           tie_planes[tie.rview].dspwin);
             else if (!(tie.flags & TIE_ADRG))
                {
                zoom_image(w,zoom_refwin_bulletinboard,
                           tie_ref_fullres[REFERENCE].dspwin);
                display_zoomoutline(REFERENCE);
                }
             }
   
         /*
          *  *** if computed, apply rough transformation 
          */
          if (tie_rt.calc_rt != RT_SEL)
             {
            /* 
             *  *** compute search line and sample and warp pointer
             */
             if (tie.mode == IM2MAP)
                {
                if (tie_parms.manual)
                   {
                   rs = tie_select.rtie[tie_select.seltie][0];
                   rl = tie_select.rtie[tie_select.seltie][1];
                   }
                else
                   {
                   tempx = (double)event->xbutton.x;
                   tempy = (double)event->xbutton.y;
                   rs = c_eval(&tie_mapparm.order, tie_mapparm.coef_x,
                            &tempx, &tempy);
                   rl = c_eval(&tie_mapparm.order, tie_mapparm.coef_y,
                             &tempx, &tempy);
                   }
                }
             else
                {
                getlinesamp(w, tie_zoom[REFERENCE].x, tie_zoom[REFERENCE].y,
                            &temps, &templ);
                if (tie_parms.flags & PARMS_LINESAMP)
                  {
                  rs = (double)temps;
                  rl = (double)templ;
                  }
                else
                  {
                  image2proj(REFERENCE,tmpll,(double)temps,(double)templ);
                  if (tie_parms.flags & PARMS_LATLONG)
                    proj2degrees(tmpll,1,REFERENCE);
                  rs = tmpll[0];
                  rl = tmpll[1];
                  }
                }
   
             ref2srch(rs, rl, &ss, &sl);

            /*
             *  *** perform search zoom
             */
             if (w == fullref_win_bulletinboard)
                {
                if (!(tie.flags & TIE_RESELECTING))
                   {
                   tie_zoom[SEARCH].x = ss-tie_srch_fullres[0].window[SS];
                   tie_zoom[SEARCH].y = sl-tie_srch_fullres[0].window[SL];
                   tie_zoom[SEARCH].flags = tie_zoom[SEARCH].flags
                                       & ~(ZOOM_MAIN | ZOOM_CLEAR)
                                       | ZOOM_FULLRES;
                   zoom_image(fullsrch_win_bulletinboard,
                              zoom_srchwin_bulletinboard,
                              tie_srch_fullres[0].dspwin);
                   display_zoomoutline(SEARCH);
   
                   full2zoom(ss, sl, &temps, &templ, 
                             tie_srch_fullres[0].window[SL],
                             tie_srch_fullres[0].window[SS]);
                   }
                }
             else
                {
                tie_zoom[SEARCH].x = ss - tie_planes[tie.sview].window[SS];
                tie_zoom[SEARCH].y = sl - tie_planes[tie.sview].window[SL];
                tie_zoom[SEARCH].flags = tie_zoom[SEARCH].flags
                                       & ~(ZOOM_FULLRES | ZOOM_CLEAR)
                                       | ZOOM_MAIN;
                zoom_image(search_image_bulletinboard,
                           zoom_srchwin_bulletinboard,
                           tie_planes[tie.sview].dspwin);
 
                temps = templ = tie_parms.factor*tie_parms.zsize/2;
                }

   
             if (!(tie.flags & TIE_ADRG))
                XWarpPointer(tie.display, None, 
                    XtWindow(zoom_refwin_bulletinboard),
                    None, None, None, None, temps, templ);
             else
                XWarpPointer(tie.display, None, 
                    XtWindow(zoom_srchwin_bulletinboard),
                    None, None, None, None, temps, templ);
             }
         /*
          *  *** if no RT, warp pointer and clear search zoom window
          */
          else
             {
             if (w == fullref_win_bulletinboard)
                XWarpPointer(tie.display, None, 
                       XtWindow(fullsrch_win_bulletinboard),
                       None, None, None, None,
                       event->xbutton.x, event->xbutton.y);
             else
                XWarpPointer(tie.display, None, 
                       XtWindow(search_image_bulletinboard),
                       None, None, None, None,
                       event->xbutton.x, event->xbutton.y);
   
             XSetWindowBackground(tie.display,
                       XtWindow(zoom_srchwin_bulletinboard),
                       tie.color_table[RGBBW_IDX + BLACK_COL].pixel);
             XClearWindow(tie.display, XtWindow(zoom_srchwin_bulletinboard));
             }
          }
      /*
       *  ***  if search window
       */
       else if ((w == search_image_bulletinboard) ||
          ((w == fullsrch_win_bulletinboard) && (tie_srch_fullres != NULL)))
          {
         /*
          *  *** for some reason, values above 0x7fff come out negative????
          */
          if ((tie.mode == IM2MAP) && (!tie_parms.manual))
             {
             event->xbutton.x &= 0x0000ffff;
             event->xbutton.y &= 0x0000ffff;
             }
   
          tie_zoom[SEARCH].x = event->xbutton.x;
          tie_zoom[SEARCH].y = event->xbutton.y;
     
         /*
          *  *** if search is not subsampled, zoom from overview
          *         otherwise zoom from fullres and display outline
          */
          if (w == search_image_bulletinboard)
             zoom_image(w,zoom_srchwin_bulletinboard,
                           tie_planes[tie.sview].dspwin);
          else
             {
             zoom_image(w,zoom_srchwin_bulletinboard, 
                           tie_srch_fullres[0].dspwin);
             display_zoomoutline(SEARCH);
             }

          if (tie.mode == IM2VECT)
             {
             if (tie.flags & TIE_RESELECTING)
                {
                rs = tie_select.rtie[tie_select.reselect][0];
                rl = tie_select.rtie[tie_select.reselect][1];
                ref2srch(rs, rl, &ss, &sl);
                }
             else
                {
                /* tjr
                if (!(tie_select.rtemp_flag))
                   {
                   */
                   ss = (double)event->xbutton.x + 
                                tie_srch_fullres[0].window[SS];
                   sl = (double)event->xbutton.y +
                                tie_srch_fullres[0].window[SL];
                   /* tjr
                   }
                else
                   {
                   rs = tie_select.rtemp[0];
                   rl = tie_select.rtemp[1];
                   ref2srch(rs, rl, &ss, &sl);
                   }
                   */
                }
             }
          else
             {
             ss = (double)event->xbutton.x + tie_srch_fullres[0].window[SS];
             sl = (double)event->xbutton.y + tie_srch_fullres[0].window[SL];
             }

          if ((tie.mode == IM2IM) && !(tie.flags & TIE_ADRG))
             {
             if (!(tie.flags & TIE_RESELECTING))
                XWarpPointer(tie.display, None,
                       XtWindow(zoom_refwin_bulletinboard),
                       None, None, None, None,
                       tie_parms.factor*tie_parms.zsize/2, 
                       tie_parms.factor*tie_parms.zsize/2);
             }
          else
             {
             full2zoom(ss, sl, &temps, &templ, 
                 tie_srch_fullres[0].window[SL],
                 tie_srch_fullres[0].window[SS]);

             XWarpPointer(tie.display, None, 
                       XtWindow(zoom_srchwin_bulletinboard),
                       None, None, None, None, temps, templ);
             }
          }
       }
}
   

/*******************************************************************************
Purpose:
    This routine allocates the appropriate buffers and initializes the 
    appropriate fields in the zoom parameter structure in preparation for
    the enlargement process.
                                                                
By D. Steinwand, USGS/EROS Data Center, 3/89

Algorithm Description:
    This routine was taken from the LAS TIEPTS application.

Algorithm References:
  1.  D.E. Friedmann, "Two-Dimensional Resampling of Line Scan Imagery
       by One-Dimensional Processing," Photogrammetric Engineering and
       Remote Sensing, Vol. 47, No. 10, pp. 1459-1467, October 1981.
  2.  D.E. Friedmann, "Operational Resampling for Correcting Images
       to a Geocoded Format," MacDonald, Dettwiler & Associates Ltd.,
       Richmond, B.C., Canada, May 1981.
  3.  S.K. Park and R.A. Schowengerdt, "Image Reconstruction by
       Parametric Cubic Convolution," Computer Vision, Graphics,
       and Image Processing 23, Academic Press, Inc. 1983. pp. 258-272.
*******************************************************************************/
int zoom_init
(
    struct ENLARGE *zoom,    /* Enlargement parameters & buffers (I/O)   */
    int view                 /* image display view                          */
)
{
   struct 
   RWTAB        user_rwt;       /* User entered resampling weight table.      */
   static int first[2] = {TRUE, TRUE};  
                                /* Flag--has init_zoom been called before?    */
   int          *index;         /* Weight table index.                        */
   float        tmpdel;         /* Temporary delta value.                     */
   float        delta;          /* Delta value; I (loop counter) * FACTOR.    */
   float        inc;            /* Table increment for table-based resampling.*/
   float        inv;            /* 1.0 / zoom->factor.                        */
   float        offset;         /* Offset for even zoom factors.              */
   int          count;          /* Index incrementer for nn table calculation.*/
   int          i,j,k;          /* Loop indices.                              */
   size_t       size;           /* Size of weight table buffer.               */

  /*
   *  ***  If this is not the first time this routine has been called, 
   *           free any previously allocated buffer space.
   */
   if (!(first[view]))
      {
      free(zoom->horiz_table);
      free(zoom->work);
      free(zoom->input);
      free(zoom->output);
      }
   else
      first[view] = FALSE;

  /*
   *  *** Calculate 1/zoom->factor.
   */
   inv = 1.0 / (float) tie_parms.factor;

  /*
   *  ***  Nearest neighbor table initialization.
   */
   if (strcmp(tie_parms.method, "NN") == 0)
      {
      size = 2 * tie_parms.factor * sizeof(int);
      zoom->horiz_table = zoom->vert_table = (int *)malloc(size);
      if (zoom->horiz_table == NULL) 
           {
           tie_message(Shell_Main, "Insufficient memory available");
           return(TIE_F);
           }
      count = tie_parms.factor / 2;
      if ((float)count < ((float)tie_parms.factor / 2.0)) count++;
      for (index = zoom->horiz_table, i = count; i--;)
           {
           *index++ = 256;
           *index++ = 0;
           }
      for (i = tie_parms.factor - count; i--;)
           {
           *index++ = 0;
           *index++ = 256;
           }
      zoom->horiz_dim = zoom->vert_dim = 2;
      zoom->extract_lines = zoom->in_lines + 1;
      zoom->extract_samps = zoom->in_samps + 1;
      }

  /*
   *  *** Parametric cubic convolution table initialization.
   */
   if (strcmp(tie_parms.method, "CC") == 0)
      {
      size = 4 * tie_parms.factor * sizeof(int);
      zoom->horiz_table = zoom->vert_table = (int *)malloc(size);
      if (zoom->horiz_table == NULL) 
           {
           tie_message(Shell_Main, "Insufficient memory available");
           return(TIE_F);
           }
   
      count = tie_parms.factor / 2;
      if ((float)count < ((float)tie_parms.factor / 2.0)) offset = 0;
      else offset = 1.0 / (2.0 * tie_parms.factor);
      for(index = zoom->horiz_table, i = 0; i < tie_parms.factor; i++)
           {
           delta = (i * inv) + offset;
           tmpdel = 1 - delta;
           *index++ = 256 * tie_parms.alpha * delta * SQUARE(tmpdel);
           *index++ = 256 * (1.0 - ((tie_parms.alpha+3.0) * SQUARE(delta)) +
              ((tie_parms.alpha + 2.0) * CUBE(delta)));
           *index++ = 256 * (1.0 - ((tie_parms.alpha+3.0) * SQUARE(tmpdel)) +
              ((tie_parms.alpha+2.0) * CUBE(tmpdel)));
           *index++ = 256 * tie_parms.alpha * tmpdel * SQUARE(delta);
           }
      zoom->horiz_dim = zoom->vert_dim = 4;
      zoom->extract_lines = zoom->in_lines + 3;
      zoom->extract_samps = zoom->in_samps + 3;
      }
   
  /*
   *  ***  Bilinear interpolation table initialization.
   */
   if (strcmp(tie_parms.method, "BI") == 0)
      {
      size = 2 * tie_parms.factor * sizeof(int);
      zoom->horiz_table = zoom->vert_table = (int *)malloc(size);

      if (zoom->horiz_table == NULL) 
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return(TIE_F);
         }
   
      count = tie_parms.factor / 2;
      if ((float)count < ((float)tie_parms.factor / 2.0)) 
         offset = 0;
      else 
         offset = 1.0 / (2.0 * tie_parms.factor);

      for (index = zoom->horiz_table, i = 0; i < tie_parms.factor; i++)
         {
         delta = (i * inv) + offset;
         *index++ = 256 * (1 - delta);
         *index++ = 256 * delta;
         }

      zoom->horiz_dim = zoom->vert_dim = 2;
      zoom->extract_lines = zoom->in_lines + 1;
      zoom->extract_samps = zoom->in_samps + 1;
      }

  /*
   *  ***  User entered table-based resampling table read & initialization.
   */
   if (strcmp(tie_parms.method, "TABLE") == 0)
      {
      if (c_getrwt(zoom->table_file, &user_rwt) != E_SUCC) 
         return(E_FAIL);

      zoom->horiz_dim = user_rwt.dim[0];
      zoom->vert_dim = user_rwt.dim[1];
      zoom->extract_lines = zoom->in_lines + (zoom->vert_dim - 1);
      zoom->extract_samps = zoom->in_samps + (zoom->horiz_dim - 1);
      size=(zoom->horiz_dim * tie_parms.factor * sizeof(int)) +
            (zoom->vert_dim * tie_parms.factor * sizeof(int));
      zoom->horiz_table = (int *)malloc(size);

      if (zoom->horiz_table == NULL)
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return(TIE_F);
         }
      zoom->vert_table = zoom->horiz_table + 
                         (zoom->horiz_dim * tie_parms.factor);

      count = tie_parms.factor / 2;

      if ((float)count < ((float)tie_parms.factor / 2.0)) 
         offset = 0;
      else 
         offset = 32.0 / (2.0 * tie_parms.factor);
 
      index = zoom->horiz_table;
      inc = 32.0 / (float)tie_parms.factor;

      for (k = 0; k < tie_parms.factor; k++)
         for (j = 0,i = (k * inc) + offset; j < zoom->horiz_dim; j++)
            *index++ = 256 * user_rwt.table1[i][j];

      index = zoom->vert_table;
      for (k = 0; k < tie_parms.factor; k++)
         for (j = 0,i = (k * inc) + offset; j < zoom->vert_dim; j++)
            *index++ = 256 * user_rwt.table2[i][j];
      }

  /*
   *  ***  Allocate input, hybrid, and output space buffers.
   */
   zoom->hybrid_samps = zoom->in_samps * tie_parms.factor;

   size = zoom->extract_lines * zoom->extract_samps * sizeof(unsigned char);
   zoom->input = (unsigned char *)malloc(size);
   if (zoom->input == NULL) 
      {
      tie_message(Shell_Main, "Insufficient memory available");
      return(TIE_F);
      }

   size = zoom->extract_lines * zoom->hybrid_samps * sizeof(int);
   zoom->work = (int*)malloc(size);

   if (zoom->work == NULL) 
      {
      tie_message(Shell_Main, "Insufficient memory available");
      return(TIE_F);
      }

   size = zoom->in_lines * tie_parms.factor * 
          zoom->in_samps * tie_parms.factor * 
          sizeof(unsigned char);
   zoom->output = (unsigned char *)malloc(size);

   if (zoom->output == NULL) 
      {
      tie_message(Shell_Main, "Insufficient memory available");
      return(TIE_F);
      }

   return(E_SUCC);
}


/*******************************************************************************
Purpose:
    This routine extracts the appropriate image data, enlarges it based on the
    zoom factor and resampling method selected and displays it in the zoom
    image window.
                                                                
    TIEPTS:  Assume from fullres window (jjh 1/27/92)

By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
*******************************************************************************/
void zoom_image
(
    Widget parent,
    Widget w,
    int   dspwin[4]     /* Display window.                            */
)
{
   int  bands,                  /* Number of bands displayed.                 */
        dtype = EBYTE,          /* Data type.                                 */
        i,j,k,                  /* Looping indices.                           */
        plane_idx[3],           /* Memory plane indices of displayed image.   */
        nbytes,                 /* Number of bytes to copy.                   */
        nl,                     /* Number of lines in zoomed image.           */
        ns,                     /* Number of samples in zoomed image.         */
        nsamps,                 /* Number of samples in displayed image.      */
        dummy[3],               /* Dummy array to be passed to load_display.  */
        sl,                     /* Starting line of window to zoom.           */
        ss,                     /* Starting sample of window to zoom.         */
        view;                   /* Composite view index                       */

   unsigned char
        *imgptr,                /* Pointer to displayed image data.           */
        *inptr,                 /* Pointer to input array for enlarge routine.*/
        *zmptr;                 /* Pointer to zoomed data.                    */

  /*
   *  *** if image to vector, clear graphics
   */
   XClearArea(tie.display, XtWindow(w), 0, 0, 0, 0, True);

  /*
   *  ***  Determine the memory plane indices of the displayed image.
   */
   if (w == zoom_refwin_bulletinboard) 
      view = REFERENCE;
   else if (w == zoom_srchwin_bulletinboard) 
      view = SEARCH;
   
   tie_zoom[view].parms.in_lines = tie_parms.zsize;
   tie_zoom[view].parms.in_samps = tie_parms.zsize;

   plane_idx[RED] = tie_composite[view].rgbplanes[RED] - 1;
   plane_idx[GREEN] = tie_composite[view].rgbplanes[GREEN] - 1;
   plane_idx[BLUE] = tie_composite[view].rgbplanes[BLUE] - 1;
   
   zoom_init(&tie_zoom[view].parms, view);
   
  /*
   *   Continue resetting the zoom window size until zoom_init returns values
   *   to extract which fall within the displayed image limits.
   */
   while (tie_zoom[view].parms.extract_lines > dspwin[NL])
      {
      tie_zoom[view].parms.in_lines--;
      zoom_init(&tie_zoom[view].parms, view);
      }

   while (tie_zoom[view].parms.extract_samps > dspwin[NS])
      {
      tie_zoom[view].parms.in_samps--;
      zoom_init(&tie_zoom[view].parms, view);
      }

  /*
   *  ***  Determine the number of lines and samples in the zoomed image.
   */
   nl = tie_parms.factor * tie_zoom[view].parms.in_lines;
   ns = tie_parms.factor * tie_zoom[view].parms.in_samps;
   nbytes = nl * ns;

  /*
   *  *** Determine the starting line and sample of the window to be extracted.
   */
   ss = tie_zoom[view].x - tie_zoom[view].parms.extract_samps / 2;
   sl = tie_zoom[view].y - tie_zoom[view].parms.extract_lines / 2;

   if (sl < 0)
      {
      tie_zoom[view].y = tie_zoom[view].parms.extract_lines / 2 - 1;
      sl = 0;
      }
   if (ss < 0)
      {
      tie_zoom[view].x = tie_zoom[view].parms.extract_samps / 2 - 1;
      ss = 0;
      }

  /*
   *  *** Reset starting line and sample if the window falls outside 
   *      the displayed image.
   */
   if ((sl + tie_zoom[view].parms.extract_lines) > dspwin[NL])
      {
      sl = dspwin[NL] - tie_zoom[view].parms.extract_lines;

      if (sl < 0)
         sl = 0;

      tie_zoom[view].y = sl +  tie_zoom[view].parms.extract_lines / 2;
      }

   if ((ss + tie_zoom[view].parms.extract_samps) > dspwin[NS])
      {
      ss = dspwin[NS] - tie_zoom[view].parms.extract_samps;
      if (ss < 0)
         ss = 0;

      tie_zoom[view].x = ss + tie_zoom[view].parms.extract_samps / 2;
      }

  /*
   *  *** Free the previously allocated zoomed image pixmap.
   */
   if (tie_zoom[view].image_pixmap != NULL)
      XFreePixmap(tie.display,tie_zoom[view].image_pixmap);

  /*
   *  ***  Free previously allocated resources.
   */
   if (tie_zoom[view].image_data != NULL)
      free(tie_zoom[view].image_data);

  /*
   *  ***  Allocate buffer space for the output image.
   */
   if ((tie_composite[view].rgbplanes[RED] ==
       tie_composite[view].rgbplanes[GREEN]) && (
       tie_composite[view].rgbplanes[GREEN] ==
       tie_composite[view].rgbplanes[BLUE]))
      {
      bands = 1;
      tie_zoom[view].image_data = (unsigned char *)malloc(nbytes);

      if (tie_zoom[view].image_data == NULL)
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return;
         }
      }
   else
      {
      bands = 3;
      tie_zoom[view].image_data = (unsigned char *)malloc(nbytes * bands); 
      if (tie_zoom[view].image_data == NULL)
         {
         tie_message(Shell_Main, "Insufficient memory available");
         return;
         }
      }

  /*
   *  ***  Zoom the extracted image window.
   */
   for (i=0,zmptr = tie_zoom[view].image_data; 
        i < bands; i++, zmptr += (nl * ns))
      {
      if (plane_idx[i] >= 0)
         {
        /*
         *  *** Extract the appropriate window.
         */
         inptr = tie_zoom[view].parms.input;
         if (parent == ref_image_bulletinboard) 
            imgptr = tie_planes[tie.rview+i].image_data;
         else if (parent == search_image_bulletinboard) 
            imgptr = tie_planes[tie.sview+i].image_data;
         else if (parent == fullref_win_bulletinboard)
            imgptr = tie_ref_fullres[i].image_data;
         else if (parent == fullsrch_win_bulletinboard)
            imgptr = tie_srch_fullres[i].image_data;

         nsamps = dspwin[NS];
         for (j=sl; j < sl + tie_zoom[view].parms.extract_lines; j++)
            for (k=ss; k < ss + tie_zoom[view].parms.extract_samps; k++,inptr++)
                *inptr = *(imgptr + (j * nsamps + k));
        /*
         *  Enlarge the extracted image data based on the previously selected 
         *  zoom factor and resampling method.
         */
         zoom_enlarge(&tie_zoom[view].parms);
         c_pxcopy(tie_zoom[view].parms.output,zmptr,&dtype,&nbytes);
         }
      }

  /*
   *  ***  Load the zoomed data to the appropriate window.
   */
   if (bands == 1)
      {
      load_display(dummy,&bands,XtWindow(w),Shell_Zoom,
           w,&nl,&ns,&tie_planes[plane_idx[RED]].
           maxval,tie_zoom[view].parms.output,NULL,NULL,FALSE,&tie_zoom[view].
           image_pixmap,FALSE);
      }
   else
      {
      load_display(dummy,&bands,XtWindow(w),Shell_Zoom,
            w,&nl,&ns,&tie_planes[plane_idx[RED]].
            maxval,tie_zoom[view].image_data,tie_zoom[view].image_data+nbytes,
            tie_zoom[view].image_data+(2*nbytes),FALSE,
            &tie_zoom[view].image_pixmap,FALSE);
      }

   tie_zoom[view].parms.in_lines = tie_parms.zsize;
   tie_zoom[view].parms.in_samps = tie_parms.zsize;
}

/*******************************************************************************
Purpose:  
    This routine enlarges a window using an interpolated zoom.  Resampling 
    techniques include nearest neighbor, bilinear interpolation,
    parametric cubic convolution, and a user entered table of
    resampling weights.

By D. Steinwand, USGS/EROS Data Center, 3/89

Algorithm Description:
    This routine was taken from the LAS TIEPTS application.

Algorithm References:
  1.  D.E. Friedmann, "Two-Dimensional Resampling of Line Scan Imagery
       by One-Dimensional Processing," Photogrammetric Engineering and
       Remote Sensing, Vol. 47, No. 10, pp. 1459-1467, October 1981.
  2.  D.E. Friedmann, "Operational Resampling for Correcting Images
       to a Geocoded Format," MacDonald, Dettwiler & Associates Ltd.,
       Richmond, B.C., Canada, May 1981.
  3.  S.K. Park and R.A. Schowengerdt, "Image Reconstruction by
       Parametric Cubic Convolution," Computer Vision, Graphics,
       and Image Processing 23, Academic Press, Inc. 1983. pp. 258-272.
*******************************************************************************/
void zoom_enlarge
(
    struct ENLARGE *zoom        /* Enlargement parameters & buffers (I/O) */
)
{
  /*
   *  ***  Horizontal pass.
   */
   horiz_zoom(zoom);

  /*
   *  ***  Vertical pass.
   */
   vert_zoom(zoom);

   return;
}

/*******************************************************************************
Purpose:
    This routine performs interpolated enlargement in horizontal direction.
                                                                
By D. Steinwand, USGS/EROS Data Center, 3/89 

Algorithm Description:
    This routine was taken from the LAS TIEPTS application.
*******************************************************************************/
void horiz_zoom
(
    struct ENLARGE *zoom        /* Enlargement parameters & buffers (I/O) */
)
{
   int          *hybrid;        /* Pointer to current pixel in hybrid space.  */
   int          *weight;        /* Pointer to current weight table location.  */
   unsigned char *in_buf;       /* Pointer to current input pixel.            */
   int          h, i, j, k;     /* Loop counters.                             */
   int          pad;            /* Pad so extract_lines = in_lines - pad.     */

  /*
   *  ***  Initialize offsets and pointers to buffers.
   */
   pad = zoom->extract_lines - zoom->in_lines;
   hybrid = zoom->work;
   in_buf = zoom->input;

/*
 * Perform the horizontal resampling.  The resampling type is determined
 * in zoom_init() and is placed in a table.  This routine applies the table,
 * regardless of the resampling method chosen (yes, even nearest neightbor).
 * The table is scaled up by 8 bits (multiplied by 256) to keep some 
 * precision beyond the radix point and to allow the elimination of all
 * floating point calculations.  The removal of the floating point calculations
 * sped this algorithm up significantly.  The original range of the data
 * in the hybrid space is not restored until the completion of the vertical
 * pass.
 */
   for (h = zoom->extract_lines; h--; in_buf += pad)
      for (i = zoom->in_samps; i--; in_buf++)
         for (weight = zoom->horiz_table, j = tie_parms.factor; j--; hybrid++)
            for (*hybrid = 0, k = 0; k < zoom->horiz_dim; k++)
               *hybrid += *(in_buf + k) * *weight++;
   return;
}

/*******************************************************************************
Purpose:
    This routine performs interpolated enlargement in vertical direction.
                                                                
By D. Steinwand, USGS/EROS Data Center, 3/89

Algorithm Description:
    This routine was taken from the LAS TIEPTS application.
*******************************************************************************/
void vert_zoom
(
    struct ENLARGE *zoom        /* Enlargement parameters & buffers (I/O) */
)
{
   int          temp;           /* Temporary floating point output result.    */
   int          *hybrid;        /* Pointer to current pixel in hybrid space.  */
   int          *weight;        /* Pointer to current weight table location.  */
   unsigned char 
                *out_buf;       /* Pointer to current output pixel.           */
   int          inc;            /* Loop counter.                              */
   int          h, i, j, k;     /* Loop counters.                             */

/*
 *  The vertical resampling pass is very similar to the horizontal resampling
 *  pass, however, the added complexity in the code comes from accessing the
 *  data in columns instead of rows (lines).  As in the horizontal pass, the
 *  weight table is scaled by 8 bits to remove floating point calculations.
 *  Since this pass and the horizontal pass involve multiplication of scaled
 *  values, the final result must be adjusted by performing an integer division
 *  by 16 bits (8 bits applied in the horizontal pass and 8 bits applied in
 *  the vertical pass).  A bit shift would be more efficient, but probably not
 *  portable, so it is avoided.
 */
   for (h = 0; h < zoom->hybrid_samps; h++)
      {
      out_buf = zoom->output + h;
      hybrid = zoom->work + h;
      for (i = zoom->in_lines; i--; hybrid += zoom->hybrid_samps)
         for (weight=zoom->vert_table,j=tie_parms.factor;j--;
              out_buf+=zoom->hybrid_samps)
            {
            temp = 0;
            for (inc = 0, k = 0; k < zoom->vert_dim; k++, 
                 inc += zoom->hybrid_samps)
               temp += *(hybrid + inc) * *weight++;

            if (temp < 65536) 
               *out_buf = 0;
            else if (temp > 16711680) 
               *out_buf = 255;
            else 
               *out_buf = temp / 65536;
         }
      }
}

