#include "topo.h"
#include "util.h"
#include "protos.h"

void compute
(
    long bands[][MAXBND+1],	/* band numbers for the input image */
    long nbands[],		/* number of band in input image */
    long opt, 			/* option code(1-3) */
    long nimages,          	/* no. of images */
    long num_inbands,      	/* no. of bands */
    long wind[][4],        	/* window parameter */
    float cparm,			/* contour parameter */
    long otype,			/* output data type */
    long method, 		/* method no. (1-4) */
    long nbc,			/* neighborhood size code(1-2) */
    float sampdist,		/* column distance */
    float linedist,    		/* row distance */
    float aoff,      		/* mask value */
    long msgtim,			/* no. of elapse seconds */
    long itype, 			/* input image data type */
    float alpha,			/* sun's elevation angle */
    float gamma			/* azimuth angle in degrees */
)

{

/*
--------------------------------------------------------------------------

NAME: COMPUTE

FUNCTION:   Computes aspect, slope or contour values and writes results 
            to the output file.

PROGRAM HISTORY:

  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       03/20/84     M. Nishihama      
    2.0       03/24/87     R. Mckinney      Prf 4207
    3.0       02/18/89     B. Allam         Conversion
    4.0       07/20/89     B. Allam         Passed itype
    5.0       07/19/90     K. Narayanan     Corrected calling sequence for
                                            process
    6.0       08/22/90     K. Kunska        Added more detailed documentation
    7.0	      11/13/90	   D. Akkerman	    Fixed swapped linedist and
					    sampdist in the calling of the
					    process routine.
ALGORITHM:

   Initialize variables.
   Initialize and enable processing message utility
   For each image, 
      For each band
         If subcommand is "beta", 
            Call PROCESS to compute the beta angles.
         If Subcommand is either "slope" or "aspect", 
            Call SACOMP to generate a slope image or an aspect image.
         If subcommand is "contour", 
            Call CONTOUR to generate a contour image.
      End-For
   End-For
   Close the input and output files

---------------------------------------------------------------------------
*/

long    bnd,		/* loop counter            */
	i,		/* loop counter            */
	img,		/* loop counter            */
	inband,		/* input band number       */
	msg_flag,       /* format flag             */
        nlf,		/* no. of lines of input image */
	npf,		/* no. of pixels per line  */
	outband;	/* output band number      */

/*
Initialize variables.
----------------------
*/
img = 0;
npf = wind[img][NS];
nlf = wind[img][NL];

/*
Initialize and enable processing message utility
--------------------------------------------------
*/
msg_flag = ONECAT;
c_inittm ("TOPO", "Processed", "line", "band", " ", &wind[0][NL], 
           &num_inbands, &msgtim, &msg_flag);
curent = 1;
curcat = 1;
c_startm();
outband = 0;

/*
For each input image and each band:
   If subcommand is "beta", call PROCESS to compute the beta angles.
   If Subcommand is either "slope" or "aspect", call SACOMP to generate a
   slope image or an aspect image.
   If subcommand is "contour", call CONTOUR to generate a contour image.
-----------------------------------------------------------------------
*/
for (img = 0; img < nimages; img++)
   {
   for (bnd = 1; bnd <= nbands[img]; bnd++)
      {
      curent = 1;
      inband = bands[img][bnd-1];
      outband++;
      if (opt == 4)
         {
         process (npf, nlf, method, alpha, gamma, inband, outband, sampdist, 
                  linedist, aoff, otype, img);
         }
      else if (opt <= 2)
         {
         sacomp (method, opt, npf, nlf, nbc, sampdist, linedist, aoff, img, 
                 inband, outband, itype);
         }
      else
         {
         contour (npf, nlf, cparm, otype, img, inband, outband);
         }
      curcat++;
      }
   }

c_stoptm();

/*
Close the input and output files
------------------------------------
*/
for (i=0; i < nimages; i++)
  c_eclose (&fdesc[i]);
c_eclose (&fdescout);

}
