#include "topo.h"
#include "protos.h"

void contour
(
    long np,			/* number  of pixels per line                */
    long nl,			/* number of lines in the input image        */
    float cont_val,		/* contour parameter                         */
    long otype,			/* output data type                          */
    long img,		        /* curent image being processed              */
    long bnd,      		/* input band number                         */
    long obnd			/* output image band number                  */
)

/*-----------------------------------------------------------------------------

NAME:	CONTOUR

FUNCTION:   Reads input image data and computes contour values at each point.
            The contour values are written to the output image.

PROGRAM HISTORY:

  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       03/20/84     M. Nishihama      
    2.0       03/24/87     R. Mckinney      Prf 4207
    3.0       02/18/89     B. Allam         Conversion
    4.0       07/20/89     B. Allam         Changed pval type to double
    5.0       08/22/90     K. Kunska        Added more detailed documentation

ALGORITHM:

   Allocate dynamic memory for the input and output buffers.
   For each line 
      For each pixel, 
         Read an input image line and 
         Compute the contour pixel value.
         Fit the output pixel value to the output image data type.
      End-fo
      Write curent line to the output image.
   End-do
   Free the dynamic memory

----------------------------------------------------------------------------*/
{

static	long one = 1;

long	cur_lin,	   	/* current line                             */
	cur_pix;		/* current pixel                            */

float	*inbuf,			/* image input buffer                       */
	*outbuf,		/* image output buffer                      */
	pix_val;		/* input pixel value 	                    */

double	cval;			/* contour value                            */

/*
Allocate dynamic memory for the input and output buffers.
---------------------------------------------------------
*/
if((inbuf = (float *) malloc(np * sizeof(float))) == NULL)
   msg_err("!Error: cannot allcate dynamic memory", "topo-alloc");
  
if((outbuf = (float *) malloc(np * sizeof(float))) == NULL)
   msg_err("!Error: cannot allcate dynamic memory", "topo-alloc");

/*
For each line and each pixel, read an input image line and 
compute the contour pixel value.
Fit the output pixel value to the output image data type.
---------------------------------------------------------------
*/
for (cur_lin = 1; cur_lin <= nl; cur_lin++) 
   {
   curent = cur_lin;
   c_eread (&fdesc[img], &bnd, &cur_lin, inbuf, &one);

   for (cur_pix = 0; cur_pix < np; cur_pix++) 
      {
      pix_val= inbuf[cur_pix];
      if (pix_val < 0.0) 
         pix_val  = 0.0;

      cval = (pix_val + cont_val - 0.00001) / cont_val;
      if (otype == EBYTE) 
         {
         if (cval >= 255.0) 
            cval = 254.0;          
         }
      else if (cval >= 32767.0) 
         cval = 32766.0; 

      outbuf[cur_pix] = cval;
      }

/*
Write curent line to the output image.
--------------------------------------
*/
    c_ewrite (&fdescout, &obnd, &cur_lin, outbuf, &one);
   }

/*
Free the dynamic memory
------------------------
*/
free(inbuf);
free(outbuf);

}

