#include "topo.h"
#include "protos.h"

void initio
(
    long bands[][MAXBND+1],	/* band numbers for each image   */
    long nbands[],		/* number of bands in each image */
    char hosin[][CMLEN],          /* input image host file name   */
    char hosout[],                /* output image host file name  */
    char taeout[],                /* output image tae file name   */
    long nimages,                /* no. of input images           */
    long wind[][4],              /* input image windows           */
    long totbnd,                 /* total no. of bands            */
    long otype 			/* output data type              */
)

{
/*
---------------------------------------------------------------------------
NAME: INITIO

FUNCTION:   Opens input and output images.

PROGRAM HISTORY:

  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       03/20/84     M. Nishihama      
    2.0       03/24/87     R. Mckinney      Prf 4207
    3.0       02/18/89     B. Allam         Conversion
    4.0       07/20/89     B. Allam         Changed types of i/o files
    5.0       08/22/90     K. Kunska        Added more detailed documetation

ALGORITHM:

   Open the input image file
   Open the output image file
   Update DDR

----------------------------------------------------------------------------
*/
extern  struct PARBLK block;
extern  struct FDESC *fdescout;
extern  char    out[1][CMLEN];
extern  long    num_files;

static  long zero = 0;		/* Value of zero		*/
static  long one  = 1;		/* Value of one                 */

double  y_inc, x_inc;           /* line and sample increments   */

long	access,			/* file acess type              */
	n,			/* counter variable             */
	optn[2],		/* optional parameter array     */
	stat,			/* return status                */
	type;			/* data type 			*/

/*
Open the input image file
--------------------------
*/
type = EREAL;
access = IREAD;
for (n = 0; n < nimages ; n++)
   {
   stat = c_eopenr(&fdesc[n], hosin[n], &nbands[n], &wind[n][SL],
                   &wind[n][SS], &wind[n][NL], &wind[n][NS], 
                   &access, &type, &zero, &one);
   if (stat != E_SUCC)
       msg_err("!Error opening input image", "topo_input");
   }
fdesc[++n] = 0;

/*
Open the output image file
--------------------------
*/
access = IWRITE;
optn[0] = otype;
optn[1] = 0;
stat = c_eopenr (&fdescout, hosout, &totbnd, &one, &one, &wind[0][NL], 
                 &wind[0][NS], &access, &type, optn, &one);
 
if (stat != E_SUCC)
   msg_err("!Error opening output image","topo_output");

num_files = 0;
strcpy(out[0], taeout);
num_files++;

/*
Update DDR
----------
*/
y_inc = 1;
x_inc = 1;
c_upddr(&block, hosin, hosout, &nimages, wind, bands, nbands, 
        &y_inc, &x_inc, &totbnd, &zero);

}

