#ifndef PROTOS_H
#define PROTOS_H

void bmaxmin
(
    float matrix[3][3],			/* matrix with pixel under  */
    float *beta,				/* beta angle */
    float coldis, 			/* distance between columns */
    float rowdis, 			/* distance between rows    */
    float aoff, 				/* mask value */
    long method, 			/* calculation method       */
    float sun_x, 				/* sun's ray vector: x-axis */
    float sun_y, 				/* sun's ray vector: y-axis */
    float sun_z				/* sun's ray vector: z-axis */
);

void bmean
(
    float matrix[3][3],			/* matrix with center pixel */
    float *beta,				/* beta angle               */
    float aoff, 				/* mask value               */
    float coldis, 			/* distance between columns */
    float rowdis, 			/* distance between rows    */
    float sun_x, 				/* sun's ray vector: x-axis */
    float sun_y, 				/* sun's ray vector: y-axis */
    float sun_z				/* sun's ray vector: z-axis */
);

void compute
(
    long bands[][MAXBND+1],	/* band numbers for the input image */
    long nbands[],		/* number of band in input image */
    long opt, 			/* option code(1-3) */
    long nimages,          	/* no. of images */
    long num_inbands,      	/* no. of bands */
    long wind[][4],        	/* window parameter */
    float cparm,			/* contour parameter */
    long otype,			/* output data type */
    long method, 		/* method no. (1-4) */
    long nbc,			/* neighborhood size code(1-2) */
    float sampdist,		/* column distance */
    float linedist,    		/* row distance */
    float aoff,      		/* mask value */
    long msgtim,			/* no. of elapse seconds */
    long itype, 			/* input image data type */
    float alpha,			/* sun's elevation angle */
    float gamma			/* azimuth angle in degrees */
);

void contour
(
    long np,			/* number  of pixels per line                */
    long nl,			/* number of lines in the input image        */
    float cont_val,		/* contour parameter                         */
    long otype,			/* output data type                          */
    long img,		        /* curent image being processed              */
    long bnd,      		/* input band number                         */
    long obnd			/* output image band number                  */
);

void getpar
(
    long bands[][MAXBND+1],	/* Band numbers for each input image       */
    long nbands[],		/* Number of bands per input image         */
    long *method, 		/* Method of calculation                   */
    char t1[][CMLEN],		/* Buffer for input TAE names              */
    char t2[][CMLEN],			/* Buffer for output TAE names             */
    char h1[][CMLEN],		/* Buffer for input host names             */
    char h2[][CMLEN],			/* Buffer for output host names            */
    long *nmrimg,		/* Number of input images                  */
    long *totbnd,		/* total no. of bands                      */
    long *msgtim,		/* Time processing message time interval   */
    long *opt,			/* Subcommand option                       */
    long *otype,			/* Output data type                        */
    long *nbrsiz,		/* Number of neighbouring points           */
    float *sampdist,		/* Distance between samples                */
    float *linedist, 		/* Distance between lines                  */
    float *aoff, 			/* Mask value                              */
    float *cparm,			/* Contour interval                        */
    long window[][4],	 	/* Window specifications                   */
    long *min_max,		/* Min/max value                           */
    long *nbc,			/* Neighborhood code                       */
    float *alpha,			/* Sun's elevation angle                   */
    float *gamma			/* Azimuth angle in degrees                */
);

void initio
(
    long bands[][MAXBND+1],	/* band numbers for each image   */
    long nbands[],		/* number of bands in each image */
    char hosin[][CMLEN],          /* input image host file name   */
    char hosout[],                /* output image host file name  */
    char taeout[],                /* output image tae file name   */
    long nimages,                /* no. of input images           */
    long wind[][4],              /* input image windows           */
    long totbnd,                 /* total no. of bands            */
    long otype 			/* output data type              */
);

void maxmin
(
    float matrix[3][3],		/* matrix with center as pixel under study */
    short *bufer,			/* holds slope/aspect result               */
    long method,			/* method number                           */
    long opt,			/* option code                             */
    long nbc,			/* neighborhood size code (1-2)            */
    float dist[8],		/* array for 8 directional distances       */
    float angle[9],		/* array for 9 angles used for aspect      */
    float aoff,			/* mask value                              */
    long dtype			/* data type of input image                */
);

void mean
(
    float matrix[3][3],	/* 3 x 3 matrix with center as pixel under study */
    short *bufer,		/* holds slope/aspect result upon return         */
    long opt,		/* option code(1-3)                              */
    long nbc,		/* neighborhood size code (1-2)                  */
    float coldis, 	/* column distance                               */
    float rowdis,		/* row distance                                  */
    float aoff		/* mask value                                    */
);

void process
(
    long np, 				/* number of samples                */
    long nl,				/* number of lines                  */
    long method, 			/* calculation method               */
    float alpha, 				/* sun's elevation angle    */
    float gamma,				/* azimuth angle in degrees */
    long bnd, 				/* band number                      */
    long obnd,				/* output band number               */
    float coldis,				/* distance between columns */
    float rowdis,				/* distance between rows    */
    float aoff, 				/* mask value               */
    long odtype, 			/* image data type                  */
    long img 				/* image number                     */
);

void sacomp
(
    long method,				/* method no. (1-4)            */
    long opt,				/* option code(1-3)            */
    long np,				/* no. of pixels per line      */
    long nl,				/* no. of lines in input image */
    long nbc,				/* neighborhood size code(1-2) */
    float coldis,				/* column distance             */
    float rowdis,    			/* row distance                */
    float aoff,				/* mask value                  */
    long img,       			/* current image index         */
    long bnd,       			/* input band number           */
    long obnd,				/* output band number          */
    long itype				/* input image type 	       */
);

void msg_err
(
    char *message,		/* Contents of the error message */
    char *key			/* Error message key             */
);

#endif
