#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "protos.h"

extern	long	curcat;                        /* message utility         */
extern	long	curent;                        /* parameters for processing */

struct PARBLK block;
struct FDESC * fdesc[MAXIMG];			/* input file descriptor  */
struct FDESC * fdescout;                       /* output file descriptor */
char    out[1][CMLEN];
long	num_files;

/*-----------------------------------------------------------------------------
NAME:	TOPO

FUNCTION:   Generates a slope, aspect, contour, or beta angle image.
            For the slope, aspect, or beta angle computation, it uses
            one of four different methods:  the mean method, the maximum 
            drop method, the maximum difference method, or the minimum 
            difference method.

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    1.0       03/20/84     M. Nishihama      
    2.0       03/24/87     R. Mckinney      Prf 4207
    3.0       02/18/89     B. Allam         Conversion
    4.0       07/20/89     B. Allam         Passed itype
    5.0	      12/04/89     J. Marmon        Fix minmax & change to run
                                            on power node
    6.0        7/19/90     K. Narayanan     Corrected calling sequence for
                                            process in compute.c 
    7.0       08/22/90     K. Kunska        fixed dynamic memory allocation
                                            by using a (3 x np) work buffer
                                            instead of (nl x np) work buffer
    7.1       09/21/92     D. Etrheim       Modifed global include file

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	
		Runs under TAE

ALGORITHM:
   Allocate the dynamic memory.
   Initialize LAS
   Call GETPAR to get user specified parameters.
   Call INITIO to open input and outut files.
   Call COMPUTE to calculate slope, aspect, contour or beta angle values and 
        write results to the output file.
   Update the output file history.
   Update the minimum and maximum fields in the output image ddr.
   Free the dynamic memory.
   Print a completion message.

-----------------------------------------------------------------------------
*/

int main
(void) /* topo */
{
static long one = 1;

char ihos[MAXIMG][CMLEN],		 		/* host input file name(s) */
	itae[MAXIMG][CMLEN],				/* tae input file name(s) */
	ohos[1][CMLEN],				/* host output file name(s) */
    otae[1][CMLEN];		 		/* tae output file name(s) */

double   maxout[MAXBND], minout[MAXBND]; /* min and max samples */

float   alpha,				/* sun's elevation angle */
	aoff,				/* mask value */
	cparm,				/* contour parameter */
	gamma,				/* azimuth angle in degrees */
	linedist,    			/* row distance */
	sampdist;			/* column distance */

long	bands[MAXIMG][MAXBND+1],	/* band numbers for each image */
	i,				/* loop counter */
	itype = 0, 				/* input image data type */
	method, 			/* method no. */
	min_max,			/* minmax flag */
	msgtim, 			/* message timer */
	nbands[MAXIMG], 		/* number of bands in input image */
	nbrsiz,				/* Number of neighbouring points */
	nimages,			/* no. of images */
	nbc,				/* neighborhood size code */
	opt,				/* option code */
	otype, 				/* output data type */
	obands[1][MAXBND+1],		/* output band numbers */
	outwind[1][4],			/* output window specification */
	stadout, 			/* flag parameter */
	totbnd,				/* total no. of bands */
	window[MAXIMG][4];		/* window flag */

/*
Initialize LAS.
---------------
*/
num_files = 0;
c_inlas (&block, &stadout);

/*
Call GETPAR to get user specified parameters.
-----------------------------------------------
*/
getpar(bands, nbands, &method, itae, otae, ihos, ohos, &nimages, &totbnd,
       &msgtim, &opt, &otype, &nbrsiz, &sampdist, &linedist, &aoff,
       &cparm, window, &min_max, &nbc, &alpha, &gamma);

/*
Call INITIO to open input and outut files.
--------------------------------------------
*/
initio (bands, nbands,ihos,ohos[0], otae[0], nimages, window, totbnd, otype);

/*
Call COMPUTE to calculate slope,  aspect, contour or beta angle values and 
write results to the output file.
-----------------------------------------------------------------------------
*/
compute (bands, nbands, opt, nimages, totbnd, window, cparm, otype, method, 
         nbc, sampdist, linedist, aoff, msgtim, itype, alpha, gamma);

/*
Update the output file history.
-------------------------------
*/
c_hstout(&block, ohos[0], ihos, &nimages, " ");

/*
Update the minimum and maximum fields in the output image ddr .
-----------------------------------------------------------------
*/
if (min_max)  
   {
   for (i = 0; i < totbnd; i++)
      obands[0][i] = i + 1;
   obands[0][i] = 0;
   outwind[0][0] = 1;
   outwind[0][1] = 1;
   outwind[0][2] = window[0][2];
   outwind[0][3] = window[0][3];
   c_minmax (ohos, &one, outwind, obands, &totbnd, minout, maxout);
   c_svmnmx (ohos[0], &totbnd, minout, maxout, "YES");
   }

/*
Free the dynamic memory.
-------------------------
*/

/*
Print a completion message.
----------------------------
*/
c_complt("TOPO", out, &one);

return EXIT_SUCCESS;

}
