#include "topo.h"
#include "util.h"
#include "protos.h"


/*
-----------------------------------------------------------------------------

 NAME		MSG_ERR

 PURPOSE	Displays an error message and exits TOPO if it is fatal.
		The first character of the error message determines whether
		or not the error is fatal.

 PROGRAM HISTORY

	Programmer		Date		Reason
	----------		----		------
	Eric Ivancich / SAR	13-Aug-87	C conversion
        Bani Allam              8-Aug-88        Retrocode
        K. Kunska               22-Aug-90       Added comments

 ALGORITHM:

   If the first character to the error message = ?
      Print informational message
   End-if
   If the first character of the error message = !
      Print fatal message
      Clean-up output files
   End-if

-----------------------------------------------------------------------------*/

void msg_err
(
    char *message,		/* Contents of the error message */
    char *key			/* Error message key             */
)
{

extern  struct PARBLK block;	/* TAE paramter block            */

extern  char    out[1][CMLEN];	/* List of output file names     */
extern  long    num_files;	/* Number of output files        */

static	long	inf_err = 1;	/* Informational error code      */


switch (*message++)
   {
   case '?' :
             c_errmsg (message, key, &inf_err);
             break;
   case '!' : 
             {
             c_errmsg (message, key, &inf_err);
             c_clean (&block, "TOPO", out, &num_files);
             }
   }
}
