/****************************************************************************
NAME:				TPL2MTP

PURPOSE:  Converts a TPL file to a MTP file

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 7/92	D. Steinwand
  1.1	 9/92	D. Steinwand--added SWAP flag for BACKMATCHING
  2.0	12/92	T. Mittan		Released to baseline
  3.0    5/94   D. Bridges    Converted output name to a LAS name

****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"

main()		
{
struct PARBLK vblk;		/* TAE parameter block 			*/
struct VARIABLE *p_find();	/* TAE parameter function 		*/
struct VARIABLE *var;		/* TAE variable pointer 		*/
struct TPLHEAD header;		/* TPL header data 			*/
struct TPLDATA tpldata;		/* TPL data record 			*/
struct MTPHEAD mtphead;		/* MTP header data 			*/
struct MTPDATA mtpdata;		/* MTP data record 			*/
long msgtim;			/* Message timer 			*/
long std_out;			/* Standard output LUN 			*/
long status;			/* Function return status		*/
long fatal = -1;		/* Fatal flag for errmsg 		*/
long zero = 0;			/* Constant of 0 			*/
long one = 1;			/* Constant of 1 			*/
long two = 2;			/* Constant of 2 			*/
long i;				/* Loop counter 			*/
long ptyp;			/* Type of file used (for Pt I/O) 	*/
long acc;			/* Access mode of TPL file 		*/
long swap;			/* Swap flag for backmatching 		*/
int *intpl,*outmtp;		/* File descriptors 			*/
char tpl_name[CMLEN];		/* The input Tie Point Location file name  */
char out_mtp[CMLEN];		/* The output Tie Point Location file name */
extern long curent;
extern long curcat;

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk, "INTL", &zero, tpl_name, "tpl", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","tpl2mtp-fatal",&fatal);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(&vblk, "OUTMT", &zero, out_mtp, "mtp", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","tpl2mtp-fatal",&fatal);

/* Get the swap parameter
  ----------------------*/
var = p_find(&vblk, "SWAPOPT");
if (strcmp("YES",SVAL(*var,0)) == 0) swap = 1; 
else swap = 0;

/* Get the message timer
  ---------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);

/* Open the input Tie Point Location file
  ---------------------------------------*/
c_inittm("TPL2MTP","Processed","coordinate","","",&two,&two, &msgtim, &two);
acc = IREAD;  
ptyp=TPL;
if(c_ptopen(&intpl, tpl_name, &ptyp, &acc, &header) != E_SUCC) 
	c_errmsg("Fatal error encountered","tpl2mtp-fatal",&fatal);

/* Open output Merged Tie Point Location file
  -----------------------------------------*/
for(i=0;i<8;i++) 
   mtphead.corners[i]=header.corners[i];
mtphead.pdist[0]=header.pdist[0];
mtphead.pdist[1]=header.pdist[1];
for(i=0;i<15;i++) 
   mtphead.projprms[i]=header.projprms[i];
mtphead.proj_valid=header.proj_valid;
mtphead.code=header.code;
mtphead.zone=header.zone;
mtphead.datum=header.datum;
strcpy(mtphead.chip_mode,header.ref_mode);
strcpy(mtphead.units,header.units);
strcpy(mtphead.sea_name,header.sea_name);
strcpy(mtphead.ref_name,header.ref_name);
acc = IWRITE;  
ptyp=MTP;
if(c_ptopen(&outmtp, out_mtp, &ptyp, &acc, &mtphead) != E_SUCC) 
	c_errmsg("Fatal error encountered","tpl2mtp-fatal",&fatal);

/* Process all points in the file (Loop until EOF)
  -----------------------------------------------*/
curent = 1; 
c_startm(); 
curent = 0;
for (;;)
   {
   status = c_ptio(&intpl, &tpldata);
   if (status == E_EOF) break;
   if (status == E_FAIL) 
	c_errmsg("Fatal error encountered","tpl2mtp-fatal",&fatal);
   curent++;
   mtpdata.geo_coord[0]=tpldata.geo_coord[0];
   mtpdata.geo_coord[1]=tpldata.geo_coord[1];
   /* if swap switch the reference and search points
   -----------------------------------------------*/
   if(swap)
      {
      mtpdata.nominal[0]=tpldata.ref_coord[0];
      mtpdata.nominal[1]=tpldata.ref_coord[1];
      mtpdata.ref_coord[0]=tpldata.sea_coord[0];
      mtpdata.ref_coord[1]=tpldata.sea_coord[1];
      }
   else
      {
      mtpdata.nominal[0]=tpldata.sea_coord[0];
      mtpdata.nominal[1]=tpldata.sea_coord[1];
      mtpdata.ref_coord[0]=tpldata.ref_coord[0];
      mtpdata.ref_coord[1]=tpldata.ref_coord[1];
      }
   mtpdata.elevation=tpldata.elevation;
   mtpdata.zoom_factor=tpldata.zoom_factor;
   mtpdata.chip_size[0]=tpldata.ref_size[0];
   mtpdata.chip_size[1]=tpldata.ref_size[1];
   mtpdata.offsets[0]=(tpldata.offsets[0]==0)?16:tpldata.offsets[0];
   mtpdata.offsets[1]=(tpldata.offsets[1]==0)?16:tpldata.offsets[1];
   strcpy(mtpdata.pt_id,tpldata.pt_id);
   strcpy(mtpdata.ref_chip,tpldata.ref_chip);
   strcpy(mtpdata.sea_chip,tpldata.ref_chip);
   /* Write header information
   -------------------------*/
   if (c_ptio(&outmtp, &mtpdata) != E_SUCC) 
	c_errmsg("Fatal error encountered","tpl2mtp-fatal",&fatal);
   }
c_stoptm();
c_complt("TPL2MTP", out_mtp, &one);
}
