/****************************************************************************
NAME:				TPL2TPS

PURPOSE:  Creates reference and search TPS files from the original files
          while leaving out the inactive points specified in the TPL file.

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	  	REASON
-------	 ----	------	   	------
  1.0	 5/94   D. Etrheim	Original development.  PRF 6353

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   Must be run under TAE.

PROJECT:                        LAS

ALGORITHM DESCRIPTION:
   NONE
****************************************************************************/
#include <stdio.h>
#include "las.h"
#include "geompak.h"

main()		
{
extern long curent;
extern long curcat;

struct PARBLK vblk;		/* TAE parameter block 			*/
struct VARIABLE *p_find();	/* TAE parameter function 		*/
struct VARIABLE *var;		/* TAE variable pointer 		*/
struct TPLHEAD header;		/* TPL header data 			*/
struct TPLDATA tpldata;		/* TPL data record 			*/
struct TPSHEAD refhead;		/* TPS header data 			*/
struct TPSDATA refdata;		/* TPS data record 			*/
struct TPSHEAD srchead;		/* TPS header data 			*/
struct TPSDATA srcdata;		/* TPS data record 			*/
long msgtim;			/* Message timer 			*/
long std_out;			/* Standard output LUN 			*/
long status;			/* Function return status		*/
long fatal = -1;		/* Fatal flag for errmsg 		*/
long zero = 0;			/* Constant of 0 			*/
long one = 1;			/* Constant of 1 			*/
long two = 2;			/* Constant of 2 			*/
long ptyp;			/* Type of file used (for Pt I/O) 	*/
long acc;			/* File access mode 			*/
int *intpl,*outref,*outsrc;	/* File descriptors 			*/
int *inref,*insrc;		/* File descriptors 			*/
char tpl_name[CMLEN];		/* The input Tie Point Location file names */
char out_tps[2][CMLEN];		/* The output Tie Point Selection file names */
char in_tps[2][CMLEN];		/* The input Tie Point Selection file names */

 
/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk, "INTL", &zero, tpl_name, "tpl", &acc) != E_SUCC) 
      c_errmsg("Fatal error encountered","tpl2tps-fatal",&fatal);

/* Get the input tie point selection file names
  --------------------------------------------*/
if(c_getnam(&vblk, "INTS", &zero, in_tps[0], "tps", &acc) != E_SUCC) 
     c_errmsg("Fatal error encountered","tpl2tps-fatal",&fatal);
if(c_getnam(&vblk, "INTS", &one, in_tps[1], "tps", &acc) != E_SUCC) 
     c_errmsg("Fatal error encountered","tpl2tps-fatal",&fatal);

/* Get the output tie point selection file names
  ---------------------------------------------*/
acc = IWRITE;
if(c_getnam(&vblk, "OUTTS", &zero, out_tps[0], "tps", &acc) != E_SUCC) 
     c_errmsg("Fatal error encountered","tpl2tps-fatal",&fatal);
if(c_getnam(&vblk, "OUTTS", &one, out_tps[1], "tps", &acc) != E_SUCC) 
     c_errmsg("Fatal error encountered","tpl2tps-fatal",&fatal);

/* Retrieve global
------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);

/* Open the input Tie Point Location file and input Tie Point Selection files
  ---------------------------------------------------------------------------*/
acc = IREAD;  
ptyp = TPL;
if(c_ptopen(&intpl, tpl_name, &ptyp, &acc, &header) != E_SUCC) 
     c_errmsg("Fatal error encountered","tpl2tps-fatal",&fatal);

ptyp = TPS;
if(c_ptopen(&inref, in_tps[0], &ptyp, &acc, &refhead) != E_SUCC) 
     c_errmsg("Fatal error encountered","tpl2tps-fatal",&fatal);
if(c_ptopen(&insrc, in_tps[1], &ptyp, &acc, &srchead) != E_SUCC) 
     c_errmsg("Fatal error encountered","tpl2tps-fatal",&fatal);

/* Open output TPS files
------------------------*/
acc = IWRITE;  
ptyp = TPS;
if(c_ptopen(&outref, out_tps[0], &ptyp, &acc, &refhead) != E_SUCC) 
     c_clean(&vblk,"tpl2tps",out_tps,&two);
if(c_ptopen(&outsrc, out_tps[1], &ptyp, &acc, &srchead) != E_SUCC) 
     c_clean(&vblk,"tpl2tps",out_tps,&two);

/* Initialize processing message
   -----------------------------*/
c_inittm("TPL2TPS","Processed","coordinate","","",&two,&two, &msgtim, &two);
curent = 1; 
c_startm(); 
curent = 0;

/* Process all points in the file (Loop until EOF)
  -----------------------------------------------*/
for (;;)
   {
   status = c_ptio(&intpl, &tpldata);
   if (status == E_EOF) break;
   if (status == E_FAIL) 
      c_clean(&vblk,"tpl2tps",out_tps,&two);
   curent++;
   /* Read data information
   -------------------------*/
   if (c_ptio(&inref, &refdata) != E_SUCC) 
      c_clean(&vblk,"tpl2tps",out_tps,&two);
   if (c_ptio(&insrc, &srcdata) != E_SUCC) 
      c_clean(&vblk,"tpl2tps",out_tps,&two);

   if ((strcmp(tpldata.pt_id,refdata.pt_id) != 0) ||
       (strcmp(tpldata.pt_id,srcdata.pt_id) != 0))
       {
       c_stoptm();
       c_errmsg("Tie point ID's do not match","tpl2tps-match",&one);
       c_clean(&vblk,"tpl2tps",out_tps,&two);
       }
    
   if ((tpldata.active == 0) || (tpldata.active == 1) || (tpldata.active > 9 ))
      {
      /* Write header information
      -------------------------*/
      if (c_ptio(&outref, &refdata) != E_SUCC) 
        c_clean(&vblk,"tpl2tps",out_tps,&two);
      if (c_ptio(&outsrc, &srcdata) != E_SUCC) 
        c_clean(&vblk,"tpl2tps",out_tps,&two);
      }
   }
c_stoptm();

if (c_ptclse(&outref) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tpl2tps-warn",&one);
if (c_ptclse(&outsrc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tpl2tps-warn",&one);
if (c_ptclse(&inref) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tpl2tps-warn",&one);
if (c_ptclse(&insrc) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tpl2tps-warn",&one);

c_complt("TPL2TPS", out_tps, &two);
}
