/*****************************************************************************
NAME				 fit

*****************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "lasmath.h"
#include "geompak.h"
#include "protos.h"

void fit
(
    struct GEOGRID *grid,	/* Geometric mapping grid */
    double *a,
    double *b		/* Mapping coefficients   */
)
{
long row,		/* Reference row			*/
     col,		/*  Reference column			*/
     i;			/* loop variable			*/
long index;		/* Search index				*/
long size;		/* Size of ref/srch lines & samps	*/
double *reflines,	/* Reference lines 			*/
       *refsamps,	/* Reference samples			*/
       *srchlines,	/* Search lines				*/
       *srchsamps;      /* Search samples			*/

size = (long) (grid->nrows*grid->ncols*sizeof(double));
reflines = (double *)malloc(size);
refsamps = (double *)malloc(size);
srchlines = (double *)malloc(size);
srchsamps = (double *)malloc(size);

for (i=0, row = 0; row < grid->nrows; row++)
   for (col = 0; col < grid->ncols; i++,col++)
     {
     index = row * grid->ncols + col;
     reflines[i] = grid->out_lines[row],
     refsamps[i] = grid->out_samps[col],
     srchlines[i] = grid->in_lines[index],
     srchsamps[i] = grid->in_samps[index];
     }
fitpts(reflines,refsamps,srchlines,srchsamps,grid->nrows*grid->ncols,4,a,b);
free(reflines);
free(refsamps);
free(srchlines);
free(srchsamps);
}


/****************************************************************************
NAME:				FITPTS

PURPOSE:  Preforms a least-squares fit and calculates residual errors
		(inverse--note search & ref are switched)

****************************************************************************/
int fitpts
(
    double *srcl,
    double *srcs,
    double *refl,
    double *refs,
    long npts,			/* Number of points in TPL file 	*/
    long degree,			/* Degree of polynomial used 		*/
    double *a,
    double *b			/* Mapping coeficients			*/
)
{
double *v;			/* Work buffer for LSQ fit 		*/
double *b1;                     /* Vector containing X coordinates 	*/
double *b2;                     /* Vector containing Y coordinates 	*/
double *A;			/* Matrix A (used for LSQ fit) 		*/
double c_eval();		/* Function to evaluate polynomial @ x,y */

long nonfatal = 1;              /* non fatal error flag                 */
long i,k;			/* Loop counters 			*/
long size;			/* Size of vectors (based on poly degree) */

if(degree==1)size=3;
else if(degree==2)size=6;
else if(degree==3)size=10;
else if(degree==4)size=15;
else {degree=1; size=3;}

if ((A  = (double *)malloc(npts * size * sizeof(double))) == NULL ||
    (b1 = (double *)malloc(npts * sizeof(double)))==NULL ||
    (b2 = (double *)malloc(npts * sizeof(double)))==NULL ||
    (v  = (double *)malloc(npts * sizeof(double)))==NULL)
   {
   c_errmsg("Unable to allocate dynamic memory","tplmap-alloc",&nonfatal);
   return(E_FAIL);
   }

/* Loop on tie point pairs.  If the tie point is active, use it in the
   fit calculations; first for x, then for y.
  ------------------------------------------*/
for(k=0,i=0; i < npts; i++)
      {
      A[k] =               1.0;
      A[k+npts] =     refs[i];
      A[k+(2*npts)] = refl[i];
      if(degree>1)
           {
           A[k+(3*npts)]=refs[i]*refs[i];
           A[k+(4*npts)]=refs[i]*refl[i];
           A[k+(5*npts)]=refl[i]*refl[i];
           }
      if(degree>2)
	   {
           A[k+(6*npts)]=refs[i]*refs[i]*refs[i];
           A[k+(7*npts)]=refs[i]*refs[i]*refl[i];
           A[k+(8*npts)]=refs[i]*refl[i]*refl[i];
           A[k+(9*npts)]=refl[i]*refl[i]*refl[i];
           }
      if(degree>3)
           {
           A[k+(10*npts)]=refs[i]*refs[i]*refs[i]*refs[i];
           A[k+(11*npts)]=refs[i]*refs[i]*refs[i]*refl[i];
           A[k+(12*npts)]=refs[i]*refs[i]*refl[i]*refl[i];
           A[k+(13*npts)]=refs[i]*refl[i]*refl[i]*refl[i];
           A[k+(14*npts)]=refl[i]*refl[i]*refl[i]*refl[i];
           }

      b1[k] = srcs[i];
      b2[k] = srcl[i];
      k++;
      }
c_qrdecomp(A, npts, size, v, 0);
c_qrsolve(A, npts, size, v, b1, 0);
c_qrsolve(A, npts, size, v, b2, 0);

/* Calculate average residuals
  ---------------------------*/

for(i=0;i<size;i++)
   a[i]=b1[i];
for(i=0;i<size;i++)
   b[i]=b2[i];

free(A); free(b1); free(b2); free(v);
return(E_SUCC);
}
