/****************************************************************************
NAME                      GETPAR


PURPOSE         To obtain the necessary parameters for the TPLMAP function.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   GETPAR must be run under TAE.

PROJECT                  LAS
******************************************************************************/
#include "las.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

int getpar
(
    struct PARBLK *vblk,          /* TAE parameter block               */
    struct GEOGRID *grid,         /* Geometric Mapping Grid structure  */
    char tpl_name[CMLEN],         /* The input Tie Point Location file name */
    char out_tpl[CMLEN],          /* The output Tie Point Location file name */
    char mflg[],                  /* Invert indicator */
    char grid_name[CMLEN],        /* The Geometric Mapping Grid file name */
    long *msgtim                  /* Message timer */
)

{
struct VARIABLE *var;           /* TAE variable pointer */

long acc;                       /* Access mode of TPL file */
long fatal = -1;                /* Fatal flag for errmsg */
long zero = 0;                  /* Constant of 0 */


/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INTL", &zero, tpl_name, "tpl", &acc) != E_SUCC)
        c_errmsg("Fatal error encountered","tplmap-fatal",&fatal);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(vblk, "OUTTL", &zero, out_tpl, "tpl", &acc) != E_SUCC)
        c_errmsg("Fatal error encountered","tplmap-fatal",&fatal);

/* Get the mapping direction indicator
  -------------------------*/
var = p_find(vblk, "MAPOPT");
strcpy(mflg, SVAL (*var,0));

/* Get the grid name, open the grid and extract the polynomial coefficients
  ------------------------------------------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INGRID", &zero, grid_name, "grid", &acc) != E_SUCC)
        c_errmsg("Fatal error encountered","tplmap-fatal",&fatal);
if(c_getgrd(grid_name, grid) != E_SUCC)
        c_errmsg("Fatal error encountered","tplmap-fatal",&fatal);

/* Get the message timer
  ---------------------*/
var = p_find(vblk, "$MSGTIM");
*msgtim = IVAL(*var, 0);

return(E_SUCC);
}
