/***************************************************************************
NAME:				TPLMAP

PURPOSE: Maps tie point coordinates using a geometric mapping grid. 

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
 1.0     6/94   N. Holzapfel	Derived from tplpoly--maps into and out of
				input space using geometric mapping grid
 1.2     7/94   D. Eichacker    Updated to LAS standards

*****************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "lasmath.h"
#include "geompak.h"
#include "util.h"
#include "protos.h"

#define PDELTA 2.0;

int main
(void)		
{
extern long curent;

struct PARBLK vblk;		/* TAE parameter block */
struct TPLHEAD header;		/* TPL header data */
struct TPLDATA tpldata;		/* TPL data record */
struct GEOGRID grid;		/* Geometric Mapping Grid structure */

double a[25],b[25];		/* Mapping coeficients */
double bx1[4],by1[4],v1[4],A1[16];
double bx2[4],by2[4],v2[4],A2[16];
double in_ulx1,in_urx1,in_lrx1,in_llx1;
double in_ulx2,in_urx2,in_lrx2,in_llx2;
double in_uly1,in_ury1,in_lry1,in_lly1;
double in_uly2,in_ury2,in_lry2,in_lly2;
double out_ulx1,out_urx1,out_lrx1,out_llx1;
double out_ulx2,out_urx2,out_lrx2,out_llx2;
double out_uly1,out_ury1,out_lry1,out_lly1;
double out_uly2,out_ury2,out_lry2,out_lly2;
double ref_line,ref_samp,sea_line,sea_samp;
double ssamp, sline;            /* Sea line and samp coords of input space */
double rsamp, rline;            /* Ref line and samp coords of input space */


long acc;			/* Access mode of TPL file */
long fatal = -1;		/* Fatal flag for errmsg */
long four = 4;                  /* Constant of 4 */
long i;                         /* Loop counter */
long msgtim;			/* Message timer */
long nonfatal = 1;              /* Fatal flag for errmsg */
long ocount = 0;                /* Number of output files created */
long ptyp = TPL;		/* Type of TPL file used (for I/O) */
long status;			/* Function return status */
long std_out;			/* Standard output LUN */
long two = 2;			/* Constant of 2 */

struct PTDESC *intpl,*outtpl;		/* File descriptors */

char grid_name[CMLEN];		/* The Geometric Mapping Grid file name */
char mflg[7];                   /* Mapping direction flag */
char out[1][CMLEN];             /* Output file names created */
char out_tpl[CMLEN];		/* The output Tie Point Location file name */
char tpl_name[CMLEN];		/* The input Tie Point Location file name */


/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

getpar(&vblk,&grid,tpl_name,out_tpl,mflg,grid_name,&msgtim);

/* Save the temporary out file name for clean up
  ---------------------------------------------*/
strcpy(out[ocount], out_tpl);
ocount++;

/* Open the input Tie Point Location file
  ---------------------------------------*/
c_inittm("TPLMAP","Processed","coordinate number","","",&two,&two, &msgtim, &two);
acc = IREAD;
if(c_ptopen(&intpl, tpl_name, &ptyp, &acc, &header) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplmap-fatal",&fatal);

/* Open output Tie Point Location file
  ------------------------------------*/
acc = IWRITE;
if(c_ptopen(&outtpl, out_tpl, &ptyp, &acc, &header) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplmap-fatal",&fatal);

/* Process all points in the file (Loop until EOF)
  -----------------------------------------------*/
if (mflg[0] == 'I')
   fit(&grid,a,b);

/* Process all points in the file (Loop until EOF)
  -----------------------------------------------*/
curent = 1;
c_startm(); 
curent = 0;
for (;;)
   {
   status = c_ptio(&intpl, &tpldata);
   if (status == E_EOF) break;
   if (status == E_FAIL) 
      {
      c_ptclse(&intpl); 
      c_ptclse(&outtpl); 
      c_clean(&vblk,"tplmap",out,&ocount);
      }
   curent++;
  
   /* Perform inverse mapping 
     ------------------------------------------------------------------*/
   if (mflg[0] == 'D')
      {
      grid_eval(&grid,tpldata.ref_coord[1],tpldata.ref_coord[0],&rsamp,&rline);
      tpldata.ref_coord[0] = rline;
      tpldata.ref_coord[1] = rsamp;
      grid_eval(&grid,tpldata.sea_coord[1],tpldata.sea_coord[0],&ssamp,&sline);
      tpldata.sea_coord[0] = sline;
      tpldata.sea_coord[1] = ssamp; 
      }

   /* Invert inverse mapping (input to output)
     ----------------------------------------*/
   else if (mflg[0] == 'I')
      {
     rline = tpldata.ref_coord[0];
     rsamp = tpldata.ref_coord[1];
     sline = tpldata.sea_coord[0];
     ssamp = tpldata.sea_coord[1];
     ref_line = c_eval(&four,b,&rsamp,&rline);
     ref_samp = c_eval(&four,a,&rsamp,&rline);
     sea_line = c_eval(&four,b,&ssamp,&sline);
     sea_samp = c_eval(&four,a,&ssamp,&sline);

   /* Calculate mapping back into the space in which control was selected
     ---------------------------------------------------------------------*/

   /* Reference coords
     ------------------*/
   out_ulx1 = ref_samp - PDELTA;
   out_uly1 = ref_line - PDELTA;
   out_urx1 = ref_samp + PDELTA;
   out_ury1 = ref_line - PDELTA;
   out_llx1 = ref_samp - PDELTA;
   out_lly1 = ref_line + PDELTA;
   out_lrx1 = ref_samp + PDELTA;
   out_lry1 = ref_line + PDELTA;
   grid_eval(&grid,out_ulx1,out_uly1,&in_ulx1,&in_uly1);
   grid_eval(&grid,out_urx1,out_ury1,&in_urx1,&in_ury1);
   grid_eval(&grid,out_llx1,out_lly1,&in_llx1,&in_lly1);
   grid_eval(&grid,out_lrx1,out_lry1,&in_lrx1,&in_lry1);

   /* Search coords
     --------------*/
   out_ulx2 = sea_samp - PDELTA;
   out_uly2 = sea_line - PDELTA;
   out_urx2 = sea_samp + PDELTA;
   out_ury2 = sea_line - PDELTA;
   out_llx2 = sea_samp - PDELTA;
   out_lly2 = sea_line + PDELTA;
   out_lrx2 = sea_samp + PDELTA;
   out_lry2 = sea_line + PDELTA;
   grid_eval(&grid,out_ulx2,out_uly2,&in_ulx2,&in_uly2);
   grid_eval(&grid,out_urx2,out_ury2,&in_urx2,&in_ury2);
   grid_eval(&grid,out_llx2,out_lly2,&in_llx2,&in_lly2);
   grid_eval(&grid,out_lrx2,out_lry2,&in_lrx2,&in_lry2);

   /* Calculate a set of first order polynomial coefficients which map input
      space CCT-x coordinates in the neighborhood of the tie point to output
      space (the space the tiepoints were selected in) line,sample coordinates
     -------------------------------------------------------------------------*/
      for (i = 0; i < 4; i++)
        {
          A1[i] = 1.0;
          A2[i] = 1.0;
        }

     A1[4]=in_ulx1;                     A2[4]=in_ulx2;
     A1[5]=in_urx1;                     A2[5]=in_urx2;
     A1[6]=in_lrx1;                     A2[6]=in_lrx2;
     A1[7]=in_llx1;                     A2[7]=in_llx2;

     A1[8]=in_uly1;                     A2[8]=in_uly2;
     A1[9]=in_ury1;                     A2[9]=in_ury2;
     A1[10]=in_lry1;                    A2[10]=in_lry2;
     A1[11]=in_lly1;                    A2[11]=in_lly2;

     A1[12]=in_ulx1*in_uly1;            A2[12]=in_ulx2*in_uly2;
     A1[13]=in_urx1*in_ury1;            A2[13]=in_urx2*in_ury2;
     A1[14]=in_lrx1*in_lry1;            A2[14]=in_lrx2*in_lry2;
     A1[15]=in_llx1*in_lly1;            A2[15]=in_llx2*in_lly2;

     bx1[0]= out_ulx1;                  bx2[0]= out_ulx2;
     bx1[1]= out_urx1;                  bx2[1]= out_urx2;
     bx1[2]= out_lrx1;                  bx2[2]= out_lrx2;
     bx1[3]= out_llx1;                  bx2[3]= out_llx2;

     by1[0]= out_uly1;                  by2[0]= out_uly2;
     by1[1]= out_ury1;                  by2[1]= out_ury2;
     by1[2]= out_lry1;                  by2[2]= out_lry2;
     by1[3]= out_lly1;                  by2[3]= out_lly2;

c_qrdecomp(A1, 4, 4, v1, 0);
c_qrsolve(A1, 4, 4, v1, bx1, 0);
c_qrsolve(A1, 4, 4, v1, by1, 0);

c_qrdecomp(A2, 4, 4, v2, 0);
c_qrsolve(A2, 4, 4, v2, bx2, 0);
c_qrsolve(A2, 4, 4, v2, by2, 0);

  tpldata.ref_coord[0] = by1[0]+by1[1]*rsamp+by1[2]*rline+by1[3]*rline*rsamp;
  tpldata.ref_coord[1] = bx1[0]+bx1[1]*rsamp+bx1[2]*rline+bx1[3]*rline*rsamp;
  tpldata.sea_coord[0] = by2[0]+by2[1]*ssamp+by2[2]*sline+by2[3]*sline*ssamp;
  tpldata.sea_coord[1] = bx2[0]+bx2[1]*ssamp+bx2[2]*sline+bx2[3]*sline*ssamp;
     }

   if (c_ptio(&outtpl, &tpldata) != E_SUCC) 
      {
      c_ptclse(&intpl); 
      c_ptclse(&outtpl); 
      c_clean(&vblk,"tplmap",out,&ocount);
      }
   }
if(c_ptclse(&intpl) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tplmap-warn",&nonfatal);
if(c_ptclse(&outtpl) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tplmap-warn",&nonfatal);
c_stoptm();
c_complt("TPLMAP", out, &ocount);

return EXIT_SUCCESS;

}
