/****************************************************************************
NAME:				TPLSCALE

PURPOSE:  Scales active image coordinates in a Tiepoint Location file 

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	  
-------	 ----	------	   
  1.0	 7/92	D. Steinwand
  1.1    7/94   D. Eichacker           Updated to meet LAS standards.

*****************************************************************************/
#include "las.h"
#include "geompak.h"

main()		
{
extern long curent;
extern long curcat;

struct PARBLK vblk;		/* TAE parameter block */
struct VARIABLE *p_find();	/* TAE parameter function */
struct VARIABLE *var;		/* TAE variable pointer */
struct TPLHEAD header;		/* TPL header data */
struct TPLDATA tpldata;		/* TPL data record */

float scale;			/* Scale factor */

long acc;			/* Access mode of TPL file */
long fatal = -1;		/* Fatal flag for errmsg */
long msgtim;			/* Message timer */
long nonfatal = 1;		/* Fatal flag for errmsg */
long ocount = 1;		/* Number of output files created */
long ptyp = TPL;		/* Type of TPL file used (for I/O) */
long status;			/* Function return status */
long std_out;			/* Standard output LUN */
long two = 2;			/* Constant of 2 */
long zero = 0;			/* Constant of 0 */

int *intpl,*outtpl;		/* File descriptors */

char out_tpl[CMLEN];		/* The output Tie Point Location file name */
char tpl_name[CMLEN];		/* The input Tie Point Location file name */

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

/* Get the input tie point location file name
  ------------------------------------------*/
acc = IREAD;
if (c_getnam(&vblk, "INTL", &zero, tpl_name, "tpl", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplscale-fatal",&fatal);

/* Get the output tie point location file name
  -------------------------------------------*/
acc = IWRITE;
if(c_getnam(&vblk, "OUTTL", &zero, out_tpl, "tpl", &acc) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplscale-fatal",&fatal);

/* Get the scale factor
  --------------------*/
var = p_find(&vblk, "SCALFACT");
scale = RVAL(*var, 0);

/* Get the message timer
  ---------------------*/
var = p_find(&vblk, "$msgtim");
msgtim = IVAL(*var, 0);

/* Open the input Tie Point Location file
  ---------------------------------------*/
c_inittm("TPLSCALE","Processed","coordinate number","","",&two,&two, &msgtim, &two);
acc = IREAD;
if(c_ptopen(&intpl, tpl_name, &ptyp, &acc, &header) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplscale-fatal",&fatal);

/* Adjust the output TPL's header for the scaling
   ---------------------------------------------*/
header.pdist[0] /= scale;
header.pdist[1] /= scale;

/* Open output Tie Point Location file
  ------------------------------------*/
acc = IWRITE;
if(c_ptopen(&outtpl, out_tpl, &ptyp, &acc, &header) != E_SUCC) 
	c_errmsg("Fatal error encountered","tplscale-fatal",&fatal);

/* Process all points in the file (Loop until EOF)
  -----------------------------------------------*/
curent = 1;
c_startm();
curent = 0;

for (;;)
   {
   status = c_ptio(&intpl, &tpldata);
   if (status == E_EOF) break;
   if (status == E_FAIL)
      {
      c_ptclse(&intpl);
      c_ptclse(&outtpl);
      c_clean(&vblk,"tplscale",out_tpl,&ocount);
      }
   /* If coordinates have been accepted 
      scale
      -------------------------------*/
   if((tpldata.active == 0)||(tpldata.active == 1)||(tpldata.active > 9))
      {
      curent++;
      tpldata.sea_coord[0] *= scale;
      tpldata.ref_coord[0] *= scale;
      tpldata.sea_coord[1] *= scale;
      tpldata.ref_coord[1] *= scale;
      if (c_ptio(&outtpl, &tpldata) != E_SUCC) 
          {
          c_ptclse(&intpl);
          c_ptclse(&outtpl);
          c_clean(&vblk,"tplscale",out_tpl,&ocount);
          }
      }
   }
if(c_ptclse(&outtpl) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tplscale-warn",&nonfatal);
if(c_ptclse(&intpl) != E_SUCC)
   c_errmsg("Nonfatal error encountered","tplscale-warn",&nonfatal);
c_stoptm();
c_complt("TPLSCALE", out_tpl, &ocount);
}
