/****************************************************************************
NAME				 show_tps

PURPOSE    Writes a tie point selection file to a project parameter file (.ppf).


PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
 5.0       T. Jensen       10/92      EDC       Original



PROJECT			LAS

ALGORITHM DESCRIPTION
        Open the tie point file
        Print the point report in tabular form
        Close the tie point file
        Return

ALGORITHM REFERENCES	Adapted from show_tps.c in DSPTIE.
*****************************************************************************/
#include "tps2ppf.h"
#include "util.h"
#include "protos.h"

int show_tps
(
    char *fname,			/* Name of TPS file 			*/
    char *filename		/* File pointer for output 		*/
)
{
struct TPSDATA data;		/* Tie point selection data records 	*/
struct TPSHEAD header;		/* Tie point selection header records 	*/
struct PTDESC  *tie;			/* Pointer to tie point file descriptor */
long n;				/* Loop counters 			*/
long acc = IREAD;		/* File access mode 			*/
long ptyp = TPS;		/* Tie point type is TPS	 	*/
long status;			/* Return status flag 			*/
long lng_val[2];
char parmkey[20];	        /* Parmeter Key 			*/
char parmkey1[20];	        /* Parmeter Key 			*/
char space1[1][CMLEN];
double dbl_val[1];


/* Open TPS file
  -------------*/
if (c_ptopen(&tie, fname, &ptyp, &acc, &header) != E_SUCC) return(E_FAIL);

space1[0][0] = '\0';
n=1;
for(;;)
   {
   status = c_ptio(&tie, &data);
   if (status == E_EOF) break;
   if (status == E_FAIL) return(E_FAIL);

   lng_val[0] = (long) (data.image_coord[0] + 0.5);
   lng_val[1] = (long)(data.image_coord[1] + 0.5);

   sprintf(space1[0],"pt_%ld",n);
   sprintf(parmkey1,"pt%ld",n);
   c_putparm(filename, parmkey1, space1, "I", dbl_val, 2, lng_val);
   sprintf(parmkey1,"ptid%ld",n);
   c_putparm(filename, parmkey1, space1, "S", dbl_val, 1, lng_val);
/*
   fprintf(filep,"pt%-9ld I 2",n);  
   fprintf(filep," %ld",(long)(data.image_coord[0] + 0.5));
   fprintf(filep," %ld\n",(long)(data.image_coord[1] + 0.5));
   fprintf(filep,"ptid%-8ldS 1 pt_%ld\n",n,n); 
*/
   n++;
   }
n=n-1;
   lng_val[0] = n;
   sprintf(parmkey,"npts");
   c_putparm(filename, parmkey, space1, "I", dbl_val, 1, lng_val);
/*
fprintf(filep,"%-11s I 1 %d\n",parmkey,n);
*/
c_ptclse(&tie);
return(E_SUCC);
}
