/*****************************************************************************
NAME				TPS2PPF 

PURPOSE    Reads a tie point file (TPS) and writes the point coordinates, the
           pt_id descrition, and the number of points in the file to an ASCII
           project parameter file (.ppf).  To be used in process annotating
           points and their ids to an image.
        


PROGRAM HISTORY
VERSION    PROGRAMMER	   DATE     CODE/CONT	REASON
-------    ----------      ----     ---------   ------
  5.0      T. Jensen       10/92        EDC      Original development
  5.1	   T. Mittan	   12/92	EDC      Release to baseline



PROJECT			LAS

ALGORITHM DESCRIPTION
	Initialize LAS processing
        Get the ppf file name
	Open file for output
	Get the tie point file name 
	Write the tie points, their ids, and the number of points to ppf.
	Print completion message & exit
*****************************************************************************/
#include <stdlib.h>
#include "tps2ppf.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK  vblk;		/* TAE variable block 			*/
struct VARIABLE *var;		/* TAE variable value... 		*/
long  std_out;			/* Standard output LUN for inlas 	*/
long  mode = IREAD;		/* IREAD flag for getnam routine 	*/
long  fatal = -1;		/* Fatal error flag for c_errmsg 	*/
long  zero = 0;			/* Constant of zero 			*/
long  num_out = 0;		/* Number of output files 	 	*/
char  fname[CMLEN];		/* Host name of tie point file 		*/
char  filename[CMLEN];		/* Partial file name (wo/ext) for complt*/
char  out_file[CMLEN];		/* Name of output file for complt 	*/

/* Initialize LAS processing
  -------------------------*/
c_inlas(&vblk, &std_out);

if(c_getnam(&vblk, "INTS", &zero, fname, "tps", &mode) != E_SUCC)
     c_errmsg("Fatal error encountered","tps2ppf-fatal",&fatal);

mode= IWRITE;

if(c_getnam(&vblk, "OUTPPF", &zero, filename, "ppf", &mode) != E_SUCC)
       c_errmsg("Fatal error encountered","tps2ppf-fatal",&fatal);
var = p_find(&vblk,"OUTPPF");
strcpy(filename, SVAL(*var,0));

    
   if(show_tps(fname, filename) != E_SUCC)
       c_errmsg("Fatal error encountered","tps2ppf-fatal",&fatal);

/* Write completion message and exit program
  -----------------------------------------*/
c_complt("TPS2PPF", out_file, &num_out);

return EXIT_SUCCESS;

}
