/*******************************************************************************
NAME      		   BNDCHK 

PURPOSE Retrieve and check the band specification.  

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   TRANSLAS must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)
******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "protos.h"

int bndchk
(
    long bands[MAXBND+1], 	/* O: Band numbers specified    	*/
    long *inbands,		/* I: Number of bands in transfer file  */
    long *totbnd, 		/* O: Number of bands being converted   */
    struct PRS_LST p_list       /* I: Image list returned from imgprs   */
)

{
long i;				/* Loop index			     */
long nonfatal = 1;		/* nonfatal error flag		     */
char err_text[ERRLEN+1];	/* error message text                */

/* Initialize values
--------------------*/
*totbnd = 0;
for (i = 0; i < *inbands; i++)
   bands[i] = FALSE;

/* Check the band specification
 -------------------------------*/
if (p_list.bands[0] == 0) 
   {
   for (i = 0; i < *inbands; i++) 
      bands[i] = TRUE;
   *totbnd = *inbands;
   }
else 
   {
   i = 0;
   while (p_list.bands[i]>0) 
      {
      if (i >= MAXBND) 
         {
         sprintf( err_text, "%s %d %s",
                  "Error: Too many bands were specified.  Only",
                   MAXBND, "bands can be specified." );
         c_errmsg(err_text,"translas-invband",&nonfatal);
	 return(E_FAIL);
         }
      else if (p_list.bands[i] > *inbands)
         {
         sprintf( err_text, "%s %d %s%d%s",
                  "Error: Band", p_list.bands[i],
                  "is greater than the number of bands (",
                  *inbands, ") in the image" );
         c_errmsg(err_text,"translas-invband",&nonfatal);
         return(E_FAIL);
         }
      bands[p_list.bands[i]-1] = TRUE;
      *totbnd += 1;
      i++;
      }
   }

if (*totbnd > *inbands)
   {
   sprintf( err_text,"%s %d %s %d %s",
          "Number of bands specified (", *totbnd,
          ") is greater than the number of bands being converted (",
          *inbands, ")" );
   c_errmsg(err_text,"translas-invband",&nonfatal);
   return(E_FAIL);
   }

return(E_SUCC);
}
