/*******************************************************************************
NAME:			INDAL	

PURPOSE:
 INDAL reads records from the input DAL file(s) specified and writes them
 to a LAS image file.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    INDAL must be run under TAE.

PROJECT:			LAS

ALGORITHM REFERENCES:		none
******************************************************************************/


#include <stdio.h>
#include <math.h>
#ifdef aiws
#include <fcntl.h>
#endif
#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif
#include "las.h"	
#include "util.h"
#include "pixman.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc[MAXIMG+1];	/* image file descriptors used for image I/O */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

int indal
(
        struct PARBLK *vblock,          /* TAE parameter block                */
        char  hosout[][CMLEN],          /* host name of output image          */
        long  nfiles,                   /* number of transfer files           */
        long  nl,                       /* number of lines                    */
        long  ns,                       /* number of samples                  */
        long  nbands,                   /* number of bands                    */
        long  *window,                  /* window area of image to be copied  */
        long  bands[MAXBND+1],          /* bands to be processed              */
        long  nheader,                  /* number of header bytes             */
        long  nmrout,                   /* number of output images            */
        char  ioname[][CMLEN],          /* array of input image names         */
        long  idtype,                   /* input data type                    */
        long  odtype,                   /* output data type                   */
        long  msgtim,                   /* message time                       */
        char  *minmax,                  /* min max flag                       */
        long  dflag,                    /* ddr flag option                    */
        long  totbnd                    /* number of bands to be converted    */
) 
  
{
	long 	acc=IWRITE;		/* Image file open access	      */
	long	block;			/* Loop counter for i/o blocking      */
	long	block2;			/* Loop counter for i/o blocking      */
        long  cb;                       /* current band being copied          */
	long    fatal = -1;		/* fatal error flag		      */
	long 	flag = ONECAT;		/* Display message flag		      */
	long    i;			/* index counter		      */
	FILE 	*infd;			/* Input file descriptor	      */
	long	nbytes;			/* number of bytes per image line     */
	long    nonfatal = 1;		/* nonfatal error flag		      */
	long    one = 1;		/* the number one for errors	      */
static	long	opt[3]={0,0,0};		/* Image file open option	      */
        long    insize;                 /* Number of bytes/pixel input dtype  */
	long 	sl=1;			/* Image file starting line	      */
	long	ss=1;			/* Image fiel starting sample	      */
	long 	status;			/* Status indicator		      */

double *minval;		/* min pixel value array	      */
double *maxval;		/* max pixel value array	      */


unsigned char	*thebuf;		/* Input buffer			      */
unsigned char	*header_rec;		/* Header record buffer	              */
unsigned char   *in_ptr;                /* pointer to input buffer window     */

struct  DDR ddr;		/* ddr structure		      */
struct  BDDR bddr[MAXBND];      /* band ddr structure                 */


if ((minval = (double *) malloc(totbnd * sizeof(double))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","translas-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","translas-fatal",&fatal);
   }

if ((maxval = (double *) malloc(totbnd * sizeof(double))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","translas-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","translas-fatal",&fatal);
   }

/* Initialize min/max arrays
----------------------------------------*/
if (strcmp(minmax,"YES") == 0)
   c_inmnmx(&idtype,&totbnd,minval,maxval);
	
/* Determine the number bytes/pixel for input dtype
-------------------------------------------------*/
switch(idtype)
   {
   case EBYTE:  insize = 1; break;
   case EWORD:  insize = 2; break;
   case ELONG:	insize = 4; break;
   case EREAL:  insize = 4; break;
   }

/* set opt array to convert to the output data type
-------------------------------------------------*/
opt[0] = odtype;


/* Open the output image file(s) for write access
-------------------------------------------------*/
if (nmrout == 1)
   {
   status = c_eopenr((struct FDESC **)&fdesc[0],hosout[0],&totbnd,&sl,&ss,
		      &window[NL],&window[NS], &acc, &idtype, opt, &one);	
   if ( status != E_SUCC)
      c_errmsg("Fatal error encountered", "translas-fatal",&fatal);
   }
else
   for (i = 0; i < nmrout; i++)
      {
      status = c_eopenr((struct FDESC **)&fdesc[i],hosout[i],&one,&sl,&ss,
		         &window[NL],&window[NS], &acc, &idtype, opt, &one);
      if ( status != E_SUCC)
         c_errmsg("Fatal error encountered", "translas-fatal",&fatal);
      }

/* Allocate the input buffer
-------------------------------------------------*/
if ((thebuf = (unsigned char *) malloc(ns*insize)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","translas-alloc", &nonfatal);
   for(i = 0; i < nmrout; i++)
      if (c_eclose((struct FDESC **)&fdesc[i]) != E_SUCC)
         c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
   c_clean(vblock,"translas",hosout,&nmrout);
   }
if (nheader != 0)
   {
   if ((header_rec = (unsigned char *) malloc(nheader)) == NULL)
      {
      c_errmsg("Error allocating dynamic memory","translas-alloc", &nonfatal);
      for(i = 0; i < nmrout; i++)
         if (c_eclose((struct FDESC **)&fdesc[i]) != E_SUCC)
            c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
      c_clean(vblock,"translas",hosout,&nmrout);
      }
   }

/* Initialize the processing message
----------------------------------------*/
curcat=1;
curent=1;
c_inittm("TRANSLAS","Processed","line","bands"," ",&nl, &totbnd,&msgtim,&flag);
c_startm();

/* Open the input file
-----------------------*/
if (nfiles == 1)
   {
   infd = fopen(ioname[0],"r");
   if (infd == NULL)
      {
      if (c_eclose((struct FDESC **)&fdesc[0]) != E_SUCC)
         c_errmsg("Nonfatal error encountered","translas-warn", &nonfatal);
      c_lasmsg(TRUE,"translas-inopen",
                  "Error opening input TRANSFER image %s", ioname[0]);
      c_clean(vblock,"translas",hosout,&nmrout);
      }
   }

cb = 0;

/* Loop on each band in the image
----------------------------------------*/
for (curcat=1; curcat<=nbands; curcat++)
   {

   if (bands[curcat-1])
      {
      cb++;
      /* Open the input file if a separate input file was specified 
         for each band 
      -------------------------------------------------------------*/
      if (nfiles != 1)
         {
	 infd = fopen(ioname[cb-1], "r");
         if (infd == NULL)
	    {
            for(i = 0; i < nmrout; i++)
	       if (c_eclose((struct FDESC **)&fdesc[i]) != E_SUCC)
	          c_errmsg("Nonfatal error encountered","translas-warn",
                           &nonfatal);
	    c_lasmsg(TRUE,"translas-inopen",
                    "Error opening input TRANSFER image %s",ioname[cb-1]);
       	    c_clean(vblock,"translas",hosout,&nmrout);
            }
	 }

      /* Read header record
      ------------------*/
      if (nheader != 0)
	 if (((nfiles != totbnd) && (curcat == 1)) || (nfiles == totbnd))
	    {
	    status = read_input_file(infd,(char *)header_rec,nheader); 
	    if (status < nheader)
	       {
               for(i = 0; i < nmrout; i++)
	          if (c_eclose((struct FDESC **)&fdesc[i]) != E_SUCC)
	             c_errmsg("Nonfatal error encountered","translas-warn",
								&nonfatal);
	       c_errmsg("Error reading header record of input image",
					    "translas-header",&nonfatal);
	       c_clean(vblock,"translas",hosout,&nmrout);
	       }
	    }

/* For each block in the input file read a block from the DAL file and write
   it to the LAS image file
---------------------------------------------------------------------------*/
      curent = 1;
      nbytes = ns * insize;
      if (window[SL] > 1)
         {
         status = fseek(infd, ((window[SL] - 1) * nbytes), 1);
         }
      for (block=0; block < window[NL]; block++)
         {
         block2 = block + 1;
         if (read_input_file(infd,(char *)thebuf,nbytes) != nbytes)
	    {
            for(i = 0; i < nmrout; i++)
	       if (c_eclose((struct FDESC **)&fdesc[i]) != E_SUCC)
	          c_errmsg("Nonfatal error encountered","translas-warn", &nonfatal);
	    c_errmsg("Error reading input TRANSFER image","translas-read",
								 &nonfatal);
	    c_clean(vblock,"translas",hosout,&nmrout);
	    }
         in_ptr = &thebuf[(window[SS] - 1) * insize];

         if (strcmp(minmax,"YES") == 0)
	    {
	    status = c_pxmnmx ((long *)in_ptr,&window[NS],&minval[curcat-1], 
                               &maxval[curcat-1], &idtype);
	    if (status != E_SUCC)
	       c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
	    }
         if (nmrout == 1)
            status = c_ewrite((struct FDESC **)&fdesc[0], &cb, &block2, in_ptr,
                              &one);
         else
            status = c_ewrite((struct FDESC **)&fdesc[cb-1], &one, &block2,
                               in_ptr, &one);
         if (status == E_FAIL)
	    {
	    for(i = 0; i < nmrout; i++)
	       if (c_eclose((struct FDESC **)&fdesc[i]) != E_SUCC)
	          c_errmsg("Nonfatal error encountered","translas-warn", &nonfatal);
	    c_clean(vblock,"translas",hosout,&nmrout);
	    }
         curent ++;
         }      	/* end of block loop */
      }
   else if (nfiles == 1)
      if ((nheader != 0) && (curcat == 1))
         fseek(infd,((nheader + (nl * ns)) * insize),1);
      else
         fseek(infd,(nl * ns * insize),1);
   }		     /* end of curcat loop */

/* Free the buffer space previously allocated
-------------------------------------------------*/
free (thebuf);

/* Free the buffer space previously allocated
-------------------------------------------------*/
if (nheader != 0)
   free (header_rec);

/* Update the DDR record file
-------------------------------------------------*/
status = c_getddr((char *)hosout,&ddr);
for (i = 1; i < totbnd+1; i++)
   if (nmrout == 1)
      c_getbdr(hosout[0],&bddr[i-1],&i);
   else
      c_getbdr(hosout[i-1],&bddr[0],&one);
for (i = 0; i < 8; i++)
   ddr.valid[i] = dflag;
for (i = 0; i < nmrout; i++)
   status = c_putddr(hosout[i],&ddr);
for (i = 0; i < totbnd; i++)
   if (nmrout == 1)
      c_putbdr((char *)hosout,&bddr[i]); 
   else
      c_putbdr(hosout[i],&bddr[0]);


/* Update the associated min/max file
-------------------------------------------------*/
if (nmrout == 1)
   {
   if (c_svmnmx ((char *)hosout,&totbnd,minval,maxval,minmax) != E_SUCC)
      c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
   }
else
   {
   for (i = 0; i < nmrout; i++)
      if (c_svmnmx(hosout[i],&one,&minval[i],&maxval[i],minmax) != E_SUCC)
         c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
   }
free(minval);
free(maxval);

/* Close the input and output files
-------------------------------------------------*/
fclose(infd);
for (i = 0; i < nmrout; i++)
   c_eclose((struct FDESC **)&fdesc[i]);
c_stoptm();
return(E_SUCC);
} /* End of indal */


/*******************************************************************************
read input file  will read from the file into the buffer, but will read until
		 the specified number of bytes have been read, disregarding
		 the record and block boundaries encoutered in vms disk files.
*******************************************************************************/
long read_input_file
(
	FILE *infd,			/* I: input file descriptor     */
	char *buf,			/* I: read buffer		*/
	long nbytes 			/* I: number of bytes to read   */
)
{
int size;					/* int num bytes to read */
long one = 1;					/* 1 */
long status;					/* return status */
long total;					/* bytes read for return */
char *ptr;					/* pointer for read buf */

ptr = buf;
size = nbytes;
for (total = 0; total < nbytes; total += status)
    {
    status = (long)fread(ptr,1,size,infd);
    if (status == 0)
        {
        c_errmsg("Unexpected EOF encountered","translas-eof",&one);
	return(E_FAIL);
        }
    else if (status < 0)
	{
	c_errmsg("Error reading input TRANSFER image","translas-read",&one);
	return(E_FAIL);
	}
    ptr += status;
    size -= status;
    }
return(total);
}
