/******************************************************************************
NAME:			OUTDAL	

PURPOSE: OUTDAL reads LAS images and converts them to the host operating
	 system disk files

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
    OUTDAL must be run under TAE.

PROJECT:			LAS

ALGORITHM DESCRIPTION:
	Open the input images for read access 
	Open the output files for write access
	The data is read and converted to image disk file
	The files are closed

ALGORITHM REFERENCES:		none
******************************************************************************/

#include <stdio.h>
#include <math.h>
#ifdef aiws
#include <fcntl.h>
#endif
#ifdef vms
#include <file.h>
#include <rms.h>
#include <ssdef.h>
#include <worgen.h>
#include "desc.h"
#else
#include <sys/file.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif
#include "las.h"	
#include "diskio.h"
#include "util.h"
#include "protos.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc[MAXIMG+1];	/* image file descriptors used for image I/O */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */

int outdal
(
	struct PARBLK *vblock,          /* I: TAE parameter block            */
	char  hosin[][CMLEN],		/* I: host name of input image       */
        long  nfiles,                   /* I: number of transfer files       */	
        long  window[][4],              /* I: window specifier               */
	long  bands[][MAXBND+1],	/* I: Band numbers specified         */
        long  *nbands,                  /* I: number of bands                */
        long  odtype,                   /* I: output data type               */
        char  ioname[][CMLEN],          /* I: Output file names              */
	long  msgtim,			/* I: message time counter	     */
	long  nmrimg,			/* I: number of input images         */
	long  totbnd 			/* I: total number of bands          */
)
{
#ifdef vms
	struct DESC ionamedsc;		/* structure for file name	     */
	struct RAB  *outfdsc[MAXIMG];	/* structure for file descriptors    */
	struct RAB  *alocfd();		/* function to allocate fd's	     */
#endif
	long 	acc=IREAD;		/* Image file open access	     */
	long	block;			/* Loop counter for i/o blocks       */
	long	cur_band;		/* Loop control variable	     */
	long    fatal = -1;		/* fatal error flag		     */
	long long filesize;		/* file to be created size in bytes  */
	long 	flag = ONECAT;		/* Display message flag		     */
	long	image;			/* Image loop counter		     */
	long    nbnd;			/* open and read the number of bands */
	long	nl;			/* number of lines		     */
	long    nonfatal = 1;		/* nonfatal error flag		     */
	long    ns;			/* number of samples		     */
        long    one = 1;                /* number of output names in c_clean */
static	long	opt[3]={0,0,0};		/* Image file open option	     */
	long	outfd;			/* Output file descriptor	     */
	long    recsize;		/* lines to be created size in bytes */
	long	size;    		/* Number of bytes/pixel	     */
	long 	sl=1;			/* Image file starting line	     */
	long	ss=1;			/* Image field starting sample	     */
	long 	status;			/* Status indicator		     */
	long    wait = 600;             /* seconds between dkcre() retries   */ 

	char	*filename;		/* name of output file		     */

unsigned char	*thebuf;		/* Input buffer			     */
unsigned char	defacc = 0;		/* default access flg for output file*/

/*  allocat memory
------------------*/
if ((filename = (char *) malloc(CMLEN * sizeof(char))) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","translas-alloc",&nonfatal);
   c_errmsg("Fatal error encountered","translas-fatal",&fatal);
   }

/* Determine the number bytes/pixel
-------------------------------------------------*/
switch(odtype)
   {
   case EBYTE:  size = 1; break;
   case EWORD:  size = 2; break;
   case ELONG:	size = 4; break;
   case EREAL:  size = 4; break;
   }

/* Open the image file for read access	
-------------------------------------------------*/
for (image = 0; image < nmrimg; image++)
   {
   sl = window[image][SL];
   ss = window[image][SS];
   nl = window[image][NL];
   ns = window[image][NS];
   acc = IREAD;
   opt[0] = 0;
   nbnd = 0;
   status = c_eopenr((struct FDESC **)&fdesc[image], hosin[image], &nbnd, &sl,
                      &ss, &nl, &ns, &acc, &odtype, opt, &one);
   if ( status != E_SUCC)
      c_errmsg("Fatal error encountered", "translas-fatal",&fatal);
   }

/* Allocate the input buffer
-------------------------------------------------*/
if ((thebuf = (unsigned char *) malloc(ns * size)) == NULL) 
   {
   if (c_eclose((struct FDESC **)fdesc) != E_SUCC)
      c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
   c_errmsg("Error allocating dynamic memory","translas-alloc", &nonfatal);
   c_errmsg("Fatal error encountered","translas-fatal",&fatal);
   }

/* Initialize the processing message
-------------------------------------------------*/
curcat=1;
curent=1;
c_inittm("TRANSLAS","Processed","line","bands"," ",&nl, &totbnd,&msgtim,&flag);
c_startm();

/* Open the output file each time through the loop if a separate output
   name was specified for each band or open the output file the first
   time through the loop if only one name was specified
-------------------------------------------------------------------------*/

curcat=0;
for (image = 0; image < nmrimg; image++)
 {
 for (cur_band = 0; cur_band < nbands[image]; cur_band++)
   {
   curcat++;
   /* Case of each band to separate output file 
   ---------------------------------------------*/
   if (nfiles == totbnd)
      {
      filesize = (long long)nl * ns * size;
      recsize = ns * size;
			/* open file - UNIX */
#ifndef vms
         status = c_dkcre(NULL,ioname[curcat-1],filesize,wait);
         if (status != E_SUCC) 
	    {
   	    if (c_eclose((struct FDESC **)fdesc) != E_SUCC)
               c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
	    if (status == DISK_EXISTS)
	       c_errmsg("The TRANSFER file already exists",
					        "translas-dkcre",&fatal);
            else 
               c_errmsg("Error searching for disk space",
                              			"translas-disk",&fatal);
	    }
	 outfd = open(ioname[curcat-1], O_CREAT | O_WRONLY | O_TRUNC, 0666);
	 if (outfd < 0)
	    { 
   	    if (c_eclose((struct FDESC **)fdesc) != E_SUCC)
               c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
	    c_errmsg("Error opening output TRANSFER image", "translas-outopen",
								&nonfatal);
            c_clean(vblock,"translas",ioname,&one);
	    }
			/* open file - VMS */
#else
	 strcpy(filename,ioname[curcat-1]);
	 ionamedsc.addr = filename;
	 ionamedsc.length = strlen(ioname[curcat-1]);
	 outfdsc[0] = alocfd();
	 status = creatf(&ionamedsc,outfdsc,&defacc);
         if (status != 65537)
            {
   	    if (c_eclose(fdesc) != E_SUCC)
               c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
            closef(outfdsc);
	    c_errmsg("Error opening output TRANSFER image", "translas-outopen",
								&nonfatal);
            c_clean(vblock,"translas",ioname,&one);
            }
#endif
      }

   /* Case of all bands to the same output file -- BSQ format 
    --------------------------------------------------------*/
   else if (curcat == 1)
      {
      filesize = (long long)nl * ns * size * totbnd;
      recsize = ns * size;
      status = c_dkcre(NULL,ioname[0],filesize,wait);
      if (status != E_SUCC) 
	 {
   	 if (c_eclose((struct FDESC **)fdesc) != E_SUCC)
            c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
	 if (status == DISK_EXISTS)
	    c_errmsg("The TRANSFER file already exists",
					        "translas-dkcre",&fatal);
         else 
            c_errmsg("Error searching for disk space",
                              			"translas-disk",&fatal);
	 }

#ifndef vms			/* open file - UNIX */
	 outfd = open(ioname[0], O_CREAT | O_WRONLY | O_TRUNC, 0666);
	 if (outfd < 0)
	    { 
   	    if (c_eclose((struct FDESC **)fdesc) != E_SUCC)
               c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
	    c_errmsg("Error opening output TRANSFER image", "translas-outopen",
								&nonfatal);
            c_clean(vblock,"translas",ioname,&one);
	    }
#else				/* open file - VMS */
	 strcpy(filename,ioname[0]);
	 ionamedsc.addr = filename;
	 ionamedsc.length = strlen(ioname[0]);
	 outfdsc[0] = alocfd();
	 status = creatf(&ionamedsc,outfdsc,&defacc);
         if (status != 65537)
            {
   	    if (c_eclose(fdesc) != E_SUCC)
               c_errmsg("Nonfatal error encountered","translas-warn",&nonfatal);
            closef(outfdsc);
	    c_errmsg("Error opening output TRANSFER image", "translas-outopen",
								&nonfatal);
            c_clean(vblock,"translas",ioname,&one);
            }
#endif
      }

/* For each block in the input file, read a block from the LAS image file
   and write out in DAL format
-------------------------------------------------------------------------*/
   curent = 1;
   for (block=0; block < nl; block++, curent ++)
      {
      status = c_eread((struct FDESC **)&fdesc[image], &bands[image][cur_band],
                       &curent, thebuf,&one);
      if (status == E_FAIL)
         {
   	 if (c_eclose((struct FDESC **)fdesc) != E_SUCC)
            c_errmsg("Nonfatal error encountered","translas-warn", &nonfatal);
#ifndef vms
 	 close(outfd);
#else
	 closef(outfdsc);
#endif
         c_clean(vblock,"translas",ioname,&one);
         }
#ifndef vms
	    status = write(outfd, thebuf, ns * size);
	    if (status < 0)
                {
   	        if (c_eclose((struct FDESC **)fdesc) != E_SUCC)
                   c_errmsg("Nonfatal error encountered","translas-warn",
								&nonfatal);
	        c_errmsg ("Error writing output TRANSFER image", 
						"translas-write", &nonfatal);
		close(outfd);
                c_clean(vblock,"translas",ioname,&one);
                }
#else
	    status = putrec(outfdsc,thebuf,&recsize);
	    if (status != 65537)
               {
   	       if (c_eclose(fdesc) != E_SUCC)
                  c_errmsg("Nonfatal error encountered","translas-warn",
								&nonfatal);
               closef(outfdsc);
	       c_errmsg("Error writing output TRANSFER image", 
						"translas-write", &nonfatal);
               c_clean(vblock,"translas",ioname,&one);
               }
#endif
      }

/* Close output files 
--------------------------*/
   if ((nfiles == totbnd)||(curcat == totbnd))
      {
#ifndef vms
         close(outfd);
#endif
      status = c_dkfree(ioname[curcat-1]);
      if (status != E_SUCC)
         c_errmsg("Error freeing disk space", "translas-dkfree",&nonfatal);
      }

   } 				/* END OF curcat<=nbands FOR LOOP */
 }
free(filename);
#ifdef vms
   closef(outfdsc);
#endif

/* Free the buffer space previously allocated
------------------------------------------------*/
free (thebuf);

/* Close the input files
------------------------------------------------*/
c_eclose((struct FDESC **)fdesc);
c_stoptm();
return(E_SUCC);

}  				/* End outdal */
