/*******************************************************************************
NAME      		   WNDCHK 

PURPOSE  Check that the specified windows are all the same size and are valid. 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   TRANSLAS must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	
******************************************************************************/
#include <stdio.h>
#include "las.h"	
#include "protos.h"

int wndchk
(
    long window[][4], 		/* I: Window specification	 	*/
    long *nl,            	/* I: Number of lines in input image    */
    long *ns,            	/* I: Number of samples in input image  */
    long *nfiles		/* I: Number of transfer files          */
)

{

long i;				/* Loop index			     */
long nonfatal = 1;		/* nonfatal error flag		     */


/* Check that sl,ss,nl, and ns are all positive
-----------------------------------------------*/
if (window[0][SL] < 0)
   {
   c_errmsg("Starting line must be positive","translas-wind",&nonfatal);
   return(E_FAIL);
   }
if (window[0][SS] < 0)
   {
   c_errmsg("Starting sample must be positive","translas-wind",&nonfatal);
   return(E_FAIL);
   }
if (window[0][NL] < 0)
   {
   c_errmsg("Number of lines must be positive","translas-wind",&nonfatal);
   return(E_FAIL);
   }
if (window[0][NS] < 0)
   {
   c_errmsg("Number of samples must be positive","translas-wind",&nonfatal);
   return(E_FAIL);
   }

/* Check that windows are the same size
---------------------------------------*/
for (i = 1; i < *nfiles; i++)
   {
   if ((window[i-1][SL] != window[i][SL]) ||
      (window[i-1][SS] != window[i][SS]) ||
      (window[i-1][NL] != window[i][NL]) ||
      (window[i-1][NS] != window[i][NS])) 
      {
      c_errmsg("Input images do not have the same window size",
               "translas-size",&nonfatal);
      return(E_FAIL);
      }
   }

/* Check window specification
-----------------------------*/
if (window[0][SL] == 0)
   window[0][SL] = 1;
if (window[0][SS] == 0)
   window[0][SS] = 1;
if (window[0][NL] == 0)
   window[0][NL] = *nl;
if (window[0][NS] == 0)
   window[0][NS] = *ns;
if (window[0][SL] > *nl)
   {
   c_errmsg("Starting line must be less than total lines",
            "translas-start",&nonfatal);
   return(E_FAIL);
   }
if ((window[0][SL] + window[0][NL] -1) > *nl)
   {
   c_errmsg("Too many lines specified",
            "translas-number",&nonfatal);
   window[0][NL] = *nl - window[0][SL] +1;
   }
if (window[0][SS] > *ns)
   {
   c_errmsg("Starting sample must be less than total samples",
            "translas-start",&nonfatal);
   return(E_FAIL);
   }
if ((window[0][SS] + window[0][NS] -1) > *ns)
   {
   c_errmsg("Too many samples specified",
            "translas-number",&nonfatal);
   window[0][NS] = *ns - window[0][SS] + 1;
   }

return(E_SUCC);
}
