#include "las.h"
#include "worgen.h"
#include "util.h"
#include "protos.h"


int calc_scale
(
    float *minpix,	/* Minimum pixel value. (I)			*/
    float *maxpix,	/* Maximum pixel value. (I)			*/
    float scoff[]	/* Slope and offset to be computed by calc_scale. (O)*/
)

/******************************************************************************
 *									      *
 *	Purpose: This routine will take minimum and maximum values of the     *
 *		 input data type and compute the slope and offset needed to   *
 *		 scale all values to the specified range using the following  *
 *		 formula: 						      *
 *			output pixel = input pixel * slope + offset.	      *
 *									      *
 *	By D. Akkerman, USGS/EROS Data Center, 11/89.			      *
 *	 - Adapted from the DMS dtcvrt routine.				      *
 *									      *
 *	Algorithm (flow):						      *
 *									      *
 *	See in-line documentation.					      *
 *									      *
 ******************************************************************************/
{
char	msg[132];	/* Informational message string.		*/
long	one = 1,  	/* Specifies a nonfatal error message.		*/
	status;	  	/* Return status variable.			*/

double	maxout,minout;	/* Maximum & minimum value for output data type.*/

extern long intype;
extern long out_type;


/* Obtain minimum & maximum for output data type.  Note that the maxout and
   minout parameters are in reverse order to allow the inmnmx routine to
   assign these correctly.						*/

status = c_inmnmx(&out_type,&one,&maxout,&minout);
if (status != E_SUCC)
  {
  c_errmsg("Invalid data type","unary-bdata",&one);
  return(status);
  }

/* If the input data type is REAL then always scale the range. This is 
   because even a small REAL data range (say 0 to 3) could have a very
   large mantissa and thus require scaling, even though the integer
   portion is within the BYTE range.  					*/

if (intype == EREAL)
   {
    scoff[0] = (maxout - minout)/(*maxpix - *minpix);
    scoff[1] = minout - (scoff[0] * (*minpix));
   }
else 
   {
       if ((*minpix < minout) || (*maxpix > maxout))
            {
            if ((*maxpix - *minpix) <= (maxout - minout))
	            {
                scoff[1] = minout - *minpix;
                }
            else
	            {
	            scoff[0] = (maxout - minout)/(*maxpix - *minpix);
	            scoff[1] = minout - (scoff[0] * (*minpix));
	            }
            }
    }
if ((scoff[0] != 1.0) && (scoff[1] != 0.0))
 {
  sprintf(msg,"Mapped %f : %f    to      %f : %f\n",*minpix,*maxpix,
               minout,maxout);
  c_errmsg(msg,"informational",&one);
  sprintf(msg,"Slope = %.10f       Offset = %.10f\n",scoff[0],scoff[1]);
  c_errmsg(msg,"informational",&one);
 }

return(E_SUCC);
}
