#include "unary.h"
#include "util.h"
#include "protos.h"

void filspec
(
    char inhos[][CMLEN],     /* input image host names                        */
    char outhos[],           /* output image host name                        */
    char fopt[4],            /* math function option */
    long *num_inimages,	    /* number of input images			*/
    long num_image_bands[],       /* number of bands in each input image*/
    long image_band_num[][MAXBND + 1], /* band numbers in each input image   */
    long *num_outbands,	/* number of output bands */
    long wind[][4],	/* input image window */
    long out_img_bands[][MAXBND+1]  /* band numbers for the output image*/
)
	
/*
**************************************************************************
NAME: FILSPEC

PURPOSE: Set up file specifications.

PROGRAM HISTORY:
  Version	Date       Author	           Request
  -------	----	   ------                  -------
   1.0         01-Feb-90   W. Xia/STX         conversion to 5.0

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	 Runs under TAE

PROJECT:	LAS

ALGORITHM DESCRIPTION:

   Open input image files.
   Open the output image file.                                               
   Update the ddr file.
   Group all the images files into a group file
**************************************************************************
*/
{
/* local variables */

double	line_inc = 1.0,		/* line and sample increments for          */
	samp_inc = 1.0;		/* output image ddr                        */

long	access,			/* file access type                        */
	bufsiz,			/* image i/o buffer size                   */
	comb_flag=0,		/* image combination flag 		   */
        n, nib,			/* loop index */
	stat;			/* return status                           */
long    fatal_err = -1;                /* fatal error return status */
long    zero = 0;
long    one = 1;

extern long intype;
extern long out_type;

static long	opt[3]={0,0,0};	/* optional parameter array for output file */

opt[0]=0;
/*
Open input image files.                       
-----------------------                  	*/
if (strcmp(fopt, "NOT") == 0)
	intype = out_type;
else
	intype = EREAL;

access = IREAD;

for (n = 0, nib = 0; n < *num_inimages; n++)
{
	stat = c_eopens( &fdesc[n], inhos[n], image_band_num[n],
			    &wind[0][SL], &wind[0][SS], &wind[0][NL],
			    &wind[0][NS], &ipoint[nib], &access, 
			    &intype, &zero, opt);
	
	if (stat != E_SUCC )
		c_errmsg ("Error opening input image",
			  "unary-eopens",&fatal_err);
	   nib += num_image_bands[n];
}

/*
Open the output image file.
--------------------------				*/
access = IWRITE;
if (strcmp(fopt, "NOT") == 0)          
   {  
   stat = c_eopens (&fdesc[n], outhos, num_outbands, &one, &one,
		  &wind[0][NL], &wind[0][NS], opoint, &access, 
	       	  &out_type, &zero, opt); 
   }  
else
   {
	opt[0]=out_type;
	opt[1]=0;
	stat = c_eopens (&fdesc[n], outhos, num_outbands, &one, &one,
		  &wind[0][NL], &wind[0][NS], opoint, &access, 
		  &intype, &zero, opt);
   }	       
if (stat != E_SUCC )       
   {	       
   c_abort (&block, "Error opening output image", "unary-outopen",
               "unary", out_files, &num_outfiles);
   } 
fdesc[n+1]=0;
	          
for (n = 0; n < *num_outbands; n++)
	out_img_bands[0][n] = n + 1;
	
/*
Update the ddr file.
--------------------							*/
c_upddr (&block, inhos, outhos, num_inimages, wind, image_band_num, 
	 num_image_bands, &line_inc, &samp_inc, num_outbands, &comb_flag);
	
	  
/*
Set up group containing input, output bands.  Handle 
allocation for NOT subcommand as a special case.
----------------------------------------------------			*/
if (strcmp(fopt, "NOT") == 0)
{
	switch (out_type)
	{
		case EBYTE:
			if ((biobuf = (unsigned char *) 
				      c_ealloc(fdesc, &bufsiz)) == NULL)
			{
				   c_errmsg ("Error allocating dynamic memory",
					     "unary-ealloc",&fatal_err);
			}
			c_egroup (fdesc, &gdesc, biobuf, &bufsiz);
			break;

		case EWORD:
			if ((siobuf = (short *) 
			     c_ealloc(fdesc, &bufsiz)) == NULL)
			   	c_errmsg ("Error allocating dynamic memory",
					     "unary-ealloc",&fatal_err);
			c_egroup (fdesc, &gdesc, siobuf, &bufsiz);
			break;

		case ELONG:
			if ((liobuf = (long *) 
			    c_ealloc(fdesc, &bufsiz)) == NULL)
		   		c_errmsg ("Error allocating dynamic memory",
					     "unary-ealloc",&fatal_err);
			c_egroup (fdesc, &gdesc, liobuf, &bufsiz);
			break;

		case EREAL:
			if ((riobuf = (float *) 
			    c_ealloc(fdesc, &bufsiz)) == NULL)
				c_errmsg ("Error allocating dynamic memory",
					  "unary-ealloc",&fatal_err);
			c_egroup (fdesc, &gdesc, riobuf, &bufsiz);
			break;

	}	/* Switch  */
}
else
{
	if ((iobuf = (float *) c_ealloc(fdesc, &bufsiz)) == NULL)
		   c_errmsg ("Error allocating dynamic memory",
			     "unary-ealloc",&fatal_err);

	c_egroup (fdesc, &gdesc, iobuf, &bufsiz);
}
}
