#include "unary.h"
#include "util.h"
#include "protos.h"
         
void getpar
(
    char intae[][CMLEN],         /*    input image tae name       */
    char inhos[][CMLEN],         /*    input image host name      */
    char outtae[][CMLEN],         	/*    output image tae name      */
    char outhos[][CMLEN],         	/*    output image host name     */
    long wind[][4], 		/* input image window		*/
    long *num_inimages,		/* number of input images           */
    long num_image_bands[],	/* number of bands in each input image */
    long image_band_num[][MAXBND+1],  /* band numbers for each image */
    long *num_inbands,           /* total number of input image bands   */
    long *num_outbands,		/* number of output image bands     */
    char fopt[4],            	/* math function option */
    float *base,               	/* log/exp base */
    float *exsc,               	/* scale for exp input images */
    float scoff[],           	/* scale and offset */
    long *scflg,                     /* scale/no scale flag */
    long *calcscflg,             /* calculate/don't calculate scale flag */
    float *pscale,            	/* scale for sine and cosine  */
    float *phase              	/* offset for sine and cosine */
)

/*
******************************************************************
NAME: GETPAR

PURPOSE:
	getpar is used to receive the paramter block from tm,
	initialize the catalog manager, and to get the parameters
	from the parameter block.

PROGRAM HISTORY:
  Version   Date       Author                       Request
  -------   ----       ------                       -------
                          
  1.0     30-Mar-90    W. Xia			conversion to 5.0
                 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS
     
ALGORITHM DESCRIPTION:
                      
   Call c_inlas to initialize TAE.
   Get the input image file name.
   Get the output image file name.
   Get the data type of input images.
   Get requested function names and parameters.
   Get the LAS global parameters: $MAXTIM, $MINMAX.

*************************************************************************
*/
{

/*  local variables */

struct  VARIABLE *var;               /* pointer to a variable in TAE block */

char    mnmx[4],      		/* minimum and maximum option */
	od_type[5];		/* Output data type specification buffer */

long    dtype=EREAL,            /* data type */                    
        otype = 0,              /* request output type */
	imglimit=MAXIMG, 	/* maximum number of image allowed */
 	inimg_flag[MAXIMG][2],	/* flag for subwindow */             
        len,
	same_wnd,	/* flag indication if images have same window */
	stat,                       /* return status                      */
  	std_out,     	       	/* standard output device  */ 
        intype = 0;         /* data type of input image */
long    fatal_err = -1;                /* fatal error return status */
long    zero = 0;
long    one = 1;
long    two = 2;

/*
Initialize with TAE etc.
________________________			*/
c_inlas(&block,&std_out);

len = 5;
  
/*
Get the input image name                       
------------------------						*/
stat = c_getin (&block, "IN", &zero, intae, inhos, num_inimages, wind,
		image_band_num, num_image_bands, num_inbands, &imglimit, 
		&same_wnd, inimg_flag);    
                                              
*num_outbands = *num_inbands;            

if (stat != E_SUCC)
	c_errmsg ("Cannot access input file names",
		  "unary-getin", &fatal_err);

if (same_wnd == 0)                                  
		c_errmsg ("Image windows differ in size", 
			  "unary-window", &fatal_err);

/*                      
Get the output image file name.
-------------------------------					*/
stat = c_getout (&block, "OUT", outtae, outhos);  

num_outfiles=0;
strcpy (out_files[num_outfiles++], outtae[0]);

if (stat != E_SUCC)
   c_errmsg ("Cannot access output file name", "unary-getout", &fatal_err);

/*
Get requested function name
---------------------------                                     */
c_getstr (&block, "_SUBCMD", &one, &len, fopt);
if ((strcmp(fopt,"COS") == 0) || (strcmp(fopt,"SIN") == 0))
{
        c_getnum(&block, "SCALFACT", &one, &dtype, pscale);
        c_getnum (&block, "OFFSET",  &one, &dtype, phase);
}

/*     
Get the data type of input/output image
---------------------------------------				*/
c_getstr (&block, "ODTYPE", &one, &len, od_type);
if ((strcmp(fopt,"SCA") == 0) || (strcmp(od_type,"SAME") == 0))
{
     c_getddr(inhos[0], &ddr);
     intype = ddr.dtype;
}
if (strcmp(od_type,"SAME") == 0)
     otype = ddr.dtype;
else if (strcmp(od_type,"BYTE") == 0)
     otype = EBYTE;
else if (strcmp(od_type,"I*2") == 0)
     otype = EWORD;
else if (strcmp(od_type,"I*4") == 0)
     otype = ELONG;
else if (strcmp(od_type,"R*4") == 0)
     otype = EREAL;

out_type=otype;


/*
If needed, get base for LOG, EXP or scoff.
------------------------------------------			*/
if (strcmp(fopt, "NOT") != 0)
{
   if (strcmp(fopt, "SCA") == 0)
   {
      var = p_find(&block,"SCOFF");
      if (var->v_count == 0)
         if(out_type < intype)
            *calcscflg=TRUE;
         else
         {
            scoff[0]=1.0;
            scoff[1]=0.0;
            *calcscflg = FALSE;
         }
      else
      {
         *calcscflg=FALSE;
         scoff[0] = RVAL(*var,0);
         scoff[1] = RVAL(*var,1);
      }
   }
   else
   {
      c_getnum (&block, "SCOFF", &two, &dtype, scoff); 
      *calcscflg = FALSE;
   }
   if (scoff[0] != 1.0 || scoff[1] != 0.0)
          *scflg = TRUE;
   else   *scflg = FALSE;
}
if (( strcmp(fopt,"LOG") == 0 ) || ( strcmp(fopt,"EXP") == 0 ))
{
	   c_getnum (&block, "BASE", &one, &dtype, base);
	   if ( strcmp(fopt,"EXP") == 0 )
	      c_getnum (&block, "EXPSF", &one, &dtype, exsc);
}

/*
Get LAS global parameters
-------------------------         		*/
var = p_find( &block, "$MSGTIM");
msgtim = IVAL( *var, 0);

var = p_find( &block, "$MINMAX");
strcpy( mnmx, SVAL( *var, 0));

if (mnmx[0] == 'Y') 
	min_max = TRUE;
else 
	min_max = FALSE;

}
