/* include files, global parameters, and definitions for UNARY      */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "las.h"
#include "pixman.h"
#include "util.h"
#include "protos.h"

struct	PARBLK	block;          /* TAE parameter block     */
struct  DDR	ddr;            /* ddr                     */
FILE	*fd;			/* temp file descriptor    */

float   *iobuf;                	/* General image i/o buffer        */

unsigned char 	*biobuf;	/* byte  image i/o buffer for not */
short 	*siobuf;                /* short image i/o buffer for not */
long	*liobuf;                /* long image i/o buffer for not  */
float   *riobuf;                /* float image i/o buffer for not */


char    out_files[2][CMLEN];             /* output file names       */

struct FDESC *	fdesc[MAXIMG+2];             /*  file descriptors  */
struct GDESC *	gdesc;                       /*  group descriptor  */
                           
long	*ipoint;   /* offsets into image i/o  */
long	opoint[MAXBND];                /* for input and output bands */

long	min_max;                       /* minmax option flag      */
long 	msgtim;                        /* message interval        */
long	num_outfiles;                  /* number of output files  */
long    intype=0;                    /* Data type for input image */
long	out_type;	           /* output image data type */

extern	long	curent;                        /* current entity           */
extern	long	curcat;                        /* current category         */

int main
(void)
{
/*
********************************************************************

NAME: UNARY 
              
PURPOSE: 

PROGRAM HISTORY:
  Version   Date       Author                       Request
  -------   ----       ------                       -------
                          
  1.0     05-Mar-84    S. Ross/SAR            scn#20-188
  2.0     22-Mar-84    S. Ross/SAR            scn#20-204 
  3.0     18-May-84    S. Ross/SAR            scn#20-295
  4.0     26-Mar-86    K. Gacke                 lasd
  5.0     01-Feb-90    W. Xia                 conversion to 5.0
  5.1     19-Sep-92    D. Etrheim/EDC         Modifed global include file. 
					Changed parameters PSCALE,PHASE,EXSCALE
					to SCALFACT,OFFSET,EXPSF for standards
  5.2     16-Sep-93    D. Bridges/EDC         Modified subcmd SCA to calculate
					scale and offset if not specified
                 
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         runs under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION:
 
call getpar to receive the parameters from the user
call filspec to open input, modulating and output images
call runopt to perform the user-specified operation on one input image
call c_egclse to close all the images
call c_hstout to generate the history file
call c_complt to print a completion message

*************************************************************************
*/

char    
	fopt[4],                /* function option */
        inhos[MAXIMG][CMLEN],   /*    input image host name      */  
        intae[MAXIMG][CMLEN],   /*    input image tae name       */
        outhos[1][CMLEN],        /*    output image host name     */
        outtae[1][CMLEN],        	/*    output image tae name      */  
        outbuf[ERRLEN];  	/* message buffer */         

double
	*amax,		        /*    maximum pixel value        */
        *amin,                  /*    minimum pixel value        */  
	mnout,			/*    MIN/MAX check varible	 */
	mxout;			/*    MIN/MAX check varible	 */

long    ibnd;				/* band index */
static long image_band_num[MAXIMG][MAXBND+1];  /* band numbers selected for each
					      image specified */
long	num_inbands,             /* total number of input image bands*/
	num_inimages,            /* number of input images           */
	num_outbands;            /* number of bands in output image  */
static long out_img_bands[MAXIMG][MAXBND+1],  /* bands numbers for the output 
                                             image */
	wind[MAXIMG][4]; 		/* input image window		*/
long	*num_image_bands; /* number of bands in each input image  */

float
	scoff[2],                  /* scale and offset 		 */
        minpix,                    /* minimum pixal value in image */
        maxpix,                    /* maximum pixal value in image */
	base,                      /* log/exp base */
	div,			   /* Quotient variable  */
	exsc,                      /* scale for exp */
	pscale,                    /* sin, cos scale */
        phase;                     /* sin, cos offset */

long	*clipped,           /* data clip flags for exp */
        itype=ELONG,	           /* integer data type */
	rtype = EREAL,		   /* Data type for c_px calls */
	scflg,                     /* scale/no scale flag */
        calcscflg=0, 		   /* calculate/don't calc scale flag */
        i;   			   /* counter */
long    inf_err = 1;                   /* info error return status  */
long    fatal_err = -1;                /* fatal error return status */
long    zero = 0;
long    one = 1;

/* Allocate space */
amin = (double *) malloc(MAXBND * sizeof(double));
if (amin == NULL)
   c_errmsg("Error allocating dynamic memory", "unary-alloc", &fatal_err);
amax = (double *) malloc(MAXBND * sizeof(double));
if (amax == NULL)
   c_errmsg("Error allocating dynamic memory", "unary-alloc", &fatal_err);
clipped = (long *) malloc(MAXBND * sizeof(long));
if (clipped == NULL)
   c_errmsg("Error allocating dynamic memory", "unary-alloc", &fatal_err);
num_image_bands = (long *) malloc(MAXIMG * sizeof(long));
if (num_image_bands == NULL)
   c_errmsg("Error allocating dynamic memory", "unary-alloc", &fatal_err);
ipoint = (long *) malloc(MAXIMG * MAXBND + 1 * sizeof(long));
if (ipoint == NULL)
   c_errmsg("Error allocating dynamic memory", "unary-alloc", &fatal_err);

/*
Get the user specified parameters
---------------------------------			*/                 
getpar(intae, inhos, outtae, outhos, wind, &num_inimages, num_image_bands, 
       image_band_num, &num_inbands, &num_outbands, fopt, &base, &exsc, 
       scoff, &scflg, &calcscflg, &pscale, &phase); 

/*                             
Get min and max for all input files if executing the log or not function.
-------------------------------------------------------------------------  */
c_minmax (inhos, &num_inimages, wind, image_band_num, num_image_bands,
	  amin, amax);       

/*
find the minimum and maxium pixel values in the image 
-----------------------------------------------------*/
minpix=amin[0];
maxpix=amax[0];
for(i=1;i<*num_image_bands;i++)
   {
   if(minpix > amin[i])
      minpix = amin[i];
   if(maxpix < amax[i])
      maxpix = amax[i];
   }
/*
calculate the scale and offset values if calcscflg is set
---------------------------------------------------------*/
if (strcmp(fopt,"SCA") == 0)
   if (calcscflg)
      calc_scale(&minpix, &maxpix, scoff);

/*
Perform tests for potential output failure (except NOT).
--------------------------------------------------------		*/

for (ibnd = 0; ibnd < num_inbands; ibnd++)
{
	if (strcmp(fopt,"NOT") == 0)
		break;

	if (strcmp(fopt,"SIN") == 0)
	{
		mnout = sin(pscale * amin[ibnd] + phase);
		mxout = sin(pscale * amax[ibnd] + phase);
	}
	else if (strcmp(fopt,"COS") == 0)
	{
		mnout = cos(pscale * amin[ibnd]  + phase);
		mxout = sin(pscale * amax[ibnd] + phase);
	}
	else if (strcmp(fopt,"EXP") == 0)
	{
		mnout = pow( base , ( exsc * (amin[ibnd])));
		mxout = pow( base , ( exsc * (amax[ibnd])));
	}
	else if (strcmp(fopt,"SCA") == 0)
	{
		c_pxline (&amin[ibnd], &amin[ibnd], &one,
			  &rtype, &scoff[0], &scoff[1]);      
		c_pxline (&amax[ibnd],  &amax[ibnd], &one,
			  &rtype, &scoff[0], &scoff[1]);      

	}
	else if ( strcmp(fopt,"LOG") == 0 )  
	{ 
           if (amin[ibnd] <= 0.0 )
	   {                   
	      sprintf(outbuf, "Pixel values less than or equal to 0.0 found ");
   	      c_errmsg(outbuf,"unary-pixel",&inf_err);
              sprintf(outbuf,"in band <%3d>",ibnd+1);
              c_errmsg(outbuf,"unary-pixel",&inf_err);
	      c_errmsg ("Log function aborted",
				  "unary-pixel",&fatal_err);
	   }
	      mnout = log10(amin[ibnd]);
              if (base != 10.0)
	      {
		 div = 1./log10(base);    
		 c_pxline (&mnout, &mnout, &one,
			  &rtype, &div, &zero);
	      }
	      mxout = log10(amax[ibnd]);
	      if (base != 10.0)
	      {
	  	 div = 1./log10(base);    
		 c_pxline (&mxout, &mxout, &one,
			  &rtype, &div, &zero);
	      }
	}
	mnout = (scoff[0] * mnout) + scoff[1];
	mxout = (scoff[0] * mxout) + scoff[1];
	if (mxout > 1.7e38 || mnout < -1.7e38)
		c_errmsg(      
		"SCALE and OFFSET setting will result floating-point overflow.",
		"unary-scoff",&inf_err);
}

/*
Initialize the data clip flags for exp
--------------------------------------			*/
c_pxfill(clipped, &itype, &num_inbands, &zero);

/*      
Set up file specifications.
---------------------------				*/
if (num_outbands != 0)                                          
	filspec (inhos, outhos[0], fopt, &num_inimages, num_image_bands, 
	image_band_num, &num_outbands, wind, out_img_bands);

/*
Process input images 
------------------------------------               			*/
runopt (fopt, wind, base, exsc, scoff, scflg, pscale, phase, clipped,
	num_inbands, &num_outbands);

/*
Calculate the minimum and maximum pixel values for the output image
------------------------------------------------------------------- 	*/
if (min_max)
{
	wind[0][SL] = wind[0][SS] = one;
	c_minmax (outhos, &one, wind, out_img_bands, 
		  &num_inbands, amin, amax);
	c_svmnmx (outhos[0], &num_inbands, amin, amax, "YES");
}

/*
Generate the history file and print the completion message.
------------------------------------------------------------	*/
c_hstout (&block, outhos[0], inhos, &num_inimages, " ");
c_complt ("UNARY", out_files, &num_outfiles);

return EXIT_SUCCESS;

}
