#include "unknown.h"
#include "util.h"
#include "protos.h"

void getparm
(
    char ihos[MAXIMG][CMLEN],    /* input host file name */
    char ohos[][CMLEN],           /* output host file name */
    char hos_stat[CMLEN],        /* stats file host name */
    long *nimages,               /* no. of images in input image */
    long *minimages,             /* no. of images in PMIN image */
    long nbands[MAXIMG],         /* no. of bands in each image */
    long bands[][MAXBND + 1],    /* band nos. of each image */
    long *totbnd,                /* total no. of bands in input image */
    char olas[][CMLEN],           /* output LAS file name */
    long *minbnd,                /* total no. of bands in PMIN image */
    long wind[][4],              /* image parameters */
    float *low,                   /* threshold value */
    long *pixval,         /* and INT form */
    short *unkgry                /* key for unknown classification */
)

/*
------------------------------------------------------------------
NAME: GETPARM

PROGRAM HISTORY:
    Version             date            Author		Reason
    -------             ----            ------		------
       3              07-dec-88        B. Allam  	Retrocode & Update
       2              14-aug-85        F. Irani     
       1              30-june-83       J. Hwang
      
ALGORITHM: Get and validate input and output image parameters
           Get input stats file parameters
           Get pixel value and threshold percentage
           Get LAS globals
------------------------------------------------------------------
*/
{
char  ilas[MAXIMG][CMLEN],          /* input LAS file name */
      minmax[4];                    /* minmax option */
long  std_out,                      /* logical unit for standard output */
      access,                       /* mode of access */
      same_window,                  /* flag indicating same windows */
      stat,                         /* return status */
      flag[MAXIMG][2],              /* flag for subwindows */
      imglimit = MAXIMG;            /* maximum no. of images */
struct VARIABLE *vp,                /* pointers to VARIABLE structure */
      *var, *vl;
long    zero = 0;

/*
initialize program
------------------
*/
c_inlas (&block, &std_out); 

/*
get and validate input image parameters 
---------------------------------------
*/
stat = c_getin (&block, "IN", &zero, ilas[0], ihos[0], nimages, wind[0], 
                bands[0], &nbands[0], totbnd, &imglimit, &same_window, flag[0]);
if (stat != E_SUCC)
   iso_err(badinp, "unknown_badinp");
if (*totbnd > 1)  
   {
   iso_err(singbnd, "unknown_singbnd");
   }  
if (flag[0][0])
 /*  no window allowed */
   {
   iso_err (ignorwind, "unknown_ignorwind"); 
   }  
c_getddr (ihos[0], &ddr);
if (ddr.dtype != EBYTE)
   iso_err ("!Input image must be BYTE", "unknown_type");
stat = c_getin (&block, "INMAXCHI", &zero, ilas[1], ihos[1], minimages, 
                wind[1], bands[1], &nbands[1], minbnd, &imglimit, &same_window, 
                flag[1]);
if (stat != E_SUCC)
   iso_err (badmin, "unknown_badmin");
if (*minbnd > 1) 
   {
   iso_err (maxchibnd, "unknown_maxchibnd"); 
   }  
if (flag[1][0])
 /*  no window allowed */
   {
   iso_err (ignorwind, "unknown_ignorwind"); /*** NEWLAS call ***/
   }  
c_getddr (ihos[1], &ddr);
if (ddr.dtype != EBYTE)
   iso_err ("!Input image must be BYTE", "unknown_type");

/*
get sfin parameters
-------------------
*/
access = IREAD;
stat = c_getnam (&block, "INSTAT", &zero, hos_stat, "stat", &access);
if (stat != E_SUCC)
   iso_err (badstat, "unknown_badstat");

/*
get and validate output image parameters
----------------------------------------
*/
stat = c_getout (&block, "OUT", olas, ohos); 
if (stat != E_SUCC)
   iso_err (badoutp, "unknown_badstat");

/*
get pixel value and threshold percentage
----------------------------------------
*/
vp = p_find (&block, "PIXVAL");
*pixval = IVAL(*vp, 0);
*unkgry = (short) (*pixval);
vl = p_find (&block, "THRPERC");
*low = RVAL(*vl, 0);
if (!(*vl).v_count)
  *low = 0.0;
if (*low < 0.0 || *low > 100.0)
  iso_err ("!Invalid value for THRPERC", "unknown_thresh");
/*
get LAS globals
---------------
*/
var = p_find(&block, "$msgtim");
msgtim = IVAL(*var, 0);

var = p_find(&block, "$minmax");
strcpy(minmax, SVAL(*var, 0));
if (minmax[0] == 'Y' || minmax[0] == 'y') min_max = TRUE;
else min_max = FALSE;
}
