/**************************************************************************

NAME:				GETPAR	

PURPOSE:   Get user entered parameters from the TAE parameter block 


****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "verify2.h"
#include "util.h"
#define TERM 0
#define PFILE 1
#define LP 2


void getpar
(
    struct PARBLK *vblk,/* I: TAE parameter block			*/
    char intpl[],	/* O: The input Tie Point Location file name 	*/
    double *psout,	/* O: Pixel size in output space 		*/
    long *linereg,	/* O: Number of line regions in image 		*/
    long *sampreg,	/* O: Number of sample regions in image 	*/
    long *sl,		/* O: starting line				*/
    long *ss,		/* O: starting sample				*/
    long *lines,	/* O: Total number of lines in the image 	*/
    long *samps,	/* O: Total number of samples in the image 	*/
    long *accnum,	/* O: Number of user specified accuracy levels 	*/
    double pixacc[],	/* O: User input pixel accuracy desired 	*/
    long *log_flag,	/* O: the flag for the 1st output file		*/
    char logfile[],	/* O: The image statistics logfile 		*/
    long *plot_flag,	/* O: the plot flag 				*/
    char plotname[],	/* O: The plot file 				*/
    char prtfile[],    	/* O: print file name                      	*/
    char filename[],	/* O: print file name				*/
    long print[]        /* O: print option                         	*/
)


{
struct VARIABLE *var;		/* Pointer to a TAE variable in vblk 	*/

long acc;		/* Access flag 					*/
long non_fatal = 1;	/* non fatal error flag				*/
long fatal = -1;	/* Fatal flag for errmsg 			*/
long zero = 0;		/* Constant of zero 				*/
long j;			/* Counter 					*/


/* Get the tie point location file name
-------------------------------------*/
acc = IREAD;
if (c_getnam(vblk, "INTL", &zero, intpl, "tpl", &acc) != E_SUCC)
c_errmsg("Fatal error encountered","verify2-fatal",&fatal);

/* Get the pixel size
-------------------*/
var = p_find(vblk, "PIXSIZ");
if ((*var).v_count > 0)
   {
   *psout = RVAL(*var,0);
   }

/* Get the number of line regions in the grid
--------------------------------------------*/
var = p_find(vblk, "NLREG");
*linereg = IVAL(*var,0);

/* Get the number of sample regions in the grid
---------------------------------------------*/
var = p_find(vblk, "NSREG");
*sampreg = IVAL(*var,0);

if ((*linereg * *sampreg) > MAX_REGIONS)
   {
   c_errmsg("Too Many regions specified","verify2-regions",&non_fatal);
   c_errmsg("Fatal error encountered","verify2-fatal",&fatal);
   }

/* Get the desired pixel accuracy
-------------------------------*/
var = p_find(vblk, "ACCINC");
pixacc[0] = RVAL(*var,0);
*accnum = (*var).v_count;
for (j = 1; j < *accnum; ++j)
  pixacc[j] = RVAL(*var,j);

/* Get the input window
---------------------*/
var = p_find(vblk, "WINDOW");
if ((*var).v_count == 4)
   {
   *sl = IVAL(*var,0);
   *ss = IVAL(*var,1);
   *lines = IVAL(*var,2);
   *samps = IVAL(*var,3);
   }
else
   {
   *sl = 1;
   *ss = 1;
   }

/* Get the image statistics file name
-----------------------------------*/
acc = IUPDATE;
var = p_find(vblk, "LOGFILE");
if ((*var).v_count > 0)
   {
   if (c_getnam(vblk, "LOGFILE", &zero, logfile, "txt", &acc) != E_SUCC)
       c_errmsg("Fatal error encountered","verify2-fatal",&fatal);
   *log_flag = TRUE;
   }
else
   *log_flag = FALSE;

/* Get print options
------------------*/
strcpy(prtfile,"verify2");
if (c_setprt(vblk,&print[TERM],&print[PFILE],&print[LP],
             prtfile,filename) != E_SUCC)
   c_errmsg("Fatal error encountered","verify-fatal",&fatal);


/* Get the point file name
  ------------------------*/
acc = IWRITE;
var = p_find(vblk, "OUTFILE");
if ((*var).v_count > 0)
   {
   if (c_getnam(vblk, "OUTFILE", &zero, plotname, "txt", &acc) != E_SUCC)
       c_errmsg("Fatal error encountered","verify2-fatal",&fatal);
   *plot_flag = TRUE;
   }
else
   *plot_flag = FALSE;

}
