/*************************************************************************

NAME:				REPORT

PURPOSE: Calculates statistics and generates reports for verify2

*************************************************************************/
#include <math.h>
#include <stdio.h>
#include "las.h"
#include "geompak.h"
#include "verify2.h"

void report
(
    char intpl[],		/* I: The input Tie Point Location file name */
    char logfile[],		/* I: The image statistics logfile           */
    char prtfile[],		/* I: print file name                        */
    char plotname[],		/* I: The plot file                          */
    long npts,			/* I: Total number of verification points    */
    double psout, 		/* I: Pixel size in output space             */
    long linereg, 		/* I: Number of line regions in image 	     */
    long sampreg, 		/* I: Number of sample regions in image      */
    double pixacc[],		/* I: User input pixel accuracy desired      */
    struct POINT pointref[], 	/* I: Reference image point array 	     */
    struct POINT pointsrch[],	/* I: Search image point array 		     */
    char units[],		/* I: units				     */
    long sl,             	/* I: starting line                          */
    long ss,             	/* I: starting sample                        */
    long lines, 		/* I: Total number of lines in the image     */
    long samps, 		/* I: Total number of samples in the image   */
    long accnum,		/* I: Number of user specified accuracy levels*/
    long log_flag,		/* I: log flag				     */
    long plot_flag		/* I: plot flag				     */
)
{
double *dline; 			/* Array containing errors in line direction */
double *dsamp;			/* Array containing errors in samp direction */
double *tdisp;			/* Array containing total error in ver points */
double rmssq[MAX_REGIONS];      /* RMSE of the TDISP 			    */
double rmsem;           	/* RMSE in meters in output space 	    */
double ale[MAX_REGIONS];    /* Mean error of the verification points in line*/
double ase[MAX_REGIONS];    /* Mean error of the verification points in sample*/
double rmsel[MAX_REGIONS];  /* RMSE of line errors in the verification points */
double rmses[MAX_REGIONS];  /* RMSE of sample errors in the points          */
double mle[MAX_REGIONS];    /* Minimum error of the points in line          */
double mse[MAX_REGIONS];    /* Minimum error of the points in samp          */
double mxle[MAX_REGIONS];   /* Maximum error of the points in line          */
double mxse[MAX_REGIONS];   /* Maximum error of the points in samp          */
double lres;                /* Line residual error 			    */
double sres;                /* Sample residual error 		     	    */
double mlres[MAX_REGIONS];  /* Mean line residual error 		    */
double msres[MAX_REGIONS];  /* Mean sample residual error 		    */
double tres[MAX_REGIONS];   /* Total mean residual error 		    */
double clres;           /* Current line residual error 			    */
double csres;           /* Current sample residual error 		    */
double ctres;           /* Current total residual error 		    */
double lstd;            /* Line standard deviation 		   	    */
double sstd;            /* Sample standard deviation 	  	  	    */
double tstd;            /* Total standard deviation 			    */
double smedian;		/* Sample median value 			   	    */
double lmedian;		/* Line median value 				    */
double tmedian;		/* Total error median value 			    */
double tmp1,tmp2;   	/* Temporary variables for computations 	    */
double temp,temp2;      /* Temporary variable for computations 	    	    */
double temp3;           /* Temporary variable for computations 	   	    */
double pix;             /* Pixel accuracy of current point 		    */
double pcent[7];        /* Percentage of points not meeting pixel accuracy  */
long lines1;            /* Number of lines  per region 			    */
long samp1;             /* Number of samples per region 		    */
long pt1, pt2;          /* line and sample coordinates 			    */
long *xregion;    /* Holds beginning and ending of line 
				       region 				    */
long *yregion;    /* Holds beginning and ending of sample 
				       region 				    */
long regionpts[MAX_REGIONS];        /* Holds number of points in each region*/
long regions;           	    /* Total number of regions 		    */
struct PTREG *regionarr[MAX_REGIONS];  /* Array holding point 
                                                         numbers in a region*/
long j, x,y,z;          /* Counters 					    */
long index;             /* Holds number of current point 		    */
long pctr[MAX_REGIONS][7];        /* Number of points that do not meet pixel 
                                     accuracy 				    */
long left, right;	/* Array boundaries 				    */
long ii;		/* loop counter					    */
long remainder;         /* Indicates even/odd number of points 		    */
long mid, mid1, mid2;   /* Middle element(s) of the error arrays 	    */
FILE *print_fp;         /* File to hold region statistics 		    */
FILE *log_fp;           /* File to hold image statistics 		    */
FILE *plot_fp;          /* File to hold point values for embedpts 	    */

/* Initialize variables
---------------------*/
lines1 = (lines - sl + 1)/linereg;
samp1 = (samps - ss + 1)/sampreg;
regions = linereg * sampreg;
dline =(double *)malloc((sizeof(double )*npts));
dsamp =(double *)malloc((sizeof(double )*npts));
tdisp =(double *)malloc((sizeof(double )*npts));
xregion =(long *)malloc((sizeof(long )*2 * npts));
yregion =(long *)malloc((sizeof(long )*2 * npts));

for (ii = 0; ii < MAX_REGIONS; ii++)
   regionarr[ii] =(struct PTREG  *)malloc((sizeof(struct PTREG) * npts));

/* Initialize line regions starting and ending points
---------------------------------------------------*/
z = 0;
for (x = 0; x < linereg; ++x)
  for (y = 0; y < sampreg; ++y)
    {
    xregion[2 * z + 1] = lines1 * (x+1) + sl;
    xregion[2*z] = xregion[2 *z + 1] - (lines1 - 1);
    ++z;
    }

/* Initialize sample regions starting and ending points
-----------------------------------------------------*/
for (z=0; z<sampreg; ++z)
  {
  yregion[2 * z + 1] = samp1 * (z+1) + ss;
  yregion[2 * z] = yregion[2 * z + 1] - (samp1 - 1);
  }

for (z=0; z<linereg; ++z)
  for (x=0; x<sampreg; ++x)
    {
    yregion[2 * (x+(sampreg*z))] = yregion[2 * x];
    yregion[2 * (x+(sampreg*z)) + 1] = yregion[2 * x + 1];
    }

/* Initialize number of points per region to zero
-----------------------------------------------*/
for (x=0; x<regions; ++x)
  regionpts[x] = 0;

/* Assign point to a region
-------------------------*/
for(x=0; x<regions; ++x)
   for(y=0,z=0; y<npts; ++y)
      {
      pt1 = (long)pointref[y].x;
      pt2 = (long)pointref[y].y;
      if ((pt1 >= xregion[2 * x]) && (pt1 < xregion[2 * x + 1]) &&
          (pt2 >= yregion[2 * x]) && (pt2 < yregion[2 * x + 1]))
        {
        regionpts[x] = regionpts[x] + 1;
        strncpy(regionarr[x][z].ptnum, pointref[y].ptnum, 21);
        regionarr[x][z].num = y;
        ++z;
        }
      }

/* Open output files
  -------------------*/
print_fp = fopen(prtfile, "w");
if (regions==1 && (plot_flag == TRUE)) 
   plot_fp = fopen(plotname, "w");

if (log_flag)
   log_fp = fopen(logfile, "a");


fprintf(print_fp, "Verify filename: %s\n",intpl);
fprintf(print_fp,"Output image pixel size in %s:     %9.2f\n\n",units, psout);


/* Initialize variables
  ----------------------*/
for (x=0; x<regions; ++x)
  {
  ale[x] = 0.0; 
  ase[x] = 0.0;
  rmsel[x] = 0.0; 
  rmses[x] = 0.0; 
  rmssq[x] = 0.0;
  lres = 0.0; 
  sres = 0.0; 
  mlres[x] = 0.0; 
  msres[x] = 0.0; 
  tres[x] = 0.0;
  pix = 0.0;
  for (y=0; y<accnum; ++y) 
      pctr[x][y] = 0;

  fprintf(print_fp, "REGION: %d          Number of points: %d\n",x,regionpts[x]);
  if (regions==1 && (plot_flag == TRUE)) 
     fprintf(plot_fp, "%d\n", regionpts[x]);

  /* Output table header to printer file
  --------------------------------------*/
  if (regionpts[x] != 0)
    {
    fprintf(print_fp, "\n");
    fprintf(print_fp, "         Reference Image   Registered Image           Errors (%s)\n", units);
    fprintf(print_fp, "Pt Id     Line   Sample     Line   Sample       Line      Sample   Total\n");
    fprintf(print_fp, "------   ---------------   ---------------     ------------------------------\n");
    /* Calculate statistics
    ----------------------*/
    for (y=0; y<regionpts[x]; ++y)
      {
      z = regionarr[x][y].num;
      dline[y] = (pointref[z].x - pointsrch[z].x) * psout;
      dsamp[y] = (pointref[z].y - pointsrch[z].y) * psout;
      tmp1 = dline[y] * dline[y];
      tmp2 = dsamp[y] * dsamp[y];
      tdisp[y] = sqrt(tmp1 + tmp2);
      pix = tdisp[y]/psout;
      for (j = 0; j < accnum; ++j)
        if (pix > pixacc[j]) ++(pctr[x][j]);
          fprintf(print_fp, "%s %8.1f %8.1f %8.1f %8.1f %10.2f %10.2f %10.2f\n",
            regionarr[x][y].ptnum, pointref[z].x, pointref[z].y, pointsrch[z].x,
            pointsrch[z].y, dline[y], dsamp[y], tdisp[y]);
      if (regions==1 && (plot_flag == TRUE)) 
         fprintf(plot_fp, "%f %f %f\n", pointsrch[z].x, pointsrch[z].y, 
                           tdisp[y]);
      ale[x] += dline[y];
      ase[x] += dsamp[y];
      rmssq[x] += tmp1 + tmp2;
      rmsel[x] += tmp1;
      rmses[x] += tmp2;

      /* Initialize min and max if first time through loop
      ---------------------------------------------------*/
      if (y == 0)
        {
        mle[x] = dline[y];
        mse[x] = dsamp[y];
        mxle[x] = dline[y];
        mxse[x] = dsamp[y];
        }
      /* Determine min and max
      ----------------------*/
      else
        {
        if(mle[x] > dline[y]) mle[x] = dline[y];
        if (mxle[x] < dline[y]) mxle[x] = dline[y];
        if (mse[x] > dsamp[y]) mse[x] = dsamp[y];
        if (mxse[x] < dsamp[y]) mxse[x] = dsamp[y];
        }
      /* Calculate residuals
      ----------------------*/
      lres = pointref[z].x - pointsrch[z].x;
      sres = pointref[z].y - pointsrch[z].y;
      mlres[x] += lres;
      msres[x] += sres;
      tres[x] += (lres + sres);
      }

    /* Sort dsamp, dline, & tdisp arrays in ascending order 
    ------------------------------------------------------*/ 
    if (regionpts[x] > 1)
      {
      left = 0;
      right = regionpts[x] - 1;
      quicksort(dline, left, right);
      quicksort(dsamp, left, right);
      quicksort(tdisp, left, right);

      /* Calculate median values 
      ------------------------*/
      remainder = regionpts[x] % 2;
      if (remainder != 0)
        {
        mid = (regionpts[x]/2);
        smedian = dsamp[mid];
        lmedian = dline[mid];
        tmedian = tdisp[mid];
        }
      else
        {
        mid1 = (regionpts[x]/2) - 1; 
        mid2 = (regionpts[x]/2);
        smedian = (dsamp[mid1] + dsamp[mid2])/2;
        lmedian = (dline[mid1] + dline[mid2])/2;
        tmedian = (tdisp[mid1] + tdisp[mid2])/2;
        }
      }
    else 
    if (regionpts[x]==1)
      {
      smedian = dsamp[0];
      lmedian = dline[0];
      tmedian = tdisp[0];
      }

    /* Calculate rms values
    ----------------------*/
    ale[x] = ale[x] / regionpts[x];
    ase[x] = ase[x] / regionpts[x];
    rmssq[x] = sqrt(rmssq[x] / regionpts[x]);
    rmses[x] = sqrt(rmses[x] / regionpts[x]);
    rmsel[x] = sqrt(rmsel[x] / regionpts[x]);
    rmsem = rmssq[x] / psout;

    /* Calculate st. dev. and percentage of points not meeting pixel accuracy
    -----------------------------------------------------------------------*/
    if (regionpts[x] != 1)
      {
      mlres[x] = mlres[x] / regionpts[x];
      msres[x] = msres[x] / regionpts[x];
      tres[x] = tres[x] / regionpts[x];
      temp = 0.0; temp2 = 0.0; temp3 = 0.0;
      for (z=0; z<regionpts[x]; ++z)
        {
        index = regionarr[x][z].num;
        clres = pointref[index].x - pointsrch[index].x;
        csres = pointref[index].y - pointsrch[index].y;
        ctres = clres + csres;
        temp += ((clres - mlres[x])*(clres - mlres[x]));
        temp2 += ((csres - msres[x])*(csres - msres[x]));
        temp3 += ((ctres - tres[x])*(ctres - tres[x]));
        }
      lstd = sqrt(temp/(regionpts[x] - 1));
      sstd = sqrt(temp2/(regionpts[x] - 1));
      tstd = sqrt(temp3/(regionpts[x] - 1));
      for (j = 0; j < accnum; ++j)
        pcent[j] = ((double)pctr[x][j]/(double)regionpts[x])*100;
      }
    else
      {
      lstd = 0.0;
      sstd = 0.0;
      tstd = 0.0;
      for (j = 0; j < accnum; ++j)
        { 
        if ((pctr[x][j]==1)&&(regionpts[x]==1)) 
           pcent[j] = 100.0;
        else 
           pcent[j] = 0.0;
        }
      }

    /* Change units back (feet, meter, etc) */
    lstd *= psout;
    sstd *= psout;
    tstd *= psout;

    /* Output region statistics to screen and file with headings
    ----------------------------------------------------------*/
    fprintf (print_fp, "\n**LINE ERROR**                                 **SAMPLE ERROR**\n");
    fprintf(print_fp, "average= %f %s			average= %f %s\n",
           ale[x], units, ase[x], units);
    fprintf(print_fp, "range = %3.2f to %3.2f %s			range = %3.2f to %3.2f %s\n", 
	    mle[x], mxle[x], units, mse[x], mxse[x], units);
    fprintf(print_fp, "RMSE=%f %s				RMSE= %f %s\n",
           rmsel[x], units, rmses[x], units);
    fprintf(print_fp, "Median = %f %s			Median = %f %s\n",
          lmedian, units, smedian, units);
    fprintf(print_fp, "st.dev.= %f %s			st.dev.= %f %s\n",
           lstd, units, sstd, units);
    fprintf(print_fp, "\n**TOTALS**\n");
    fprintf(print_fp, "Total st. dev. = %f %s\n", tstd, units);
    fprintf(print_fp, "RMSE of total error= %f %s\n",rmssq[x], units);
    fprintf(print_fp, "RMSE of total error in pixels= %f\n", rmsem);
    fprintf(print_fp, "\n");
    fprintf(print_fp, "Median of total error= %f %s\n", tmedian, units);
    fprintf(print_fp, "Median of total error in pixels= %f\n", tmedian/psout);
    fprintf(print_fp, "\n");
    for (j = 0; j < accnum; ++j)
      {
      fprintf(print_fp, "Number of points not meeting %1.3f pixel accuracy: %d\n",
            pixacc[j], pctr[x][j]);
      fprintf(print_fp, "Percentage of points not meeting %1.3f pixel accuracy: %3.2f",
            pixacc[j], pcent[j]);
      fprintf(print_fp, "\n\n");
      }
    }
  else
    {
    fprintf(print_fp, "            ***** NO POINTS IN THIS REGION *****\n\n");
    }

  /* Output image statistics to a file w/o headings
  ------------------------------------------------*/
  if (log_flag)
     {
     fprintf(log_fp, intpl);
     fprintf(log_fp, " %d ", regionpts[0]);
     fprintf(log_fp,"%2.2f %3.1f %3.1f ", ale[0], mle[0], mxle[0]);
     fprintf(log_fp,"%2.2f %2.2f %1.3f ", rmsel[0], lmedian, lstd);
     fprintf(log_fp,"%2.2f %3.1f %3.1f ", ase[0], mse[0], mxse[0]);
     fprintf(log_fp,"%2.2f %2.2f %1.3f ", rmses[0], smedian, sstd);
     fprintf(log_fp, "%2.2f ", tstd);
     fprintf(log_fp, "%3.2f ", rmssq[0]);
     fprintf(log_fp, "%1.3f ", rmsem);
     fprintf(log_fp, "%2.2f %2.2f ", tmedian, tmedian/psout); 
     for (j = 0; j < accnum; ++j)
        { 
        fprintf(log_fp, "%d ", pctr[0][j]);
        fprintf(log_fp, "%1.2f ", pcent[j]);
        }
     fprintf(log_fp, "\n"); 
     }
  }
if (log_flag)
    fclose(log_fp);
fclose(print_fp);
if (plot_flag == TRUE)
   fclose(plot_fp);

for (ii = 0; ii < MAX_REGIONS; ii++)
   free(regionarr[ii]);
free(dline);
free(dsamp);
free(tdisp);
free(xregion);
free(yregion);

}

