/*****************************************************************************

NAME: array_list_utils.c

PURPOSE: Utilities for creating and destroying linked list members associated
         with WAVEMERGE.

PROGRAM HISTORY:
  Version       Date       Author             Comments
  -------       ----       ------             -------
    1           05/00    L. Johnson           Original development.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT: WAVEMERGE

ALGORITHM DESCRIPTION:
     N/A.

REFERENCES:
     None.
*****************************************************************************/

#include<stdlib.h>
#include "las.h"
#include "wavemerge.h"


/*****************************************************************************

NAME: create_array_buf

PURPOSE: Allocate (and initialize) storage for a new linked list member.

ALGORITHM DESCRIPTION:
     See inline comments.

REFERENCES:
     None.
*****************************************************************************/
struct array_buf *create_array_buf
(
    struct array_buf *buf         /* I: existing buffer with which to link.
                                        If NULL, the new buffer will be the
                                        origin of the list.                 */
)
{
  struct array_buf *new_buf;     /* the new linked list member */

  
  new_buf = (struct array_buf *)malloc(sizeof(struct array_buf));
  if (new_buf == NULL)
  {
      c_lasmsg(FALSE, "wavemerge-alloc", "Error allocating memory.");
      return NULL;
  }

  /* Link to existing list.  If buf is NULL, this will mark
     new_buf as the top of the list.
     ------------------------------------------------------*/
  new_buf->prev = buf;
  
  /* Terminate linked list and initialize buffers.
   -----------------------------------------------*/
  new_buf->next = NULL;
  new_buf->buf = NULL;
  new_buf->shift_buf[NONE] = NULL;
  new_buf->shift_buf[RIGHT] = NULL;
  new_buf->shift_buf[DOWN] = NULL;
  new_buf->shift_buf[DIAGONAL] = NULL;
  
  return new_buf;
}


/*****************************************************************************

NAME: free_all_linked_buffers

PURPOSE: Free all linked list buffers.

ALGORITHM DESCRIPTION:
     See inline comments.

REFERENCES:
     None.
*****************************************************************************/
void free_all_linked_buffers
(
  struct array_buf *buf     /* I: Pointer to last member of linked list.    */
)
{
  free_shift_buf(buf->shift_buf);
  free(buf->buf);
  buf->buf = NULL;

  /* Step backwards through list, freeing all storage.
     -------------------------------------------------*/
  while (buf->prev != NULL)
  {
      buf = buf->prev;
      free_shift_buf(buf->shift_buf);
      free(buf->buf);
      buf->buf = NULL;
      free(buf->next);
      buf->next = NULL;
  }
  
  free(buf);
}
