#ifndef _WAVEMERGE_H_
#define _WAVEMERGE_H_

#include "las.h"
#include "wavelet.h"


/* Define the wavelet (and its size) to be used. */

#define WAVELET daub4  /* four node discrete Daubechies wavelet */
#define LOF 4          /* length of filter (wavelet)            */


/* Structure definitions */

/* primary structure for doubly linked list */
struct array_buf
{
    shift_buf shift_buf;          /* ptr to four (transform) shift arrays   */
    float *buf;                   /* ptr to an (inverse transform) array    */
    struct array_buf *prev;       /* ptr to previous element in list        */
    struct array_buf *next;       /* ptr to next element in list            */
};


/* Function prototypes */

struct array_buf *create_array_buf
(
    struct array_buf *buf         /* I: existing buffer with which to link.
                                        If NULL, the new buffer will be the
                                        origin of the list.                 */
);

void free_all_linked_buffers
(
  struct array_buf *buf     /* I: Pointer to last member of linked list.    */
);

float get_hp_factor(void);

int getpar
(
    struct PARBLK *vblock,        /* I: Pointer to TAE parameter block      */
    char (*lowtae)[],             /* O: TAE file name of low res image      */
    char (*lowhos)[],             /* O: Host file name of low res image     */
    char (*hightae)[],            /* O: TAE file name of high res image     */
    char (*highhos)[],            /* O: Host file name of high res image    */
    char (*outtae)[],             /* O: TAE file name of output image       */
    char (*outhos)[],             /* O: Host file name of output image      */
    long highwind[][4],           /* O: Window specification for high res   */
    long lowwind[][4],            /* O: Window specification for low res    */
    long *lowband,                /* O: Band number for low res intensity   */
    long *highband,               /* O: Band number for high res            */
    int *nlevels,                 /* O: Number of transform levels to apply */
    char *minmax,                 /* O: Min max flag buffer                 */
    int *msgtim                   /* O: Time between display messages       */
);

struct array_buf *inverse_transform
                              /* Returns a pointer to the generated inverse
                                 transform array (level 0).  The calling
                                 routine is responsible for freeing the
                                 memory.                                    */
(
    struct array_buf *l_last, /* I: Ptr to "last" low res buf in linked list*/
    int nl,                   /* I: Number of lines in merged image block   */
    int ns,                   /* I: Number of samples in merged image block */
    int nlevels               /* I: Number of transform levels to apply     */
);

void max_merge_data
(
    shift_buf h_t,           /* I: High resolution wavelet transform array */
    shift_buf l_t,           /* I/O: On input: low res wavelet transform
                                     On output: merged wavelet transform   */
    int nrows,               /* I: Number of rows in array                 */
    int ncols                /* I: Number of columns in array              */
);

int merge_image_block
(
    float *lr_data_buf, /* I/O: On input: low res image block
                                On output: merged image block               */
    float *hr_data_buf, /* I: High resolution image block                   */
    int nl,             /* I: Number of lines in image block                */
    int ns,             /* I: Number of samples in image block              */
    int nlevels         /* I: Number of transform levels to apply           */
);

struct array_buf *transform_merge
                        /* Returns a pointer to the generated transform
                           array linked list.  The calling routine is
                           responsible for freeing the memory.              */
(
    float *lr_data_buf, /* I: Low resolution image block                    */
    float *hr_data_buf, /* I: High resolution image block                   */
    int *nl,            /* I/O: On input: num of lines in low res image blk
                                On output: num of lines in merged image blk */
    int *ns,            /* I/O: On input: num of samps in low res image blk
                                On output: num of samps in merged image blk */
    int nlevels         /* I: Number of transform levels to apply           */
);

#endif /* #ifndef _WAVEMERGE_H_ */
