/******************************************************************************
NAME:   		      PROCES 

PURPOSE    To proces the extracted windows and create a new image. 

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------      ----		---------	------
  5.0	   R. McKinney     6/88		  EDC		Add UPDDR routine
							Take descriptors out 
							the calling sequences 
							Replace newlas.h with 
							las.h
							Delete tae.h
							Place 'c_' in front of 
							las subroutines
							Place subroutines in a 
							separate file

  5.1	   S. Nelson	  3/92				Broke down proces.c
							into four functions.
							Added masked value
							option.

  6.1	   D. Eichacker	  5/94				Added opt[3].

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PROCES must be run under TAE.

PROJECT			LAS

ALGORITHM REFERENCES		none
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "windop.h"

/*
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc[MAXIMG+1];	/* image file descriptors used for image IN  */
extern int  fdesc1[2];		/* image file descriptors used for image OUT */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */
extern unsigned char *thebuf;	/* image input buffer			     */
extern unsigned char *thebuf1;	/* image output buffer			     */

void proces
(
  struct PARBLK *vblock,          /* I: TAE parameter block */
  char hosin[][CMLEN],            /* I: Host names of input images */
  char *hosout,                   /* I: Host name of output image */
  long nmrimg,                    /* I: Number of input images */
  long (*bands)[MAXBND+1],        /* I: Band specification */
  long *nbands,                   /* I: Number of bands per input image */
  long (*window)[4],              /* I: Window specs of input images */
  long msgtim,                    /* I: Time between display messages */
  long mnmx,                      /* I: Min/Max values */
  long totbnd,                    /* I: Total number of bands to process */
  long *kernel,                   /* I: Kernel dimensions (nl,ns) */
  long *skippix,                  /* I: Number of pixels to skip */
  char *typeopt,                  /* I: Type of averaging */
  long odtype,                    /* I: Output data type */
  long masknum,                   /* I: Number of mask values */
  float maskval[][2],             /* I: Mask values */
  float pixval                    /* I: Pixel value to replace mask values */
)
{

double  minval[MAXBND];  /* min pixel value array                     */
double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;
double  samp_inc;

long	access;			/* image I/O access type                     */
long	band;			/* loop index counting the bands             */
long	bandout;		/* the number of bands to create             */
long    fatal = -1;		/* fatal error flag			     */
long	image;			/* loop index counting input images          */
long    maxsamp;		/* number of samples in output image	     */
long    maxline;		/* number of lines in output image           */
long	msg_flag=ONECAT;	/* flag for processing message               */
long 	nl;			/* number of image lines                     */
long	nonfatal = 1;		/* value of one                              */
long 	ns;			/* number of image samples                   */
long 	one = 1;		/* one  		                     */
long	size;			/* number of bytes per pixel 	 	     */
long	sl;			/* starting image line                       */
long	ss;			/* starting image sample                     */
long	status;			/* function return status code               */
long    windl;			/* maximum line to retrieve		     */
long    winds;			/* maximum sample to retrieve		     */
long    upddr_flag;	        /* flag for update DDR			     */

static	long	opt[3]={0,0,0}; /* option array for eopenr                   */

double  fabs();			/* absolute value function		     */
double  sqrt();			/* square root function			     */

char	*c_ealloc();		/* function to allocate image I/O buffer     */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][2];
ns = window[0][3];

for (band=0,image=0;image<nmrimg;image++)
   {
   sl = window[image][0];
   ss = window[image][1];
   status = c_eopenr(&fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
                   &access,&odtype,opt,&one);
   if (status != E_SUCC)
      c_errmsg("Fatal error encountered","windop-fatal",&fatal);
   band += nbands[image];

   }
fdesc[nmrimg] = 0;

/* Open output image
--------------------*/
access = IWRITE;	
maxline = ((nl-kernel[0])/skippix[0] + 1);
maxsamp = ((ns-kernel[1])/skippix[1] + 1);
if (maxline > nl)
   maxline -= kernel[0];
if (maxsamp > ns)
   maxsamp -= kernel[1];
bandout = totbnd;
if (strcmp(typeopt,"MEAN") == 0)
   bandout *= 2;
status = c_eopenr(&fdesc1[0],hosout,&bandout,&one,&one,&maxline,&maxsamp,
         &access,&odtype,opt,&one);
if (status != E_SUCC)
      c_clean(vblock,"windop",hosout,&one);
fdesc1[1] = 0;

/* Determine size of output image
--------------------------------*/
switch (odtype)
  {
  case EBYTE: size = 1; break;
  case EWORD: size = 2; break;
  case ELONG: size = 4; break;
  case EREAL: size = 4; break;
  }

/* Allocate I/O buffers
----------------------*/
if ((thebuf = (unsigned char *) malloc(ns*size)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","windop-alloc",&nonfatal);
   if (c_eclose(&fdesc[0]) != E_SUCC)
       c_errmsg("Nonfatal error encountered","windop-warn",&one);
   c_clean(vblock,"windop",hosout,&one);
   }

if ((thebuf1 = (unsigned char *) malloc(maxsamp*size)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","windop-alloc",&nonfatal);
   if (c_eclose(&fdesc1[0]) != E_SUCC)
       c_errmsg("Nonfatal error encountered","windop-warn",&one);
   c_clean(vblock,"windop",hosout,&one);
   }


/* Setup for processing message utility
---------------------------------------*/
curent = 1;    
curcat = 1;
c_inittm("WINDOP","Processed","line","band"," ",&nl,
		&totbnd,&msgtim,&msg_flag);
c_startm();

/* Initialize min/max arrays
----------------------------*/
if (mnmx)
   c_inmnmx(&odtype,&bandout,minval,maxval);

/* Calculate the maximum number of lines and samples to use
---------------------------------------------------------*/
windl = (nl / skippix[0]) * skippix[0] + kernel[0];
winds = (ns / skippix[1]) * skippix[1] + kernel[1];
if (windl > nl)
    windl -= kernel[0];
if (winds > ns)
    winds -= kernel[1];

/* Call processing function
--------------------------*/
if (strcmp(typeopt,"MEAN") == 0)
   promean(nmrimg,nbands,hosout,vblock,bands,mnmx,kernel,skippix,odtype,
           maxsamp,windl,size,minval,maxval,masknum,maskval,pixval,ns);

else if (strcmp(typeopt,"MAX") == 0)
   procmax(nmrimg,nbands,hosout,vblock,bands,mnmx,kernel,skippix,odtype,
           maxsamp,windl,size,minval,maxval,masknum,maskval,pixval,ns);

else if (strcmp(typeopt,"MIN") == 0)
   procmin(nmrimg,nbands,hosout,vblock,bands,mnmx,kernel,skippix,odtype,
           maxsamp,windl,size,minval,maxval,masknum,maskval,pixval,ns);

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Close all image files in the groups
-------------------------------------*/
if (c_eclose(&fdesc[0]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","windop-warn",&one);
if (c_eclose(&fdesc1[0]) != E_SUCC)
   c_errmsg("Nonfatal error encountered","windop-warn",&one);

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin[0],hosout,&nmrimg,window,bands,nbands,&line_inc,
		 &samp_inc,&bandout,&upddr_flag);
if (status != E_SUCC)
      c_clean(vblock,"windop",hosout,&one);

/* Create associated min/max file
---------------------------------*/
if (mnmx)
   if (c_svmnmx(hosout,&bandout,minval,maxval,"YES") != E_SUCC)
      c_errmsg("Nonfatal error encountered","windop-warn",&one);

/* Free buffers
--------------*/
free(thebuf);
free(thebuf1);
}
