/*******************************************************************************
NAME:				PROCMIN

PURPOSE	   To proces the extracted windows and create a new image.

PROGRAM HISTORY:
VERSION	   PROGRAMMER	   DATE		CODE/CONT	REASON
-------	   ----------	   ----		---------	------
 5.0	   R. McKinney	   6/88		  EDC	 	Add UPDDR routine
                                                        Take descriptors out
                                                        the calling sequences
                                                        Replace newlas.h with
                                                        las.h
                                                        Delete tae.h
                                                        Place 'c_' in front of
                                                        las subroutines
                                                        Place subroutines in a
                                                        separate file

 5.1	   S. Nelson	   3/92		  		Broke down proces.c
							into four functions.
							Added masked value
							option.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   PROCMAX must be run unter TAE

PROJECT			LAS

ALGORITHM REFERENCES	none
******************************************************************************/
#include <stdio.h>
#include <math.h>
#include "las.h"
#include "util.h"
#include "typlim.h"
#include "windop.h"

/* 
   The following variables must be declared global for image I/O routines
*/
extern int  fdesc[MAXIMG+1];	/* image file descriptors used for image IN  */
extern int  fdesc1[2];		/* image file descriptors used for image OUT */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */
extern unsigned char *thebuf;	/* image input buffer			     */
extern unsigned char *thebuf1;	/* image output buffer			     */

int procmin
(
  long nmrimg,                   /* I: Number of input images */
  long *nbands,                  /* I: Number of bands per input image */
  char *hosout,                  /* I: Host name of output image */
  struct PARBLK *vblock,         /* I: TAE parameter block */
  long (*bands)[MAXBND+1],       /* I: Bands to be processed for all images */
  long mnmx,                     /* I: Min/Max values */
  long *kernel,                  /* I: Kernel dimensions (nl,ns) */
  long *skippix,                 /* I: Number of pixels to skip */
  long odtype,                   /* I: Output data type */
  long maxsamp,                  /* I: Number of samples in output image */
  long windl,                    /* I: Maximum line to retrieve */
  long size,                     /* I: Number of bytes per pixel */
  double minval[MAXBND],         /* O: Min pixel value array */
  double maxval[MAXBND],         /* O: Max pixel value array */
  long masknum,                  /* I: Number of mask values */
  float maskval[][2],            /* I: Mask values */
  float pixval,                  /* I: Pixel value to replace mask values */
  long ns                        /* I: Number of samples in input */
)
{

long    band;                   /* loop index counting the bands             */
long    bando1;                 /* loop index counting the bands             */
long    bando2;                 /* loop index counting the bands             */
long    bnd;                    /* loop index counting the bands             */
long    bottom;                 /* bottom of the sample block                */
long    curlin;                 /* current line being processed              */
long    image;                  /* loop index counting input images          */
long    j, i;			/* counter for number of rows		     */
long    left;                   /* left side of the sample block             */
long	mask;			/* loop index counting mask values           */
long	nonfatal = 1;		/* value of one				     */
long    one = 1;                /* one                                       */
long    right;                  /* right side of the sample block            */
long    sample;                 /* sample indicator                          */
long    status;                 /* function return status code               */
long    top;                    /* top of the sample block                   */
static long *mask_flag;	/* flag for mask value test		     */

float   temp;                   /* temporary storage area                    */
static float	*values;	/* array to house the values                 */

bando1 = bando2 = 0;

mask_flag = (long *)malloc(sizeof(long) * ns);
if (mask_flag == NULL)
  {
    c_lasmsg(FALSE,"windop-alloc","Error allocating dynamic memory");
  return(E_FAIL);
  }

values = (float *)malloc(sizeof(float) * ns);
if (values == NULL)
  {
  c_lasmsg(FALSE,"windop-alloc","Error allocating dynamic memory");
  return(E_FAIL);
  }

/* Main processing loop for min value
-----------------------------------*/
switch (odtype)
{
case EBYTE:
for (image = 0; image < nmrimg; image++)
{
for (band = 1; band <= nbands[image]; band++)
   {
   bnd = bands[image][band-1];
   curlin = 1;
   bando1++;
   bando2++;
   for (j = 0; j < maxsamp; j++)
      {
      values[j] = MAXUTINY;
      mask_flag[j] = FALSE;
      }
   top = 0;
   bottom = top + kernel[0];
   while (bottom <= windl)		/* loop for each output line */
      {
					/* loop for each line of block */
      for (curent = (top+1);curent <= bottom; curent++)
         {
         status = c_eread(&fdesc[image], &bnd, &curent, thebuf, &one);
         if (status == E_FAIL)
            {
            if (c_eclose(&fdesc1[0]) != E_SUCC)
               c_errmsg("Nonfatal error encountered","windop-warn",&one);
            c_clean(vblock,"windop",hosout,&one);
            }

	 /* Process if no mask values exist
	    ------------------------------*/
	 if (masknum == 0)
	    {
	    for (j = 0; j < maxsamp; j++)
	       {
	       left = (j * skippix[1]);
	       right = (left + kernel[1]);
	       for (sample = left; sample < right; sample++)
	          {
                  temp = *(thebuf + (sample));
		  if (temp < values[j])
		     values[j] = temp;
	          } 		       /* end of sample loop   */
	       }		    /* end of maxsamp loop  */
	    }

	 /* Process if mask values exist
	   -----------------------------*/
	 else
	    {
	    for (j = 0; j < maxsamp; j++)
	       {
	       left = j * skippix[1];
	       right = left + kernel[1];
	       for (sample = left; sample < right; sample++)
	          {
                  temp = *(thebuf + (sample));
		  for (mask = 0; (mask < masknum) && (!mask_flag[j]); mask++)
		     {
		     if ((temp >= maskval[mask][0]) && temp<=maskval[mask][1])
			{
			mask_flag[j] = TRUE;
			values[j] = pixval;
			}
		     }
		  if (!mask_flag[j])
		     {
		     if (temp < values[j])
		        values[j] = temp;
		     }
	          } 			/* end of sample loop  		 */
	       }		    /* end of maxsamp loop 	     */
	    }			/* end of mask value condition  */
	 }		     /* end of curent loop  	   */
/*
 Assign minimum value to output
 -------------------------------*/
      for (i = 0; i < maxsamp; i++)
	 {
       	 *(thebuf1 + (i * size)) = values[i];
	 mask_flag[i] = 0.0;
         values[i] = MAXUTINY;
	 }
      if (mnmx)
	 {
         status = c_pxmnmx(thebuf1, &maxsamp, &minval[bando1-1],
						&maxval[bando1-1], &odtype); 
	 if (status != E_SUCC)
	    c_errmsg("Nonfatal error encountered","windop-warn",&nonfatal);
	 }
      status = c_ewrite(fdesc1, &bando1, &curlin, thebuf1, &one);
      if (status == E_FAIL)
         {
         if (c_eclose(&fdesc1[0]) != E_SUCC)
	    c_errmsg("Nonfatal error encountered","windop-warn",&nonfatal);
         c_clean(vblock,"windop",hosout,&one);
         }
      curlin ++;
      top = bottom + skippix[0] - kernel[0];
      bottom = top + kernel[0];
      }		     /*   end of while loop   */
   curcat ++;
   }              /*   end of band loop    */
}               /*   end of nmrimg loop  */
break;

case EWORD:
for (image = 0; image < nmrimg; image++)
{
for (band = 1; band <= nbands[image]; band++)
   {
   bnd = bands[image][band-1];
   curlin = 1;
   bando1++;
   bando2++;
   for (j = 0; j < maxsamp; j++)
      {
      values[j] = MAXWORD;
      mask_flag[j] = FALSE;
      }
   top = 0;
   bottom = top + kernel[0];
   while (bottom <= windl)		/* loop for each output line */
      {
					/* loop for each line of block */
      for (curent = (top+1);curent <= bottom; curent++)
         {
         status = c_eread(&fdesc[image], &bnd, &curent, thebuf, &one);
         if (status == E_FAIL)
            {
            if (c_eclose(&fdesc1[0]) != E_SUCC)
               c_errmsg("Nonfatal error encountered","windop-warn",&one);
            c_clean(vblock,"windop",hosout,&one);
            }

	 /* Process if no mask values exist
	    ------------------------------*/
	 if (masknum == 0)
	    {
	    for (j = 0; j < maxsamp; j++)
	       {
	       left = j * skippix[1] * size;
	       right = left + (kernel[1] * size);
	       for (sample = left; sample < right; sample += size)
	          {
                  temp = *(short *)(thebuf + (sample));
		  if (temp < values[j])
		     values[j] = temp;
	          } 		       /* end of sample loop   */
	       }		    /* end of maxsamp loop  */
	    }

	 /* Process if mask values exist
	   -----------------------------*/
	 else
	    {
	    for (j = 0; j < maxsamp; j++)
	       {
	       left = j * skippix[1] * size;
	       right = left + (kernel[1] * size);
	       for (sample = left; sample < right; sample += size)
	          {
                  temp = *(short *)(thebuf + (sample));
		  for (mask = 0; (mask < masknum) && (!mask_flag[j]); mask++)
		     {
		     if ((temp >= maskval[mask][0]) && temp<=maskval[mask][1])
			{
			mask_flag[j] = TRUE;
			values[j] = pixval;
			}
		     }
		  if (!mask_flag[j])
		     {
		     if (temp < values[j])
		        values[j] = temp;
		     }
	          } 			/* end of sample loop  		 */
	       }		    /* end of maxsamp loop 	     */
	    }			/* end of mask value condition  */
	 }		     /* end of curent loop  	   */
/*
 Assign minimum value to output
 -------------------------------*/
      for (i = 0; i < maxsamp; i++)
	 {
       	 *(short *)(thebuf1 + (i * size)) = values[i];
	 mask_flag[i] = 0.0;
         values[i] = MAXWORD;
	 }
      if (mnmx)
	 {
         status = c_pxmnmx(thebuf1, &maxsamp, &minval[bando1-1],
						&maxval[bando1-1], &odtype); 
	 if (status != E_SUCC)
	    c_errmsg("Nonfatal error encountered","windop-warn",&nonfatal);
	 }
      status = c_ewrite(fdesc1, &bando1, &curlin, thebuf1, &one);
      if (status == E_FAIL)
         {
         if (c_eclose(&fdesc1[0]) != E_SUCC)
	    c_errmsg("Nonfatal error encountered","windop-warn",&nonfatal);
         c_clean(vblock,"windop",hosout,&one);
         }
      curlin ++;
      top = bottom + skippix[0] - kernel[0];
      bottom = top + kernel[0];
      }		     /*   end of while loop   */
   curcat ++;
   }              /*   end of band loop    */
}               /*   end of nmrimg loop  */
break;

case ELONG:
for (image = 0; image < nmrimg; image++)
{
for (band = 1; band <= nbands[image]; band++)
   {
   bnd = bands[image][band-1];
   curlin = 1;
   bando1++;
   bando2++;
   for (j = 0; j < maxsamp; j++)
      {
      values[j] = MAXLONG;
      mask_flag[j] = FALSE;
      }
   top = 0;
   bottom = top + kernel[0];
   while (bottom <= windl)		/* loop for each output line */
      {
					/* loop for each line of block */
      for (curent = (top+1);curent <= bottom; curent++)
         {
         status = c_eread(&fdesc[image], &bnd, &curent, thebuf, &one);
         if (status == E_FAIL)
            {
            if (c_eclose(&fdesc1[0]) != E_SUCC)
               c_errmsg("Nonfatal error encountered","windop-warn",&one);
            c_clean(vblock,"windop",hosout,&one);
            }

	 /* Process if no mask values exist
	    ------------------------------*/
	 if (masknum == 0)
	    {
	    for (j = 0; j < maxsamp; j++)
	       {
	       left = j * skippix[1] * size;
	       right = left + (kernel[1] * size);
	       for (sample = left; sample < right; sample += size)
	          {
                  temp = *(long *)(thebuf + (sample));
		  if (temp < values[j])
		     values[j] = temp;
	          } 		       /* end of sample loop   */
	       }		    /* end of maxsamp loop  */
	    }

	 /* Process if mask values exist
	   -----------------------------*/
	 else
	    {
	    for (j = 0; j < maxsamp; j++)
	       {
	       left = j * skippix[1] * size;
	       right = left + (kernel[1] * size);
	       for (sample = left; sample < right; sample += size)
	          {
                  temp = *(long *)(thebuf + (sample));
		  for (mask = 0; (mask < masknum) && (!mask_flag[j]); mask++)
		     {
		     if ((temp >= maskval[mask][0]) && temp<=maskval[mask][1])
			{
			mask_flag[j] = TRUE;
			values[j] = pixval;
			}
		     }
		  if (!mask_flag[j])
		     {
		     if (temp < values[j])
		        values[j] = temp;
		     }
	          } 			/* end of sample loop  		 */
	       }		    /* end of maxsamp loop 	     */
	    }			/* end of mask value condition  */
	 }		     /* end of curent loop  	   */
/*
 Assign minimum value to output
 -------------------------------*/
      for (i = 0; i < maxsamp; i++)
	 {
       	 *(long *)(thebuf1 + (i * size)) = values[i];
	 mask_flag[i] = 0.0;
         values[i] = MAXLONG;
	 }
      if (mnmx)
	 {
         status = c_pxmnmx(thebuf1, &maxsamp, &minval[bando1-1],
						&maxval[bando1-1], &odtype); 
	 if (status != E_SUCC)
	    c_errmsg("Nonfatal error encountered","windop-warn",&nonfatal);
	 }
      status = c_ewrite(fdesc1, &bando1, &curlin, thebuf1, &one);
      if (status == E_FAIL)
         {
         if (c_eclose(&fdesc1[0]) != E_SUCC)
	    c_errmsg("Nonfatal error encountered","windop-warn",&nonfatal);
         c_clean(vblock,"windop",hosout,&one);
         }
      curlin ++;
      top = bottom + skippix[0] - kernel[0];
      bottom = top + kernel[0];
      }		     /*   end of while loop   */
   curcat ++;
   }              /*   end of band loop    */
}               /*   end of nmrimg loop  */
break;

case EREAL:
for (image = 0; image < nmrimg; image++)
{
for (band = 1; band <= nbands[image]; band++)
   {
   bnd = bands[image][band-1];
   curlin = 1;
   bando1++;
   bando2++;
   for (j = 0; j < maxsamp; j++)
      {
      values[j] = MAXREAL;
      mask_flag[j] = FALSE;
      }
   top = 0;
   bottom = top + kernel[0];
   while (bottom <= windl)		/* loop for each output line */
      {
					/* loop for each line of block */
      for (curent = (top+1);curent <= bottom; curent++)
         {
         status = c_eread(&fdesc[image], &bnd, &curent, thebuf, &one);
         if (status == E_FAIL)
            {
            if (c_eclose(&fdesc1[0]) != E_SUCC)
               c_errmsg("Nonfatal error encountered","windop-warn",&one);
            c_clean(vblock,"windop",hosout,&one);
            }

	 /* Process if no mask values exist
	    ------------------------------*/
	 if (masknum == 0)
	    {
	    for (j = 0; j < maxsamp; j++)
	       {
	       left = j * skippix[1] * size;
	       right = left + (kernel[1] * size);
	       for (sample = left; sample < right; sample += size)
	          {
                  temp = *(float *)(thebuf + (sample));
		  if (temp < values[j])
		     values[j] = temp;
	          } 		       /* end of sample loop   */
	       }		    /* end of maxsamp loop  */
	    }

	 /* Process if mask values exist
	   -----------------------------*/
	 else
	    {
	    for (j = 0; j < maxsamp; j++)
	       {
	       left = j * skippix[1] * size;
	       right = left + (kernel[1] * size);
	       for (sample = left; sample < right; sample += size)
	          {
                  temp = *(float *)(thebuf + (sample));
		  for (mask = 0; (mask < masknum) && (!mask_flag[j]); mask++)
		     {
		     if ((temp >= maskval[mask][0]) && temp<=maskval[mask][1])
			{
			mask_flag[j] = TRUE;
			values[j] = pixval;
			}
		     }
		  if (!mask_flag[j])
		     {
		     if (temp < values[j])
		        values[j] = temp;
		     }
	          } 			/* end of sample loop  		 */
	       }		    /* end of maxsamp loop 	     */
	    }			/* end of mask value condition  */
	 }		     /* end of curent loop  	   */
/*
 Assign minimum value to output
 -------------------------------*/
      for (i = 0; i < maxsamp; i++)
	 {
       	 *(float *)(thebuf1 + (i * size)) = values[i];
	 mask_flag[i] = 0.0;
         values[i] = MAXREAL;
	 }
      if (mnmx)
	 {
         status = c_pxmnmx(thebuf1, &maxsamp, &minval[bando1-1],
						&maxval[bando1-1], &odtype); 
	 if (status != E_SUCC)
	    c_errmsg("Nonfatal error encountered","windop-warn",&nonfatal);
	 }
      status = c_ewrite(fdesc1, &bando1, &curlin, thebuf1, &one);
      if (status == E_FAIL)
         {
         if (c_eclose(&fdesc1[0]) != E_SUCC)
	    c_errmsg("Nonfatal error encountered","windop-warn",&nonfatal);
         c_clean(vblock,"windop",hosout,&one);
         }
      curlin ++;
      top = bottom + skippix[0] - kernel[0];
      bottom = top + kernel[0];
      }		     /*   end of while loop   */
   curcat ++;
   }              /*   end of band loop    */
}               /*   end of nmrimg loop  */
break;
}
return(E_SUCC);
}
