/***************************************************************************

NAME      		   WINDOP

PURPOSE   Performs systematic pixel or cluster averaging.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
R.MCKINNEY      April 1989	Conversion to LAS 5.0
B.Ailts		Feb 1990	Made error messages between pdf and code the 
				same

S.NELSON	March 1992	Added masked value

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
   WINDOP must be run under TAE.

PROJECT   	         LAS		                

LANGUAGE		  C	

ALGORITHM (FLOW)

The LAS program is initialized.  Next the parameters are
retrieved thru GETPAR.  The main processing routine is called
to apply the sampling.  Finally, the
history file is updated and the routine completed.

ALGORITHM REFERENCES     

None.

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "windop.h"

/* The following variables must be declared global for image I/O routines
-------------------------------------------------------------------------*/
int  fdesc[MAXIMG+1];           /* image file descriptors used for image IN  */
int  fdesc1[2];                 /* image file descriptors used for image OUT */
int  gdesc;		        /* group descriptor used for image IN        */
int  gdesc1;		        /* group descriptor used for image OUT       */
extern long curent;		/* current processing line		     */
extern long curcat;		/* current processing band		     */
long bands[MAXIMG][MAXBND + 1]; /* band specification for first image        */
double minval[MAXBND];          /* min pixel value array		     */
double maxval[MAXBND];          /* max pixel value array		     */

unsigned char *thebuf;		/* image input buffer		 	     */
unsigned char *thebuf1;		/* image output buffer		 	     */

void main
(
  void
)
{
struct PARBLK vblock;           /* TAE parameter block    		     */

long kernel[2];			/* kernel dimensions (nl,ns)		     */
long masknum;                   /* number of mask values                     */
long msgtim;			/* elpased time between display messages     */
long mnmx;			/* min/max value			     */
long nmrimg;			/* number of input images for image          */
long nonfatal = 1;		/* nonfatal error flag			     */
long nbands[MAXIMG];		/* number of bands per input image           */
long odtype;			/* odtype for processing                     */
long one = 1;			/* number one				     */
long skippix[2];		/* number of pixels to skip (nl,ns)	     */
long std_out;			/* logical unit # for standard output        */
long totbnd;			/* total number of bands in first image      */
long window[MAXIMG][4];		/* window specification for first image      */

float maskval[10][2];           /* mask values                               */
float pixval;                   /* pixel value to replace a mask value       */

static char hosin[MAXIMG][CMLEN];/* buffer storage        	   	     */
static char hosout[CMLEN];       /* buffer storage        		     */
static char taein[MAXIMG][CMLEN];/* buffer storage       		     */
static char taeout[CMLEN];       /* buffer storage       		     */
char   typeopt[5];               /* type of averaging                        */


c_inlas(&vblock, &std_out);  /* initialize las and tae  */

/* Get user-specified parameters and TAE globals
------------------------------------------------*/
getpar(&vblock, taein, hosin, taeout, hosout, &nmrimg, bands, nbands,
		window, &msgtim, &mnmx, &totbnd, kernel,skippix,typeopt,
		&odtype, maskval, &masknum, &pixval);

/* Perform the windopt algebra 
------------------------------*/
proces(&vblock,hosin,hosout,nmrimg,bands,nbands,
		window,msgtim,mnmx,totbnd,kernel,skippix,typeopt,odtype,
		masknum, maskval, pixval); 

/* Create associated history file
---------------------------------*/
if (c_hstout(&vblock, hosout, hosin[0] , &nmrimg, " ") != E_SUCC)
   c_errmsg("Nonfatal error encountered", "windop-warn",&nonfatal);

/* Inform user of successful completion
---------------------------------------*/
c_complt("WINDOP", taeout, &one);
}
