/*******************************************************************************
NAME        	    	           GET_INFO	

PURPOSE     GET_INFO gets the ancillary information stored in the input image's
            associated corrections file.

PROGRAM HISTORY
PROGRAMMER	  DATE		REASON
----------	  ----		------
T. Jensen         5/92

ALGORITHM:
This file is then opened and the values are retrieved.  Next, the weight
table to be used in resampling is generated when using the -CUBIC or
-BILINEAR subcommands.  If the -TABLE subcommand is specified, the weight
table is read from INFILE, the disk file containing the table of user-
entered resampling weights.
 

ALGORITHM REFERENCES:
1.  This function was derived from the LAS CORRECT function, get_info.c
    routine. 
*******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "xcorrect.h"
#include "protos.h"


void get_info
(
    double mnlinlen[],     /* mean actual line length	                */
    long *nomlen,       /* nominal line length      		    	     */
    char *xcorhos      /* pointer to correction file host name descriptor */
)

{

FILE *fp = NULL;  	      /* corrections file pointer	                     */			
double sum = 0.00;     /* sum of actual line lengths                          */

long  linlen [MAXLINE];/* actual line lengths                                */
long  i;              /* loop index					     */
long  fatal = -1;     /* fatal error flag		   	             */
long  temp;	      /* temporary variable				     */

  
/* Open associated corrections file */
fp = fopen(xcorhos,"r");
if (fp == NULL)
   c_errmsg("Error opening correction file","xcorrect-openerr",&fatal);

/* Retrieve nominal line length */
fscanf(fp,"%d",nomlen);

/* Retrieve actual line lengths */
for (i=0; i< MAXLINE; i++)
	{
	fscanf(fp,"%d %d",&temp,&linlen[i]);
        sum = sum + linlen[i];
        if (i+1 != temp)
	    {
	    fclose(fp);
	    c_errmsg("Error reading correction file","xcorrect-rderr",&fatal);
	    }
      	}
/* Use avg line length in calculations to follow in other routines         */
for(i = 0; i < MAXLINE; i++)
   {
   mnlinlen [i] = sum / MAXLINE;
   }

fclose(fp);
return;
}
