/*
 *  This is the preferences function source file for the preferences panel of 
 *  the X Image Display application.  It contains the callbacks, action 
 *  routines, event handlers, and procedure code required for the panel.
 *
 *  By L.Huewe, USGS/EROS Data Center, 02/93.
 *    - X Image Display, v1.0.
 *  Modified by L.Huewe, USGS/EROS Data Center, 3/94
 *    - Modified the preferences panel to require the user to select the
 *      "apply" button before the colormap change is employed. 
 *
 */

#include "xid.h"
#include "xid_func.h"

/* local function prototypes */
void pref_destroy
(void);

void pref_colormap
(
   Widget w
);
/* end prototypes */

/* ARGSUSED */
void pref_apply
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine closes the "Preferences" panel and applys any changes that
   have been made. 
                                                                
By L. Huewe, USGS/EROS Data Center, 3/94

Algorithm Description:
******************************************************************************/
{
   if (XmToggleButtonGetState(pref_200) && (*xid.colormap_ptr != xid.colormap)) 
      pref_colormap(pref_200);
   else if (XmToggleButtonGetState(pref_256) &&
            (*xid.colormap_ptr == xid.colormap)) 
      pref_colormap(pref_256);

   /* Close the preferences panel.  */
   pref_destroy();
}

/* ARGSUSED */
void pref_cancel
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine closes the "Preferences" panel without applying any changes. 
                                                                
By L. Huewe, USGS/EROS Data Center, 3/94

Algorithm Description:
******************************************************************************/
{
   /* Close the preferences panel.  */
   pref_destroy();
}

/* ARGSUSED */
void pref_destroy
(void)

/******************************************************************************
Purpose:
   This routine closes the "Preferences" panel. 
                                                                
By L. Huewe, USGS/EROS Data Center, 2/93

Algorithm Description:
******************************************************************************/
{
   /* Set the "Preferences" button to sensitive.  */
   XtSetSensitive(main_preferences_button,True);

   /* Pop down the "Preferences" panel.  */
   XtUnmanageChild(Pref_form);
   XtPopdown(Shell_Pref);
}

/* ARGSUSED */
void pref_setcursor
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   Set the current cursor color as well as setting the foreground color
   of the option menu to show the selected color.
                                                                
By L. Huewe, USGS/EROS Data Center, 2/93

Algorithm Description:
******************************************************************************/
{
   Arg       args[1];          /* Arguments for setting resource values.     */

   /* Set the foreground color of the option menu to reflect the color
   selected.  */
   xid.cursor_color = (long)client;
   XtSetArg(args[0], XmNforeground,
            xid.color_table[RGBBW_IDX + xid.cursor_color].pixel); 
   XtSetValues(pref_cursor_option, args, 1);
}

/* ARGSUSED */
void pref_colormap
(
   Widget w
)

/******************************************************************************
Purpose:
   This switches the image colormap based on whether wishes to use 200
   colors for the image or 256 colors.
                                                                
By L. Huewe, USGS/EROS Data Center, 3/93

Algorithm Description:
******************************************************************************/
{
   unsigned long   pixelval[256],  /* Array for pixel values.                */
                   plane_masks[1], /* Working plane mask.                    */
                   wk_pixels[1],   /* Return pixel value array.              */
                   red_mask,       /* Mask values to reference 24-bit pixels */
                   green_mask,
                   blue_mask;
   long            bwflag = 0,     /* Indicates whether or not the displayed
                                      image is a black and white image.      */
                   i,              /* Looping variable.                      */
                   one = 1,        /* Constant one.                          */
                   rgbmem[3],      /* RGB memory plane IDs.                  */
                   status,         /* Status indicator.                      */
                   three = 3;      /* Constant three.                        */

   /* Get the currently displayed memory plane IDs.  */
   for (i=0; i < 3; i++)
      rgbmem[i] = xid_view.rgbplanes[i];

   /* Determine whether or not a black and white image is displayed.  */
   if (xid.view != -1)
   {
      if ((rgbmem[RED] == rgbmem[GREEN]) && (rgbmem[GREEN] == rgbmem[BLUE]))
         bwflag = TRUE;
      else
         bwflag = FALSE;
   }

   if (w == pref_200)
   { 
      XtVaSetValues(Shell_MainWin,XtNcolormap,xid.colormap,NULL);
      XtVaSetValues(Shell_FullResWin,XtNcolormap,xid.colormap,NULL);
      XtVaSetValues(Shell_ZoomWin,XtNcolormap,xid.colormap,NULL);
      xid.colormap_ptr = &xid.colormap;
      xid.colortable_ptr = &xid.color_table[0];
      if ((xid.view != -1) &&(bwflag))
      {
         status = load_display(rgbmem,&one,XtWindow(main_image_bulletinboard),
                               Shell_MainWin,main_image_bulletinboard,
                               &xid_planes[rgbmem[RED] - 1].dspwin[NL],
                               &xid_planes[rgbmem[RED] - 1].dspwin[NS],
                               &xid_planes[rgbmem[RED] - 1].maxval,
                               xid_planes[rgbmem[RED] - 1].image_data,NULL,
                               NULL,TRUE,&xid_mem.image_pixmap,TRUE,FALSE,
                               FALSE,FALSE);
      }
      else if (xid.view != -1) 
      {
         status = load_display(rgbmem,&three,
                               XtWindow(main_image_bulletinboard),
                               Shell_MainWin,main_image_bulletinboard,
                               &xid_planes[rgbmem[RED] - 1].dspwin[NL],
                               &xid_planes[rgbmem[RED] - 1].dspwin[NS],
                               &xid_planes[rgbmem[RED] - 1].maxval,
                               xid_planes[rgbmem[RED] - 1].image_data,
                               xid_planes[rgbmem[GREEN] - 1].image_data,
                               xid_planes[rgbmem[BLUE] - 1].image_data,TRUE,
                               &xid_mem.image_pixmap,TRUE,FALSE,FALSE,FALSE);
      }
   }
   else if (w == pref_256)
   {

      /* Create the 256 image color colormap.  */
      if (xid.imagemap == NULL)
      {
         xid.imagemap = XCreateColormap(xid.display, RootWindow(xid.display,
                                        xid.screen), xid.visual_info[0].visual,
                                        AllocNone);
         if (xid.imagemap == DefaultColormap(xid.display,xid.screen))
         {
            xid_message("Error creating colormap");
            return;
         }

         /* Allocate the working colorcells.  */
         if (xid.visual_info[0].visual->class == PseudoColor)
         {

            /* Allocate the colorcells.  */
            status = XAllocColorCells(xid.display,xid.imagemap,True,
                                      plane_masks, 0,pixelval,256);
            if (status == 0)
            {
               xid_message("Unable to allocate colorcells");
               return;
            }

            /* Assign the remaining colorcells for image data.  */
            for (i = 0; i < 256; i++)
               xid.color_image[i].pixel = pixelval[i]; 
         }
         else            /* DirectColor */
         {

            /* Allocate color planes.  */
            status = XAllocColorPlanes(xid.display,xid.imagemap,True,wk_pixels,
                                       1,8,8,8,&red_mask,&green_mask,
                                       &blue_mask);
            if (status == 0)
            {
               xid_message("Unable to allocate color planes");
               return;
            }

            /* Assign the pixel values for the color cells.  */
            for (i = 0; i < 256; i++)
               xid.color_image[i].pixel = (i) + (i << 8) + (i << 16);

            /* Assign a gray scale ramp to the image data colors.  */
            for (i = 0; i < 256; i++)
            {
               xid.color_image[i].red = xid.color_image[i].green = 
                            xid.color_image[i].blue = (unsigned short)(i << 8);
               xid.color_image[i].flags = DoRed | DoGreen | DoBlue;
            }
            XStoreColors(xid.display,xid.imagemap,xid.color_image,256);
         }
      }
      XtVaSetValues(Shell_MainWin,XtNcolormap,xid.imagemap,NULL);
      XtVaSetValues(Shell_FullResWin,XtNcolormap,xid.imagemap,NULL);
      XtVaSetValues(Shell_ZoomWin,XtNcolormap,xid.imagemap,NULL);
      xid.colormap_ptr = &xid.imagemap;
      xid.colortable_ptr = &xid.color_image[0];

      if ((xid.view != -1) && (bwflag)) 
      {
         status = load_display(rgbmem,&one,XtWindow(main_image_bulletinboard),
                               Shell_MainWin,main_image_bulletinboard,
                               &xid_planes[rgbmem[RED] - 1].dspwin[NL],
                               &xid_planes[rgbmem[RED] - 1].dspwin[NS],
                               &xid_planes[rgbmem[RED] - 1].maxval,
                               xid_planes[rgbmem[RED] - 1].image_data,NULL,
                               NULL,TRUE,&xid_mem.image_pixmap,TRUE,FALSE,
                               FALSE,FALSE);
      }
      else if (xid.view != -1) 
      {
         status = load_display(rgbmem,&three,
                               XtWindow(main_image_bulletinboard),
                               Shell_MainWin,main_image_bulletinboard,
                               &xid_planes[rgbmem[RED] - 1].dspwin[NL],
                               &xid_planes[rgbmem[RED] - 1].dspwin[NS],
                               &xid_planes[rgbmem[RED] - 1].maxval,
                               xid_planes[rgbmem[RED] - 1].image_data,
                               xid_planes[rgbmem[GREEN] - 1].image_data,
                               xid_planes[rgbmem[BLUE] - 1].image_data,TRUE,
                               &xid_mem.image_pixmap,TRUE,FALSE,FALSE,FALSE);
      }
   }
}
