/*
 *  This is the zoom function source file for the zoom panel of the
 *  X Image Display application.  It contains the callbacks, action routines,
 *  event handlers, and procedure code required for the panel.
 *
 *  By D. Akkerman/L.Huewe, USGS/EROS Data Center, 9/91.
 *    - X Image Display, v1.0.
 *
 */
#include <X11/IntrinsicP.h>
#include "geompak.h"
#include "pixman.h" /* c_pxcopy definition */
#include "xid.h"
#include "xid_graphics.h"
#include "xid_func.h"
#include "../xid_widgets/ProcessM.h"

/* local function prototypes */
void zoom_image
(
   Widget w
);

void zoom_enlarge
(
   struct ENLARGE *zoom         /* Enlargement parameters & buffers (I/O)    */
);

void horiz_zoom
(
   struct ENLARGE *zoom         /* Enlargement parameters & buffers (I/O)    */
);

void vert_zoom
(
   struct ENLARGE *zoom         /* Enlargement parameters & buffers (I/O)    */
);
int zoom_init
(
   struct ENLARGE *zoom         /* Enlargement parameters & buffers (I/O)    */
);
/* end prototypes */

void zoom_arrows
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine increases or decreases the current zoom factor based on the
   which arrow button was pressed.  Currently there is an arbitrary zoom
   factor limit of 16.
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   long    i;                  /* Looping index.                             */
   char    outval[3];          /* Zoom factor to be output.                  */

   /* Increase or decrease the current zoom factor.  The minimum zoom factor is
      2 and the maximum is currently 16 (this is an arbitrary limit).  */
   if (strcmp(client,"UP") == 0)
      xid_zoom.factor++;
   else
      xid_zoom.factor--;

   xid_zoom.factor = (xid_zoom.factor > 16) ? 2  : xid_zoom.factor;
   xid_zoom.factor = (xid_zoom.factor < 2)  ? 16 : xid_zoom.factor;

   /* Set the text string to reflect the new zoom factor.  */
   sprintf(outval,"%ld",xid_zoom.factor);
   XmTextSetString(zoom_text,outval);

   /* Set the zoom factor in the parameter structure used in init_zoom.  */
   xid_zoom.parms.factor = xid_zoom.factor;

   /* Free and recreate the zoom pixmaps if the graphics window is active.  */
   if ((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.zoom_pix[0] != 0))
   {
      if (xid_graphics.zoom_pix[0] != 0)
      {
         for (i=0; i < USER_GRCOLORS; i++)
            XFreePixmap(xid.display,xid_graphics.zoom_pix[i]);
      }

      /* Create zoom window pixmaps.  */
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);

      for (i=0; i < USER_GRCOLORS; i++)
      {
         xid_graphics.zoom_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                  zoom_win_bulletinboard),
                                                  xid_zoom.parms.factor * 
                                                  xid_zoom.parms.in_samps,
                                                  xid_zoom.parms.factor * 
                                                  xid_zoom.parms.in_lines,1);
         XFillRectangle(xid.display,xid_graphics.zoom_pix[i],
                        xid_graphics.rb_gc,0,0,
                        xid_zoom.parms.factor * xid_zoom.parms.in_samps,
                        xid_zoom.parms.factor * xid_zoom.parms.in_lines);
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
   }
}

void zoom_create
(void)

/******************************************************************************
Purpose:
   This routine initializes the parameter structure used later by the
   routine zoom_enlarge.  Checks are made to be sure that enough data exists
   to extract the amount required by the enlargement routine.
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   long    create = FALSE;     /* Indicates whether or not full resolution
                                  pixmaps were created.                      */
   long    i;                  /* Looping index.                             */

   /* If the viewed image is not large enough to extract ZOOM_WINX samples
      and/or ZOOM_WINY lines, reset the zoom window size.  */
   xid_zoom.parms.factor = xid_zoom.factor;
   xid_zoom.parms.in_lines = ZOOM_WINY;
   xid_zoom.parms.in_samps = ZOOM_WINX;

   /* Assign the pixmap and image data values to NULL.  */
   xid_zoom.image_pixmap = 0;
   xid_zoom.image_data = (unsigned char *)NULL;

   /* Create the zoom window pixmaps if the graphics window is active or if the
      main window graphics pixmaps exist.  */
   if ((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.pixmap[0] != 0))
   {
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);

      if (xid_graphics.zoom_pix[0] == 0)
      {
         for (i=0; i < USER_GRCOLORS; i++)
         {
            xid_graphics.zoom_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                     zoom_win_bulletinboard),
                                                     xid_zoom.parms.factor * 
                                                     xid_zoom.parms.in_samps,
                                                     xid_zoom.parms.factor * 
                                                     xid_zoom.parms.in_lines,1);
         }
         create = TRUE;
      }
      if (create) 
      {
         for (i=0; i < USER_GRCOLORS; i++)
         {
            XFillRectangle(xid.display,xid_graphics.zoom_pix[i],
                           xid_graphics.rb_gc,0,0,
                           xid_zoom.parms.factor * xid_zoom.parms.in_samps,
                           xid_zoom.parms.factor * xid_zoom.parms.in_lines);
         }
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
   }

   /* Add appropriate event handlers if the cursor window is active.  */
   if ((xid_cursor.flags & CURSOR_ACTIVE) && (xid_cursor.flags & CURSOR_MAN))
   {
      XtAddEventHandler(zoom_win_bulletinboard,ButtonReleaseMask|
                        KeyReleaseMask,FALSE,cursor_track,NULL);
   }
   else if ((xid_cursor.flags & CURSOR_ACTIVE) &&
            (xid_cursor.flags & CURSOR_AUTO))
   {
      XtAddEventHandler(zoom_win_bulletinboard,ButtonReleaseMask|
                        PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,
                        NULL);
   }

   /* Add event handlers to detect pointer motion and button release events. */
   XtAddEventHandler(zoom_win_bulletinboard,PointerMotionMask|KeyReleaseMask,
                     FALSE,cursor_pointer,NULL);

   /* Add an event handler to detect exposure events in the zoom image window*/
   XtAddEventHandler(zoom_win_bulletinboard,ExposureMask,FALSE,graphics_expose,
                     NULL);
 
   /* Set active flag.  */
   xid_zoom.flags |= ZOOM_ACTIVE;
}

void zoom_destroy
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine removes event handlers no longer needed, frees resources and
   closes both the zoom panel and zoom image panel.
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   long    i;                  /* Looping variable.                          */

   /* Remove event handlers no longer needed.  */
   if (xid_graphics.flags & GR_ACTIVE)
   {
      XtRemoveEventHandler(zoom_win_bulletinboard,ButtonReleaseMask,FALSE,
                           graphics_expose,NULL);
   }
   if ((xid_cursor.flags & CURSOR_ACTIVE) && (xid_cursor.flags & CURSOR_MAN))
   {
      XtRemoveEventHandler(zoom_win_bulletinboard,ButtonReleaseMask|
                           KeyReleaseMask,FALSE,cursor_track,NULL);
   }
   else if ((xid_cursor.flags & CURSOR_ACTIVE) &&
            (xid_cursor.flags & CURSOR_AUTO))
   {
      XtRemoveEventHandler(zoom_win_bulletinboard,ButtonReleaseMask|
                           PointerMotionMask|KeyReleaseMask,FALSE,cursor_track,
                           NULL);
   }
   XtRemoveEventHandler(zoom_win_bulletinboard,PointerMotionMask|
                        KeyReleaseMask,FALSE,cursor_pointer,NULL);

   /* Clear the zoom window.  */
   XSetWindowBackground(xid.display,XtWindow(zoom_win_bulletinboard),
                        xid.color_table[RGBBW_IDX + BLACK_COL].pixel);
   XClearWindow(xid.display,XtWindow(zoom_win_bulletinboard));

   /* Free the zoomed image pixmap and image data buffer.  */
   if (xid_zoom.image_pixmap != 0)
   {
      XFreePixmap(xid.display,xid_zoom.image_pixmap);
      xid_zoom.image_pixmap = 0;
   }
   if (xid_zoom.image_data != NULL)
   {
      free(xid_zoom.image_data);
      xid_zoom.image_data = (unsigned char *)NULL;
   }
   if (xid_zoom.parms.horiz_table != NULL)
   {
      free(xid_zoom.parms.horiz_table);
      xid_zoom.parms.horiz_table = NULL;
   }
   if (xid_zoom.parms.work != NULL)
   {
      free(xid_zoom.parms.work);
      xid_zoom.parms.work = NULL;
   }
   if (xid_zoom.parms.input != NULL)
   {
      free(xid_zoom.parms.input);
      xid_zoom.parms.input = NULL;
   }
   if (xid_zoom.parms.output != NULL)
   {
      free(xid_zoom.parms.output);
      xid_zoom.parms.output = NULL;
   }

   /* Clear out the zoomed image graphics pixmaps.  */
   if (xid_graphics.zoom_pix[0] != 0)
   {
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);

      for (i=0; i < USER_GRCOLORS; i++)
      {
         XFillRectangle(xid.display,xid_graphics.zoom_pix[i],
                        xid_graphics.rb_gc,0,0,
                        xid_zoom.parms.factor * xid_zoom.parms.in_samps,
                        xid_zoom.parms.factor * xid_zoom.parms.in_lines);
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
   }

   /* Close both the zoom panel and zoom image panel and set the zoom button on
      the main panel to be sensitive again.  */
   XtUnmanageChild(Zoom_bulletinboard);
   XtUnmanageChild(Zoom_scrollwin);
   XtPopdown(Shell_Zoom);
   XtPopdown(Shell_ZoomWin);
   XtSetSensitive(main_zoom_button,True);

   /* Reset the active flag.  */
   xid_zoom.flags = xid_zoom.flags ^ ZOOM_ACTIVE;
}

void zoom_entry
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine allows the user to enter a specific zoom factor (rather than
   using the arrow buttons).
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   int     i;
   char    outval[3];

   /* Get the user specified zoom factor and verify it is valid.  */
   sscanf(XmTextGetString(zoom_text),"%ld",&xid_zoom.factor);

   xid_zoom.factor = (xid_zoom.factor > 16) ? 2  : xid_zoom.factor;
   xid_zoom.factor = (xid_zoom.factor < 2)  ? 16 : xid_zoom.factor;

   /* Set the text string to reflect the zoom factor.  */
   sprintf(outval,"%ld",xid_zoom.factor);
   XmTextSetString(zoom_text,outval);

   /* Set the zoom parameter structure to reflect the new zoom factor.  */
   xid_zoom.parms.factor = xid_zoom.factor;

   /* Free and recreate the zoom pixmaps if the graphics window is active.  */
   if ((xid_graphics.flags & GR_ACTIVE) || (xid_graphics.zoom_pix[0] != 0))
   {
      if (xid_graphics.zoom_pix[0] != 0)
      {
         for (i=0; i < USER_GRCOLORS; i++)
            XFreePixmap(xid.display,xid_graphics.zoom_pix[i]);
      }

      /* Create zoom window pixmaps.  */
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);

      for (i=0; i < USER_GRCOLORS; i++)
      {
         xid_graphics.zoom_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                  zoom_win_bulletinboard),
                                                  xid_zoom.parms.factor * 
                                                  xid_zoom.parms.in_samps,
                                                  xid_zoom.parms.factor * 
                                                  xid_zoom.parms.in_lines,1);
         XFillRectangle(xid.display,xid_graphics.zoom_pix[i],
                        xid_graphics.rb_gc,0,0,
                        xid_zoom.parms.factor * xid_zoom.parms.in_samps,
                        xid_zoom.parms.factor * xid_zoom.parms.in_lines);
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
   }
}

void zoom_mode
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine sets the appropriate flags indicating whether to operate in
   "continuous zoom" mode or "point and click" mode.  In continuous zoom
   mode, the zoom window is updated each time the user moves the cursor.  In
   point and click mode, the zoom window is updated only when the user presses
   the appropriate mouse button. 
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   char *data = (char *)client;

   /* Set the appropriate flags.  */
   xid_zoom.flags = xid_zoom.flags & ~(ZOOM_POINT | ZOOM_CONT);
   if (*data == 'P')
      xid_zoom.flags = xid_zoom.flags | ZOOM_POINT;
   else
      xid_zoom.flags = xid_zoom.flags | ZOOM_CONT;
}

void zoom_type
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine sets the appropriate flags indicating the resampling method
   to use during the enlargement process.  
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   struct ENLARGE tmp_parms;       /* Temporary zoom enlargement parameters. */
   long           i;               /* Looping index.                         */

   /* Set the flag indicating the resampling type.  */
   xid_zoom.flags = xid_zoom.flags & ~(ZOOM_NN | ZOOM_BI | ZOOM_CC);
   if (strcmp(client,"NN") == 0)
   {
      xid_zoom.flags = xid_zoom.flags | ZOOM_NN;
      strcpy(xid_zoom.parms.method,"NN");
   }
   else if (strcmp(client,"BI") == 0)
   {
      xid_zoom.flags = xid_zoom.flags | ZOOM_BI;
      strcpy(xid_zoom.parms.method,"BI");
   }
   else if (strcmp(client,"CC") == 0)
   {
      xid_zoom.flags = xid_zoom.flags | ZOOM_CC;
      strcpy(xid_zoom.parms.method,"CC");
   }

   /* Free and recreate the zoom pixmaps if the graphics window is active.  */
   if (xid_graphics.flags & GR_ACTIVE)
   {
      /* Call zoom_init to make sure the in_lines and in_samps reflects the new
         resampling method.  */
      tmp_parms = xid_zoom.parms;
      zoom_init(&tmp_parms);

      for (i=0; i < USER_GRCOLORS; i++)
         XFreePixmap(xid.display,xid_graphics.zoom_pix[i]);

      /* Create zoom window pixmaps.  */
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);

      for (i=0; i < USER_GRCOLORS; i++)
      {
         xid_graphics.zoom_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                  zoom_win_bulletinboard),
                                                  tmp_parms.factor *
                                                  tmp_parms.in_samps,
                                                  tmp_parms.factor *
                                                  tmp_parms.in_lines,1);
         XFillRectangle(xid.display,xid_graphics.zoom_pix[i],
                        xid_graphics.rb_gc,0,0,
                        tmp_parms.factor * tmp_parms.in_samps,
                        tmp_parms.factor * tmp_parms.in_lines);
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
   }
}

void zoom_button
(
   Widget w,
   caddr_t client,
   caddr_t call
)

/******************************************************************************
Purpose:
   This routine updates the zoom window from either the main image window
   or full resolution window.
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   XButtonEvent    *event;         /* X Event structure.                     */
   long            i;              /* Looping index.                         */
   long            plane_idx;      /* Displayed image memory plane index.    */

   event = (XButtonEvent *)call;

   /* Return if the zoom window is not active.  */
   if (!(xid_zoom.flags & ZOOM_ACTIVE))
      return;

   /* If the middle mouse button was selected in the main image window, update
      the zoomed window.  */
   if ((w == main_image_bulletinboard) && (event->button == 2) &&
       (xid.view != -1))
   {
      xid_zoom.flags = xid_zoom.flags & ~(ZOOM_MAIN | ZOOM_FULLRES);
      xid_zoom.x = event->x;
      xid_zoom.y = event->y;
      xid_zoom.flags = xid_zoom.flags | ZOOM_MAIN;
      zoom_image(w);
   }
   else if ((w == fullres_win_bulletinboard) && (event->button == 2) &&
            (xid.view != -1))
   {
      xid_zoom.flags = xid_zoom.flags & ~(ZOOM_MAIN | ZOOM_FULLRES);
      xid_zoom.x = event->x;
      xid_zoom.y = event->y;
      xid_zoom.flags = xid_zoom.flags | ZOOM_FULLRES;
      zoom_image(w);
   }

   /* Update the graphics information.  */
   if ((xid_graphics.pixmap[0] != 0) && (xid.view != -1) && 
       (event->button == 2))
   {
      if (xid_graphics.zoom_pix[0] == 0)
      {
         /* Create zoom window pixmaps.  */
         XSetBackground(xid.display,xid_graphics.rb_gc,0);
         XSetForeground(xid.display,xid_graphics.rb_gc,0);

         for (i=0; i < USER_GRCOLORS; i++)
         {
            xid_graphics.zoom_pix[i] = XCreatePixmap(xid.display,XtWindow(
                                                     zoom_win_bulletinboard),
                                                     xid_zoom.parms.factor * 
                                                     xid_zoom.parms.in_samps,
                                                     xid_zoom.parms.factor *
                                                     xid_zoom.parms.in_lines,
                                                     1);
            XFillRectangle(xid.display,xid_graphics.zoom_pix[i],
                           xid_graphics.rb_gc,0,0,
                           xid_zoom.parms.factor * xid_zoom.parms.in_samps,
                           xid_zoom.parms.factor * xid_zoom.parms.in_lines);
         }
         XSetBackground(xid.display,xid_graphics.rb_gc,1);
         XSetForeground(xid.display,xid_graphics.rb_gc,1);
      }
      plane_idx = xid_view.rgbplanes[RED] - 1;
      XSetBackground(xid.display,xid_graphics.rb_gc,0);
      XSetForeground(xid.display,xid_graphics.rb_gc,0);
      for (i=0; i < USER_GRCOLORS; i++)
      {
         XFillRectangle(xid.display,xid_graphics.zoom_pix[i],
                        xid_graphics.rb_gc,0,0,
                        xid_zoom.parms.factor * xid_zoom.parms.in_samps, 
                        xid_zoom.parms.factor * xid_zoom.parms.in_lines);
      }
      XSetBackground(xid.display,xid_graphics.rb_gc,1);
      XSetForeground(xid.display,xid_graphics.rb_gc,1);
      graphics_update(2,plane_idx);
   }
}

int zoom_init
(
   struct ENLARGE *zoom         /* Enlargement parameters & buffers (I/O)    */
)

/******************************************************************************
Purpose:
   This routine allocates the appropriate buffers and initializes the 
   appropriate fields in the zoom parameter structure in preparation for
   the enlargement process.
                                                                
By D. Steinwand, USGS/EROS Data Center, 3/89

Algorithm Description:
   This routine was taken from the LAS TIEPTS application.

Algorithm References:
  1.  D.E. Friedmann, "Two-Dimensional Resampling of Line Scan Imagery
       by One-Dimensional Processing," Photogrammetric Engineering and
       Remote Sensing, Vol. 47, No. 10, pp. 1459-1467, October 1981.
  2.  D.E. Friedmann, "Operational Resampling for Correcting Images
       to a Geocoded Format," MacDonald, Dettwiler & Associates Ltd.,
       Richmond, B.C., Canada, May 1981.
  3.  S.K. Park and R.A. Schowengerdt, "Image Reconstruction by
       Parametric Cubic Convolution," Computer Vision, Graphics,
       and Image Processing 23, Academic Press, Inc. 1983. pp. 258-272.
******************************************************************************/
{
   struct RWTAB   user_rwt; /* User entered resampling weight table.         */
   long           *index;   /* Weight table index.                           */
   float          tmpdel;   /* Temporary delta value.                        */
   float          delta;    /* Delta value; I (loop counter) * FACTOR.       */
   float          inc;      /* Table increment for table-based resampling.   */
   float          inv;      /* 1.0 / zoom->factor.                           */
   float          offset;   /* Offset for even zoom factors.                 */
   long           count;    /* Index incrementer for nn table calculation.   */
   long           i,j,k;    /* Loop indices.                                 */
   unsigned long  size;     /* Size of weight table buffer.                  */

   /* Free any previously allocated buffer space.  */
   if (zoom->horiz_table != NULL)
   {
      free(zoom->horiz_table);
      zoom->horiz_table = NULL;
   }
   if (zoom->work != NULL)
   {
      free(zoom->work);
      zoom->work = NULL;
   }
   if (zoom->input != NULL)
   {
      free(zoom->input);
      zoom->input = NULL;
   }
   if (zoom->output != NULL)
   {
      free(zoom->output);
      zoom->output = NULL;
   }

   /* Calculate 1/zoom->factor.  */
   inv = 1.0 / (float) zoom->factor;

   /* Nearest neighbor table initialization.  */
   if (strcmp(zoom->method, "NN") == 0)
   {
      size = 2 * zoom->factor * sizeof(long);
      zoom->horiz_table = zoom->vert_table = (long *)malloc(size);
      if (zoom->horiz_table == NULL) 
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }
      count = zoom->factor / 2;
      if ((float)count < ((float)zoom->factor / 2.0)) count++;
      for (index = zoom->horiz_table, i = count; i > 0;i--)
      {
         *index++ = 256;
         *index++ = 0;
      }
      for (i = zoom->factor - count; i--;)
      {
         *index++ = 0;
         *index++ = 256;
      }
      zoom->horiz_dim = zoom->vert_dim = 2;
      zoom->extract_lines = zoom->in_lines + 1;
      zoom->extract_samps = zoom->in_samps + 1;
   }

   /* Parametric cubic convolution table initialization.  */
   if (strcmp(zoom->method, "CC") == 0)
   {
      size = 4 * zoom->factor * sizeof(long);
      zoom->horiz_table = zoom->vert_table = (long *)malloc(size);
      if (zoom->horiz_table == NULL) 
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }
      count = zoom->factor / 2;
      if ((float)count < ((float)zoom->factor / 2.0)) offset = 0;
      else offset = 1.0 / (2.0 * zoom->factor);
      for(index = zoom->horiz_table, i = 0; i < zoom->factor; i++)
      {
         delta = ((float)i * inv) + offset;
         tmpdel = 1 - delta;
         *index++ = (long)(256 * zoom->alpha * delta * SQUARE(tmpdel));
         *index++ = (long)(256 * (1.0 - ((zoom->alpha+3.0) * SQUARE(delta)) +
                           ((zoom->alpha + 2.0) * CUBE(delta))));
         *index++ = (long)(256 * (1.0 - ((zoom->alpha+3.0) * SQUARE(tmpdel)) +
                           ((zoom->alpha+2.0) * CUBE(tmpdel))));
         *index++ = (long)(256 * zoom->alpha * tmpdel * SQUARE(delta));
      }
      zoom->horiz_dim = zoom->vert_dim = 4;
      zoom->extract_lines = zoom->in_lines + 3;
      zoom->extract_samps = zoom->in_samps + 3;
   }

   /* Bilinear interpolation table initialization.  */
   if (strcmp(zoom->method, "BI") == 0)
   {
      size = 2 * zoom->factor * sizeof(long);
      zoom->horiz_table = zoom->vert_table = (long *)malloc(size);
      if (zoom->horiz_table == NULL) 
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }
      count = zoom->factor / 2;
      if ((float)count < ((float)zoom->factor / 2.0)) offset = 0;
      else offset = 1.0 / (2.0 * zoom->factor);
      for (index = zoom->horiz_table, i = 0; i < zoom->factor; i++)
      {
         delta = ((float)i * inv) + offset;
         *index++ = (long)(256 * (1 - delta));
         *index++ = (long)(256 * delta);
      }
      zoom->horiz_dim = zoom->vert_dim = 2;
      zoom->extract_lines = zoom->in_lines + 1;
      zoom->extract_samps = zoom->in_samps + 1;
   }

   /* User entered table-based resampling table read & initialization.  */
   if (strcmp(zoom->method, "TABLE") == 0)
   {
      if(c_getrwt(zoom->table_file, &user_rwt) != E_SUCC)
         return(E_FAIL);
      zoom->horiz_dim = user_rwt.dim[0];
      zoom->vert_dim = user_rwt.dim[1];
      zoom->extract_lines = zoom->in_lines + (zoom->vert_dim - 1);
      zoom->extract_samps = zoom->in_samps + (zoom->horiz_dim - 1);
      size=(zoom->horiz_dim * zoom->factor * sizeof(long)) +
              (zoom->vert_dim * zoom->factor * sizeof(long));
      zoom->horiz_table = (long *)malloc(size);
      if (zoom->horiz_table == NULL)
      {
         xid_message("Insufficient memory available");
         return(XID_F);
      }
      zoom->vert_table = zoom->horiz_table + (zoom->horiz_dim * zoom->factor);

      count = zoom->factor / 2;
      if ((float)count < ((float)zoom->factor / 2.0))
         offset = 0;
      else
         offset = 32.0 / (2.0 * zoom->factor);
 
      index = zoom->horiz_table;
      inc = 32.0 / (float)zoom->factor;
      for (k = 0; k < zoom->factor; k++)
      {
         for (j=0,i = (long)(((float)k * inc) + offset);j<zoom->horiz_dim;j++)
            *index++ = (long)(256 * user_rwt.table1[i][j]);
      }

      index = zoom->vert_table;
      for (k = 0; k < zoom->factor; k++)
      {
         for (j = 0,i = (long)(((float)k * inc) + offset);j<zoom->vert_dim;j++)
            *index++ = (long)(256 * user_rwt.table2[i][j]);
      }
   }

   /* Allocate input, hybrid, and output space buffers.  */
   zoom->hybrid_samps = zoom->in_samps * zoom->factor;

   size = zoom->extract_lines * zoom->extract_samps * sizeof(unsigned char);
   zoom->input = (unsigned char *)malloc(size);
   if (zoom->input == NULL) 
   {
      xid_message("Insufficient memory available");
      return(XID_F);
   }

   size = zoom->extract_lines * zoom->hybrid_samps * sizeof(long);
   zoom->work = (long*)malloc(size);
   if (zoom->work == NULL) 
   {
      xid_message("Insufficient memory available");
      return(XID_F);
   }

   size = zoom->in_lines * zoom->factor * zoom->in_samps * zoom->factor * 
             sizeof(unsigned char);
   zoom->output = (unsigned char *)malloc(size);
   if (zoom->output == NULL) 
   {
      xid_message("Insufficient memory available");
      return(XID_F);
   }
   return(E_SUCC);
}

void zoom_image
(
   Widget w
)

/******************************************************************************
Purpose:
   This routine extracts the appropriate image data, enlarges it based on the
   zoom factor and resampling method selected and displays it in the zoom
   image window.
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   long           bands;           /* Number of bands displayed.             */
   long           dspwin[4];       /* Display window.                        */
   long           dtype = EBYTE;   /* Data type.                             */
   long           dummy[3];        /* Dummy array to be passed to
                                      load_display.                          */
   long           i,j,k;           /* Looping indices.                       */
   long           nbytes;          /* Number of bytes to copy.               */
   long           nl;              /* Number of lines in zoomed image.       */
   long           ns;              /* Number of samples in zoomed image.     */
   long           nsamps;          /* Number of samples in displayed image.  */
   long           plane_idx[3];    /* Memory plane indices of displayed
                                      image.                                 */
   long           sl;              /* Starting line of window to zoom.       */
   long           ss;              /* Starting sample of window to zoom.     */
   unsigned char  *imgptr = NULL;  /* Pointer to displayed image data.       */
   unsigned char  *inptr;          /* Pointer to input array for enlarge
                                      routine.                               */
   unsigned char  *zmptr;          /* Pointer to zoomed data.                */

   /* Determine the memory plane indices and display window of the displayed
      image.  */
   plane_idx[RED] = xid_view.rgbplanes[RED] - 1;
   plane_idx[GREEN] = xid_view.rgbplanes[GREEN] - 1;
   plane_idx[BLUE] = xid_view.rgbplanes[BLUE] - 1;

   if (w == main_image_bulletinboard)
   {
      dspwin[SL] = xid_planes[plane_idx[RED]].dspwin[SL];
      dspwin[SS] = xid_planes[plane_idx[RED]].dspwin[SS];
      dspwin[NL] = xid_planes[plane_idx[RED]].dspwin[NL];
      dspwin[NS] = xid_planes[plane_idx[RED]].dspwin[NS];
   }
   else if (w == fullres_win_bulletinboard)
   {
      dspwin[SL] = xid_fullres[0].dspwin[SL];
      dspwin[SS] = xid_fullres[0].dspwin[SS];
      dspwin[NL] = xid_fullres[0].dspwin[NL];
      dspwin[NS] = xid_fullres[0].dspwin[NS];
   }

   /* Set up the processing message widget with the correct title.  */
   XtVaSetValues(load_dialog_shell,XmNtitle,"Zoom - Processing",NULL);

   /* Display a processing message.  */
   XeProcessMBegin(process_widget,0,1,"Zooming image, please wait ...  ",True,
                   False,False);

   /* Initialize the zoom parameter structure.  */
   zoom_init(&xid_zoom.parms);

   /* Continue resetting the zoom window size until zoom_init returns values to
      extract which fall within the displayed image limits.  */
   while (xid_zoom.parms.extract_lines > dspwin[NL])
   {
      xid_zoom.parms.in_lines--;
      zoom_init(&xid_zoom.parms);
   }
   while (xid_zoom.parms.extract_samps > dspwin[NS])
   {
      xid_zoom.parms.in_samps--;
      zoom_init(&xid_zoom.parms);
   }

   /* Determine the number of lines and samples in the zoomed image.  */
   nl = xid_zoom.parms.factor * xid_zoom.parms.in_lines;
   ns = xid_zoom.parms.factor * xid_zoom.parms.in_samps;
   nbytes = nl * ns;

   /* Determine the starting line and sample of the window to be extracted.  */
   sl = xid_zoom.y - xid_zoom.parms.extract_lines / 2;
   if (sl < 0)
      sl = 0;
   ss = xid_zoom.x - xid_zoom.parms.extract_samps / 2;
   if (ss < 0)
      ss = 0;

   /* Reset starting line and sample if the window falls outside the displayed
      image.  */
   if ((sl + xid_zoom.parms.extract_lines) > dspwin[NL])
   {
      sl = dspwin[NL] - xid_zoom.parms.extract_lines - 1;
      if (sl < 0)
         sl = 0;
   }

   if ((ss + xid_zoom.parms.extract_samps) > dspwin[NS])
   {
      ss = dspwin[NS] - xid_zoom.parms.extract_samps - 1;
      if (ss < 0)
         ss = 0;
   }

   xid_zoom.window[SL] = sl;
   xid_zoom.window[SS] = ss;
   xid_zoom.window[NL] = xid_zoom.parms.in_lines; 
   xid_zoom.window[NS] = xid_zoom.parms.in_samps;

   /* Free previously allocated resources.  */
   if (xid_zoom.image_data != NULL)
      free(xid_zoom.image_data);

   /* Allocate buffer space for the output image.  */
   if ((xid_view.rgbplanes[RED] == xid_view.rgbplanes[GREEN]) &&
       (xid_view.rgbplanes[GREEN] == xid_view.rgbplanes[BLUE]))
   {
      bands = 1;
      xid_zoom.image_data = (unsigned char *)malloc(nbytes);
      if (xid_zoom.image_data == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
   }
   else
   {
      bands = 3;
      xid_zoom.image_data = (unsigned char *)malloc(nbytes * bands); 
      if (xid_zoom.image_data == NULL)
      {
         xid_message("Insufficient memory available");
         return;
      }
   }

   /* Zoom the extracted image window.  */
   for (i=0,zmptr = xid_zoom.image_data; i < bands; i++, zmptr += (nl * ns))
   {
      if (plane_idx[i] >= 0)
      {
         /* Extract the appropriate window.  */
         inptr = xid_zoom.parms.input;
         if (w == main_image_bulletinboard)
            imgptr = xid_planes[plane_idx[i]].image_data;
         else if (w == fullres_win_bulletinboard)
            imgptr = xid_fullres[i].image_data;
         nsamps = dspwin[NS];
         for (j=sl; j < sl + xid_zoom.parms.extract_lines; j++)
         {
            for (k=ss; k < ss + xid_zoom.parms.extract_samps; k++,inptr++)
               *inptr = *(imgptr + (j * nsamps + k));
         }

         /* Enlarge the extracted image data based on the previously selected
            zoom factor and resampling method.  */
         zoom_enlarge(&xid_zoom.parms);
         c_pxcopy(xid_zoom.parms.output,zmptr,&dtype,&nbytes);
      }
   }

   /* Free the previously allocated zoomed image pixmap.  */
   if (xid_zoom.image_pixmap != 0)
      XFreePixmap(xid.display,xid_zoom.image_pixmap);

   /* Load the zoomed data to the appropriate window.  */
   if (bands == 1)
   {
      load_display(dummy,&bands,XtWindow(zoom_win_bulletinboard),Shell_ZoomWin,
                   zoom_win_bulletinboard,&nl,&ns,
                   &xid_planes[plane_idx[RED]].maxval,xid_zoom.parms.output,
                   NULL,NULL,FALSE,&xid_zoom.image_pixmap,FALSE,TRUE,FALSE,
                   FALSE);
   }
   else
   {
      load_display(dummy,&bands,XtWindow(zoom_win_bulletinboard),Shell_ZoomWin,
                   zoom_win_bulletinboard,&nl,&ns,
                   &xid_planes[plane_idx[RED]].maxval,xid_zoom.image_data,
                   xid_zoom.image_data+nbytes,xid_zoom.image_data+(2*nbytes),
                   FALSE,&xid_zoom.image_pixmap,FALSE,TRUE,FALSE,FALSE);
   }
   xid_zoom.parms.in_lines = ZOOM_WINY;
   xid_zoom.parms.in_samps = ZOOM_WINX;

   /* Close the processing message window.  */
   XeProcessMEnd(process_widget);
}

void zoom_enlarge
(
   struct ENLARGE *zoom         /* Enlargement parameters & buffers (I/O)    */
)

/******************************************************************************
Purpose:  
   This routine enlarges a window using an interpolated zoom.  Resampling 
   techniques include nearest neighbor, bilinear interpolation,
   parametric cubic convolution, and a user entered table of
   resampling weights.

By D. Steinwand, USGS/EROS Data Center, 3/89

Algorithm Description:
   This routine was taken from the LAS TIEPTS application.

Algorithm References:
  1.  D.E. Friedmann, "Two-Dimensional Resampling of Line Scan Imagery
       by One-Dimensional Processing," Photogrammetric Engineering and
       Remote Sensing, Vol. 47, No. 10, pp. 1459-1467, October 1981.
  2.  D.E. Friedmann, "Operational Resampling for Correcting Images
       to a Geocoded Format," MacDonald, Dettwiler & Associates Ltd.,
       Richmond, B.C., Canada, May 1981.
  3.  S.K. Park and R.A. Schowengerdt, "Image Reconstruction by
       Parametric Cubic Convolution," Computer Vision, Graphics,
       and Image Processing 23, Academic Press, Inc. 1983. pp. 258-272.
******************************************************************************/
{
   /* Horizontal pass.  */
   horiz_zoom(zoom);

   /* Vertical pass.  */
   vert_zoom(zoom);

   return;
}

void horiz_zoom
(
   struct ENLARGE *zoom         /* Enlargement parameters & buffers (I/O)    */
)

/******************************************************************************
Purpose:
   This routine performs interpolated enlargement in horizontal direction.
                                                                
By D. Steinwand, USGS/EROS Data Center, 3/89 

Algorithm Description:
   This routine was taken from the LAS TIEPTS application.
******************************************************************************/
{
   long           h,i,j,k;  /* Loop counters.                                */
   long           *hybrid;  /* Pointer to current pixel in hybrid space.     */
   long           pad;      /* Pad so extract_lines = in_lines - pad.        */
   long           *weight;  /* Pointer to current weight table location.     */
   unsigned char  *in_buf;  /* Pointer to current input pixel.               */

   /* Initialize offsets and pointers to buffers.  */
   pad = zoom->extract_lines - zoom->in_lines;
   hybrid = zoom->work;
   in_buf = zoom->input;

   /* Perform the horizontal resampling.  The resampling type is determined in
      zoom_init() and is placed in a table.  This routine applies the table,
      regardless of the resampling method chosen (yes, even nearest neighbor).
      The table is scaled up by 8 bits (multiplied by 256) to keep some
      precision beyond the radix point and to allow the elimination of all
      floating point calculations.  The removal of the floating point
      calculations sped this algorithm up significantly.  The original range of
      the data in the hybrid space is not restored until the completion of the
      vertical pass.  */
   for (h = zoom->extract_lines; h--; in_buf += pad)
   {
      for (i = zoom->in_samps; i--; in_buf++)
      {
         for (weight = zoom->horiz_table, j = zoom->factor; j--; hybrid++)
         {
            for (*hybrid = 0, k = 0; k < zoom->horiz_dim; k++)
               *hybrid += *(in_buf + k) * *weight++;
         }
      }
   }
   return;
}

void vert_zoom
(
   struct ENLARGE *zoom         /* Enlargement parameters & buffers (I/O)    */
)

/******************************************************************************
Purpose:
   This routine performs interpolated enlargement in vertical direction.
                                                                
By D. Steinwand, USGS/EROS Data Center, 3/89

Algorithm Description:
   This routine was taken from the LAS TIEPTS application.
******************************************************************************/
{
   long           h,i,j,k;     /* Loop counters.                             */
   long           *hybrid;     /* Pointer to current pixel in hybrid space.  */
   long           inc;         /* Loop counter.                              */
   long           temp;        /* Temporary floating point output result.    */
   long           *weight;     /* Pointer to current weight table location.  */
   unsigned char  *out_buf;    /* Pointer to current output pixel.           */

   /* The vertical resampling pass is very similar to the horizontal resampling
      pass, however, the added complexity in the code comes from accessing the
      data in columns instead of rows (lines).  As in the horizontal pass, the
      weight table is scaled by 8 bits to remove floating point calculations.
      Since this pass and the horizontal pass involve multiplication of scaled
      values, the final result must be adjusted by performing an integer
      division by 16 bits (8 bits applied in the horizontal pass and 8 bits
      applied in the vertical pass).  A bit shift would be more efficient, but
      probably not portable, so it is avoided.  */
   for (h = 0; h < zoom->hybrid_samps; h++)
   {
      out_buf = zoom->output + h;
      hybrid = zoom->work + h;
      for (i = zoom->in_lines; i--; hybrid += zoom->hybrid_samps)
      {
         for(weight=zoom->vert_table,j=zoom->factor; j > 0; j--)
         {
            temp = 0;
            for (inc = 0, k = 0; k < zoom->vert_dim; k++)
            {
               temp += *(hybrid + inc) * *weight++;
               inc += zoom->hybrid_samps;
            }
            if (temp < 65536) *out_buf = 0;
            else if (temp > 16711680) *out_buf = 255;
            else *out_buf = (unsigned char)(temp / 65536);
            out_buf += zoom->hybrid_samps;
         }
      }
   }
   return;
}

void zoom_alpha
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine pops up the panel which allows the user to enter the 
   parametric cubic convolution alpha parameter.
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   Arg         args[1];    /* Variable used to set widget resources.         */
   Dimension   height;     /* Height of zoom panel.                          */
   Position    popx;       /* X position of pop-up panel.                    */
   Position    popy;       /* Y position of pop-up panel.                    */

   XmToggleButtonCallbackStruct *acs=(XmToggleButtonCallbackStruct*)call;

   /* Retrieve the height of the zoom bulletinboard widget.  */
   XtSetArg(args[0],XmNheight,&height);
   XtGetValues(zoom_bulletinboard,args,1);

   /* Translate the X/Y position relative to the zoom panel to an X/Y position
      relative to the screen.  */
   XtTranslateCoords(zoom_bulletinboard,(Position)0,(Position)height/2,
                     &popx,&popy);

   /* Position the pop-up panel.  */
   XtMoveWidget(zoom_alpha_dialog,popx,popy);

   /* Pop-up the panel.  */
   if (acs->set == True)
      XtPopup(zoom_alpha_dialog,XtGrabExclusive);
}

void zoom_ok
(
   Widget w,
   XtPointer client,
   XtPointer call
)

/******************************************************************************
Purpose:
   This routine reads the value entered for the parametric cubic convolution
   alpha parameter and pops down the panel.
                                                                
By L. Huewe, USGS/EROS Data Center, 12/91

Algorithm Description:
******************************************************************************/
{
   char    *tmpstr;            /* Temporary storage for a text string.       */

   /* Get the user entered value.  */
   tmpstr = XmTextGetString(zoom_alpha_text);
   sscanf(tmpstr,"%f",&xid_zoom.parms.alpha);
   XtFree(tmpstr);

   /* Pop down the panel.  */
   XtPopdown(zoom_alpha_dialog);
}
