/*
 *   ContextP.h
 */

/*
 *  Private declarations/definitions for Context.c
 */

#ifndef _XeContextP_h
#define _XeContextP_h
 
/*
 * Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of USGS not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  USGS makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
 
/*
 *  Change log:
 *
 *      D. Akkerman, USGS/EROS Data Center, 7/92
 *       - inception.
 *
 */

#include <Xm/BulletinBP.h>
#include "Context.h"
#include "ContextI.h"

/* Definitions for default resource values.  */

/* Working constants.  */

#define GAP             5               /* Spacing between widget instances. */
#define PIXMAP_HEIGHT   50              /* Informational pixmap dimensions.  */
#define PIXMAP_WIDTH    50              /* Default widget event mask.        */
#define EVENT_MASK      (ButtonReleaseMask)

/* Private Structures.  */

typedef struct _Context_Internal
{
   char      context_token[ELEMENT_MAX_LENGTH];
                                        /* Token string used for searchs.    */
   XmString  context_name;              /* Motif string name for the context */
   XmString  element_names[MAX_ELEMENTS];
                                        /* Motif string name for the element */
   long       num_elements;             /* Total # of elements.              */
   long       num_widget_ids;           /* Total # of widget IDs that send
                                           events to the context.            */
   Widget     event_widgets[MAX_WIDGETS];
                                        /* Widget IDs that can generate
                                           events for the context.           */
   Boolean    event_handler;            /* If TRUE then first element_handler
                                           is an eventhandler for the mouse. */
   Boolean    nonmaskable;              /* Call eventhandler on nonmaskable
                                           events? (almost always FALSE)     */
   EventMask  event_mask;               /* Event mask for eventhandler.      */
 
   /* Pointers to callbacks for elements.  If event_handler is TRUE then the
      first function pointer must be to the eventhandler for the mouse.      */
   void (*element_handlers[MAX_ELEMENTS])();
   XtPointer  client_data[MAX_ELEMENTS];
                                       /* Argument to pass to callback.      */
  void (*activate_callback)();         /* Callback when context is activated */
  XtPointer  active_client_data;       /* Client data for active callback.   */

} Context_Internal;

/* Class definition.  */

typedef struct _XeContextClassPart
{
   long dummy;                          /* Xt placeholder.                   */
} XeContextClassPart;

typedef struct _XeContextClassRec
{
   CoreClassPart                 core_class;
   CompositeClassPart            composite_class;
   ConstraintClassPart           constraint_class;
   XmManagerClassPart            manager_class;
   XmBulletinBoardClassPart      bulletinboard_class;
   XeContextClassPart            context_class;
} XeContextClassRec;

extern XeContextClassRec xeContextClassRec;

/* Instance definition.  */

typedef struct _XeContextPart
{
   XFontStruct *label_font;             /* Font structs for the label and    */
   XFontStruct *button_font;            /* button widgets.                   */
   XmFontList  label_font_list;         /* Motif font lists for the fonts.   */
   XmFontList  button_font_list; 
   Context_Internal
               *context_list[MAX_CONTEXTS];
                                        /* Internal context structures.      */
   long        current_context;         /* Current context index.            */
   long        num_contexts;            /* Current total # of contexts.      */
   Dimension   option_button_border_width;
                                        /* Border widths for the option menu.*/
   Dimension   option_menu_border_width;
   Dimension   drawingarea_border_width;
                                        /* Border width for the drawingarea. */
   Dimension   additional_button_border_width;
                                        /* Border widths for the additional  */
   Dimension   additional_menu_border_width;
                                        /* menu.                             */
   Widget      frame;                   /* Internal widget tree ids.         */
   Widget      frame_bulletinBoard;
   Widget      drawingArea[NUM_BUTTONS];
   Widget      context_label;
   Widget      button_label[NUM_BUTTONS];
   Widget      optionMenu;
   Widget      option_menuShell;
   Widget      option_pulldownMenu;
   Widget      option_pushButton[MAX_CONTEXTS];
   Widget      additional_menuShell;
   Widget      additional_menuBar;
   Widget      additional_cascadeButton;
   Widget      additional_pulldownMenu;
   Widget      additional_pushButton[MAX_ELEMENTS];
   Pixmap      button_pix[NUM_BUTTONS]; /* Pixmaps for the mouse buttons.    */
   XtEventHandler
               current_eventhandler;    /* Current eventhandler for a context*/
   EventMask   current_eventmask;       /* Current event mask for a context. */
   Boolean     current_nonmaskable;     /* Current nonmaskable flag.         */
   XtPointer   current_client;          /* Current client data for a context.*/
   Visual      *visual;                 /* Visual for the internal popup
                                           shells.                           */
} XeContextPart;

typedef struct _XeContextRec
{
   CorePart              core;
   CompositePart         composite;
   ConstraintPart        constraint;
   XmManagerPart         manager;
   XmBulletinBoardPart   bulletin_board;
   XeContextPart         context;
} XeContextRec;

#endif /* _XeContextP_h */
