/*
 *  ProcessM.h
 */

/*
 *  Public header file for the processing message widget.
 */

/*
 *  Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of USGS not be used in advertising or 
 *  publicity pertaining to distribution of the software without specific, 
 *  written prior permission.  USGS makes no representations about the 
 *  suitability of this software for any purpose.  It is provided "as is" 
 *  without express or implied warranty.
 *
 *  USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS 
 *  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 *  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 *  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *  The Xpm pixmap routine use is:
 *
 *  Copyright 1990-93 GROUPE BULL
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of GROUPE BULL not be used in advertising
 *  or publicity pertaining to distribution of the software without specific,
 *  written prior permission.  GROUPE BULL makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 *
 *  GROUPE BULL disclaims all warranties with regard to this software,
 *  including all implied warranties of merchantability and fitness,
 *  in no event shall GROUPE BULL be liable for any special,
 *  indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits,
 *  whether in an action of contract, negligence or other tortious
 *  action, arising out of or in connection with the use
 *  or performance of this software.
 *
 */

/*
 *  Change log:
 *
 *      D. Akkerman, USGS/EROS Data Center, 9/92
 *       - inception.
 *
 *      D. Akkerman, USGS/EROS Data Center, 7/93
 *       - updated for IPC use.
 *
 *      D. Akkerman, USGS/EROS Data Center, 12/93
 *       - Fixed problems with using icon for processing.
 *       - Added code for more efficient color assignment for "pie" processing.
 *       - Added code to reduce CPU usage while paused.
 *
 */

/*
 *  Resources:
 *
 *    Name                Class            Data Type     Default        Access
 *
 * XeNprocessMMessageString XeCProcessMMessageString
 *                                              String   
 *                                              "Processing, please wait . . ."
 *                                                                       C,S
 * XeNprocessMColors      XeCProcessMColors     XeProcessMColor
 *                                                        NULL           C,S
 * XeNprocessMUpdatePercent XeCProcessMUpdatePercent
 *                                              XeProcessMIterator
 *                                                        20             C,S
 * XeNprocessMColorList   XeCProcessMColorList  XeProcessMIconColor*
 *                                                        NULL           C
 * XeNprocessMDepth       XeCProcessMDepth      int       NULL           C,S
 *
 * XeNprocessMWindow      XeCProcessMWindow     Window    NULL           C,S
 *
 * XeNprocessMVisual      XeCProcessMVisual     Visual*   CopyFromParent C,S
 *
 */

/* Make it safe to include this file multiple times.  */

#ifndef PROCESSM_H
#define PROCESSM_H

/* Include the public header file for the superclass.  */

#include <Xm/MessageB.h>

/* Include the public header file for the XPM package.  */

#include "xpm.h"

/* Define the new resources for the processing message widget.  */

#define XeNprocessMMessageString        "processMMessageString"
#define XeCProcessMMessageString        "ProcessMMessageString"

#define XeNprocessMColors               "processMColors"
#define XeCProcessMColors               "ProcessMColors"

#define XeNprocessMUpdatePercent        "processMUpdatePercent"
#define XeCProcessMUpdatePercent        "ProcessMUpdatePercent"

#define XeNprocessMColorList            "processMColorList"
#define XeCProcessMColorList            "ProcessMColorList"

#define XeNprocessMDepth                "processMDepth"
#define XeCProcessMDepth                "ProcessMDepth"

#define XeNprocessMWindow               "processMWindow"
#define XeCProcessMWindow               "ProcessMWindow"

#define XeNprocessMVisual               "processMVisual"
#define XeCProcessMVisual               "ProcessMVisual"

/* Define the type definition for the processing message widget.  */

typedef struct _XeProcessMRec           *XeProcessMWidget;

/* Define the external reference to the class record pointer.  */

extern WidgetClass xeProcessMWidgetClass;

/* Define the public constants.  */

#define XePROCESSM_MAX_COLORS                   10
#define XePROCESSM_DEFAULT_LABEL                "Processing, please wait . . ."
#define XePROCESSM_DEFAULT_UPDATE_PERCENT       20
#define XePROCESSM_GAP                          5
#define XePROCESSM_NO_RESPONSE                  0
#define XePROCESSM_CANCEL                       1
#define XePROCESSM_PAUSE                        2

/* Define the default labels for the processing message widget.  */

#define XePROCESSM_DEFAULT_OK_PAUSE             "Pause"
#define XePROCESSM_DEFAULT_OK_RESUME            "Resume"

/* Define the warning/error messages.  */

#define XePROCESSM_BAD_COLOR_SPEC \
                                "ProcessM: Bad color specification"
#define XePROCESSM_BAD_UPDATE_PERCENTAGE \
                                "ProcessM: Bad update percentage specified"
#define XePROCESSM_BAD_ITERATOR_VALUE \
                                "ProcessM: Bad iterator value specified"
#define XePROCESSM_NO_INIT      "Processm: No initialization"

/* Define the type definitions for message processing.  */

typedef unsigned long           XeProcessMIterator;
typedef unsigned long           XeProcessMResponse;


#define XePROCESSM_PIXMAP_SIZE          (unsigned int)50
#define XePROCESSM_NO_INITIALIZATION    -1

/* Define the public structure for color specifications.  */

typedef struct _XeProcessMColor
{
   unsigned int  num_colors;
   Pixel         color_array[XePROCESSM_MAX_COLORS];
} XeProcessMColor;

/* Define the constants for the xpm software.  */

#define XePROCESSM_XPM_MAX_COLORS        3

#define XePROCESSM_COLOR1                "symbol_1"
#define XePROCESSM_COLOR2                "symbol_2"
#define XePROCESSM_COLOR3                "symbol_3"

/* Define the pixmap colors for the icons.  */

#define XePROCESSM_ICON_COLOR1          "gray30"
#define XePROCESSM_ICON_COLOR2          "black"
#define XePROCESSM_ICON_COLOR3          "sandybrown"

/* Define a structure used to pass in pixmap colors.  */

typedef struct _XeProcessMIconColor
{
   Pixel pixel_array[XePROCESSM_XPM_MAX_COLORS];
} XeProcessMIconColor;

/* Define the public functions for the processing message widget.  */

extern void
#ifdef _NO_PROTO
XeProcessMBegin();
/*
 *  Widget processm_widget;
 *  XeProcessMIterator iterator_start;
 *  XeProcessMIterator iterator_end;
 *  String processing_message;
 *  Boolean use_symbol;
 *  Boolean use_pause;
 *  Boolean use_cancel;
 */
#else
XeProcessMBegin(Widget processm_widget,
                XeProcessMIterator iterator_start,
                XeProcessMIterator iterator_end,
                String processing_message,
                Boolean use_symbol,
                Boolean use_pause,
                Boolean use_cancel);
#endif

extern XeProcessMResponse
#ifdef _NO_PROTO
XeProcessMProcess();
/*
 *  Widget processm_widget;
 *  XeProcessMIterator iterator_current;
 */
#else
XeProcessMProcess(Widget processm_widget,
                  XeProcessMIterator iterator_current);
#endif

void XeProcessMBegin
(
  Widget processm_widget,
  XeProcessMIterator iterator_start,
  XeProcessMIterator iterator_end,
  String processing_message,
  Boolean use_symbol,
  Boolean use_pause,
  Boolean use_cancel
);

XeProcessMResponse XeProcessMProcess
(
  Widget processm_widget,
  XeProcessMIterator iterator_current
);

void XeProcessMEnd
(
  Widget processm_widget
);

#endif

