/*
 *  ProcessMP.h
 */

/*
 *  Private header file for the processing message widget.
 */

/*
 *  Copyright (c) 1992 U.S. Geological Survey (USGS)
 *
 *  Permission to use, copy, modify, distribute, and sell this software and its
 *  documentation for any purpose is hereby granted without fee, provided that
 *  the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of USGS not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  USGS makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 *
 *  USGS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL USGS
 *  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 *  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 *  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

/*
 *  The Xpm pixmap routine use is:
 *
 *  Copyright 1990-93 GROUPE BULL
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of GROUPE BULL not be used in advertising
 *  or publicity pertaining to distribution of the software without specific,
 *  written prior permission.  GROUPE BULL makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 *
 *  GROUPE BULL disclaims all warranties with regard to this software,
 *  including all implied warranties of merchantability and fitness,
 *  in no event shall GROUPE BULL be liable for any special,
 *  indirect or consequential damages or any damages
 *  whatsoever resulting from loss of use, data or profits,
 *  whether in an action of contract, negligence or other tortious
 *  action, arising out of or in connection with the use
 *  or performance of this software.
 *
 */
 
/*
 *  Change log:
 *
 *      D. Akkerman, USGS/EROS Data Center, 9/92
 *       - inception.
 *
 *      D. Akkerman, USGS/EROS Data Center, 7/93
 *       - updated for IPC.
 *
 *      D. Akkerman, USGS/EROS Data Center, 12/93
 *       - Fixed problems with using icon for processing.
 *       - Added code for more efficient color assignment for "pie" processing.
 *       - Added code to reduce CPU usage while paused.
 *
 */

/* Make it safe to include this file multiple times.  */

#ifndef PROCESSMP_H
#define PROCESSMP_H

/* Include the public header file for the widget.  */

#include "ProcessM.h"

/* Include the private implementation file.  */

#include "ProcessMI.h"

/* Include the private header file for the superclass.  */

#include <Xm/MessageBP.h>

/* Define the message processing widget instance part.  */

typedef struct _XeProcessMPart
{
   String     message_string;        /* Processing message working string.   */
   XeProcessMColor
              *color_struct;         /* Color specification structure.       */
   XeProcessMColor
              color_store;
   XmString   pause_mlabel;          /* Motif label strings.                 */
   XmString   resume_mlabel;
   Pixmap     symbol_pixmap;         /* Working pixmaps.                     */
   Pixmap     shape_mask;
   Pixmap     drawing_pixmap;
   Pixmap     label_pixmap;          /* Pixmap ID of the process message
                                        pixmap (in the label gadget).        */
   Boolean    pause_set;             /* Pause/Resume toggle flag.            */
   Boolean    init_flag;             /* Flag for proper initialization.      */
   Boolean    use_symbol;            /* Flag for using specified pixmap.     */
   Boolean    use_cancel;            /* Booleans for the cancel and pause    */
   Boolean    use_pause;             /* buttons.                             */
   Boolean    set_color;             /* Flag for assigning color(s) to the
                                        processing pie.                      */
   XeProcessMResponse
              user_response;         /* Variable for user response:
                                        XePROCESSM_NO_RESPONSE
                                        XePROCESSM_CANCEL                    */
   GC         gc;                    /* GC for process drawing.              */
   XeProcessMIconColor
              *input_color_list;     /* Input color specification for
                                        pixmaps.                             */
   Pixel      color_list[XePROCESSM_MAX_COLORS];
                                     /* Copy of the color specification for
                                        pixmaps.                             */
   Pixel      draw_colors[XePROCESSM_MAX_COLORS];
                                     /* Processing colors.                   */
   Widget     dialog_shell;          /* Internal widget IDs.                 */
   Widget     ok_button;
   Widget     cancel_button;
   Widget     help_button;
   Widget     symbol_label;
   Widget     separator;
   XeProcessMIterator
              percent;               /* Percent to be drawn.                 */
   XeProcessMIterator
              start;                 /* Starting iterator amount.            */
   XeProcessMIterator
              end;                   /* Ending iterator amount.              */
   XeProcessMIterator
              old;                   /* Old percent value used to limit
                                        message output.                      */
   XeProcessMIterator
              current_color;         /* Current drawing color.               */
   XeProcessMIterator
              num_colors;            /* Total # of drawing colors.           */
   XeProcessMIterator
              update_percent;        /* Percent increments for updates.      */
   short      new_angle;             /* Angles for processing message draw.  */
   short      old_angle;
   Visual     *visual;               /* Visual for the xpm routines.         */

} XeProcessMPart;

/* Define the full instance record.  */

typedef struct _XeProcessMRec
{
   CorePart              core;
   CompositePart         composite;
   ConstraintPart        constraint;
   XmManagerPart         manager;
   XmBulletinBoardPart   bulletin_board;
   XmMessageBoxPart      message_box;
   XeProcessMPart        processm;
} XeProcessMRec;

/* Define the message processing widget class part.  */

typedef struct _XeProcessMClassPart
{
   XtPointer extension;          /* Pointer to extension record.             */
} XeProcessMClassPart;

/* Define the full class record.  */

typedef struct _XeProcessMClassRec
{
   CoreClassPart                 core_class;
   CompositeClassPart            composite_class;
   ConstraintClassPart           constraint_class;
   XmManagerClassPart            manager_class;
   XmBulletinBoardClassPart      bulletin_board_class;
   XmMessageBoxClassPart         message_box_class;
   XeProcessMClassPart           processm_class;
} XeProcessMClassRec, *XeProcessMClass;

/* Define the external definition for the class record.  */

extern XeProcessMClassRec xeProcessMClassRec;

/* Define the private constants.  */

#define EVENT_MASK      (ButtonPressMask | ButtonReleaseMask | KeyPressMask | KeyReleaseMask)

#endif
