/****************************************************************************
NAME:				XRECTIFY

PURPOSE:  

PROGRAM HISTORY:
VERSION	 DATE	AUTHOR	   CODE/CONT	REASON
-------	 ----	------	   ---------	-----------------------------
  1.0	 01/90	D. Steinwand  CSB	REMAP Original development
  2.0	 03/92  D. Steinwand
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "xrectify.h"
#include "util.h"
#include "protos.h"

int main
(void)
{
struct PARBLK vblk;		/* TAE parameter block			*/
struct FRAME frame;		/* Framing & gridding parameters	*/
struct PROJ outp;		/* Output space projection parameters	*/

long	num = 1;		/* Number of output files		*/
long	nonfatal = 1;		/* Non-fatal error code			*/
long	fatal = -1;		/* Flag -- fatal error message		*/
long	msgtim;			/* Time between error messages		*/
long	std_out;		/* Dummy variable to standard output	*/
long	status;			/* Return status			*/
 
char	pname[CMLEN];		/* Printer file name			*/
char	siat[CMLEN];
char	taenam[2][CMLEN];	/* Tae name of output file for complete */

/* Initalize las
  -------------*/
c_inlas(&vblk,&std_out);

/* Get the parameters
  ------------------*/
if (getpar(&vblk,&frame,&outp,pname,siat,&msgtim) != E_SUCC)
    c_errmsg("Fatal error encountered", "xrectify-fatal", &fatal);

/* Initialize the model
  --------------------*/
init_mss_model(siat,0.0,0.0);

/* Make the output mapping grid
  ----------------------------*/
status = make_grid(&frame,&outp,&msgtim);
if (status == NO_RECTIFY)
    {
    c_errmsg("Unable to rectify to selected output frame","xrectify-nomap",
	      &nonfatal);
    c_errmsg("Fatal error encountered", "xrectify-fatal", &fatal);
    }
else if (status == E_FAIL)
   c_errmsg("Fatal error encountered","xrectify-fatal",&fatal);

/* Generate a tae name from the grid file and add it to the output files
   created list
  ------------*/
strcpy(&(taenam[0][0]), frame.name);

/* If a print file was created generate a tae name and add it to the output
   files created list
  ------------------*/
if (strcmp(pname, " ") != 0)
    {
    num++;
    strcpy(&(taenam[1][0]), pname);
    }

/* Display a standard completion message
  -------------------------------------*/
c_complt("XRECTIFY", taenam, &num);

return EXIT_SUCCESS;

}
