#include "zip.h"

double	angle
(
float   x0,               /* I: x coordinate of pivot point */
long    y0,               /* I: y coordinate of pivot point */
long	x1,               /* I: x and y coordinates of first vertex */ 
long    y1,
long    x2,	          /* I: x and y coordinates of second vertex */ 
long    y2          
)
/*
------------------------------------------------------------------
NAME: ANGLE

PROGRAM HISTORY:
      Version           date               Author
      -------           ----               ------   
         3            05-nov-88           B. Allam
         2            11-feb-87           R. Mckinney 
         1            15-mar-82           J. Marvin

FUNCTION:  finds rotation angle between two given vertices relative
           to given pivot point.

-------------------------------------------------------------------
  finds the rotation angle between the points (x1,y1) and     
  (x2,y2) relative to the central pivot point (x0,y0)         
                                                              
                                                              
		calculated by using the ratio of the cross     
		product (proportonal to sine(angle)) to the    
		dot product (proportional to cosine(angle))    
                                                              
		the sense of the rotation (+ or -) is          
		important.                                     
-------------------------------------------------------------------
*/
{
double  angle,       /* rotation angle in radians */
	cross,       /* cross product */
        dot;         /* dot product */
float   xx1, xx2,    /* difference of x coordinates */
        yy1, yy2;    /* difference of y coordinates */

xx1 = (float)x1-x0;
yy1 = (float)(y1-y0);
xx2 = (float)x2-x0;
yy2 = (float)(y2-y0);

cross = xx1*yy2 - xx2*yy1;
dot   = xx1*xx2 + yy1*yy2;

/*
---------------------------------------------------------------
		check for 90 degree angle:                     
---------------------------------------------------------------
*/
if ( abs(dot)  ==  0.0 )
   angle = 1.5707963;
else
  {
   angle = atan( abs(cross/dot) );
   if ( dot  <  0.0 ) angle = 3.1415926 - angle;
  }

if ( cross  <  0.0) angle = -angle;

return(angle);
}
