#include "zip.h"
#include "pixman.h"

void domap
(
    long ibands,                 /* I: total number of images           */
    long itype,                  /* I: data type                        */
    double cmean[],              /* I: class means                      */
    long sp,                     /* I: starting pixel                   */
    long np,                     /* I: end pixel                        */
    long inp,                    /* I: number of pixels                 */
    short lut[],                 /* I: look up table for map=3,         */
    float scale[],               /* I: scale ratios for pwl             */
    float minp[],         	 /* I: min and max pixel values         */
    float pairmin[],             /* I: scale ratios lower than specified*/
    float pairmax[],             /* I: scale ratios higher than         */
    double constval,             /* I: constant pixel value for map=2   */
    long map,                    /* I: option flag                      */
    long npairs,                 /* I: no. of from-to pairs             */
    float from[50],
    float to[50]        	 /* I: from-to pairs                    */
)

/*
---------------------------------------------------------------------
NAME: DOMAP

PROGRAM HISTORY:
      Version            date            Author
      -------            ----            ------
         3               nov -88        B. Allam
         2               feb -87        R. Mckinney
         1               june-84        Y. Sheu
 
FUNCTION:    DOMAP applies values to the line segments within sites 
             according to map option 

ALGORITHM:   
   
   If map = 1 fill the line segment with class means
   If map = 2 fill the line segment with constant pixel value
   If map = 3 apply mapping to pixels inside polygon

-------------------------------------------------------------------------
*/
{
long	dtype,             /* data type of pixels */
        n;                 /* counter            */
float	val;               /* replacement value  */

/*
fill the line segment with class means for map=1
------------------------------------------------
*/
dtype = EREAL;
if (map == 1)
{
 for (n=0; n < ibands; n++)   
   {
   val = cmean[n];
   c_pxfill((unsigned char *)&iobuf[opoint[n] + sp-1], &dtype, &inp,
		 (unsigned char *)&val); 
   } 
}

/*
fill line segment with constant for map=2 
-----------------------------------------
*/
if (map == 2)
{
 val= constval; 
 for (n=0; n < ibands; n++)
   {
   c_pxfill((unsigned char *)&iobuf[opoint[n] + sp-1], &dtype, &inp, 
		(unsigned char *)&val);
   }  
}

/*
apply piecewise linear mapping to pixels inside 
-----------------------------------------------
*/
if (map == 3)
 {
 if (itype  ==  1  ||  itype  ==  2)  
   {
   applylut (lut, itype, sp, inp, ibands);
   }
 else
   {
   pwlin (ibands, sp, np, scale, minp, pairmin, pairmax, npairs,
          from, to);
   }  
 }
}
