#include "zip.h"

void getpair
(
    float scale[50],            /* O: scaling ratios in different ranges */
    long ibands,                /* I: total no. of bands */
    float from[50],
    float to[50],               /* I: mapping pairs */
    float minp[],
    float maxp[],               /* I: min and max pixel values on input image */
    float pairmin[],            /* I/O: scaling ratios for pixels lower than */
    float pairmax[],            /* I/O: scaling ratios for pixels higher than */
    long npairs                 /* I: no. of from to pairs */
)

/*
------------------------------------------------------------------
NAME: GETPAIR

PROGRAM HISTORY:
      Version          date                Author
      -------          ----                ------
         3            nov-88              B. Allam
         2            feb-87              R. Mckinney
         1            jun-84              Y. Sheu

FUNCTION:   Computes the scale ratios from the input parameter
            arrays from+to for later use in subroutine pwlin

---------------------------------------------------------------------
*/
{
long	n, i;               /* counter variables */
float	frommin, frommax,   /* min and max from values */
        tomin, tomax,       /* min and max to values */
        range;              /* range of from values */

for (n=0; n < ibands; n++)
   {

/*
 check if from values go beyond pixel dynamic range
---------------------------------------------------
*/
   if (from[0]  <  minp[n]) 
      {
      zip_err("!FIRST from VALUE SMALLER THAN MINIMUM PIXEL VALUE", 
               "zip-from");
      }  
   if (from[npairs-1]  >  maxp[n]) 
      {
      zip_err("LAST from VALUE LARGER THAN MAXIMUM PIXEL VALUE", 
               "zip-from");
      }  
   if (from[0]  >  maxp[n]) 
      {
      zip_err("FIRST from VALUE LARGER THAN MAXIMUM PIXEL VALUE", 
               "zip-from");
      }  
   if (from[npairs-1]  <  minp[n]) 
      {
      zip_err("LAST from VALUE SMALLER THAN MINIMUM PIXEL VALUE", 
               "zip-from");
      }  


/*
 compute the scale outside of specified from + to ranges
--------------------------------------------------------
*/
   frommin = from[0]-minp[n];
   tomin = to[1]-minp[n];
   if (frommin  ==  0.0) 
      {
      pairmin[n]=0.0;
      }
   else 
      {
      pairmin[n]=tomin/frommin;
      } 

   frommax = maxp[n]-from[npairs-1];
   tomax = maxp[n]-to[npairs-1];
   if (frommax  ==  0.0) 
      pairmax[n]=0.0;
   else
      pairmax[n]=tomax/frommax;
   }  

/*
 set up scales in each range
----------------------------
*/
for (i=0; i < npairs-1; i++)
   {
   range=from[i+1]-from[i];
   if (range  !=  0.0) scale[i]=(to[i+1]-to[i])/range;
   }  

}
