#include "zip.h"
#include "util.h"

void getparms
(
    char hostat[CMLEN],			  /* O: stats file host name */
    char hosin[MAXIMG][CMLEN],		  /* O: input image host name */
    char hosout[CMLEN],			  /* O: output image host name */ 
    long (*wind)[4],                      /* O: window parameters */
    long *nib,                            /* O: no. of images */
    long *itype,                          /* O: data type of pixels */
    long *totbnd,                         /* O: total no. of input image bands*/
    char same[],                          /* O: option flag */
    long *map,                            /* O: mode flag */
    float from[50],
    float to[50],                         /* O: from-to arrays for map=3 */
    float *pixval,                        /* O: user specified constant */
    long *nc,                             /* O: no. of classes */
    long *npairs,                         /* O: no. of from, to pairs */
    char lasout[CMLEN]                    /* O: input/output file names */
)
/*
-----------------------------------------------------------------------
NAME: GETPARMS

PROGRAM HISTORY:
      Version          date                Author
      -------          ----                ------
         3            nov -88              B. Allam
         2            feb -87              R. Mckinney
         1            june-84              Y. Sheu
 
FUNCTION: Receives parameters from tae process definition file.
 
----------------------------------------------------------------------
*/
{
static long    access,		/* mode of access */
        flag[MAXIMG][2],	/* window or band option flag */
        imglimit = MAXIMG,	/* max no. of images */
	jj, ir, i,		/* counter variables */
        same_window,		/* flag for same window size */
        stat;			/* status returned */
long	std_out;                /* logical unit */
long    zero = 0;
char	lasin[MAXIMG][CMLEN];	/* input image las name */
char    minmax[4],          	/* min/max flag */
        runtyp[12];
char	map2[6];		/* option parameter */

struct VARIABLE *var, *vr, *vc, *vo, *vm, *vp, *vf, *vt;

/*
 initialize program
-------------------
*/
c_inlas(&block, &std_out);


/*
 get and validate input image parameters
----------------------------------------
*/
stat = c_getin (&block, "IN", &zero, lasin, hosin, nib, wind, bands, nbands,
                totbnd, &imglimit, &same_window, flag);
if (stat != E_SUCC)
   zip_err("!Error accessing the input file name", "zip-infile");

if (flag[0][0] == 1)
   zip_err("!No window may be specified", "zip-wind");

/*
get sfin parameters
-------------------
*/
access = IREAD;
stat = c_getnam(&block, "INSTAT", &zero, hostat, "stat", &access); 
if (stat  !=  E_SUCC)
  zip_err ("!Error accessing statistics file name", "zip-stat"); 


/*
----------------------------------------------------
 call get_ddr to access data type of pixels
----------------------------------------------------
*/
c_getddr (hosin[0], &ddr); 
*itype = ddr.dtype;

/*
get and validate output image parameters
----------------------------------------
*/
stat = c_getout(&block, "OUT", (char (*)[256])lasout, (char (*)[256])hosout);
if (stat != E_SUCC)
  zip_err("!Error getting output image name", "zip-out");

/*
   Get implicit global variable $runtype
   -------------------------------------
*/
vr = p_find (&block, "$RUNTYPE");
strcpy (runtyp, SVAL (*vr, 0));

/*
--------------------------------------------------------
 get number of user-specified classes and mapping option
--------------------------------------------------------
*/
vc = p_find(&block, "NCLASS"); 
if (!(*vc).v_count)
  *nc = 0;
else
  *nc = IVAL (*vc, 0);
                     
if ((*nc != 0) && (strncmp(runtyp, "BATCH", 5) == 0))
  {                
  zip_err ("!Cannot run in batch when NCLASS is not equal to zero",
          "zip-batch");
  }
       
/*                                                                      
-------------------------
 input values for mapping
-------------------------
*/
vo = p_find(&block, "REPLOPT");
strcpy(map2, SVAL(*vo, 0));
if (strcmp(map2,"MEAN") == 0)  *map=1;
if (strcmp(map2, "CONST") == 0) *map=2;
if (strcmp(map2, "MAP") == 0)  *map=3;

if (*map == 2)
  {
  vm = p_find(&block, "REPLMODE"); 
  strcpy(same, SVAL(*vm,0));
  if (!strcmp(same, "SAME")) 
    {
     vp = p_find(&block, "PIXVAL"); 
     *pixval = RVAL(*vp,0);
    }                                                          
  if ((strcmp(same, "DIFF") == 0) && (strncmp(runtyp, "BATCH", 5) == 0))
    zip_err ("!Cannot run in batch when REPLMODE = DIFF", "zip-batch");
  }
/*
-----------------------------------------------------
 retrieve and ensure complete and valid from/to pairs
-----------------------------------------------------
*/
if (*map == 3)
  {
   vf = p_find(&block, "FROM"); 
   vt = p_find(&block, "TO"); 
   if ((*vf).v_count != (*vt).v_count)
      zip_err ("!INCOMPLETE MAPPING PAIR SPECIFIED", "zip-pair"); 

   *npairs = (*vf).v_count;
   for (i = 0; i < (*vf).v_count; i++)
      from[i] = RVAL(*vf,i);
   for (i = 0; i < (*vt).v_count; i++)
      to[i] = RVAL(*vt, i);
   for (jj = 0; jj < *npairs; jj++)
     {
     ir = 0;
     range (from[jj], *itype, (long *)ir);
     if (ir != 0) 
       zip_err ("!PARAMETER VALUES DO NOT CONFORM TO INPUT DATA", "zip-range");
     range (to[jj], *itype, (long *)ir);
     if (ir != 0) 
       zip_err ("!PARAMETER VALUES DO NOT CONFORM TO INPUT DATA", "zip-range");
     } 
/*
---------------------------
 check order of from values
---------------------------
*/
  for (i = 1; i < *npairs; i++)
    {
    if (from[i]  <=  from[i-1])      /* out of order */
      zip_err ("!Invalid pairorder", "zip-pair");
    }  
  } 

/*
--------------------------------------------------------
 get the globals values
--------------------------------------------------------
*/
var = p_find(&block, "$MSGTIM"); 
msgtim = IVAL(*var,0);

var = p_find(&block, "$MINMAX"); 
strcpy(minmax, SVAL(*var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max= FALSE;

}
