#include "zip.h"

void indices
(
    long nc,                   /* I: number of classes specified by user */
    long ncproc,               /* I: number of classes to be analyzed */
    long icno[MAXCLS],         /* O: set of class indices */
    long *isno[MAXCLS],        /* I/O: set of site indices for each class */
    long nsites[MAXCLS],       /* I/O: no. of sites to be analyzed for a class*/
    char same[3],              /* I: flag for map=2 constant */
    float classconst[MAXCLS],  /* O: constant for each class for map=2 */
    double pixval,             /* I: user specified constant */
    long itype,                /* I: data type */
    long map                   /* I: option flag */
)
/*
-----------------------------------------------------------
NAME: INDICES

PROGRAM HISTORY:
      Version            Date             Author
      -------            ----             ------
         4              mar-89           K. Narayanan    was trying to free
                                                         unallocated space
         3              july-88          B. Allam
         2              feb-87	         R. Mckinney
         1              june-84          Y. Sheu
  
FUNCTION:    Receives user specified class and site names and
             retrieves their associated index numbers from statistics 
             file.

------------------------------------------------------------------------
*/
{
static char bname[100], outname[100], /* class name from stats file */
        numstr[3],           /* reply for all sites */
        stname[MAXCLS][100], /* class name */
        str[3],              /* reply for continue? */
        sname[99][100];      /* site name */
long	shape[9],            /* statistics data descriptor */
        scount,               /* no. of user errors entering site name */
        count = 0,            /* no. of user errors entering class name */
        ierr, stat, cerr, derr, serr,/* status of calls to statistics file */
        ic,                  /* class index number */
        is,                  /* site index number */
        ns;                  /* no. of sites per class */
long	iret,               /* flag for valid range */
        valid, cont, found, resp, name, /* loop control */
        done, numsite, given; /* variables */
long    m, k, nchk, i, j;     /* counter variables */
long	zero = 0;

/*
loop through classes
---------------------
*/
for (i=0; i < ncproc; i++)
  {
   count=0;
   if (nc != 0) 
     {
      found = 0;
/*
not all classes being used => prompt user for name and
check against stats file
-------------------------------------------------------
*/
      while (!found)
        {
         printf("\n$_Enter Class name : ");    
         scanf("%s", stname[i]);
         ierr = c_stfdcl ((FILE **)&fd_sfin, stname[i], &ic); 
         if (ierr == 2)
           zip_err("!general I/O error", "zip-i/o");
/*
---------------------------------------------
 class name not found.
 user is given 3 chances to enter valid name.
---------------------------------------------

*/
         if (ierr == 1) 
           {
            count=count+1;
            if (count == 3) 
              zip_err("!3 ERRORS FOR CLASS NAME.  RESTART.", "zip-cname");
            else
             {
              zip_err("?Class not found.  3 chances to reenter a class name.",
                       "zip-cname");
              continue;
             } 
           } 
         else
           found = 1;
/*
--------------------------------------------------------------------
 write class index # into list of class indices if not already given
--------------------------------------------------------------------
*/
         given = FALSE;
         for (nchk = 0; nchk <= i; nchk++)
           if (icno[nchk] == ic) given = TRUE;
         if (given)  
           {
            zip_err("?Class name already specified.", "zip-cname");
            continue;
           } 
         found = 1;
        } 

      icno[i]=ic;
/*
-------------------------------------
 get constant for map=2 in this class
-------------------------------------
*/
      if (map  ==  2 ) 
        {
         if (same[0]  ==  'S') 
            classconst[i] = pixval;
         else 
            {
            valid = 0;
            while (!valid)
              {
               printf("\n$_Enter constant value for Class %s:", stname[i]); 
               scanf("%f", &classconst[i]);
               iret = 0;
               range (classconst[i], itype, &iret);
               if (iret != 0) continue;
               valid = 1;
              } 
            }  
         }  
/*
--------------------------------------------------
 prompt user for # of sites and read from terminal
--------------------------------------------------
*/
       numsite = 0;
       while (!numsite)
         {
          printf(" Enter the number of Sites to process for Class %s\n", stname[i]); /*--- (20) ---*/
          printf(" :-");
          scanf("%d", &nsites[i]);
          if (nsites[i] < 0) 
            {
             printf("$_***** INVALID RESPONSE *****\n");    
             continue;
            }  
/*
-----------------------------------------
 check requested # of sites vs. # in file
-----------------------------------------
*/
         stat = c_stget((FILE **)&fd_sfin, "NSITES", &ic, &zero, shape, 
			(unsigned char *)&ns);
         if (stat != E_SUCC)
           zip_err ("!Error getting no. of sites", "zip-nsite");
         if (ns == 0) 
           {
            zip_err ("?No sites present for class.", "zip-nosite");
            printf("$_Continue? (YES/NO):");
            scanf("%s", str);
            if (str[0] == 'N' || str[0] == 'n')
              zip_err ("!ZIP HAS ABORTED AT YOUR REQUEST", "zip-abort");
           }  
         if (nsites[i] == 0) 
           nsites[i] = ns;
         else if (nsites[i] > ns) 
           {
            done = 0;
            while (!done)
              {
    /*25 */    printf("$_Class: %s does not have %d Sites\n", stname[i], nsites[i]);
               printf("$_Do you wish to use all available Sites? (YES/NO):");      /*--- (320) ---*/
               resp = 0;
               while (!resp)
                 {
                  scanf("%s", numstr);      
                  if (numstr[0] == 'y' || numstr[0] == 'Y') 
                    nsites[i] = ns;
                  else if (numstr[0] == 'n' || numstr[0] == 'N')
                    {
                     done = 1; 
                     break;
                    }
                  else
                    {
                     printf("$_***** INVALID RESPONSE *****\n");    
                     printf("$_Respond with YES or NO :");
                     continue;
                    }  
                  resp = 1;
                 } 
               done = 1;
              } 
               if (done) {
                 numsite = 1;
                 continue;
               }
           }  
         numsite = 1; 
        } 
/*
---------------------------------------------------------------- 
 if not all sites are being used then prompt user for site names
---------------------------------------------------------------- 
*/
      if (nsites[i] != ns) 
        {
         printf ("$_Enter Site names , 1 per line : ");     
         for (j=0; j < nsites[i]; j++)
           {
            scount=0;
            name = 0;
            while (!name)
              {
               scanf("%s", sname[j]);     
               cerr = c_stfdsi ((FILE **)&fd_sfin,stname[i],sname[j],&ic,&is); 
/*
------------------------------------------
 site name not found.
 user given 3 chances to enter valid name.
------------------------------------------
*/
               if (cerr == 3) 
                 {
                  scount=scount+1;
                  if (scount == 3) 
                     zip_err ("!3 ERRORS FOR SITE NAME. RESTART", "zip-sname");
                  else                                  
                    {
                     zip_err ("?Site not found. 3 chances to reenter a site",
                              "zip-sname");	
                     printf ("\n$_Enter Site name again : ");     
                     continue;
                    }  
                 } 
               name = 1;
              } 
/*
---------------------------------------------
 write site index # into list of site indices
---------------------------------------------
*/
            isno[i][j] = is;
           }  
/*
----------------------------------------------------------------
 all sites are being used. site indices incremented sequentially
----------------------------------------------------------------
*/
        }
      else
        {
         for (j=0; j < ns; j++)
           isno[i][j] = j+1;
        }  
     } 
/*
------------------------------------------------
 all classes and all sites requested.
 class and site indices incremented sequentially
------------------------------------------------
*/
   else
     {
      icno[i] = i+1;
/*
-----------------------------
 prompt for constant if map=2
-----------------------------
*/
      if (map  ==  2 ) 
        {
         if (same[0]  ==  'S') 
           classconst[i] = pixval;
         else
           {
            m = i + 1;
            derr = c_stget ((FILE **)&fd_sfin, "NAME", &m, &zero, shape, 
			(unsigned char *)bname);
            if (derr  !=  E_SUCC)
            zip_err("!STAT I/O ERROR FOR CLASS.  CHECK STATS FILE","zip-class"); 
            cont = 0;
            strcpy (outname, bname);
            while (!cont)
              {
               printf ("\n$_Enter constant value for Class %s:", outname); 
               scanf("%f", &classconst[i]);
               iret = 0;
               range (classconst[i], itype, &iret);
               if (iret != 0) continue;
               cont = 1;
              } 
           }  
        }  
      k = i + 1;
      serr = c_stget ((FILE **)&fd_sfin,"NSITES",&k,&zero,shape,
			(unsigned char *)&nsites[i]); 
      if (serr != E_SUCC) 
         zip_err ("!STAT I/O ERROR FOR SITES. CHECK STATS FILE", "zip-site");
      for (j=0; j < nsites[i]; j++)
        {
         isno[i][j] = j+1;
        }  
     } 
  } 

}
