#include "zip.h"
#include "util.h"
#include "pixman.h"

void mapping
(
    long ibands,                /* I: no. of bands */
    long window[][4],           /* I: window parameters */
    float minp[],               /* I: min and max pixel values */
    long *isno[MAXCLS],         /* I: set of site indices */
    long nsites[],              /* I: number of sites for a class */
    long nverts[1023],          /* I: no. of site vertices */
    long ncx,                   /* I: no. of classes to be analyzed */
    long area[1024][4],         /* I: site bounding rectangles for site */
    long itype,                 /* I: data type of input images */
    double *cmean[MAXCLS],      /* I: class means */
    long lmin,                  /* I: min line no. bounding all sites */
    long lmax,                  /* I: max line no. bounding all sites */
    short lut[],                /* I: look-up table for pwlin when itype=1,2 */
    float scale[],              /* I: scale ratios for pwlin when itype=3,4 */
    float pairmin[],            /* I: scale ratios for pixels lower 
                                   than specified range */
    float pairmax[],            /* I: scale ratios for pixels higher 
                                   than specified range */
    float classconst[], 	/* I: constant values for each class */
    long map,                   /* I: option flag */
    long npairs,		/* I: no. of from-to pairs */
    float from[50],
    float to[50]                /* I: from-to pairs */
)

/*
--------------------------------------------------------------------
NAME: MAPPING

PROGRAM HISTORY:
      Version             date               Author
      -------             ----               ------
         3               nov-88             B. Allam
         2               feb-87             R. Mckinney
         1               aug-84             Y. Sheu
 
FUNCTION: applying mapping to the pixels inside the polygons

----------------------------------------------------------------------
*/
{
long    flag; 			/* formatting flag */
long	dtype, 			/* output image data type */
        nlin, npix, 		/* no. of lines and samples */
        ln, n, j, i, ii,	/* counter variables */
        icount;               	/* no. of polygons */
long    is,                     /* site index number */
        np, nl, 		/* no. of pixels and lines */
        nls, 			/* no. of lines */
        line, 			/* line no. */
        nxs, 			/* no. of intersections */
        lastpix, inp;		/* no. of pixels inside polygon */
static long	sxs[200]; 		/* coords of intersections */
long	sp, sl,                 /* starting line and pixel */
        begpix, endpix; 	/* beginning and ending pixel */


nlin = window[0][NL];
npix = window[0][NS];

/* 
initialize and enable processing message utility
------------------------------------------------
*/
flag = ALLCAT;
curent = 1;
c_inittm("ZIP", "Processed", "line", "bands", " ", &nlin, &ibands,
         &msgtim, &flag);
c_startm();

/*
process a line at a time
------------------------
*/
for (ln = 1; ln <= nlin; ln++)
  {
/*
------------------
 copy current line 
------------------
*/
   c_estep((struct GDESC **)&gdesc);
   dtype = EREAL;
   for (n = 0; n < ibands; n++)
     {
      c_pxcopy((unsigned char *)&iobuf[ipoint[n]], 
		(unsigned char *)&iobuf[opoint[n]], &dtype, &npix); 
     }

   if (ln >= lmin && ln <= lmax) 
     {
      icount=0;
  
      for (i=0; i < ncx; i++)
        {
         for (j=0; j < nsites[i]; j++)
           {
            is = isno[i][j];
            if (is  !=  0)  
              {
/*
--------------------------------
 find bounding rectangle of site.
--------------------------------
*/
               sl = area[icount][0];
               sp = area[icount][1];
               nl = area[icount][2];
               np = area[icount][3];
/*
-------------------------------------------
 skip this site if line is not in the range
-------------------------------------------
*/
               nls = sl + nl - 1;
               if (ln >= sl && ln <= nls)  
                 {
/*
------------------------------------------------------------
 for the line in site--find the number of intersections with
 polygon boundary
------------------------------------------------------------
*/
                 line=ln;
                 sect (xaxis[icount], yaxis[icount], nverts[icount], line, 
                       sxs, &nxs);
/*
------------------------------------------------------------
 find interior points in polygon. set value for these points
------------------------------------------------------------
*/
                 for (ii = 1; ii < nxs; ii++)
                   {
                    if (insde(xaxis[icount], yaxis[icount], nverts[icount], 
                        sxs[ii-1], sxs[ii], line)) 
                      {
/*
---------------------------------------------------------
 fill interior points with value according to map option
---------------------------------------------------------
*/
                       begpix = max(sxs[ii-1], sp);
                       lastpix = sp + np - 1;
                       endpix = min(sxs[ii], lastpix);
                       inp = endpix - begpix + 1;
                       domap(ibands, itype, cmean[i], begpix, endpix, inp, lut, 
                             scale, minp, pairmin, pairmax, 
                             classconst[i], map, npairs, from, to);
                      }  
                   }  
                 } 
               icount=icount+1;
              } 
           }  /* end of site loop */      
        }  /* end of class loop */
     }
   curent++;
  }  /* end of line loop */

c_estep((struct GDESC **)&gdesc);

c_stoptm();

}
