#ifndef PROTOS_H
#define PROTOS_H

double  angle
(
float   x0,               /* x coordinate of pivot point */
long    y0,               /* y coordinate of pivot point */
long    x1,               /* x and y coordinates of first vertex */
long    y1,
long    x2,               /* x and y coordinates of second vertex */ 
long    y2           
);


void applylut
(
    short lut[],          /* I: look up table */
    long type,            /* I: indicates data type of image  */
    long sp,              /* I: starting pixel in the image line in site */
    long np,              /* I: number of pixels in each image line in site */
    long ibands           /* I: total no. of bands */
);

void classtat
(
    long ibands,                /* I: total no. of bands */
    double smean[],             /* I: site mean */
    long npts,                  /* I: no. of points in site */
    double cmean[],             /* I/O: class mean */
    long *cpts                  /* I: current # of points for class */
);

void domap
(
    long ibands,                 /* I: total number of images           */
    long itype,                  /* I: data type                        */
    double cmean[],              /* I: class means                      */
    long sp,                     /* I: starting pixel                   */
    long np,                     /* I: end pixel                        */
    long inp,                    /* I: number of pixels                 */
    short lut[],                 /* I: look up table for map=3,         */
    float scale[],               /* I: scale ratios for pwl             */
    float minp[],         	 /* I: min and max pixel values         */
    float pairmin[],             /* I: scale ratios lower than specified*/
    float pairmax[],             /* I: scale ratios higher than         */
    double constval,             /* I: constant pixel value for map=2   */
    long map,                    /* I: option flag                      */
    long npairs,                 /* I: no. of from-to pairs             */
    float from[50],
    float to[50]        	 /* I: from-to pairs                    */
);

void dostats
(
    unsigned char *mask,          /* I/O: indicates which pixels ar inside */
    long np,                      /* I: no. of pixels in line for a site */
    long iband,                   /* I: no. of bands of data */
    double smean[MAXBND],         /* O: current mean */
    double sum[MAXBND],           /* O: current sum of pixel values */
    long *k,                      /* O: current no. of points */
    float *buf[]                  /* I: buffer containing pixels */
);

void dyn_alloc
(
    long totbnd,          	/* I: total no. of bands */
    long maxcls,          	/* I: total no. of classes */
    double *cmean[MAXCLS],  	/* O: class means */
    long *isno[MAXCLS],   	/* O: site indices */
    short **lut,		/* O: look up table for mapping */
    float **minp,
    float **maxp,     		/* O: min and max pixel values */
    float **pairmin,            /* O: scaling ratios for pixels */
    long window[][4]		/* I: window parameters */
);

void getpair
(
    float scale[50],            /* O: scaling ratios in different ranges */
    long ibands,                /* I: total no. of bands */
    float from[50],
    float to[50],               /* I: mapping pairs */
    float minp[],
    float maxp[],               /* I: min and max pixel values on input image */
    float pairmin[],            /* I/O: scaling ratios for pixels lower than */
    float pairmax[],            /* I/O: scaling ratios for pixels higher than */
    long npairs                 /* I: no. of from to pairs */
);

void getparms
(
    char hostat[CMLEN],			  /* O: stats file host name */
    char hosin[MAXIMG][CMLEN],		  /* O: input image host name */
    char hosout[CMLEN],			  /* O: output image host name */ 
    long (*wind)[4],                      /* O: window parameters */
    long *nib,                            /* O: no. of images */
    long *itype,                          /* O: data type of pixels */
    long *totbnd,                         /* O: total no. of input image bands*/
    char same[],                          /* O: option flag */
    long *map,                            /* O: mode flag */
    float from[50],
    float to[50],                         /* O: from-to arrays for map=3 */
    float *pixval,                        /* O: user specified constant */
    long *nc,                             /* O: no. of classes */
    long *npairs,                         /* O: no. of from, to pairs */
    char lasout[CMLEN]                    /* O: input/output file names */
);

void getpoly
(
    long ic,                    /* I: class index number */
    long is,                    /* I: site index number */
    long xaxis[],
    long yaxis[], 		/* O: x and y co-ords of site vertices */
    long *nverts,               /* O: number of vertices */
    long xoff,
    long yoff                   /* I: x and y offsets */
);

void indices
(
    long nc,                   /* I: number of classes specified by user */
    long ncproc,               /* I: number of classes to be analyzed */
    long icno[MAXCLS],         /* O: set of class indices */
    long *isno[MAXCLS],        /* I/O: set of site indices for each class */
    long nsites[MAXCLS],       /* I/O: no. of sites to be analyzed for a class*/
    char same[3],              /* I: flag for map=2 constant */
    float classconst[MAXCLS],  /* O: constant for each class for map=2 */
    double pixval,             /* I: user specified constant */
    long itype,                /* I: data type */
    long map                   /* I: option flag */
);

long    insde
(
   long    xverts[],
   long    yverts[],                   /* x and y coordinates of vertices */
   long    nverts,                     /* number of vertices */
   long    xpos1,                      /* x-coord of first intersection pt */
   long    xpos2,                      /* x-coord of second intersection pt */
   long    ypos                        /* y-coord of test point */
);

void makelut
(
    short lut[],               /* I: look up table data */
    float from[50],
    float to[50],              /* I: mapping pairs */
    long npairs,               /* I/O: number of pairs in partial lut */
    long type                  /* I: indicates data type of lut */
);

void mapping
(
    long ibands,                /* I: no. of bands */
    long window[][4],           /* I: window parameters */
    float minp[],               /* I: min and max pixel values */
    long *isno[MAXCLS],         /* I: set of site indices */
    long nsites[],              /* I: number of sites for a class */
    long nverts[1023],          /* I: no. of site vertices */
    long ncx,                   /* I: no. of classes to be analyzed */
    long area[1024][4],         /* I: site bounding rectangles for site */
    long itype,                 /* I: data type of input images */
    double *cmean[MAXCLS],      /* I: class means */
    long lmin,                  /* I: min line no. bounding all sites */
    long lmax,                  /* I: max line no. bounding all sites */
    short lut[],                /* I: look-up table for pwlin when itype=1,2 */
    float scale[],              /* I: scale ratios for pwlin when itype=3,4 */
    float pairmin[],            /* I: scale ratios for pixels lower 
                                   than specified range */
    float pairmax[],            /* I: scale ratios for pixels higher 
                                   than specified range */
    float classconst[], 	/* I: constant values for each class */
    long map,                   /* I: option flag */
    long npairs,		/* I: no. of from-to pairs */
    float from[50],
    float to[50]                /* I: from-to pairs */
);

void pwlin
(
    long ibands,           /* I: no. of images */
    long sp,               /* I: starting pixel */
    long np,               /* I: ending pixel */
    float scale[],         /* I: ratios of from ranges and to ranges */
    float minp[],          /* I: min and max pixel values */
    float pairmin[],       /* I: min value of mapping array */
    float pairmax[],       /* I: max value of mapping array */
    long npairs,           /* I: no. of mapping pairs */
    float from[],
    float to[]             /* I: mapping pairs */
);

void range
(
    double cval,          /* I: value to be checked */
    long itype,           /* I: data type of input image */
    long *abo             /* I/O: constant type */
);

void sect
(
    long xverts[],
    long yverts[],        	/* I/O: x and y vertices */
    long nverts,                /* I: no. of vertices */
    long line,                  /* I: line no. to be checked */
    long sects[],               /* O: x coords of intersections */
    long *nsect                 /* O: no. of intersections */
);

void setarea
(
    long ncx,                   /* I: no. of classes to be analyzed */
    long icno[],                /* I: set of class indices */
    long *isno[MAXCLS],         /* I/O: set of site indices */
    long nsites[],              /* I: no. of sites for a class */
    long area[1024][4],         /* O: array containing site bounding rectangle*/
    long nverts[],              /* O: no. of vertices */
    long *lmin,                 /* O: min line no. for all site boundings */
    long *lmax,                 /* O: max line no. for all site boundings */
    long xoff,
    long yoff,                  /* I: x and y offsets */
    long window[][4]            /* I: window parameters */
);

void setfgin
(
    char hosin[][CMLEN],          /*I: input image host name descriptors */
    long nimages,                 /*I: no. of input images           */
    long wind[][4]                /*I: input image windows           */
);

void setfsg
(
    char hosin[][CMLEN],          /* I: input image host name descriptors */
    char hosout[],                /* I: output image host name descriptor */
    long nimages,                 /* I: no. of input images           */
    long wind[][4],               /* I: input image windows           */
    long totbnd,                  /* I: total no. of bands           */
    long itype                    /* I: data type of pixels           */
);

void statchk
(
    char hostat[],		/* I: stats file host name */
    long nclass, 		/* I: no. of classes */
    long *ncproc		/* O: no. of classes processed */
);

void stats
(
    long ibands,                     /* I: total no. of bands */
    long icno[],                     /* I: set of class indices */
    long *isno[MAXCLS],              /* I: set of site indices */
    long nsites[],                   /* I: no. of sites for a class */
    long ncx,                        /* I: no. of classes to be analyzed */
    long area[1024][4],              /* I: site bounding rectangles */
    long nimages,                    /* I: no. of images */
    long nverts[],                   /* I: number of vertices */
    double *cmean[MAXCLS],           /* I/O: class mean */
    long xoff,
    long yoff,                       /* I: x and y offsets */
    long window[][4]	             /* I: window parameters */
);

void zip_err
(
    char *message, 		
    char *key		           /* I: Error message and severity */
);

void verts
(
    long size[4],                  /* O: site bounding rectangle   */
    long xaxis[],
    long yaxis[],  		   /* I: x and y image coordinates */
    long nverts                    /* I: no. of vertices           */
);
#endif
