#include "zip.h"

void setfsg
(
    char hosin[][CMLEN],          /* I: input image host name descriptors */
    char hosout[],                /* I: output image host name descriptor */
    long nimages,                 /* I: no. of input images           */
    long wind[][4],               /* I: input image windows           */
    long totbnd,                  /* I: total no. of bands           */
    long itype                    /* I: data type of pixels           */
)
{
/*
------------------------------------------------------------------------

NAME: SETFSG

FUNCTION: SETFSG is called by ZIP to initialize the input file
          action group to set up the next output file.

PROGRAM HISTORY:
    Version             date            Author
    -------             ----            ------
       4              15-apr-88        b.allam/sar  scn#20-140
       3              28-feb-84        s.ross/sar   scn#20-181
       1              12-dec-83        s.ross/sasc  scn#20-105

ALGORITHM:

    Call C_EOPENS to open input and output images
    Call C_EGROUP to group the image

------------------------------------------------------------------------
*/

   long    stat,                  /* flag for EOPENS procedure */
           access,                /* file specifications       */
           type,                  /* data type                 */
           n, nib,                /* counter variables         */
           opt[2],                /* optional parameter array  */
           bufsiz;                /* image i/o buffer size     */
long	   zero = 0;
double     y_inc, x_inc;          /* line and sample increments*/
long    one = 1;

/*
 set up input file group
 -----------------------
*/

   access = IREAD;
   type = EREAL;
   for (n = 0, nib = 0; n < nimages ; n++)
   {
     stat = c_eopens((struct FDESC **)&fdesc[n],hosin[n],bands[n],&wind[n][SL],
                     &wind[n][SS], &wind[n][NL], &wind[n][NS], &ipoint[nib],
                     &access, &type, &zero, &zero);
     if (stat != E_SUCC)
       zip_err("!Error opening input image", "zip-input");
     nib += nbands[n]; 
    }
/*
 set up output file 
 ------------------
*/

   access = IWRITE;
   opt[0] = itype;
   opt[1] = 0;
   stat = c_eopens ((struct FDESC **)&fdesc[n],hosout,&totbnd,&one,&one,
		&wind[0][NL],&wind[0][NS],opoint,&access,&type,&zero,opt);
 
   if (stat != E_SUCC)
     zip_err("!Error opening output image","zip-output");
   fdesc[++n] = 0;

/*
update DDR
----------
*/
y_inc = 1.0;
x_inc = 1.0; 
c_upddr(&block, hosin, hosout, &nimages, wind, bands, nbands, 
        &y_inc, &x_inc, &totbnd, &zero);
 
for (n = 0; n < totbnd; n++)   out_img_bands[0][n] = n + 1;

/*
set up group containing input and output bands
----------------------------------------------
*/
   if ((iobuf = (float *) c_ealloc((struct FDESC **)fdesc, &bufsiz)) == NULL)
     zip_err("!Error allocating dynamic memory", "zip-alloc");

   if (c_egroup((struct FDESC **)fdesc,(struct GDESC **) &gdesc,
		(unsigned char *)iobuf,&bufsiz) != E_SUCC)
     zip_err("!Error grouping image bands", "zip-group");
 }
