#include "zip.h"
#include "util.h"
#include "pixman.h"

void stats
(
    long ibands,                     /* I: total no. of bands */
    long icno[],                     /* I: set of class indices */
    long *isno[MAXCLS],              /* I: set of site indices */
    long nsites[],                   /* I: no. of sites for a class */
    long ncx,                        /* I: no. of classes to be analyzed */
    long area[1024][4],              /* I: site bounding rectangles */
    long nimages,                    /* I: no. of images */
    long nverts[],                   /* I: number of vertices */
    double *cmean[MAXCLS],           /* I/O: class mean */
    long xoff,
    long yoff,                       /* I: x and y offsets */
    long window[][4]	             /* I: window parameters */
)
/*
-------------------------------------------------------------------
NAME: STATS

PROGRAM HISTORY:
      Version            date            Author
      -------            ----            ------
         3                nov -88       B. Allam
         2                feb -87       R. Mckinney
         1                june-84       Y. Sheu
 
FUNCTION: calculates requested statistics for sites 

---------------------------------------------------------------------------
*/

{
static double	smean[MAXBND],          /* site mean vector */
        sum[MAXBND];		/* sum of pixels in each band */
long	sp, sl; 		/* starting line and pixel */
static long	isarea[1024][4], 	/* area of polygon */
        vshape[9];		/* shape vector */
long	cpts;			/* no. of pts inside polygon */
static long	sxs[200]; 		/* coordinates of intersections */
char	obuf[100], cname[100];	/* class names */
static float   *buf[MAXBND], *tbuf[MAXBND]; /* temporary buffers */
float   *b_matrix, *t_matrix;	/* buffer matrices */
long	dtype; 			/* data type of input image */
char	bname[80];
unsigned char  *mask;           /* indicates which pixels are 
                                    inside site polygon */
long	pk, kk, i, ic, j, is, n, icount, ib, k, jj, np, nl, lnk;
long	ch, lmx, nxs, ii, ist, ien, jfill,
        img, ind,               /* counter variable */
        nim,                    /* total no. of buffers */
        stat,                   /* status returned */
        bufsiz, nb;
long	zero = 0;
long    one = 1;


/* Allocate memory 
------------------
*/
mask = (unsigned char *) malloc (window[0][NS] * sizeof(unsigned char));
if (mask == NULL)
    zip_err ("!Error allocating storage", "zip-alloc");
   

/*
 reset area values to input image coordinates
---------------------------------------------
*/
kk=0;
for (i=0; i < ncx; i++)
   {
   ic=icno[i];   
   for (j=0; j < nsites[i]; j++)
      {
      is=isno[i][j];
      if (is  !=  0)  
        {
         isarea[kk][0] = area[kk][0] + yoff-1;
         isarea[kk][1] = area[kk][1] + xoff-1;
         isarea[kk][2] = area[kk][2];
         isarea[kk][3] = area[kk][3];
         kk=kk+1;
         }  
      }  
   }  

/*
Allocate temporary storage
--------------------------
*/
      bufsiz = (long)(ibands * window[0][NS] * sizeof(float));
      if ((b_matrix = (float *) malloc(bufsiz)) == NULL ||
         (t_matrix = (float *) malloc(bufsiz)) == NULL)
         zip_err ("!Error allocating storage", "zip-alloc");
      for (ind=0,n=0; ind < ibands; ind++, n += window[0][NS]) {
         buf[ind] = &b_matrix[n];
         tbuf[ind] = &t_matrix[n];
      }
/*
do stats 
--------
*/
icount= -1;
for (i=0; i < ncx; i++)
   {
   ic = icno[i];
   cpts=0;
   for (ib=0; ib < ibands; ib++)
      cmean[i][ib] = 0.;

   for (j=0; j < nsites[i]; j++)
      {
      is = isno[i][j];
      if (is  !=  0) 
        {
         icount=icount+1;
         k=0;
         for (jj = 0; jj < ibands; jj++)
            {
             smean[jj] = 0.;
             sum[jj] = 0.;
            }  


/*
find bounding rectangle of site
--------------------------------
*/
      sl = isarea[icount][0];
      sp = isarea[icount][1];
      nl = isarea[icount][2];
      np = isarea[icount][3];

/*
for each line in site--find the number of intersections with
polygon boundary.
------------------------------------------------------------
*/
      dtype = EBYTE;
      for (lnk = 1; lnk <= nl; lnk++)
         {   /*** initialize buffer ***/ 
         c_pxfill(mask, &dtype, &np, (unsigned char *)&zero); 
         lmx = sl + lnk - yoff;
         nim = 0;
         for (img=0; img < nimages; img++)
         {
         for (nb=0; nb < nbands[img]; nb++)
           {
           c_eread((struct FDESC **)&fdesc[img], &bands[img][nb], &lmx,
			(unsigned char *)buf[nim], &one);
           for (pk = 0; pk < np; pk++)
              tbuf[nim][pk] = buf[nim][sp+pk-1];
           nim += 1;
           }
         }
         sect (xaxis[icount], yaxis[icount], nverts[icount],
               lmx, sxs, &nxs);

/*
find interior points in polygon. set mask = 1 for these points
--------------------------------------------------------------
*/
         for (ii = 1; ii < nxs; ii++)
           {
            if (insde(xaxis[icount], yaxis[icount], nverts[icount], 
                      sxs[ii-1], sxs[ii], lmx)) 
               {
               ist = sxs[ii-1] - sp + xoff;
               ien = sxs[ii] - sp + xoff;
               if (ist <= np && ien >= 1)
                 {
                  ist=max(ist, 1);
                  ien=min(ien, np);
                  ist = ist - 1;
                  for (jfill = ist; jfill < ien; jfill++)
                     mask[jfill] = (unsigned char )one;
                 } 
               }  
            }  

/*
do statistics for all interior points
-------------------------------------
*/
         dostats (mask, np, ibands, smean, sum, &k, tbuf);
         }  
      classtat(ibands, smean, k, cmean[i], &cpts);
      }  
   }  

/*
tell user the class means 
-------------------------
*/
   if (cpts  >  0) 
      {
      for (ch = 0; ch < 80; ch++)
        bname[ch] = ' ';
      stat = c_stget((FILE **)&fd_sfin, "NAME", &ic, &zero, vshape,
		(unsigned char *)bname);
      if (stat != E_SUCC)
        zip_err ("!Error reading class names", "zip-cname");
      for (ch = 0; ch < strlen(bname); ch++)
        if (bname[ch] == ' ')
          bname[ch] = '\0';
      sprintf(cname, " MEAN VALUES FOR THIS CLASS %s:", bname);
      c_pterm(cname, &zero);
      for (nb = 0; nb < ibands; nb++)
        {
        sprintf(obuf, "BAND %3d:     %f\n",nb+1,cmean[i][nb]);
        c_pterm(obuf, &zero);                      
        }
      }  
   }  
free(b_matrix); free(t_matrix);
free(mask);

/* close the opened image 
------------------------------*/
   for (i=0; i < nimages; i++)   c_eclose((struct FDESC **)&fdesc[i]);             
}
