/* include files, global parameters, and definitions for ZIP */

#include <stdio.h>
#include <math.h>
#include "las.h"
#include "typlim.h"
#include "statio.h"
#include "protos.h"
#include "util.h"

#define MAX_POLYGON 1023

struct PARBLK block;                   /* TAE parameter block     */
struct DDR  ddr;

FILE	*fd;                           /* output file descriptor  */

long    *isno_matrix;                  /* site index matrix */
double  *cmean_matrix;                 /* class mean matrix */
long    *x_matrix, *y_matrix;          /* polygon matrices */
long    fd_sfin;
long	fdesc[MAXIMG];                 /* file descriptors        */
long	gdesc;                         /* group descriptor        */
long	*ipoint,  		       /* offsets into image i/o  */
	opoint[MAXBND+1];              /* for input and output bands */
long	min_max;                       /* minmax option flag      */
long 	msgtim;                        /* message interval        */
 
long    count;                         /* no. of files opened */
long    nbands[MAXIMG];

char    out[MAXIMG];                   /* names of files opened */
float	*iobuf;                        /* image i/o buffer        */

extern	long	curcat;                /* message utility         */
extern	long	curent;                /* parameters for processing */

short	*temp1,     		       /* input array for look-up table */
	*temp2;       		       /* output array of look-up table */

long    out_img_bands[MAXIMG][MAXBND+1];
long    bands[MAXIMG][MAXBND+1];
long    *xaxis[MAX_POLYGON];
long    *yaxis[MAX_POLYGON];

void main
(void) /* zip */
{
/*
----------------------------------------------------------------------
NAME: ZIP

PROGRAM HISTORY:
      Version          date                Author
      -------          ----                ------
         5            Nov-90               W. Xia (STX) add min/max option 
         4            mar-89               K. Narayanan  changed indices.c
         3            july-88              B. Allam
         2            feb -87              R. Mckinney
         1            june-84              Y. Sheu
	 6            Feb-91               D. Etrheim (EDC) Allocate ipoint
					   	      dynamicly
         7            Sep-92               D. Etrheim (EDC) Modify global
       						include file
         8            Dec-99		   D. Hames Remove reference to MAXNS
					        in dostats, dyn_alloc and stats
						and replace with number of lines
						and samples in the image. 
					 	Modified to support images >2gig
					 	Add prototypes. SRF #893
ALGORITHM:
----------------------------------------------------------------------
*/
long    totbnd;                         /* total no. of input image bands   */
long	window[MAXIMG][4],              /* window parameters */
	xoff, yoff,                     /* x and y offsets */
        map,                            /* option flag */
        nimages,                        /* no. of images */
        area[1024][4],                  /* array containing site bounding */
                                        /* rectangle */
        nc;                             /* no. of classes */
char	same[5];                        /* mode flag */
float	from[50], to[50],               /* mapping arrays */
        pixval,                         /* user specified constant */
        scale[50];                      /* scaling ratios in different ranges */
static long	*isno[MAXCLS],          /* site index numbers */
        nsites[MAXCLS],                 /* number of sites */
        icno[MAXCLS];                   /* class index numbers */
static long    nverts[MAX_POLYGON];     /* no. of vertices in polygon */
char	hostat[CMLEN],                  /* stats file host name */
        hosin[MAXIMG][CMLEN],           /* input image host name */
        lasout[CMLEN],                  /* las output image name */
        hosout[CMLEN];                  /* output image host name */
long	itype,                          /* data type of pixel values */
        ncx,                            /* no. of classes to be processed */
        lmin, lmax,                     /* min and max line numbers */
        buf_siz,                        /* buffer for storage */
        i, m, indx, inc, ins,           /* counter varibles */
        npairs;                         /* no. of from-to pairs */
short	*lut;                      	/* look up table for mapping */
float	*minp, *maxp,     		/* min and max pixel values */
        *pairmin;                	/* scaling ratios for pixels */
static float	pairmax[MAXBND],        /* outside ranges */
        classconst[MAXCLS];             /* constants for each class */
static double	*cmean[MAXCLS];         /* class means */
static double  amin[MAXBND];
static double  amax[MAXBND];
long    one = 1;


count = 0;
/*
-----------------------     
 get tae parameters
-----------------------     
*/
getparms(hostat, hosin, hosout, window, &nimages, &itype, 
         &totbnd, same, &map, from, to, &pixval, &nc, &npairs, lasout);

/*
------------------------------------------------
 open stats file and check the number of classes
------------------------------------------------
*/
statchk (hostat, nc, &ncx);

/*
Allocate dynamic storage
------------------------
*/
dyn_alloc (totbnd, ncx, cmean, isno, &lut,&minp,&maxp,&pairmin,window);

/*
------------------------------------------------------------
 get user class and site specifications and associated stats
 file indices
------------------------------------------------------------
*/
indices (nc, ncx, icno, isno, nsites, same, classconst, pixval, itype, map);

/*
Allocate storage for polygon matrices
-------------------------------------
*/
buf_siz = MAX_POLYGON * 201 * sizeof(long);
if ((x_matrix = (long *) malloc (buf_siz)) == NULL ||
    (y_matrix = (long *) malloc (buf_siz)) == NULL)
  zip_err ("!Error allocating storage", "zip-alloc");
for (i=0, m=0; i < MAX_POLYGON; i++, m+= 201) {
   xaxis[i] = &x_matrix[m];
   yaxis[i] = &y_matrix[m];
}
/*
---------------------------------
 set up site windows for polygons
---------------------------------
*/
xoff = 1;
yoff = 1;
setarea (ncx, icno, isno, nsites, area, nverts, &lmin, &lmax, 
         xoff, yoff, window);

/*
------------------------------
 if map=1, compute class means 
------------------------------
*/
if (map  ==  1) 
   {
   setfgin(hosin, nimages, window);
   stats (totbnd, icno, isno, nsites, ncx, area, nimages,
          nverts, cmean, xoff, yoff, window);
   }  /* if */

/*
--------------------------------
 set up file specification group
--------------------------------
*/
setfsg(hosin, hosout, nimages, window, totbnd, itype);

/*
-------------------------------------------------------------
 if map=2, check validity of constants according to data type
-------------------------------------------------------------
*/
if (map  ==  2) 
   {
   indx = 0;
   for (inc = 0; inc < ncx; inc++)
      {
      for (ins = 1; ins <= nsites[inc]; ins++)
         {
         range (classconst[indx], itype, &one);
         indx = indx+1;
         }  /* do */
      }  /* do */
    }  /* if */
/*
---------------------------------------------------------
 if map=3, make look_up table for byte and i*2 data types
---------------------------------------------------------
*/
if (map  ==  3) 
   {
   if (itype  ==  1  ||  itype  ==  2) 
      {
      makelut (lut, from, to, npairs, itype);
      }
   else
      {
      getpair (scale, totbnd, from, to, minp, maxp, pairmin, pairmax, npairs);
      } /* if */
   }  /* if */

/*
-----------
 do mapping
-----------   
*/
mapping (totbnd, window, minp, isno, nsites, nverts, 
         ncx, area, itype, cmean, lmin, lmax, lut, scale, pairmin, pairmax, 
         classconst, map, npairs, from, to);

/*
close files
-----------
*/
c_stclse((FILE **)&fd_sfin); /*** NEWLAS call ***/
c_egclse((struct GDESC **)&gdesc);

/*
Calculate the minimum and maximum pixel values for the output image
------------------------------------------------------------------- 	*/
if (min_max)
{
	window[0][SL] = window[0][SS] = 1;
	c_minmax ((char (*)[256])hosout, &one, window, out_img_bands, nbands,
			 amin, amax); 
	c_svmnmx (hosout, &totbnd, amin, amax, "YES");
}

free(iobuf);
free(x_matrix);
free(y_matrix);
free(ipoint);
free(temp1);
free(temp2);

/*
update output file histories
----------------------------
*/
c_hstout(&block, hosout, hosin, &nimages, " ");

/* 
completion message
------------------
*/

c_complt("ZIP", (char (*)[256])lasout, &one);

}
