/******************************************************************************

FUNCTION:	BSF

PURPOSE:	Part of the DITTO program which back space files on a DITTO
		device.  This routine is used to back space on the input
		device or output device.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      11/85    K. Gacke     	initial development
  1.1	    7/86    K. Gacke		LAS standards
  5.0      10/88    D. Hollaren		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	The parameters are inputted into the program.  From parameter WHICH,
	it is determined which device is to be processed.  The command to back 
	space the number of files is issued for the appropriate device.
ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "ditto.h"			/* ditto include file                */
#include "protos.h"
 
struct PARBLK   vblock;			/* TAE parameter block               */
struct DITTO_DEV ditto_device;		/* ditto I/O devices descriptors     */

int main
(void)
{
struct VARIABLE *val;			/* pointer to TAE variable           */

long	count;				/* number of files to be back spaced */
long	std_out;			/* Fortran LUN for standard output   */
long	zero = 0;			/* value of zero                     */

char	out[1][CMLEN];			/* Output files for completion msg   */
char	*device;			/* DITTO device (tape or disk)       */
char	*which;				/* DITTO input/output device         */

c_inlas(&vblock,&std_out);		/* initialzie TAE                    */

ditto_device.input_dev = NULL;		/* set input and output devices to   */
ditto_device.output_dev = NULL;		/*   no values                       */

val = p_find(&vblock,"devtype");	/* get DITTO device to be processed  */
which = SVAL(*val,0);

val = p_find(&vblock,"nfiles");		/* get no. of files to be back spaced*/
count = IVAL(*val,0);

if (strcmp(which,"INPUT") == 0)		/* retrieve the input or output      */ 
   val = p_find(&vblock,"idevice");	/*   device depending on parameter   */
else					/*   "which".                        */
   val = p_find(&vblock,"odevice");

if (val->v_count == 0)
   ditto_error("INPUT\0",DITTO_ALLOC,ERROR_SEV,NO_UPDATE_FD);

device = SVAL(*val,0);

if (strcmp(device,"DISK") == 0)	
   proc_disk(&vblock,which,count);	/* back space records on disk file   */
else
   proc_tape(which,count);		/* back space records on tape device */

c_complt("DITTO BSF",out,&zero);

return EXIT_SUCCESS;

}
