/******************************************************************************
FUNCTION:	PROC_DISK

PURPOSE:	Back spaces the specified number of files on the specified
		disk file.  This is done by updating the attribute "cur_rec"
		and "cur_file" associated with the disk file.

ALGORITHM 
  Retrieve current disk file number
  Set current record to zero 
  back space specified # of files
  Update current record and file attributes of specified device
  Write the new parameter block

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_disk
(
    struct PARBLK *vblock,		/* TAE parameter block               */
    char *which,			/* DITTO input/output device         */
    long count			/* number records to be back spaced  */
)
{
struct DISK_FD fd;			/* disk file attributes              */

struct VARIABLE *fil;			/* pointer to TAE variable           */

int temp = 0;               /* Temp integer */


/*
  Retrieve current disk file number
*/
if (strcmp(which,"INPUT") == 0)
   fil = p_find(vblock,"ifile");
else
   fil = p_find(vblock,"ofile");
fd.cur_file = IVAL(*fil,0);

fd.cur_rec = 0;				/* set current record to zero        */

if (fd.cur_file == NOVAL)
   ditto_error(which,DITTO_BSF,ERROR_WARN,NO_UPDATE_FD);
else
   {
   fd.cur_file -= count;		/* back space specified # of files   */
   if (fd.cur_file < 1)
      {
      ditto_error(which,DITTO_BSF,ERROR_WARN,NO_UPDATE_FD);
      fd.cur_file = 1;
      }
   }

/*
  Update current record and file attributes of specified device
*/
if (strcmp(which,"INPUT") == 0)
   {
   temp = (int) fd.cur_rec;
   q_intg(vblock,"irec",1, &temp,P_UPDATE);
   temp = (int) fd.cur_file;
   q_intg(vblock,"ifile",1, &temp,P_UPDATE);
   }
else
   {
   temp = (int) fd.cur_rec;
   q_intg(vblock,"orec",1, &temp,P_UPDATE);
   temp = (int) fd.cur_file;
   q_intg(vblock,"ofile",1, &temp,P_UPDATE);
   }

/*
  Write the new parameter block
*/
q_out(vblock);

return;
}
