/******************************************************************************
FUNCTION:	PROC_TAPE

PURPOSE:	Back spaces the specified number of files on the specified
		magnetic tape.  This is done by calling the tape I/O routine
		"tpbsf" and updating the attribute "cur_rec" and "cur_file"
		associated with the tape.

ALGORITHM 
  Reopen the tape device
  Back space specified # of records
  Set current record to zero 
  Update attributes "cur_rec" and "cur_file" of the tape device
  Close the tape device

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_tape
(
    char *which,			/* DITTO input/output device         */
    long count			/* number of files to be back spaced */
)
{
struct TAPE_FD  tape;			/* tape descriptor		     */

long mode = HANDLE;			/* tape package outputs error msg    */
long nonfatal = 1;			/* Non-fatal error flag		     */
long fatal = -1;			/* Fatal error flag		     */
long status;				/* function return status code       */

char msg[ERRLEN];			/* error message to be outputted     */

/*
  Reopen the tape device
*/
status = reopen_tape(which,NOREW,TREAD,&tape);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto bsf-fatal",&fatal);

/*
  Back space specified # of records
*/
status = c_tpbsf(&tape.unit,&count,&mode);

tape.cur_rec = 0;			/* set current record to zero        */
tape.cur_file -= count;			/* update attributes "cur_rec" and   */
if (tape.cur_file < 0)			/* "cur_file" of the tape device     */
   tape.cur_file = NOVAL;

if (status == TAPE_BOT)
   {
   sprintf(msg,"Reached BOT ... Back spaced %d files",count);
   c_errmsg(msg," ",&nonfatal);
   tape.cur_file = 0;
   }
else if (status != TAPE_SUCC)
   {
   sprintf(msg,"Back spaced %d files",count);
   c_errmsg(msg," ",&nonfatal);
   ditto_error(which,DITTO_BSF,ERROR_WARN,NO_UPDATE_FD);
   tape.cur_file = 0;
   }

/*
  Close the tape device
*/
close_tape(which,&tape,NOREW,1);

return;
}
