/******************************************************************************

FUNCTION:	BSR

PURPOSE:	Part of the DITTO subsystem which back space records on a
		DITTO device.  This routine is used to back space on the
		input device or output device.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      11/85    K. Gacke   		initial development
  1.1	    7/86    K. Gacke		LAS standards
  5.0      10/88    D. Hollaren		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	The parameters are inputted into the program.  From parameter WHICH,
	it is determined which DITTO device is to be processed.  The appropriate
	subroutine is called to forward space the number of records for either 
	the tape or disk file.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include                  */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"
 
struct PARBLK   vblock;			/* TAE parameter block               */
struct DITTO_DEV ditto_device;		/* ditto I/O devices descriptors     */

int main
(void)
{
struct VARIABLE *val;			/* pointer to TAE variable           */

long	count;				/* number of files to be back spaced */
long	std_out;			/* Fortran LUN for standard output   */
long	zero = 0;			/* value of zero                     */

char	out[1][CMLEN];			/* Output files for completion msg   */
char	*device;			/* DITTO device (tape or disk)       */
char	*which;				/* DITTO input/output device         */

/*
  Initialize TAE
*/
c_inlas(&vblock,&std_out);

/*
  Set input and output devices to no values
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Retrieve device type parameter
*/
val = p_find(&vblock,"devtype");
which = SVAL(*val,0);

/*
  Retrieve number of records parameter
*/
val = p_find(&vblock,"nrecs");
count = IVAL(*val,0);

/*
  Retrieve the input or output device depending on parameter "which"
*/
if (strcmp(which,"INPUT") == 0)
   val = p_find(&vblock,"idevice");
else
   val = p_find(&vblock,"odevice");

device = SVAL(*val,0);

if (strcmp(device,"DISK") == 0)	
   proc_disk(which,count);		/* back space records on disk file   */
else
   proc_tape(which,count);		/* back space records on tape device */

/*
  Output completion message
*/
c_complt("DITTO BSR",out,&zero);

return EXIT_SUCCESS;

}
