/******************************************************************************
FUNCTION:	PROC_DISK

PURPOSE:	Back spaces the specified number of records on the specified
		disk file.  This is done by updating the attribute "cur_rec"
		associated with the disk file.

ALGORITHM
	Retrieve current disk attributes 
	Back space specified # of records
	update disk attributes 

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_disk
(
    char *which,			/* DITTO input/output device         */
    long count			/* number records to be back spaced  */
)
{
struct DISK_FD fd;			/* disk file attributes              */
long status;				/* Status variable		     */
long fatal = -1;			/* Fatal error code		     */

/*
  Retrieve current disk attributes
*/
status = get_disk_fd(which,&fd);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto bsr-fatal",&fatal);

/*
  Back space specified number of records
*/
fd.cur_rec -= count;
if (fd.cur_rec < 0)
   {
   ditto_error(which,DITTO_BSR,ERROR_WARN,NO_UPDATE_FD);
   fd.cur_rec = 0;
   }

/*
  Update disk attributes
*/
put_disk_fd(which,&fd);

return;
}
