/******************************************************************************
FUNCTION:	PROC_TAPE

PURPOSE:	Back spaces the specified number of records on the specified
		magnetic tape.  This is done by calling the tape I/O routine
		"tpbsr" and updating the attribute "cur_rec" associated with
		the tape.

ALGORITHM 
  Reopen the tape device
  Back space specified # of records
  Update attributes "cur_rec" and "cur_file" of the tape device 
  Close the tape device

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include                  */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_tape
(
    char *which,			/* DITTO input/output device         */
    long count			/* number of files to be back spaced */
)
{
struct TAPE_FD  tape;			/* tape descriptor		     */

long   mode = HANDLE;			/* tape package outputs error msg    */
long   nonfatal = 1;			/* Non-fatal error code		     */
long   fatal = -1;			/* Fatal error code		     */
long   status;				/* function return status code       */

char  msg[ERRLEN];			/* error message to be outputted     */

/*
  Reopen the tape device
*/
status = reopen_tape(which,NOREW,TREAD,&tape);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto bsr-fatal",&fatal);

/*
  Back space specified # of records
*/
status = c_tpbsr(&tape.unit,&count,&mode);

tape.cur_rec -= count;			/* update attributes "cur_rec" and   */
if (tape.cur_rec < 0)			/*   "cur_file" of the tape device   */
   {
   tape.cur_rec = NOVAL;
   tape.cur_file = NOVAL;
   }

if (status != TAPE_SUCC)
   {
   sprintf(msg,"Back spaced %d records",count);
   c_errmsg(msg," ",&nonfatal);
   ditto_error(which,DITTO_BSR,ERROR_WARN,NO_UPDATE_FD);
   tape.cur_rec = 0;
   }

/*
  Close the tape device
*/
close_tape(which,&tape,NOREW,1);

return;
}
