/******************************************************************************
FUNCTION:	CLOSE_DEVICE

PURPOSE:	Closes the input and output devices currently allocated to
		the DITTO program.  It also updates the attributes of each
		device (i.e. current record and current file).

ALGORITHM 
    If input device is disk file then
	Update current record and file attributes
	Update DITTO global variables
	Close the disk file
    Else if input device is tape then
	Update current record and file attributes
	Update DITTO global variables
	Close the tape (no rewind)

    If output device is disk file then
	Update current record and file attributes
	Update DITTO global variables
	Close the disk file
    Else if output device is tape then
	Update current record and file attributes
	Update DITTO global variables
	Close the tape (no rewind)

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"		/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "copy.h"     
#include "protos.h"

void close_device
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev	/* output DITTO device               */
)
{
extern struct DISK_FD idisk;		/* input disk file                   */
extern struct DISK_FD odisk;		/* output disk file                  */
extern struct TAPE_FD itape;		/* input tape                        */
extern struct TAPE_FD otape;		/* output tape                       */

long status;			/* function return status code       */

/*
  If the input device is a disk file then update current record and file
  attributes
*/
if (indev->device == DISK)
   {
   if ((indev->cur_file > 0) && (idisk.cur_file != NOVAL))
      {
      idisk.cur_file += indev->cur_file;
      idisk.cur_rec = 0;
      }
   else if (idisk.cur_rec != NOVAL)
      idisk.cur_rec += indev->cur_rec;

   /*
     Update the DITTO global variables
   */
   put_disk_fd("INPUT",&idisk);

   /*
     Close the disk file
   */
   status = dclose(&idisk);
   if (status != E_SUCC)
	ditto_error("INPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else
   /*
     If the input device is tape then update current record and file attributes
   */
   {
   if (indev->cur_file > 0)
      {
      if (itape.cur_file != NOVAL)
	 itape.cur_file += indev->cur_file;
      itape.cur_rec = 0;
      }
   else if (itape.cur_rec != NOVAL)
      itape.cur_rec += indev->cur_rec;

   /*
     Close the tape (no rewind)
   */
   close_tape("INPUT",&itape,NOREW,1);
   }

/*
  If the output device is a disk file then update current record and
  file attributes
*/
if (outdev->device == DISK)
   {
   if ((outdev->cur_file > 0) && (odisk.cur_file != NOVAL))
      {
      odisk.cur_file += outdev->cur_file;
      odisk.cur_rec = 0;
      }
   else if (odisk.cur_rec != NOVAL)
      odisk.cur_rec += outdev->cur_rec;

   /*
     Update the DITTO global variables
   */
   put_disk_fd("OUTPUT",&odisk);

   /*
     Close the disk file
   */
   status = dclose(&odisk);
   if (status != E_SUCC)
	ditto_error("OUTPUT",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else
   /*
     If output device is tape then update current record and file attributes
   */
   {
   if ((outdev->cur_file > 0) && (otape.cur_file != NOVAL))
      otape.cur_file += outdev->cur_file;

   otape.cur_rec = 0;

   /*
     Close the tape (no rewind)
   */
#ifdef vms
   close_tape("OUTPUT",&otape,NOREW,2);
#else
   close_tape("OUTPUT",&otape,NOREW,1);
#endif
   }

return;
}
