#ifndef PROTOS_H
#define PROTOS_H

void close_device
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev	/* output DITTO device               */
);

int copy_proc
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev,	/* output DITTO device               */
    long nrecords,			/* number of records to be copied    */
    char *trans,			/* type of transformations           */
    long swap,				/* byte swapping flag                */
    long msgtim			/* # of seconds between display msg  */
);

void file_copy
(
    char *idevice,			/* input DITTO device (tape or disk) */
    char *odevice			/* output DITTO device (tape or disk)*/
);

int file_open
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice,		/* output DITTO device (tape or disk)*/
    struct COPY_DEV *indev,	/* input device descriptor           */
    struct COPY_DEV *outdev,	/* output device descriptor          */
    long flag				/* error check flag                  */
);

void file_close
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev	/* output DITTO device               */
);

void modrec_copy
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice		/* output DITTO device (tape or disk)*/
);

int get_modrec
(
    long   currec,		/* current record being processed    */
    long   nrecords 		/* number of records to be processed */
);

void modify_record
(
    unsigned char *bufptr,	/* buffer to be modified             */
    long nbyte		/* number of bytes in physical record*/
);

unsigned char *get_modstr
(
    char *string,
   char *type,
   long *mod_length
);

void open_device
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice,		/* output DITTO device (tape or disk)*/
    struct COPY_DEV *indev,	/* input device descriptor           */
    struct COPY_DEV *outdev,	/* output device descriptor          */
    long flag				/* error check flag                  */
);

void pits_copy
(
    char *idevice,			/* input DITTO device (tape or disk) */
    char *odevice			/* output DITTO device (tape or disk)*/
);

int pits_open
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice,		/* output DITTO device (tape or disk)*/
    struct COPY_DEV *indev,	/* input device descriptor           */
    struct COPY_DEV *outdev,	/* output device descriptor          */
    long flag				/* error check flag                  */
);

void pits_close
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev	/* output DITTO device               */
);

int copy_pits
(
    struct COPY_DEV *indev,	/* input DITTO device                */
    struct COPY_DEV *outdev,	/* output DITTO device               */
    long *filnmr,			/* current file number being copied  */
    long msgtim			/* # of seconds between display msg  */
);

int pits
(
    struct COPY_DEV *indev,
   struct COPY_DEV *outdev,
   long *filnmr,
   long msgtim,
   long prev_nbyte,
   char *bufptr
);

int pits_dcopy
(
    struct COPY_DEV *indev,
   struct COPY_DEV *outdev,
   long *filnmr,
   long msgtim
);

int pits_read_device
(
   struct COPY_DEV *indev,
   unsigned char *inbuf
);

void proc_error
(
    struct COPY_DEV *indev,	/* input copy device                 */
    struct COPY_DEV *outdev,	/* output copy device                */
    char *which,		/* which device (input or output)    */
    long error_code		/* error code encountered            */
);

int read_device
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    unsigned char *inbuf	/* buffer to read record into        */
);

void reblock_copy
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice		/* output DITTO device (tape or disk)*/
);

void reblk_proc
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    long nrecords,			/* # of input records to be copied   */
    char *trans,			/* type of transformation            */
    long swap,				/* byte swapping flag                */
    unsigned char pad_val,		/* pad value                         */
    long msgtim			/* # seconds between display messages*/
);

void reblk_pad
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    unsigned char *outbuf,	/* output buffer to pad              */
    long pad_val,		/* the pad value                     */
    long *outcur,		/* current output byte               */
    long *outtot		/* total number of output bytes      */
);

void reblk_prewrite
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    unsigned char *outbuf,	/* output buffer                     */
    long *outtot		/* total # bytes in output record    */
);

void record_copy
(
    char *idevice,		/* input DITTO device (tape or disk) */
    char *odevice		/* output DITTO device (tape or disk)*/
);

void transform
(
    unsigned char *bufptr,	/* buffer to perform transformation on*/
    long *nbyte,		/* number of bytes in buffer          */
    long *swap,		/* # bytes in entity to be swapped    */
    char *trans		/* type of transformation             */
);

void write_device
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    unsigned char *outbuf,	/* buffer to write to output device  */
    long nbyte				/* number of bytes to be written     */
);

#endif
