/******************************************************************************
FUNCTION:	READ_DEVICE

PURPOSE: 	Reads one physical record from the input DITTO device.

ALGORITHM 
    If input device is DISK
	Read one physical record from disk
    Else if input device is TAPE
	Read one physical record from tape

    Return the number of bytes read

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "copy.h"
#include "protos.h"

int read_device
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    unsigned char *inbuf	/* buffer to read record into        */
)
{
extern struct DISK_FD idisk;		/* DITTO input disk file descriptor  */
extern struct TAPE_FD itape;		/* DITTO input tape descriptor       */

long mode = HANDLE;			/* tape I/O package output error msg */

long  nbyte;			/* number of bytes read              */

/*
  If input device is disk read one physical record from disk file
*/
if (indev->device == DISK)
   {
   nbyte = dread(&idisk,inbuf,idisk.blksiz);
   if (nbyte == E_FAIL)
      proc_error(indev,outdev,"INPUT",DITTO_DREAD);
   }
else
   /*
     If input device is tape read one physical record from tape
   */
   {
   nbyte = c_tpread(&itape.unit,inbuf,&itape.blksiz,&mode);
   if (nbyte < 0)
      {
      switch(nbyte)
         {
         case TAPE_EOF:	return(0);	  
         case TAPE_EOV:	return(COPY_EOV); 
         case TAPE_EOT:	return(COPY_EOV); 
         default:	proc_error(indev,outdev,"INPUT",DITTO_TPREAD); break;
         }
      }
   }

return(nbyte);
}
