/******************************************************************************
FUNCTION:	RECORD_COPY

PURPOSE: 	Copies the specified number of records of a single file from
		the input device to the output device.  The record and block
		sizes of the output device should be equal to the input device.
		The record types of both the input and output devices should
		also be the same.

ALGORITHM 
    Retrieve input parameters
    Open the input and output DITTO device
    Copy the specified number of input records to output device
    Close the input and output DITTO devices

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>	 		/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "ditto.h"			/* ditto include file                */
#include "copy.h"
#include "protos.h"

void record_copy
(
    register char *idevice,		/* input DITTO device (tape or disk) */
    register char *odevice		/* output DITTO device (tape or disk)*/
)
{
extern struct PARBLK vblock;		/* TAPE parameter block              */
struct COPY_DEV indev;			/* input copy device descriptor      */
struct COPY_DEV outdev;			/* output copy device descriptor     */
struct VARIABLE *val;			/* pointer to TAE parameters         */

long  msgtim;				/* # seconds between display messages*/
long  nrecords;				/* # input records to be processed   */
long  swap;				/* byte swapping flag                */
char trans[10];				/* type of transformation            */

val = p_find(&vblock,"nrecs");		/* retrieve parameter "nrecs"        */
if (val->v_count > 0)
   nrecords = IVAL(*val,0);
else
   nrecords = ALL;

val = p_find(&vblock,"tranopt");	/* retrieve parameter "transopt"     */
strcpy(trans,SVAL(*val,0));

val = p_find(&vblock,"nbytes");		/* retrieve parameter "nbytes"       */
swap = IVAL(*val,0);

val = p_find(&vblock,"$msgtim");	/* retrieve parameter "$msgtim"      */
msgtim = IVAL(*val,0);

/*
  Open the input and output DITTO device
*/
open_device(idevice,odevice,&indev,&outdev,CHECK);

/*
  Copy the specified number of input records to output device
*/
copy_proc(&indev,&outdev,nrecords,trans,swap,msgtim); 

/*
  Close the input and output DITTO devices
*/
close_device(&indev,&outdev);

return;
}
