/******************************************************************************
FUNCTION:	WRITE_DEVICE

PURPOSE: 	Writes one physical record to the output DITTO device.

ALGORITHM 
    If output device is disk
	write one physical record to disk file
    Else if output device is tape
	write one physical record to tape

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h> 			/* C standard I/O include file       */
#include "las.h"	 		/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h" 			/* ditto include file                */
#include "copy.h"  
#include "protos.h"

void write_device
(
    struct COPY_DEV *indev,	/* input copy device descriptor      */
    struct COPY_DEV *outdev,	/* output copy device descriptor     */
    unsigned char *outbuf,	/* buffer to write to output device  */
    long nbyte				/* number of bytes to be written     */
)
{
extern struct DISK_FD odisk;		/* DITTO output disk file descriptor */
extern struct TAPE_FD otape;		/* DITTO output tape descriptor      */

long mode = HANDLE;			/* tape I/O package output error msg */
long nonfatal = 1;			/* Non-fatal error code		     */
long   status;			/* function return status code       */

/*
  If output device is disk write one physical record to disk file
*/
if (outdev->device == DISK)
   {
   status = dwrite(&odisk,outbuf,nbyte);
   if (status == E_FAIL)
      proc_error(indev,outdev,"OUTPUT",DITTO_DWRITE);
   }
else
   /*
     If output device is tape write one physical record to tape
   */
   {
   status = c_tpwrit(&otape.unit,outbuf,&nbyte,&mode);
   switch (status)
      {
      case TAPE_EOT: 
	c_errmsg("Error writing to tape -- End of tape encountered",
		"ditto copy-eot",&nonfatal);
	break;

      case TPWRIT_SIZE: 
	c_errmsg("Error writing to tape -- Too many or too few bytes requested",
                "ditto copy-size",&nonfatal);
	break;

      case TPWRIT_WRITE: 
	c_errmsg("Error writing to tape -- Device error","ditto copy-deverr",
		&nonfatal);
	break;
      }

   if (status != nbyte)
      proc_error(indev,outdev,"OUTPUT",DITTO_TPWRIT);
   }

return;
}
