/******************************************************************************
 
FUNCTION:	DEALLOC

PURPOSE:	Part of the DITTO subsystem which deallocates the input/output
		devices.  If the device is "tape", then the tape is dismounted.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      10/85    K. Gacke     	initial development
  1.1	    7/86    K. Gacke		LAS standards
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1      03/97    D. Etrheim		Modified to pass REWIND to reopen_tape()
					Reopen_tape was changed to make the
					c_tpbsf error a fatal error on 9/96.
					Passing REWIND to reopen_tape() will
					disable the c_tpbsf portion which was
					causing problems if the tapes had 
					encountered an io error.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	From the input parameters it is determined which input/output DITTO
	is to be deallocated.  If the device is allocated to a magnetic tape,
	the tape is dismounted.  The DITTO global variables are updated to
	indicate that the device is not allocated.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"		 	/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* DITTO include file                */
#include "protos.h"
 
struct PARBLK   vblock;			/* TAE parameter block               */
struct DITTO_DEV ditto_device;		/* ditto I/O devices descriptors     */

int main
(void)
{
struct VARIABLE *val;			/* pointer to TAE variable           */

long	std_out;			/* Fortran LUN for standard output   */
long	zero = 0;			/* value of zero                     */

char	out[1][CMLEN];			/* output files for completion msg   */
char	*which;				/* DITTO input/output device         */

/*
  Initialize TAE
*/
c_inlas(&vblock,&std_out);

/*
  Set input and output devices to no values
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Retrieve the device type
*/
val = p_find(&vblock,"devtype");
which = SVAL(*val,0);

if ((strcmp(which,"INPUT") == 0) || (strcmp(which,"BOTH") == 0))
   proc_input();

if ((strcmp(which,"OUTPUT") == 0) || (strcmp(which,"BOTH") == 0))
   proc_output();

c_complt("DITTO DEALLOC",out,&zero);

return EXIT_SUCCESS;

}
