/******************************************************************************
FUNCTION:	PROC_INPUT

PURPOSE:	Deallocates the DITTO input device.  If the device is
		allocated to a tape, then the tape is dismounted.  The DITTO
		global variables are updated to indicate that the DITTO input
		device is not allocated.

ALGORITHM 
  Retrieve the input device
  If the input device is not allocated
	display warning message
  If the input device is allocated to tape
	reopen the tape
	dismount the tape
  Update the DITTO global variables
  Write updated TAE parameter block

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* DITTO include file                */
#include "protos.h"

void proc_input
(void)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
struct VARIABLE *val;			/* pointer to TAE variable           */
struct TAPE_FD tape;			/* tape descriptor                   */

int   idummy[1];			/* dummy integer value               */
long  fatal = -1;			/* Fatal error code		     */
long  status;				/* function return status code       */

char idevice[10];			/* input DITTO device                */
char *tdummy[1];			/* dummy character string pointer    */

idummy[0] = 0;				/* set dummy variables to null       */
tdummy[0] = "\0";

/*
  Retrieve the input device
*/
val = p_find(&vblock,"idevice");
if (val->v_count == 0)			/* Input device not allocated	     */
   {
   ditto_error("INPUT\0",DITTO_NALLOC,ERROR_WARN,NO_UPDATE_FD);
   return;
   }

strcpy(idevice,SVAL(*val,0));

/*
  If the input device is allocated to tape dismount the tape
*/
if (strcmp(idevice,"TAPE") == 0)
   {
   status = reopen_tape("INPUT\0",REWIND,TREAD,&tape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto dealloc-fatal",&fatal);

   close_tape("INPUT\0",&tape,REWOFFL,1);
   }

/*
  Update the DITTO global variables
*/
status = q_string(&vblock,"idevice",0,tdummy,P_UPDATE);
status = q_string(&vblock,"ihost",0,tdummy,P_UPDATE);
status = q_string(&vblock,"irectyp",0,tdummy,P_UPDATE);
status = q_string(&vblock,"icc",0,tdummy,P_UPDATE);
status = q_intg(&vblock,"irecsiz",0,idummy,P_UPDATE);
status = q_intg(&vblock,"iblksiz",0,idummy,P_UPDATE);
status = q_intg(&vblock,"irec",0,idummy,P_UPDATE);
status = q_intg(&vblock,"ifile",0,idummy,P_UPDATE);
status = q_intg(&vblock,"iskip",0,idummy,P_UPDATE);
status = q_string(&vblock,"idensity",0,tdummy,P_UPDATE);

/*
  Write updated TAE parameter block
*/
status = q_out(&vblock);

return;
}
