/******************************************************************************
FUNCTION:	PROC_OUTPUT

PURPOSE:	Deallocates the DITTO output device.  If the device is
		allocated to a tape, then the tape is dismounted.  The
		DITTO global variables are updated to indicate that the
		DITTO output device is not allocated.

ALGORITHM 
  Retrieve the output device
  If the output device is not allocated
	display a warning message
  If the output device is tape
	reopen the tape
	dismount the tape
  Update the DITTO global variables
  Write the updated parameter block

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* DITTO include file                */
#include "protos.h"

void proc_output
(void)
{
extern struct PARBLK vblock;		/* TAE parameter block               */

struct VARIABLE *val;			/* pointer to TAE variable           */
struct TAPE_FD tape;			/* tape descriptor                   */

int   idummy[1];			/* dummy integer value               */
long  fatal = -1;			/* Fatal error code		     */
long  status;				/* function return status code       */

char odevice[10];			/* output DITTO device               */
char *tdummy[1];			/* dummy character string pointer    */

idummy[0] = 0;				/* assign dummy variables to null    */
tdummy[0] = "\0";

/*
  Retrieve the output device
*/
val = p_find(&vblock,"odevice");
if (val->v_count == 0)			/* Output device not allocated	     */
   {
   ditto_error("OUTPUT\0",DITTO_NALLOC,ERROR_WARN,NO_UPDATE_FD);
   return;
   }
strcpy(odevice,SVAL(*val,0));

/*
  If the output device is tape dismount the tape
*/
if (strcmp(odevice,"TAPE") == 0)
   {
   status = reopen_tape("OUTPUT\0",REWIND,TREAD,&tape);
   if (status != E_SUCC)
	c_errmsg("Fatal error encountered","ditto dealloc-fatal",&fatal);

   close_tape("OUTPUT\0",&tape,REWOFFL,1);
   }

/*
  update the DITTO global variables
*/
status = q_string(&vblock,"odevice",0,tdummy,P_UPDATE);
status = q_string(&vblock,"ohost",0,tdummy,P_UPDATE);
status = q_string(&vblock,"orectyp",0,tdummy,P_UPDATE);
status = q_string(&vblock,"occ",0,tdummy,P_UPDATE);
status = q_intg(&vblock,"orecsiz",0,idummy,P_UPDATE);
status = q_intg(&vblock,"oblksiz",0,idummy,P_UPDATE);
status = q_intg(&vblock,"orec",0,idummy,P_UPDATE);
status = q_intg(&vblock,"ofile",0,idummy,P_UPDATE);
status = q_intg(&vblock,"oskip",0,idummy,P_UPDATE);
status = q_string(&vblock,"odensity",0,tdummy,P_UPDATE);

/*
  Write the updated parameter block
*/
status = q_out(&vblock);

return;
}
