/******************************************************************************
FUNCTION:	CLOSE_DEVICE

PURPOSE: 	Closes the input or output DITTO device that was processed.
		It also updates the attributes of the device (i.e. current
		record and current file).

ALGORITHM 
  If the device is a disk file
     Update attributes of the device
     Close the disk file
  Else the device is a tape
     Update attributes of the device
     Close the tape (no rewind)

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>  			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void close_device
(
    char *which,			/* which device (input or output)    */
    char *device,			/* type of device (tape or disk)     */
    long currec,			/* Current record		     */
    long bytes_read			/* # bytes read on last read         */
)
{
extern struct DISK_FD disk;		/* DITTO disk file descriptor        */
extern struct TAPE_FD tape;		/* DITTO tape descriptor             */

long status;				/* function return status code       */

/*
  Close the DITTO disk file
*/
if (strcmp(device,"DISK") == 0)
   {
   if ((bytes_read == 0) && (disk.cur_file != NOVAL))
      {
      disk.cur_file++;
      disk.cur_rec = 0;
      }
   else if (disk.cur_rec != NOVAL)
      disk.cur_rec += currec;

   /*
     Update the disk information
   */
   put_disk_fd(which,&disk);

   /*
     Close the attributes of the disk file
   */
   status = dclose(&disk);
   if (status != E_SUCC)
	ditto_error(which,DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else
   /*
     Close the DITTO tape device
   */
   {
   if (bytes_read == 0)
      {
      if (tape.cur_file != NOVAL)
	 tape.cur_file++;

      tape.cur_rec = 0;
      }
   else if (tape.cur_rec != NOVAL)
      tape.cur_rec += currec;

   /*
     Close the tape (no rewind)
   */
   close_tape(which,&tape,NOREW,1);
   }

return;
}
