#include <stdio.h>  			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"
 
/******************************************************************************
FUNCTION:	LP_CHAR

PURPOSE: 	Formats a physical record and outputs it to a file that is
		to be spooled to the line printer.  The record is dumped in
		ascii format with non-printable characters are represented
		by the character '.'

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
void lp_char
(
    FILE  *lp_fp,		/* file pointer of dump file         */
    unsigned char *buf,	/* buffer containing physical record */
    long   nbyte,		/* number of bytes in physical record*/
    long        currec 			/* current record being dumped       */
)
{
long	   max_byte = 96;
long	   total = 0;	
long   width;
char       out_buf[135];

/*
  Write out header information
*/
fprintf(lp_fp,"\n\nBLOCK NUMBER %d CONTAINS %d BYTES\n",currec,nbyte);
*(out_buf+max_byte) = *(out_buf+max_byte+1) = ' ';

for (width=0; nbyte--;width++,buf++)
   {
   if (width == max_byte)
      {
      sprintf(out_buf+width+2,"%06d",max_byte * total);
      fprintf(lp_fp,"%s\n",out_buf);
      width = 0;
      ++total;
      }

   if ((*buf < ' ') || (*buf > '~'))
      *buf = '.';

   *(out_buf + width) = (char) *buf;
   }

if (width > 0)
   {
   for (; width<max_byte; width++)
      *(out_buf+width) = ' ';

   sprintf(out_buf+width+2,"%06d",max_byte * total);
   fprintf(lp_fp,"%s\n",out_buf);
   }

return;
}

/******************************************************************************
FUNCTION:	LP_HEX

PURPOSE: 	Formats a physical record and outputs it to a file that is
		to be spooled to the line printer.  The record is dumped in
		hexidecimal and ascii format.  

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/

void lp_hex
(
    FILE *lp_fp,		/* file pointer of dump file         */
    unsigned char *buf,	/* buffer containing physical record */
    long nbyte,		/* number of bytes in physical record*/
    long currec			/* current record being dumped       */
)
{
long	   ch_start = 74;
long	   max_byte = 32;
long	   total = 0;
long	   width;

char	   hex_val[3];
char       out_buf[150];

char  *out_char;
char  *out_hex;

out_hex = out_buf;
out_char = out_buf + ch_start;

fprintf(lp_fp,"\n\nBLOCK NUMBER %d CONTAINS %d BYTES\n",currec,nbyte);
sprintf(out_buf,"%-135s"," ");

for (width=0; nbyte--;width++,buf++)
   {
   if (width == max_byte)
      {
      sprintf(out_char+2,"%06d",max_byte * total);
      fprintf(lp_fp,"%s\n",out_buf);
      width = 0;
      ++total;
      out_hex = out_buf;
      out_char = out_buf + ch_start;
      }

   if (width % 4 == 0)
      out_hex++;

   sprintf(hex_val,"%02X",*buf);
   *out_hex++ = *hex_val;
   *out_hex++ = *(hex_val + 1);

   if ((*buf < ' ') || (*buf > '~'))
      *buf = '.';

   *out_char++ = (char) *buf;
   }

if (width > 0)
   {
   for (; width<max_byte; width++,out_char++,out_hex+=2)
      {
      if (width % 4 == 0)
         out_hex++;

      *out_char = *out_hex = *(out_hex + 1) = ' ';
      }

   sprintf(out_char+2,"%06d",max_byte * total);
   fprintf(lp_fp,"%s\n",out_buf);
   }

return;
}
