/******************************************************************************
FUNCTION:	READ_DEVICE

PURPOSE:	Reads one physical record from the DITTO device being dumped.

ALGORITHM 
  If the device is a disk file
     Read one physical record from the disk file
  Else the device is a tape
     Read one physical record from the tape

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

int read_device
(
    char *device,		/* device type (tape or disk)        */
    int currec,
    unsigned char *buf	/* buffer to read record into        */
)
{
extern struct DISK_FD disk;		/* Disk device structure	     */
extern struct TAPE_FD tape;		/* Tape device structure	     */

long mode = HANDLE;			/* Error mode for tape I/O           */
long one = 1;				/* Value of one			     */
long nonfatal = 1;			/* Nonfatal error code		     */
long nbyte;				/* Number of bytes read		     */

/*
  If the device is a disk file
*/
if (strcmp(device,"DISK") == 0)
   {
   /*
     Read one physical record from the disk file
   */
   nbyte = dread(&disk,buf,disk.blksiz);
   if (nbyte == E_FAIL)
      proc_error(device,currec,DITTO_DREAD);
   }
else
   {
   /*
     Read one physical record from the tape
   */
   nbyte = c_tpread(&tape.unit,buf,&tape.blksiz,&mode);
   switch (nbyte)
      {
      case TAPE_EOF:
	c_errmsg("End of file detected","ditto dump-eof",&nonfatal);
	nbyte = 0;
	break;

      case TAPE_EOV:
	c_errmsg("End of volume detected","ditto dump-eov",&nonfatal);
	nbyte = 0;
        c_tpbsr(&tape.unit,&one,&mode);
	break;

      case TAPE_EOT:
	c_errmsg("End of tape detected","ditto dump-eot",&nonfatal);
	nbyte = 0;
	break;
      }

   if (nbyte < 0)
      proc_error(device,currec,DITTO_TPREAD);
   }

return(nbyte);
}
