#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

/******************************************************************************
FUNCTION:	TERM_CHAR

PURPOSE: 	Formats a physical record and outputs it to the terminal
		screen.  The record is dumped in ascii format with non-
		printable characters are represented by the character '.'

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/

void term_char
(
    unsigned char *buf,	/* buffer containing physical record */
    long nbyte,		/* number of bytes in physical record*/
    long currec			/* current record being dumped       */
)
{
long   max_byte = 64;
long   total = 0;
long   width;

long flag = 0;
char out_buf[100];

c_pterm(" ",&flag);

sprintf(out_buf,"BLOCK NUMBER %d CONTAINS %d BYTES",currec,nbyte);
c_pterm(out_buf,&flag);

*(out_buf+max_byte) = *(out_buf+max_byte+1) = ' ';
for (width=0; nbyte--;width++,buf++)
   {
   if (width == max_byte)
      {
      sprintf(out_buf+width+2,"%06d",max_byte * total);
      c_pterm(out_buf,&flag);
      width = 0;
      ++total;
      }

   if ((*buf < ' ') || (*buf > '~'))
      *buf = '.';

   *(out_buf + width) = (char) *buf;
   }

if (width > 0)
   {
   for (; width<max_byte; width++)
      *(out_buf+width) = ' ';

   sprintf(out_buf+width+2,"%06d",max_byte * total);
   c_pterm(out_buf,&flag);
   }

flag = 1;
c_pterm(" ",&flag);

return;
}

/******************************************************************************
FUNCTION:	TERM_HEX

PURPOSE:	Formats a physical record and outputs it to the terminal
		screen.  The record is dumped in hexidecimal and ascii format.

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
void term_hex
(
    unsigned char *buf,	/* buffer containing physical record */
    long nbyte,		/* number of bytes in physical record*/
    long currec			/* current record being dumped       */
)
{
long	   ch_start = 38;
long	   max_byte = 16;
long	   flag = 0;
long	   total = 0;
long	   width;

char	   hex_val[3];
char       out_buf[100];

char  *out_char;
char  *out_hex;

out_hex = out_buf;
out_char = out_buf + ch_start;

c_pterm(" ",&flag);

sprintf(out_buf,"BLOCK NUMBER %d CONTAINS %d BYTES",currec,nbyte);
c_pterm(out_buf,&flag);

sprintf(out_buf,"%-80s"," ");
for (width=0; nbyte--;width++,buf++)
   {
   if (width == max_byte)
      {
      sprintf(out_char+2,"%06d",max_byte * total);
      c_pterm(out_buf,&flag);
      width = 0;
      ++total;
      out_hex = out_buf;
      out_char = out_buf + ch_start;
      }

   if (width % 4 == 0)
      out_hex++;

   sprintf(hex_val,"%02X",*buf);
   *out_hex++ = *hex_val;
   *out_hex++ = *(hex_val + 1);

   if ((*buf < ' ') || (*buf > '~'))
      *buf = '.';

   *out_char++ = (char) *buf;
   }

if (width > 0)
   {
   for (; width<max_byte; width++,out_char++,out_hex+=2)
      {
      if (width % 4 == 0)
         out_hex++;

      *out_char = *out_hex = *(out_hex + 1) = ' ';
      }

   sprintf(out_char+2,"%06d",max_byte * total);
   c_pterm(out_buf,&flag);
   }

flag = 1;
c_pterm(" ",&flag);

return;
}
