/******************************************************************************
FUNCTION:	PROC_DISK

PURPOSE:	Forward spaces the specified number of files on the
		specified disk file.  This is done by updating the
		attribute "cur_rec" and "cur_file" associated with
		the disk file.  Error checking is done to sure the
		disk files exist.

ALGORITHM 
  Retrieve current disk attibutes
  Set current record to zero
  Forward space the number of disk files
  Update the disk attributes

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_disk
(
    char *which,			/* DITTO input/output device         */
    long count			/* number records to be back spaced  */
)
{
struct DISK_FD fd;			/* disk file attributes              */
long	index;				/* loop counter                      */
long	status;				/* function return status code       */
long    fatal = -1;			/* Fatal error code		     */

/*
  Retrieve current disk attibutes
*/
status = get_disk_fd(which,&fd);
if (status != E_SUCC)
    c_errmsg("fatal error encountered","ditto fsf-fatal",&fatal);

status = dstat(&fd,0);
if (status != E_SUCC)
    {
    ditto_error(which,DITTO_DSTAT,ERROR_WARN,NO_UPDATE_FD);
    c_errmsg("Fatal error encountered","ditto fsf-fatal",&fatal);
    }

/*
  Set current record to zero
*/
fd.cur_rec = 0;

/*
  Forward space the number of disk files
*/
if (fd.cur_file == NOVAL)
   ditto_error(which,DITTO_FSF,ERROR_WARN,NO_UPDATE_FD);
else
   {
   for (index=0; index<count; index++,fd.cur_file++)
      {
      if (dstat(&fd,0) == E_FAIL)
         {
         ditto_error(which,DITTO_FSF,ERROR_WARN,NO_UPDATE_FD);
	 break;
	 }
      }
   }

/*
  Update the disk attributes
*/
put_disk_fd(which,&fd);			/* update disk attributes            */

return;
}
