/******************************************************************************
FUNCTION:	PROC_TAPE

PURPOSE:	Forward spaces the specified number of files on the
		specified magnetic tape.  This is done by calling the
		tape I/O routine "tpfsf" and updating the attribute
		"cur_rec" and "cur_file" associated with the tape.

ALGORITHM 
  Reopen the tape device
  Forward space specified number of records
  Set current record to zero
  Update record and file attibutes for the tape device
  Close the tape device

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_tape
(
    char *which,			/* DITTO input/output device         */
    long count			/* number of files to be back spaced */
)
{
struct TAPE_FD  tape;			/* tape descriptor		     */

long mode = HANDLE;			/* tape package outputs error msg    */
long nonfatal = 1;			/* Non-fatal error code		     */
long fatal = -1;			/* Fatal error code		     */
long status;				/* function return status code       */
long temp;				/* save number of files to be skipped*/

char msg[ERRLEN];			/* error message to be outputted     */

temp = count;

/*
  Reopen the tape device
*/
status = reopen_tape(which,NOREW,TREAD,&tape);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto fsf-fatal",&fatal);

/*
  Forward space specified number of records
*/
status=c_tpfsf(&tape.unit,&count,&mode);

/*
  Set current record to zero
*/
tape.cur_rec = 0;

/*
  Update record and file attibutes for the tape device
*/
if (tape.cur_file != NOVAL)
   tape.cur_file += count;

switch(status)
   {
   case TAPE_SUCC:	break;

   case TAPE_EOV: 	
	if (count != temp)
  	   {
   	   sprintf(msg,"Forward spaced %d files",count);
   	   c_errmsg(msg," ",&nonfatal);
	   ditto_error(which,DITTO_FSF,ERROR_WARN,NO_UPDATE_FD);
	   }
	break;

   default: 
   	sprintf(msg,"Forward spaced %d files",count);
   	c_errmsg(msg," ",&nonfatal);
	ditto_error(which,DITTO_FSF,ERROR_WARN,NO_UPDATE_FD);
	break;
   }

/*
  Close the tape device
*/
close_tape(which,&tape,NOREW,1);

return;
}
