/******************************************************************************
FUNCTION:	PROC_DISK

PURPOSE:	Forward space records on disk file

ALGORITHM 
  Retrieve disk file attributes
  Insure disk file exists
  Open the disk file
  Forward skip the specified number of records
  Update disk file attributes
  Write attributes
  Close the disk file

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_disk
(
    char *which,			/* DITTO input/output device         */
    long count			/* number of files to be back spaced */
)
{
struct DISK_FD disk_fd;			/* DITTO disk file descriptor        */
register long  status;			/* function return status code       */
long fatal = -1;			/* Fatal error code		     */

/*
  Retrieve disk file attributes
*/
status = get_disk_fd(which,&disk_fd);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto fsr-fatal",&fatal);

/*
  Insure disk file exists
*/
status = dstat(&disk_fd,0);
if (status == E_FAIL)
   {
   ditto_error(which,DITTO_DSTAT,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto fsr-fatal",&fatal);
   }

/*
  Open the disk file
*/
status = dopen(&disk_fd,DREAD);
if (status == E_FAIL)
   {
   ditto_error(which,DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto fsr-fatal",&fatal);
   }

/*
  Forward skip the specified number of records
*/
status = dskip(&disk_fd,&count);
if (status == E_FAIL)
   ditto_error(which,DITTO_FSR,ERROR_WARN,NO_UPDATE_FD);

/*
  Update disk file attributes
*/
disk_fd.cur_rec += count;

/*
  Write attributes
*/
put_disk_fd(which,&disk_fd);

/*
  Close the disk file
*/
status = dclose(&disk_fd);
if (status != E_SUCC)
    ditto_error(" ",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);

return;
}
