/******************************************************************************
FUNCTION:	PROC_TAPE

PURPOSE:	Forward space records on tape

ALGORITHM 
  Reopen tape device
  Forwad space the specified number of records
	Update tape attribute current record
  Close tape device

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file 	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_tape
(
    char *which,			/* DITTO input/output device         */
    long count			/* number of files to be back spaced */
)
{
struct TAPE_FD  tape;			/* tape descriptor		     */
long mode = HANDLE;			/* tape I/O output error messages    */
long nonfatal = 1;			/* Non-fatal error code		     */
long fatal = -1;			/* Fatal error code		     */
long status;				/* function return status code       */

char msg[ERRLEN];			/* error message to be outputted     */

/*
  Reopen tape device
*/
status = reopen_tape(which,NOREW,TREAD,&tape);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto fsr-fatal",&fatal);

/*
  Forwad space the specified number of records
*/
status=c_tpfsr(&tape.unit,&count,&mode);
if (tape.cur_rec != NOVAL)		/* update tape attribute current     */
   tape.cur_rec += count; 		/* record	                     */

if (status != TAPE_SUCC)
   {
   sprintf(msg,"Forward spaced %d records",count);
   c_errmsg(msg," ",&nonfatal);
   ditto_error(which,DITTO_FSR,ERROR_WARN,NO_UPDATE_FD);
   }

/*
  Close tape device
*/
close_tape(which,&tape,NOREW,1);

return;
}
