/******************************************************************************
FUNCTION:	MAP_DISK

PURPOSE:	Does a mapper on the specifed disk file.  If the generated
		output is to be printed on the line printer, it writes the
		output to the disk file "mapper.prt".

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <string.h>
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void map_disk
(
    char *which,			/* DITTO input/output device         */
    long *print,			/* print destination                 */
    FILE *lp_fp			/* file pointer to line printer      */
)
{
struct DISK_FD disk_file;		/* structure with disk file info     */

long   last = 0;			/* record length of previous record  */
long   nonfatal = 1;			/* Non-fatal error code		     */
long   fatal = -1;			/* Fatal error code		     */
long   nbyte;				/* number of bytes in current record */
long   rec;				/* count of records with same length */
long   status;				/* function return status code       */
long   totrec;				/* total # of records read in file   */
long  trectape=0;                       /* total number of records in volume */

unsigned char *bufptr;			/* pointer to buffer to read into    */

char msg[ERRLEN];			/* buffer for generated output       */
char file_name[CMLEN];			/* File name			     */
char ext[CMLEN];			/* Extention location		     */
char *adir;				/* address of directory within fname */
char *aext;				/* address of directory within fname */

/*
*   Retrieve disk file attributes, open the disk file, and allocate buffer
*   space to read physical records from the disk file.
*/
status = get_disk_fd(which,&disk_file);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto mapper-fatal",&fatal);

disk_file.cur_rec = 0;

/*
*   Read physical records from the disk file until the end of file is reached.
*   Two counters keep track of the number of consecutive records with the same
*   record size, and to keep track of the total number of records.  The 
*   output is written to the terminal, line printer (file "mapper.prt"), or
*   both depending on input parameter "print".
*/
for (disk_file.cur_file=1; ; disk_file.cur_file++)
   {
   if (dstat(&disk_file,0) == E_FAIL)
      break;

/*
  Check the current file and append it to the file name
*/
   strcpy(file_name,disk_file.fname);
   if (disk_file.cur_file != NOVAL)
	{
	adir = strrchr(file_name,HOST_DIR_END);
	if (adir == 0)
	    adir = file_name;
	aext = strchr(adir,HOST_EXT);

	if (disk_file.cur_file == NOVAL)
	    disk_file.cur_file = 1;

	if (aext == 0)
	    sprintf(file_name,"%s_%d",disk_file.fname,disk_file.cur_file);
	else
	    {
	    strcpy(ext,aext);
	    sprintf(aext,"_%d%s",disk_file.cur_file,ext);
	    }
	}

   sprintf(msg,"********** Mapper of disk file %s **********",file_name);
   outmsg(msg,print,lp_fp);

   if (dopen(&disk_file,DREAD) == E_FAIL)
      {
      ditto_error(which,DITTO_DOPEN,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto mapper-fatal",&fatal);
      }

   if ((bufptr = malloc((disk_file.blksiz + 1))) == NULL)
      {
      ditto_error(which,DITTO_CALLOC,ERROR_WARN,UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto mapper-fatal",&fatal);
      }

   for (rec=0,totrec=0;;totrec++)
      {
      nbyte = dread(&disk_file,bufptr,disk_file.blksiz);
      if (nbyte == E_FAIL)
         {
         sprintf(msg,"Read %d records",totrec);
	 c_errmsg(msg,"ditto mapper-dread",&nonfatal);
         ditto_error(which,DITTO_DREAD,ERROR_VERIFY,UPDATE_FD);
         }

      if (nbyte == 0)
         break;

      if ((nbyte != last) && (rec != 0))
         {
         sprintf(msg,"  %6d RECORDS %6d BYTES LONG",rec,last);
         outmsg(msg,print,lp_fp);
         rec = 0;
         }

      ++rec;
      last = nbyte;
      }
/*
   Total the records in volume 
*/
   trectape =trectape + totrec;

   sprintf(msg,"  %6d RECORDS %6d BYTES LONG",rec,last);
   outmsg(msg,print,lp_fp);

   sprintf(msg,"END OF FILE  >>>>> %d TOTAL RECORDS.",totrec);
   outmsg(msg,print,lp_fp);
   outmsg(" ",print,lp_fp);
   outmsg(" ",print,lp_fp);
   outmsg(" ",print,lp_fp);

   free(bufptr);
   status = dclose(&disk_file);
   if (status == E_FAIL)
      ditto_error(which,DITTO_DCLOSE,ERROR_WARN,UPDATE_FD);

   if (disk_file.cur_file == NOVAL)
      break;
   }


strcpy(msg,"**********  END OF VOLUME  **********");
outmsg(msg,print,lp_fp);
outmsg(" ",print,lp_fp);
outmsg(" ",print,lp_fp);

sprintf(msg,"--------------------------------");
outmsg(msg,print,lp_fp);
sprintf(msg,"       %d RECORDS IN VOLUME.",trectape);
outmsg(msg,print,lp_fp);

/*  
*   Update the disk file attributes, close the disk file, and deallocate the
*   buffer space.
*/
if (disk_file.cur_file != NOVAL)
   disk_file.cur_file = 1;

put_disk_fd(which,&disk_file);

return;
}
