/******************************************************************************
FUNCTION:	PROC_TAPE

PURPOSE:	Rewinds the specified tape device.  This is done by calling
		the tape I/O routine "tpbsf" and updating the attribute
		"cur_rec" and "cur_file" associated with the tape.

ALGORITHM 
  Reopen the tape device
  Rewind the tape
  Update the record and file attributes for the tape device
  Close the tape device (no rewind)

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void proc_tape
(
    char *which			/* DITTO input/output device         */
)
{
struct TAPE_FD  tape;			/* tape descriptor		     */

long   mode = HANDLE;			/* tape package outputs error msg    */
long   rewopt = REWIND;			/* rewind option                     */
long   fatal = -1;			/* Fatal error code		     */
long   status;				/* function return status code       */

/*
  Reopen the tape device
*/
status = reopen_tape(which,NOREW,TREAD,&tape);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto rewind-fatal",&fatal);

/*
  Rewind the tape
*/
status = c_tprew(&tape.unit,&mode,&rewopt);
if (status != TAPE_SUCC)
   ditto_error(which,DITTO_TPREW,ERROR_WARN,NO_UPDATE_FD);

/*
  Update the record and file attributes for the tape device
*/
if (status == TAPE_SUCC)
   {
   tape.cur_rec = 0;
   tape.cur_file = 1;
   }
else
   {
   tape.cur_rec = NOVAL;
   tape.cur_file = NOVAL;
   }

/*
  Close the tape device (no rewind)
*/
close_tape(which,&tape,NOREW,1);

return;
}
