/******************************************************************************

FUNCTION:	REWIND

PURPOSE: 	Part of the DITTO program which rewinds the specified
		DITTO device.  

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      11/85    K. Gacke		initial development
  1.1	    7/86    K. Gacke		LAS standards
  5.0      10/88    D. Hollaren		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	The parameters are inputted into the program.  From parameter WHICH,
	it is determined which device is to be processed.  The command to 
	rewind the device is issued.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"
 
struct PARBLK   vblock;		/* TAE parameter block               */
struct DITTO_DEV ditto_device;	/* ditto I/O devices descriptors     */

int main
(void)
{
struct VARIABLE *val;			/* pointer to TAE variable           */

long	std_out;			/* Fortran LUN for standard output   */
long	zero = 0;			/* value of zero                     */
long    nonfatal = 1;			/* Non-fatal error code		     */
long    fatal = -1;			/* Fatal error code		     */

char	out[1][CMLEN];			/* output files for completion msg   */
char	*device;			/* DITTO device (tape or disk)       */
char	*which;				/* DITTO input/output device         */

/*
  Initialize TAE
*/
c_inlas(&vblock,&std_out);

/*
  Set input and output devices to no value
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Get DITTO device to be processed
*/
val = p_find(&vblock,"devtype");
which = SVAL(*val,0);

/*
  Rewind the input device
*/
if ((strcmp(which,"INPUT") == 0) || (strcmp(which,"BOTH") == 0))
   {
   c_errmsg("Rewinding input DITTO device"," ",&nonfatal);

   /*
     Get the input device
   */
   val = p_find(&vblock,"idevice");
   if (val->v_count == 0)
      {
      ditto_error("INPUT",DITTO_ALLOC,ERROR_WARN,NO_UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto rewind-fatal",&fatal);
      }
   device = SVAL(*val,0);

   if (strcmp(device,"DISK") == 0)	
      proc_disk(&vblock,"INPUT");	/* rewind disk file                  */
   else
      proc_tape("INPUT");		/* rewind tape                       */
   }

/*
  Rewind the output device
*/
if ((strcmp(which,"OUTPUT") == 0) || (strcmp(which,"BOTH") == 0))
   {
   c_errmsg("Rewinding output DITTO device"," ",&nonfatal);

   /*
     Get the output device
   */
   val = p_find(&vblock,"odevice");
   if (val->v_count == 0)
      {
      ditto_error("OUTPUT",DITTO_ALLOC,ERROR_WARN,NO_UPDATE_FD);
      c_errmsg("Fatal error encountered","ditto rewind-fatal",&fatal);
      }
   device = SVAL(*val,0);

   if (strcmp(device,"DISK") == 0)	
      proc_disk(&vblock,"OUTPUT");	/* rewind disk file                  */
   else
      proc_tape("OUTPUT");		/* rewind tape                       */
   }

c_complt("DITTO REWIND",out,&zero);

return EXIT_SUCCESS;

}
