/******************************************************************************
FUNCTION:	CLOSE_DEVICE

PURPOSE: 	Closes the input or output device that is being processed.
		It also updates the attributes of each device (i.e. current
		record and current file.

ALGORITHM 
  If device is a disk file 
     Update current record and file attributes
     Update the attribute structure
     Close the disk file
  Else device is a tape
     Update the current record and file attributes
     Close the tape device (No rewind)
ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

void close_device
(
    char *which,			/* which device (input or output)    */
    char *device,			/* device type  (tape or disk)       */
    int currec,
    long bytes_read			/* bytes read on last read           */
)
{
extern struct DISK_FD disk;		/* disk file descriptor              */
extern struct TAPE_FD tape;		/* tape descriptor                   */
long     status;			/* function return status code       */

/*
  If device is a disk file then update current record and file attributes
*/
if (strcmp(device,"DISK") == 0)
   {
   if ((bytes_read == 0) && (disk.cur_file != NOVAL))
      {
      disk.cur_file++;
      disk.cur_rec = 0;
      }
   else if (disk.cur_rec != NOVAL)
      disk.cur_rec += currec;

   /*
     Update the attribute structure
   */
   put_disk_fd(which,&disk);

   /*
     Close the disk file
   */
   status = dclose(&disk);
   if (status != E_SUCC)
       ditto_error(" ",DITTO_DCLOSE,ERROR_WARN,NO_UPDATE_FD);
   }
else				/* Device is a tape		*/
   {
   /*
     Update the current record and file attributes
   */
   if (bytes_read == 0)
      {
      if (tape.cur_file != NOVAL)
	 tape.cur_file++;

      tape.cur_rec = 0;
      }
   else if (tape.cur_rec != NOVAL)
      tape.cur_rec += currec;

   /*
     Close the tape device (No rewind)
   */
   close_tape(which,&tape,NOREW,1);
   }

return;
}
