/******************************************************************************
FUNCTION:	COMPUTE_SCAN

PURPOSE:	Converts the character/hexidecimal scan string into a byte
		array.

ALGORITHM 
  If character format
     copy the scan string into byte valued array
  Else hex format
     Determine the length of the scan line and allocate space
     For each pair of ascii hex digits
	call HEX2BYT to convert the digit into a byte value
  Return byte value array

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

unsigned char *compute_scan
(
    long *scan_length			/* # of bytes in scan is returned    */
)
{
extern char *scan;			/* scan string in char or hex format */
extern char *type;			/* specifies format of scan string   */

long	index;				/* loop index                        */
long	length;				/* length of scan string	     */

unsigned char   strhex[3];		/* char hex to be converted to byte  */
unsigned char	*temp;			/* temporary pointer                 */
unsigned char   *scan_string;		/* byte array scan string            */

/*
  If character format, copy the scan string into byte valued array
*/
if (strcmp(type,"CHAR") == 0)
   {
   *scan_length = (long) strlen(scan);
   scan_string = (unsigned char *) malloc(*scan_length);
   strcpy((char *) scan_string,scan);
   }
else				/* hexidecimal format			*/
   {
   /*
     Determine the length of the scan line and allocate space
   */
   length = (long) strlen(scan);
   if (length % 2 != 0)
      {
      (length)++;
      temp = (unsigned char *) malloc(length);
      sprintf((char *)temp,"0%s",scan);
      }
   else
      {
      temp = (unsigned char *) malloc(length);
      strcpy((char *) temp,scan);
      }

   *scan_length = length / 2;
   scan_string = (unsigned char *) malloc(*scan_length);
   strhex[2] = '\0';

   /*
     For each pair of ascii hex digits, call HEX2BYT to convert the digit
     into a byte value
   */
   for (index = 0; index < *scan_length; index++)
      {
      strhex[0] = *temp++;
      strhex[1] = *temp++;
      hex2byt((char *) strhex,&scan_string[index]);
      }
   }

/*
  Return byte value array
*/
return(scan_string);
}
