/******************************************************************************
FUNCTION:	READ_DEVICE

PURPOSE: 	Reads one physical record from appropriate DITTO device.

ALGORITHM 
  If device is a disk file
     Read one physical record from the disk file
  Else device is a tape
     Read one physical record from the tape
  Return the number of bytes read

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

int read_device
(
    char *device,			/* device type (tape or disk)        */
    int currec,
    unsigned char *buf			/* buffer to read record into        */
)
{
extern struct DISK_FD disk;		/* disk file descriptor              */
extern struct TAPE_FD tape;		/* tape descriptor                   */

long mode = HANDLE;			/* tape I/O outputs error messages   */
long one = 1;				/* value of one                      */
long nonfatal = 1;			/* Non-fatal error code		     */
long nbyte;				/* number of bytes read              */

/*
  Device is a disk file
*/
if (strcmp(device,"DISK") == 0)
   {
   /*
     Read one physical record from the disk file
   */
   nbyte = dread(&disk,buf,disk.blksiz);
   if (nbyte == E_FAIL)
      proc_error(device,currec,DITTO_DREAD);
   }
else				/* Device is tape			*/
   {
   /*
     Read one physical record from the tape
   */
   nbyte = c_tpread(&tape.unit,buf,&tape.blksiz,&mode);
   switch (nbyte)
      {
      case TAPE_EOF:
	c_errmsg("End of File detected","ditto scan-eof",&nonfatal);
	nbyte = 0;
	break;

      case TAPE_EOV:
	c_errmsg("End of Volume detected","ditto scan-eov",&nonfatal);
	nbyte = 0;
        c_tpbsr(&tape.unit,&one,&mode);
	break;

      case TAPE_EOT:
	c_errmsg("End of Tape detected","ditto scan-eot",&nonfatal);
	nbyte = 0;
	break;
      }

   if (nbyte < 0)
      proc_error(device,currec,DITTO_TPREAD);
   }

/*
  Return the number of bytes read
*/
return(nbyte);
}
