/******************************************************************************
FUNCTION:	SCANBUF

PURPOSE:	Scans the record for matches.  If a match is found the
		record number and byte offset are outputted to the
		appropriate destination (terminal, line printer, or both).
		Multiple matches within the same record are found.

ALGORITHM 
  Determine the number of bytes to be scaned
     Look for specified scan string in the buffer
        Match found
	    write info to the printer, file, or terminal
  Return the number of matches found

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file	     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

#define TERM	0			/* print report to terminal screen   */
#define FIL	1			/* print report to disk file         */
#define LP	2			/* print report to line printer      */

int scanbuf
(
    unsigned char *buf,	/* buffer containing the record      */
    long start,		/* starting offset to begin scan     */
    long recsiz,		/* number of bytes in record         */
    long currec,		/* current record being processed    */
    unsigned char *scan_string, /* scan string to be searched for  */
    long scan_length,	/* number of bytes in scan string    */
    long *print			/* print destination (term,lp,both)  */
)
{
extern FILE *lp_fp;			/* file pointer to line printer file */
long   count;				/* byte offset of a match            */
long   match;				/* number of bytes that have matched */
long   nbytes;				/* actual # of bytes to be scanned   */
long   nmr_matches = 0;			/* # of matches found within record  */
char  out_msg[100];			/* message to be outputted           */

/*
  Determine the number of bytes to be scaned
*/
nbytes = recsiz - scan_length + 1;
for (buf += start,count = start; count < nbytes; buf++,count++)
   {
   /*
     Look for specified scan string in the buffer
   */
   if (*buf == *scan_string)
      {
      for (match = 1;
	   (match < scan_length) && (*(buf + match) == *(scan_string + match));
 	   match++)
		;

      /*
        Match found, write info to the printer, file, or terminal
      */
      if (match == scan_length)
         {
         sprintf(out_msg,"Match found in record %4d  ----  byte offset %4d",
                 currec,count);

	 outmsg(out_msg,print,lp_fp);
         nmr_matches++;
         }
      }
   }

/*
  Return the number of matches found
*/
return(nmr_matches);
}
