/******************************************************************************
FUNCTION:	PROC_OUTPUT

PURPOSE:	Reads the parameters of the output DITTO device, formats them 
		and outputs the information to the terminal screen.

ALGORITHM 
  Retrieve the output device
  Retrieve the output host name
  Retrieve the output record type
  Retrieve the output carrage control
  Retrieve the output record size
  Retrieve the output block size
  Retrieve the output record
  Retrieve the output file
  Retrieve the output density
  If output device allocated to a magnetic tape 
	output status in tape format
	(Information includes the drive number, tape density, record size,
	 block size, record type, type of carriage control, current file
	 position, and current record position within the file.)
  If output device allocated to a disk file
	 output status in disk file format
 	 (Information includes the file name, record size, block size, 
	  record type, type of carriage control, and current record
	  position within the file.)

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "ditto.h"			/* ditto include file                */
#include "protos.h"

#define TERM	0			/* index for terminal		     */
#define FIL	1			/* index for file		     */
#define LP	2			/* index for line printer	     */

void proc_output
(void)
{
extern struct PARBLK vblock;		/* TAE parameter block               */
extern long print[3];			/* Flag for term, lp, or file output */

struct VARIABLE *val;			/* pointer to TAE variable           */

long  blksiz;				/* block size                        */
long  cur_file;				/* current file                      */
long  cur_rec;				/* current record                    */
long  line = 10;			/* terminal line number              */
long  recsiz;                            /* record size                       */
char *cc;				/* type of carriage control          */
char *device;				/* DITTO I/O device (tape or disk)   */
char *host;				/* DITTO host (drive or file name)   */
char *rectyp;				/* record type (fixed or variable)   */
char msg[200];				/* storage buffer for messages       */
char density[5];			/* tape density                      */
char tapeid[CMLEN];

/*
  Retrieve the output device
*/
val = p_find(&vblock,"odevice");
if (val->v_count == 0)
   {
   if (print[LP] || print[FIL])
	prt_term(" ",&line);

   sprintf(msg,"DITTO output device is not currently allocated");
   if (print[TERM])
	t_highlight(msg);

   prt_term(msg,&line);
   return;
   }

device = SVAL(*val,0);

/*
  Retrieve the output host name
*/
val = p_find(&vblock,"ohost");
host = SVAL(*val,0);

/*
  Retrieve the output record type
*/
val = p_find(&vblock,"orectyp");
if (val->v_count == 0)
   rectyp = "default for current file";
else
   rectyp = SVAL(*val,0);

/*
  Retrieve the output carrage control
*/
val = p_find(&vblock,"occ");
if (val->v_count == 0)
   cc = "undefined";
else
   cc = SVAL(*val,0);

/*
  Retrieve the output record size
*/
val = p_find(&vblock,"orecsiz");
if (val->v_count == 0)
   recsiz = NOVAL;
else
   recsiz = IVAL(*val,0);

/*
  Retrieve the output block size
*/
val = p_find(&vblock,"oblksiz");
if (val->v_count == 0)
   blksiz = NOVAL;
else
   blksiz = IVAL(*val,0);

/*
  Retrieve the output record
*/
val = p_find(&vblock,"orec");
if (val->v_count == 0)
   cur_rec = NOVAL;
else
   cur_rec = IVAL(*val,0);

/*
  Retrieve the output file
*/
val = p_find(&vblock,"ofile");
if (val->v_count == 0)
   cur_file = NOVAL;
else
   cur_file = IVAL(*val,0);

/*
  Retrieve the output density
*/
val = p_find(&vblock,"odensity");
if (val->v_count == 0)
   strcpy(density,"0");
else
   strcpy(density,SVAL(*val,0));

val = p_find(&vblock,"otapeid");
if (val->v_count == 0)
   strcpy(tapeid,"0");
else
   strcpy(tapeid,SVAL(*val,0));

/*
  If output device allocated to a magnetic tape then output status in tape
  format.  Information includes the drive number, tape density, record size,
  block size, record type, type of carriage control, current file position,
  and current record position within the file.
*/
if (strcmp(device,"TAPE") == 0)	
   {
   if (print[LP] || print[FIL])
	prt_term(" ",&line);

   sprintf(msg,"DITTO output device is allocated to TAPE drive %s",host);
   if (print[TERM])
	t_highlight(msg);

   prt_term(msg,&line);

   sprintf(msg,"     Tape Density     = %s",density);
   prt_term(msg,&line);

   if (recsiz == NOVAL)
      sprintf(msg,"     Record Size      = default for current file");
   else
      sprintf(msg,"     Record Size      = %d",recsiz);
   prt_term(msg,&line);

   if (blksiz == NOVAL)
      sprintf(msg,"     Block Size       = default for current file");
   else
      sprintf(msg,"     Block Size       = %d",blksiz);
   prt_term(msg,&line);

   sprintf(msg,"     Record Type      = %s",rectyp);
   prt_term(msg,&line);

   sprintf(msg,"     Carriage Control = %s",cc);
   prt_term(msg,&line);

   if (cur_file == NOVAL)
      sprintf(msg,"     Current File     = unknown");
   else
      sprintf(msg,"     Current File     = %d",cur_file);
   prt_term(msg,&line);

   if (cur_rec == NOVAL)
      sprintf(msg,"     Current Record   = unknown");
   else
      sprintf(msg,"     Current Record   = %d",cur_rec);
   prt_term(msg,&line);
      sprintf(msg,"     Tapeid           = %s",tapeid);
   prt_term(msg,&line);
   }

/*
  If output device allocated to a disk file then output status in disk file
  format.  Information includes the file name, record size, block size, 
  record type, type of carriage control, and current record position within 
  the file.
*/
else
   {
   if (print[LP] || print[FIL])
	prt_term(" ",&line);

   sprintf(msg,"DITTO output device is allocated to DISK file");
   if (print[TERM])
	t_highlight(msg);

   prt_term(msg,&line);

   sprintf(msg,"     File Name        = %s",host);
   prt_term(msg,&line);

   if (recsiz == NOVAL)
      sprintf(msg,"     Record Size      = default for current file");
   else
      sprintf(msg,"     Record Size      = %d",recsiz);
   prt_term(msg,&line);

   if (blksiz == NOVAL)
      sprintf(msg,"     Block Size       = default for current file");
   else
      sprintf(msg,"     Block Size       = %d",blksiz);
   prt_term(msg,&line);

   sprintf(msg,"     Record Type      = %s",rectyp);
   prt_term(msg,&line);

   sprintf(msg,"     Carriage Control = %s",cc);
   prt_term(msg,&line);

   if (cur_file == NOVAL)
      sprintf(msg,"     Current File     = 1");
   else
      sprintf(msg,"     Current File     = %d",cur_file);
   prt_term(msg,&line);

   if (cur_rec == NOVAL)
      sprintf(msg,"     Current Record   = unknown");
   else
      sprintf(msg,"     Current Record   = %d",cur_rec);
   prt_term(msg,&line);
   }

return;
}
