/******************************************************************************
 
FUNCTION:	STATUS

PURPOSE: 	Part of the DITTO subsystem.  The status function
		outputs to the terminal the current status of the
		DITTO input and output devices.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0       1/86    K. Gacke		initial development
  1.1	    7/86    K. Gacke		LAS standards
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1       1/90    B. Davis		make.com refered to [-.lib]ditto/olb
					instead of WORLIB:ditto/olb
  5.2	    9/92    T. Mittan		Made use of tapeid to print tapeid
					with report.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	The parameters for the input DITTO device are read from the TAE 
	parameter block and outputted to the terminal screen.  The parameters 
	for the output DITTO device are read from the TAE parameter block and 
	outputted to the terminal screen.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include file 		     */
#include "util.h"
#include "ditto.h"			/* ditto include file                */
#include "protos.h"
 
#define TERM	0			/* index for terminal		     */
#define FIL	1			/* index for file		     */
#define LP	2			/* index for line printer	     */

struct PARBLK   vblock;			/* TAE parameter block               */
struct DITTO_DEV ditto_device;		/* ditto I/O devices descriptors     */
long print[3];				/* Flag for term, lp, or file output */
FILE      *lp_fp;			/* file pointer to line printer file */

int main
(void)
{

long status;				/* function return status code       */
long std_out;				/* Fortran LUN for standard output   */
long outcnt = 0;			/* Number of output files	     */
long fatal = -1;			/* Fatal error code		     */
long nonfatal = 0;			/* Non-fatal error code		     */

static char out[1][CMLEN];		/* output files for completion msg   */
static char prtfile[CMLEN];		/* Printer working file		     */
static char filename[CMLEN];		/* Output print file name	     */

/*
  Initailize TAE
*/
c_inlas(&vblock,&std_out);

/*
  Get the working output print file
*/
strcpy(prtfile,"status");
status = c_setprt(&vblock,&print[0],&print[1],&print[2],prtfile,filename);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto status-fatal",&fatal);

/*
  Open the temporary print file if going to the line printer or a file
*/
if (print[LP] || print[FIL])
    {
    if ((lp_fp = fopen(prtfile,"a+")) == NULL)
	{
	c_errmsg("Error opening the print file","ditto status-open",&nonfatal);
	}

    /*
      Hold onto the print host file name so clean up can be done when an
      error occurs
    */
    strcpy(out[0],prtfile);
    outcnt=1;
    }

/*
  Set input and output devices to no value
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Process the input DITTO device
*/
proc_input();

/*
  Process the output DITTO device
*/
proc_output();

/*
  Position cursor on terminal screen
*/
if (print[TERM])
    t_pos(21,1);

/*
  Add the output print file to the list of created files.
*/
if (print[FIL])
    {
    strcpy(out[0],filename);
    outcnt = 1;
    }
else
    outcnt=0;


c_complt("DITTO STATUS",out,&outcnt);

return EXIT_SUCCESS;

}
