/******************************************************************************
FUNCTION:	PROC_ERROR

PURPOSE: 	Processes error conditions.  It saves the old DITTO
		input/output attributes, update the attributes to current 
		processing point, and calls ditto_error to output the
		error message and to determine if processing should
		continue.  If control is returned from ditto_error,
		the old attributes are reassigned.

ALGORITHM 

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"			/* ditto include file                */
#include "verify.h"
#include "protos.h"

void proc_error
(
    struct VERIFY_DEV *indev,		/* input device descriptor           */
    struct VERIFY_DEV *outdev,		/* output device descriptor          */
    char *which,			/* which device (input or output)    */
    long error_code			/* DITTO error code encountered      */
)
{
extern struct DISK_FD idisk;		/* input disk file descriptor        */
extern struct DISK_FD odisk;		/* ouput disk file descriptor        */
extern struct TAPE_FD itape;		/* input tape descriptor	     */
extern struct TAPE_FD otape;		/* output tape descriptor	     */

long   old_in;				/* old input current record	     */
long   old_out;				/* old output current record	     */
long   *inrec;				/* pointer to input current record   */
long   *outrec;				/* pointer to output current record  */

if (indev->device == DISK)
   inrec = &idisk.cur_rec;
else
   inrec = &itape.cur_rec;

if (outdev->device == DISK)
   outrec = &odisk.cur_rec;
else
   outrec = &otape.cur_rec;

old_in = *inrec;
old_out = *outrec;
if (*inrec != NOVAL)
   *inrec += indev->cur_rec;

if (*outrec != NOVAL)
   *outrec += outdev->cur_rec;

ditto_error(which,error_code,ERROR_VERIFY,UPDATE_FD);
*inrec = old_in;
*outrec = old_out;

return;
}
