/******************************************************************************
FUNCTION:	RECORD_VERIFY

PURPOSE:	Verifies the specified number of records of a single file on
		the input and output DITTO devices.

ALGORITHM 
  Retrieve the number of records
  Retrieve message time
  Open the input and output device
  Verify the devices
  Close the input and output devices

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file                  */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"  			/* ditto include file                */
#include "verify.h"
#include "protos.h"

void record_verify
(
    char *idevice,			/* Pointer to input device	*/
    char *odevice			/* Pointer to output device	*/
)
{
extern struct PARBLK vblock;		/* TAE variable block		*/
struct VARIABLE   *val;			/* Pointer to parameter		*/
struct VERIFY_DEV indev;		/* Input device structure	*/
struct VERIFY_DEV outdev;		/* Output device structure	*/

long  msgtim;				/* Message time			*/
long  nrecords;				/* Number of records		*/

/*
  Retrieve the number of records
*/
val = p_find(&vblock,"nrecs");
if (val->v_count > 0)
   nrecords = IVAL(*val,0);
else
   nrecords = ALL;

/*
  Retrieve message time
*/
val = p_find(&vblock,"$msgtim");
msgtim = IVAL(*val,0);

/*
  Open the input and output device
*/
open_device(idevice,odevice,&indev,&outdev);

/*
  Verify the devices
*/
verify_proc(&indev,&outdev,nrecords,msgtim); 

/*
  Close the input and output devices
*/
close_device(&indev,&outdev);

return;
}
