/******************************************************************************

FUNCTION:	VERIFY

PURPOSE:	Part of the DITTO subsystem.  The verify function compares
		a specified number of records or files from the current
		position of the input DITTO device to the current position
		of the output DITTO device.  Multiple files can only be
		compared if both the input and output DITTO devices are
		allocated to magnetic tapes.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      11/85    K. Gacke		initial development
  1.1       7/86    K. Gacke		LAS standards
  5.0      10/88    D. Hollaren		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	The DITTO input devices are opened for read access.  The specified
	number of records are read from each device.  Each record is compared
	byte for byte.  If a difference is found an error message is outputted.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <stdlib.h>
#include "las.h"			/* LAS include file                  */
#include "util.h"
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h" 			/* ditto include file                */
#include "verify.h"
#include "protos.h"
 
struct PARBLK   vblock;		/* TAE parameter block               */
struct DITTO_DEV ditto_device;	/* ditto I/O devices descriptors     */
struct DISK_FD idisk;		/* input disk file descriptor        */
struct DISK_FD odisk;		/* output disk file descriptor       */
struct TAPE_FD itape;		/* input tape descriptor             */
struct TAPE_FD otape;		/* output tape descriptor            */

int main
(void)
{
struct VARIABLE *val;			/* pointer to TAE variable           */

long std_out;				/* Fortran LUN for standard output   */
long zero = 0;				/* value of zero                     */
long fatal = -1;			/* Fatal error code		     */

char	out[1][CMLEN];			/* output files for completion msg   */
char    idevice[20];			/* input device (tape or disk)       */
char    odevice[20];			/* output device (tape or disk)      */
char	*subcmd;			/* subcommand being executed         */

/*
  Initialize TAE
*/
c_inlas(&vblock,&std_out);

/*
  Set input and output devices to no value
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Retrieve the input device
*/
val = p_find(&vblock,"idevice");
if (val->v_count == 0)
   ditto_error("INPUT\0",DITTO_ALLOC,ERROR_SEV,NO_UPDATE_FD);
strcpy(idevice,SVAL(*val,0));

/*
  Retrieve the output device
*/
val = p_find(&vblock,"odevice");
if (val->v_count == 0)
   {
   ditto_error("OUTPUT\0",DITTO_ALLOC,ERROR_WARN,NO_UPDATE_FD);
   c_errmsg("Fatal error encountered","ditto verify-fatal",&fatal);
   }
strcpy(odevice,SVAL(*val,0));

/*
  Retrieve the subcommand
*/
val = p_find(&vblock,"_SUBCMD");
subcmd = SVAL(*val,0);

if (strcmp(subcmd,"FILE") == 0)	
   file_verify(idevice,odevice);	/* verify specified number of files  */
else if (strcmp(subcmd,"RECORD") == 0)
   record_verify(idevice,odevice);	/* verify specified number of files  */

c_complt("DITTO VERIFY",out,&zero);

return EXIT_SUCCESS;

}
