/******************************************************************************
 
FUNCTION:	WEOF

PURPOSE:	Write the specified number of EOF marks on tape

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/85    K. Gacke		initial development
  1.1	    7/86    K. Gacke		LAS standards
  5.0      10/88    D. Hollaren		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
	The parameter "count" is inputted into the program.  The DITTO output
        tape device is opened for write access.  The tape device is then 
	closed writing "count" end of file marks.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "tapeio.h"			/* tape I/O include file             */
#include "ditto.h"			/* ditto include file                */
 
struct PARBLK   vblock;		/* TAE parameter block               */
struct DITTO_DEV ditto_device;	/* ditto I/O devices descriptors     */

main()
{
struct TAPE_FD  tape;			/* tape descriptor                   */
struct VARIABLE *p_find();		/* function to find TAE variable     */
struct VARIABLE *val;			/* pointer to TAE variable           */

int     eof_mark;			/* number of EOF marks to be written */
int     status;				/* function return status code       */
int	std_out;			/* Fortran LUN for standard output   */
int	zero = 0;			/* value of zero                     */

long fatal = -1;

char	out[1][CMLEN];			/* output files for completion msg   */

/*
  Initialize TAE
*/
c_inlas(&vblock,&std_out);

/*
  Set input and output devices to no values
*/
ditto_device.input_dev = NULL;
ditto_device.output_dev = NULL;

/*
  Retrieve prameter "ntmarks"
*/
val = p_find(&vblock,"ntmarks");
eof_mark = IVAL(*val,0);

/*
  Reopen DITTO output tape device
*/
status = reopen_tape("OUTPUT",NOREW,TWRITE,&tape);
if (status != E_SUCC)
    c_errmsg("Fatal error encountered","ditto weof-fatal",&fatal);

/*
  Close tape writing EOF marks
*/
close_tape("OUTPUT",&tape,NOREW,eof_mark);

c_complt("DITTO WEOF",out,&zero);
}
