 #ifndef CM_H
#define CM_H

#include "desc.h"
#include "imageio.h" /* MAXBND definition */
#include "tae.h"
#include "worgen.h"


/*****************************************************************************
 *                                                                           *
 *      This is the include file for the 'cm' routines.                      *
 *                                                                           *
 *      Modified by D. Akkerman, USGS/EROS Data Center, 5/90.                *
 *       - Added the TAE_UPATH define for changes to the cmhost and          *
 *         cmtae routines.                                                   *
 *                                                                           *
 *      Modified by B. Davis, USGS/EROS Data Center, 9/90.                   *
 *       - added the cm tape structures and defines for cmstor/cmrestor.     *
 *                                                                           *
 *      Modified by D. Etrheim, USGS/EROS Data Center, 11/90.                *
 *       - added online/offline/both defines                                 *
 *****************************************************************************/

#define TAE_EXT  ';'
#define HOST_EXT '.'

#define TAE_DIR_BEG '['
#define HOST_DIR_BEG '/'
#define HOST_DIR_SEP '/'
#define HOST_DIR_END '/'

/* Define the use of the external compression environment variable.          */
#define EXT_COMP 99

#define MAXFILES 100              /* Maximum number of files to be
                                      associated with and image,
                                      in order to help limit the
                                      size of array declarations.            */

/* Structure for parse list. */
struct  PRS_LST
{
    char    name[CMLEN+1];        /* Image name                              */
    double  window[4];            /* Image window                            */
    int     bands[MAXBND+1];      /* Image bands                             */
    int     proj[MAXBND+1];       /* Image bands                             */
    int     qflag[2];             /* Window and band flags                   */
};


int las_antoi
(
    const char *srce,             /* I: Source string to convert             */
    int sz                        /* I: Number of chars to convert           */
);

int c_cmhost
(
    const char *tae,              /* I: Input TAE file name                  */
    char *hname                   /* I/O: Output host file name              */
);

int c_copy_image
(
    struct PARBLK *vblock,        /* I: TAE parameter block                  */
    const char hosin[][CMLEN],    /* I: Filenames of input images (Unix)
                                        hosin may be a single- or multi-band
                                        image and may contain BYTE, I*2,
                                        I*4, or R*4 pixel data.              */
    const char hosout[CMLEN],     /* I: Filename of output image (Unix)      */
    int comp,                     /* I: Compression flag
                                        TRUE = Compress output image
                                        FALSE = Don't compress output image
                                        EXT_COMP = Use external compres_opt
                                        or COMPRES1 environment variable     */
    int nmrimg,                   /* I: Number of input images               */
    int window[][4],              /* I: Window for each input image
                                        (SL, SS, NL, NS)
                                        If all values are 0s then the entire
                                        image will be processed.  If the
                                        images are not of the same size then
                                        processing will be stopped.          */
    int bands[][MAXBND+1],        /* I: Bands to process - all images
                                        If all 0s then all bands in all the
                                        images will be processed.  nbands
                                        also needs to be set to 0 for the
                                        default.                             */
    int  nbands[],                /* I: Number of bands per input image
                                        If bands are all 0s and nbands[i]
                                        is 0 then all bands in all the
                                        images will be processed             */
    int totbnd,                   /* I: Total number of bands.  Set to 0 if
                                        processing the default bands         */
    int odtype,                   /* I: Data type of output image
                                        (0=SAME, EBYTE=BYTE, EWORD=I*2,
                                        ELONG=I*4, EREAL=R*4)                */
    int rtopt_in,                 /* I: Round/truncation option
                                        TRUE = Round the input data
                                        FALSE = Truncate the input data      */
    int msgtim,                   /* I: Elapsed time between display msg
                                        for TAE                              */
    char minmax[4]                /* I: Min max flag for TAE                 */
);

int c_getfiles
(
    const char *filespec,         /* I: Host file specification for files    */
    char file_names[][CMLEN],     /* O: Array of file names to be output     */
    int *nfiles,                  /* O: # of files found matching filespec   */
    int maxfiles,                 /* I: Max # of files to return at one time */
    int first_call,               /* I: Flag for first call to this function */
    int ck_lock                   /* I: 1 = return list of unlocked files only*/
                                  /*    0 = return list of all files found   */
);

int cmlock
(
    const char *hname,            /* I: Host name of file to open            */
    int *fd,                      /* O: File descriptor                      */
    int flags,                    /* I: File access                          */
    int maxtime                   /* I/O: Max time to wait (seconds)         */
);

int copyfile
(
    const char *hname,            /* I: Host name of existing file           */
    const char *hnewname,         /* I: Host name of new file                */
    int  report                   /* I: Report file?                         */
                                  /*    (0=tae,errs only, 1=taename,         */
                                  /*    2=host,errs only,3=hostname)         */
);

int deletefile
( 
    const char *hname,            /* I/O: Host name of file                  */
    const char *type,             /* I: File or dir                          */
    int  verify,                  /* I: Verify? (yes=1)                      */
    int  report                   /* I: Report it? (yes=1)                   */
);

int dirlist
(
    const char *nameoffile,       /* I: Pointer to temp file of file names   */
    const char *filespec,         /* I: Input spec                           */
    char file_ary[][CMLEN],       /* O: Array of file names to be output     */
    int  *nfiles,                 /* O: Number of files generated in list    */
    int  maxfiles,                /* I: Max number to list at a time         */
    int  startf                   /* I: Flag for first call                  */
);

int dirnm
(
    char *hnam                    /* I/O: Pointer to host path name          */
);

int getcredate
(
    const char *hostname,         /* I: Host file name                       */
    char *credate                 /* O: Date host file created               */
);

int getdirs
(
    const char *filespec,         /* I: Host description of path and filename*/
    char directories[][CMLEN],    /* O: Array of directory names found       */
    int *ndirs,                   /* O: Number of directories found          */
    int maxdirs,                  /* I: Maximum # of directories to look for */
    int startf                    /* I: Number of directories already found  */
);

int getmydef
(
    char *dir                     /* O: Default directory searching for      */
);

int mkdr
(
    char *hostnm                  /* I/O: Pointer to directory path          */
);

void las_processfile
(
    struct PARBLK *vblock,        /* I: Ptr to TAE param block               */
    const char *func,             /* I: Function to perform                  */
    const char *name,             /* I: Current filename                     */
    char *newname,                /* O: New filename                         */
    int  imgset,                  /* I: Img & assoc? (1=yes)                 */
    int  imageflg,                /* I: Process image file                   */
    char aftype[][CMLEN],         /* I: Assoc file types                     */
    int  acount,                  /* I: # assoc file types                   */
    int  verify,                  /* O: Verify? (1=yes)                      */
    int  *nproc,                  /* O: # files processed                    */
    int  copyflg                  /* I: Flag if im data is copied or renamed */
);

int renamefile
(
    char *hname,                  /* I/O: Current host name                  */
    const char *hnewname,         /* I: New host name                        */
    int  verify,                  /* I: Verify? (1=yes)                      */
    int  report,                  /* I: Report file? (1=yes)                 */
    int  copyflg                  /* I: Flag if im data is copied or renamed */
);


int las_unlock
(
    int fd,                       /* I: File descriptor                      */
    int maxtime                   /* I: Max time to wait (seconds)           */
);

BOOL isdir
(
   const char *path               /* I: Host name                            */
);

#ifndef NO_OBSOLETE_INTERFACE

int antoi
(
    const char *srce,             /* I: Source string to convert             */
    const int *sz,                /* I: Number of chars to convert           */
    int  *value                   /* O: Decimal value to return              */
);

int c_cmtae
(
    const char *hostname,         /* I: Input host file name                 */
    char *tname                   /* O: Output TAE file name                 */
);

int cmtaedir
(
    const char *hostdir,          /* I: Host directory specification         */
    char *taedir                  /* O: TAE directory specification          */
);

char *gettoken
(
    char *string,                 /* I: Input string                         */
    const char *delim             /* I: Token delimiters                     */
);

void processfile
(
    struct PARBLK *vblock,        /* I: Ptr to TAE param block               */
    const char *func,             /* I: Function to perform                  */
    const char *name,             /* I: Current filename                     */
    char *newname,                /* O: New filename                         */
    int imgset,                   /* I: Img & assoc? (1=yes)                 */
    int imageflg,                 /* I: Process image file                   */
    char aftype[][CMLEN],         /* I: Assoc file types                     */
    const long *acount,           /* I: # assoc file types                   */
    int  verify,                  /* O: Verify? (1=yes)                      */
    int *nproc,                   /* O: # files processed                    */
    int copyflg                   /* I: Flag if im data is copied or renamed */
);

int unlock
(
    const int *fd,                /* I: File descriptor                      */
    int maxtime                   /* I: Max time to wait (seconds)           */
);

#endif /* obsolete interfaces */

#endif
