#define HEADER_SIZE 1536       /* size of the fast format header            */
#define NUMFIELDS 117          /* number of fields in Fast-B/C header       */
#define NUM_ADMIN_FIELDS 133   /* number of fields in Fast-L7 admin header  */
#define NUM_RAD_FIELDS 65      /* number of fields in Fast-L7 radiom header */
#define NUM_GEOM_FIELDS 132    /* number of fields in Fast-L7 geom header   */

/* GEOM will be added to the documented field numbers to get to the geometric
 * fields in this table.  At this point the radiometric fields are not used.
 * The administrative fields can be used as defined in the Fast-L7A docs, since
 * they are at the beginning of the header.  Space Imaging Fast-C will be
 * read from the Fast-L7A tables.
 */
#define GEOM NUM_ADMIN_FIELDS+NUM_RAD_FIELDS

/* The following defines are for each field in the header field lookup table.
   This table defines the position, data type, and formatting information for
   each field in the fast format header specification                        */

#define FSBYTE      0               /* starting byte position field          */
#define FEBYTE      1               /* ending byte position field            */
#define FDTYPE      2               /* data type field                       */
#define FLEN1       3               /* formatting value width field          */
#define FLEN2       4               /* formatting value precision field      */


/* Defines for the visisble, thermal, and pan bands */
#define VISIBLE 30
#define THERMAL 60
#define PAN     15


#define LASPROJKEYLEN    5
#define PROJCOUNT       21

struct PROJKN {     /* Projection keywords and name */
    long    projnum;                /* projection number */
    char    projname[LASPROJKEYLEN];    /* descriptive name */
    };

/*  Projection key definitions  */
#define SETPROJKN(tablename) \
  struct PROJKN tablename[PROJCOUNT] = { \
    {1,     "UTM"       },\
    {2,     "SPCS"      },\
    {3,     "ACEA"      },\
    {4,     "LCC"       },\
    {5,     "MER"       },\
    {6,     "PS"        },\
    {7,     "PC"        },\
    {8,     "EC"        },\
    {9,     "TM"        },\
    {10,    "SG"        },\
    {11,    "LAEA"      },\
    {12,    "AE"        },\
    {13,    "GNO"       },\
    {14,    "OG"        },\
    {15,    "GVNP"      },\
    {16,    "SIN"       },\
    {17,    "ER"        },\
    {18,    "MC"        },\
    {19,    "VDG"       },\
    {20,    "OM"        },\
    {22,    "SOM"       }   }


#define SPHERECOUNT     20
#define SPHERENAMELEN   40
#define MONTHCOUNT      12
#define MONTHNAMELEN    4

struct MONTHDEF {  /* Month names, numbers */
    char    monthnum[3];
    char    monthname[MONTHNAMELEN];
    };

/* Month name-number definitions */
#define SETMONTHDEF(tablename) \
  struct MONTHDEF tablename[MONTHCOUNT] = { \
    {"01",  "JAN"   },\
    {"02",  "FEB"   },\
    {"03",  "MAR"   },\
    {"04",  "APR"   },\
    {"05",  "MAY"   },\
    {"06",  "JUN"   },\
    {"07",  "JUL"   },\
    {"08",  "AUG"   },\
    {"09",  "SEP"   },\
    {"10",  "OCT"   },\
    {"11",  "NOV"   },\
    {"12",  "DEC"   } }

