#ifndef LASENDIAN_H
#define LASENDIAN_H

/* NOTE: The "_macro" added to the variables in the macro definitions is an
   attempt to prevent name collisions with variables used in the macro call.
   We have had a case where "i" was used as part of one of the parameters to
   one of the macros and it collided with an "i" in the macro.  The result
   was a difficult to track bug.  This really begs for inline functions, but
   "inline" isn't well supported on all our platforms. */

/* macro to swap the bytes in a 16-bit variable */
#define swapbytes16(x) \
{ \
    unsigned short int data_macro = *(unsigned short int*)&(x); \
    data_macro = ((data_macro & 0xff00) >> 8) |    \
                 ((data_macro & 0x00ff) << 8);     \
    *(unsigned short int*)&(x) = data_macro;       \
}
/* macro to swap the bytes in a 32-bit variable */
#define swapbytes32(x) \
{ \
    unsigned int data_macro = *(unsigned int*)&(x); \
    data_macro = ((data_macro & 0xff000000) >> 24) |    \
                 ((data_macro & 0x00ff0000) >>  8) |    \
                 ((data_macro & 0x0000ff00) <<  8) |    \
                 ((data_macro & 0x000000ff) << 24);     \
    *(unsigned int*)&(x) = data_macro;            \
}

/* macro to swap the bytes in a 64-bit variable */
#define swapbytes64(x) \
{ \
    unsigned int *words_macro = (unsigned int *)&(x); \
    unsigned int temp0_macro;        \
    unsigned int temp1_macro;        \
    temp0_macro = words_macro[0];    \
    swapbytes32(temp0_macro);        \
    temp1_macro = words_macro[1];    \
    swapbytes32(temp1_macro);        \
    words_macro[1] = temp0_macro;    \
    words_macro[0] = temp1_macro;    \
}

/* macro to swap bytes in a large buffer of 16-bit data */
#define swapbytes16_in_buffer(buf_ptr,items) \
{ \
    int i_macro; \
    int end_macro = (items); \
    unsigned short int *buf16_ptr_macro = (unsigned short int*)(buf_ptr); \
    for (i_macro = 0; i_macro < end_macro; i_macro++)    \
        swapbytes16(buf16_ptr_macro[i_macro]); \
}

/* macro to swap bytes in a large buffer of 32-bit data */
#define swapbytes32_in_buffer(buf_ptr,items) \
{ \
    int i_macro; \
    int end_macro = (items); \
    unsigned int *buf32_ptr_macro = (unsigned int*)(buf_ptr); \
    for (i_macro = 0; i_macro < end_macro; i_macro++)    \
        swapbytes32(buf32_ptr_macro[i_macro]); \
}

/* macro to swap bytes in a large buffer of 64-bit data */
#define swapbytes64_in_buffer(buf_ptr,items) \
{ \
    int i_macro; \
    int end_macro = (items); \
    double *buf64_ptr_macro = (double *)(buf_ptr); \
    for (i_macro = 0; i_macro < end_macro; i_macro++)    \
        swapbytes64(buf64_ptr_macro[i_macro]); \
}
        
/* macro to swap bytes given a number of bytes in a word */
#define swapbytes_in_buffer(buf_ptr,bytes_per_word,items) \
{ \
    if ((bytes_per_word) == 4) \
    { \
        swapbytes32_in_buffer((buf_ptr),(items)); \
    } \
    else if ((bytes_per_word) == 8) \
    { \
        swapbytes64_in_buffer((buf_ptr),(items)); \
    } \
    else if ((bytes_per_word) == 2) \
    { \
        swapbytes16_in_buffer((buf_ptr),(items)); \
    } \
}
        
/* All binary values written to files are written as big endian.  So, for 
   intel processors (little endian), swap the bytes.  For all others, don't 
   do any byte swapping. */
#ifdef __i386__
#define LAS_LITTLE_ENDIAN
/* i386 processors are little endian byte order */

/* currently, little endian is not the standard order for LAS files, so define
   the following macros to swap byte order */
#define localendian16(x) swapbytes16((x))
#define localendian32(x) swapbytes32((x))
#define localendian64(x) swapbytes64((x))
#define localendian16_buf(buf,items) swapbytes16_in_buffer((buf),(items))
#define localendian32_buf(buf,items) swapbytes32_in_buffer((buf),(items))
#define localendian64_buf(buf,items) swapbytes64_in_buffer((buf),(items))
#define localendian_buf(buf,bytes_per_word,items) \
            swapbytes_in_buffer((buf),(bytes_per_word),(items))
#define exportendian16(x) swapbytes16(x)
#define exportendian32(x) swapbytes32(x)
#define exportendian64(x) swapbytes64(x)
#define exportendian16_buf(buf,items) swapbytes16_in_buffer((buf),(items))
#define exportendian32_buf(buf,items) swapbytes32_in_buffer((buf),(items))
#define exportendian64_buf(buf,items) swapbytes64_in_buffer((buf),(items))
#define exportendian_buf(buf,bytes_per_word,items) \
            swapbytes_in_buffer((buf),(bytes_per_word),(items))

#else
/* not i386 processor */

#define LAS_BIG_ENDIAN

#define LAS_STANDARD_ENDIAN
/* Big endian is the standard LAS endian order.  If a routine needs to 
   conditionally compile code based on endian, it should test this define 
   to allow easily changing it in the future. */

/* define the following macros to be null operations */
#define localendian16(x)
#define localendian32(x)
#define localendian64(x)
#define localendian16_buf(buf,items)
#define localendian32_buf(buf,items)
#define localendian64_buf(buf,items)
#define localendian_buf(buf,bytes_per_word,items)
#define exportendian16(x)
#define exportendian32(x)
#define exportendian64(x)
#define exportendian16_buf(buf,items)
#define exportendian32_buf(buf,items)
#define exportendian64_buf(buf,items)
#define exportendian_buf(buf,bytes_per_word,items)
#endif /* processor type */

#endif
