#ifndef PIXMAN_H
#define PIXMAN_H

#define TOOBIG          77              /* Output value to large for data type*/
#define TOOSMALL        88              /* Output value to small for data type*/

#define TYPERR          66              /* not a correct data type            */
                                        /* need to move this to a local pixman*/
                                        /* prototype .h file.  Then, change   */
                                        /* util/c_inmnmx.c to use E_FAIL where*/
                                        /*  TYPERR is being used.             */

int las_pixadd
(
    int ns,                 /* I:  number of samples                      */
    const void *in1,        /* I:  input array one                        */ 
    const void *in2,        /* I:  input array two                        */
    void *out,              /* O:  output array                           */
    float scale1,           /* I:  scale applied to in1 before addition   */ 
    float scale2,           /* I:  scale applied to in2 before addition   */
    float offset,           /* I:  offset added in after sum              */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */ 
);

int las_pixbitand
(
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output "anded" array                   */
    int ns,                 /* I:  number of samples                      */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int las_pixbitor
(
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output "ored" array                    */
    int ns,                 /* I:  number of samples                      */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */ 
);    

int las_pixbitxor            
( 
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output "xored" array                   */
    int ns,                 /* I:  number of samples                      */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int las_pixcomplement
(
    const void *in,         /* I:  input array                            */
    void *out,              /* O:  output array                           */
    int ns,                 /* I:  number of samples                      */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int las_pixconvert
(
    int ftyp,               /* I:  from typ                               */   
    int totyp,              /* I:  to type                                */
    const void *frombuf,    /* I:  from buffer                            */
    void *tobuf,            /* O:  to buffer                              */
    int size                /* I:  number of samples in the input buffer  */
);

void las_pixcopy
(
    const void *frombuf,    /* I:  source buffer of the data to be copied */
    void *tobuf,            /* O:  destination buffer of the data         */
    int dtype,              /* I:  data type of the buffers               */
    int ns                  /* I:  number of elements or pixels           */
);

int las_pixdivide
(
    int ns,                 /* I:  number of samples                      */
    const void *in1,        /* I:  input array one                        */
    const void *in2,        /* I:  input array two                        */
    void *out,              /* O:  output array                           */
    float scale,            /* I:  scale applied to in. arrays after mult */
    float offset,           /* I:  offset added in after sum              */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    const void *errval      /* I:  value to place in array where div zero */
);

void las_pixfill
(
    void *tobuf,            /* O:  the buffer to be filled with fillval   */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    int ns,                 /* I:  number of elements/pixels to be filled */ 
    const void  *fillval    /* I:  fill value for the buffer              */
);

int las_pxhiss
(
    int ns,                 /* I:  number of samples in buffer            */ 
    const void *bufptr,     /* I:  4 byte buffer pointer                  */
    int datatyp,            /* I:  type of buffer                         */
    int *histo,             /* O:  pointer to a 4 byte histogram array    */
    int *count,             /* O:  total number of values histogramed     */
    int skip                /* *:  sampling step                          */
);

int las_pxhist
(
    int ns,                 /* I:  number of samples in buffer            */
    const void *bufptr,     /* I:  4 byte buffer pointer                  */
    int datatyp,            /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    int *histo,             /* O:  pointer to a 4 byte histogram array    */
    int *count              /* O:  number of values in histogram array    */
);

int las_pxindr
(
    const void *in,         /* I:  input array                            */
    int val,                /* I:  value to be searched for               */
    int ns,                 /* I:  number of samples                      */
    int *index,             /* 0:  index to be returned                   */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int las_pixlinear
(
    const void *in,         /* I:  input array                            */ 
    void *out,              /* O:  output array                           */  
    int ns,                 /* I:  number of samples                      */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    float scale,            /* I:  scale factor                           */
    float offset            /* I:  offset to be added                     */
);

int las_pxlookup              
(
    const void *in,         /* I:  input array                            */
    void *out,              /* O:  output array                           */
    int ns,                 /* I:  number of samples                      */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    const void *lut,        /* I:  lookup table array                     */
    int luttype             /* I:  data type of lookup table              */ 
);

int las_pixmaximum
(
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output array                           */
    int ns,                 /* I:  number of samples                      */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int las_pixminimum
(
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output array                           */
    int ns,                 /* I:  number of samples                      */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int las_pixminmax
(
    const void *in,         /* I:  input array                            */
    int ns,                 /* I:  number of samples                      */
    double *dmin,           /* I/O: minimum pixel value                   */ 
    double *dmax,           /* I/O: maximum pixel value                   */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);    

int las_pixmultiply
(
    int ns,                 /* I:  number of samples                      */
    const void *in1,        /* I:  input array one                        */
    const void *in2,        /* I:  input array two                        */
    void *out,              /* I:  output array                           */
    float scale,            /* I:  scale applied to in. arrays after mul. */
    float offset,           /* I:  offset added in after sum              */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);    

void las_pixsubsample
(
    int inns,               /* I:  number of samples in input buffer      */ 
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    int inc,                /* I:  sample increment to subsample by       */
    void *buf               /* I/O:  buffer to be subsampled              */
);

int las_pixsubtract
(
    int ns,                 /* I:  number of samples in input buffer      */    
    const void *in1,        /* I:  input array one                        */
    const void *in2,        /* I:  input array two                        */
    void *out,              /* O:  output array                           */
    float scale1,           /* I: scale applied to in. array1 before sub. */
    float scale2,           /* I: scale applied to in. array2 before sub. */
    float offset,           /* I: offset added in after difference        */
    int dtype               /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int las_pixswap
(
    void *buf,              /* I/O: data buffer which will be byte swapped*/   
    int ns,                 /* I:  number of entities in the buffer       */
    int size                /* I:  number of bytes per entity             */
);

int las_pxsys  
(
    long insys,             /* I:  input computer system                  */   
    long outsys,            /* I:  output computer system                 */
    void *buf,              /* I/O: array of real numbers to be converted */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    int size,               /* I:  number of elements in array            */ 
    const void *under,      /* I:  value assigned if underflow detected   */
    const void *over,       /* I:  value assigned if overflow detected    */
    int flag                /* I:  flag used for error detection          */
);


#ifndef NO_OBSOLETE_INTERFACE

int c_pxadd
(
    const long *ns,         /* I:  number of samples                      */
    const void *in1,        /* I:  input array one                        */
    const void *in2,        /* I:  input array two                        */
    void *out,              /* O:  output array                           */
    const float *scale1,    /* I:  scale applied to in1 before addition   */
    const float *scale2,    /* I:  scale applied to in2 before addition   */
    const float *offset,    /* I:  offset added in after sum              */
    const long *dtype       /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int c_pxbtan
(
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output "anded" array                   */
    const long *ns,         /* I:  number of samples                      */
    const long *dtype       /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int c_pxbtor
(
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output "ored" array                    */
    const long *ns,         /* I:  number of samples                      */
    const long *dtype       /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int c_pxbxor
(
    const void *in,         /* I:  input array                            */
    const void *map,        /* I:  input mapping array                    */
    void *out,              /* O:  output "xored" array                   */
    const long *ns,         /* I:  number of samples                      */
    const long *dtype       /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int c_pxcomp
(
    const void *in,         /* I:  input array                            */
    void *out,              /* O:  output array                           */
    const long *ns,         /* I:  number of samples                      */
    const long *dtype       /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
);

int c_pxconv
(
    const long *ftyp,       /* I:  from type                              */
    const long *totyp,      /* I:  to type                                */
    const void *frombuf,    /* I:  from buffer                     */
    void *tobuf,            /* O:  to buffer                              */
    const long *size        /* I:  number of samples in the input buffer  */
);

void c_pxcopy
(
    const void *frombuf,    /* I:source buffer of data to be copied       */
    void *tobuf,            /* O:  destination buffer of the data         */
    const long *dtype,      /* I:  data type of the buffers               */
    const long *ns          /* I:  number of elements or pixels           */
);

int c_pxdiv
(
    const long *ns,         /* I: number of samples                       */
    const void *in1,        /* I: input array one                         */
    const void *in2,        /* I: input array two                         */
    void *out,              /* O: output array                            */
    const float *scale,     /* I: scale applied to in. arrays after mult  */
    const float *offset,    /* I: offset added in after sum               */
    const long *dtype,      /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
    const long *errval      /* I: value to place in array where div zero  */
);

void c_pxfill
(
    void *tobuf,           /* O: the buffer to be filled with fillval     */
    const long *dtype,     /* I: data type (EBYTE,EWORD,ELONG,EREAL)      */
    const long *ns,        /* I: number of elements/pixels to be filled   */
    const void *fillval    /* I: fill value for the buffer                */
);

int c_pxhiss
(
    const long *ns,         /* I: pointer to number of samples in buffer  */
    const void *bufptr,     /* I: 4 byte buffer pointer                   */
    const long *datatyp,    /* I: pointer to the type of buffer           */
    long *histo,            /* O: pointer to a 4 byte histogram array     */
    long *count,            /* O: total number of values the histogramed  */
    const long *skip        /* I: sampling step                           */
);

int c_pxhist
(
    const long *ns,         /* I: pointer to number of samples in buffer  */
    const void *bufptr,     /* I: 4 byte buffer pointer                   */
    const long *datatyp,    /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
    long *histo,            /* O: pointer to a 4 byte histogram array     */
    long *count             /* O: number of values in histogram array     */
);

int c_pxindr
(
    const void *in,         /* I: input array                             */
    const long *val,        /* I: value to be searched for                */
    const long *ns,         /* I: number of samples                       */
    long *index,            /* O: index to be returned                    */
    const long *dtype       /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
);

int c_pxline
(
    const void *in,         /* I: input array                             */
    void *out,              /* O: output array                            */
    const long *ns,         /* I: number of samples                       */
    const long *dtype,      /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
    const float *scale,     /* I: scale factor                            */
    const float *offset     /* I: offset to be added                      */
);

int c_pxlkup
(
    const void *in,         /* I: input array                             */
    void *out,              /* O: output array                            */
    const long *ns,         /* I: number of samples                       */
    const long *dtype,      /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
    const void *lut,        /* I: lookup table array                      */
    const long *luttype     /* I: data type of lookup table               */
);

int c_pxmax
(
    const void *in,         /* I: input array                             */
    const void *map,        /* I: input mapping array                     */
    void *out,              /* O: output  array                           */
    const long *ns,         /* I: number of samples                       */
    const long *dtype       /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
);

int c_pxmin
(
    const void *in,         /* I: input array                             */
    const void  *map,       /* I: input mapping array                     */
    void *out,              /* O: output  array                           */
    const long *ns,         /* I: number of samples                       */
    const long *dtype       /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
);

int c_pxmnmx
(
    const void  *in,        /* I: input array                             */
    const long *ns,         /* I: number of samples                       */
    double *dmin,           /* I/O: minimum pixel value                   */
    double *dmax,           /* I/O: maximum pixel value                   */
    const long *dtype       /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
);

int c_pxmul
(
    const long *ns,         /* I: number of samples                       */
    const void *in1,        /* I: input array one                         */
    const void *in2,        /* I: input array two                         */
    void *out,              /* O: output array                            */
    const float *scale,     /* I: scale applied to in. arrays after mul.  */
    const float *offset,    /* I: offset added in after sum               */
    const long *dtype       /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
);

void c_pxsamp
(
    const long *inns,       /* I: number of samples in input buffer       */
    const long *dtype,      /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
    const long *inc,        /* I: sample increment to subsample by        */
    void *buf               /* I/O: buffer to be subsampled               */
);

int c_pxsub
(
    const long *ns,         /* I: number of samples                       */
    const void *in1,        /* I: input array one                         */
    const void *in2,        /* I: input array two                         */
    void *out,              /* O: output array                            */
    const float *scale1,    /* I: scale applied to in. array1 before sub. */
    const float *scale2,    /* I: scale applied to in. array2 before sub. */
    const float *offset,    /* I: offset added in after difference        */
    const long *dtype       /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
);

int c_pxswap
(
    void *buf,              /* I/O: data buffer which will be byte swapped*/
    const long *ns,         /* I: number of entities in the buffer        */
    const long *size        /* I: number of bytes per entity              */
);

int c_pxsys
(
    const long *insys,      /* I: input computer system                   */
    const long *outsys,     /* I: output computer system                  */
    unsigned char *buf,     /* I/O: array of real numbers to be converted */
    const long *dtype,      /* I: data type (EBYTE,EWORD,ELONG,EREAL)     */
    const long *size,       /* I: number of elements in array             */
    const void *under,      /* I: value assigned if underflow detected    */
    const void *over,       /* I: value assigned if overflow detected     */
    const long *flag        /* I: flag used for error detection           */
);

#endif /* obsolete interfaces */

#endif
