/****************************************************************************
FUNCTION:       ANTOI

PURPOSE:        ANTOI takes an ascii string that is neither null-terminated 
                nor white-space-terminated and converts it to a decimal value.
                It can do so by knowing the size of the string.

PROGRAM HISTORY:
  Version       Author       Date    Change Request
  -------       ------       -----   --------------
  5.0           B.Davis      05/89   Original development

                                        
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                none.

PROJECT:        LAS 5.0

ALGORITHM(flow):
                for size of characters
                        take ascii value of character times 10
                        index it by the ascii value of 0


ALGORITHM REFERENCES:
                none
*****************************************************************************/

#include <ctype.h>
#include "las.h"
#include "cm.h"

/***   Convert ascii digit string to a decimal value    ***/

/**
 * las_antoi:
 * @srce: I: Source to convert to a decimal value.
 * @sz: I: Number of characters in the string to convert.
 *
 * The las_antoi() routine takes an ascii string that is neither
 * null-terminated nor white-space-terminated and converts it
 * to a decimal value.  The only way it can do this is  by 
 * knowing the "size" of the string.
 *
 * Returns:  Decimal value converted.  
 */  
int las_antoi
(
    const char *srce,          /* I: Source string to convert     */
    int sz                     /* I: Number of chars to convert   */
)
{
    int value = 0;

    for (; (sz)-- && isdigit(*srce); srce++)
    {
        value *= 10;
        value += (*srce - '0');
    }
    return(value);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * antoi:
 * @srce: I: Source to convert to a decimal value.
 * @sz: I: Number of characters in the string to convert.
 * @value: O: Decimal value converted by antoi(). 
 *
 * The antoi() routine takes an ascii string that is neither
 * null-terminated nor white-space-terminated and converts it
 * to a decimal value.  The only way it can do this is  by 
 * knowing the "size" of the string.
 *
 * Deprecated.  Use las_antoi() instead.
 *
 * Returns:  E_SUCC:  Successful completion of conversion.
 */ 
int antoi
(
    const char *srce,          /* I: Source string to convert     */
    const int  *sz,            /* I: Number of chars to convert   */
    int  *value                /* O: Decimal value to return      */
)
{

    *value = las_antoi(srce,*sz);

    return(E_SUCC);
}

#endif /* obsolete interfaces */


