/******************************************************************************
FUNCTION:       bldname

PURPOSE:        given the name of an existing file, and the user-supplied
                file specification of the output file (possibly including
                wildcard characters), bldname constructs and returns an
                output file name with no wildcard characters.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      11/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            include file changes
  5.0      04/88    D. Hollaren         changed include spec to las


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Find the directory, file, and extension parts of the user-supplied output file
specification.
if no wildcards were specified
   return the user-supplied name,
else if the file part is a wildcard
   use the file part from the existing input file,
else if the extension part is a wildcard
   use the extension from the existing input file.

ALGORITHM REFERENCES    none
*****************************************************************************/

#include <string.h>
#include "las.h"
#include "cm.h"
#include "local_protos.h"

void bldname
(
    const char *fname,      /* I: Current host filename                 */
    const char *newspec,    /* I: New host name specified               */
    char *newname           /* O: New filename (no wildcard)            */
)
{
    int i;                  /* Index                                    */

    char c;                 /* Directory delimiter character            */
    char newdirpart[CMLEN]; /* Directory part of new name               */
    char newextpart[40];    /* Extension part of new name               */
    char newfilepart[40];   /* File part of new name                    */
    char oldextpart[40];    /* Extension part of new name               */
    char oldfilepart[40];   /* File part of new name                    */
    char *p;                /* Temporary pointer                        */

   
/************************************************************************/

    /***  construct the filename  ***/
    c = '/';                /* unix */

    if ((p = strrchr (newspec,c)) == NULL)          /* Isolate file, dir    */
    {                                               /* Of newspec           */
        strcpy (newdirpart,"");                     /* No directory         */
        strcpy (newfilepart,newspec);               /* Isolate filepart     */
        i = strcspn (newfilepart,".");
        newfilepart[i] = 0;
    }
    else
    {
        i = (strlen(newspec) - strlen(p)) + 1;      /* Isolate directory    */
        strncpy (newdirpart,newspec,i);
        newdirpart[i] = 0;
        strcpy (newfilepart,++p);                   /* Isolate filename     */
        i = strcspn (newfilepart,".");
        newfilepart[i] = 0;
    }

    p = strrchr (newspec,'.');                      /* Isolate ext of newspec*/
    if (p == NULL) 
    {
        strcpy (newextpart,"*");                    
    }
    else
    {
        strcpy (newextpart,++p);                    
    }


    if ((strcmp(newfilepart,"*") != 0) &&           /* Both filepart and    */
        (strcmp(newextpart,"*") != 0))              /* Extension specified: */
    {
        strcpy (newname,newspec);                   /* Use specified name   */
    }

    else if (strcmp(newfilepart,"*") == 0)          /* Filepart is "*"      */
    {
        if ((p = strrchr (fname,c)) == NULL)        /* Isolate file part    */
            strcpy  (oldfilepart,fname);            /* Of fname             */
        else
            strcpy (oldfilepart,++p);
        i = strcspn (oldfilepart,".");
        oldfilepart[i] = 0;

        strcpy (newname,newdirpart);                /* Use new directory    */
        strcat (newname,oldfilepart);               /* Use old filepart     */
        strcat (newname,".");
        strcat (newname,newextpart);                /* Use new extension    */
    }

    else if (strcmp(newextpart,"*") == 0)           /* Extension is "*"     */
    {   
        p = strrchr (fname,'.');                    /* Isolate extension    */
        strcpy (oldextpart,++p);                    /* Of fname             */

        strcpy (newname,newdirpart);                /* Use new directory    */
        strcat (newname,newfilepart);               /* Use new filepart     */
        strcat (newname,".");
        strcat (newname,oldextpart);                /* Use old extension    */
    }
}
