/*********************   Catalog Manager Routines   *************************
FUNCTION:       C_CMTAE

PURPOSE:        Given a host file name, c_cmtae returns the TAE file name.
                Note that it only returns the fully specified TAE name if
                the host name is fully specified.

PROGRAM HISTORY:
  Version       Author       Date    Change Request
  -------       ------       -----   --------------
                K. Gacke     8/86    initial development
                B. Ailts     12/87   replace descriptors with character strings
                                     rename to c_cmtae
   5.0          D. Hollaren  04/88   changed include spec to las
                                     changed cerrmsg to c_errmsg and
                                     passes status code by address
                                     return status E_SUCC/E_FAIL
   5.0          D. Akkerman  05/90   modified to use the entire host path
                                     if a valid user name cannot be found
   5.1          J.Fenno      09/91   Modified to drop a portion of the
                                     specified path only when the user's home
                                     directory is a prefix of the given path.
   6.0          J.Fenno      03/94   Ensure the filename/path is all lowercase

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Remove that portion of the directory specification prior to the user's home
directory when under UNIX.

Find the beginning and ending locations of the directory spec
Translate each character of the host directory into a TAE directory
specification.
Translate the remaining host file name into a TAE file name.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "cm.h"


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_cmtae:
 * @hostname: I: Input host filename.
 * @tname: O: Output TAE filename.
 *
 * Given a host file name, c_cmtae() returns the TAE file name.  Note
 * that it only returns the fully specified TAE name if the host name
 * is fully specified.
 * 
 * This routine is now obsolete since once a file reaches the host format,
 * it is never changed back to the TAE format.
 *
 * Returns:  E_SUCC:  Successful completion of operation.
 */

int c_cmtae
(
    const char *hostname,      /* I: Input host file name                   */
    char *tname                /* O: Output TAE file name                   */
)
{
    /* This routine is obsolete, since the standard LAS filename syntax
       is now Unix.                                                         */
    strcpy(tname, hostname);
    return E_SUCC;
}

#endif /* obsolete interfaces */

