/******************************************************************************

FUNCTION:       DIRNM

PURPOSE:        Receives a host name including the full path and returns the
                path without the file name, only the preceding path.

PROGRAM HISTORY:
  Version       Author       Date    Change Request
  -------       ------       -----   --------------
  5.0           B.Davis      05/89   Original development
  5.1           D.Etrheim    01/91   Standardized error message handling
                                        
COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                none.

PROJECT:        LAS 5.0

ALGORITHM(flow): 
                point to the character after the last host directory separator


ALGORITHM REFERENCES:
                none.
*******************************************************************************/

#include <string.h>
#include "las.h"
#include "cm.h"

/**
 * dirnm:
 * @hnam:  I/O: Pointer to host path name.
 *
 * The dirnm() routine receives a host name including the full path
 * and returns the path without the file name, only the preceding path.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int dirnm
(
    char *hnam                  /* I/O: Pointer to host path name */
)
{
    char *ptr;                  /* local character pointer */

    ptr = strrchr(hnam,HOST_DIR_END);

    if (ptr == 0)
    {
        c_lasmsg(FALSE, "dirnm-strrchr","Error stripping file from host name");
        return(E_FAIL);
    }

    ptr++;

    *ptr = 0;

    return(E_SUCC);
}
