/******************************************************************************
SUBPROGRAM:     GETCREDATE

PURPOSE:        Returns a string that is the create date and time for a file

PROGRAM HISTORY:
 mode Version   Date        Author              Change Request
  -------       ----        ------              --------------
   1.0          10/86       T. Butzer           New Development
   1.1          12/87       B. Ailts            include file
                                                spec. changes
   5.0          04/88       D. Hollaren         changed include spec to las
                                                changed cerrmsg to c_errmsg and
                                                passed status code by address
   5.1          01/91       D.Etrheim           Standardized error message 
                                                handling

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Get the create date and time for the specified file.
Append the time to the create date
Return the string.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include "las.h"
#include "cm.h"

/**
 * getcredate:
 * @hostname:  I: Host file name.
 * @credate:  O:  Date that the host file was created.
 *
 * The getcredate() routine returns a string that is the create date
 * and time for a specified file.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int getcredate
(
    const char *hostname,           /* I: Host file name               */
    char *credate                   /* O: Date host file created       */
)
{
    struct stat st;
    int status;                     /* Status variable                 */
    int len;                        /* Length of the create date       */
    char msg[ERRLEN];               /* Error message string            */

    status = stat(hostname, &st);
    if (status < 0)
    {
        sprintf(msg,"Error retrieving creation date of %s",hostname);
        c_lasmsg(FALSE, "getcredate-date", msg);
        return(E_FAIL);
    }

    strcpy(credate, ctime(&st.st_mtime));
    len = strlen(credate);
    credate[len-1] = '\0'; /* lose the \n char */

    return(E_SUCC);
}
