/******************************************************************************
SUBPROGRAM:     GETMYDEF

PURPOSE:        Returns a string that is the defualt directory for the
                person running the calling program .

PROGRAM HISTORY:
  Version       Date        Author           Change Request
  -------       ----        ------           --------------
   1.0          11/86       T. Butzer        New Development
   1.1          12/87       B. Ailts         use raw C types
                                             change include dir. spec.
   5.0          04/88       D. Hollaren      change include spec. to las
                                             return E_SUCC/E_FAIL
   5.1          01/91       D. Etrheim       Standardized error message handling

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Get the callers usercode.
From the usercode get the user's working directory.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include "las.h"
#include "util.h"
#include "cm.h"

/**
 * getmydef:
 * @dir:  O:  Default directory that user is searching for.
 *
 * The getmydef() routine returns a string that is the default directory
 * for the person running the calling program.  First it gets the caller's
 * usercode.  From that usercode it gets the user's working directory.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int getmydef
(
    char *dir                  /* O: Default directory searching for         */ 
)
{
    char usercode[40];
    struct passwd *pwd;

    getusern(usercode);

    pwd = getpwnam(usercode);
    if (pwd == 0)
    {
        c_lasmsg (FALSE, "getmydef-dir", "Error obtaining default directory");
        return(E_FAIL);
    }

    strcpy(dir, pwd->pw_dir);
    return(E_SUCC);
}
