/******************************************************************************
FUNCTION:       gettoken

PURPOSE:        Gettoken accepts an input string and a set of token delimiters,
                and returns one token at a time to the calling routine.
                If the input string is not NULL, the first token in the string
                is returned.
                If the input string is NULL, the next token in the string is
                returned (gettoken keeps track of its current position in the
                string).
                When no tokens remain in the string, NULL is returned.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           ??/??    ????                Original development
  5.0      04/88    D. Hollaren         Added prolog
                                        changed include spec to las

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 

ALGORITHM REFERENCES    Adapted from strtok
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/file.h>
#include "las.h"
#include "cm.h"

#ifndef NO_OBSOLETE_INTERFACE

/**
 * gettoken:
 * @string: I:  Input string.
 * @delim:  I:  Token delimiters.
 * 
 * The gettoken() routine accepts an input string and a set of token
 * delimiters, and returns one token at a time to the calling routine.
 * If the input string is not NULL, the first token in the string is
 * returned.  If the input string is NULL, the next token in the string
 * is returned.  When no tokens remain in the string, NULL is returned.
 *
 * Deprecated.  The ANSI C standard strtok() should be used instead.
 *
 * Returns:  Tokens to the calling routine,
 *           NULL when no tokens remain.
 */
char *gettoken
(
    char *string,                 /* I: Input string                        */
    const char *delim             /* I: Token delimiters                    */

)
{
    char  *p;
    char  *q;
    char  *r;                     /* work pointers                          */
    static char     *saveptr;     /* saved string                           */

    if (string != NULL)                             /* 1st call             */
        p = string;
    else                                            /* subsequent call      */
        p = saveptr;

    if (p == 0)                                     /* no more tokens       */
    {
        return(NULL);
    }

    q = p + strspn(p, delim);                       /* skip leading delims  */

    if (*q == '\0')                                 /* no more tokens       */
    {
        return(NULL);
    }

    r = strpbrk(q, delim);                          /* move past token      */
    if (r == NULL)                                  /* if last token,       */
        saveptr = 0;                                /* indicate last token  */
    else
    {
        *r = '\0';
        saveptr = ++r;
    }

    return (q);
}

#endif /* obsolete interfaces */

