/******************************************************************************
FUNCTION:       isdir

PURPOSE:        Determine if the input file is a directory name

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           ??/??    ????                Original development
  5.0      04/88    D. Hollaren         Added prolog
                                        changed include spec to las
                                        return E_SUCC/E_FAIL
  5.1      09/89    B. Davis            Changed unix half to disregaurd sockets,
                                        make the check for dirs more stringent.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Determine if the input file is a directory name

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "cm.h"

/**
 * isdir:
 * @path:  I: Host name.
 *
 * The isdir() routine determines if the input file is a directory name.
 *
 * Returns:  TRUE:  Input file is a directory name,
 *           FALSE: Input file is not a directory name.
 */
int isdir
(
    const char *path        /* I: Host name                         */

)
{
    struct stat buf;        /* structure for receiving file info    */
    int rslt;

    rslt = stat (path, &buf);
    if (rslt < 0)
    {
        return(FALSE);
    }

    if ((buf.st_mode & S_IFSOCK) == (S_IFSOCK))
        return(FALSE);

    else if ((buf.st_mode & S_IFDIR)  == (S_IFDIR))
        return(TRUE);
    else
        return(FALSE);
}
