/******************************************************************************

PROGRAM:        MKDR

PURPOSE:        Creates a new directory in the user's directory tree.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  5.0      01/90    B. Davis            Original development
  5.1      01/91    D. Etrheim          Standardized error message handling

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
                None.

PROJECT:        LAS

ALGORITHM:
                Creates a new directory in the user's directory tree.

ALGORITHM REFERENCES:   
                Used to be part of cmmkdir, taken out and made a support
                so that other code could call it.

FATAL ERROR CONDITIONS:
                Error creating directory: <directory name>


ALGORITHM(flow):
                call lib/creat if vms or mkdir if unix

******************************************************************************/


#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <errno.h>
#include "las.h"
#include "cm.h"

/**
 * mkdr:
 * @hostnm:  I/O: Pointer to a directory path.
 *
 * The mkdr() routine creates a new directory in the user's directory tree.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */ 
int mkdr
(
    char *hostnm                        /* I/O: Pointer to directory path    */
)
{
    char msgtxt[CMLEN];                 /* Message text                      */
    char tstring[CMLEN];                /* Temporary string                  */
    int  len;                           /* Length of hostname                */
    int  mode=511;                      /* Mode for mkdir (2**9 -1)          */

    len = (long)strlen(hostnm);
    if (hostnm[len-1] == '/')
        hostnm[len-1] = 0;
    if (mkdir(hostnm,mode) != 0)
    {
        sprintf (msgtxt,"Error creating directory: %s",tstring);
        c_lasmsg (FALSE, "mkdr-create", msgtxt);
        sprintf (msgtxt,"%s",strerror(errno));
        c_lasmsg (FALSE, "mkdr-create", msgtxt);
        return(E_FAIL);
    }
    strcat (hostnm,"/");

    return(E_SUCC);
}
