/******************************************************************************
FUNCTION:       unlock

PURPOSE:        calls flock as does cmlock, but passes LOCK_UN to the opera-
                tion argument.  this means unlock.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      05/90    B. Davis            New Development
  1.1      11/90    D. Etrheim          Close the file on VMS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Open the specified host file.
VMS automatically locks a file so nothing to be done
    error if the file was to be opened for output and it already exists.
In UNIX
   attempt to unlock the file with the unblock option until it gets locked
   or the maximum wait time is reached.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <unistd.h>
#include "las.h"
#include "cm.h"


/**
 * las_unlock:
 * @fd:  I: File descriptor for specified host file.
 * @maxtime: I: Maximum time to wait while attempting to unlock
 *              the file.
 *
 * The las_unlock() routine calls lockf, as does cmlock, but
 * passes LOCK_UN to the operation argument.  This means unlock.
 * The las_unlock() routine attempts to unlock the file 
 * until it gets unlocked or the maximum wait time
 * is reached.
 *
 * Returns:  E_SUCC:  Successful completion of operation.
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int las_unlock
(
    int fd,                 /* I: File descriptor              */
    int maxtime             /* I: Max time to wait (seconds)   */
)
{
    int status;             /* return status                   */
    int gotit=0;            /* got the lock? (1=yes)           */

    while ((!gotit) && (maxtime>0))
    {
        if ( lockf(fd,F_ULOCK,0) == 0)
            gotit = 1;
        else
        {
            sleep(5);
            maxtime -= 5;
        }
    }

    if (maxtime <= 0)
        return (E_FAIL);

    status = close (fd);
    if (status == EOF)
        return (E_FAIL); 

    return (E_SUCC);
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * unlock:
 * @fd:  I: File descriptor for specified host file.
 * @maxtime: I: Maximum time to wait while attempting to unlock
 *              the file.
 *
 * The unlock() routine calls lockf, as does cmlock, but
 * passes LOCK_UN to the operation argument.  This means unlock.
 * The unlock() routine attempts to unlock the file until 
 * it gets unlocked or the maximum wait time is reached.
 *
 * Deprecated.  Use las_unlock() instead.
 *
 * Returns:  E_SUCC:  Successful completion of operation.
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int unlock
(
    const int *fd,          /* I: File descriptor              */
    int maxtime             /* I: Max time to wait (seconds)   */
)
{
    int status;             /* return status                   */
    
    status = las_unlock (*fd,maxtime);

    return (status);

}


#endif /* obsolete interfaces */


