/*******************************************************************************
Purpose:
    This routine defines the line types for a given coverage of the 
    DCW coverages of interest.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
  5.1           2/93    J. Hemmer         CSB       Was clipping last character
                                                    in vdt.desc

 
PROJECT:  LAS

*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "dcw.h"

int dcw_definecover
(
    char *cdpath,        /* location of CD drive                       */
    char *library,       /* DCW library (ex. noamer)                   */
    struct DCW_COVERAGE *coverage      /* DCW coverage                               */
)
   {
   FILE		*fd;		/* File descriptor for value desc. table(VDT) */
   long		headrsize;	/* size of header in VDT		      */
   long		firstrec;	/* file offset of first LT record	      */
   long		i, j;		/* loop counter				      */
   char 	path[CMLEN];	/* path to CDT file			      */
   char 	vdtattrib[16];	/* attribute type of vdt record		      */
   struct	
   DCW_VDT	vdt;		/* VDT record				      */

  /*
   *  *** if vdtlist is undefined, get line type from VDT file
   */
   strcpy(path, cdpath);	
   strcat(path, "/dcw/");
   strcat(path, library);
   strcat(path, "/");
   strncat(path, coverage->name, 2);
   strcat(path, "/");
   strcat(path, DCW_VDTFILE); 

   if ((fd = fopen(path, "r")) == NULL)
      return(E_FAIL);

  /*
   *  *** read header size and skip over it
   */
   dcw_read((unsigned char *)&headrsize, sizeof(long), 1, fd);
   fseek(fd, headrsize, 1); 

  /*
   *  *** count number of different line types 
   */
   while(dcw_read((unsigned char *)&vdt.id, sizeof(long), 1, fd))
      {
      if (coverage->numvdt == 0)
         firstrec = (long)(ftell(fd) - sizeof(long));

      fseek(fd, 12, 1); /* attribute feature table  */
      fread(vdtattrib, 1, 16, fd);

      dcw_read((unsigned char *)&vdt.attribute, sizeof(long), 1, fd);
      fread(vdt.desc, 1, 50, fd);

     /*
      *  *** if defines a line type, increment counter
      *      otherwise, line types already defined, break out
      *      I'm assuming all line type definitions are contiguos
      */
      if (strncmp(vdtattrib+2, "LNTYPE", 6) == 0)
         coverage->numvdt++;
      else if (coverage->getstatus && strncmp(vdtattrib+2, "LNSTAT", 6) == 0)
         coverage->numstatus++;
      else if (coverage->numvdt > 0)
         break;
      }

  /*
   *  *** allocate memory for each of the line type definitions
   */
   coverage->vdt = (struct DCW_VDT*) malloc(coverage->numvdt*
                                            sizeof(struct DCW_VDT));
   if (coverage->numstatus)
      coverage->status = (struct DCW_VDT*)
                        malloc(coverage->numstatus* sizeof(struct DCW_VDT));
 
  /*
   *  *** read each of the line type definitions
   */
   fseek(fd, firstrec, 0);
   for (i = 0; i < coverage->numvdt; i++)
      {
      dcw_read((unsigned char *)&coverage->vdt[i].id, sizeof(long), 1, fd);
      fseek(fd, 12, 1); /* attribute feature table  */
      fseek(fd, 16, 1); /* attribute table          */

      dcw_read((unsigned char *)&coverage->vdt[i].attribute, sizeof(long),1,fd);
      fread(coverage->vdt[i].desc, 1, 50, fd);

      j = 49;
      while ((coverage->vdt[i].desc[--j] == ' ') && (j > 0));
      coverage->vdt[i].desc[j+1] = '\0';

      coverage->vdt[i].selected = FALSE;
      coverage->vdt[i].loaded = FALSE;
      }

     /*
      *  *** read each line status
      */
      for (i = 0; i < coverage->numstatus; i++)
         {
         dcw_read((unsigned char *)&coverage->status[i].id, sizeof(long),1, fd);
         fseek(fd, 12, 1); /* attribute feature table  */
         fseek(fd, 16, 1); /* attribute table          */
         dcw_read((unsigned char *)&coverage->status[i].attribute,
                  sizeof(long),1,fd);
         fread(coverage->status[i].desc, 1, 50, fd);
         j = 49;
         while ((coverage->status[i].desc[j] == ' ') && (--j > 0));
         coverage->status[i].desc[j] = '\0';

         coverage->status[i].selected = TRUE;
         }

   fclose(fd);
   return(E_SUCC);
   }
