/*******************************************************************************
Purpose:
    This routine returns the latitude of the lower left coordinate for 
    a tile of the Digital Chart of the World.  The tilename is of 
    the form "A/B/31"; where A is the 15 degree longitude letter, 
    B is the 15 degree latitude letter, 3 is the 5 minute column tile,
    and 1 is the 5 minute row tile.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
 
PROJECT:  LAS
*******************************************************************************/
#include "dcw.h"

int dcw_getlat
(
    char *tile		/* pathname of tile of interest		*/
)
   {
   int		lat;		/* return value				*/
  
  /*
   *   *** latitude are alphabetical in 15 degree increments excluding 'I'
   */
   if (tile[2] > 'i')
      lat = 15 + (tile[2] - 'i')*15;
   else 
      lat = -90 + (tile[2] - 'a')*15;

  /*
   *  *** the 15 degree tile are separated into three 5 degree tiles
   */
   lat += (tile[5] - '1') * 5;

   return(lat);
   }


