/*******************************************************************************
Purpose:
   This routine returns the 5 minute row or column number for a 
   given coordinate.  The minflag signals to return the minimum
   value in case the coordinate falls on a 5 minute boundary.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
 
PROJECT:  LAS
*******************************************************************************/
#include <math.h>
#include "las.h"
#include "dcw.h"

char dcw_getrowcol
(
    double coord,		/* lat/lon coordinate of interest	*/
    int minflag	/* flag denoting min value requested 	*/
)
   {
   float	modvalue;	/* modula 15 value			*/
   char 	retval;		/* character value returned		*/

   modvalue = abs(fmod(coord,15.0)); 

  /*
   *  *** determine row/column number
   */
   if (modvalue == 0.0)
      {
      if (minflag)
         retval = '1';
      else
         retval = '3';

      return(retval);
      }
   else if (coord >= 0.0)
      retval = '1' + (int)(modvalue/5.0);
   else 
      retval = '3' - (int)(modvalue/5.0);

  /*
   *  *** if on tile boundary and minflag, decrement 
   */
   if (fmod(modvalue,5.0) == 0)
      {
      if (!minflag && (coord > 0.0)) 
         retval--;
      else if (minflag && coord < 0.0)
         retval++;
      }

   return(retval);
   }

