/*******************************************************************************
Purpose:
    This routine returns the tile ID for the given tile pathname.
    The tile area feature table associates tile ID's with a tile
    pathname.  Tile pathnames are of the form A/B/31;  where
    A is the 15 minute longitude tile, B is the 15 minute latitude tile,
    3 is the 5 minute longitude tile and 1 is the 5 minute latitude tile.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
 
PROJECT:  LAS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "dcw.h"

int dcw_gettileid
(
    char *cdpath,	/* location of cd drive			*/
    char *lib,		/* DCW library (ex. eurnasia) 		*/
    char *tilename	/* DCW library (ex. eurnasia) 		*/
)
   {
   FILE		*fd;		/* file descriptor for tileref.aft	*/
   char		fullpath[256]; 	/* full pathname of tileref.aft file	*/
   long		headrsize,	/* size of aft header			*/
   		id;		/* id of tile record			*/
   char		tile[7];	/* tile pathname			*/
   
  /*
   *  *** generate pathname of tile area feature table
   */
   strcpy(fullpath, cdpath);
   strcat(fullpath, "/dcw/");
   strcat(fullpath, lib);
   strcat(fullpath, "/tileref/tileref.aft");
   
  /*
   *  *** open area feature table
   */
   if ((fd = fopen(fullpath, "r")) == NULL)   
      return(E_FAIL);
  
  /*
   *  *** read and skip over header
   */
   dcw_read((unsigned char *)&headrsize, sizeof(long), 1, fd);
   fseek(fd, headrsize, 1);

  /*
   *  *** while not EOF
   */
   while (dcw_read((unsigned char *)&id, sizeof(long), 1, fd) != NULL)
      {
      fread(tile, 1, 6, fd);

     /*
      *  *** change backslash (DOS) to slash (UNIX)
      */
      tile[1] = '/';
      tile[3] = '/';
      tile[6] = '\0';

     /*
      *  *** if name matches requested tile, return the id
      */
      if (strcasecmp(tile, tilename) == 0)
         {
         fclose(fd);
         return(id);
         }
      }

   fclose(fd);
   return(E_FAIL);
   }

