/*******************************************************************************
Purpose:
   This routine returns the offset of a tiles lines within the given
   tiles line feature table.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
 
PROJECT:  LAS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "dcw.h"

int dcw_getoffset
(
    char *cdpath,        /* location of cd drive			*/
    char *lib,           /* current DCW library (i.e. CD         */
    char *cover,         /* coverage of interest         	*/
    int tileid,		/* id of tile of interest		*/
    long *recnum,	/* byte recnum file (output)		*/
    long *numelem	/* number of elements in lft (output)	*/
)
   {
   int          headrsize;      /* size of LFT header                   */
   int		ncodes;		/* number of codes 			*/
   int		nrows;		/* number of rows			*/
   char		itype;		/* type of index			*/
   char		ftype;		/* field type				*/
   int		nperdir;	/* number of elements per dir 		*/
   char		dtype;		/* data type for data portion		*/

   FILE		*fd;		/* File descriptor of thematic index	*/
   char         fullpath[256];  /* full pathname of LFT                 */
   short	tile;		/* tile in record			*/
   long		i,		/* loop counter				*/
       		off,		/* offset of record			*/
       		num;		/* number of elements of record		*/
      
    
  /*
   *  *** generate path to line thematic index
   */
   strcpy(fullpath, cdpath);
   strcat(fullpath, "/dcw/");
   strcat(fullpath, lib);
   strcat(fullpath, "/");
   strcat(fullpath, cover);
   strcat(fullpath, "/");
   strcat(fullpath, cover);
   strcat(fullpath, "line.lti");

  /*
   *  *** open thematic index
   */
   if ((fd = fopen(fullpath, "r")) == NULL)
      return(E_FAIL);
 
  /*
   *  *** read header information
   */
   dcw_read((unsigned char *)&headrsize, sizeof(long), 1, fd);
   dcw_read((unsigned char *)&ncodes, sizeof(long), 1, fd);
   dcw_read((unsigned char *)&nrows, sizeof(long), 1, fd);
   dcw_read((unsigned char *)&itype, sizeof(char), 1, fd);
   dcw_read((unsigned char *)&ftype, sizeof(char), 1, fd);
   dcw_read((unsigned char *)&nperdir, sizeof(long), 1, fd);
   dcw_read((unsigned char *)&dtype, sizeof(char), 1, fd);

   fseek(fd, 60, 0);

  /*
   *  *** read records until tile of interest is found
   */
   for (i = 0; i < ncodes; i++)
      {
      dcw_read((unsigned char *)&tile, sizeof(short), 1, fd);
      dcw_read((unsigned char *)&off, sizeof(long), 1, fd);
      dcw_read((unsigned char *)&num, sizeof(long), 1, fd);
     
     /*
      *  *** if is the tile of interest
      */
      if (tile == tileid)
         {
        /*
         *  *** compute number of records based on offset and data type
         */
         if (dtype == 'I')
            *recnum = (off - headrsize)/(long)sizeof(long);
         else
            *recnum = (off - headrsize)/(long)sizeof(short);

         *numelem = num;
         fclose(fd);
         return(E_SUCC);
         }
      }

   fclose(fd);
   return(E_FAIL);
   }

