/*******************************************************************************
Purpose:
    This routine returns a list of the ID's of all line fitting the
    define query and the list of their corresponing line types.
    The number of lines found is returned.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
  5.2		07/93	D. Etrheim	  CSB       Fix error messages
 
PROJECT:  LAS
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "dcw.h"

int dcw_linequery
(
    char *cdpath,        /* location of CD drive		 	 */
    char *lib,           /* current DCW library (i.e. CD          */
    struct DCW_COVERAGE *coverage,      /* coverage of interest         	 */
    int **line,		/* list of request line ID's		 */
    int **type,		/* list of request line types		 */
    int tileid,         /* only for debuggin		         */
    int startrec,	/* first record to read			 */
    int numrec		/* number of records to read		 */
)
   {
   struct	
   DCW_LFT	lft;		/* line feature table			*/
   FILE         *fd;		/* descriptor of line feature table	*/
   int          headrsize;      /* size of LFT header                   */
   int		checkstatus,	/* flag denoting status check required  */
		i,		/* loop counter			        */
		maxline,	/* amount of space allocaed in line list */
   		numline;	/* number of lines found		*/
   long		fatal=-1;	/* fatal error flag			*/
   char         fullpath[256];  /* full pathname of LFT                 */
 
  /*
   *  *** generate pathname of line feature table
   */
   strcpy(fullpath, cdpath);
   strcat(fullpath, "/dcw/");
   strcat(fullpath, lib);
   strcat(fullpath, "/");
   strcat(fullpath, coverage->name);
   strcat(fullpath, "/");
   strcat(fullpath, coverage->name);
   strcat(fullpath, "line.lft");
 
  /*
   *  *** open line feature table
   */
   if ((fd = fopen(fullpath, "r")) == NULL)
      return(E_FAIL);

  /*
   *  *** initialize line list
   */
   maxline = 100;
   *line = (int*)malloc(100*sizeof(int));
   *type = (int*)malloc(100*sizeof(int));

  /*
   *  *** skip to first record
   */
   if (!dcw_read((unsigned char *)&headrsize, sizeof(long), 1, fd))
      return(E_FAIL);

  /*
   *  *** utilties or physiography records are only 14 bytes
   */
   if ((coverage->name[0] == 'u') || (strcmp(coverage->name, "ph") == 0))
      fseek(fd, headrsize + startrec*14, 1);
   else
      fseek(fd, headrsize + startrec*18, 1);

   numline = 0;

  /*
   *  *** if coverage has line status and all are requested,
   *          do not check status
   */
   checkstatus = FALSE;

   if (coverage->getstatus)
      {
      for (i = 0; i < coverage->numstatus; i++)
         {
         if (!coverage->status[i].selected)
            {
            checkstatus = TRUE;
            break;
            }
         }
      }

  /*
   *  *** for each record
   */
   for (i = 0; i < numrec; i++)
      {
      if (!dcw_read((unsigned char *)&lft.id, sizeof(long), 1, fd))
         return(E_FAIL);

     /*
      *  *** the hypsography definition is in reverse 
      */
      if (coverage->name[0] == 'h')
         {
         dcw_read((unsigned char *)&lft.linestatus, sizeof(long), 1, fd);
         dcw_read((unsigned char *)&lft.linetype, sizeof(long), 1, fd);
         }
      else
         {
         dcw_read((unsigned char *)&lft.linetype, sizeof(long), 1, fd);
        /*
         *  *** utilities and physiography do not have line status field
         */
         if ((coverage->name[0] != 'u') && (strcmp(coverage->name, "ph") != 0))
            dcw_read((unsigned char *)&lft.linestatus, sizeof(long), 1, fd);
         }

      dcw_read((unsigned char *)&lft.tile, sizeof(short), 1, fd);
      dcw_read((unsigned char *)&lft.edge, sizeof(long), 1, fd);

     /*
      *  *** if this line type is requested and not loaded, add to the list
      */
      if (dcw_checktype(coverage, lft.linetype))
         {
         if (!checkstatus || dcw_checkstatus(coverage, lft.linestatus))
            {
            *(*line+numline) = lft.edge;
            *(*type+numline) = lft.linetype;
            numline++;
   
           /*
            *  *** allocated memory as needed
            */
            if (numline == maxline)
               {
               maxline += 100;
               *line = (int*)realloc(*line, maxline*sizeof(int));
               if (*line == NULL)
                  c_errmsg("Error reallocating memory","dcw_lquery-alloc",
                           &fatal);
               *type = (int*)realloc(*type, maxline*sizeof(int));
               if (*type == NULL)
                  c_errmsg("Error reallocating memory","dcw_lquery-alloc",
                           &fatal);
               }
            }
         }
      }

   fclose(fd);
   return(numline);
   }


