/*******************************************************************************
Purpose:
    This routine performs a query on the DCW for the given library,
    coverage, and area of interest.
 
PROGRAM HISTORY:
VERSION         DATE    AUTHOR          CODE/CONT   REASON
-------         ----    ------          ---------   ------
  5.1           11/92   J. Hemmer         CSB       Original development
 
PROJECT:  LAS
*******************************************************************************/
#include "las.h"
#include "dcw.h"
 
int dcw_query
(
    char *cdpath,	/* location of CD drive			*/
    char *library,	/* DCW library name			*/
    struct DCW_COVERAGE *dcw_coverage,/* defines DCW categories information     */
    struct DCW_VECTOR *dcw_vector,	/* resulting vector information		  */
    double lllat,	/* bounding rectangle of area of interest */
    double lllon,
    double urlat,
    double urlon
)
   {
   long		  i,j,k,	/* loop counters			*/
		  r,c;
   long           numlft,	/* number of lft records                */
                  numline,	/* number of lines found in query	*/
                  recnum,	/* record number in LFT                 */
                  status,	/* Funciton return status		*/ 
                  tileid;	/* DCW tile ID				*/ 
   int            *line,	/* list of selected line id's           */
                  *type; 	/* list of selected line types          */

   char		  endcol, 	/* ending 5 degree column		*/
		  endrow,	/* ending 5 degree row			*/
   		  minlontile, 	/* first 15 degree longitude tile	*/
		  maxlontile,	/* last 15 degree longitude tile	*/
                  minlattile, 	/* first 15 degree latitude tile	*/
		  maxlattile, 	/* last 15 degree latitude tile 	*/
                  startcol, 
  		  startrow,     /* starting 5 degree row/column     	*/
		  tilename[7];	/* pathname of tile of interest     	*/

  /*
   *  *** get 15 degree tile boundaries
   */
   minlontile = dcw_lontile(lllon, TRUE);
   maxlontile = dcw_lontile(urlon, FALSE);
   minlattile = dcw_lattile(lllat, TRUE);
   maxlattile = dcw_lattile(urlat, FALSE);

  /*
   *  *** determine 5 minute boundaries
   */
   startcol = dcw_getrowcol(lllon, TRUE);
   endcol   = dcw_getrowcol(urlon, FALSE);
   startrow = dcw_getrowcol(lllat, TRUE);
   endrow   = dcw_getrowcol(urlat, FALSE);
 
  /*
   *  *** for each longitude 15 minute tile
   */
   for (i = minlontile; i <= maxlontile; i++)
      {
     /*
      *  *** for each latitude 15 minute tile
      */
      for (j = minlattile; j <= maxlattile; j++)
         {
        /*
         *  *** for each 5 minute latitude within this 15 minute tile
         */
         for (r = '1'; r < '4'; r++)
            {
            if (((j == minlattile) && (r < startrow)) ||
                ((j == maxlattile) && (r > endrow)))
               continue;
 
           /*
            *  *** for each 5 minute longitude within this 15 minute tile
            */
            for (c = '1'; c < '4'; c++)
               {
               if (((i == minlontile) && (c < startcol)) ||
                   ((i == maxlontile) && (c > endcol)))
                  continue;
 
              /*
               *  *** generate tile pathname
               */
               tilename[0] = (char)(i - ('A' - 'a'));
               tilename[1] = '/';
               tilename[2] = (char)(j - ('A' - 'a'));
               tilename[3] = '/';
               tilename[4] = (char)c;
               tilename[5] = (char)r;
               tilename[6] = '\0';
 
              /*
               *  *** get tile ID from tile Area Feature Table
               */
               if ((tileid=dcw_gettileid(cdpath, library, tilename)) == E_FAIL)
                  continue;
 
              /*
               *  *** for each coverage type
               */
               k = 0;
               while (dcw_coverage[k].desc != NULL)
                  {
		  if (dcw_coverage[k].numvdt == 0)
			{
			k++;
			continue;
			}

                 /*
                  *  *** find offset into Line Feature Table for this tile
                  */
                  if (dcw_getoffset(cdpath, library, dcw_coverage[k].name,
                                    tileid, &recnum, &numlft) == E_SUCC)
                     {
                    /*
                     *   *** get line and type list for this tile
                     */
                     numline = dcw_linequery(cdpath, library, &dcw_coverage[k], 
                                             &line, &type, tileid, recnum, 
                                             numlft);
 
                    /*
                     *  *** extract requested lines from the tile
                     */
                     if (numline > 0)
                        {
                        status = dcw_readlines(cdpath,library,&dcw_coverage[k],
                                      &dcw_vector->line, &dcw_vector->numlines,
                                      &dcw_vector->maxline, tilename,
                                      line, type, numline, 
                                      lllat, lllon, urlat, urlon, TRUE);
			if (status != E_SUCC)
			    return(E_FAIL);
                        }
 
                    /*
                     *  *** free line and type lists
                     */
                     free(line);
                     free(type);
                     }

                  k++;
                  }
               }
            }
         }
      }

   return(E_SUCC);
   }
