/******************************************************************************

NAME:   CKMNMX

FUNCTION:
        CKMNMX is a function which is called by MINMAX to 
        read the DDR file for the specified image.  

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         5/86       B.Ailts      initial development
    2.0         8/86       K. Gacke     used the label service routines
    3.0         9/86       K. Gacke     used host file names instead of TAE
                                        file names
    4.0         7/88       B.Ailts      LAS 5.0 standards
    5.0        12/90       B.Ailts      Updated error messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:        LAS

ALGORITHM DESCRIPTION:
For each band specified
   GETBDR is called to retreive the min, max, and valid values of DDR
   Bddr.valid is copied into the valid array
   If valid 
      Bddr.minval is copied into the minval array
      Bddr.maxval is copied into the maxval array

ALGORITHM REFERENCES:

******************************************************************************/
#include <stdio.h>
#include "las.h"
#include "local_protos.h"

int ckmnmx
(
    const  char *hname,    /* I: host image name                             */
    int    *bands,         /* I/O: array containing bands that were processed*/
    int    nbands,         /* I: # of bands processed                        */
    double *minval,        /* I/O: pointer to array of minimum pixel values  */
    double *maxval,        /* I/O: pointer to array of maximum pixel values  */
    int    *valid          /* O: array containing a flag stating if valid    */
)

{
    struct  BDDR  bddr;   /* band ddr structure                             */
    int     i;            /* counter -- current band                        */
    int     status;       /* function return status code                    */

    /*  Retreive the min/max and validity values from the DDR file of all
        needed bands                                                        */
    for (i = 0; i < nbands; i++)
    {
        status = las_getbdr(hname,&bddr,(bands[i]));
        if (status != E_SUCC)
        {
            c_lasmsg (FALSE, "minmax-call", "Error returned from getbdr");
            return(E_FAIL);
        }
        valid[i] = bddr.valid;
        if (valid[i] == VALID)
        {
            minval[i] = bddr.minval;
            maxval[i] = bddr.maxval;
        }
    }

    return(E_SUCC);
}  /*  ckmnmx  */
