/*****************************************************************************
        NAME:  CONVERT_DDR

        PURPOSE:  To convert the ddr's corner coordinates and projection
                  distance to the projection units specified by the input
                  units.  The ddr fields are converted from DMS to degrees
                  for calculations or from degrees to DMS to restore the
                  ddr to its original units.

        PROGRAM HISTORY:
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        J.Janssen       6/98    Original development

        COMPUTER HARDWARE AND/OR SOFTWRE LIMITATIONS:
                Must be run under TAE

        PROJECT:        LAS

        ALGORITHM:
        If the input parameter "units" is degrees, convert ddr corner
        coordinates and projection distance from DMS to total degrees.
        Else if "units" is DMS, convert ddr corner coordinates and
        projection distance from degrees to DMS.
        Else error - invalid projection unit specified.

        ALGORITHM REFERENCES            none

******************************************************************************/

#include <string.h>
#include "las.h"
#include "geompak.h"
#include "local_protos.h"

int convert_ddr
(
    struct DDR *ddr,       /* I/O: input ddr                                */
    const char *units      /* I: units the ddr fields are to be converted to*/
)

{


    /*  if units are degrees then convert ddr corner coordinates
        and projection distance from DMS to total degrees                   */
    if (strcmp(units,"DEG") == 0)
    {
        c_decdeg(&ddr->upleft[0],"DMS","LAT");
        c_decdeg(&ddr->loleft[0],"DMS","LAT");
        c_decdeg(&ddr->upright[0],"DMS","LAT");
        c_decdeg(&ddr->loright[0],"DMS","LAT");
        c_decdeg(&ddr->upleft[1],"DMS","LON");
        c_decdeg(&ddr->loleft[1],"DMS","LON");
        c_decdeg(&ddr->upright[1],"DMS","LON");
        c_decdeg(&ddr->loright[1],"DMS","LON");
        c_decdeg(&ddr->pdist_y,"DMS","LAT");
        c_decdeg(&ddr->pdist_x,"DMS","LON");
    }

    /*  else if units are DMS then convert ddr corner coordinates
        and projection distance from degrees to DMS.                        */
    else if (strcmp(units,"DMS") == 0)
    {
        c_degdms(&ddr->upleft[0],&ddr->upleft[0],"DEG","LAT");
        c_degdms(&ddr->loleft[0],&ddr->loleft[0],"DEG","LAT");
        c_degdms(&ddr->upright[0],&ddr->upright[0],"DEG","LAT");
        c_degdms(&ddr->loright[0],&ddr->loright[0],"DEG","LAT");
        c_degdms(&ddr->upleft[1],&ddr->upleft[1],"DEG","LON");
        c_degdms(&ddr->loleft[1],&ddr->loleft[1],"DEG","LON");
        c_degdms(&ddr->upright[1],&ddr->upright[1],"DEG","LON");
        c_degdms(&ddr->loright[1],&ddr->loright[1],"DEG","LON");
        c_degdms(&ddr->pdist_y,&ddr->pdist_y,"DEG","LAT");
        c_degdms(&ddr->pdist_x,&ddr->pdist_x,"DEG","LON");
    }

    /*  else error - invalid projections units specified                    */
    else
    {
        c_lasmsg (FALSE, "upddr-convert", "Invalid projection units specified");
        return(E_FAIL);
    }

    return(E_SUCC);
}

