#ifndef LOCAL_PROTOS_H
#define LOCAL_PROTOS_H

/*  Below are constants used only within the DDR support routines
----------------------------------------------------------------------------*/
#define USAME 2            /* unknown and equal validity flag               */
#define UDIFF 3            /* unknown and not equal validity flag           */
#define DBSIZE 2           /* number of doubles in band record(s) of DDR    */
#define DDTSLN 152         /* length of all strings in band record of DDR   */
#define DDSRLN 32          /* length of the SOURCE character string         */
#define DDINLN 32          /* length of the INSTRUMENT character string     */
#define DDDRLN 64          /* length of the DIRECTION character string      */
#define DDCDLN 10          /* length of the capture DATE character string   */
#define DDBNLN 4           /* length of the band no. character string       */
#define DDVLLN 2           /* length of the valid flag character string     */
#define DDPULN 12          /* length of the PROJ_UNITS character string     */
#define DDLDLN 12          /* length of the LAST_USED_DATE char. string     */
#define DDLTLN 12          /* length of the LAST_USED_TIME char. string     */

int ckmnmx
(
    const char *hname,     /* I: host image name                            */
    int  *bands,           /* I/O: array containing bands processed         */
    int  nbands,           /* I: # of bands processed                       */
    double *minval,        /* I/O: pointer to array of minimum pixel values */
    double *maxval,        /* I/O: pointer to array of maximum pixel values */
    int  *valid            /* O: array containing a flag stating if valid   */
);

int clmnmx
(
    const char *hostname,  /* I: host name of input image                   */
    int  *window,          /* O: window specified for the input image       */
    int  *cl_bands,        /* I: bands that the min/max values are needed   */
    const int *mmbands,    /* I: the correct place in the min/max arrays that
                                 the min/max values should be placed        */
    double *minval,        /* I/O: minimum value array                      */
    double *maxval         /* I/O: maximum value array                      */
);

void com_ppar
(
    const struct DDR ddr1, /* I: First input DDR                            */
    const struct DDR ddr2, /* I: Second input DDR                           */
    int  *ppar_flag,       /* O: Processing projection parameter flag       */
    int  default_flag,     /* I: default flag -- UNKNOW or VALID            */
    int  image1,           /* I: number of first input image                */
    int  image2            /* I: number of second input image               */
);

int convert_ddr
(
    struct DDR *ddr,       /* I/O: input ddr                                */
    const char *units      /* I: units the ddr fields are to be converted   */
);

int upmnmx
(
    const char *hname,     /* I: input host image name                      */
    int  *cl_bands,        /* I/O: bands of image that were processed       */
    int  *mmbands,         /* I/O: bands of min/max arrays to be used       */
    double minval[],       /* I/O: minimum intensity values                 */
    double maxval[]        /* I/O: maximum intensity values                 */
);



#endif







