/****************************************************************************** NAME:      main_function

PURPOSE:   This was written for the purpose of testing c_recalc_pixel_size

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"

int main
(void)

{
    double upleft_y[8];    /* Upper left corner of image in y direction    */
    double loleft_y[8];    /* Lower left corner of image in y direction    */
    double loright_y[8];   /* Lower right corner of image in y direction   */
    double upleft_x[8];    /* Upper left corner of image in x direction    */
    double loleft_x[8];    /* Lower left corner of image in x direction    */
    double loright_x[8];   /* Lower right corner of image in x direction   */
    double pixel_y;        /* The test outcome of the pixel size in the 
                              y direction                                  */ 
    double pixel_x;        /* The test outcome of the pixel size in the 
                              x direction                                  */
    float orig_pixel_y[8]; /* What the result should be in the y direction */
    float orig_pixel_x[8]; /* What the result should be in the x direction */
    int index;             /* variable used for indexing                   */
    int nl[8];             /* Total number of line in the image            */
    int ns[8];             /* Total number of samples in the image         */

    /* TEST CASE 1 
       These coordinates represent an image that is not rotated.  The
       output pixel value should be 30.0 meters in both the y and x 
       direction. 
    ---------------------------------------------------------------*/
    upleft_y[0] = 4568340.0;
    upleft_x[0] = 390780.0;
    loleft_y[0] = 4361010.0;
    loleft_x[0] = 390780.0;
    loright_y[0] = 4361010.0;
    loright_x[0] = 623400.0;
    nl[0] = 6912;
    ns[0] = 7755; 
    orig_pixel_y[0] = 30.0;
    orig_pixel_x[0] = 30.0;

    /* TEST CASE 2
       These coordinates represent an image that is rotated 45 degrees. 
       The pixel size in the y nd x direction should be 30.0 meters. 
    -----------------------------------------------------------------------*/
    upleft_y[1] = 3506627.380718;
    upleft_x[1] = -2953981.004814;
    loleft_y[1] = 3360022.931774;
    loleft_x[1] = -2807376.555870;
    loright_y[1] = 3524510.111214;
    loright_x[1] = -2642889.376431;
    nl[1] = 6912;
    ns[1] = 7755;
    orig_pixel_y[1] = 30.0;
    orig_pixel_x[1] = 30.0;

    /* TEST CASE 3
       These coordinates represent an image that is rotated 90 degrees. The 
       output pixel value should be 28.5 meters in both y and x direction.
    ------------------------------------------------------------------------*/
    upleft_y[2] = 8806233.39742616;
    upleft_x[2] = 483138.999557148;
    loleft_y[2] = 8777149.46805352;
    loleft_x[2] = 678750.185163946;
    loright_y[2] = 8955141.83380093;
    loright_x[2] = 705214.507120837;
    nl[2] = 6940;
    ns[2] = 6315;
    orig_pixel_y[2] = 28.5;
    orig_pixel_x[2] = 28.5;


   /* TEST CASE 4
      These coordinates represent an image that is rotated 120 degrees. The
      pixel size in the y and x direction are not equal.  The pixel size
      in the y direction should be 30.0 meters and 15.0 meters in the x 
      direction. (NOTE:  If the image was not rotated, the pixel distance in
      the y direction would be 15.0 and 30.0 in the x direction.)
    -----------------------------------------------------------------------*/
    upleft_y[3] = -1945744.592709;
    upleft_x[3] = -4151688.493125;
    loleft_y[3] = -1842079.592709;
    loleft_x[3] = -3972135.446158;
    loright_y[3] = -1640624.763281;
    loright_x[3] = -4088445.446158;
    nl[3] = 13823;
    ns[3] = 7755;
    orig_pixel_y[3] = 30.0;
    orig_pixel_x[3] = 15.0;

    /* TEST CASE 5
       These coordinates represent an image that is rotated 180 degrees. The 
       pixel size in the y and x direction are not equal. The pixel size in the
       y direction should be 15.0 meters and 30.0 meters in the x direction.  
    ------------------------------------------------------------------------*/
    upleft_y[4] = -4568340.000000;
    upleft_x[4] = -390780.000000;
    loleft_y[4] = -4361010.000000;
    loleft_x[4] = -390780.000000;
    loright_y[4] = -4361010.000000;
    loright_x[4] = -623400.000000;
    nl[4] = 13823;
    ns[4] = 7755;
    orig_pixel_y[4] = 15.0;
    orig_pixel_x[4] = 30.0;

    /* TEST CASE 6
       These coordinates represent an image that is rotated 210 degrees. The 
       pixel size in the y and x direction should be 28.5 meters.
    -----------------------------------------------------------------------*/
    upleft_y[5] = -7867991.333605;
    upleft_x[5] = 3984706.051538;
    loleft_y[5] = -7940609.504729;
    loleft_x[5] = 3800759.830851;
    loright_y[5] = -8107987.576125;
    loright_x[5] = 3866837.238616;
    nl[5] = 6940;
    ns[5] = 6315;
    orig_pixel_y[5] = 28.5;
    orig_pixel_x[5] = 28.5;

    /* TEST CASE 7
       These coordinates represent an image that is rotated 270 degrees. The 
       pixel size in the y and x direction are not equal.  The pixel size in
       the y direction should be 60.0 meters and 30.0 meters in the x 
       direction. (NOTE:  If the image was not rotated, the pixel distance in
       the y direction would be 30.0 and 60.0 in the x direction.)
    -------------------------------------------------------------------------*/
    upleft_y[6] = -390780.000000;
    upleft_x[6] = 4568340.000000;
    loleft_y[6] = -390780.000000;
    loleft_x[6] = 4361010.000000;
    loright_y[6] = -623400.000000;
    loright_x[6] = 4361010.000000;
    nl[6] = 6912;
    ns[6] = 3878;
    orig_pixel_y[6] = 60.0;
    orig_pixel_x[6] = 30.0;

    /* TEST CASE 8
       These coordinates represent an image that is rotated 330 degrees. The 
       pixel size in the y and x direction should be 28.5 meters.
    ------------------------------------------------------------------------*/
    upleft_y[7] = 7384852.334047;
    upleft_x[7] = 4821527.345889;
    loleft_y[7] = 7261859.319565;
    loleft_x[7] = 4976389.637202;
    loright_y[7] = 7402773.069004;
    loright_x[7] = 5088304.595184;
    nl[7] = 6940;
    ns[7] = 6315;
    orig_pixel_y[7] = 28.5;
    orig_pixel_x[7] = 28.5;

    /* Calculate the pixel size
    --------------------------*/
    for (index = 0; index < 8; index++)
    {
        if (c_recalc_pixel_size(&pixel_y,&pixel_x,upleft_y[index],
                                loleft_y[index],loright_y[index],
                                upleft_x[index],loleft_x[index], 
                                loright_x[index],nl[index],ns[index]) != 0)
        {
            printf("Error recalculating pixel size");
            exit(1);
        }

        /* Compare test pixel with the original pixel results and
           output them to the test.log
        ---------------------------------------------------------*/
        if (orig_pixel_y[index] != (float)pixel_y)    
        {
            printf("Test Case %d FAILED. Pixel y does not equal %f \n",
                     index+1, orig_pixel_y[index]); 
        }
        else
        {
            printf("Test Case %d PASSED. Pixel y equals %f \n",
                    index+1, orig_pixel_y[index]);
        }
        
        if (orig_pixel_x[index] != (float)pixel_x)
        {
            printf("Test Case %d FAILED. Pixel x does not equal %f \n",
                    index+1, orig_pixel_x[index]);
        }
        else
        {
            printf("Test Case %d PASSED. Pixel x equals %f \n",
                    index+1, orig_pixel_x[index]);
        }
    }    

    return 0;

}
