/*****************************************************************************
NAME:    c_dkbuf

PURPOSE: Returns an appropriate buffer size based on the given hostname from
         a table in $LASTABLES

Programmer       Date   Reason
---------------  -----  ------------------------------------------------------
C. Engebretson   05/96  New development to allow system-dependent buffer size

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>  /* gethostname prototype                               */
#include "tae.h"
#include "worgen.h"
#include "local_protos.h"

int c_dkbuf
(
    int *bufsize            /* O: Size of buffer                            */
)
{
    FILE   *file_ptr;       /* File pointer to input file                   */

    char host_name[CMLEN];  /* Host system name                             */
    char *wor_ptr;          /* Pointer to env variable                      */
    char filenm[CMLEN];     /* File name that contains system buffer sizes  */
    char temp[CMLEN];       /* Temp working string                          */
    char *ptr;              /* Pointer to printer name                      */
    int found = FALSE;      /* Loop control variable                        */
    int  len;               /* Length of string                             */
    long one = 1;           /* Constant of one                              */

    /* Get the directory of the file */
    wor_ptr = getenv("LASTABLES");
    if (wor_ptr == NULL)
    {
        c_errmsg("Global variable LASTABLES not defined","c_dkbuf-env",&one);
        return(E_FAIL);
    }

    /* Add the file name  */
    sprintf(filenm,"%s/disk.config",wor_ptr);

    /* Open the file.  Assume a default size of 5M if there are problems.   */
    if ((file_ptr = fopen(filenm, "r")) == NULL)
    {
        c_errmsg("Error opening disk.config file","c_dkbuf-open",&one);
        *bufsize = 5242880;
    }
    else
    {
        /* Read each line of the file until a match is found with 
           the system name or the default buffer size is found.             */
        gethostname(host_name, CMLEN);
        len = strlen(host_name);
        while (!found)
        {
            fscanf(file_ptr, "%s", temp);
            if ((strncmp(temp,"default",7) == 0)   ||
                (strncmp(host_name,temp,len) == 0))
            {
                found = TRUE;
                ptr = strchr(temp,':');
                ptr++;
                *bufsize = atol(ptr);
            }
        }
        fclose(file_ptr);
    }

    return(E_SUCC);
}
