/******************************************************************************
FUNCTION:	DCLOSE

PURPOSE:	Part of the DITTO program which closes a disk file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE.  It is also assumed that this routine is used
	only by the DITTO program.

PROJECT		LAS

ALGORITHM 
	The input parameter is the disk file descriptor.  From the disk file
 	descriptor the C function to close the disk file is called.

	If running under the VMS operating system, all files are opened using
	the file descriptor.  The file attribute list is built based on the
	input parameter "disk".

	If running under the UNIX operating system, files with carriage return
	carriage control are opened by file pointer (buffered I/O), otherwise
	it is opened by file descriptor.


ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file		     */ 
#include "ditto.h"			/* DITTO include file                */

int dclose
(
    struct DISK_FD *fd		/* DITTO disk file descriptor        */
)
{
register long status;			/* function return status code       */

/*
  If the carrage conrol is CARRIAGE
*/
if (fd->cc == CARRIAGE)
      {
      /*
        Flush anything that was in the buffer
      */
      status = fflush(fd->fp);

      /*
        Close the file
      */
      status = fclose(fd->fp);
      if (status != 0)
         return(E_FAIL);
      }
else
      {
      /*
        Close the file
      */
      status = close(fd->fd);
      if (status == -1)
         return(E_FAIL);
      }

return(E_SUCC);
}
