/******************************************************************************
FUNCTION:	DOPEN

PURPOSE:	Part of the DITTO program which creates/opens a disk file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1      04/93    D. Etrheim          Changed mode to 666 for file open


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE.  It is also assumed that this routine is used
	only by the DITTO program.

PROJECT			LAS

ALGORITHM 
	The input parameter is the disk file descriptor.  From the disk file
 	descriptor the C function to create/open the disk file is called.

	If running under the VMS operating system, all files are opened using
	the file descriptor.  The file attribute list is built based on the
	input parameter "disk".

	If running under the UNIX operating system, files with carriage return
	carriage control are opened by file pointer (buffered I/O), otherwise
	it is opened by file descriptor.


ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <string.h>

#ifdef vms
#include <file.h>
#else
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif

#include "las.h"			/* LAS include file                  */
#include "ditto.h"			/* DITTO include file                */

int dopen
(
    struct DISK_FD *fd,         /* DITTO disk file descriptor        */
    long  amode                 /* type of access mode               */
)
{
long mode = 0666;		/* file protection mode              */
long status;			/* function return status code       */

#ifdef vms
char     cmdcc[20];			/* type of carriage control          */
char     cmdblk[20];			/* block size                        */
char     cmdrec[20];			/* record size                       */
char     cmdtyp[20];			/* record type                       */
#endif
char     fname[CMLEN];			/* host file name                    */
char	 ext[CMLEN];			/* Extention location		     */
char	*adir;				/* address of directory within fname */
char	*aext;				/* address of directory within fname */

strcpy(fname,fd->fname);

if (fd->cur_file != NOVAL)
    {
    adir = strrchr(fname,HOST_DIR_END);
    if (adir == 0)
	adir = fname;
    aext = strchr(adir,HOST_EXT);
    if (aext == 0)
	sprintf(fname,"%s_%d",fd->fname,fd->cur_file);
    else
	{
	strcpy(ext,aext);
	sprintf(aext,"_%d%s",fd->cur_file,ext);
	}
    }

/*
*   VMS VERSION
*/
#ifdef vms
   if (fd->cc == CARRIAGE)		/* if carriage return carriage control*/
      {					/*    open disk file with file pointer*/
      switch (amode)			/*    (buffered I/O)                  */
         {
         case DCREATE:			/* create new file                    */
		if (access(fname,0) != 0) /* Test for existance		      */
		    {
      		    fd->fp = fopen(fname,"w");
		    fd->cur_rec = 0;
		    }
		else
		    fd->fp = NULL;
		break;

         case DREAD:			/* open for read access               */
		if (access(fname,4) == 0) /* Test for read access	      */
      		    fd->fp = fopen(fname,"r");
		else
		    fd->fp = NULL;
		break;

         case DWRITE:			/* open for write access              */
		if (access(fname,0) != 0) /* Test for existance		      */
      		    fd->fp = fopen(fname,"w");
		else
		    fd->fp = NULL;
		break;

         case DUPDATE:			/* open for read and write access     */
		if (access(fname,6) == 0) /* Test for update access	      */
      		    fd->fp = fopen(fname,"r+");
		else
		    fd->fp = NULL;
		break;

         case DAPPEND:			/* open for append access             */
		if (access(fname,6) == 0) /* Test for update access	      */
      		    fd->fp = fopen(fname,"a");
		else
		    fd->fp = NULL;
		break;
         }

      if (fd->fp == NULL)
         return(E_FAIL);

      status = dskip(fd,&fd->cur_rec);	/* skip to current record             */
      status = fseek(fd->fp,0,1);	/* required to write to file          */
      }
   else					/* if NOT carriage return carriage    */
      {
      switch (amode)
         {
         case DCREATE:			/* create new file                    */
	   if (access(fname,0) == 0)    /* Test for existance		      */
		{
		fd->fd = -1;
		break;
		}

  	   sprintf(cmdblk,"bls=%d",fd->blksiz);  /* build file attribute list */
 	   sprintf(cmdrec,"mrs=%d",fd->recsiz);  /* based on input parameter  */

   	   switch (fd->rectyp)		         /* "disk"                    */
	      {
	      case NOVAL:

	      case VAR:   sprintf(cmdtyp,"rfm=%s","var"); break;

	      case FIX:   sprintf(cmdtyp,"rfm=%s","fix"); break;

	      default:    printf("\nfile record type = %d",fd->rectyp);
	      }

	   switch (fd->cc)
	      {
	      case CARRIAGE: 
	         sprintf(cmdcc,"rat=%s","cr");
	         fd->fd = creat(fname,mode,cmdblk,cmdrec,cmdcc,cmdtyp);
	         break;

	      case FORTRAN:  
	         sprintf(cmdcc,"rat=%s","ftn");
	         fd->fd = creat(fname,mode,cmdblk,cmdrec,cmdcc,cmdtyp);
	         break;

	      case PRINT:    
	         sprintf(cmdcc,"rat=%s","prn");
	         fd->fd = creat(fname,mode,cmdblk,cmdrec,cmdcc,cmdtyp);
	         break;

	      case NONE:

	      case NOVAL:
	         fd->fd = creat(fname,mode,cmdblk,cmdrec,cmdtyp);
	         break;
	      }
	   break;

         case DREAD:			/* open disk file for read access     */
	    if (access(fname,4) == 0)   /* Test for read access		      */
		fd->fd = open(fname,O_RDONLY,mode);
	    else
		fd->fd = -1;
	   break;

         case DWRITE:			/* open disk file for write access    */
	    if (access(fname,0) != 0)   /* Test for existance		      */
   	        fd->fd = open(fname,O_WRONLY,mode);
	    else
		fd->fd = -1;
	   break;

         case DAPPEND:			/* open disk file for append access   */
	    if (access(fname,6) == 0)   /* Test for update access	      */
   	        fd->fd = open(fname,O_APPEND,mode);
	    else
		fd->fd = -1;
	    break;

         case DUPDATE:			/* open file for read/write access    */
	    if (access(fname,6) == 0)   /* Test for update access	      */
   	        fd->fd = open(fname,O_RDWR,mode);
	    else
		fd->fd = -1;
	   break;
         }

      if (fd->fd == -1)
         return(E_FAIL);

      status = dskip(fd,&fd->cur_rec);	/* skip to current record            */
      }

   return(status);
   }

/*
*   UNIX VERSION
*/
#else
   if (fd->cc == CARRIAGE)		/* if carriage return carriage control*/
      {					/*    open disk file with file pointer*/
      switch (amode)			/*    (buffered I/O)                  */
         {
         case DCREATE:			/* create new file                    */
		if (access(fname,0) != 0) /* Test for existance	      */
		    {
      		    fd->fp = fopen(fname,"w");
		    fd->cur_rec = 0;
		    }
		else
		    fd->fp = NULL;
		break;

         case DREAD:			/* open for read access               */
		if (access(fname,4) == 0) /* Test for read access	      */
      		    fd->fp = fopen(fname,"r");
		else
		    fd->fp = NULL;
		break;

         case DWRITE:			/* open for write access              */
		if (access(fname,0) != 0) /* Test for existance		      */
      		    fd->fp = fopen(fname,"w");
		else
		    fd->fp = NULL;
		break;

         case DUPDATE:			/* open for read and write access     */
		if (access(fname,6) == 0) /* Test for update access	      */
      		    fd->fp = fopen(fname,"r+");
		else
		    fd->fp = NULL;
		break;

         case DAPPEND:			/* open for append access             */
		if (access(fname,6) == 0) /* Test for update access	      */
      		    fd->fp = fopen(fname,"a");
		else
		    fd->fp = NULL;
		break;
         }

      if (fd->fp == NULL)
         return(E_FAIL);

      status = dskip(fd,&fd->cur_rec);	/* skip to current record             */
      status = fseek(fd->fp,0,1);	/* required to write to file          */
      }
   else					/* if NOT carriage return carriage    */
      {					/*    control, open file using the    */
      switch (amode)			/*    file descriptor.                */
          {
          case DCREATE:			/* create new file                    */
		if (access(fname,0) != 0) /* Test for existance		      */
		    {
 		    fd->fd = open(fname,O_CREAT+O_TRUNC+O_RDWR,mode);
		    fd->cur_rec = 0;
		    }
		else
		    fd->fd = -1;
		break;

          case DREAD:			/* open for read access               */
		if (access(fname,4) == 0) /* Test for read access	      */
    		    fd->fd = open(fname,O_RDONLY,mode);
		else
		    fd->fd = -1;
		break;

          case DWRITE:			/* open for write access              */
		if (access(fname,0) != 0) /* Test for existance		      */
   		    fd->fd = open(fname,O_WRONLY,mode);
		else
		    fd->fd = -1;
		break;

          case DAPPEND:			/* open for append access             */
		if (access(fname,6) == 0) /* Test for update access	      */
   		    fd->fd = open(fname,O_APPEND,mode);
		else
		    fd->fd = -1;
		break;

          case DUPDATE:			/* open for read and write access     */
		if (access(fname,6) == 0) /* Test for update access	      */
   		    fd->fd = open(fname,O_RDWR,mode);
		else
		    fd->fd = -1;
		break;
          }

      if (fd->fd == -1)
         return(E_FAIL);

      status = dskip(fd,&fd->cur_rec);	/* skip to current record             */
      }

   return(status);
   }
#endif
