/******************************************************************************
FUNCTION:	DREAD

PURPOSE:	Part of the DITTO program which reads one physical record
		from a disk file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1      04/90    B. Davis		if carriage(which means vms carriage
					return carriage control file, normally)
					we were checking for EOF after fread,
					whereas fread actually returns a 0 upon
					end-of-file.
  5.2     05/90    B. Ailts		Added c_sigblk and c_sigunblk in order
 					that the interupts produced by the 
					processing message does corrupt the I/O.
  5.3     11/91    K. Gacke		Undo the KLUDEGE. Use fgets instead of 
					fread. fread reads the nbytes specified
					while fgets reads to the end-of-line.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE.  It is also assumed that this routine is used
	only by the DITTO program.

PROJECT			LAS

ALGORITHM 
	The input parameter is the disk file descriptor.  From the disk file
 	descriptor the C function to read the disk file is called.

	If running under the VMS operating system, all files are opened using
	the file descriptor.  The file attribute list is built based on the
	input parameter "disk".

	If running under the UNIX operating system, files with carriage return
	carriage control are opened by file pointer (buffered I/O), otherwise
	it is opened by file descriptor.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include <string.h>
#include "las.h"			/* las include file 		     */
#include "util.h"                       /* c_sigblk prototype */
#include "ditto.h"			/* DITTO include file                */

int dread
(
    struct DISK_FD *fd,	/* DITTO disk file descriptor        */
    unsigned char *buf,	/* buffer to read data into          */
    long nbytes		/* number of bytes to be read        */
)
{
int sig_mask;
long  bytes_read;		/* actual number of bytes read       */
char *ptr;			/* pointer to a character string     */

c_sigblk(&sig_mask);
if (fd->cc == CARRIAGE)
      {
      ptr = fgets((char *)buf,nbytes+1,fd->fp); 
      if (ptr == NULL)
	 bytes_read = 0;
      else
         bytes_read = (long)strlen((char *)buf);
/* KLUDEGE to read new line char     */
/*      
      bytes_read = fread(buf,sizeof(*buf),nbytes+1,fd->fp); 
      if (bytes_read == 0)
         bytes_read = 0;
      else
         bytes_read = strlen(buf);
*/
      }
else
      {
      bytes_read = read(fd->fd,buf,nbytes);
      if (bytes_read == EOF)
         return(E_FAIL);
      }

c_sigunblk(&sig_mask);
return(bytes_read);
}
