/******************************************************************************
FUNCTION:	DSKIP

PURPOSE:	Part of the DITTO program which skips a specified number
		of physical records within a disk file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1 	   01/94    D. Etrheim		Change malloc to allocate one more byte
					and free the buffer when done
					

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE.  It is also assumed that this routine is used
	only by the DITTO program.

PROJECT			LAS

ALGORITHM 
	From the disk file descriptor and count, the function "dread" is called
	to skip through the records within the file.

	If running under the VMS operating system, all files are opened using
	the file descriptor.  The file attribute list is built based on the
	input parameter "disk".

	If running under the UNIX operating system, files with carriage return
	carriage control are opened by file pointer (buffered I/O), otherwise
	it is opened by file descriptor.


ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* las include file 		     */
#include "ditto.h"			/* DITTO include file                */

int dskip
(
    struct DISK_FD *fd,		/* DITTO disk file descriptor        */
    long *count		        /* number of records to skip         */
)
{
long  rec;		/* Number of records to be skipped	*/
long  status;		/* Status variable			*/
unsigned char *buf;	/* buffer for read/skip			*/
long	  nbyte;	/* Number of bytes to read		*/
long      one = 1;

/*
  Set the number of bytes to read
*/
nbyte = fd->blksiz;

/*
  Allocate buffer space
*/
buf = (unsigned char *) malloc(fd->blksiz + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory","ditto-alloc",&one);
   return(E_FAIL);
   }

/*
  Read the number of records to be skipped
*/
for (rec=0; rec < *count; rec++)
   {
   /*
     Read a record
   */
   status = dread(fd,buf,nbyte);
   if ((status == E_FAIL) || (status == 0))
      {
      *count = rec;
      return(E_FAIL);
      }
   }

free(buf);
return(E_SUCC);
}
