/******************************************************************************
FUNCTION:	DWRITE

PURPOSE:	Part of the DITTO program which writes one physical record
		to a disk file.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      12/85    K. Gacke		initial development
  5.0      10/88    D. Hollaren		LAS 5.0 conversion
  5.1     05/90    B. Ailts		Added c_sigblk and c_sigunblk in order
 					that the interupts produced by the 
					processing message does corrupt the I/O.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE.  It is also assumed that this routine is used
	only by the DITTO program.

PROJECT			LAS

ALGORITHM 
	The input parameter is the disk file descriptor.  From the disk file
 	descriptor the C function to write to the disk file is called.

	If running under the VMS operating system, all files are opened using
	the file descriptor.  The file attribute list is built based on the
	input parameter "disk".

	If running under the UNIX operating system, files with carriage return
	carriage control are opened by file pointer (buffered I/O), otherwise
	it is opened by file descriptor.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>			/* C standard I/O include file       */
#include "las.h"			/* LAS include file 		     */
#include "util.h"           /* c_sigblk prototype */
#include "ditto.h"			/* DITTO include file                */

int dwrite
(
    struct DISK_FD *fd,	/* DITTO disk file descriptor        */
    unsigned char *buf,	/* buffer to write from              */
    long nbytes		/* number of bytes to be written     */
)
{
int sig_mask;
long bytes_written;		/* actual number of bytes written    */

c_sigblk(&sig_mask);
if (fd->cc == CARRIAGE)
      {
/*      bytes_written = fputs(buf,fd->fp); */
      bytes_written = (long)fwrite(buf,sizeof(*buf),nbytes,fd->fp);
      if (bytes_written == EOF)
         return(E_FAIL);

      bytes_written = (long)strlen((char *)buf);
      }
else
      {
      bytes_written = write(fd->fd,buf,nbytes);
      if (bytes_written != nbytes)
         return(E_FAIL);
      }

c_sigunblk(&sig_mask);
return(bytes_written);
}
