#! /usr/local/bin/perl
#
# This script parses the directory listing file, locating those that were
# of 0 size originally and are now non-zero (indicates they were added since
# the last time we retrieved the source from remotesensing.org) and
# schedule them for adding to the LAS repository.
###########################################################################

if ($#ARGV != 0)
{
    die "Incorrect number of params supplied to add_new_files.pl\n";
}

# Parse the directory listing file, scheduling new files for adding
# to the LAS repository
open (LS_FILE, "<$ARGV[0]") || die "Could not open directory listing file\n";
my $cur_dir = ".";
# Hash to track the size of each file in the current directory
my %cur_dir_files = {};
while (<LS_FILE>)
{
    chomp;
    my $cur_line = $_;
    if (/^$/)
    {
        # Blank line. Skip
        next;
    }
    elsif (/^total/)
    {
        # This line tells the total count of blocks(??) in the directory.
        # We are not interested in this information
        next;
    }
    elsif (/^\.\/([\w\/\.]+)\:/)
    {
        # This line is the name of the directory the next set of files
        # belong to. Add all the new files in the previous directory to
        # the repository before continuing
        foreach my $file (keys %cur_dir_files)
        {
            my @file_spec = split (/\./, $file);
            if (($cur_dir_files{$file} eq "0") &&
                ($file_spec[$#file_spec] eq "orig"))
            {
                pop @file_spec; # remove orig extension
                my $new_file = join (".", @file_spec);
                my $sys_cmd = "cvs add $cur_dir/$new_file";
                system ($sys_cmd);
            }
        }

        # Save the new directory name and re-initialize the file hash
        $cur_dir = $1;
        %cur_dir_files={};
    }
    else
    {
        # This line lists a particular file's information. Split it up into
        # its separate components
        $_ = $cur_line;
        my ($permissions, $num_refs, $owner, $group, $size, $mon, $day, $time, $file) = split /\s+/;
        # Save the file name and size in hash
        if ($file ne "$ARGV[0]")
        {
            $cur_dir_files{$file} = $size;
        }
    }
}

close LS_FILE;
exit 0;
