#! /sbin/csh

set cur_sync_date = `date '+%m/%d/%Y %H:%M:%S'`;

# Retrieve previous syncronization date/time string
set prev_sync_date = `retrieve_date.pl`
if ("$prev_sync_date" == "") then
   echo "Error retrieving previous syncronization date/time"
   exit 1
endif

# Export GDAL from their repository using the last sync
# date/time as retrieved from the log file
setenv CVSROOT :pserver:anonymous@www.remotesensing.org:/cvsroot
cvs export -D "$prev_sync_date" -d remot_sync_gdal gdal
if ($status) then
   echo "Error checking out previous version of GDAL from remotesensing.org"
   exit 1
endif

# Export the current version of GDAL from their repository
cvs export -D "$cur_sync_date" -d remot_cur_gdal gdal
if ($status) then
   echo "Error checking out current version of GDAL from remotesensing.org"
   exit 1
endif
setenv CVSROOT /sgs20/csb/software/src

# Create a patch file that contains the differences between
# the previous contents of their repository and the current
cd remot_sync_gdal
diff -urN . ../remot_cur_gdal > ../gdal_sync.patch
if ($status == 2) then
   echo "Error creating diff patch file"
   exit 1
endif
cd ..

# Check out GDAL from our repository
#cvs co -d las_cur_gdal gdal - THIS IS WHAT TO USE ONCE GDAL CHECKED INTO TRUNK
cvs co -r gdal_dev_branch -d las_cur_gdal gdalsrc
if ($status) then
   echo "Error checking out current version of GDAL from the LAS repository"
   exit 1
endif

# Remove the exported previous GDAL module
rm -r remot_sync_gdal
# Add a line to log file for the date/time we syncronized
echo $cur_sync_date >> sync_gdal_log.txt

# Apply the patch file to the most recent version of our code
cd las_cur_gdal

# Create any new directories first
../create_new_dirs.pl

# Apply the patch of differences to the LAS GDAL source code
patch < ../gdal_sync.patch
if ($status) then
   echo "Error applying patch file to the current LAS GDAL code"
   echo "Note: synchronization date/time entry already added to sync_gdal_log.txt"
   exit 1
endif

# Remove the patch and exported current GDAL module
rm ../gdal_sync.patch
rm -r ../remot_cur_gdal/

# Find all new files and execute a 'cvs add' for them
# New files have a .orig file with size 0
ls -laR > file.list
../add_new_files.pl file.list
if ($status) then
   rm file.list
   echo "Error while adding new files to the LAS repository"
   echo "Note: synchronization date/time entry already added to sync_gdal_log.txt"
   exit 1
endif
rm file.list

# Remove all the .orig files
find . -name "*orig" > rm.list
foreach file (`cat rm.list`)
   rm $file
end
rm rm.list

# Find all the files that have been removed and execute a 'cvs rm' for them
# Removed files have a file size of 0
ls -laR > file.list
../rm_zero_lngth_files.pl file.list
if ($status) then
   rm file.list
   echo "Error while removing files that have size of 0 (removed in new GDAL)"
   echo "Note: synchronization date/time entry already added to sync_gdal_log.txt"
   exit 1
endif
rm file.list

# Prune the empty directories by simply executing an update with the option
cvs up -P

# Let user know what to do to complete the process
echo "Syncronization complete."
echo "Execute a 'cvs up' in las_cur_gdal to see the differences. Verify the"
echo "differences are as expected then manually execute a 'cvs commit' to"
echo "commit the changes to the LAS repository."

exit 0
