/*******************************************************************************
NAME                          c_recip2minor

PURPOSE      Calculate semi minor axis based on recipricol flattening 

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  --------------------------------------
1.0      08/99  D. Hames          C    Original developement

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Must be run under UNIX and TAE

PROJECT: LAS

*******************************************************************************/
#include <stdlib.h>
#include <math.h>
#include "tae.h"
#include "worgen.h"
#include "geompak.h"

int c_recip2minor
(
    double *recipflat,		/* I: recipricol flattening             */
    double *semi_major,		/* I: semi major axis                   */
    double *semi_minor		/* O: semi minor axis                   */
)
{

double flat = 0.0;              /* Flattening of the spheroid 		*/
long nonfatal = 1;		/* Fatal error flag	      		*/
	

/* If the recipricol flattening is zero it is a sphere, otherwise compute minor
 -----------------------------------------------------------------------------*/
if (*recipflat != 0.0)
    {
    flat = fabs(1.0/(*recipflat));
    *semi_minor = ((*semi_major) * (1.0 - flat));
    }
else if (*semi_major == 6370997.0)
    *semi_minor = 0.0;
else
    {
    c_errmsg("Error calculating semi minor","recip2minor",&nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
