/*****************************************************************************
NAME:           c_unit2string

PURPOSE:        This routine will take an input projection unit code and
                convert it to its string value. It was written with the 
                intention of having a standard string value to be used
                for updating the proj_unit field in the LAS DDR.

                             Input        Output
                             unit         string
                             code:        value:
                           ----------     ------
                           0 (RADIAN)     radians
                           1 (FEET)       feet
                           2 (METER)      meters
                           3 (SECOND)     seconds
                           4 (DEGREE)     degrees
                           5 (DMS)        dms

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
$Log: c_unit2string.c,v $
Revision 6.2  2001/08/08 13:03:10  hames
Since DMS is an acronym, change the output string value for DMS from lower case
to upper case.

Revision 6.1  2001/07/03 14:22:30  hames
Rename c_ckunit.c to c_string2unit.c and make minor changes to c_string2unit.c
Keep c_ckunit as a wrapper function for c_string2unit since c_ckunit is
still used in some applications.  Add routine c_unit2string which converts
a projection unit code to a string value. CR 3205.


PROJECT:	LAS

******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "cproj.h"

int c_unit2string
(
    int  inunit,         /* I: Projection unit code                     */
    char *outunit        /* O: String value of projection units. The
                               buffer must be at least 7 characters     */
)
{

    /* Assign the output string projection code based on the input unit code 
    -----------------------------------------------------------------------*/
    if (inunit == RADIAN)
        strcpy(outunit, "radians");
    else if (inunit == FEET)
        strcpy(outunit, "feet");
    else if (inunit == METER)
        strcpy(outunit, "meters");
    else if (inunit == SECOND)
        strcpy(outunit, "seconds");
    else if (inunit == DEGREE)
        strcpy(outunit, "degrees");
    else if (inunit == DMS)
        strcpy(outunit, "DMS");
    else
    {  
        c_lasmsg(FALSE, "unit2string-unit", "Input projection unit code %d "
                 "is undefined", inunit);
        return (E_FAIL);
    }

    return(E_SUCC);
}
