/*******************************************************************************
NAME:        TESTPROJ

PURPOSE:     Makes a grid of GCTPc converted map projection points.  The User
             enters the Min and Max Lat and Long and the increment value.  A
             grid of points will be converted from Long/Lat to X/Y and back to
             Long/Lat.  The results will be entered into a file specified by
             the user.  The user optionally may compare the transformed data
             to the data in an existing file of the same format.

PROGRAM HISTORY:
$log$

ALGORITHM REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Proffessional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "geompak.h" /* c_transinit and c_trans routines */
#include "worgen.h" /* CMLEN definition */
#include "proj.h" /* FEET, METER, and GEO definitions */

int main(void)
{
    FILE *fptr1 = NULL;        /* file pointer for new data file             */
    FILE *fptr2 = NULL;        /* file pointer for existing file             */
    FILE *fptr3 = NULL;        /* file pointer for output comp file          */

    char buf2[82];             /* buffer for existing comp file information  */
    char fflag[2];             /* flag for comparing converted values to
                                  values in an existing comparison file      */
    char file1[CMLEN];         /* name of new data file                      */
    char file2[CMLEN];         /* name of existing (comparison) file         */
    char file3[CMLEN];         /* name of output comparison file             */
    char temp2[82];            /* temp buffer for comp file header strings   */

    double cmp_incoor1[2];     /* original input lon/lat values from
                                  comparison data file                       */
    double cmp_incoor2[2];     /* transformed lon/lat values from
                                  comparison data file                       */
    double cmp_outcoor1[2];    /* transformed x/y values from comp data file */
    double false_easting;      /* false-easting value                        */
    double incoor1[2];         /* current original lon/lat proj coordinates  */
    double incoor2[2];         /* transformed lon/lat proj coordinates       */
    double inparm[15];         /* input proj parms for forward conversion &
                                  output proj parms for inverse conversion   */
    double outcoor1[2];        /* transformed x/y projection coordinates     */
    double outcoor2[2];        /* transformed x/y projection coordinates     */
    double outparm[15];        /* output proj parms for forward conversion &
                                  input proj parms for inverse conversion    */
    double tempcoor;           /* used for swapping the X/Y coor.  values if
                                  projection is SOM                          */

    double i;                   /* looping counter                            */
    double j;                   /* looping counter                            */
    double lat_inc;             /* latitude increment in degrees              */
    double lon_inc;             /* longitude increment in degrees             */
    double max_lat;             /* maximum latitude in degrees                */
    double max_lon;             /* maximum longitude in degrees               */
    double min_lat;             /* minimum latitude in degrees                */
    double min_lon;             /* minimum longitude in degrees               */
    double tol;                 /* tolerance value for acceptable comparisons */

    int count;                 /* number of points transformed               */


    long count2;               /* number of lines of existing comp file read
                                  this is used when skipping the header info */
    long indatum;              /* input datum for forward conversion         */
    long inproj = GEO;         /* input proj code for forward conversion &
                                  output proj code for inverse conversion    */
    long inunit = DEGREE;      /* input proj units for forward conversion &
                                  output proj units for inverse conversion   */
    long inzone = 62;          /* input zone code for forward conversion &
                                  output zone code for inverse conversion    */
    long k;                    /* looping counter                            */
    long middatum;             /* output datum for forward conversion &
                                  input datum for inverse conversion         */
    long outdatum;             /* output datum for inverse conversion        */
    long outunit = METER;      /* output proj units for forward conversion &
                                  input proj units for inverse conversion    */
    long outzone;              /* output zone code for forward conversion &
                                  input zone code for inverse conversion     */
    long outproj;              /* output proj code for forward conversion &
                                  input proj code for inverse conversion     */
    long prtprm[2];            /* Report destination for c_transinit (F/F)   */

    /* Get needed information
    -----------------------*/
    printf(" 0    GEOGRAPHIC            ");
    printf(" 1    UTM \n");
    printf(" 2    STATE PLANE           ");
    printf(" 3    ALBERS CONICAL EQ AR \n");
    printf(" 4    LAMBERT CONFORMAL     ");
    printf(" 5    MERCATOR \n");
    printf(" 6    POLAR STEREOGRAPHIC   ");
    printf(" 7    POLYCONIC \n");
    printf(" 8    EQUIDISTANT CONIC     ");
    printf(" 9    TRANSVERSE MERCATOR \n");
    printf("10    STEREOGRAPHIC         ");
    printf("11    LAMBERT AZIMUTHAL \n");
    printf("12    AZIMUTHAL EQUDISTANT  ");
    printf("13    GNOMONIC \n");
    printf("14    ORHTOGRAPHIC          ");
    printf("15    GVNSP \n");
    printf("16    SINUSIODAL            ");
    printf("17    EQUIRECTANGULAR \n");
    printf("18    MILLER                ");
    printf("19    VAN DER GRINTEN \n");
    printf("20    OBLIQUE MERCATOR      ");
    printf("21    ROBINSON \n");
    printf("22    SOM                   ");
    printf("23    ALASKA CONFORMAL \n");
    printf("24    GOODE                 ");
    printf("25    MOLLWEIDE \n");
    printf("26    INTERRUPTED MOLLEIDE  ");
    printf("27    HAMMER \n");
    printf("28    WAGNER IV             ");
    printf("29    WAGNER VII \n");
    printf("30    OBLATED EQ AR         ");
    printf("31    INTEGERIZED SINUSOIDAL\n");

    printf(" Please enter the projection number \n");
    scanf("%ld",&outproj);

    if ((outproj == UTM) || (outproj == SPCS))
    {
        printf("\n Please enter the zone number \n");
        scanf("%ld",&outzone);

        printf(" 0    CLARKE 1866             ");
        printf(" 1    CLARKE 1880 \n");
        printf(" 2    BESSEL                  ");
        printf(" 3    INTERNATIONAL 1967 \n");
        printf(" 4    INTERNATIONAL 1909      ");
        printf(" 5    WGS 72 \n");
        printf(" 6    EVEREST                 ");
        printf(" 7    WGS 66 \n");
        printf(" 8    GRS 1980 (NAD 1983)     ");
        printf(" 9    AIRY \n");
        printf("10    MODIFIED EVEREST        ");
        printf("11    MODIFIED AIRY \n");
        printf("12    WGS 84                  ");
        printf("13    SOUTHEAST ASIA \n");
        printf("14    AUSTRALIAN NATIONAL     ");
        printf("15    KRASOVSKY \n");
        printf("16    HOUGH                   ");
        printf("17    MERCURY 1960 \n");
        printf("18    MODIFIED MERCURY 1968   ");
        printf("19    6370997 SPHERE \n");
        printf("20    BESSEL 1841             ");
        printf("21    EVEREST SABAH & SARAWAK \n");
        printf("22    EVEREST INDIA 1956      ");
        printf("23    EVEREST MALAYSIA 1969 \n");
        printf("24    EVEREST MALAY/SING 1948 ");
        printf("25    EVEREST PAKISTAN \n");
        printf("26    HAYFORD                 ");
        printf("27    HELMERT 1906 \n");
        printf("28    INDONESIAN 1974         ");
        printf("29    SOUTH AMERICAN 1969 \n");
        printf("30    WGS 60                  ");
        printf("31    MODIS SPHERE\n");
    }
    else
    {
        outzone = 62;
    }

    printf("\n Please enter the input projection's spheroid/datum number \n");
    scanf("%ld",&indatum);
     
    printf("\n Please enter the intermediate spheroid/datum number \n");
    scanf("%ld",&middatum);
     
    printf("\n Please enter the output projection's spheroid/datum number \n");
    scanf("%ld",&outdatum);

    for (k = 0; k < 15; k++)
    {
        printf("\n Please enter projection parameter %ld \n",k);
        scanf("%lf",&outparm[k]);
        inparm[k] = 0.0;
    }

    printf("\nPlease enter the minimum longitude in degrees (negative "
           "for west of zero)\n");
    scanf("%lf",&min_lon);
    printf("\nPlease enter the maximum longitude in degrees (negative "
           "for west of zero)\n");
    scanf("%lf",&max_lon);
    printf("\n Please enter the minimum latitude in degrees\n");
    scanf("%lf",&min_lat);
    printf("\n Please enter the maximum latitude in degrees\n");
    scanf("%lf",&max_lat);

    printf("\n Please enter the longitude increment in degrees\n");
    scanf("%lf",&lon_inc);
    printf("\n Please enter the latitude increment in degrees\n");
    scanf("%lf",&lat_inc);

    /* Get name of new data file
       -------------------------*/
    printf("\n Please enter name of the new data file\n");
    scanf("%s",file1);

    /* If files are to be saved with converted values get file names
      --------------------------------------------------------------*/
    printf("\n Are the values to be compared? (Y or N)\n");
    scanf("%s",fflag);
    *fflag = toupper(*fflag);

    if (!strcmp(fflag,"Y"))
    {
        printf("\n Please enter name of the existing data file\n");
        scanf("%s",file2);
    }

    if (!strcmp(fflag,"Y"))
    {
        printf("\n Please enter the tolerance value\n");
        scanf("%lf",&tol);

        printf("\n Please enter name of the Comparison file\n");
        scanf("%s",file3);
        fptr3 = fopen(file3,"w");
        if (fptr3 == NULL)
        {
            printf("\nError opening output comparison file %s\n",file3);
            return 1;
        }

        /* open existing data file
          ------------------------*/
        fptr2 = fopen(file2,"r");
        if (fptr2 == NULL)
        {
            printf("\nError opening comparison file--discontinue comparison\n");
            fprintf(fptr3,"\nError opening comp file--discontinue "
                    "comparison\n");
            strcpy(fflag,"N");
            fclose(fptr3);
        }
        count2 = 0;

        /* Get past the heading, will stop after the line beginning with "-"
          ------------------------------------------------------------------*/
        if (!strcmp(fflag,"Y"))
        {
            for(;;)
            {
                count2++;
                if (fgets(buf2,81,fptr2) == NULL)
                {
                    printf("\nError reading comparison file--discontinue "
                           "comparison\n");
                    fprintf(fptr3,"\nError reading comp file--discontinue "
                            "comparison\n");
                    strcpy(fflag,"N");
                    fclose(fptr2);
                    fclose(fptr3);
                    break;
                }

                /* Break at "-",  test to avoid invalid data file (count2)
                ----------------------------------------------------------*/
                sscanf(buf2,"%s",temp2);
                if (temp2[1] == '-')
                    break;
                else if (count2 > 50)
                {
                    printf("\nError reading comparison file--discontinue "
                           "comparison\n");
                    fprintf(fptr3,"\nError reading comp file--discontinue "
                            "comparison\n");
                    strcpy(fflag,"N");
                    fclose(fptr2);
                    fclose(fptr3);
                    break;
                }
            }
        }
    }

    /* Set the false easting from the projection parameter at index 6, except 
       for projections geo, utm, spcs, good, imoll for which it is undefined
    -------------------------------------------------------------------------*/
    if ((outproj == GEO) || (outproj == UTM) || (outproj == SPCS) ||
        (outproj == GOOD) || (outproj == IMOLL))
    {
        false_easting = 0.0;
    }
    else
        false_easting = outparm[6];

    /* Empty contents of new data file if it exists
    -----------------------------------------------*/
    if((fptr1 = fopen(file1,"w")) != NULL)
        fclose(fptr1);

    /* Test projection 
    ------------------*/
    /* set variables not entered with prompt 
    ----------------------------------------*/
    count = 0;
    prtprm[0] = prtprm[1] = 0;

    /*  Begin conversion of grid points
    -----------------------------------*/
    for(j = min_lon; j <= max_lon; j += fabs(lon_inc))
    {
        for(i = min_lat; i <= max_lat; i += fabs(lat_inc))
        {
            incoor1[0] =  j;
            incoor1[1] =  i;

            /* Initialize the Projection Transformation Package
               for forward transformation
            ---------------------------------------------------*/
            if(c_transinit(&inproj,&inunit,&inzone,&indatum,inparm,&outproj,
                           &outunit,&outzone,&middatum,outparm,prtprm,"")
               != E_SUCC)
            {
                printf("\nError returned from transinit\n");
                return 1;
            }

            /* Do forward conversion - lat/long to meters
            ---------------------------------------------*/
            if(c_trans(&inproj,&inunit,&outproj,&outunit,&incoor1[0],
                       &incoor1[1],&outcoor1[0],&outcoor1[1]) != E_SUCC)
            {
                printf("\nError returned from trans\n");
                return 1;
            }

            /* Swap and switch signs on transformed coordinates if SOM
            ----------------------------------------------------------*/
            if(outproj == SOM)
            {
               tempcoor = outcoor1[0];
               outcoor1[0] = -1.0 * outcoor1[1];
               outcoor1[1] = tempcoor;
            }

            /* Print Headings
            -----------------*/
            if ((i == min_lat) && (j == min_lon))
            {
                if((fptr1 = fopen(file1,"a")) == NULL)
                {
                    printf("\nError opening output data file");
                    return 1;
                }
                fprintf(fptr1,"\n INPUT LONG LAT        X              Y   "
                        "        OUTPUT LON      OUTPUT LAT");
                fprintf(fptr1,"\n--------------- --------------- "
                        "--------------- --------------- ---------------\n");
                fclose(fptr1);
            }

            /* Need to swap and switch signs again if SOM
            ---------------------------------------------*/
            if(outproj == SOM)
            {
               outcoor2[0] = outcoor1[1];
               outcoor2[1] = -1.0 * outcoor1[0];
            }
            else
            {
               outcoor2[0] = outcoor1[0];
               outcoor2[1] = outcoor1[1];
            }

            /* Initialize the Projection Transformation Package
               for inverse transformation
            ---------------------------------------------------*/
            if(c_transinit(&outproj,&outunit,&outzone,&middatum,outparm,
                           &inproj,&inunit,&inzone,&indatum,inparm,prtprm,"")
               != E_SUCC)
            {
                printf("\nError returned from transinit\n");
                return 1;
            }

            /* Do inverse conversion - meters to lat/long
            ---------------------------------------------*/
            if(c_trans(&outproj,&outunit,&inproj,&inunit,&outcoor2[0],
                       &outcoor2[1],&incoor2[0],&incoor2[1]) != E_SUCC)
            {
                printf("\nError returned from trans\n");
                return 1;
            }

            /*  Print points to file
            ------------------------*/
            if((fptr1 = fopen(file1,"a")) == NULL)
            {
                printf("\nError opening output data file");
                return 1;
            }
            fprintf(fptr1,"%7.2f %6.2f %15.5f %15.5f %15.5f %15.5f\n",
                    incoor1[0],incoor1[1],outcoor1[0],outcoor1[1],incoor2[0],
                    incoor2[1]);
            fclose(fptr1);

            count++;

            /* Compare values to data file
            ------------------------------*/
            if (!strcmp(fflag,"Y"))
            {
                if (fscanf(fptr2,"%lf %lf %lf %lf %lf %lf",&cmp_incoor1[0],
                           &cmp_incoor1[1],&cmp_outcoor1[0],&cmp_outcoor1[1],&cmp_incoor2[0],
                           &cmp_incoor2[1]) != 6)
                {
                    printf("\nError reading comparison file--discontinue "
                           "comparison\n");
                    fprintf(fptr3,"\nError reading comp file--discontinue "
                            "comparison\n");
                    strcpy(fflag,"N");
                    fclose(fptr3);
                    fclose(fptr2);
                }

                /* Check if original lat/long are the same--if not, error in 
                 * comp file
                -------------------------------------------------------------*/
                if (!strcmp(fflag,"Y"))
                {
                    if(fabs(incoor1[0]-cmp_incoor1[0])>tol || 
                       (fabs(incoor1[1]-cmp_incoor1[1])>tol))
                    {
                        printf("\nInconsistent input Long/Lat--discontinue "
                               "comparison\n");
                        fprintf(fptr3,"\nInconsistent input Long/Lat--"
                                "discontinue comparison\n");
                        strcpy(fflag,"N");
                        fclose(fptr3);
                        fclose(fptr2);
                    }
                }
                if (!strcmp(fflag,"Y"))
                {
                    /* if the difference (including tolerance is greater than 
                       360, convert the negative value to positive to make
                       sure -180 vs +180 isn't reported as a difference
                    ---------------------------------------------------------*/
                    if (fabs(incoor2[0] - cmp_incoor2[0]) + tol > 360.0)
                    {
                        if (incoor2[0] < 0.0)
                        incoor2[0] += 360.0;
                        if (cmp_incoor2[0] < 0.0)
                        cmp_incoor2[0] += 360.0;
                    }

                    /* if the input latitude is +/- 90 (within tolerance) and 
                       X adjusted for the false easting is zero, ignore the 
                       comparison on the longitudes since it degenerates to a 
                       single point
                    ---------------------------------------------------------*/
                    if ((fabs((fabs(cmp_incoor1[1]) - 90.0)) < tol) && 
                        (fabs(cmp_outcoor1[0] - false_easting) < tol))
                    {
                        incoor2[0] = cmp_incoor2[0];
                    }

                    if ((fabs(outcoor1[0] - cmp_outcoor1[0]) > tol) ||
                        (fabs(outcoor1[1] - cmp_outcoor1[1]) > tol))
                    {
                        fprintf(fptr3,"Differences exist at long %f lat %f\n",
                                incoor1[0],incoor1[1]);
                        printf("\nDifferences exist at long %f lat %f",
                                incoor1[0],incoor1[1]);
                        fprintf(fptr3,"X n %f X o %f Y n %f Y o %f\n",
                              outcoor1[0], cmp_outcoor1[0], outcoor1[1], cmp_outcoor1[1]);
                    }
                    else if ((fabs(incoor2[0] - cmp_incoor2[0]) > tol) ||
                             (fabs(incoor2[1] - cmp_incoor2[1]) > tol))
                    {
                        fprintf(fptr3,"Differences exist at long %f lat %f\n",
                                incoor1[0],incoor1[1]);
                        printf("\nDifferences exist at long %f lat %f",
                                incoor1[0],incoor1[1]);
                        fprintf(fptr3,"lon new %f lon old %f lat new %f lat "
                                "old %f\n",incoor2[0], cmp_incoor2[0], incoor2[1],
                                cmp_incoor2[1]);
                    }
                }
            } /* end if comparison checks */
        } /* end of for each lat (i) */
    } /* end of for each long (j) */
    if (!strcmp(fflag,"Y"))
    {
        fclose(fptr3);
        fclose(fptr2);
    }
    printf("\nNumber of points transformed %d \n",count);

    return 0;   
}
