/****************************************************************************
NAME:           c_gofopn

PURPOSE: Open the specified graphics overlay file.  If the file 
         does not exist, one will be created with a null header
         record.

PROGRAM HISTORY:
Version         Date    Contrib         Change
-------         ----    -------         ------
 1.0            3/86    L. Huewe        Intial code
 2.0            5/92    L. Huewe        Modified to be a more generic
                                        WORLD support routine.
 2.1            3/93    D. Etrheim      Changed pmode to 666 so users
                                        can use umask correctly

PROJECT: LAS

*****************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include "worgen.h"
#include "util.h"
#include "labserv.h"
#include "gof.h"

int c_gofopn 
(
    int  *fd,           /* File descriptor for the opened file. (O) */
    char *hostname,     /* Host file name of the image whose GOF is to
                           be opened. (I)  On output, the host name of
                           the GOF. (O)*/ 
    char *goflasnm,     /* LAS name of GOF opened. (O)*/
    char *type,         /* Type of GOF being opened. (I)*/
    int  *access,       /* Access mode (IREAD, IWRITE, IUPDATE). (I)*/
    int  *cflag         /* Create flag.  A value of FALSE means that the
                           routine did not have to create the GOF.  A 
                           value of TRUE means that the routine created a 
                           GOF. (O)*/
)
{
    static  int     pmode = 0666;    /* Protection for new file */

    struct  RWHDR   buf;             /* Attribute record buffer */

    char    hostnm[CMLEN],           /* Host name of the GOF */
            ext[16],                 /* File extension for dfgtaso */
            mess[60],                /* Array for error message */
            temp[LENL];              /* Temporary string containing
                                        character and data lengths */

    int     flags,                   /* Open flags */
            status,                  /* Status indicator */
            i;                       /* Loop index */
    long    one = 1;                 /* Indicates nonfatal error */
    long    length;                  /* String length */

    /* Set initial value of cflag. */
    *cflag = FALSE;

    /* Set up the GOF extension based on the type specified. */
    sprintf(ext,"GOF%c%s",GOF_SEP,type);
    length = (long)strlen(ext);
    c_up2low(ext,&length);

    /* Get the associated file name. */
    c_lsmknm(hostname,ext,hostnm);
    strcpy(goflasnm, hostnm);
    strcpy(hostname,hostnm);

    /* Set up the open flags. */
    if (*access == IREAD)                /* set flags: */
        flags = O_RDONLY;                /* read only */
    else                                 /* read/write */
        flags = O_RDWR | O_CREAT;        /* may create new file */

    /* Open the file. */
    *fd = open (hostnm, flags, pmode);
    if (*fd == -1)
    {
        sprintf(mess,"Unable to open file %s",hostnm);
        c_errmsg(mess,"gofopn-open",&one);
        return (E_FAIL);
    }

    /* Read the first record to determine whether the file 
       previously existed. */
    status = read (*fd,&buf,HDRLEN);
    if (status < 0)
    {
        c_errmsg ("Error reading GOF","gofopn-read",&one);
        return (E_FAIL);
    }

    /* If the file does not contain an attribute record, create a null one. */
    if (status == 0)
    {
        sprintf (temp,"%d/%d",HDRLEN-LENTYP-MAXATT,MAXATT);
        sprintf (buf.length,"%-*s",LENL,temp);
        sprintf (buf.type,"%-*s",TYPL,"B");
        strcpy (buf.key,HDRKEY);
        strcpy (buf.rnum,"0");
        for (i=0; i < MAXATT; i++)
        {
            buf.mask[i] = 0;
            strcpy (buf.attrb[i],"");
            strcpy (buf.attyp[i],"");
        }
        status = write (*fd,&buf,HDRLEN);
        if (status != HDRLEN)
        {
            c_errmsg ("Error writing to GOF","gofopn-write",&one);
            return (E_FAIL);
        }
        *cflag = TRUE;
    }

    /* Position file pointer at the beginning of the file. */
    lseek (*fd,0,0);

    return(E_SUCC);
}
