/************************************
*  Common includes and definitions  *
************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "fhdr.h"
#include "proj.h"
#include "header.h"

/*******************************************************************************
FUNCTION:	c_convertab	

PURPOSE: 	Converts an fast format A header to a fast format B
                header 

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
1.0	 T. Mittan	  11/92	 LAS   Original development
1.1      D. Etrheim       11/93  LAS   Make change for new GCTP projection 
				       number for SOM 

ALGORITHM REFERENCES	none
*******************************************************************************/

int c_convertab
(
   char  *header_a,                  /* fast format a header bfer     */
   char  *header_b                   /* fast format b header buffer     */
)
{
   char temp[CMLEN];
   char **bigptr;
   long i;
   long nonfatal = 1;
   long proj_number;
   
   
   /* fill in B header file with headings
      ------------------------------------*/
   template_header(header_b);
   
   
   /* Convert an A header to a B header
      ----------------------------------*/
   strncpy(&header_b[26],&header_a[26],9);
   strncpy(&header_b[54],&header_a[54],8);
   strncpy(&header_b[74],&header_a[74],2);
   strncpy(&header_b[89],&header_a[89],4);
   /*	strncpy(&header_b[108],&header_a[108],13);    ------- don't know */ 
   strncpy(temp,&header_a[108],13);
   temp[13] = '\0';
   if (strncmp(temp,"FULL SCENE   ",13) == 0)
     strncpy(&header_b[137],"FULL SCENE ",10);
   else
     if (strncmp(temp,"QUARTER SCENE",13) == 0)
       strncpy(&header_b[137],"SUBSCENE   ",10);
     else
       if (strncmp(temp,"MAP PRODUCT  ",13) == 0)
       {
	  strncpy(&header_b[137],"MAP SHEET  ",10);
	  strncpy(&header_b[147],&header_a[122],78);
       }
   /*	strncpy(&header_b[255],&header_a[252],10);    ------- don't know */
   strncpy(&header_b[278],&header_a[278],2);
   strncpy(&header_b[494],&header_a[494],6);
   strncpy(&header_b[513],&header_a[513],4);
   strncpy(&header_b[537],&header_a[537],6);
   strncpy(&header_b[559],&header_a[559],6);
   strncpy(&header_b[594],&header_a[594],360);	
   strncpy(&header_b[972],&header_a[972],20);
   strncpy(&header_b[1010],&header_a[1010],11);
   strncpy(&header_b[1039],&header_a[1039],11);
   strncpy(&header_b[1063],&header_a[1063],5);
   strncpy(&header_b[1086],&header_a[1086],4);
   strncpy(&header_b[1108],&header_a[1108],4);
   strncpy(&header_b[1116],&header_a[1116],228);
   strncpy(&header_b[1360],&header_a[1360],7);
   strncpy(&header_b[1385],&header_a[1385],4);
   strncpy(&header_b[1405],&header_a[1405],5);
   strncpy(&header_b[1426],&header_a[1426],2);
   strncpy(&header_b[1442],&header_a[1442],3);
   
   bigptr = (char **)malloc(15  * sizeof(char *));
   if (bigptr == NULL)
   {
      c_errmsg("Error allocating dynamic memory.","convertab-alloc",&nonfatal);
      return(E_FAIL);
   }
   
   sscanf(&header_b[538],"%d",&proj_number);
   for (i = 0; i < 15; i++)
     bigptr[i] = &header_b[594 + (i * 24)];
   

   /* Change the projection number for the new GCTP.
      SOM changed from 21 to 22.
   ------------------------------------------------*/
   if (proj_number == 21)
       proj_number = SOM;

   /* convert projection parms to proper DMS units 
      ---------------------------------------------*/
   convert_dms(bigptr, proj_number, 0);
   
   free(bigptr);
   return(E_SUCC);
}
