/***************************************************************************
NAME:				C_EDIPS_ANC

PURPOSE:  Reads EDIPS Ancillary record and puts information in ANSII file

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  1.0		12/92	S. Nelson	Original development

***************************************************************************/

#define C_EDIPS_ANN_C
#include <stdio.h>
#include "las.h"
#include "labserv.h"
#include "edips.h"
#include "pixman.h"
#include "header.h"
#include "local_protos.h"

long	hdr_dbdtype = EDOUBLE;	/* eight byte floating point data */
long	hdr_fldtype = EREAL;	/* four byte floating point data */
long	hdr_flag = SYS_ERR;		/* Return bad status code	*/
long	hdr_insys = SYS_IBM_MVS;	/* input system			*/
long	hdr_outsys = SYS_IEEE_STD;	/* output system		*/
float	hdr_fover = MAXREAL;	/* value if overflow		*/
float	hdr_funder = MINREAL;	/* value if underflow		*/
double	hdr_dover = MAXREAL;	/* value if overflow		*/
double	hdr_dunder = MINREAL;	/* value if underflow		*/

/* Routine to read MSS EDIPS ANCILLARY FILE
  ----------------------------------------*/
int c_edips_anc
(
    char *outh,		/* image host file name				*/
    long outsys,		/* output system				*/
    struct EDPSAC *anc,	/* EDIPS ancillary structure			*/
    struct EDPSAC_EPH *anceph, /* EDIPS ancillary structure--Ephemeris Data */
    struct EDPSAC_ATT *ancatt, /* EDIPS ancillary structure--Attitude Data */
    struct EDPSAC_RS *ancutm, /* EDIPS ancillary structure--Horizontal and
				Vertical resampling for UTM/PS data	*/
    struct EDPSAC_RS *ancsom /* EDIPS ancillary structure--Horizontal and
				Vertical resampling for SOM/HOM data	*/
)

{
FILE *fptr;	/* file pointer					*/

long status;		/* return status				*/
long i, j;		/* loop counters				*/
long  mode = 0;         /* flag value for lsstat                */
long one = 1;		/* one						*/
long nonfatal = 1;	/* nonfatal error indicator			*/

char finame[CMLEN];	/* Host file name				*/

unsigned char buf[7193];/* buffer for EDIPS tape information 		*/
unsigned char templon[4];/* buffer for EDIPS tape file			*/

/* Open ancillary file
   ------------------*/
c_lsmknm(outh,"edips_ancil",finame);
  status = c_lsstat(finame,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("ANC file does not exist on disk","edips_anc-stat",&nonfatal);
     return(E_FAIL);
     }

fptr=fopen(finame,"r");
if(!fptr)
   {
   c_errmsg("Error opening ancillary file","edips_anc-open",&nonfatal);
   return(E_FAIL);
   }
fseek(fptr,2,0);
fread(buf,1,7192,fptr);

/*  read binary record -- RECORD 1
   ------------------------------*/
anc->r1_record_length = ((long)buf[0] * 256+(long)buf[1]);
anc->r1_record_number = (long)buf[5];
anc->r1_record_type_code = (long)buf[7];
anc->input_pixels_per_line = get_long_var(&buf[8]);
anc->input_lines_per_image = get_long_var(&buf[12]);
c_pxsys(&insys,&outsys,&buf[16],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->input_inter_pixel_dis = *(double *)&buf[16];
c_pxsys(&insys,&outsys,&buf[24],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->input_inter_line_dis = *(double *)&buf[24];
anc->output_pixels_per_line = get_long_var(&buf[32]);
anc->output_lines_per_image = get_long_var(&buf[36]);
c_pxsys(&insys,&outsys,&buf[40],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->output_inter_pixl_dis = *(double *)&buf[40];
c_pxsys(&insys,&outsys,&buf[48],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->output_inter_line_dis = *(double *)&buf[48];
c_pxsys(&insys,&outsys,&buf[56],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->spacecraft_altitude = *(double *)&buf[56];
c_pxsys(&insys,&outsys,&buf[64],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->input_image_width = *(double *)&buf[64];
for (i = 0; i < 4; i++)
   {
   c_pxsys(&insys,&outsys,&buf[72+i],&dbdtype,&one,&dunder,
							&dover,&hdr_flag);
   anc->mss_mirror_coef[i] = *(double *)&buf[72+(8*i)];
   }
c_pxsys(&insys,&outsys,&buf[104],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->mss_max_mirror_ang = *(double *)&buf[104];
c_pxsys(&insys,&outsys,&buf[112],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->scan_skew_constant = *(double *)&buf[112];
c_pxsys(&insys,&outsys,&buf[120],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->tim_bet_mirror_sweeps = *(double *)&buf[120];
c_pxsys(&insys,&outsys,&buf[128],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->tim_active_mirror_swp = *(double *)&buf[128];
c_pxsys(&insys,&outsys,&buf[136],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->semi_major_axis = *(double *)&buf[136];
c_pxsys(&insys,&outsys,&buf[144],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->semi_minor_axis = *(double *)&buf[144];
c_pxsys(&insys,&outsys,&buf[152],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->earth_curvature_const = *(double *)&buf[152];
for (i = 0; i < 24; i++)
   {
   c_pxsys(&insys,&outsys,&buf[160+(i*4)],&fldtype,&one,&funder,
							&fover, &hdr_flag);
   anc->mss_samp_delay_const[i] = *(float *)&buf[160+(i*4)];
   }
for (i = 0; i < 3; i++)
   {
   c_pxsys(&insys,&outsys,&buf[264+(i*4)],&fldtype,&one,&funder,
							&fover, &hdr_flag);
   anc->mss_band_offsets[i] = *(float *)&buf[264+(i*4)];
   }
   

/* read binary record -- RECORD 2
   -----------------------------*/
	/* Spacecraft state vector at scene center
	   --------------------------------------*/
c_pxsys(&insys,&outsys,&buf[3744],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_state_vector_pitch = *(double *)&buf[3744];
c_pxsys(&insys,&outsys,&buf[3752],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_state_vector_roll = *(double *)&buf[3752];
c_pxsys(&insys,&outsys,&buf[3760],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_state_vector_yaw = *(double *)&buf[3760];
c_pxsys(&insys,&outsys,&buf[3768],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_state_vector_x = *(double *)&buf[3768];
c_pxsys(&insys,&outsys,&buf[3776],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_state_vector_y = *(double *)&buf[3776];
c_pxsys(&insys,&outsys,&buf[3784],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_state_vector_z = *(double *)&buf[3784];
c_pxsys(&insys,&outsys,&buf[3792],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_s_vctr_delta_pitch = *(double *)&buf[3792];
c_pxsys(&insys,&outsys,&buf[3800],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_s_vctr_delta_roll = *(double *)&buf[3800];
c_pxsys(&insys,&outsys,&buf[3808],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_s_vctr_delta_yaw = *(double *)&buf[3808];
c_pxsys(&insys,&outsys,&buf[3816],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_s_vctr_delta_x = *(double *)&buf[3816];
c_pxsys(&insys,&outsys,&buf[3824],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_s_vctr_delta_y = *(double *)&buf[3824];
c_pxsys(&insys,&outsys,&buf[3832],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_s_vctr_delta_z = *(double *)&buf[3832];
anc->cps_in_attitude = get_long_var(&buf[3856]);
anc->gcps_in_attitude = get_long_var(&buf[3860]);
anc->cp_correlatns_attempt = get_long_var(&buf[3864]);
anc->cps_reject_correlatns = get_long_var(&buf[3868]);
c_pxsys(&insys,&outsys,&buf[3872],&fldtype,&one,&funder,&fover,&hdr_flag);
anc->rms_along_track_error = *(float *)&buf[3872];
c_pxsys(&insys,&outsys,&buf[3876],&fldtype,&one,&funder,&fover,&hdr_flag);
anc->rms_across_track_error = *(float *)&buf[3876];
for (i = 0; i < 25; i++)
   {
   anc->cps_per_zone[i] = (long)buf[3886+i];
   strncpy(anc->cp_identificatn[i],(char*)&buf[3912+(i*8)],8);
   anc->cp_identificatn[i][8] = '\0';
   }

/* re-read file into buf to typecast doubles--the "fread" statement reads the
   same information as before but starts at a different byte as set by "fseek".
   This was necessary to avoid core dumps on the dg5 that occurred when 
   typecasting and calling "pxsys".
   --------------------------------------------------------------------------*/
fseek(fptr,6,0);
fread(buf,1,7192,fptr);

anc->r2_record_length = ((long)buf[3600] * 256+(long)buf[3601]);
anc->r2_record_number = (long)buf[3605];
anc->r2_record_type_code = (long)buf[3607];
strncpy(anc->wrs_frame_number,(char *)&buf[3608],4);
anc->wrs_frame_number[4] = '\0';
strncpy(anc->wrs_orbit_number,(char *)&buf[3612],4);
anc->wrs_orbit_number[4] = '\0';
c_pxsys(&insys,&outsys,&buf[3616],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->wrs_ctr_lat = *(double *)&buf[3616];
c_pxsys(&insys,&outsys,&buf[3624],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->wrs_ctr_long = *(double *)&buf[3624];
strncpy(anc->center_time,(char *)&buf[3632],14);
anc->center_time[14] = '\0';
c_pxsys(&insys,&outsys,&buf[3656],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->scene_ctr_lat = *(double *)&buf[3656];
c_pxsys(&insys,&outsys,&buf[3664],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->scene_ctr_long = *(double *)&buf[3664];
for (i = 0; i < 3; i++)
   {
   c_pxsys(&insys,&outsys,&buf[3672+(i*8)],&dbdtype,&one,&dunder,
								&dover,&hdr_flag);
   anc->sc_earth_cntrd_coords[i] = *(double *)&buf[3672+(i*8)];
   }
c_pxsys(&insys,&outsys,&buf[3696],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_heading_angle = *(double *)&buf[3696];
c_pxsys(&insys,&outsys,&buf[3704],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_scan_line_coord = *(double *)&buf[3704];
c_pxsys(&insys,&outsys,&buf[3712],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_pixel_coord = *(double *)&buf[3712];
c_pxsys(&insys,&outsys,&buf[3720],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_norm_velocity_err = *(double *)&buf[3720];
c_pxsys(&insys,&outsys,&buf[3728],&dbdtype,&one,&dunder,&dover,&hdr_flag);
anc->sc_earth_rotat_velcty = *(double *)&buf[3728];

c_pxsys(&insys,&outsys,&buf[3736],&fldtype,&one,&funder,&fover,&hdr_flag);
anc->earth_rotation_param = *(float *)&buf[3736];

	/* Geometric Correction Parameters Ephemeris Data
	   ---------------------------------------------*/
/* re-read file into buf to typecast doubles and call pxsys
   -------------------------------------------------------*/
fseek(fptr,4,0);
fread(buf,1,7192,fptr);
strncpy(anceph->time_of_first_set,(char *)&buf[4270],14);
anceph->time_of_first_set[14] = '\0';
c_pxsys(&insys,&outsys,&buf[4284],&fldtype,&one,&funder,&fover,&hdr_flag);
anceph->interval_between_sets = *(float *)&buf[4284];

		/* set first byte to even for get_long_buf
	   	  --------------------------------------*/
for (i = 0; i < 4; i++)
   templon[i] = buf[4288 + i];
anceph->number_of_sets = get_long_var(templon);

		/* set up to 16 sets of ephemeris entries
		   --------------------------------------*/
for (i = 0; i < 16; i++)
   {
   c_pxsys(&insys,&outsys,&buf[4292+(i*28)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   anceph->spccrft_locatn_x[i] = *(float *)&buf[4292+(i*28)];
   c_pxsys(&insys,&outsys,&buf[4296+(i*28)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   anceph->spccrft_locatn_y[i] = *(float *)&buf[4296+(i*28)];
   c_pxsys(&insys,&outsys,&buf[4300+(i*28)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   anceph->spccrft_locatn_z[i] = *(float *)&buf[4300+(i*28)];
   c_pxsys(&insys,&outsys,&buf[4304+(i*28)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   anceph->spccrft_velocity_x[i] = *(float *)&buf[4304+(i*28)];
   c_pxsys(&insys,&outsys,&buf[4308+(i*28)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   anceph->spccrft_velocity_y[i] = *(float *)&buf[4308+(i*28)];
   c_pxsys(&insys,&outsys,&buf[4312+(i*28)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   anceph->spccrft_velocity_z[i] = *(float *)&buf[4312+(i*28)];
  /*  
   for (j = 0; j < 4; j++)
      templon[j] = buf[(4316+(i*28)) + j];
   anceph->data_quality[i] = get_long_var(templon);
  */
   anceph->data_quality[i] = (long)buf[4319+(i*28)];
   }
	/* Attitude Data
	   ------------*/
/* re-read file into buf to typecast doubles and call pxsys
   -------------------------------------------------------*/
fseek(fptr,2,0);
fread(buf,1,7192,fptr);
strncpy(ancatt->time_of_first_set,(char *)&buf[4742],14);
ancatt->time_of_first_set[14] = '\0';
c_pxsys(&insys,&outsys,&buf[4756],&fldtype,&one,&funder,&fover,&hdr_flag);
ancatt->interval_between_sets = *(float *)&buf[4756];

		/* set first byte to even for get_long_buf
	   	  --------------------------------------*/

for (i = 0; i < 4; i++)
   templon[i] = buf[4760 + i];
ancatt->number_of_sets = get_long_var(templon);

		/* set up to 60 sets of attitude entries
		   ------------------------------------*/
for (i = 0; i < 60; i++)
   {
   c_pxsys(&insys,&outsys,&buf[4764+(i*16)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   ancatt->pitch_angle[i] = *(float *)&buf[4764+(i*16)];
   c_pxsys(&insys,&outsys,&buf[4768+(i*16)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   ancatt->roll_angle[i] = *(float *)&buf[4768+(i*16)];
   c_pxsys(&insys,&outsys,&buf[4772+(i*16)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
   ancatt->yaw_angle[i] = *(float *)&buf[4772+(i*16)];
   
   ancatt->data_quality[i] = get_long_var(&buf[4776]);
   }
		/* Partial Derivatives for SOM projection
		   --------------------------------------*/
for (i = 0; i < 3; i++)
   {
   for (j = 0; j < 5; j++)
      {
      c_pxsys(&insys,&outsys,&buf[5724+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_along_trk_x[i][j] = *(float *)&buf[5724+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[5784+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_along_trk_y[i][j] = *(float *)&buf[5784+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[5844+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_acros_trk_x[i][j] = *(float *)&buf[5844+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[5904+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_acros_trk_y[i][j] = *(float *)&buf[5904+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[5964+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_altitude_x[i][j] = *(float *)&buf[5964+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[6024+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_altitude_y[i][j] = *(float *)&buf[6024+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[6084+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_pitch_x[i][j] = *(float *)&buf[6084+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[6144+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_pitch_y[i][j] = *(float *)&buf[6144+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[6204+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_roll_x[i][j] = *(float *)&buf[6204+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[6264+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_roll_y[i][j] = *(float *)&buf[6264+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[6324+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_yaw_x[i][j] = *(float *)&buf[6324+(i*20)+(j*4)];
      c_pxsys(&insys,&outsys,&buf[6384+(i*20)+(j*4)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
      ancatt->pd_yaw_y[i][j] = *(float *)&buf[6384+(i*20)+(j*4)];
      }
   }
	/* Multiplicative and additive radiometric correction constants
  	   -----------------------------------------------------------*/
for (i = 0; i < 4; i++)
   if (!strncmp((char *)&buf[6612+i*48]," ",1))
      for (j = 0; j < 6; j++)
	 {
         ancatt->rc_mult_band_det[i][j] = 0.0;
         ancatt->rc_add_band_det[i][j] = 0.0;
         }
   else
      for (j = 0; j < 6; j++)
	 {
         c_pxsys(&insys,&outsys,&buf[6612+(i*48)+(j*8)],&fldtype,&one,
							&funder,&fover,&hdr_flag);
         ancatt->rc_mult_band_det[i][j] = *(float *)&buf[6612+(i*48)+(j*8)];
         c_pxsys(&insys,&outsys,&buf[6612+(i*48)+(j*8)+4],&fldtype,&one,
							&funder,&fover,&hdr_flag);
         ancatt->rc_add_band_det[i][j] = *(float *)&buf[6612+(i*48)+(j*8)+4];
         }

/* Find Map Projection Formats for HRS pixel Coordinates and
   VRS pixel Coordinates--data taken from 16 records
   ---------------------------------------------------------*/
   /* UTM/PS data
      ----------*/
fseek(fptr,7210,0);
fread(buf,1,7192,fptr);

/*  information taken from record 4 -- rows 1-12 of HRS coordinates for UTM/PS
    -------------------------------------------------------------------------*/
ancutm->rn4_rec_type_code = (long)buf[7];

for (i = 0; i < 12; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[8+(j*4)+(i*252)],&ancutm->hrs_row_coord[i][j]);
  status = c_readfpg(&buf[252+(i*252)],&ancutm->hrs_ln_fill_left[i]); 
  status = c_readfpg(&buf[256+(i*252)],&ancutm->hrs_ln_fill_rgt[i]); 
  }

/*  information taken from record 5 -- rows 13-24 of HRS coordinates for UTM/PS
    -------------------------------------------------------------------------*/
ancutm->rn5_rec_type_code = (long)buf[3611];
for (i = 0; i < 12; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[3612+(j*4)+(i*252)],&ancutm->hrs_row_coord[i+12][j]);
  status = c_readfpg(&buf[3856+(i*252)],&ancutm->hrs_ln_fill_left[i+12]); 
  status = c_readfpg(&buf[3860+(i*252)],&ancutm->hrs_ln_fill_rgt[i+12]); 
  }

/* read records 6 and 7
   -------------------*/
fseek(fptr,14418,0);
fread(buf,1,7192,fptr);

/*  information taken from record 6 -- rows 25-36 of HRS coordinates for UTM/PS
    -------------------------------------------------------------------------*/
ancutm->rn6_rec_type_code = (long)buf[7];
for (i = 0; i < 12; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[8+(j*4)+(i*252)],&ancutm->hrs_row_coord[i+24][j]);
  status = c_readfpg(&buf[252+(i*252)],&ancutm->hrs_ln_fill_left[i+24]); 
  status = c_readfpg(&buf[256+(i*252)],&ancutm->hrs_ln_fill_rgt[i+24]); 
  }

/*  information taken from record 7 -- rows 37-48 of HRS coordinates for UTM/PS
    -------------------------------------------------------------------------*/
ancutm->rn7_rec_type_code = (long)buf[3611];
for (i = 0; i < 12; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[3612+(j*4)+(i*252)],&ancutm->hrs_row_coord[i+36][j]);
  status = c_readfpg(&buf[3856+(i*252)],&ancutm->hrs_ln_fill_left[i+36]); 
  status = c_readfpg(&buf[3860+(i*252)],&ancutm->hrs_ln_fill_rgt[i+36]); 
  }

/* read records 8 and 9
   -------------------*/
fseek(fptr,21626,0);
fread(buf,1,7192,fptr);

/*  information taken from record 8 -- rows 49-51 of HRS coordinates for UTM/PS
    and rows 1-8 of VRS coordinates for UTM/PS map projections
    -------------------------------------------------------------------------*/
ancutm->rn8_rec_type_code = (long)buf[7];
for (i = 0; i < 3; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[8+(j*4)+(i*252)],&ancutm->hrs_row_coord[i+48][j]);
  status = c_readfpg(&buf[252+(i*252)],&ancutm->hrs_ln_fill_left[i+48]); 
  status = c_readfpg(&buf[256+(i*252)],&ancutm->hrs_ln_fill_rgt[i+48]); 
  }

    /*  find first 8 rows of VRS coordinates for UTM/PS map projections
        --------------------------------------------------------------*/
for (i = 0; i < 8; i++)
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[1016+(j*4)+(i*244)],&ancutm->vrs_row_coord[i][j]);

/*  information taken from record 9 -- rows 9-20 of VRS coordinates for UTM/PS
    -------------------------------------------------------------------------*/
ancutm->rn9_rec_type_code = (long)buf[3611];
for (i = 0; i < 12; i++)
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[3612+(j*4)+(i*244)],&ancutm->vrs_row_coord[i+8][j]);

/* read records 10 and 11
   ---------------------*/
fseek(fptr,28834,0);
fread(buf,1,7192,fptr);

/*  information taken from record 10 -- rows 21-32 of VRS coordinates for UTM/PS
    --------------------------------------------------------------------------*/
ancutm->rn10_rec_type_code = (long)buf[7];
for (i = 0; i < 12; i++)
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[8+(j*4)+(i*244)],&ancutm->vrs_row_coord[i+20][j]);

/*  information taken from record 11 -- rows 33-44 of VRS coordinates for UTM/PS
    --------------------------------------------------------------------------*/
ancutm->rn11_rec_type_code = (long)buf[3611];
for (i = 0; i < 12; i++)
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[3612+(j*4)+(i*244)],&ancutm->vrs_row_coord[i+32][j]);

ancutm->pixel_wrs_cntr = (long)buf[6684]*256+(long)buf[6685];

/* Offset of WRS cntr is two bytes, the most significant bit indicates the sign
   ---------------------------------------------------------------------------*/
if (buf[6686]&0x80)		/* negative--most significant bit is 1 */
   {
   buf[6686] &= 0x7f;
   ancutm->offset_wrs_cntr = -1*((long)buf[6686]*256+(long)buf[6687]);
   }
else				/* positive--most significant bit is 0 */
   ancutm->offset_wrs_cntr = (long)buf[6686]*256+(long)buf[6687];

strncpy(ancutm->temp_reg_scene_id,(char *)&buf[6688],20);
ancutm->temp_reg_scene_id[20] = '\0';
for (i = 0; i < 4; i++)
   {
   ancutm->tr_cur_imag_line[i]=((long)buf[6708+i*8]*256+(long)buf[6709+i*8]);
   ancutm->tr_cur_imag_pixel[i]=((long)buf[6710+i*8]*256+(long)buf[6711+i*8]);
   ancutm->tr_ref_imag_line[i]=((long)buf[6712+i*8]*256+(long)buf[6713+i*8]);
   ancutm->tr_ref_imag_pixel[i]=((long)buf[6714+i*8]*256+(long)buf[6715+i*8]);
   }
ancutm->first_overlap_line = ((long)buf[6740] * 256 + (long)buf[6741]);
ancutm->first_overlap_pixel = ((long)buf[6742] * 256 + (long)buf[6743]);
ancutm->second_overlap_line = ((long)buf[6744] * 256 + (long)buf[6745]);
ancutm->second_overlap_pixel = ((long)buf[6746] * 256 + (long)buf[6747]);
ancutm->third_overlap_line = ((long)buf[6748] * 256 + (long)buf[6749]);
ancutm->third_overlap_pixel = ((long)buf[6750] * 256 + (long)buf[6751]);
ancutm->fourth_overlap_line = ((long)buf[6752] * 256 + (long)buf[6753]);
ancutm->fourth_overlap_pixel = ((long)buf[6754] * 256 + (long)buf[6755]);

ancutm->tick_marks_top = (long)buf[6756];
ancutm->tick_marks_left = (long)buf[6757];
ancutm->tick_marks_right = (long)buf[6758];
ancutm->tick_marks_bottom = (long)buf[6759];

for (i = 0; i < 4; i++)
   ancutm->samp_corner_pts[i]=((long)buf[6760+(i*2)]*256+(long)buf[6761+(i*2)]);

c_pxsys(&insys,&outsys,&buf[6768],&dbdtype,&one,&dunder,&dover,&hdr_flag);
ancutm->image_orientatn_ang = *(double *)&buf[6768];
ancutm->num_sweeps = ((long)buf[6776]*256+(long)buf[6777]);

/* SOM/HOM data
   -----------*/
/* Open file for records 12 and 13
   ------------------------------*/
fseek(fptr,36042,0);
fread(buf,1,7192,fptr);

/*  information taken from record 12 -- rows 1-12 of HRS coordinates for SOM/HOM
    -------------------------------------------------------------------------*/
ancsom->rn4_rec_type_code = (long)buf[7];

for (i = 0; i < 12; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[8+(j*4)+(i*252)],&ancsom->hrs_row_coord[i][j]);
  status = c_readfpg(&buf[252+(i*252)],&ancsom->hrs_ln_fill_left[i]); 
  status = c_readfpg(&buf[256+(i*252)],&ancsom->hrs_ln_fill_rgt[i]); 
  }

/*  information taken from record 13--rows 13-24 of HRS coordinates for SOM/HOM
    --------------------------------------------------------------------------*/
ancsom->rn5_rec_type_code = (long)buf[3611];
for (i = 0; i < 12; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[3612+(j*4)+(i*252)],&ancsom->hrs_row_coord[i+12][j]);
  status = c_readfpg(&buf[3856+(i*252)],&ancsom->hrs_ln_fill_left[i+12]); 
  status = c_readfpg(&buf[3860+(i*252)],&ancsom->hrs_ln_fill_rgt[i+12]); 
  }

/* read records 13 and 14
   ---------------------*/
fseek(fptr,43250,0);
fread(buf,1,7192,fptr);

/*  information taken from record 14--rows 25-36 of HRS coordinates for SOM/HOM
    --------------------------------------------------------------------------*/
ancsom->rn6_rec_type_code = (long)buf[7];
for (i = 0; i < 12; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[8+(j*4)+(i*252)],&ancsom->hrs_row_coord[i+24][j]);
  status = c_readfpg(&buf[252+(i*252)],&ancsom->hrs_ln_fill_left[i+24]); 
  status = c_readfpg(&buf[256+(i*252)],&ancsom->hrs_ln_fill_rgt[i+24]); 
  }

/*  information taken from record 15--rows 37-48 of HRS coordinates for SOM/HOM
    -------------------------------------------------------------------------*/
ancsom->rn7_rec_type_code = (long)buf[3611];
for (i = 0; i < 12; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[3612+(j*4)+(i*252)],&ancsom->hrs_row_coord[i+36][j]);
  status = c_readfpg(&buf[3856+(i*252)],&ancsom->hrs_ln_fill_left[i+36]); 
  status = c_readfpg(&buf[3860+(i*252)],&ancsom->hrs_ln_fill_rgt[i+36]); 
  }

/* read records 16 and 17
   ---------------------*/
fseek(fptr,50458,0);
fread(buf,1,7192,fptr);

/*  information taken from record 16--rows 49-51 of HRS coordinates for SOM/HOM
    and rows 1-8 of VRS coordinates for SOM/HOM map projections
    -------------------------------------------------------------------------*/
ancsom->rn8_rec_type_code = (long)buf[7];
for (i = 0; i < 3; i++)
  {
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[8+(j*4)+(i*252)],&ancsom->hrs_row_coord[i+48][j]);
  status = c_readfpg(&buf[252+(i*252)],&ancsom->hrs_ln_fill_left[i+48]); 
  status = c_readfpg(&buf[256+(i*252)],&ancsom->hrs_ln_fill_rgt[i+48]); 
  }

    /*  find first 8 rows of VRS coordinates for SOM/HOM map projections
        ---------------------------------------------------------------*/
for (i = 0; i < 8; i++)
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[1016+(j*4)+(i*244)],&ancsom->vrs_row_coord[i][j]);

/*  information taken from record 17--rows 9-20 of VRS coordinates for SOM/HOM
    --------------------------------------------------------------------------*/
ancsom->rn9_rec_type_code = (long)buf[3611];
for (i = 0; i < 12; i++)
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[3612+(j*4)+(i*244)],&ancsom->vrs_row_coord[i+8][j]);

/* read records 18 and 19
   ---------------------*/
fseek(fptr,57666,0);
fread(buf,1,7192,fptr);
fclose(fptr);

/*  information taken from record 18--rows 21-32 of VRS coordinates for SOM/HOM
    --------------------------------------------------------------------------*/
ancsom->rn10_rec_type_code = (long)buf[7];
for (i = 0; i < 12; i++)
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[8+(j*4)+(i*244)],&ancsom->vrs_row_coord[i+20][j]);

/*  information taken from record 19--rows 33-44 of VRS coordinates for SOM/HOM
    --------------------------------------------------------------------------*/
ancsom->rn11_rec_type_code = (long)buf[3611];
for (i = 0; i < 12; i++)
  for (j = 0; j < 61; j++)
    status=c_readfpg(&buf[3612+(j*4)+(i*244)],&ancsom->vrs_row_coord[i+32][j]);

ancsom->pixel_wrs_cntr = (long)buf[6684]*256+(long)buf[6685];

/* Offset of WRS cntr is two bytes, the most significant bit indicates the sign
   ---------------------------------------------------------------------------*/
if (buf[6686]&0x80)		/* negative--most significant bit is 1 */
   {
   buf[6686] &= 0x7f;
   ancsom->offset_wrs_cntr = -1*((long)buf[6686]*256+(long)buf[6687]);
   }
else				/* positive--most significant bit is 0 */
   ancsom->offset_wrs_cntr = (long)buf[6686]*256+(long)buf[6687];

strncpy(ancsom->temp_reg_scene_id,(char *)&buf[6688],20);
ancsom->temp_reg_scene_id[20] = '\0';
for (i = 0; i < 4; i++)
   {
   ancsom->tr_cur_imag_line[i]=((long)buf[6708+i*8]*256+(long)buf[6709+i*8]);
   ancsom->tr_cur_imag_pixel[i]=((long)buf[6710+i*8]*256+(long)buf[6711+i*8]);
   ancsom->tr_ref_imag_line[i]=((long)buf[6712+i*8]*256+(long)buf[6713+i*8]);
   ancsom->tr_ref_imag_pixel[i]=((long)buf[6714+i*8]*256+(long)buf[6715+i*8]);
   }
ancsom->first_overlap_line = ((long)buf[6740] * 256 + (long)buf[6741]);
ancsom->first_overlap_pixel = ((long)buf[6742] * 256 + (long)buf[6743]);
ancsom->second_overlap_line = ((long)buf[6744] * 256 + (long)buf[6745]);
ancsom->second_overlap_pixel = ((long)buf[6746] * 256 + (long)buf[6747]);
ancsom->third_overlap_line = ((long)buf[6748] * 256 + (long)buf[6749]);
ancsom->third_overlap_pixel = ((long)buf[6750] * 256 + (long)buf[6751]);
ancsom->fourth_overlap_line = ((long)buf[6752] * 256 + (long)buf[6753]);
ancsom->fourth_overlap_pixel = ((long)buf[6754] * 256 + (long)buf[6755]);

ancsom->tick_marks_top = (long)buf[6756];
ancsom->tick_marks_left = (long)buf[6757];
ancsom->tick_marks_right = (long)buf[6758];
ancsom->tick_marks_bottom = (long)buf[6759];

for (i = 0; i < 4; i++)
   ancsom->samp_corner_pts[i]=((long)buf[6760+(i*2)]*256+(long)buf[6761+(i*2)]);

c_pxsys(&insys,&outsys,&buf[6768],&dbdtype,&one,&dunder,&dover,&hdr_flag);
ancsom->image_orientatn_ang = *(double *)&buf[6768];
ancsom->num_sweeps = ((long)buf[6776]*256+(long)buf[6777]);

return(E_SUCC);
}		/* end of c_edips_anc */
