/***************************************************************************
NAME:				C_EDIPS_ANN

PURPOSE:  Reads EDIPS Annotation record and puts information in ANSII structure

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  1.0		12/92	S. Nelson	Original development

***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "edips.h"
#include "header.h"

/* Routine to read MSS EDIPS ANNOTATION FILE
  -----------------------------------------*/
int c_edips_ann
(
    char *outh,
    struct EDIPSA *annot
)

{
FILE *fptr;	/* file pointer				*/

long status;		/* return status			*/
long i;			/* loop counters                        */
long  mode = 0;         /* flag value for lsstat                */
long nonfatal = 1;	/* nonfatal error indicator             */
long temp;		/* temporary storage location		*/

char finame[CMLEN];	/* Host file name			*/

char buf[3596];

/* Open annotation file
*  -------------------*/
c_lsmknm(outh,"edips_annot",finame);
  status = c_lsstat(finame,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("ANN file does not exist on disk","edips_ann-stat",&nonfatal);
     return(E_FAIL);
     }

fptr=fopen(finame,"r");
if(!fptr)
   {
   c_errmsg("Error opening Annotation file","edips_ann-open",&nonfatal);
   return(E_FAIL);
   }
fread(buf,1,3596,fptr);
fclose(fptr);

/*  read annotation file into structure
   -----------------------------------*/
annot->record_number = (long)buf[7];
annot->record_type = (long)buf[9];
strncpy(annot->exposure_day_mon_yr,&buf[10],8);
annot->exposure_day_mon_yr[8] = '\0';
strncpy(annot->format_ctr_lat_long,&buf[18],16);
annot->format_ctr_lat_long[16] = '\0';
strncpy(annot->path_row_identifier,&buf[35],8);
annot->path_row_identifier[8] = '\0';
strncpy(annot->wrs_ctr_lat_long,&buf[44],16);
annot->wrs_ctr_lat_long[16] = '\0';
strncpy(annot->sensor_band_id_code,&buf[61],7);
annot->sensor_band_id_code[7] = '\0';
strncpy(annot->transmission,&buf[69],1);
annot->transmission[1] = '\0';
strncpy(annot->sun_angles,&buf[71],13);
annot->sun_angles[13] = '\0';
strncpy(annot->correction_applied,&buf[85],1);
annot->correction_applied[1] = '\0';
	/* Landsat 1-3 only
	   ----------------*/
strncpy(annot->scale_of_image,&buf[86],1);
annot->scale_of_image[1] = '\0';
strncpy(annot->projection,&buf[87],1);
annot->projection[1] = '\0';
strncpy(annot->resample_algorithm,&buf[89],1);
annot->resample_algorithm[1] = '\0';
strncpy(annot->type_ephemeris_data,&buf[90],1);
annot->type_ephemeris_data[1] = '\0';
strncpy(annot->process_procedure,&buf[92],1);
annot->process_procedure[1] = '\0';
strncpy(annot->sensor_gain_opt,&buf[94],1);
annot->sensor_gain_opt[1] = '\0';
strncpy(annot->mss_transmission,&buf[95],1);
annot->mss_transmission[1] = '\0';
strncpy(annot->agency_and_project,&buf[97],12);
annot->agency_and_project[12] = '\0';
strncpy(annot->frame_id_number,&buf[110],15);
annot->frame_id_number[15] = '\0';

/* Read in Tick Mark data
   ---------------------*/
   /* top/bottom edge of image */

for (i = 0; i < 16; i++)
   {			/* top */
   temp = (long)buf[408 + (i*9)];
   if (temp < 128)	/* annotation is right of tick mark--trailing blanks */
      {
      annot->tm_top_location[i] = temp * 256 + (long)buf[409 + (i*9)];
      strncpy(annot->tm_top_loctn_flg[i],"0",1);
      annot->tm_top_loctn_flg[i][1] = '\0';
      }
   else			/* annotation is left of tick mark--leading blanks */
      {
      annot->tm_top_location[i] = (temp - 128) * 256 + (long)buf[409 + (i*9)];
      strncpy(annot->tm_top_loctn_flg[i],"1",1);
      annot->tm_top_loctn_flg[i][1] = '\0';
      }
			/* bottom */
   temp = (long)buf[2398 + (i*9)];
   if (temp < 128)	/* annotation is right of tick mark--trailing blanks */
      {
      annot->tm_bot_location[i] = temp * 256 + (long)buf[2399 + (i*9)];
      strncpy(annot->tm_bot_loctn_flg[i],"0",1);
      annot->tm_bot_loctn_flg[i][1] = '\0';
      }
   else			/* annotation is left of tick mark--leading blanks */
      {
      annot->tm_bot_location[i] = (temp - 128) * 256 + (long)buf[2399 + (i*9)];
      strncpy(annot->tm_bot_loctn_flg[i],"1",1);
      annot->tm_bot_loctn_flg[i][1] = '\0';
      }

   strncpy(annot->tm_top_coord[i],&buf[410+(i*9)],7);
   annot->tm_top_coord[i][7] = '\0';
   strncpy(annot->tm_bot_coord[i],&buf[2400+(i*9)],7);
   annot->tm_bot_coord[i][7] = '\0';
   }

   /* left/right edge of image (two separate sections) */
for (i = 0; i < 18; i++)
   {
				/* left
				------*/
   temp = (long)buf[806 + (i*9)];
   if (temp < 128)	/* annotation is below of tick mark--trailing blanks */
      {
      annot->tm_left_location[i] = temp * 256 + (long)buf[807 + (i*9)];
      strncpy(annot->tm_left_loctn_flg[i],"0",1);
      annot->tm_left_loctn_flg[i][1] = '\0';
      }
   else			/* annotation is above of tick mark--leading blanks */
      {
      annot->tm_left_location[i] = (temp - 128) * 256 + (long)buf[807 + (i*9)];
      strncpy(annot->tm_left_loctn_flg[i],"1",1);
      annot->tm_left_loctn_flg[i][1] = '\0';
      }
				/* right
				-------*/
   temp = (long)buf[1602 + (i*9)];
   if (temp < 128)	/* annotation is below of tick mark--trailing blanks */
      {
      annot->tm_rgt_location[i] = temp * 256 + (long)buf[1603 + (i*9)];
      strncpy(annot->tm_rgt_loctn_flg[i],"0",1);
      annot->tm_rgt_loctn_flg[i][1] = '\0';
      }
   else			/* annotation is above of tick mark--leading blanks */
      {
      annot->tm_rgt_location[i] = (temp - 128) * 256 + (long)buf[1603 + (i*9)];
      strncpy(annot->tm_rgt_loctn_flg[i],"1",1);
      annot->tm_rgt_loctn_flg[i][1] = '\0';
      }

   strncpy(annot->tm_left_coord[i],&buf[808+(i*9)],7);
   annot->tm_left_coord[i][7] = '\0';
   strncpy(annot->tm_rgt_coord[i],&buf[1604+(i*9)],7);
   annot->tm_rgt_coord[i][7] = '\0';
   }

for (i = 0; i < 7; i++)
   {
				/* left
				------*/
   temp = (long)buf[1204 + (i*9)];
   if (temp < 128)	/* annotation is below of tick mark--trailing blanks */
      {
      annot->tm_left_location[i + 18] = temp * 256 + (long)buf[1205 + (i*9)];
      strncpy(annot->tm_left_loctn_flg[i + 18],"0",1);
      annot->tm_left_loctn_flg[i + 18][1] = '\0';
      }
   else			/* annotation is above of tick mark--leading blanks */
      {
      annot->tm_left_location[i+18] = (temp - 128) * 256+(long)buf[1205+(i*9)];
      strncpy(annot->tm_left_loctn_flg[i+18],"1",1);
      annot->tm_left_loctn_flg[i + 18][1] = '\0';
      }
				/* right
				-------*/
   temp = (long)buf[2000 + (i*9)];
   if (temp < 128)	/* annotation is below of tick mark--trailing blanks */
      {
      annot->tm_rgt_location[i+18] = temp * 256 + (long)buf[2001 + (i*9)];
      strncpy(annot->tm_rgt_loctn_flg[i+18],"0",1);
      annot->tm_rgt_loctn_flg[i +18][1] = '\0';
      }
   else			/* annotation is above of tick mark--leading blanks */
      {
      annot->tm_rgt_location[i+18] = (temp-128) * 256 + (long)buf[2001 + (i*9)];
      strncpy(annot->tm_rgt_loctn_flg[i+18],"1",1);
      annot->tm_rgt_loctn_flg[i +18][1] = '\0';
      }
   strncpy(annot->tm_left_coord[i + 18],&buf[1206 +(i*9)],7);
   annot->tm_left_coord[i+18][7] = '\0';
   strncpy(annot->tm_rgt_coord[i + 18],&buf[2002 +(i*9)],7);
   annot->tm_rgt_coord[i+18][7] = '\0';
   }

return(E_SUCC);
}		/* end of c_edips_ann */
