/***************************************************************************
NAME:				C_EDIPS_HDR

PURPOSE:  Reads EDIPS header record and puts information in ANSII file

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  1.0		12/92	S. Nelson	Original Development
  2.0		 1/93	T. Mittan	Null terminated strings

***************************************************************************/
#include <stdio.h>
#include "las.h"
#include "labserv.h"
#include "pixman.h"
#include "edips.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read MSS EDIPS HEADER FILE
  --------------------------------------*/
int c_edips_hdr
(
    char *outh,		/* host image name			*/
    long outsys,		/* output system			*/
    struct EDIPSH *hdr	/* EDIPS header structure		*/
)

{
FILE *fptr;

long i, j;		/* loop counters			*/
long  mode = 0;         /* flag value for lsstat                */
long nonfatal = 1;	/* nonfatal error indicator		*/
long one = 1;		/* one					*/
long stat_tmp[32];	/* temporary storage for status values	*/
long status;		/* return status			*/
long temp;		/* temporary storage location		*/

unsigned char tempflt[4];	/* temporary array to convert to float	*/
unsigned char tempdub[8];	/* temporary array to convert to double */

char finame[CMLEN];	/* name of EDIPS tape file		*/

unsigned char buf[3603];/* buffer for EDIPS tape information	*/

/* Open header file
   -----------------*/
c_lsmknm(outh,"edips_headr",finame);
  status = c_lsstat(finame,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("HDR file does not exist on disk","edips_hdr-stat",&one);
     return(E_FAIL);
     }

fptr=fopen(finame,"r");
if(!fptr)
   {
   c_errmsg("Error opening Header file","edips_hdr-open",&nonfatal);
   return(E_FAIL);
   }
fread(buf,1,3603,fptr);
fclose(fptr);

/*  read EDIPS header file
   ----------------------*/
    /* A. Scene Identification
    -------------------------*/
hdr->record_number = (long)buf[7];
hdr->record_type_code = (long)buf[9];
hdr->record_length = ((long)buf[2] * 256 + (long)buf[3]);
strncpy(hdr->image_id,(char *)&buf[10],12);
hdr->image_id[12] = '\0';
strncpy(hdr->wrs_designator,(char *)&buf[22],8);
hdr->wrs_designator[8] = '\0';
hdr->tape_generation_day = (long)buf[30];
hdr->tape_generation_mon = (long)buf[31];
hdr->tape_generation_yr = (long)buf[32];

    /* B. Spacecraft Description
    ---------------------------*/
strncpy(hdr->sensor_id,(char *)&buf[40],3);
hdr->sensor_id[3] = '\0';
hdr->mission_number = (long)buf[49];
hdr->orbit_number = ((long)buf[50] * 256 + (long)buf[51]);

		/* bitwise munipulation with 4 bytes 1=active, 0=inactive
		   put in temporary storage at first since bits and bands
		   do not correlate */

for (j = 0; j < 4; j++)
   for (i = 0; i < 8; i++)
      stat_tmp[(j*8)+i] = ((buf[52+j] << i) & 0x80) ? 1 : 0;
for (j = 0; j < 5; j++)
   for (i = 0; i < 6; i++)
      hdr->status_bnd_detect[j][i] = stat_tmp[(j*6)+i];

/*hdr->active_detector = (long)buf[60];*/
hdr->pix_original_image = ((long)buf[61] * 256 + (long)buf[62]);

    /* C. Time of Exposure/WRS Designator
    ------------------------------------*/
hdr->wrs_scan_line = ((long)buf[76] * 256 + (long)buf[77]);
hdr->wrs_pixel_number = ((long)buf[78] * 256 + (long)buf[79]);
strncpy(hdr->center_year,(char *)&buf[80],2);
hdr->center_year[2] = '\0';
strncpy(hdr->center_julian_day,(char *)&buf[82],3);
hdr->center_hour[2] = '\0';
strncpy(hdr->center_hour,(char *)&buf[85],2);
hdr->center_hour[2] = '\0';
strncpy(hdr->center_min,(char *)&buf[87],2);
hdr->center_min[2] = '\0';
strncpy(hdr->center_sec,(char *)&buf[89],2);
hdr->center_sec[2] = '\0';
strncpy(hdr->center_milliseconds,(char *)&buf[90],3);
hdr->center_milliseconds[3] = '\0';

   /* D. Data Identification and Characteristics
   ---------------------------------------------*/
hdr->header_length = ((long)buf[96] * 256 + (long)buf[97]);
hdr->number_header_records = (long)buf[99];
hdr->header_bytes = ((long)buf[100] * 256 + (long)buf[101]);

	/* Annotation Data Characteristics
	---------------------------------*/
hdr->annotat_length = ((long)buf[102] * 256 + (long)buf[103]);
hdr->number_annotat_records = (long)buf[105];

	/* Ancillary Data Characteristics
	--------------------------------*/
hdr->ancill_length = ((long)buf[106] * 256 + (long)buf[107]);
hdr->number_ancill_records = (long)buf[109];
if ((long)buf[110] == 0)
   strncpy(hdr->geom_correct,"no ",3);
else
   strncpy(hdr->geom_correct,"yes",3);
hdr->geom_correct[3] = '\0';

if ((long)buf[111] == 0)
   strncpy(hdr->geom_data,"no ",3);
else
   strncpy(hdr->geom_data,"yes",3);
hdr->geom_data[3] = '\0';

if ((long)buf[112] == 0)
   strncpy(hdr->radiometric_correct,"no ",3);
else
   strncpy(hdr->radiometric_correct,"yes",3);
hdr->radiometric_correct[3] = '\0';

if ((long)buf[113] == 0)
   strncpy(hdr->radiometric_data,"no ",3);
else
   strncpy(hdr->radiometric_data,"yes",3);
hdr->radiometric_data[3] = '\0';

	/* Image Data Characteristics
	----------------------------*/
hdr->image_record_length = ((long)buf[114] * 256 + (long)buf[115]);
hdr->calibration_words_line = ((long)buf[118] * 256 + (long)buf[119]);
if ((long)buf[120] == 0)
   strncpy(hdr->image_data_format,"unframed        ",16);
else if((long)buf[120] == 255)
   strncpy(hdr->image_data_format,"framed rectangle",16);
else
   strncpy(hdr->image_data_format,"framed sqare    ",16);
hdr->image_data_format[16] = '\0';

if ((long)buf[123] == 0)
   strncpy(hdr->interleav_type,"BSQ",3);
else
   strncpy(hdr->interleav_type,"BIL",3);
hdr->interleav_type[3] = '\0';
hdr->bil_interleaving_count = (long)buf[124];
hdr->bits_per_pixel = (long)buf[125];
hdr->resampling = (long)buf[126];
hdr->map_projection = (long)buf[127];

temp = (long)buf[128];  /* assume most significant bit is left */
if ((temp) < 128)     /* offset is positive--most significant digit is 0 */
   hdr->wrs_center_offset = (temp * 256 + (long)buf[129]);
else		      /* offset is negative--most significant digit is 1 */ 
   hdr->wrs_center_offset = -((temp-128) * 256 + (long)buf[129]);
if ((long)buf[132] == 0)
   strncpy(hdr->data_justification,"left ",5);
else
   strncpy(hdr->data_justification,"right",5);
hdr->data_justification[5] = '\0';
if ((long)buf[133] == 0)
   strncpy(hdr->most_signifcnt_bit,"left ",5);
else
   strncpy(hdr->most_signifcnt_bit,"right",5);
hdr->most_signifcnt_bit[5] = '\0';

hdr->pixels_per_line = ((long)buf[134] * 256 + (long)buf[135]);
hdr->images_per_scene = (long)buf[138];
strncpy(hdr->mss_band_number,(char *)&buf[139],1);
hdr->mss_band_number[1] = '\0';
hdr->support_data = ((long)buf[143] * 256 + (long)buf[144]);

	/* Trailer Data Characteristics
	------------------------------*/
hdr->trailer_length = ((long)buf[145] * 256 + (long)buf[146]);
hdr->number_trailer_record = ((long)buf[147] * 256 + (long)buf[148]);

	/* Special Purpose Fields
	------------------------*/
if ((long)buf[154] == 0)
   strncpy(hdr->orbit_direction,"day  ",5);
else
   strncpy(hdr->orbit_direction,"night",5);
hdr->orbit_direction[5] = '\0';
		/* convert to double--landsat 4
		   ---------------------------*/
for (i = 0; i < 8; i++)
   tempdub[i] = buf[156 + i];
status = c_pxsys(&insys,&outsys,tempdub,&dbdtype,&one,&dunder,&dover,&hdr_flag);
hdr->orientation_angle = *(double *)tempdub;
		/* all mission numbers
		   ------------------*/
hdr->sensor_mode = (long)buf[165];
	/* Temporal Registration Data
	----------------------------*/
   strncpy(hdr->temporal_scene_id,(char *)&buf[166],12);
   hdr->temporal_scene_id[12] = '\0';
   strncpy(hdr->temp_wrs_designator,(char *)&buf[178],8);
   hdr->temp_wrs_designator[8] = '\0';

if ((long)buf[110] != 0) 	/* corrected data */
   {
   for (i = 0; i < 4; i++)
      {
      hdr->cur_imag_line[i] = ((long)buf[186+i*8] * 256 + (long)buf[187+i*8]);
      hdr->cur_imag_pixel[i] = ((long)buf[188+i*8] * 256 + (long)buf[189+i*8]);
      hdr->ref_imag_line[i] = ((long)buf[190+i*8] * 256 + (long)buf[191+i*8]);
      hdr->ref_imag_pixel[i] = ((long)buf[192+i*8] * 256 + (long)buf[193+i*8]);
      }
   }
else				/* uncorrected data */
   {
   for (i = 0; i < 4; i++)
      {
      hdr->cur_imag_line[i] = 0;
      hdr->cur_imag_pixel[i] = 0;
      hdr->ref_imag_line[i] = 0;
      hdr->ref_imag_pixel[i] = 0;
      }
   }
hdr->first_overlap_line = ((long)buf[218] * 256 + (long)buf[219]);
hdr->first_overlap_pixel = ((long)buf[220] * 256 + (long)buf[221]);
hdr->second_overlap_line = ((long)buf[222] * 256 + (long)buf[223]);
hdr->second_overlap_pixel = ((long)buf[224] * 256 + (long)buf[225]);
hdr->third_overlap_line = ((long)buf[226] * 256 + (long)buf[227]);
hdr->third_overlap_pixel = ((long)buf[228] * 256 + (long)buf[229]);
hdr->fourth_overlap_line = ((long)buf[230] * 256 + (long)buf[231]);
hdr->fourth_overlap_pixel = ((long)buf[232] * 256 + (long)buf[233]);
hdr->overlap_pixel_offset = (long)buf[234];
strncpy(hdr->quality_code,(char *)&buf[235],1); 
hdr->quality_code[1] = '\0';
hdr->tick_marks_top = (long)buf[236];   
hdr->tick_marks_left = (long)buf[237];   
hdr->tick_marks_right = (long)buf[238];   
hdr->tick_marks_bottom = (long)buf[239];   

	/* Data not available for landsat 1,2 and 3
	   ---------------------------------------*/
if (hdr->mission_number > 3)
   strncpy(hdr->band_quality,(char *)&buf[240],1); 
else
   strncpy(hdr->band_quality," ",1); 
hdr->band_quality[1] = '\0';

hdr->radiometric_calibratn = (long)buf[241];

		/* value is a float--assure first byte is even */
for (i = 0; i < 4; i++)
   tempflt[i] = buf[243 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->calibration_accuracy = *(float *)tempflt;

	/* Input Data Quality Indicators
	   ----------------------------*/
		/* Telemetry:
		------------*/
hdr->tel_ephemeris_points = ((long)buf[250] * 256 + (long)buf[251]);
hdr->tel_rejected_ephemeris = ((long)buf[251] * 256 + (long)buf[253]); 
hdr->tel_attitude_points = ((long)buf[254] * 256 + (long)buf[255]);
hdr->tel_rejected_attitude = ((long)buf[256] * 256 + (long)buf[257]);
		/* value is a float--assure first byte is even */
for (i = 0; i < 4; i++)
   tempflt[i] = buf[258 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->tel_interval_length = *(float *)tempflt;

		/* Accuracy of fit, RMS difference in meters */
for (i = 0; i < 4; i++)
   tempflt[i] = buf[264 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->tel_ephm_accuracy_alt = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[268 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->tel_ephm_acc_along_trck = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[272 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->tel_ephm_acc_acros_trck = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[278 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->tel_att_accuracy_pitch = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[282 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->tel_att_accuracy_roll = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[286 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->tel_att_accuracy_yaw = *(float *)tempflt;

		/* Control Points:
		   --------------*/
for (i = 0; i < 4; i++)
   {
   if (hdr->mission_number > 3)
       strncpy(hdr->cp_band_quality[i],(char *)&buf[290 + i],1); 
   else                /* landsat 1-3 has binary null, set to blank */
       strncpy(hdr->cp_band_quality[i]," ",1); 
   hdr->cp_band_quality[i][1] = '\0';
   }
hdr->cp_geodetic_points = (long)buf[294];
hdr->cp_registratn_success = (long)buf[295];

for (i = 0; i < 4; i++)
   tempflt[i] = buf[298 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->cp_autocorrelatn_init = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[302 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->cp_err_ellipse_along = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[306 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->cp_err_ellipse_across = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[310 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->cp_autocorrelatn_ref = *(float *)tempflt;

for (i = 0; i < 4; i++)
   tempflt[i] = buf[314 + i];
status = c_pxsys(&insys,&outsys,tempflt,&fldtype,&one,&funder,&fover,&hdr_flag);
hdr->cp_suitability = *(float *)tempflt;

if (hdr->mission_number > 3)
    strncpy(hdr->data_source,(char *)&buf[318],1); 
else                /* landsat 1-3 has binary null, set to blank */
    strncpy(hdr->data_source," ",1); 
hdr->data_source[1] = '\0';
hdr->uncorrect_ecc_count = ((long)buf[324] * 256 + (long)buf[325]);
hdr->bit_error_rate = ((long)buf[326] * 256 + (long)buf[327]);
hdr->cal_wedge_value_use = (long)buf[330];

hdr->window_size = (long)buf[331];
for (i = 0; i < 36; i++)
   hdr->cal_wedge_values[i] = (long)buf[332 + i];

for (i = 0; i < 6; i++)
   for (j = 0; j < 6; j++)
      hdr->wed_qual_sens_samp[i][j] = (long)buf[368+(i*6)+j];

for (i = 0; i < 8; i++)
   tempdub[i] = buf[404 + i];
status = c_pxsys(&insys,&outsys,tempdub,&dbdtype,&one,&dunder,&dover,&hdr_flag);
hdr->wrs_center_lat = *(double *)tempdub;

for (i = 0; i < 8; i++)
   tempdub[i] = buf[412 + i];
status = c_pxsys(&insys,&outsys,tempdub,&dbdtype,&one,&dunder,&dover,&hdr_flag);
hdr->wrs_center_long = *(double *)tempdub;

hdr->contrast_enhancement = (long)buf[3586];
hdr->atmospheric_scatter = (long)buf[3587];
hdr->edge_enhancement = (long)buf[3588];

for (i = 0; i < 4; i++)
    {
    strncpy(hdr->sensor_gain_opt[i],(char *)&buf[3590 + i],1); 
    hdr->sensor_gain_opt[i][1] = '\0';
    strncpy(hdr->transmission_type[i],(char *)&buf[3595 + i],1); 
    hdr->transmission_type[i][1] = '\0';
    }
  
return(E_SUCC);
}		/* end of c_edips_hdr */
