/************************************
*  Common includes and definitions  *
************************************/
#include <stdio.h>
#include "las.h"
#include "labserv.h"
#include "jsc.h"
#include "header.h"
#include "local_protos.h"

#define  ANC_SIZE 178 

/*******************************************************************************
FUNCTION:	c_jsc_anc	

PURPOSE: 	To read the annotation  file and fill the JSCIN structure.	

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
1.0	 T. Mittan	  2/92	 LAS	Original development.

ALGORITHM REFERENCES	none
*******************************************************************************/
int c_jsc_anc
(
    struct JSCIN *jscin,	/* TIPSP sturcture declaration		*/
    char *name,     		/* image name				*/
    long *nl		    	/* line number to seek to		*/
)
{
   FILE *fp;			/* file pointer				*/

   long  status;		/* return status			*/
   long  mode = 0;		/* flag value for lsstat		*/
   long  one = 1;		/* constant value of one		*/

   char temp[4];		/* temporary variable(must start on even 
				   boundary 				*/
   char  buf[3601];             /* header buffer                        */
   char  hostheader[CMLEN];	/* header name				*/

  c_lsmknm(name,"anc",hostheader);
  status = c_lsstat(hostheader,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("ANC file does not exist on disk","read_annot-stat",&one);
     return(E_FAIL);
     }
  /* use data from band quality data record
  ---------------------------------------*/
  fp = (fopen(hostheader,"r"));
  if(!fp)
   {
   c_errmsg("Error opening Ancillary file","read_anc-open",&one);
   return(E_FAIL);
   }

  fseek(fp,*nl * ANC_SIZE,0);
  fread(buf,1,ANC_SIZE,fp);
  fclose(fp);

  jscin->anc_start = get_short(&buf[104]);
  jscin->anc_stop = get_short(&buf[106]);
 
  temp[0] = 0;
  temp[1] = buf[108];
  temp[2] = buf[109];
  temp[3] = buf[110];
  jscin->anc_line_length = get_long(temp);
  jscin->anc_days = (long)buf[111];
  jscin->anc_hours = (long)buf[112];
  jscin->anc_minutes = (long)buf[113];
  jscin->anc_seconds = (long)buf[114];
  jscin->anc_hundreth_seconds = (long)buf[115];

  jscin->anc_sensor = (long)buf[116];

  jscin->anc_cal_band_4[0] = (long)buf[117];
  jscin->anc_cal_band_4[1] = (long)buf[118];
  jscin->anc_cal_band_4[2] = (long)buf[119];
  jscin->anc_cal_band_4[3] = (long)buf[120];
  jscin->anc_cal_band_4[4] = (long)buf[121];
  jscin->anc_cal_band_4[5] = (long)buf[122];
  jscin->anc_cal_band_5[0] = (long)buf[123];
  jscin->anc_cal_band_5[1] = (long)buf[124];
  jscin->anc_cal_band_5[2] = (long)buf[125];
  jscin->anc_cal_band_5[3] = (long)buf[126];
  jscin->anc_cal_band_5[4] = (long)buf[127];
  jscin->anc_cal_band_5[5] = (long)buf[128];
  jscin->anc_cal_band_6[0] = (long)buf[129];
  jscin->anc_cal_band_6[1] = (long)buf[130];
  jscin->anc_cal_band_6[2] = (long)buf[131];
  jscin->anc_cal_band_6[3] = (long)buf[132];
  jscin->anc_cal_band_6[4] = (long)buf[133];
  jscin->anc_cal_band_6[5] = (long)buf[134];
  jscin->anc_cal_band_7[0] = (long)buf[135];
  jscin->anc_cal_band_7[1] = (long)buf[136];
  jscin->anc_cal_band_7[2] = (long)buf[137];
  jscin->anc_cal_band_7[3] = (long)buf[138];
  jscin->anc_cal_band_7[4] = (long)buf[139];
  jscin->anc_cal_band_7[5] = (long)buf[140];
  jscin->anc_cal_thermal[0] = (long)buf[141];
  jscin->anc_cal_thermal[1] = (long)buf[142];
  jscin->anc_cal_thermal[2] = (long)buf[143];
  jscin->anc_cal_thermal[3] = (long)buf[144];
  jscin->anc_cal_thermal[4] = (long)buf[145];
  jscin->anc_cal_thermal[5] = (long)buf[146];

return(E_SUCC);
}
