/***************************************************************************
NAME:				C_JSC_LHDR

PURPOSE:  Reads JSC Landsat header record and puts information in ANSII file

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  1.0		1/93	S. Nelson	Original development
  1.1		2/93	D. Etrheim	Pad the lat/long fields to be
					five characters (DDDMM)

***************************************************************************/
#include <stdio.h>
#include <ctype.h>
#include "las.h"
#include "labserv.h"
#include "util.h"
#include "jsc.h"
#include "header.h"

/* Routine to read MSS JSC LANDSAT HEADER FILE
  -------------------------------------------*/
int c_jsc_lhdr
(
    char *outh,		/* image host file name				*/
    struct JSCLH *lhdr	/* JSC Landsat header structure			*/
)

{
FILE *fptr;	/* file pointer					*/

long i;		/* loop counters				*/
long mode = 0;		/* flag value for lsstat              	  	*/
long nonfatal = 1;	/* nonfatal error indicator			*/
long status;		/* return status				*/
long temp;		/* temporary value				*/

char finame[CMLEN];	/* Host file name				*/
char form[6];		/* format flag					*/

char buf[3600];     /* buffer for JSC tape information 		*/

/* Open LANDSAT header file
   -------------------------*/
c_lsmknm(outh,"lhdr",finame);
  status = c_lsstat(finame,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("LHDR file does not exist on disk","jsc_lhdr-stat",&nonfatal);
     return(E_FAIL);
     }

fptr=fopen(finame,"r");
if(!fptr)
   {
   c_errmsg("Error opening landsat header file","jsc_lhdr-open",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,3600,fptr);

/* check to see if EBCDIC or ASCII
   -------------------------------*/
temp = 1;
if (strncmp (&buf[1369],"1",1) == 0)
   strncpy(form,"ASCII",5);
else
   {
   c_eb2asc(&buf[1369],&temp);
   if (strncmp (&buf[1369],"0",1) == 0)
      strncpy(form,"EBCDIC",6);
   else
      {
      fclose(fptr);
      c_errmsg("Can not distinguish Landsat header format","jsc_lhdr-form",
								&nonfatal);
      return(E_FAIL);
      }
   }
strncpy(lhdr->character_code_for_lhdr,&buf[1369],1);
lhdr->character_code_for_lhdr[1] = '\0';

/* If EBCDIC, convert to ASCII
   --------------------------*/ 
if (strncmp (form,"EBCDIC",6) == 0)
   {
   temp = 1400;
   c_eb2asc(&buf[0],&temp);
   }
   
/* Begin assigning values to structure
   ----------------------------------*/
strncpy(lhdr->productn_syst_id_code,&buf[0],10);
lhdr->productn_syst_id_code[10] = '\0';

strncpy(lhdr->id_code_description,&buf[10],30);
lhdr->id_code_description[30] = '\0';

strncpy(lhdr->mission_id,&buf[80],10);
lhdr->mission_id[10] = '\0';

strncpy(lhdr->title1,&buf[90],30);
lhdr->title1[30] = '\0';

strncpy(lhdr->day_num_since_launch,&buf[160],10);
lhdr->day_num_since_launch[10] = '\0';

strncpy(lhdr->title2,&buf[170],30);
lhdr->title2[30] = '\0';

strncpy(lhdr->orbit_number,&buf[240],10);
lhdr->orbit_number[10] = '\0';

strncpy(lhdr->title3,&buf[250],30);
lhdr->title3[30] = '\0';

strncpy(lhdr->frame_centre_time,&buf[320],10);
lhdr->frame_centre_time[10] = '\0';

strncpy(lhdr->title4,&buf[330],30);
lhdr->title4[30] = '\0';

strncpy(lhdr->frame_centre_lat,&buf[400],10);
lhdr->frame_centre_lat[10] = '\0';

/* If a space exists, zero fill after the negative sign
  ----------------------------------------------------*/
for (i = 9; i > 4; i--)
   if (isspace(lhdr->frame_centre_lat[i]))
	{
	if (lhdr->frame_centre_lat[i+1] != '-')
            lhdr->frame_centre_lat[i] = '0';
	else
	    {
	    lhdr->frame_centre_lat[i] = '-';
	    lhdr->frame_centre_lat[i+1] = '0';
	    }
	}

strncpy(lhdr->title5,&buf[410],30);
lhdr->title5[30] = '\0';

strncpy(lhdr->frame_centre_long,&buf[480],10);
lhdr->frame_centre_long[10] = '\0';
for (i = 9; i > 4; i--)
   if (isspace(lhdr->frame_centre_long[i]))
	{
	if (lhdr->frame_centre_long[i+1] != '-')
            lhdr->frame_centre_long[i] = '0';
	else
	    {
	    lhdr->frame_centre_long[i] = '-';
	    lhdr->frame_centre_long[i+1] = '0';
	    }
	}

strncpy(lhdr->title6,&buf[490],30);
lhdr->title6[30] = '\0';

strncpy(lhdr->utm_zone_number,&buf[560],10);
lhdr->utm_zone_number[10] = '\0';

strncpy(lhdr->title7,&buf[570],30);
lhdr->title7[30] = '\0';

strncpy(lhdr->path_number,&buf[640],10);
lhdr->path_number[10] = '\0';

strncpy(lhdr->title8,&buf[650],30);
lhdr->title8[30] = '\0';

strncpy(lhdr->row_number,&buf[720],10);
lhdr->row_number[10] = '\0';

strncpy(lhdr->title9,&buf[730],30);
lhdr->title9[30] = '\0';

strncpy(lhdr->cycle_number,&buf[800],10);
lhdr->cycle_number[10] = '\0';

strncpy(lhdr->title10,&buf[810],30);
lhdr->title10[30] = '\0';
 

strncpy(lhdr->data_acquisition_ddmmyy,&buf[880],10);
lhdr->data_acquisition_ddmmyy[10] = '\0';

strncpy(lhdr->title11,&buf[890],15);
lhdr->title11[15] = '\0';

strncpy(lhdr->data_acquisition_ddmmmyy,&buf[905],15);
lhdr->data_acquisition_ddmmmyy[15] = '\0';


strncpy(lhdr->master_cct_producd_ddmmyy,&buf[960],10);
lhdr->master_cct_producd_ddmmyy[10] = '\0';

strncpy(lhdr->title12,&buf[970],20);
lhdr->title12[20] = '\0';

strncpy(lhdr->master_cct_producd_ddmmmyy,&buf[990],10);
lhdr->master_cct_producd_ddmmmyy[10] = '\0';


strncpy(lhdr->date_copy_producd_ddmmyy,&buf[1040],10);
lhdr->date_copy_producd_ddmmyy[10] = '\0';

strncpy(lhdr->title13,&buf[1050],15);
lhdr->title13[15] = '\0';

strncpy(lhdr->date_copy_producd_ddmmmyy,&buf[1065],15);
lhdr->date_copy_producd_ddmmmyy[15] = '\0';


strncpy(lhdr->heading,&buf[1120],10);
lhdr->heading[10] = '\0';

strncpy(lhdr->title14,&buf[1130],30);
lhdr->title14[30] = '\0';

strncpy(lhdr->seq_number_of_tape,&buf[1200],10);
lhdr->seq_number_of_tape[10] = '\0';

strncpy(lhdr->title15,&buf[1210],30);
lhdr->title15[30] = '\0';

strncpy(lhdr->tape_start_time,&buf[1280],10);
lhdr->tape_start_time[10] = '\0';

strncpy(lhdr->title16,&buf[1290],30);
lhdr->title16[30] = '\0';

strncpy(lhdr->title17,&buf[1370],30);
lhdr->title17[30] = '\0';
strncpy(lhdr->radiometric_calibration,&buf[1363],1);
lhdr->radiometric_calibration[1] = '\0';
strncpy(lhdr->radiometric_levels,&buf[1364],1);
lhdr->radiometric_levels[1] = '\0';
strncpy(lhdr->earth_rotatn_correctn,&buf[1365],1);
lhdr->earth_rotatn_correctn[1] = '\0';
strncpy(lhdr->mirror_scan_velocity_corr,&buf[1366],1);
lhdr->mirror_scan_velocity_corr[1] = '\0';
strncpy(lhdr->data_representation,&buf[1367],1);
lhdr->data_representation[1] = '\0';
strncpy(lhdr->line_length_correction,&buf[1368],1);
lhdr->line_length_correction[1] = '\0';
/* 
strncpy(lhdr->character_code_for_lhdr,&buf[1369],1);
lhdr->character_code_for_lhdr[1] = '\0';
*/

/* Geometric Transformation Record
   ------------------------------*/
fseek(fptr,1440,0);
fread(buf,1,100,fptr);
fclose(fptr);

/* If EBCDIC, convert to ASCII
   --------------------------*/
if (strncmp (form,"EBCDIC",6) == 0)
   {
   temp = 100;
   c_eb2asc(&buf[0],&temp);
   }
   
/* Assign values in Geometric Transformation structure
   --------------------------------------------------*/
strncpy(lhdr->gt_utm_zone_number,&buf[0],20);
lhdr->gt_utm_zone_number[20] = '\0';
strncpy(lhdr->gt_utm_coordinate_northing,&buf[20],20);
lhdr->gt_utm_coordinate_northing[20] = '\0';
strncpy(lhdr->gt_utm_coordinate_easting,&buf[40],20);
lhdr->gt_utm_coordinate_easting[20] = '\0';
strncpy(lhdr->gt_orientation,&buf[60],20);
lhdr->gt_orientation[20] = '\0';

return(E_SUCC);
}		/* end of c_jsc_lhdr */
