/***************************************************************************
NAME:				C_JSC_RADTRAN

PURPOSE:  Reads the JSC Landsat header record and puts the Radiometric
  	  Transformation information in an ANSII file.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  1.0		1/93	S. Nelson	Original development

***************************************************************************/
#define	LGBYTE	4140
#define	SMBYTE	2160
#define NUMBYTES 1620

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "util.h"
#include "jsc.h"
#include "header.h"

/* Routine to read MSS JSC LANDSAT HEADER FILE
  -------------------------------------------*/
int c_jsc_radtran
(
    char *outh,		/* image host file name				*/
    struct JSCRT *jscrt4,	/* JSC Landsat header Radiometric Transformation
			   band 4					*/
    struct JSCRT *jscrt5,	/* JSC Landsat header Radiometric Transformation
			   band 5					*/
    struct JSCRT *jscrt6,	/* JSC Landsat header Radiometric Transformation
			   band 6					*/
    struct JSCRT *jscrt7,	/* JSC Landsat header Radiometric Transformation
			   band 7					*/
    struct JSCRT *jscrt8	/* JSC Landsat header Radiometric Transformation
			   band 8					*/
)

{
FILE *fptr;	/* file pointer					*/

long bytflag;		/* flag for number of bytes in Geom. Trans. section */
long i;		/* loop counters				*/
long  mode = 0;         /* flag value for lsstat                	*/
long nonfatal = 1;	/* nonfatal error indicator			*/
long status;		/* return status				*/
long temp;		/* temporary value				*/

char finame[CMLEN];	/* Host file name				*/
char form[6];		/* Flag for format of data (EBCDIC or ASCII)	*/

char buf[3600];     /* buffer for JSC tape information 		*/

/* Open LANDSAT header file
   -------------------------*/
c_lsmknm(outh,"lhdr",finame);
  status = c_lsstat(finame,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("LHDR file does not exist on disk","jsc_radtran-stat",&nonfatal);
     return(E_FAIL);
     }

fptr=fopen(finame,"r");
if(!fptr)
   {
   c_errmsg("Error opening landsat header file","jsc_radtran-open",&nonfatal);
   return(E_FAIL);
   }

/* Read first section to check format and # of bytes
   ------------------------------------------------*/
fread(buf,1,3600,fptr);

		/* Test if EBCDIC or ASCII
		   ----------------------*/
if (strncmp (&buf[1369],"1",1) != 0)
   {
   strncpy(form,"EBCDIC",6);
   temp = 3;
   c_eb2asc(&buf[15],&temp);
   }
else
   strncpy(form,"ASCII",5);

/* Find the number of bytes in Geom. Tran. section
   ----------------------------------------------*/
if (strncmp (&buf[15],"ELS",3) == 0)
   bytflag = SMBYTE;
else
   bytflag = LGBYTE;

/* Radiometric Transformation Band 4
   --------------------------------*/

fseek(fptr,bytflag,0);
fread(buf,1,1620,fptr);

/* If EBCDIC, convert to ASCII
   --------------------------*/
if (strncmp (form,"EBCDIC",6) == 0)
   {
   temp = 1620;
   c_eb2asc(&buf[0],&temp);
   }

/* Assign values to structure
   -------------------------*/
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt4->radiometric_transformation_detector_1[i],&buf[i*4],4);
  jscrt4->radiometric_transformation_detector_1[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt4->radiometric_transformation_detector_2[i],&buf[256+i*4],4);
  jscrt4->radiometric_transformation_detector_2[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt4->radiometric_transformation_detector_3[i],&buf[512+i*4],4);
  jscrt4->radiometric_transformation_detector_3[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt4->radiometric_transformation_detector_4[i],&buf[768+i*4],4);
  jscrt4->radiometric_transformation_detector_4[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt4->radiometric_transformation_detector_5[i],&buf[1024+i*4],4);
  jscrt4->radiometric_transformation_detector_5[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt4->radiometric_transformation_detector_6[i],&buf[1280+i*4],4);
  jscrt4->radiometric_transformation_detector_6[i][4] = '\0';
  }

/* Radiometric Transformation Band 5
   --------------------------------*/

fseek(fptr,bytflag + NUMBYTES * 1,0);
fread(buf,1,1620,fptr);

/* If EBCDIC, convert to ASCII
   --------------------------*/
if (strncmp (form,"EBCDIC",6) == 0)
   {
   temp = 1620;
   c_eb2asc(&buf[0],&temp);
   }

/* Assign values to structure
   -------------------------*/
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt5->radiometric_transformation_detector_1[i],&buf[i*4],4);
  jscrt5->radiometric_transformation_detector_1[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt5->radiometric_transformation_detector_2[i],&buf[256+i*4],4);
  jscrt5->radiometric_transformation_detector_2[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt5->radiometric_transformation_detector_3[i],&buf[512+i*4],4);
  jscrt5->radiometric_transformation_detector_3[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt5->radiometric_transformation_detector_4[i],&buf[768+i*4],4);
  jscrt5->radiometric_transformation_detector_4[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt5->radiometric_transformation_detector_5[i],&buf[1024+i*4],4);
  jscrt5->radiometric_transformation_detector_5[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt5->radiometric_transformation_detector_6[i],&buf[1280+i*4],4);
  jscrt5->radiometric_transformation_detector_6[i][4] = '\0';
  }

/* Radiometric Transformation Band 6
   --------------------------------*/

fseek(fptr,bytflag + NUMBYTES * 2,0);
fread(buf,1,1620,fptr);

/* If EBCDIC, convert to ASCII
   --------------------------*/
if (strncmp (form,"EBCDIC",6) == 0)
   {
   temp = 1620;
   c_eb2asc(&buf[0],&temp);
   }

/* Assign values to structure
   -------------------------*/
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt6->radiometric_transformation_detector_1[i],&buf[i*4],4);
  jscrt6->radiometric_transformation_detector_1[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt6->radiometric_transformation_detector_2[i],&buf[256+i*4],4);
  jscrt6->radiometric_transformation_detector_2[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt6->radiometric_transformation_detector_3[i],&buf[512+i*4],4);
  jscrt6->radiometric_transformation_detector_3[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt6->radiometric_transformation_detector_4[i],&buf[768+i*4],4);
  jscrt6->radiometric_transformation_detector_4[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt6->radiometric_transformation_detector_5[i],&buf[1024+i*4],4);
  jscrt6->radiometric_transformation_detector_5[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt6->radiometric_transformation_detector_6[i],&buf[1280+i*4],4);
  jscrt6->radiometric_transformation_detector_6[i][4] = '\0';
  }

/* Radiometric Transformation Band 7
   --------------------------------*/

fseek(fptr,bytflag + NUMBYTES * 3,0);
fread(buf,1,1620,fptr);

/* If EBCDIC, convert to ASCII
   --------------------------*/
if (strncmp (form,"EBCDIC",6) == 0)
   {
   temp = 1620;
   c_eb2asc(&buf[0],&temp);
   }

/* Assign values to structure
   -------------------------*/
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt7->radiometric_transformation_detector_1[i],&buf[i*4],4);
  jscrt7->radiometric_transformation_detector_1[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt7->radiometric_transformation_detector_2[i],&buf[256+i*4],4);
  jscrt7->radiometric_transformation_detector_2[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt7->radiometric_transformation_detector_3[i],&buf[512+i*4],4);
  jscrt7->radiometric_transformation_detector_3[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt7->radiometric_transformation_detector_4[i],&buf[768+i*4],4);
  jscrt7->radiometric_transformation_detector_4[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt7->radiometric_transformation_detector_5[i],&buf[1024+i*4],4);
  jscrt7->radiometric_transformation_detector_5[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt7->radiometric_transformation_detector_6[i],&buf[1280+i*4],4);
  jscrt7->radiometric_transformation_detector_6[i][4] = '\0';
  }

/* Radiometric Transformation Band 8
   --------------------------------*/

fseek(fptr,bytflag + NUMBYTES * 4,0);
fread(buf,1,1620,fptr);
fclose(fptr);

/* If EBCDIC, convert to ASCII
   --------------------------*/
if (strncmp (form,"EBCDIC",6) == 0)
   {
   temp = 1620;
   c_eb2asc(&buf[0],&temp);
   }

/* Assign values to structure
   -------------------------*/
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt8->radiometric_transformation_detector_1[i],&buf[i*4],4);
  jscrt8->radiometric_transformation_detector_1[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt8->radiometric_transformation_detector_2[i],&buf[256+i*4],4);
  jscrt8->radiometric_transformation_detector_2[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt8->radiometric_transformation_detector_3[i],&buf[512+i*4],4);
  jscrt8->radiometric_transformation_detector_3[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt8->radiometric_transformation_detector_4[i],&buf[768+i*4],4);
  jscrt8->radiometric_transformation_detector_4[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt8->radiometric_transformation_detector_5[i],&buf[1024+i*4],4);
  jscrt8->radiometric_transformation_detector_5[i][4] = '\0';
  }
for (i = 0; i < 64; i++)
  {
  strncpy(jscrt8->radiometric_transformation_detector_6[i],&buf[1280+i*4],4);
  jscrt8->radiometric_transformation_detector_6[i][4] = '\0';
  }
return(E_SUCC);
}		/* end of c_jsc_radtran */
