/***************************************************************************
NAME:				C_JSC_UHDR

PURPOSE:  Reads JSC Universal header record and puts information in ANSII file

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  1.0		1/93	S. Nelson	Original development

***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "util.h"
#include "jsc.h"
#include "header.h"

/* Routine to read MSS JSC UNIVERSAL HEADER FILE
  ---------------------------------------------*/
int c_jsc_uhdr
(
    char *outh,		/* image host file name				*/
    struct JSCUH *uhdr	/* JSC Universal header structure		*/
)

{
FILE *fptr;	/* file pointer					*/

long i;		/* loop counters				*/
long  mode = 0;         /* flag value for lsstat                */
long nonfatal = 1;	/* nonfatal error indicator			*/
long status;		/* return status				*/
long temp;		/* temporary variable				*/

char finame[CMLEN];	/* Host file name				*/

char buf[7193];     /* buffer for JSC tape information 		*/
unsigned char templon[4];/* buffer for JSC tape file			*/

/* Open universal header file
   -------------------------*/
c_lsmknm(outh,"uhdr",finame);
  status = c_lsstat(finame,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("UHDR file does not exist on disk","jsc_uhdr-stat",&nonfatal);
     return(E_FAIL);
     }

fptr=fopen(finame,"r");
if(!fptr)
   {
   c_errmsg("Error opening universal header file","jsc_uhdr-open",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,3058,fptr);

temp = 32;
c_eb2asc(&buf[0],&temp);
strncpy(uhdr->computing_sys_id,&buf[0],temp);
uhdr->computing_sys_id[temp] = '\0';
temp = 20;
c_eb2asc(&buf[32],&temp);
strncpy(uhdr->tape_library_id,&buf[32],temp);
uhdr->tape_library_id[temp] = '\0';
temp = 8;
c_eb2asc(&buf[52],&temp);
strncpy(uhdr->sensor_id,&buf[52],temp);
uhdr->sensor_id[temp] = '\0';
uhdr->master_tape_gen_day = (long)buf[60];
uhdr->master_tape_gen_mon = (long)buf[61];
uhdr->master_tape_gen_yr = (long)buf[62];
uhdr->tape_seq_number = (long)buf[63];
uhdr->mission_number = ((long)buf[64] * 256 + (long)buf[65]);
uhdr->row_number = ((long)buf[66] * 256 + (long)buf[67]);
uhdr->path_number = (long)buf[68];
uhdr->run_number = (long)buf[69];
uhdr->orbit_number = ((long)buf[70] * 256 + (long)buf[71]);

uhdr->first_scan_millisec = ((long)buf[72] * 256 + (long)buf[73]);
uhdr->first_scan_seconds = (long)buf[74];
uhdr->first_scan_minutes = (long)buf[75];
uhdr->first_scan_hours = (long)buf[76];
uhdr->first_scan_days = (long)buf[77];
uhdr->first_scan_month_num = (long)buf[78];
uhdr->first_scan_year = (long)buf[79];

		/* Active channels are bit 1, Other channels are bit 0 
		   --------------------------------------------------*/
for (i = 0; i < 8; i++)
   {
   if (buf[80+i] & 0x80)
      uhdr->channels_active[i*8] = 1;
   else
      uhdr->channels_active[i*8] = 0;

   if (buf[80+i] & 0x40)
      uhdr->channels_active[(i*8)+1] = 1;
   else
      uhdr->channels_active[(i*8)+1] = 0;

   if (buf[80+i] & 0x20)
      uhdr->channels_active[(i*8)+2] = 1;
   else
      uhdr->channels_active[(i*8)+2] = 0;

   if (buf[80+i] & 0x10)
      uhdr->channels_active[(i*8)+3] = 1;
   else
      uhdr->channels_active[(i*8)+3] = 0;

   if (buf[80+i] & 0x08)
      uhdr->channels_active[(i*8)+4] = 1;
   else
      uhdr->channels_active[(i*8)+4] = 0;

   if (buf[80+i] & 0x04)
      uhdr->channels_active[(i*8)+5] = 1;
   else
      uhdr->channels_active[(i*8)+5] = 0;

   if (buf[80+i] & 0x02)
      uhdr->channels_active[(i*8)+6] = 1;
   else
      uhdr->channels_active[(i*8)+6] = 0;

   if (buf[80+i] & 0x01)
      uhdr->channels_active[(i*8)+7] = 1;
   else
      uhdr->channels_active[(i*8)+7] = 0;

   }
	
uhdr->processing_flag = (long)buf[88];
uhdr->num_channels_on_tape = (long)buf[89];
uhdr->num_bits_in_picture = (long)buf[90];
uhdr->byt_loctn_strt_of_video_data = ((long)buf[91]*256 + (long)buf[92]);
uhdr->byt_loctn_strt_of_calibratn = ((long)buf[93]*256 + (long)buf[94]);
uhdr->num_video_elemnts_per_scan = ((long)buf[95]*256 + (long)buf[96]);
uhdr->num_calibratn_elemnts_scan = ((long)buf[97]*256 + (long)buf[98]);
uhdr->record_size_in_bytes = ((long)buf[99]*256 + (long)buf[100]);
uhdr->channels_per_record = (long)buf[101];
uhdr->num_records_per_scan = (long)buf[102];
uhdr->num_records_data_set = (long)buf[103];
uhdr->byts_ancill_data_per_data_set = ((long)buf[104]*256 + (long)buf[105]);
uhdr->data_order_indicator = (long)buf[106];
uhdr->start_pixel_number = ((long)buf[107]*256 + (long)buf[108]);
uhdr->stop_pixel_number = ((long)buf[109]*256 + (long)buf[110]);

	/* Ao Coeffiecients per channel */
for (i = 0; i < 64; i++)
   if (buf[111+(2*i)] & 0x80)
      {
      buf[111+(2*i)] &= 0x7f;
      uhdr->a0_coefficient[i] = -1*((long)buf[111+2*i]*256+(long)buf[112+2*i]);
      }
   else
      uhdr->a0_coefficient[i] = ((long)buf[111+2*i]*256+(long)buf[112+2*i]);

	/* Eo Exponents per channel */
for (i = 0; i < 64; i++)
   if (buf[239+(2*i)] & 0x80)
      {
      buf[239+(2*i)] &= 0x7f;
      uhdr->e0_exponent[i] = -1*((long)buf[239+2*i]*256+(long)buf[240+2*i]);
      }
   else
      uhdr->e0_exponent[i] = ((long)buf[239+2*i]*256+(long)buf[240+2*i]);

	/* A1 Coeffiecients per channel */
for (i = 0; i < 64; i++)
   if (buf[367+(2*i)] & 0x80)
      {
      buf[367+(2*i)] &= 0x7f;
      uhdr->a1_coefficient[i] = -1*((long)buf[367+2*i]*256+(long)buf[368+2*i]);
      }
   else
      uhdr->a1_coefficient[i] = ((long)buf[367+2*i]*256+(long)buf[368+2*i]);

	/* E1 Exponents per channel */
for (i = 0; i < 64; i++)
   if (buf[495+(2*i)] & 0x80)
      {
      buf[495+(2*i)] &= 0x7f;
      uhdr->e1_exponent[i] = -1*((long)buf[495+2*i]*256+(long)buf[496+2*i]);
      }
   else
      uhdr->e1_exponent[i] = ((long)buf[495+2*i]*256+(long)buf[496+2*i]);

for (i = 0; i < 64; i++)
   uhdr->color_code_data_chanl[i] = (long)buf[623+i];
uhdr->picture_field_width = (long)buf[750];
uhdr->pixel_registration = (long)buf[751];
uhdr->computer_word_size = (long)buf[752];

temp = 8;
for (i = 0; i < 64; i++)
   {
   c_eb2asc(&buf[753+16*i],&temp);
   strncpy(uhdr->lowr_lmt_wavelength[i],&buf[753+16*i],temp);
   uhdr->lowr_lmt_wavelength[i][temp] = '\0';
   c_eb2asc(&buf[761+16*i],&temp);
   strncpy(uhdr->uppr_lmt_wavelength[i],&buf[761+16*i],temp);
   uhdr->uppr_lmt_wavelength[i][temp] = '\0';
   }
uhdr->data_sets_per_record = (long)buf[1777];
uhdr->byte_address_start_second_calibratn=((long)buf[1778]*256+(long)buf[1779]);
uhdr->calibration_elements_second_area=((long)buf[1780]*256+(long)buf[1781]);
uhdr->calibration_source_indicator = (long)buf[1782];
uhdr->file_skip_flag = (long)buf[1783];
uhdr->bands_in_first_record=((long)buf[1784]*256+(long)buf[1785]);
uhdr->bytes_per_scan=((long)buf[1786]*256+(long)buf[1787]);
uhdr->pixel_skip_factor=((long)buf[1788]*256+(long)buf[1789]);
uhdr->scan_skip_factor=((long)buf[1790]*256+(long)buf[1791]);

temp = 8;
for (i = 0; i < 4; i++)
   {
   c_eb2asc(&buf[2488+8*i],&temp);
   strncpy(uhdr->general_information[i],&buf[2488+8*i],temp);
   uhdr->general_information[i][temp] = '\0';
   }
c_eb2asc(&buf[2737],&temp);
strncpy(uhdr->sun_elevation_angle_centre_deg,&buf[2737],temp);
uhdr->sun_elevation_angle_centre_deg[temp] = '\0';

c_eb2asc(&buf[2745],&temp);
strncpy(uhdr->sun_azimuth_angle_centre_deg,&buf[2745],temp);
uhdr->sun_azimuth_angle_centre_deg[temp] = '\0';

uhdr->number_auxiliary_files = (long)buf[2753];
uhdr->start_scan_line=((long)buf[2754]*256+(long)buf[2755]);
uhdr->stop_scan_line=((long)buf[2756]*256+(long)buf[2757]);
uhdr->scan_lins_per_frame_per_thous = (long)buf[2758];

/* Image Annotation Data
   ---------------------*/
temp = 1;
c_eb2asc(&buf[2759],&temp);
strncpy(uhdr->ann_satellite_station_id,&buf[2759],temp);
uhdr->ann_satellite_station_id[temp] = '\0';

temp = 3;
c_eb2asc(&buf[2761],&temp);
strncpy(uhdr->ann_path,&buf[2761],temp);
uhdr->ann_path[temp] = '\0';

temp = 3;
c_eb2asc(&buf[2765],&temp);
strncpy(uhdr->ann_row,&buf[2765],temp);
uhdr->ann_row[temp] = '\0';

temp = 2;
c_eb2asc(&buf[2769],&temp);
strncpy(uhdr->ann_day,&buf[2769],temp);
uhdr->ann_day[temp] = '\0';

temp = 2;
c_eb2asc(&buf[2772],&temp);
strncpy(uhdr->ann_month,&buf[2772],temp);
uhdr->ann_month[temp] = '\0';

temp = 2;
c_eb2asc(&buf[2775],&temp);
strncpy(uhdr->ann_year,&buf[2775],temp);
uhdr->ann_year[temp] = '\0';

temp = 1;
c_eb2asc(&buf[2778],&temp);
strncpy(uhdr->ann_mission,&buf[2778],temp);
uhdr->ann_mission[temp] = '\0';

temp = 4;
c_eb2asc(&buf[2779],&temp);
strncpy(uhdr->ann_days_since_launch,&buf[2779],temp);
uhdr->ann_days_since_launch[temp] = '\0';

temp = 2;
c_eb2asc(&buf[2784],&temp);
strncpy(uhdr->ann_hours,&buf[2784],temp);
uhdr->ann_hours[temp] = '\0';

temp = 2;
c_eb2asc(&buf[2786],&temp);
strncpy(uhdr->ann_minutes,&buf[2786],temp);
uhdr->ann_minutes[temp] = '\0';

temp = 1;
c_eb2asc(&buf[2788],&temp);
strncpy(uhdr->ann_tens_of_seconds,&buf[2788],temp);
uhdr->ann_tens_of_seconds[temp] = '\0';

templon[0] = 0;
for (i = 1; i < 4; i++)
   templon[i] = buf[2789+i-1];
uhdr->nominal_altitude_met = get_long_var(templon);

uhdr->nominal_ground_speed_met_sec=((long)buf[2792]*256+(long)buf[2793]);
uhdr->scan_type = (long)buf[2794];
uhdr->arc_angle_deg = (long)buf[2795];
uhdr->camera = (long)buf[2796];
uhdr->input_device = (long)buf[2797];
uhdr->truncation = (long)buf[2798];

		/* Requested channels are bit 1, Other channels are bit 0 
		   -----------------------------------------------------*/
for (i = 0; i < 8; i++)
   {
   if (buf[2799+i] & 0x80)
      uhdr->channels_requested[i*8] = 1;
   else
      uhdr->channels_requested[i*8] = 0;

   if (buf[2799+i] & 0x40)
      uhdr->channels_requested[(i*8)+1] = 1;
   else
      uhdr->channels_requested[(i*8)+1] = 0;

   if (buf[2799+i] & 0x20)
      uhdr->channels_requested[(i*8)+2] = 1;
   else
      uhdr->channels_requested[(i*8)+2] = 0;

   if (buf[2799+i] & 0x10)
      uhdr->channels_requested[(i*8)+3] = 1;
   else
      uhdr->channels_requested[(i*8)+3] = 0;

   if (buf[2799+i] & 0x08)
      uhdr->channels_requested[(i*8)+4] = 1;
   else
      uhdr->channels_requested[(i*8)+4] = 0;

   if (buf[2799+i] & 0x04)
      uhdr->channels_requested[(i*8)+5] = 1;
   else
      uhdr->channels_requested[(i*8)+5] = 0;

   if (buf[2799+i] & 0x02)
      uhdr->channels_requested[(i*8)+6] = 1;
   else
      uhdr->channels_requested[(i*8)+6] = 0;

   if (buf[2799+i] & 0x01)
      uhdr->channels_requested[(i*8)+7] = 1;
   else
      uhdr->channels_requested[(i*8)+7] = 0;
   }
uhdr->processing_mode = (long)buf[2807];
uhdr->color_select = (long)buf[2873];
uhdr->image_format = (long)buf[2874];
uhdr->repeat_pixels_per_scan = (long)buf[2875];
uhdr->repeat_of_scan = (long)buf[2876];
uhdr->partial_scan_strt_pixel=((long)buf[2877]*256+(long)buf[2878]);
uhdr->partial_scan_end_pixel=((long)buf[2879]*256+(long)buf[2880]);
uhdr->sensor_scan_rate=((long)buf[2881]*256+(long)buf[2882]);
uhdr->pixel_size = (long)buf[2883];
	
if (buf[2884] & 0x80)
   {
   buf[2884] &= 0x7f;
   uhdr->angle_of_drift_int = -1*(long)buf[2884];
   }
else
   uhdr->angle_of_drift_int = (long)buf[2884];
uhdr->angle_of_drift_fract = (long)buf[2885];

return(E_SUCC);
}		/* end of c_jsc_uhdr */

long get_long_var
(
    unsigned char *val
)
{
    long *alias = (long *)val;
    return(*alias);
}
