/***************************************************************************
NAME:				C_LTWG_EAA

PURPOSE:  Reads LTWG ephomeris volume descriptor attitude ancillary record 
          from a file that is passed in already opened, and put it into a 
          structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG EPHOMERIS & ATTITUDE ANCILLARY RECORD 
-------------------------------------------------------------*/
int c_ltwg_eaa
(
    struct LTWGEAA *eaa, /* OUTPUT - LTWG ephomeris volume descriptor */
                         /* attitude ancillary rec structure 	*/
    FILE   *fin,         /* INPUT - File pointer			*/
    int    *size         /* INPUT - record size in bytes		*/
)
{

char *buf;          /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long   i, j, k;		/* counting variables			*/
long  nonfatal = 1;	

buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,*size,fin);

/*  read LTWG ephomeris volume descriptor attitude ancillary record
    ---------------------------------------------------------------*/
for (k=0; k<4; k++)
   temp[k]=buf[k];
eaa->rec_number = get_long(temp);

eaa->frst_rec_code = (long)buf[4];

eaa->rec_code = (long)buf[5];

eaa->scnd_rec_code = (long)buf[6];

eaa->thrd_rec_code = (long)buf[7];

eaa->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(tmpbuf,&buf[12],4);
tmpbuf[4] = '\0';
eaa->rec_seq_num = atol(tmpbuf);

strncpy(tmpbuf,&buf[16],1);
tmpbuf[1] = '\0';
eaa->ephemeris_flag = atol(tmpbuf);

strncpy(tmpbuf,&buf[17],1);
tmpbuf[1] = '\0';
eaa->attitude_flag = atol(tmpbuf);  

strncpy(tmpbuf,&buf[18],1);
tmpbuf[1] = '\0';
eaa->gyro_flag = atol(tmpbuf);  

strncpy(tmpbuf,&buf[19],1);
tmpbuf[1] = '\0';
eaa->gyro_drift_flag = atol(tmpbuf);  

      /*   PCD MAJOR FRAME START TIME    */
strncpy(tmpbuf,&buf[20],4);
tmpbuf[4] = '\0';
eaa->pcd_start_day = atol(tmpbuf);

strncpy(tmpbuf,&buf[24],8);
tmpbuf[8] = '\0';
eaa->pcd_start_msec = atol(tmpbuf);

strncpy(tmpbuf,&buf[32],3);
tmpbuf[3] = '\0';
eaa->pcd_start_usec = atol(tmpbuf);

strncpy(tmpbuf,&buf[35],1);
tmpbuf[1]='0';
eaa->pcd_start_tenths = atol(tmpbuf); 

     /* EPHEMERIS DATA */
strncpy(tmpbuf,&buf[36],10);
tmpbuf[10] = '\0';
eaa->eph_time_offset = atol(tmpbuf);

for (i=0; i<3; i++)
   {
   strncpy(tmpbuf,&buf[46+(i*22)],22);
   strncpy(&tmpbuf[18],"E",1);
   tmpbuf[22]='\0';
   eaa->craft_pos[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[112+(i*22)],22);
   strncpy(&tmpbuf[18],"E",1);
   tmpbuf[22] = '\0';
   eaa->craft_vel[i] = atof(tmpbuf);
   }

     /* ATTITUDE DATA */
strncpy(tmpbuf,&buf[178],10);
tmpbuf[10] = '\0';
eaa->att_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[188],22);
strncpy(&tmpbuf[18],"E",1);
tmpbuf[22] = '\0';
eaa->epa1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[210],22);
strncpy(&tmpbuf[18],"E",1);
tmpbuf[22] = '\0';
eaa->epa2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[232],22);
strncpy(&tmpbuf[18],"E",1);
tmpbuf[22] = '\0';
eaa->epa3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[254],22);
strncpy(&tmpbuf[18],"E",1);
tmpbuf[22] = '\0';
eaa->epa4 = atof(tmpbuf);

     /* GYRO DATA  */
strncpy(tmpbuf,&buf[276],10);
tmpbuf[10] = '\0';
eaa->gyro_time_offset = atol(tmpbuf);

for (i=0; i<64; i++)
   {
   for (j=0; j<3; j++)
      {
      strncpy(tmpbuf,&buf[286+(i*66)+(j*22)],22);
      strncpy(&tmpbuf[18],"E",1);
      tmpbuf[22] = '\0';
      eaa->gyro_meas[j][i] = atof(tmpbuf);
      }
   }

     /* GYRO DRIFT DATA */
strncpy(tmpbuf,&buf[4510],10);
tmpbuf[10] = '\0';
eaa->gyro_drift_time_offset = atol(tmpbuf);

for (i=0; i<3; i++)
   {
   strncpy(tmpbuf,&buf[4520+(i*22)],22);
   strncpy(&tmpbuf[18],"E",1);
   tmpbuf[22] = '\0';
   eaa->gyro_drift[i] = atof(tmpbuf);
   }
free (buf);
return(E_SUCC);
}                   /* end of c_ltwg_eaa */
