/***************************************************************************
NAME:				C_LTWG_IDR

PURPOSE:  Reads LTWG image data record from a file that is passed
          in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG IMAGE DATA RECORD 
-----------------------------------------*/
int c_ltwg_idr
(
    struct LTWGIDR *idr,    /* OUTPUT - LTWG image data rec structure	*/
    FILE *fin               /* INPUT - File pointer				*/
)
{

char buf[90]; /* buffer for LTWG tape information	*/
char temp[4];
long k;

fread(buf,1,88,fin);

/*  read LTWG image data record
    -----------------------------*/
     /* PREFIX DATA */

for(k=0;k<4;k++)
   temp[k]=buf[k];
idr->scan_line = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+4];
idr->band_num = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+8];
idr->start_time = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+12];
idr->left_fill = get_long(temp);
 
for(k=0;k<4;k++)
   temp[k]=buf[k+16];
idr->right_fill = get_long(temp);

     /* SUFFIX DATA */
idr->sync_loss = (long)buf[20];

idr->local_qual_flag1= (long)buf[21]; 

idr->detect_sub = (long)buf[22]; 

idr->local_qual_flag2 = (long)buf[23]; 
 
temp[0]=temp[1]=0;
for(k=2;k<4;k++)
   temp[k]=buf[k+26];
idr->cal_pulse_width = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+28];
idr->line_lgth = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+32];
idr->embedded_lgth = get_long(temp);

temp[0]=temp[1]=0;
for(k=2;k<4;k++)
   temp[k]=buf[k+36];
idr->time_err_start = get_long(temp);

temp[0]=temp[1]=0;
for(k=2;k<4;k++)
   temp[k]=buf[k+38];
idr->time_err_end = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+40];
idr->scan_direction = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+44];
idr->half_line_lngth = get_long(temp);

sprintf(idr->sat_time_code,"%.2d%.2d%.2d%.2d%.2d%.2d%.2d%.2d",get_bcd(&buf[48]),
	get_bcd(&buf[49]),get_bcd(&buf[50]),get_bcd(&buf[51]),
	get_bcd(&buf[52]),get_bcd(&buf[53]),get_bcd(&buf[54]),
	get_bcd(&buf[55]));

    /* CALIBRATION INFORMATION */

idr->detect_id = (long)buf[56]; 

idr->calib_lamp_qual = (long)buf[57];         

idr->calib_lamp_state = (long)buf[58];         

idr->calib_state_seq = (long)buf[59];         

for(k=0;k<4;k++)
   temp[k]=buf[k+60];
idr->low_calib_value_bef = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+64];
idr->high_calib_value = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+68];
idr->comput_gain =get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+72];
idr->comput_bais = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+76];
idr->app_gain = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+80];
idr->app_bais = get_long(temp);

for(k=0;k<4;k++)
   temp[k]=buf[k+84];
idr->low_calib_value_aft = get_long(temp);

free (buf);

return(E_SUCC);
}                   /* end of c_ltwg_idr */
