/***************************************************************************
NAME:				C_LTWG_IFD

PURPOSE:  Read LTWG imagery file descriptor record from a file that is passed 
          in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG LEADER FILE DESCRIPTOR RECORD 
-----------------------------------------------------*/
int c_ltwg_ifd
(
    struct LTWGIFD *ifd,    /* OUTPUT - LTWG imagery file descriptor  	*/
    FILE *fin               /* INPUT - File pointer				*/
)
{

char buf[7021];     /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long  k;		/* Counter variables			*/

fread(buf,1,7020,fin);

/*  read LTWG imagery file descriptor record
    ---------------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
ifd->rec_number = get_long(temp);

ifd->frst_rec_code = (long)buf[4];

ifd->rec_code = (long)buf[5];

ifd->scnd_rec_code = (long)buf[6];

ifd->thrd_rec_code = (long)buf[7];

ifd->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(ifd->a_e_flag,&buf[12],2);
ifd->a_e_flag[2] = '\0';

strncpy(ifd->control_doc_num,&buf[16],12);
ifd->control_doc_num[12] ='\0';

strncpy(ifd->control_doc_rev,&buf[28],2);
ifd->control_doc_rev[2] ='\0';

strncpy(ifd->file_des_rev,&buf[30],2);
ifd->file_des_rev[2] ='\0';

strncpy(ifd->rel_num,&buf[32],12);
ifd->rel_num[12] ='\0';

strncpy(tmpbuf,&buf[44],4);
tmpbuf[4] = '\0';
ifd->file_num = atol(tmpbuf);

strncpy(ifd->file_name,&buf[48],15);
ifd->file_name[15] ='\0';

strncpy(ifd->record_seq,&buf[64],4);
ifd->record_seq[4] ='\0';

strncpy(tmpbuf,&buf[68],8);
tmpbuf[8] = '\0';
ifd->seq_num_loc = atol(tmpbuf);

strncpy(tmpbuf,&buf[76],4);
tmpbuf[4] = '\0';
ifd->seq_num_lgt = atol(tmpbuf);

strncpy(ifd->rec_loc_flg,&buf[80],4);
ifd->rec_loc_flg[4] ='\0';

strncpy(tmpbuf,&buf[84],8);
tmpbuf[8] = '\0';
ifd->rec_loc = atol(tmpbuf);

strncpy(tmpbuf,&buf[92],4);
tmpbuf[4] = '\0';
ifd->rec_lth = atol(tmpbuf);

strncpy(ifd->rec_lth_flg,&buf[96],4);
ifd->rec_lth_flg[4] ='\0';

strncpy(tmpbuf,&buf[100],8);
tmpbuf[8] = '\0';
ifd->rec_lth_loc = atol(tmpbuf);

strncpy(tmpbuf,&buf[108],4);
tmpbuf[4] = '\0';
ifd->rec_fld_lth = atol(tmpbuf);

strncpy(ifd->data_interp_des,&buf[112],1); 
ifd->data_interp_des[1]='\0';

strncpy(ifd->data_interp_oth,&buf[113],1);
ifd->data_interp_oth[1]='\0';

strncpy(ifd->data_disp_des,&buf[114],1); 
ifd->data_disp_des[1]='\0';

strncpy(ifd->data_disp_oth,&buf[115],1);
ifd->data_disp_oth[1]='\0';

strncpy(tmpbuf,&buf[180],6);
tmpbuf[6] = '\0';
ifd->number_img_rec = atol(tmpbuf);

strncpy(tmpbuf,&buf[186],6);
tmpbuf[6] = '\0';
ifd->record_lth = atol(tmpbuf);

strncpy(tmpbuf,&buf[216],4);
tmpbuf[4] = '\0';
ifd->bits_per_pixel = atol(tmpbuf);

strncpy(tmpbuf,&buf[220],4);
tmpbuf[4] = '\0';
ifd->pixel_per_group = atol(tmpbuf);

strncpy(tmpbuf,&buf[224],4);
tmpbuf[4] = '\0';
ifd->bytes_per_group = atol(tmpbuf);

strncpy(ifd->pix_just,&buf[228],4);
ifd->pix_just[4] = '\0';

strncpy(tmpbuf,&buf[232],4);
tmpbuf[4] = '\0';
ifd->num_bands = atol(tmpbuf);

strncpy(tmpbuf,&buf[236],8);
tmpbuf[8] = '\0';
ifd->lines_band = atol(tmpbuf);

strncpy(tmpbuf,&buf[244],4);
tmpbuf[4] = '\0';
ifd->left_pix_line = atol(tmpbuf);

strncpy(tmpbuf,&buf[248],8);
tmpbuf[8] = '\0';
ifd->img_pix_line = atol(tmpbuf);

strncpy(tmpbuf,&buf[256],4);
tmpbuf[4] = '\0';
ifd->right_pix_line = atol(tmpbuf);

strncpy(tmpbuf,&buf[260],4);
tmpbuf[4] = '\0';
ifd->top_lines = atol(tmpbuf);

strncpy(tmpbuf,&buf[264],4);
tmpbuf[4] = '\0';
ifd->bot_lines = atol(tmpbuf);

strncpy(ifd->file_type,&buf[268],4);
ifd->file_type[4] = '\0';

strncpy(tmpbuf,&buf[272],2);
tmpbuf[2] = '\0';
ifd->records_line = atol(tmpbuf);

strncpy(tmpbuf,&buf[274],2);
tmpbuf[2] = '\0';
ifd->recs_multispec_line = atol(tmpbuf);

strncpy(tmpbuf,&buf[276],4);
tmpbuf[4] = '\0';
ifd->size_prefix = atol(tmpbuf);

strncpy(tmpbuf,&buf[280],8);
tmpbuf[8] = '\0';
ifd->size_image = atol(tmpbuf);

strncpy(tmpbuf,&buf[288],4);
tmpbuf[4] = '\0';
ifd->size_suffix = atol(tmpbuf);

strncpy(ifd->line_num_loc,&buf[296],8);
ifd->line_num_loc[8] = '\0';

strncpy(ifd->band_num_loc,&buf[304],8);
ifd->band_num_loc[8] = '\0';

strncpy(ifd->location_time,&buf[312],8);
ifd->location_time[8] = '\0';

strncpy(ifd->left_fill_loc,&buf[320],8);
ifd->left_fill_loc[8] = '\0';

strncpy(ifd->right_fill_loc,&buf[328],8);
ifd->right_fill_loc[8] = '\0';

strncpy(ifd->unknown1,&buf[368],8);
ifd->unknown1[8] = '\0';

strncpy(ifd->unknown2,&buf[376],8);
ifd->unknown2[8] = '\0';

strncpy(tmpbuf,&buf[432],4);
tmpbuf[4] = '\0';
ifd->left_fill = atol(tmpbuf);

strncpy(tmpbuf,&buf[436],4);
tmpbuf[4] = '\0';
ifd->right_fill = atol(tmpbuf);

strncpy(ifd->unknown3,&buf[440],8);
ifd->unknown3[8] = '\0';

return(E_SUCC);
}                   /* end of c_ltwg_ifd */
