/***************************************************************************
NAME:				C_LTWG_LFD

PURPOSE:  Read LTWG leader file descriptor record from a file that is passed 
          in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG LEADER FILE DESCRIPTOR RECORD 
-----------------------------------------------------*/
int c_ltwg_lfd
(
    struct LTWGLFD *lfd,    /* OUTPUT - LTWG leader file descriptor  	*/
    FILE *fin               /* INPUT - File pointer				*/
)
{

char buf[4321];     /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long  k;		/* Counter variables			*/

fread(buf,1,4320,fin);

/*  read LTWG leader file descriptor record
    ---------------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
lfd->rec_number = get_long(temp);

lfd->frst_rec_code = (long)buf[4];

lfd->rec_code = (long)buf[5];

lfd->scnd_rec_code = (long)buf[6];

lfd->thrd_rec_code = (long)buf[7];

lfd->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(lfd->a_e_flag,&buf[12],2);
lfd->a_e_flag[2] = '\0';

strncpy(lfd->control_doc_num,&buf[16],12);
lfd->control_doc_num[12] ='\0';

strncpy(lfd->control_doc_rev,&buf[28],2);
lfd->control_doc_rev[2] ='\0';

strncpy(lfd->file_des_rev,&buf[30],2);
lfd->file_des_rev[2] ='\0';

strncpy(lfd->rel_num,&buf[32],12);
lfd->rel_num[12] ='\0';

strncpy(tmpbuf,&buf[44],4);
tmpbuf[4] = '\0';
lfd->file_num = atol(tmpbuf);

strncpy(lfd->file_name,&buf[48],15);
lfd->file_name[15] ='\0';

strncpy(lfd->record_seq,&buf[64],4);
lfd->record_seq[4] ='\0';

strncpy(tmpbuf,&buf[68],8);
tmpbuf[8] = '\0';
lfd->seq_num_loc = atol(tmpbuf);

strncpy(tmpbuf,&buf[76],4);
tmpbuf[4] = '\0';
lfd->seq_num_lgt = atol(tmpbuf);

strncpy(lfd->rec_loc_flg,&buf[80],4);
lfd->rec_loc_flg[4] ='\0';

strncpy(tmpbuf,&buf[84],8);
tmpbuf[8] = '\0';
lfd->rec_loc = atol(tmpbuf);

strncpy(tmpbuf,&buf[92],4);
tmpbuf[4] = '\0';
lfd->rec_lth = atol(tmpbuf);

strncpy(lfd->rec_lth_flg,&buf[96],4);
lfd->rec_lth_flg[4] ='\0';

strncpy(tmpbuf,&buf[100],8);
tmpbuf[8] = '\0';
lfd->rec_lth_loc = atol(tmpbuf);

strncpy(tmpbuf,&buf[108],4);
tmpbuf[4] = '\0';
lfd->rec_fld_lth = atol(tmpbuf);

strncpy(lfd->data_interp_des,&buf[112],1); 
lfd->data_interp_des[1]='\0';

strncpy(lfd->data_interp_oth,&buf[113],1);
lfd->data_interp_oth[1]='\0';

strncpy(lfd->data_disp_des,&buf[114],1); 
lfd->data_disp_des[1]='\0';

strncpy(lfd->data_disp_oth,&buf[115],1);
lfd->data_disp_oth[1]='\0';

strncpy(tmpbuf,&buf[180],6);
tmpbuf[6] = '\0';
lfd->num_header = atol(tmpbuf);

strncpy(tmpbuf,&buf[186],6);
tmpbuf[6] = '\0';
lfd->lth_header = atol(tmpbuf);

strncpy(tmpbuf,&buf[192],6);
tmpbuf[6] = '\0';
lfd->num_map_proj = atol(tmpbuf);

strncpy(tmpbuf,&buf[198],6);
tmpbuf[6] = '\0';
lfd->lth_map_proj = atol(tmpbuf);

strncpy(tmpbuf,&buf[204],6);
tmpbuf[6] = '\0';
lfd->num_ancill_rec = atol(tmpbuf);

strncpy(tmpbuf,&buf[210],6);
tmpbuf[6] = '\0';
lfd->lth_ancill_rec = atol(tmpbuf);

strncpy(lfd->scene_id_loc,&buf[216],16);
lfd->scene_id_loc[16] = '\0';

strncpy(lfd->wrs_id_loc,&buf[232],16);
lfd->wrs_id_loc[16] = '\0';

strncpy(lfd->miss_id_loc,&buf[248],16);
lfd->miss_id_loc[16] = '\0';

strncpy(lfd->sensor_id_loc,&buf[264],16);
lfd->sensor_id_loc[16] = '\0';

strncpy(lfd->exp_date_loc,&buf[280],16);
lfd->exp_date_loc[16] = '\0';

strncpy(lfd->geog_ref_loc,&buf[296],16);
lfd->geog_ref_loc[16] = '\0';

strncpy(lfd->imag_proc_loc,&buf[312],16);
lfd->imag_proc_loc[16] = '\0';

strncpy(lfd->imag_for_loc,&buf[328],16);
lfd->imag_for_loc[16] = '\0';

strncpy(lfd->band_ind_loc,&buf[344],16);
lfd->band_ind_loc[16] = '\0';

strncpy(lfd->quad_ind_loc,&buf[360],16);
lfd->quad_ind_loc[16] = '\0';

strncpy(lfd->inter_scale_loc,&buf[376],16);
lfd->inter_scale_loc[16] = '\0';

strncpy(lfd->quad_ver_ovrlp,&buf[392],16);
lfd->quad_ver_ovrlp[16] = '\0';

strncpy(lfd->quad_hor_ovrlp,&buf[408],16);
lfd->quad_hor_ovrlp[16] = '\0';

return(E_SUCC);
}                   /* end of c_ltwg_lfd */
