/***************************************************************************
NAME:				C_LTWG_MPR

PURPOSE:  Reads LTWG Map projection ancillary record from a file that is 
          passed in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG MAP PROJECTION ANCILLARY RECORD 
-------------------------------------------------------*/
int c_ltwg_mpr
(
    struct LTWGMPR *mpr,    /* OUTPUT - LTWG map projection rec structure	*/
    FILE *fin,              /* INPUT - File pointer				*/
    long *size              /* INPUT - number of bytes to read for file	*/
)
{

char tmpbuf[50];	/* temporary buffer storage		*/
char *buf;          /* buffer for LTWG tape information     */
char temp[4];
long  i,k;		/* counter variable			*/
long  nonfatal = 1;

buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }


fread(buf,1,*size,fin);

/*  read LTWG map projection ancillary record
    -----------------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
mpr->rec_number = get_long(temp);

mpr->frst_rec_code = (long)buf[4];

mpr->rec_code = (long)buf[5];

mpr->scnd_rec_code = (long)buf[6];

mpr->thrd_rec_code = (long)buf[7];

mpr->rec_length = (long)buf[10]*256 + (long)buf[11];

      /*  Inut scene related data  */
strncpy(tmpbuf,&buf[12],16);
tmpbuf[16] = '\0';
mpr->nominal_num_pixels = atol(tmpbuf);

strncpy(tmpbuf,&buf[28],16);
tmpbuf[16] = '\0';
mpr->nominal_num_lines = (long)atof(tmpbuf);

strncpy(tmpbuf,&buf[44],16);
tmpbuf[16] = '\0';
mpr->nominal_scale_pixel = atof(tmpbuf);

strncpy(tmpbuf,&buf[60],16);
tmpbuf[16] = '\0';
mpr->nominal_scale_lines = atof(tmpbuf);

strncpy(tmpbuf,&buf[76],16);
tmpbuf[16] = '\0';
mpr->image_skew = atof(tmpbuf);

     /*  UTM/PS Related data for input scene  */
strncpy(mpr->utm_datum,&buf[92],6);
mpr->utm_datum[6] = '\0';

strncpy(tmpbuf,&buf[98],10);
tmpbuf[10] = '\0';
mpr->utm_zone = atol(tmpbuf);

strncpy(tmpbuf,&buf[108],16);
tmpbuf[16] = '\0';
mpr->wrs_northing = atof(tmpbuf);

strncpy(tmpbuf,&buf[124],16);
tmpbuf[16] = '\0';
mpr->wrs_easting = atof(tmpbuf);

strncpy(tmpbuf,&buf[140],16);
tmpbuf[16] = '\0';
mpr->input_northing = atof(tmpbuf);

strncpy(tmpbuf,&buf[156],16);
tmpbuf[16] = '\0';
mpr->input_easting = atof(tmpbuf);

strncpy(tmpbuf,&buf[172],16);
tmpbuf[16] = '\0';
mpr->vert_offset = atof(tmpbuf);

strncpy(tmpbuf,&buf[188],16);
tmpbuf[16] = '\0';
mpr->horz_offset = atof(tmpbuf);

strncpy(tmpbuf,&buf[204],16);
tmpbuf[16] = '\0';
mpr->orientation = atof(tmpbuf);

      /*  Processed scene related data */
strncpy(tmpbuf,&buf[332],16);
tmpbuf[16] = '\0';
mpr->prod_pixel_line = atof(tmpbuf);

strncpy(tmpbuf,&buf[348],16);
tmpbuf[16] = '\0';
mpr->prod_num_lines = atof(tmpbuf);

strncpy(tmpbuf,&buf[364],16);
tmpbuf[16] = '\0';
mpr->inter_pixel_scale = atof(tmpbuf);

strncpy(tmpbuf,&buf[380],16);
tmpbuf[16] = '\0';
mpr->inter_line_scale = atof(tmpbuf);

strncpy(mpr->proc_utm_datum,&buf[396],6);
mpr->proc_utm_datum[6] = '\0';

strncpy(tmpbuf,&buf[402],10);
tmpbuf[10] = '\0';
mpr->proc_utm_zome = atol(tmpbuf);

strncpy(tmpbuf,&buf[412],16);
tmpbuf[16] = '\0';
mpr->proc_center_line = atof(tmpbuf);

strncpy(tmpbuf,&buf[428],16);
tmpbuf[16] = '\0';
mpr->proc_center_pix = atof(tmpbuf);

strncpy(tmpbuf,&buf[444],16);
tmpbuf[16] = '\0';
mpr->proc_orient = atof(tmpbuf);

      /*  Sensor data  */
strncpy(tmpbuf,&buf[460],16);
tmpbuf[16] = '\0';
mpr->inclination = atof(tmpbuf);

strncpy(tmpbuf,&buf[476],16);
tmpbuf[16] = '\0';
mpr->ascend_node = atof(tmpbuf);

strncpy(tmpbuf,&buf[492],16);
tmpbuf[16] = '\0';
mpr->altitude = atof(tmpbuf);

strncpy(tmpbuf,&buf[508],16);
tmpbuf[16] = '\0';
mpr->ground_speed = atof(tmpbuf);

strncpy(tmpbuf,&buf[524],16);
tmpbuf[16] = '\0';
mpr->sat_heading = atof(tmpbuf);

strncpy(tmpbuf,&buf[556],16);
tmpbuf[16] = '\0';
mpr->cross_track_fov = atof(tmpbuf);

strncpy(tmpbuf,&buf[572],16);
tmpbuf[16] = '\0';
mpr->scan_rate = atof(tmpbuf);

strncpy(tmpbuf,&buf[588],16);
tmpbuf[16] = '\0';
mpr->samp_rate = atof(tmpbuf);

strncpy(tmpbuf,&buf[604],16);
tmpbuf[16] = '\0';
mpr->sun_elev = atof(tmpbuf);

strncpy(tmpbuf,&buf[620],16);
tmpbuf[16] = '\0';
mpr->sun_azimuth = atof(tmpbuf);

for (i=0; i<2; i++)
   {
   strncpy(tmpbuf,&buf[636+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->utm_corner_1[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[668+(i*16)],16);
   tmpbuf[16] = '\0'; 
   mpr->utm_corner_2[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[700+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->utm_corner_3[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[732+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->utm_corner_4[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[764+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->lat_long_corner1[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[796+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->lat_long_corner2[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[828+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->lat_long_corner3[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[860+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->lat_long_corner4[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[892+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->pix_line_corner1[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[924+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->pix_line_corner2[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[956+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->pix_line_corner3[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[988+(i*16)],16);
   tmpbuf[16] = '\0';
   mpr->pix_line_corner4[i] = atof(tmpbuf);
   } 

return(E_SUCC);
}                   /* end of c_ltwg_mpr */
