/***************************************************************************
NAME:				C_LTWG_MTR

PURPOSE:  Reads LTWG mission telemetry ancillary record from a file that 
	  is passed in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG MISSION TELEMETRY ANCILLARY RECORD 
----------------------------------------------------------*/
int c_ltwg_mtr
(
    struct LTWGMTR *mtr, /* OUTPUT - LTWG mission telemetry anc rec structure */
    FILE *fin,           /* INPUT - File pointer				*/
    long *size           /* INPUT - record size in bytes			*/
)
{

char *buf;          /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long i, j, k;		/* counting variables			*/
long nonfatal = 1;	

buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,*size,fin);

/*  read LTWG mission telemetry ancillary record
    --------------------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
mtr->rec_number = get_long(temp);

mtr->frst_rec_code = (long)buf[4];

mtr->rec_code = (long)buf[5];

mtr->scnd_rec_code = (long)buf[6];

mtr->thrd_rec_code = (long)buf[7];

mtr->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(tmpbuf,&buf[12],4);
tmpbuf[4] = '\0';
mtr->rec_seq_num = atol(tmpbuf);

    /* TELEMETRY MAJOR FRAME START TIME */
strncpy(tmpbuf,&buf[20],4);
tmpbuf[4] = '\0';
mtr->day_start = atol(tmpbuf);

strncpy(tmpbuf,&buf[24],8);
tmpbuf[8] = '\0';
mtr->millisec_start = atol(tmpbuf);

strncpy(tmpbuf,&buf[32],3);
tmpbuf[3] = '\0';
mtr->microsec_start = atol(tmpbuf);

strncpy(tmpbuf,&buf[35],1);
tmpbuf[1]='\0';
mtr->tenths_microsec_start = atol(tmpbuf);  

    /* EPHEMERIS DATA */
strncpy(tmpbuf,&buf[36],10);
tmpbuf[10] = '\0';
mtr->eph_time_offset = atol(tmpbuf);

for (i=0; i<3; i++)
   {
   strncpy(tmpbuf,&buf[46+(i*22)],22);
   tmpbuf[22] = '\0';
   mtr->craft_pos[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[112+(i*22)],22);
   tmpbuf[22] = '\0';
   mtr->craft_vel[i] = atof(tmpbuf);
   }
    /* ATTIDTUDE DATA */
strncpy(tmpbuf,&buf[178],10);
tmpbuf[10] = '\0';
mtr->att_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[188],22);
tmpbuf[22] = '\0';
mtr->epa1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[210],22);
tmpbuf[22] = '\0';
mtr->epa2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[232],22);
tmpbuf[22] = '\0';
mtr->epa3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[254],22);
tmpbuf[22] = '\0';
mtr->epa4 = atof(tmpbuf);

    /* ROLL, PITCH, YAW AXIS ANGULAR INCREMENTS */
strncpy(tmpbuf,&buf[276],10);
tmpbuf[10] = '\0';
mtr->inc_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[286],22);
tmpbuf[22] = '\0';
mtr->roll_inc = atof(tmpbuf);

strncpy(tmpbuf,&buf[308],22);
tmpbuf[22] = '\0';
mtr->pitch_inc = atof(tmpbuf);

strncpy(tmpbuf,&buf[330],22);
tmpbuf[22] = '\0';
mtr->yaw_inc = atof(tmpbuf);

    /* ROLL, PITCH, YAW ANGULAR RATE */
strncpy(tmpbuf,&buf[352],10);
tmpbuf[10] = '\0';
mtr->rate_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[362],22);
tmpbuf[22] = '\0';
mtr->roll_rate = atof(tmpbuf);

strncpy(tmpbuf,&buf[384],22);
tmpbuf[22] = '\0';
mtr->pitch_rate = atof(tmpbuf);

strncpy(tmpbuf,&buf[406],22);
tmpbuf[22] = '\0';
mtr->yaw_rate = atof(tmpbuf);

    /* ROLL, PITCH, YAW ATTITUDE ERROR */
strncpy(tmpbuf,&buf[428],10);
tmpbuf[10] = '\0';
mtr->err_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[438],22);
tmpbuf[22] = '\0';
mtr->roll_err = atof(tmpbuf);

strncpy(tmpbuf,&buf[460],22);
tmpbuf[22] = '\0';
mtr->pitch_err = atof(tmpbuf);

strncpy(tmpbuf,&buf[482],22);
tmpbuf[22] = '\0';
mtr->yaw_err = atof(tmpbuf);

    /* HOUSEKEEPING DATA SAMPLES */
for (i=0; i<6; i++)
   {
   strncpy(tmpbuf,&buf[526+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item1_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[598+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item2_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[670+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item3_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[742+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item4_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[814+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item5_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[886+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item6_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[958+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item7_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[1030+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item8_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[1102+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item9_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[1174+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item10_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[1246+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item11_coef[i] = atof(tmpbuf);

   strncpy(tmpbuf,&buf[1318+(i*12)],4);
   tmpbuf[4] = '\0';
   mtr->item12_coef[i] = atof(tmpbuf);
   }
for (i=0; i<128; i++)
   for (j=0; j<12; j++)
      {
      strncpy(tmpbuf,&buf[1390+(i*12)+j],1);
      tmpbuf[1]='\0';
      mtr->samples[j][i] = atol(tmpbuf);
      }

free (buf);

return(E_SUCC);
}                   /* end of c_ltwg_mtr */
