/***************************************************************************
NAME:				C_LTWG_RCR

PURPOSE:  Reads LTWG Radiometric calibration ancillary record from a file 
          that is passed in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG RADIOMETRIC CALIBRATION ANCILLARY RECORD 
----------------------------------------------------------------*/
int c_ltwg_rcr
(
    struct LTWGRCR *rcr, /* OUTPUT - LTWG radiometric calibration rec struct */
    FILE *fin,           /* INPUT - File pointer				*/
    long *size           /* INPUT - Record size in bytes			*/
)
{

char *buf;          /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long i,j,k;		/* counter variables			*/
long nonfatal = 1;

buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,*size,fin);

/*  read LTWG radiometric calibration ancillary record 
    --------------------------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
rcr->rec_number = get_long(temp);

rcr->frst_rec_code = (long)buf[4];

rcr->rec_code = (long)buf[5];

rcr->scnd_rec_code = (long)buf[6];

rcr->thrd_rec_code = (long)buf[7];

rcr->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(tmpbuf,&buf[12],4);
tmpbuf[4] = '\0';
rcr->band_number = atol(tmpbuf);

strncpy(tmpbuf,&buf[16],4);
tmpbuf[4] = '\0';
rcr->reflectance_lower = atol(tmpbuf);

strncpy(tmpbuf,&buf[20],4);
tmpbuf[4] = '\0';
rcr->reflectance_upper = atol(tmpbuf);

strncpy(tmpbuf,&buf[28],20);
tmpbuf[20] = '\0';
rcr->a0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[48],20);
tmpbuf[20] = '\0';
rcr->a1 = atof(tmpbuf);

for (i=0; i<16; i++)
   for (j=0; j<256; j++)
      {
      rcr->detector_data[i][j] = (long)buf[68+(1*256)+j];
      }

free (buf);

return(E_SUCC);
}                   /* end of c_ltwg_rcr */
