/***************************************************************************
NAME:				C_LTWG_RJR

PURPOSE:  Reads LTWG raw jitter measurement ancillary record from a file 
          that is passed in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG RAW JITTER MEASUREMENTS ANCILLARY RECORD
----------------------------------------------------------------*/
int c_ltwg_rjr
(
    struct LTWGRJR *rjr,    /* OUTPUT - LTWG raw jitter meas rec structure	*/
    FILE *fin,              /* INPUT - File pointer				*/
    long *size              /* INPUT - record size in bytes			*/
)
{

char *buf;          /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long nonfatal = 1;	
long i,j,k;


buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,*size,fin);

/*  read LTWG raw jitter measurement ancillary record
    -------------------------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
rjr->rec_number = get_long(temp);

rjr->frst_rec_code = (long)buf[4];

rjr->rec_code = (long)buf[5];

rjr->scnd_rec_code = (long)buf[6];

rjr->thrd_rec_code = (long)buf[7];

rjr->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(tmpbuf,&buf[12],4);
tmpbuf[4] = '\0';
rjr->rec_seq_num = atol(tmpbuf);

strncpy(tmpbuf,&buf[16],4);
tmpbuf[4] = '\0';
rjr->seq_num = atol(tmpbuf);

strncpy(tmpbuf,&buf[20],4);
tmpbuf[4] = '\0';
rjr->day_start = atol(tmpbuf);

strncpy(tmpbuf,&buf[24],8);
tmpbuf[8] = '\0';
rjr->millisec_start = atol(tmpbuf);

strncpy(tmpbuf,&buf[32],3);
tmpbuf[3] = '\0';
rjr->microsec_start = atol(tmpbuf); 

strncpy(tmpbuf,&buf[35],1);
tmpbuf[1]='\0';
rjr->tenths_microsec_start = atol(tmpbuf);  

    /* ADS TEMPERATURE DATA */
strncpy(tmpbuf,&buf[36],10);
tmpbuf[10] = '\0';
rjr->ads_temp1_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[46],12);
tmpbuf[12] = '\0';
rjr->ads_temp1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[58],10);
tmpbuf[10] = '\0';
rjr->ads_temp2_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[68],12);
tmpbuf[12] = '\0';
rjr->ads_temp2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[80],10);
tmpbuf[10] = '\0';
rjr->ads_temp3_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[90],12);
tmpbuf[12] = '\0';
rjr->ads_temp3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[102],10);
tmpbuf[10] = '\0';
rjr->ads_temp4_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[112],12);
tmpbuf[12] = '\0';
rjr->ads_temp4 = atof(tmpbuf);

    /* ADS MEASUREMENTS */
for (i=0; i<3; i++)
   {
   strncpy(tmpbuf,&buf[124+(i*10)],10);
   tmpbuf[10] = '\0';
   rjr->ads_measur1[i] = atol(tmpbuf);
   }

temp[0]=temp[1]=0;
for (i=0; i<1024; i++)
   {
   for (j=0; j<3; j++) 
      {
      for (k=2; k<4; k++)
         temp[k] = buf[154+(i*6)+(j*2)+k-2]; 
      rjr->ads_measur[j][i] = get_long(temp);
      }
   }

free (buf);

return(E_SUCC);
}                   /* end of c_ltwg_rjr */
