/***************************************************************************
NAME:				C_LTWG_SHR

PURPOSE:  Reads LTWG scene header record from a file that is passed
          in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG SCENE HEADER RECORD 
-------------------------------------------*/
int c_ltwg_shr
(
    struct  LTWGSHR *shr,   /* OUTPUT - LTWG volume descriptor rec structure */
    FILE    *fin,           /* INPUT - File pointer				*/
    long    *size           /* INPUT - number of bytes to read in for file	*/
)
{
long  i,j;		/* Counter variables			*/
char *buf;     /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long k;
long nonfatal = 1;

buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,*size,fin);

/*  read LTWG scene header record 
    -----------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
shr->rec_number = get_long(temp);

shr->frst_rec_code = (long)buf[4];

shr->rec_code = (long)buf[5];

shr->scnd_rec_code = (long)buf[6];

shr->thrd_rec_code = (long)buf[7];

shr->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(tmpbuf,&buf[12],4);
tmpbuf[4] = '\0';
shr->header_seq_num = atol(tmpbuf);

strncpy(shr->prod_type,&buf[20],16);
shr->prod_type[16] = '\0';

strncpy(shr->in_scene_id,&buf[36],16);
shr->in_scene_id[16] = '\0';

strncpy(tmpbuf,&buf[52],16);
tmpbuf[16] = '\0';
shr->in_scene_centre_lat = atof(tmpbuf);

strncpy(tmpbuf,&buf[68],16);
tmpbuf[16] = '\0';
shr->in_scene_centre_lon = atof(tmpbuf);

strncpy(tmpbuf,&buf[84],16);
tmpbuf[16] = '\0';
shr->in_line_num_cent = atof(tmpbuf);

strncpy(tmpbuf,&buf[100],16);
tmpbuf[16] = '\0';
shr->in_pixel_num_cent = atof(tmpbuf);

strncpy(shr->time_centre,&buf[116],32);
shr->time_centre[32] = '\0';

strncpy(tmpbuf,&buf[148],16);
tmpbuf[16] = '\0';
shr->time_offset = atof(tmpbuf);

strncpy(shr->wrs_design,&buf[164],16);
shr->wrs_design[16] = '\0';

strncpy(shr->wrs_cycle,&buf[180],16);
shr->wrs_cycle[16] = '\0';

strncpy(shr->pro_scene_id,&buf[196],16);
shr->pro_scene_id[16] = '\0';

strncpy(tmpbuf,&buf[212],16);
tmpbuf[16] = '\0';
shr->pro_scene_centre_lat = atof(tmpbuf);

strncpy(tmpbuf,&buf[228],16);
tmpbuf[16] = '\0';
shr->pro_scene_centre_lon = atof(tmpbuf);

strncpy(tmpbuf,&buf[244],16);
tmpbuf[16] = '\0';
shr->pro_line_num_cent = atof(tmpbuf);

strncpy(tmpbuf,&buf[260],16);
tmpbuf[16] = '\0';
shr->pro_pixel_num_cent = atof(tmpbuf);

strncpy(tmpbuf,&buf[276],16);
tmpbuf[16] = '\0';
shr->ovrlap_lines = atol(tmpbuf);

strncpy(tmpbuf,&buf[292],16);
tmpbuf[16] = '\0';
shr->ovrlap_pixels = atol(tmpbuf);

      /* MISSION PARAMETERS */
strncpy(shr->miss_id,&buf[308],16);
shr->miss_id[16] = '\0';

strncpy(shr->sensor_id,&buf[324],16);
shr->sensor_id[16] = '\0';

strncpy(tmpbuf,&buf[340],16);
tmpbuf[16] = '\0';
shr->orbit_num = atol(tmpbuf);

strncpy(shr->asc_des_flag,&buf[356],16);
shr->asc_des_flag[16] = '\0';

      /* SENSOR PARAMETERS */
for(i = 0; i < 64; i++)
   {
   for(j = 0; j < 2; j++)
      {
      strncpy(tmpbuf,&buf[388+(i*16)+(j*8)],8);
      tmpbuf[8] = '\0';
      shr->wavelgth_range[i][j] = atol(tmpbuf);
      }
   }
strncpy(tmpbuf,&buf[1412],16);
tmpbuf[16] = '\0';
shr->num_bands = atol(tmpbuf);

strncpy(tmpbuf,&buf[1428],16);
tmpbuf[16] = '\0';
shr->pixels_per_line = atol(tmpbuf);

strncpy(tmpbuf,&buf[1444],16);
tmpbuf[16] = '\0';
shr->num_lines = atol(tmpbuf);

      /* PROCESSING PARAMETERS */
strncpy(shr->rad_cal_des,&buf[1476],16);
shr->rad_cal_des[16] = '\0';

strncpy(tmpbuf,&buf[1492],16);
tmpbuf[16] = '\0';
shr->rad_res_des = atol(tmpbuf);

strncpy(shr->scenic_rad_des,&buf[1508],16);
shr->scenic_rad_des[16] = '\0';

strncpy(shr->geom_corr_des,&buf[1524],16);
shr->geom_corr_des[16] = '\0';

strncpy(shr->resamp_des,&buf[1540],16);
shr->resamp_des[16] = '\0';

strncpy(shr->map_proj_id,&buf[1556],16);
shr->map_proj_id[16] = '\0';

strncpy(tmpbuf,&buf[1572],16);
tmpbuf[16] = '\0';
shr->process_level = atol(tmpbuf);

strncpy(tmpbuf,&buf[1588],16);
tmpbuf[16] = '\0';
shr->num_map_proj_anc = atol(tmpbuf);

strncpy(shr->replac_detect,&buf[1604],16);
shr->replac_detect[16] = '\0';

strncpy(shr->kernal_for_replac,&buf[1620],16);
shr->kernal_for_replac[16] = '\0';

strncpy(tmpbuf,&buf[1636],16);
tmpbuf[16] = '\0';
shr->num_rad_ancill = atol(tmpbuf);

strncpy(shr->active_bands,&buf[1652],64);
shr->active_bands[64] = '\0';

strncpy(shr->interleave_ind,&buf[1716],16);
shr->interleave_ind[16] = '\0';

for (i=0; i<100; i++)
   {
   strncpy(tmpbuf,&buf[1732 + (i*4)],4);
   tmpbuf[4] = '\0';
   shr->detector_subst[i] = atol(tmpbuf);
   }
strncpy(shr->detector_smooth,&buf[2132],100);
shr->detector_smooth[100] = '\0';

for (i=0; i<12; i++)
   {
   strncpy(tmpbuf,&buf[2232 + (i*16)],16);
   tmpbuf[16] = '\0';
   shr->mirror_scan_vel[i] = atof(tmpbuf);
   }
for (i=0; i<4; i++)
   {
   for (j=0; j<16; j++)
      {
       strncpy(tmpbuf,&buf[2424 + (i*64) + (j*4)],4);
       tmpbuf[4] = '\0';
       shr->dect_adjust[i][j] = atol(tmpbuf);
      }
   }

free (buf);

return(E_SUCC);
}                   /* end of c_ltwg_shr */
