/***************************************************************************
NAME:				C_LTWG_SIH

PURPOSE:  Reads LTWG supplemental interval header record from a file that is 
          passed in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG SUPPLEMENTAL INTERVAL HEADER RECORD 
-----------------------------------------------------------*/
int c_ltwg_sih
(
    struct LTWGSIH *sih,    /* OUTPUT - LTWG interval header rec structure	*/
    FILE   *fin,            /* INPUT - File pointer				*/
    long   *size            /* INPUT - record size in bytes			*/
)
{

char *buf;     /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long k;
long nonfatal = 1;

buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,*size,fin);

/*  read LTWG supplemental interval header record
    ---------------------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
sih->rec_number = get_long(temp);

sih->frst_rec_code = (long)buf[4];

sih->rec_code = (long)buf[5];

sih->scnd_rec_code = (long)buf[6];

sih->thrd_rec_code = (long)buf[7];

sih->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(tmpbuf,&buf[12],4);
tmpbuf[4] = '\0';
sih->header_num = atol(tmpbuf);

strncpy(sih->input_start_time,&buf[24],16);
sih->input_start_time[16] = '\0';

strncpy(sih->input_stop_time,&buf[40],16);
sih->input_stop_time[16] = '\0';

strncpy(sih->pcd_start_time,&buf[56],16);
sih->pcd_start_time[16] = '\0';

strncpy(sih->pcd_stop_time,&buf[72],16);
sih->pcd_stop_time[16] = '\0';

strncpy(tmpbuf,&buf[88],4);
tmpbuf[4] = '\0';
sih->pcd_frames = atol(tmpbuf);

strncpy(tmpbuf,&buf[92],8);
tmpbuf[8] = '\0';
sih->orbit_num = atol(tmpbuf);

free (buf);

return(E_SUCC);
}                   /* end of c_ltwg_sih */
