/***************************************************************************
NAME:				C_LTWG_TMHK

PURPOSE:  Reads LTWG TM house keeping record from a file that is passed
          in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG TM HOUSE KEEPING RECORD
-----------------------------------------------*/
int c_ltwg_tmhk
(
    struct LTWGTMHK *tmhk, /* OUTPUT - LTWG TM house keeping rec structure	*/
    FILE   *fin,           /* INPUT - File pointer				*/
    long   *size           /* INPUT - record size in bytes			*/
)
{

char *buf;          /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long k;
long nonfatal = 1;

buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,*size,fin);

/*  read LTWG TM house keeping record
    ---------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
tmhk->rec_number = get_long(temp);

tmhk->frst_rec_code = (long)buf[4];

tmhk->rec_code = (long)buf[5];

tmhk->scnd_rec_code = (long)buf[6];

tmhk->thrd_rec_code = (long)buf[7];

tmhk->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(tmpbuf,&buf[12],4);
tmpbuf[4] = '\0';
tmhk->rec_num = atol(tmpbuf);

strncpy(tmpbuf,&buf[16],4);
tmpbuf[4] = '\0';
tmhk->pcd_ident = atol(tmpbuf);

      /*   PCD MAJOR FRAME START TIME    */
strncpy(tmpbuf,&buf[20],4);
tmpbuf[4] = '\0';
tmhk->pcd_start_day = atol(tmpbuf);

strncpy(tmpbuf,&buf[24],8);
tmpbuf[8] = '\0';
tmhk->pcd_start_msec = atol(tmpbuf);

strncpy(tmpbuf,&buf[32],3);
tmpbuf[3] = '\0';
tmhk->pcd_start_usec = atol(tmpbuf);

strncpy(tmpbuf,&buf[35],1);
tmpbuf[1]='\0';
tmhk->pcd_start_tenths = atol(tmpbuf);

      /*   TM HOUSEKEEPING DATA    */
  /* Blackbody temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[36],10);
tmpbuf[10] = '\0';
tmhk->bb_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[46],4);
tmpbuf[4] = '\0';
tmhk->bb_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[50],12);
tmpbuf[12] = '\0';
tmhk->bb_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[62],12);
tmpbuf[12] = '\0';
tmhk->bb_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[74],12);
tmpbuf[12] = '\0';
tmhk->bb_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[86],12);
tmpbuf[12] = '\0';
tmhk->bb_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[98],12);
tmpbuf[12] = '\0';
tmhk->bb_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[110],12);
tmpbuf[12] = '\0';
tmhk->bb_A5 = atof(tmpbuf);

  /* Focal plane temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[122],10);
tmpbuf[10] = '\0';
tmhk->ff_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[132],4);
tmpbuf[4] = '\0';
tmhk->ff_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[136],12);
tmpbuf[12] = '\0';
tmhk->ff_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[148],12);
tmpbuf[12] = '\0';
tmhk->ff_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[160],12);
tmpbuf[12] = '\0';
tmhk->ff_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[172],12);
tmpbuf[12] = '\0';
tmhk->ff_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[184],12);
tmpbuf[12] = '\0';
tmhk->ff_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[196],12);
tmpbuf[12] = '\0';
tmhk->ff_A5 = atof(tmpbuf);

  /* Calib shutter temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[208],10);
tmpbuf[10] = '\0';
tmhk->st_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[218],4);
tmpbuf[4] = '\0';
tmhk->st_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[222],12);
tmpbuf[12] = '\0';
tmhk->st_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[234],12);
tmpbuf[12] = '\0';
tmhk->st_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[246],12);
tmpbuf[12] = '\0';
tmhk->st_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[258],12);
tmpbuf[12] = '\0';
tmhk->st_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[260],12);
tmpbuf[12] = '\0';
tmhk->st_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[272],12);
tmpbuf[12] = '\0';
tmhk->st_A5 = atof(tmpbuf);

  /* Baffle temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[380],10);
tmpbuf[10] = '\0';
tmhk->bt_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[390],4);
tmpbuf[4] = '\0';
tmhk->bt_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[394],12);
tmpbuf[12] = '\0';
tmhk->bt_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[406],12);
tmpbuf[12] = '\0';
tmhk->bt_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[418],12);
tmpbuf[12] = '\0';
tmhk->bt_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[430],12);
tmpbuf[12] = '\0';
tmhk->bt_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[442],12);
tmpbuf[12] = '\0';
tmhk->bt_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[454],12);
tmpbuf[12] = '\0';
tmhk->bt_A5 = atof(tmpbuf);

  /* Cold stage temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[466],10);
tmpbuf[10] = '\0';
tmhk->cs_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[476],4);
tmpbuf[4] = '\0';
tmhk->cs_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[480],12);
tmpbuf[12] = '\0';
tmhk->cs_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[492],12);
tmpbuf[12] = '\0';
tmhk->cs_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[504],12);
tmpbuf[12] = '\0';
tmhk->cs_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[516],12);
tmpbuf[12] = '\0';
tmhk->cs_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[528],12);
tmpbuf[12] = '\0';
tmhk->cs_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[540],12);
tmpbuf[12] = '\0';
tmhk->cs_A5 = atof(tmpbuf);

  /* Scan line corrector temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[724],10);
tmpbuf[10] = '\0';
tmhk->lc_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[734],4);
tmpbuf[4] = '\0';
tmhk->lc_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[738],12);
tmpbuf[12] = '\0';
tmhk->lc_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[750],12);
tmpbuf[12] = '\0';
tmhk->lc_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[762],12);
tmpbuf[12] = '\0';
tmhk->lc_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[774],12);
tmpbuf[12] = '\0';
tmhk->lc_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[786],12);
tmpbuf[12] = '\0';
tmhk->lc_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[798],12);
tmpbuf[12] = '\0';
tmhk->lc_A5 = atof(tmpbuf);

  /* Shutter hub temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[810],10);
tmpbuf[10] = '\0';
tmhk->sh_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[820],4);
tmpbuf[4] = '\0';
tmhk->sh_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[824],12);
tmpbuf[12] = '\0';
tmhk->sh_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[836],12);
tmpbuf[12] = '\0';
tmhk->sh_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[848],12);
tmpbuf[12] = '\0';
tmhk->sh_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[860],12);
tmpbuf[12] = '\0';
tmhk->sh_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[872],12);
tmpbuf[12] = '\0';
tmhk->sh_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[884],12);
tmpbuf[12] = '\0';
tmhk->sh_A5 = atof(tmpbuf);

  /* Relay optics temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[1068],10);
tmpbuf[10] = '\0';
tmhk->ro_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[1078],4);
tmpbuf[4] = '\0';
tmhk->ro_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[1082],12);
tmpbuf[12] = '\0';
tmhk->ro_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1094],12);
tmpbuf[12] = '\0';
tmhk->ro_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1106],12);
tmpbuf[12] = '\0';
tmhk->ro_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1118],12);
tmpbuf[12] = '\0';
tmhk->ro_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1130],12);
tmpbuf[12] = '\0';
tmhk->ro_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1142],12);
tmpbuf[12] = '\0';
tmhk->ro_A5 = atof(tmpbuf);

  /* Unpacked as serial word B  */
strncpy(tmpbuf,&buf[1412],10);
tmpbuf[10] = '\0';
tmhk->wb_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[1422],4);
tmpbuf[4] = '\0';
tmhk->wb_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[1426],12);
tmpbuf[12] = '\0';
tmhk->wb_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1438],12);
tmpbuf[12] = '\0';
tmhk->wb_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1450],12);
tmpbuf[12] = '\0';
tmhk->wb_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1462],12);
tmpbuf[12] = '\0';
tmhk->wb_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1474],12);
tmpbuf[12] = '\0';
tmhk->wb_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1586],12);
tmpbuf[12] = '\0';
tmhk->wb_A5 = atof(tmpbuf);

  /* Unpacked as serial word D  */
strncpy(tmpbuf,&buf[1584],10);
tmpbuf[10] = '\0';
tmhk->wd_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[1594],4);
tmpbuf[4] = '\0';
tmhk->wd_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[1598],12);
tmpbuf[12] = '\0';
tmhk->wd_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1610],12);
tmpbuf[12] = '\0';
tmhk->wd_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1622],12);
tmpbuf[12] = '\0';
tmhk->wd_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1634],12);
tmpbuf[12] = '\0';
tmhk->wd_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1646],12);
tmpbuf[12] = '\0';
tmhk->wd_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1658],12);
tmpbuf[12] = '\0';
tmhk->wd_A5 = atof(tmpbuf);

  /* Unpacked as serial word E  */
strncpy(tmpbuf,&buf[1670],10);
tmpbuf[10] = '\0';
tmhk->we_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[1680],4);
tmpbuf[4] = '\0';
tmhk->we_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[1684],12);
tmpbuf[12] = '\0';
tmhk->we_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1696],12);
tmpbuf[12] = '\0';
tmhk->we_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1708],12);
tmpbuf[12] = '\0';
tmhk->we_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1720],12);
tmpbuf[12] = '\0';
tmhk->we_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1732],12);
tmpbuf[12] = '\0';
tmhk->we_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1744],12);
tmpbuf[12] = '\0';
tmhk->we_A5 = atof(tmpbuf);

  /* Unpacked as serial word F  */
strncpy(tmpbuf,&buf[1756],10);
tmpbuf[10] = '\0';
tmhk->wf_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[1766],4);
tmpbuf[4] = '\0';
tmhk->wf_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[1770],12);
tmpbuf[12] = '\0';
tmhk->wf_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1782],12);
tmpbuf[12] = '\0';
tmhk->wf_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1794],12);
tmpbuf[12] = '\0';
tmhk->wf_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1806],12);
tmpbuf[12] = '\0';
tmhk->wf_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1818],12);
tmpbuf[12] = '\0';
tmhk->wf_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1830],12);
tmpbuf[12] = '\0';
tmhk->wf_A5 = atof(tmpbuf);

  /* Unpacked as serial word G  */
strncpy(tmpbuf,&buf[1842],10);
tmpbuf[10] = '\0';
tmhk->wg_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[1852],4);
tmpbuf[4] = '\0';
tmhk->wg_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[1856],12);
tmpbuf[12] = '\0';
tmhk->wg_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1868],12);
tmpbuf[12] = '\0';
tmhk->wg_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1880],12);
tmpbuf[12] = '\0';
tmhk->wg_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1892],12);
tmpbuf[12] = '\0';
tmhk->wg_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1904],12);
tmpbuf[12] = '\0';
tmhk->wg_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[1916],12);
tmpbuf[12] = '\0';
tmhk->wg_A5 = atof(tmpbuf);

  /* Unpacked as serial word L  */
strncpy(tmpbuf,&buf[2014],10);
tmpbuf[10] = '\0';
tmhk->wl_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[2024],4);
tmpbuf[4] = '\0';
tmhk->wl_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[2028],12);
tmpbuf[12] = '\0';
tmhk->wl_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2040],12);
tmpbuf[12] = '\0';
tmhk->wl_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2052],12);
tmpbuf[12] = '\0';
tmhk->wl_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2064],12);
tmpbuf[12] = '\0';
tmhk->wl_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2076],12);
tmpbuf[12] = '\0';
tmhk->wl_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2088],12);
tmpbuf[12] = '\0';
tmhk->wl_A5 = atof(tmpbuf);

  /* Primary mirror temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[2100],10);
tmpbuf[10] = '\0';
tmhk->pm_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[2110],4);
tmpbuf[4] = '\0';
tmhk->pm_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[2114],12);
tmpbuf[12] = '\0';
tmhk->pm_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2126],12);
tmpbuf[12] = '\0';
tmhk->pm_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2138],12);
tmpbuf[12] = '\0';
tmhk->pm_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2150],12);
tmpbuf[12] = '\0';
tmhk->pm_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2162],12);
tmpbuf[12] = '\0';
tmhk->pm_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2174],12);
tmpbuf[12] = '\0';
tmhk->pm_A5 = atof(tmpbuf);

  /* Secondary mirror temperature sample and conversion to Degrees C */
strncpy(tmpbuf,&buf[2272],10);
tmpbuf[10] = '\0';
tmhk->sm_time_offset = atol(tmpbuf);

strncpy(tmpbuf,&buf[2282],4);
tmpbuf[4] = '\0';
tmhk->sm_hk_sample = atol(tmpbuf);

strncpy(tmpbuf,&buf[2286],12);
tmpbuf[12] = '\0';
tmhk->sm_A0 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2298],12);
tmpbuf[12] = '\0';
tmhk->sm_A1 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2310],12);
tmpbuf[12] = '\0';
tmhk->sm_A2 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2322],12);
tmpbuf[12] = '\0';
tmhk->sm_A3 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2334],12);
tmpbuf[12] = '\0';
tmhk->sm_A4 = atof(tmpbuf);

strncpy(tmpbuf,&buf[2346],12);
tmpbuf[12] = '\0';
tmhk->sm_A5 = atof(tmpbuf);

strncpy(tmhk->ser_wd_b,&buf[2788],8);
tmhk->ser_wd_b[8] = '\0';

strncpy(tmhk->ser_wd_d,&buf[2796],8);
tmhk->ser_wd_d[8] = '\0';

strncpy(tmhk->ser_wd_e,&buf[2804],8);
tmhk->ser_wd_e[8] = '\0';

strncpy(tmhk->ser_wd_f,&buf[2812],8);
tmhk->ser_wd_f[8] = '\0';

strncpy(tmhk->ser_wd_g,&buf[2820],8);
tmhk->ser_wd_g[8] = '\0';

strncpy(tmhk->ser_wd_l,&buf[2828],8);
tmhk->ser_wd_l[8] = '\0';

free (buf);

return(E_SUCC);
}                   /* end of c_ltwg_tmhk */
