/***************************************************************************
NAME:				C_LTWG_TR 

PURPOSE:  Reads LTWG trailer record from a file that is passed
          in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG TRAILER RECORD 
--------------------------------------*/
int c_ltwg_tr
(
    struct LTWGTR *tr,  /* OUTPUT - LTWG trailer rec structure		*/
    FILE *fin,          /* INIPUT - File pointer			*/
    long *size          /* INIPUT - record size in bytes		*/
)
{

char *buf;          /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long i,j,k;		/* counting variables			*/
long nonfatal = 1;

buf = (char *)malloc(*size + 1);
if (buf == NULL)
   {
   c_errmsg("Error allocating dynamic memory.","dspassoc-alloc",&nonfatal);
   return(E_FAIL);
   }

fread(buf,1,*size,fin);

/*  read LTWG trailer record
    ------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
tr->rec_number = get_long(temp);

tr->frst_rec_code = (long)buf[4];

tr->rec_code = (long)buf[5];

tr->scnd_rec_code = (long)buf[6];

tr->thrd_rec_code = (long)buf[7];

tr->rec_length = (long)buf[10]*256 + (long)buf[11];

strncpy(tmpbuf,&buf[12],4);
tmpbuf[4] = '\0';
tr->record_seq_num = atol(tmpbuf);

strncpy(tmpbuf,&buf[16],4);
tmpbuf[4] = '\0';
tr->seq_in_band = atol(tmpbuf);

for (i=0; i<4; i++)
   {
   for (j=0; j<256; j++)
      {
      for (k = 0; k < 4; k++)
         temp[k] = buf[20+(i*1024)+(j*4)+k]; 
      tr->detect_histo[i][j] = get_long(temp);
      }
   }

strncpy(tmpbuf,&buf[4116],5);
tmpbuf[5] = '\0';
tr->parity_count = atol(tmpbuf);

strncmp(tr->quality_summary,&buf[4121],200);
tr->quality_summary[200] = '\0';

free(buf);

return(E_SUCC);
}                   /* end of c_ltwg_tr */
