/***************************************************************************
NAME:				C_LTWG_VDR

PURPOSE:  Reads LTWG volume descriptor record from a file that is passed
 	  in already opened, and put it into a structure.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  6.0		 3/94   D. Bridges	Orginal Development

***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "ltwg.h"
#include "header.h"
#include "local_protos.h"

/* Routine to read LTWG VOLUME DESCRIPTOR RECORD 
------------------------------------------------*/
int c_ltwg_vdr
(
    struct LTWGVDR *vdr, /* OUTPUT - LTWG volume descriptor rec structure */
    FILE *fin            /* INPUT - File pointer				*/
)
{

char buf[361];      /* buffer for LTWG tape information	*/
char tmpbuf[50];	/* temporary buffer storage		*/
char temp[4];
long k;

fread(buf,1,360,fin);

/*  read LTWG volume descriptor record
    ----------------------------------*/
for(k=0;k<4;k++)
   temp[k]=buf[k];
vdr->rec_number = get_long(temp);

vdr->frst_rec_code = (long)buf[4];

vdr->rec_code = (long)buf[5];

vdr->scnd_rec_code = (long)buf[6];

vdr->thrd_rec_code = (long)buf[7];

vdr->rec_length = (long)buf[10]*256 + (long)buf[11]; 

strncpy(vdr->a_e_flag,&buf[12],2);
vdr->a_e_flag[2] = '\0';

strncpy(vdr->control_num,&buf[16],12);
vdr->control_num[12] = '\0';

strncpy(vdr->control_rev_num,&buf[28],2);
vdr->control_rev_num[2] = '\0';

strncpy(vdr->format_rev,&buf[30],2);
vdr->format_rev[2] = '\0'; 

strncpy(vdr->release_num,&buf[32],12);
vdr->release_num[12] = '\0';

strncpy(vdr->tape_id,&buf[44],16);
vdr->tape_id[16] = '\0';

strncpy(vdr->logical_vol_id,&buf[60],16);
vdr->logical_vol_id[16] = '\0';

strncpy(vdr->vol_set_id,&buf[76],16);
vdr->vol_set_id[16] = '\0';

strncpy(tmpbuf,&buf[92],2);
tmpbuf[2] = '\0';
vdr->num_phy_vol = atol(tmpbuf);

strncpy(tmpbuf,&buf[94],2);
tmpbuf[2] = '\0';
vdr->start_log_vol = atol(tmpbuf);

strncpy(tmpbuf,&buf[96],2);
tmpbuf[2] = '\0';
vdr->end_log_vol = atol(tmpbuf);

strncpy(tmpbuf,&buf[98],2);
tmpbuf[2] = '\0'; 
vdr->volume_num = atol(tmpbuf);

strncpy(tmpbuf,&buf[100],4);
tmpbuf[4] = '\0';
vdr->frst_ref_file = atol(tmpbuf);

strncpy(tmpbuf,&buf[104],4);
tmpbuf[4] = '\0';
vdr->log_vol_num_set = atol(tmpbuf);

strncpy(tmpbuf,&buf[108],4);
tmpbuf[4] = '\0';
vdr->log_vol_num_phys = atol(tmpbuf);

strncpy(vdr->log_creat_date,&buf[112],8);
vdr->log_creat_date[8] = '\0';

strncpy(vdr->log_creat_time,&buf[120],8);
vdr->log_creat_time[8] = '\0';

strncpy(vdr->log_gen_country,&buf[128],12);
vdr->log_gen_country[12] = '\0';

strncpy(vdr->log_gen_agency,&buf[140],8);
vdr->log_gen_agency[8] = '\0';

strncpy(vdr->log_gen_facility,&buf[148],8);
vdr->log_gen_facility[12] = '\0';

strncpy(tmpbuf,&buf[160],4);
tmpbuf[4] = '\0';
vdr->num_point_rec = atol(tmpbuf);

strncpy(tmpbuf,&buf[164],4);
tmpbuf[4] = '\0';
vdr->num_records = atol(tmpbuf);

return(E_SUCC);
}                   /* end of c_ltwg_vdr */
