/************************************
*  Common includes and definitions  *
************************************/
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "labserv.h"
#include "fhdr.h"
#include "header.h"

/*******************************************************************************
FUNCTION:	c_open_header_fst

PURPOSE: 	To read or write the header file if it exits.	

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
1.0	 T. Mittan	 11/92	 LAS   Original development.
7.2	 G. Schmidt	 04/99	 LAS   Updated to support Fast-L7A.  
7.2	 G. Schmidt	 07/99	 LAS   Updated to support Space Imaging Fast-C  

ALGORITHM REFERENCES:	none

NOTE: This is the same as the c_open_header routine, but it has been updated to
      work with Fast-L7A and Space Imaging Fast-C headers.
*******************************************************************************/

int c_open_header_fst(
    char  *buf,                 /* header buffer                        */
    char  *name,		/* image name				*/
    char  *hdr_ext,		/* header extension for the current img */
    long  *flag,		/* flag to determine read or write acc	*/
    long  hsize)                /* size of the header in bytes          */
    {
   FILE *fp;			/* file pointer				*/
   
   long status;			/* return status			*/
   char hostheader[CMLEN];	/* header name				*/
   long mode = 0;		/* flag value for lsstat		*/
   long one = 1;		/* constant value of one		*/
   
   if (*flag == IREAD)
   {
      c_lsmknm(name,hdr_ext,hostheader);  /* make associated header file name */
      status = (long) c_lsstat(hostheader,&mode);
           /* make sure header file exists    */
      if (status != E_SUCC)
      {
	 c_errmsg("HDR file does not exist on disk","informational",&one);
	 c_errmsg("Creating a template HDR","informational",&one);
	 template_header(buf);
	 return(E_SUCC);
      }
      
      fp = (fopen(hostheader,"r"));
      status = (long) fread(buf,1,hsize,fp);
      fclose(fp);
      if (status != hsize)
      {
	 c_errmsg("Error reading header","open_header-read",&one);
	 return(E_FAIL);
      }
   }
   else if (*flag == IWRITE)
   {
      c_lsmknm(name,hdr_ext,hostheader);/* make associated header file name */
      fp = (fopen(hostheader,"w"));
      status = (long) fwrite(buf,1,hsize,fp);
      fclose(fp);
      if (status != hsize)
      {
	 c_errmsg("Error writing header","open_header-write",&one);
	 return(E_FAIL);
      }
   }
   return(E_SUCC);
}
