/************************************
*  Common includes and definitions  *
************************************/
#include <stdio.h>
#include "las.h"
#include "tipsp.h"
#include "labserv.h"
#include "header.h"
#include "local_protos.h"

#define  HEADER_SIZE 3600 

/*******************************************************************************
FUNCTION:	c_read_annot	

PURPOSE: 	To read the annotation  file and fill the TIPSP structure.	

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
1.0	 T. Mittan	 12/92	 LAS	Original development.

ALGORITHM REFERENCES	none
*******************************************************************************/
int c_read_annot
(
    struct TIPSP *tipsp,		/* TIPSP sturcture declaration		*/
    char *name           		/* image name				*/
)
{
   FILE *fp;			/* file pointer				*/

   long  status;		/* return status			*/
   long  i;			/* array index				*/
   long  j;			/* array index				*/
   long  mode = 0;		/* flag value for lsstat		*/
   long  one = 1;		/* constant value of one		*/
   long bit;			/* first bit in tick mark		*/

   char  temp[2];		/* temporary variable(must start on even boundary */
   char  buf[3601];             /* header buffer                        */
   char  hostheader[CMLEN];	/* header name				*/


  c_lsmknm(name,"tipsp_annot",hostheader);
  status = c_lsstat(hostheader,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("ANB file does not exist on disk","read_annot-stat",&one);
     return(E_FAIL);
     }
  /* use data from band quality data record
  ---------------------------------------*/
  fp = (fopen(hostheader,"r"));
  if(!fp)
   {
   c_errmsg("Error opening Annotation file","read_annot-open",&one);
   return(E_FAIL);
   }

  fread(buf,1,HEADER_SIZE,fp);

  strncpy(tipsp->band_quality,&buf[12],1);
  tipsp->band_quality[1] = '\0';

  /* radiometric correction data
  ----------------------------*/
  tipsp->band_number_of_scenes = (long) swap_long(&buf[16]);
  tipsp->band_scene_sequence = (long) swap_long(&buf[20]);
  tipsp->band_number_radiometric_corrections = (long) swap_long(&buf[24]);
  tipsp->band_number_scans_per_rc = (long) swap_long(&buf[28]);
  tipsp->band_number_subsegments_per_rc = (long) swap_long(&buf[32]);
  strncpy(tipsp->band_calibration,&buf[36],1);
  tipsp->band_calibration[1] = '\0';
  strncpy(tipsp->band_selected_histogram_processing,&buf[37],1); 
  tipsp->band_selected_histogram_processing[1] = '\0';
  temp[0] = buf[39];
  temp[1] = buf[38];
  tipsp->band_number_hist_iterations = get_short(temp);
  tipsp->band_minimum_radiance = (float) swap_real(&buf[40]);
  tipsp->band_maximum_radiance = (float) swap_real(&buf[44]);
  strncpy(tipsp->band_lamp_mode,&buf[48],1);
  tipsp->band_lamp_mode[1] = '\0';
  strncpy(tipsp->band_calibration_lamp_use,&buf[49],2); 
  tipsp->band_calibration_lamp_use[2] = '\0';

  /* Mirror scan correction data
  ----------------------------*/
  tipsp->band_nominal_line_forward = (float) swap_real(&buf[76]);
  tipsp->band_nominal_line_backward = (float) swap_real(&buf[80]);
  tipsp->band_maximum_line_error_forward = (float) swap_real(&buf[84]);
  tipsp->band_maximum_line_error_backward = (float) swap_real(&buf[88]);
  tipsp->band_minimum_line_error_forward = (float) swap_real(&buf[92]);
  tipsp->band_minimum_line_error_backward = (float) swap_real(&buf[96]);
  tipsp->band_mean_line_error_forward = (float) swap_real(&buf[100]);
  tipsp->band_mean_line_error_backward = (float) swap_real(&buf[104]);
  tipsp->band_sigma_line_error_forward = (float) swap_real(&buf[108]);
  tipsp->band_sigma_line_error_backward = (float) swap_real(&buf[112]);
  tipsp->band_nominal_first_line_error_forward = (float) swap_real(&buf[116]);
  tipsp->band_nominal_first_line_error_backward = (float) swap_real(&buf[120]);
  tipsp->band_maximum_first_line_error_forward = (float) swap_real(&buf[124]);
  tipsp->band_maximum_first_line_error_backward = (float) swap_real(&buf[128]);
  tipsp->band_minimum_first_line_error_forward = (float) swap_real(&buf[132]);
  tipsp->band_minimum_first_line_error_backward = (float) swap_real(&buf[136]);
  tipsp->band_mean_first_line_error_forward = (float) swap_real(&buf[140]);
  tipsp->band_mean_first_line_error_backward = (float) swap_real(&buf[144]);
  tipsp->band_sigma_first_line_error_forward = (float) swap_real(&buf[148]);
  tipsp->band_sigma_first_line_error_backward = (float) swap_real(&buf[152]);
  tipsp->band_nom_second_line_error_forward = (float) swap_real(&buf[156]);
  tipsp->band_nom_second_line_error_backward = (float) swap_real(&buf[160]);
  tipsp->band_max_second_line_error_forward = (float) swap_real(&buf[164]);
  tipsp->band_max_second_line_error_backward = (float) swap_real(&buf[168]);
  tipsp->band_min_second_line_error_forward = (float) swap_real(&buf[172]);
  tipsp->band_min_second_line_error_backward = (float) swap_real(&buf[176]);
  tipsp->band_mean_second_line_error_forward = (float) swap_real(&buf[180]);
  tipsp->band_mean_second_line_error_backward = (float) swap_real(&buf[184]);
  tipsp->band_sigma_second_line_error_forward = (float) swap_real(&buf[188]);
  tipsp->band_sigma_second_line_error_backward = (float) swap_real(&buf[192]);
  tipsp->band_max_nonlin_line_error_forward = (float) swap_real(&buf[196]);
  tipsp->band_max_nonlin_line_error_backward = (float) swap_real(&buf[200]);
  tipsp->band_min_nonlin_line_error_forward = (float) swap_real(&buf[204]);
  tipsp->band_min_nonlin_line_error_backward = (float) swap_real(&buf[208]);
  tipsp->band_mean_nonlin_line_error_forward = (float) swap_real(&buf[212]);
  tipsp->band_mean_nonlin_line_error_backward = (float) swap_real(&buf[216]);
  tipsp->band_sigma_nonlin_line_error_forward = (float) swap_real(&buf[220]);
  tipsp->band_sigma_nonlin_line_error_backward = (float) swap_real(&buf[224]);

  /* Control point historical information
  -------------------------------------*/
  strncpy(tipsp->band_quality_codes,&buf[268],14); 
  tipsp->band_quality_codes[14] = '\0';
  tipsp->band_number_scences_in_cp = (float) swap_long(&buf[284]);
  tipsp->band_sequence_number_in_cp = (float) swap_long(&buf[288]);
  tipsp->band_number_geodetic_points_in_cp = (float) swap_long(&buf[292]);
  tipsp->band_number_geodetic_points_in_reference = (float)swap_long(&buf[296]);
  tipsp->band_average_autocorrelation = (float) swap_real(&buf[300]);
  tipsp->band_average_initial_peak = (float) swap_real(&buf[304]);
  strncpy(tipsp->band_scene_id,&buf[308],12); 
  tipsp->band_scene_id[12] = '\0';
  tipsp->band_elipse_along_track_interval_error = (float) swap_real(&buf[320]);
  tipsp->band_elipse_across_track_interval_error = (float) swap_real(&buf[324]);
  tipsp->band_elipse_along_track_ref_error = (float) swap_real(&buf[328]);
  tipsp->band_elipse_across_track_ref_error = (float) swap_real(&buf[332]);
  tipsp->band_elipse_average_success = (float) swap_real(&buf[336]);

  /* Current control point information
  ----------------------------------*/
  tipsp->band_number_scenes_interval = (long) swap_long(&buf[340]);
  tipsp->band_sequence_number_in_interval = (long) swap_long(&buf[344]);
  tipsp->band_number_cp_interval = (long) swap_long(&buf[348]);
  tipsp->band_number_cp_scene = (long) swap_long(&buf[352]);
  tipsp->band_number_cp_previous_scenes = (long) swap_long(&buf[356]);
  tipsp->band_number_geodetic_cp = (long) swap_long(&buf[360]);
  tipsp->band_number_cp_correlations_interval = (long) swap_long(&buf[364]);
  tipsp->band_number_cp_rejected_interval = (long) swap_long(&buf[368]);
  tipsp->band_number_cp_rejected_prefiltering_interval = 
						(long) swap_long(&buf[372]);
  tipsp->band_number_cp_rejected_filtering_interval = 
						(long) swap_long(&buf[376]);
  tipsp->band_number_cp_correlations_scene = (long) swap_long(&buf[380]);
  tipsp->band_number_cp_rejected_scene = (long) swap_long(&buf[384]);
  tipsp->band_number_cp_rejected_prefiltering_scene = 
						(long) swap_long(&buf[388]);
  tipsp->band_number_cp_rejected_filtering_scene = 
						(long) swap_long(&buf[392]);
  strncpy(tipsp->band_interval_correction,&buf[396],2); 
  tipsp->band_interval_correction[2] = '\0';
  strncpy(tipsp->band_scene_correction,&buf[398],2); 
  tipsp->band_scene_correction[2] = '\0';
  strncpy(tipsp->band_cp_band_mode,&buf[400],2); 
  tipsp->band_cp_band_mode[2] = '\0';
  strncpy(tipsp->band_cp_interval_mode,&buf[402],2); 
  tipsp->band_cp_interval_mode[2] = '\0';
  for (i = 0; i < 25; i++)
      {
      strncpy(tipsp->band_cp_id[i],&buf[404 + (i * 24)],15); 
      tipsp->band_cp_id[i][15] = '\0';
      tipsp->band_cp_line[i] = (float) swap_real(&buf[420 + (i * 24)]); 
      tipsp->band_cp_sample[i] = (float) swap_real(&buf[424 + (i * 24)]); 
      }
      
  tipsp->band_average_cp_peak = (float) swap_real(&buf[1004]); 
  tipsp->band_average_cp_curve = (float) swap_real(&buf[1008]); 

  /* geometric correction
  ---------------------*/
  strncpy(tipsp->band_geometric_quality,&buf[1036],1); 
  tipsp->band_geometric_quality[1] = '\0';
  tipsp->band_value_for_e = (float) swap_real(&buf[1040]); 
  tipsp->band_geometric_along_track_interval = (float) swap_real(&buf[1044]); 
  tipsp->band_geometric_across_track_interval = (float) swap_real(&buf[1048]); 
  tipsp->band_geometric_along_track_scene = (float) swap_real(&buf[1052]); 
  tipsp->band_geometric_across_track_scene = (float) swap_real(&buf[1056]); 
  tipsp->band_state_vector_radial_position = (float) swap_real(&buf[1060]); 
  tipsp->band_state_vector_along_track_position = (float) swap_real(&buf[1064]);
  tipsp->band_state_vector_across_track_position= (float) swap_real(&buf[1068]);
  tipsp->band_state_vector_radial_rate = (float) swap_real(&buf[1072]);
  tipsp->band_state_vector_along_track_rate = (float) swap_real(&buf[1076]); 
  tipsp->band_state_vector_across_track_rate = (float) swap_real(&buf[1080]);
  tipsp->band_state_vector_angle_roll = (float) swap_real(&buf[1084]);
  tipsp->band_state_vector_angle_pitch = (float) swap_real(&buf[1088]);
  tipsp->band_state_vector_angle_yaw = (float) swap_real(&buf[1092]);
  tipsp->band_state_vector_roll_angular_rate = (float) swap_real(&buf[1096]);
  tipsp->band_state_vector_pitch_angular_rate = (float) swap_real(&buf[1100]);
  tipsp->band_state_vector_yaw_angular_rate = (float) swap_real(&buf[1104]);
    /* get the state error covariance matrix
    --------------------------------------*/
    for (j = 0; j < 12; j++)
       for (i = 0; i < 12; i++)
        tipsp->band_state_error_cov_matrix[j][i] = 
  			(float) swap_real(&buf[1132 + (j * 48) + ( i * 4)]);
  tipsp->band_error_matrix_along_track_sigma = (float) swap_real(&buf[2428]);
  tipsp->band_error_matrix_across_track_sigma = (float) swap_real(&buf[2432]);
  tipsp->band_scan_gap_maximum = (float) swap_real(&buf[2796]);
  tipsp->band_scan_gap_minimum = (float) swap_real(&buf[2800]);
  tipsp->band_scan_gap_mean = (float) swap_real(&buf[2804]);
  tipsp->band_scan_gap_exceeding_maximum = (long) swap_long(&buf[2808]);
  tipsp->band_scan_gap_exceeding_minimum = (long) swap_long(&buf[2812]);
  tipsp->band_scan_skew_maximum = (float) swap_real(&buf[2816]);
  tipsp->band_scan_skew_minimum = (float) swap_real(&buf[2820]);
  tipsp->band_scan_skew_mean = (float) swap_real(&buf[2824]);
  tipsp->band_scan_skew_exceeding_maximum = (long) swap_long(&buf[2828]);
  tipsp->band_scan_skew_exceeding_minimum = (long) swap_long(&buf[2832]);
  tipsp->band_horizontal_strip_maximum = (float) swap_real(&buf[2836]);
  tipsp->band_horizontal_strip_minimum = (float) swap_real(&buf[2840]);
  tipsp->band_horizontal_strip_mean = (float) swap_real(&buf[2844]);
  tipsp->band_horizontal_strip_exceeding_maximum = (long) swap_long(&buf[2848]);
  tipsp->band_horizontal_strip_exceeding_minimum = (long) swap_long(&buf[2852]);
  tipsp->band_horizontal_strip_maximum_rotation = (float) swap_real(&buf[2856]);
  

  /* use data from annotation record
  --------------------------------*/
  fseek(fp,3600,0);
  fread(buf,1,540,fp);
  fclose(fp);

  strncpy(tipsp->annot_wrs_path,&buf[12],12);
  tipsp->annot_wrs_path[12] = '\0';
  strncpy(tipsp->annot_acquistion_date,&buf[24],8);
  tipsp->annot_acquistion_date[8] = '\0';
  strncpy(tipsp->annot_center_latitude_longitude,&buf[32],17);
  tipsp->annot_center_latitude_longitude[17] = '\0';
  strncpy(tipsp->annot_wrs_latitude_longitude,&buf[49],17);
  tipsp->annot_wrs_latitude_longitude[17] = '\0';
  strncpy(tipsp->annot_band_id_code,&buf[66],10);
  tipsp->annot_band_id_code[10] = '\0';
  strncpy(tipsp->annot_sun_angle,&buf[76],14);
  tipsp->annot_sun_angle[14] = '\0';
  strncpy(tipsp->annot_geometric_correction_header,&buf[90],1);
  tipsp->annot_geometric_correction_header[1] = '\0';
  strncpy(tipsp->annot_projection,&buf[92],1);
  tipsp->annot_projection[1] = '\0';
  strncpy(tipsp->annot_resampling_algorithm,&buf[94],1);
  tipsp->annot_resampling_algorithm[1] = '\0';
  strncpy(tipsp->annot_ephemeris,&buf[95],1);
  tipsp->annot_ephemeris[1] = '\0';
  strncpy(tipsp->annot_processing_procedure,&buf[96],1);
  tipsp->annot_processing_procedure[1] = '\0';
  strncpy(tipsp->annot_agency,&buf[102],13);
  tipsp->annot_agency[13] = '\0';
  strncpy(tipsp->annot_scene_identification,&buf[115],25);
  tipsp->annot_scene_identification[25] = '\0';


  /* update tick mark information
  -----------------------------*/
  for (i = 0; i < 8; i++)  
    {
    bit = buf[140 + (i * 9)] >> 7;
    buf[140 + (i * 9)] <<= 1;
    buf[140 + (i * 9)] >>= 1;
    temp[0] = buf[140 + (i * 9)];
    temp[1] = buf[141 + (i * 9)];
    tipsp->annot_top_tick[i] = (short) get_short(temp);
    if (bit == 1)
       tipsp->annot_top_tick[i] = -tipsp->annot_top_tick[i];
    strncpy(tipsp->annot_top_tick_ascii[i],&buf[142 + (i * 9)],7);
    tipsp->annot_top_tick_ascii[i][7] = '\0';
    }
  for (i = 0; i < 8; i++)  
    {
    bit = buf[212 + (i * 9)] >> 7;
    buf[212 + (i * 9)] <<= 1;
    buf[212 + (i * 9)] >>= 1;
    temp[0] = buf[212 + (i * 9)];
    temp[1] = buf[213 + (i * 9)];
    tipsp->annot_left_tick[i] = get_short(temp);
    if (bit == 1)
       tipsp->annot_left_tick[i] = -tipsp->annot_left_tick[i];
    strncpy(tipsp->annot_left_tick_ascii[i],&buf[214 + (i * 9)],7);
    tipsp->annot_left_tick_ascii[i][7] = '\0';
    }
  for (i = 0; i < 8; i++)  
    {
    bit = buf[284 + (i * 9)] >> 7;
    buf[284 + (i * 9)] <<= 1;
    buf[284 + (i * 9)] >>= 1;
    temp[0] = buf[284 + (i * 9)];
    temp[1] = buf[285 + (i * 9)];
    tipsp->annot_right_tick[i] = get_short(temp);
    if (bit == 1)
       tipsp->annot_right_tick[i] = -tipsp->annot_right_tick[i];
    strncpy(tipsp->annot_right_tick_ascii[i],&buf[286 + (i * 9)],7);
    tipsp->annot_right_tick_ascii[i][7] = '\0';
    }
  for (i = 0; i < 8; i++)  
    {
    bit = buf[356 + (i * 9)] >> 7;
    buf[356 + (i * 9)] <<= 1;
    buf[356 + (i * 9)] >>= 1;
    temp[0] = buf[356 + (i * 9)];
    temp[1] = buf[357 + (i * 9)];
    tipsp->annot_bottom_tick[i] = get_short(temp);
    if ( bit == 1)
          tipsp->annot_bottom_tick[i] = -tipsp->annot_bottom_tick[i];
    strncpy(tipsp->annot_bottom_tick_ascii[i],&buf[358 + (i * 9)],7);
    tipsp->annot_bottom_tick_ascii[i][7] = '\0';
    }
return(E_SUCC);
}
