/************************************
*  Common includes and definitions  *
************************************/
#include <stdio.h>
#include "las.h"
#include "tipsp.h"
#include "labserv.h"
#include "header.h"

#define  HEADER_SIZE 4320 

/*******************************************************************************
FUNCTION:	c_read_header	

PURPOSE: 	To read header file and fill the TIPSP structure.	

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
1.0	 T. Mittan	 11/92	 LAS	Original development.

ALGORITHM REFERENCES	none
*******************************************************************************/
int c_read_head
(
    struct TIPSP *tipsp,
    char *name			/* image name				*/
)
{
   FILE *fp;			/* file pointer				*/

   char  buf[HEADER_SIZE];      /* header buffer                        */
   long status;			/* return status			*/
   char hostheader[CMLEN];	/* header name				*/
   long mode = 0;		/* flag value for lsstat		*/
   long i;			/* array index				*/
   long one = 1;		/* constant value of one		*/

  c_lsmknm(name,"tipsp_head",hostheader);/* make associated header file name */
  status = c_lsstat(hostheader,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("HDR file does not exist on disk","read_head-stat",&one);
     return(E_FAIL);
     }

  fp = (fopen(hostheader,"r"));
  if(!fp)
   {
   c_errmsg("Error opening Header file","read_head-open",&one);
   return(E_FAIL);
   }

  fread(buf,1,HEADER_SIZE,fp);

  strncpy(tipsp->head_product_id,&buf[20],16);
  tipsp->head_product_id[16] = '\0';
  strncpy(tipsp->head_scene_id,&buf[36],16);
  tipsp->head_scene_id[16] = '\0';
  sscanf(&buf[52],"%f",&tipsp->head_center_latitude);
  sscanf(&buf[68],"%f",&tipsp->head_center_longitude);
  sscanf(&buf[84],"%f",&tipsp->head_center_line);
  sscanf(&buf[100],"%f",&tipsp->head_center_sample);
  strncpy(tipsp->head_center_time,&buf[116],32);
  tipsp->head_center_time[32] = '\0';
  sscanf(&buf[148],"%d",&tipsp->head_center_time_offset);
  strncpy(tipsp->head_wrs_designator,&buf[164],16);
  tipsp->head_wrs_designator[16] = '\0';
  sscanf(&buf[180],"%d",&tipsp->head_wrs_cycle);
  strncpy(tipsp->head_parent_id,&buf[196],16);
  tipsp->head_parent_id[16] = '\0';
  sscanf(&buf[212],"%f",&tipsp->head_pquadrant_lat);
  sscanf(&buf[228],"%f",&tipsp->head_pquadrant_long);
  sscanf(&buf[244],"%f",&tipsp->head_quadrant_line);
  sscanf(&buf[260],"%f",&tipsp->head_quadrant_sample);
  sscanf(&buf[276],"%d",&tipsp->head_vquadrant_overlap);
  sscanf(&buf[292],"%d",&tipsp->head_hquadrant_overlap);
  strncpy(tipsp->head_mission_id,&buf[308],16);
  tipsp->head_mission_id[16] = '\0';
  strncpy(tipsp->head_sensor_id,&buf[324],16);
  tipsp->head_sensor_id[16] = '\0';
  sscanf(&buf[340],"%d",&tipsp->head_orbit_number);
  strncpy(tipsp->head_orbit_direction,&buf[356],16);
  tipsp->head_orbit_direction[16] = '\0';
  sscanf(&buf[1412],"%d",&tipsp->head_bands);
  sscanf(&buf[1428],"%d",&tipsp->head_average_samples);
  sscanf(&buf[1444],"%d",&tipsp->head_lines);
  strncpy(tipsp->head_radiometric_resolution,&buf[1492],16);
  tipsp->head_radiometric_resolution[16] = '\0';
  strncpy(tipsp->head_radiometric_correction,&buf[1508],16);
  tipsp->head_radiometric_correction[16] = '\0';
  strncpy(tipsp->head_geometric_correction,&buf[1524],16);
  tipsp->head_geometric_correction[16] = '\0';
  strncpy(tipsp->head_resampling_designator,&buf[1540],16);
  tipsp->head_resampling_designator[16] = '\0';
  strncpy(tipsp->head_map_projection,&buf[1556],16);
  tipsp->head_map_projection[16] = '\0';
  strncpy(tipsp->head_product_class,&buf[1572],16);
  tipsp->head_product_class[16] = '\0';
  strncpy(tipsp->head_map_ancillary_records,&buf[1588],16);
  tipsp->head_map_ancillary_records[16] = '\0';
  strncpy(tipsp->head_radiometric_ancillary_records,&buf[1636],16);
  tipsp->head_radiometric_ancillary_records[16] = '\0';
  
  for (i = 0; i < 7;i++)
      sscanf(&buf[1652 + i],"%1d",&tipsp->head_bands_present[i]);
  for (i = 0; i < 100; i++)
      sscanf(&buf[1732 + (i * 4)],"%d",&tipsp->head_detector_map[i]);
  strncpy(tipsp->head_dectector_smooth_array,&buf[2132],100);
  

  strncpy(tipsp->head_interleave,&buf[1716],16);


  /* read data from Map Projection Record which is in the header record
  -------------------------------------------------------------------*/
  fseek(fp,4320,0);
  fread(buf,1,636,fp);


  /* input scene related information
  --------------------------------*/
  sscanf(&buf[12],"%d",&tipsp->map_nominal_pixels);
  sscanf(&buf[28],"%d",&tipsp->map_nominal_lines);
  sscanf(&buf[44],"%f",&tipsp->map_nominal_interpixel);
  sscanf(&buf[60],"%f",&tipsp->map_nominal_interline);
  sscanf(&buf[76],"%f",&tipsp->map_skew);

  /* UTM/PS related information
  ---------------------------*/
  strncpy(tipsp->map_utm_datum,&buf[92],6);
  tipsp->map_utm_datum[6] = '\0';
  sscanf(&buf[98],"%d",&tipsp->map_utm_zone);
  sscanf(&buf[108],"%f",&tipsp->map_utm_wrs_northing);
  sscanf(&buf[124],"%f",&tipsp->map_utm_wrs_easting);
  sscanf(&buf[140],"%f",&tipsp->map_utm_center_northing);
  sscanf(&buf[156],"%f",&tipsp->map_utm_center_easting);
  sscanf(&buf[172],"%f",&tipsp->map_utm_vertical_offset);
  sscanf(&buf[188],"%f",&tipsp->map_utm_horizontal_offset);
  sscanf(&buf[204],"%f",&tipsp->map_utm_rotation);

  /* SOM releate information
  ------------------------*/
  sscanf(&buf[220],"%f",&tipsp->map_som_wrs_horizontal);
  sscanf(&buf[236],"%f",&tipsp->map_som_wrs_vertical);
  sscanf(&buf[252],"%f",&tipsp->map_som_center_horizontal);
  sscanf(&buf[268],"%f",&tipsp->map_som_center_vertical);
  sscanf(&buf[284],"%f",&tipsp->map_som_vertical_offset);
  sscanf(&buf[300],"%f",&tipsp->map_som_horizontal_offset);
  sscanf(&buf[316],"%f",&tipsp->map_som_rotation);
  
  /* Processed Quadrant related information
  ---------------------------------------*/
  sscanf(&buf[332],"%f",&tipsp->map_pixels);
  sscanf(&buf[348],"%f",&tipsp->map_lines);
  sscanf(&buf[364],"%f",&tipsp->map_corrected_interpixel);
  sscanf(&buf[380],"%f",&tipsp->map_corrected_interline);
  sscanf(&buf[396],"%f",&tipsp->map_processed_zone);
  sscanf(&buf[412],"%d",&tipsp->map_center_line_number);
  sscanf(&buf[428],"%d",&tipsp->map_center_sample_number);
  sscanf(&buf[444],"%f",&tipsp->map_rotation);
  sscanf(&buf[460],"%f",&tipsp->map_orbit_inclination);
  sscanf(&buf[476],"%f",&tipsp->map_ascending_longitude);
  sscanf(&buf[492],"%f",&tipsp->map_altitude);
  sscanf(&buf[508],"%f",&tipsp->map_ground_speed);
  sscanf(&buf[524],"%f",&tipsp->map_satellite_heading);
  sscanf(&buf[556],"%f",&tipsp->map_cross_track_field_of_view);
  sscanf(&buf[572],"%f",&tipsp->map_scan_rate);
  sscanf(&buf[588],"%f",&tipsp->map_sampling_rate);
  sscanf(&buf[604],"%f",&tipsp->map_sun_elevation);
  sscanf(&buf[620],"%f",&tipsp->map_sun_azimuth);
  

  /* Read data from the Radiometric calibration record which is in the header
  -------------------------------------------------------------------------*/
  fseek(fp,8640,0);
  fread(buf,1,68,fp);
  fclose(fp);
 
  strncpy(tipsp->radio_band,&buf[12],4);
  tipsp->radio_band[4] = '\0';
  strncpy(tipsp->radio_lower_reflectance,&buf[16],4);
  tipsp->radio_lower_reflectance[4] = '\0';
  strncpy(tipsp->radio_upper_reflectance,&buf[20],4);
  tipsp->radio_upper_reflectance[4] = '\0';
  sscanf(&buf[28],"%le",&tipsp->radio_offset);
  sscanf(&buf[48],"%le",&tipsp->radio_gain);
  
  
return(E_SUCC);
}
