/************************************
*  Common includes and definitions  *
************************************/
#include <stdio.h>
#ifdef vms
#include <types.h>
#include <stat.h>
#else
#include <sys/types.h>
#include <sys/stat.h>
#endif

/*******************************************************************************
FUNCTION:	c_read_siat	

PURPOSE: 	To read the siat file and fill the SIAT structure.	

HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
1.0	 T. Mittan	 12/92	 LAS	Original development.
1.1	 S. Nelson	  6/94	 LAS	Added version 2 SIAT option.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include "las.h"
#include "siat.h"
#include "labserv.h"
#include "util.h"
#include "header.h"
#include "local_protos.h"

static long get_long_temp
(
    char *var
);


int c_read_siat
(
    struct SIAT *siat,      /* SIAT sturcture declaration		*/
    char  *name             /* image name				*/
)
{
   FILE *fp;			/* file pointer				*/

   long  sflag;			/* siat flag				*/
   long  status;		/* return status			*/
   long  i;			/* array index				*/
   long  mode = 0;		/* flag value for lsstat		*/
   long  nbyte;			/* number of bytes for c_eb2asc		*/
   long  one = 1;		/* constant value of one		*/
   long  old = FALSE;		/* flag for which siat file		*/

   char  temp[4];		/* temporary variable(must start on even 
				   boundary 				*/
   char  buf[3601];    		/* siat buffer                          */
   char  hostheader[CMLEN];	/* header name				*/
   char  err_buf[CMLEN];	/* error text buffer			*/
   struct stat status_buf;	/* structure for file statistics	*/

  c_lsmknm(name,"cctx_siat",hostheader);/* make associated header file name */
  status = c_lsstat(hostheader,&mode);    /* make sure header file exists     */
  if (status != E_SUCC)
     {
     c_errmsg("SIAT file does not exist on disk","read_siat-stat",&one);
     return(E_FAIL);
     }
  /* use data from logical tape record
  -----------------------------------*/
  fp = (fopen(hostheader,"r"));
  if(!fp)
   {
   c_errmsg("Error opening Siat file","read_siat-open",&one);
   return(E_FAIL);
   }

  fread(buf,1,2048,fp);
  stat(hostheader,&status_buf);
  if(status_buf.st_size==3658) 
     {
     old = TRUE;
     sflag = 1;
     }
  else if(status_buf.st_size==4168) 
     {
     old = TRUE;
     sflag = 2;
     }
  else if(status_buf.st_size==4138) 
     {
     old = FALSE;
     sflag = 3;
     }
  else 
    {
    sprintf(err_buf,"Error reading siat file (unknown size of %ld bytes)\n",
            (long)status_buf.st_size);
    c_errmsg(err_buf,"read_siat-filesize",&one);
    fclose(fp);
    return(E_FAIL);
    }

  /* Read logical tape information
  ------------------------------*/
  if (sflag != 3)
     {
     strncpy(siat->tape_number,&buf[0],8);
     siat->tape_number[8] = '\0';
     strncpy(siat->tape_date,&buf[8],10);
     siat->tape_date[10] = '\0';
     strncpy(siat->tape_siat_number,&buf[28],8);
     siat->tape_siat_number[8] = '\0';
     strncpy(siat->tape_rbv_number,&buf[36],8);
     siat->tape_rbv_number[8] = '\0';
     strncpy(siat->tape_mss_number,&buf[44],8);
     siat->tape_mss_number[8] = '\0';
     temp[0] = buf[52];
     temp[1] = buf[53];
     siat->tape_number_data_files = get_short(temp); 
     temp[0] = buf[58];
     temp[1] = buf[59];
     siat->tape_number_rbv_vtc = get_short(temp);
     temp[0] = buf[60];
     temp[1] = buf[61];
     siat->tape_number_mss_vtc = get_short(temp);
     temp[0] = buf[62];
     temp[1] = buf[63];
     siat->tape_number_rbv_tfc = get_short(temp);
     temp[0] = buf[64];
     temp[1] = buf[65];
     siat->tape_number_mss_tfc = get_short(temp);
     for (i = 0; i < 64; i++)
         {
         strncpy(siat->tape_rvb_scene_id[i],&buf[68 + (i * 12)],12);
         siat->tape_rvb_scene_id[i][12] = '\0';
         }
     for (i = 0; i < 64; i++)
         {
         strncpy(siat->tape_mss_scene_id[i],&buf[836 + (i * 12)],12);
         siat->tape_mss_scene_id[i][12] = '\0';
         }
     } 
  else
     {
     strncpy(siat->tape_number,&buf[0],12);
     siat->tape_number[12] = '\0';
     strncpy(siat->tape_sequence,&buf[12],2);
     siat->tape_sequence[2] = '\0';
     strncpy(siat->tape_date,&buf[14],10);
     siat->tape_date[10] = '\0';
     strncpy(siat->tape_siat_number,&buf[34],14);
     siat->tape_siat_number[14] = '\0';
     strncpy(siat->tape_rbv_number,&buf[48],8);
     siat->tape_rbv_number[8] = '\0';
     strncpy(siat->tape_mss_number,&buf[56],8);
     siat->tape_mss_number[8] = '\0';
     temp[0] = buf[64];
     temp[1] = buf[65];
     siat->tape_number_data_files = get_short(temp); 
     temp[0] = buf[70];
     temp[1] = buf[71];
     siat->tape_number_rbv_vtc = get_short(temp);
     temp[0] = buf[72];
     temp[1] = buf[73];
     siat->tape_number_mss_vtc = get_short(temp);
     temp[0] = buf[74];
     temp[1] = buf[75];
     siat->tape_number_rbv_tfc = get_short(temp);
     temp[0] = buf[76];
     temp[1] = buf[77];
     siat->tape_number_mss_tfc = get_short(temp);
     for (i = 0; i < 64; i++)
         {
         strncpy(siat->tape_rvb_scene_id[i],&buf[78 + (i * 12)],12);
         siat->tape_rvb_scene_id[i][12] = '\0';
         }
     for (i = 0; i < 64; i++)
         {
         strncpy(siat->tape_mss_scene_id[i],&buf[846 + (i * 12)],12);
         siat->tape_mss_scene_id[i][12] = '\0';
         }
     }
    
  /* use data from calibration modifiers record
  -------------------------------------------*/
  if (sflag != 1)
   {
   fseek(fp,2048,0);
   fread(buf,1,268,fp);

   /* read calibration modifiers record
   ----------------------------------*/ 
   temp[0] = buf[0];
   temp[1] = buf[1];
   siat->calibration_id = get_short(temp);
   temp[0] = buf[2];
   temp[1] = buf[3];
   siat->calibration_days = get_short(temp);
   for (i = 0; i < 6; i++)
     siat->calibration_band4_low_compressed_multiplicative_sensor[i] = 
		(float) buf[4 + (i * 2)] + get_fraction(&buf[5 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band4_low_compressed_additive_sensor[i] = 
		(float) buf[16 + (i * 2)] + get_fraction(&buf[17 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band4_low_linear_multiplicative_sensor[i] = 
		(float) buf[28 + (i * 2)] + get_fraction(&buf[29 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band4_low_linear_additive_sensor[i] = 
		(float) buf[40 + (i * 2)] + get_fraction(&buf[41 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band4_high_compressed_multiplicative_sensor[i] = 
		(float) buf[52 + (i * 2)] + get_fraction(&buf[53 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band4_high_compressed_additive_sensor[i] = 
		(float) buf[64 + (i * 2)] + get_fraction(&buf[65 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band4_high_linear_multiplicative_sensor[i] = 
		(float) buf[76 + (i * 2)] + get_fraction(&buf[77 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band4_high_linear_additive_sensor[i] = 
		(float) buf[88 + (i * 2)] + get_fraction(&buf[89 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band5_low_compressed_multiplicative_sensor[i] = 
	(float) buf[100 + (i * 2)] + get_fraction(&buf[101 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band5_low_compressed_additive_sensor[i] = 
	(float) buf[112 + (i * 2)] + get_fraction(&buf[113 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band5_low_linear_multiplicative_sensor[i] = 
	(float) buf[124 + (i * 2)] + get_fraction(&buf[125 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band5_low_linear_additive_sensor[i] = 
	(float) buf[136 + (i * 2)] + get_fraction(&buf[137 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band5_high_compressed_multiplicative_sensor[i] = 
 	(float) buf[148 + (i * 2)] + get_fraction(&buf[149 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band5_high_compressed_additive_sensor[i] = 
	(float) buf[160 + (i * 2)] + get_fraction(&buf[161 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band5_high_linear_multiplicative_sensor[i] = 
	(float) buf[172 + (i * 2)] + get_fraction(&buf[173 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band5_high_linear_additive_sensor[i] = 
	(float) buf[184 + (i * 2)] + get_fraction(&buf[185 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band6_low_linear_multiplicative_sensor[i] = 
	(float) buf[196 + (i * 2)] + get_fraction(&buf[197 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band6_low_linear_additive_sensor[i] = 
	(float) buf[208 + (i * 2)] + get_fraction(&buf[209 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band6_low_compressed_multiplicative_sensor[i] = 
	(float) buf[220 + (i * 2)] + get_fraction(&buf[221 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band6_low_compressed_additive_sensor[i] = 
	(float) buf[232 + (i * 2)] + get_fraction(&buf[233 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band7_low_linear_multiplicative_sensor[i] = 
	(float) buf[244 + (i * 2)] + get_fraction(&buf[245 + (i * 2)]);
   for (i = 0; i < 6; i++)
     siat->calibration_band7_low_linear_additive_sensor[i] = 
	(float) buf[256 + (i * 2)] + get_fraction(&buf[257 + (i * 2)]);
   }

  /* use data from processing record
  --------------------------------*/
  if (old)
    {
    if (sflag == 1)
       fseek(fp,2048,0);
    else if (sflag == 2)
       fseek(fp,2318,0);
    fread(buf,1,216,fp);

    temp[0] = buf[0];
    temp[1] = buf[1];
    siat->processing_scenes_remaining_rbv_vfc = get_short(temp); 
    temp[0] = buf[2];
    temp[1] = buf[3];
    siat->processing_scenes_remaining_mss_vfc = get_short(temp); 
    temp[0] = buf[4];
    temp[1] = buf[5];
    siat->processing_scenes_remaining_rbv_vtc = get_short(temp); 
    temp[0] = buf[6];
    temp[1] = buf[7];
    siat->processing_scenes_remaining_mss_vtc = get_short(temp); 
    strncpy(siat->processing_scene_id,&buf[12],10);
    siat->processing_scene_id[10] = '\0';
    strncpy(siat->processing_preceding_id,&buf[22],10);
    siat->processing_preceding_id[10] = '\0';
    strncpy(siat->processing_succeedingid,&buf[32],10);
    siat->processing_succeedingid[10] = '\0';
    siat->processing_mission_number = (long) buf[42]; 
    siat->processing_day_number_from_launch = ((long) buf[43] * 64) + 
							(long) buf[44]; 
    siat->processing_hours = (long) buf[45]; 
    siat->processing_minutes = (long) buf[46]; 
    siat->processing_seconds = (long) buf[47]; 
    for (i = 0; i < 8; i++)
       {
       strncpy(siat->processing_band_information[i],&buf[50 + (i * 8)],10);
       siat->processing_band_information[i][10] = '\0';
       }
    strncpy(siat->processing_instructions,&buf[114],72);
    siat->processing_instructions[72] = '\0';
    siat->processing_mission_number2 = (long) buf[186]; 
    siat->processing_day_number_from_launch2 = ((long) buf[187] * 64) + 
							(long) buf[188]; 
    siat->processing_hours2 = (long) buf[189]; 
    siat->processing_minutes2 = (long) buf[190]; 
    siat->processing_seconds2 = (long) buf[191]; 
    temp[0] = buf[202];
    temp[1] = buf[203];
    siat->processing_code_slat = get_short(temp); 
    temp[0] = buf[204];
    temp[1] = buf[205];
    siat->processing_code_mss = get_short(temp); 
    temp[0] = buf[206];
    temp[1] = buf[207];
    siat->processing_code_psp = get_short(temp); 
    }
  else				/* sflag == 3 */
    {
    fseek(fp,2318,0);
    fread(buf,1,222,fp);

    temp[0] = buf[0];
    temp[1] = buf[1];
    siat->processing_scenes_remaining_rbv_vfc = get_short(temp); 
    temp[0] = buf[2];
    temp[1] = buf[3];
    siat->processing_scenes_remaining_mss_vfc = get_short(temp); 
    temp[0] = buf[4];
    temp[1] = buf[5];
    siat->processing_scenes_remaining_rbv_vtc = get_short(temp); 
    temp[0] = buf[6];
    temp[1] = buf[7];
    siat->processing_scenes_remaining_mss_vtc = get_short(temp); 
    strncpy(siat->processing_scene_id,&buf[12],12);
    siat->processing_scene_id[12] = '\0';
    siat->processing_mission_number = (long) buf[48]; 
    siat->processing_day_number_from_launch = ((long) buf[49] * 64) + 
							(long) buf[50]; 
    siat->processing_hours = (long) buf[51]; 
    siat->processing_minutes = (long) buf[52]; 
    siat->processing_seconds = (long) buf[53]; 
    siat->processing_mission_number2 = (long) buf[192]; 
    siat->processing_day_number_from_launch2 = ((long) buf[193] * 64) + 
							(long) buf[194]; 
    siat->processing_hours2 = (long) buf[195]; 
    siat->processing_minutes2 = (long) buf[196]; 
    siat->processing_seconds2 = (long) buf[197]; 
/*
  siat->processing_output_id = 
*/
    temp[0] = buf[208];
    temp[1] = buf[209];
    siat->processing_code_slat = get_short(temp); 
    temp[0] = buf[210];
    temp[1] = buf[211];
    siat->processing_code_mss = get_short(temp); 
    temp[0] = buf[212];
    temp[1] = buf[213];
    siat->processing_code_psp = get_short(temp); 
    }

  /* use data from Spacecraft Performance Record -- version 1 and 2 are
     the same format but version 3 is different
  --------------------------------------------------------------------*/
  if (old)  
    {
    if (sflag == 1)
       fseek(fp,2318,0);
    else if (sflag == 2)
       fseek(fp,2534,0);
    fread(buf,1,205,fp);

    strncpy(siat->spacecraft_rbv1_transmission,&buf[0],8);
    siat->spacecraft_rbv1_transmission[8] = '\0';
    strncpy(siat->spacecraft_rbv1_duration,&buf[8],2);
    siat->spacecraft_rbv1_duration[2] = '\0';
    strncpy(siat->spacecraft_rbv1_aperture,&buf[10],2);
    siat->spacecraft_rbv1_aperture[2] = '\0';
    strncpy(siat->spacecraft_rbv2_transmission,&buf[12],8);
    siat->spacecraft_rbv2_transmission[8] = '\0';
    strncpy(siat->spacecraft_rbv2_duration,&buf[20],2);
    siat->spacecraft_rbv2_duration[2] = '\0';
    strncpy(siat->spacecraft_rbv2_aperture,&buf[22],2);
    siat->spacecraft_rbv2_aperture[2] = '\0';
    strncpy(siat->spacecraft_rbv3_transmission,&buf[24],8);
    siat->spacecraft_rbv3_transmission[8] = '\0';
    strncpy(siat->spacecraft_rbv3_duration,&buf[32],2);
    siat->spacecraft_rbv3_duration[2] = '\0';
    strncpy(siat->spacecraft_rbv3_aperture,&buf[34],2);
    siat->spacecraft_rbv3_aperture[2] = '\0';
    strncpy(siat->spacecraft_rbv4_transmission,&buf[36],12);
    siat->spacecraft_rbv4_transmission[12] = '\0';
    strncpy(siat->spacecraft_rbv5_transmission,&buf[48],12);
    siat->spacecraft_rbv5_transmission[12] = '\0';
    strncpy(siat->spacecraft_rbv6_transmission,&buf[60],12);
    siat->spacecraft_rbv6_transmission[12] = '\0';
    strncpy(siat->spacecraft_rbv7_transmission,&buf[72],12);
    siat->spacecraft_rbv7_transmission[12] = '\0';
    strncpy(siat->spacecraft_rbv8_transmission,&buf[84],12);
    siat->spacecraft_rbv8_transmission[12] = '\0';
    siat->spacecraft_sensor_gain_band4 = (buf[96] >> (7));
    buf[96] <<= 1;
    siat->spacecraft_sensor_gain_band5 = (buf[96] >> (7));
    siat->spacecraft_sensor_encoding_band4 = (buf[98] >> (7));
    buf[98] <<= 1;
    siat->spacecraft_sensor_encoding_band5 = (buf[98] >> (7));
    buf[98] <<= 1;
    siat->spacecraft_sensor_encoding_band6 = (buf[98] >> (7));
    strncpy(siat->spacecraft_tape_id,&buf[100],8);
    siat->spacecraft_tape_id[8] = '\0';
    strncpy(siat->spacecraft_sun_day,&buf[108],4);
    siat->spacecraft_sun_day[4] = '\0';
    for (i = 0; i < 24; i++)
      {
      temp[0] = buf[112 + (i * 2)];
      temp[1] = buf[113 + (i * 2)];
      siat->processing_mss_sun_sensor[i] = get_short(temp); 
      }
    strncpy(siat->spacecraft_sun_day2,&buf[196],4);
    siat->spacecraft_sun_day2[4] = '\0';
    strncpy(siat->spacecraft_sun_flag,&buf[200],4);
    siat->spacecraft_sun_flag[4] = '\0';
    }
  else
    {
    fseek(fp,2543,0);
    fread(buf,1,160,fp);

    strncpy(siat->spacecraft_rbv1_transmission,&buf[0],8);
    siat->spacecraft_rbv1_transmission[8] = '\0';
    strncpy(siat->spacecraft_rbv1_duration,&buf[8],2);
    siat->spacecraft_rbv1_duration[2] = '\0';
    strncpy(siat->spacecraft_rbv1_aperture,&buf[10],2);
    siat->spacecraft_rbv1_aperture[2] = '\0';
    strncpy(siat->spacecraft_rbv2_transmission,&buf[12],8);
    siat->spacecraft_rbv2_transmission[8] = '\0';
    strncpy(siat->spacecraft_rbv2_duration,&buf[20],2);
    siat->spacecraft_rbv2_duration[2] = '\0';
    strncpy(siat->spacecraft_rbv2_aperture,&buf[22],2);
    siat->spacecraft_rbv2_aperture[2] = '\0';
    strncpy(siat->spacecraft_rbv3_transmission,&buf[24],8);
    siat->spacecraft_rbv3_transmission[8] = '\0';
    strncpy(siat->spacecraft_rbv3_duration,&buf[32],2);
    siat->spacecraft_rbv3_duration[2] = '\0';
    strncpy(siat->spacecraft_rbv3_aperture,&buf[34],2);
    siat->spacecraft_rbv3_aperture[2] = '\0';
    strncpy(siat->spacecraft_rbv4_transmission,&buf[36],12);
    siat->spacecraft_rbv4_transmission[12] = '\0';
    strncpy(siat->spacecraft_rbv5_transmission,&buf[48],12);
    siat->spacecraft_rbv5_transmission[12] = '\0';
    strncpy(siat->spacecraft_rbv6_transmission,&buf[60],12);
    siat->spacecraft_rbv6_transmission[12] = '\0';
    strncpy(siat->spacecraft_rbv7_transmission,&buf[72],12);
    siat->spacecraft_rbv7_transmission[12] = '\0';
    strncpy(siat->spacecraft_rbv8_transmission,&buf[84],12);
    siat->spacecraft_rbv8_transmission[12] = '\0';

    siat->spacecraft_sensor_gain_band4 = (buf[96] >> (7));
    buf[96] <<= 1;
    siat->spacecraft_sensor_gain_band5 = (buf[96] >> (7));
    siat->spacecraft_sensor_encoding_band4 = (buf[98] >> (7));
    buf[98] <<= 1;
    siat->spacecraft_sensor_encoding_band5 = (buf[98] >> (7));
    buf[98] <<= 1;
    siat->spacecraft_sensor_encoding_band6 = (buf[98] >> (7));
    strncpy(siat->spacecraft_tape_id,&buf[100],8);
    siat->spacecraft_tape_id[8] = '\0';
    strncpy(siat->spacecraft_sun_day,&buf[108],4);
    siat->spacecraft_sun_day[4] = '\0';
    for (i = 0; i < 24; i++)
      {
      temp[0] = buf[112 + (i * 2)];
      temp[1] = buf[113 + (i * 2)];
      siat->processing_mss_sun_sensor[i] = get_short(temp); 
      }
    }

  /* use data from Annotation Record
  --------------------------------*/
  if (sflag == 1) 
    {
    fseek(fp,2543,0);
    fread(buf,1,144,fp);

    strncpy(siat->annotation_day,&buf[0],2);
    siat->annotation_day[2] = '\0';
    strncpy(siat->annotation_month,&buf[2],3);
    siat->annotation_month[3] = '\0';
    strncpy(siat->annotation_year,&buf[5],2);
    siat->annotation_year[2] = '\0';
    strncpy(siat->annotation_latitude,&buf[10],6);
    siat->annotation_latitude[6] = '\0';
    strncpy(siat->annotation_longitude,&buf[17],7);
    siat->annotation_longitude[7]='\0';
    strncpy(siat->annotation_nadir_latitude,&buf[27],6);
    siat->annotation_nadir_latitude[6] = '\0';
    strncpy(siat->annotation_nadir_longitude,&buf[34],8);
    siat->annotation_nadir_longitude[8] = '\0';
    strncpy(siat->annotation_sun_elevation,&buf[54],8);
    siat->annotation_sun_elevation[8] = '\0';
    strncpy(siat->annotation_sun_azimuth,&buf[62],6);
    siat->annotation_sun_azimuth[6] = '\0';
    strncpy(siat->annotation_heading,&buf[68],4);
    siat->annotation_heading[4] = '\0';
    strncpy(siat->annotation_rev,&buf[72],6);
    siat->annotation_rev[6] = '\0';
    strncpy(siat->annotation_data_acquistion,&buf[78],4);
    siat->annotation_data_acquistion[4] = '\0';
    strncpy(siat->annotation_orbit_type,&buf[84],2);
    siat->annotation_orbit_type[2] = '\0';
    strncpy(siat->annotation_scene_id,&buf[101],10);
    siat->annotation_scene_id[10] = '\0';
    strncpy(siat->annotation_rci_calibration,&buf[113],1);
    siat->annotation_rci_calibration[1] = '\0';
    strncpy(siat->annotation_rbv_mode1,&buf[116],5);
    siat->annotation_rbv_mode1[5] = '\0';
    strncpy(siat->annotation_rbv_shutter1,&buf[121],2);
    siat->annotation_rbv_shutter1[2] = '\0';
    strncpy(siat->annotation_rbv_mode2,&buf[123],6);
    siat->annotation_rbv_mode2[6] = '\0';
    strncpy(siat->annotation_rbv_shutter2,&buf[129],2);
    siat->annotation_rbv_shutter2[2] = '\0';
    strncpy(siat->annotation_rbv_mode3,&buf[131],6);
    siat->annotation_rbv_mode3[6] = '\0';
    strncpy(siat->annotation_rbv_shutter3,&buf[137],2);
    siat->annotation_rbv_shutter3[2] = '\0';
    strncpy(siat->annotation_mss_mode,&buf[131],5);
    siat->annotation_mss_mode[5] = '\0';
    }
  else
    {
    if (sflag == 2)
       fseek(fp,2738,0);
    else if (sflag == 3)
       fseek(fp,2708,0);
    fread(buf,1,144,fp);

    strncpy(siat->annotation_day,&buf[0],2);
    siat->annotation_day[2] = '\0';
    strncpy(siat->annotation_month,&buf[2],3);
    siat->annotation_month[3] = '\0';
    strncpy(siat->annotation_year,&buf[5],3);
    siat->annotation_year[3] = '\0';
    strncpy(siat->annotation_latitude,&buf[10],6);
    siat->annotation_latitude[6] = '\0';
    strncpy(siat->annotation_longitude,&buf[17],7);
    siat->annotation_longitude[7] = '\0';
    strncpy(siat->annotation_direction,&buf[25],1);
    siat->annotation_direction[1] = '\0';
    strncpy(siat->annotation_path_row,&buf[26],7);
    siat->annotation_path_row[7] = '\0';
    strncpy(siat->annotation_nadir_latitude,&buf[36],6);
    siat->annotation_nadir_latitude[6] = '\0';
    strncpy(siat->annotation_nadir_longitude,&buf[43],6);
    siat->annotation_nadir_longitude[6] = '\0';
    strncpy(siat->annotation_sensor_code,&buf[51],1);
    siat->annotation_sensor_code[1] = '\0';
    strncpy(siat->annotation_designated_band,&buf[52],7);
    siat->annotation_designated_band[7] = '\0';
    strncpy(siat->annotation_transmission_mode,&buf[59],1);
    siat->annotation_transmission_mode[1] = '\0';
    strncpy(siat->annotation_sun_elevation,&buf[60],9);
    siat->annotation_sun_elevation[9] = '\0';
    strncpy(siat->annotation_sun_azimuth,&buf[69],6);
    siat->annotation_sun_azimuth[6] = '\0';
    strncpy(siat->annotation_image_prcessing_codes,&buf[75],12);
    siat->annotation_image_prcessing_codes[12] = '\0';
    strncpy(siat->annotation_scene_id,&buf[100],13);
    siat->annotation_scene_id[13] = '\0';
    strncpy(siat->annotation_rbv,&buf[116],24);
    siat->annotation_rbv[24] = '\0';
    strncpy(siat->annotation_mss_mode_site,&buf[140],4);
    siat->annotation_mss_mode_site[4] = '\0';
    }


  /* use data from RBV computational record
  ---------------------------------------*/
  if (sflag == 1)
    fseek(fp,2708,0);
  else if (sflag == 2)
    fseek(fp,2883,0);
  else
    fseek(fp,2853,0);
  fread(buf,1,76,fp);

  siat->rbv_spacecraft_ddd = get_bcd(&buf[2])*100+get_bcd(&buf[3]);
  siat->rbv_spacecraft_hh = get_bcd(&buf[4]);
  siat->rbv_spacecraft_mm = get_bcd(&buf[5]);
  siat->rbv_spacecraft_ss = get_bcd(&buf[6]);
  siat->rbv_spacecraft_cc = get_bcd(&buf[7]);

  siat->rbv_greenwich_ddd = get_bcd(&buf[9])*100+get_bcd(&buf[10]);
  siat->rbv_greenwich_hh = get_bcd(&buf[11]);
  siat->rbv_greenwich_mm = get_bcd(&buf[12]);
  siat->rbv_greenwich_ss = get_bcd(&buf[13]);
  siat->rbv_greenwich_mmm = get_bcd(&buf[14])*100+get_bcd(&buf[15]);

  siat->rbv_altitude_change = get_frac(&buf[16]);
  strncpy(siat->rbv_gmt_date,&buf[18],10);
  siat->rbv_gmt_date[10] = '\0';
  strncpy(siat->rbv_gmt_time,&buf[28],8);
  siat->rbv_gmt_time[8] = '\0';
  siat->rbv_latitude_format_center = (float) get_long_temp(&buf[36]) * 1e-6;
  siat->rbv_longitude_format_center = (float) get_long_temp(&buf[40]) * 1e-6;
  siat->rbv_latitude_nadir = (float) get_long_temp(&buf[44]) * 1e-6;
  siat->rbv_longitude_nadir = (float) get_long_temp(&buf[48]) * 1e-6;
  siat->rbv_altitude = (float) get_long_temp(&buf[52]) * 1e-6;
  siat->rbv_gmt_exposure = (float) get_long_temp(&buf[56]) * 1e-6;
  siat->rbv_flight_heading = (float) get_long_temp(&buf[60]) * 1e-6;
  siat->rbv_pitch = (float) get_long_temp(&buf[64]) * 1e-6;
  siat->rbv_roll = (float) get_long_temp(&buf[68]) * 1e-6;
  siat->rbv_yaw = (float) get_long_temp(&buf[72]) * 1e-6;

  /* use data from mss computational record
  ---------------------------------------*/
  if (sflag == 1)
    fseek(fp,2853,0);
  else if (sflag == 2)
    fseek(fp,3208,0);
  else if (sflag == 3)
    fseek(fp,3178,0);

  fread(buf,1,324,fp);

  siat->mss_timesc_ddd = get_bcd(&buf[2])*100+get_bcd(&buf[3]);
  siat->mss_timesc_hh = get_bcd(&buf[4]);
  siat->mss_timesc_mm = get_bcd(&buf[5]);
  siat->mss_timesc_ss = get_bcd(&buf[6]);
  siat->mss_timesc_cc = get_bcd(&buf[7]);

  siat->mss_gmtsc_ddd = get_bcd(&buf[9])*100+get_bcd(&buf[10]);
  siat->mss_gmtsc_hh = get_bcd(&buf[11]);
  siat->mss_gmtsc_mm = get_bcd(&buf[12]);
  siat->mss_gmtsc_ss = get_bcd(&buf[13]);
  siat->mss_gmtsc_mmm = get_bcd(&buf[14])*100+get_bcd(&buf[15]);

  for(i=0;i<9;i++)
   {
   siat->mss_norm_alt_change_ic[i] = get_frac(&buf[16+i*2]);
   siat->mss_alt[i] = get_short(&buf[34+i*2]) / 32.0;
   }
  siat->mss_vehicle_roll_ic = get_frac(&buf[52]);
  siat->mss_vehicle_pitch_ic = get_frac(&buf[54]);
  siat->mss_vehicle_yaw_ic = get_frac(&buf[56]);

  for(i=0;i<11;i++)
   {
   siat->mss_roll[i] = get_frac(&buf[58+i*2]);
   siat->mss_pitch[i] = get_frac(&buf[76+i*2]);
   siat->mss_yaw[i] = get_frac(&buf[94+i*2]);
   }
  siat->mss_skew = get_frac(&buf[112]);
  siat->mss_norm_vel_change = get_frac(&buf[114]);
  siat->mss_mean_pitch = get_long(&buf[116])*1e-6;
  siat->mss_mean_roll = get_long(&buf[120])*1e-6;
  siat->mss_mean_yaw = get_long(&buf[124])*1e-6;
  siat->mss_mean_pitch_rate = get_long(&buf[128])*1e-6;
  siat->mss_mean_roll_rate = get_long(&buf[132])*1e-6;
  siat->mss_mean_yaw_rate = get_long(&buf[136])*1e-6;
  siat->mss_mean_alt = (float)get_long(&buf[140]);
  siat->mss_mean_alt_rate = (float)get_long(&buf[144]);

  for(i=0;i<11;i++)
   {
   siat->mss_gmt_ms[i]=get_long(&buf[192+i*4]);
   siat->mss_latitude[i]=get_long(&buf[192+i*4])*1e-6;
   siat->mss_longitude[i]=get_long(&buf[236+i*4])*1e-6;
   siat->mss_latlon_alt[i]=(float)get_long(&buf[280+i*4]);
   }

  /* use data from mss computational record
  ---------------------------------------*/
  if (sflag == 1)
    fseek(fp,3178,0);
  else if (sflag == 2)
    fseek(fp,3688,0);
  else
    fseek(fp,3658,0);
  fread(buf,1,480,fp);
  
  nbyte = 8;
  for (i = 0; i < 48; i++) 
      c_eb2asc(&buf[2 + (i * 10)],&nbyte);

  for (i = 0; i < 6; i++)
     {
     siat->image_rbv_top_point[i] = get_tick(&buf[i * 10],2);
     strncpy(siat->image_rbv_top_point_ascii[i],&buf[2 + (i * 10)],8);
     siat->image_rbv_top_point_ascii[i][8] = '\0';
     }
  for (i = 0; i < 6; i++)
     {
     siat->image_rbv_left_point[i] = get_tick(&buf[60 + (i * 10)],2);
     strncpy(siat->image_rbv_left_point_ascii[i],&buf[62 + (i * 10)],8);
     siat->image_rbv_left_point_ascii[i][8] = '\0';
     }
  for (i = 0; i < 6; i++)
     {
     siat->image_rbv_right_point[i] = get_tick(&buf[120 + (i * 10)],2);
     strncpy(siat->image_rbv_right_point_ascii[i],&buf[122 + (i * 10)],8);
     siat->image_rbv_right_point_ascii[i][8] = '\0';
     }
  for (i = 0; i < 6; i++)
     {
     siat->image_rbv_bottom_point[i] = get_tick(&buf[180 + (i * 10)],2);
     strncpy(siat->image_rbv_bottom_point_ascii[i],&buf[182 + (i * 10)],8);
     siat->image_rbv_bottom_point_ascii[i][8] = '\0';
     }
  for (i = 0; i < 6; i++)
     {
     siat->image_mss_top_point[i] = get_tick(&buf[240 + (i * 10)],2);
     strncpy(siat->image_mss_top_point_ascii[i],&buf[242 + (i * 10)],8);
     siat->image_mss_top_point_ascii[i][8] = '\0';
     }
  for (i = 0; i < 6; i++)
     {
     siat->image_mss_left_point[i] = get_tick(&buf[300 + (i * 10)],2);
     strncpy(siat->image_mss_left_point_ascii[i],&buf[302 + (i * 10)],8);
     siat->image_mss_left_point_ascii[i][8] = '\0';
     }
  for (i = 0; i < 6; i++)
     {
     siat->image_mss_right_point[i] = get_tick(&buf[360 + (i * 10)],2);
     strncpy(siat->image_mss_right_point_ascii[i],&buf[362 + (i * 10)],8);
     siat->image_mss_right_point_ascii[i][8] = '\0';
     }
  for (i = 0; i < 6; i++)
     {
     siat->image_mss_bottom_point[i] = get_tick(&buf[420 + (i * 10)],2);
     strncpy(siat->image_mss_bottom_point_ascii[i],&buf[422 + (i * 10)],8);
     siat->image_mss_bottom_point_ascii[i][8] = '\0';
     }

fclose(fp);
return(E_SUCC);
}

static long get_long_temp
(
    char *var
)
{
char temp[4];

  temp[0] = var[0];
  temp[1] = var[1];
  temp[2] = var[2];
  temp[3] = var[3];
  
return(get_long(temp));
}
