/*****************************************************************************
NAME:				C_READFPG

PURPOSE:  Converts values from the binary FPG format (See description in
	  "Landsat Multispectral Scanner Computer-Compatible Tape Format",
	  Appendix A, Version 1.0) to a float.

PROGRAM HISTORY:
VERSION		DATE	AUTHOR		DESCRIPTION
-------		----	------		-----------
  1.0		12/92	S. Nelson	Original Development

*****************************************************************************/
#include <stdio.h>
#include <strings.h>
#include "las.h"
#include "header.h"

int c_readfpg
(
    unsigned char *buf,     /* value in FPG format			*/
    double	*tempdbl        /* converted value of double		*/
)
{
double doub1;		

unsigned long	intg1;
unsigned long	frac1;
unsigned char 	sign[1];	/* Sign portion of value		*/

#ifndef sun
bcopy(buf,&intg1,4);
bcopy(buf,&frac1,4);
#else
memcpy(buf,&intg1,4);
memcpy(buf,&frac1,4);
#endif

*sign = buf[0] & 0x80;

intg1 = (intg1 & 0x7ffc0000) >> 18;
frac1 = (frac1 & 0x0003ffff);

doub1 = (double)frac1;

while ((int)doub1 > 1)
  doub1 /= 10.0;
if (sign[0]==NULL)
   *tempdbl = doub1 + (double)intg1;
else
   *tempdbl = -1*(doub1 + (double)intg1);

return(E_SUCC);
}
