#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "header.h"

/******************************************************************************

NAME: cdbl2ftn

PURPOSE: Converts a C double to a string compatible with FORTRAN formatting
         example: 1.234345E+06 becomes 0.12343450000000D+07

PROGRAM HISTORY:
  Version       Date       Author           Request
  -------       ----	   ------           -------
    1.0         11/92      K. Sheffield     Initial Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	

PROJECT: Landsat Pathfinder

ALGORITHM DESCRIPTION:
  The double is printed out to a 24.15lf format to a temporary string.
  The exponent value is parsed out.
  The most significant digit of the mantissa is shifted over one character.
  The new string is printed to the output string in the new format.

******************************************************************************/


int cdbl2ftn
(
    char *buffer,        /* the buffer to print the string to */
    double value         /* the value to convert */
)
{
   char tmpstr[30];                   /* temporary working string */
   int exponent;                      /* exponent of the value */
   char esign = '+';                  /* character sign of the exponent */
   char msign = ' ';                  /* character sign of the mantissa */
   char *cptr;
   
   long one = 1;
   

/* print out the value */

   sprintf(tmpstr,"%24.15E",value);
   /*if(status == EOF)
   {
      c_errmsg("Error creating output string","cdbl2ftn-sprintf",&one);
      return(E_FAIL);
   }*/
   
   
/* extract the exponent */

   cptr = strchr(tmpstr,'E');
   if(cptr == NULL)
   {
      c_errmsg("Error creating output string","cdbl2ftn-strchr",&one);
      return(E_FAIL);
   }
   

   if((sscanf(cptr+1,"%d",&exponent)) != 1)
   {
      c_errmsg("Error creating output string","cdbl2ftn-sscanf",&one);
      return(E_FAIL);
   }
   
   
/* now switch the most sig. digit over one */

   cptr = strchr(tmpstr,'.');
   if(cptr == NULL)
   {
      c_errmsg("Error creating output string","cdbl2ftn-strchr",&one);
      return(E_FAIL);
   }
   

   *cptr = *(cptr - 1);
   
   cptr[15] = 0;
   
   if(value < 0.0)
     msign = '-';
   
   if((strpbrk(cptr,"123456789")) != NULL)
     exponent++;

   if(exponent < 0)
   {
      esign = '-';
      exponent *= -1;
   }
   
   sprintf(buffer,"  %c0.%15sD%c%02d ",msign,cptr,esign,exponent);
   /*if(status == EOF)
      {
      c_errmsg("Error creating output string","cdbl2ftn-sprintf",&one);
      return(E_FAIL);
      }
   else*/

   return(E_SUCC);
}

   
/******************************************************************************

NAME: fstr2cdbl

PURPOSE: Converts a Fortran string to a C double
         example: 0.12343450000000D+07 becomes 1.234345E+06

PROGRAM HISTORY:
  Version       Date       Author           Request
  -------       ----	   ------           -------
    1.0         11/92      K. Sheffield     Initial Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:	

PROJECT: Landsat Pathfinder

ALGORITHM DESCRIPTION:
  Search for the D in the FORTRAN format and replace it with an E
  Use scanf to read the value in.

******************************************************************************/


int fstr2cdbl
(
    char *buffer,                          /* the buffer to with the string */
    double *value                         /* the return address */
)
{
   char *cptr;
   long one = 1;
   
/* search for the D */

   cptr = strchr(buffer,'D');
   if(cptr == NULL)
      cptr = strchr(buffer,'d');
   
/* ok, if we don't find a D or d, lets just assume that it's ok as it is */
   if( cptr != NULL)
     *cptr = 'E';
   
/* ok now read it */

   if((sscanf(buffer,"%lf",value)) != 1)
   {
      c_errmsg("Error extracting value from input string","fstr2cdbl-sscanf",
	       &one);
      return(E_FAIL);
   }
   
   else
     return(E_SUCC);
}

   
