/******************************************************************************
FUNCTION:       c_eclose

PURPOSE         closes the file after all processing is done

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           06/85    T. Butzer           Original implementation
           03/86    T. Butzer           dealloc the buffer space used
           03/86    K. Johnson          handle multiple file groups
           12/87    B. Ailts            change include directory specifications
                                        change to raw 'C' types
                                        place bridge routines is seperate file
           04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with c_errmsg
           05/88    B.Ailts             Replaced xi___ calls with i___ calls
           10/88    B.Ailts             Added statements to free dynamic
                                        allocated memory
           02/91    B.Ailts             Standardized error messages
           10/92    T.Mittan            Added compression option.
           12/92    T.Mittan            Corrected problem closing file.
           12/92    T.Mittan            Corrected problem freeing memory.
            1/94    K.Gacke             Modified to read/write dal images.  TAE
                                        images can be read, but all images are
                                        written as a BSQ DAL image.
           08/99    T.Beckmann          Added support for files >2GB
                                        

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Has to be run under TAE

PROJECT                 LAS

ALGORITHM 
        If CONVERSION or MOVEDATA or UPDATE
           Free conversion buffer memory
        Close the file
        Free the file descriptor memory
        Return

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>      /* fclose prototype */
#include <unistd.h>     /* close prototype */

#include "las.h"
#include "imageio.h"    /* c_eclose prototype */
#include "diskio.h"     /* c_dk prototypes */
#include "lasfileio.h"  /* c_fwrite prototype */
#include "taeimageio.h" /* tae image i/o prototypes */
#include "fdesc.h"


static long nonfatal = 1;               /* non-fatal error code */

int c_eclose
(
    struct FDESC **fdesc  /* I: file descriptor array       */
)
{
    int disp;
    int return_value = E_SUCC;
    
    if (((*fdesc)->flags & CONVERSION) || ((*fdesc)->flags & MOVEDATA) || 
        ((*fdesc)->acc == IUPDATE))
    {
         free((*fdesc)->conv_buf);
    }
    if ((*fdesc)->compres == TRUE)
    {
        if ((*fdesc)->acc == IWRITE)
        {
            if (FSEEK((*fdesc)->fp,18,SEEK_SET) != 0)
            {
                c_errmsg("error seeking in file","eread-seek",&nonfatal);
                return_value = E_FAIL;
            }
            else if (c_fwrite((*fdesc)->offset_table,(*fdesc)->file_offset_size,
                       ((*fdesc)->nl * (*fdesc)->nbands),(*fdesc)->fp) !=
                       ((*fdesc)->nl * (*fdesc)->nbands))
            {
                    c_errmsg("error writing file","eread-write",&nonfatal);
                    return_value = E_FAIL;
            }
            free((*fdesc)->offset_table);
            (*fdesc)->offset_table = NULL;
            free((*fdesc)->tempbuf);
            (*fdesc)->tempbuf = NULL;
        }
        else
        {
            free((*fdesc)->foffset);
            (*fdesc)->foffset = NULL;
        }
        fclose((*fdesc)->fp);
        (*fdesc)->fp = NULL;
        c_dkfree((*fdesc)->fname);
    }
    else if ((*fdesc)->tae == TRUE)
    {
        /**************
        printf("tae i/o not supported\n");
        return(E_FAIL);
        ***************/
        disp = I_SAVE;
        i_clse(&((*fdesc)->ifcb), disp);
        c_dkfree((*fdesc)->fname);
    }
    else if ((*fdesc)->dal == TRUE)
    {
        close((*fdesc)->dalfd);
        c_dkfree((*fdesc)->fname);
    }
    free(*fdesc);
    *fdesc = NULL;
    return(return_value);
}
