/******************************************************************************
FUNCTION:       c_ewrite

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original Development
  2.0      12/87    B. Ailts            Placed bridge routinue in a separate 
                                        file
                                        Changed include file specifications
                                        Renamed fron ewrite to c_ewrite
  2.1      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with c_errmsg
  2.2      05/88    B. Ailts            Replaced eerh with c_errmsg
                                        Deleted eerror.h
  2.3      08/88    L. Huewe            Added argument for number of lines to
                                        be read at one time
  2.4      09/88    L. Huewe            Fixed the calling sequence to i_read
  2.5      10/88    B. Ailts            Do not set cbufptr unless needed
  2.6      12/88    B. Ailts            Output an error message if values are
                                        clipped
  2.7       2/89    B. Ailts            Replaced the 'ceil' function -- did not
                                        work properly
  2.8       5/90    B. Ailts            Fixed a problem on VMS with the calc. of
                                        the index variable
  2.9      10/90    T. Baltzer          Rewrite so IUPDATE mode can convert data
  3.0      02/91    B. Ailts            Standardized error messages
  3.1      05/90    B. Ailts            Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  3.2      10/92    T. Mittan           Added compression option
  3.3      4/93     T. Mittan           Corrected problem when multiple lines
                                        are written in compressed mode.
            1/94    K.Gacke             Modified to read/write dal images.  TAE
                                        images can be read, but all images are
                                        written as a BSQ DAL image.
  3.5      08/99    T.Beckmann          Added support for files >2GB


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
   
ALGORITHM REFERENCES    none

NOTES:
- VAX I/O is limited to being done in 512 byte blocks.  To support this,
  the lines for an image on a VAX are grouped into 512 byte blocks.
*******************************************************************************/

#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "las.h"
#include "util.h"          /* c_sigblk prototype */
#include "lasfileio.h"     /* c_fwrite, c_write prototypes */
#include "datasize.h"
#include "fdesc.h"
#include "pixman.h"
#include "local_protos.h"

#define MAXBUF 65536

static long write_dal
(
    struct FDESC *fd,        /*  file descriptor                     */
    long *eband,             /*  band number to write                */
    long *eline,             /*  line number to write                */
    unsigned char *euserbuf, /*  buffer                              */
    long *nlines             /*  number of lines to write            */
);

static long nonfatal = 1;

long c_ewrite
(
    struct FDESC **fdesc,    /* I: file descriptor */
    long *eband,             /* I: band number to write */
    long *eline,             /* I: line number to write */
    unsigned char *euserbuf, /* I: buffer */
    long *nlines             /* I: number of lines to write */
)
{
    struct FDESC *fd;
    long count1;
    long line;
    long status;
    long i,j;
    long nwrites;
    long numlines;
    long index;
    long one = 1;
    long conv_type;
    
    static short big_first = TRUE;
    static short small_first = TRUE;
    
    unsigned char *cbufptr;
    unsigned char *ubufptr;
    int sig_mask;
    
    fd = *fdesc;
    if (fd->dal)
    {
        status = write_dal(fd, eband, eline, euserbuf, nlines);
        return(status);
    }
    /*
    * Process Compress Image
    */
    c_sigblk(&sig_mask);
    line = *eline + fd->sl - 1;
    
    nwrites = 1; 
    numlines = *nlines; 
    index = 0;

    for (i=0; i < nwrites; i++)
    {
        if ((fd->flags & CONVERSION) || (fd->flags & MOVEDATA) ||
            (fd->acc == IUPDATE))
        {
            if (fd->acc == IUPDATE)
            { 
                c_errmsg("Image update access not allowed in Compression mode",
                         "ewrite",&nonfatal);
                return(E_FAIL);
            }

            ubufptr = &euserbuf[index];
            for (j=0; j < numlines; j++)
            {
                if (fd->flags & CONVERSION)
                {
                    cbufptr = fd->conv_buf;
                    status = c_pxconv(&fd->dtype, &fd->conv_type, ubufptr,
                                      cbufptr, &fd->ns);
                    if (status != E_SUCC)
                    {
                        if (((status & TOOBIG) == TOOBIG) && (big_first))
                        {
                            c_errmsg("Pixel values were larger than maximum "
                                     "allowed by output data type",
                                     "ewrite-pixel",&nonfatal);
                            big_first = FALSE;
                        }
                        else if (((status & TOOSMALL) == TOOSMALL) && 
                                 (small_first))
                        {
                            c_errmsg("Pixel values were smaller than minimum "
                                     "allowed by output data type",
                                     "ewrite-pixel",&nonfatal);
                            small_first = FALSE;
                        }
                    }
                }
                else
                {
                    cbufptr = ubufptr;
                }
                if ( fd->compres == TRUE)
                {
                    if (FSEEK(fd->fp,0,SEEK_END) != 0)
                    {
                        c_errmsg("error seeking in file","ewrite-seek",
                                 &nonfatal);
                        return(E_FAIL);
                    }

                    /* set the file offset for the current line in the offset
                       table */
                    if (fd->file_offset_size == sizeof(long long))
                    {
                        ((long long *)fd->offset_table)[((line + j - 1) * 
                                fd->nbands) + (*eband - 1)] = FTELL(fd->fp);
                    }
                    else
                    {
                        ((int *)fd->offset_table)[((line + j - 1)*fd->nbands)+ 
                                (*eband - 1)] = (int)FTELL(fd->fp);
                    }
                    
                    if (fd->conv_type == 0)
                        conv_type = fd->dtype;
                    else
                        conv_type = fd->conv_type; 
                    encode_76(cbufptr,fd->tempbuf,fd->ns, &count1,
                              datasize[conv_type]);
                    if (c_fwrite(&count1,sizeof(long), one,fd->fp) != one)
                    {
                        c_errmsg("error writing file","ewrite-write",&nonfatal);
                        return(E_FAIL);
                    }
                    if (c_fwrite(fd->tempbuf,sizeof(char),count1,fd->fp)
                               !=count1)
                    {
                        c_errmsg("error writing file","ewrite-write",&nonfatal);
                        return(E_FAIL);
                    }
                }
                ubufptr += (fd->ns * datasize[fd->dtype]);
            }
        }
        else
        {
            /* no conversion, update, or move needed branch */
            if ( fd->compres == TRUE)
            {
                for (j=0; j < numlines; j++)
                {
                    index = j * (fd->ns * datasize[fd->dtype]);
                    if (FSEEK(fd->fp,0,SEEK_END) != 0)
                    {
                        c_errmsg("error seeking in file","ewrite-seek",
                                 &nonfatal);
                        return(E_FAIL);
                    }

                    /* set the file offset for the current line in the offset
                       table */
                    if (fd->file_offset_size == sizeof(long long))
                    {
                        ((long long *)fd->offset_table)[((line + j - 1) * 
                                fd->nbands) + (*eband -1)] = FTELL(fd->fp);
                    }
                    else
                    {
                        ((int *)fd->offset_table)[((line + j - 1) * fd->nbands) 
                                + (*eband -1)] = (int)FTELL(fd->fp);
                    }
                    
                            
                    if (fd->conv_type == 0)
                        conv_type = fd->dtype;
                    else
                        conv_type = fd->conv_type; 
                    encode_76(&euserbuf[index],fd->tempbuf,fd->ns, &count1,
                              datasize[conv_type]);
                    if (c_fwrite(&count1,sizeof(long), one,fd->fp) != one)
                    {
                        c_errmsg("error writing file","ewrite-write",&nonfatal);
                        return(E_FAIL);
                    }
                    if (c_fwrite(fd->tempbuf,sizeof(char),count1,fd->fp)
                            !=count1)
                    {
                        c_errmsg("error writing file","ewrite-write",&nonfatal);
                        return(E_FAIL);
                    }
                }
            }
        }
#ifdef DEBUG
        printf("write line = %d nlines = %d\n", line, *nlines);
#endif
    }
    c_sigunblk(&sig_mask);
    return(fd->ns * (*nlines));
}


static long write_dal
(
    struct FDESC *fd,        /*  file descriptor                     */
    long *eband,             /*  band number to write                */
    long *eline,             /*  line number to write                */
    unsigned char *euserbuf, /*  buffer                              */
    long *nlines             /*  number of lines to write            */
)
{
    static short big_first = TRUE;
    static short small_first = TRUE;
    long dsize;
    long line;
    long lsize;
    long nbyte;
    long status;
    int sig_mask;
    LSEEK_OFF_T newpos;
    unsigned char *cbufptr;
    unsigned char *ubufptr;
    
#ifdef DEBUG
    printf("write line = %d nlines = %d\n", line, *nlines);
#endif
    c_sigblk(&sig_mask);
    lsize = (fd->flags & CONVERSION) ? fd->cnvlsz : fd->linsiz;
    dsize = (fd->flags & CONVERSION) ? datasize[fd->conv_type] :
                                       datasize[fd->dtype];
                                       
    if ((fd->flags & CONVERSION) || (fd->flags & MOVEDATA) || 
        (fd->acc == IUPDATE))
    {
        ubufptr = euserbuf;
        for (line=0; line<*nlines; line++)
        {
            cbufptr = ubufptr;
            if (fd->flags & CONVERSION)
            {
                cbufptr = fd->conv_buf;
                status=c_pxconv(&fd->dtype,&fd->conv_type,ubufptr,cbufptr,
                                &fd->ns);
                if (status != E_SUCC)
                {
                    if (((status & TOOBIG) == TOOBIG) && (big_first))
                    {
                        c_errmsg("Pixel values were larger than maximum "
                                 "allowed by output data type","ewrite-pixel",
                                 &nonfatal);
                        big_first = FALSE;
                    }
                    else if (((status & TOOSMALL) == TOOSMALL) && (small_first))
                    {
                        c_errmsg("Pixel values were smaller than minimum "
                                 "allowed by output data type","ewrite-pixel",
                                 &nonfatal);
                        small_first = FALSE;
                    }
                }
            }
            /*
            *  Need to seek in case a window was specified.
            */
            newpos = ((LSEEK_OFF_T)(*eband - 1) * fd->img_nl * lsize) +
                     ((*eline + fd->sl + line - 2) * (LSEEK_OFF_T)lsize) + 
                     ((fd->ss - 1) * (LSEEK_OFF_T)dsize);
            if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
            {
                perror("c_ewrite-seek");
                c_errmsg("error seeking dal file","ewrite",&nonfatal);
                return(E_FAIL);
            }
            nbyte = fd->ns * dsize;
            status = c_write(fd->dalfd,cbufptr,nbyte,dsize);
            if (status < nbyte)
            {
                perror("c_ewrite-write");
                c_errmsg("error writing dal file","ewrite",&nonfatal);
                return(E_FAIL);
            }
            ubufptr += (fd->ns * datasize[fd->dtype]);
        }
    }
    else
    {
        ubufptr = euserbuf;
        newpos = ((LSEEK_OFF_T)(*eband - 1) * fd->img_nl * lsize) +
                 ((*eline + fd->sl - 2) * (LSEEK_OFF_T)lsize) + 
                 ((fd->ss - 1) * (LSEEK_OFF_T)datasize[fd->dtype]);
        if (LSEEK(fd->dalfd,newpos,SEEK_SET) < 0)
        {
            perror("c_ewrite-seek");
            c_errmsg("error seeking dal file","ewrite",&nonfatal);
            return(E_FAIL);
        }
        nbyte = lsize * *nlines;
        status = c_write(fd->dalfd,ubufptr,nbyte,datasize[fd->dtype]);
        if (status < nbyte)
        {
            perror("c_ewrite-write");
            c_errmsg("error writing dal file","ewrite",&nonfatal);
            return(E_FAIL);
        }
    }
    c_sigunblk(&sig_mask);
    return(fd->ns * (*nlines));
}
