/******************************************************************************
FUNCTION:       chkwindow

PURPOSE:        Checks to see if the specified window is outside the actual
                window for a LAS file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           06/85    T. Butzer           Original implementation
           12/87    B. Ailts            change include directory structure
           12/87    B. Ailts            implemented new DDR structure
           02/91    B. Ailts            Standardized error messages
           08/99    T.Beckmann          Added support for files >2GB

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Check the window specifications against the DDR window specs.
If the window specifications are outside the DDR window specs
   Return a fatal error

ALGORITHM REFERENCES    none
*******************************************************************************/

#include "las.h"
#include "fdesc.h"

int check_window
(
    struct FDESC *fd,  /* I: file descriptor */
    struct DDR *ddr    /* I: ddr structure */
)
{
    long nonfatal = 1;
    
    char *errtxt;

    
    if ((fd->sl + fd->nl - 1) > ddr->nl)
        return(E_FAIL);
    
    if ((fd->ss + fd->ns - 1) > ddr->ns)
        return (E_FAIL);
    
    if ((fd->sl != 1) || (fd->ss != 1) || 
        (fd->nl != ddr->nl) || (fd->ns != ddr->ns))
    {
        if ((errtxt = (char *)malloc(ERRLEN)) == NULL)
        {
            c_errmsg("Error allocating dynamic memory","chkwindow-alloc",
                     &nonfatal);
            return(E_FAIL);
        }

        c_lasmsg(FALSE,"chkwindow-window","Window specified for file %s is "
                 "(%d %d %d %d)",fd->fname,fd->sl,fd->ss,fd->nl,fd->ns);
        free(errtxt);
    }
    return(E_SUCC);
} 
