/******************************************************************************
FUNCTION:      decode_76.c 

PURPOSE:       Decode_76 decodes a -7 ,6 shift to ends encoding scheme. 

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      07/92    T. Mittan           Original implementation

*/

#include <stdio.h>
#include "las.h"
#include "lasendian.h"

#define   shift_down   0x000D           /*     1010  4                  */
#define   m6           0x007F           /* 111 1111  7                  */
#define   m5           0x003E           /*  11 1110  6                  */
/*
#define   m4           0x0005                1 1110  5                  
#define   m3           0x0006                  1101  4                  
#define   m2           0x0007                  1100  4                  
*/
#define   m1           0x0004           /*      100  3                  */
/*
#define   zero         0x0000                    00  2                  
#define   p1           0x000A                  0110  4                  
#define   p2           0x000B                  0111  4                  
#define   p3           0x000C                  0100  4                  
#define   p4           0x001C                  0101  4                  
#define   p5           0x001D                1 1100  5          
#define   p6           0x001E                1 1101  5          
#define   p7           0x007E              111 1110  7                  
*/
#define   shift_up     0x0004           /*     1011  4                  */

void decode_76
(
   unsigned char  *inpt,  /* I: input buffer */
   unsigned char  *outpt, /* O: output buffer */
   long ns,               /* I: number of samples */
   long width             /* I: number of bytes per sample */
)
{
    long code;                           /* encoded shift code           */
    long temp1;                          /* temp value of previous inpt  */
    long temp2;                          /* temp value of previous inpt  */
    long j;                              /* counter                      */
    long temp;                           /* temporary variable           */
    long input_bit = 0;                  /* number of input bits         */
    unsigned char  *in_ptr;              /* output buffer                */
    unsigned char  *out_ptr;             /* output buffer                */
    unsigned long input_buffer = 0;      /* input bit buffer             */
    long      i,                         /* counter                      */ 
    shift;                      /* amount of shift to be applied*/
    int msb_index;     /* index to most significant byte */
    int endian_step;   /* direction to step for least significant bytes */
    
#ifdef LAS_BIG_ENDIAN
    /* setup for big endian byte order */
    msb_index = 0;
    endian_step = 1;
#else
    /* setup for little endian byte order */
    msb_index = width - 1;
    endian_step = -1;
#endif
    
    in_ptr = inpt;
    out_ptr = outpt;
    inpt++;
    while (input_bit <=24)
    {
        input_buffer |= ((unsigned long) *inpt << (24- input_bit));
        inpt++;
        input_bit += 8;
    }
    *(outpt + msb_index) = (char)(input_buffer >> 24);
    input_buffer <<= 8;
    input_bit -= 8;
    
    temp = width;
    temp1 = *(outpt + msb_index);
    temp2 = 1;
    /* for each byte of data type decode NS bits of data
    ---------------------------------------------------*/
    if (in_ptr[0] == (char)1)       /* if data has been compres */
    {
        for (j = msb_index; (j >= 0) && (j < width); j += endian_step)
        {
            for(outpt = out_ptr + j + temp,i =  temp2; i < ns ; 
                i++, outpt +=width) 
            {
                shift = 0;
                for(;;) 
                {
                    if (input_bit <=24)
                    {
                        input_buffer |= 
                                ((unsigned long) *inpt << (24 - input_bit));
                        inpt++;
                        input_bit += 8;
                    }
                    if ((input_buffer >> 30) == 0)
                    {
                        input_buffer <<= 2;
                        input_bit -= 2;
                    }
                    else
                    if ((input_buffer >> 29) == m1)
                    {
                        shift -=1; 
                        input_buffer <<= 3;
                        input_bit -= 3;
                    }
                    else
                    {
                        code = (long)(input_buffer >> 28);
                        if (code == shift_down)
                        {
                            shift -=13;
                            input_buffer <<= 4;
                            input_bit -= 4;
                            continue;
                        }
                        else
                        if (code == shift_up)
                        {
                            shift +=13;
                            input_buffer <<= 4;
                            input_bit -= 4;
                            continue;
                        }
                        else
                        if (code < 13)
                        {
                            shift += code -9;
                            input_buffer <<= 4;
                            input_bit -= 4;
                        }
                        else
                        {
                            code = (long)(input_buffer >> 27);
                            if (code < 31)
                            {
                                shift += code - 24;
                                input_buffer <<= 5;
                                input_bit -= 5;
                            }
                            else
                            if ((input_buffer >> 26) == m5)
                            {
                                shift -=5;
                                input_buffer <<= 6;
                                input_bit -= 6;
                            }
                            else
                            {
                                if ((input_buffer >> 25) == m6)
                                    shift -=6;
                                else
                                    shift += 7;
                                input_buffer <<= 7;
                                input_bit -= 7;
                            }
                        }
                    }
                    break;
                }
                /* save output value
                ------------------*/
                temp1 += shift;
                *outpt = (char)temp1;
            }      
            temp = 0;
            temp2 = 0;
        }
    }
    else
    {  /* data has not been compres */
        inpt = in_ptr;
        inpt++;
        outpt = out_ptr;
        for (j = 0; j < width; j++)
        {
            for(i =  0; i < ns ; i++, outpt++, inpt++) 
                *outpt = *inpt;
        }
    
        /* convert the data to the local endian */
        localendian_buf(out_ptr, width, ns);
    }
}
