/******************************************************************************
FUNCTION:       do_open

PURPOSE:

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0               T.Butzer            Original development
  2.0      04/88    D.Hollaren          Replaced newlas.h with las.h
                                        Replaced cerrmsg with c_errmsg
  2.1      05/88    B.Ailts             Replaced errh with c_errmsg
                                        Took out eerror.h
  2.2      02/91    B.Ailts             Standardized error messages
  2.3      05/92    T. Mittan           Output more descriptive message
                                        when not enough room is available on a
                                        disk on VMS
  2.4      10/92    T. Mittan           Added compression option
  2.5      12/92    T. Mittan           Modified memory allocation for 
                                        fd->foffset
            1/94    K.Gacke             Modified to read/write dal images.  TAE
                                        images can be read, but all images are
                                        written as a BSQ DAL image.
  2.6      08/94    D. Etrheim          Change 777 to 666 on file open
  2.7      08/99    T.Beckmann          Added support for files >2GB


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
If write access
   Open a new file
Else
   Open an existing file

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include "las.h"
#include "lasfileio.h"    /* c_fread, c_fwrite prototypes */
#include "taeimageio.h"   /* tae image i/o prototypes */
#include "fdesc.h"
#include "local_protos.h"

short glun = 7;

int do_open
(
    struct FDESC *fd /* I/O: file descriptor */
)
{
    char errtxt[ERRLEN + 1];
    long type, status, lun;
    long nonfatal = 1;
    long herr;
    long num;                      /* temporary holding value      */
    int  opflg;                    /* open flag */
    char buf[64];                  /* buffer to check type of file */
    
    if (fd->acc == IWRITE)
    {
        lun = glun++;
        
        /* If data compression is desired insert header file
        ---------------------------------------------------*/
        if (fd->compres)
        {
            /* allocate space for the offset table */
            fd->offset_table = calloc((fd->nl * fd->nbands), 
                                      fd->file_offset_size);
            if (fd->offset_table == NULL)
            {
                c_errmsg("Error allocating dynamic memory","doopen-table",
                         &nonfatal);
                return(E_FAIL);
            }

            if ((fd->fp = fopen(fd->fname,"w")) == NULL)
            {
                c_errmsg("Error opening file","doopen-open",&nonfatal);
                return(E_FAIL);
            }
            /* Insert header file 
            --------------------*/
            num = 10;
            if (fd->file_offset_size == sizeof(long long))
                strcpy(buf,"Comprlarge");
            else
                strcpy(buf,"Compressed");
            if (c_fwrite(buf,sizeof(char),num,fd->fp) != num)
            {
                c_errmsg("error writing to file","open-write",&nonfatal);
                return(E_FAIL);
            }
            if (FSEEK(fd->fp,14,SEEK_SET) != 0)
            {
                c_errmsg("error seeking in file","open-seek",&nonfatal);
                return(E_FAIL);
            }
            num = 1;
            if (c_fwrite(&num,sizeof(long),1,fd->fp) != 1)
            {
                c_errmsg("error writing to file","open-write",&nonfatal);
                return(E_FAIL);
            }

            /* write the empty offset table to the file - it will be filled
               in when the file is closed */
            num = fd->nl * fd->nbands * fd->file_offset_size;
            if (c_fwrite(fd->offset_table,sizeof(char),num,fd->fp) != num)
            {
                c_errmsg("error writing to file","open-write",&nonfatal);
                return(E_FAIL);
            }
            if (FSEEK(fd->fp,10,SEEK_SET) != 0)
            {
                c_errmsg("error seeking in file","open-seek",&nonfatal);
                return(E_FAIL);
            }
            if (c_fwrite(&fd->nbands,sizeof(long),1,fd->fp) != 1)
            {
                c_errmsg("error writing to file","open-write",&nonfatal);
                return(E_FAIL);
            }
            if (FSEEK(fd->fp,0,SEEK_END) != 0)
            {
                c_errmsg("error seeking in file","open-seek",&nonfatal);
                return(E_FAIL);
            }
        }
        else
        {
            opflg = O_RDWR | O_CREAT | O_TRUNC;
            if ((fd->dalfd = open(fd->fname,opflg,0666)) == -1)
            {
                perror("doopen-dal");
                c_errmsg("Error opening file","doopen-open",&nonfatal);
                return(E_FAIL);
            }
        }
    }
    else
    {
        lun = glun++;
        if (fd->acc == IREAD)
            type = I_INPUT;
        else 
            type = I_INOUT; 
        
        /* open file and check to see if it is in compresssed format
        ----------------------------------------------------------*/
        fd->compres = fd->tae = fd->dal = FALSE;
        if ((fd->fp = fopen(fd->fname,"r")) == NULL)
        {
            c_errmsg("Error opening file","doopen-open",&nonfatal);
            return(E_FAIL);
        }
        if (c_fread(buf,sizeof(char),sizeof(buf),fd->fp) == E_FAIL)
        {
            c_errmsg("Error reading file","doopen-read",&nonfatal);
            return(E_FAIL);
        }
        if ((strncmp(buf,"Compressed",strlen("Compressed")) == 0) ||
            (strncmp(buf,"Comprlarge",strlen("Comprlarge")) == 0))
        {
            fd->compres = TRUE;
            if (FSEEK(fd->fp,10,SEEK_SET) != 0)
            {
                c_errmsg("error seeking in file","open-seek",&nonfatal);
                return(E_FAIL);
            }
            if (c_fread(&fd->mbands,sizeof(long),1,fd->fp) != 1)
            {
                c_errmsg("error reading from file","open-read",&nonfatal);
                return(E_FAIL);
            }
            if (fd->nbands > fd->mbands)
                fd->foffset = (long *)calloc(fd->nbands, sizeof(long));
            else
                fd->foffset = (long *)calloc(fd->mbands, sizeof(long));

            /* set the file offset size indication */
            if (strncmp(buf,"Comprlarge",strlen("Comprlarge")) == 0)
                fd->file_offset_size = sizeof(long long);
            else
                fd->file_offset_size = sizeof(int);
        }
        else if (strncmp(buf+48,"IMAG",strlen("IMAG")) == 0)
        {
            /*********
            printf("tae i/o not supported\n");
            return(E_FAIL);
            ************/
            fclose(fd->fp);
            fd->tae = TRUE;
            if (type == I_INOUT)
            {
                c_errmsg("Update mode not allowed with TAE image format",
                         "doopen-open",&nonfatal);
                sprintf(errtxt,"%s %s",
                    "Please use LAS copy function to create DAL image for file",
                    fd->fname);
                c_errmsg(errtxt,"doopen-open",&nonfatal);
                return(E_FAIL);
            }
            status = i_opin(&(fd->ifcb), lun,fd->fname, type);
            if (status != SUCCESS)
            {
                herr = i_herr(&(fd->ifcb));
                c_lasmsg(FALSE,"doopen-open","Error %d encountered opening "
                         "file %s",herr,fd->fname);
                return(E_FAIL);
            }
        }
        else
        {
            fd->dal = TRUE;
            fclose(fd->fp);
            if (fd->acc == IREAD)
                opflg = O_RDONLY;
            else 
                opflg = O_RDWR;
            if ((fd->dalfd = open(fd->fname,opflg,0666)) == -1)
            {
                perror("doopen-dal");
                c_errmsg("Error opening file","doopen-open",&nonfatal);
                return(E_FAIL);
            }
        }
    }
    return(E_SUCC);
}
