#ifndef FDESC_H
#define FDESC_H

#include <stdio.h>
#include "buf.h"

struct FDESC            /* file descriptor */
{

    long nl;                /* number of lines in image window */
    long ns;                /* number of samples in image window */
    long img_nl;            /* image number of lines */
    long img_ns;            /* image number of samples */
    long sl;                /* starting line */
    long ss;                /* starting sample */
    long *offset;           /* pointer to users offset array */
    long acc;               /* access READ WRITE or UPDATE */
    long method;            /* access method RANDOM or SEQ */
    long dtype;             /* data type of data */
    long nbuf;              /* number of circular buffers */
    long linsiz;            /* actual line size in bytes */
    long conv_type;         /* data type to be converted to */
    long cnvlsz;            /* actual line size of converted data */
    char fname[CMLEN];      /* host name of the data file */
    long nbands;            /* number of bands to be processed */
    unsigned long flags;    /* misc flags */
    long bands[MAXBND];     /* band array of specific band numbers */
    struct IFCB ifcb;       /* the i_ ifcb */       
    struct BUF *buf;        /* buffer structure */
    struct UPBUF *upbuf;    /* update buffer structure */
    FILE *fp;               /* file pointer for compressed or dal files */
    int dalfd;              /* dal image file descriptor */
    unsigned char *conv_buf;/* conversion buffer address */
    long file_linsiz;       /* linesize of the file */
    long compres;           /* compression image flag    */
    long dal;               /* dal image flag */
    long tae;               /* tae image flag */
    long *foffset;          /* compression file pointer           */
    void *offset_table;     /* table of offsets to each line in a compressed
                               image, indexed by line,band.  This is currently
                               only used for writing compressed images. */
    int  file_offset_size;  /* number of bytes in a file offset.  Used for 
                               determining if a compressed file is < 2GB or
                               > 2GB. */
    long mbands;            /* number of bands in compressed file */
    long mns;               /* number of samples in compressed file */
    unsigned char *tempbuf; /* temporary buffer for a single line */
};

struct GDESC            /* group descriptor */
{
    struct FDESC *fdarray[MAXIMG];  /* array of file descriptors group */
    long    nl;                     /* number of lines to be processed */
    long    nfiles;                 /* number of files in group */
    long    line_cnt;               /* line counter for current line */
    unsigned char   *thebuf;        /* address of buffer pool */
    long    thebufsz;               /* size of buffer pool */
    unsigned char   *the_nxt_space; /* next allocatable address in buffer
                                       pool */
    long    gbl_nlines;             /* number of lines in buffer pool */
};
#endif
