/*********************   Label Services Subsystem   *********************
FUNCTION:       lsmknm

PURPOSE:        Given an image name, make a label services file name.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           08/85    T. Bodoh            initial development
           07/86    K. Gacke            newLAS development
           03/87    D. Gordon           added end of directory check for vax
           12/87    B. Ailts            change include directory specifications
                                        place bridge routines in a seperate file
                                        replace DESC arguments with char strings
                                        use raw 'C' types
  5.0      04/88    D. Hollaren         changed include spec to las

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Make sure the name isn't to long
Otherwise, append the specified extention

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"

/**
 * c_lsmknm:
 * @inname: I: Input host name with which the label services file is
 * associated.
 * @ext: I: Input extension for label services file.
 * @outname: O: The label services filename.  It is inname with 
 * extension replaced with ext.
 *
 * The c_lsmknm() call is used to make a label services filename.
 * Note that it only makes the name.  It does not open the file.
 *
 *
 */
void c_lsmknm
(
    const char *inname, /* I: input host image name */
    const char *ext,    /* I: input extension for label services file */
    char *outname       /* O: output host label services file name */
)
{
    char        *ptr;
    char        temp[CMLEN];

    strcpy (temp,inname);
    if ((ptr = strrchr(temp,HOST_EXT)) > strrchr(temp,HOST_DIR_END))
       *ptr = '\0';

    sprintf(outname,"%s%c%s",temp,HOST_EXT,ext);
}
