/***********************************************************************
FUNCTION:       labserv_datasize

PURPOSE:        Return the number of bytes for a datatype specified as a
                string.  Types recognized are 
                        B for Byte, 
                        I2 for 16-bit integer,
                        I4 for 32-bit integer,
                        R4 for float,
                        R8 for double

PROGRAM HISTORY:
$Log: datasize.c,v $
Revision 1.4  2002/01/04 17:58:19  tbeckman
Merged library cleanup from CR #4098

Revision 1.3.2.1  2001/06/05 22:03:34  ametzger
All tags were added to labserv.c files for the
automated code documentation process.

Revision 1.3  2001/02/13 20:20:45  tbeckman
Changed the name of datasize to labserv_datasize to prevent name collision.  CR #4649.

Revision 1.2  2001/02/07 15:38:08  tbeckman
Force a rebuild

Revision 1.1  2001/02/03 23:25:57  tbeckman
Adding routines to support portability improvements

**************************************************************************/
#include <ctype.h>
#include "local_protos.h"

/**
 * labserv_datasize:
 * @dtype: I: Data type of the data portion of the record.
 *            Valid data types are:
 *            B  = byte,  
 *            I2 = 16 bit integer,
 *            I4 = 32 bit integer,
 *            R4 = floating point,        
 *            R8 =  double.
 * 
 * Returns the number of bytes for a datatype specified as a string.
 * Types recognized are:
 * B for Byte,
 * I2 for 16-bit integer,
 * I4 for 32-bit integer
 * R4 for a float,
 * R8 for a double.
 * 
 * Returns: int bytes_per_element
 */
int labserv_datasize
(
    const char *dtype   /* I: data type (B,I2,I4,R4,R8) */
)
{
    char type = toupper(dtype[0]);
    int bytes_per_element = 0;
    
    if ((type == 'I') || (type == 'R'))
    {
        if ((dtype[1] == '2') || (dtype[1] == '4') || (dtype[1] == '8'))
        {
            bytes_per_element = dtype[1] - '0';
        }
    }
    else if (type == 'B')
        bytes_per_element = 1;

    return bytes_per_element;
}
