/******************************************************************************
NAME:   

FUNCTION:       Bridge routines for the LT calls

PROGRAM HISTORY:
  Version  Date   Author       Request
  -------  ----   ------       -------
    1      12/86  K. Gacke      original development
    2       4/87  K. Gacke      called squeeze routine for the file type and 
                                file description in the foptab routine when 
                                opening for write access.
    3       5/87  K. Gacke      added bridge routines for Unix
    4       7/87  K. Gacke      changed the calls malloc and free to xalloc and 
                                xree for the sel.  These routines are the C run 
                                time routines renamed and put into the las.a
                                library to avoid linking the malloc and free in 
                                the fortran libraries.
    5       8/87  K. Gacke      added the DATE and TIME data types.  Created the
                                routines fgdate, fgtime, fpdate, and fptime.
    6       9/87  K. Gacke      removed mod for "version 4" after installing
                                new OS (UTX 2.0)
    7      12/87  B. Ailts      Change include file directory specifications
    8      05/88  B. Ailts      Places brigde routines in a seperate file
                                Replaced las.h with worgen.h
                                Standardized err messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "ltable.h"
#include "desc.h"
#include "local_protos.h"

void foptab
(
    struct TAB_DEF **fd,
    struct DESC *hname,
    int *access,
    int *nsubs,
    struct DESC *sub_file,
    struct DESC *ftype,
    struct DESC *fdesc
)
{
    struct TAB_DEF *tab;
    char *fname;
    char *ptr;
    char *subfil;

    fname = squeeze(hname->addr,hname->length);
    if ((tab = (struct TAB_DEF *) malloc(sizeof(struct TAB_DEF))) == NULL)
    {
        tab_error(tab,TAB_ALLOC,FATAL,"","foptab");
        return;
    }
    if (*access == 1)
    {
        ptr = sub_file->addr;
        for (tab->nsubs=0; tab->nsubs<*nsubs; tab->nsubs++)
        {
            subfil = squeeze(ptr,sub_file->length);
            tab->subfile[tab->nsubs] = subfil;
            ptr += sub_file->length;
        }
        tab->ftype = malloc(ftype->length);
        tab->fdesc = malloc(fdesc->length);
        if ((tab->ftype == NULL) || (tab->fdesc == NULL))
        {
            tab_error(tab,TAB_ALLOC,FATAL,"","foptab");
            return;
        }
        ptr = squeeze(ftype->addr,ftype->length);
        strcpy(tab->ftype,ptr);
        ptr = squeeze(fdesc->addr,fdesc->length);
        strcpy(tab->fdesc,ptr);
    }
    open_tab(tab,fname,*access);
    *fd = tab;
    if (*access == 0)
    {
        ptr = sub_file->addr;
        for (*nsubs=0; *nsubs<tab->nsubs; (*nsubs)++)
        {
            strcpy(ptr,tab->subfile[*nsubs]);
            ptr += sub_file->length;
        }
        strcpy(ftype->addr,tab->ftype);
        strcpy(fdesc->addr,tab->fdesc);
    }
    free(fname);
}

void foptab_
(
    struct TAB_DEF **fd,
    char *hname,
    int *access,
    int *nsubs,
    char *sub_file,
    char *ftype,
    char *fdesc,
    int hname_len,
    int sub_file_len,
    int ftype_len,
    int fdesc_len
)
{
    struct DESC dhname,dsub_file,dftype,dfdesc;

    dhname.addr = hname;
    dhname.length = hname_len;
    dsub_file.addr = sub_file;
    dsub_file.length = sub_file_len;
    dftype.addr = ftype;
    dftype.length = ftype_len;
    dfdesc.addr = fdesc;
    dfdesc.length = fdesc_len;
    foptab(fd,&dhname,access,nsubs,&dsub_file,&dftype,&dfdesc);
}
