/******************************************************************************
NAME:   

FUNCTION:       Bridge routines for the LT calls

PROGRAM HISTORY:
  Version  Date   Author       Request
  -------  ----   ------       -------
    1      12/86  K. Gacke      original development
    2       4/87  K. Gacke      called squeeze routine for the file type and 
                                file description in the foptab routine when 
                                opening for write access.
    3       5/87  K. Gacke      added bridge routines for Unix
    4       7/87  K. Gacke      changed the calls malloc and free to xalloc and 
                                xree for the sel.  These routines are the C run 
                                time routines renamed and put into the las.a
                                library to avoid linking the malloc and free in 
                                the fortran libraries.
    5       8/87  K. Gacke      added the DATE and TIME data types.  Created the
                                routines fgdate, fgtime, fpdate, and fptime.
    6       9/87  K. Gacke      removed mod for "version 4" after installing
                                new OS (UTX 2.0)
    7      12/87  B. Ailts      Change include file directory specifications
    8      05/88  B. Ailts      Places brigde routines in a seperate file
                                Replaced las.h with worgen.h
                                Standardized err messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

******************************************************************************/
#include "worgen.h"
#include "ltable.h"
#include "local_protos.h"

void fpmat
(
    struct VECTOR **rec_desc,
    int size[],
    char *matrix,
    int *sep
)
{
    struct VECTOR *rec;
    double *r8_ptr;
    float  *r4_ptr;
    struct TAB_DEF  dummy;
    int    *i4_ptr;
    int    index;
    int    nelem;
    short  *i2_ptr;
    char   *i1_ptr;

    rec = (struct VECTOR *) *rec_desc;
    rec->size.nrow = size[0];
    rec->size.ncol = size[1];
    switch (*sep)
    {
        case FSEP1:
            rec->sep = FIELD_SEP1;
            break;
        case FSEP2:
            rec->sep = FIELD_SEP2;
            break;
        case RSEP:
            rec->sep = RECORD_SEP;
            break;
    }
    nelem = rec->size.nrow * rec->size.ncol;
    switch (rec->dtype)
    {
        case I1:  
            i1_ptr = (char *) matrix;
            for (index=0; index<nelem; index++,i1_ptr++)
                *(rec->ptr.i1 + index) = *i1_ptr;
            break;
        case I2:
            i2_ptr = (short *) matrix;
            for (index=0; index<nelem; index++,i2_ptr++)
                *(rec->ptr.i2 + index) = *i2_ptr;
            break;
        case I4:
            i4_ptr = (int *) matrix;
            for (index=0; index<nelem; index++,i4_ptr++)
                *(rec->ptr.i4 + index) = *i4_ptr;
            break;
        case R4:
            r4_ptr = (float *) matrix;
            for (index=0; index<nelem; index++,r4_ptr++)
                *(rec->ptr.r4 + index) = *r4_ptr;
            break;
        case R8:
            r8_ptr = (double *) matrix;
            for (index=0; index<nelem; index++,r8_ptr++)
                *(rec->ptr.r8 + index) = *r8_ptr;
            break;
        default: 
            tab_error(&dummy,TAB_LT_DTYPE,FATAL,
              "Tried to retrieve character data as a numerical matrix",
                  "fpmat");
            return;
    }
}

void fpmat_
(
    struct VECTOR **rec_desc,
    int size[],
    char *matrix,
    int *sep
)
{
    fpmat(rec_desc,size,matrix,sep);
}
