
/*******************************************************************************
FUNCTION:       get_lt_date

PURPOSE:        Return date from Labeled Table I/O buffer

PROGRAM HISTORY:
  Version  Date   Author        Request
  -------  ----   ------        -------
    1      12/86  K. Gacke      Original development
    2       3/87  K. Gacke      Modified open_tab to allow directories to be 
                                specified for the subfiles.  Replaced "strcat" 
                                calls with "strcpy" to make the code more 
                                efficient.  Replaced "sscanf" calls with "atol"
                                and "atof" to make the code more efficient.
    3       5/87  K. Gacke      Modified "put_vector" to search the link list 
                                for each attribute defined.
    4       6/87  K. Gacke      Allow the LT to be opened for append.  Modified
                                "close_tab", "get_vector", "open_tab", and 
                                "put_vector".
    5       7/87  K. Gacke      changed the calls malloc and free to xalloc and
                                xree for the sel.  These routines are the C run
                                time routines renamed and put into the newlas.a
                                library to avoid linking the malloc and free in
                                the fortran libraries.
    6       8/87  K. Gacke      added the DATE and TIME data types.  Modified
                                the routines get_vector, get_record, put_record,
                                and put_vector.  Created the routines
                                get_lt_date, get_lt_time, put_lt_date, and
                                put_lt_time.
    7       9/87  K. Gacke      removed mod for "version 5" after installing
                                new OS (UTX 2.0)
    8      12/87  B. Ailts      Change include directory structure
    9      05/88  B. Ailts      Standardized err messages
                                Split routines into separate files
                                Added worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

ALGORITHM:
Reads the date field from the Labeled Table and returns it to the application 
program in a structure.  The structure is defined to be:
                        struct LT_DATE
                            {
                            long month;
                            long day;
                            long year;
                            }
It is assumed the date is stored as an ASCII string within the Labeled Table 
I/O buffer with the following format:
                        mm/dd/yyyy
Error checking is not done to ensure the date is valid.
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "ltable.h"
#include "local_protos.h"

/*
 * get_lt_date:
 * @vec:  I/O: Pointer to a structure defining a field within an LT logical
 *             record.  Note that vec is the linked list defining the LT record.
 * @date: O: The date read from the LT.
 *
 * The get_lt_date() routine reads the date field from the labeled table and 
 * returns it to the application program in a structure.    
 *       
 * It is assumed the date is stored as an ASCII string within the labeled table
 * I/O bufer with the following format: mm/dd/yyyy.  Error checking is not
 * done to ensure the data is valid.
 */
void get_lt_date
(
    struct VECTOR *vec,         /* I/O: Defines the LT record element         */
    struct LT_DATE *date        /* O: Date read from the LT I/O buffer        */
)
{
    struct TAB_DEF dummy;
    int status;

    if (vec->dtype != LTDATE)
        tab_error(&dummy,TAB_LT_DTYPE,FATAL,"Invalid data type for LT date",
                  "get_lt_date");

    status = sscanf(vec->ptr.c,"%d/%d/%d",&date->month,
                    &date->day,&date->year);
    if (status != 3)
        tab_error(&dummy,TAB_LT_GETDATE,FATAL,"","get_lt_date");

}
