/*******************************************************************************
FUNCTION:       get_lt_time

PURPOSE:        Return time from Labeled Table I/O buffer

PROGRAM HISTORY:
  Version  Date   Author        Request
  -------  ----   ------        -------
    1      12/86  K. Gacke      Original development
    2       3/87  K. Gacke      Modified open_tab to allow directories to be 
                                specified for the subfiles.  Replaced "strcat" 
                                calls with "strcpy" to make the code more 
                                efficient.  Replaced "sscanf" calls with "atol"
                                and "atof" to make the code more efficient.
    3       5/87  K. Gacke      Modified "put_vector" to search the link list 
                                for each attribute defined.
    4       6/87  K. Gacke      Allow the LT to be opened for append.  Modified
                                "close_tab", "get_vector", "open_tab", and 
                                "put_vector".
    5       7/87  K. Gacke      changed the calls malloc and free to xalloc and
                                xree for the sel.  These routines are the C run
                                time routines renamed and put into the newlas.a
                                library to avoid linking the malloc and free in
                                the fortran libraries.
    6       8/87  K. Gacke      added the DATE and TIME data types.  Modified
                                the routines get_vector, get_record, put_record,
                                and put_vector.  Created the routines
                                get_lt_date, get_lt_time, put_lt_date, and
                                put_lt_time.
    7       9/87  K. Gacke      removed mod for "version 5" after installing
                                new OS (UTX 2.0)
    8      12/87  B. Ailts      Change include directory structure
    9      05/88  B. Ailts      Standardized err messages
                                Split routines into separate files
                                Added worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  none.

PROJECT:        LAS

ALGORITHM:
Reads the time field from the Labeled Table and returns it to the application 
program in a structure.  The structure is defined to be:
                        struct LT_TIME
                            {
                            long hour;
                            long minute;
                            long second;
                            }
It is assumed the time is stored as an ASCII string within the Labeled Table 
I/O buffer with the following format:
                        hh:mm:ss
Error checking is not done to ensure the time is valid.
*******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "ltable.h"
#include "local_protos.h"

/*
 * get_lt_time:
 * @vec:  I/O: Pointer to a structure defining a field within an LT logical
 *             record.  Note that vec is the linked list defining the LT
 *             record.
 * @time:  O: The time read from the LT.
 *
 * The get_lt_time() routine reads the time field from the labeled table and
 * returns it to the application program in a structure.  
 *       
 * It is assumed the time is stored as an ASCII string within the labeled table
 * I/O bufer with the following format: hr/min/sec.  Error checking is not
 * done to ensure the data is valid.
 */
void get_lt_time
(
    struct VECTOR *vec,         /* I/O: Struct for the LT record element     */
    struct LT_TIME *tim         /* O: Time read from the LT I/O buffer       */
)
{
    struct TAB_DEF dummy;
    int status;

    if (vec->dtype != LTTIME)
        tab_error(&dummy,TAB_LT_DTYPE,FATAL,"Invalid data type for LT time",
                  "get_lt_time");

    status = sscanf(vec->ptr.c,"%d:%d:%d",&tim->hour,&tim->minute,
                    &tim->second);

    if (status != 3)
        tab_error(&dummy,TAB_LT_GETTIME,FATAL,"","get_lt_time");

}
