/*******************************************************************************
NAME				C_RANDNM

PURPOSE:	C_RANDNM is a random number generator that will work
		consistently on all machines with 32 bit integers.
		It returns a random number between 0 and 1 as a double.
			

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.VANDERZEE	12/90	Original development 

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  	NONE
		

PROJECT			LAS

ALGORITHM 		See reference

ALGORITHM REFERENCES:   
 	The algorithm was taken from the October 1988 issue
	of Communications of the ACM Volume 31. The article
	is titled RANDOM NUMBER GENERATORS: GOOD ONES ARE HARD TO FIND
	by Stephen K. Park and Keith W. Miller
******************************************************************************/

#include <math.h>
#include "lasmath.h"

double c_randnm
(
    long *seed                   /* seed value to be reused */
)

{
static long   a=16807;		/* constant value*/
static long   m=2147483647;	/* maximum value */
static long   q=127773;		/* value of m divided by a */ 
static long   r=2836;  	        /* m mod a */

long	hi,lo,test;             /* variables used in calculating the new seed */
double	rand;		

hi = *seed / q;
lo = *seed % q;
test = a * lo - r * hi;
if (test > 0.0) *seed = test;
else *seed = (test + m);

rand = (double)*seed/(double)m;
	
return (rand); 
}
