      FUNCTION ERFF(X,N1)                                               ERFF    
C ----PURPOSE:  TO COMPUTE THE VALUE OF THE ERROR FUNCTION OR           ERFF    
C     THE STANDARD NORMAL CUMULATIVE DISTRIBUTION FUNCTION              ERFF    
C     AT A GIVEN POINT.                                                 ERFF    
C                                                                       ERFF    
C ----CALLING SEQUENCE :  VARIABLE = ERFF(X,N1)                         ERFF    
C                                                                       ERFF    
C     X     IS THE POINT WHERE THE FUNCTION IS EVALUATED (INPUT)        ERFF    
C                                                                       ERFF    
C     N1    =1, IF THE ERROR FUNCTION IS DESIRED.                       ERFF    
C           =2, IF THE COMPLIMENTARY ERROR FUNCTION IS DESIRED.         ERFF    
C           =3, IF THE STANDARD NORMAL CUMULATIVE DISTRIBUTION          ERFF    
C           FUNCTION IS DESIRED.                                        ERFF    
C                                                                       ERFF    
C ----RESTRICTIONS AND COMMENTS: NONE.                                  ERFF    
      REAL ABS,T,V                                                              
      GO TO (1,2,3),N1                                                  ERFF    
C                                                                       ERFF    
C ----ERF                                                               ERFF    
      ENTRY ERF(X)                                                      ERFF    
    1 N=1                                                               ERFF    
      T=X                                                               ERFF    
      GO TO 4                                                           ERFF    
C                                                                       ERFF    
C ----ERFC                                                              ERFF    
      ENTRY ERFC(X)                                                     ERFF    
    2 N=2                                                               ERFF    
      T=X                                                               ERFF    
      GO TO 4                                                           ERFF    
C                                                                       ERFF    
C ----PHI                                                               ERFF    
      ENTRY PHI(X)                                                      ERFF    
    3 N=3                                                               ERFF    
      T=X/1.4142136                                                     ERFF    
    4 M=1                                                               ERFF    
      IF(X.LT.0.) M=2                                                   ERFF    
      V=ABS(T)                                                          ERFF    
      U=T*T                                                             ERFF    
C                                                                       ERFF    
C ----TEST V                                                            ERFF    
      IF(V-0.5) 10,10,20                                                ERFF    
C                                                                       ERFF    
C ----CASE V.LE.0.5                                                     ERFF    
   10 ERFF= ((0.31665289*U+1.7222758)*U+21.385332)/((U+7.8437457)*U     ERFF    
     1    +18.952257)*T                                                 ERFF    
C                                                                       ERFF    
C ----BRANCH TO RETURN THE APPROPRIATE FUNCTION                         ERFF    
      GO TO (5,6,7),N                                                   ERFF    
    5 RETURN                                                            ERFF    
    6 ERFF=1.0-ERFF                                                     ERFF    
      RETURN                                                            ERFF    
    7 ERFF=0.5+0.5*ERFF                                                 ERFF    
      RETURN                                                            ERFF    
C                                                                       ERFF    
   20 X1=V                                                              ERFF    
      X2=V*V                                                            ERFF    
      X3=V*X2                                                           ERFF    
      X4=X2*X2                                                          ERFF    
C                                                                       ERFF    
C ----TEST V                                                            ERFF    
      IF(V-8.0) 25,25,30                                                ERFF    
C                                                                       ERFF    
C ----CASE  V BETWEEN 0.5 AND 8.0 (HART #5703,P 294)                    ERFF    
   25 ERFF=EXP(-X2)*(6.1337529+X1*6.1772458+X2*2.8501393+X3*0.56409092) ERFF    
     1  /  (6.1337546+X1*13.098327+X2*11.497651+X3*5.0472398+X4)        ERFF    
      GO TO 40                                                          ERFF    
C                                                                       ERFF    
C ----EXCEPT FOR THE LAST CASE, ERFF=0                                  ERFF    
   30 ERFF=0.                                                           ERFF    
C                                                                       ERFF    
C ----TEST N AND V                                                      ERFF    
      IF((N.NE.2.AND.M.EQ.1).OR.(N.NE.3.AND.M.EQ.2).OR.(V.GT.13.))GOTO40ERFF    
C                                                                       ERFF    
C ----CASE  V BETWEEN 8. AND 13.5 AND ERFC OR PHI(-V) IS DESIRED        ERFF    
C     (HART #5721,P 296)                                                ERFF    
      ERFF=EXP(-X2)*(0.14845921+X1*0.56418774)/(0.51143728+             ERFF    
     1     X1*0.26277059+X2)                                            ERFF    
C                                                                       ERFF    
C ----BRANCH IF X.LT.0 ,OTHERWISE BRANCH TO RETURN THE                  ERFF    
C     APPROPRIATE FUNCTION.                                             ERFF    
   40 IF(M.GT.1) GO TO 50                                               ERFF    
      GO TO(41,5,42),N                                                  ERFF    
   41 ERFF=1.0-ERFF                                                     ERFF    
      RETURN                                                            ERFF    
   42 ERFF=0.5-0.5*ERFF+0.5                                             ERFF    
      RETURN                                                            ERFF    
C                                                                       ERFF    
C ----BRANCH TO RETURN THE APPROPRIATE FUNCTION                         ERFF    
   50 GO TO (51,52,53),N                                                ERFF    
   51 ERFF=ERFF-1.0                                                     ERFF    
      ERF=ERFF                                                          ERFF    
      RETURN                                                            ERFF    
   52 ERFF=2.0-ERFF                                                     ERFF    
      ERFC=ERFF                                                         ERFF    
      RETURN                                                            ERFF    
   53 ERFF=0.5*ERFF                                                     ERFF    
      PHI=ERFF                                                          ERFF    
      RETURN                                                            ERFF    
      END                                                               ERFF    
