#include <stdio.h>
#include <stdlib.h>
#include "tae.h"
#include "worgen.h"
#include "lasmath.h"


/*
************************************************************************

     linv1f computes the inverse of a real matrix

        a       input, real matrix
                matrix must be stored in Fortran way, i.e., in column
                major order

        n       input, integer, the order of the matrix  a .

        ia      input, integer, row dimension of a in the calling routine

        ainv    output, inverse matrix

     calls  netlib routine kvert


  K. A. Narayanan
  October 1988

************************************************************************
*/
void linv1f
(
    float a[],
    long *n,
    long *ia,
    float ainv[]
)
{
float	*ap,
        *wkarea;               /* work area            */
long	i, j, k, one = 1;

ap = a;
/*
If necessary, copy the matrix to the inverse, and perform 
inversion in place
*/
if (a != ainv)
   {
   for (i = 0, k = 0; i < *n; i++)
      {
      for (j = 0; j < *n; j++) ainv[k+j] = a[k+j];
      k += *ia;
      }
   ap = ainv;
   }
/*
Allocate storage for work area 
Did not return an status code in order to prevent a retrocode */
wkarea = (float *) malloc (2*(*n) * sizeof (float));
if (wkarea == NULL)
   c_errmsg("Error allocating dynamic memory","math-alloc",&one);

/*
Call the corresponding Fortran routine
*/
#ifdef unix
#ifdef _IBMR2
#define kvert_ kvert
#endif
kvert_ (ap, n, ia, wkarea);
#endif

#ifdef ultrix
kvert_ (ap, n, ia, wkarea);
#endif

#ifdef vms
kvert (ap, n, ia, wkarea);
#endif
free(wkarea);
}
