#include "lasmath.h"

/*
************************************************************************

     linv2p computes the determinant and inverse
     of a real symmetric positive definite matrix

        a       input, real, matrix in (n*(n+1)/2) array form
        n       input, integer, the order of the matrix  a

        ainv    output, real, (n*(n+1)/2), the inverse

        det     output, real(2), determinant of original matrix

                determinant = det(1) * 10.0**det(2)
                with  1.0 .le. det(1) .lt. 10.0
                or  det(1) .eq. 0.0 

     calls  netlib routines sppfa and sppdi
        - needs netlib routines sdot, saxpy, sscal

  K. A. Narayanan
  October 1988

************************************************************************
*/
void linv2p
(
    double *a,
    long *n,
    double *ainv,
    float det[2]
)
{
double *ap, *ainvp;
long	i, info, job;

/*
If necessary, copy the matrix to the inverse, and perform 
inversion in place
*/
ap = a;
if (a != ainv)
   {
   ainvp = ainv;
   for (i = 0; i < (*n)*(*n+1)/2; i++) *ainvp++ = *ap++;
   ap = ainv;
   }
/*
Call the netlib inversion routines
*/
job = 11;

#ifdef unix
#ifdef _IBMR2
#define sppfa_ sppfa
#define sppdi_ sppdi
#endif
sppfa_ (ap, n, &info);
sppdi_ (ap, n, det, &job);
#endif

#ifdef ultrix
sppfa_ (ap, n, &info);
sppdi_ (ap, n, det, &job);
#endif

#ifdef vms
sppfa (ap, n, &info);
sppdi (ap, n, det, &job);
#endif

}
