/*******************************************************************************
FUNCTION:	c_lasmsg

PURPOSE:	Write both nonfatal and fatal error messages to the terminal.
		If a fatal error is issued, this function will automatically
		issue the standard "[xxxxx-fatal] Fatal error encountered" after
		printing the specified error message.


Programmer       Date   Modifications
---------------  -----  --------------------------------------------------------
D.Zokaites       01/95  Development
D.Zokaites       04/95  Update to LAS standards.
D.Zokaites       05/95  Modified to write two error messages when a fatal error
			key is supplied.  Applications do not need to issue the
			"[xxxxx-fatal] Fatal error encountered" message any
			longer.
*******************************************************************************/
#include <string.h>
#include <stdarg.h>	/* For variable number of arguments */
#include "las.h"
#include "tae.h"


#define MSGLEN 1024	/* Maximum message length                             */


int c_lasmsg
(
    int fatalflag,
    const char *key,
    const char *format,
    ...
)
{
/*
  Arguments to this function should be:
*/
va_list argumentp;	/* Variable number of arguments for 'format' string   */


char msg[MSGLEN];	/* Message text sent to c_errmsg()                    */
char mkey[MSGLEN];	/* Local copy of the message key                      */
char *ptr;		/* Ptr to a character                                 */

long fatal	= -1;	/* Fatal error flag                                   */
long nonfatal	=  1;	/* Nonfatal error flag                                */


/*
  Get the arguments to this function
*/
va_start(argumentp,format);
va_end(argumentp);


/*
  Create the desired output message
*/
vsprintf (msg, format, argumentp);


/*
  Is the key a default
*/
if ((strcmp(key, "") == 0) || (strcmp(key, " ") == 0))
    strcpy(mkey, "informational");
else
    strcpy(mkey, key);


/*
  If this is a fatal error and the message is the standard "Fatal error
  encountered" message, call c_errmsg with a fatal error key.  In such cases
  there is no need to print an additional standard fatal error message.
*/
if (fatalflag)
    {
    ptr = (char *)strchr(mkey, '-');
    if ((ptr != NULL) && (strcmp(ptr, "-fatal") == 0) &&
	(strcmp(msg, "Fatal error encountered") == 0))
	c_errmsg(msg, mkey, &fatal);
    }


/*
  Print the message using c_errmsg()
*/
c_errmsg(msg, mkey, &nonfatal);


/*
  If this is a fatal error, issue the standard LAS fatal error message
*/
if (fatalflag)
    {
    /*
      The message key should be of the form "function-key".  Transform
      this into the standard key, "function-fatal".
    */
    ptr = (char *)strchr(mkey, '-');
    if (ptr == NULL)
	{
	c_errmsg("Invalid message key", "c_lasmsg-key", &nonfatal);
	strcpy(mkey, "fatal-err");
	}
    else
	strcpy(++ptr, "fatal");


    /*
      Issue the fatal error
    */
    c_errmsg ("Fatal error encountered", mkey, &fatal);
    }

return(E_SUCC);
}
