/*****************************************************************************
FUNCTION:	C_PARSEPAR

PURPOSE:	Build a parameter list by parsing argv[].

Version  Date	Programmer	Modifications
-------  -----	---------------	---------------------------------------------
  1.0    02/98	D.Lloyd		Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under unix

PROJECT:	ADAPS
*****************************************************************************/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "tae.h"
#include "notae.h"
#include "getargv.h"
#include "worgen.h" /* E_SUCC/E_FAIL definition */



int c_parsepar
(
    long argc,			/* IN: argument count			*/
    char *argv[],		/* IN: argument list			*/
    struct PARAMETER *parms,
    char *scmdlist[],
    char list[][CMLEN]		/* OUT: list of input parsed parameters	*/
)

{
char	**eq_ptr;		/* pointers to equal signs in argv[]	*/
char	msg[CMLEN];		/* LAS message string			*/
char	parm_name[32];		/* name of the parameter		*/
char	*ptr;			/* pointer for parsing strings		*/
char	temp_name[32];		/* temporary name of parameter		*/
char	tmpstr[CMLEN];		/* temporary string			*/

long	eqi = 0;		/* index of '=' in tmpstr		*/
long	i;			/* index variable into argv[]		*/
long	j;			/* index variable into return list	*/
long	k;			/* index variable into parameter list	*/
long	numparm;		/* number of parameters in parmlist	*/
long	numscmd;		/* number of subcommands in scmdlist	*/
long	p_type;			/* parameter type			*/
long	positional = TRUE;	/* flag for positional values 		*/


/*
  If no parameters are listed, print a usage message and return failure
------------------------------------------------------------------------*/
if ( argc < 2 )
    {
    sprintf(msg,"Usage: %s <argument list>", argv[0] );
    c_lasmsg(FALSE,"parsepar-noarg",msg);
    return E_FAIL;
    }


/*
  Determine number of parameters and subcommands
-------------------------------------------------*/
i = j = numparm = numscmd = 0;
while ( parms->plist[i++] != NULL ) numparm++;
while ( scmdlist[j++] != NULL ) numscmd++;



/*
  Allocate and assign pointers to equal signs.  We need a couple
  extra NULL pointers to make sure that later checks don't result
  in a segmentation fault.
------------------------------------------------------------------*/
eq_ptr = (char **)malloc( sizeof(char *) * ( argc + 2 ));
for ( i = 0; i < argc; i++ )
    {
    ptr = strchr( argv[i], '=' );
    eq_ptr[i] = ptr;
    }
for ( i = argc; i < argc + 2; i++ )
    eq_ptr[i] = NULL;


/*
  Check for presense of subcommand.  This argument must be in argv[1],
  must begin with a '-' and must equal one of the listed subcommands.
------------------------------------------------------------------------*/
if ( numscmd > 0 )
    {
    ptr = strchr( argv[1], '-' );
    if ( ptr == NULL )
	c_lasmsg( TRUE, "parsepar-parm ", "No subcommand" );
    else
	{
	if ( ptr - argv[1] == 0 )
	    {
	    ptr++;
	    if ( check_name( ptr, scmdlist, numscmd, tmpstr ) != E_SUCC )
		{
		sprintf( msg, "Invalid subcommand: %s", ptr );
		c_lasmsg( TRUE, " ", msg );
		}
	    else
		strcpy ( list[0], tmpstr );
	    }
	}
    }


/*
  Parse argv[] for parameters. If subcommand must be specified,
  skip argv[1].

  The argument strings can be one of the following cases:

    1) a positional value -- as long as name parameters haven't been
       previously specified.
    
    2) a single string:
   
	"infile=[test]in.l1b"

    3) two strings split after the equal sign:

	"infile="  "[test]in.l1b"

    4) two strings split before the equal sign:

	"infile"   "=[test]in.l1b"

    5) three strings split at the equal sign:

	"infile"   "="   "[test]in.l1b"
--------------------------------------------------------------------*/
if ( numscmd > 0 )
    {
    i = 2;		/* index into argv[]				*/
    j = 1;		/* index into return list[]			*/
    k = 0;		/* index into parameter list 			*/
    }
else
    {
    i = 1;		/* index into argv[]				*/
    j = 0;		/* index into return list[]			*/
    k = 0;		/* index into parameter list 			*/
    }

while ( i < argc )	/* while there are still arguments to process	*/
    {
    /*
      Determine which of the cases apply to the current argument
    -------------------------------------------------------------*/
    if ( positional && eq_ptr[i] == NULL )
	{
	if ( eq_ptr[i+1] == NULL )		/* we have a positional	*/
	    {					/* value		*/
	    p_type = 1;
	    }
	else if ( eq_ptr[i+1] != argv[i+1] )	/* it's positional but	*/
	    {					/* next	one is named	*/
	    positional = FALSE;
	    p_type = 1;
	    }
	else if ( eq_ptr[i+1] == argv[i+1] )	/* it's a named param	*/
	    {
	    positional = FALSE;
	    continue;
	    }
	else
	    {					/* error condition	*/
	    ;
	    }
	}

    else
	{
	positional = FALSE;
	/* error */
	if ( eq_ptr[i] == NULL && eq_ptr[i+1] == NULL && eq_ptr[i+2] == NULL )
	    {
	    c_lasmsg(TRUE," ","Positional value may not follow named values");
	    p_type = 0;
	    }
	/* case 2 */
	else if ( eq_ptr[i] != NULL && (char)*(eq_ptr[i] + 1) != '\0' )
	    {
	    p_type = 2;
	    }
	/* case 3 */
	else if ( eq_ptr[i] != NULL && (char)*(eq_ptr[i] + 1) == '\0' )
	    {
	    p_type = 3;
	    }
	/* case 4 */
	else if ( eq_ptr[i] == NULL && (char)*(eq_ptr[i+1] + 1) != '\0' )
	    {
	    p_type = 4;
	    }
	/* case 5 */
	else
	    {
	    p_type = 5;
	    }
	}
    

    /*
      Make sure we can continue -- ie check for too many parameters
      specified or positional value following named values.
    ------------------------------------------------------------------*/
    if ( p_type == 0 )
	{
	i = argc;
	continue;
	}
    else if ( j > numparm )
	{
	c_lasmsg(TRUE," ","Too many parameters specified");
	i = argc;
	continue;
	}

    switch ( p_type )
	{
	case 1 :
	    sprintf( tmpstr, "%s=%s", parms->plist[k], argv[i] );
	    k++;
	    i++;
	    break;
	case 2 :
	    sprintf( tmpstr, "%s", argv[i] );
	    i++;
	    break;
	case 3 :
	    sprintf( tmpstr, "%s%s", argv[i], argv[i+1] );
	    i += 2;
	    break;
	case 4 :
	    sprintf( tmpstr, "%s%s", argv[i], argv[i+1] );
	    i += 2;
	    break;
	case 5 :
	    sprintf( tmpstr, "%s=%s", argv[i], argv[i+2] );
	    i += 3;
	    break;
	}
    

    eqi = (long)( strchr( tmpstr, '=' ) - tmpstr );


    /*
      At this point tmpstr holds a complete parameter specification
      of the form:

		'infile=[./test]input.l1b'\

      and eqi is the index of the '=' sign in tmpstr.  We must now
      make sure the parameter name is valid and expanded to its
      full name.
    ----------------------------------------------------------------*/

    /* copy characters up to, not including the '=' sign to a temporary	*/
    strncpy( temp_name, tmpstr, eqi );
    temp_name[eqi] = 0;			/* and NULL terminate it	*/

    /* if temp_name is valid the expanded name is returned in parm_name	*/
    if ( check_name( temp_name, parms->plist, numparm, parm_name ) == E_SUCC )
	{
	strcpy( list[j], parm_name ); /* copy expanded name to list*/
	ptr = &tmpstr[eqi];		/* get address of '=' sign	*/
	strcat( list[j], ptr );	/* append the rest of the spec	*/
	j++;			/* and update the list index	*/
	}
    else
	{
	sprintf( msg, "Invalid parameter name: %s", temp_name );
	c_lasmsg( TRUE, " ", msg );
	}
    }


free( eq_ptr );

/*
  Check that parameter values match parameter type
---------------------------------------------------*/
if ( check_type( parms, list, numparm ) != E_SUCC )
    return E_FAIL;



return E_SUCC;
}
