/**************************************************************************
*    Function:	valdens
*
*    Purpose:	An array of valid densities based upon the machine
*
*    Program History:
*	Version		Date		Contrib		Change
*	-------		----		-------		------
*	 1.0						Original development
*  	 2.0		05/88		B. Ailts	Valid densities are now
*							initialized from the
*							logical WORTPINFO
*        2.0            11/88           D. Holaren      Validate the densities
*							in this routine from
*							the logical WORTPVDENS
*
*	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		none
*
*	PROJECT: LAS
*
* ALGORITHM DESCRIPTION:
*  Get the logical or environment variable of valid tape densities
*      Parse the valid densities
*  Determine if the density is in the valid list of densities
*  Not a valid density
*	Return an error code
*  Return success
*
* ALGORITHM REFERENCES:
*	The enviornment/logical variable WORTPVDENS contains the valid
*	tape densities of the system.  The valid densities must be
*	seperated in the enviornment/logical variable by spaces and must
*	have DENSCT (defined in tapeio.h) elements with the last element
*	being a "0" for termination.
*	ex)  setenv WORTPVDENS "800 1600 6250 0"
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "notae.h"
#include "worgen.h"
#include "tapeio.h"

int valdens
(
    long density
)

{
long i;
long valid_density[DENSCT];
char *vdens;

#ifdef vms
char tpinfo[CMLEN];
#else
char *tpinfo;
#endif

/*
  Get the logical or environment variable of valid tape densities
*/
#ifdef vms
if (get_logical("WORTPVDENS",tpinfo) == -1)
#else
if ((tpinfo = getenv("WORTPVDENS")) == NULL)
#endif
    return(TAPE_FAIL);
else 
    {
    /*
      Parse the valid densities
    */
    vdens = strtok(tpinfo," ,");
    sscanf(vdens,"%d",&valid_density[0]);
    for (i = 1; i < DENSCT; i++)
	{
        vdens = strtok(NULL," ,");
        sscanf(vdens,"%d",&valid_density[i]);
	}
    }

/*
  Determine if the density is in the valid list of densities
*/
for (i=0; valid_density[i] != 0; i++)
    if (density == valid_density[i])
	break;

/*
  Not a valid density
*/
if (valid_density[i] == 0)
    return(TPOPEN_DENS);

return(TAPE_SUCC);
}
