/*****************************************************************************
NAME                    pxcopy

PURPOSE         Copy an input array to an output array.  The 
                output array will be of the same type as the input array.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        5/7/85   Original development for NEWLAS
        J.REED          11/12/85 Optimized byte portion of routine.
        B.Ailts          12/87   Changed include directory specifications
                                 Use raw 'C' types
                                 Place bridge routines in a seperate file
        B.Ailts          04/88   replaced newlas.a with las.a
        B.Ailts          01/88   Added the VAX memory copy call to help speed up
                                 pxcopy

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system.

PROJECT                         NEWLAS

ALGORITHM 

receive:
        -the input array and its data type
        -the output array 
        -the number of samples of the input array contains.

Multiply the number of samples times the number of bytes that the input array's
data type takes up

for each byte of the input arrays
        copy each byte from each input array the output array
return

ALGORITHM REFERENCES            none
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"
#include <string.h> 

/**
 * las_pixcopy:
 * @frombuf: I: The source buffer of the data to be copied.
 * @tobuf: O: The destination buffer of the data to be copied.
 * @dtype: I: An integer value that represents the type of data
 *            in the source and destination buffers.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 * @ns: I: The number of elements or pixels to be converted or copied.
 * 
 * The las_pixcopy() call is used to copy #ns pixels of #dtype
 * from #frombuf to #tobuf.  
 *
 */
void las_pixcopy
(
    const void *frombuf,    /* I:  source buffer of the data to be copied */
    void *tobuf,            /* O:  destination buffer of the data         */
    int dtype,              /* I:  data type of the buffers               */
    int ns                  /* I:  number of elements or pixels           */
)
{
    static const int datasize[] = {0,1,2,4,4,8};
    int lcnt;

    lcnt = ns * datasize[dtype];
    memcpy (tobuf,frombuf,lcnt);
}


#ifndef NO_OBSOLETE_INTERFACE
/**
 * c_pxcopy:
 * @frombuf: I: The source buffer of the data to be copied.
 * @tobuf: O: The destination buffer of the data to be copied.
 * @dtype: I: An integer value that represents the type of data
 *            in the source and destination buffers.
 *            Legal values include:
 *            EBYTE:  Unsigned byte data,
 *            EWORD:  Signed two byte data,
 *            ELONG:  Four byte floating point data,
 *            EREAL:  Four byte floating point data.
 * @ns: I: The number of elements or pixels to be converted or copied.
 * 
 * The c_pxcopy() call is used to copy #ns pixels of #dtype
 * from #frombuf to #tobuf.  
 *
 * Deprecated.  Use las_pixcopy() instead.
 */
void c_pxcopy
(
    const void *frombuf,    /* I: source buffer of the data to be copied */
    void *tobuf,            /* O: destination buffer of the data         */
    const long *dtype,      /* I: data type of the buffers               */
    const long *ns          /* I: number of elements or pixels           */
)

{
    las_pixcopy (frombuf,tobuf,*dtype,*ns);
}
#endif /* obsolete interfaces */

