/*****************************************************************************
NAME                    pxsamp

PURPOSE         Subsample an array of binary numbers.  The buffer is
                updated.

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        K.GACKE         11/88   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
                Part of the LAS system.

PROJECT         LAS

ALGORITHM

depending on the type of input array values
        call a function for that particular data type 
subsample the array
return

ALGORITHM REFERENCES            none
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"

static void samp_byte
(
    int inns,
    int inc,
    unsigned char *buf
)
{
    unsigned char *ptr1;
    unsigned char *ptr2;

    ptr1 = ptr2 = buf;
    for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
    {
        *ptr1 = *ptr2;
    }
}

static void samp_word
(
    int inns,
    int inc,
    short *buf
)
{
    short *ptr1;
    short *ptr2;

    ptr1 = ptr2 = buf;
    for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
    {
        *ptr1 = *ptr2;
    }
}

static void samp_long
(
    int inns,
    int inc,
    int *buf
)
{
    int *ptr1;
    int *ptr2;

    ptr1 = ptr2 = buf;
    for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
    {
        *ptr1 = *ptr2;
    }
}

static void samp_real
(
    int inns,
    int inc,
    float *buf
)
{
    float *ptr1;
    float *ptr2;

    ptr1 = ptr2 = buf;
    for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
    {
        *ptr1 = *ptr2;
    }
}

static void samp_double
(
    int inns,
    int inc,
    double *buf
)
{
    double *ptr1;
    double *ptr2;

    ptr1 = ptr2 = buf;
    for (; ptr2<buf+inns; ++ptr1,ptr2+=inc)
    {
        *ptr1 = *ptr2;
    }
}

/**
 * las_pixsubsample:
 * @inns: I: The number of samples in the input data buffer.
 * @dtype: I: The data type of the input data buffer.
 *            Legal values include the following:
 *            EBYTE:   Unsigned byte data,
 *            EWORD:   Signed two byte data,
 *            ELONG:   Signed four byte data,
 *            EREAL:   Four byte floating point data.
 * @inc: I: The sample increment value the data buffer is
 *          to be subsampled by.
 * @buf: I/O: The data buffer which is to be subsampled.
 *            The parameter ns in the data buffer, and
 *            parameter dtype buffer.
 *
 * The las_pixsubsample() call is used to subsample values within a data
 * buffer.  Every inc sample, starting with sample 1, is selected
 * from the data buffer.
 *
 */

void las_pixsubsample
(
    int inns,               /* I:  number of samples in input buffer      */
    int dtype,              /* I:  data type (EBYTE,EWORD,ELONG,EREAL)    */
    int inc,                /* I:  sample increment to subsample by       */
    void *buf               /* I/O:  buffer to be subsampled              */
)

{
    switch (dtype)
    {
        case EBYTE:   
            samp_byte(inns,inc,buf);
            break;
        case EWORD:   
            samp_word(inns,inc,(short *)buf);
            break;
        case ELONG:   
            samp_long(inns,inc,(int *)buf);
            break;
        case EREAL:   
            samp_real(inns,inc,(float *)buf);
            break;
        case EDOUBLE: 
            samp_double(inns,inc,(double *)buf);
            break;
    }
}

/**
 * c_pxsamp:
 * @inns: I: The number of samples in the input data buffer.
 * @dtype: I: The data type of the input data buffer.
 *            Legal values include the following:
 *            EBYTE:   Unsigned byte data,
 *            EWORD:   Signed two byte data,
 *            ELONG:   Signed four byte data,
 *            EREAL:   Four byte floating point data.
 * @inc: I: The sample increment value the data buffer is
 *          to be subsampled by.
 * @buf: I/O: The data buffer which is to be subsampled.
 *            The parameter ns in the data buffer, and
 *            parameter dtype buffer.
 *
 * The c_pxsamp() call is used to subsample values within a data
 * buffer.  Every inc sample, starting with sample 1, is selected
 * from the data buffer.
 *
 * Deprecated.  Use las_pixsubsample() instead.
 */
void c_pxsamp
(
    const long *inns,  /* I: number of samples in input buffer            */
    const long *dtype, /* I: data type (EBYTE,EWORD,ELONG,EREAL)          */
    const long *inc,   /* I: sample increment to subsample by             */
    void *buf /* I/O: buffer to be subsampled                    */
)

{
    las_pixsubsample (*inns,*dtype,*inc,buf);
}
