#include <string.h>
#include "las.h"
#include "util.h"
#include "statio.h"
/************************************************************************

NAME:	CLPROMPT

PURPOSE:
	Prompt the user to select subset of classes in the stats
	file.  User may select classes by giving a range, individual
	class numbers, or delete classes

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
   1         30-Nov-87     N. Deodhar
   2         20-Oct-88     K. Narayanan       Implemented paging feature
                                              to handle 999 classes
   2.1	    	07/90	   D. VanderZee		Standardized error handling

***********************************************************************/

void clprompt
(
    FILE **fd,          /* i   stats file descriptor                  */
    long nclass,		/* o   # classes in stats file                */
    long *nuse,			/* i/o # classes selected                     */
    long clind[]	    /* o   class indices                          */
)
{
long	broken,			/* Flag indicating we have broken from a loop */
	do_maxsel,		/* Maximum/selected line is to be output      */
	fatal_err = -1,		/* fatal error flag                           */
	found,			/* Class requested was found                  */
	i,			/* Loop counter                               */
	ib2,			/* Beginning of second range value in string  */
	ic,			/* A class number                             */
	id,			/* Flag indicating - (dash) position in string*/
	ifn,			/* Index into class index array               */
	il,			/* Length of user response                    */
	il1,			/* Length of user response                    */
	inf_err = 1,		/* informational error flag                   */
	irn,			/* Number of classes requested by user        */
	ival1,			/* User entered class number                  */
	ival2,			/* User entered class number                  */
	jj,			/* Loop counter on user entered classes       */
	loc[MAXCLS][2],   	/* Screen location of output                  */
	n,			/* Loop counter                               */
	new_panel,
	num_panels,
	panel,
	save_nuse,		/* Original number of classes requested       */
	term,			/* How the string was terminated              */
	there;			/* Flag indicating a class was found          */

int ival;			/* A user entered class number                */

char	ans[80],		/* Buffer for user's response                 */
	lch,			/* Temporary character used for checking      */
	msg[ERRLEN+1],		/* Output message buffer                      */
	scnams[MAXCLS][17],	/* Names to write on screeen          */
	temp_str[100];		/* Temporary string for outputting to screen  */

static	char	ex[] = {"EXIT"},	/* Exit command                       */
		del[] = {"DELETE"},	/* Delete command                     */
		page[] = {"PAGE"},      /* Page command                       */
		run[] = {"RUN"},	/* Run command                        */
		maxsel[80],
		header[80] = {"Available Class Names"},
		prompt[80] = 
                {"Enter a class number or range, D[elete], P[age], or R[un]"},
		epromt[80] = {"***INVALID RESPONSE*** Enter a class number or range, D[elete], or R[un]"},
		notdun[80] = {"All class names have not been given... do you still want to run? (Y/N)"};

#define PANEL_NCLASS 100

num_panels = (nclass + PANEL_NCLASS - 1)/PANEL_NCLASS;

/*
Display the available class names on the terminal screen
--------------------------------------------------------
*/
ret_wait ();
t_clear ();
t_output (1, 1, header);
save_nuse = *nuse;
ic = 1;
panel = 0;
get_panel (fd, panel, &ic, scnams, loc, nclass);
panel++;
/*
Prompt the user for class numbers
---------------------------------
*/
i = 0;
for (;;)                 
   {
   do_maxsel = TRUE;
   new_panel = FALSE;
   i++;
   for (;;)              
      {
      if (do_maxsel)
         {
         sprintf (maxsel, "Maximum = %-6ld  Selected = %-6ld",save_nuse, (i-1));
	 t_output (1, 40, maxsel);
         t_lclear (23, 1);
	 t_output (23, 1, prompt);
         }
      do_maxsel = TRUE;

      t_lclear (24, 1);

      t_input (24, 1, ans, &term);
      il = (long)strlen (ans);
/*
      Check user input for errors
      ---------------------------
*/
      if (il == 0)
         {
         do_maxsel = TRUE;
         continue;
	 /* ---(GOTO 4)--- */
         }
      if (il > 6)
	 {
         t_lclear (23, 1);
	 t_output (23, 1, epromt);
         do_maxsel = FALSE;
         continue;
	 /* ---(GOTO 5)--- */
	 }
      c_low2up (ans, &il);
/*
      Does user want to delete an entry?
      ----------------------------------
*/
      if ((strncmp (ans, del, il)) == 0)
	 {
	 if (i == 1)
	    {
            t_lclear (23, 1);
	    t_output (23, 1, "No classes defined. Enter a class number or range, D[elete], or R[un]");
	    do_maxsel = FALSE;
	    continue;
	    }
         t_lclear (23, 1);
	 t_output (23, 1, "Enter the class number to be deleted");
	 t_lclear (24, 1);

         t_input (24, 1, ans, &term);
         il1 = (long)strlen (ans);

	 c_low2up (ans, &il);
	 if (il1 == 0 || ((strncmp (ans, ex, il)) == 0))
            {
	    do_maxsel = TRUE;
	    continue;
	    /* ---(GOTO 4)--- */
            }
	 if (il1 > 2)
	    {
            t_lclear (23, 1);
	    t_output (23, 1, epromt);
	    do_maxsel = FALSE;
	    continue;
	    }
         broken = FALSE;
	 for (n = 0; n < il1; n++)
	    {
	    lch = ans[n];
	    if (lch < 48 || lch > 57)
	       {
               t_lclear (23, 1);
	       t_output (23, 1, epromt);
	       do_maxsel = FALSE;
               broken = TRUE;
	       break;
	       }
	    }
         if (broken)
            continue;
         sscanf (ans, "%d", &ival);
         if (ival <= (panel-1)*100 || ival > panel*100 || ival > nclass)
            {
            t_lclear (23, 1);
            t_output (23, 1, epromt);
            do_maxsel = FALSE;
            continue;
            }
	 found = FALSE;
	 for (n = 0; n < i - 1; n++)
	    {
	    if (ival == clind[n])
	       {
	       found = TRUE;
	       ifn = n;
	       }
	    }
	 if (!found)
	    {
            t_lclear (23, 1);
	    t_output (23, 1, "Class not given... enter a class number or range, D[elete] or R[un]");
	    do_maxsel = FALSE;
	    continue;
	    }
	 i--;
	 clind[ifn] = clind[i - 1];
	 clind[i - 1] = 0;
	 t_output (loc[ival - 1][0], loc[ival - 1][1], scnams[ival - 1]);
         do_maxsel = TRUE;
	 continue;
	 }
      else
	 {
	 if (i > *nuse && ((strncmp (ans, run, il)) != 0))
	    {
            t_lclear (23, 1);
	    t_output (23, 1, "No more classes may be given... enter D[elete] or R[un]");
	    do_maxsel = FALSE;
	    continue;
	    }
	 }
/*
      Does user want to exit?
      -----------------------
*/
      if ((strncmp (ans, run, il)) == 0)
	 {
	 if (i == 1)
	    c_errmsg ("Error: No classes were requested for computations", 
                      "clprompt-noclass", &fatal_err);
	 if (i <= *nuse)
	    {
            t_lclear (23, 1);
	    t_output (23, 1, notdun);
	    t_lclear (24, 1);

            t_input (24, 1, ans, &term);
            il = (long)strlen (ans);

	    if (ans[0] == 'N' || ans[0] == 'n')
               {
	       do_maxsel = TRUE;
	       continue;
               }
	    *nuse = i - 1;
	    sprintf (msg, "Number of classes to be used is now reset to: %ld",
                     *nuse);
	    c_errmsg (msg, "clprompt-info", &inf_err);
	    }
	 return;
	 }
/*    Does the user want to see more classes?
      --------------------------------------
*/
      if ((strncmp (ans, page, il)) == 0)
         {
         if (panel == num_panels)
            {
            t_lclear (23, 1);
            t_output (23, 1, 
 "No more pages ... Enter a class number or range, D[elete], or R[un]");
            do_maxsel = FALSE;
            continue;
            }
         else
            {
            get_panel (fd, panel, &ic, scnams, loc, nclass);
            panel++;
            do_maxsel = TRUE;
            new_panel = TRUE;
            i--;
            }
         }
/*
      Make sure user entered digits from 0-9 prior to decoding
      --------------------------------------------------------
*/
      if (new_panel) break;

      id = -1;
      il1 = 0;
      ib2 = 0;
      broken = FALSE;
      for (n = 0; n < il; n++)
	 {
	 if (ans[n] == '-' && id == -1)
	    {
	    id  = n;
	    ib2 = n + 1;
	    il1 = n;
	    }
	 else
	    {
	    lch = ans[n];
	    if (lch < 48 || lch > 57)
	       {
               t_lclear (23, 1);
	       t_output (23, 1, epromt);
	       do_maxsel = FALSE;
               broken = TRUE;
	       break;
	       }
	    }
	 }
      if (broken)
         continue;
      if (id == 0 || id == il - 1)
	 {
         t_lclear (23, 1);
	 t_output (23, 1, "Invalid range...enter a class number or range, D[elete] or R[un]");
         do_maxsel = FALSE;
         continue;
	 }
      if (ib2 == 0) il1 = il;
      sscanf (ans, "%ld", &ival1);
      if (ival1 <= (panel-1)*100 || ival1 > panel*100 || ival1 > nclass)
         {
         t_lclear (23, 1);
         t_output (23, 1, epromt);
         do_maxsel = FALSE;
         continue;
         }
      if (ib2 != 0)
	 {
         sscanf (&ans[ib2], "%ld", &ival2);
	 if (ival2 < ival1 || ival2 > nclass || ival2 > panel*100 || id == n - 1)
	    {
            t_lclear (23, 1);
	    t_output (23, 1, "Invalid range...enter a class number or range, D[elete] or R[un]");
            do_maxsel = FALSE;
            continue;
	    }
	 irn = ival2 - ival1;
	 if ((irn + i) > *nuse)
	    {
            t_lclear (23, 1);
	    t_output (23, 1, "Range will exceed the specified no. of classes...Re-enter");
            do_maxsel = FALSE;
            continue;
	    }
	 for (jj = ival1; jj <= ival2; jj++)
	    {
	    there = FALSE;
	    for (n = 0; n < i - 1; n++)
	       if (jj == clind[n])
		  there = TRUE;
	    if (!there)
	       {
	       clind[i - 1] = jj;
               strcpy (temp_str, scnams[jj - 1]);
               t_highlight (temp_str);
               t_output (loc[jj - 1][0], loc[jj - 1][1], temp_str);
	       i++;
	       }
	    }
	 i--;
	 }
      else
	 {
/*
         Check value against current array and no. of classes in stats file
         ------------------------------------------------------------------
*/
	 if (ival1 == 0 || ival1 > nclass)
	    {
            t_lclear (23, 1);
	    t_output (23, 1, epromt);
            do_maxsel = FALSE;
            continue;
	    }
         broken = FALSE;
	 for (n = 0; n < i - 1; n++)
	    {
	    if (ival1 == clind[n])
	       {
               t_lclear (23, 1);
	       t_output (23, 1, "Class already given...enter a class number or range, D[elete] or R[un]");
               do_maxsel = FALSE;
               broken = TRUE;
               break;
	       }
	    }
         if (broken)
            continue;
/*
         Value given is ok. Save number and highlight name on terminal
         -------------------------------------------------------------
*/
	 clind[i - 1] = ival1;
         strcpy (temp_str, scnams[ival1 - 1]);
         t_highlight (temp_str);
         t_output (loc[ival1 - 1][0], loc[ival1 - 1][1], temp_str);
	 }
      break;
      }
   } 
}

