#include <stdio.h>
#include <math.h>
#include "las.h"
#include "statio.h"

/*
**************************************************************************

NAME: DETERM

FUNCTION: Computes determinant of a matrix

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  1.0         02/25/83  K. A. Narayanan
  2.0         01/11/88  K. A. Narayanan
   2.1	    	07/90	   D. VanderZee		Standardized error handling


**************************************************************************
*/

float determ
(
    double *matrix,
    long norder
)
{
double	*array, *ap, *mp;
double	save;

float	determ;

long	fatal_err = -1;
long	i, j, k;

if ((array = (double *) malloc (norder*norder*sizeof(double))) == NULL)
   c_errmsg ("Error allocating dynamic memory", "determ-alloc", &fatal_err);

/*
copy input matrix
-----------------
*/
ap = array;
mp = matrix;
for (i = 0; i < norder*norder; i++) *(ap++) = *(mp++);

determ = 1;      
for (k = 0; k < norder; k++)
   {
/*
   interchange columns if diagonal element is zero
   -----------------------------------------------
*/
   if (array[k*norder+k] == 0) 
      {
      for (j = k; j < norder; j++)      
         {
         if (array[k*norder+j] != 0) break;
         }
      if (j == norder) { determ = 0; return (determ); }

      for (i = k; i < norder; i++)      
         {
         save = array[i*norder+j];
         array[i*norder+j] = array[i*norder+k];
         array[i*norder+k] = save;      
         }
      determ = -determ;
      }
/*
   subtract row k from lower rows to get diagonal matrix
   -----------------------------------------------------
*/
   determ *= array[k*norder+k];      
   if (k < norder-1)
      {
      for (i = k+1; i < norder; i++)
         for (j = k+1; j < norder; j++)
            {
            array[i*norder+j] -= array[i*norder+k] * array[k*norder+j] / 
                                 array[k*norder+k];      
            } 
      }
   }
free (array);

return (determ);
}
