#include <math.h>
#include "las.h"
#include "statio.h"

/*
**************************************************************************

NAME: EGVCVL

FUNCTION: Compute eigenvalues and eigenvectors of a tridiagonal matrix

          See Fortran SUBROUTINE IMTQL2 in "Matrix Eigensystem Routines 
          EISPACK Guide" by B.T.Smith et al springer vertag (1976)

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         01/11/88  K. A. Narayanan
  1.0         06/15/85  S. Kondal

**************************************************************************
*/

void egvcvl
(
    long nm,             /* dimension of z                             */
    long n,              /* order of z                                 */
    float d[],           /* diagonal elements of tridiagonal matrix    */
    float e[],           /* subdiagonal elements of tridiagonal matrix */
    float *z[],          /* input tridiagonal matrix                   */
    long ierr            /* return status                              */
                         /* output eigen vector matrix                 */
)
{
long	dum, i, j, k, l, m, ii, mml;
float	b, c, f, g, p, r, s, machep;

/*
 ******************machap is a machine dependent parameter specifying
                   the relative pricision of floating point arithmetic


*/
machep = 1.0e-07;
ierr = 0;
if (n  ==  1) return;

for (i = 1; i < n; i++) e[i-1] = e[i];      
e[n-1] = 0.0;

for (l = 0; l < n; l++)
   {
   j = 0;
/*
************ look for small sub-diagonal element *******
*/
   for (dum = 1; dum > 0; )
      {
      for (m = l; m < n; m++)
         {
         if (m == n-1) break;
         if (fabs((double) e[m])  <=  machep * (fabs((double) d[m]) + 
               fabs((double) d[m+1]))) break;
         }

      p = d[l];      
      if (m  ==  l) break;
      if (j++  ==  30) 
         {
/*
          set error -- no convergence to an eigenvalue after 30 
          iterations
*/
         ierr = l;
         return;
         }
/*
******************** form shift ********************
*/
      g = (d[l+1] - p) / (2.0 * e[l]);
      r = sqrt((double) (g*g+1.0));
      g = d[m] - p + e[l] / (g + ((g >= 0) ? fabs((double) r) :
                                             -fabs((double) r)));
      s = 1.0;
      c = 1.0;
      p = 0.0;
      mml = m - l;
/*
*******************for i=m-1 step-1 until l do
*/
      for (ii = 0; ii <  mml; ii++)
         {
         i = m - ii - 1;
         f = s * e[i];
         b = c * e[i];
         if (fabs((double) f)  >=  fabs((double) g)) 
           {
            c = g / f;
            r = sqrt((double) (c*c+1.0));
            e[i+1] = f * r;
            s = 1.0 / r;
            c = c * s;
            }
         else
            {
            s = f / g;      
            r = sqrt((double) (s*s+1.0));
            e[i+1] = g * r;
            c = 1.0 / r;
            s = s * c;
            }
         g = d[i+1] - p;      
         r = (d[i] - g) * s + 2.0 * c * b;
         p = s * r;
         d[i+1] = g + p;
         g = c * r - b;
/*
****************** form vector *******************
*/
         for (k = 0; k < n; k++)
            {
            f = z[k][i+1];
            z[k][i+1] = s * z[k][i] + c * f;
            z[k][i] = c * z[k][i] - s * f;
            }
         }
/*
           
*/
      d[l] = d[l] - p;
      e[l] = g;
      e[m] = 0.0;
      }
   }
}


