#include <stdio.h>
#include <math.h>
#include "las.h"

void tridia
(
    long nm,                /* row dimension of matrices a and z  */
    long n,                 /* order of a                         */
    float *a[],              /* input matrix to be transformed     */
    float d[],               /* diagonal elements of tridiagonal   */
    float e[],               /* subdiagonal elements of tridiognal */
    float *z[]              /* orthognal transformation matrix    */
)
                                   /* produced during reduction to       */
                                   /* tridiagonal form                   */

/*
****************************************************************************

NAME: TRIDIA

FUNCTION: Transform a symetric matrix to tridiagonal form

          See Fortran SUBROUTINE TRED2 in "Matrix Eigensystem Routines 
          EISPACK Guide" by B.T.Smith et al springer vertag (1976)

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
  2.0         01/11/88  K. A. Narayanan
  1.0         06/15/85  S. Kondal

****************************************************************************
*/
{
long	i, j, k, l, ii, jp1;
float	f, g, h, hh, scale;

for (i = 0; i < n; i++)
   for (j = 0; j < n; j++)
      z[i][j] = a[i][j];        

if (n != 1) 
   {
   for (ii = 1; ii < n; ii++)
      {
      i = n - ii;
      l = i - 1;
      h = 0.0;
      scale = 0.0;
      if (l  <=  0)
         e[i] = z[i][l];
      else
         {
         for (k = 0; k <= l; k++)
            scale += fabs((double) z[i][k]);
         if (scale  ==  0.0) 
            e[i] = z[i][l];       
         else
            {
            for (k = 0; k <= l; k++)      
               {
               z[i][k] /= scale;
               h += z[i][k] * z[i][k];
               }

            f = z[i][l];
            g = -((f >= 0) ? fabs(sqrt((double) h)) : -fabs(sqrt((double) h)));
            e[i] = scale * g;
            h = h - f * g;
            z[i][l] = f - g;
            f = 0.0;

            for (j = 0; j <= l; j++)
               {
               z[j][i] = z[i][j] / h;
               g = 0.0;
/*
   **************************form elements of a*u ***********
*/
               for (k = 0; k <= j; k++)
                  g += z[j][k] * z[i][k];      

               jp1 = j + 1;
               if (l  >=  jp1) 
                  for (k = jp1; k <= l; k++)
                     g += z[k][j] * z[i][k];      
/*
   *********************form element of p
*/
               e[j] = g / h;      
               f += e[j] * z[i][j];
               }

            hh = f / (h + h);
/*
   ****************form reduced a **************
*/
            for (j = 0; j <= l; j++)
               {
               f = z[i][j];
               g = e[j] - hh * f;
               e[j] = g;

               for (k = 0; k <= j; k++)
                  z[j][k] -= (f * e[k] + g * z[i][k]);
               }
            }
         }
      d[i] = h;      
      }
   }
d[0] = 0.0;      
e[0] = 0.0;
/*
   ********************** accumulation of transformation matrices ***
*/
for (i = 0; i < n; i++)
   {
   if ( d[i]  !=  0.0)
      {
      for (j = 0; j < i; j++)
         {
         g = 0.0;
 
         for (k = 0; k < i; k++)
            g += z[i][k] * z[k][j];       

         for (k = 0; k < i; k++)
            z[k][j] -= g * z[k][i];
         }
      }
   d[i] = z[i][i];      
   z[i][i] = 1.0;

   if (i  >=  1) 
      for (j = 0; j < i; j++)
         {
         z[i][j] = 0.0;
         z[j][i] = 0.0;
         }
   }
}
