/**************************************************************************
*
*       NAME: c_tpbsr
*
*       PURPOSE:  To backspace a specified number of records
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0		12/87	B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file Added 'c_' to function name
    2.1         6/90    P. Pease        Moved extern errno inside ifdef unix
    2.2         7/90    K.Gacke         output system status codes on error condition.
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
   		9/95 	K.Gacke         DGUX 5.4R3.10 support
    3.4        11/95    K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)
    3.5         1/96    K.Gacke         Port to Solaris


*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Backspace the specified number of records
*         Check for the beginning of the tape after each record backspaced
*         Check for the beginning of the file after each record backspaced
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>

int c_tpbsr
(
    long *tapefd,           /*  tape file descriptor                */
    long *count,            /*  number of records to backspace      */
    long *mode             /*  error mode                          */
)
{
	int  sig_mask;
        long stat,status;
        struct TAPEDESC *drive;
        int i;
        long lcount;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        drive = (struct TAPEDESC *) *tapefd;
        if (*count < 1)
                {
                if (*count == 0)
                   return(TAPE_SUCC);
                *count = 0;
                return(tperror(drive,TP_SKIP,*mode));
                }
	if (drive->flags & NOBSR)
	    {
	    *count = 0;
            return(tperror(drive,TPBSR_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpbsr(drive,count);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        for (i=0; i<*count; i++)
        {
                status = sys$qiow(0,drive->fd, IO$_SKIPRECORD,&iosb,0,0,
                        -1,0,0,0,0,0);
                if ((iosb.status != SS$_NORMAL) &&
                        (iosb.status != SS$_ENDOFFILE))
                        {
                        *count = i;
			drive->err = iosb.status;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPBSR_BSR,*mode));
                        }
/* must test for bot before bof (eof) since VMS says bot is eof */
                if (testbot(drive))
                        {
                        if ( *count > i)
                                {
                                *count = i;
				tpsig_unblk(&sig_mask);
                                return(TAPE_BOT);
                                }
                        }
                if (iosb.status == SS$_ENDOFFILE)
                        {
                        *count = i;
                        lcount = 1;
                        status = c_tpfsr(tapefd, &lcount, mode);
                        if ((status != TAPE_SUCC) && (status != TAPE_EOF))
				{
				tpsig_unblk(&sig_mask);
                                return(TPBSR_BSR);
				}
			tpsig_unblk(&sig_mask);
                        return(TAPE_BOF);
                        }
        }
	tpsig_unblk(&sig_mask);
        return(TAPE_SUCC);
}
#endif

#ifdef  DGUX
#undef unix
#include <ctype.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <errno.h>

int c_tpbsr
(
    long *tapefd,           /*  tape file descriptor                */
    long *count,            /*  number of records to backspace      */
    long *mode              /*  error mode                          */
)
{
	struct mtop mt_com;
        struct mtget mt_status;
        struct TAPEDESC *drive;
	long locstat;
	long rstat;
	int  sig_mask;
        long status;
        int i;
        long lcount;

	rstat = TAPE_SUCC;
        drive = (struct TAPEDESC *) *tapefd;
        if (*count < 1)
            {
            if (*count == 0)
                return(TAPE_SUCC);
            *count = 0;
            return(tperror(drive,TP_SKIP,*mode));
            }
	if (drive->flags & NOBSR)
	    {
	    *count = 0;
            return(tperror(drive,TPBSR_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpbsr(drive,count);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        mt_com.mt_count = 1;
        mt_com.mt_op = MTBSR;
        for (i=0; i<*count; i++)
            {
            status = ioctl(drive->fd, MTIOCTOP, &mt_com);
            if (testbot(drive))
                {
                if (*count > i)
                    {
                    *count = i;
		    tpsig_unblk(&sig_mask);
                    return(TAPE_BOT);
                    }
                }
            if (testeof(drive))
                {
                *count = i;
        	mt_com.mt_count = 1;
        	mt_com.mt_op = MTFSF;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
		    {
		    tpsig_unblk(&sig_mask);
		    return(TAPE_FAIL);
		    }
		tpsig_unblk(&sig_mask);
                return(TAPE_BOF);
                }
            if (status < 0)         /* check status of ioctl call */
                {
                *count = i;
		drive->err = errno;
		tpsig_unblk(&sig_mask);
                return(tperror(drive,TPBSR_BSR,*mode));
                }
            }
	tpsig_unblk(&sig_mask);
        return(rstat);
}
#endif


#ifdef  unix
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>

int c_tpbsr
(
    long *tapefd,           /*  tape file descriptor                */
    long *count,            /*  number of records to backspace      */
    long *mode              /*  error mode                          */
)
{
	struct mtop mt_com;
        struct mtget mt_status;
        struct TAPEDESC *drive;
	long locstat;
	long rstat;
	int  sig_mask;
        long status;
        int i;
        long lcount;

	rstat = TAPE_SUCC;
        drive = (struct TAPEDESC *) *tapefd;
        if (*count < 1)
            {
            if (*count == 0)
                return(TAPE_SUCC);
            *count = 0;
            return(tperror(drive,TP_SKIP,*mode));
            }
	if (drive->flags & NOBSR)
	    {
	    *count = 0;
            return(tperror(drive,TPBSR_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpbsr(drive,count);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
	switch (drive->dev_type)
	    {
	    case SGI_XY:
	    case SOLARIS_SCSI:
		lcount = *count;
		mt_com.mt_count = 1;			/* get status */
                mt_com.mt_op = MTNOP;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return(TPBSR_BSR);
		    }
                if (ioctl(drive->fd, MTIOCGET, &mt_status) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return(TPBSR_BSR);
		    }
                if (*count < mt_status.mt_blkno)	/* determine position */
                    mt_com.mt_count = *count;
                else
                    mt_com.mt_count = *count = mt_status.mt_blkno;
		mt_com.mt_op = MTBSR;			/* backspace records */
                status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                if (status < 0)
                    {
                    drive->err = errno;
		    tpsig_unblk(&sig_mask);
                    return(tperror(drive,TPBSR_BSR,*mode));
                    }
		mt_com.mt_count = 1;			/* get status */
                mt_com.mt_op = MTNOP;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return(TPBSR_BSR);
		    }
                if (ioctl(drive->fd, MTIOCGET, &mt_status) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return(TPBSR_BSR);
		    }
                if ((mt_status.mt_blkno == 0) && (*count < lcount))
		    {
		    if (mt_status.mt_fileno == 0)
                        rstat = TAPE_BOT;
		    else
			rstat = TAPE_BOF;
		    }
		break;
	    default:
                mt_com.mt_count = 1;
                mt_com.mt_op = MTBSR;
                for (i=0; i<*count; i++)
                    {
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    if (testbot(drive))
                        {
                        if (*count > i)
                            {
                            *count = i;
			    tpsig_unblk(&sig_mask);
                            return(TAPE_BOT);
                            }
                        }
                    if (testeof(drive))
                        {
                        *count = i;
                        lcount = 1;
                        locstat = c_tpfsr(tapefd, &lcount, mode);
                        if ((locstat != TAPE_SUCC) && (locstat != TAPE_EOF))
			    {
			    tpsig_unblk(&sig_mask);
                            return(TPBSR_BSR);
			    }
			tpsig_unblk(&sig_mask);
                        return(TAPE_BOF);
                        }
                    if (status < 0)         /* check status of ioctl call */
                        {
                        *count = i;
		        drive->err = errno;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPBSR_BSR,*mode));
                        }
                    }
		break;
	    }
	tpsig_unblk(&sig_mask);
        return(rstat);
}
#endif

