/**************************************************************************
*	
*	NAME: c_tpclos
*
*	PURPOSE:  Close a tape device after read or write.
*
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0					Original development
    2.0		10/87	F. Irani	Allow zero input specification for opt2
					Allows tape close with no eofs written 
					on tape (VMS change only).  
    2.1		12/87	B. Ailts	Change include directory specifications
					Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2		12/89	B. Davis	After the tape is closed, back-space 
					over file marks when the tape was 
					opened for update access also.
    2.3		1/90	B. Davis	Reinserted ifdefs that somehow got 
					destroyed.
    2.4		6/90	K. Gacke	If opened for write access, require at 
					least one tape mark
    2.5         6/90    P. Pease        Moved extern errno inside ifdef sun to 
					fix linker warning in VMS
    2.6         7/90    K.Gacke         output system status code on error condition
    2.8         8/90    B. Davis        Allow EOT to be a valid status to be 
					returned from c_tpweof 
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
    3.3		9/95	K.Gacke		Modified for DGUX 5.4R3.10
    3.4	       11/95    K.Gacke		Modified for IRIX 5.3 (Fugitsu 9-track)
    3.5         1/96    K.Gacke         Port to Solaris
    3.6         12/96   K.Gacke         Solaris 8mm requires to write data
					after tape marks in order to detect
					difference between EOV and EOT.


*							
*
*	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		none
*
*	PROJECT: LAS
*
*	ALGORITHM DESCRIPTION:
*	If the tape was open for write access
*	   Write the number of end-of-file markers as specified by opt2
*	Rewind the tape as specified by opt1
*	Close the tape drive
*
*	ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>

int c_tpclos
(
    long *tapefd,  /* tape drive unit number */
    long *mode,    /* error message option:
                          = handle   = 1 = report errors to user
                          = nohandle = 0 = return error status to
                                           calling program */
    long *opt1,    /* rewind option:
                          = norew   = 0 = do not rewind tape 
                          = rewind  = 1 = rewind tape after close
                          = rewoffl = 2 = rewind tape drive and 
                                          take drive offline after close */
    long *opt2    /* end of file option:
                          = 1, 2 or 3 tape eof marks to write
                            at tape write device close. */
)
{
        extern char *_tpwork;
	struct TAPEDESC *drive;
	long cnt;
	long status;
	long local_mode;
	long neof;
	int  sig_mask;
        char cmd[CMLEN];
        struct
            {
            unsigned short status;
            unsigned short count;
            long devdep;
            } iosb;

	local_mode = 0;
	drive = (struct TAPEDESC *) *tapefd;
#ifdef NETWORK
	if (drive->netflg)			/* close remote tape drive */
	    {
	    status = nt_tpclose(drive,opt1,opt2);
	    return(status);
	    }
#endif
	if ((drive->inout != TREAD) && (drive->eof_flg))
	    {
	    neof = *opt2;
	    if (neof <= 0 || neof > 3)
		return(tperror(drive,TPCLOS_EOF,*mode));
	    status = c_tpweof(tapefd, &neof, &local_mode);
	    if ((status != TAPE_SUCC) && (status != TAPE_EOT))
		    return(tperror(drive,TPCLOS_CLOSE,*mode));
            status = sys$qiow(0,drive->fd, IO$_WRITEVBLK,
                                &iosb,0,0, _tpwork, drive->minbyte,0,0,0,0);
	    status = sys$qiow(0,drive->fd, IO$_SKIPRECORD,&iosb,0,0,
                        -1,0,0,0,0,0);
	    }
	if ( *opt1 > 0)
	    {
	    status = c_tprew( tapefd, &local_mode, opt1);
	    if (status != TAPE_SUCC)
		return(tperror(drive,TPCLOS_CLOSE,*mode));
	    }
	tpsig_blk(&sig_mask);
	status = sys$dassgn(drive->fd);
	if (!(status & 1))
            {
            drive->err = status;
	    tpsig_unblk(&sig_mask);
	    return(tperror(drive,TPCLOS_CLOSE,*mode));
            }
	tpsig_unblk(&sig_mask);
        free((char *)drive);
	free(_tpwork);
	return(TAPE_SUCC);
}  /*  c_tpclos  */
#endif


#ifdef unix
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>

int c_tpclos_dg
(
    long *tapefd,  /* tape drive unit number */
    long *mode,    /* error message option:
                          = handle   = 1 = report errors to user
                          = nohandle = 0 = return error status to
                                           calling program */
    long *opt1,    /* rewind option:
                          = norew   = 0 = do not rewind tape 
                          = rewind  = 1 = rewind tape after close
                          = rewoffl = 2 = rewind tape drive and 
                                          take drive offline after close */
    long *opt2    /* end of file option:
                          = 1, 2 or 3 tape eof marks to write
                            at tape write device close. */
);


int c_tpclos
(
    long *tapefd,  /* tape drive unit number */
    long *mode,    /* error message option:
                          = handle   = 1 = report errors to user
                          = nohandle = 0 = return error status to
                                           calling program */
    long *opt1,    /* rewind option:
                          = norew   = 0 = do not rewind tape 
                          = rewind  = 1 = rewind tape after close
                          = rewoffl = 2 = rewind tape drive and 
                                          take drive offline after close */
    long *opt2    /* end of file option:
                          = 1, 2 or 3 tape eof marks to write
                            at tape write device close. */
)
{
        extern char *_tpwork;
        struct mtop mt_com;
	struct TAPEDESC *drive;
	long status;
	long local_mode;
	long loc_stat;
	long neof;
	int  sig_mask;
	long weof_flg = FALSE;
        char syscmd[CMLEN];

	local_mode = 0;
	drive = (struct TAPEDESC *) *tapefd;
#ifdef NETWORK
	if (drive->netflg)			/* close remote tape drive */
	    {
	    status = nt_tpclose(drive,opt1,opt2);
	    if ((*opt1 == REWOFFL) &&
		((strcmp(drive->dtype,"cd") == 0) ||
		 (strcmp(drive->dtype,"dvd")==0)))
		{
	        sprintf(syscmd,"/usr/local/bin/umntcd_nfs %s",drive->lname);
	        loc_stat = system(syscmd);
	        if (loc_stat != 0)
		    status = TAPE_FAIL;
		}
	    return(status);
	    }
#endif
	if ((strcmp(drive->dtype,"cdr") == 0) || 
	    (strcmp(drive->dtype,"dvdr") == 0))
	    {
	    status = TAPE_SUCC;
            if (*opt1 == REWOFFL)	/* umount only if REWOFFL specified */
	        {
	        status = del_tpmnt(drive);
                free(drive);            /* prevent a memory leak */
		}
	    return(status);
	    }
	if ((strcmp(drive->dtype,"cd") == 0) ||
	    (strcmp(drive->dtype,"dvd") == 0))
	    {
	    status = TAPE_SUCC;
            if (*opt1 == REWOFFL)	/* umount only if REWOFFL specified */
	        {
	        sprintf(syscmd,"/usr/local/bin/op_umntcd %s",drive->lname);
	        loc_stat = system(syscmd);
	        if (loc_stat != 0)
		    status = TAPE_FAIL;
	        status = del_tpmnt(drive);
                free(drive);            /* prevent a memory leak */
		}
	    return(status);
	    }
	if ((drive->inout != TREAD) && (drive->eof_flg))
	    {
	    neof = *opt2;
	    if ((neof <= 0) || (neof > 3))
		return(tperror(drive,TPCLOS_EOF,*mode));
	    switch (drive->dev_type)
		{
		case DG54_GEN:	/* Systems that write tape mark on close */
		    --neof;
		    break;
	        default:
		    break;
	        }
	    status = c_tpweof(tapefd, &neof, &local_mode);
	    if ((status != TAPE_SUCC) && (status != TAPE_EOT))
		return(tperror(drive,TPCLOS_CLOSE,*mode));
	    switch (drive->dev_type)
	        {
		case DG54_GEN:
		case IRIX_EXABYTE:
		case IRIX_SCSI:
		case SOLARIS_SCSI:
		    break;
		default:
		    status = write(drive->fd,_tpwork,drive->minbyte);
                    weof_flg = TRUE;
		    break;
	        }
	    }
	if ( *opt1 > 0)
	    {
	    status = c_tprew( tapefd, &local_mode, opt1);
	    if (status != TAPE_SUCC)
		return(tperror(drive,TPCLOS_CLOSE,*mode));
	    }
	tpsig_blk(&sig_mask);
	status = close(drive->fd);
	if (status < 0)
            {
            drive->err = errno;
	    tpsig_unblk(&sig_mask);
	    return(tperror(drive,TPCLOS_CLOSE,*mode));
            }
	if (*opt1 == REWOFFL)
	    {
	    status = del_tpmnt(drive);
	    if (status != TAPE_SUCC)
	        return(tperror(drive,TPCLOS_CLOSE,*mode));
	    }
	if ((*opt1 == NOREW) && (weof_flg))
	    {
            drive->fd = open(drive->name,0);
	    switch (drive->dev_type)
	        {
		case DG54_GEN:
		case IRIX_EXABYTE:
		case IRIX_SCSI:
		    break;
		case SOLARIS_SCSI:
                    mt_com.mt_count = 1;
                    mt_com.mt_op = MTNBSF;
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    close(drive->fd);
                    break;
	        default:	
/*     
        Systems that do not write EOF on close need to backspace the dummy
        record written above
*/
                    mt_com.mt_count = 1;
                    mt_com.mt_op = MTBSR;
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    close(drive->fd);
                    break;
	        }
	    }
	tpsig_unblk(&sig_mask);
        free((char *)drive);
	free(_tpwork);
	return(TAPE_SUCC);
}  /*  c_tpclos  */

#endif

