/******************************************************************************

NAME:   TPFSF

FUNCTION:
        Forward skip "count" files on tape.

   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1                                   Original development
    2           12/87   K. Gacke      	Modified to detect EOV and EOT properly
    2.1         05/87   B. Ailts      	Changed fron tpfsf to c_tpfsf
                                        Moved bridge routines to a seperate file
                                        Standardized the error messages
    2.2         6/90    K. Gacke      	Modified tape I/O so that if the tape
                                        is opened for write access, it is
                                        actually opened for read/write access
    2.3         7/90    K.Gacke       	output system status code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91    K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
    3.4        11/95    K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)
    3.5         1/96    K.Gacke		Port to Solaris
    3.6		2/98	K.Gacke		Add support for STK D-3 and DLT. 


ALGORITHM:

NOTES:
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>
/****************************************************************************
        On VMS the IO$_SKIPFILE parameter does not detect EOT (the tape runs
        off the end of the reel).  To solve this problem, the TPFSF function
        for VMS calls the TPFSR routine to forward skip the tape.  This method
        may not be as efficient, but at least it works properly.  The vms
        skip file command used to be:
           status = sys$qiow(0,drive->fd,IO$_SKIPFILE,&iosb,0,0,1,0,0,0,0,0);

****************************************************************************/


int c_tpfsf
(
    long *tapefd,           /*  tape file descriptor                */
    long *count,            /*  number of files to forward skip     */
    long *mode             /*  error mode                          */
)
{
	int  sig_mask;
        long status;
        int i;
        struct TAPEDESC *drive;
        int nrec,locmod,locstat;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;



        drive = (struct TAPEDESC *) *tapefd;
/*
*  A count of -1 implies skip to EOD; therefore, on the VAX set skip count
*  to a very large number (assume there are always less than 100000 files
*  on a given tape).
*/
	if (*count == -1)
    	    *count = 100000;
        if (*count < 1)
            {
            if (*count == 0)
               return(TAPE_SUCC);
            *count = 0;
            return(tperror(drive,TP_SKIP,*mode));
            }
	if (drive->flags & NOFSF)
	    {
	    *count = 0;
            return(tperror(drive,TPFSF_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpfsf(drive,count);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        locmod = NO_HANDLE;
        status = TAPE_SUCC;
	switch (drive->dev_type)
	    {
	    case VAX_FSF:	/* execute SKIPFILE function */
                for (i=0; (status==TAPE_SUCC) && (i<*count); i++)
		    {
           	    locstat = sys$qiow(0,drive->fd,IO$_SKIPFILE,&iosb,
			0,0,1,0,0,0,0,0);
	            status = tpchk(tapefd);
	            switch (status)
		        {
		        case TAPE_EOF:
		            status = TAPE_SUCC;
		            break;
		        case TAPE_EOV:
		            *count = i + 1;
		            break;
		        case TAPE_EOT:
		            *count = i;
		            break;
		        }
                    if (iosb.status != SS$_NORMAL) 
                        {
                        *count = i;
			drive->err = iosb.status;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPFSF_FSF,*mode));
                        }
	 	    }
		break;
	    case VAX_GEN:	/* execute FSR to skip files */
	    default:
                for (i=0; (status==TAPE_SUCC) && (i<*count); i++)
		    {
                    nrec = 2000000;
                    for (locstat=TAPE_SUCC; locstat==TAPE_SUCC; )
                        locstat = c_tpfsr(&drive,&nrec,&locmod);
                    switch (locstat)
                        {
                        case TAPE_EOF:
                            status = TAPE_SUCC;
                            break;
                        case TAPE_EOV:
                            *count = i + 1;
                            status = TAPE_EOV;
                            break;
                        case TAPE_EOT:
                            *count = i;
                            status = TAPE_EOT;
                            break;
                        default:
			    tpsig_unblk(&sig_mask);
                            return(tperror(drive,locstat,*mode));
                            break;
                        }
	 	    }
		break;
	    }
	tpsig_unblk(&sig_mask);
        return(status);
}
#endif

#ifdef unix
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>

int tpchk();

int c_tpfsf
(
    long *tapefd,           /*  tape file descriptor                */
    long *count,            /*  number of files to forward skip     */
    long *mode             /*  error mode                          */
)
{
struct TAPEDESC *drive;
struct mtop mt_com;
long i;
long locmod,locstat;
long nrec;
int  sig_mask;
long status;

drive = (struct TAPEDESC *) *tapefd;
if (*count == 0)
    return(TAPE_SUCC);
/*
*  A count of -1 implies skip to EOD
*/
if ((*count < 0) && (*count != -1))
    {
    *count = 0;
    return(tperror(drive,TP_SKIP,*mode));
    }
if (drive->flags & NOFSF)
    {
    *count = 0;
    return(tperror(drive,TPFSF_NO,*mode));
    }
#ifdef NETWORK
if (drive->netflg)			/* remote tape drive */
    {
    status = nt_tpfsf(drive,count);
    return(status);
    }
#endif
tpsig_blk(&sig_mask);
locmod = HANDLE;
status = TAPE_SUCC;
/*
*  If count is -1, forward skip to end of media.  It is assumed that the
*  end of media has two tape marks; therefore, we back space one file which
*  should leave the tape in position to start writing next file.
*
*  DG does not support EOM operation; therefore, assume less then 2000000
*  files on the tape.
*/
if (*count == -1)
    {
#ifdef DGUX
    mt_com.mt_count = 2000000;
    mt_com.mt_op = MTFSF;
#else
    mt_com.mt_count = 1;
    mt_com.mt_op = MTEOM;
#endif
    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
    if ((locstat < 0) && (errno == ENOSPC))
        {
        *count = 0;
        status = TAPE_EOT;
        }
    if (locstat < 0)
        {
	drive->err = errno;
	tpsig_unblk(&sig_mask);
        return(tperror(drive,TPFSF_FSF,*mode));
        }
    locstat = tpchk(tapefd);
    if (locstat == TAPE_EOF)
        locstat = TAPE_SUCC;
    return(locstat);
    }
else
    {
    mt_com.mt_count = 1;
    mt_com.mt_op = MTFSF;
    switch (drive->dev_type)
	{
	case IRIX_EXABYTE: 
	case IRIX_SCSI:
	case SOLARIS_SCSI: 
/*
*  The 9-track tape drives do not detect EOV/EOT properly all of the time
*  if the skip is done within a single ioctl; therefore, we issue an ioctl
*  for each file count and check to see if we are at EOV/EOT.
*/
		if (strcmp(drive->dtype,"9-track") == 0)
		    {
		    for (i=0; (status==TAPE_SUCC) && (i<*count); i++)
			{
    		        mt_com.mt_count = 1;
    		        mt_com.mt_op = MTFSF;
		        locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
                        status = tpchk(tapefd);
                        if (status == TAPE_EOF)
                            status = TAPE_SUCC;
			}
		    }
/*
*  For all other tape devices, issue a single ioctl to FSF (this also engages
*  the fast speed locate on the new tape devices (9840, 9940, SDLT, LTO, etc).
*  Once the skip is completed, check for EOV/EOT.
*/
		else
		    {
    		    mt_com.mt_count = *count;
    		    mt_com.mt_op = MTFSF;
		    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    status = tpchk(tapefd);
                    if (status == TAPE_EOF)
                        status = TAPE_SUCC;
		    }
		break;
	default:
		for (i=0; (status==TAPE_SUCC) && (i<*count); i++)
                    {
                    nrec = 2000000;
                    for (locstat=TAPE_SUCC; locstat==TAPE_SUCC; )
                        locstat = c_tpfsr(tapefd,&nrec,&locmod);
                    switch (locstat)
                        {
                        case TAPE_EOF:
                            status = TAPE_SUCC;
                            break;
                        case TAPE_EOV:
                            *count = i + 1;
                            status = TAPE_EOV;
                            break;
                        case TAPE_EOT:
                            *count = i;
                            status = TAPE_EOT;
                            break;
                        default:
                            tpsig_unblk(&sig_mask);
                            return(tperror(drive,locstat,*mode));
                        }
                    }
                break;
	}
    }
tpsig_unblk(&sig_mask);
return(status);
}
#endif
