/**************************************************************************
*
*       NAME: c_tpfsr
*
*       PURPOSE:  To foward space a record
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0		12/87	B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.1         6/89    K.Gacke         Port to SUN 4
    2.2         6/90    K. Gacke        Modified tape I/O so that if the tape 
					is opened for write access, it is 
					actually opened for read/write access
    2.3         7/90    K.Gacke         Port to Prime, output system error code 
					on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
    3.4        11/95    K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)
    3.5         1/96    K.Gacke         Port to Solaris


*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       #ifdef vms
*       for each record to be skipped
*          Read to end of record
*          Check for the end-of-file
*             Check for the end-of-volume
*       #endif
*       #ifdef unix
*       Read the number of records specified
*       Check for the end-of-file
*       #endif
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>

int c_tpfsr
(
    long *tapefd,           /*  tape file descriptor                */
    long *count,            /*  number of records to forward skip   */
    long *mode             /*  error mode                          */
)
{
	int  sig_mask;
	long rstat;
        long status;
        int  i,stat;
        struct TAPEDESC *drive;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        drive = (struct TAPEDESC *) *tapefd;
        if (*count < 1)
            {
            if (*count == 0)
               return(TAPE_SUCC);
            *count = 0;
            return(tperror(drive,TP_SKIP,*mode));
            }
	if (drive->flags & NOFSR)
	    {
	    *count = 0;
            return(tperror(drive,TPFSR_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    rstat = nt_tpfsr(drive,count);
	    return(rstat);
	    }
#endif
	tpsig_blk(&sig_mask);
        rstat = TAPE_SUCC;
        for (i=0; (i<*count)&&(rstat==TAPE_SUCC); i++)
        	{
                status = sys$qiow(0,drive->fd, IO$_SKIPRECORD,&iosb,0,0,
                        1,0,0,0,0,0);
                if ((iosb.status != SS$_NORMAL) &&
                        (iosb.status != SS$_ENDOFFILE))
                    {
                    *count = i;
		    drive->err = iosb.status;
		    tpsig_unblk(&sig_mask);
                    return(tperror(drive,TPFSR_FSR,*mode));
                    }
                if (testeot(drive))
                    {
                    *count = i;
                    rstat = TAPE_EOT;
                    }
                if (iosb.status == SS$_ENDOFFILE)
                    {
                    *count = i;
	            rstat = tpchk(tapefd);
                    }
        	}
	tpsig_unblk(&sig_mask);
        return(rstat);
}
#endif

#ifdef unix
#include <ctype.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <sys/errno.h>

extern int errno;

int c_tpfsr
(
    long *tapefd,           /*  tape file descriptor                */
    long *count,            /*  number of records to forward skip   */
    long *mode             /*  error mode                          */
)
{
        extern char *_tpwork;
	int  sig_mask;
	long bufsz;
	long locmod;
        long status;
        int i,devstat;
        struct TAPEDESC *drive;
        struct mtop mt_com;

        drive = (struct TAPEDESC *) *tapefd;
        if (*count < 1)
            {
            if (*count == 0)
               return(TAPE_SUCC);
            *count = 0;
            return(tperror(drive,TP_SKIP,*mode));
            }
	if (drive->flags & NOFSR)
	    {
	    *count = 0;
            return(tperror(drive,TPFSR_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpfsr(drive,count);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        mt_com.mt_count = 1;
        mt_com.mt_op = MTFSR;
        locmod = HANDLE;
        status = TAPE_SUCC;
	switch (drive->dev_type)
	    {
	    case IRIX_EXABYTE:
	    case IRIX_SCSI:
	    case SOLARIS_SCSI:
        	mt_com.mt_count = *count;
        	mt_com.mt_op = MTFSR;
                devstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
		if (devstat == 0)
		    status = TAPE_SUCC;
		else
		    status = tpchk(tapefd);
		break;
	    case SGI_XY:	/* unable to test EOF with MTFSR command */
	    default:
		bufsz = drive->maxbyte;
                for (i=0; (status==TAPE_SUCC) && (i<*count); i++)
                    {
		    status = c_tpread(tapefd,(unsigned char *)_tpwork,&bufsz,&locmod);
		    if (status > 0)
		        status = TAPE_SUCC;
		    else
		        *count = i;
		    }
		break;
            }                                       /* end for stmt         */
	tpsig_unblk(&sig_mask);
        return(status);
}
#endif
