/*******************************************************************************
NAME:    c_tplblpr

PURPOSE: Using the system host name, determine which label printer should
         be used. 

Programmer       Date   Reason
---------------  -----  --------------------------------------------------------
D. Etrheim       07/94	New development to allow a label printer to be
			specified for a system.
D. Etrheim       07/94	New development to allow a label printer
J. Fenno	 08/95	Changed WORCONFIG to LASTABLES.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
         Must be run under TAE.

PROJECT: LAS
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#endif


int c_tplblpr
(
    char *printer		/* output printer name 				*/
)
{
FILE   *file_ptr;	/* file pointer to input file 			*/

char host_name[CMLEN];	/* host system name				*/
char *wor_ptr;		/* pointer to env variable			*/
char filenm[CMLEN];	/* file name that contains label printer names  */
char temp[CMLEN];	/* temp working string				*/
long one = 1;		/* constant of one				*/
char *ptr;		/* pointer to printer name			*/
long found = FALSE;	/* loop control variable			*/
unsigned int len;	/* length of string				*/

/* Get the directory of the file
--------------------------------*/
wor_ptr = getenv("LASTABLES");
if (wor_ptr == NULL)
   {
   c_errmsg("Global variable LASTABLES not defined","tplblpr-env",&one);
   return(E_FAIL);
   }

/* Add the file name
--------------------*/
sprintf(filenm,"%s/initlbl.config",wor_ptr);

/* Open the file.  Apply a default printer it there are problems.
----------------------------------------------------------------*/
if ((file_ptr = fopen(filenm, "r")) == NULL)
     {
     c_errmsg("Error opening initlbl.config file","tplblpr-open",&one);
     strcpy(printer, "labels-t");
     }
   else
      {
      /* Read each line of the file until a match is found with 
         the system name or the default printer is found.
      ----------------------------------------------------------*/
      gethostname(host_name, CMLEN);
      len = strlen(host_name);
      while (!found)
         {
         fscanf(file_ptr, "%s", temp);
         if ((strncmp(temp,"default",7) == 0)   ||
             (strncmp(host_name,temp,len) == 0))
            {
            found = TRUE;
            ptr = strchr(temp,':');
            *ptr++;
            strcpy (printer,ptr);
            }
         }
      fclose(file_ptr);
      }

return(E_SUCC);
}
