/*********************   TAPE I/O Subsystem   *********************************
FUNCTION:	tplswr

PURPOSE:	Write a label services record to the associated file on tape.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
           01/89    D. Hollaren		Modified version of LSWRIT to write
					a label services file to tape.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
Truncate the key if it's to long
Go to the end of the file
Write the header information
Write the character information
Write the data information

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef vms
#include <file.h>
#else
#include <sys/file.h>
#endif

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#endif

#include "vll.h"
#include "tapeio.h"

int c_tplswr
(
    long *unit,		/* tape unit number			(input)       */
    char *key,		/* key of record to be written		(input)       */
    long *clen,		/* number of char bytes to be written   (input)       */
    long *dlen,		/* number of data bytes to be written   (input)       */
    char *cbuf,		/* character buffer to be written       (input)       */
    char *dbuf,		/* data buffer to be written		(input)       */
    char *dtype		/* datatype of data record (I2,B,R4,...)(input)       */
)
{
long count;		/* Looping variable				      */
long buflen;		/* Length of output Label services record buffer      */
long out_byte;		/* Number of bytes actually written to tape	      */
long nonfatal = 1;	/* Non-fatal error code				      */
long mode = HANDLE;	/* Handle tape write error flag			      */

char header[HDRL];	/* Header from the LS record			      */
char len[LENL];		/* Length of the character and data protions	      */

unsigned char *outbuf;	/* Output buffer				      */
unsigned char *tptr;	/* Temporary pointer to the output buffer	      */

/*
  Set the size of the LS output record buffer
*/
buflen = HDRL + *clen + *dlen;

/*
  Allocate memory for the output buffer
*/
outbuf = (unsigned char *) malloc(buflen);
if (outbuf == NULL)
    {
    c_errmsg("Error writing header to label services file on tape",
	     "labserv-malloc",&nonfatal);
    return(TAPE_FAIL);
    }

/*
  Check to make sure key isn't to big
*/
if (strlen(key) >= KEYL)
    {
    c_errmsg("Key length too long.  Key has been truncated.","labserv-key",
		&nonfatal);
    *(key + KEYL - 1) = '\0';
    }

/*
  Create the length protion of the header
*/
if (*clen > 0)
   sprintf(len,"%-d/%-d",*clen,*dlen);
else
   sprintf(len,"%-d",*dlen);

/*
  Generate the header protion of label services record
*/
sprintf(header,"%-*s%-*s%-*s",LENL,len,TYPL,dtype,KEYL-1,key);

/*
  Copy the header into the output buffer
*/
tptr = (unsigned char *) header;
for (count = HDRL; count--; outbuf++,tptr++)
	*outbuf = *tptr;

/*
  Copy the character record into the output buffer
*/
tptr = (unsigned char *) cbuf;
for (count = *clen; count--; outbuf++, tptr++)
	*outbuf = *tptr;

/*
  Copy the data record into the output buffer
*/
tptr = (unsigned char *) dbuf;
for (count = *dlen; count--; outbuf++, tptr++)
	*outbuf = *tptr;

outbuf -= buflen;

out_byte = c_tpwrit(unit,outbuf,&buflen,&mode);
if (out_byte == TAPE_EOT)
    return(TAPE_EOT);
else if (out_byte != buflen)
    {
    c_errmsg("Error writing header to label services file on tape",
	     "labserv-tpwrite",&nonfatal);
    return(TAPE_FAIL);
    }

free(outbuf);

return(TAPE_SUCC);
}
