/**************************************************************************
*    Function:  tpreop
*
*    Purpose:   Reopen a tape drive.
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name.
                                        Replaces DESC arguments with char. 
					strings. Standardized error msg.
    2.1         6/89    K.Gacke         Port to SUN 4
    2.2         2/90    B.Ailts         Checked for allocation errors
    2.3         6/90    K. Gacke        Modified tape I/O so that if the tape is
                                        opened for write access, it is actually
					opened for read/write access
    2.4         7/90    K.Gacke         output system status code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Open the tape drive
*       Rewind tape is specified
*       Skip the number of files specified
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#endif

int c_tpreop
(
    long *unit,             /*  tape unit number                    */
    long *skip,             /*  number of files to skip             */
    long *mode,             /*  error mode                          */
    long *opt1,             /*  rewind flag                         */
    char *dens,             /*  tape density                        */
    long *inout,            /*  tape access                         */
    char *tapename         /*  tape drive name                     */
)
{
        long local_mode;
	int  sig_mask;
        long status;
        struct TAPEDESC *drive;

        drive = (struct TAPEDESC *)malloc(sizeof(struct TAPEDESC));
	if (drive == NULL)
           {
           tperror(drive,TPOPEN_OPEN,*mode);
           return(TAPE_FAIL);
           }
	drive->eof_flg = FALSE;
        *unit = (long)drive;
        local_mode = ~HANDLE;
	status = tprdconf(drive,tapename,dens,*inout);
        if (status != TAPE_SUCC)
            return(tperror(drive,status,HANDLE));
	if (!drive->netflg)			/* open local tape drive */
	    {
	    tpsig_blk(&sig_mask);
            status = open_tape(drive,TREOPEN);
	    tpsig_unblk(&sig_mask);
	    }
	else					/* open remote tape drive */
	    {
#ifdef NETWORK
	    status = nt_tpopen(drive,tapename);
#else
	    status = TP_NONET;
#endif
	    }
        if (status != TAPE_SUCC)
           {
           tperror(drive,status,*mode);
           return(TPOPEN_OPEN);
           }
        if ((*opt1 == REWIND) && (las_is_tape(drive)))
                {
                status = c_tprew(unit, &local_mode, opt1);
                if (status != TAPE_SUCC)
                        {
                        tperror(drive,TPOPEN_REW,*mode);
                        return(TPOPEN_REW);
                        }
                }
        if (*skip > 0)
                {
                status = c_tpfsf(unit, skip, &local_mode);
                if ((status != TAPE_SUCC) && (status != TAPE_EOV))
                        {
                        tperror(drive,TPOPEN_SKIP,*mode);
                        return(TPOPEN_SKIP);
                        }
                }
	tplen(drive);
        return(TAPE_SUCC);
}
