/**************************************************************************
*    Function:  tprew
*
*    Purpose:   Rewind a tape.

   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
    3.3         2/96    K.Gacke         STK 3490 bug fix to wait for the
      					rewind to complete.
    3.4		9/96	K.Gacke		DLT support on the SGI, STK 3490 bug
					fix no longer needed -- STK updated
					the firmware on the drive
    3.5		12/96	K.Gacke		Solaris 8mm seems to error on tape
					rewind.  If so, a second rewind is
					issued (if second rewind succeeds, a
					good status is returned).
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       VMS
*       Rewind the tape using system call
*       If rewind-offline flag is true
*         take offline
*
*       UNIX
*       Close the tape which will rewind the tape
*       Open the tape
*       Check the rewind-offline flag
*       If true take offline
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>

int c_tprew
(
    long *tapefd,           /*  tape unit number                    */
    long *mode,             /*  error mode                          */
    long *opt1             /*  rewind offline flag                 */
)
{
	int  sig_mask;
        long status;
        struct TAPEDESC *drive;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        drive = (struct TAPEDESC *) *tapefd;
#ifdef NETWORK
	if (drive->netflg)
	    {
            status = nt_tprew(drive,opt1);
	    tplen(drive);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        if (*opt1 == REWOFFL)
            tpdism(drive->name);
        else
            {
            status = sys$qiow(0,drive->fd,IO$_REWIND,&iosb,0,0,0,0,0,0,0,0);
            if (!(iosb.status & 1))
		{
		drive->err = iosb.status;
		tpsig_unblk(&sig_mask);
                return(tperror(drive,TPREW_REW,*mode));
		}
            }
	tpsig_unblk(&sig_mask);
	tplen(drive);
        return(TAPE_SUCC);
}
#endif

#ifdef unix
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <unistd.h>

int c_tprew
(
    long *tapefd,           /*  tape unit number                    */
    long *mode,             /*  error mode                          */
    long *opt1             /*  rewind offline flag                 */
)
{
	int  sig_mask;
        long status;
        struct TAPEDESC *drive;
        struct mtop mt_com;

        drive = (struct TAPEDESC *) *tapefd;
#ifdef NETWORK
	if (drive->netflg)
	    {
            status = nt_tprew(drive,opt1);
	    tplen(drive);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        if (*opt1 == REWOFFL)
            {
	    switch (drive->dev_type)
		{
		case IRIX_EXABYTE:
                    mt_com.mt_count = 1;
#ifdef sgi
                    mt_com.mt_op = MTUNLOAD;
#endif /*  sgi  */
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    if (status < 0)
		        {
		        drive->err = errno;
		        tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPREW_OFFL,*mode));
		        }
		    break;
		default:
                    mt_com.mt_count = 1;
                    mt_com.mt_op = MTOFFL;
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    if (status < 0)
		        {
		        drive->err = errno;
		        tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPREW_OFFL,*mode));
		        }
		    break;
		}
            }
        else
            {
            mt_com.mt_count = 1;
            mt_com.mt_op = MTREW;
            status = ioctl(drive->fd, MTIOCTOP, &mt_com);
/*
*  If running on Solaris, the rewind may fail in which we will try a
*  second time.  Not known why this situation is encountered, but it
*  seems to work.
*/
	    if ((status < 0) && (drive->dev_type == SOLARIS_SCSI))
		{
#ifdef DEBUG
		printf("c_tprew: Solaris tape rewind failed, trying again\n");
#endif
		sleep(10);
                status = ioctl(drive->fd, MTIOCTOP, &mt_com);
		}
            if (status < 0)
		{
		drive->err = errno;
		tpsig_unblk(&sig_mask);
                return(tperror(drive,TPREW_REW,*mode));
		}
            }
	drive->eof_flg = FALSE; 
	tpsig_unblk(&sig_mask);
	tplen(drive);
        return(TAPE_SUCC);
}
#endif

