/**************************************************************************
*    Function:  tpweof
*
*    Purpose:  To write the end-of-file marker
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         11/89   K.Gacke         Check for EOT
    2.3         7/90    K.Gacke         output system error code on error condition
    2.5         8/90    B.Davis         check for EOT before writing eofs also.
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       do the number of times specified ty count
*          write an end-of-file mark
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>

int c_tpweof
(
    long *tapefd,           /*  tape unit number                    */
    long *count,            /*  number of end-of-file marks to be written */
    long *mode             /*  error mode                          */
)
{
        int i;
	int  sig_mask;
        long status;
        struct TAPEDESC *drive;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        drive = (struct TAPEDESC *) *tapefd;
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpweof(drive,count);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        for (i=0; i < *count; i++)
        {
                if (testeot(drive))
			{
			tpsig_unblk(&sig_mask);
                        return(TAPE_EOT);
			}
                status = sys$qiow(0,drive->fd, IO$_WRITEOF,
                        &iosb,0,0,0,0,0,0,0,0);
                if (!(iosb.status & 1))
			{
			drive->err = iosb.status;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPWEOF_WEOF,*mode));
			}
                if (testeot(drive))
			{
			tpsig_unblk(&sig_mask);
			drive->eof_flg = FALSE;
                        return(TAPE_EOT);
			}
		tp_setlen(drive,TAPE_EOF);
        }
	tpsig_unblk(&sig_mask);
        return(TAPE_SUCC);
}
#endif

#ifdef unix
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>

int c_tpweof
(
    long *tapefd,           /*  tape unit number                    */
    long *count,            /*  number of end-of-file marks to be written */
    long *mode             /*  error mode                          */
)
{
        int i;
	int  sig_mask;
        long status;
        struct TAPEDESC *drive;
        struct mtop mt_com;

        drive = (struct TAPEDESC *) *tapefd;
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpweof(drive,count);
	    for (i=0; i<*count; i++)
	        tp_setlen(drive,TAPE_EOF);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        mt_com.mt_count = 1;
        mt_com.mt_op = MTWEOF;
        for (i=0; i < *count; i++)
            {
            if (testeot(drive))
		{
		tpsig_unblk(&sig_mask);
		drive->eof_flg = FALSE;
                return(TAPE_EOT);
		}
#ifdef DGUX
	    mt_com.mt_pad1 = 0;
#endif
            status = ioctl(drive->fd, MTIOCTOP, &mt_com);
            if (status < 0)
		{
		drive->err = errno;
		tpsig_unblk(&sig_mask);
                return(tperror(drive,TPWEOF_WEOF,*mode));
		}
       	    if (testeot(drive))
		{
		tpsig_unblk(&sig_mask);
		drive->eof_flg = FALSE;
                return(TAPE_EOT);
		}
	    tp_setlen(drive,TAPE_EOF);
            }
	tpsig_unblk(&sig_mask);
        return(TAPE_SUCC);
}
#endif
