/**************************************************************************
*    Function:  tpwrit
*
*    Purpose:  To write to  a tape.
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/92	K.Gacke		If physical EOT is detected, write 3 
					tape marks to indicate software EOT.
    3.3		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
    3.4        11/95    K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)
    3.5         1/96    K.Gacke         Port to Solaris
    3.5		12/96	K.Gacke		If write error is encountered, set the
					eof flag to true so that tape marks are
					not written on close.


*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Check buffer size, so that it is within limits of the system
*       Write the buffer to the tape
*       Check the return status for end of tape
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>

#include "worgen.h"
#include "tapeio.h"
#include "pixman.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>

int c_tpwrit
(
    long *tapefd,           /*  Tape unit number                    */
    unsigned char *buffer,  /*  buffer                              */
    long *bufsz,            /*  buffer size -- number of bytes      */
    long *mode             /*  error mode                          */
)
{
	long cnt;
	int  sig_mask;
        long status;
        struct TAPEDESC *drive;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        drive = (struct TAPEDESC *) *tapefd;
        if ((*bufsz < drive->minbyte) || (*bufsz > drive->maxbyte))
                return(tperror(drive,TPWRIT_SIZE,*mode));
#ifdef NETWORK
	if (drive->netflg)
	    {
	    if (drive->tape_len < 0)
		{
		cnt = 3;
	        status = nt_tpweof(drive,&cnt);
		status = TAPE_EOT;
		}
            else
		status = nt_tpwrit(drive,buffer,*bufsz,FALSE);
	    tp_setlen(drive,status);
	    return(status);
	    }
#endif
	drive->eof_flg = TRUE;
	tpsig_blk(&sig_mask);
        status = sys$qiow(0,drive->fd, IO$_WRITEVBLK,
                                &iosb,0,0, buffer, *bufsz,0,0,0,0);
        if (testeot(drive))
	    	{
		tpsig_unblk(&sig_mask);
		drive->eof_flg = FALSE;
		tp_eot(drive);
                return(TAPE_EOT);
		}
        if (iosb.status != 1)
        {
                if (iosb.status == SS$_ENDOFVOLUME)
                        status = TAPE_EOV;
                else
                {
			drive->err = iosb.status;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPWRIT_WRITE,*mode));
                }
        }
        else
        {
		tpsig_unblk(&sig_mask);
                status = iosb.count;
        }
	tpsig_unblk(&sig_mask);
	tp_setlen(drive,status);
        return(status);
}
#endif

#ifdef unix
#include <errno.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <unistd.h>

void tp_eot
(
	struct TAPEDESC *drive
);


int c_tpwrit
(
    long *tapefd,           /*  Tape unit number                    */
    unsigned char *buffer,  /*  buffer                              */
    long *bufsz,            /*  buffer size -- number of bytes      */
    long *mode             /*  error mode                          */
)
{
	extern char *_tpwork;
	long dtype;
	int  sig_mask;
        long status;
	unsigned char *bufptr;
        struct TAPEDESC *drive;

        drive = (struct TAPEDESC *) *tapefd;
        if ((*bufsz < drive->minbyte) || (*bufsz > drive->maxbyte))
            return(tperror(drive,TPWRIT_SIZE,*mode));
#ifdef NETWORK
	if (drive->netflg)
	    {
	    if (drive->tape_len < 0)
		{
		long cnt = 3;
	        status = nt_tpweof(drive,&cnt);
		status = TAPE_EOT;
		}
            else
                status = nt_tpwrit(drive,buffer,*bufsz,FALSE);
	    tp_setlen(drive,status);
	    return(status);
	    }
#endif
	drive->eof_flg = TRUE;
	switch (drive->dev_type)
	    {
	    case IRIX_SCSI: 		/* buffer needs to be word aligned */
	    case IRIX_EXABYTE:
		bufptr = buffer;
		if ((int) bufptr % 4 != 0)
		    {
		    bufptr = (unsigned char *) _tpwork;
		    dtype = EBYTE;
		    c_pxcopy(buffer,_tpwork,&dtype,bufsz);
		    }
		break;
	    default:
        	bufptr = buffer;
	 	break;
	    }
	tpsig_blk(&sig_mask);
        status = write(drive->fd, bufptr, *bufsz);
	tpsig_unblk(&sig_mask);
	switch (drive->dev_type)
	    {
	    case SGI_XY:
	    case IRIX_SCSI:
	    case SOLARIS_SCSI:
		if (status == 0)		/* physical EOT detected */
		    {
		    drive->eof_flg = FALSE;
                    status = TAPE_EOT;
		    tp_eot(drive);
		    goto xit;
		    }
		break;
	    case IRIX_EXABYTE:
		break;
	    default:
        	if (testeot(drive))
		    {
		    drive->eof_flg = FALSE;
                    status = TAPE_EOT;
		    tp_eot(drive);
		    goto xit;
		    }
	 	break;
	    }
        if (status < 0)
            {
            if (errno == ENOSPC)
		{
                status = TAPE_EOT;
		tp_eot(drive);
		goto xit;
		}
	    drive->eof_flg = FALSE;
	    drive->err = errno;
            return(tperror(drive,TPWRIT_WRITE,*mode));
            }
xit:
	tp_setlen(drive,status);
        return(status);
}
#endif


/*
	TP_EOT is called when the physical end of tape is reached.  It 
	writes three tape marks to indicate software end of tape.  The
	software end of tape is more reliable in detecting EOT when the
	tape is read on a different drive and/or computer system.

	On VMS, the current record was written to tape; therefore, the
	tape is backspaced one record prior to writing the three tape
	marks.
*/
void tp_eot
(
	struct TAPEDESC *drive
)
{
#ifdef vms
long mode;
long status;
struct
    {
    unsigned short status;
    unsigned short count;
    long devdep;
    } iosb;

mode = HANDLE;
status = sys$qiow(0,drive->fd,IO$_SKIPRECORD,&iosb,0,0,-1,0,0,0,0,0);
if ((iosb.status != SS$_NORMAL) && (iosb.status != SS$_ENDOFFILE))
    {
    drive->err = iosb.status;
    tperror(drive,TPBSR_BSR,mode);
    }
status = sys$qiow(0,drive->fd, IO$_WRITEOF,&iosb,0,0,0,0,0,0,0,0);
status = sys$qiow(0,drive->fd, IO$_WRITEOF,&iosb,0,0,0,0,0,0,0,0);
status = sys$qiow(0,drive->fd, IO$_WRITEOF,&iosb,0,0,0,0,0,0,0,0);
#endif

#ifdef unix
struct mtop mt_com;

/*********
mt_com.mt_count = 3;
mt_com.mt_op = MTWEOF;
locstat = ioctl(drive->fd,MTIOCTOP,&mt_com);
**********/
mt_com.mt_count = 1;
mt_com.mt_op = MTWEOF;
ioctl(drive->fd,MTIOCTOP,&mt_com);
ioctl(drive->fd,MTIOCTOP,&mt_com);
ioctl(drive->fd,MTIOCTOP,&mt_com);
#endif
}
