/**************************************************************************
*    Function:  chktap
*
*    Purpose:   check tape status code
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    2.3         7/90    K.Gacke         Port to Prime, output system error 
					code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               prime
*
* ALGORITHM DESCRIPTION:
*  Check status code of tape operation.
*
* ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "tapeio.h"

#ifdef prime
#include <errd.ins.cc>

int chktap
(
    int tpnum,
    int instr,
    unsigned short statv[],
    int code
)
{
int rstat;
short buffno;
short loc_instr;
short loc_code;
short loc_statv[6];
short nhw;

buffno = nhw = 0;
loc_instr = TP_STATUS;
for (; statv[0]==1; )        /* wait for previous command to complete */
    {
    t$mt(tpnum,buffno,nhw,loc_instr,loc_statv,loc_code);
    }
switch (code)
    {
    case e$nass:
         printf("tape error: device not assigned\n");
         rstat = TAPE_FAIL;
         break;
    case e$ivcm:
         printf("tape error: invalid command\n");
         rstat = TAPE_FAIL;
         break;
    case e$dnct:
         printf("tape error: device specified in physical-unit not\n");
         printf("tape error:    connected, or no controller\n");
         rstat = TAPE_FAIL;
         break;
    case e$bnwd:
         printf("tape error: Invalid number of halfwords (0 <= nhw < 6144)\n");
         rstat = TAPE_FAIL;
         break;
    default:
         rstat = TAPE_SUCC;
         break;
    }
if (statv[1] & TPST_PARITY)
    {
    printf("tape error: tape parity error -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
if (statv[1] & TPST_RUNAWAY)
    {
    printf("tape error: tape runaway -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
if (statv[1] & TPST_CRC)
    {
    printf("tape error: CRC error -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
if (statv[1] & TPST_LRC)
    {
    printf("tape error: LRC error -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
if (statv[1] & TPST_SMALL)
    {
    printf("tape error: False gap or more record data than was requested\n");
    printf("tape error:    during a read -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
if (statv[1] & TPST_ERROR)
    {
    printf("tape error: Uncorrectable error -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
if (statv[1] & TPST_WRITE)
    {
    printf("tape error: Read after write detected on write -- statv[1] = %x\n",
         statv[1]);
    rstat = TAPE_FAIL;
    }
if ((!(statv[1] & TPST_READY)) && (instr != TP_REWIND))
    {
    printf("tape error: Transport NOT ready -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
if ((!(statv[1] & TPST_ONLINE)) && (instr != TP_REWIND))
    {
    printf("tape error: Tape drive NOT online -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
/*
if (statv[1] & TPST_WRPROT)
    {
    printf("tape error: Tape write protected -- statv[1] = %x\n",statv[1]);
    rstat = TAPE_FAIL;
    }
*/
if (statv[1] & TPST_OVERRUN)
    {
    printf("tape error: DMX overrun or no formatter -- statv[1] = %x\n",
         statv[1]);
    rstat = TAPE_FAIL;
    }
if (statv[1] & TPST_RWDONE)
    {
    printf("Rewind complete\n");
    }
return(rstat);
}
#endif
