/**************************************************************************
* Function:	istape
*
* Purpose:	Check that a specified tape drive name is a tape device.
* 
* Programmer: 	N. Deodhar/SAR          10-20-87
* 
*	
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
*
*
*	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*		none
*
*	PROJECT: LAS
*
*	ALGORITHM DESCRIPTION:
* 	This routine checks to see if the device name passed is the name of a 
*	tape drive.  
*
*	NOTE:  This routine is specific to VMS.
*
*	ALGORITHM REFERENCES:
*
******************************************************************************/
#ifdef vms

/*  Include c library 
---------------------*/
#include <stdio.h>

/*  Include system library 
--------------------------*/
#include <dvidef.h>
#include <dcdef.h>
#include <ssdef.h>

/* Include world files 
----------------------*/
#include "worgen.h"
#include "desc.h"
#include "tapeio.h"

int istape
(
    char devnam_buf[]		/* Device name to check       */
)
{
long	devclass,		/* Device class returned by system services   */
	istat,			/* System service status                      */
	ret_length;		/* Length of system service returned data     */

struct DESC	devnam;		/* String structure for device name           */

struct  ITMLST			/* Item list for system service request       */
   {
        short	bufl,
		itmcode;
        long	*badr,
		*rlenadr,
		null;
   } itmlst;


/* Set up string descriptor
---------------------------*/
devnam.addr   = devnam_buf;
devnam.type   = TYPE_STRING ;
devnam.class  = CLASS_ARRAY ;
devnam.length = strlen (devnam_buf);

/* Set up item list
-------------------*/
devclass = 0;
itmlst.bufl    = 4;
itmlst.itmcode = DVI$_DEVCLASS;
itmlst.badr    = &devclass;
itmlst.rlenadr = &ret_length;
itmlst.null    = 0;

/* Call system service routine to check device class
----------------------------------------------------*/
istat = sys$getdviw (0, 0, &devnam, &itmlst, 0, 0, 0, 0);

/* Return tape drive status: TRUE or FALSE.
-------------------------------------------*/
return (devclass == DC$_TAPE);

}

#endif
