/**************************************************************************
*
*       NAME: opentape
*
*       PURPOSE:  Open the tape drive
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       VMS
*       Allocate the tape drive
*       Open the tape drive
*       Check the status of the open for write lock
*
*       UNIX
*       Open the tape drive
*       Check the status of the open for write lock
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>
#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include "desc.h"

int open_tape
(
    struct TAPEDESC *drv,           /*  tape descriptor             */
    long opnflg
)
{
        short status;
        struct DESC devdsc;

        status = alloc_mount(drv);
        if (status != TAPE_SUCC)
           return(status);
        if (drv->inout != TREAD)
               {
               if (!canwrit(drv->name))               /* if no write ring   */
                   {                                  /* dismount the drive */
                   tpdism(drv->name);                 /* and return status  */
                   return(TPOPEN_WLOCK);              /* to try again.      */
                   }
               }
        devdsc.length = strlen(drv->name);
        devdsc.addr = drv->name;
        status = sys$assign(&devdsc, &drv->fd, 0, 0);
        if (!(status & 1))
                return(TPOPEN_ASN);
        return(TAPE_SUCC);
}
#endif

#ifdef unix
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <unistd.h>

int open_tape
(
    struct TAPEDESC *drive,
	long opnflg
)
{
        struct mtop mt_com;
        struct mtget mt_status;
	int errcnt;
	int loc_stat;
        int status;
	char cmd[256];

        status = TAPE_SUCC;
        status = add_tpmnt(drive,opnflg);
        if (status != TAPE_SUCC)
           return(status);
	errcnt = 0;
	if ((strcmp(drive->dtype,"cdr") == 0) ||
	    (strcmp(drive->dtype,"dvdr") == 0))
	    {
	    return(status);
	    }
	if ((strcmp(drive->dtype,"cd") == 0) ||
	    (strcmp(drive->dtype,"dvd") == 0))
	    {
	    sprintf(cmd,"/usr/local/bin/op_mntcd %s",drive->lname);
	    loc_stat = system(cmd);
	    if (loc_stat != 0)
		status = TAPE_FAIL;
	    return(status);
	    }
retry:
        drive->fd = open(drive->name, drive->inout);
        if (drive->fd == -1)
           {
	   if (errcnt < 5)
		{
		++errcnt;
		sleep(1);
		goto retry;
		}
           switch (errno)
              {
              case EIO:
                if (drive->inout == TREAD)
                   status = TPOPEN_OFFLINE;
                else
                   status = TPOPEN_WLOCK;
                break;
              default:
                status = TPOPEN_OPEN;
                break;
              }
           }
	if (status == TAPE_SUCC)
	    {
/**********
            mt_com.mt_count = 1;
            mt_com.mt_op = MTNOP;
            if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                return(TAPE_FAIL);
*********/
            if (ioctl(drive->fd, MTIOCGET, &mt_status) < 0)
                return (TAPE_FAIL);
#ifdef sgi
	    if (!(mt_status.mt_dposn & MT_ONL))
		{
                status = TPOPEN_OFFLINE;
		close(drive->fd);
		}
            if ((drive->inout != TREAD) && (mt_status.mt_dposn & MT_WPROT))
		{
                status = TPOPEN_WLOCK;
		close(drive->fd);
		}
#endif 
	    }
#ifdef sgi
	if (errno == ENOSPC)
	    {
            mt_com.mt_count = 1;
            mt_com.mt_op = MTREW;
            if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                return(TAPE_FAIL);
	    errno = 0;
	    }
#endif /* sgi */
/*        get_tpblksz(drive); */
	if (status != TAPE_SUCC)
	    (void) del_tpmnt(drive);
        return(status);
}
#endif

#if 0
/*
* Since the tape is re-positioned with the GETBLKINFO, we are
* currently hardcoding the min/max blocksize.
*
* If the tape was closed on a tape mark and reopened, for some
* reason the GETBLKINFO command positions ahead of the tape mark.  
* The code below attempt to re-position the tape correctly.
*/
static int get_tpblksz
(
	struct TAPEDESC *drive
)
{
#ifdef sgi
extern char *_tpwork;
struct mtblkinfo mt_blkinfo;
struct mtop mt_com;


if (ioctl(drive->fd, MTIOCGETBLKINFO, &mt_blkinfo) < 0)
    return (TAPE_FAIL);
if (drive->maxbyte > 0)
    drive->maxbyte = mt_blkinfo.maxblksz;
if (drive->minbyte > 0)
    drive->minbyte = mt_blkinfo.minblksz;
#endif
return(TAPE_SUCC);
}
#endif
