/**************************************************************************
*	
*	NAME: add_tpmnt, del_tpmnt, ls_tpmnt
*
	PURPOSE:  
	To add or delete entry from tape mount table (for unix only).  The
	tape mount file is necessary to allow operator's to know which drives
	are free (8mm tapes on SGI, and DG do not eject, 9-track DG
	does not unload).

   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    3.1		3/91	K.Gacke		initial development, add Silicon 
					Graphics
    3.2		3/91	K. Gacke	Add drive type field to tape.config
    3.3         8/94    D. Etrheim      Change 777 to 666 on file open
    4.0		9/97	K. Gacke	Rebuilt to increase MAXDRIV
    5.0		6/98	K. Gacke	File lock the /tmp/tape.mnt file

*
*	PROJECT: LAS
*
*	ALGORITHM DESCRIPTION:
*
*	ALGORITHM REFERENCES:
*
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/file.h>

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#include "util.h"
#endif

#include "tapeio.h"
#include "local_protos.h"


void loc_tpmsg
(
	char *emsg
);

int get_mntfil
(
	char *tpmnt_fil
);

int rdtpmnt
(
	int fd,
	char *tpmnt_fil,
	struct TMNT tmnt[],
	long *tmnt_cnt
);

int wrtpmnt
(
	int fd,
	char *tpmnt_fil,
	struct TMNT tmnt[],
	long *tmnt_cnt
);

int rdtpconf
(
	struct TPREC tprec[],
	long *tprec_cnt
);

static long nonfatal = 1;

#ifdef unix

int add_tpmnt
(
	struct TAPEDESC *drive,
	long opnflg             /* TOPEN or TREOPEN */
)
{
#ifdef NETWORK
extern char *TPREMOTE;
extern char *TPRUSER;
#endif /* NETWORK */
struct TMNT tmnt[MAXDRIV];
int  fd;
long cnt;
long crflg;
long prot;
long status;
long tmnt_cnt;
long tmnt_cnt_free;
char emsg[CMLEN];
char lname[CMLEN];
char tpmnt_fil[CMLEN];
char *ptr;
char uname[CMLEN];

status = get_mntfil(tpmnt_fil);
if (status != TAPE_SUCC)
    return(status);
if (access(tpmnt_fil,0) == 0)
    crflg = FALSE;
else
    crflg = TRUE;
prot = 0777;
fd = open(tpmnt_fil,O_RDWR | O_CREAT,prot);
if (fd == -1)
    {
    perror("add_tpmnt");
    sprintf(emsg,"Error opening tape mount table file %s",tpmnt_fil);
    tp_errmsg(emsg,"tapeio-mount",&nonfatal);
    return(TAPE_FAIL);
    }
#ifdef sgi
if (flock(fd,LOCK_EX) == -1)
    {
    perror("add_tpmnt");
    sprintf(emsg,"Error locking tape mount table file %s",tpmnt_fil);
    tp_errmsg(emsg,"tapeio-mount",&nonfatal);
    return(TAPE_FAIL);
    }
#endif
status = rdtpmnt(fd,tpmnt_fil,tmnt,&tmnt_cnt);
if (status != TAPE_SUCC)
    {
    close(fd);
    return(status);
    }
#ifdef NETWORK
    if (TPREMOTE != NULL)
	{
        strcpy(uname,TPRUSER);
	}
    else
	{
        getusern(uname);
	}
#else
    getusern(uname);
#endif /*  NETWORK  */
ptr = strchr(drive->lname,'@');
if (ptr != NULL)
    strcpy(lname,drive->lname);
else
    sprintf(lname,"%s@%s",drive->thost,drive->lname);
tmnt_cnt_free = -1;
for (cnt=0; cnt<tmnt_cnt; cnt++)
    {
    if (strcmp(tmnt[cnt].lname," ") == 0)
	tmnt_cnt_free = cnt;
    if (opnflg == TREOPEN)
	{
        if ((strcmp(lname,tmnt[cnt].lname) == 0) &&	/* already allocated */
	    (strcmp(uname,tmnt[cnt].user) == 0))	/* to user, return   */
	    {						/* good status */
	    close(fd);
	    return(TAPE_SUCC);
	    }
	}
    if ((strcmp(lname,tmnt[cnt].lname) == 0) &&
	(strcmp(drive->thost,tmnt[cnt].host) == 0))
	{
	sprintf(emsg,"Tape drive %s already allocated by user %s",
	    tmnt[cnt].lname,tmnt[cnt].user);
	tp_errmsg(emsg,"tapeio-mount",&nonfatal);
	loc_tpmsg(emsg);
	close(fd);
	return(TAPE_FAIL);
	}
    }
if (tmnt_cnt_free == -1)
    {
    tmnt_cnt_free = tmnt_cnt;
    ++tmnt_cnt;
    }
if (tmnt_cnt_free >= MAXDRIV)
    {
    sprintf(emsg,"Max number of tape drives to be allocated is %d",MAXDRIV);
    tp_errmsg(emsg,"tapeio-mount",&nonfatal);
    loc_tpmsg(emsg);
    close(fd);
    return(TAPE_FAIL);
    }
strcpy(tmnt[tmnt_cnt_free].lname,lname);
#ifdef NETWORK
    if (TPREMOTE != NULL)
	{
        strcpy(tmnt[tmnt_cnt_free].host,TPREMOTE);
        strcpy(tmnt[tmnt_cnt_free].user,TPRUSER);
	}
    else
	{
        strcpy(tmnt[tmnt_cnt_free].host,drive->thost);
        strcpy(tmnt[tmnt_cnt_free].user,uname);
	}
#else
    strcpy(tmnt[tmnt_cnt_free].host,drive->thost);
    strcpy(tmnt[tmnt_cnt_free].user,uname);
#endif /*  NETWORK  */
strcpy(tmnt[tmnt_cnt_free].name,drive->name);
strcpy(tmnt[tmnt_cnt_free].dtype,drive->dtype);
status = wrtpmnt(fd,tpmnt_fil,tmnt,&tmnt_cnt);
if (status != TAPE_SUCC)
    {
    close(fd);
    return(status);
    }
close(fd);
if (crflg)
    {
    status = chmod(tpmnt_fil,0666);
    if (status == -1)
        {
        perror("add_tpmnt");
        sprintf(emsg,"Error changing mode to 0666 for file %s",tpmnt_fil);
	tp_errmsg(emsg,"tapeio-mount",&nonfatal);
        }
    }
return(TAPE_SUCC);
}

int del_tpmnt
(
	struct TAPEDESC *drive
)
{
struct TMNT tmnt[MAXDRIV];
int  fd;
long cnt;
long flg;
long status;
long tmnt_cnt;
char emsg[CMLEN];
char lname[CMLEN];
char *ptr;
char tpmnt_fil[CMLEN];

status = get_mntfil(tpmnt_fil);
if (status != TAPE_SUCC)
    return(status);
fd = open(tpmnt_fil,O_RDWR | O_CREAT,0777);
if (fd == -1)
    {
    perror("del_tpmnt");
    sprintf(emsg,"Error opening tape mount table file %s",tpmnt_fil);
    tp_errmsg(emsg,"tapeio-mount",&nonfatal);
    return(TAPE_FAIL);
    }
#ifdef sgi
if (flock(fd,LOCK_EX) == -1)
    {
    perror("del_tpmnt");
    sprintf(emsg,"Error locking tape mount table file %s",tpmnt_fil);
    tp_errmsg(emsg,"tapeio-mount",&nonfatal);
    return(TAPE_FAIL);
    }
#endif
status = rdtpmnt(fd,tpmnt_fil,tmnt,&tmnt_cnt);
if (status != TAPE_SUCC)
    {
    close(fd);
    return(status);
    }
flg = FALSE;
ptr = strchr(drive->lname,'@');
if (ptr != NULL)
    strcpy(lname,drive->lname);
else
    sprintf(lname,"%s@%s",drive->thost,drive->lname);
for (cnt=0; cnt<tmnt_cnt; cnt++)
    {
    if (strcmp(lname,tmnt[cnt].lname) == 0)
	{
	flg = TRUE;
        strcpy(tmnt[cnt].lname," ");
        strcpy(tmnt[cnt].host," ");
        strcpy(tmnt[cnt].user," ");
        strcpy(tmnt[cnt].name," ");
        strcpy(tmnt[cnt].dtype," ");
	break;
	}
    }
if (!flg)
    {
    sprintf(emsg,"Drive %s not allocated",lname);
    tp_errmsg(emsg,"tapeio-mount",&nonfatal);
    loc_tpmsg(emsg);
    close(fd);
    return(TAPE_FAIL);
    }
status = wrtpmnt(fd,tpmnt_fil,tmnt,&tmnt_cnt);
if (status != TAPE_SUCC)
    {
    close(fd);
    return(status);
    }
close(fd);
return(TAPE_SUCC);
}


int ls_tpmnt(void)
{
struct TMNT tmnt[MAXDRIV];
struct TPREC tprec[MAXDRIV];
int  fd;
long cnt;
long cnt1;
long found;
long status;
long tmnt_cnt;
long tprec_cnt;
char devnam[CMLEN];
char emsg[CMLEN];
char lname[CMLEN];
char *ptr;
char tpmnt_fil[CMLEN];

status = get_mntfil(tpmnt_fil);
if (status != TAPE_SUCC)
    return(status);
tmnt_cnt = 0;
if (access(tpmnt_fil,0) == 0)
    {
    fd = open(tpmnt_fil,O_RDONLY,0777);
    if (fd == -1)
        {
        perror("ls_tpmnt");
        sprintf(emsg,"Error opening tape mount table file %s",tpmnt_fil);
        tp_errmsg(emsg,"tapeio-mount",&nonfatal);
        return(TAPE_FAIL);
        }
#ifdef sgi
    if (flock(fd,LOCK_SH) == -1)
        {
        perror("ls_tpmnt");
        sprintf(emsg,"Error locking tape mount table file %s",tpmnt_fil);
        tp_errmsg(emsg,"tapeio-mount",&nonfatal);
        return(TAPE_FAIL);
        }
#endif
    status = rdtpmnt(fd,tpmnt_fil,tmnt,&tmnt_cnt);
    if (status != TAPE_SUCC)
	{
	close(fd);
        return(status);
	}
    close(fd);
    }
tprec_cnt = 0;
status = rdtpconf(tprec,&tprec_cnt);
printf("\n");
printf("  DRIVE NAME      DRIVE TYPE      HOST      USER      DEVICE\n");
for (cnt=0; cnt<tprec_cnt; cnt++)
    {
    found = FALSE;
    for (cnt1=0; cnt1<tmnt_cnt; cnt1++)
	{
	ptr = strchr(tmnt[cnt1].lname,'@');
	if (ptr == NULL)
	    strcpy(lname,tmnt[cnt1].lname);
	else
	    strcpy(lname,ptr+1);
	if (strcmp(lname,tprec[cnt].drive) == 0)
	    {
	    found = TRUE;
	    break;
	    }
	}
    if (found)
	{
        printf("  %-16.16s%-16.16s%-10.10s%-10.10s%s\n",tmnt[cnt1].lname,
	    tmnt[cnt1].dtype,tmnt[cnt1].host,tmnt[cnt1].user,tmnt[cnt1].name);
	}
    else
	{
	sprintf(devnam,"%s%s",tprec[cnt].devnam,tprec[cnt].drvnmr[DEN_DEF]);
        printf("  %-16.16s%-16.16s%-10.10s%-10.10s%s\n",tprec[cnt].drive,
	    tprec[cnt].dtype,tprec[cnt].host,"----",devnam);
	}
    }
printf("\n");
return(TAPE_SUCC);
}

int get_mntfil
(
	char *tpmnt_fil
)
{
strcpy(tpmnt_fil,"/tmp/tape.mnt");
return(TAPE_SUCC);
}

int rdtpconf
(
	struct TPREC tprec[],
	long *tprec_cnt
)
{
	struct TPREC tprec_tmp;
	FILE *fp;
	long cnt;
	long len;
	long status;
	char buf[1024];
	char lhost[CMLEN];
	char *ptr;
	char tpconfig[CMLEN];

	if (gethostname(lhost,CMLEN) != 0)
    	    return(E_FAIL);
	len = (long)strlen(lhost);
	c_up2low(lhost,&len);
	if (tpgetconf(tpconfig) == TAPE_FAIL)
	    return(TAPE_FAIL);
	fp = fopen(tpconfig,"r");
	if (fp == NULL)
    	    {
    	    printf("Error opening tape configuration file %s\n",tpconfig);
    	    return(TAPE_FAIL);
    	    }
	for (status=E_SUCC; status!=E_EOF; )
    	    {
    	    init_tprec(&tprec_tmp);
	    for (cnt=0; ; cnt++)
	        {
	        status = tpconfig_getfield(fp,buf);
	        if ((status == E_EOL) || (status == E_EOF))
		    break;
	        switch (cnt)
	            {
	            case 0:		/* first field is logical drive name */
	                strcpy(tprec_tmp.drive,buf);
		        break;
	            case 1:
	                strcpy(tprec_tmp.host,buf);
		        break;
	            case 2:
	                strcpy(tprec_tmp.dtype,buf);
		        break;
	            case 3:
		        break;
	            case 4:
	                strcpy(tprec_tmp.devnam,buf);
		        break;
	            default:
		        ptr = strchr(buf,'=');
		        if (ptr != NULL)
		            get_density(buf,&tprec_tmp);
		        else
		            get_tpflag(buf,&tprec_tmp);
		        break;
	            }
	        }
	    if (strcmp(lhost,tprec_tmp.host) == 0)
		{
		strcpy(tprec[*tprec_cnt].drive,tprec_tmp.drive);
		strcpy(tprec[*tprec_cnt].host,tprec_tmp.host);
		strcpy(tprec[*tprec_cnt].dtype,tprec_tmp.dtype);
		strcpy(tprec[*tprec_cnt].devnam,tprec_tmp.devnam);
		for (cnt=0; cnt<DENSCT; cnt++)
		    strcpy(tprec[*tprec_cnt].drvnmr[cnt],tprec_tmp.drvnmr[cnt]);
		(*tprec_cnt)++;
		}
            }
        fclose(fp);
        return(TAPE_SUCC);
}

int rdtpmnt
(
	int fd,
	char *tpmnt_fil,
	struct TMNT tmnt[],
	long *tmnt_cnt
)
{
long status;
char emsg[CMLEN];

lseek(fd,0,0);
for (*tmnt_cnt=0; *tmnt_cnt<MAXDRIV; (*tmnt_cnt)++)
    {
	status = read(fd,&tmnt[*tmnt_cnt],sizeof(struct TMNT));
    if (status == 0)
	    break;
    if (status < 0)
        {
        perror("rdtpmnt");
        sprintf(emsg,"Error reading tape mount table file %s",tpmnt_fil);
        tp_errmsg(emsg,"tapeio-mount",&nonfatal);
        return(TAPE_FAIL);
        }
    }
return(TAPE_SUCC);
}

int wrtpmnt
(
	int fd,
	char *tpmnt_fil,
	struct TMNT tmnt[],
	long *tmnt_cnt
)
{
long cnt;
long status;
char emsg[CMLEN];

lseek(fd,0,0);
for (cnt=0; cnt<*tmnt_cnt; cnt++)
    {
    status = write(fd,&tmnt[cnt],sizeof(struct TMNT));
    if (status != sizeof(struct TMNT))
        {
	perror("wrtpmnt");
	sprintf(emsg,"Error writing tape mount table file %s",tpmnt_fil);
	tp_errmsg(emsg,"tapeio-mount",&nonfatal);
	}
    }
return(TAPE_SUCC);
}

void loc_tpmsg
(
	char *emsg
)
{
char loc_msg[CMLEN];

sprintf(loc_msg,"\n\n%s\n",emsg);
tpmsg(loc_msg);

}
#endif
