/**************************************************************************
*
*       NAME: c_tpbsf
*
*       PURPOSE:  To backspace a file
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0        	12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file Added 'c_' to function name
    2.1         8/89    K. Gacke        Modified unix version to test BOT prior 
					to check of BSF command.
    2.2         6/90    K.GAcke         output system status codes on error
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
    3.4        11/95    K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)

*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Backspace the number of files as specified by count
*       Check for beginning of the tape after each backspace
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>


#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>

int c_tpbsf
(
    long *tapefd,          /*  tape file descriptor                */
    long *count,           /*  number of files to backspace        */
    long *mode             /*  Error mode                          */
)
{
	int sig_mask;
        long status, stat;
        struct TAPEDESC *drive;
        int i;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        drive = (struct TAPEDESC *) *tapefd;
/* First we should parse our parameters */
        if (*count < 0)
            {
            *count = 0;
            return(tperror(drive,TP_SKIP,*mode));
            }
	if (drive->flags & NOBSF)
	    {
	    *count = 0;
            return(tperror(drive,TPBSF_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpbsf(drive,count);
	    return(status);
	    }
#endif
/*
*  back over appropriate number of eof's
*/
	tpsig_blk(&sig_mask);
        for (i=0; i<*count+1; i++)
                {
                status = sys$qiow(0,drive->fd, IO$_SKIPFILE,&iosb,0,0,-1,
                        0,0,0,0,0);
                if ((iosb.status != SS$_NORMAL) && 
		    (iosb.status != SS$_ENDOFVOLUME) &&
		    (iosb.status != SS$_ENDOFFILE))
                        {
                        *count = i;
			drive->err = iosb.status;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPBSF_BSF,*mode));
                        }
                if (testbot(drive))
                        {
                        *count = i;
			tpsig_unblk(&sig_mask);
                        return(TAPE_BOT);
                        }
                }

/* forward space back to begining of file */

        status = sys$qiow(0,drive->fd,IO$_SKIPRECORD,&iosb,
                0,0,1,0,0,0,0,0);
        if ((iosb.status != SS$_NORMAL) && 
	    (iosb.status != SS$_ENDOFVOLUME) &&
	    (iosb.status != SS$_ENDOFFILE))
		{
		drive->err = iosb.status;
		tpsig_unblk(&sig_mask);
                return(tperror(drive,TPBSF_BSF,*mode));
		}
	tpsig_unblk(&sig_mask);
        return(TAPE_SUCC);
}  /*  c_tpbsf  */
#endif

#ifdef DGUX				/* data general */
#undef unix
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>

c_tpbsf
(
    long *tapefd,           /*  tape file descriptor                */
    long *count,            /*  number of files to backspace        */
    long *mode              /*  Error mode                          */
)
{
        struct mtop mt_com;
        struct mtget mt_status;
	long rstat;
	long sig_mask;
        long status;
        struct TAPEDESC *drive;
        int i;

	rstat = TAPE_SUCC;
        drive = (struct TAPEDESC *) *tapefd;
        if (*count < 0)
            {
            *count = 0;
            return(tperror(drive,TP_SKIP,*mode));
            }
	if (drive->flags & NOBSF)
	    {
	    *count = 0;
            return(tperror(drive,TPBSF_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpbsf(drive,count);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
	if (*count == 0)		/* backspace to beginning of file */
	    {
       	    mt_com.mt_count = *count;
            mt_com.mt_op = MTBSF;
            status = ioctl(drive->fd, MTIOCTOP, &mt_com);
            if (testbot(drive))
		rstat = TAPE_BOT;
            if (status < 0)
                {
		drive->err = errno;
		tpsig_unblk(&sig_mask);
                return(tperror(drive,TPBSF_BSF,*mode));
                }
	    }
	else
	    {
       	    mt_com.mt_count = 1;
            mt_com.mt_op = MTBSF;
            for (i=0; i<*count; i++)	
                {
                status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                if (testbot(drive))
                    {
                    *count = i;
		    rstat = TAPE_BOT;
		    break;
                    }
                if (status < 0)
                    {
                    *count = i;
		    drive->err = errno;
		    tpsig_unblk(&sig_mask);
                    return(tperror(drive,TPBSF_BSF,*mode));
                    }
                }
	    }
	tpsig_unblk(&sig_mask);
        return(rstat);
}  /*  c_tpbsf  */
#endif


#ifdef  unix					/* General Unix */
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>

int c_tpbsf
(
    long *tapefd,          /*  tape file descriptor                */
    long *count,           /*  number of files to backspace        */
    long *mode             /*  Error mode                          */
)
{
        struct mtop mt_com;
        struct mtget mt_status;
	long rstat;
	int  sig_mask;
        long status = 0;
        struct TAPEDESC *drive;
        int i;

	rstat = TAPE_SUCC;
        drive = (struct TAPEDESC *) *tapefd;
        if (*count < 0)
            {
            *count = 0;
            return(tperror(drive,TP_SKIP,*mode));
            }
	if (drive->flags & NOBSF)
	    {
	    *count = 0;
            return(tperror(drive,TPBSF_NO,*mode));
	    }
#ifdef NETWORK
	if (drive->netflg)			/* remote tape drive */
	    {
	    status = nt_tpbsf(drive,count);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
	switch (drive->dev_type)
	    {
	    case SOLARIS_SCSI:
                mt_com.mt_count = 1;
                mt_com.mt_op = MTNOP;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return(TAPE_FAIL);
		    }
                if (ioctl(drive->fd, MTIOCGET, &mt_status) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return (TAPE_FAIL);
		    }
		if (*count < mt_status.mt_fileno)
        	    mt_com.mt_count = *count;
		else
		    mt_com.mt_count = *count = mt_status.mt_fileno;
        	mt_com.mt_op = MTNBSF;
                status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                if (status < 0)
                    {
                    *count = 0;
		    drive->err = errno;
		    tpsig_unblk(&sig_mask);
                    return(tperror(drive,TPBSF_BSF,*mode));
                    }
                mt_com.mt_count = 1;
                mt_com.mt_op = MTNOP;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return(TAPE_FAIL);
		    }
                if (ioctl(drive->fd, MTIOCGET, &mt_status) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return (TAPE_FAIL);
		    }
		if (mt_status.mt_fileno == 0)
		    rstat = TAPE_BOT;
		break;
	    case SGI_XY:
                mt_com.mt_count = 1;
                mt_com.mt_op = MTNOP;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return(TAPE_FAIL);
		    }
                if (ioctl(drive->fd, MTIOCGET, &mt_status) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return (TAPE_FAIL);
		    }
		if (*count < mt_status.mt_fileno)
		    {
        	    mt_com.mt_op = MTBSF;
        	    mt_com.mt_count = *count + 1;
		    }
		else
		    {
		    rstat = TAPE_BOT;
        	    mt_com.mt_op = MTREW;
		    mt_com.mt_count = 1;
		    *count = mt_status.mt_fileno;
		    }
                status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                if (status < 0)
                    {
                    *count = 0;
		    drive->err = errno;
		    tpsig_unblk(&sig_mask);
                    return(tperror(drive,TPBSF_BSF,*mode));
                    }
		if (rstat != TAPE_BOT) 	/* forward to beginning of file */
		    {
		    mt_com.mt_count = 1;
        	    mt_com.mt_op = MTFSF;
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    if (status < 0)
                        {
		        drive->err = errno;
		        tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPBSF_BSF,*mode));
                        }
		    }
                mt_com.mt_count = 1;
                mt_com.mt_op = MTNOP;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return(TAPE_FAIL);
		    }
                if (ioctl(drive->fd, MTIOCGET, &mt_status) < 0)
		    {
		    tpsig_unblk(&sig_mask);
                    return (TAPE_FAIL);
		    }
		break;
	    case IRIX_EXABYTE:
        	mt_com.mt_count = 1;
        	mt_com.mt_op = MTBSF;
        	for (i=0; i<*count+1; i++)	/* back "count + 1" EOF's */
                    {
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    if (testbot(drive))
                        {
                        *count = i;
			rstat = TAPE_BOT;
			break;
                        }
                    if (status < 0)
                        {
                        *count = i;
			drive->err = errno;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPBSF_BSF,*mode));
                        }
                    }
		if (rstat == TAPE_SUCC)
		    {
        	    mt_com.mt_count = 1; /* forward to beginning of file */
        	    mt_com.mt_op = MTFSF;
        	    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
        	    if (status < 0)
		        {
		        drive->err = errno;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPBSF_BSF,*mode));
		        }
		    }
		if (status == TAPE_BOT)	/* required for high density 8mm */
    		    {
        	    mt_com.mt_count = 1;
        	    mt_com.mt_op = MTREW;
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
    		    }
		break;
	    default:
        	mt_com.mt_count = 1;
        	mt_com.mt_op = MTBSF;
        	for (i=0; i<*count+1; i++)	/* back "count + 1" EOF's */
                    {
                    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    if (testbot(drive))
                        {
                        *count = i;
			rstat = TAPE_BOT;
			break;
                        }
                    if (status < 0)
                        {
                        *count = i;
			drive->err = errno;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPBSF_BSF,*mode));
                        }
                    }
		if (rstat == TAPE_SUCC)
		    {
        	    mt_com.mt_count = 1; /* forward to beginning of file */
        	    mt_com.mt_op = MTFSF;
        	    status = ioctl(drive->fd, MTIOCTOP, &mt_com);
        	    if (status < 0)
		        {
		        drive->err = errno;
			tpsig_unblk(&sig_mask);
                        return(tperror(drive,TPBSF_BSF,*mode));
		        }
		    }
		break;
	    }
	tpsig_unblk(&sig_mask);
        return(rstat);
}  /*  c_tpbsf  */
#endif

