/**************************************************************************
*    Function:  tpread
*
*    Purpose:   Read next record from tape.
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
		9/95 	K.Gacke         DGUX 5.4R3.10 support
    3.4        11/95    K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)
    3.5        12/95	K.Gacke		IRIX 5.3 8mm cannot read odd byte
					records correctly. 
    3.6         1/96    K.Gacke         Port to Solaris


*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Read nbytes specified
*       Check for end-of-file
*       Check for end-of-volume
*       Check for end-of-tape
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>

#include "worgen.h"
#include "tapeio.h"
#include "pixman.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>

int c_tpread
(
    long *tapefd,        /*  tape unit number            */
    unsigned char *buffer,        /*  buffer                      */
    long *bufsz,         /*  buffer size                 */
    long *mode          /*  error mode                  */
)
{
        long status;
	int  sig_mask;
        int locmod;
        struct TAPEDESC *drive;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb, iosb2;

        drive = (struct TAPEDESC *) *tapefd;
#ifdef NETWORK
	if (drive->netflg)
	    {
            status = nt_tpread(drive,buffer,*bufsz);
	    tp_setlen(drive,status);
	    return(status);
	    }
#endif
	tpsig_blk(&sig_mask);
        status = sys$qiow(0,drive->fd, IO$_READVBLK, &iosb,0,0, buffer, *bufsz,
                                0,0,0,0);
        if ((iosb.status != SS$_ENDOFFILE) && (iosb.status != SS$_NORMAL))
        {
		printf("requested %d bytes, got %d\n",*bufsz,iosb.count);
                drive->err = iosb.status;
	        tpsig_unblk(&sig_mask);
                return(tperror(drive,TPREAD_READ,*mode));
        }
        if (iosb.status != SS$_ENDOFFILE)
            {
            status = iosb.count;
            }
        else		/* EOF detected, check for EOV */
            {
	    status = tpchk(tapefd);
	    }
	tpsig_unblk(&sig_mask);
	tp_setlen(drive,status);
        return(status);
}
#endif

#ifdef unix
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>


int c_tpread
(
    long *tapefd,        /*  tape unit number            */
    unsigned char *buffer,        /*  buffer                      */
    long *bufsz,         /*  buffer size                 */
    long *mode          /*  error mode                  */
)
{
/*#ifdef sgi
        struct mtblkinfo blkinfo;
  #endif 
*/
        struct TAPEDESC *drive;
	long cp_flg;
        long status;
	int  sig_mask;
	long loc_bufsz;
	unsigned char *bufptr;

        drive = (struct TAPEDESC *) *tapefd;
#ifdef NETWORK
	if (drive->netflg)
	    {
            status = nt_tpread(drive,buffer,*bufsz);
	    tp_setlen(drive,status);
	    return(status);
	    }
#endif
	if (drive->rah_nb > 0)
	    {
	    (void) memcpy(buffer,drive->rah,drive->rah_nb);
	    status = drive->rah_nb;
	    drive->rah_nb = 0;
	    tp_setlen(drive,status);
	    return(status);
	    }
	cp_flg = FALSE;
	switch (drive->dev_type)
	    {
	    case IRIX_SCSI:		/* buffer needs to be word aligned */
		bufptr = buffer;
		loc_bufsz = *bufsz;
		if ((int) bufptr % 4 != 0)
		    {
		    bufptr = (unsigned char *) drive->rah;
		    cp_flg = TRUE;
		    }
		break;
	    case IRIX_EXABYTE:
		bufptr = buffer;
		loc_bufsz = *bufsz;
		if ((int) bufptr % 4 != 0)	/* buffer word aligned*/
		    {
		    bufptr = (unsigned char *) drive->rah;
		    cp_flg = TRUE;
		    }
		if (*bufsz % 2 != 0)		/* check for odd buffer length*/
		    {
		    bufptr = (unsigned char *) drive->rah;
		    loc_bufsz = *bufsz + 1;
		    cp_flg = TRUE;
		    }
		break;
	    default:
		loc_bufsz = *bufsz;
        	bufptr = buffer;
	 	break;
	    }
	tpsig_blk(&sig_mask);
        status = read(drive->fd, bufptr, loc_bufsz);
        if ((status == 0) || ((status < 0) && (errno == ENOSPC)))
            {
	    status = tpchk(tapefd);
            }
	else if (status < 0)
            {
	    drive->err = errno;
	    tpsig_unblk(&sig_mask);
            return(tperror(drive,TPREAD_READ,*mode));
            }
	if ((status > 0) && (cp_flg))	/* check for buffer copy flag */
	    {
	    (void) memcpy(buffer,drive->rah,status);
	    }
	tpsig_unblk(&sig_mask);
	tp_setlen(drive,status);
        return(status);
}
#endif 

