#ifdef TPMNT_NET
/**************************************************************************
*
*   NAME: gettapnm
*
*   PURPOSE:
*       To retrieve the tape name from the operator.  There are two
*       environment variables used to control how a tape drive is assigned.
*       1) WORTPDRIVE - if a system only has one tape drive, this variable
*          may be assigned to the drive name so that an operator is not
*          required to reply to mount requests.
*       2) WORTPMTREQ - If a mount request is issued, this variable controls
*          the process that is required to reply to the mount request.  If
*          set to "1", the mount request is issued to the user's local
*          terminal.  If the variable is not set, the mount request is
*          issued to all users logged in with group id "ops".
*
*
   Program History:
   Date	Contrib		Change
   ----	-------		------
   2/93	K.Gacke		Original development -- modified method for assigning 
			tape drives to route all mount requests to a single 
			window. Uses the network server tpmntsrv.
  11/93	K.Gacke		Added tape length field to mount request.
   2/98 K.Gacke		If WORTPDRIV environment variable is set, we no longer
			need to inform ops (ie. removed call to tpmsg).
  10/98 D.Hames/        Add code to comply with localtime and y2k
        L. Johnson      
*
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Check the logical(environment variable) WORTDRIVE
*       If set
*          Check to see if the value is a valid drive name
*       Else
*          Connect to tpmntsrv and wait for operator to assign drive
*          Check to see if the drive name is a valid drive name
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#ifdef unix
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <unistd.h>
#endif
#ifdef vms
#include <unixio.h>
#include <file.h>
#endif

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#endif

#include "tapeio.h"
#include "tpmntsrv.h"
#include "local_protos.h"

int get_tape_nam
(
    char *tapenam,
	char *tapeid,
	long *tapelen,
    char *msg
)
{
extern char Tape_error[];	/* tape error buffer */
struct NT_MTREQ nt_mtreq;	/* network mount request structure */
long assflg;			/* assigned flag */
long mtflg;			/* mount flag */
long nonfatal = 1;		/* non fatal error */
long status;			/* function status code */
char cerr[256];			/* error buffer */

assflg = mtflg = FALSE;
if (get_logical("WORTPMTREQ",tapenam) == 0)
    mtflg = TRUE;
if (get_logical("WORTPDRIVE",tapenam) == -1)
    *tapenam = '\0';
else
    {
    sprintf(cerr,"Operator received message: %s",msg);
    tp_errmsg(cerr," ",&nonfatal);
    return(TAPE_SUCC);
    }
sprintf(cerr,"Operator received message: %s",msg);
tp_errmsg(cerr," ",&nonfatal);
if (!mtflg)
    {
    nt_mntinit(&nt_mtreq);
    strcpy(nt_mtreq.errtext,Tape_error);
    for (assflg=FALSE; !(assflg); )
        {
        status = nt_mntopen(TPUSER,TPPWD,"mntreq");
        if (status != E_SUCC)
            {
            sprintf(cerr,"%s %s %d %s",
		"Error opening network connection for mount request.",
		"Will retry in",TPMNT_SLEEP,"seconds.");
	    tp_errmsg(cerr," ",&nonfatal);
            sleep(TPMNT_SLEEP);
            }
        status = nt_addmnt(&nt_mtreq,msg,TPMNT_WAIT);
        if (status != E_SUCC)
	    {
	    tp_errmsg("Error adding/checking tape mount request"," ",&nonfatal);
	    tp_errmsg("Mount request is being resent, please wait",
		" ",&nonfatal);
	    nt_mntclose();
	    }
        if (nt_mtreq.status == TPMNT_ASSIGN)
	    {
	    assflg = TRUE;
	    strcpy(tapenam,nt_mtreq.drive);
	    strcpy(tapeid,nt_mtreq.tapeid);
	    if (strlen(nt_mtreq.tapelen))
		sscanf(nt_mtreq.tapelen,"%d",tapelen);
	    status = nt_delmnt(&nt_mtreq);
            if (status != E_SUCC)
	        {
	        tp_errmsg("Error deleting tape mount request"," ",&nonfatal);
	        nt_mntclose();
	        }
	    }
        else
	    sleep(TPMNT_SLEEP);
        }
    status = nt_mntclose();
    if (status != E_SUCC)
	{
	tp_errmsg("Error closing mount request network connection",
		" ",&nonfatal);
	}
    }
if (mtflg)
    {
    fprintf(stderr,"%s\n", msg);
    fprintf(stderr,"Enter Tape drive number: ");
    fflush(stderr);
    scanf("%s",tapenam);
    fprintf(stderr,"Enter Tape ID: ");
    fflush(stderr);
    scanf("%s",tapeid);
    fprintf(stderr,"Enter Tape Length: ");
    scanf("%d",tapelen);
    }
if (strcmp(tapenam,"-1") == 0)
    {
    return(TPOPEN_NOAVAIL);
    }
return(TAPE_SUCC);
}

#else
/**************************************************************************
*
*   NAME: gettapnm
*
*   PURPOSE:
*       To retrieve the tape name from the operator.  There are two
*       environment variables used to control how a tape drive is assigned.
*       1) WORTPDRIVE - if a system only has one tape drive, this variable
*          may be assigned to the drive name so that an operator is not
*          required to reply to mount requests.
*       2) WORTPMTREQ - If a mount request is issued, this variable controls
*          the process that is required to reply to the mount request.  If
*          set to "1", the mount request is issued to the user's local
*          terminal.  If the variable is not set, the mount request is
*          issued to all users logged in with group id "ops".
*
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         10/87   F. Irani/SAR    Check that device name recieved is a 
					tape device.
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         1/90    K.Gacke         On unix allow ops to be logged onto 
					either a terminal or the console
    2.3         3/90    K.Gacke         On vms, if wortpmtreq is set then 
					output mount msg to local terminal
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
*
*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Check the logical(environment variable) WORTDRIVE
*       If set
*          Check to see if the value is a valid drive name
*       Else
*          Prompt the operator for the drive name
*          Check to see if the drive name is a valid drive name
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#endif

#include "tapeio.h"

#ifdef vms
#include <opcdef.h>
#include <iodef.h>
#include <ssdef.h>
#include "desc.h"

#define BUFSZ 512

int get_tape_nam
(
    char *tapenam,      /*  tape name                           */
	char *tapeid,		/*  tape ID */
	long *tapelen,		/*  tape length */
    char *msg           /*  message to the operator             */
)
{
        long nonfatal = 1;
        long ntry;

        int i;

        short chan;
        short status;

        char mtflg;
        char mtreq[CMLEN];
        char message[CMLEN];
        struct
        {
                char m_type;
                short m_target;
                char m_zero;
                long m_myid;
                char m_text[120];
        }opreq;

        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        char buffer[BUFSZ];

        struct DESC msgdesc;
        char cerr[ERRLEN];

/* If logical (VMS) or environment variable (UNIX) WORTPDRIVE exists then
   do not prompt operator to assign a tape drive.  The value of WORTPDRIVE
   is used as the drive to be allocated.
   ----------------------------------------------------------------------*/
        if (get_logical("WORTPDRIVE",tapenam) == -1)
           *tapenam = '\0';

        if (*tapenam != '\0')
                {
                if (istape(tapenam))
                        {
                        return(TAPE_SUCC);
                        }
                else
                        {
                        sprintf(msg,"%s %s","Device name defined by",
                                "logical WORTPDRIVE is not a tape device");
                        tp_errmsg(msg,"tapeio-device",&nonfatal);
                        *tapenam = '\0';
                        }
                }

        mtflg = FALSE;
        if (get_logical("WORTPMTREQ",mtreq) != -1)
           mtflg = TRUE;

        if (mtflg)
                {
                for (ntry=0; ntry<10; ntry++)
                        {
                        tp_errmsg(msg," ",&nonfatal);
                        printf("\nPlease enter drive name: ");
                        scanf("%s",tapenam);
                        if (strcmp(tapenam,"-1") == 0)
                                return(TPOPEN_NOAVAIL);
                        printf("\nPlease enter tape ID: ");
                        scanf("%s",tapeid);
                        if (istape(tapenam))
                                {
                                return(TAPE_SUCC);
                                }
                        else
                                {
                                sprintf(msg,"%s is not a tape device",tapenam);
                                tp_errmsg(msg,"tapeio-tape",&nonfatal);
                                }
                        return(TAPE_FAIL);
                        }
                }

        strcpy(message, msg);
        strcat(message,"\r\nREPLY/TO=nn drive_name tape_id");

        status = sys$crembx(0,&chan,BUFSZ,0,0,0,0);
        if (!(status & 1))
                return(TPOPEN_OPS);

retry:
        opreq.m_type = OPC$_RQ_RQST;
        opreq.m_target = OPC$M_NM_TAPES;
        opreq.m_zero = 0;
        opreq.m_myid = getpid();
        strcpy(opreq.m_text, message);

        msgdesc.length = 8 + strlen(opreq.m_text);
        msgdesc.addr = &opreq;

        status = sys$sndopr(&msgdesc, chan);
        if (!(status & 1))
                return(TPOPEN_OPSWR);
        sprintf(cerr,"Operator received message: %s",msg);
        tp_errmsg(cerr," ",&nonfatal);

        status = sys$qiow(0,chan,IO$_READVBLK,&iosb,0,0,buffer,BUFSZ,0,0,0,0);
        if (!(iosb.status & 1 ))
                return(TPOPEN_OPSRD);

        buffer[iosb.count] = '\0';

        sscanf (&buffer[8], "%s %s", tapenam,tapeid);
        if ((tapenam[0] == '-') && (tapenam[1] == '1'))
           return(TPOPEN_NOAVAIL);
        if (!istape(tapenam))
        {
                strcpy(message, msg);
                strcat(message,
                       "\r\ninvalid tape drive name REPLY/TO=nn drive_name");
                goto retry;
        }
        strcpy(msg,tapenam);
return(TAPE_SUCC);
}
#endif

#ifdef unix
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

int get_tape_nam
(
    char *tapenam,
    char *tapeid,
    long *tapelen,
    char *msg
)
{
        struct timeval stimout;
        struct tm      *tm_ptr;
        long status;
        long readfds;
        long nonfatal = 1;
        long pid;
        long ntry;
        long year;                      /* year */
        int sd,nsd;
        int stat;
        struct sockaddr sckta, from;
        int size, len;
        static char bells[] = {"\n\n\7\7\7\7\7\7\7\7\7\7\7\7\7\7\7\n\r"};
        static char tmsg[] = {"\n\r rp job# drive tapeid"};
	char buffer[100];
	char lmsg[4096];
        char mtflg;
        char mdata[160];
        char timstr[80];
        char cerr[ERRLEN];
        char *cptr,*getenv();
        long my_time;
        int i;

        mtflg = FALSE;
        cptr = getenv("WORTPMTREQ");
        if (cptr != NULL)
            mtflg = TRUE;
        cptr = getenv("WORTPDRIVE");
        if (cptr == NULL)
            *tapenam = '\0';
        else
            {
            strcpy(tapenam,cptr);
            sprintf(cerr,"Operator received message: %s",msg);
            tp_errmsg(cerr," ",&nonfatal);
	    sprintf(lmsg,"%s%s",bells,msg);
	    tpmsg(msg);
            return(TAPE_SUCC);
            }
        ntry = 0;
        sprintf(cerr,"Operator received message: %s",msg);
        tp_errmsg(cerr," ",&nonfatal);
        len = sizeof(from);
        pid = getpid();
        my_time=time(0);
        tm_ptr=localtime(&my_time);
        year = tm_ptr->tm_year;           
        year = year%100;                   /* used to retrieve 2 year digits for
					      dates after the year 2000 */

        sprintf(timstr, "%2d/%02d/%02d %02d:%02d:%02d "
                ,tm_ptr->tm_mon+1,tm_ptr->tm_mday,year
                ,tm_ptr->tm_hour,tm_ptr->tm_min,tm_ptr->tm_sec);
        sprintf(mdata, "\r\n %s job# = %d%s",timstr, pid, tmsg);
        if (mtflg)
            {
            fprintf(stderr,"%s\n", msg);
            fprintf(stderr,"Enter Tape drive number: ");
	    fflush(stderr);
            scanf("%s",tapenam);
            fprintf(stderr,"Enter Tape ID: ");
	    fflush(stderr);
            scanf("%s",tapeid);
            }
        else
            {
            sd = socket(AF_UNIX, SOCK_STREAM, 0);
            if (sd < 0)
		{
		perror("socket");
		printf("Error opening socket, errno = %d\n",errno);
                return(TPOPEN_SOCKET);
		}
            sckta.sa_family = AF_UNIX;
            sprintf (sckta.sa_data, "/tmp/rp%d", pid);
            while (strlen(sckta.sa_data) < 13) /*  fix for 4.1c BSD */
                {
                strcat(sckta.sa_data, "Z");
                }
            size = sizeof(sckta.sa_family) + strlen(sckta.sa_data);
            stat = bind(sd, &sckta, size);
            if (stat < 0)
		{
		perror("bind");
		printf("Error binding socket %s, errno = %d\n",
			sckta.sa_data,errno);
                return(TPOPEN_SOCKET);
		}
	    stat = chmod(sckta.sa_data,0777);
            do
                {
                my_time=time(0);
                tm_ptr=localtime(&my_time);
                year = tm_ptr->tm_year;            /* used to retrieve 2 year digits for
                year = year%100;                      dates after the year 2000 */

                sprintf(timstr, "%2d/%02d/%02d %02d:%02d:%02d "
                    ,tm_ptr->tm_mon+1,tm_ptr->tm_mday,year
                    ,tm_ptr->tm_hour,tm_ptr->tm_min,tm_ptr->tm_sec);
                sprintf(mdata, "\r\nFOR %s \r\n %s job# = %d%s",
                    getenv("USER"), timstr, pid, tmsg);
		sprintf(lmsg,"%s%s%s",bells,msg,mdata);
		tpmsg(lmsg);
                for (i=0; i<61; i++)
                    {
                    stat = listen(sd, 10);
                    if (stat < 0)
                        return(TPOPEN_OPSRD);
                    stimout.tv_sec = 1;
                    stimout.tv_usec = 0;
                    readfds = 1<<sd;
                    status = select(20, &readfds, NULL, NULL, &stimout);
                    if (status < 0)
                        perror("select");
                    if (status != 0)
                        break;
                    }
                } while (status == 0);
            nsd = accept(sd, &from, &len);
            if (nsd < 0)
                return(TPOPEN_OPSRD);
            stat = recv(nsd, buffer, CMLEN, 0);
            if (stat < 0)
                return(TPOPEN_OPSRD);
            buffer[stat] = '\0';
	    sscanf(buffer,"%s %s",tapenam,tapeid);
            close(sd);
	    close(nsd);
            unlink(sckta.sa_data);
            sprintf(lmsg,"\r\nRequest %d for %s completed\r\n",pid,
                    getenv("USER"));
	    tpmsg(lmsg);
            }
	if (strcmp(tapenam,"-1") == 0)
            {
            return(TPOPEN_NOAVAIL);
            }
        return(TAPE_SUCC);
        }

#endif
#endif
