/******************************************************************************
NAME:    las_is_tape 

PURPOSE: Using the input structure TAPEDESC, return TRUE 
         if a tape is mounted. 

RETURN VALUE:
Type = int
Value           Description
-----           -----------
TRUE            It is a Tape
FALSE           It is a CD/DVD

PROJECT:        LAS

******************************************************************************/
#include <stdio.h>
#include <string.h>

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#endif

#include "tapeio.h"

/**
 * las_is_tape:
 * @TAPEDESC drive: I: tape descriptor structure 
 * 
 * Using the input structure TAPEDESC, return TRUE if a tape is mounted.
 * 
 * Returns: TRUE: It is a Tape
 *          FALSE: It is a CD/DVD  
 */

int las_is_tape 
(
    const struct TAPEDESC *drive         /* I: tape descriptor structure */
)

{
    char drive_type[CMLEN];              /* temporary variable to store 
                                            drive type */
    long len;

    /* copy to drive_type and set to lower case
    -------------------------------------------*/
    strcpy (drive_type, drive->dtype);

    len = strlen(drive_type);
    c_up2low(drive_type,&len);

    /* Return FALSE if the drive type is cd (reader), cdr (writer),
       dvd (reader), or dvdr (writer) 
    ---------------------------------------------------------------*/
    if ((strcmp(drive_type, "cd") == 0) || 
        (strcmp(drive_type, "cdr") == 0) ||
        (strcmp(drive_type, "dvd") == 0) ||
        (strcmp(drive_type, "dvdr") == 0))
    {
        return (FALSE);
    }

    /* return tape 
    --------------*/
    else
    {
        return (TRUE);
    }
}
    

