/**************************************************************************
*
*
*       NAME: sense
*
*       PURPOSE: To test for the end-of-tape, beginning-of-tape, or end-of-file
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
    3.4        11/95    K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)
    3.5         1/96    K.Gacke         Port to Solaris


*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       TESTEOT
*       Test for the end-of-tape
*
*       TESTBOT
*       Test for the beginning-of-tape
*
*       TESTEOF
*       NOTE: this is for UNIX only
*       Test for the end-of-file
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>

int testeot
(
    struct TAPEDESC *drive
)
{
        globalvalue     MT$M_EOT;  /* External constant for VMS EOT check */
        int status;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        status = sys$qiow(0,drive->fd, IO$_SENSEMODE, &iosb,0,0,0,0,0,0,0,0);
        if ((iosb.devdep & MT$M_EOT) == MT$M_EOT)
            return(TRUE);
        else
            return(FALSE);
}

int testbot
(
    struct TAPEDESC *drive
)
{
        int status;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        status = sys$qiow(0,drive->fd, IO$_SENSEMODE, &iosb,0,0,
            0,0,0,0,0,0);
        if ((iosb.devdep & 65536) == 65536)
            return(TRUE);
        else
            return(FALSE);
	return(FALSE);
}
#endif

#ifdef DGUX
#undef unix
#include <ctype.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>

testeot(drive)
        struct TAPEDESC *drive;
{
        struct mtop mt_com;
        struct mtget mt_status;
        register struct mtget *bp;

        mt_com.mt_count = 1;
        mt_com.mt_op = MTNOP;
        if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                return(TAPE_FAIL);
        bp = &mt_status;
        if (ioctl(drive->fd, MTIOCGET, bp) < 0)
                return (TAPE_FAIL);
	switch (drive->dev_type)
	    {
	    default:
        	return (bp->mt_status1 & drive->eot);
		break;
	    }
	return(FALSE);
}

int testbot
(
    struct TAPEDESC *drive
)
{
        struct mtop mt_com;
        struct mtget mt_status;
        register struct mtget *bp;

        mt_com.mt_count = 1;
        mt_com.mt_op = MTNOP;
        if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                return(TAPE_FAIL);
        bp = &mt_status;
        if (ioctl(drive->fd, MTIOCGET, bp) < 0)
                return (TAPE_FAIL);
	switch (drive->dev_type)
	    {
	    default:
        	return (bp->mt_status1 & drive->bot);
		break;
	    }
	return(FALSE);
}

int testeof
(
    struct TAPEDESC *drive	/* tape descriptor */
)
{
        struct mtop mt_com;
        struct mtget mt_status;
        register struct mtget *bp;

        mt_com.mt_count = 1;
        mt_com.mt_op = MTNOP;
        if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                return(TAPE_FAIL);
        bp = &mt_status;
        if (ioctl(drive->fd, MTIOCGET, bp) < 0)
                return (TAPE_FAIL);
	switch (drive->dev_type)
	    {
	    default:
            	return (bp->mt_status1 & drive->eof);
		break;
	    }
	return(FALSE);
}
#endif

#ifdef unix
#include <ctype.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <errno.h>

int testeot
(
    struct TAPEDESC *drive
)
{
        struct mtop mt_com;
        struct mtget mt_status;
        register struct mtget *bp;

        mt_com.mt_count = 1;
        mt_com.mt_op = MTNOP;
        if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                return(TAPE_FAIL);
        bp = &mt_status;
        if (ioctl(drive->fd, MTIOCGET, bp) < 0)
                return (TAPE_FAIL);
	switch (drive->dev_type)
	    {
	    case LINUX_TS:
	    case LINUX_SCSI:
#ifdef DEBUG
	printf("\n\n\ntesteot Linux not supported:\n");
	printf("testeot: mt_type  =%x\n",bp->mt_type);
	printf("testeot: mt_dsreg =%x\n",bp->mt_dsreg);
	printf("testeot: mt_erreg =%x\n",bp->mt_erreg);
	printf("testeot: mt_resid =%x\n",bp->mt_resid);
	printf("testeot: mt_fileno=%x\n",bp->mt_fileno);
	printf("testeot: mt_blkno =%x\n",bp->mt_blkno);
#endif 
		return(FALSE);
	    default:
        	return (bp->mt_dsreg & drive->eot);
	    }
}

int testbot
(
    struct TAPEDESC *drive
)
{
        struct mtop mt_com;
        struct mtget mt_status;
        register struct mtget *bp;
/************
        mt_com.mt_count = 1;
        mt_com.mt_op = MTNOP;
        if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                return(TAPE_FAIL);
*************/
        bp = &mt_status;
        if (ioctl(drive->fd, MTIOCGET, bp) < 0)
                return (TAPE_FAIL);
	switch (drive->dev_type)
	    {
	    case SOLARIS_SCSI:
		printf("sense: solaris scsi testbot cannot be detected\n");
		return(FALSE);
	    case LINUX_TS:
	    case LINUX_SCSI:
#ifdef DEBUG
	printf("\n\n\ntestbot:\n");
	printf("testbot: mt_type  =%x\n",bp->mt_type);
	printf("testbot: mt_dsreg =%x\n",bp->mt_dsreg);
	printf("testbot: mt_erreg =%x\n",bp->mt_erreg);
	printf("testbot: mt_resid =%x\n",bp->mt_resid);
	printf("testbot: mt_fileno=%x\n",bp->mt_fileno);
	printf("testbot: mt_blkno =%x\n",bp->mt_blkno);
#endif 
		if ((bp->mt_fileno == 0) && (bp->mt_blkno == 0))
        	    return (TRUE);
		return(FALSE);
#ifdef sgi
	    case IRIX_SCSI:
        	return (bp->mt_dposn & drive->bot);
#endif /* sgi */
/*
* SGI IRIX 5.3 requires the rewind otherwise all subsequent i/o results in
* no space error 0x1c
*/
	    case IRIX_EXABYTE:
		if (bp->mt_dsreg & drive->bot)
		    {			
        	    mt_com.mt_count = 1;
        	    mt_com.mt_op = MTREW;
        	    if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                	return(TAPE_FAIL);
		    }
        	return (bp->mt_dsreg & drive->bot);
	    default:
        	return (bp->mt_dsreg & drive->bot);
	    }
}

int testeof
(
    struct TAPEDESC *drive	/* tape descriptor */
)
{
        struct mtop mt_com;
        struct mtget mt_status;
        register struct mtget *bp;

        mt_com.mt_count = 1;
        mt_com.mt_op = MTNOP;
        if (ioctl(drive->fd, MTIOCTOP, &mt_com) < 0)
                return(TAPE_FAIL);
        bp = &mt_status;
        if (ioctl(drive->fd, MTIOCGET, bp) < 0)
                return (TAPE_FAIL);
#ifdef DEBUG
	printf("\n\n\ntesteof:\n");
	printf("testeof: mt_type  =%x\n",bp->mt_type);
	printf("testeof: mt_dsreg =%x\n",bp->mt_dsreg);
	printf("testeof: mt_erreg =%x\n",bp->mt_erreg);
	printf("testeof: mt_resid =%x\n",bp->mt_resid);
	printf("testeof: mt_fileno=%x\n",bp->mt_fileno);
	printf("testeof: mt_blkno =%x\n",bp->mt_blkno);
/****
	printf("testeof: mt_flags =%x\n",bp->mt_flags);
	printf("testeof: mt_bf    =%x\n",bp->mt_bf);
****/
#endif 
	switch (drive->dev_type)
	    {
	    case SOLARIS_SCSI:
		printf("sense: solaris scsi testeof cannot be detected\n");
            	return (FALSE);
	    case LINUX_TS:
	    case LINUX_SCSI:
		if (bp->mt_blkno == 0)
		    return(TRUE);
            	return (FALSE);
	    case SGI_XY:
		if (bp->mt_dsreg == drive->eof)
		    return(TRUE);
		break;
	    case IRIX_EXABYTE:
	    case IRIX_SCSI:
            	return (bp->mt_dsreg & drive->eof);
	    default:
            	return (bp->mt_erreg & drive->eof);
	    }
	return(FALSE);
}
#endif

