/**************************************************************************
*
*       NAME: tlbsr
*
*       PURPOSE:  To backspace over end-of-file markers
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2		9/95	K.Gacke		Modified for DGUX 5.4R3.10
    3.5         1/96    K.Gacke         Port to Solaris
    3.6         12/96   K.Gacke         Solaris 8mm bug fix

*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*       Do the number specified in count
*         Backspace over a EOF
*         Check for the beginning of the tape
*         Check for the beginning of the file
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>

/* local tape routine that allows back skipping over eof's */
int tlbsr
(
    long *tapefd,           /*  tape file descriptor                */
    long *mode,             /*  error mode                          */
    long *tpstatus		/*  tape status EOV or EOF */
)

{
        long stat,status;
        struct TAPEDESC *drive;
        int i;
	long cmd;
	long cnt;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb;

        drive = (struct TAPEDESC *) *tapefd;
	switch (*tpstatus)
	    {
	    case TAPE_EOF:
		cnt = -1;
		cmd = IO$_SKIPRECORD;
		break;
	    case TAPE_EOV:
		cnt = -2;
		cmd = IO$_SKIPRECORD;
		break;
	    case TAPE_EOT:
		cnt = -2;
		cmd = IO$_SKIPFILE;
		break;
	    }
        status = sys$qiow(0,drive->fd, cmd,&iosb,0,0,
                        cnt,0,0,0,0,0);
        if ((iosb.status != SS$_NORMAL) && (iosb.status != SS$_ENDOFFILE))
            {
	    drive->err = iosb.status;
            return(tperror(drive,TPBSR_BSR,*mode));
            }
/* must test for bot before bof (eof) since VMS says bot is eof */
        if (testbot(drive))
            {
            return(TAPE_BOT);
            }
        if (iosb.status == SS$_ENDOFFILE)
            {
            return(TAPE_BOF);
            }
        return(TAPE_SUCC);
}
#endif

#ifdef unix
#include <ctype.h>
#include <sys/types.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <errno.h>

/* local tape routine that allows back skipping over eof's */
int tlbsr
(
    long *tapefd,           /*  tape file descriptor                */
    long *mode,             /*  error mode                          */
    long *tpstatus  		/*  tape status EOV or EOF */
)
{
        struct TAPEDESC *drive;
        struct mtop mt_com;
        int locstat;

        drive = (struct TAPEDESC *) *tapefd;
/*
   Don't check status on DG drives.  Network code needs to backspace record
   in which the record is a tape mark.  The ioctl completes the operation,
   but returns bad status with errno set to EIO.
*/
	if ((*tpstatus == TAPE_SUCC) || (*tpstatus == TAPE_EOF))
	    {
	    mt_com.mt_count = 1;
            mt_com.mt_op = MTBSR;
            locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
            if (locstat < 0)
                {
		drive->err = errno;
                return(tperror(drive,TPBSR_BSR,*mode));
                }
	    }
	else if (*tpstatus == TAPE_EOV)	
	    {
	    switch(drive->dev_type)
		{
		case DG54_GEN: 		/* DGUX 5.4R3.10 */
                    mt_com.mt_count = 2;
                    mt_com.mt_op = MTBSF;
            	    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
		    break;
		case SOLARIS_SCSI:	/* Solaris */
		default:		/* if EOV backspace 1 file mark */
                    mt_com.mt_count = 1;
                    mt_com.mt_op = MTBSF;
            	    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
		    break;
		}
	    }
	else if (*tpstatus == TAPE_EOT) 
	    {
	    switch(drive->dev_type)
		{
		case DG54_GEN: 		/* DGUX 5.4R3.10 */
                    mt_com.mt_count = 3;
                    mt_com.mt_op = MTBSF;
            	    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
		    break;
		case SOLARIS_SCSI:	/* Solaris */
                    mt_com.mt_count = 2;
                    mt_com.mt_op = MTBSF;
            	    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
		    break;
	        default:		/* if EOT backspace 2 file mark */
                    mt_com.mt_count = 1;
                    mt_com.mt_op = MTBSF;
            	    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
                    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
		    break;
		}
	    }
        return(TAPE_SUCC);
}
#endif
