/**************************************************************************
*    Function:  tpchk
*
*    Purpose:   
	If EOF detected (1 tape mark), then tpchk is called to check for
	EOV (2 tape marks) or EOT (3 tape marks).  Physical end of tape
	is also checked.

   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0         7/92    K. Gacke        Original development
    1.2		8/95    K.Gacke         IRIX 5.3 support, added cdr as device 
					type
		9/95 	K.Gacke         DGUX 5.4R3.10 support
    3.4        11/95    K.Gacke         Modified for IRIX 5.3 (Fugitsu 9-track)
    3.5         1/96    K.Gacke         Port to Solaris


*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <iodef.h>
#include <ssdef.h>

int tpchk
(
    long *tapefd        /*  tape unit number            */
)
{
        long status;
	int  sig_mask;
        long locstat;
        int locmod;
        struct TAPEDESC *drive;
        struct
        {
                unsigned short status;
                unsigned short count;
                long devdep;
        } iosb, iosb2;

        drive = (struct TAPEDESC *) *tapefd;
	tpsig_blk(&sig_mask);
        locstat = sys$qiow(0,drive->fd, IO$_READVBLK, &iosb,0,0, drive->rah,
                20,0,0,0,0);
        if (testeot(drive))
	    {
	    status = TAPE_EOT;
            goto xit;
	    }
        if (iosb.status == SS$_ENDOFFILE)
            {
            status = TAPE_EOV;
            locstat = sys$qiow(0,drive->fd, IO$_READVBLK, &iosb,0,0, drive->rah,
                20,0,0,0,0);
            if (testeot(drive))
	        {
	        status = TAPE_EOT;
                goto xit;
	        }
            if (iosb.status == SS$_ENDOFFILE)
		status = TAPE_EOT;
            }
        else
            {
            status = TAPE_EOF; /* just end of file EOF */
            }
        locmod = HANDLE;
        locstat = tlbsr(tapefd,&locmod,&status);
        if ((locstat < 0) && (locstat != TAPE_BOF))
            status = TAPE_FAIL;
xit:
	tpsig_unblk(&sig_mask);
        return(status);
}
#endif

#ifdef unix
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mtio.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <unistd.h>


/*
	For the DG with OS >= DGUX 5.4R3.10 do the following code.
*/
int tpchk_dg54
(
    long *tapefd
)
{
struct TAPEDESC *drive;
struct mtop mt_com;
long status;
long locstat;
long locmod;

drive = (struct TAPEDESC *) *tapefd;
if (testeot(drive))
    {
    status = TAPE_EOT;
    goto xit;
    }
mt_com.mt_count = 1;
mt_com.mt_op = MTFSR;
locstat = ioctl(drive->fd, MTIOCTOP, &mt_com); 
if (locstat == 0)
    status = TAPE_EOF;
else if ((locstat < 0) && (errno == EIO))
    {
    mt_com.mt_count = 1;
    mt_com.mt_op = MTFSF;
    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
    if (locstat == 0)
	{
        locstat = read(drive->fd, drive->rah, drive->maxbyte);
	if (locstat > 0)
	    status = TAPE_EOV;
	else if ((locstat == -1) && (errno == ENXIO))
	    status = TAPE_EOV;
	else 
	    status = TAPE_EOT;
	}
    else 
	{
	status = TAPE_EOT;
	goto xit;
	}
    }
else
    {
    status = TAPE_EOT;
    goto xit;
    }
locmod = HANDLE;
locstat = tlbsr(tapefd,&locmod,&status);
if ((locstat < 0) && (locstat != TAPE_BOF))
    status = TAPE_FAIL;
locstat = testeof(drive);
xit:
return(status);
}

int tpchk_gen
(
	long *tapefd
)
{
struct TAPEDESC *drive;
struct mtop mt_com;
long cnt;
long bsr_cnt;
long rstat;
long locstat;

drive = (struct TAPEDESC *) *tapefd;
rstat = TAPE_EOF;
bsr_cnt = 0;
locstat = read(drive->fd, drive->rah, drive->maxbyte);
if (locstat > 0)
    drive->rah_nb = locstat;
else if (locstat == 0)
    {
    rstat = TAPE_EOV;
    locstat = read(drive->fd, drive->rah, drive->maxbyte);
    if (locstat > 0)
	bsr_cnt = 1;
    else if ((errno==EIO) && 
         (strcmp(drive->dtype,"9-track")==0) && (drive->dev_type==IRIX_SCSI))
	bsr_cnt = 1;
    else if (locstat == 0)
	{
	bsr_cnt = 0;
	rstat = TAPE_EOT;
	}
    }
else if (errno == ENOSPC)
    rstat = TAPE_EOT;
/*
*  The following added for the 9-track tape drives on sgi.  Needed for the
*  situation in which an FSF greater than the actual number of files on
*  tape, the read may returns an i/o error.  Rather than returning an error, we
*  assume that we are at EOV.  Also required for Linux systems.
*/
else if (errno==EIO)
    {
    if ((strcmp(drive->dtype,"9-track")==0) && (drive->dev_type==IRIX_SCSI))
        rstat = TAPE_EOV;
    else if ((drive->dev_type==LINUX_SCSI) || (drive->dev_type==LINUX_TS))
	rstat = TAPE_EOV;
    else
	rstat = TAPE_FAIL;
    }
else
    rstat = TAPE_FAIL;

if (bsr_cnt > 0)
    {
    mt_com.mt_count = bsr_cnt;
    mt_com.mt_op = MTBSR;
    locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
    }
if (rstat != TAPE_EOF)
    {
    switch (drive->dev_type)
	{
/*
*  On SGI, the MTBSR ioctl returns error if skipping tape mark; however, it 
*  completes the operation.  Continue to backspace until the MTBSR returns 
*  a good status, then issue tape reads to position tape after first tape 
*  mark.  Limit the number of MTBSR commands to ten.
* 
*  On Solaris, the MTBSR does not position tape behind a tape mark and 
*  resturns a bad status; therefore, if MTBSR returns an error, the MTBSF
*  is used to backskip over the tapemark.
*/
	case IRIX_SCSI:
	case IRIX_EXABYTE:
	case LINUX_TS:
            for (cnt=0; cnt<10; cnt++)	
                {
                mt_com.mt_count = 1;
                mt_com.mt_op = MTBSR;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) >= 0)
                    break;
                }
	    if (cnt > 10)
		rstat = TAPE_FAIL;
	    break;
	case SOLARIS_SCSI:
	case LINUX_SCSI:
	default:
            for (cnt=0; cnt<10; cnt++)	
                {
                mt_com.mt_count = 1;
                mt_com.mt_op = MTBSR;
                if (ioctl(drive->fd, MTIOCTOP, &mt_com) >= 0)
                    break;
                mt_com.mt_count = 1;
                mt_com.mt_op = MTBSF;
                locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
                }
	    break;
	}
/* 
*  Position tape after first tape mark.  
*/
    switch (drive->dev_type)
        {
/* 
*  On SGI IRIX, it is an error to write after a read; therefore, use FSF to 
*  position the tape past the tape mark.  
*/
        case IRIX_EXABYTE:
        case IRIX_SCSI:
	case LINUX_TS:
            mt_com.mt_count = 1;
            mt_com.mt_op = MTFSF;
            locstat = ioctl(drive->fd, MTIOCTOP, &mt_com);
	    if (locstat < 0)
		rstat = TAPE_FAIL;
	    break;
        default:
            for (;;)		
                {
                locstat = read(drive->fd, drive->rah, drive->maxbyte);
                if (locstat <= 0)
                    break;
                }
	    break;
        }
    }
xit:
return(rstat);
}

int tpchk
(
    long *tapefd        /*  tape unit number            */
)
{
        struct TAPEDESC *drive;
        long status;
	int  sig_mask;

        drive = (struct TAPEDESC *) *tapefd;
	tpsig_blk(&sig_mask);
	switch (drive->dev_type)
	    {
	    case DG54_GEN:
		status = tpchk_dg54(tapefd);
		break;
	    default:
		status = tpchk_gen(tapefd);
		break;
	    }
        if (testeot(drive))
	    status = TAPE_EOT;
	tpsig_unblk(&sig_mask);
        return(status);
}
#endif
