/**************************************************************************
*    Function:  tplen
*
*    Purpose:   
	Set the default tape length, bpi, and inter record gap values for
	the specific tape device.  It is assumed that a tape mark is 
	about 10 times the irg.  Not sure if all values specified are
	correct: 
	sdlt tape len is unknown; however, total capacity is known to be 110GB:
	lto tape len is unknown; however, total capacity is known to be 100GB:
	lto2 tape len is unknown; however, total capacity is known to be 200GB:
	9940B tape len is unknown; however, total capacity is known to be 200GB:

	HP Website -- LTO1/LTO2 media is 609m, LTO3 media is 680m.

	Device      Len (ft)  BPI       IRG (in)    Capacity	Spec
	9-track     2400      800       .8
	9-track     2400      1600      .8
	9-track     2400      6250      .4          150 MB
	3480        600       31000     .11  ??     200 MB
	3490	    1100      62000     .0001??     800 MB 
	8mm (low)   365       512000    .002 ??     2.5 GB
	8mm (high)  365      1024000    .001 ??     5.0 GB
	4mm         270       400000    .01  ??     1.3 GB
	ait         2190     1024000    .00001??   25.0 GB	30GB
	dlt2000	    1100     1000000    .0002??    10.0 GB 
	dlt4000	    1200     1100000    .0002??    20.0 GB
	dlt7000	    1200     1925000    .0002??    35.0 GB
	9840         580     3185000    .0001??    20.0 GB
	sdlt	    2400     3440000    .00001??   92.3 GB	110GB
	lto	    2400     3620000    .00001??   97.0 GB	100GB
	lto2	    4800     3620000    .00001??  194.1 GB	200GB
	lto3	    9400     3620000    .00001??  380.3 GB	400GB
	9940B	    4800     3580000    .00001??  192.0 GB	200GB

   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0          8/92   K. Gacke        Original development
    3.5         1/96    K.Gacke         Port to Solaris
    3.6		9/96	K.Gacke		DLT supported
    3.7		12/96	K.Gacke		STK D-3 supported

*
*       COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*               none
*
*       PROJECT: LAS
*
*       ALGORITHM DESCRIPTION:
*
*       ALGORITHM REFERENCES:
*
******************************************************************************/
#include <stdio.h>
#include <string.h>

#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

extern long Tapelen;

void tplen
(
    struct TAPEDESC *drive
)
{
if (strcmp(drive->dtype,"9-track") == 0)
    {
    if ((strcmp(drive->density,"800") == 0) || 
	(strcmp(drive->density,"LO") == 0))
        {
        drive->tape_bpi = 800;
        drive->tape_irg = .8;
	drive->tape_len = TPLEN_9TRACK * 12;
        }
    else if ((strcmp(drive->density,"1600") == 0) ||
	     (strcmp(drive->density,"MED") == 0))
        {
        drive->tape_bpi = 1600;
        drive->tape_irg = .8;
	drive->tape_len = TPLEN_9TRACK * 12;
        }
    else if ((strcmp(drive->density,"6250")==0)||
             (strcmp(drive->density,"HI")==0)  ||
             (strcmp(drive->density,"0")==0))
        {
        drive->tape_bpi = 6250;
        drive->tape_irg = .4;
	drive->tape_len = TPLEN_9TRACK * 12;
        }
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming 6250\n",drive->density);
        drive->tape_bpi = 6250;
        drive->tape_irg = .4;
	drive->tape_len = TPLEN_9TRACK * 12;
        }
    }
else if (strcmp(drive->dtype,"3480") == 0)
    {
    if ((strcmp(drive->density,"LO") == 0) || (strcmp(drive->density,"0")==0))
        {
        drive->tape_bpi = 31000;
        drive->tape_irg = .11;
	drive->tape_len = TPLEN_3480 * 12;
        }
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
        drive->tape_bpi = 31000;
        drive->tape_irg = .11;
	drive->tape_len = TPLEN_3480 * 12;
        }
    }
else if (strcmp(drive->dtype,"3490") == 0)
    {
    drive->tape_bpi = 62000;
    drive->tape_irg = .0001;
    if ((strcmp(drive->density,"LO") == 0) || (strcmp(drive->density,"0")==0))
	drive->tape_len = TPLEN_3490 * 12;
    else if ((strcmp(drive->density,"HI") == 0) ||
        (strcmp(drive->density,"LOC") == 0)     ||
        (strcmp(drive->density,"HIC") == 0))       /* 20% compression rate */
	{
	drive->tape_len = TPLEN_3490 * 12 * 1.20;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = TPLEN_3490 * 12;
        }
    }
else if (strcmp(drive->dtype,"dlt2000") == 0)
    {
    drive->tape_bpi = 1000000;
    drive->tape_irg = .0002;
    if ((strcmp(drive->density,"LO") == 0) || (strcmp(drive->density,"0")==0))
	drive->tape_len = 1100 * 12;
    else if (strcmp(drive->density,"HI") == 0)  /* 20% compression rate */
	drive->tape_len = 1100 * 12 * 1.20;
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = 1100 * 12;
        }
    }
else if (strcmp(drive->dtype,"dlt4000") == 0)
    {
    drive->tape_bpi = 1100000;
    drive->tape_irg = .0002;
    if ((strcmp(drive->density,"LO") == 0) || (strcmp(drive->density,"0")==0))
	drive->tape_len = TPLEN_DLT * 12;
    else if (strcmp(drive->density,"HI") == 0)  /* 20% compression rate */
	drive->tape_len = TPLEN_DLT * 12 * 1.20;
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = TPLEN_DLT * 12;
        }
    }
else if (strcmp(drive->dtype,"dlt7000") == 0)
    {
    if ((strcmp(drive->density,"LO") == 0) || 		/* dlt 4000 */
	(strcmp(drive->density,"LOC")==0))
	{
        drive->tape_bpi = 1100000;
        drive->tape_irg = .0002;
	drive->tape_len = TPLEN_DLT * 12;
	}
    else if ((strcmp(drive->density,"HI") == 0) ||	/* dlt 7000 */
        (strcmp(drive->density,"HIC") == 0)     ||
        (strcmp(drive->density,"0") == 0))
	{
        drive->tape_bpi = 1925000;
        drive->tape_irg = .0002;
	drive->tape_len = TPLEN_DLT * 12;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = TPLEN_DLT * 12;
        }
    if ((strcmp(drive->density,"HIC") == 0) || 		/* 20% compression */
	(strcmp(drive->density,"LOC")==0))
	{
	drive->tape_len *= 1.2;
	}
    }
else if (strcmp(drive->dtype,"sdlt") == 0)
    {
    drive->tape_bpi = 3440000;
    drive->tape_irg = .00001;
    if ((strcmp(drive->density,"LO") == 0) || 
	(strcmp(drive->density,"HI")==0) ||
	(strcmp(drive->density,"0")==0))
	{
	drive->tape_len = 2400 * 12;
	}
    else if ((strcmp(drive->density,"LOC") == 0) || 
        (strcmp(drive->density,"HIC") == 0))        /* 20% compression rate */
	{
	drive->tape_len = 2400 * 12 * 1.20;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = 2400 * 12;
        }
    }
else if (strcmp(drive->dtype,"lto") == 0)
    {
    drive->tape_bpi = 3620000;
    drive->tape_irg = .00001;
    if ((strcmp(drive->density,"LO") == 0) || 
	(strcmp(drive->density,"HI")==0) ||
	(strcmp(drive->density,"0")==0))
	{
	drive->tape_len = 2400 * 12;
	}
    else if ((strcmp(drive->density,"LOC") == 0) || 
        (strcmp(drive->density,"HIC") == 0))        /* 20% compression rate */
	{
	drive->tape_len = 2400 * 12 * 1.20;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = 2400 * 12;
        }
    }
else if (strcmp(drive->dtype,"lto2") == 0)
    {
    drive->tape_bpi = 3620000;
    drive->tape_irg = .00001;
    if ((strcmp(drive->density,"LO") == 0) || 
	(strcmp(drive->density,"HI")==0) ||
	(strcmp(drive->density,"0")==0))
	{
	drive->tape_len = 4800 * 12;
	}
    else if ((strcmp(drive->density,"LOC") == 0) || 
        (strcmp(drive->density,"HIC") == 0))        /* 20% compression rate */
	{
	drive->tape_len = 4800 * 12 * 1.20;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = 4800 * 12;
        }
    }
else if (strcmp(drive->dtype,"lto3") == 0)
    {
    drive->tape_bpi = 3620000;
    drive->tape_irg = .00001;
    if ((strcmp(drive->density,"LO") == 0) || 
	(strcmp(drive->density,"HI")==0) ||
	(strcmp(drive->density,"0")==0))
	{
	drive->tape_len = 9400 * 12;
	}
    else if ((strcmp(drive->density,"LOC") == 0) || 
        (strcmp(drive->density,"HIC") == 0))        /* 20% compression rate */
	{
	drive->tape_len = 9400 * 12 * 1.20;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = 9400 * 12;
        }
    }
else if (strcmp(drive->dtype,"9940b") == 0)
    {
    drive->tape_bpi = 3580000;
    drive->tape_irg = .00001;
    if ((strcmp(drive->density,"LO") == 0) || 
	(strcmp(drive->density,"HI")==0) ||
	(strcmp(drive->density,"0")==0))
	{
	drive->tape_len = 4800 * 12;
	}
    else if ((strcmp(drive->density,"LOC") == 0) || 
        (strcmp(drive->density,"HIC") == 0))        /* 20% compression rate */
	{
	drive->tape_len = 4800 * 12 * 1.20;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = 4800 * 12;
        }
    }
else if (strcmp(drive->dtype,"9840") == 0)
    {
    drive->tape_bpi = 3185000;
    drive->tape_irg = .0001;
    if ((strcmp(drive->density,"LO") == 0) || 
	(strcmp(drive->density,"HI")==0) ||
	(strcmp(drive->density,"0")==0))
	{
	drive->tape_len = TPLEN_9840 * 12;
	}
    else if ((strcmp(drive->density,"LOC") == 0) || 
        (strcmp(drive->density,"HIC") == 0))        /* 20% compression rate */
	{
	drive->tape_len = TPLEN_9840 * 12 * 1.20;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = TPLEN_9840 * 12;
        }
    }
else if (strcmp(drive->dtype,"8mm") == 0)
    {
    if ((strcmp(drive->density,"LO") == 0) || (strcmp(drive->density,"0")==0))
        {
        drive->tape_bpi = 512000;
        drive->tape_irg = .002;
	drive->tape_len = TPLEN_8MM * 12;
        }
    else if (strcmp(drive->density,"HI") == 0)
        {
        drive->tape_bpi = 1024000;
        drive->tape_irg = .001;
	drive->tape_len = TPLEN_8MM * 12;
        }
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
        drive->tape_bpi = 512000;
        drive->tape_irg = .05;
	drive->tape_len = 300 * 12;
        }
    }
else if (strcmp(drive->dtype,"4mm") == 0)
    {
    if ((strcmp(drive->density,"LO") == 0) || (strcmp(drive->density,"0")==0))
        {
        drive->tape_bpi = 400000;
        drive->tape_irg = .01;
	drive->tape_len = TPLEN_4MM * 12;
        }
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
        drive->tape_bpi = 400000;
        drive->tape_irg = .01;
	drive->tape_len = TPLEN_4MM * 12;
        }
    }
else if (strcmp(drive->dtype,"ait") == 0)
    {
    drive->tape_bpi = 1024000;
    drive->tape_irg = .00001;
    if ((strcmp(drive->density,"LO") == 0) || 
	(strcmp(drive->density,"HI")==0) ||
	(strcmp(drive->density,"0")==0))
	{
	drive->tape_len = 2190 * 12;
	}
    else if ((strcmp(drive->density,"LOC") == 0) || 
        (strcmp(drive->density,"HIC") == 0))        /* 20% compression rate */
	{
	drive->tape_len = 2190 * 12 * 1.20;
	}
    else
        {
        fprintf(stderr,"Unknown density <%s>, assuming denlo\n",drive->density);
	drive->tape_len = 2190 * 12;
        }
    }
if (Tapelen != NULLVAL)
    drive->tape_len = (float)(Tapelen * 12);
/*
*  Subtract 5% from actual tape length
*/
drive->tape_len -= (drive->tape_len * .05);
#ifdef DEBUG
fprintf(stderr,"tplen: len=%8.2f   bpi=%8.2f   irg=%8.2f\n",
	drive->tape_len,drive->tape_bpi,drive->tape_irg);
#endif
}

void tp_setlen
(
    struct TAPEDESC *drive,
    int status
)
{
switch (status)
    {
    case TAPE_EOF:
        drive->tape_len -= (10 * drive->tape_irg);
	break;
    case TAPE_EOV:
        drive->tape_len -= (20 * drive->tape_irg);
	break;
    case TAPE_EOT:
        drive->tape_len -= (30 * drive->tape_irg);
	break;
    default:
	if (status > 0)
            drive->tape_len -= (status / drive->tape_bpi + drive->tape_irg);
	break;
    }
}
