/**************************************************************************
*	
*	NAME: TPMSG
*
*	PURPOSE:  
*		Subroutine writes a message to the operator's console.  On 
*		unix it will write the message to all users logged onto the 
*		group denoted by MTREQ (defined in tapeio.h).
*
   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
*
*	COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
*
*	PROJECT: LAS
*
*	ALGORITHM DESCRIPTION:
*
*	ALGORITHM REFERENCES:
*
******************************************************************************/

#include <stdio.h>
#include <string.h>

#ifdef linux
#include "worgen.h"
#else
#include "las.h"
#endif

#include "tapeio.h"
#include "local_protos.h"

#ifdef vms
#include <opcdef.h>
#include <iodef.h>
#include "desc.h"


int tpmsg
(
    char *msg		/*  message to write to the operator	*/
)
{
	extern char Tape_error[];
	short status;
	long nparm;
	long nonfatal = 1;
	struct 
	{
		char m_type;
		short m_target;
		char m_zero;
		long m_myid;
		char m_text[120];
	}opreq;	
	struct DESC msgdesc;
	char cerr[ERRLEN];

	strcpy(Tape_error,msg);
	nparm = 0;
	opreq.m_type = OPC$_RQ_RQST;
	opreq.m_target = OPC$M_NM_TAPES;
	opreq.m_zero = 0;
	opreq.m_myid = getpid();
	strcpy(opreq.m_text, msg);

	msgdesc.length = 8 + strlen(opreq.m_text);
	msgdesc.addr = &opreq;

	status = sys$sndopr(&msgdesc, 0);
	if (!(status & 1))
	{
		sprintf(cerr,"System error status code is %d", status); 
		tp_errmsg(cerr, "tapeio-status",&nonfatal);
		return(TPOPEN_OPSWR);
	}
return;
}
#endif

#ifdef unix
#include <utmp.h>
#include <grp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

static int send_msg
(
	char	*line,		/* last compoenet of the device name */
	char	*msg		/* the message to be sent */
);

int groupmsg
(
	char	*grpname,		/* name of the operator group */
	char	*msg			/* msg to be written */
);

void tpmsg
(
	char *msg
)
{
extern char Tape_error[];
long nonfatal = 1;
char emsg[CMLEN];

strcpy(Tape_error,msg);
#ifndef linux
if (groupmsg(MTGRP,msg) == 0)
    {
    sprintf(emsg,"%s %s","Error writing message to operator,",
	"sending to local terminal");
    tp_errmsg(emsg," ",&nonfatal);
    printf("%s\n",msg);
    }
#endif
return;
}
/*
 *	write a message to any users logged in with the specified group.
 *	Returns the number of successful writes.
 */
int groupmsg
(
	char	*grpname,		/* name of the operator group */
	char	*msg			/* msg to be written */
)
{
	struct group	*group;		/* operator group */
	char		**grpmem;	/* group members */
	struct utmp	utmp;		/* buffer for utmp records */
	int		fd;		/* file desc. for utmp file */
	char		*utmpname =	/* name of the utmp file */
				"/etc/utmp";
	int		count;		/* number of successful messages */

	count = 0;
	/*
	 *	Get the target group structure, mainly for the list
	 *	of members.
	 */
	group = getgrnam(grpname);
	if (group == (struct group *)0)
		return(0);

	/*
	 *	Read the utmp file to see who is logged on.
	 */
	fd = open(utmpname, 0);
	if (fd < 0)
		return(0);
	while (read(fd, &utmp, sizeof(utmp)) > 0)
		{
#ifndef sgi
		if (utmp.ut_name[0] != '\0')
#endif
#ifdef sgi
		if (utmp.ut_user[0] != '\0')
#endif
			{
			/*
			 *	Check this user against the group
			 *	member list.
			 */
			for (grpmem = group->gr_mem; *grpmem; grpmem++)
				{
#ifndef sgi
#ifdef DGUX
				if ((strcmp(*grpmem, utmp.ut_name) == 0) &&
				    (utmp.ut_type == USER_PROCESS))
#else
				if (strcmp(*grpmem, utmp.ut_name) == 0) 
#endif
#endif
#ifdef sgi
				if ((strcmp(*grpmem, utmp.ut_user) == 0) &&
				    (utmp.ut_type == USER_PROCESS) &&
				    (validpid(utmp.ut_pid) == 0))
#endif
					/*
					 *	Found one.  Open a file
					 *	and write the message.
					 */
					count += send_msg(utmp.ut_line, msg);
				}
			}
		}
	close(fd);
	if (count == 0)		/* try system console */
		count += send_msg("console",msg);
	return(count);
	}
/*
 *	send_msg opens the TTY device and writes the message to it.
 *	Returns the 1 for success, 0 for failure.
 */
static int send_msg
(
	char	*line,		/* last compoenet of the device name */
	char	*msg		/* the message to be sent */
)
{
	struct stat stat_buf;	/* file stat buffer to find mode of file */
	char	devname[80];	/* full pathname of the device */
	int	dev;		/* file descriptor for the TTY device */
	int	value;		/* to be returned */

	/*
	 *	Build the full pathname and open the device.  An error
	 *	probably means the operator turned off messages on that
	 *	terminal, so just return.
	 */
	strcpy(devname,"/dev/");
	strcat(devname, line);
	if (stat(devname,&stat_buf) < 0)	/* ensure world write access */
	    return(0);
/*	printf("tpmsg: st_mode = 0x%x\n",stat_buf.st_mode); */
	if (!(stat_buf.st_mode & 0x2))  /* Ensure world write access */
	    return(0);
	dev = open(devname, 1);
	if (dev < 0)
		return(0);

	/*
	 *	Write the message.
	 */
	if (write(dev, msg, strlen(msg)) > 0)
		value = 1;
	else
		value = 0;
	close(dev);
	return(value);
	}
#endif
