/*******************************************************************************
NAME:				C_PRONUM

PURPOSE:	Returns a projection code from input title string
		These input title strings are checked as defined by the
		"EOSAT Fast Format Document for TM Digital Products,
		Effective March 1, 1990."
		It could become a candidate for a support if need be,
		but if so, should really know whether to check for EOSAT
		definitions, or those expected by the LAS geolib supports,
		because they are different.

PROGRAM HISTORY:
	PROGRAMMER	  DATE		REASON
	----------	  ----		------
	B. Davis	  6/90		Original Development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	C_PROSTR must be run under TAE.

PROJECT:			LAS

ALGORITHM:
	Depending on the input projection title
 	   Return the correct code

******************************************************************************/
#include <string.h>
#include "tae.h"
#include "worgen.h"
#include "util.h"
#include "proj.h"
#include "tm.h"


void c_pronum
(
    char *string,
    long *proj_code
)

{  /*  c_pronum  */
long len;				/* string length		*/

len = (long)strlen(string);

c_low2up(string,&len);

if (strcmp(string,"GEO") == 0)
   {
   *proj_code = GEO;			/* GEOGRAPHIC */
   return;
   }
if (strcmp(string,"UTM") == 0)
   {
   *proj_code = UTM;			/* UTM (universal transverse mercator */
   return;
   }
if (strcmp(string,"STPL") == 0)
   {
   *proj_code = SPCS;			/* STATE PLANE */
   return;
   }
if (strcmp(string,"ACEA") == 0)
   {
   *proj_code = ALBERS;			/* ALBERS CONICAL EQUAL AREA */
   return;
   }
if (strcmp(string,"LCC") == 0)
   {
   *proj_code = LAMCC;			/* LAMBERT CONFORMAL CONIC */
   return;
   }
if (strcmp(string,"MER") == 0)
   {
   *proj_code = MERCAT;			/* MERCATOR */
   return;
   }
if (strcmp(string,"PS") == 0)
   {
   *proj_code = PS;			/* POLAR STEREOGRAPHIC */
   return;
   }
if (strcmp(string,"TM") == 0)
   {
   *proj_code = TM;			/* TRANSVERSE MERCATOR */
   return;
   }
if (strcmp(string,"LAEA") == 0)
   {
   *proj_code = LAMAZ; 			/* LAMBERT AZIMUTHAL EQUAL AREA */
   return;
   }
if (strcmp(string,"OM") == 0)
   {
   *proj_code = HOM;			/* OBLIQUE MERCATOR (Hotine) */
   return;
   }
if (strcmp(string,"SOM") == 0)
   {
   *proj_code = SOM;			/* SOM (Space Oblique Mercator) */
   return;
   }
else
   *proj_code = USDEF; 			/* USER DEFINED */

}  /* pronum  */
