/******************************************************************************
FUNCTION:	GET_IMGFD

PURPOSE:	Reads the image file descriptor record to determine the
		number of lines and samples in the image file

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development

ALGORITHM 
  Read file descriptor record from tape
  Get number of lines from file descriptor record
  Get number of samples from file descriptor record

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <stdio.h>
#include "las.h"     
#include "tapeio.h"
#include "tm.h"

int get_imgfd
(
    long *unit,		/* pointer to tape unit number			     */
    long *nl,		/* pointer to number of output lines		     */
    long *ns		/* pointer to number of output samples per line	     */
)
{
long mode = HANDLE;	/* tape I/O outputs error messages		     */
long nbytes = 4320;	/* number of bytes to be read from tape  	     */
long status;		/* function return status code			     */

char temp[5];	/* temporary variable used to get nl and ns	     */
unsigned char inbuf[4320];/* input data buffer				     */

/*
  Read file descriptor record from tape
*/
status = c_tpread(unit,inbuf,&nbytes,&mode);
if (status != nbytes)
    return(TIPS_READ);

/*
  Get number of lines from file descriptor record
*/
strncpy(temp,(char *)&inbuf[240],4);
temp[4] = '\0';
sscanf(temp,"%d",nl);

/*
  Get number of samples from file descriptor record
*/
strncpy(temp,(char *)&inbuf[252],4);
temp[4] = '\0';
sscanf(temp,"%d",ns);

return(E_SUCC);
}
