/******************************************************************************
FUNCTION:	SOMPRM

PURPOSE:	Fills the 15 parameter values necessary for the SOM projection

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development
  2.0		12/90		BALTZER		Check return status's

ALGORITHM 
  Initialize the SOM parameter values
  Fill the following paramters:
	Semi-major axis of ellipsoid
	Flattening of the ellipsoid
	Eccentricity squared of the ellipsoid
	Inclination of orbit at ascending node
	Longitude of ascending orbit at equator
	Period of satellite revolution in minutes
	Lansat ratio to compensate for confusion at northern end of orbit

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <math.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "tm.h"


int somprm
(
    long *path,		/* WRS path designator				*/
    double parms[]		/* SOM projection parameters			*/
)
{
double f;		/* Flattening of the ellipsoid			*/
double var;		/* Inclination of orbit at ascending node	*/

long i;			/* Loop index					*/

/*
  Initialize the parameter values
*/
for (i = 0; i < COEFCT; i++)
    parms[i] = 0.0;

/*
  Semi-major axis of ellipsoid
*/
parms[0] = 6378388.0;

/*
  Flattening of the ellipsoid
*/
f = 1.0 / 297.0;

/*
  Eccentricity squared of the ellipsoid
*/
parms[1] = (f * 2.0) - (f * f);

/*
  Inclination of orbit at ascending node
*/
var = 98.20;
if ( c_degdms(&var,&parms[3],"DEG","CIR") != E_SUCC )
   return(E_FAIL);

/*
  Longitude of ascending orbit at equator
*/
var = 127.760513 - (360.0 / 233.0) * ((double)(*path) - 1.0);
if (var < -180.0)
    var = var + 360.0;
else if (var > 180.0)
    var = var - 360.0;

if ( c_degdms(&var,&parms[4],"DEG","LON") != E_SUCC )
   return(E_FAIL);

/*
  Period of satellite revolution in minutes
*/
parms[8] = 98.88412;

/*
  Lansat ratio to compensate for confusion at northern end of orbit
*/
parms[9] = 0.52016130;

return(E_SUCC);
}
