/******************************************************************************
FUNCTION:	UTMPRM

PURPOSE:	Fills the 15 parameter values necessary for the UTM projection
		based on zone.

PROGRAM HISTORY:
VERSION		DATE		PROGRAMMER	DESCRIPTION
-------		----		----------	-----------
  1.0		4/89		HOLLAREN	Original development
  2.0		12/90		BALTZER		Check return status's
  7.1		1/99		J. Willems	Took out assignment of semi-
						major and minor since it was 
						hard-coded to Interntl 1909.

ALGORITHM 
  Initialize the UTM parameter values
  Fill the following paramters:
  	Scale factor at central meridian
  	Longitude of central meridian
  	False easting
  	False northing

ALGORITHM REFERENCES	none
*******************************************************************************/

#include <math.h>
#include "las.h"
#include "proj.h"
#include "geompak.h"
#include "tm.h"


int utmprm
(
    long *zone,		/* UTM zone number				*/
    double *lat,		/* Scene center latitude			*/
    double parms[]		/* UTM parameters				*/
)
{
double var;		/* Longitude of central meridian		*/

long i;			/* Looping index				*/

/*
  Initialize the UTM parameter values
*/
for (i = 0; i < COEFCT; i++)
    parms[i] = 0.0;

/*
  Scale factor at central meridian
*/
parms[2] = 0.9996;

/*
  Longitude of central meridian
*/
var = 6.0 * (double)(*zone) - 183.0;
if ( c_degdms(&var,&parms[4],"DEG","LON") != E_SUCC )
   return(E_FAIL);

/*
  False easting
*/
parms[6] = 500000.0;

/*
  False northing
*/
if (*lat < 0) 
    parms[7] = 10000000.0;

return(E_SUCC);
}
