/*
**************************************************************************
NAME: C_ABORT

FUNCTION: Issue fatal error message, delete files, and abort

PROGRAM HISTORY:
  Version       Date       Author             Request
  -------       ----       ------             -------
   1          27-JUL-88  K. Narayanan, SAR    

**************************************************************************/
#include "las.h"
#include "util.h"

/**
 * las_abort:
 * @block: I: TAE parameter block.  It is assumed that TAE has been
 *            initialized before calling las_abort().
 * @message: I: Fatal error message issued.
 * @key: I: Error message key.
 * @prog_name: I: Program name.
 * @files: I: Files to be deleted.
 * @num_files: I: Number of files to be deleted.
 *
 * The las_abort() routine issues a fatal error message, deletes the 
 * indicated files, and aborts.
 */
void las_abort
(
    struct PARBLK *block,       /* I: TAE parameter block                   */
    const char message[],       /* I: error message                         */
    const char key[],           /* I: error message key                     */
    const char prog_name[],     /* I: program name                          */
    const char files[][CMLEN],  /* I: files to be deleted                   */
    int   num_files             /* I: number of files to be deleted         */
)
{

    c_lasmsg (TRUE, key, message);
    las_clean (block, prog_name, files, num_files);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_abort:
 * @block: I: TAE parameter block.  It is assumed that TAE has been
 *            initialized before calling las_abort().
 * @message: I: Fatal error message issued.
 * @key: I: Error message key.
 * @prog_name: I: Program name.
 * @files: I: Files to be deleted.
 * @num_files: I: Number of files to be deleted.
 *
 * The c_abort() routine issues a fatal error message, deletes the 
 * indicated files, and aborts.
 *
 * Deprecated.  Use las_abort() instead.
 */
void c_abort
(
    struct PARBLK *block,       /* I: TAE parameter block                   */
    const char message[],       /* I: error message                         */
    const char key[],           /* I: error message key                     */
    const char prog_name[],     /* I: program name                          */
    const char files[][CMLEN],  /* I: files to be deleted                   */
    const long *num_files       /* I: number of files to be deleted         */
)
{
    
    las_abort (block,message,key,prog_name,files,*num_files);
}

#endif /* obsolete interfaces */

 
