/*******************************************************************************
NAME                            C_CLEAN

PURPOSE         

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        B.AILTS         05/88   Original development 
        B.AILTS         08/88   Changed $DELFLAG to $DELFLG
        L.HUEWE         09/88   Added call to LSSTAT to make sure the file
                                exists before deleting it

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        C_CLEAN must be run under TAE.

PROJECT                 LAS

ALGORITHM 
        Retrieve the TAE global value $DELFLAG
        If $DELFLAG is set to 'YES' then
           for each output file
                if the output file exists
                    delete the output file
        exit

ALGORITHM REFERENCES            none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "labserv.h"

/**
 * las_clean_noexit:
 * @vblock: I: TAE parameter block.  It is assumed that TAE has been 
 *             initialized prior to calling c_clean().
 * @pname: I: Name of the application program.
 * @hnames: I: Array of output host file names to be deleted if $DELFLG
 *             equals YES.
 * @count: I: Number of output host file names specified in "hnames".
 *
 * The las_clean_noexit() routine is used only to delete the list of output
 * file names specified by the parameter "hnames" if $DELFLG equals YES. It
 * does not fatal error. las_clean() should be used if the caller wants this
 * support utility to fatal error and not return.
 */
void las_clean_noexit
(
    struct PARBLK *vblock,       /* I:  TAE parameter block                 */
    const char hnames[][CMLEN],  /* I:  output file names                   */
    int count                    /* I:  number of output files              */
)
{
    struct VARIABLE *val;

    int   i;                     /* loop index                              */
    int   mode=0;                /* Indicates LSSTAT is to check for
                                    the existence of the file               */
    char delflag[4];

    val = p_find(vblock,"$DELFLG");
    strcpy(delflag,SVAL(*val,0));

    if (strcmp(delflag,"YES") == 0)
    {
        for (i = 0; i < count; i++)
        {
            if (c_labstat(hnames[i],mode) == E_SUCC)
                c_delfil(hnames[i]);
        }
    }
}

/**
 * las_clean:
 * @vblock: I: TAE parameter block.  It is assumed that TAE has been 
 *             initialized prior to calling c_clean().
 * @pname: I: Name of the application program.
 * @hnames: I: Array of output host file names to be deleted if $DELFLG
 *             equals YES.
 * @count: I: Number of output host file names specified in "hnames".
 *
 * The las_clean() routine is used to output a fatal error message.  It also
 * deletes the list of output file names specified by the parameter "hnames"
 * if $DELFLG equals YES.
 */
void las_clean
(
    struct PARBLK *vblock,       /* I:  TAE parameter block                 */
    const char *pname,           /* I:  program name                        */
    const char hnames[][CMLEN],  /* I:  output file names                   */
    int count                    /* I:  number of output files              */
)

{
    char key[30];

    las_clean_noexit (vblock,hnames,count);
    sprintf (key,"%s-fatal",pname);
    c_lasmsg (TRUE, key, "Fatal error encountered");
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_clean:
 * @vblock: I: TAE parameter block.  It is assumed that TAE has been 
 *             initialized prior to calling c_clean().
 * @pname: I: Name of the application program.
 * @hnames: I: Array of output host file names to be deleted if $DELFLG
 *             equals YES.
 * @count: I: Number of output host file names specified in "hnames".
 *
 * The c_clean() routine is used to output a fatal error message.  It also
 * deletes the list of output file names specified by the parameter "hnames"
 * if $DELFLG equals YES.
 *
 * Deprecated.  Use las_clean() instead.
 */
void c_clean
(
    struct PARBLK *vblock,       /* I:  TAE parameter block                 */
    const char *pname,           /* I:  program name                        */
    const char hnames[][CMLEN],  /* I:  output file names                   */
    const long *count            /* I:  number of output files              */
)
{
    las_clean (vblock,pname,hnames,*count);
}

#endif /* obsolete interfaces */


