/******************************************************************************
NAME:   CPASSO

FUNCTION:
        CPASSO is called to copy all files associated with the input 
        image (have same root name) to files associated with the output image. 

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         4/91       D. VanderZee initial development


PROJECT:        LAS

ALGORITHM:
        If the TAE global $CPASSO = "YES" then the input image(s) associated 
        files are copied to the output image.

******************************************************************************/
#include <string.h>
#include "las.h"
#include "util.h"
#include "local_protos.h"


void c_cpasso
(
    struct PARBLK *vblock,      /* I: pointer to TAE parameter block        */
    const char *hosout,         /* I: output image name                     */
    const char (*hosin)[CMLEN], /* I: input image names                     */
    int numin                   /* I: number of images specified in "hosin" */
)

{
    struct VARIABLE *var;       /* the value of the parameter               */
    char    cpflag[4];          /* copy associated files flag               */

    int  copyflg = TRUE;        /* dummy value for processfile              */
    int  imgset = TRUE;         /* img & assoc files? (t=1)                 */
    int  imageflg = FALSE;      /* Copy image flag                          */
    int  acount = 1;            /* associated file count                    */
    int  ncopied = 0;           /* # files copied                           */

    static char aftype[100][CMLEN]; /* List of associated file extensions   */
    char name[CMLEN];               /* current cm TAE filename              */
    char newname[CMLEN];            /* new cm TAE filename                  */
    char msg[81];                   /* output message                       */

    var = p_find(vblock,"$CPASSO");
    strcpy (cpflag,SVAL(*var,0));

    if (strncmp(cpflag,"YES",3) == 0)
    {
        if (numin == 1)
        {
            strcpy(name, hosin[0]);
            name[strlen(name)-3]='\0';
            strcpy(newname, hosout);
            newname[strlen(newname)-3]='*'; 
            newname[strlen(newname)-2] = '\0';
            strcpy(aftype[0],"*");

            las_processfile (vblock,"cmcopy",name,newname,imgset,imageflg,
                             aftype,acount,0,&ncopied,copyflg);

            if (ncopied != 0)
            {     
                sprintf (msg,"Number of associated files copied: %d\n",
                         ncopied - 3);
                c_lasmsg (FALSE, " ", msg);
            }
        }
        else
        {
            c_lasmsg (FALSE, " ", "Multiple input files specified, "
                      "associated files not copied.");
        }
    }
}

