/*******************************************************************************
NAME                            C_GETCNT

PURPOSE         Retrieves the number of input files specified by the user

PROGRAM HISTORY
        PROGRAMMER        DATE          REASON
        ----------        ----          ------
        B. Ailts          5/88          Original development.
        D. VanderZee    07/90   Standardized error handling 
                                


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        C_GETCNT must be run under TAE.

PROJECT                         LAS

ALGORITHM:
        get the count of the input image parameter

ALGORITHM REFERENCES

******************************************************************************/

#include "las.h"
#include "util.h"

/**
 * c_getcnt:
 * @block: I: TAE parameter block passed by reference.  It is assumed that
 *            TAE has been initialized prior to calling c_getcnt().
 * @param_buf: I: Defines the parameter name in the procedure which contains
 *                the input image(s) specification.  In most cases, the 
 *                value of this parameter is IN.
 * @count: O: Returns the number of logical images specified by the input
 *            parameter array specified by "param_buf".
 *
 * The c_getcnt() routine is used to retrieve the number of logical images
 * specified in the input image specification.  It is used when the 
 * number of logical input images that can be specified by the user is not
 * known.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_getcnt
(
    struct PARBLK *block,       /* I:  TAE parameter block                  */
    const char *param_buf,      /* I:  Input image parameter name           */
    int  *count                 /* O:  Element array number of param.       */
)

{
    struct VARIABLE *i_spec;    /* point to param_buf in PARBLK             */


    /* Get image specification and parse it.                                */
    if( NULL == (i_spec = p_find( block, param_buf )))
    {
        c_lasmsg (FALSE, "getcnt-param",
                 "Error: Input image parameter does not exist");
        return(E_FAIL);
    }

    /*  Retrieve the count                                                  */
    *count = (*i_spec).v_count;

    return(E_SUCC);
}  /*  getcnt  */
