/******************************************************************************
NAME:   

FUNCTION:

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         12/88      GSFC         Original development
     5.0        07/90      D. VanderZee Standardized error handling 


PROJECT:        LAS

ALGORITHM:

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "labserv.h"


/**
 * las_history_close:
 * @fd: I: File descriptor of input image's history file.
 * @hosin: I: Input host name.
 *
 * The las_hasclosed() routine calls c_labclose() which is used to close
 * the input image's history file.  
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int las_history_close
(
    int fd,                 /* I: File descriptor of input images history   */
    const char hosin[]      /* I: Input host name                           */
)
{
    int   action = 0;
    int   stat;
    char  inhist[CMLEN];    /* buffer for input history host file name      */

    c_lsmknm (hosin, "his", inhist);

    stat = c_labclose (fd,inhist,action);
    if (stat == E_FAIL)   
        c_lasmsg (FALSE, "hsclos-call", "Error returned from lsclos");

    return (stat);
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_hsclos:
 * @fd: I: File descriptor of input image's history file.
 * @hosin: I: Input host name.
 *
 * The c_hsclos() routine calls c_labclose() which is used to close
 * the input image's history file.  
 *
 * Deprecated.  Use las_history_close() instead.
 *
 * Returns:   E_SUCC:  Successful completion of operation,
 *            E_FAIL:  Unsuccessful completion of operation.
 */
int c_hsclos
(
    const long *fd,         /* I: File descriptor of input images history   */
    const char hosin[]      /* I: Input host name                           */
)
{
    int status;
    
    status = las_history_close (*fd,hosin);

    return (status);

}

#endif /* obsolete interfaces */

