/******************************************************************************
NAME:   

FUNCTION:

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
   1.0          12/88      GSFC         Original Development
     5.0        07/90      D. VanderZee Standardized error handling 


PROJECT:        LAS

ALGORITHM:

******************************************************************************/

#include <stdio.h>
#include "las.h"
#include "util.h"
#include "labserv.h"

/**
 * las_history_open:
 * @hosin: I: Input image name.
 * @access: I: Type of read/write access to the history file to be opened.
 *             Valid values are:
 *             Read, 
 *             Write (existing file is overwritten),
 *             Write (append to end of existing file),
 *             Read/write (replace a record).
 *
 * The las_history_open() routine calls c_labopen() to open the associated file.
 *
 * Returns:  The file descriptor,
 *           A negative number is returned if unsuccessful completion of 
 *           operation.
 */
int las_history_open
(
    const char hosin[],     /* I: Input image name                          */
    int access              /* I: Type of access to input image hist        */
)

{

    int fd;
    char  inhist[CMLEN];     /* buffer for input history host file name     */

    c_lsmknm (hosin, "his", inhist);

    fd = c_labopen (inhist,access); 
    if (fd < 0) 
        c_lasmsg (FALSE, "hsopen-call", "Error returned from lsopen");

    return (fd);
}

#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_hsopen:
 * @fd: I: File descriptor of input image's history file.
 * @hosin: I: Input image name.
 * @access: I: Type of read/write access to the history file to be opened.
 *             Valid values are:
 *             Read, 
 *             Write (existing file is overwritten),
 *             Write (append to end of existing file),
 *             Read/write (replace a record).
 *
 * The c_hsopen() routine calls c_labopen() to open the associated file.
 *
 * Deprecated.  Use las_history_open() instead.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
long c_hsopen
(
    int   *fd,               /* O: File descriptor of input images hist */
    const char hosin[],      /* I: Input image name                     */
    const long *access       /* I: Type of access to input image hist   */
)
{

    *fd = las_history_open (hosin,*access);

    if (fd < 0) 
        return (E_FAIL);
    return (E_SUCC);    
}

#endif /* obsolete interfaces */


