/****************************************************************************

NAME: c_reset_handler

PURPOSE: Reset the signal handler to its original state.

RETURN VALUE:
Type = int
Value           Description
-----           -----------
E_SUCC          Successful completion
E_FAIL          An error occurred during processing

PROGRAM HISTORY:
Version  Date   Programmer       Code  Reason
-------  -----  ---------------  ----  -------------------------------------
  1.0    01/01  G.Schmidt        EDC   Original development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
  Must be run under TAE.

PROJECT: LAS

NOTES:

****************************************************************************/

#include <stdio.h>
#include <signal.h>
#include "worgen.h"
#include "util.h"
#include "local_protos.h"

int c_reset_handler
(
    int signo,                          /* I: signal to reset               */
    const struct sigaction *signal_act  /* I: specifies the action previously
                                              associated with signal so it can 
                                              be restored                   */
)

{
    int status;             /* return status from sigaction                 */
    struct sigaction signal_oact;
                            /* specifies where the action previously associated
                              with signal is to be stored on return from
                              sigaction                                     */

    /* Reset the signal handler for signo                                   */
    status = sigaction (signo, signal_act, &signal_oact);
    if (status != 0)
        return (E_FAIL);

    return (E_SUCC);
}
