/*******************************************************************************
FUNCTION: C_splitname

PURPOSE:  Split a file specification into a directory, a root file name and an
          extension.  The last character of the directory path will be either
          a '/' or ']'.

PROGRAM HISTORY:

Version  Date   Author           Reason
-------  -----  ---------------  -----------------------------------------------
  1.0    07/92  J.Fenno          Development


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
         Run under TAE

PROJECT: ADAPS

*******************************************************************************/
#include <stdio.h>        /* Needed for standard i/o                   */
#include <string.h>
#include "las.h"
#include "util.h"



/**
 * c_splitname:
 * @filespec: I: File specification to be split into three parts.
 * @directory: O: Directory portion of file specification after the split 
 *                occurs.
 * @root: O: Root portion of the file name after the split occurs.
 * @extension: O: Extension portion of the file name after the split occurs.
 *
 * The c_splitname() routine splits a file specification into a directory,
 * a root file name, and an extension.  The last character of the
 * directory path will be either a '/' or ']'.
 */

 
void c_splitname
(
    const char filespec[CMLEN],  /* I:  File specification                  */
    char directory[CMLEN],       /* O:  Directory portion of filespec       */
    char root[CMLEN],            /* O:  Root portion of the file name       */
    char extension[CMLEN]        /* O:  Extension portion of the file name  */
)

{
    char file_name[CMLEN];       /* Local copy of filespec                  */
    char *ptr;                   /* String pointer                          */
    char *ptr2;                  /* String pointer                          */


    /* Make a local copy of filespec so it is not destroyed                 */
    strcpy(file_name, filespec);

    /* Check for a directory path                                           */
    ptr  = strchr (file_name, HOST_DIR_BEG);
    ptr2 = strrchr(file_name, HOST_DIR_END);
    if ((ptr != NULL) && (ptr != ptr2))        /* watch out for /filename   */
    {
        strcpy(directory, file_name);
        ptr = strrchr(directory, HOST_DIR_END);
        ptr++;
        strcpy(file_name, ptr);
        *ptr = '\0';
    }
    else if ((ptr != NULL) && (ptr == ptr2))  /* watch for filedir/filename */
    {
        strcpy(directory, file_name);
        ptr  = strchr (directory, HOST_DIR_BEG);
        ptr++;
        *ptr = '\0';
        ptr2++;
        strcpy(file_name, ptr2);
    }
    else
        strcpy(directory, "");


    /* Check for a file extension                                           */
    ptr = strrchr(file_name, '.');
    if (ptr != NULL)
    {
        *(ptr++) = '\0';
        strcpy(root, file_name);
        strcpy(extension, ptr);
    }
    else
    {
        strcpy(root, file_name);
        strcpy(extension, "");
    }
}
