/*****************************************************************************
FUNCTION:       C_4DIGITYR

PURPOSE:        Convert a 2- or 3- digit year in the range of 0 to 199
                into a 4-digit year returning the 4-digit year and leaving
                the input parameter unchanged.  3-digit year inputs are
                assumed to be in [2000,2099].  4-digit years are returned
                unchanged.  For all other values an error is generated.

Version  Date   Programmer      Modifications
-------  -----  --------------- ---------------------------------------------
  1.0    07/98  D.Lloyd         Initial development

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        Run under TAE

PROJECT:        LAS
*****************************************************************************/
#include <stdio.h>
#include <time.h>
#include "las.h"
#include "util.h"

#define  XX  50
#define  YY  5


/**
 * c_4digityr:
 * @year: I: Year value to be converted.
 *
 * The c_4digityr() routine converts a 2- or 3- digit year in the range of 0
 * to 199 into a 4-digit year returning the 4-digit year and leaving the
 * input parameter unchanged.  3-digit year inputs are assumed to be in 
 * [2000, 2099]. 4-digit years are returned unchanged.  For all other values
 * an error is generated.
 *
 * Returns: A 4-digit year from conversion.
 */
int  c_4digityr
(
    int  year       /* I: year value to be converted                        */
)
{
    struct  tm  *lctm;              /* local time structure                 */
    int     lyear;                  /* local derived year value             */
    time_t  t;                      /* seconds since Jan 1, 1970            */

    t = time( NULL );               /* get the local time                   */
    lctm = localtime( &t );         /* fill the tm structure                */


    /* 2-digit year.  If the value is less than XX (tbd -- say 50)
       then we will assume the year is > 2000 unless adding 2000
       would make the date more than YY (tbd -- say 5) years in the
       future.  Example: year=03, lctm->tm_year=1998 assume year is
       2003, however if year=04 assume 1904.                 */

    if ( year >= 0 && year < 100 )
    {
        if ( year < XX )
        {
            lyear = year + 2000;
            if ( lyear > ( lctm->tm_year + 1900 + YY ))
                lyear -= 100;
        }
        else
            lyear = year + 1900;
    }

    /* 3-digit year.  Assumed to be a count of the number of
       years since 1900 so simply add 1900                   */                               
    else if ( year >= 100 && year < 200 )
        lyear = year + 1900;

    /* Any 4-digit year is just returned                     */
    else if ( year >= 1000 && year < 2100 )
        lyear = year;

    /* Year outside of acceptable range is an error those in [200, 999]     */
    else
    {
        c_lasmsg(0,"4digityr-val","Year out of range");
        lyear = -1;
    }

    return lyear;
}

