#include <stdio.h>
#include <string.h>
#include "tae.h"
#include "las.h"
#include "desc.h"
#include "util.h"

/**
 * c_getgbl:
 * @vblock: I: TAE parameter block passed by reference.  It is assumed that
 *             TAE has been initizialied prior to calling las_getstr().
 * @histry: O: History flag.  Copying history records can consume time and
 *             disk space.  Specify NO if the history records from the
 *             input images are not needed.  If NO specified, one record
 *             will still be created which lists all the parameter values
 *             for the procedure.
 *             YES:  Copy input history files,
 *             NO: Do not copy input history files.
 * @minmax: O: Minmax flag.  Determines if minimum and maximum image values
 *             are updated by the application modules. 
 *             YES: Update image min/max values.  The minimum and maximum
 *                  pixel values and validity flags are updated.
 *             NO:  Do not update the min/max values.  The output minimum
 *                  and maximum values are NOT updated by the application
 *                  modules.
 * @msgtim: O: Message timing.  Determines the timing interval for the
 *             processing messages generated by an application module.
 *             The value defines the number of clock seconds between
 *             messages.
 * 
 * The c_getgbl() routine gets the TAE globals that most programs will need.
 */
void c_getgbl
(
    struct PARBLK *vblock, /* I: TAE parameter block passed by reference    */
    char histry[],         /* O: History flag                               */
    char minmax[],         /* O: Minmax flag                                */
    int  *msgtim           /* O: Message timing                             */
)
    /* This routine will get the TAE globals that most programs will need   */
{
    struct      VARIABLE *var;          /* Pointer into variable block      */

    var = p_find (vblock, "$MSGTIM");
    *msgtim = IVAL (*var, 0);

    var = p_find (vblock, "$HISTRY");
    strcpy (histry, SVAL (*var, 0));

    var = p_find (vblock, "$MINMAX");
    strcpy (minmax, SVAL (*var, 0));
}

