/******************************************************************************

NAME:   GETSYS

FUNCTION:
        GETSYS retrieves the character data stored in the environment
        variable(logical) DATASYS and returns it to the calling program.

PROGRAM HISTORY:
  Version       Date       Author       Request
  -------       ----       ------       -------
    1.0         10/87       B.Ailts     initial development
    1.1         12/87       B.Ailts     Change include directory specification
                                        Replace DESC arguments with char. string
                                        Place bridge routines in a seperate file
    1.2         04/88       B.Ailts     Replaced newlas.h with las.h
                                        Replaced SYSTEM with LASSYS
    1.3         12/90       B.Ailts     Updated error messages
    1.4          4/91       B.Ailts     Replaced LASSYS with DATASYS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
                Run under TAE

PROJECT:        LAS

ALGORITHM:
        The character data stored in the environment variable(logical)
         DATASYS is retrieved using the utility GETENV.  
        This data is then returned to the calling program.

ALGORITHM REFERENCES:
******************************************************************************/
#define SYSTEM "DATASYS"
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"

/**
 * c_getsys:
 * @system: O: The name of the system on which the processing is currently
 *             being done.
 *
 * The c_getsys() routine is used to retrieve the DATASYS environment
 * variable or logical.
 *
 * Returns:  E_SUCC:  Successful completion of operation,
 *           E_FAIL:  Unsuccessful completion of operation.
 */
int c_getsys
(
    char *system         /* O: System of CPU                                */
)

{


    char *temp;          /*  temporary pointer                              */

    temp = getenv(SYSTEM);
    if (temp == NULL)
    {
        c_lasmsg (FALSE, "getsys-system",
                "Environment variable(logical) DATASYS could not be found");
       return(E_FAIL);
    }

    strcpy(system,temp);

    return(E_SUCC);
}
