/************************************************************************
 *                                                                      *
 *    Purpose:  C_INDEX checks if string2 is a substring of string1 and *
 *              returns the position (1-relative) of the substring      *
 *              in string1.  If string2 is not a substring of string1,  *
 *              zero is returned as the position.  If either string is  *
 *              null, a zero is returned as the position.               *
 *                                                                      *
 *    By:  S. Contractor, Code 933.1/NASA/GSFC, 5/24/84                 *
 *    Modifications:                                                    *
 *      C. Greenhagen, USGS/EROS Data Center, 5/88                      *
 *         DMS v3_0 retrocode.                                          *
 *      L. Huewe, USGS/EROS Data Center, 10/90                          *
 *         Taken from the DMS routine DTSUBS and made into a general    *
 *         world routine.                                               *
 *                                                                      *
 ***********************************************************************/
#include <string.h>
#include "las.h"
#include "util2.h"



#ifndef NO_OBSOLETE_INTERFACE


/**
 * c_index:
 * @string1: I: The ASCII character string to be searched.  The position of
 *              the character substring specified by "string2" is returned
 *              in the parameter "pos".
 * @string2: I: The ASCII character substring to be searched for in the input
 *              string specified by "string1".  The position of "string2"
 *              within "string1" is returned in the parameter "pos".
 * @pos: O: The position of "string2" within "string1".  The value returned
 *          is 1 relative and 0 is returned is "string2" is not found within
 *          "string1".
 *
 * The c_index() routine determines whether "string2" is a substring of 
 * "string1" and returns the postion (1 relative) of the substring.  If
 * either string is a null string, a 0 is returned as the position.  Note
 * that this routine is callable from C only.
 *  
 * Deprecated.  Use the standard C routine strstr instead.
 */
void c_index
(
    const char string1[], /* I: String for comparison.                      */
    const char string2[], /* I: Possible substring.                         */
    int  *pos             /* O: Pos. of string2 in string1(if not substring)*/
)
{
    int   len1;           /* String lengths.                            */
    int   len2;
    const char *s1;       /* Pointers into input strings.               */
    const char  *addr;

    /* Set pointer to start of string1,determine the length of both strings */
    s1 = string1;
    len1 = strlen(string1);
    len2 = strlen(string2);

    if ((len1 == 0) || (len2 == 0))
    {
        *pos = 0;
        return;
    }

    for( ; ; )
    {
        /* Find the address of the first occurrence in the search string 
           of the first character from string2.                             */
        addr = strchr (s1,*string2);

        /* If the address returned is zero then the first character from
           string2 is not in the search string.  Set the position 
           variable to zero indicating that string2 is not a substring
           of string1.                                                      */
        if (addr == 0)
        {
            *pos = 0;
            break;
        }

        /* If string2 is equal to the portion of string1 starting at the 
           address just found above and having the same length as string2,
           then string2 is a substring of string1.  Calculate the position
           of the substring in string1.                                     */
        if (strncmp(addr,string2,len2) == 0)
        {
            *pos = len1 - strlen(addr) + 1;
            break;
        }
        /* If string2 is not a substring at the address just found,
           increment the pointer into string1 to start at the next digit
           (address found above plus one).  This will in effect shorten
           the search string so that this portion of string1 can be searched
           for the substring string2.                                       */
        else
            s1 = addr + 1;
    }
        
}


#endif /* obsolete interfaces   */

