/******************************************************************************
NAME                            C_LOW2UP

PURPOSE         Upper cases an input string

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        5/7/85  Original development for NEWLAS
        D. Hollaren     04/88   changed include spec to las.h
                                removed call to squeeze routine
        B.Ailts         06/88   Changed include spec to worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        C_LOW2UP must be run under TAE.

PROJECT                         LAS

ALGORITHM DESCRIPTION:
   For each character of the string
      If lower case
         Change to upper case

ALGORITHM REFERENCES            none
******************************************************************************/
#include <ctype.h> 
#include "las.h"
#include "util.h"

/**
 * las_low2up:
 * @buf: I/O: On input, "buf" is the ASCII character string that may contain
 *            upper and lowercase characters.  On output, "buf" is the ASCII
 *            character string with all uppercase characters.  The
 *            nonalphabetic and uppercase characters are unchanged.
 *
 * The las_low2up() routine is used to convert an ASCII character string
 * to uppercase.
 */
void las_low2up
(
    char *buf         /* I/O: buffer to convert                            */
)
{
    char *ptr;
    int  size;

    size = strlen (buf);

    ptr = buf;
    for(; size > 0;size--,  ptr++)
        *ptr = toupper(*ptr);
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_low2up:
 * @buf: I/O: On input, "buf" is the ASCII character string that may contain
 *            upper and lowercase characters.  On output, "buf" is the ASCII
 *            character string with all uppercase characters.  The
 *            nonalphabetic and uppercase characters are unchanged.
 * @size: I: Number of bytes in "buf" that are to be converted.  
 *
 * The c_low2up() routine is used to convert an ASCII character string
 * to uppercase.
 *
 * Deprecated.  Use las_low2up() instead.  las_low2up() converts the whole
 * string and if c_low2up() is being used to convert a partial string, it
 * may need to be undeprecated.
 */
void c_low2up
(
    char *buf,         /* I/O: Buffer to convert                            */
    const long *size   /* I: Size of string in buffer                       */
)

{

    long cnt;
    char *ptr;

    cnt = *size;
    ptr = buf;
    for(; cnt--; ptr++)
        *ptr = toupper(*ptr);
}


#endif /* obsolete interfaces   */

