/*******************************************************************************
NAME                            C_UP2LOW

PURPOSE         Lower cases an input string

PROGRAM HISTORY
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        D.GORDON        5/7/85  Original development for NEWLAS
        D. Hollaren     04/88   changed include spec to las
                                removed call to squeeze routine
        B.Ailts         06/88   Changed include spec to worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
        C_UP2LOW must be run under TAE.

PROJECT                         LAS

ALGORITHM DESCRIPTION:
   For each character of the string
      If upper case
         Change to lower case

ALGORITHM REFERENCES            none
*******************************************************************************/
#include <ctype.h>

#include "las.h"
#include "util.h"

/**
 * las_up2low:
 * @buf: I/O: On input, "buf" is the ASCII character string that may 
 *            contain upper and lowercase characters.  On output, "buf" is
 *            the ASCII character string with all lowercase characters.
 *            The nonalphabetic and lowercase characters are unchanged.
 *
 * The las_up2low() routine is used to convert an ASCII character string
 * to lowercase.
 */
void las_up2low
(
   char *buf  /* I/O: Buffer converted from upper - lower */
)
{
    char *ptr;
    int size;

    size = strlen (buf);

    ptr = buf;
    for(; size > 0; size--, ptr++)
        *ptr = tolower(*ptr);
}


#ifndef NO_OBSOLETE_INTERFACE

/**
 * c_up2low:
 * @buf: I/O: On input, "buf" is the ASCII character string that may 
 *            contain upper and lowercase characters.  On output, "buf" is
 *            the ASCII character string with all lowercase characters.
 *            The nonalphabetic and lowercase characters are unchanged.
 * @size: I: Number of bytes in "buf" that are to be converted.
 *
 * The c_up2low() routine is used to convert an ASCII character string
 * to lowercase.
 *
 * Deprecated.  Use las_up2low() instead.
 */
void c_up2low
(
   char *buf,        /* I/O: Buffer converted from upper - lower */
   const long *size  /* I: Number of bytes to convert in buffer  */
)
{


    long cnt;
    char *ptr;

    cnt = *size;
    ptr = buf;
    for(; cnt--; ptr++)
        *ptr = tolower(*ptr);

}

#endif /* obsolete interfaces   */



