#include <string.h>     /* memset prototype */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "las.h"
#include "lasfileio.h"

#define BUFSIZE 8192
#define TESTFILE1 "./test1.file"
#define TESTFILE2 "./test2.file"


void fillbuf
(
    void *buffer,
    int bufsize,
    int size
)
{
    int index; 
    long long value = 0x0102030405060708LL;

    if (size == 1)
    {
        unsigned char *buf = (unsigned char *)buffer;

        for (index = 0; index < bufsize; index++)
        {
            buf[index] = (unsigned char)value;
            value = value + (value >> 8);
        }
    }
    else if (size == 2)
    {
        short int *buf = (short int *)buffer;

        for (index = 0; index < bufsize/2; index++)
        {
            buf[index] = (short int)value;
            value = value + (value >> 8);
        }
    }
    else if (size == 4)
    {
        int *buf = (int *)buffer;

        for (index = 0; index < bufsize/4; index++)
        {
            buf[index] = (int)value;
            value = value + (value >> 8);
        }
    }
    else if (size == 8)
    {
        long long *buf = (long long *)buffer;

        for (index = 0; index < bufsize/8; index++)
        {
            buf[index] = value;
            value = value + (value >> 8);
        }
    }
}

int main(void)
{
    int bufsizes[4] = {1, 16, 1024, 8192};
    int size_index;
    int bytes;
    unsigned char writebuf[BUFSIZE];
    unsigned char readbuf[BUFSIZE];
    int fd;
    FILE *fp;

    /* open a file for writing a test pattern of data to a file */
    printf("using c_write\n");
    fd = open(TESTFILE1,O_WRONLY|O_CREAT, S_IRUSR|S_IWUSR);
    if (fd < 0)
    {
        printf("Error opening file\n");
        exit(1);
    }

    /* repeatedly write data to the file with different sizes */
    for (bytes = 1; bytes < 10; bytes++)
    {
        /* fill the buffer with a test pattern */
        fillbuf(writebuf,BUFSIZE,bytes);
    
        for (size_index = 0; size_index < 4; size_index++)
        {
            if (c_write(fd, writebuf, bufsizes[size_index], bytes) != 
                    bufsizes[size_index])
            {
                printf("Error writing file with word size %d and buffer "
                       "size %d\n", bytes, bufsizes[size_index]);
            }
        }
    }
    
    /* close the file */
    close (fd);

    /* open the file for reading */
    printf("using c_read\n");
    fd = open(TESTFILE1,O_RDONLY);
    if (fd < 0)
    {
        printf("Error opening read file\n");
        exit(1);
    }

    /* repeatedly read data to the file with different sizes */
    for (bytes = 1; bytes < 10; bytes++)
    {
        /* fill the buffer with a test pattern */
        fillbuf(writebuf,BUFSIZE,bytes);
    
        for (size_index = 0; size_index < 4; size_index++)
        {
            if (c_read(fd, readbuf, bufsizes[size_index], bytes) != 
                    bufsizes[size_index])
            {
                printf("Error reading file with word size %d and buffer "
                       "size %d\n", bytes, bufsizes[size_index]);
            }
            else
            {
                /* read successfully, so verify data is correct */
                if (memcmp(readbuf, writebuf, bufsizes[size_index]) != 0)
                {
                    printf("Incorrect data read for word size %d and buffer "
                           "size %d\n", bytes, bufsizes[size_index]);
                }

                /* clear the read buffer after each successful read */
                memset(readbuf,0,bufsizes[size_index]);
            }
        }
    }

    close(fd);
    
    /* open a file for writing a test pattern of data to a file */
    printf("using c_fwrite\n");
    fp = fopen(TESTFILE2,"wb");
    if (fp == NULL)
    {
        printf("Error opening file\n");
        exit(1);
    }

    /* repeatedly write data to the file with different sizes */
    for (bytes = 1; bytes < 10; bytes++)
    {
        /* fill the buffer with a test pattern */
        fillbuf(writebuf,BUFSIZE,bytes);
    
        for (size_index = 0; size_index < 4; size_index++)
        {
            if (c_fwrite(writebuf, bytes, bufsizes[size_index]/bytes, fp) != 
                    bufsizes[size_index]/bytes)
            {
                printf("Error writing file with word size %d and buffer "
                       "size %d\n", bytes, bufsizes[size_index]);
            }
        }
    }
    
    /* close the file */
    fclose (fp);

    /* open the file for reading */
    printf("using c_fread\n");
    fp = fopen(TESTFILE2,"rb");
    if (fp == NULL)
    {
        printf("Error opening read file\n");
        exit(1);
    }

    /* repeatedly read data to the file with different sizes */
    for (bytes = 1; bytes < 10; bytes++)
    {
        /* fill the buffer with a test pattern */
        fillbuf(writebuf,BUFSIZE,bytes);
    
        for (size_index = 0; size_index < 4; size_index++)
        {
            if (c_fread(readbuf, bytes, bufsizes[size_index]/bytes, fp) !=
                    bufsizes[size_index]/bytes)
            {
                printf("Error reading file with word size %d and buffer "
                       "size %d\n", bytes, bufsizes[size_index]);
            }
            else
            {
                /* read successfully, so verify data is correct */
                if (memcmp(readbuf, writebuf, bufsizes[size_index]) != 0)
                {
                    printf("Incorrect data read for word size %d and buffer "
                           "size %d\n", bytes, bufsizes[size_index]);
                }

                /* clear the read buffer after each successful read */
                memset(readbuf,0,bufsizes[size_index]);
            }
        }
    }

    fclose(fp);
    return 0;
}
