/*****************************************************************************
NAME:		ASCII2DLTF

PURPOSE:	

PROGRAM HISTORY:
VERSION	     DATE	PROGRAMMER   CODE/CONT	REQUEST
-------	     ----  	----------   ---------	-------

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	ASCII2DLTF must be run under TAE.

PROJECT:	LAS

ALGORITHM:

*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "dltf.h"
#include "protos.h"

#define RED 	0
#define GREEN	1
#define BLUE	2
#define DMSLEN	12

int main
(void)
{
long	i,j,		/* Loop indices; working variables.		*/
	status,		/* Return status indicator. 			*/
	red,		/* Red value.					*/
	green,		/* Green value.					*/
	blue,		/* Blue value.					*/
	index,		/* Gray level value.				*/
	zero = 0,	/* Constant zero.				*/
	maxgray = 255,  /* Maximum gray level value constant.         	*/
	fatal = -1,	/* Specifies a fatal error.			*/
	one = 1,	/* Specifies a non-fatal error.			*/
	fd;		/* File descriptor for the DLTF file. 		*/

double	slope = 1.0,	/* Slope of the LUT.				*/
	offset = 0.0;	/* Offset of the LUT.				*/

FILE	*ifp;		/* Pointer to output file. 			*/

char	hostnm[CMLEN],  /* Host name of the DLTF assoiciated file.	*/
	dtype[3],       /* Data type string.                          	*/
	infile[CMLEN],
	dltfnam[DMSLEN+1];/* Name of the DLTF entry to be written. 	*/

    
unsigned char   lut[3][256];    
			/* Red, Green and Blue look-up tables.		*/

struct ASDLTF asdltf;	/* Structure to contain DLTF information for a
			   detailed listing of a DLTF entry.		*/

struct PARBLK vblock;	/* TAE variable block.				*/

/* Get the user specified parameters.
  -----------------------------------*/
getpar(&vblock,dltfnam,infile,hostnm);

/*
    Always store the LUT as byte.
*/
strcpy(dtype,"B");

/*
    Always assign three LUTs (red, green and blue).
*/
asdltf.numbnd = 3;

/*
    Assign the band numbers.
*/
asdltf.band[RED] = 1;
asdltf.band[GREEN] = 2;
asdltf.band[BLUE] = 3;

/*
    Assign the window.
*/
for (i=0; i < 4; i++)
    asdltf.window[i] = 0; 

asdltf.gzoom[0] = asdltf.gzoom[1] = 1;
asdltf.gshift[0] = asdltf.gshift[1] = 0;

/*
    Check for duplicate DLTF entries and open the DLTF for output.
*/
status = c_ckdltf(hostnm,dltfnam,&fd);
if (status != E_SUCC)
    {
    c_errmsg("Error opening DLTF","ascii2dltf-open",&one);
    c_errmsg("Fatal error encountered","ascii2dltf-fatal",&fatal);
    }

/*
    Write the first DLTF record.
*/
status = c_w1dltf(&fd,dltfnam,"Generated from ascii2dltf",dtype,&asdltf);
if (status != E_SUCC)
    {
    c_errmsg("Error writing DLTF record","ascii2dltf-write",&one);
    c_errmsg("Fatal error encountered","ascii2dltf-fatal",&fatal);
    }

/*
    Process the LUT for each band specified; each becomes the second type
    of DLTF record.  Note also that the j variable is used to generate the
    LUT numbers of the second DLTF record.  These numbers are appended to the
    the DLTF entry name to form the key for each of the second record types.
*/

/*
    Open the input ascii file.
*/
ifp = fopen(infile,"r");
if (ifp == NULL)
    {
    c_errmsg("Error opening input file","ascii2dltf-inopen",&one);
    c_errmsg("Fatal error encountered","ascii2dltf-fatal",&fatal);
    }

/*
    Initialize the LUTs to linear.
*/
for (i=0; i < 256; i++)
    lut[RED][i] = lut[GREEN][i] = lut[BLUE][i] = (unsigned char) i; 

status = fscanf(ifp,"%d %d %d %d",&index,&red,&green,&blue);
while (status != EOF)
    {
    lut[RED][index] = (unsigned char) red;
    lut[GREEN][index] = (unsigned char) green;
    lut[BLUE][index] = (unsigned char) blue;
    status = fscanf(ifp,"%d %d %d %d",&index,&red,&green,&blue);
    }

/*
    Write the LUTs to the DLTF entry.
*/
for (i=0, j=1; i < 3; i++, j++)
    {
    status = c_w2dltf(&fd,dtype,dltfnam,&slope,&offset,&zero,&maxgray,&j,
	lut[i]);
    if (status != E_SUCC)
        {
        c_errmsg("Error writing DLTF record","ascii2dltf-write",&one);
        c_errmsg("Fatal error encountered","ascii2dltf-fatal",&fatal);
        }
    }

/*
    Close the DLTF.
*/
status = c_cldltf(&fd,hostnm);
if (status != E_SUCC)
    c_errmsg("Error closing DLTF","ascii2dltf-close",&one);

/* Issue a standard completion message.
   ------------------------------------*/
fclose(ifp);
c_complt("ASCII2DLTF",hostnm,&one);

return EXIT_SUCCESS;

}
