/***************************************************************************

PURPOSE:	Performs filtering of an image to reduce or eliminate the 
		swathing or banding found in Landsat TM images. 

PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1           7/91     M. Choate      Driver program for BADLINE

PROJECT: 	LAS

ALGORITHM DESCRIPTION:
***************************************************************************/
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

#define MAX_BAND       7	/* maximum number of bands in TM scene */
#define SL             0
#define SS             1
#define NL             2
#define NS             3

#define MAXSAMP      7000
#define MAXBUFSIZE 280000	/* 7000 samps * 40 lines */
#define TOPOFBUF   273000
#define HEIGHT         13

#define INC(x)    x = (x >= TOPOFBUF) ? 0 : x + MAXSAMP

struct PARBLK parblk;           /* TAE parameter block       */

static long first;
static long line;

struct FDESC * infd;                   /* file descriptors          */
struct FDESC * outfd;                  /* file descriptors          */
long    min_max;                /* minmax option flag        */
long    msgtim;                 /* message display interval  */

IMPORT  long    curcat;         /* parameters for processing */
IMPORT  long    curent;         /* message utility           */
long    fatal = -1;
long    inf_err = 1;

long    zero = 0;
long    one = 1;
char *prog_name = "BADLINE";    /* program name               */
char		hosout[1][CMLEN];	/* host name of output image  */

int main
(void) 
{
double		line_inc = 1;        /* line interval for upddr        */
static double	maxval[MAX_BAND+1];  /* output image maximum value     */
static double	minval[MAX_BAND+1];  /* output image minimum value     */
double		samp_inc = 1;        /* sample interval for upddr      */

long		acc;
static long	bands[MAXIMG][MAXBND+1];    /* image band numbers    */
long		bndout;              /* output band number             */
long		comb_flag = NOCOMB;  /* for upddr                      */
long		curimg;              /* the current image counter      */
long		i;                   /* a counter                      */
long		intype;              /* input data type                */ 
static long	nbands[MAXIMG];      /* number of bands in each image  */
long		nl;                  /* number of lines in image       */
long		nmrimg;              /* number of input images         */ 
long		ns;                  /* number of samples/line in image*/
long		onecat = ONECAT;     /* for inittm                     */
long		opt[2];              /* for eopenr                     */
long		otype;               /* output image data type         */
long		status;              /* function return status code    */ 
long		totbnd;              /* total number of bands          */
long		bndin;
static long	window[MAXIMG][4];   /* window for each input image    */

static char	hosin[MAXIMG][CMLEN];	/* host names of input images  */
char		taeout[1][CMLEN];		/* TAE name of output image    */

unsigned char outbuf[MAXSAMP];
static unsigned char buf[MAXBUFSIZE];

long numbands;
long in;
long in1,in2,in3,in4,in5,in6,in7,in8,in9,in10;
long lineno;
float tol;

/* Get user-specified parameters and tae global variables
---------------------------------------------------------*/
getpar (hosin, taeout, hosout, &totbnd, bands, nbands, &nl, &ns,
        window, &nmrimg, &tol);

/* Force the input image data to byte.  This will have to change if one wants
   to process images in other data types
  -------------------------------------*/
intype = otype = EBYTE;

/* Initialize display message
-----------------------------*/
c_inittm("BADLINE","Processed","line","band"," ",&nl,&totbnd,&msgtim,&onecat);

/* Initialize min and max arrays
--------------------------------*/
c_inmnmx (&otype, &totbnd, minval, maxval);

/* Open output image
--------------------*/
opt[0] = 0;
opt[1] = 0;
acc = IWRITE;
status=c_eopenr(&outfd,hosout[0],&totbnd,&one,&one,&nl,&ns,&acc,&otype,opt,&one);
if(status!=E_SUCC)c_errmsg("Fatal error encountered","badline-fatal",&fatal);

/* Initialize output band counter
--------------------------------*/
bndout = 0;
curent = curcat = 1;
c_startm ();
for (curimg = 0; curimg < nmrimg; curimg++)
   {

/* Open input image
-------------------*/
   numbands = 0;
   acc = IREAD;
   status=c_eopenr(&infd,hosin[curimg],&numbands,&(window[curimg][SL]),
	&(window[curimg][SS]),&(window[curimg][NL]),&(window[curimg][NS]),
	&acc,&intype,&zero,&one);
   if(status!=E_SUCC)c_errmsg("Fatal error encountered","badline-fatal",&fatal);

/* Process each band
--------------------*/
   bndin = 1;
   for (i = 0; i < nbands[curimg]; i++, curcat++, bndin++)
      {
      init_get_line();
      bndout++;

/* Process each line
  -----------------*/
      curent = 1;
      for (lineno = 1; lineno <= nl; lineno++, curent++)
	{
	get_line(bndin, nl, ns, buf, &in, &in1, &in2, &in3,                             &in4, &in5, &in6, &in7, &in8, &in9, &in10);
       filter2(&(buf[in]),&(buf[in1]),&(buf[in2]),&(buf[in3]),                             &(buf[in4]),&(buf[in5]),&(buf[in6]),&(buf[in7]),                                &(buf[in8]),&(buf[in9]),&(buf[in10]),tol,nl,ns,lineno,outbuf);

/* Write completed line to output image
 --------------------------------------*/
        status = c_ewrite (&outfd, &bndout, &lineno, outbuf, &one);
        if (status  !=  ns) 
            {
            c_eclose (&outfd);
            c_abort (&parblk, "Fatal error encountered", "badline-fatal", 
                     prog_name, hosout, &one);
            }
	}
    }

c_stoptm ();

/* Close input and output images
--------------------------------*/
   status = c_eclose (&infd);
   if (status  !=  E_SUCC) 
      c_errmsg ("Nonfatal error encountered", "badline-warn", &inf_err);
   status = c_eclose (&outfd);
   if (status  !=  E_SUCC) 
      c_errmsg ("Nonfatal error encountered", "badline-warn", &inf_err);
   }

/* Update ddr for output image
-----------------------------*/
c_upddr (&parblk, hosin, hosout[0], &nmrimg, window, bands,
         nbands, &line_inc, &samp_inc, &totbnd, &comb_flag);

/* Create associated history file for output image
--------------------------------------------------*/
status = c_hstout(&parblk, hosout[0], hosin, &nmrimg, " ");

/* Output completion message
----------------------------*/
c_complt ("BADLINE", taeout, &one);

return EXIT_SUCCESS;

}

/**************************************************************************
NAME: 				GETPAR

PURPOSE: Gets the user-specified parameters and tae globals

******************************************************************/
void getpar
(
    char hosin[][CMLEN],
    char taeout[][CMLEN],
    char hosout[][CMLEN],
    long *totbnd,
    long bands[][MAXBND+1],
    long nbands[],
    long *nl,
    long *ns,
    long window[][4],
    long *nmrimg,
    float *tol
)
{
struct VARIABLE *var;
long	maximg = MAXIMG;
long	qflag[MAXIMG][2];
long	same;
long	status;
long	std_out;
long    dtype;

char	minmax[4];
char    taein[MAXIMG][CMLEN];

/* Initialization
-----------------*/
c_inlas(&parblk,&std_out);

/* Get globals                     
--------------*/
var = p_find(&parblk, "$msgtim");
msgtim = IVAL(*var, 0);

var = p_find(&parblk, "$minmax");
strcpy( minmax, SVAL(*var, 0));
if (minmax[0] == 'Y') min_max = TRUE;
else min_max = FALSE;

/* Get input file information
-----------------------------*/
status = c_getin(&parblk, "IN", &zero, taein, hosin, nmrimg, window, bands, 
                 nbands, totbnd, &maximg, &same, qflag);
if(status!=E_SUCC)c_errmsg("Fatal error encountered","badline-fatal",&fatal);

if (same  !=  1) 
   c_errmsg ("Input images do not have the same window sizes", 
             "badline-window", &fatal);

status = c_gettyp(hosin, nmrimg, &dtype);
if (status  !=  E_SUCC) 
   {
   c_errmsg ("Input images are not of same data type", 
             "badline-input", &inf_err);
   c_errmsg ("Fatal error encountered", "badline-fatal", &fatal);
   } 

if ((dtype  ==  ELONG)  ||  (dtype  ==  EREAL) || (dtype == EWORD)) 
    c_errmsg
      ("The badline filter allows only BYTE input", 
       "badline-dtype", &fatal);

*nl = window[0][NL];
*ns = window[0][NS];

/* Get output file name
-----------------------*/
status = c_getout (&parblk, "OUT", taeout, hosout);
if (status  !=  E_SUCC) 
   c_errmsg ("Fatal error encountered", "badline-fatal", &fatal);

/* Get kernel parameters
------------------------*/
var = p_find(&parblk, "TOLVAL");
*tol = RVAL(*var, 0);


return;
}

void init_get_line
(void)
{
first = 1;
line = 1;
return;
}


void get_line
(
    long bndin,
    long nl,
    long ns,
    unsigned char *buf,
    long *in,
    long *in1,
    long *in2,
    long *in3,
    long *in4,
    long *in5,
    long *in6,
    long *in7,
    long *in8,
    long *in9,
    long *in10
)
{
long status;
long i;
long counter;

counter=(HEIGHT-1)/2;
if (first)
   {
   *in = *in10 = -MAXSAMP;
   *in1 = *in2 = *in3 = *in4 = *in5 = -MAXSAMP;
   *in6 = *in7 = *in8 = *in9 = 0;
   for (i = 0; i < counter; line++, i++)
      {
      INC(*in10);
      status = c_eread (&infd, &bndin, &line, &(buf[*in10]), &one);
      if (status  !=  ns)
        {
        c_eclose(&outfd);
        c_abort(&parblk, "Fatal error encountered", "badline-fatal",
               prog_name, hosout, &one);
        }   
      }
   first = 0;
   *in6 = *in10-(4*MAXSAMP);
   *in7 = *in10-(3*MAXSAMP);
   *in8 = *in10-(2*MAXSAMP);
   *in9 = *in10-(MAXSAMP);
   }
if( line > nl )
   {
   INC(*in5);
   INC(*in4);
   INC(*in3);
   INC(*in2);
   INC(*in1);
   INC(*in);
   }
else
   {
   INC(*in);
   if(line > counter+2 ) INC(*in5);
   if(line > counter+3 ) INC(*in4);
   if(line > counter+4 ) INC(*in3);
   if(line > counter+5 ) INC(*in2);
   if(line > counter+6 ) INC(*in1);
   if(line < (nl-4)) INC(*in6);
   if(line < (nl-3)) INC(*in7);
   if(line < (nl-2)) INC(*in8);
   if(line < (nl-1)) INC(*in9);
   if(line < nl ) INC(*in10);
   status = c_eread (&infd, &bndin, &line, &(buf[*in10]), &one);
   if (status  !=  ns)
        {
        c_eclose(&outfd);
        c_abort(&parblk, "Fatal error encountered", "badline-fatal",
               prog_name, hosout, &one);
        }   
   line++;
   }
return;
}
