/**************************************************************************
NAME: FILTER2 

PURPOSE: Corrects single line detector drops in the image.
         
PROGRAM HISTORY:
  Version	Date       Author	      Request
  -------	----	   ------             -------
    1         1/91      M. Choate & J. Hood 

PROJECT: LAS

ALGORITHM REFERENCES:  Designed by Dr. Dennis Helder, South Dakota State
		       University Electrical Engineering Department.
*************************************************************************/
#include "las.h"
#include <math.h>
#include "protos.h"

#define HEIGHT 13
#define MAXSAMPS 7000
#define NULL_CORRECT 100.0

void filter2
(
    unsigned char inbuf[],     /* input buffer of current line */
    unsigned char inbuf1[],    /* buffers used to determine correction values */
    unsigned char inbuf2[],
    unsigned char inbuf3[],
    unsigned char inbuf4[],
    unsigned char inbuf5[],
    unsigned char inbuf6[],
    unsigned char inbuf7[],
    unsigned char inbuf8[],
    unsigned char inbuf9[],
    unsigned char inbuf10[],
    float tolrn,			/* tolerance value for rejection */
    long nl,			/* number of lines to be processed */
    long ns,			/* number of samples per line to be processed */
    long iinline,		/* input line number */
    unsigned char outbuf[]     /* output buffer */
)
{
register float	sumz;	      /* summing variable */
register float   zcf;
register long	j;
register long	nz;
long	l;
long    i;
long    ntop;                 /* counter for top correction values */
long    nbottom;              /* counter for bottom correction values */

float topcf;                  /* final top correction value */
float bottomcf;               /* fianl bottom correction value */
float zbuf[MAXSAMPS];         /* intermediate buffer for correction value */
float top[5];                 /* buffer for top correction values */
float bottom[5];              /* buffer for bottom correction values */
float hold;                   /* temporary storage variable */

char linecorrect[MAXSAMPS];

unsigned char *optr;
unsigned char *iptr; 
char *bptr;
long temp;

/* Find correction data line by line 
  ---------------------------------*/

if(iinline<=HEIGHT)
  {
  for(j=0 ; j<ns ; j++)
    {

/* Find a lower data point 
   ----------------------*/

    nbottom=0; 
    bottomcf = 0.0;
    if(abs((int)(inbuf[j]-inbuf6[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf6[j]);
       nbottom++;
       }
    if(abs((int)(inbuf[j]-inbuf7[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf7[j]);
       nbottom++;
       }
    if(abs((int)(inbuf[j]-inbuf8[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf8[j]);
       nbottom++;
       }
    if(abs((int)(inbuf[j]-inbuf9[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf9[j]);
       nbottom++;
       }
    if(abs((int)(inbuf[j]-inbuf10[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf10[j]);
       nbottom++;
       }

/* Find the median value of valid data points 
   -----------------------------------------*/ 

    if(nbottom >= 3)
       {
       for(i=0 ; i<=2 ; i++)
          {
          for(l=i+1 ; l<nbottom ; l++)
             {
             if(bottom[l]<bottom[i])
               {
               hold=bottom[i];
               bottom[i]=bottom[l];
               bottom[l]=hold;
               }
             }
          }
       if(nbottom==3 || nbottom==4)
          bottomcf=bottom[1];
       else
          bottomcf=bottom[2];
       zbuf[j]=bottomcf;
       }
    else
       zbuf[j]=NULL_CORRECT;
    }
}

else if((iinline>HEIGHT) && (iinline<(nl-HEIGHT)))
    {
    for(j=0 ; j<ns ; j++)
       {
    ntop=0;
    topcf=0.0;

/* Find the upper data point 
   ------------------------*/

    if(abs((int)(inbuf[j]-inbuf5[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf5[j]);
       ntop++;
       }
    if(abs((int)(inbuf[j]-inbuf4[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf4[j]);
       ntop++;
       }
    if(abs((int)(inbuf[j]-inbuf3[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf3[j]);
       ntop++;
       }
    if(abs((int)(inbuf[j]-inbuf2[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf2[j]);
       ntop++;
       }
    if(abs((int)(inbuf[j]-inbuf1[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf1[j]);
       ntop++;
       }

/* Find the median value of the valid upper data points 
   ---------------------------------------------------*/

    if(ntop >= 3)
       {
       for(i=0 ; i<=2 ; i++)
          {
          for(l=i+1 ; l<ntop ; l++)
             {
             if(top[l]<top[i])
               {
               hold=top[i];
               top[i]=top[l];
               top[l]=hold;
               }
             }
          }
        if(ntop==3 || ntop==4)
           topcf=top[1];
        else 
           topcf=top[2];
        }
    else
       topcf=NULL_CORRECT;

/* Find the lower data points
   -------------------------*/

    bottomcf=0.0;
    nbottom=0; 
    if(abs((int)(inbuf[j]-inbuf6[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf6[j]);
       nbottom++;
       }
    if(abs((int)(inbuf[j]-inbuf7[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf7[j]);
       nbottom++;
       }
    if(abs((int)(inbuf[j]-inbuf8[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf8[j]);
       nbottom++;
       }
    if(abs((int)(inbuf[j]-inbuf9[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf9[j]);
       nbottom++;
       }
    if(abs((int)(inbuf[j]-inbuf10[j]))<=tolrn)
       {
       bottom[nbottom]=(float)(inbuf[j])-(float)(inbuf10[j]);
       nbottom++;
       }

/* Find the median value of valid lower data points
   -----------------------------------------------*/

    if(nbottom >= 3)
       {
       for(i=0 ; i<=2 ; i++)
          {
          for(l=i+1 ; l<nbottom ; l++)
             {
             if(bottom[l]<bottom[i])
               {
               hold=bottom[i];
               bottom[i]=bottom[l];
               bottom[l]=hold;
               }
             }
          }
       if(nbottom==3 || nbottom==4)
          bottomcf=bottom[1];
       else
          bottomcf=bottom[2];
       }
    else
       bottomcf=NULL_CORRECT;
if((topcf!=NULL_CORRECT)&&(bottomcf!=NULL_CORRECT))
   {

/* Find the smallest of the upper and lower data points 
   ---------------------------------------------------*/

     if(abs(topcf)<=abs(bottomcf))
       zbuf[j]=topcf;
     else
       zbuf[j]=bottomcf;
   }
else if((topcf!=NULL_CORRECT)&&(bottomcf==NULL_CORRECT))
   zbuf[j]=topcf;
else if((topcf==NULL_CORRECT)&&(bottomcf!=NULL_CORRECT))
   zbuf[j]=bottomcf;
else 
   zbuf[j]=NULL_CORRECT;
   }
}
   

else if(iinline>=(nl-HEIGHT))
    {
    for(j=0 ; j<ns ; j++)
       {
    ntop=0;
    topcf=0.0;

/* Find the upper data point
   ------------------------*/

    if(abs((int)(inbuf[j]-inbuf5[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf5[j]);
       ntop++;
       }
    if(abs((int)(inbuf[j]-inbuf4[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf4[j]);
       ntop++;
       }
    if(abs((int)(inbuf[j]-inbuf3[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf3[j]);
       ntop++;
       }
    if(abs((int)(inbuf[j]-inbuf2[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf2[j]);
       ntop++;
       }
    if(abs((int)(inbuf[j]-inbuf1[j]))<=tolrn)
       {
       top[ntop]=(float)(inbuf[j])-(float)(inbuf1[j]);
       ntop++;
       }
    if(ntop >= 3)
       {

/* Find the median value of the valid data points
   ---------------------------------------------*/

       for(i=0 ; i<=2 ; i++)
          {
          for(l=i+1 ; l<ntop ; l++)
             {
             if(top[l]<top[i])
               {
               hold=top[i];
               top[i]=top[l];
               top[l]=hold;
               }
             }
          }
        if(ntop==3 || ntop==4)
           topcf=top[1];
        else 
           topcf=top[2];
        zbuf[j]=topcf;
        }
    else
       zbuf[j]=NULL_CORRECT;
    }
}

/* Now do a horizontal smoothing of the correction data using a 35 pixel wide 
   window, starting at left edge
  -----------------------------*/
sumz = 0.0;
nz = 0;
for (j=0;j<=25;j++)
    {
    if (zbuf[j] != NULL_CORRECT)
        {
        sumz = sumz + zbuf[j];
        nz++;
        }
    }
if(nz != 0) zcf = sumz/(float)nz;
else zcf = 0.0;
if(zcf<0.0) 
   temp = (long) (zcf - 0.5);
else
   temp = (long) (zcf + 0.5);
linecorrect[j] = (char)temp;

for (j=1;j<=25;j++)
    {

    if (zbuf[j+25] != NULL_CORRECT)
        {
        sumz = sumz + zbuf[j+25];
        nz++;
        }
    if (nz != 0) zcf = sumz/(float)nz;
    else zcf = 0.0;
if(zcf <0.0) 
  temp = (long) (zcf - 0.5);
else
  temp = (long) (zcf + 0.5);
linecorrect[j] = (char)temp; 
    }

/* Calculate correction factor for rest of the line
  -------------------------------------------------*/
for (j=26;j<=ns-25;j++) 		/* Maybe should be: j < ns-17; */
    {
    if (zbuf[j+25] != NULL_CORRECT)
       {
       sumz = sumz + zbuf[j+25];
       nz++;
       }
    if (zbuf[j-25] != NULL_CORRECT)
       {
       sumz = sumz - zbuf[j-25];
       nz--;
       }
    if(nz != 0) zcf = sumz/(float)nz;
    else zcf = 0.0;

    if(zbuf[j]!=NULL_CORRECT) 
       {
       if(zcf<0.0) 
           temp = (long) (zcf-0.5); 
       else
           temp = (long) (zcf+0.5);
       linecorrect[j]=(char)temp;
       }
    else linecorrect[j] = 0;
    }

/* Calculate for right edge of image
  ---------------------------------*/
for (j=ns-24;j<ns;j++) 		/* Maybe should be:j = ns-17; */
    {

    if (zbuf[j-25] != NULL_CORRECT)
        {
        sumz = sumz - zbuf[j-25];
        nz--;
        }
    if (nz != 0) zcf = sumz/(float)nz;
    else zcf = 0.0;

    if (zbuf[j]!=NULL_CORRECT)
       {
       if(zcf<0.0) 
           temp = (long) (zcf-0.5);
       else
           temp = (long) (zcf+0.5);
       linecorrect[j]=(char)temp;
       }
    else linecorrect[j]=0;
    }

/* NOW APPLY CORRECTION FACTOR TO IMAGE
  ------------------------------------*/
optr = &(outbuf[0]);
iptr = &(inbuf[0]);
bptr = &(linecorrect[0]);
for (j=ns;j--;)
    {
    temp = *iptr++ - *bptr++;
    temp = (temp < 0) ? 0: temp;
    *optr++ = (unsigned char) ((temp > 255) ? 255: temp);
    }
return;
}
