
/******************************************************************************
NAME:     cwheel.c 
PURPOSE:  program main() 

PROGRAM HISTORY
-------------------------------------------------------------------------------
Date      Programmer       Changes
--------  ---------------  ----------------------------------------------------
    1991  David Zokaites   Original development as an engineering tool that
                           required connecting several applications.
Oct 1995  David Zokaites   Development of CLRWHEEL application
Nov 1995  David Zokaites   Chargrey now varies from chip to chip instead of 
			   being constant, released application to $LASCONTRIB 
******************************************************************************/
#include <string.h>  /* for calling strcpy() and strlen() */ 
#include "cwheel.h"  /* application header file */ 
#include "util.h"

 
/* variables common to main() and errclean() */ 
struct OUTIMAGE outimage; /* info on output las image */ 
struct ANPROC   anproc;   /* proc to run annotate */ 


/* PROGRAM MAIN */ 
int main ()                 
{	                    
/* declare variables */ 
   char outfiles[2][CMLEN];/* output file names, parameter to c_complt() */ 
   long nouts,             /* number of output files, parameter to c_complt() */
	std_out;           /* standard output, parameter to c_inlas() */ 
   struct IGEN    igen;    /* image generation parameters */ 
   struct OUTINFO outinfo; /* processing parameters for output image */ 
   struct PARBLK  pblock;  /* tae parameter block */
   struct UPARMS  uparms;  /* user tae parameters */ 

/* initialize las and tae */ 
   c_inlas (&pblock, &std_out); 

/* initialize image status flags used for error handling, must precede getpar()
   for correct error handling */
   out_flags (&outimage); 

/* get user parameters */ 
   getpar (&pblock, &uparms, &igen); 

/* get memory for image generating luts */ 
   if (igen.chipsflag)
      get_memory (&igen); 

/* calculate image generation parameters */ 
   image_parms (&uparms, &igen);

/* set variables for chips subcommand */ 
   if (igen.chipsflag)
      {
      /* set ring2chip, number of chips in each color ring */
	 set_r2c (&uparms, &igen); 
      /* allocate memory */ 
	 get_memory2 (&igen); 
      /* set anglelim and huelut */ 
	 set_ah (&igen); 
      }

/* open output image after setting processing parameters */ 
   outinfo.parmname  = "OUT";      /* image name is in parameter OUT */ 
   outinfo.dtypein   = NOTAPPLY;   /* there is no input image data type */ 
   outinfo.dtypeap   = EBYTE;      /* image data processed as BYTE */ 
   outinfo.dtypeout  = NOTAPPLY;   /* don't convert data upon writing to disk */
   outinfo.iomode    = BILIO;      /* process image in BIL mode */
   outinfo.blocksize = NOTAPPLY;   /* block size is not applicable */ 
   outinfo.nl        = igen.nlns;  /* set image size */ 
   outinfo.ns        = igen.nlns;  /* set image size */ 
   outinfo.nbands    = NBANDS;     /* set image size */ 
   open_outimage (&pblock, &outinfo, &outimage); 

/* create image */ 
   make_image (&pblock, &igen, &outimage); 

/* make ddr & history file */ 
   finish_out2 (&pblock, &outimage);

/* get values for anproc;  set dummy return value if anproc is not created */ 
   if (!uparms.addtext)
      anproc.returned[0] = "";
   else
      {
      /* get host name for anproc */ 
	 newname ("clrwheel", ".pdf", anproc.hostname); 
      /* set value to return to tae, deleting file extension */ 
	 anproc.returned[0] = anproc.rmemory;  /*get memory for returned value*/
	 strncpy (anproc.returned[0], anproc.hostname, 
	         strlen(anproc.hostname) - strlen(".pdf")  );
      /* get tae name for c_complt().  3 function calls are needed to get full
	 path of anproc */
         strcpy(anproc.taename, anproc.hostname);
	 if (c_cmhost(anproc.taename, anproc.hostname) != E_SUCC)
	    errclean ("cwheel-fatal", "Fatal error encountered"); 
         strcpy(anproc.taename, anproc.hostname);
      }

/* optionally create a pdf to annotate the color chips.  Must come after the 
   image is created as the image name is referenced */ 
   if (uparms.addtext)
      make_pdf (&igen, anproc.hostname, outimage.taename[0]); 

/* free memory for image generating luts */ 
   if (igen.chipsflag)
      free_memory (&igen); 

/* return data to tae */ 
   if (
   q_string (&pblock, "anproc", 1, anproc.returned, P_UPDATE) != SUCCESS ||
   q_out    (&pblock)                                         != SUCCESS  )
      errclean ("cwheel-totae", "Error returning parameters to TAE"); 

/* set variables for c_complt() and end program */ 
   nouts=0; 
   strcpy (outfiles[nouts++], outimage.taename[0]);
   if (uparms.addtext)
      strcpy (outfiles[nouts++], anproc.taename); 
   c_complt ("CWHEEL", outfiles, &nouts);  

   return 0;
}


#ifdef ANOTE /* this ifdef'ed function call will pop up from    % lasmsg */ 
   c_lasmsg (ReadThisNote, 
   The unix application lasmsg can not be used directly on this application
   because of the function errclean.  Run tests/lasmsg2 instead.  );
#endif 
