
/******************************************************************************
NAME:     cwheel.h 
PURPOSE:  application header file 
******************************************************************************/
#include "las.h"    /* las include file */
#include "util.h"
#include "image.h"  /* for cwheel's data structures, function prototypes */


#define BYTEMAX    255   /* max data value for byte output */ 
#define BYTEMIN      0   /* min data value for byte output */ 
#define CHARHEIGHT  10   /* annotated character are 10 pixels high * SCALFACT */
#define CHARWIDTH    8   /* annotated character are 8 pixels wide * SCALFACT */
#define MAXASTRINGS 10   /* max number of strings in call to annotate-inplace */
#define MAXINTS    255.0 /* max intensity  expected by his2rgb(), min=0 */ 
#define MAXHUE     360.0 /* max hue        expected by his2rgb(), min=0 */ 
#define MAXSAT     255.0 /* max saturation expected by his2rgb(), min=0 */ 
#define NBANDS       3   /* output image has 3 bands for RGB */ 
#define NOHUE       -1.0 /* his2rgb() expects a hue of NOHUE for greys */ 
#define PRIHUES      6   /* output image has color chips centered at 6 PRImary 
		            HUES: red, green, blue, cyan, magenta, yellow */ 
#define PRIMARIES    3   /* a smaller set of primaries for red, green, blue */ 
#define SPACER       1   /* # of pixels to skip between rows of text */ 



/* I could set MAXINTS and MAXHUE to 1.0 and factor them out of the formulas
   in this application, but then his2rgb() would get further away from the 
   his2rgb() in LAS>HIS.  I decided to keep both versions of his2rgb() similar*/




/* data structure for user tae parameters.  parameter OUT appears in struct 
   OUTIMAGE, chipsflag, nrings, rwidth are in struct IGEN instead of UPARMS */ 

   struct UPARMS 
   {
   /* parameters for both subcommands */ 
      char *odtype;   /* output data type */ 
      float textsize, /* annotation text size as a percentage of color chip
                         height */ 
            backints, /* intensity for image background */ 
	    bdrints,  /* intensity of the border around the color wheel */
            intens;   /* intensity of image */ 
      long bdrwidth;  /* border width in pixels */ 
   /* parameters for chips subcommand */ 
      float aspect;   /* aspect ratio, ratio of the chip width to height */ 
      long addtext;   /* flag, does user want annotation text, = TRUE or FALSE*/
   /* parameters for smooth subcommand */ 
      long wwidth;    /* diameter of the color wheel in pixels */ 
   };





/* data structure for image generation parameters and luts (look-up-tables) */
   struct IGEN 
   {
      double center;    /* center of image in 0-relative, line/sample 
                           coordinates.  since image is square, central line = 
			   central sample */ 
      long chipsflag,   /* flag, was subcommand chips selected, = T or F */ 
           nlns,        /* image size */ 
	   rwidth,      /* width of each color ring in pixels */
	   scalfact,    /* SCALFACT for call to ANNOTATE-INPLACE */ 
	   totalchips;  /* total number of chips to create */ 
      double backints,  /* background  intensity, scaled for rgb2his() */ 
	     bdrints,   /* border      intensity, scaled for rgb2his() */ 
	     wheelints, /* color wheel intensity, scaled for rgb2his() */ 
             bdrmaxrad, /* max radius for color wheel border in pixels */ 
	   wheelmaxrad; /* max radius for color wheel in terms of pixels */ 
   /* variables and look-up-tables used for creating a chip image */ 
      float **anglelim, /* index1 = ring number, index2 = chip number, lut
                           value is max angle (in radians) for that chip */ 
	    **huelut,   /* index1 = ring number, index2 = chip number, lut
		           value is hue for that chip, scaled for his2rgb() */
            *ringlim,   /* index is ring number, lut value is the max radius
                           for that color ring in pixels */ 
	    *ringsat;   /* index is ring number, lut value is saturation for 
			   that color ring, scaled for his2rgb() */ 
      long  nrings,     /* number of color rings */ 
           *ring2chip,  /* index is color ring number, lut value is number of 
			   color chips in that ring */ 
           *ring2chipp1;/* index is color ring number, lut value is number of 
			   color chips in that ring plus 1 */ 
   }; 





/* a structure to keep track of the various versions of anproc, a proc to 
   call annotate-inplace */ 
struct ANPROC
{
      char hostname[CMLEN]; /* host name of file */ 
	  char taename[CMLEN];  /* tae  name of file */ 
	  char *returned[1];    /* value returned to tae with no file extension */
	  char rmemory[CMLEN];  /* provides memory for returned[] */ 
}; 





   /* getpar.c */ 
void getpar (struct PARBLK *pblock, struct UPARMS *uparms, 
	   struct IGEN *igen); 
   /* his2rgb.c */ 
void his2rgb (double huein, double intensity, double saturation,
	   unsigned char *outbuf[NBANDS]); 
   /* iparms.c */
void image_parms (struct UPARMS *uparms, struct IGEN *igen);
void set_r2c (struct UPARMS *uparms, struct IGEN *igen);
void set_ah (struct IGEN *igen);
   /* make_image.c */
void make_image (struct PARBLK *pblock, struct IGEN *igen, 
	   struct OUTIMAGE *outi);
void hue_sat (struct IGEN *igen, long line, long sample, double *hue, 
	   double *intens, double *satur);
   /* make_pdf.c */ 
void make_pdf (struct IGEN *igen, char anproc[CMLEN], char *imagename); 
   /* memory.c */
void get_memory  (struct IGEN *igen);
void get_memory2 (struct IGEN *igen);
void free_memory (struct IGEN *igen);
   /* newname.c */ 
void newname (char inbase[CMLEN], char inend[CMLEN], char outname[CMLEN]);



