
/*******************************************************************************
NAME:     errclean() 
PURPOSE:  errclean is called in the event of a fatal error.  It closes any open
	  images;  if $delflg="yes" output images are deleted;  a fatal error
	  message is issued.  This function can be called from anywhere in an
	  application after out_flags().  This function accepts a variable
	  number of input arguments much like c_lasmsg does.  Usage:  errclean
	  (messagekey, sprintf format, arguments to format).
*******************************************************************************/
#include <stdarg.h>    /* variable argument list support */
#include <stdlib.h>    /* for call to access() */ 
#include "cwheel.h"    /* application header file */ 



/* vars common to errclean() and main(), declared common to prevent excessive 
   parameter passing */ 
extern struct OUTIMAGE outimage;  /* info on output las image */ 
extern struct ANPROC   anproc;    /* proc to run annotate */ 



void errclean
(
    char *mkey,
    char *format,
    ...
)
{
    va_list argumentp; /* lists a variable number of function arguments */ 
    char  message[3*ERRLEN]; /* text of message printed by c_lasmsg() */

    /* get input variables and format message string */ 
    va_start (argumentp, format);  
    vsprintf (message, format, argumentp); 
    va_end (argumentp);

    /* close any open image files and free buffers */
    if (outimage.opened && !outimage.closed)  
        close_outimage(&outimage); 

    /* optionally delete output image */ 
    if (outimage.opened && outimage.delflg) 
        c_delfil (outimage.hostname[0]); 

    /* optionally delete anproc if it exists */ 
    if (outimage.delflg && !access(anproc.hostname,0) )
        c_delfil (anproc.hostname); 

    /* issue fatal error and terminate processing */ 
    c_lasmsg (TRUE, mkey, message); 
}


