
/******************************************************************************
NAME:     getpar() 
PURPOSE:  get and return user tae parameters 
******************************************************************************/
#include "cwheel.h"  /* application header file */ 
#include "typlim.h"  /* for MAXLONG macro */ 



FUNCTION void getpar (pblock, uparms, igen)
   struct PARBLK *pblock;   /* input:   tae parameter block */ 
   struct UPARMS *uparms;   /* output:  tae parameters */ 
   struct IGEN   *igen;     /* output:  image generation parameters based on
                                        user parameters */ 
{

/* declare variables */ 
   double tblock;           /* number of lines in each text block */ 
   struct VARIABLE *tparm;  /* points to various tae parameters */ 

/* get parameters common to both subcommands */ 
   /* backints, bdrints, bdrwidth, intens, odtype */ 
      tparm = p_find (pblock,"backints");  uparms->backints = RVAL (*tparm,0); 
      tparm = p_find (pblock,"bdrints");   uparms->bdrints  = RVAL (*tparm,0); 
      tparm = p_find (pblock,"bdrwidth");  uparms->bdrwidth = IVAL (*tparm,0); 
      tparm = p_find (pblock,"intens");    uparms->intens   = RVAL (*tparm,0); 
      tparm = p_find (pblock,"odtype");    uparms->odtype   = SVAL (*tparm,0);

   /* addtext */ 
      tparm = p_find (pblock, "addtext");
      if (*(SVAL(*tparm,0)) == 'y' || *(SVAL(*tparm,0)) == 'Y')  /* y for yes */
	 uparms->addtext = TRUE;
      else 
	 uparms->addtext = FALSE;

   /* subcommand to execute */ 
      tparm = p_find (pblock, "_subcmd");  
      if (*(SVAL(*tparm,0)) == 'c' || *(SVAL(*tparm,0)) == 'C') /*C for chips */
	 igen->chipsflag = TRUE;
      else 
	 igen->chipsflag = FALSE;

/* get parameters which apply to one subcommand */ 
   if (igen->chipsflag) /* get parameters for chips subcmd */ 
      {
      tparm = p_find (pblock, "aspect");   uparms->aspect  = RVAL (*tparm,0); 
      tparm = p_find (pblock, "nrings");   igen  ->nrings  = IVAL (*tparm,0); 
      tparm = p_find (pblock, "rwidth");   igen  ->rwidth  = IVAL (*tparm,0); 
      tparm = p_find (pblock, "textsize"); uparms->textsize= RVAL (*tparm,0); 
      }
   else  /* smooth subcommand */ 
      {
      tparm = p_find (pblock, "wwidth");   uparms->wwidth = IVAL (*tparm,0); 
      } 

/* error check:  nrings > 1 when intensity = 0 or 1 */ 
   if (igen->chipsflag && igen->nrings > 1 && 
   (uparms->intens == 0.0 || uparms->intens == 1.0) )
      errclean("cwheel-nrings", "NRINGS > 1 when INTENS = %.0f",uparms->intens);

/* determine image size and check for errors */ 
   if (igen->chipsflag)
      {
      /* central ring is rwidth wide */
      /* 2 is for outer rings appearing on both sides of the center ring */
      if (igen->nrings)
	 igen->nlns = igen->rwidth + igen->rwidth * 2 * (igen->nrings -1);
      else
	 igen->nlns = 0; 
      } 
   else
      igen->nlns = uparms->wwidth;  /* color wheel size */
   /* 2 is for outer border appearing on both sides of the color wheel */
   igen->nlns += uparms->bdrwidth * 2;

   /* user message */
   c_lasmsg (FALSE, "informational",
   "OUT will have %d lines and samples", igen->nlns);

   /* check image size against MAXNL and MAXNS */ 
   if (igen->nlns > MAXNL || igen->nlns > MAXNS)
      c_lasmsg (FALSE, "cwheel-fatal", 
     "Calculated image size exceeds LAS limits of %d NL %d NS.", MAXNL, MAXNS); 

   /* check size against disk file limit. this test is more stringent than
   above */ 
   if ( (double)igen->nlns * igen->nlns * NBANDS > MAXLONG)
     errclean ("cwheel-size", 
     "Output disk image size would exceed system limit of 2 Gigabytes");

/* determine SCALFACT for call to ANNOTATE.  scalfact calculation does not
   include a correction for round off.  If textsize==100, correcting for round
   off could cause the text to exceed the height of the chip.  That would be
   unacceptable.  */

   if (!uparms->addtext)
      return; 

   tblock = igen->rwidth                /* height of color chips in pixels */ 
	  * (uparms->textsize / 100.0); /* fractional part of height to use */ 

   igen->scalfact = tblock / (NBANDS     /* a block of text has NBANDS lines */
	          * CHARHEIGHT           /* each line is CHARHEIGHT tall */ 
	          + SPACER *(NBANDS-1)); /* spacer between text lines */ 

   if (igen->scalfact < 1)
      errclean ("cwheel-scalfact",  
      "Calculated size of annotation text is too small."); 
}




