
/******************************************************************************
NAME:     his2rgb() 
PURPOSE:  convert data from HIS to RGB. 
	  I grabbed this algorithm from LAS>HIS, cleaned it up a bit, and 
	  modified it for CLRWHEEL.  
******************************************************************************/
#include "cwheel.h"    /* application header file */ 




FUNCTION void his2rgb (huein, intensity, saturation, outbuf)
   double huein;                   /* input:  HIS color to convert */ 
   double intensity;               /* input:  HIS color to convert */ 
   double saturation;              /* input:  HIS color to convert */ 
   unsigned char *outbuf[NBANDS];  /* output:  RGB color */ 
{

/* declare variables */ 
   double red,      /* the red band output */
          red255,   /* the red band output */
          green,    /* the green band output */
          green255, /* the green band output */
          blue,     /* the blue band output */
          blue255,  /* the blue band output */
          m1,       /* value used for determining RGB */
          m2,       /* value used for determining RGB */
          scalei,   /* scaled intensity value */
          scales,   /* scaled saturation value */
          hue,      /* hue */
          savehue;  /* save the hue for future processing */

/* scale initial HIS values */ 
   hue = huein; 
   scalei = intensity  / MAXINTS; 
   scales = saturation / MAXSAT; 

/* do something */ 
   red = green = blue = 0.0;
   m2 = 0.0;

   if (scalei <= 0.50) 
      m2 = scalei*(1.0 + scales);
   else if (scalei > 0.50)
      m2 = scalei+scales-(scalei*scales);
   m1 = 2.0 * scalei - m2;

/* check for greys */ 
   if (scales == 0.0)
      {
      if (hue == NOHUE)
	 {
	 red   = scalei;
	 green = scalei;
	 blue  = scalei;
	 }
      }
   else
   {

   /* calculate the red band */
      savehue = hue + 120.0;
      if (savehue > 360.0)
	 savehue -= 360.0;
      if (savehue < 0.0)
	 savehue += 360.0;
      if (savehue < 60.0)
	 red = m1 + (m2-m1) * savehue/60.0;
      else
      if (savehue < 180.0)
	 red = m2;
      else
      if (savehue < 240.0)
	 red = m1 + (m2-m1) * (240.0-savehue)/60.0;
      else
	 red = m1;

   /* calculate the green band */
      savehue = hue;
      if (savehue > 360.0)
	 savehue -= 360.0;
      if (savehue < 0.0)
	 savehue += 360.0;
      if (savehue < 60.0)
	 green = m1 + (m2-m1) * savehue/60.0;
      else
      if (savehue < 180.0)
	 green = m2;
      else
      if (savehue < 240.0)
	 green = m1 + (m2-m1) * (240.0-savehue)/60.0;
      else
	 green = m1;

   /* calculate the blue band */
      savehue = hue - 120.0;
      if (savehue > 360.0)
	 savehue -= 360.0;
      if (savehue < 0.0)
	 savehue += 360.0;
      if (savehue < 60.0)
	 blue = m1 + (m2-m1) * savehue/60.0;
      else
      if (savehue < 180.0)
	 blue = m2;
      else
      if (savehue < 240.0)
	 blue = m1 + (m2-m1) * (240.0-savehue)/60.0;
      else
	 blue = m1;
   }

   red255 = red*255.0;
   green255=green*255.0;
   blue255 = blue*255.0;
   if (red255 > 255.0)
       red = 255.0;
       else
       red = red255;
   if (green255 > 255.0)
       green = 255.0;
       else
       green = green255;
   if (blue255 > 255.0)
       blue = 255.0;
       else
       blue = blue255;

/* correct for round-off error */ 
   red   += 0.5; 
   green += 0.5;
   blue  += 0.5; 

/* limit output to the byte range */ 
   if (red   > 255.0) red   = 255.0; 
   if (green > 255.0) green = 255.0;
   if (blue  > 255.0) blue  = 255.0;
   
   if (red   < 0.0) red   = 0.0;
   if (green < 0.0) green = 0.0;
   if (blue  < 0.0) blue  = 0.0;

/* set return values */ 
   *(outbuf[0]) = (unsigned char) red  ; 
   *(outbuf[1]) = (unsigned char) green;
   *(outbuf[2]) = (unsigned char) blue ;

/* at maximum saturation, some return values = 1 that should = 0 */ 
/* this section is not in LAS>HIS */ 
   if (
   *(outbuf[0]) == 255 || 
   *(outbuf[1]) == 255 || 
   *(outbuf[2]) == 255  ) 
      {
      if (*(outbuf[0]) == 1)  *(outbuf[0]) = 0; 
      if (*(outbuf[1]) == 1)  *(outbuf[1]) = 0; 
      if (*(outbuf[2]) == 1)  *(outbuf[2]) = 0; 
      } 
}


