
/******************************************************************************
NAME:     make_image.c 
PURPOSE:  functions to create a color wheel image 
******************************************************************************/
#include <math.h>   /* for trig functions and M_PI macro */ 
#include "cwheel.h" /* application header file */ 





/******************************************************************************
NAME:     make_image() 
PURPOSE:  create disk image of a color wheel.
          implicit input:  TAE global $msgtim 
******************************************************************************/
FUNCTION void make_image (pblock, igen, outi)
   struct PARBLK *pblock;  /* input:  tae parameter block */ 
   struct IGEN *igen;      /* input:  image generation parameters */ 
   struct OUTIMAGE *outi;  /* input:  specs for output image */ 
{
/* declare variables */ 
   extern long curent;      /* current entity, for processing message */ 
   long band,               /* loop index:  image band */ 
        line,               /* loop index:  image line */ 
        messform=ALLCAT,    /* output message format, arg to c_startm() */ 
        msgtim,             /* TAE global $msgtim */ 
	sample;             /* loop index:  image sample */ 
   struct VARIABLE *tparm;  /* points to a TAE parameter in pblock */ 
   double hue,              /* hue for the current pixel */ 
	  intensity,        /* intensity  for this image */ 
          saturation;       /* saturation for the current pixel */ 
   unsigned char *outbuf[NBANDS]; /* pixel locations in buffer */ 

/* initialize and start processing line message */ 
   tparm = p_find (pblock, "$msgtim");    
   msgtim = IVAL(*tparm, 0); 
   curent = 1; 
   c_inittm ("CWHEEL", "Processed", "line", "bands", "", &outi->nl,
   &outi->totalbands, &msgtim, &messform); 
   c_startm (); 

/* process image */ 
   for (line=0;  line<outi->nl;  line++, curent++)
      {
      /* set outbuf pixel locations */ 
	 for (band=0;  band<NBANDS;  band++)
	    outbuf[band] = outi->linebuf[band]; 

      /* calculate image data */ 
	 for (sample=0;  sample<outi->ns;  sample++)
	    {
	    /* determine hue and saturation */ 
	       hue_sat (igen, line, sample, &hue, &intensity, &saturation); 

	    /* convert from HIS to RGB */ 
	       his2rgb (hue, intensity, saturation, outbuf);

	    /* increment pixel locations */ 
	       for (band=0;  band<NBANDS;  band++)
		  outbuf[band]++;  
	    }

      /* update minmax */ 
	 minmax_bil (outi); 
      /* write a line of data */ 
	 write_bil (line, outi); 
      }

c_stoptm(); 
}









/******************************************************************************
NAME:     hue_sat() 
PURPOSE:  return hue, intensity, saturation given a pixel location  
	  some of the logic herein also appears in text_locations() 
******************************************************************************/
FUNCTION void hue_sat (igen, line, sample, hue, intensity, saturation)
   struct IGEN *igen;  /* input:   image generation parameters */ 
   long line;          /* input:   pixel location */ 
   long sample;        /* input:   pixel location */ 
   double *hue;        /* output:  hue for this pixel */ 
   double *intensity;  /* output:  intensity for this pixel */ 
   double *saturation; /* output:  saturation for this pixel */ 
{			
/* declare variables */ 
   static double    /* variables to prevent repeated calculation of constants */
          twopi = M_PI * 2.0,              /* two PI */ 
	  rtohue = MAXHUE / (2.0 * M_PI);  /* convert from radians to hue */ 
   double lined,   /* pixel line   cartesian coordinate */ 
	  sampd,   /* pixel sample cartesian coordinate */ 
          angle,   /* polar coordinate of this pixel location */ 
	  radius;  /* polar coordinate of this pixel location */ 
   long chipnum,   /* loop index, color chip number */ 
	inwheel,   /* flag, is the radius within the color wheel */ 
        ringnum;   /* loop index, color ring number */ 

/* transform input line,sample coordiates to cartesian coordinates with the 
   origin at the center of the image */ 
   lined = (double)line   - igen->center; 
   sampd = (double)sample - igen->center; 

/* convert from cartesian coordinates to polar coordinates.  angle will be in 
   the range (-pi,+pi] */ 
   radius = sqrt (sampd * sampd + lined * lined); 
   if (sampd == 0.0)  /* atan2() fails in these cases */ 
      {
      if (lined > 0)
	 angle =  M_PI_2;  /* pi/2 */ 
      else
	 angle = -M_PI_2;  
      }
   else 
      angle = atan2 (lined, sampd);  

/* rotate angle so that red is at the top of the hue circle.  Even after the 
   addition of M_PI_2, some angles are still negative.  Negative angles
   will be back in range (0,pi*2] */ 

   angle += M_PI_2;      
   if (angle < 0.0)
      angle += twopi;    

/* quantize radius and angle */ 
   if (igen->chipsflag)
      {
      /* quantize radius, get ring number */ 
	 for (inwheel=FALSE, ringnum=0;  ringnum<igen->nrings;  ringnum++)
	    if (radius < igen->ringlim[ringnum])
	       {
	       inwheel = TRUE; 
	       break; 
	       }

      /* for ring 0, set output grey values */ 
	 if (inwheel && ringnum == 0)
	    {
	    *hue        = NOHUE;           /* grey has no hue */ 
	    *intensity  = igen->wheelints; /* set wheel intensity */ 
	    *saturation = 0;               /* grey has 0 saturation */ 
	    return; 
	    }

      /* set return values for pixels within the color wheel */ 
	 if (inwheel) 
	    {
	    /* quantize angle, get chip number */ 
	       for (chipnum=0;  chipnum<igen->ring2chipp1[ringnum];  chipnum++)
		  if (angle < igen->anglelim[ringnum][chipnum])
		     break;  
	    *hue        = igen->huelut[ringnum][chipnum];
	    *intensity  = igen->wheelints; 
	    *saturation = igen->ringsat[ringnum];
	    return; 
	    }
      }

/* set border and background for surrounding greys */ 
   if (radius > igen->wheelmaxrad)
      {
      *hue = NOHUE;                   /* grey has no hue */ 
      *saturation = 0;                /* grey has 0 saturation */ 
      if (radius <= igen->bdrmaxrad) 
	 *intensity = igen->bdrints;  /* set border intensity */ 
      else
	 *intensity = igen->backints; /* set background intensity */ 
      return;
      }

/* set hue, intensity, and saturation for color wheel */ 
   /* scale angle to (0,360] for his2rgb() */ 
   *hue = angle * rtohue;        
   *intensity = igen->wheelints; 
   /* scale radius to (0,MAXSAT) for his2rgb() */
   *saturation = radius * MAXSAT / igen->wheelmaxrad;
}




