
/******************************************************************************
NAME:     memory.c 
PURPOSE:  functions to get and free memory  
******************************************************************************/
#include <stdlib.h>  /* for call to malloc() */
#include "cwheel.h"  /* application header file */ 





/******************************************************************************
NAME:     get_memory() 
PURPOSE:  allocate memory for image generating luts  
******************************************************************************/
FUNCTION void get_memory (igen)
   struct IGEN  *igen;  /* output:  luts with memory allocated */ 
{
/* get memory for ringlim, ringsat, ring2chip, ring2chipp1 */ 
   if ((igen->ringlim = (float *) malloc (sizeof(float) * igen->nrings))
   ==NULL)
      errclean ("cwheel-memory", "Unable to get memory for ringlim");

   if ((igen->ringsat = (float *) malloc (sizeof(float) * igen->nrings))
   ==NULL)
      errclean ("cwheel-memory", "Unable to get memory for ringsat");

   if ((igen->ring2chip = (long *) malloc (sizeof(long) * igen->nrings)) 
   ==NULL)
      errclean ("cwheel-memory", "Unable to get memory for ring2chip");

   if ((igen->ring2chipp1 = (long *) malloc (sizeof(long) * igen->nrings)) 
   ==NULL)
      errclean ("cwheel-memory", "Unable to get memory for ring2chipp1");

}





/******************************************************************************
NAME:     get_memory2() 
PURPOSE:  allocate memory for more image generating luts.  
          this function can't be combined with get_memory() 
******************************************************************************/
FUNCTION void get_memory2 (igen)
   struct IGEN *igen;    /* output:  luts with memory allocated */ 
{
/* declare variables */ 
   long ringnum;  /* loop index, color ring */ 
/* allocate memory for anglelim */ 
   if ((igen->anglelim = (float **) malloc (sizeof (float *) * igen->nrings))
   == NULL)
      errclean ("cwheel-memory", "Unable to get memory for anglelim");

   for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
      if ((igen->anglelim[ringnum] = (float *) malloc 
      (sizeof (float) * igen->ring2chipp1[ringnum])) == NULL)
	 errclean ("cwheel-memory", "Unable to get memory for anglelim");

/* allocate memory for huelut */ 
   if ((igen->huelut = (float **) malloc (sizeof (float *) * igen->nrings))
   == NULL)
      errclean ("cwheel-memory", "Unable to get memory for huelut");

   for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
      if ((igen->huelut[ringnum] = (float *) malloc 
      (sizeof (float) * igen->ring2chipp1[ringnum])) == NULL)
	 errclean ("cwheel-memory", "Unable to get memory for huelut");
}





/******************************************************************************
NAME:     free_memory() 
PURPOSE:  free memory for image generating luts.  memory was allocated in
          get_memory() and get_memory2() 
******************************************************************************/
FUNCTION void free_memory (igen)
   struct IGEN *igen;  /* input:  free this memory */ 
{
/* declare variables */ 
   long ringnum;   /* loop index, ring number */ 
/* free memory for 1-d arrays */  
   free (igen->ringlim);
   free (igen->ringsat);
   free (igen->ring2chip); 
   free (igen->ring2chipp1); 
/* free 2-d arrays */ 
   for (ringnum=0;  ringnum<igen->nrings;  ringnum++)
   {
      free (igen->anglelim[ringnum]); 
      free (igen->huelut  [ringnum]); 
   }
   free (igen->anglelim); 
   free (igen->huelut  ); 
}


