#include "convolve.h"
#include "util.h"
#include "protos.h"
/**********************************************************************

NAME: compute

PURPOSE: Perform spatial domain convolution for each image band.
        
PROGRAM HISTORY:
VERSION   DATE      AUTHOR       CODE/CONTR.   CHANGE REQUEST
1	10/15/89     John McGuthy    STX       First created for conversion
2       10/15/90     F. Irani	     STX       Add mutiple image handling,
					       doc., etc.
3        5/29/92     K. Sheffield   EDC/STX    Add data mirroring capability

PDL:

	Begin compute

		Initialize variables
		Initialize Processing Line Message (PLM)
		Start PLM
		for each input image specified
			call (*conv) to apply convolution
		Stop PLM
		Close files

	End compute 

**********************************************************************
*/


extern long    mirflg;				/* mirror data flag */
void compute
(
    char hosout[][CMLEN],         		/* output image host name       */
    long images,				/* Number of input images	*/
    long num_bands,			/* Total number input bands	*/
    long image_band_num[][MAXBND+1],	/* band numbers/image 		*/
    long num_image_bands[],		/* number of bands per image    */
    long *dtype,				/* Input data type		*/
    long row,				/* matrix row number		*/
    long col,				/* matrix col number		*/
    float wght[],				/* Input weight file		*/
    long wind[][4]			/* Input windows		*/
)

{

long    band,				/* Current band loop counter	*/
	image,				/* Image loop counter		*/
        msg_flag,			/* inittm parameter		*/
        nsout,				/* Number of output samps	*/
        nlout;				/* Number of output lines	*/

msg_flag = ONECAT;
nlout = wind[0][NL];
nsout = wind[0][NS];

/*
Initialize messages
------------------- 							*/
c_inittm("CONVOLVE", "Processed", "line", "band", " ", &nlout, &num_bands,
	 &msgtim, &msg_flag);

curent = 1;   
curcat = 1;
c_startm(); 
for (image = 0; image < images; ++image)
{
	for (band = 0; band < num_image_bands[image]; ++band, ++curcat)
	{
        if( mirflg)
        {
            convmf(hosout, wght, row, col, image, image_band_num[image][band],
                   nlout, nsout, *dtype);
        }
        else
        {
            convzf(hosout, wght, row, col, image, image_band_num[image][band],
                   nlout, nsout, *dtype);
        }
	}
}

/*
Stop output messages
-------------------- 							*/
c_stoptm();

/*
Close input and output datasets.
--------------------------------					*/
for (image = 0; image < images; ++image)
{
	c_eclose ( &fdesc_in[image]);
}

c_eclose ( &fdesc_out);

}
