#include <stdio.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"


/*
***************************************************************************

NAME: CONVOLVE


PURPOSE: Read input image and perform spatial domain convolution.
        
PROGRAM HISTORY:
VERSION   DATE      AUTHOR       CODE/CONTR.   CHANGE REQUEST
 1      3/10/82     R. Wenger       CSC     
10      6/24/85     F. IRANI        SAR            NON-AP VERSION
11      2/06/87     D.SULLIVAN      U. OF MD.      Timed proc. msg.
12     10/15/89     John McGuthy    STX            LAS V5.0 Conversion
13     10/15/90     F. Irani	    STX	  Add mutiple image handling, doc., etc.
14      5/29/92     K. Sheffield    EDC/STX	   Add edge mirroring
15	9/24/92	    T. Mittan	    EDC/STX	   Added extern to curent and
						   curcat to eliminate 
						   warnings on link

PDL:

Begin convolve

	Call getpar to get user specified parameters
	Call filspec to set up input and output file specifications
	Call compute to perform convolve
	Determine the minimum and maximum values of the output image.
	Write the min, max and the validity flag values to the ddr file.
	Update output image history.
	Issue completion message.

End convolve

**********************************************************************
*/


/* the following variables have been declared external image I/O routines */

struct PARBLK vblock;                     /* TAE parameter block      */

char    *prog_name = "CONVOLVE";  /* program name            */

long	one = 1;
long	zero = 0;
long	inf_err = 1;
long	fatal_err = -1;

long 	msgtim;                          /* message interval         */
long    min_max;                         /* min max flag             */

extern long	curcat;                  /* message utility          */
extern long	curent;                  /* parameters for processing*/

long    mirflg;				 /* mirror data flag */
struct FDESC *fdesc_in[MAXIMG];		 /* file descriptors         */
struct FDESC *fdesc_out;                       /* file descriptors         */

int main
(void) 
{

static char	hosin[MAXIMG][CMLEN];/* input images host names        	*/
static char	hosout[1][CMLEN];   /* output image host name           	*/
static char    lasout[1][CMLEN];    /* output image las name            	*/

static double 	out_max[MAXBND],/* Image min per band array		*/
	out_min[MAXBND];	/* Image max per band array		*/

static float   wgt[225];        /* input weight storage 		*/
float   norm;                   /* input norm           		*/
float   bias;                   /* input bias           		*/
float   wmax;                   /* weighted maximum     		*/
float   wmin;                   /* weighted minimum     		*/

static long	band,		/* Current band loop counter		*/
	image_band_num[MAXIMG][MAXBND + 1], /* band numbers/image 	*/
	num_bands,              /* total number of input image bands	*/
	num_image_bands[MAXIMG],/* number of bands in each image    	*/
	images,             	/* number of input images           	*/
        dtype,                  /* data type of images              	*/
	wind[MAXIMG][4],        /* input image windows              	*/
        row,        		/* number of rows in weights        	*/
        col;                    /* number of colums in weights      	*/

/*
Get parameters
--------------*/
getpar (hosin, lasout, hosout, &images, &num_bands,
	num_image_bands, image_band_num, wind, wgt, &norm,
	&bias, &row, &col, &wmax, &wmin );


/*
Set up input and output file specifications
-------------------------------------------*/
filspec (hosin, hosout, &images, num_bands, 
	 num_image_bands, image_band_num, wind, &dtype);

/*
perform convolve
----------------*/
compute (hosout, images, num_bands, image_band_num, num_image_bands, 
	 &dtype, row, col, wgt, wind );

/* 
Determine the minimum and maximum values of the output image.
Write the min, max and the validity flag values to the ddr file.
---------------------------------------------------------------- 	*/
if (min_max)
{
	for (band = 0; band < num_bands; ++band)
	{
		image_band_num[0][band] = band + 1;
	}

	wind[0][0] = wind[0][1] = 1;
	num_image_bands[0] = num_bands;
	c_minmax (hosout, &one, wind, image_band_num,
		  num_image_bands, out_min, out_max);

   	c_svmnmx (hosout[0], &num_bands, out_min, out_max, "YES"); 
}

/*
Update output image history and issue completion message
--------------------------------------------------------		*/
c_hstout (&vblock, hosout[0], hosin, &one, " "); 

/*
Issue completion message.
-------------------------						*/
c_complt ("CONVOLVE", lasout, &one);

return EXIT_SUCCESS;

}
