#include "convolve.h"
#include "util.h"
#include "protos.h"

/*
**********************************************************************
NAME: FILSPEC

PROGRAM HISTORY:
  Version	Date       Author      	    Request
  -------	----	   ------           -------
    5.0       10-15-90    F. Irani, STX	    Fixes
    3.0       30-MAR-90   J. MCGUTHRY	    Conversion
    2.0       20-APR-87   B. KOVALICK
    1.0       01-OCT-84   B. DEALY

FUNCTION: sets up the image i/o interface

ALGORITHM:
Open input images for reading
Open output image for writing

***********************************************************************

*/

void filspec
(
    char hosin[][CMLEN],		/* input image host names		*/
    char hosout[][CMLEN],		/* Output image host name		*/
    long *images,		/* Number of input images specified	*/
    long num_bands,		/* total number of input bands		*/
    long num_image_bands[],	/* number of bands in each image    	*/
    long image_band_num[][MAXBND+1],	/* band numbers/image 		*/
    long wind[][4],		/* input image windows              	*/
    long *dtype			/* Input data type			*/
)

{

double    line_inc = 1.0,		/* upddr parameter		*/
          samp_inc = 1.0;		/* upddr parameter		*/

long      access,			/* eopenr parameter		*/
          comb_flag = COMB,		/* upddr parameter		*/
	  image,			/* Input image loop counter	*/
          type;				/* Output data type		*/

/*
Open input image for reading
----------------------------						*/
*dtype = 0;
access = IREAD;
for (image= 0; image < *images; image++)
{
	if (c_eopenr (&fdesc_in[image], hosin[image], &num_image_bands[image],
		      &wind[image][SL], &wind[image][SS], &wind[image][NL],
		      &wind[image][NS], &access, dtype, &zero, &one) != E_SUCC)
	   c_errmsg ("Error opening input image", "convolve-inopen",&fatal_err);
}
fdesc_in[++image] = 0;

/*
Open output image for writing
-----------------------------						*/
type = EREAL;
access = IWRITE;
if (c_eopenr (&fdesc_out, hosout[0], &num_bands,
                 &one, &one, &wind[0][NL], &wind[0][NS],
                 &access, &type, &zero, &one) != E_SUCC)
{
	c_abort (&vblock, "Error opening output image",
		 "convolve-outopen", prog_name, hosout, &one);
}

/*
Update image ddr information.
-----------------------------						*/
if (c_upddr (&vblock, hosin, hosout[0], &one, wind, 
   	     image_band_num, num_image_bands,
	     &line_inc, &samp_inc, &num_bands, &comb_flag) != E_SUCC)
{
	c_abort (&vblock, "Error opening output image",
		 "convolve-upddr", prog_name, hosout, &one);
}

}
