#include "convolve.h"
#include "util.h"
#include "protos.h"
/**********************************************************************

   name- getpar

   language- C

   version- 1.0    date- oct-84     programmer- y.sheu
   version- 2.0    date- Sep-84     Programmer- F. Irani V5.0 Conversion
   version- 3.0    date- May-92     Programmer- K. Sheffield Mirror capability

   function- receives parameters from tae process definition file 

	pdl:
	begin getpar

		Initialize TAE parameters
		Get  input image name(s)
		Get output image name(s)
		Call getwts to extract weight file from disk file
		Terminate if image window is too small to process.
		Fill weight matrix, norm and offset values
		Get variable $msgtime for timed processing message.
		Get variable $mimnax.
		Get reset parmeter
		Process norm and bias according to value of reset flag.
		Get mirror parmeter
		Print mean (norm) and offset to screen.
		Warn user if even numbered grid was specified.
		Print input grid to screen.
		Determine min and max of weight values.
	end getpar
**********************************************************************

*/

extern long    mirflg;				/* mirror data flag */
void getpar
(
    char hos_in[][CMLEN],		/* input image name     	*/
    char las_out[][CMLEN],		/* input las name		*/
    char hos_out[][CMLEN],     	/* output image name    	*/
    long *images,          	/* number of imput images  	*/
    long *num_bands,           	/* total number of input bands  */
    long num_image_bands[],	/* number of bands per image    */
    long image_band_num[][MAXBND+1], /* band numbers/image 	*/
    long wind[][4],		/* window specifications 	*/
    float wgt[],			/* Input weight file		*/
    float *norm,			/* Average/weight matrix	*/
    float *bias,			/* Offset for weight matrix	*/
    long *row,                	/* number of rows in weights    */
    long *col,                	/* number of colums in weights  */
    float *wmax,			/* weight mamximum		*/
    float *wmin			/* weight minimum		*/
)

{

struct VARIABLE *var;		/* TAE function parameter	*/

static char las_in[MAXIMG][CMLEN];	/* Input las image name	*/
static char minmax[7];		/* Minmax flag variable		*/

char   line[CMLEN];		/* Output message line buffer	*/
char   reset[10];		/* reset flag value		*/
char   deftext[CMLEN];		/* input weight file extention 	*/
char   filein[CMLEN];		/* input cwtgen file name	*/

double dwgt[225];		/* Weight matrix 		*/
double dnorm;			/* Weight norm			*/
double dbias;			/* weight offset		*/

long   access;			/* File open access value	*/
long   cnum;			/* Display column undex		*/
long   flag[1][2];			/* c_getin flag			*/
long   i, k, k2;		/* Loop counters		*/
long   imglimit = MAXIMG;	/* c_getin parameter		*/
long   otype;			/* c_getnum parameter		*/
long   same_window;		/* c_getin window flag		*/
long   std_out;			/* inlas parameter		*/
long   wrote = 0;
   
/*
Initialize program
------------------ */
c_inlas (&vblock, &std_out); 

if (c_getin (&vblock, "IN", &zero, las_in, hos_in, images, wind, 
	     image_band_num, num_image_bands, num_bands, &imglimit, 
	     &same_window, flag) != E_SUCC)
	c_errmsg ("Error accessing input image ", "convolve-in", &fatal_err);

if (c_getout (&vblock,"OUT",las_out, hos_out) != E_SUCC)
	c_errmsg ("Error accessing output image ","convolve-out", &fatal_err);

/*
Get input weight file
--------------------- */
access=IREAD;
strcpy(deftext,"prt");
if (c_getnam (&vblock,"INWT",&zero,filein,deftext,&access) != E_SUCC) 
	c_errmsg ("Error accessing input datafile", "convovle-in", &fatal_err);

getwts(row, col, &dnorm, &dbias, dwgt, filein);

/*
Terminate if image window is too small to process.
--------------------------------------------------			*/
if ( wind[0][NL] < *col || wind[0][NS] < *row )
	c_errmsg ("Windowed image is smaller than weight function",
		  "convolve-window", &fatal_err);

*norm = (float) dnorm;
*bias = (float) dbias;
for ( i = 0; i < (*col) * (*row); ++i)
	wgt[i] = (float)dwgt[i];

/*
Get variable $msgtime for timed processing message.
--------------------------------------------------- */
var = p_find(&vblock,"$msgtim");
msgtim = IVAL (*var,0);

/*
Get variable $mimnax.
-------------------- */
var = p_find(&vblock,"$minmax");
strcpy(minmax, SVAL(*var,0));
if (strcmp(minmax,"YES") == 0) 
	min_max = 1;
else
	min_max = 0;

/* Get reset parm and process norm and bias accordingly
------------------------------------------------------- */
var = p_find (&vblock,"RESETFLG");
strcpy(reset, SVAL(*var,0));
if (strcmp(reset, "YES") ==  0) 
{
	otype = EREAL;
	c_getnum (&vblock, "NORMVAL", &one, &otype,  norm);
	c_getnum (&vblock, "OFFSET", &one, &otype, bias);
} 

/* Get mirror parameter
------------------------------------------------------- */
var = p_find (&vblock,"MIRROR");

strcpy(reset, SVAL(*var,0));
if (strcmp(reset, "YES") ==  0) 
	mirflg = 1;
else 
	mirflg = 0;

/*
Print mean and offset to screen.
--------------------------------					*/
sprintf (line, "%d x %d  NORM = %6.3f OFFSET = %6.3f",
	 *row, *col, *norm, *bias);
t_write(line, T_STDCC);

/*
Warn user if even numbered grid was specified.
----------------------------------------------				*/
if ( ( *row % 2 ) == 0 || ( *col % 2 ) == 0 )
{
	c_errmsg ("Even dimension kernal may", 
		  "convovle-evenknl", &inf_err);
	c_errmsg ("produce output pixels which are",
		  "convovl-evenknl", &inf_err);
	c_errmsg ("noncoincident with any input pixel", 
		  "convovle-evenknl", &inf_err);
}

/*
Print input grid to screen.
---------------------------						*/
for (i = 0; i < *row; ++i)
{
       sprintf(line, "Row %2d ", i+1); 
       k2 = (i + 1) * (*col);
       cnum = 0;
       for(k = k2 - (*col); k < k2; ++k)
       {
		++cnum;
		wrote = FALSE;
		sprintf(&line[strlen(line)], "%10.3f ", wgt[k]);
		if (cnum % 5 == 0)
		{
			t_write(line, T_STDCC);
			strcpy(line, "       ");
			wrote = TRUE;
		}
       }
       if (!wrote)
		t_write(line, T_STDCC);
} 

/*
Determine weight min and max.
-----------------------------						*/
*wmax = wgt[0] / (*norm);
*wmin = wgt[0] / (*norm);
for (i = 0; i < (*row) * (*col); ++i)
{
	wgt[i] = wgt[i] / *norm;
	if (wgt[i]  >  *wmax) *wmax = wgt[i];
	if (wgt[i]  <  *wmin) *wmin = wgt[i];
} 

}
