#include "convolve.h"
#include "protos.h"
/*****************************************************************************
PROGRAM:  getwts

PURPOSE: Collect weight matrix values form disk file

PROGRAM HISTORY:
VERSION   DATE      AUTHOR       CODE/CONTR.   CHANGE REQUEST
1	10/15/89  J. McGuthry	  STX		First version
2	10/15/90  F. IRANI        STX		Update doc 

PDL:

	Begin getwts
		Open file.
		Read Number of rows and columns.
		Get norm, offset, and other appropriate values.
		Read Weight matrix.
	End getwts
****************************************************************************

*/
void getwts
(
    long *row,	/* Row Pointer					*/
    long *col,	/* Column Pointer				*/
    double *norm,	/* Mean						*/
    double *bias,	/* offset					*/
    double wgt[],	/* Weight matrix				*/
    char filein[]	/* Input weight file name			*/
)

{

FILE *fpout;	/* Input file pointer				*/

char blank_word[20];	/* text buffer				*/
char type[20];		/* Weight type buffer			*/
char blank_ch;		/* Blandk character space buffer	*/

double temp;		/* termporary variable			*/

long i, k, k2;		/* loop counters			*/

long num;		/* Weight Value 			*/

/*
Open file.
----------							*/
fpout = fopen(filein, "r");

/*
Read Number of rows and columns.
--------------------------------				*/
fscanf(fpout, "%s %d %c %d", type, row, &blank_ch, col);

/*
Get norm, offset, etc.
----------------------						*/
if (strcmp(type, "MANUAL") == 0 ) 
{
	fscanf(fpout, "%s %lf", blank_word, norm);
	fscanf(fpout, "%s %lf", blank_word, bias);
}
else if ( strcmp(type, "BOX") == 0 ) 
{
	fscanf(fpout, "%s %lf", blank_word, norm);
	fscanf(fpout, "%s %lf", blank_word, bias);
}
else if ( strcmp(type, "CUBIC") == 0 )
{
	fscanf(fpout, "%s %lf", blank_word, norm);
	fscanf(fpout, "%s %lf", blank_word, bias);
	fscanf(fpout, "%s %lf", blank_word, &temp);
	fscanf(fpout, "%s %lf", blank_word, &temp);
}
else if ( strcmp(type, "DIFF") == 0 ) 
{
	fscanf(fpout, "%s %lf", blank_word, norm);
	fscanf(fpout, "%s %lf", blank_word, bias);
}
else if ( strcmp(type, "GAUSS") == 0 ) 
{
	fscanf(fpout, "%s %lf", blank_word, norm);
	fscanf(fpout, "%s %lf", blank_word, bias);
	fscanf(fpout, "%s %lf", blank_word, &temp);
	fscanf(fpout, "%s %lf", blank_word, &temp);
	fscanf(fpout, "%s %lf", blank_word, &temp);
}
else if ( strcmp(type, "LAPLACE") == 0 ) 
{
	fscanf(fpout, "%s %lf", blank_word, norm);
	fscanf(fpout, "%s %lf", blank_word, bias);
} 

/*
Read Weight matrix.
-------------------						*/
for (i = 0; i < *row; ++i)
{
	k2 = (i + 1) * *col;
	fscanf(fpout, "%s %d", blank_word, &num);
	for (k = k2 - *col; k < k2; ++k)
	{
		fscanf(fpout, "%lf", &wgt[k]);
	}
}

}
