/******************************************************************************
PROGRAM:  CPD2T


PURPOSE: 
	Copy disk to tape program.

			
PROGRAM HISTORY:
  Version       Date        Author      Change Request
  -------       ----        ------      --------------
   1.0           8/92       K. Gacke    initial development 

PROJECT:        Calcomp Plot Files

Algorithm:
******************************************************************************/

#include <stdio.h>

#ifdef vms
#include <unixio.h>
#include <file.h>
#include <stat.h>
#endif

#ifdef unix
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#endif

#ifdef sun
#include <fcntl.h>
#endif

#include "las.h"
#include "tapeio.h"

static long fatal = -1;
static long nonfatal = 1;

main ()
{
struct PARBLK	vblock;
struct VARIABLE *p_find(), *val;
long	blksiz;
long	msgtim;
long	status;
long    std_out;
long	zero = 0;
char	density[32];
char	fname[CMLEN];
char	mntmsg[CMLEN];
char	tapeid[CMLEN];

c_inlas(&vblock,&std_out); 
val = p_find(&vblock,"INFILE");
strcpy(fname,SVAL(*val,0));
val = p_find(&vblock,"COMMENT");
strcpy(mntmsg,SVAL(*val,0));
val = p_find(&vblock,"TAPEDENS");
strcpy(density,SVAL(*val,0));
strcpy(density,"0");
val = p_find(&vblock,"BLKSIZ");
blksiz = IVAL(*val,0);
val = p_find(&vblock,"$MSGTIM");
msgtim = IVAL(*val,0);
status = cpd2t(fname,mntmsg,density,blksiz,msgtim,tapeid);
print_label(&vblock,mntmsg,tapeid,status);
c_complt("CPD2T","",&zero);
}

FUNCTION cpd2t(fname,mntmsg,density,blksiz,msgtim,tapeid)
	char *fname;
	char *mntmsg;
	char *density;
	long blksiz;
	long msgtim;
	char *tapeid;
{
extern long curent;
long access;
long dummy;
long eofopt;
long fd;
long mode;
long msgflg;
long nbyte;
long rewopt;
long rstat;
long skip;
long status;
long unit;
char *buf;
char emsg[CMLEN];

rstat = E_SUCC;
fd = open(fname,O_RDONLY,0777);
if (fd == -1)
    {
    rstat = E_FAIL;
    perror("cpd2t");
    sprintf(emsg,"Error opening file %s",fname);
    c_errmsg(emsg," ",&nonfatal);
    goto xit1;
    }
skip = 0;
mode = HANDLE;
rewopt = REWIND;
access = TWRITE;
status = c_tpopen(&unit,&skip,&mode,&rewopt,density,&access,mntmsg,tapeid);
if (status != TAPE_SUCC)
    {
    rstat = E_FAIL;
    c_errmsg("Error mounting tape"," ",&nonfatal);
    goto xit1;
    }
sprintf(emsg,"Tape %s has been mounted on drive %s",tapeid,mntmsg);
c_errmsg(emsg," ",&nonfatal);
buf = (char *)malloc(blksiz);
if (buf == NULL)
    {
    rstat = E_FAIL;
    sprintf(emsg,"Error allocating %d byte data buffer",blksiz);
    c_errmsg(emsg," ",&nonfatal);
    goto xit;
    }
msgflg = GEN1;
c_inittm("CPD2T","Processed","record",""," ",&dummy,&dummy,&msgtim,&msgflg);
curent = 0;
c_startm();
for (;;)
    {
    nbyte = read_input_file(fd,buf,blksiz);
    if (nbyte < 0)
        {
	rstat = E_FAIL;
        sprintf(emsg,"Error reading record %d of file %s",curent,fname);
	c_errmsg(emsg," ",&nonfatal);
	goto xit;
        }
    if (nbyte == 0)
	break;
    ++curent;
    status = c_tpwrit(&unit,buf,&nbyte,&mode);
    if (status < 0)
	{
	switch (status)
	    {
	    case TAPE_EOT:
	        c_errmsg("EOT detected"," ",&nonfatal);
	        goto xit;
	        break;
	    default:
	        sprintf(emsg,"Error writing record %d to tape",curent);
	        c_errmsg(emsg," ",&nonfatal);
		rstat = E_FAIL;
	        goto xit;
	    }
	}
    }
xit:
c_stoptm();
close(fd);
rewopt = REWOFFL;
/* rewopt = REWIND; */
eofopt = 2;
status = c_tpclos(&unit,&mode,&rewopt,&eofopt);
if (status != TAPE_SUCC)
    c_errmsg("Error dismounting tape"," ",&nonfatal);
xit1:
return(rstat);
}

FUNCTION print_label(vblock,tpdrive,tapeid,status)
	struct PARBLK *vblock;
	char *tpdrive;
	char *tapeid;
	long status;
{
struct VARIABLE *p_find(), *val;
long cnt;
long nmonth;
long rstat;
char *thebuf[11];
char bufspace[11][256];

for (cnt=0; cnt<11; cnt++)
    thebuf[cnt] = bufspace[cnt];
val = p_find(vblock,"CODE");
strcpy(thebuf[0],SVAL(*val,0));
val = p_find(vblock,"TAPEDENS");
strcpy(thebuf[1],SVAL(*val,0));
val = p_find(vblock,"DESCRPT");
strcpy(thebuf[2],SVAL(*val,0));
strcpy(thebuf[3],tpdrive);
val = p_find(vblock,"PROJECT");
strcpy(thebuf[4],SVAL(*val,0));
val = p_find(vblock,"NAME");
strcpy(thebuf[5],SVAL(*val,0));
strcpy(thebuf[6],"1/1");
strcpy(thebuf[7],tapeid);
if (status == E_SUCC)
    strcpy(thebuf[8],"SUCC");
else
    strcpy(thebuf[8],"ERROR");
strcpy(thebuf[9],"");
strcpy(thebuf[10],"");
val = p_find(vblock,"NMONTHS");
nmonth = IVAL(*val,0);
rstat = c_tapelabel(thebuf,nmonth,TRUE,1,"xx");
if (rstat != E_SUCC)
    c_errmsg("Error printing tape label"," ",&nonfatal);
return(rstat);
}

FUNCTION read_input_file(infd,buf,nbytes)
        int infd;
        char *buf;
        long nbytes;
{

#ifdef unix
    int status;

    status = read(infd,buf,nbytes);
    if (status < 0)
        {
        perror("read_input_file");
        }
    return(status);
#endif
#ifdef vms
    static long First = TRUE;
    static long Crlf;
    static long Total_bytes;
    static long St_fab_mrs;
    struct stat statbuf;
    int size;
    int status;
    int total;
    char *malloc();
    char *ptr;
    char *tptr;

    if (First)
        {
	First = FALSE;
	Crlf = FALSE;
	St_fab_mrs = 0;
        status = fstat(infd,&statbuf);
        if (statbuf.st_fab_rat == 2)	/* carriage control line feed */
	    {
	    Crlf = TRUE;
	    St_fab_mrs = statbuf.st_fab_mrs;
	    }
	Total_bytes = 0;
        }
    ptr = buf;
    size = nbytes;
    for (total=0; total<nbytes; total+=status)
        {
        status = read(infd,ptr,size);
        if (status == 0)
            {
            break;
            }
        else if (status < 0)
            {
            perror("read_input_file");
	    total = status;
            break;
            }
	Total_bytes += status;
	if ((Crlf) && (Total_bytes % (St_fab_mrs + 1) == 0))
	    {
	    --status;
	    }
        ptr += status;
        size -= status;
        }
    return(total);
#endif
}
