#include "cwtgen.h"
#include "protos.h"

void cwtbox
(

/*
--------------------------------------------------------------------
     FUNCTION cwtbox

     purpose:
        create box filter.

     calling sequence:  call cwtbox(block,nrow,ncol,norm,wgt)

     arguments:  
        name         type     i/o    description
        ----         ----     ---    -----------
        nrow         long      o     number of rows in weight function
        ncol         long      o     number of columns in weight function
        norm         float     o    normalization factor
        wgt          float     o     weight array

     file references: none  

     common block references: none 

     subroutines called: 
        name         description
        ----         -----------
        cwtdim       get dimensions of weight function

     error conditions: none
 
     program history:
     version   date     author         code/contr.   change request
        1      5/12/82  r. wenger         csc
        2      2/7/84   s. brodd          sar        scn 20-154  
	3      7/10/84  s. brodd          sar        scn 20-265
        4      6/21/89  J. Mcguthry       STX        conversion to C
        4      9/12/90  F. Irani	  STX        Inline doc cleanup

     computer:  vax-vms/Unix 

     project:  land analysis system 

     language:  C
*
**   pdl:

     begin

         call cwtdim to get dimensions of weight function
         fill array with ones
         set norm to number of weights

**   end
-----------------------------------------------------------------------
*/
    long *row,              	/* number of rows in weight function 	*/
    long *col,              	/* number of rows in weight function 	*/
    double *norm,             	/* norm for weight function 		*/
    double wgt[]            	/* weight function 			*/
)

{

long i;				/* loop counter				*/
long rowcol;			/* Total number of matrix elements	*/

/*
Get dimensions of weight function.
----------------------------------					*/
cwtdim ( row, col);
rowcol = *row * *col;

/*
Fill matrix with ones.
----------------------							*/
for (i = 0; i < rowcol; ++i)
	wgt[i] = 1.;

/*
Set norm to total number of elements.
-------------------------------------					*/
*norm = rowcol;

return;

}
