#include "cwtgen.h"
#include "util.h"
#include "protos.h"

void cwtcub
(

/**************************************************************************
*    FUNCTION cwtcub

     purpose:
        create cubic convolution filter.

     program history:
     version   date     author         code/contr.   change request
        1      5/12/82  r. wenger         csc 
        3      7/10/84  s. brodd          sar        scn 20-265
	4      6/21/89  J. McGuthry       STX        conversion to C
   	5      9/10/90  F. Irani	  STX	     inline doc. cleanup

	PDL:
		Get user specifiections from TAE.
		Initialize variables
		Calculate convolution filter for X
		Calculate convolution filter for Y
		Calculate Weight function
		Set normalization value if necessary
		Fill output string buffer with DX and DY values
		Return

***************************************************************************/
    long *row,                   /* number of rows and cols of weight function */
    long *col,                   /* number of rows and cols of weight function */
    double *norm,                  /* normalization constant 	*/
    double wgt[],                  /* weight function 		*/
    char label[]		/* Output string buffer		*/
)

{

long    one = 1;            /* constant     */

double  wx[4];			/* Convolution parameters for X		   */
double  wy[4];			/* Convolution parameters for Y		   */
double  dx;			/* float equivalent convolution coefficient */
double  dy;			/* float equivalent convolution coefficient */

float   dx1;			/* User specified convolution coefficients */
float   dy1;			/* User specified convolution coefficients */

long    type;			/* TAE Parameter data type specifier	   */
long    i,j,k;			/* Loop counters			   */

/*
Get user specifiections from TAE.
---------------------------------		*/
type = EREAL;
c_getnum ( &vblock, "DX", &one, &type, &dx1);
c_getnum ( &vblock, "DY", &one, &type, &dy1);

/*
Initialize variables.
---------------------				*/
dx = (double) dx1;
dy = (double) dy1;
*row = 4;
*col = 4;

/*
Calculate convolution filter for x.
-----------------------------------		*/
wx[0] = -dx * (1. - dx) * (1. - dx);
wx[1] = (1. - 2. * (dx * dx) + (dx * dx * dx));
wx[2] = dx * (1. + dx - dx * dx);
wx[3] = (dx - 1.) * dx * dx;

/*
Calculate convolution filter for y.
-----------------------------------		*/
wy[0] = -dy * (1. - dy) * (1. - dy);
wy[1] = (1. - 2. * (dy * dy) + (dy * dy * dy));
wy[2] = dy * (1. + dy - dy * dy);
wy[3] = (dy - 1.) * dy * dy;

/*
Calculate Weight function.
--------------------------			*/
for(i = 0; i < *row; ++i)
{
	for(j = 0; j < *col; ++j)
	{
		k = (i) * (*col) + j;
		wgt[k] = wy[i] * wx[j];
	} 
} 

/*
Set normalization if necessary.
-------------------------------			*/
if (*norm  ==  0.)  
	*norm = 1.; 

/*
Fill output string buffer with DX and DY values.
------------------------------------------------		*/
sprintf ( label, "DX= %4.3f DY= %4.3f", dx, dy);

return;
}
