#include "cwtgen.h"
#include <string.h>
#include "util.h"
#include "protos.h"
#define sqr(x)     (x*x)

void cwtgau
(

/*************************************************************************
     FUNCTION cwtgau 

     purpose:
       construct gaussian filter.

     program history:
     version   date     author         code/contr.   change request
        1      5/12/82  r. wenger         csc   
        2      2/7/84   s. brodd          sar        scn 20-154
	4      6/17/89  J. McGuthry       STX        conversion to C
	5      9/10/90  F. Irani	  STX        Inline Doc cleanup
						     Error Messages
						     Calculation corrections
    pdl:

     begin
	 Initialize output header.
         Get weight function dimensions from TAE block
         Get horizontal and vertical parameters from TAE block
         Get maximum pixel estimate from TAE block
     
         validate user specified weight function dimensions
         if requested gaussian must be truncated then
             signal user "truncated gaussian"
         endif
         compute weight array (see user's guide)
         set norm to 1
         construct special information (gaussian parameters)
         for descriptor label     
     end
***********************************************************************

*/
    long *row,      	/* number of rows and cols of weight function 	*/
    long *col,      	/* number of rows and cols of weight function 	*/
    double *norm,     	/* normalization constant 			*/
    double wgt[],     	/* weight function 				*/
    char label[]   	/* descriptor label 				*/
)

{

long    inf = 1;            /* c_errmsg flag    */
long    one = 1;            /* constant     */

char    msg[CMLEN];  	/* Message buffer				*/

double  c;		/* weight function dimension validator 		*/
double  colfloat;	/* Float value for column 			*/
double  con;		/* Weight coefficient				*/
double	horz;           /* horizontal and vertical gaussian parameters 	*/
double  pi=3.141593;	/* Constant PI 					*/
double	pmax;           /* maximum pixel value estimate 		*/
double  r;		/* weight function dimension validator 		*/
double  rowfloat;	/* Float value for row  			*/
double	vert;           /* horizontal and vertical gaussian parameters 	*/

float   horz1;		/* HORTVAL variable  				*/
float   pmax1;		/* MAXPIX variable  				*/
float   vert1;		/* VERTVAL variable  				*/

long    kc;		/* Temporary variable	     			*/
long    kcol;		/* Midpoint for column	     			*/
long    kr;		/* Temporary variable	     			*/
long    krow;		/* Midpoint for row	     			*/
long    i,j,k;		/* column, row, matrix element counters 	*/
long    type;		/* Input parameter data type 			*/

/*
Initialize output header.
-------------------------						*/
strcpy(label, "");

/* 
Get dimensions
--------------								*/
cwtdim ( row, col);

/* 
Get horz, vert and max.
-----------------------							*/
type = EREAL;
c_getnum (&vblock, "HORIZVAL", &one, &type, &horz1);
c_getnum (&vblock, "VERTVAL", &one, &type, &vert1);
c_getnum (&vblock, "MAXPIX", &one, &type, &pmax1);

pmax = (double)pmax1;
horz = (double)horz1;
vert = (double)vert1;

/*
Validate user specified weight function dimensions.
---------------------------------------------------			*/
r = 2.0 * sqrt((2.0 * sqr(horz)) * log(pmax / (pi * horz * vert)));
c = 2.0 * sqrt((2.0 * sqr(vert)) * log(pmax / (pi * horz * vert)));
krow = *row / 2;
kcol = *col / 2;

/*
Check for odd dimensions.
-------------------------						*/
if (krow * 2  ==  *row  ||  kcol * 2  ==  *col) 
{
    *row = 2 * krow + 1;
    *col = 2 * kcol + 1;
    sprintf(msg,
	    "Gaussian needs odd dimensions. (NROW,NCOL) reset to (%d,%d).",
	    *row, *col);
    c_errmsg(msg, "cwtgen-oddgau", &inf);
} 

/*
Check for appropriate dimensions.
---------------------------------					*/
rowfloat = (double) *row;
colfloat = (double) *col;
if (r  >  rowfloat  ||  c  >  colfloat ) 
{
	kr = (long) (r + 0.9999);
	kc = (long) (c + 0.9999);
	kr = kr + ((kr + 1) % 2);
	kc = kc + ((kc + 1) % 2);
	sprintf(msg,
		"Gaussian parameters suggest (%d, %d). Truncated to (%d,%d).",
		kr, kc, *row, *col);
	c_errmsg(msg, "cwtgen-trungau", &inf);
	strcat(label, "TRUNGAU ");
} 

/*
Compute each weight function value
----------------------------------					*/
con = 1.0 / (2. * pi * horz * vert);
k = 0;
for(j = krow; j >= -krow; --j)
{
    for(i = -kcol ;i <= kcol; ++i, ++k)
    {
      /*
      Find value for each element.
      ----------------------------		*/
      func(i, j, horz, vert, con, &wgt[k]);
    } 
} 

*norm = 1.;

sprintf(msg, "HORIZVAL=%10.3f VERTVAL= %10.3f MAXPIX= %10.3f ", horz,
           vert, pmax);

strcat(label, msg);

return;

}

void func
(
    long i,		/* col number 			*/
    long j,		/* row number 			*/
    double horz,	/* HORTVAL specification 	*/
    double vert,	/* VERTVAL specification 	*/
    double con,	/* Weight coefficient		*/
    double *value  /* Weight value		 	*/
)

{
   *value =  (double)( con * exp(-.5 * (sqr(j / horz) + sqr(i / vert) )) );
}
