/*****************************************************************************
     program cwtgen

     purpose:
       generate weight functions to be used by the convolve program.

     program history:
     version   date      author         code/contr.   change request
        1      5/12/82   r. wenger         csc      
	2      2/7/84    s. brodd          sar        scn 20-154
        3      7/10/84   s. brodd          sar        scn 20-265
        4      8/ 8/85   f. irani          sar        scn 20-511
	5                J. MCGuthry       STX        conversion to C
	6      69/6/90   F. Irani          STX        Testing fixes, e.g.
						      printer output,
						      complt message, and
						      inline doc cleanup
 
     project:  land analysis system 

     language:  C
 
     pdl:

     begin

	Get TAE parameters and image labels
	Retrieve name of output file from TAE block
	Retrieve subcomand name 
	Retrieve bias value 
	Check if an output file was specified
	If subcommand is MANUAL
		Get value for INWT from TAE block
		If INWT specified
		Get specified values from TAE block
		If output file name was defaulted
			open default output file cwtgenin;dat
		Call cwtman for user interaction
	If subcommand is BOX
		Call cwtbox
	If subcommand is CUBIC
		Get value of NORMVAL from TAE block
		Call cwtcub
	If subcommand is DIFF
		Get value of NORMVAL from TAE block
	    	call cwtdif
	If subcommand is GAUSS
	    	call cwtgau
	If subcommand is LAPLACE
		Get value of NORMVAL from TAE block
		call cwtlap

	If return status is zero
	   Construct descriptor label for weight function and write to screen

	If return status is less than 2
		Display weight function
		Write weight function to ascii file

	if return status is less than zero  (user cancelled function).
		delete output file created by this run
		call complete with no output file name

	else if output file was specified
		call complete with user specified output file name

	else if output file was not specified
		call complete with cwtgenin.dat as output file name

	else
		call complete with no output file name
	
	end cwtgen

**************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "las.h"
#include "util.h"
#include "protos.h"

struct PARBLK vblock;                   /* TAE parameter block  */

static  long    fatal = -1;                     /* c_errmsg flag        */
static  long    inf = 1;                        /* c_errmsg flag        */
static  long    zero = 0;                       /* constant             */
static  long    one = 1;                        /* constant             */
static  long    two = 2;                        /* constant             */

#define TERM 0                                  /* Fileutil print index */
#define LP   1                                  /* Fileutil print index */
#define FIL  2                                  /* Fileutil print index */


int main
(void)

{

FILE    *fpout = NULL;		/* Output file pointer 				*/

char    deftext[10];    /* "dat" file extension string			*/
char    filein[CMLEN];  /* Output file name 				*/
char    hosout[1][CMLEN];  /* Input host file name 			*/
char    output[CMLEN];  /* Descriptor label string		 	*/
char    prtout[CMLEN];  /* Output line printer file name 	*/
char	*ptr;		/* Character string pointer for q_string	*/
char	subcmd[20];     /* name of weight generation choice 		*/
char    spec[80];	/* Specification string for options 		*/
char    tmpstr[1][CMLEN];  /* Temp String */
char    wgtext[1024];   /* Descriptor label for weight function. 	*/

double	bias;           /* norm and bias of weight function 		*/
double	norm;           /* norm and bias of weight function 		*/ 
double	wgt[226];       /* weight function array 			*/

long    access;		/* open status: read/write  			*/
long	cnum;           /* display formatter flag			*/
long	col;            /* dimension of weight function 		*/
long    dtype;		/* Parameter type 	 			*/
long    eight = 8;	/* Constatnt 					*/
long    i, k, k2;	/* Loop dimensions and counters 		*/
long	len;		/* string length buffer				*/
long	manstat;	/* return status 				*/
long	print[3];	/* Fileutil output destination parameter	*/
long	row;            /* dimension of weight function 		*/
long    std_out;	/* Inlas parameter       			*/
long	tprint[3];	/* c_txtout terminal only output parameter	*/
long    wrote = 0;		/* Output display formatter flag		*/

/*
Initialize TAE and some parameters.
-----------------------------------					*/
c_inlas(&vblock, &std_out);

strcpy(wgtext, "\0");
strcpy(spec, "\0");
access = IWRITE;

/*
Retrieve name of subcomand file.
--------------------------------					*/
if (c_getstr ( &vblock, "_SUBCMD", &two, &eight, subcmd) != one)
	c_errmsg ("Error accessing subcommand name", "cwt-in", &fatal);
else
{
	len = (long) strlen(subcmd);
	c_low2up (subcmd, &len);
}

/*
Retrieve bias value 
-------------------							*/
dtype = EDOUBLE;
if (c_getnum ( &vblock, "OFFSET", &one, &dtype, &bias) != 1)
	c_errmsg("Error retrieving value for OFFSET","cwt-noval", &fatal);


/*
Retrieve name of output destination.
------------------------------------					*/
strcpy(prtout, "CWT");
if ( c_setprt(&vblock, &print[TERM], &print[FIL], &print[LP], prtout, hosout[0])
    != E_SUCC)
	c_errmsg("Error specifying output destination.", 
		 "cwt-print", &fatal); 

/*
Initialize terminal destination pointer.
----------------------------------------				*/
tprint[LP] = tprint[FIL] = 0;
tprint[TERM] = print[TERM];

/*
Check if an output file was specified.
--------------------------------------					*/
if (print[FIL] || print[LP])
{
	/*
	Open output for printer. Fileutil will copy to a permanent file.
	---------------------------------------------------------------- */
	fpout = fopen(prtout,"w");
}


norm = 0.0;

manstat = 0;
if (strcmp(subcmd,"MANUAL") == 0 ) 
{
    access = IREAD;
    strcpy(deftext,"prt");
    if (c_getnam ( &vblock, "INWT", &zero, filein, deftext, &access) != E_SUCC)
    {
	/*
	User cancelled output.
	----------------------						*/
	if (print[FIL] || print[LP])
	{
		/*
		Delete opened file and terminate.
		---------------------------------			*/
		fclose(fpout);
		c_delfil(prtout);
	}
        c_errmsg("File specified by INWT not found", "cwt-nofil", &fatal);
    }
    if (strlen(filein) > 0) 
    {
	getwts (&row, &col, &norm, &bias, wgt, filein);
        cwtman ( &row, &col, &norm, wgt, &manstat, &one);
    }
    else
    {
        cwtman ( &row, &col, &norm, wgt, &manstat, &zero);
    }

}
else if ( strcmp(subcmd,"BOX") == 0 ) 
    cwtbox ( &row, &col, &norm, wgt);

else if ( strcmp(subcmd,"CUBIC") == 0 )
{
    dtype = EDOUBLE;
    c_getnum ( &vblock, "NORMVAL", &one, &dtype, &norm); 
    cwtcub ( &row, &col, &norm, wgt, spec);
}
else if ( strcmp(subcmd,"DIFF") == 0 ) 
{
    dtype = EDOUBLE;
    c_getnum ( &vblock, "NORMVAL", &one, &dtype, &norm); 
    cwtdif (&row, &col, &norm, wgt );
}
else if ( strcmp(subcmd,"GAUSS") == 0 ) 
{
    cwtgau ( &row, &col, &norm, wgt, spec);
}
else if ( strcmp(subcmd, "LAPLACE") == 0 ) 
{
    dtype = EDOUBLE;
    c_getnum ( &vblock, "NORMVAL", &one, &dtype, &norm); 
    cwtlap ( &row, &col, &norm, wgt );
} 

if (manstat == 0) 	
{

   /*
   Construct descriptor label for weight function.
   ----------------------------------------------- 			*/
   sprintf(output,"  TYPE   SIZE");
   c_txtout(output, tprint, fpout);
   sprintf(wgtext, "%7s %2d X %2d\nNORM= %10.3f OFFSET=%10.3f",
           subcmd, row, col, norm, bias);
}

/*
Display weight function
-----------------------							*/
if (manstat < 2) 
{
    c_txtout(wgtext, print, fpout);
    c_txtout(spec, print, fpout);
    for(i = 0; i < row; ++i)
    {
       sprintf(output, "Row %2d ", i+1); 
       k2 = (i + 1) * col;

       cnum = 0;
       for (k = k2 - col; k < k2; ++k)
       {
		++cnum;
		wrote = FALSE;
		sprintf(&output[strlen(output)], "%10.3f ", wgt[k]);
		if (cnum % 5 == 0)
		{
			c_txtout(output, print, fpout);
			strcpy(output, "       ");
			wrote = TRUE;
		}
       }
       if (!wrote)
	        c_txtout(output, print, fpout);
    } 
}
else 
{
	/*
	User cancelled output.
	----------------------						*/
	if (print[FIL] || print[LP])
	{
		/*
		Delete opened file and terminate.
		---------------------------------			*/
		fclose(fpout);
		c_delfil(prtout);

		/*
		Reset output filename to avoid running fileutil.
		------------------------------------------------	*/
	        strcpy(prtout, "--");
	        ptr = &prtout[0];
	        q_string (&vblock, "PRTFILE", one, &ptr, P_UPDATE);
	        q_out (&vblock);
	}
	c_errmsg(" Processing terminated by user.", "cwt-uterm", &inf);
    strcpy(tmpstr[0], "\0");
	c_complt ("CWTGEN", tmpstr, &zero);
}	/* If user termination */

/*
Print appropriate completion message.
-------------------------------------					*/
if ( print[FIL] )
{
	/*
	Normal completion with user specified output file created.
	---------------------------------------------------------- 	*/
	c_complt ("CWTGEN", hosout, &one);
}
else
{
	/*
	Normal completion with no output file created.
	----------------------------------------------			*/
    strcpy(tmpstr[0], "\0");
	c_complt ("CWTGEN", tmpstr, &zero);
}

return EXIT_SUCCESS;
}
