#include "cwtgen.h"
#include "protos.h"

void cwtlap
(
/**********************************************************************
*    FUNCTION cwtlap

     purpose:
        Create Laplacian filter.

     program history:
     version   date     author         code/contr.   change request
        1      5/12/82  r. wenger         csc
	2      6/17/89  J. McGuthry       STX        conversion to C
	3      9/10/90  F. Irani 	  STX        Inline doc cleanup

     language:  C

     pdl:

     begin

         set dimensions to 3x3
         fill array with laplacian elements (see functional design)

         if norm is zero then
             set norm to 1
         endif

     end


*************************************************************************/
    long *row,	/* number of rows and cols of weight function 		*/
    long *col,   /* number of rows and cols of weight function 		*/
    double *norm,  /* norm for weight function 				*/
    double wgt[]  /* weight function 					*/
)

{

/*
Set dimensions to 3x3.
----------------------			*/
*row = 3;
*col = 3;

/*
Fill array with Laplacian elements.
-----------------------------------	*/
wgt[0] = 0.;
wgt[1] = 1.;
wgt[2] = 0.;
wgt[3] = 1.;
wgt[4] = -4.;
wgt[5] = 1.;
wgt[6] = 0.;
wgt[7] = 1.;
wgt[8] = 0.;

/*
Set norm to 1 if necessary.
---------------------------		*/
if (*norm  ==  0.0)
	*norm = 1.0;

return;

}
