#include "cwtgen.h"
#include <string.h>
#include "util.h"
#include "protos.h"

void cwtman
(
/*************************************************************************

     purpose:
        Allow user to manually enter or edit convolution weights.

     program history:
     version   date     author         code/contr.   change request
        1      5/12/82  r. wenger         csc
        2      2/7/84   s. brodd          sar        scn 20-154
        3      7/10/84  s. brodd          sar        scn 20-265
	4      6/17/89  J. McGuthry       STX        conversion to C
	5      9/10/90  F. Irani	  STX        Inline doc cleanup
						     Allow user to enter
						     comma or blank separators

     language:  C

     pdl:

     begin
	Get new input from user
	Prompt user for input elements according to top/bottom, 
	left/right or no symmetry as requested.
	begin editor functions
	write editor options
	dowhile more editing is desired
	   request and read command
	   switch on command:
		if 'l' then list array matrix
		if 's' then sum elements of matrix and print
		if 'd' then divide matrix by specified amount
		if 'm' then modify specified element of matrix
		if 'e' then copy matrix into array wgt,
		   set norm to sum of elements, and return
		if 'c' then cancel without output
		if 'h' then print command list
		else reprint editor options
	   end switch
	enddo
     return

***************************************************************************/
    long *row,        	/* number of columns and rows of weight function */
    long *col,        	/* number of columns and rows of weight function */
    double *norm,     	/*  norm and weights 				*/
    double wgt[],     	/*  norm and weights 				*/
    long *manstat,	/* Return status 				*/
    long *input		/* Flag for input weight file   		*/
)

{

long    inf = 1;            /* c_errmsg flag    */

FILE    *tout = NULL;		/* Terminal Output pointer (dummy)		*/

char	ans[CMLEN];	/* User response buffer				*/
char	move;		/* Character response equivalent for switch	*/

double  fval;		/* Matrix element modifier variable		*/
double	matrix[15][15];	/* array for weight matrix 			*/
double  sum;		/* Sum of matrix elements			*/

long    collr;
long	first;  	/* First pass on menu display flag		*/
long    len;		/* String length				*/
long    i,j,k,l,m;	/* Loop counters				*/
long    lrsym;		/* Left/right symmetry flag			*/
long    ncol;		/* Column counter				*/
long    nrow;		/* Row counter					*/
long	ok = FALSE;   	/* check if function satisfies user 		*/
long    rowtb;		/* row midpoint					*/
long    stat;		/* Editing flag					*/
long    tbsym;		/* Top/bottom symmetry flag			*/
int     term;		/* TAE t_read flag				*/
long    tprint[3];	/* c_txtout write to screen only flag		*/
long    wrote = 0;		/* Edit display formatter flag			*/

/*
Initialize flag.
----------------					*/
tprint[TERM] = 1;
tprint[LP] = tprint[FIL] = 0;
/*
Get new input from user
-----------------------					*/
if (*input == 0) 
{
   cwtdim ( row, col);
   stat = FALSE;
   t_write("Do you wish right/left symmetry? (q = Quit; Default = No):",
	   T_PROMPT); 
   gets (ans);
   if (strncmp(ans, "q", 1) == 0 || strncmp(ans, "Q", 1) == 0)
   {
	*manstat = 2;	
	return;
   }
   if ( *ans != '\0' ) 
   {
      if (strncmp(ans, "YES", 1) == 0 || strncmp(ans, "yes", 1) == 0)
	lrsym = TRUE;
      else
	lrsym = FALSE;
   }
   else 
	lrsym = FALSE;

   t_write(" Do you wish top/bottom symmetry? (q = Quit; Default = No):",
	   T_PROMPT);
   gets (ans);
   if (strncmp(ans, "q", 1) == 0 || strncmp(ans, "Q", 1) == 0)
   {
	*manstat = 2;	
	return;
   }
   if ( *ans != '\0' ) 
   {
      len = (long) strlen(ans);
      if ( strncmp(ans, "yes",  len) == 0 || strncmp(ans, "YES",  len) == 0 ) 
	tbsym = TRUE;
      else                        
	tbsym = FALSE;
   }
   else                        
      tbsym = FALSE;

/*
READ routine for top/bottom symmetry; possible left/right symmetry
------------------------------------------------------------------	*/
   if (tbsym) 
   {
      rowtb = *row / 2 + ( *row % 2 );
      for(i = 0; i < rowtb; ++i)
      {
         if (lrsym) 
         {
            collr =  *col / 2 + ( *col % 2);
            for (j = 0; j < collr;++j)
            {
              stat = FALSE;
              while ( !stat )
              {
                sprintf(ans, "Enter weight %2d for row %2d (q = Quit):", 
			      j+1, i+1);
                t_write(ans, T_PROMPT);
                gets (ans);
		if (strncmp(ans, "q", 1) == 0 || strncmp(ans, "Q", 1) == 0)
		{
			*manstat = 2;	
			return;
		}
                if ( *ans != '\0' )
                {
                  sscanf(ans, "%lf", &matrix[i][j]);
                  stat =  TRUE;
                }
              }
            }
            for (m = 0, k = (*col-1); k >= collr;--k, ++m)
            {
               matrix[i][k] = matrix[i][m];
            } 
         }
         else
         {
            for (j = 0; j < *col; ++j)
            {
              stat = FALSE;
              while ( !stat )
              {
                sprintf(ans,"Enter weight %2d for row %2d (q = Quit):", 
			j+1, i+1);
                t_write(ans, T_PROMPT);
                gets (ans);
		if (strncmp(ans, "q", 1) == 0  || strncmp(ans, "Q", 1) == 0) 
		{
			*manstat = 2;	
			return;
		}
                if ( *ans != '\0' )
                {
			sscanf(ans, "%lf", &matrix[i][j]);
			stat =  TRUE;
                }
              }
            }
         } 
      } 	
/*
Fill out matrix with correct values
-----------------------------------					*/
      for(k = (*row-1), m = 0; k >= rowtb; --k, ++m)
      {
         for(l = 0; l < *col; ++l)
         {
            matrix[k][l] = matrix[m][l];
         }
      }
/*
No top/bottom symmetry; possible left/right symmetry
----------------------------------------------------			*/
   }
   else
   {
      for(i = 0; i < *row; ++i)
      {
         if (lrsym) 
         {
            collr =  *col / 2 + (*col % 2);
            for (j = 0; j < collr; ++j)
            {
              stat = FALSE;
              while ( !stat )
              {
                sprintf(ans, "Enter weight %2d for row %2d (q = Quit):",
			j+1, i+1);
                t_write(ans, T_PROMPT);
                gets (ans);
		if (strncmp(ans, "q", 1) == 0 || strncmp(ans, "Q", 1) == 0) 
		{
			*manstat = 2;	
			return;
		}
                if ( *ans != '\0' )
		{
                  sscanf(ans, "%lf", &matrix[i][j]);
                  stat =  TRUE;
                }
              }
            }
            for (k = (*col-1), m = 0; k >= 0; --k, ++m)
                matrix[i][k] = matrix[i][m];
         }
         else
         {
            for (j = 0; j < *col; ++j)
            {
              stat = FALSE;
              while ( !stat )
              {
                sprintf(ans, "Enter weight %2d for row %2d (q = Quit):",
			j+1, i+1);
                t_write(ans, T_PROMPT);
                gets (ans);
		if (strncmp(ans, "q", 1) == 0 || strncmp(ans, "Q", 1) == 0) 
		{
			*manstat = 2;	
			return;
		}
                if ( *ans != '\0' )
                {
                  sscanf(ans, "%lf", &matrix[i][j]);
                  stat =  TRUE;
                }
              }
            }
         }
      } 
/*
Priori input exists
-------------------							*/
   }
}
else
{
   l = 0;
   for(i = 0; i < *row; ++i)
   {
      for(j = 0; j < *col; ++j, ++l)
      {
         matrix[i][j] = wgt[l];
      } 
   } 
}

/*
Begin editor functions
----------------------							*/
move = 'h';
first = TRUE;

while(!ok)
{

if (!first)
{
  t_write ( "\n PLEASE ENTER A COMMAND: ", T_PROMPT);
  scanf ("%10s", ans);
  move = ans[0];
}
else
{
  first = FALSE;
  move = 'h';
}

/*
These are the various editor options and their effects
------------------------------------------------------			*/
  switch (move)
  {
    case 'h':
    case 'H':
      t_write("", T_STDCC);
      t_write("DIVIDE --Divide all elements by a specified number", T_STDCC);
      t_write("EXIT   --Write out weight file and terminate program", T_STDCC);
      t_write("HELP   --List command names", T_STDCC);
      t_write("LIST   --List the weight file", T_STDCC);
      t_write("MODIFY --Modify a specified element", T_STDCC);
      t_write("QUIT   --Terminate program without output", T_STDCC);
      t_write("SUM    --Sum all elements in the weight file", T_STDCC);
      t_write(
      "In all cases, the first letter of the command is sufficient", T_STDCC);
      break;
    case 'l':
    case 'L':
      for(i = 0; i < *row; ++i)
      {
        sprintf(ans, "Row %2d", i+1);
        for(j = 1;  j <= *col; ++j)
        {
          wrote = FALSE;
          len = (long) strlen(ans);
          sprintf(&ans[len], " %10.3f", matrix[i][j-1]);
	  if (j % 5 == 0)
	  {
	        c_txtout(ans, tprint, tout);

		/*
		Padd remainder of row with blanks for display.
		----------------------------------------------		*/
		strcpy(ans,"      ");
		wrote = TRUE;
	  }
        } 
        if (!wrote)
	        c_txtout(ans, tprint, tout);

      }
      break;
    case 's':
    case 'S':
      sum = 0.;
      for(i = 0; i < *row;++i)
      {
         for(j = 0; j < *col; ++j)
             sum = sum+matrix[i][j];
      } 
      sprintf (ans, "The Sum of the weights is: %10.3f", sum);
      t_write (ans, T_STDCC);
      break;
    case 'd': 
    case 'D': 
      stat = FALSE;
      t_write ("Enter the real-valued divisor (q = Quit):", T_PROMPT);
      while ( !stat )
      {
         gets (ans);
 	 if (strncmp(ans, "q", 1) == 0 || strncmp(ans, "Q", 1) == 0) break;
         if ( *ans != '\0' )
         {
           sscanf(ans, "%lf", &fval);
           stat =  TRUE;
         }
      }
      if (fval == 0.0) 
      {
         c_errmsg ("Division by zero is not allowed.","cwt-zediv", &inf);
         break;
      } 
      for(i = 0; i < *row; ++i)
      {
        for(j = 0; j < *col; ++j)
        {
         matrix[i][j] = matrix[i][j] / fval;
        } 
      }
      break;
    case 'm': 
    case 'M': 
      stat = FALSE;
      while ( !stat )
      {
	   t_write 
	   ("Please enter row, column, and modified real value (q = Quit):",
	     T_PROMPT);
	   if (t_read(ans, &term) == SUCCESS)
	   {
	    if (strncmp(ans, "q", 1) == 0 || strncmp(ans, "Q", 1) == 0) break;
            if (sscanf(&ans[0], "%d%*c%d%*c%lf", &nrow, &ncol, &fval) != EOF )
            {
             stat =  TRUE;
             --nrow;
             --ncol;
  	     if ((nrow < 0) || (nrow >= *row) || (ncol < 0) || (ncol >= *col))
	     {
		stat = FALSE;
		sprintf(ans, 
	        "Row/Column must be in the range 1 - %d/1 - %d.", *row, *col);
        	c_errmsg (ans, "cwt-range", &inf);
	     }
	    }
           }
      }
      matrix[nrow][ncol] = fval;
      break;
    case 'e': 
    case 'E': 
      ok = TRUE;
      *manstat = 0;
      l = 0;
      for(i = 0; i < *row; ++i)
      {
        for(j = 0;j  < *col; ++j, ++l)
           wgt[l] = matrix[i][j];
      } 
      *norm = 0;
      for(i = 0; i < (*row) * (*col); ++i)
        *norm = *norm + wgt[i];
      if (*norm  ==  0.) 
      {
        *norm = 1.;
	c_errmsg("Sum of weights is zero.  NORM reset to 1.",
		 "cwtgen-sumzero", &inf);
      }
      break;
    case 'q': 
    case 'Q': 
      ok = TRUE;
      *manstat = 2;
      break;
  }

}

return;

}
