#include "cwtgen.h"
#include "protos.h"
/*****************************************************************************
     program cwtwts

     purpose:
       generate weight functions to be used by the convolve program.

     program history:
     version   date      author         code/contr.   change request
        1      5/12/82   r. wenger         csc      
	2                J. MCGuthry       STX        conversion to C
	3      69/6/90   F. Irani          STX        Inline doc cleanup
 
     project:  land analysis system 

     language:  C
 
     pdl:

     begin

	Open cwtgen  file.
	Get subcommand name, row and column.
	Read appropriate parameters according to subcommand.
	Read in weight matrix.
	Return

     end


**************************************************************************/

void getwts
(
    long *row,		/* Matrix row dimension		*/
    long *col,		/* Matrix column dimension	*/
    double *norm,		/* Input specification for norm	*/
    double *bias,		/* Input specification for norm	*/
    double wgt[],		/* Input specification for norm	*/
    char filein[]		/* Input specification file	*/
)

{

FILE *fpout;

char subcmd[20];	/* Subcommand string		*/
char blank_ch;		/* Blank string holder		*/
char blank_word[20];	/* String buffer 		*/

double temp;		/* Dummy value			*/

long i, k, k2;		/* Loop indicies		*/
long num;		/* Column number		*/
 
/*
Open cwtgen  file.
------------------						*/
fpout = fopen(filein, "r");

/*
Get subcommand name, row and column.
------------------------------------				*/
fscanf(fpout, "%s %d %c %d", subcmd, row, &blank_ch, col);

/*
Read appropriate parameters according to subcommand.
----------------------------------------------------		*/
if (strcmp(subcmd, "MANUAL") == 0 ) 
{
    fscanf(fpout, "%s %lf", blank_word, norm);
    fscanf(fpout, "%s %lf", blank_word, bias);
}
else if ( strcmp(subcmd, "BOX") == 0 ) 
{
    fscanf(fpout, "%s %lf", blank_word, norm);
    fscanf(fpout, "%s %lf", blank_word, bias);
}
else if ( strcmp(subcmd, "CUBIC") == 0 )
{
    fscanf(fpout, "%s %lf", blank_word, norm);
    fscanf(fpout, "%s %lf", blank_word, bias);
    fscanf(fpout, "%s %lf", blank_word, &temp);
    fscanf(fpout, "%s %lf", blank_word, &temp);
}
else if ( strcmp(subcmd, "DIFF") == 0 ) 
{
    fscanf(fpout, "%s %lf", blank_word, norm);
    fscanf(fpout, "%s %lf", blank_word, bias);
}
else if ( strcmp(subcmd, "GAUSS") == 0 ) 
{
    fscanf(fpout, "%s %lf", blank_word, norm);
    fscanf(fpout, "%s %lf", blank_word, bias);
    fscanf(fpout, "%s %lf", blank_word, &temp);
    fscanf(fpout, "%s %lf", blank_word, &temp);
    fscanf(fpout, "%s %lf", blank_word, &temp);
}
else if ( strcmp(subcmd, "LAPLACE") == 0 ) 
{
    fscanf(fpout, "%s %lf", blank_word, norm);
    fscanf(fpout, "%s %lf", blank_word, bias);
} 

/*
Read in weight matrix.
----------------------					*/
for (i = 0; i < *row; ++i)
{
    k2 = (i + 1) * *col;
    fscanf(fpout, "%s %d", blank_word, &num);
    for (k = k2 - *col; k < k2; ++k)
    {
         fscanf(fpout, "%lf", &wgt[k]);
    }
}

return;

}
