
/* the main() for dctfil */ 

#include <stdlib.h>
#include <stdio.h>       /* for call to ftell() and fseek() */ 
#define GLOBALVARS       /* defined only in this file, flag for dctfil.h */
#include "dctfil.h"      /* for global vars and function prototypes */ 
#include "pixman.h"
#include "protos.h"

/* global vars */ 
   /* current entity, for processing message, must be global for las IO */ 
      extern long curent; 
   /* size of dct blocks */ 
      long bsize;     





/* FUNCTION:  read file of filter weights */ 
void read_wfile
(
    char *wfile,               /* input:   file to read */
    long nbands,               /* input:   number of bands in input image */
    long fwband[MAXBND],       /* output:  a lut to the band index for fweights,
				 accommodates separate/identical filter
				 weights for each image band */
    float **fweights[MAXBND+1]/* output:  weights in file, +1 aids check for
				 wrong number of data values in wfile */
) 
{
/* declare variables */ 
   char message[CMLEN]; /* output error message string */ 
   float testend;       /* a var used to test for end of file */ 
   FILE *filep;         /* file pointer */ 
   long band,           /* loop index, band number */ 
	filepos,        /* position in file */ 
	go_on=1,        /* argument to c_errmsg, continue processing */ 
        line,           /* loop index, line number */ 
	negvalue=FALSE, /* flag, was a negative filter weight read in */ 
        samp,           /* loop index, sample */
	wbands;         /* the number of bands of weights in fweights */ 
/* open file */ 
   if ((filep = fopen (wfile, "r")) == NULL)
   {
      sprintf (message, "Unable to open WFILE %s for reading.", wfile);
      clean2 (message); 
   }
/* read file, allocating fweights memory along the way, memory will be freed in
   bimage() */ 
   for (band=0; band<=MAXBND; band++)
   {
   /* get memory for this band */ 
      if ((fweights[band] = (float **) malloc (sizeof (float *) * bsize))==NULL)
	 clean2 ("Unable to allocate memory for filter weights."); 
   /* open loop for lines */ 
      for (line=0; line<bsize; line++)
      {
      /* get memory for this line */ 
	 if ((fweights[band][line] = (float *) malloc (sizeof (float) * bsize)) 
	 == NULL)
	    clean2 ("Unable to allocate memory for filter weights."); 
      /* read all sample points, checking for negative values */ 
	 for (samp=0; samp<bsize; samp++)
	 {
	    if (fscanf (filep, " %f",  fweights[band][line]+samp) != 1)
	    {
	       sprintf (message, 
	       "WFILE %s has the wrong number of filter weights.", wfile);
	       clean2 (message); 
	    }
	    if (fweights[band][line][samp] < (float)0)
	       negvalue = TRUE; 
	 }
      }
      /* get file position */ 
	 if ((filepos = ftell(filep)) == -1)
	    clean2 ("Error determining current position in WFILE."); 
      /* see if there is more data */ 
	 if (fscanf (filep, " %f",  &testend) == EOF)
	    break; 
      /* reset file position */ 
	 if (fseek (filep, filepos, 0))
	    clean2 ("Error seeking in WFILE."); 
   }
/* error check for wrong number of bands, negative filter values */ 
   fclose (filep); 
   wbands = band+1; 
   if (wbands != nbands && wbands !=1)
   {
      sprintf (message, 
      "WFILE %s has the wrong number of filter weights.", wfile);
      clean2 (message); 
   }
   if (negvalue) 
      c_errmsg (
      "A phase-reversing negative filter weight(s) appeared in WFILE.",
      "dctfil-warn", &go_on); 
/* fill fwband */ 
   for (band=0; band<nbands; band++)
      if (wbands==1)
	 fwband[band] = 0; 
      else 
	 fwband[band] = band; 
}





/* FUNCTION: create disk image, using random IO, processing the image in 
   blocks of lines */
void bimage
(
    float **fweights[MAXBND+1], /* input:  dct filter weights */
    long fwband[MAXBND],        /* input:  lut index for band for fweights */
    struct lasinimage *ini,    /* input:  specs for input image */
    struct lasoutimage *outi   /* input:  specs for output image */
) 
{
/* declare vars */ 
   float **dctdata,    /* block of dct image data */ 
         **imaged = NULL;     /* block of image data */ 
   long band,          /* for loop index:  image band */ 
	fixtrun=TRUE,  /* flag, correct for truncation (round-off) error */ 
	ftype=EREAL,   /* data type of imaged[][] */ 
        inoff,         /* offset to pixel location in bufloc[] */ 
	line,          /* for loop index:  line number */ 
        lineb,         /* for loop index:  block of image lines */ 
        outoff,        /* offset to pixel location in bufloc[] */ 
	sampb;         /* for loop index:  block in sample direction */ 
/* set constants used in dct calculations, allocate memory */ 
   set_const (bsize);
   if (outi->datatype==EREAL)
      fixtrun=FALSE; 
/* allocate memory for dctdata and imaged */ 
   if (
   (dctdata = (float **) malloc (sizeof (float *) * bsize)) == NULL ||
   (imaged  = (float **) malloc (sizeof (float *) * bsize)) == NULL  )
      clean2 ("Unable to allocate memory for image data."); 
   for (line=0; line<bsize; line++)
      if (
      (dctdata[line] = (float *) malloc (sizeof (float) * bsize)) == NULL ||
      (imaged [line] = (float *) malloc (sizeof (float) * bsize)) == NULL  )
	 clean2 ("Unable to allocate memory for image data."); 
/* open loops for each block of lines, each band */ 
   for (lineb=0; lineb<outi->lblocks; lineb++)
      for (band=0; band<outi->totalbands; band++) 
      {
      /* read data */ 
	 bread (lineb, band, ini); 
      /* loop for each block of samples */ 
	 for (sampb=0; sampb<outi->sblocks; sampb++, curent++)
	 {
	 /* copy data from ini to imaged */ 
	    inoff = sampb * bsize * ini->datasize;
	    outoff = sampb * bsize * outi->datasize;
	    for (line=0; line<bsize; line++)
	       c_pxconv (&ini->datatype, &ftype, ini->bufloc[line] + inoff, 
	       (unsigned char *) imaged[line], &bsize); 
	 /* process data */ 
	    /* perform forward DCT */ 
	       fdct (imaged, dctdata);   
	    /* apply filter weights */
	       app_weights (fweights[fwband[band]], dctdata);   
	    /* perform reverse DCT */ 
	       rdct (dctdata, imaged);   
	    /* adjust for round off error */ 
	       if (fixtrun)
		  fix_truncate (imaged); 
	 /* copy data from imaged to outi */ 
	    for (line=0; line<bsize; line++)
	       outi->truncate = outi->truncate |
	       c_pxconv (&ftype, &outi->datatype,(unsigned char *) imaged[line],
	       outi->bufloc[line] + outoff, &bsize); 
	 }
      /* update minmax data */ 
	 minmax (band, outi); 
      /* write data */ 
	 bwrite (lineb, band, outi); 
      }
/* free memory */ 
   /* memory allocated in set_const() */ 
      free_struct (); 
   /* fweights memory */ 
      for (line=0; line<bsize; line++)
	 free (fweights[line]); 
      free (fweights); 
   /* dctdata and imaged */ 
      for (line=0; line<bsize; line++)
      {
	 free (dctdata[line]); 
	 free (imaged [line]); 
      }
      free (dctdata); 
      free (imaged ); 
}





int main
(void)                    /* should be void main, but vax compiler */
{	                        /* likes long main better */    
/* declare variables */ 
   char odtype[10],             /* output data type user parm */ 
	wfile[101];             /* file of dct weights */ 
   float **fweights[MAXBND+1];  /* dct filter weights */ 
   long fwband[MAXBND],         /* a lut to the band index for fweights*/
	minmax,                 /* $minmax as a flag, = TRUE or FALSE */
        msgtim;                 /* tae global $msgtim */ 
   struct VARIABLE *in;         /* pointer to var within parameter block */
/* get image names and parameters from tae.  begin_las() must be called first */
   begin_las ("dctfil");        /* initialize las and tae */ 
   getin2  ("in",  &inimage);   /* get input image name */ 
   getout2 ("out", &outimage);  /* get output image name */ 
   /* get bsize parm */ 
      in = p_find (&pblock, "bsize");    
      bsize = IVAL(*in, 0); 
   /* odtype */ 
      in = p_find (&pblock, "odtype");    
      strcpy (odtype, SVAL(*in, 0)); 
   /* wfile */ 
      in = p_find (&pblock, "wfile"); 
      strcpy (wfile, SVAL(*in, 0)); 
   /* $msgtim */ 
      in = p_find (&pblock, "$msgtim");    
      msgtim = IVAL(*in, 0); 
   /* $minmax */ 
      in = p_find (&pblock, "$minmax");    
      if (*SVAL(*in,0)=='Y')
	 minmax = TRUE;
      else 
	 minmax = FALSE;
/* open images, input must be opened before output */
   openrin (BIO, bsize, &inimage); 
   openrout (BIO, &inimage, minmax, msgtim, odtype, &outimage); 
/* read wfile */ 
   read_wfile (wfile, inimage.totalbands, fwband, fweights); 
/* create image */ 
   bimage (fweights, fwband, &inimage, &outimage); 
/* close image files and end program */ 
   end_las (&inimage, &outimage); 

   return EXIT_SUCCESS;
}
