
/* include file dctfil.h by David Zokaites.  File is slightly modified from 
   appname.h which I wrote to consolidate LAS image processing calls.  File 
   includes data structures for LAS images, global vars, and function
   prototypes. */
#include "las.h"    /* for las image IO */ 
/* types of image IO, BIO is used for this proc */
   enum iotypes 
   {
      SIO, /* sequential, line by line */ 
      RIO, /* line by line, using LAS random IO */ 
      BIO  /* in blocks, using LAS random IO */ 
   };





/* data structure for one las input image, used to facilitate IO.  Because las
   image specs allow for grouping image components, (e.g. in="image1 + image2")
   one input image may correspond to multiple disk image files.  The same
   structure is used all types of IO, although all structure members are not 
   always used. */
   struct lasinimage
   {
      char hostname[MAXIMG][CMLEN], /* host name for image files */ 
           taename[MAXIMG][CMLEN];  /* tae name for image components */ 
      struct FDESC *fdarray[MAXIMG+1]; /* file descriptor array */ 
      struct GDESC *groupd;         /* group descriptor, used for SIO */ 
      long bands[MAXIMG][MAXBND+1], /* band numbers */ 
           blocksize,               /* size of blocks for BIO */ 
	   blocknl, blockns,        /* image size after expansion for blocking*/
           bufoff[MAXBND],          /* offsets into data buffer, for SIO */ 
           cblut[MAXIMG*MAXBND],    /* a lut for current band, for RIO & BIO */ 
           datasize,                /* number of bytes per pixel */ 
           datatype,                /* image data type = EBYTE, EWORD, ... */ 
           fdlut[MAXIMG*MAXBND],    /* a lut for file descriptor, for RIO, BIO*/
	   fillns,                  /* ns needed to fill last block, for BIO */
           iotype,                  /* type of io, = SIO, RIO, BIO */ 
           lblocks,                 /* number of blocks in line direction, 
				       for BIO */
           nbands[MAXIMG],          /* number of bands in each component */ 
           ncomponents,             /* number of image components */ 
           qflag[MAXIMG][2],        /* flag, do image components have a band or
				       window specified */ 
           samewindow,              /* flag, do image components have the same 
				       window size */ 
           sblocks,                 /* number of blocks in sample direction, 
				       for BIO */ 
           totalbands,              /* total number of bands */ 
           window[MAXIMG][4];       /* window (sl ss nl ns) for each component*/
      unsigned char **bufloc,       /* location in data buffer for each line of
	                               data, for BIO */ 
                     *buffer;       /* buffer for image data */ 
   }; 



/* data structure for one las output image, used to facilitate IO */ 
   struct lasoutimage
   {
      char   hostname[1][CMLEN],       /* host name for image file */ 
             taename[1][CMLEN];        /* tae name for image */ 
      double minval[MAXBND],        /* min pixel value, used to update ddr */ 
             maxval[MAXBND];        /* max pixel value, used to update ddr */ 
      struct FDESC* fdarray[1+1];   /* file descriptor array */ 
      struct GDESC* groupd;         /* group descriptor for SIO */ 
      long blocksize,               /* size of blocks for BIO */ 
	   blocknl, blockns,        /* image size after expansion for blocking*/
           bufoff[MAXBND],          /* offsets into data buffer, for SIO */
           datasize,                /* number of bytes per pixel */ 
           datatype,                /* image data type = EBYTE, EWORD, ... */ 
	   fillns,                  /* ns needed to fill last block, for BIO */
           iotype,                  /* type of io, = SIO, RIO, BIO */ 
           lblocks,                 /* number of blocks in line direction, 
				       for BIO */
           minmax,                  /* flag, determine min/max for ddr? = T,F */
           nl, ns,                  /* size of output image */ 
           sblocks,                 /* number of blocks in sample direction, for
				       BIO */ 
           totalbands,              /* total number of bands */ 
	   truncate;                /* flag, was the output data truncated to
	                               fit in the allowed range */ 
      unsigned char **bufloc,       /* location in data buffer for each line of
	                               data, for BIO */ 
                     *buffer;       /* buffer for image data */ 
   }; 



/* data structures for parameters with multiple values, not used for this proc*/
   /* integer parms */ 
      struct iparm
      {
	 long count;   /* number of values for this parm */ 
	 long *value;  /* parm values */ 
      };
   /* real parms */ 
      struct rparm
      {
	 long count;    /* number of values for this parm */ 
	 float *value;  /* parm values */ 
      };
   /* string parms */ 
      struct sparm
      {
	 long count;           /* number of values for this parm */ 
	 char *value[MAXVAL];  /* parm values */ 
      };




/* global vars, must be global for las image IO routines */
#ifdef GLOBALVARS
      struct lasinimage  inimage;  /* input image */ 
      struct lasoutimage outimage; /* output image */ 
      struct PARBLK pblock;        /* tae parameter block */ 
#else 
      extern struct lasinimage  inimage;  
      extern struct lasoutimage outimage; 
      extern struct PARBLK pblock; 
#endif 



/* prototype local functions */ 
#ifdef LANGUAGE_C
   /* functions in dctfil.c */ 
      int main (void); 
      void read_wfile (char *wfile, long nbands, long fwband[MAXBND], 
              float **fweights[MAXBND+1]),
           bimage (float **fweights[MAXBND+1], long fwband[MAXBND], 
	      struct lasinimage *ini, struct lasoutimage *outi);
   /* dct.c */ 
      void set_const (long bsize), 
           fdct (float **imaged, float **dctdata), 
	   app_weights (float **fweights, float **dctdata), 
	   rdct (float **dctdata, float **imaged),
	   fix_truncate (float **imaged), 
	   free_struct (void);
   /* imageio.c */ 
      void begin_las (char *proc), 
           integer_parm (char *parmname, struct iparm *parmval), 
           real_parm (char *parmname, struct rparm *parmval), 
           string_parm (char *parmname, struct sparm *parmval), 
           clean2 (char *message), 
           getin2 (char *parmname, struct lasinimage *ini), 
           getout2 (char *parmname, struct lasoutimage *outi), 
           opensin (struct lasinimage *ini), 
           opensout (struct lasinimage *ini, long minmax, long msgtim,
   	      char *odtype, struct lasoutimage *outi), 
           openrin (long iotype, long blocksize, struct lasinimage *ini), 
           openrout (long iotype, struct lasinimage *ini, long minmax, long 
	      msgtim, char *odtype, struct lasoutimage *outi), 
           rread (long curline, long curband, struct lasinimage *ini), 
           rwrite (long curline, long curband, struct lasoutimage *outi), 
           bread (long curblock, long curband, struct lasinimage *ini), 
           bwrite (long curblock, long curband, struct lasoutimage *outi), 
           minmax (long band, struct lasoutimage *outi), 
           end_las (struct lasinimage *ini, struct lasoutimage *outi);
#endif  
