
/* imageio.c by David Zokaites 12-92 to 5-93.  This file helps to consolidate 
   TAE and LAS image processing calls.  */ 
   
#include <math.h>        /* for call to ceil() */ 
#include "dctfil.h"      /* for globals vars and local image IO functions */ 
#include "util.h"
#include "pixman.h"      /* for TOOBIG, TOOSMALL macros */ 
#include "protos.h"

/* global vars, declared global for convenience */ 
   char procname[30];    /* name of las proc, uppercase */ 
   char procfail[40];    /* name of las proc, lowercase and "-fail" */
   char procwarn[40];    /* name of las proc, lowercase and "-warn" */
   long fatal= -1;       /* fatal error, parm to c_errmsg() */ 
   long go_on=  1;       /* warning message, informational, parm to c_errmsg()*/
   char message[CMLEN];  /* error message string, parm to c_errmsg()*/





/* FUNCTION: initialize tae and las processing.  Must be called before all
   other functions in this file */ 
void begin_las
(
    char *proc             /* input:  name of las proc */
) 
{
/* declare vars */ 
   long length,            /* length of a char string */ 
        std_out;           /* standard output */ 
/* initialize las */ 
   c_inlas (&pblock, &std_out); 
/* set global vars for proc name, procname is uppercase, procfail and procwarn
   are lower case */ 
   strcpy  (procname, proc); 
   length = (long) strlen (procname);   
   c_low2up (procname, &length);  
   length = (long) strlen (proc);       
   c_up2low (proc, &length); 
   sprintf (procfail, "%s-fail", proc); 
   sprintf (procwarn, "%s-warn", proc); 
}





/* FUNCTION:  return an integer parm with multiple values */ 
void integer_parm
(
    char *parmname,         /* input:  name of parameter */
    struct iparm *parmval  /* output: parm values */
) 
{
/* declare variables */ 
   long index;            /* for loop index */ 
   struct VARIABLE *var;  /* pointer to a var in pblock */ 
/* find count of parm and allocate memory */ 
   var = p_find (&pblock, parmname);
   parmval->count = (*var).v_count;
   if ((parmval->value = (long *) malloc (parmval->count * sizeof(long)) ) 
   == NULL)
      c_errmsg ("Unable to allocate memory for a parameter.", procfail, &fatal);
/* set parm values */ 
   for (index=0; index < parmval->count; index++)
      parmval->value[index] = IVAL(*var,index);
}





/* FUNCTION:  return a real parm with multiple values */ 
void real_parm
(
    char *parmname,         /* input:  name of parameter */
    struct rparm *parmval  /* output: parm values */
) 
{
/* declare variables */ 
   long index;            /* for loop index */ 
   struct VARIABLE *var;  /* pointer to a var in pblock */ 
/* find count of parm and allocate memory */ 
   var = p_find (&pblock, parmname);
   parmval->count = (*var).v_count;
   if ((parmval->value = (float *) malloc (parmval->count * sizeof(float)) ) 
   == NULL)
      c_errmsg ("Unable to allocate memory for a parameter.", procfail, &fatal);
/* set parm values */ 
   for (index=0; index < parmval->count; index++)
      parmval->value[index] = RVAL(*var,index);
}





/* FUNCTION:  return a string parm with multiple values */ 
void string_parm
(
    char *parmname,         /* input:  name of parameter */
    struct sparm *parmval  /* output: parm values */
) 
{
/* declare variables */ 
   long index;             /* for loop index */ 
   struct VARIABLE *var;   /* pointer to a var in pblock */ 
/* find count of parm */ 
   var = p_find (&pblock, parmname);
   parmval->count = (*var).v_count;
/* set parm values */ 
   for (index=0; index < parmval->count; index++)
      parmval->value[index] = SVAL(*var,index);
}





/* FUNCTION: cleanup from an error, and delete image files */ 
void clean2
(
    char *message  /* input:  message to write */
) 
   /* additional inputs are inimage and outimage global variables */ 
{
/* declare var, write message to user */ 
   long imagenum,  /* for loop index */ 
        one=1;     /* number of output file names in outimage.hostname */ 
   c_errmsg (message, procfail, &go_on); 
/* close input image files */ 
   if (inimage.iotype == SIO)   /* close group descriptor */ 
   {
      if (c_egclse(&inimage.groupd) ) 
	 c_errmsg ("Error closing input image files.", procwarn, &go_on); 
   }
   else               /* close individual file descriptors */ 
      for (imagenum=0; imagenum<inimage.ncomponents; imagenum++)
	 if (c_eclose (inimage.fdarray+imagenum))
	    c_errmsg ("Error closing input image files.", procwarn, &go_on); 
/* close output image files */ 
   if (outimage.iotype == SIO)   /* close group descriptor */ 
   {
      if (c_egclse(&outimage.groupd) ) 
	 c_errmsg ("Error closing output image file.", procwarn, &go_on); 
   }
   else                /* close individual file descriptors */ 
      if (c_eclose (outimage.fdarray))
	 c_errmsg ("Error closing output image file.", procwarn, &go_on); 
/* cleanup disk files */ 
   c_clean (&pblock, procname, outimage.hostname, &one); 
}





/* FUNCTION: parse input image spec */ 
void getin2
(
    char *parmname,             /* input:  tae parameter name for image */
    struct lasinimage *ini     /* output: data structure of input las image */
)
{
   long   
        maximg=MAXIMG,         /* max number of images allowed */ 
        zero=0;                /* value of 0 */ 
/* call c_getin() */ 
   if (c_getin (&pblock, parmname, &zero, ini->taename, ini->hostname, 
   &ini->ncomponents, ini->window, ini->bands, ini->nbands, 
   &ini->totalbands, &maximg, &ini->samewindow, ini->qflag ))
   {
      sprintf (message, "Error getting input image for parm %s.", parmname); 
      c_errmsg (message, procfail, &fatal); 
   }
/* error check, different window sizes causes c_egroup() to fail */ 
   if (!ini->samewindow)
   {
      sprintf (message, 
      "Input images specified by parm %s do not have the same window size.",
      parmname);
      c_errmsg (message, procfail, &fatal); 
   }
/* debugging statements to dump ini components */ 
   /* 
   printf ("dumping some ini components in getin2() \n"); 
   printf ("   ncomponents=%d totalbands=%d samewindow=%d \n", ini->ncomponents,
   ini->totalbands, ini->samewindow); 
   for (index=0; index<ini->ncomponents; index++)
   {
      printf ("   index=%d \n", index); 
      printf ("   taename[]=%s \n", ini->taename[index]); 
      printf ("   hostname[]=%s \n", ini->hostname[index]); 
      printf ("   nbands[]=%d \n", ini->nbands[index]); 
      printf ("   window[]= %d %d %d %d \n", 
      ini->window[index][SL], ini->window[index][SS], 
      ini->window[index][NL], ini->window[index][NS]); 
   } 
   */ 
}





/* FUNCTION: get output image name */ 
void getout2
(
    char *parmname,            /* input:  tae parameter name for image */
    struct lasoutimage *outi  /* output: data structure of las image */
)
{
   if (c_getout (&pblock, parmname, outi->taename, outi->hostname) )
   {
      sprintf (message, "Error getting output image for parm %s.", parmname); 
      c_errmsg (message, procfail, &fatal); 
   }
}





/* FUNCTION: open an input image for sequential access and prepare 
   data buffer */ 
void opensin
(
    struct lasinimage *ini       /* input and output: input image to open */
) 
{
/* declare vars */ 
   long accessm=IREAD,           /* image access mode */ 
        bandnum,                 /* loop index, band number */ 
        imagenum,                /* loop index, image number */ 
        buffersz,                /* buffer size returned by ealloc */
        zero=0;                  /* value of zero */ 
/* open image file and set vars */ 
   ini->iotype = SIO;                 /* sequential IO */ 
   ini->fdarray[ini->ncomponents]=0;  /* null terminate array */ 
   ini->datatype=0; 
   for (bandnum=imagenum=0; imagenum<ini->ncomponents;  
   bandnum+=ini->nbands[imagenum], imagenum++)
      if (c_eopens (ini->fdarray+imagenum, ini->hostname[imagenum], 
      ini->bands[imagenum], 
      &ini->window[imagenum][SL], &ini->window[imagenum][SS], 
      &ini->window[imagenum][NL], &ini->window[imagenum][NS], 
      ini->bufoff+bandnum, &accessm, &ini->datatype, &zero, &zero) )
      {
	 sprintf (message, "Error opening image file for image %s",
	 ini->taename[imagenum]); 
	 c_errmsg (message, procfail, &fatal); 
      }
   switch (ini->datatype)
   {
      case EBYTE:  ini->datasize=1;  break; 
      case EWORD:  ini->datasize=2;  break; 
      case ELONG:  ini->datasize=4;  break; 
      case EREAL:  ini->datasize=4;  break; 
   }
/* allocate memory for image lines */ 
   if ((ini->buffer = (unsigned char *) 
   c_ealloc(ini->fdarray, &buffersz)) == NULL)
      c_errmsg ("Unable to allocate memory for input image buffer.", 
      procfail, &fatal); 
/* group image files */ 
   if (c_egroup (ini->fdarray, &ini->groupd, ini->buffer, 
   &buffersz) )
      c_errmsg ("Error grouping input image files.", procfail, &fatal);
/* initialize first line of image */ 
   if (c_estep (&ini->groupd) )
      c_errmsg ("Error stepping through input images.", procfail, &fatal);
}





/* FUNCTION: open an output image for sequential access and prepare 
   data buffer */ 
void opensout
(
    struct lasinimage *ini,     /* input:  input image, needed for nl,ns,nbands*/
    long minmax,                /* input:  flag, update min/max */
    long msgtim,                /* input:  $msgtim */
    char *odtype,               /* input:  data type for output image */
    struct lasoutimage *outi   /* input and output:  output image to open */
)
{
/* declare vars */ 
   long accessm=IWRITE,   /* image access mode */ 
        buffersz,         /* buffer size returned by ealloc */
        messform=ALLCAT,  /* output message format */ 
        one =1,           /* value of one */ 
        zero=0;           /* value of zero */ 
/* set vars in data structure */ 
   outi->nl = ini->window[0][NL]; 
   outi->ns = ini->window[0][NS]; 
   outi->iotype = SIO;    /* sequential IO */ 
   outi->totalbands = ini->totalbands; 
   outi->truncate = E_SUCC; /* output data was tentatively not truncated */ 
   outi->fdarray[1] = 0;  /* null terminate array */ 
   /* datatype */ 
      if      (!strcmp (odtype, "SAME"))
	 outi->datatype = ini->datatype; 
      else if (!strcmp (odtype, "BYTE"))
	 outi->datatype = EBYTE; 
      else if (!strcmp (odtype, "I*2"))
	 outi->datatype = EWORD; 
      else if (!strcmp (odtype, "I*4"))
	 outi->datatype = ELONG; 
      else if (!strcmp (odtype, "R*4"))
	 outi->datatype = EREAL; 
      else
	 c_errmsg ("Parm ODTYPE has an illegal value.", procfail, &fatal); 
/* open image file and set datasize */ 
   if (c_eopens (outi->fdarray, outi->hostname[0], &outi->totalbands, 
   &one, &one, &outi->nl, &outi->ns, outi->bufoff, &accessm, 
   &outi->datatype, &zero, &zero) )
   {
      sprintf (message, "Error opening output image file for image %s",
      outi->taename[0]); 
      clean2 (message); 
   }
   switch (outi->datatype)
   {
      case EBYTE:  outi->datasize=1;  break; 
      case EWORD:  outi->datasize=2;  break; 
      case ELONG:  outi->datasize=4;  break; 
      case EREAL:  outi->datasize=4;  break; 
   }
/* allocate memory for image lines */ 
   if ((outi->buffer = (unsigned char *) 
   c_ealloc(outi->fdarray, &buffersz)) == NULL)
      clean2 ("Unable to allocate memory for output image buffer."); 
/* group image files */ 
   if (c_egroup (outi->fdarray, &outi->groupd, outi->buffer, 
   &buffersz) )
      clean2 ("Error grouping output image file.");
/* initialize first line of image */ 
   if (c_estep (&outi->groupd) )
      clean2 ("Error stepping through output image.");
/* initialize min max arrays */ 
   outi->minmax = minmax; 
   if (outi->minmax && c_inmnmx (&outi->datatype, 
   &outi->totalbands, outi->minval, outi->maxval))
      clean2 ("Error initializing min/max arrays."); 
/* initialize and start processing line message */ 
   c_inittm (procname, "Processed", "line", "bands", "", &outi->nl,
   &outi->totalbands, &msgtim, &messform); 
   c_startm (); 
}





/* FUNCTION: open an input image for random access and allocate memory for
   image lines */ 
void openrin
(
    long iotype,             /* input:  type of IO = RIO or BIO */
    long blocksize,          /* input:  size of image blocks, largely ignored
                               for RIO */
    struct lasinimage *ini  /* input and output: input image to open */
) 
{
/* declare vars */ 
   long accessm=IREAD,      /* image access mode */ 
        bandnum,            /* loop index, band number */ 
        linenum,            /* loop index, line number */ 
        imagenum,           /* loop index, image number */ 
	one=1,              /* value of one */ 
        tband,              /* loop index, total band number */ 
        zero=0;             /* value of zero */ 
/* open image file and set vars.  some vars are only for BIO, but it is easier
   to just set all the vars for RIO and BIO */ 
   if (iotype != RIO && iotype != BIO)
      c_errmsg ("logic error in openrin():  invalid iotype", procfail, &fatal);
   ini->iotype = iotype;  
   ini->blocksize = blocksize; 
   ini->lblocks = (long) ceil ( (double)ini->window[0][NL] / ini->blocksize ); 
   ini->sblocks = (long) ceil ( (double)ini->window[0][NS] / ini->blocksize ); 
   ini->blocknl= ini->blocksize * ini->lblocks; 
   ini->blockns= ini->blocksize * ini->sblocks; 
   ini->fillns = ini->blockns - ini->window[0][NS];
   ini->datatype=0; 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      if (c_eopenr (ini->fdarray+imagenum, ini->hostname[imagenum], 
      ini->nbands+imagenum, 
      &ini->window[imagenum][SL], &ini->window[imagenum][SS], 
      &ini->window[imagenum][NL], &ini->window[imagenum][NS], 
      &accessm, &ini->datatype, &zero, &one)) 
      {
	 sprintf (message, "Error opening image file for image %s",
	 ini->taename[imagenum]); 
	 c_errmsg (message, procfail, &fatal); 
      }
   switch (ini->datatype)
   {
      case EBYTE:  ini->datasize=1;  break; 
      case EWORD:  ini->datasize=2;  break; 
      case ELONG:  ini->datasize=4;  break; 
      case EREAL:  ini->datasize=4;  break; 
   }
/* build luts for later access by rread() */ 
   for (tband=imagenum=0; imagenum<ini->ncomponents; imagenum++)
      for (bandnum=0; bandnum<ini->nbands[imagenum]; bandnum++, tband++)
      {
	 /* the parm eband for c_eread() is relative to the bands in an image
	 file, it is not relative to the total number of bands for the output
	 image.  This is accommodated in the next program line.  Users may
	 subband input images, and the lut for current band must allow for
	 this. */ 
	 ini->cblut[tband] = ini->bands[imagenum][bandnum];
	 ini->fdlut[tband] = imagenum; 
      } 
/* allocate memory for image lines and BIO variables */ 
   if ((ini->buffer = (unsigned char *) malloc 
   (ini->blocksize * ini->blockns * ini->datasize) ) == NULL)
      c_errmsg ("Unable to allocate memory for input image buffer.", 
      procfail, &fatal); 
   if (ini->iotype == BIO)
   {
      if ((ini->bufloc = (unsigned char **) 
      malloc ( sizeof(unsigned char *) * ini->blocksize)) == NULL)
	 c_errmsg ("Unable to allocate memory for bufloc.", procfail, &fatal); 
      for (linenum=0; linenum<ini->blocksize; linenum++)
	 ini->bufloc[linenum] = ini->buffer + 
	 linenum * ini->datasize * ini->blockns; 
   }
/* dump general ini components */ 
   /* 
   printf ("dumping some ini components in openrin() \n");
   printf ("totalbands=%d \n", ini->totalbands); 
   for (tband=0; tband<ini->totalbands; tband++)
      printf ("%d cblut[]=%d fdlut[]=%d \n", tband, 
      ini->cblut[tband], ini->fdlut[tband]); 
   printf ("   blocksize=%d datatype=%d datasize=%d \n", 
   ini->blocksize, ini->datatype, ini->datasize); 
   for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
      printf ("   fdarray[%d]=%d \n", imagenum, ini->fdarray[imagenum]); 
   */ 
/* dump ini components for BIO processing */ 
   /* 
   printf ("in openrin(), ini components for BIO processing: \n");
   printf ("blocksize=%d \n",  ini->blocksize); 
   printf ("nl=%d ns=%d \n", ini->window[0][NL], ini->window[0][NS]); 
   printf ("lblocks=%d sblocks=%d \n", ini->lblocks, ini->sblocks); 
   printf ("blocknl=%d blockns=%d \n", ini->blocknl, ini->blockns); 
   printf ("fillns=%d  \n", ini->fillns); 
   printf ("buffer=%d bufloc[0]=%d bufloc[1]=%d bufloc[2]=%d \n", 
   ini->buffer, ini->bufloc[0], ini->bufloc[1], ini->bufloc[2]); 
   printf ("\n"); 
   */ 
}






/* FUNCTION: open an output image for random access and prepare data buffer,
   must be called after openrin() */ 
void openrout
(
    long iotype,                /* input:  type of IO = RIO or BIO */
    struct lasinimage *ini,     /* input:  data structure with blocksize, nl,
                                  ns, nbands to be used for outi */
    long minmax,                /* input:  flag, update min/max */
    long msgtim,                /* input:  $mstgim */
    char *odtype,               /* input:  data type for output image */
    struct lasoutimage *outi   /* input and output:  output image to open */
)
{
/* declare vars */ 
   long accessm=IWRITE,   /* image access mode */ 
	linenum,          /* for loop index:  line number */ 
        messform=ALLCAT,  /* output message format */ 
        one =1,           /* value of one */ 
	tblocks,          /* total number of blocks for BIO processing */ 
        zero=0;           /* value of zero */ 
/* set vars in data structure.  some vars are only for BIO, but it is easier 
   to just set all the vars for RIO and BIO */ 
   if (iotype != RIO && iotype != BIO)
      c_errmsg ("logic error in openrout():  invalid iotype", procfail, &fatal);
   outi->iotype = iotype;  
   outi->nl = ini->window[0][NL]; 
   outi->ns = ini->window[0][NS]; 
   outi->blocksize = ini->blocksize; 
   outi->lblocks = (long) ceil ( (double)outi->nl / outi->blocksize ); 
   outi->sblocks = (long) ceil ( (double)outi->ns / outi->blocksize ); 
   outi->blocknl= outi->blocksize * outi->lblocks; 
   outi->blockns= outi->blocksize * outi->sblocks; 
   outi->fillns = outi->blockns - outi->ns;
   outi->totalbands = ini->totalbands; 
   outi->truncate = E_SUCC; /* output data was tentatively not truncated */ 
   /* datatype */ 
      if      (!strcmp (odtype, "SAME"))
	 outi->datatype = ini->datatype; 
      else if (!strcmp (odtype, "BYTE"))
	 outi->datatype = EBYTE; 
      else if (!strcmp (odtype, "I*2"))
	 outi->datatype = EWORD; 
      else if (!strcmp (odtype, "I*4"))
	 outi->datatype = ELONG; 
      else if (!strcmp (odtype, "R*4"))
	 outi->datatype = EREAL; 
      else
	 c_errmsg ("Parm ODTYPE has an illegal value.", procfail, &fatal); 
/* open image file and set datasize */ 
   if (c_eopenr (outi->fdarray, outi->hostname[0], &outi->totalbands, 
   &one, &one, &outi->nl, &outi->ns, &accessm, 
   &outi->datatype, &zero, &one))
   {
      sprintf (message, "Error opening output image file for image %s",
      outi->taename[0]); 
      clean2 (message); 
   }
   switch (outi->datatype)
   {
      case EBYTE:  outi->datasize=1;  break; 
      case EWORD:  outi->datasize=2;  break; 
      case ELONG:  outi->datasize=4;  break; 
      case EREAL:  outi->datasize=4;  break; 
   }
/* allocate memory for image lines and BIO variables */ 
   if ((outi->buffer = (unsigned char *) 
   malloc (outi->blocksize * outi->blockns * outi->datasize) ) == NULL)
      clean2 ("Unable to allocate memory for output image buffer."); 
   if (outi->iotype == BIO)
   {
      if ((outi->bufloc = (unsigned char **) 
      malloc ( sizeof(unsigned char *) * outi->blocksize)) == NULL)
	 c_errmsg ("Unable to allocate memory for bufloc.", procfail, &fatal); 
      for (linenum=0; linenum<outi->blocksize; linenum++)
	 outi->bufloc[linenum] = outi->buffer + 
	 linenum * outi->datasize * outi->blockns; 
   }
/* initialize min max arrays */ 
   outi->minmax = minmax; 
   if (outi->minmax && c_inmnmx (&outi->datatype, 
   &outi->totalbands, outi->minval, outi->maxval))
      clean2 ("Error initializing min/max arrays."); 
/* dump outi components for BIO processing */ 
   /*  
   printf ("in openrout(), outi components for BIO processing: \n");
   printf ("blocksize=%d \n",  outi->blocksize); 
   printf ("nl=%d ns=%d \n", outi->nl, outi->ns); 
   printf ("lblocks=%d sblocks=%d \n", outi->lblocks, outi->sblocks); 
   printf ("blocknl=%d blockns=%d \n", outi->blocknl, outi->blockns); 
   printf ("fillns=%d  \n", outi->fillns); 
   printf ("buffer=%d bufloc[0]=%d bufloc[1]=%d bufloc[2]=%d \n", 
   outi->buffer, outi->bufloc[0], outi->bufloc[1], outi->bufloc[2]); 
   printf ("\n");
   */
/* initialize and start processing line message */ 
   if (outi->iotype == BIO) 
   {
      tblocks = outi->lblocks * outi->sblocks * outi->totalbands; 
      c_inittm (procname, "Processed", "block", "bands", "", &tblocks,
      &outi->totalbands, &msgtim, &messform); 
   }
   else 
      c_inittm (procname, "Processed", "line", "bands", "", &outi->nl,
      &outi->totalbands, &msgtim, &messform); 
   c_startm (); 
}





/* FUNCTION:  read data from input image file, using random IO */ 
void rread
(
    long curline,           /* input: starting line of block of data to read, 
			      0 relative */
    long curband,           /* input: band to read, 0 relative */
    struct lasinimage *ini /* input and output: data structure of input image 
			      to read */
)
{
   long imagenum;  /* image number, index to fdesc[] */ 
/* first determine which image file to access then translate from curband as 
   input to curband relative to the number of bands in a particular image 
   file. */ 
   imagenum = ini->fdlut[curband]; 
   curband  = ini->cblut[curband]; 
/* convert from 0 relative to 1 relative */ 
   curline++; 
/* read data from image */ 
   if (c_eread (ini->fdarray+imagenum, &curband, &curline, ini->buffer, 
   &ini->blocksize) != ini->blocksize * ini->window[0][NS])
      clean2 ("Error reading input image data.");
}





/* FUNCTION:  write data to disk, using random IO */ 
void rwrite
(
    long curline,             /* input: starting line of block of data to write, 
			        0 relative */
    long curband,             /* input: band to write, 0 relative */
    struct lasoutimage *outi /* input: data structure of image to write */
)
{
/* convert from 0 relative to 1 relative */ 
   curline++; 
   curband++; 
/* write data to disk */ 
   if (c_ewrite (outi->fdarray, &curband, &curline, outi->buffer, 
   &outi->blocksize) != outi->blocksize * outi->ns)
      clean2 ("Error writing output image data.");
}





/* FUNCTION:  read a block of lines from input image file.  If the image does
   not contain an integral number of blocks in the line and sample directions, 
   the last few samples/lines will be mirrored to create an integral number of
   blocks in both directions. */ 
void bread
(
    long curblock,          /* input: read data for this block of lines */
    long curband,           /* input: band to read, 0 relative */
    struct lasinimage *ini /* input and output: data structure of input image 
			      to read */
)
{
/* declare variables */ 
   long imagenum,          /* image number, index to ini->fdarray[] */ 
        linenum,           /* number of this line in the image, 1 relative */ 
	npix,              /* loop index, number of pixels to mirror */ 
        one=1,             /* value of one */ 
	reread,            /* flag, re-read lines of image */
	rline;             /* loop index, lines read */ 
   unsigned char *frompix, /* location of pixel to mirror */ 
		 *lastpix, /* location of last pixel in line of image data */ 
		 *topix;   /* location of mirrored pixel */
/* before reading image, first determine which image file to access then 
   translate from curband as input to curband relative to the number of bands
   in a particular image file. */ 
   imagenum = ini->fdlut[curband]; 
   curband  = ini->cblut[curband]; 
/* read each line and mirror data as necessary */ 
   for (rline=0,  linenum = curblock * ini->blocksize +1,  reread=FALSE; 
   rline<ini->blocksize;   rline++)
   {
   /* read data from file */ 
      if (c_eread (ini->fdarray+imagenum, &curband, &linenum, 
      ini->bufloc[rline], &one) != ini->window[0][NS])
	 clean2 ("Error reading data from image file."); 
   /* mirror pixels if necessary */ 
      for (npix=1; npix<=ini->fillns; npix++)
      {
	 lastpix= ini->bufloc[rline] + (ini->window[0][NS]-1) * ini->datasize;
	 frompix = lastpix - npix * ini->datasize; 
	 topix   = lastpix + npix * ini->datasize; 
	 switch (ini->datatype)
	 {
	    case EBYTE: *         topix = *         frompix;  break; 
	    case EWORD: *(short *)topix = *(short *)frompix;  break; 
	    case ELONG: *(long  *)topix = *(long  *)frompix;  break;
	    case EREAL: *(float *)topix = *(float *)frompix;  break; 
	 }
      }
   /* dump data 
      printf ("linenum=%d rline=%d ", linenum, rline); 
      for (sample=0; sample<ini->blockns; sample++)
      {
	 location = ini->bufloc[rline] + (sample * ini->datasize); 
	 switch (ini->datatype)
	 {
	    case EBYTE: printf ("%d ",   *          location);   break; 
	    case EWORD: printf ("%d ",   *(short *) location);   break; 
	    case ELONG: printf ("%d ",   *(long  *) location);   break; 
	    case EREAL: printf ("%.0f ", *(float *) location);   break; 
	 }
      }
      printf ("\n"); */ 
   /* increment (decrement) linenum to mirror in line direction */ 
      if (linenum==ini->window[0][NL])
	 reread=TRUE;  
      if (reread)
	 linenum--; 
      else 
	 linenum++; 
   }
}




/* FUNCTION:  write a block of lines to output image file */ 
void bwrite
(
    long curblock,            /* input: write data for this block of lines */
    long curband,             /* input: band to write, input 0 relative, 
				converted to 1 relative */
    struct lasoutimage *outi /* input: data structure of input image to write */
)
{
/* declare variables */ 
   long linenum,   /* line number in the image, 1 relative */ 
        one=1,     /* value of one */ 
	wline;     /* loop index, lines written */ 
/* write data to disk */ 
   for (wline=0, linenum=curblock*outi->blocksize +1, curband++; 
   wline<outi->blocksize && linenum<=outi->nl; 
   wline++, linenum++) 
      if (c_ewrite (outi->fdarray, &curband, &linenum, outi->bufloc[wline], 
      &one) != outi->ns)
	 clean2 ("Error writing data to image file."); 
}





/* FUNCTION:  optionally update minmax for a line (or block of lines) of image 
   data */ 	
void minmax
(
    long band,                /* input:  image buffer holds data for this band */
    struct lasoutimage *outi /* input and output:  structure of image data to 
				update min/max for */
) 
{
/* declare vars */ 
   long line;                /* loop index to image lines */ 
/* return if min/max does not need updated */ 
   if (!outi->minmax)
      return; 
/* different logic applies for each type of processing */ 
   switch (outi->iotype)
   {
      case SIO: 
	 if (c_pxmnmx (outi->buffer + (outi->bufoff[band] * outi->datasize), 
	 &outi->ns, outi->minval+band, outi->maxval+band, &outi->datatype))
	    clean2 ("Error updating min max arrays."); 
	 break; 
      case RIO:  
	 if (c_pxmnmx (outi->buffer, &outi->ns, outi->minval+band, 
	 outi->maxval+band, &outi->datatype) )
	    clean2 ("Error updating min max arrays."); 
	 break; 
      case BIO: 
	 for (line=0; line<outi->blocksize; line++)
	    if (c_pxmnmx (outi->bufloc[line], &outi->ns, outi->minval+band, 
	    outi->maxval+band, &outi->datatype) )
	       clean2 ("Error updating min max arrays."); 
	 break; 
   }
}





/* FUNCTION close image files and end program, last function in this file to be
   called */ 
void end_las
(
    struct lasinimage *ini,
    struct lasoutimage *outi
) 
{
/* vars for ddr */ 
   char m_flag[8];      /* minmax flag, = YES, NO, or BOUNDED */ 
   long cflag=NOCOMB,   /* combine flag for c_upddr() */ 
        imagenum,       /* loop index:  image number */ 
        one=1;          /* value of one */ 
   double oned=1.0;     /* one as a double */ 
/* stop processing line message */ 
   c_stoptm (); 
/* check for data type truncation */ 
   if (outi->truncate==TOOSMALL || outi->truncate== (TOOBIG | TOOSMALL))
      c_errmsg (
      "Data values exceeding lower limit of output data type were truncated.",
      procwarn, &go_on); 
   if (outi->truncate==TOOBIG   || outi->truncate== (TOOBIG | TOOSMALL))
      c_errmsg (
      "Data values exceeding upper limit of output data type were truncated.",
      procwarn, &go_on); 
/* close input image files */ 
   if (ini->iotype == SIO) /* close group descriptor */ 
   {
      if (c_egclse(&ini->groupd) ) 
	 c_errmsg ("Error closing input image files.", procwarn, &go_on); 
   }
   else               /* close individual file descriptors */ 
      for (imagenum=0; imagenum<ini->ncomponents; imagenum++)
	 if (c_eclose (ini->fdarray+imagenum))
	    c_errmsg ("Error closing input image files.", procwarn, &go_on); 
/* close output image files */ 
   if (outi->iotype == SIO) /* close group descriptor */ 
   {
      if (c_egclse(&outi->groupd) ) 
	 c_errmsg ("Error closing output image file.", procwarn, &go_on); 
   }
   else                /* close individual file descriptors */ 
      if (c_eclose (outi->fdarray))
	 c_errmsg ("Error closing output image file.", procwarn, &go_on); 
/* free buffers */ 
   free (ini->buffer); 
   free (outi->buffer); 
   if (ini->iotype  == BIO)
      free (ini->bufloc);
   if (outi->iotype == BIO)
      free (outi->bufloc);
/* create ddr for outname image */ 
   if (c_upddr (&pblock, ini->hostname, outi->hostname[0], 
   &ini->ncomponents, ini->window, ini->bands, ini->nbands, 
   &oned, &oned, &outi->totalbands, &cflag))
   {
      c_errmsg ("Error creating ddr for output image.", procfail, &go_on); 
      c_clean (&pblock, procname, outi->hostname, &one); 
   }
/* update ddr with min max */ 
   if (outi->minmax)
   {
      strcpy (m_flag, "YES"); 
      if (c_svmnmx (outi->hostname[0], &outi->totalbands, outi->minval, 
      outi->maxval, m_flag))
	 c_errmsg ("Error updating min/max values in ddr.", procwarn, &go_on); 
   }
/* create history file */ 
   if (c_hstout (&pblock, outi->hostname[0], ini->hostname, 
   &ini->ncomponents, "") )
   {
      c_errmsg ("Error creating history file for output image.", procfail,
      &go_on); 
      c_clean (&pblock, procname, outi->hostname, &one); 
   }
/* end program */ 
   c_complt(procname, outi->taename, &one);
}




